<-- Begin file 3 of 26:  Letter C (Version 0.48) 
        
           This file is part 3 of the GNU version of
     The Collaborative International Dictionary of English
               Also referred to as GCIDE
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 

GCIDE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GCIDE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this copy of GCIDE; see the file COPYING.  If not, write 
to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
           * * * * * * * * * * * * * * * * * * * * * 

           This dictionary was derived from the
         Webster's Revised Unabridged Dictionary
                 Version published 1913
               by the  C. & G. Merriam Co.
                   Springfield, Mass.
                 Under the direction of
                Noah Porter, D.D., LL.D.

                        and from
           WordNet, a semantic network created by
              the Cognitive Science Department
                 of Princeton University
                  under the direction of
                   Prof. George Miller

             and is being updated and supplemented by
         an open coalition of volunteer collaborators from
                       around the world.

     This electronic dictionary is the starting point for an 
ongoing project to develop a modern on-line comprehensive encyclopedic
dictionary, by the efforts of all individuals willing to help build a
large and freely available knowledge base.  Contributions of data,
time, and effort are requested from any person willing to assist creation
of a comprehensive and organized knowledge base for free access on the
internet.  Anyone willing to assist in any way in constructing such a
knowledge base should contact:

     Patrick Cassidy          pc@worldsoul.org
     735 Belvidere Ave.       Office: (908)668-5252
     Plainfield, NJ 07062
     (908) 561-3416

   Last edit October 6, 2002.

 -->

<p><-- p. 199 --></p>

<p><centered><point26>C.</point26></centered></p>

<p><ent>C</ent><br/
<hw>C</hw>. <pr>(s<emac/)</pr> <sn>1.</sn> <def>C is the third letter of the English alphabet. It is from the Latin letter C, which in old Latin represented the sounds of <xex>k</xex>, and <xex>g</xex> (in <xex>go</xex>); its original value being the latter. In Anglo-Saxon words, or Old English before the Norman Conquest, it always has the sound of <xex>k</xex>. The Latin C was the same letter as the Greek <GAMMA/, <gamma/, and came from the Greek alphabet. The Greeks got it from the Ph<oe/nicians. The English name of C is from the Latin name <xex>ce</xex>, and was derived, probably, through the French. Etymologically C is related to <xex>g</xex>, <xex>h</xex>, <xex>k</xex>, <xex>q</xex>, <xex>s</xex> (and other sibilant sounds). Examples of these relations are in L. a<xex>c</xex>utus, E. a<xex>c</xex>ute, a<xex>g</xex>ue; E. a<xex>c</xex>rid, ea<xex>g</xex>er, vine<xex>g</xex>ar; L. <xex>c</xex>ornu, E. <xex>h</xex>orn; E. <xex>c</xex>at, <xex>k</xex>itten; E. <xex>c</xex>oy, <xex>q</xex>uiet; L. <xex>c</xex>ir<xex>c</xex>are, OF. <xex>c</xex>er<xex>ch</xex>ier, E. <xex>s</xex>ear<xex>ch</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>See <xex>Guide to Pronunciation</xex>, <sect/<sect/ 221-228.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The keynote of the normal or <ldquo/natural<rdquo/ scale, which has neither flats nor sharps in its signature; also, the third note of the relative minor scale of the same.</def> <sd>(b)</sd> <def>C after the clef is the mark of common time, in which each measure is a semibreve (four fourths or crotchets); for <xex>alla breve</xex> time it is written <?/.</def> <sd>(c)</sd> <def>The <ldquo/C clef,<rdquo/ a modification of the letter C, placed on any line of the staff, shows that line to be middle C.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>As a numeral, C stands for Latin <ets>centum</ets> or 100, CC for 200, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>C spring</b></col>, <cd>a spring in the form of the letter C.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ca</ent><br/
<hw>Ca</hw> <pos>n.</pos> <def>the chemical symbol for <sig>calcium</sig>, the fifth most abundant element in the earth's crust, having an atomic number of 20.</def><br/
<syn><b>Syn. --</b> calcium.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caaba</ent><br/
||<hw>Ca*a"ba</hw> <pr>(k<adot/*<amac/"b<adot/)</pr>, <pos>n.</pos> <ety>[Ar. <ets>ka'bah</ets>, lit., a square building, fr. <ets>ka'b</ets> cube.]</ety> <def>The small and nearly cubical stone building, toward which all Mohammedans must pray.</def> <altsp>[Written also <asp>kaaba</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <ex>Caaba</ex> is situated in Mecca, a city of Arabia, and contains a famous black stone said to have been brought from heaven. Before the time of Mohammed, the <ex>Caaba</ex> was an idolatrous temple, but it has since been the chief sanctuary and object of pilgrimage of the Mohammedan world.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caas</ent><br/
<hw>Caas</hw> <pr>(k<aum/s)</pr>, <pos>n. sing. & pl.</pos> <def>Case.</def> <mark>[Obs.]</mark> <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caatinga</ent><br/
||<hw>Caa*tin"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Tupi <ets>caa-tinga</ets> white forest.]</ety> <fld>(Phytogeography)</fld> <def>A forest composed of stunted trees and thorny bushes, found in areas of small rainfall in Brazil.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cab</ent><br/
<hw>Cab</hw> <pr>(k<acr/b)</pr>, <pos>n.</pos> <ety>[Abbrev. fr. <ets>cabriolet</ets>.]</ety> <sn>1.</sn> <def>A kind of close carriage with two or four wheels, usually a public vehicle.</def> <ldquo/A <xex>cab</xex> came clattering up.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>cab</xex> may have two seats at right angles to the driver's seat, and a door behind; or one seat parallel to the driver's, with the entrance from the side or front.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hansom cab</b></col>. <cd>See <er>Hansom</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The covered part of a locomotive, in which the engineer has his station.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cab</ent><br/
<hw>Cab</hw> <pr>(k<acr/b)</pr>, <pos>n.</pos> <ety>[Heb. <ets>qab</ets>, fr. <ets>q<amac/bab</ets> to hollow.]</ety> <def>A Hebrew dry measure, containing a little over two (2.37) pints.</def>  <rj><au>W. H. Ward.</au> <au>2 Kings vi. 25.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabal</ent><br/
<hw>Ca*bal"</hw> <pr>(k<adot/*b<acr/l")</pr>, <pos>n.</pos> <ety>[F. <ets>cabale</ets> cabal, cabala, LL. <ets>cabala</ets> cabala, fr. Heb. <ets>qabb<amac/l<emac/h</ets> reception, tradition, mysterious doctrine, fr. <ets>q<amac/bal</ets> to take or receive, in Pi<eum/l <ets>qibbel</ets> to adopt (a doctrine).]</ety> <sn>1.</sn> <def>Tradition; occult doctrine. See <er>Cabala</er>.</def> <mark>[Obs.]</mark>  <rj><au>Hakewill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A secret.</def> <mark>[Obs.]</mark> <ldquo/The measuring of the temple, a <xex>cabal</xex> found out but lately.<rdquo/  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A number of persons united in some close design, usually to promote their private views and interests in church or state by intrigue; a secret association composed of a few designing persons; a junto.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>It so happend, by a whimsical coincidence, that in 1671 the cabinet consisted of five persons, the initial letters of whose names made up the word <ex>cabal</ex>; Clifford, Arlington, Buckingham, Ashley, and Lauderdale.  <rj><au>Macaulay.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The secret artifices or machinations of a few persons united in a close design; intrigue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By cursed <qex>cabals</qex> of women.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Junto; intrigue; plot; combination; conspiracy.</syn> -- <usage><er>Cabal</er>, <er>Combination</er>, <er>Faction</er>.  An association for some purpose considered to be bad is the idea common to these terms.  A <xex>combination</xex> is an organized union of individuals for mutual support, in urging their demands or resisting the claims of others, and may be good or bad according to circumstances; as, a <xex>combiniation</xex> of workmen or of employers to effect or to prevent a change in prices.  A <xex>cabal</xex> is a secret association of a few individuals who seek by cunning practices to obtain office and power.  A <xex>faction</xex> is a larger body than a <xex>cabal</xex>, employed for selfish purposes in agitating the community and working up an excitement with a view to change the existing order of things. <ldquo/Selfishness, insubordination, and laxity of morals give rise to <xex>combinations</xex>, which belong particularly to the lower orders of society. Restless, jealous, ambitious, and little minds are ever forming <xex>cabals</xex>.  <xex>Factions</xex> belong especially to free governments, and are raised by busy and turbulent spirits for selfish purposes<rdquo/.  <rj><qau>Crabb.</qau></rj></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabal</ent><br/
<hw>Ca*bal"</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caballed</conjf> <pr>(-b<acr/ld")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caballing</conjf>]</vmorph>. <ety>[Cf. F. <ets>cabaler</ets>.]</ety> <def>To unite in a small party to promote private views and interests by intrigue; to intrigue; to plot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Caballing</qex> still against it with the great.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabala</ent><br/
<hw>Cab"a*la</hw> <pr>(k<acr/b"<adot/*l<adot/)</pr>, <pos>n.</pos> <ety>[LL. See <er>Cabal</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A kind of occult theosophy or traditional interpretation of the Scriptures among Jewish rabbis and certain medi<ae/val Christians, which treats of the nature of god and the mystery of human existence.  It assumes that every letter, word, number, and accent of Scripture contains a hidden sense; and it teaches the methods of interpretation for ascertaining these occult meanings.  The cabalists pretend even to foretell events by this means.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Secret science in general; mystic art; mystery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabalism</ent><br/
<hw>Cab"a*lism</hw> <pr>(k<acr/b"<adot/*l<icr/z'm)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cabalisme</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The secret science of the cabalists.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A superstitious devotion to the mysteries of the religion which one professes.</def> <mark>[R]</mark>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabalist</ent><br/
<hw>Cab"a*list</hw> <pr>(-l<icr/st)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cabaliste</ets>.]</ety> <def>One versed in the cabala, or the mysteries of Jewish traditions.</def> <ldquo/Studious cabalists.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabalistical</ent><br/
<ent>Cabalistic</ent><br/
<mhw>{ <hw>Cab`a*lis"tic</hw> <pr>(k<acr/b`<adot/*l<icr/s"t<icr/k)</pr>, <hw>Cab`a*lis"tic*al</hw> <pr>(-t<icr/*k<ait/l)</pr> }</mhw> <pos>a.</pos> <def>Of or pertaining to the cabala; containing or conveying an occult meaning; mystic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Heptarchus is a <qex>cabalistic</qex> exposition of the first chapter of Genesis</q>. <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabalistically</ent><br/
<hw>Cab`a*lis"tic*al*ly</hw>, <pos>adv.</pos> <def>In a cabalistic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabalize</ent><br/
<hw>Cab"a*lize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. F. <ets>cabaliser</ets>.]</ety> <def>To use cabalistic language.</def> <mark>[R]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caballer</ent><br/
<hw>Ca*bal"ler</hw> <pr>(k<adot/*b<acr/l"l<etil/r)</pr>, <pos>n.</pos> <def>One who cabals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A close <qex>caballer</qex> and tongue-valiant lord.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caballeria</ent><br/
||<hw>Ca`bal*le*ri"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. See <er>Caballero</er>.]</ety> <def>An ancient Spanish land tenure similar to the English knight's fee; hence, in Spain and countries settled by the Spanish, a land measure of varying size.  In Cuba it is about 33 acres; in Puerto Rico, about 194 acres; in the Southwestern United States, about 108 acres.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>caballero</ent><br/
||<hw>ca`bal*le"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. Cf. <er>Cavalier</er>.]</ety> <def>A knight or cavalier; hence, a gentleman.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>caballine</ent><br/
<hw>cab"al*line</hw> <pr>(k<acr/b"<ait/l*l<imac/n)</pr>, <pos>a.</pos> <ety>[L.<ets>caballinus</ets>, fr. <ets>caballus</ets> a nag. Cf.  <er>Cavalier</er>.]</ety> <def>Of or pertaining to a horse.</def>   -- <def2><pos>n.</pos> <def>Caballine aloes.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>caballine aloes</b></col>, <cd>an inferior and impure kind of aloes formerly used in veterinary practice; -- called also <altname>horse aloes</altname>.</cd> -- <col><b>Caballine spring</b></col>, <cd>the fountain of Hippocrene, on Mount Helicon; -- fabled to have been formed by a stroke from the foot of the winged horse Pegasus.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caballo</ent><br/
||<hw>Ca*bal"lo</hw> <pr>(k<adot/*v<aum/l"y<osl/; 220)</pr>, <pos>n.</pos> <altsp>[Written also <asp>cavallo</asp>.]</altsp> <ety>[Sp., fr. L. <ets>caballus</ets> a nag. See <er>Cavalcade</er>.]</ety> <def>A horse.</def> <mark>[Sp. Amer.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cabaret</ent><br/
<hw>Cab"a*ret</hw> <pr>(k<acr/b"<adot/*r<ecr/t; 277)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A tavern; a house where liquors are retailed.</def> <mark>[Obs. as an English word.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A type of restaurant where liquor and dinner is served, and entertainment is provided, as by musicians, dancers, or comedians, and usually providing space for dancing by the patrons; -- similar to a nightclub.  In some cases, the performers dance or sing on the floor between the tables, after the practice of a certain class of French taverns.  The term <ex>cabaret</ex> is often used in the names of such an establishment.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>The type of entertainment provided in a cabaret{2}.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cabas</ent><br/
||<hw>Ca*bas"</hw> <pr>(k<adot/*b<aum/")</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A flat basket or frail for figs, etc.; hence, a lady's flat workbasket, reticule, or hand bag; -- often written <asp>caba</asp>.</def>  <rj><au>C. Bront<eacute/.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabassou</ent><br/
||<hw>Ca*bas"sou</hw> <pr>(k<adot/*b<acr/s"s<oomac/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A species of armadillo of the genus <gen>Xenurus</gen> (<spn>Xenurus unicinctus</spn> and <spn>Xenurus hispidus</spn>); the tatouay.</def> <altsp>[Written also <asp>kabassou</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabbage</ent><br/
<hw>Cab"bage</hw> <pr>(k<acr/b"b<asl/j)</pr>, <pos>n.</pos> <ety>[OE. <ets>cabage</ets>, fr. F. <ets>cabus</ets> headed (of cabbages), chou <ets>cabus</ets> headed cabbage, cabbage head; cf. It. <ets>capuccio</ets> a little head, <ets>cappuccio</ets> cowl, hood, cabbage, fr. <ets>capo</ets> head, L. <ets>caput</ets>, or fr. It. <ets>cappa</ets> cape. See <er>Chief</er>, <er>Cape</er>.]</ety> <fld>(Bot.)</fld> <sn>1.</sn> <def>An esculent vegetable of many varieties, derived from the wild <spn>Brassica oleracea</spn> of Europe.  The common cabbage has a compact head of leaves.  The cauliflower, Brussels sprouts, etc., are sometimes classed as cabbages.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The terminal bud of certain palm trees, used, like, cabbage, for food. See <cref>Cabbage tree</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The cabbage palmetto. See below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cabbage aphis</b></col> <fld>(Zool.)</fld>, <cd>a green plant-louse (<spn>Aphis brassic<ae/</spn>) which lives upon the leaves of the cabbage.</cd> -- <col><b>Cabbage beetle</b></col> <fld>(Zool.)</fld>, <cd>a small, striped flea-beetle (<spn>Phyllotreta vittata</spn>) which lives, in the larval state, on the roots, and when adult, on the leaves, of cabbage and other cruciferous plants.</cd> -- <col><b>Cabbage fly</b></col> <fld>(Zool.)</fld>, <cd>a small two-winged fly (<spn>Anthomyia brassic<ae/</spn>), which feeds, in the larval or maggot state, on the roots of the cabbage, often doing much damage to the crop.</cd> -- <col><b>Cabbage head</b></col>, <cd>the compact head formed by the leaves of a cabbage; -- contemptuously or humorously, and colloquially, a very stupid and silly person; a numskull.</cd> -- <col><b>Cabbage palmetto</b></col>, <cd>a species of palm tree (<spn>Sabal Palmetto</spn>) found along the coast from North Carolina to Florida.</cd> -- <col><b>Cabbage rose</b></col> <fld>(Bot.)</fld>, <cd>a species of rose (<spn>Rosa centifolia</spn>) having large and heavy blossoms.</cd> -- <mcol><col><b>Cabbage tree</b></col>, <col><b>Cabbage palm</b></col></mcol>, <cd>a name given to palms having a terminal bud called a <ex>cabbage</ex>, as the <spn>Sabal Palmetto</spn> of the United States, and the <spn>Euterpe oleracea</spn> and <spn>Oreodoxa oleracea</spn> of the West Indies.</cd>  -- <col><b>Sea cabbage</b></col>.<fld>(Bot.)</fld> <sd>(a)</sd> <cd>Sea kale</cd> <sd>(b)</sd>. <cd>The original Plant (<spn>Brassica oleracea</spn>), from which the cabbage, cauliflower, broccoli, etc., have been derived by cultivation.</cd> -- <col><b>Thousand-headed cabbage</b></col>. <cd>See <er>Brussels sprouts</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabbage</ent><br/
<hw>Cab"bage</hw>, <pos>v. i.</pos> <def>To form a head like that the cabbage; <as>as, to make lettuce <ex>cabbage</ex></as>.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabbage</ent><br/
<hw>Cab"bage</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cabbaged</conjf> <pr>(-b<asl/jd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cabbaging</conjf> <pr>(-b<asl/*j<icr/ng)</pr>.]</vmorph> <ety>[F.<ets>cabasser</ets>, fr. OF. <ets>cabas</ets> theft; cf. F. <ets>cabas</ets> basket, and OF. <ets>cabuser</ets> to cheat.]</ety> <def>To purloin or embezzle, as the pieces of cloth remaining after cutting out a garment; to pilfer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your tailor . . . <qex>cabbages</qex> whole yards of cloth.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabbage</ent><br/
<hw>Cab"bage</hw>, <pos>n.</pos> <def>Cloth or clippings cabbaged or purloined by one who cuts out garments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cabbage butterfly</ent><br/
<hw>cab"bage butterfly</hw> <fld>(Zool.)</fld>, <def>a white butterfly (<spn>Pieris rap<ae/</spn> of both Europe and America, and the allied <spn>Pieris oleracea</spn>, a native American species) which, in the larval state, devours the leaves of the cabbage and the turnip. See also <er>Cabbage worm</er>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cabbageworm</ent><br/
<ent>cabbage worm</ent><br/
<mhw><hw>cab"bage worm`</hw>, <hw>cab"bage*worm`</hw></mhw> <pos>n.</pos>  <fld>(Zool.)</fld>, <def>the larva of several species of moths and butterflies, which attacks cabbages. The most common is the toxic green larva of a white butterfly, the <er>cabbage butterfly</er>, (<spn>Pieris rap<ae/</spn>).  The cabbage cutworms, which eat off the stalks of young plants during the night, are the larv<ae/ of several species of moths, of the genus <gen>Agrotis</gen>. See <er>Cutworm</er>.  larva of a cabbage butterfly.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cabbler</ent><br/
<hw>Cab"bler</hw> <pr>(k<acr/b"bl<etil/r)</pr>, <pos>n.</pos> <def>One who works at cabbling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabbling</ent><br/
<hw>Cab"bling</hw> <pr>(-bl<icr/ng)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <def>The process of breaking up the flat masses into which wrought iron is first hammered, in order that the pieces may be reheated and wrought into bar iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabesse</ent><br/
<ent>Cabeca</ent><br/
<mhw>{ ||<hw>Ca*be"<cced/a</hw> <pr>(k<adot/*b<asl/"s<adot/)</pr>, ||<hw>Ca*besse"</hw> <pr>(k<adot/*b<ecr/s")</pr>, }</mhw> <pos>n.</pos> <ety>[Pg. <ets>cabe<cced/a</ets>, F. <ets>cabesse</ets>.]</ety> <def>The finest kind of silk received from India.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caber</ent><br/
<hw>Ca"ber</hw> <pr>(k<amac/"b<etil/r)</pr>, <pos>n.</pos> <ety>[Gael. <ets>cabar</ets>.]</ety> <def>A pole or beam, esp. one used in Gaelic games for tossing as a trial of strength.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cabezon</ent><br/
<hw>Cab`e*zon"</hw> <pr>(k<acr/b`<asl/*z<ocr/n" <it>or</it> k<aum/*b<asl/*th<osl/n")</pr>, <pos>n.</pos> <ety>[Sp., properly, big head. Cf. <er>Cavesson</er>.]</ety> <fld>(Zool.)</fld> <def>A California fish (<spn>Hemilepidotus spinosus</spn>), allied to the sculpin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabiai</ent><br/
<hw>Cab"i*ai</hw> <pr>(k<acr/b"<icr/*<imac/)</pr>, <pos>n.</pos> <ety>[Native South American name.]</ety> <fld>(Zool.)</fld> <def>The capybara. See <er>Capybara</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabin</ent><br/
<hw>Cab"in</hw> <pr>(k<acr/b"<icr/n)</pr>, <pos>n.</pos> <ety>[OF. <ets>caban</ets>, fr. W. <ets>caban</ets> booth, cabin, dim. of <ets>cab</ets> cot, tent; or fr. F. <ets>cabane</ets>, <ets>cabine</ets>, LL. <ets>cabanna</ets>, perh. from the Celtic.]</ety> <sn>1.</sn> <def>A cottage or small house; a hut.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A hunting <qex>cabin</qex> in the west.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small room; an inclosed place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So long in secret <qex>cabin</qex> there he held<br/
Her captive.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A room in ship for officers or passengers.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cabin boy</b></col>, <cd>a boy whose duty is to wait on the officers and passengers in the cabin of a ship.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabin</ent><br/
<hw>Cab"in</hw> <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cabined</conjf> <pr>(-<icr/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cabining</conjf>.]</vmorph> <def>To live in, or as in, a cabin; to lodge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll make you . . . <qex>cabin</qex> in a cave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabin</ent><br/
<hw>Cab"in</hw>, <pos>v. t.</pos> <def>To confine in, or as in, a cabin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>cabined</qex>, cribbed, confined, bound in<br/
To saucy doubts and fears.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabinet</ent><br/
<hw>Cab"i*net</hw> <pr>(k<acr/b"<icr/*n<ecr/t)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>cabine</ets> or <ets>cabane</ets>. See <er>Cabin</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A hut; a cottage; a small house.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hearken a while from thy green <qex>cabinet</qex>,<br/
The rural song of careful Colinet.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small room, or retired apartment; a closet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A private room in which consultations are held.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Philip passed some hours every day in his father's <ex>cabinet</ex>.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The advisory council of the chief executive officer of a nation; a cabinet council.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, the <ex>cabinet</ex> or <xex>cabinet council</xex> consists of those privy councilors who actually transact the immediate business of the government. <au>Mozley & W.</au> -- In the United States, the <ex>cabinet</ex> is composed of the heads of the executive departments of the government, namely, the Secretary of State, of the Treasury, of War, of the Navy, of the Interior, and of Agiculture, the Postmaster-general, and the Attorney-general.</note><-- ##??add new departments, eliminate war & navy: defense, commerce, Education, HHS --><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <sd>(a)</sd> <def>A set of drawers or a cupboard intended to contain articles of value. Hence:</def> <sd>(b)</sd> <def>A decorative piece of furniture, whether open like an <eacute/tag<egrave/re or closed with doors. See <er><eacute/tag<egrave/re</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Any building or room set apart for the safe keeping and exhibition of works of art, etc.; also, the collection itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cabinet council</b></col>. <sd>(a)</sd> <cd>Same as <er>Cabinet</er>, <pos>n.</pos>, 4 (of which body it was formerly the full title).</cd> <sd>(b)</sd> <cd>A meeting of the cabinet.</cd> -- <col><b>Cabinet councilor</b></col>, <cd>a member of a cabinet council.</cd> -- <col><b>Cabinet photograph</b></col>, <cd>a photograph of a size smaller than an imperial, though larger than a <cref>carte de visite</cref>.</cd> -- <col><b>Cabinet picture</b></col>, <cd>a small and generally highly finished picture, suitable for a small room and for close inspection.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabinet</ent><br/
<hw>Cab"i*net</hw>, <pos>a.</pos> <def>Suitable for a cabinet; small.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Varnhagen von Ense] is a walking <qex>cabinet</qex> edition of Goethe.</q> <rj><qau>For. Quar. Rev.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabinet</ent><br/
<hw>Cab"i*net</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cabineted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cabineting</conjf>.]</vmorph> <def>To inclose</def> <mark>[R.]</mark>  <rj><au>Hewyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabinetmaker</ent><br/
<hw>Cab"i*net*mak`er</hw> <pr>(-m<amac/k`<etil/r)</pr>, <pos>n.</pos> <def>One whose occupation is to make cabinets or other choice articles of household furniture, as tables, bedsteads, bureaus, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabinetmaking</ent><br/
<hw>Cab"i*net*mak`ing</hw>, <pos>n.</pos> <def>The art or occupation of making the finer articles of household furniture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabinetwork</ent><br/
<hw>Cab"i*net*work`</hw> <pr>(-w<ucir/rk`)</pr>, <pos>n.</pos> <def>The art or occupation of working upon wooden furniture requiring nice workmanship; also, such furniture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabirean</ent><br/
<hw>Cab`i*re"an</hw> <pr>(k<acr/b`<icr/*r<emac/"<ait/n)</pr>, <pos>n.</pos> <def>One of the Cabiri.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabiri</ent><br/
||<hw>Ca*bi"ri</hw> <pr>(k<adot/*b<imac/"r<imac/)</pr>, <pos>prop. n. pl.</pos> <ety>[ NL., fr. Gr. <ets><grk>Ka`beiroi</grk></ets>.]</ety> <fld>(Myth.)</fld> <def>Certain deities originally worshiped with mystical rites by the Pelasgians in Lemnos and Samothrace and afterwards throughout Greece; -- also called sons of Heph<ae/stus (or Vulcan), as being masters of the art of working metals.</def> <altsp>[Written also <asp>Cabeiri</asp>.]</altsp>  <rj><au>Liddell & Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabirian</ent><br/
<hw>Ca*bir"i*an</hw> <pr>(k<adot/*b<icr/r"<icr/*<ait/n)</pr>, <pos>a.</pos> <def>Same as <er>Cabiric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabiric</ent><br/
<hw>Ca*bir"ic</hw> <pr>(k<adot/*b<icr/r"<icr/k)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>Cabirique</ets>]</ety> <def>Of or pertaining to the Cabiri, or to their mystical worship.</def> <altsp>[Written also <asp>Cabiritic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cable</ent><br/
<hw>Ca"ble</hw> <pr>(k<amac/"b'l)</pr>, <pos>n.</pos> <ety>[F. <ets>c<acir/ble</ets>, LL. <ets>capulum</ets>, <ets>caplum</ets>, a rope, fr. L. <ets>capere</ets> to take; cf. D., Dan., & G. <ets>kabel</ets>, from the French. See <er>Capable</er>.]</ety> <sn>1.</sn> <def>A large, strong rope or chain, of considerable length, used to retain a vessel at anchor, and for other purposes. It is made of hemp, of steel wire, or of iron links.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rope of steel wire, or copper wire, usually covered with some protecting or insulating substance; <as>as, the <ex>cable</ex> of a suspension bridge; a telegraphic <ex>cable</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch)</fld> <def>A molding, shaft of a column, or any other member of convex, rounded section, made to resemble the spiral twist of a rope; -- called also <altname>cable molding</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bower cable</b></col>, <cd>the cable belonging to the bower anchor.</cd> -- <col><b>Cable road</b></col>, <cd>a railway on which the cars are moved by a continuously running endless rope operated by a stationary motor.</cd> -- <col><b>Cable's length</b></col>, <cd>the length of a ship's cable. Cables in the merchant service vary in length from 100 to 140 fathoms or more; but as a maritime measure, a cable's length is either 120 fathoms (720 feet), or about 100 fathoms (600 feet, an approximation to one tenth of a nautical mile).</cd> -- <col><b>Cable tier</b></col>. <sd>(a)</sd> <cd>That part of a vessel where the cables are stowed.</cd> <sd>(b)</sd> <cd>A coil of a cable.</cd> -- <col><b>Sheet cable</b></col>, <cd>the cable belonging to the sheet anchor.</cd> -- <col><b>Stream cable</b></col>, <cd>a hawser or rope, smaller than the bower cables, to moor a ship in a place sheltered from wind and heavy seas.</cd> -- <col><b>Submarine cable</b></col>. <cd>See <er>Telegraph</er>.</cd> -- <mcol><col><b>To pay out the cable</b></col>, <col><b>To veer out the cable</b></col></mcol>, <cd>to slacken it, that it may run out of the ship; to let more cable run out of the hawse hole.</cd> -- <col><b>To serve the cable</b></col>, <cd>to bind it round with ropes, canvas, etc., to prevent its being, worn or galled in the hawse, et.</cd> -- <col><b>To slip the cable</b></col>, <cd>to let go the end on board and let it all run out and go overboard, as when there is not time to weigh anchor. Hence, in sailor's use, to die.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 200 --></p>

<p><ent>Cable</ent><br/
<hw>Ca"ble</hw> <pr>(k<amac/"b'l)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To fasten with a cable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>To ornament with cabling.  See <er>Cabling</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cable</ent><br/
<hw>Ca"ble</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cabled</conjf> <pr>(-b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cabling</conjf> <pr>(-bl<icr/ng)</pr>.]</vmorph> <def>To telegraph by a submarine cable</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabled</ent><br/
<hw>Ca"bled</hw> <pr>(-b'ld)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Fastened with, or attached to, a cable or rope.</def> <ldquo/The <xex>cabled</xex> stone.<rdquo/  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>Adorned with cabling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cablegram</ent><br/
<hw>Ca"ble*gram`</hw> <pr>(k<amac/"b'l*gr<acr/m`)</pr>, <pos>n.</pos> <ety>[<ets>Cable</ets>, n. + Gr. <grk>gra`mma</grk> a writing, a letter.]</ety> <def>A message sent by a submarine telegraphic cable.</def> <note>[A recent hybrid, sometimes found in the newspapers.]</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cablelaid</ent><br/
<hw>Ca"ble*laid`</hw> <pr>(-l<amac/d`)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <def>Composed of three three-stranded ropes, or hawsers, twisted together to form a cable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Twisted after the manner of a cable; <as>as, a <ex>cable-laid</ex> gold chain</as>.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cablet</ent><br/
<hw>Ca"blet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>cable</ets>; cf. F. <ets>c<acir/blot</ets>.]</ety> <def>A little cable less than ten inches in circumference.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabling</ent><br/
<hw>Ca"bling</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The decoration of a fluted shaft of a column or of a pilaster with reeds, or rounded moldings, which seem to be laid in the hollows of the fluting.  These are limited in length to about one third of the height of the shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabman</ent><br/
<hw>Cab"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cabmen</plw> <pr>(<?/)</pr>.</plu> <def>The driver of a cab.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabob</ent><br/
<hw>Ca*bob"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hindi <ets>kab<amac/b</ets>]</ety> <altsp>[Now more commonly spelled <asp>kabob</asp>.]</altsp> <sn>1.</sn> <def>A small piece of mutton or other meat roasted on a skewer; -- so called in Turkey and Persia.</def> <altsp>[Also spelled <asp>kebab</asp>, <asp>kebob</asp>, or <asp>kabab</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A leg of mutton roasted, stuffed with white herrings and sweet herbs.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>shish cabob</b></col> <cd>See <er>shish kebob</er>.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Cabob</ent><br/
<hw>Ca*bob"</hw>, <pos>v. t.</pos>  <altsp>[Now more commonly spelled <asp>kabob</asp>.]</altsp> <def>To roast, as a cabob.</def>  <rj><au>Sir. T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caboched</ent><br/
<hw>Ca*boched"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>caboche</ets> head. Cf. 1st <er>Cabbage</er>.]</ety> <fld>(Her.)</fld> <def>Showing the full face, but nothing of the neck; -- said of the head of a beast in armorial bearing.</def> <altsp>[Written also <asp>caboshed</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>cabochon</ent><br/
||<hw>ca`bo`chon"</hw> <pr>(k<acr/`b<ucr/*sh<aum/n"; Fr. k<adot/`b<osl/`sh<ocir/N")</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Jewelry)</fld> <def>A stone of convex form, highly polished, but not faceted; also, the style of cutting itself.  Such stones are said to be cut <xex>en cabochon</xex>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>cabochon</ent><br/
||<hw>ca`bo`chon"</hw> <pr>(k<acr/`b<ucr/*sh<aum/n"; Fr. k<adot/`b<osl/`sh<ocir/N")</pr>, <pos>a.</pos> <ety>[see <er>cabochon</er>, <pos>n.</pos>.]</ety> <fld>(Jewelry)</fld> <def>Of, pertaining to, containing, or in the style of, a <er>cabochon</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cabombaceae</ent><br/
<hw>Cabombaceae</hw> <pos>prop. n.</pos> <def>A subfamily of plants, in some classifications considered as an independent family of water lilies; it comprises the genera <gen>Cabomba</gen> and <gen>Brasenia</gen>.</def><br/
<syn><b>Syn. --</b> subfamily <fam>Cabombaceae</fam>, water-shield family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caboodle</ent><br/
<hw>ca*boo"dle</hw> <pr>(k<adot/*b<oomac/"d'l)</pr>, <pos>n.</pos> <def>The whole collection; the entire quantity or number; -- usually in the phrase <col><b>the whole caboodle</b></col> or <col><b>the whole kit and caboodle</b></col>.</def> <mark>[Slang, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Caboose</ent><br/
<hw>Ca*boose"</hw> <pr>(k<adot/*b<oomac/s")</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>kabuis</ets>, <ets>kombuis</ets>, Dan. <ets>kabys</ets>, Sw. <ets>kabysa</ets>, G. <ets>kabuse</ets> a little room or hut.  The First part of the word seems to be allied to W. <ets>cab</ets> cabin, booth.  Cf. <er>Cabin</er>.]</ety> <altsp>[Written also <asp>camboose</asp>.]</altsp> <sn>1.</sn> <fld>(Naut.)</fld> <def>A house on deck, where the cooking is done; -- commonly called the <altname>galley</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Railroad)</fld> <def>A car used on freight or construction trains as travelling quarters for brakemen, workmen, etc.; a tool car.  It usually is the last car of the train.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cabotage</ent><br/
<hw>Cab"o*tage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cabotage</ets>, fr. <ets>caboter</ets> to sail along the coast; cf. Sp. <ets>cabo</ets> cape.]</ety> <fld>(Naut.)</fld> <def>Navigation along the coast; the details of coast pilotage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabree</ent><br/
||<hw>Ca*br<eacute/e"</hw> <pr>(k<adot/*br<asl/")</pr>, <pos>n.</pos> <ety>[French Canadian.]</ety> <fld>(Zool.)</fld> <def>The pronghorn antelope.</def> <altsp>[Also written <asp>cabrit</asp>, <asp>cabret</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabrerite</ent><br/
<hw>Ca*brer"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>An apple-green mineral, a hydrous arseniate of nickel, cobalt, and magnesia; -- so named from the Sierra <etsep>Cabrera</etsep>, Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabrilla</ent><br/
||<hw>Ca*bril"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., prawn.]</ety> <fld>(Zool)</fld> <def>A name applied to various species of edible fishes of the genus <gen>Serranus</gen>, and related genera, inhabiting the Meditarranean, the coast of California, etc.  In California, some of them are also called <stype>rock bass</stype> and <stype>kelp salmon</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabriole</ent><br/
<hw>Cab"ri*ole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Cabriolet</er>, and cf. <er>Capriole</er>.]</ety> <fld>(Man.)</fld> <def>A curvet; a leap. See <er>Capriole</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cabrioles</qex> which his charger exhibited.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabriolet</ent><br/
<hw>Cab`ri*o*let"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>cabriole</ets> a leap, caper, from It. <ets>capriola</ets>, fr. dim. of L. <ets>caper</ets> he-goat, <ets>capra</ets> she-goat.  This carriage is so called from its skipping lightness.  Cf. <er>Cab</er>, <er>Caper</er> a leap.]</ety> <def>A one-horse carriage with two seats and a calash top.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabrit</ent><br/
<hw>Ca*brit"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Cabr<eacute/e</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caburn</ent><br/
<hw>Cab"urn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cable</er>, <pos>n.</pos>]</ety> <fld>(Naut.)</fld> <def>A small line made of spun yarn, to bind or worm cables, seize tackles, etc.</def></p>

<p><ent>ca-ca</ent><br/
<hw>ca-ca</hw> <pos>n.</pos> <sn>n.</sn> <ety>[see <er>cack</er>.]</ety> <def>Feces; excrement; -- used in talking with very young children.</def> <mark>[baby talk]</mark><br/
[<source>PJC</source>]</p>

<p><ent>ca-ca</ent><br/
<hw>ca-ca</hw> <pos>v.</pos>  <ety>[see <er>cack</er>.]</ety> <def>to have a bowel movement.</def> <mark>[baby talk]</mark><br/
<syn><b>Syn. --</b> defecate, shit, take a shit, take a crap, crap, make.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cachaemia</ent><br/
<mhw>{||<hw>Ca*c<ae/"mi*a</hw> <pr>(k<adot/*s<emac/"m<icr/*<adot/)</pr>, ||<hw>Ca*ch<ae/"mi*a</hw> <pr>(k<adot/*k<esl/"m<icr/*<adot/)</pr>,}</mhw> <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kako`s</grk> bad+ <grk>a"i^ma</grk> blood.]</ety> <fld>(Med.)</fld> <def>A degenerated or poisoned condition of the blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cacaine</ent><br/
<hw>ca*ca"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The essential principle of cacao; -- now called <altname>theobromine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacajao</ent><br/
||<hw>Ca*ca*j<atil/o"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg.]</ety> <fld>(Zool)</fld> <def>A South American short-tailed monkey (<spn>Pithecia melanocephala</spn> syn. <spn>Brachyurus melanocephala</spn>).</def> <altsp>[Written also <asp>cacajo</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>cacao</ent><br/
<hw>ca*ca"o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. Mex. <ets>kakahuatl</ets>. Cf. <er>Cocoa</er>, <er>Chocolate</er>]</ety> <fld>(Bot.)</fld> <def>A small evergreen tree (<spn>Theobroma Cacao</spn>) of South America and the West Indies.  Its fruit contains an edible pulp, inclosing seeds about the size of an almond, from which cocoa, chocolate, and broma are prepared.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachemia</ent><br/
<ent>Cachaemia</ent><br/
<mhw>||<hw>Ca*ch<ae/"mi*a</hw>, ||<hw>Ca*che"mi*a</hw> <pr>(<?/)</pr></mhw>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ bad + <?/ blood.]</ety> <fld>(Med.)</fld> <def>A degenerated or poisoned condition of the blood.</def> -- <wordforms><wf>Ca*ch<ae/"mic</wf>, <wf>Ca*che"mic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cachalot</ent><br/
<hw>cach"a*lot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cachalot</ets>.]</ety> <fld>(Zool.)</fld> <def>The sperm whale (<spn>Physeter macrocephalus</spn>).  It has in the top of its head a large cavity, containing an oily fluid, which, after death, concretes into a whitish crystalline substance called <prod>spermaceti</prod>. See <er>Sperm whale</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cache</ent><br/
<hw>cache</hw> <pr>(k<acr/sh)</pr>, <pos>n.</pos> <ety>[F., a hiding place, fr. <ets>cacher</ets> to conceal, to hide.]</ety> <sn>1.</sn> <def>A hole in the ground, or other hiding place, for concealing and preserving provisions which it is inconvenient to carry.</def>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is hidden in a cache{2}; a hoard; a stockpile.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Computers)</fld> <def>A form of memory in a computer which has a faster access time than most of main memory, and is usually used to store the most frequently accessed data in main memory during execution of a program.</def><br/
[<source>PJC</source>]</p>

<p><ent>cache</ent><br/
<hw>cache</hw> <pr>(k<acr/sh)</pr>, <pos>v. t.</pos> <def>To store in a cache{1}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cachectical</ent><br/
<ent>Cachectic</ent><br/
<mhw>{ <hw>Ca*chec"tic</hw> <pr>(?)</pr>, <hw>Ca*chec"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cachecticus</ets>, Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/: cf. F. <ets>cachectique</ets>.]</ety> <def>Having, or pertaining to, cachexia; <as>as, <ex>cachectic</ex> remedies; <ex>cachectical</ex> blood</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachepot</ent><br/
||<hw>Cache`pot"</hw> <pr>(k<adot/sh`p<osl/")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>cacher</ets> to hide + <ets>pot</ets> a pot.]</ety> <def>An ornamental casing for a flowerpot, of porcelain, metal, paper, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachet</ent><br/
||<hw>Cach"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. fr. <ets>cacher</ets> to hide.]</ety> <def>A seal, as of a letter.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lettre de cachet</b></col> <ety>[F.]</ety>, <cd>a sealed letter, especially a letter or missive emanating from the sovereign; -- much used in France before the Revolution as an arbitrary order of imprisonment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachexy</ent><br/
<ent>Cachexia</ent><br/
<mhw>{ ||<hw>Ca*chex"i*a</hw> <pr>(?)</pr>, <hw>Ca*chex"y</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>cachexia</ets>, Gr. <grk>kachexi`a</grk>; <grk>kako`s</grk> bad + <grk>"e`xis</grk> condition.]</ety> <def>A condition of ill health and impairment of nutrition due to impoverishment of the blood, esp. when caused by a specific morbid process (as cancer or tubercle).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachinnation</ent><br/
<hw>Cach`in*na"tion</hw> <pr>(k<acr/k`<icr/n*n<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>cachinnatio</ets>, fr. <ets>cachinnare</ets> to laugh aloud, cf. Gr. <grk>kacha`zein</grk>.]</ety> <def>Loud or immoderate laughter; -- often a symptom of hysterical or maniacal affections.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hideous grimaces . . . attended this unusual <qex>cachinnation</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachinnatory</ent><br/
<hw>Ca*chin"na*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of, or accompanied by, immoderate laughter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cachinnatory</qex> buzzes of approval.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachiri</ent><br/
||<hw>Ca*chi"ri</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fermented liquor made in Cayenne from the grated root of the manioc, and resembling perry.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacholong</ent><br/
<hw>Cach"o*long</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cacholong</ets>, said to be from <ets>Cach</ets>, the name of a river in Bucharia + <ets>cholon</ets>, a Calmuck word for <ets>stone</ets>; or fr. a Calmuck word meaning <ldquo/beautiful stone<rdquo/]</ety> <fld>(Min.)</fld> <def>An opaque or milk-white chalcedony, a variety of quartz; also, a similar variety of opal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachou</ent><br/
<hw>Ca`chou"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <ets>Cashoo</ets>.]</ety> <def>A silvered aromatic pill, used to correct the odor of the breath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachucha</ent><br/
||<hw>Ca*chu"cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>An Andalusian dance in three-four time, resembling the bolero.</def> <altsp>[Sometimes in English spelled <asp>cachuca</asp> <pr>(<?/)</pr>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The orchestra plays the <qex>cachucha</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cachunde</ent><br/
||<hw>Ca*chun"de</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <fld>(Med.)</fld> <def>A pastil or troche, composed of various aromatic and other ingredients, highly celebrated in India as an antidote, and as a stomachic and antispasmodic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacique</ent><br/
||<hw>Ca*cique"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>See Cazique.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cack</ent><br/
<hw>Cack</hw> <pr>(k<acr/k)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>cakken</ets>, fr. L. <ets>cacare</ets>; akin to Gr. <grk>kakka^n</grk>, and to OIr. <ets>cacc</ets> dung; cf. AS. <ets>cac</ets>.]</ety> <def>To ease the body by stool; to go to stool.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cackerel</ent><br/
<hw>Cack"er*el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>caquerel cagarel</ets> (<ets>Cotgr</ets>.), from the root of E. <ets>cack</ets>.]</ety> <fld>(Zool.)</fld> <def>The mendole; a small worthless Mediterranean fish considered poisonous by the ancients. See <er>Mendole</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cackle</ent><br/
<hw>Cac"kle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cackled</conjf> <pr>(-k'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cackling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>cakelen</ets>; cf. LG. <ets>kakeln</ets>, D. <ets>kakelen</ets>, G. <ets>gackeln</ets>, <ets>gackern</ets>; all of imitative origin. Cf. <er>Gagle</er>, <er>Cake</er> to cackle.]</ety> <sn>1.</sn> <def>To make a sharp, broken noise or cry, as a hen or goose does.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When every goose is <qex>cackling</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To laugh with a broken noise, like the cackling of a hen or a goose; to giggle.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To talk in a silly manner; to prattle.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cackle</ent><br/
<hw>Cac"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The sharp broken noise made by a goose or by a hen that has laid an egg.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By her <qex>cackle</qex> saved the state.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Idle talk; silly prattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a buzz and <qex>cackle</qex> all around regarding the sermon.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cackler</ent><br/
<hw>Cac"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A fowl that cackles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who prattles, or tells tales; a tattler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cackling</ent><br/
<hw>Cac"kling</hw>, <pos>n.</pos> <def>The broken noise of a goose or a hen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacochymy</ent><br/
<ent>Cacochymia</ent><br/
<mhw>{ ||<hw>Cac`o*chym"i*a</hw> <pr>(?)</pr>, <hw>Cac"o*chym`y</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>cacochymia</ets>, fr. Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/; <grk>kako`s</grk> bad + <?/<?/<?/<?/<?/ juice: cf. F. <ets>cacochymie</ets>.]</ety> <fld>(Med.)</fld>  <def>A vitiated state of the humors, or fluids, of the body, esp. of the blood.</def> -- <wordforms><wf>Cac`o*chym"ic</wf> <pr>(#)</pr>, <wf>Cac`o*chym"ic*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cacochymical</ent><br/
<ent>Cacochymic</ent><br/
<mhw>{ <hw>Cac`o*chym"ic</hw> <pr>(?)</pr>, <hw>Cac`o*chym"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having the fluids of the body vitiated, especially the blood.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacodemon</ent><br/
<hw>Cac`o*de"mon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/<?/; <grk>kako`s</grk> bad + <?/<?/<?/<?/<?/<?/ demon: cf. F. <ets>cacod<eacute/mon</ets>.]</ety> <sn>1.</sn> <def>An evil spirit; a devil or demon.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The nightmare.</def>  <rj><au>Dunaglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacodoxical</ent><br/
<hw>Cac`o*dox"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Heretical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacodoxy</ent><br/
<ent>Cacodoxy</ent><br/
<hw>Cac"o*dox`y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/ perverted opinion; <grk>kako`s</grk> bad + <?/<?/<?/<?/<?/ opinion.]</ety> <def>Erroneous doctrine; heresy; heterodoxy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Heterodoxy, or what Luther calls <qex>cacodoxy</qex>.</q> <rj><qau>R. Turnbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacodyl</ent><br/
<hw>Cac"o*dyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/<?/<?/ ill-smelling (<grk>kako`s</grk> bad + <?/<?/<?/<?/<?/ to smell) + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>Alkarsin; a colorless, poisonous, arsenical liquid, <chform>As2(CH3)4</chform>, spontaneously inflammable and possessing an intensely disagreeable odor. It is the type of a series of compounds analogous to the nitrogen compounds called hydrazines.</def> <altsp>[Written also <asp>cacodyle</asp>, and <asp>kakodyl</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacodylic</ent><br/
<hw>Cac`o*dyl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, cacodyl.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cacodylic acid</b></col>, <cd>a white, crystalline, deliquescent substance, <chform>(CH3)2AsO.OH</chform>, obtained by the oxidation of cacodyl, and having the properties of an exceedingly stable acid; -- also called <altname>alkargen</altname>.</cd></cs>
<-- # error in original formula corrected! --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacoethes</ent><br/
||<hw>Cac`o*<eum/"thes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/<?/<?/<?/<?/<?/ of ill habits, <?/<?/ <?/<?/<?/<?/<?/<?/<?/<?/ an ill habit; <grk>kako`s</grk>  bad + <?/ habit]</ety> <sn>1.</sn> <def>A bad custom or habit; an insatiable desire; <as>as, <ex>caco<eum/thes scribendi</ex>, <ldquo/The itch for writing<rdquo/</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A bad quality or disposition in a disease; an incurable ulcer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacogastric</ent><br/
<hw>Cac`o*gas"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kako`s</grk> bad + <?/<?/<?/<?/<?/<?/ stomach.]</ety> <def>Troubled with bad digestion.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cacogenics</ent><br/
<hw>cacogenics</hw> <pos>n.</pos> <def>the study of the operation of factors causing degeneration in the type of offspring produced.</def><br/
<syn><b>Syn. --</b> dysgenics.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cacographic</ent><br/
<hw>Cac`o*graph`ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or characterized by, cacography; badly written or spelled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacography</ent><br/
<hw>Ca*cog`ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kako`s</grk> bad + <ets>-graphy</ets>; cf. F. <ets>cacographie</ets>.]</ety> <def>Incorrect or bad writing or spelling.</def>  <rj><au>Walpole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacolet</ent><br/
||<hw>Ca`co*let"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A chair, litter, or other contrivance fitted to the back or pack saddle of a mule for carrying travelers in mountainous districts, or for the transportation of the sick and wounded of an army.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacology</ent><br/
<hw>Ca*col"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kako`s</grk> bad + <ets>-logy</ets>: cf. F. <ets>cacologie</ets>.]</ety> <def>Bad speaking; bad choice or use of words.</def>  <rj><au>Buchanan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacomixl</ent><br/
<ent>Cacomixtle</ent><br/
<ent>Cacomixle</ent><br/
<mhw>{ ||<hw>Ca`co*mix"le</hw> <pr>(?)</pr>, <hw>Ca`co*mix"tle</hw> <pr>(?)</pr>, <hw>Ca"co*mix`l</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Mexican name.]</ety> <def>A North American carnivore (<spn>Bassaris astuta</spn>), about the size of a cat, related to the raccoons. It inhabits Mexico, Texas, and California.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacoon</ent><br/
<hw>Ca*coon"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the seeds or large beans of a tropical vine (<spn>Entada scandens</spn>) used for making purses, scent bottles, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacophonious</ent><br/
<ent>Cacophonous</ent><br/
<ent>Cacophonical</ent><br/
<ent>Cacophonic</ent><br/
<mhw>{ <hw>Cac`o*phon"ic</hw> <pr>(?)</pr>, <hw>Cac`o*phon"ic*al</hw> <pr>(?)</pr>, <hw>Ca*coph"o*nous</hw> <pr>(?)</pr>, <hw>Cac`o*pho"ni*ous</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <def>Harsh-sounding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacophony</ent><br/
<hw>Ca*coph"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cacophonies</plw> <pr>(#)</pr>.</plu> <ety>[Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/; <grk>kako`s</grk> bad + <?/<?/<?/<?/ sound: cf. F. <ets>Cacophonie</ets>.]</ety> <sn>1.</sn> <fld>(Rhet.)</fld> <def>An uncouth or disagreable sound of words, owing to the concurrence of harsh letters or syllables.</def> <ldquo/<xex>Cacophonies</xex> of all kinds.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A combination of discordant sounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>An unhealthy state of the voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacostomia</ent><br/
||<hw>Cac`o*sto"mi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kako`s</grk> bad + <?/ mouth.]</ety> <fld>(Med.)</fld> <def>Diseased or gangrenous condition of the mouth.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cacotechny</ent><br/
<hw>Cac"o*tech`ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>kako`s</grk> bad + <?/ art.]</ety> <def>A corruption or corrupt state of art.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacoxenite</ent><br/
<ent>Cacoxene</ent><br/
<mhw>{ <hw>Ca*cox"ene</hw> <pr>(?)</pr>, <hw>Ca*cox"e*nite</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Gr. <grk>kako`s</grk> bad + <?/<?/<?/<?/<?/ guest.]</ety> <fld>(Min.)</fld> <def>A hydrous phosphate of iron occurring in yellow radiated tufts. The phosphorus seriously injures it as an iron ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cactaceous</ent><br/
<hw>Cac*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Belonging to, or like, the family of plants of which the prickly pear is a common example.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cactus</ent><br/
<hw>Cac"tus</hw> <pr>(?)</pr>, <pos>n.</pos> ; <plu><it>pl.</it> E. <plw>Cactuses</plw> <pr>(#)</pr>, <plw>Cacti</plw> <pr>(-t<imac/)</pr>.</plu> <ety>[L., a kind of cactus, Gr. <?/<?/<?/<?/<?/<?/.]</ety> <fld>(Bot.)</fld> <def>Any plant of the order <ord>Cactac<ae/</ord>, as the prickly pear and the night-blooming cereus. See <er>Cereus</er>. They usually have leafless stems and branches, often beset with clustered thorns, and are mostly natives of the warmer parts of America.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cactus wren</b></col> <fld>(Zool.)</fld>, <cd>an American wren of the genus <gen>Campylorhynchus</gen>, of several species.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacuminal</ent><br/
<hw>Ca*cu"mi*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cacumen</ets>, <ets>cacuminis</ets>, the top, point.]</ety> <fld>(Philol.)</fld> <def>Pertaining to the top of the palate; cerebral; -- applied to certain consonants; <as>as, <ex>cacuminal</ex> (or cerebral) letters</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cacuminate</ent><br/
<hw>Ca*cu"mi*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>cacuminatus</ets>, p. p. of <ets>cacuminare</ets> to point, fr. <ets>cacumen</ets> point.]</ety> <def>To make sharp or pointed.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cad</ent><br/
<hw>Cad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. fr. <ets>cadet</ets>.]</ety> <sn>1.</sn> <def>A person who stands at the door of an omnibus to open and shut it, and to receive fares; an idle hanger-on about innyards.</def> <mark>[Eng.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lowbred, presuming person; a mean, vulgar fellow.</def> <mark>[Cant]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cadaster</ent><br/
<hw>cadaster</hw> <pos>n.</pos> <def>same as <er>cadastre</er>.</def><br/
<syn><b>Syn. --</b> cadastre.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cadastral</ent><br/
<hw>Ca*das"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>Of or pertaining to landed property.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cadastral survey</b></col>, <it>or</it> <col><b>Cadastral map</b></col></mcol>, <cd>a survey, map, or plan on a large scale (Usually <frac1x2500/ of the linear measure of the ground, or twenty-five inches to the mile or about an inch to the acre) so as to represent the relative positions and dimensions of objects and estates exactly; -- distinguished from a <xex>topographical</xex> map, which exaggerates the dimensions of houses and the breadth of roads and streams, for the sake of distinctness.</cd>  <rj><au>Brande & C.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadaster</ent><br/
<ent>Cadastre</ent><br/
<mhw>{ ||<hw>Ca*das"tre</hw>, <hw>Ca*das"ter</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cadastre</ets>.]</ety> <fld>(Law.)</fld> <def>An official statement of the quantity and value of real estate for the purpose of apportioning the taxes payable on such property; a public register showing the details of ownership and value of land.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 201 --></p>

<p><ent>Cadaver</ent><br/
||<hw>Ca*da"ver</hw> <pr>(k<adot/*d<acr/"v<etil/r; k<adot/*d<amac/"v<etil/r)</pr>, <pos>n.</pos> <ety>[L., fr <ets>cadere</ets> to fall.]</ety> <def>A dead human body; a corpse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadaveric</ent><br/
<hw>Ca*dav"er*ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a corpse, or the changes produced by death; cadaverous; <as>as, <ex>cadaveric</ex> rigidity</as>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cadaveric alkaloid</b></col>, <cd>an alkaloid generated by the processes of decomposition in dead animal bodies, and thought by some to be the cause of the poisonous effects produced by the bodies. See <er>Ptomaine</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadaverin</ent><br/
<ent>Cadaverine</ent><br/
<mhw>{ <hw>Ca*dav"er*ine</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Ca*dav"er*in</hw>  }</mhw>. <ety>[From <er>Cadaver</er> and am<ets>ine</ets>.]</ety> <fld>(Chem.)</fld> <def>A sirupy, nontoxic ptomaine, <chform>H2N.(CH2)5.NH2</chform> (chemically <altname>pentamethylene diamine</altname>), formed in putrefaction of flesh, etc.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Cadaverous</ent><br/
<hw>Ca*dav"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cadaverosus</ets>.]</ety> <sn>1.</sn> <def>Having the appearance or color of a dead human body; pale; ghastly; <as>as, a <ex>cadaverous</ex> look</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to, or having the qualities of, a dead body.</def> <ldquo/The scent <xex>cadaverous</xex>.<rdquo/</p>

<p>-- <wordforms><wf>Ca*dav"er*ous*ly</wf>, <pos>adv.</pos> -- <wf>Ca*dav"er*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadbait</ent><br/
<hw>Cad"bait`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>codbait</ets>, <ets>cadbote</ets> fly.]</ety> <fld>(Zool.)</fld> <def>See <er>Caddice</er>.</def></p>

<p><ent>Caddis</ent><br/
<ent>Caddice</ent><br/
<mhw>{ <hw>Cad"dice</hw>, <hw>Cad"dis</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>caddy</ets>, <ets>cadew</ets>; cf. G. <ets>k<oum/der</ets> bait.]</ety> <fld>(Zool.)</fld> <def>The larva of a caddice fly. These larv<ae/ generally live in cylindrical cases, open at each end, and covered externally with pieces of broken shells, gravel, bits of wood, etc.  They are a favorite bait with anglers. Called also <altname>caddice worm</altname>, or <altname>caddis worm</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caddice fly</ent><br/
<hw>Caddice fly</hw> <fld>(Zool.)</fld>, <def>a small mothlike species of trichopterous insect, whose larva is the caddice;  it has two pairs of hairy membranous wings and aquatic larvae.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Caddie</ent><br/
<hw>Cad"die</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>caddy</asp>, <asp>cadie</asp>, <asp>cady</asp>, and <asp>cawdy</asp>.]</altsp> <ety>[See <er>Cadet</er>.]</ety> <sn>1.</sn>  <def>A cadet.</def> <mark>[Obs. Scot.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A lad; young fellow.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>One who does errands or other odd jobs.</def> <mark>[Scot.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>An attendant who carries a golf player's clubs, tees his ball, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Caddis</ent><br/
<hw>Cad"dis</hw>, <pos>n.</pos> <ety>[OE. <ets>caddas</ets>, Scot. <ets>caddis</ets> lint, <ets>caddes</ets> a kind of woolen cloth, cf. Gael. <ets>cada</ets>, <ets>cadadh</ets>, a kind of cloth, cotton, fustian, W. <ets>cadas</ets>, F. <ets>cadis</ets>.]</ety> <def>A kind of worsted lace or ribbon.</def> <ldquo/<xex>Caddises</xex>, cambrics, lawns.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>caddisfly</ent><br/
<ent>caddis fly</ent><br/
<mhw><hw>caddis fly</hw>, <hw>caddisfly</hw></mhw> <pos>n.</pos> <def>same as <er>caddice fly</er>.</def><br/
<syn><b>Syn. --</b> caddice fly, caddicefly.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caddish</ent><br/
<hw>Cad"dish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a cad; lowbred and presuming.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caddow</ent><br/
<hw>Cad"dow</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE.  <ets>cadawe</ets>, prob. fr. <ets>ca</ets> chough + <ets>daw</ets> jackdaw; cf. Gael. <ets>cadhag</ets>, <ets>cathag</ets>. Cf. <er>Chough</er>, <er>Daw</er>, <pos>n.</pos>]</ety> <fld>(Zool.)</fld> <def>A jackdaw.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caddy</ent><br/
<hw>Cad"dy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Caddies</plw> <pr>(#)</pr>.</plu> <ety>[Earlier spelt <ets>catty</ets>, fr. Malay <ets>kat<imac/</ets> a weight of 1<frac13/ pounds. Cf. <er>Catty</er>.]</ety> <sn>1.</sn> <def>A small box, can, or chest to keep tea in, also called <altname>tea caddy</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a container to hold objects when not in use.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Computers)</fld> <def>a container to hold a compact disk, used in some types of compact disk devices, which is inserted into the CD player during playing, or in the case of recordable CD-ROMS, during recording.  It is approximately square and thin, slightly larger than the compact disk.  However, many CD players have a drawer for the compact disk, requiring no <ex>caddy</ex>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cade</ent><br/
<hw>Cade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. OE. <ets>cad</ets>, <ets>kod</ets>, lamb, also <er>Cosset</er>, <er>Coddle</er>.]</ety> <def>Bred by hand; domesticated; petted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He brought his <qex>cade</qex> lamb with him.</q> <rj><qau>Sheldon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cade</ent><br/
<hw>Cade</hw>, <pos>v. t.</pos> <def>To bring up or nourish by hand, or with tenderness; to coddle; to tame.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cade</ent><br/
<hw>Cade</hw>, <pos>n.</pos> <ety>[L. <ets>cadus</ets> jar, Gr. <?/.]</ety> <def>A barrel or cask, as of fish.</def> <ldquo/A <xex>cade</xex> of herrings.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cade</qex> of herrings is 500, of sprats 1,000.</q> <rj><qau>Jacob, Law Dict.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cade</ent><br/
<hw>Cade</hw>, <pos>n.</pos> <ety>[F. & Pr.; LL. <ets>cada</ets>.]</ety> <def>A species of juniper (<spn>Juniperus Oxycedrus</spn>) of Mediterranean countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Oil of cade</b></col>, <cd>a thick, black, tarry liquid, obtained by destructive distillation of the inner wood of the cade. It is used as a local application in skin diseases.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadence</ent><br/
<hw>Ca"dence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>cadence</ets>, <ets>cadens</ets>, LL. <ets>cadentia</ets> a falling, fr. L. <ets>cadere</ets> to fall; cf. F. <ets>cadence</ets>, It. <ets>cadenza</ets>. See <er>Chance</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The act or state of declining or sinking.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Now was the sun in western <qex>cadence</qex> low.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fall of the voice in reading or speaking, especially at the end of a sentence.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A rhythmical modulation of the voice or of any sound; <as>as, music of bells in <ex>cadence</ex> sweet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Blustering winds, which all night long<br/
Had roused the sea, now with hoarse <qex>cadence</qex> lull<br/
Seafaring men o'erwatched.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The accents . . . were in passion's tenderest <qex>cadence</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Rhythmical flow of language, in prose or verse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Golden <qex>cadence</qex> of poesy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If in any composition much attention was paid to the flow of the rhythm, it was said (at least in the 14th and 15th centuries) to be <ldquo/prosed in faire <qex>cadence</qex>.<rdquo/</q> <rj><qau>Dr. Guest.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Her.)</fld> <def>See <er>Cadency</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Man.)</fld> <def>Harmony and proportion in motions, as of a well-managed horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mil.)</fld> <def>A uniform time and place in marching.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The close or fall of a strain; the point of rest, commonly reached by the immediate succession of the tonic to the dominant chord.</def> <sd>(b)</sd> <def>A <xex>cadenza</xex>, or closing embellishment; a pause before the end of a strain, which the performer may fill with a flight of fancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Imperfect cadence</b></col>. <fld>(Mus.)</fld> <cd>See under <er>Imperfect</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadence</ent><br/
<hw>Ca"dence</hw>, <pos>v. t.</pos> <def>To regulate by musical measure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These parting numbers, <qex>cadenced</qex> by my grief.</q> <rj><qau>Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadency</ent><br/
<hw>Ca"den*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Descent of related families; distinction between the members of a family according to their ages.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Marks of cadency</b></col> <fld>(Her.)</fld>, <cd>bearings indicating the position of the bearer as older or younger son, or as a descendant of an older or younger son. See <er>Difference</er> <fld>(Her.)</fld>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadene</ent><br/
<hw>Ca*dene"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cad<egrave/ne</ets>.]</ety> <def>A species of inferior carpet imported from the Levant.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadent</ent><br/
<hw>Ca"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cadens</ets>, <ets>-entis</ets>, p. pr. of <ets>cadere</ets> to fall.]</ety> <def>Falling.</def> <mark>[R.]</mark> <ldquo/<xex>Cadent</xex> tears.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadenza</ent><br/
<hw>Ca*den"za</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A parenthetic flourish or flight of ornament in the course of a piece, commonly just before the final cadence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cader</ent><br/
<hw>Ca"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cadre</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadet</ent><br/
<hw>Ca*det"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cadet</ets> a younger or the youngest son or brother, dim. fr. L. <ets>caput</ets> head; <it>i. e.</it>, a smaller head of the family, after the first or eldest. See <er>Chief</er>, and cf. <er>Cad</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The younger of two brothers; a younger brother or son; the youngest son.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cadet</qex> of an ancient and noble family.</q> <rj><qau>Wood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A gentleman who carries arms in a regiment, as a volunteer, with a view of acquiring military skill and obtaining a commission.</def> <sd>(b)</sd> <def>A young man in training for military or naval service; esp. a pupil in a military or naval school, as at West Point, Annapolis, or Woolwich.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ All the undergraduates at Annapolis are <xex>Naval cadets</xex>. The distinction between <xex>Cadet midshipmen</xex> and <xex>Cadet engineers</xex> was abolished by Act of Congress in 1882.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In New Zealand, a young gentleman learning sheep farming at a station; also, any young man attached to a sheep station.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>A young man who makes a business of ruining girls to put them in brothels.</def> <mark>[Slang, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cadetship</ent><br/
<hw>Ca*det"ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The position, rank, or commission of a cadet; <as>as, to get a <ex>cadetship</ex></as>.</def></p>

<p><ent>Cadeworm</ent><br/
<ent>Cadew</ent><br/
<mhw>{ <hw>Ca*dew"</hw> <pr>(?)</pr>, <hw>Cade"worm`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A caddice. See <er>Caddice</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadge</ent><br/
<hw>Cadge</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cadged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cadging</conjf>.]</vmorph> <ety>[Cf. Scot. <ets>cache</ets>, <ets>caich</ets>, <ets>cadge</ets>, to toss, drive, OE. <ets>cachen</ets> to drive, catch, <ets>caggen</ets> to bind, or perh. E. <ets>cage</ets>. Cf. <er>Cadger</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To carry, as a burden.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hawk or peddle, as fish, poultry, etc.</def> <mark>[Prov.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To intrude or live on another meanly; to beg.</def> <mark>[Prov. or Slang, Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadge</ent><br/
<hw>Cadge</hw>, <pos>n.</pos> <ety>[Cf. 2d <er>Cadger</er>.]</ety> <fld>(Hawking)</fld> <def>A circular frame on which cadgers carry hawks for sale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadger</ent><br/
<hw>Cadg"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cadge</er>, <pos>v. t.</pos>, cf. <er>Codger</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A packman or itinerant huckster.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who gets his living by trickery or begging.</def> <mark>[Prov. or Slang]</mark> <ldquo/The gentleman <xex>cadger</xex>.<rdquo/  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadger</ent><br/
<hw>Cadg"er</hw>, <pos>n.</pos> <ety>[OF. <ets>cagier</ets> one who catches hawks. Cf. <er>Cage</er>.]</ety> <fld>(Hawking)</fld> <def>One who carries hawks on a cadge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadgy</ent><br/
<hw>Cadg"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cheerful or mirthful, as after good eating or drinking; also, wanton.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadi</ent><br/
<hw>Ca"di</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk. See <er>Alcalde</er>.]</ety> <def>An inferior magistrate or judge among the Mohammedans, usually the judge of a town or village.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caddie</ent><br/
<ent>Cadie</ent><br/
<mhw>{ <hw>Cad"ie</hw>, <hw>Cad"die</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A Scotch errand boy, porter, or messenger.</def> <altsp>[Written also <asp>cady</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Every Scotchman, from the peer to the <qex>cadie</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadilesker</ent><br/
<hw>Ca`di*les"ker</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>q<amac/<dsdot/<imac/</ets> judge + <ets>al'sker</ets> the army, Per. <ets>leshker</ets>.]</ety> <def>A chief judge in the Turkish empire, so named originally because his jurisdiction extended to the cases of soldiers, who are now tried only by their own officers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadillac</ent><br/
<hw>Ca*dil"lac</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. from <ets>Cadillac</ets>, a French town.]</ety> <def>A large pear, shaped like a flattened top, used chiefly for cooking.</def>  <rj><au>Johnson.</au></rj></p>

<p><sn>2.</sn> <ety>[From <ets>Cadillac</ets>, the high-end expensive line of automobiles made by General Motors, often thought of as the best American-made automobile.]</ety> <def>The best; -- metaphorical, from the automobile.  Often used in the phrase <ldquo/the <ex>Cadillac</ex> of . . . <rdquo/</def><br/
[<source>PJC</source>]</p>

<p><ent>Cadis</ent><br/
<hw>Cad"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A kind of coarse serge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadmean</ent><br/
<hw>Cad*me"an</hw> <pr>(k<acr/d*m<emac/"<ait/n)</pr>, <pos>a.</pos> <ety>[L. <etsep>Cadmeus</etsep>, Gr. <grk>Kadmei^os</grk>, from <grk>Ka`dmos</grk> (L. <ets>Cadmus</ets>), which name perhaps means lit. a man from the East; cf. Heb. <ets>qedem</ets> east.]</ety> <def>Of or pertaining to Cadmus, a fabulous prince of Thebes, who was said to have introduced into Greece the sixteen simple letters of the alphabet -- <alpha/, <beta/, <gamma/, <delta/, <epsilon/, <iota/, <kappa/, <lambda/, <mu/, <nu/, <omicron/, <pi/, <rho/, <sigma/, <tau/, <upsilon/. These are called <ex>Cadmean</ex> letters.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cadmean victory</b></col>, <cd>a victory that damages the victors as much as the vanquished; probably referring to the battle in which the soldiers who sprang from the dragon's teeth sown by Cadmus slew each other.  Similar to a <er>Pyhrric victory</er>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cadmia</ent><br/
<hw>Cad"mi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cadmia</ets> calamine, Gr. <?/. Cf. <er>Calamine</er>.]</ety> <fld>(Min.)</fld> <def>An oxide of zinc which collects on the sides of furnaces where zinc is sublimed. Formerly applied to the mineral <xex>calamine</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadmian</ent><br/
<hw>Cad"mi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <mark>[R.]</mark> <def>See <er>Cadmean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadmic</ent><br/
<hw>Cad"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, derived from, or containing, cadmium; <as>as, <ex>cadmic</ex> sulphide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadmium</ent><br/
<hw>Cad"mi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Cadmia</er>.]</ety> <fld>(Chem.)</fld> <def>A comparatively rare element related to zinc, and occurring in some zinc ores. It is a white metal, both ductile and malleable. Symbol Cd. Atomic weight 111.8. It was discovered by Stromeyer in 1817, who named it from its association with zinc or zinc ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cadmium yellow</b></col>, <cd>a compound of cadmium and sulphur, of an intense yellow color, used as a pigment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadrans</ent><br/
<hw>Cad"rans</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cadran</ets>. Cf. <er>Quadrant</er>.]</ety> <def>An instrument with a graduated disk by means of which the angles of gems are measured in the process of cutting and polishing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cadre</ent><br/
||<hw>Ca"dre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cadre</ets>, It. <ets>quadro</ets> square, from L. <ets>quadrum</ets>, fr. <ets>quatuor</ets> four.]</ety> <fld>(Mil.)</fld> <def>The framework or skeleton upon which a regiment is to be formed; the officers of a regiment forming the staff.</def> <altsp>[Written also <asp>cader</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caducary</ent><br/
<hw>Ca*du"ca*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Caducous</er>.]</ety> <fld>(Law)</fld> <def>Relating to escheat, forfeiture, or confiscation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caducean</ent><br/
<hw>Ca*du"ce*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or belonging to Mercury's caduceus, or wand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caduceus</ent><br/
<hw>Ca*du"ce*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caduceum</ets>, <ets>caduceus</ets>; akin to Gr. <?/ a herald's wand, fr. <?/ herald.]</ety> <fld>(Myth.)</fld> <def>The official staff or wand of Hermes or Mercury, the messenger of the gods. It was originally said to be a herald's staff of olive wood, but was afterwards fabled to have two serpents coiled about it, and two wings at the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caducibranchiate</ent><br/
<hw>Ca*du`ci*bran"chi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caducus</ets> falling (fr. <ets>cadere</ets> to fall) + E. <ets>branchiate</ets>.]</ety> <fld>(Zool.)</fld> <def>With temporary gills: -- applied to those Amphibia in which the gills do not remain in adult life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caducity</ent><br/
<hw>Ca*du"ci*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>caducitas</ets>: cf.  F. <ets>caducit<eacute/</ets>. See <er>Caducous</er>.]</ety> <def>Tendency to fall; the feebleness of old age; senility.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[A] jumble of youth and <qex>caducity</qex>.</q> <rj><qau>Chesterfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caducous</ent><br/
<hw>Ca*du"cous</hw> <pr>(?)</pr>, <ety>[L. <ets>caducus</ets> falling, inclined to fall, fr. <ets>cadere</ets> to fall. See <er>Cadence</er>.]</ety> <fld>(Bot. & Zool.)</fld> <def>Dropping off or disappearing early, as the calyx of a poppy, or the gills of a tadpole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caduke</ent><br/
<hw>Ca*duke"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>caduc</ets>. See <er>Caducous</er>.]</ety> <def>Perishable; frail; transitory.</def> <mark>[Obs.]</mark>  <rj><au>Hickes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>caduke</qex> pleasures of his world.</q> <rj><qau>Bp. Fisher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cady</ent><br/
<hw>Cad"y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cadie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caeca</ent><br/
||<hw>C<ae/"ca</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>C<ae/cum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caecal</ent><br/
<hw>C<ae/"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Of or pertaining to the c<ae/cum, or blind gut.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the form of a c<ae/cum, or bag with one opening; baglike; <as>as, the <ex>c<ae/cal</ex> extremity of a duct</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caecias</ent><br/
||<hw>C<ae/"ci*as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caecias</ets>, Gr. <?/.]</ety> <def>A wind from the northeast.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caecilian</ent><br/
<hw>C<ae/*cil"i*an</hw> <pr>(?; 106)</pr>, <pos>n.</pos> <ety>[L. <ets>caecus</ets> blind. So named from the supposed blindness of the species, the eyes being very minute.]</ety> <fld>(Zool.)</fld> <def>A limbless amphibian belonging to the order <ord>C<ae/cili<ae/</ord> or <ord>Ophimorpha</ord>. See <er>Ophiomorpha</er>.</def> <altsp>[Written also <asp>c<oe/cilian</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>caecum</ent><br/
<hw>cae"cum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>C<ae/cums</plw>, L. <plw>C<ae/ca</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>caecus</ets> blind, invisible, concealed.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>A cavity open at one end, as the blind end of a canal or duct.</def> <sd>(b)</sd> <def>The blind part of the large intestine beyond the entrance of the small intestine; -- called also the <altname>blind gut</altname>.</def> <altsp>[Also spelled <asp>cecum</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>c<ae/cum</xex> is comparatively small in man, and ends in a slender portion, the <xex>vermiform appendix</xex>; but in herbivorous mammals it is often as large as the rest of the large intestine. In fishes there are often numerous intestinal c<ae/ca.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caelatura</ent><br/
<hw>C<ae/`la*tu"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>caelare</ets> to engrave in relief.]</ety> <def>Art of producing metal decorative work other than statuary, as reliefs, intaglios, engraving, chasing, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Caenozoic</ent><br/
<hw>C<ae/`no*zo"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>See <er>Cenozoic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caen stone</ent><br/
<hw>Ca"en stone"</hw> <pr>(?)</pr>, <def>A cream-colored limestone for building, found near Caen, France.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caesar</ent><br/
<hw>c<ae/"sar</hw> <pr>(s<emac/"z<etil/r)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A Roman emperor, as being the successor of <person>Augustus C<ae/sar</person>.  Hence, a kaiser, or emperor of Germany, or any emperor or powerful ruler.  See <er>Kaiser</er>, <er>Kesar</er>, <er>Tsar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Marlborough anticipated the day when he would be servilely flattered and courted by <qex>C<ae/sar</qex> on one side and by Louis the Great on the other.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>C<ae/sarian</ent><br/
<ent>C<ae/sarean</ent><br/
<mhw>{ <hw>C<ae/*sa"re*an</hw>, <hw>C<ae/*sa"ri*an</hw> <pr>(s<esl/*z<amac/"r<icr/*<ait/n)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>Caesareus</ets>, <ets>Caesarianus</ets>.]</ety> <def>Of or pertaining to C<ae/sar or the C<ae/sars; imperial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caesarean</ent><br/
<hw>caesarean</hw> <pos>n.</pos> <def>same as <er>caesarean section</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caesarean section</ent><br/
<hw>caesarean section</hw> <pos>n.</pos> <fld>(Surg.)</fld>, <def>the operation of taking a child from the womb by cutting through the walls of the abdomen and uterus; -- so called because Julius C<ae/sar is reported to have been brought into the world by such an operation; -- called also <altname>caesarean</altname>.</def><br/
<syn><b>Syn. --</b> cesarean section, cesarian section, caesarean section, caesarian section, C-section, cesarean, cesarian.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caesarism</ent><br/
<hw>C<ae/"sar*ism</hw> <pr>(s<emac/"z<etil/r*<icr/z'm)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>C<eacute/sarisme</ets>.]</ety> <def>A system of government in which unrestricted power is exercised by a single person, to whom, as C<ae/sar or emperor, it has been committed by the popular will; imperialism; also, advocacy or support of such a system of government.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word came into prominence in the time of Napoleon III., as an expression of the claims and political views of that emperor, and of the politicians of his court.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caesaropapism</ent><br/
<hw>Caesaropapism</hw> <pos>n.</pos> <def>The doctrine that the state is supreme over the church in ecclesiastical matters.</def><br/
<syn><b>Syn. --</b> Erastianism, Byzantinism.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caesious</ent><br/
<hw>C<ae/"si*ous</hw> <pr>(s<emac/"z<esl/*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>caesius</ets> bluish gray.]</ety> <fld>(Nat. Hist.)</fld> <def>Of the color of lavender; pale blue with a slight mixture of gray.</def>  <rj><au>Lindley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caesium</ent><br/
<ent>caesium</ent><br/
<mhw><hw>caesium</hw>, <hw>C<ae/"si*um</hw></mhw> <pr>(s<emac/"z<esl/*<ucr/m)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>caesius</ets> bluish gray.]</ety> <fld>(Chem.)</fld> <def>A rare alkaline metal found in mineral water; -- so called from the two characteristic blue lines in its spectrum. It was the first element discovered by spectrum analysis, and is the most strongly basic and electro-positive substance known. Symbol Cs.  Atomic number 55.  Atomic weight 132.6.</def><br/
<syn><b>Syn. --</b> cesium, Cs</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caespitose</ent><br/
<hw>C<ae/s"pi*tose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Cespitose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caesura</ent><br/
<hw>cae*su"ra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>caesuras</plw> <pr>(<?/)</pr>, L. <plw>C<ae/sur<ae/</plw> <pr>(<?/)</pr></plu> <ety>[L. <ets>caesura</ets> a cutting off, a division, stop, fr. <ets>caedere</ets>, <ets>caesum</ets>, to cut off. See <er>Concise</er>.]</ety> <def>A metrical break in a verse, occurring in the middle of a foot and commonly near the middle of the verse; a sense pause in the middle of a foot. Also, a long syllable on which the c<ae/sural accent rests, or which is used as a foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the following line the <xex>c<ae/sura</xex> is between <xex>study</xex> and <xex>of</xex>.<br/
[<source>1913 Webster</source>]</p>

<p><q>The prop | er stud |  y || of | mankind | is man.</q>
</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a pause or interruption (as in a conversation); <as>as, after an ominous <ex>caesura</ex> the preacher continued</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caesural</ent><br/
<hw>C<ae/*su"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a c<ae/sura.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>C<ae/sural pause</b></col>, <cd>a pause made at a c<ae/sura.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cafe</ent><br/
||<hw>Ca`f<eacute/"</hw> <pr>(k<acr/*f<amac/" or F. k<adot/`f<asl/")</pr>, <pos>n.</pos> <ety>[F. See <er>Coffee</er>.]</ety> <def>A coffeehouse; a restaurant, especially a small restaurant where drinks and snacks are sold; also, a room in a hotel or restaurant where coffee and liquors are served.</def><br/
<syn><b>Syn. --</b> coffeehouse, coffee shop, coffee bar.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>cafe latte</ent><br/
<hw>ca`f<eacute/" lat"te</hw> <pr>(k<adot/`f<amac/"l<adot/*t<amac/")</pr>, <pos>n.</pos> <def>A type of espresso coffee topped with foamy steamed milk, and usually served in a tall glass or mug; also called <altname>latte</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cafeneh</ent><br/
<ent>Cafenet</ent><br/
<mhw>{ <hw>Caf"e*net</hw> <pr>(?)</pr>, <hw>Caf"e*neh</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Turk. <ets>qahveh kh<amac/neh</ets> coffeehouse.]</ety> <def>A humble inn or house of rest for travelers, where coffee is sold.</def> <mark>[Turkey]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cafeteria</ent><br/
<hw>Caf`e*te"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cafeti<egrave/re</ets>.]</ety> <sn>1.</sn> <def>A restaurant or caf<eacute/ at which the patrons serve themselves with food kept at a counter, typically paying a cashier at the end of the counter and taking the food to tables to eat.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a room within a building functioning in the same manner as a cafeteria{1}.  In certain organizations, as schools, the food may be provided gratis.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cafetorium</ent><br/
<hw>Caf`e*to"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[from <ets>cafe</ets>teria and audi<ets>torium</ets>.]</ety> <def>a large room within a building, especially a school, which functions as both a cafeteria and an auditorium.</def><br/
[<source>PJC</source>]</p>

<p><ent>caff</ent><br/
<hw>caff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[from <er>cafe</er>.]</ety> <def>a cafe.</def> <mark>[British slang]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Caffeic</ent><br/
<hw>Caf*fe"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Coffee</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or obtained from, coffee.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caffeic acid</b></col>, <cd>an acid obtained from coffee tannin, as a yellow crystalline substance, <chform>C9H8O4</chform>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caffeine</ent><br/
<ent>caffein</ent><br/
<mhw><hw>caf*fe"in</hw>, <hw>Caf*fe"ine</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>caf<eacute/ine</ets>. See <er>Coffee</er>.]</ety> <fld>(Chem.)</fld> <def>A white, bitter, crystallizable substance, found in coffee and tea. It is identical with the <isa>alkaloid</isa> <altname>theine</altname> from tea leaves, and with <altname>guaranine</altname> from guarana.  It is responsible for most of the stimulating effect of coffee or tea.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Caffetannic</ent><br/
<hw>Caf`fe*tan"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Caffe</ets>ic + <ets>tannic</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or derived from, the tannin of coffee.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caffetannic acid</b></col>, <cd>a variety of tannin obtained from coffee berries, regarded as a glucoside.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caffila</ent><br/
||<hw>Caf"fi*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar.]</ety> <def>See <er>Cafila</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caffre</ent><br/
<hw>Caf"fre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Kaffir</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cafileh</ent><br/
<ent>Cafila</ent><br/
<mhw>{ ||<hw>Ca"fi*la</hw> <pr>(?)</pr>, ||<hw>Ca"fi*leh</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Ar.]</ety> <def>A caravan of travelers; a military supply train or government caravan; a string of pack horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caftan</ent><br/
<hw>Caf"tan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk. <ets>qaft<amac/n</ets>: cf. F. <ets>cafetan</ets>.]</ety> <def>A garment worn throughout the Levant, consisting of a long gown with sleeves reaching below the hands. It is generally fastened by a belt or sash.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caftan</ent><br/
<hw>Caf"tan</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To clothe with a caftan.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The turbaned and <qex>caftaned</qex> damsel.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cag</ent><br/
<hw>Cag</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Keg</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cage</ent><br/
<hw>Cage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cage</ets>, fr. L. <ets>cavea</ets> cavity, cage, fr. <ets>cavus</ets> hollow. Cf. <er>Cave</er>, <pos>n.</pos>, <er>Cajole</er>, <er>Gabion</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A box or inclosure, wholly or partly of openwork, in wood or metal, used for confining birds or other animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In his <qex>cage</qex>, like parrot fine and gay.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 202 --></p>

<p><sn>2.</sn> <def>A place of confinement for malefactors</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Stone walls do not a prison make,<br/
Nor iron bars a <qex>cage</qex>.</q> <rj><qau>Lovelace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Carp.)</fld> <def>An outer framework of timber, inclosing something within it; <as>as, the <ex>cage</ex> of a staircase</as>.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A skeleton frame to limit the motion of a loose piece, as a ball valve.</def> <sd>(b)</sd> <def>A wirework strainer, used in connection with pumps and pipes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The box, bucket, or inclosed platform of a lift or elevator; a cagelike structure moving in a shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mining)</fld> <def>The drum on which the rope is wound in a hoisting whim.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Baseball)</fld> <def>The catcher's wire mask.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cage</ent><br/
<hw>Cage</hw> <pr>(k<amac/j)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caged</conjf> <pr>(k<amac/jd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caging</conjf>.]</vmorph> <def>To confine in, or as in, a cage; to shut up or confine.</def> <ldquo/<xex>Caged</xex> and starved to death.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caged</ent><br/
<hw>Caged</hw> <pr>(k<amac/jd)</pr>, <pos>a.</pos> <def>Confined in, or as in, a cage; like a <xex>cage</xex> or prison.</def> <ldquo/The <xex>caged</xex> cloister.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cageling</ent><br/
<hw>Cage"ling</hw> <pr>(k<amac/j"l<icr/ng)</pr>, <pos>n.</pos> <ety>[<ets>Cage</ets> + <ets>-ling</ets>]</ety> <def>A bird confined in a cage; esp. a young bird.</def> <mark>[Poetic]</mark>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cagit</ent><br/
||<hw>Ca"git</hw> <pr>(k<amac/"j<icr/t)</pr>, <pos>n.</pos> <fld>(Zool)</fld> <def>A kind of parrot, of a beautiful green color, found in the Philippine Islands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cagmag</ent><br/
<hw>Cag"mag</hw> <pr>(k<acr/g"m<acr/g)</pr>, <pos>n.</pos> <def>A tough old goose; hence, coarse, bad food of any kind.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cagot</ent><br/
||<hw>Ca`got"</hw> <pr>(k<adot/`g<osl/")</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>One of a race inhabiting the valleys of the Pyrenees, who until 1793 were political and social outcasts (Christian Pariahs). They are supposed to be a remnant of the Visigoths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cahenslyism</ent><br/
<hw>Ca*hens"ly*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(R. C. Ch.)</fld> <def>A plan proposed to the Pope in 1891 by P. P. Cahensly, a member of the German parliament, to divide the foreign-born population of the United States, for ecclesiastical purposes, according to European nationalities, and to appoint bishops and priests of like race and speaking the same language as the majority of the members of a diocese or congregation. This plan was successfully opposed by the American party in the Church.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cahier</ent><br/
||<hw>Ca`hier"</hw> <pr>(k<adot/`y<asl/" <it>or</it> k<adot/`h<emac/r)</pr>, <pos>n.</pos> <ety>[F., fr. OF. <ets>cayer</ets>, fr. LL. <ets>quaternum</ets>. See <er>Quire</er> of paper. The sheets of manuscript were folded into parts.]</ety> <sn>1.</sn> <def>A number of sheets of paper put loosely together; esp. one of the successive portions of a work printed in numbers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A memorial of a body; a report of legislative proceedings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cahinca root</ent><br/
<hw>Ca*hin"ca root`</hw> <pr>(?)</pr>. <altsp>[Written also <asp>cainca root</asp>.]</altsp> <ety>[See <er>Cahincic</er>.]</ety> <fld>(Bot.)</fld> <def>The root of an American shrub (<spn>Chiococca racemosa</spn>), found as far north as Florida Keys, from which cahincic acid is obtained; also, the root of the South American <spn>Chiococca anguifuga</spn>, a celebrated antidote for snake poison.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cahincic</ent><br/
<hw>Ca*hin"cic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or derived from, <etsep>cahinca</etsep>, the native name of a species of Brazilian <gen>Chiococca</gen>, perhaps <spn>Chiococca racemosa</spn>; <as>as, <ex>cahincic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cahoot</ent><br/
<hw>Ca*hoot"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps fr. f. <ets>cohorte</ets> a company or band.]</ety> <def>Partnership; league; <as>as, to go in <ex>cahoot</ex> (or in <ex>cahoots</ex>) with a person</as>.  Usually used in the plural, and in modern usage often used to imply that the joint effort is unethical, shady, questionable, or illegal; <as>as, a shill in <ex>cahoots</ex> with a pickpocket, to serve as a distraction</as>.</def> <mark>[Slang, southwestern U. S.]</mark> <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Caimacam</ent><br/
||<hw>Cai`ma*cam"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk.]</ety> <def>The governor of a sanjak or district in Turkey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caiman</ent><br/
<hw>Cai"man</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Cayman</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cainozoic</ent><br/
<hw>Cai`no*zo"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>See <er>Cenozic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caique</ent><br/
||<hw>Ca*<ium/que"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Turk. <ets>q<amac/<imac/q</ets> boat.]</ety> <fld>(Naut.)</fld> <def>A light skiff or rowboat used on the Bosporus; also, a Levantine vessel of larger size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>ca ira</ent><br/
||<hw><cced/a" i*ra"</hw> <pr>(?)</pr>. <ety>[F. <ets><cced/a ira, <cced/a ira, les aristocrates <agrave/ la lanterne</ets>, it shall go on, it shall go on, [hang]the arictocrats to the lantern (lamp-post).]</ety> <def>The refrain of a famous song of the French Revolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caird</ent><br/
<hw>Caird</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir. <ets>ceard</ets> a tinker.]</ety> <def>A traveling tinker; also a tramp or sturdy beggar.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cairn</ent><br/
<hw>Cairn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>carn</ets>, gen. <ets>cairn</ets>, a heap: cf. Ir. & W. <ets>carn</ets>.]</ety> <sn>1.</sn> <def>A rounded or conical heap of stones erected by early inhabitants of the British Isles, apparently as a sepulchral monument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now here let us place the gray stone of her <qex>cairn</qex>.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pile of stones heaped up as a landmark, or to arrest attention, as in surveying, or in leaving traces of an exploring party, etc.</def>  <rj><au>C. Kingsley. Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cairned</ent><br/
<hw>cairned</hw> <pos>adj.</pos> <def>piled up, like a cairn.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cairo</ent><br/
<hw>Cairo</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>Egypt</country>.  It is the largest city in Africa.  Population (2000) = 7,010,000.</def><br/
<syn><b>Syn. --</b> El Qahira, Egyptian capital, capital of Egypt.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cairngormstone</ent><br/
<ent>cairngorm</ent><br/
<mhw><hw>cairn"gorm</hw>, <hw>Cairn*gorm"stone`</hw></mhw> <pr>(?)</pr>. <ety>[Gael. <ets>carn</ets> a cairn + <ets>gorm</ets> azure.]</ety> <fld>(Min.)</fld> <def>A smoky yellow or brown variety of rock crystal, or crystallized quartz, found especially, in the mountain of <etsep>Cairngorm</etsep>, in Scotland.</def><br/
<syn><b>Syn. --</b> smoky quartz.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Caisson</ent><br/
<hw>Cais"son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>caisse</ets>, case, chest. See 1st <er>Case</er>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A chest to hold ammunition.</def> <sd>(b)</sd> <def>A four-wheeled carriage for conveying ammunition, consisting of two parts, a body and a limber. In light field batteries there is one caisson to each piece, having two ammunition boxes on the body, and one on the limber.</def> <au>Farrow.</au> <sd>(c)</sd> <def>A chest filled with explosive materials, to be laid in the way of an enemy and exploded on his approach.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>A water-tight box, of timber or iron within which work is carried on in building foundations or structures below the water level.</def> <sd>(b)</sd> <def>A hollow floating box, usually of iron, which serves to close the entrances of docks and basins.</def> <sd>(c)</sd> <def>A structure, usually with an air chamber, placed beneath a vessel to lift or float it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A sunk panel of ceilings or soffits.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pneumatic caisson</b></col> <fld>(Engin.)</fld>, <cd>a caisson, closed at the top but open at the bottom, and resting upon the ground under water. The pressure of air forced into the caisson keeps the water out. Men and materials are admitted to the interior through an air lock. See <er>Lock</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caisson disease</ent><br/
<hw>Cais"son dis*ease"</hw>. <fld>(Med.)</fld> <def>A disease frequently induced by remaining for some time in an atmosphere of high pressure, as in caissons, diving bells, etc. It is characterized by neuralgic pains and paralytic symptoms. It is caused by the release of bubbles of gas, usually nitrogen, from bodily fluids into the blood and tissues, when a person, having been in an environment with high air pressure, moves to a lower pressure environment too rapidly for the excess dissolved gases to be released through normal breathing.  It may be fatal, but can be reversed or alleviated by returning the affected person to a high air pressure, and then gradually decreasing the pressure to allow the gases to be released from the body fluids.  It is a danger well known to divers.  It is also called <altname>the bends</altname> and <altname>decompression sickness</altname>.  It can be prevented in divers by a slow return to normal pressure, or by using a breathing mixture of oxygen combined with a gas having low solubility in water, such as helium.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Caitiff</ent><br/
<hw>Cai"tiff</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>caitif</ets>, <ets>cheitif</ets>, captive, miserable, OF. <ets>caitif</ets>, <ets>chaitif</ets>, captive, mean, wretched, F.  <ets>ch<eacute/tif</ets>, fr. L. <ets>captivus</ets> captive, fr. <ets>capere</ets> to take, akin to E. <ets>heave</ets>. See <er>Heave</er>, and cf. <er>Captive</er>.]</ety> <sn>1.</sn> <def>Captive; wretched; unfortunate.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Base; wicked and mean; cowardly; despicable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Arnold had sped his <qex>caitiff</qex> flight.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caitiff</ent><br/
<hw>Cai"tiff</hw>, <pos>n.</pos> <def>A captive; a prisoner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Avarice doth tyrannize over her <qex>caitiff</qex> and slave.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wretched or unfortunate man.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mean, despicable person; one whose character meanness and wickedness meet.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>The deep-felt conviction of men that slavery breaks down the moral character . . . speaks out with . . . distinctness in the change of meaning which <xex>caitiff</xex> has undergone signifying as it now does, one of a base, abject disposition, while there was a time when it had nothing of this in it.</note>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajanus</ent><br/
<hw>Cajanus</hw> <pos>n.</pos> <def>a genus of erect densely branched shrubby perennials of Old World tropics; naturalized in other warm regions.</def><br/
<syn><b>Syn. --</b> genus <gen>Cajanus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cajeput</ent><br/
<hw>Caj"e*put</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cajuput</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajole</ent><br/
<hw>Ca*jole"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cajoled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cajoling</conjf>.]</vmorph> <ety>[F. <ets>cajoler</ets>, orig., to chatter like a bird in a cage, to sing; hence, to amuse with idle talk, to flatter, from the source of OF.  <ets>goale</ets>, <ets>jaiole</ets>, F. <ets>ge<ocir/le</ets>, dim. of <ets>cage</ets> a cage. See <er>Cage</er>, <er>Jail</er>.]</ety> <def>To deceive with flattery or fair words; to wheedle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am not about to <qex>cajole</qex> or flatter you into a reception of my views.</q> <rj><qau>F. W. Robertson.</qau></rj></p>

<p><syn><b>Syn.</b> -- To flatter; wheedle; delude; coax; entrap.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajolement</ent><br/
<hw>Ca*jole"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of cajoling; the state of being cajoled; cajolery.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajoler</ent><br/
<hw>Ca*jol"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A flatterer; a wheedler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajolery</ent><br/
<hw>Ca*jol"er*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cajoleries</plw> <pr>(<?/)</pr>.</plu> <def>A wheedling to delude; words used in cajoling; flattery.</def> <ldquo/Infamous <xex>cajoleries</xex>.<rdquo/  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajun</ent><br/
<hw>Ca"jun</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of <er>Acadian</er>.]</ety> <fld>(Ethnol.)</fld> <def>In Louisiana, a person reputed to be Acadian French descent.  Also used attributively, as in <ex>Cajun</ex> cooking.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Cajuput</ent><br/
<hw>Caj"u*put</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of Malayan origin; <ets>k<amac/yu</ets> tree + <ets>p<umac/tih</ets> white.]</ety> <fld>(Med.)</fld> <def>A highly stimulating volatile inflammable oil, distilled from the leaves of an East Indian tree (<spn>Melaleuca cajuputi</spn>, etc.) It is greenish in color and has a camphoraceous odor and pungent taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cajuputene</ent><br/
<hw>Caj"u*put*ene`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A colorless or greenish oil extracted from cajuput.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cake</ent><br/
<hw>Cake</hw> <pr>(k<amac/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>cake</ets>, <ets>kaak</ets>; akin to Dan. <ets>kage</ets>, Sw. & Icel. <ets>kaka</ets>, D. <ets>koek</ets>, G.<ets>kuchen</ets>, OHG. <ets>chuocho</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A small mass of dough baked; especially, a thin loaf from unleavened dough; <as>as, an oatmeal <ex>cake</ex>; johnny<ex>cake</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sweetened composition of flour and other ingredients, leavened or unleavened, baked in a loaf or mass of any size or shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A thin wafer-shaped mass of fried batter; a griddlecake or pancake; as buckwheat <xex>cakes</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A mass of matter concreted, congealed, or molded into a solid mass of any form, esp. into a form rather flat than high; <as>as, a <ex>cake</ex> of soap; an ague <ex>cake</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cakes</qex> of rusting ice come rolling down the flood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cake urchin</b></col> <fld>(Zool)</fld>, <cd>any species of flat sea urchins belonging to the <gen>Clypeastroidea</gen></cd>. -- <col><b>Oil cake</b></col> <cd>the refuse of flax seed, cotton seed, or other vegetable substance from which oil has been expressed, compacted into a solid mass, and used as food for cattle, for manure, or for other purposes.</cd> -- <col><b>To have one's cake dough</b></col>, <cd>to fail or be disappointed in what one has undertaken or expected.</cd>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cake</ent><br/
<hw>Cake</hw>, <pos>v. i.</pos> <def>To form into a cake, or mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cake</ent><br/
<hw>Cake</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caking</conjf>.]</vmorph> <def>To concrete or consolidate into a hard mass, as dough in an oven; to coagulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Clotted blood that <qex>caked</qex> within.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cake</ent><br/
<hw>Cake</hw>, <pos>v. i.</pos> <def>To cackle as a goose.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caking coal</ent><br/
<hw>Cak"ing coal`</hw> <pr>(?)</pr>. <def>See <er>Coal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cal</ent><br/
<hw>Cal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Cornish Mines)</fld> <def>Wolfram, an ore of tungsten.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>calaba</ent><br/
<hw>calaba</hw> <pos>n.</pos> <def>a West Indian tree (<spn>Calophyllum calaba</spn>) having racemes of fragrant white flowers and yielding a durable timber and resinous juice.</def><br/
<syn><b>Syn. --</b> Santa Maria tree.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calabar</ent><br/
<hw>Cal"a*bar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A district on the west coast of Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calabar bean</b></col>, <cd>The of a climbing legumious plant (<spn>Physostigma venenosum</spn>), a native of tropical Africa. It is highly poisonous. It is used to produce contraction of the pupil of the eye; also in tetanus, neuralgia, and rheumatic diseases; -- called also <altname>ordeal bean</altname>, being used by the negroes in trials for witchcraft.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calabarine</ent><br/
<hw>Cal"a*bar*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An alkaloid resembling physostigmine and occurring with it in the calabar bean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calabash</ent><br/
<hw>Cal"a*bash</hw> <pr>(k<acr/l"<adot/*b<acr/sh)</pr>, <pos>n.</pos> <ety>[Sp. <ets>calabaza</ets>, or Pg. <ets>calaba<cced/a</ets>, <ets>caba<cced/a</ets> (cf. F. <ets>Calebasse</ets>), lit., a dry gourd, fr. Ar. <ets>qar'</ets>, fem., a kind of gourd + <ets>aibas</ets> dry.]</ety> <sn>1.</sn> <def>The common gourd (plant or fruit).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fruit of the calabash tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A water dipper, bottle, bascket, or other utensil, made from the dry shell of a calabash or gourd.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calabash tree</b></col>. <fld>(Bot.)</fld>, <cd>a tree of tropical America (<spn>Crescentia cujete</spn>), producing a large gourdlike fruit, containing a purgative pulp. Its hard shell, after the removal of the pulp, is used for cups, bottles, etc.  The <stype>African calabash tree</stype> is the baobab.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calaboose</ent><br/
<hw>Cal`a*boose"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of Sp. <ets>calabozo</ets> dungeon.]</ety> <def>A prison; a jail.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calabozo</ent><br/
||<hw>Ca`la*bo"zo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>A jail. See <er>Calaboose</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>calabura</ent><br/
<hw>calabura</hw> <pos>n.</pos> <def>a fast-growing tropical American evergreen (<spn>Muntingia calabura</spn>) having white flowers and white fleshy edible fruit; bark yields a silky fiber used in cordage and wood is valuable for staves.</def><br/
<syn><b>Syn. --</b> Jamaican cherry, calabur tree, silkwood.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calade</ent><br/
||<hw>Ca*lade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A slope or declivity in a manege ground down which a horse is made to gallop, to give suppleness to his haunches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caladenia</ent><br/
<hw>caladenia</hw> <pos>n.</pos> <def>any of various orchids of the genus <gen>Caladenia</gen>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caladium</ent><br/
||<hw>Ca*la"di*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A genus of aroideous plants, of which some species are cultivated for their immense leaves (which are often curiously blotched with white and red), and others (in Polynesia) for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calaite</ent><br/
<hw>Cal"a*ite</hw> <pr>(k<acr/l`<asl/*<imac/t)</pr>, <pos>n.</pos> <ety>[L. <ets>calla<ium/s</ets>, Gr. <grk>ka`lai:s</grk>, <grk>ka`llai:s</grk>; cf. F. <ets>cala<ium/te</ets>.]</ety> <def>A mineral. See <er>Turquoise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamanco</ent><br/
<hw>Cal`a*man"co</hw> <pr>(k<acr/l`<adot/*m<acr/<nsm/"k<osl/)</pr>, <pos>n.</pos> <ety>[LL. <ets>calamancus</ets>, <ets>calamacus</ets>; cf. <ets>camelaucum</ets>; a head covering made of camel's hair, NGr. <grk>kamelay`kion</grk>, and F. <ets>calmande</ets> a woolen stuff.]</ety> <def>A glossy woolen stuff, plain, striped, or checked.</def> <ldquo/A gay <xex>calamanco</xex> waistcoat.<rdquo/  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamander wood</ent><br/
<hw>Cal"a*man`der wood</hw> <pr>(k<acr/l"<adot/*m<acr/n`d<etil/r w<oocr/d`)</pr>. <def>A valuable furniture wood from India and Ceylon, of a hazel-brown color, with black stripes, very hard in texture. It is a species of ebony, and is obtained from the <spn>Diospyros qu<ae/sita</spn>. Called also <altname>Coromandel wood</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>calamari</ent><br/
<hw>Cal"a*mar`i</hw> <pr>(k<acr/l`<adot/*m<aum/r"<esl/)</pr>, <pos>n.</pos> <ety>[It. <ets>calamari</ets>, fr. LL. <ets>calamarium</ets> inkstand, fr. L. <ets>calamus</ets> a reed pen: cf. F. <ets>calmar</ets>, <ets>calemar</ets>, pen case, calamar.]</ety> <fld>(Cookery)</fld> <def>Squid, used as a food; -- from the Italian word. See <er>Squid</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Calambac</ent><br/
<hw>Cal"am*bac</hw> <pr>(k<acr/l"<acr/m*b<acr/k)</pr>, <pos>n.</pos> <ety>[F. <ets>calambac</ets>, <ets>calambour</ets>, from Malay <ets>Kalambaq</ets> a king of fragrant wood.]</ety> <fld>(Bot.)</fld> <def>A fragrant wood; agalloch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calambour</ent><br/
<hw>Cal"am*bour</hw> <pr>(k<acr/l"<acr/m*b<oomac/r)</pr>, <pos>n.</pos> <ety>[See <er>Calambac</er>.]</ety> <def>A species of agalloch, or aloes wood, of a dusky or mottled color, of a light, friable texture, and less fragrant than calambac; -- used by cabinetmakers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamiferous</ent><br/
<hw>Cal`a*mif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calamus</ets> reed + <ets>ferous</ets>.]</ety> <def>Producing reeds; reedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamine</ent><br/
<hw>Cal"a*mine</hw> <pr>(k<acr/l"<adot/*m<imac/n <it>or</it> -m<icr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>calamine</ets>, LL. <ets>calamina</ets>, fr. L. <ets>Cadmia</ets>. See <er>Cadmia</er>.]</ety> <fld>(min.)</fld> <def>A mineral, the hydrous silicate of zinc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name was formerly applied to both the carbonate and silicate of zinc each of which is valuabic as an ore; but it is now usually restricted to the latter, the former being called <xex>smithsonite</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamint</ent><br/
<hw>Cal"a*mint</hw> <pr>(-m<icr/nt)</pr>, <pos>n.</pos> <ety>[OE. <ets>calamint</ets>, <ets>calemente</ets> (cf. F. <ets>calament</ets>) fr. L. <ets>calamintha</ets>, Gr. <grk>kalami`nqh</grk>, <grk>kala`minqos</grk>. See 1st <er>Mint</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of perennial plants (<gen>Calamintha</gen>) of the Mint family, esp. the <spn>Calamintha Nepeta</spn> and <spn>Calamintha Acinos</spn>, which are called also <stype>basil thyme</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamist</ent><br/
<hw>Cal"a*mist</hw> <pr>(-m<icr/st)</pr>, <pos>n.</pos> <ety>[L. <ets>calamus</ets> a reed.]</ety> <def>One who plays upon a reed or pipe.</def> <mark>[Obs.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamistrate</ent><br/
<hw>Cal`a*mis"trate</hw> <pr>(-m<icr/s"tr<amac/t)</pr>, <pos>v. i.</pos> <ety>[L. <ets>calamistratus</ets>, curled with the curling iron, fr. <ets>calamistrum</ets> curling iron, fr. <ets>calamus</ets> a reed.]</ety> <def>To curl or friz, as the hair.</def> <mark>[Obs.]</mark>  <rj><au>Cotgrave.</au></rj></p>

<p><ent>Calamistration</ent><br/
<hw>Cal`a*mis*tra"tion</hw> <pr>(k<acr/l`<adot/*m<icr/s*tr<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <def>The act or process of curling the hair.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamistrum</ent><br/
||<hw>Cal`a*mis"trum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a curling iron.]</ety> <fld>(Zool.)</fld> <def>A comblike structure on the metatarsus of the hind legs of certain spiders (<fam>Ciniflonid<ae/</fam>), used to curl certain fibers in the construction of their webs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamite</ent><br/
<hw>Cal"a*mite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calamus</ets> a reed: cf. F. <ets>calamite</ets>.]</ety> <fld>(Paleon.)</fld> <def>A fossil plant of the coal formation, having the general form of plants of the modern <gen>Equiseta</gen> (the Horsetail or Scouring Rush family) but sometimes attaining the height of trees, and having the stem more or less woody within. See <er>Acrogen</er>, and <er>Asterophyllite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamitous</ent><br/
<hw>Ca*lam"i*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Calamitosus</ets>; cf. F. <ets>calamiteux</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Suffering calamity; wretched; miserable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ten thousands of <qex>calamitous</qex> persons.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Producing, or attended with distress and misery; making wretched; wretched; unhappy.</def> <ldquo/This sad and <xex>calamitous</xex> condition.<rdquo/ <au>South.</au> <ldquo/A <xex>calamitous</xex> prison<rdquo/ <au>Milton.</au></p>

<p><syn><b>Syn.</b> -- Miserable; deplorable; distressful; afflictive; wretched; grievous; baleful; disastrous; adverse; unhappy; severe; sad; unfortunate.</syn></p>

<p>-- <wordforms><wf>Ca*lam"i*tous*ly</wf>, <pos>adv.</pos> -- <wf>Ca*lam"i*tous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamity</ent><br/
<hw>Ca*lam"i*ty</hw> <pr>(?)</pr> <pos>n.</pos>; <plu><it>pl.</it> <plw>Calamities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>calamitas</ets>, akin to in-<ets>columis</ets> unharmed: cf. F. <ets>calamit<eacute/</ets>]</ety> <sn>1.</sn> <def>Any great misfortune or cause of misery; -- generally applied to events or disasters which produce extensive evil, either to communities or individuals.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>The word <xex>calamity</xex> was first derived from <xex>calamus</xex> when the corn could not get out of the stalk.</note>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Strokes of <qex>calamity</qex> that scathe and scorch the soul.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state or time of distress or misfortune; misery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The deliberations of <qex>calamity</qex> are rarely wise.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where'er I came I brought <qex>calamity</qex>.</q> <rj><qau>Tennyson.</qau></rj></p>

<p><syn><b>Syn.</b> -- Disaster; distress; affliction; adversity; misfortune; unhappiness; infelicity; mishap; mischance; misery; evil; extremity; exigency; downfall.</syn> <usage> -- <er>Calamity</er>, <er>Disaster</er>, <er>Misfortune</er>, <er>Mishap</er>, <er>Mischance</er>. Of these words, <xex>calamity</xex> is the strongest. It supposes a somewhat continuous state, produced not usually by the direct agency of man, but by natural causes, such as fire, flood, tempest, disease, etc, <xex>Disaster</xex> denotes literally <xex>ill-starred</xex>, and is some unforeseen and distressing event which comes suddenly upon us, as if from hostile planet. <xex>Misfortune</xex> is often due to no specific cause; it is simply the bad fortune of an individual; a link in the chain of events; an evil independent of his own conduct, and not to be charged as a fault. <xex>Mischance</xex> and <xex>mishap</xex> are misfortunes of a trivial nature, occurring usually to individuals. <ldquo/A <xex>calamity</xex> is either public or private, but more frequently the former; a <xex>disaster</xex> is rather particular than private; it affects things rather than persons; journey, expedition, and military movements are often attended with <xex>disasters</xex>; <xex>misfortunes</xex> are usually personal; they immediately affect the interests of the individual.<rdquo/</usage>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calamus</ent><br/
<hw>Cal"a*mus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Calami</plw> <pr>(#)</pr>.</plu> <ety>[L., a reed. See <er>Halm</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The indian cane, a plant of the Palm family. It furnishes the common rattan. See <er>Rattan</er>, and <er>Dragon's blood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A species of <gen>Acorus</gen> (<spn>Acorus calamus</spn>), commonly called <altname>calamus</altname>, or <altname>sweet flag</altname>. The root has a pungent, aromatic taste, and is used in medicine as a stomachic; the leaves have an aromatic odor, and were formerly used instead of rushes to strew on floors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The horny basal portion of a feather; the barrel or quill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calando</ent><br/
||<hw>Ca*lan"do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Gradually diminishing in rapidity and loudness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calandrinia</ent><br/
<hw>Calandrinia</hw> <pos>n.</pos> <def>a large genus of low-growing herbs; widespread throughout tropical and warm temperate regions having usually basal leaves and panicles of purplish ephemeral flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Calandrinia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>calanthe</ent><br/
<hw>calanthe</hw> <pos>n.</pos> <def>any of various showy orchids of the genus <gen>Calanthe</gen> having white or yellow or rose-colored flowers and broad leaves folded lengthwise.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calash</ent><br/
<hw>Ca*lash"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cal<egrave/che</ets>; of Slavonic origin; cf. Bohem. <ets>kolesa</ets>, Russ. <ets>koliaska</ets> calash, <ets>koleso</ets>, <ets>kolo</ets>, wheel.]</ety> <sn>1.</sn> <def>A light carriage with low wheels, having a top or hood that can be raised or lowered, seats for inside, a separate seat for the driver, and often a movable front, so that it can be used as either an open or a closed carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The baroness in a <qex>calash</qex> capable of holding herself, her two children, and her servants.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In Canada, a two-wheeled, one-seated vehicle, with a calash top, and the driver's seat elevated in front.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A hood or top of a carriage which can be thrown back at pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A hood, formerly worn by ladies, which could be drawn forward or thrown back like the top of a carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 203 --></p>

<p><ent>Calaveras skull</ent><br/
<hw>Ca`la*ve"ras skull</hw> <pr>(?)</pr>. <def>A human skull reported, by <person>Prof. J. D. Whitney</person>, as found in 1886 in a Tertiary auriferous gravel deposit, lying below a bed of black lava, in <etsep>Calaveras</etsep> County, California. It is regarded as very doubtful whether the skull really belonged to the deposit in which it was found. If it did, it indicates an unprecedented antiquity for human beings of an advanced type.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Calaverite</ent><br/
<hw>Ca`la*ve"rite</hw> <pr>(<?/)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A bronze-yellow massive mineral with metallic luster; a telluride of gold; -- first found in <etsep>Calaveras</etsep> County California.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcaneal</ent><br/
<hw>Cal*ca"ne*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anal.)</fld> <def>Pertaining to the calcaneum; <as>as, <ex>calcaneal</ex> arteries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcaneum</ent><br/
||<hw>Cal*ca"ne*um</hw> <pr>(?)</pr> <pos>n.</pos>; <plu><it>pl.</it> E. <plw>-neums</plw>, L. <plw>-nea</plw>.</plu> <ety>[L. the heel, fr. <ets>calx</ets>, <ets>calcis</ets>, the heel.]</ety> <fld>(Anal.)</fld> <def>One of the bones of the tarsus which in man, forms the great bone of the heel; -- called also <altname>fibulare</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcar</ent><br/
<hw>Cal"car</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calcaria</ets> lime kiln, fr. <ets>calx</ets>, <ets>calcis</ets>, lime. See <er>Calx</er>.]</ety> <fld>(Glass manuf.)</fld> <def>A kind of oven, or reverberatory furnace, used for the calcination of sand and potash, and converting them into frit.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcar</ent><br/
||<hw>Cal"car</hw>, <pos>n.</pos>; L. <plu><it>pl.</it> <plw>Calcaria</plw> <pr>(#)</pr>.</plu> <ety>[L., a spur, as worn on the heel, also the spur of a cock, fr. <ets>calx</ets>, <ets>calcis</ets>, the heel.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A hollow tube or spur at the base of a petal or corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A slender bony process from the ankle joint of bats, which helps to support the posterior part of the web, in flight.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>A spur, or spurlike prominence.</def> <sd>(b)</sd> <def>A curved ridge in the floor of the leteral ventricle of the brain; the <xex>calcar avis</xex>, <xex>hippocampus minor</xex>, or <xex>ergot</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcarated</ent><br/
<ent>Calcarate</ent><br/
<mhw>{ <hw>Cal"ca*rate</hw> <pr>(?)</pr>, <hw>Cal"ca*ra`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[LL. <ets>calcaratus</ets>, fr. L. <ets>calcar</ets>. See 2d <ets>Calcar</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Bot.)</fld> <def>Having a spur, as the flower of the toadflax and larkspur; spurred.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Armed with a spur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcareo-argillaceous</ent><br/
<hw>Cal*ca"re*o-ar`gil*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>consisting of, or containing, calcareous and argillaceous earths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcareo-bituminous</ent><br/
<hw>Cal*ca"re*o-bi*tu"mi*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of, or containing, lime and bitumen.</def>  <rj><au>Lyell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcareo-siliceous</ent><br/
<hw>Cal*ca"re*o-si*li"ceous</hw> <pr>(?)</pr>, <pos>a.</pos><def>Consisting of, or containing calcareous and siliceous earths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcareous</ent><br/
<hw>Cal*ca"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calcarius</ets> pertaining to lime. See <er>Calx</er>.]</ety> <def>Partaking of the nature of calcite or calcium carbonate; consisting of, or containing, calcium carbonate or carbonate of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calcareous spar</b></col>. <cd>See as <er>Calcite</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcareousness</ent><br/
<hw>Cal*ca"re*ous*ness</hw>, <pos>n.</pos> <def>Quality of being calcareous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcariferous</ent><br/
<hw>Cal`ca*rif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calcarius</ets> of lime + <ets>ferous</ets>.]</ety> <def>Lime-yielding; calciferous</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcarine</ent><br/
<hw>Cal"ca*rine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to, or situated near, the calcar of the brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcavella</ent><br/
<hw>Cal`ca*vel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sweet wine from Portugal; -- so called from the district of Carcavelhos.</def> <altsp>[Written also <asp>Calcavellos</asp> or <asp>Carcavelhos</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calceated</ent><br/
<hw>Cal"ce*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calceatus</ets>, p. p. of <ets>pelceare</ets> to ahoe, fr. <ets>catceus</ets> shoe, fr. <ets>calx</ets>, <ets>calcic</ets>, <ets>heel</ets>.]</ety> <def>Fitted with, or wearing, shoes.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calced</ent><br/
<hw>Calced</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Calceated</er>.]</ety> <def>Wearing shoes; calceated; -- in distintion from <xex>discalced</xex> or <xex>barefooted</xex>; as the <xex>calced</xex> Carmelites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcedon</ent><br/
<hw>Cal"ce*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chalcedony</er>.]</ety> <def>A foul vein, like chalcedony, in some precious stones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcedonian</ent><br/
<ent>Calcedonic</ent><br/
<mhw>{ <hw>Cal`ce*don"ic</hw> <pr>(?)</pr>, <hw>Cal`ce*do"ni*an</hw>, }</mhw> <pos>a.</pos> <def>See <er>Chalcedonic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calceiform</ent><br/
<hw>Cal"ce*i*form`</hw> <pr>(k<acr/l"s<esl/*<icr/*f<ocir/rm`)</pr>, <pos>a.</pos> <ety>[L. <ets>calceus</ets> shoe + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Shaped like a slipper, as one petal of the lady's-slipper; calceolate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>calceolaria</ent><br/
||<hw>cal`ce*o*la"ri*a</hw> <pr>(k<acr/l`s<esl/*<osl/*l<amac/"r<icr/*<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>calceolarius</ets> shoemaker, fr. <ets>calceolus</ets>, a dim. of <ets>calceus</ets> shoe.]</ety> <fld>(Bot.)</fld> <def>A genus of showy herbaceous or shrubby plants, brought from South America; slipperwort. It has a yellow or purple flower, often spotted or striped, the shape of which suggests its name.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calceolate</ent><br/
<hw>Cal"ce*o*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Calceolaria</er>.]</ety> <def>Slipper-ahaped. See <er>Calceiform</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calces</ent><br/
||<hw>Cal"ces</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Calx</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcic</ent><br/
<hw>Cal"cic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime: cf. F. <ets>calcique</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, derived from, or containing, calcium or lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>calciferol</ent><br/
<hw>cal*cif"er*ol</hw> <pos>n.</pos> <def>one of the forms of <er>vitamin D</er>, a fat-soluble vitamin that prevents rickets.  It is an unsaturated alcohol, <chform>C28H43OH</chform> found in foods such as milk and fish liver oils.  It is used as a dietary supplement in milk.</def><br/
<syn><b>Syn. --</b> vitamin D, ergocalciferol, cholecarciferol, D.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Calciferous</ent><br/
<hw>Cal*cif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime + <ets>-ferous</ets>.]</ety> <def>Bearing, producing, or containing calcite, or carbonate of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calciferous epoch</b></col> <fld>(Geol.)</fld>, <cd>an epoch in the American lower Silurian system, immediately succeeding the Cambrian period. The name alludes to the peculiar mixture of calcareous and siliceous characteristics in many of the beds. See the Diagram under <er>Geology</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcific</ent><br/
<hw>Cal*cif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Calciferous. Specifically: <fld>(Zool.)</fld> of or pertaining to the portion of the oviduct which forms the eggshell in birds and reptiles.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcification</ent><br/
<hw>Cal`ci*fi*ca"tion</hw> <pr>(k<acr/l`s<icr/*f<icr/*k<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The process of change into a stony or calcareous substance by the deposition of lime salt; -- normally, as in the formation of bone and of teeth; abnormally, as in calcareous degeneration of tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcified</ent><br/
<hw>Cal"ci*fied</hw> <pr>(k<acr/l"s<icr/*f<imac/d)</pr>, <pos>a.</pos> <def>Consisting of, or containing, calcareous matter or lime salts; calcareous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calciform</ent><br/
<hw>Cal"ci*form</hw> <pr>(k<acr/l"s<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime + <ets>-form</ets>.]</ety> <def>In the form of chalk or lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcify</ent><br/
<hw>Cal"ci*fy</hw> <pr>(k<acr/l"s<icr/*f<imac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calcified</conjf> <pr>(-f<imac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calcifying</conjf>.]</vmorph> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime + <ets>-fy</ets>.]</ety> <def>To make stony or calcareous by the deposit or secretion of salts of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcify</ent><br/
<hw>Cal"ci*fy</hw>, <pos>v. i.</pos> <def>To become changed into a stony or calcareous condition, in which lime is a principal ingredient, as in the formation of teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcigenous</ent><br/
<hw>Cal*cig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime + <ets>-genouse</ets>.]</ety> <fld>(Chem.)</fld> <def>Tending to form, or to become, a calx or earthlike substance on being oxidized or burnt; as magnesium, calcium. etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcigerous</ent><br/
<hw>Cal*cig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, <ets>lime</ets> + <ets>-gerouse</ets>.]</ety> <def>Holding lime or other earthy salts; <as>as, the <ex>calcigerous</ex> cells of the teeth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcimine</ent><br/
<hw>Cal"ci*mine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime.]</ety> <def>A white or colored wash for the ceiling or other plastering of a room, consisting of a mixture of clear glue, Paris white or zinc white, and water.</def> <altsp>[Also spelt <asp>kalsomine</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcimine</ent><br/
<hw>Cal"ci*mine</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calcimined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calcimining</conjf>.]</vmorph> <def>To wash or cover with calcimine; <as>as, to <ex>calcimine</ex> walls</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calciminer</ent><br/
<hw>Cal"ci*mi`ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who calcimines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcinable</ent><br/
<hw>Cal*cin"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be calcined; <as>as, a <ex>calcinable</ex> fossil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcinate</ent><br/
<hw>Cal"ci*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To calcine.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcination</ent><br/
<hw>Cal`ci*na"tion</hw> <pr>(k<acr/l`s<icr/*n<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>calcination</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Chem.)</fld> <def>The act or process of disintegrating a substance, or rendering it friable by the action of heat, esp. by the expulsion of some volatile matter, as when carbonic and acid is expelled from carbonate of calcium in the burning of limestone in order to make lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of reducing a metal to an oxide or metallic calx; oxidation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcinatory</ent><br/
<hw>Cal*cin"a*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel used in calcination.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcine</ent><br/
<hw>Cal*cine"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calciden</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calcining</conjf>.]</vmorph> <ety>[F. <ets>calciner</ets>, fr. L. <ets>calx</ets>, <ets>calcis</ets>, lime. See <er>Calx</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To reduce to a powder, or to a friable state, by the action of heat; to expel volatile matter from by means of heat, as carbonic acid from limestone, and thus (usually) to produce disintegration; as to, <xex>calcine</xex> bones.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To oxidize, as a metal by the action of heat; to reduce to a metallic calx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcine</ent><br/
<hw>Cal*cine"</hw>, <pos>v. i.</pos> <def>To be converted into a powder or friable substance, or into a calx, by the action of heat.</def> <ldquo/<xex>Calcining</xex> without fusion<rdquo/  <rj><au>Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calciner</ent><br/
<hw>Cal*cin"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, calcines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcispongiae</ent><br/
||<hw>Cal`ci*spon"gi*<ae/</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>calx</ets>, <ets>calcis</ets>, lime + <ets>spongia</ets> a sponge.]</ety> <fld>(Zool.)</fld> <def>An order of marine sponges, containing calcareous spicules. See <er>Porifera</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcite</ent><br/
<hw>Cal"cite</hw> <pr>(k<acr/l"s<imac/t)</pr>, <pos>n.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime.]</ety> <fld>(Min.)</fld> <def>Calcium carbonate, or carbonate of lime. It is rhombohedral in its crystallization, and thus distinguished from aragonite.  It includes common limestone, chalk, and marble. Called also <altname>calc-spar</altname> and <altname>calcareous spar</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Argentine</xex> is a pearly lamellar variety; <xex>aphrite</xex> is foliated or chalklike; <xex>dogtooth spar</xex>, a form in acute rhombohedral or scalenohedral crystals; <xex>calc-sinter</xex> and <xex>calc-tufa</xex> are lose or porous varieties formed in caverns or wet grounds from calcareous deposits; <xex>agaric mineral</xex> is a soft, white friable variety of similar origin; <xex>stalaclite</xex> and <xex>stalagmite</xex> are varieties formed from the drillings in caverns. <xex>Iceland spar</xex> is a transparent variety, exhibiting the strong double refraction of the species, and hence is called <xex>doubly refracting spar</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcitrant</ent><br/
<hw>Cal"ci*trant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calcitrans</ets>, p. pr. of <ets>calcitrare</ets> to kick, fr. <ets>calx</ets>, <ets>calcis</ets> , heel.]</ety> <def>Kicking.  Hence: Stubborn; refractory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcitrate</ent><br/
<hw>Cal"ci*trate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[L. <ets>calcitratus</ets>, p. p. of <ets>calcitrare</ets>. See <er>Calcitrant</er>.]</ety> <def>To kick.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcitration</ent><br/
<hw>Cal`ci*tra"tion</hw> <pr>(-tr<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <def>Act of kicking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcium</ent><br/
<hw>Cal"ci*um</hw> <pr>(k<acr/l"s<icr/*<ucr/m)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>calx</ets>, <ets>calcis</ets>, lime; cf F. <ets>calcium</ets>. See <er>Calx</er>.]</ety> <fld>(Chem.)</fld> <def>An elementary substance; a metal which combined with oxygen forms lime. It is of a pale yellow color, tenacious, and malleable. It is a member of the alkaline earth group of elements. Atomic weight 40. Symbol Ca.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Calcium</xex> is widely and abundantly disseminated, as in its compounds <xex>calcium carbonate</xex> or limestone, <xex>calcium sulphate</xex> or gypsum, <xex>calcium fluoride</xex> or fluor spar, <xex>calcium phosphate</xex> or apatite.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calcium light</b></col>, <cd>an intense light produced by the incandescence of a stick or ball of lime in the flame of a combination of oxygen and hydrogen gases, or of oxygen and coal gas; -- called also <altname>Drummond light</altname> and <altname>lime light</altname>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Calcivorous</ent><br/
<hw>Cal*civ"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calx</ets> lime + <ets>vorare</ets> to devour.]</ety> <def>Eroding, or eating into, limestone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcographer</ent><br/
<hw>Cal*cog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who practices calcography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcographical</ent><br/
<ent>Calcographic</ent><br/
<mhw>{ <hw>Cal`co*graph"ic</hw> <pr>(?)</pr>, <hw>Cal`co*graph"ic*al</hw>, }</mhw> <pos>a.</pos> <def>Relating to, or in the style of, calcography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcography</ent><br/
<hw>Cal*cog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calx</ets>, <ets>calcis</ets>, lime, chalk + <ets>-graphy</ets>.]</ety> <def>The art of drawing with chalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calc-sinter</ent><br/
<hw>Calc"-sin`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>kalk</ets> (L. <ets>calx</ets>, <ets>calcis</ets>) lime + E. <ets>sinter</ets>.]</ety> <def>See under <er>Calcite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calc-spar</ent><br/
<hw>Calc"-spar`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>kalk</ets> (L. <ets>calx</ets>) lime E. <ets>spar</ets>.]</ety> <def>Same as <er>Calcite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calc-tufa</ent><br/
<hw>Calc"-tu`fa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>kalk</ets> (l. <ets>calx</ets>) lime + E. <ets>tufa</ets>.]</ety> <def>See under <er>Calcite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculable</ent><br/
<hw>Cal"cu*la*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>calculable</ets>.]</ety> <def>That may be calculated or ascertained by calculation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculary</ent><br/
<hw>Cal"cu*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calculus</ets> a pebble, a calculus; cf <ets>calcularius</ets> pertaining to calculation.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to calculi.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculary</ent><br/
<hw>Cal"cu*la*ry</hw>, <pos>n.</pos> <def>A congeries of little stony knots found in the pulp of the pear and other fruits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculate</ent><br/
<hw>Cal"cu*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calculater</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calculating</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L, <ets>calculatus</ets>, p. p. of <ets>calculate</ets>, fr. <ets>calculus</ets> a pebble, a stone used in reckoning; hence, a reckoning, fr. <ets>calx</ets>, <ets>calcis</ets>, a stone used in gaming, limestone. See <er>Calx</er>.]</ety> <sn>1.</sn> <def>To ascertain or determine by mathematical processes, usually by the ordinary rules of arithmetic; to reckon up; to estimate; to compute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A calencar exacity <qex>calculated</qex> than any othe.</q> <rj><qau>North.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ascertain or predict by mathematical or astrological computations the time, circumstances, or other conditions of; to forecast or compute the character or consequences of; <as>as, to <ex>calculate</ex> or cast one's nativity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A cunning man did <qex>calculate</qex> my birth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To adjust for purpose; to adapt by forethought or calculation; to fit or prepare by the adaptation of means to an end; <as>as, to <ex>calculate</ex> a system of laws for the government and protection of a free people</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Religion] is . . . <qex>calculated</qex> for our benefit.</q> <rj><qau>Abp. Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To plan; to expect; to think.</def> <mark>[Local, U. S.]</mark></p>

<p><syn><b>Syn.</b> -- To compute; reckon; count; estimate; rate.</syn> <usage> -- <er>To Calculate</er>, <er>Compute</er>. <er>Reckon</er>, <er>Count</er>. These words indicate the means by which we arrive at a given result in regard to quantity. We <xex>calculate</xex> with a view to obtain a certain point of knowledge; as, to <xex>calculate</xex> an eclipse. We <xex>compute</xex> by combining given numbers, in order to learn the grand result. We <xex>reckon</xex> and <xex>count</xex> in carrying out the details of a computation. These words are also used in a secondary and figurative sense. <ldquo/<xex>Calculate</xex> is rather a conjection from what is, as to what may be; <xex>computation</xex> is a rational estimate of what has been, from what is; <xex>reckoning</xex> is a conclusive conviction, a pleasing assurance that a thing will happen; <xex>counting</xex> indicates an expectation. We <xex>calculate</xex> on a gain; we <xex>compute</xex> any loss sustained, or the amount of any mischief done; we <xex>reckon</xex> on a promised pleasure; we <xex>count</xex> the hours and minutes until the time of enjoyment arrives<rdquo/</usage>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculate</ent><br/
<hw>Cal"cu*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make a calculation; to forecast consequences; to estimate; to compute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The strong passions, whether good or bad, never <qex>calculate</qex>.</q> <rj><qau>F. W. Robertson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculated</ent><br/
<hw>Cal"cu*la`ted</hw> <pr>(?)</pr>, <pos>p. p. & a.</pos> <sn>1.</sn> <def>Worked out by calculation; <as>as <ex>calculated</ex> tables for computing interest</as>; ascertained or conjectured as a result of calculation; <as>as, the <ex>calculated</ex> place of a planet; the <ex>calculated</ex> velocity of a cannon ball</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Adapted by calculation, contrivance. or forethought to accomplish a purpose; <as>as, to use arts <ex>calculated</ex> to deceive the people; a <ex>calculated</ex> response</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Likely to produce a certain effect, whether intended or not; fitted; adapted; suited.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The only danger that attends multiplicity of publication is, that some of them may be <qex>calculated</qex> to injure rather than benefit society.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The minister, on the other hand, had never gone through an experience <qex>calculated</qex> to lead him beyond the scope of generally received laws</q>. <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculating</ent><br/
<hw>Cal"cu*la`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to mathematical calculations; performing or able to perform mathematical calculations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Given to contrivance or forethought; forecasting; scheming; <as>as, a cool <ex>calculating</ex> disposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calculating machine</b></col>, <cd>a machine for the mechanical performance of mathematical operations, for the most part invented by Charles Babbage and G. and E. Scheutz. It computes logarithmic and other mathematical tables of a high degree of intricacy, imprinting the results on a leaden plate, from which a stereotype plate is then directly made.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculating</ent><br/
<hw>Cal"cu*la`ting</hw>, <pos>n.</pos> <def>The act or process of making mathematical computations or of estimating results.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculation</ent><br/
<hw>Cal`cu*la"tion</hw> <pr>(-l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>calculation</ets>, fr. L. <ets>calculatio</ets>; cf. OF. <ets>calcucation</ets>.]</ety> <sn>1.</sn> <def>The act or process, or the result, of calculating; computation; reckoning, estimate.</def> <ldquo/The <xex>calculation</xex> of eclipses.<rdquo/  <rj><au>Nichol.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mountain is not so his <qex>calculation</qex> makes it.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An expectation based on circumstances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lazy gossips of the port,<br/
Abhorrent of a <qex>calculation</qex> crost,<br/
Began to chafe as at a personal wrong.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculative</ent><br/
<hw>Cal"cu*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to calculation; involving calculation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Long habits of <qex>calculative</qex> dealings.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculator</ent><br/
<hw>Cal"cu*la*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>calculateur</ets>.]</ety> <def>One who computes or reckons: one who estimates or considers the force and effect of causes, with a view to form a correct estimate of the effects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ambition is no exact <qex>calculator</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculatory</ent><br/
<hw>Cal"cu*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calculatorius</ets>.]</ety> <def>Belonging to calculation.</def>  <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcule</ent><br/
<hw>Cal"cule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>calcul</ets>, fr. L. <ets>calculus</ets>. See <er>Calculus</er>.]</ety> <def>Reckoning; computation.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcule</ent><br/
<hw>Cal"cule</hw>, <pos>v. i.</pos> <def>To calculate</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculi</ent><br/
<hw>Cal"cu*li</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Calculus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculous</ent><br/
<hw>Cal"cu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calculosus</ets>.]</ety> <sn>1.</sn> <def>Of the nature of a calculus; like stone; gritty; <as>as, a <ex>calculous</ex> concretion</as>.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Caused, or characterized, by the presence of a calculus or calculi; a, a <xex>calculous</xex> disorder; affected with gravel or stone; <as>as, a <ex>calculous</ex> person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calculus</ent><br/
<hw>Cal"cu*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Calculi</plw> <pr>(#)</pr>.</plu> <ety>[L, <ets>calculus</ets>. See <er>Calculate</er>, and <er>Calcule</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Any solid concretion, formed in any part of the body, but most frequent in the organs that act as reservoirs, and in the passages connected with them; <as>as, biliary <ex>calculi</ex>; urinary <ex>calculi</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>A method of computation; any process of reasoning by the use of symbols; any branch of mathematics that may involve calculation.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Barycentric calculus</b></col>, <cd>a method of treating geometry by defining a point as the center of gravity of certain other points to which co<eum/fficients or weights are ascribed.</cd> -- <col><b>Calculus of functions</b></col>, <cd>that branch of mathematics which treats of the forms of functions that shall satisfy given conditions.</cd> -- <col><b>Calculus of operations</b></col>, <cd>that branch of mathematical logic that treats of all operations that satisfy given conditions.</cd> -- <col><b>Calculus of probabilities</b></col>, <cd>the science that treats of the computation of the probabilities of events, or the application of numbers to chance.</cd> -- <col><b>Calculus of variations</b></col>, <cd>a branch of mathematics in which the laws of dependence which bind the variable quantities together are themselves subject to change.</cd> -- <col><b>Differential calculus</b></col>, <cd>a method of investigating mathematical questions by using the ratio of certain indefinitely small quantities called <xex>differentials</xex>. The problems are primarily of this form: to find how the change in some variable quantity alters at each instant the value of a quantity dependent upon it.</cd> -- <col><b>Exponential calculus</b></col>, <cd>that part of algebra which treats of exponents.</cd> -- <col><b>Imaginary calculus</b></col>, <cd>a method of investigating the relations of real or imaginary quantities by the use of the imaginary symbols and quantities of algebra.</cd> -- <col><b>Integral calculus</b></col>, <cd>a method which in the reverse of the differential, the primary object of which is to learn from the known ratio of the indefinitely small changes of two or more magnitudes, the relation of the magnitudes themselves, or, in other words, from having the differential of an algebraic expression to find the expression itself.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calcuttan</ent><br/
<hw>Calcuttan</hw> <pos>adj.</pos> <def>of or pertaining to Calcutta.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 204 --></p>

<p><ent>Caldron</ent><br/
<hw>Cal"dron</hw> <pr>(k<add/l"dr<ucr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>caldron</ets>, <ets>caudron</ets>, <ets>caudroun</ets>, OF. <ets>caudron</ets>, <ets>chauderon</ets>, F. <ets>chaudron</ets>, an aug. of F. <ets>chaudi<egrave/re</ets>, LL. <ets>caldaria</ets>, fr. L. <ets>caldarius</ets> suitable for warming, fr. <ets>caldus</ets>, <ets>calidus</ets>, warm, fr. <ets>calere</ets> to be warm; cf. Skr. <ets><cced/r<amac/</ets> to boil. Cf. <er>Chaldron</er>, <er>Calaric</er>, <er>Caudle</er>.]</ety> <def>A large kettle or boiler of copper, brass, or iron. <altsp>[Written also <asp>cauldron</asp>.]</altsp></def> <ldquo/<xex>Caldrons</xex> of boiling oil.<rdquo/  <rj><au>Prescott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caleche</ent><br/
||<hw>Ca*l<egrave/che"</hw> <pr>(k<adot/*l<amac/sh")</pr>, <pos>n.</pos> <ety>[F. <ets>cal<egrave/che</ets>.]</ety> <def>See <er>Calash</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caledonia</ent><br/
<hw>Cal`e*do"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The ancient Latin name of Scotland; -- still used in poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caledonian</ent><br/
<hw>Cal`e*do"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Caledonia or Scotland; Scottish; Scotch.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Caledonia or Scotland.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caledonite</ent><br/
<hw>Ca*led"o*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A hydrous sulphate of copper and lead, found in some parts of Caledonia or Scotland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefacient</ent><br/
<hw>Cal`e*fa"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calefaciens</ets> p. pr. of <ets>calefacere</ets> to make warm; <ets>calere</ets> to be warm + <ets>facere</ets> to make.]</ety> <def>Making warm; heating.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefacient</ent><br/
<hw>Cal`e*fa"cient</hw>, <pos>n.</pos> <def>A substance that excites warmth in the parts to which it is applied, as mustard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefaction</ent><br/
<hw>Cal`e*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calefactio</ets>: cf. F. <ets>cal<eacute/faction</ets>.]</ety> <sn>1.</sn> <def>The act of warming or heating; the production of heat in a body by the action of fire, or by communication of heat from other bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being heated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefactive</ent><br/
<hw>Cal`e*fac"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Calefactory</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefactor</ent><br/
<hw>Cal`e*fac"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A heater; one who, or that which, makes hot, as a stove, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefactory</ent><br/
<hw>Cal`e*fac"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calefactorius</ets>.]</ety> <def>Making hot; producing or communicating heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefactory</ent><br/
<hw>Cal`e*fac"to*ry</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Eccl.)</fld> <def>An apartment in a monastery, warmed and used as a sitting room.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hollow sphere of metal, filled with hot water, or a chafing dish, placed on the altar in cold weather for the priest to warm his hands with.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefy</ent><br/
<hw>Cal"e*fy</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calefied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calefying</conjf>.]</vmorph> <ety>[L. <ets>calere</ets> to be warm + <ets>-fy</ets>]</ety> <def>To make warm or hot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calefy</ent><br/
<hw>Cal"e*fy</hw>, <pos>v. i.</pos> <def>To grow hot or warm.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calembour</ent><br/
||<hw>Cal"em*bour`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A pun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendar</ent><br/
<hw>Cal"en*dar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>kalender</ets>, <ets>calender</ets>, fr. L. <ets>kalendarium</ets> an interest or account book (cf. F. <ets>calendrier</ets>, OF. <ets>calendier</ets>) fr. L. <ets>calendue</ets>, <ets>kalendae</ets>, calends. See <er>Calends</er>.]</ety> <sn>1.</sn> <def>An orderly arrangement of the division of time, adapted to the purposes of civil life, as years, months, weeks, and days; also, a register of the year with its divisions; an almanac.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A tabular statement of the dates of feasts, offices, saints' days, etc., esp. of those which are liable to change yearly according to the varying date of Easter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An orderly list or enumeration of persons, things, or events; a schedule; <as>as, a <ex>calendar</ex> of state papers; a <ex>calendar</ex> of bills presented in a legislative assembly; a <ex>calendar</ex> of causes arranged for trial in court; a <ex>calendar</ex> of a college or an academy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>Shepherds of people had need know the <xex>calendars</xex> of tempests of state.</note>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calendar clock</b></col>, <cd>one that shows the days of the week and month.</cd> -- <col><b>Calendar month</b></col>. <cd>See under <er>Month</er>.</cd> -- <col><b>French Republican calendar</b></col>. <cd>See under <er>Vend<eacute/miaire</er>.</cd> -- <mcol><col><b>Gregorian calendar</b></col>, <col><b>Julian calendar</b></col>, <col><b>Perpetual calendar</b></col></mcol>. <cd>See under <er>Gregorian</er>, <er>Julian</er>, and <er>Perpetual</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendar</ent><br/
<hw>Cal"en*dar</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calendared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calendaring</conjf>.]</vmorph> <def>To enter or write in a calendar; to register.</def>  <rj><au>Waterhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendarial</ent><br/
<hw>Cal`en*da"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the calendar or a calendar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendary</ent><br/
<hw>Cal"en*da*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Calendarial.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calender</ent><br/
<hw>Cal"en*der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>calandre</ets>, LL. <ets>calendra</ets>, corrupted fr. L. <ets>cylindrus</ets> a cylinder, Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/. See <er>Cylinider</er>.]</ety> <sn>1.</sn> <def>A machine, used for the purpose of giving cloth, paper, etc., a smooth, even, and glossy or glazed surface, by cold or hot pressure, or for watering them and giving them a wavy appearance. It consists of two or more cylinders revolving nearly in contact, with the necessary apparatus for moving and regulating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who pursues the business of calendering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My good friend the <qex>calender</qex>.</q> <rj><qau>Cawper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calender</ent><br/
<hw>Cal"en*der</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calendered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calendering</conjf>.]</vmorph> <ety>[Cf. F. <ets>calandrer</ets>. See <er>Calender</er>, <pos>n.</pos>]</ety> <def>To press between rollers for the purpose of making smooth and glossy, or wavy, as woolen and silk stuffs, linens, paper, etc.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calender</ent><br/
<hw>Cal"en*der</hw>, <pos>n.</pos> <ety>[Per. <ets>qalender</ets>.]</ety> <def>One of a sect or order of fantastically dressed or painted dervishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendographer</ent><br/
<hw>Cal`en*dog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Calendar</ets> + <ets>-graph</ets> + <ets>er</ets>.]</ety> <def>One who makes calendars.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendrer</ent><br/
<hw>Cal"en*drer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who calenders cloth; a calender.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendrical</ent><br/
<ent>Calendric</ent><br/
<mhw>{ <hw>Ca*len"dric</hw> <pr>(?)</pr>, <hw>Ca*len"dric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos>, <def>Of or pertaining to a calendar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calends</ent><br/
<hw>Cal"ends</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OE. <ets>kalendes</ets> month, calends, AS. <ets>calend</ets> month, fr. L. <ets>calendae</ets>; akin to <ets>calare</ets> to call, proclaim, Gr. <?/<?/<?/<?/<?/<?/. CF. <er>Claim</er>.]</ety> <def>The first day of each month in the ancient Roman calendar.</def> <altsp>[Written also <asp>kalends</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Greek calends</b></col>, <cd>a time that will never come, as the Greeks had no calends.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendula</ent><br/
||<hw>Ca*len"du*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>calendae</ets> calends.]</ety> <fld>(Bot.)</fld> <def>A genus of composite herbaceous plants. One species, <spn>Calendula officinalis</spn>, is the common marigold, and was supposed to blossom on the calends of every month, whence the name.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calendulin</ent><br/
<hw>Ca*len"du*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A gummy or mucilaginous tasteless substance obtained from the marigold or calendula, and analogous to bassorin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calenture</ent><br/
<hw>Cal"en*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>calenture</ets>, fr. Sp. <ets>calenture</ets> heat, fever, fr. <ets>calentar</ets> to heat, fr. p. pr. of L. <ets>calere to be warm</ets>.]</ety> <fld>(Med.)</fld> <def>A name formerly given to various fevers occuring in tropics; esp. to a form of furious delirium accompanied by fever, among sailors, which sometimes led the affected person to imagine the sea to be a green field, and to throw himself into it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calenture</ent><br/
<hw>Cal"en*ture</hw>, <pos>v. i.</pos> <def>To see as in the delirium of one affected with calenture.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hath fed on pageants floating through the air<br/
Or <qex>calentures</qex> in depths of limpid flood.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calescence</ent><br/
<hw>Ca*les"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calescens</ets>, p. pr. of <ets>calescere</ets>, incho. of <ets>calere</ets> to be warm.]</ety> <def>Growing warmth; increasing heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calf</ent><br/
<hw>Calf</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Calves</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>calf</ets>, <ets>kelf</ets>, AS. <ets>cealf</ets>; akin to D. <ets>kalf</ets>, G. <ets>kalb</ets>, Icel. <ets>k<amac/lfr</ets>, Sw. <ets>kalf</ets>, Dan. <ets>kalv</ets>, Goth. <ets>kalb<omac/</ets>; cf. Skr. <ets>garbha</ets> fetus, young, Gr. <?/<?/<?/<?/<?/, Skr <ets>grabh</ets> to seize, conceive, Ir. <ets>colpa</ets>, <ets>colpach</ets>, a calf. <root/222.]</ety> <sn>1.</sn> <def>The young of the cow, or of the Bovine family of quadrupeds. Also, the young of some other mammals, as of the elephant, rhinoceros, hippopotamus, and whale.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Leather made of the skin of the calf; especially, a fine, light-colored leather used in bookbinding; <as>as, to bind books in <ex>calf</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An awkward or silly boy or young man; any silly person; a dolt.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some silly, doting, brainless <qex>calf</qex>.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small island near a larger; <as>as, the <ex>Calf</ex> of Man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A small mass of ice set free from the submerged part of a glacier or berg, and rising to the surface.</def>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[Cf. Icel. <ets>k<amac/lfi</ets>.]</ety> <def>The fleshy hinder part of the leg below the knee.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calf's-foot jelly</b></col>, <cd>jelly made from the feet of calves. The gelatinous matter of the feet is extracted by boiling, and is flavored with sugar, essences, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calfskin</ent><br/
<hw>Calf"skin`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The hide or skin of a calf; or leather made of the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cali</ent><br/
||<hw>Ca"li</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Hindoo Myth.)</fld> <def>The tenth avatar or incarnation of the god Vishnu.</def> <altsp>[Written also <asp>Kali</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calibre</ent><br/
<ent>Caliber</ent><br/
<mhw>{ <hw>Cal"i*ber</hw>, <hw>Cal"ibre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>calibre</ets>, perh. fr. L. <ets>qualibra</ets> of what pound, of what weight; hence, of what size, applied first to a ball or bullet; cf. also Ar. <ets>q<amac/lib</ets> model, mold. Cf. <er>Calipers</er>, <er>Calivere</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Gunnery)</fld> <def>The diameter of the bore, as a cannon or other firearm, or of any tube; or the weight or size of the projectile which a firearm will carry; <as>as, an 8 inch gun, a 12-pounder, a 44 <ex>caliber</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>caliber</qex> of empty tubes.</q> <rj><qau>Reid.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A battery composed of three guns of small <qex>caliber</qex>.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>caliber</xex> of firearms is expressed in various ways. Cannon are often designated by the weight of a solid spherical shot that will fit the bore; as, a 12-pounder; pieces of ordnance that project shell or hollow shot are designated by the diameter of their bore; as, a 12 inch mortar or a 14 inch shell gun; small arms are designated by hundredths of an inch expressed decimally; as, a rifle of .44 inch <xex>caliber</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The diameter of round or cylindrical body, as of a bullet or column.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Capacity or compass of mind.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caliber compasses</b></col>. <cd>See <er>Calipers</er>.</cd> -- <col><b>Caliber rule</b></col>, <cd>a gunner's calipers, an instrument having two scales arranged to determine a ball's weight from its diameter, and conversely.</cd> -- <col><b>A ship's caliber</b></col>, <cd>the weight of her armament.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calibrate</ent><br/
<hw>Cal"i*brate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To ascertain the caliber of, as of a thermometer tube; also, more generally, to determine or rectify the graduation of, as of the various standards or graduated instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calibration</ent><br/
<hw>Cal`ibra"*tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The process of estimating the caliber a tube, as of a thermometer tube, in order to graduate it to a scale of degrees; also, more generally, the determination of the true value of the spaces in any graduated instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calice</ent><br/
<hw>Cal"ice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Calice</er>.]</ety> <def>See <er>Chalice</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caliche</ent><br/
<hw>ca*li"che</hw> <pr>(k<adot/*l<emac/"ch<esl/)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a deposit of sand or clay on the surface containing crystallized salts such as sodium chloride or sodium nitrate; -- used especially of the sodium nitrate deposits of Chile and Peru.</def> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a stratum of calcium carbonate in the soil of an arid or semiarid region.</def><br/
[<source>PJC</source>]</p>

<p><ent>caliche-topped</ent><br/
<hw>caliche-topped</hw> <pos>adj.</pos> <def>covered with caliche{2}, a hard calcium-carbonate encrusted soil.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calicle</ent><br/
<hw>Cal"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caliculus</ets> a small cup, dim. of <ets>calicis</ets>, a cup. Cf <er>Calycle</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the small cuplike cavities, often with elevated borders, covering the surface of most corals. Each is formed by a polyp. <sd>(b)</sd> One of the cuplike structures inclosing the zooids of certain hydroids. See <er>Campanularian</er>.</def> <altsp>[Written also <asp>calycle</asp>. See <er>Calycle</er>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calico</ent><br/
<hw>Cal"i*co</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Calicoes</plw> <pr>(#)</pr>.</plu> <ety>[So called because first imported from <ets>Calicut</ets>, in the East Indies: cf. F. <ets>calicot</ets>.]</ety> <sn>1.</sn> <def>Plain white cloth made from cotton, but which receives distinctive names according to quality and use, <as>as, super <ex>calicoes</ex>, shirting <ex>calicoes</ex>, unbleached <ex>calicoes</ex>, etc.</as></def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The importation of printed or stained <qex>colicoes</qex> appears to have been coeval with the establishment of the East India Company</q>. <rj><qau>Beck (Draper's Dict. ).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cotton cloth printed with a figured pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States the term <xex>calico</xex> is applied only to the printed fabric.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calico bass</b></col> <fld>(Zool.)</fld>, <cd>an edible, fresh-water fish (<spn>Pomoxys sparaides</spn>) of the rivers and lake of the Western United States (esp. of the Misissippi valley.), allied to the sunfishes, and so called from its variegated colors; -- called also <altname>calicoback</altname>, <altname>grass bass</altname>, <altname>strawberry bass</altname>, <altname>barfish</altname>, and <altname>bitterhead</altname>.</cd> -- <col><b>Calico printing</b></col>, <cd>the art or process of impressing the figured patterns on calico.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calico</ent><br/
<hw>Cal"i*co</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made of, or having the appearance of, calico; -- often applied to an animal, as a horse or cat, on whose body are large patches of a color strikingly different from its main color.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calicoback</ent><br/
<hw>Cal"i*co*back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The calico bass.</def> <sd>(b)</sd> <def>An hemipterous insect (<spn>Murgantia histrionica</spn>) which injures the cabbage and other garden plants; -- called also <altname>calico bug</altname> and <altname>harlequin cabbage bug</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caliculate</ent><br/
<ent>Calicular</ent><br/
<mhw>{ <hw>Ca*lic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <hw>Ca*lic"u*late</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Relating to, or resembling, a cup; also improperly used for <xex>calycular</xex>, <xex>calyculate</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calid</ent><br/
<hw>Cal"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calidus</ets>, fr. <ets>calere</ets> to be hot.]</ety> <def>Hot; burning; ardent.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calidity</ent><br/
<hw>Ca*lid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Heat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caliduct</ent><br/
<hw>Cal"i*duct</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Caloriduct</er>.]</ety> <def>A pipe or duct used to convey hot air or steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Subterranean <qex>caliducts</qex> have been introduced.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Califate</ent><br/
<ent>Calif</ent><br/
<mhw>{ <hw>Ca"lif</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Cal"i*fate</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos>, etc. <def>Same as <er>Caliph</er>, <er>Caliphate</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>California condor</ent><br/
<hw>Cal`i*for"ni*a con"dor</hw> <fld>(Zool.)</fld> <def>a large vulture (<spn>Gymnogyps californianus</spn>), also called <altname>California vulture</altname>.</def> <note>In the late 20th century it is classed as an endangered species.  The <ex>California condor</ex> used to number in the thousands and ranged along the entire west coast of the United States.  By 1982 only 21 to 24 individuals could be identified in the wild.  A breeding program was instituted, and by 1996 over 50 birds were alive in captivity.  As of 1997, fewer than ten of the bred birds had been reintroduced into the wild.</note><br/
[<source>PJC</source>]</p>

<p><ent>California jack</ent><br/
<hw>Cal`i*for"ni*a jack"</hw> <pr>(?)</pr>. <def>A game at cards, a modification of seven-up, or all fours.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Californian</ent><br/
<hw>Cal`i*for"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to California.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of California.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caligation</ent><br/
<hw>Cal`i*ga"tion</hw> <pr>(-g<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>caligatio</ets>, fr. <ets>caligare</ets> to emit vapor, to be dark, from <ets>caligo</ets> mist, darkness.]</ety> <def>Dimness; cloudiness.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caliginosity</ent><br/
<hw>Ca*lig`i*nos"ity</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caliginosus</ets> dark. See <er>Caligation</er>.]</ety> <def>Darkness.</def> <mark>[R.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caliginous</ent><br/
<hw>Ca*lig"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caliginosus</ets>; cf. F. <ets>caligineux</ets>.]</ety> <def>Affected with darkness or dimness; dark; obscure.</def> <mark>[R.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>caliginous</qex> regions of the air.</q> <rj><qau>Hallywell.</qau></rj></p>

<p>-- <wordforms><wf>Ca*lig"i*nous*ly</wf>, <pos>adv.</pos> -- <wf>Ca*lig"i*nous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caligo</ent><br/
||<hw>Ca*li"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., darkness.]</ety> <fld>(Med.)</fld> <def>Dimness or obscurity of sight, dependent upon a speck on the cornea; also, the speck itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caligraphic</ent><br/
<hw>Cal`i*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Calligraphic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caligraphy</ent><br/
<hw>Ca*lig"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caligraphy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calin</ent><br/
||<hw>Ca"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Malay <ets>kelany</ets> tin, or fr. <ets>Kala'a</ets>, a town in India, fr. which it came.]</ety> <def>An alloy of lead and tin, of which the Chinese make tea canisters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calipash</ent><br/
<hw>Cal`i*pash"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carapace</ets>, Sp. <ets>carapacho</ets>. Cf <er>Calarash</er>, <er>Carapace</er>.]</ety> <def>A part of a turtle which is next to the upper shell. It contains a fatty and gelatinous substance of a dull greenish tinge, much esteemed as a delicacy in preparations of turtle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calipee</ent><br/
<hw>Cal"i*pee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Calipash</er>]</ety> <def>A part of a turtle which is attached to the lower shell. It contains a fatty and gelatinous substance of a light yellowish color, much esteemed as a delicacy.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calipers</ent><br/
<hw>Cal"i*pers</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Corrupted from <ets>caliber</ets>.]</ety> <def>An instrument, usually resembling a pair of dividers or compasses with curved legs, for measuring the diameter or thickness of bodies, as of work shaped in a lathe or planer, timber, masts, shot, etc.; or the bore of firearms, tubes, etc.; -- called also <altname>caliper compasses</altname>, or <altname>caliber compasses</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caliper square</b></col>, <cd>a draughtsman's or mechanic's square, having a graduated bar and adjustable jaw or jaws.</cd> <au>Knight.</au> -- <col><b>Vernier calipers</b></col>. <cd>See <er>Vernier</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caliph</ent><br/
<hw>Ca"liph</hw> <pr>(k<amac/"l<icr/f)</pr>, <pos>n.</pos> <ety>[OE. <ets>caliphe</ets>, <ets>califfe</ets>, F. <ets>calife</ets> (cf. Sp. <ets>califa</ets>), fr. Ar. <ets>khal<imac/fan</ets> successor, fr. <ets>khalafa</ets> to succed (i. e. a successor of Mohammed).]</ety> <def>Successor or vicar; the civil and religious leader of a Muslim state; -- a title of the successors of Mohammed both as temporal and spiritual rulers, used formerly by the sultans of Turkey.</def> <altsp>[Written also <asp>calif</asp>, <asp>kaliph</asp>, <asp>kalif</asp>, <asp>khalif</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caliphate</ent><br/
<hw>Cal"i*phate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>califat</ets>.]</ety> <def>The office, dignity, or government of a caliph or of the caliphs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calippic</ent><br/
<hw>Ca*lip"pic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <etsep>Calippus</etsep>, an Athenian astronomer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calippic period</b></col>, <cd>a period of seventy-six years, proposed by Calippus, as an improvement on the Metonic cycle, since the 6940 days of the Metonic cycle exceeded 19 years by about a quarter of a day, and exceeded 235 lunations by something more.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calisaya bark</ent><br/
<hw>Cal`i*sa"ya bark</hw> <pr>(?)</pr>. <def>A valuable kind of Peruvian bark obtained from the <spn>Cinchona Calisaya</spn>, and other closely related species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calistheneum</ent><br/
||<hw>Cal`is*the"ne*um</hw>, <pos>n.</pos> <ety>[NL.]</ety> <def>A gymnasium; esp. one for light physical exercise by women and children.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calisthenic</ent><br/
<hw>Cal`is*then"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kalo`s</grk> beautiful + <grk>sqe`nos</grk> strength.]</ety> <def>Of or pertaining to calisthenics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calisthenics</ent><br/
<hw>Cal`is*then"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science, art, or practice of healthful exercise of the body and limbs, to promote strength, gracefulness, and  general fitness; light gymnastics.</def><br/
<syn><b>Syn. --</b> calisthenics, calisthenic exercise, callisthenic exercise.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Caliver</ent><br/
<hw>Cal"i*ver</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted fr. <ets>caliber</ets>.]</ety> <def>An early form of hand gun, a variety of the arquebus; originally a gun having a regular size of bore.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calix</ent><br/
||<hw>Ca"lix</hw> <pr>(k<amac/"l<icr/ks)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A cup. See <er>Calyx</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calk</ent><br/
<hw>Calk</hw> <pr>(k<add/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calking</conjf>.]</vmorph> <ety>[Either corrupted fr. F. <ets>calfater</ets> (cf. Pg. <ets>calafetar</ets>, Sp. <ets>calafetear</ets>), fr. Ar. <ets>qalafa</ets> to fill up crevices with the fibers of palm tree or moss; or fr. OE. <ets>cauken</ets> to tred, through the French fr. L. <ets>calcare</ets>, fr. <ets>calx</ets> heel. Cf. <er>Calk</er> to copy, <ets>Inculcate</ets>.]</ety> <sn>1.</sn> <def>To drive tarred oakum into the seams between the planks of (a ship, boat, etc.), to prevent leaking. The calking is completed by smearing the seams with melted pitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make an indentation in the edge of a metal plate, as along a seam in a steam boiler or an iron ship, to force the edge of the upper plate hard against the lower and so fill the crevice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calk</ent><br/
<hw>Calk</hw> <pr>(k<acr/lk)</pr>, <pos>v. t.</pos> <ety>[E.<ets>calquer</ets> to trace, It. <ets>caicare</ets> to trace, to trample, fr. L. <ets>calcare</ets> to trample, fr. <ets>calx</ets> heel. Cf. <er>Calcarate</er>.]</ety> <def>To copy, as a drawing, by rubbing the back of it with red or black chalk, and then passing a blunt style or needle over the lines, so as to leave a tracing on the paper or other thing against which it is laid or held.</def> <altsp>[Written also <asp>calque</asp>]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 205 --></p>

<p><ent>Calk</ent><br/
<hw>Calk</hw> <pr>(k<add/k)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>calc</ets> shoe, hoof, L. <ets>calx</ets>, <ets>calcis</ets>, heel, <ets>calcar</ets>, spur.]</ety> <sn>1.</sn> <def>A sharp-pointed piece of iron or steel projecting downward on the shoe of a horse or an ox, to prevent the animal from slipping; -- called also <altname>calker</altname>, <altname>calkin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument with sharp points, worn on the sole of a shoe or boot, to prevent slipping.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>same as <er>caulk{2}</er>, <pos>n.</pos>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Calk</ent><br/
<hw>Calk</hw> <pr>(k<add/k)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To furnish with calks, to prevent slipping on ice; <as>as, to <ex>calk</ex> the shoes of a horse or an ox</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wound with a calk; as when a horse injures a leg or a foot with a calk on one of the other feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>same as <er>caulk{2}</er>, <pos>v. t.</pos>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Calker</ent><br/
<hw>Calk"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who calks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A calk on a shoe. See <er>Calk</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calkin</ent><br/
<hw>Calk"in</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A calk on a shoe. See <er>Calk</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calking</ent><br/
<hw>Calk"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of making seems tight, as in ships, or of furnishing with calks, as a shoe, or copying, as a drawing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calking iron</b></col>, <cd>a tool like a chisel, used in calking ships, tightening seams in ironwork, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Their left hand does the <qex>calking iron</qex> guide.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Call</ent><br/
<hw>Call</hw> <pr>(k<add/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Called</conjf> <pr>(k<add/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calling</conjf>]</vmorph> <ety>[OE. <ets>callen</ets>, AS. <ets>ceallian</ets>; akin to Icel. & Sw. <ets>kalla</ets>, Dan. <ets>kalde</ets>, D. <ets>kallen</ets> to talk, prate, OHG. <ets>kall<omac/n</ets> to call; cf. Gr. <grk>ghry`ein</grk> to speak, sing, Skr. <ets>gar</ets> to praise. Cf. <er>Garrulous</er>.]</ety> <sn>1.</sn> <def>To command or request to come or be present; to summon; <as>as, to <ex>call</ex> a servant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Call</qex> hither Clifford; bid him come amain</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To summon to the discharge of a particular duty; to designate for an office, or employment, especially of a religious character; -- often used of a divine summons; <as>as, to be <ex>called</ex> to the ministry</as>; sometimes, to invite; <as>as, to <ex>call</ex> a minister to be the pastor of a church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Paul . . . <qex>called</qex> to be an apostle</q> <rj><qau>Rom. i. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Holy Ghost said, Separate me Barnabas and Saul for the work whereunto I have <qex>called</qex> them.</q> <rj><qau>Acts xiii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To invite or command to meet; to convoke; -- often with <xex>together</xex>; <as>as, the President <ex>called</ex> Congress together</as>; to appoint and summon; <as>as, to <ex>call</ex> a meeting of the Board of Aldermen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>call</qex> we our high court of Parliament.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To give name to; to name; to address, or speak of, by a specifed name.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If you would but <qex>call</qex> me Rosalind.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And God <qex>called</qex> the light Day, and the darkness he <qex>called</qex> Night.</q> <rj><qau>Gen. i. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To regard or characterize as of a certain kind; to denominate; to designate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What God hath cleansed, that <qex>call</qex> not thou common.</q> <rj><qau>Acts x. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To state, or estimate, approximately or loosely; to characterize without strict regard to fact; <as>as, they <ex>call</ex> the distance ten miles; he <ex>called</ex> it a full day's work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The] army is <qex>called</qex> seven hundred thousand men.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To show or disclose the class, character, or nationality of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This speech <qex>calls</qex> him Spaniard.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To utter in a loud or distinct voice; -- often with <xex>off</xex>; <as>as, to <ex>call</ex>, or <ex>call off</ex>, the items of an account; to <ex>call</ex> the roll of a military company</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No parish clerk who <qex>calls</qex> the psalm so clear.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To invoke; to appeal to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>call</qex> God for a witness.</q> <rj><qau>2 Cor. i. 23 [Rev. Ver. ]</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To rouse from sleep; to awaken.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If thou canst awake by four o' the clock.<br/
I prithee <qex>call</qex> me. Sleep hath seized me wholly.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To call a bond</b></col>, <cd>to give notice that the amount of the bond will be paid.</cd> -- <col><b>To call a party</b></col> <fld>(Law)</fld>, <cd>to cry aloud his name in open court, and command him to come in and perform some duty requiring his presence at the time on pain of what may befall him.</cd> -- <col><b>To call back</b></col>, <cd>to revoke or retract; to recall; to summon back.</cd> -- <col><b>To call down</b></col>, <cd>to pray for, as blessing or curses.</cd> -- <col><b>To call forth</b></col>, <cd>to bring or summon to action; as, <xex>to call forth</xex> all the faculties of the mind.</cd> -- <col><b>To call in</b></col>, <sd>(a)</sd> <cd>To collect; as, <xex>to call in</xex> debts or money; ar to withdraw from cirulation; as, <xex>to call in</xex> uncurrent coin.</cd> <sd>(b)</sd> <cd>To summon to one's side; to invite to come together; as, <xex>to call in</xex> neighbors.</cd> -- <col><b>To call (any one) names</b></col>, <cd>to apply contemptuous names (to any one).</cd> -- <col><b>To call off</b></col>, <cd>to summon away; to divert; as, <xex>to call off</xex> the attention; <xex>to call off</xex> workmen from their employment.</cd> -- <col><b>To call out</b></col>. <cd><sd>(a)</sd> To summon to fight; to challenge. <sd>(b)</sd> To summon into service; as, <xex>to call out</xex> the militia.</cd> -- <col><b>To call over</b></col>, <cd>to recite separate particulars in order, as a roll of names.</cd> -- <col><b>To call to account</b></col>, <cd>to demand explanation of.</cd> -- <col><b>To call to mind</b></col>, <cd>to recollect; to revive in memory.</cd> -- <col><b>To call to order</b></col>, <cd>to request to come to order</cd>; as: <sd>(a)</sd> <cd>A public meeting, when opening it for business.</cd> <sd>(b)</sd> <cd>A person, when he is transgressing the rules of debate.</cd> -- <col><b>To call to the bar</b></col>, <cd>to admit to practice in courts of law.</cd> -- <col><b>To call up</b></col>. <sd>(a)</sd> <cd>To bring into view or recollection; as <xex>to call up</xex> the image of deceased friend.</cd> <sd>(b)</sd> <cd>To bring into action or discussion; to demand the consideration of; as, <xex>to call up</xex> a bill before a legislative body.</cd></cs></p>

<p><syn><b>Syn.</b> -- To name; denominate; invite; bid; summon; convoke; assemble; collect; exhort; warn; proclaim; invoke; appeal to; designate.</syn> -- <usage> <er>To Call</er>, <er>Convoke</er>, <er>Summon</er>. <xex>Call</xex> is the generic term; as, to <xex>call</xex> a public meeting. To <xex>convoke</xex> is to require the assembling of some organized body of men by an act of authority; as, the king <xex>convoked</xex> Parliament. To <xex>summon</xex> is to require attendance by an act more or less stringent anthority; as, to <xex>summon</xex> a witness.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Call</ent><br/
<hw>Call</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To speak in loud voice; to cry out; to address by name; -- sometimes with <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You must <qex>call</qex> to the nurse.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The angel of God <qex>called</qex> to Hagar.</q> <rj><qau>Gen. xxi. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a demand, requirement, or request.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>called</qex> for rooms, and he showed them one.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a brief visit; also, to stop at some place designated, as for orders.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He ordered her to <qex>call</qex> at the house once a week.</q> <rj><qau>Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To call for</b></col> <sd>(a)</sd> <cd>To demand; to require; <as>as, a crime <ex>calls for</ex> punishment; a survey, grant, or deed <ex>calls for</ex> the metes and bounds, or the quantity of land, etc., which it describes</as>.</cd> <sd>(b)</sd> <cd>To give an order for; to request.</cd>  <ldquo/Whenever the coach stopped, the sailor <xex>called for</xex> more ale.<rdquo/ <au>Marryat.</au> -- <mcol><col><b>To call on</b></col>, <col><b>To call upon</b></col></mcol>, <sd>(a)</sd> <cd>To make a short visit to; <as>as, <ex>call on</ex> a friend</as>.</cd> <sd>(b)</sd> <cd>To appeal to; to invite; to request earnestly; <as>as, <ex>to call upon</ex> a person to make a speech</as>.</cd> <sd>(c)</sd> <cd>To solicit payment, or make a demand, of a debt.</cd> <sd>(d)</sd> <cd>To invoke or play to; to worship; <as>as, <ex>to call upon</ex> God</as>.</cd> -- <col><b>To call out</b></col> <cd>To call or utter loudly; to brawl.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Call</ent><br/
<hw>Call</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of calling; -- usually with the voice, but often otherwise, as by signs, the sound of some instrument, or by writing; a summons; an entreaty; an invitation; <as>as, a <ex>call</ex> for help; the bugle's <ex>call</ex></as>.</def> <ldquo/<xex>Call</xex> of the trumpet.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I rose as at thy <qex>call</qex>, but found thee not.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A signal, as on a drum, bugle, trumpet, or pipe, to summon soldiers or sailors to duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>An invitation to take charge of or serve a church as its pastor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A requirement or appeal arising from the circumstances of the case; a moral requirement or appeal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dependence is a perpetual <qex>call</qex> upon humanity.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Running into danger without any <qex>call</qex> of duty.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A divine vocation or summons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>St. Paul himself believed he did well, and that he had a <qex>call</qex> to it, when he persecuted the Christians.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Vocation; employment.</def> <note>[In this sense, <xex>calling</xex> is generally used.]</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A short visit; <as>as, to make a <ex>call</ex> on a neighbor</as>; also, the daily coming of a tradesman to solicit orders.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The baker's punctual <qex>call</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Hunting)</fld> <def>A note blown on the horn to encourage the hounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Naut.)</fld> <def>A whistle or pipe, used by the boatswain and his mate, to summon the sailors to duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Fowling)</fld> <def>The cry of a bird; also a noise or cry in imitation of a bird; or a pipe to call birds by imitating their note or cry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Amer. Land Law)</fld> <def>A reference to, or statement of, an object, course, distance, or other matter of description in a survey or grant requiring or calling for a corresponding object, etc., on the land.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>The privilege to demand the delivery of stock, grain, or any commodity, at a fixed, price, at or within a certain time agreed on.</def> <mark>[Brokers' Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>See <er>Assessment</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>At call</b></col>, <it>or</it> <col><b>On call</b></col></mcol>, <cd>liable to be demanded at any moment without previous notice; as money on deposit.</cd> -- <col><b>Call bird</b></col>, <cd>a bird taught to allure others into a snare.</cd> -- <col><b>Call boy</b></col> <sd>(a)</sd> <cd>A boy who calls the actors in a theater; a boy who transmits the orders of the captain of a vessel to the engineer, helmsman, etc.</cd> <sd>(b)</sd> <cd>A waiting boy who answers a cal, or cames at the ringing of a bell; a bell boy.</cd> -- <col><b>Call note</b></col>, <cd>the note naturally used by the male bird to call the female. It is artificially applied by birdcatchers as a decoy.</cd> <au>Latham.</au> -- <col><b>Call of the house</b></col> <fld>(Legislative Bodies)</fld>, <cd>a calling over the names of members, to discover who is absent, or for other purposes; a calling of names with a view to obtaining the ayes and noes from the persons named.</cd> -- <col><b>Call to the bar</b></col>, <cd>admission to practice in the courts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calla</ent><br/
<hw>Cal"la</hw> <pr>(k<acr/l"l<adot/)</pr>, <pos>n.</pos> <ety>[Linn<ae/us derived <ets>Calla</ets> fr. Gr. <?/<?/<?/<?/<?/<?/<?/ a cock's wattles but cf. L. <ets>calla</ets>, <ets>calsa</ets>, name of an unknown plant, and Gr. <grk>kalo`s</grk> beautiful.]</ety> <fld>(Bot.)</fld> <def>A genus of plants, of the order <ord>Arace<ae/</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common <gen>Calla</gen> of cultivation is <spn>Richardia Africana</spn>, belonging to another genus of the same order. Its large spathe is pure white, surrounding a fleshy spike, which is covered with minute apetalous flowers.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>callable</ent><br/
<hw>callable</hw> <pos>adj.</pos> <def>subject to a demand for payment or redemption before the due date; -- of financial instruments; <as>as, <ex>callable</ex> bonds</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callat</ent><br/
<hw>Cal"lat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Callet</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>callat</qex> of boundless tongue.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>callathump</ent><br/
<hw>callathump</hw> <pos>n.</pos> <sn>1.</sn> <def>a noisy boisterous parade.</def><br/
<syn><b>Syn. --</b> callithump, callithump parade.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a noisy mock serenade (made by banging pans and kettles) to a newly married couple.</def><br/
<syn><b>Syn. --</b> shivaree, chivaree, charivari, callithump.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>call-back</ent><br/
<hw>call-back</hw> <pos>n.</pos> <sn>1.</sn> <def>a return call, especially a telephone call.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the recall of an employee to work after a layoff.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>call-board</ent><br/
<hw>call-board</hw> <pos>n.</pos> <def>a bulletin board backstage in a theater.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calle</ent><br/
<hw>Calle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Caul</er>.]</ety> <def>A kind of head covering; a caul.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caller</ent><br/
<hw>Call"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who calls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caller</ent><br/
||<hw>Cal"ler</hw> <pr>(?)</pr>, <pos>a.</pos> <mark>[Scot.]</mark> <sn>1.</sn> <def>Cool; refreshing; fresh; <as>as, a <ex>caller</ex> day; the <ex>caller</ex> air</as>.</def>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fresh; in good condition; <as>as, <ex>caller</ex> berrings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callet</ent><br/
<hw>Cal"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Ir. & Gael. <ets>caile</ets> a country woman, strumpet.]</ety> <def>A trull or prostitute; a scold or gossip.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>callat</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callet</ent><br/
<hw>Cal"let</hw> <pos>v. i.</pos> <def>To rail or scold.</def> <mark>[Obs.]</mark>  <rj><au>Brathwait.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callid</ent><br/
<hw>Cal"lid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>callidus</ets>, fr. <ets>callere</ets> to be thick-skinned, to be hardened, to be practiced, fr. <ets>callum</ets>, <ets>callus</ets>, callous skin, callosity, callousness.]</ety> <def>Characterized by cunning or shrewdness; crafty.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callidity</ent><br/
<hw>Cal*lid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calliditas</ets>.]</ety> <def>Acuteness of discernment; cunningness; shrewdness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Her eagly-eyed <qex>callidity</qex>.</q> <rj><qau>C. Smart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calligrapher</ent><br/
<hw>Cal*lig"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in calligraphy; a good penman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calligraphical</ent><br/
<ent>Calligraphic</ent><br/
<mhw>{ <hw>Cal`li*graph"ic</hw> <pr>(?)</pr>, <hw>Cal`li*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos>, <ety>[Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/; pref. <?/<?/<?/<?/<?/- (fr. <grk>kalo`s</grk> beautiful) + <grk>gra`fein</grk> to write; cf. F. <ets>calligraphique</ets>.]</ety> <def>Of or pertaining to calligraphy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Excellence in the <qex>calligraphic</qex> act.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calligraphist</ent><br/
<hw>Cal*lig"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A calligrapher</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calligraphy</ent><br/
<hw>Cal*lig"ra*phy</hw>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/: cf. F. <ets>calligraphie</ets>.]</ety> <def>Fair or elegant penmanship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callimorpha</ent><br/
<hw>Callimorpha</hw> <pos>n.</pos> <def>a genus of insects comprising the cinnabar moths.</def><br/
<syn><b>Syn. --</b> genus <gen>Callimorpha</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callinectes</ent><br/
<hw>Callinectes</hw> <pos>n.</pos> <def>a genus of crustaceans comprising the New World blue crabs.</def><br/
<syn><b>Syn. --</b> genus <gen>Callinectes</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calling</ent><br/
<hw>Call"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who calls; a crying aloud, esp. in order to summon, or to attact the attention of, some one.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A summoning or convocation, as of Parliament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The frequent <qex>calling</qex> and meeting of Parlaiment.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A divine summons or invitation; also, the state of being divinely called.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who hath . . . called us with an holy <qex>calling</qex>.</q> <rj><qau>2 Tim. i. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Give diligence to make yior <qex>calling</qex> . . . sure.</q> <rj><qau>2 Pet. i. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A naming, or inviting; a reading over or reciting in order, or a call of names with a view to obtaining an answer, as in legislative bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>One's usual occupation, or employment; vocation; business; trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The humble <qex>calling</qex> of ter female parent.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The persons, collectively, engaged in any particular professions or employment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To impose celibacy on wholy <qex>callings</qex>.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Title; appellation; name.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I am more proud to be Sir Rowland's son<br/
His youngest son, and would not change that <qex>calling</qex>.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Occupation; employment; business; trade; profession; office; engagement; vocation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callionymidae</ent><br/
<hw>Callionymidae</hw> <pos>n.</pos> <def>a natural family comprising the dragonets.</def><br/
<syn><b>Syn. --</b> family <fam>Callionymidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calliope</ent><br/
<hw>Cal*li"o*pe</hw> <pr>(k<acr/l*l<imac/"<osl/*p<esl/)</pr>, <pos>n.</pos> <ety>[L. <ets>Calliope</ets>, Gr. <grk>Kallio`ph</grk>, lit, the beautiful-voiced; pref. <grk>kalli-</grk> (from <grk>kalo`s</grk> beautiful) + <grk>'o`ps</grk>, <grk>'opo`s</grk>, voice.]</ety> <sn>1.</sn> <fld>(Class. Myth.)</fld> <def>The Muse that presides over eloquence and heroic poetry; mother of Orpheus, and chief of the nine Muses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>One of the asteroids. See <er>Solar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A musical instrument consisting of a series of steam whistles, toned to the notes of the scale, and played by keys arranged like those of an organ. It is sometimes attached to steamboat boilers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A beautiful species of humming bird (<spn>Stellula Calliope</spn>) of California and adjacent regions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calliophis</ent><br/
<hw>Calliophis</hw> <pos>n.</pos> <def>a genus comprising the Asian coral snakes.</def><br/
<syn><b>Syn. --</b> genus <gen>Calliophis</gen>, Callophis, genus <gen>Callophis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calliopsis</ent><br/
||<hw>Cal`li*op"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. pref. <grk>kalli-</grk> (fr. <grk>kalo`s</grk> beautiful) + <grk>'o`psis</grk> appearance.]</ety> <fld>(Bot.)</fld> <def>A popular name given to a few species of the genus <gen>Coreopsis</gen>, especially to <spn>Coreopsis tinctoria</spn> of Arkansas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callipash</ent><br/
<hw>Cal`li*pash"</hw> <pr>(<?/)</pr>, <pos>n.</pos> <def>See <er>Calipash</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callipee</ent><br/
<hw>Cal`li*pee"</hw> <pr>(<?/)</pr>, <pos>n.</pos> <def>See <er>Calipee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callipers</ent><br/
<hw>Cal`li*pers</hw> <pr>(<?/)</pr>, <pos>n. pl.</pos> <def>See Calipers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calliphoridae</ent><br/
<hw>Calliphoridae</hw> <pos>n.</pos> <def>a natural family comprising the blowflies.</def><br/
<syn><b>Syn. --</b> family <fam>Calliphoridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callirhoe</ent><br/
<hw>Callirhoe</hw> <pos>n.</pos> <def>small genus of North American herbs having usually red or purple flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Callirhoe</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callisaurus</ent><br/
<hw>Callisaurus</hw> <pos>n.</pos> <def>a genus of lizards including the the zebra-tailed lizard.</def><br/
<syn><b>Syn. --</b> genus <gen>Callisaurus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callisection</ent><br/
<hw>Cal`li*sec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>callere</ets> to be insensible + E. <ets>section</ets>.]</ety> <def>Painless vivisection; -- opposed to <contr>sentisection</contr>.</def>  <rj><au>B. G. Wilder.</au></rj></p>

<p><ent>Callistephus</ent><br/
<hw>Callistephus</hw> <pos>n.</pos> <def>a genus of plants having only one species, an erect Asiatic herb with large flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Callistephus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callisthenics</ent><br/
<ent>Callisthenic</ent><br/
<mhw>{ <hw>Cal`lis*then"ic</hw>, <pos>a.</pos>, <hw>Cal`lis*then"ics</hw> <pr>(?)</pr>, <pos>n.</pos> }</mhw> <def>See <er>Calisthenic</er>, <er>Calisthenics</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callithricidae</ent><br/
<hw>Callithricidae</hw> <pos>n.</pos> <def>a natural family comprising the marmosets.</def><br/
<syn><b>Syn. --</b> family <fam>Callithricidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callithrix</ent><br/
<hw>Callithrix</hw> <pos>n.</pos> <def>the type genus of the <fam>Callithricidae</fam>; the true marmosets.</def><br/
<syn><b>Syn. --</b> genus <gen>Callithrix</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callithump</ent><br/
<hw>Cal"li*thump`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A somewhat riotous parade, accompanied with the blowing of tin horns, and other discordant noises; also, a burlesque serenade; a charivari.</def> <mark>[U. S.]</mark> <altsp>[Also spelled <asp>callathump</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callithumpian</ent><br/
<hw>Cal`li*thump"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a callithump.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callitris</ent><br/
<hw>Callitris</hw> <pos>n.</pos> <def>a genus of evergreen monoecious coniferous trees or shrubs; the cypress pines.</def><br/
<syn><b>Syn. --</b> genus <gen>Callitris</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callophis</ent><br/
<hw>Callophis</hw> <pos>n.</pos> <def>a genus of Asian coral snakes.</def><br/
<syn><b>Syn. --</b> Calliophis, genus <gen>Calliophis</gen>, genus <gen>Callophis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callorhinus</ent><br/
<hw>Callorhinus</hw> <pos>n.</pos> <def>a genu sof fur seals.</def><br/
<syn><b>Syn. --</b> genus <gen>Callorhinus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callosan</ent><br/
<hw>Cal*lo"san</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of the callosum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callose</ent><br/
<hw>Cal"lose</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Callous</er>.]</ety> <fld>(Bot.)</fld> <def>Furnished with protuberant or hardened spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callosity</ent><br/
<hw>Cal*los"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Callosities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>callasitas</ets>; cf. F. <ets>calost<eacute/</ets>.]</ety> <def>A hard or thickened spot or protuberance; a hardening and thickening of the skin or bark of a part, eps. as a result of continued pressure or friction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callosum</ent><br/
||<hw>Cal*lo"sum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. <ets>callosus</ets> callous, hard.]</ety> <fld>(Anat.)</fld> <def>The great band commissural fibers which unites the two cerebral hemispheres. See <cref>corpus callosum</cref>, under <er>Carpus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callot</ent><br/
<hw>Cal"lot</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A plant coif or skullcap. Same as <er>Calotte</er>.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callous</ent><br/
<hw>Cal"lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>callosus</ets> callous hard, fr. <ets>callum</ets>, <ets>callus</ets>, callous skin: cf. F. <ets>calleux</ets>.]</ety> <sn>1.</sn> <def>Hardened; indurated.</def>  <ldquo/A <xex>callous</xex> hand.<rdquo/ <au>Goldsmith.</au> <ldquo/A <xex>callous</xex> ulcer.<rdquo/ <au>Dunglison.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hardened in mind; insensible; unfeeling; unsusceptible.</def> <ldquo/The <xex>callous</xex> diplomatist.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is an immense blessing to be perfectly <qex>callous</qex> to ridicule.</q> <rj><qau>T. Arnold.</qau></rj></p>

<p><syn><b>Syn.</b> -- Obdurate; hard; hardened; indurated; insensible; unfeeling; unsusceptible. See <er>Obdurate</er>.</syn></p>

<p>-- <wordforms><wf>Cal"lous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>callousness</qex> and numbness of soul.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>calloused</ent><br/
<hw>cal"loused</hw> <pos>adj.</pos> <def>having callouses; -- of skin.</def><br/
<syn><b>Syn. --</b> thickened.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>callousness</ent><br/
<hw>cal"lous*ness</hw> <pos>n.</pos> <def>lack of passion or feeling.</def><br/
<syn><b>Syn. --</b> unfeelingness, hardness, insensibility.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callow</ent><br/
<hw>Cal"low</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>calewe</ets>, <ets>calu</ets>, bald, AS. <ets>calu</ets>; akin to D. <ets>kaal</ets>, OHG. <ets>chalo</ets>, G. <ets>Kuhl</ets>; cf. L. <ets>calvus</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Destitute of feathers; naked; unfledged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An in the leafy summit, spied a nest,<br/
Which, o'er the <qex>callow</qex> young, a sparrow pressed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Immature; boyish; <ldquo/green<rdquo/; <as>as, a <ex>callow</ex> youth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I perceive by this, thou art but a <qex>callow</qex> maid.</q> <rj><qau>Old Play [1675].</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Callow</ent><br/
<hw>Cal*low"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <ety>[Named from its note.]</ety> <def>A kind of duck. See <er>Old squaw</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>callowness</ent><br/
<hw>cal"low*ness</hw> <pos>n.</pos> <def>having a lack of experience of life.</def><br/
<syn><b>Syn. --</b> jejuneness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calluna</ent><br/
<hw>Calluna</hw> <pos>n.</pos> <def>a genus having only one species.</def><br/
<syn><b>Syn. --</b> genus <gen>Calluna</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Callus</ent><br/
<hw>Cal"lus</hw> <pr>(k<acr/l"l<ucr/s)</pr>, <pos>n.</pos> <ety>[L. See <er>Callous</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>Same as <er>Callosity</er></def>. <sd>(b)</sd> <def>The material of repair in fractures of bone; a substance exuded at the site of fracture, which is at first soft or cartilaginous in consistence, but is ultimately converted into true bone and unites the fragments into a single piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Hort.)</fld> <def>The new formation over the end of a cutting, before it puts out rootlets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calm</ent><br/
<hw>Calm</hw> <pr>(k<aum/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>calme</ets>, F. <ets>calme</ets>, fr. It. or Sp. <ets>calma</ets> (cf. Pg. <ets>calma</ets> heat), prob. fr. LL. <ets>cauma</ets> heat, fr. Gr. <grk>kay^ma</grk> burning heat, fr. <grk>kai`ein</grk> to burn; either because during a great heat there is generally also a calm, or because the hot time of the day obliges us seek for shade and quiet; cf. <er>Caustic</er>]</ety> <def>Freedom from motion, agitation, or disturbance; a cessation or absence of that which causes motion or disturbance, as of winds or waves; tranquility; stillness; quiet; serenity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wind ceased, and there was a great <qex>calm</qex>.</q> <rj><qau>Mark. iv. 39.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>calm</qex> before a storm is commonly a peace of a man's own making.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calm</ent><br/
<hw>Calm</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calmed</conjf> <pr>(k<aum/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Calming</conjf>.]</vmorph> <ety>[Cf. F. <ets>calmer</ets>. See <er>Calm</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To make calm; to render still or quiet, as elements; <as>as, <ex>to calm</ex> the winds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>calm</qex> the tempest raised by Eolus.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deliver from agitation or excitement; to still or soothe, as the mind or passions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Passions which seem somewhat <qex>calmed</qex>.</q> <rj><qau>Atterbury.</qau></rj></p>

<p><syn><b>Syn.</b> -- To still; quiet; appease; allay; pacify; tranquilize; soothe; compose; assuage; check; restrain.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 206 --></p>

<p><ent>Calm</ent><br/
<hw>Calm</hw> <pr>(k<aum/m)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Calmer</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Calmest</adjf> <pr>(-<ecr/st)</pr>]</amorph> <sn>1.</sn> <def>Not stormy; without motion, as of winds or waves; still; quiet; serene; undisturbed.</def> <ldquo/<xex>Calm</xex> was the day.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now all is <qex>calm</qex>, and fresh, and still.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Undisturbed by passion or emotion; not agitated or excited; tranquil; quiet in act or speech.</def>  <ldquo/<xex>Calm</xex> and sinless peace.<rdquo/ <au>Milton.</au> <ldquo/With <xex>calm</xex> attention.<rdquo/ <au>Pope.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Such <qex>calm</qex> old age as conscience pure<br/
And self-commanding hearts ensure.</q> <rj><qau>Keble.</qau></rj></p>

<p><syn><b>Syn.</b> -- Still; quiet; undisturbed; tranquil; peaceful; serene; composed; unruffled; sedate; collected; placid.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calmer</ent><br/
<hw>Calm"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, makes calm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>calming</ent><br/
<hw>calming</hw> <pos>adj.</pos> <sn>1.</sn>  <def>tending to soothe or calm or tranquilize.</def><br/
<syn><b>Syn. --</b> ataractic, ataraxic, sedative, soothing, tranquilizing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>freeing from fear and anxiety.</def><br/
<syn><b>Syn. --</b> assuasive, pacifying, soothing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calmly</ent><br/
<hw>Calm"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a calm manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gentle stream which <qex>calmly</qex> flows.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calmness</ent><br/
<hw>Calm"ness</hw>, <pos>n.</pos> <def>The state of quality of being calm; quietness; tranquillity; self-repose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gentle <qex>calmness</qex> of the flood.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hes <qex>calmness</qex> was the repose of conscious power.</q> <rj><qau>E. Everett.</qau></rj></p>

<p><syn><b>Syn.</b> -- Quietness; quietude; stillness; tranquillity; serenity; repose; composure; sedateness; placidity.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calmucks</ent><br/
<hw>Cal"mucks</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Calmuck</singw>.</sing> <def>A branch of the Mongolian race inhabiting parts of the Russian and Chinese empires; also (<singf>sing.</singf>), the language of the Calmucks.</def> <altsp>[Written also <asp>Kalmucks</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calmy</ent><br/
<hw>Calm"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Fr. <er>Calm</er>, <pos>n.</pos>]</ety> <def>Tranquil; peaceful; calm.</def> <mark>[Poet.]</mark> <ldquo/A still and <xex>calmy</xex> day<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calocedrus</ent><br/
<hw>Calocedrus</hw> <pos>n.</pos> <def>a genus of tall evergreens of West North America and East Asia; formerly included in genus <gen>Libocedrus</gen>.</def><br/
<syn><b>Syn. --</b> genus <gen>Calocedrus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calochortus</ent><br/
<hw>Calochortus</hw> <pos>n.</pos> <def>large genus of West North American leafy-stemmed bulbous herbs.</def><br/
<syn><b>Syn. --</b> genus <gen>Calochortus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calomel</ent><br/
<hw>Cal"o*mel</hw> <pr>(k<acr/l"<osl/*m<ecr/l)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kalo`s</grk> beautiful + <grk>me`las</grk> black.  So called from its being white, though made from a black mixture of mercury and corrosive sublimate. Cf. F. <ets>calom<eacute/las</ets>.]</ety> <fld>(Chem.)</fld> <def>Mild chloride of mercury, <chform>Hg2Cl2</chform>, a heavy, white or yellowish white substance, insoluble and tasteless, much used in medicine as a mercurial and purgative; mercurous chloride. It occurs native as the mineral <altname>horn quicksilver</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calopogon</ent><br/
<hw>Calopogon</hw> <pos>n.</pos> <def>a genus of terrestrial orchids of North America.</def><br/
<syn><b>Syn. --</b> genus <gen>Calopogon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calorescence</ent><br/
<hw>Cal`o*res"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calor</ets> heat.]</ety> <fld>(Physics)</fld> <def>The conversion of obscure radiant heat into light; the transmutation of rays of heat into others of higher refrangibility.</def>  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caloric</ent><br/
<hw>Ca*lor"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calor</ets> heat; cf. F. <ets>calorique</ets>.]</ety> <fld>(Physics)</fld> <def>The principle of heat, or the agent to which the phenomena of heat and combustion were formerly ascribed; -- not now used in scientific nomenclature, but sometimes used as a general term for heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Caloric</qex> expands all bodies.</q> <rj><qau>Henry.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caloric</ent><br/
<hw>Ca*lor"ic</hw>, <pos>a.</pos> <def>Of or pertaining to caloric.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caloric engine</b></col>, <cd>a kind of engine operated by heated air.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caloricity</ent><br/
<hw>Cal`o*ric"ity</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>A faculty in animals of developing and preserving the heat necessary to life, that is, the animal heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caloriduct</ent><br/
<hw>Ca*lor"i*duct</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calor</ets> heat (fr. <ets>calere</ets> to warm) + E. <ets>duct</ets>.]</ety> <def>A tube or duct for conducting heat; a caliduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorie</ent><br/
<hw>Cal"o*rie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>calor</ets> heat.]</ety> <fld>(Physics)</fld> <def>The unit of heat according to the French standard; the amount of heat required to raise the temperature of one kilogram (sometimes, one gram) of water one degree centigrade, or from 0<deg/ to 1<deg/. Compare the English standard unit, <cref>Foot pound</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorifacient</ent><br/
<hw>Ca*lor`i*fa"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>See <er>Calorificient</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorifere</ent><br/
<hw>Ca*lor"i*fere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>calorif<egrave/re</ets>, fr. L. <ets>calor</ets> heat + <ets>ferre</ets> to bear.]</ety> <def>An apparatus for conveying and distributing heat, especially by means of hot water circulating in tubes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorifiant</ent><br/
<hw>Ca*lor`i*fi"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>See <er>Calorificient</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorific</ent><br/
<hw>Cal`o*rif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calorificus</ets>; <ets>calor</ets> heat + <ets>facere</ets> to make; cf. F. <ets>calorifique</ets>.]</ety> <def>Possessing the quality of producing heat; heating.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calorific rays</b></col>, <cd>the invisible, heating rays which emanate from the sun, and from burning and heated bodies.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorification</ent><br/
<hw>Ca*lor`i*fi*ca"tion</hw> <pr>(k<adot/*l<ocr/r`<icr/*f<icr/*k<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>calorification</ets>.]</ety> <def>Production of heat, esp. animal heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorificient</ent><br/
<hw>Ca*lor`i*fi"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Having, or relating to the power of producing heat; -- applied to foods which, being rich in carbon, as the fats, are supposed to give rise to heat in the animal body by oxidation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorimeter</ent><br/
<hw>Cal`o*rim"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calor</ets> heat + <ets>-meter</ets>; cf. F. <ets>calorim<egrave/tre</ets>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>An apparatus for measuring the amount of heat contained in bodies or developed by some mechanical or chemical process, as friction, chemical combination, combustion, etc.  For combustion processes, a <styp><er>bomb calorimeter</er></styp> may be used.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <fld>(Engineering)</fld> <def>An apparatus for measuring the proportion of unevaporated water contained in steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorimetric</ent><br/
<hw>Ca*lor`i*met"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the process of using the calorimeter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Satisfactory <qex>calorimetric</qex> results.</q> <rj><qau>Nichol.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorimetry</ent><br/
<hw>Cal`o*rim"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>Measurement of the quantities of heat in bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calorimotor</ent><br/
<hw>Ca*lor`i*mo"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>calor</ets> heat + E. <ets>motor</ets>.]</ety> <fld>(Physics)</fld> <def>A voltaic battery, having a large surface of plate, and producing powerful heating effects.</def></p>

<p><ent>Calorisator</ent><br/
||<hw>Ca*lor"i*sa`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., heater, fr. L. <ets>calor</ets> heat.]</ety> <def>An apparatus used in beet-sugar factories to heat the juice in order to aid the diffusion.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Callot</ent><br/
<ent>Calotte</ent><br/
<mhw>{ ||<hw>Ca*lotte"</hw> <pr>(?)</pr>, <hw>Cal"lot</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[F. <ets>calotte</ets>, dim. of <ets>cale</ets> a sort of flat cap. Cf. <er>Caul</er>.]</ety> <def>A close cap without visor or brim.</def> Especially: <sd>(a)</sd> <def>Such a cap, worn by English serjeants at law.</def> <sd>(b)</sd> <def>Such a cap, worn by the French cavalry under their helmets.</def> <sd>(c)</sd> <def>Such a cap, worn by the clergy of the Roman Catholic Church.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To assume the calotte</b></col>, <cd>to become a priest.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calotype</ent><br/
<hw>Cal"o*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kalo`s</grk> beautiful + <grk>ty`pos</grk> type.]</ety> <fld>(Photog.)</fld> <def>A method of taking photographic pictures, on paper sensitized with iodide of silver; -- also called <altname>Talbotype</altname>, from the inventor, Mr. Fox. <ets>Talbot</ets>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caloyer</ent><br/
<hw>Ca*loy"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. NGr. <?/<?/<?/<?/<?/<?/<?/<?/<?/ a monk; <grk>kalo`s</grk> beautiful, good + <?/<?/<?/<?/<?/, equiv. to Gr. <?/<?/<?/<?/<?/ an old man.]</ety> <def>A monk of the Greek Church; a cenobite, anchoret, or recluse of the rule of St. Basil, especially, one on or near Mt. Athos.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calpe</ent><br/
<hw>Calpe</hw> <pos>n.</pos> <def>the Rock of Gibraltar, a limestone promontory at the southern tip of Spain; associated with Britain.</def><br/
<syn><b>Syn. --</b> Gibraltar, Rock of Gibraltar.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calque</ent><br/
<hw>Calque</hw>, <pos>v. t.</pos> <def>See 2d <er>Calk</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caltrap</ent><br/
<ent>Caltrop</ent><br/
<mhw>{ <hw>Cal"trop</hw> <pr>(?)</pr>, <hw>Cal"trap</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[OE. <ets>calketrappe</ets>, <ets>calletrappe</ets>, caltor (in both senses), fr. AS. <ets>collr<ae/ppe</ets>, <ets>calcetreppe</ets>, sort of thistle; cf. F. <ets>chaussetrape</ets> star thistle, trap, It. <ets>calcatreppo</ets>, <ets>calcatreppolo</ets>, star thistle. Perh. from L. <ets>calx</ets> heel + the same word as E. <ets>trap</ets>. See 1st <er>Trap</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of herbaceous plants (<gen>Tribulus</gen>) of the order <ord>Zygophylle<ae/</ord>, having a hard several-celled fruit, armed with stout spines, and resembling the military instrument of the same name. The species grow in warm countries, and are often very annoying to cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>An instrument with four iron points, so disposed that, any three of them being on the ground, the other projects upward. They are scattered on the ground where an enemy's cavalry are to pass, to impede their progress by endangering the horses' feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumba</ent><br/
<hw>Ca*lum"ba</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[from <ets>kalumb</ets>, its native name in Mozambique.]</ety> <fld>(Med.)</fld> <def>The root of a plant (<spn>Jateorrhiza Calumba</spn>, and probably <spn>Cocculus palmatus</spn>), indigenous in Mozambique. It has an unpleasantly bitter taste, and is used as a tonic and antiseptic.</def> <altsp>[Written also <asp>colombo</asp>, <asp>columbo</asp>, and <asp>calombo</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>American calumba</b></col>, <cd>the <spn>Frasera Carolinensis</spn>, also called <altname>American gentian</altname>. Its root has been used in medicine as bitter tonic in place of <xex>calumba</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumbin</ent><br/
<hw>Ca*lum"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A bitter principle extracted as a white crystalline substance from the calumba root.</def> <altsp>[Written also <asp>colombin</asp>, and <asp>columbin</asp>]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumet</ent><br/
<hw>Cal"u*met</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>calumet</ets>, fr. L. <ets>calamus</ets> reed. See <er>Halm</er>, and cf. <er>Shawm</er>.]</ety> <def>A kind of pipe, used by the North American Indians for smoking tobacco. The bowl is usually made of soft red stone, and the tube is a long reed often ornamented with feathers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Smoked the <qex>calumet</qex>, the Peace pipe,<br/
As a signal to the nations.</q> <rj><qau>Lowgfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>calumet</xex> is used as a symbol of peace. To accept the calumet is to agree to terms of peace, and to refuse it is to reject them. The calumet of peace is used to seal or ratify contracts and alliances, and as an evidence to strangers that they are welcome.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumniate</ent><br/
<hw>Ca*lum"ni*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calumniated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>calumniating</conjf>.]</vmorph> <ety>[L. <ets>calumniatus</ets>, p. p. of <ets>calumniari</ets>. See <er>Calumny</er>, and cf. <er>Challenge</er>, <pos>v. t.</pos>]</ety> <def>To accuse falsely and maliciously of a crime or offense, or of something disreputable; to slander; to libel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hatred unto the truth did always falsely report and <qex>calumniate</qex> all godly men's doings.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Syn</b></col>. -- <cd>To asperse; slander; defame; vilify; traduce; belie; bespatter; blacken; libel. See <er>Asperse</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumniate</ent><br/
<hw>Ca*lum"ni*ate</hw>, <pos>v. i.</pos> <def>To propagate evil reports with a design to injure the reputation of another; to make purposely false charges of some offense or crime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumniation</ent><br/
<hw>Ca*lum`ni*a"tion</hw> <pr>(k<adot/*l<ucr/m`n<icr/*<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <def>False accusation of crime or offense, or a malicious and false representation of the words or actions of another, with a view to injure his good name.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>calumniation</qex> of her principal counselors.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumniator</ent><br/
<hw>Ca*lum`ni*a"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who calumniates.</def></p>

<p><syn><b>Syn.</b> -- Slanderer; defamer; libeler; traducer.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumniatory</ent><br/
<hw>Ca*lum"ni*a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing calumny; slanderous.</def>  <rj><au>Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumnious</ent><br/
<hw>Ca*lum"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calumniosus</ets>.]</ety> <def>Containing or implying calumny; false, malicious, and injurious to reputation; slanderous; <as>as, <ex>calumnious</ex> reports</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Virtue itself 'scapes not <qex>calumnious</qex> strokes.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn>. Slanderous; defamatory; scurrilous; opprobrious; derogatory; libelous; abusive.</syn></p>

<p>-- <wordforms><wf>Ca*lum"ni*ous*ly</wf>, <pos>adv.</pos> -- <wf>Ca*lum"ni*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calumny</ent><br/
<hw>Cal"um*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Calumnies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>calumnia</ets>, fr. <ets>calvi</ets> to devise tricks, deceive; cf. F. <ets>calomnie</ets>. Cf. <er>Challenge</er>, <pos>n.</pos>]</ety> <def>False accusation of a crime or offense, maliciously made or reported, to the injury of another; malicious misrepresentation; slander; detraction.</def> <ldquo/Infamous <xex>calumnies</xex>.<rdquo/  <rj><au>Motley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be thou as chaste as ice, as pure as snow, thou shalt not escape <qex>calumny</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvaria</ent><br/
||<hw>Cal*va"ri*a</hw> <pr>(k<acr/l*v<amac/"r<icr/*<adot/)</pr>, <pos>n.</pos> <ety>[L. See <er>Calvary</er>.]</ety> <fld>(Anat.)</fld> <def>The bones of the cranium; more especially, the bones of the domelike upper portion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvary</ent><br/
<hw>Cal"va*ry</hw> <pr>(k<acr/l"v<adot/*r<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>calvaria</ets> a bare skull, fr. <ets>calva</ets> the scalp without hair. fr. <ets>calvus</ets> bald; cf. F. <ets>calvaire</ets>.]</ety> <sn>1.</sn> <def>The place where Christ was crucified, on a small hill outside of Jerusalem.</def>  <rj><au>Luke xxiii. 33.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Latin <xex>calvaria</xex> is a translation of the Greek <grk>krani`on</grk> of the Evangelists, which is an interpretation of the Hebrew <xex>Golgotha</xex>.</note>  <rj><au>Dr. W. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A representation of the crucifixion, consisting of three crosses with the figures of Christ and the thieves, often as large as life, and sometimes surrounded by figures of other personages who were present at the crucifixion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Her.)</fld> <def>A cross, set upon three steps; -- more properly called <altname>cross calvary</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvatia</ent><br/
<hw>Calvatia</hw> <pos>n.</pos> <def>a genus of puffballs having outer casings whose upper parts break at maturity into angular pieces to expose the spores.</def><br/
<syn><b>Syn. --</b> genus <gen>Calvatia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calve</ent><br/
<hw>Calve</hw> <pr>(k<aum/v)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Calved</conjf> 3; <pos>p. pr. & vb. n.</pos> <conjf>Calving</conjf>.]</vmorph> <ety>[AS. <ets>cealfian</ets>. See <er>Calf</er>.]</ety> <sn>1.</sn> <def>To bring forth a calf.</def> <ldquo/Their cow <xex>calveth</xex>.<rdquo/  <rj><au>Job xxi. 10.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring forth young; to produce offspring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Canst thou mark when the hinds do <qex>calve</qex>?</q> <rj><qau>Job xxxix. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The grassy clods now <qex>calved</qex>.</q> <rj><qau>Molton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Phys. Geog.)</fld> <def>To throw off fragments which become icebergs; -- said of a glacier.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Calver</ent><br/
<hw>Cal"ver</hw> <pr>(k<acr/l"v<etil/r)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To cut in slices and pickle, as salmon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For a change, leave <qex>calvered</qex> salmon and eat sprats.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To crimp; <as>as, <ex>calvered</ex> salmon</as>.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calver</ent><br/
<hw>Cal"ver</hw>, <pos>v. i.</pos> <def>To bear, or be susceptible of, being calvered; <as>as, grayling's flesh will <ex>calver</ex></as>.</def>  <rj><au>Catton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvessnout</ent><br/
<hw>Calves"*snout</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Snapdragon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvin</ent><br/
<hw>Cal"vin</hw> <pos>n.</pos> <sn>1.</sn> <def><person>John Calvin</person>, a French theologian and reformer; born 1509, died 1564.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calvinism</ent><br/
<hw>Cal"vin*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>Calvinisme</ets>.]</ety> <def>The theological tenets or doctrines of John Calvin (a French theologian and reformer of the 16th century) and his followers, or of the so-called calvinistic churches.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The distinguishing doctrines of this system, usually termed the <xex>five points of Calvinism</xex>, are original sin or total depravity, election or predestination, particular redemption, effectual calling, and the perseverance of the saints. It has been subject to many variations and modifications in different churches and at various times.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvinist</ent><br/
<hw>Cal"vin*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>Calviniste</ets>.]</ety> <def>A follower of Calvin; a believer in Calvinism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvinistical</ent><br/
<ent>Calvinistic</ent><br/
<mhw>{ <hw>Cal`vin*is"tic</hw> <pr>(?)</pr>, <hw>Cal`vin*is"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to Calvin, or Calvinism; following Calvin; accepting or Teaching Calvinism.</def> <ldquo/<xex>Calvinistic</xex> training.<rdquo/  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvinize</ent><br/
<hw>Cal"vin*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To convert to Calvinism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calvish</ent><br/
<hw>Calv"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a calf; stupid.</def>  <rj><au>Sheldon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calx</ent><br/
<hw>Calx</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Calxes</plw> <pr>(#)</pr>, L. <plw>Calces</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>Calx</ets>, <ets>calcis</ets>. limestone; cf. Gr. <?/ gravel. <?/, <?/, pebble, Skr. <?/ gravel, Ir. <ets>carraic</ets> rock Gael. <ets>carraig</ets>, W. <ets>careg</ets>, stone. Cf. <er>Chalk</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Quicklime.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>The substance which remains when a metal or mineral has been subjected to calcination or combustion by heat, and which is, or may be, reduced to a fine powder.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Metallic calxes are now called <xex>oxides</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Broken and refuse glass, returned to the post.</def></p>

<p><ent>calyceal</ent><br/
<hw>calyceal</hw> <pos>adj.</pos> <def>of or pertaining to a calyx.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>callyciflorous</ent><br/
<ent>Calycifloral</ent><br/
<mhw>{ <hw>Ca*lyc`i*flo"ral</hw> <pr>(?)</pr>, <hw>cal*lyc`i*flo"rous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>calyx</ets>, <ets>-ycis</ets>, calyx + <ets>flos</ets>, <ets>floris</ets>, flower.]</ety> <fld>(Bot.)</fld> <def>Having the petals and stamens adnate to the calyx; -- applied to a subclass of dicotyledonous plants in the system of the French botanist <person>Candolle</person>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calyciform</ent><br/
<hw>Ca*lyc"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>calyx</ets>, <ets>calycis</ets>, calyx + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the form or appearance of a calyx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calycine</ent><br/
<ent>Calycinal</ent><br/
<mhw>{ <hw>Ca*lyc"i*nal</hw> <pr>(?)</pr>, <hw>Cal"y*cine</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Pertaining to a calyx; resembling or having the nature of a calyx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calycle</ent><br/
<hw>Cal"y*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.<ets>calyculus</ets> small flower bud, calyx, dim. of <ets>calyx</ets>. See <er>Calyx</er>, and cf. <er>Calicle</er>.]</ety> <fld>(Bot.)</fld> <def>A row of small bracts, at the base of the calyx, on the outside.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calycled</ent><br/
<hw>Cal"y*cled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Calyculate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calycozoa</ent><br/
||<hw>Cal`y*co*zo"a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/, <?/, cup or calyx a flower + <?/ animal.]</ety> <fld>(Zool.)</fld> <def>A group of acalephs of which <xex>Lucernaria</xex> is the type. The body is cup-shaped with eight marginal lobes bearing clavate tentacles. An aboral sucker serves for attachment. The interior is divided into four large compartments. See <er>Lucernarida</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calycular</ent><br/
<hw>Ca*lyc"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Pertaining to, or resembling, the bracts of a calycle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calyculated</ent><br/
<ent>Calyculate</ent><br/
<mhw>{ <hw>Ca*lyc"u*late</hw> <pr>(?)</pr>, <hw>Ca*lyc"u*la`ted</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having a set of bracts resembling a calyx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calymene</ent><br/
<hw>Ca*lym"e*ne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. (<?/) concealed, p. p. of <?/ to conceal.]</ety> <fld>(Zool.)</fld> <def>A genus of trilobites characteristic of the Silurian age.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calyon</ent><br/
<hw>Cal"yon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Flint or pebble stone, used in building walls, etc.</def>  <rj><au>Haliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calypso</ent><br/
<hw>Ca*lyp"so</hw> <pr>(k<adot/*l<icr/p"s<osl/)</pr>, <pos>n.</pos> <ety>[The Latinized Greek name of a beautiful nymph.]</ety> <fld>(Bot.)</fld> <def>A small and beautiful species of orchid, having a flower variegated with purple, pink, and yellow. It grows in cold and wet localities in the northern part of the United States. The <spn>Calypso borealis</spn> is the only orchid which reaches 68<deg/ N.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>calypter</ent><br/
<hw>calypter</hw> <pos>n.</pos> <fld>(Zool.)</fld> <def>a scalelike structure between the base of the wing and the halter of a two-winged fly.</def><br/
<syn><b>Syn. --</b> alula.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calyptra</ent><br/
<hw>Ca*lyp"tra</hw> <pr>(k<adot/*l<icr/p"tr<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kaly`ptra</grk> a covering for the head, fr. <grk>kaly`ptein</grk> to cover.]</ety>  <fld>(Bot.)</fld> <def>A little hood or veil, resembling an extinguisher in form and position, covering each of the small flasklike capsules which contain the spores of mosses; also, any similar covering body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calyptriform</ent><br/
<hw>Ca*lyp"tri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Calyptra</ets> + <ets>-form</ets>.]</ety> <def>Having the form a calyptra, or extinguisher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calystegia</ent><br/
<hw>Calystegia</hw> <pos>n.</pos> <def>a genus of climbing or scrambling herbs; bindweed.</def><br/
<syn><b>Syn. --</b> genus <gen>Calystegia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Calyx</ent><br/
<hw>Ca"lyx</hw> <pr>(k<amac/"l<icr/ks; 277)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Calyxes</plw> <pr>(k<amac/"l<icr/ks*<ecr/z)</pr>, L. <plw>Calyces</plw> <pr>(k<acr/l"<icr/*s<emac/z)</pr>.</plu> <ety>[L. <ets>calyx</ets>, <ets>-ycis</ets>, fr. Gr. <grk>ka`lyx</grk> husk, shell, calyx, from the root of <grk>kaly`ptein</grk> to cover, conceal. Cf. <er>Chalice Helmet</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The covering of a flower. See <er>Flower</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>calyx</xex> is usually green and foliaceous, but becomes delicate and petaloid in such flowers as the anemone and the four-o'clock. Each leaf of the calyx is called a <xex>sepal</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A cuplike division of the pelvis of the kidney, which surrounds one or more of the renal papill<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Calzoons</ent><br/
<hw>Cal*zoons"</hw> <pr>(k<acr/l*z<oomac/nz")</pr>, <pos>n. pl.</pos> <ety>[F. <ets>cale<cced/ons</ets> (cf. It. <ets>calzoni</ets> breeches), fr. L. <ets>calceus</ets> shoe.]</ety> <def>Drawers.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cam</ent><br/
<hw>Cam</hw> <pr>(k<acr/m)</pr>, <pos>n.</pos> <ety>[Dan. <ets>kam</ets> comb, ridge; or cf. W., Gael., and Ir., <ets>cam</ets> bent. See 1st <er>Comb</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>A turning or sliding piece which, by the shape of its periphery or face, or a groove in its surface, imparts variable or intermittent motion to, or receives such motion from, a rod, lever, or block brought into sliding or rolling contact with it.</def> <sd>(b)</sd> <def>A curved wedge, movable about an axis, used for forcing or clamping two pieces together.</def> <sd>(c)</sd> <def>A projecting part of a wheel or other moving piece so shaped as to give alternate or variable motion to another piece against which it acts.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 207 --></p>

<p><note><hand/ <ex>Cams</ex> are much used in machinery involving complicated, and irregular movements, as in the sewing machine, pin machine, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A ridge or mound of earth.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cam wheel</b></col> <fld>(Mach.)</fld>, <cd>a wheel with one or more projections (cams) or depressions upon its periphery or upon its face; one which is set or shaped eccentrically, so that its revolutions impart a varied, reciprocating, or intermittent motion.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cam</ent><br/
<hw>Cam</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Kam</er>.]</ety> <def>Crooked.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>camachile</ent><br/
<hw>camachile</hw> <pos>n.</pos> <def>a common thorny tropical American tree (<spn>Pithecellobium dulce</spn>) having terminal racemes of yellow flowers followed by sickle-shaped or circinate edible pods and yielding good timber and a yellow dye and mucilaginous gum.</def><br/
<syn><b>Syn. --</b> manila tamarind, huamachil, wild tamarind.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camaieu</ent><br/
<hw>Ca*ma"ieu</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.; of unknown origin. Cf. <er>Cameo</er>.]</ety> <sn>1.</sn> <def>A cameo.</def> <mark>[Obs.]</mark>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fine Arts)</fld> <def>Painting in shades of one color; monochrome.</def>  <rj><au>Mollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camail</ent><br/
<hw>Ca*mail"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>camail</ets> (cf. It. <ets>camaglio</ets>), fr. L. <ets>caput</ets> head + source of E. <ets>mail</ets>.]</ety> <sn>1.</sn> <fld>(Ancient Armor)</fld> <def>A neck guard of chain mall, hanging from the bascinet or other headpiece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hood of other material than mail;</def> <specif>esp.</specif> <fld>(Eccl.)</fld>, <def>a hood worn in church services, -- the amice, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camara dos Deputados</ent><br/
<ent>Camara dos Pares</ent><br/
<ent>Camara</ent><br/
||<hw>Ca"ma*ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg.]</ety> <def>Chamber; house; -- used in <hw>Ca"ma*ra dos Pa"res</hw> <pr>(<?/)</pr>, and <hw>Ca"ma*ra dos De`pu*ta"dos</hw> <pr>(<?/)</pr>. See <er>Legislature</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Camaraderie</ent><br/
||<hw>Ca`ma`ra`de*rie"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Comrade</er>.]</ety> <def>Comradeship and loyalty.</def></p>

<p><q>The spirit of <qex>camaraderie</qex> is strong among these riders of the plains.</q>  <rj><qau>W. A. Fraser.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Camarasaurus</ent><br/
||<hw>Cam`a*ra*sau"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. fr. Gr. <?/ a vaulted chamber + <?/ lizard.]</ety> <fld>(Paleon.)</fld> <def>A genus of gigantic American Jurassic dinosaurs, having large cavities in the bodies of the dorsal vertebr<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camarilla</ent><br/
||<hw>Ca`ma*ril"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., a small room.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The private audience chamber of a king.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A company of secret and irresponsible advisers, as of a king; a cabal or clique.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>camas</ent><br/
<hw>camas</hw> <pos>n.</pos> <def>same as <er>camass</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>camash</ent><br/
<hw>camash</hw> <pos>n.</pos> <def>same as <er>camass</er>.</def><br/
<syn><b>Syn. --</b> camas, camass, quamash, camosh.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camassia</ent><br/
<hw>Camassia</hw> <pos>n.</pos> <def>a genus of scapose herbs of North and South America having large edible bulbs.</def><br/
<syn><b>Syn. --</b> genus <gen>Camassia</gen>, Quamassia, genus <gen>Quamassia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camass</ent><br/
<hw>Cam"ass</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[American Indian name.]</ety> <fld>(Bot.)</fld> <def>any of several plants of the genus Camassia of North and South America, especially the blue-flowered liliaceous plant (<spn>Camassia esculenta</spn>) of northwestern America, the bulbs of which were collected for food by the Indians.</def> <altsp>[Written also <asp>camas</asp>, <asp>cammas</asp>, and <asp>quamash</asp>.]</altsp><br/
<syn><b>Syn. --</b> camass, quamash, camosh, camash.</syn><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <stype>Eastern cammass</stype> is <styp><spn>Camassia Fraseri</spn></styp>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camass</ent><br/
<hw>Cam"ass</hw> <pr>(?)</pr>. <pos>n.</pos> <ety>[Origin uncert.]</ety> <def>A small prairie in a forest; a small grassy plain among hills.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Camber</ent><br/
<hw>Cam"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of. <ets>cambre</ets> bent, curved; akin to F. <ets>cambrer</ets> to vault, to bend, fr. L. <ets>camerare</ets> to arch over, fr. <ets>camera</ets> vault, arch. See <er>Chamber</er>, and cf. <er>Camerate</er>.]</ety> <sn>1.</sn> <fld>(Shipbuilding)</fld> <def>An upward convexity of a deck or other surface; <as>as, she has a high <ex>camber</ex> (said of a vessel having an unusual convexity of deck)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An upward concavity in the under side of a beam, girder, or lintel; also, a slight upward concavity in a straight arch. See <er>Hogback</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camber arch</b></col> <fld>(Arch.)</fld>, <cd>an arch whose intrados, though apparently straight, has a slightly concave curve upward.</cd> -- <col><b>Camber beam</b></col> <fld>(Arch.)</fld>, <cd>a beam whose under side has a concave curve upward.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camber</ent><br/
<hw>Cam"ber</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cambered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cambering</conjf>.]</vmorph> <def>To cut bend to an upward curve; to construct, as a deck, with an upward curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camber</ent><br/
<hw>Cam"ber</hw>, <pos>v. i.</pos> <def>To curve upward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camberkeeled</ent><br/
<hw>Cam"ber*keeled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Having the keel arched upwards, but not actually hogged; -- said of a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambial</ent><br/
<hw>Cam"bi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>cambialis</ets>, fr. <ets>cambiars</ets>. See <er>Change</er>.]</ety> <def>Belonging to exchanges in commerce; of exchange.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambist</ent><br/
<hw>Cam"bist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cambiste</ets>, It. <ets>cambista</ets>, fr. L. <ets>cambire</ets> to exchange. See <er>Change</er>.]</ety> <def>A banker; a money changer or broker; one who deals in bills of exchange, or who is skilled in the science of exchange.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambistry</ent><br/
<hw>Cam"bist*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science of exchange, weight, measures, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambium</ent><br/
<hw>Cam"bi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>cambium</ets> exchange, fr. L. <ets>cambire</ets> to exchange. It was supposed that cambium was sap changing into wood.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A series of formative cells lying outside of the wood proper and inside of the inner bark. The growth of new wood takes place in the cambium, which is very soft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A fancied nutritive juice, formerly supposed to originate in the blood, to repair losses of the system, and to promote its increase.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camblet</ent><br/
<hw>Cam"blet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Camlet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camboge</ent><br/
<hw>Cam*boge"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Gamboge</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camboose</ent><br/
<hw>Cam*boose"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Caboose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambrasine</ent><br/
<hw>Cam"bra*sine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of linen cloth made in Egypt, and so named from its resemblance to cambric.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambrel</ent><br/
<hw>Cam"brel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Gambrel</er>, <pos>n.</pos>, 2.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambria</ent><br/
<hw>Cam"bri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The ancient Latin name of Wales. It is used by modern poets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambrian</ent><br/
<hw>Cam"bri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Geog.)</fld> <def>Of or pertaining to Cambria or Wales.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>Of or pertaining to the lowest subdivision of the rocks of the Silurian or Molluscan age; -- sometimes described as inferior to the Silurian. It is named from its development in <xex>Cambria</xex> or Wales. See the Diagram under <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambrian</ent><br/
<hw>Cam"bri*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>A native of Cambria or Wales.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>The Cambrian formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cambric</ent><br/
<hw>cam"bric</hw> <pr>(k<acr/m"br<icr/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>camerike</ets>, fr. <ets>Cambrai</ets> (Flemish <ets>Kamerik</ets>), a city of France (formerly of Flanders), where it was first made.]</ety> <sn>1.</sn> <def>A fine, thin, and white fabric made of flax or linen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He hath ribbons of all the colors i' the rainbow; . . . inkles, caddises, <qex>cambrics</qex>, lawns.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fabric made, in imitation of linen cambric, of fine, hardspun cotton, often with figures of various colors; -- also called <altname>cotton cambric</altname>, and <altname>cambric muslin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cambro-Briton</ent><br/
<hw>Cam"bro-Brit"on</hw> <pr>(k<acr/m"br<osl/*br<icr/t"<ucr/n)</pr>, <pos>n.</pos> <def>A Welshman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Came</ent><br/
<hw>Came</hw> <pr>(k<amac/m)</pr>, <def><pos>imp.</pos> of <er>Come</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>came</ent><br/
<hw>came</hw> <pr>(k<amac/m)</pr>, <pos>n.</pos> <ety>[Cf. Scot. <ets>came</ets>, <ets>caim</ets>, comb, and OE. <ets>camet</ets> silver.]</ety> <def>A slender rod of cast lead, with or without grooves, used, in casements and stained-glass windows, to hold together the panes or pieces of glass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>camel</ent><br/
<hw>cam"el</hw> <pr>(k<acr/m"<ecr/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>camel</ets>, <ets>chamel</ets>, OF. <ets>camel</ets>, <ets>chamel</ets>, F. <ets>chameau</ets> L. <ets>camelus</ets>, fr. Gr. <grk>ka`mhlos</grk>; of Semitic origin; cf. Heb. <ets>g<amac/m<amac/l</ets>, Ar. <ets>jamal</ets>. Cf. As. <ets>camel</ets>, fr. L. <ets>camelus</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A large ruminant used in Asia and Africa for carrying burdens and for riding.  The camel is remarkable for its ability to go a long time without drinking.  Its hoofs are small, and situated at the extremities of the toes, and the weight of the animal rests on the callous. The dromedary (<spn>Camelus dromedarius</spn>) has one hump on the back, while the Bactrian camel (<spn>Camelus Bactrianus</spn>) has two.  The llama, alpaca, and vicu<ntil/a, of South America, belong to a related genus (<gen>Auchenia</gen>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A water-tight structure (as a large box or boxes) used to assist a vessel in passing over a shoal or bar or in navigating shallow water. By admitting water, the camel or camels may be sunk and attached beneath or at the sides of a vessel, and when the water is pumped out the vessel is lifted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camel bird</b></col> <fld>(Zool.)</fld>, <cd>the ostrich.</cd> -- <col><b>Camel locust</b></col> <fld>(Zool.)</fld>, <cd>the mantis.</cd> -- <col><b>Camel's thorn</b></col> <fld>(Bot.)</fld>, <cd>a low, leguminous shrub (<spn>Alhagi maurorum</spn>) of the Arabian desert, from which exudes a sweetish gum, which is one of the substances called <xex>manna</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camel-backed</ent><br/
<hw>Cam"el-backed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a back like a camel; humpbacked.</def>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cameleon</ent><br/
<hw>Ca*me"le*on</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chaceleon</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>camelhair</ent><br/
<hw>cam"el*hair</hw> <pos>n.</pos> <def>a soft tan cloth made with the hair of a camel.</def><br/
<syn><b>Syn. --</b> camel's hair.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>camelia</ent><br/
<hw>camelia</hw> <pos>n.</pos> <def>any of several shrubs or small evergreen trees having solitary white or pink or reddish flowers; the <altname>camellia</altname>.</def><br/
<syn><b>Syn. --</b> camellia.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camelidae</ent><br/
<hw>Camelidae</hw> <pos>n.</pos> <def>a natural family comprising the camels and llamas and vicunas.</def><br/
<syn><b>Syn. --</b> family <fam>Camelidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camelina</ent><br/
<hw>Camelina</hw> <pos>n.</pos> <def>a genus of annual and biennial herbs of Mediterranean to Central Asia.</def><br/
<syn><b>Syn. --</b> genus <gen>Camelina</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camellia</ent><br/
<hw>Ca*mel"li*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., after <person>Georg Josef <etsep>Kamel</etsep></person>, or <etsep>Camelli</etsep>, a Jesuit who is said to have brought it from the East.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An Asiatic genus of small shrubs, often with shining leaves and showy flowers. <spn>Camellia Japonica</spn> is much cultivated for ornament, and <spn>Camellia Sassanqua</spn> and <spn>Camellia oleifera</spn> are grown in China for the oil which is pressed from their seeds. The tea plant is now referred to this genus under the name of <spn>Camellia Thea</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Hort.)</fld> <def>An ornamental greenhouse shrub (<spn>Thea japonica</spn>) with glossy evergreen leaves and roselike red or white double flowers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Camelopard</ent><br/
<hw>Ca*mel"o*pard</hw> <pr>(k<adot/*m<ecr/l"<osl/*p<aum/rd <it>or</it> k<acr/m"<ecr/l*<osl/*p<aum/rd; 277)</pr>, <pos>n.</pos> <ety>[LL. <ets>camelopardus</ets>, L. <ets>camelopardalus</ets>, <ets>camelopardalis</ets>, fr. Gr. <grk>kamhlopa`rdalis</grk>; <grk>ka`mhlos</grk> a camel + <grk>pa`rdalis</grk> pard, leopard: cf. F. <ets>cam<eacute/lopard</ets>. The camelopard has a neck and head like a camel, and is spotted like a pard. See <er>Camel</er>, and <er>Pard</er>.]</ety> <fld>(Zool.)</fld> <def>An African ruminant; the <altname>giraffe</altname>.  See <er>Giraffe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camelot</ent><br/
<hw>Came"lot</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Camelet</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camelot</ent><br/
<hw>Cam"e*lot</hw> <pr>(k<acr/m"<ucr/*l<ocr/t)</pr>, <pos>prop. n.</pos> <sn>1.</sn>  <def>The legendary site of <person>King Arthur's</person> court and castle.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Hence, any place or time thought of as having idyllic peace, justice, and happiness.</def><br/
[<source>PJC</source>]</p>

<p><ent>Camelry</ent><br/
<hw>Cam"el*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Troops that are mounted on camels.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Camelshair</ent><br/
<hw>Cam"els*hair`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of camel's hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camel's-hair pencil</b></col>, <cd>a small brush used by painters in water colors, made of camel's hair or similar materials.</cd> -- <col><b>Camel's-hair shawl</b></col>. <cd>A name often given to a <altname>cashmere shawl</altname>. See <cref>Cashmere shawl</cref> under <er>Cashmere</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camembert cheese</ent><br/
<ent>Camembert</ent><br/
<mhw>||<hw>Ca`mem`bert"</hw> <pr>(?)</pr>, <pos>n.</pos>, <it>or</it> <hw>Camembert cheese</hw></mhw>. <def>A kind of rich, soft, unpressed cream cheese made in the vicinity of Camembert, near Argentan, France; also, any cheese of the same type, wherever made.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cameo</ent><br/
<hw>Cam"e*o</hw> <pr>(k<acr/m"<esl/*<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cameos</plw> <pr>(k<acr/m"<esl/*<omac/z)</pr>.</plu> <ety>[It. <ets>cammeo</ets>; akin to F. <ets>cam<eacute/e</ets>, <ets>cama<ium/eu</ets>, Sp. <ets>camafeo</ets>, LL. <ets>camaeus</ets>, <ets>camahutus</ets>; of unknown origin.]</ety> <def>A carving in relief, esp. one on a small scale used as a jewel for personal adornment, or like.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ Most cameos are carved in a material which has layers of different colors, such stones as the onyx and sardonyx, and various kinds of shells, being used.  The classical <ex>cameos</ex> made in Italy are carved on a seashell (see <cref>cameo conch</cref>, below), having an olive figure carved from the inner layer of the shell in relief on the white background of the outer layer of the shell.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cameo conch</b></col> <fld>(Zool.)</fld>, <cd>a large, marine, univalve shell, esp. <spn>Cassis cameo</spn>, <spn>Cassis rua</spn>, and allied species, used for cutting cameos. See <er>Quern conch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camera</ent><br/
<hw>Cam"e*ra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Cameras</plw> <pr>(#)</pr>, L. <plw>Camerae</plw> <pr>(#)</pr>.</plu> <ety>[L. vault, arch, LL., chamber. See <er>Chamber</er>.]</ety> <def>A chamber, or instrument having a chamber.  Specifically: The <cref>camera obscura</cref> when used in photography. See <er>Camera</er>, and <er>Camera obscura</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bellows camera</b></col>. <cd>See under <er>Bellows</er>.</cd> -- <col><b>In camera</b></col> <fld>(Law)</fld>, <cd>in a judge's chamber, that is, privately; <as>as, a judge hears testimony which is not fit for the open court <ex>in camera</ex></as>.</cd> -- <mcol><col><b>Panoramic camera</b></col>, <it>or</it> <col><b>Pantascopic camera</b></col></mcol>, <cd>a photographic camera in which the lens and sensitized plate revolve so as to expose adjacent parts of the plate successively to the light, which reaches it through a narrow vertical slit; -- used in photographing broad landscapes.</cd>  <rj><au>Abney.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camerade</ent><br/
<hw>Came"rade</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Comrade</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cameralistic</ent><br/
<hw>Cam`e*ra*lis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to finance and public revenue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cameralistics</ent><br/
<hw>Cam`e*ra*lis"tics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cam<eacute/ralistique</ets>, G. <ets>kameralistik</ets>, fr. L. <ets>camera</ets> vault, LL., chamber, treasury.]</ety> <def>The science of finance or public revenue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camera lucida</ent><br/
||<hw>Cam"e*ra lu"ci*da</hw> <pr>(?)</pr>. <ety>[L. <ets>camera</ets> chamber + L. <ets>lucidus</ets>, <ets>lucida</ets>, lucid, light.]</ety> <fld>(Opt.)</fld> <def>An instrument which by means of a prism of a peculiar form, or an arrangement of mirrors, causes an apparent image of an external object or objects to appear as if projected upon a plane surface, as of paper or canvas, so that the outlines may conveniently traced. It is generally used with the microscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camera obscura</ent><br/
||<hw>Cam"e*ra ob*scu"ra</hw> <pr>(?)</pr>. <ety>[LL. <ets>camera</ets> chamber + L. <ets>obscurus</ets>, <ets>obscura</ets>, dark.]</ety> <fld>(Opt.)</fld> <sn>1.</sn> <def>An apparatus in which the images of external objects, formed by a convex lens or a concave mirror, are thrown on a paper or other white surface placed in the focus of the lens or mirror within a darkened chamber, or box, so that the outlines may be traced.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Photog.)</fld> <def>An apparatus in which the image of an external object or objects is, by means of lenses, thrown upon a sensitized plate or surface placed at the back of an extensible darkened box or chamber variously modified; -- commonly called simply <altname>the camera</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camerate</ent><br/
<hw>Cam"er*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Camerated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Camerzting</conjf>.]</vmorph> <ety>[L. <ets>cameratus</ets>, p. p. of <ets>camerare</ets>. See <er>Camber</er>.]</ety> <sn>1.</sn> <def>To build in the form of a vault; to arch over.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To divide into chambers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cameration</ent><br/
<hw>Cam`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cameratio</ets>.]</ety> <def>A vaulting or arching over.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camerlingo</ent><br/
||<hw>Ca`mer*lin"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>The papal chamberlain; the cardinal who presides over the pope's household. He has at times possessed great power.</def> <altsp>[Written also <asp>camerlengo</asp> and <asp>camarlengo</asp>.]</altsp></p>

<p><ent>Cameronian</ent><br/
<hw>Cam`e*ro"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A follower of the Rev. Richard <etsep>Cameron</etsep>, a Scotch Covenanter of the time of Charles II.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>Cameron and others refused to accept the <ldquo/indulgence<rdquo/ offered the Presbyterian clergy, insisted on the Solemn league and Covenant, and in 1680 declared Charles II. deposed for tyranny, breach of faith, etc.  Cameron was killed at the battle of Airdmoss, but his followers became a denomination (afterwards called Reformed Presbyterians) who refused to recognize laws or institutions which they believed contrary to the kingdom of Christ, but who now avail themselves of political rights.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camis</ent><br/
<hw>Cam"is</hw> <pr>(k<acr/m"<icr/s)</pr>, <pos>n.</pos> <ety>[See <er>Chemise</er>.]</ety> <def>A light, loose dress or robe.</def> <altsp>[Also written <asp>camus</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All in a <qex>camis</qex> light of purple silk.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camisado</ent><br/
<ent>Camisade</ent><br/
<mhw>{ <hw>Cam`i*sade"</hw> <pr>(?)</pr>, <hw>Cam`i*sa"do</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>camisade</ets> a night attack; cf. It. <ets>camiciata</ets>. See <er>Camis</er>.]</ety> <mark>[Obs.]</mark> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A shirt worn by soldiers over their uniform, in order to be able to recognize one another in a night attack.</def> <sd>(b)</sd> <def>An attack by surprise by soldiers wearing the camisado.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give them a <qex>camisado</qex> in night season.</q> <rj><qau>Holinshed.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camisard</ent><br/
||<hw>Cam"i*sard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>One of the French Protestant insurgents who rebelled against Louis XIV, after the revocation of the edict of Nates; -- so called from the peasant's smock (<xex>camise</xex>) which they wore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camisated</ent><br/
<hw>Cam"i*sa`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Dressed with a shirt over the other garments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camisole</ent><br/
||<hw>Cam"i*sole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>chemise</er>.]</ety> <sn>1.</sn> <def>A short dressing jacket for women.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of straitjacket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camlet</ent><br/
<hw>Cam"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>camelot</ets> (akin to Sp. <ets>camelote</ets>, <ets>chamelote</ets>, It. <ets>cambellbito</ets>, <ets>ciambellotto</ets>, LL. <ets>camelotum</ets>, <ets>camelinum</ets>, fr. Ar. <ets>khamlat</ets> camlet, fr. <ets>kaml</ets> pile, plush. The word was early confused with <xex>camel</xex>, camel's hair also being used in making it. Cf. <er>Calamanco</er>]</ety> <def>A woven fabric originally made of camel's hair, now chiefly of goat's hair and silk, or of wool and cotton.</def> <altsp>[Sometimes written <asp>camelot</asp> and <asp>camblet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They have been made plain and twilled, of single warp and weft, of double warp, and sometimes with double weft also, with thicker yarn.</note>  <rj><au>Beck (Draper's Dict. )</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camleted</ent><br/
<hw>Cam"let*ed</hw>, <pos>a.</pos> <def>Wavy or undulating like camlet; veined.</def>  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cammas</ent><br/
<hw>Cam"mas</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Camass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cammock</ent><br/
<hw>Cam"mock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cammoc</ets>.]</ety> <fld>(Bot.)</fld> <def>A plant having long hard, crooked roots, the <spn>Ononis spinosa</spn>; -- called also <altname>rest-harrow</altname>. The <spn>Scandix Pecten-Veneris</spn> is also called <ex>cammock</ex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamomile</ent><br/
<ent>Camomile</ent><br/
<mhw>{ <hw>Cam"o*mile</hw>, <hw>Cham"o*mile</hw>  }</mhw> <pr>(k<acr/m"<osl/*m<imac/l)</pr>, <pos>n.</pos> <ety>[LL. <ets>camonilla</ets>, corrupted fr. Gr. <grk>chamai`mhlon</grk>, lit. earth apple, being so called from the smell of its flower. See <er>Humble</er>, and <er>Melon</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of herbs (<gen>Anthemis</gen>) of the Composite family.  The common camomile, <spn>Anthemis nobilis</spn>, is used as a popular remedy.  Its flowers have a strong and fragrant and a bitter, aromatic taste.  They are tonic, febrifugal, and in large doses emetic, and the volatile oil is carminative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camonflet</ent><br/
||<hw>Ca*mon"flet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Mil.)</fld> <def>A small mine, sometimes formed in the wall or side of an enemy's gallery, to blow in the earth and cut off the retreat of the miners.</def>  <rj><au>Farrow.</au></rj></p>

<p><ent>Camorra</ent><br/
||<hw>Ca*mor"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A secret organization formed at Naples, Italy, early in the 19th century, and used partly for political ends and partly for practicing extortion, violence, etc.</def> -- <wordforms><wf>Ca*mor"rist</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Camoys</ent><br/
<ent>Camous</ent><br/
<mhw>{ <hw>Ca"mous</hw> <pr>(?)</pr>, <hw>Ca"moys</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[F. <ets>camus</ets> (equiv. to <ets>camard</ets>) flat-nosed, fr. Celtic <ets>Cam</ets> croked + suff. <ets>-us</ets>; akin to L. <ets>camur</ets>, <ets>camurus</ets>, croked.]</ety> <def>Flat; depressed; crooked; -- said only of the nose.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camoused</ent><br/
<hw>Ca"moused</hw>, <pr>(<?/)</pr>, <pos>a.</pos> <ety>[From <er>Camouse</er>]</ety> <def>Depressed; flattened.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though my nose be <qex>cammoused</qex>.</q> <rj><qau>B. Jonson</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camously</ent><br/
<hw>Ca"mous*ly</hw>, <pos>adv.</pos> <def>Awry.</def> <mark>[Obs.]</mark>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camp</ent><br/
<hw>Camp</hw> <pr>(k<acr/mp)</pr>, <pos>n.</pos> <ety>[F. <ets>camp</ets>, It. <ets>campo</ets>, fr. L. <ets>campus</ets> plant, field; akin to Gr. <grk>kh^pos</grk> garden. Cf. <er>Campaign</er>, <er>Champ</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The ground or spot on which tents, huts, etc., are erected for shelter, as for an army or for lumbermen, etc.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A collection of tents, huts, etc., for shelter, commonly arranged in an orderly manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Forming a <qex>camp</qex> in the neighborhood of Boston.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A single hut or shelter; <as>as, a hunter's <ex>camp</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The company or body of persons encamped, as of soldiers, of surveyors, of lumbermen, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>camp</qex> broke up with the confusion of a flight.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Agric.)</fld> <def>A mound of earth in which potatoes and other vegetables are stored for protection against frost; -- called also <altname>burrow</altname> and <altname>pie</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[Cf. OE. & AS. <ets>camp</ets> contest, battle. See <er>champion</er>.]</ety> <def>An ancient game of football, played in some parts of England.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camp bedstead</b></col>, <cd>a light bedstead that can be folded up onto a small space for easy transportation.</cd> -- <col><b>camp ceiling</b></col> <fld>(Arch.)</fld>, <cd>a kind ceiling often used in attics or garrets, in which the side walls are inclined inward at the top, following the slope of the rafters, to meet the plane surface of the upper ceiling.</cd> -- <col><b>Camp chair</b></col>, <cd>a light chair that can be folded up compactly for easy transportation; the seat and back are often made of strips or pieces of carpet.</cd> -- <col><b>Camp fever</b></col>, <cd>typhus fever.</cd> -- <col><b>Camp follower</b></col>, <cd>a civilian accompanying an army, as a sutler, servant, etc.</cd> -- <col><b>Camp meeting</b></col>, <cd>a religious gathering for open-air preaching, held in some retired spot, chiefly by Methodists. It usually last for several days, during which those present lodge in tents, temporary houses, or cottages.</cd> -- <col><b>Camp stool</b></col>, <cd>the same as <cref>camp chair</cref>, except that the stool has no back.</cd> -- <col><b>Flying camp</b></col> <fld>(Mil.)</fld>, <cd>a camp or body of troops formed for rapid motion from one place to another.</cd> <au>Farrow.</au> -- <col><b>To pitch (a) camp</b></col>, <cd>to set up the tents or huts of a camp.</cd> -- <col><b>To strike camp</b></col>, <cd>to take down the tents or huts of a camp.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 208 --></p>

<p><ent>Camp</ent><br/
<hw>Camp</hw> <pr>(k<acr/mp)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Camped</conjf> <pr>(k<acr/mt; 215)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Camping</conjf>.]</vmorph> <def>To afford rest or lodging for, as an army or travelers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Had our great palace the capacity<br/
To <qex>camp</qex> this host, we all would sup together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camp</ent><br/
<hw>Camp</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To pitch or prepare a camp; to encamp; to lodge in a camp; -- often with <ptcl>out</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>camped</qex> out at night, under the stars.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[See <er>Camp</er>, <pos>n.</pos>, 6]</ety> <def>To play the game called camp.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campagna</ent><br/
<hw>Cam*pa"gna</hw> <pr>(c<adot/m*p<aum/"ny<adot/)</pr>, <pos>n.</pos> <ety>[It. See <er>Campaign</er>.]</ety> <def>An open level tract of country; especially <ldquo/<xex>Campagna</xex> di Roma.<rdquo/ The extensive undulating plain which surrounds Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Its length is commonly stated to be about ninety miles, and its breadth from twenty-seven to forty miles. The ground is almost entirely volcanic, and vapors which arise from the district produce malaria.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campagnol</ent><br/
||<hw>Cam`pa`gnol"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. , fr. <ets>campagne</ets> field.]</ety> <fld>(Zool.)</fld> <def>A mouse (<spn>Arvicala agrestis</spn>), called also <altname>meadow mouse</altname>, which often does great damage in fields and gardens, by feeding on roots and seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campaign</ent><br/
<hw>Cam*paign"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>campagne</ets>, It. <ets>campagna</ets>, fr. L. <ets>Campania</ets> the level country about Naples, fr. <ets>campus</ets> field. See <er>Camp</er>, and cf. <er>Champaign</er>, <er>Champagne</er>.]</ety> <sn>1.</sn> <def>An open field; a large, open plain without considerable hills. See<er>Champaign</er>.</def>  <rj><au>Grath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A connected series of military operations forming a distinct stage in a war; the time during which an army keeps the field.</def>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Political operations preceding an election, by candidates, their assistants, and supporters, for the purpose of convincing voters to vote for the candidate.  It usually consists of one or more methods of contacting voters including advertising, distribution or mailing of printed leaflets or letters; speeches, interviews with news media, and door-to-door visits with potential voters.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <specif>Hence:</specif> <def>Any coordinated effort to contact potential supporters or customers and solicit their support or patronage; <as>as, an advertising <ex>campaign</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Metal.)</fld> <def>The period during which a blast furnace is continuously in operation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campaign</ent><br/
<hw>Cam*paign"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To serve in a campaign.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campaigner</ent><br/
<hw>Cam*paign"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who has served in an army in several campaigns; an old soldier; a veteran.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>one who is campaigning, especially a politician running for elective office, or one of his/her supporters.</def><br/
[<source>PJC</source>]</p>

<p><ent>campaigning</ent><br/
<hw>campaigning</hw> <pos>n.</pos> <def>The campaign of a candidate to be elected.</def><br/
<syn><b>Syn. --</b> candidacy, candidature, electioneering, political campaign.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Campana</ent><br/
<hw>Cam*pa"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>campana</ets> bell. Cf. <er>Campanle</er>.]</ety> <sn>1.</sn> <fld>(Eccl.)</fld> <def>A church bell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The pasque flower.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Doric Arch.)</fld> <def>Same as <er>Gutta</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campaned</ent><br/
<hw>Cam*paned"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Furnished with, or bearing, campanes, or bells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanero</ent><br/
||<hw>Cam`pa*ne"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., a bellman.]</ety> <fld>(Zool.)</fld> <def>The bellbird of South America. See <er>Bellbird</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanes</ent><br/
<hw>Cam*panes"</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[See <ets>Campana</ets>.]</ety> <fld>(Her.)</fld> <def>Bells.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campania</ent><br/
||<hw>Cam*pa"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Campaig</er>.]</ety> <def>Open country.</def>  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campaniform</ent><br/
<hw>Cam*pan"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>campana</ets> bell + <ets>-form</ets>: cf. F. <ets>companiforme</ets>.]</ety> <def>Bell-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanile</ent><br/
||<hw>Cam`pa*ni"le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>campanile</ets> bell tower, steeple, fr. It. & LL. <ets>campana</ets> bell.]</ety> <fld>(Arch.)</fld> <def>A bell tower, esp. one built separate from a church.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many of the <qex>campaniles</qex> of Italy are lofty and magnificent structures.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campaniliform</ent><br/
<hw>Cam`pa*nil"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Campaniform</er>.]</ety> <def>Bell-shaped; campanulate; campaniform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanologist</ent><br/
<hw>Cam`pa*nol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in campanology; a bell ringer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanology</ent><br/
<hw>Cam`pa*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>campana</ets> bell + <ets>-logy</ets>.]</ety> <def>The art of ringing bells, or a treatise on the art.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanula</ent><br/
||<hw>Cam*pan"u*la</hw> <pr>(k<acr/m*p<acr/n"<usl/*l<adot/)</pr>, <pos>prop. n.</pos> <ety>[LL. <ets>campanula</ets> a little bell; dim. of <ets>campana</ets> bell.]</ety> <fld>(Bot.)</fld> <def>A large genus of plants bearing bell-shaped flowers, often of great beauty; -- also called <altname>bellflower</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanulaceae</ent><br/
<hw>Campanulaceae</hw> <pos>prop. n.</pos> <def>A natural family of flowering plants, which in some classifications includes the <fam>Lobeliaciae</fam>.</def><br/
<syn><b>Syn. --</b> family <fam>Campanulaceae</fam>, bellflower family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Campanulaceous</ent><br/
<hw>Cam*pan`u*la"ceous</hw> <pr>(k<acr/m*p<acr/n`<usl/*l<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of pertaining to, or resembling, the family of plants (<fam>Campanulace<ae/</fam>) of which Campanula is the type, and which includes the Canterbury bell, the harebell, and the Venus's looking-glass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanulales</ent><br/
<hw>Campanulales</hw> <pos>n.</pos> <def>an order of plants which includes the <fam>Campanulaceae</fam>; <fam>Lobeliaceae</fam>; <fam>Cucurbitaceae</fam>; <fam>Goodeniaceae</fam>; <fam>Stylidiaceae</fam>; <fam>Calyceraceae</fam>; and <fam>Compositae</fam>.</def><br/
<syn><b>Syn. --</b> order <ord>Campanulales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>campanular</ent><br/
<hw>cam*pan"u*lar</hw> <pr>(k<acr/m*p<acr/n"<usl/*l<etil/r)</pr>, <pos>adj.</pos>  <def>shaped like a bell or campana; campaniform.</def><br/
<syn><b>Syn. --</b> campanulate, campanulated, campaniform, campaniliform.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Campanularian</ent><br/
<hw>Cam*pan`u*la"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>campanula</ets> a bell.]</ety> <fld>(Zool.)</fld> <def>A hydroid of the family <fam>Campanularid<ae/</fam>, characterized by having the polyps or zooids inclosed in bell-shaped calicles or hydrothec<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campanulate</ent><br/
<hw>Cam*pan"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Bell-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campbellite</ent><br/
<hw>Camp"bell*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <person>Alexander <etsep>Campbell</etsep></person>, of Virginia.]</ety> <fld>(Eccl.)</fld> <def>A member of the denomination called <org>Christians or Disciples of Christ</org>.  They themselves repudiate the term <ex>Campbellite</ex> as a nickname.  See <er>Christian</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campeachy Wood</ent><br/
<hw>Cam*peach"y Wood`</hw> <pr>(?)</pr>. <ety>[From the bay of <etsep>Campeachy</etsep>, in Mexico.]</ety> <def>Logwood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camper</ent><br/
<hw>Camp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who lodges temporarily in a hut or camp, especially who sleeps in a wilderness for recreation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who stays at a summer camp or day camp.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>A vehicle, such as a small truck, or a pickup truck with a hood over the back, equipped for convenience while camping out.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>happy camper</b></col> <cd>a person who is pleased with the situation in which s/he finds him/herself.  Often used ironically or in understatement, especially in the negative; <as>as, the passengers left behind on the island were not a bunch of <ex>happy campers</ex></as>.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Campestrian</ent><br/
<ent>Campestral</ent><br/
<mhw>{ <hw>Cam*pes"tral</hw> <pr>(?)</pr>, <hw>Cam*pes"tri*an</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>campester</ets>, fr. <ets>campus</ets> field.]</ety> <def>Relating to an open field; growing in a field, or open ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campfight</ent><br/
<hw>Camp"fight`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Camp</er>, <pos>n.</pos>, 6.]</ety> <fld>(O. Eng. Law.)</fld> <def>A duel; the decision of a case by a duel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphene</ent><br/
<hw>Cam"phene</hw> <pr>(k<acr/m"f<emac/n <it>or</it> k<acr/m*f<emac/n")</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>One of a series of substances <chform>C10H16</chform>, resembling camphor, regarded as modified terpenes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphine</ent><br/
<hw>Cam*phine"</hw> <pr>(k<acr/m*f<emac/n" <it>or</it> k<acr/m"f<icr/n)</pr>, <pos>n.</pos> <ety>[From <er>Camphor</er>.]</ety> <def>Rectified oil of turpentine, used for burning in lamps, and as a common solvent in varnishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is also applied to a mixture of this substance with three times its volume of alcohol and sometimes a little ether, used as an illuminant.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphire</ent><br/
<hw>Cam"phire</hw> <pr>(k<acr/m"f<imac/r)</pr>, <pos>n.</pos> <def>An old spelling of <er>Camphor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphogen</ent><br/
<hw>Cam"pho*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Camphor</ets> + <ets>-gen</ets>: -- formerly so called as derived from camphor: cf. F. <ets>camphog<egrave/ne</ets>.]</ety> <fld>(Chem.)</fld> <def>See <er>Cymene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphol</ent><br/
<hw>Cam"phol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Camphor</ets> + <ets>-ol</ets>.]</ety> <fld>(Chem.)</fld> <def>See <er>Borneol</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphor</ent><br/
<hw>Cam"phor</hw> <pr>(k<acr/m"f<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>camfere</ets>, F. <ets>camphre</ets> (cf. It. <ets>canfora</ets>, Sp. <ets>camfora</ets>, <ets>alcanfor</ets>, LL. <ets>canfora</ets>, <ets>camphora</ets>, NGr. <grk>kafoyra`</grk>), fr. Ar. <ets>k<amac/f<umac/r</ets>, prob. fr. Skr. <ets>karp<umac/ra</ets>.]</ety> <sn>1.</sn> <def>A tough, white, aromatic resin, or gum, obtained from different species of the <fam>Laurus</fam> family, esp. from <spn>Cinnamomum camphara</spn> (the <spn>Laurus camphora</spn> of Linn<ae/us.). Camphor, <chform>C10H16O</chform>, is volatile and fragrant, and is used in medicine as a diaphoretic, a stimulant, or sedative.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>originally, a gum resembling ordinary camphor, obtained from a tree (<spn>Dryobalanops aromatica</spn> formerly <spn>Dryobalanops camphora</spn>) growing in Sumatra and Borneo; now applied to its main constituent, a terpene alcohol obtainable as a white solid <chform>C10H18O</chform>, called also <altname>Borneo camphor</altname>, <altname>Malay camphor</altname>, <altname>Malayan camphor</altname>, <altname>camphor of Borneo</altname>, <altname>Sumatra camphor</altname>, <altname>bornyl alcohol</altname>, <altname>camphol</altname>, and <altname>borneol</altname>.  The isomer from <gen>Dryobalanops</gen> is dextrorotatory; the levoratatory form is obtainable from other species of plants, and the racemic mixture may be obtained by reduction of camphor.  It is used in perfumery, and for manufacture of its esters. See <er>Borneol</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ The name <xex>camphor</xex> is also applied to a number of bodies of similar appearance and properties, as <stype>cedar camphor</stype>, obtained from the red or pencil cedar (<spn>Juniperus Virginiana</spn>), and <stype>peppermint camphor</stype>, or <stype>menthol</stype>, obtained from the oil of peppermint.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camphor oil</b></col> <fld>(Chem.)</fld>, <cd>name variously given to certain oil-like products, obtained especially from the camphor tree.</cd> -- <col><b>Camphor tree</b></col>, <cd>a large evergreen tree (<spn>Cinnamomum Camphora</spn>) with lax, smooth branches and shining triple-nerved lanceolate leaves, probably native in China, but now cultivated in most warm countries. Camphor is collected by a process of steaming the chips of the wood and subliming the product.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphor</ent><br/
<hw>Cam"phor</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To impregnate or wash with camphor; to camphorate.</def> <mark>[R.]</mark>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphoraceous</ent><br/
<hw>Cam`pho*ra"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of camphor; containing camphor.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphorate</ent><br/
<hw>Cam"phor*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To impregnate or treat with camphor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphorate</ent><br/
<hw>Cam"phor*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>camphorate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of camphoric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphorated</ent><br/
<ent>Camphorate</ent><br/
<mhw>{ <hw>Cam"phor*ate</hw> <pr>(?)</pr>, <hw>Cam"phor*a`ted</hw> <pr>(?)</pr>, }</mhw> <def>Combined or impregnated with camphor.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camphorated oil</b></col>, <cd>an oleaginous preparation containing camphor, much used as an embrocation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphoric</ent><br/
<hw>Cam*phor"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>camphorique</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, camphor.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camphoric acid</b></col>, <cd>a white crystallizable substance, <chform>C10H16O4</chform>, obtained from the oxidation of camphor.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Other acids of camphor are <xex>campholic acid</xex>, <chform>C10H18O2</chform>, and <xex>camphoronic acid</xex>, <chform>C9H12O5</chform>, white crystallizable substances.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camphretic</ent><br/
<hw>Cam*phret"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[rom <er>Camphor</er>.]</ety> <def>Pertaining to, or derived from camphor.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camping</ent><br/
<hw>Camp"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Lodging in a camp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[See <er>Camp</er>, <pos>n.</pos>, 6]</ety> <def>A game of football.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campion</ent><br/
<hw>Cam"pi*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. fr. L. <ets>campus</ets> field.]</ety> <fld>(Bot.)</fld> <def>A plant of the Pink family (<spn>Cucubalus bacciferus</spn>), bearing berries regarded as poisonous.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bladder campion</b></col>, <cd>a plant of the Pink family (<spn>Cucubalus Behen</spn> or <spn>Silene inflata</spn>), having a much inflated calyx. See <er>Behen</er>.</cd> -- <col><b>Rose campion</b></col>, <cd>a garden plant (<spn>Lychnis coronaria</spn>) with handsome crimson flowers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>campmate</ent><br/
<hw>camp"mate`</hw> <pos>n.</pos> <def>someone who lives in the same camp as another.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>campong</ent><br/
<hw>campong</hw> <pos>n.</pos> <def>a native village in Malaysia.</def><br/
<syn><b>Syn. --</b> Kampong.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camponotus</ent><br/
<hw>Camponotus</hw> <pos>n.</pos> <def>a genus comprising the carpenter ants.</def><br/
<syn><b>Syn. --</b> genus <gen>Camponotus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>campstool</ent><br/
<hw>camp"stool`</hw> <pos>n.</pos> <def>a folding stool.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camptosorus</ent><br/
<hw>Camptosorus</hw> <pos>n.</pos> <def>a classification used in some esp. former systems for plants usually placed in the genus <gen>Asplenium</gen>.</def><br/
<syn><b>Syn. --</b> genus <gen>Camptosorus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Campus</ent><br/
||<hw>Cam"pus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a field.]</ety> <sn>1.</sn> <def>The principal grounds of a college or school, between the buildings or within the main inclosure; <as>as, the college <ex>campus</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a college or university.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>a division of a university with its own buildings and a separate faculty, especially one separated geographically from other divisiona, but sharing top administration with other units of the university; <as>as, the Newark <ex>campus</ex> of Rutgers</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>higher education considered as a whole; <as>as, the financial effects of research cutbacks on the <ex>campus</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a business site with pleasant landscaping; <as>as, the Squibb research <ex>campus</ex> at Princeton</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Campyloneurum</ent><br/
<hw>Campyloneurum</hw> <pos>n.</pos> <def>a genus of epiphytic ferns of tropical America.</def><br/
<syn><b>Syn. --</b> genus <gen>Campyloneurum</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Campylorhynchus</ent><br/
<hw>Campylorhynchus</hw> <pos>n.</pos> <def>an alternative genus classification for the cactus wrens.</def><br/
<syn><b>Syn. --</b> genus <gen>Campylorhynchus</gen>, Heleodytes, genus <gen>Heleodytes</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Campylospermous</ent><br/
<hw>Cam`py*lo*sper"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ curved + <?/ seed.]</ety> <fld>(Bot.)</fld> <def>Having seeds grooved lengthwise on the inner face, as in sweet cicely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Campylotropous</ent><br/
<hw>Cam`py*lot"ro*pous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ curved + <?/ a turning.]</ety> <fld>(Bot.)</fld> <def>Having the ovules and seeds so curved, or bent down upon themselves, that the ends of the embryo are brought close together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>camshaft</ent><br/
<hw>camshaft</hw> <pos>n.</pos> <def>a shaft that has cams attached to it, forming part of a mechanical device.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Camus</ent><br/
<hw>Cam"us</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Camis</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Camwood</ent><br/
<hw>Cam"wood</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Barwood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Can</ent><br/
<hw>Can</hw> <pr>(k<acr/n)</pr>, <def>an <mark>obs.</mark> form of <ets>began</ets>, imp. & p. p. of <er>Begin</er>, sometimes used in old poetry. <note>[See <er>Gan</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><q>With gentle words he <qex>can</qex> faile gree.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Can</ent><br/
<hw>Can</hw>, <pos>n.</pos> <ety>[OE. & AS. <ets>canne</ets>; akin to D. <ets>Kan</ets>, G. <ets>Kanne</ets>, OHG. <ets>channa</ets>, Sw. <ets>Kanna</ets>, Dan. <ets>kande</ets>.]</ety> <sn>1.</sn> <def>A drinking cup; a vessel for holding liquids.</def>  <rj><au>[Shak. ]</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fill the cup and fill <qex>can</qex>,<br/
Have a rouse before the morn.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel or case of tinned iron or of sheet metal, of various forms, but usually cylindrical; <as>as, a <ex>can</ex> of tomatoes; an oil <ex>can</ex>; a milk <ex>can</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>can</xex> may be a cylinder open at the top, as for receiving the sliver from a carding machine, or with a removable cover or stopper, as for holding tea, spices, milk, oysters, etc., or with handle and spout, as for holding oil, or hermetically sealed, in canning meats, fruits, etc. The name is also sometimes given to the small glass or earthenware jar used in canning.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Can</ent><br/
<hw>Can</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Canned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Canning</conjf>.]</vmorph> <def>To preserve by putting in sealed cans</def> <mark>[U. S.]</mark> <ldquo/<xex>Canned</xex> meats<rdquo/  <rj><au>W. D. Howells.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canned goods</b></col>, <cd>a general name for fruit, vegetables, meat, or fish, preserved in hermetically sealed cans.</cd></cs></p>
<-- ## canned laughter = recorded , canned (response) = prepared, rehearsed, standardized; lacking originality -->

<p><ent>Can</ent><br/
<hw>Can</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <note>[The transitive use is obsolete.]</note> <vmorph>[<pos>imp.</pos> <conjf>Could</conjf> <pr>(#)</pr>.]</vmorph> <ety>[OE. <ets>cunnen</ets>, <ets>cannen</ets> (1st sing. pres. <ets>I can</ets>), to know, know how, be able, AS. <ets>cunnan</ets>, 1st sing. pres. <ets>ic cann</ets> or <ets>can</ets>, pl. <ets>cunnon</ets>, 1st sing. imp. <ets>c<umac/<edh/e</ets> (for <ets>cun<edh/e</ets>); p. p. <ets>c<umac/<edh/</ets> (for <ets>cun<edh/</ets>); akin to OS. <ets>Kunnan</ets>, D. <ets>Kunnen</ets>, OHG. <ets>chunnan</ets>, G. <ets>k<oum/nnen</ets>, Icel. <ets>kunna</ets>, Goth. <ets>Kunnan</ets>, and E. <ets>ken</ets> to know. The present tense <ets>I can</ets> (AS. <ets>ic cann</ets>) was originally a preterit, meaning <ets>I have known</ets> or <ets>Learned</ets>, and hence <ets>I know</ets>, <ets>know how</ets>. <root/45. See <er>Ken</er>, <er>Know</er>; cf. <er>Con</er>, <er>Cunning</er>, <er>Uncouth</er>.]</ety>  <sn>1.</sn> <def>To know; to understand.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>can</qex> rimes of Robin Hood.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>can</qex> no Latin, quod she.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the priest in surplice white,<br/
That defunctive music <qex>can</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be able to do; to have power or influence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The will of Him who all things <qex>can</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For what, alas, <qex>can</qex> these my single arms?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>M<ae/c<ae/nas and Agrippa, who <qex>can</qex> most with C<ae/sar.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be able; -- followed by an infinitive without <xex>to</xex>; <as>as, I <ex>can</ex> go, but do not wish to</as>.</def></p>

<p><syn><b>Syn.</b> -- <er>Can but</er>, <er>Can not but</er>. It is an error to use the former of these phrases where the sens requires the latter. If we say, <ldquo/I <xex>can but</xex> perish if I go,<rdquo/ <ldquo/But<rdquo/ means <xex>only</xex>, and denotes that this is all or the worst that can happen. When the apostle Peter said. <ldquo/We <xex>can not but</xex> speak of the things which we have seen and heard.<rdquo/ he referred to a moral constraint or necessety which rested upon him and his associates; and the meaning was, We cannot help speaking, We cannot refrain from speaking. This idea of a moral necessity or constraint is of frequent occurrence, and is also expressed in the phrase, <ldquo/I <xex>can not</xex> help it.<rdquo/ Thus we say. <ldquo/I <xex>can not but</xex> hope,<rdquo/ <ldquo/I <xex>can not but</xex> believe,<rdquo/ <ldquo/I <xex>can not but</xex> think,<rdquo/ <ldquo/I <xex>can not but</xex> remark,<rdquo/ etc., in cases in which it would be an error to use the phrase <xex>can but</xex>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet he <qex>could not but</qex> acknowledge to himself that there was something calculated to impress awe, . . . in the sudden appearances and vanishings . . . of the masque</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Tom felt that this was a rebuff for him, and <qex>could not but</qex> understand it as a left-handed hit at his employer.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canaan</ent><br/
<hw>Canaan</hw> <pos>n.</pos> <def>an ancient country is southwest Asia on the eastern coast of the Mediterranean.</def><br/
<syn><b>Syn. --</b> Palestine, Holy Land.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canaanite</ent><br/
<hw>Ca"naan*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A descendant of Canaan, the son of Ham, and grandson of Noah.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A Native or inhabitant of the land of Canaan, esp. a member of any of the tribes who inhabited Canaan at the time of the exodus of the Israelites from Egypt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canaanite</ent><br/
<hw>Ca"naan*ite</hw>, <pos>n.</pos> <ety>[From an Aramaic word signifying <ldquo/zeal.<rdquo/]</ety> <def>A zealot.</def> <ldquo/Simon the <xex>Canaanite</xex>.<rdquo/  <rj><au>Matt. x. 4.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This was the <ldquo/Simon called Zelotes<rdquo/ (<au>Luke vi. 15</au>), <it>i.e.</it>, Simon the zealot.</note>  <rj><au>Kitto.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canaanitish</ent><br/
<hw>Ca"naan*i`tish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Canaan or the Canaanites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canada</ent><br/
<ent>Canyada</ent><br/
||<hw>Ca*<ntil/a"da</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <sn>1.</sn> <def>A small ca<ntil/on; a narrow valley or glen; also, but less frequently, an open valley.</def> <mark>[Local, Western U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dry riverbed.</def><br/
[<source>PJC</source>]</p>

<p><ent>Canachites</ent><br/
<hw>Canachites</hw> <pos>prop. n.</pos> <def>A genus of birds including the spruce grouse <spn>Canachites canadensis</spn>.</def><br/
<syn><b>Syn. --</b> genus <gen>Canachites</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canada</ent><br/
<hw>Can"a*da</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A country in North America, bordering the United States on the north.  It is a federation which includes English-speaking provinces and the French-speaking Province of Quebec.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Canada balsam</b></col>. <cd>See under <er>Balsam</er>.</cd> -- <col><b>Canada goose</b></col>. <fld>(Zool.)</fld> <cd>See <er>Wild goose</er>.</cd> -- <col><b>Canada jay</b></col>. <cd>See <er>Whisky Jack</er>.</cd> -- <col><b>Canada lynx</b></col>. <fld>(Zool.)</fld> <cd>See <er>Lynx</er>.</cd> -- <col><b>Canada lily</b></col>. <fld>(Bot.)</fld> <cd>a plant of eastern North America (<spn>Lilium canadense</spn>) having yellow or orange flowers with dark spots; called also <altname>meadow lily</altname>.</cd> <au>RHUD</au> -- <col><b>Canada porcupine</b></col> <fld>(Zool.)</fld> <cd>See <er>Porcupine</er>, and <er>Urson</er>.</cd> -- <col><b>Canada rice</b></col> <fld>(Bot.)</fld> <cd>See under <er>Rick</er>.</cd> -- <col><b>Canada robin</b></col> <fld>(Zool.)</fld>, <cd>the cedar bird.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canadian</ent><br/
<hw>Ca*na"di*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Canada.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Canada.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canadian period</b></col> <fld>(Geol.)</fld>, <cd>A subdivision of the American Lower Silurian system embracing the calciferous, Quebec, and Chazy epochs. This period immediately follows the primordial or Cambrian period, and is by many geologists regarded as the beginning of the Silurian age, See the Diagram, under <er>Geology</er>.</cd> -- <col><b>Canadian goose</b></col>, an erroneous variant of <er>Canada goose</er>.</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>canafistula</ent><br/
<ent>canafistola</ent><br/
<mhw><hw>canafistola</hw>, <hw>canafistula</hw></mhw> <pos>n.</pos> <def>a deciduous or semi-evergreen tree (<spn>Cassia fistula</spn>) having scented sepia to yellow flowers in drooping racemes and pods whose pulp is used medicinally; it grows in tropical Asia, Central and South America, and Australia.</def><br/
<syn><b>Syn. --</b> golden shower tree, drumstick tree, purging cassia, pudding pipe tree, canafistola.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canaille</ent><br/
<hw>Ca*naille"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>canaille</ets> (cf. It. <ets>canaglia</ets>), prop. and orig. a pack of dogs, fr. L. <ets>Canis</ets> dog.]</ety> <sn>1.</sn> <def>The lowest class of people; the rabble; the vulgar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shorts or inferior flour.</def> <mark>[Canadian]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canakin</ent><br/
<hw>Can"a*kin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>can</ets>.]</ety> <def>A little can or cup.</def> <ldquo/And let me the <xex>canakin</xex> clink.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canal</ent><br/
<hw>Ca*nal"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>canal</ets>, from L. <ets>canalis</ets> canal, channel; prob. from a root signifying <ldquo/to cut<rdquo/; cf. D. <ets>kanaal</ets>, fr. the French. Cf. <er>Channel</er>, <er>Kennel</er> gutter.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An artificial channel filled with water and designed for navigation, or for irrigating land, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A tube or duct; <as>as, the alimentary <ex>canal</ex>; the semicircular <ex>canals</ex> of the ear</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A long and relatively narrow arm of the sea, approximately uniform in width; -- used chiefly in proper names; <as>as, Portland <ex>Canal</ex>; Lynn <ex>Canal</ex></as>.</def> <mark>[Alaska]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Canal boat</b></col>, <cd>a boat for use on a canal; esp. one of peculiar shape, carrying freight, and drawn by horses walking on the towpath beside the canal.</cd> -- <col><b>Canal lock</b></col>. <cd>See <er>Lock</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canal coal</ent><br/
<hw>Can"al coal`</hw> <pr>(?)</pr>. <def>See <er>Cannel coal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canaliculated</ent><br/
<ent>Canaliculate</ent><br/
<mhw>{ <hw>Can`a*lic"u*late</hw> <pr>(?)</pr>, <hw>Can`a*lic"u*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>canaliculatus</ets> channeled, fr. <ets>canaliculus</ets>, dim. of <ets>canalis</ets>. See <er>Canal</er>.]</ety> <def>Having a channel or groove, as in the leafstalks of most palms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canaliculus</ent><br/
||<hw>Can`a*lic"u*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canaliculi</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Anat.)</fld> <def>A minute canal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canalization</ent><br/
<hw>Ca*nal`i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Construction of, or furnishing with, a canal or canals.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>canalize</ent><br/
<hw>canalize</hw> <pos>v.</pos> <sn>1.</sn> <def>to provide with a canal, as of a city.</def><br/
<syn><b>Syn. --</b> canal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to to direct the flow of; -- also used abstractly, as of money or information.</def><br/
<syn><b>Syn. --</b> channel.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canape</ent><br/
||<hw>Ca`na`p<eacute/"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., orig. a couch with mosquito curtains. See <er>Canopy</er>.]</ety> <sn>1.</sn>  <def>A sofa or divan.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Cookery)</fld> <def>an appetizer consisting of a thin slice or piece of bread toasted or fried in butter or oil, on which anchovies, mushrooms, caviar, cheese, or other savory foods, are served.</def><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Canape confident</ent><br/
||<hw>Ca`na`p<eacute/" con`fi`dent"</hw> <pr>(?)</pr>. <def>A sofa having a seat at each end at right angles to the main seats.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Canard</ent><br/
<hw>Ca*nard"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., properly, a duck.]</ety> <def>An extravagant or absurd report or story; a fabricated sensational report or statement; esp. one set afloat in the newspapers to hoax the public.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canarese</ent><br/
<hw>Can`a*rese"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to Canara, a district of British India.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canary</ent><br/
<hw>Ca*na"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>Canarie</ets>, L. <ets>Canaria insula</ets> one of the Canary islands, said to be so called from its large dogs, fr. <ets>canis</ets> dog.]</ety> <sn>1.</sn> <def>Of or pertaining to the Canary Islands; <as>as, <ex>canary</ex> wine; <ex>canary</ex> birds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of a pale yellowish color; <as>as, <ex>Canary</ex> stone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canary grass</b></col>, <cd>a grass of the genus <gen>Phalaris</gen> (<spn>Phalaris Canariensis</spn>), producing the seed used as food for canary birds.</cd> -- <col><b>Canary stone</b></col> <fld>(Min.)</fld>, <cd>a yellow species of carnelian, named from its resemblance in color to the plumage of the canary bird.</cd> -- <col><b>Canary wood</b></col>, <cd>the beautiful wood of the trees <spn>Persea Indica</spn> and <spn>Persea  Canariensis</spn>, natives of Madeira and the Canary Islands.</cd> -- <col><b>Canary vine</b></col>. <cd>See <cref>Canary bird flower</cref>, under <er>Canary bird</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canary</ent><br/
<hw>Ca*na"ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canaries</plw> <pr>(#)</pr>.</plu> <sn>1.</sn> <def>Wine made in the Canary Islands; sack.</def> <ldquo/A cup of <xex>canary</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A canary bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A pale yellow color, like that of a canary bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A quick and lively dance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Make you dance <qex>canary</qex><br/
With sprightly fire and motion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canary</ent><br/
<hw>Ca*na"ry</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To perform the canary dance; to move nimbly; to caper.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But to jig of a tune at the tongue's end, <qex>canary</qex> to it with your feet.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canary bird</ent><br/
<hw>Ca*na"ry bird`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A small singing bird of the Finch family (<spn>Serinus Canarius</spn>), a native of the Canary Islands. It was brought to Europe in the 16th century, and made a household pet. It generally has a yellowish body with the wings and tail greenish, but in its wild state it is more frequently of gray or brown color. It is sometimes called <altname>canary finch</altname>.</def><-- and <altname>canary.</altname> --><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 209 --></p>

<p><cs><col><b>Canary bird flower</b></col> <fld>(Bot.)</fld>, <cd>a climbing plant (<spn>Trop<ae/olum peregrinum</spn>) with canary-colored flowers of peculiar form; -- called also <altname>canary vine</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>canasta</ent><br/
<hw>canasta</hw> <pos>n.</pos> <def>a form of rummy using two decks and four jokers; jokers and deuces are wild; the object is to meld groups of seven of the same rank.</def><br/
<syn><b>Syn. --</b> basket rummy, meld.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canaster</ent><br/
<hw>Ca*nas"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>canasta</ets>, <ets>canastro</ets>, basket, fr. L. <ets>canistrum</ets>. See <er>Canister</er>.]</ety> <def>A kind of tobacco for smoking, made of the dried leaves, coarsely broken; -- so called from the rush baskets in which it is packed in South America.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canavalia</ent><br/
<hw>Canavalia</hw> <pos>n.</pos> <def>a genus of herbs or woody vines of mainly American tropics and subtropics.</def><br/
<syn><b>Syn. --</b> genus <gen>Canavalia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>canavanine</ent><br/
<hw>canavanine</hw> <pos>n.</pos> <def>An amino acid found in the jack bean.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canberra</ent><br/
<hw>Canberra</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>Australia</country>.  Population (2000) = 307,700.</def><br/
[<source>PJC</source>]</p>

<p><ent>Can buoy</ent><br/
<hw>Can" buoy`</hw> <pr>(?)</pr>. <def>See under <er>Buoy</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancan</ent><br/
||<hw>Can"can</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A rollicking French dance, accompanied by indecorous or extravagant postures and gestures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancel</ent><br/
<hw>Can"cel</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Canceled</conjf> <it>or</it> <conjf>Cancelled</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Canceling</conjf> <it>or</it> <conjf>Cancelling</conjf>.]</vmorph> <ety>[L. <ets>cancellare</ets> to make like a lattice, to strike or cross out (cf. Fr. <ets>canceller</ets>, OF. <ets>canceler</ets>) fr. <ets>cancelli</ets> lattice, crossbars, dim. of <ets>cancer</ets> lattice; cf. Gr. <?/ latticed gate. Cf. <er>Chancel</er>.]</ety> <sn>1.</sn> <def>To inclose or surround, as with a railing, or with latticework.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A little obscure place <qex>canceled</qex> in with iron work is the pillar or stump at which . . . our Savior was scourged.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shut out, as with a railing or with latticework; to exclude.</def> <mark>[Obs.]</mark> <ldquo/<xex>Canceled</xex> from heaven.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cross and deface, as the lines of a writing, or as a word or figure; to mark out by a cross line; to blot out or obliterate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A deed may be avoided by delivering it up to be <qex>cancelled</qex>; that is, to have lines drawn over it in the form of latticework or <qex>cancelli</qex>; though the phrase is now used figuratively for any manner of obliterating or defacing it.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To annul or destroy; to revoke or recall.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The indentures were <qex>canceled</qex>.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was unwilling to <qex>cancel</qex> the interest created through former secret services, by being refractory on this occasion.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <def>To suppress or omit; to strike out, as matter in type.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canceled figures</b></col> <fld>(Print)</fld>, <cd>figures cast with a line across the face., as for use in arithmetics.</cd></cs></p>

<p><syn><b>Syn.</b> -- To blot out; obliterate; deface; erase; efface; expunge; annul; abolish; revoke; abrogate; repeal; destroy; do away; set aside. See <er>Abolish</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancel</ent><br/
<hw>Can"cel</hw>, <pos>n.</pos> <ety>[See <er>Cancel</er>, <pos>v. i.</pos>, and cf. <er>Chancel</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An inclosure; a boundary; a limit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A prison is but a retirement, and opportunity of serious thoughts, to a person whose spirit . . . desires no enlargement beyond the <qex>cancels</qex> of the body.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print)</fld> <sd>(a)</sd> <def>The suppression or striking out of matter in type, or of a printed page or pages.</def> <sd>(b)</sd> <def>The part thus suppressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancelier</ent><br/
<hw>Can`cel*ier"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[F. <ets>chanceler</ets>, OF. <ets>canseler</ets>, to waver, orig. to cross the legs so as not to fall; from the same word as E. <ets>cancel</ets>.]</ety> <fld>(Falconry)</fld> <def>To turn in flight; -- said of a hawk.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He makes his stoop; but wanting breath, is forced<br/
To <qex>cancelier</qex>.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canceleer</ent><br/
<ent>Cancelier</ent><br/
<mhw>{ <hw>Can`cel*ier"</hw> <pr>(?)</pr>, <hw>Can"cel*eer</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <fld>(Falconry)</fld> <def>The turn of a hawk upon the wing to recover herself, when she misses her aim in the stoop.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The fierce and eager hawks, down thrilling from the skies,<br/
Make sundry <qex>canceliers</qex> ere they the fowl can reach.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancellarean</ent><br/
<hw>Can`cel*la"re*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cancellarean.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancellate</ent><br/
<hw>Can"cel*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cancellatus</ets>, p. p. of <ets>cancellare</ets>, See <er>Cancel</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Consisting of a network of veins, without intermediate parenchyma, as the leaves of certain plants; latticelike.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having the surface coveres with raised lines, crossing at right angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancellated</ent><br/
<hw>Can"cel*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Crossbarred; marked with cross lines.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Open or spongy, as some porous bones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancellation</ent><br/
<hw>Can`cel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cancellatio</ets>: cf. F. <ets>cancellation</ets>.]</ety> <sn>1.</sn> <def>The act, process, or result of canceling; <as>as, the <ex>cansellation</ex> of certain words in a contract, or of the contract itself</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>The operation of striking out common factors, in both the dividend and divisor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancelli</ent><br/
||<hw>Can*cel"li</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L., a lattice. See <er>Cancel</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>An interwoven or latticed wall or inclosure; latticework, rails, or crossbars, as around the bar of a court of justice, between the chancel and the nave of a church, or in a window.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>The interlacing osseous plates constituting the elastic porous tissue of certain parts of the bones, esp. in their articular extremities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancellous</ent><br/
<hw>Can"cel*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>cancellosus</ets> covered with bars.]</ety> <fld>(Anat.)</fld> <def>Having a spongy or porous structure; made up of cancelli; cancellated; <as>as, the <ex>cancellous</ex> texture of parts of many bones</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancer</ent><br/
<hw>Can"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cancer</ets>, <ets>cancri</ets>, crab, ulcer, a sign of the zodiac; akin to Gr. <grk>karki`nos</grk>, Skr. <ets>karka<tsdot/a</ets> crab, and prob. Skr. <ets>karkara</ets> hard, the crab being named from its hard shell. Cf. <er>Canner</er>, <er>Chancre</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of decapod Crustacea, including some of the most common shore crabs of Europe and North America, as the rock crab, Jonah crab, etc. See <er>Crab</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The fourth of the twelve signs of the zodiac. The first point is the northern limit of the sun's course in summer; hence, the sign of the summer solstice. See <er>Tropic</er>.</def> <sd>(b)</sd> <def>A northern constellation between Gemini and Leo.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Formerly, any malignant growth, esp. one attended with great pain and ulceration, with cachexia and progressive emaciation. It was so called, perhaps, from the great veins which surround it, compared by the ancients to the claws of a crab. The term is now restricted to such a growth made up of aggregations of epithelial cells, either without support or embedded in the meshes of a trabecular framework.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Four kinds of cancers are recognized: (1) <stype><colf>Epithelial cancer</colf>, <it>or</it> <colf>Epithelioma</colf></stype>, in which there is no trabecular framework. See <er>Epithelioma</er>. (2) <stype><colf>Scirrhous cancer</colf>, <it>or</it> <colf>Hard cancer</colf></stype>, in which the framework predominates, and the tumor is of hard consistence and slow growth. (3) <stype><colf>Encephaloid  cancer</colf></stype>, <stype><colf>Medullary  cancer</colf></stype>, <it>or</it> <stype><colf>Soft cancer</colf></stype>, in which the cellular element predominates, and the tumor is soft, grows rapidy, and often ulcerates. (4) <stype><colf>Colloid cancer</colf></stype>, in which the cancerous structure becomes gelatinous. The last three varieties are also called <stype>carcinoma</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cancer cells</b></col>, <cd>cells once believed to be peculiar to cancers, but now know to be epithelial cells differing in no respect from those found elsewhere in the body, and distinguished only by peculiarity of location and grouping.</cd> -- <col><b>Cancer root</b></col> <fld>(Bot.)</fld>, <cd>the name of several low plants, mostly parasitic on roots, as the beech drops, the squawroot, etc.</cd> -- <col><b>Tropic of Cancer</b></col>. <cd>See <er>Tropic</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancerate</ent><br/
<hw>Can"cer*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cancerated</conjf>.]</vmorph> <ety>[LL. <ets>canceratus</ets> eaten by a cancer. See <er>Cancer</er>.]</ety> <def>To grow into a cancer; to become cancerous.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canceration</ent><br/
<hw>Can`cer*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or state of becoming cancerous or growing into a cancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancerite</ent><br/
<hw>Can"cer*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>canc<eacute/reux</ets>.]</ety> <def>Like a cancer; having the qualities or virulence of a cancer; affected with cancer.</def> <ldquo/<xex>Cancerous</xex> vices.<rdquo/  <rj><au>G. Eliot.</au></rj></p>

<p><ent>cancerous</ent><br/
<hw>can"cer*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>canc<eacute/reux</ets>]</ety> <def>Like a cancer; having the qualities or virulence of a cancer; affected with cancer; <as>as, a <ex>cancerous</ex> growth</as>.</def>   <ldquo/<ex>cancerous</ex> vices<rdquo/ <au>G. Eliot.</au><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Can"cer*ous*ly</wf>, <pos>adv.</pos> -- <wf>Can"cer*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>cancerweed</ent><br/
<hw>cancerweed</hw> <pos>n.</pos> <def>a sage of eastern U. S. (<spn>Salvia lyrata</spn>).</def><br/
<syn><b>Syn. --</b> cancer weed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cancridae</ent><br/
<hw>Cancridae</hw> <pos>n.</pos> <def>a natural family including many of the best known edible crabs.</def><br/
<syn><b>Syn. --</b> family <fam>Cancridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cancriform</ent><br/
<hw>Can"cri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cancer</ets> + <ets>-form</ets>; cf. F. <ets>cancriforme</ets>.]</ety> <sn>1.</sn> <def>Having the form of, or resembling, a crab; crab-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a cancer; cancerous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancrine</ent><br/
<hw>Can"crine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cancer</er>.]</ety> <def>Having the qualities of a crab; crablike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancrinite</ent><br/
<hw>Can"cri*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after Count <ets>Cancrin</ets>, a minister of finance in Russia.]</ety> <fld>(Min.)</fld> <def>A mineral occurring in hexagonal crystals, also massive, generally of a yellow color, containing silica, alumina, lime, soda, and carbon dioxide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cancroid</ent><br/
<hw>Can"croid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cancer</ets> + <ets>oid</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Resembling a crab; pertaining to the <fam>Cancroidea</fam>, one of the families of crabs, including the genus <gen>Cancer</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a cancer; <as>as, a <ex>cancroid</ex> tumor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cand</ent><br/
<hw>Cand</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Fluor spar. See <er>Kand</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>candela</ent><br/
<hw>candela</hw> <pos>n.</pos> <def>the basic unit of luminous intensity adopted under the System International d'Unites.</def><br/
<syn><b>Syn. --</b> candle, cd, standard candle.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>candelabra</ent><br/
<hw>candelabra</hw> <pos>n.</pos> <sn>1.</sn> <def>a branched, ornamental candlestick having several sockets for candles.</def><br/
<syn><b>Syn. --</b> candelabrum.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a branched, ornamental electric light fixture, resembling an candelabra[wn1], having several sockets for lights.</def><br/
<syn><b>Syn. --</b> candelabrum.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candelabrum</ent><br/
<hw>Can`de*la"brum</hw> <pr>(?)</pr> <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Candelabra</plw> <pr>(#)</pr>, E. <plw>Candelabrums</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>candela</ets> candle. See <er>candle</er>.]</ety> <sn>1.</sn> <fld>(Antiq.)</fld> <sd>(a)</sd> <def>A lamp stand of any sort.</def> <sd>(b)</sd> <def>A highly ornamented stand of marble or other ponderous material, usually having three feet, -- frequently a votive offering to a temple.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large candlestick, having several branches; also called <altname>candelabra</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>candellila</ent><br/
<hw>candellila</hw> <pos>n.</pos> <sn>1.</sn> <def>either of two Mexican shrubs <spn>Pedilanthus bracteatus</spn> or <spn>Pedilanthus pavonis</spn>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a wax-coated shrub (<spn>Euphorbia antisyphilitica</spn>) of northern Mexico and the southwestern U.S.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candent</ent><br/
<hw>Can`dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>candens</ets>, p. pr. of <ets>cand<eum/re</ets> to glitter. See <er>Candid</er>.]</ety> <def>Heated to whiteness; glowing with heat.</def> <ldquo/A <xex>candent</xex> vessel.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canderos</ent><br/
||<hw>Can"de*ros</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An East Indian resin, of a pellucid white color, from which small ornaments and toys are sometimes made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candescence</ent><br/
<hw>Can*des"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Incandescence</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candescent</ent><br/
<hw>Can*des"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>candescens</ets>, <ets>-entis</ets>, p. pr. of <ets>candescere</ets>, v. incho. fr. <ets>candere</ets> to shine.]</ety> <def>Glowing; luminous; incandescent.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Candicant</ent><br/
<hw>Can"di*cant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>candicans</ets>, p. pr. of <ets>candicare</ets> to be whitish.]</ety> <def>Growing white.</def> <mark>[Obs.]</mark>
<-- #sic.  glowing white? or becoming white? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candid</ent><br/
<hw>Can*did</hw> <pr>(k<acr/n"d<icr/d)</pr>, <pos>a.</pos> <ety>[F. <ets>candide</ets> (cf. It. <ets>candido</ets>), L. <ets>candidus</ets> white, fr. <ets>cand<emac/re</ets> to be of a glowing white; akin to ac<ets>cend<ecr/re</ets>, in<ets>cend<ecr/re</ets>, to set on fire, Skr. <ets>chand</ets> to shine. Cf. <er>Candle</er>, <er>Incense</er>.]</ety> <sn>1.</sn> <def>White.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The box receives all black; but poured from thence,<br/
The stones came <qex>candid</qex> forth, the hue of innocence.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Free from undue bias; disposed to think and judge according to truth and justice, or without partiality or prejudice; fair; just; impartial; <as>as, a <ex>candid</ex> opinion</as>.</def> <ldquo/<xex>Candid</xex> and dispassionate men.<rdquo/  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Open; frank; ingenuous; outspoken.</def></p>

<p><syn><b>Syn.</b> -- Fair; open; ingenuous; impartial; just; frank; artless; unbiased; equitable.</syn> <usage> -- <er>Candid</er>, <er>Fair</er>, <er>Open</er>, <er>Frank</er>, <er>Ingenuous</er>. A man is <xex>fair</xex> when he puts things on a just or equitable footing; he is <xex>candid</xex> when be looks impartially on both sides of a subject, doing justice especially to the motives and conduct of an opponent; he is <xex>open</xex> and <xex>frank</xex> when he declares his sentiments without reserve; he is <xex>ingenuous</xex> when he does this from a noble regard for truth. <xex>Fair</xex> dealing; <xex>candid</xex> investigation; an <xex>open</xex> temper; a <xex>frank</xex> disposition; an <xex>ingenuous</xex> answer or declaration.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>candida</ent><br/
<hw>candida</hw> <pos>n.</pos> <sn>1.</sn> <def>any of the yeastlike imperfect fungi of the genus <gen>Candida</gen>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candidacy</ent><br/
<hw>Can"di*da*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The position of a candidate; state of being a candidate; candidateship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candidate</ent><br/
<hw>Can"di*date</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Candidatus</ets>, n. (because candidates for office in Rome were clothed in a white toga.) fr. <ets>candidatus</ets> clothed in white, fr. <ets>candidus</ets>littering, white: cf. F. <ets>candidat</ets>.]</ety> <def>One who offers himself, or is put forward by others, as a suitable person or an aspirant or contestant for an office, privilege, or honor; <as>as, a <ex>candidate</ex> for the office of governor; a <ex>candidate</ex> for holy orders; a <ex>candidate</ex> for scholastic honors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candidateship</ent><br/
<hw>Can"di*date*ship</hw>, <pos>n.</pos> <def>Candidacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candidating</ent><br/
<hw>Can"di*da`ting</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The taking of the position of a candidate; specifically, the preaching of a clergyman with a view to settlement.</def> <mark>[Cant, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candidature</ent><br/
<hw>Can"di*da*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Candidacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>candidiasis</ent><br/
<hw>candidiasis</hw> <pos>n.</pos> <def>an infection caused by fungi of the genera <gen>Monilia</gen> or <gen>Candida</gen>, especially <spn>Candida albicans</spn>.</def><br/
<syn><b>Syn. --</b> moniliasis, monilia disease.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candidly</ent><br/
<hw>Can"did*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a candid manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candidness</ent><br/
<hw>Can"did*ness</hw>, <pos>n.</pos> <def>The quality of being candid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candied</ent><br/
<hw>Can"died</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From 1st <er>Candy</er>.]</ety> <sn>1.</sn> <def>Preserved in or with sugar; incrusted with a candylike substance; <as>as, <ex>candied</ex> fruits</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>Converted wholly or partially into sugar or candy; as <xex>candied</xex> sirup.</def> <sd>(b)</sd> <def>Conted or more or less with sugar; <as>as, <ex>candidied</ex> raisins</as></def>. <sd>(c)</sd> <def>Figuratively; Honeyed; sweet; flattering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the <qex>candied</qex> tongue lick absurd pomp.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Covered or incrusted with that which resembles sugar or candy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Will the cold brook,<br/
<qex>Candied</qex>with ice, caudle thy morning tast?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>smoothly coated with crystals of sugar; -- used especially of fruits; <as>as, a <ex>candied</ex> apple</as>.</def><br/
<syn><b>Syn. --</b> candied, crystallized, glac<eacute/, glac<eacute/ed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candify</ent><br/
<hw>Can"di*fy</hw> <pr>(?)</pr>, <pos>v. t. <or/ v. i.</pos> <ety>[L. <ets>candificare</ets>; <ets>cand<eum/re</ets> to be white + <ets>-facere</ets> to make.]</ety> <def>To make or become white, or candied.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candiot</ent><br/
<hw>Can"di*ot</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>candiote</ets>.]</ety> <def>Of or pertaining to Candia; Cretary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candite</ent><br/
<hw>Can"dite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A variety of spinel, of a dark color, found at Candy, in Ceylon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candle</ent><br/
<hw>Can"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>candel</ets>, <ets>candel</ets>, AS, <ets>candel</ets>, fr. L. <ets>candela</ets> a (white) light made of wax or tallow, fr. <ets>cand<eum/re</ets> to be white. See <er>Candid</er>, and cf. <er>Chandler</er>, <er>Cannel</er>, <er>Kindle</er>.]</ety> <sn>1.</sn> <def>A slender, cylindrical body of tallow, containing a wick composed of loosely twisted linen of cotton threads, and used to furnish light.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How far that little <qex>candle</qex> throws his beams!<br/
So shines a good deed in a naughty world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Candles are usually made by repeatedly dipping the wicks in the melted tallow, etc. (<ldquo/dipped <xex>candles</xex><rdquo/), or by casting or running in a mold.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which gives light; a luminary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By these blessed <qex>candles</qex> of the night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Candle nut</b></col>, <cd>the fruit of a euphorbiaceous shrub (<spn>Aleurites triloba</spn>), a native of some of the Pacific islands; -- socalled because, when dry, it will burn with a bright flame, and is used by the natives as a candle. The oil has many uses.</cd> -- <col><b>Candle power</b></col> <fld>(Photom.)</fld>, <cd>illuminating power, as of a lamp, or gas flame, reckoned in terms of the light of a standard candle.</cd> -- <col><b>Electric candle</b></col>, <cd>A modification of the electric arc lamp, in which the carbon rods, instead of being placed end to end, are arranged side by side, and at a distance suitable for the formation of the arc at the tip; -- called also, from the name of the inventor, <altname>Jablockoff candle</altname>.</cd> -- <col><b>Excommunication by inch of candle</b></col>, <cd>a form of excommunication in which the offender is allowed time to repent only while a candle burns.</cd> -- <col><b>Not worth the candle</b></col>, <cd>not worth the cost or trouble.</cd> -- <col><b>Rush candle</b></col>, <cd>a candle made of the pith of certain rushes, peeled except on one side, and dipped in grease.</cd> -- <col><b>Sale by inch of candle</b></col>, <cd>an auction in which persons are allowed to bid only till a small piece of candle burns out.</cd> -- <col><b>Standard candle</b></col> <fld>(Photom.)</fld>, <cd>a special form of candle employed as a standard in photometric measurements; usually, a candle of spermaceti so constructed as to burn at the rate of 120 grains, or 7.8 grams, per hour.</cd> -- <col><b>To curse by bell, book and candle</b></col>. <cd>See under <er>Bell</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candleberry tree</ent><br/
<hw>Can"dle*ber`ry tree</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A shrub (the <spn>Myrica cerifera</spn>, or wax-bearing myrtle), common in North America, the little nuts of which are covered with a greenish white wax, which was formerly, used for hardening candles; -- also called <altname>bayberry tree</altname>, <altname>bayberry</altname>, or <altname>candleberry</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candlebomb</ent><br/
<hw>Can"dle*bomb`</hw> <pr>(#)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A small glass bubble, filled with water, which, if placed in the flame of a candle, bursts by expansion of steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pasteboard shell used in signaling. It is filled with a composition which makes a brilliant light when it explodes.</def>  <rj><au>Farrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candle coal</ent><br/
<hw>Can"dle coal`</hw> <pr>(#)</pr>. <def>See <er>Cannel coal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candlefish</ent><br/
<hw>Can"dle*fish`</hw> <pr>(#)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A marine fish (<spn>Thaleichthys Pacificus</spn>), allied to the smelt, found on the north Pacific coast; -- called also <altname>eulachon</altname>. It is so oily that, when dried, it may be used as a candle, by drawing a wick through it</def>. <sd>(b)</sd> <def>The beshow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candle foot</ent><br/
<hw>Candle foot</hw>. <fld>(Photom.)</fld> <def>The illumination produced by a British standard candle at a distance of one foot; -- used as a unit of illumination.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Candleholder</ent><br/
<hw>Can"dle*hold`er</hw> <pr>(#)</pr>, <pos>n.</pos> <def>One who, or that which, holds a candle; also, one who assists another, but is otherwise not of importance.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candlelight</ent><br/
<hw>Can"dle*light`</hw>, <pos>n.</pos> <def>The light of a candle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never went by <qex>candlelight</qex> to bed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>candlemaker</ent><br/
<hw>candlemaker</hw> <pos>n.</pos> <def>a person who makes or sells candles.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candlemas</ent><br/
<hw>Can"dle*mas</hw> <pr>(#)</pr>, <pos>n.</pos> <ety>[AS. <ets>candelm<ae/sse</ets>, <ets>candel</ets> candle + <ets>m<ae/sse</ets> mass.]</ety> <def>The second day of February, on which is celebrated the feast of the Purification of the Virgin Mary; -- so called because the candles for the altar or other sacred uses are blessed on that day.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candle meter</ent><br/
<hw>Candle meter</hw>. <fld>(Photom.)</fld> <def>The illumination given by a standard candle at a distance of one meter; -- used as a unit of illumination, except in Great Britain.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Candlenut</ent><br/
<hw>Can"dle*nut`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The fruit of a euphorbiaceous tree or shrub (<spn>Aleurites moluccana</spn>), native of some of the Pacific islands. It is used by the natives as a candle, the nut kernels being strung together. The oil from the nut (<mcol><prod><col><b>candlenut oil</b></col></prod> <it>or</it> <prod><col><b>kekune oil</b></col></prod></mcol>) has many uses, including as a varnish.</def><br/
<syn><b>Syn. --</b> varnish tree.</syn><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The tree itself (<spn>Aleurites moluccana</spn>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Candlepin</ent><br/
<hw>Can`dle*pin`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Tenpins)</fld> <sd>(a)</sd> <def>A form of bowling pin slender and nearly straight like a candle.</def> <sd>(b)</sd> <def>The game played with such pins; -- in form <altname>candlepins</altname>, used as a singular.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Candle power</ent><br/
<ent>candlepower</ent><br/
<mhw><hw>candlepower</hw>, <hw>Candle power</hw></mhw>. <fld>(Photom.)</fld> <def>luminous intensity; illuminating power, as of a lamp, or gas flame, measured in candelas, referring to the light of a standard candle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Candlestick</ent><br/
<hw>Can"dle*stick`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>candel-sticca</ets>; <ets>candel</ets> candle + <ets>sticca</ets> stick.]</ety> <def>An instrument or utensil for supporting a candle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candlewaster</ent><br/
<hw>Can"dle*wast`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who consumes candles by being up late for study or dissipation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bookworm, a <qex>candlewaster</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>candlewood</ent><br/
<hw>candlewood</hw> <pos>n.</pos> <def>any of several resinous trees or shrubs often burned for light.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>can-do</ent><br/
<hw>can-do</hw> <pos>adj.</pos> <def>having an eager willingness to accept and overcome challenges; <as>as, a <ex>can-do</ex> kind of person; the city's indomitable optimism and <ex>can-do</ex> spirit</as>.</def> <mark>[slang]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candock</ent><br/
<hw>Can"dock</hw> <pr>(?)</pr> <pos>n.</pos> <ety>[Prob. fr. <ets>can + dock</ets> (the plant). Cf. G. <ets>kannenkraut</ets> horsetail, lit. <ldquo/canweed.<rdquo/]</ety> <fld>(Bot.)</fld> <def>A plant or weed that grows in rivers; a species of <gen>Equisetum</gen>; also, the yellow frog lily (<spn>Nuphar luteum</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candor</ent><br/
<hw>Can"dor</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>candour</asp>.]</altsp> <ety>[L. <ets>candor</ets>, fr. <ets>cand<eum/re</ets>; cf. F. <ets>candeur</ets>. See <er>candid</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Whiteness; brightness; (as applied to moral conditions) usullied purity; innocence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor yor unquestioned integrity<br/
Shall e'er be sullied with one taint or spot<br/
That may take from your innocence and <qex>candor</qex>.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A disposition to treat subjects with fairness; freedom from prejudice or disguise; frankness; sincerity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Attribute superior sagacity and <qex>candor</qex> to those who held that side of the question.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candroy</ent><br/
<hw>Can"droy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A machine for spreading out cotton cloths to prepare them for printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candy</ent><br/
<hw>Can"dy</hw> <pr>(k<acr/n"d<ycr/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Candied</conjf> <pr>(k<acr/n"d<emac/d)</pr>; <pos>p. pr & vb. n.</pos> <conjf>Candying</conjf>.]</vmorph> <ety>[F. <ets>candir</ets> (cf. It. <ets>candire</ets>, Sp. <ets>az<uacute/car cande</ets> or <ets>candi</ets>), fr. Ar. & Pers. <ets>qand</ets>, fr. Skr. <ets>Kha<nsdot/<dsdot/da</ets> piece, sugar in pieces or lumps, fr. <ets>kha<nsdot/<dsdot/</ets>, <ets>kha<dsdot/</ets> to break.]</ety> <sn>1.</sn> <def>To conserve or boil in sugar; <as>as, to <ex>candy</ex> fruits; to <ex>candy</ex> ginger</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make sugar crystals of or in; to form into a mass resembling candy; <as>as, to <ex>candy</ex> sirup</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To incrust with sugar or with candy, or with that which resembles sugar or candy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those frosts that winter brings<br/
Which <qex>candy</qex> every green.</q> <rj><qau>Drayson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 210 --></p>

<p><ent>Candy</ent><br/
<hw>Can"dy</hw> <pr>(k<acr/n"d<ycr/)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To have sugar crystals form in or on; <as>as, fruits preserved in sugar <ex>candy</ex> after a time</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be formed into candy; to solidify in a candylike form or mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Candy</ent><br/
<hw>Can"dy</hw> <pos>n.</pos> <ety>[F. <ets>candi</ets>. See <er>Candy</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Any sweet, more or less solid article of confectionery, especially those prepared in small bite-sized pieces or small bars, having a wide variety of shapes, consistencies, and flavors, and manufactured in a variety of ways.  It is often flavored or colored, or covered with chocolate, and sometimes contains fruit, nuts, etc.; it is often made by boiling sugar or molasses to the desired consistency, and than crystallizing, molding, or working in the required shape.  Other types may consist primarily of chocolate or a sweetened gelatin.  The term may be applied to a single piece of such confection or to the substance of which it is composed.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>Cocaine.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Candy</ent><br/
||<hw>Candy</hw>, <pos>n.</pos> <ety>[Mahratta <ets>kha<nsdot/<dsdot/<imac/</ets>, Tamil <ets>ka<nsdot/<dsdot/i</ets>.]</ety> <def>A weight, at Madras 500 pounds, at Bombay 560 pounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>candyfloss</ent><br/
<hw>candyfloss</hw> <pos>n.</pos> <def>a candy made by spinning sugar that has been boiled to a high temperature; -- called commonly <altname>cotton candy</altname>.</def><br/
<syn><b>Syn. --</b> spun sugar.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>candymaker</ent><br/
<hw>candymaker</hw> <pos>n.</pos> <def>someone who makes candies and other sweets.</def><br/
<syn><b>Syn. --</b> confectioner.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Candytuft</ent><br/
<hw>Can"dy*tuft`</hw> <pr>(k<acr/n"d<esl/*t<ucr/ft`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An annual plant of the genus <gen>Iberis</gen>, cultivated in gardens.  The name was originally given to the <spn>Iberis umbellata</spn>, first, discovered in the island of <etsep>Candia</etsep> (The Italian name for Crete).  It is grown as an ornamental plant, having tufted red,violet, purple, or pink flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>candyweed</ent><br/
<hw>can"dy*weed`</hw> <pr>(k<acr/n"d<esl/*w<emac/d`)</pr>, <pos>n.</pos> <def>bog plant (<spn>Polygala lutea</spn>) of pine barrens of southeastern U.S. having spikes of irregular yellow-orange flowers.</def><br/
<syn><b>Syn. --</b> orange milkwort, yellow milkwort, yellow bachelor's button.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cane</ent><br/
<hw>Cane</hw> <pr>(k<amac/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>cane</ets>, <ets>canne</ets>, OF. <ets>cane</ets>, F. <ets>canne</ets>, L. <ets>canna</ets>, fr. Gr. <grk>ka`nna</grk>, <grk>ka`nnh</grk>; prob. of Semitic origin; cf. Heb. <ets>q<amac/neh</ets> reed. Cf. <er>Canister</er>, <er>canon</er>, 1st <er>Cannon</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A name given to several peculiar palms, species of <gen>Calamus</gen> and <gen>D<ae/manorops</gen>, having very long, smooth flexible stems, commonly called rattans.</def> <sd>(b)</sd> <def>Any plant with long, hard, elastic stems, as reeds and bamboos of many kinds; also, the sugar cane.</def> <sd>(c)</sd> <def>Stems of other plants are sometimes called canes; <as>as, the <ex>canes</ex> of a raspberry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like light <qex>canes</qex>, that first rise big and brave.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the Southern United States <stype>great cane</stype> is the <spn>Arundinaria macrosperma</spn>, and <stype>small cane</stype> is. <spn>Arundinaria tecta</spn>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A walking stick; a staff; -- so called because originally made of one of the species of cane.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Stir the fire with your master's <qex>cane</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A lance or dart made of cane.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Judgelike thou sitt'st, to praise or to arraign<br/
The flying skirmish of the darted <qex>cane</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A local European measure of length. See <er>Canna</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cane borer</b></col> <fld>(Zool.)</fld>, <cd>A beetle <spn>(Oberea bimaculata)</spn> which, in the larval state, bores into pith and destroy the canes or stalks of the raspberry, blackberry, etc.</cd> -- <col><b>Cane mill</b></col>, <cd>a mill for grinding sugar canes, for the manufacture of sugar.</cd> -- <col><b>Cane trash</b></col>, <cd>the crushed stalks and other refuse of sugar cane, used for fuel, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cane</ent><br/
<hw>Cane</hw> <pr>(k<amac/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caned</conjf> <pr>(k<amac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caning</conjf>.]</vmorph> <sn>1.</sn> <def>To beat with a cane.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make or furnish with cane or rattan; <as>as, to <ex>cane</ex> chairs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canebrake</ent><br/
<hw>Cane"brake</hw> <pr>(k<amac/n"br<amac/k`)</pr>, <pos>n.</pos> <def>A thicket of canes.</def>  <rj><au>Ellicott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cane cutter</ent><br/
<ent>canecutter</ent><br/
<mhw><hw>cane"cut`ter</hw>, <hw>cane" cut`ter</hw></mhw> <pr>(k<amac/n"k<ucr/t`<etil/r)</pr>, <pos>n.</pos> <def>a type of rabbit (<spn>Sylvilagus aquaticus</spn>) inhabiting southeastern U.S. swamps and lowlands; -- called also <altname>swamp rabbit</altname>.</def><br/
<syn><b>Syn. --</b> swamp rabbit, swamp hare.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caned</ent><br/
<hw>Caned</hw> <pr>(k<amac/nd)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>canus</ets> white.]</ety> <def>Filled with white flakes; mothery; -- said vinegar when containing mother.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canella</ent><br/
||<hw>Ca*nel"la</hw> <pr>(k<adot/*n<ecr/l"l<adot/)</pr>, <pos>n.</pos> <ety>[LL. (OE. <ets>canel</ets>, <ets>canelle</ets>, cinnamon, fr. F. <ets>cannelle</ets>), Dim. of L. <ets>canna</ets> a reed. <ets>Canella</ets> is so called from the shape of the rolls of prepared bark. See <er>Cane</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of trees of the order <ord>Canellace<ae/</ord>, growing in the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The principal species is <spn>Canella alba</spn>, and its bark is a spice and drug exported under the names of <prod>wild cinnamon</prod> and <prod>whitewood bark</prod>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canella-alba</ent><br/
<hw>Canella-alba</hw> <pos>n.</pos> <def>A large evergreen shrub or small tree (<spn>Canella alba</spn> or <spn>Canella winterana</spn>) having white aromatic bark and leathery leaves and small purple to red flowers in terminal cymes.  Its bark is called <prod>wild cinnamon</prod>.</def><br/
<syn><b>Syn. --</b> wild cinnamon, white cinnamon tree, <spn>Canella winterana</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canellaceae</ent><br/
<hw>Canellaceae</hw> <pos>n.</pos> <sn>1.</sn> <def>a natural family having one genus: aromatic tropical trees of East Africa and Florida to West Indies.</def> <note>In 1913 the family was classed as an order: see <er>canella</er>.</note><br/
<syn><b>Syn. --</b> family <fam>Canellaceae</fam>, canella family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canescent</ent><br/
<hw>Ca*nes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>canescens</ets>, p. pr. of <ets>canescere</ets>, v. inchoative of <ets>canere</ets> to be white.]</ety> <def>Growing white, or assuming a color approaching to white.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cangue</ent><br/
<hw>Cangue</hw> <pr>(k<acr/ng)</pr>, <pos>n.</pos> <altsp>[Written also <asp>cang</asp>.]</altsp> <ety>[F. <ets>cangue</ets>, fr. Pg. <ets>canga</ets> yoke.]</ety> <def>A very broad and heavy wooden collar which certain offenders in China are compelled to wear as a punishment.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Can hook</ent><br/
<hw>Can" hook`</hw> <pr>(?)</pr>. <def>A device consisting of a short rope with flat hooks at each end, for hoisting casks or barrels by the ends of the staves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canicula</ent><br/
||<hw>Ca*nic"u*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>canicula</ets>, lit., a little dog, a diminutive of <ets>canis</ets> dog; cf. F. <ets>canicule</ets>.]</ety> <fld>(Astron.)</fld> <def>The Dog Star; Sirius.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canicular</ent><br/
<hw>Ca*nic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>canicularis</ets>; cf. F. <ets>caniculaire</ets>.]</ety> <def>Pertaining to, or measured, by the rising of the Dog Star.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canicular days</b></col>, <cd>the dog days, See <er>Dog days</er>.</cd> -- <col><b>Canicular year</b></col>, <cd>the Egyptian year, computed from one heliacal rising of the Dog Star to another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canicule</ent><br/
<hw>Can"i*cule</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Canicula.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>canid</ent><br/
<hw>can"id</hw> <pr>(k<acr/n"<icr/d; k<amac/n"<icr/d)</pr>, <pos>n.</pos> <def>any doglike animal of the family <er>canidae</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Canidae</ent><br/
<hw>Can"id*ae</hw> <pos>n.</pos> <def>a natural family of doglike mammals including the domestic dogs, wolves, jackals, foxes, hyenas, and coyotes.</def><br/
<syn><b>Syn. --</b> family <fam>Canidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caninal</ent><br/
<hw>Ca*ni"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Canine</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canine</ent><br/
<hw>Ca*nine"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caninus</ets>, fr. <ets>canis</ets> dog: cf. F. <ets>canin</ets>. See <er>Hound</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the family <fam>Canid<ae/</fam>, or dogs and wolves; having the nature or qualities of a dog; like that or those of a dog.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the pointed tooth on each side the incisors.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canine appetite</b></col>, <cd>a morbidly voracious appetite; bulimia.</cd> -- <col><b>Canine letter</b></col>, <cd>the letter r. See <er>R</er>.</cd> -- <col><b>Canine madness</b></col>, <cd>hydrophobia.</cd> -- <col><b>Canine tooth</b></col>, <cd>a tooth situated between the incisor and bicuspid teeth, so called because well developed in dogs; usually, the third tooth from the front on each side of each jaw; an eyetooth, or the corresponding tooth in the lower jaw.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canine</ent><br/
<hw>Ca*nine"</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>A canine tooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caning</ent><br/
<hw>can"ing</hw> <pos>n.</pos> <def>something made of interlaced slender branches of especially willow.</def><br/
<syn><b>Syn. --</b> wicker, wickerwork.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canis</ent><br/
||<hw>Ca"nis</hw> <pr>(k<acr/"n<icr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canes</plw> <pr>(-n<emac/z)</pr>.</plu> <ety>[L., a dog.]</ety> <fld>(Zool.)</fld> <def>A genus of carnivorous mammals, of the family <fam>Canid<ae/</fam>, including the dogs and wolves.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Canis major</b></col> <ety>[L., larger dog]</ety>, <cd>a constellation to the southeast of Orion, containing Sirius or the Dog Star.</cd> -- <col><b>||Canis minor</b></col> <ety>[L., smaller dog]</ety>, <cd>a constellation to the east of Orion, containing Procyon, a star of the first magnitude.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>canistel</ent><br/
<hw>canistel</hw> <pos>n.</pos> <sn>1.</sn> <def>a tropical tree of Florida and West Indies (<spn>Pouteria campechiana nervosa</spn>) yielding edible fruit.</def><br/
<syn><b>Syn. --</b> canistel tree.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an ovoid orange-yellow mealy sweet fruit of Florida and West Indies.</def><br/
<syn><b>Syn. --</b> eggfruit.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canister</ent><br/
<hw>Can"is*ter</hw> <pr>(k<acr/n"<icr/s*t<etil/r)</pr>, <pos>n.</pos> <ety>[L. <ets>canistrum</ets> a basket woven from reeds Gr. <?/, fr. <grk>ka`nh</grk>, <grk>ka`nna</grk> reed; cf. F. <ets>canistre</ets>. See <er>Cane</er>, and <er>Canaster</er>.]</ety> <sn>1.</sn> <def>A small basket of rushes, reeds, or willow twigs, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small box or case for holding tea, coffee, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>A kind of case shot for cannon, in which a number of lead or iron balls in layers are inclosed in a case fitting the gun; -- called also <altname>canister shot</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker</ent><br/
<hw>Can"ker</hw> <pr>(k<acr/<nsm/"k<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>canker</ets>, <ets>cancre</ets>, AS. <ets>cancer</ets> (akin to D. <ets>kanker</ets>, OHG <ets>chanchar</ets>.), fr. L. <ets>cancer</ets> a cancer; or if a native word, cf. Gr. <?/ excrescence on tree, <?/ gangrene. Cf. also OF. <ets>cancre</ets>, F. <ets>chancere</ets>, fr. L. <ets>cancer</ets>. See <er>cancer</er>, and cf. <er>Chancre</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A corroding or sloughing ulcer; esp. a spreading gangrenous ulcer or collection of ulcers in or about the mouth; -- called also <altname>water canker</altname>, <altname>canker of the mouth</altname>, and <altname>noma</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which corrodes, corrupts, or destroy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cankers</qex> of envy and faction.</q> <rj><qau>Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Hort.)</fld> <def>A disease incident to trees, causing the bark to rot and fall off.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Far.)</fld> <def>An obstinate and often incurable disease of a horse's foot, characterized by separation of the horny portion and the development of fungoid growths; -- usually resulting from neglected thrush.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A kind of wild, worthless rose; the dog-rose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To put down Richard, that sweet lovely rose.<br/
And plant this thorm, this <qex>canker</qex>, Bolingbroke.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black canker</b></col>. <cd>See under <er>Black</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker</ent><br/
<hw>Can"ker</hw> <pr>(k<acr/<nsm/"k<etil/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cankered</conjf> <pr>(-k<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cankering</conjf>.]</vmorph> <sn>1.</sn> <def>To affect as a canker; to eat away; to corrode; to consume.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No lapse of moons can <qex>canker</qex> Love.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To infect or pollute; to corrupt.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A tithe purloined <qex>cankers</qex> the whole estate.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker</ent><br/
<hw>Can"ker</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To waste away, grow rusty, or be oxidized, as a mineral.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Silvering will sully and <qex>canker</qex> more than gliding.</q> <rj><qau>Bacom.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be or become diseased, or as if diseased, with canker; to grow corrupt; to become venomous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Deceit and <qex>cankered</qex> malice.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As with age his body uglier grows,<br/
So his mind <qex>cankers</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker-bit</ent><br/
<hw>Can"ker-bit`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Eaten out by canker, or as by canker.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker bloom</ent><br/
<hw>Can"ker bloom`</hw> <pr>(?)</pr>. <def>The bloom or blossom of the wild rose or dog-rose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker blossom</ent><br/
<hw>Can"ker blos`som</hw> <pr>(?)</pr>. <def>That which blasts a blossom as a canker does.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O me! you juggler! you <qex>canker blossom</qex>!<br/
You thief of Love!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cankered</ent><br/
<hw>Can"kered</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Affected with canker; <as>as, a <ex>cankered</ex> mouth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected mentally or morally as with canker; sore, envenomed; malignant; fretful; ill-natured.</def> <ldquo/A <xex>cankered</xex> grandam's will.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cankeredly</ent><br/
<hw>Can"kered*ly</hw>, <pos>adv.</pos> <def>Fretfully; spitefully.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker fly</ent><br/
<hw>Can"ker fly`</hw> <pr>(?)</pr>. <def>A fly that preys on fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cankerous</ent><br/
<hw>Can"ker*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affecting like a canker.</def> <ldquo/<xex>Canrerous</xex> shackles.<rdquo/  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Misdeem it not a <qex>cankerous</qex> change.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canker rash</ent><br/
<hw>Can"ker rash`</hw> <pr>(?)</pr>. <fld>(Med.)</fld> <def>A form of scarlet fever characterized by ulcerated or putrid sore throat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cankerworm</ent><br/
<hw>Can"ker*worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The larva of two species of geometrid moths which are very injurious to fruit and shade trees by eating, and often entirely destroying, the foliage. Other similar larv<ae/ are also called cankerworms.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The autumnal species (<spn>Anisopteryx pometaria</spn>) becomes adult late in autumn (after frosts) and in winter.  The spring species (<spn>Anisopteryx vernata</spn>) remains in the ground through the winter, and matures in early spring.  Both have winged males and wingless females. The larv<ae/ are similar in appearance and habits, and belong to the family of measuring worms or spanworms. These larv<ae/ hatch from the eggs when the leaves begin to expand in spring.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cankery</ent><br/
<hw>Can"ker*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Like a canker; full of canker.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Surly; sore; malignant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canna</ent><br/
||<hw>Can"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A measure of length in Italy, varying from six to seven feet. See <er>Cane</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canna</ent><br/
||<hw>Can"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a reed. See <er>Cane</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of tropical plants, with large leaves and often with showy flowers. The Indian shot (<spn>Canna Indica</spn>) is found in gardens of the northern United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannabene</ent><br/
<hw>Can"na*bene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cannabis</er>.]</ety> <fld>(Chem.)</fld> <def>A colorless oil obtained from hemp by distillation, and possessing its intoxicating properties.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannabidaceae</ent><br/
<hw>Cannabidaceae</hw> <pos>n.</pos> <sn>1.</sn> <def>a grouping of plants containing two genera of erect or twining wind-pollinated herbs, the genera <gen>Cannabis</gen> and <gen>Humulus</gen>; this term is not used in all classifications; in some the genus <gen>Cannabis</gen> is placed in the family <fam>Moraceae</fam> and the genus <gen>Humulus</gen> in the family <fam>Urticaceae</fam>.</def><br/
<syn><b>Syn. --</b> family <fam>Cannabidaceae</fam>, hemp family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cannabin</ent><br/
<hw>Can"na*bin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A poisonous resin extracted from <prodby>hemp</prodby> (<spn>Cannabis sativa</spn>, variety <varn>Indica</varn>).  The euphoric and narcotic effects of hasheesh are due to this resin; the active ingredient is tetrahydrocannabinol, THC.  See <cref>Cannabis Indica</cref>, below.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cannabine</ent><br/
<hw>Can"na*bine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cannabinus</ets>.]</ety> <def>Pertaining to hemp; hempen.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannabis</ent><br/
||<hw>Can"na*bis</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[L., hemp. See <er>Canvas</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of a single species belonging to the order <ord>Uricace<ae/</ord>; hemp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The Indian hemp plant plant <spn>Cannabis sativa</spn> syn. <spn>Cannabis Indica</spn>.  See <cref>Cannabis Indica</cref>, below.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Cannabis Indica</b></col> <pr>(<?/)</pr>, <cd>the Indian hemp, now considered a variety of the common hemp (<spn>Cannabis sativa</spn>), originating in Asia but now grown widely around the world.  It can be formed into a tough fiber that is used to make rope and tough fabric.  The dried leaves and female flowers are smoked or chewed by people to produce a euphoric or hallucinogenic effect, caused mainly by the substance <prod>tetrahydrocannabinol</prod> (THC) contained in it.  The THC-containing products of Cannabis are classified as a controlled substance, and posession and sale are illegal in most states of the United States.  Once classified as a narcotic, the plant is now more commonly referred to in the category of euphoric or hallucinogenic agents.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cannaceae</ent><br/
<hw>Cannaceae</hw> <pos>prop. n.</pos> <def>A natural family of plants coextensive with the genus <gen>Canna</gen>.</def><br/
<syn><b>Syn. --</b> family <fam>Cannaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cannae</ent><br/
<hw>Cannae</hw> <pos>n.</pos> <def>the name of a battle in which <persfn>Hannibal</persfn> defeated the Romans in 216 <sc>b. c.</sc>  Called also <altname>battle of Cannae</altname>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>canned</ent><br/
<hw>canned</hw> <pos>adj.</pos> <sn>1.</sn> <def>Recorded; in contrast with <contr>live</contr>; -- used of sound or video broadcasts.</def> <br/
<syn><b>Syn. --</b> filmed, taped, tape-recorded.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>Sealed in a can or jar.</def><br/
<syn><b>Syn. --</b> tinned.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cannel coal</ent><br/
<hw>Can"nel coal`</hw> <pr>(?)</pr>. <ety>[Corrupt. fr. <ets>candle coal</ets>.]</ety> <def>A kind of mineral coal of a black color, sufficiently hard and solid to be cut and polished.  It burns readily, with a clear, yellow flame, and on this account has been used as a substitute for candles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannele</ent><br/
||<hw>Can`ne*l<eacute/"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., pop., fluted.]</ety> <fld>(Textiles)</fld> <def>A style of interweaving giving to fabrics a channeled or fluted effect; also, a fabric woven so as to have this effect; a rep.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cannelloni</ent><br/
<hw>cannelloni</hw> <pos>n.</pos> <def>a tubular pasta filled with meat or cheese.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cannelure</ent><br/
<hw>Can"ne*lure</hw> <pr>(k<acr/n"n<esl/*l<usl/r)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>canneler</ets> to groove.]</ety> <fld>(Mil.)</fld> <def>A groove in any cylinder; specif., a groove around the cylinder of an elongated bullet for small arms to contain a lubricant, or around the rotating band of a gun projectile to lessen the resistance offered to the rifling. Also, a groove around the base of a cartridge, where the extractor takes hold.</def> -- <wordforms><wf>Can"ne*lured</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cannery</ent><br/
<hw>Can"ner*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A place where the business of canning fruit, meat, etc., is carried on.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannibal</ent><br/
<hw>Can"ni*bal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cannibale</ets>. Columbus, in a letter to the Spanish monarchs written in Oct., 1498, mentions that the people of Haiti lived in great fear of the <ets>Caribales</ets> (equivalent to E. <ets>Caribbees</ets>.), the inhabitants of the smaller Antilles; which form of the name was afterward changed into NL. <ets>Canibales</ets>, in order to express more forcibly their character by a word intelligible through a Latin root <ldquo/propter rabiem <ets>caninam</ets> anthropophagorum gentis.<rdquo/ The Caribbees call themselves, in their own language. <ets>Calinago</ets>, <ets>Carinago</ets>, <ets>Calliponam</ets>, and, abbreviated, <ets>Calina</ets>, signifying a brave, from which Columbus formed his <ets>Caribales</ets>.]</ety> <def>A human being that eats human flesh; hence, any that devours its own kind.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannibal</ent><br/
<hw>Can"ni*bal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to cannibals or cannibalism.</def> <ldquo/<xex>Cannibal</xex> terror.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cannibalic</ent><br/
<hw>cannibalic</hw> <pos>adj.</pos>  <def>resembling a beast; showing lack of human sensibility.</def><br/
<syn><b>Syn. --</b> beastly, bestial, brute(prenominal), brutish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cannibalize</ent><br/
<ent>cannibalise</ent><br/
<mhw><hw>cannibalise</hw>, <hw>cannibalize</hw></mhw> <pos>v. i.</pos> <def>to eat human flesh.</def><br/
<syn><b>Syn. --</b> practice cannibalism.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cannibalize</ent><br/
<ent>cannibalise</ent><br/
<mhw><hw>cannibalise</hw>, <hw>cannibalize</hw></mhw> <pos>v. t.</pos> <def>to use parts of (something, such as a machine), to repair something else.</def><br/
<syn><b>Syn. --</b> use parts of.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cannibalism</ent><br/
<hw>Can"ni*bal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cannibalisme</ets>.]</ety> <def>The act or practice of eating human flesh by mankind. Hence; Murderous cruelty; barbarity.</def>  <rj><au>Berke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cannibalize</ent><br/
<hw>cannibalize</hw> <pos>v. i. & t.</pos> <def>same as <er>cannibalise</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cannibally</ent><br/
<hw>Can"ni*bal*ly</hw>, <pos>adv.</pos> <def>In the manner of cannibal.</def> <ldquo/An he had been <xex>cannibally</xex> given.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannikin</ent><br/
<hw>Can"ni*kin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Can</ets> + <ets>-kin</ets>.]</ety> <def>A small can or drinking vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannily</ent><br/
<hw>Can"ni*ly</hw>, <pos>adv.</pos> <def>In a canny manner.</def> <mark>[N. of Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canniness</ent><br/
<hw>Can"ni*ness</hw>, <pos>n.</pos> <def>Caution; crafty management.</def> <mark>[N. of Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannon</ent><br/
<hw>Can"non</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cannons</plw> <pr>(#)</pr>, collectively <plw>Cannon</plw>.</plu> <ety>[F. <ets>cannon</ets>, fr. L. <ets>canna</ets> reed, pipe, tube. See <er>Cane</er>.]</ety> <sn>1.</sn> <def>A great gun; a piece of ordnance or artillery; a firearm for discharging heavy shot with great force.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Cannons are made of various materials, as iron, brass, bronze, and steel, and of various sizes and shapes with respect to the special service for which they are intended, as intended, as siege, seacoast, naval, field, or mountain, guns. They always aproach more or less nearly to a cylindrical from, being usually thicker toward the breech than at the muzzle. Formerly they were cast hollow, afterwards they were cast, solid, and bored out. The cannon now most in use for the armament of war vessels and for seacoast defense consists of a forged steel tube reinforced with massive steel rings shrunk upon it. Howitzers and mortars are sometimes called cannon. See <er>Gun</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A hollow cylindrical piece carried by a revolving shaft, on which it may, however, revolve independently.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Printing.)</fld> <def>A kind of type. See <er>Canon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cannon ball</b></col>, <cd>strictly, a round solid missile of stone or iron made to be fired from a cannon, but now often applied to a missile of any shape, whether solid or hollow, made for cannon. Elongated and cylindrical missiles are sometimes called <xex>bolts</xex>; hollow ones charged with explosives are properly called <xex>shells</xex>.</cd> -- <col><b>Cannon bullet</b></col>, <cd>a cannon ball</cd>. <mark>[Obs.]</mark> -- <col><b>Cannon cracker</b></col>, <cd>a fire cracker of large size.</cd> -- <col><b>Cannon lock</b></col>, <cd>a device for firing a cannon by a percussion primer.</cd> -- <col><b>Cannon metal</b></col>. <cd>See <er>Gun Metal</er>.</cd> -- <col><b>Cannon pinion</b></col>, <cd>the pinion on the minute hand arbor of a watch or clock, which drives the hand but permits it to be moved in setting.</cd> -- <col><b>Cannon proof</b></col>, <cd>impenetrable by cannon balls.</cd> -- <col><b>Cannon shot</b></col>. <sd>(a)</sd> <cd>A cannon ball.</cd> <sd>(b)</sd> <cd>The range of a cannon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannon</ent><br/
<hw>Can"non</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To discharge cannon.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To collide or strike violently, esp. so as to glance off or rebound; to strike and rebound.</def></p>

<p><q>He heard the right-hand goal post crack as a pony <qex>cannoned</qex> into it -- crack, splinter, and fall like a mast.</q>  <rj><qau>Kipling.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cannon</ent><br/
<hw>Can"non</hw>, <pos>n. & v.</pos> <fld>(Billiards)</fld> <def>See <er>Carom</er>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannonade</ent><br/
<hw>Can`non*ade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>Canonnade</ets>; cf. It. <ets>cannanata</ets>.]</ety> <sn>1.</sn> <def>The act of discharging cannon and throwing ball, shell, etc., for the purpose of destroying an army, or battering a town, ship, or fort; -- usually, an attack of some continuance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A furious <qex>cannonade</qex> was kept up from the whole circle of batteries on the devoted towm.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.; A loud noise like a cannonade; a booming.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Blue Walden rolls its <qex>cannonade</qex>.</q> <rj><qau>Ewerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannonade</ent><br/
<hw>Can`non*ade"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cannonade</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cannonading</conjf>.]</vmorph> <def>To attack with heavy artillery; to batter with cannon shot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannonade</ent><br/
<hw>Can`non*ade"</hw>, <pos>v. i.</pos> <def>To discharge cannon; <as>as, the army <ex>cannonaded</ex> all day</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannon bone</ent><br/
<hw>Can"non bone</hw> <pr>(?)</pr>. <fld>(Anat.)</fld> <def>See <cref>Canon Bone</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannoned</ent><br/
<hw>Can"noned</hw> <pr>(<?/)</pr>, <pos>a.</pos> <def>Furnished with cannon.</def> <mark>[Poetic]</mark> <ldquo/Gilbralter's <qex>cannoned</qex> steep.<rdquo/ <au>M. Arnold.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannonier</ent><br/
<ent>Cannoneer</ent><br/
<mhw>{ <hw>Can`non*eer"</hw>, <hw>Can`non*ier"</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>canonnier</ets>.]</ety> <def>A man who manages, or fires, cannon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannonering</ent><br/
<hw>Can`non*er"ing</hw>, <pos>n.</pos> <def>The use of cannon.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannonry</ent><br/
<hw>Can"non*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cannon, collectively; artillery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ringing of bells and roaring of <qex>cannonry</qex> proclaimed his course through the country.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannot</ent><br/
<hw>Can"not</hw> <pr>(?)</pr>. <ety>[<ets>Can</ets> to be able + <ets>-not</ets>.]</ety> <def>Am, is, or are, not able; -- written either as one word or two.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannula</ent><br/
<hw>Can"nu*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cannula</ets> a small tube of dim. of <ets>canna</ets> a reed, tube.]</ety> <fld>(Surg.)</fld> <def>A small tube of metal, wood, or India rubber, used for various purposes, esp. for injecting or withdrawing fluids. It is usually associated with a trocar.</def> <altsp>[Written also <asp>canula</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannular</ent><br/
<hw>Can"nu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of a tube; tubular.</def> <altsp>[Written also <asp>canular</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannulated</ent><br/
<hw>Can"nu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Hollow; affording a passage through its interior length for wire, thread, etc.; <as>as, a <ex>cannulated</ex> (suture) needle</as>.</def> <altsp>[Written also <asp>canulated</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cannei</ent><br/
<ent>Canny</ent><br/
<mhw>{ <hw>Can"ny</hw>, <hw>Can"nei</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. Icel. <ets>kenn</ets> skilled, learned, or E. <ets>canny</ets>. Cf. <er>Kenn</er>.]</ety> <mark>[North of Eng. & Scot.]</mark> <sn>1.</sn> <def>Artful; cunning; shrewd; wary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Skillful; knowing; capable.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Cautious; prudent; safe..</def>  <rj><au>Ramsay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having pleasing or useful qualities; gentle.</def>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Reputed to have magical powers.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>No canny</b></col>, <cd>not safe, not fortunate; unpropitious.</cd> <mark>[Scot.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canoe</ent><br/
<hw>Ca*noe"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canoes</plw> <pr>(#)</pr>.</plu> <ety>[Sp. <ets>canoa</ets>, fr. Caribbean <ets>can<aacute/oa</ets>.]</ety> <sn>1.</sn> <def>A boat used by rude nations, formed of trunk of a tree, excavated, by cutting of burning, into a suitable shape. It is propelled by a paddle or paddles, or sometimes by sail, and has no rudder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others devised the boat of one tree, called the <qex>canoe</qex>.</q> <rj><qau>Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A boat made of bark or skins, used by savages.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A birch <qex>canoe</qex>, with paddles, rising, falling, on the water.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A light pleasure boat, especially designed for use by one who goes alone upon long excursions, including portage. It it propelled by a paddle, or by a small sail attached to a temporary mast.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 211 --></p>

<p><ent>canoe</ent><br/
<hw>ca*noe"</hw> <pr>(k<adot/*n<oomac/")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Canoed</conjf> <pr>(k<adot/*n<oomac/d")</pr> <pos>p. pr. & vb. n.</pos> <conjf>Canoeing</conjf> <pr>(k<adot/*n<oomac/"<icr/ng)</pr>.]</vmorph> <def>To manage a canoe, or voyage in a canoe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>canoeing</ent><br/
<hw>ca*noe"ing</hw> <pos>n.</pos> <def>The act or art of using a canoe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>canoeist</ent><br/
<hw>ca*noe"ist</hw> <pr>(k<adot/*n<oomac/"<icr/st)</pr>, <pos>n.</pos> <def>A canoeman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>canoeman</ent><br/
<hw>ca*noe"man</hw>  <pr>(k<adot/*n<oomac/"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canoemen</plw>  <pr>(k<adot/*n<oomac/"m<eit/n)</pr>.</plu> <def>One who uses a canoe; one who travels in a canoe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cabins and clearing greeted the eye of the passing <qex>canoeman</qex>.</q> <rj><qau>Parkman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>canon</ent><br/
<hw>can"on</hw> <pr>(k<acr/n"<ucr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>canon</ets>, <ets>canoun</ets>, AS. <ets>canon</ets> rule (cf. F. <ets>canon</ets>, LL. <ets>canon</ets>, and, for sense 7, F. <ets>chanoine</ets>, LL. <ets>canonicus</ets>), fr. L. <ets>canon</ets> a measuring line, rule, model, fr. Gr. <grk>kanw`n</grk> rule, rod, fr. <grk>ka`nh</grk>, <grk>ka`nnh</grk>, reed.  See <er>Cane</er>, and cf. <er>Canonical</er>.]</ety> <sn>1.</sn> <def>A law or rule.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or that the Everlasting had not fixed<br/
His <qex>canon</qex> 'gainst self-slaughter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A law, or rule of doctrine or discipline, enacted by a council and confirmed by the pope or the sovereign; a decision, regulation, code, or constitution made by ecclesiastical authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Various <qex>canons</qex> which were made in councils held in the second centry.</q> <rj><qau>Hook.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The collection of books received as genuine Holy Scriptures, called the <altname>sacred canon</altname>, or general rule of moral and religious duty, given by inspiration; the Bible; also, any one of the canonical Scriptures. See <cref>Canonical books</cref>, under <er>Canonical</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In monasteries, a book containing the rules of a religious order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A catalogue of saints acknowledged and canonized in the Roman Catholic Church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A member of a cathedral chapter; a person who possesses a prebend in a cathedral or collegiate church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mus.)</fld> <def>A musical composition in which the voices begin one after another, at regular intervals, successively taking up the same subject.  It either winds up with a <part>coda</part> (tailpiece), or, as each voice finishes, commences anew, thus forming a perpetual fugue or round.  It is the strictest form of imitation. See <er>Imitation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Print.)</fld> <def>The largest size of type having a specific name; -- so called from having been used for printing the canons of the church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The part of a bell by which it is suspended; -- called also <altname>ear</altname> and <altname>shank</altname>.</def> <note>[See <xex>Illust.</xex> of <er>Bell</er>.]</note>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Billiards)</fld> <def>See <er>Carom</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Apostolical canons</b></col>. <cd>See under <er>Apostolical</er>.</cd> -- <mcol><col><b>Augustinian canons</b></col>, <col><b>Black canons</b></col></mcol>. <cd>See under <er>Augustinian</er>.</cd> -- <mcol><col><b>Canon capitular</b></col>, <col><b>Canon residentiary</b></col></mcol>, <cd>a resident member of a cathedral chapter (during a part or the whole of the year).</cd> -- <col><b>Canon law</b></col>. <cd>See under <er>Law</er>.</cd> -- <col><b>Canon of the Mass</b></col> <fld>(R. C. Ch.)</fld>, <cd>that part of the mass, following the Sanctus, which never changes.</cd> -- <col><b>Honorary canon</b></col>, <cd>a canon{6} who neither lived in a monastery, nor kept the canonical hours.</cd> -- <col><b>Minor canon</b></col> <fld>(Ch. of Eng.)</fld>, <cd>one who has been admitted to a chapter, but has not yet received a prebend.</cd> -- <col><b>Regular canon</b></col> <fld>(R. C. Ch.)</fld>, <cd>one who lived in a conventual community and followed the rule of St. Austin; a Black canon.</cd> -- <col><b>Secular canon</b></col> <fld>(R. C. Ch.)</fld>, <cd>one who did not live in a monastery, but kept the hours.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>canyon</ent><br/
||<hw>Ca*<ntil/on"</hw> <pr>(k<aum/*ny<osl/n"; anglicized k<acr/n"y<ucr/n)</pr>, <pos>n.</pos> <ety>[Sp., a tube or hollow, fr. <ets>ca<ntil/a</ets> reed, fr. L. <ets>canna</ets>. See <er>Cane</er>.]</ety> <def>A deep gorge, ravine, or gulch, between high and steep banks, worn by water courses.</def> <mark>[Mexico & Western U. S.]</mark>  <altsp>[Also spelled <asp>canyon</asp>.]</altsp><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Canon bit</ent><br/
<hw>Can"on bit`</hw> <pr>(?)</pr>. <ety>[F. <ets>canon</ets>, fr. L. <ets>canon</ets> a rule.]</ety> <def>That part of a bit which is put in a horse's mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canon bone</ent><br/
<hw>Can"on bone`</hw> <pr>(?)</pr>. <ety>[F. <ets>canon</ets>, fr. L. <ets>canon</ets> a rule. See <er>canon</er>.]</ety> <fld>(Anat.)</fld> <def>The shank bone, or great bone above the fetlock, in the fore and hind legs of the horse and allied animals, corresponding to the middle metacarpal or metatarsal bone of most mammals. See <er>Horse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>canoncito</ent><br/
||<hw>ca<ntil/`on*ci"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Amer. Sp. dim. See <er>ca<ntil/on</er>.]</ety> <mark>[Southwestern U. S.]</mark>  <sn>1.</sn> <def>A small ca<ntil/on.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A narrow passage or lane through chaparral or a forest.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Canoness</ent><br/
<hw>Can"on*ess</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>canonissa</ets>.]</ety> <def>A woman who holds a canonry in a conventual chapter.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Regular canoness</b></col>, <cd>one bound by the vow of poverty, and observing a strict rule of life.</cd> -- <col><b>Secular canoness</b></col>, <cd>one allowed to hold private property, and bound only by vows of chastity and obedience so long as she chose to remain in the chapter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>canonical</ent><br/
<ent>canonic</ent><br/
<mhw>{ <hw>ca*non"ic</hw> <pr>(k<adot/*n<ocr/n"<icr/k)</pr>, <hw>ca*non"ic*al</hw> <pr>(k<adot/*n<ocr/n"<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>canonicus</ets>, LL. <ets>canonicalis</ets>, fr. L. <ets>canon</ets>: cf. F. <ets>canonique</ets>. See <er>canon</er>.]</ety> <def>Of or pertaining to a canon; established by, or according to, a canon or canons.</def>  <ldquo/The oath of <xex>canonical</xex> obedience.<rdquo/  <rj><au>Hallam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Appearing in a Biblical canon; <as>as, a <ex>canonical</ex> book of the Christian New Testament</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>Accepted as authoritative; recognized.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Math.)</fld> <def>In its standard form, usually also the simplest form; -- of an equation or coordinate.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Linguistics)</fld> <def>Reduced to the simplest and most significant form possible without loss of generality; <as>as, a <ex>canonical</ex> syllable pattern</as>.  Opposite of <ant>nonstandard</ant>.</def><br/
<syn><b>Syn. --</b> standard.</syn>
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn> <def>Pertaining to or resembling a musical canon.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Canonical books</b></col>, <it>or</it> <col><b>Canonical Scriptures</b></col></mcol>, <cd>those books which are declared by the canons of the church to be of divine inspiration; -- called collectively <altname>the canon</altname>. The Roman Catholic Church holds as canonical several books which Protestants reject as apocryphal.</cd> -- <col><b>Canonical epistles</b></col>, <cd>an appellation given to the epistles called also <xex>general</xex> or <xex>catholic</xex>. See <cref>Catholic epistles</cref>, under <er>Canholic</er>.</cd> -- <col><b>Canonical form</b></col> <fld>(Math.)</fld>, <cd>the simples or most symmetrical form to which all functions of the same class can be reduced without lose of generality.</cd> -- <col><b>Canonical hours</b></col>, <cd>certain stated times of the day, fixed by ecclesiastical laws, and appropriated to the offices of prayer and devotion; also, certain portions of the Breviary, to be used at stated hours of the day. In England, this name is also given to the hours from 8 <sc>a. m.</sc> to 3 <sc>p. m.</sc> (formerly 8 <sc>a. m.</sc> to 12 <sc>m.</sc>) before and after which marriage can not be legally performed in any parish church.</cd> -- <col><b>Canonical letters</b></col>, <cd>letters of several kinds, formerly given by a bishop to traveling clergymen or laymen, to show that they were entitled to receive the communion, and to distinguish them from heretics.</cd> -- <col><b>Canonical life</b></col>, <cd>the method or rule of living prescribed by the ancient clergy who lived in community; a course of living prescribed for the clergy, less rigid than the monastic, and more restrained that the secular.</cd> -- <col><b>Canonical obedience</b></col>, <cd>submission to the canons of a church, especially the submission of the inferior clergy to their bishops, and of other religious orders to their superiors.</cd> -- <col><b>Canonical punishments</b></col>, <cd>such as the church may inflict, as excommunication, degradation, penance, etc.</cd> -- <col><b>Canonical sins</b></col> <fld>(Anc. Church.)</fld>, <cd>those for which capital punishment or public penance decreed by the canon was inflicted, as idolatry, murder, adultery, heresy.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonically</ent><br/
<hw>Ca*non"ic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a canonical manner; according to the canons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonicalness</ent><br/
<hw>Ca*non"ic*al*ness</hw>, <pos>n.</pos> <def>The quality of being canonical; canonicity.</def>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonicals</ent><br/
<hw>Ca*non"ic*als</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The dress prescribed by canon{2} to be worn by a clergyman when officiating.  Sometimes, any distinctive professional dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Full canonicals</b></col>, <cd>the complete costume of an officiating clergyman or ecclesiastic.</cd></cs></p>

<p><ent>Canonicate</ent><br/
<hw>Ca*non"i*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>canonucatus</ets> canonical: cf. F. <ets>canonicat</ets>.]</ety> <def>The office of a canon; a canonry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonicity</ent><br/
<hw>Can`on*ic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>canonicit<eacute/</ets>.]</ety> <def>The state or quality of being canonical; agreement with the canon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonist</ent><br/
<hw>Can"on*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>canoniste</ets>.]</ety> <def>A professor of canon law; one skilled in the knowledge and practice of ecclesiastical law.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonistic</ent><br/
<hw>Can`on*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a canonist.</def> <ldquo/This <xex>canonistic</xex> exposition.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonization</ent><br/
<hw>Can`on*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>canonisation</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(R. C. Ch.)</fld> <def>The final process or decree (following beatifacation) by which the name of a deceased person is placed in the catalogue (canon) of saints and commended to perpetual veneration and invocation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Canonization</qex> of saints was not known to the Christian church titl toward the middle of the tenth century.</q> <rj><qau>Hoock.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being canonized or sainted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonize</ent><br/
<hw>Can"on*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Canonized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Canonizing</conjf>.]</vmorph> <ety>[F. <ets>canoniser</ets> or LL. <ets>canonizare</ets>, fr. L. <ets>canon</ets>.. See <er>Canon</er>.]</ety> <sn>1.</sn> <fld>(Eccl.)</fld> <def>To declare (a deceased person) a saint; to put in the catalogue of saints; <as>as, Thomas a Becket was <ex>canonized</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To glorify; to exalt to the highest honor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fame in time to come <qex>canonize</qex> us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rate as inspired; to include in the canon.</def><mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonry</ent><br/
<hw>Can"on*ry</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <er>Canonries</er> <pr>(<?/)</pr>. <def>A benefice or prebend in a cathedral or collegiate church; a right to a place in chapter and to a portion of its revenues; the dignity or emoluments of a canon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canonship</ent><br/
<hw>Can"on*ship</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Canopus in Egypt; <as>as, the <ex>Canopic</ex> vases, used in embalming</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>canopic jar</ent><br/
<hw>ca*no"pic jar`</hw> <pos>n.</pos> <def>a jar used in ancient Egyptian tombs to contain the intestines of a person who was mummified for burial; -- also called <altname>canopic vase</altname>.</def> <altsp>[Also spelled <asp>Canopic jar</asp>.]</altsp><br/
[<source>PJC</source>]</p>

<p><ent>canopied</ent><br/
<hw>canopied</hw> <pos>adj.</pos> <def>covered with or as with a canopy; <as>as, a <ex>canopied</ex> bed; streets <ex>canopied</ex> by stately trees</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Canopus</ent><br/
||<hw>Ca*no"pus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Canopus</ets>, fr. Gr. <?/, town of Egypt.]</ety> <fld>(Astron.)</fld> <def>A star of the first magnitude in the southern constellation Argo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canopy</ent><br/
<hw>Can"o*py</hw> <pr>(k<acr/n"<osl/*p<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canopies</plw> <pr>(-p<icr/z)</pr>.</plu> <ety>[OE. <ets>canapie</ets>, F. <ets>canap<eacute/</ets> sofa, OF. <ets>conop<eacute/e</ets>, <ets>conopeu</ets>, <ets>conopieu</ets>, canopy, vail, pavilion (cf. It. <ets>canop<egrave/</ets> canopy, sofa), LL. <ets>conopeum</ets> a bed with mosquito curtains, fr. Gr. <grk>kwnwpei^on</grk>, fr. <grk>kw`nwps</grk> gnat, <grk>kw`nos</grk> cone + <grk>'w`ps</grk> face. See <er>Cone</er>, and <er>Optic</er>.]</ety> <sn>1.</sn> <def>A covering fixed over a bed, dais, or the like, or carried on poles over an exalted personage or a sacred object, etc. chiefly as a mark of honor.</def> <ldquo/Golden <xex>canopies</xex> and beds of state.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>An ornamental projection, over a door, window, niche, etc.</def> <sd>(b)</sd> <def>Also, a rooflike covering, supported on pillars over an altar, a statue, a fountain, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canopy</ent><br/
<hw>Can"o*py</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Canopes</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Canopying</conjf>.]</vmorph> <def>To cover with, or as with, a canopy.</def> <ldquo/A bank with ivy <xex>canopied</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canorous</ent><br/
<hw>Ca*no"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>canorus</ets>, from <ets>nor</ets> melody, fr. <ets>canere</ets> to sing.]</ety> <def>Melodious; musical.</def> <ldquo/Birds that are most <xex>canorous</xex>.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A long, lound, and <qex>canorous</qex> peal of laughter.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canorousness</ent><br/
<hw>Ca*no"rous*ness</hw>, <pos>n.</pos> <def>The quality of being musical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He chooses his language for its rich <qex>canorousness</qex>.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canstick</ent><br/
<hw>Can"stick`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Candlestick.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF., edge, angle, prof. from L. <ets>canthus</ets> the iron ring round a carriage wheel, a wheel, Gr. <?/ the corner of the eye, the felly of a wheel; cf. W. <ets>cant</ets> the stake or tire of a wheel. Cf. <er>Canthus</er>, <er>Canton</er>, <er>Cantle</er>.]</ety> <sn>1.</sn> <def>A corner; angle; niche.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The first and principal person in the temple was Irene, or Peace; she was placed aloft in a <qex>cant</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An outer or external angle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An inclination from a horizontal or vertical line; a slope or bevel; a titl.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sudden thrust, push, kick, or other impulse, producing a bias or change of direction; also, the bias or turn so give; <as>as, to give a ball a <ex>cant</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Coopering)</fld> <def>A segment forming a side piece in the head of a cask.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mech.)</fld> <def>A segment of he rim of a wooden cogwheel.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>A piece of wood laid upon the deck of a vessel to support the bulkheads.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cant frames</b></col>, <col><b>Cant timbers</b></col></mcol> <fld>(Naut.)</fld>, <cd>timber at the two ends of a ship, rising obliquely from the keel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Canted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Canting</conjf>.]</vmorph> <sn>1.</sn> <def>To incline; to set at an angle; to tilt over; to tip upon the edge; <as>as, to <ex>cant</ex> a cask; to <ex>cant</ex> a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give a sudden turn or new direction to; <as>as, to <ex>cant</ex> round a stick of timber; to <ex>cant</ex> a football</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut off an angle from, as from a square piece of timber, or from the head of a bolt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw>, <pos>n.</pos> <ety>[Prob. from OF. <ets>cant</ets>, F. <ets>chant</ets>, singing, in allusion to the singing or whining tine of voice used by beggars, fr. L. <ets>cantus</ets>. See <er>Chant</er>.]</ety> <sn>1.</sn> <def>An affected, singsong mode of speaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The idioms and peculiarities of speech in any sect, class, or occupation.</def>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cant</qex> of any profession.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The use of religious phraseology without understanding or sincerity; empty, solemn speech, implying what is not felt; hypocrisy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall hear no <qex>cant</qex> from me.</q> <rj><qau>F. W. Robertson</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Vulgar jargon; slang; the secret language spoker by gipsies, thieves, tramps, or beggars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of cant; affected; vulgar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To introduce and multiply <qex>cant</qex> words in the most ruinous corruption in any language.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To speak in a whining voice, or an affected, singsong tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make whining pretensions to goodness; to talk with an affectation of religion, philanthropy, etc.; to practice hypocrisy; <as>as, a <ex>canting</ex> fanatic</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rankest rogue that ever <qex>canted</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To use pretentious language, barbarous jargon, or technical terms; to talk with an affectation of learning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The doctor here,<br/
When he discourseth of dissection,<br/
Of vena cava and of vena porta,<br/
The meser<ae/um and the mesentericum,<br/
What does he else but <qex>cant</qex>.</q> <rj><qau>B. Jonson</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That uncouth affected garb of speech, or <qex>canting</qex> language, if I may so call it.</q> <rj><qau>Bp. Sanderson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw>, <pos>n.</pos> <ety>[Prob. from OF. <ets>cant</ets>, equiv. to L. <ets>quantum</ets>; cf. F. <ets>encan</ets>, fr. L. <ets>in quantum</ets>, i.e. <ldquo/for how much?<rdquo/]</ety> <def>A call for bidders at a public sale; an auction.</def> <ldquo/To sell their leases by <xex>cant</xex>.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant</ent><br/
<hw>Cant</hw>, <pos>v. t.</pos> <def>to sell by auction, or bid a price at a sale by auction.</def> <mark>[Archaic]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Can't</ent><br/
<hw>Can't</hw> <pr>(?)</pr>. <def>A colloquial contraction for <xex>can not</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantab</ent><br/
<hw>Can"tab</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbreviated from <ets>Cantabrigian</ets>.]</ety> <def>A Cantabrigian.</def> <mark>[Colloq.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantabile</ent><br/
||<hw>Can*ta"bi*le</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It., <ets>cantare</ets> to sing.]</ety> <fld>(Mus.)</fld> <def>In a melodious, flowing style; in a singing style, as opposed to <xex>bravura</xex>, <xex>recitativo</xex>, or <xex>parlando</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantabile</ent><br/
||<hw>Can*ta"bi*le</hw>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A piece or passage, whether vocal or instrumental, peculiarly adapted to singing; -- sometimes called <altname>cantilena</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantabrian</ent><br/
<hw>Can*ta"bri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Cantabria on the Bay of Biscay in Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantabrigian</ent><br/
<hw>Can`ta*brig"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A native or resident of Cambridge; esp. a student or graduate of the university of Cambridge, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantalever</ent><br/
<hw>Can"ta*lev`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cant</ets> an external angle + <ets>lever</ets> a supporter of the roof timber of a house.]</ety> <altsp>[Written also <asp>cantaliver</asp> and <asp>cantilever</asp>.]</altsp> <sn>1.</sn> <fld>(Arch.)</fld> <def>A bracket to support a balcony, a cornice, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Engin.)</fld> <def>A projecting beam, truss, or bridge unsupported at the outer end; one which overhangs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cantalever bridge</b></col>, <cd>a bridge in which the principle of the cantalever is applied. It is usually a trussed bridge, composed of two portions reaching out from opposite banks, and supported near the middle of their own length on piers which they overhang, thus forming cantalevers which meet over the space to be spanned or sustain a third portion, to complete the connection.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantaloupe</ent><br/
<hw>Can"ta*loupe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cantaloup</ets>, It. <ets>cantalupo</ets>, so called from the caste of <ets>Cantalupo</ets>, in the Marca d'Ancona, in Italy, where they were first grown in Europe, from seed said to have been imported from Armenia.]</ety> <def>A muskmelon of several varieties, having when mature, a yellowish skin, and flesh of a reddish orange color.</def> <altsp>[Written also <asp>cantaleup</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantankerous</ent><br/
<hw>Can*tan"ker*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Perverse; contentious; ugly; malicious.</def> <mark>[Colloq.]</mark> -- <wordforms><wf>Can*tan"ker*ous*ly</wf>, <pos>adv.</pos> -- <wf>Can*tan"ker*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cantankerous</qex> old maiden aunt.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantarro</ent><br/
<ent>Cantar</ent><br/
<mhw>{ <hw>Can"tar</hw> <pr>(?)</pr>, ||<hw>Can*tar"ro</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[It. <ets>cantaro</ets> (in sense 1), Sp. <ets>cantaro</ets> (in sense 2).]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A weight used in southern Europe and East for heavy articles. It varies in different localities; thus, at Rome it is nearly 75 pounds, in Sardinia nearly 94 pounds, in Cairo it is 95 pounds, in Syria about 503 pounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A liquid measure in Spain, ranging from two and a half to four gallons.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantata</ent><br/
||<hw>Can*ta"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., fr. <ets>cantare</ets> to sing, fr. L. <ets>cantare</ets> intens of <ets>canere</ets> to sing.]</ety> <fld>(Mus.)</fld> <def>A poem set to music; a musical composition comprising choruses, solos, interludes, etc., arranged in a somewhat dramatic manner; originally, a composition for a single noise, consisting of both recitative and melody.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantation</ent><br/
<hw>Can*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cantatio</ets>.]</ety> <def>A singing.</def> <mark>[Obs.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantatory</ent><br/
<hw>Cant"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing cant or affectation; whining; singing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantatrice</ent><br/
||<hw>Can`ta*tri"ce</hw> <pr>(k<adot/n`t<adot/*tr<emac/"ch<asl/)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A female professional singer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canted</ent><br/
<hw>Cant"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From 2d <er>Cant</er>.]</ety> <sn>1.</sn> <def>Having angles; <as>as, a six <ex>canted</ex> bolt head; a <ex>canted</ex> window</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canted column</b></col> <fld>(Arch.)</fld>, <cd>a column polygonal in plan.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inclined at an angle to something else; tipped; sloping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canteen</ent><br/
<hw>Can*teen"</hw> <pr>(k<acr/n*t<emac/n")</pr>, <pos>n.</pos> <ety>[F. <ets>cantine</ets> bottle case, canteen (cf. Sp. & It. <ets>cantina</ets> cellar, bottle case), either contr. fr. It. <ets>canovettina</ets>, dim. of <ets>canova</ets> cellar, or, more likely, fr. OF. <ets>cant</ets>. corner, It. & Sp. <ets>canto</ets>. See 1st <er>Cant</er>.]</ety> <fld>(Mil.)</fld> <sn>1.</sn> <def>A small vessel used by soldiers or hikers for carrying water, liquor, or other drink.</def> <altsp>[Written also <asp>cantine</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In 1910 in the English service the <ex>canteen</ex> is made of wood and holds three pints; in the United States it is usually a tin flask.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A chest containing culinary and other vessels for military officers in a garrison.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The sutler's shop in a garrison.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <specif>Hence:</specif> <def>A store or small shop within a larger establishment where refreshments and sometimes other supplies are sold.  At a military base the canteen may be as large as a general store; within a school or small company it may be only a small counter with very limited supplies, or a snack bar.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>A temporary location where food is dispensed during an emergency.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cantel</ent><br/
<hw>Can"tel</hw> <pr>(k<acr/n"t'l)</pr>, <pos>n.</pos> <def>See <er>Cantle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canter</ent><br/
<hw>Can"ter</hw> <pr>(k<acr/n"t<etil/r)</pr>, <pos>n.</pos> <ety>[An abbreviation of <ets>Canterbury</ets>. See <ets>Canterbury gallop</ets>, under <er>Canterbury</er>.]</ety> <sn>1.</sn> <def>A moderate and easy gallop adapted to pleasure riding.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>canter</xex> is a thoroughly artificial pace, at first extremely tiring to the horse, and generally only to be produced in him by the restraint of a powerful bit, which compels him to throw a great part of his weight on his haunches . . . There is so great a variety in the mode adopted by different horses for performing the canter, that no single description will suffice, nor indeed is it easy . . . to define any one of them.</note>  <rj><au>J. H. Walsh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 212 --></p>

<p><sn>2.</sn> <def>A rapid or easy passing over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A rapid <qex>canter</qex> in the Times over all the topics.</q> <rj><qau>Sir J. Stephen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canter</ent><br/
<hw>Can"ter</hw> <pr>(k<acr/n"t<etil/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cantered</conjf> <pr>(k<acr/n"t<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cantering</conjf>.]</vmorph> <def>To move in a canter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canter</ent><br/
<hw>Can"ter</hw>, <pos>v. t.</pos> <def>To cause, as a horse, to go at a canter; to ride (a horse) at a canter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canter</ent><br/
<hw>Cant"er</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who cants or whines; a beggar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who makes hypocritical pretensions to goodness; one who uses canting language.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The day when he was a <qex>canter</qex> and a rebel.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canterbury</ent><br/
<hw>Can"ter*bur*y</hw> <pr>(k<acr/n"t<etil/r*b<ecr/r*r<ycr/)</pr>, <pos>prop. n.</pos> <sn>1.</sn> <def>A city in <country>England</country>, giving its name various articles.  It is the seat of the <person>Archbishop of Canterbury</person> (primate of all England), and contains the shrine of <person>Thomas <agrave/ Becket</person>, to which pilgrimages were formerly made.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stand with divisions in it for holding music, loose papers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Canterbury ball</b></col> <fld>(Bot.)</fld>, <cd>a species of <gen>Campanula</gen> of several varieties, cultivated for its handsome bell-shaped flowers.</cd> -- <col><b>Canterbury gallop</b></col>, <cd>a gentle gallop such as was used by pilgrims riding to <city>Canterbury</city>; a canter.</cd> -- <col><b>Canterbury tale</b></col>, <cd>one of the tales which <persfn>Chaucer</persfn> puts into the mouths of certain pilgrims to <city>Canterbury</city>. Hence, any tale told by travelers to pass away the time.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantharidal</ent><br/
<hw>Can*thar"*i*dal</hw> <pr>(k<acr/n*th<acr/r"<icr/*d<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to cantharides or made of cantharides; <as>as, <ex>cantharidal</ex> plaster</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantharides</ent><br/
<hw>Can*thar"i*des</hw> <pr>(k<acr/n*th<acr/r"<icr/*d<emac/z)</pr>, <pos>n. pl.</pos> <def>See <er>Cantharis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantharidin</ent><br/
<hw>Can*thar"i*din</hw> <pr>(k<acr/n*th<acr/r"<icr/*d<icr/n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The active principle of the cantharis, or Spanish fly, a volatile, acrid, bitter solid, crystallizing in four-sided prisms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantharis</ent><br/
<hw>Can"tha*ris</hw> <pr>(k<acr/n"th<adot/*r<icr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cantharides</plw> <pr>(k<acr/n*th<acr/r"<icr/*d<emac/z)</pr>.</plu> <ety>[L., a kind of beetle, esp. the Spanish fly, Gr. <grk>kanqari`s</grk>.]</ety> <fld>(Zool.)</fld> <def>A beetle (<spn>Lytta  vesicatoria</spn>, <it>syn.</it> <spn>Cantharis vesicatoria</spn>), having an elongated cylindrical body of a brilliant green color, and a nauseous odor; the blister fly or blister beetle, of the apothecary; -- also called <altname>Spanish fly</altname>. Many other species of <gen>Lytta</gen>, used for the same purpose, take the same name. See <cref>Blister beetle</cref>, under <er>Blister</er>. The plural form in usually applied to the dried insects used in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cant hook</ent><br/
<hw>Cant" hook`</hw> <pr>(?)</pr>. <def>A wooden lever with a movable iron hook. hear the end; -- used for canting or turning over heavy logs, etc.</def> <mark>[U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canthoplasty</ent><br/
<hw>Can"tho*plas`ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr.<?/, corner of the eye + <?/ to from.]</ety> <fld>(Surg.)</fld> <def>The operation of forming a new canthus, when one has been destroyed by injury or disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canthus</ent><br/
||<hw>Can"thus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canthi</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/.]</ety> <fld>(Anat.)</fld> <def>The corner where the upper and under eyelids meet on each side of the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canticle</ent><br/
<hw>Can"ti*cle</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Canticles</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>canticulum</ets> a little song, dim. of <ets>canticum</ets> song, fr. <ets>cantus</ets> a singing, fr. <ets>coner</ets> to sing. See <er>Chant</er>.]</ety> <sn>1.</sn> <def>A song; esp. a little song or hymn.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl</pluf>. <def>The Song of Songs or Song of Solomon, one of the books of the Old Testament.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A canto or division of a poem</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A psalm, hymn, or passage from the Bible, arranged for chanting in church service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canticoy</ent><br/
<hw>Can"ti*coy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of American Indian origin.]</ety> <def>A social gathering; usually, one for dancing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantile</ent><br/
<hw>Can"tile</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>Same as <er>Cantle</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantilena</ent><br/
||<hw>Can`ti*le"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. & L.]</ety> <fld>(Mus.)</fld> <def>See <er>Cantabile</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantilever</ent><br/
<hw>Can"ti*lev`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Cantalever</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantillate</ent><br/
<hw>Can"til*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>cantillatus</ets>, p. p. of <ets>cantillare</ets> to sing low, dim. of <ets>cantare</ets>. See <er>Cantata</er>.]</ety> <def>To chant; to recite with musical tones.</def>  <rj><au>M. Stuart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantillation</ent><br/
<hw>Can`til*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chanting; recitation or reading with musical modulations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantine</ent><br/
<hw>Can*tine"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Canteen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canting</ent><br/
<hw>Cant"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Speaking in a whining tone of voice; using technical or religious terms affectedly; affectedly pious; <as>as, a <ex>canting</ex> rogue; a <ex>canting</ex> tone</as>.</def></p>

<p>-- <wordforms><wf>Cant"ing*ly</wf>, <pos>adv.</pos> -- <wf>Cant"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Canting arms</b></col>, <col><b>Canting heraldry</b></col></mcol> <fld>(Her.)</fld>, <cd>bearings in the nature of a rebus alluding to the name of the bearer. Thus, the <xex>Castletons</xex> bear three castles, and Pope Adrian IV. (Nicholas Breakspeare) bore a broken spear.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canting</ent><br/
<hw>Cant"ing</hw>, <pos>n.</pos> <def>The use of cant; hypocrisy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantiniere</ent><br/
||<hw>Can`ti*niere"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>cantine</ets> a sutler's shop, canteen.]</ety> <fld>(Mil)</fld> <def>A woman who carries a canteen for soldiers; a vivandi<egrave/re.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantion</ent><br/
<hw>Can"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cantio</ets>, from <ets>canere</ets> to sing.]</ety> <def>A song or verses.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantle</ent><br/
<hw>Can"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cantel</ets>, <ets>chantel</ets>, corner, side, piece, F. <ets>chanteau</ets> a piece cut from a larger piece, dim. of OF. <ets>cant</ets> edge, corner. See 1st <er>Cant</er>.]</ety> <sn>1.</sn> <def>A corner or edge of anything; a piece; a fragment; a part.</def> <ldquo/In one <xex>cantle</xex> of his law.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cuts me from the best of all my land<br/
A huge half moon, a monstrous <qex>cantle</qex> out.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The upwardly projecting rear part of saddle, opposite to the pommel.</def> <altsp>[Written also <asp>cante</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantle</ent><br/
<hw>Can"tle</hw>, <pos>v. t.</pos> <def>To cut in pieces; to cut out from.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>cantile</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantlet</ent><br/
<hw>Cant"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>cantle</ets>.]</ety> <def>A piece; a fragment; a corner.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canto</ent><br/
<hw>Can"to</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cantos</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>canto</ets>, fr. L. <ets>cantus</ets> singing, song. See <er>Chant</er>.]</ety> <sn>1.</sn> <def>One of the chief divisions of a long poem; a book.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The highest vocal part; the air or melody in choral music; anciently the tenor, now the soprano.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Canto fermo</b></col> <pr>(<?/)</pr> <ety>[It.]</ety> <fld>(Mus.)</fld>, <cd>the plain ecclesiastical chant in cathedral service; the plain song.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canton</ent><br/
<hw>Can"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A song or canto</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Write loyal <qex>cantons</qex> of contemned love.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canton</ent><br/
<hw>Can"ton</hw>, <pos>n.</pos> <ety>[F. <ets>canton</ets>, augm. of OF. <ets>cant</ets> edge, corner. See 1st <er>Cant</er>.]</ety> <sn>1.</sn> <def>A small portion; a division; a compartment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That little <qex>canton</qex> of land called the <ldquo/English pale<rdquo/</q> <rj><qau>Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is another piece of Holbein's, . . . in which, in six several <qex>cantons</qex>, the several parts of our Savior's passion are represented.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small community or clan.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small territorial district; esp. one of the twenty-two independent states which form the Swiss federal republic; in France, a subdivision of an arrondissement. See <er>Arrondissement</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Her.)</fld> <def>A division of a shield occupying one third part of the chief, usually on the dexter side, formed by a perpendicular line from the top of the shield, meeting a horizontal line from the side.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king gave us the arms of England to be borne in a <qex>canton</qex> in our arms.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canton</ent><br/
<hw>Can"ton</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cantoned</conjf> ; <pos>p. pr. & vb. n.</pos> <conjf>Cantoning</conjf>.]</vmorph> <ety>[Cf. F.<ets>cantonner</ets>.]</ety> <sn>1.</sn> <def>To divide into small parts or districts; to mark off or separate, as a distinct portion or division.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>canton</qex> out themselves a little Goshen in the intellectual world.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>To allot separate quarters to, as to different parts or divisions of an army or body of troops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantonal</ent><br/
<hw>Can"ton*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a canton or cantons; of the nature of a canton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canton crape</ent><br/
<hw>Can"ton crape"</hw> <pr>(kr<amac/p")</pr>. <def>A soft, white or colored silk fabric, of a gauzy texture and wavy appearance, used for ladies' scarfs, shawls, bonnet trimmings, etc.; -- called also <altname>Oriental crape</altname>.</def>  <rj><au>De Colange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantoned</ent><br/
<hw>Can"toned</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Her.)</fld> <def>Having a charge in each of the four corners; -- said of a cross on a shield, and also of the shield itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>Having the angles marked by, or decorated with, projecting moldings or small columns; <as>as, a <ex>cantoned</ex> pier or pilaster</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canton flannel</ent><br/
<hw>Can"ton flan"nel</hw> <pr>(?)</pr>. <def>See <er>Cotton flannel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantonize</ent><br/
<hw>Can"ton*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To divide into cantons or small districts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantonment</ent><br/
<hw>Can"ton*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cantonnement</ets>.]</ety> <def>A town or village, or part of a town or village, assigned to a body of troops for quarters; temporary shelter or place of rest for an army; quarters.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When troops are sheltered in huts or quartered in the houses of the people during any suspension of hostilities, they are said to be in <xex>cantonment</xex>, or to be cantoned. In India, permanent military stations, or military towns, are termed <xex>cantonments</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantoon</ent><br/
<hw>Can*toon"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cotton stuff showing a fine cord on one side and a satiny surface on the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantor</ent><br/
<hw>Can"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a singer, fr. <ets>caner</ets> to sing.]</ety> <def>A singer; esp. the leader of a church choir; a precentor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cantor</qex> of the church intones the Te Deum.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantoral</ent><br/
<hw>Can"tor*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or belonging to a cantor.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cantoral staff</b></col>, <cd>the official staff or baton of a cantor or precentor, with which time is marked for the singers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantoris</ent><br/
<hw>Can*to"ris</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L., lit., of the cantor, gen. of <ets>cantor</ets>.]</ety> <def>Of or pertaining to a cantor; <as>as, the <ex>cantoris</ex> side of a choir; a <ex>cantoris</ex> stall</as>.</def>  <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantrip</ent><br/
<ent>Cantrap</ent><br/
<mhw>{ <hw>Can"trap</hw> <pr>(?)</pr>, <hw>Can"trip</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. Icel. <ets>gandar</ets>, ODan. & OSw. <ets>gan</ets>, witchcraft, and E. <ets>trap</ets> a snare, <ets>tramp</ets>.]</ety> <def>A charm; an incantation; a shell; a trick; adroit mischief.</def> <altsp>[Written also <asp>cantraip</asp>.]</altsp> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cantref</ent><br/
<ent>Cantred</ent><br/
<mhw>{ <hw>Can"tred</hw> <pr>(?)</pr>, ||<hw>Can"tref</hw>, }</mhw> <pos>n.</pos> <ety>[W. <ets>cantref</ets>; <ets>cant</ets> hundred + <ets>tref</ets> dwelling place, village.]</ety> <def>A district comprising a hundred villages, as in Wales.</def> <altsp>[Written also <asp>kantry</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canty</ent><br/
<hw>Can"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cheerful; sprightly; lively; merry.</def> <ldquo/The <xex>canty</xex> dame.<rdquo/  <rj><au>Wordsworth <mark>[Scot. & Prov. Eng.]</mark></au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Contented with little, and <qex>canty</qex> with mair.</q> <rj><qau>Burns.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canuck</ent><br/
<hw>Ca*nuck"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A Canadian.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small or medium-sized hardy horse, common in Canada.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canulated</ent><br/
<ent>Canular</ent><br/
<ent>Canula</ent><br/
<mhw>{ <hw>Can"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Can"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos>, <hw>Can"u*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>See <er>Cannula</er>, <er>Cannular</er>, and <er>Cannulated</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvas</ent><br/
<hw>Can"vas</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>canvas</ets>, <ets>canevas</ets>, F. <ets>canevas</ets>, LL. <ets>canabacius</ets> hempen cloth, canvas, L. <ets>cannabis</ets> hemp, fr. G. <?/. See <er>Hemp</er>.]</ety> <sn>1.</sn> <def>A strong cloth made of hemp, flax, or cotton; -- used for tents, sails, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By glimmering lanes and walls of <qex>canvas</qex> led.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>A coarse cloth so woven as to form regular meshes for working with the needle, as in tapestry, or worsted work.</def> <sd>(b)</sd> <def>A piece of strong cloth of which the surface has been prepared to receive painting, commonly painting in oil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>History . . . does not bring out clearly upon the <qex>canvas</qex> the details which were familiar.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something for which canvas is used: <sd>(a)</sd> A sail, or a collection of sails. <sd>(b)</sd> A tent, or a collection of tents. <sd>(c)</sd> A painting, or a picture on canvas.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To suit his <qex>canvas</qex> to the roughness of the see.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Light, rich as that which glows on the <qex>canvas</qex> of Claude.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A rough draft or model of a song, air, or other literary or musical composition; esp. one to show a poet the measure of the verses he is to make.</def>  <rj><au>Grabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvas</ent><br/
<hw>Can"vas</hw>, <pos>a.</pos> <def>Made of, pertaining to, or resembling, canvas or coarse cloth; <as>as, a <ex>canvas</ex> tent</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvasback</ent><br/
<hw>Can"vas*back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A Species of duck (<spn>Aythya vallisneria</spn>), esteemed for the delicacy of its flesh. It visits the United States in autumn; particularly Chesapeake Bay and adjoining waters; -- so named from the markings of the plumage on its back.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvass</ent><br/
<hw>Can"vass</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>canvassed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Canvassing</conjf>.]</vmorph> <ety>[OF. <ets>Canabasser</ets> to examine curiously, to search or sift out; properly, to sift through canvas. See <er>Canvas</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To sift; to strain; to examine thoroughly; to scrutinize; <as>as, to <ex>canvass</ex> the votes cast at an election; to <ex>canvass</ex> a district with reference to its probable vote</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have made careful search on all hands, and <qex>canvassed</qex> the matter with all possible diligence.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To examine by discussion; to debate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An opinion that we are likely soon to <qex>canvass</qex>.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To go through, with personal solicitation or public addresses; <as>as, to <ex>canvass</ex> a district for votes; to <ex>canvass</ex> a city for subscriptions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvass</ent><br/
<hw>Can"vass</hw>, <pos>v. i.</pos> <def>To search thoroughly; to engage in solicitation by traversing a district; <as>as, to <ex>canvass</ex> for subscriptions or for votes; to <ex>canvass</ex> for a book, a publisher, or in behalf of a charity</as>; -- commonly followed by <xex>for</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvass</ent><br/
<hw>Can"vass</hw>, <pos>n.</pos> <sn>1.</sn> <def>Close inspection; careful review for verification; <as>as, a <ex>canvass</ex> of votes</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Examination in the way of discussion or debate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Search; exploration; solicitation; systematic effort to obtain votes, subscribers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No previous <qex>canvass</qex> was made for me.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canvasser</ent><br/
<hw>Can"vass*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who canvasses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cany</ent><br/
<hw>Can"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cane</er>.]</ety> <def>Of or pertaining to cane or canes; abounding with canes.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canyon</ent><br/
<hw>Can"yon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The English form of the Spanish word <er>Ca<ntil/on</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canzone</ent><br/
||<hw>Can*zo"ne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., a song, fr. L. <ets>cantio</ets>, fr. <ets>canere</ets> to sing. Cf. <er>Chanson</er>, <er>Chant</er>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A song or air for one or more voices, of Proven<cced/al origin, resembling, though not strictly, the madrigal.</def> <sd>(b)</sd> <def>An instrumental piece in the madrigal style.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Canzonet</ent><br/
<hw>Can`zo*net"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>canzonetta</ets>, dim. of <ets>canzone</ets>.]</ety> <fld>(Mus.)</fld> <def>A short song, in one or more parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caoutchin</ent><br/
<hw>Caout"chin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An inflammable, volatile, oily, liquid hydrocarbon, obtained by the destructive distillation of caoutchouc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caoutchouc</ent><br/
<hw>Caout"chouc</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caoutchouc</ets>, from the South American name.]</ety> <def>A tenacious, elastic, gummy substance obtained from the milky sap of several plants of tropical South America (esp. the euphorbiaceous tree <spn>Siphonia elastica</spn> or <spn>Hevea caoutchouc</spn>), Asia, and Africa. Being impermeable to liquids and gases, and not readly affected by exposure to air, acids, and alkalies, it is used, especially when vulcanized, for many purposes in the arts and in manufactures. Also called <altname>India rubber</altname> (because it was first brought from India, and was formerly used chiefly for erasing pencil marks) and <altname>gum elastic</altname>. See <er>Vulcanization</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Mineral caoutchouc</b></col>. <cd>See under <er>Mineral</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caoutchoucin</ent><br/
<hw>Caout"chou*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caoutchin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cap</ent><br/
<hw>Cap</hw> <pr>(k<acr/p)</pr>, <pos>n.</pos> <ety>[OE. <ets>cappe</ets>, AS. <ets>c<ae/ppe</ets>, cap, cape, hood, fr. LL, <ets>cappa</ets>, <ets>capa</ets>; perhaps of Iberian origin, as Isidorus of Seville mentions it first: <ldquo/<ets>Capa</ets>, quia quasi totum capiat hominem; it. capitis ornamentum.<rdquo/ See 3d <er>Cape</er>, and cf. 1st <er>Cope</er>.]</ety> <sn>1.</sn> <def>A covering for the head</def>; esp. <sd>(a)</sd> <def>One usually with a visor but without a brim, for men and boys</def>; <sd>(b)</sd> <def>One of lace, muslin, etc., for women, or infants</def>; <sd>(c)</sd> <def>One used as the mark or ensign of some rank, office, or dignity, as that of a cardinal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The top, or uppermost part; the chief.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art the <qex>cap</qex> of all the fools alive.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A respectful uncovering of the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that will give a <qex>cap</qex> and make a leg in thanks.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The whole top of the head of a bird from the base of the bill to the nape of the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything resembling a cap in form, position, or use</def>; as: <sd>(a)</sd> <fld>(Arch.)</fld> <def>The uppermost of any assemblage of parts; <as>as, the <ex>cap</ex> of column, door, etc.</as>; a capital, coping, cornice, lintel, or plate.</def> <sd>(b)</sd> <def>Something covering the top or end of a thing for protection or ornament.</def> <sd>(c)</sd> <fld>(Naut.)</fld> <def>A collar of iron or wood used in joining spars, as the mast and the topmast, the bowsprit and the jib boom; also, a covering of tarred canvas at the end of a rope.</def> <sd>(d)</sd> <def>A percussion cap. See under <er>Percussion</er>.</def> <sd>(e)</sd> <fld>(Mech.)</fld> <def>The removable cover of a journal box.</def> <sd>(f)</sd> <fld>(Geom.)</fld> <def>A portion of a spherical or other convex surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A large size of writing paper; <as>as, flat <ex>cap</ex>; fools<ex>cap</ex>; legal <ex>cap</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cap of a cannon</b></col>, <cd>a piece of lead laid over the vent to keep the priming dry; -- now called an <xex>apron</xex>.</cd> -- <col><b>Cap in hand</b></col>, <cd>obsequiously; submissively.</cd> -- <col><b>Cap of liberty</b></col>. <cd>See <cref>Liberty cap</cref>, under <er>Liberty</er>.</cd> -- <col><b>Cap of maintenance</b></col>, <cd>a cap of state carried before the kings of England at the coronation. It is also carried before the mayors of some cities.</cd> -- <col><b>Cap money</b></col>, <cd>money collected in a cap for the huntsman at the death of the fox.</cd> -- <col><b>Cap paper</b></col>. <sd>(a)</sd> <cd>A kind of writing paper including flat cap, foolscap, and legal cap.</cd> <sd>(b)</sd> <cd>A coarse wrapping paper used for making caps to hold commodities.</cd> -- <col><b>Cap rock</b></col> <fld>(Mining)</fld>, <cd>The layer of rock next overlying ore, generally of barren vein material.</cd> -- <col><b>Flat cap</b></col>, <cd>cap See <er>Foolscap</er>.</cd> -- <col><b>Forage cap</b></col>, <cd>the cloth undress head covering of an officer of soldier.</cd> -- <col><b>Legal cap</b></col>, <cd>a kind of folio writing paper, made for the use of lawyers, in long narrow sheets which have the fold at the top or <ldquo/narrow edge.<rdquo/</cd> -- <col><b>To set one's cap</b></col>, <cd>to make a fool of one.</cd> <mark>(Obs.)</mark> <au>Chaucer.</au> -- <col><b>To set one's cap for</b></col>, <cd>to try to win the favor of a man with a view to marriage.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 213 --></p>

<p><ent>Cap</ent><br/
<hw>Cap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capped</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Capping</conjf>.]</vmorph> <sn>1.</sn> <def>To cover with a cap, or as with a cap; to provide with a cap or cover; to cover the top or end of; to place a cap upon the proper part of; <as>as, to <ex>cap</ex> a post; to <ex>cap</ex> a gun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bones next the joint are <qex>capped</qex> with a smooth cartilaginous substance.</q> <rj><qau>Derham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deprive of cap.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To complete; to crown; to bring to the highest point or consummation; <as>as, to <ex>cap</ex> the climax of absurdity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To salute by removing the cap.</def> <mark>[Slang. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Tom . . . <qex>capped</qex> the proctor with the profoundest of bows.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To match; to mate in contest; to furnish a complement to; <as>as, to <ex>cap</ex> text; to <ex>cap</ex> proverbs</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now I have him under girdle I'll <qex>cap</qex> verses with him to the end of the chapter.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In <xex>capping</xex> verses, when one quotes a verse another must <xex>cap</xex> it by quoting one beginning with the last letter of the first letter, or with the first letter of the last word, or ending with a rhyming word, or by applying any other arbitrary rule may be agreed upon.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cap</ent><br/
<hw>Cap</hw>, <pos>v. i.</pos> <def>To uncover the head respectfully.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capability</ent><br/
<hw>Ca`pa*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Capabilities</plw> <pr>(#)</pr>.</plu> <sn>1.</sn> <def>The quality of being capable; capacity; capableness; esp. intellectual power or ability.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>capability</qex> to take a thousand views of a subject.</q> <rj><qau>H. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capacity of being used or improved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capable</ent><br/
<hw>Ca"pa*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>capable</ets>, LL. <ets>capabilis</ets> capacious, capable, fr. L. <ets>caper</ets> to take, contain. See <er>Heave</er>.]</ety> <sn>1.</sn> <def>Possessing ability, qualification, or susceptibility; having capacity; of sufficient size or strength; <as>as, a room <ex>capable</ex> of holding a large number; a castle <ex>capable</ex> of resisting a long assault</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Concious of joy and <qex>capable</qex> of pain.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Possessing adequate power; qualified; able; fully competent; <as>as, a <ex>capable</ex> instructor; a <ex>capable</ex> judge; a mind <ex>capable</ex> of nice investigations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>More <qex>capable</qex> to discourse of battles than to give them.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Possessing legal power or capacity; <as>as, a man <ex>capable</ex> of making a contract, or a will</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Capacious; large; comprehensive.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Capable</xex> is usually followed by <xex>of</xex>, sometimes by an infinitive.</note></p>

<p><syn><b>Syn.</b> -- Able; competent; qualified; fitted; efficient; effective; skillful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capableness</ent><br/
<hw>Ca"pa*ble*ness</hw>, <pos>n.</pos> <def>The quality or state of being capable; capability; adequateness; competency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>capaciate</ent><br/
<hw>capaciate</hw> <pos>v. i.</pos> <fld>(Biol.)</fld> <def>to become active so as to be able to penetrate an ovum; -- of sperm, in the female reproduction system.</def> <-- ##?? check --><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Capacify</ent><br/
<hw>Ca*pac"i*fy</hw> <pr>(k<adot/*p<acr/s"<icr/*f<imac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capacified</conjf> <pr>(k<adot/*p<acr/s"<icr/*f<imac/d)</pr>.]</vmorph> <ety>[L. <ets>capax</ets>, <ets>-acis</ets>, capacious + <ets>-fy</ets>.]</ety> <def>To quality.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The benefice he is <qex>capacified</qex> and designed for.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capacious</ent><br/
<hw>Ca*pa"cious</hw> <pr>(k<adot/*p<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>capax</ets>, <ets>-acis</ets>, fr. <ets>capere</ets> to take. See <er>Heave</er>.]</ety> <sn>1.</sn> <def>Having capacity; able to contain much; large; roomy; spacious; extended; broad; <as>as, a <ex>capacious</ex> vessel, room, bay, or harbor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>capacious</qex> recesses of his mind.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Able or qualified to make large views of things, as in obtaining knowledge or forming designs; comprehensive; liberal.</def> <ldquo/A <xex>capacious</xex> mind.<rdquo/  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capaciously</ent><br/
<hw>Ca*pa"cious*ly</hw>, <pos>adv.</pos> <def>In a capacious manner or degree; comprehensively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capaciousness</ent><br/
<hw>Ca*pa"cious*ness</hw>, <pos>n.</pos> <def>The quality of being capacious, as of a vessel, a reservoir a bay, the mind, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>capacitance</ent><br/
<hw>ca*pac"i*tance</hw> <pr>(k<adot/*p<acr/s"<icr/*t<ait/ns)</pr>, <pos>n.</pos> <sn>1.</sn> <def>an electrical phenomenon whereby an electric charge is stored.</def><br/
<syn><b>Syn. --</b> electrical capacity, capacity.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Electronics)</fld> <def>a measure of the ability of a capacitor to store electrical charge; the ratio of the charge on one plate of a capacitor to the potential difference between the plates.</def><br/
[<source>PJC</source>]</p>

<p><ent>Capacitate</ent><br/
<hw>Ca*pac"i*tate</hw> <pr>(k<adot/*p<acr/s"<icr/*t<amac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capacitated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Capacitating</conjf>.]</vmorph> <def>To render capable; to enable; to qualify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By this instruction we may be <qex>capaciated</qex> to observe those errors.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>capacitive</ent><br/
<hw>ca*pac"i*tive</hw> <pr>(k<adot/*p<acr/s"<icr/*t<icr/v)</pr>, <pos>adj.</pos> <def>of or pertaining to capacitance.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>capacitor</ent><br/
<hw>ca*pac"i*tor</hw> <pr>(k<adot/*p<acr/s"<icr/*t<etil/r)</pr>, <pos>n.</pos> <def>a device used in electronic circuits to hold electrical charge, consisting of two conducting plates separated by a nonconducting (dielectric) medium; it is characterized by its capacitance.</def><br/
<syn><b>Syn. --</b> condenser, electrical condenser.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Capacity</ent><br/
<hw>Ca*pac"i*ty</hw> <pr>(k<adot/*p<acr/s"<icr/*t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Capacities</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[L. <ets>capacitus</ets>, fr. <ets>capax</ets>, <ets>capacis</ets>; fr. F. <ets>capacit<eacute/</ets>. See <er>Capacious</er>.]</ety> <sn>1.</sn> <def>The power of receiving or containing; extent of room or space; passive power; -- used in reference to physical things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Had our great palace the <qex>capacity</qex><br/
To camp this host, we all would sup together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>capacity</qex> of the exhausted cylinder.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The power of receiving and holding ideas, knowledge, etc.; the comprehensiveness of the mind; the receptive faculty; capability of understanding or feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Capacity</qex> is now properly limited to these [the mere passive operations of the mind]; its primary signification, which is literally <qex>room for</qex>, as well as its employment, favors this; although it can not be denied that there are examples of its usage in an active sense.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Ability; power pertaining to, or resulting from, the possession of strength, wealth, or talent; possibility of being or of doing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>capacity</qex> of blessing the people.</q> <rj><qau>Alex. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A cause with such <qex>capacities</qex> endued.</q> <rj><qau>Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Outward condition or circumstances; occupation; profession; character; position; <as>as, to work in the <ex>capacity</ex> of a mason or a carpenter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>Legal or moral qualification, as of age, residence, character, etc., necessary for certain purposes, as for holding office, for marrying, for making contracts, wills, etc.; legal power or right; competency.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capacity for heat</b></col>, <cd>the power of absorbing heat. Substances differ in the amount of heat requisite to raise them a given number of thermometric degrees, and this difference is the measure of, or depends upon, what is called their <ex>capacity for heat</ex>. See <cref>Specific heat</cref>, under <er>Heat</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Ability; faculty; talent; capability; skill; efficiency; cleverness. See <er>Ability</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capape</ent><br/
<hw>Cap`*a*pe"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>See <er>Cap-a-pie</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capapie</ent><br/
||<hw>Cap`*a*pie"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OF. (<?/) <ets>cap-a-pie</ets>, from head to foot, now <ets>de pied en cap</ets> from foot to head; L. <ets>pes</ets> foot + <ets>caput</ets> head.]</ety> <def>From head to foot; at all points.</def> <ldquo/He was armed <xex>cap-a-pie</xex>.<rdquo/  <rj><au>Prescott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caparison</ent><br/
<hw>Ca*par"i*son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>capara<cced/on</ets>, fr. Sp. <ets>caparazon</ets> a cover for a saddle, coach, etc.; <ets>capa</ets> cloak, cover (fr. LL. <ets>capa</ets>, cf. LL. <ets>caparo</ets> also fr. <ets>capa</ets>) + the term. <ets>azon</ets>. See <er>Cap</er>.]</ety> <sn>1.</sn> <def>An ornamental covering or housing for a horse; the harness or trappings of a horse, taken collectively, especially when decorative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their horses clothed with rich <qex>caparison</qex>.</q> <rj><qau>Drylen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gay or rich clothing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My heart groans beneath the gay <qex>caparison</qex>.</q> <rj><qau>Smollett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caparison</ent><br/
<hw>Ca*par"i*son</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caparisoned</conjf> <pr>(?)</pr> <pos>p. pr. & vb. n.</pos> <conjf>Caparisoning</conjf>.]</vmorph> <ety>[Cf. F <ets>capara<cced/onner</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To cover with housings, as a horse; to harness or fit out with decorative trappings, as a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The steeds, <qex>caparisoned</qex> with purple, stand.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To adorn with rich dress; to dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>caparisoned</qex> like a man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caparro</ent><br/
||<hw>Ca*par"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native Indian name.]</ety> <fld>(Zool.)</fld> <def>A large South American monkey (<spn>Lagothrix Humboldtii</spn>), with prehensile tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capcase</ent><br/
<hw>Cap"case`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small traveling case or bandbox; formerly, a chest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>capcase</qex> for your linen and your plate.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cape</ent><br/
<hw>Cape</hw> <pr>(k<amac/p)</pr>, <pos>n.</pos> <ety>[F. <ets>cap</ets>, fr. It. <ets>capo</ets> head, cape, fr. L. <ets>caput</ets> heat, end, point. See <er>Chief</er>.]</ety> <def>A piece or point of land, extending beyond the adjacent coast into the sea or a lake; a promontory; a headland.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cape buffalo</b></col> <fld>(Zool.)</fld> <cd>a large and powerful buffalo of South Africa (<spn>Bubalus Caffer</spn>). It is said to be the most dangerous wild beast of Africa. See <er>Buffalo</er>, 2.</cd> -- <mcol><col><b>Cape jasmine</b></col>, <col><b>Cape jessamine</b></col></mcol>. <cd>See <er>Jasmine</er>.</cd> -- <col><b>Cape pigeon</b></col> <fld>(Zool.)</fld>, <cd>a petrel (<spn>Daptium Capense</spn>) common off the Cape of Good Hope. It is about the size of a pigeon.</cd> -- <col><b>Cape wine</b></col>, <cd>wine made in South Africa</cd> <mark>[Eng.]</mark> -- <col><b>The Cape</b></col>, <cd>the Cape of Good Hope, in the general sense of the southern extremity of Africa. Also used of Cape Horn, and, in New England, of Cape Cod.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cape</ent><br/
<hw>Cape</hw>, <pos>v. i.</pos> <fld>(Naut.)</fld> <def>To head or point; to keep a course; <as>as, the ship <ex>capes</ex> southwest by south</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cape</ent><br/
<hw>Cape</hw>, <pos>n.</pos> <ety>[OE. <ets>Cape</ets>, fr. F. <ets>cape</ets>; cf. LL. <ets>cappa</ets>. See <er>Cap</er>, and cf. 1st <er>Cope</er>, <er>Chape</er>.]</ety> <def>A sleeveless garment or part of a garment, hanging from the neck over the back, arms, and shoulders, but not reaching below the hips. See <er>Cloak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cape</ent><br/
<hw>Cape</hw>, <pos>v. i.</pos> <ety>[See <er>Gape</er>.]</ety> <def>To gape.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caple</ent><br/
<ent>Capel</ent><br/
<mhw>{ <hw>Ca"pel</hw> <pr>(k<amac/"p<ecr/l)</pr>, <hw>Ca"ple</hw> <pr>(-p'l)</pr> }</mhw>, <pos>n.</pos> <ety>[Icel. <ets>kapall</ets>; cf. L. <ets>caballus</ets>.]</ety> <def>A horse; a nag.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capel</ent><br/
<hw>Ca"pel</hw> <pr>(k<amac/"p<ecr/l)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>A composite stone (quartz, schorl, and hornblende) in the walls of tin and copper lodes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capelan</ent><br/
<hw>Cap"e*lan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Capelin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capelin</ent><br/
<hw>Cape"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>capelan</ets>, <ets>caplan</ets>.]</ety> <fld>(Zool.)</fld> <def>Either of two small marine fishes formerly classified in the family <fam>Salmonid<ae/</fam>, now within the smelt family <fam>Osmeridae</fam>: <spn>Mallotus villosus</spn>, very abundant on the coasts of Greenland, Iceland, Newfoundland, and Alaska; or <spn>Mallotus catervarius</spn>, found in the North Pacific. The Atlantic variety has been used as a bait for the cod.</def> <altsp>[Written also <asp>capelan</asp> and <asp>caplin</asp>.]</altsp><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><note><hand/ This fish, which is like a smelt, is called by the Spaniards <xex>anchova</xex>, and by the Portuguese <xex>capelina</xex>.  However the <er>anchovy</er> used as a food is a different fish.</note>  <rj><au>Fisheries of U. S. (1884).</au></rj><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Capeline</ent><br/
||<hw>Ca"pe*line`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>capella</ets>. See <er>Chapel</er>.]</ety> <fld>(Med.)</fld> <def>A hood-shaped bandage for the head, the shoulder, or the stump of an amputated limb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capella</ent><br/
<hw>Ca*pel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a little goat, dim. of <ets>caper</ets> a goat.]</ety> <fld>(Asrton.)</fld> <def>A brilliant star in the constellation Auriga.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capellane</ent><br/
<hw>Cap"el*lane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chaplain</er>.]</ety> <def>The curate of a chapel; a chaplain.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capelle</ent><br/
||<hw>Ca*pel"le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G.]</ety> <fld>(Mus.)</fld> <def>The private orchestra or band of a prince or of a church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capellet</ent><br/
<hw>Cap"el*let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>capelet</ets>.]</ety> <fld>(Far.)</fld> <def>A swelling, like a wen, on the point of the elbow (or the heel of the hock) of a horse, caused probably by bruises in lying down.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capellmeister</ent><br/
||<hw>Ca*pell"meis`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., fr. <ets>capelle</ets> chapel, private band of a prince + <ets>meister</ets> a master.]</ety> <def>The musical director in a royal or ducal chapel; a choir-master.</def> <altsp>[Written also <asp>kapellmeister</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caper</ent><br/
<hw>Ca"per</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capered</conjf> <pos>p. pr. & vb. n.</pos> <conjf>capering</conjf>.]</vmorph> <ety>[From older <ets>capreoll</ets> to caper, cf. F.  <ets>se cabrer</ets> to prance; all ultimately fr. L. <ets>caper</ets>, <ets>capra</ets>, goat. See <er>Capriole</er>.]</ety> <def>To leap or jump about in a sprightly manner; to cut capers; to skip; to spring; to prance; to dance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>capers</qex>, he dances, he has eyes of youth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caper</ent><br/
<hw>Ca"per</hw>, <pos>n.</pos> <def>A frolicsome leap or spring; a skip; a jump, as in mirth or dancing; a prank.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cut a caper</b></col>, <cd>to frolic; to make a sportive spring; to play a prank.</cd>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caper</ent><br/
<hw>Ca"per</hw>, <pos>n.</pos> <ety>[D. <ets>kaper</ets>.]</ety> <def>A vessel formerly used by the Dutch, privateer.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caper</ent><br/
<hw>Ca"per</hw>, <pos>n.</pos> <ety>[F. <ets>c<acir/pre</ets>, fr. L. <ets>capparis</ets>, Gr. <?/; cf. Ar. & Per. <ets>al-kabar</ets>.]</ety> <sn>1.</sn> <def>The pungent grayish green flower bud of the European and Oriental caper (<spn>Capparis spinosa</spn>), much used for pickles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Capparis</gen>; -- called also <altname>caper bush</altname>, <altname>caper tree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <spn>Capparis spinosa</spn> is a low prickly shrub of the Mediterranean coasts, with trailing branches and brilliant flowers; -- cultivated in the south of Europe for its buds. The <spn>Capparis sodada</spn> is an almost leafless spiny shrub of central Africa (Soudan), Arabia, and southern India, with edible berries.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bean caper</b></col>. <cd>See <er>Bran caper</er>, in the <er>Vocabulary</er>.</cd> -- <col><b>Caper sauce</b></col>, <cd>a kind of sauce or catchup made of capers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caperberry</ent><br/
<hw>Ca"per*ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The small olive-shaped berry of the European and Oriental caper, said to be used in pickles and as a condiment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The currantlike fruit of the African and Arabian caper (<spn>Capparis sodado</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caper tree</ent><br/
<ent>Caper bush</ent><br/
<mhw>{ <hw>Ca"per bush`</hw> <pr>(?)</pr>, <hw>Ca"per tree`</hw> <pr>(?)</pr>. }</mhw><def>See <er>Capper</er>, a plant, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capercally</ent><br/
<ent>Capercailzie</ent><br/
<mhw>{ <hw>Ca"per*cail`zie</hw> <pr>(?)</pr>, or <hw>Ca"per*cal`ly</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Gael, <ets>capulcoile</ets>.]</ety> <fld>(Zool.)</fld> <def>A species of black Old World grouse (<spn>Tetrao uragallus</spn>) of large size and fine flavor, found in northern Europe and formerly in Scotland; -- called also <altname>cock of the woods</altname> and <altname>horse of the wood</altname>.</def> <altsp>[Written also <asp>capercaillie</asp>, <asp>capercaili</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caperclaw</ent><br/
<hw>Ca"per*claw`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To treat with cruel playfulness, as a cat treats a mouse; to abuse.</def> <mark>[Obs.]</mark>  <rj><au>Birch.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caperer</ent><br/
<hw>Ca"per*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who capers, leaps, and skips about, or dances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The nimble <qex>caperer</qex> on the cord.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>capeweed</ent><br/
<hw>cape"weed</hw> <pos>n.</pos> <sn>1.</sn> <def>a European weed (<spn>Hypochaeris radicata</spn>) widely naturalized in North America, having yellow flower heads and leaves resembling a cat's ears.</def><br/
<syn><b>Syn. --</b> cat's-ear, cat's ear, California dandelion, gosmore.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <ety>[<ets>Cape</ets>, from the Cape of Good Hope, South Africa + <ets>weed</ets>.]</ety> <def>a type of low-growing, creeping plant (<spn>Arctotheca calendulaceae</spn>), used as a ground cover in warm climates.</def> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>Capful</ent><br/
<hw>Cap"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Capfuls</er> <pr>(<?/)</pr></plu>. <def>As much as will fill a cap.</def><br/
[<source>1913 Webster</source>] <source>+PJC</source></p>

<p><cs><col><b>A capful of wind</b></col> <fld>(Naut.)</fld>, <cd>a light puff of wind.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capias</ent><br/
||<hw>Ca"pi*as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. thou mayst take.]</ety> <fld>(Low)</fld> <def>A writ or process commanding the officer to take the body of the person named in it, that is, to arrest him; -- also called <altname>writ of capias</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ One principal kind of <xex>capias</xex> is a writ by which actions at law are frequently commenced; another is a writ of execution issued after judgment to satisfy damages recovered; a <xex>capias</xex> in criminal law is the process to take a person charged on an indictment, when he is not in custody.  <rj><au>Burrill. Wharton.</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capibara</ent><br/
<hw>Ca`pi*ba"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Capybara</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillaceous</ent><br/
<hw>Cap`il*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capillaceus</ets> hairy, fr. <ets>capillus</ets> hair.]</ety> <def>Having long filaments; resembling a hair; slender. See <er>Capillary</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillaire</ent><br/
<hw>Cap`il*laire"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>capillaire</ets> maiden-hair; sirop de <ets>capillaire</ets> capillaire; fr. L. herba <ets>capillaris</ets> the maidenhair.]</ety> <sn>1.</sn> <def>A sirup prepared from the maiden-hair, formerly supposed to have medicinal properties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any simple sirup flavored with orange flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillament</ent><br/
<hw>Ca*pil"la*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>capillamentum</ets>, fr. <ets>capillus</ets> hair: cf. F. <ets>capillament</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A filament.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Any villous or hairy covering; a fine fiber or filament, as of the nerves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillariness</ent><br/
<hw>Cap"il*la*ri*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being capillary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillarity</ent><br/
<hw>Cap`il*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>capillarit<eacute/</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The quality or condition of being capillary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physics)</fld> <def>The peculiar action by which the surface of a liquid, where it is in contact with a solid (as in a capillary tube), is elevated or depressed; capillary attraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Capillarity</xex> depends upon the relative attaction of the modecules of the liquid for each other and for those of the solid, and is especially observable in capillary tubes, where it determines the ascent or descent of the liquid above or below the level of the liquid which the tube is dipped; -- hence the name.<--   It is especially important in certain plants, to allow flow of water from the roots to upper parts--></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillary</ent><br/
<hw>Cap"il*la*ry</hw> <pr>(k<acr/p"<icr/l*l<asl/*r<ycr/ <it>or</it> k<adot/*p<icr/l"l<adot/*r<ycr/; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>capillaris</ets>, fr. <ets>capillus</ets> hair. Cf. <er>Capillaire</er>.]</ety> <sn>1.</sn> <def>Resembling a hair; fine; minute; very slender; having minute tubes or interspaces; having very small bore; <as>as, the <ex>capillary</ex> vessels of animals and plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to capillary tubes or vessels; <as>as, <ex>capillary</ex> action</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Capillary attraction</b></col>, <col><b>Capillary repulsion</b></col></mcol>, <cd>the apparent attraction or repulsion between a solid and liquid caused by capillarity. See <er>Capillarity</er>, and <er>Attraction</er>.</cd> -- <col><b>Capillarity tubes</b></col>. <cd>See the <er>Note</er> under <er>Capillarity</er>.</cd></cs><br/
<br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillary</ent><br/
<hw>Cap"il*la*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <er>Capillaries</er> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A tube or vessel, extremely fine or minute.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A minute, thin-walled vessel; particularly one of the smallest blood vessels connecting arteries and veins, but used also for the smallest lymphatic and biliary vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillation</ent><br/
<hw>Cap`il*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>capillatio</ets> the hair.]</ety> <def>A capillary blood vessel.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillature</ent><br/
<hw>Ca*pil"la*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>capillatura</ets>.]</ety> <def>A bush of hair; frizzing of the hair.</def>  <rj><au>Clarke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capilliform</ent><br/
<hw>Ca*pil"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capillus</ets> hair + <ets>-form</ets>.]</ety> <def>In the shape or form of, a hair, or of hairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capillose</ent><br/
<hw>Cap"il*lose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capillosus</ets>.]</ety> <def>Having much hair; hairy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capistrate</ent><br/
<hw>Ca*pis"trate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capistratus</ets>, p. p. of <ets>capistrare</ets> halter.]</ety> <fld>(Zool.)</fld> <def>Hooded; cowled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capital</ent><br/
<hw>Cap"i*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>capital</ets>, L. <ets>capitalis</ets> capital (in senses 1 & 2), fr. <ets>caput</ets> head. See <er>Chief</er>, and cf. <er>Capital</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to the head.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Needs must the Serpent now his <qex>capital</qex> bruise<br/
Expect with mortal pain.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having reference to, or involving, the forfeiture of the head or life; affecting life; punishable with death; <as>as, <ex>capital</ex> trials; <ex>capital</ex> punishment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many crimes that are <qex>capital</qex> among us.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To put to death a <qex>capital</qex> offender.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>First in importance; chief; principal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>capital</qex> article in religion</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whatever is <qex>capital</qex> and essential in Christianity.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Chief, in a political sense, as being the seat of the general government of a state or nation; <as>as, Washington and Paris are <ex>capital</ex> cities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Of first rate quality; excellent; <as>as, a <ex>capital</ex> speech or song</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 214 --></p>

<p><cs><col><b>Capital letter</b></col> <ety>[F, <ets>lettre capitale</ets>]</ety> <fld>(Print.)</fld>, <cd>a leading or heading letter, used at the beginning of a sentence and as the first letter of certain words, distinguished, for the most part, both by different form and larger size, from the small (<xex>lower-case</xex>) letters, which form the greater part of common print or writing.</cd> -- <col><b>Small capital letters</b></col> <cd>have the form of capital letters and height of the body of the lower-case letters.</cd> -- <col><b>Capital stock</b></col>, <cd>money, property, or stock invested in any business, or the enterprise of any corporation or institution.</cd>  <rj><au>Abbott.</au></rj>
</cs></p>

<p><syn><b>Syn.</b> -- Chief; leading; controlling; prominent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>capital</ent><br/
<hw>cap"i*tal</hw> <pr>(k<acr/p"<icr/*t<ait/l)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>capitellum</ets> and <ets>capitulum</ets>, a small head, the head, top, or capital of a column, dim. of <ets>caput</ets> head; F. <ets>chapiteau</ets>, OF. <ets>capitel</ets>. See <er>chief</er>, and cf. <er>cattle</er>, <er>chattel</er>, <er>chapiter</er>, <er>chapter</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>The head or uppermost member of a column, pilaster, etc.  It consists generally of three parts, <part>abacus</part>, <part>bell</part> (or <part>vase</part>), and <part>necking</part>.  See these terms, and <er>Column</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. F. <ets>capilate</ets>, fem., sc. <ets>ville</ets>.]</ety> <fld>(Geog.)</fld> <def>The seat of government; the chief city or town in a country; a metropolis.</def> <ldquo/A busy and splendid <xex>capital</xex><rdquo/  <rj><au>Macauly.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Cf. F. <ets>capital</ets>.]</ety> <def>Money, property, or stock employed in trade, manufactures, etc.; the sum invested or lent, as distinguished from the income or interest. See <cref>Capital stock</cref>, under <er>Capital</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Polit. Econ.)</fld> <def>That portion of the produce of industry, which may be directly employed either to support human beings or to assist in production.</def>  <rj><au>M'Culloch.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When wealth is used to assist production it is called <ex>capital</ex>.  The capital of a civilized community includes <stype>fixed capital</stype> (i.e. buildings, machines, and roads used in the course of production and exchange) and <stype>circulating capital</stype> (i.e., food, fuel, money, etc., spent in the course of production and exchange).</note>  <rj><au>T. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything which can be used to increase one's power or influence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He tried to make <qex>capital</qex> out of his rival's discomfiture.</q> <rj><qau>London Times.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Fort.)</fld> <def>An imaginary line dividing a bastion, ravelin, or other work, into two equal parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A chapter, or section, of a book.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Holy St. Bernard hath said in the 59th <qex>capital</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Print.)</fld> <def>See <cref>Capital letter</cref>, under <er>Capital</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Active capital</b></col>. <cd>See under <er>Active</er>,</cd> -- <col><b>Small capital</b></col> <fld>(Print.)</fld>, <cd>a small capital letter; informally referred to (in the plural) as <altname>small caps</altname>; as, the technical terms are listed in <altname>small caps</altname>. See under <er>Capital</er>, <pos>a.</pos></cd> -- <col><b>To live on one's capital</b></col>, <cd>to consume one's capital without producing or accumulating anything to replace it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>capitalisation</ent><br/
<hw>capitalisation</hw> <pos>n.</pos> <sn>1.</sn> <def>same as <er>capitalization</er>.</def><br/
<syn><b>Syn. --</b> capitalization.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>capitalism</ent><br/
<hw>cap"i*tal*is`m</hw> <pr>(k<acr/p"<icr/*t<ait/l*<icr/z`m)</pr>, <pos>n.</pos> <def>An economic system based on predominantly private (individual or corporate) investment in and ownership of the means of production, distribution, and exchange of goods and wealth; contrasted with <contr>socialism</contr> or especially <contr>communism</contr>, in which the state has the predominant role in the economy.</def><br/
<syn><b>Syn. --</b> capitalist economy.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Capitalist</ent><br/
<hw>Cap"i*tal*ist</hw> <pr>(k<acr/p"<icr/*t<ait/l*<icr/st)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>capitaliste</ets>.]</ety> <def>One who has capital; one who has money for investment, or money invested; esp. a person of large property, which is employed in business.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The expenditure of the <qex>capitalist</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>capitalistic</ent><br/
<hw>capitalistic</hw> <pos>adj.</pos> <sn>1.</sn>  <def>Of or relating to capitalism or capitalists.</def> <illu><ex>capitalistic</ex> methods and incentives</illu><br/
<syn><b>Syn. --</b> capitalist.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>Favoring or practicing capitalism.</def> [Narrower terms: <stype>bourgeois</stype>] <ant>socialistic</ant><br/
<syn><b>Syn. --</b> capitalist.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Capitalization</ent><br/
<hw>Cap"i*tal*i*za`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of capitalizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitalize</ent><br/
<hw>Cap"i*tal*ize</hw> <pr>(k<acr/p"<icr/*t<ait/l*<imac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capitalized</conjf> <pr>(k<acr/p"<icr/*t<ait/l*<imac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Capitalizing</conjf>.]</vmorph> <sn>1.</sn> <def>To convert into capital, or to use as capital.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To compute, appraise, or assess the capital value of (a patent right, an annuity, etc.)</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To print in capital letters, or with an initial capital.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To supply capital for (an enterprise), especially by selling capital stock.</def><br/
[<source>PJC</source>]</p>

<p><ent>Capitalize on</ent><br/
<hw>Cap"i*tal*ize on`</hw> <pr>(k<acr/p"<icr/*t<ait/l*<imac/z <ocr/n`)</pr>, <pos>v. t.</pos> <def>To turn (an opportunity) to one's advantage; to take advantage of (a situation); to profit from; <as>as, to <ex>capitalize on</ex> an opponent's mistakes</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Capitally</ent><br/
<hw>Cap*i*tal*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a way involving the forfeiture of the head or life; <as>as, to punish <ex>capitally</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a capital manner; excellently.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitalness</ent><br/
<hw>Cap"i*tal*ness</hw>, <pos>n.</pos> <def>The quality of being capital; preeminence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitan Pacha</ent><br/
<ent>Capitan Pasha</ent><br/
<mhw>{ <hw>Ca`pi*tan" Pa*sha"</hw> or <hw>Ca`pi*tan" Pa*cha"</hw> <pr>(k<aum/`p<esl/*t<aum/n" p<adot/*sh<aum/")</pr> }</mhw>. <ety>[See <er>capitan</er>.]</ety> <def>The chief admiral of the Turkish fleet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitate</ent><br/
<hw>Cap"i*tate</hw> <pr>(k<acr/p"<icr/*t<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>capitatus</ets> fr. <ets>caput</ets> head.]</ety> <sn>1.</sn> <def>Headlike in form; also, having the distal end enlarged and rounded, as the stigmas of certain flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having the flowers gathered into a head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>capitatim</ent><br/
<hw>cap`i*ta"tim</hw> <pr>(k<acr/p`<icr/*t<amac/"t<icr/m)</pr>, <pos>a.</pos> <ety>[NL.]</ety> <def>Of so much per head; <as>as, a <ex>capitatim</ex> tax; a <ex>capitatim</ex> grant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>capitation</ent><br/
<hw>cap`i*ta"tion</hw> <pr>(k<acr/p`<icr/*t<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>capitatio</ets> a poll tax, fr. <ets>caput</ets> head; cf. F. <ets>capitation</ets>.]</ety> <sn>1.</sn> <def>A numbering of heads or individuals.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tax upon each head or person, without reference to property; a poll tax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capite</ent><br/
||<hw>Cap"i*te</hw> <pr>(k<acr/p`<icr/*t<emac/)</pr>, <pos>n.</pos> <ety>[L., abl. of <ets>caput</ets> head.]</ety> <def>See under <er>Tenant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitellate</ent><br/
<hw>Cap`i*tel"late</hw> <pr>(k<acr/p`<icr/*t<ecr/l"l<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>capitellum</ets>, dim. of <ets>caput</ets> head.]</ety> <fld>(Bot.)</fld> <def>Having a very small knoblike termination, or collected into minute capitula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitibranchiata</ent><br/
||<hw>Cap`i*ti*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>caput</ets>, <ets>capitis</ets>, head + <ets>-branchiae</ets> gills.]</ety> <fld>(Zool.)</fld> <def>A division of annelids in which the gills arise from or near the head. See <er>Tubicola</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitol</ent><br/
<hw>Cap"i*tol</hw> <pr>(?)</pr>, <ety>[L. <ets>capitolium</ets>, fr. <ets>caput</ets> head: cf. F. <ets>capitole</ets>. See <er>Chief</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The temple of <persfn>Jupiter</persfn>, at <city>Rome</city>, on the Mona Capitolinus, where the Senate met.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Comes C<ae/sar to the <qex>Capitol</qex> to-morrow?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The edifice at Washington occupied by the Congress of the United States; also, the building in which the legislature of a State holds its sessions; a statehouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitoline</ent><br/
<ent>Capitolian</ent><br/
<mhw>{ <hw>Cap`i*to"li*an</hw> <pr>(?)</pr>, <hw>Cap"i*to*line</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>capitolinus</ets>: cf. F. <ets>capitolin</ets>.]</ety> <def>Of or pertaining to the Capitol in Rome.</def> <ldquo/<xex>Capitolian</xex> Jove.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capitoline games</b></col> <fld>(Antiq.)</fld>, <cd>annual games instituted at <city>Rome</city> by <persfn>Camillus</persfn>, in honor of <person>Jupiter Capitolinus</person>, on account of the preservation of the Capitol from the Gauls; when reinstituted by <persfn>Domitian</persfn>, after a period of neglect, they were held every fifth year.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitula</ent><br/
||<hw>Ca*pit"u*la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Capitulum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitular</ent><br/
<hw>Ca*pit"u*lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>capitulare</ets>, <ets>capitularium</ets>, fr. L. <ets>capitulum</ets> a small head, a chapter, dim. of <ets>capit</ets> head, chapter.]</ety> <sn>1.</sn> <def>An act passed in a chapter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of a chapter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The chapter itself, and all its members or <qex>capitulars</qex>.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The head or prominent part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitular</ent><br/
<hw>Ca*pit"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Eccl.)</fld> <def>Of or pertaining to a chapter; capitulary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From the pope to the member of the <qex>capitular</qex> body.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Growing in, or pertaining to, a capitulum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Pertaining to a capitulum; <as>as, the <ex>capitular</ex> process of a vertebra, the process which articulates with the capitulum of a rib</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitularly</ent><br/
<hw>Ca*pit"u*lar*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In the manner or form of an ecclesiastical chapter.</def>  <rj><au>Sterne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulary</ent><br/
<hw>Ca*pit"u*la*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Capitularies</plw> <pr>(#)</pr></plu>. <ety>[See <er>Capitular</er>.]</ety> <sn>1.</sn> <def>A capitular.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The body of laws or statutes of a chapter, or of an ecclesiastical council.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A collection of laws or statutes, civil and ecclesiastical, esp. of the Frankish kings, in chapters or sections.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Several of Charlemagne's <qex>capitularies</qex>.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulary</ent><br/
<hw>Ca*pit"u*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to the chapter of a cathedral; capitular.</def> <ldquo/<xex>Capitulary</xex> acts.<rdquo/  <rj><au>Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulate</ent><br/
<hw>Ca*pit"u*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capitulated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Capitulating</conjf>.]</vmorph> <ety>[LL. <ets>capitulatus</ets>, p. p. of <ets>capitulare</ets> to capitulate: cf. F. <ets>capituler</ets>. See <er>Capitular</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To settle or draw up the heads or terms of an agreement, as in chapters or articles; to agree.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There <qex>capitulates</qex> with the king . . . to take to wife his daughter Mary.</q> <rj><qau>Heylin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no reason why the reducing of any agreement to certain heads or capitula should not be called to <qex>capitulate</qex>.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To surrender on terms agreed upon (usually, drawn up under several heads); <as>as, an army or a garrison <ex>capitulates</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Irish, after holding out a week, <qex>capitulated</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulate</ent><br/
<hw>Ca*pit"u*late</hw>, <pos>v. t.</pos> <def>To surrender or transfer, as an army or a fortress, on certain conditions.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulation</ent><br/
<hw>Ca*pit`u*la"tion</hw> <pr>(k<adot/*p<icr/t`<usl/*l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>capitulation</ets>, LL. <ets>capitulatio</ets>.]</ety> <sn>1.</sn> <def>A reducing to heads or articles; a formal agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With special <qex>capitulation</qex> that neither the Scots nor the French shall refortify.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of capitulating or surrendering to an enemy upon stipulated terms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The instrument containing the terms of an agreement or surrender.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulator</ent><br/
<hw>Ca*pit"u*la`tor</hw> <pr>(k<adot/*p<icr/t"<usl/*l<amac/`t<etil/r)</pr>, <pos>n.</pos> <ety>[LL.]</ety> <def>One who capitulates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitule</ent><br/
<hw>Cap"i*tule</hw> <pr>(k<acr/p"<icr/*t<umac/l)</pr>, <pos>n.</pos> <ety>[L. <ets>capitulum</ets> small head, chapter.]</ety> <def>A summary.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capitulum</ent><br/
||<hw>Ca*pit"u*lum</hw> <pr>(k<adot/*p<icr/t"<usl/*l<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Capitula</er> <pr>(k<adot/*p<icr/t"<usl/*l<adot/)</pr>.</plu> <ety>[L., a small head.]</ety> <sn>1.</sn> <def>A thick head of flowers on a very short axis, as a clover top, or a dandelion; a composite flower. A capitulum may be either globular or flat.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A knoblike protuberance of any part, esp. at the end of a bone or cartilage. <note>[See <xex>Illust.</xex> of <er>Artiodactyla</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capivi</ent><br/
<hw>Ca*pi"vi</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Copaiba</er>.]</ety> <def>A balsam of the Spanish West Indies.  See <er>Copaiba</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caple</ent><br/
<hw>Ca"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Capel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caplin</ent><br/
<hw>Cap"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Capelin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capling</ent><br/
<ent>Caplin</ent><br/
<mhw>{ <hw>Cap"lin</hw> <pr>(?)</pr>, <hw>Cap"ling</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The cap or coupling of a flail, through which the thongs pass which connect the handle and swingel.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capnomancy</ent><br/
<hw>Cap"no*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kapno`s</grk> smoke + <ets>mancy</ets>: cf. F. <ets>capnomancie</ets>.]</ety> <def>Divination by means of the ascent or motion of smoke.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capnomor</ent><br/
<hw>Cap"no*mor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kapno`s</grk> smoke + <grk>mo`ra</grk>, equiv. to <grk>moi^ra</grk> part.]</ety> <fld>(Chem.)</fld> <def>A limpid, colorless oil with a peculiar odor, obtained from beech tar.</def>  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capoc</ent><br/
||<hw>Ca*poc"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>k<amac/poq</ets>.]</ety> <def>A sort of cotton so short and fine that it can not be spun, used in the East Indies to line palanquins, to make mattresses, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capoch</ent><br/
<hw>Ca*poch"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Capoches</plw> <pr>(#)</pr>.</plu> <ety>[Cf. Sp. <ets>capucho</ets>, It. <ets>cappucio</ets>, F. <ets>Capuce</ets>, <ets>capuchon</ets>, LL. <ets>caputium</ets>, fr. <ets>capa</ets> cloak. See <ets>Cap</ets>.]</ety> <def>A hood; especially, the hood attached to the gown of a monk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capoch</ent><br/
<hw>Ca*poch"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capoched</conjf> <pr>(?)</pr>.]</vmorph> <def>To cover with, or as with, a hood; hence, to hoodwink or blind.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capon</ent><br/
<hw>Ca"pon</hw> <pr>(k<amac/"p'n <it>or</it> k<amac/"p<ucr/n; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>capon</ets>, <ets>chapoun</ets>, AS. <ets>cap<umac/n</ets> (cf. F. <ets>chapon</ets>), L. <ets>capo</ets>, fr. Gr. <grk>ka`pwn</grk> akin to <grk>ko`ptein</grk> to cut, OSlav. <ets>skopiti</ets> to castrate. Cf. <er>Comma</er>.]</ety> <def>A castrated cock, esp. when fattened; a male chicken gelded to improve his flesh for the table.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The merry thought of a <qex>capon</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capon</ent><br/
<hw>Ca"pon</hw>, <pos>v. t.</pos> <def>To castrate; to make a capon of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caponet</ent><br/
<hw>Ca"pon*et</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A young capon.</def> <mark>[R.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caponiere</ent><br/
<hw>Cap`o*niere"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caponni<egrave/re</ets>, fr. Sp. <ets>caponera</ets>, orig., a cage for fattening capons, hence, a place of refuge; cf. It. <ets>capponiera</ets>. See <er>Capon</er>.]</ety> <fld>(Fort.)</fld> <def>A work made across or in the ditch, to protect it from the enemy, or to serve as a covered passageway.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caponize</ent><br/
<hw>Ca"pon*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To castrate, as a fowl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caporal</ent><br/
||<hw>Ca`po*ral"</hw> <pr>(k<aum/`p<osl/*r<aum/l")</pr>, <pos>n.</pos> <ety>[Sp. See <er>Corporal</er>, <pos>n.</pos>]</ety> <def>One who directs work; an overseer.</def> <mark>[Sp. Amer.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Capot</ent><br/
<hw>Ca*pot"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A winning of all the tricks at the game of piquet. It counts for forty points.</def>  <rj><au>Hoyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capot</ent><br/
<hw>Ca*pot"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capotted</conjf>.]</vmorph> <def>To win all the tricks from, in playing at piquet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capo tasto</ent><br/
||<hw>Ca"po tas"to</hw> <pr>(?)</pr>. <ety>[It. <ets>capotasto</ets>.]</ety> <fld>(Music)</fld> <def>A sort of bar or movable nut, attached to the finger board of a guitar or other  fretted instrument for the purpose of raising uniformly the pitch of all the strings.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Capote</ent><br/
<hw>Ca*pote"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>capote</ets> (cf. F. <ets>capote</ets>.), fr. LL. <ets>capa</ets> cape, cloak. See <er>Cap</er>.]</ety> <def>A long cloak or overcoat, especially one with a hood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capouch</ent><br/
<hw>Ca*pouch"</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>Same as <er>Capoch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cappadine</ent><br/
<hw>Cap"pa*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A floss or waste obtained from the cocoon after the silk has been reeled off, used for shag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cappaper</ent><br/
<hw>Cap"pa`per</hw> <pr>(?)</pr>, <def>See <er>cap</er>, <pos>n.</pos>, also <er>Paper</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cappeak</ent><br/
<hw>Cap"peak`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The front piece of a cap; -- now more commonly called <xex>visor</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cappella</ent><br/
||<hw>Cap*pel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>A cappella</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capper</ent><br/
<hw>Cap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One whose business is to make or sell caps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A by-bidder; a decoy for gamblers.</def> <mark>[Slang, U. S.]</mark><--shill?--><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An instrument for applying a percussion cap to a gun or cartridge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capping plane</ent><br/
<hw>Cap"ping plane`</hw> <pr>(?)</pr>. <fld>(Join.)</fld> <def>A plane used for working the upper surface of staircase rails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capra</ent><br/
||<hw>Ca"pra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a she goat.]</ety> <fld>(Zool.)</fld> <def>A genus of ruminants, including the common goat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprate</ent><br/
<hw>Cap"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of capric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprella</ent><br/
<hw>Caprella</hw> <pos>n.</pos> <def>a genus somprising the skeleton shrimp.</def><br/
<syn><b>Syn. --</b> genus <gen>Caprella</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Capreolate</ent><br/
<hw>Cap"re*o*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capreolus</ets> wild goat, tendril, fr.<ets>caper</ets> goat: cf. F. <ets>capr<eacute/ol<eacute/</ets>.]</ety> <fld>(Bot.)</fld> <def>Having a tendril or tendrils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capreoline</ent><br/
<hw>Cap"re*o*line</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capreolus</ets> wild goat, fr. <ets>caper</ets> goat.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the roebuck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capreolus</ent><br/
<hw>Capreolus</hw> <pos>n.</pos> <def>a genus of deer including the roe deer (<spn>Capreolus capreolus</spn>).</def><br/
<syn><b>Syn. --</b> genus <gen>Capreolus</gen>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Capri</ent><br/
<hw>Ca"pri</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Wine produced on the island of Capri, commonly a light, dry, white wine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Capric</ent><br/
<hw>Cap"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caper</ets> goat.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to capric acid or its derivatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Capric acid</b></col> (also called <altname>decanoic acid</altname>), <chform>C9H19.CO.OH</chform>, <col><b>Caprylic acid</b></col> (also called <altname>octanoic acid</altname>), <chform>C7H15.CO2.H</chform>, <it>and</it> <col><b>Caproic acid</b></col> (also called <altname>hexanoic acid</altname>), <chform>C5H11.CO2.H</chform></mcol>, <cd>are fatty acids occurring in small quantities in butter, cocoanut oil, etc., united with glycerin; they are colorless oils, or white crystalline solids, of an unpleasant odor like that of goats or sweat.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capriccio</ent><br/
||<hw>Ca*pric"cio</hw> <pr>(k<adot/*pr<emac/t"ch<osl/)</pr>, <pos>n.</pos> <ety>[It. See <er>Caprice</er>.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A piece in a free form, with frequent digressions from the theme; a fantasia; -- often called <altname>caprice</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A caprice; a freak; a fancy.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capriccioso</ent><br/
||<hw>Ca*pric*cio"so</hw> <pr>(k<adot/*pr<emac/t*ch<omac/"s<osl/)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus)</fld> <def>In a free, fantastic style.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprice</ent><br/
<hw>Ca*price"</hw> <pr>(k<adot/*pr<emac/s")</pr>, <pos>n.</pos> <ety>[F. <ets>caprice</ets>, It. <ets>capriccio</ets>, caprice (perh. orig. a fantastical goat leap), fr. L. <ets>caper</ets>, <ets>capra</ets>, goat. Cf <er>Capriole</er>, <er>Cab</er>, <er>Caper</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>An abrupt change in feeling, opinion, or action, proceeding from some whim or fancy; a freak; a notion.</def> <ldquo/<xex>Caprices</xex> of appetite.<rdquo/  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>See <er>Capriccio</er>.</def></p>

<p><syn><b>Syn.</b> -- Freak; whim; crotchet; fancy; vagary; humor; whimsey; fickleness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capricious</ent><br/
<hw>Ca*pri"cious</hw> <pr>(k<adot/*pr<icr/sh"<ucr/s)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>capricieux</ets>, It. <ets>capriccioso</ets>.]</ety> <def>Governed or characterized by caprice; apt to change suddenly; freakish; whimsical; changeable.</def> <ldquo/<xex>Capricious</xex> poet.<rdquo/ <au>Shak.</au> <ldquo/<xex>Capricious</xex> humor.<rdquo/ <au>Hugh Miller.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>capricious</qex> partiality to the Romish practices.</q> <rj><qau>Hallam.</qau></rj></p>

<p><syn><b>Syn.</b> -- Freakish; whimsical; fanciful; fickle; crotchety; fitful; wayward; changeable; unsteady; uncertain; inconstant; arbitrary.</syn></p>

<p>-- <wordforms><wf>Ca*pri"cious*ly</wf>, <pos>adv.</pos> -- <wf>Ca*pri"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capricorn</ent><br/
<hw>Cap"ri*corn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>capricornus</ets>; <ets>caper</ets> goat + <ets>cornu</ets> horn: cf. F. <ets>capricorne</ets>.]</ety> <sn>1.</sn> <fld>(Astron.)</fld> <def>The tenth sign of zodiac, into which the sun enters at the winter solstice, about December 21. See <er>Tropic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sun was entered into <qex>Capricorn</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A southern constellation, represented on ancient monuments by the figure of a goat, or a figure with its fore part like a fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capricorn beetle</b></col> <fld>(Zool.)</fld>, <cd>any beetle of the family <fam>Carambucid<ae/</fam>; one of the long-horned beetles. The larv<ae/ usually bore into the wood or bark of trees and shrubs and are often destructive. See <er>Girdler</er>, <er>Pruner</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprid</ent><br/
<hw>Cap"rid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caper</ets>, <ets>capra</ets>, goat.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the tribe of ruminants of which the goat, or genus <gen>Capra</gen>, is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprification</ent><br/
<hw>Cap`ri*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caprificatio</ets>, fr. <ets>caprificare</ets> to ripen figs by caprification, fr. <ets>caprificus</ets> the wild fig; <ets>caper</ets> goat + <ets>ficus</ets> fig.]</ety> <def>The practice of hanging, upon the cultivated fig tree, branches of the wild fig infested with minute hymenopterous insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is supposed that the little insects insure fertilization by carrying the pollen from the male flowers near the opening of the fig down to the female flowers, and also accelerate ripening the fruit by puncturing it.  The practice has existed since ancient times, but its benefit has been disputed.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprifole</ent><br/
<hw>Cap"ri*fole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caper</ets> goat + <ets>folium</ets> leaf.]</ety> <def>The woodbine or honeysuckle.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprifoliaceous</ent><br/
<hw>Cap"ri*fo`li*a`ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, the Honeysuckle family of plants (<fam>Caprifoliac<ae/</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capriform</ent><br/
<hw>Cap"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caper</ets> goat + <ets>-form</ets>.]</ety> <def>Having the form of a goat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprigenous</ent><br/
<hw>Ca*prig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caprigenus</ets>; <ets>caper</ets> goat + <ets>gegnere</ets> to produce.]</ety> <def>Of the goat kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprimulgidae</ent><br/
<hw>Caprimulgidae</hw> <pos>n.</pos> <ety>[L. <ets>capris</ets> goat + <ets>mulgere</ets> to milk.]</ety> <def>a widely distributed natural family of nocturnally active birds including the <stype>whip-poor-will</stype> (<spn>Caprimulgus vociferus</spn>), the <stype>chuck-will's-widow</stype> (<spn>Caprimulgus carolinensis</spn>), and the <stype>common nighthawk</stype> (<spn>Chordeiles minor</spn>); -- called popularly the <altname>goatsuckers</altname> or <altname>nightjars</altname>.  The nighthawks are sometimes active during the day.</def><br/
<syn><b>Syn. --</b> goatsuckers, nightjars, family <fam>Caprimulgidae</fam>.</syn><br/
[<source>PJC</source>]</p>

<p><q>The family . . . is alternately known as the nightjars (derived from the "churring" sounds of several species -- "jarring" the night air), or goatsuckers, a nonsense name that should be discontinued as it has its origin in the preposterous myth that the birds sucked the milk of nanny goats until they were dry.</q> <rj><qau>Terence Michael Short (Wild Birds of the Americas)</qau></rj></p>

<p><ent>Caprimulgiformes</ent><br/
<hw>Caprimulgiformes</hw> <pos>n.</pos> <def>an order of birds including the goatsuckers (<fam>Caprimulgidae</fam>), frogmouths, and the oilbird (guacharo) (family <fam>Steatornithidae</fam>).</def><br/
<syn><b>Syn. --</b> order <ord>Caprimulgiformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caprimulgus</ent><br/
<hw>Caprimulgus</hw> <pos>n.</pos> <def>the type genus of the <fam>Caprimulgidae</fam>, including the <stype>whip-poor-will</stype> (<spn>Caprimulgus vociferus</spn>) and the <stype>chuck-will's-widow</stype> (<spn>Caprimulgus carolinensis</spn>).</def><br/
<syn><b>Syn. --</b> genus <gen>Caprimulgus</gen>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Caprine</ent><br/
<hw>Cap"rine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caprinus</ets>.]</ety> <def>Of or pertaining to a goat; <as>as, <ex>caprine</ex> gambols</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capriole</ent><br/
<hw>Cap"ri*ole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>capriole</ets>, <ets>cabriole</ets>, It. <ets>capriola</ets>, fr. L. <ets>caper</ets> goat. Cf. <er>Caper</er>, <pos>v. i.</pos> <er>Cabriole</er>, <er>Caprice</er>, <er>Cheveril</er>.]</ety> <sn>1.</sn> <fld>(Man.)</fld> <def>A leap that a horse makes with all fours, upwards only, without advancing, but with a kick or jerk of the hind legs when at the height of the leap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A leap or caper, as in dancing.</def> <ldquo/With lofty turns and <xex>caprioles</xex>.<rdquo/  <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capriole</ent><br/
<hw>Cap"ri*ole</hw>, <pos>v. i.</pos> <def>To perform a capriole.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capriped</ent><br/
<hw>Cap"ri*ped</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>capripers</ets>; <ets>caper</ets> goat + <ets>pes pedis</ets>, foot.]</ety> <def>Having feet like those of a goat.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 215 --></p>

<p><ent>Caproate</ent><br/
<hw>Cap"ro*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of caproic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caproic</ent><br/
<hw>Ca*pro"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>See under <er>Capric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprylate</ent><br/
<hw>Cap"ry*late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of caprylic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caprylic</ent><br/
<hw>Ca*pryl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>See under <er>Capric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsaicin</ent><br/
<hw>Cap*sa"i*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Capsicum</er>.]</ety> <fld>(Chem.)</fld> <def>A colorless crystalline substance extracted from the <spn>Capsicum annuum</spn>, and giving off vapors of intense acridity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsheaf</ent><br/
<hw>Cap"sheaf`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The top sheaf of a stack of grain: (<mark>fig.</mark>) the crowning or finishing part of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsicin</ent><br/
<hw>Cap"si*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Capsicum</er>.]</ety> <fld>(Chem.)</fld> <def>A red liquid or soft resin extracted from various species of capsicum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsicine</ent><br/
<hw>Cap"si*cine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Capsicum</er>.]</ety> <fld>(Chem.)</fld> <def>A volatile alkaloid extracted from <spn>Capsicum annuum</spn> or from capsicin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsicum</ent><br/
<hw>Cap"si*cum</hw> <pr>(k<acr/p"s<icr/*k<ucr/m)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>capsa</ets> box, chest.]</ety> <fld>(Bot.)</fld> <def>A genus of plants of many species, producing capsules or dry berries of various forms, which have an exceedingly pungent, biting taste, and when ground form the red or Cayenne pepper of commerce.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The most important species are <spn>Capsicum baccatum</spn> or bird pepper, <spn>Capsicum fastigiatum</spn> or chili pepper, <spn>Capsicum frutescens</spn> or spur pepper (from which tabasco is obtained), <spn>Capsicum chinense</spn>, which includes the fiery-hot habanero pepper, and <spn>Capsicum annuum</spn> or Guinea pepper, which includes the bell pepper, the jalapeno pepper, the cayenne pepper, and other common garden varieties.  The fruit is much used, both in its green and ripe state, in pickles and in cookery.   These contain varying levels of the substance <prod>capsaicin</prod> (<chform>C18H27O3N</chform>), which gives the peppers their hot taste.  The habanero is about 25-50 times hotter than the jalapeno according to a scale developed by <person>Wilbur Scoville</person> in 1912.  See also <er>Cayenne pepper</er>, <er>pepper</er> and http://www.chili-pepper-plants.com/.</note><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><sn>3.</sn> <def>Any plant of the genus <gen>Capsicum</gen> (of the <fam>Solanaceae</fam> family, which are unrelated to <gen>Piper</gen>), and its fruit; red pepper; chili pepper; <as>as, the <stype>bell <ex>pepper</ex></stype> and the <stype>jalapeno <ex>pepper</ex></stype> (both <spn>Capsicum annuum</spn>) and the <stype>habanero <ex>pepper</ex></stype> (<spn>Capsicum chinense</spn>); </as>.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>



<p><ent>Capsidae</ent><br/
<hw>Capsidae</hw> <pos>n.</pos> <def>a natural family comprising the leaf bugs.</def><br/
<syn><b>Syn. --</b> Miridae, family <fam>Miridae</fam>, family <fam>Capsidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Capsize</ent><br/
<hw>Cap*size"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Capsized</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Capsizing</conjf>.]</vmorph> <ety>[Cf. Sp. <ets>cabecear</ets> to nod, pitch, <ets>capuzar</ets>, <ets>chapuzar</ets>, to sink (a vessel) by the head; both fr. L. <ets>caput</ets> head.]</ety> <def>To upset or overturn, as a vessel or other body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But what if carrying sail <qex>capsize</qex> the boat?</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsize</ent><br/
<hw>Cap"size`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An upset or overturn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsquare</ent><br/
<hw>Cap"*square</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Gun.)</fld> <def>A metal covering plate which passes over the trunnions of a cannon, and holds it in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capstan</ent><br/
<hw>Cap"stan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cabestan</ets>, fr. Sp. <ets>cabestrante</ets>, <ets>cabrestante</ets>, fr. <ets>cabestrar</ets> to bind with a halter, fr. <ets>cabestro</ets>halter, fr. L. <ets>capistrum</ets> halter, fr. <ets>capere</ets> to hold (see <er>Capacious</er>); or perh. the Spanish is fr. L. <ets>caper</ets> goat + <ets>stans</ets>, p. pr. of <ets>stare</ets> to stand; cf. F. <ets>ch<egrave/vre</ets> she-goat, also a machine for raising heavy weights.]</ety> <def>A vertical cleated drum or cylinder, revolving on an upright spindle, and surmounted by a drumhead with sockets for bars or levers. It is much used, especially on shipboard, for moving or raising heavy weights or exerting great power by traction upon a rope or cable, passing around the drum. It is operated either by steam power or by a number of men walking around the capstan, each pushing on the end of a lever fixed in its socket.</def> <altsp>[Sometimes spelt <asp>Capstern</asp>, but improperly.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capstan bar</b></col>, <cd>one of the long bars or levers by which the capstan is worked; a handspike..</cd> -- <col><b>To pawl the capstan</b></col>, <cd>to drop the pawls so that they will catch in the notches of the pawl ring, and prevent the capstan from turning back.</cd> -- <col><b>To rig the capstan</b></col>, <cd>to prepare the for use, by putting the bars in the sockets.</cd> -- <col><b>To surge the capstan</b></col>, <cd>to slack the tension of the rope or cable wound around it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capstone</ent><br/
<hw>Cap"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>A fossil echinus of the genus <gen>Cannulus</gen>; -- so called from its supposed resemblance to a cap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsulary</ent><br/
<ent>Capsular</ent><br/
<mhw>{ <hw>Cap"su*lar</hw> <pr>(?)</pr>, <hw>Cap"su*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>capsulaire</ets>.]</ety> <def>Of or pertaining to a capsule; having the nature of a capsule; hollow and fibrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capsular ligament</b></col> <fld>(Anat.)</fld>, <cd>a ligamentous bag or capsule surrounding many movable joints in the skeleton.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsulated</ent><br/
<ent>Capsulate</ent><br/
<mhw>{ <hw>Cap"su*late</hw> <pr>(?)</pr>, <hw>Cap"su*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Inclosed in a capsule, or as in a chest or box;</def> <specif>esp.</specif> <fld>(Botany)</fld> <def>enclosed in a seed capsule; -- of seeds or spores.</def><br/
<syn><b>Syn. --</b> encapsulated.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capsule</ent><br/
<hw>Cap"sule</hw> <pr>(k<acr/p"s<umac/l)</pr>, <pos>n.</pos> <ety>[L. <ets>capsula</ets> a little box or chest, fr. <ets>capsa</ets> chest, case, fr. <ets>capere</ets> to take, contain: cf. F. <ets>capsule</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>a dry fruit or pod which is made up of several parts or carpels, and opens to discharge the seeds, <as>as, the <ex>capsule</ex> of the poppy, the flax, the lily, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A small saucer of clay for roasting or melting samples of ores, etc.; a scorifier.</def> <sd>(b)</sd> <def>a small, shallow, evaporating dish, usually of porcelain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A small cylindrical or spherical gelatinous envelope in which nauseous or acrid doses are inclosed to be swallowed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anat.)</fld> <def>A membranous sac containing fluid, or investing an organ or joint; <as>as, the <ex>capsule</ex> of the lens of the eye</as>. Also, a capsulelike organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A metallic seal or cover for closing a bottle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A small cup or shell, as of metal, for a percussion cap, cartridge, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Atrabiliary capsule</b></col>. <cd>See under <er>Atrabiliary</er>.</cd> -- <col><b>Glisson's capsule</b></col>, <cd>a membranous envelope, entering the liver along with the portal vessels and insheathing the latter in their course through the organ.</cd> -- <col><b>Suprarenal capsule</b></col>, <cd>a ductless gland secreting epinephrine, norepinephrine, and steroid hormones, on the upper end of each kidney.  It is also called the <altname>adrenal gland</altname>, <altname>glandula suprarenalis</altname>, <altname>suprarenal gland</altname>, <altname>epinephros</altname>, <altname>atrabiliary capsule</altname>, and <altname>adrenal capsule</altname>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Capsulitis</ent><br/
||<hw>Cap`su*li"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; E. <ets>capsule</ets> + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of a capsule, as that of the crystalline lens.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Capsulotomy</ent><br/
<hw>Cap`su*lot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Capsule</ets> + Gr. <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The incision of a capsule, esp. of that of the crystalline lens, as in a cataract operation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Captain</ent><br/
<hw>Cap"tain</hw> <pr>(k<acr/p"t<icr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>capitain</ets>, <ets>captain</ets>, OF. <ets>capitain</ets>, F. <ets>capitaine</ets> (cf. Sp. <ets>capitan</ets>, It. <ets>capitano</ets>), LL. <ets>capitaneus</ets>, <ets>capitanus</ets>, fr. L. <ets>caput</ets> the head. See under <er>Chief</er>, and cf. <er>Chieftain</er>.]</ety> <sn>1.</sn> <def>A head, or chief officer</def>; as: <sd>(a)</sd> <def>The military officer who commands a company, troop, or battery, or who has the rank entitling him to do so though he may be employed on other service.</def> <sd>(b)</sd> <def>An officer in the United States navy, next above a commander and below a commodore, and ranking with a colonel in the army.</def> <sd>(c)</sd> <def>By courtesy, an officer actually commanding a vessel, although not having the rank of captain.</def> <sd>(d)</sd> <def>The <xex>master</xex> or commanding officer of a merchant vessel.</def> <sd>(e)</sd> <def>One in charge of a portion of a ship's company; <as>as, a <ex>captain</ex> of a top, <ex>captain</ex> of a gun, etc.</as></def> <sd>(f)</sd> <def>The foreman of a body of workmen.</def> <sd>(g)</sd> <def>A person having authority over others acting in concert; <as>as, the <ex>captain</ex> of a boat's crew; the <ex>captain</ex> of a football team</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A trainband <qex>captain</qex> eke was he.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Rhodian <qex>captain</qex>, relying on . . . the lightness of his vessel, passed, in open day, through all the guards.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A military leader; a warrior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Foremost <qex>captain</qex> of his time.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Captain general</b></col>. <sd>(a)</sd> <cd>The commander in chief of an army or armies, or of the militia.</cd> <sd>(b)</sd> <cd>The Spanish governor of Cuba and its dependent islands.</cd> -- <col><b>Captain lieutenant</b></col>, <cd>a lieutenant with the rank and duties of captain but with a lieutenant's pay, -- as in the first company of an English regiment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captain</ent><br/
<hw>Cap"tain</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To act as captain of; to lead.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Men who <qex>captained</qex> or accompanied the exodus from existing forms.</q>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captain</ent><br/
<hw>Cap"tain</hw>, <pos>a.</pos> <def>Chief; superior.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>captain</qex> jewes in the carcanet.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captaincy</ent><br/
<hw>Cap"tain*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Captaincies</er> <pr>(<?/)</pr>.</plu> <def>The rank, post, or commission of a captain.</def>  <rj><au>Washington.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Captaincy general</b></col>, <cd>the office, power, territory, or jurisdiction of a captain general; <as>as, the <ex>captaincy general</ex> of La Habana (Cuba and its islands)</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captainry</ent><br/
<hw>Cap"tain*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>capitainerie</ets>.]</ety> <def>Power, or command, over a certain district; chieftainship.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captainship</ent><br/
<hw>Cap"tain*ship</hw>, <pos>n.</pos> <sn>1.</sn> <def>The condition, rank, post, or authority of a captain or chief commander.</def> <ldquo/To take the <xex>captainship</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Military skill; <as>as, to show good <ex>captainship</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captation</ent><br/
<hw>Cap*ta`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>captatio</ets>, fr. <ets>captare</ets> to catch, intens. of <ets>caper</ets> to take: cf. F. <ets>captation</ets>.]</ety> <def>A courting of favor or applause, by flattery or address; a captivating quality; an attraction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Without any of those dresses, or popular <qex>captations</qex>, which some men use in their speeches.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caption</ent><br/
<hw>Cap"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>captio</ets>, fr. <ets>caper</ets> to take. In senses 3 and 4, perhaps confounded in meaning with L. <ets>caput</ets> a head. See <er>Capacious</er>.]</ety> <sn>1.</sn> <def>A caviling; a sophism.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This doctrine is for <qex>caption</qex> and contradiction.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of taking or arresting a person by judicial process.</def> <mark>[R.]</mark>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>That part of a legal instrument, as a commission, indictment, etc., which shows where, when, and by what authority, it was taken, found, or executed.</def>  <rj><au>Bouvier.</au> <au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The heading of a chapter, section, or page.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captious</ent><br/
<hw>Cap"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>captieux</ets>, L. <ets>captiosus</ets>. See <er>Caption</er>.]</ety> <sn>1.</sn> <def>Apt to catch at faults; disposed to find fault or to cavil; eager to object; difficult to please.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>captious</qex> and suspicious age.</q> <rj><qau>Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am sensible I have not disposed my materials to abide the test of a <qex>captious</qex> controversy.</q> <rj><qau>Bwike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fitted to harass, perplex, or insnare; insidious; troublesome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Captious</qex> restraints on navigation.</q> <rj><qau>Bancroft.</qau></rj></p>

<p><syn><b>Syn.</b> -- Caviling, carping, fault-finding; censorious; hypercritical; peevish, fretful; perverse; troublesome.</syn> <usage> -- <er>Captious</er>, <er>caviling</er>, <er>Carping</er>. A <xex>captious</xex> person is one who has a fault-finding habit or manner, or is disposed to catch at faults, errors, etc., with quarrelsome intent; a <xex>caviling</xex> person is disposed to raise objections on frivolous grounds; <xex>carping</xex> implies that one is given to ill-natured, persistent, or unreasonable fault-finding, or picking up of the words or actions of others.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Caviling</qex> is the <qex>carping</qex> of argument, <qex>carping</qex> the <qex>caviling</qex> of ill temper.</q> <rj><qau>C. J. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captiously</ent><br/
<hw>Cap"tious*ly</hw>, <pos>adv.</pos> <def>In a captious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captiousness</ent><br/
<hw>Cap"tious*ness</hw>, <pos>n.</pos> <def>Captious disposition or manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captivate</ent><br/
<hw>Cap"ti*vate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Captivated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Captivating</conjf>.]</vmorph> <ety>[L. <ets>captivatus</ets>, p. p. of <ets>captivare</ets> to capture, fr. <ets>captivus</ets> captive. See <er>Captive</er>.]</ety> <sn>1.</sn> <def>To take prisoner; to capture; to subdue.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Their woes whom fortune <qex>captivates</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To acquire ascendancy over by reason of some art or attraction; to fascinate; to charm; <as>as, Cleopatra <ex>captivated</ex> Antony; the orator <ex>captivated</ex> all hearts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Small landscapes of <qex>captivating</qex> loveliness.</q> <rj><qau>W. Irving.</qau></rj></p>

<p><syn><b>Syn.</b> -- To enslave; subdue; overpower; charm; enchant; bewitch; facinate; capture; lead captive.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captivate</ent><br/
<hw>Cap"ti*vate</hw> <pr>(?)</pr>, <pos>p. a.</pos> <ety>[L. <ets>captivatus</ets>.]</ety> <def>Taken prisoner; made captive; insnared; charmed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Women have been <qex>captivate</qex> ere now.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>captivated</ent><br/
<hw>captivated</hw> <pos>adj.</pos> <sn>1.</sn> <def>having an affection or admiration, caused by charm of the person or object.</def><br/
<syn><b>Syn. --</b> charmed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>filled with wonder and delight.</def><br/
<syn><b>Syn. --</b> beguiled, charmed, delighted, enthralled, entranced.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Captivating</ent><br/
<hw>Cap"ti*va`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having power to captivate or charm; fascinating; <as>as, <ex>captivating</ex> smiles</as>.</def> -- <wordforms><wf>Cap"ti*va`ting*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captivation</ent><br/
<hw>Cap"ti*va`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>capticatio</ets>.]</ety> <def>The act of captivating.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>captivation</qex> of our understanding.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captive</ent><br/
<hw>Cap"tive</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>captivus</ets>, fr. <ets>capere</ets> to take: cf. F. <ets>captif</ets>. See <er>Caitiff</er>.]</ety> <sn>1.</sn> <def>A prisoner taken by force or stratagem, esp., by an enemy, in war; one kept in bondage or in the power of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then, when I am thy <qex>captive</qex>, talk of chains.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One charmed or subdued by beaty, excellence, or affection; one who is captivated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captive</ent><br/
<hw>Cap"tive</hw>, <pos>a.</pos> <sn>1.</sn> <def>Made prisoner, especially in war; held in bondage or in confinement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A poor, miserable, <qex>captive</qex> thrall.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Subdued by love; charmed; captivated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even in so short a space, my wonan's heart<br/
Grossly grew <qex>captive</qex> to his honey words.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to bondage or confinement; serving to confine; <as>as, <ex>captive</ex> chains; <ex>captive</ex> hours</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captive</ent><br/
<hw>Cap"tive</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Captived</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Captiving</conjf>.]</vmorph> <def>To take prisoner; to capture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their inhabitans slaughtered and <qex>captived</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captivity</ent><br/
<hw>Cap*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>captivitas</ets>: cf. F. <ets>captivit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state of being a captive or a prisoner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>More celebrated in his <qex>captivity</qex> that in his greatest triumphs.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state of being under control; subjection of the will or affections; bondage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sink in the soft <qex>captivity</qex> together.</q> <rj><qau>Addison.</qau></rj></p>

<p><syn><b>Syn.</b> -- Imprisonment; confinement; bondage; subjection; servitude; slavery; thralldom; serfdom.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Captor</ent><br/
<hw>Cap"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a cather (of animals), fr. <ets>caper</ets> to take.]</ety> <def>One who captures any person or thing, as a prisoner or a prize.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capture</ent><br/
<hw>Cap"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>capture</ets>, fr. <ets>caper</ets> to take: cf. F. <ets>capture</ets>. See <er>Caitiff</er>, and cf. <er>aptive</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The act of seizing by force, or getting possession of by superior power or by stratagem; <as>as, the <ex>capture</ex> of an enemy, a vessel, or a criminal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even with regard to <qex>captures</qex> made at sea.</q> <rj><qau>Bluckstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The securing of an object of strife or desire, as by the power of some attraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The thing taken by force, surprise, or stratagem; a prize; prey.</def></p>

<p><syn><b>Syn.</b> -- Seizure; apprehension; arrest; detention.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capture</ent><br/
<hw>Cap"ture</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Captured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Capturing</conjf>.]</vmorph> <sn>1.</sn> <def>To seize or take possession of by force, surprise, or stratagem; to overcome and hold; to secure by effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to record or make a lasting representation of (sound or images); <as>as, to <ex>capture</ex> an event on videotape; the artist <ex>captured</ex> the expression of grief on his face</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Games)</fld> <def>to take control of, or remove from play; <as>as, to <ex>capture</ex> a piece in chess</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>to exert a strong psychological influence on; <as>as, to <ex>capture</ex> the heart of a maiden; to <ex>capture</ex> the attention of the nation</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Computers)</fld> <def>to record (data) in a computer-readable form; <as>as, to <ex>capture</ex> a transaction in a database</as>.</def><br/
[<source>PJC</source>]</p>

<p><q>Her heart is like some fortress that has been <qex>captured</qex>.</q> <rj><qau>W. Ivring.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capuccio</ent><br/
||<hw>Ca*puc"cio</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>cappucio</ets>. See <er>Capoch</er>.]</ety> <def>A capoch or hood.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capuched</ent><br/
<hw>Ca*puched"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Capoch</er>.]</ety> <def>Cover with, or as with, a hood.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capuchin</ent><br/
<hw>Cap`u*chin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>capucin</ets> a monk who wears a cowl, fr. It. <ets>cappuccio</ets> hood. See <er>Capoch</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Eccl.)</fld> <def>A Franciscan monk of the austere branch established in 1526 by Matteo di Baschi, distinguished by wearing the long pointed cowl or capoch of St. Francis.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bare-footed and long-bearded <qex>capuchin</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A garment for women, consisting of a cloak and hood, resembling, or supposed to resemble, that of <xex>capuchin</xex> monks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A long-tailed South American monkey (<spn>Cabus capucinus</spn>), having the forehead naked and wrinkled, with the hair on the crown reflexed and resembling a monk's cowl, the rest being of a grayish white; -- called also <altname>capucine monkey</altname>, <altname>weeper</altname>, <altname>sajou</altname>, <altname>sapajou</altname>, and <altname>sai</altname>.</def> <sd>(b)</sd> <def>Other species of <gen>Cabus</gen>, as <spn>Cabus fatuellus</spn> (the <stype>brown capucine</stype> or <stype>horned capucine</stype>.), <spn>Cabus albifrons</spn> (the <stype>cararara</stype>), and <spn>Cabus apella</spn>.</def> <sd>(c)</sd> <def>A variety of the domestic pigeon having a hoodlike tuft of feathers on the head and sides of the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capuchin nun</b></col>, <cd>one of an austere order of Franciscan nuns which came under Capuchin rule in 1538. The order had recently been founded by Maria Longa.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capucine</ent><br/
<hw>Cap"u*cine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Capuchin</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capulet</ent><br/
<hw>Cap"u*let</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>Same as <er>Capellet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capulin</ent><br/
<hw>Cap"u*lin</hw> <pr>(-l<icr/n)</pr>, <pos>n.</pos> <ety>[Sp. <ets>capuli</ets>.]</ety> <def>The Mexican cherry (<spn>Prunus Capollin</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caput</ent><br/
||<hw>Ca"put</hw> <pr>(k<amac/"p<ucr/t)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Capita</plw> <pr>(k<acr/p"<icr/*t<adot/)</pr>.</plu> <ety>[L., the head.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The head; also, a knoblike protuberance or capitulum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The top or superior part of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eng.)</fld> <def>The council or ruling body of the University of Cambridge prior to the constitution of 1856.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>caputs</qex> and heads of colleges.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caput mortuum</b></col> <pr>(<?/)</pr>. <ety>[L., dead head.]</ety> <fld>(Old Chem.)</fld> <cd>The residuum after distillation or sublimation; hence, worthless residue.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Capybara</ent><br/
<hw>Ca`py*ba"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>capibara</ets>, fr. the native name.]</ety> <fld>(Zool.)</fld> <def>A large South American rodent (<spn>Hydroch<ae/rus capybara</spn>) Living on the margins of lakes and rivers. It is the largest extant rodent, being about three feet long, and half that in height. It somewhat resembles the Guinea pig, to which it is related; -- called also <altname>cabiai</altname> and <altname>water hog</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 216 --></p>

<p><ent>Car</ent><br/
<hw>Car</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>car</ets>, <ets>char</ets>, F. <ets>cahr</ets>, fr. L. <ets>carrus</ets>, Wagon: a Celtic word; cf. W. <ets>car</ets>, Armor. <ets>karr</ets>, Ir. & Gael. <ets>carr</ets>. cf. <er>Chariot</er>.]</ety> <sn>1.</sn> <def>A small vehicle moved on wheels; usually, one having but two wheels and drawn by one horse; a cart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vehicle adapted to the rails of a railroad.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England a railroad <xex>passenger car</xex> is called a railway <xex>carriage</xex>; a <xex>freight car</xex> a <xex>goods wagon</xex>; a <xex>platform car</xex> a <xex>goods truck</xex>; a <xex>baggage car</xex> a <xex>van</xex>. But styles of car introduced into England from America are called <xex>cars</xex>; as, tram <xex>car</xex>. Pullman <xex>car</xex>. See <er>Train</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A chariot of war or of triumph; a vehicle of splendor, dignity, or solemnity.</def> <mark>[Poetic]</mark>.<br/
[<source>1913 Webster</source>]</p>

<p><q>The gilded <qex>car</qex> of day.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The towering <qex>car</qex>, the sable steeds.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Astron.)</fld> <def>The stars also called Charles's Wain, the Great Bear, or the Dipper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Pleiads, Hyads, and the Northern <qex>Car</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The cage of a lift or elevator.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The basket, box, or cage suspended from a balloon to contain passengers, ballast, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A floating perforated box for living fish.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Car coupling</b></col>, <it>or</it> <col><b>Car coupler</b></col></mcol>, <cd>a shackle or other device for connecting the cars in a railway train.</cd> <mark>[U. S.]</mark> -- <col><b>Dummy car</b></col> <fld>(Railroad)</fld>, <cd>a car containing its own steam power or locomotive.</cd> -- <col><b>Freight car</b></col> <fld>(Railrood)</fld>, <cd>a car for the transportation of merchandise or other goods.</cd> <mark>[U. S.]</mark> -- <col><b>Hand car</b></col> <fld>(Railroad)</fld>, <cd>a small car propelled by hand, used by railroad laborers, etc.</cd> <mark>[U. S.]</mark> -- <mcol><col><b>Horse car</b></col>, <it>or</it> <col><b>Street car</b></col></mcol>, <cd>an omnibus car, draw by horses or other power upon rails laid in the streets.</cd> <mark>[U. S.]</mark> -- <mcol><col><b>Palace car</b></col>, <col><b>Drawing-room car</b></col>, <col><b>Sleeping car</b></col>, <col><b>Parlor car</b></col>, etc.</mcol> <fld>(Railroad)</fld>, <cd>cars especially designed and furnished for the comfort of travelers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carabao</ent><br/
||<hw>Ca`ra*ba"o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>The water buffalo of the Philippines.</def> <mark>[Phil. Islands]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carabid</ent><br/
<hw>Car"a*bid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of, pertaining to, or resembling, the genus <gen>Carabus</gen> or family <fam>Carabidae</fam>.</def> -- <def2><pos>n.</pos> <def>One of the <fam>Carabidae</fam>, a family of active insectivorous beetles.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carabidae</ent><br/
<hw>Carabidae</hw> <pos>n.</pos> <def>a family of nocturnal, terrestrial insects consisting of the <altname>ground beetles</altname>, which feed chiefly on other insects.</def><br/
<syn><b>Syn. --</b> family <fam>Carabidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carabine</ent><br/
<hw>Car"a*bine</hw> <pr>(k<acr/r"<adot/*b<imac/n)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>A carbine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carabineer</ent><br/
<hw>Car`a*bi*neer"</hw> <pr>(k<acr/r`<adot/*b<icr/*n<emac/r")</pr>, <pos>n.</pos> <def>A carbineer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caraboid</ent><br/
<hw>Car"a*boid</hw> <pr>(k<acr/r"<adot/*boid)</pr>, <pos>a.</pos> <ety>[<ets>Carabus</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like, or pertaining to the genus <gen>Carabus</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carabus</ent><br/
||<hw>Car"a*bus</hw> <pr>(k<acr/r"<adot/*b<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ka`rabos</grk> a horned beetle.]</ety> <fld>(Zool.)</fld> <def>A genus of ground beetles, including numerous species. They devour many injurious insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carac</ent><br/
<hw>Car"ac</hw> <pr>(k<acr/r"<acr/k)</pr>, <pos>n.</pos> <def>See <er>Carack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracal</ent><br/
<hw>Car"a*cal</hw> <pr>(k<acr/r"<adot/*k<acr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>caracal</ets>, fr. Turk <ets>qarahqootaq</ets>; <ets>qarah</ets> black + <ets>qootaq</ets> ear.]</ety> <fld>(Zool.)</fld> <def>A lynx (<spn>Felis caracal</spn> syn. <spn>Lynx caracal</spn>).  It is a native of Africa and Asia.  Its ears are black externally, and tipped with long black hairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracara</ent><br/
<hw>Ca`ra*ca"ra</hw> <pr>(k<aum/`r<adot/k<aum/"r<adot/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A south American bird of several species and genera, resembling both the eagles and the vultures. The caracaras act as scavengers, and are also called <altname>carrion buzzards</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The black caracara is <spn>Ibycter ater</spn>; the chimango is <spn>Milvago chimango</spn>; the Brazilian is <spn>Polyborus Braziliensis</spn>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracas</ent><br/
<hw>Caracas</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>Venezuela</country>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Carack</ent><br/
<hw>Car"ack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caraque</ets> (cf. Sp. & Pg. <ets>carraca</ets>, It. <ets>caracca</ets>.), LL. <ets>carraca</ets>, fr. L. <ets>carrus</ets> wagon; or perh. fr. Ar. <ets>qorq<umac/r</ets> (pl. <ets>qar<amac/qir</ets>) a carack.]</ety> <fld>(Naut.)</fld> <def>A kind of large ship formerly used by the Spaniards and Portuguese in the East India trade; a galleon.</def> <altsp>[Spelt also <asp>carrack</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The bigger whale like some huge <qex>carrack</qex> lay.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracole</ent><br/
<hw>Car"a*cole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caracole</ets>, <ets>caracol</ets>, fr. Sp. <ets>caracol</ets> snail, winding staircase, a wheeling about.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Man.)</fld> <def>A half turn which a horseman makes, either to the right or the left.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>A staircase in a spiral form.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||En caracole</b></col> <pr>(<?/)</pr> <ety>[F.]</ety>, <cd>spiral; -- said of a staircase.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracole</ent><br/
<hw>Car"a*cole</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caracoled</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>caracoler</ets>.]</ety> <fld>(Man.)</fld> <def>To move in a caracole, or in caracoles; to wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prince John <qex>caracoled</qex> within the lists.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracoly</ent><br/
<hw>Car"a*col`y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An alloy of gold, silver, and copper, of which an inferior quality of jewelry is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracora</ent><br/
<ent>Caracore</ent><br/
<mhw>{ <hw>Car"a*core</hw> <pr>(?)</pr>, <hw>Car"a*co`ra</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Malay <ets>kurakura</ets>.]</ety> <def>A light vessel or proa used by the people of Borneo, etc., and by the Dutch in the East Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caracul</ent><br/
<hw>Ca`ra*cul"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Var. of <er>Karakul</er>, a kind of fur.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carafe</ent><br/
||<hw>Ca*rafe"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A glass water bottle for the table or toilet; -- called also <altname>croft</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caragheen</ent><br/
<ent>Carageen</ent><br/
<mhw>{ <hw>Car"a*geen`</hw> <or/ <hw>Car"a*gheen`</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Carrageen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carambola</ent><br/
<hw>Ca`ram*bo"la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An East Indian tree (<spn>Averrhoa Carambola</spn>), and its acid, juicy fruit; called also <altname>Coromandel gooseberry</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caramel</ent><br/
<hw>Car"a*mel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caramel</ets> (cf. Sp. <ets>caramelo</ets>), LL. <ets>canna mellis</ets>, <ets>cannamella</ets>, <ets>canamella</ets>, <ets>calamellus mellitus</ets>, sugar cane, from or confused with L. <ets>canna</ets> reed + <ets>mel</ets>, <ets>mellis</ets>, honey. See <er>Cane</er>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>Burnt sugar; a brown or black porous substance obtained by heating sugar. It is soluble in water, and is used for coloring spirits, gravies, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of confectionery, usually a small cube or square of tenacious paste, or candy, of varying composition and flavor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caramelise</ent><br/
<hw>caramelise</hw> <pos>v. i.</pos> <sn>1.</sn> <def>to to change into caramel, by the action of heat; <as>as, The sugar <ex>caramelized</ex> quickly</as>.</def><br/
<syn><b>Syn. --</b> caramelize.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to convert to caramel.</def><br/
<syn><b>Syn. --</b> caramelize.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caramelize</ent><br/
<hw>caramelize</hw> <pos>v. t.</pos> <fr>1</fr> <def>to to convert [usu. sugar] into caramel, by the action of heat.</def><br/
<syn><b>Syn. --</b> caramelise.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carancha</ent><br/
<hw>carancha</hw> <pos>n.</pos> <def>the South American <er>caracara</er>.</def><br/
<syn><b>Syn. --</b> Polyborus plancus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caranday</ent><br/
<ent>caranda</ent><br/
<mhw><hw>caranda</hw>, <hw>caranday</hw></mhw> <pos>n.</pos> <def>a South American palm (<spn>Copernicia australis</spn> <it>or</it> <spn>Copernicia alba</spn>) yielding a wax similar to carnauba wax.</def><br/
<syn><b>Syn. --</b> caranday, caranda palm, wax palm.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carangid</ent><br/
<hw>carangid</hw> <pos>adj.</pos> <def>of or pertaining to the <fam>Carangidae</fam>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carangid</ent><br/
<hw>carangid</hw> <pos>n.</pos> <def>any fish of the family <fam>Carangidae</fam>, including the <stype>cavallas</stype>, <stype>jacks</stype>, <stype>pompanos</stype> and <stype>scads</stype>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carangidae</ent><br/
<hw>Carangidae</hw> <pos>n.</pos> <sn>1.</sn> <def>large family of narrow-bodied marine food fishes allied to the mackerels, with widely forked tails; chiefly of warm seas; it includes the <stype>caranx</stype>, <stype>American bluefish</stype>, and the <stype>pilot fish</stype>.</def><br/
<syn><b>Syn. --</b> family Carangidae.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carangoid</ent><br/
<hw>Ca*ran"goid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Caranx</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Belonging to the <fam>Carangid<ae/</fam>, a family of fishes allied to the mackerels, and including the caranx, American bluefish, and the pilot fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caranx</ent><br/
||<hw>Ca"ranx</hw> <pr>(k<amac/"r<acr/<nsm/ks)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A genus of fishes, common on the Atlantic coast, including the yellow or golden mackerel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carapace</ent><br/
<hw>Car"a*pace</hw> <pr>(k<acr/r"<adot/*p<amac/s)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Zool.)</fld> <def>The thick shell or shield which covers the back of the tortoise, or turtle, the crab, and other crustaceous animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carapato</ent><br/
||<hw>Ca`ra*pa"to</hw> <pr>(k<aum/`r<adot/*p<aum/"t<osl/)</pr>, <pos>n.</pos> <ety>[Pg. <ets>carrapato</ets>.]</ety> <fld>(Zool.)</fld> <def>A south American tick of the genus <gen>Amblyomma</gen>. There are several species, very troublesome to man and beast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carapax</ent><br/
<hw>Car"a*pax</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Carapace</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carat</ent><br/
<hw>Car"at</hw> <pr>(k<acr/r"<acr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>carat</ets> (cf. It. <ets>carato</ets>, OPg. <ets>quirate</ets>, Pg. & Sp. <ets>quilate</ets>), Ar. <ets>q<imac/r<amac/t</ets> bean or pea shell, a weight of four grains, a carat, fr. Gr. <grk>kera`tion</grk> a little horn, the fruit of the carob tree, a weight, a carat. See <er>Horn</er>.]</ety> <sn>1.</sn> <def>The weight by which precious stones and pearls are weighed.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <ex>carat</ex> equals three and one fifth grains Troy, and is divided into four grains, sometimes called <part>carat grains</part>. Diamonds and other precious stones are estimated by carats and fractions of carats, and pearls, usually, by carat grains.</note>  <rj><au>Tiffany.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A twenty-fourth part; -- a term used in estimating the proportionate fineness of gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A mass of metal is said to be so many <xex>carats</xex> fine, according to the number of twenty-fourths of pure gold which it contains; as, 22 <ex>carats</ex> fine (goldsmith's standard) = 22 parts of gold, 1 of copper, and 1 of silver.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caravan</ent><br/
<hw>Car"a*van</hw> <pr>(k<acr/r"<adot/*v<acr/n <it>or</it> k<acr/r*<adot/*v<acr/n"; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>caravane</ets> (cf. Sp. <ets>caravana</ets>), fr. Per. <ets>karw<amac/n</ets> a caravan (in sense 1). Cf. <er>Van</er> a wagon.]</ety> <sn>1.</sn> <def>A company of travelers, pilgrims, or merchants, organized and equipped for a long journey, or marching or traveling together, esp. through deserts and countries infested by robbers or hostile tribes, as in Asia or Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large, covered wagon, or a train of such wagons, for conveying wild beasts, etc., for exhibition; an itinerant show, as of wild beasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A covered vehicle for carrying passengers or for moving furniture, etc.; -- sometimes shorted into <altname>van</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caravaneer</ent><br/
<hw>Car`a*van*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>caravanier</ets>.]</ety> <def>The leader or driver of the camels in caravan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>caravanning</ent><br/
<hw>caravanning</hw> <pos>n.</pos> <def>the practice of taking holidays in a caravan.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caravansary</ent><br/
<hw>Car`a*van"sa*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Caravansaries</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>caravans<eacute/rai</ets>, fr. Per. <ets>karw<amac/nsar<amac/<ium/</ets>; <ets>karw<amac/n</ets> caravan + <ets>-sar<amac/<ium/</ets> palace, large house, inn.]</ety> <def>A kind of inn, in the East, where caravans rest at night, being a large, rude, unfurnished building, surrounding a court.</def> <altsp>[Written also <asp>caravanserai</asp> and <asp>caravansera</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caravel</ent><br/
<hw>Car"a*vel</hw> <pr>(k<acr/r"<adot/*v<ecr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>caravelle</ets> (cf. It. <ets>caravella</ets>, Sp. <ets>carabela</ets>), fr. Sp. <ets>caraba</ets> a kind of vessel, fr. L. <ets>carabus</ets> a kind of light boat, fr. Gr. <grk>ka`rabos</grk> a kind of light ship, NGr. <grk>kara`bi</grk> ship, vessel.]</ety> <altsp>[written also <asp>carvel</asp> and <asp>caravelle</asp>.]</altsp> <fld>(Naut.)</fld> <def>A name given to several kinds of vessels.</def> <sd>(a)</sd> <def>The caravel of the 16th century was a small vessel with broad bows, high, narrow poop, four masts, and lateen sails. <as>Columbus commanded three <ex>caravels</ex> on his great voyage</as>.</def> <sd>(b)</sd> <def>A Portuguese vessel of 100 or 150 tons burden.</def> <sd>(c)</sd> <def>A small fishing boat used on the French coast.</def> <sd>(d)</sd> <def>A Turkish man-of-war.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caraway</ent><br/
<hw>Car"a*way</hw> <pr>(k<acr/r"<adot/*w<asl/)</pr>, <pos>n.</pos> <ety>[F. <ets>carvi</ets> (cf. Sp. <ets>carvi</ets> and <ets>al-caravea</ets>, <ets>al-carahueya</ets>, Pg. <ets>al-caravia</ets>) fr. Ar. <ets>karaw<imac/<amac/</ets>, <ets>karw<imac/<amac/</ets> fr. Gr. <grk>ka`ron</grk>; cf. L. <ets>careum</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A biennial plant of the Parsley family (<spn>Carum Carui</spn>). The seeds have an aromatic smell, and a warm, pungent taste. They are used in cookery and confectionery, and also in medicine as a carminative.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cake or sweetmeat containing caraway seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Caraways</qex>, or biscuits, or some other [comfits].</q> <rj><qau>Cogan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbamic</ent><br/
<hw>Car*bam"ic</hw> <pr>(k<aum/r*b<acr/m"<icr/k)</pr>, <pos>a.</pos> <ety>[<ets>Carb</ets>on + <ets>am</ets>ido.]</ety> <fld>(Chem.)</fld> <def>Pertaining to an acid so called.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carbamic acid</b></col> <fld>(Chem.)</fld>, <cd>an amido acid, <chform>H2N.CO2.H</chform>, not existing in the free state, but occurring as a salt of ammonium in commercial ammonium carbonate; -- called also <altname>amido formic acid</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbamide</ent><br/
<hw>Car*bam"ide</hw> <pr>(k<aum/r*b<acr/m"<icr/d <it>or</it> -<imac/d)</pr>, <pos>n.</pos> <ety>[<ets>Carb</ets>onyl + <ets>amide</ets>.]</ety> <fld>(Chem.)</fld> <def>The technical name for urea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbamine</ent><br/
<hw>Car*bam"ine</hw> <pr>(k<aum/r*b<acr/m"<icr/n <it>or</it> -<emac/d)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An isocyanide of a hydrocarbon radical. The carbamines are liquids, usually colorless, and of unendurable odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbanil</ent><br/
<hw>Car"ba*nil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carb</ets>onyl + <ets>anil</ets>ine.]</ety> <fld>(Chem.)</fld> <def>A mobile liquid, <chform>CO.N.C6H5</chform>, of pungent odor. It is the phenyl salt of isocyanic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbazol</ent><br/
<hw>Car"ba*zol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carb</ets>on + <ets>azo</ets> + <ets>-ol</ets>.]</ety> <fld>(Chem.)</fld> <def>A white crystallized substance, <chform>C12H8NH</chform>, derived from aniline and other amines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbazotate</ent><br/
<hw>Car*baz"o*tate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of carbazotic or picric acid; a picrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbazotic</ent><br/
<hw>Car`ba*zot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Carb</ets>on + azole.]</ety> <def>Containing, or derived from, carbon and nitrogen.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carbazotic acid</b></col> <fld>(Chem.)</fld>, <cd>picric acid. See under <er>Picric</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbide</ent><br/
<hw>Car"bide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carb</ets>on + <ets>-ide</ets>.]</ety> <fld>(Chem.)</fld> <def>A binary compound of carbon with some other element or radical, in which the carbon plays the part of a negative; -- formerly termed <altname>carburet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbimide</ent><br/
<hw>Car"bi*mide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>imide</ets>]</ety> <fld>(Chem.)</fld> <def>The technical name for isocyanic acid. See under <er>Isocyanic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbine</ent><br/
<hw>Car"bine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carbine</ets>, OF. <ets>calabrin</ets> carabineer (cf. Ot. <ets>calabrina</ets> a policeman), fr. OF & Pr. <ets>calabre</ets>, OF. <ets>cable</ets>, <ets>chable</ets>, an engine of war used in besieging, fr. LL. <ets>chadabula</ets>, <ets>cabulus</ets>, a kind of projectile machine, fr. Gr. <?/ a throwing down, fr. <?/ to throw; <?/ down + <?/ to throw. Cf. <er>Parable</er>.]</ety> <fld>(Mil.)</fld> <def>A short, light musket or rifle, esp. one used by mounted soldiers or cavalry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbineer</ent><br/
<hw>Car`bi*neer"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carabinier</ets>.]</ety> <fld>(Mil.)</fld> <def>A soldier armed with a carbine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbinol</ent><br/
<hw>Car"bi*nol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbin</ets> (Kolbe's name for the radical) + <ets>-ol</ets>.]</ety> <fld>(Chem.)</fld> <def>Methyl alcohol, <chform>CH3OH</chform>; -- also, by extension, any one in the homologous series of paraffine alcohols of which methyl alcohol is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbohydrate</ent><br/
<hw>Car`bo*hy"drate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>hydrate</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>One of a group of compounds including the sugars, starches, and gums, which contain six (or some multiple of six) carbon atoms, united with a variable number of hydrogen and oxygen atoms, but with the two latter always in proportion as to form water; as dextrose, <chform>C6H12O6</chform>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbohydride</ent><br/
<hw>Car`bo*hy"dride</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>hydrogen</ets>.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbolic</ent><br/
<hw>Car*bol"ic</hw> <pr>(k<aum/r*b<ocr/l"<icr/k)</pr>, <pos>a.</pos> <ety>[L. <ets>carbo</ets> coal + <ets>oleum</ets> oil.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, an acid derived from coal tar and other sources; <as>as, <ex>carbolic</ex> acid (called also <xex>phenic acid</xex>, and <xex>phenol{1}</xex>)</as>. See <er>Phenol</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbolic acid</ent><br/
<hw>Car*bol"ic ac"id</hw> <pr>(k<aum/r*b<ocr/l"<icr/k <acr/s"<icr/d)</pr>, <pos>n.</pos> <ety>[L. <ets>carbo</ets> coal + <ets>oleum</ets> oil.]</ety> <fld>(Chem.)</fld> <def>same as <er>phenol{1}</er>, (<chform>C6H5.OH</chform>). See <er>phenol{1}</er>.</def><br/
<syn><b>Syn. --</b> phenol{1}, hydroxybenzene, phenyl hydroxide, phenic acid.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Carbolize</ent><br/
<hw>Car"bo*lize</hw> <pr>(k<aum/r"b<osl/*l<imac/z)</pr>, <pos>v. t.</pos> <fld>(Med.)</fld> <def>To apply carbolic acid to; to wash or treat with carbolic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>carbomycin</ent><br/
<hw>carbomycin</hw> <pos>n.</pos> <def>a colorless basic macrolide antibiotic that inhibits the growth of Gram-positive organisms.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carbon</ent><br/
<hw>Car"bon</hw> <pr>(k<aum/r"b<ocr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>carbone</ets>, fr. L. <ets>carbo</ets> coal; cf. Skr. <ets><cced/r<amac/</ets> to cook.]</ety> <fld>(Chem.)</fld> <sn>1.</sn> <def>An elementary substance, not metallic in its nature, which is present in all organic compounds. Atomic weight 11.97. Symbol C. it is combustible, and forms the base of lampblack and charcoal, and enters largely into mineral coals. In its pure crystallized state it constitutes the diamond, the hardest of known substances, occuring in monometric crystals like the octahedron, etc. Another modification is graphite, or blacklead, and in this it is soft, and occurs in hexagonal prisms or tables. When united with oxygen it forms carbon dioxide, commonly called carbonic acid, or carbonic oxide, according to the proportions of the oxygen; when united with hydrogen, it forms various compounds called hydrocarbons. Compare <er>Diamond</er>, and <er>Graphite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Elec.)</fld> <def>A carbon rod or pencil used in an arc lamp; also, a plate or piece of carbon used as one of the elements of a voltaic battery.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>a sheet of carbon paper.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a carbon copy.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Carbon compounds</b></col>, <col><b>Compounds of carbon</b></col></mcol> <fld>(Chem.)</fld>, <cd>those compounds consisting largely of carbon, commonly produced by animals and plants, and hence called <xex>organic compounds</xex>, though their synthesis may be effected in many cases in the laboratory.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>The formation of the <qex>compounds of carbon</qex> is not dependent upon the life process.</q> <rj><qau>I. Remsen</qau></rj></p>

<p>-- <col><b>carbon copy</b></col>, <cd>originally, a copy of a document made by use of a <cref>carbon paper</cref>, but now used generally to refer to any copy of a document made by a mechanical process, such as xerographic copying.</cd> -- <mcol><col><b>Carbon dioxide</b></col>, <col><b>Carbon monoxide</b></col></mcol>. <fld>(Chem.)</fld> <cd>See under <er>Carbonic</er>.</cd> -- <col><b>Carbon light</b></col> <fld>(Elec.)</fld>, <cd>an extremely brilliant electric light produced by passing a galvanic current through two carbon points kept constantly with their apexes neary in contact.</cd> -- <col><b>Carbon point</b></col> <fld>(Elec.)</fld>, <cd>a small cylinder or bit of gas carbon moved forward by clockwork so that, as it is burned away by the electric current, it shall constantly maintain its proper relation to the opposing point.</cd> -- <col><b>Carbon paper</b></col>, <cd>a thin type of paper coated with a dark-colored waxy substance which can be transferred to another sheet of paper underneath it by pressing on the carbon paper.  It is used by placing a sheet between two sheets of ordinary writing paper, and then writing or typing on the top sheet, by which process a copy of the writing or typing is transferred to the second sheet below, making a copy without the need for writing or typing a second time.  Multiple sheets may be used, with a carbon paper placed above each plain paper to which an impression is to be transferred.  In 1997 such paper was still used, particularly to make multiple copies of filled-in purchase invoice forms, but in most applications this technique has been superseded by the more faithful xerographic reproduction and computerized printing processes.</cd> -- <col><b>Carbon tissue</b></col>, <cd>paper coated with gelatine and pigment, used in the autotype process of photography.</cd> <au>Abney.</au> -- <col><b>Gas carbon</b></col>, <cd>a compact variety of carbon obtained as an incrustation on the interior of gas retorts, and used for the manufacture of the carbon rods of pencils for the voltaic, arc, and for the plates of voltaic batteries, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>carbon-14</ent><br/
<hw>carbon-14</hw> <pos>n.</pos> <def>a radioactive isotope of carbon with a half-life of 5730 years.  It occurs naturally in minute quantities, and is used as the basis for <er>radiocarbon dating</er>.</def><br/
<syn><b>Syn. --</b> radiocarbon, carbon 14.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carbonaceous</ent><br/
<hw>Car"bo*na`ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, containing, or composed of, carbon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonado</ent><br/
<ent>Carbonade</ent><br/
<mhw>{ <hw>Car"bo*nade</hw> <pr>(?)</pr>, <hw>Car`bo*na"do</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>carbonnade</ets>, It. <ets>carbonata</ets>, Sp. <ets>carbonada</ets>, from L. <ets>carbo</ets> coal.]</ety> <fld>(Cookery)</fld> <def>Flesh, fowl, etc., cut across, seasoned, and broiled on coals; a chop.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonade</ent><br/
<ent>Carbonado</ent><br/
<mhw>{ <hw>Car`bo*na"do</hw> <pr>(?)</pr>, <hw>Car"bo*nade</hw> <pr>(?)</pr>, }</mhw> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carbonadoed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carbonadoing</conjf>.]</vmorph> <sn>1.</sn> <def>To cut (meat) across for frying or broiling; to cut or slice and broil.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A short-legged hen daintily <qex>carbonadoed</qex>.</q> <rj><qau>Bean. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut or hack, as in fighting.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll so <qex>carbonado</qex> your shanks.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonado</ent><br/
<hw>Car`bo*na"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carbonadoes</plw> <pr>(#)</pr>.</plu> <ety>[Pg., carbonated.]</ety> <fld>(Min.)</fld> <def>A black variety of diamond, found in Brazil, and used for diamond drills. It occurs in irregular or rounded fragments, rarely distinctly crystallized, with a texture varying from compact to porous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonarism</ent><br/
<hw>Car`bo*na"rism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The principles, practices, or organization of the Carbonari.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonaro</ent><br/
||<hw>Car`bo*na"ro</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carbonari</plw> <pr>(#)</pr>.</plu> <ety>[It., a coal man.]</ety> <def>A member of a secret political association in Italy, organized in the early part of the nineteenth centry for the purpose of changing the government into a republic.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The origin of the <xex>Carbonari</xex> is uncertain, but the society is said to have first met, in 1808, among the charcoal burners of the mountains, whose phraseology they adopted.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonatation</ent><br/
<hw>Car`bon*a*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Carbonate</er>.]</ety> <fld>(Sugar Making)</fld> <def>The saturation of defecated beet juice with carbonic acid gas.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonate</ent><br/
<hw>Car"bon*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>carbonate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt or carbonic acid, as in limestone, some forms of lead ore, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonated</ent><br/
<hw>Car"bon*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Combined or impregnated with carbonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbone</ent><br/
<hw>Car"bone</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Carbonado</er>.]</ety> <def>To broil. <mark>[Obs.]</mark> <ldquo/We had a calf's head <xex>carboned</xex><rdquo/.</def>  <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonic</ent><br/
<hw>Car*bon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>carbonique</ets>. See <er>Carbon</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or obtained from, carbon; <as>as, <ex>carbonic</ex> oxide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carbonic acid</b></col> <fld>(Chem.)</fld>, <cd>an acid <chform>HO.CO.OH</chform>, not existing separately, which, combined with positive or basic atoms or radicals, forms carbonates. In common language the term is very generally applied to a compound of carbon and oxygen, <chform>CO2</chform>, more correctly called <altname>carbon dioxide</altname>. It is a colorless, heavy, irrespirable gas, extinguishing flame, and when breathed destroys life. It can be reduced to a liquid and solid form by intense pressure. It is produced in the fermentation of liquors, and by the combustion and decomposition of organic substances, or other substances containing carbon. It is formed in the explosion of fire damp in mines, and is hence called <altname>after damp</altname>; it is also know as <altname>choke damp</altname>, and <altname>mephitic air</altname>. Water will absorb its own volume of it, and more than this under pressure, and in this state becomes the common soda water of the shops, and the carbonated water of natural springs. Combined with lime it constitutes limestone, or common marble and chalk. Plants imbibe it for their nutrition and growth, the carbon being retained and the oxygen given out.</cd> -- <col><b>Carbonic oxide</b></col> <fld>(Chem.)</fld>, <cd>a colorless gas, <chform>CO</chform>, of a light odor, called more correctly <altname>carbon monoxide</altname>. It is almost the only definitely known compound in which carbon seems to be divalent. It is a product of the incomplete combustion of carbon, and is an abundant constituent of water gas. It is fatal to animal life, extinguishes combustion, and burns with a pale blue flame, forming carbon dioxide.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 217 --></p>

<p><ent>Carbonide</ent><br/
<hw>Car"bon*ide</hw> <pr>(k<aum/r"b<ocr/n*<icr/d <it>or</it> -<imac/d)</pr>, <pos>n.</pos> <def>A carbide.</def> <mark>[R.]</mark></p>

<p><ent>Carboniferous</ent><br/
<hw>Car`bon*if"er*ous</hw> <pr>(k<aum/r`b<ocr/n*<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[<ets>Carbon</ets> + <ets>-ferous</ets>.]</ety> <def>Producing or containing carbon or coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carboniferous age</b></col> <fld>(Geol.)</fld>, <cd>the age immediately following the <xex>Devonian</xex>, or <xex>Age of fishes</xex>, and characterized by the vegetation which formed the coal beds. This age embraces three periods, the <xex>Subcarboniferous</xex>, the <xex>Carboniferous</xex>, and <xex>Permian</xex>. See <cref>Age of acrogens</cref>, under <er>Acrogen</er>.</cd> -- <col><b>Carboniferous formation</b></col> <fld>(Geol.)</fld>, <cd>the series of rocks (including sandstones, shales, limestones, and conglomerates, with beds of coal) which make up the strata of the <ex>Carboniferous age</ex> or <ex>Carboniferous period</ex>.  See the Diagram under <er>Geology</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>carbonisation</ent><br/
<hw>car`bon*i*sa"tion</hw> <pos>n.</pos> <def>same as <er>carbonization</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Carbonite</ent><br/
<hw>Car"bon*ite</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Carbon</ets> + <ets>-ite</ets>.]</ety> <sn>1.</sn>  <def>An explosive consisting essentially of nitroglycerin, wood meal, and some nitrate, as that of sodium.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>An explosive composed of nitrobenzene, saltpeter, sulphur, and kieselguhr.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carbonization</ent><br/
<hw>Car`bon*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>carbonisation</ets>.]</ety> <sn>1.</sn> <def>The act or process of carbonizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the destructive distillation of coal (as in coke ovens).</def><br/
<syn><b>Syn. --</b> carbonisation.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carbonize</ent><br/
<hw>Car"bon*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carbonized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carbonizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>carboniser</ets>.]</ety> <sn>1.</sn> <def>To convert (an animal or vegetable substance) into a residue of carbon by the action of fire or some corrosive agent; to char.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To impregnate or combine with carbon, as in making steel by cementation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbonometer</ent><br/
<hw>Car`bon*om"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>-meter</ets>.]</ety> <def>An instrument for detecting and measuring the amount of carbon which is present, or more esp. the amount of carbon dioxide, by its action on limewater or by other means.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>carbonous</ent><br/
<hw>carbonous</hw> <pos>adj.</pos>  <def>relating to, containing, or rich in carbon.</def><br/
<syn><b>Syn. --</b> carbonaceous, carbonic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carbon process</ent><br/
<hw>Car"bon process</hw>. <fld>(Photog.)</fld> <def>A printing process depending on the effect of light on bichromatized gelatin. Paper coated with a mixture of the gelatin and a pigment is called <col><b>carbon paper</b></col> <it>or</it> <col><b>carbon tissue</b></col>. This is exposed under a negative and the film is transferred from the paper to some other support and developed by washing (the unexposed portions being dissolved away). If the process stops here it is called <stype>single transfer</stype>; if the image is afterward transferred in order to give an unreversed print, the method is called <stype>double transfer</stype>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carbon steel</ent><br/
<hw>Carbon steel</hw>. <def>Steel deriving its qualities from carbon chiefly, without the presence of other alloying elements; -- opposed to <contr>alloy steel</contr>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carbon transmitter</ent><br/
<hw>Carbon transmitter</hw>. <def>A telephone transmitter in which a carbon contact is used.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carbonyl</ent><br/
<hw>Car"bon*yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The radical (<chform>=CO</chform>), occuring, always combined, in many compounds, as the aldehydes, the ketones, urea, carbonyl chloride, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Though denoted by a formula identical with that of carbon monoxide, it is chemically distinct, as carbon seems to be divalent in carbon monoxide, but tetravalent in carbonyl compounds.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carbonyl chloride</b></col> <fld>(Chem.)</fld>, <cd>a colorless gas, <chform>COCl2</chform>, of offensive odor, and easily condensable to liquid.  It is formed from chlorine and carbon monoxide, under the influence of light, and hence has been called <altname>phosgene</altname>, or <altname>phosgene gas</altname>; -- called also <altname>carbon oxychloride</altname>.  It is used in chemical synthesis, and was also used as a poison gas in World War I.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Carborundum</ent><br/
<hw>Car`bo*run"dum</hw> <pr>(k<aum/r`b<osl/*r<ucr/n"d<ucr/m)</pr>, <ety>[a trade name, from <ets>Carbo</ets>n + <ets>corundum</ets>.]</ety> <def>A beautiful crystalline compound, <altname>silicon carbide</altname> (<chform>SiC</chform>), consisting of carbon and silicon in combination; -- also called <altname>carbon silicide</altname>. It is made by heating carbon and sand together in an electric furnace. The commercial article is dark-colored and iridescent. It is harder than emery, and is used as an abrasive.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>carborundum paper</ent><br/
<ent>carborundum cloth</ent><br/
<mhw>{ <hw>carborundum cloth</hw> <it>or</it> <hw>carborundum paper</hw>  }</mhw>. <def>Cloth or paper covered with powdered carborundum; -- a foprm of <isa>sandpaper</isa>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Carbostyril</ent><br/
<hw>Car`bo*sty"ril</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>styr</ets>ene.]</ety> <def>A white crystalline substance, <chform>C9H6N.OH</chform>, of acid properties derived from one of the amido cinnamic acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carboxide</ent><br/
<hw>Car*box"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>oxide</ets>.]</ety> <fld>(Chem.)</fld> <def>A compound of carbon and oxygen, as carbonyl, with some element or radical; <as>as, potassium <ex>carboxide</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Potassium carboxide</b></col>, <cd>a grayish explosive crystalline compound, <chform>C6O6K</chform>, obtained by passing carbon monoxide over heated potassium.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carboxyl</ent><br/
<hw>Car*box"yl</hw> <pr>(k<aum/r*b<ocr/ks"<icr/l)</pr>, <pos>n.</pos> <ety>[<ets>Carbon</ets> + <ets>oxygen</ets> + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The complex radical, <chform>CO.OH</chform>, regarded as the essential and characteristic constituent which all oxygen acids of carbon (as formic, acetic, benzoic acids, etc.) have in common; -- called also <altname>oxatyl</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>carboxylate</ent><br/
<hw>car*box"yl*ate</hw> <pos>v. t.</pos> <def>to treat a chemical compound with carboxyl or carboxylic acid.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carboxyl group</ent><br/
<hw>car*box"yl group`</hw> <pr>(k<aum/r*b<ocr/ks"<icr/l gr<oomac/p)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>a monovalent organic radical (written <chform>-COOH</chform>, <chform>-CO2H</chform> or <chform>-CO.OH</chform>) whose presence confers acidic properties on the compound containing it; -- called also <altname>carboxylic group</altname>.  Monocarboxylic acids (such as acetic acid or butyric acid) have one carboxyl group, dicarboxylic acids (such as oxalic acid or malonic acid) have two, tricarboxylic acids (such as citric acid) have three.  It is weakly acidic, the pKa of acetic acid being 4.75.</def><br/
[<source>PJC</source>]</p>

<p><ent>carboxylic</ent><br/
<hw>car`box*yl"ic</hw> <pr>(k<aum/r`b<ocr/ks*<icr/l"<icr/k)</pr>, <pos>adj.</pos> <fld>(Chem.)</fld> <def>of or pertaining to the <er>carboxyl group</er>; having a carboxyl group (written <chform>-COOH</chform>, <chform>-CO2H</chform> or <chform>-CO.OH</chform>); <as>as, <ex>carboxylic</ex> acids are weakly acidic</as>.</def><br/
<syn><b>Syn. --</b> carboxyl.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carboxylic acid</ent><br/
<hw>carboxylic acid</hw> <pos>n.</pos> <fld>(Chem.)</fld> <def>an organic compound whose acidity is due to the presence of a <er>carboxyl gorup</er>.  Monocarboxylic acids (such as acetic acid or butyric acid) have one carboxyl group, dicarboxylic acids (such as oxalic acid or malonic acid) have two, tricarboxylic acids (such as citric acid) have three.</def><br/
[<source>PJC</source>]</p>

<p><ent>Carboy</ent><br/
<hw>Car"boy</hw> <pr>(k<aum/r"boi)</pr>, <pos>n.</pos> <ety>[Cf. Ir. & Gael <ets>carb</ets> basket; or Pers <ets>qur<amac/bah</ets> a sort of bottle.]</ety> <def>A large, globular glass bottle, esp. one of green glass, inclosed in basket work or in a box, for protection; -- used commonly for carrying corrosive liquids; as sulphuric acid, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbuncle</ent><br/
<hw>Car"bun*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>carbunculus</ets> a little coal, a bright kind of precious stone, a kind of tumor, dim. of <ets>carbo</ets> coal: cf. F. <ets>carboncle</ets>. See <er>Carbon</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Min.)</fld> <def>A beautiful gem of a deep red color (with a mixture of scarlet) called by the Greeks <xex>anthrax</xex>; found in the East Indies. When held up to the sun, it loses its deep tinge, and becomes of the color of burning coal. The name belongs for the most part to ruby sapphire, though it has been also given to red spinel and garnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A very painful acute local inflammation of the subcutaneous tissue, esp. of the trunk or back of the neck, characterized by brawny hardness of the affected parts, sloughing of the skin and deeper tissues, and marked constitutional depression. It differs from a boil in size, tendency to spread, and the absence of a central core, and is frequently fatal. It is also called <altname>anthrax</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Her.)</fld> <def>A charge or bearing supposed to represent the precious stone. It has eight scepters or staves radiating from a common center. Called also <altname>escarbuncle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbuncled</ent><br/
<hw>Car"bun*cled</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Set with carbuncles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He has deserves it [armor], were it <qex>carbuncled</qex><br/
Like holy Phabus' car.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected with a carbuncle or carbuncles; marked with red sores; pimpled and blotched.</def> <ldquo/A <xex>carbuncled</xex> face.<rdquo/  <rj><au>Brome.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbuncular</ent><br/
<hw>Car*bun"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to a carbuncle; resembling a carbuncle; red; inflamed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbunculation</ent><br/
<hw>Car*bun`cu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>carbunculatio</ets>.]</ety> <def>The blasting of the young buds of trees or plants, by excessive heat or cold.</def>  <rj><au>Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carburet</ent><br/
<hw>Car"bu*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Carbon</er>.]</ety> <fld>(Chem.)</fld> <def>A carbide. See <er>Carbide</er></def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carburet</ent><br/
<hw>Car"bu*ret</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carbureted</conjf> or <conjf>Carburetted</conjf> (<?/); <pos>p. pr. & vb. n.</pos> <conjf>Carbureting</conjf> or <conjf>Carburetting</conjf>.]</vmorph> <def>To combine or to impregnate with carbon, as by passing through or over a liquid hydrocarbon; to carbonize or carburize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>carbureting</qex> the gas you may use poorer coal.</q> <rj><qau>Knight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carburetant</ent><br/
<hw>Car"bu*ret`ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any volatile liquid used in charging illuminating gases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carbureted</ent><br/
<hw>Car"bu*ret`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>Combined with carbon in the manner of a carburet or carbide.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Saturated or impregnated with some volatile carbon compound; <as>as, water gas is <ex>carbureted</ex> to increase its illuminating power</as>.</def></p>

<p><altsp>[Written also <asp>carburetted</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carbureted hydrogen gas</b></col>, <cd>any one of several gaseous compounds of carbon and hydrogen, some of with make up illuminating gas.</cd> -- <col><b>Light carbureted hydrogen</b></col>, <cd>methane (<chform>CH4</chform>), also called <altname>marsh gas</altname>, and <altname>fire damp</altname>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Carburettor</ent><br/
<ent>Carburetor</ent><br/
<mhw>{ <hw>Car"bu*ret`or</hw>, <hw>Car"bu*ret`tor</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>An apparatus in which coal gas, hydrogen, or air is passed through or over a volatile hydrocarbon, in order to confer or increase illuminating power.</def> <altsp>[Written also <asp>carburettor</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One that carburets; specif., an apparatus in which air or gas is carbureted, as by passing it through a light petroleum oil. The carburetor for a gasoline engine is usually either a <stype><col><b>surface carburetor</b></col></stype>, or alternatively a <stype><col><b>float carburetor</b></col></stype> (called also <stype><col><b>float-feed carburetor</b></col></stype>, <it>or</it>  <stype><col><b>spray carburetor</b></col></stype>). In the former air is charged by being passed over the surface of gasoline. In the latter a fine spray of gasoline is drawn from an atomizing nozzle by a current of air induced by the suction of the engine piston, the supply of gasoline being regulated by a float which actuates a needle valve controlling the outlet of the feed pipe. Alcohol and other volatile inflammable liquids may be used instead of gasoline.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carburization</ent><br/
<hw>Car"bu*ri*za`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The act, process, or result of carburizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carburize</ent><br/
<hw>Car"bu*rize</hw> <pr>(k<aum/r"b<usl/*r<imac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carburized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carburizing</conjf>.]</vmorph> <fld>(Chem.)</fld> <def>To combine with carbon or a carbon compound; -- said esp. of a process for conferring a higher degree of illuminating power on combustible gases by mingling them with a vapor of volatile hydrocarbons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcajou</ent><br/
<hw>Car"ca*jou</hw> <pr>(k<aum/r"k<adot/*j<oomac/)</pr>, <pos>n.</pos> <ety>[Probably a Canadian French corruption of an Indian name of the wolverene.]</ety> <fld>(Zool.)</fld> <def>The wolverene; -- also applied, but erroneously, to the Canada lynx, and sometimes to the American badger. See <er>Wolverene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcanet</ent><br/
<hw>Car"ca*net</hw> <pr>(k<aum/r"k<adot/*n<ecr/t)</pr>, <pos>n.</pos> <ety>[Dim. fr. F. <ets>carcan</ets> the iron collar or chain of a criminal, a chain of precious stones, LL. <ets>carcannum</ets>, fr. Armor.  <ets>kerchen</ets> bosom, neck, <ets>kelchen</ets> collar, fr. <ets>kelch</ets> circle; or Icel. <ets>kverk</ets> troat, OHG. <ets>querca</ets> throat.]</ety> <def>A jeweled chain, necklace, or collar.</def> <altsp>[Also written <asp>carkanet</asp> and <asp>carcant</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcase</ent><br/
<hw>Car"case</hw> <pr>(k<aum/r"k<ait/s)</pr>, <pos>n.</pos> <def>See <er>Carcass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcass</ent><br/
<hw>Car"cass</hw> <pr>(k<aum/r"k<ait/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carcasses</plw> <pr>(#)</pr>.</plu> <altsp>[Written also <asp>carcase</asp>.]</altsp> <ety>[F. <ets>carcasse</ets>, fr. It. <ets>carcassa</ets>, fr. L. <ets>caro</ets> flesh + <ets>capsa</ets> chest, box, case. Cf. <er>Carnal</er>, <er>Case</er> a sheath.]</ety> <sn>1.</sn> <def>A dead body, whether of man or beast; a corpse; now commonly the dead body of a beast.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He turned to see the <qex>carcass</qex> of the lion.</q> <rj><qau>Judges xiv. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This kept thousands in the town whose <qex>carcasses</qex> went into the great pits by cartloads.</q> <rj><qau>De Foe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The living body; -- now commonly used in contempt or ridicule.</def> <ldquo/To pamper his own <xex>carcass</xex>.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lovely her face; was ne'er so fair a creature.<br/
For earthly <qex>carcass</qex> had a heavenly feature.</q> <rj><qau>Oldham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The abandoned and decaying remains of some bulky and once comely thing, as a ship; the skeleton, or the uncovered or unfinished frame, of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A rotten <qex>carcass</qex> of a boat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>A hollow case or shell, filled with combustibles, to be thrown from a mortar or howitzer, to set fire to buldings, ships, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A discharge of <qex>carcasses</qex> and bombshells.</q> <rj><qau>W. Iving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcavelhos</ent><br/
||<hw>Car`ca*vel"hos</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sweet wine. See <er>Calcavella</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcel</ent><br/
<hw>Car"cel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Photom.)</fld> <def>A light standard much used in France, being the light from a Carcel lamp of stated size and construction consuming 42 grams of colza oil per hour with a flame 40 millimeters in height. Its illuminating power is variously stated at from 8.9 to 9.6 British standard candles.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carcelage</ent><br/
<hw>Car"ce*lage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>carcelladium</ets>, <ets>carceragium</ets>, fr. L. <ets>carcer</ets> prison.]</ety> <def>Prison fees.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcel lamp</ent><br/
<hw>Car"cel lamp`</hw> <pr>(?)</pr>. <ety>[Named after <er>Carcel</er>, the inventor.]</ety> <def>A French mechanical lamp, for lighthouses, in which a superabundance of oil is pumped to the wick tube by clockwork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carceral</ent><br/
<hw>Car"cer*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carceralis</ets>, fr. <ets>carcer</ets> prison.]</ety> <def>Belonging to a prison.</def> <mark>[R.]</mark>  <rj><au>Foxe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcharias</ent><br/
<hw>Carcharias</hw> <pos>prop. n.</pos> <def>The type and sole genus of <fam>Carchariidae</fam>; the sand sharks.</def><br/
<syn><b>Syn. --</b> genus <gen>Carcharias</gen>, Odontaspis, genus <gen>Odontaspis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carchariidae</ent><br/
<hw>Carchariidae</hw> <pos>prop. n.</pos> <def>a natural family comprising the sand sharks; in some classifications coextensive with the family <fam>Carcharhinidae</fam>.</def><br/
<syn><b>Syn. --</b> family <fam>Carchariidae</fam>, <fam>Odontaspididae</fam>, family <fam>Odontaspididae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carcharodon</ent><br/
<hw>Carcharodon</hw> <pos>prop. n.</pos> <def>A genus of sharks including man-eating sharks, such as the great white shark, <spn>Carcharodon carcharias</spn>.</def><br/
<syn><b>Syn. --</b> genus <gen>Carcharodon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carcino-</ent><br/
<hw>carcino-</hw> <pos>pref.</pos> <ety>[From Gr. <grk>karki`nos</grk>, a crab.]</ety> <def>A combining form meaning <sig>cancer</sig>, as in <ex>carcino</ex>gen.</def><br/
[<source>PJC</source>]</p>

<p><ent>carcinogen</ent><br/
<hw>car*cin"o*gen</hw> <pr>(k<aum/r*s<icr/n"<osl/*j<ecr/n)</pr>, <pos>n.</pos> <def>Any substance that produces cancer; <as>as, to test for <ex>carcinogens</ex> in the drinking water</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carcinogenic</ent><br/
<hw>car`cin*o*gen"ic</hw> <pr>(k<aum/r`s<icr/n*<osl/*j<ecr/n"<icr/k)</pr>, <pos>adj.</pos> <def>Causing or tending to cause cancer; <as>as, the <ex>carcinogenic</ex> action of certain chemicals or of ionizing radiation</as>.</def> -- <wordforms><wf>car`cin*o*gen*ic"i*ty</wf>, <pos>n.</pos></wordforms><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>carcinoid</ent><br/
<hw>carcinoid</hw> <ety>[From Gr. <grk>karki`nos</grk>, a crab + <ets>-oid</ets>.]</ety> <pos>n.</pos> <def>A small tumor (benign or malignant) arising from the mucosa of the gastrointestinal tract; it is usually associated with excessive secretion of serotonin.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carcinological</ent><br/
<hw>Car`ci*no*log"ic*al</hw> <pr>(k<aum/r`s<icr/*n<ocr/*l<ocr/j"<icr/*k<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to carcinology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcinology</ent><br/
<hw>Car`ci*nol"o*gy</hw> <pr>(k<aum/r`s<icr/*n<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karki`nos</grk> a crab + <ets>-logy</ets>.]</ety> <fld>(Zool.)</fld> <def>The department of Zoology which treats of the <class>Crustacea</class> (lobsters, crabs, etc.); -- called also <altname>malacostracology</altname> and <altname>crustaceology</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>carcinoma</ent><br/
<hw>car`ci*no"ma</hw> <pr>(k<aum/r`s<icr/*n<omac/"m<adot/)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>karki`nwma</grk>, fr. <grk>karki`nos</grk> crab, cancer. See <er>-oma</er>.]</ety> <fld>(Med.)</fld> <def>A form of malignant cancer arising from epithelial tissue.  The term was earlier applied to all forms of cancer, or to certain non-malignant forms. It is contrasted with <contr>sarcoma</contr>, a malignant form of cancer arising from connective tissue.  See <er>Cancer</er>.</def>  <rj><au>Dunglison.</au>  <au>Stedman.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Carcinomatous</ent><br/
<hw>Car`ci*nom"a*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to carcinoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carcinosys</ent><br/
||<hw>Car`ci*no"sys</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>karki`nos</grk> cancer.]</ety> <def>The affection of the system with cancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Card</ent><br/
<hw>Card</hw> <pr>(k<aum/rd)</pr>, <pos>n.</pos> <ety>[F. <ets>carte</ets>, fr. L. <ets>charta</ets> paper, Gr. <?/ a leaf of paper. Cf. <er>Chart</er>.]</ety> <sn>1.</sn> <def>A piece of pasteboard, or thick paper, blank or prepared for various uses; <as>as, a playing <ex>card</ex>; a visiting <ex>card</ex>; a <ex>card</ex> of invitation</as>; <pluf>pl.</pluf> a game played with cards.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our first <qex>cards</qex> were to Carabas House.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A published note, containing a brief statement, explanation, request, expression of thanks, or the like; <as>as, to put a <ex>card</ex> in the newspapers</as>. Also, a printed programme, and (<xex>fig</xex>.), an attraction or inducement; <as>as, this will be a good <ex>card</ex> for the last day of the fair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A paper on which the points of the compass are marked; the dial or face of the mariner's compass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the quartere that they know<br/
I' the shipman's <qex>card</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Weaving)</fld> <def>A perforated pasteboard or sheet-metal plate for warp threads, making part of the Jacquard apparatus of a loom. See <er>Jacquard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An indicator card. See under <er>Indicator</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Business card</b></col>, <cd>a card on which is printed an advertisement or business address.</cd> -- <col><b>Card basket</b></col> <sd>(a)</sd> <cd>A basket to hold visiting cards left by callers.</cd> <sd>(b)</sd> <cd>A basket made of cardboard.</cd> -- <col><b>Card catalogue</b></col>. <cd>See <er>Catalogue</er>.</cd> -- <col><b>Card rack</b></col>, <cd>a rack or frame for holding and displaying business or visiting card.</cd> -- <col><b>Card table</b></col>, <cd>a table for use inplaying cards, esp. one having a leaf which folds over.</cd> -- <col><b>On the cards</b></col>, <cd>likely to happen; foretold and expected but not yet brought to pass; -- a phrase of fortune tellers that has come into common use; also, according to the programme.</cd> -- <col><b>Playing card</b></col>, <cd>cards used in playing games; specifically, the cards cards used playing which and other games of chance, and having each pack divided onto four kinds or suits called hearts, diamonds, clubs, and spades. The full or whist pack contains fifty-two cards.</cd> -- <col><b>To have the cards in one's own hands</b></col>, <cd>to have the winning cards; to have the means of success in an undertaking.</cd> -- <col><b>To play one's cards well</b></col>, <cd>to make no errors; to act shrewdly.</cd> -- <col><b>To play snow one's cards</b></col>, <cd>to expose one's plants to rivals or foes.</cd> -- <col><b>To speak by the card</b></col>, <cd>to speak from information and definitely, not by guess as in telling a ship's bearing by the compass card.</cd> -- <col><b>Visiting card</b></col>, <cd>a small card bearing the name, and sometimes the address, of the person presenting it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Card</ent><br/
<hw>Card</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Carding</conjf>.]</vmorph> <def>To play at cards; to game.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Card</ent><br/
<hw>Card</hw>, <pos>n.</pos> <ety>[F. <ets>carde</ets> teasel, the head of a thistle, card, from L. <ets>carduus</ets>, <ets>cardus</ets>, thistle, fr. <ets>carere</ets> to card.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An instrument for disentangling and arranging the fibers of cotton, wool, flax, etc.; or for cleaning and smoothing the hair of animals; -- usually consisting of bent wire teeth set closely in rows in a thick piece of leather fastened to a back.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A roll or sliver of fiber (as of wool) delivered from a carding machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Card clothing</b></col>, <cd>strips of wire-toothed card used for covering the cylinders of carding machines.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Card</ent><br/
<hw>Card</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To comb with a card; to cleanse or disentangle by carding; <as>as, to <ex>card</ex> wool; to <ex>card</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>card</qex> the short comb the longer flakes.</q> <rj><qau>Dyer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To clean or clear, as if by using a card.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This book [must] be <qex>carded</qex> and purged.</q> <rj><qau>T. Shelton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To mix or mingle, as with an inferior or weaker article.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>card</qex> your beer, if you guests being to be drunk. -- half small, half strong.</q> <rj><qau>Greene.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the manufacture of wool, cotton, etc., the process of carding disentangles and collects together all the fibers, of whatever length, and thus differs from combing, in which the longer fibers only are collected, while the short straple is combed away. See <er>Combing</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardamine</ent><br/
<hw>Car"da*mine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cardamina</ets>, Gr. <?/: cf. F. <ets>cardamine</ets>.]</ety> <fld>(Bot.)</fld> <def>A genus of cruciferous plants, containing the lady's-smock, cuckooflower, bitter cress, meadow cress, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardamom</ent><br/
<hw>Car"da*mom</hw> <pr>(k<aum/r"d<adot/*m<ucr/m)</pr>, <pos>n.</pos> <ety>[L. <ets>cardamomun</ets>, Gr. <grk>karda`mwmon</grk>]</ety> <sn>1.</sn> <def>The aromatic fruit, or capsule with its seeds, of several plants of the Ginger family growing in the East Indies and elsewhere, and much used as a condiment, and in medicine.</def> <altsp>[Also spelled <asp>cardamum</asp>.]</altsp><br/
<syn><b>Syn. --</b> cardamon.</syn><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A rhizomatous herb which produces cardamoms, esp. <spn>Elettaria Cardamomum</spn> and several species of <gen>Amomum</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cardamon</ent><br/
<hw>cardamon</hw> <pos>n.</pos> <def>same as <er>cardamom</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cardboard</ent><br/
<hw>Card"board`</hw> <pr>(k<aum/rd"b<omac/rd`)</pr>, <pos>n.</pos> <def>A stiff compact pasteboard of various qualities, for making cards, etc., often having a polished surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardcase</ent><br/
<hw>Card"case`</hw> <pr>(k<aum/rd"k<amac/s`)</pr>, <pos>n.</pos> <def>A case for visiting cards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardecu</ent><br/
<hw>Car"de*cu</hw> <pr>(k<aum/r"d<esl/*k<usl/)</pr>, <pos>n.</pos> <ety>[Corrupt, from F. <ets>quart d'<eacute/cu</ets>.]</ety> <def>A quarter of a crown.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The bunch of them were not worth a <qex>cardecu</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carder</ent><br/
<hw>Card"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which cards wool flax, etc.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardia</ent><br/
<hw>Car"di*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ heart, or upper orifice of the stomach.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The heart.</def> <sd>(b)</sd> <def>The anterior or cardiac orifice of the stomach, where the esophagus enters it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiac</ent><br/
<hw>Car"di*ac</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cardiacus</ets>, Gr. <ets><?/</ets>, fr. <?/ heart: cf. F. <ets>cardiaque</ets>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>Pertaining to, resembling, or hear the heart; <as>as, the <ex>cardiac</ex> arteries; the <ex>cardiac</ex>, or left, end of the stomach</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Exciting action in the heart, through the medium of the stomach; cordial; stimulant.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cardiac passion</b></col> <fld>(Med.)</fld> <cd>cardialgia; heartburn.</cd> <mark>[Archaic]</mark> -- <col><b>Cardiac wheel</b></col>. <fld>(Mach.)</fld> <cd>See <er>Heart wheel</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiac</ent><br/
<hw>Car"di*ac</hw> <pos>n.</pos> <fld>(Med.)</fld> <def>A medicine which excites action in the stomach; a cardial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiacal</ent><br/
<hw>Car*di"a*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cardiac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiacle</ent><br/
<hw>Car"di*a*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pain about the heart.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiagraph</ent><br/
<hw>Car"di*a*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cardiograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardialgy</ent><br/
<ent>Cardialgla</ent><br/
<mhw>{ ||<hw>Car`di*al"gl*a</hw> <pr>(?)</pr>, <hw>Car"di*al`gy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>cardialgia</ets>, fr. Gr. <?/; <?/ heart + <?/ pain: cf. F. <ets>cardialgie</ets>.]</ety> <fld>(Med.)</fld> <def>A burning or gnawing pain, or feeling of distress, referred to the region of the heart, accompanied with cardiac palpitation; heartburn. It is usually a symptom of indigestion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardigan jacket</ent><br/
<ent>Cardigan</ent><br/
<mhw><hw>Car"di*gan</hw>, <hw>Car"di*gan jack`et</hw></mhw> <pr>(#)</pr>. <ety>[From the Earl of <etsep>Cardigan</etsep>, who was famous in the Crimean campaign of 1854-55.]</ety> <sn>1.</sn> <def>A warm jacket of knit worsted with or without sleeves, especially a knitted jacket with sleeves that is fastened up the front with buttons or a zipper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardigan</ent><br/
<hw>Car"di*gan</hw> <pr>(#)</pr>, <def>a slightly bow-legged variety of corgi having rounded ears and a long tail.</def><br/
<syn><b>Syn. --</b> Cardigan Welsh corgi.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cardiidae</ent><br/
<hw>Cardiidae</hw> <pos>n.</pos> <def>a natural family of somewhat heart-shaped sand-burrowing bivalve mollusks.</def><br/
<syn><b>Syn. --</b> family <fam>Cardiidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cardinal</ent><br/
<hw>Car"di*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cardinalis</ets>, fr. <ets>cardo</ets> the hinge of a door, that on which a thing turns or depends: cf. F. <ets>cardinal</ets>.]</ety> <def>Of fundamental importance; pre<eum/minent; superior; chief; principal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cardinal</qex> intersections of the zodiac.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Impudence is now a <qex>cardinal</qex> virtue.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But <qex>cardinal</qex> sins, and hollow hearts, I fear ye.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cardinal numbers</b></col>, <cd>the numbers <member>one</member>, <member>two</member>, <member>three</member>, etc., in distinction from <xex>first</xex>, <xex>second</xex>, <xex>third</xex>, etc., which are called <cref>ordinal numbers</cref>.</cd> -- <col><b>Cardinal points</b></col> <sd>(a)</sd> <fld>(Geol.)</fld> <cd>The four principal points of the compass, or intersections of the horizon with the meridian and the prime vertical circle, north, south east, and west.</cd> <sd>(b)</sd> <fld>(Astrol.)</fld> <cd>The rising and setting of the sun, the zenith and nadir.</cd> -- <col><b>Cardinal signs</b></col> <fld>(Astron.)</fld> <cd>Aries, Libra, Cancer, and Capricorn.</cd> -- <col><b>Cardinal teeth</b></col> <fld>(Zool.)</fld>, <cd>the central teeth of bivalve shell. See <er>Bivalve</er>.</cd> -- <col><b>Cardinal veins</b></col> <fld>(Anat.)</fld>, <cd>the veins in vertebrate embryos, which run each side of the vertebral column and returm the blood to the heart. They remain through life in some fishes.</cd> -- <col><b>Cardinal virtues</b></col>, <cd>pre<eum/minent virtues; among the ancients, prudence, justice, temperance, and fortitude.</cd> -- <col><b>Cardinal winds</b></col>, <cd>winds which blow from the cardinal points due north, south, east, or west.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardinal</ent><br/
<hw>Car"di*nal</hw>, <pos>n.</pos> <ety>[F. <ets>carinal</ets>, It. <ets>cardinale</ets>, LL. <ets>cardinalis</ets> (ecclesi<ae/ Roman<ae/). See <er>Cardinal</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <fld>(R. C. Ch.)</fld> <def>One of the ecclesiastical princes who constitute the pope's council, or the sacred college.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The clerics of the supreme Chair are called <qex>Cardinals</qex>, as undoubtedly adhering more nearly to the hinge by which all things are moved.</q> <rj><qau>Pope Leo IX.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>cardinals</xex> are appointed by the pope. Since the time of Sixtus V., their number can never exceed seventy (six of episcopal rank, fifty priests, fourteen deacons), and the number of cardinal priests and deacons is seldom full. When the papel chair is vacant a pope is elected by the college of cardinals from among themselves. The cardinals take precedence of all dignitaries except the pope. The principal parts of a cardinal's costume are a red cassock, a rochet, a short purple mantle, and a red hat with a small crown and broad brim, with cords and tessels of a special pattern hanging from it.</note><-- the number was expanded -- when? --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A woman's short cloak with a hood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where's your <qex>cardinal</qex>! Make haste.</q> <rj><qau>Lloyd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mulled red wine.</def>  <rj><au>Hotten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>the <cref>cardinal bird</cref>, also called the <altname>northern cardinal</altname>.</def><br/
[<source>PJC</source>]</p>

<p><-- p. 218 --></p>

<p><cs><mcol><col><b>Cardinal bird</b></col>, <it>or</it> <col><b>Cardinal grosbeak</b></col></mcol> <fld>(Zool.)</fld>, <cd>an American song bird (<spn>Cardinalis cardinalis</spn>, or <spn>Cardinalis Virginianus</spn>), of the family <fam>Fringillid<ae/</fam>, or finches of which the male has a bright red plumage, and both sexes have a high, pointed crest on its head; -- it is also called the <altname>northern cardinal</altname> or <altname>eastern cardinal</altname>.  The males have loud and musical notes resembling those of a fife.  Other related species are also called <ex>cardinal birds</ex>.</cd> -- <col><b>Cardinal flower</b></col> <fld>(Bot.)</fld>, <cd>an herbaceous plant (<spn>Lobelia cardinalis</spn>) bearing brilliant red flowers of much beauty.</cd> -- <col><b>Cardinal red</b></col>, <cd>a color like that of a cardinal's cassock, hat, etc.; a bright red, darker than scarlet, and between scarlet and crimson.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardinalate</ent><br/
<hw>Car"di*nal*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cardinalat</ets>, LL. <ets>cardinalatus</ets>.]</ety> <def>The office, rank, or dignity of a cardinal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardinalize</ent><br/
<hw>Car"di*nal*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To exalt to the office of a cardinal.</def>  <rj><au>Sheldon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardinalship</ent><br/
<hw>Car"di*nal*ship</hw>, <pos>n.</pos> <def>The condition, dignity, of office of a cardinal</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carding</ent><br/
<hw>Card"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>The act or process of preparing staple for spinning, etc., by carding it. See the Note under <er>Card</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A roll of wool or other fiber as it comes from the carding machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Carding engine</b></col>, <col><b>Carding machine</b></col></mcol>, <cd>a machine for carding cotton, wool, or other fiber, by subjecting it to the action of cylinders, or drums covered with wire-toothed cards, revolving nearly in contact with each other, at different rates of speed, or in opposite directions.  The staple issues in soft sheets, or in slender rolls called <prodmac>slivers</prodmac>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cardiogram</ent><br/
<hw>car"di*o*gram`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kardi`a</grk> heart + <ets>-gram</ets>.]</ety> <fld>(Physiol.)</fld> <def>Any instrumental record made of heart activity, especially the curve or tracing made by a cardiograph.  Sometimes used synonymously with <altname>electrocardiogram</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>cardiography</ent><br/
<hw>car`di*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Description of the heart.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Physiol.)</fld> <def>Examination of heart activity using a cardiograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cardiograph</ent><br/
<hw>car"di*o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kardi`a</grk> heart + <ets>-graph</ets>.]</ety> <fld>(Med.)</fld> <def>An instrument which, when placed in contact with the chest, will register graphically the comparative duration and intensity of the heart's movements.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiographic</ent><br/
<hw>Car`di*o*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Of or pertaining to, or produced by, a cardiograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardioid</ent><br/
<hw>Car"di*oid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kardio-eidh`s</grk> heart-shaped; <grk>kardi`a</grk> heart + <grk>e'i^dos</grk> shape.]</ety> <fld>(Math.)</fld> <def>An algebraic curve, so called from its resemblance to a heart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardioinhibitory</ent><br/
<hw>Car`di*o*in*hib"i*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Checking or arresting the heart's action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiolgy</ent><br/
<hw>Car`di*ol"*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kardi`a</grk> heart + <ets>-ology</ets>.]</ety> <def>The science which treats of the heart and its functions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardiometry</ent><br/
<hw>Car`di*om"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kardi`a</grk> heart + <ets>-metry</ets>.]</ety> <fld>(Med.)</fld> <def>Measurement of the heart, as by percussion or auscultation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cardiorespiratory</ent><br/
<ent>cardiopulmonary</ent><br/
<mhw><hw>cardiopulmonary</hw> <hw>cardiorespiratory</hw></mhw> <pos>adj.</pos> <def>Of or pertaining to or affecting both the heart and the lungs and their functions; <as>as, <ex>cardiopulmonary</ex> resuscitation</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cardiopulmonary resuscitation</ent><br/
<hw>cardiopulmonary resuscitation</hw> <pos>n.</pos> <def>An emergency procedure to revive heart and lung function in persons whose heart has apparently stopped beating, involving forced respiration, periodic pressure on the heart by pushing on the chest, and sometimes electrical or mechanical equipment.  It is often referred to by the acronym <altname>CPR</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>cardiosclerosis</ent><br/
||<hw>car"di*o*scle*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kardi`a</grk> heart + <ets>sclerosis</ets>.]</ety> <def>Induration of the heart, caused by development of fibrous tissue in the cardiac muscle.</def> <mark>[Obsolescent]</mark> <br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Cardiospermum</ent><br/
<hw>Car"di*o*sper`mum</hw> <pos>prop. n.</pos> <def>A genus of tendril-climbing herbs or shrubs whose seeds have a white heart-shaped spot.</def><br/
<syn><b>Syn. --</b> genus <gen>Cardiospermum</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cardiosphygmograph</ent><br/
<hw>Car`di*o*sphyg"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A combination of cardiograph and sphygmograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cardiovascular</ent><br/
<hw>cardiovascular</hw> <pos>adj.</pos> <def>Of or pertaining to or involving the heart and blood vessels; <as>as, <ex>cardiovascular</ex> conditioning</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carditis</ent><br/
||<hw>Car*di"tis</hw> <pr>(k<aum/r*d<imac/"t<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kardi`a</grk> heart + <ets>-itis</ets>: cf. F. <ets>cardite</ets>.]</ety> <fld>(Med)</fld> <def>Inflammation of the fleshy or muscular substance of the heart. See <er>Endocarditis</er> and <er>Pericarditis</er>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardo</ent><br/
||<hw>Car"do</hw> <pr>(k<aum/r"d<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cardines</plw> <pr>(#)</pr>.</plu>) <ety>[L., a hinge.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The basal joint of the maxilla in insects.</def> <sd>(b)</sd> <def>The hinge of a bivalve shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardol</ent><br/
<hw>Car"dol</hw> <pr>(k<aum/r"d<omac/l)</pr>, <pos>n.</pos> <ety>[NL. Ana<ets>card</ets>ium generic name of the cashew + L. <ets>oleum</ets> oil.]</ety> <fld>(Chem.)</fld> <def>A yellow oily liquid, extracted from the shell of the cashew nut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cardoon</ent><br/
<hw>Car*doon"</hw> <pr>(k<aum/r*d<oomac/n")</pr>, <pos>n.</pos> <ety>[F. <ets>cardon</ets>. The same word as F. <ets>cardon</ets> thistle, fr. L. <ets>carduus</ets>, <ets>cardus</ets>, LL. <ets>cardo</ets>. See 3d <er>Card</er>.]</ety> <fld>(Bot.)</fld> <def>A large herbaceous plant (<spn>Cynara Cardunculus</spn>) related to the artichoke; -- used in cookery and as a salad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cardsharper</ent><br/
<ent>cardsharp</ent><br/
<mhw><hw>card"sharp`</hw>, <hw>card"sharp`er</hw></mhw> <pos>n.</pos> <def>a professional card player who makes a living by cheating at card games.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carduelinae</ent><br/
<hw>Carduelinae</hw> <pos>n.</pos> <def>a subfamily of birds consisting of the old world finches, many of which have taken up residence in the Americas; -- it is a subgroup of finches used in some classifications, in which the <fam>Fringillidae</fam> are divided into two subfamilies, the <fam>Carduelinae</fam> (including the goldfinches, siskins, redpolls, crossbills, and linnets) and <fam>Fringillinae</fam> (the brambling and the chaffinches).  The siskins and goldfinches have migrated as far south as the southern end of South America.</def> <au>T. M. Shortt.</au><br/
<syn><b>Syn. --</b> subfamily <fam>Carduelinae</fam>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Carduelis</ent><br/
<hw>Carduelis</hw> <pos>n.</pos> <def>the type genus of a subfamily <fam>Carduelinae</fam> of the <fam>Fringillidae</fam>, used in some classifications.  It includes the goldfinches (such as the American goldfinch, <spn>Cardulis tristis</spn>, and the European goldfinch, <spn>Carduelis carduelis</spn>); siskins; redpolls; and linnets, as well as the common northern cardinal (<spn>Carduelis cardinalis</spn>, also called the <cref>cardinal bird</cref>).</def><br/
<syn><b>Syn. --</b> genus <gen>Carduelis</gen>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Carduus</ent><br/
<hw>Carduus</hw> <pos>n.</pos> <def>a genus of annual or perennial Old World prickly thistles.</def><br/
<syn><b>Syn. --</b> genus Carduus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Care</ent><br/
<hw>Care</hw> <pr>(k<acir/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>caru</ets>, <ets>cearu</ets>; akin to OS. <ets>kara</ets> sorrow, Goth. <ets>kara</ets>, OHG <ets>chara</ets>, lament, and perh. to Gr. <grk>gh^rys</grk> voice. Not akin to <it>cure</it>. Cf. <er>Chary</er>.]</ety> <sn>1.</sn> <def>A burdensome sense of responsibility; trouble caused by onerous duties; anxiety; concern; solicitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Care</qex> keeps his watch in every old man's eye,<br/
And where <qex>care</qex> lodges, sleep will never lie.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Charge, oversight, or management, implying responsibility for safety and prosperity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>care</qex> of all the churches.</q> <rj><qau>2 Cor. xi. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Him thy <qex>care</qex> must be to find.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Perplexed with a thousand <qex>cares</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Attention or heed; caution; regard; heedfulness; watchfulness; <as>as, take <ex>care</ex>; have a <ex>care</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I thank thee for thy <qex>care</qex> and honest pains.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The object of watchful attention or anxiety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Right sorrowfully mourning her bereaved <qex>cares</qex>.</q> <rj><qau>Spenser.</qau></rj></p>

<p><syn><b>Syn.</b> -- Anxiety; solicitude; concern; caution; regard; management; direction; oversight. -- <er>Care</er>, <er>Anxiety</er>, <er>Solicitude</er>, <er>Concern</er>. These words express mental pain in different degress. <xex>Care</xex> belongs primarily to the intellect, and becomes painful from overburdened thought. <xex>Anxiety</xex> denotes a state of distressing uneasiness fron the dread of evil. <xex>Solicitude</xex> expresses the same feeling in a diminished degree. <xex>Concern</xex> is opposed to <xex>indifference</xex>, and implies exercise of anxious thought more or less intense. We are <xex>careful</xex> about the means, <xex>solicitous</xex> and <xex>anxious</xex> about the end; we are <xex>solicitous</xex> to obtain a good, <xex>anxious</xex> to avoid an evil.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Care</ent><br/
<hw>Care</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caring</conjf>.]</vmorph> <ety>[AS. <ets>cearian</ets>. See <er>Care</er>, <pos>n.</pos>]</ety> <def>To be anxious or solicitous; to be concerned; to have regard or interest; -- sometimes followed by an objective of measure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would not <qex>care</qex> a pin, if the other three were in.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Master, <qex>carest</qex> thou not that we perish?</q> <rj><qau>Mark. iv. 38.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To care for</b></col>. <sd>(a)</sd> <cd>To have under watchful attention; to take care of.</cd> <sd>(b)</sd> <cd>To have regard or affection for; to like or love.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>cared</qex> not for the affection of the house.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cared-for</ent><br/
<hw>cared-for</hw> <pos>adj.</pos> <def>having needed care and attention; <as>as, well-<ex>cared-for</ex> children</as>.  Opposite of <ant>uncared-for</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Careen</ent><br/
<hw>Ca*reen"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Careened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Careening</conjf>.]</vmorph> <ety>[OF. <ets>cariner</ets>, F. <ets>car<eacute/ner</ets>, fr. OF. <ets>car<egrave/ne</ets>, the bottom of a ship, keel, fr. L. <ets>carina</ets>.]</ety> <fld>(Naut.)</fld> <def>To cause (a vessel) to lean over so that she floats on one side, leaving the other side out of water and accessible for repairs below the water line; to case to be off the keel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Careen</ent><br/
<hw>Ca*reen"</hw> <pr>(<?/)</pr>, <pos>v. i.</pos> <def>To incline to one side, or lie over, as a ship when sailing on a wind; to be off the keel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Careenage</ent><br/
<hw>Ca*reen"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>car<eacute/nage</ets>.]</ety> <fld>(Naut.)</fld> <sd>(a)</sd> <def>Expense of careening ships.</def> <sd>(b)</sd> <def>A place for careening.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Career</ent><br/
<hw>Ca*reer"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carri<egrave/re</ets> race course, high road, street, fr. L. <ets>carrus</ets> wagon. See <er>Car</er>.]</ety> <sn>1.</sn> <def>A race course: the ground run over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To go back again the same <qex>career</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A running; full speed; a rapid course.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When a horse is running in his full <qex>career</qex>.</q> <rj><qau>Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>General course of action or conduct in life, or in a particular part or calling in life, or in some special undertaking; usually applied to course or conduct which is of a public character; <as>as, Washington's <ex>career</ex> as a soldier</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An impartial view of his whole <qex>career</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Falconry)</fld> <def>The flight of a hawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Career</ent><br/
<hw>Ca*reer"</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Careered</conjf> 3; <pos>p. pr. & vb. n.</pos> <conjf>Careering</conjf>]</vmorph> <def>To move or run rapidly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Careering</qex> gayly over the curling waves.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>careerism</ent><br/
<hw>careerism</hw> <pos>n.</pos> <def>The practice of advancing one's career at the expense of one's personal integrity.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>careerist</ent><br/
<hw>careerist</hw> <pos>n.</pos> <def>A professional who follows a career.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carefree</ent><br/
<hw>care"free`</hw> <pr>(k<acir/r"fr<emac/`)</pr>, <pos>adj.</pos> <sn>1.</sn> <def>Free of trouble and worry and care; <as>as, the <ex>carefree</ex> joys of childhood; <ex>carefree</ex> millionaires, untroubled financially</as>.</def><br/
<syn><b>Syn. --</b> happy, lighthearted, unworried.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>nonchalant in a cheerful manner.</def><br/
<syn><b>Syn. --</b> breezy, airy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>cheerfully irresponsible; <as>as, <ex>carefree</ex> with his money</as>.  Opposite of <ant>careful</ant>.</def><br/
<syn><b>Syn. --</b> devil-may-care, happy-go-lucky, harum-scarum, slaphappy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>same as <er>unconcerned</er>.   Opposite of <ant>concerned</ant>.</def><br/
<syn><b>Syn. --</b> blithe.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Careful</ent><br/
<hw>Care"ful</hw> <pr>(k<acir/r"f<usdot/l)</pr>, <pos>a.</pos> <ety>[AS. <ets>cearful</ets>.]</ety> <sn>1.</sn> <def>Full of care; anxious; solicitous.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Be <qex>careful</qex> [Rev. Ver. <ldquo/anxious<rdquo/] for nothing.</q> <rj><qau>Phil. iv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>careful</qex> plowman doubting stands.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Filling with care or solicitude; exposing to concern, anxiety, or trouble; painful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>careful</qex> cold beginneth for to creep.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By Him that raised me to this <qex>careful</qex> height.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Taking care; giving good heed; watchful; cautious; provident; not indifferent, heedless, or reckless; -- often followed by <xex>of</xex>, <xex>for</xex>, or the infinitive; <as>as, <ex>careful</ex> of money; <ex>careful</ex> to do right</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast been careful for us with all this care.</q> <rj><qau>2. Kings iv, 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What could a <qex>careful</qex> father more have done?</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Anxious; solicitous; provident; thoughtful; cautious; circumspect; heedful; watchful; vigilant.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carefully</ent><br/
<hw>Care"ful*ly</hw>, <pos>adv.</pos> <def>In a careful manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carefulness</ent><br/
<hw>Care"ful*ness</hw>, <pos>n.</pos> <def>Quality or state of being careful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>care-laden</ent><br/
<hw>care-laden</hw> <pos>adj.</pos> <def>burdened by cares.</def><br/
<syn><b>Syn. --</b> heavy-laden.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Careless</ent><br/
<hw>Care"less</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>cearle<aacute/s</ets>.]</ety> <sn>1.</sn> <def>Free from care or anxiety. hence, cheerful; light-hearted.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sleep she as sound as <qex>careless</qex> infancy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having no care; not taking ordinary or proper care; negligent; unconcerned; heedless; inattentive; unmindful; regardless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My brother was too <qex>careless</qex> of his charge.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He grew <qex>careless</qex> of himself.</q> <rj><qau>Steele.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Without thought or purpose; without due care; without attention to rule or system; unstudied; inconsiderate; spontaneous; rash; <as>as, a <ex>careless</ex> throw; a <ex>careless</ex> expression</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He framed the <qex>careless</qex> rhyme.</q> <rj><qau>Beattie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not receiving care; uncared for.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Their many wounds and <qex>careless</qex> harms.</q> <rj><qau>Spenser.</qau></rj></p>

<p><syn><b>Syn.</b> -- Negligent; heedless; thoughtless; unthinking; inattentive; incautious; remiss; supine; forgetful; regardless; inconsiderate; listless.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carelessly</ent><br/
<hw>Care"less*ly</hw>, <pos>adv.</pos> <def>In a careless manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carelessness</ent><br/
<hw>Care"less*ness</hw>, <pos>n.</pos> <def>The quality or state of being careless; heedlessness; negligence; inattention.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carene</ent><br/
<hw>Ca*rene"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>carena</ets>, corrupted fr. <ets>quarentena</ets>. See <er>Quarantine</er>.]</ety> <fld>(Ecol.)</fld> <def>A fast of forty days on bread and water.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caress</ent><br/
<hw>Ca*ress"</hw> <pr>(k<adot/*r<ecr/s")</pr>, <pos>n.</pos> <ety>[F. <ets>caresse</ets>, It. <ets>carezza</ets>, LL. <ets>caritia</ets> dearness, fr. L. <ets>carus</ets> dear. See <er>Charity</er>.]</ety> <def>An act of endearment; any act or expression of affection; an embracing, or touching, with tenderness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wooed her with his soft <qex>caresses</qex>.</q> <rj><qau>Langfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He exerted himself to win by indulgence and <qex>caresses</qex> the hearts of all who were under his command.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caress</ent><br/
<hw>Ca*ress"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caressed</conjf> <pr>(k<adot/*r<ecr/st")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caressing</conjf>.]</vmorph> <ety>[F. <ets>caresser</ets>, fr. It. <ets>carezzare</ets>, fr. <ets>carezza</ets> caress. See <er>Caress</er>., <pos>n.</pos>]</ety> <def>To treat with tokens of fondness, affection, or kindness; to touch or speak to in a loving or endearing manner; to fondle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lady <qex>caresses</qex> the rough bloodhound.</q> <rj><qau>Sir W. Scott.</qau></rj></p>

<p><syn><b>Syn.</b> -- To fondle; embrace; pet; coddle; court; flatter.</syn> <usage> -- <er>Caress</er>, <er>Fondle</er>. <ldquo/We <xex>caress</xex> by words or actions; we <xex>fondle</xex> by actions only.<rdquo/</usage>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>caressive</ent><br/
<ent>caressing</ent><br/
<mhw><hw>caressing</hw> <hw>caressive</hw></mhw> <pos>adj.</pos>  <def>showing love.</def> <illu><ex>caressing</ex> words</illu><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caressing</ent><br/
<hw>caressing</hw> <pos>n.</pos> <sn>1.</sn> <def>the act of embracing; holding in one's arms .</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>foreplay without contact with the genital organs.</def><br/
<syn><b>Syn. --</b> caressing, cuddling, fondling, hugging, kissing, necking, petting, smooching, snuggling.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caressingly</ent><br/
<hw>Ca*ress"ing*ly</hw>, <pos>adv.</pos> <def>In caressing manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caret</ent><br/
<hw>Ca"ret</hw> <pr>(k<amac/"r<ecr/t <it>or</it> k<acr/r"<ecr/t)</pr>, <pos>n.</pos> <ety>[L. <ets>caret</ets> there is wanting, fr. <ets>carere</ets> to want.]</ety> <def>A mark [^] used by writers and proof readers to indicate that something is interlined above, or inserted in the margin, which belongs in the place marked by the caret.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caret</ent><br/
||<hw>Ca`ret"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a species of tortoise.]</ety> <fld>(Zool.)</fld> <def>The hawkbill turtle. See <er>Hawkbill</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caretta</ent><br/
<hw>Caretta</hw> <pos>n.</pos> <def>a genus consisting of the loggerhead turtles.</def><br/
<syn><b>Syn. --</b> genus <gen>Caretta</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Care-tuned</ent><br/
<hw>Care"-tuned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Weary; mournful.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Careworn</ent><br/
<hw>Care"worn`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Worn or burdened with care; <as>as, <ex>careworn</ex> look or face</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carex</ent><br/
||<hw>Ca"rex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., sedge.]</ety> <fld>(Bot.)</fld> <def>A numerous and widely distributed genus of perennial herbaceous plants of the order <ord>Cypreace<ae/</ord>; the sedges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carf</ent><br/
<hw>Carf</hw> <pr>(k<aum/rf)</pr>, <def><pos>pret.</pos> of <er>Carve</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>carful</ent><br/
<hw>car"ful</hw> <pos>n.</pos> <def>the quantity that a car will hold.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cargason</ent><br/
<hw>Car"ga*son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cargaison</ets>, Sp. <ets>cargazon</ets>, LL. <ets>cargare</ets> to load. See <er>rgo</er>.]</ety> <def>A cargo.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cargo</ent><br/
<hw>Car"go</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cargoes</plw> <pr>(#)</pr>.</plu> <ety>[Sp. <ets>cargo</ets>, <ets>carga</ets>, burden, load, from <ets>cargar</ets> to load, from <ets>cargar</ets> to load, charge, See <er>Charge</er>.]</ety> <def>The lading or freight of a ship or other vessel; the goods, merchandise, or whatever is conveyed in a vessel or boat; load; freight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cargoes</qex> of food or clothing.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>cargo</xex>, in law, is usually applied to goods only, and not to live animals or persons.</note>  <rj><au>Burill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cargoose</ent><br/
<hw>Car"goose`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. fr. Gael. & Ir. <ets>cir</ets>, <ets>cior</ets> (pronounced kir, kior), crest, comb + E. <ets>goose</ets>. Cf. <er>Crebe</er>.]</ety> <fld>(Zool.)</fld> <def>A species of grebe (<spn>Podiceps crisratus</spn>); the crested grebe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cariama</ent><br/
||<hw><cced/a`ri*a"ma</hw> <pr>(s<aum/`r<esl/*<adot/"m<adot/)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>A large, long-legged cranelike South American wading bird (<spn>Cariama cristata</spn>, formerly <spn>Dicholophus cristatus</spn>) which preys upon snakes, etc.; it is also called the <altname>seriema</altname>, but that latter name is also applied to another South American bird.  See <er>Seriema</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cariama</ent><br/
<hw>Cariama</hw> <pos>prop. n.</pos> <def>The type genus of the Cariamidae, consisting only of the <spn>Cariama cristata</spn>, also called the <altname>seriema</altname>.</def><br/
<syn><b>Syn. --</b> genus <gen>Cariama</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cariamidae</ent><br/
<hw>Cariamidae</hw> <pos>n.</pos> <def>a natural family of cranelike South American wading birds.</def><br/
<syn><b>Syn. --</b> family <fam>Cariamidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carib</ent><br/
<hw>Car"ib</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Caries</plw>.</plu> <ety>[See <er>Cannibal</er>.]</ety> <fld>(Ethol.)</fld> <def>A native of the Caribbee islands or the coasts of the Caribbean sea; esp., one of a tribe of Indians inhabiting a region of South America, north of the Amazon, and formerly most of the West India islands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caribbee</ent><br/
<ent>Caribbean</ent><br/
<mhw>{ <hw>Car`ib*be"an</hw> <pr>(?)</pr>, <hw>Car`ib*bee</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to the Caribs, to their islands (the eastern and southern West Indies), or to the sea (called the Caribbean sea) lying between those islands and Central America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caribbee</ent><br/
<hw>Car"ib*bee</hw>, <pos>n.</pos> <def>A Carib.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caribe</ent><br/
||<hw>Ca*ri"be</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. a cannibal.]</ety> <fld>(Zool)</fld>. <def>A south American fresh water fish of the genus <gen>Serrasalmo</gen> of many species, remarkable for its voracity. When numerous they attack man or beast, often with fatal results.</def></p>

<p><-- Illustration of a Caribe (<spn>Serrasalmo scapularis</spn>) --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caribou</ent><br/
<hw>Car"i*bou</hw> <pr>(k<acr/r"<icr/*b<oomac/)</pr>, <pos>n.</pos> <ety>[Canadian French.]</ety> <fld>(Zool.)</fld> <def>The American reindeer, especially the common or woodland species (<spn>Rangifer Caribou</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Barren Ground caribou</b></col>. <cd>See under <er>Barren</er>.</cd> -- <col><b>Woodland caribou</b></col>, <cd>the common reindeer (<spn>Rangifer Caribou</spn>) of the northern forests of America.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carica</ent><br/
<hw>Carica</hw> <pos>n.</pos> <def>the type genus of the <fam>Caricaceae</fam>; consisting of tropical American trees; the papayas.</def><br/
<syn><b>Syn. --</b> genus <gen>Carica</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caricaceae</ent><br/
<hw>Caricaceae</hw> <pos>n.</pos> <def>a natural family of trees native to tropical America and Africa with milky juice and large palmately lobed leaves.</def><br/
<syn><b>Syn. --</b> family <fam>Caricaceae</fam>, papaya family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caricature</ent><br/
<hw>Car"i*ca*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>caricatura</ets>, fr. <ets>caricare</ets> to charge, overload, exaggerate. See <er>Charge</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>An exaggeration, or distortion by exaggeration, of parts or characteristics, as in a picture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A picture or other figure or description in which the peculiarities of a person or thing are so exaggerated as to appear ridiculous; a burlesque; a parody.</def> <altsp>[Formerly written <asp>caricatura</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The truest likeness of the prince of French literature will be the one that has most of the look of a <qex>caricature</qex>.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A grotesque <qex>caricature</qex> of virtue.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caricature</ent><br/
<hw>Car"i*ca*ture</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caricatured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caricaturing</conjf>.]</vmorph> <def>To make or draw a caricature of; to represent with ridiculous exaggeration; to burlesque.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He could draw an ill face, or <qex>caricature</qex> a good one, with a masterly hand.</q> <rj><qau>Lord Lyttelton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caricaturist</ent><br/
<hw>Car"i*ca*tu`rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who caricatures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caricous</ent><br/
<hw>Car"i*cous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carica</ets> a kind of dry fig.]</ety> <def>Of the shape of a fig; <as>as, a <ex>caricous</ex> tumor</as>.</def>  <rj><au>Graig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caries</ent><br/
||<hw>Ca"ri*es</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., decay.]</ety> <fld>(Med.)</fld> <def>Ulceration of bone; a process in which bone disintegrates and is carried away piecemeal, as distinguished from <xex>necrosis</xex>, in which it dies in masses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carillon</ent><br/
<hw>Car"il*lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carillon</ets> a chime of bells, originally consisting of four bells, as if fr.. (assumed) L. <ets>quadrilio</ets>, fr. <ets>quatuer</ets> four.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Mus.)</fld> <def>A chime of bells diatonically tuned, played by clockwork or by finger keys.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tune adapted to be played by musical bells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carina</ent><br/
||<hw>Ca*ri"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., keel.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A keel</def>. <sd>(a)</sd> <def>That part of a papilionaceous flower, consisting of two petals, commonly united, which incloses the organs of fructification</def>. <sd>(b)</sd> <def>A longitudinal ridge or projection like the keel of a boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The keel of the breastbone of birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carinaria</ent><br/
<hw>Car`i*na"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>carina</ets> keel.]</ety> <fld>(Zool.)</fld> <def>A genus of oceanic heteropod Mollusca, having a thin, glassy, bonnet-shaped shell, which covers only the nucleus and gills.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carinatae</ent><br/
||<hw>Car`i*na"t<ae/</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., Fem. pl. fr. L. <er>carinatus</er>. See <er>Carinate</er>.]</ety> <def>A grand division of birds, including all existing flying birds; -- So called from the <xex>carina</xex> or keel on the breastbone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carinated</ent><br/
<ent>Carinate</ent><br/
<mhw>{ <hw>Car"i*nate</hw> <pr>(?)</pr>, <hw>Car"i*na`ted</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[L. <ets>carinatus</ets>, fr. <ets>carina</ets> keel.]</ety> <def>Shaped like the keel or prow of a ship; having a carina or keel; <as>as, a <ex>carinate</ex> calyx or leaf; a <ex>carinate</ex> sternum (of a bird)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cariole</ent><br/
<hw>Car"i*ole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carriole</ets>, dim. fr. L. <ets>carrus</ets>. See <er>Car</er>, and <er>Carryall</er>.]</ety> <sd>(a)</sd> <def>A small, light, open one-horse carriage</def>. <sd>(b)</sd> <def>A covered cart</def>. <sd>(c)</sd> <def>A kind of calash. See <er>Carryall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cariopsis</ent><br/
<hw>Car`i*op"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caryopsis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cariosity</ent><br/
<hw>Ca`ri*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Caries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carious</ent><br/
<hw>Ca"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cariosus</ets>, fr. <ets>caries</ets> dacay.]</ety> <def>Affected with caries; decaying; <as>as, a <ex>carious</ex> tooth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>carissa</ent><br/
<hw>carissa</hw> <pos>n.</pos> <def>a shrub of the genus <gen>Carissa</gen> having fragrant white flowers and plumlike red to purple-black fruits.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carjack</ent><br/
<hw>carjack</hw> <pos>v. t.</pos> <ety>[<ets>car</ets> + hi<ets>jack</ets>.]</ety> <def>to take (a car) by an act of <er>carjacking</er>; <as>as, the three teens <ex>carjacked</ex> two automobiles in one night</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>carjacking</ent><br/
<hw>car"jack*ing</hw> <pos>n.</pos> <ety>[<ets>car</ets> + hi<ets>jacking</ets>.]</ety> <def>the forcible taking of a car while the driver is in it; the robber may force the driver out, force the driver to drive while under threat of harm, or be forced to relinquish the controls while also being forced to remain in the car.  In the latter two cases, the act also constitutes a kidnapping.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cark</ent><br/
<hw>Cark</hw> <pr>(k<aum/rk)</pr>, <pos>n.</pos> <ety>[OE. <ets>cark</ets>, fr. a dialectic form of F. <ets>charge</ets>; cf. W. <ets>carc</ets> anxiety, care, Arm <ets>karg</ets> charge, burden. See <er>Charge</er>, and cf. <er>Cargo</er>.]</ety> <def>A noxious or corroding care; solicitude; worry.</def> <mark>[Archaic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His heavy head, devoid of careful <qex>cark</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fling <qex>cark</qex> and care aside.</q> <rj><qau>Motherwell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Freedom from the cares of money and the <qex>cark</qex> of fashion.</q> <rj><qau>R. D. Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 219 --></p>

<p><ent>Cark</ent><br/
<hw>Cark</hw> <pr>(k<aum/rk)</pr>, <pos>v. i.</pos> <def>To be careful, anxious, solicitous, or troubled in mind; to worry or grieve.</def> <mark>[R.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cark</ent><br/
<hw>Cark</hw>, <pos>v. t.</pos> <def>To vex; to worry; to make by anxious care or worry.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor can a man, independently . . . of God's blessing, care and <qex>cark</qex> himself one penny richer.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carkanet</ent><br/
<hw>Car"ka*net</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A carcanet.</def>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carking</ent><br/
<hw>Cark"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Distressing; worrying; perplexing; corroding; <as>as, <ex>carking</ex> cares</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carl</ent><br/
<hw>Carl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel, <ets>karl</ets> a male, a man; akin to AS. <ets>ceorl</ets>, OHG. charal, G. <ets>kerl</ets> fellow. See <er>Churl</er>.]</ety> <altsp>[Written also <asp>carle</asp>.]</altsp> <sn>1.</sn> <def>A rude, rustic man; a churl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The miller was a stout <qex>carl</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Large stalks of hemp which bear the seed; -- called also <altname>carl hemp</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>A kind of food. See citation, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Caring or <qex>carl</qex> are gray steeped in water and fried the next day in butter or fat.  They are eaten on the second Sunday before Easter, formerly called <qex>Carl</qex> Sunday.</q> <rj><qau>Robinson's Whitby Glossary (1875).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carlin</ent><br/
<hw>Car"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim., fr. <ets>carl</ets> male.]</ety> <def>An old woman.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroline</ent><br/
<ent>Carline</ent><br/
<mhw>{ <hw>Car"line</hw> <pr>(?)</pr>, <hw>Car"o*line</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[F. <ets>carin</ets>; cf. It. <ets>carlino</ets>; -- so called from <ets>Carlo</ets> (Charles) VI. of Naples.]</ety> <def>A silver coin once current in some parts of Italy, worth about seven cents.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carling</ent><br/
<ent>Carline</ent><br/
<mhw>{ <hw>Car"line</hw> <pr>(?)</pr>, <hw>Car"ling</hw> <pr>(?)</pr> }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>carlingur</ets>, Sp. Pg., & It. <ets>carlinga</ets>.]</ety> <fld>(Naut.)</fld> <def>A short timber running lengthwise of a ship, from one transverse desk beam to another; also, one of the cross timbers that strengthen a hath; -- usually in <pluf>pl.</pluf></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carline thistle</ent><br/
<hw>Car"line this`tle</hw> <pr>(?)</pr>. <ety>[F. <ets>carline</ets>, It., Sp., & Pg., <ets>carlina</ets>. Said to be so called from the Emperor Charlemagne, whose army is reputed to have used it as a remedy for pestilence.]</ety> <fld>(Bot.)</fld> <def>A prickly plant of the genus <gen>Carlina</gen> (<spn>Carlina vulgaris</spn>), found in Europe and Asia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carlings</ent><br/
<hw>Car"lings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Same as <er>Carl</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carling Sunday</b></col>, <cd>a Sunday in Lent when carls are eaten. In some parts of England, Passion Sunday. See <er>Carl</er>, 4.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carlist</ent><br/
<hw>Car"list</hw> <pr>(k<aum/r"l<icr/st)</pr>, <pos>n.</pos> <def>A partisan of <etsep>Charles X.</etsep> of France, or of Don <etsep>Carlos</etsep> of Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carlock</ent><br/
<hw>Car"lock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carlock</ets>, fr. Russ. <ets>Karl<uacute/k'</ets>.]</ety> <def>A sort of Russian isinglass, made from the air bladder of the sturgeon, and used in clarifying wine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carlot</ent><br/
<hw>Car"lot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Carl</er>.]</ety> <def>A churl; a boor; a peasant or countryman.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carlovingian</ent><br/
<hw>Car`lo*vin"gi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>Carlovingen</ets>.]</ety> <def>Pertaining to, founded by, of descended from, Charlemagne; <as>as, the <ex>Carlovingian</ex> race of kings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carmagnole</ent><br/
||<hw>Car`ma`gnole"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A popular or Red Rebublican song and dance, of the time of the first French Revolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They danced and yelled the <qex>carmagnole</qex>.</q> <rj><qau>Compton Reade.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bombastic report from the French armies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carman</ent><br/
<hw>Car"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Carmen</er> <pr>(<?/)</pr></plu> <def>A man whose employment is to drive, or to convey goods in, a car or car.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carmelin</ent><br/
<ent>Carmelite</ent><br/
<mhw>{ <hw>Car"mel*ite</hw> <pr>(?)</pr>, <hw>Car"mel*in</hw>  }</mhw> <pos>a.</pos> <def>Of or pertaining to the order of Carmelites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carmelite</ent><br/
<hw>Car"mel*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Eccl. Hist.)</fld> <def>A friar of a mendicant order (the Order of Our Lady of Mount Carmel) established on Mount Carmel, in Syria, in the twelfth century; a White Friar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A nun of the Order of Our lady of Mount Carmel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Car mile</ent><br/
<hw>Car mile</hw>. <fld>(Railroads)</fld> <def>A mile traveled by a single car, taken as a unit of computation, as in computing the average travel of each car of a system during a given period.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Car mileage</ent><br/
<hw>Car mileage</hw>. <fld>(Railroads)</fld> <sd>(a)</sd> <def>Car miles collectively.</def> <sd>(b)</sd> <def>The amount paid by one road for the use of cars of another road.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carminated</ent><br/
<hw>Car"mi*na`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, relating to, or mixed with, carmine; <as>as, <ex>carminated</ex> lake</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carminative</ent><br/
<hw>Car*min"ative</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>carminativus</ets> (1622), fr. <ets>carminare</ets> to card, hence to cleanse, fr. <ets>carmen</ets> a card for freeing wool or flax from the coarser parts, and from extraneous matter: cf. F. <ets>carminatif</ets>.]</ety> <def>Expelling wind from the body; warming; antispasmodic.</def> <ldquo/<xex>Carminative</xex> hot seeds.<rdquo/  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carminative</ent><br/
<hw>Car*min"a*tive</hw>, <pos>n.</pos> <def>A substance, esp. an aromatic, which tends to expel wind from the alimentary canal, or to relieve colic, griping, or flatulence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carmine</ent><br/
<hw>Car"mine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carmin</ets> (cf. Sp. <ets>carmin</ets>, It. <ets>carminio</ets>), contr. from LL. <ets>carmesinus</ets> purple color. See <er>Crimson</er>.]</ety> <sn>1.</sn> <def>A rich red or crimson color with a shade of purple.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A beautiful pigment, or a lake, of this color, prepared from cochineal, and used in miniature painting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>The essential coloring principle of cochineal, extracted as a purple-red amorphous mass. It is a glucoside and possesses acid properties; -- hence called also <altname>carminic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carmine red</b></col> <fld>(Chem.)</fld>, <cd>a coloring matter obtained from carmine as a purple-red substance, and probably allied to the phthale<ium/ns.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carminic</ent><br/
<hw>Car*min"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to, or derived from, carmine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carminic acid</b></col>. <cd>Same as <er>Carmine</er>, 3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carmot</ent><br/
<hw>Car"mot</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Alchemy)</fld> <def>The matter of which the philosopher's stone was believed to be composed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnage</ent><br/
<hw>Car"nage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carnage</ets>, LL. <ets>carnaticum</ets> tribute of animals, flesh of animals, fr. L. <ets>caro</ets>, <ets>carnis</ets>, flesh. See <er>Carnal</er>.]</ety> <sn>1.</sn> <def>Flesh of slain animals or men.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A miltitude of dogs came to feast on the <qex>carnage</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Great destruction of life, as in battle; bloodshed; slaughter; massacre; murder; havoc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The more fearful <qex>carnage</qex> of the Bloody Circuit.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnal</ent><br/
<hw>Car"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carnalis</ets>, fr. <ets>caro</ets>, <ets>carnis</ets>, flesh; akin to Gr. <?/, Skr. <ets>kravya</ets>; cf. F. <ets>charnel</ets>, Of. also <ets>carnel</ets>. Cf. <er>Charnel</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the body or its appetites; animal; fleshly; sensual; given to sensual indulgence; lustful; human or worldly as opposed to <contr>spiritual</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For ye are yet <qex>carnal</qex>.</q> <rj><qau>1 Cor. iii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Not sunk in <qex>carnal</qex> pleasure.</q> <rj><qau>Milton</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Carnal</qex> desires after miracles.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Flesh-devouring; cruel; ravenous; bloody.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>carnal</qex> cur<br/
Preys on the issue of his mother's body.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carnal knowledge</b></col>, <cd>sexual intercourse; -- used especially of an unlawful act on the part of the man.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnalism</ent><br/
<hw>Car"nal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being carnal; carnality; sensualism.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnalist</ent><br/
<hw>Car"nal*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sensualist.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnality</ent><br/
<hw>Car*nal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>carnalitas</ets>.]</ety> <def>The state of being carnal; fleshly lust, or the indulgence of lust; grossness of mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Because of the <qex>carnality</qex> of their hearts.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnalize</ent><br/
<hw>Car"nal*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carnalized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carnalizing</conjf>.]</vmorph> <def>To make carnal; to debase to carnality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sensual and <qex>carnalized</qex> spirit.</q> <rj><qau>John Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnallite</ent><br/
<hw>Car"nal*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>carnallit</ets>, fr. Von <etsep>Carnall</etsep>, a Prussian.]</ety> <fld>(Min.)</fld> <def>A hydrous chloride of potassium and magnesium, sometimes found associated with deposits of rock salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnally</ent><br/
<hw>Car"nal*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>According to the flesh, to the world, or to human nature; in a manner to gratify animal appetites and lusts; sensually.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For to be <qex>carnally</qex> minded is death; but to be spiritually minded is life and peace.</q> <rj><qau>Rom. viii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnal-minded</ent><br/
<hw>Car"nal-mind`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Worldly-minded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnal-mindedness</ent><br/
<hw>Car"nal-mind"ed*ness</hw>, <pos>n.</pos> <def>Grossness of mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnary</ent><br/
<hw>Car"na*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>carnarium</ets>, fr. <ets>caro</ets>, <ets>carnis</ets>, flesh.]</ety> <def>A vault or crypt in connection with a church, used as a repository for human bones disintered from their original burial places; a charnel house.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnassial</ent><br/
<hw>Car*nas"si*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>carnassier</ets> carnivorous, and L. <ets>caro</ets>, <ets>carnis</ets>, flesh.]</ety> <fld>(Anat.)</fld> <def>Adapted to eating flesh.</def> -- <def2><pos>n.</pos> <def>A carnassial tooth; especially, the last premolar in many carnivores.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnate</ent><br/
<hw>Car"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carnatus</ets> fleshy.]</ety> <def>Invested with, or embodied in, flesh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnation</ent><br/
<hw>Car*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carnation</ets> the flesh tints in a painting, It <ets>carnagione</ets>, fr. L. <ets>carnatio</ets> fleshiness, fr. <ets>caro</ets>, <ets>carnis</ets>, flesh. See <er>Carnal</er>.]</ety> <sn>1.</sn> <def>The natural color of flesh; rosy pink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her complexion of the delicate <qex>carnation</qex>.</q> <rj><qau>Ld. Lytton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Paint.)</fld> <def>Those parts of a picture in which the human body or any part of it is represented in full color; the flesh tints.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The flesh tints in painting are termed <qex>carnations</qex>.</q> <rj><qau>Fairholt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A species of <gen>Dianthus</gen> (<spn>Dianthus Caryophyllus</spn>) or pink, having very beautiful flowers of various colors, esp. white and usually a rich, spicy scent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnationed</ent><br/
<hw>Car*na"tioned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a flesh color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnauba</ent><br/
||<hw>Car*nau"ba</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The Brazilian wax palm. See <er>Wax palm</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnegiea</ent><br/
<hw>Carnegiea</hw> <pos>n.</pos> <def>a genus of plants consisting of one species, the <er>saguaro</er>.</def><br/
<syn><b>Syn. --</b> genus <gen>Carnegiea</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carnelian</ent><br/
<hw>Car*nel"ian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>carnelian</ets>; influenced by L. <ets>carneus</ets> fleshy, of flesh, because of its flesh red color. See <er>Cornellan</er>.]</ety> <fld>(Min.)</fld> <def>A variety of chalcedony, of a clear, deep red, flesh red, or reddish white color. It is moderately hard, capable of a good polish, and often used for seals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carneous</ent><br/
<hw>Car"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carneus</ets>, from <ets>caro</ets>, <ets>carnis</ets>, flesh.]</ety> <def>Consisting of, or like, flesh; carnous; fleshy.</def> <ldquo/<xex>Carneous</xex> fibers.<rdquo/  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carney</ent><br/
<hw>Car"ney</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>carneus</ets> flesh.]</ety> <fld>(Far.)</fld> <def>A disease of horses, in which the mouth is so furred that the afflicted animal can not eat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnic</ent><br/
<hw>Car"nic</hw> <pr>(k<aum/r"n<icr/k)</pr>, <pos>a.</pos> <ety>[L. <ets>caro</ets>, <ets>carnis</ets>, flesh.]</ety> <def>Of or pertaining to flesh;</def> <specif>specif.</specif> <fld>(Physiol. Chem.)</fld>, <def>pertaining to or designating a hygroscopic monobasic acid, <chform>C10H15O5N3</chform>, obtained as a cleavage product from an acid of muscle tissue.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carnifex</ent><br/
||<hw>Car"ni*fex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>caro</ets>, <ets>carnis</ets>, flesh + <ets>facere</ets> to make.]</ety> <fld>(Antiq.)</fld> <def>The public executioner at Rome, who executed persons of the lowest rank; hence, an executioner or hangman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnification</ent><br/
<hw>Car`ni*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>carnification</ets>.]</ety> <def>The act or process of turning to flesh, or to a substance resembling flesh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnify</ent><br/
<hw>Car"ni*fy</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[LL. <ets>carnificare</ets>, fr. L. <ets>caro</ets>, <ets>carnis</ets>, flesh + <ets>facere</ets> to make: cf. F. <ets>carnifier</ets>.]</ety> <def>To form flesh; to become like flesh.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnin</ent><br/
<hw>Car"nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caro</ets>, <ets>canis</ets> , flesh.]</ety> <fld>(Chem.)</fld> <def>A white crystalline nitrogenous substance, found in extract of meat, and related to xanthin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnival</ent><br/
<hw>Car"ni*val</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>carnevale</ets>, prob. for older <ets>carnelevale</ets>, prop., the putting away of meat; fr. L. <ets>caro</ets>, <ets>carnis</ets>, flesh + <ets>levare</ets> to take away, lift up, fr. <ets>levis</ets> light.]</ety> <sn>1.</sn> <def>A festival celebrated with merriment and revelry in Roman Gatholic countries during the week before Lent, esp. at Rome and Naples, during a few days (three to ten) before Lent, ending with Shrove Tuesday.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>carnival</qex> at Venice is everywhere talked of.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any merrymaking, feasting, or masquerading, especially when overstepping the bounds of decorum; a time of riotous excess.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He saw the lean dogs beneath the wall<br/
Hold o'er the dead their <qex>carnival</qex></q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnivora</ent><br/
||<hw>Car*niv"o*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., neut. pl. from L. <ets>carnivorus</ets>. See <er>Carnivorous</er>.]</ety> <fld>(Zool.)</fld> <def>An order of Mammallia including the lion, tiger, wolf bear, seal, etc. They are adapted by their structure to feed upon flesh, though some of them, as the bears, also eat vegetable food. The teeth are large and sharp, suitable for cutting flesh, and the jaws powerful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnivoracity</ent><br/
<hw>Car*niv`o*rac"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Greediness of appetite for flesh.</def> <mark>[Sportive.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnivore</ent><br/
<hw>Car`ni*vore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>carnivore</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the <ord>Carnivora</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnivorous</ent><br/
<hw>Car*niv"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carnivorus</ets>; <ets>caro</ets>, <ets>carnis</ets>, flesh + <ets>varare</ets> to devour.]</ety> <def>Eating or feeding on flesh. The term is applied: <sd>(a)</sd> to animals which naturally seek flesh for food, as the tiger, dog, etc.; <sd>(b)</sd> to plants which are supposed to absorb animal food; <sd>(c)</sd> to substances which destroy animal tissue, as caustics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnous</ent><br/
<ent>Carnose</ent><br/
<mhw>{ <hw>Car*nose</hw> <pr>(?)</pr>, <hw>Car"*nous</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>carnosus</ets>, fr. <ets>caro</ets>, <ets>carnis</ets>, flesh: cf. OF. <ets>carneux</ets>, F. <ets>charneux</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to flesh; fleshy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A distinct <qex>carnose</qex> muscle.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Of a fleshy consistence; -- applied to succulent leaves, stems, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnosity</ent><br/
<hw>Car*nos"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>carnosit<eacute/</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Med.)</fld> <def>A fleshy excrescence; esp. a small excrescence or fungous growth.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fleshy substance or quality; fleshy covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Consciences] overgrown with so hard a <qex>carnosity</qex>.</q> <rj><qau>Spelman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The olives, indeed be very small there, and bigger than capers; yet commended they are for their <qex>carnosity</qex>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carnot's cycle</ent><br/
<hw>Car`not's" cy"cle</hw> <pr>(k<aum/r`n<omac/z")</pr>. <ety>[After N. L. S. <etsep>Carnot</etsep>, French physicist.]</ety> <fld>(Thermodynamics)</fld> <def>An ideal heat-engine cycle in which the working fluid goes through the following four successive operations: (1) Isothermal expansion to a desired point; (2) adiabatic expansion to a desired point; (3) isothermal compression to such a point that (4) adiabatic compression brings it back to its initial state.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carob</ent><br/
<hw>Car"ob</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>caroube</ets> fruit of the carob tree, Sp. <ets>garrobo</ets>, <ets>al-garrobo</ets>, carob tree, fr. Ar. <ets>kharr<umac/b</ets>, Per. <ets>Kharn<umac/b</ets>. Cf. <er>Clgaroba</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An evergreen leguminous tree (<spn>Ceratania Siliqua</spn>) found in the countries bordering the Mediterranean; the St. John's bread; -- called also <altname>carob tree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the long, sweet, succulent, pods of the carob tree, which are used as food for animals and sometimes eaten by man; -- called also <altname>St. John's bread</altname>, <altname>carob bean</altname>, and <altname>algaroba bean</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroche</ent><br/
<hw>Ca*roche"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>carrache</ets>, F. <ets>carrose</ets> from It. <ets>carrocio</ets>, <ets>carrozza</ets>, fr. <ets>carro</ets>, L. <ets>carus</ets>. See <er>Car</er>.]</ety> <def>A kind of pleasure carriage; a coach.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To mount two-wheeled <qex>caroches</qex>.</q> <rj><qau>Butler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroched</ent><br/
<hw>Ca*roched"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Placed in a caroche.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Beggary rides <qex>caroched</qex>.</q> <rj><qau>Massenger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroigne</ent><br/
<hw>Car"oigne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Carrion</er>.]</ety> <def>Dead body; carrion.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carol</ent><br/
<hw>Car"ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>carole</ets> a kind of dance wherein many dance together, fr. <ets>caroler</ets> to dance; perh. from Celtic; cf. Armor. <ets>koroll</ets>, n., <ets>korolla</ets>, <ets>korolli</ets>, v., Ir. <ets>car</ets> music, turn, circular motion, also L. <ets>choraula</ets> a flute player, <ets>charus</ets> a dance, chorus, choir.]</ety> <sn>1.</sn> <def>A round dance.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A song of joy, exultation, or mirth; a lay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The costly feast, the <qex>carol</qex>, and the dance.</q> <rj><qau>Dryden</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was the <qex>carol</qex> of a bird.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A song of praise of devotion; <as>as, a Christmas or Easter <ex>carol</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heard a <qex>carol</qex>, mournful, holy.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the darkness sing your <qex>carol</qex> of high praise.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Joyful music, as of a song.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I heard the bells on Christmans Day<br/
Their old, familiar <qex>carol</qex> play.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carol</ent><br/
<hw>Car"ol</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caroled</conjf> <pr>(?)</pr>, or <conjf>Carolled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Caroling</conjf>, or <conjf>Carolling</conjf>.]</vmorph><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To praise or celebrate in song.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Shepherds at their festivals<br/
<qex>Carol</qex> her goodness.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sing, especially with joyful notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hovering swans . . . <qex>carol</qex> sounds harmonious.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carol</ent><br/
<hw>Car"ol</hw>, <pos>v. i.</pos> <def>To sing; esp. to sing joyfully; to warble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>carol</qex> of love's high praise.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The gray linnets <qex>carol</qex> from the hill.</q> <rj><qau>Beattie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrol</ent><br/
<ent>Carol</ent><br/
<mhw>{ <hw>Car"ol</hw>, <hw>Car"rol</hw>  }</mhw>, <pos>n.</pos> <ety>[OF. <ets>carole</ets> a sort of circular space, or carol.]</ety> <fld>(Arch.)</fld> <def>A small closet or inclosure built against a window on the inner side, to sit in for study. The word was used as late as the 16th century.  The term <er>carrel</er>, of the same has largely superseded its use.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>A bay window may thus be called a <qex>carol</qex>.</q> <rj><qau>Parker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carolin</ent><br/
<hw>Car"o*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Carolus</ets> Charles.]</ety> <def>A former gold coin of Germany worth nearly five dollars; also, a gold coin of Sweden worth nearly five dollars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carolina pink</ent><br/
<hw>Car`o*li"na pink`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <er>Pinkboot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroline</ent><br/
<hw>Car"o*line</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coin. See <er>Carline</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroling</ent><br/
<hw>Car"ol*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A song of joy or devotion; a singing, as of carols.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such heavenly notes and <qex>carolings</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carolinian</ent><br/
<hw>Car`o*lin"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A native or inhabitant of north or South Carolina.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carolitic</ent><br/
<hw>Car`o*lit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Arch.)</fld> <def>Adorned with sculptured leaves and branches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carolus</ent><br/
<hw>Car"o*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Caroluses</plw> <pr>(#)</pr>, L. <plw>Caroli</plw> <pr>(#)</pr>.</plu> <ety>[L., Charles.]</ety> <def>An English gold coin of the value of twenty or twenty-three shillings. It was first struck in the reign of Charles I.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Told down the crowns and <qex>Caroluses</qex>.</q> <rj><qau>Macawlay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carom</ent><br/
<hw>Car"om</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. corrupted fr. F. <ets>carumboler</ets> to carom, <ets>carambolage</ets> a carom, <ets>carambole</ets> the red ball in billiards.]</ety> <fld>(Billiards)</fld> <def>A shot in which the ball struck with the cue comes in contact with two or more balls on the table; a hitting of two or more balls with the player's ball. In England it is called <altname>cannon</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carom</ent><br/
<hw>Car"om</hw>, <pos>v. i.</pos> <fld>(Billiards)</fld> <def>To make a carom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caromel</ent><br/
<hw>Car"o*mel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caramel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caroteel</ent><br/
<hw>Car`o*teel"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Com.)</fld> <def>A tierce or cask for dried fruits, etc., usually about 700 lbs.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carotic</ent><br/
<hw>Ca*rot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ stupefying. See <er>Carotid</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to stupor; <as>as, a <ex>carotic</ex> state</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Carotid; <as>as, the <ex>carotic</ex> arteries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carotid</ent><br/
<hw>Ca*rot"id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, pl., from <?/ heavy sleep: cf. F. <ets>carotide</ets>. The early Greeks believed that these arteries in some way caused drowsiness.]</ety> <fld>(Anat.)</fld> <def>One of the two main arteries of the neck, by which blood is conveyed from the aorta to the head. <note>[See <xex>Illust.</xex> of <er>Aorta</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carotidal</ent><br/
<ent>Carotid</ent><br/
<mhw>{ <hw>Ca*rot"id</hw> <pr>(?)</pr>, <hw>Ca*rot"id*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to, or near, the carotids or one of them; <as>as, the <ex>carotid</ex> gland</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carotin</ent><br/
<hw>Ca*ro"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A red crystallizable tasteless substance, extracted from the carrot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carotte</ent><br/
||<hw>Ca`rotte"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prop., carrot.]</ety> <def>A cylindrical roll of tobacco; <as>as, a <ex>carotte</ex> of perique</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carousal</ent><br/
<hw>Ca*rous"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Carouse</er>, but also cf. F. <ets>carrousel</ets> tilt.]</ety> <def>A jovial feast or festival; a drunken revel; a carouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The swains were preparing for a <qex>carousal</qex>.</q> <rj><qau>Sterne.</qau></rj></p>

<p><syn><b>Syn.</b> -- Banquet; revel; orgie; carouse. See <er>Feast</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 220 --></p>

<p><ent>Carouse</ent><br/
<hw>Ca*rouse"</hw> <pr>(k<adot/*rouz")</pr>, <pos>n.</pos> <ety>[F. <ets>carrousse</ets>, earlier <ets>carous</ets>, fr. G. <ets>garaus</ets> finishing stroke, the entire emptying of the cup in drinking a health; <ets>gar</ets> entirely + <ets>aus</ets> out. See <er>Yare</er>, and <er>Out</er>.]</ety> <sn>1.</sn> <def>A large draught of liquor.</def> <mark>[Obs.]</mark> <ldquo/A full <xex>carouse</xex> of sack.<rdquo/   <au>Sir J. Davies.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Drink <qex>carouses</qex> to the next day's fate.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A drinking match; a carousal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The early feast and late <qex>carouse</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carouse</ent><br/
<hw>Ca*rouse"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caroused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carousing</conjf>.]</vmorph> <def>To drink deeply or freely in compliment; to take part in a carousal; to engage in drunken revels.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had been aboard, <qex>carousing</qex> to his mates.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carouse</ent><br/
<hw>Ca*rouse"</hw> <pos>v. t.</pos> <def>To drink up; to drain; to drink freely or jovially.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Guests <qex>carouse</qex> the sparkling tears of the rich grape.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Egypt's wanton queen,<br/
<qex>Carousing</qex> gems, herself dissolved in love.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>carousel</ent><br/
<hw>carousel</hw> <pr>(k<acr/r`<osl/*s<ecr/l")</pr>, <pos>n.</pos> <sn>1.</sn> <def>a continuous belt oriented horizontally or near-horizontally around a central structure, driven in a circular motion by some mechanism, which carries luggage delivered from the luggage hold of an airplane to air travelers; -- used at airports.</def><br/
<syn><b>Syn. --</b> luggage carousel.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>same as <er>merry-go-round</er>.</def><br/
<syn><b>Syn. --</b> merry-go-round, roundabout, whirligig.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carouser</ent><br/
<hw>Ca*rous"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who carouses; a reveler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carousing</ent><br/
<hw>Ca*rous"ing</hw>, <pos>a.</pos> <def>That carouses; relating to a carouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carousingly</ent><br/
<hw>Ca*rous"ing*ly</hw>, <pos>adv.</pos> <def>In the manner of a carouser.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carp</ent><br/
<hw>Carp</hw> <pr>(k<aum/rp)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carped</conjf> <pr>(k<aum/rpt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carping</conjf>.]</vmorph> <ety>[OE. <ets>carpen</ets> to say, speak; from Scand. (cf. Icel. <ets>karpa</ets> to boast), but influenced later by L. <ets>carpere</ets> to pluck, calumniate.]</ety> <sn>1.</sn> <def>To talk; to speak; to prattle.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To find fault; to cavil; to censure words or actions without reason or ill-naturedly; -- usually followed by <xex>at</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Carping</qex> and caviling at faults of manner.</q> <rj><qau>Blackw. Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And at my actions <qex>carp</qex> or catch.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carp</ent><br/
<hw>Carp</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To say; to tell.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To find fault with; to censure.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carp</ent><br/
<hw>Carp</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carp</plw>, formerly <plw>Carps</plw>.</plu> <ety>[Cf. Icel. <ets>karfi</ets>, Dan. <ets>karpe</ets>, Sw. <ets>karp</ets>, OHG. <ets>charpho</ets>, G. <ets>karpfen</ets>, F. <ets>carpe</ets>, LL. <ets>carpa</ets>.]</ety> <fld>(Zool.)</fld> <def>A fresh-water herbivorous fish (<spn>Cyprinus carpio</spn>.). Several other species of <gen>Cyprinus</gen>, <gen>Catla</gen>, and <gen>Carassius</gen> are called <xex>carp</xex>. See <er>Cruclan carp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>carp</xex> was originally from Asia, whence it was early introduced into Europe, where it is extensively reared in artificial ponds. Within a few years it has been introduced into America, and widely distributed by the government. Domestication has produced several varieties, as the <xex>leather carp</xex>, which is nearly or quite destitute of scales, and the <xex>mirror carp</xex>, which has only a few large scales. Intermediate varieties occur.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carp louse</b></col> <fld>(Zool.)</fld>, <cd>a small crustacean, of the genus <gen>Argulus</gen>, parasitic on carp and allied fishes. See <er>Branchiura</er>.</cd> -- <col><b>Carp mullet</b></col> <fld>(Zool.)</fld>, <cd>a fish (<spn>Moxostoma carpio</spn>) of the Ohio River and Great Lakes, allied to the suckers.</cd> -- <col><b>Carp sucker</b></col> <fld>(Zool.)</fld>, <cd>a name given to several species of fresh-water fishes of the genus Carpiodes in the United States; -- called also quillback.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpal</ent><br/
<hw>Car"pal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Carpus</er>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the carpus, or wrist.</def> -- <def2><pos>n.</pos> <def>One of the bones or cartilages of the carpus; a carpale.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carpal angle</b></col> <fld>(Zool.)</fld>, <cd>the angle at the last joint of the folded wing of a bird.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpale</ent><br/
||<hw>Car*pa"le</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carpalia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. E. <ets>carpus</ets>.]</ety> <fld>(Anat.)</fld> <def>One of the bones or cartilages of the carpus; esp. one of the series articulating with the metacarpals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpathian</ent><br/
<hw>Car*pa"thi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a range of mountains in Austro-Hungary, called the Carpathians, which partially inclose Hungary on the north, east, and south.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpellum</ent><br/
<ent>Carpel</ent><br/
<mhw>{ <hw>Car"pel</hw> <pr>(k<aum/r"p<ecr/l)</pr>, ||<hw>Car*pel"lum</hw> <pr>(-p<ecr/l"l<ucr/m)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>carpellum</ets>, fr. Gr. <grk>karpo`s</grk> fruit.]</ety> <fld>(Bot.)</fld> <def>A simple pistil or single-celled ovary or seed vessel, or one of the parts of a compound pistil, ovary, or seed vessel. See <xex>Illust</xex> of <er>Carpaphore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpellary</ent><br/
<hw>Car"pel*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Belonging to, forming, or containing carpels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>carpellate</ent><br/
<hw>carpellate</hw> <pos>adj.</pos> <fld>(Botany)</fld> <def>bearing or consisting of carpels.  Opposite of <ant>acarpelous</ant>.</def><br/
<syn><b>Syn. --</b> pistillate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carpenter</ent><br/
<hw>Car"pen*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>carpentier</ets>, F. <ets>charpentier</ets>, LL. <ets>carpentarius</ets>, fr. L. <ets>carpentum</ets> wagon, carriage.]</ety> <def>An artificer who works in timber; a framer and builder of houses, ships, etc.</def></p>

<p><syn><b>Syn.</b> -- <er>Carpenter</er>, <er>Joiner</er>.</syn> <usage> The <xex>carpenter</xex> frames and puts together roofs, partitions, floors, and other structural parts of a building. The <xex>joiner</xex> supplies stairs, doors shutters, mantelpieces, cupboards, and other parts necessary to finishing the building. In America the two trades are commonly united.</usage><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carpenter ant</b></col> <fld>(Zool.)</fld>, <cd>any species of ant which gnaws galleries in the wood of trees and constructs its nests therein. They usually select dead or somewhat decayed wood. The common large American species is <spn>Formica Pennsylvanica</spn>.</cd> -- <col><b>Carpenter bee</b></col> <fld>(Zool.)</fld>, <cd>a large hymenopterous insect of the genus <gen>Xylocopa</gen>; -- so called because it constructs its nest by gnawing long galleries in sound timber. The common American species is <spn>Xylocopa Virginica</spn>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>carpenteria</ent><br/
<hw>carpenteria</hw> <pos>n.</pos> <def>a California evergreen shrub (<spn>Carpenteria californica</spn>) having glossy opposite leaves and terminal clusters of a few fragrant white flowers.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carpentering</ent><br/
<hw>Car"pen*ter*ing</hw>, <pos>n.</pos> <def>The occupation or work of a carpenter; the act of working in timber; carpentry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpentry</ent><br/
<hw>Car"pen*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>charpenterie</ets>, OF. also <ets>carpenterie</ets>. See <er>Carpenter</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The art of cutting, framing, and joining timber, as in the construction of buildings.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assemblage of pieces of timber connected by being framed together, as the pieces of a roof, floor, etc.; work done by a carpenter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carper</ent><br/
<hw>Carp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who carps; a caviler.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpet</ent><br/
<hw>Car"pet</hw> <pr>(k<aum/r"p<ecr/t)</pr>, <pos>n.</pos> <ety>[OF. <ets>carpite</ets> rug, soft of cloth, F. <ets>carpette</ets> coarse packing cloth, rug (cf. It. <ets>carpita</ets> rug, blanket), LL. <ets>carpeta</ets>, <ets>carpita</ets>, woolly cloths, fr. L. <ets>carpere</ets> to pluck, to card (wool); cf. Gr. <grk>karpo`s</grk> fruit, E. <er>Harvest</er>.]</ety> <sn>1.</sn> <def>A heavy woven or felted fabric, usually of wool, but also of cotton, hemp, straw, etc.; esp. a floor covering made in breadths to be sewed together and nailed to the floor, as distinguished from a rug or mat; originally, also, a wrought cover for tables.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tables and beds covered with copes instead of <qex>carpets</qex> and coverlets.</q> <rj><qau>T. Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A smooth soft covering resembling or suggesting a carpet.</def> <ldquo/The grassy <xex>carpet</xex> of this plain.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Carpet beetle</b></col> <it>or</it> <col><b>Carpet bug</b></col></mcol> <fld>(Zool.)</fld>, <cd>a small beetle (<spn>Anthrenus scrophulari<ae/</spn>), which, in the larval state, does great damage to carpets and other woolen goods; -- also called <altname>buffalo bug</altname>.</cd> -- <col><b>Carpet knight</b></col>. <sd>(a)</sd> <cd>A knight who enjoys ease and security, or luxury, and has not known the hardships of the field; a hero of the drawing room; an effeminate person.</cd> <au>Shak.</au> <sd>(b)</sd> <cd>One made a knight, for some other than military distinction or service.</cd> -- <col><b>Carpet moth</b></col> <fld>(Zool.)</fld>, <cd>the larva of an insect which feeds on carpets and other woolen goods.  There are several kinds.  Some are the larv<ae/ of species of <gen>Tinea</gen> (as <spn>Tinea tapetzella</spn>); others of beetles, esp. <gen>Anthrenus</gen>.</cd> -- <col><b>Carpet snake</b></col> <fld>(Zool.)</fld>, <cd>an Australian snake. See <cref>Diamond snake</cref>, under <er>Diamond</er>.</cd> -- <col><b>Carpet sweeper</b></col>, <cd>an apparatus or device for sweeping carpets.</cd> -- <col><b>To be on the carpet</b></col>, <cd>to be under consideration; to be the subject of deliberation; to be in sight; -- an expression derived from the use of carpets as table cover.</cd> -- <col><b>Brussels carpet</b></col>. <cd>See under <er>Brussels</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpet</ent><br/
<hw>Car"pet</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carpeted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Carpeting</conjf>.]</vmorph> <def>To cover with, or as with, a carpet; to spread with carpets; to furnish with a carpet or carpets.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Carpeted</qex> temples in fashionable squares.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpetbag</ent><br/
<hw>Car"pet*bag`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A portable bag for travelers; -- so called because originally made of carpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpetbagger</ent><br/
<hw>Car"pet*bag`ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An adventurer; -- a term of contempt for a Northern man seeking private gain or political advancement in the southern part of the United States after the Civil War (1865)<-- used now for any politician moving to a new location to take advantage of more favorable chances for election-->.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>carpeted</ent><br/
<hw>carpeted</hw> <pos>adj.</pos> <def>covered with or as if with carpeting or with carpeting as specified; often used in combination; <as>as, the <ex>carpeted</ex> hallway; a flower-<ex>carpeted</ex> hillside</as>.  Opposite of <ant>uncarpeted</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carpeting</ent><br/
<hw>Car"pet*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of covering with carpets.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cloth or materials for carpets; carpets, in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The floor was covered with rich <qex>carpeting</qex>.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpetless</ent><br/
<hw>Car"pet*less</hw>, <pos>a.</pos> <def>Without a carpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpetmonger</ent><br/
<hw>Car"pet*mon`ger</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who deals in carpets; a buyer and seller of carpets.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One fond of pleasure; a gallant.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpetway</ent><br/
<hw>Car"pet*way`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Agric.)</fld> <def>A border of greensward left round the margin of a plowed field.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carphology</ent><br/
<hw>Car*phol"o*gy</hw> <pr>(k<aum/r*f<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ka`rfos</grk> any small dry body + <ets>-logy</ets>: cf. F. <ets>carphologie</ets>.]</ety> <fld>(Med.)</fld> <def>See <er>Floccillation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpinaceae</ent><br/
<hw>Carpinaceae</hw> <pos>n.</pos> <def>a natural family used in some classification systems to include the genera <gen>Carpinus</gen>; <gen>Ostryopsis</gen>; and <gen>Ostryopsis</gen>.</def><br/
<syn><b>Syn. --</b> subfamily <fam>Carpinaceae</fam>, family <fam>Carpinaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carping</ent><br/
<hw>Carp"ing</hw> <pr>(k<aum/rp"<icr/ng)</pr>, <pos>a.</pos> <def>Fault-finding; censorious caviling. See <er>Captious</er>.</def></p>

<p>-- <wordforms><wf>Carp"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpintero</ent><br/
||<hw>Car`pin*te"ro</hw> <pr>(k<aum/r`p<esl/n*t<asl/"r<osl/)</pr>, <pos>n.</pos> <ety>[Sp., a carpenter, a woodpecker.]</ety> <def>A california woodpecker (<spn>Melanerpes formicivorus)</spn>, noted for its habit of inserting acorns in holes which it drills in trees. The acorns become infested by insect larv<ae/, which, when grown, are extracted for food by the bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpodacus</ent><br/
<hw>Carpodacus</hw> <pos>n.</pos> <def>a genus of birds including the house finch (<spn>Carpodacus mexicanus</spn>) and purple finch (<spn>Carpodacus purpurea</spn>).</def><br/
<syn><b>Syn. --</b> genus <gen>Carpodacus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carpogenic</ent><br/
<hw>Car`po*gen"ic</hw> <pr>(k<aum/r`p<osl/*j<ecr/n"<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <ets>-gen</ets>.]</ety> <fld>(Bot.)</fld> <def>Productive of fruit, or causing fruit to be developed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpolite</ent><br/
<hw>Car"po*lite</hw> <pr>(k<aum/r"p<osl/*l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <ets>-lite</ets>, cf. F. <ets>carpolithe</ets>.]</ety> <def>A general term for a fossil fruit, nut, or seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpological</ent><br/
<hw>Car`po*log"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to carpology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpologist</ent><br/
<hw>Car*pol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who describes fruits; one versed in carpology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpology</ent><br/
<hw>Car*pol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <ets>-logy</ets>.]</ety> <def>That branch of botany which relates to the structure of seeds and fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpophagous</ent><br/
<hw>Car*poph"a*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <?/ to eat.]</ety> <def>Living on fruits; fruit-consuming.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpophore</ent><br/
<hw>Car"po*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <grk>fe`rein</grk> to bear.]</ety> <fld>(Bot.)</fld> <def>A slender prolongation of the receptacle as an axis between the carpels, as in Geranium and many umbelliferous plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpophyll</ent><br/
<hw>Car"po*phyll</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <?/ leaf.]</ety> <fld>(Bot.)</fld> <def>A leaf converted into a fruit or a constituent portion of a fruit; a carpel. <note>[See <xex>Illust.</xex> of <er>Gymnospermous</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpophyte</ent><br/
<hw>Car"po*phyte</hw> <pr>(k<aum/r"p<osl/*l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karpo`s</grk> fruit + <grk>fyto`n</grk> plant.]</ety> <fld>(Bot.)</fld> <def>A flowerless plant which forms a true fruit as the result of fertilization, as the red seaweeds, the <fam>Ascomycetes</fam>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The division of alg<ae/ and fungi into four classes called Carpophytes, Oophytes, Protophytes, and Zygophytes (or <class>Carpospore<ae/</class>, <class>Oospore<ae/</class>, <class>Protophyta</class>, and <class>Zygospore<ae/</class>) was proposed by Sachs about 1875.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carpospore</ent><br/
<hw>Car"po*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>karpo`s</grk> + <ets>-spore</ets>.]</ety> <fld>(Bot.)</fld> <def>A kind of spore formed in the conceptacles of red alg<ae/.</def> -- <wordforms><wf>Car`po*spor"ic</wf> <pr>(<?/)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>carposporous</ent><br/
<ent>carposporic</ent><br/
<mhw><hw>car`po*spor"ic</hw>, <hw>car`po*spor"ous</hw></mhw> <pos>adj.</pos> <sn>1.</sn> <def>of or relating to a carpospore.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>having carpospores.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carpus</ent><br/
||<hw>Car"pus</hw> <pr>(k<aum/r"p<ucr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carpi</plw> <pr>(k<aum/r"p<imac/)</pr>.</plu> <ety>[NL., fr. Gr. <grk>karpo`s</grk> wrist.]</ety> <fld>(Anat.)</fld> <def>The wrist; the bones or cartilages between the forearm, or antibrachium, and the hand or forefoot; in man, consisting of eight short bones disposed in two rows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrack</ent><br/
<hw>Car"rack</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Carack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrigeen</ent><br/
<ent>Carrageen</ent><br/
<mhw>{ <hw>Car"ra*geen`</hw> <pr>(?)</pr>, <hw>Car"ri*geen`</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>A small, purplish, branching, cartilaginous seaweed (<spn>Chondrus crispus</spn>), which, when bleached, is the <altname>Irish moss</altname> of commerce.</def> <altsp>[Also written <asp>carragheen</asp>, <asp>carageen</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrigeenin</ent><br/
<ent>Carrageenan</ent><br/
<mhw>{ <hw>Car`ra*geen"an</hw> <pr>(?)</pr>, <hw>Car`ri*geen"in</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>a colloidal material obtained from <prodby>seaweed</prodby> or <prodby>Irish moss</prodby>, used as an thickening or emulsifying agent and for stabilizing foods, cosmetics, and pharmaceuticals.</def><br/
[<source>PJC</source>]</p>

<p><ent>Carrancha</ent><br/
||<hw>Car*ran"cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>The Brazilian kite (<spn>Polyborus Brasiliensis</spn>); -- so called in imitation of its notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carraway</ent><br/
<hw>Car"ra*way</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caraway</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrel</ent><br/
<hw>Car"rel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Quarrel</er>, an arrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrel</ent><br/
<hw>Car"rel</hw>, <pos>n.</pos> <fld>(Arch.)</fld> <sn>1.</sn> <def>Same as 4th <er>Carol</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a table partitioned by vertical boards into small areas where an individual may read or study with minimal distraction from activities nearby.  They are used especially in libraries.  Also, the term is used to refer to one partition of such a table.  Related etymologically to the 4th <er>carol</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Carriable</ent><br/
<hw>Car"ri*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being carried.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carriage</ent><br/
<hw>Car"riage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cariage</ets> luggage, carriage, <ets>chariage</ets> carriage, cart, baggage, F. <ets>charriage</ets>, cartage, wagoning, fr. OF. <ets>carier</ets>, <ets>charier</ets>, F. <ets>charrier</ets>, to cart. See <er>Carry</er>.]</ety> <sn>1.</sn> <def>That which is carried; burden; baggage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>David left his <qex>carriage</qex> in the hand of the keeper of the <qex>carriage</qex>.</q> <rj><qau>1. Sam. xvii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And after those days we took up our <qex>carriages</qex> and went up to Jerusalem.</q> <rj><qau>Acts. xxi. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of carrying, transporting, or conveying.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nine days employed in <qex>carriage</qex>.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The price or expense of carrying.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which carries of conveys,</def> as: <sd>(a)</sd> <def>A wheeled vehicle for persons, esp. one designed for elegance and comfort.</def> <sd>(b)</sd> <def>A wheeled vehicle carrying a fixed burden, <as>as a gun <ex>carriage</ex></as>.</def> <sd>(c)</sd> <def>A part of a machine which moves and carries of supports some other moving object or part.</def> <sd>(d)</sd> <def>A frame or cage in which something is carried or supported; <as>as, a bell <ex>carriage</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The manner of carrying one's self; behavior; bearing; deportment; personal manners.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His gallant <qex>carriage</qex> all the rest did grace.</q> <rj><qau>Stirling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The act or manner of conducting measures or projects; management.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The passage and whole <qex>carriage</qex> of this action.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carriage horse</b></col>, <cd>a horse kept for drawing a carriage.</cd> -- <col><b>Carriage porch</b></col> <fld>(Arch.)</fld>, <cd>a canopy or roofed pavilion covering the driveway at the entrance to any building. It is intended as a shelter for those who alight from vehicles at the door; -- sometimes erroneously called in the United States <altname>porte-coch<egrave/re</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carriageable</ent><br/
<hw>Car"riage*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Passable by carriages; that can be conveyed in carriages.</def> <mark>[R.]</mark>  <rj><au>Ruskin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>carriageway</ent><br/
<hw>carriageway</hw> <pos>n.</pos> <def>one of the two sides of a motorway where traffic travels in one direction only, usually in two or three lanes.</def> <mark>[British]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>carriage bolt</ent><br/
<hw>carriage bolt</hw> <pos>n.</pos> <def>a type of bolt threaded only at the end opposite the head, used mostly for fastening pieces of timber together, and inserted into pre-drilled holes.</def><br/
[<source>PJC</source>]</p>

<p><ent>carriage house</ent><br/
<hw>carriage house</hw> <pos>n.</pos> <def>a small building usually near a large residence or part of an estate, used for keeping coaches, carriages, or other vehicles; -- also called <altname>coach house</altname>.  It is now (1998) obsolescent and its function has been taken over by the garage, which is usually attached to a residence or main building.  Carriage houses are still found on older estates, though not usually used for their original purpose.</def><br/
[<source>PJC</source>]</p>

<p><ent>Carriboo</ent><br/
<hw>Car"ri*boo</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caribou</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrick</ent><br/
<hw>Car"rick</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A carack. See <er>Carack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carrick bend</b></col> <fld>(Naut.)</fld>, <cd>a kind of knot, used for bending together hawsers or other ropes.</cd> -- <col><b>Carrick bitts</b></col> <fld>(Naut.)</fld>, <cd>the bitts which support the windlass.</cd>  <rj><au>Totten.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrier</ent><br/
<hw>Car"ri*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Carry</er>.]</ety> <sn>1.</sn> <def>One who, or that which, carries or conveys; a messenger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The air which is but . . . a <qex>carrier</qex> of the sounds.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is employed, or makes it his business, to carry goods for others for hire; a porter; a teamster.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The roads are crowded with <qex>carriers</qex>, laden with rich manufactures.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>That which drives or carries; as: <sd>(a)</sd> A piece which communicates to an object in a lathe the motion of the face plate; a lathe dog. <sd>(b)</sd> A spool holder or bobbin holder in a braiding machine. (c) A movable piece in magazine guns which transfers the cartridge to a position from which it can be thrust into the barrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carrier pigeon</b></col> <fld>(Zool.)</fld>, <cd>a variety of the domestic pigeon used to convey letters from a distant point to to its home.</cd> -- <col><b>Carrier shell</b></col> <fld>(Zool.)</fld>, <cd>a univalve shell of the genus <gen>Phorus</gen>; -- so called because it fastens bits of stones and broken shells to its own shell, to such an extent as almost to conceal it.</cd> -- <col><b>Common carrier</b></col> <fld>(Law.)</fld> <cd>See under <er>Common</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrion</ent><br/
<hw>Car"ri*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>caroyne</ets>, OF. <ets>caroigne</ets>, F. <ets>charogne</ets>, LL. <ets>caronia</ets>, fr. L. <ets>caro</ets> flesh Cf. <er>Crone</er>, <er>Crony</er>.]</ety> <sn>1.</sn> <def>The dead and putrefying body or flesh of an animal; flesh so corrupted as to be unfit for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They did eat the dead <qex>carrions</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A contemptible or worthless person; -- a term of reproach.</def> <mark>[Obs.]</mark> <ldquo/Old feeble <xex>carrions</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrion</ent><br/
<hw>Car"ri*on</hw>, <pos>a.</pos> <def>Of or pertaining to dead and putrefying carcasses; feeding on carrion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A prey for <qex>carrion</qex> kites.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carrion beetle</b></col> <fld>(Zool.)</fld>, <cd>any beetle that feeds habitually on dead animals; -- also called <altname>sexton beetle</altname> and <altname>burying beetle</altname>.  There are many kinds, belonging mostly to the family <fam>Silphid<ae/</fam>.</cd> -- <col><b>Carrion buzzard</b></col> <fld>(Zool.)</fld>, <cd>a South American bird of several species and genera (as <gen>Ibycter</gen>, <gen>Milvago</gen>, and <gen>Polyborus</gen>), which act as scavengers. See <er>Caracara</er>.</cd> -- <col><b>Carrion crow</b></col>, <cd>the common European crow (<spn>Corvus corone</spn>) which feeds on carrion, insects, fruits, and seeds.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrol</ent><br/
<hw>Car"rol</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>See 4th <er>Carol</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrom</ent><br/
<hw>Car"rom</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Billiards)</fld> <def>See <er>Carom</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carromata</ent><br/
||<hw>Car`ro*ma"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. in Phil. I.]</ety> <def>In the Philippines, a light, two-wheeled, boxlike vehicle usually drawn by a single native pony and used to convey passengers within city limits or for traveling. It is the common public carriage.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Carronade</ent><br/
<hw>Car`ron*ade</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Carron</ets>, in Scotland where it was first made.]</ety> <fld>(Med.)</fld> <def>A kind of short cannon, formerly in use, designed to throw a large projectile with small velocity, used for the purpose of breaking or smashing in, rather than piercing, the object aimed at, as the side of a ship. It has no trunnions, but is supported on its carriage by a bolt passing through a loop on its under side.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 221 --></p>

<p><ent>Carron oil</ent><br/
<hw>Car"ron oil</hw> <pr>(k<acr/r"r<ucr/n oil)</pr>. <def>A lotion of linseed oil and lime water, used as an application to burns and scalds; -- first used at the Carron iron works in Scotland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrot</ent><br/
<hw>Car"rot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carotte</ets>, fr. L. <ets>carota</ets>; cf. Gr. <?/]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An umbelliferous biennial plant (<spn>Daucus Carota</spn>), of many varieties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The esculent root of cultivated varieties of the plant, usually spindle-shaped, and of a reddish yellow color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carroty</ent><br/
<hw>Car"rot*y</hw>, <pos>a.</pos> <def>Like a carrot in color or in taste; -- an epithet given to reddish yellow hair, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrow</ent><br/
<hw>Car"row</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir & Gael. <ets>carach</ets> cunning.]</ety> <def>A strolling gamester.</def> <mark>[Ireland]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carry</ent><br/
<hw>Car"ry</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carried</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carrying</conjf>.]</vmorph> <ety>[OF. <ets>carier</ets>, <ets>charier</ets>, F. <ets>carrier</ets>, to cart, from OF. <ets>car</ets>, <ets>char</ets>, F. <ets>car</ets>, car. See <er>Car</er>.]</ety> <sn>1.</sn> <def>To convey or transport in any manner from one place to another; to bear; -- often with <xex>away</xex> or <xex>off</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When he dieth he shall <qex>carry</qex> nothing away.</q> <rj><qau>Ps. xiix. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Devout men <qex>carried</qex> Stephen to his burial.</q> <rj><qau>Acts viii, 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Another <qex>carried</qex> the intelligence to Russell.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sound will be <qex>carried</qex>, at the least, twenty miles.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have or hold as a burden, while moving from place to place; to have upon or about one's person; to bear; <as>as, to <ex>carry</ex> a wound; to <ex>carry</ex> an unborn child</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the ideas . . . were <qex>carried</qex> along with us in our minds.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To move; to convey by force; to impel; to conduct; to lead or guide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go, <qex>carry</qex> Sir John Falstaff to the Fleet.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>carried</qex> away all his cattle.</q> <rj><qau>Gen. xxxi. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Passion and revenge will <qex>carry</qex> them too far.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To transfer from one place (as a country, book, or column) to another; <as>as, to <ex>carry</ex> the war from Greece into Asia; to <ex>carry</ex> an account to the ledger; to <ex>carry</ex> a number in adding figures</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To convey by extension or continuance; to extend; <as>as, to <ex>carry</ex> the chimney through the roof; to <ex>carry</ex> a road ten miles farther</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To bear or uphold successfully through conflict, as a leader or principle; hence, to succeed in, as in a contest; to bring to a successful issue; to win; <as>as, to <ex>carry</ex> an election</as>.</def> <ldquo/The greater part <ex>carries</ex> it.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>carrying</qex> of our main point.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To get possession of by force; to capture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The town would have been <qex>carried</qex> in the end.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To contain; to comprise; to bear the aspect of ; to show or exhibit; to imply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He thought it <qex>carried</qex> something of argument in it.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>carries</qex> too great an imputation of ignorance.</q> <rj><qau>Lacke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To bear (one's self); to behave, to conduct or demean; -- with the reflexive pronouns.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>carried</qex> himself so insolently in the house, and out of the house, to all persons, that he became odious.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To bear the charges or burden of holding or having, as stocks, merchandise, etc., from one time to another; <as>as, a merchant is <ex>carrying</ex> a large stock; a farm <ex>carries</ex> a mortgage; a broker <ex>carries</ex> stock for a customer; to <ex>carry</ex> a life insurance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carry arms</b></col> <fld>(Mil. Drill)</fld>, <cd>a command of the Manual of Arms directing the soldier to hold his piece in the right hand, the barrel resting against the hollow of the shoulder in a nearly perpendicular position. In this position the soldier is said to stand, and the musket to be held, <xex>at carry</xex>.</cd> -- <col><b>To carry all before one</b></col>, <cd>to overcome all obstacles; to have uninterrupted success.</cd> -- <col><b>To carry arms</b></col> <sd>(a)</sd> <cd>To bear weapons.</cd> <sd>(b)</sd> <cd>To serve as a soldier.</cd> -- <col><b>To carry away</b></col>. <sd>(a)</sd> <fld>(Naut.)</fld> <cd>to break off; to lose; as, <xex>to carry away</xex> a fore-topmast.</cd> <sd>(b)</sd> <cd>To take possession of the mind; to charm; to delude; as, <xex>to be carried</xex> by music, or by temptation.</cd> -- <col><b>To carry coals</b></col>, <cd>to bear indignities tamely, a phrase used by early dramatists, perhaps from the mean nature of the occupation.</cd> <au>Halliwell.</au> -- <col><b>To carry coals to Newcastle</b></col>, <cd>to take things to a place where they already abound; to lose one's labor.</cd> -- <col><b>To carry off</b></col> <sd>(a)</sd> <cd>To remove to a distance.</cd> <sd>(b)</sd> <cd>To bear away as from the power or grasp of others.</cd> <sd>(c)</sd> <cd>To remove from life; <as>as, the plague <ex>carried off</ex> thousands</as>.</cd> -- <col><b>To carry on</b></col> <sd>(a)</sd> <cd>To carry farther; to advance, or help forward; to continue; <as>as, <ex>to carry on</ex> a design</as>.</cd> <sd>(b)</sd> <cd>To manage, conduct, or prosecute; <as>as, <ex>to carry on</ex> husbandry or trade</as>.</cd> -- <col><b>To carry out</b></col>. <sd>(a)</sd> <cd>To bear from within.</cd> <sd>(b)</sd> <cd>To put into execution; to bring to a successful issue.</cd> <sd>(c)</sd> <cd>To sustain to the end; to continue to the end.</cd> -- <col><b>To carry through</b></col>. <sd>(a)</sd> <cd>To convey through the midst of.</cd> <sd>(b)</sd> <cd>To support to the end; to sustain, or keep from falling, or being subdued.</cd> <ldquo/Grace will <xex>carry</xex> us . . . <xex>through</xex> all difficulties.<rdquo/ <au>Hammond.</au> <sd>(c)</sd> <cd>To complete; to bring to a successful issue; to succeed.</cd> -- <col><b>To carry up</b></col>, <cd>to convey or extend in an upward course or direction; to build.</cd> -- <col><b>To carry weight</b></col>. <sd>(a)</sd> <cd>To be handicapped; to have an extra burden, as when one rides or runs.</cd> <ldquo/He <xex>carries weight</xex>, he rides a race<rdquo/ <au>Cowper.</au> <sd>(b)</sd> <cd>To have influence.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carry</ent><br/
<hw>Car"ry</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To act as a bearer; to convey anything; <as>as, to fetch and <ex>carry</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have propulsive power; to propel; <as>as, a gun or mortar <ex>carries</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hold the head; -- said of a horse; <as>as, to <ex>carry</ex> well i</as>. e., to hold the head high, with arching neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Hunting)</fld> <def>To have earth or frost stick to the feet when running, as a hare.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To carry on</b></col>, <cd>to behave in a wild, rude, or romping manner.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carry</ent><br/
<hw>Car"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Carries</plw> <pr>(#)</pr></plu>. <def>A tract of land, over which boats or goods are carried between two bodies of navigable water; a carrying place; a portage.</def> <ety>[<ets>U.S</ets>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Carryall</ent><br/
<hw>Car"ry*all`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted fr. <ets>cariole</ets>.]</ety> <def>A light covered carriage, having four wheels and seats for four or more persons, usually drawn by one horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrying</ent><br/
<hw>Car"ry*ing</hw>, <pos>n.</pos> <def>The act or business of transporting from one place to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carrying place</b></col>, <cd>a carry; a portage.</cd> -- <col><b>Carrying trade</b></col>, <cd>the business of transporting goods, etc., from one place or country to another by water or land; freighting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>We are rivals with them in . . . the <qex>carrying trade</qex>.</q> <rj><qau>Jay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carryk</ent><br/
<hw>Car"ryk</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A carack.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carrytale</ent><br/
<hw>Car"ry*tale`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A talebearer.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carse</ent><br/
<hw>Carse</hw> <pr>(k<aum/rs)</pr>, <pos>n.</pos> <ety>[Of Celtic origin; cf. W. <ets>cars</ets> bog, fen. <ets>carsen</ets> reed, Armor. <ets>kars</ets>, <ets>korsen</ets>, bog plant, reed.]</ety> <def>Low, fertile land; a river valley.</def> <mark>[Scot.]</mark>  <rj><au>Jomieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>carsick</ent><br/
<hw>car"sick`</hw> <pr>(k<aum/r"s<icr/k`)</pr>, <pos>adj.</pos> <def>feeling nauseous due to the movement of a car or other land vehicle; -- similar to <er>airsick</er> and <er>seasick</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cart</ent><br/
<hw>Cart</hw> <pr>(k<aum/rt)</pr>, <pos>n.</pos> <ety>[AS. <ets>cr<ae/t</ets>; cf. W. <ets>cart</ets>, Ir. & Gael. <ets>cairt</ets>, or Icel. <ets>kartr</ets>. Cf. <er>Car</er>.]</ety> <sn>1.</sn> <def>A common name for various kinds of vehicles, as a Scythian dwelling on wheels, or a chariot.</def> <ldquo/Ph<oe/bus' <xex>cart</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A two-wheeled vehicle for the ordinary purposes of husbandry, or for transporting bulky and heavy articles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Packing all his goods in one poor <qex>cart</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A light business wagon used by bakers, grocerymen, butchers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An open two-wheeled pleasure carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cart horse</b></col>, <cd>a horse which draws a cart; a horse bred or used for drawing heavy loads; -- also spelled <asp>carthorse</asp>.</cd> -- <col><b>Cart rope</b></col>, <cd>a stout rope for fastening a load on a cart; any strong rope.</cd> -- <mcol><col><b>To put the cart before the horse</b></col>, <col><b>To get the cart before the horse</b></col>, <it>or</it> <col><b>To set the cart before the horse</b></col></mcol>, <cd>to invert the order of related facts or ideas, as by putting an effect for a cause; to do things in an improper order.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cart</ent><br/
<hw>Cart</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Carting</conjf>.]</vmorph> <sn>1.</sn> <def>To carry or convey in a cart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To expose in a cart by way of punishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She chuckled when a bawd was <qex>carted</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cart</ent><br/
<hw>Cart</hw>, <pos>v. i.</pos> <def>To carry burdens in a cart; to follow the business of a carter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartage</ent><br/
<hw>Cart"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of carrying in a cart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The price paid for carting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartbote</ent><br/
<hw>Cart"bote`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cart</ets> + <ets>bote</ets>.]</ety> <fld>(Old Eng. Law.)</fld> <def>Wood to which a tenant is entitled for making and repairing carts and other instruments of husbandry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carte</ent><br/
||<hw>Carte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See 1st <er>Card</er>.]</ety> <sn>1.</sn> <def>Bill of fare.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Short for <er>Carte de visite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carte</ent><br/
<ent>Quarte</ent><br/
<mhw>{ <hw>Carte</hw>. ||<hw>Quarte</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>quarte</ets>, prop., a fourth. Cf. <er>Quart</er>.]</ety> <fld>(Fencing)</fld> <def>A position in thrusting or parrying, with the inside of the hand turned upward and the point of the weapon toward the adversary's right breast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carte blanche</ent><br/
||<hw>Carte` blanche"</hw> <pr>(?)</pr>. <ety>[F., fr. OF. <ets>carte</ets> paper + <ets>-blanc</ets>, <ets>blanche</ets>, white. See 1st <er>Card</er>.]</ety> <def>A blank paper, with a person's signature, etc., at the bottom, given to another person, with permission to superscribe what conditions he pleases. Hence: Unconditional terms; unlimited authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carte de visite</ent><br/
||<hw>Carte" de vi*site`</hw> <pr>(?)</pr>, <plu><it>pl.</it> <plw>Cartes de visite</plw> <pr>(<?/)</pr></plu>. <ety>[F.]</ety> <sn>1.</sn> <def>A visiting card.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A photographic picture of the size formerly in use for a visiting card.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartel</ent><br/
<hw>Car*tel"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>cartellus</ets> a little paper, dim. fr. L. <ets>charta</ets>. See 1st <er>Card</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Mil.)</fld> <def>An agreement between belligerents for the exchange of prisoners.</def>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A letter of defiance or challenge; a challenge to single combat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He is cowed at the very idea of a <qex>cartel</qex>.,</q> <rj><qau>Sir W. Scott.</qau></rj></p>

<p><-- 3. a formal or informal arrangement (sometimes unlawful) among independent commercial enterprises organized for the purpose of common gain, as by limiting competition or fixing prices --><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cartel</b></col>, or <col><b>Cartel ship</b></col></mcol>, <cd>a ship employed in the exchange of prisoners, or in carrying propositions to an enemy; a ship beating a flag of truce and privileged from capture.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartel</ent><br/
<hw>Car"tel</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To defy or challenge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You shall <qex>cartel</qex> him.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carter</ent><br/
<hw>Cart"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A charioteer.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A man who drives a cart; a teamster.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any species of <gen>Phalangium</gen>; -- also called <altname>harvestman</altname></def>. <sd>(b)</sd> <def>A British fish; the whiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartesian</ent><br/
<hw>Car*te"sian</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From Renatus <ets>Cartesius</ets>, Latinized from of Ren<eacute/ <ets>Descartes</ets>: cf. F. <ets>cart<eacute/sien</ets>.]</ety> <def>Of or pertaining to the French philosopher Ren<eacute/ Descartes, or his philosophy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>Cartesion</qex> argument for reality of matter.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cartesian coordinates</b></col> <fld>(Geom)</fld>, <cd>distance of a point from lines or planes; -- used in a system of representing geometric quantities, invented by Descartes.</cd> -- <col><b>Cartesian devil</b></col>, <cd>a small hollow glass figure, used in connection with a jar of water having an elastic top, to illustrate the effect of the compression or expansion of air in changing the specific gravity of bodies.</cd> -- <col><b>Cartesion oval</b></col> <fld>(Geom.)</fld>, <cd>a curve such that, for any point of the curve <mathex>mr + m<prime/r<prime/ = c</mathex>, where <it>r</it> and <it>r<prime/</it> are the distances of the point from the two foci and <it>m</it>, <it>m<prime/</it> and <it>c</it> are constant; -- used by Descartes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartesian</ent><br/
<hw>Car*te"sian</hw>, <pos>n.</pos> <def>An adherent of Descartes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartesianism</ent><br/
<hw>Car*te"sian*ism</hw>, <pos>n.</pos> <def>The philosophy of Descartes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carthaginian</ent><br/
<hw>Car`tha*gin"i*an</hw>, <pos>a.</pos> <def>Of a pertaining to ancient Carthage, a city of northern Africa.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Carthage.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carthamin</ent><br/
<hw>Car"tha*min</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A red coloring matter obtained from the safflower, or <spn>Carthamus tinctorius</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cart horse</ent><br/
<ent>carthorse</ent><br/
<mhw><hw>cart"horse`</hw>, <hw>cart" horse`</hw></mhw> <pos>n.</pos> <def>a horse kept for pulling carts; a horse bred or used for drawing heavy loads.</def><br/
<syn><b>Syn. --</b> drayhorse.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Carthusian</ent><br/
<hw>Car*thu"sian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>Cartusianus</ets>, <ets>Cartusiensis</ets>, from the town of <ets>Chartreuse</ets>, in France.]</ety> <fld>(Eccl. Hist.)</fld> <def>A member of an exceeding austere religious order, founded at Chartreuse in France by <person>St. Bruno</person>, in the year 1086.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carthusian</ent><br/
<hw>Car*thu"sian</hw>, <pos>a.</pos> <def>Pertaining to the Carthusian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartilage</ent><br/
<hw>Car"ti*lage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cartilago</ets>; cf. F. <ets>cartilage</ets>.]</ety> <fld>(Anat.)</fld> <def>A translucent, elastic tissue; gristle.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cartilage</xex> contains no vessels, and consists of a homogeneous, intercellular matrix, in which there are numerous minute cavities, or capsules, containing protoplasmic cells, the cartilage corpuscul. See <xex>Illust</xex> under <er>Duplication</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Articular cartilage</b></col>, <cd>cartilage that lines the joints.</cd> -- <col><b>Cartilage bone</b></col> <fld>(Anat.)</fld>, <cd>any bone formed by the ossification of cartilage.</cd> -- <col><b>Costal cartilage</b></col>, <cd>cartilage joining a rib with he sternum. See <xex>Illust.</xex> of <er>Thorax</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartilagineous</ent><br/
<hw>Car`ti*la*gin"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cartilageneus</ets>.]</ety> <def>See <er>Cartilaginous</er>.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartilaginification</ent><br/
<hw>Car`ti*la*gin`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cartilago</ets>, <ets>-laginis</ets>, cartilage + <ets>facere</ets> to make.]</ety> <def>The act or process of forming cartilage.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartilaginous</ent><br/
<hw>Car`ti*lag"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cartilaginosus</ets>: cf. F. <ets>cartilagineux</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to cartilage; gristly; firm and tough like cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having the skeleton in the state of cartilage, the bones containing little or no calcareous matter; said of certain fishes, as the sturgeon and the sharks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartist</ent><br/
<hw>Car"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cartista</ets>, fr. <ets>carta</ets> paper, document (cf. Pg. <ets>carta</ets>). See <er>Charta</er>; cf. <er>Chartist</er>.]</ety> <def>In Spain and Portugal, one who supports the constitution.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cart load</ent><br/
<ent>cartload</ent><br/
<mhw><hw>cartload</hw>, <hw>cart load</hw></mhw> <pos>n.</pos> <def>as much as will fill or load a cart; the quantity that a cart holds. In excavating and carting sand, gravel, earth, etc., one third of a cubic yard of the material before it is loosened is estimated to be a cart load.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartman</ent><br/
<hw>Cart"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Cartmen</er> <pr>(<?/)</pr>.</plu> <def>One who drives or uses a cart; a teamster; a carter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartogram</ent><br/
<hw>Car"to*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cartogramme</ets>.]</ety> <def>A map showing geographically, by shades or curves, statistics of various kinds; a statistical map.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cartographer</ent><br/
<hw>Car*tog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes charts or maps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartographical</ent><br/
<ent>Cartographic</ent><br/
<mhw>{ <hw>Car`to*graph"ic</hw> <pr>(?)</pr>, <hw>Car`to*graph"ic*al</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <def>Of or pertaining to cartography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartographically</ent><br/
<hw>Car`to*graph"ic*al*ly</hw>, <pos>adv.</pos> <def>By cartography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartography</ent><br/
<hw>Car*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cartographie</ets>. See <er>Card</er>, and <er>-graphy</er>.]</ety> <def>The art or business of forming charts or maps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartomancy</ent><br/
<hw>Car"to*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cartomancie</ets>. See <er>Card</er>, and <er>-mancy</er>.]</ety> <def>The art of telling fortunes with cards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carton</ent><br/
<hw>Car"ton</hw> <pr>(k<aum/r"t<ocr/n)</pr>, <pos>n.</pos> <ety>[F. See <er>Cartoon</er>.]</ety> <def>Pasteboard for paper boxes; also, a pasteboard box.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Carton pierre</b></col> <pr>(<?/)</pr>, <cd>a species of <isa>papier-mach<eacute/</isa>, imitating stone or bronze sculpture.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cartonful</ent><br/
<hw>cartonful</hw> <pos>n.</pos> <def>the quantity contained in a carton.</def><br/
<syn><b>Syn. --</b> carton.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cartoon</ent><br/
<hw>Car*toon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carton</ets> (cf. It. <ets>cartone</ets> pasteboard, cartoon); fr. L. <ets>charta</ets>. See 1st <er>card</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A design or study drawn of the full size, to serve as a model for transferring or copying; -- used in the making of mosaics, tapestries, fresco pantings and the like; <as>as, the <ex>cartoons</ex> of Raphael</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large pictorial sketch, as in a journal or magazine; esp. a pictorial caricature; <as>as, the <ex>cartoons</ex> of <ldquo/Puck.<rdquo/</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>same as <er>comic strip</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a motion picture consisting of a series of frames, each being a photograph of a drawing rather than a frame produced by filming a scene of true action, and in which the objects are displaced slightly in succeeding frames so as to give the appearance of motion when projected as a motion picture on the screen.  The types of characters portrayed in such films are often similar or identical to those in a <er>comic strip</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cartoonist</ent><br/
<hw>Car*toon"ist</hw>, <pos>n.</pos> <def>One skilled in drawing cartoons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cartouche</ent><br/
<ent>Cartouch</ent><br/
<mhw><hw>Car*touch"</hw>, <hw>car*touche"</hw></mhw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cartouches</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>cartouche</ets>, It. <ets>cartuccia</ets>, <ets>cartoccio</ets>, cornet, cartouch, fr. L. <ets>charta</ets> paper. See 1st <er>Card</er>, and cf. <er>Cartridge</er>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A roll or case of paper, etc., holding a charge for a firearm; a cartridge</def>. <sd>(b)</sd> <def>A cartridge box.</def> <sd>(c)</sd> <def>A wooden case filled with balls, to be shot from a cannon.</def> <sd>(d)</sd> <def>A gunner's bag for ammunition</def>. <sd>(e)</sd> <def>A military pass for a soldier on furlough.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A cantalever, console, corbel, or modillion, which has the form of a scroll of paper</def>. <sd>(b)</sd> <def>A tablet for ornament, or for receiving an inscription, formed like a sheet of paper with the edges rolled up; hence, any tablet of ornamental form.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Egyptian Antiq.)</fld> <def>An oval figure on monuments, and in papyri, containing the name of a sovereign.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartridge</ent><br/
<hw>Car"tridge</hw> <pr>(k<aum/r"tr<icr/j)</pr>, <pos>n.</pos> <ety>[Formerly <ets>cartrage</ets>, corrupted fr. F. <ets>cartouche</ets>. See <er>Cartouch</er>.]</ety> <fld>(Mil.)</fld> <def>A complete charge for a firearm, contained in, or held together by, a case, capsule, or shell of metal, pasteboard, or other material.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ball cartridge</b></col>, <cd>a cartridge containing a projectile.</cd> -- <col><b>Blank cartridge</b></col>, <cd>a cartridge without a projectile.</cd> -- <col><b>Center-fire cartridge</b></col>, <cd>a cartridge in which the fulminate occupies an axial position usually in the center of the base of the capsule, instead of being contained in its rim. In the Prussian needle gun the fulminate is applied to the middle of the base of the bullet.</cd> -- <col><b>Rim-fire cartridge</b></col>, <cd>a cartridge in which the fulminate is contained in a rim surrounding its base.</cd> -- <col><b>Cartridge bag</b></col>, <cd>a bag of woolen cloth, to hold a charge for a cannon.</cd> -- <col><b>Cartridge belt</b></col>, <cd>a belt having pockets for cartridges.</cd> -- <col><b>Cartridge box</b></col>, <cd>a case, usually of leather, attached to a belt or strap, for holding cartridges.</cd> -- <col><b>Cartridge paper</b></col>. <sd>(a)</sd> <cd>A thick stout paper for inclosing cartridges.</cd> <sd>(b)</sd> <cd>A rough tinted paper used for covering walls, and also for making drawings upon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cartroad</ent><br/
<hw>cartroad</hw> <pos>n.</pos> <def>any road or path affording passage especially a rough one.</def><br/
<syn><b>Syn. --</b> track, cart track.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cartulary</ent><br/
<hw>Car"tu*la*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cartularies</plw>.</plu> <ety>[LL. <ets>cartularium</ets>, <ets>chartularium</ets>, fr. L. <ets>charta</ets> paper: cf. F. <ets>cartulaire</ets>. See 1st <er>Card</er>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A register, or record, as of a monastery or church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An ecclesiastical officer who had charge of records or other public papers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cartway</ent><br/
<hw>Cart"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A way or road for carts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cartwheel</ent><br/
<hw>cartwheel</hw> <pos>n.</pos> <sn>1.</sn> <def>the type of wheel used on a cart; it typically has wooden spokes and a metal rim.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an acrobatic maneuver in which the arms and legs are outstretched like the spokes of a wheel, and the body is turned sideways through one or more revolutions, by first touching the hands and then the feet to the ground, in rapid succession so as to mimic the rolling of a wheel; in the course of this feat, the person performing it is alternately upright and upside-down.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>a silver dollar; a dollar made of silver.</def> <mark>[Colloq.]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cartwheel</ent><br/
<hw>cartwheel</hw> <pos>v. i.</pos> <fr>1</fr> <def>to perform a <er>cartwheel{2}</er>.</def><br/
<syn><b>Syn. --</b> do cartwheels.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cartwright</ent><br/
<hw>Cart"wright`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cart</ets> + <ets>wright</ets>.]</ety> <def>An artificer who makes carts; a cart maker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carucage</ent><br/
<hw>Car"u*cage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>carrucagium</ets> (OF. <ets>charuage</ets>.), fr. LL. <ets>carruca</ets> plow, fr. L. <ets>carruca</ets> coach.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Old Eng. Law.)</fld> <def>A tax on every plow or plowland.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of plowing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carucate</ent><br/
<hw>Car"u*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>carucata</ets>, <ets>carrucata</ets>. See <er>Carucage</er>.]</ety> <def>A plowland; as much land as one team can plow in a year and a day; -- by some said to be about 100 acres.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caruncula</ent><br/
<ent>Caruncle</ent><br/
<mhw>{ <hw>Car"un*cle</hw> <pr>(?)</pr>, ||<hw>Ca*run"cu*la</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>caruncula</ets> a little piece of flesh, dim. of <ets>caro</ets> flesh.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A small fleshy prominence or excrescence; especially the small, reddish body, the <col><b>caruncula lacrymalis</b></col>, in the inner angle of the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>An excrescence or appendage surrounding or near the hilum of a seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A naked, flesh appendage, on the head of a bird, as the wattles of a turkey, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carunculous</ent><br/
<ent>Caruncular</ent><br/
<mhw>{ <hw>Ca*run"cu*lar</hw> <pr>(?)</pr>, <hw>Ca*run"cu*lous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of, pertaining to, or like, a caruncle; furnished with caruncles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carunculated</ent><br/
<ent>Carunculate</ent><br/
<mhw>{ <hw>Ca*run"cu*late</hw> <pr>(?)</pr>, <hw>Ca*run"cu*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having a caruncle or caruncles; caruncular.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 222 --></p>

<p><ent>Carus</ent><br/
||<hw>Ca"rus</hw> <pr>(k<amac/"r<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ka`ros</grk>.]</ety> <fld>(Med.)</fld> <def>Coma with complete insensibility; deep lethargy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carvacrol</ent><br/
<hw>Car"va*crol</hw> <pr>(k<aum/r"v<adot/*kr<omac/l)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A thick oily liquid, <chform>C10H13.OH</chform>, of a strong taste and disagreeable odor, obtained from oil of caraway (<spn>Carum carui</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carve</ent><br/
<hw>Carve</hw> <pr>(k<aum/rv)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Carved</conjf> <pr>(k<aum/rvd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Carving</conjf>.]</vmorph> <ety>[AS. <ets>ceorfan</ets> to cut, carve; akin to D. <ets>kerven</ets>, G. <ets>kerben</ets>, Dan. <ets>karve</ets>, Sw. <ets>karfva</ets>, and to Gr. <grk>gra`fein</grk> to write, orig. to scratch, and E. <ets>-graphy</ets>. Cf. <er>Graphic</er>.]</ety> <sn>1.</sn> <def>To cut.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Or they will <qex>carven</qex> the shepherd's throat.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut, as wood, stone, or other material, in an artistic or decorative manner; to sculpture; to engrave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Carved</qex> with figures strange and sweet.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make or shape by cutting, sculpturing, or engraving; to form; <as>as, to <ex>carve</ex> a name on a tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An angel <qex>carved</qex> in stone.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>carved</qex> not a line, and we raised not a stone.</q> <rj><qau>C. Wolfe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cut into small pieces or slices, as meat at table; to divide for distribution or apportionment; to apportion.</def> <ldquo/To <xex>carve</xex> a capon.<rdquo/  <rj><au>Shak.</au></rj>
<-- = carve up --><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cut: to hew; to mark as if by cutting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My good blade <qex>carved</qex> the casques of men.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A million wrinkles <qex>carved</qex> his skin.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To take or make, as by cutting; to provide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who could easily have <qex>carved</qex> themselves their own food.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To lay out; to contrive; to design; to plan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lie ten nights awake <qex>carving</qex> the fashion of a new doublet.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To carve out</b></col>, <cd>to make or get by cutting, or as if by cutting; to cut out.</cd>  <ldquo/[Macbeth] with his brandished steel . . . <xex>carved out</xex> his passage.<rdquo/  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Fortunes were <qex>carved out</qex> of the property of the crown.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carve</ent><br/
<hw>Carve</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To exercise the trade of a sculptor or carver; to engrave or cut figures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut up meat; <as>as, to <ex>carve</ex> for all the guests</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carve</ent><br/
<hw>Carve</hw>, <pos>n.</pos> <def>A carucate.</def> <mark>[Obs.]</mark>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>carved</ent><br/
<hw>carved</hw> <pos>adj.</pos> <def>formed by carving or having a design carved into the surface.</def> [Narrower terms: <stype>sliced</stype>]<br/
<syn><b>Syn. --</b> carven.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Carvel</ent><br/
<hw>Car"vel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. fr. <ets>caravel</ets>.]</ety> <sn>1.</sn> <def>Same as <er>Caravel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A species of jellyfish; sea blubber.</def>  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carvelbuilt</ent><br/
<hw>Car"vel*built</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Shipbuilding)</fld> <def>Having the planks meet flush at the seams, instead of lapping as in a clinker-built vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carven</ent><br/
<hw>Car"ven</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wrought by carving; ornamented by carvings; carved.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>carven</qex> bowl well wrought of beechen tree.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>carven</qex> cedarn doors.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A screen of <qex>carven</qex> ivory.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carvene</ent><br/
<hw>Car"vene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>carvi</ets> caraway.]</ety> <def>An oily substance, <chform>C10H16</chform>, extracted from oil caraway.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carver</ent><br/
<hw>Carv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who carves; one who shapes or fashions by carving, or as by carving; esp. one who carves decorative forms, architectural adornments, etc.</def> <ldquo/The <xex>carver's</xex> chisel.<rdquo/  <rj><au>Dodsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>carver</qex> of his fortunes.</q> <rj><qau>Sharp (Richardson's Dict. )</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who carves or divides meat at table.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A large knife for carving.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carving</ent><br/
<hw>Carv"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or art of one who carves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of decorative work cut in stone, wood, or other material.</def> <ldquo/<xex>Carving</xex> in wood.<rdquo/  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The whole body of decorative sculpture of any kind or epoch, or in any material; <as>as, the Italian <ex>carving</ex> of the 15th century</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carvist</ent><br/
<hw>Car"vist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>carry fist</ets>.]</ety> <fld>(Falconary)</fld> <def>A hawk which is of proper age and training to be carried on the hand; a hawk in its first year.</def>  <rj><au>Booth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Carvol</ent><br/
<hw>Car"vol</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>One of a species of aromatic oils, resembling carvacrol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Car wheel</ent><br/
<hw>Car" wheel`</hw> <pr>(?)</pr>, <def>A flanged wheel of a railway car or truck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryatid</ent><br/
<ent>Caryatic</ent><br/
<mhw>{ <hw>Car`y*at"ic</hw> <pr>(?)</pr>, <hw>Car`y*at"id</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a caryatid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryatid</ent><br/
<hw>Car`y*at"id</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Caryatids</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Caryatides</er>.]</ety> <fld>(Arch.)</fld> <def>A draped female figure supporting an entablature, in the place of a column or pilaster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryatides</ent><br/
||<hw>Car`y*at"i*des</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L., fr. Gr. <?/ (<?/) priestesses in the temple of Diana (the Greek Artemis) at Cary<ae/ (Gr. <?/), a village in Laconia; as an architectural term, caryatids.]</ety> <fld>(Arch)</fld> <def>Caryatids.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Corresponding male figures were called <xex>Atlantes</xex>, <xex>Telamones</xex>, and <xex>Persians</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryophyllaceous</ent><br/
<hw>Car`y*o*phyl*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ clove tree; <?/ nut + <?/ leaf.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Having corollas of five petals with long claws inclosed in a tubular, calyx, as the pink</def>. <sd>(b)</sd> <def>Belonging to the family of which the pink and the carnation are the types.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryophyllales</ent><br/
<hw>Caryophyllales</hw> <pos>n.</pos> <def>an order of plants which corresponds approximately to the older group Centrospermae.</def><br/
<syn><b>Syn. --</b> order <ord>Caryophyllales</ord>, Chenopodiales, order-Chenopodiales.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caryophyllidae</ent><br/
<hw>Caryophyllidae</hw> <pos>n.</pos> <def>a group of families of mostly flowers having basal or free-central placentation and trinucleate pollen (binucleate pollen is commoner in flowering plants); it contains 14 families including: <fam>Caryophyllaceae</fam> (carnations and pinks); <fam>Aizoaceae</fam>; <fam>Amaranthaceae</fam>; <fam>Batidaceae</fam>; <fam>Chenopodiaceae</fam>; <fam>Cactaceae</fam> (order <ord>Opuntiales</ord>); <fam>Nyctaginaceae</fam>; <fam>Phytolaccaceae</fam>; it corresponds approximately to order <ord>Caryophyllales</ord>; it is sometimes classified as a superorder.</def><br/
<syn><b>Syn. --</b> subclass Caryophyllidae.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caryophyllin</ent><br/
<hw>Car`y*oph"yl*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A tasteless and odorless crystalline substance, extracted from cloves, polymeric with common camphor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryophyllous</ent><br/
<hw>Car`y*oph"yl*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Caryophyllaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caryopsis</ent><br/
<hw>Car`y*op"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Caryopses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. gr. <?/ hut, kernel + <?/ sight, form.]</ety> <fld>(Bot.)</fld> <def>A one-celled, dry, indehiscent fruit, with a thin membranous pericarp, adhering closely to the seed, so that fruit and seed are incorporated in one body, forming a single grain, as of wheat, barley, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casa</ent><br/
||<hw>Ca"sa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. or It., fr. L. <ets>casa</ets>  cabin.]</ety> <def>A house or mansion.</def> <mark>[Sp. Amer. & Phil. Islands]</mark></p>

<p><q>I saw that Enriquez had made no attempt to modernize the old <qex>casa</qex>, and that even the garden was left in its lawless native luxuriance.</q>  <rj><qau>Bret Harte.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>casaba</ent><br/
<hw>casaba</hw> <pos>n.</pos> <def>a winter melon having a yellowish rind; -- called also <altname>casaba melon</altname>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casal</ent><br/
<hw>Ca"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Gram.)</fld> <def>Of or pertaining to case; <as>as, a <ex>casal</ex> ending</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casanova</ent><br/
<hw>Casanova</hw> <pos>n.</pos> <sn>1.</sn> <def>an Italian adventurer (<person>Giovanni Giacomo Casanova</person>; b. 1725; d. 1798) who wrote vivid accounts of his sexual encounters.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>any man noted for his amorous adventures.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>casava</ent><br/
<hw>casava</hw> <pos>n.</pos> <def>any of several plants of the genus <gen>Manihot</gen> having fleshy roots yielding a nutritious starch.</def><br/
<syn><b>Syn. --</b> cassava.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cascabel</ent><br/
<hw>Cas"ca*bel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cascabel</ets> a little bell, also (fr. the shape), a knob at the breech end of a cannon.]</ety> <def>The projection in rear of the breech of a cannon, usually a knob or breeching loop connected with the gun by a neck. In old writers it included all in rear of the base ring. <note>[See <xex>Illust.</xex> of <er>Cannon</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cascade</ent><br/
<hw>Cas*cade"</hw> <pr>(k<acr/s*k<amac/d")</pr>, <pos>n.</pos> <ety>[F. <ets>cascade</ets>, fr. It. <ets>cascata</ets>, fr. <ets>cascare</ets> to fall.]</ety> <def>A fall of water over a precipice, as in a river or brook; a waterfall less than a cataract.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The silver brook . . . pours the white <qex>cascade</qex>.</q> <rj><qau>Longjellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now murm'ring soft, now roaring in <qex>cascade</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cascade</ent><br/
<hw>Cas*cade"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To fall in a cascade.</def>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To vomit.</def> <mark>[Slang]</mark>  <rj><au>Smollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cascade method</ent><br/
<hw>Cas*cade" meth"od</hw>. <fld>(Physics)</fld> <def>A method of attaining successively lower temperatures by utilizing the cooling effect of the expansion of one gas in condensing another less easily liquefiable, and so on.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cascade system</ent><br/
<hw>Cascade system</hw>. <fld>(Elec.)</fld> <def>A system or method of connecting and operating two induction motors so that the primary circuit of one is connected to the secondary circuit of the other, the primary circuit of the latter being connected to the source of supply; also, a system of electric traction in which motors so connected are employed. The cascade system is also called <mcol><col><b>tandem system</b></col>, <it>or</it> <col><b>concatenated system</b></col></mcol>; the connection a <mcol><col><b>cascade connection</b></col>, <col><b>tandem connection</b></col>, <it>or</it> <col><b>concatenated connection</b></col></mcol>, or <col><b>a concatenation</b></col>; and the control of the motors so obtained a <mcol><col><b>tandem control</b></col>, <it>or</it> <col><b>concatenation control</b></col></mcol>.</def> <note>In the cascade system of traction the cascade connection is used for starting and for low speeds up to half speed. For full speed the short-circuited motor is cut loose from the other motor and is either left idle or (commonly) connected direct to the line.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cascalho</ent><br/
||<hw>Cas*cal"ho</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg., a chip of stone, gravel.]</ety> <def>A deposit of pebbles, gravel, and ferruginous sand, in which the Brazilian diamond is usually found.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cascara</ent><br/
<hw>cascara</hw> <pos>n.</pos> <def>the dried bark of the <prodby>cascara buckthorn</prodby> used as a laxative; -- called also <altname>cascara sagrada</altname>.</def><br/
<syn><b>Syn. --</b> chittam bark, chittem bark.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cascara buckthorn</ent><br/
<hw>Cas"ca*ra buck"thorn`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The buckthorn (<spn>Rhamnus Purshiana</spn>) of the Pacific coast of the United States, which yields <prod>cascara sagrada</prod>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cascara sagrada</ent><br/
||<hw>Cas"ca*ra sa*gra"da</hw> <pr>(k<adot/s"k<adot/*r<adot/ s<adot/*gr<aum/"d<adot/)</pr>. <ety>[Sp.]</ety> <def>Holy bark; the bark of the California buckthorn (<spn>Rhamnus Purshianus</spn>), used as a mild cathartic or laxative; -- called also <altname>cascara</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cascarilla</ent><br/
<hw>Cas`ca*ril"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., small thin bark, Peruvian bark, dim. of <ets>c<aacute/scara</ets> bark.]</ety> <fld>(Bot.)</fld> <def>A euphorbiaceous West Indian shrub (<spn>Croton Eleutheria</spn>); also, its aromatic bark.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cascarilla bark</b></col> (<it>or</it> <col><b>Cascarilla</b></col></mcol>) <fld>(Med.)</fld>, <cd>the bark of <spn>Croton Eleutheria</spn>.  It has an aromatic odor and a warm, spicy, bitter taste, and when burnt emits a musky odor. It is used as a gentle tonic, and sometimes, for the sake of its fragrance, mixed with smoking tobacco, when it is said to occasion vertigo and intoxication.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cascarillin</ent><br/
<hw>Cas`ca*ril"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white, crystallizable, bitter substance extracted from oil of cascarilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cascaron</ent><br/
||<hw>Cas`ca*ron"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cascar<oacute/n</ets>.]</ety> <def>Lit., an eggshell; hence, an eggshell filled with confetti to be thrown during balls, carnivals, etc.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Case</ent><br/
<hw>Case</hw> <pr>(k<amac/s)</pr>, <pos>n.</pos> <ety>[OF. <ets>casse</ets>, F. <ets>caisse</ets> (cf. It. <ets>cassa</ets>), fr. L. <ets>capsa</ets> chest, box, case, fr. <ets>capere</ets> to take, hold. See <er>Capacious</er>, and cf. 4th <er>Chase</er>, <er>Cash</er>, <er>Enchase</er>, 3d <er>Sash</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A box, sheath, or covering; <as>as, a <ex>case</ex> for holding goods; a <ex>case</ex> for spectacles; the <ex>case</ex> of a watch; the <ex>case</ex> (capsule) of a cartridge; a <ex>case</ex> (cover) for a book</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A box and its contents; the quantity contained in a box; <as>as, a <ex>case</ex> of goods; a <ex>case</ex> of instruments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Print.)</fld> <def>A shallow tray divided into compartments or <ldquo/boxes<rdquo/ for holding type.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Cases</ex> for type are usually arranged in sets of two, called respectively the <xex>upper</xex> and the <xex>lower</xex> case. The <stype>upper case</stype> contains capitals, small capitals, accented and marked letters, fractions, and marks of reference: the <stype>lower case</stype> contains the small letters, figures, marks of punctuation, quadrats, and spaces.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An inclosing frame; a casing; <as>as, a door <ex>case</ex>; a window <ex>case</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mining)</fld> <def>A small fissure which admits water to the workings.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Case</ent><br/
<hw>Case</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cased</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Casing</conjf>.]</vmorph> <sn>1.</sn> <def>To cover or protect with, or as with, a case; to inclose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The man who, <qex>cased</qex> in steel, had passed whole days and nights in the saddle.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strip the skin from; <as>as, to <ex>case</ex> a box</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Case</ent><br/
<hw>Case</hw>, <pos>n.</pos> <ety>[F. <ets>cas</ets>, fr. L. <ets>casus</ets>, fr. <ets>cadere</ets> to fall, to happen. Cf. <er>Chance</er>.]</ety> <sn>1.</sn> <def>Chance; accident; hap; opportunity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By aventure, or sort, or <qex>cas</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which befalls, comes, or happens; an event; an instance; a circumstance, or all the circumstances; condition; state of things; affair; <as>as, a strange <ex>case</ex>; a <ex>case</ex> of injustice; the <ex>case</ex> of the Indian tribes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In any <qex>case</qex> thou shalt deliver him the pledge.</q> <rj><qau>Deut. xxiv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If the <qex>case</qex> of the man be so with his wife.</q> <rj><qau>Matt. xix. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And when a lady's in the <qex>case</qex><br/
You know all other things give place.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You think this madness but a common <qex>case</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am in <qex>case</qex> to justle a constable,</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med. & Surg.)</fld> <def>A patient under treatment; an instance of sickness or injury; <as>as, ten <ex>cases</ex> of fever</as>; also, the history of a disease or injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A proper remedy in hypochondriacal <qex>cases</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>The matters of fact or conditions involved in a suit, as distinguished from the questions of law; a suit or action at law; a cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us consider the reason of the <qex>case</qex>, for nothing is law that is not reason.</q> <rj><qau>Sir John Powell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Not one <qex>case</qex> in the reports of our courts.</q> <rj><qau>Steele.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gram.)</fld> <def>One of the forms, or the inflections or changes of form, of a noun, pronoun, or adjective, which indicate its relation to other words, and in the aggregate constitute its declension; the relation which a noun or pronoun sustains to some other word.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Case</qex> is properly a <qex>falling off</qex> from the nominative or first state of word; the name for which, however, is now, by extension of its signification, applied also to the nominative.</q> <rj><qau>J. W. Gibbs.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cases</xex> other than the nominative are <xex>oblique cases</xex>. <xex>Case endings</xex> are terminations by which certain cases are distinguished. In old English, as in Latin, nouns had several cases distinguished by <xex>case endings</xex>, but in modern English only that of the possessive case is retained.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Action on the case</b></col> <fld>(Law)</fld>, <cd>according to the old classification (now obsolete), was an action for redress of wrongs or injuries to person or property not specially provided against by law, in which the whole cause of complaint was set out in the writ; -- called also <altname>trespass on the case</altname>, or simply <altname>case</altname>.</cd> -- <col><b>All a case</b></col>, <cd>a matter of indifference.</cd>  <mark>[Obs.]</mark> <ldquo/It is <xex>all a case</xex> to me.<rdquo/ <au>L'Estrange.</au> -- <col><b>Case at bar</b></col>. <cd>See under <er>Bar</er>, <pos>n.</pos></cd> -- <col><b>Case divinity</b></col>, <cd>casuistry.</cd> -- <col><b>Case lawyer</b></col>, <cd>one versed in the reports of cases rather than in the science of the law.</cd> -- <mcol><col><b>Case stated</b></col> <it>or</it> <col><b>Case agreed on</b></col></mcol> <fld>(Law)</fld>, <cd>a statement in writing of facts agreed on and submitted to the court for a decision of the legal points arising on them.</cd> -- <col><b>A hard case</b></col>, <cd>an abandoned or incorrigible person.</cd> <mark>[Colloq.]</mark> -- <col><b>In any case</b></col>, <cd>whatever may be the state of affairs; anyhow.</cd> -- <mcol><col><b>In case</b></col>, <it>or</it> <col><b>In case that</b></col></mcol>, <cd>if; supposing that; in the event or contingency; if it should happen that.</cd>  <ldquo/<xex>In case</xex> we are surprised, keep by me.<rdquo/ <au>W. Irving.</au> -- <col><b>In good case</b></col>, <cd>in good condition, health, or state of body.</cd> -- <col><b>To put a case</b></col>, <cd>to suppose a hypothetical or illustrative case.</cd></cs></p>

<p><syn><b>Syn.</b> -- Situation, condition, state; circumstances; plight; predicament; occurrence; contingency; accident; event; conjuncture; cause; action; suit.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Case</ent><br/
<hw>Case</hw>, <pos>v. i.</pos> <def>To propose hypothetical cases.</def> <mark>[Obs.]</mark> <ldquo/<xex>Casing</xex> upon the matter.<rdquo/  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>caseate</ent><br/
<hw>caseate</hw> <pos>v. i.</pos> <sn>1.</sn> <def>to turn into cheese; -- of milk.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to undergo caseation; -- of dead tissue.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caseation</ent><br/
<hw>Ca`se*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cas<eacute/ation</ets>. See <er>Casein</er>.]</ety> <fld>(Med.)</fld> <def>A degeneration of animal tissue into a cheesy or curdy mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Case-bay</ent><br/
<hw>Case"-bay`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The space between two principals or girders</def>. <sd>(b)</sd> <def>One of the joists framed between a pair of girders in naked flooring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>casebook</ent><br/
<hw>case"book`</hw> <pos>n.</pos> <def>a book in which detailed written records of cases are kept and which are a source of information for subsequent work.  Such books are often used as supplements to texts in law schools.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>cased</ent><br/
<hw>cased</hw> <pos>adj.</pos> <sn>1.</sn> <def>covered or protected with or as if with a case; <as>as, knights <ex>cased</ex> in steel</as>.</def><br/
<syn><b>Syn. --</b> encased, incased.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>same as <er>bound</er>; -- of books.</def><br/
<syn><b>Syn. --</b> covered.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caseful</ent><br/
<hw>caseful</hw> <pos>n.</pos> <def>the quantity contained in a case.</def><br/
<syn><b>Syn. --</b> case.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caseharden</ent><br/
<hw>Case"hard`en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To subject to a process which converts the surface of iron into steel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To render insensible to good influences.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casehardened</ent><br/
<hw>Case"hard`ened</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having the surface hardened, as iron tools.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hardened against, or insusceptible to, good influences; rendered callous by persistence in wrongdoing or resistance of good influences; -- said of persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casehardening</ent><br/
<hw>Case"hard`en*ing</hw>, <pos>n.</pos> <def>The act or process of converting the surface of iron into steel.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Casehardening</xex> is now commonly effected by cementation with charcoal or other carbonizing material, the depth and degree of hardening (carbonization) depending on the time during which the iron is exposed to the heat. See <er>Cementation</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caseic</ent><br/
<hw>Ca"se*ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cas<eacute/ique</ets>, fr. L. <ets>caseus</ets> cheese.]</ety> <def>Of or pertaining to cheese; <as>as, <ex>caseic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casein</ent><br/
<hw>Ca"se*in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cas<eacute/ine</ets>, fr. L. <ets>caseur</ets> cheese. Cf. <er>Cheese</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A proteid substance present in both the animal and the vegetable kingdom. In the animal kingdom it is chiefly found in milk, and constitutes the main part of the curd separated by rennet; in the vegetable kingdom it is found more or less abundantly in the seeds of leguminous plants. Its reactions resemble those of alkali albumin.</def> <altsp>[Written also <asp>caseine</asp>.]</altsp></p>

<p><ent>Case knife</ent><br/
<-- no pos in original. = n. -->
<hw>Case" knife`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>A knife carried in a sheath or case.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large table knife; -- so called from being formerly kept in a <xex>case</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casemate</ent><br/
<hw>Case"mate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>casemate</ets>, fr. It. <ets>casamatta</ets>, prob. from <ets>casa</ets> house + <ets>matto</ets>, f. <ets>matta</ets>, mad, weak, feeble, dim. from the same source as E. <ets>-mate</ets> in <ets>checkmate</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Fort.)</fld> <def>A bombproof chamber, usually of masonry, in which cannon may be placed, to be fired through embrasures; or one capable of being used as a magazine, or for quartering troops.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>A hollow molding, chiefly in cornices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casemated</ent><br/
<hw>Case"ma`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with, protected by, or built like, a casemate.</def>  <rj><au>Campbell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casement</ent><br/
<hw>Case"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Shortened fr. <ets>encasement</ets>. See <er>Incase 1st Case</er>, and cf. <er>Incasement</er>.]</ety> <fld>(Arch.)</fld> <def>A window sash opening on hinges affixed to the upright side of the frame into which it is fitted. <mark>(Poetically)</mark> A window.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>casement</qex> of the great chamber window.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casemented</ent><br/
<hw>Case"ment*ed</hw>, <pos>a.</pos> <def>Having a casement or casements.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caseose</ent><br/
<hw>Ca"se*ose</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Case</ets>in + <ets>-ose</ets>.]</ety> <fld>(Physiol.Chem.)</fld> <def>A soluble product (proteose) formed in the gastric and pancreatic digestion of casein and caseinogen.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Caseous</ent><br/
<hw>Ca"se*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caseus</ets>. Cf. <er>Casein</er>.]</ety> <def>Of, pertaining to, or resembling, cheese; having the qualities of cheese; cheesy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caseous degeneration</b></col>, <cd>a morbid process, in scrofulous or consumptive persons, in which the products of inflammation are converted into a cheesy substance which is neither absorbed nor organized.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casern</ent><br/
<hw>Ca"sern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caserne</ets>.]</ety> <def>A lodging for soldiers in garrison towns, usually near the rampart; barracks.</def>  <rj><au>Bescherelle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Case shot</ent><br/
<hw>Case" shot`</hw> <pr>(?)</pr>. <fld>(Mil.)</fld> <def>A collection of small projectiles, inclosed in a case or canister.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States a <xex>case shot</xex> is a thin spherical or oblong cast-iron shell containing musket balls and a bursting charge, with a time fuse; -- called in Europe <altname>shrapnel</altname>. In Europe the term <xex>case shot</xex> is applied to what in the United States is called <xex>canister</xex>.</note>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Case system</ent><br/
<hw>Case system</hw>. <fld>(Law)</fld> <def>The system of teaching law in which the instruction is primarily a historical and inductive study of leading or selected cases, with or without the use of textbooks for reference and collateral reading.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Caseum</ent><br/
||<hw>Ca"se*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caseus</ets> cheese.]</ety> <def>Same as <er>Casein</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caseworm</ent><br/
<hw>Case"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A worm or grub that makes for itself a case. See <er>Caddice</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cash</ent><br/
<hw>Cash</hw> <pr>(k<acr/sh)</pr>, <pos>n.</pos> <ety>[F. <ets>caisse</ets> case, box, cash box, cash. See <er>Case</er> a box.]</ety> <def>A place where money is kept, or where it is deposited and paid out; a money box.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This bank is properly a general <qex>cash</qex>, where every man lodges his money.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><pound/20,000 are known to be in her <qex>cash</qex>.</q> <rj><qau>Sir R. Winwood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Com.)</fld> <sd>(a)</sd> <def>Ready money; especially, coin or specie; but also applied to bank notes, drafts, bonds, or any paper easily convertible into money</def>. <sd>(b)</sd> <def>Immediate or prompt payment in current funds; <as>as, to sell goods for <ex>cash</ex>; to make a reduction in price for <ex>cash</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cash account</b></col> <fld>(Bookkeeping)</fld>, <cd>an account of money received, disbursed, and on hand.</cd> -- <col><b>Cash boy</b></col>, <cd>in large retail stores, a messenger who carries the money received by the salesman from customers to a cashier, and returns the proper change.</cd> <mark>[Colloq.]</mark> -- <col><b>Cash credit</b></col>, <cd>an account with a bank by which a person or house, having given security for repayment, draws at pleasure upon the bank to the extent of an amount agreed upon; -- called also <altname>bank credit</altname> and <altname>cash account</altname>.</cd> -- <col><b>Cash sales</b></col>, <cd>sales made for ready, money, in distinction from those on which credit is given; stocks sold, to be delivered on the day of transaction.</cd></cs>
<-- cash on the nail.  A cash payment made immediately upon receiving the thing purchased. --></p>

<p><syn><b>Syn.</b> -- Money; coin; specie; currency; capital.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cash</ent><br/
<hw>Cash</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Casing</conjf>.]</vmorph> <def>To pay, or to receive, cash for; to exchange for money; <as>as, <ex>cash</ex> a note or an order</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cash</ent><br/
<hw>Cash</hw>, <pos>v. t.</pos> <ety>[See <er>Cashier</er>.]</ety> <def>To disband.</def> <mark>[Obs.]</mark>  <rj><au>Garges.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cash</ent><br/
<hw>Cash</hw>, <pos>n. sing. & pl.</pos> <def>A Chinese coin.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In 1913 the <xex>cash</xex> (<xex>Chinese tsien</xex>) was the only current coin made by the chinese government. It is a thin circular disk of a very base alloy of copper, with a square hole in the center. 1,000 to 1,400 <xex>cash</xex> were equivalent to a dollar.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>cashable</ent><br/
<hw>cashable</hw> <pos>adj.</pos> <def>able to be converted into ready money; <as>as, a <ex>cashable</ex> check; <ex>cashable</ex> gambling chips</as>.</def><br/
<syn><b>Syn. --</b> redeemable.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 223 --></p>

<p><ent>Cashbook</ent><br/
<hw>Cash"book</hw> <pr>(k<acr/sh"b<oocr/k)</pr>, <pos>n.</pos> <fld>(Bookkeeping)</fld> <def>A book in which is kept a register of money received or paid out.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cashbox</ent><br/
<hw>cashbox</hw> <pos>n.</pos> <def>a box for holding cash.</def><br/
<syn><b>Syn. --</b> money box, till.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cashed</ent><br/
<hw>cashed</hw> <pos>adj.</pos> <def>converted into currency; -- of financial instruments; <as>as, a <ex>cashed</ex> check</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cashew</ent><br/
<hw>Ca*shew"</hw> <pr>(k<adot/*sh<oomac/")</pr>, <pos>n.</pos> <ety>[F. <ets>acajou</ets>, for <ets>cajou</ets>, prob. from Malay <ets>k<amac/yu</ets> tree; cf. Pg. <ets>acaju</ets>, cf. <er>Acajou</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A tree (<spn>Anacardium occidentale</spn>) of the same family which the sumac. It is native in tropical America, but is now naturalized in all tropical countries. Its fruit, a kidney-shaped nut, grows at the extremity of an edible, pear-shaped hypocarp, about three inches long.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the <cref>cashew nut</cref>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Cashew nut</b></col>, <cd>the large, kidney-shaped fruit of the cashew, which is edible after the caustic oil has been expelled from the shell by roasting the nut.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cashier</ent><br/
<hw>Cash*ier"</hw> <pr>(k<acr/sh*<emac/r")</pr>, <pos>n.</pos> <ety>[F. <ets>caissier</ets>, fr. <ets>caisse</ets>. See <er>Cash</er>.]</ety> <def>One who has charge of money; a cash keeper; the officer who has charge of the payments and receipts (moneys, checks, notes), of a bank or a mercantile company.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cashier</ent><br/
<hw>Cash*ier"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cashiered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cashiering</conjf>.]</vmorph> <ety>[Earlier <ets>cash</ets>, fr. F. <ets>casser</ets> to break, annul, cashier, fr. L. <ets>cassare</ets>, equiv. to <ets>cassum reddere</ets>, to annul; cf. G. <ets>cassiren</ets>. Cf. <er>Quash</er> to annul, <er>Cass</er>.]</ety> <sn>1.</sn> <def>To dismiss or discard; to discharge; to dismiss with ignominy from military service or from an office or place of trust.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They have <qex>cashiered</qex> several of their followers.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had insolence to <qex>cashier</qex> the captain of the lord lieutenant's own body guard.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put away or reject; to disregard.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Connections formed for interest, and endeared</q><br/
[<source>1913 Webster</source>]</p>

<p><q>By selfish views, [are] censured and <qex>cashiered</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They absolutely <qex>cashier</qex> the literal express sense of the words.</q> <rj><qau>Sowth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cashierer</ent><br/
<hw>Cash*ier"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who rejects, discards, or dismisses; <as>as, a <ex>cashierer</ex> of monarchs</as>.</def> <mark>[R.]</mark>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cashier's check</ent><br/
<hw>Cash*ier's" check</hw> <pr>(?)</pr>. <fld>(Banking)</fld> <def>A check drawn by a bank upon its own funds, signed by the cashier.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cashmere</ent><br/
<hw>Cash"mere</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A rich stuff for shawls, scarfs, etc., originally made in Cashmere from the soft wool found beneath the hair of the goats of Cashmere, Tibet, and the Himalayas. Some cashmere, of fine quality, is richly embroidered for sale to Europeans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dress fabric made of fine wool, or of fine wool and cotton, in imitation of the original cashmere.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cashmere shawl</b></col>, <cd>a rich and costly shawl made of cashmere; -- often called <altname>camel's-hair shawl</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cashmerette</ent><br/
<hw>Cash`me*rette"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of dress goods, made with a soft and glossy surface like cashmere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cashoo</ent><br/
<hw>Ca*shoo"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cachou</ets>, NL. <er>catechu</er>, Cochin-Chin. <ets>cay cau</ets> from the tree called <ets>mimosa</ets>, or <ets>areca catechu</ets>. Cf. <er>Catechu</er>.]</ety> <def>See <er>Catechu</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cash railway</ent><br/
<hw>Cash railway</hw>. <def>A form of cash carrier in which a small carrier or car travels upon a kind of track.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cash register</ent><br/
<hw>Cash register</hw>. <def>A device for recording the amount of cash received, usually having an automatic adding machine and a money drawer and exhibiting the amount of the sale.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Casing</ent><br/
<hw>Cas"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of inclosing in, or covering with, a case or thin substance, as plaster, boards, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An outside covering, for protection or ornament, or to precent the radiation of heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An inclosing frame; esp. the framework around a door or a window. See <er>Case</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casings</ent><br/
<hw>Ca"sings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Dried dung of cattle used as fuel.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Waterland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casino</ent><br/
||<hw>Ca*si"no</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Casinos</plw> <pr>(#)</pr>, It. <plw>Casini</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>casino</ets>, dim. of <ets>casa</ets> house, fr. L. <ets>casa</ets> cottage. Cf. <er>Cassing</er>.]</ety> <sn>1.</sn> <def>A small country house.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A building or room used for meetings, or public amusements, for dancing, etc., sometimes having gambling tables or devices.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>A game at cards. See <er>Cassino</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>a business establishment for which the primary business is gambling, having various types of gambling tables, such as blackjack or roulette, and usually also slot machines, and sometimes other forms of gambling, such as poker; also, the main room of such an establishment where the gambling is conducted; <as>as, the Trump <ex>casinos</ex> in Atlantic City</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>casino-hotel</ent><br/
<hw>casino-hotel</hw> <pos>n.</pos> <def>a business establishment that combines a casino and a hotel.</def><br/
<syn><b>Syn. --</b> hotel-casino.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a building that houses both a hotel and a casino.</def><br/
<syn><b>Syn. --</b> hotel-casino.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cask</ent><br/
<hw>Cask</hw> <pr>(k<adot/sk)</pr>, <pos>n.</pos> <ety>[Sp. <ets>casco</ets> potsherd, skull, helmet, prob. fr. <ets>cascar</ets> to break, fr. L. <ets>Quassure</ets> to break. Cf. <er>Casque</er>, <er>Cass</er>.]</ety> <sn>1.</sn> <def>Same as <er>Casque</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A barrel-shaped vessel made of staves headings, and hoops, usually fitted together so as to hold liquids. It may be larger or smaller than a barrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The quantity contained in a cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A casket; a small box for jewels.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cask</ent><br/
<hw>Cask</hw>, <pos>v. t.</pos> <def>To put into a cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casket</ent><br/
<hw>Cas"ket</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>casquet</ets>, dim. of <ets>casque</ets> belmet, fr. Sp. <ets>casco</ets>.]</ety> <sn>1.</sn> <def>A small chest or box, esp. of rich material or ornamental character, as for jewels, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The little <qex>casket</qex> bring me hither.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of burial case.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything containing or intended to contain something highly esteemed</def>; as: <sd>(a)</sd> <def>The body.</def> (<au>Shak.</au>) <sd>(b)</sd> <def>The tomb.</def> (<au>Milton</au>). <sd>(c)</sd> <def>A book of selections.</def> <mark>[poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They found him dead . . . an empty <qex>casket</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casket</ent><br/
<hw>Cas"ket</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A gasket. See <er>Gasket</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casket</ent><br/
<hw>Cas"ket</hw>, <pos>v. t.</pos> <def>To put into, or preserve in, a casket.</def> <mark>[Poetic]</mark> <ldquo/I have <xex>casketed</xex> my treasure.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>caskful</ent><br/
<hw>caskful</hw> <pos>n.</pos> <def>the quantity a cask will hold.</def><br/
<syn><b>Syn. --</b> cask.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casmerodius</ent><br/
<hw>Casmerodius</hw> <pos>n.</pos> <def>a genus of white egrets.</def><br/
<syn><b>Syn. --</b> genus <gen>Casmerodius</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casque</ent><br/
<hw>Casque</hw> <pr>(k<adot/sk)</pr>, <pos>n.</pos> <ety>[F. <ets>casque</ets>, fr. Sp. <ets>casco</ets> See <er>Cask</er>.]</ety> <def>A piece of defensive or ornamental armor (with or without a vizor) for the head and neck; a helmet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>casque</qex> overshadowed with brilliant plumes.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cass</ent><br/
<hw>Cass</hw> <pr>(k<acr/s)</pr>, <pos>v. t.</pos> <ety>[F. <ets>casser</ets>, LL. <ets>cassare</ets>, fr. L. <ets>cassus</ets> empty, hollow, and perhaps influenced by L. <ets>quassare</ets> to shake, shatter, v. intens. of <ets>quatere</ets> to shake. Cf. <er>Cashier</er>, <pos>v. t.</pos>, <er>Quash</er>, <er>Cask</er>.]</ety> <def>To render useless or void; to quash; to annul; to reject; to send away.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassada</ent><br/
<hw>Cas"sa*da</hw> <pr>(k<acr/s"s<adot/*d<adot/; 277)</pr>, <pos>n.</pos> <def>See <er>Cassava</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassareep</ent><br/
<hw>Cas"sa*reep</hw> <pr>(k<acr/s"s<adot/*r<emac/p)</pr>, <pos>n.</pos> <def>A condiment made from the sap of the bitter cassava (<spn>Manihot utilissima</spn>) deprived of its poisonous qualities, concentrated by boiling, and flavored with aromatics. See <er>Pepper pot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassate</ent><br/
<hw>Cas"sate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[LL. <ets>cassare</ets>. See <er>Cass</er>.]</ety> <def>To render void or useless; to vacate or annul.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassation</ent><br/
<hw>Cas*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cassation</ets>. See <er>Cass</er>.]</ety> <def>The act of annulling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A general <qex>cassation</qex> of their constitutions.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Court of cassation</b></col>, <cd>the highest court of appeal in France, which has power to quash (<itran>Casser</itran>) or reverse the decisions of the inferior courts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassava</ent><br/
<hw>Cas"sa*va</hw> <pr>(k<acr/s"s<adot/*v<adot/)</pr>, <pos>n.</pos> <ety>[F. <ets>cassave</ets>, Sp. <ets>cazabe</ets>, fr. <ets>kasabi</ets>, in the language of Haiti.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A shrubby euphorbiaceous plant of the genus <gen>Manihot</gen>, with fleshy rootstocks yielding an edible starch; -- called also <altname>manioc</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are two species, <xex>bitter</xex> and <xex>sweet</xex>, from which the cassava of commerce is prepared in the West Indies, tropical America, and Africa. The bitter (<spn>Manihot utilissima</spn>) is the more important; this has a poisonous sap, but by grating, pressing, and baking the root the poisonous qualities are removed. The sweet (<spn>Manihot Aipi</spn>) is used as a table vegetable.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A nutritious starch obtained from the rootstocks of the cassava plant, used as food and in making tapioca.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassava wood</ent><br/
<hw>Cas"sa*va wood`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A West Indian tree (<spn>Turpinia occidentalis</spn>) of the family <fam>Staphyleace<ae/</fam>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cassel earth</ent><br/
<ent>Cassel brown</ent><br/
<mhw>{ <hw>Cas"sel brown</hw>, <hw>Cas"sel earth</hw>  }</mhw> <pr>(?)</pr>. <def>A brown pigment of varying permanence, consisting of impure lignite. It was found originally near <etsep>Cassel</etsep> (now Kassel), Germany.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Casse Paper</ent><br/
<hw>Cas"se Pa"per</hw> <pr>(?)</pr>. <ety>[F. <ets>papier cass<eacute/</ets>. See <er>Cass</er>.]</ety> <def>Broken paper; the outside quires of a ream.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casserole</ent><br/
<hw>Cas"se*role</hw> <pr>(#)</pr> <pos>n.</pos> <ety>[F. a saucepan, dim. from <ets>casse</ets> a basin.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A small round dish with a handle, usually of porcelain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cookery)</fld> <def>A mold (in the shape of a hollow vessel or incasement) of boiled rice, mashed potato or paste, baked, and afterwards filled with vegetables or meat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casse-tete</ent><br/
||<hw>Casse`-t<ecir/te"</hw> <pr>(k<adot/s`t<amac/t")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>casser</ets> to break (see 2d <er>Quash</er>) + <ets>t<ecir/te</ets> head.]</ety> <def>A small war club, esp. of savages; -- so called because of its supposed use in crushing the skull.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cassette</ent><br/
<ent>Cassette</ent><br/
<mhw><hw>Cas`sette"</hw> <hw>Cas`sette"</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prop., a casket, dim. of <ets>casse</ets> a case. See 1st <er>Case</er>.]</ety> <sn>1.</sn> <def>Same as <er>Seggar</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a small case to hold a removable part of some mechanism, designed for convenience in inserting and removing the working part.  Most commonly referring to a tape cassette, made mostly of plastic, which holds a reel of magnetic tape for tape players or tape recorders or video cassette recorders.  The term may refer, as for tape cassettes, to the integrated case plus tape, or sometimes only to the case itself separate from the contents.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Cassia</ent><br/
<hw>Cas"sia</hw> <pr>(k<acr/sh"<adot/)</pr>, <pos>n.</pos> <ety>[L. <ets>cassia</ets> and <ets>casia</ets>, Gr. <grk>kassi`a</grk> and <grk>kasi`a</grk>; of Semitic origin; cf. Heb. <ets>qets<imac/<amac/h</ets>, fr. <ets>q<amac/tsa'</ets> to cut off, to peel off.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of leguminous plants (herbs, shrubs, or trees) of many species, most of which have purgative qualities. The leaves of several species furnish the senna used in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The bark of several species of <prodby><gen>Cinnamomum</gen></prodby> grown in China, etc.; Chinese cinnamon.  It is imported as <altname>cassia</altname>, but commonly sold as cinnamon, from which it differs more or less in strength and flavor, and the amount of outer bark attached.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The medicinal <ldquo/cassia<rdquo/ (<xex>Cassia pulp</xex>) is the laxative pulp of the pods of a leguminous tree (<spn>Cassia fistula</spn> or <xex>Pudding-pipe tree</xex>), native in the East Indies but naturalized in various tropical countries.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cassia bark</b></col>, <cd>the bark of <prodby><spn>Cinnamomum cassia</spn></prodby>, etc. The coarser kinds are called <stype>Cassia lignea</stype>, and are often used to adulterate true cinnamon.</cd> -- <col><b>Cassia buds</b></col>, <cd>the dried flower buds of several species of cinnamon (<spn>Cinnamomum cassia</spn>, atc..).</cd> -- <col><b>Cassia oil</b></col>, <cd>oil extracted from cassia bark and cassia buds; -- called also <altname>oil of cinnamon</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassican</ent><br/
<hw>Cas"si*can</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>cassicus</ets> helmeted, fr. L. <ets>cassis</ets> a belmet.]</ety> <fld>(Zool.)</fld> <def>An American bird of the genus <gen>Cassicus</gen>, allied to the starlings and orioles, remarkable for its skillfully constructed and suspended nest; the crested oriole. The name is also sometimes given to the piping crow, an Australian bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassideous</ent><br/
<hw>Cas*sid"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Cassis</ets> helmet.]</ety> <fld>(Bot.)</fld> <def>Helmet-shaped; -- applied to a corolla having a broad, helmet-shaped upper petal, as in aconite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cassie</ent><br/
<hw>cassie</hw> <pos>n.</pos> <def>a type of tropical American thorny shrub or small tree (<spn>Acacia farnesiana</spn>); it bears fragrant yellow flowers used in making perfumery.</def><br/
<syn><b>Syn. --</b> huisache, mimosa bush, sweet wattle, sweet acacia, scented wattle, flame tree, Acacia farnesiana.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cassidony</ent><br/
<hw>Cas"si*do*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>cassidonium</ets>, F. <ets>cassidoine</ets>. See <er>Chalcedony</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The French lavender (<spn>Lavandula St<oe/chas</spn>)</def>. <sd>(b)</sd> <def>The goldilocks <spn>(Chrysocoma Linosyris)</spn> and perhaps other plants related to the genus <gen>Gnaphalium</gen> or cudweed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassimere</ent><br/
<hw>Cas"si*mere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>casimir</ets>, prob. of the same origin as E. <ets>cashmere</ets>. Cf. <er>Kerseymere</er>.]</ety> <def>A thin, twilled, woolen cloth, used for men's garments.</def> <altsp>[Written also <asp>kerseymere</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassinette</ent><br/
<hw>Cas`si*nette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>casinete</ets>, G. <ets>cassinet</ets>.]</ety> <def>A cloth with a cotton warp, and a woof of very fine wool, or wool and silk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassinian ovals</ent><br/
<hw>Cas*sin"i*an o"vals</hw> <pr>(?)</pr>. <fld>(Math.)</fld> <def>See under <er>Oval</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassino</ent><br/
<hw>Cas*si"no</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>casino</ets> a small house, a gaming house. See <er>casino</er>.]</ety> <def>A game at cards, played by two or more persons, usually for twenty-one points.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Great cassino</b></col>, <cd>the ten of diamonds.</cd> -- <col><b>Little cassino</b></col>, <cd>the two of spades.</cd></cs></p>

<p><-- 2. Casino: a gaming house, often containing slot machines, roulette tables, craps tables and/or card games. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassioberry</ent><br/
<hw>Cas"si*o*ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>cassine</ets>, from the language of the Florida Indians.]</ety> <def>The fruit of the <spn>Viburnum obovatum</spn>, a shrub which grows from Virginia to Florida.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassiopeia</ent><br/
<hw>Cas`si*o*pe"ia</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/.]</ety> <fld>(Astron.)</fld> <def>A constellation of the northern hemisphere, situated between Cepheus and Perseus; -- so called in honor of the wife of Cepheus, a fabulous king of Ethiopia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cassiopeia's Chair</b></col>, <cd>a group of six stars, in Cassiopeia, somewhat resembling a chair.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassiterite</ent><br/
<hw>Cas*sit"er*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ tin.]</ety> <fld>(Min.)</fld> <def>Native tin dioxide; tin stone; a mineral occurring in tetragonal crystals of reddish brown color, and brilliant adamantine luster; also massive, sometimes in compact forms with concentric fibrous structure resembling wood (<stype>wood tin</stype>), also in rolled fragments or pebbly (<stype>Stream tin</stype>). It is the chief source of metallic tin. See <er>Black tin</er>, under <er>Black</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassius</ent><br/
<hw>Cas"sius</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the name of the discoverer, A. <etsep>Cassius</etsep>, a German physician of the 17th centry.]</ety> <def>A brownish purple pigment, obtained by the action of some compounds of tin upon certain salts of gold. It is used in painting and staining porcelain and glass to give a beautiful purple color. Commonly called <altname>Purple of Cassius</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassock</ent><br/
<hw>Cas"sock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>casaque</ets>, fr. It. <ets>casacca</ets>, perh. fr. L. <ets>casa</ets> cottage, in It., house; or of Slavic origin.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A long outer garment formerly worn by men and women, as well as by soldiers as part of their uniform.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A garment resembling a long frock coat worn by the clergy of certain churches when officiating, and by others as the usually outer garment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassocked</ent><br/
<hw>Cas"socked</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Clothed with a cassock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassolette</ent><br/
||<hw>Cas`so*lette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>a box, or vase, with a perforated cover to emit perfumes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassonade</ent><br/
<hw>Cas`son*ade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>casson</ets>, for <ets>caisson</ets> a large chest. This sugar comes from Brazil in large chests.]</ety> <def>Raw sugar; sugar not refined.</def>  <rj><au>Mc Elrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassowary</ent><br/
<hw>Cas"so*wa*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cassowaries</plw> <pr>(#)</pr>.</plu> <ety>[Malay <ets>kasu<amac/ri</ets>.]</ety> <fld>(Zool.)</fld> <def>A large bird, of the genus <gen>Casuarius</gen>, found in the east Indies. It is smaller and stouter than the ostrich. Its head is armed with a kind of helmet of horny substance, consisting of plates overlapping each other, and it has a group of long sharp spines on each wing which are used as defensive organs. It is a shy bird, and runs with great rapidity. Other species inhabit New Guinea, Australia, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cassumuniar</ent><br/
<ent>Cassumunar</ent><br/
<mhw>{ <hw>Cas`su*mu"nar</hw> <pr>(?)</pr>, <hw>Cas`su*mu"ni*ar</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Hind.]</ety> <fld>(Med.)</fld> <def>A pungent, bitter, aromatic, gingerlike root, obtained from the East Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cast</ent><br/
<hw>Cast</hw> <pr>(k<adot/st)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cast</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Casting</conjf>.]</vmorph> <ety>[Cf. Dan. <ets>kaste</ets>, Icel. & Sw. <ets>kasta</ets>; perh. akin to L. <er>gerere</er> to bear, carry. E. <ets>jest</ets>.]</ety> <sn>1.</sn> <def>To send or drive by force; to throw; to fling; to hurl; to impel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Uzziah prepared . . . slings to <qex>cast</qex> stones.</q> <rj><qau>2 Chron. xxvi. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cast</qex> thy garment about thee, and follow me.</q> <rj><qau>Acts. xii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We must be <qex>cast</qex> upon a certain island.</q> <rj><qau>Acts. xxvii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To direct or turn, as the eyes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How earnestly he <qex>cast</qex> his eyes upon me!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To drop; to deposit; <as>as, to <ex>cast</ex> a ballot</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To throw down, as in wrestling.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To throw up, as a mound, or rampart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thine enemies shall <qex>cast</qex> a trench [bank] about thee.</q> <rj><qau>Luke xix. 48.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To throw off; to eject; to shed; to lose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His filth within being <qex>cast</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither shall your vine <qex>cast</qex> her fruit.</q> <rj><qau>Mal. iii. 11</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The creatures that <qex>cast</qex> the skin are the snake, the viper, etc.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To bring forth prematurely; to slink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy she-goats have not <qex>cast</qex> their young.</q> <rj><qau>Gen. xxi. 38.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To throw out or emit; to exhale.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This . . . <qex>casts</qex> a sulphureous smell.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To cause to fall; to shed; to reflect; to throw; <as>as, to <ex>cast</ex> a ray upon a screen; to <ex>cast</ex> light upon a subject</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To impose; to bestow; to rest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The government I <qex>cast</qex> upon my brother.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cast</qex> thy burden upon the Lord.</q> <rj><qau>Ps. iv. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To dismiss; to discard; to cashier.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The state can not with safety <qex>cast</qex> him.</q><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To compute; to reckon; to calculate; <as>as, to <ex>cast</ex> a horoscope</as>.</def> <ldquo/Let it be <xex>cast</xex> and paid.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>cast</qex> the event of war, my noble lord.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To contrive; to plan.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The cloister . . . had, I doubt not, been <qex>cast</qex> for [an orange-house].</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>To defeat in a lawsuit; to decide against; to convict; <as>as, to be <ex>cast</ex> in damages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She was <qex>cast</qex> to be hanged.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Were the case referred to any competent judge, they would inevitably be <qex>cast</qex>.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>To turn (the balance or scale); to overbalance; hence, to make preponderate; to decide; <as>as, a <ex>casting</ex> voice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How much interest <qex>casts</qex> the balance in cases dubious!</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <def>To form into a particular shape, by pouring liquid metal or other material into a mold; to fashion; to found; <as>as, to <ex>cast</ex> bells, stoves, bullets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <fld>(Print.)</fld> <def>To stereotype or electrotype.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>To fix, distribute, or allot, as the parts of a play among actors; also to assign (an actor) for a part.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our parts in the other world will be new <qex>cast</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cast anchor</b></col> <fld>(Naut.)</fld> <cd>See under <er>Anchor</er>.</cd> -- <col><b>To cast a horoscope</b></col>, <cd>to calculate it.</cd> -- <mcol><col><b>To cast a</b></col> <col><b>horse, sheep</b></col>, or other animal</mcol>, <cd>to throw with the feet upwards, in such a manner as to prevent its rising again.</cd> -- <col><b>To cast a shoe</b></col>, <cd>to throw off or lose a shoe, said of a horse or ox.</cd> -- <col><b>To cast aside</b></col>, <cd>to throw or push aside; to neglect; to reject as useless or inconvenient.</cd> -- <col><b>To cast away</b></col>. <sd>(a)</sd> <cd>To throw away; to lavish; to waste.</cd> <ldquo/<xex>Cast away</xex> a life<rdquo/ <au>Addison.</au> <sd>(b)</sd> <cd>To reject; to let perish.</cd>  <ldquo/<xex>Cast away</xex> his people.<rdquo/ <au>Rom. xi. 1.</au> <ldquo/<xex>Cast</xex> one <xex>away</xex>.<rdquo/ <au>Shak.</au> <sd>(c)</sd> <cd>To wreck.</cd> <ldquo/<xex>Cast away</xex> and sunk.<rdquo/ <au>Shak.</au> -- <col><b>To cast by</b></col>, <cd>to reject; to dismiss or discard; to throw away.</cd> -- <col><b>To cast down</b></col>, <cd>to throw down; to destroy; to deject or depress, as the mind.</cd>  <ldquo/Why art thou <xex>cast down</xex>. O my soul?<rdquo/ <au>Ps. xiii. 5.</au> -- <col><b>To cast forth</b></col>, <cd>to throw out, or eject, as from an inclosed place; to emit; to send out.</cd> -- <col><b>To cast in one's lot with</b></col>, <cd>to share the fortunes of.</cd> -- <col><b>To cast in one's teeth</b></col>, <cd>to upbraid or abuse one for; to twin.</cd> -- <col><b>To cast lots</b></col>. <cd>See under <er>Lot</er>.</cd> -- <col><b>To cast off</b></col>. <sd>(a)</sd> <cd>To discard or reject; to drive away; to put off; to free one's self from.</cd> <sd>(b)</sd> <fld>(Hunting)</fld> <cd>To leave behind, as dogs; also, to set loose, or free, as dogs.</cd> <au>Crabb.</au> <sd>(c)</sd> <fld>(Naut.)</fld> <cd>To untie, throw off, or let go, as a rope.</cd> -- <col><b>To cast off copy</b></col>, <fld>(Print.)</fld>, <cd>to estimate how much printed matter a given amount of copy will make, or how large the page must be in order that the copy may make a given number of pages.</cd> -- <mcol><col><b>To cast one's self on</b></col> <it>or</it> <col><b>To cast one's self upon</b></col></mcol> <cd>to yield or submit one's self unreservedly to, as to the mercy of another.</cd> -- <col><b>To cast out</b></col>, <cd>to throw out; to eject, as from a house; to cast forth; to expel; to utter.</cd> -- <col><b>To cast the lead</b></col> <fld>(Naut.)</fld>, <cd>to sound by dropping the lead to the bottom.</cd> -- <col><b>To cast the water</b></col> <fld>(Med.)</fld>, <cd>to examine the urine for signs of disease.</cd> <mark>[Obs.]</mark>. -- <col><b>To cast up</b></col>. <sd>(a)</sd> <cd>To throw up; to raise.</cd> <sd>(b)</sd> <cd>To compute; to reckon, as the cost.</cd> <sd>(c)</sd> <cd>To vomit.</cd> <sd>(d)</sd> <cd>To twit with; to throw in one's teeth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 224 --></p>

<p><ent>Cast</ent><br/
<hw>Cast</hw> <pr>(k<adot/st)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To throw, as a line in angling, esp, with a fly hook.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To turn the head of a vessel around from the wind in getting under weigh.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Weigh anchor, <qex>cast</qex> to starboard.</q> <rj><qau>Totten.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To consider; to turn or revolve in the mind; to plan; <as>as, to <ex>cast</ex> about for reasons</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She . . . <qex>cast</qex> in her mind what manner of salution this should be.</q> <rj><qau>Luke. i. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To calculate; to compute.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Who would <qex>cast</qex> and balance at a desk.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To receive form or shape in a mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It will not run thin, so as to <qex>cast</qex> and mold.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To warp; to become twisted out of shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Stuff is said to <qex>cast</qex> or warp when . . . it alters its flatness or straightness.</q> <rj><qau>Moxon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To vomit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These verses . . . make me ready to <qex>cast</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cast</ent><br/
<hw>Cast</hw>, <def><pos>3d pers. pres.</pos> of <er>Cast</er>, for <xex>Casteth</xex>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cast</ent><br/
<hw>Cast</hw>, <pos>n.</pos> <ety>[Cf. Icel., Dan., & Sw. <ets>kast</ets>.]</ety> <sn>1.</sn> <def>The act of casting or throwing; a throw.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The thing thrown.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cast</qex> of dreadful dust.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The distance to which a thing is or can be thrown.</def> <ldquo/About a stone's <xex>cast</xex>.<rdquo/  <rj><au>Luke xxii. 41.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A throw of dice; hence, a chance or venture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An even <qex>cast</qex> whether the army should march this way or that way.</q>  <rj><au>Sowth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have set my life upon a <qex>cast</qex>,<br/
And I will stand the hazard of the die.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which is throw out or off, shed, or ejected; <as>as, the skin of an insect, the refuse from a hawk's stomach, the excrement of a earthworm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The act of casting in a mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And why such daily <qex>cast</qex> of brazen cannon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An impression or mold, taken from a thing or person; amold; a pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>That which is formed in a mild; esp. a reproduction or copy, as of a work of art, in bronze or plaster, etc.; a casting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Form; appearence; mien; air; style; <as>as, a peculiar <ex>cast</ex> of countenance</as>.</def>  <ldquo/A neat <xex>cast</xex> of verse.<rdquo/ <au>Pope.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>An heroic poem, but in another <qex>cast</qex> and figure.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And thus the native hue of resolution<br/
Is sicklied o'er with the pale <qex>cast</qex> of thought.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A tendency to any color; a tinge; a shade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gray with a <qex>cast</qex> of green.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A chance, opportunity, privilege, or advantage; specifically, an opportunity of riding; a lift.</def> <mark>[Scotch]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We bargained with the driver to give us a <qex>cast</qex> to the next stage.</q> <rj><qau>Smollett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If we had the <qex>cast</qex> o' a cart to bring it.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>The assignment of parts in a play to the actors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Falconary)</fld> <def>A flight or a couple or set of hawks let go at one time from the hand.</def>  <rj><au>Grabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As when a <qex>cast</qex> of falcons make their flight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>A stoke, touch, or trick.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This was a <qex>cast</qex> of Wood's politics; for his information was wholly false.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>A motion or turn, as of the eye; direction; look; glance; squint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cast</qex> of the eye is a gesture of aversion.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And let you see with one <qex>cast</qex> of an eye.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This freakish, elvish <qex>cast</qex> came into the child's eye.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <def>A tube or funnel for conveying metal into a mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <def>Four; that is, as many as are thrown into a vessel at once in counting herrings, etc; a warp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>Contrivance; plot, design.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A cast of the eye</b></col>, <cd>a slight squint or strabismus.</cd> -- <col><b>Renal cast</b></col> <fld>(Med.)</fld>, <cd>microscopic bodies found in the urine of persons affected with disease of the kidneys; -- so called because they are formed of matter deposited in, and preserving the outline of, the renal tubes.</cd> -- <col><b>The last cast</b></col>, <cd>the last throw of the dice or last effort, on which every thing is ventured; the last chance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castalian</ent><br/
<hw>Cas*ta"li*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Castalius</ets>]</ety> <def>Of or pertaining to Castalia, a mythical fountain of inspiration on Mt. Parnassus sacred to the Muses.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castanea</ent><br/
||<hw>Cas*ta"ne*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a chestnut, fr. Gr. <?/.]</ety> <fld>(Bot.)</fld> <def>A genus of nut-bearing trees or shrubs including the chestnut and chinquapin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castanet</ent><br/
<hw>Cas"ta*net</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Castanets</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castanets</ent><br/
<hw>Cas"ta*nets</hw>, <pos>n. pl.</pos> <ety>[F. <ets>castagnettes</ets>, Sp. <ets>casta<ntil/etas</ets>, fr. L. <ets>castanea</ets> (Sp. <ets>casta<ntil/a</ets>) a chestnut. So named from the resemblance to two chestnuts, or because chestnuts were first used for castanets. See <er>Chestnut</er>.]</ety> <def>Two small, concave shells of ivory or hard wood, shaped like spoons, fastened to the thumb, and beaten together with the middle finger; -- used by the Spaniards and Moors as an accompaniment to their dance and guitars.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The singular, <xex>castanet</xex>, is used of <xex>one</xex> of the pair, or, sometimes, of the pair forming the instrument.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>The dancer, holding a <qex>castanet</qex> in each hand, rattles them to the motion of his feet.</q> <rj><qau>Moore (Encyc. of Music).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castanopsis</ent><br/
<hw>Castanopsis</hw> <pos>n.</pos> <def>a genus of evergreen trees and shrubs of warm regions valued for their foliage; found in southeastern U.S. and eastern Australia and northern New Zealand.</def><br/
<syn><b>Syn. --</b> genus <gen>Castanopsis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Castanospermum</ent><br/
<hw>Castanospermum</hw> <pos>n.</pos> <sn>1.</sn> <def>a genus consisting of one species, the Moreton Bay chestnut.</def><br/
<syn><b>Syn. --</b> genus <gen>Castanospermum</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Castaway</ent><br/
<hw>Cast"a*way</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, is cast away or shipwrecked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is ruined; one who has made moral shipwreck; a reprobate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest . . . <qex>when I have preached to others</qex>, <qex>I myself should be a castaway</qex>.</q> <rj><qau>1 Cor. ix. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castaway</ent><br/
<hw>Cast"a*way</hw>, <pos>a.</pos> <def>Of no value; rejected; useless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caste</ent><br/
<hw>Caste</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg. <ets>casta</ets> race, lineage, fr. L. <ets>castus</ets> pure, chaste: cf. F. <ets>caste</ets>, of same origin.]</ety> <sn>1.</sn> <def>One of the hereditary classes into which the Hindu are divided according to the laws of Brahmanism.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The members of the same caste are theoretically of equal rank, and same profession or occupation, and may not eat or intermarry with those not of their own caste. The original are four, viz., the <xex>Brahmans</xex>, or sacerdotal order; the <xex>Kshatriyas</xex>, or soldiers and rulers; the <xex>Vaisyas</xex>, or husbandmen and merchants; and the <xex>Sudras</xex>, or laborers and mechanics.  Men of no caste are <xex>Pariahs</xex>, outcasts.  Numerous mixed classes, or <xex>castes</xex>, have sprung up in the progress of time.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A separate and fixed order or class of persons in society who chiefly hold intercourse among themselves.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tinkers then formed an hereditary <qex>caste</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To lose caste</b></col>, <cd>to be degraded from the caste to which one has belonged; to lose social position or consideration.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castellan</ent><br/
<hw>Cas"tel*lan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>castelain</ets>, F. <ets>ch<acir/telain</ets>, L. <ets>castellanus</ets> pertaining to a castle, an occupant of a caste, LL., a governor of a castle, fr. L. <ets>castellum</ets> castle, citadel, dim. of <ets>castrum</ets> fortified place. See <er>Castle</er>, and cf. <er>Chatelaine</er>.]</ety> <def>A governor or warden of a castle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castellany</ent><br/
<hw>Cas"tel*la*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Castellanies</plw> <pr>(#)</pr>.</plu> <ety>[LL. <ets>castellania</ets>.]</ety> <def>The lordship of a castle; the extent of land and jurisdiction appertaining to a castle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castellated</ent><br/
<hw>Cas"tel*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>castellatus</ets>, fr. <ets>castellare</ets>. See <er>Castle</er>.]</ety> <sn>1.</sn> <def>Inclosed within a building; <as>as, a fountain or cistern <ex>castellated</ex></as>.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Furnished with turrets and battlements, like a castle; built in the style of a castle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castellation</ent><br/
<hw>Cas`tel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>castellation</ets>, fr. <ets>castellare</ets>, fr. L. <ets>castellum</ets>. See <er>Castle</er>.]</ety> <def>The act of making into a castle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caster</ent><br/
<hw>Cast"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who casts; <as>as, <ex>caster</ex> of stones, etc.</as> ; a <xex>caster</xex> of cannon; a <xex>caster</xex> of accounts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vial, cruet, or other small vessel, used to contain condiments at the table; <as>as, a set of <ex>casters</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stand to hold a set of cruets.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small wheel on a swivel, on which furniture is supported and moved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castigate</ent><br/
<hw>Cas"ti*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Castigated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Castigating</conjf>.]</vmorph> <ety>[L. <ets>castigatus</ets>, p. p. of <ets>castigare</ets> to correct, punish; <ets>castus</ets> pure, chaste + <ets>agere</ets> to move, drive. See <er>Caste</er>, and cf. <er>Chasten</er>.]</ety> <sn>1.</sn> <def>To punish by stripes; to chastise by blows; to chasten; also, to chastise verbally; to reprove; to criticise severely.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To emend; to correct.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castigation</ent><br/
<hw>Cas`ti*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>catigatio</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Corrective punishment; chastisement; reproof; pungent criticism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The keenest <qex>castigation</qex> of her slanderers.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Emendation; correction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castigator</ent><br/
<hw>Cas`ti*ga"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who castigates or corrects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castigatory</ent><br/
<hw>Cas`ti*ga*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>castigatorius</ets>.]</ety> <def>Punitive in order to amendment; corrective.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castigatory</ent><br/
<hw>Cas"ti*ga*to*ry</hw>, <pos>n.</pos> <def>An instrument formerly used to punish and correct arrant scolds; -- called also a <altname>ducking stool</altname>, or <altname>trebucket</altname>.</def>  <rj><au>Blacktone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castile soap</ent><br/
<hw>Cas"tile soap`</hw> <pr>(?)</pr>. <ety>[From <etsep>Castile</etsep>, or <etsep>Castilia</etsep>, a province in Spain, from which it originally came.]</ety> <def>A kind of fine, hard, white or mottled soap, made with olive oil and soda; also, a soap made in imitation of the above-described soap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castilian</ent><br/
<hw>Cas*til"ian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>castellano</ets>, from <ets>Castila</ets>, NL. <ets>Castilia</ets>, <ets>Castella</ets>. Castile, which received its name from the castles erected on the frontiers as a barrier against the Moors.]</ety> <sn>1.</sn> <def>An inhabitant or native of Castile, in Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The Spanish language as spoken in Castile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castillan</ent><br/
<hw>Cas*til"lan</hw>, <pos>a.</pos> <def>Of or pertaining to Castile, in Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casting</ent><br/
<hw>Cast"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who casts or throws, as in fishing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of making casts or impressions, or of shaping metal or plaster in a mold; the act or the process of pouring molten metal into a mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is cast in a mold; esp. the mass of metal so cast; <as>as, a <ex>casting</ex> in iron; bronze <ex>casting</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The warping of a board.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The act of casting off, or that which is cast off, as skin, feathers, excrement, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Casting of draperies</b></col>, <cd>the proper distribution of the folds of garments, in painting and sculpture.</cd> -- <col><b>Casting line</b></col> <fld>(Fishing)</fld>, <cd>the leader; also, sometimes applied to the long reel line.</cd> -- <col><b>Casting net</b></col>, <cd>a net which is cast and drawn, in distinction from a net that is set and left.</cd> -- <mcol><col><b>Casting voice</b></col>, <col><b>Casting vote</b></col></mcol>, <cd>the decisive vote of a presiding officer, when the votes of the assembly or house are equally divided.</cd> <ldquo/When there was an equal vote, the governor had the <xex>casting voice</xex>.<rdquo/ <au>B. Trumbull.</au><-- = tie-breaking vote --> -- <col><b>Casting weight</b></col>, <cd>a weight that turns a balance when exactly poised.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cast iron</ent><br/
<hw>Cast" i`ron</hw> <pr>(?)</pr>. <def>Highly carbonized iron, the direct product of the blast furnace; -- used for making castings, and for conversion into wrought iron and steel. It can not be welded or forged, is brittle, and sometimes very hard. Besides carbon, it contains sulphur, phosphorus, silica, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cast-iron</ent><br/
<hw>Cast"-i`ron</hw>, <pos>a.</pos> <def>Made of cast iron. Hence, Fig.: like cast iron; hardy; unyielding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castle</ent><br/
<hw>Cas"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>castel</ets>, fr. L. <ets>castellum</ets>, dim. of <ets>castrum</ets> a fortified place, castle.]</ety> <sn>1.</sn> <def>A fortified residence, especially that of a prince or nobleman; a fortress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The house of every one is to him <qex>castle</qex> and fortress, as well for his defense againts injury and violence, as for his repose.</q> <rj><qau>Coke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>castle's</qex> strength<br/
Will laugh a siege to scorn.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Originally the medi<ae/val <xex>castle</xex> was a single strong tower or keep, with a palisaded inclosure around it and inferior buidings, such as stables and the like, and surrounded by a moat; then such a keep or donjon, with courtyards or baileys and accessory buildings of greater elaboration a great hall and a chapel, all surrounded by defensive walls and a moat, with a drawbridge, etc. Afterwards the name was retained by large dwellings that had formerly been fortresses, or by those which replaced ancient fortresses.</note></p>

<p><-- Illustration of "Castle at Pierrefonds, France": --></p>

<p><caption><xex>A</xex> Donjon or Keep, an irregular building containing the dwelling of the lord and his family; <xex>B C</xex> Large round towers ferming part of the donjon and of the exterior; <xex>D</xex> Square tower, separating the two inner courts and forming part of the donjon; <xex>E</xex> Chapel, whose apse forms a half-round tower, <xex>F</xex>, on the exterior walls; <xex>G H</xex> Round towers on the exterior walls; <xex>K</xex> Postern gate, reached from outside by a removable fight of steps or inclined plane for hoisting in stores, and leading to a court, <xex>L</xex> (see small digagram) whose pavement is on a level with the sill of the postern, but below the level of the larger court, with which it communicates by a separately fortified gateway; <xex>M</xex> Turret, containing spiral stairway to all the stories of the great tower, <xex>B</xex>, and serving also as a station for signal fire, banner, etc.; <xex>N</xex> Turret with stairway for tower, <xex>C</xex>; <xex>O</xex> Echauguettes; <xex>P P P</xex> Battlemants consisting of merlons and crenels alternately, the merlons being pierced by loopholes; <xex>Q Q</xex> Machicolations (those at <xex>Q</xex> defend the postern <xex>K</xex>); <xex>R</xex> Outwork defending the approach, which is a road ascending the hill and passing under all four faces of the castle; <xex>S S</xex> Wall of the outer bailey. The road of approach enters the bailey at <xex>T</xex> and passes thence into the castle by the main entrance gateway (which is in the wall between, and defended by the towers, <xex>C H</xex>) and over two drawbridges and through fortified passages to the inner court.</caption></p>

<p><-- end of illustration caption. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any strong, imposing, and stately mansion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small tower, as on a ship, or an elephant's back.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A piece, made to represent a castle, used in the game of chess; a rook.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Castle in the air</b></col>, <cd>a visionary project; a baseless scheme; an air castle; -- sometimes called a <altname>castle in Spain</altname> (F. <xex>Ch<acir/teau en Espagne</xex>).</cd></cs></p>

<p><syn><b>Syn.</b> -- Fortress; fortification; citadel; stronghold. See <er>Fortress</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castle</ent><br/
<hw>Cas"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Castled</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Castling</conjf> <pr>(?)</pr>.]</vmorph> <fld>(Chess)</fld> <def>To move the castle to the square next to king, and then the king around the castle to the square next beyond it, for the purpose of covering the king.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castlebuilder</ent><br/
<hw>Cas"tle*build`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Fig.: one who builds castles in the air or forms visionary schemes.</def></p>

<p>-- <wordforms><wf>Cas"tle*build`ing</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castled</ent><br/
<hw>Cas"tled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a castle or castles; supporting a castle; <as>as, a <ex>castled</ex> height or crag</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fortified; turreted; <as>as, <ex>castled</ex> walls</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castle-guard</ent><br/
<hw>Cas"tle-guard`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The guard or defense of a castle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O. Eng. Law)</fld> <def>A tax or imposition an a dwelling within a certain distance of a castle, for the purpose of maintaining watch and ward in it; castle-ward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A feudal tenure, obliging the tenant to perform service within the realm, without limitation of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castlery</ent><br/
<hw>Cas"tle*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>castelerie</ets>. See <er>Castle</er>.]</ety> <def>The government of a castle.</def>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castlet</ent><br/
<hw>Cas"tlet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small castle.</def>  <rj><au>Leland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castleward</ent><br/
<hw>Cas"tle*ward`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Castleguard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castling</ent><br/
<hw>Cast"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which is cast or brought forth prematurely; an abortion.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castling</ent><br/
<hw>Cas"tling</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chess)</fld> <def>A compound move of the king and castle. See <er>Castle</er>, <pos>v. i.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cast-off</ent><br/
<hw>Cast"-off`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cast or laid aside; thrown away; discarded; <as>as, <ex>cast-off</ex> clothes</as>.</def><br/
<syn><b>Syn. --</b> discarded, junked, scrap(prenominal), waste.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castor</ent><br/
<hw>Cas"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>castor</ets> the beaver, Gr. <?/; of uncertain origin.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of rodents, including the beaver. See <er>Beaver</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Castoreum. See <er>Castoreum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A hat, esp. one made of beaver fur; a beaver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have always been known for the jaunty manner in which I wear my <qex>castor</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A heavy quality of broadcloth for overcoats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castor</ent><br/
<hw>Cast"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caster</er>, a small wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castor</ent><br/
<hw>Cas"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Astron.)</fld> <def>the northernmost of the two bright stars in the constellation Gemini, the other being Pollux.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castorite</ent><br/
<ent>Castor</ent><br/
<mhw>{ <hw>Cas"tor</hw>, <hw>Cas"tor*ite</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[The minerals <ets>castor</ets> and <ets>pollux</ets> were so named because found <ets>together</ets> on the island of Elba. See <er>Castor and Pollux</er>.]</ety> <fld>(Min.)</fld> <def>A variety of the mineral called petalite, from Elba.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castor and Pollux</ent><br/
<hw>Cas"tor and Pol"lux</hw> <pr>(?)</pr>. <ety>[<ets>Castor</ets> and <ets>Pollux</ets> were twin sons of Jupiter and Leda.]</ety> <fld>(Naut.)</fld> <def>See <cref>Saint Elmo's fire</cref>, under <er>Saint</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castor bean</ent><br/
<hw>Cas"tor bean`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The bean or seed of the castor-oil plant (<spn>Ricinus communis</spn>, or <spn>Palma Christi</spn>.)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castoreum</ent><br/
<hw>Cas*to"re*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Castor</er>.]</ety> <def>A peculiar bitter orange-brown substance, with strong, penetrating odor, found in two sacs between the anus and external genitals of the beaver; castor; -- used in medicine as an antispasmodic, and by perfumers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castoridae</ent><br/
<hw>Castoridae</hw> <pos>n.</pos> <def>a natural family comprising the beavers.</def><br/
<syn><b>Syn. --</b> family <fam>Castoridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Castoroides</ent><br/
<hw>Castoroides</hw> <pos>n.</pos> <def>a genus of extinct beavers of the Pleistocene; of eastern and southern U.S.</def><br/
<syn><b>Syn. --</b> genus <gen>Castoroides</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 225 --></p>

<p><ent>Castorin</ent><br/
<hw>Cas"to*rin</hw> <pr>(k<acr/s"t<osl/*r<icr/n)</pr>, <pos>n.</pos> <ety>[From 1st <er>Castor</er>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline substance obtained from castoreum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castor oil</ent><br/
<hw>Cas"tor oil</hw> <pr>(k<acr/s"t<etil/r oil`)</pr>. <def>A mild cathartic oil, expressed or extracted from the seeds of the <spn>Ricinus communis</spn>, or <spn>Palma Christi</spn>. When fresh the oil is inodorous and insipid.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Castor-oil plant</b></col>. <cd>Same as <er>Palma Christi</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castrametation</ent><br/
<hw>Cas`tra*me*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>castram<eacute/tation</ets>, fr. L. <ets>castra</ets> camp + <ets>metari</ets> to measure off, fr. <ets>meta</ets> limit.]</ety> <fld>(Mil.)</fld> <def>The art or act of encamping; the making or laying out of a camp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castrate</ent><br/
<hw>Cas"trate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Castrated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Castrating</conjf>.]</vmorph> <ety>[L. <ets>castrarus</ets>, p; p. of <ets>castrare</ets> to castrate, asin to Skr. <ets><cced/astra</ets> knife.]</ety> <sn>1.</sn> <def>To deprive of the testicles; to emasculate; to geld; to alter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut or take out; esp. to remove anything erroneous, or objectionable from, as the obscene parts of a writing; to expurgate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My . . . correspondent . . . has sent me the following letter, which I have <qex>castrated</qex> in some places.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>castrated</ent><br/
<hw>castrated</hw> <pos>adj.</pos> <def>deprived of reproductive organs or sexual attributes.</def><br/
<syn><b>Syn. --</b> altered, neutered, unsexed, fixed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Castration</ent><br/
<hw>Cas*tra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>castratio</ets>; cf. F. <ets>castration</ets>.]</ety> <def>The act of castrating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castrato</ent><br/
||<hw>Cas*tra"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., properly p. p. of <ets>castrare</ets>. See <er>Castrate</er>.]</ety> <def>A male person castrated for the purpose of improving his voice for singing; an artificial, or male, soprano.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castrel</ent><br/
<hw>Cas"trel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cr<eacute/cerelle</ets>, <ets>cristel</ets>, OF. <ets>crecel</ets>, <ets>cercele</ets>. Cf. <er>Kestrel</er>.]</ety> <fld>(Zool.)</fld> <def>See <er>Kestrel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castrensial</ent><br/
<hw>Cas*tren"sial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>castrensis</ets>, fr. <ets>castra</ets> camp.]</ety> <def>Belonging to a camp.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castrensian</ent><br/
<hw>Cas*tren"sian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Castrensial.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Castries</ent><br/
<hw>Castries</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>St. Lucia</country>.  Population (2000) = 13,600.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cast steel</ent><br/
<hw>Cast" steel`</hw> <pr>(?)</pr>. <def>See <cref>Cast steel</cref>, under <er>Steel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casual</ent><br/
<hw>Cas"u*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>casuel</ets>, F. <ets>casuel</ets>, fr. L. <ets>casualis</ets>, fr. <ets>casus</ets> fall, accident, fr. <ets>cadere</ets> to fall. See <er>Case</er>.]</ety> <sn>1.</sn> <def>Happening or coming to pass without design, and without being foreseen or expected; accidental; fortuitous; coming by chance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Casual</qex> breaks, in the general system.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Coming without regularity; occasional; incidental; <as>as, <ex>casual</ex> expenses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A constant habit, rather than a <qex>casual</qex> gesture.</q> <rj><qau>Hawthorne.</qau></rj></p>

<p><syn><b>Syn.</b> -- Accidental; fortutious; incidental; occasional; contingent; unforeseen. See <er>Accidental</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casual</ent><br/
<hw>Cas"u*al</hw>, <pos>n.</pos> <def>One who receives relief for a night in a parish to which he does not belong; a vagrant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casualism</ent><br/
<hw>Cas"u*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine that all things exist or are controlled by chance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casualist</ent><br/
<hw>Cas"u*al*ist</hw>, <pos>n.</pos> <def>One who believes in casualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casually</ent><br/
<hw>Cas"u*al*ly</hw>, <pos>adv.</pos> <def>Without design; accidentally; fortuitously; by chance; occasionally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casualness</ent><br/
<hw>Cas"u*al*ness</hw>, <pos>n.</pos> <def>The quality of being casual.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casualty</ent><br/
<hw>Cas"u*al*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Casualties</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>casualit<eacute/</ets>, LL. <ets>casualitas</ets>.]</ety> <sn>1.</sn> <def>That which comes without design or without being foreseen; contingency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Losses that befall them by mere <qex>casualty</qex>.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any injury of the body from accident; hence, death, or other misfortune, occasioned by an accident; <as>as, an unhappy <ex>casualty</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Mil. & Naval)</fld> <def>Numerical loss caused by death, wounds, discharge, or desertion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Casualty ward</b></col>, <cd>A ward in a hospital devoted to the treatment of injuries received by accident.</cd></cs></p>

<p><syn><b>Syn.</b> -- Accident; contingency; fortuity; misfortune.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casuaridae</ent><br/
<hw>Casuaridae</hw> <pos>n.</pos> <def>a natural family of large ostrichlike birds; the cassowaries.</def><br/
<syn><b>Syn. --</b> family <fam>Casuaridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casuariiformes</ent><br/
<hw>Casuariiformes</hw> <pos>n.</pos> <def>an order of ratite birds comprising the cassowaries and emus.</def><br/
<syn><b>Syn. --</b> order <ord>Casuariiformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casuarina</ent><br/
||<hw>Cas`u*a*ri"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., supposed to be named from the resemblance of the twigs to the feathers of the cassowary, of the genus <gen><etsep>Casuarius</etsep></gen>.]</ety> <fld>(Bot.)</fld> <def>A genus of leafless trees or shrubs, with drooping branchlets of a rushlike appearance, mostly natives of Australia. Some of them are large, producing hard and heavy timber of excellent quality, called <prod>beefwood</prod> from its color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casuarinaceae</ent><br/
<hw>Casuarinaceae</hw> <pos>n.</pos> <def>a natural family of plants having only one genus, <gen>Casuarina</gen>.</def><br/
<syn><b>Syn. --</b> family <fam>Casuarinaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casuarinales</ent><br/
<hw>Casuarinales</hw> <pos>n.</pos> <def>an order of chiefly Australian trees and shrubs comprising the casuarinas, having only one family, <fam>Casuarinaceae</fam>.</def><br/
<syn><b>Syn. --</b> order <ord>Casuarinales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casuarius</ent><br/
<hw>Casuarius</hw> <pos>n.</pos> <def>the type and sole genus of the <fam>Casuaridae</fam>; the cassowaries.</def><br/
<syn><b>Syn. --</b> genus <gen>Casuarius</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Casuist</ent><br/
<hw>Cas"u*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>casus</ets> fall, case; cf. F. <ets>casuiste</ets>. See <er>Casual</er>.]</ety> <def>One who is skilled in, or given to, casuistry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The judment of any <qex>casuist</qex> or learned divine concerning the state of a man's soul, is not sufficient to give him confidence.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casuist</ent><br/
<hw>Cas"u*ist</hw>, <pos>v. i.</pos> <def>To play the casuist.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casuistical</ent><br/
<ent>Casuistic</ent><br/
<mhw>{ <hw>Cas`u*is"tic</hw> <pr>(?)</pr>, <hw>Cas`u*is"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to casuists or casuistry; <as>as, <ex>casuistic</ex> thinking</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casuistry</ent><br/
<hw>Cas"u*ist*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>The science or doctrine of dealing with cases of conscience, of resolving questions of right or wrong in conduct, or determining the lawfulness or unlawfulness of what a man may do by rules and principles drawn from the Scriptures, from the laws of society or the church, or from equity and natural reason; the application of general moral rules to particular cases.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The consideration of these nice and puzzling question in the science of ethics has given rise, in modern times, to a particular department of it, distinguished by the title of <qex>casuistry</qex>.</q> <rj><qau>Stewart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Casuistry</qex> in the science of cases (i.e., oblique deflections from the general rule).</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sophistical, equivocal, or false reasoning or teaching in regard to duties, obligations, and morals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Casus</ent><br/
||<hw>Ca"sus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>An event; an occurrence; an occasion; a combination of circumstances; a case; an act of God. See the Note under <er>Accident</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Casus belli</b></col>, <cd>an event or combination of events which is a cause war, or may be alleged as a justification of war.</cd> -- <col><b>Casus fortuitus</b></col>, <cd>an accident against which due prudence could not have provided. See <cref>Act of God</cref>, under <er>Act</er>.</cd> -- <col><b>Casus omissus</b></col>, <cd>a case not provided for by the statute.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cat</ent><br/
<hw>cat</hw> <pr>(k<acr/t)</pr>, <pos>n.</pos> <ety>[AS. <ets>cat</ets>; akin to D. & Dan. <ets>kat</ets>, Sw. <ets>katt</ets>, Icel. <ets>k<oum/ttr</ets>, G. <ets>katze</ets>, <ets>kater</ets>, Ir. <ets>cat</ets>, W. <ets>cath</ets>, Armor. <ets>kaz</ets>, LL. <ets>catus</ets>, Bisc. <ets>catua</ets>, NGr. <grk>ga`ta</grk>, <grk>ga`tos</grk>, Russ. & Pol. <ets>kot</ets>, Turk. <ets>kedi</ets>, Ar. <ets>qitt</ets>; of unknown origin. Cf. <er>Kitten</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any animal belonging to the natural family <fam>Felidae</fam>, and in particular to the various species of the genera <gen>Felis</gen>, <gen>Panthera</gen>, and <gen>Lynx</gen>.  The domestic cat is <spn>Felis domestica</spn>.  The European wild cat (<spn>Felis catus</spn>) is much larger than the domestic cat.  In the United States the name <stype>wild cat</stype> is commonly applied to the bay lynx (<spn>Lynx rufus</spn>).  The larger felines, such as the lion, tiger, leopard, and cougar, are often referred to as <ex>cats</ex>, and sometimes as <it>big cats</it>.  See <er>Wild cat</er>, and <er>Tiger cat</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ The domestic cat includes many varieties named from their place of origin or from some peculiarity; as, the <stype>Angora cat</stype>; the <stype>Maltese cat</stype>; the <stype>Manx cat</stype>; the <stype>Siamese cat</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>  Laying aside their often rancorous debate over how best to preserve the <stype>Florida panther</stype>, state and federal wildlife officials, environmentalists, and independent scientists endorsed the proposal, and in 1995 the eight <ex>cats</ex> [female Texas cougars] were brought from Texas and released. . . .<br/
   Uprooted from the arid hills of West Texas, three of the imports have died, but the remaining five adapted to swamp life and have each given birth to at least one litter of kittens.</q> <rj><qau>Mark Derr (N. Y. Times, Nov. 2, 1999, Science Times p. F2).</qau></rj><br/
[<source>PJC</source>]</p>

<p><note>The word <ex>cat</ex> is also used to designate other animals, from some fancied resemblance; as, civet <ex>cat</ex>, fisher <ex>cat</ex>, <ex>cat</ex>bird, <ex>cat</ex>fish shark, sea <ex>cat</ex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A strong vessel with a narrow stern, projecting quarters, and deep waist.  It is employed in the coal and timber trade.</def> <sd>(b)</sd> <def>A strong tackle used to draw an anchor up to the cathead of a ship.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A double tripod (for holding a plate, etc.), having six feet, of which three rest on the ground, in whatever position it is placed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An old game;</def> <specif>specifically:</specif> <sd>(a)</sd> <def>The game of tipcat and the implement with which it is played. See <er>Tipcat</er>.</def> <sd>(b)</sd> <def>A game of ball, called, according to the number of batters, <xex>one old cat</xex>, <xex>two old cat</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>same as <er>cat o' nine tails</er>; <as>as, British sailors feared the <ex>cat</ex></as>.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><sn>6.</sn> <def>A <er>catamaran</er>.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Angora cat</b></col>, <col><b>blind cat</b></col></mcol>, <cd>See under <er>Angora</er>, <er>Blind</er>.</cd> -- <col><b>Black cat</b></col> <cd>the fisher. See under <er>Black</er>.</cd> -- <col><b>Cat and dog</b></col>, <cd>like a cat and dog; quarrelsome; inharmonious.</cd>  <ldquo/I am sure we have lived a <xex>cat and dog</xex> life of it.<rdquo/ <au>Coleridge.</au> -- <col><b>Cat block</b></col> <fld>(Naut.)</fld>, <cd>a heavy iron-strapped block with a large hook, part of the tackle used in drawing an anchor up to the cathead.</cd> -- <col><b>Cat hook</b></col> <fld>(Naut.)</fld>, <cd>a strong hook attached to a cat block.</cd> -- <col><b>Cat nap</b></col>, <cd>a very short sleep.</cd> <mark>[Colloq.]</mark> -- <col><b>Cat o' nine tails</b></col>, <cd>an instrument of punishment consisting of nine pieces of knotted line or cord fastened to a handle; -- formerly used to flog offenders on the bare back.</cd> -- <col><b>Cat's cradle</b></col>, <cd>game played, esp. by children, with a string looped on the fingers so, as to resemble small cradle.  The string is transferred from the fingers of one to those of another, at each transfer with a change of form. See <er>Cratch</er>, <er>Cratch cradle</er>.</cd> -- <col><b>To bell the cat</b></col>, <cd>to perform a very dangerous or very difficult task; -- taken metaphorically from a fable about a mouse who proposes to put a bell on a cat, so as to be able to hear the cat coming.</cd>  -- <col><b>To let the cat out of the bag</b></col>, <cd>to tell a secret, carelessly or willfully.</cd> <mark>[Colloq.]</mark> -- <col><b>Bush cat</b></col>, <cd>the serval. See <er>Serval</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat</ent><br/
<hw>Cat</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Catted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Catting</conjf>.]</vmorph> <fld>(Naut.)</fld> <def>To bring to the cathead; <as>as, to <ex>cat</ex> an anchor</as>. See <er>Anchor</er>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cata</ent><br/
<hw>Cat"a</hw> <pr>(?)</pr>. <ety>[Gr. <grk>kata`</grk>.]</ety> <def>The Latin and English form of a Greek preposition, used as a prefix to signify <sig>down</sig>, <sig>downward</sig>, <sig>under</sig>, <sig>against</sig>, <sig>contrary</sig> or <sig>opposed to</sig>, <sig>wholly</sig>, <sig>completely</sig>; as in <xex>cata</xex>clysm, <xex>cata</xex>rrh. It sometimes drops the final vowel, as in <xex>cato</xex>ptric; and is sometimes changed to <xex>cath</xex>, as in <xex>cath</xex>artic, <xex>cath</xex>olic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catabaptist</ent><br/
<hw>Cat`a*bap"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>cata + aptist</ets>. See <er>Baptist</er>.]</ety> <fld>(Eccl.)</fld> <def>One who opposes baptism, especially of infants.</def> <mark>[Obs.]</mark>  <rj><au>Featley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catabasion</ent><br/
||<hw>Cat`a*ba"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kataba`sion</grk>.]</ety> <def>A vault under altar of a Greek church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catabiotic</ent><br/
<hw>Cat`a*bi*ot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See under <er>Force</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catabolic</ent><br/
<hw>catabolic</hw> <pos>adj.</pos> <altsp>[Formerly spelled <asp>katabolic</asp>.]</altsp> <sn>1.</sn> <def>Of or pertaining to catabolism; <as>as, <ex>catabolic</ex> processes, which give rise to substances (katastates) of decreasing complexity and increasing stability</as>.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>characterized by destructive metabolism; contrasted with <ant>anabolic</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>catabolism</ent><br/
<hw>catabolism</hw> <pos>n.</pos> <def>the breakdown of more complex substances into simpler ones with release of energy, in living organisms; destructive or downward metabolism; -- a form of <isa>metabolism</isa>, opposed to <contr>anabolism</contr>. See also <er>Disassimilation</er>.</def>  <altsp>[Formerly spelled <asp>katabolism</asp>.]</altsp><br/
<syn><b>Syn. --</b> katabolism, dissimilation, destructive metabolism.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catacala</ent><br/
<hw>Catacala</hw> <pos>n.</pos> <def>a genus of moths whose larvae are cutworms: underwings.</def><br/
<syn><b>Syn. --</b> genus <gen>Catacala</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catacaustic</ent><br/
<hw>Cat`a*caus"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>cata + caustic</ets>.]</ety> <fld>(Physics)</fld> <def>Relating to, or having the properties of, a caustic curve formed by reflection. See <er>Caustic</er>, <pos>a.</pos></def>  <rj><au>Nichol.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catacaustic</ent><br/
<hw>Cat`a*caus"tic</hw>, <pos>n.</pos> <fld>(Physics)</fld> <def>A caustic curve formed by reflection of light.</def>  <rj><au>Nichol.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catachresis</ent><br/
<hw>Cat`a*chre"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. fr. Gr. <?/ misuse, fr. <?/ to misuse; <grk>kata`</grk> against + <?/ to use.]</ety> <fld>(Rhet.)</fld> <def>A figure by which one word is wrongly put for another, or by which a word is wrested from its true signification; as, <ldquo/To take arms against a sea of troubles<rdquo/. <au>Shak.</au> <ldquo/Her voice was but the shadow of a sound.<rdquo/ <au>Young.</au></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catachrestical</ent><br/
<ent>Catachrestic</ent><br/
<mhw>{ <hw>Cat`a*chres"tic</hw> <pr>(?)</pr>, <hw>Cat`a*chres"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Belonging to, or in the manner of, a catachresis; constituting,  characterized by or given to catachresis; wrested from its natural sense or form; forced; far-fetched.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Cat`a*chres"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>[A] <qex>catachrestical</qex> and improper way of speaking.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataclasm</ent><br/
<hw>Cat"a*clasm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ down + <?/ to break.]</ety> <def>A breaking asunder; disruption.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cataclinal</ent><br/
<hw>cataclinal</hw> <pos>adj.</pos> <fld>(Geol)</fld> <def>running in the direction of the dip in surrounding rock strata; -- of valleys and rivers.  Opposite of <ant>anaclinal</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cataclysm</ent><br/
<hw>Cat"a*clysm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cataclysmos</ets>, Gr. <grk>kataklysmo`s</grk>, from <?/ to dash over, inundate; <grk>kata`</grk> downward, against + <?/ to wash or dash over: cf. F. <ets>cataclysme</ets>.]</ety> <sn>1.</sn> <def>An extensive overflow or sweeping flood of water; a deluge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>Any violent catastrophe, involving sudden and extensive changes of the earth's surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataclysmic</ent><br/
<ent>Cataclysmal</ent><br/
<mhw>{ <hw>Cat`a*clys"mal</hw> <pr>(?)</pr>, <hw>Cat`a*clys"mic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a cataclysm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataclysmist</ent><br/
<hw>Cat`a*clys"mist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who believes that the most important geological phenomena have been produced by cataclysms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catacomb</ent><br/
<hw>Cat"a*comb</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>catacomba</ets>, fr. L. <ets>catacumba</ets> perh. from Gr. <grk>kata`</grk> downward, down + <grk>ky`mbh</grk> cavity.]</ety> <def>A cave, grotto, or subterraneous place of large extent used for the burial of the dead; -- commonly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The terms is supposed to have been applied originally to the tombs under the church of St. Sebastian in Rome. The most celebrated catacombs are those near Rome, on the Appian Way, supposed to have been the place or refuge and interment of the early Christians; those of Egypt, extending for a wide distance in the vicinity of Cairo; and those of Paris, in abandoned stone quarries, excavated under a large portion of the city.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>cata-cornered</ent><br/
<hw>cata-cornered</hw> <pos>adj.</pos>  <def>having a slanted or oblique direction.</def><br/
<syn><b>Syn. --</b> bias, catacorner, catercorner, cater-cornered, catty-corner, catty-cornered, diagonal, kitty-corner, kitty-cornered, oblique, skew, skewed, slanted.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catacoustic</ent><br/
<hw>Cat`a*cous"tic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>cata</ets> + <ets>acoustics</ets>: cf. F. <ets>caraconstique</ets>.]</ety> <fld>(Physics)</fld> <def>That part of acoustics which treats of reflected sounds or echoes See <er>Acoustics</er>.</def>  <rj><au>Hutton.</au></rj></p>

<p><ent>Catacrotic</ent><br/
<hw>Cat`a*crot"ic</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Cata-</ets> + Gr. <?/ a beating.]</ety> <fld>(Physiol.)</fld> <def>Designating, pertaining to, or characterized by, that form of pulse tracing, or sphygmogram, in which the descending portion of the curve is marked by secondary elevations due to two or more expansions of the artery in the same beat.</def> -- <wordforms><wf>Ca*tac"rotism</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Catadicrotism</ent><br/
<hw>Cat`a*di"cro*tism</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Cata-</ets> + <ets>dicrotism</ets>.]</ety> <fld>(Physiol.)</fld> <def>Quality or state of being catacrotic.</def> -- <wordforms><wf>Cat`a*di*crot"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Catadioptrical</ent><br/
<ent>Catadioptric</ent><br/
<mhw>{ <hw>Cat`a*di*op"tric</hw> <pr>(?)</pr>, <hw>Cat`a*di*op"tric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>cata</ets> + <ets>dioptric</ets>: cf. F. <ets>catadioptrique</ets>.]</ety> <fld>(Physics)</fld> <def>Pertaining to, produced by, or involving, both the reflection and refraction of light; <as>as, a <ex>catadioptric</ex> light</as>.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catadioptrics</ent><br/
<hw>Cat`a*di*op"trics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science which treats of catadioptric phenomena, or of the used of catadioptric instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catadrome</ent><br/
<hw>Cat"a*drome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kata`dromos</grk> race course; <grk>kata`</grk> down + <grk>dro`mos</grk> course.]</ety> <sn>1.</sn> <def>A race course.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>A machine for raising or lowering heavy weights.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catadromous</ent><br/
<hw>Ca*tad"ro*mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kata`</grk> down + <grk>dro`mos</grk> a running.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Having the lowest inferior segment of a pinna nearer the rachis than the lowest superior one; -- said of a mode of branching in ferns, and opposed to <contr>anadromous</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Living in fresh water, and going to the sea to spawn; -- opposed to <contr>anadromous</contr>, and said of the eel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catafalco</ent><br/
||<hw>Cat`a*fal"co</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>See <er>Catafalque</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catafalque</ent><br/
<hw>Cat"a*falque`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. It. <ets>catafalco</ets>, scaffold, funeral canopy; of uncertain origin; cf. Sp. <ets>catafalso</ets>, <ets>cadahalso</ets>, <ets>cadalso</ets>, Pr. <ets>casafalc</ets>, OF. <ets>chafaut</ets>. Cf. <er>Scaffold</er>.]</ety> <def>A temporary structure sometimes used in the funeral solemnities of eminent persons, for the public exhibition of the remains, or their conveyance to the place of burial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catagmatic</ent><br/
<hw>Cat`*ag*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ fracture, fr. <?/ to break in places; <grk>kata`</grk> down + <grk>'agny`nai</grk> to break: cf. F. <ets>catagmatique</ets>.]</ety> <fld>(Med.)</fld> <def>Having the quality of consolidating broken bones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataian</ent><br/
<hw>Ca*ta"ian</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A native of Cathay or China; a foreigner; -- formerly a term of reproach.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalan</ent><br/
<hw>Cat"a*lan</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Catalonia.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Catalonia; also, the language of Catalonia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Catalan furnace</b></col>, <col><b>Catalan forge</b></col></mcol> <fld>(Metal.)</fld>, <cd>a kind of furnace for producing wrought iron directly from the ore. It was formerly much used, esp. in Catalonia, and is still used in some parts of the United States and elsewhere.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>catalase</ent><br/
<hw>catalase</hw> <pos>n.</pos> <def>an enzyme found in most plant and animal cells that functions as an oxidative catalyst; it decomposes hydrogen peroxide into hydrogen and water.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>catalatic</ent><br/
<hw>catalatic</hw> <pos>adj.</pos> <def>of, pertaining to, or characteristic of catalases.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catalectic</ent><br/
<hw>Cat`a*lec"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>catalecticus</ets>, Gr. <?/ incomplete, fr. <?/ to leave off; <grk>kata`</grk> down, wholly + <grk>lh`gein</grk> to stop.]</ety> <sn>1.</sn> <fld>(Pros.)</fld> <def>Wanting a syllable at the end, or terminating in an imperfect foot; <as>as, a <ex>catalectic</ex> verse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Photog. & Chem.)</fld> <def>Incomplete; partial; not affecting the whole of a substance.</def>  <rj><au>Abney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalepsis</ent><br/
<ent>Catalepsy</ent><br/
<mhw>{ <hw>Cat"a*lep`sy</hw> <pr>(?)</pr>, ||<hw>Cat`a*lep"sis</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>catalepsis</ets>, fr. Gr. <?/ a seizure, fr. <?/ to seize upon; <grk>kata`</grk> down + <?/ to take, seize.]</ety> <fld>(Med.)</fld> <def>A sudden suspension of sensation and volition, the body and limbs preserving the position that may be given them, while the action of the heart and lungs continues.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataleptic</ent><br/
<hw>Cat`a*lep"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>katalhptiko`s</grk>.]</ety> <def>Pertaining to, or resembling, catalepsy; affected with catalepsy; <as>as, a <ex>cataleptic</ex> fit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catallacta</ent><br/
||<hw>Cat`al*lac"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/. See <er>Catallactics</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Protozoa, of which <gen>Magosph<ae/ra</gen> is the type. They exist both in a myxopod state, with branched pseudopodia, and in the form of ciliated bodies united in free, spherical colonies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catallactics</ent><br/
<hw>Cat`al*lac"tics</hw> <pr>(?)</pr> <pos>n.</pos> <ety>[Gr. <?/ to exchange; <grk>kata`</grk> wholly + <?/ to change.]</ety> <def>The science of exchanges, a branch of political economy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalog</ent><br/
<hw>Cat"a*log</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>Catalogue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalogize</ent><br/
<hw>Cat"a*lo*gize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To insert in a catalogue; to register; to catalogue.</def> <mark>[R.]</mark>  <rj><au>Coles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalogue</ent><br/
<hw>Cat"a*logue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>catalogus</ets>, fr. Gr. <?/ a counting up, list, fr. <?/ to count up; <grk>kata`</grk> down, completely + <?/ to say.]</ety> <def>A list or enumeration of names, or articles arranged methodically, often in alphabetical order; <as>as, a <ex>catalogue</ex> of the students of a college, or of books, or of the stars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Card catalogue</b></col>, <cd>a catalogue, as of books, having each item entered on a separate card, and the cards arranged in cases by subjects, or authors, or alphabetically.</cd> -- <col><b>Catalogue raisonn<eacute/</b></col> <pr>(?)</pr> <ety>[F.]</ety>, <cd>a catalogue of books, etc., classed according to their subjects.</cd></cs></p>

<p><syn><b>Syn.</b> -- List; roll; index; schedule; enumeration; inventory. See <er>List</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalogue</ent><br/
<hw>Cat"a*logue</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Catalogued</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cataloguing</conjf> <pr>(?)</pr>.]</vmorph> <def>To make a list or catalogue; to insert in a catalogue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataloguer</ent><br/
<hw>Cat"a*log`uer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A maker of catalogues; esp. one skilled in the making of catalogues.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalonia</ent><br/
<hw>Catalonia</hw> <pos>n.</pos> <def>a geographical and administrative region of northeastern Spain.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catalpa</ent><br/
<hw>Ca*tal"pa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the language of the Indians of Carolina, where Catesby discovered this tree in the year 1726.]</ety> <fld>(Bot.)</fld> <def>A genus of American and East Indian trees, of which the best known species are the <spn>Catalpa bignonioides</spn>, a large, ornamental North American tree, with spotted white flowers and long cylindrical pods, and the <spn>Catalpa speciosa</spn>, of the Mississipi valley; -- called also <altname>Indian bean</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catalufa</ent><br/
<hw>catalufa</hw> <pos>n.</pos> <def>a type of brightly colored carnivorous fish (<spn>Priacanthus arenatus</spn>) of the West Atlantic and West Indies waters.</def><br/
<syn><b>Syn. --</b> Priacanthus arenatus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catalyse</ent><br/
<hw>Cat"a*lyse`</hw> <pr>(k<acr/t"<adot/*l<imac/z)</pr>, <pos>v. t.</pos> <ety>[ML., fr. Gr. <?/ dissolution, fr. <?/ to destroy, dissolve; <grk>kata`</grk> down, wholly + <?/ to loose.]</ety> <def>same as <er>catalyze</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Catalysis</ent><br/
<hw>Ca*tal"y*sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Catalyse</plw>.</plu> <pr>(#)</pr> <ety>[ML., fr. Gr. <?/ dissolution, fr. <?/ to destroy, dissolve; <grk>kata`</grk> down, wholly + <?/ to loose.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Dissolution; degeneration; decay.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Sad <qex>catalysis</qex> and declension of piety.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A process by which a chemical reaction is accelerated in the presence of certain agents which were formerly believed to exert an influence by mere contact. It is now believed that such reactions are attended with the formation of an intermediate compound or compounds, so that by alternate composition and decomposition the agent is apparenty left unchanged; <as>as, the <ex>catalysis</ex> of making ether from alcohol by means of sulphuric acid; or <ex>catalysis</ex> in the action of enzymes (as diastase, or ptyalin) on starch</as>.</def> <sd>(b)</sd> <def>The catalytic force.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>catalyst</ent><br/
<hw>catalyst</hw> <pos>n.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>a substance that initiates or accelerates a chemical reaction without itself being affected; <as>as, thousands of enzymes serve in concert as <ex>calaysts</ex> to produce the sequence of reactions we call <ldquo/life<rdquo/; the industrial production of cheap ammonia depended on finding a good <ex>catalyst</ex></as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>something that serves as a precipitating occasion for an event; <as>as, the invasion acted as a <ex>catalyst</ex> to unite the country</as>.</def> <note>A <ex>catalyst</ex> is never the main cause of an event, but may serve to hasten events for which the underlying causes are present prior to the appearance or occurrence of the <ex>catalyst</ex>.</note><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>something or someone that causes events to happen with itself being changed.</def><br/
[<source>PJC</source>]</p>

<p><ent>Catalytic</ent><br/
<hw>Cat`a*ly"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to, or causing, catalysis.</def> <ldquo/The <xex>catalytic</xex> power is ill understood.<rdquo/  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Catalytic force</b></col>, <cd>that form of chemical energy formerly supposed to determine catalysis.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catalytic</ent><br/
<hw>Cat`a*lyt"ic</hw>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An agent employed in catalysis, as platinum black, aluminium chloride, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catalyze</ent><br/
<hw>catalyze</hw> <pos>v. t.</pos> <sn>1.</sn> <fr>1</fr> <fld>(chemistry)</fld> <def>to cause (a chemical reaction) to proceed more quickly by contacting the reactant(s) with another substance; <as>as, moisture <ex>catalyzes</ex> the oxidation of iron in air</as>.</def> <altsp>[Also spelled <asp>catalyse</asp>.]</altsp><br/
<syn><b>Syn. --</b> change by catalysis, cause to catalyze.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catamaran</ent><br/
<hw>Cat`a*ma*ran"</hw>, <pos>n.</pos> <ety>[The native East Indian name.]</ety> <sn>1.</sn> <def>A kind of raft or float, consisting of two or more logs or pieces of wood lashed together, and moved by paddles or sail; -- used as a surf boat and for other purposes on the coasts of the East and West Indies and South America. Modified forms are much used in the lumber regions of North America, and at life-saving stations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any vessel with twin hulls, whether propelled by sails or by steam; esp., one of a class of double-hulled pleasure boats remarkable for speed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of fire raft or torpedo bat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The incendiary rafts prepared by Sir Sidney Smith for destroying the French flotilla at Boulogne, 1804, were called <qex>catamarans</qex>.</q> <rj><qau>Knight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A quarrelsome woman; a scold.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 226 --></p>

<p><ent>Catamenia</ent><br/
||<hw>Cat`a*me"nia</hw> <pr>(k<acr/t`<adot/*m<emac/"n<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ta` katamh`nia</grk>.]</ety> <fld>(Med.)</fld> <def>The monthly courses of women; menstrual discharges; menses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catamenial</ent><br/
<hw>Cat`a*me"ni*al</hw> <pr>(k<acr/t`<adot/*m<emac/"n<icr/*<ait/l)</pr>, <pos>a.</pos> <ety>[Gr. <grk>katamh`nios</grk>  monthly; <grk>kata`</grk> down, back, again + <grk>mh`n</grk> month.]</ety> <def>Pertaining to the catamenia, or menstrual discharges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catamite</ent><br/
<hw>Cat"a*mite</hw> <pr>(k<acr/t"<adot/*m<imac/t)</pr>, <pos>n.</pos> <ety>[L. <ets>Catamitus</ets>, an old form of <ets>Ganymedes</ets> Ganymede, Gr. <grk>Ganymh`dhs</grk>.]</ety> <def>A boy kept for unnatural purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catamount</ent><br/
<hw>Cat"a*mount</hw> <pr>(k<acr/t"<adot/*mount)</pr>, <pos>n.</pos> <ety>[<ets>Cat</ets> + <ets>mount</ets>; cf. Sp. <ets>gato montes</ets> mountain cat.]</ety> <fld>(Zool.)</fld> <def>The <er>cougar</er>. Applied also, in some parts of the United States, to the <er>lynx</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cat-a-mountain</ent><br/
<ent>catamountain</ent><br/
<mhw><hw>cat"a*moun`tain</hw>, <hw>cat"-a-moun`tain</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>the bushy-tailed European wildcat (<spn>Felis silvestris</spn>) resembling the domestic tabby and regarded as the ancestor of the domestic cat.</def><br/
<syn><b>Syn. --</b> European wildcat, <spn>Felis silvestris</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catanadromous</ent><br/
<hw>Cat"a*nad`ro*mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kata`</grk> down + <grk>'ana`</grk> up + <grk>dro`mos</grk> running, course.]</ety> <fld>(Zool.)</fld> <def>Ascending and descending fresh streams from and to the sea, as the salmon; anadromous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catapasm</ent><br/
<hw>Cat"a*pasm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to besprinkle; <grk>kata`</grk> down, wholly + <?/ to strew, or sprinkle.]</ety> <fld>(Med.)</fld> <def>A compound medicinal powder, used by the ancients to sprinkle on ulcers, to absorb perspiration, etc.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catapeltic</ent><br/
<hw>Cat`a*pel"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a catapult.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catapetalous</ent><br/
<hw>Cat`a*pet"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>cata</ets> + <ets>petalous</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the petals held together by stamens, which grow to their bases, as in the mallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataphonic</ent><br/
<hw>Cat`a*phon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or relating to cataphonics; catacoustic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataphonics</ent><br/
<hw>Cat`a*phon"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>cata</ets> + <ets>phonic</ets>: cf. F. <ets>cataphonique</ets>.]</ety> <fld>(Physics)</fld> <def>That branch of acoustics which treats of reflected sounds; catacoustics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataphract</ent><br/
<hw>Cat"a*phract</hw> <pr>(k<acr/t"<adot/*fr<acr/kt)</pr>, <pos>n.</pos> <ety>[L. <ets>cataphractes</ets>, Gr. <?/, fr. <?/ covered, fr. <?/ to cover; <grk>kata`</grk> down, wholly + <grk>fra`ssein</grk> to inclose.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Mil. Antiq.)</fld> <def>Defensive armor used for the whole body and often for the horse, also, esp. the linked mail or scale armor of some eastern nations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A horseman covered with a cataphract.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Archers and slingers, <qex>cataphracts</qex>, and spears.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The armor or plate covering some fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataphracted</ent><br/
<hw>Cat"a*phract`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Covered with a cataphract, or armor of plates, scales, etc.; or with that which corresponds to this, as horny or bony plates, hard, callous skin, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataphractic</ent><br/
<hw>Cat`a*phrac"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a cataphract.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataphysical</ent><br/
<hw>Cat`a*phys"ic*al</hw>, <pos>a.</pos> <ety>[Pref. <ets>cata</ets> + <ets>physical</ets>.]</ety> <def>Unnatural; contrary to nature.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some artists . . . have given to Sir Walter Scott a pile of forehead which is unpleassing and <qex>cataphysical</qex>.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataplasm</ent><br/
<hw>Cat"a*plasm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cataplasma</ets>, Gr. <?/, fr. <?/ to spread over; <grk>kata`</grk> down, wholly + <?/ to form, mold.]</ety> <fld>(Med.)</fld> <def>A soft and moist substance applied externally to some part of the body; a poultice.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataplexy</ent><br/
<hw>Cat"a*plex`y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ amazement: cf. <er>Apoplexy</er>.]</ety> <fld>(Med.)</fld> <def>A morbid condition caused by an overwhelming shock or extreme fear and marked by rigidity of the muscles.</def> -- <wordforms><wf>Cat`a*plec"tic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Catapuce</ent><br/
<hw>Cat"a*puce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Bot.)</fld> <def>Spurge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catapult</ent><br/
<hw>Cat"a*pult</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>catapulta</ets>, Gr. <?/, prob. from <grk>kata`</grk> down + <?/ to shake, hurl.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Mil. Antiq.)</fld> <def>An engine somewhat resembling a massive crossbow, used by the ancient Greeks and Romans for throwing stones, arrows, spears, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A forked stick with elastic band for throwing small stones, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataract</ent><br/
<hw>Cat"a*ract</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cataracta</ets>, <ets>catarracles</ets>, a waterfall, Gr. <?/, <?/, fr. <?/ to break down; in the passive, to fall or rush down (of tumors) to burst; <grk>kata`</grk> down + <?/ to break.]</ety> <sn>1.</sn> <def>A great fall of water over a precipice; a large waterfall.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>An opacity of the crystalline lens, or of its capsule, which prevents the passage of the rays of light and impairs or destroys the sight.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>A kind of hydraulic brake for regulating the action of pumping engines and other machines; -- sometimes called <altname>dashpot</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cataractous</ent><br/
<hw>Cat`a*rac"tous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of a cataract in the eye; affected with cataract.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catarrh</ent><br/
<hw>Ca*tarrh"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>catarrhus</ets>, Gr. <?/, <?/, a running down, rheum, fr. <?/; <grk>kata`</grk> down + <?/ to flow. See <er>Stream</er>.]</ety> <fld>(Med.)</fld> <def>An inflammatory affection of any mucous membrane, in which there are congestion, swelling, and an altertion in the quantity and quality of mucus secreted; <as>as, <ex>catarrh</ex> of the stomach; <ex>catarrh</ex> of the bladder</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In America, the term <ex>catarrh</ex> is applied especially to a chronic inflammation of, and hypersecretion fron, the membranes of the nose or air passages; in England, to an acute influenza, resulting a cold, and attended with cough, thirst, lassitude, and watery eyes; also, to the cold itself.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catarrhal</ent><br/
<hw>Ca*tarrh"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, produced by, or attending, catarrh; of the nature of catarrh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catarrhine</ent><br/
<hw>Cat"ar*rhine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kata`rris</grk> with hanging or curved nose; <grk>kata`</grk> down + <grk>"ri`s</grk>, <grk>"rino`s</grk> nose.]</ety> <fld>(Zool.)</fld> <def>One of the <xex>Catarrhina</xex>, a division of Quadrumana, including the Old World monkeys and apes which have the nostrils close together and turned downward. See <er>Monkey</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catarrhous</ent><br/
<hw>Ca*tarrh"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Catarrhal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catastaltic</ent><br/
<hw>Cat`a*stal"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ to check; <grk>kata`</grk> down, wholy + <?/ to set.]</ety> <fld>(Med.)</fld> <def>Checking evacuations through astringent or styptic qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catastasis</ent><br/
||<hw>Ca*tas"ta*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to set; <grk>kata`</grk> down + <?/ to place.]</ety> <sn>1.</sn> <fld>(Rhet.)</fld> <def>That part of a speech, usually the exordium, in which the orator sets forth the subject matter to be discussed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The state, or condition of anything; constitution; habit of body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catasterism</ent><br/
<hw>Ca*tas"ter*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to place among the stars.]</ety> <def>A placing among the stars; a catalogue of stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>catasterisms</qex> of Eratosthenes.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catastrophe</ent><br/
<hw>Ca*tas"tro*phe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>catastropha</ets>, Gr. <?/, fr. <?/ to turn up and down, to overturn; <grk>kata`</grk> down + <?/ to turn.]</ety> <sn>1.</sn> <def>An event producing a subversion of the order or system of things; a final event, usually of a calamitous or disastrous nature; hence, sudden calamity; great misfortune.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The strange <qex>catastrophe</qex> of affairs now at London.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The most horrible and portentous <qex>catastrophe</qex> that nature ever yet saw.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The final event in a romance or a dramatic piece; a denouement, as a death in a tragedy, or a marriage in a comedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>A violent and widely extended change in the surface of the earth, <as>as, an elevation or subsidence of some part of it, effected by internal causes</as>.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catastrophic</ent><br/
<hw>Cat`a*stroph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of a pertaining to a catastrophe.</def>  <rj><au>B. Powell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catastrophism</ent><br/
<hw>Ca*tas"tro*phism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>The doctrine that the geological changes in the earth's crust have been caused by the sudden action of violent physical causes; -- opposed to the doctrine of <contr>uniformism</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catastrophist</ent><br/
<hw>Ca*tas"tro*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>One who holds the theory or catastrophism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catatonia</ent><br/
<hw>cat`a*ton"ia</hw> <pos>n.</pos> <ety>[Gr. <grk>katatonos</grk>, stretching down, depressed, fr. <grk>kata`</grk> down + <grk>to`nos</grk> stretching, straining, tone (sound). <au>Stedman.</au>]</ety> <fld>(Psychiatry)</fld> <def>an abnormal behavioral syndrome characterized by stupor, negativism, and muscular rigidity, sometimes alternating with purposeless excitement, and seen most frequently in schizophrenia; called also <altname>catatonic schizophrenia</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>catatonic</ent><br/
<hw>cat`a*ton"ic</hw> <pos>adj.</pos> <sn>1.</sn> <def>of or pertaining to catatonia; suffering from catatonia.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>appearing mentally stupefied, unresponsive, and motionless, or almost so; seemingly unaware of one's environment.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>catatonic</ent><br/
<hw>catatonic</hw> <pos>n.</pos> <def>a person suffering from catatonia.</def><br/
[<source>PJC</source>]</p>

<p><ent>Catawba</ent><br/
<hw>Ca*taw"ba</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A well known light red variety of American grape.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A light-colored, sprightly American wine from the Catawba grape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catawbas</ent><br/
<hw>Ca*taw"bas</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Catawba</singw>.</sing> <fld>(Ethnol.)</fld> <def>An Appalachian tribe of Indians which originally inhabited the regions near the Catawba river and the head waters of the Santee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catbird</ent><br/
<hw>Cat"bird</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An American bird (<spn>Galeoscoptes Carolinensis</spn>), allied to the mocking bird, and like it capable of imitating the notes of other birds, but less perfectly. Its note resembles at times the mewing of a cat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catboat</ent><br/
<hw>Cat"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A small sailboat, with a single mast placed as far forward as possible, carring a sail extended by a gaff and long boom. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catbrier</ent><br/
<hw>catbrier</hw> <pos>n.</pos> <def>a very prickly woody vine of eastern U.S. (<spn>Smilax rotundifolia</spn>) growing in tangled masses, having tough round stems with shiny leathery leaves and small greenish flowers followed by clusters of inedible shiny black berries.</def><br/
<syn><b>Syn. --</b> bullbrier, greenbrier, horsebrier, brier, briar, <spn>Smilax rotundifolia</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catcall</ent><br/
<hw>Cat"call`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sound like the cry of a cat, such as is made in playhouses to express dissatisfaction with a play; also, a small shrill instrument for making such a noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon the rising of the curtain. I was very much surprised with the great consort of <qex>catcalls</qex> which was exhibited.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catch</ent><br/
<hw>Catch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caught</conjf> <pr>(?)</pr> <or/ <conjf>Catched</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Catching</conjf>. <xex>Catched</xex> is rarely used.]</vmorph> <ety>[OE. <ets>cacchen</ets>, OF. <ets>cachier</ets>, dialectic form of <ets>chacier</ets> to hunt, F. <ets>chasser</ets>, fr. (assumend) LL. <ets>captiare</ets>, for L. <ets>capture</ets>, V. intens. of <ets>capere</ets> to take, catch. See <er>Capacious</er>, and cf. <er>Chase</er>, <er>Case</er> a box.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To lay hold on; to seize, especially with the hand; to grasp (anything) in motion, with the effect of holding; <as>as, to <ex>catch</ex> a ball</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To seize after pursuing; to arrest; <as>as, to <ex>catch</ex> a thief</as>.</def> <ldquo/They pursued . . . and <xex>caught</xex> him.<rdquo/  <rj><au>Judg. i. 6.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take captive, as in a snare or net, or on a hook; <as>as, to <ex>catch</ex> a bird or fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence: To insnare; to entangle.</def> <ldquo/To <xex>catch</xex> him in his words<rdquo/.  <rj><au>Mark xii. 13.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To seize with the senses or the mind; to apprehend; <as>as, to <ex>catch</ex> a melody</as>.</def> <ldquo/Fiery thoughts . . . whereof I <xex>catch</xex> the issue.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To communicate to; to fasten upon; <as>as, the fire <ex>caught</ex> the adjoining building</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To engage and attach; to please; to charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The soothing arts that <qex>catch</qex> the fair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To get possession of; to attain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Torment myself to <qex>catch</qex> the English throne.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To take or receive; esp. to take by sympathy, contagion, infection, or exposure; <as>as, to <ex>catch</ex> the spirit of an occasion; to <ex>catch</ex> the measles or smallpox; to <ex>catch</ex> cold; the house <ex>caught</ex> fire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To come upon unexpectedly or by surprise; to find; <as>as, to <ex>catch</ex> one in the act of stealing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To reach in time; to come up with; <as>as, to <ex>catch</ex> a train</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To catch fire</b></col>, <cd>to become inflamed or ignited.</cd> -- <col><b>to catch it</b></col> <cd>to get a scolding or beating; to suffer punishment.</cd> <mark>[Colloq.]</mark> -- <col><b>To catch one's eye</b></col>, <cd>to interrupt captiously while speaking.</cd> <mark>[Colloq.]</mark> <ldquo/You <xex>catch me up</xex> so very short.<rdquo/ <au>Dickens.</au> -- <col><b>To catch up</b></col>, <cd>to snatch; to take up suddenly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catch</ent><br/
<hw>Catch</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To attain possession.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Have is have, however men do <qex>catch</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be held or impeded by entanglement or a light obstruction; <as>as, a kite <ex>catches</ex> in a tree; a door <ex>catches</ex> so as not to open</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take hold; <as>as, the bolt does not <ex>catch</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To spread by, or as by, infecting; to communicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Does the sedition <qex>catch</qex> from man to man?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To catch at</b></col>, <cd>to attempt to seize; to be eager to get or use.</cd> <ldquo/[To] <xex>catch at</xex> all opportunities of subverting the state.<rdquo/ <au>Addison.</au> -- <col><b>To catch up with</b></col>, <cd>to come up with; to overtake.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catch</ent><br/
<hw>Catch</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act of seizing; a grasp.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That by which anything is caught or temporarily fastened; <as>as, the <ex>catch</ex> of a gate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The posture of seizing; a state of preparation to lay hold of, or of watching he opportunity to seize; <as>as, to lie on the <ex>catch</ex></as>.</def> <mark>[Archaic]</mark>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The common and the canon law . . . lie at <qex>catch</qex>, and wait advantages one againt another.</q> <rj><qau>T. Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is caught or taken; profit; gain; especially, the whole quantity caught or taken at one time; <as>as, a good <ex>catch</ex> of fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hector shall have a great <qex>catch</qex> if he knock out either of your brains.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Something desirable to be caught, esp. a husband or wife in matrimony.</def> <mark>[Colloq.]</mark>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <def>Passing opportunities seized; snatches.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It has been writ by <qex>catches</qex> with many intervals.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A slight remembrance; a trace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We retain a <qex>catch</qex> of those pretty stories.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mus.)</fld> <def>A humorous canon or round, so contrived that the singers catch up each other's words.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchable</ent><br/
<hw>Catch"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being caught.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catch-basin</ent><br/
<hw>Catch"-ba`sin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cistern or vault at the point where a street gutter discharges into a sewer, to catch bulky matters which would not pass readily through the sewer.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catch crop</ent><br/
<hw>Catch crop</hw>. <def>Any crop grown between the rows of another crop or intermediate between two crops in ordinary rotation in point of time.</def> -- <wordforms><wf>Catch"-crop`ping</wf>, <pos>n.</pos></wordforms></p>

<p><q>Radishes . . . are often grown as a <qex>catch crop</qex> with other vegetables.</q>  <rj><qau>L. H. Bailey.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Catchdrain</ent><br/
<hw>Catch"drain`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A ditch or drain along the side of a hill to catch the surface water; also, a ditch at the side of a canal to catch the surplus water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catcher</ent><br/
<hw>Catch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, catches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Baseball)</fld> <def>The player who stands behind the batsman to catch the ball.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchfly</ent><br/
<hw>Catch"fly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant with the joints of the stem, and sometimes other parts, covered with a viscid secretion to which small insects adhere. The species of <gen>Silene</gen> are examples of the catchfly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catching</ent><br/
<hw>Catch"ing</hw> <pos>a.</pos> <sn>1.</sn> <def>Infectious; contagious.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Captivating; alluring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catching</ent><br/
<hw>Catch"ing</hw>, <pos>n.</pos> <def>The act of seizing or taking hold of.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Catching bargain</b></col> <fld>(Law)</fld>, <cd>a bargain made with an heir expectant for the purchase of his expectancy at an inadequate price.</cd>  <rj><au>Bouvier.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catch-meadow</ent><br/
<hw>Catch"-mead`ow</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A meadow irrigated by water from a spring or rivulet on the side of hill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchment</ent><br/
<hw>Catch"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A surface of ground on which water may be caught and collected into a reservoir.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchpenny</ent><br/
<hw>Catch"pen*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made or contrived for getting small sums of money from the ignorant or unwary; <as>as, a <ex>catchpenny</ex> book; a <ex>catchpenny</ex> show</as>.</def> -- <def2><pos>n.</pos> <def>Some worthless catchpenny thing.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchpoll</ent><br/
<hw>Catch"poll`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chacepol</ets>, <ets>chacipol</ets>.]</ety> <def>A bailiff's assistant.</def></p>

<p><ent>Catch title</ent><br/
<hw>Catch title</hw>. <def>A short expressive title used for abbreviated book lists, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Catsup</ent><br/
<ent>Catchup</ent><br/
<mhw>{ <hw>Catch"up</hw> <pr>(?)</pr>, <hw>Cat"sup</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Probably of East Indian origin, because it was originally a kind of East Indian pickles.  Cf. also Malay <ets>k<ecr/chap</ets> fish sauce. <au>MW10</au>.]</ety> <def>A pureed table sauce made predominantly from tomatoes, flavored with onions, sugar, salt and spices; called also <altname>tomato ketchup</altname>. The term is also applied to pureed sauces containing mushrooms, walnuts, etc., being called in such cases <altname>mushroom ketchup</altname>, <altname>walnut ketchup</altname>, etc.</def> <altsp>[Written also <asp>ketchup</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchwater</ent><br/
<hw>Catch"wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A ditch or drain for catching water. See <er>Catchdrain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchweed</ent><br/
<hw>Catch"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Cleavers</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchweight</ent><br/
<hw>Catch"weight`</hw> <pr>(?)</pr>, <pos>adv.</pos> <fld>(Horseracing)</fld> <def>Without any additional weight; without being handicapped; <as>as, to ride <ex>catchweight</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchword</ent><br/
<hw>Catch"word`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Among theatrical performers, the last word of the preceding speaker, which reminds one that he is to speak next; cue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>The first word of any page of a book after the first, inserted at the right hand bottom corner of the preceding page for the assistance of the reader. It is seldom used in modern printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A word or phrase caught up and repeated for effect; <as>as, the <ex>catchword</ex> of a political party, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchwork</ent><br/
<hw>Catch"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A work or artificial water-course for throwing water on lands that lie on the slopes of hills; a catchdrain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catchy</ent><br/
<hw>Catch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Apt or tending to catch the fancy or attention; catching; taking; <as>as, <ex>catchy</ex> music</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Tending to catch or insnare; entangling; -- usually used fig.; <as>as, a <ex>catchy</ex> question</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>Consisting of, or occuring in, disconnected parts or snatches; changeable; <as>as, a <ex>catchy</ex> wind</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>It [the fox's scent] is . . . flighty or <qex>catchy</qex>, if variable.</q>  <rj><qau>Encyc. of Sport.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cate</ent><br/
<hw>Cate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Food.</def> <mark>[Obs.]</mark> <see>See <er>Cates</er></see>.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechetical</ent><br/
<ent>Catechetic</ent><br/
<mhw>{ <hw>Cat`e*chet"ic</hw> <pr>(?)</pr>, <hw>Cat`e*chet"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/. See <er>Catechise</er>.]</ety> <def>Relating to or consisting in, asking questions and receiving answers, according to the ancient manner of teaching.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Socrates introduced a <qex>catechetical</qex> method of arguing.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechetically</ent><br/
<hw>Cat`e*chet"ic*al*ly</hw>, <pos>adv.</pos> <def>In a catechetical manner; by question and answer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechetics</ent><br/
<hw>Cat`e*chet"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science or practice of instructing by questions and answers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechin</ent><br/
<hw>Cat"e*chin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>One of the tannic acids (<chform>C15H14O6</chform>), extracted from <er>catechu</er> as a white, crystalline substance; -- called also <altname>catechuic acid</altname>, <altname>catechinic acid</altname>, <altname>cyanidol</altname>, and <altname>catechuin</altname>.  It is a <isa>flavonoid</isa> found generally in higher woody plants, and is used in dyeing and tanning.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Catechisation</ent><br/
<hw>Cat`e*chi*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>catechizatio</ets>.]</ety> <def>The act of catechising.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechise</ent><br/
<hw>Cat"e*chise</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Catechised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Catechising</conjf>.]</vmorph> <ety>[L. <ets>catechizare</ets>, Gr. <?/, equiv. to <?/ to resound, sound a thing into one's ears, impress it upon one by word of mouth; <?/ + <?/ to sound, <?/ a sound.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To instruct by asking questions, receiving answers, and offering explanations and corrections, -- esp. in regard to points of religious faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To question or interrogate; to examine or try by questions; -- sometimes with a view to reproof, by eliciting from a person answers which condemn his own conduct.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 227 --></p>

<p><ent>Catechiser</ent><br/
<hw>Cat"e*chi`ser</hw> <pr>(k<acr/t"<esl/*k<imac/`z<etil/r)</pr>, <pos>n.</pos> <def>One who catechises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechism</ent><br/
<hw>Cat"e*chism</hw> <pr>(-k<icr/z'm)</pr>, <pos>n.</pos> <ety>[L. <ets>catechismus</ets>, fr. Gr. See <er>Catechise</er>.]</ety> <sn>1.</sn> <def>A form of instruction by means of questions and answers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A book containing a summary of principles, especially of religious doctrine, reduced to the form of questions and answers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Jews, even till this day, have their <qex>catechisms</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>The Larger Catechism</b></col>, <col><b>The Shorter Catechism</b></col></mcol>. <cd>See <cref>Westminster Assembly</cref>, under <er>Assembly</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechismal</ent><br/
<hw>Cat`e*chis"mal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a catechism, having the form of questions and answers; catechetical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechist</ent><br/
<hw>Cat"e*chist</hw> <pr>(k<acr/t"<esl/*k<icr/st)</pr>, <pos>n.</pos> <ety>[L. <ets>catechista</ets>, fr. Gr.]</ety> <def>One who instructs by question and answer, especially in religions matters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechistical</ent><br/
<ent>Catechistic</ent><br/
<mhw>{ <hw>Cat`e*chis"tic</hw> <pr>(-k<icr/s"t<icr/k)</pr>, <hw>Cat`e*chis"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a catechist or to a catechism.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechize</ent><br/
<hw>Cat"e*chize</hw>, <pos>v. t.</pos> <def>See <er>Catechise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catechol</ent><br/
<hw>cat"e*chol</hw> <pr>(k<acr/t"<ecr/*k<add/l`)</pr>, <pos>n.</pos> <ety>[.]</ety> <def>any of a group of chemicals having a benzene ring substituted with two hydroxyl groups in ortho position, especially ortho-dihydroxybenzene itself (<chform>C6H6O2</chform>), also called <altname>pyrocatechol</altname>, used in photography and as a reagent.</def><br/
[<source>PJC</source>]</p>

<p><ent>catecholamine</ent><br/
<hw>catecholamine</hw> <pos>n.</pos> <ety>[<ets>catechol</ets> + <ets>amine</ets>.]</ety> <def>any of a group of chemicals including epinephrine and norepinephrine that are produced in the medulla of the adrenal gland.  They function as nerotransmitters in the sympathetic nervous system.  Also, any structurally related chemical substance, whether natural or synthetic.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Catechu</ent><br/
<hw>Cat"e*chu</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cashoo</er>.]</ety> <fld>(Chem.)</fld> <def>A dry, brown, astringent extract, obtained by decoction and evaporation from the <spn>Acacia catechu</spn>, and several other plants growing in India. It contains a large portion of tannin or tannic acid, and is used in medicine and in the arts. It is also known by the names <altname>terra japonica</altname>, <altname>cutch</altname>, <altname>gambier</altname>, etc.</def>  <rj><au>Ure.</au> <au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechuic</ent><br/
<hw>Cat`e*chu"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to catechu or its derivatives. See <er>catechin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechumen</ent><br/
<hw>Cat"e*chu`men</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>catechumenus</ets>, Gr. <grk>kathchoy`menos</grk> instructed, from <grk>kathchei^n</grk>. See <er>Catechise</er>.]</ety> <fld>(Eccl.)</fld> <def>One who is receiving rudimentary instruction in the doctrines of Christianity; a neophyte; in the primitive church, one officially recognized as a Christian, and admitted to instruction preliminary to admission to full membership in the church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechumenate</ent><br/
<hw>Cat`e*chu"men*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or condition of a catechumen or the time during which one is a catechumen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechumenical</ent><br/
<hw>Cat`e*chu*men"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to catechumens; <as>as, <ex>catechumenical</ex> instructions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catechumenist</ent><br/
<hw>Cat`e*chu"men*ist</hw>, <pos>n.</pos> <def>A catechumen.</def>  <rj><au>Bp. Morton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Categorematic</ent><br/
<hw>Cat`e*gor`e*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ predicate. See <er>Category</er>.]</ety> <fld>(Logic.)</fld> <def>Capable of being employed by itself as a term; -- said of a word.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Categorical</ent><br/
<hw>Cat`e*gor"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a category.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not hypothetical or relative; admitting no conditions or exceptions; declarative; absolute; positive; express; <as>as, a <ex>categorical</ex> proposition, or answer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The scriptures by a multitude of <qex>categorical</qex> and intelligible decisions . . . distinguish between the things seen and temporal and those that are unseen and eternal.</q>  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Categorically</ent><br/
<hw>Cat`e*gor"ic*al*ly</hw>, <pos>adv.</pos> <def>Absolutely; directly; expressly; positively; <as>as, to affirm <ex>categorically</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Categoricalness</ent><br/
<hw>Cat`e*gor"ic*al*ness</hw>, <pos>n.</pos> <def>The quality of being categorical, positive, or absolute.</def>  <rj><au>A. Marvell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Categorist</ent><br/
<hw>Cat"e*go*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who inserts in a category or list; one who classifies.</def>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Categorize</ent><br/
<hw>Cat"e*go*rize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To insert in a category or list; to class; to catalogue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>categorized</ent><br/
<hw>categorized</hw> <pos>adj.</pos> <def>placed in a category.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Category</ent><br/
<hw>Cat"e*go*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Categories</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>categoria</ets>, Gr. <?/, fr. <?/ to accuse, affirm, predicate; <?/ down, against + <?/ to harrangue, assert, fr. <?/ assembly.]</ety> <sn>1.</sn> <fld>(Logic.)</fld> <def>One of the highest classes to which the objects of knowledge or thought can be reduced, and by which they can be arranged in a system; an ultimate or undecomposable conception; a predicament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>categories</qex> or predicaments -- the former a Greek word, the latter its literal translation in the Latin language -- were intended by Aristotle and his followers as an enumeration of all things capable of being named; an enumeration by the <qex>summa genera</qex> i.e., the most extensive classes into which things could be distributed.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Class; also, state, condition, or predicament; <as>as, we are both in the same <ex>category</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is in modern literature a whole class of writers standing within the same <qex>category</qex>.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catel</ent><br/
<hw>Cat"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chattel</er>.]</ety> <def>Property; -- often used by Chaucer in contrast with <xex>rent</xex>, or <xex>income</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/<qex>For loss of catel</qex> may recovered be,<br/
But loss of tyme shendeth us,<rdquo/ quod he.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catelectrode</ent><br/
<hw>Cat`e*lec"trode</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>cata + elecrode</ets>.]</ety> <fld>(Physics)</fld> <def>The negative electrode or pole of a voltaic battery.</def>  <rj><au>Faraday.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catelectrotonic</ent><br/
<hw>Cat`e*lec`tro*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physics)</fld> <def>Relating to, or characterized by, catelectrotonus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catelectrotonus</ent><br/
||<hw>Cat`e*lec*trot"o*nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ down + <?/ (see <er>Electro-</er>) + <?/ tone.]</ety> <fld>(Physics)</fld> <def>The condition of increased irritability of a nerve in the region of the cathode or negative electrode, on the passage of a current of electricity through it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catena</ent><br/
||<hw>Ca*te"na</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Catene</plw> <pr>(#)</pr>.</plu> <ety>[L., a chain.]</ety> <def>A chain or series of things connected with each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have . . . in no case sought to construct those <qex>caten<ae/</qex> of games, which it seems now the fashion of commentators to link together.</q> <rj><qau>C. J. Ellicott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catenarian</ent><br/
<ent>Catenary</ent><br/
<mhw>{ <hw>Cat"e*na*ry</hw> <pr>(?)</pr>, <hw>Cat`e*na"ri*an</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>catenarius</ets>, fr. <ets>catena</ets> a chain. See <er>Chain</er>.]</ety> <def>Relating to a chain; like a chain; <as>as, a <er>catenary</er> curve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catenary</ent><br/
<hw>Cat"e*na*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <er>Catenaries</er> <pr>(<?/)</pr>.</plu> <fld>(Geol.)</fld> <def>The curve formed by a rope or chain of uniform density and perfect flexibility, hanging freely between two points of suspension, not in the same vertical line.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catenate</ent><br/
<hw>Cat"e*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Catenated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Catenating</conjf>.]</vmorph> <ety>[L. <ets>catenatus</ets>, p. p. of <ets>catenare</ets>, fr. <ets>catena</ets> chain. See <er>Chain</er>.]</ety> <def>To connect, in a series of links or ties; to chain.</def>  <rj><au>E. Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catenation</ent><br/
<hw>Cat`e*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>catenatio</ets>.]</ety> <def>Connection of links or union of parts, as in a chain; a regular or connected series. See <er>Concatenation</er>.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catenulate</ent><br/
<hw>Ca*ten"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>catenuia</ets>, dim. of <ets>catena</ets> chain.]</ety> <sn>1.</sn> <def>Consisting of little links or chains.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Chainlike; -- said both or color marks and of indentations when arranged like the links of a chain, as on shells, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cater</ent><br/
<hw>Ca"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>catour</ets> purchaser, caterer, OF. <ets>acator</ets>, fr. <ets>acater</ets>, F. <ets>acheter</ets>, to buy, provide, fr. LL. <ets>accaptare</ets>; L. <ets>ad</ets> + captare to strive, to seize, intens, of <ets>capere</ets> to take, seize. Cf. <er>Acater</er>, <er>Capacious</er>.]</ety> <def>A provider; a purveyor; a caterer.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cater</ent><br/
<hw>Ca"ter</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Catered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Catering</conjf>.]</vmorph> <ety>[From <er>Cater</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To provide food; to buy, procure, or prepare provisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] providently <qex>caters</qex> for the sparrow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>By extension: To supply what is needed or desired, at theatrical or musical entertainments; -- followed by <xex>for</xex> or <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cater</ent><br/
<hw>Ca"ter</hw>, <pos>n.</pos> <ety>[F. <ets>quatre</ets> four.]</ety> <def>The four of cards or dice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cater</ent><br/
<hw>Ca"ter</hw>, <pos>v. t.</pos> <def>To cut diagonally.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cateran</ent><br/
<hw>Cat"e*ran</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>ceatharnach</ets>. Cf. <er>Kern</er> Irish foot soldier.]</ety> <def>A Highland robber: a kind of irregular soldier.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cater-cornered</ent><br/
<hw>Ca"ter-cor`nered</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Cater</er> to cut diagonally.]</ety> <def>Diagonal.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cater-cousin</ent><br/
<hw>Ca"ter-cous`in</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A remote relation. See <er>Quater-cousin</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caterer</ent><br/
<hw>Ca"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who caters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The little fowls in the air have God for Their provider and <qex>caterer</qex>.</q> <rj><qau>Shelton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cateress</ent><br/
<hw>Ca"ter*ess</hw>, <pos>n.</pos> <def>A woman who caters.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caterpillar</ent><br/
<hw>Cat"er*pil`lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>catyrpel</ets>, corrupted fr. OF. <ets>chatepelouse</ets>, or <ets>cate pelue</ets>, fr. <ets>chate</ets>, F. <ets>chatte</ets>, she-cat, fem. of <ets>chat</ets>, L. <ets>catus</ets> + L. <ets>pilosus</ets> hairy, or F. <ets>pelu</ets> hairy, fr. L. <ets>pilus</ets> hair. See <er>Cat</er>, and <er>Pile</er> hair.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The larval state of a butterfly or any lepidopterous insect; sometimes, but less commonly, the larval state of other insects, as the sawflies, which are also called false caterpillars. The true caterpillars have three pairs of true legs, and several pairs of abdominal fleshy legs (prolegs) armed with hooks. Some are hairy, others naked. They usually feed on leaves, fruit, and succulent vegetables, being often very destructive, Many of them are popularly called worms, as the <stype>cutworm</stype>, <stype>cankerworm</stype>, <stype>army worm</stype>, <stype>cotton worm</stype>, <stype>silkworm</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Scorpiurus</gen>, with pods resembling caterpillars.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Caterpillar catcher</b></col>, <it>or</it> <col><b>Caterpillar eater</b></col></mcol> <fld>(Zool.)</fld>, <cd>a bird belonging to the family of Shrikes, which feeds on caterpillars. The name is also given to several other birds.</cd> -- <col><b>Caterpillar hunter</b></col> <fld>(Zool.)</fld>, <cd>any species of beetles of the genus <gen>Callosoma</gen> and other allied genera of the family <fam>Carabid<ae/</fam> which feed habitually upon caterpillars.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caterpillar track</ent><br/
<hw>Cat"er*pil`lar track</hw> <pr>(?)</pr>, <pos>n.</pos> <def>an endless belt looped over several wheels, used in pairs in place of wheels to propel vehicles, such as tractors or military tanks, over rough ground.</def><br/
[<source>PJC</source>]</p>

<p><ent>caterpillar-tracked</ent><br/
<hw>caterpillar-tracked</hw> <pos>adj.</pos>  <def>having caterpillar treads or tracks on the wheels; <as>as, a <ex>caterpillar-tracked</ex> earthmover</as>.</def><br/
<syn><b>Syn. --</b> caterpillar.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caterwaul</ent><br/
<hw>Cat"er*waul</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caterwauled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caterwauling</conjf>.]</vmorph> <ety>[<ets>Cat</ets> + <ets>waul</ets>, <ets>wawl</ets>, to cry as a cat.]</ety> <def>To cry as cats in rutting time; to make a harsh, offensive noise.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caterwaul</ent><br/
<hw>Cat"er*waul</hw>, <pos>n.</pos> <def>A caterwauling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caterwauling</ent><br/
<hw>Cat"er*waul`ing</hw>, <pos>n.</pos> <def>The cry of cats; a harsh, disagreeable noise or cry like the cry of cats.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catery</ent><br/
<hw>Ca"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cater</er>, <pos>n.</pos>]</ety> <def>The place where provisions are deposited.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cates</ent><br/
<hw>Cates</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Cf. <er>Acates</er>, and see <er>Cater</er>, <pos>n.</pos>]</ety> <def>Provisions; food; viands; especially, luxurious food; delicacies; dainties.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cates</qex> for which Apicius could not pay.</q> <rj><qau>Shurchill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Choicest <qex>cates</qex> and the fiagon's best spilth.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-eyed</ent><br/
<hw>Cat"-eyed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having eyes like a cat; hence, able to see in the dark.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catfall</ent><br/
<hw>Cat"fall`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A rope used in hoisting the anchor to the cathead.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catfish</ent><br/
<hw>Cat"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A name given in the United States to various species of siluroid fishes; <as>as, the <ex>yellow cat</ex> (<ex>Amiurus natalis</ex>); the <ex>bind cat</ex> (<spn>Gronias nigrilabrus</spn>); the <ex>mud cat</ex> (<spn>Pilodictic oilwaris</spn>), the <ex>stone cat</ex> (<spn>Noturus flavus</spn>); the <ex>sea cat</ex> (<spn>Arius felis</spn>), etc.</as>  This name is also sometimes applied to the <altname>wolf fish</altname>. See <er>Bullhrad</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catgut</ent><br/
<hw>Cat"gut`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cat</ets> + <ets>gut</ets>.]</ety> <sn>1.</sn> <def>A cord of great toughness made from the intestines of animals, esp. of sheep, used for strings of musical instruments, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sort of linen or canvas, with wide interstices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catharine wheel</ent><br/
<hw>Cath"a*rine wheel`</hw> <pr>(?)</pr>. <def>See <er>catherine wheel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catharist</ent><br/
<hw>Cath"a*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>catharista</ets>, fr. Gr. <?/ clean, pure.]</ety> <def>One aiming at or pretending to a greater purity of like than others about him; -- applied to persons of various sects. See <er>Albigenses</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-harpin</ent><br/
<hw>Cat"-harp`in</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cat-harping</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-harping</ent><br/
<hw>Cat"-harp`ing</hw> <pos>n.</pos> <fld>(Naut.)</fld> <def>One of the short ropes or iron cramps used to brace in the shrouds toward the masts so a to give freer sweep to the yards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catharsis</ent><br/
||<hw>Ca*thar"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/. See <er>Cathartic</er>.]</ety> <fld>(Med.)</fld> <sn>1.</sn> <def>A natural or artificial purgation of any passage, as of the mouth, bowels, etc.</def></p>

<p><sn>2.</sn> <fld>(Psychotherapy)</fld> <def>The process of relieving an abnormal excitement by re<eum/stablishing the association of the emotion with the memory or idea of the event that first caused it, and of eliminating it by complete expression (called the <xex>abreaction</xex>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Catharical</ent><br/
<ent>Cathartic</ent><br/
<mhw>{ <hw>Ca*thar"tic</hw> <pr>(?)</pr>, <hw>Ca*thar"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ to cleanse, fr. <?/ pure; akin to F. <ets>chaste</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Cleansing the bowels; promoting evacuations by stool; purgative.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the purgative principle of senna, as <xex>cathartic</xex> acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathartic</ent><br/
<hw>Ca*thar"tic</hw>, <pos>n.</pos> <ety>[Gr. <?/.]</ety> <fld>(Med.)</fld> <def>A medicine that promotes alvine discharges; a purge; a purgative of moderate activity.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>cathartics</xex> are more energetic and certain in action that the <xex>laxatives</xex>, which simply increase the tendency to alvine evacuation; and less powerful and irritaint that the <xex>drastic</xex> purges, which cause profuse, repeated, and watery evacuations.</note></p>

<p>-- <wordforms><wf>Ca*thar"tic*al*ly</wf>, <pos>adv.</pos> -- <wf>Ca*thar"tic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>cathartin</ent><br/
<hw>ca*thar"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The bitter, purgative principle of senna. It is a glucoside with the properties of a weak acid; -- called also <altname>cathartic acid</altname>, and <altname>cathartina</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathay</ent><br/
<hw>Ca*thay"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>China; -- an old name for the Celestial Empire, said have been introduced by Marco Polo and to be a corruption of the Tartar name for North China (<xex>Khitai</xex>, the country of the Khitans.)</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Better fifty years of Europe than a cycle of <qex>Cathay</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathead</ent><br/
<hw>Cat"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A projecting piece of timber or iron near the bow of vessel, to which the anchor is hoisted and secured.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cathect</ent><br/
<hw>cathect</hw> <pos>v.</pos> <def>to inject with libidinal energy.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cathectic</ent><br/
<hw>cathectic</hw> <pos>adj.</pos> <def>of or pertaining to cathexis; invested with mental or emotional energy.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cathedra</ent><br/
||<hw>Cath"e*dra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/ seat. See <er>Chair</er>.]</ety> <def>The official chair or throne of a bishop, or of any person in high authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ex cathedra</b></col> <ety>[L., from the chair]</ety>, <cd>in the exercise of one's office; with authority.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The Vatican Council declares that the Pope, is infallible <ldquo/when he speaks <qex>ex cathedra</qex>.<rdquo/</q> <rj><qau>Addis & Arnold's Cath. Dict.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathedral</ent><br/
<hw>Ca*the"dral</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>cathedralis</ets> (sc. <ets>ecclesia</ets>): cf. F. <ets>cath<eacute/drale</ets>. See <er>Cathedra</er>.]</ety> <def>The principal church in a diocese, so called because in it the bishop has his official chair (<xex>Cathedra</xex>) or throne.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathedral</ent><br/
<hw>Ca*the"dral</hw>, <pos>a.</pos> <ety>[LL. <ets>cathedralis</ets>: cf. F. <ets>cath<eacute/dral</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Pertaining to the head church of a diocese; <as>as, a <ex>cathedral</ex> church; <ex>cathedral</ex> service</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Emanating from the chair of office, as of a pope or bishop; official; authoritative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, what solemnity can be more required for the pope to make a <qex>cathedral</qex> determination of an article!</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Resembling the aisles of a cathedral; <as>as, <ex>cathedral</ex> walks</as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathedralic</ent><br/
<hw>Cath`e*dral"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cathedral.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathedrated</ent><br/
<hw>Cath`e*dra"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cathedra</er>.]</ety> <def>Relating to the chair or office of a teacher.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catheretic</ent><br/
<hw>Cath`e*ret"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to bring down or raze; <?/ down + <?/ to take.]</ety> <fld>(Med.)</fld> <def>A mild kind caustic used to reduce warts and other excrescences.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catherine wheel</ent><br/
<hw>Cath"er*ine wheel`</hw> <pr>(?)</pr>. <ety>[So called from <ets>St. Catherine</ets> of Alexandria, who is represented with a <ets>wheel</ets>, in allusion to her martyrdom.]</ety> <sn>1.</sn> <fld>(Geoth.Arth.)</fld> <def>Same as <er>Rose window</er> and <er>Wheel window</er>. Called also <altname>Catherine-wheel window</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Pyrotechny)</fld> <def>A revolving piece of fireworks resembling in form the window of the same name.</def> <altsp>[Written also <asp>Catharine wheel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catheter</ent><br/
<hw>Cath"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/ a thing let down or put in, catheter, fr. <?/ to send down, to let down; <?/ + <?/ to send.]</ety> <fld>(Med.)</fld> <def>The name of various instruments for passing along mucous canals, esp. applied to a tubular instrument to be introduced into the bladder through the urethra to draw off the urine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Eustachian catheter</b></col>. <cd>See under <er>Eustachian</er>.</cd> -- <col><b>Prostatic catheter</b></col>, <cd>one adapted for passing an enlarged prostate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catheterization</ent><br/
<ent>Catheterism</ent><br/
<mhw>{ <hw>Cath"e*ter*ism</hw> <pr>(?)</pr>, <hw>Cath`e*ter*i*za"tion</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Med.)</fld> <def>The operation of introducing a catheter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catheterize</ent><br/
<hw>Cath"e*ter*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Catheterized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Catheterizing</conjf>.]</vmorph> <fld>(Med.)</fld> <def>To operate on with a catheter.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathetometer</ent><br/
<hw>Cath`e*tom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From Gr. <?/ vertical height + <ets>-meter</ets>.]</ety> <def>An instrument for the accurate measurement of small differences of height; esp. of the differences in the height of the upper surfaces of two columns of mercury or other fluid, or of the same column at different times. It consists of a telescopic leveling apparatus <it>(d)</it>, which slides up or down a perpendicular metallic standard very finely graduated (<it>bb</it>). The telescope is raised or depressed in order to sight the objects or surfaces, and the differences in vertical height are thus shown on the graduated standard.</def> <altsp>[Written also <asp>kathetometer</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathetus</ent><br/
||<hw>Cath"e*tus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>catheti</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/ a perpendicular line, fr. <?/ let down, fr. <?/. See <er>Catheter</er>.]</ety> <fld>(Geom.)</fld> <def>One line or radius falling perpendicularly on another; <as>as, the <ex>catheti</ex> of a right-angled triangle, that is, the two sides that include the right angle</as>.</def>  <rj><au>Barlow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathexis</ent><br/
<hw>Cath*ex"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ka`qexis</grk>, keeping.]</ety> <sn>1.</sn> <fld>(Psychiatry)</fld> <def>the process of investing mental, emotional, or libidinal energy or significance in an object, person, or idea.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <fld>(Psychiatry)</fld> <def>the emotional or libidinal energy invested in an object, person, or idea.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cathode</ent><br/
<hw>Cath"ode</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ descent; <?/ down + <?/ way.]</ety> <fld>(Physics)</fld> <def>The part of a voltaic battery by which the electric current leaves substances through which it passes, or the surface at which the electric current passes out of the electrolyte; the negative pole; -- opposed to <xex>anode</xex>.</def>  <rj><au>Faraday.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cathode ray</b></col> <fld>(Phys.)</fld>, <cd>a kind of ray generated at the cathode in a vacuum tube, by the electrical discharge<--  X-ray -->.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathodic</ent><br/
<hw>Ca*thod"ic</hw> <pr>(k<adot/*th<ocr/d"<icr/k)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>A term applied to the centrifugal, or efferent, course of the nervous influence.</def>  <rj><au>Marshall Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cathodegraph</ent><br/
<ent>Cathodograph</ent><br/
<mhw>{ <hw>Ca*thod"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Ca*thod"e*graph</hw> <pr>(?)</pr> }</mhw>. <ety>[<ets>Cathode</ets> + <ets>-graph</ets>.]</ety> <fld>(Physics)</fld> <def>A picture produced by the Roentgen rays; a radiograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cat-hole</ent><br/
<hw>Cat"-hole`</hw> <pr>(k<acr/t"h<omac/l`)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>One of two small holes astern, above the gunroom ports, through which hawsers may be passed.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 228 --></p>

<p><ent>Catholic</ent><br/
<hw>Cath"o*lic</hw> <pr>(k<acr/th"<osl/*<icr/k)</pr>, <pos>a.</pos> <ety>[L. <ets>catholicus</ets>, Gr. <grk>kaqoliko`s</grk>, universal, general; <grk>kata`</grk> down, wholly + <grk>"o`los</grk> whole, probably akin to E. <ets>solid</ets>: cf. F. <ets>catholique</ets>.]</ety> <sn>1.</sn> <def>Universal or general; <as>as, the <ex>catholic</ex> faith</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men of other countries [came] to bear their part in so great and <qex>catholic</qex> a war.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This epithet, which is applicable to the whole Christian church, or its faith, is claimed by Roman Catholics to belong especially to their church, and in popular usage is so limited.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not narrow-minded, partial, or bigoted; liberal; <as>as, <ex>catholic</ex> tastes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to, or affecting the Roman Catholics; <as>as, the <ex>Catholic</ex> emancipation act</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Catholic epistles</b></col>, <cd>the epistles of the apostles which are addressed to all the faithful, and not to a particular church; being those of James, Peter, Jude, and John.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholic</ent><br/
<hw>Cath"o*lic</hw>, <pos>n.</pos> <sn>1.</sn> <def>A person who accepts the creeds which are received in common by all parts of the orthodox Christian church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An adherent of the Roman Catholic church; a Roman Catholic.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Old Catholic</b></col>, <cd>the name assumed in 1870 by members of the Roman Catholic church, who denied the ecumenical character of the Vatican Council, and rejected its decrees, esp. that concerning the infallibility of the pope, as contrary to the ancient Catholic faith.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholical</ent><br/
<hw>Ca*thol"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Catholic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicism</ent><br/
<hw>Ca*thol"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>catholicisme</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The state or quality of being catholic or universal; catholicity.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liberality of sentiment; breadth of view.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The faith of the whole orthodox Christian church, or adherence thereto.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The doctrines or faith of the Roman Catholic church, or adherence thereto.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicity</ent><br/
<hw>Cath`o*lic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state or quality of being catholic; universality.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liberality of sentiments; catholicism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Adherence or conformity to the system of doctrine held by all parts of the orthodox Christian church; the doctrine so held; orthodoxy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Adherence to the doctrines of the church of Rome, or the doctrines themselves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicize</ent><br/
<hw>Ca*thol"i*cize</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To make or to become catholic or Roman Catholic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicly</ent><br/
<hw>Cath"o*lic*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a catholic manner; generally; universally.</def>  <rj><au>Sir L. Cary.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicness</ent><br/
<hw>Cath"o*lic*ness</hw>, <pos>n.</pos> <def>The quality of being catholic; universality; catholicity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicon</ent><br/
<hw>Ca*thol"i*con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, neut. <?/, universal. See <er>Catholic</er>.]</ety> <fld>(Med.)</fld> <def>A remedy for all diseases; a panacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catholicos</ent><br/
||<hw>Ca*thol"i*cos</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Catholic</er>.]</ety> <fld>(Eccl.)</fld> <def>The spiritual head of the Armenian church, who resides at Etchmiadzin, Russia, and has ecclesiastical jurisdiction over, and consecrates the holy oil for, the Armenians of Russia, Turkey, and Persia, including the Patriarchs of Constantinople, Jerusalem, and Sis.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Patriarch of Constantinople is the <xex>civil</xex> head of the Armenians in Turkey.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catilinarian</ent><br/
<hw>Cat`i*li*na"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Catilinarius</ets>.]</ety> <def>Pertaining to Catiline, the Roman conspirator; resembling Catiline's conspiracy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cation</ent><br/
<hw>Cat"i*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ downward + <?/ going, p. pr. of <?/ to go.]</ety> <fld>(Chem.)</fld> <def>a positively charged atom, radical, or molecule, which in electrolysis migrates to the cathode; a positive <isa>ion</isa>; -- opposed to <contr>anion</contr>.</def>  <rj><au>Faraday.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>cationic</ent><br/
<hw>cationic</hw> <pos>adj.</pos> <fld>(Chem.)</fld> <def>of or pertaining to cations; having a net positive charge; positively <isa>ionic</isa>; -- said of ions.  Contrasted with <ant>anionic</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catkin</ent><br/
<hw>Cat"kin</hw> <pr>(k<acr/t"k<icr/n)</pr>, <pos>n.</pos> <ety>[<ets>Cat</ets> + <ets>-kin</ets>.]</ety> <fld>(Bot.)</fld> <def>An ament; a species of inflorescence, consisting of a slender axis with many unisexual apetalous flowers along its sides, as in the willow and poplar, and (as to the staminate flowers) in the chestnut, oak, hickory, etc. -- so called from its resemblance to a cat's tail. See <xex>Illust.</xex> of <er>Ament</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catkinate</ent><br/
<hw>catkinate</hw> <pos>adj.</pos> <def>of, pertaining to, or resembling a catkin.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catlike</ent><br/
<hw>Cat"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a cat; stealthy; noiseless.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Catling</ent><br/
<hw>Cat"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cat</ets> + <ets>-ing</ets>.]</ety> <sn>1.</sn> <def>A little cat; a kitten.</def> <ldquo/Cat nor <xex>catling</xex>.<rdquo/  <rj><au>Drummond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Catgut; a catgut string.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Surg.)</fld> <def>A double-edged, sharp-pointed dismembering knife.</def> <altsp>[Spelt also <asp>catlin</asp>.]</altsp>  <rj><au>Crobb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catlinite</ent><br/
<hw>Cat"lin*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From George <ets>Catlin</ets>, an American traveler.]</ety> <def>A red clay from the Upper Missouri region, used by the Indians for their pipes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catmint</ent><br/
<ent>Catnip</ent><br/
<mhw>{ <hw>Cat"nip`</hw> <pr>(?)</pr>, <hw>Cat"mint`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Bot.)</fld> <def>A well-know plant of the genus <gen>Nepeta</gen> (<spn>Nepeta Cataria</spn>), somewhat like mint, having a string scent, and sometimes used in medicine. It is so called because cats have a peculiar fondness for it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cato-cathartic</ent><br/
<hw>Cat`o-ca*thar"tic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ down + <?/ serving to purge. See <er>Cathartic</er>.]</ety> <fld>(Med.)</fld> <def>A remedy that purges by alvine discharges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catonian</ent><br/
<hw>Ca*to"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Catonionus</ets>.]</ety> <def>Of, pertaining to, or resembling, the stern old Roman, Cato the Censor; severe; inflexible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cat-o'-nine-tails</ent><br/
<ent>Cat o' nine tails</ent><br/
<mhw><hw>Cat" o' nine" tails`</hw>, <hw>cat"-o'-nine"-tails`</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>a whip used as an instrument of punishment consisting of nine pieces of knotted line or cord fastened to a handle; -- formerly used to flog offenders on the bare back; -- called also the <altname>cat</altname>.  It was used in the British Navy to maintain discipline on board sailing ships.</def><br/
<syn><b>Syn. --</b> cat.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Catoptron</ent><br/
<ent>Catopter</ent><br/
<mhw>{ <hw>Ca*top"ter</hw> <pr>(?)</pr>, <hw>Ca*top"tron</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <?/ mirror, fr. <?/ visible.]</ety> <def>A reflecting optical glass or instrument; a mirror.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catoptrical</ent><br/
<ent>Catoptric</ent><br/
<mhw>{ <hw>Ca*top"tric</hw> <pr>(?)</pr>, <hw>Ca*top"tric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/. See <er>Catopter</er>.]</ety> <def>Of or pertaining to catoptrics; produced by reflection.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Catoptric light</b></col>, <cd>a light in which the rays are concentrated by reflectors into a beam visible at a distance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catoptrics</ent><br/
<hw>Ca*top"trics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>catoptrique</ets>. See <er>Catropric</er>.]</ety> <fld>(Physics)</fld> <def>That part of optics which explains the properties and phenomena of reflected light, and particularly that which is reflected from mirrors or polished bodies; -- formerly called <altname>anacamptics</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catoptromancy</ent><br/
<hw>Ca*top"tro*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ mirror + <ets>-mancy</ets>. See <er>Catopter</er>.]</ety> <fld>(Antiq.)</fld> <def>A species of divination, which was performed by letting down a mirror into water, for a sick person to look at his face in it. If his countenance appeared distorted and ghastly, it was an ill omen; if fresh and healthy, it was favorable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catoptron</ent><br/
<hw>Ca*top"tron</hw> <pr>(k<adot/*t<ocr/p"tr<ocr/n)</pr>, <pos>n.</pos> <mark>[Obs.]</mark> <def>See <er>Catopter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catpipe</ent><br/
<hw>Cat`pipe"</hw> <pr>(k<acr/t"p<imac/p`)</pr>, <pos>n.</pos> <def>See <er>Catcall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-rigged</ent><br/
<hw>Cat"-rigged`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Rigged like a catboat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-salt</ent><br/
<hw>Cat"-salt`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sort of salt, finely granulated, formed out of the bittern or leach brine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat's-eye</ent><br/
<hw>Cat's"-eye`</hw> <pr>(k<acr/ts"<imac/`)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A variety of quartz or chalcedony, exhibiting opalescent reflections from within, like the eye of a cat. The name is given to other gems affording like effects, esp. the chrysoberyl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat's-foot</ent><br/
<hw>Cat's`-foot</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant (<spn>Nepeta Glechoma</spn>) of the same genus with catnip; ground ivy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-silver</ent><br/
<hw>Cat"-sil`ver</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Mica.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catskill period</ent><br/
<hw>Cats"kill pe`ri*od</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>The closing subdivision of the Devonian age in America. The rocks of this period are well developed in the Catskill mountains, and extend south and west under the Carboniferous formation. See the Diagram under <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catso</ent><br/
<hw>Cat"so</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Catsos</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>cazzo</ets>.]</ety> <def>A base fellow; a rogue; a cheat.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat's-paw</ent><br/
<hw>Cat's"-paw`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A light transitory air which ruffles the surface of the water during a calm, or the ripples made by such a puff of air.</def> <sd>(b)</sd> <def>A particular hitch or turn in the bight of a rope, into which a tackle may be hooked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dupe; a tool; one who, or that which, is used by another as an instrument to a accomplish his purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense the term refers to the fable of the monkey using the cat's paw to draw the roasting chestnuts out of the fire.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat's-tail</ent><br/
<hw>Cat's"-tail</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Timothy</er>, <er>Cat-tail</er>, <er>Cirrus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catstick</ent><br/
<hw>Cat"stick`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stick or club employed in the game of ball called <xex>cat</xex> or <xex>tipcat</xex>.</def>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catstitch</ent><br/
<hw>Cat"stitch</hw> <pr>(k<acr/t"st<icr/ch)</pr>, <pos>v. t.</pos> <fld>(Needlework)</fld> <def>To fold and sew down the edge of with a coarse zigzag stitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Catsup</ent><br/
<hw>Cat"sup</hw> <pr>(k<acr/t"s<ucr/p)</pr>, <pos>n.</pos> <def>Same as <er>Catchup</er>, and <er>Ketchup</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cat-tail</ent><br/
<ent>cattail</ent><br/
<mhw><hw>cat"tail</hw>, <hw>Cat"-tail</hw></mhw> <pr>(k<acr/t"t<amac/l)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A tall erect rush or flag (<spn>Typha latifolia</spn>) growing widely in fresh and salt marshes, with long, flat, sword-shaped leaves, having clusters of small brown flowers in a dense cylindrical spike at the top of the stem; -- called also <altname>bulrush</altname> and <altname>reed mace</altname>. The leaves are frequently used for seating chairs, making mats, etc. See <er>Catkin</er>.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><note><hand/ The <stype>lesser cat-tail</stype> is <spn>Typha angustifolia</spn>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>cattalo</ent><br/
<hw>cattalo</hw> <pos>n.</pos> <def>hardy breed of cattle resulting from crossing domestic cattle with the American buffalo; it yields leaner beef than conventional breeds; it is called also <altname>beefalo</altname>.</def><br/
<syn><b>Syn. --</b> beefalo.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cattish</ent><br/
<hw>Cat"tish</hw> <pr>(k<acr/t"t<icr/sh)</pr>, <pos>a.</pos> <def>Catlike; feline</def>  <rj><au>Drummond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cattle</ent><br/
<hw>Cat"tle</hw> <pr>(k<acr/t"t'l)</pr>, <pos>n. pl.</pos> <ety>[OE. <ets>calet</ets>, <ets>chatel</ets>, goods, property, OF. <ets>catel</ets>, <ets>chatel</ets>, LL. <ets>captale</ets>, <ets>capitale</ets>, goods, property, esp. cattle, fr. L. <ets>capitals</ets> relating to the head, chief; because in early ages beasts constituted the chief part of a man's property. See <er>Capital</er>, and cf. <er>Chattel</er>.]</ety> <def>Quadrupeds of the Bovine family; sometimes, also, including all domestic quadrupeds, as sheep, goats, horses, mules, asses, and swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Belted cattle</b></col>, <col><b>Black cattle</b></col></mcol>. <cd>See under <er>Belted</er>, <er>Black</er>.</cd> -- <col><b>Cattle guard</b></col>, <cd>a trench under a railroad track and alongside a crossing (as of a public highway). It is intended to prevent cattle from getting upon the track.</cd> -- <col><b>cattle louse</b></col> <fld>(Zool.)</fld>, <cd>any species of louse infecting cattle. There are several species. The <spn>H<ae/matatopinus eurysternus</spn> and <spn>H<ae/matatopinus vituli</spn> are common species which suck blood; <spn>Trichodectes scalaris</spn> eats the hair.</cd> -- <col><b>Cattle plague</b></col>, <cd>the rinderpest; called also <altname>Russian cattle plague</altname>.</cd> -- <mcol><col><b>Cattle range</b></col>, <it>or</it> <col><b>Cattle run</b></col></mcol>, <cd>an open space through which cattle may run or range.</cd> <mark>[U. S.]</mark> <au>Bartlett.</au> -- <col><b>Cattle show</b></col>, <cd>an exhibition of domestic animals with prizes for the encouragement of stock breeding; -- usually accompanied with the exhibition of other agricultural and domestic products and of implements.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cattleship</ent><br/
<hw>cattleship</hw> <pos>n.</pos> <def>a cargo ship for the transport of livestock.</def><br/
<syn><b>Syn. --</b> cattle boat.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cattleya</ent><br/
<hw>cattleya</hw> <pos>n.</pos> <def>any orchid of the genus <gen>Cattleya</gen> characterized by a hood-shaped three-lobed lip enclosing the column; they are among the most popular and most extravagantly beautiful orchids known.  Called also <altname>cattleya orchid</altname>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Catty</ent><br/
<hw>Cat"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>kat<imac/</ets>. See <er>Caddy</er>.]</ety> <def>An East Indian Weight of 1<frac13/ pounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>catty-cornered</ent><br/
<ent>catty-corner</ent><br/
<mhw><hw>catty-corner</hw> <hw>catty-cornered</hw></mhw> <pos>adj.</pos> <sn>1.</sn>  <def>Having a slanted or oblique direction.</def><br/
<syn><b>Syn. --</b> bias, catacorner, cata-cornered, catercorner, cater-cornered, diagonal, kitty-corner, kitty-cornered, oblique, skew, skewed, slanted.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>catwalk</ent><br/
<hw>catwalk</hw> <pos>n.</pos> <sn>1.</sn> <def>A narrow walkway projecting from a stage into the seating area of a theater; it is used, <it>e. g.</it> by models displaying clothes on it at a fashion show.  Called also <altname>runway</altname>.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A narrow walkway high in the air to allow workers access to parts of a structure otherwise difficult to reach.  Catwalks are located, e.g. above a stage in a theater, between parts of a building, along the side of a bridge, on the outside of a railroad car, on the outside of a large storage tank, etc.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Caucasia</ent><br/
<hw>Caucasia</hw> <pos>n.</pos> <fld>(Geog.)</fld> <def>a large mountainous region between the Black and Caspian seas.</def><br/
<syn><b>Syn. --</b> Caucasus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caucasoid</ent><br/
<hw>caucasoid</hw> <pos>adj.</pos>  <def>belonging to the caucasian racial group.</def><br/
<syn><b>Syn. --</b> caucasian.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caucasus</ent><br/
<hw>Caucasus</hw> <pos>n.</pos> <sn>1.</sn>  <fld>(Geog.)</fld> <def>a large region between the Black and Caspian seas.</def><br/
<syn><b>Syn. --</b> Caucasia.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <fld>(Geog.)</fld> <def>the mountain range located between the Black and Caspian seas.</def><br/
[<source>PJC</source>]</p>

<p><ent>Caucasian</ent><br/
<hw>Cau*ca"sian</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to the Caucasus, a mountainous region between the Black and Caspian seas.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the white races of mankind, of whom the people about Mount Caucasus were formerly taken as the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caucasian</ent><br/
<hw>Cau*ca"sian</hw>, <pos>n.</pos> <sn>1.</sn> <def>A native or inhabitant of the Caucasus, esp. a Circassian or Georgian.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of any of the white races of mankind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caucus</ent><br/
<hw>Cau"cus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymology uncertain. Mr. J. H. Trumbull finds the origin of <ets>caucus</ets> in the N. A. Indian word <ets>cawcawwassough</ets> or <ets>ca<uacute/ cau-as'u</ets> one who urges or pushes on, a promoter. See citation for an early use of the word <ets>caucus</ets>.]</ety> <def>A meeting, especially a preliminary meeting, of persons belonging to a party, to nominate candidates for public office, or to select delegates to a nominating convention, or to confer regarding measures of party policy; a political primary meeting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This day learned that the <qex>caucus</qex> club meets, at certain times, in the garret of Tom Dawes, the adjutant of the Boston regiment.</q> <rj><qau>John Adams's Diary [Feb. , 1763].</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caucus</ent><br/
<hw>Cau"cus</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caucused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caucusing</conjf>.]</vmorph> <def>To hold, or meet in, a caucus or caucuses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudad</ent><br/
<hw>Cau"dad</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L. <ets>cauda</ets> tail + <ets>ad</ets> to.]</ety> <fld>(Zool.)</fld> <def>Backwards; toward the tail or posterior part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauda galli</ent><br/
||<hw>Cau"da gal*li</hw>, <pr>(<?/)</pr>. <ety>[L., tail of a cock.]</ety> <fld>(Paleon.)</fld> <def>A plume-shaped fossil, supposed to be a seaweed, characteristic of the lower Devonian rocks; <as>as, the <ex>cauda galli</ex> grit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cauda galli epoch</b></col> <fld>(Geol.)</fld>, <cd>an epoch at the begining of the Devonian age in eastern America, so named from the characteristic gritty sandstone marked with impressions of <spn>Cauda galli</spn>. See the Diagram under <er>Geology</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudal</ent><br/
<hw>Cau"dal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Cauda</ets> tail. Cf. <er>Coward</er>.]</ety> <def>Of the nature of, or pertaining to, a tail; having a tail-like appendage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The male widow-bird, remarkable for his <qex>caudal</qex> plumes.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caudal fin</b></col> <fld>(Zool.)</fld>, <cd>the terminal fin (or <ldquo/tail<rdquo/) of a fish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudata</ent><br/
||<hw>Cau*da"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>cauda</ets> tail.]</ety> <fld>(Zool.)</fld> <def>See <er>Urodela</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudated</ent><br/
<ent>Caudate</ent><br/
<mhw>{ <hw>Cau"date</hw> <pr>(?)</pr>, <hw>Cau"da*ted</hw> <pr>(?)</pr>. }</mhw> <pos>a.</pos> <ety>[L. <ets>cauda</ets> tail.]</ety> <def>Having a tail; having a terminal appendage like a tail.  Opposite of <ant>acaudate</ant>.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Caudex</ent><br/
||<hw>Cau"dex</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Caudices</plw> <pr>(#)</pr>, E. <plw>Caudexes</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Bot.)</fld> <def>The stem of a tree., esp. a stem without a branch, as of a palm or a tree fern; also, the perennial rootstock of an herbaceous plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudicula</ent><br/
<ent>Caudicle</ent><br/
<mhw>{ <hw>Cau"di*cle</hw> <pr>(?)</pr>, ||<hw>Cau*dic"u*la</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Dim. of L. <ets>cauda</ets> tail, appendage.]</ety> <fld>(Bot.)</fld> <def>A slender, elastic process, to which the masses of pollen in orchidaceous plants are attached.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudle</ent><br/
<hw>Cau"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>caudel</ets>, F. <ets>chaudeau</ets>, dim. of LL <ets>calidum</ets> a sweet drink, fr. L. <ets>caidus</ets> warm. See <er>Caldron</er>.]</ety> <def>A kind of warm drink for sick persons, being a mixture of wine with eggs, bread, sugar, and spices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caudle</ent><br/
<hw>Cau"dle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caudled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caudling</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To make into caudle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Too serve as a caudle to; to refresh.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauf</ent><br/
<hw>Cauf</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. akin to Celtic <ets>caff</ets>, <ets>cav</ets>, <ets>cau</ets>, L. <ets>cavus</ets> hollow, or to L. <ets>caphinus</ets>, Gr. <?/ basket.]</ety> <def>A chest with holes for keeping fish alive in water.</def>  <rj><au>Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caufle</ent><br/
<hw>Cau"fle</hw>, <pos>n.</pos> <def>A gang of slaves. Same as <er>Coffle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caught</ent><br/
<hw>Caught</hw> <pr>(k<add/t)</pr>, <def><pos>imp. & p. p.</pos> of <er>Catch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauker</ent><br/
<ent>Cauk</ent><br/
<mhw>{ <hw>Cauk</hw> <pr>(k<add/k)</pr>, <pos>n.</pos>, <hw>Cauk"er</hw> <pr>(-<etil/r)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Cawk</er>, <er>Calker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caul</ent><br/
<hw>Caul</hw> <pr>(k<add/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>calle</ets>, <ets>kelle</ets>, prob. fr. F. <ets>cale</ets>; cf. Ir. <ets>calla</ets> a veil.]</ety> <sn>1.</sn> <def>A covering of network for the head, worn by women; also, a net.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>The fold of membrane loaded with fat, which covers more or less of the intestines in mammals; the great omentum. See <er>Omentum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>caul</qex> serves for the warming of the lower belly.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A part of the amnion, one of the membranes enveloping the fetus, which sometimes is round the head of a child at its birth; -- called also a <altname>veil</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>It is deemed lucky to be with a <qex>caul</qex> or membrane over the face. This <qex>caul</qex> is esteemed an infallible preservative against drowning . . . According to Chrysostom, the midwives frequently sold it for magic uses.</q> <rj><qau>Grose.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was born with a <qex>caul</qex>, which was advertised for sale, in the newspapers, at the low price of fifteen guineas.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cauldron</ent><br/
<hw>caul"dron</hw> <pr>(k<add/l"dr<ucr/n)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a very large pot.</def><br/
<syn><b>Syn. --</b> caldron.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Caulescent</ent><br/
<hw>Cau*les"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caulis</ets> stalk, stem: cf. F. <ets>caulescent</ets>.]</ety> <fld>(Bot.)</fld> <def>Having a leafy stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caulicle</ent><br/
<hw>Cau"li*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A short caulis or stem, esp. the rudimentary stem seen in the embryo of a seed; -- otherwise called a <altname>radicle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauliculus</ent><br/
||<hw>Cau*lic"u*lus</hw> <pr>(k<add/*l<icr/k"<usl/*l<ucr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cauliculi</plw> <pr>(-l<imac/)</pr>.</plu> <ety>[L. <ets>cauliculus</ets> little stalk, dim. of <ets>caulis</ets>.]</ety> <fld>(Arch.)</fld> <def>In the Corinthian capital, one of the eight stalks rising out of the lower leafage and terminating in leaves which seem to support the volutes. See <er>Illust</er>. of <er>Corinthian order</er>, under <er>Corinthian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauliflower</ent><br/
<hw>Cau"li*flow`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>choufleur</ets>, modified by E. <ets>Cole</ets>. L. <ets>caulis</ets>, and by E. <ets>flower</ets>; F. <ets>chou</ets> cabbage is fr. L. <ets>caulis</ets> stalk, cabbage, and <ets>fleur</ets> flower is fr. L. <ets>flos</ets> flower. See <er>Cole</er>, and <er>Flower</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An annual variety of <spn>Brassica oleracea</spn>, or cabbage, of which the cluster of young flower stalks and buds is eaten as a vegetable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The edible head or <ldquo/curd<rdquo/ of a cauliflower plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauliform</ent><br/
<hw>Cau"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caulis</ets> + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the form of a caulis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauline</ent><br/
<hw>Cau"line</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Growing immediately on a caulis; of or pertaining to a caulis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caulis</ent><br/
||<hw>Cau"lis</hw> <pr>(?)</pr>, <pos>n.</pos>; L. <plu><it>pl.</it> <plw>Caules</plw> <pr>(#)</pr>.</plu> <ety>[L., a stem.]</ety> <fld>(Bot.)</fld> <def>An herbaceous or woody stem which bears leaves, and may bear flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caulk</ent><br/
<hw>Caulk</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>See <er>Calk</er>, <pos>v.</pos>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to fill in the seams or cracks of, with a waterproof material such as caulk.</def><br/
[<source>PJC</source>]</p>

<p><ent>Caulk</ent><br/
<hw>Caulk</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Calk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a viscous semisolid material of varying composition used to fill in seams of objects which are exposed to water, such as wooden ships or bath tiles; -- called also <altname>calk</altname> and <altname>caulking</altname>.  After applying in a semisolid form, the material hardens and dries to form a waterproof seal.  It is used in the process of caulking.  It is sometimes applied together with a rope-like cord to fill larger seams.</def><br/
[<source>PJC</source>]</p>

<p><ent>caulked</ent><br/
<hw>caulked</hw> <pos>adj.</pos> <sn>1.</sn> <def>having cracks and crevices stopped up with a filler such as caulk.  Contrasted with <ant>uncaulked</ant>.</def>  [Narrower terms: <stype>weather-stripped</stype>]<br/
<syn><b>Syn. --</b> calked, chinked, stopped-up.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>having the seams between planks packed with waterproof material; -- of boats and ships.</def><br/
<syn><b>Syn. --</b> calked.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>caulking</ent><br/
<hw>caulk"ing</hw> <pos>n.</pos> <sn>1.</sn> <def>the process of sealing cracks and crevices with a filler such as caulk{2}.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>same as <er>caulk{2}</er>, <pos>n.</pos>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Caulocarpous</ent><br/
<hw>Cau`lo*car"pous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ stem + <grk>karpo`s</grk> fruit.]</ety> <fld>(Bot.)</fld> <def>Having stems which bear flowers and fruit year after year, as most trees and shrubs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caulome</ent><br/
<hw>Cau"lome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kalo`s</grk> stem + <ets>-ome</ets> as in rhi<ets>zome</ets>.]</ety> <fld>(Bot.)</fld> <def>A stem structure or stem axis of a plant, viewed as a whole.</def> -- <wordforms><wf>Cau*lom"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cauma</ent><br/
||<hw>Cau"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/ a burning heat.]</ety> <fld>(Med.)</fld> <def>Great heat, as of the body in fever.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauponize</ent><br/
<hw>Cau"po*nize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>cauponari</ets>, fr. <ets>caupo</ets> huckster, innkeeper.]</ety> <def>To sell wine or victuals.</def> <mark>[Obs.]</mark>  <rj><au>Warburfon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causable</ent><br/
<hw>Caus"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being caused.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causal</ent><br/
<hw>Caus"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>causalis</ets>. See <er>Cause</er>.]</ety> <def>Relating to a cause or causes; inplying or containing a cause or causes; expressing a cause; causative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Causal</qex> propositions are where two propositions are joined by <qex>causal</qex> words.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causal</ent><br/
<hw>Caus"al</hw>, <pos>n.</pos> <def>A causal word or form of speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Anglo-Saxon <qex>drencan</qex> to drench, <qex>causal</qex> of Anglo-Saxon <qex>drincan</qex> to drink.</q> <rj><qau>Skeat.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causality</ent><br/
<hw>Cau*sal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Causalities</er> <pr>(<?/)</pr></plu>. <sn>1.</sn> <def>The agency of a cause; the action or power of a cause, in producing its effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>causality</qex> of the divine mind.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phren.)</fld> <def>The faculty of tracing effects to their causes.</def>  <rj><au>G. Combe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causally</ent><br/
<hw>Caus"al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>According to the order or series of causes; by tracing effects to causes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causally</ent><br/
<hw>Caus"al*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining.)</fld> <def>The lighter, earthy parts of ore, carried off washing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causation</ent><br/
<hw>Cau*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of causing; also the act or agency by which an effect is produced.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The kind of <qex>causation</qex> by which vision is produced.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Law of universal causation</b></col>, <cd>the theoretical or asserted law that every event or phenomenon results from, or is the sequel of, some previous event or phenomenon, which being present, the other is certain to take place.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causationist</ent><br/
<hw>Cau*sa"tion*ist</hw>, <pos>n.</pos> <def>One who believes in the law of universal causation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causative</ent><br/
<hw>Caus"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>causativus</ets> pertaining to a lawsuit (<ets>causa</ets>), but in the English sense from E. <ets>cause</ets>.]</ety> <sn>1.</sn> <def>Effective, as a cause or agent; causing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Causative</qex> in nature of a number of effects.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Expressing a cause or reason; causal; <as>as, the ablative is a <ex>causative</ex> case</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 229  --></p>

<p><ent>Causative</ent><br/
<hw>Caus"a*tive</hw> <pr>(k<add/"z<adot/*t<icr/v)</pr>, <pos>n.</pos> <def>A word which expresses or suggests a cause.</def><-- note syllabification is different in HW and PR. sic. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causatively</ent><br/
<hw>Caus"a*tive*ly</hw>, <pos>adv.</pos> <def>In a causative manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causator</ent><br/
<hw>Cau*sa"tor</hw> <pr>(k<add/*z<amac/"t<ocr/r)</pr>, <pos>n.</pos> <ety>[See <er>Cause</er>.]</ety> <def>One who causes.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cause</ent><br/
<hw>Cause</hw> <pr>(k<add/z)</pr>, <pos>n.</pos> <ety>[F. <ets>cause</ets>, fr. L. <ets>causa</ets>. Cf. <er>Cause</er>, <pos>v.</pos>, <er>Kickshaw</er>.]</ety> <sn>1.</sn> <def>That which produces or effects a result; that from which anything proceeds, and without which it would not exist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cause</qex> is substance exerting its power into act, to make one thing begin to be.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is the occasion of an action or state; ground; reason; motive; <as>as, <ex>cause</ex> for rejoicing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sake; interest; advantage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I did it not for his <qex>cause</qex>.</q> <rj><qau>2 Cor. vii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>A suit or action in court; any legal process by which a party endeavors to obtain his claim, or what he regards as his right; case; ground of action.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Any subject of discussion or debate; matter; question; affair in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What counsel give you in this weighty <qex>cause</qex>!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The side of a question, which is espoused, advocated, and upheld by a person or party; a principle which is advocated; that which a person or party seeks to attain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God befriend us, as our <qex>cause</qex> is just.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The part they take against me is from zeal to the <qex>cause</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Efficient cause</b></col>, <cd>the agent or force that produces a change or result.</cd> -- <col><b>Final cause</b></col>, <cd>the end, design, or object, for which anything is done.</cd> -- <col><b>Formal cause</b></col>, <cd>the elements of a conception which make the conception or the thing conceived to be what it is; or the idea viewed as a formative principle and cooperating with the matter.</cd> -- <col><b>Material cause</b></col>, <cd>that of which anything is made.</cd> -- <col><b>Proximate cause</b></col>. <cd>See under <er>Proximate</er>.</cd> -- <col><b>To make common cause with</b></col>, <cd>to join with in purposes and aims.</cd>  <rj><au>Macaulay.</au></rj></cs></p>

<p><syn><b>Syn.</b> -- Origin; source; mainspring; motive; reason; incitement; inducement; purpose; object; suit; action.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cause</ent><br/
<hw>Cause</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Causing</conjf>.]</vmorph> <ety>[F. <ets>causer</ets>, fr. <ets>cause</ets>, fr. L. <ets>causa</ets>. See <er>Cause</er>, <pos>n.</pos>, and cf. <er>Acouse</er>.]</ety> <def>To effect as an agent; to produce; to be the occasion of; to bring about; to bring into existence; to make; -- usually followed by an infinitive, sometimes by <xex>that</xex> with a finite verb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>cause</qex> it to rain upon the earth forty days.</q> <rj><qau>Gen. vii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cause</qex> that it be read also in the church of the Laodiceans.</q> <rj><qau>Col. iv. 16.</qau></rj></p>

<p><syn><b>Syn.</b> -- To create; produce; beget; effect; occasion; originate; induce; bring about.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cause</ent><br/
<hw>Cause</hw>, <pos>v. i.</pos> <def>To assign or show cause; to give a reason; to make excuse.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cause</ent><br/
<hw>Cause</hw>, <pos>conj.</pos> <def>Abbreviation of <er>Because</er>.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causeful</ent><br/
<hw>Cause"ful</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Having a cause.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causeless</ent><br/
<hw>Cause"less</hw>, <pos>a.</pos> <def>1. Self-originating; uncreated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without just or sufficient reason; groundless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My fears are <qex>causeless</qex> and ungrounded.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causeless</ent><br/
<hw>Cause"less</hw>, <pos>adv.</pos> <def>Without cause or reason.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causelessness</ent><br/
<hw>Cause"less*ness</hw>, <pos>n.</pos> <def>The state of being causeless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causer</ent><br/
<hw>Caus"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who or that which causes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causerie</ent><br/
||<hw>Cause`rie"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>causer</ets> to chat.]</ety> <def>Informal talk or discussion, as about literary matters; light conversation; chat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Causeuse</ent><br/
||<hw>Cau`seuse"</hw> <pr>(k<osl/`z<etil/z")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>causer</ets> to talk.]</ety> <def>A kind of sofa for two persons. A <cref>t<ecir/te-<agrave/-t<ecir/te</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causey</ent><br/
<ent>Causeway</ent><br/
<mhw>{ <hw>Cause"way</hw> <pr>(k<add/z"w<asl/)</pr>, <hw>Cau"sey</hw> <pr>((k<add/"z<ycr/)</pr>, }</mhw> <pos>n.</pos> <ety>[OE. <ets>cauci</ets>, <ets>cauchie</ets>, OF. <ets>cauchie</ets>, F. <ets>chauss<eacute/e</ets>, from LL. (<ets>via</ets>) <ets>calciata</ets>, fr <ets>calciare</ets> to make a road, either fr. L. <ets>calx</ets> lime, hence, to pave with limestone (cf. E. <ets>chalk</ets>), or from L. <ets>calceus</ets> shoe, from <ets>calx</ets> heel, hence, to shoe, pave, or wear by treading.]</ety> <def>A way or road raised above the natural level of the ground, serving as a dry passage over wet or marshy ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But that broad <qex>causeway</qex> will direct your way.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The other way Satan went down<br/
The <qex>causey</qex> to Hell-gate.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causeyed</ent><br/
<ent>Causewayed</ent><br/
<mhw>{ <hw>Cause"wayed</hw> <pr>(?)</pr>, <hw>Cau"seyed</hw> <pr>(?)</pr>. }</mhw> <pos>a.</pos> <def>Having a raised way (causeway or causey); paved.</def>  <rj><au>Sir W. Scott. C. Bront<eacute/.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causidical</ent><br/
<hw>Cau*sid"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>causidicakis</ets>; <ets>causa</ets> a cause in law + <ets>dicare</ets> to say.]</ety> <def>Pertaining to an advocate, or to the maintenance and defense of suits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caustical</ent><br/
<ent>Caustic</ent><br/
<mhw>{ <hw>Caus"tic</hw> <pr>(?)</pr>, <hw>Caus"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>caustucs</ets>, Ge. <?/, fr. <?/ to burn. Cf. <er>Calm</er>, <er>Ink</er>.]</ety> <sn>1.</sn> <def>Capable of destroying the texture of anything or eating away its substance by chemical action; burning; corrosive; searing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Severe; satirical; sharp; <as>as, a <ex>caustic</ex> remark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caustic curve</b></col> <fld>(Optics)</fld>, <cd>a curve to which the ray of light, reflected or refracted by another curve, are tangents, the reflecting or refracting curve and the luminous point being in one plane.</cd> -- <col><b>Caustic lime</b></col>. <cd>See under <er>Lime</er>.</cd> -- <mcol><col><b>Caustic potash</b></col>, <col><b>Caustic soda</b></col></mcol> <fld>(Chem.)</fld>, <cd>the solid hydroxides potash, <chform>KOH</chform>, and soda, <chform>NaOH</chform>, or solutions of the same.</cd> -- <col><b>Caustic silver</b></col>, <cd>nitrate of silver, lunar caustic.</cd> -- <col><b>Caustic surface</b></col> <fld>(Optics)</fld>, <cd>a surface to which rays reflected or refracted by another surface are tangents. Caustic curves and surfaces are called <xex>catacaustic</xex> when formed by reflection, and <xex>diacaustic</xex> when formed by refraction.</cd></cs></p>

<p><syn><b>Syn.</b> -- Stinging; cutting; pungent; searching.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caustic</ent><br/
<hw>Cau"stic</hw>, <pos>n.</pos> <ety>[L. <ets>causticum</ets> (sc. <ets>medicamentum</ets>). See <er>Caustic</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Any substance or means which, applied to animal or other organic tissue, burns, corrodes, or destroys it by chemical action; an escharotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Optics)</fld> <def>A caustic curve or caustic surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caustically</ent><br/
<hw>Caus"tic*al*ly</hw>, <pos>adv.</pos> <def>In a caustic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causticily</ent><br/
<hw>Caus*tic"i*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality of being caustic; corrosiveness; <as>as, the <ex>causticity</ex> of potash</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Severity of language; sarcasm; <as>as, the <ex>causticity</ex> of a reply or remark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Causticness</ent><br/
<hw>Caus"tic*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being caustic; causticity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautel</ent><br/
<hw>Cau"tel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caut<egrave/le</ets>, L. <ets>cautela</ets>, fr. <ets>cavere</ets> to be on one's guard, to take care.]</ety> <sn>1.</sn> <def>Caution; prudence; wariness.</def> <mark>[Obs.]</mark>  <rj><au>Fulke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Craft; deceit; falseness.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautelous</ent><br/
<hw>Cau"te*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>cauteleux</ets>, LL. <ets>cautelosus</ets>. See <er>Cautel</er>.]</ety> <sn>1.</sn> <def>Caution; prudent; wary.</def> <mark>[Obs.]</mark> <ldquo/<xex>Cautelous</xex>, though young.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Crafty; deceitful; false.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj></p>

<p>-- <wordforms><wf>Cau"te*lous*ly</wf>, <pos>adv.</pos> -- <wf>Cau"te*lous*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauter</ent><br/
<hw>Cau"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caut<egrave/re</ets>, L. <ets>cauterium</ets>, fr. Gr. <?/ a branding iron, fr. <?/ to burn. Cf. <er>Caustic</er>, <er>Cautery</er>.]</ety> <def>A hot iron for searing or cauterizing.</def>  <rj><au>Minsheu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauterant</ent><br/
<hw>Cau"ter*ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cauterizing substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauterism</ent><br/
<hw>Cau"ter*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The use or application of a caustic; cautery.</def>  <rj><au>Ferrand.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauterization</ent><br/
<hw>Cau`ter*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>caut<egrave/risation</ets>.]</ety> <fld>(Med.)</fld> <def>The act of searing some morbid part by the application of a cautery or caustic; also, the effect of such application.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cauterize</ent><br/
<hw>Cau"ter*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cauterized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cauterizing</conjf>.]</vmorph> <ety>[L. <ets>cauterizare</ets>, Gr. <?/, fr. a branding iron: cf. F. <ets>caut<eacute/rised</ets>.. See <er>cauter</er>.]</ety> <sn>1.</sn> <def>To burn or sear with a cautery or caustic.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sear, as the conscience.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautery</ent><br/
<hw>Cau"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cauteries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>cauterium</ets>, Gr. <?/. See <er>Cauter</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A burning or searing, as of morbid flesh, with a hot iron, or by application of a caustic that will burn, corrode, or destroy animal tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The iron of other agent in cauterizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Actual cautery</b></col>, <cd>a substance or agent (as a hot iron) which cauterizes or sears by actual heat; or the burning so effected.</cd> -- <col><b>Potential cautery</b></col>, <cd>a substance which cauterizes by chemical action; as, lunar <xex>caustic</xex>; also, the cauterizing produced by such substance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caution</ent><br/
<hw>Cau"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>caution</ets> a security, L. <ets>cautio</ets>, fr. <ets>cavere</ets> (For <ets>scavere</ets>) to be on one's guard, to take care (orig.) to be on the watch, see; akin to E. <ets>show</ets>.]</ety> <sn>1.</sn> <def>A careful attention to the probable effects of an act, in order that failure or harm may be avoided; prudence in regard to danger; provident care; wariness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Security; guaranty; bail.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Parliament would yet give his majesty sufficient <qex>caution</qex> that the war should be prosecuted.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Precept or warning against evil of any kind; exhortation to wariness; advice; injunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In way of <qex>caution</qex> I must tell you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Civil & Scots Law)</fld> <def>A pledge, bond, or other security for the performance of an obligation either in or out of judicial proceedings; the promise or contract of one not for himself but another; security.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Caution money</b></col>, <cd>money deposited by way of security or guaranty, as by a student at an English university.</cd></cs></p>

<p><syn><b>Syn.</b> -- Care; forethought; forecast; heed; prudence; watchfulness; vigilance; circumspection; anxiety; providence; counsel; advice; warning; admonition.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caution</ent><br/
<hw>Cau"tion</hw> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cautioned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cautioning</conjf>.]</vmorph> <def>To give notice of danger to; to warn; to exhort [one] to take heed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>cautioned</qex> me against their charms.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautionary</ent><br/
<hw>Cau"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Conveying a caution, or warning to avoid danger; <as>as, <ex>cautionary</ex> signals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Given as a pledge or as security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He hated Barnevelt, for his getting the <qex>cautionary</qex> towns out of his hands.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Wary; cautious.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautionary block</ent><br/
<hw>Cau"tion*a*ry block</hw>. <fld>(Railroads)</fld> <def>A block in which two or more trains are permitted to travel, under restrictions imposed by a caution card or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cautioner</ent><br/
<hw>Cau"tion*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who cautions or advises.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Scots Law)</fld> <def>A surety or sponsor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautionry</ent><br/
<hw>Cau"tion*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Scots Law)</fld> <def>Suretyship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautious</ent><br/
<hw>Cau"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>cautus</ets>, fr. <ets>caver</ets>. See <er>Caution</er>.]</ety> <def>Attentive to examine probable effects and consequences of acts with a view to avoid danger or misfortune; prudent; circumspect; wary; watchful; <as>as, a <ex>cautious</ex> general</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cautious</qex> feeling for another's pain.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be swift to hear; but <qex>cautious</qex> of your tongue.</q> <rj><qau>Watts.</qau></rj></p>

<p><syn><b>Syn.</b> -- Wary; watchful; vigilant; prudent; circumspect; discreet; heedful; thoughtful; scrupulous; anxious; careful.</syn> <usage> -- <er>Cautious</er>, <er>Wary</er>, <er>Circumspect</er>. A man is <xex>cautious</xex> who realizes the constant possibility of danger; one may be <xex>wary</xex>, and yet bold and active; a man who is <xex>circumspect</xex> habitually examines things on every side in order to weigh and deliberate. It is necessary to be <xex>cautious</xex> at all times; to be <xex>wary</xex> in cases of extraordinary danger; to be <xex>circumspect</xex> in matters of peculiar delicacy and difficulty.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautiously</ent><br/
<hw>Cau"tious*ly</hw>, <pos>adv.</pos> <def>In a cautious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cautiousness</ent><br/
<hw>Cau"tious*ness</hw>, <pos>n.</pos> <def>The quality of being cautious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalcade</ent><br/
<hw>Cav"al*cade`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cavalcade</ets>, fr. It. <ets>cavalcata</ets>, fr. <ets>cavalcare</ets> to go on horseback, fr. LL. <ets>caballicare</ets>, fr. L. <ets>caballus</ets> an inferior horse, Gr. <?/. Cf. <er>Cavalier</er>, <er>Cavalry</er>.]</ety> <def>A procession of persons on horseback; a formal, pompous march of horsemen by way of parade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He brought back war-worn <qex>cavalcade</qex> to the city.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavaliero</ent><br/
<ent>Cavalero</ent><br/
<mhw>{ <hw>Cav`a*le"ro</hw>, <hw>Cav`a*lie"ro</hw> <pr>(k<acr/v`<adot/*l<emac/"ro)</pr>, }</mhw> <pos>n.</pos> <ety>[Sp. <ets>caballero</ets>. See <er>Cavalier</er>.]</ety> <def>A cavalier; a gallant; a libertine.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalier</ent><br/
<hw>Cav`a*lier"</hw> <pr>(k<acr/v`<adot/*l<emac/r")</pr>, <pos>n.</pos> <ety>[F. <ets>cavalier</ets>, It. <ets>cavaliere</ets>, LL. <ets>caballarius</ets>, fr. L. <ets>caballus</ets>. See <er>Cavalcade</er>, and cf. <er>Chevalier</er>, <er>Caballine</er>.]</ety> <sn>1.</sn> <def>A military man serving on horseback; a knight.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gay, sprightly, military man; hence, a gallant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of the court party in the time of king Charles I. as contrasted with a Roundhead or an adherent of Parliament.</def>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Fort.)</fld> <def>A work of more than ordinary height, rising from the level ground of a bastion, etc., and overlooking surrounding parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalier</ent><br/
<hw>Cav`a*lier"</hw>, <pos>a.</pos> <def>offhand; unceremonious; gay; easy; frank.  Opposed to <ant>serious</ant>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>The plodding, persevering scupulous accuracy of the one, and the easy, <qex>cavalier</qex>, verbal fluency of the other, form a complete contrast.</q>  <rj><au>Hazlitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>High-spirited.</def> <mark>[Obs.]</mark> <ldquo/The people are naturally not valiant, and not much <xex>cavalier</xex>.<rdquo/  <rj><au>Suckling.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Supercilious; haughty; disdainful; curt; brusque.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to the party of King Charles I.</def> <ldquo/An old <xex>Cavalier</xex> family.<rdquo/  <rj><au>Beaconsfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalierish</ent><br/
<hw>Cav`a*lier"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat like a cavalier.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalierism</ent><br/
<hw>Cav`a*lier"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practice or principles of cavaliers.</def>  <rj><au>Sir. W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalierly</ent><br/
<hw>Cav`a*lier"ly</hw>, <pos>adv.</pos> <def>In a supercilious, disdainful, or haughty manner; arrogantly.</def>  <rj><au>Junius.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalierness</ent><br/
<hw>Cav`a*lier"ness</hw>, <pos>n.</pos> <def>A disdainful manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavally</ent><br/
<hw>Ca*val"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Pg. <ets>cavalla</ets> a kind of fish; Sp. <ets>caballa</ets>; prob. fr. Pg. <ets>cavallo</ets> horse, Sp. <ets>caballa</ets>.]</ety> <fld>(Zool.)</fld> <def>A carangoid fish of the Atlantic coast (<spn>Caranx hippos</spn>): -- called also <altname>horse crevall<eacute/</altname>. <note>[See <xex>Illust.</xex> under <er>Carangoid</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalry</ent><br/
<hw>Cav"al*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cavalerie</ets>, fr. It. <ets>cavalleria</ets>. See <er>Cavalier</er>, and cf. <er>chivalry</er>.]</ety> <fld>(Mil.)</fld> <def>That part of military force which serves on horseback.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <stype>Heavy cavalry</stype> and <stype>light cavalry</stype> are so distinguished by the character of their armament, and by the size of the men and horses.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavalryman</ent><br/
<hw>Cav"al*ry*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Cavalrymen</er> <pr>(<?/)</pr>.</plu> <def>One of a body of cavalry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cavalry-sword</ent><br/
<hw>cavalry-sword</hw> <pos>n.</pos> <def>a stout sword with a curved blade and thick back.</def><br/
<syn><b>Syn. --</b> saber, sabre.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cavatina</ent><br/
||<hw>Ca`va*ti"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Originally, a melody of simpler form than the <xex>aria</xex>; a song without a second part and a <xex>da capo</xex>; -- a term now variously and vaguely used.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cave</ent><br/
<hw>Cave</hw> <pr>(k<amac/v)</pr>, <pos>n.</pos> <ety>[F. <ets>cave</ets>, L. <ets>cavus</ets> hollow, whence <ets>cavea</ets> cavity. Cf. <er>Cage</er>.]</ety> <sn>1.</sn> <def>A hollow place in the earth, either natural or artificial; a subterraneous cavity; a cavern; a den.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any hollow place, or part; a cavity.</def> <mark>[Obs.]</mark> <ldquo/The <xex>cave</xex> of the ear.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eng. Politics)</fld> <def>A coalition or group of seceders from a political party, as from the Liberal party in England in 1866. See <er>Adullam</er>, <er>Cave of</er>, in the Dictionary of Noted Names in Fiction.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Cave bear</b></col> <fld>(Zool.)</fld>, <cd>a very large fossil bear (<spn>Ursus spel<ae/us</spn>) similar to the grizzly bear, but large; common in European caves.</cd> -- <col><b>Cave dweller</b></col>, <cd>a savage of prehistoric times whose dwelling place was a cave.</cd> <au>Tylor.</au> -- <col><b>Cave hyena</b></col> <fld>(Zool.)</fld>, <cd>a fossil hyena found abundanty in British caves, now usually regarded as a large variety of the living African spotted hyena.</cd> -- <col><b>Cave lion</b></col> <fld>(Zool.)</fld>, <cd>a fossil lion found in the caves of Europe, believed to be a large variety of the African lion.</cd> -- <col><b>Bone cave</b></col>. <cd>See under <er>Bone</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cave</ent><br/
<hw>Cave</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caving</conjf>.]</vmorph> <ety>[Cf. F. <ets>caver</ets>. See <er>Cave</er>, <pos>n.</pos>]</ety> <def>To make hollow; to scoop out.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The mouldred earth <qex>cav'd</qex> the banke.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cave</ent><br/
<hw>Cave</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To dwell in a cave.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[See <ets>To cave in</ets>, below.]</ety> <def>To fall in or down; <as>as, the sand bank <ex>caved</ex></as>. Hence <mark>(Slang)</mark>, to retreat from a position; to give way; to yield in a disputed matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cave in</b></col>. <ety>[Flem. <ets>inkalven</ets>.]</ety> <sd>(a)</sd> <cd>To fall in and leave a hollow, as earth on the side of a well or pit.</cd> <sd>(b)</sd> <cd>To submit; to yield.</cd> <mark>[Slang]</mark>  <rj><au>H. Kingsley.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caveat</ent><br/
||<hw>Ca"ve*at</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caved</ets> let him beware, pres. subj. of <ets>cavere</ets> to be on one's guard to, beware.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Law)</fld> <def>A notice given by an interested party to some officer not to do a certain act until the party is heard in opposition; <as>as, a <ex>caveat</ex> entered in a probate court to stop the proving of a will or the taking out of letters of administration, etc.</as></def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(U. S. Patent Laws)</fld> <def>A description of some invention, designed to be patented, lodged in the patent office before the patent right is applied for, and operating as a bar to the issue of letters patent to any other person, respecting the same invention.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>caveat</xex> is operative for one year only, but may be renewed.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Intimation of caution; warning; protest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We think it right to enter our <qex>caveat</qex> against a conclusion.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Caveat emptor</b></col> <ety>[L.]</ety> <fld>(Law)</fld>, <cd>let the purchaser beware, <it>i. e.</it>, let him examine the article he is buying, and act on his own judgment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caveating</ent><br/
<hw>Ca"ve*a`ting</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Fencing)</fld> <def>Shifting the sword from one side of an adversary's sword to the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caveator</ent><br/
<hw>Ca"ve*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who enters a caveat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavendish</ent><br/
<hw>Cav"en*dish</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Leaf tobacco softened, sweetened, and pressed into plugs or cakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cut cavendish</b></col>, <cd>the plugs cut into long shreds for smoking.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavern</ent><br/
<hw>Cav"ern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caverna</ets>, fr. <ets>cavus</ets> hollow: cf. F. <ets>caverne</ets>.]</ety> <def>A large, deep, hollow place in the earth; a large cave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caverned</ent><br/
<hw>Cav"erned</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Containing caverns.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wolves yelled on the <qex>caverned</qex> hill.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Living in a cavern.</def> <ldquo/<xex>Caverned</xex> hermit.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavernous</ent><br/
<hw>Cav"ern*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cavernosus</ets>: cf. F. <ets>caverneux</ets>.]</ety> <sn>1.</sn> <def>Full of caverns; resembling a cavern or large cavity; hollow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Filled with small cavities or cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having a sound caused by a cavity.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cavernous body</b></col>, <cd>a body of erectile tissue with large interspaces which may be distended with blood, as in the penis or clitoris.</cd> -- <col><b>Cavernous respiration</b></col>, <cd>a peculiar respiratory sound andible on auscultation, when the bronchial tubes communicate with morbid cavities in the lungs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavernulous</ent><br/
<hw>Ca*ver"nu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cavernula</ets>, dim. of <ets>caverna</ets> cavern.]</ety> <def>Full of little cavities; <as>as, <ex>cavernulous</ex> metal</as>.</def>  <rj><au>Black.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavezon</ent><br/
<ent>Cavesson</ent><br/
<mhw>{ <hw>Cav"es*son</hw> <pr>(?)</pr>, <hw>Cav"e*zon</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>cave<cced/on</ets>, augm. fr. LL. <ets>capitium</ets> a head covering hood, fr. L. <ets>caput</ets> head. Cf. <er>Caberzon</er>.]</ety> <fld>(Man.)</fld> <def>A kind of noseband used in breaking and training horses.</def> <altsp>[Written also <asp>caveson</asp>, <asp>causson</asp>.]</altsp>  <rj><au>White.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavetto</ent><br/
||<hw>Ca*vet"to</hw> <pr>(k<adot/*v<ecr/t"t<osl/)</pr>, <pos>n.</pos> <ety>[It. <ets>cavetto</ets>, fr. <ets>cavo</ets> hollow, L. <ets>cavus</ets>.]</ety> <fld>(Arch.)</fld> <def>A concave molding; -- used chiefly in classical architecture.  See <xex>Illust.</xex> of <er>Column</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caviare</ent><br/
<ent>Caviar</ent><br/
<mhw>{ <hw>Cav"i*ar</hw> <pr>(?)</pr>, <hw>Ca*viare"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>caviar</ets>, fr. It. <ets>caviale</ets>, fr. Turk. <ets>Hav<imac/<amac/r</ets>.]</ety> <def>The roes of the sturgeon, prepared and salted; -- used as a relish, esp. in Russia.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Caviare</xex> was considered a delicacy, by some, in Shakespeare's time, but was not relished by most. Hence Hamlet says of a certain play. <ldquo/'T was <xex>caviare</xex> to the general,<rdquo/ <it>i. e.</it>, above the taste of the common people.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 230 --></p>

<p><ent>Cavicorn</ent><br/
<hw>Cav"i*corn</hw> <pr>(k<acr/v"<icr/*k<ocir/rn)</pr>, <pos>a.</pos> <ety>[L. <ets>cavus</ets> hollow + <ets>cornu</ets> horn.]</ety> <fld>(Zool.)</fld> <def>Having hollow horns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavicornia</ent><br/
||<hw>Cav`i*cor"ni*a</hw> <pr>(k<acr/v`<icr/*k<ocir/r"n<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A group of ruminants whose horns are hollow, and planted on a bony process of the front, as the ox.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavil</ent><br/
<hw>Cav"il</hw> <pr>(k<acr/v"<icr/l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Caviled</conjf> <or/ <conjf>Cavilled</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Caviling</conjf> <or/ <conjf>Cavilling</conjf>.]</vmorph> <ety>[L. <ets>cavillari</ets> to practice jesting, to censure, fr. <ets>cavilla</ets> bantering jests, sophistry: cf. OF. <ets>caviller</ets>.]</ety> <def>To raise captious and frivolous objections; to find fault without good reason.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You do not well in obstinacy<br/
To <qex>cavil</qex> in the course of this contract.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavil</ent><br/
<hw>Cav"il</hw>, <pos>v. t.</pos> <def>To cavil at.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavil</ent><br/
<hw>Cav"il</hw>, <pos>n.</pos> <def>A captious or frivolous objection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the <qex>cavils</qex> of prejudice and unbelief.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caviller</ent><br/
<ent>Caviler</ent><br/
<mhw>{ <hw>Cav"il*er</hw> <it>or</it> <hw>Cav"il*ler</hw> <pr>(-<etil/r)</pr>, }</mhw> <pos>n.</pos> <def>One who cavils.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cavilers</qex> at the style of the Scriptures.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caviling</ent><br/
<hw>Cav"il*ing</hw>, <pos>a.</pos> <def>Disposed to cavil; finding fault without good reason. See <er>Captious</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His depreciatory and <qex>caviling</qex> criticism.</q> <rj><qau>Lewis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavilingly</ent><br/
<hw>Cav"il*ing*ly</hw>, <pos>adv.</pos> <def>In a caviling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavillation</ent><br/
<hw>Cav`il*la"tion</hw> <pr>(-l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>cavillation</ets>, L. <ets>cavillatio</ets>.]</ety> <def>Frivolous or sophistical objection.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavillous</ent><br/
<ent>Cavilous</ent><br/
<mhw>{ <hw>Cav"il*ous</hw> <or/ <hw>Cav"il*lous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cavillosus</ets>.]</ety> <def>Characterized by caviling, or disposed to cavil; quibbing.</def> <mark>[R.]</mark></p>

<p>-- <wordforms><wf>Cav"il*ous*ly</wf>, <pos>adv.</pos> <mark>[R.]</mark> -- <wf>Cav"il*ous*ness</wf>, <pos>n.</pos> <mark>[R.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavin</ent><br/
<hw>Cav"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Cave</er>.]</ety> <fld>(Mil.)</fld> <def>A hollow way, adapted to cover troops, and facilitate their aproach to a place.</def>  <rj><au>Farrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavitary</ent><br/
<hw>Cav"i*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Containing a body cavity; <as>as, the <ex>cavitary</ex> or nematoid worms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavity</ent><br/
<hw>Cav"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cavities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>cavus</ets> hollow: cf. F. <ets>cavit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Hollowness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cavity</qex> or hollowness of the place.</q> <rj><qau>Goodwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hollow place; a hollow; <as>as, the abdominal <ex>cavity</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An instrument with a small <qex>cavity</qex>, like a small spoon</q>. <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Abnormal spaces or excavations are frequently formed in the lungs, which are designated <qex>cavities</qex> or vomic<ae/.</q> <rj><qau>Quain.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Body cavity</b></col>, <cd>the c<oe/lum. See under <er>Body</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavo-relievo</ent><br/
<hw>Ca"vo-re*lie"vo</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cavo-rilievo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavo-rilievo</ent><br/
||<hw>Ca"vo-ri*lie"vo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Sculp.)</fld> <def>Hollow relief; sculpture in relief within a sinking made for the purpose, so no part of it projects beyond the plain surface around.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavort</ent><br/
<hw>Ca*vort"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cavorted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cavorting</conjf>.]</vmorph> <def>To prance ostentatiously; -- said of a horse or his rider.</def> <mark>[Local slang, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cavy</ent><br/
<hw>Ca"vy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cavies</plw> <pr>(<?/)</pr>.</plu> <ety>[NL. <ets>cavia</ets>, fr. Brazilian <ets>cabiai</ets>: cf. F. <ets>cabiai</ets>.]</ety> <fld>(Zool.)</fld> <def>A rodent of the genera <gen>Cavia</gen> and <gen>Dolichotis</gen>, as the guinea pig (<spn>Cavia cobaya</spn>). Cavies are natives of South America.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Water cavy</b></col> <fld>(Zool.)</fld>, <cd>The capybara.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caw</ent><br/
<hw>Caw</hw> <pr>(k<add/)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cawed</conjf> <pr>(k<add/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cawing</conjf>.]</vmorph> <ety>[Imitative. <root/22 Cf. <er>Chough</er>.]</ety> <def>To cry like a crow, rook, or raven.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rising and <qex>cawing</qex> at the gun's report.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caw</ent><br/
<hw>Caw</hw>, <pos>n.</pos> <def>The cry made by the crow, rook, or raven.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cawk</ent><br/
<hw>Cawk</hw> <pr>(k<add/k)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>cauk</ets> limestone. A doublet of <ets>chalk</ets>.]</ety> <fld>(Min.)</fld> <def>An opaque, compact variety of barite, or heavy spar.</def> <altsp>[Also written <er>cauk</er>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cawker</ent><br/
<hw>Cawk"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Calker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cawky</ent><br/
<hw>Cawk"y</hw>, <pos>a.</pos> <def>Of or pertaining to cawk; like cawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caxon</ent><br/
<hw>Cax"on</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of wig.</def> <mark>[Obs.]</mark>  <rj><au>Lamb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Caxton</ent><br/
<hw>Cax"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bibliog.)</fld> <def>Any book printed by William <etsep>Caxton</etsep>, the first English printer.</def>  <rj><au>Hansard.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cay</ent><br/
<hw>Cay</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Key</er>, a ledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cayenne</ent><br/
<hw>Cay*enne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <etsep>Cayenne</etsep>, a town and island in French Guiana, South America.]</ety> <def>Cayenne pepper.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cayenne pepper</b></col>. <sd>(a)</sd> <fld>(Bot.)</fld> <cd>A species of <gen>Capsicum</gen> (<spn>Capsicum frutescens</spn>) with small and intensely pungent fruit.</cd> <sd>(b)</sd> <cd>A very pungent spice made by drying and grinding the fruits or seeds of several species of the genus <gen>Capsicum</gen>, esp. <spn>Capsicum annuum</spn> and <spn>Capsicum Frutescens</spn>; -- called also <altname>red pepper</altname>. It is used chiefly as a condiment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cayman</ent><br/
<hw>Cay"man</hw> <pr>(k<amac/"m<ait/n)</pr>, <pos>n.</pos> <ety>[From the language of Guiana: cf. Sp. <ets>caiman</ets>.]</ety> <fld>(Zool.)</fld> <def>The south America alligator. See <er>Alligator</er>.</def> <altsp>[Sometimes written <asp>caiman</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cayo</ent><br/
||<hw>Ca"yo</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-yos</plw> <pr>(#)</pr></plu>. <ety>[Sp.]</ety> <def>A small island or ledge of rock in the water; a key.</def> <mark>[Sp. Am.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cayugas</ent><br/
<hw>Ca*yu"gas</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Cayuga</singw>.</sing> <fld>(Ethnol.)</fld> <def>A tribe of Indians formerly inhabiting western New-York, forming part of the confederacy called the Five Nations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cayuse</ent><br/
<hw>Cay*use"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An Indian pony.</def> <mark>[Northw. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cazic</ent><br/
<ent>Cazique</ent><br/
<mhw>{ <hw>Ca*zique"</hw>, <hw>Ca*zic"</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>Cacique</ets>, fr. the language of Haiti.]</ety> <def>A chief or petty king among some tribes of Indians in America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>CB</ent><br/
<hw>CB</hw> <pos>n.</pos> <def>same as <er>citizens' band</er>; that portion of the radio frequency spectrum allocated by the FCC for the use of individual citizens for short-distance personal or business use, from either fixed or mobile stations.  Also used attributively, as <col><b>CB radio</b></col>.</def> <mark>[acronym]</mark> <br/
<syn><b>Syn. --</b> Citizens' Band.</syn><br/
[<source>PJC</source>]</p>

<p><ent>CBC</ent><br/
<hw>CBC</hw> <pos>n.</pos> <def>the complete blood count; a clinical test which counts the number of white and red blood cells and the number of platelets in one cubic milimeter of blood.</def> <mark>[acronym]</mark> <br/
<syn><b>Syn. --</b> complete blood count, blood profile.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cd</ent><br/
<hw>Cd</hw> <pos>n.</pos> <sn>1.</sn> <def>the chemical suymbol for <er>cadmium</er>, a metallic element of atomic number 48.</def><br/
<syn><b>Syn. --</b> cadmium.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the abbreviation for the <er>candela</er>, the basic unit of luminous intensity adopted under the System International d'Unites.</def> <mark>[abbr.]</mark> <br/
<syn><b>Syn. --</b> candle, candela, standard candle.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>CD</ent><br/
<hw>CD</hw> <pos>n.</pos> <sn>1.</sn> <def>an acronym for <er>compact disk</er>, a disk-shaped recording of binary data that is smaller than a phonograph record, and is recorded and played back by a <er>compact disk player</er>, an electronic device containing  a laser.  The term is also used generically to refer to the medium as a data storage medium.</def><br/
<syn><b>Syn. --</b> compact disc, compact disk.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a <er>compact disk player</er>.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>an acronym for <er>certificate of deposit</er>, a debt instrument issued by a bank for a fixed time period, usually paying interest.</def><br/
<syn><b>Syn. --</b> certificate of deposit.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>CD-R</ent><br/
<hw>CD-R</hw> <pos>n.</pos>  <ety>[from <ets>C</ets>ompact <ets>D</ets>isk-<ets>R</ets>ecordable.]</ety> <def>a recordable compact disk; a compact disc on which you can write only once and thereafter is read-only.</def><br/
<syn><b>Syn. --</b> compact disc recordable, CD-WO, compact disc write-once.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>CD-ROM</ent><br/
<hw>CD-ROM</hw> <pos>n.</pos> <def>a <isa>compact disk</isa> that is used with a computer (rather than with an audio system); a large amount of digital information can be stored and accessed but it cannot be altered by the user.</def><br/
<syn><b>Syn. --</b> compact disc read-only memory.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>CD-WO</ent><br/
<hw>CD-WO</hw> <pos>n.</pos> <ety>[from <ets>C</ets>ompact <ets>D</ets>isk-<ets>W</ets>rite <ets>O</ets>nce.]</ety> <def>A recordable compact disk; a compact disc on which you can write only once and thereafter is read-only.</def><br/
<syn><b>Syn. --</b> CD-R, compact disc recordable, compact disc write-once.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ce</ent><br/
<hw>Ce</hw> <pos>n.</pos> <def>the chemical symbol for <er>cerium</er>, the most abundant <isa>element</isa> of the rare-earth group.</def><br/
<syn><b>Syn. --</b> cerium.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cease</ent><br/
<hw>Cease</hw> <pr>(s<emac/s)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Ceased</conjf> <pr>(s<emac/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Ceasing</conjf>.]</vmorph> <ety>[OE. <ets>cessen</ets>, <ets>cesen</ets>, F. <ets>cesser</ets>, fr. L. <ets>cessare</ets>, v. intensive fr. <ets>cedere</ets> to withdraw. See <er>Cede</er> , and cf. <er>Cessation</er>.]</ety> <sn>1.</sn> <def>To come to an end; to stop; to leave off or give over; to desist; <as>as, the noise <ex>ceased</ex></as>.</def> <ldquo/To <ex>cease</ex> from strife.<rdquo/  <rj><au>Prov. xx. 3.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be wanting; to fail; to pass away.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The poor shall never <qex>cease</qex> out of the land.</q> <rj><qau>Deut. xv. 11.</qau></rj></p>

<p><syn><b>Syn.</b> -- To intermit; desist; stop; abstain; quit; discontinue; refrain; leave off; pause; end.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cease</ent><br/
<hw>Cease</hw>, <pos>v. t.</pos> <def>To put a stop to; to bring to an end.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But he, her fears to <qex>cease</qex><br/
Sent down the meek-eyed peace.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cease</qex>, then, this impious rage.</q> <rj><qau>Milton</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cease</ent><br/
<hw>Cease</hw>, <pos>n.</pos> <def>Extinction.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceaseless</ent><br/
<hw>Cease"less</hw>, <pos>a.</pos> <def>Without pause or end; incessant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceaseless</ent><br/
<hw>Cease"less</hw>, <pos>adv.</pos> <def>Without intermission or end.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cebidae</ent><br/
<hw>Cebidae</hw> <pos>prop. n.</pos> <def>A natural family including all the New World monkeys except marmosets and tamarins.</def><br/
<syn><b>Syn. --</b> family <fam>Cebidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cebu</ent><br/
<hw>Cebu</hw> <pos>prop. n.</pos> <sn>1.</sn> <def>one of the Visayan islands of the central Philippines, important for its fine harbor.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an important seaport on the Island of Cebu in the Philippines.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cebuan</ent><br/
<hw>Cebuan</hw> <pos>prop. n.</pos> <sn>1.</sn> <def>An inhabitant of the island of Cebu; a member of the Visayan (or Bisayan) people of the Philippines.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>The language of the people of Cebu in the Philippines; its lexicon contributes to the official language of the Philippines.</def><br/
<syn><b>Syn. --</b> Cebuano.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cebuano</ent><br/
<hw>Cebuano</hw> <pos>prop. n.</pos> <def>The language of the people of Cebu in the Philippines; its lexicon contributes to the official languag of the Philippines.</def><br/
<syn><b>Syn. --</b> Cebuan.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cebuella</ent><br/
<hw>Cebuella</hw> <pos>prop. n.</pos> <def>A genus of pygmy marmosets.</def><br/
<syn><b>Syn. --</b> genus <gen>Cebuella</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cebus</ent><br/
<hw>Cebus</hw> <pos>prop. n.</pos> <def>the type genus of the <fam>Cebidae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Cebus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cecal</ent><br/
<hw>cecal</hw> <pos>adj.</pos> <def>of, pertaining to, or like a cecum.</def><br/
<syn><b>Syn. --</b> caecal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cecidomyia</ent><br/
||<hw>Cec`i*do*my"i*a</hw> <pr>(s<ecr/s`<icr/*d<osl/*m<imac/"<icr/*<adot/)</pr>, <pos>prop. n.</pos> <ety>[Nl., fr. Gr. <grk>khki`s</grk>, <grk>khki^dos</grk>, a gall nut + <grk>myi^a</grk> a fly.]</ety> <fld>(Zool.)</fld> <def>A genus of small dipterous files, including several very injurious species, as the <stype>Hessian fly</stype>. See <er>Hessian fly</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cecidomyidae</ent><br/
<hw>Cecidomyidae</hw> <pos>prop. n.</pos> <def>A natural family comprising the gall midges.</def><br/
<syn><b>Syn. --</b> family <fam>Cecidomyidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cecity</ent><br/
<hw>ce"ci*ty</hw> <pr>(s<emac/"s<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>caecitas</ets>, fr. <ets>caecus</ets> blind: cf. F. <ets>c<eacute/cit<eacute/</ets>.]</ety> <def>Blindness.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cecropiaceae</ent><br/
<hw>Cecropiaceae</hw> <pos>prop. n.</pos> <def>A natural family of insects, in some classifications included in family <fam>Moraceae</fam>.</def><br/
<syn><b>Syn. --</b> family <fam>Cecropiaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cecum</ent><br/
<hw>cecum</hw> <pos>n.</pos> <def>The <er>caecum</er>, the cavity in which the large intestine begins and into which the ileum opens.</def><br/
<syn><b>Syn. --</b> caecum, blind gut.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cecutiency</ent><br/
<hw>ce*cu"tien*cy</hw> <pr>(s<esl/*k<umac/"*sh<eit/n*s<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>caecutire</ets> to be blind, fr. <ets>caecus</ets> blind.]</ety> <def>Partial blindness, or a tendency to blindness.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cedar</ent><br/
<hw>ce"dar</hw> <pr>(s<emac/"d<etil/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>ceder</ets>, fr. L. <ets>cedrus</ets>, Gr. <grk>ke`dros</grk>.]</ety> <fld>(Bot.)</fld> <def>The name of several evergreen trees.  The wood is remarkable for its durability and fragrant odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The cedar of Lebanon is the <spn>Cedrus Libani</spn>; the white cedar (<spn>Cupressus thyoides</spn>) is now called <spn>Cham<oe/cyparis sph<ae/roidea</spn>; American red cedar is the <spn>Juniperus Virginiana</spn>; Spanish cedar, the West Indian <spn>Cedrela odorata</spn>. Many other trees with odoriferous wood are locally called <ex>cedar</ex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedar</ent><br/
<hw>Ce"dar</hw>, <pos>a.</pos> <def>Of or pertaining to cedar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cedarbird</ent><br/
<ent>cedar bird</ent><br/
<mhw><hw>ce"dar bird`</hw>, <hw>ce"dar*bird`</hw></mhw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>cedar waxwing</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>cedar waxwing</ent><br/
<hw>ce"dar wax"wing</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>a species of chatterer (<spn>Bombycilla cedrorum</spn>, formerly <spn>Ampelis cedrorum</spn>) widely distributed over temperate North America, so named from its frequenting cedar trees; -- called also <altname>cedar bird</altname>, <altname>cherry bird</altname>, <altname>Canada robin</altname>, and <altname>American waxwing</altname>.  It is a brownish bird about 7 inches long, between the size of a robin and a sparrow, has a crest on the head, a black face mask, and a yellow-tipped tail.  The name comes from the black color of the tips of the wings, like that of a black sealing wax.  They sometimes are seen in flocks.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>cedarwood</ent><br/
<hw>ce"dar*wood`</hw> <pos>n.</pos> <def>The durable aromatic wood of any of numerous cedar and cedarlike trees; especially the wood of the red cedar, often used for cedar chests.</def><br/
<syn><b>Syn. --</b> cedar.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cedared</ent><br/
<hw>ce"dared</hw> <pr>(s<emac/"d<etil/rd)</pr>, <pos>a.</pos> <def>Covered, or furnished with, cedars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cedarn</ent><br/
<hw>ce"darn</hw> <pr>(s<emac/"d<etil/rn)</pr>, <pos>a.</pos> <def>Of or pertaining to the cedar or its wood.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cede</ent><br/
<hw>cede</hw> <pr>(s<emac/d)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>ceded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>ceding</conjf>.]</vmorph> <ety>[L. <ets>cedere</ets> to withdraw, yield; akin to <ets>cadere</ets> to fall, and to E. <ets>chance</ets>; cf. F. <ets>c<eacute/der</ets>.]</ety> <def>To yield or surrender; to give up; to resign; <as>as, to <ex>cede</ex> a fortress, a province, or country, to another nation, by treaty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The people must <xex>cede</xex> to the government some of their natural rights.</q> <rj><qau>Jay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cedilla</ent><br/
<hw>ce*dil"la</hw> <pr>(s<esl/*d<icr/l"l<adot/)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cedilla</ets>, cf. F. <ets>c<eacute/dille</ets>; dim. of <ets>zeta</ets>, the Gr. name of the letter <ets>z</ets>, because this letter was formerly written after the <ets>c</ets>, to give it the sound of <ets>s</ets>.]</ety> <def>A mark placed under the letter <xex>c</xex> [thus, <cced/], to show that it is to be sounded like <xex>s</xex>, as in <xex>fa<cced/ade</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedrat</ent><br/
<hw>Ce"drat</hw> <pr>(s<emac/"dr<acr/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>c<eacute/drat</ets>. See <er>Cedar</er>.]</ety> <fld>(Bot.)</fld> <def>Properly the citron, a variety of <spn>Citrus medica</spn>, with large fruits, not acid, and having a high perfume.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedrene</ent><br/
<hw>Ce"drene</hw> <pr>(s<emac/"dr<emac/n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A rich aromatic oil, <chform>C15H24</chform>, extracted from oil of red cedar, and regarded as a polymeric terpene; also any one of a class of similar substances, as the essential oils of cloves, cubebs, juniper, etc., of which cedrene proper is the type.</def> <altsp>[Written also <asp>cedren</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedrine</ent><br/
<hw>Ce"drine</hw> <pr>(s<emac/"dr<icr/n; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>cedrinus</ets>, Gr. <grk>ke`drinos</grk>. See <er>Cedar</er>.]</ety> <def>Of or pertaining to cedar or the cedar tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedriret</ent><br/
<hw>Ce"dri*ret</hw> <pr>(s<emac/"dr<icr/*r<ecr/t)</pr>, <pos>n.</pos> <def>Same as <er>C<oe/rulignone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedry</ent><br/
<hw>Ce"dry</hw> <pr>(s<emac/"dr<ycr/)</pr>, <pos>a.</pos> <def>Of the nature of cedar.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cedule</ent><br/
<hw>Ced"ule</hw> <pr>(s<ecr/d"<usl/l)</pr>, <pos>n.</pos> <ety>[F. <ets>c<eacute/dule</ets>, fr. L. <ets>shedula</ets>. See <er>Shedule</er>.]</ety> <def>A scroll; a writing; a schedule.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceduous</ent><br/
<hw>Ced"u*ous</hw> <pr>(s<ecr/d"<usl/*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>caeduus</ets>, fr. <ets>caedere</ets> to cut down.]</ety> <def>Fit to be felled.</def> <mark>[Obs.]</mark>  <rj><au>Eyelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceiba</ent><br/
<hw>Ceiba</hw> <pos>n.</pos> <def>a genus of tropical American trees with palmately compound leaves and showy bell-like flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Ceiba</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>ceibo</ent><br/
<hw>ceibo</hw> <pos>n.</pos> <def>a small South American spiny tree (<spn>Erythrina crista-galli</spn>) with dark crimson and scarlet flowers solitary or clustered.</def><br/
<syn><b>Syn. --</b> crybaby tree, cry-baby tree, common coral tree, <spn>Erythrina crista-galli</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceil</ent><br/
<hw>Ceil</hw> <pr>(s<emac/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Ceiled</conjf> <pr>(s<emac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Ceiling</conjf>.]</vmorph> <ety>[From an older noun, fr. F. <ets>ciel</ets> heaven, canopy, fr. L. <ets>caelum</ets> heaven, vault, arch, covering; cf. Gr. <grk>koi^los</grk> hollow.]</ety> <sn>1.</sn> <def>To overlay or cover the inner side of the roof of; to furnish with a ceiling; <as>as, to <ex>ceil</ex> a room</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The greater house he <qex>ceiled</qex> with fir tree.</q> <rj><qau>2 Chron. iii. 5</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To line or finish a surface, as of a wall, with plaster, stucco, thin boards, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceiling</ent><br/
<hw>Ceil"ing</hw>, <pos>n.</pos> <ety>[See <er>Cell</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The inside lining of a room overhead; the under side of the floor above; the upper surface opposite to the floor.</def> <sd>(b)</sd> <def>The lining or finishing of any wall or other surface, with plaster, thin boards, etc.; also, the work when done.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>The inner planking of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Camp ceiling</b></col>. <cd>See under <er>Camp</er>.</cd> -- <col><b>Ceiling boards</b></col>, <cd>Thin narrow boards used to ceil with.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceint</ent><br/
<hw>Ceint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cincture</er>.]</ety> <def>A girdle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceinture</ent><br/
||<hw>Cein`ture"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A cincture, girdle, or belt; -- chiefly used in English as a dressmaking term.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Celadon</ent><br/
<hw>Cel"a*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A pale sea-green color; also, porcelain or fine pottery of this tint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celandine</ent><br/
<hw>Cel"an*dine</hw> <pr>(s<ecr/l"<acr/n*d<imac/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>celidoine</ets>, OF. <ets>celidoine</ets>, F. <ets>ch<eacute/lidoine</ets>, fr. L. <ets>chelidonia</ets> (sc. <ets>herba</ets>), fr. <ets>chelidonius</ets> pertaining to the swallow, Gr. <grk>chelido`nios</grk>, fr. <grk>chelidw`n</grk> the swallow, akin to L. <ets>hirundo</ets> a swallow.]</ety> <fld>(Bot.)</fld> <def>A perennial herbaceous plant (<spn>Chelidonium majus</spn>) of the poppy family, with yellow flowers. It is used as a medicine in jaundice, etc., and its acrid saffron-colored juice is used to cure warts and the itch; -- called also <altname>greater celandine</altname> and <altname>swallowwort</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lasser celandine</b></col>, <cd>the pilewort (<spn>Ranunculus Ficaria</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celastrus</ent><br/
<hw>Celastrus</hw> <pos>n.</pos> <def>the type genus of the <fam>Celastraceae</fam>, comprising the plants called <altname>climbing bittersweet</altname>; bittersweet and shrubby bittersweet; they are woody vines and shrubs native chiefly to Asia and Australia, bearing yellow to orange capsules which open to expose red-coated seeds.  See also <er>bittersweet{3b}</er>.</def><br/
<syn><b>Syn. --</b> genus <gen>Celastrus</gen>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Celature</ent><br/
<hw>Cel"a*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caelatura</ets>, fr. <ets>caelare</ets> to engrave in relief.]</ety> <sn>1.</sn> <def>The act or art of engraving or embossing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is engraved.</def> <mark>[Obs.]</mark>  <rj><au>Hakewill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebrant</ent><br/
<hw>Cel"e*brant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>celebrans</ets>, p. pr. of <ets>celebrare</ets>. See <er>Celebrate</er>.]</ety> <def>One who performs a public religious rite; -- applied particularly to an officiating priest in the Roman Catholic Church, as distinguished from his assistants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebrate</ent><br/
<hw>Cel"e*brate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Celebrated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Celebrating</conjf>.]</vmorph> <ety>[L. <ets>celebratus</ets>, p. p. of <ets>celebrare</ets> to frequent, to celebrate, fr. <ets>celeber</ets> famous.]</ety> <sn>1.</sn> <def>To extol or honor in a solemn manner; <as>as, to <ex>celebrate</ex> the name of the Most High</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To honor by solemn rites, by ceremonies of joy and respect, or by refraining from ordinary business; to observe duly; to keep; <as>as, to <ex>celebrate</ex> a birthday</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From even unto even shall ye <qex>celebrate</qex> your Sabbath.</q> <rj><qau>Lev. xxiii. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To perform or participate in, as a sacrament or solemn rite; to solemnize; to perform with appropriate rites; <as>as, to <ex>celebrate</ex> a marriage</as>.</def></p>

<p><syn><b>Syn.</b> -- To commemorate; distinguish; honor.</syn> <usage> -- To <er>Celebrate</er>, <er>Commemorate</er>. We <xex>commemorate</xex> events which we desire to keep in remembrance, when we recall them by some special observace; as, to <xex>commemorate</xex> the death of our Savior. We <xex>celebrate</xex> by demonstrations of joy or solemnity or by appropriate ceremonies; as, to <xex>celebrate</xex> the birthday of our Independence.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>We are called upon to <qex>commemorate</qex> a revolution as surprising in its manner as happy in its consequences.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Earth, water, air, and fire, with feeling glee,<br/
Exult to <qex>celebrate</qex> thy festival.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebrated</ent><br/
<hw>Cel"e*bra`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having celebrity; distinguished; renowned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Celebrated</qex> for the politeness of his manners.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- Distinguished; famous; noted; famed; renowned; illustrious. See <er>Distinguished</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebration</ent><br/
<hw>Cel`e*bra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>celebratio</ets>.]</ety> <def>The act, process, or time of celebrating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His memory deserving a particular <qex>celebration</qex>.</q> <rj><qau>Clarendok.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Celebration</qex> of Mass is equivalent to offering Mass</q> <rj><qau>Cath. Dict.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To hasten the <qex>celebration</qex> of their marriage.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebrator</ent><br/
<hw>Cel"e*bra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who celebrates; a praiser.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebrious</ent><br/
<hw>Ce*le"bri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Famous.</def> <mark>[Obs.]</mark>  <rj><au>Speed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celebrity</ent><br/
<hw>Ce*leb"ri*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Celebrities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>celebritas</ets>: cf. F. <ets>c<eacute/l<eacute/brit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Celebration; solemnization.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>celebrity</qex> of the marriage.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state or condition of being celebrated; fame; renown; <as>as, the <ex>celebrity</ex> of Washington</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An event of great <qex>celebrity</qex> in the history of astronomy.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person of distinction or renown; -- usually in the plural; <as>as, he is one of the <ex>celebrities</ex> of the place</as>.</def><-- " a person who is famous for being famous" --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celeriac</ent><br/
<hw>Ce*le"ri*ac</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Turnip-rooted celery, a from of celery with a large globular root, which is used for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celerity</ent><br/
<hw>Ce*ler"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>celeritas</ets>, from <ets>celer</ets> swiftm speedy: sf. F. <ets>c<eacute/l<eacute/rit<eacute/</ets>.]</ety> <def>Rapidity of motion; quickness; swiftness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Time, with all its <qex>celerity</qex>, moves slowly to him whose whole employment is to watch its flight.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celery</ent><br/
<hw>Cel"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>c<eacute/leri</ets>, cf. Prov. It. <ets>seleno</ets>, <ets>seler</ets>; fr. Gr. <?/ parsley, in Lgr. & NGr. <ets>celery</ets>. <ets>Cf</ets>. <er>Parsley</er>.]</ety> <fld>(Bot.)</fld> <def>A plant of the Parsley family (<spn>Apium graveolens</spn>), of which the blanched leafstalks are used as a salad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>celesta</ent><br/
<hw>celesta</hw> <pos>n.</pos> <fld>(Mus.)</fld> <def>a musical instrument consisting of graduated steel plates that are struck by hammers activated by a keyboard.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Celestial</ent><br/
<hw>Ce*les"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>celestial</ets>, <ets>celestied</ets>, fr. L. <ets>caelestic</ets>, fr. <ets>caelum</ets> heaved. See <er>Cell</er>.]</ety> <sn>1.</sn> <def>Belonging to the a<eum/rial regions, or visible heavens.</def> <ldquo/The twelve <xex>celestial</xex> signs.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the spiritual heaven; heavenly; divine.</def>  <ldquo/<xex>Celestial</xex> spirits.<rdquo/ <ldquo/<xex>Celestial</xex> light,<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of, pertaining to, or characteristic of, the Chinese, or Celestial, Empire, of the Chinese people.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Celestial city</b></col>, <cd>heaven; the heavenly Jerusalem.</cd> <au>Bunyan.</au> -- <col><b>Celestial empire</b></col>, <cd>China; -- so called from the Chinese words, <xex>tien chan</xex>, Heavenly Dynasty, as being the kingdom ruled over by the dynasty appointed by heaven.</cd>  <rj><au>S. W. Williams.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celestial</ent><br/
<hw>Ce*les"tial</hw>, <pos>n.</pos> <sn>1.</sn> <def>An inhabitant of heaven.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A native of China; a Chinaman; a Chinese.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Celestialize</ent><br/
<hw>Ce*les"tial*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make celestial.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celestially</ent><br/
<hw>Ce*les"tial*ly</hw>, <pos>adv.</pos> <def>In a celestial manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celestify</ent><br/
<hw>Ce*les"ti*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>caelestis</ets> heavenly + <ets>-fly</ets>.]</ety> <def>To make like heaven.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celestite</ent><br/
<ent>Celestine</ent><br/
<mhw>{ <hw>Cel"es*tine</hw> <pr>(?)</pr>, <hw>Cel"es*tite</hw> <pr>(?)</pr>, }</mhw>, <pos>n.</pos> <ety>[LL. <ets>caelestinus</ets> bine.]</ety> <fld>(Min.)</fld> <def>Native strontium sulphate, a mineral so named from its occasional delicate blue color. It occurs crystallized, also in compact massive and fibrous forms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celestinian</ent><br/
<ent>Celestine</ent><br/
<mhw>{ <hw>Cel"es*tine</hw> <pr>(?)</pr>, <hw>Cel`es*tin"i*an</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A monk of the austere branch of the Franciscan Order founded by Celestine V. in the 13th centry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celiac</ent><br/
<hw>Ce"li*ac</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>See C<oe/llac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celibacy</ent><br/
<hw>Ce*lib"a*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Celibate</er>, <pos>n.</pos>]</ety> <def>The state of being unmarried; single life, esp. that of a bachelor, or of one bound by vows not to marry.</def> <ldquo/The <xex>celibacy</xex> of the clergy.<rdquo/  <rj><au>Hallom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celibate</ent><br/
<hw>Cel"i*bate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>aelibatus</ets>, fr. <ets>caelebs</ets> unmarried, single.]</ety> <sn>1.</sn> <def>Celibate state; celibacy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . preferreth holy <qex>celibate</qex> before the estate of marriage.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is unmarried, esp. a bachelor, or one bound by vows not to marry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celibate</ent><br/
<hw>Cel"i*bate</hw>, <pos>a.</pos> <def>Unmarried; single; <as>as, a <ex>celibate</ex> state</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celibatist</ent><br/
<hw>Ce*lib"a*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lives unmarried.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celidography</ent><br/
<hw>Cel`i*dog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/ stain, spot + <ets>-graphy</ets>: cf. F. <ets>c<eacute/lidographie</ets>.]</ety> <def>A description of apparent spots on the disk of the sun, or on planets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cell</ent><br/
<hw>Cell</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>celle</ets>, fr. L. <ets>cella</ets>; akin to <ets>celare</ets> to hide, and E. <ets>hell</ets>, <ets>helm</ets>, <ets>conceal</ets>. Cf. <er>Hall</er>.]</ety> <sn>1.</sn> <def>A very small and close apartment, as in a prison or in a monastery or convent; the hut of a hermit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The heroic confessor in his <qex>cell</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small religious house attached to a monastery or convent.</def> <ldquo/<xex>Cells</xex> or dependent priories.<rdquo/  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any small cavity, or hollow place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The space between the ribs of a vaulted roof.</def> <sd>(b)</sd> <def>Same as <er>Cella</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Elec.)</fld> <def>A jar of vessel, or a division of a compound vessel, for holding the exciting fluid of a battery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Biol.)</fld> <def>One of the minute elementary structures, of which the greater part of the various tissues and organs of animals and plants are composed.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ All cells have their origin in the primary cell from which the organism was developed. In the lowest animal and vegetable forms, one single cell constitutes the complete individual, such being called <xex>unicelluter orgamisms</xex>. A typical cell is composed of a semifluid mass of protoplasm, more or less granular, generally containing in its center a nucleus which in turn frequently contains one or more nucleoli, the whole being surrounded by a thin membrane, the cell wall. In some cells, as in those of blood, in the am<oe/ba, and in embryonic cells (both vegetable and animal), there is no restricting cell wall, while in some of the unicelluliar organisms the nucleus is wholly wanting. See <xex>Illust.</xex> of <er>Bipolar</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Air cell</b></col>. <cd>See <er>Air cell</er>.</cd> -- <col><b>Cell development</b></col> <cd>(called also <altname>cell genesis</altname>, <altname>cell formation</altname>, and <altname>cytogenesis</altname>), the multiplication, of cells by a process of reproduction under the following common forms; <xex>segmentation</xex> or <xex>fission</xex>, <xex>gemmation</xex> or <xex>budding</xex>, <xex>karyokinesis</xex>, and <xex>endogenous multiplication</xex>. See <er>Segmentation</er>, <er>Gemmation</er>, etc.</cd> -- <col><b>Cell theory</b></col>. <fld>(Biol.)</fld> <cd>See <cref>Cellular theory</cref>, under <er>Cellular</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 231 --></p>

<p><ent>Cell</ent><br/
<hw>Cell</hw> <pr>(s<ecr/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Celled</conjf> <pr>(s<ecr/ld)</pr>.]</vmorph> <def>To place or inclose in a cell.</def> <ldquo/<xex>Celled</xex> under ground.<rdquo/ <mark>[R.]</mark>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cella</ent><br/
||<hw>Cel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Arch.)</fld> <def>The part inclosed within the walls of an ancient temple, as distinguished from the open porticoes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellar</ent><br/
<hw>Cel"lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>celer</ets>, OF. <ets>celier</ets>, F. <ets>celier</ets>, fr. L. <ets>cellarium</ets> a receptacle for food, pantry, fr. <ets>cella</ets> storeroom. See <er>Cell</er>.]</ety> <def>A room or rooms under a building, and usually below the surface of the ground, where provisions and other stores are kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellarage</ent><br/
<hw>Cel"lar*age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The space or storerooms of a cellar; a cellar.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You hear this fellow in the <qex>cellarage</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Chare for storage in a cellar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellarer</ent><br/
<hw>Cel"lar*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>cellararius</ets>, equiv. to L. <ets>cellarius</ets> steward: cf. F. <ets>cell<eacute/rier</ets>. See <er>Cellar</er>.]</ety> <fld>(Eccl.)</fld> <def>A steward or butler of a monastery or chapter; one who has charge of procuring and keeping the provisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellaret</ent><br/
<hw>Cel`lar*et"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim of <ets>cellar</ets>.]</ety> <def>A receptacle, as in a dining room, for a few bottles of wine or liquor, made in the form of a chest or coffer, or a deep drawer in a sideboard, and usually lined with metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellarist</ent><br/
<hw>Cel"lar*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Cellarer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celled</ent><br/
<hw>Celled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing a cell or cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellepore</ent><br/
<hw>Cel"le*pore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cella</ets> cell + <ets>porus</ets>, Gr. <?/, passage.]</ety> <fld>(Zool.)</fld> <def>A genus of delicate branching corals, made up of minute cells, belonging to the Bryozoa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celliferous</ent><br/
<hw>Cel*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cell</ets> + <ets>-ferous</ets>.]</ety> <def>Bearing or producing cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cell-like</ent><br/
<hw>cell-like</hw> <pos>adj.</pos> <def>resembling a cell.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cello</ent><br/
||<hw>Cel"lo</hw> <pr>(ch<ecr/l"l<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Cellos</plw> <pr>(ch<ecr/l"l<osl/z)</pr>, It. <plw>Celli</plw> <pr>(ch<ecr/l"l<emac/)</pr>.</plu> <def>A contraction for <er>Violoncello</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cellophane</ent><br/
<hw>cellophane</hw> <pr>(s<ecr/l"<osl/*f<amac/n`)</pr>, <pos>n.</pos> <def>a transparent paper-like product made of regenerated cellulose, produced in sheets and rolls, which is impervious to moisture and germs, and which is used to wrap candy, cigarettes, and a wide variety of other products for distribution and retail sale; <as>as, a pack of cigarettes in a <ex>cellophane</ex> wrapper</as>.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cellular</ent><br/
<hw>Cel"lu*lar</hw> <pr>(s<ecr/l"<ucr/*l<etil/r; 135)</pr>, <pos>a.</pos> <ety>[L. <ets>cellula</ets> a little cell: cf. F. <ets>cellulaire</ets>. See <er>Cellule</er>.]</ety> <sn>1.</sn> <def>Consisting of, or containing, cells; of or pertaining to a cell or cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>porous; containing cavities.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>pertaining to or using a system of transmission of telephone signals by radio, in which areas are divided into geographical parts (cells), each of which is served by a transmitter whose range is limited to that region, thus permitting a single transmission frequency to be used simulataneously in different parts of the same area.  <ex>Cellular</ex> telephones are typically small and battery powered, allowing a subscriber with such a telephone to carry the telephone in a pocket or purse, over the entire area served, and to be contacted by a single telephone number.  The system became widespread and popular in the 1980's and 1990's; <as>as, <ex>cellular</ex> telephones sometimes lose their link unpredictably</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Cellular plants</b></col>, <col><b>Cellular cryptogams</b></col></mcol> <fld>(Bot.)</fld>, <cd>those flowerless plants which have no ducts or fiber in their tissue, as mosses, fungi, lichens, and alg<ae/.</cd> -- <mcol><col><b>Cellular theory</b></col>, <it>or</it> <col><b>Cell theory</b></col></mcol> <fld>(Biol.)</fld>, <cd>a theory, according to which the essential element of every tissue, either vegetable or animal, is a cell; the whole series of cells having been formed from the development of the germ cell and by differentiation converted into tissues and organs which, both in plants and animals, are to be considered as a mass of minute cells communicating with each other.</cd> -- <col><b>Cellular tissue</b></col>. <sd>(a)</sd> <fld>(Anat.)</fld> <cd>See <cref>conjunctive tissue</cref> under <er>Conjunctive</er>.</cd> <sd>(b)</sd> <fld>(Bot.)</fld> <cd>Tissue composed entirely of parenchyma, and having no woody fiber or ducts.</cd></cs>
<-- <col><b>cellular telephone</b></col>,  <cd>a portable radio-telephone transmitting and receiving the radio-telephonic signals from one of a group of transmitter-receiver stations so arranged that they provide adequate signal contact for such telephones over a certain geographical area.  The area within which one transmitter may service such portable telephones is called its <ldquo/cell<rdquo/.</cd> -- <col><b>cellular slime mold</b></col> <cd>a myxomycete differing from the true slime molds in being cellular throughout the life cycle.  See <er>slime mold</er>.</cd> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>cellularity</ent><br/
<hw>cellularity</hw> <pos>n.</pos> <def>the state of having cells.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cellulated</ent><br/
<hw>Cel"lu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cellular.</def>  <rj><au>Caldwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellule</ent><br/
<hw>Cel"lule</hw> <pr>(s<ecr/l"<umac/l)</pr>, <pos>n.</pos> <ety>[L. <ets>cellula</ets> a small apartment, dim. of <ets>cella</ets>: cf. F. <ets>cellule</ets>. See <er>Cell</er>.]</ety> <def>A small cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celluliferous</ent><br/
<hw>Cel`lu*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cellula</ets> + <ets>-ferous</ets>.]</ety> <def>Bearing or producing little cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cellulite</ent><br/
<hw>cel"lu*lite`</hw> <pr>(s<ecr/l"<ucr/*l<imac/t` <it>or</it> s<ecr/l"<ucr/*l<emac/t`)</pr>, <pos>n.</pos> <def>small lumpy deposits of body fat esp. on women's thighs and buttocks.  Not used as a technical term.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cellulitis</ent><br/
||<hw>Cel`lu*li"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>cellula</ets> + <ets>-itis</ets>.]</ety> <def>An inflammantion of the cellular or areolar tissue, esp. of that lying immediately beneath the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celluloid</ent><br/
<hw>Cel"lu*loid`</hw> <pr>(s<ecr/l"<usl/*loid)</pr>, <pos>n.</pos> <ety>[<ets>Cellul</ets>ose + <ets>-oid</ets>.]</ety> <def>A substance composed essentially of gun cotton and camphor, and when pure resembling ivory in texture and color, but variously colored to imitate coral, tortoise shell, amber, malachite, etc. It is used in the manufacture of jewelry and many small articles, as combs, brushes, collars, and cuffs; -- originally called <altname>xylonite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellulose</ent><br/
<hw>Cel"lu*lose`</hw> <pr>(s<ecr/l"<usl/*l<omac/s`)</pr>, <pos>a.</pos> <def>Consisting of, or containing, cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cellulose</ent><br/
<hw>Cel"lu*lose`</hw>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The substance which constitutes the essential part of the solid framework of plants, of ordinary wood, cotton, linen, paper, etc. It is also found to a slight extent in certain animals, as the tunicates. It is a carbohydrate, <chform>(C6H10O5)n</chform>, isomeric with starch, and is convertible into starches and sugars by the action of heat and acids. When pure, it is a white amorphous mass. See <er>Starch</er>, <er>Granulose</er>, <er>Lignin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Unsized, well bleached linen paper is merely pure <qex>cellulose</qex>.</q> <rj><qau>Goodale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Starch cellulose</b></col>, <cd>the delicate framework which remains when the soluble part (granulose) of starch is removed by saliva or pepsin.</cd>  <rj><au>Goodale.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cellulosid</ent><br/
<hw>cellulosid</hw> <pos>adj.</pos>  <def>of or containing or made from cellulose.</def><br/
<syn><b>Syn. --</b> cellulosic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Celosia</ent><br/
<hw>Celosia</hw> <pos>n.</pos> <def>a genus of annual or perennial herbs or vines of tropical and subtropical America and Asia and Africa.</def><br/
<syn><b>Syn. --</b> genus <gen>Celosia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Celotomy</ent><br/
<hw>Ce*lot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ hernia + <?/ to cut.]</ety> <fld>(Med.)</fld> <def>The act or operation of cutting, to relieve the structure in strangulated hernia.</def> <altsp>[Frequently written <asp>kelotomy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celsiture</ent><br/
<hw>Cel"si*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>celstudo</ets>, from <ets>celsus</ets> high: cf. <ets>celsitude</ets>.]</ety> <def>Height; altitude.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celsius</ent><br/
<hw>Cel"si*us</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The Celsius thermometer or scale, so called from Anders Celsius, a Swedish astronomer, who invented it. It is the same as the <xex>centigrade</xex> thermometer or scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celt</ent><br/
<hw>Celt</hw> <pr>(s<ecr/lt; k<ecr/lt)</pr>, <pos>n.</pos> <ety>[L. <ets>Celtae</ets>, Gr. <grk>Keltoi`</grk>, <grk>Ke`ltai</grk>, pl.: cf. W. <ets>Celtiad</ets> one that dwells in a covert, an inhabitant of the wood, a Celt, fr. <ets>celt</ets> covert, shelter, <ets>celu</ets> to hide.]</ety> <def>One of an ancient race of people, who formerly inhabited a great part of Central and Western Europe, and whose descendants at the present day occupy Ireland, Wales, the Highlands of Scotland, and the northern shores of France.</def> <altsp>[Written also <asp>Kelt</asp>. The letter C was pronounced hard in Celtic languages.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celt</ent><br/
<hw>Celt</hw>, <pos>n.</pos> <ety>[LL. <ets>celts</ets> a chisel.]</ety> <fld>(Arch<ae/ol.)</fld> <def>A weapon or implement of stone or metal, found in the tumuli, or barrows, of the early Celtic nations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celtiberian</ent><br/
<hw>Celt`i*be"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Celtiber</ets>, <ets>Celtibericus</ets>.]</ety> <def>Of or pertaining to the ancient Celtiberia (a district in Spain lying between the Ebro and the Tagus) or its inhabitants the Celtiberi (Celts of the river Iberus).</def> -- <def2><pos>n.</pos> <def>An inhabitant of Celtiberia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celtic</ent><br/
<hw>Celt"ic</hw> <pr>(s<ecr/lt"<icr/k; k<ecr/lt"<icr/k)</pr>, <pos>a.</pos> <ety>[L. <ets>Celticus</ets>, Gr. <grk>Keltiko`s</grk>. See <er>Celt</er>.]</ety> <def>Of or pertaining to the Celts; <as>as, <ex>Celtic</ex> people, tribes, literature, tongue</as>.</def> <altsp>[Written also <asp>Keltic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celtic</ent><br/
<hw>Celt"ic</hw>, <pos>n.</pos> <def>The language of the Celts.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The remains of the old Celtic language are found in the Gaelic, the Erse or Irish; the Manx, and the Welsh and its cognate dialects Cornish and Bas Breton.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celticism</ent><br/
<hw>Celt"i*cism</hw> <pr>(s<ecr/l"t<icr/*s<icr/z'm)</pr>, <pos>n.</pos> <def>A custom of the Celts, or an idiom of their language.</def>  <rj><au>Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celticize</ent><br/
<hw>Celt"i*cize`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To render Celtic; to assimilate to the Celts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celtium</ent><br/
<hw>Cel"ti*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Chem.)</fld> <def>The first proposed name for a supposed new element of the rare-earth group, accompanying lutecium and scandium in the gadolinite earths. It is now considered identical to <er>Lutetium</er>. The proposed symbol was <it>Ct</it> (no period), but this name and symbol are no longer used.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>celtuce</ent><br/
<hw>celtuce</hw> <pos>n.</pos> <sn>1.</sn> <def>a type of lettuce (<spn>Lactuca sativa</spn> <varn>asparagina</varn>) valued especially for its edible stems.</def><br/
<syn><b>Syn. --</b> stem lettuce, Lactuca sativa asparagina.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a salad green having characteristics of both lettuce and celery; stems eaten either raw or cooked.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cembalo</ent><br/
||<hw>Cem"ba*lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. See <er>Cymbal</er>.]</ety> <fld>(Mus.)</fld> <def>An old name for the harpsichord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cement</ent><br/
<hw>Ce*ment"</hw> <pr>(s<ecr/*m<ecr/nt" <it>or</it> s<ecr/m"<ecr/nt)</pr>, <pos>n.</pos> <ety>[OF. <ets>cement</ets>, <ets>ciment</ets>, F. <ets>ciment</ets>, fr. L. <ets>caementum</ets> a rough, unhewn stone, pieces or chips of marble, from which mortar was made, contr. fr. <ets>caedimentum</ets>, fr. <ets>caedere</ets> to cut, prob. akin to <ets>scindere</ets> to cleave, and to E. <ets>shed</ets>, v. t.]</ety> <sn>1.</sn> <def>Any substance used for making bodies adhere to each other, as mortar, glue, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of calcined limestone, or a calcined mixture of clay and lime, for making mortar which will harden under water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The powder used in cementation. See <er>Cementation</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Bond of union; that which unites firmly, as persons in friendship, or men in society.</def> <ldquo/The <xex>cement</xex> of our love.<rdquo/<br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Anat.)</fld> <def>The layer of bone investing the root and neck of a tooth; -- called also <altname>cementum</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hydraulic cement</b></col>. <cd>See under <er>Hydraulic</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cement</ent><br/
<hw>Ce*ment"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cemented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cementing</conjf>.]</vmorph> <ety>[Cf. F. <ets>cimenter</ets>. See <er>Cement</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To unite or cause to adhere by means of a cement.</def>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite firmly or closely.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To overlay or coat with cement; <as>as, to <ex>cement</ex> a cellar bottom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cement</ent><br/
<hw>Ce*ment"</hw>, <pos>v. i.</pos> <def>To become cemented or firmly united; to cohere.</def>  <rj><au>S. Sharp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cemental</ent><br/
<hw>Ce*ment"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to cement, as of a tooth; <as>as, <ex>cemental</ex> tubes</as>.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cementation</ent><br/
<hw>Cem`en*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of cementing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A process which consists in surrounding a solid body with the powder of other substances, and heating the whole to a degree not sufficient to cause fusion, the physical properties of the body being changed by chemical combination with powder; thus iron becomes steel by cementation with charcoal, and green glass becomes porcelain by cementation with sand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cementatory</ent><br/
<hw>Ce*ment"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the quality of cementing or uniting firmly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cementer</ent><br/
<hw>Ce*ment"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person or thing that cements.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cementitious</ent><br/
<hw>Cem`en*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caementitius</ets> pertaining to quarry stones. See <er>Cement</er>, <pos>n.</pos> ]</ety> <def>Of the nature of cement.</def> <mark>[R.]</mark>  <rj><au>Forsyth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cement steel</ent><br/
<hw>Ce*ment" steel</hw>. <def>Steel produced by cementation; blister steel.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cemeterial</ent><br/
<hw>Cem`e*te"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a cemetery.</def> <ldquo/<xex>Cemeterial</xex> cells.<rdquo/ <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cemetery</ent><br/
<hw>Cem"e*ter*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cemeteries</plw> <pr>(<?/)</pr>.</plu> <ety>[L. <ets>cemeterium</ets>, Gr. <?/ a sleeping chamber, burial place, fr. <?/ to put to sleep.]</ety> <def>A place or ground set apart for the burial of the dead; a graveyard; a churchyard; a necropolis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenanthy</ent><br/
<hw>Ce*nan"thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ empty + <?/ a flower.]</ety> <fld>(Bot.)</fld> <def>The absence or suppression of the essential organs (stamens and pistil) in a flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenation</ent><br/
<hw>Ce*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cenatio</ets>.]</ety> <def>Meal-taking; dining or supping.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenatory</ent><br/
<hw>Cen"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cenatorius</ets>, fr. <ets>cenare</ets> to dine, sup, fr. <ets>cena</ets>, <ets>coena</ets>, dinner, supper.]</ety> <def>Of or pertaining to dinner or supper.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Romans washed, were anointed, and wore a <qex>cenatory</qex> garment.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenobite</ent><br/
<hw>Cen"o*bite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coenobita</ets>, fr. Gr. <grk>koino`bios</grk>; <grk>koino`s</grk> common + <grk>bi`os</grk> life: cf. F. <ets>c<eacute/nobite</ets>.]</ety> <def>One of a religious order, dwelling in a convent, or a community, in opposition to an anchoret, or hermit, who lives in solitude.</def>  <rj><au>Gibbon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenobitical</ent><br/
<ent>Cenobitic</ent><br/
<mhw>{ <hw>Cen`o*bit"ic</hw> <pr>(?)</pr>, <hw>Cen`o*bit"ic*al</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>c<eacute/nobitique</ets>.]</ety> <def>Of or pertaining to a cenobite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenobitism</ent><br/
<hw>Cen"o*bi*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being a cenobite; the belief or practice of a cenobite.</def>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenogamy</ent><br/
<hw>Ce*nog"a*my</hw> <pr>(s<esl/*n<ocr/g"<adot/*m<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>koino`s</grk> common + <grk>ga`mos</grk> marriage.]</ety> <def>The state of a community which permits promiscuous sexual intercourse among its members, as in certain societies practicing communism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cenogenesis</ent><br/
<hw>cen`o*gen"e*sis</hw> <pos>n.</pos> <ety>[Gr. <?/ new + E. <ets>genesis</ets>.]</ety> <fld>(Biol.)</fld> <def>The introduction during embryonic development of characters or structure not present in the earlier evolutionary history of the strain or species (as addition of the placenta in mammalian evolution); a modified evolution, in which nonprimitive characters make their appearance in consequence of a secondary adaptation of the embryo to the peculiar conditions of its environment; -- distinguished from <contr>palingenesis</contr>.</def> <altsp>[Also spelled <asp>caenogenesis</asp> and formerly <asp>kenogenesis</asp>.]</altsp><br/
[<source>WordNet 1.5</source> + <source>1913 Webster</source>]</p>

<p><ent>cenogenetic</ent><br/
<hw>cenogenetic</hw> <pos>adj.</pos> <def>of or pertaining to cenogenesis.  Opposite of <ant>palingenetic</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cenotaph</ent><br/
<hw>Cen"o*taph</hw> <pr>(s<esl/n"<osl/*t<adot/f)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kenota`fion</grk>; <grk>keno`s</grk> empty + <grk>ta`fos</grk> burial, tomb: cf. F. <ets>c<eacute/notaphe</ets>.]</ety> <def>An empty tomb or a monument erected in honor of a person who is buried elsewhere.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cenotaph</qex> in Westminster Abbey.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenotaphy</ent><br/
<hw>Cen"o*taph`y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cenotaph.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Lord Cobham honored him with a <qex>cenotaphy</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cenozoic</ent><br/
<hw>Ce`no*zo"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ recent + <?/ life.]</ety> <fld>(Geol.)</fld> <def>Belonging to the most recent division of geological time, including the tertiary, or Age of mammals, and the Quaternary, or Age of man. <altsp>[Written also <asp>c<ae/nozoic</asp>, <asp>cainozoic</asp>, <asp>kainozoic</asp>.]</altsp> See <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is used by many authors as synonymous with <xex>Tertiary</xex>, the Quaternary Age not being included.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cense</ent><br/
<hw>Cense</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cense</ets>, F. <ets>cens</ets>, L. <ets>census</ets>. See <er>Census</er>.]</ety> <sn>1.</sn> <def>A census; -- also, a public rate or tax.</def> <mark>[Obs.]</mark>  <rj><au>Howell. Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Condition; rank.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cense</ent><br/
<hw>Cense</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Censed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Censing</conjf>.]</vmorph> <ety>[Abbrev. from <ets>incense</ets>.]</ety> <def>To perfume with odors from burning gums and spices.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Salii sing and <qex>cense</qex> his altars round.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cense</ent><br/
<hw>Cense</hw>, <pos>v. i.</pos> <def>To burn or scatter incense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censer</ent><br/
<hw>Cen"ser</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>incenser</ets>, fr. OF. <ets>encensier</ets>, F. <ets>encensoir</ets>, fr. LL. <ets>incensarium</ets>, <ets>incensorium</ets>, fr. L. <ets>incensum</ets> incense. See <er>Incense</er>, and cf. <er>Incensory</er>.]</ety> <def>A vessel for perfumes; esp. one in which incense is burned.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The ecclesiastical censer is usually cup-shaped, has a cover pierced with holes, and is hung by chains. The censer bearer swings it to quicken the combustion.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Her thoughts are like the fume of frankincense<br/
Which from a golden <qex>censer</qex> forth doth rise.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censor</ent><br/
<hw>Cen"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>censor</ets>, fr. <ets>censere</ets> to value, tax.]</ety> <sn>1.</sn> <fld>(Antiq.)</fld> <def>One of two magistrates of Rome who took a register of the number and property of citizens, and who also exercised the office of inspector of morals and conduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is empowered to examine manuscripts before they are committed to the press, and to forbid their publication if they contain anything obnoxious; -- an official in some European countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One given to fault-finding; a censurer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor can the most circumspect attention, or steady rectitude, escape blame from <qex>censors</qex> who have no inclination to approve.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A critic; a reviewer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Received with caution by the <qex>censors</qex> of the press.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>censored</ent><br/
<hw>censored</hw> <pos>adj.</pos> <def>suppressed or subjected to censorship; <as>as, the <ex>censored</ex> press in some countries</as>.  Opposite of <ant>uncensored</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Censorial</ent><br/
<hw>Cen*so"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Belonging to a censor, or to the correction of public morals.</def>  <rj><au>Junius.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Full of censure; censorious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>censorial</qex> declamation of Juvenal.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censorian</ent><br/
<hw>Cen*so"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Censorial.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censorious</ent><br/
<hw>Cen*so"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>censorius</ets> pertaining to the censor. See <er>Censor</er>.]</ety> <sn>1.</sn> <def>Addicted to censure; apt to blame or condemn; severe in making remarks on others, or on their writings or manners.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A dogmatical spirit inclines a man to be <qex>consorious</qex> of his neighbors.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Implying or expressing censure; <as>as, <ex>censorious</ex> remarks</as>.</def></p>

<p><syn><b>Syn.</b> -- Fault-finding; carping; caviling; captious; severe; condemnatory; hypercritical.</syn></p>

<p>-- <wordforms><wf>Cen*so"ri*ous*ly</wf>, <pos>adv.</pos> -- <wf>Cen*so"ri*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censorship</ent><br/
<hw>Cen"sor*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office or power of a censor; <as>as, to stand for a <ex>censorship</ex></as>.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The press was not indeed at that moment under a general <qex>censorship</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censual</ent><br/
<hw>Cen"su*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>censualis</ets>, fr. <ets>census</ets>.]</ety> <def>Relating to, or containing, a census.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He caused the whole realm to be described in a <qex>censual</qex> roll.</q> <rj><qau>Sir R. Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censurable</ent><br/
<hw>Cen"sur*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deserving of censure; blamable; culpable; reprehensible; <as>as, a <ex>censurable</ex> person, or <ex>censurable</ex> conduct</as>.</def></p>

<p>-- <wordforms><wf>Cen"sur*a*bleness</wf>, <pos>n.</pos> -- <wf>Cen"sur*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censure</ent><br/
<hw>Cen"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>censura</ets> fr. <ets>censere</ets>: cf. F. <ets>censure</ets>. Cf. <er>Censor</er>.]</ety> <sn>1.</sn> <def>Judgment either favorable or unfavorable; opinion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Take each man's <qex>censure</qex>, but reserve thy judgment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of blaming or finding fault with and condemning as wrong; reprehension; blame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Both the <qex>censure</qex> and the praise were merited.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Judicial or ecclesiastical sentence or reprimand; condemnatory judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Excommunication or other <qex>censure</qex> of the church.</q> <rj><qau>Bp. Burnet.</qau></rj></p>

<p><syn><b>Syn.</b> -- Blame; reproof; condemnation; reprobation; disapproval; disapprobation; reprehension; animadversion; reprimand; reflection; dispraise; abuse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censure</ent><br/
<hw>Cen"sure</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Censured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Censuring</conjf>.]</vmorph> <ety>[Cf. F. <ets>ensurer</ets>.]</ety> <sn>1.</sn> <def>To form or express a judgment in regard to; to estimate; to judge.</def> <mark>[Obs.]</mark> <ldquo/Should I say more, you might well <xex>censure</xex> me a flatterer.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To find fault with and condemn as wrong; to blame; to express disapprobation of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I may be <qex>censured</qex> that nature thus gives way to loyalty.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To condemn or reprimand by a judicial or ecclesiastical sentence.</def>  <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- To blame; reprove; rebuke; condemn; reprehend; reprimand.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censure</ent><br/
<hw>Cen"sure</hw>, <pos>v. i.</pos> <def>To judge.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Censurer</ent><br/
<hw>Cen"sur*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who censures.</def>  <rj><au>Sha.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Census</ent><br/
<hw>Cen"sus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>census</ets>, fr. <ets>censere</ets>. See <er>Censor</er>.]</ety> <sn>1.</sn> <fld>(Bot. Antiq.)</fld> <def>A numbering of the people, and valuation of their estate, for the purpose of imposing taxes, etc.; -- usually made once in five years.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An official registration of the number of the people, the value of their estates, and other general statistics of a country.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A general census of the United States was first taken in 1790, and one has been taken at the end of every ten years since.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cent</ent><br/
<hw>Cent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cent</ets> hundred, L. <ets>centum</ets>. See <er>Hundred</er>.]</ety> <sn>1.</sn> <def>A hundred; <as>as, ten per <ex>cent</ex>, the proportion of ten parts in a hundred</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A United States coin, the hundredth part of a dollar, formerly made of copper, now of copper, tin, and zinc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An old game at cards, supposed to be like piquet; -- so called because 100 points won the game.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centage</ent><br/
<hw>Cent"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Rate by the hundred; percentage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cental</ent><br/
<hw>Cen"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>centum</ets> a hundred.]</ety> <def>A weight of one hundred pounds avoirdupois; -- called in many parts of the United States a <altname>Hundredweight</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cental</ent><br/
<hw>Cen"tal</hw>, <pos>n.</pos> <def>Relating to a hundred.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cental system</b></col>, <cd>the method of buying and selling by the cental, or hundredweight.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centare</ent><br/
<hw>Cen"tare`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>centiare</ets>; <ets>centi-</ets> (L. <ets>centum</ets>) + <ets>-are</ets>.]</ety> <def>A measure of area, the hundredth part of an are; one square meter, or about 1<frac15/ square yards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centaur</ent><br/
<hw>Cen"taur</hw> <pr>(s<ecr/n"t<add/r)</pr>, <pos>n.</pos> <ety>[L. <ets>centaurus</ets>, Gr. <grk>Ke`ntayros</grk>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Class. Myth.)</fld> <def>A fabulous being, represented as half man and half horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A constellation in the southern heavens between Hydra and the Southern Cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centaurea</ent><br/
||<hw>Cen`tau*re"a</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[NL. See <er>Centaury</er>.]</ety> <fld>(Bot.)</fld> <def>A large genus of composite plants, related to the thistles and including the cornflower or bluebottle (<spn>Centaurea Cyanus</spn>) and the star thistle (<spn>Centaurea Calcitrapa</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centaurium</ent><br/
<hw>Centaurium</hw> <pos>prop. n.</pos> <def>A genus of low-growing herbs mostly of the northern hemisphere having flowers with protruding spirally twisted anthers.</def><br/
<syn><b>Syn. --</b> genus <gen>Centaurium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centaurus</ent><br/
<hw>Centaurus</hw> <pos>n.</pos> <def>a conspicuous constellation in the southern hemisphere near the Southern Cross.</def><br/
<syn><b>Syn. --</b> Centaur, The Centaur.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 232 --></p>

<p><ent>Centauromachy</ent><br/
<hw>Cen`tau*rom"a*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ centaur + <?/ battle.]</ety> <fld>(Ancient Art)</fld> <def>A fight in which centaurs take part, -- a common theme for relief sculpture, as in the Parthenon metopes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Centaury</ent><br/
<hw>Cen"tau*ry</hw> <pr>(s<ecr/n"t<add/*r<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>centaureum</ets> and <ets>centauria</ets>, Gr. <grk>kentay`rion</grk>, <grk>kentay`reion</grk>, and <grk>kentayri`h</grk>, fr. the Centaur Chiron.]</ety> <fld>(Bot.)</fld> <def>A gentianaceous plant not fully identified. The name is usually given to the <spn>Eryther<ae/a Centaurium</spn> and the <spn>Chlora perfoliata</spn> of Europe, but is also extended to the whole genus <gen>Sabbatia</gen>, and even to the unrelated <gen>Centaurea</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>centavo</ent><br/
<hw>centavo</hw> <pos>n.</pos> <def>a fractional monetary unit of several countries such as El Salvador, St. Thomas and Principe, Brazil,  Argentina, Bolivia, Colombia, Cuba, the Dominican Republic, Ecuador, El Salvador, Guatemala, Honduras, Mexico, Nicaragua, Peru, the Philippines, and Portugal.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centenarian</ent><br/
<hw>Cen`te*na"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or relating to a hundred years.</def> -- <def2><pos>n.</pos> <def>A person a hundred years old.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centenary</ent><br/
<hw>Cen"te*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centenarius</ets>, fr. <ets>centum</ets> a hundred.]</ety> <sn>1.</sn> <def>Relating to, or consisting of, a hundred.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Occurring once in every hundred years; centennial.</def> <ldquo/<xex>Centenary</xex> solemnities.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centenary</ent><br/
<hw>Cen"te*na*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Centenaries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The aggregate of a hundred single things; specifically, a century.</def> <ldquo/Every <xex>centenary</xex> of years.<rdquo/  <rj><au>Hakewill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A commemoration or celebration of an event which occurred a hundred years before.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centennial</ent><br/
<hw>Cen*ten"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centum</ets> a hundred + <ets>annus</ets> year.]</ety> <sn>1.</sn> <def>Relating to, or associated with, the commemoration of an event that happened a hundred years before; <as>as, a <ex>centennial</ex> ode</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Happening once in a hundred years; <as>as, <ex>centennial</ex> jubilee; a <ex>centennial</ex> celebration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Lasting or aged a hundred years.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That opened through long lines<br/
Of sacred ilex and <qex>centennial</qex> pines.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centennial</ent><br/
<hw>Cen*ten"ni*al</hw>, <pos>n.</pos> <def>The celebration of the hundredth anniversary of any event; a centenary.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centennially</ent><br/
<hw>Cen*ten"ni*al*ly</hw>, <pos>adv.</pos> <def>Once in a hundred years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centennial State</ent><br/
<hw>Centennial State</hw>. <def>Colorado; -- a nickname alluding to the fact that it was admitted to the Union in the centennial year, 1876.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>center</ent><br/
<hw>cen"ter</hw> <pr>(s<ecr/n"t<etil/r)</pr>, <pos>n.</pos> <ety>[F. <ets>centre</ets>, fr. L. <ets>centrum</ets>, fr. Gr. <grk>ke`ntron</grk> any sharp point, the point round which a circle is described, fr. <grk>kentei^n</grk> to prick, goad.]</ety> <sn>1.</sn> <def>A point equally distant from the extremities of a line, figure, or body, or from all parts of the circumference of a circle; the middle point or place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The middle or central portion of anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A principal or important point of concentration; the nucleus around which things are gathered or to which they tend; an object of attention, action, or force; <as>as, a <ex>center</ex> of attaction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The earth.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Those members of a legislative assembly (as in France) who support the existing government. They sit in the middle of the legislative chamber, opposite the presiding officer, between the conservatives or monarchists, who sit on the right of the speaker, and the radicals or advanced republicans who occupy the seats on his left, See <er>Right</er>, and <er>Left</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Arch.)</fld> <def>A temporary structure upon which the materials of a vault or arch are supported in position until the work becomes self-supporting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mech.)</fld> <sd>(a)</sd> <def>One of the two conical steel pins, in a lathe, etc., upon which the work is held, and about which it revolves.</def> <sd>(b)</sd> <def>A conical recess, or indentation, in the end of a shaft or other work, to receive the point of a center, on which the work can turn, as in a lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In a lathe the <col><b>live center</b></col> is in the spindle of the head stock; the <col><b>dead center</b></col> is on the tail stock. <col><b>Planer centers</b></col> are stocks carrying centers, when the object to be planed must be turned on its axis.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Center of an army</b></col>, <cd>the body or troops occupying the place in the line between the wings.</cd> -- <mcol><col><b>Center of a curve</b></col> <it>or</it> <col><b>Center of a surface</b></col></mcol> <fld>(Geom.)</fld> <sd>(a)</sd> <cd>A point such that every line drawn through the point and terminated by the curve or surface is bisected at the point.</cd> <sd>(b)</sd> <cd>The fixed point of reference in polar coordinates. See <er>Coordinates</er>.</cd> -- <col><b>Center of curvature of a curve</b></col> <fld>(Geom.)</fld>, <cd>the center of that circle which has at any given point of the curve closer contact with the curve than has any other circle whatever. See <er>Circle</er>.</cd> -- <col><b>Center of a fleet</b></col>, <cd>the division or column between the van and rear, or between the weather division and the lee.</cd> -- <col><b>Center of gravity</b></col> <fld>(Mech.)</fld>, <cd>that point of a body about which all its parts can be balanced, or which being supported, the whole body will remain at rest, though acted upon by gravity.</cd> -- <col><b>Center of gyration</b></col> <fld>(Mech.)</fld>, <cd>that point in a rotating body at which the whole mass might be concentrated (theoretically) without altering the resistance of the intertia of the body to angular acceleration or retardation.</cd> -- <col><b>Center of inertia</b></col> <fld>(Mech.)</fld>, <cd>the center of gravity of a body or system of bodies.</cd> -- <col><b>Center of motion</b></col>, <cd>the point which remains at rest, while all the other parts of a body move round it.</cd> -- <col><b>Center of oscillation</b></col>, <cd>the point at which, if the whole matter of a suspended body were collected, the time of oscillation would be the same as it is in the actual form and state of the body.</cd> -- <col><b>Center of percussion</b></col>, <cd>that point in a body moving about a fixed axis at which it may strike an obstacle without communicating a shock to the axis.</cd> -- <col><b>Center of pressure</b></col> <fld>(Hydros.)</fld>, <cd>that point in a surface pressed by a fluid, at which, if a force equal to the whole pressure and in the same line be applied in a contrary direction, it will balance or counteract the whole pressure of the fluid.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centre</ent><br/
<ent>Center</ent><br/
<mhw>{ <hw>Cen"ter</hw>, <hw>Cen"tre</hw>  }</mhw> <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Centered</conjf> or <conjf>Centred</conjf> <pr>(s<ecr/n"t<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Centering</conjf> or <conjf>Centring</conjf>.]</vmorph> <sn>1.</sn> <def>To be placed in a center; to be central.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be collected to a point; to be concentrated; to rest on, or gather about, as a center.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where there is no visible truth wherein to <qex>center</qex>, error is as wide as men's fancies.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our hopes must <qex>center</qex> in ourselves alone.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centre</ent><br/
<ent>Center</ent><br/
<mhw>{ <hw>Cen"ter</hw> , <hw>Cen"tre</hw>  }</mhw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To place or fix in the center or on a central point.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To collect to a point; to concentrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy joys are <qex>centered</qex> all in me alone.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>To form a recess or indentation for the reception of a center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrebit</ent><br/
<ent>Centerbit</ent><br/
<mhw>{ <hw>Cen"ter*bit`</hw>, <hw>Cen"tre*bit`</hw>, }</mhw> <pos>n.</pos> <def>An instrument turning on a center, for boring holes. See <er>Bit</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centreboard</ent><br/
<ent>Centerboard</ent><br/
<mhw>{ <hw>Cen"ter*board`</hw>, <hw>Cen"tre*board</hw>, }</mhw> <pr>(s<ecr/n"t<etil/r*b<omac/rd`)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A retractable or sliding keel used on sailboats, formed of a broad board or slab of wood or metal which may be raised into a water-tight case amidships, when in shallow water, or may be lowered to increase the area of lateral resistance and prevent drifting to leeward when the vessel is beating to windward. It is used in vessels of all sizes along the coast of the United States</def><br/
<syn><b>Syn. --</b> centreboard, drop keel, sliding keel.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>centered</ent><br/
<hw>centered</hw> <pr>(s<ecr/n"t<etil/rd)</pr> <pos>adj.</pos> <sn>1.</sn> <def>being or placed in the center.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>concentrated on or clustered around a central point or purpose.</def><br/
<syn><b>Syn. --</b> centred, centralized, focused.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centerfield</ent><br/
<ent>center field</ent><br/
<mhw><hw>cen"ter*field"</hw>, <hw>cen"ter field"</hw></mhw> <pr>(s<ecr/n"t<etil/r*f<emac/ld")</pr> <pos>n.</pos> <fld>(Baseball)</fld> <def>The part of the outfield directly ahead of the catcher.</def><br/
<syn><b>Syn. --</b> center.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centerfire cartridge</ent><br/
<hw>Cen"ter*fire` car"tridge</hw>. <def>See under <er>Cartridge</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centering</ent><br/
<hw>Cen"ter*ing</hw>, <pos>n.</pos> <fld>(Arch.)</fld> <def>Same as <er>Center</er>, <pos>n.</pos>, 6.</def> <altsp>[Written also <asp>centring</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrepiece</ent><br/
<ent>Centerpiece</ent><br/
<mhw>{ <hw>Cen"ter*piece`</hw>, <hw>Cen"tre*piece`</hw>  }</mhw> <pr>(s<ecr/n"t<etil/r*p<emac/s`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An ornament to be placed in the center, as of a table, ceiling, atc.; a central article or figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <def>An arrangement of flowers placed in the center of a table as an ornament while dining.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>The central or most important feature of an activity, plan, or object; <as>as, the <ex>centerpiece</ex> of her campaign was a proposal to slash taxes</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Centre punch</ent><br/
<ent>Center punch</ent><br/
<mhw>{ <hw>Center punch</hw>, <it>or</it> <hw>Centre punch</hw> }</mhw>. <fld>(Mech.)</fld> <sd>(a)</sd> <def>A punch for making indentations or dots in a piece of work, as for suspension between lathe centers, etc.</def> <sd>(b)</sd> <def>A punch for punching holes in sheet metal, having a small conical center to insure correct locating.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Centre seal</ent><br/
<ent>Center seal</ent><br/
<mhw>{ <hw>Cen"ter seal</hw>, <it>or</it> <hw>Cen"tre seal</hw>,}</mhw>. <fld>(Gas Manuf.)</fld> <def>A compound hydraulic valve for regulating the passage of the gas through a set of purifiers so as to cut out each one in turn for the renewal of the lime.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>center seal</ent><br/
<ent>centre seal</ent><br/
<mhw>{<hw>Center seal</hw>, <hw>Centre seal</hw> }</mhw>. <fld>(Gas Manuf.)</fld> <def>A compound hydraulic valve for regulating the passage of the gas through a set of purifiers so as to cut out each one in turn for the renewal of the lime.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Centesimal</ent><br/
<hw>Cen*tes"i*mal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centesimus</ets> the hundredth, fr. <ets>centum</ets> a hundred: cf. F. <ets>cent<eacute/simal</ets>.]</ety> <def>Hundredth.</def> -- <def2><pos>n.</pos> <def>A hundredth part.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><q>The neglect of a few <qex>centesimals</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centesimation</ent><br/
<hw>Cen*tes`i*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>centesimore</ets> to take out or select every hundredth, fr. <ets>centesimus</ets> hundredth.]</ety> <fld>(Mil.)</fld> <def>The infliction of the death penalty upon one person in every hundred, as in cases of mutiny.</def></p>

<p><-- "centesm" out of order in original -- some error. = centesim?--><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centesimo</ent><br/
<hw>Cen*tes"i*mo</hw> <pr>(s<ecr/n"t<ecr/s"<icr/*m<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-mi</plw> <pr>(s<ecr/n"t<ecr/s"<icr/*m<emac/)</pr>.</plu> <ety>[It. & Sp.]</ety> <def>A copper coin of Italy and Spain equivalent to a centime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centesm</ent><br/
<hw>Cen"tesm</hw> <pr>(s<ecr/n"t<ecr/z'm)</pr>, <pos>n.</pos> <ety>[L. <ets>centesima</ets>.]</ety> <def>Hundredth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centiare</ent><br/
<hw>Cen"ti*are`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <ets>Centare</ets>.]</ety> <def>See <er>centare</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centicipitous</ent><br/
<hw>Cen`ti*cip"i*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centiceps</ets>, <ets>-cipitis</ets>; <ets>centum</ets> a hunder + <ets>caput</ets> head.]</ety> <def>Hundred-headed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centifidous</ent><br/
<hw>Cen*tif"i*dous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centifidus</ets>; <ets>centum + findere</ets> to split.]</ety> <def>Divided into a hundred parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centifolious</ent><br/
<hw>Cen`ti*fo"li*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centifolius</ets>; <ets>centum + folium</ets> leaf.]</ety> <def>Having a hundred leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centigrade</ent><br/
<hw>Cen"ti*grade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centum</ets> a hundred + <ets>gradus</ets> degree: cf. F. <ets>centigrade</ets>.]</ety> <def>Consisting of a hundred degrees; graduated into a hundred divisions or equal parts.</def> <specif>Specifically:</specif> <def>Of or pertaining to the centigrade thermometer; <as>as, 10<deg/ <ex>centigrade</ex> (or 10<deg/ <it>C.</it></as>).  In measurements, abreviated C.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Centigrade thermometer</b></col>, <cd>a thermometer having the zero or 0 at the point indicating the freezing state of water, and the distance between that and the point indicating the boiling state of water divided into one hundred degrees.  It is called also the <altname>Celsius thermometer</altname>, from <person>Anders Celsius</person>, the originator of this scale.  Measurements are reported as <deg/ C.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centigramme</ent><br/
<ent>Centigram</ent><br/
<mhw>{ <hw>Cen"ti*gram</hw> <pr>(?)</pr>, <hw>Cen"ti*gramme</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>centigramme</ets>; <ets>centi-</ets> (L. <ets>centum</ets>) + <ets>gramme</ets>.  See <er>Gram</er>.]</ety> <def>The hundredth part of a gram; a mass equal to .15432 of a grain.  See 3rd <er>Gram</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centilitre</ent><br/
<ent>Centiliter</ent><br/
<mhw>{ <hw>Cen"ti*li`ter</hw>, <hw>Cen"ti*li`tre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>centilitre</ets>; <ets>centi</ets> (L. <ets>centum</ets>) + <ets>litre</ets>. See <er>Liter</er>.]</ety> <def>The hundredth part of a liter; a measure of volume or capacity equal to a little more than six tenths (0.6102) of a cubic inch, or one third (0.338) of a fluid ounce.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centiloquy</ent><br/
<hw>Cen*til"o*quy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>centum</ets> hundred + <ets>logui</ets> to speak.]</ety> <def>A work divided into a hundred parts.</def> <mark>[R.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centime</ent><br/
||<hw>Cen`time"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>centesimus</ets>. See <er>Centesimal</er>.]</ety> <fld>(F. Coinage)</fld> <def>The hundredth part of a franc; a small French copper coin and money of account.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centimetre</ent><br/
<ent>Centimeter</ent><br/
<mhw>{ <hw>Cen"ti*me`ter</hw>, <hw>Cen"ti*me`tre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>centim<egrave/tre</ets>; <ets>centi-</ets> (L. <ets>centum</ets>) + <ets>m<egrave/tre</ets>. See <er>Meter</er>.]</ety> <def>The hundredth part of a meter; a measure of length equal to rather more than thirty-nine hundredths (0.3937) of an inch. See <er>Meter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centinel</ent><br/
<hw>Cen"ti*nel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sentinel.</def> <mark>[Obs.]</mark>  <rj><au>Sackville.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centinody</ent><br/
<hw>Cen*tin"o*dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>centum</ets> a hundred + <ets>nodus</ets> knot: cf. F. <ets>centinode</ets>.]</ety> <fld>(Bot.)</fld> <def>A weed with a stem of many joints (<spn>Illecebrum verticillatum</spn>); also, the <spn>Polygonum aviculare</spn> or knotgrass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centipede</ent><br/
<hw>Cen"ti*pede</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>centipeda</ets>; <ets>centum</ets> a hundred + <ets>pes</ets>, <ets>pedis</ets>, foot: cf. F. <ets>centip<egrave/de</ets>.]</ety> <fld>(Zool.)</fld> <def>A species of the <class>Myriapoda</class>; esp. the large, flattened, venomous kinds of the order <ord>Chilopoda</ord>, found in tropical climates. they are many-jointed, and have a great number of feet.</def> <altsp>[Formerly also <grk>chi`lioi</grk> <asp>centiped</asp> (<?/).]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centistere</ent><br/
<hw>Cen"ti*stere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>centist<egrave/re</ets>; <ets>centi-</ets> (l. <ets>centum</ets>) + <ets>st<egrave/re</ets>.]</ety> <def>The hundredth part of a stere, equal to 0.353 cubic feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centner</ent><br/
<hw>Cent"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>centner</ets> a hundred-weight, fr. L. <ets>centenarius</ets> of a hundred, fr. <ets>centum</ets> a hundred.]</ety> <sn>1.</sn> <fld>(Metal. & Assaying)</fld> <def>A weight divisible first into a hundred parts, and then into smaller parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The metallurgists use a weight divided into a hundred equal parts, each one pound; the whole they call a <xex>centner</xex>: the pound is divided into thirty-two parts, or half ounces; the half ounce into two quarters; and each of these into two drams.  But the assayers use different weights. With them a <xex>centner</xex> is one dram, to which the other parts are proportioned.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The commercial hundredweight in several of the continental countries, varying in different places from 100 to about 112 pounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cento</ent><br/
<hw>Cen"to</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Centos</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>cento</ets> a garment of several pieces sewed together, patchwork, a poem made up of various verses of another poem.]</ety> <def>A literary or a musical composition formed by selections from different authors disposed in a new order.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centonism</ent><br/
<hw>Cen"to*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The composition of a cento; the act or practice of composing a cento or centos.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Central</ent><br/
<hw>Cen"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centralis</ets>, fr. <ets>centrum</ets>: cf. F. <ets>central</ets>. See <er>Center</er>.]</ety> <def>Relating to the center; situated in or near the center or middle; containing the center; of or pertaining to the parts near the center<-- original had "or of.." -->; equidistant or equally accessible from certain points.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Central force</b></col> <fld>(Math.)</fld>, <cd>a force acting upon a body towards or away from a fixed or movable center.</cd> -- <col><b>Center sun</b></col> <fld>(Astron.)</fld>, <cd>a name given to a hypothetical body about which M<aum/dler supposed the solar system together with all the stars in the Milky Way, to be revolving. A point near Alcyone in the Pleiades was supposed to possess characteristics of the position of such a body.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrale</ent><br/
<ent>Central</ent><br/
<mhw>{ <hw>Cen"tral</hw> <pr>(?)</pr>, ||<hw>Cen*tra"le</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>centrale</ets>, fr. L. <ets>centralis</ets>.]</ety> <fld>(Anat.)</fld> <def>The central, or one of the central, bones of the carpus or or tarsus. In the tarsus of man it is represented by the navicular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>centralisation</ent><br/
<hw>centralisation</hw> <pos>n.</pos> <def>same as <er>centralization</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centralism</ent><br/
<hw>Cen"tral*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state or condition of being central; the combination of several parts into one whole; centralization.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The system by which power is centralized, as in a government.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>centralistic</ent><br/
<ent>centralist</ent><br/
<mhw><hw>centralist</hw> <hw>centralistic</hw></mhw> <pos>adj.</pos> <def>advocating centralization.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centrality</ent><br/
<hw>Cen*tral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Centralities</plw> <pr>(<?/)</pr>.</plu> <def>The state of being central; tendency towards a center.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Meantime there is a great <qex>centrality</qex>, a centripetence equal to the centrifugence.</q> <rj><qau>R. W. Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centralization</ent><br/
<hw>Cen`tral*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>centralisation</ets>.]</ety> <def>The act or process of centralizing, or the state of being centralized; the act or process of combining or reducing several parts into a whole; <as>as, the <ex>centralization</ex> of power in the general government; the <ex>centralization</ex> of commerce in a city</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centralize</ent><br/
<hw>Cen"tral*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Centralized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Centralizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>centraliser</ets>.]</ety> <def>To draw or bring to a center point; to gather into or about a center; to bring into one system, or under one control.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[To] <qex>centralize</qex> the power of government.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>centralized</ent><br/
<hw>centralized</hw> <pos>adj.</pos> <sn>1.</sn> <def>drawn toward a center or brought under the control of a central authority; <as>as, <ex>centralized</ex> control of emergency relief efforts</as>; <ex>centralized</ex> government.  Opposite of <ant>decentralized</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>concentrated on or clustered around a central point or purpose; -- contrasting with <contr>distributed</contr>.</def><br/
<syn><b>Syn. --</b> centered, centred, focused.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centralizing</ent><br/
<hw>centralizing</hw> <pos>adj. prenom.</pos>  <def>causing to concentrate at a center.  Opposite of <ant>decentralizing</ant>.</def> [Narrower terms: <stype>centripetal, unifying(prenominal)</stype>]<br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centrally</ent><br/
<hw>Cen"tral*ly</hw>, <pos>adv.</pos> <def>In a central manner or situation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>central nervous system</ent><br/
<hw>central nervous system</hw> <pos>n.</pos> <def>the portion of the vertebrate nervous system consisting of the brain and spinal cord; -- abbreviated <er>CNS</er>.</def><br/
<syn><b>Syn. --</b> CNS.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centranthus</ent><br/
<hw>Centranthus</hw> <pos>n.</pos> <def>a genus of southern European herbs and subshrubs.</def><br/
<syn><b>Syn. --</b> genus <gen>Centranthus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centrarchid</ent><br/
<hw>centrarchid</hw> <pos>n.</pos> <def>any of a group of small carnivorous freshwater percoid fishes of North America usually having a laterally compressed body and metallic luster: crappies; black bass; bluegills; pumpkinseed.</def><br/
<syn><b>Syn. --</b> sunfish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centrarchidae</ent><br/
<hw>Centrarchidae</hw> <pos>n.</pos> <def>a natural family of fish comprising the sunfishes.  See <er>sunfish</er>.</def><br/
<syn><b>Syn. --</b> family <fam>Centrarchidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centre</ent><br/
<hw>cen"tre</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Center</er>.</def> <mark>[chiefly British]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centreboard</ent><br/
<hw>centreboard</hw> <pos>n.</pos> <def>same as <er>centerboard</er>.</def><br/
<syn><b>Syn. --</b> centerboard, drop keel, sliding keel.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centred</ent><br/
<hw>centred</hw> <pos>adj.</pos> <def>same as <er>centered</er>.</def><br/
<syn><b>Syn. --</b> centered, centralized, focused.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centrepiece</ent><br/
<hw>centrepiece</hw> <pos>n.</pos> <def>same as <er>centerpiece</er>.</def><br/
<syn><b>Syn. --</b> centerpiece.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centrex</ent><br/
<hw>centrex</hw> <pos>n.</pos> <def>a contracted form of CENTRal EXchange.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centrical</ent><br/
<ent>Centric</ent><br/
<mhw>{ <hw>Cen"tric</hw> <pr>(?)</pr>, <hw>Cen"tric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Placed in the center or middle; central; situated at or near a center.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At York or some other <qex>centrical</qex> place.</q> <rj><qau>Sir W. Scott.</qau></rj></p>

<p>-- <wordforms><wf>Cen"tric*al*ly</wf>, <pos>adv.</pos> -- <wf>Cen"tric*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centricity</ent><br/
<hw>Cen*tric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being centric; centricalness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrifugal</ent><br/
<hw>Cen*trif"u*gal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centrum</ets> center + <ets>fugere</ets> to flee.]</ety> <sn>1.</sn> <def>Tending, or causing, to recede from the center.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Expanding first at the summit, and later at the base, as a flower cluster.</def> <sd>(b)</sd> <def>Having the radicle turned toward the sides of the fruit, as some embryos.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Centrifugal force</b></col> <fld>(Mech.)</fld>, <cd>a force whose direction is from a center.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When a body moves in a circle with uniform velocity, a force must act on the body to keep it in the circle without change of velocity. The direction of this force is towards the center of the circle. If this force is applied by means of a string to the body, the string will be in a state of tension. To a person holding the other end of the string, this tension will appear to be directed toward the body as if the body had a tendency to move away from the center of the circle which it is describing. Hence this latter force is often called <xex>centrifugal force</xex>. The force which really acts on the body being directed towards the center of the circle is called <xex>centripetal force</xex>, and in some popular treatises the centripetal and centrifugal forces are described as opposing and balancing each other. But they are merely the different aspects of the same stress.</note>  <rj><au>Clerk Maxwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Centrifugal impression</b></col> <fld>(Physiol.)</fld>, <cd>an impression (motor) sent from a nerve center <xex>outwards</xex> to a muscle or muscles by which motion is produced.</cd> -- <col><b>Centrifugal machine</b></col>, <cd>A machine for expelling water or other fluids from moist substances, or for separating liquids of different densities by centrifugal action; a whirling table.</cd> -- <col><b>Centrifugal pump</b></col>, <cd>a machine in which water or other fluid is lifted and discharged through a pipe by the energy imparted by a wheel or blades revolving in a fixed case. Some of the largest and most powerful pumps are of this kind.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrifugal</ent><br/
<hw>Cen*trif"u*gal</hw>, <pos>n.</pos> <def>A centrifugal machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrifugal filter</ent><br/
<hw>Cen*trif"u*gal fil"ter</hw>. <def>A filter, as for sugar, in which a cylinder with a porous or foraminous periphery is rapidly rotated so as to drive off liquid by centrifugal action.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>centrifugate</ent><br/
<hw>centrifugate</hw> <pos>v. t.</pos> <def>to drive out; to subject to the action of a centrifuge.</def><br/
<syn><b>Syn. --</b> centrifuge.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centrifugation</ent><br/
<hw>centrifugation</hw> <pos>n.</pos> <def>the process of separating substances by the use of a centrifuge.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centrifuge</ent><br/
<hw>centrifuge</hw> <pos>n.</pos> <def>an apparatus having containers for liquids arrayed around a central pivot and rotated at a high speed, thus generating <cref>centrifugal force</cref> on the liquid, and separating substances (such as particles of solid or globules of an immiscible liquid) mixed together in suspension within the liquid.  Suspensions which would settle only very slowly or not at all under gravity can be made to separate quickly in such a device.</def> <note>The containers for holding the liquid  in a centrifuge are held in a metal frame called the <part>head</part> or <part>trunnion</part>.  The solid material collected at teh bottom of the liquid container is called the pellet. A centrifuge designed to run at very high speeds and thus generate very high centrifugal force is called an <stype>ultracentrifuge</stype>.</note><br/
<syn><b>Syn. --</b> extractor, separator.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>centrifuge</ent><br/
<hw>centrifuge</hw> <pos>v. t.</pos> <fr>1</fr> <def>to drive out; to subject to the action of a centrifuge.</def><br/
<syn><b>Syn. --</b> centrifugate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centrifuge out</ent><br/
<hw>centrifuge out</hw> <pos>v. t.</pos> <def>to separate (particles in a suspension) from a liquid by centrifugation.</def><br/
<syn><b>Syn. --</b> centrifugate.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Centrifugence</ent><br/
<hw>Cen*trif"u*gence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The property or quality of being centrifugal.</def>  <rj><au>R. W. Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centring</ent><br/
<hw>Cen"tring</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Centring</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>centriole</ent><br/
<hw>centriole</hw> <pos>n.</pos> <fld>(Biol.)</fld> <def>one of two small cylindrical cell organelles composes of nine triplet microtubules, which form the asters during mitosis.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centripetal</ent><br/
<hw>Cen*trip"e*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centrum</ets> center + <ets>petere</ets> to move toward.]</ety> <sn>1.</sn> <def>Tending, or causing, to approach the center.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Expanding first at the base of the inflorescence, and proceeding in order towards the summit.</def> <sd>(b)</sd> <def>Having the radicle turned toward the axis of the fruit, as some embryos.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Progressing by changes from the exterior of a thing toward its center; <as>as, the <ex>centripetal</ex> calcification of a bone</as>.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Centripetal force</b></col> <fld>(Mech.)</fld>, <cd>a force whose direction is towards a center, as in case of a planet revolving round the sun, the center of the system, See <cref>Centrifugal force</cref>, under <er>Centrifugal</er>.</cd> -- <col><b>Centripetal impression</b></col> <fld>(Physiol.)</fld>, <cd>an impression (sensory) transmitted by an afferent nerve from the exterior of the body <xex>inwards</xex>, to the central organ.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centripetence</ent><br/
<hw>Cen*trip"e*tence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Centripetency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centripetency</ent><br/
<hw>Cen*trip"e*ten*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tendency toward the center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centriscoid</ent><br/
<hw>Cen*tris"coid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Centriscus</ets> (r. Gr. <?/ a kind of fish) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Allied to, or resembling, the genus <gen>Centriscus</gen>, of which the bellows fish is an example.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrobaric</ent><br/
<hw>Cen`tro*bar"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. (<?/) <?/ a treatise of Archimedes on finding the center of gravity, fr. <?/ gravitating toward the center; <?/ center + <?/ weight.]</ety> <def>Relating to the center of gravity, or to the process of finding it.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Centrobaric method</b></col> <fld>(Math.)</fld>, <cd>a process invented for the purpose of measuring the area or the volume generated by the rotation of a line or surface about a fixed axis, depending upon the principle that every figure formed by the revolution of a line or surface about such an axis has for measure the product of the line or surface by the length of the path of its center of gravity; -- sometimes called <altname>theorem of Pappus</altname>, also, incorrectly, <altname>Guldinus's properties</altname>. See <cref>Barycentric calculus</cref>, under <er>Calculus</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 233  --></p>

<p><ent>Centrode</ent><br/
<hw>Cen"trode</hw> <pr>(s<ecr/n"tr<omac/d)</pr>, <pos>n.</pos> <fld>(Kinematics)</fld> <def>In two figures having relative motion, one of the two curves which are the loci of the instantaneous center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centroid</ent><br/
<hw>Cen"troid</hw> <pr>(s<ecr/n"troid)</pr>, <pos>n.</pos> <ety>[L. <ets>centrum</ets> + <ets>-oid</ets>.]</ety> <def>The center of mass, inertia, or gravity of a body or system of bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>centrolecithal</ent><br/
<hw>cen`tro*lec"i*thal</hw> <pr>(s<ecr/n`tr<osl/*l<ecr/s"<icr/*th<ait/l)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ke`ntron</grk> center + <grk>le`kiqos</grk> yolk of an egg.]</ety> <fld>(Biol.)</fld> <def>Having the food yolk placed at the center of the ovum, segmentation being either regular or unequal.</def>  <rj><au>Balfour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>centrolinead</ent><br/
<hw>cen`tro*lin"e*ad</hw> <pr>(s<ecr/n`tr<osl/*l<icr/n"<esl/*<acr/d)</pr>, <pos>n.</pos> <def>An instrument for drawing lines through a point, or lines converging to a center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrolineal</ent><br/
<hw>Cen`tro*lin"e*al</hw> <pr>(s<ecr/n`tr<osl/*l<icr/n"<esl/*<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>centrum</ets> + <ets>linea</ets> line.]</ety> <def>Converging to a center; -- applied to lines drawn so as to meet in a point or center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>centromere</ent><br/
<hw>centromere</hw> <pos>n.</pos> <fld>(Biol.)</fld> <def>a specialized condensed region of a chromosomes that appears during mitosis where the chromatids are held together to form an X shape.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>centromeric</ent><br/
<hw>centromeric</hw> <pos>adj.</pos> <fld>(Biol.)</fld> <def>pertaining to the centromere, the dense specialized portion of a chromosome to which the spindle attaches during mitosis.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centropomidae</ent><br/
<hw>Centropomidae</hw> <pos>n.</pos> <def>a natural family of fishes comprising the robalos (also called snooks).</def><br/
<syn><b>Syn. --</b> family <fam>Centropomidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centropomus</ent><br/
<hw>Centropomus</hw> <pos>n.</pos> <def>the type genus of the <fam>Centropomidae</fam>, comprising the <er>snooks</er>, and including <spn>Centropomus undecimalis</spn>, a large bony sport fish also esteemed as food.</def><br/
<syn><b>Syn. --</b> genus <gen>Centropomus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centropristis</ent><br/
<hw>Centropristis</hw> <pos>n.</pos> <def>a genus of sea basses.</def><br/
<syn><b>Syn. --</b> genus <gen>Centropristis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centrospermae</ent><br/
<hw>Centrospermae</hw> <pos>n.</pos> <def>used in former classification systems; approximately synonymous with order <ord>Caryophyllales</ord>.</def><br/
<syn><b>Syn. --</b> group Centrospermae.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Centrosome</ent><br/
<hw>Cen"tro*some`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ center + -<?/ the body.]</ety> <fld>(Biol.)</fld> <def>A peculiar rounded body lying near the nucleus of a cell. It is regarded as the dynamic element by means of which the machinery of cell division is organized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrosphere</ent><br/
<hw>Cen"tro*sphere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ centre + <ets>sphere</ets>.]</ety> <sn>1.</sn>  <fld>(Geol.)</fld> <def>The nucleus or central part of the earth, forming most of its mass; -- disting. from <contr>lithosphere</contr>, <contr>hydrosphere</contr>, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Biol.)</fld> <def>The central mass of an aster from which the rays extend and within which the centrosome lies when present; the attraction sphere. The name has been used both as excluding and including the centrosome, and also to designate a modified mass of protoplasm about a centrosome whether aster rays are developed or not.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Centrostaltic</ent><br/
<hw>Cen`tro*stal"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ center + <?/ checking.]</ety> <fld>(Physiol.)</fld> <def>A term applied to the action of nerve force in the spinal center.</def>  <rj><au>Marshall Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centrum</ent><br/
||<hw>Cen"trum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Centrums</plw> <pr>(#)</pr>, L. <plw>Centra</plw> <pr>(#)</pr>.</plu> <ety>[L., center.]</ety> <fld>(Anat.)</fld> <def>The body, or axis, of a vertebra. See <er>Vertebra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centry</ent><br/
<hw>Cen"try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sentry</er>.</def> <mark>[Obs.]</mark>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centumvir</ent><br/
||<hw>Cen*tum"vir</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Centumviri</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>centum</ets> hundred + <ets>Vir</ets> man.]</ety> <fld>(Rom. Hist.)</fld> <def>One of a court of about one hundred judges chosen to try civil suits. Under the empire the court was increased to 180, and met usually in four sections.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centumviral</ent><br/
<hw>Cen*tum"vi*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centumvitalis</ets>.]</ety> <def>Of or pertaining to the centumviri, or to a centumvir.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centumvirate</ent><br/
<hw>Cen*tum"vi*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>centumvirat</ets>.]</ety> <def>The office of a centumvir, or of the centumviri.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centuple</ent><br/
<hw>Cen"tu*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centuplex</ets>; <ets>centum + plicare</ets> to fold; cf. F. <ets>centuple</ets>.]</ety> <def>Hundredfold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centuple</ent><br/
<hw>Cen"tu*ple</hw>, <pos>v. t.</pos> <def>To increase a hundredfold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centuplicate</ent><br/
<hw>Cen*tu"pli*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Centuplicated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Centuplicating</conjf>.]</vmorph> <ety>[L. <ets>centuplicare</ets>. See <er>Centuple</er>, <pos>a.</pos>]</ety> <def>To make a hundredfold; to repeat a hundred times.</def> <mark>[R.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centurial</ent><br/
<hw>Cen*tu"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. See <er>Century</er>.]</ety> <def>Of or pertaining to a century; <as>as, a <ex>centurial</ex> sermon</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centuriate</ent><br/
<hw>Cen*tu"ri*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>centuriatus</ets>, p. p. of <ets>centuriare</ets> to divide (men) into centuries.]</ety> <def>Pertaining to, or divided into, centuries or hundreds.</def> <mark>[R.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centuriate</ent><br/
<hw>Cen*tu"ri*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>century</er>.]</ety> <def>To divide into hundreds.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centurist</ent><br/
<ent>Centuriator</ent><br/
<mhw>{ <hw>Cen*tu"ri*a`tor</hw> <pr>(?)</pr>, <hw>Cen"tu*rist</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>centuriateur</ets>.]</ety> <def>An historian who distinguishes time by centuries, esp. one of those who wrote the <ldquo/Magdeburg Centuries.<rdquo/ See under <er>Century</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Centurion</ent><br/
<hw>Cen*tu"ri*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>centurio</ets>, fr. <ets>centuria</ets>; cf. F. <ets>centurion</ets>. See <er>Century</er>.]</ety> <fld>(Rom. Hist.)</fld> <def>A military officer who commanded a minor division of the Roman army; a captain of a century.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>centurion</qex> of the hand called the Italian band.</q> <rj><qau>Acts x. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Century</ent><br/
<hw>Cen"tu*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Centuries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>centuria</ets> (in senses 1 & 3), fr. <ets>centum</ets> a hundred: cf. F. <ets>centurie</ets>. See <er>Cent</er>.]</ety> <sn>1.</sn> <def>A hundred; <as>as, a <ex>century</ex> of sonnets</as>; an aggregate of a hundred things.</def> <mark>[Archaic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And on it said a <qex>century</qex> of prayers.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A period of a hundred years; <as>as, this event took place over two <ex>centuries</ex> ago</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Century</xex>, in the reckoning of time, although often used in a general way of any series of hundred consecutive years (as, a <xex>century</xex> of temperance work), usually signifies a division of the Christian era, consisting of a period of one hundred years ending with the hundredth year from which it is named; as, the <xex>first century</xex> (<er>a</er>. <er>d</er>. 1-100 inclusive); the <xex>seventh century</xex> (<er>a</er>.<er>d</er>. 601-700); the <xex>eighteenth century</xex> (<er>a</er>.<er>d</er>. 1701-1800). With words or phrases connecting it with some other system of chronology it is used of similar division of those eras; as, the <xex>first century</xex> of Rome (A.U.C.  1-100).</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Rom. Antiq.)</fld> <sd>(a)</sd> <def>A division of the Roman people formed according to their property, for the purpose of voting for civil officers.</def> <sd>(b)</sd> <def>One of sixty companies into which a legion of the army was divided. It was Commanded by a centurion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Century plant</b></col> <fld>(Bot.)</fld>, <cd>the <spn>Agave Americana</spn>, formerly supposed to flower but once in a century; -- hence the name. See <er>Agave</er>.</cd> -- <col><b>The Magdeburg Centuries</b></col>, <cd>an ecclesiastical history of the first thirteen centuries, arranged in thirteen volumes, compiled in the 16th century by Protestant scholars at Magdeburg.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceorl</ent><br/
<hw>Ceorl</hw> <pr>(k<ecr/<ocir/rl <it>or</it> ch<etil/rl)</pr>, <pos>n.</pos> <ety>[AS. See <er>Churl</er>, <pos>n.</pos>]</ety> <fld>(O. Eng. Hist.)</fld> <def>A freeman of the lowest class; one not a thane or of the servile classes; a churl.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cepaceous</ent><br/
<hw>Ce*pa"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cepa</ets>, <ets>caepa</ets>, onion.]</ety> <def>Of the nature of an onion, as in odor; alliaceous.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cepevorous</ent><br/
<hw>Ce*pev"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cepa</ets> an onion + <ets>varare</ets> to devour.]</ety> <def>Feeding upon onions.</def> <mark>[R.]</mark>  <rj><au>Sterling.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalad</ent><br/
<hw>Ceph"a*lad</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Gr. <grk>kefalh`</grk> head + L. <ets>ad</ets> toward.]</ety> <fld>(Zool.)</fld> <def>Forwards; towards the head or anterior extremity of the body; opposed to <ant>caudad</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalalgy</ent><br/
<ent>Cephalalgia</ent><br/
<mhw>{ ||<hw>Ceph`a*lal"gi*a</hw> <pr>(?)</pr>, <hw>Ceph"a*lal`gy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>cephalalgia</ets>, Gr. <?/; <?/ + <?/ pain: cf. F. <ets>c<eacute/phalalgie</ets>.]</ety> <fld>(Med.)</fld> <def>Pain in the head; headache.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalalgic</ent><br/
<hw>Ceph`a*lal"gic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cephalalgicus</ets>, Gr. <?/.]</ety> <fld>(Med.)</fld> <def>Relating to, or affected with, headache.</def> -- <def2><pos>n.</pos> <def>A remedy for the headache.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalanthium</ent><br/
||<hw>Ceph`a*lan"thi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ + <?/ flower.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Anthodium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalaspis</ent><br/
||<hw>Ceph`a*las"pis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kefalh`</grk> head + <?/ a shield.]</ety> <fld>(Paleon.)</fld> <def>A genus of fossil ganoid fishes found in the old red sandstone or Devonian formation. The head is large, and protected by a broad shield-shaped helmet prolonged behind into two lateral points.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalata</ent><br/
||<hw>Ceph`a*la"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>kefalh`</grk> head.]</ety> <fld>(Zool.)</fld> <def>A large division of Mollusca, including all except the bivalves; -- so called because the head is distinctly developed. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalate</ent><br/
<hw>Ceph"a*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalic</ent><br/
<hw>Ce*phal"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cephalicus</ets>, Gr. <?/, fr. <grk>kefalh`</grk> head: cf. F. <ets>c<eacute/phalique</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the head. See the Note under <er>Anterior</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cephalic index</b></col> <fld>(Anat.)</fld>, <cd>the ratio of the breadth of the cranium to the length, which is taken as the standard, and equal to 100; the breadth index.</cd> -- <col><b>Cephalic vein</b></col>, <cd>a large vein running from the back of the head alond the arm; -- so named because the ancients used to open it for disorders of the head.</cd>  <rj><au>Dunglison.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalic</ent><br/
<hw>Ce*pha"lic</hw>, <pos>n.</pos> <def>A medicine for headache, or other disorder in the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cephalin</ent><br/
<hw>ceph"a*lin</hw> <pr>(k<ecr/f"<adot/*l<icr/n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kefalh`</grk> the head; it was supposed by <person>Thudichum</person> to exist in brain tissue.]</ety> <fld>(Physiol. Chem.)</fld> <def>One of a group of phospholipids (nitrogenous phosphorized fatty substances), present in all living cells and particularly evident in nervous tissue.  The cephalins consist of glycerol phosphate in which the two free hydroxyls of the glycerol are esterified with fatty acids, and the phosphate forms an ester linkage to the hydroxyl of ethanolamine.  The phosphate may be linked to the alpha (end) or beta (middle) hydroxyl of the glycerol portion. The natural isomers are of the alpha form, and have the general formula <chform>R.O.CH2.CHOR<prime/.CH2.O.PO2.O.CH2.CH2.NH2</chform>, where R and R<prime/ are the acyl residues of long-chain fatty acids, which may be the same or different.</def> <altsp>[Also spelled <asp>kephalin</asp>.]</altsp><br/
<syn><b>Syn. --</b> phosphatidylethanolamine.</syn>
[<source>PJC</source>]</p>

<p><ent>Cephalism</ent><br/
<hw>Ceph"a*lism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kefalh`</grk> head.]</ety> <fld>(Anthropol.)</fld> <def>Form or development of the skull; <as>as, the races of man differ greatly in <ex>cephalism</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cephalitis</ent><br/
||<hw>Ceph`a*li"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kefalh`</grk> head + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Same as <er>Phrenitis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalization</ent><br/
<hw>Ceph`a*li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Domination of the head in animal life as expressed in the physical structure; localization of important organs or parts in or near the head, in animal development.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalo</ent><br/
<hw>Ceph"a*lo</hw> <pr>(?)</pr>. <ety>[Gr. <grk>kefalh`</grk> head.]</ety> <def>A combining form denoting <xex>the head</xex>, <xex>of the head</xex>, <xex>connected with the head</xex>; <as>as, <ex>cephalo</ex>some, <ex>cephalo</ex>pod</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalocercal</ent><br/
<hw>Ceph`a*lo*cer"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cephalo-</ets> + Gr. <grk>ke`rkos</grk> tail.]</ety> <fld>(Zool.)</fld> <def>Relating to the long axis of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephaloid</ent><br/
<hw>Ceph"a*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cephalo-</ets> + <ets>-oid</ets>.]</ety> <def>Shaped like the head.</def>  <rj><au>Craing.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalology</ent><br/
<hw>Ceph`a*lol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + <ets>-logy</ets>.]</ety> <def>The science which treats of the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalomere</ent><br/
<hw>Ceph"a*lo*mere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + <ets>-mere</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the somites (arthromeres) which make up the head of arthropods.</def>  <rj><au>Packard.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalometer</ent><br/
<hw>Ceph`a*lom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + <ets>-meter</ets>.]</ety> <fld>(Med.)</fld> <def>An instrument measuring the dimensions of the head of a fetus during delivery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalometry</ent><br/
<hw>Ceph`a*lom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anthropometry)</fld> <def>The measurement of the heads of living persons.</def> -- <wordforms><wf>Ceph`a*lo*met"ric</wf> <pr>(#)</pr>,<pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cephalon</ent><br/
||<hw>Ceph"a*lon</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalophora</ent><br/
||<hw>Ceph`a*loph"o*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>kefalh`</grk> head + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>The cephalata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalopode</ent><br/
<ent>Cephalopod</ent><br/
<mhw>{ <hw>Ceph"a*lo*pod</hw> <pr>(s<ecr/f"<adot/*l<osl/*p<ocr/d)</pr>, <hw>Ceph"a*lo*pode</hw> <pr>(s<ecr/f"<adot/*l<osl/*p<omac/d)</pr> }</mhw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the <class>Cephalopoda</class>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalopoda</ent><br/
||<hw>Ceph`a*lop"o*da</hw> <pr>(s<ecr/f`<adot/*l<ocr/p"<osl/*d<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., gr. Gr. <grk>kefalh`</grk> head + <ets>-poda</ets>: cf. F. <ets>c<eacute/phalopode</ets>.]</ety> <fld>(Zool.)</fld> <def>The highest class of <phylum>Mollusca</phylum>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They have, around the front of the head, a group of elongated muscular arms, which are usually furnished with prehensile suckers or hooks.  The head is highly developed, with large, well organized eyes and ears, and usually with a cartilaginous brain case.  The higher forms, as the cuttlefishes, squids, and octopi, swim rapidly by ejecting a jet of water from the tubular siphon beneath the head.  They have a pair of powerful horny jaws shaped like a parrot's beak, and a bag of inklike fluid which they can eject from the siphon, thus clouding the water in order to escape from their enemies.  They are divided into two orders, the <ord>Dibranchiata</ord>, having two gills and eight or ten sucker-bearing arms, and the <ord>Tetrabranchiata</ord>, with four gills and numerous arms without suckers.  The latter are all extinct except the <gen>Nautilus</gen>. See <er>Octopus</er>, <er>Squid</er>, <er>Nautilus</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalopodous</ent><br/
<ent>Cephalopodic</ent><br/
<mhw>{ <hw>Ceph`a*lo*pod"ic</hw> <pr>(s<ecr/f`<adot/*l<osl/*p<ocr/d"<icr/k)</pr>, <hw>Ceph`a*lop"o*dous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Zool.)</fld> <def>Belonging to, or resembling, the cephalopods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephaloptera</ent><br/
||<hw>Ceph`a*lop"te*ra</hw> <pr>(s<ecr/f`<adot/*l<ocr/p"t<esl/*r<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kefalh`</grk> head + <grk>ptero`n</grk> wing.]</ety> <fld>(Zool.)</fld> <def>One of the generic names of the gigantic ray (<spn>Manta birostris</spn>) of the family <fam>Mobulidae</fam>, known as <altname>devilfish</altname>, <altname>sea devil</altname>, <altname>manta</altname> and <altname>manta ray</altname>.  It is common on the coasts of South Carolina, Florida, and farther south, and is sometimes found as far north as New York Bay.  Some of them grow to enormous size, becoming twenty feet of more across the body, and weighing more than a ton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalosome</ent><br/
<hw>Ceph"a*lo*some</hw> <pr>(s<ecr/f"<adot/*l<osl/*s<omac/m</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + <ets>-some</ets> body.]</ety> <fld>(Zool.)</fld> <def>The anterior region or head of insects and other arthropods.</def>  <rj><au>Packard.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalosporin</ent><br/
<hw>Ceph"a*lo*spor`in</hw> <pr>(s<ecr/f"<adot/*l<osl/*sp<omac/r`<icr/n)</pr>, <pos>n.</pos> <ety>[from <ets>Cephalosporium</ets>, a fungus producing the first of the series discovered.]</ety> <fld>(Chem.)</fld> <def>any of a class of chemical substances, some of which have therapeutically useful antibacterial activity, whose structure contains a beta-lactam ring fused to a six-membered ring containing a sulfur and a nitrogen atom.  The first of the series, <stype>cephalosporin C</stype>, was discovered by <person>G. Brotzu</person> in 1955 in the culture broth of a <gen>Cephalosporium</gen> species found off the coast of Sardinia.  Other cephalosporins have been found to be produced by species of soil bacteria (actinomycetes).  Many semisynthetic analogs have been tested for antibacterial effect, and several of them have found use as important clinically useful antibacterial agents, some of which may be taken orally for treatment of bacterial infections.  The cephalosporins are the second class of <isa>beta-lactam antibiotic</isa> to be discovered, the first being the <contr>penicillins</contr> and more recent classes being the <contr>thienamycins</contr> and <contr>sulfazecins</contr>.  The <stype>cephamycins</stype> are a variant of cephalosporins with a methoxyl group on the beta-lactam ring, rendering them more resistant to penicillinases.  Among the <ex>cephalosporins</ex> which have been found clinically useful are <stype>cephalexin</stype>, <stype>cephaloridine</stype>, and <stype>cephalothin</stype>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cephalostyle</ent><br/
<hw>Ceph"a*lo*style</hw> <pr>(s<ecr/f"<adot/*l<osl/*st<imac/l)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + Gr. <grk>sty^los</grk> a pillar.]</ety> <fld>(Anat.)</fld> <def>The anterior end of the notochord and its bony sheath in the base of cartilaginous crania.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalothorax</ent><br/
<hw>Ceph`a*lo*tho"rax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + <ets>thorax</ets>.]</ety> <fld>(Zool.)</fld> <def>The anterior portion of any one of the Arachnida and higher Crustacea, consisting of the united head and thorax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalotome</ent><br/
<hw>Ceph"a*lo*tome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + Gr. <?/ to cut.]</ety> <fld>(Med.)</fld> <def>An instrument for cutting into the fetal head, to facilitate delivery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalotomy</ent><br/
<hw>Ceph`a*lot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Dissection or opening of the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Craniotomy; -- usually applied to bisection of the fetal head with a saw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalotribe</ent><br/
<hw>Ceph"a*lo*tribe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cephalo-</ets> + Gr. to rub, grind.]</ety> <def>An obstetrical instrument for performing cephalotripsy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalotripsy</ent><br/
<hw>Ceph"a*lo*trip`sy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cephalotribe</er>.]</ety> <fld>(Med.)</fld> <def>The act or operation of crushing the head of a fetus in the womb in order to effect delivery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalotrocha</ent><br/
||<hw>Ceph`a*lot"ro*cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kefalh`</grk> head + <?/ wheel.]</ety> <fld>(Zool.)</fld> <def>A kind of annelid larva with a circle of cilia around the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cephalous</ent><br/
<hw>Ceph"a*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kefalh`</grk> head.]</ety> <fld>(Zool.)</fld> <def>Having a head; -- applied chiefly to the Cephalata, a division of mollusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cepheus</ent><br/
<hw>Ce"pheus</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Astron.)</fld> <def>A northern constellation near the pole. Its head, which is in the Milky Way, is marked by a triangle formed by three stars of the fourth magnitude. See <er>Cassiopeia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cepphus</ent><br/
<hw>Cepphus</hw> <pos>n.</pos> <def>a genus comprising the gillemots.</def><br/
<syn><b>Syn. --</b> genus <gen>Cepphus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>CER</ent><br/
<hw>CER</hw> <pos>n.</pos> <def>a <er>conditioned emotional response</er>, an emotional response that has been acquired by conditioning.</def> <mark>[acronym]</mark> <br/
<syn><b>Syn. --</b> conditioned emotional response, conditioned emotion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceraceous</ent><br/
<hw>Ce*ra"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cera</ets> wax.]</ety> <def>Having the texture and color of new wax; like wax; waxy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerago</ent><br/
<hw>Ce*ra"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax.]</ety> <def>Beebread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceramic</ent><br/
<hw>Ce*ram"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ earthenware. Cf. <er>Keramic</er>.]</ety> <def>Of or pertaining to pottery; relating to the art of making earthenware; <as>as, <ex>ceramic</ex> products; <ex>ceramic</ex> ornaments for ceilings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceramics</ent><br/
<hw>Ce*ram"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Ceramic</er>.]</ety> <sn>1.</sn> <def>The art of making things of baked clay; as pottery, tiles, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Work formed of clay in whole or in part, and baked; <as>as, vases, urns, etc.</as></def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerargyrite</ent><br/
<hw>Ce*rar"gy*rite</hw> <pr>(s<esl/*r<aum/r"j<ycr/*r<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ke`ras</grk> horn + <grk>'a`rgyros</grk> silver.]</ety> <fld>(Min.)</fld> <def>Native silver chloride, a mineral of a white to pale yellow or gray color, darkening on exposure to the light. It may be cut by a knife, like lead or horn (hence called <altname>horn silver</altname>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>ceras</ent><br/
<hw>ceras</hw> <pos>n.</pos> <def>one of the often brightly colored and branching hornlike structures on the backs of nudibranchs and other related mollusks that serve as gills.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cerastium</ent><br/
<hw>Cerastium</hw> <pos>n.</pos> <def>a genus of weedy plants of the pink family, comprising the mouse-ear chickweeds.</def><br/
<syn><b>Syn. --</b> genus <gen>Cerastium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cerasin</ent><br/
<hw>Cer"a*sin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white amorphous substance, the insoluble part of cherry gum; -- called also <altname>meta-arabinic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A gummy mucilaginous substance; -- called also <altname>bassorin</altname>, <altname>tragacanthin</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerasinous</ent><br/
<hw>Ce*ras"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, or containing, cerasin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of a cherry color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerastes</ent><br/
||<hw>Ce*ras"tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a horned serpent, fr. Gr. <grk>kera`sths</grk> horned, fr. <grk>ke`ras</grk> horn.]</ety> <fld>(Zool.)</fld> <def>A genus of poisonous African serpents, with a horny scale over each eye; the horned viper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerate</ent><br/
<hw>Ce"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>ceratum</ets>, ceratm, fr. <ets>cera</ets> wax.]</ety> <fld>(Med.)</fld> <def>An unctuous preparation for external application, of a consistence intermediate between that of an ointment and a plaster, so that it can be spread upon cloth without the use of heat, but does not melt when applied to the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cerate</xex> consists essentially of wax (for which resin or spermaceti is sometimes substituted) mixed with oil, lard, and various medicinal ingredients. The cerate (formerly called <xex>simple cerate</xex>) of the United States Pharmacopoeia is a mixture of three parts of white wax and seven parts of lard.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerated</ent><br/
<hw>Ce"ra*ted</hw> <pr>(?)</pr>, <pos>p. a.</pos> <ety>[L. <ets>ceratus</ets>, p. p. of <ets>cerare</ets> to wax, fr. <ets>cera</ets> wax.]</ety> <def>Covered with wax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceratine</ent><br/
<hw>Cer"a*tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ the fallacy called <ldquo/the horns.<rdquo/ fr. <grk>ke`ras</grk> a horn.]</ety> <fld>(Lagic.)</fld> <def>Sophistical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceratobranchia</ent><br/
||<hw>Cer`a*to*bran"chi*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ke`ras</grk>, <grk>ke`ratos</grk>, horn + <?/, <ets>n. pl.</ets>, gills.]</ety> <fld>(Zool.)</fld> <def>A group of nudibranchiate Mollusca having on the back papilliform or branched organs serving as gills.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceratobranchial</ent><br/
<hw>Cer`a*to*bran"chi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to the bone, or cartilage, below the epibranchial in a branchial arch.</def> -- <def2><pos>n.</pos> <def>A ceratobranchial bone, or cartilage.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceratodus</ent><br/
||<hw>Ce*rat"o*dus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ke`ras</grk>, <grk>ke`ratos</grk> horn + <?/ tooth.]</ety> <fld>(Zool.)</fld> <def>A genus of ganoid fishes, of the order Dipnoi, first known as Mesozoic fossil fishes; but recently two living species have been discovered in Australian rivers. They have lungs so well developed that they can leave the water and breathe in air. In Australia they are called <altname>salmon</altname> and <altname>baramunda</altname>. See <er>Dipnoi</er>, and <er>Archipterygium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceratohyal</ent><br/
<hw>Cer`a*to*hy"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ke`ras</grk> horn + the letter <UPSILON/.]</ety> <fld>(Anat.)</fld> <def>Pertaining to the bone, or cartilage, below the epihyal in the hyoid arch.</def> -- <def2><pos>n.</pos> <def>A ceratohyal bone, or cartilage, which, in man, forms one of the small horns of the hyoid.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceratophyllaceae</ent><br/
<hw>Ceratophyllaceae</hw> <pos>n.</pos> <def>a natural family coextensive with the genus <gen>Ceratophyllum</gen>; the hornworts.</def><br/
<syn><b>Syn. --</b> family <fam>Ceratophyllaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratophyllum</ent><br/
<hw>Ceratophyllum</hw> <pos>n.</pos> <def>the sole genus constituting the family <gen>Ceratophyllaceae</gen>; the hornworts.</def><br/
<syn><b>Syn. --</b> genus <gen>Ceratophyllum</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratopogon</ent><br/
<hw>Ceratopogon</hw> <pos>n.</pos> <def>the type genus of the <fam>Ceratopogonidae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Ceratopogon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratopogonidae</ent><br/
<hw>Ceratopogonidae</hw> <pos>n.</pos> <def>a natural family of insects including the biting midges and sand flies.</def><br/
<syn><b>Syn. --</b> family <fam>Ceratopogonidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratopsia</ent><br/
<hw>Ceratopsia</hw> <pos>n.</pos> <def>a suborder of extinct animals including triceratops.</def><br/
<syn><b>Syn. --</b> suborder Ceratopsia.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>ceratopsian</ent><br/
<hw>ceratopsian</hw> <pos>n.</pos> <def>any of several four-footed herbivorous horned dinosaurs with enormous beaked skulls, of the late Cretaceous in North America and Mongolia.</def><br/
<syn><b>Syn. --</b> horned dinosaur.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratopsidae</ent><br/
<hw>Ceratopsidae</hw> <pos>n.</pos> <def>an extinct family of American ceratopsian dinosaurs.</def><br/
<syn><b>Syn. --</b> family <fam>Ceratopsidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratopteris</ent><br/
<hw>Ceratopteris</hw> <pos>n.</pos> <fld>(Bot.)</fld> <def>a genus of water ferns.</def><br/
<syn><b>Syn. --</b> genus <gen>Ceratopteris</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>ceratosaur</ent><br/
<hw>ceratosaur</hw> <pos>n.</pos> <fld>(Paleon.)</fld> <def>the <er>ceratosaurus</er>.</def><br/
<syn><b>Syn. --</b> ceratosaurus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceratosaurus</ent><br/
||<hw>Cer`a*to*sau"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ke`ras</grk> a horn + <?/ lizard.]</ety> <fld>(Paleon.)</fld> <def>A swift-running bipedal carnivorous American Jurassic dinosaur allied to the European Megalosaurus. The animal was nearly twenty feet in length, and the skull bears a bony short horn between the nostrils on the united nasal bones. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Ceratospongiae</ent><br/
||<hw>Cer`a*to*spon"gi*<ae/</hw> <pr>(?)</pr>, <pos>prop. n. pl.</pos> <ety>[NL., fr. Gr. <grk>ke`ras</grk>, <grk>ke`ratos</grk> horn + <?/ sponge.]</ety> <fld>(Zool.)</fld> <def>An order of sponges in which the skeleton consists of horny fibers. It includes all the commercial sponges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>ceratozamia</ent><br/
<hw>ceratozamia</hw> <pos>n.</pos> <fld>(Bot.)</fld> <def>a small cycad of the genus <gen>Ceratozamia</gen> having a short scaly woody trunk and fernlike foliage and woody cones; Mexico.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceraunics</ent><br/
<hw>Ce*rau"nics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ thunder and lightning.]</ety> <def>That branch of physics which treats of heat and electricity.</def>  <rj><au>R. Park.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceraunoscope</ent><br/
<hw>Ce*rau"no*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ thunder and lightning + <ets>-scope</ets>.]</ety> <def>An instrument or apparatus employed in the ancient mysteries to imitate thunder and lightning.</def>  <rj><au>T. Moore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 234 --></p>

<p><ent>Cerberean</ent><br/
<hw>Cer*be"re*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to, or resembling, Cerberus.</def> <altsp>[Written also <asp>Cerberian</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>With wide <qex>Cerberean</qex> mouth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerberus</ent><br/
<hw>Cer"be*rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. Cerberus (in sense 1), gr. <?/.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Class. Myth.)</fld> <def>A monster, in the shape of a three-headed dog, guarding the entrance into the infernal regions, Hence: Any vigilant custodian or guardian, esp. if surly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of East Indian serpents, allied to the pythons; the bokadam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cercal</ent><br/
<hw>Cer"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ke`rkos</grk> tail.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cercaria</ent><br/
||<hw>Cer*ca"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cercarle</plw> <pr>(<?/)</pr></plu> <ety>[NL., fr. Gr. <grk>ke`rkos</grk> tail.]</ety> <fld>(Zool.)</fld> <def>The larval form of a trematode worm having the shape of a tadpole, with its body terminated by a tail-like appendage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cercarian</ent><br/
<hw>Cer*ca"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of, like, or pertaining to, the Cercari<ae/.</def> -- <def2><pos>n.</pos> <def>One of the Cercari<ae/.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cercopidae</ent><br/
<hw>Cercopidae</hw> <pos>n.</pos> <def>a natural family comprising the froghoppers or spittlebugs.</def><br/
<syn><b>Syn. --</b> family <fam>Cercopidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cercopithecidae</ent><br/
<hw>Cercopithecidae</hw> <pos>n.</pos> <def>a natural family of Old World monkeys including the guenon, baboon, colobus monkey, langur, macaque, mandrill, mangabey, patas, and proboscis monkey.</def><br/
<syn><b>Syn. --</b> family <fam>Cercopithecidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cercopithecus</ent><br/
<hw>Cercopithecus</hw> <pos>n.</pos> <def>type genus of the <fam>Cercopithecidae</fam>, consisting of one genus of guenons.</def><br/
<syn><b>Syn. --</b> genus <gen>Cercopithecus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cercopod</ent><br/
<hw>Cer"co*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ke`rkos</grk> tail + <ets>-pod</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the jointed antenniform appendages of the posterior somites of certain insects.</def>  <rj><au>Packard.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cercospora</ent><br/
<hw>Cercospora</hw> <pos>n.</pos> <def>form genus of imperfect fungi that are leaf parasites with long slender spores.</def><br/
<syn><b>Syn. --</b> genus <gen>Cercospora</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cercosporella</ent><br/
<hw>Cercosporella</hw> <pos>n.</pos> <def>form genus of imperfect fungi lacking pigment in the spores and conidiophores.</def><br/
<syn><b>Syn. --</b> genus <gen>Cercosporella</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cercus</ent><br/
||<hw>Cer"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cerci</plw> <pr>(<?/)</pr></plu>. <ety>[NL., fr. Gr. <grk>ke`rkos</grk> tail.]</ety> <fld>(Zool.)</fld> <def>See <er>Cercopod</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cere</ent><br/
<hw>Cere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax: cf. F. <ets>cire</ets>.]</ety> <fld>(Zool.)</fld> <def>The soft naked sheath at the base of the beak of birds of prey, parrots, and some other birds. See <er>Beak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cere</ent><br/
<hw>Cere</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cering</conjf>.]</vmorph> <ety>[L. <ets>cerare</ets>, fr. <ets>cera</ets> wax: cf. F. <ets>cirer</ets>.]</ety> <def>To wax; to cover or close with wax.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cereal</ent><br/
<hw>Ce"re*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Cerealis</ets> pert. to Ceres, and hence, to agriculture. See <er>Ceres</er>.]</ety> <def>Of or pertaining to the grasses which are cultivated for their edible seeds (as wheat, maize, rice, etc.), or to their seeds or grain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cereal</ent><br/
<hw>Ce"re*al</hw> <pos>n.</pos> <def>Any grass cultivated for its edible grain, or the grain itself; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerealia</ent><br/
||<hw>Ce`re*a"li*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L. See <er>Cereal</er>.]</ety><br/
[<source>1913 Webster</source>] <sn>1.</sn> <fld>(Antiq.)</fld> <def>Public festivals in honor of Ceres.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cereals.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerealin</ent><br/
<hw>Ce"re*a*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A nitrogenous substance closely resembling diastase, obtained from bran, and possessing the power of converting starch into dextrin, sugar, and lactic acid.</def>  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebel</ent><br/
<hw>Cer"e*bel</hw>, <pos>n.</pos> <def>The cerebellum.</def>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebellous</ent><br/
<ent>Cerebellar</ent><br/
<mhw>{ <hw>Cer`e*bel"lar</hw> <pr>(?)</pr>, <hw>Cer`e*bel"lous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>of or pertaining to the cerebellum; <as>as, the <ex>cerebellar</ex> artery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebellum</ent><br/
<hw>Cer`e*bel"lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Cerebellums</plw> <pr>(<?/)</pr>, L. <plw>Cerebella</plw> <pr>(<?/)</pr></plu>. <ety>[L., dim. of <ets>cerebrum</ets> brain.]</ety> <fld>(Anat.)</fld> <def>The large lobe of the hind brain in front of and above the medulla; the little brain. It controls combined muscular action. See <er>Brain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebral</ent><br/
<hw>Cer"e*bral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cerebrum</ets> brain; akin to Gr. <grk>ka`ra</grk> head: cf. F. <ets>c<eacute/r<eacute/bral</ets>. See <er>Cheer</er>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the cerebrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cerebral apoplexy</b></col>. <cd>See under <er>Apoplexy</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebral</ent><br/
<hw>Cer"e*bral</hw>, <pos>n.</pos> <ety>[A false translation of the Skr. <ets>m<umac/rdhanya</ets>, lit., head-sounds.]</ety> <def>One of a class of lingual consonants in the East Indian languages. See <er>Lingual</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Prof. W. D. Whitney calls these letters <xex>linguals</xex>, and this is their usual designation in the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebralism</ent><br/
<hw>Cer"e*bral*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Philos.)</fld> <def>The doctrine or theory that psychical phenomena are functions or products of the brain only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebralist</ent><br/
<hw>Cer"e*bral*ist</hw>, <pos>n.</pos> <def>One who accepts cerebralism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebrate</ent><br/
<hw>Cer"e*brate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <fld>(Physiol.)</fld> <def>To exhibit mental activity; to have the brain in action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebration</ent><br/
<hw>Cer`e*bra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Action of the brain, whether conscious or unconscious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebric</ent><br/
<hw>Cer"e*bric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or derived from, the brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cerebric acid</b></col> <fld>(Physiol. Chem.)</fld>, <cd>a name formerly sometimes given to cerebrin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebricity</ent><br/
<hw>Cer`e*bric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Brain power.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebriform</ent><br/
<hw>Ce*reb"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cerebrum</ets> + <ets>-form</ets>.]</ety> <def>Like the brain in form or substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebrifugal</ent><br/
<hw>Cer`e*brif"u*gal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cerebrum</ets> + L. <ets>fugere</ets> to flee.]</ety> <fld>(Physiol.)</fld> <def>Applied to those nerve fibers which go from the brain to the spinal cord, and so transfer cerebral impulses (centrifugal impressions) outwards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebrin</ent><br/
<hw>Cer"e*brin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cerebrum</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A nonphosphorized, nitrogenous substance, obtained from brain and nerve tissue by extraction with boiling alcohol. It is uncertain whether it exists as such in nerve tissue, or is a product of the decomposition of some more complex substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebripetal</ent><br/
<hw>Cer`e*brip"e*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cerebrum</ets> + L. <ets>petere</ets> to seek.]</ety> <fld>(Physiol.)</fld> <def>Applied to those nerve fibers which go from the spinal cord to the brain and so transfer sensations (centripetal impressions) from the exterior inwards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebritis</ent><br/
||<hw>Cer`e*bri"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. E. <ets>cerebrum</ets> + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the cerebrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebroid</ent><br/
<hw>Cer"e*broid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cerebrum</ets> + <ets>-oid</ets>.]</ety> <def>Resembling, or analogous to, the cerebrum or brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebrology</ent><br/
<hw>Cer`e*brol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cerebrum</ets> + <ets>-logy</ets>.]</ety> <def>The science which treats of the cerebrum or brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebropathy</ent><br/
<hw>Cer`e*brop"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cerebrum</ets> + Gr. <?/ suffering.]</ety> <fld>(Med.)</fld> <def>A hypochondriacal condition verging upon insanity, occurring in those whose brains have been unduly taxed; -- called also <altname>brain fag</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebroscopy</ent><br/
<hw>Cer`e*bros"co*py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cerebrum</ets> + <ets>-scopy</ets>.]</ety> <fld>(Med.)</fld> <def>Examination of the brain for the diagnosis of disease; esp., the act or process of diagnosticating the condition of the brain by examination of the interior of the eye (as with an ophthalmoscope).</def>  <rj><au>Buck.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebrose</ent><br/
<hw>Cer`e*brose"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cerebrum</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A sugarlike body obtained by the decomposition of the nitrogenous non-phosphorized principles of the brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebro-spinal</ent><br/
<hw>Cer`e*bro-spi"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cerebrum</ets> + <ets>spinal</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the central nervous system consisting of the brain and spinal cord.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cerebro-spinal fluid</b></col> <fld>(Physiol.)</fld>, <cd>a serous fluid secreted by the membranes covering the brain and spinal cord.</cd> -- <mcol><col><b>Cerebro-spinal meningitis</b></col>, <col><b>Cerebro-spinal fever</b></col></mcol> <fld>(Med.)</fld>, <cd>a dangerous epidemic, and endemic, febrile disease, characterized by inflammation of the membranes of the brain and spinal cord, giving rise to severe headaches, tenderness of the back of the neck, paralysis of the ocular muscles, etc. It is sometimes marked by a cutaneous eruption, when it is often called <xex>spotted fever</xex>. It is not contagious.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerebrum</ent><br/
<hw>Cer"e*brum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Cerebrums</plw> <pr>(#)</pr>, L. <plw>Cerebra</plw> <pr>(#)</pr>.</plu> <ety>[L., the brain.]</ety> <fld>(Anat.)</fld> <def>The anterior, and in man the larger, division of the brain; the seat of the reasoning faculties and the will. See <er>Brain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerecloth</ent><br/
<hw>Cere"cloth`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax + E. <ets>cloth</ets>.]</ety> <def>A cloth smeared with melted wax, or with some gummy or glutinous matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Linen, besmeared with gums, in manner of <qex>cerecloth</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerement</ent><br/
<hw>Cere"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax: cf. F. <ets>cirement</ets>.]</ety> <sd>(a)</sd> <def>A cerecloth used for the special purpose of enveloping a dead body when embalmed.</def> <sd>(b)</sd> <def>Any shroud or wrapping for the dead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremonial</ent><br/
<hw>Cer`e*mo"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caerimonialis</ets>: cf. F. <ets>c<eacute/rimonial</ets>. See <er>Ceremony</er>.]</ety> <sn>1.</sn> <def>Relating to ceremony, or external rite; ritual; according to the forms of established rites.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Ceremonial</qex> observances and outward show.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Observant of forms; ceremonious. <note>[In this sense <xex>ceremonious</xex> is now preferred.]</note></def>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He moves in the dull <qex>ceremonial</qex> track.</q> <rj><qau>Druden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremonial</ent><br/
<hw>Cer`e*mo"ni*al</hw>, <pos>n.</pos> <sn>1.</sn> <def>A system of rules and ceremonies, enjoined by law, or established by custom, in religious worship, social intercourse, or the courts of princes; outward form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gorgeous <qex>ceremonial</qex> of the Burgundian court.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The order for rites and forms in the Roman Catholic church, or the book containing the rules prescribed to be observed on solemn occasions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremonialism</ent><br/
<hw>Cer`e*mo"ni*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Adherence to external rites; fondness for ceremony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremonially</ent><br/
<hw>Cer`e*mo"ni*al*ly</hw>, <pos>adv.</pos> <def>According to rites and ceremonies; <as>as, a person <ex>ceremonially</ex> unclean</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremonialness</ent><br/
<hw>Cer`e*mo"ni*al*ness</hw>, <pos>n.</pos> <def>Quality of being ceremonial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremonious</ent><br/
<hw>Cer`e*mo"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>c<eacute/r<eacute/monieux</ets>, L. <ets>Caerimoniosus</ets>.]</ety> <sn>1.</sn> <def>Consisting of outward forms and rites; ceremonial. <note>[In this sense <xex>ceremonial</xex> is now preferred.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>ceremonious</qex> part of His worship.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>According to prescribed or customary rules and forms; devoted to forms and ceremonies; formally respectful; punctilious.</def> <ldquo/<xex>Ceremonious</xex> phrases.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Too <qex>ceremonious</qex> and traditional.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Formal; precise; exact. See <er>Formal</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremoniously</ent><br/
<hw>Cer`e*mo"ni*ous*ly</hw>, <pos>adv.</pos> <def>In a ceremonious way.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremoniousness</ent><br/
<hw>Cer`e*mo"ni*ous*ness</hw>, <pos>n.</pos> <def>The quality, or practice, of being ceremonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceremony</ent><br/
<hw>Cer"e*mo*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Ceremonies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>c<eacute/r<eacute/monie</ets>, L. <ets>caerimonia</ets>; perh. akin to E. <ets>create</ets> and from a root signifying to do or <ets>make</ets>.]</ety> <sn>1.</sn> <def>Ar act or series of acts, often of a symbolical character, prescribed by law, custom, or authority, in the conduct of important matters, as in the performance of religious duties, the transaction of affairs of state, and the celebration of notable events; <as>as, the <ex>ceremony</ex> of crowning a sovereign; the <ex>ceremonies</ex> observed in consecrating a church; marriage and baptismal <ex>ceremonies</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>According to all the rites of it, and according to all the <qex>ceremonies</qex> thereof shall ye keep it [the Passover].</q> <rj><qau>Numb. ix. 3</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bring her up the high altar, that she may<br/
The sacred <qex>ceremonies</qex> there partake.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[The heralds] with awful <qex>ceremony</qex><br/
And trumpet's sound, throughout the host proclaim<br/
A solemn council.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Behavior regulated by strict etiquette; a formal method of performing acts of civility; forms of civility prescribed by custom or authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Ceremony</qex> was but devised at first<br/
To set a gloss on . . . hollow welcomes . . . <br/
But where there is true friendship there needs none.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Al <qex>ceremonies</qex> are in themselves very silly things; but yet a man of the world should know them.</q> <rj><qau>Chesterfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A ceremonial symbols; an emblem, as a crown, scepter, garland, etc.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Disrobe the images,<br/
If you find them decked with <qex>ceremonies</qex>.<br/
 . . . Let no images<br/
Be hung with C<ae/sar's trophies.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sign or prodigy; a portent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>C<ae/sar, I never stood on <qex>ceremonies</qex>,<br/
Yet, now they fright me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Master of ceremonies</b></col>, <cd>an officer who determines the forms to be observed, or superintends their observance, on a public occasion.</cd> -- <col><b>Not to stand on ceremony</b></col>, <cd>not to be ceremonious; to be familiar, outspoken, or bold.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cereous</ent><br/
<hw>Ce"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cereus</ets>, fr. <ets>cera</ets> was.]</ety> <def>Waxen; like wax.</def> <mark>[Obs.]</mark>  <rj><au>Gayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceres</ent><br/
<hw>Ce"res</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., Ceres, also corn, grain, akin to E. <ets>create</ets>.]</ety> <sn>1.</sn> <fld>(Class. Myth.)</fld> <def>The daughter of Saturn and Ops or Rhea, the goddess of corn and tillage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Actron.)</fld> <def>The first discovered asteroid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceresin</ent><br/
<hw>Cer"e*sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax.]</ety> <fld>(Chem.)</fld> <def>A white wax, made by bleaching and purifying ozocerite, and used as a substitute for beeswax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cereus</ent><br/
||<hw>Ce"re*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a wax candle, fr. <ets>cera</ets> wax. So named from the resemblance of one species to the columnar shape of a wax candle.]</ety> <fld>(Bot.)</fld> <def>A genus of plants of the Cactus family. They are natives of America, from California to Chili.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Although several species flower in the night, the name <stype>Night-blooming cereus</stype> is specially applied to the <spn>Cereus grandiflorus</spn>, which is cultivated for its beautiful, shortlived flowers. The <spn>Cereus giganteus</spn>, whose columnar trunk is sometimes sixty feet in height, is a striking feature of the scenery of New Mexico, Texas, etc.<-- saguaro?= Carnegiea gigantea --></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerevis</ent><br/
<hw>Cer"e*vis</hw> <pr>(s<ecr/r"<esl/*v<icr/s; <it>G.</it> ts<ecr/r`<etil/*v<emac/s")</pr>, <pos>n.</pos> <ety>[G., fr. L. <ets>cerevisia</ets>, <ets>cervisia</ets>, beer.]</ety> <def>A small visorless cap, worn by members of German student corps. It is made in the corps colors, and usually bears the insignia of the corps.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Ceria</ent><br/
<hw>Ce"ri*a</hw> <pr>(s<emac/"r<icr/*<adot/)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Cerium oxide, <chform>CeO2</chform>, a white infusible substance constituting about one per cent of the material of the common incandescent mantle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cerial</ent><br/
<hw>Cer"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Cerrial</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceriferous</ent><br/
<hw>Ce*rif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>ra</ets> wax + <ets>-ferous</ets>.]</ety> <def>Producing wax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerin</ent><br/
<hw>Ce"rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax + <ets>-in</ets>: cf. L. <ets>cerinus</ets> wax-colored.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A waxy substance extracted by alcohol or ether from cork; sometimes applied also to the portion of beeswax which is soluble in alcohol.</def>  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>A variety of the mineral <xex>allanite</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerinthian</ent><br/
<hw>Ce*rin"thi*an</hw>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>One of an ancient religious sect, so called from <etsep>Cerinthus</etsep>, a Jew, who attempted to unite the doctrines of Christ with the opinions of the Jews and Gnostics.</def>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceriph</ent><br/
<hw>Cer"iph</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Type Founding)</fld> <def>One of the fine lines of a letter, esp. one of the fine cross strokes at the top and bottom of letters.</def> <altsp>[Spelt also <asp>seriph</asp>.]</altsp>  <rj><au>Savage.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerise</ent><br/
||<hw>Ce*rise"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., a cherry. See <er>Cherry</er>.]</ety> <def>Cherry-colored; a light bright red; -- applied to textile fabrics, especially silk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerite</ent><br/
<hw>Ce"rite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ke`ras</grk> horn.]</ety> <fld>(Zool.)</fld> <def>A gastropod shell belonging to the family <fam>Cerithi<ium/d<ae/</fam>; -- so called from its hornlike form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerite</ent><br/
<hw>Ce"rite</hw>, <pos>n.</pos> <ety>[From <er>Cherium</er>.]</ety> <fld>(Min.)</fld> <def>A mineral of a brownish of cherry-red color, commonly massive. It is a hydrous silicate of cerium and allied metals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerium</ent><br/
<hw>Ce"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named by Berzelius in 1803 from the asteroid <etsep>Ceres</etsep>, then just discovered (1801).]</ety> <fld>(Chem.)</fld> <def>A rare metallic element, occurring in the minerals cerite, allanite, monazite, etc.  Symbol Ce.  Atomic weight 141.5. It resembles iron in color and luster, but is soft, and both malleable and ductile. It tarnishes readily in the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cernuous</ent><br/
<hw>Cer"nu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cernuus</ets> with the face turned toward the earth.]</ety> <fld>(Bot.)</fld> <def>Inclining or nodding downward; pendulous; drooping; -- said of a bud, flower, fruit, or the capsule of a moss.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cero</ent><br/
<hw>Ce"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupt. fr. Sp. <ets>sierra</ets> saw, sawfish, cero.]</ety> <fld>(Zool.)</fld> <def>A large and valuable fish of the Mackerel family, of the genus <gen>Scomberomorus</gen>.  Two species are found in the West Indies and less commonly on the Atlantic coast of the United States, -- the common cero (<spn>Scomberomorus caballa</spn>), called also <altname>kingfish</altname>, and spotted, or king, cero (<spn>Scomberomorus regalis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerograph</ent><br/
<hw>Ce"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <ets><grk>khro`s</grk></ets> wax + <ets>-graph</ets>.]</ety> <def>A writing on wax.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerographical</ent><br/
<ent>Cerographic</ent><br/
<mhw>{ <hw>Ce`ro*graph"ic</hw> <pr>(?)</pr>, <hw>Ce`ro*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to cerography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerographist</ent><br/
<hw>Ce*rog"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who practices cerography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerography</ent><br/
<hw>Ce*rog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>khro`s</grk> wax + <ets>-graphy</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The art of making characters or designs in, or with, wax.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A method of making stereotype plates from inscribed sheets of wax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerolite</ent><br/
<hw>Cer"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>khro`s</grk> wax + <ets>-lite</ets>.]</ety> <fld>(Min.)</fld> <def>A hydrous silicate of magnesium, allied to serpentine, occurring in waxlike masses of a yellow or greenish color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceroma</ent><br/
||<hw>Ce*ro"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/ ointment for wrestlers, the place for wrestling, fr. <grk>khroy^n</grk> to wax over, fr. <grk>khro`s</grk> wax.]</ety> <sn>1.</sn> <def>The unguent (a composition of oil and wax) with which wrestlers were anointed among the ancient Romans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anc. Arch.)</fld> <def>That part of the baths and gymnasia in which bathers and wrestlers anointed themselves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The cere of birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceromancy</ent><br/
<hw>Cer"o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>khro`s</grk> wax + <ets>-mancy</ets>.]</ety> <def>Divination by dropping melted wax in water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceroon</ent><br/
<hw>Ce*roon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Seroon</er>.]</ety> <def>A bale or package. covered with hide, or with wood bound with hide; <as>as, a <ex>ceroon</ex> of indigo, cochineal, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceroplastic</ent><br/
<hw>Ce`ro*plas"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ for modeling in wax; <grk>khro`s</grk> wax + <?/ to form, mold.]</ety> <fld>(Fine arts)</fld> <sd>(a)</sd> <def>Relating to the art of modeling in wax.</def> <sd>(b)</sd> <def>Modeled in wax; <as>as, a <ex>ceroplastic</ex> figure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceroplasty</ent><br/
<ent>Ceroplastics</ent><br/
<mhw>{ <hw>Ce`ro*plas"tics</hw> <pr>(?)</pr>, <hw>Ce`ro*plas"ty</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <?/ (sc. <?/ art): cf. F. <ets>c<eacute/roplastique</ets>.]</ety> <def>The art of modeling in wax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerosin</ent><br/
<hw>Cer"o*sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax.]</ety> <fld>(Chem.)</fld> <def>A waxy substance obtained from the bark of the sugar cane, and crystallizing in delicate white lamin<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerote</ent><br/
<hw>Ce"rote</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Obs</ets>.]</ety> <def>See <er>Cerate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerotene</ent><br/
<hw>Cer"o*tene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cerotum</ets> a pomade. See <er>Cerate</er>.]</ety> <fld>(Chem.)</fld> <def>A white waxy solid obtained from Chinese wax, and by the distillation of cerotin.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 235 --></p>

<p><ent>Cerotic</ent><br/
<hw>Ce*rot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Cerotene</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or derived from, beeswax or Chinese wax; <as>as, <ex>cerotic</ex> acid or alcohol</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerotin</ent><br/
<hw>Cer"o*tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cerotene</er>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline substance, <chform>C27H55.OH</chform>, obtained from Chinese wax, and regarded as an alcohol of the paraffin series; -- called also <altname>cerotic alcohol</altname>, <altname>ceryl alcohol</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerotype</ent><br/
<hw>Ce"ro*type`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ wax + <ets>-type</ets>.]</ety> <def>A printing process of engraving on a surface of wax spread on a steel plate, for electrotyping.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cerrial</ent><br/
<hw>Cer"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cerreus</ets>, fr. <ets>cerrus</ets> a kind of oak.]</ety> <fld>(Bot.)</fld> <def>Of or pertaining to the cerris.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Chaplets green of <qex>cerrial</qex> oak.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerris</ent><br/
||<hw>Cer"ris</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cerrus</ets>.]</ety> <fld>(Bot.)</fld> <def>A species of oak (<spn>Quercus cerris</spn>) native in the Orient and southern Europe; -- called also <altname>bitter oak</altname> and <altname>Turkey oak</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certain</ent><br/
<hw>Cer"tain</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>certain</ets>, fr. (assumed) LL. <ets>certanus</ets>, fr. L. <ets>certus</ets> determined, fixed, certain, orig. p. p. of <ets>cernere</ets> to perceive, decide, determine; akin to Gr. <?/ to decide, separate, and to E. <ets>concern</ets>, <ets>critic</ets>, <ets>crime</ets>, <ets>riddle</ets> a sieve, <ets>rinse</ets>, v.]</ety> <sn>1.</sn> <def>Assured in mind; having no doubts; free from suspicions concerning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To make her <qex>certain</qex> of the sad event.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I myself am <qex>certain</qex> of you.</q> <rj><qau>Wyclif.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Determined; resolved; -- used with an infinitive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>However, I with thee have fixed my lot,<br/
<qex>Certain</qex> to undergo like doom.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not to be doubted or denied; established as a fact.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dream is <qex>certain</qex>, and the interpretation thereof sure.</q> <rj><qau>Dan. ii. 45.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Actually existing; sure to happen; inevitable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Virtue that directs our ways<br/
Through <qex>certain</qex> dangers to uncertain praise.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Death, as the Psalmist saith, is certain to all.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Unfailing; infallible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have often wished that I knew as <qex>certain</qex> a remedy for any other distemper.</q> <rj><qau>Mead.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Fixed or stated; regular; determinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The people go out and gather a <qex>certain</qex> rate every day.</q> <rj><qau>Ex. xvi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Not specifically named; indeterminate; indefinite; one or some; -- sometimes used independenty as a noun, and meaning certain persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It came to pass when he was in a <qex>certain</qex> city.</q> <rj><qau>Luke. v. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>About everything he wrote there was a <qex>certain</qex> natural grace und decorum.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>For certain</b></col>, <cd>assuredly.</cd> -- <col><b>Of a certain</b></col>, <cd>certainly.</cd></cs></p>

<p><syn><b>Syn.</b> -- Bound; sure; true; undeniable; unquestionable; undoubted; plain; indubitable; indisputable; incontrovertible; unhesitating; undoubting; fixed; stated.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certain</ent><br/
<hw>Cer"tain</hw>, <pos>n.</pos> <sn>1.</sn> <def>Certainty.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A certain number or quantity.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certain</ent><br/
<hw>Cer"tain</hw>, <pos>adv.</pos> <def>Certainly.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certainly</ent><br/
<hw>Cer"tain*ly</hw>, <pos>adv.</pos> <def>Without doubt or question; unquestionably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certainness</ent><br/
<hw>Cer"tain*ness</hw>, <pos>n.</pos> <def>Certainty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certainty</ent><br/
<hw>Cer"tain*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Certainties</plw> <pr>(#)</pr>.</plu> <ety>[OF. <ets>certainet<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality, state, or condition, of being certain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>certainty</qex> of punishment is the truest security against crimes.</q>  <rj><au>Fisher Ames.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fact or truth unquestionable established.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Certainties</qex> are uninteresting and sating.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Clearness; freedom from ambiguity; lucidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Of a certainty</b></col>, <cd>certainly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certes</ent><br/
<hw>Cer"tes</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[F. <ets>certes</ets>, for <ets><agrave/ certes</ets>, fr. L. <ets>certus</ets>. See <er>Certain</er>.]</ety> <def>Certainly; in truth; verily.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Certes</qex> it great pity was to see<br/
Him his nobility so foul deface.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certhiidae</ent><br/
<hw>Certhiidae</hw> <pos>n.</pos> <def>a natural family of birds cosisting of several species of creepers, such as <spn>Certhia americana</spn>, the tree creeper.</def><br/
<syn><b>Syn. --</b> family <fam>Certhiidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>certifiable</ent><br/
<hw>certifiable</hw> <pos>adj.</pos> <sn>1.</sn> <def>presenting symptoms of mental illness sufficient for legal commitment to a mental institution.</def> <br/
<syn><b>Syn. --</b> certified.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>capable of being guaranteed or certified: <illu>a <ex>certifiable</ex> fact</illu>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Certificate</ent><br/
<hw>Cer*tif"i*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>certificat</ets>, fr. LL. <ets>certificatus</ets> made certain, p. p. of <ets>certificare</ets>. See <er>tify</er>.]</ety> <sn>1.</sn> <def>A written testimony to the truth of any fact; <as>as, <ex>certificate</ex> of good behavior</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A written declaration legally authenticated.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trial by certificate</b></col>, <cd>a trial which the testimony of the person certifying is the only proper criterion of the point in dispute; as, when the issue is whether a person was absent in the army, this is tried by the certificate of the proper officer in writing, under his seal.</cd>  <rj><au>Blackstone.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certificate</ent><br/
<hw>Cer*tif"i*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Certificated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Certificating</conjf>.]</vmorph> <ety>[See <er>Certify</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To verify or vouch for by certificate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish with a certificate; <as>as, to <ex>certificate</ex> the captain of a vessel; a <ex>certificated</ex> teacher</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>certificated</ent><br/
<hw>certificated</hw> <pos>adj.</pos> <def>officially documented; -- of people.</def><br/
<syn><b>Syn. --</b> credentialed, documented.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Certification</ent><br/
<hw>Cer`ti*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>certificatio</ets>: cf. F. <ets>certification</ets>.]</ety> <def>The act of certifying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>certified</ent><br/
<hw>certified</hw> <pos>adj.</pos> <sn>1.</sn> <def>endorsed authoritatively as having met certain requirements; guranteed; <as>as, <ex>certified</ex> milk; <ex>certified</ex> mail; a <ex>certified</ex> check</as>.  Opposite of <ant>uncertified</ant>.</def> [Narrower terms: <stype>certificated, credentialed, documented</stype>; <stype>guaranteed</stype>] <see>See also <ant>certifiable</ant>.</see><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>legally insane according to clinical criteria.</def><br/
<syn><b>Syn. --</b> certifiable.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>holding appropriate documentation and officially on record as qualified to perform a specified function or practice a specified skill.</def><br/
<syn><b>Syn. --</b> qualified.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>certified check</ent><br/
<hw>certified check</hw> <pos>n.</pos> <def>a check drawn on a bank and bearing marks from that bank guaranteeing that funds have been reserved for payment.</def><br/
[<source>PJC</source>]</p>

<p><ent>Certifier</ent><br/
<hw>Cer"ti*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who certifies or assures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certify</ent><br/
<hw>Cer"ti*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Certified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Certifying</conjf>.]</vmorph> <ety>[F. <ets>certifier</ets>, LL. <ets>certificare</ets>; L. <ets>certus</ets> certain + <ets>facere</ets> to make. See <er>Certain</er>, and cf. <er>Certificate</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To give cetain information to; to assure; to make certain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>certify</qex> the king, that . . . thou shalt have no portion on this side the river.</q> <rj><qau>Ezra iv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give certain information of; to make certain, as a fact; to verify.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The industry of science at once <qex>certifies</qex> and greatly extends our knowledge of the vastness of the creation.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To testify to in writing; to make a declaration concerning, in writing, under hand, or hand and seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The judges shall <qex>certify</qex> their opinion to the chancellor, and upon such certificate the decree is usually founded.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Certified check</b></col>, <cd>A bank check, the validity of which is certified by the bank on which it is drawn.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certiorari</ent><br/
<hw>Cer`ti*o*ra"ri</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named from the emphatic word <ets>certiorari</ets> in the Latin form of the writ, which read <ets>certiorar volumus</ets> we wish to be certified.]</ety> <fld>(Law)</fld> <def>A writ issuing out of chancery, or a superior court, to call up the records of a inferior court, or remove a cause there depending, in order that the party may have more sure and speedy justice, or that errors and irregularities may be corrected. It is obtained upon complaint of a party that he has not received justice, or can not have an impartial trial in the inferior court.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>certiorari</xex> is the correct process to remove the proceedings of a court in which cases are tried in a manner different from the course of the common law, as of county commissioners. It is also used as an auxiliary process in order to obtain a full return to some other process.</note>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Certitude</ent><br/
<hw>Cer"ti*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>certitudo</ets>, fr. L. <ets>certus</ets>: cf. F.  <ets>certitude</ets>. See <er>Certain</er>.]</ety> <def>Freedom from doubt; assurance; certainty.</def>  <rj><au>J. H. Newman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerule</ent><br/
<hw>Cer"ule</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caerulus</ets>, eguiv. to <ets>caeruleus</ets>.]</ety> <def>Blue; cerulean.</def> <mark>[Obs.]</mark>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerulean</ent><br/
<hw>Ce*ru"le*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caeruleus</ets>.]</ety> <def>Sky-colored; blue; azure.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Blue, blue, as if that sky let fall</q><br/
[<source>1913 Webster</source>]</p>

<p><q>A flower from its <qex>cerulean</qex> wall.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerulein</ent><br/
<hw>Ce*ru"le*in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caeruleus</ets> sky-blue.]</ety> <fld>(Chem.)</fld> <def>A fast dyestuff, <chform>C20H8O6</chform>, made by heating gallein with strong sulphuric acid. It dyes mordanted fabrics green.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cerulenin</ent><br/
<hw>Ce`ru*len"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[isolated from Cephalosporium <ets>caerulens</ets>.]</ety> <fld>(Chem.)</fld> <def>an antifungal antibiotic, <chform>C12H17NO3</chform>.  It inhibits the growth of yeasts by interfering with the synthesis of sterols and fatty acids.</def><br/
[<source>PJC</source>]</p>

<p><ent>Ceruleous</ent><br/
<hw>Ce*ru"le*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cerulean.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerulescent</ent><br/
<hw>Ce`ru*les"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caeruleus</ets> sky-blue + <ets>-escent</ets>.]</ety> <def>Tending to cerulean; light bluish.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Ceruleum</ent><br/
<hw>Ce*ru"le*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A greenish blue pigment prepared in various ways, consisting essentially of cobalt stannate. Unlike other cobalt blues, it does not change color by gaslight.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cerulific</ent><br/
<hw>Cer`u*lif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caerulus</ets> dark blue + <ets>facere</ets> to make.]</ety> <def>Producing a blue or sky color.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerumen</ent><br/
||<hw>Ce*ru"men</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>cera</ets> wax.]</ety> <fld>(Physiol.)</fld> <def>The yellow, waxlike secretion from the glands of the external ear; the earwax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceruminous</ent><br/
<hw>Ce*ru"mi*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Pertaining to, or secreting, cerumen; <as>as, the <ex>ceruminous</ex> glands</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceruse</ent><br/
<hw>Ce"ruse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>c<eacute/ruse</ets>, L. <ets>cerussa</ets>.]</ety> <sn>1.</sn> <def>White lead, used as a pigment. See <cref>White lead</cref>, under <er>White</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cosmetic containing white lead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To distinguish <qex>ceruse</qex> from natural bloom.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Min.)</fld> <def>The native carbonate of lead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerused</ent><br/
<hw>Ce"rused</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Washed with a preparation of white lead; <as>as, <ex>cerused</ex> face</as>.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cerussite</ent><br/
<ent>Cerusite</ent><br/
<mhw>{ <hw>Ce"ru*site</hw> <pr>(?)</pr>, <hw>Ce"rus*site</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Min.)</fld> <def>Native lead carbonate; a mineral occurring in colorless, white, or yellowish transparent crystals, with an adamantine, also massive and compact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cervantite</ent><br/
<hw>Cer"van*tite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named from <ets>Cervantes</ets> a town in Spain.]</ety> <fld>(Min.)</fld> <def>See under <er>Antimony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cervelat</ent><br/
<hw>Cer"ve*lat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Mus.)</fld> <def>An ancient wind instrument, resembling the bassoon in tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cervical</ent><br/
<hw>Cer"vi*cal</hw>, <pos>a.</pos> <ety>[L. <ets>cervix</ets>, <ets>-icis</ets>, neck: cf. F. <ets>cervical</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the neck; <as>as, the <ex>cervical</ex> vertebr<ae/</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cervicide</ent><br/
<hw>Cer"vi*cide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cervus</ets> deer + <ets>caedere</ets> to kill.]</ety> <def>The act of killing deer; deer-slaying.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cervid</ent><br/
<hw>cervid</hw> <pos>n.</pos> <def>any member of the family <fam>Cervidae</fam>, a type of deer distinguished from the <fam>Bovidae</fam> by the male's having solid deciduous antlers.</def><br/
<syn><b>Syn. --</b> deer.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cervidae</ent><br/
<hw>Cervidae</hw> <pos>n.</pos> <def>a natural family of deer including the reindeer, moose, elks, muntjacs, and roe deer.</def><br/
<syn><b>Syn. --</b> family Cervidae.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cervine</ent><br/
<hw>Cer"vine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cervinus</ets>, fr. <ets>cervus</ets> deer: cf. F. <ets>cervin</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the deer, or to the family <fam>Cervid<ae/</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cervix</ent><br/
||<hw>Cer"vix</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Cervixes</plw> <pr>(#)</pr>, L. <plw>Cervices</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Anat.)</fld> <def>The neck; also, the necklike portion of any part, as of the womb. See <xex>Illust.</xex> of <er>Bird</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cervus</ent><br/
||<hw>Cer"vus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a deer.]</ety> <fld>(Zool.)</fld> <def>A genus of ruminants, including the red deer and other allied species.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Formerly all species of deer were included in the genus <gen>Cervus</gen>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceryl</ent><br/
<hw>Ce"ryl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cera</ets> wax + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>A radical, <chform>C27H55</chform> supposed to exist in several compounds obtained from Chinese wax, beeswax, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesarian</ent><br/
<ent>Cesarean</ent><br/
<mhw>{ <hw>Ce*sa"re*an</hw> <pr>(?)</pr>, <hw>Ce*sa"ri*an</hw>, }</mhw> <pos>a.</pos> <def>Same as <er>C<ae/sarean</er>, <er>C<ae/sarian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cesarian</ent><br/
<ent>cesarean</ent><br/
<mhw><hw>cesarean</hw> <hw>cesarian</hw></mhw> <pos>adj.</pos> <sn>1.</sn>  <def>same as <er>caesarian</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cesarian</ent><br/
<ent>cesarean</ent><br/
<mhw><hw>cesarean</hw>, <hw>cesarian</hw></mhw> <pos>n.</pos> <def>Same as <er>cesarean section</er>.</def><br/
<syn><b>Syn. --</b> cesarean section, cesarian section, caesarean section, C-section, cesarean, caesarean.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cesarean section</ent><br/
<hw>cesarean section</hw> <pos>n.</pos> <fld>(Surg.)</fld>, <def>the operation of taking a child from the womb by cutting through the walls of the abdomen and uterus; -- so called because Julius C<ae/sar is reported to have been brought into the world by such an operation; -- called also <altname>caesarean</altname>.</def><br/
<syn><b>Syn. --</b> caesarean section, cesarian section, caesarian section, C-section, cesarean, cesarian.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesarism</ent><br/
<hw>Ce"sar*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>C<ae/sarism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cesium</ent><br/
<hw>cesium</hw> <pos>n.</pos> <def>the chemical <isa>element</isa> of atomic number 55.  It is a univalent element, the most electropositive metal.  Symbol Cs; atomic weight 132.905.  IT has a melting point of 28.4<deg/ C.</def><br/
<syn><b>Syn. --</b> caesium, Cs.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cespitine</ent><br/
<hw>Ces"pi*tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>caespes</ets>, <ets>caespitis</ets>, a turf.]</ety> <def>An oil obtained by distillation of peat, and containing various members of the pyridine series.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cespititious</ent><br/
<hw>Ces"pi*ti`tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caespiticius</ets>, fr. <ets>caespes</ets> turf.]</ety> <def>Same as <er>Cespitious</er>.</def> <mark>[R.]</mark>  <rj><au>Gough.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cespitose</ent><br/
<hw>Ces"pi*tose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>caespes</ets> turf.]</ety> <fld>(Bot.)</fld> <def>Having the form a piece of turf, <it>i. e.</it>, many stems from one rootstock or from many entangled rootstocks or roots.</def> <altsp>[Written also <asp>c<ae/spitose</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cespitous</ent><br/
<hw>Ces"pi*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Cespitose</er>.]</ety> <def>Pertaining to, consisting, of resembling, turf; turfy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cespitous</qex> or turfy plant has many stems from the same root, usually forming a close, thick carpet of matting.</q>  <rj><au>Martyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cess</ent><br/
<hw>Cess</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>sess</ets>, conts. from <er>Assess</er>.]</ety> <sn>1.</sn> <def>A rate or tax.</def> <mark>[Obs. or Prof. Eng. & Scot.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bound; measure.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The poor jade is wrung in the withers out of all <qex>cess</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cess</ent><br/
<hw>Cess</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cessed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cessing</conjf>.]</vmorph> <def>To rate; to tax; to assess.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cess</ent><br/
<hw>Cess</hw>, <pos>v. i.</pos> <ety>[F. <ets>cesser</ets>. See <er>Cease</er>.]</ety> <def>To cease; to neglect.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessant</ent><br/
<hw>Ces"sant</hw> <pr>(?)</pr> <pos>a.</pos> <ety>[L. <ets>cessans</ets>, p. pr. of <ets>cessare</ets>. See <er>Cease</er>.]</ety> <def>Inactive; dormant</def> <mark>[Obs.]</mark>  <rj><au>W. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessation</ent><br/
<hw>Ces*sa"tion</hw> <pr>(s<ecr/s*s<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>cessation</ets>, L. <ets>cessatio</ets>, fr. <ets>cessare</ets>. See <er>Cease</er>.]</ety> <def>A ceasing or discontinuance, as of action, whether temporary or final; a stop; <as>as, a <ex>cessation</ex> of the war</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The temporary <qex>cessation</qex> of the papal iniquities.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The day was yearly observed for a festival by <qex>cessation</qex> from labor.</q> <rj><qau>Sir J. Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cessation of arms</b></col> <fld>(Mil.)</fld>, <cd>an armistice, or truce, agreed to by the commanders of armies, to give time for a capitulation, or for other purposes.</cd></cs></p>

<p><syn><b>Syn.</b> -- Stop; rest; stay; pause; discontinuance; intermission; interval; respite; interruption; recess; remission.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessavit</ent><br/
||<hw>Ces*sa"vit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., he has ceased.]</ety> <mark>[O. Eng. Law]</mark> <def>A writ given by statute to recover lands when the tenant has for two years failed to perform the conditions of his tenure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesser</ent><br/
<hw>Ces"ser</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cess</er>, <pos>v. i.</pos>]</ety> <fld>(Law)</fld> <def>a neglect of a tenant to perform services, or make payment, for two years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessible</ent><br/
<hw>Ces"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cessible</ets>. See <er>Cession</er>.]</ety> <def>Giving way; yielding.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Ces`si*bil"i*ty</wf> <pr>(#)</pr>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms>  <rj><au>Sir K. Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cession</ent><br/
<hw>Ces"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cessio</ets>, fr. <ets>cedere</ets> to give way: cf. F. <ets>Cession</ets>. See <er>Cede</er>.]</ety> <sn>1.</sn> <def>A yielding to physical force.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Concession; compliance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A yielding, or surrender, as of property or rights, to another person; the act of ceding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cession</qex> of the island of New Orleans.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eccl. Law)</fld> <def>The giving up or vacating a benefice by accepting another without a proper dispensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Civil Law)</fld> <def>The voluntary surrender of a person's effects to his creditors to avoid imprisonment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessionary</ent><br/
<hw>Ces"sion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>cessionarius</ets>, from <ets>cessionare</ets> to cede, fr. L. <ets>cessio</ets>: cf. F. <ets>cessionnaire</ets>. See <er>Cession</er>.]</ety> <def>Having surrendered the effects; <as>as, a <ex>cessionary</ex> bankrupt</as>.</def>  <rj><au>Martin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessment</ent><br/
<hw>Cess"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cess</er>, <pos>v. t.</pos>]</ety> <def>An assessment or tax.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessor</ent><br/
<hw>Ces"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cess</er>, <pos>v. i.</pos> Cf. <er>Cesser</er>.]</ety> <fld>(Law)</fld> <def>One who neglects, for two years, to perform the service by which he holds lands, so that he incurs the danger of the writ of cessavit. See <er>Cessavit</er>.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cessor</ent><br/
<hw>Ces"sor</hw>, <pos>n.</pos> <ety>[From <er>Cess</er>, <pos>v. t.</pos>]</ety> <def>An assessor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesspipe</ent><br/
<hw>Cess"pipe`</hw> <pr>(s<ecr/s"p<imac/p`)</pr>, <pos>n.</pos> <def>A pipe for carrying off waste water, etc., from a sink or cesspool.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesspool</ent><br/
<hw>Cess"pool`</hw> <pr>(-p<oomac/l`)</pr>, <pos>n.</pos> <ety>[See <er>Sesspol</er>.]</ety> <def>A cistern in the course, or the termination, of a drain, to collect sedimentary or superfluous matter; a privy vault; any receptacle of filth.</def> <altsp>[Written also <asp>sesspool</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cest</ent><br/
<hw>Cest</hw> <pr>(s<ecr/st)</pr>, <pos>n.</pos> <ety>[L. <ets>cestus</ets>: cf. OF. <ets>ceste</ets>.]</ety> <def>A woman's girdle; a cestus.</def> <mark>[R.]</mark>  <rj><au>Collins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestidae</ent><br/
<hw>Cestidae</hw> <pos>n.</pos> <def>A family of invertebrates coextensive with the order <ord>Cestida</ord>; ctenophores having a greatly flattened and elongated body.</def><br/
<syn><b>Syn. --</b> family <fam>Cestidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cestoda</ent><br/
<hw>Ces*tod"a</hw> <pr>(s<ecr/s*t<omac/d"<adot/)</pr>, <pos>prop. n.</pos> <ety>[NL., gr. Gr. <grk>kesto`s</grk> girdle + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>A subclass of parasitic worms of the class <class>Cestoidea</class>.  In some classifications, it is not differentiated from the parent class.</def><br/
[<source>PJC</source>]</p>

<p><ent>cestode</ent><br/
<hw>ces"tode</hw> <pr>(s<ecr/s"t<omac/d)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the <class>Cestoidea</class>.</def> -- <def2><pos>n.</pos> <def>One of the <class>Cestoidea</class>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>cestoid</ent><br/
<hw>ces"toid</hw> <pr>(s<ecr/s"toid)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the <class>Cestoidea</class>.</def> -- <def2><pos>n.</pos> <def>One of the Cestoda.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestoidea</ent><br/
||<hw>Ces*toid"e*a</hw> <pr>(s<ecr/s*toid"<esl/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., gr. Gr. <grk>kesto`s</grk> girdle + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>A class of parasitic worms of the phylum <phylum>Platyhelminthes</phylum> (formerly <phylum>Platelminthes</phylum>) of which the tapeworms are the most common examples.  The body is flattened, and usually but not always long, and composed of numerous joints or segments, each of which may contain a complete set of male and female reproductive organs. They have neither mouth nor intestine. See <er>Tapeworm</er>.</def> <altsp>[Written also <asp>Cestoidea</asp>.]</altsp><br/
<syn><b>Syn. --</b> class <class>Cestoidea</class>, Cestoidea.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestoidean</ent><br/
<hw>Ces*toid"e*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cestoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestraciont</ent><br/
<hw>Ces*tra"ci*ont</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a kind of fish.]</ety> <fld>(Zool.)</fld> <def>A shark of the genus <gen>Cestracion</gen>, and of related genera. The posterior teeth form a pavement of bony plates for crushing shellfish. Most of the species are extinct. The Port Jackson shark and a similar one found in California are living examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestraciont</ent><br/
<hw>Ces*tra"ci*ont</hw>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to, or characteristic of, the genus <gen>Cestracion</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestus</ent><br/
<hw>Ces"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cestus</ets> girdle, Gr. <?/, lit., stitched, embroidered.]</ety> <sn>1.</sn> <fld>(Antiq.)</fld> <def>A girdle; particularly that of Aphrodite (or Venus) which gave the wearer the power of exciting love.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of Ctenophora. The typical species (<spn>Cestus Veneris</spn>) is remarkable for its brilliant iridescent colors, and its long, girdlelike form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestus</ent><br/
<hw>Ces"tus</hw>, <pos>n.</pos> <ety>[L. <ets>caestus</ets>, and <ets>cestus</ets>.]</ety> <fld>(Antiq.)</fld> <def>A covering for the hands of boxers, made of leather bands, and often loaded with lead or iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cestui</ent><br/
<ent>Cestuy</ent><br/
<mhw>{ ||<hw>Ces"tuy</hw> or ||<hw>Ces"tui</hw> <pr>(?)</pr>, }</mhw> <pos>pron.</pos> <ety>[Norm. F.]</ety> <fld>(Law)</fld> <def>He; the one.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cestuy que trust</b></col> <pr>(<?/)</pr> <ety>[norm. F.]</ety>, <cd>a person who has the equitable and beneficial interest in property, the legal interest in which is vested in a trustee.</cd> <au>Wharton.</au> -- <col><b>Cestuy que use</b></col> <pr>(<?/)</pr> <ety>[Norm. F.]</ety>, <cd>a person for whose use land, etc., is granted to another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesura</ent><br/
<hw>Ce*su"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>C<ae/sura</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cesural</ent><br/
<hw>Ce*su"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>C<ae/sural</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetacea</ent><br/
||<hw>Ce*ta"ce*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>cetus</ets> whale, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>An order of marine mammals, including the whales. Like ordinary mammals they breathe by means of lungs, and bring forth living young which they suckle for some time. The anterior limbs are changed to paddles; the tail flukes are horizontal. There are two living suborders:</def> <sd>(a)</sd> <stype>The <suborder>Mysticete</suborder> or whalebone whales, having no true teeth after birth, but with a series of plates of whalebone [see <er>Baleen</er>.] hanging down from the upper jaw on each side, thus making a strainer, through which they receive the small animals upon which they feed.</stype> <sd>(b)</sd> <stype>The <suborder>Denticete</suborder>, including the dolphins and sperm whale, which have teeth. Another suborder (<suborder>Zeuglodontia</suborder>) is extinct. The <ord>Sirenia</ord> were formerly included in the Cetacea, but are now made a separate order.</stype><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetacean</ent><br/
<hw>Ce*ta"cean</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cetacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetaceous</ent><br/
<hw>Ce*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Cetacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cete</ent><br/
||<hw>Ce"te</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., <ets>pl</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the Cetacea, or collectively, the Cetacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetene</ent><br/
<hw>Ce"tene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cete</er>.]</ety> <fld>(Chem.)</fld> <def>An oily hydrocarbon, <chform>C16H32</chform>, of the ethylene series, obtained from spermaceti.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceterach</ent><br/
<hw>Cet"e*rach</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>c<eacute/t<eacute/rac</ets>, fr. Ar. <ets>shetrak</ets>.]</ety> <fld>(Bot.)</fld> <def>A species of fern with fronds (<spn>Asplenium Ceterach</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetewale</ent><br/
<hw>Cet"e*wale</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>citoal</ets>, F. <ets>zedoaire</ets>. See <er>Zedoary</er>.]</ety> <def>Same as <er>Zedoary</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetic</ent><br/
<hw>Ce"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a whale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetin</ent><br/
<hw>Ce"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cetus</ets> whale.]</ety> <fld>(Chem.)</fld> <def>A white, waxy substance, forming the essential part of spermaceti.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 236 --></p>

<p><ent>Cetological</ent><br/
<hw>Ce`to*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to cetology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetologist</ent><br/
<hw>Ce*tol"o*gist</hw> <pr>(?)</pr>, <pos>a.</pos> <def>One versed in cetology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetology</ent><br/
<hw>Ce*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ whale + <ets>-logy</ets>: cf. F. <ets>c<eacute/tologie</ets>.]</ety> <def>The description or natural history of cetaceous animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetraric</ent><br/
<hw>Ce*trar"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or derived from, the lichen, Iceland moss (<spn>Cetaria Islandica</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cetraric acid</b></col>. <cd>See <er>Cetrarin</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetrarin</ent><br/
<hw>Cet"ra*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Cetraria</ets> Islandica, the scientific name of Iceland moss.]</ety> <fld>(Chem.)</fld> <def>A white substance extracted from the lichen, Iceland moss (<spn>Cetraria Islandica</spn>). It consists of several ingredients, among which is <xex>cetraric acid</xex>, a white, crystalline, bitter substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetyl</ent><br/
<hw>Ce"tyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ whale + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>A radical, <chform>C16H33</chform>, not yet isolated, but supposed to exist in a series of compounds homologous with the ethyl compounds, and derived from spermaceti.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cetylic</ent><br/
<hw>Ce*tyl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, spermaceti.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cetylic alcohol</b></col> <fld>(Chem.)</fld>, <cd>a white, waxy, crystalline solid, obtained from spermaceti, and regarded as homologous with ordinary, or ethyl, alcohol; ethal; -- called also <altname>cetyl alcohol</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceylanite</ent><br/
<hw>Cey"lan*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>Ceylan</ets> Ceylon.]</ety> <fld>(Min.)</fld> <def>A dingy blue, or grayish black, variety of spinel. It is also called <altname>pleonaste</altname>.</def> <altsp>[Written also <asp>ceylonite</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ceylon</ent><br/
<hw>Ceylon</hw> <pos>n.</pos> <def>the former name of an island republic in the Indian Ocean off the southeast coast of India, now called <altname>Sri Lanka</altname>.  It is an independent nation with an area of 25,332 sq. mi. and a population of about 14 million people.</def><br/
<syn><b>Syn. --</b> Sri Lanka.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ceylonese</ent><br/
<hw>Cey`lon*ese"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Ceylon; -- since the change of name, replaced by <altname>Sri Lankan</altname>.</def> -- <def2><pos>n. sing. & pl.</pos> <def>A native or natives of Ceylon; -- since the change of name, replaced by <altname>Sri Lankan</altname>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>C. G. S.</ent><br/
<hw>C. G. S.</hw> <def>An abbreviation for <er>Centimeter</er>, <er>Gram</er>, <er>Second</er>. -- applied to a system of units much employed in physical science, based upon the centimeter as the unit of length, the gram as the unit of weight or mass, and the second as the unit of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>C. G. T.</ent><br/
<hw>C. G. T.</hw> <def>An abbreviation for Conf<eacute/d<eacute/ration G<eacute/n<eacute/rale du Travail (the French syndicalist labor union).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cha</ent><br/
<hw>Cha</hw> <pr>(ch<aum/)</pr>, <pos>n.</pos> <ety>[Chin. <ets>ch<lsquo/a</ets>.]</ety> <altsp>[Also <asp>chaa</asp>, <asp>chais</asp>, <asp>tsia</asp>, etc.]</altsp> <def>Tea; -- the Chinese (Mandarin) name, used generally in early works of travel, and now for a kind of rolled tea used in Central Asia.</def></p>

<p><q>A pot with hot water . . . made with the powder of a certain herb called <qex>chaa</qex>, which is much esteemed.</q>  <rj><qau>Tr. J. Van Linschoten's Voyages (1598).</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chab</ent><br/
<hw>Chab</hw> <pr>(ch<acr/b)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The red-bellied woodpecker (<spn>Melanerpes Carolinus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cabazite</ent><br/
<ent>Chabasite</ent><br/
<mhw>{ <hw>Chab"a*site</hw> <pr>(k<acr/b"<adot/*s<imac/t)</pr>, <hw>Cab"a*zite</hw> <pr>(k<acr/b"<adot/*z<imac/t)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <grk>chabazi`os</grk> one of twenty species of stones mentioned in the poem <grk>Peri` li`qwn</grk>, ascribed to Orpheus.]</ety> <fld>(Min.)</fld> <def>A mineral occuring in glassy rhombohedral crystals, varying in color from white to yellow or red. It is essentially a hydrous silicate of alumina and lime. Called also <altname>chabasie</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chablis</ent><br/
||<hw>Cha*blis"</hw> <pr>(sh<adot/*bl<emac/")</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A white wine made near <etsep>Chablis</etsep>, a town in France.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a white wine resembling Chablis{1}, but made elsewhere, as in California.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chabuk</ent><br/
<ent>Chabouk</ent><br/
<mhw>{ ||<hw>Cha*bouk"</hw>, ||<hw>Cha*buk"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Hind. <ets>ch<amac/buk</ets> horsewhip.]</ety> <def>A long whip, such as is used in the East in the infliction of punishment.</def>  <rj><au>Balfour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chace</ent><br/
<hw>Chace</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 3d <er>Chase</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chace</ent><br/
<hw>Chace</hw>, <pos>v. t.</pos> <def>To pursue. See <er>Chase</er> <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cha-cha</ent><br/
<hw>cha-cha</hw> <pos>n.</pos> <def>a modern ballroom dance from Latin America; it uses two slow steps followed by three quick small steps, with swaying movements of the hips, and has many variations of movements based on that rhythm.</def><br/
<syn><b>Syn. --</b> cha-cha.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>cha-cha-cha</ent><br/
<hw>cha-cha-cha</hw> <pos>n.</pos> <def>same as <er>cha-cha</er>.</def><br/
<syn><b>Syn. --</b> cha-cha.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chachalaca</ent><br/
||<hw>Cha`cha*la"ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name, prob. given in imitation of its cry.]</ety> <fld>(Zool.)</fld> <def>The Texan guan (<spn>Ortalis vetula</spn>).</def> <altsp>[written also <asp>chiacalaca</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chack</ent><br/
<hw>Chack</hw> <pr>(ch<acr/k)</pr>, <pos>v. i.</pos> <def>To toss up the head frequently, as a horse to avoid the restraint of the bridle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chacma</ent><br/
||<hw>Chac"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <def>A large species of African baboon (<spn>Cynocephalus porcarius</spn>); -- called also <altname>ursine baboon</altname>. <note>[See <xex>Illust.</xex> of <er>Baboon</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaconne</ent><br/
||<hw>Cha*conne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Sp. <ets>chacona</ets>.]</ety> <fld>(Mus.)</fld> <def>An old Spanish dance in moderate three-four measure, like the Passacaglia, which is slower. Both are used by classical composers as themes for variations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chad</ent><br/
<hw>Chad</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shad</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaenopsis</ent><br/
<hw>Chaenopsis</hw> <pos>n.</pos> <def>a genus of clinid fishes consisting of the pikeblennies.</def><br/
<syn><b>Syn. --</b> genus <gen>Chaenopsis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaeronea</ent><br/
<hw>Chaeronea</hw> <pos>n.</pos> <fld>(Greek antiquity)</fld> <def>either of two battles in ancient Greece, one in which Philip of Macedon defeated the Athenians and Thebans (338 BC), or another in which Sulla defeated Mithridates (86 BC).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chaeta</ent><br/
<hw>chae"ta</hw> <pr>(k<emac/"t<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>chaetae</plw> <pr>(k<emac/"t<emac/)</pr></plu>. <ety>[NL., fr. Gr. <grk>chai`th</grk> hair.]</ety> <fld>(Zool.)</fld> <def>a stiff chitinous seta or bristle especially of an annelid worm.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chaetal</ent><br/
<hw>chaetal</hw> <pos>adj.</pos> <def>of or relating to chaetae (setae or bristles).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaetetes</ent><br/
||<hw>Ch<ae/*te"tes</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[NL., fr. Gr. <grk>chai`th</grk> hair.]</ety> <fld>(Zool.)</fld> <def>A genus of fossil corals, common in the lower Silurian limestones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaetiferous</ent><br/
<hw>Ch<ae/*tif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>chai`th</grk> hair + <ets>-ferous</ets>.]</ety> <fld>(Zool.)</fld> <def>Bearing set<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaetodont</ent><br/
<hw>Ch<ae/"to*dont</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chai`th</grk> hair + <?/, <?/, tooth.]</ety> <fld>(Zool.)</fld> <def>A marine fish of the family <fam>Ch<ae/todontid<ae/</fam>. The ch<ae/todonts have broad, compressed bodies, and usually bright colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaetodont</ent><br/
<hw>Ch<ae/to*dont</hw>, <pos>a.</pos> <def>Of or pertaining to the Ch<ae/todonts or the family <fam>Ch<ae/todontid<ae/</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaetodontidae</ent><br/
<hw>Chaetodontidae</hw> <pos>n.</pos> <def>a natural family comprising the butterfly fishes.</def><br/
<syn><b>Syn. --</b> family <fam>Chaetodontidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaetognath</ent><br/
<hw>Ch<ae/"tog*nath</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Ch<ae/tognatha.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chaetognath</ent><br/
<hw>chaetognath</hw> <pos>n.</pos> <def>any worm of the Chaetognatha; transparent marine worm with horizontal lateral and caudal fins and a row of movable curved spines at each side of the mouth.</def><br/
<syn><b>Syn. --</b> arrowworm.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaetognatha</ent><br/
<ent>Chaetognatha</ent><br/
<mhw><hw>Chae*tog"na*tha</hw>, ||<hw>Ch<ae/*tog"na*tha</hw></mhw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ hair + <?/ jaw.]</ety> <fld>(Zool)</fld> <def>a phylum consisting of the arrowworms, a group of small active transparent free-swimming marine worms, of which the genus <gen>Sagitta</gen> is the type. They have groups of curved spines on each side of the head.</def><br/
<syn><b>Syn. --</b> phylum Chaetognatha.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>chaetognathan</ent><br/
<hw>chaetognathan</hw> <pos>adj.</pos> <def>of or pertaining to the Chaetognatha.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaetopod</ent><br/
<hw>Ch<ae/"to*pod</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to the Ch<ae/topoda.</def> -- <def2><pos>n.</pos> <def>One of the Ch<ae/topoda.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaetopoda</ent><br/
||<hw>Ch<ae/*top"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ hair + <ets>-poda</ets>.]</ety> <fld>(Zool.)</fld> <def>A very extensive order of Annelida (segmented worms), characterized by the presence of lateral set<ae/, or spines, on most or all of the segments.  They are divided into two principal groups: <suborder>Oligoch<ae/ta</suborder>, including the earthworms and allied forms, and <suborder>Polych<ae/ta</suborder>, including most of the marine species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaetotaxy</ent><br/
<hw>Ch<ae/"to*tax`y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ hair + <?/ arrangement.]</ety> <fld>(Zool.)</fld> <def>The arrangement of bristles on an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafe</ent><br/
<hw>Chafe</hw> <pr>(ch<amac/f)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chafed</conjf> <pr>(ch<amac/ft)</pr>; <pos>p pr. & vb. n.</pos> <conjf>Chafing</conjf>.]</vmorph> <ety>[OE. <ets>chaufen</ets> to warm, OF. <ets>chaufer</ets>, F. <ets>chauffer</ets>, fr. L. <ets>calefacere</ets>, <ets>calfacere</ets>, to make warm; <ets>calere</ets> to be warm + <ets>facere</ets> to make. See <er>Caldron</er>.]</ety> <sn>1.</sn> <def>To excite heat in by friction; to rub in order to stimulate and make warm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To rub her temples, and to <qex>chafe</qex> her skin.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To excite passion or anger in; to fret; to irritate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her intercession <qex>chafed</qex> him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fret and wear by rubbing; <as>as, to <ex>chafe</ex> a cable</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two slips of parchment which she sewed round it to prevent its being <qex>chafed</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj></p>

<p><syn><b>Syn.</b> -- To rub; fret; gall; vex; excite; inflame.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafe</ent><br/
<hw>Chafe</hw>, <pos>v. i.</pos> <def>To rub; to come together so as to wear by rubbing; to wear by friction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Made its great boughs <qex>chafe</qex> together.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The troubled Tiber <qex>chafing</qex> with her shores.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be worn by rubbing; <as>as, a cable <ex>chafes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have a feeling of vexation; to be vexed; to fret; to be irritated.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He will <qex>chafe</qex> at the doctor's marrying my daughter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafe</ent><br/
<hw>Chafe</hw>, <pos>n.</pos> <sn>1.</sn> <def>Heat excited by friction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Injury or wear caused by friction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vexation; irritation of mind; rage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cardinal in a <qex>chafe</qex> sent for him to Whitehall.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafer</ent><br/
<hw>Chaf"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who chafes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel for heating water; -- hence, a dish or pan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>chafer</qex> of water to cool the ends of the irons.</q> <rj><qau>Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafer</ent><br/
<hw>Chaf"er</hw>, <pos>n.</pos> <ety>[AS. <ets>ceafor</ets>; akin to D. <ets>kever</ets>, G <ets>k<eum/fer</ets>.]</ety> <fld>(Zool.)</fld> <def>A kind of beetle; the cockchafer. The name is also applied to other species; <as>as, the rose <ex>chafer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafery</ent><br/
<hw>Chaf"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chafe</er>, <pos>v. t.</pos>]</ety> <fld>(Iron Works)</fld> <def>An open furnace or forge, in which blooms are heated before being wrought into bars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffwax</ent><br/
<ent>Chafewax</ent><br/
<mhw>{ <hw>Chafe"wax`</hw> <pr>(?)</pr>, <or/ <hw>Chaff"wax`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Eng. Law)</fld> <def>Formerly a chancery officer who fitted wax for sealing writs and other documents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafeweed</ent><br/
<hw>Chafe"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The cudweed (<gen>Gnaphalium</gen>), used to prevent or cure chafing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaff</ent><br/
<hw>Chaff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AC. <ets>ceaf</ets>; akin to D. <ets>kaf</ets>, G. <ets>kaff</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The glumes or husks of grains and grasses separated from the seed by threshing and winnowing, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So take the corn and leave the <qex>chaff</qex> behind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Old birds are not caught with <qex>caff</qex>.</q> <rj><qau>Old Proverb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything of a comparatively light and worthless character; the refuse part of anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chaff</qex> and ruin of the times.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Straw or hay cut up fine for the food of cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By adding <qex>chaff</qex> to his corn, the horse must take more time to eat it. In this way <qex>chaff</qex> is very useful.</q> <rj><qau>Ywatt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Light jesting talk; banter; raillery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <def>The scales or bracts on the receptacle, which subtend each flower in the heads of many <xex>Composit<ae/</xex>, as the sunflower.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chaff cutter</b></col>, <cd>a machine for cutting, up straw, etc., into <ldquo/chaff<rdquo/ for the use of cattle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaff</ent><br/
<hw>Chaff</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chaffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chaffing</conjf>.]</vmorph> <def>To use light, idle language by way of fun or ridicule; to banter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaff</ent><br/
<hw>Chaff</hw>, <pos>v. t.</pos> <def>To make fun of; to turn into ridicule by addressing in ironical or bantering language; to quiz.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Morgan saw that his master was <qex>chaffing</qex> him.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A dozen honest fellows . . . <qex>chaffed</qex> each other about their sweethearts.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffer</ent><br/
<hw>Chaff"er</hw>, <pos>n.</pos> <def>One who chaffs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffer</ent><br/
<hw>Chaf"fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>chaffare</ets>, <ets>cheapfare</ets>; AS. <ets>ce<aacute/p</ets> a bargain, price + <ets>faru</ets> a journey; hence, originally, a going to barain, to market. See <er>Cheap</er>, and <er>Fare</er>.]</ety> <def>Bargaining; merchandise.</def> <mark>[Obs.]</mark>  <rj><au>Holished.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffer</ent><br/
<hw>Chaf"fer</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chaffered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chaffering</conjf>.]</vmorph> <ety>[OE. <ets>chaffaren</ets>, fr. <ets>chaffare</ets>, <ets>chapfare</ets>, <ets>cheapfare</ets>, a bargaining. See <er>Chaffer</er>, <pos>n.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To treat or dispute about a purchase; to bargain; to haggle or higgle; to negotiate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>chaffer</qex> for preferments with his gold.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To talk much and idly; to chatter.</def>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffer</ent><br/
<hw>Chaf"fer</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To buy or sell; to trade in.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>chaffered</qex> chairs in which churchmen were set.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exchange; to bandy, as words.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafferer</ent><br/
<hw>Chaf"fer*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chaffers; a bargainer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffern</ent><br/
<hw>Chaf"fern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chafe</er>, <pos>v. t.</pos>]</ety> <def>A vessel for heating water.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffery</ent><br/
<hw>Chaf"fer*y</hw>, <pos>n.</pos> <def>Traffic; bargaining.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffinch</ent><br/
<hw>Chaf"finch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chiff-chaff</er>.]</ety> <fld>(Zool.)</fld> <def>A bird of Europe (<spn>Fringilla c<oe/lebs</spn>), having a variety of very sweet songs, and highly valued as a cage bird; -- called also <altname>copper finch</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffing</ent><br/
<hw>Chaff"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The use of light, frivolous language by way of fun or ridicule; raillery; banter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffless</ent><br/
<hw>Chaff"less</hw>, <pos>a.</pos> <def>Without chaff.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaffy</ent><br/
<hw>Chaff"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding in, or resembling, chaff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chaffy</qex> grain beneath the thresher's flail.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Light or worthless as chaff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Slight and <qex>chaffy</qex> opinion.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Resembling chaff; composed of light dry scales.</def> <sd>(b)</sd> <def>Bearing or covered with dry scales, as the under surface of certain ferns, or the disk of some composite flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chafing</ent><br/
<hw>Chaf"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chafe</er>, <pos>v. t.</pos>]</ety> <def>The act of rubbing, or wearing by friction; making by rubbing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chafing dish</b></col>, <cd>a dish or vessel for cooking on the table, or for keeping food warm, either by coals, by a lamp, or by hot water; a portable grate for coals.</cd> -- <col><b>Chafing gear</b></col> <fld>(Naut.)</fld>, <cd>any material used to protect sails, rigging, or the like, at points where they are exposed to friction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chagas' disease</ent><br/
<hw>Chagas' disease</hw> <pos>n.</pos> <ety>[Carlos <etsep>Chagas</etsep>, a physician in Brazil, 1879-1934]</ety> <def>a form of trypanosomiasis caused by infection with <spn>Trypamosoma cruzi</spn>, found principally in South America.  It is transmitted by certain species of reduviid bugs, and has natural reservoirs in dogs, armadillos, rodents and other domestic and wild mammals.</def> <au>Stedman's Med. Dict.</au>, <au>Dorland's Illustr. Med. Dict.</au><br/
<syn><b>Syn. --</b> South Amercian trypanosomiasis, Chagas-Cruz disease.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Chagreen</ent><br/
<hw>Cha*green"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shagreen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chagres fever</ent><br/
<hw>Cha"gres fe"ver</hw> <pr>(?)</pr>. <fld>(Med.)</fld> <def>A form of malarial fever occurring along the Chagres River, Panama.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chagrin</ent><br/
<hw>Cha*grin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>chagrin</ets> shagreen, a particular kind of rough and grained leather; also a rough fishskin used for graters and files; hence (<it>Fig</it>.), a gnawing, corroding grief. See <er>Shagreen</er>.]</ety> <def>Vexation; mortification.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I must own that I felt rather vexation and <qex>chagrin</qex> than hope and satisfaction.</q> <rj><qau>Richard Porson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hear me, and touch Belinda with <qex>chagrin</qex>.</q> <rj><qau>Pope.</qau></rj></p>

<p><syn><b>Syn.</b> -- Vexation; mortification; peevishness; fretfulness; disgust; disquiet.</syn> <usage> <er>Chagrin</er>, <er>Vexation</er>, <er>Mortification</er>. These words agree in the general sense of pain produced by untoward circumstances. <xex>Vexation</xex> is a feeling of disquietude or irritating uneasiness from numerous causes, such as losses, disappointments, etc. <xex>Mortification</xex> is a stronger word, and denotes that keen sense of pain which results from wounded pride or humiliating occurrences. <xex>Chagrin</xex> is literally the cutting pain produced by the friction of <xex>Shagreen</xex> leather; in its figurative sense, it varies in meaning, denoting in its lower degrees simply a state of vexation, and its higher degrees the keenest sense of mortification.</usage></p>

<p><ldquo/<xex>Vexation</xex> arises chiefly from our wishes and views being crossed: <xex>mortification</xex>, from our self-importance being hurt; <xex>chagrin</xex>, from a mixture of the two.<rdquo/ <rj><qau>Crabb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chagrin</ent><br/
<hw>Cha*grin"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chagrined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chargrining</conjf>.]</vmorph> <ety>[Cf. F. <ets>chagriner</ets> See <er>Chagrin</er>, <pos>n.</pos>]</ety> <def>To excite ill-humor in; to vex; to mortify; <as>as, he was not a little <ex>chagrined</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chagrin</ent><br/
<hw>Cha*grin"</hw>, <pos>v. i.</pos> <def>To be vexed or annoyed.</def>  <rj><au>Fielding.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chagrin</ent><br/
<hw>Cha*grin"</hw>, <pos>a.</pos> <def>Chagrined.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chagrined</ent><br/
<hw>chagrined</hw> <pos>adj.</pos> <def>feeling vexed, especially due to feeling inferior or unworthy and hence embarrassed; <as>as, <ex>chagrined</ex> at the poor sales of his book</as>.</def><br/
<syn><b>Syn. --</b> embarrassed, mortified.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chain</ent><br/
<hw>Chain</hw> <pr>(ch<amac/n)</pr>, <pos>n.</pos> <ety>[F. <ets>cha<icir/ne</ets>, fr. L. <ets>catena</ets>. Cf. <er>Catenate</er>.]</ety> <sn>1.</sn> <def>A series of links or rings, usually of metal, connected, or fitted into one another, used for various purposes, as of support, of restraint, of ornament, of the exertion and transmission of mechanical power, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] put a <qex>chain</qex> of gold about his neck.</q> <rj><qau>Dan. v. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which confines, fetters, or secures, as a chain; a bond; <as>as, the <ex>chains</ex> of habit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Driven down<br/
To <qex>chains</qex> of darkness and the undying worm.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A series of things linked together; or a series of things connected and following each other in succession; <as>as, a <ex>chain</ex> of mountains; a <ex>chain</ex> of events or ideas</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Surv.)</fld> <def>An instrument which consists of links and is used in measuring land.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ One commonly in use is <xex>Gunter's chain</xex>, which consists of one hundred links, each link being seven inches and ninety-two one hundredths in length; making up the total length of rods, or sixty-six, feet; hence, a measure of that length; hence, also, a unit for land measure equal to four rods square, or one tenth of an acre.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <fld>(Naut.)</fld> <def>Iron links bolted to the side of a vessel to bold the dead-eyes connected with the shrouds; also, the channels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Weaving)</fld> <def>The warp threads of a web.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chain belt</b></col> <fld>(Mach.)</fld>, <cd>a belt made of a chain; -- used for transmitting power.</cd> -- <col><b>Chain boat</b></col>, <cd>a boat fitted up for recovering lost cables, anchors, etc.</cd> -- <col><b>Chain bolt</b></col> <sd>(a)</sd> <fld>(Naut.)</fld> <cd>The bolt at the lower end of the chain plate, which fastens it to the vessel's side.</cd> <sd>(b)</sd> <cd>A bolt with a chain attached for drawing it out of position.</cd> -- <col><b>Chain bond</b></col>. <cd>See <cref>Chain timber</cref>.</cd> -- <col><b>Chain bridge</b></col>, <cd>a bridge supported by chain cables; a suspension bridge.</cd> -- <col><b>Chain cable</b></col>, <cd>a cable made of iron links.</cd> -- <col><b>Chain coral</b></col> <fld>(Zool.)</fld>, <cd>a fossil coral of the genus <gen>Halysites</gen>, common in the middle and upper Silurian rocks. The tubular corallites are united side by side in groups, looking in an end view like links of a chain. When perfect, the calicles show twelve septa.</cd> -- <col><b>Chain coupling</b></col>. <sd>(a)</sd> <cd>A shackle for uniting lengths of chain, or connecting a chain with an object.</cd> <sd>(b)</sd> <fld>(Railroad)</fld> <cd>Supplementary coupling together of cars with a chain.</cd> -- <col><b>Chain gang</b></col>, <cd>a gang of convicts chained together.</cd> -- <col><b>Chain hook</b></col> <fld>(Naut.)</fld>, <cd>a hook, used for dragging cables about the deck.</cd> -- <col><b>Chain mail</b></col>, <cd>flexible, defensive armor of hammered metal links wrought into the form of a garment.</cd> -- <col><b>Chain molding</b></col> <fld>(Arch.)</fld>, <cd>a form of molding in imitation of a chain, used in the Normal style.</cd> -- <col><b>Chain pier</b></col>, <cd>a pier suspended by chain.</cd> -- <col><b>Chain pipe</b></col> <fld>(Naut.)</fld>, <cd>an opening in the deck, lined with iron, through which the cable is passed into the lockers or tiers.</cd> -- <col><b>Chain plate</b></col> <fld>(Shipbuilding)</fld>, <cd>one of the iron plates or bands, on a vessel's side, to which the standing rigging is fastened.</cd> -- <col><b>Chain pulley</b></col>, <cd>a pulley with depressions in the periphery of its wheel, or projections from it, made to fit the links of a chain.</cd> -- <col><b>Chain pumps</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Chain rule</b></col> <fld>(Arith.)</fld>, <cd>a theorem for solving numerical problems by composition of ratios, or compound proportion, by which, when several ratios of equality are given, the consequent of each being the same as the antecedent of the next, the relation between the first antecedent and the last consequent is discovered.</cd> -- <col><b>Chain shot</b></col> <fld>(Mil.)</fld>, <cd>two cannon balls united by a shot chain, formerly used in naval warfare on account of their destructive effect on a ship's rigging.</cd> -- <col><b>Chain stitch</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Chain timber</b></col>. <fld>(Arch.)</fld> <cd>See <cref>Bond timber</cref>, under <er>Bond</er>.</cd> -- <col><b>Chain wales</b></col>. <fld>(Naut.)</fld> <cd>Same as <er>Channels</er>.</cd> -- <col><b>Chain wheel</b></col>. <cd>See in the Vocabulary.</cd> -- <mcol><col><b>Closed chain</b></col>, <col><b>Open chain</b></col></mcol> <fld>(Chem.)</fld>, <cd>terms applied to the chemical structure of compounds whose rational formul<ae/ are written respectively in the form of a closed ring (see <cref>Benzene nucleus</cref>, under <er>Benzene</er>), or in an open extended form.</cd> -- <col><b>Endless chain</b></col>, <cd>a chain whose ends have been united by a link.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chain</ent><br/
<hw>Chain</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. p. p.</pos> <conjf>Chained</conjf> <pr>(ch<amac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chaining</conjf>.]</vmorph> <sn>1.</sn> <def>To fasten, bind, or connect with a chain; to fasten or bind securely, as with a chain; <as>as, <ex>to chain</ex> a bulldog</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chained</qex> behind the hostile car.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To keep in slavery; to enslave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And which more blest? who <qex>chained</qex> his country, say<br/
Or he whose virtue sighed to lose a day?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite closely and strongly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And in this vow do <qex>chain</qex> my soul to thine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Surveying)</fld> <def>To measure with the chain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To protect by drawing a chain across, as a harbor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chained</ent><br/
<hw>chained</hw> <pos>adj.</pos> <def>bound with chains; <as>as, prisoners <ex>chained</ex> together to prevent escape</as>.</def><br/
<syn><b>Syn. --</b> enchained, in chains(predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 237 --></p>

<p><ent>Chainless</ent><br/
<hw>Chain"less</hw> <pr>(ch<amac/n"l<ecr/s)</pr>, <pos>a.</pos> <def>Having no chain; not restrained or fettered.</def> <ldquo/The <xex>chainless</xex> mind.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chainlet</ent><br/
<hw>Chain"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small chain.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chainlike</ent><br/
<hw>chainlike</hw> <pos>adj.</pos>  <def>having a form resembling a chain; <as>as, <ex>chainlike</ex> clusters of bacteria</as>.</def><br/
<syn><b>Syn. --</b> catenulate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chain pump</ent><br/
<hw>Chain" pump`</hw> <pr>(?)</pr>. <def>A pump consisting of an endless chain, running over a drum or wheel by which it is moved, and dipping below the water to be raised. The chain has at intervals disks or lifts which fit the tube through which the ascending part passes and carry the water to the point of discharge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chains</ent><br/
<hw>chains</hw> <pr>(ch<amac/ns)</pr>, <pos>n.</pos> <def>metal shackles connected by chains, used to bind hands or legs; <as>as, he was kept two weeks in <ex>chains</ex></as>.</def><br/
<syn><b>Syn. --</b> iron, irons, chain.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chainsaw</ent><br/
<hw>chain"saw`</hw> <pr>(ch<amac/n"s<add/)</pr>, <pos>n.</pos> <def>a portable power saw having teeth that are linked to form an endless chain, rotated about two pivot points by a power mechanism, such as an electric motor or a gasoline engine.</def> <note>They are used to efficiently cut trees, logs, or thick branches out  of doors.</note><br/
<syn><b>Syn. --</b> chain saw.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chain stitch</ent><br/
<hw>Chain" stitch`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>An ornamental stitch like the links of a chain; -- used in crocheting, sewing, and embroidery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Machine Sewing)</fld> <def>A stitch in which the looping of the thread or threads forms a chain on the under side of the work; the loop stitch, as distinguished from the lock stitch. See <er>Stitch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chain tie</ent><br/
<hw>Chain tie</hw>. <fld>(Arch.)</fld> <def>A tie consisting of a series of connected iron bars or rods.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chain wheel</ent><br/
<hw>Chain" wheel`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>A chain pulley, or sprocket wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inversion of the chain pump, by which it becomes a motor driven by water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chainwork</ent><br/
<hw>Chain"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work looped or linked after the manner of a chain; chain stitch work.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chair</ent><br/
<hw>Chair</hw> <pr>(ch<acir/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>chaiere</ets>, <ets>chaere</ets>, OF. <ets>chaiere</ets>, <ets>chaere</ets>, F. <ets>chaire</ets> pulpit, fr. L. <ets>cathedra</ets> chair, armchair, a teacher's or professor's chair, Gr. <?/ down + <?/ seat, <?/ to sit, akin to E. <ets>sit</ets>. See <er>Sit</er>, and cf. <er>Cathedral</er>, <er>chaise</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A movable single seat with a back.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An official seat, as of a chief magistrate or a judge, but esp. that of a professor; hence, the office itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chair</qex> of a philosophical school.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>chair</qex> of philology.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The presiding officer of an assembly; a chairman; <as>as, to address the <ex>chair</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A vehicle for one person; either a sedan borne upon poles, or two-wheeled carriage, drawn by one horse; a gig.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Think what an equipage thou hast in air,<br/
And view with scorn two pages and a <qex>chair</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An iron block used on railways to support the rails and secure them to the sleepers.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chair days</b></col>, <cd>days of repose and age.</cd> -- <col><b>To put into the chair</b></col>, <cd>to elect as president, or as chairman of a meeting.</cd> <au>Macaulay</au>. -- <col><b>To take the chair</b></col>, <cd>to assume the position of president, or of chairman of a meeting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chair</ent><br/
<hw>Chair</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chaired</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chairing</conjf>.]</vmorph> <sn>1.</sn> <def>To place in a chair.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To carry publicly in a chair in triumph.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To function as chairperson of (a meeting, committee, etc.); <as>as, he <ex>chaired</ex> the meeting</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>chairlift</ent><br/
<hw>chair"lift</hw> <pos>n.</pos> <def>a ski lift on which riders are seated.</def><br/
<syn><b>Syn. --</b> chair lift.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chairman</ent><br/
<hw>Chair"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chairmen</plw> <pr>(<?/)</pr></plu>. <sn>1.</sn> <def>The presiding officer of a committee, or of a public or private meeting, or of any organized body.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One whose business it is to cary a chair or sedan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Breaks watchmen's heads and <qex>chairmen's</qex> glasses.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chairmanship</ent><br/
<hw>Chair"man*ship</hw>, <pos>n.</pos> <def>The office of a chairman of a meeting or organized body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chairperson</ent><br/
<hw>chairperson</hw> <pos>n.</pos> <def>the officer who presides at the meetings of an organization; -- same as <altname>chair{3}</altname> or <altname>chairman{1}</altname>, but used to achieve a sex-neutral register.</def><br/
<syn><b>Syn. --</b> president, chairman, chairwoman, chair.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>chairwoman</ent><br/
<hw>chairwoman</hw> <pos>n.</pos> <def>a female officer who presides at the meetings of a committee, meeting, or organization; a female chairperson.</def><br/
<syn><b>Syn. --</b> president, chairman, chair, chairperson.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaise</ent><br/
<hw>Chaise</hw> <pr>(sh<amac/z)</pr>, <pos>n.</pos> <ety>[F. <ets>chaise</ets> seat, or chair, chaise or carriage, for <ets>chaire</ets>, from a peculiar Parisian pronunciation. See <er>Chair</er>.]</ety> <sn>1.</sn> <def>A two-wheeled carriage for two persons, with a calash top, and the body hung on leather straps, or thorough-braces. It is usually drawn by one horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>Loosely</mark>, <def>a carriage in general.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaja</ent><br/
||<hw>Cha"ja</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>The crested screamer of Brazil (<spn>Palamedea chavaria</spn> <it>syn.</it> <spn>Chauna chavaria</spn>), so called in imitation of its notes; -- called also <altname>chauna</altname>, and <altname>faithful kamichi</altname>. It is often domesticated and is useful in guarding other poultry. See <er>Kamichi</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalaza</ent><br/
||<hw>Cha*la"za</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Chalazas</plw>, L. <plw>Chalaz<ae/</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ hail, pimple.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The place on an ovule, or seed, where its outer coats cohere with each other and the nucleus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>A spiral band of thickened albuminous substance which exists in the white of the bird's egg, and serves to maintain the yolk in its position; the treadle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalazal</ent><br/
<hw>Cha*la"zal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the chalaza.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalaze</ent><br/
<hw>Cha*laze"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Chalaza</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalaziferous</ent><br/
<hw>Chal`a*zif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chalaza</ets> + <ets>-ferous</ets>.]</ety> <def>Having or bearing chalazas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalazion</ent><br/
||<hw>Cha*la"zi*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ dim. of <?/ hail, pimple.]</ety> <fld>(Med.)</fld> <def>A small circumscribed tumor of the eyelid caused by retention of secretion, and by inflammation of the Melbomian glands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalazogamy</ent><br/
<hw>Chal`a*zog"a*my</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Chalaza</ets> + <ets>-gamy</ets>, as in <ets>polygamy</ets>.]</ety> <fld>(Bot.)</fld> <def>A process of fecundation in which the pollen tube penetrates to the embryosac through the tissue of the chalaza, instead of entering through the micropyle. It was originally discovered by Treub in <gen>Casuarina</gen>, and has since been found to occur regularly in the families <fam>Betulace<ae/</fam> and <fam>Juglandace<ae/</fam>. Partial chalazogamy is found in <gen>Ulmus</gen>, the tube here penetrating the nucleus midway between the chalaza and micropyle.</def> -- <wordforms><wf>Chal`a*zo*gam"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chalcanthite</ent><br/
<hw>Chal*can"thite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>chalcanthum</ets> a solution of blue vitriol, Gr. <?/.]</ety> <fld>(Min.)</fld> <def>Native blue vitriol. See <cref>Blue vitriol</cref>, under <er>Blue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcedonic</ent><br/
<hw>Chal"ce*don"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to chalcedony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcedony</ent><br/
<hw>Chal*ced"o*ny</hw> <pr>(k<acr/l*s<ecr/d"<osl/*n<ycr/ <it>or</it> k<acr/l"s<esl/*d<osl/*n<ycr/; 277)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chalcedonies</plw> <pr>(-n<icr/z)</pr>.</plu> <ety>[ L. <ets>chalcedonius</ets>, fr. Gr. <grk>CHalkhdw`n</grk> Chalcedon, a town in Asia Minor, opposite to Byzantium: cf. <ets>calc<eacute/doine</ets>, OE. <ets>calcidoine</ets>, <ets>casidoyne</ets>. Cf. <er>Cassidony</er>.]</ety> <fld>(Min.)</fld> <def>A cryptocrystalline, translucent variety of quartz, having usually a whitish color, and a luster nearly like wax.</def> <altsp>[Written also <asp>calcedony</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When chalcedony is variegated with with spots or figures, or arranged in differently colored layers, it is called <stype>agate</stype>; and if by reason of the thickness, color, and arrangement of the layers it is suitable for being carved into cameos, it is called <stype>onyx</stype>. <stype>Chrysoprase</stype> is green chalcedony; <stype>carnelian</stype>, a flesh red, and <stype>sard</stype>, a brownish red variety.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalchihuitl</ent><br/
||<hw>Chal`chi*huitl"</hw> <pr>(ch<adot/l`ch<esl/*w<esl/tl")</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>The Mexican name for turquoise. See <er>Turquoise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcid fly</ent><br/
<hw>Chal"cid fly`</hw> <pr>(?)</pr>. <ety>[From Gr. <grk>chalko`s</grk> copper; in allusion to its metallic colors.]</ety> <fld>(Zool.)</fld> <def>One of a numerous family of hymenopterous insects (<fam>Chalcidid<ae/</fam>. Many are gallflies, others are parasitic on insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcidian</ent><br/
<hw>Chal*cid"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>chalcis</ets> a lizard, Gr. <grk>chalki`s</grk>.]</ety> <fld>(Zool.)</fld> <def>One of a tropical family of snakelike lizards (<fam>Chalcid<ae/</fam>), having four small or rudimentary legs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcis</ent><br/
<hw>Chalcis</hw> <pos>n.</pos> <def>the type genus of the <fam>Chalcididae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Chalcis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chalcocite</ent><br/
<hw>Chal"co*cite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chalko`s</grk> brass.]</ety> <fld>(Min.)</fld> <def>Native copper sulphide, called also <altname>copper glance</altname>, and <altname>vitreous copper</altname>; a mineral of a black color and metallic luster.</def> <altsp>[Formerly written <asp>chalcosine</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcographist</ent><br/
<ent>Chalcographer</ent><br/
<mhw>{ <hw>Chal*cog"ra*pher</hw> <pr>(?)</pr>, <hw>Chal*cog"ra*phist</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>An engraver on copper or brass; hence, an engraver of copper plates for printing upon paper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcography</ent><br/
<hw>Chal*cog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chalko`s</grk> copper, brass + <ets>-graphy</ets>.]</ety> <def>The act or art of engraving on copper or brass, especially of engraving for printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalcopyrite</ent><br/
<hw>Chal`co*pyr"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chalko`s</grk> brass + E. <ets>pyrite</ets>. So named from its color.]</ety> <fld>(Min.)</fld> <def>Copper pyrites, or yellow copper ore; a common ore of copper, containing copper, iron, and sulphur. It occurs massive and in tetragonal crystals of a bright brass yellow color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaldaic</ent><br/
<hw>Chal*da"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Chaldaicus</ets>.]</ety> <def>Of or pertaining to Chaldea.</def> -- <def2><pos>n.</pos> <def>The language or dialect of the Chaldeans; Chaldee.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaldaism</ent><br/
<hw>Chal"da*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An idiom or peculiarity in the Chaldee dialect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaldean</ent><br/
<hw>Chal*de"an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Chaldaeus</ets>.]</ety> <def>Of or pertaining to Chaldea.</def>  -- <def2><pos>n.</pos> <sd>(a)</sd> <def>A native or inhabitant of Chaldea.</def> <sd>(b)</sd> <def>A learned man, esp. an astrologer; -- so called among the Eastern nations, because astrology and the kindred arts were much cultivated by the Chaldeans.</def> <sd>(c)</sd> <def>Nestorian.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaldee</ent><br/
<hw>Chal"dee</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Chaldea.</def> -- <def2><pos>n.</pos> <def>The language or dialect of the Chaldeans; eastern Aramaic, or the Aramaic used in Chaldea.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chaldee Paraphrase</b></col>, <cd>A targum written in Aramaic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalder</ent><br/
<ent>Chaldrich</ent><br/
<mhw>{ <hw>Chal"drich</hw> <pr>(?)</pr>, <hw>Chal"der</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Icel. <ets>tjaldr</ets>.]</ety> <fld>(Zool.)</fld> <def>A kind of bird; the oyster catcher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaldron</ent><br/
<hw>Chal"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chaldron</ets>, F. <ets>chaudron</ets> kettle. The same word as <ets>caldron</ets>.]</ety> <def>An English dry measure, being, at London, 36 bushels heaped up, or its equivalent weight, and more than twice as much at Newcastle. Now used exclusively for coal and coke.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States the <xex>chaldron</xex> is ordinarily 2,940 lbs, but at New York it is 2,500 lbs.</note>  <rj><au>De Colange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalet</ent><br/
||<hw>Cha*let"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A herdsman's hut in the mountains of Switzerland.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chalets</qex> are summer huts for the Swiss herdsmen.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A summer cottage or country house in the Swiss mountains; any country house built in the style of the Swiss cottages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalice</ent><br/
<hw>Chal"ice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OR. <ets>chalis</ets>, <ets>calice</ets>, OF. <ets>chalice</ets>, <ets>calice</ets>, F. <ets>calice</ets>, fr. L. <ets>calix</ets>, akin to Gr. <?/ and E. <ets>helmet</ets>. Cf. <er>Calice</er>, <er>Calyx</er>.]</ety> <def>A cup or bowl; especially, the cup used in the sacrament of the Lord's Supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaliced</ent><br/
<hw>Chal"iced</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a calyx or cup; cup-shaped.</def> <ldquo/<xex>Chaliced</xex> flowers.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalk</ent><br/
<hw>Chalk</hw> <pr>(ch<add/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>cealc</ets> lime, from L. <ets>calx</ets> limestone. See <er>Calz</er>, and <er>Cawk</er>.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>A soft, earthy substance, of a white, grayish, or yellowish white color, consisting of calcium carbonate, and having the same composition as common limestone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fine Arts)</fld> <def>Finely prepared chalk, used as a drawing implement; also, by extension, a compound, as of clay and black lead, or the like, used in the same manner. See <er>Crayon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black chalk</b></col>, <cd>a mineral of a bluish color, of a slaty texture, and soiling the fingers when handled; a variety of argillaceous slate.</cd> -- <col><b>By a long chalk</b></col>, <cd>by a long way; by many degrees.</cd> <mark>[Slang]</mark> <au>Lowell.</au> -- <col><b>Chalk drawing</b></col> <fld>(Fine Arts)</fld>, <cd>a drawing made with crayons. See <er>Crayon</er>.</cd> -- <col><b>Chalk formation</b></col>. <cd>See <cref>Cretaceous formation</cref>, under <er>Cretaceous</er>.</cd> -- <col><b>Chalk line</b></col>, <cd>a cord rubbed with chalk, used for making straight lines on boards or other material, as a guide in cutting or in arranging work.</cd> -- <col><b>Chalk mixture</b></col>, <cd>a preparation of chalk, cinnamon, and sugar in gum water, much used in diarrheal affection, esp. of infants.</cd> -- <col><b>Chalk period</b></col>. <fld>(Geol.)</fld> <cd>See <cref>Cretaceous period</cref>, under <er>Cretaceous</er>.</cd> -- <col><b>Chalk pit</b></col>, <cd>a pit in which chalk is dug.</cd> -- <col><b>Drawing chalk</b></col>. <cd>See <er>Crayon</er>, <pos>n.</pos>, 1.</cd> -- <col><b>French chalk</b></col>, <cd>steatite or soapstone, a soft magnesian mineral.</cd> -- <col><b>Red chalk</b></col>, <cd>an indurated clayey ocher containing iron, and used by painters and artificers; reddle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalk</ent><br/
<hw>Chalk</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chalked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chalking</conjf>.]</vmorph> <sn>1.</sn> <def>To rub or mark with chalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To manure with chalk, as land.</def>  <rj><au>Morimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make white, as with chalk; to make pale; to bleach.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let a bleak paleness <qex>chalk</qex> the door.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To chalk out</b></col>, <cd>to sketch with, or as with, chalk; to outline; to indicate; to plan.</cd> <mark>[Colloq.]</mark> <ldquo/I shall pursue the plan I have <xex>chalked out</xex>.<rdquo/  <rj><au>Burke.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>chalkboard</ent><br/
<hw>chalk"board`</hw> <pr>(ch<add/k"b<omac/rd`)</pr>, <pos>n.</pos> <def>a dark sheet of slate used as a surface for writing on, with chalk.</def><br/
<syn><b>Syn. --</b> blackboard.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chalkcutter</ent><br/
<hw>Chalk"cut`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A man who digs chalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalkiness</ent><br/
<hw>Chalk"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being chalky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chalk line</ent><br/
<hw>chalk" line`</hw> <pr>(ch<add/k" l<imac/n`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a string impregnated with powdered chalk, used in construction to mark a straight line on a surface, such as a wall.  The string is held taut on the surface between the two end points of the line to be made, the center of the line is drawn away from the surface, and the string is released and snapped against the surface, leaving a line of chalk in the desired location.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the line of chalk on a surface left by a <er>chalk line{1}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>chalkpit</ent><br/
<hw>chalk"pit</hw> <pr>(ch<add/k"p<icr/t)</pr>, <pos>n.</pos> <def>a quarry for mining chalk.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chalkstone</ent><br/
<hw>Chalk"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A mass of chalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>chalkstones</qex> . . . beaten in sunder.</q> <rj><qau>Isa. xxvii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A chalklike concretion, consisting mainly of urate of sodium, found in and about the small joints, in the external ear, and in other situations, in those affected with gout; a tophus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalky</ent><br/
<hw>Chalk"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of, or resembling, chalk; containing chalk; <as>as, a <ex>chalky</ex> cliff; a <ex>chalky</ex> taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Challenge</ent><br/
<hw>Chal"lenge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>chalenge</ets> claim, accusation, challenge, OF. <ets>chalenge</ets>, <ets>chalonge</ets>, claim, accusation, contest, fr. L. <ets>calumnia</ets> false accusation, chicanery. See <er>Calumny</er>.]</ety> <sn>1.</sn> <def>An invitation to engage in a contest or controversy of any kind; a defiance; specifically, a summons to fight a duel; also, the letter or message conveying the summons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>challenge</qex> to controversy.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of a sentry in halting any one who appears at his post, and demanding the countersign.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A claim or demand.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There must be no <qex>challenge</qex> of superiority.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Hunting)</fld> <def>The opening and crying of hounds at first finding the scent of their game.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>An exception to a juror or to a member of a court martial, coupled with a demand that he should be held incompetent to act; the claim of a party that a certain person or persons shall not sit in trial upon him or his cause.</def>  <rj><au>Blackstone</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An exception to a person as not legally qualified to vote. The challenge must be made when the ballot is offered.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Challenge to the array</b></col> <fld>(Law)</fld>, <cd>an exception to the whole panel.</cd> -- <col><b>Challenge to the favor</b></col>, <cd>the alleging a special cause, the sufficiency of which is to be left to those whose duty and office it is to decide upon it.</cd> -- <col><b>Challenge to the polls</b></col>, <cd>an exception taken to any one or more of the individual jurors returned.</cd> -- <col><b>Peremptory challenge</b></col>, <cd>a privilege sometimes allowed to defendants, of challenging a certain number of jurors (fixed by statute in different States) without assigning any cause.</cd> -- <col><b>Principal challenge</b></col>, <cd>that which the law allows to be sufficient if found to be true.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Challenge</ent><br/
<hw>Chal"lenge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Challenged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Challenging</conjf>.]</vmorph> <ety>[OE. <ets>chalengen</ets> to accuse, claim, OF. <ets>chalengier</ets>, <ets>chalongier</ets>, to claim, accuse, dispute, fr. L. <ets>calumniar</ets> to attack with false accusations. See <er>Challenge</er>, <pos>n.</pos>, and cf. <er>Calumniate</er>.]</ety> <sn>1.</sn> <def>To call to a contest of any kind; to call to answer; to defy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>challenge</qex> any man to make any pretense to power by right of fatherhood.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To call, invite, or summon to answer for an offense by personal combat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By this I <qex>challenge</qex> him to single fight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To claim as due; to demand as a right.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Challenge</qex> better terms.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To censure; to blame.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He complained of the emperors . . . and <qex>challenged</qex> them for that he had no greater revenues . . . from them.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mil.)</fld> <def>To question or demand the countersign from (one who attempts to pass the lines); <as>as, the sentinel <ex>challenged</ex> us, with  <ldquo/Who comes there?<rdquo/</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To take exception to; question; <as>as, to <ex>challenge</ex> the accuracy of a statement or of a quotation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>To object to or take exception to, as to a juror, or member of a court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To object to the reception of the vote of, as on the ground that the person in not qualified as a voter.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To challenge to the</b></col> <col><b>array, favor, polls</b></col></mcol>. <cd>See under <er>Challenge</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Challenge</ent><br/
<hw>Chal"lenge</hw>, <pos>v. i.</pos> <def>To assert a right; to claim a place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where nature doth with merit <qex>challenge</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Challengeable</ent><br/
<hw>Chal"lenge*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be challenged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>challenged</ent><br/
<hw>challenged</hw> <pos>adj.</pos> <sn>1.</sn> <def>having doubts expressed about its truth.</def><br/
<syn><b>Syn. --</b> disputed, questioned.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>handicapped or disabled; -- used as a euphemism, especially in combinations; <as>as, physically <ex>challenged</ex>; mentally <ex>challenged</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Challenger</ent><br/
<hw>Chal"len*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who challenges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>challenging</ent><br/
<hw>challenging</hw> <pos>adj.</pos> <sn>1.</sn>  <def>requiring full use of one's abilities or resources; <as>as, <ex>challenging</ex> task</as>.</def><br/
<syn><b>Syn. --</b> ambitious, demanding.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>disposed to or engaged in defiance of established authority.</def><br/
<syn><b>Syn. --</b> insubordinate, resistant, resistive.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Challis</ent><br/
<hw>Chal"lis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chaly</ets>, <ets>challis</ets>, a stuff made of goat's hair.]</ety> <def>A soft and delicate woolen, or woolen and silk, fabric, for ladies' dresses.</def> <altsp>[Written also <asp>chally</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalon</ent><br/
<hw>Cha"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bed blanket.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalybean</ent><br/
<hw>Cha*lyb"e*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>chalybe<ium/us</ets>, fr. <ets>chalybs</ets> steel, Gr. <?/.]</ety> <sn>1.</sn> <def>Of or pertaining to the Chalybes, an ancient people of Pontus in Asia Minor, celebrated for working in iron and steel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of superior quality and temper; -- applied to steel.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalybeate</ent><br/
<hw>Cha*lyb"e*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>chalybeatus</ets>, fr. <ets>chalube<ium/us</ets>. See <er>Chalubean</er>.]</ety> <def>Impregnated with salts of iron; having a taste like iron; <as>as, <ex>chalybeate</ex> springs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalybeate</ent><br/
<hw>Cha*lyb"e*ate</hw>, <pos>n.</pos> <def>Any water, liquid, or medicine, into which iron enters as an ingredient.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalybeous</ent><br/
<hw>Cha*lyb"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Steel blue; of the color of tempered steel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chalybite</ent><br/
<hw>Chal"y*bite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Native iron carbonate; -- usually called <xex>siderite</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cham</ent><br/
<hw>Cham</hw> <pr>(ch<acr/m)</pr>, <pos>v. t.</pos> <ety>[See <er>Chap</er>.]</ety> <def>To chew.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cham</ent><br/
<hw>Cham</hw> <pr>(k<acr/m)</pr>, <pos>n.</pos> <ety>[See <er>Khan</er>.]</ety> <def>The sovereign prince of Tartary; -- now usually written <asp>khan</asp>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamade</ent><br/
<hw>Cha*made</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chamade</ets>, fr. Pg. <ets>chamada</ets>, fr. <ets>chamar</ets> to call, fr. L. <ets>clamare</ets>.]</ety> <fld>(Mil.)</fld> <def>A signal made for a parley by beat of a drum.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They beat the <qex>chamade</qex>, and sent us carte blanche.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamaecytisus</ent><br/
<hw>Chamaecytisus</hw> <pos>n.</pos> <def>a genus of small late-flowering trees or subshrubs having yellow to red flowers and leathery or woody pods; -- often used especially for those formerly included in genus <gen>Cytisus</gen>.</def><br/
<syn><b>Syn. --</b> genus <gen>Chamaecytisus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chamaedaphne</ent><br/
<hw>Chamaedaphne</hw> <pos>n.</pos> <def>a genus consisting of only one species, the leatherleaf.</def><br/
<syn><b>Syn. --</b> genus <gen>Chamaedaphne</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chamaeleo</ent><br/
<hw>Chamaeleo</hw> <pos>n.</pos> <def>the type genus of the <fam>Chamaeleontidae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Chamaeleo</gen>, genus <gen>Chamaeleon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chamaeleon</ent><br/
<hw>chamaeleon</hw> <pos>n.</pos> <ety>[L. <ets>Chamaeleon</ets>, Gr. <grk>chamaile`wn</grk>, lit., <ldquo/ground lion;<rdquo/ <grk>chamai`</grk> on the ground + <grk>le`wn</grk> lion.]</ety> <fld>(Zool.)</fld> <def>a lizard of Africa and Madagascar able to change skin color and having a projectile tongue.</def><br/
<syn><b>Syn. --</b> chameleon.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chamaeleon</ent><br/
<hw>Chamaeleon</hw> <pos>n.</pos> <fld>(Astronomy)</fld> <def>a small constellation in the southern skies between Hydrus and Musca.</def><br/
<syn><b>Syn. --</b> Chameleon.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Chamaeleontidae</ent><br/
<ent>Chamaeleonidae</ent><br/
<mhw><hw>Chamaeleonidae</hw>, <hw>Chamaeleontidae</hw></mhw> <pos>n.</pos> <def>a natural family comprising the Old World chameleons; in some classifications Rhiptoglossa is considered a superfamily of Sauria.</def><br/
<syn><b>Syn. --</b> family <fam>Chamaeleontidae</fam>, family <fam>Chamaeleonidae</fam>, Rhiptoglossa, family <fam>Rhiptoglossa</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chamaemelum</ent><br/
<hw>Chamaemelum</hw> <pos>n.</pos> <def>a small genus of plants sometimes included in genus <gen>Anthemis</gen>; chamomile.</def><br/
<syn><b>Syn. --</b> genus <gen>Chamaemelum</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chamaephyte</ent><br/
<hw>cham"ae*phyte</hw> <pr>(k<acr/m"<esl/*f<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chamai`</grk> on the ground + <ets>-phyte</ets>.]</ety> <def>a perennial plant that sets its dormant vegetative buds just at or above the surface of the ground.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chamal</ent><br/
||<hw>Cha"mal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>The Angora goat. See <cref>Angora goat</cref>, under <er>Angora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamber</ent><br/
<hw>Cham"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chambre</ets>, fr. L. <ets>camera</ets> vault, arched roof, in LL. chamber, fr. Gr. <?/ anything with a vaulted roof or arched covering; cf. Skr. <ets>kmar</ets> to be crooked. Cf. <er>Camber</er>, <er>Camera</er>, <er>Comrade</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A retired room, esp. an upper room used for sleeping; a bedroom; <as>as, the house had four <ex>chambers</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 238 --></p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Apartments in a lodging house.</def> <ldquo/A bachelor's life in <xex>chambers</xex>.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A hall, as where a king gives audience, or a deliberative body or assembly meets; <as>as, presence <ex>chamber</ex>; senate <ex>chamber</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A legislative or judicial body; an assembly; a society or association; <as>as, the <ex>Chamber</ex> of Deputies; the <ex>Chamber</ex> of Commerce</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A compartment or cell; an inclosed space or cavity; <as>as, the <ex>chamber</ex> of a canal lock; the <ex>chamber</ex> of a furnace; the <ex>chamber</ex> of the eye</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <fld>(Law.)</fld> <def>A room or rooms where a lawyer transacts business; a room or rooms where a judge transacts such official business as may be done out of court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A chamber pot.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>That part of the bore of a piece of ordnance which holds the charge, esp. when of different diameter from the rest of the bore; -- formerly, in guns, made smaller than the bore, but now larger, esp. in breech-loading guns.</def> <sd>(b)</sd> <def>A cavity in a mine, usually of a cubical form, to contain the powder.</def> <sd>(c)</sd> <def>A short piece of ordnance or cannon, which stood on its breech, without any carriage, formerly used chiefly for rejoicings and theatrical cannonades.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Air chamber</b></col>. <cd>See <er>Air chamber</er>, in the Vocabulary.</cd> -- <col><b>Chamber of commerce</b></col>, <cd>a board or association to protect the interests of commerce, chosen from among the merchants and traders of a city.</cd> -- <col><b>Chamber council</b></col>, <cd>a secret council.</cd> <au>Shak.</au> -- <mcol><col><b>Chamber counsel</b></col> <it>or</it> <col><b>Chamber counselor</b></col></mcol>, <cd>a counselor who gives his opinion in private, or at his chambers, but does not advocate causes in court.</cd> -- <col><b>Chamber fellow</b></col>, <cd>a chamber companion; a roommate; a chum.</cd> -- <col><b>Chamber hangings</b></col>, <cd>tapestry or hangings for a chamber.</cd> -- <col><b>Chamber lye</b></col>, <cd>urine.</cd> <au>Shak.</au> -- <col><b>Chamber music</b></col>, <cd>vocal or instrumental music adapted to performance in a chamber or small apartment or audience room, instead of a theater, concert hall, or church.</cd> -- <col><b>Chamber practice</b></col> <fld>(Law.)</fld>, <cd>the practice of counselors at law, who give their opinions in private, but do not appear in court.</cd> -- <col><b>To sit at chambers</b></col>, <cd>to do business in chambers, as a judge.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamber</ent><br/
<hw>Cham"ber</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chambered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chambering</conjf>.]</vmorph> <sn>1.</sn> <def>To reside in or occupy a chamber or chambers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be lascivious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamber</ent><br/
<hw>Cham"ber</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To shut up, as in a chamber.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish with a chamber; <as>as, to <ex>chamber</ex> a gun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chambered</ent><br/
<hw>Cham"bered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a chamber or chambers; <as>as, a <ex>chambered</ex> shell; a <ex>chambered</ex> gun</as>.</def><-- the chambered nautilis --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamberer</ent><br/
<hw>Cham"ber*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who attends in a chamber; a chambermaid.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A civilian; a carpetmonger.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chambering</ent><br/
<hw>Cham"ber*ing</hw>, <pos>n.</pos> <def>Lewdness.</def> <mark>[Obs.]</mark>  <rj><au>Rom. xiii. 13.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamberlain</ent><br/
<hw>Cham"ber*lain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chamberlain</ets>, <ets>chambrelencF</ets>. <ets>chambellon</ets>, OHG. <ets>chamerling</ets>, <ets>chamarlinc</ets>, G. <ets>k<aum/mmerling</ets>, <ets>kammer</ets> chamber (fr. L. <ets>camera</ets>) + <ets>-ling</ets>. See <er>Chamber</er>, and <er>-ling</er>.]</ety> <altsp>[Formerly written <asp>chamberlin</asp>.]</altsp> <sn>1.</sn> <def>An officer or servant who has charge of a chamber or chambers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An upper servant of an inn.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An officer having the direction and management of the private chambers of a nobleman or monarch; hence, in Europe, one of the high officers of a court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A treasurer or receiver of public money; <as>as, the <ex>chamberlain</ex> of London, of North Wales, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The lord chamberlain of England</b></col>, <cd>an officer of the crown, who waits upon the sovereign on the day of coronation, and provides requisites for the palace of Westminster, and for the House of Lords during the session of Parliament. Under him are the gentleman of the black rod and other officers. His office is distinct from that of the <xex>lord chamberlain of the Household</xex>, whose functions relate to the royal housekeeping.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamberlainship</ent><br/
<hw>Cham"ber*lain*ship</hw>, <pos>n.</pos> <def>Office of a chamberlain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chambermaid</ent><br/
<hw>Cham"ber*maid`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A maidservant who has the care of chambers, making the beds, sweeping, cleaning the rooms, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lady's maid.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chambertin</ent><br/
||<hw>Cham`ber*tin"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A red wine from Chambertin near Dijon, in Burgundy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chambranle</ent><br/
||<hw>Cham`bran"le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Arch.)</fld> <def>An ornamental bordering or framelike decoration around the sides and top of a door, window, or fireplace. The top piece is called the <b>traverse</b> and the side pieces the <b>ascendants</b>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chambray</ent><br/
<hw>Cham"bray</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Cambrai</ets>, France. Cf. <er>Cambric</er>.]</ety> <def>A gingham woven in plain colors with linen finish.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chambrel</ent><br/
<hw>Cham"brel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Gambrel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chameck</ent><br/
||<hw>Cha*meck"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native Brazilian name.]</ety> <fld>(Zool.)</fld> <def>A kind of spider monkey (<spn>Ateles chameck</spn>), having the thumbs rudimentary and without a nail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chameleon</ent><br/
<hw>Cha*me"le*on</hw> <pr>(k<adot/*m<emac/"l<esl/*<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>Chamaeleon</ets>, Gr. <grk>chamaile`wn</grk>, lit., <ldquo/ground lion;<rdquo/ <grk>chamai`</grk> on the ground + <grk>le`wn</grk> lion. See <er>Humble</er>, and <er>Lion</er>.]</ety> <fld>(Zool.)</fld> <sn>1.</sn> <def>A lizardlike reptile of the genus <gen>Cham<ae/leo</gen>, of several species, found in Africa, Asia, and Europe. The skin is covered with fine granulations; it has eyes which can move separately, the tail is prehensile, and the body is much compressed laterally, giving it a high back.  It is remarkable for its ability to change the color of its skin to blend with its surroundings.</def> <altsp>[Also sometimes spelled <asp>chamaeleon</asp>.]</altsp><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ Its color changes more or less with the color of the objects about it, or with its temper when disturbed. In a cool, dark place it is nearly white, or grayish; on admitting the light, it changes to brown, bottle-green, or blood red, of various shades, and more or less mottled in arrangment. The American chameleons belong to <gen>Anolis</gen> and allied genera of the family <fam>Iguanid<ae/</fam>. They are more slender in form than the true chameleons, but have the same power of changing their colors.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a person who changes opinions, ideas, or behavior to suit the prevailing social climate; an opportunist.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Chameleon mineral</b></col> <fld>(Chem.)</fld>, <cd>the compound called <altname>potassium permanganate</altname>, a dark violet, crystalline substance, <chform>KMnO4</chform>, which in formation passes through a peculiar succession of color from green to blue, purple, red, etc. See <cref>Potassium permanganate</cref>, under <er>Potassium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chameleonize</ent><br/
<hw>Cha*me"le*on*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To change into various colors.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamfer</ent><br/
<hw>Cham"fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chamfron</er>.]</ety> <def>The surface formed by cutting away the arris, or angle, formed by two faces of a piece of timber, stone, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamfer</ent><br/
<hw>Cham"fer</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chamfered</conjf> ; <pos>p. pr. & vb. n.</pos> <conjf>Chamfering</conjf>. <pr>(<?/)</pr>]</vmorph><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Carp.)</fld> <def>To cut a furrow in, as in a column; to groove; to channel; to flute.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a chamfer on.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamfret</ent><br/
<hw>Cham"fret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chamfron</er>.]</ety> <sn>1.</sn> <fld>(Carp.)</fld> <def>A small gutter; a furrow; a groove.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A chamfer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamfron</ent><br/
<hw>Cham"fron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chanfrein</ets>.]</ety> <fld>(Anc. Armor)</fld> <def>The frontlet, or head armor, of a horse.</def> <altsp>[Written also <asp>champfrain</asp> and <asp>chamfrain</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamisal</ent><br/
||<hw>Cha`mi*sal"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Amer. Sp., fr. Sp. <ets>chamiza</ets> a kind of wild cane.]</ety> <sn>1.</sn>  <fld>(Bot.)</fld> <def>A California rosaceous shrub (<spn>Adenostoma fasciculatum</spn>) which often forms an impenetrable chaparral.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A chaparral formed by dense growths of this shrub.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chamlet</ent><br/
<hw>Cham"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Camlet</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chamois</ent><br/
<hw>Cham"ois</hw> <pr>(sh<acr/m"m<ycr/ <it>or</it> sh<adot/*moi"; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>chamois</ets>, prob. fr. OG. <ets>gamz</ets>, G. <ets>gemse</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A small species of antelope (<spn>Rupicapra tragus</spn>), living on the loftiest mountain ridges of Europe, as the Alps, Pyrenees, etc. It possesses remarkable agility, and is a favorite object of chase.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A soft leather made from the skin of the chamois, or from sheepskin, etc.; -- called also <altname>chamois leather</altname>, and <altname>chammy</altname> or <altname>shammy leather</altname>. See <er>Shammy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chamosite</ent><br/
<hw>chamosite</hw> <pos>n.</pos> <def>a greenish gray or black silicate of iron and aluminum.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chamomile</ent><br/
<hw>Cham"o*mile</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Camomile</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champ</ent><br/
<hw>Champ</hw> <pr>(ch<acr/mp)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Champed</conjf> <pr>(ch<acr/mt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Champing</conjf>.]</vmorph> <ety>[Prob, of Scand. orgin; cf. dial. Sw. <ets>k<aum/msa</ets> to chew with difficulty, champ; but cf. also OF. <ets>champier</ets>, <ets>champeyer</ets>, <ets>champoyer</ets>, to graze in fields, fr. F. <ets>champ</ets> field, fr. L. <ets>campus</ets>. Cf. <er>Camp</er>.]</ety> <sn>1.</sn> <def>To bite with repeated action of the teeth so as to be heard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Foamed and <qex>champed</qex> the golden bit.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bite into small pieces; to crunch.</def>  <rj><au>Steele.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champ</ent><br/
<hw>Champ</hw>, <pos>v. i.</pos> <def>To bite or chew impatiently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They began . . . irefully to <qex>champ</qex> upon the bit.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champe</ent><br/
<ent>Champ</ent><br/
<mhw>{ <hw>Champ</hw>, <hw>Champe</hw>, }</mhw> <pos>n.</pos> <ety>[F. <ets>champ</ets>, L. <ets>campus</ets> field.]</ety> <fld>(Arch.)</fld> <def>The field or ground on which carving appears in relief.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champagne</ent><br/
<hw>Cham*pagne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Champaign</er>.]</ety> <def>A light wine, of several kinds, originally made in the province of Champagne, in France.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Champagne</xex> properly includes several kinds not only of sparkling but of still wines; but in America the term is usually restricted to wines which effervesce.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champaign</ent><br/
<hw>Cham*paign"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>champaigne</ets>; same word as <ets>campagne</ets>.]</ety> <def>A flat, open country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fair <qex>champaign</qex>, with less rivers interveined.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Through Apline vale or <qex>champaign</qex> wide.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champaign</ent><br/
<hw>Cham*paign"</hw>, <pos>a.</pos> <def>Flat; open; level.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A wide, <qex>champaign</qex> country, filled with herds.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champer</ent><br/
<hw>Champ"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who champs, or bites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champertor</ent><br/
<hw>Cham"per*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>champarteur</ets> a divider of fields or field rent. See <er>Champerty</er>.]</ety> <fld>(Law)</fld> <def>One guilty of champerty; one who purchases a suit, or the right of suing, and carries it on at his own expense, in order to obtain a share of the gain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champerty</ent><br/
<hw>Cham"per*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>champart</ets> field rent, L. <ets>campipars</ets>; <ets>champ</ets> (L. <ets>campus</ets>) field + <ets>part</ets> (L. <ets>pars</ets>) share.]</ety> <sn>1.</sn> <def>Partnership in power; equal share of authority.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Beaut<eacute/ ne sleighte, strengthe ne hardyness,<br/
Ne may with Venus holde <qex>champartye</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The prosecution or defense of a suit, whether by furnishing money or personal services, by one who has no legitimate concern therein, in consideration of an agreement that he shall receive, in the event of success, a share of the matter in suit; maintenance with the addition of an agreement to divide the thing in suit. See <er>Maintenance</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ By many authorities champerty is defined as an <xex>agreement</xex> of this nature. From early times the offence of champerty has been forbidden and punishable.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champignon</ent><br/
<hw>Cham*pi"gnon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a mushroom, ultimately fr. L. <ets>campus</ets> field. See <er>Camp</er>.]</ety> <fld>(Bot.)</fld> <def>An edible species of mushroom (<spn>Agaricus campestris</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Fairy ring champignon</b></col>, <cd>the <spn>Marasmius oreades</spn>, which has a strong flavor but is edible.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champion</ent><br/
<hw>Cham"pi*on</hw> <pr>(ch<acr/m"p<icr/*<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>champion</ets>, fr. LL.<ets>campio</ets>, of German origin; cf. OHG. <ets>chempho</ets>, <ets>chemphio</ets>, fighter, <ets>champf</ets>, G. <ets>kampf</ets>, contest; perh. influenced by L. <ets>campus</ets> field, taken in the sense of <ldquo/field of battle.<rdquo/]</ety> <sn>1.</sn> <def>One who engages in any contest; especially one who in ancient times contended in single combat in behalf of another's honor or rights; or one who now acts or speaks in behalf of a person or a cause; a defender; an advocate; a hero.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A stouter <qex>champion</qex> never handled sword.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Champions</qex> of law and liberty.</q> <rj><qau>Fisher Ames.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who by defeating all rivals, has obtained an acknowledged supremacy in any branch of athletics or game of skill, and is ready to contend with any rival; <as>as, the <ex>champion</ex> of England</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Champion</xex> is used attributively in the sense of <xex>surpassing all competitors</xex>; <xex>overmastering</xex>; as, <xex>champion</xex> pugilist; <xex>champion</xex> chess player.</note></p>

<p><syn><b>Syn.</b> -- Leader; chieftain; combatant; hero; warrior; defender; protector.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champion</ent><br/
<hw>Cham"pi*on</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Championed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Championing</conjf>.]</vmorph> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish with a champion; to attend or defend as champion; to support or maintain; to protect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Championed</qex> or unchampioned, thou diest.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Championness</ent><br/
<hw>Cham"pi*on*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female champion.</def>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Championship</ent><br/
<hw>Cham"pi*on*ship</hw>, <pos>n.</pos> <def>State of being champion; leadership; supremacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champlain period</ent><br/
<hw>Cham*plain" pe"ri*od</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>A subdivision of the Quaternary age immediately following the Glacial period; -- so named from beds near Lake Champlain.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The earlier deposits of this period are diluvial in character, as if formed in connection with floods attending the melting of the glaciers, while the later deposits are of finer material in more quiet waters, as the alluvium.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Champleve</ent><br/
||<hw>Champ`le*v<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., p. p. of <ets>champlever</ets> to engrave. See 3d <er>Champ</er>, <er>Camp</er>, <er>Lever</er> a bar.]</ety> <fld>(Art)</fld> <def>Having the ground engraved or cut out in the parts to be enameled; inlaid in depressions made in the ground; -- said of a kind of enamel work in which depressions made in the surface are filled with enamel pastes, which are afterward fired; also, designating the process  of making such enamel work.</def> -- <def2><pos>n.</pos>  <def>A piece of champlev<eacute/ enamel; also, the process or art of making such enamel work; champlev<eacute/ work.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chamsin</ent><br/
||<hw>Cham*sin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>See <er>Kamsin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chanar</ent><br/
<hw>chanar</hw> <pos>n.</pos> <def>a thorny shrub or small tree (<spn>Geoffroea decorticans</spn>) common in central Argentina having small orange or yellow flowers followed by edible berries.</def><br/
<syn><b>Syn. --</b> chanal, <spn>Geoffroea decorticans</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chance</ent><br/
<hw>Chance</hw> <pr>(ch<adot/ns)</pr>, <pos>n.</pos> <ety>[F. <ets>chance</ets>, OF. <ets>cheance</ets>, fr. LL. <ets>cadentia</ets> a allusion to the falling of the dice), fr. L. <ets>cadere</ets> to fall; akin to Skr. <ets><cced/ad</ets> to fall, L. <ets>cedere</ets> to yield, E. <ets>cede</ets>. Cf. <er>Cadence</er>.]</ety> <sn>1.</sn> <def>A supposed material or psychical agent or mode of activity other than a force, law, or purpose; fortune; fate; -- in this sense often personified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is strictly and philosophically true in nature and reason that there is no such thing as <qex>chance</qex> or accident; it being evident that these words do not signify anything really existing, anything that is truly an agent or the cause of any event; but they signify merely men's ignorance of the real and immediate cause.</q> <rj><qau>Samuel Clark.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>Many of the everyday events which people observe and attribute to chance fall into the category described by Clark, as being in practice too complex for people to easily predict, but in theory predictable if one were to know the actions of the causal agents in great detail.  At the subatomic level, however, there is much evidence to support the notion derived from <persfn>Heisenberg's</persfn> uncertaintly principle, that phenomena occur in nature which are truly randomly determined, not merely too complex to predict or observe accurately.  Such phenomena, however, are observed only with one or a very small number of subatomic particles.  When the probabilities of observed events are determined by the behavior of aggregates of millions of particles, the variations due to such quantum indeterminacy becomes so small as to be unobservable even over billions of repetitions, and may therefore be ignored in practical situations; such variations are so improbable that it would be irrational to condition anything of consequence upon the occurrence of such an improbable event.  A clever experimenter, nevertheless, may contrive a system where a very visible event (such as the dynamiting of a building) depends on the occurrence of a truly chance subatomic event (such as the disintegration of a single radioactive nucleus).  In such a contrived situation, one may accurately speak of an event determined by <ex>chance</ex>, in the sense of a random occurrence completely unpredictable, at least as to time.</note><br/
[<source>PJC</source>]</p>

<p><q>Any society into which <qex>chance</qex> might throw him.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That power<br/
Which erring men call <qex>Chance</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The operation or activity of such agent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By chance a priest came down that way.</q> <rj><qau>Luke x. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The supposed effect of such an agent; something that befalls, as the result of unknown or unconsidered forces; the issue of uncertain conditions; an event not calculated upon; an unexpected occurrence; a happening; accident; fortuity; casualty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the field of observation, <qex>chance</qex> favors only the mind that is prepared.</q> <rj><qau>Louis Pasteur.</qau></rj><br/
[<source>PJC</source>]</p>

<p><note>This quotation is usually found in the form "Chance favors the prepared mind."  It is a common rejoinder to the assertion that a scientist was "lucky" to have made some particular discovery because of unanticipated factors.  A related quotation, from the Nobel-Prize-winning chemist <person>R. B. Woodward</person>, is that "A scientist has to work wery hard to get to the point where he can be lucky."</note><br/
[<source>PJC</source>]</p>

<p><q>It was a <qex>chance</qex> that happened to us.</q> <rj><qau>1 Sam. vi. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Knave of Diamonds tries his wily arts,<br/
And wins (O shameful <qex>chance</qex>!) the Queen of Hearts.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I spake of most disastrous <qex>chance</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A possibility; a likelihood; an opportunity; -- with reference to a doubtful result; <as>as, a <ex>chance</ex> to escape; a <ex>chance</ex> for life; the <ex>chances</ex> are all against him</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So weary with disasters, tugged with fortune.<br/
That I would get my life on any <qex>chance</qex>,<br/
To mend it, or be rid on 't</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Math.)</fld> <def>Probability.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The mathematical expression, of a <xex>chance</xex> is the ratio of frequency with which an event happens in the long run. If an event may happen in <xex>a</xex> ways and may fail in <xex>b</xex> ways, and each of these <xex>a</xex> + <xex>b</xex> ways is equally likely, the <xex>chance</xex>, or probability, that the event will happen is measured by the fraction <fract>a/<mathex>a + b</mathex></fract>, and the <xex>chance</xex>, or probability, that it will fail is measured by <fract>b/<mathex>a + b</mathex></fract>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chance comer</b></col>, <cd>one who comes unexpectedly.</cd> -- <col><b>The last chance</b></col>, <cd>the sole remaining ground of hope.</cd> -- <col><b>The main chance</b></col>, <cd>the chief opportunity; that upon which reliance is had, esp. self-interest.</cd> -- <mcol><col><b>Theory of chances</b></col>, <col><b>Doctrine of chances</b></col></mcol> <fld>(Math.)</fld>, <cd>that branch of mathematics which treats of the probability of the occurrence of particular events, as the fall of dice in given positions.</cd> -- <col><b>To mind one's chances</b></col>, <cd>to take advantage of every circumstance; to seize every opportunity.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chance</ent><br/
<hw>Chance</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chanced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chancing</conjf>.]</vmorph> <def>To happen, come, or arrive, without design or expectation.</def> <ldquo/Things that <xex>chance</xex> daily.<rdquo/  <rj><au>Robynson (More's Utopia).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If a bird's nest <qex>chance</qex> to be before thee.</q> <rj><qau>Deut. xxii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>chanced</qex> on this letter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>Often used impersonally; as, how <xex>chances</xex> it?</note><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>chance</qex>, thou art returned so soon?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chance</ent><br/
<hw>Chance</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To take the chances of; to venture upon; -- usually with <xex>it</xex> as object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come what will, I will <qex>chance</qex> it.</q> <rj><qau>W. D. Howells.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To befall; to happen to.</def> <mark>[Obs.]</mark>  <rj><au>W. Lambarde.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chance</ent><br/
<hw>Chance</hw>, <pos>a.</pos> <def>Happening by chance; casual.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chance</ent><br/
<hw>Chance</hw>, <pos>adv.</pos> <def>By chance; perchance.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanceable</ent><br/
<hw>Chance"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fortuitous; casual.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanceably</ent><br/
<hw>Chance"a*bly</hw>, <pos>adv.</pos> <def>By chance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanceful</ent><br/
<hw>Chance"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Hazardous.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chancel</ent><br/
<hw>Chan"cel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chancel</ets>, F. <ets>chanceau</ets>, <ets>cancel</ets>, fr. L. <ets>cancelli</ets> lattices, crossbars. (The <ets>chancel</ets> was formerly inclosed with lattices or crossbars) See <er>Cancel</er>, <pos>v. t.</pos>]</ety> <fld>(Arch.)</fld> <sd>(a)</sd> <def>That part of a church, reserved for the use of the clergy, where the altar, or communion table, is placed.</def> Hence, in modern use; <sd>(b)</sd> <def>All that part of a cruciform church which is beyond the line of the transept farthest from the main front.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chancel aisle</b></col> <fld>(Arch.)</fld>, <cd>the aisle which passes on either side of or around the chancel.</cd> -- <col><b>Chancel arch</b></col> <fld>(Arch.)</fld>, <cd>the arch which spans the main opening, leading to the chancel.</cd> -- <col><b>Chancel casement</b></col>, <cd>the principal window in a chancel.</cd> <au>Tennyson</au>. -- <col><b>Chancel table</b></col>, <cd>the communion table.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chancellery</ent><br/
<hw>Chan"cel*ler*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chancery</er>.]</ety> <def>Chancellorship.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chancellor</ent><br/
<hw>Chan"cel*lor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>canceler</ets>, <ets>chaunceler</ets>, F. <ets>chancelier</ets>, LL. <ets>cancellarius</ets> chancellor, a director of chancery, fr. L. <ets>cancelli</ets> lattices, crossbars, which surrounded the seat of judgment. See <er>Chancel</er>.]</ety> <def>A judicial court of chancery, which in England and in the United States is distinctively a court with equity jurisdiction.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>chancellor</xex> was originally a chief scribe or secretary under the Roman emperors, but afterward was invested with judicial powers, and had superintendence over the other officers of the empire. From the Roman empire this office passed to the church, and every bishop has his chancellor, the principal judge of his consistory. In later times, in most countries of Europe, the chancellor was a high officer of state, keeper of the great seal of the kingdom, and having the supervision of all charters, and like public instruments of the crown, which were authenticated in the most solemn manner. In France a secretary is in some cases called a <xex>chancellor</xex>. In Scotland, the appellation is given to the foreman of a jury, or assize. In the present German empire, the <xex>chancellor</xex> is the president of the federal council and the head of the imperial administration. In the United States, the title is given to certain judges of courts of chancery or equity, established by the statutes of separate States.</note>  <rj><au>Blackstone. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Chancellor of a bishop</b></col> <it>or</it> <col><b>Chancellor of a diocese</b></col></mcol> <fld>(R. C. Ch. & ch. of Eng.)</fld>, <cd>a law officer appointed to hold the bishop's court in his diocese, and to assist him in matter of ecclesiastical law.</cd> -- <col><b>Chancellor of a cathedral</b></col>, <cd>one of the four chief dignitaries of the cathedrals of the old foundation, and an officer whose duties are chiefly educational, with special reference to the cultivation of theology.</cd> -- <col><b>Chancellor of the Duchy of Lancaster</b></col>, <cd>an officer before whom, or his deputy, the court of the duchy chamber of Lancaster is held. This is a special jurisdiction.</cd> -- <col><b>Chancellor of a university</b></col>, <cd>the chief officer of a collegiate body. In Oxford, he is elected for life; in Cambridge, for a term of years; and his office is honorary, the chief duties of it devolving on the vice chancellor.</cd> -- <col><b>Chancellor of the exchequer</b></col>, <cd>a member of the British cabinet upon whom devolves the charge of the public income and expenditure as the highest finance minister of the government.</cd> -- <col><b>Chancellor of the order of the Garter</b></col> (or other military orders), <cd>an officer who seals the commissions and mandates of the chapter and assembly of the knights, keeps the register of their proceedings, and delivers their acts under the seal of their order.</cd> -- <col><b>Lord high chancellor of England</b></col>, <cd>the presiding judge in the court of chancery, the highest judicial officer of the crown, and the first lay person of the state after the blood royal. He is created chancellor by the delivery into his custody of the great seal, of which he becomes keeper. He is privy counselor by his office, and prolocutor of the House of Lords by prescription.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 239 --></p>

<p><ent>Chancellorship</ent><br/
<hw>Chan"cel*lor*ship</hw> <pr>(ch<adot/n"s<ecr/l*l<etil/r*sh<icr/p)</pr>, <pos>n.</pos> <def>The office of a chancellor; the time during which one is chancellor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chance-medley</ent><br/
<hw>Chance"-med`ley</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chance</ets> + <ets>medley</ets>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>The killing of another in self-defense upon a sudden and unpremeditated encounter. See <er>Chaud-Medley</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term has been sometimes applied to any kind of homicide by misadventure, or to any accidental killing of a person without premeditation or evil intent, but, in strictness, is applicable to such killing as happens in defending one's self against assault.</note>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Luck; chance; accident.</def>  <rj><au>Milton.</au> <au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chancery</ent><br/
<hw>Chan"cer*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chancellerie</ets>, LL. <ets>cancellaria</ets>, from L. <ets>cancellarius</ets>. See <er>Chancellor</er>, and cf. <er>Chancellery</er>.]</ety> <sn>1.</sn> <def>In England, formerly, the highest court of judicature next to the Parliament, exercising jurisdiction at law, but chiefly in equity; but under the jurisdiction act of 1873 it became the chancery division of the High Court of Justice, and now exercises jurisdiction only in equity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the Unites States, a court of equity; equity; proceeding in equity.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A court of chancery, so far as it is a court of equity, in the English and American sense, may be generally, if not precisely, described as one having jurisdiction in cases of rights, recognized and protected by the municipal jurisprudence, where a plain, adequate, and complete remedy can not be had in the courts of common law. In some of the American States, jurisdiction at law and in equity centers in the same tribunal. The courts of the United States also have jurisdiction both at law and in equity, and in all such cases they exercise their jurisdiction, as courts of law, or as courts of equity, as the subject of adjudication may require. In others of the American States, the courts that administer equity are distinct tribunals, having their appropriate judicial officers, and it is to the latter that the appellation <xex>courts of chancery</xex> is usually applied; but, in American law, the terms <xex>equity</xex> and <xex>court of equity</xex> are more frequently employed than the corresponding terms <xex>chancery</xex> and <xex>court of chancery</xex>.</note>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Inns of chancery</b></col>. <cd>See under <er>Inn</er>.</cd> -- <col><b>To get (<it>or</it> to hold) In chancery</b></col> <fld>(Boxing)</fld>, <cd>to get the head of an antagonist under one's arm, so that one can pommel it with the other fist at will; hence, to have wholly in One's power. The allusion is to the condition of a person involved in the chancery court, where he was helpless, while the lawyers lived upon his estate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chancre</ent><br/
<hw>Chan"cre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chancere</ets>. See <er>Cancer</er>.]</ety> <fld>(Med.)</fld> <def>A venereal sore or ulcer; specifically, the initial lesion of true syphilis, whether forming a distinct ulcer or not; -- called also <altname>hard chancre</altname>, <altname>indurated chancre</altname>, and <altname>Hunterian chancre</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soft chancre</b></col>. <cd>A chancroid. See <er>Chancroid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chancroid</ent><br/
<hw>Chan"croid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chancre</ets> + <ets>-oil</ets>.]</ety> <fld>(Med.)</fld> <def>A venereal sore, resembling a chancre in its seat and some external characters, but differing from it in being the starting point of a purely local process and never of a systemic disease; -- called also <altname>soft chancre</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chancroidal</ent><br/
<hw>chancroidal</hw> <pos>adj.</pos> <def>of or pertaining to a chancroid.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chancrous</ent><br/
<hw>Chan"crous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>chancreux</ets>.]</ety> <fld>(Med.)</fld> <def>Of the nature of a chancre; having chancre.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chancy</ent><br/
<hw>chancy</hw> <pos>adj.</pos> <sn>1.</sn> <def>depending on chance.</def><br/
<syn><b>Syn. --</b> chanceful, dicey, dodgy, hazardous, risky.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>subject to accident or chance or change.</def><br/
<syn><b>Syn. --</b> fluky, flukey, iffy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chandelier</ent><br/
<hw>Chan`de*lier"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Chandler</er>.]</ety> <sn>1.</sn> <def>A candlestick, lamp, stand, gas fixture, or the like, having several branches; esp., one hanging from the ceiling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fort.)</fld> <def>A movable parapet, serving to support fascines to cover pioneers.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chandelle</ent><br/
<hw>chandelle</hw> <pos>n.</pos> <fld>(Aeronautics)</fld> <def>an abrupt climbing turn made by an airplane, in which the plane's momentum is used to achieve a higher than normal rate of climb, without stalling the aircraft.</def><br/
[<source>PJC</source>]</p>

<p><ent>chandelle</ent><br/
<hw>chandelle</hw> <pos>v. i.</pos> <def>to perform a chandelle, as of an airplane.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chandi</ent><br/
<hw>Chandi</hw> <pos>n.</pos> <fld>(Hinduism)</fld> <def>the malevolent aspect of Devi: "the fierce".</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chandler</ent><br/
<hw>Chan"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chandelier</ets> a candlestick, a maker or seller of candles, LL. <ets>candelarius</ets> chandler, fr. L. <ets>candela</ets> candle. See <er>Candle</er>, and cf. <ets>Chandelier</ets>.]</ety> <sn>1.</sn> <def>A maker or seller of candles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chandler's</qex> basket, on his shoulder borne,<br/
With tallow spots thy coat.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dealer in other commodities, which are indicated by a word prefixed; <as>as, ship <ex>chandler</ex>, corn <ex>chandler</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chandlerly</ent><br/
<hw>Chan"dler*ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a chandler; in a petty way.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chandlery</ent><br/
<hw>Chan"dler*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Commodities sold by a chandler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chandoo</ent><br/
||<hw>Chan*doo"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An extract or preparation of opium, used in China and India for smoking.</def>  <rj><au>Balfour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chandry</ent><br/
<hw>Chan"dry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Chandlery.</def> <mark>[Obs.]</mark> <ldquo/Torches from the <xex>chandry</xex>.<rdquo/  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanfrin</ent><br/
<hw>Chan"frin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chanfrein</ets>. Cf. <er>Chamfron</er>.]</ety> <def>The fore part of a horse's head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changan</ent><br/
<hw>Changan</hw> <pos>n.</pos> <def>the capital of the ancient Chinese empire.</def><br/
<syn><b>Syn. --</b> Sian, Singan, Xian.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Change</ent><br/
<hw>Change</hw> <pr>(ch<amac/nj)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Changed</conjf> <pr>(ch<amac/njd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Changing</conjf>.]</vmorph> <ety>[F. <ets>changer</ets>, fr. LL. <ets>cambiare</ets>, to exchange, barter, L. <ets>cambire</ets>. Cf. <er>Cambial</er>.]</ety> <sn>1.</sn> <def>To alter; to make different; to cause to pass from one state to another; <as>as, to <ex>change</ex> the position, character, or appearance of a thing; to <ex>change</ex> the countenance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore will I <qex>change</qex> their glory into shame.</q> <rj><qau>Hosea. iv. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To alter by substituting something else for, or by giving up for something else; <as>as, to <ex>change</ex> the clothes; to <ex>change</ex> one's occupation; to <ex>change</ex> one's intention</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They that do <qex>change</qex> old love for new,<br/
Pray gods, they change for worse!</q> <rj><qau>Peele.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give and take reciprocally; to exchange; -- followed by <xex>with</xex>; <as>as, to <ex>change</ex> place, or hats, or money, with another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Look upon those thousands with whom thou wouldst not, for any interest, <qex>change</qex> thy fortune and condition.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Specifically: To give, or receive, smaller denominations of money (technically called <xex>change</xex>) for; <as>as, to <ex>change</ex> a gold coin or a bank bill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He pulled out a thirty-pound note and bid me <qex>change</qex> it.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To change a horse, <it>or</it> To change hand</b></col> <fld>(Man.)</fld>, <cd>to turn or bear the horse's head from one hand to the other, from the left to right, or from the right to the left.</cd> -- <col><b>To change hands</b></col>, <cd>to change owners.</cd> -- <col><b>To change one's tune</b></col>, <cd>to become less confident or boastful.</cd> <mark>[Colloq.]</mark> -- <col><b>To change step</b></col>, <cd>to take a break in the regular succession of steps, in marching or walking, as by bringing the hollow of one foot against the heel of the other, and then stepping off with the foot which is in advance.</cd></cs></p>

<p><syn><b>Syn.</b> -- To alter; vary; deviate; substitute; innovate; diversify; shift; veer; turn. See <er>Alter</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Change</ent><br/
<hw>Change</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be altered; to undergo variation; <as>as, men sometimes <ex>change</ex> for the better</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For I am Lord, I <qex>change</qex> not.</q> <rj><qau>Mal. iii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass from one phase to another; <as>as, the moon <ex>changes</ex> to-morrow night</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Change</ent><br/
<hw>Change</hw>, <pos>n.</pos> <ety>[F. <ets>change</ets>, fr. <ets>changer</ets>. See <er>Change</er>. <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Any variation or alteration; a passing from one state or form to another; <as>as, a <ex>change</ex> of countenance; a <ex>change</ex> of habits or principles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Apprehensions of a <qex>change</qex> of dynasty.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the days of my appointed time will I wait, till my <qex>change</qex> come.</q> <rj><qau>Job xiv. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A succesion or substitution of one thing in the place of another; a difference; novelty; variety; <as>as, a <ex>change</ex> of seasons</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our fathers did for <qex>change</qex> to France repair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ringing grooves of <qex>change</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A passing from one phase to another; <as>as, a <ex>change</ex> of the moon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Alteration in the order of a series; permutation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which makes a variety, or may be substituted for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thirty <qex>change</qex> (R.V. <qex>changes</qex>) of garments.</q> <rj><qau>Judg. xiv. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Small money; the money by means of which the larger coins and bank bills are made available in small dealings; hence, the balance returned when payment is tendered by a coin or note exceeding the sum due.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <ety>[See <er>Exchange</er>.]</ety> <def>A place where merchants and others meet to transact business; a building appropriated for mercantile transactions.</def> <mark>[Colloq. for Exchange.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A public house; an alehouse.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They call an alehouse a <qex>change</qex>.</q> <rj><qau>Burt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mus.)</fld> <def>Any order in which a number of bells are struck, other than that of the diatonic scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Four bells admit twenty-four <qex>changes</qex> in ringing.</q> <rj><qau>Holder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Change of life</b></col>, <cd>the period in the life of a woman when menstruation and the capacity for conception cease, usually occurring between forty-five and fifty years of age.</cd> -- <col><b>Change ringing</b></col>, <cd>the continual production, without repetition, of changes on bells, See def. 9. above.</cd> -- <col><b>Change wheel</b></col> <fld>(Mech.)</fld>, <cd>one of a set of wheels of different sizes and number of teeth, that may be changed or substituted one for another in machinery, to produce a different but definite rate of angular velocity in an axis, as in cutting screws, gear, etc.</cd> -- <col><b>To ring the changes on</b></col>, <cd>to present the same facts or arguments in variety of ways.</cd></cs></p>

<p><syn><b>Syn.</b> -- Variety; variation; alteration; mutation; transition; vicissitude; innovation; novelty; transmutation; revolution; reverse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeability</ent><br/
<hw>Change`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Changeableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeable</ent><br/
<hw>Change"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>changeable</ets>.]</ety> <sn>1.</sn> <def>Capable of change; subject to alteration; mutable; variable; fickle; inconstant; <as>as, a <ex>changeable</ex> humor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Appearing different, as in color, in different lights, or under different circumstances; <as>as, <ex>changeable</ex> silk</as>.</def></p>

<p><syn><b>Syn.</b> -- Mutable; alterable; variable; inconstant; fitful; vacillating; capricious; fickle; unstable; unsteady; unsettled; wavering; erratic; giddy; volatile.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeableness</ent><br/
<hw>Change"a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being changeable; fickleness; inconstancy; mutability.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeably</ent><br/
<hw>Change"a*bly</hw>, <pos>adv.</pos> <def>In a changeable manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeful</ent><br/
<hw>Change"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of change; mutable; inconstant; fickle; uncertain.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His course had been <qex>changeful</qex>.</q> <rj><qau>Motley.</qau></rj></p>

<p>-- <wordforms><wf>Change"ful*ly</wf>, <pos>adv.</pos> -- <wf>Change"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Change gear</ent><br/
<hw>Change gear</hw>. <fld>(Mach.)</fld> <def>A gear by means of which the speed of machinery or of a vehicle may be changed while that of the propelling engine or motor remains constant; -- called also <altname><col><b>change-speed gear</b></col></altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Change key</ent><br/
<hw>Change key</hw>. <def>A key adapted to open only one of a set of locks; -- distinguished from a <contr>master key</contr>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Changeless</ent><br/
<hw>Change"less</hw>, <pos>a.</pos> <def>That can not be changed; constant; <as>as, a <ex>changeless</ex> purpose</as>.</def></p>

<p>-- <wordforms><wf>Change"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeling</ent><br/
<hw>Change"ling</hw>, <pos>n.</pos> <ety>[<ets>Change</ets> + <ets>-ling</ets>.]</ety> <sn>1.</sn> <def>One who, or that which, is left or taken in the place of another, as a child exchanged by fairies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such, men do <qex>changelings</qex> call, so changed by fairies' theft.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>changeling</qex> [a substituted writing] never known.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A simpleton; an idiot.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Changelings</qex> and fools of heaven, and thence shut out.</q><br/
[<source>1913 Webster</source>]</p>

<p><q>Wildly we roam in discontent about.</q>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One apt to change; a waverer.</def> <ldquo/Fickle <xex>changelings</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Changeling</ent><br/
<hw>Change"ling</hw>, <pos>a.</pos> <sn>1.</sn> <def>Taken or left in place of another; changed.</def> <ldquo/A little <xex>changeling</xex> boy.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Given to change; inconstant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some are so studiously <qex>changeling</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>change-of-pace</ent><br/
<hw>change-of-pace</hw> <pos>n.</pos> <fld>(Baseball)</fld> <def>a baseball pitch thrown with little velocity when the batter is expecting a fastball; -- called also <altname>change-up</altname>.</def><br/
<syn><b>Syn. --</b> change-up, change-of-pace ball, off-speed pitch.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>changeover</ent><br/
<hw>changeover</hw> <pos>n.</pos> <def>an event that results in a transformation.</def><br/
<syn><b>Syn. --</b> conversion, transition.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Changer</ent><br/
<hw>Chan"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who changes or alters the form of anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who deals in or changes money.</def>  <rj><au>John ii. 14.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One apt to change; an inconstant person.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>an electronic device which changes one replaceable medium for another, such as a <stype>record changer</stype>, which can store several records and move each one automatically to the playing table; or a <stype>CD changer</stype>, whch can store multiple compact disks and move each one to the reading slot, in a sequence determined by the user.</def><br/
[<source>PJC</source>]</p>

<p><ent>change-ringing</ent><br/
<hw>change-ringing</hw> <pos>n.</pos> <def>ringing tuned bells in a fixed order that is continually changing.  See <er>change{9}</er>, <pos>n.</pos></def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>change-up</ent><br/
<hw>change-up</hw> <pos>n.</pos> <fld>(Baseball)</fld> <def>same as <er>change-of-pace</er>.</def><br/
<syn><b>Syn. --</b> change-of-pace, change-of-pace ball, off-speed pitch.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chank</ent><br/
||<hw>Chank"</hw> <pr>(ch<acr/<nsm/k)</pr>, <pos>n.</pos> <ety>[Skr. <ets><cced/a<ndot/kha</ets>. See <er>Conch</er>.]</ety> <fld>(Zool.)</fld> <def>The East Indian name for the large spiral shell of several species of sea conch much used in making bangles, esp. <spn>Turbinella pyrum</spn>. Called also <altname>chank shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Channel</ent><br/
<hw>Chan"nel</hw> <pr>(ch<acr/n"n<ecr/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>chanel</ets>, <ets>canel</ets>, OF. <ets>chanel</ets>, F. <ets>chenel</ets>, fr. L. <ets>canalis</ets>. See <er>Canal</er>.]</ety> <sn>1.</sn> <def>The hollow bed where a stream of water runs or may run.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The deeper part of a river, harbor, strait, etc., where the main current flows, or which affords the best and safest passage for vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geog.)</fld> <def>A strait, or narrow sea, between two portions of lands; <as>as, the British <ex>Channel</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That through which anything passes; a means of passing, conveying, or transmitting; <as>as, the news was conveyed to us by different <ex>channels</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The veins are converging <qex>channels</qex>.</q> <rj><qau>Dalton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At best, he is but a <qex>channel</qex> to convey to the National assembly such matter as may import that body to know.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A gutter; a groove, as in a fluted column.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <ety>[Cf. <er>Chain wales</er>.]</ety> <fld>(Naut.)</fld> <def>Flat ledges of heavy plank bolted edgewise to the outside of a vessel, to increase the spread of the shrouds and carry them clear of the bulwarks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <def>official routes of communication, especially the official means by which information should be transmitted in a bureaucracy; <as>as, to submit a request through <ex>channels</ex>; you have to go through <ex>channels</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn> <def>a band of electromagnetic wave frequencies that is used for one-way or two-way radio communication; especially, the frequency bands assigned by the FTC for use in television broadcasting, and designated by a specific number; <as>as, <ex>channel</ex> 2 in New York is owned by CBS</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>9.</sn> <def>one of the signals in an electronic device which receives or sends more than one signal simultaneously, as in stereophonic radios, records, or CD players, or in measuring equipment which gathers multiple measurements simultaneously.</def><br/
[<source>PJC</source>]</p>

<p><sn>10.</sn> <fld>(Cell biology)</fld> <def>an opening in a cell membrane which serves to actively transport or allow passive transport of substances across the membrane; <as>as, an ion <ex>channel</ex> in a nerve cell</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>11.</sn> <fld>(Computers)</fld> <def>a path for transmission of signals between devices within a computer or between a computer and an external device; <as>as, a DMA <ex>channel</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Channel bar</b></col>, <col><b>Channel iron</b></col></mcol> <fld>(Arch.)</fld>, <cd>an iron bar or beam having a section resembling a flat gutter or channel.</cd> -- <col><b>Channel bill</b></col> <fld>(Zool.)</fld>, <cd>a very large Australian cuckoo (<spn>Scythrops Nov<ae/hollandi<ae/</spn>.</cd> -- <col><b>Channel goose</b></col>. <fld>(Zool.)</fld> <cd>See <er>Gannet</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Channel</ent><br/
<hw>Chan"nel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Channeled</conjf> <pr>(?)</pr>, or <conjf>Channelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Channeling</conjf>, or <conjf>Channelling</conjf>.]</vmorph> <sn>1.</sn> <def>To form a channel in; to cut or wear a channel or channels in; to groove.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No more shall trenching war <qex>channel</qex> her fields.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To course through or over, as in a channel.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Channeling</ent><br/
<hw>Chan"nel*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of forming a channel or channels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A channel or a system of channels; a groove.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanson</ent><br/
<hw>Chan"son</hw>, <pos>n.</pos> <ety>[F., fr. L. <ets>cantion</ets> song. See <er>Cantion</er>, <er>Canzone</er>.]</ety> <def>A song.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanson de geste</ent><br/
||<hw>Chan`son" de geste"</hw> <pr>(?)</pr>. <ety>[F., prop., song of history.]</ety> <def>Any Old French epic poem having for its subject events or exploits of early French history, real or legendary, and written originally in assonant verse of ten or twelve syllables. The most famous one is the <stype>Chanson de Roland</stype>.</def></p>

<p><q>Langtoft had written in the ordinary measure of the later <qex>chansons de geste</qex>.</q>  <rj><qau>Saintsbury.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chansonnette</ent><br/
||<hw>Chan`son*nette"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chansonnettes</plw> <pr>(#)</pr>.</plu> <ety>[F., dim. of <ets>chanson</ets>.]</ety> <def>A little song.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These pretty little <qex>chansonnettes</qex> that he sung.</q> <rj><qau>Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chant</ent><br/
<hw>Chant</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chanted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Chanting</conjf>.]</vmorph> <ety>[F. <ets>chanter</ets>, fr. L. <ets>cantare</ets>, intens. of <ets>canere</ets> to sing. Cf. <er>Cant</er> affected speaking, and see <er>Hen</er>.]</ety> <sn>1.</sn> <def>To utter with a melodious voice; to sing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cheerful birds . . . do <qex>chant</qex> sweet music.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To celebrate in song.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The poets <qex>chant</qex> in the theaters.</q> <rj><qau>Bramhall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>To sing or recite after the manner of a chant, or to a tune called a chant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chant</ent><br/
<hw>Chant</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To make melody with the voice; to sing.</def> <ldquo/<xex>Chant</xex> to the sound of the viol.<rdquo/  <rj><au>Amos vi. 5.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>To sing, as in reciting a chant.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To chant horses</b></col> <it>or</it> <col><b>To chaunt horses</b></col></mcol>, <cd>to sing their praise; to overpraise; to cheat in selling. See <er>Chaunter</er>.</cd>  <rj><au>Thackeray.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chant</ent><br/
<hw>Chant</hw>, <pos>n.</pos> <ety>[F. <ets>chant</ets>, fr. L. <ets>cantus</ets> singing, song, fr. <ets>canere</ets> to sing. See <er>Chant</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Song; melody.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A short and simple melody, divided into two parts by double bars, to which unmetrical psalms, etc., are sung or recited. It is the most ancient form of choral music.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A psalm, etc., arranged for chanting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Twang; manner of speaking; a canting tone.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His strange face, his strange <qex>chant</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ambrosian chant</b></col>, See under <er>Ambrosian</er>. <col><b>Chant royal</b></col> <ety>[F.]</ety>, <cd>in old French poetry, a poem containing five strophes of eleven lines each, and a concluding stanza. -- each of these six parts ending with a common refrain.</cd> -- <col><b>Gregorian chant</b></col>. <cd>See under <er>Gregorian</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chantant</ent><br/
||<hw>Chan`tant"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. singing.]</ety> <fld>(Mus.)</fld> <def>Composed in a melodious and singing style.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanter</ent><br/
<hw>Chant"er</hw> <pr>(ch<adot/nt"<etil/r)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chanteur</ets>.]</ety> <sn>1.</sn> <def>One who chants; a singer or songster.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The chief singer of the chantry.</def>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The flute or finger pipe in a bagpipe. See <er>Bagpipe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The hedge sparrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanterelle</ent><br/
||<hw>Chan`te*relle"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Bot.)</fld> <def>A name for several species of mushroom, of which one (<spn>Cantharellus cibrius</spn>) is edible, the others reputed poisonous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chantey</ent><br/
<hw>Chant"ey</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chanter</ets> to sing, and <er>Chant</er>. <pos>n.</pos>]</ety> <def>A sailor's song.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>May we lift a deep-sea <qex>chantey</qex> such as seamen use at sea?</q>  <rj><qau>Kipling.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chanticleer</ent><br/
<hw>Chan"ti*cleer</hw> <pr>(ch<acr/n"t<icr/*kl<emac/r)</pr>, <pos>n.</pos> <ety>[F. <etsep>Chanteclair</etsep>, name of the cock in the <ets>Roman du Renart</ets> (<ets>Reynard the Fox</ets>); <ets>chanter</ets> to chant + <ets>clair</ets> clear. See <er>Chant</er>, and <er>Clear</er>.]</ety> <def>A cock, so called from the clearness or loudness of his voice in crowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanting</ent><br/
<hw>Chant"ing</hw> <pr>(ch<adot/nt"<icr/ng)</pr>, <pos>n.</pos> <def>Singing, esp. as a chant is sung.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chanting falcon</b></col> <fld>(Zool.)</fld>, <cd>an African falcon (<spn>Melierax canorus or musicus</spn>). The male has the habit, remarkable in a bird of prey, of singing to his mate, while she is incubating.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chantor</ent><br/
<hw>Chant"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chanter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chantress</ent><br/
<hw>Chant"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>chanteresse</ets>.]</ety> <def>A female chanter or singer.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 240 --></p>

<p><ent>Chantry</ent><br/
<hw>Chant"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chantries</plw> <pr>(#)</pr>.</plu> <ety>[OF. <ets>chanterie</ets>, fr. <ets>chanter</ets> to sing.]</ety> <sn>1.</sn> <def>An endowment or foundation for the chanting of masses and offering of prayers, commonly for the founder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A chapel or altar so endowed.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chanuka</ent><br/
<hw>Chan"u*ka</hw> <pr>(?)</pr>, <pos>n.</pos> <def>same as <er>Hanukka</er>; -- a variant spelling.</def><br/
<syn><b>Syn. --</b> Hanukkah, Festival of Lights, Feast of Dedication, Feast of the Dedication.</syn><br/
[<source>WordNet 1.5</source> + <source>PJC</source>]</p>

<p><ent>Chaomancy</ent><br/
<hw>Cha"o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the atmosphere + <ets>-mancy</ets>.]</ety> <def>Divination by means of appearances in the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaos</ent><br/
<hw>Cha"os</hw> <pr>(k<amac/"<ocr/s)</pr>, <pos>n.</pos> <ety>[L. <ets>chaos</ets> chaos (in senses 1 & 2), Gr. <grk>cha`os</grk>, fr. <grk>cha`inein</grk> (root <grk>cha</grk>) to yawn, to gape, to open widely. Cf. <er>Chasm</er>.]</ety> <sn>1.</sn> <def>An empty, immeasurable space; a yawning chasm.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Between us and there is fixed a great <qex>chaos</qex>.</q> <rj><qau>Luke xvi. 26 (Rhemish Trans.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The confused, unorganized condition or mass of matter before the creation of distinct and orderly forms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any confused or disordered collection or state of things; a confused mixture; confusion; disorder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaotic</ent><br/
<hw>Cha*ot"ic</hw> <pr>(k<asl/*<ocr/t"<icr/k)</pr>, <pos>a.</pos> <def>Resembling chaos; confused.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaotically</ent><br/
<hw>Cha*ot"ic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a chaotic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chap</ent><br/
<hw>Chap</hw> <pr>(ch<acr/p <it>or</it> ch<ocr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chapped</conjf> <pr>(ch<acr/pt <it>or</it> ch<ocr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chapping</conjf>.]</vmorph> <ety>[See <er>Chop</er> to cut.]</ety> <sn>1.</sn> <def>To cause to open in slits or chinks; to split; to cause the skin of to crack or become rough.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then would unbalanced heat licentious reign,<br/
Crack the dry hill, and <qex>chap</qex> the russet plain.</q> <rj><qau>Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor winter's blast <qex>chap</qex> her fair face.</q> <rj><qau>Lyly.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike; to beat.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chap</ent><br/
<hw>Chap</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To crack or open in slits; <as>as, the earth <ex>chaps</ex>; the hands <ex>chap</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike; to knock; to rap.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chap</ent><br/
<hw>Chap</hw>, <pos>n.</pos> <ety>[From <er>Chap</er>, <ets>v. t. & i.</ets>]</ety> <sn>1.</sn> <def>A cleft, crack, or chink, as in the surface of the earth, or in the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A division; a breach, as in a party.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many clefts and <qex>chaps</qex> in our council board.</q> <rj><qau>T. Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A blow; a rap.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chap</ent><br/
<hw>Chap</hw> <pr>(ch<ocr/p)</pr>, <pos>n.</pos> <ety>[OE. <ets>chaft</ets>; of Scand. origin; cf. Icel <ets>kjaptr</ets> jaw, Sw. <ets>K<aum/ft</ets>, D. <ets>ki<ae/ft</ets>; akin to G. <ets>kiefer</ets>, and E. <ets>jowl</ets>. Cf. <er>Chops</er>.]</ety> <sn>1.</sn> <def>One of the jaws or the fleshy covering of a jaw; -- commonly in the plural, and used of animals, and colloquially of human beings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>chaps</qex> were all besmeared with crimson blood.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He unseamed him [Macdonald] from the nave to the <qex>chaps</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the jaws or cheeks of a vise, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chap</ent><br/
<hw>Chap</hw> <pr>(ch<acr/p)</pr>, <pos>n.</pos> <ety>[Perh. abbreviated fr. <ets>chapman</ets>, but used in a more general sense; or cf. Dan. <ets>ki<ae/ft</ets> jaw, person, E. <ets>chap</ets> jaw.]</ety> <sn>1.</sn> <def>A buyer; a chapman.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If you want to sell, here is your <qex>chap</qex>.</q> <rj><qau>Steele.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A man or boy; a youth; a fellow.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chap</ent><br/
<hw>Chap</hw>, <pos>v. i.</pos> <ety>[See <er>Cheapen</er>.]</ety> <def>To bargain; to buy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaparajos</ent><br/
||<hw>Cha`pa*ra"jos</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Mex. Sp.]</ety> <def>Overalls of sheepskin or leather, usually open at the back, worn, esp. by cowboys, to protect the legs from thorny bushes, as in the chaparral; -- called also <altname>chapareras</altname> or colloq. <altname>chaps</altname>.</def> <mark>[Sp. Amer.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chapareras</ent><br/
||<hw>Cha`pa*re"ras</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Mex. Sp.]</ety> <def>Same as <er>Chaparajos</er>.</def> <mark>[Sp. Amer.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chaparral</ent><br/
||<hw>Cha`par*ral"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. <ets>chaparro</ets> an evergeen oak.]</ety> <sn>1.</sn> <def>A thicket of low evergreen oaks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An almost impenetrable thicket or succession of thickets of thorny shrubs and brambles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Chaparral cock</b></col>; <xex>fem</xex>. <col><b>Chaparral hen</b></col></mcol> <fld>(Zool.)</fld>, <cd>a bird of the cuckoo family (<spn>Geococcyx Californianus</spn>), noted for running with great speed. It ranges from California to Mexico and eastward to Texas; -- called also <altname>road runner</altname>, <altname>ground cuckoo</altname>, <altname>churea</altname>, and <altname>snake killer</altname>.  It is the state bird of <state>New Mexico</state>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>chapatti</ent><br/
<ent>chapati</ent><br/
<mhw><hw>chapati</hw>, <hw>chapatti</hw></mhw> <pos>n.</pos> <def>a flat pancakelike bread cooked on a griddle, originating in India.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chapbook</ent><br/
<hw>Chap"book`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chap</er> to cheapen.]</ety> <def>Any small book carried about for sale by chapmen or hawkers.  Hence, any small book; a toy book.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chape</ent><br/
<hw>Chape</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a churchman's cope, a cover, a chape, fr. L. <ets>cappa</ets>. See <er>Cap</er>.]</ety> <sn>1.</sn> <def>The piece by which an object is attached to something, as the frog of a scabbard or the metal loop at the back of a buckle by which it is fastened to a strap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The transverse guard of a sword or dagger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The metal plate or tip which protects the end of a scabbard, belt, etc.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapeau</ent><br/
<hw>Cha`peau"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chapeux</plw> <pr>(#)</pr>.</plu> <ety>[F., fr. OF. <ets>chapel</ets> hat. See <er>Chaplet</er>.]</ety> <sn>1.</sn> <def>A hat or covering for the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>A cap of maintenance. See <er>Maintenance</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Chapeau bras</b></col> <pr>(<?/)</pr> <ety>[F. <ets>chapeau</ets> hat + <ets>bras</ets> arm]</ety>, <cd>a hat so made that it can be compressed and carried under the arm without injury. Such hats were particularly worn on dress occasions by gentlemen in the 18th century. A <xex>chapeau bras</xex> is now worn in the United States army by general and staff officers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaped</ent><br/
<hw>Chaped</hw> <pr>(?)</pr>, <pos>p. p. <or/ a.</pos> <def>Furnished with a chape or chapes.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapel</ent><br/
<hw>Chap"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chapele</ets>, F. <ets>chapelle</ets>, fr. LL. <ets>capella</ets>, orig., a short cloak, hood, or cowl; later, a reliquary, sacred vessel, chapel; dim. of <ets>cappa</ets>, <ets>capa</ets>, cloak, cape, cope; also, a covering for the head. The chapel where St. Martin's cloak was preserved as a precious relic, itself came to be called <ets>capella</ets>, whence the name was applied to similar paces of worship, and the guardian of this cloak was called <ets>capellanus</ets>, or chaplain. See <er>Cap</er>, and cf. <er>Chaplain</er>., <er>Chaplet</er>.]</ety> <sn>1.</sn> <def>A subordinate place of worship</def>; as, <sd>(a)</sd> <def>a small church, often a private foundation, as for a memorial</def>; <sd>(b)</sd> <def>a small building attached to a church</def>; <sd>(c)</sd> <def>a room or recess in a church, containing an altar.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In Catholic churches, and also in cathedrals and abbey churches, <xex>chapels</xex> are usually annexed in the recesses on the sides of the aisles.  <rj><au>Gwilt.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place of worship not connected with a church; <as>as, the <ex>chapel</ex> of a palace, hospital, or prison</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In England, a place of worship used by dissenters from the Established Church; a meetinghouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A choir of singers, or an orchestra, attached to the court of a prince or nobleman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <sd>(a)</sd> <def>A printing office, said to be so called because printing was first carried on in England in a chapel near Westminster Abbey.</def> <sd>(b)</sd> <def>An association of workmen in a printing office.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chapel of ease</b></col>. <sd>(a)</sd> <cd>A chapel or dependent church built for the ease or a accommodation of an increasing parish, or for parishioners who live at a distance from the principal church.</cd> <sd>(b)</sd> <cd>A privy.</cd> <fld>(Law)</fld> -- <col><b>Chapel master</b></col>, <cd>a director of music in a chapel; the director of a court or orchestra.</cd> -- <col><b>To build a chapel</b></col> <fld>(Naut.)</fld>, <cd>to chapel a ship. See <er>Chapel</er>, <pos>v. t.</pos>, 2.</cd> -- <col><b>To hold a chapel</b></col>, <cd>to have a meeting of the men employed in a printing office, for the purpose of considering questions affecting their interests.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapel</ent><br/
<hw>Chap"el</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To deposit or inter in a chapel; to enshrine.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To cause (a ship taken aback in a light breeze) so to turn or make a circuit as to recover, without bracing the yards, the same tack on which she had been sailing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapeless</ent><br/
<hw>Chape"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without a chape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapelet</ent><br/
<hw>Chap"e*let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Chaplet</er>.]</ety> <sn>1.</sn> <def>A pair of straps, with stirrups, joined at the top and fastened to the pommel or the frame of the saddle, after they have been adjusted to the convenience of the rider.</def> <altsp>[Written also <asp>chaplet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of chain pump, or dredging machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapellany</ent><br/
<hw>Chap"el*la*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chapellanies</plw> <pr>(#)</pr>.</plu> <ety>[Cf. E. <ets>chapellenie</ets>, LL. <ets>capellania</ets>. See Chaplain.]</ety> <def>A chapel within the jurisdiction of a church; a subordinate ecclesiastical foundation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapelry</ent><br/
<hw>Chap"el*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>chapelerie</ets>.]</ety> <def>The territorial district legally assigned to a chapel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaperon</ent><br/
<hw>Chap"er*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chaperon</ets>. See <er>Chape</er>, <er>Cape</er>, <er>Cap</er>.]</ety> <sn>1.</sn> <def>A hood; especially, an ornamental or an official hood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His head and face covered with a <qex>chaperon</qex>, out of which there are but two holes to look through.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A device placed on the foreheads of horses which draw the hearse in pompous funerals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A matron who accompanies a young lady in public, for propriety, or as a guide and protector.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaperon</ent><br/
<hw>Chap"er*on</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chaperoned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chaperoning</conjf>.]</vmorph> <ety>[Cf. F. <ets>chaperonner</ets>, fr. <ets>chaperon</ets>.]</ety> <def>To attend in public places as a guide and protector; to matronize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fortunately Lady Bell Finley, whom I had promised to <qex>chaperon</qex>, sent to excuse herself.</q> <rj><qau>Hannah More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaperonage</ent><br/
<hw>Chap"er*on`age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Attendance of a chaperon on a lady in public; protection afforded by a chaperon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapfallen</ent><br/
<hw>Chap"fall`en</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the lower chap or jaw drooping, -- an indication of humiliation and dejection; crestfallen; discouraged. See <er>Chopfallen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapiter</ent><br/
<hw>Chap"i*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chapitel</ets>, F. <ets>chapiteau</ets>, from L. <ets>capitellum</ets>, dim. of <ets>caput</ets> head. Cf. <er>Capital</er>, <er>Chapter</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A capital <mark>[Obs.]</mark> See <er>Chapital</er>.</def>  <rj><au>Ex. xxxvi. 38.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Old Eng. Law)</fld> <def>A summary in writing of such matters as are to be inquired of or presented before justices in eyre, or justices of assize, or of the peace, in their sessions; -- also called <altname>articles</altname>.</def>  <rj><au>Jacob.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaplain</ent><br/
<hw>Chap"lain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chapelain</ets>, fr. LL. <ets>capellanus</ets>, fr. <ets>capella</ets>. See <er>Chapel</er>.]</ety> <sn>1.</sn> <def>An ecclesiastic who has a chapel, or who performs religious service in a chapel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A clergyman who is officially attached to the army or navy, to some public institution, or to a family or court, for the purpose of performing divine service.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any person (clergyman or layman) chosen to conduct religious exercises for a society, etc.; <as>as, a <ex>chaplain</ex> of a Masonic or a temperance lodge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaplaincy</ent><br/
<hw>Chap"lain*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <er>Chaplaincies</er> <pr>(<?/)</pr>.</plu> <def>The office, position, or station of a chaplain.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaplainship</ent><br/
<hw>Chap"lain*ship</hw>, <pos>n.</pos> <sn>1.</sn> <def>The office or business of a chaplain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Bethesda of some knight's <qex>chaplainship</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The possession or revenue of a chapel.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapless</ent><br/
<hw>Chap"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no lower jaw; hence, fleshless.</def> <mark>[R.]</mark> <ldquo/Yellow, <xex>chapless</xex> skulls.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaplet</ent><br/
<hw>Chap"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chapelet</ets>, dim. of OF. <ets>chapel</ets> hat, garland, dim. fr. LL. <ets>cappa</ets>. See <er>Cap</er>, and cf. <er>Chapelet</er>, <er>Chapeau</er>.]</ety> <sn>1.</sn> <def>A garland or wreath to be worn on the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A string of beads, or part of a string, used by Roman Catholic in praying; a third of a rosary, or fifty beads.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her <qex>chaplet</qex> of beads and her missal.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A small molding, carved into beads, pearls, olives, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Man.)</fld> <def>A chapelet. See <er>Chapelet</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Founding)</fld> <def>A bent piece of sheet iron, or a pin with thin plates on its ends, for holding a core in place in the mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A tuft of feathers on a peacock's head.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaplet</ent><br/
<hw>Chap"let</hw>, <pos>n.</pos> <def>A small chapel or shrine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaplet</ent><br/
<hw>Chap"let</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chapleted</conjf>.]</vmorph> <def>To adorn with a chaplet or with flowers.</def>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chapleted</ent><br/
<hw>chapleted</hw> <pos>adj.</pos> <def>provided with a chaplet; wearing a chaplet.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chapman</ent><br/
<hw>Chap"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chapmen</plw> <pr>(#)</pr>.</plu> <ety>[AS. <ets>ce<aacute/pman</ets>; <ets>ce<aacute/p</ets> trade + <ets>man</ets> man; akin to D. <ets>koopman</ets>, Sw. <ets>k<oum/pman</ets>, Dan. <ets>ki<oum/pmand</ets>, G. <ets>kaufmann</ets>.f. <er>Chap</er> to cheapen, and see <er>Cheap</er>.]</ety> <sn>1.</sn> <def>One who buys and sells; a merchant; a buyer or a seller.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The word of life is a quick commodity, and ought not, as a drug to be obtruded on those <qex>chapmen</qex> who are unwilling to buy it.</q> <rj><qau>T. Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A peddler; a hawker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chappy</ent><br/
<hw>Chap"py</hw> <pr>(?)</pr>, <def>Full of chaps; cleft; gaping; open.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaps</ent><br/
<hw>Chaps</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The jaws, or the fleshy parts about them. See <er>Chap</er>.</def> <ldquo/Open your <xex>chaps</xex> again.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaps</ent><br/
<hw>Chaps</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Short for <er>Chaparajos</er>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chapter</ent><br/
<hw>Chap"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chapitre</ets>, F. <ets>chapitre</ets>, fr. L. <ets>capitulum</ets>, dim. of <ets>caput</ets> head, the chief person or thing, the principal division of a writing, chapter. See <er>Chief</er>, and cf, <er>Chapiter</er>.]</ety> <sn>1.</sn> <def>A division of a book or treatise; <as>as, Genesis has fifty <ex>chapters</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <sd>(a)</sd> <def>An assembly of monks, or of the prebends and other clergymen connected with a cathedral, conventual, or collegiate church, or of a diocese, usually presided over by the dean.</def><sd>(b)</sd> <def>A community of canons or canonesses.</def><sd>(c)</sd> <def>A bishop's council.</def><sd>(d)</sd> <def>A business meeting of any religious community.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An organized branch of some society or fraternity as of the Freemasons.</def>  <rj><au>Robertson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A meeting of certain organized societies or orders.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A chapter house.</def> <mark>[R.]</mark>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A decretal epistle.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A location or compartment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In his bosom! In what <qex>chapter</qex> of his bosom?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Chapter head</b></col>, <it>or</it> <col><b>Chapter heading</b></col></mcol>, <cd>that which stands at the head of a chapter, as a title.</cd> -- <col><b>Chapter house</b></col>, <cd>a house or room where a chapter meets, esp. a cathedral chapter.</cd> -- <col><b>The chapter of accidents</b></col>, <cd>chance.</cd> <au>Marryat.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chapter</ent><br/
<hw>Chap"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To divide into chapters, as a book.</def>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To correct; to bring to book, <it>i. e.</it>, to demand chapter and verse.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaptrel</ent><br/
<hw>Chap"trel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chapiter</er>.]</ety> <fld>(Arch.)</fld> <def>An impost.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charr</ent><br/
<ent>Char</ent><br/
<mhw>{ <hw>Char</hw>, <hw>Charr</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Ir. <ets>cear</ets>, Gael. <ets>ceara</ets>, lit., red, blood-colored, fr. <ets>cear</ets> blood. So named from its red belly.]</ety> <fld>(Zool.)</fld> <def>One of the several species of fishes of the genus <gen>Salvelinus</gen>, allied to the spotted trout and salmon, inhabiting deep lakes in mountainous regions in Europe. In the United States, the brook trout (<spn>Salvelinus fontinalis</spn>) is sometimes called a <ex>char</ex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Char</ent><br/
<hw>Char</hw>, <pos>n.</pos> <ety>[F.]</ety> <def>A car; a chariot.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Char</ent><br/
<hw>Char</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>cherr</ets>, <ets>char</ets> a turning, time, work, AS. <ets>cerr</ets>, <ets>cyrr</ets>, turn, occasion, business, fr. <ets>cerran</ets>, <ets>cyrran</ets>, to turn; akin to OS. <ets>k<eum/rian</ets>, OHG. <ets>ch<eum/ran</ets>, G. <ets>kehren</ets>. Cf. <er>Chore</er>, <er>Ajar</er>.]</ety> <def>Work done by the day; a single job, or task; a chore.</def> <altsp>[Written also <asp>chare</asp>.]</altsp> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When thou hast done this <qex>chare</qex>, I give thee leave<br/
To play till doomsday.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chare</ent><br/
<ent>Char</ent><br/
<mhw>{ <hw>Char</hw>, <hw>Chare</hw>, }</mhw> <pos>v. t.</pos> <ety>[See 3d <er>Char</er>.]</ety> <sn>1.</sn> <def>To perform; to do; to finish.</def> <mark>[Obs.]</mark>  <rj><au>Nores.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thet char is <qex>chared</qex>, as the good wife said when she had hanged her husband.</q> <rj><qau>Old Proverb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To work or hew, as stone.</def>  <rj><au>Oxf. Gloss.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chare</ent><br/
<ent>Char</ent><br/
<mhw>{ <hw>Char</hw>, <hw>Chare</hw>, }</mhw> <pos>v. i.</pos> <def>To work by the day, without being a regularly hired servant; to do small jobs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Char</ent><br/
<hw>Char</hw> <pr>(ch<aum/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charred</conjf> <pr>(ch<aum/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Charring</conjf>.]</vmorph> <ety>[Prob. the same word as <ets>char</ets> to perform (see <er>Char</er>, <pos>n.</pos>), the modern use coming from <ets>charcoal</ets>, prop. <ets>coal-turned</ets>, turned to coal.]</ety> <sn>1.</sn> <def>To reduce to coal or carbon by exposure to heat; to reduce to charcoal; to burn to a cinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To burn slightly or partially; <as>as, to <ex>char</ex> wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chara</ent><br/
||<hw>Cha"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., of uncertain origin.]</ety> <fld>(Bot.)</fld> <def>A genus of flowerless plants, having articulated stems and whorled branches. They flourish in wet places.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Char-a-bancs</ent><br/
||<hw>Char`-a-bancs"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chars-a-banc</plw> <pr>(#)</pr>.</plu> <ety>[F.]</ety> <def>A long, light, open vehicle, with benches or seats running lengthwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>characid</ent><br/
<hw>characid</hw> <pos>n.</pos> <def>any freshwater fish of the family <fam>Characinidae</fam>; -- also called <altname>characin</altname>.</def><br/
<syn><b>Syn. --</b> characin, characin fish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Characidae</ent><br/
<hw>Characidae</hw> <pos>n.</pos> <def>a natural family of tropical freshwater fishes of Africa and southern and central America.</def><br/
<syn><b>Syn. --</b> family <fam>Characidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>characin</ent><br/
<hw>characin</hw> <pos>n.</pos> <def>any freshwater fish of the family <fam>Characinidae</fam>; -- also called <altname>characid</altname>.</def><br/
<syn><b>Syn. --</b> characin fish, characid.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Characinidae</ent><br/
<hw>Characinidae</hw> <pos>n.</pos> <def>former name of the family <er>Characidae</er>.</def><br/
<syn><b>Syn. --</b> family <fam>Characinidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Charact</ent><br/
<hw>Char"act</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A distinctive mark; a character; a letter or sign. <mark>[Obs.]</mark> See <er>Character</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In all his dressings, <qex>characts</qex>, titles, forms.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Character</ent><br/
<hw>Char"ac*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., an instrument for marking, character, Gr. <?/, fr. <?/ to make sharp, to cut into furrows, to engrave: cf. F. <ets>caract<egrave/re</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A distinctive mark; a letter, figure, or symbol.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It were much to be wished that there were throughout the world but one sort of <qex>character</qex> for each letter to express it to the eye.</q> <rj><qau>Holder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Style of writing or printing; handwriting; the peculiar form of letters used by a particular person or people; <as>as, an inscription in the Runic <ex>character</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You know the <qex>character</qex> to be your brother's?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The peculiar quality, or the sum of qualities, by which a person or a thing is distinguished from others; the stamp impressed by nature, education, or habit; that which a person or thing really is; nature; disposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>character</qex> or that dominion.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Know well each Ancient's proper <qex>character</qex>;<br/
His fable, subject, scope in every page;<br/
Religion, Country, genius of his Age.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man of . . . thoroughly subservient <qex>character</qex>.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Strength of mind; resolution; independence; individuality; <as>as, he has a great deal of <ex>character</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Moral quality; the principles and motives that control the life; <as>as, a man of <ex>character</ex>; his <ex>character</ex> saves him from suspicion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Quality, position, rank, or capacity; quality or conduct with respect to a certain office or duty; <as>as, in the miserable <ex>character</ex> of a slave; in his <ex>character</ex> as a magistrate; her <ex>character</ex> as a daughter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The estimate, individual or general, put upon a person or thing; reputation; <as>as, a man's <ex>character</ex> for truth and veracity; to give one a bad <ex>character</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This subterraneous passage is much mended since Seneca gave so bad a <qex>character</qex> of it.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A written statement as to behavior, competency, etc., given to a servant.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A unique or extraordinary individuality; a person characterized by peculiar or notable traits; a person who illustrates certain phases of character; <as>as, Randolph was a <ex>character</ex>; C<ae/sar is a great historical <ex>character</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>One of the persons of a drama or novel.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/It would be well if <xex>character</xex> and <xex>reputation</xex> were used distinctively. In truth, character is what a person is; reputation is what he is supposed to be. Character is in himself, reputation is in the minds of others. Character is injured by temptations, and by wrongdoing; reputation by slanders, and libels. Character endures throughout defamation in every form, but perishes when there is a voluntary transgression; reputation may last through numerous transgressions, but be destroyed by a single, and even an unfounded, accusation or aspersion.<rdquo/  <rj><au>Abbott.</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Character</ent><br/
<hw>Char"ac*ter</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charactered</conjf> <pr>(?)</pr>.]</vmorph><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To engrave; to inscribe.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>These trees shall be my books.<br/
And in their barks my thoughts I 'll <qex>character</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To distinguish by particular marks or traits; to describe; to characterize.</def> <mark>[R.]</mark>  <rj><au>Mitford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 241 --></p>

<p><ent>Characterism</ent><br/
<hw>Char"ac*ter*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a characterizing.]</ety> <def>A distinction of character; a characteristic.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Characteristic</ent><br/
<hw>Char`ac*ter*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>charact<eacute/ristique</ets>.]</ety> <def>Pertaining to, or serving to constitute, the character; showing the character, or distinctive qualities or traits, of a person or thing; peculiar; distinctive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Characteristic</qex> clearness of temper.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Characteristic</ent><br/
<hw>Char`ac*ter*is"tic</hw>, <pos>n.</pos> <sn>1.</sn> <def>A distinguishing trait, quality, or property; an element of character; that which characterized.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>characteristics</qex> of a true critic.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>The integral part (whether positive or negative) of a logarithm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Characteristical</ent><br/
<hw>Char`ac*ter*is"tic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characteristic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Characteristically</ent><br/
<hw>Char`ac*ter*is"tic*al*ly</hw>, <pos>adv.</pos> <def>In a characteristic manner; in a way that characterizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Characterization</ent><br/
<hw>Char`ac*ter*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of characterizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Characterize</ent><br/
<hw>Char"ac*ter*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Characterized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Characterizing</conjf>.]</vmorph> <ety>[LL. <ets>characterizare</ets>, Gr. <?/: cf. F. <ets>charact<eacute/riser</ets>.]</ety> <sn>1.</sn> <def>To make distinct and recognizable by peculiar marks or traits; to make with distinctive features.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>European, Asiatic, Chinese, African, and Grecian faces are <qex>Characterized</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To engrave or imprint.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To indicate the character of; to describe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under the name of Tamerlane he intended to <qex>characterize</qex> King William.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be a characteristic of; to make, or express the character of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The softness and effeminacy which <qex>characterize</qex> the men of rank in most countries.</q> <rj><qau>W. Irving.</qau></rj></p>

<p><sn>5.</sn> <fld>(Chem.)</fld> <def>to identify the structure or nature of; <as>as, the antibiotic activity in the sample was <ex>characterized</ex> by HPLC, and proved to be erythromycin</as>.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- To describe; distinguish; mark; designate; style; particularize; entitle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>characterized</ent><br/
<hw>characterized</hw> <pos>adj.</pos> <def>stated precisely; -- of the meaning of words or concepts.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Characterless</ent><br/
<hw>Char"ac*ter*less</hw>, <pos>a.</pos> <def>Destitute of any distinguishing quality; without character or force.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charactery</ent><br/
<hw>Char"ac*ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The art or means of characterizing; a system of signs or characters; symbolism; distinctive mark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fairies use flowers for their <qex>charactery</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is charactered; the meaning.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will construe to thee<br/
All the <qex>charactery</qex> of my sad brows.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charade</ent><br/
<hw>Cha*rade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>charade</ets>, cf. Pr. <ets>charrada</ets> long chat, It <ets>ciarlare</ets> to chat, whence E. <ets>charlatan</ets>.]</ety> <def>A verbal or acted enigma based upon a word which has two or more significant syllables or parts, each of which, as well as the word itself, is to be guessed from the descriptions or representations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charadriidae</ent><br/
<hw>Charadriidae</hw> <pos>n.</pos> <def>a natural family of birds comprising the plovers.</def><br/
<syn><b>Syn. --</b> family <fam>Charadriidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Charadriiformes</ent><br/
<hw>Charadriiformes</hw> <pos>n.</pos> <def>large diverse order of aquatic birds found along seacoasts and inland waters; shorebirds and coastal diving birds; most feed on animal life.</def><br/
<syn><b>Syn. --</b> order <ord>Charadriiformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Charadrius</ent><br/
<hw>Charadrius</hw> <pos>n.</pos> <def>the type genus of the Charadriidae; the plovers.</def><br/
<syn><b>Syn. --</b> genus <gen>Charadrius</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Charales</ent><br/
<hw>Charales</hw> <pos>n.</pos> <def>a small order of macroscopic fresh and brackish water algae with a distinct axis; the stoneworts.</def><br/
<syn><b>Syn. --</b> order <ord>Charales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Charbocle</ent><br/
<hw>Char"bo*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Carbuncle.</def> <altsp>[Written also <asp>Charboncle</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charbon</ent><br/
<hw>Char"bon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., coal, charbon.]</ety> <sn>1.</sn> <fld>(Far.)</fld> <def>A small black spot or mark remaining in the cavity of the corner tooth of a horse after the large spot or mark has become obliterated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A very contagious and fatal disease of sheep, horses, and cattle. See <er>Maligmant pustule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charcoal</ent><br/
<hw>Char"coal`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Char</er>, <pos>v. t.</pos>, to burn or to reduce to coal, and <er>Coal</er>.]</ety> <sn>1.</sn> <def>Impure carbon prepared from vegetable or animal substances; esp., coal made by charring wood in a kiln, retort, etc., from which air is excluded. It is used for fuel and in various mechanical, artistic, and chemical processes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fine Arts)</fld> <def>Finely prepared charcoal in small sticks, used as a drawing implement.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Animal charcoal</b></col>, <cd>a fine charcoal prepared by calcining bones in a closed vessel; -- used as a filtering agent in sugar refining, and as an absorbent and disinfectant.</cd> -- <col><b>Charcoal blacks</b></col>, <cd>the black pigment, consisting of burnt ivory, bone, cock, peach stones, and other substances.</cd> -- <col><b>Charcoal drawing</b></col> <fld>(Fine Arts)</fld>, <cd>a drawing made with charcoal. See <er>Charcoal</er>, 2. Until within a few years this material has been used almost exclusively for preliminary outline, etc., but at present many finished drawings are made with it.</cd> -- <col><b>Charcoal point</b></col>, <cd>a carbon pencil prepared for use in an electric light apparatus.</cd> -- <col><b>Mineral charcoal</b></col>, <cd>a term applied to silky fibrous layers of charcoal, interlaminated in beds of ordinary bituminous coal; -- known to miners as <xex>mother of coal</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>charcoal-grey</ent><br/
<ent>charcoal-gray</ent><br/
<mhw><hw>charcoal-gray</hw> <hw>charcoal-grey</hw></mhw> <pos>adj.</pos>  <def>very dark gray.</def><br/
<syn><b>Syn. --</b> charcoal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>charcuterie</ent><br/
<hw>charcuterie</hw> <pos>n.</pos> <sn>1.</sn> <def>a delicatessen that specializes in dressed meats and meat dishes, particularly pork products.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the items typically sold in a <er>charcuterie{1}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chard</ent><br/
<hw>Chard</hw> <pr>(ch<aum/rd)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>carde</ets> esculent thistle.]</ety> <sn>1.</sn> <def>The tender leaves or leafstalks of the artichoke, white beet, etc., blanched for table use.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A variety of the white beet, which produces large, succulent leaves and leafstalks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chardonnay</ent><br/
<hw>Char`don*nay"</hw> <pr>(ch<aum/r`d'n*<amac/")</pr>, <pos>n.</pos> <sn>1.</sn> <def>a white wine grape.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>dry white Chablis-type table wine made from Chardonnay grapes.</def><br/
<syn><b>Syn. --</b> Pinot Chardonnay.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chare</ent><br/
<hw>Chare</hw> <pr>(ch<acir/r)</pr>, <pos>n.</pos> <def>A narrow street.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chare</ent><br/
<hw>Chare</hw>, <pos>n. & v.</pos> <def>A chore; to chore; to do. See <er>Char</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charge</ent><br/
<hw>Charge</hw> <pr>(ch<aum/rj)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charged</conjf> <pr>(ch<aum/rjd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Charging</conjf>.]</vmorph> <ety>[OF. <ets>chargier</ets>, F. <ets>charger</ets>, fr. LL. <ets>carricare</ets>, fr. L. <ets>carrus</ets> wagon. Cf. <er>Cargo</er>, <er>Caricature</er>, <er>Cark</er>, and see <er>Car</er>.]</ety> <sn>1.</sn> <def>To lay on or impose, as a load, tax, or burden; to load; to fill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A carte that <qex>charged</qex> was with hay.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>charging</qex> of children's memories with rules.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lay on or impose, as a task, duty, or trust; to command, instruct, or exhort with authority; to enjoin; to urge earnestly; <as>as, to <ex>charge</ex> a jury; to <ex>charge</ex> the clergy of a diocese; to <ex>charge</ex> an agent</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moses . . . <qex>charged</qex> you to love the Lord your God.</q> <rj><qau>Josh. xxii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cromwell, I <qex>charge</qex> thee, fling away ambition.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To lay on, impose, or make subject to or liable for.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When land shall be <qex>charged</qex> by any lien.</q> <rj><qau>Kent.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fix or demand as a price; <as>as, he <ex>charges</ex> two dollars a barrel for apples</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To place something to the account of as a debt; to debit, <as>as, to <ex>charge</ex> one with goods</as>. Also, to enter upon the debit side of an account; <as>as, to <ex>charge</ex> a sum to one</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To impute or ascribe; to lay to one's charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No more accuse thy pen, but <qex>charge</qex> the crime<br/
On native sloth and negligence of time.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To accuse; to make a charge or assertion against (a person or thing); to lay the responsibility (for something said or done) at the door of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If he did that wrong you <qex>charge</qex> him with.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To place within or upon any firearm, piece of apparatus or machinery, the quantity it is intended and fitted to hold or bear; to load; to fill; <as>as, to <ex>charge</ex> a gun; to <ex>charge</ex> an electrical machine, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their battering cannon <qex>charged</qex> to the mouths.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To ornament with or cause to bear; <as>as, to <ex>charge</ex> an architectural member with a molding</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Her.)</fld> <def>To assume as a bearing; <as>as, he <ex>charges</ex> three roses or</as>; to add to or represent on; <as>as, he <ex>charges</ex> his shield with three roses or</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To call to account; to challenge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>charge</qex> me to an answer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To bear down upon; to rush upon; to attack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Charged</qex> our main battle's front.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To intrust; command; exhort; instruct; accuse; impeach; arraign. See <er>Accuse</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charge</ent><br/
<hw>Charge</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To make an onset or rush; <as>as, to <ex>charge</ex> with fixed bayonets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like your heroes of antiquity, he <qex>charges</qex> in iron.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/<qex>Charge</qex> for the guns!<rdquo/ he said.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To demand a price; <as>as, to <ex>charge</ex> high for goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To debit on an account; <as>as, to <ex>charge</ex> for purchases</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To squat on its belly and be still; -- a command given by a sportsman to a dog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charge</ent><br/
<hw>Charge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>charge</ets>, fr. <ets>charger</ets> to load. See <er>Charge</er>, <pos>v. t.</pos>, and cf. <er>Cargo</er>, <er>Caricature</er>.]</ety> <sn>1.</sn> <def>A load or burder laid upon a person or thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person or thing commited or intrusted to the care, custody, or management of another; a trust.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The people of a parish or church are called the <xex>charge</xex> of the clergyman who is set over them.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Custody or care of any person, thing, or place; office; responsibility; oversight; obigation; duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'Tis a great <qex>charge</qex> to come under one body's hand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Heed; care; anxiety; trouble.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Harm.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An order; a mandate or command; an injunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king gave <qex>cherge</qex> concerning Absalom.</q> <rj><qau>2. Sam. xviii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An address (esp. an earnest or impressive address) containing instruction or exhortation; <as>as, the <ex>charge</ex> of a judge to a jury; the <ex>charge</ex> of a bishop to his clergy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>An accusation of a wrong of offense; allegation; indictment; specification of something alleged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>charge</qex> of confounding very different classes of phenomena.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Whatever constitutes a burden on property, as rents, taxes, lines, etc.; costs; expense incurred; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The price demanded for a thing or service.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>An entry or a account of that which is due from one party to another; that which is debited in a business transaction; <as>as, a <ex>charge</ex> in an account book</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>That quantity, as of ammunition, electricity, ore, fuel, etc., which any apparatus, as a gun, battery, furnace, machine, etc., is intended to receive and fitted to hold, or which is actually in it at one time</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>The act of rushing upon, or towards, an enemy; a sudden onset or attack, as of troops, esp. cavalry; hence, the signal for attack; <as>as, to sound the <ex>charge</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never, in any other war afore, gave the Romans a hotter <qex>charge</qex> upon the enemies.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>charge</qex> of the light brigade.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>A position (of a weapon) fitted for attack; <as>as, to bring a weapon to the <ex>charge</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Far.)</fld> <def>A sort of plaster or ointment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <fld>(Her.)</fld> <def>A bearing. See <er>Bearing</er>, <pos>n.</pos>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <ety>[Cf. <er>Charre</er>.]</ety> <def>Thirty-six pigs of lead, each pig weighing about seventy pounds; -- called also <altname>charre</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>Weight; import; value.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many suchlike <ldquo/as's<rdquo/ of great <qex>charge</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Back charge</b></col>. <cd>See under <er>Back</er>, <pos>a.</pos></cd> -- <col><b>Bursting charge</b></col>. <sd>(a)</sd> <fld>(Mil.)</fld> <cd>The charge which bursts a shell, etc.</cd> <sd>(b)</sd> <fld>(Mining)</fld> <cd>A small quantity of fine powder to secure the ignition of a charge of coarse powder in blasting.</cd> -- <col><b>Charge and discharge</b></col> <fld>(Equity Practice)</fld>, <cd>the old mode or form of taking an account before a master in chancery.</cd> -- <col><b>Charge sheet</b></col>, <cd>the paper on which are entered at a police station all arrests and accusations.</cd> -- <col><b>To sound the charge</b></col>, <cd>to give the signal for an attack.</cd></cs></p>

<p><syn><b>Syn.</b> -- Care; custody; trust; management; office; expense; cost; price; assault; attack; onset; injunction; command; order; mandate; instruction; accusation; indictment.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeable</ent><br/
<hw>Charge"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>That may be charged, laid, imposed, or imputes; <as>as, a duty <ex>chargeable</ex> on iron; a fault <ex>chargeable</ex> on a man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Subject to be charge or accused; liable or responsible; <as>as, revenues <ex>chargeable</ex> with a claim; a man <ex>chargeable</ex> with murder</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Serving to create expense; costly; burdensome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That we might not be <qex>chargeable</qex> to any of you.</q> <rj><qau>2. Thess. iii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For the sculptures, which are elegant, were very <qex>chargeable</qex>.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeableness</ent><br/
<hw>Charge"a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being chargeable or expensive.</def> <mark>[Obs.]</mark>  <rj><au>Whitelocke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeably</ent><br/
<hw>Charge"a*bly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>At great cost; expensively.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeant</ent><br/
<hw>Char"geant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>chargeant</ets>, fr. <ets>charger</ets> to load.]</ety> <def>Burdensome; troublesome.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charge d'affaires</ent><br/
||<hw>Char`g<eacute/" d'af`faires"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Charg<eacute/s d'affaires</plw>.</plu> <ety>[F., <ldquo/charged with affairs.<rdquo/]</ety> <def>A diplomatic representative, or minister of an inferior grade, accredited by the government of one state to the minister of foreign affairs of another; also, a substitute, <it>ad interim</it>, for an ambassador or minister plenipotentiary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeful</ent><br/
<hw>Charge"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Costly; expensive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The fineness of the gold and <qex>chargeful</qex> fashion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargehouse</ent><br/
<hw>Charge"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A schoolhouse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeless</ent><br/
<hw>Charge"less</hw>, <pos>a.</pos> <def>Free from, or with little, charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeous</ent><br/
<hw>Char"geous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Burdensome.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I was <qex>chargeous</qex> to no man.</q> <rj><qau>Wyclif, (2 Cor. xi. 9).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charger</ent><br/
<hw>Char"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which charges.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for measuring or inserting a charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A large dish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Give me here John Baptist's head in a <qex>charger</qex>.</q> <rj><qau>Matt. xiv. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A horse for battle or parade.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And furious every <qex>charger</qex> neighed.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chargeship</ent><br/
<hw>Char*ge"ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office of a <xex>charg<eacute/ d'affaires</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charily</ent><br/
<hw>Char"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a chary manner; carefully; cautiously; frugally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charina</ent><br/
<hw>Charina</hw> <pos>n.</pos> <def>a genus of boas of western North America.</def><br/
<syn><b>Syn. --</b> genus <gen>Charina</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chariness</ent><br/
<hw>Char"i*ness</hw>, <pos>n.</pos> <def>The quality of being chary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chariot</ent><br/
<hw>Char"i*ot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>Chariot</ets>, from <ets>char</ets> car. See <er>Car</er>.]</ety> <sn>1.</sn> <fld>(Antiq.)</fld> <def>A two-wheeled car or vehicle for war, racing, state processions, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>First moved the <qex>chariots</qex>, after whom the foot.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A four-wheeled pleasure or state carriage, having one seat.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chariot</ent><br/
<hw>Char"i*ot</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charioted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Charioting</conjf>.]</vmorph> <def>To convey in a chariot.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chariotee</ent><br/
<hw>Char`i*ot*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A light, covered, four-wheeled pleasure carriage with two seats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charioteer</ent><br/
<hw>Char`i*ot*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who drives a chariot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A constellation. See <er>Auriga</er>, and <er>Wagones</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charism</ent><br/
<hw>Cha"rism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <ets><?/ gift</ets>.]</ety> <fld>(Eccl.)</fld> <def>A miraculously given power, as of healing, speaking foreign languages without instruction, etc., attributed to some of the early Christians.</def><-- now usually charisma, pl. charismata. <sn>2.</sn>  a special charm or other quality of character inspiring admiration and loyalty by others to a person; -- in political leaders, it helps a politician to gather a following; in entertainers, it increases popularity and drawing power. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charismatic</ent><br/
<hw>Char`is*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a charism.</def><-- n. a member of a religious group among which charisma is considered an important element. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charitable</ent><br/
<hw>Char"i*ta*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. See <er>Charity</er>.]</ety> <sn>1.</sn> <def>Full of love and good will; benevolent; kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be thy intents wicked or <qex>charitable</qex>, . . . <br/
 . . . I will speak to thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liberal in judging of others; disposed to look on the best side, and to avoid harsh judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Liberal in benefactions to the poor; giving freely; generous; beneficent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>charitable</qex> men afford to beggars.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to charity; springing from, or intended for, charity; relating to almsgiving; eleemosynary; <as>as, a <ex>charitable</ex> institution</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Dictated by kindness; favorable; lenient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By a <qex>charitable</qex> construction it may be a sermon.</q> <rj><qau>L. Andrews.</qau></rj></p>

<p><syn><b>Syn.</b> -- Kind; beneficent; benevolent; generous; lenient; forgiving; helpful; liberal; favorable; indulgent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charitableness</ent><br/
<hw>Char"i*ta*ble*ness</hw>, <pos>n.</pos> <def>The quality of being charitable; the exercise of charity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charitably</ent><br/
<hw>Char"i*ta*bly</hw>, <pos>adv.</pos> <def>In a charitable manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charity</ent><br/
<hw>Char"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Charities</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>charit<eacute/</ets> fr. L. <ets>caritas</ets> dearness, high regard, love, from <ets>carus</ets> dear, costly, loved; asin to Skr. <ets>kam</ets> to wish, love, cf. Ir. <ets>cara</ets> a friend, W. <ets>caru</ets> to love. Cf. <er>Caress</er>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Love; universal benevolence; good will.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now abideth faith, hope, <qex>charity</qex>, three; but the greatest of these is <qex>charity</qex>.</q> <rj><qau>1. Cor. xiii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They, at least, are little to be envied, in whose hearts the great <qex>charities</qex> . . . lie dead.</q> <rj><qau>Ruskin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With malice towards none, with <qex>charity</qex> for all.</q> <rj><qau>Lincoln.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liberality in judging of men and their actions; a disposition which inclines men to put the best construction on the words and actions of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The highest exercise of <qex>charity</qex> is <qex>charity</qex> towards the uncharitable.</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Liberality to the poor and the suffering, to benevolent institutions, or to worthy causes; generosity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The heathen poet, in commending the <qex>charity</qex> of Dido to the Trojans, spake like a Christian.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Whatever is bestowed gratuitously on the needy or suffering for their relief; alms; any act of kindness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She did ill then to refuse her a <qex>charity</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A charitable institution, or a gift to create and support such an institution; <as>as, Lady Margaret's <ex>charity</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <fld>(Law)</fld> <def>Eleemosynary appointments [grants or devises] including relief of the poor or friendless, education, religious culture, and public institutions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>charities</qex> that soothe, and heal, and bless,<br/
Are scattered at the feet of man like flowers.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sisters of Charity</b></col> <fld>(R. C. Ch.)</fld>, <cd>a sisterhood of religious women engaged in works of mercy, esp. in nursing the sick; -- a popular designation. There are various orders of the Sisters of Charity.</cd></cs></p>

<p><syn><b>Syn.</b> -- Love; benevolence; good will; affection; tenderness; beneficence; liberality; almsgiving.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charivari</ent><br/
||<hw>Cha*ri`va*ri"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A mock serenade of discordant noises, made with kettles, tin horns, etc., designed to annoy and insult; -- called also <altname>shivaree</altname>.</def><br/
<syn><b>Syn. --</b> shivaree, charivari, callithump, callathump.</syn><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It was at first performed before the house of any person of advanced age who married a second time.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chark</ent><br/
<hw>Chark</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. fr. <ets>charcoal</ets>.]</ety> <def>Charcoal; a cinder.</def> <mark>[Obs.]</mark>  <rj><au>DeFoe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chark</ent><br/
<hw>Chark</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charked</conjf> <pr>(?)</pr>.]</vmorph> <def>To burn to a coal; to char.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charlatan</ent><br/
<hw>Char"la*tan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>charlatan</ets>, fr. It. <ets>ciarlatano</ets>, fr. <ets>ciarlare</ets> to chartter, prate; of imitative origin; cf. It. <ets>zirlare</ets> to whistle like a thrush.]</ety> <def>One who prates much in his own favor, and makes unwarrantable pretensions; a quack; an impostor; an empiric; a mountebank.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 242 --></p>

<p><ent>Charlatanical</ent><br/
<ent>Charlatanic</ent><br/
<mhw>{ <hw>Char`la*tan"ic</hw> <pr>(?)</pr>, <hw>Char`la*tan"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or like a charlatan; making undue pretension; empirical; pretentious; quackish.</def> -- <wordforms><wf>Char`la*tan"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charlatanism</ent><br/
<hw>Char"la*tan*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>charlatanisme</ets>.]</ety> <def>Charlatanry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charlatanry</ent><br/
<hw>Char"la*tan*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>charlatanrie</ets>, from It. <ets>ciarlataneria</ets>. See <er>Charlatan</er>.]</ety> <def>Undue pretensions to skill; quackery; wheedling; empiricism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charles's Wain</ent><br/
<hw>Charles's Wain</hw> <pr>(?)</pr>. <ety>[<ets>Charles</ets> + <ets>wain</ets>; cf. AS. <ets>Carles w<aemac/n</ets> (for <ets>w<ae/gn</ets>), Sw. <ets>karlvagnen</ets>, Dan. <ets>karlsvogn</ets>. See <er>Churl</er>, and <er>Wain</er>.]</ety> <fld>(Astron.)</fld> <def>The group of seven stars, commonly called the <xex>Big Dipper</xex>, in the constellation <xex>Ursa Major</xex>, or <xex>Great Bear</xex>. See <er>Ursa major</er>, under <er>Ursa</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ The name is sometimes also applied to the Constellation.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charlie</ent><br/
<hw>Char"lie</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A familiar nickname or substitute for <xex>Charles</xex>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A night watchman; -- an old name.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A short, pointed beard, like that worn by Charles I.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>As a proper name, a fox; -- so called in fables and familiar literature.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Charlock</ent><br/
<hw>Char"lock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cerlic</ets>; the latter part perh. fr. AS. <ets>le<aacute/c</ets> leek. Cf. <er>Hemlock</er>.]</ety> <fld>(Bot.)</fld> <def>A cruciferous plant (<spn>Brassica sinapistrum</spn>) with yellow flowers; wild mustard. It is troublesome in grain fields. Called also <altname>chardock</altname>, <altname>chardlock</altname>, <altname>chedlock</altname>, and <altname>kedlock</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Jointed charlock</b></col>, <col><b>White charlock</b></col></mcol>, <cd>a troublesome weed (<spn>Raphanus Raphanistrum</spn>) with straw-colored, whitish, or purplish flowers, and jointed pods: wild radish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charlotte</ent><br/
<hw>Char"lotte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A kind of pie or pudding made by lining a dish with slices of bread, and filling it with bread soaked in milk, and baked.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Charlotte Russe</b></col> <pr>(<?/)</pr>, <it>or</it> <col><b>||Charlotte <agrave/ la russe</b></col></mcol> <ety>[F., lit., Russian charlotte]</ety> <fld>(Cookery)</fld>, <cd>a dish composed of custard or whipped cream, inclosed in sponge cake.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charm</ent><br/
<hw>Charm</hw> <pr>(ch<aum/rm)</pr>, <pos>n.</pos> <ety>[F. <ets>charme</ets>, fr. L. <ets>carmen</ets> song, verse, incantation, for <ets>casmen</ets>, akin to Skr. <ets><cced/asman</ets>, <ets><cced/as<amac/</ets>, a laudatory song, from a root signifying <ets>to praise</ets>, <ets>to sing</ets>.]</ety> <sn>1.</sn> <def>A melody; a song.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>charm</qex> of earliest birds.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Free liberty to chant our <qex>charms</qex> at will.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A word or combination of words sung or spoken in the practice of magic; a magical combination of words, characters, etc.; an incantation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My high <qex>charms</qex> work.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which exerts an irresistible power to please and attract; that which fascinates; any alluring quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Charms</qex> strike the sight, but merit wins the soul.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>charm</qex> of beauty's powerful glance.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Anything worn for its supposed efficacy to the wearer in averting ill or securing good fortune.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Any small decorative object worn on the person, as a seal, a key, a silver whistle, or the like. Bunches of <xex>charms</xex> are often worn at the watch chain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Physics)</fld> <def>a property of certain quarks which may take the value of +1, -1 or 0.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn. --</b> Spell; incantation; conjuration; enchantment; fascination; attraction.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charm</ent><br/
<hw>Charm</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charmed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Charming</conjf>.]</vmorph> <ety>[Cf. F. <ets>charmer</ets>. See <er>Charm</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To make music upon; to tune.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here we our slender pipes may safely <qex>charm</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subdue, control, or summon by incantation or supernatural influence; to affect by magic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No witchcraft <qex>charm</qex> thee!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To subdue or overcome by some secret power, or by that which gives pleasure; to allay; to soothe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Music the fiercest grief can <qex>charm</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To attract irresistibly; to delight exceedingly; to enchant; to fascinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They, on their mirth and dance<br/
Intent, with jocund music <qex>charm</qex> his ear.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To protect with, or make invulnerable by, spells, charms, or supernatural influences; <as>as, a <ex>charmed</ex> life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I, in my own woe <qex>charmed</qex>,<br/
Could not find death.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> - To fascinate; enchant; enrapture; captivate; bewitch; allure; subdue; delight; entice; transport.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charm</ent><br/
<hw>Charm</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To use magic arts or occult power; to make use of charms.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The voice of charmers, <qex>charming</qex> never so wisely.</q> <rj><qau>Ps. lviii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To act as, or produce the effect of, a charm; to please greatly; to be fascinating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a musical sound.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>charmed</ent><br/
<hw>charmed</hw> <pos>adj.</pos> <sn>1.</sn>  <def>same as <er>captivated</er>.</def><br/
<syn><b>Syn. --</b> captivated.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn>  <def>filled with wonder and delight.</def><br/
<syn><b>Syn. --</b> beguiled, captivated, delighted, enthralled, entranced.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Charmel</ent><br/
||<hw>Char"mel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb.]</ety> <def>A fruitful field.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Libanus shall be turned into <qex>charmel</qex>, and <qex>charmel</qex> shall be esteemed as a forest.</q> <rj><qau>Isa. xxix. 17 (Douay version).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charmer</ent><br/
<hw>Charm"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who charms, or has power to charm; one who uses the power of enchantment; a magician.</def>  <rj><au>Deut. xviii. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who delights and attracts the affections.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charmeress</ent><br/
<hw>Charm"er*ess</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An enchantress.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charmful</ent><br/
<hw>Charm"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding with charms.</def> <ldquo/His <xex>charmful</xex> lyre.<rdquo/  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charming</ent><br/
<hw>Charm"ing</hw>, <pos>a.</pos> <def>Pleasing the mind or senses in a high degree; delighting; fascinating; attractive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>charming</qex> is divine philosophy.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> - Enchanting; bewitching; captivating; enrapturing; alluring; fascinating; delightful; pleasurable; graceful; lovely; amiable; pleasing; winning.</syn></p>

<p> -- <wordforms><wf>Charm"ing*ly</wf>, <pos>adv.</pos> -- <wf>Charm"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charmless</ent><br/
<hw>Charm"less</hw>, <pos>a.</pos> <def>Destitute of charms.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charnico</ent><br/
<ent>Charneco</ent><br/
<mhw>{ <hw>Char"ne*co</hw>, <hw>Char"ni*co</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>A sort of sweet wine.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charnel</ent><br/
<hw>Char"nel</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>charnel</ets> carnal, fleshly, fr. L. <ets>carnalis</ets>. See <er>Carnal</er>.]</ety> <def>Containing the bodies of the dead.</def> <ldquo/<xex>Charnel</xex> vaults.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Charnel house</b></col>, <cd>a tomb, vault, cemetery, or other place where the bones of the dead are deposited; originally, a place for the bones thrown up when digging new graves in old burial grounds.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charnel</ent><br/
<hw>Char"nel</hw>, <pos>n.</pos> <def>A charnel house; a grave; a cemetery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In their proud <qex>charnel</qex> of Thermopyl<ae/.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charon</ent><br/
<hw>Cha"ron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/.]</ety> <fld>(Cless. Myth.)</fld> <def>The son of Erebus and Nox, whose office it was to ferry the souls of the dead over the Styx, a river of the infernal regions.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charpie</ent><br/
||<hw>Char"pie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., properly fem. p. p. of OF. <ets>charpir</ets>, <ets>carpir</ets>, to pluck, fr. L. <ets>carpere</ets>. Cf. <er>Carpet</er>.]</ety> <fld>(Med.)</fld> <def>Straight threads obtained by unraveling old linen cloth; -- used for surgical dressings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charqui</ent><br/
||<hw>Char"qui</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. A term used in South America, Central America, and the Western United States.]</ety> <def>Jerked beef; beef cut into long strips and dried in the wind and sun.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charr</ent><br/
<hw>Charr</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 1st <er>Char</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charras</ent><br/
||<hw>Char"ras</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The gum resin of the hemp plant (<spn>Cannabis sativa</spn>). Same as <er>Churrus</er>.</def>  <rj><au>Balfour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charre</ent><br/
<hw>Charre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>charrus</ets> a certain weight.]</ety> <def>See <er>Charge</er>, <pos>n.</pos>, 17.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charry</ent><br/
<hw>Char"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See 6th <er>Char</er>.]</ety> <def>Pertaining to charcoal, or partaking of its qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chart</ent><br/
<hw>Chart</hw> <pr>(ch<aum/rt; k<aum/rt <it>Obs.</it>)</pr>, <pos>n.</pos> <ety>[A doublet of <ets>card</ets>: cf. F. <ets>charte</ets> charter, <ets>carte</ets> card. See <er>Card</er>, and cf. <er>Charter</er>.]</ety> <sn>1.</sn> <def>A sheet of paper, pasteboard, or the like, on which information is exhibited, esp. when the information is arranged in tabular form; <as>as, an historical <ex>chart</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A map; esp., a hydrographic or marine map; a map on which is projected a portion of water and the land which it surrounds, or by which it is surrounded, intended especially for the use of seamen; <as>as, the United States Coast Survey <ex>charts</ex>; the English Admiralty <ex>charts</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A written deed; a charter.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Globular chart</b></col>, <cd>a chart constructed on a globular projection. See under <er>Globular</er>.</cd> -- <col><b>Heliographic chart</b></col>, <cd>a map of the sun with its spots.</cd> -- <col><b>Mercator's chart</b></col>, <cd>a chart constructed on the principle of Mercator's projection. See <er>Projection</er>.</cd> -- <col><b>Plane chart</b></col>, <cd>a representation of some part of the superficies of the globe, in which its spherical form is disregarded, the meridians being drawn parallel to each other, and the parallels of latitude at equal distances.</cd> -- <col><b>Selenographic chart</b></col>, <cd>a map representing the surface of the moon.</cd> -- <col><b>Topographic chart</b></col>, <cd>a minute delineation of a limited place or region.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chart</ent><br/
<hw>Chart</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Charted</conjf>.]</vmorph> <def>To lay down in a chart; to map; to delineate; <as>as, to <ex>chart</ex> a coast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charta</ent><br/
||<hw>Char"ta</hw> <pr>(k<aum/r"t<adot/)</pr>, <pos>n.</pos> <ety>[L., leaf of paper. See <er>Chart</er>.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>Material on which instruments, books, etc., are written; parchment or paper.</def> <sd>(b)</sd> <def>A charter or deed; a writing by which a grant is made. See <er>Magna Charta</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartaceous</ent><br/
<hw>Char*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>chartaceus</ets>. See <er>Charta</er>.]</ety> <def>Resembling paper or parchment; of paper-like texture; papery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charte</ent><br/
||<hw>Charte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Chart</er>.]</ety> <def>The constitution, or fundamental law, of the French monarchy, as established on the restoration of Louis XVIII., in 1814.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charter</ent><br/
<hw>Char"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chartre</ets>, F. <ets>chartre</ets>, <ets>charte</ets>, fr. L. <ets>chartula</ets> a little paper, dim. of <ets>charta</ets>. See <er>Chart</er>, <er>Card</er>.]</ety> <sn>1.</sn> <def>A written evidence in due form of things done or granted, contracts made, etc., between man and man; a deed, or conveyance.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument in writing, from the sovereign power of a state or country, executed in due form, bestowing rights, franchises, or privileges.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king [John, <sc>a.d</sc>. 1215], with a facility somewhat suspicious, signed and sealed the <qex>charter</qex> which was required of him. This famous deed, commonly called the <ldquo/Great <qex>Charter</qex>,<rdquo/ either granted or secured very important liberties and privileges to every order of men in the kingdom.</q> <rj><qau>Hume.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An act of a legislative body creating a municipal or other corporation and defining its powers and privileges. Also, an instrument in writing from the constituted authorities of an order or society (as the Freemasons), creating a lodge and defining its powers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A special privilege, immunity, or exemption.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My mother,<br/
Who has a <qex>charter</qex> to extol her blood,<br/
When she does praise me, grieves me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Com.)</fld> <def>The letting or hiring a vessel by special contract, or the contract or instrument whereby a vessel is hired or let; <as>as, a ship is offered for sale or <ex>charter</ex></as>. See <cref>Charter party</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Charter land</b></col> <fld>(O. Eng. Law)</fld>, <cd>land held by charter, or in socage; bookland.</cd> -- <col><b>Charter member</b></col>, <cd>one of the original members of a society or corporation, esp. one named in a charter, or taking part in the first proceedings under it.</cd> -- <col><b>Charter party</b></col> <ety>[F. <ets>chartre partie</ets>, or <ets>charte partie</ets>, a divided charter; from the practice of cutting the instrument of contract in two, and giving one part to each of the contractors]</ety> <fld>(Com.)</fld>, <cd>a mercantile lease of a vessel; a specific contract by which the owners of a vessel let the entire vessel, or some principal part of the vessel, to another person, to be used by the latter in transportation for his own account, either under their charge or his.</cd> -- <col><b>People's Charter</b></col> <mark>(Eng. Hist.)</mark>, <cd>the document which embodied the demands made by the Chartists, so called, upon the English government in 1838.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charter</ent><br/
<hw>Char"ter</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chartered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chartering</conjf>.]</vmorph> <sn>1.</sn> <def>To establish by charter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hire or let by charter, as a ship. See <cref>Charter party</cref>, under <er>Charter</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartered</ent><br/
<hw>Char"tered</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Granted or established by charter; having, or existing under, a charter; having a privilege by charter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sufficiency of <qex>chartered</qex> rights.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The air, a <qex>chartered</qex> libertine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hired or let by charter, as a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charterer</ent><br/
<hw>Char"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who charters; esp. one who hires a ship for a voyage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charterhouse</ent><br/
<hw>Char"ter*house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A well known public school and charitable foundation in the building once used as a Carthusian monastery (<xex>Chartreuse</xex>) in London.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charterist</ent><br/
<hw>Char"ter*ist</hw>, <pos>n.</pos> <def>Same as <er>Chartist</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartism</ent><br/
<hw>Chart"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>charte</ets> charter. Cf. <er>Charte</er>, <er>Chart</er>.]</ety> <def>The principles of a political party in England (1838-48), which contended for universal suffrage, the vote by ballot, annual parliaments, equal electoral districts, and other radical reforms, as set forth in a document called the <xex>People's Charter</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartist</ent><br/
<hw>Chart"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A supporter or partisan of chartism.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartless</ent><br/
<hw>Chart"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Without a chart; having no guide.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not mapped; uncharted; vague.</def>  <rj><au>Barlow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartography</ent><br/
<ent>Chartographic</ent><br/
<ent>Chartographer</ent><br/
<mhw><hw>Char*tog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Char`to*graph"ic</hw> <pr>(<?/)</pr>, <pos>a.</pos>, <hw>Char*tog"ra*phy</hw> <pr>(<?/)</pr>, <pos>n.</pos>, etc.</mhw> <def>Same as <er>Cartographer</er>, <er>Cartographic</er>, <er>Cartography</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartomancy</ent><br/
<hw>Char"to*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>charta</ets> paper + <ets>-mancy</ets>. Cf. <er>Cartomancy</er>.]</ety> <def>Divination by written paper or by cards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartometer</ent><br/
<hw>Char*tom"e*ter</hw> <pr>(ch<acir/r*t<ocr/m"<esl/*t<etil/r)</pr>, <pos>n.</pos> <ety>[<ets>Chart</ets> + <ets>-meter</ets>.]</ety> <def>An instrument for measuring charts or maps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartreuse</ent><br/
||<hw>Char`treuse"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A Carthusian monastery; esp. <altname>La Grande Chartreuse</altname>, mother house of the order, in the mountains near Grenoble, France.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An alcoholic cordial, distilled from aromatic herbs; -- made at <xex>La Grande Chartreuse</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartreux</ent><br/
||<hw>Char`treux"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A Carthusian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chartulary</ent><br/
<hw>Char"tu*la*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cartulary</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Charwoman</ent><br/
<hw>Char"wom`an</hw> <pr>(ch<acir/r"w<oocr/m`<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Charwomen</plw> <pr>(ch<acir/r"w<icr/m`<ecr/n)</pr>.</plu> <ety>[See <er>Char</er> a chore.]</ety> <def>A woman hired for odd work or for single days.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chary</ent><br/
<hw>Char"y</hw> <pr>(ch<acir/r"<ycr/ <it>or</it> ch<amac/"r<ycr/; 277)</pr>, <pos>a.</pos> <ety>[AS. <ets>cearig</ets> careful, fr. <ets>cearu</ets> care. See <er>Care</er>.]</ety> <sn>1.</sn> <def>Careful; wary; cautious; not rash, or reckless; <as>as, the latest internet IPO's were shunned by investors made <ex>chary</ex> by the poor performance of the first wave of companies that went public</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>His rising reputation made him more <qex>chary</qex> of his fame.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Saving; frugal; sparing; not spendthrift; -- often used with <ptcl>of</ptcl>; <as>as, <ex>chary</ex> of his praise</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>Fastidious; picky; choosy.</def><br/
[<source>PJC</source>]</p>

<p><ent>Charybdis</ent><br/
<hw>Cha*ryb"dis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., Gr. <grk>cha`rybdis</grk>.]</ety> <def>A dangerous whirlpool on the coast of Sicily opposite Scylla on the Italian coast. It is personified as a female monster. See <er>Scylla</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasable</ent><br/
<hw>Chas"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being chased; fit for hunting.</def>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chase</ent><br/
<hw>Chase</hw> <pr>(ch<amac/s)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chased</conjf> <pr>(ch<amac/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chasing</conjf>.]</vmorph> <ety>[OF. <ets>chacier</ets>, F. <ets>chasser</ets>, fr. (assumed) LL. <ets>captiare</ets>, fr. L. <ets>captare</ets> to strive to seize. See <er>Catch</er>.]</ety> <sn>1.</sn> <def>To pursue for the purpose of killing or taking, as an enemy, or game; to hunt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are those which <qex>chased</qex> you from the field.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Philologists, who <qex>chase</qex><br/
A panting syllable through time and place.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To follow as if to catch; to pursue; to compel to move on; to drive by following; to cause to fly; -- often with <xex>away</xex> or <xex>off</xex>; <as>as, to <ex>chase</ex> the hens away</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chased</qex> by their brother's endless malice from prince to prince and from place to place.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pursue eagerly, as hunters pursue game.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chasing</qex> each other merrily.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chase</ent><br/
<hw>Chase</hw>, <pos>v. i.</pos> <def>To give chase; to hunt; <as>as, to <ex>chase</ex> around after a doctor</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chase</ent><br/
<hw>Chase</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>chasse</ets>, fr. <ets>chasser</ets>. See <er>Chase</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>Vehement pursuit for the purpose of killing or capturing, as of an enemy, or game; an earnest seeking after any object greatly desired; the act or habit of hunting; a hunt.</def> <ldquo/This mad <xex>chase</xex> of fame.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You see this <qex>chase</qex> is hotly followed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is pursued or hunted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nay, Warwick, seek thee out some other <qex>chase</qex>,<br/
For I myself must hunt this deer to death.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An open hunting ground to which game resorts, and which is private properly, thus differing from a forest, which is not private property, and from a park, which is inclosed. Sometimes written <xex>chace</xex>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Court Tennis)</fld> <def>A division of the floor of a gallery, marked by a figure or otherwise; the spot where a ball falls, and between which and the dedans the adversary must drive his ball in order to gain a point.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chase gun</b></col> <fld>(Naut.)</fld>, <cd>a cannon placed at the bow or stern of an armed vessel, and used when pursuing an enemy, or in defending the vessel when pursued.</cd> -- <col><b>Chase port</b></col> <fld>(Naut.)</fld>, <cd>a porthole from which a chase gun is fired.</cd> -- <col><b>Stern chase</b></col> <fld>(Naut.)</fld>, <cd>a chase in which the pursuing vessel follows directly in the wake of the vessel pursued.</cd> -- <col><b>cut to the chase</b></col> <fld>(Film)</fld>, <cd>a term used in action movies meaning, to shift the scene to the most exciting part, where someone is being chased.  It is used metaphorically to mean <ldquo/get to the main point<rdquo/.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Chase</ent><br/
<hw>Chase</hw>, <pos>n.</pos> <ety>[F. <ets>ch<aacute/se</ets>, fr. L. <ets>capsa</ets> box, case. See <er>Case</er> a box.]</ety> <fld>(Print.)</fld> <sn>1.</sn> <def>A rectangular iron frame in which pages or columns of type are imposed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>The part of a cannon from the re<eum/nforce or the trunnions to the swell of the muzzle. See <er>Cannon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A groove, or channel, as in the face of a wall; a trench, as for the reception of drain tile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Shipbuilding)</fld> <def>A kind of joint by which an overlap joint is changed to a flush joint, by means of a gradually deepening rabbet, as at the ends of clinker-built boats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chase</ent><br/
<hw>Chase</hw>, <pos>v. t.</pos> <ety>[A contraction of <ets>enchase</ets>.]</ety> <sn>1.</sn> <def>To ornament (a surface of metal) by embossing, cutting away parts, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut, so as to make a screw thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chased</ent><br/
<hw>chased</hw> <pr>(ch<amac/st)</pr>, <pos>n.</pos> <def>a person who is being chased; <as>as, better to be the chaser than the <ex>chased</ex></as>.</def><br/
<syn><b>Syn. --</b> pursued.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chaser</ent><br/
<hw>Chas"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who or that which chases; a pursuer; a driver; a hunter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>Same as <er>Chase gun</er>, esp. in terms <stype>bow chaser</stype> and <stype>stern chaser</stype>. See under <er>Bow</er>, <er>Stern</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaser</ent><br/
<hw>Chas"er</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who chases or engraves. See 5th <er>Chase</er>, and <er>Enchase</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A tool with several points, used for cutting or finishing screw threads, either external or internal, on work revolving in a lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasible</ent><br/
<hw>Chas"i*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chasuble</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasing</ent><br/
<hw>Chas"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art of ornamenting metal by means of chasing tools; also, a piece of ornamental work produced in this way.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasm</ent><br/
<hw>Chasm</hw> <pr>(k<acr/z'm)</pr>, <pos>n.</pos> <ety>[L. <ets>chasma</ets>, Gr. <grk>cha`sma</grk>, fr. <grk>chai`nein</grk> to gape, to open wide. See <er>Chaos</er>.]</ety> <sn>1.</sn> <def>A deep opening made by disruption, as a breach in the earth or a rock; a yawning abyss; a cleft; a fissure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That deep, romantic <qex>chasm</qex> which slanted down the green hill.</q>  <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A void space; a gap or break, as in ranks of men.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Memory . . . fills up the <qex>chasms</qex> of thought.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasmed</ent><br/
<hw>Chasmed</hw> <pr>(k<acr/z'md)</pr>, <pos>a.</pos> <def>Having gaps or a chasm.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasmy</ent><br/
<hw>Chas"my</hw> <pr>(k<acr/z"m<ycr/)</pr>, <pos>a.</pos> <def>Of or pertaining to a chasm; abounding in chasms.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They cross the <qex>chasmy</qex> torrent's foam-lit bed.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 243 --></p>

<p><ent>Chasse</ent><br/
<hw>Chas`s<eacute/"</hw> <pr>(sh<adot/s`s<asl/")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>chass<eacute/</ets>, p. p. of <ets>chasser</ets> to chase.]</ety> <sn>1.</sn> <def>A movement in dancing, as across or to the right or left.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gliding step in dancing in which one foot is kept in front of the other while moving.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chasse</ent><br/
<hw>Chas`s<eacute/"</hw>, <pos>v. i.</pos> <fld>(Dancing)</fld> <def>To make the movement called chass<eacute/; <as>as, all <ex>chass<eacute/</ex>; <ex>chass<eacute/</ex> to the right or left</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasse</ent><br/
||<hw>Chasse</hw> <pr>(sh<adot/s)</pr>, <pos>n.</pos> <ety>[See <er>Chasse-caf<eacute/</er>]</ety> <def>A small potion of spirituous liquor taken to remove the taste of coffee, tobacco, or the like; -- originally <altname>chasse-caf<eacute/</altname>, lit., <ldquo/coffee chaser.<rdquo/</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chasse-cafe</ent><br/
||<hw>Chasse`-ca`f<eacute/"</hw> <pr>(sh<adot/s` k<adot/`f<asl/")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>chasser</ets> to chase + <ets>caf<eacute/</ets> coffee.]</ety> <def>See <er>Chasse</er>, <pos>n.</pos>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chasse-maree</ent><br/
||<hw>Chasse`-ma`r<eacute/e"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>chasser</ets> to chase + <ets>mar<eacute/e</ets> tide.]</ety> <fld>(Naut.)</fld> <def>A French coasting lugger.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chasselas</ent><br/
<hw>Chas"se*las</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from the village of <etsep>Chasselas</etsep>.]</ety> <def>A white grape, esteemed for the table.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chassepot</ent><br/
||<hw>Chasse`pot"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the French inventor, A. A. <ets>Chassepot</ets>.]</ety> <fld>(Mil.)</fld> <def>A kind of breechloading, center-fire rifle, or improved needle gun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasseur</ent><br/
<hw>Chas`seur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a huntsman. See <er>Chase</er> to pursue.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>One of a body of light troops, cavalry or infantry, trained for rapid movements.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An attendant upon persons of rank or wealth, wearing a plume and sword.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great <qex>chasseur</qex> who had announced her arrival.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chassis</ent><br/
<hw>Chas"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>ch<acir/ssis</ets>.]</ety> <fld>(Mil.)</fld> <sn>1.</sn> <def>A traversing base frame, or movable railway, along which the carriage of a barbette or casemate gun moves backward and forward. [See <er>Gun carriage</er>.]</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The under part of an automobile or other motor vehicle, consisting of the frame (on which the body is mounted) with the wheels and machinery.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Chast</ent><br/
<hw>Chast</hw> <pr>(ch<amac/st)</pr>, <pos>v. t.</pos> <def>to chasten.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaste</ent><br/
<hw>Chaste</hw> <pr>(ch<amac/st)</pr>, <pos>a.</pos> <ety>[F. <ets>chaste</ets>, from L. <ets>castus</ets> pure, chaste; cf. Gr. <grk>kaqaro`s</grk> pure, Skr. <ets><cced/udh</ets> to purify.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Pure from unlawful sexual intercourse; virtuous; continent.</def> <ldquo/As <xex>chaste</xex> as Diana.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose bed is undefiled and <qex>chaste</qex> pronounced.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pure in thought and act; innocent; free from lewdness and obscenity, or indecency in act or speech; modest; <as>as, a <ex>chaste</ex> mind; <ex>chaste</ex> eyes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pure in design and expression; correct; free from barbarisms or vulgarisms; refined; simple; <as>as, a <ex>chaste</ex> style in composition or art</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That great model of <qex>chaste</qex>, lofty, and eloquence, the Book of Common Prayer.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Unmarried.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj></p>

<p><syn><b>Syn.</b> -- Undefiled; pure; virtuous; continent; immaculate; spotless.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chaste tree</b></col>. <cd>Same as <er>Agnus castus</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastely</ent><br/
<hw>Chaste"ly</hw>, <pos>adv.</pos> <def>In a chaste manner; with purity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasten</ent><br/
<hw>Chas"ten</hw> <pr>(ch<amac/"s'n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chastened</conjf> <pr>(-s'nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chastening</conjf>.]</vmorph> <ety>[OE. <ets>chastien</ets>, OF. <ets>Chastier</ets>, F. <ets>Ch<?/tier</ets>, fr. L. <ets>castigare</ets> to punish, chastise; <ets>castus</ets> pure + <ets>agere</ets> to lead, drive. See <er>Chaste</er>, <er>Act</er>, and cf. <er>Castigate</er>, <er>Chastise</er>.]</ety> <sn>1.</sn> <def>To correct by punishment; to inflict pain upon the purpose of reclaiming; to discipline; <as>as, to <ex>chasten</ex> a son with a rod</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For whom the Lord loveth he <qex>chasteneth</qex>.</q> <rj><qau>Heb. xii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To purify from errors or faults; to refine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They [classics] <qex>chasten</qex> and enlarge the mind, and excite to noble actions.</q> <rj><qau>Layard.</qau></rj></p>

<p><syn><b>Syn.</b> -- To chastise; punish; correct; discipline; castigate; afflict; subdue; purify.</syn> <usage>To <er>Chasten</er>, <er>Punish</er>, <er>Chastise</er>. To <xex>chasten</xex> is to subject to affliction or trouble, in order to produce a general change for the better in life or character. To <xex>punish</xex> is to inflict penalty for violation of law, disobedience to authority, or intentional wrongdoing. To <xex>chastise</xex> is to punish a particular offense, as with stripes, especially with the hope that suffering or disgrace may prevent a repetition of faults.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastened</ent><br/
<hw>Chas"tened</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Corrected; disciplined; refined; purified; toned down.</def>  <rj><au>Sir. W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of such a finished <qex>chastened</qex> purity.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastener</ent><br/
<hw>Chas"ten*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chastens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chasteness</ent><br/
<hw>Chaste"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Chastity; purity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Literature & Art)</fld> <def>Freedom from all that is meretricious, gaudy, or affected; <as>as, <ex>chasteness</ex> of design</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastisable</ent><br/
<hw>Chas*tis"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable or deserving of chastisement; punishable.</def>  <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastise</ent><br/
<hw>Chas*tise"</hw> <pr>(ch<acr/s*t<imac/z"; ch<acr/s"t<imac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chastised</conjf> <pr>(ch<acr/s*t<imac/zd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chastising</conjf>.]</vmorph> <ety>[OE. <ets>chastisen</ets>; <ets>chastien</ets> + ending <ets>-isen</ets> + modern <ets>-ise</ets>, <ets>-ize</ets>, L. <ets>-izare</ets>, Gr. <grk>-i`zein</grk>. See <er>Chasten</er>.]</ety> <sn>1.</sn> <def>To inflict pain upon, by means of stripes, or in any other manner, for the purpose of punishment or reformation; to punish, as with stripes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How fine my master is! I am afraid<br/
He will <qex>chastise</qex> me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am glad to see the vanity or envy of the canting chemists thus discovered and <qex>chastised</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reduce to order or obedience; to correct or purify; to free from faults or excesses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gay, social sense, by decency <qex>chastised</qex>.</q> <rj><qau>Thomson.</qau></rj></p>

<p><sn>3.</sn> <def>To criticize (a person) strongly and directly in order to correct behavior.</def><br/
<syn><b>Syn. --</b> castigate, objurgate, chasten, correct, dress down.</syn><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Chasten</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>chastised</ent><br/
<hw>chastised</hw> <pos>adj.</pos> <def>having bad behavior criticised and punished; <as>as, the <ex>chastised</ex> child sat humbly in the corner</as>.</def><br/
<syn><b>Syn. --</b> corrected, disciplined.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chastisement</ent><br/
<hw>Chas"tise*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Chastise</er>.]</ety> <def>The act of chastising; pain inflicted for punishment and correction; discipline; punishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall I so much dishonor my fair stars,<br/
On equal terms to give him <qex>chastesement</qex>!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have borne <qex>chastisement</qex>; I will not offend any more.</q> <rj><qau>Job xxxiv. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastiser</ent><br/
<hw>Chas*tis"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chastises; a punisher; a corrector.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chastiser</qex> of the rich.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chastity</ent><br/
<hw>Chas"ti*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chastet<eacute/</ets>, fr. L. <ets>castitas</ets>, fr. <ets>castus</ets>. See <er>Chaste</er>.]</ety> <sn>1.</sn> <def>The state of being chaste; purity of body; freedom from unlawful sexual intercourse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She . . . hath preserved her spotless <qex>chastity</qex>.</q> <rj><qau>T. Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Moral purity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So dear to heaven is saintly <qex>chastity</qex>,<br/
That, when a soul is found sicerely so<br/
A thousand liveried angels lackey her.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The unmarried life; celibacy.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Literature & Art)</fld> <def>Chasteness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chastize</ent><br/
<hw>chas*tize"</hw> <pos>v. t.</pos> <def>same as <er>chastise</er>; -- a variant spelling; <as>as, She <ex>chastized</ex> him for his insensitive remarks</as>.</def><br/
<syn><b>Syn. --</b> chastise, castigate, objurgate, chasten, correct.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chasuble</ent><br/
<hw>Chas"u*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chasuble</ets>, LL. <ets>casubula</ets>, <ets>cassibula</ets>, <ets>casula</ets>, a hooded garment, covering the person like a little house; cf. It. <ets>casupola</ets>, <ets>casipola</ets>, cottage, dim of L. <ets>casa</ets> cottage.]</ety> <fld>(Eccl.)</fld> <def>The outer vestment worn by the priest in saying Mass, consisting, in the Roman Catholic Church, of a broad, flat, back piece, and a narrower front piece, the two connected over the shoulders only. The back has usually a large cross, the front an upright bar or pillar, designed to be emblematical of Christ's sufferings. In the Greek Church the chasuble is a large round mantle.</def> <altsp>[Written also <asp>chasible</asp>, and <asp>chesible</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chat</ent><br/
<hw>Chat</hw> <pr>(ch<acr/t)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chatted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Chatting</conjf>.]</vmorph> <ety>[From <er>Chatter</er>. <root/22.]</ety> <def>To talk in a light and familiar manner; to converse without form or ceremony; to gossip.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>chat</qex> a while on their adventures.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- To talk; chatter; gossip; converse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chat</ent><br/
<hw>Chat</hw>, <pos>v. t.</pos> <def>To talk of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chat</ent><br/
<hw>Chat</hw>, <pos>n.</pos> <sn>1.</sn> <def>Light, familiar talk; conversation; gossip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Snuff, or fan, supply each pause of <qex>chat</qex>,<br/
With singing, laughing, ogling, and all that.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A bird of the genus <gen>Icteria</gen>, allied to the warblers, in America.  The best known species are the yellow-breasted chat (<spn>Icteria viridis</spn>), and the long-tailed chat (<spn>Icteria longicauda</spn>). In Europe the name is given to several birds of the family <fam>Saxicolid<ae/</fam>, as the <stype>stonechat</stype>, and <stype>whinchat</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bush chat</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Bush</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chat</ent><br/
<hw>Chat</hw>, <pos>n.</pos> <sn>1.</sn> <def>A twig, cone, or little branch. See <er>Chit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Mining)</fld> <def>Small stones with ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chat potatoes</b></col>, <cd>small potatoes, such as are given to swine.</cd> <mark>[Local.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chateau</ent><br/
||<hw>Cha`teau"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chateux</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>ch<acir/teau</ets> a castle. See <er>Castle</er>.]</ety> <sn>1.</sn> <def>A castle or a fortress in France.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A manor house or residence of the lord of the manor; a gentleman's country seat; also, particularly, a royal residence; <as>as, the <ex>chateau</ex> of the Louvre; the <ex>chateau</ex> of the Luxembourg</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The distinctive, French term for a fortified castle of the middle ages is <xex>ch<acir/teau-fort</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chateaubriand</ent><br/
<hw>Chateaubriand</hw> <pos>n.</pos> <def>a double-thick center cut of beef tenderloin, broiled and served with a sauce and potatoes.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><cs><col><b>||Chateau en Espagne</b></col> <pr>(<?/)</pr> <ety>[F.]</ety>, <cd>a castle in Spain, that is, a castle in the air, Spain being the region of romance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chateau-Thierry</ent><br/
<hw>Chateau-Thierry</hw> <pos>n.</pos> <def>a World War I battle in northwestern France where the Allies defeated the Germans in 1918.</def><br/
<syn><b>Syn. --</b> battle of the Marne, Belleau Wood, Marne River.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chatelaine</ent><br/
<hw>Chat"e*laine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>ch<acir/telaine</ets> the wife of a castellan, the mistress of a chateau, a chatelaine chain.]</ety> <def>An ornamental hook, or brooch worn by a lady at her waist, and having a short chain or chains attached for a watch, keys, trinkets, etc. Also used adjectively; <as>as, a <ex>chatelaine</ex> chain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatelet</ent><br/
<hw>Chat"e*let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>ch<acir/telet</ets>, dim. of <ets>ch<acir/teau</ets>. <ets>See</ets> <er>Castle</er>.]</ety> <def>A little castle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatellany</ent><br/
<hw>Chat"el*la*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>ch<acir/tellenie</ets>.]</ety> <def>Same as <er>Castellany</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chati</ent><br/
||<hw>Cha`ti"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chat</ets> cat.]</ety> <fld>(Zool.)</fld> <def>A small South American species of tiger cat (<spn>Felis mitis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatoyant</ent><br/
<hw>Cha*toy"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., p. pr. of <ets>chatoyer</ets> to be chatoyant, fr. <ets>chat</ets> cat.]</ety> <fld>(Min.)</fld> <def>Having a changeable, varying luster, or color, like that of a changeable silk, or oa a cat's eye in the dark.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatoyant</ent><br/
<hw>Cha*toy"ant</hw>, <pos>n.</pos> <fld>(Min.)</fld> <def>A hard stone, as the <xex>cat's-eye</xex>, which presents on a polished surface, and in the interior, an undulating or wary light.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatoyment</ent><br/
<hw>Cha*toy"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chatoiement</ets>. See <er>Chatoyant</er>.]</ety> <def>Changeableness of color, as in a mineral; play of colors.</def>  <rj><au>Cleaceland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chattel</ent><br/
<hw>Chat"tel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chatel</ets>; another form of <ets>catel</ets>. See <er>Cattle</er>.]</ety> <fld>(Law)</fld> <def>Any item of movable or immovable property except the freehold, or the things which are parcel of it. It is a more extensive term than <xex>goods</xex> or <xex>effects</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Chattels</xex> are personal or real: <xex>personal</xex> are such as are movable, as goods, plate, money; <xex>real</xex> are such rights in land as are less than a freehold, as leases, mortgages, growing corn, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chattel mortgage</b></col> <fld>(Law)</fld>, <cd>a mortgage on personal property, as distinguished from one on real property.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chattelism</ent><br/
<hw>Chat"tel*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or condition of holding chattels; the state of being a chattel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatter</ent><br/
<hw>Chat"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chattered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chattering</conjf>.]</vmorph> <ety>[Of imitative origin. Cf. <er>Chat</er>, <pos>v. i.</pos> <er>Chitter</er>.]</ety> <sn>1.</sn> <def>To utter sounds which somewhat resemble language, but are inarticulate and indistinct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The jaw makes answer, as the magpie <qex>chatters</qex>.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To talk idly, carelessly, or with undue rapidity; to jabber; to prate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To tame a shrew, and charm her <qex>chattering</qex> tongue.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a noise by rapid collisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>chattering</qex> teeth, and bristling hair upright.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatter</ent><br/
<hw>Chat"ter</hw>, <pos>v. t.</pos> <def>To utter rapidly, idly, or indistinctly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Begin his witless note apace to <qex>chatter</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatter</ent><br/
<hw>Chat"ter</hw>, <pos>n.</pos> <sn>1.</sn> <def>Sounds like those of a magpie or monkey; idle talk; rapid, thoughtless talk; jabber; prattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your words are but idle and empty <qex>chatter</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Noise made by collision of the teeth, as in shivering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatteration</ent><br/
<hw>Chat*ter*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or habit of chattering.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatterer</ent><br/
<hw>Chat"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A prater; an idle talker.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A bird of the family <fam>Ampelid<ae/</fam> -- so called from its monotonous note.  The <stype>Bohemion chatterer</stype> (<spn>Ampelis garrulus</spn>) inhabits the arctic regions of both continents. In America the <stype>cedar bird</stype> is a more common species. See <cref>Bohemian chatterer</cref>, and <cref>Cedar bird</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chattering</ent><br/
<hw>Chat"ter*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or habit of talking idly or rapidly, or of making inarticulate sounds; the sounds so made; noise made by the collision of the teeth; chatter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatter mark</ent><br/
<hw>Chat"ter mark`</hw>. <sd>(a)</sd> <fld>(Mach.)</fld> <def>One of the fine undulations or ripples which are formed on the surface of work by a cutting tool which chatters.</def> <sd>(b)</sd> <fld>(Geol.)</fld> <def>A short crack on a rock surface planed smooth by a glacier.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chattiness</ent><br/
<hw>Chat"ti*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being chatty, or of talking easily and pleasantly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatty</ent><br/
<hw>Chat"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Given to light, familiar talk; talkative.</def>  <rj><au>Lady M. W. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatty</ent><br/
||<hw>Chat"ty</hw>, <pos>n.</pos> <ety>[Tamil <ets>sh<amac/ti</ets>.]</ety> <def>A porous earthen pot used in India for cooling water, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chatwood</ent><br/
<hw>Chat"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chat</ets> a little stick + <ets>wood</ets>.]</ety> <def>Little sticks; twigs for burning; fuel.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaud-medley</ent><br/
<hw>Chaud"-med`ley</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chaude m<ecir/l<eacute/e</ets>; <ets>chaud</ets> hot + <ets>m<ecir/ler</ets> (Formerly sometimes spelt medler) to mingle.]</ety> <fld>(Law)</fld> <def>The killing of a person in an affray, in the heat of blood, and while under the influence of passion, thus distinguished from <xex>chance-medley</xex> or killing in self-defense, or in a casual affray.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaudron</ent><br/
<hw>Chau"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chawdron</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chauffer</ent><br/
<hw>Chauf"fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chauffoir</ets> a kind of stone, fr. <ets>chauffer</ets> to heat. See <er>Chafe</er>.]</ety> <fld>(Chem.)</fld> <def>A table stove or small furnace, usually a cylindrical box of sheet iron, with a grate at the bottom, and an open top.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chauffeur</ent><br/
||<hw>Chauf`feur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., lit., stoker.]</ety> <sn>1.</sn>  [<pluf>pl.</pluf>] <fld>(F. Hist.)</fld> <def>Brigands in bands, who, about 1793, pillaged, burned, and killed in parts of France; -- so called because they used to burn the feet of their victims to extort money.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>One who manages the running of an automobile or limousine; esp., the paid operator of a motor vehicle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chauffeuse</ent><br/
||<hw>Chauf`feuse"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fem. of <ets>chauffeur</ets>.]</ety> <def>A woman chauffeur.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chauldron</ent><br/
<hw>Chaul"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chawdron</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaun</ent><br/
<hw>Chaun</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A gap.</def> <mark>[Obs.]</mark>  <rj><au>Colgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaun</ent><br/
<hw>Chaun</hw>, <pos>v. t. & i.</pos> <def>To open; to yawn.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O, <qex>chaun</qex> thy breast.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaunt</ent><br/
<hw>Chaunt</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Chant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaunter</ent><br/
<hw>Chaunt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A street seller of ballads and other broadsides.</def> <mark>[Slang, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A deceitful, tricky dealer or horse jockey.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was a horse <qex>chaunter</qex>; he's a leg now.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The flute of a bagpipe. See <er>Chanter</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaunterie</ent><br/
<hw>Chaunt"er*ie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chantry</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaus</ent><br/
||<hw>Cha"us</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>a lynxlike animal of Asia and Africa (<spn>Lynx Lybicus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chausses</ent><br/
||<hw>Chausses</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[F.]</ety> <def>The garment for the legs and feet and for the body below the waist, worn in Europe throughout the Middle Ages; applied also to the armor for the same parts, when fixible, as of chain mail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaussure</ent><br/
||<hw>Chaus`sure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A foot covering of any kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chautauqua</ent><br/
<hw>Chau*tau"qua</hw> <pr>(?)</pr>. <def>a meeting, usually held in the summer outdoors or under a temporary tent, providing public lectures combined with entertainment such as concerts and plays.  It originated in the village of Chautauqua, N. Y., in 1874, and was popular in the late 19th and early 20th centuries.  Also, a meeting similar to this.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chautauqua system</ent><br/
<hw>Chau*tau"qua sys"tem</hw> (of education) <pr>(?)</pr>. <def>The system of home study established in connection with the summer schools assembled at Chautauqua, N. Y., by the Methodist Episcopal bishop, <person>J. H. Vincent</person>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chauvinism</ent><br/
<hw>Chau"vin*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chauvinisme</ets>, from <person>Nicolas <etsep>Chauvin</etsep></person>, a character represented as making grotesque and threatening displays of his attachment to his fallen chief, Napoleon I., in 1815 (in the play La Cocarde tricolor, 1831).]</ety> <sn>1.</sn> <def>Blind and absurd devotion to a fallen leader or an obsolete cause; hence, absurdly vainglorious or exaggerated patriotism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>exaggerated and unreasoning partisanship to any group or cause; -- <as>as, male <ex>chauvinism</ex>, i.e. belief in the superiority of males</as>.</def><br/
[<source>PJC</source>]</p>

<p>-- <wordforms><wf>Chau"vin*ist</wf>, <pos>n. & adj.</pos> -- <wf>Chau`vin*is"tic</wf> <pr>(<?/)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ To have a generous belief in the greatness of one's country is not <xex>chauvinism</xex>. It is the character of the latter quality to be wildly extravagant, to be fretful and childish and silly, to resent a doubt as an insult, and to offend by its very frankness.</note>  <rj><au>Prof. H. Tuttle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chauvinistic</ent><br/
<hw>chauvinistic</hw> <pos>adj.</pos> <def>fanatically patriotic.</def><br/
<syn><b>Syn. --</b> flag-waving(prenominal), jingoistic, nationalistic, superpatriotic, ultrapatriotic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chavender</ent><br/
<hw>Chav"en*der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cheven</er>.]</ety> <fld>(Zool.)</fld> <def>The chub.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaw</ent><br/
<hw>Chaw</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chawed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chawing</conjf>.]</vmorph> <ety>[See <er>Chew</er>.]</ety> <sn>1.</sn> <def>To grind with the teeth; to masticate, as food in eating; to chew, as the cud; to champ, as the bit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The trampling steed, with gold and purple trapped,<br/
<qex>Chawing</qex> the foamy bit, there fiercely stood.</q> <rj><qau>Surrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ruminate in thought; to consider; to keep the mind working upon; to brood over.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>A word formerly in good use, but now regarded as vulgar.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chaw</ent><br/
<hw>Chaw</hw>, <pos>n.</pos> <ety>[See <er>Chaw</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>As much as is put in the mouth at once; a chew; a quid.</def> <mark>[Law]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. <er>Jaw</er>.]</ety> <def>The jaw.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chaw bacon</b></col>, <cd>a rustic; a bumpkin; a lout.</cd> <fld>(Law)</fld> -- <col><b>Chaw tooth</b></col>, <cd>a grinder. <fld>(Law)</fld></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chawdron</ent><br/
<hw>Chaw"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chaudun</ets>, <ets>caudun</ets>, <ets>caldun</ets>; cf. G. <ets>kaldaunen</ets> guts, bowels, LL. <ets>calduna</ets> intestine, W. <ets>coluddyn</ets> gut, dim. of <ets>coludd</ets> bowels.]</ety> <def>Entrails.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>chaudron</asp>, <asp>chauldron</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chay root</ent><br/
<hw>Chay" root`</hw> <pr>(?)</pr>. <ety>[Tamil <ets>sh<amac/ya</ets>.]</ety> <def>The root of the <spn>Oldenlandia umbellata</spn>, native in India, which yieds a durable red dyestuff.</def> <altsp>[Written also <asp>choy root</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chayote</ent><br/
<hw>Cha"yo"te</hw> <pr>(?)</pr>. <sn>1.</sn> <def>a tropical West Indian vine (<spn>Sechium edule</spn>) of the gourd family, which bears small white flowers and produces an edible fruit.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the edible fruit of the <er>chayote{1}</er> vine, having a pear shape with a furrowed skin, and usually green or white.  It is also called <altname>choyote</altname>, <altname>christophene</altname>, <altname>mirliton</altname>, and <altname>vegetable pear</altname></def><br/
[<source>PJC</source>]</p>

<p><ent>Chazy epoch</ent><br/
<hw>Cha*zy" ep"och</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>An epoch at the close of the Canadian period of the American Lower Silurian system; -- so named from a township in Clinton Co., New York. See the Diagram under <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheap</ent><br/
<hw>Cheap</hw> <pr>(ch<emac/p)</pr>, <pos>n.</pos> <ety>[AS. <ets>ce<aacute/p</ets> bargain, sale, price; akin to D. <ets>koop</ets> purchase, G. <ets>kauf</ets>, Icel. <ets>kaup</ets> bargain. Cf. <er>Cheapen</er>, <er>Chapman</er>, <er>Chaffer</er>, <er>Cope</er>, v. i.]</ety> <def>A bargain; a purchase; cheapness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sack that thou hast drunk me would have bought me lights as good <qex>cheap</qex> at the dearest chandler's in Europe.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheap</ent><br/
<hw>Cheap</hw>, <pos>a.</pos> <ety>[Abbrev. fr. <ldquo/good <ets>cheap</ets><rdquo/: a good purchase or bargain; cf. F. <ets>bon march<eacute/</ets>, <ets><agrave/ bon march<eacute/</ets>. See <er>Cheap</er>, <pos>n.</pos>, <er>Cheapen</er>.]</ety> <sn>1.</sn> <def>Having a low price in market; of small cost or price, as compared with the usual price or the real value.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where there are a great sellers to a few buyers, there the thing to be sold will be <qex>cheap</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of comparatively small value; common; mean.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You grow <qex>cheap</qex> in every subject's eye.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dog cheap</b></col>, <cd>very cheap, -- a phrase formed probably by the catachrestical transposition of <xex>good</xex> cheap. <mark>[Colloq.]</mark><-- = dirt cheap?--></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheap</ent><br/
<hw>Cheap</hw>, <pos>adv.</pos> <def>Cheaply.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheap</ent><br/
<hw>Cheap</hw>, <pos>v. i.</pos> <def>To buy; to bargain.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 244 --></p>

<p><ent>Cheapen</ent><br/
<hw>Cheap"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cheapened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cheapening</conjf>.]</vmorph> <ety>[OE. <ets>cheapien</ets>, <ets>chepen</ets>, to trade, buy, sell, AS. <ets>ce<aacute/pian</ets>; akin to D. <ets>koopen</ets> to buy, G. <ets>kaufen</ets>, Icel. <ets>kaupa</ets>, Goth. <ets>kaup<omac/n</ets> to trade. Cf. <er>Chap</er> to bargain.]</ety> <sn>1.</sn> <def>To ask the price of; to bid, bargain, or chaffer for.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pretend to <qex>cheapen</qex> goods, but nothing buy.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. <er>Cheap</er>, <pos>a.</pos>]</ety> <def>To beat down the price of; to lessen the value of; to depreciate.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My proffered love has <qex>cheapened</qex> me.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheapener</ent><br/
<hw>Cheap"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who cheapens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheap-john</ent><br/
<ent>Cheap-jack</ent><br/
<mhw>{ <hw>Cheap"-jack`</hw> <pr>(?)</pr>, <hw>Cheap"-john`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A seller of low-priced, shoddy, or second goods; a hawker.</def><br/
<syn><b>Syn. --</b> huckster, hawker, tout.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheaply</ent><br/
<hw>Cheap"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>At a small price; at a low value; in a common or inferior manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheapness</ent><br/
<hw>Cheap"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Lowness in price, considering the usual price, or real value.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chear</ent><br/
<hw>Chear</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <mark>[Obs.]</mark> <def>See <er>Cheer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheat</ent><br/
<hw>Cheat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[rob. an abbrevation of <ets>escheat</ets>, lands or tenements that fall to a lord or to the state by forfeiture, or by the death of the tenant without heirs; the meaning being explained by the frauds, real or supposed, that were resorted to in procuring escheats. See <er>Escheat</er>.]</ety> <sn>1.</sn> <def>An act of deception or fraud; that which is the means of fraud or deception; a fraud; a trick; imposition; imposture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When I consider life, 'tis all a <qex>cheat</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who cheats or deceives; an impostor; a deceiver; a cheater.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Airy wonders, which <qex>cheats</qex> interpret.</q> <rj><qau>Johnson</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A troublesome grass, growing as a weed in grain fields; -- called also <altname>chess</altname>. See <er>Chess</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>The obtaining of property from another by an intentional active distortion of the truth.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When <xex>cheats</xex> are effected by deceitful or illegal symbols or tokens which may affect the public at large and against which common prudence could not have guarded, they are indictable at common law.  <rj><au>Wharton.</au></rj>
</note></p>

<p><syn><b>Syn.</b> -- Deception; imposture; fraud; delusion; artifice; trick; swindle; deceit; guile; finesse; stratagem.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheat</ent><br/
<hw>Cheat</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cheated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cheating</conjf>.]</vmorph> <ety>[See <er>Cheat</er>, <pos>n.</pos>, <er>Escheat</er>.]</ety> <sn>1.</sn> <def>To deceive and defraud; to impose upon; to trick; to swindle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am subject to a tyrant, a sorcerer, that by his cunning hath <qex>cheated</qex> me of this island.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beguile.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>cheat</qex> winter of its dreariness.</q> <rj><qau>W. Irving.</qau></rj></p>

<p><syn><b>Syn.</b> -- To trick; cozen; gull; chouse; fool; outwit; circumvent; beguile; mislead; dupe; swindle; defraud; overreach; delude; hoodwink; deceive; bamboozle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheat</ent><br/
<hw>Cheat</hw>, <pos>v. i.</pos> <def>To practice fraud or trickery; <as>as, to <ex>cheat</ex> at cards</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheat</ent><br/
<hw>Cheat</hw>, <pos>n.</pos> <ety>[Perh. from OF. <ets>chet<eacute/</ets> goods, chattels.]</ety> <def>Wheat, or bread made from wheat.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their purest <qex>cheat</qex>,<br/
Thrice bolted, kneaded, and subdued in paste.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheatable</ent><br/
<hw>Cheat"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being cheated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheatableness</ent><br/
<hw>Cheat"a*ble*ness</hw>, <pos>n.</pos> <def>Capability of being cheated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheater</ent><br/
<hw>Cheat"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who cheats.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An escheator.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chebacco</ent><br/
<hw>Che*bac"co</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <etsep>Chebacco</etsep>, the former name of Essex, a town in Massachusetts where such vessels were built.]</ety> <fld>(Naut.)</fld> <def>A narrow-sterned boat formerly much used in the Newfoundland fisheries; -- called also <altname>pinkstern</altname> and <altname>chebec</altname>.</def>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chebec</ent><br/
<hw>Che"bec</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Chebacco</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chebec</ent><br/
<hw>Che*bec"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named from its note.]</ety> <fld>(Zool.)</fld> <def>A small American bird (<spn>Empidonax minimus</spn>); the least flycatcher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chechoslovak</ent><br/
<hw>Chechoslovak</hw> <pos>n.</pos> <def>a native or inhabitant of Czechoslovakia; a Czechoslovakian.</def><br/
<syn><b>Syn. --</b> Czechoslovakian, Czech.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Check</ent><br/
<hw>Check</hw> <pr>(ch<ecr/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>chek</ets>, OF. <ets>eschec</ets>, F. <ets><eacute/chec</ets>, a stop, hindrance, orig. check in the game of chess, pl. <ets><eacute/checs</ets> chess, through Ar., fr. Pers. <ets>sh<amac/h</ets> king. See <er>Shah</er>, and cf. <er>Checkmate</er>, <er>Chess</er>, <er>Checker</er>.]</ety> <sn>1.</sn> <fld>(Chess)</fld> <def>A word of warning denoting that the king is in danger; such a menace of a player's king by an adversary's move as would, if it were any other piece, expose it to immediate capture. A king so menaced is said to be <xex>in check</xex>, and must be made safe at the next move.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A condition of interrupted or impeded progress; arrest; stop; delay; <as>as, to hold an enemy in <ex>check</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which gave a remarkable <qex>check</qex> to the first progress of Christianity.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>check</qex>, no stay, this streamlet fears.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Whatever arrests progress, or limits action; an obstacle, guard, restraint, or rebuff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Useful <qex>check</qex> upon the administration of government.</q> <rj><qau>Washington.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man whom no <qex>check</qex> could abash.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A mark, certificate, or token, by which, errors may be prevented, or a thing or person may be identified; <as>as, <ex>checks</ex> placed against items in an account; a <ex>check</ex> given for baggage; a return <ex>check</ex> on a railroad</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A written order directing a bank or banker to pay money as therein stated. See <cref>Bank check</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A woven or painted design in squares resembling the patten of a checkerboard; one of the squares of such a design; also, cloth having such a figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Falconry)</fld> <def>The forsaking by a hawk of its proper game to follow other birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Small chick or crack.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bank check</b></col>, <cd>a written order on a banker or broker to pay money in his keeping belonging to the signer.</cd> -- <col><b>Check book</b></col>, <cd>a book containing blank forms for checks upon a bank.</cd> -- <col><b>Check hook</b></col>, <cd>a hook on the saddle of a harness, over which a checkrein is looped.</cd> -- <col><b>Check list</b></col>, <cd>a list or catalogue by which things may be verified, or on which they may be checked.</cd> -- <col><b>Check nut</b></col> <fld>(Mech.)</fld>, <cd>a secondary nut, screwing down upon the primary nut to secure it.</cd> <au>Knight.</au> -- <col><b>Check valve</b></col> <fld>(Mech.)</fld>, <cd>a valve in the feed pipe of a boiler, or other conduit, to prevent the return of the feed water or other fluid.</cd> -- <col><b>To take check</b></col>, <cd>to take offense.</cd> <mark>[Obs.]</mark> <au>Dryden.</au></cs></p>

<p><syn><b>Syn.</b> -- Hindrance; setback; interruption; obstruction; reprimand; censure; rebuke; reproof; repulse; rebuff; tally; counterfoil; counterbalance; ticket; draft.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Check</ent><br/
<hw>Check</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Checked</conjf> <pr>(ch<ecr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>checking</conjf>.]</vmorph> <sn>1.</sn> <fld>(Chess)</fld> <def>To make a move which puts an adversary's piece, esp. his king, in check; to put in check.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put a sudden restraint upon; to stop temporarily; to hinder; to repress; to curb.</def> <specif>Hence,</specif> <fld>(Ice Hockey)</fld> <def>To obstruct the motion of an opposing player by contacting him with one's body.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>So many clogs to <qex>check</qex> and retard the headlong course of violence and oppression.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To verify, to guard, to make secure, by means of a mark, token, or other check; to distinguish by a check; to put a mark against (an item) after comparing with an original or a counterpart in order to secure accuracy.</def> <specif>Hence,</specif> <def>To compare with an original or with some standard, to insure correctness; <as>as, to <ex>check</ex> an account; to <ex>check</ex> with a supervisor about procedures</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>To chide, rebuke, or reprove.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The good king, his master, will <qex>check</qex> him for it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>To slack or ease off, as a brace which is too stiffly extended.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To make checks or chinks in; to cause to crack; <as>as, the sun <ex>checks</ex> timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To leave (something) in the temporary custody of another; <as>as, to <ex>check</ex> baggage; to <ex>check</ex> one's firearms at the door; to <ex>check</ex> one's coat at the cloakroom</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn> <def>To accept (something) for temporary custody from another; <as>as, to <ex>check</ex> a customer's baggage;  to <ex>check</ex> a customer's coat</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>9.</sn> <def>To make a checkered pattern upon.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- To restrain; curb; bridle; repress; control; hinder; impede; obstruct; interrupt; tally; rebuke; reprove; rebuff.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Check</ent><br/
<hw>Check</hw> <pr>(ch<ecr/k)</pr>, <pos>v. i.</pos> <def>To make a stop; to pause; -- with <xex>at</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind, once jaded by an attempt above its power, either is disabled for the future, or else <qex>checks</qex> at any vigorous undertaking ever after.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To clash or interfere.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To act as a curb or restraint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [his presence] <qex>checks</qex> too strong upon me.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To crack or gape open, as wood in drying; or to crack in small checks, as varnish, paint, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Falconry)</fld> <def>To turn, when in pursuit of proper game, and fly after other birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And like the haggard, <qex>check</qex> at every feather<br/
That comes before his eye.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Check</ent><br/
<hw>Check</hw>, <pos>a.</pos> <def>Checkered; designed in checks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>checkage</ent><br/
<hw>check"age</hw> <pr>(ch<ecr/k"<asl/j)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of checking; <as>as, the <ex>checkage</ex> of a name or of an item in a list</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The items, or the amount, to which attention is called by a check or checks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>checked</ent><br/
<hw>checked</hw> <pr>(ch<ecr/kt)</pr>, <pos>adj.</pos> <sn>1.</sn> <def>Held back from some action especially by force.</def><br/
<syn><b>Syn. --</b> curbed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>having a pattern of alternating dark and light squares in rows and columns.</def><br/
<syn><b>Syn. --</b> checkered.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Checker</ent><br/
<hw>Check"er</hw> <pr>(ch<ecr/k"<etil/r)</pr>, <pos>n.</pos> <ety>[From <er>Check</er>, <pos>v. t.</pos>]</ety> <def>One who checks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checker</ent><br/
<hw>Check"er</hw> <pr>(ch<ecr/k"<etil/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Checkered</conjf> <pr>(ch<ecr/k"<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Checkering</conjf>.]</vmorph> <ety>[From OF. <ets>eschequier</ets> a chessboard, F. <ets><eacute/chiquier</ets>. See <er>Check</er>, <pos>n.</pos>, and cf. 3d <er>Checker</er>.]</ety> <sn>1.</sn> <def>To mark with small squares like a checkerboard, as by crossing stripes of different colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To variegate or diversify with different qualities, colors, scenes, or events; esp., to subject to frequent alternations of prosperity and adversity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our minds are, as it were, <qex>checkered</qex> with truth and falsehood.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checker</ent><br/
<hw>Check"er</hw>, <pos>n.</pos> <ety>[OF. <ets>eschequier</ets>. See <er>Checker</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A piece in the game of draughts or checkers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pattern in checks; a single check.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Checkerwork.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is also written <altname>chequer</altname>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkerberry</ent><br/
<hw>Check"er*ber`ry</hw> <pr>(ch<ecr/k"<etil/r*b<ecr/r`r<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Checkerberries</plw> <pr>(#)</pr>.</plu> <fld>(Bot.)</fld> <def>A spicy plant and its bright red berry; the wintergreen (<spn>Gaultheria procumbens</spn>).  Also incorrectly applied to the partridge berry (<spn>Mitchella repens</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>checkerbloom</ent><br/
<hw>checkerbloom</hw> <pos>n.</pos> <def>a perennial purple-flowered wild mallow of West North America (<spn>Sidalcea malviflora</spn>) that is also cultivated.</def><br/
<syn><b>Syn. --</b> wild hollyhock, <spn>Sidalcea malviflora</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Checkerboard</ent><br/
<hw>Check"er*board</hw> <pr>(ch<ecr/k"<etil/r*b<omac/rd`)</pr>, <pos>n.</pos> <def>A board with sixty-four squares of alternate color, used for playing checkers, chess, or draughts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkered</ent><br/
<hw>Check"ered</hw> <pr>(ch<ecr/k"<etil/rd)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Marked with alternate squares or checks of different color or material.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dancing in the <qex>checkered</qex> shade.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Diversified or variegated in a marked manner, as in appearance, character, circumstances, etc.; <as>as, a character with a <ex>checkered</ex> past</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>checkered</qex> narrative.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkers</ent><br/
<hw>Check"ers</hw> <pr>(ch<ecr/k"<etil/rz)</pr>, <pos>n. pl.</pos> <ety>[See <er>Checher</er>, <pos>v.</pos>]</ety> <def>A game, called also <altname>draughts</altname>, played on a checkerboard by two persons, each having twelve men (counters or checkers) which are moved diagonally.  The game is ended when either of the players has lost all his men, or can not move them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkerwork</ent><br/
<hw>Check"er*work`</hw> <pr>(ch<ecr/k"<etil/r*w<ucir/rk`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Work consisting of or showing checkers varied alternately as to colors or materials.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any aggregate of varied vicissitudes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How strange a <qex>checkerwork</qex> of Providence is the life of man.</q> <rj><qau>De Foe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>check in</ent><br/
<hw>check` in"</hw> <pr>(ch<ecr/k`<icr/n")</pr>, <pos>v. i.</pos> <def>To register as a guest at a hotel, inn, motel. etc.   Converse of <contr>check out</contr>.</def><br/
[<source>PJC</source>]</p>

<p><ent>check-in</ent><br/
<hw>check"-in</hw> <pr>(ch<ecr/k"<icr/n)</pr>, <pos>n.</pos> <def>The act of registering as a guest at a hotel, inn, motel. etc; the act of checking in.  Used also attributively, as the <ex>check-in</ex> counter at a hotel.</def><br/
[<source>PJC</source>]</p>

<p><ent>Checklaton</ent><br/
<hw>Check"la*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Ciclatoun.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gilded leather.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkless</ent><br/
<hw>Check"less</hw>, <pos>a.</pos> <def>That can not be checked or restrained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkmate</ent><br/
<hw>Check"mate</hw> <pr>(ch<ecr/k"m<amac/t)</pr>, <pos>n.</pos> <ety>[F. <ets><eacute/chec et mat</ets>, fr. Per. <ets>sh<amac/h m<amac/t</ets> checkmate, lit., the king is dead, fr. Ar. <ets>m<amac/ta</ets> he died, is dead.  The king, when made prisoner, or checkmated, is assumed to be dead, and the game is finished.  See <er>Chess</er>.]</ety> <sn>1.</sn> <def>The position in the game of chess when a king is in check and cannot be released, -- which ends the game.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A complete check; utter defeat or overthrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkmate</ent><br/
<hw>Check"mate</hw> <pr>(-m<amac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Checkmated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Checkmating</conjf>.]</vmorph> <sn>1.</sn> <fld>(Chess)</fld> <def>To check (an adversary's king) in such a manner that escape in impossible; to defeat (an adversary) by putting his king in check from which there is no escape.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To defeat completely; to terminate; to thwart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>checkmate</qex> and control my just demands.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>check out</ent><br/
<hw>check` out"</hw> <pos>v. i.</pos> <sn>1.</sn> <def>To vacate a room at a hotel, notel, etc.  Converse of <contr>check in</contr>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To die.</def> <mark>[Colloq.]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>check out</ent><br/
<hw>check" out</hw> <pos>v. t.</pos> <sn>1.</sn>  <def>To inspect or examine.</def> <mark>[Colloq.]</mark> <br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To make a record of having borrowed (something) for temporary use; <as>as, to <ex>check out</ex> a book at the library; to <ex>check out</ex> equipment from a depository.</as></def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>To bring (items to be purchased) to a <cref>checkout counter</cref> (as at a supermarket or other retail store) where the prices may be totaled and the bill may be paid.</def><br/
[<source>PJC</source>]</p>

<p><ent>checkout</ent><br/
<hw>checkout</hw> <pos>n.</pos> <sn>1.</sn> <def>the latest time for vacating a hotel room without being charged for extra time; <as>as, the <ex>checkout</ex> here is 12 noon</as>.</def><br/
<syn><b>Syn. --</b> checkout time.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>A counter in a supermarket or other retail store where one pays for one's purchases.</def><br/
<syn><b>Syn. --</b> checkout counter.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>checkout counter</ent><br/
<hw>checkout counter</hw> <pos>n.</pos> <def>A counter in a supermarket where one pays for one's purchases.</def><br/
<syn><b>Syn. --</b> checkout.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>checkout time</ent><br/
<hw>checkout time</hw> <pos>n.</pos> <sn>1.</sn> <def>the latest time for vacating a hotel room without being charged for extra time; <as>as, the <ex>checkout</ex> here is 12 noon</as>.</def><br/
<syn><b>Syn. --</b> checkout.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>checkout counter</ent><br/
<hw>checkout counter</hw> <pos>n.</pos> <def>a counter in a supermarket where you pay for your purchases.</def><br/
<syn><b>Syn. --</b> checkout.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>checkpoint</ent><br/
<hw>checkpoint</hw> <pos>n.</pos> <def>a place (as at a frontier) where travellers are stopped for inspection and clearance.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Checkrein</ent><br/
<hw>Check"rein`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A short rein looped over the check hook to prevent a horse from lowering his head; -- called also a <altname>bearing rein</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A branch rein connecting the driving rein of one horse of a span or pair with the bit of the other horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkroll</ent><br/
<hw>Check"roll`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A list of servants in a household; -- called also <altname>chequer roll</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkstring</ent><br/
<hw>Check"string`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cord by which a person in a carriage or horse car may signal to the driver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checkwork</ent><br/
<hw>Check"work</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Anything made so as to form alternate squares like those of a checkerboard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Checky</ent><br/
<hw>Check"y</hw> <pr>(ch<ecr/k"<ycr/)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Divided into small alternating squares of two tinctures; -- said of the field or of an armorial bearing.</def> <altsp>[Written also <asp>checquy</asp>, <asp>chequy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheddar</ent><br/
<hw>Ched"dar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to, or made at, <etsep>Cheddar</etsep>, in England; <as>as, <ex>Cheddar</ex> cheese</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheek</ent><br/
<hw>Cheek</hw> <pr>(ch<emac/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>cheke</ets>, <ets>cheoke</ets>, AS. <ets>ce<agrave/ce</ets>, <ets>ce<ograve/ce</ets>; cf. Goth. <ets>kukjan</ets> to kiss, D. <ets>kaak</ets> cheek; perh. akin to E. <ets>chew</ets>, <ets>jaw</ets>.]</ety> <sn>1.</sn> <def>The side of the face below the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cheek bone.</def> <mark>[Obs.]</mark>  <rj><au>Caucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Mech.)</fld> <def>Those pieces of a machine, or of any timber, or stone work, which form corresponding sides, or which are similar and in pair; <as>as, the <ex>cheeks</ex> (jaws) of a vise; the <ex>cheeks</ex> of a gun carriage, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>The branches of a bridle bit.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Founding)</fld> <def>A section of a flask, so made that it can be moved laterally, to permit the removal of the pattern from the mold; the middle part of a flask.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Cool confidence; assurance; impudence.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cheek of beef</b></col>. <cd>See <xex>Illust.</xex> of <er>Beef</er>.</cd> -- <col><b>Cheek bone</b></col> <fld>(Anat.)</fld> <cd>the bone of the side of the face; esp., the malar bone.</cd> -- <col><b>Cheek by jowl</b></col>, <cd>side by side; very intimate.</cd> -- <col><b>Cheek pouch</b></col> <fld>(Zool.)</fld>, <cd>a sacklike dilation of the cheeks of certain monkeys and rodents, used for holding food.</cd> -- <col><b>Cheeks of a block</b></col>, <cd>the two sides of the shell of a tackle block.</cd> -- <col><b>Cheeks of a mast</b></col>, <cd>the projection on each side of a mast, upon which the trestletrees rest.</cd> -- <col><b>Cheek tooth</b></col> <fld>(Anat.)</fld>, <cd>a hinder or molar tooth.</cd> -- <col><b>Butment cheek</b></col>. <cd>See under <er>Butment</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheek</ent><br/
<hw>Cheek</hw> <pr>(ch<emac/k)</pr>, <pos>v. t.</pos> <def>To be impudent or saucy to.</def> <mark>[Slang.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cheekbone</ent><br/
<hw>cheekbone</hw> <pos>n.</pos> <def>the arch of bone beneath the eye that forms the prominence of the cheek.</def><br/
<syn><b>Syn. --</b> zygomatic bone, malar bone, jugal bone, os zygomaticum.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cheekless</ent><br/
<hw>cheekless</hw> <pos>adj.</pos> <def>having no cheek.  Opposite of <ant>cheeked</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cheeked</ent><br/
<hw>Cheeked</hw> <pr>(ch<emac/kt)</pr>, <pos>a.</pos> <def>Having a cheek; -- used in composition.</def> <ldquo/Rose-<xex>cheeked</xex> Adonis.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheeky</ent><br/
<hw>Cheek"y</hw>, <def>a Brazen-faced; impudent; bold.</def> <mark>[Slang.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheep</ent><br/
<hw>Cheep</hw> <pr>(ch<emac/p)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cheeped</conjf> <pr>(ch<emac/pt)</pr>.]</vmorph> <ety>[Cf. <er>Chirp</er>]</ety>. <def>To chirp, as a young bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheep</ent><br/
<hw>Cheep</hw>, <pos>v. t.</pos> <def>To give expression to in a chirping tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cheep</qex> and twitter twenty million loves.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheep</ent><br/
<hw>Cheep</hw>, <pos>n.</pos> <def>A chirp, peep, or squeak, as of a young bird or mouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheer</ent><br/
<hw>Cheer</hw> <pr>(ch<emac/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>chere</ets> face, welcome, cheer, OF. <ets>chiere</ets>, F. <ets>ch<egrave/re</ets>, fr. LL. <ets>cara</ets> face, Gr. <grk>ka`ra</grk> head; akin to Skr. <ets><cced/iras</ets>, L. <ets>cerebrum</ets> brain, G. <ets>hirn</ets>, and E. <ets>cranium</ets>.]</ety> <sn>1.</sn> <def>The face; the countenance or its expression.</def> <mark>[Obs.]</mark> <ldquo/Sweat of thy <xex>cheer</xex>.<rdquo/  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Feeling; spirit; state of mind or heart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be of good <qex>cheer</qex>.</q> <rj><qau>Matt. ix. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The parents . . . fled away with heavy <qex>cheer</qex>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Gayety; mirth; cheerfulness; animation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not that alacrity of spirit,<br/
Nor <qex>cheer</qex> of mind, that I was wont to have.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>That which promotes good spirits or cheerfulness; provisions prepared for a feast; entertainment; <as>as, a table loaded with good <ex>cheer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A shout, hurrah, or acclamation, expressing joy enthusiasm, applause, favor, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Welcome her, thundering <qex>cheer</qex> of the street.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>What cheer?</b></col> <cd>How do you fare? What is there that is cheering?</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheer</ent><br/
<hw>Cheer</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cheered</conjf> <pr>(ch<emac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>cheering</conjf>.]</vmorph> <sn>1.</sn> <def>To cause to rejoice; to gladden; to make cheerful; -- often with <xex>up</xex>.</def>  <rj><au>Cowpe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To infuse life, courage, animation, or hope, into; to inspirit; to solace or comfort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The proud he tamed, the penitent he <qex>cheered</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To salute or applaud with cheers; to urge on by cheers; <as>as, to <ex>cheer</ex> hounds in a chase</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cheer ship</b></col>, <cd>to salute a passing ship by cheers of sailors stationed in the rigging.</cd></cs></p>

<p><syn><b>Syn.</b> -- To gladden; encourage; inspirit; comfort; console; enliven; refresh; exhilarate; animate; applaud.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheer</ent><br/
<hw>Cheer</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To grow cheerful; to become gladsome or joyous; -- usually with <ptcl>up</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At sight of thee my gloomy soul <qex>cheers</qex> up.</q> <rj><qau>A. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be in any state or temper of mind.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>cheer'st</qex> thou, Jessica?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter a shout or shouts of applause, triumph, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And even the ranks of Tusculum<br/
Could scare forbear to <qex>cheer</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerer</ent><br/
<hw>Cheer"er</hw> <pr>(ch<emac/r"<etil/r)</pr>, <pos>n.</pos> <def>One who cheers; one who, or that which, gladdens.</def>  <ldquo/Thou <xex>cheerer</xex> of our days.<rdquo/ <au>Wotton.</au> <ldquo/Prime <xex>cheerer</xex>, light.<rdquo/ <au>Thomson.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerful</ent><br/
<hw>Cheer"ful</hw> <pr>(ch<emac/r"f<usdot/l; 277)</pr>, <pos>a.</pos> <def>Having or showing good spirits or joy; cheering; cheery; contented; happy; joyful; lively; animated; willing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To entertain a <qex>cheerful</qex> disposition.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cheerful</qex> birds of sundry kind<br/
Do chant sweet music.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cheerful</qex> confidence in the mercy of God.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This general applause and <qex>cheerful</qex> shout.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Lively; animated; gay; joyful; lightsome; gleeful; blithe; airy; sprightly; jocund; jolly; joyous; vivacious; buoyant; sunny; happy; hopeful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerfully</ent><br/
<hw>Cheer"ful*ly</hw>, <pos>adv.</pos> <def>In a cheerful manner, gladly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerfulness</ent><br/
<hw>Cheer"ful*ness</hw>, <pos>n.</pos> <def>Good spirits; a state of moderate joy or gayety; alacrity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerily</ent><br/
<hw>Cheer"i*ly</hw> <pr>(ch<emac/r"<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a cheery manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheeriness</ent><br/
<hw>Cheer"i*ness</hw>, <pos>n.</pos> <def>The state of being cheery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheeringly</ent><br/
<hw>Cheer"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a manner to cheer or encourage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerishness</ent><br/
<hw>Cheer"ish*ness</hw>, <pos>n.</pos> <def>Cheerfulness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no Christian duty that is not to be seasoned and set off with <qex>cheerishness</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerless</ent><br/
<hw>Cheer"less</hw>, <pos>a.</pos> <def>Without joy, gladness, or comfort.</def></p>

<p>-- <wordforms><wf>Cheer"less*ly</wf>, <pos>adv.</pos> -- <wf>Cheer"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>My cheerful day is turned to <qex>cheerless</qex> night.</q> <rj><qau>Spenser.</qau></rj></p>

<p><syn><b>Syn.</b> -- Gloomy; sad; comfortless; dispiriting; disconsolate; dejected; melancholy; forlorn.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerly</ent><br/
<hw>Cheer"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Gay; cheerful.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheerly</ent><br/
<hw>Cheer"ly</hw>, <pos>adv.</pos> <def>Cheerily.</def> <mark>[Archaic]</mark>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheery</ent><br/
<hw>Cheer"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Displaying evident cheerfulness; cheerful; lively; gay; bright; pleasant; <as>as, a <ex>cheery</ex> person; a <ex>cheery</ex> hello</as>.</def><br/
<syn><b>Syn. --</b> pleasant, sunny.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><q>His <qex>cheery</qex> little study, where the sunshine glimmered so pleasantly.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 245 --></p>

<p><ent>Cheese</ent><br/
<hw>Cheese</hw> <pr>(ch<emac/z)</pr>, <pos>n.</pos> <ety>[OE. <ets>chese</ets>, AS. <ets>c<emac/se</ets>, fr. L. <ets>caseus</ets>, LL. <ets>casius</ets>. Cf. <er>Casein</er>.]</ety> <sn>1.</sn> <def>The curd of milk, coagulated usually with rennet, separated from the whey, and pressed into a solid mass in a hoop or mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mass of pomace, or ground apples, pressed together in the form of a cheese.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The flat, circular, mucilaginous fruit of the dwarf mallow (<spn>Malva rotundifolia</spn>).</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A low courtesy; -- so called on account of the cheese form assumed by a woman's dress when she stoops after extending the skirts by a rapid gyration.</def>  <rj><au>De Quincey.</au> <au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cheese cake</b></col>, <cd>a cake made of or filled with, a composition of soft curds, sugar, and butter.</cd> <au>Prior</au>. -- <col><b>Cheese fly</b></col> <fld>(Zool.)</fld>, <cd>a black dipterous insect (<spn>Piophila casei</spn>) of which the larv<ae/ or maggots, called <xex>skippers</xex> or <xex>hoppers</xex>, live in cheese.</cd> -- <col><b>Cheese mite</b></col> <fld>(Zool.)</fld>, <cd>a minute mite (<spn>Tryoglyhus siro</spn>) in cheese and other articles of food.</cd> -- <col><b>Cheese press</b></col>, <cd>a press used in making cheese, to separate the whey from the curd, and to press the curd into a mold.</cd> -- <col><b>Cheese rennet</b></col> <fld>(Bot.)</fld>, <cd>a plant of the Madder family (<spn>Golium verum</spn>, or <altname>yellow bedstraw</altname>), sometimes used to coagulate milk. The roots are used as a substitute for madder.</cd> -- <col><b>Cheese vat</b></col>, <cd>a vat or tub in which the curd is formed and cut or broken, in cheese making.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cheeseboard</ent><br/
<hw>cheeseboard</hw> <pos>n.</pos> <def>a board on which cheeses are served.</def><br/
<syn><b>Syn. --</b> cheese tray.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cheeseburger</ent><br/
<hw>cheeseburger</hw> <pos>n.</pos> <def>a hamburger with melted cheese on it.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cheese cloth</ent><br/
<ent>cheesecloth</ent><br/
<mhw><hw>cheese"cloth`</hw>, <hw>Cheese" cloth`</hw></mhw> <pr>(ch<emac/z" kl<ocr/th`; 115)</pr>. <def>A thin, loosely-woven cotton cloth of a gauze texture, such as is used in pressing cheese curds.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cheeseflower</ent><br/
<hw>cheeseflower</hw> <pos>n.</pos> <def>an erect or decumbent Old World perennial (<spn>Malva sylvestris</spn>) with axillary clusters of rosy-purple flowers; introduced into the U. S.</def><br/
<syn><b>Syn. --</b> tall mallow, high mallow, cheese, <spn>Malva sylvestris</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cheeselep</ent><br/
<hw>Cheese"lep</hw> <pr>(ch<emac/z"l<ecr/p)</pr>, <pos>n.</pos> <ety>[Cf. <er>Keslop</er>.]</ety> <def>A bag in which rennet is kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheesemonger</ent><br/
<hw>Cheese"mon`ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who deals in cheese.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheeseparing</ent><br/
<hw>Cheese"par`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A thin portion of the rind of a cheese.</def> -- <def2><pos>a.</pos>  <def>Scrimping; mean; <as>as, <ex>cheeseparing</ex> economy</as>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheesiness</ent><br/
<hw>Chees"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being cheesy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheesy</ent><br/
<hw>Chees"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the nature, qualities, taste, form, consistency, or appearance of cheese.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheetah</ent><br/
<hw>Chee"tah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>ch<imac/t<amac/</ets>.]</ety> <fld>(Zool.)</fld> <def>A species of leopard (<spn>Cyn<ae/lurus jubatus</spn>) tamed and used for hunting in India. The woolly cheetah of South Africa is <spn>Cyn<ae/lurus laneus</spn>.  It runs very fast in short spurts while hunting.</def> <altsp>[Written also <asp>chetah</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>cheewink</ent><br/
<hw>cheewink</hw> <pos>n.</pos> <fld>(Zool.)</fld> <def>the common rufous-sided towhee of eastern North America, <spn>Pipilo erythrophthalmus</spn>.</def><br/
<syn><b>Syn. --</b> chewink, rufous-sided towhee, <spn>Pipilo erythrophthalmus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chef</ent><br/
||<hw>Chef</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A chief or head person.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The head cook of large establishment, as a club, a family, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Her.)</fld> <def>Same as <er>Chief</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chef-d'oeuvre</ent><br/
||<hw>Chef`-d'<oe/uvre"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chefs-d'<oe/uvre</plw> <pr>(#)</pr>.</plu> <ety>[F.]</ety> <def>A masterpiece; a capital work in art, literature, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chegre</ent><br/
<ent>Chegoe</ent><br/
<mhw>{ <hw>Cheg"oe</hw> <pr>(?)</pr>, <hw>Cheg"re</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>See <er>Chigoe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheiloplasty</ent><br/
<hw>Chei"lo*plas`ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a lip + <ets>-plasty</ets>.]</ety> <fld>(Surg.)</fld> <def>The process of forming an artificial tip or part of a lip, by using for the purpose a piece of healthy tissue taken from some neighboring part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheilopoda</ent><br/
||<hw>Chei*lop"o*da</hw> <pr>(k<isl/*l<ocr/p"<osl/*d<adot/)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>See <er>Chilopoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cheiropter</ent><br/
<hw>chei*rop"ter</hw> <pr>(k<isl/*r<ocr/p"t<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Chiroptera; -- an older spelling of <er>chiropter</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cheiroptera</ent><br/
||<hw>Chei*rop"te*ra</hw> <pr>(k<isl/*r<ocr/p"t<esl/*r<adot/)</pr>, <pos>prop. n. pl.</pos> <ety>[NL., fr. Gr. <grk>chei`r</grk> hand + <grk>ptero`n</grk> wing.]</ety> <fld>(Zool.)</fld> <def>An earlier spelling for <asp>Chiroptera</asp>, an order of <class>Mammalia</class>, including the bats, having four toes of each of the anterior limbs elongated and connected by a web, so that they can be used like wings in flying.  See <er>Chiroptera</er> and <er>Bat</er>.</def> <mark>[archaic]</mark> <br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cheiropterous</ent><br/
<hw>Chei*rop"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>chiropterous; belonging to the Chiroptera, or Bat family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheiropterygium</ent><br/
||<hw>Chei*rop`te*ryg"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cheiropterygia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>chei`r</grk> hand + <grk>pte`ryx</grk>, <grk>pte`rygos</grk> wing, fin.]</ety> <fld>(Anat.)</fld> <def>The typical pentadactyloid limb of the higher vertebrates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheirosophy</ent><br/
<hw>Chei*ros"o*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <?/ knowledge.]</ety> <def>The art of reading character as it is delineated in the hand.</def></p>

<p>-- <wordforms><wf>Chei*ros"o*phist</wf> <pr>(<?/)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheirotherium</ent><br/
||<hw>Chei`ro*the"ri*um</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[NL., fr. Gr. <grk>chei`r</grk> hand + <grk>qhri`on</grk> beast.]</ety> <fld>(Poleon.)</fld> <def>A genus of extinct animals, so named from fossil footprints rudely resembling impressions of the human hand, and believed to have been made by labyrinthodont reptiles. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chekelatoun</ent><br/
<hw>Chek`e*la*toun"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Ciclatoun</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chekmak</ent><br/
||<hw>Chek"mak</hw> <pr>(ch<ecr/k"m<acr/k)</pr>, <pos>n.</pos> <def>A turkish fabric of silk and cotton, with gold thread interwoven.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chela</ent><br/
||<hw>Che"la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chel<ae/</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>chhlh`</grk> claw.]</ety> <fld>(Zool.)</fld> <def>The pincherlike claw of Crustacea and Arachnida.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chela</ent><br/
||<hw>Che"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>ch<emac/la</ets>, orig., slave, fr. Skr. <ets>c<emac/<tsdot/a</ets>, <ets>c<emac/<tsdot/aka</ets>, slave, servant.]</ety> <def>In India, a dependent person occupying a position between that of a servant or slave and a disciple; hence, a disciple or novice.</def> -- <wordforms><wf>Che"la*ship</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chelate</ent><br/
<hw>Che"late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Same as <er>Cheliferous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chelate</ent><br/
<hw>che"late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>a compound formed by combination of a metal ion in solution with a compound containing heteroatoms (such as oxygen or nitrogen), in which at least two heteroatoms are bound to the metal.  The bonds may be covalent or non-covalent.  Chelates in some cases have very low dissociation constants, and organic compounds (<er>chelating agents</er>) which form such chelates are therefore used to reduce the concentration of free metal ions in solution.</def><br/
[<source>PJC</source>]</p>

<p><ent>chelated</ent><br/
<hw>chelated</hw> <pos>adj.</pos> <def>combined with a chelator so as to form a chelate; -- of metal ions.</def><br/
<syn><b>Syn. --</b> chelate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chelating agent</ent><br/
<hw>che"lat*ing a`gent</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>a compound which forms a chelate with a metal ion.</def> <note>Commonly used chelating agents in laboratory use are <stype>EDTA</stype> and <stype>sodium citrate</stype>.  Citrate is used as an additive to blood collected in blood banks, to chelate the calcium ions and retard clotting.</note><br/
[<source>PJC</source>]</p>

<p><ent>chelation</ent><br/
<hw>chelation</hw> <pos>n.</pos> <def>the process of removing a heavy metal from a solution by means of a chelate;</def> as <fld>(Med.)</fld> <def>the treatment of lead or mercury poisoning by adding a chelating agent to the blood.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>the process of forming a ring by forming one or more hydrogen bonds.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelerythrine</ent><br/
<hw>Chel`e*ryth"rine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ celandine + <grk>'eryqro`s</grk> red.]</ety> <fld>(Chem.)</fld> <def>An alkaloidal principle obtained from the celandine, and named from the red color of its salts. It is a colorless crystalline substance, and acts as an acrid narcotic poison. It is identical with <altname>sanguinarine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelicera</ent><br/
||<hw>Che*lic"e*ra</hw> <pr>(k<esl/*l<icr/s"<esl/*r<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chelicer<ae/</plw> <pr>(-r<emac/)</pr>.</plu> <ety>[NL., fr. Gr. <grk>chhlh`</grk> claw + <grk>ke`ras</grk> horn.]</ety> <fld>(Zool.)</fld> <def>One of the anterior pair of mouth organs, terminated by a pincherlike claw, in scorpions and allied Arachnida. They are homologous with the falcers of spiders, and probably with the mandibles of insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chelicerous</ent><br/
<hw>chelicerous</hw> <pos>adj.</pos> <def>having chelicerae.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelidon</ent><br/
<hw>Chel"i*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>chelidw`n</grk>.]</ety> <fld>(Anat.)</fld> <def>The hollow at the flexure of the arm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelidonic</ent><br/
<hw>Chel`i*don"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Celandine</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, the celandine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chelidonic acid</b></col>, <cd>a weak acid extracted from the celandine (<spn>Chelidonium majus</spn>), as a white crystalline substance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelidonium</ent><br/
<hw>Chelidonium</hw> <pos>n.</pos> <def>a genu having only one species; the greater celandine.</def><br/
<syn><b>Syn. --</b> genus <gen>Chelidonium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelidonius</ent><br/
||<hw>Chel`i*do"ni*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. (sc. <ets>lapillus</ets>.)]</ety> <def>A small stone taken from the gizzard of a young swallow. -- anciently worn as a medicinal charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelifer</ent><br/
<hw>Chel"i*fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chhlh`</grk> claw + <ets>-fer</ets>.]</ety> <fld>(Zool.)</fld> <def>See <cref>Book scorpion</cref>, under <er>Book</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheliferous</ent><br/
<hw>Che*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>chhlh`</grk> claw + <ets>-ferous</ets>.]</ety> <fld>(Zool.)</fld> <def>Having cheliform claws, like a crab.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheliform</ent><br/
<hw>Chel"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>chhlh`</grk> claw + <ets>-form</ets>.]</ety> <fld>(Zool.)</fld> <def>Having a movable joint or finger closing against a preceding joint or a projecting part of it, so that the whole may be used for grasping, as the claw of a crab; pincherlike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelone</ent><br/
||<hw>Che*lo"ne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chelw`nh</grk> a tortoise. So named from shape of the upper lip of the corolla.]</ety> <fld>(Bot.)</fld> <def>A genus of hardy perennial flowering plants, of the order <ord>Scrophulariace<ae/</ord>, natives of North America; -- called also <altname>snakehead</altname>, <altname>turtlehead</altname>, <altname>shellflower</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelonethida</ent><br/
<hw>Chelonethida</hw> <pos>n.</pos> <def>an order comprising the false scorpions.</def><br/
<syn><b>Syn. --</b> order <ord>Chelonethida</ord>, Pseudoscorpionida, order Pseudoscorpionida, Pseudoscorpiones, order Pseudoscorpiones.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelonia</ent><br/
||<hw>Che*lo"ni*a</hw> <pr>(k<esl/*l<omac/"n<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>chelw`nh</grk> a tortoise.]</ety> <fld>(Zool.)</fld> <def>An order of reptiles, including the tortoises and turtles, peculiar in having a part of the vertebr<ae/, ribs, and sternum united with the dermal plates so as to form a firm shell. The jaws are covered by a horny beak. See <er>Reptilia</er>; also, <xex>Illust.</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelonian</ent><br/
<hw>Che*lo"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to animals of the tortoise kind.</def> -- <def2><pos>n.</pos> <def>One of the Chelonia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheloniidae</ent><br/
<hw>Cheloniidae</hw> <pos>n.</pos> <def>a natural family including the green turtles; hawksbills.</def><br/
<syn><b>Syn. --</b> family <fam>Cheloniidae</fam>, Chelonidae, family <fam>Chelonidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelura</ent><br/
||<hw>Che*lu"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>chhlh`</grk> claw + <grk>o'yra`</grk> tail.]</ety> <fld>(Zool.)</fld> <def>A genus of marine amphipod crustacea, which bore into and sometimes destroy timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chely</ent><br/
<hw>Che"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A claw. See <er>Chela</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chelyabinsk</ent><br/
<hw>Chelyabinsk</hw> <pos>n.</pos> <def>a city in the Asian part of Russia.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelydra</ent><br/
<hw>Chelydra</hw> <pos>n.</pos> <def>a genus of snapping turtles.</def><br/
<syn><b>Syn. --</b> genus <gen>Chelydra</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chelydridae</ent><br/
<hw>Chelydridae</hw> <pos>n.</pos> <def>a natural family comprising the snapping turtles.</def><br/
<syn><b>Syn. --</b> family <fam>Chelydridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chemic</ent><br/
<hw>Chem"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chenistry</er>.]</ety> <sn>1.</sn> <def>A chemist; an alchemist.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bleaching)</fld> <def>A solution of chloride of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemic</ent><br/
<hw>Chem"ic</hw>, <pos>a.</pos> <def>Chemical.</def>  <rj><au>Blackw. Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemical</ent><br/
<hw>Chem"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to chemistry; characterized or produced by the forces and operations of chemistry; employed in the processes of chemistry; <as>as, <ex>chemical</ex> changes; <ex>chemical</ex> combinations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Chemical attraction</b></col> <it>or</it> <col><b>Chemical affinity</b></col></mcol>. <cd>See under <er>Attraction</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemical</ent><br/
<hw>Chem"ic*al</hw>, <pos>n.</pos> <def>A substance used for producing a chemical effect; a reagent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemically</ent><br/
<hw>Chem"ic*al*ly</hw>, <pos>adv.</pos> <def>According to chemical principles; by chemical process or operation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemiglyphic</ent><br/
<hw>Chem`i*glyph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chemical</ets> + <?/ to engrave.]</ety> <def>Engraved by a voltaic battery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemigraphy</ent><br/
<hw>Che*mig"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Chemical</ets> + <ets>-graphy</ets>.]</ety> <def>Any mechanical engraving process depending upon chemical action; specif., a process of zinc etching not employing photography.</def> -- <wordforms><wf>Chem`i*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chemiloon</ent><br/
<hw>Chem`i*loon"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A garment for women, consisting of chemise and drawers united in one.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemise</ent><br/
<hw>Che*mise"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., shirt, fr. LL. <ets>camisa</ets>, <ets>camisia</ets>, shirt, thin dress; cf. G. <ets>hemd</ets>, or OIr. <ets>caimmse</ets> sort of garment. Cf. <er>Camis</er>.]</ety> <sn>1.</sn> <def>A shift, or undergarment, worn by women.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wall that lines the face of a bank or earthwork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemisette</ent><br/
<hw>Chem`i*sette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>chemise</ets>.]</ety> <def>An under-garment, worn by women, usually covering the neck, shoulders, and breast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemism</ent><br/
<hw>Chem"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chimisme</ets>. See <er>Chemistry</er>.]</ety> <def>The force exerted between the atoms of elementary substance whereby they unite to form chemical compounds; chemical attaction; affinity; -- sometimes used as a general expression for chemical activity or relationship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chemosorptive</ent><br/
<ent>chemisorptive</ent><br/
<mhw><hw>chemisorptive</hw> <hw>chemosorptive</hw></mhw> <pos>adj.</pos> <def>have the capacity to adsorb by chemical as contrasted with physical forces.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chemist</ent><br/
<hw>Chem"ist</hw>, <pos>n.</pos> <ety>[Shortened from <ets>alchemist</ets>; cf. F. <ets>chimiste</ets>.]</ety> <def>A person versed in chemistry or given to chemical investigation; an analyst; a maker or seller of chemicals or drugs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemistry</ent><br/
<hw>Chem"is*try</hw> <pr>(k<ecr/m"<icr/s*tr<ycr/; 277)</pr>, <pos>n.</pos> <ety>[From <er>Chemist</er>. See <er>Alchemy</er>.]</ety> <sn>1.</sn> <def>That branch of science which treats of the composition of substances, and of the changes which they undergo in consequence of alterations in the constitution of the molecules, which depend upon variations of the number, kind, or mode of arrangement, of the constituent atoms. These atoms are not assumed to be indivisible, but merely the finest grade of subdivision hitherto attained. Chemistry deals with the changes in the composition and constitution of molecules. See <er>Atom</er>, <er>Molecule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Historically, chemistry is an outgrowth of alchemy (or alchemistry), with which it was anciently identified.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An application of chemical theory and method to the consideration of some particular subject; <as>as, the <ex>chemistry</ex> of iron; the <ex>chemistry</ex> of indigo</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A treatise on chemistry.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word and its derivatives were formerly written with <xex>y</xex>, and sometimes with i, instead of <xex>e</xex>, in the first syllable, <xex>chymistry</xex>, <xex>chymist</xex>, <xex>chymical</xex>, etc., or <xex>chimistry</xex>, <xex>chimist</xex>, <xex>chimical</xex>, etc.; and the pronunciation was conformed to the orthography.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Inorganic chemistry</b></col>, <cd>that which treats of inorganic or mineral substances.</cd> -- <col><b>Organic chemistry</b></col>, <cd>that which treats of the substances which form the structure of organized beings and their products, whether animal or vegetable; -- called also <altname>chemistry of the carbon compounds</altname>.  There is no fundamental difference between organic and inorganic chemistry.</cd> -- <col><b>Physiological chemistry</b></col>, <cd>the chemistry of the organs and tissues of the body, and of the various physiological processes incident to life.</cd> -- <mcol><col><b>Practical chemistry</b></col>, <it>or</it> <col><b>Applied chemistry</b></col></mcol>, <cd>that which treats of the modes of manufacturing the products of chemistry that are useful in the arts, of their applications to economical purposes, and of the conditions essential to their best use.</cd> -- <col><b>Pure chemistry</b></col>, <cd>the consideration of the facts and theories of chemistry in their purely scientific relations, without necessary reference to their practical applications or mere utility.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>chemist's</ent><br/
<hw>chemist's</hw> <pos>n.</pos> <def>a retail shop where medicine and other articles are sold; a <altname>drugstore</altname>.</def> <mark>[British]</mark> <br/
<syn><b>Syn. --</b> drugstore, apothecary's shop, chemist's shop, dispensary, pharmacy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chemitype</ent><br/
<hw>Chem"i*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chemi</ets>cal + <ets>-type</ets>.]</ety> <fld>(Engraving)</fld> <def>One of a number of processes by which an impression from an engraved plate is obtained in relief, to be used for printing on an ordinary printing press.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemolysis</ent><br/
<hw>Che*mol"y*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chemi</ets>cal + Gr. <?/ a loosing.]</ety> <def>A term sometimes applied to the decomposition of organic substance into more simple bodies, by the use of chemical agents alone.</def>  <rj><au>Thudichum.</au></rj><br/
[<source>1913 Webster</source>]</p>


<p><ent>Chemosis</ent><br/
||<hw>Che*mo"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a swelling of the cornea resembling a cockleshell, fr. <?/ a gaping, hence a cockleshell.]</ety> <fld>(Med.)</fld> <def>Inflammatory swelling of the conjunctival tissue surrounding the cornea.</def> -- <wordforms><wf>Che*mot"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chemosmosis</ent><br/
<hw>Chem`os*mo"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chem</ets>ical + <ets>osmosis</ets>.]</ety> <def>Chemical action taking place through an intervening membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chemosmotic</ent><br/
<hw>Chem`os*mot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or produced by, chemosmosis.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chemosorption</ent><br/
<hw>chemosorption</hw> <pos>n.</pos> <def>adsorption (especially when irreversible) by means of chemical instead of physical forces; <as>as, <ex>chemosorption</ex> of gaseous nitrogen on iron catalysts</as>.</def><br/
<syn><b>Syn. --</b> chemisorption.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chemosynthesis</ent><br/
<hw>Chem`o*syn"the*sis</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Chem</ets>ical + <ets>synthesis</ets>.]</ety> <fld>(Plant Physiol.)</fld> <def>Synthesis of organic compounds by energy derived from chemical changes or reactions. <xex>Chemosynthesis</xex> of carbohydrates occurs in the nitrite bacteria through the oxidation of ammonia to nitrous acid, and in the nitrate bacteria through the conversion of nitrous into nitric acid.</def> -- <wordforms><wf>Chem`o*syn*thet"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chemiotaxis</ent><br/
<ent>Chemotaxis</ent><br/
<mhw>{<hw>Chem`o*tax"is</hw> <pr>(?)</pr>, <pos>n.</pos> Formerly also <hw>Chem`i*o*tax"is</hw>}</mhw>. <ety>[<ets>Chem</ets>ical + Gr. <?/ arrangement, fr. <?/ to arrange.]</ety> <fld>(Biol.)</fld> <def>The sensitiveness exhibited by small free-swimming organisms, as bacteria, zoospores of alg<ae/, etc., to chemical substances held in solution. They may be attracted (<stype>positive chemotaxis</stype>) or repelled (<stype>negative chemotaxis</stype>).</def> -- <wordforms><wf>Chem`o*tac"tic</wf> <pr>(#)</pr>, <pos>a.</pos> -- <wf>Chem`o*tac"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>chemotherapeutical</ent><br/
<ent>chemotherapeutic</ent><br/
<mhw><hw>chemotherapeutic</hw> <hw>chemotherapeutical</hw></mhw> <pos>adj.</pos> <def>of or pertaining to chemotherapy; using chemotherapy; having properties beneficial for chemotherapy.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chemotherapy</ent><br/
<hw>chemotherapy</hw> <pos>n.</pos> <def>the use of chemical agents to treat or control disease (or mental illness); -- also used especially in reference to the use of chemicals to treat cancer.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Chemung period</ent><br/
<hw>Che*mung" pe"ri*od</hw> <pr>(?)</pr>, <fld>(Geol.)</fld> <def>A subdivision in the upper part of the Devonian system in America, so named from the Chemung River, along which the rocks are well developed. It includes the Portage and Chemung groups or epochs. See the Diagram under <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chen</ent><br/
<hw>Chen</hw> <pos>n.</pos> <def>a wild goose having white adult plumage.</def><br/
<syn><b>Syn. --</b> subgenus Chen.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chenfish</ent><br/
<hw>chenfish</hw> <pos>n.</pos> <def>small silvery marine food fish (<spn>Genyonemus lineatus</spn>) found off California.</def><br/
<syn><b>Syn. --</b> white croaker, kingfish, <spn>Genyonemus lineatus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cheng</ent><br/
||<hw>Cheng</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chinese.]</ety> <def>A chinese reed instrument, with tubes, blown by the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chenille</ent><br/
<hw>Che*nille"</hw> <pr>(sh<esl/*n<emac/l")</pr>, <pos>n.</pos> <ety>[F., prop., a caterpillar.]</ety> <def>Tufted cord, of silk or worsted, for the trimming of ladies' dresses, for embroidery and fringes, and for the weft of Chenille rugs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chenomorphae</ent><br/
||<hw>Che`no*mor"ph<ae/</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ the wild goose + <grk>morfh`</grk> form.]</ety> <fld>(Zool.)</fld> <def>An order of birds, including the swans, ducks, geese, flamingoes and screamers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chenopodiales</ent><br/
<hw>Chenopodiales</hw> <pos>n.</pos> <def>an order which corresponds approximately to the older group Centrospermae.</def><br/
<syn><b>Syn. --</b> Caryophyllales, order <ord>Caryophyllales</ord>, order <ord>Chenopodiales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chenopodium</ent><br/
<hw>Chenopodium</hw> <pos>n.</pos> <def>a genus of plants comprising the goosefoots and including the pigweed.</def><br/
<syn><b>Syn. --</b> genus <gen>Chenopodium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cheoplasty</ent><br/
<hw>cheoplasty</hw> <pos>n.</pos> <def>molding artificial teeth using low-fusing metals or alloys.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chepster</ent><br/
<hw>Chep"ster</hw> <pr>(ch<ecr/p"st<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European starling.</def> <mark>[Local, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheque</ent><br/
<hw>Cheque</hw> <pr>(ch<ecr/k)</pr>, <pos>n.</pos> <def>See <er>Check</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chequer</ent><br/
<hw>Cheq"uer</hw> <pr>(ch<ecr/k"<etil/r)</pr>, <pos>n. & v.</pos> <def>Same as <er>Checker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chequin</ent><br/
<hw>Che*quin"</hw> <pr>(ch<esl/*k<emac/n")</pr>, <pos>n.</pos> <def>A coin. See <er>Sequin</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chequy</ent><br/
<hw>Cheq"uy</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>Same as <er>Checky</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherif</ent><br/
<hw>Cher"if</hw> <pr>(sh<ecr/r"<icr/f)</pr>, <pos>n.</pos> <def>See <er>Sherif</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherimoyer</ent><br/
<hw>Cher`i*moy"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>ch<eacute/rimolier</ets>.]</ety> <fld>(Bot.)</fld> <sn>1.</sn> <def>A small downy-leaved tree (<spn>Anona Cherimolia</spn>), with fragrant flowers. It is a native of Peru.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Its delicious fruit, which is succulent, dark purple, and similar to the custard apple of the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherish</ent><br/
<hw>Cher"ish</hw> <pr>(ch<ecr/r"<icr/sh)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cherished</conjf> <pr>(ch<ecr/r"<icr/sht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cherising</conjf>.]</vmorph> <ety>[F. <ets>ch<eacute/rir</ets>, fr. <ets>cher</ets> dear, fr. L. <ets>carus</ets>. See <er>Caress</er>, <er>Finish</er>.]</ety> <sn>1.</sn> <def>To treat with tenderness and affection; to nurture with care; to protect and aid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We were gentle among you, even as a nurse <qex>cherisheth</qex> her children.</q> <rj><qau>1 Thess. ii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hold dear; to embrace with interest; to indulge; to encourage; to foster; to promote; <as>as, to <ex>cherish</ex> religious principle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>cherish</qex> virtue and humanity.</q> <rj><qau>Burke.</qau></rj></p>

<p><syn><b>Syn.</b> -- To nourish; foster; nurse; nurture; entertain; encourage; comfort; protect; support; See <er>Nurture</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>cherished</ent><br/
<hw>cherished</hw> <pos>adj.</pos> <def>deeply loved or valued.</def><br/
<syn><b>Syn. --</b> precious, treasured, wanted.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cherisher</ent><br/
<hw>Cher"ish*er</hw> <pr>(ch<ecr/r"<icr/sh*<etil/r)</pr>, <pos>n.</pos> <def>One who cherishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cherisher</qex> of my flesh and blood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherishment</ent><br/
<hw>Cher"ish*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Encouragement; comfort.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Rich bounty and dear <qex>cherishment</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chermes</ent><br/
<hw>Cher"mes</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Kermes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherogril</ent><br/
<hw>Cher"o*gril</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>choerogryllus</ets>, Gr. <?/; <?/ a young swine + <?/ a pig.]</ety> <fld>(Zool.)</fld> <def>See <er>Cony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherokees</ent><br/
<hw>Cher`o*kees"</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Cherokee</singw></sing>. <fld>(Ethnol.)</fld> <def>An Appalachian tribe of Indians, formerly inhabiting the region about the head waters of the Tennessee River. They are now mostly settled in the Indian Territory, and have become one of the most civilized of the Indian Tribes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheroot</ent><br/
<hw>Che*root"</hw> <pr>(ch<esl/*r<oomac/t"; 277)</pr>, <pos>n.</pos> <ety>[Tamil <ets>shuru<tsdot/<tsdot/u</ets>, prop., a roll.]</ety> <def>A kind of cigar, originally brought from <city>Manila</city>, in the Philippine Islands; now often made of inferior or adulterated tobacco.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherry</ent><br/
<hw>Cher"ry</hw> <pr>(ch<ecr/r"r<ycr/)</pr>, <pos>n.</pos> <ety>[OE. <ets>chery</ets>, for <ets>cherys</ets>, fr. F. <ets>cerise</ets> (cf. AS. <ets>cyrs</ets> cherry), fr. LL. <ets>ceresia</ets>, fr. L. <ets>cerasus</ets> Cherry tree, Gr. <grk>keraso`s</grk>, perh. fr. <grk>ke`ras</grk> horn, from the hardness of the wood.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A tree or shrub of the genus <gen>Prunus</gen> (Which also includes the plum) bearing a fleshy drupe with a bony stone;</def> <sd>(a)</sd> <def>The common garden cherry (<spn>Prunus Cerasus</spn>), of which several hundred varieties are cultivated for the fruit, some of which are, the begarreau, blackheart, black Tartarian, oxheart, morelle or morello, May-duke (corrupted from M<eacute/doc in France).</def> <sd>(b)</sd> <def>The wild cherry; as, <spn>Prunus serotina</spn> (wild black cherry), valued for its timber; <spn>Prunus Virginiana</spn> (choke cherry), an American shrub which bears astringent fruit; <spn>Prunus avium</spn> and <spn>Prunus Padus</spn>, European trees (bird cherry).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fruit of the cherry tree, a drupe of various colors and flavors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The timber of the cherry tree, esp. of the black cherry, used in cabinetmaking, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A peculiar shade of red, like that of a cherry.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Barbadoes cherry</b></col>. <cd>See under <er>Barbadoes</er>.</cd> -- <col><b>Cherry bird</b></col> <fld>(Zool.)</fld>, <cd>an American bird; the cedar bird; -- so called from its fondness for cherries.</cd> -- <col><b>Cherry bounce</b></col>, <cd>cherry brandy and sugar.</cd> -- <col><b>Cherry brandy</b></col>, <cd>brandy in which cherries have been steeped.</cd> -- <col><b>Cherry laurel</b></col> <fld>(Bot.)</fld>, <cd>an evergreen shrub (<spn>Prunus Lauro-cerasus</spn>) common in shrubberies, the poisonous leaves of which have a flavor like that of bitter almonds.</cd> -- <col><b>Cherry pepper</b></col> <fld>(Bot.)</fld>, <cd>a species of <gen>Capsicum</gen> (<spn>Capsicum cerasiforme</spn>), with small, scarlet, intensely piquant cherry-shaped fruit.</cd> -- <col><b>Cherry pit</b></col>. <sd>(a)</sd> <cd>A child's play, in which cherries are thrown into a hole.</cd> <au>Shak.</au> <sd>(b)</sd> <cd>A cherry stone.</cd> -- <col><b>Cherry rum</b></col>, <cd>rum in which cherries have been steeped.</cd> -- <col><b>Cherry sucker</b></col> <fld>(Zool.)</fld>, <cd>the European spotted flycatcher (<spn>Musicapa grisola</spn>); -- called also <altname>cherry chopper</altname> <altname>cherry snipe</altname></cd>. -- <col><b>Cherry tree</b></col>, <cd>a tree that bears cherries.</cd> -- <mcol><col><b>Ground cherry</b></col>, <col><b>Winter cherry</b></col></mcol>, <cd>See <er>Alkekengi</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 246 --></p>

<p><ent>Cherry</ent><br/
<hw>Cher"ry</hw> <pr>(ch<ecr/r"r<ycr/)</pr>, <pos>a.</pos> <def>Like a red cherry in color; ruddy; blooming; <as>as, a <ex>cherry</ex> lip; <ex>cherry</ex> cheeks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cherrystone</ent><br/
<hw>cherrystone</hw> <pos>n.</pos> <sn>1.</sn> <def>small quahog larger than a littleneck.</def><br/
<syn><b>Syn. --</b> cherrystone clam.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a half-grown quahog.</def><br/
<syn><b>Syn. --</b> cherrystone clam.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chersonese</ent><br/
<hw>Cher"so*nese</hw> <pr>(k<etil/r"s<osl/*n<emac/s)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cherso`nhsos</grk>; <grk>che`rsos</grk> land + <grk>nh`sos</grk> island.]</ety> <def>A peninsula; a tract of land nearly surrounded by water, but united to a larger tract by a neck of land or isthmus; <as>as, the <ex>Cimbric Chersonese</ex>, or Jutland; the <ex>Tauric Chersonese</ex>, or Crimea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chert</ent><br/
<hw>Chert</hw> <pr>(ch<etil/rt)</pr>, <pos>n.</pos> <ety>[Ir. <ets>ceart</ets> stone, perh. akin to E. <ets>crag</ets>.]</ety> <fld>(Min.)</fld> <def>An impure, massive, flintlike quartz or hornstone, of a dull color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherty</ent><br/
<hw>Chert"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like chert; containing chert; flinty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherub</ent><br/
<hw>Cher"ub</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cherubs</plw> <pr>(#)</pr>; but the Hebrew plural <plw>Cherubim</plw> <pr>(#)</pr> is also used.</plu> <ety>[Heb. <ets>ker<umac/b</ets>.]</ety> <sn>1.</sn> <def>A mysterious composite being, the winged footstool and chariot of the Almighty, described in <au>Ezekiel i. and x.</au></def><br/
[<source>1913 Webster</source>]</p>

<p><q>I knew that they were the <qex>cherubim</qex>.</q> <rj><qau>Ezek. x. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He rode upon a <qex>cherub</qex> and did fly.</q> <rj><qau>Ps. xviii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbolical winged figure of unknown form used in connection with the mercy seat of the Jewish Ark and Temple.</def>  <rj><au>Ez. xxv. 18.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of a order of angels, variously represented in art. In European painting the cherubim have been shown as blue, to denote knowledge, as distinguished from the seraphim (see <er>Seraph</er>), and in later art the children's heads with wings are generally called <xex>cherubs</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A beautiful child; -- so called because artists have represented cherubs as beautiful children.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherubical</ent><br/
<ent>Cherubic</ent><br/
<mhw>{ <hw>Che*ru"bic</hw> <pr>(?)</pr>, <hw>Che*ru"bic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to cherubs; angelic.</def> <ldquo/The <xex>cherubic</xex> host.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherubim</ent><br/
<hw>Cher"u*bim</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The Hebrew plural of <er>Cherub</er>.. Cf. <er>Seraphim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cherubims</xex>, in the King James version of the bible, is an incorrect form, made by adding the English plural termination to the Hebrew plural <xex>cherubim</xex> instead of to the singular <xex>cherub</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherubin</ent><br/
<hw>Cher"u*bin</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cherubic; angelic.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherubin</ent><br/
<hw>Cher"u*bin</hw>, <pos>n.</pos> <def>A cherub.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherup</ent><br/
<hw>Cher"up</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Prob. fr. <ets>chirp</ets>.]</ety> <def>To make a short, shrill, cheerful sound; to chirp. See <er>Chirrup</er>.</def> <ldquo/Cheruping birds.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherup</ent><br/
<hw>Cher"up</hw>, <pos>v. t.</pos> <def>To excite or urge on by making a short, shrill, cheerful sound; to cherup to. See <er>Chirrup</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>cherups</qex> brisk ear-erecting steed.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cherup</ent><br/
<hw>Cher"up</hw>, <pos>n.</pos> <def>A short, sharp, cheerful noise; a chirp; a chirrup; <as>as, the <ex>cherup</ex> of a cricket</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chervil</ent><br/
<hw>Cher"vil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cerfille</ets>, fr. L. <ets>caerefolium</ets>, <ets>chaerephyllum</ets>, Gr. <?/; <?/ to rejoice + <?/ leaf.]</ety> <fld>(Bot.)</fld> <def>A plant (<spn>Anthriscus cerefolium</spn>) with pinnately divided aromatic leaves, of which several curled varieties are used in soups and salads.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ches</ent><br/
<hw>Ches</hw> <pr>(?)</pr>, <def>pret. of <er>Chese</er>.</def> <mark>[Obs.]</mark> <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chese</ent><br/
<hw>Chese</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To choose</def> <mark>[Obs.]</mark> <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chesible</ent><br/
<hw>Ches"i*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chasuble</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheslip</ent><br/
<hw>Ches"lip</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The wood louse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chess</ent><br/
<hw>Chess</hw> <pr>(ch<ecr/s)</pr>, <pos>n.</pos> <ety>[OE. <ets>ches</ets>, F. <ets><eacute/checs</ets>, prop. pl. of <ets><eacute/chec</ets> check. See 1st <er>Check</er>.]</ety> <def>A game played on a chessboard, by two persons, with two differently colored sets of men, sixteen in each set. Each player has a king, a queen, two bishops, two knights, two castles or rooks, and eight pawns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chess</ent><br/
<hw>Chess</hw>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A species of brome grass (<spn>Bromus secalinus</spn>) which is a troublesome weed in wheat fields, and is often erroneously regarded as degenerate or changed wheat; it bears a very slight resemblance to oats, and if reaped and ground up with wheat, so as to be used for food, is said to produce narcotic effects; -- called also <altname>cheat</altname> and <altname>Willard's bromus</altname>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Other species of brome grass are called <stype>upright chess</stype>, <stype>soft chess</stype>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chess-apple</ent><br/
<hw>Chess"-ap`ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The wild service of Europe (<spn>Purus torminalis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chessboard</ent><br/
<hw>Chess"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The board used in the game of chess, having eight rows of alternate light and dark squares, eight in each row. See <er>Checkerboard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The chessboard and the checkerboard are alike.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chessel</ent><br/
<hw>Ches"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The wooden mold in which cheese is pressed.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chesses</ent><br/
<hw>Chess"es</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Cf. F. <ets>chassis</ets> a framework of carpenty.]</ety> <fld>(Mil.)</fld> <def>The platforms, consisting of two or more planks doweled together, for the flooring of a temporary military bridge.</def>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/A singular, <xex>chess</xex>, is sometimes used. <ldquo/Each <xex>chess</xex> consists of three planks.<rdquo/ <rj><au>Farrow.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chessil</ent><br/
<hw>Ches"sil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>chesil</ets>, AS. <ets>ceosel</ets> gravel, sand.]</ety> <def>Gravel or pebbles.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chessman</ent><br/
<hw>Chess"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chessmen</plw> <pr>(#)</pr></plu>. <def>A piece used in the game of chess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chessom</ent><br/
<hw>Ches"som</hw> <pr>(#)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chisley</er>.]</ety> <def>Mellow earth; mold.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chesstree</ent><br/
<hw>Chess"tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chassis</ets> a framework of carpentry.]</ety> <fld>(Naut.)</fld> <def>A piece of oak bolted perpendicularly on the side of a vessel, to aid in drawing down and securing the clew of the mainsail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chessy copper</ent><br/
<hw>Ches`sy" cop"per</hw> <pr>(?)</pr>. <fld>(Min.)</fld> <def>The mineral azurite, found in fine crystallization at <etsep>Chessy</etsep>, near Lyons; called also <altname>chessylite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chest</ent><br/
<hw>Chest</hw> <pr>(ch<ecr/st)</pr>, <pos>n.</pos> <ety>[OE. <ets>chest</ets>, <ets>chist</ets>, AS. <ets>cest</ets>, <ets>cist</ets>, <ets>cyst</ets>, L. <ets>cista</ets>, fr. Gr. <grk>ki`sth</grk>. Cf. <er>Cist</er>, <er>Cistern</er>.]</ety> <sn>1.</sn> <def>A large box of wood, or other material, having, like a trunk, a lid, but no covering of skin, leather, or cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heaps of money crowded in the <qex>chest</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coffin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He is now dead and mailed in his <qex>cheste</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The part of the body inclosed by the ribs and breastbone; the thorax.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Com.)</fld> <def>A case in which certain goods, as tea, opium, etc., are transported; hence, the quantity which such a case contains.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mech.)</fld> <def>A tight receptacle or box, usually for holding gas, steam, liquids, etc.; <as>as, the steam <ex>chest</ex> of an engine; the wind <ex>chest</ex> of an organ</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bomb chest</b></col>, <cd>See under <er>Bomb</er>.</cd> -- <col><b>Chest of drawers</b></col>, <cd>a case or movable frame containing drawers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chest</ent><br/
<hw>Chest</hw> <pr>(ch<ecr/st)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chested</conjf>.]</vmorph> <sn>1.</sn> <def>To deposit in a chest; to hoard.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To place in a coffin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He dieth and is <qex>chested</qex>.</q> <rj><qau>Gen. 1. 26 (heading).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chest</ent><br/
<hw>Chest</hw> <pr>(ch<ecr/st)</pr>, <pos>n.</pos> <ety>[AS. <ets>ce<aacute/st</ets>.]</ety> <def>Strife; contention; controversy.</def> <mark>[Obs.]</mark>  <rj><au>P. Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chested</ent><br/
<hw>Chest"ed</hw>, <pos>a.</pos> <def>Having (such) a chest; -- in composition; <as>as, broad-<ex>chested</ex>; narrow-<ex>chested</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chesterfield</ent><br/
<hw>chesterfield</hw> <pos>n.</pos> <sn>1.</sn> <def>a davenport with upright armrests.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a fitted overcoat with a velvet collar.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chesterlite</ent><br/
<hw>Ches"ter*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>-lite</er>.]</ety> <def>A variety of feldspar found in crystals in the county of <etsep>Chester</etsep>, Pennsylvania.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chesteyn</ent><br/
<hw>Ches"teyn</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The chestnut tree.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wilwe, elm, plane, assch, box, <qex>chesteyn</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chest founder</ent><br/
<hw>Chest" foun`der</hw> <pr>(?)</pr>. <fld>(Far.)</fld> <def>A rheumatic affection of the muscles of the breast and fore legs of a horse, affecting motion and respiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chestnut</ent><br/
<hw>Chest"nut</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>chesten-nut</ets>; OE. <ets>chestein</ets>, <ets>chesten</ets>, <ets>chastein</ets>, chestnut, fr. AS. <ets>cisten</ets> in <ets>cistenbe<aacute/m</ets> chestnut tree, influenced by OF. <ets>chastaigne</ets>, F. <ets>ch<acir/taigne</ets>, both the AS. and the F. words coming from L. <ets>castanea</ets> a chestnut, Gr. <?/, fr. <?/ a city of Pontus, where chestnut trees grew in abundance, and whence they were introduced into Europe. Cf. <er>Castanets</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The edible nut of a forest tree (<spn>Castanea vesce</spn>) of Europe and America. Commonly two or more of the nuts grow in a prickly bur.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The tree itself, or its light, coarse-grained timber, used for ornamental work, furniture, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A bright brown color, like that of the nut.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The horse chestnut (often so used in England).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>One of the round, or oval, horny plates on the inner sides of the legs of the horse, and allied animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An old joke or story.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chestnut tree</b></col>, <cd>a tree that bears chestnuts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chestnut</ent><br/
<hw>Chest"nut</hw>, <pos>a.</pos> <def>Of or pertaining of a chestnut; of a reddish brown color; <as>as, <ex>chestnut</ex> curls</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chesty</ent><br/
<hw>chesty</hw> <pos>adj.</pos> <sn>1.</sn>  <def>marked by a large or well-developed chest.</def><br/
<syn><b>Syn. --</b> big-chested.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>having or showing feelings of unwarranted importance out of overbearing pride; <as>as, <ex>chesty</ex> as a peacock</as>.</def><br/
<syn><b>Syn. --</b> arrogant, self-important.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chetah</ent><br/
<hw>Che"tah</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Cheetah</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chetvert</ent><br/
<hw>Chet"vert</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>chetverte</ets> one fourth.]</ety> <def>A measure of grain equal to 0.7218 of an imperial quarter, or 5.95 Winchester bushels.</def> <mark>[Russia]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevachie</ent><br/
<hw>Chev"a*chie`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chivachie</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevage</ent><br/
<hw>Che"vage</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chiefage</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheval</ent><br/
||<hw>Che*val"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chevaux</plw> <pr>(#)</pr>.</plu> <ety>[F. See <er>Cavalcade</er>.]</ety> <def>A horse; hence, a support or frame.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cheval glass</b></col>, <cd>a mirror swinging in a frame, and large enough to reflect the full length figure.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheval-de-frise</ent><br/
||<hw>Che*val"-de-frise"</hw> <pr>(?)</pr>, <pos>n.</pos>; commonly used in the <plu><it>pl.</it> <plw>Chevaux-de-frise</plw></plu>. <ety>[F.; <ets>cheval</ets> horse + <etsep>Frise</etsep> Friesland, where it was first used.]</ety> <fld>(Mil.)</fld> <def>A piece of timber or an iron barrel traversed with iron-pointed spikes or spears, five or six feet long, used to defend a passage, stop a breach, or impede the advance of cavalry, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Obstructions of chain, boom, and <qex>cheval-de-frise</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevalier</ent><br/
<hw>Che`va*lier"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>caballarius</ets>. See <er>Cavaller</er>.]</ety> <sn>1.</sn> <def>A horseman; a knight; a gallant young man.</def> <ldquo/Mount, <xex>chevaliers</xex>; to arms.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of certain orders of knighthood.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Chevalier d'industrie</b></col> <pr>(<?/)</pr> <ety>[F.]</ety>, <cd>one who lives by persevering fraud; a pickpocket; a sharper.</cd> -- <col><b>The Chevalier St. George</b></col> <fld>(Eng. Hist.)</fld>, <cd>James Francis Edward Stuart (son of James II.), called <ldquo/The Pretender.<rdquo/</cd> -- <col><b>The Young Chevalier</b></col>, <cd>Charles Edward Stuart, son of the Chevalier St. George.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevaux</ent><br/
||<hw>Che*vaux"</hw> <pr>(sh<eit/*v<omac/")</pr>, <pos>n. pl.</pos> <def>See <er>Cheval</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheve</ent><br/
<hw>Cheve</hw> <pr>(ch<emac/v)</pr>, <pos>v. i.</pos> <ety>[OF. <ets>chevir</ets>. See <er>Chievance</er>.]</ety> <def>To come to an issue; to turn out; to succeed; <as>as, to <ex>cheve</ex> well in a enterprise</as>.</def> <mark>[Prov. or Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevelure</ent><br/
||<hw>Cheve*lure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., head of hair.]</ety> <def>A hairlike envelope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The nucleus and <qex>chevelure</qex> of nebulous star.</q> <rj><qau>Sir. W. Hershel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheven</ent><br/
<hw>Chev"en</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chevanne</ets>. <ets>Cf</ets>. <er>Chavender</er>.]</ety> <fld>(Zool.)</fld> <def>A river fish; the chub.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheventein</ent><br/
<hw>Chev"en*tein</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A variant of <er>Chieftain</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheveril</ent><br/
<hw>Chev"er*il</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chevrel</ets>, F. <ets>chevreau</ets>, kid, dim. of <ets>chevre</ets> goat, fr. L. <ets>capra</ets>. See <er>Caper</er>, <pos>v. i.</pos>]</ety> <def>Soft leather made of kid skin. Fig.: Used as a symbol of flexibility.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here's wit of <qex>cheveril</qex>, that stretches from an inch narrow to an ell broad.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheveril</ent><br/
<hw>Chev"er*il</hw>, <pos>a.</pos> <def>Made of cheveril; pliant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cheveril</qex> conscience and a searching wit.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheverliize</ent><br/
<hw>Chev"er*li*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make as pliable as kid leather.</def> <mark>[Obs.]</mark>  <rj><au>Br. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevet</ent><br/
||<hw>Che*vet"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., head of the bed, dim. fr. <ets>chef</ets> head. See <er>Chief</er>.]</ety> <fld>(Arch.)</fld> <def>The extreme end of the chancel or choir; properly the round or polygonal part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheviot</ent><br/
<hw>Chev"i*ot</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A valuable breed of mountain sheep in Scotland, which takes its name from the Cheviot hills.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A woolen fabric, for men's clothing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevisance</ent><br/
<hw>Chev"i*sance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of. <ets>chevisance</ets>, <ets>chevissance</ets>, fr. <ets>chevir</ets>come to an end, perform, fr. <ets>chef</ets> head, end, from L. <ets>caput</ets> head. See <er>Chieve</er>, <er>Chief</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Achievement; deed; performance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Fortune, the foe of famous <qex>chevisance</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bargain; profit; gain.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(O. Eng. Law)</fld> <sd>(a)</sd> <def>A making of contracts.</def> <sd>(b)</sd> <def>A bargain or contract; an agreement about a matter in dispute, such as a debt; a business compact.</def> <sd>(c)</sd> <def>An unlawful agreement or contract.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevrette</ent><br/
<hw>Chev*rette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>ch<eacute/vre</ets> goat, fr. L. <ets>capra</ets>. Cf. <er>Chevron</er>.]</ety> <fld>(Mil.)</fld> <def>A machine for raising guns or mortar into their carriages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevron</ent><br/
<hw>Chev"ron</hw> <pr>(sh<ecr/v"r<ucr/n)</pr>, <pos>n.</pos> <ety>[F., rafter, chevron, from <ets>ch<eacute/vre</ets> goat, OF. <ets>chevre</ets>, fr. L. <ets>capra</ets> she-goat. See <er>Cheveril</er>.]</ety> <sn>1.</sn> <fld>(Her.)</fld> <def>One of the nine honorable ordinaries, consisting of two broad bands of the width of the bar, issuing, respectively from the dexter and sinister bases of the field and conjoined at its center.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A distinguishing mark, above the elbow, on the sleeve of a non-commissioned officer's coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A zigzag molding, or group of moldings, common in Norman architecture.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chevron bones</b></col> <fld>(Anat.)</fld>, <cd>The V-shaped subvertebral arches which inclose the caudal blood vessels in some animals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevroned</ent><br/
<hw>Chev"roned</hw> <pr>(?)</pr>, <pos>p. a.</pos> <def>Having a chevron; decorated with an ornamental figure of a zigzag from.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[A garment] whose nether parts, with their bases, were of watchet cloth of silver, <qex>chevroned</qex> all over with lace.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevronel</ent><br/
<hw>Chev"ron*el</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>A bearing like a chevron, but of only half its width.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevronwise</ent><br/
<hw>Chev"ron*wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <fld>(Her.)</fld> <def>In the manner of a chevron; <as>as, the field may be divided <ex>chevronwise</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevrotain</ent><br/
<hw>Chev`ro*tain"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chevrotin</ets>, OF. <ets>chevrot</ets> little goat, roe, dim. of <ets>chevre</ets> goat. See <er>Chevron</er>.]</ety> <fld>(Zool.)</fld> <def>A small ruminant of the family <fam>Tragulid<ae/</fam> a allied to the musk deer. It inhabits Africa and the East Indies. See <er>Kanchil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevy</ent><br/
<hw>Chev"y</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Chivy</er>, <pos>v. t.</pos></def> <mark>[Slang, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>One poor fellow was <qex>chevied</qex> about among the casks in the storm for ten minutes.</q> <rj><qau>London Times.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chevy</ent><br/
<hw>Chev"y</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>chivy</asp>, and <asp>chivvy</asp>.]</altsp> <ety>[Prob. fr. the ballad of <ets>Chevy Chase</ets>; cf. Prov. E. <ets>chevychase</ets> a noise, confusion, pursuit.]</ety> <mark>[Eng.]</mark> <sn>1.</sn>  <def>A cry used in hunting.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A hunt; chase; pursuit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>The game of prisoners' base. See <er>Base</er>, <pos>n.</pos>, 24.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>chevvy</ent><br/
<hw>chevvy</hw> <pos>v.</pos> <def>to repeatedly cause annoyance or concern to; to chevy; -- same as <er>chivy</er>.</def><br/
<syn><b>Syn. --</b> harass, hassle, harry, chivy, chivvy, chevy, beset, plague, molest, provoke.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chew</ent><br/
<hw>chew</hw> <pr>(ch<udd/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chewed</conjf> <pr>(ch<udd/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chewing</conjf>.]</vmorph> <ety>[As <ets>ce<oacute/wan</ets>, akin to D. <ets>kauwen</ets>, G. <ets>kauen</ets>. Cf. <er>Chaw</er>, <er>Jaw</er>.]</ety> <sn>1.</sn> <def>To bite and grind with the teeth; to masticate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ruminate mentally; to meditate on.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>chews</qex> revenge, abjuring his offense.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To chew the cud</b></col>, <cd>to chew the food over again, as a cow; to ruminate; hence, to meditate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Every beast the parteth the hoof, and cleaveth the cleft into two claws, and <qex>cheweth the cud</qex> among the beasts, that ye shall eat.</q> <rj><qau>Deut. xxiv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chew</ent><br/
<hw>Chew</hw>, <pos>v. i.</pos> <def>To perform the action of biting and grinding with the teeth; to ruminate; to meditate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>old politicians <qex>chew</qex> wisdom past.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chew</ent><br/
<hw>Chew</hw>, <pos>n.</pos> <def>That which is chewed; that which is held in the mouth at once; a cud.</def> <mark>[Law]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chewable</ent><br/
<hw>chewable</hw> <pos>adj.</pos> <def>easy to cut or chew.</def><br/
<syn><b>Syn. --</b> cuttable.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chewer</ent><br/
<hw>Chew"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chews.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chewet</ent><br/
<hw>Chew"et</hw>, <pos>n.</pos> <def>A kind of meat pie.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chewing</ent><br/
<hw>chewing</hw> <pos>n.</pos> <def>biting your food in your mouth so it becomes soft enough to swallow.</def><br/
<syn><b>Syn. --</b> chew, mastication.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chewink</ent><br/
<hw>Che"wink</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An american bird (<spn>Pipilo erythrophthalmus</spn>) of the Finch family, so called from its note; -- called also <altname>rufous-sided towhee</altname>, <altname>towhee</altname>, <altname>towhee bunting</altname> and <altname>ground robin</altname>.</def> <altsp>[Also spelled <asp>cheewink</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cheyennes</ent><br/
<hw>Chey*ennes"</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>cheyenne</singw></sing>. <fld>(Ethnol.)</fld> <def>A warlike tribe of indians, related to the blackfeet, formerly inhabiting the region of Wyoming, but now mostly on reservations in the Indian Territory. They are noted for their horsemanship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chian</ent><br/
<hw>Chi"an</hw> <pr>(?)</pr> <pos>a.</pos> <ety>[L. <ets>chius</ets>, fr. <ets>Chios</ets> the island Chios, Gr. <?/.]</ety> <def>Of or pertaining to Chios, an island in the <AE/gean Sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chian earth</b></col>, <cd>a dense, compact kind of earth, from Chios, used anciently as an astringent and a cosmetic.</cd> -- <col><b>Chian turpentine</b></col>, <cd>a fragrant, almost transparent turpentine, obtained from the <spn>Pistacia Terebinthus</spn>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chianti</ent><br/
<hw>Chianti</hw> <pos>n.</pos> <def>a dry red Italian table wine from the Chianti region of Tuscany.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chiaroscurist</ent><br/
<hw>Chi*a`ros*cu"rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A painter who cares for and studies light and shade rather than color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiaro-oscuro</ent><br/
<ent>Chiaroscuro</ent><br/
<mhw>{ ||<hw>Chia`ro*scu"ro</hw> <pr>(?)</pr>, ||<hw>Chi*a"ro-os*cu"ro</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[It., clear dark.]</ety> <sd>(a)</sd> <def>The arrangement of light and dark parts in a work of art, such as a drawing or painting, whether in monochrome or in color.</def> <sd>(b)</sd> <def>The art or practice of so arranging the light and dark parts as to produce a harmonious effect. Cf. <er>Clair-obscur</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiasma</ent><br/
<ent>Chiasm</ent><br/
<mhw>{ <hw>Chi"asm</hw> <pr>(k<imac/"<acr/z'm)</pr>, ||<hw>Chi*as"ma</hw> <pr>(k<isl/*<acr/z"m<adot/)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>chiasma</ets>, fr. Gr. <grk>chi`asma</grk> two lines placed crosswise, fr. <?/ to mark with a <chi/.]</ety> <fld>(Anat.)</fld> <def>A commissure; an intersection or crossing of two tracts; especially, the optic commissure, or crucial union of the optic nerves.</def> -- <wordforms><wf>Chi*as"mal</wf> <pr>(<?/)</pr>, <pos>a.</pos></wordforms>.<br/
[<source>1913 Webster</source>]</p>

<p><ent>chiasmic</ent><br/
<ent>chiasmatic</ent><br/
<ent>chiasmal</ent><br/
<mhw><hw>chiasmal</hw> <hw>chiasmatic</hw> <hw>chiasmic</hw></mhw> <pos>adj.</pos> <def>of or pertaining to a chiasma.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chiasmus</ent><br/
||<hw>Chi*as"mus</hw> <pr>(k<isl/*<acr/z"m<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>chiasmo`s</grk> a placing crosswise, fr. <grk>chia`zein</grk>. See <er>Chiasm</er>.]</ety> <fld>(Rhet.)</fld> <def>An inversion of the order of words or phrases, when repeated or subsequently referred to in a sentence</def>; thus,<br/
[<source>1913 Webster</source>]</p>

<p><q>If e'er to bless thy sons<br/
My <qex>voice</qex> or <qex>hands</qex> deny,<br/
These <qex>hands</qex> let useful skill forsake,<br/
This <qex>voice</qex> in silence die.</q> <rj><qau>Dwight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 247 --></p>

<p><ent>Chiastolite</ent><br/
<hw>Chi*as"to*lite</hw> <pr>(k<isl/*<acr/s"t<osl/*l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chiasto`s</grk> marked with a <chi/ + <ets>-lite</ets>. See <er>Chiasm</er>. So called from the resemblance of the cross cuts of its crystals to the Greek letter <chi/.]</ety> <fld>(Min.)</fld> <def>A variety of andalusite; -- called also <altname>macle</altname>. The tessellated appearance of a cross section is due to the symmetrical arrangement of impurities in the crystal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chibbal</ent><br/
<hw>Chib"bal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Cibol</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chibouk</ent><br/
<ent>Chibouque</ent><br/
<mhw>{ ||<hw>Chi*bouque"</hw>, <hw>Chi*bouk"</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chibouque</ets>, fr.  Turk.]</ety> <def>A Turkish pipe, usually with a mouthpiece of amber, a stem, four or five feet long and not pliant, of some valuable wood, and a bowl of baked clay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chic</ent><br/
||<hw>Chic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. Cf. <er>Chic</er>, <pos>n.</pos>]</ety> <def>Original and in good taste or form; stylish; in current fashion, fashionable.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Chica</ent><br/
||<hw>Chi"ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>A red coloring matter. extracted from the <spn>Bignonia Chica</spn>, used by some tribes of South American Indians to stain the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fermented liquor or beer made in South American from a decoction of maize.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A popular Moorish, Spanish, and South American dance, said to be the original of the fandango, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicalote</ent><br/
||<hw>Chi`ca*lo"te</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., prob. of Mex. origin.]</ety> <fld>(Bot.)</fld> <def>A Mexican prickly poppy (<spn>Argemone platyceras</spn>), which has migrated into California.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chicane</ent><br/
<hw>Chi*cane"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prob. earlier meaning a <ets>dispute</ets>, orig. in the game of mall (F. <er>mail</er>), fr. LGr. <?/ the game of mall, fr Pers <ets>chaug<amac/n</ets> club or bat; or possibly ultimated fr. L. <ets>ciccus</ets> a trible.]</ety> <sn>1.</sn> <def>The use of artful subterfuge, designed to draw away attention from the merits of a case or question; -- specifically applied to legal proceedings; trickery; chicanery; caviling; sophistry.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To shuffle from them by <qex>chicane</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To cut short this <qex>chicane</qex>, I propound it fairly to your own conscience.</q> <rj><qau>Berkeley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Card playing)</fld> <def>In bridge, the holding of a hand without trumps, or the hand itself. It counts as simple honors.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chicane</ent><br/
<hw>Chi*cane"</hw>, <pos>v. i.</pos> <ety>[Cf. F. <ets>chicaner</ets>. See <er>Chicane</er>, <pos>n.</pos>]</ety> <def>To use shifts, cavils, or artifices.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicaner</ent><br/
<hw>Chi*can"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chicaneur</ets>.]</ety> <def>One who uses chicanery.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicanery</ent><br/
<hw>Chi*can"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chicanerie</ets>.]</ety> <def>Mean or unfair artifice to perplex a cause and obscure the truth; stratagem; sharp practice; sophistry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Irritated by perpetual <qex>chicanery</qex>.</q> <rj><qau>Hallam.</qau></rj></p>

<p><syn><b>Syn.</b> -- Trickery; sophistry; stratagem.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiccory</ent><br/
<hw>Chic"co*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chicory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chich</ent><br/
<hw>Chich</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chiches</plw> <pr>(<?/)</pr>.</plu> <ety>[F. <ets>chiche</ets>, <ets>pois chiche</ets>, a dwarf pea, from L. <ets>cicer</ets> the chick-pea.]</ety> <fld>(Bot.)</fld> <def>The chick-pea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicha</ent><br/
||<hw>Chi"cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>See <er>Chica</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chichevache</ent><br/
||<hw>Chiche"vache`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chiche</ets> lean + <ets>vache</ets> cow.]</ety> <def>A fabulous cow of enormous size, whose food was patient wives, and which was therefore in very lean condition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chichling vetch</ent><br/
<ent>Chichling</ent><br/
<mhw>{ <hw>Chich"ling</hw> <pr>(?)</pr>, <hw>Chich"ling vetch`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[<ets>Chich</ets> + <ets>-ling</ets>.]</ety> <fld>(Bot.)</fld> <def>A leguminous plant (<spn>Lathyrus sativus</spn>), with broad flattened seeds which are sometimes used for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chick</ent><br/
<hw>Chick</hw> <pr>(ch<icr/k)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>chykkyn</ets>, <ets>chyke</ets>, <ets>chicken</ets>.]</ety> <def>To sprout, as seed in the ground; to vegetate.</def> <rj><au>Chalmers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chick</ent><br/
<hw>Chick</hw>, <pos>n.</pos> <sn>1.</sn> <def>A chicken.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A child or young person; -- a term of endearment.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>a young woman; -- often considered offensive.</def> <mark>[slang]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Chickabiddy</ent><br/
<hw>Chick"a*bid`dy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chicken; a fowl; also, a trivial term of endearment for a child.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chickadee</ent><br/
<hw>Chick"a*dee`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small bird, the blackcap titmouse (<spn>Parus atricapillus</spn>), of North America; -- named from its note.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chickaree</ent><br/
<hw>Chick"a*ree`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The American <altname>red squirrel</altname> (<spn>Tamiasciurus Hudsonius</spn>, formerly <spn>Sciurus Hudsonius</spn>); -- so called from its cry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chickasaws</ent><br/
<hw>Chick"a*saws</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Chickasaw</singw></sing>. <fld>(Ethnol.)</fld> <def>A tribe of North American Indians (Southern Appalachian) allied to the Choctaws. They formerly occupied the northern part of Alabama and Mississippi, but now live in the Indian Territory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicken</ent><br/
<hw>Chick"en</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cicen</ets>, <ets>cyceun</ets>, dim. of <ets>coc</ets> cock; akin to LG. <ets>kiken</ets>, <ets>k<uum/ken</ets>, D. <ets>Kieken</ets>, <ets>kuiken</ets>, G. <ets>k<uum/chkein</ets>. See <er>Cock</er> the animal.]</ety> <sn>1.</sn> <def>A young bird or fowl, esp. a young barnyard fowl.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young person; a child; esp. a young woman; a maiden; same as <altname>spring chicken</altname>.</def> <ldquo/Stella is no <xex>chicken</xex>.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chicken cholera</b></col>, <cd>a contagious disease of fowls; -- so called because first studied during the prevalence of a cholera epidemic in France. It has no resemblance to true cholera.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicken-breasted</ent><br/
<hw>Chick"en-breast`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a narrow, projecting chest, caused by forward curvature of the vertebral column.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chickenfeed</ent><br/
<hw>chickenfeed</hw> <pos>n.</pos> <sn>1.</sn> <def>a trifling sum of money.</def><br/
<syn><b>Syn. --</b> small change.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chicken-hearted</ent><br/
<ent>chickenhearted</ent><br/
<mhw><hw>chickenhearted</hw>, <hw>Chick"en-heart`ed</hw></mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Timid; fearful; cowardly; easily frightened.</def>  <rj><au>Bunyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicken pox</ent><br/
<hw>Chick"en pox"</hw> <pr>(?)</pr>. <fld>(Med.)</fld> <def>A mild, eruptive disease, generally attacking children only; varicella.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chickenshit</ent><br/
<hw>chickenshit</hw> <pos>n.</pos> <def>a false statement that is considered to indicate timidity or fear.</def> <mark>[slang]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chickeree</ent><br/
<hw>chickeree</hw> <pos>n.</pos> <def>the far western U.S. counterpart of the red squirrel.</def> <-- ?? chickaree?? --><br/
<syn><b>Syn. --</b> Douglas squirrel, Tamiasciurus douglasi.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chick flick</ent><br/
<hw>chick flick</hw> <pos>n.</pos> <def>A sentimental <isa>motion picture</isa> that appeals particularly to women.   See <er>flick</er>, <pos>n.</pos>, movie.</def> <mark>[Colloq.]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Chickling</ent><br/
<hw>Chick"ling</hw> <pr>(ch<icr/k"l<icr/ng)</pr>, <pos>n.</pos> <ety>[<ets>Chick</ets> + <ets>-ling</ets>.]</ety> <def>A small chick or chicken.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicle gum</ent><br/
<ent>Chicle</ent><br/
<mhw>{<hw>Chic"le</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Chicle gum</hw>}</mhw>. <ety>[Amer. Sp. <ets>chicle</ets>.]</ety> <def>A gumlike substance obtained from the bully tree (<spn>Mimusops globosa</spn>) and sometimes also from the naseberry or sapodilla (<spn>Sapota zapotilla</spn>). It is more plastic than caoutchouc and more elastic than gutta-percha, as an adulterant of which it is used in England. It is used largely in the United States in making chewing gum.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chico</ent><br/
<hw>Chi"co</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Var. of <er>Chica</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The common greasewood of the western United States (<spn>Sarcobatus vermiculatus</spn>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>In the Philippines, the  sapodilla or its fruit; also, the marmalade tree or its fruit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chick-pea</ent><br/
<hw>Chick"-pea`</hw> <pr>(-p<emac/`)</pr>, <pos>n.</pos> <ety>[See <er>Chich</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A Small leguminous plant (<spn>Cicer arietinum</spn>) of Asia, Africa, and the south of Europe; the chich; the dwarf pea; the gram.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Its nutritious seed, used in cookery, and especially, when roasted (parched pulse), as food for travelers in the Eastern deserts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chickweed</ent><br/
<hw>Chick"weed`</hw> <pr>(-w<emac/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The name of several caryophyllaceous weeds, especially <spn>Stellaria media</spn>, the seeds and flower buds of which are a favorite food of small birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicky</ent><br/
<hw>Chick"y</hw> <pr>(ch<icr/k"<ycr/)</pr>, <pos>n.</pos> <def>A chicken; -- used as a diminutive or pet name, especially in calling fowls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chicory</ent><br/
<hw>Chic"o*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chicor<eacute/e</ets>, earlier also <ets>cichor<eacute/e</ets>, L. <ets>cichorium</ets>, fr. Gr. <ets><?/</ets>, <ets><?/</ets>, <ets>Cf</ets>. <er>Succory</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A branching perennial plant (<spn>Cichorium Intybus</spn>) with bright blue flowers, growing wild in Europe, Asia, and America; also cultivated for its roots and as a salad plant; succory; wild endive. See <er>Endive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The root, which is roasted for mixing with coffee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chicot</ent><br/
<hw>chicot</hw> <pos>n.</pos> <def>a handsome tree of central and eastern North America (<spn>Gymnocladus dioica</spn>) having large bipinnate leaves and green-white flowers followed by large woody brown pods whose seeds are used as a coffee substitute.</def><br/
<syn><b>Syn. --</b> Kentucky coffee tree, bonduc, <spn>Gymnocladus dioica</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chide</ent><br/
<hw>Chide</hw> <pr>(ch<imac/d)</pr>, <pos>v. t.</pos> <ety>[<pos>imp.</pos> <er>Chid</er> <pr>(ch<icr/d)</pr>, or <er>Chode</er> <pr>(ch<imac/d Obs.)</pr>; <pos>p. p.</pos> <er>Chidden</er> <pr>(?)</pr>, <er>Chid</er>; <pos>p. pr. & vb. n.</pos> <er>Chiding</er>.]</ety> <ety>[AS. <ets>c<imac/dan</ets>; of unknown origin.]</ety> <sn>1.</sn> <def>To rebuke; to reprove; to scold; to find fault with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upbraided, <qex>chid</qex>, and rated at.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To be noisy about; to chafe against.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sea that <qex>chides</qex> the banks of England.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To chide hither</b></col>, <col><b>To chide from</b></col>, <it>or</it> <col><b>To chide away</b></col></mcol>, <cd>to cause to come, or to drive away, by scolding or reproof.</cd></cs></p>

<p><syn><b>Syn.</b> -- To blame; rebuke; reprove; scold; censure; reproach; reprehend; reprimand.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chide</ent><br/
<hw>Chide</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To utter words of disapprobation and displeasure; to find fault; to contend angrily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherefore the people did <qex>chide</qex> with Moses.</q> <rj><qau>Ex. xvii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a clamorous noise; to chafe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As doth a rock againts the <qex>chiding</qex> flood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chide</ent><br/
<hw>Chide</hw>, <pos>n.</pos> <ety>[AS. <ets>c<imac/d</ets>]</ety> <def>A continuous noise or murmur.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chide</qex> of streams.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chider</ent><br/
<hw>Chid"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chides or quarrels.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chideress</ent><br/
<hw>Chid"er*ess</hw>, <pos>n.</pos> <def>She who chides.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chidester</ent><br/
<hw>Chide"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chide</ets> + <ets>-ster</ets>.]</ety> <def>A female scold.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chidingly</ent><br/
<hw>Chid"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a chiding or reproving manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chief</ent><br/
<hw>Chief</hw> <pr>(ch<emac/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>chief</ets>, <ets>chef</ets>, OF. chief, F. <ets>chef</ets>, fr. L. <ets>caput</ets> head, possibly akin to E. <ets>head</ets>. Cf. <er>Captain</er>, <er>Chapter</er>]</ety> <sn>1.</sn> <def>The head or leader of any body of men; a commander, as of an army; a head man, as of a tribe, clan, or family; a person in authority who directs the work of others; the principal actor or agent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The principal part; the most valuable portion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chief</qex> of the things which should be utterly destroyed.</q> <rj><qau>1 Sam. xv. 21</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Her.)</fld> <def>The upper third part of the field. It is supposed to be composed of the dexter, sinister, and middle chiefs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In chief</b></col>. <sd>(a)</sd> <cd>At the head; <as>as, a commander <ex>in chief</ex></as>.</cd> <sd>(b)</sd> <fld>(Eng. Law)</fld> <cd>From the king, or sovereign; <as>as, tenure <ex>in chief</ex>, tenure directly from the king</as>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Chieftain; captain; general; commander; leader; head; principal; sachem; sagamore; sheik.</syn> <usage> -- <er>Chief</er>, <er>chieftain</er>, <er>Commander</er>, <er>Leader</er>. These words fluctuate somewhat in their meaning according to circumstances, but agree in the general idea of rule and authority. The term <xex>chief</xex> is now more usually applied to one who is a head man, leader, or commander in civil or military affairs, or holds a hereditary or acquired rank in a tribe or clan; as, the <xex>chief</xex> of police; the <xex>chief</xex> of an Indian tribe. A <xex>chieftain</xex> is the chief of a clan or tribe , or a military leader. A <xex>commander</xex> directs the movements of or has control over a body of men, as a military or naval force. A <xex>leader</xex> is one whom men follow, as in a political party, a legislative body, a military or scientific expedition, etc., one who takes the command and gives direction in particular enterprises.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chief</ent><br/
<hw>Chief</hw>, <pos>a.</pos> <sn>1.</sn> <def>Highest in office or rank; principal; head.</def> <ldquo/<xex>Chief</xex> rulers.<rdquo/  <rj><au>John. xii. 42.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Principal or most eminent in any quality or action; most distinguished; having most influence; taking the lead; most important; <as>as, the <ex>chief</ex> topic of conversation; the <ex>chief</ex> interest of man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Very intimate, near, or close.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A whisperer separateth <qex>chief</qex> friends.</q> <rj><qau>Prov. xvi. 28.</qau></rj></p>

<p><syn><b>Syn.</b> -- Principal; head; leading; main; paramount; supreme; prime; vital; especial; great; grand; eminent; master.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiefage</ent><br/
<hw>Chief"age</hw> <pr>(-<asl/j)</pr>, <pos>n.</pos> <ety>[OF. <ets>chevage</ets>, fr. <ets>chief</ets> head. See <er>Chief</er>.]</ety> <def>A tribute by the head; a capitation tax.</def> <altsp>[Written also <asp>chevage</asp> and <asp>chivage</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chief baron</ent><br/
<hw>Chief" bar"on</hw> <pr>(?)</pr>. <fld>(Eng. Law)</fld> <def>The presiding judge of the court of exchequer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiefest</ent><br/
<hw>Chief"est</hw>, <pos>a.</pos> <ety>[Superl. of <er>Chief</er>.]</ety> <def>First or foremost; chief; principal.</def> <mark>[Archaic]</mark> <ldquo/Our <xex>chiefest</xex> courtier.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chiefest</qex> among ten thousand.</q> <rj><qau>Canticles v. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chief hare</ent><br/
<hw>Chief" hare`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A small rodent (<spn>Lagamys princeps</spn>) inhabiting the summits of the Rocky Mountains; -- also called <altname>crying hare</altname>, <altname>calling hare</altname>, <altname>cony</altname>, <altname>American pika</altname>, and <altname>little chief hare</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is not a true hare or rabbit, but belongs to the curious family <fam>Lagomyid<ae/</fam>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chief justice</ent><br/
<hw>Chief" jus"tice</hw> <pr>(?)</pr>. <def>The presiding justice, or principal judge, of a court.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lord Chief Justice of England</b></col>, <cd>The presiding judge of the Queen's Bench Division of the High Court of Justice. The highest judicial officer of the realm is the Lord High Chancellor.</cd> -- <col><b>Chief Justice of the United States</b></col>, <cd>the presiding judge of the Supreme Court, and Highest judicial officer of the republic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chief-justiceship</ent><br/
<hw>Chief"-jus"tice*ship</hw>, <pos>n.</pos> <def>The office of chief justice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jay selected the <qex>chief-justiceship</qex> as most in accordance with his tastes.</q> <rj><qau>The Century.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiefless</ent><br/
<hw>Chief"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without a chief or leader.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiefly</ent><br/
<hw>Chief"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In the first place; principally; pre<eum/minently; above; especially.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Search through this garden; leave unsearched no nook;<br/
But <qex>chiefly</qex> where those two fair creatures lodge.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>For the most part; mostly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those parts of the kingdom where the . . . estates of the dissenters <qex>chiefly</qex> lay.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiefrie</ent><br/
<hw>Chief"rie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small rent paid to the lord paramount.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chieftain</ent><br/
<hw>Chief"tain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>cheftayn</ets>, <ets>chevetayn</ets>, OF. <ets>chevetain</ets>, F. <ets>capitaine</ets>, LL. <ets>capitanus</ets>, fr. L. <ets>caput</ets> head. Cf. <er>Captain</er>, and see <er>chief</er>.]</ety> <def>A captain, leader, or commander; a chief; the head of a troop, army, or clan.</def></p>

<p><syn><b>Syn.</b> -- Chief; commander; leader; head. See <er>Chief</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chieftainship</ent><br/
<ent>Chieftaincy</ent><br/
<mhw>{ <hw>Chief"tain*cy</hw> <pr>(?)</pr>, <hw>Chief"tain*ship</hw>, }</mhw> <pos>n.</pos> <def>The rank, dignity, or office of a chieftain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chierte</ent><br/
<hw>Chier"te</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chert<eacute/</ets>. See <er>Charity</er>.]</ety> <def>Love; tender regard.</def> <mark>[Obs.]</mark> <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chievance</ent><br/
<hw>Chiev"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chevance</ets> property, equiv. To <ets>chevisance</ets>, fr. <ets>chevir</ets> to accomplish. See <er>Chevisance</er>.]</ety> <def>An unlawful bargain; traffic in which money is exported as discount.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chieve</ent><br/
<hw>Chieve</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Cheve</er>, <pos>v. i.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiff-chaff</ent><br/
<hw>Chiff"-chaff</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[So called from its note.]</ety> <fld>(Zool.)</fld> <def>A species of European warbler (<spn>Sylvia hippolais</spn>); -- called also <altname>chip-chap</altname>, and <altname>pettychaps</altname>.</def></p>

<p><ent>Chiffon</ent><br/
||<hw>Chif`fon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., lit., rag. See <er>Chiffonier</er>.]</ety> <sn>1.</sn>  <def>Any merely ornamental adjunct of a woman's dress, as a bunch of ribbon, lace, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of soft gauzy material used for ruches, trimmings, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chiffonier</ent><br/
<mhw>{ <hw>Chif`fo*nier"</hw> <pr>(?)</pr>, <pos>fem.</pos> <hw>Chif`fo</hw>-<hw>ni<egrave/re"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>chiffonnier</ets>, fem. <ets>chiffonni<egrave/re</ets>, fr. <ets>chiffon</ets> rag, fr. <ets>chiffe</ets> a rag, flimsy cloth.]</ety> <sn>1.</sn> <def>One who gathers rags and odds and ends; a ragpicker.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A receptacle for rags or shreds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A movable and ornamental closet or piece of furniture with shelves or drawers.</def>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chigger</ent><br/
<hw>chigger</hw> <pos>n.</pos> <sn>1.</sn> <def>a small tropical flea (<spn>Tunga penetrans</spn>, formerly <spn>Pulex penetrans</spn>); same as <er>chigoe</er>.</def> <altsp>[Also spelled <asp>chigre</asp>.]</altsp><br/
<syn><b>Syn. --</b> chigoe, chigoe flea, <spn>Tunga penetrans</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a larval mite of the family <fam>Trombiculidae</fam> that sucks the blood of vertebrates including human beings, causing intense irritation, and serving to transmit several infectious diseases.</def><br/
<syn><b>Syn. --</b> harvest mite, jigger, redbug.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chiggerflower</ent><br/
<hw>chiggerflower</hw> <pos>n.</pos> <def>an erect perennial of eastern and southern U. S. (<spn>Asclepias tuberosa</spn>) having showy orange flowers.</def><br/
<syn><b>Syn. --</b> butterfly weed, orange milkweed, pleurisy root, tuber root, Indian paintbrush, <spn>Asclepias tuberosa</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chignon</ent><br/
||<hw>Chi"gnon</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[F., prop. equiv. to <ets>cha<icir/non</ets> link, fr. <ets>cha<icir/ne</ets> chain, fr. L. <ets>catena</ets> Cf. <er>Chain</er>.]</ety> <def>A knot, boss, or mass of hair, natural or artificial, worn by a woman at the back of the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A curl that had strayed from her <qex>chignon</qex>.</q> <rj><qau>H. James.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chigre</ent><br/
<ent>Chigoe</ent><br/
<mhw>{ <hw>Chig"oe</hw> <pr>(?)</pr>, <hw>Chig"re</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>chigue</ets>, perh. fr. Catalan <ets>chic</ets> small, Sp. <ets>chico</ets>; or of Peruvian origin.]</ety> <fld>(Zool.)</fld> <def>A species of flea (<spn>Tunga penetrans</spn>, formerly <spn>Pulex penetrans</spn>), common in the West Indies and South America, which often attacks the feet or any exposed part of the human body, and burrowing beneath the skin produces great irritation. When the female is allowed to remain and breed, troublesome sores result, which are sometimes dangerous. See <er>Jigger</er>.</def> <altsp>[Written also <asp>chegre</asp>, <asp>chegoe</asp>, <asp>chique</asp>, <asp>chigger</asp>, <asp>jigger</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is sometimes erroneously given to certain mites or ticks having similar habits.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chih fu</ent><br/
||<hw>Chih" fu`</hw> <pr>(?)</pr>. <ety>[Chin. <ets>chih fu</ets>, lit., (He who) knows (the) prefecture.]</ety> <def>An official administering a prefecture of China; a prefect, supervising the civil business of the hsiens or districts comprised in his fu (which see).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chih hsien</ent><br/
||<hw>Chih" hsien`</hw> <pr>(?)</pr>. <ety>[Chin. <ets>chih hsien</ets>, lit., (He who) knows (the) district.]</ety> <def>An official having charge of a hsien, or administrative district, in China; a district magistrate, responsible for good order in his hsien (which see), and having jurisdiction in its civil and criminal cases.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chih tai</ent><br/
||<hw>Chih" tai`</hw> <pr>(?)</pr>. <ety>[Chin. <ets>chih</ets> to govern + <ets>t<lsquo/ai</ets> an honorary title.]</ety> <def>A Chinese governor general; a tsung tu (which see).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chihuahua</ent><br/
<hw>Chihuahua</hw> <pos>n.</pos> <def>An old breed of tiny short-coated dogs with protruding eyes, originating in <country>Mexico</country>, believed to antedate Aztec civilization.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chikara</ent><br/
||<hw>Chi*ka"ra</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[Hind.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The goat antelope (<spn>Tragops Bennettii</spn>) of India.</def> <sd>(b)</sd> <def>The Indian four-horned antelope (<spn>Tetraceros quadricornis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chilblains</ent><br/
<ent>Chilblain</ent><br/
<mhw><hw>Chil"blain`</hw>, <hw>chil"blains</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chill</ets> + <ets>Blain</ets>.]</ety> <def>A blain, sore, or inflammatory swelling of the feet or hands, produced by exposure to cold, and attended by itching, pain, and sometimes ulceration.</def><br/
<syn><b>Syn. --</b> pernio.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Chilblain</ent><br/
<hw>Chil"blain`</hw>, <pos>v. t.</pos> <def>To produce chilblains upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Child</ent><br/
<hw>Child</hw> <pr>(ch<imac/ld)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Children</plw> <pr>(ch<icr/l"dr<ecr/n)</pr>.</plu> <ety>[AS. <ets>cild</ets>, pl. <ets>cildru</ets>; cf. Goth. <ets>kil<thorn/ei</ets> womb, <ets>in-kil<thorn/<omac/</ets> with child.]</ety> <sn>1.</sn> <def>A son or a daughter; a male or female descendant, in the first degree; the immediate progeny of human parents; -- in <xex>law</xex>, legitimate offspring. Used also of animals and plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A descendant, however remote; -- used esp. in the plural; <as>as, the <ex>children</ex> of Israel; the <ex>children</ex> of Edom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who, by character of practice, shows signs of relationship to, or of the influence of, another; one closely connected with a place, occupation, character, etc.; <as>as, a <ex>child</ex> of God; a <ex>child</ex> of the devil; a <ex>child</ex> of disobedience; a <ex>child</ex> of toil; a <ex>child</ex> of the people</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A noble youth. See <er>Childe</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A young person of either sex. esp. one between infancy and youth; hence, one who exhibits the characteristics of a very young person, as innocence, obedience, trustfulness, limited understanding, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When I was <qex>child</qex>. I spake as a <qex>child</qex>, I understood as a <qex>child</qex>, I thought as a <qex>child</qex>; but when I became a man, I put away childish things.</q> <rj><qau>1. Cor.  xii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A female infant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A boy or a <qex>child</qex>, I wonder?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be with child</b></col>, <cd>to be pregnant.</cd> -- <col><b>Child's play</b></col>, <cd>light work; a trifling contest.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Child</ent><br/
<hw>Child</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Childed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Childing</conjf>.]</vmorph> <def>To give birth; to produce young.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This queen Genissa <qex>childing</qex> died.</q> <rj><qau>Warner.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It chanced within two days they <qex>childed</qex> both.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childbearing</ent><br/
<hw>Child"bear`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of producing or bringing forth children; parturition.</def>  <rj><au>Milton. Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childbed</ent><br/
<hw>Child"bed</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of a woman bringing forth a child, or being in labor; parturition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childbirth</ent><br/
<hw>Child"birth</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of bringing forth a child; travail; labor.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childcrowing</ent><br/
<hw>Child"crow`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The crowing noise made by children affected with spasm of the laryngeal muscles; false croup.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childe</ent><br/
<hw>Childe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cognomen formerly prefixed to his name by the oldest son, until he succeeded to his ancestral titles, or was knighted; <as>as, <ex>Childe</ex> Roland</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childed</ent><br/
<hw>Child"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with a child.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childermas day</ent><br/
<hw>Chil"dermas day`</hw> <pr>(?)</pr>. <ety>[AS. <ets>cildam<ae/sse-d<ae/g</ets>; <ets>cild</ets> child  +<ets>d<ae/g</ets> day.]</ety> <fld>(Eccl.)</fld> <def>A day (December 28) observed by mass or festival in commemoration of the children slain by Herod at Bethlehem; -- called also <altname>Holy Innocent's Day</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 248 --></p>

<p><ent>Childhood</ent><br/
<hw>Child"hood</hw> <pr>(ch<imac/ld"h<oocr/d)</pr>, <pos>n.</pos> <ety>[AS. <ets>cildh<amac/d</ets>; <ets>cild</ets> child + <ets>-h<amac/d</ets>. See <er>Child</er>, and <er>-hood</er>.]</ety> <sn>1.</sn> <def>The state of being a child; the time in which persons are children; the condition or time from infancy to puberty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have walked before you from my <qex>childhood</qex>.</q> <rj><qau>1. Sam. xii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Children, taken collectively.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The well-governed <qex>childhood</qex> of this realm.</q> <rj><qau>Sir. W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The commencement; the first period.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>childhood</qex> of our joy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Second childhood</b></col>, <cd>the state of being feeble and incapable from old age.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childing</ent><br/
<hw>Child"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Child</er>, <pos>v. i.</pos>]</ety> <def>Bearing Children; <mark>(Fig.)</mark> productive; fruitful.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childish</ent><br/
<hw>Child"ish</hw> <pr>(ch<imac/ld"<icr/sh)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of, pertaining to, befitting, or resembling, a child.</def> <ldquo/<xex>Childish</xex> innocence.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Puerile; trifling; weak.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Methinks that simplicity in her countenance is rather <qex>childish</qex> than innocent.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Childish</xex>, as applied to persons who are grown up, is in a disparaging sense; as, a <ex>childish</ex> temper.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childishly</ent><br/
<hw>Child"ish*ly</hw>, <pos>adv.</pos> <def>In the manner of a child; in a trifling way; in a weak or foolish manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childishness</ent><br/
<hw>Child"ish*ness</hw>, <pos>n.</pos> <def>The state or quality of being childish; simplicity; harmlessness; weakness of intellect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childless</ent><br/
<hw>Child"less</hw>, <pos>a.</pos> <def>Destitute of chidren or offspring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childlessness</ent><br/
<hw>Child"less*ness</hw>, <pos>n.</pos> <def>The state of being childless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childlike</ent><br/
<hw>Child"like`</hw> <pr>(ch<imac/ld"l<imac/k`)</pr>, <pos>a.</pos> <def>Resembling a child, or that which belongs to children; becoming a child; meek; submissive; dutiful.</def> <ldquo/<xex>Childlike</xex> obedience.<rdquo/  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Childlike</xex>, as applied to persons grown up, is commonly in a good sense; as, <ex>childlike</ex> grace or simplicity; <ex>childlike</ex> modesty.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childly</ent><br/
<hw>Child"ly</hw>, <pos>a.</pos> <def>Having the character of a child; belonging, or appropriate, to a child.</def>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childly</ent><br/
<hw>Child"ly</hw>, <pos>adv.</pos> <def>Like a child.</def>  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childness</ent><br/
<hw>Child"ness</hw>, <pos>n.</pos> <def>The manner characteristic of a child.</def> <mark>[Obs.]</mark> <ldquo/Varying <xex>childness</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Children</ent><br/
<hw>Chil"dren</hw> <pr>(?)</pr>, <pos>n.</pos>; <def><pluf>pl.</pluf> of <er>Child</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Childship</ent><br/
<hw>Child"ship</hw>, <pos>n.</pos> <def>The state or relation of being a child.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Child study</ent><br/
<hw>Child study</hw>. <def>A scientific study of children, undertaken for the purpose of discovering the laws of development of the body and the mind from birth to manhood.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chilean</ent><br/
<hw>Chil"e*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>of, pertaining to, or characteristic of Chile or its inhabitants; <as>as, <ex>Chilean</ex> volcanoes</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chilean</ent><br/
<hw>Chil"e*an</hw>, <pos>n.</pos> <def>A native or resident of Chile; Chilian.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chilean pine</ent><br/
<hw>Chilean pine</hw>. <fld>(Bot.)</fld> <def>Same as <er>Monkey-puzzle</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chili</ent><br/
<hw>Chil"i</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>chili</ets>, <ets>chile</ets>.]</ety> <def>A kind of red pepper. See <er>Capsicum</er></def> <altsp>[Written also <asp>chilli</asp> and <asp>chile</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliad</ent><br/
<hw>Chil"i*ad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chilia`s</grk>, <grk>chiliado`s</grk>, fr. <grk>chi`lioi</grk> a thousand.]</ety> <def>A thousand; the aggregate of a thousand things; especially, a period of a thousand years.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world, then in the seventh <qex>chiliad</qex>, will be assumed up unto God.</q> <rj><qau>Sir. T. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliagon</ent><br/
<hw>Chil"i*a*gon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>chi`lioi</grk> a thousand + <?/ angle.]</ety> <def>A plane figure of a thousand angles and sides.</def>  <rj><au>Barlow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliahedron</ent><br/
<hw>Chil"i*a*hedron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chi`lioi</grk> a thousand + <?/ base, fr. <?/ to sit.]</ety> <def>A figure bounded by a thousand plane surfaces</def> <altsp>[Spelt also <asp>chilia<eum/dron</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilian</ent><br/
<hw>Chil"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Chili.</def> -- <def2><pos>n.</pos> <def>A native or citizen of Chili.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliarch</ent><br/
<ent>Chilian</ent><br/
<mhw>{ <hw>Chil"i*an</hw> <pr>(?)</pr>, <hw>Chil"i*arch`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <?/, <?/; <grk>chi`lioi</grk> a thousand + <?/ leader, <?/ to lead.]</ety> <def>The commander or chief of a thousand men.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliarchy</ent><br/
<hw>Chil"i*arch`y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/.]</ety> <def>A body consisting of a thousand men.</def>  <rj><au>Mitford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliasm</ent><br/
<hw>Chil"i*asm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/. See Chiliad.]</ety> <sn>1.</sn> <def>The millennium.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The doctrine of the personal reign of Christ on earth during the millennium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliast</ent><br/
<hw>Chil"i*ast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/. See <er>Chiliasm</er>.]</ety> <def>One who believes in the second coming of Christ to reign on earth a thousand years; a millenarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiliastic</ent><br/
<hw>Chili*astic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Millenarian.</def> <ldquo/The obstruction offered by the <xex>chiliastic</xex> errors.<rdquo/  <rj><au>J. A. Alexander.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chill</ent><br/
<hw>Chill</hw> <pr>(ch<icr/l)</pr>, <pos>n.</pos> <ety>[AS. <ets>cele</ets>, <ets>cyle</ets>, from the same root as <ets>celan</ets>, <ets>calan</ets>, to be cold; akin to D. <ets>kil</ets> cold, coldness, Sw. <ets>kyla</ets> to chill, and E. <ets>cool</ets>. See <er>Cold</er>, and cf. <er>Cool</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A moderate but disagreeable degree of cold; a disagreeable sensation of coolness, accompanied with shivering.</def> <ldquo/[A] wintry <xex>chill</xex>.<rdquo/  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A sensation of cold with convulsive shaking of the body, pinched face, pale skin, and blue lips, caused by undue cooling of the body or by nervous excitement, or forming the precursor of some constitutional disturbance, as of a fever.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A check to enthusiasm or warmth of feeling; discouragement; <as>as, a <ex>chill</ex> comes over an assembly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An iron mold or portion of a mold, serving to cool rapidly, and so to harden, the surface of molten iron brought in contact with it.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The hardened part of a casting, as the tread of a car wheel.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chill and fever</b></col>, <cd>fever and ague.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chill</ent><br/
<hw>Chill</hw>, <pos>a.</pos> <sn>1.</sn> <def>Moderately cold; tending to cause shivering; chilly; raw.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Noisome winds, and blasting vapors <qex>chill</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected by cold.</def> <ldquo/My veins are <xex>chill</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Characterized by coolness of manner, feeling, etc.; lacking enthusiasm or warmth; formal; distant; <as>as, a <ex>chill</ex> reception</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Discouraging; depressing; dispiriting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chill</ent><br/
<hw>Chill</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chilled</conjf> <pr>(ch<icr/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chilling</conjf>.]</vmorph> <sn>1.</sn> <def>To strike with a chill; to make chilly; to cause to shiver; to affect with cold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When winter <qex>chilled</qex> the day.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To check enthusiasm or warmth of feeling of; to depress; to discourage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every thought on God <qex>chills</qex> the gayety of his spirits.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <def>To produce, by sudden cooling, a change of crystallization at or near the surface of, so as to increase the hardness; said of cast iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chill</ent><br/
<hw>Chill</hw>, <pos>v. i.</pos> <fld>(Metal.)</fld> <def>To become surface-hardened by sudden cooling while solidifying; <as>as, some kinds of cast iron <ex>chill</ex> to a greater depth than others</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilled</ent><br/
<hw>Chilled</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Hardened on the surface or edge by chilling; <as>as, <ex>chilled</ex> iron; a <ex>chilled</ex> wheel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Paint.)</fld> <def>Having that cloudiness or dimness of surface that is called <ldquo/<xex>blooming</xex>.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilli</ent><br/
<hw>Chil"li</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chili</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilliness</ent><br/
<hw>Chill"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A state or sensation of being chilly; a disagreeable sensation of coldness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A moderate degree of coldness; disagreeable coldness or rawness; <as>as, the <ex>chilliness</ex> of the air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Formality; lack of warmth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilling</ent><br/
<hw>Chill"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Making chilly or cold; depressing; discouraging; cold; distant; <as>as, a <ex>chilling</ex> breeze; a <ex>chilling</ex> manner</as>.</def></p>

<p>-- <wordforms><wf>Chill"ing"ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chillness</ent><br/
<hw>Chill"ness</hw>, <pos>n.</pos> <def>Coolness; coldness; a chill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Death is the <qex>chillness</qex> that precedes the dawn.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilly</ent><br/
<hw>Chill"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moderately cold; cold and raw or damp so as to cause shivering; causing or feeling a disagreeable sensation of cold, or a shivering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilognath</ent><br/
<hw>Chi"log*nath</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A myriapod of the order <ord>Chilognatha</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilognatha</ent><br/
||<hw>Chi*log"na*tha</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ lip + <grk>gna`qos</grk> Jaw.]</ety> <fld>(Zool.)</fld> <def>One of the two principal orders of myriapods. They have numerous segments, each bearing two pairs of small, slender legs, which are attached ventrally, near together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiloma</ent><br/
<hw>Chi*lo"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ lip, fr. <?/ lip. See <er>-oma</er>.]</ety> <fld>(Zool.)</fld> <def>The tumid upper lip of certain mammals, as of a camel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilomycterus</ent><br/
<hw>Chilomycterus</hw> <pos>n.</pos> <def>a genus of fish, consisting of the burrfishes.</def><br/
<syn><b>Syn. --</b> genus <gen>Chilomycterus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chilopod</ent><br/
<hw>Chi"lo*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A myriapod of the order <ord>Chilopoda</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilopoda</ent><br/
||<hw>Chi*lop"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ lip + <ets>-poda</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the orders of myriapods, including the centipedes. They have a single pair of elongated legs attached laterally to each segment; well developed jaws; and a pair of thoracic legs converted into poison fangs. They are insectivorous, very active, and some species grow to the length of a foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilostomata</ent><br/
<ent>Chilostoma</ent><br/
<mhw>{ ||<hw>Chi*los"to*ma</hw> <pr>(?)</pr>, <hw>Chi*lo*stom"a*ta</hw> <pr>(?)</pr>, }</mhw> <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ + <?/, <?/, outh.]</ety> <fld>(Zool.)</fld> <def>An extensive suborder of marine Bryozoa, mostly with calcareous shells. They have a movable lip and a lid to close the aperture of the cells.</def> <altsp>[Also written <asp>Chillostomata</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chilostomatous</ent><br/
<hw>Chi`lo*stoma*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Chilostoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiltern Hundreds</ent><br/
<hw>Chiltern Hundreds</hw> <pr>(?)</pr>. <ety>[<ets>AS</ets>. <ets>Chiltern</ets> the Chiltern, high hills in Buckinghamshire, perh. Fr. <ets>ceald</ets> cold + <ets>ern</ets>, <ets><ae/rn</ets>, place.]</ety> <def>A tract of crown land in Buckinghamshire and Oxfordshire, England, to which is attached the nominal office of steward. As members of Parliament cannot resign, when they wish to go out they accept this stewardship, which legally vacates their seats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimaera</ent><br/
||<hw>Chi*m<ae/"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Chimera</er>.]</ety> <fld>(Zool.)</fld> <def>A cartilaginous fish of several species, belonging to the order <ord>Chimaeriformes</ord> of the class <class>Holocephali</class>.  The teeth are few and large. The head is furnished with appendages, and the tail terminates in a point.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>Note: more information and an image is available at:
<a href="http://www.fishbase.org/Summary/SpeciesSummary.cfm?genusname=Chimaera&speciesname=monstrosa">Fishbase - chimaera</a>.</note><br/
[<source>PJC</source>]</p>

<p><ent>Chimaeroid</ent><br/
<hw>Chi*m<ae/"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chim<ae/ra</ets> + <ets>old</ets>.]</ety> <fld>(Zool.)</fld> <def>Related to, or like, the chim<ae/ra.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimango</ent><br/
<hw>Chi*man"go</hw> <ety>[Native name]</ety> <fld>(Zool.)</fld> <def>A south American carrion buzzard (<spn>Milvago chimango</spn>). See <er>Caracara</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimb</ent><br/
<hw>Chimb</hw> <pr>(ch<imac/m)</pr>, <pos>n.</pos> <ety>[AS. <ets>cim</ets>, in <ets>cimst<amac/n</ets> base of a pillar; akin to D. <ets>kim</ets>, f. Sw. <ets>kim</ets>., G. <ets>kimme</ets> f.]</ety> <def>The edge of a cask, etc; a chine. See <er>Chine</er>, <pos>n.</pos>, 3.</def> <altsp>[Written also <asp>chime</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimb</ent><br/
<hw>Chimb</hw>, <pos>v. i.</pos> <def>Chime.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chime</ent><br/
<hw>Chime</hw> <pr>(ch<imac/m)</pr>, <pos>n.</pos> <ety>[See <er>Chimb</er>.]</ety> <def>See <er>Chine</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chime</ent><br/
<hw>Chime</hw> <pr>(ch<imac/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>chimbe</ets>, prop., cymbal, OF. <ets>cymbe</ets>, <ets>cymble</ets>, in a dialectic form, <ets>chymble</ets>, F. <ets>cymbale</ets>, L. <ets>cymbalum</ets>, fr. Gr. <grk>ky`mbalon</grk>. See <er>Cymbal</er>.]</ety> <sn>1.</sn> <def>The harmonious sound of bells, or of musical instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Instruments that made melodius <qex>chime</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A set of bells musically tuned to each other; specif., in the <pluf>pl.</pluf>, the music performed on such a set of bells by hand, or produced by mechanism to accompany the striking of the hours or their divisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have heard the <qex>chimes</qex> at midnight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pleasing correspondence of proportion, relation, or sound.</def> <ldquo/<xex>Chimes</xex> of verse.<rdquo/  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chime</ent><br/
<hw>Chime</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chimed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chiming</conjf>.]</vmorph> <ety>[See <er>Chime</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To sound in harmonious accord, as bells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be in harmony; to agree; to suit; to harmonize; to correspond; to fall in with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Everything <qex>chimed</qex> in with such a humor.</q> <rj><qau>W. irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To join in a conversation; to express assent; -- followed by <xex>in</xex> or <xex>in with</xex>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make a rude correspondence of sounds; to jingle, as in rhyming.</def>  <rj><au>Cowley</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chime</ent><br/
<hw>Chime</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To cause to sound in harmony; to play a tune, as upon a set of bells; to move or strike in harmony.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>chime</qex> their sounding hammers.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter harmoniously; to recite rhythmically.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chime</qex> his childish verse.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimer</ent><br/
<hw>Chim"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chimes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimera</ent><br/
<hw>Chime"ra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chimeras</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>chimaera</ets> a chimera (in sense 1), Gr. <?/ a she-goat, a chimera, fr. <?/ he-goat; cf. Icel. <ets>qymbr</ets> a yearling ewe.]</ety> <sn>1.</sn> <fld>(Myth.)</fld> <def>A monster represented as vomiting flames, and as having the head of a lion, the body of a goat, and the tail of a dragon.</def> <ldquo/Dire <xex>chimeras</xex> and enchanted isles.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vain, foolish, or incongruous fancy, or creature of the imagination; <as>as, the <ex>chimera</ex> of an author</as>.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimere</ent><br/
<hw>Chi*mere"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chamarre</ets>., F. <ets>simarre</ets> (cf. It. <ets>zimarra</ets>), fr. Sp. <ets>chamarra</ets>, <ets>zamarra</ets>, a coat made of sheepskins, a sheepskin, perh. from Ar. <ets>samm<umac/r</ets> the Scythian weasel or marten, the sable. Cf. <er>Simarre</er>.]</ety> <def>The upper robe worn by a bishop, to which lawn sleeves are usually attached.</def>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimeric</ent><br/
<hw>Chi*mer"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Chimerical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimerical</ent><br/
<hw>Chi*mer"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Merely imaginary; fanciful; fantastic; wildly or vainly conceived; having, or capable of having, no existence except in thought; <as>as, <ex>chimerical</ex> projects</as>.</def></p>

<p><syn><b>Syn.</b> -- Imaginary; fanciful; fantastic; wild; unfounded; vain; deceitful; delusive.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimerically</ent><br/
<hw>Chi*mer"ic*al*ly</hw>, <pos>adv.</pos> <def>Wildy; vainly; fancifully.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiminage</ent><br/
<hw>Chim"i*nage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cheminage</ets>, fr. <ets>chemin</ets> way, road.]</ety> <fld>(Old Law)</fld> <def>A toll for passage through a forest.</def> <mark>[Obs.]</mark>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimney</ent><br/
<hw>Chim"ney</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chimneys</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>chemin<eacute/e</ets>, LL. <ets>caminata</ets>, fr. L. <ets>caminus</ets> furnace, fireplace, Gr. <?/ furnace, oven.]</ety> <sn>1.</sn> <def>A fireplace or hearth.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That part of a building which contains the smoke flues; esp. an upright tube or flue of brick or stone, in most cases extending through or above the roof of the building. Often used instead of <xex>chimney shaft</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hard by a cottage <qex>chimney</qex> smokes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A tube usually of glass, placed around a flame, as of a lamp, to create a draft, and promote combustion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Min.)</fld> <def>A body of ore, usually of elongated form, extending downward in a vein.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chimney board</b></col>, <cd>a board or screen used to close a fireplace; a fireboard.</cd> -- <col><b>Chimney cap</b></col>, <cd>a device to improve the draught of a chimney, by presenting an exit aperture always to leeward.</cd> -- <col><b>Chimney corner</b></col>, <cd>the space between the sides of the fireplace and the fire; hence, the fireside.</cd> -- <col><b>Chimney hook</b></col>, <cd>a hook for holding pats and kettles over a fire,</cd> -- <col><b>Chimney money</b></col>, <cd>hearth money, a duty formerly paid in England for each chimney.</cd> -- <col><b>Chimney pot</b></col> <fld>(Arch.)</fld>, <cd>a cylinder of earthenware or sheet metal placed at the top of a chimney which rises above the roof.</cd> -- <col><b>Chimney swallow</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>An American swift (<spn>Ch<ae/ture pelasgica</spn>) which lives in chimneys.</cd> <sd>(b)</sd> <cd>In England, the common swallow (<spn>Hirundo rustica</spn>).</cd> -- <mcol><col><b>Chimney sweep</b></col>, <col><b>Chimney sweeper</b></col></mcol>, <cd>one who cleans chimneys of soot; esp. a boy who climbs the flue, and brushes off the soot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chimney-breast</ent><br/
<hw>Chim"ney-breast`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The horizontal projection of a chimney from the wall in which it is built; -- commonly applied to its projection in the inside of a building only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chimneypiece</ent><br/
<ent>Chimney-piece</ent><br/
<mhw><hw>Chim"ney-piece`</hw>, <hw>chimneypiece</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A decorative construction around the opening of a fireplace; also, the shelf that projects from wall above fireplace; <altname>mantlepiece</altname>.</def> <mark>[Chiefly Brit.]</mark> <br/
<syn><b>Syn. --</b> mantel, mantelpiece, mantle, mantlepiece.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>chimneypot</ent><br/
<hw>chimneypot</hw> <pos>n.</pos> <def>a short earthenware pipe on the top of a chimney to increase the draft.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chimneystack</ent><br/
<hw>chimneystack</hw> <pos>n.</pos> <def>the part of the chimney that is above the roof; it usually has several flues.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chimneysweeper</ent><br/
<ent>chimneysweep</ent><br/
<mhw><hw>chimneysweep</hw>, <hw>chimneysweeper</hw></mhw> <pos>n.</pos> <def>someone who cleans soot from chimneys, usually as a profession.</def><br/
<syn><b>Syn. --</b> chimneysweeper, sweep.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chimonanthus</ent><br/
<hw>Chimonanthus</hw> <pos>n.</pos> <def>a small genus of Asian deciduous or evergreen shrubs having fragrant flowers: winter sweet.</def><br/
<syn><b>Syn. --</b> genus <gen>Chimonanthus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chimp</ent><br/
<hw>chimp</hw> <pos>n.</pos> <def>same as <er>chimpanzee</er>.</def><br/
<syn><b>Syn. --</b> chimpanzee, Pan troglodytes.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chimpanzee</ent><br/
<hw>Chim*pan"zee</hw> <pr>(ch<icr/m*p<acr/n"z<esl/; 277)</pr>, <pos>n.</pos> <ety>[From the native name: cf. F. <ets>chimpanz<eacute/</ets>, <ets>chimpans<eacute/</ets>, <ets>chimpanz<eacute/e</ets>.]</ety> <fld>(Zool.)</fld> <def>An african ape (<spn>Pan troglodytes</spn>, formerly <spn>Anthropithecus troglodytes</spn>, or <spn>Troglodytes niger</spn>) which approaches more nearly to man, in most respects, than any other ape. It is the most intelligent of non-human animals, and when full grown, it is from three to four feet high.  A variant called the <er>pygmy chimpanzee</er>, or <er>bonobo</er>, has been recently recognized as a separate species.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Chin</ent><br/
<hw>Chin</hw> <pr>(ch<icr/n)</pr>, <pos>n.</pos> <ety>[AS. <ets>cin</ets>, akin to OS. <ets>kin</ets>, G <ets>kinn</ets>, Icel. <ets>kinn</ets>, cheek, Dan. & Sw. <ets>kind</ets>, L. <er>gena</er>, Gr. <?/; cf. Skr. <ets>hanu</ets>.  <root/232.]</ety> <sn>1.</sn> <def>The lower extremity of the face below the mouth; the point of the under jaw.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The exterior or under surface embraced between the branches of the lower jaw bone, in birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>China</ent><br/
<hw>Chi"na</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A country in Eastern Asia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>China ware, which is the modern popular term for porcelain. See <er>Porcelain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>China aster</b></col> <fld>(Bot.)</fld>, <cd>a well-known garden flower and plant. See <er>Aster</er>.</cd> -- <col><b>China bean</b></col>. <cd>See under <er>Bean</er>, 1.</cd> -- <col><b>China clay</b></col> <cd>See <er>Kaolin</er>.</cd> -- <col><b>China grass</b></col>, <cd>Same as <er>Ramie</er>.</cd> -- <col><b>China ink</b></col>. <cd>See <er>India ink</er>.</cd> -- <col><b>China pink</b></col> <fld>(Bot.)</fld>, <cd>an anual or biennial species of <gen>Dianthus</gen> (<spn>Dianthus Chiensis</spn>) having variously colored single or double flowers; Indian pink.</cd> -- <col><b>China root</b></col> <fld>(Med.)</fld>, <cd>the rootstock of a species of <gen>Smilax</gen> (<spn>Smilax China</spn>, from the East Indies; -- formerly much esteemed for the purposes that sarsaparilla is now used for. Also the galanga root (from <spn>Alpinia Gallanga</spn> and <spn>Alpinia officinarum</spn>).</cd> -- <col><b>China rose</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A popular name for several free-blooming varieties of rose derived from the <spn>Rosa Indica</spn>, and perhaps other species.</cd> <sd>(b)</sd> <cd>A flowering hothouse plant (<spn>Hibiscus Rosa-Sinensis</spn>) of the Mallow family, common in the gardens of China and the east Indies.</cd> -- <col><b>China shop</b></col>, <cd>a shop or store for the sale of China ware or of crockery.</cd> -- <mcol><col><b>Pride of China</b></col>, <col><b>China tree</b></col></mcol>. <fld>(Bot.)</fld> <cd>See <er>Azedarach</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>chinaberry</ent><br/
<hw>chinaberry</hw> <pos>n.</pos> <sn>1.</sn> <def>an evergreen of tropical America having pulpy fruit containing saponin which was used as soap by native Americans.</def><br/
<syn><b>Syn. --</b> China tree, false dogwood, jaboncillo, Sapindus saponaria.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a tree of N India and China having purple blossoms and small inedible yellow fruits; naturalized in the southern US as a shade tree.</def><br/
<syn><b>Syn. --</b> chinaberry tree, China tree, Persian lilac, pride-of-India, azederach, azedarach, Melia azederach, Melia azedarach.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chinaware</ent><br/
<ent>China ware</ent><br/
<mhw><hw>China ware</hw>, <hw>chinaware</hw></mhw> <pos>n.</pos>  <def>dishes made of china; porcelain; -- so called in the 17th century because brought from the far East, and differing from the pottery made in Europe at that time; also, loosely, crockery in general.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 249 --></p>

<p><ent>Chinaldine</ent><br/
<hw>Chin*al"dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>chinium</ets> quinine + <ets>ald</ets>ehyde.]</ety> <fld>(Chem.)</fld> <def>See <er>Quinaldine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinaman</ent><br/
<hw>Chi"na*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chinamen</plw> <pr>(<?/)</pr>.</plu> <def>A native of China; a Chinese.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chincapin</ent><br/
<hw>Chin"ca*pin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chinquapin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinch</ent><br/
<hw>Chinch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>chinche</ets>, fr. L. <er>cimex</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The bedbug (<spn>Cimex lectularius</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A bug (<spn>Blissus leucopterus</spn>), which, in the United States, is very destructive to grass, wheat, and other grains; -- also called <altname>chiniz</altname>, <altname>chinch bug</altname>, <altname>chink bug</altname>. It resembles the bedbug in its disgusting odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chincha</ent><br/
<hw>Chin"cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chinchilla</er>.]</ety> <fld>(Zool.)</fld> <def>A south American rodent of the genus <gen>Lagotis</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinch bug</ent><br/
<hw>Chinch bug</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>see <er>chinch{2}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chinche</ent><br/
<hw>Chinche</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>chiche</ets> miserly.]</ety> <def>Parsimonious; niggardly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chincherie</ent><br/
<hw>Chinch"er*ie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Penuriousness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By cause of his skarsete and <qex>chincherie</qex>.</q> <rj><qau>Caucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinchilla</ent><br/
<hw>Chin*chil"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A small rodent (<spn>Chinchilla lanigera</spn>), of the size of a large squirrel, remarkable for its fine fur, which is very soft and of a pearly gray color. It is a native of Peru and Chili.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fur of the chinchilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A heavy, long-napped, tufted woolen cloth.</def></p>

<p><ent>Chinchillidae</ent><br/
<hw>Chinchillidae</hw> <pos>n.</pos> <def>a natural family of small bushy-tailed South American burrowing rodents.</def><br/
<syn><b>Syn. --</b> family <fam>Chinchillidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chinchillon</ent><br/
<hw>chinchillon</hw> <pos>n.</pos> <def>a gregarious burrowing rodent (<spn>Lagostomus maximus</spn>) larger than the chinchillas.</def><br/
<syn><b>Syn. --</b> viscacha, <spn>Lagostomus maximus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chincona</ent><br/
<ent>Chinchona</ent><br/
<mhw>{ <hw>Chin*cho"na</hw> <pr>(?)</pr>, <hw>Chin*co"na</hw> <pr>(?)</pr>. }</mhw> <def>any of several trees of the genus <gen>Cinchona</gen>. Same as <er>Cinchona</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chinchy</ent><br/
<hw>chinch"y</hw> <pr>(ch<icr/nch"<ycr/)</pr>, <pos>adj.</pos> <amorph>[<pos>Compar.</pos> <adjf>chinchier</adjf> <pr>(ch<icr/n"ch<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>chinchiest</adjf> <pr>(ch<icr/n"ch<icr/*<ecr/st)</pr>.]</amorph> <ety>[See <er>chinche</er>.]</ety> <def>same as <er>stingy</er>.</def><br/
<syn><b>Syn. --</b> cheap, chintzy, stingy, chinche.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chin cough</ent><br/
<hw>Chin" cough"</hw> <pr>(?)</pr>. <ety>[For <ets>chink cough</ets>; cf. As. <ets>cincung</ets> long laughter, Scot. <ets>kink</ets> a violent fit of coughing, akin to MHG. <ets>k<imac/chen</ets> to pant. Cf. <er>Kinknaust</er>, <er>Cough</er>.]</ety> <def>Whooping cough.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chine</ent><br/
<hw>Chine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chink</er>.]</ety> <def>A chink or cleft; a narrow and deep ravine; <as>as, Shanklin <ex>Chine</ex> in the Isle of Wight, a quarter of a mile long and 230 feet deep</as>.</def> <mark>[Prov. Eng.]</mark> <ldquo/The cottage in a <xex>chine</xex>.<rdquo/  <rj><au>J. Ingelow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chine</ent><br/
<hw>Chine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>eschine</ets>, F. <ets><eacute/chine</ets>, fr. OHG. <ets>skina</ets> needle, prickle, shin, G. <ets>schiene</ets> splint, <ets>schienbein</ets> shin. For the meaning cf. L. <ets>spina</ets> thorn, prickle, or spine, the backbone. Cf. <er>Shin</er>.]</ety> <sn>1.</sn> <def>The backbone or spine of an animal; the back.</def> <ldquo/And <xex>chine</xex> with rising bristles roughly spread.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of the backbone of an animal, with the adjoining parts, cut for cooking. <note>[See <xex>Illust.</xex> of <er>Beef</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The edge or rim of a cask, etc., formed by the projecting ends of the staves; the chamfered end of a stave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chine</ent><br/
<hw>Chine</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chined</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To cut through the backbone of; to cut into chine pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Too chamfer the ends of a stave and form the chine..</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chined</ent><br/
<hw>Chined</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, or having, a chine, or backbone; -- used in composition.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Broken in the back.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He's <qex>chined</qex>, goodman.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinese</ent><br/
<hw>Chi"nese"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to China; peculiar to China.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chinese paper</b></col>. <cd>See <cref>India paper</cref>, under <er>India</er>.</cd> -- <col><b>Chinese wax</b></col>, <cd>a snowy-white, waxlike substance brought from China. It is the bleached secretion of certain insects of the family <fam>Coccid<ae/</fam> especially <spn>Coccus Sinensis</spn>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinese</ent><br/
<hw>Chi*nese"</hw>, <pos>n. sing. & pl.</pos> <sn>1.</sn> <def>A native or natives of China, or one of that yellow race with oblique eyelids who live principally in China.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <singf>sing.</singf> <def>The language of China, which is monosyllabic.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Chineses</xex> was used as a plural by the contemporaries of Shakespeare and Milton.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinese Exclusion Act</ent><br/
<hw>Chinese Exclusion Act</hw>. <def>Any of several acts forbidding the immigration of Chinese laborers into the United States, originally from 1882 to 1892 by act of May 6, 1882, then from 1892 to 1902 by act May 5, 1892. By act of April 29, 1902, all existing legislation on the subject was re<eum/nacted and continued, and made applicable to the insular possessions of the United States.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chinese-red</ent><br/
<hw>Chinese-red</hw> <pos>adj.</pos>  <def>of a vivid red to reddish-orange color.</def><br/
<syn><b>Syn. --</b> vermilion, vermillion, cinibar.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw> <pr>(ch<icr/<nsm/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>chine</ets>, AS. <ets>c<imac/ne</ets> fissure, chink, fr. <ets>c<imac/nan</ets> to gape; akin to Goth. <ets>Keinan</ets> to sprout, G. <ets>keimen</ets>. Cf. <er>Chit</er>.]</ety> <def>A small cleft, rent, or fissure, of greater length than breadth; a gap or crack; <as>as, the <ex>chinks</ex> of a wall</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through one cloudless <qex>chink</qex>, in a black, stormy sky.<br/
Shines out the dewy morning star.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw> <pr>(ch<icr/<nsm/k)</pr>, <pos>n.</pos> <ety>[From <ets>chinaman</ets>.]</ety> <def>a chinaman; a chinese person; -- disparaging and offensive.</def> <mark>[slang]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chinked</conjf> <pr>(ch<icr/<nsm/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chinking</conjf>.]</vmorph> <def>To crack; to open.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to open in cracks or fissures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill up the chinks of; <as>as, to <ex>chink</ex> a wall</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw>, <pos>n.</pos> <ety>[Of imitative origin. Cf. <er>Jingle</er>.]</ety> <sn>1.</sn> <def>A short, sharp sound, as of metal struck with a slight degree of violence.</def> <ldquo/<xex>Chink</xex> of bell.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Money; cash.</def> <mark>[Cant]</mark> <ldquo/To leave his <xex>chink</xex> to better hands.<rdquo/  <rj><au>Somerville.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw>, <pos>v. t.</pos> <def>To cause to make a sharp metallic sound, as coins, small pieces of metal, etc., by bringing them into collision with each other.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chink</ent><br/
<hw>Chink</hw>, <pos>v. i.</pos> <def>To make a slight, sharp, metallic sound, as by the collision of little pieces of money, or other small sonorous bodies.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinky</ent><br/
<hw>Chink"y</hw> <pr>(ch<icr/<nsm/k"<ycr/)</pr>, <pos>a.</pos> <def>Full of chinks or fissures; gaping; opening in narrow clefts.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinned</ent><br/
<hw>Chinned</hw> <pr>(ch<icr/nd)</pr>, <pos>a.</pos> <def>Having a chin; -- used chiefly in compounds; <as>as, short-<ex>chinned</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chino</ent><br/
<hw>chi"no</hw> <pr>(ch<emac/"n<omac/)</pr>, <pos>n.</pos> <def>a coarse, tough, twilled cotton fabric used for uniforms or sports clothes.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chinoidine</ent><br/
<hw>Chi*noid"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>chin</ets>ium quinine (cf. G. & F. <ets>china</ets> Peruvian bark) + <ets>-oil</ets> + <ets>-ine</ets>.]</ety> <fld>(Chem.)</fld> <def>See <er>Quinodine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinoline</ent><br/
<hw>Chin"o*line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>chin</ets>ium quinine (see <er>Chinoldine</er>) + L. <ets>ol</ets>eum oil + <ets>-ine</ets>.]</ety> <fld>(Chem.)</fld> <def>See <er>Quinoline</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinone</ent><br/
<hw>Chi"none</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>chin</ets>ium quinine (see <er>Chinoidine</er>.) + <ets>-one</ets>.]</ety> <fld>(Chem.)</fld> <def>See <er>Quinone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinoiserie</ent><br/
||<hw>Chi`noi`se*rie"</hw> <pr>(sh<emac/`nw<adot/`z'*r<emac/")</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>Chinese conduct, art, decoration, or the like; also, a specimen of Chinese manners, art, decoration, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a style in art reflecting Chinese influence, being elaborately decorated and intricately patterned.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chinook</ent><br/
<hw>Chi*nook"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Ethnol.)</fld> <def>One of a tribe of North American Indians now living in the state of Washington, noted for the custom of flattening their skulls. Chinooks also called <altname>Flathead Indians</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A warm westerly wind from the country of the Chinooks, sometimes experienced on the slope of the Rocky Mountains, in Montana and the adjacent territory.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A jargon of words from various languages (the largest proportion of which is from that of the Chinooks) generally understood by all the Indian tribes of the northwestern territories of the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinook State</ent><br/
<hw>Chi*nook" State</hw>. <def>Washington -- a nickname. See <er>Chinook</er>, <pos>n.</pos></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>chinos</ent><br/
<hw>chinos</hw> <pos>n. pl.</pos> <def>trousers made with chino cloth.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chinquapin</ent><br/
<hw>Chin"qua*pin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A branching, nut-bearing tree or shrub (<spn>Castanea pumila</spn>) of North America, from six to twenty feet high, allied to the chestnut. Also, its small, sweet, edible nat.</def> <altsp>[Written also <asp>chincapin</asp> and <asp>chinkapin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chinquapin oak</b></col>, <cd>a small shrubby oak (<spn>Quercus prinoides</spn>) of the Atlantic States, with edible acorns.</cd> -- <col><b>Western Chinquapin</b></col>, <cd>an evergreen shrub or tree (<spn>Castanopes chrysophylla</spn>) of the Pacific coast. In California it is a shrub; in Oregon a tree 30 to 125 feet high.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chinse</ent><br/
<hw>Chinse</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chinsed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chinsing</conjf>.]</vmorph> <fld>(Naut.)</fld> <def>To thrust oakum into (seams or chinks) with a chisel , the point of a knife, or a chinsing iron; to calk slightly.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chinsing iron</b></col>, <cd>a light calking iron.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chintz</ent><br/
<hw>Chintz</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chintzes</plw> <pr>(#)</pr>.</plu> <ety>[Hindi <ets>ch<imac/nt</ets> spotted cotton clooth, <ets>ch<imac/nt<amac/ spot</ets>.]</ety> <def>Cotton cloth, printed with flowers and other devices, in a number of different colors, and often glazed.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chintzy</ent><br/
<hw>chintzy</hw> <pos>adj.</pos> <sn>1.</sn> <def>resembling chintz or decorated with chintz.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>of very poor quality.</def> <mark>[informal]</mark><br/
<syn><b>Syn. --</b> bum, cheap, cheesy, crummy, punk, sleazy, tatty, tinny, chinchy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>stingy; miserly.</def><br/
<syn><b>Syn. --</b> cheap.</syn><br/
[<source>PJC</source>]</p>

<p><ent>chin-up</ent><br/
<hw>chin-up</hw> <pos>n.</pos> <def>an arm exercise performed by pulling oneself up on a horizontal bar until the chin is level with or above the bar.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chin-wagging</ent><br/
<ent>chin-wag</ent><br/
<mhw><hw>chin-wag</hw>, <hw>chin-wagging</hw></mhw> <pos>n.</pos> <def>a light informal conversation for social occasions.</def><br/
<syn><b>Syn. --</b> chitchat, small talk, gab, gabfest, gossip, tittle-tattle, chin-wag, causerie.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chiococca</ent><br/
<hw>Chiococca</hw> <pos>n.</pos> <def>a genus of shrubs of tropical and subtropical New World.</def><br/
<syn><b>Syn. --</b> genus <gen>Chiococca</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chionanthus</ent><br/
<hw>Chionanthus</hw> <pos>n.</pos> <def>a genus of deciduous trees or shrubs: fringe tree.</def><br/
<syn><b>Syn. --</b> genus <gen>Chionanthus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chioppine</ent><br/
<hw>Chiop*pine"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Chopine</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chip</ent><br/
<hw>Chip</hw> <pr>(ch<icr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chipped</conjf> <pr>(ch<icr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chipping</conjf>.]</vmorph> <ety>[Cf. G. <ets>kippen</ets> to cut off the edge, to clip, pare. Cf. <er>Chop</er> to cut.]</ety> <sn>1.</sn> <def>To cut small pieces from; to diminish or reduce to shape, by cutting away a little at a time; to hew.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To break or crack, or crack off a portion of, as of an eggshell in hatching, or a piece of crockery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bet, as with chips in the game of poker.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To chip in</b></col>, <cd>to contribute, as to a fund; to share in the risks or expenses of.</cd> <mark>[Slang. U. S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chip</ent><br/
<hw>Chip</hw>, <pos>v. i.</pos> <def>To break or fly off in small pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chip</ent><br/
<hw>Chip</hw>, <pos>n.</pos> <sn>1.</sn> <def>A piece of wood, stone, or other substance, separated by an ax, chisel, or cutting instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fragment or piece broken off; a small piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Wood or Cuban palm leaf split into slips, or straw plaited in a special manner, for making hats or bonnets.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Anything dried up, withered, or without flavor; -- used contemptuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>One of the counters used in poker and other games.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>The triangular piece of wood attached to the log line.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Buffalo chips</b></col>. <cd>See under <er>Buffalo</er>.</cd> -- <col><b>Chip ax</b></col>, <cd>a small ax for chipping timber into shape.</cd> -- <mcol><col><b>Chip bonnet</b></col>, <col><b>Chip hat</b></col></mcol>, <cd>a bonnet or a hat made of Chip. See <er>Chip</er>, <pos>n.</pos>, 3.</cd> -- <col><b>A chip off the old block</b></col>, <cd>a child who resembles either of his parents.</cd> <mark>[Colloq.]</mark> <au>Milton.</au> -- <mcol><col><b>Potato chips</b></col>, <col><b>Saratoga chips</b></col></mcol>, <cd>thin slices of raw potato fried crisp.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>chipboard</ent><br/
<hw>chipboard</hw> <pos>n.</pos> <def>a cheap hard material made from wood chips that are pressed together and bound with synthetic resin.</def><br/
<syn><b>Syn. --</b> hardboard.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chipmunk</ent><br/
<hw>Chip"munk`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Indian name.]</ety> <fld>(Zool.)</fld> <def>A squirrel-like animal of the genus <gen>Tamias</gen>, sometimes called the <altname>striped squirrel</altname>, <altname>chipping squirrel</altname>, <altname>ground squirrel</altname>, <altname>hackee</altname>. The common species of the United States is the <spn>Tamias striatus</spn>.</def> <altsp>[Written also <asp>chipmonk</asp>, <asp>chipmuck</asp>, and <asp>chipmuk</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>chipped</ent><br/
<hw>chipped</hw> <pos>adj.</pos> <def>having a small piece broken off; <as>as, a <ex>chipped</ex> tooth</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chippendale</ent><br/
<hw>Chip"pen*dale</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Designating furniture designed, or like that designed, by <person>Thomas Chippendale</person>, an English cabinetmaker of the 18th century. Chippendale furniture was generally of simple but graceful outline with delicately carved rococo ornamentation, sculptured either in the solid wood or, in the cheaper specimens, separately and glued on. In the more elaborate pieces three types are recognized: <stype><col><b>French Chippendale</b></col></stype>, having much detail, like Louis Quatorze and Louis Quinze; <stype><col><b>Chinese Chippendale</b></col></stype>, marked by latticework and pagodalike pediments; and <stype><col><b>Gothic Chippendale</b></col></stype>, attempting to adapt medieval details. The forms, as of the cabriole and chairbacks, often resemble Queen Anne. In chairs, the seat is widened at the front, and the back toward the top widened and bent backward, except in Chinese Chippendale, in which the backs are usually rectangular.</def> -- <wordforms><wf>Chip"pen*dal*ism</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>It must be clearly and unmistakably understood, then, that, whenever painted (that is to say, decorated with painted enrichment) or inlaid furniture is described as <qex>Chippendale</qex>, no matter where or by whom, it is a million chances to one that the description is incorrect.</q>  <rj><qau>R. D. Benn.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chipper</ent><br/
<hw>Chip"per</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Cheep</er>, <er>Chirp</er>.]</ety> <def>To chirp or chirrup.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chipper</ent><br/
<hw>Chip"per</hw>, <pos>a.</pos> <def>Lively; cheerful; talkative.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chippeways</ent><br/
<hw>Chip"pe*ways</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Chippeway</singw></sing>. <fld>(Ethnol.)</fld> <def>A tribe of Indians formerly inhabiting the northern and western shores of Lake Superior; -- called also <altname>Objibways</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chipping</ent><br/
<hw>Chip"ping</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A chip; a piece separated by a cutting or graving instrument; a fragment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of cutting or breaking off small pieces, as in dressing iron with a chisel, or reducing a timber or block of stone to shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The breaking off in small pieces of the edges of potter's ware, porcelain, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chipping bird</ent><br/
<hw>Chip"ping bird`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The chippy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chipping squirrel</ent><br/
<hw>Chip"ping squir"rel</hw> <pr>(?)</pr>. <def>See <er>Chipmunk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chippy</ent><br/
<hw>Chip"py</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in, or resembling, chips; dry and tasteless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chippy</ent><br/
<hw>Chip"py</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small American sparrow (<spn>Spizella socialis</spn>), very common near dwelling; -- also called <altname>chipping bird</altname> and <altname>chipping sparrow</altname>, from its simple note.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chips</ent><br/
<hw>Chips</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A ship's carpenter.</def> <mark>[Cant.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiragra</ent><br/
||<hw>Chi*ra"gra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/; <?/ hand + <?/ seizure.]</ety> <fld>(Med.)</fld> <def>Gout in the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiragrical</ent><br/
<hw>Chi*rag"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the gout in the hand, or subject to that disease.</def>  <rj><au>Sir. T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiretta</ent><br/
||<hw>Chi*ret"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>chir<amac/<imac/t<amac/</ets>.]</ety> <def>A plant (<spn>Agathotes Chirayta</spn>) found in Northern India, having medicinal properties to the gentian, and esteemed as a tonic and febrifuge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirk</ent><br/
<hw>Chirk</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Chirp</er>, also <er>Creak</er>.]</ety> <sn>1.</sn> <def>To shriek; to gnash; to utter harsh or shrill cries.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All full of <qex>chirkyng</qex> was that sorry place.</q> <rj><qau>Cheucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To chirp like a bird.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirk</ent><br/
<hw>Chirk</hw>, <pos>v. t.</pos> <def>To cheer; to enliven; <as>as, to <ex>chirk</ex> one up</as>.</def> <mark>[Colloq. New Eng. ]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirk</ent><br/
<hw>Chirk</hw>, <pos>a.</pos> <ety>[From <er>Chirk</er>, <pos>v. i.</pos>]</ety> <def>Lively; cheerful; in good spirits.</def> <mark>[Colloq. New Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirm</ent><br/
<hw>Chirm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cirm</ets>, <ets>cyrm</ets>.]</ety> <def>Noise; din; esp.; confused noise, clamor, or hum of many voices, notes of birds, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chirm</ent><br/
<hw>Chirm</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. AS. <ets>cyrman</ets>, <ets>cirman</ets>, to cry out. <root/24. Cf. <er>Chirp</er>.]</ety> <def>To chirp or to make a mournful cry, as a bird.</def> <mark>[Obs.]</mark>  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirognomy</ent><br/
<hw>Chi*rog"no*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <?/ understanding.]</ety> <def>The art of judging character by the shape and appearance of the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirograph</ent><br/
<hw>Chi"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ written with the hand; <grk>chei`r</grk> hand + <grk>gra`fein</grk> to write.]</ety> <fld>(Old. Law)</fld> <sd>(a)</sd> <def>A writing which, requiring a counterpart, was engrossed twice on the same piece of parchment, with a space between, in which was written the word <xex>chirographum</xex>, through which the parchment was cut, and one part given to each party. It answered to what is now called a <altname>charter party</altname>.</def> <sd>(b)</sd> <def>The last part of a fine of land, commonly called the <xex>foot</xex> of the fine.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirographer</ent><br/
<hw>Chi*rog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who practice the art or business of writing or engrossing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>See <er>chirographist</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chirographer of fines</b></col> <fld>(Old Eng. Law)</fld>, <cd>an officer in the court of common pleas, who engrossed fines.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirographical</ent><br/
<ent>Chirographic</ent><br/
<mhw>{ <hw>Chi`ro*graph"ic</hw> <pr>(?)</pr>, <hw>Chi`ro*graph"ic*al</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <def>Of or pertaining to chirography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirographist</ent><br/
<hw>Chi*rog"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A chirographer; a writer or engrosser.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who tells fortunes by examining the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirography</ent><br/
<hw>Chi*rog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The art of writing or engrossing; handwriting; <as>as, skilled in <ex>chirography</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The art of telling fortunes by examining the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirogymnast</ent><br/
<hw>Chi`ro*gym"nast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <?/  trainer of athletes, gymnast.]</ety> <def>A mechanical contrivance for exercising the fingers of a pianist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirological</ent><br/
<hw>Chi`ro*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to chirology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirologist</ent><br/
<hw>Chi*rol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who communicates thoughts by signs made with the hands and fingers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirology</ent><br/
<hw>Chi*rol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <ets>-logy</ets>.]</ety> <def>The art or practice of using the manual alphabet or of communicating thoughts by sings made by the hands and fingers; a substitute for spoken or written language in intercourse with the deaf and dumb. See <er>Dactylalogy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiromancer</ent><br/
<hw>Chi"ro*man`cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who practices chiromancy.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiromancy</ent><br/
<hw>Chi"ro*mancy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <ets>-mancy</ets>.]</ety> <def>The art or practice of foretelling events, or of telling the fortunes or the disposition of persons by inspecting the hand; palmistry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiromantist</ent><br/
<ent>Chiromanist</ent><br/
<mhw>{ <hw>Chi"ro*man`ist</hw> <pr>(?)</pr>, <hw>Chi"ro*man`tist</hw> <pr>(?)</pr> }</mhw> <pos>n.</pos> <ety>[Gr. <?/.]</ety> <def>A chiromancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiromantical</ent><br/
<ent>Chiromantic</ent><br/
<mhw>{ <hw>Chi`ro*man"tic</hw> <pr>(?)</pr>, <hw>Chi`ro*man"tic*al</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <def>Of or pertaining to chiromancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiromonic</ent><br/
<hw>Chi`ro*mon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to chironomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chironomy</ent><br/
<hw>Chi*ron"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>chei`r</grk> hand + <?/ to manage.]</ety> <def>The art of moving the hands in oratory or in pantomime; gesture</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiroplast</ent><br/
<hw>Chi"ro*plast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ formed by hand; <grk>chei`r</grk> hand + <?/ to shape.]</ety> <fld>(Mus.)</fld> <def>An instrument to guid the hands and fingers of pupils in playing on the piano, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiropodist</ent><br/
<hw>Chi*rop"o*dist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <?/; <?/, foot.]</ety> <def>One who treats diseases of the hands and feet; especially, one who removes corns and bunions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chiropody</ent><br/
<hw>Chirop"ody</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art of treating diseases of the hands and feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chiropractic</ent><br/
<hw>chiropractic</hw> <pos>n.</pos> <def>a method of medical treatment that manipulates body structures, especially the spine, in the belief that it restores proper nerve functioning.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chiropractor</ent><br/
<hw>chiropractor</hw> <pos>n.</pos> <def>a therapist who practics chiropractic.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chiropter</ent><br/
<hw>chiropter</hw> <pos>n.</pos> <ety>[See <er>chiroptera</er>.]</ety> <def>any mammal belonging to the order <ord>chiroptera</ord>; a bat.</def><br/
<syn><b>Syn. --</b> chiropteran; bat.</syn><br/
[<source>PJC</source>]</p>

<p><ent>chiropteran</ent><br/
<hw>chiropteran</hw> <pos>n.</pos> <def>same as <er>chiropter</er>.</def><br/
<syn><b>Syn. --</b> chiropter; bat.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chiroptera</ent><br/
<hw>chi*rop"te*ra</hw> <pr>(k<isl/*r<ocr/p"t<etil/r)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>chei`r</grk> hand + <grk>ptero`n</grk> wing.]</ety> <fld>(Zool.)</fld> <def>an ancient order of mammalia dating to the early Eocene, including the bats. They are nocturnal mouselike mammals having four toes of each of the anterior limbs elongated and connected by a web, so that they form membranous wings that can be used in flying.  They also have anatomical adaptations, including large ears, for echolocation, by which they navigate and in some cases find insects.  The order includes the suborders <subord>Megachiroptera</subord> (the fruit bats) and <subord>Microchiroptera</subord> (insectivorous bats). See <er>Bat</er>.  Previously spelled <asp>cheiroptera</asp>.</def> <mark>[archaic]</mark><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>chiropteran</ent><br/
<hw>chiropteran</hw> <pos>n.</pos> <def>nocturnal mouselike mammal with forelimbs modified to form membranous wings and anatomical adaptations for echolocation by which they navigate.</def><br/
<syn><b>Syn. --</b> bat.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chirosophist</ent><br/
<hw>Chiros"ophist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chei`r</grk> hand + <?/ skillful, wise. See <er>Sophist</er>.]</ety> <def>A fortune teller.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirp</ent><br/
<hw>Chirp</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chirped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chirping</conjf>.]</vmorph> <ety>[Of imitative orgin. Cf. <er>Chirk</er>, <er>Chipper</er>, <er>Cheep</er>, <er>Chirm</er>, <er>Chirrup</er>.]</ety> <def>To make a shop, sharp, cheerful, as of small birds or crickets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirp</ent><br/
<hw>Chirp</hw>, <pos>n.</pos> <def>A short, sharp note, as of a bird or insect.</def> <ldquo/The <qex>chirp</qex> of flitting bird.<rdquo/  <rj><au>Bryant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirper</ent><br/
<hw>Chirp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chirps, or is cheerful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirping</ent><br/
<hw>Chirp"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cheering; enlivening.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He takes his <qex>chirping</qex> pint, he cracks his jokes.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirpingly</ent><br/
<hw>Chirp"ing*ly</hw>, <pos>adv.</pos> <def>In a chirping manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirre</ent><br/
<hw>Chirre</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. G. <ets>girren</ets>, AS. <ets>corian</ets> to murmur, complain. <root/24.]</ety> <def>To coo, as a pigeon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirrup</ent><br/
<hw>Chir"rup</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chirruped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chirruping</conjf>.]</vmorph> <ety>[See <er>Chirp</er>.]</ety> <def>To quicken or animate by chirping; to cherup.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirrup</ent><br/
<hw>Chir"rup</hw>, <pos>v. i.</pos> <def>To chirp.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The criket <qex>chirrups</qex> on the hearth.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirrup</ent><br/
<hw>Chir"rup</hw>, <pos>n.</pos> <def>The act of chirping; a chirp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sparrows' <qex>chirrup</qex> on the roof.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirrupy</ent><br/
<hw>Chir"rupy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cheerful; joyous; chatty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirurgeon</ent><br/
<hw>Chi*rur"geon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chirurgien</ets>, from <ets>chirurgie</ets> surgery, fr. Gr. <?/, fr. <?/ working or operating with the hand; <grk>chei`r</grk> hand + <?/ work. Cf. <er>Surgeon</er>, <er>Work</er>.]</ety> <def>A surgeon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirurgeonly</ent><br/
<hw>Chi*rur"geon*ly</hw>, <pos>adv.</pos> <def>Surgically.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirurgery</ent><br/
<hw>Chi*rur"ger*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chirurgeon</er>, and cf. <er>Surgery</er>.]</ety> <def>Surgery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirurgical</ent><br/
<ent>Chirurgic</ent><br/
<mhw>{ <hw>Chi*rur"gic</hw> <pr>(?)</pr>, <hw>Chirur"gical</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>chirurgique</ets>rurgical, L. <ets>Chirurgicus</ets>, Gr. <?/. See <er>Chirurgeon</er>, and cf. <er>Surgical</er>.]</ety> <def>Surgical</def> <mark>[Obs.]</mark> <ldquo/<xex>Chirurgical</xex> lore<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 250 --></p>

<p><ent>Chisel</ent><br/
<hw>Chis"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chisel</ets>, F. <ets>ciseau</ets>, fr. LL. <ets>cisellus</ets>, prob. for <ets>caesellus</ets>, fr. L. <ets>caesus</ets>, p. p. of <ets>caedere</ets> to cut. Cf. <er>Scissors</er>.]</ety> <def>A tool with a cutting edge on one end of a metal blade, used in dressing, shaping, or working in timber, stone, metal, etc.; -- usually driven by a mallet or hammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cold chisel</b></col>. <cd>See under <er>Cold</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chisel</ent><br/
<hw>Chis"el</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chiseled</conjf> <pr>(?)</pr>, or <conjf>Chiselled</conjf> (<?/); <pos>p. pr. & vb. n.</pos> <conjf>Chiseling</conjf>, or <conjf>Chiselling</conjf>.]</vmorph> <ety>[Cf. F. <ets>ciseler</ets>.]</ety> <sn>1.</sn> <def>To cut, pare, gouge, or engrave with a chisel; <as>as, to <ex>chisel</ex> a block of marble into a statue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut close, as in a bargain; to cheat.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chiseller</ent><br/
<ent>chiseler</ent><br/
<mhw><hw>chiseler</hw>, <hw>chiseller</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>a person who swindles you by means of deception or fraud.</def><br/
<syn><b>Syn. --</b> swindler, chiseler, gouger, sharper, sharpie, sharpy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chishona</ent><br/
<hw>Chishona</hw> <pos>n.</pos> <def>one of the two major languages of Zimbabwe.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chisinau</ent><br/
<hw>Chisinau</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>Moldova</country>.  Population (2000) = 676,700.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chislev</ent><br/
<hw>Chis"lev</hw> <pr>(k<icr/s"l<ucr/f; k<icr/s"l<ucr/v; k<emac/s*l<ecr/v")</pr>, <pos>n.</pos> <ety>[Heb.]</ety> <def>The third month of the Jewish civil year; the ninth month of the ecclesiastical year in the Jewish calendar, occupying a part of November and a part of December; -- same as <er>Kislev</er>.</def><br/
<syn><b>Syn. --</b> Kislev.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chisley</ent><br/
<hw>Chis"ley</hw> <pr>(ch<icr/z"l<ycr/)</pr>, <pos>a.</pos> <ety>[AS. <ets>ceosel</ets> gravel or sand. Cf. <er>Chessom</er>.]</ety> <def>Having a large admixture of small pebbles or gravel; -- said of a soil.</def>  <rj><au>Gardner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chit</ent><br/
<hw>Chit</hw> <pr>(ch<icr/t)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>c<imac/<edh/</ets> shoot, sprig, from the same root as <ets>c<imac/nan</ets> to yawn. See <er>Chink</er> a cleft.]</ety> <sn>1.</sn> <def>The embryo or the growing bud of a plant; a shoot; a sprout; <as>as, the <ex>chits</ex> of Indian corn or of potatoes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A child or babe; <as>as, a forward <ex>chit</ex></as>; also, a young, small, or insignificant person or animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A little <qex>chit</qex> of a woman.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An excrescence on the body, as a wart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small tool used in cleaving laths.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitty</ent><br/
<ent>Chit</ent><br/
<mhw>{ <hw>Chit</hw>, <hw>Chit"ty</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Hind. <ets>chi<tsdot/<tsdot/<imac/</ets>.]</ety> <sn>1.</sn>  <def>A short letter or note; a written message or memorandum; a certificate given to a servant; a pass, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A signed voucher or memorandum of a small debt, as for food and drinks at a club.</def> <mark>[India, China, etc.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chit</ent><br/
<hw>Chit</hw>, <pos>v. i.</pos> <def>To shoot out; to sprout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have known barley <qex>chit</qex> in seven hours after it had been thrown forth.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chit</ent><br/
<hw>Chit</hw>, <pos>3d pers. sing.</pos> of <er>Chide</er>. <def>Chideth.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitchat</ent><br/
<hw>Chit"chat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Chat</er>, by way of reduplication.]</ety> <def>Familiar or trifling talk; prattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitin</ent><br/
<hw>Chi"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chiton</er>.]</ety> <fld>(Chem.)</fld> <def>A white amorphous horny substance forming the harder part of the outer integument of insects, crustacea, and various other invertebrates; entomolin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitinization</ent><br/
<hw>Chi`ti*ni*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The process of becoming chitinous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitinous</ent><br/
<hw>Chi"ti*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the nature of chitin; consisting of, or containing, chitin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chitlins</ent><br/
<ent>chitlings</ent><br/
<mhw><hw>chitlings</hw>, <hw>chitlins</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>small intestines of hogs prepared as food.</def><br/
<syn><b>Syn. --</b> chitterlings.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chiton</ent><br/
||<hw>Chi"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a chiton (in sense 1).]</ety> <sn>1.</sn> <def>An under garment among the ancient Greeks, nearly representing the modern shirt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of a group of gastropod mollusks, with a shell composed of eight movable dorsal plates. See <er>Polyplacophora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chittamwood</ent><br/
<hw>chittamwood</hw> <pos>n.</pos> <sn>1.</sn> <def>a shrubby tree of the U. S. Pacific coast (<spn>Rhamnus purshianus</spn> or <spn>Rhamnus purshiana</spn>), whose bark is the <prod>cascara sagrada</prod>, used as a mild cathartic or laxative.</def><br/
<syn><b>Syn. --</b> cascara, cascara buckthorn, bearberry, bearwood, chittimwood, <spn>Rhamnus purshianus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a deciduous tree of SE U.S. and Mexico.</def><br/
<syn><b>Syn. --</b> false buckthorn, chittimwood, shittimwood, black haw, Bumelia lanuginosa.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>a shrubby tree of southern U.S. (<spn>Cotinus obovatus</spn>) having large plumes of feathery flowers resembling puffs of smoke; -- called also <altname>American smoke tree</altname>.</def><br/
<syn><b>Syn. --</b> American smokewood, <spn>Cotinus americanus</spn>, <spn>Cotinus obovatus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chitter</ent><br/
<hw>Chit"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Chatter</er>.]</ety> <sn>1.</sn> <def>To chirp in a tremulous manner, as a bird.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shiver or chatter with cold.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitterling</ent><br/
<hw>Chit"ter*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The frill to the breast of a shirt, which when ironed out resembled the small entrails. See <er>Chitterlings</er>.</def> <mark>[Obs.]</mark>  <rj><au>Gascoigne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitterlings</ent><br/
<hw>Chit"ter*lings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Cf. AS. <ets>cwi<thorn/</ets> womb, Icel. <ets>kvi<edh/</ets>, Goth. <ets>qi<thorn/us</ets>, belly, womb, stomach, G. <ets>kutteln</ets> chitterlings.]</ety> <fld>(Cookery)</fld> <def>The smaller intestines of swine, etc., fried for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chittra</ent><br/
||<hw>Chit"tra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native Indian name.]</ety> <fld>(Zool.)</fld> <def>The axis deer of India.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chitty</ent><br/
<hw>Chit"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of chits or sprouts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Childish; like a babe.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chivachie</ent><br/
<hw>Chiv"a*chie`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>chevauchie</ets>, <ets>chevauch<eacute/e</ets>; of the same origin as E. <ets>cavalcade</ets>.]</ety> <def>A cavalry raid; hence, a military expedition.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chivalric</ent><br/
<hw>Chiv"al*ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Chivalry</er>.]</ety> <def>Relating to chivalry; knightly; chivalrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chivalrous</ent><br/
<hw>Chiv"al*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>chevalerus</ets>, <ets>chevalereus</ets>, fr. <ets>chevalier</ets>. See <er>Chivalry</er>.]</ety> <def>Pertaining to chivalry or knight-errantry; warlike; heroic; gallant; high-spirited; high-minded; magnanimous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In brave pursuit of <qex>chivalrous</qex> emprise.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chivalrously</ent><br/
<hw>Chiv"al*rous*ly</hw>, <pos>adv.</pos> <def>In a chivalrous manner; gallantly; magnanimously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chivalry</ent><br/
<hw>Chiv"al*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chevalerie</ets>, fr. <ets>chevalier</ets> knight, OF., horseman. See <er>Chevalier</er>, and cf. <er>Cavalry</er>.]</ety> <sn>1.</sn> <def>A body or order of cavaliers or knights serving on horseback; illustrious warriors, collectively; cavalry.</def> <ldquo/His Memphian <xex>chivalry</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By his light<br/
Did all the <qex>chivalry</qex> of England move,<br/
To do brave acts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The dignity or system of knighthood; the spirit, usages, or manners of knighthood; the practice of knight-errantry.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The qualifications or character of knights, as valor, dexterity in arms, courtesy, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The glory of our Troy this day doth lie<br/
On his fair worth and single <qex>chivalry</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eng. Law)</fld> <def>A tenure of lands by knight's service; that is, by the condition of a knight's performing service on horseback, or of performing some noble or military service to his lord.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Exploit.</def> <mark>[Obs.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Court of chivalry</b></col>, <cd>a court formerly held before the lord high constable and earl marshal of England as judges, having cognizance of contracts and other matters relating to deeds of arms and war.</cd>  <rj><au>Blackstone.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>chivaree</ent><br/
<hw>chivaree</hw> <pos>n.</pos> <def>a noisy mock serenade (made by banging pans and kettles) to a newly married couple.</def><br/
<syn><b>Syn. --</b> shivaree, charivari, callithump, callathump.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chivarros</ent><br/
<ent>Chivarras</ent><br/
<mhw>{ ||<hw>Chi*var"ras</hw> <pr>(?)</pr>,  ||<hw>Chi*var"ros</hw> <pr>(?)</pr>, }</mhw> <pos>n. pl.</pos> <ety>[Mex. Sp.]</ety> <def>Leggings.</def> <mark>[Mex. & Southwestern U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chive</ent><br/
<hw>Chive</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A filament of a stamen.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chives</ent><br/
<ent>chive</ent><br/
<mhw><hw>chive</hw>, <hw>chives</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cive</ets>, fr. L. <ets>cepa</ets>, <ets>caepa</ets>, onion. Cf. <er>Cives</er>, <er>Cibol</er>.]</ety> <fld>(Bot.)</fld> <def>A perennial plant (<spn>Allium Schoenoprasum</spn>), allied to the onion,  having hollow cylindrical leaves used for seasoning. The young leaves are used in omelets, etc.</def> <altsp>[Written also <asp>cive</asp>.]</altsp><br/
<syn><b>Syn. --</b> chive, cive, schnittlaugh, <spn>Allium schoenoprasum</spn>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the bulbous herb <spn>Allium Schoenoprasum</spn> used fresh as a mild onion-flavored seasoning.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chivvy</ent><br/
<hw>chivvy</hw> <pos>v.</pos> <def>same as <er>chivy</er>.</def> <altsp>[Also spelled <asp>chivy</asp>, <asp>chevy</asp>, and <asp>chevvy</asp>.]</altsp><br/
<syn><b>Syn. --</b> harass, hassle, harry, beset, plague, molest, provoke.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chivy</ent><br/
<hw>Chiv"y</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chivied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chivying</conjf>.]</vmorph> <ety>[Cf. <er>Chevy</er>.]</ety> <def>To goad, drive, hunt, throw, or pitch; to repeatedly cause annoyance or concern to.</def> <mark>[Slang, Eng.]</mark>  <altsp>[Also spelled <asp>chivvy</asp>, <asp>chevy</asp>, and <asp>chevvy</asp>.]</altsp> <rj><au>Dickens.</au></rj><br/
<syn><b>Syn. --</b> harass, hassle, harry, beset, plague, molest, provoke.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Chlamydate</ent><br/
<hw>Chlam"y*date</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>chlamydatus</ets> dressed in a military cloak. See <er>Chlamys</er>.]</ety> <fld>(Zool.)</fld> <def>Having a mantle; -- applied to certain gastropods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chlamydia</ent><br/
<hw>chlamydia</hw> <pos>n.</pos> <def>a coccoid rickettsia which may infect birds and mammals; it causes infections of eyes and lungs and the genitourinary tract.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlamydiaceae</ent><br/
<hw>Chlamydiaceae</hw> <pos>prop. n.</pos> <def>A natural family of gram-negative bacteria which are parasites in warm-blooded vertebrates.</def><br/
<syn><b>Syn. --</b> family <fam>Chlamydiaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlamydomonadaceae</ent><br/
<hw>Chlamydomonadaceae</hw> <pos>prop. n.</pos> <def>A natural family of green algae some of which are colored red by hematochrome.</def><br/
<syn><b>Syn. --</b> family <fam>Chlamydomonadaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlamydomonas</ent><br/
<hw>Chlamydomonas</hw> <pos>prop. n.</pos> <def>The type genus of the <fam>Chlamydomonadaceae</fam>; they are solitary biflagellated plantlike algae common in fresh water and damp soil.  They multiply freely and are often a pest around filtration plants.</def><br/
<syn><b>Syn. --</b> genus <gen>Chlamydomonas</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlamydosaurus</ent><br/
<hw>Chlamydosaurus</hw> <pos>n.</pos> <def>a genus of frilled lizards.</def><br/
<syn><b>Syn. --</b> genus <gen>Chlamydosaurus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlamyphore</ent><br/
<hw>Chlam"y*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ cloak + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>A small South American edentate (<spn>Chlamyphorus truncatus</spn>, and <spn>Chlamyphorus retusus</spn>) allied to the armadillo. It is covered with a leathery shell or coat of mail, like a cloak, attached along the spine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlamys</ent><br/
||<hw>Chla"mys</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Chlamyses</plw> <pr>(#)</pr>, L. <plw>Chlamydes</plw> <pr>(#)</pr>.</plu> <ety>[L., from Gr. <?/.]</ety> <def>A loose and flowing outer garment, worn by the ancient Greeks; a kind of cloak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloasma</ent><br/
||<hw>Chlo*as"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ to be green.]</ety> <fld>(Med.)</fld> <def>A cutaneous affection characterized by yellow or yellowish brown pigmented spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloral</ent><br/
<hw>Chlo"ral</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chlor</ets>ine + <ets>al</ets>cohol.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A colorless oily liquid, <chform>CCl3.CHO</chform>, of a pungent odor and harsh taste, obtained by the action of chlorine upon ordinary or ethyl alcohol.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Chloral hydrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloralamide</ent><br/
<hw>Chlo"ral*am`ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chloral</ets> + <ets>amide</ets>.]</ety> <fld>(Chem.)</fld> <def>A compound of chloral and formic amide used to produce sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chloral hydrate</ent><br/
<hw>chloral hydrate</hw> <pos>n.</pos> <sn>1.</sn> <def>a chemical substance (<chform>CCl3.CH(OH)2</chform>) which is a hydrate of trichloroacetaldehyde.  It crystallizes as white monoclinic plates, obtained by treating chloral with water. It produces sleep when taken internally or hypodermically, and is used in medicine as a hypnotic and sedative; -- called also <altname>chloral</altname></def>  <note>It may be habit-forming, and is a controlled substance listed in the U. S. Code of Federal Regulations.  It is sometimes used to render a person unconscious for illegal or nefarious purposes, and in this use, a concentrated solution is one of the agents called <er>knockout drops</er>.</note><br/
[<source>PJC</source>]</p>

<p><ent>Chloralism</ent><br/
<hw>Chlo"ral*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A morbid condition of the system resulting from excessive use of chloral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloralum</ent><br/
<hw>Chlor`al"um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chlor</ets>ine + <ets>alum</ets>inium.]</ety> <def>An impure aqueous solution of chloride of aluminium, used as an antiseptic and disinfectant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloranil</ent><br/
<hw>Chlor`an"il</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chlor</ets>ine + <ets>anil</ets>ine.]</ety> <fld>(Chem.)</fld> <def>A yellow crystalline substance, <chform>C6Cl4.O2</chform>, regarded as a derivative of quinone, obtained by the action of chlorine on certain benzene derivatives, as aniline.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorate</ent><br/
<hw>Chlo"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chlorate</ets>. See <er>Chlorine</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of chloric acid; <as>as, <ex>chlorate</ex> of potassium</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloraurate</ent><br/
<hw>Chlor`au"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chlor</ets>ine + <ets>aurate</ets>.]</ety> <fld>(Chem.)</fld> <def>See <er>Aurochloride</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorhydric</ent><br/
<hw>Chlor`hy"dric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chlor</ets>ine + <ets>hydr</ets>ogen + <ets>-ic</ets>.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Hydrochloric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorhydrin</ent><br/
<hw>Chlor`hy"drin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>One of a class of compounds formed from certain polybasic alcohols (and especially glycerin) by the substitution of chlorine for one or more hydroxyl groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloric</ent><br/
<hw>Chlo"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Chlorine</er>.]</ety> <def>Pertaining to, or obtained from, chlorine; -- said of those compounds of chlorine in which this element has a valence of five, or the next to its highest; <as>as, <ex>chloric</ex> acid, <chform>HClO3</chform></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chloric ether</b></col> <fld>(Chem.)</fld>, <cd>ethylene dichloride. See <cref>Dutch liquid</cref>, under <er>Dutch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloridate</ent><br/
<hw>Chlo"ri*date</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To treat or prepare with a chloride, as a plate with chloride of silver, for the purposes of photography.</def>  <rj><au>R. Hunt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloride</ent><br/
<hw>Chlo"ride</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A binary compound of chlorine with another element or radical; <as>as, <ex>chloride</ex> of sodium (common salt)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chloride of ammonium</b></col>, <cd>sal ammoniac.</cd> -- <col><b>Chloride of lime</b></col>, <cd>bleaching powder; a grayish white substance, <chform>CaOCl2</chform>, used in bleaching and disinfecting; -- called more properly <altname>calcium hypochlorite</altname>. See <cref>Hypochlorous acid</cref>, under <er>Hypochlorous</er>.</cd> -- <col><b>Mercuric chloride</b></col>, <cd>corrosive sublimate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloridic</ent><br/
<hw>Chlo*rid"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a chloride; containing a chloride.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloridize</ent><br/
<hw>Chlo"rid*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Chloridate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorimetry</ent><br/
<hw>Chlo*rim"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chlorometry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorinate</ent><br/
<hw>Chlo"rin*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chlorinated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chlorinating</conjf>.]</vmorph> <fld>(Chem.)</fld> <def>To treat, or cause to combine, with chlorine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorination</ent><br/
<hw>Chlo`ri*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of subjecting anything to the action of chlorine; especially, a process for the extraction of gold by exposure of the auriferous material to chlorine gas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorine</ent><br/
<hw>Chlo"rine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ pale green, greenish yellow. So named from its color. See <er>Yellow</er>.]</ety> <fld>(Chem.)</fld> <def>One of the elementary substances, commonly isolated as a greenish yellow gas, two and one half times as heavy as air, of an intensely disagreeable suffocating odor, and exceedingly poisonous. It is abundant in nature, the most important compound being common salt (Sodium chloride). It is powerful oxidizing, bleaching, and disinfecting agent. Symbol Cl. Atomic weight, 35.4.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chlorine family</b></col>, <cd>the elements fluorine, chlorine, bromine, and iodine, called the <altname>halogens</altname>, and classed together from their common peculiarities.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloriodic</ent><br/
<hw>Chlor`i*od"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Compounded of chlorine and iodine; containing chlorine and iodine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloriodine</ent><br/
<hw>Chlor`i"o*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A compound of chlorine and iodine.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorite</ent><br/
<hw>Chlo"rite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ (sc. <?/), fr. <grk>chlwro`s</grk> light green.]</ety> <fld>(Min.)</fld> <def>The name of a group of minerals, usually of a green color and micaceous to granular in structure. They are hydrous silicates of alumina, iron, and magnesia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chlorite slate</b></col>, <cd>a schistose or slaty rock consisting of alumina, iron, and magnesia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorite</ent><br/
<hw>Chlo"rite</hw>, <pos>n.</pos> <ety>[<ets>Chlor</ets>ous + <ets>-ite</ets>.]</ety> <fld>(Chem.)</fld> <def>Any salt of chlorous acid; <as>as, <ex>chlorite</ex> of sodium</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloritic</ent><br/
<hw>Chlo*rit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From 1st <er>Chlorite</er>.]</ety> <def>Pertaining to, or containing, chlorite; <as>as, <ex>chloritic</ex> sand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlormethane</ent><br/
<hw>Chlor`meth"ane</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A colorless gas, <chform>CH3Cl</chform>, of a sweet odor, easily condensed to a liquid; -- called also <altname>methyl chloride</altname>.</def>
<-- Also chloromethane. b. p. -23.7<deg/ C.  It is sold as a liquid under pressure, and used to rapidly chill skin (so as to prevent swelling after an injury); in this fashion it also serves as a local anaesthetic.  It chills by the cooling effect of the rapid evaporation of the liquid form, applied directly to the skin.
  Also used as a refrigerant. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloro-</ent><br/
<hw>Chlo"ro-</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A prefix denoting that <xex>chlorine</xex> is an ingredient in the substance named.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorocruorin</ent><br/
<hw>Chlo`ro*cru"o*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chlwro`s</grk> light green + E. <ets>cruorin</ets>.]</ety> <fld>(Physiol.)</fld> <def>A green substance, supposed to be the cause of the green color of the blood in some species of worms.</def>  <rj><au>Ray Lankester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorodyne</ent><br/
<hw>Chlo"ro*dyne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>chlor</ets>ine, in imitation of ano<ets>dyne</ets>.]</ety> <fld>(Med.)</fld> <def>A patent anodyne medicine, containing opium, chloroform, Indian hemp, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloroform</ent><br/
<hw>Chlo"ro*form</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chlor</ets>ine + <ets>form</ets>yl, it having been regarded as a trichloride of this radical: cf. F. <ets>chloroforme</ets>, G. <ets>chloroform</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless volatile liquid, <chform>CHCl3</chform>, having an ethereal odor and a sweetish taste, formed by treating alcohol with chlorine and an alkali. It is a powerful solvent of wax, resin, etc., and is extensively used to produce an<ae/sthesia in surgical operations; also externally, to alleviate pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloroform</ent><br/
<hw>Chlo"ro*form</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chloroformed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chloroforming</conjf>.]</vmorph> <def>To treat with chloroform, or to place under its influence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloroleucite</ent><br/
<hw>Chlo`ro*leu"cite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chlwro`s</grk> light green + E. <ets>leucite</ets>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Chloroplastid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorometer</ent><br/
<hw>Chlo*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chlorom<egrave/tre</ets>. See <er>Chlorine</er>, and <er>-meter</er>.]</ety> <def>An instrument to test the decoloring or bleaching power of chloride of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorometry</ent><br/
<hw>Chlo*rom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The process of testing the bleaching power of any combination of chlorine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloropal</ent><br/
<hw>Chlo*ro"pal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chlwro`s</grk> light green + E. <ets>opal</ets>.]</ety> <fld>(Min.)</fld> <def>A massive mineral, greenish in color, and opal-like in appearance. It is essentially a hydrous silicate of iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloropeptic</ent><br/
<hw>Chlo`ro*pep"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chlor</ets>ine + <ets>peptic</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>Of or pertaining to an acid more generally called <altname>pepsin-hydrochloric</altname> acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorophane</ent><br/
<hw>Chlo"ro*phane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chlwro`s</grk> light green + <?/ to show: cf. F. <ets>chlorophane</ets>.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>A variety of fluor spar, which, when heated, gives a beautiful emerald green light.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The yellowish green pigment in the inner segment of the cones of the retina. See <er>Chromophane</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorophyll</ent><br/
<hw>Chlo"ro*phyll</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chlwro`s</grk> light green + <grk>fy`llon</grk> leaf: cf. F. <ets>chlorophylle</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Literally, leaf green; a green granular matter formed in the cells of the leaves (and other parts exposed to light) of plants, to which they owe their green color, and through which all ordinary assimilation of plant food takes place. Similar chlorophyll granules have been found in the tissues of the lower animals.</def> <altsp>[Written also <asp>chlorophyl</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>any of a group of green pigments found in photosynthetic organisms.  <stype>Chlorophyll a</stype> and <stype>chlorophyll b</stype> are found in higher plants and green algae; <stype>chlorophyll c</stype> is found in certain types of marine algae.  Chemically, it has a porphyrin ring with a magnesium ion bound to the four central nitrogens, and has a phytyl side chain.  It is essential for photosynthesis in most plants.  <stype>Chlorophyll a</stype> has formula <chform>C55H72N4O5Mg</chform>.</def><br/
<syn><b>Syn. --</b> chlorophyll.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chloroplast</ent><br/
<hw>Chlo"ro*plast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>chloro-</ets> + Gr. <?/ to mold, form.]</ety> <fld>(Biol.)</fld> <def>A plastid containing chlorophyll, developed only in cells exposed to the light. Chloroplasts are minute flattened granules, usually occurring in great numbers in the cytoplasm near the cell wall, and consist of a colorless ground substance saturated with chlorophyll pigments. Under light of varying intensity they exhibit phototactic movements. In animals chloroplasts occur only in certain low forms.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chloroplastid</ent><br/
<hw>Chlo`ro*plas"tid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chlwro`s</grk> light green + E. <ets>plastid</ets>.]</ety> <fld>(Bot.)</fld> <def>A granule of chlorophyll; -- also called <altname>chloroleucite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloroplatinic</ent><br/
<hw>Chlo`ro*pla*tin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>See <er>Platinichloric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorosis</ent><br/
||<hw>Chlo*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>chlwro`s</grk> light green: cf. F. <ets>chlorose</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>The green sickness; an an<ae/mic disease of young women, characterized by a greenish or grayish yellow hue of the skin, weakness, palpitation, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A disease in plants, causing the flowers to turn green or the leaves to lose their normal green color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorotic</ent><br/
<hw>Chlo*rot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>chlorotique</ets>.]</ety> <def>Pertaining to, or affected by, chlorosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chlorous</ent><br/
<hw>Chlo"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Chlorine</er>.]</ety> <sn>1.</sn> <def>Of, pertaining to, or derived from, chlorine; -- said of those compounds of chlorine in which this element has a valence of three, the next lower than in chloric compounds; <as>as, <ex>chlorous</ex> acid, <chform>HClO2</chform></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem. Physics)</fld> <def>Pertaining to, or resembling, the electro-negative character of chlorine; hence, electro-negative; -- opposed to <xex>basylous</xex> or <xex>zincous</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chloroxylon</ent><br/
<hw>Chloroxylon</hw> <pos>n.</pos> <def>a genus of deciduous trees of India and Sri Lanka.</def><br/
<syn><b>Syn. --</b> genus <gen>Chloroxylon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlorpicrin</ent><br/
<hw>Chlor`pi"crin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A heavy, colorless liquid, <chform>CCl3.NO2</chform>, of a strong pungent odor, obtained by subjecting picric acid to the action of chlorine.</def> <altsp>[Written also <asp>chloropikrin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>chlorpromazine</ent><br/
<hw>chlorpromazine</hw> <pos>n.</pos> <def>a drug derived from phenothiazine and used as a sedative and tranquilizer.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chlortetracycline</ent><br/
<hw>chlortetracycline</hw> <pos>n.</pos> <def>a yellow crystalline antibacterial antibiotic used to treat certain bacterial and rickettsial diseases. <altname>Aureomycin</altname> is one common trademark for chlortetracycline.</def><br/
<syn><b>Syn. --</b> Aureomycin.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chlorura</ent><br/
<hw>Chlorura</hw> <pos>n.</pos> <def>one of the genera of birds which comprise the towhees.</def><br/
<syn><b>Syn. --</b> genus <gen>Chlorura</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chloruret</ent><br/
<hw>Chlo"ru*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chlorure</ets>.]</ety> <fld>(Chem.)</fld> <def>A chloride.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choak</ent><br/
<hw>Choak</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>See <er>Choke</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>choanocyte</ent><br/
<hw>cho"a*no*cyte`</hw> <pr>(k<omac/"<adot/*n<osl/*s<imac/t`; k<omac/*<acr/n"<osl/*s<imac/t`)</pr>, <pos>n.</pos> <ety>[Gr. <grk>choa`nh</grk> funnel + <ets>-cyte</ets>.]</ety> <def>any of the flagellated cells in sponges having a collar of cytoplasm around base of the flagellum; they maintain a flow of water through the body.</def><br/
<syn><b>Syn. --</b> collar cell.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Choanoid</ent><br/
<hw>Cho"a*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>choa`nh</grk> funnel + <ets>-oid</ets>.]</ety> <fld>(Anat.)</fld> <def>Funnel-shaped; -- applied particularly to a hollow muscle attached to the ball of the eye in many reptiles and mammals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>choc</ent><br/
<hw>choc</hw> <pos>n.</pos> <def>chocolate; a colloquial British abbreviation; <as>as, a box ov chocs</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chocard</ent><br/
<hw>Cho"card</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The chough.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>choc-ice</ent><br/
<hw>choc-ice</hw> <pos>n.</pos> <def>colloquial British abbreviation for chocolate ice cream.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chock</ent><br/
<hw>Chock</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chocked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chocking</conjf>.]</vmorph> <def>To stop or fasten, as with a wedge, or block; to scotch; <as>as, to <ex>chock</ex> a wheel or cask</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chock</ent><br/
<hw>Chock</hw>, <pos>v. i.</pos> <def>To fill up, as a cavity.</def> <ldquo/The woodwork . . . exactly <xex>chocketh</xex> into joints.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chock</ent><br/
<hw>Chock</hw>, <pos>n.</pos> <sn>1.</sn> <def>A wedge, or block made to fit in any space which it is desired to fill, esp. something to steady a cask or other body, or prevent it from moving, by fitting into the space around or beneath it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A heavy casting of metal, usually fixed near the gunwale. It has two short horn-shaped arms curving inward, between which ropes or hawsers may pass for towing, mooring, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chock</ent><br/
<hw>Chock</hw>, <pos>adv.</pos> <fld>(Naut.)</fld> <def>Entirely; quite; <as>as, <ex>chock</ex> home; <ex>chock</ex> aft</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chock</ent><br/
<hw>Chock</hw>, <pos>v. t.</pos> <ety>[F. <ets>choquer</ets>. Cf. <er>Shock</er>, <pos>v. t.</pos>]</ety> <def>To encounter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chock</ent><br/
<hw>Chock</hw>, <pos>n.</pos> <def>An encounter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chockablock</ent><br/
<hw>Chock"a*block</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Hoisted as high as the tackle will admit; brought close together, as the two blocks of a tackle in hoisting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chockfull</ent><br/
<ent>Chock-full</ent><br/
<mhw><hw>Chock"-full`</hw> <hw>chock"full`</hw></mhw> <pr>(?)</pr>, <pos>pred. a.</pos> <def>Quite full; full to capacity; choke-full; <as>as, chowder <ex>chock-full</ex> of clams</as>.</def><br/
<syn><b>Syn. --</b> chockablock(predicate), chockful(predicate), choke-full(predicate), chuck-full(predicate), cram full.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Chocolate</ent><br/
<hw>Choc"o*late</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. the Mexican name of the cacao. Cf. <er>Cacao</er>, <er>Cocoa</er>.]</ety> <sn>1.</sn> <def>A paste or cake composed of the roasted seeds of the <spn>Theobroma Cacao</spn> ground and mixed with other ingredients, usually sugar, and cinnamon or vanilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The beverage made by dissolving a portion of the paste or cake in boiling water or milk.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chocolate house</b></col>, <cd>a house in which customers may be served with chocolate.</cd> -- <col><b>Chocolate nut</b></col>. <cd>See <er>Cacao</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choctaws</ent><br/
<hw>Choc"taws</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Choctaw</singw>.</sing> <fld>(Ethnol.)</fld> <def>A tribe of North American Indians (Southern Appalachian), in early times noted for their pursuit of agriculture, and for living at peace with the white settlers. They are now one of the civilized tribes of the Indian Territory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chode</ent><br/
<hw>Chode</hw> <pr>(ch<omac/d)</pr>, <def>the old <pos>imp.</pos> of <xex>chide</xex>. See <er>Chide</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chogset</ent><br/
<hw>Chog"set</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Cunner</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 251 --></p>

<p><ent>Choice</ent><br/
<hw>Choice</hw> <pr>(chois)</pr>, <pos>n.</pos> <ety>[OE. <ets>chois</ets>, OF. <ets>chois</ets>, F. <ets>choix</ets>, fr. <ets>choisir</ets> to choose; of German origin; cf. Goth. <ets>kausjan</ets> to examine, <ets>kiusan</ets> to choose, examine, G. <ets>kiesen</ets>. <root/46. Cf. <er>Choose</er>.]</ety> <sn>1.</sn> <def>Act of choosing; the voluntary act of selecting or separating from two or more things that which is preferred; the determination of the mind in preferring one thing to another; election.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The power or opportunity of choosing; option.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Choice</qex> there is not, unless the thing which we take be so in our power that we might have refused it.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Care in selecting; judgment or skill in distinguishing what is to be preferred, and in giving a preference; discrimination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I imagine they [the apothegms of C<ae/sar] were collected with judgment and <qex>choice</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sufficient number to choose among.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The thing or person chosen; that which is approved and selected in preference to others; selection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The common wealth is sick of their own <qex>choice</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The best part; that which is preferable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The flower and <qex>choice</qex><br/
Of many provinces from bound to bound.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To make a choice of</b></col>, <cd>to choose; to select; to separate and take in preference.</cd></cs></p>

<p><syn><b>Syn.</b> - See <er>Volition</er>, <er>Option</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choice</ent><br/
<hw>Choice</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Choicer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Choicest</adjf> <pr>(?)</pr>.]</amorph> <sn>1.</sn> <def>Worthly of being chosen or preferred; select; superior; precious; valuable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>choicest</qex> hours of life are lost.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Preserving or using with care, as valuable; frugal; -- used with <xex>of</xex>; <as>as, to be <ex>choice</ex> of time, or of money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Selected with care, and due attention to preference; deliberately chosen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Choice</qex> word measured phrase.</q> <rj><qau>Wordsworth.</qau></rj></p>

<p><syn><b>Syn.</b> - Select; precious; exquisite; uncommon; rare; chary; careful/</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choiceful</ent><br/
<hw>Choice"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Making choices; fickle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>choiceful</qex> sense with every change doth fit.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choicely</ent><br/
<hw>Choice"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>With care in choosing; with nice regard to preference.</def> <ldquo/A band of men collected <xex>choicely</xex>, from each county some.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a preferable or excellent manner; excellently; eminently.</def> <ldquo/<xex>Choicely</xex> good.<rdquo/  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choiceness</ent><br/
<hw>Choice"ness</hw>, <pos>n.</pos> <def>The quality of being of particular value or worth; nicely; excellence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choir</ent><br/
<hw>Choir</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>quer</ets>, OF. <ets>cuer</ets>, F. <ets>ch<oe/ur</ets>, fr. L. <ets>chorus</ets> a choral dance, chorus, choir, fr. Gr. <?/, orig. dancing place; prob. akin to <?/ inclosure, L. <ets>hortus</ets> garden, and E. <ets>yard</ets>. See <er>Chorus</er>.]</ety> <sn>1.</sn> <def>A band or organized company of singers, especially in church service.</def> <altsp>[Formerly written also <asp>quire</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That part of a church appropriated to the singers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>The chancel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Choir organ</b></col> <fld>(Mus.)</fld>, <cd>one of the three or five distinct organs included in the full organ, each separable from the rest, but all controlled by one performer; a portion of the full organ, complete in itself, and more practicable for ordinary service and in the accompanying of the vocal choir.</cd> -- <mcol><col><b>Choir screen</b></col>, <col><b>Choir wall</b></col></mcol> <fld>(Arch.)</fld>, <cd>a screen or low wall separating the choir from the aisles.</cd> -- <col><b>Choir service</b></col>, <cd>the service of singing performed by the choir.</cd>  <rj><au>T. Warton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>choirboy</ent><br/
<hw>choir"boy`</hw> <pos>n.</pos> <def>a boy who sings in a choir.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>choirmaster</ent><br/
<hw>choir"mas*ter</hw> <pos>n.</pos> <def>the musical director of a choir.</def><br/
<syn><b>Syn. --</b> precentor, cantor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Choke</ent><br/
<hw>Choke</hw> <pr>(ch<omac/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Choked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Choking</conjf>.]</vmorph> <ety>[OE. <ets>cheken</ets>, <ets>choken</ets>; cf. AS. <ets><amac/ceocian</ets> to suffocate, Icel. <ets>koka</ets> to gulp, E. <ets>chincough</ets>, <ets>cough</ets>.]</ety> <sn>1.</sn> <def>To render unable to breathe by filling, pressing upon, or squeezing the windpipe; to stifle; to suffocate; to strangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With eager feeding food doth <qex>choke</qex> the feeder.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To obstruct by filling up or clogging any passage; to block up.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hinder or check, as growth, expansion, progress, etc.; to stifle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Oats and darnel <qex>choke</qex> the rising corn.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To affect with a sense of strangulation by passion or strong feeling.</def> <ldquo/I was <xex>choked</xex> at this word.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To make a choke, as in a cartridge, or in the bore of the barrel of a shotgun.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To choke off</b></col>, <cd>to stop a person in the execution of a purpose; as, <xex>to choke off</xex> a speaker by uproar.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choke</ent><br/
<hw>Choke</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have the windpipe stopped; to have a spasm of the throat, caused by stoppage or irritation of the windpipe; to be strangled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be checked, as if by choking; to stick.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The words <qex>choked</qex> in his throat.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choke</ent><br/
<hw>Choke</hw>, <pos>n.</pos> <sn>1.</sn> <def>A stoppage or irritation of the windpipe, producing the feeling of strangulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gun.)</fld> <sd>(a)</sd> <def>The tied end of a cartridge.</def> <sd>(b)</sd> <def>A constriction in the bore of a shotgun, case of a rocket, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chokeberry</ent><br/
<hw>Choke"ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The small apple-shaped or pear-shaped fruit of an American shrub (<spn>Pyrus arbutifolia</spn>) growing in damp thickets; also, the shrub.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chokebore</ent><br/
<hw>Choke"bore`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>In a shotgun, a bore which is tapered to a slightly smaller diameter at a short distance (usually 2<frac12/ to 3 inches) to the rear of the muzzle, in order to prevent the rapid dispersion of the shot.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A shotgun that is made with such a bore.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chokebore</ent><br/
<hw>Choke"bore`</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chokebored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chokeboring</conjf>.]</vmorph> <def>To provide with a chokebore.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chokecherry</ent><br/
<hw>Choke"cher`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The astringent fruit of a species of wild cherry (<spn>Prunus Virginiana</spn>); also, the bush or tree which bears such fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choke damp</ent><br/
<hw>Choke" damp`</hw> <pr>(?)</pr>. <def>See <cref>Carbonic acid</cref>, under <er>Carbonic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chokedar</ent><br/
||<hw>Cho`ke*dar"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hindi <ets>chauk<imac/-d<amac/r</ets>.]</ety> <def>A watchman; an officer of customs or police.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choke-full</ent><br/
<hw>Choke"-full`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full to the brim; quite full; chock-full.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choke pear</ent><br/
<hw>Choke" pear`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>A kind of pear that has a rough, astringent taste, and is swallowed with difficulty, or which contracts the mucous membrane of the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sarcasm by which one is put to silence; anything that can not be answered.</def> <mark>[Low]</mark>  <rj><au>S. Richardson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choker</ent><br/
<hw>Chok"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, chokes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stiff wide cravat; a stock.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choke-strap</ent><br/
<hw>Choke"-strap`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Saddlery)</fld> <def>A strap leading from the bellyband to the lower part of the collar, to keep the collar in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choking</ent><br/
<hw>Chok"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>That chokes; producing the feeling of strangulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Indistinct in utterance, as the voice of a person affected with strong emotion.</def></p>

<p><ent>Choking coil</ent><br/
<hw>Choking coil</hw>. <fld>(Elec.)</fld> <def>A coil of small resistance and large inductance, used in an alternating-current circuit to impede or throttle the current, or to change its phase; -- called also <altname>reactance coil</altname> or <altname>reactor</altname>, these terms being now preferred in engineering usage.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chokey</ent><br/
<ent>Choky</ent><br/
<mhw>{ <hw>Chok"y</hw> <hw>Chok"ey</hw> }</mhw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Tending to choke or suffocate, or having power to suffocate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inclined to choke, as a person affected with strong emotion.</def> <ldquo/A deep and <xex>choky</xex> voice.<rdquo/  <rj><au>Aytoun.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The allusion to his mother made Tom feel rather <qex>chokey</qex>.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choky</ent><br/
<hw>Cho"ky</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From Hind. <ets>chauki</ets> watching, guard.]</ety> <sn>1.</sn>  <def>A station, as for collection of customs, for palanquin bearers, police, etc.</def> <mark>[India]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Specif., a prison or lockup; a jail.</def> <mark>[India, or Slang, Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cholaemia</ent><br/
||<hw>Cho*l<ae/"mi*a</hw> <pr>(k<osl/*l<emac/"m<icr/*<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>cholh`</grk> bile + <grk>a"i^ma</grk> blood.]</ety> <fld>(Med.)</fld> <def>A disease characterized by severe nervous symptoms, dependent upon the presence of the constituents of the bile in the blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholagogue</ent><br/
<hw>Chol"a*gogue</hw> <pr>(k<ocr/l"<adot/*g<ocr/g; 115)</pr>, <pos>a.</pos> <ety>[Gr. <grk>cholagwgo`s</grk>; <grk>cholh`</grk> bile + <grk>'agwgo`s</grk> leading, <grk>'a`gein</grk> to lead: cf. F. <ets>cholagogue</ets>.]</ety> <fld>(Med.)</fld> <def>Promoting the discharge of bile from the system.</def> -- <def2><pos>n.</pos> <def>An agent which promotes the discharge of bile from the system.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholate</ent><br/
<hw>Cho"late</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cholh`</grk> bile.]</ety> <fld>(Chem.)</fld> <def>A salt of cholic acid; <as>as, sodium <ex>cholate</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholecystis</ent><br/
||<hw>Chol`e*cys"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>cholh`</grk> bile + <?/ bladder.]</ety> <fld>(Anat.)</fld> <def>The gall bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholecystotomy</ent><br/
<hw>Chol`e*cys*tot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cholecystis</ets> + Gr. <grk>te`mnein</grk> to cut.]</ety> <fld>(Surg.)</fld> <def>The operation of making an opening in the gall bladder, as for the removal of a gallstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choledology</ent><br/
<hw>Chol`e*dol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cholh`</grk> bile + <ets>-logy</ets>. Cf. F. <ets>chol<eacute/dologie</ets>.]</ety> <fld>(Med.)</fld> <def>A treatise on the bile and bilary organs.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Littr<eacute/ says that the word <xex>chol<eacute/dologie</xex> is absolutely barbarous, there being no Greek word <?/. A proper form would be <xex>cholology</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choleic</ent><br/
<hw>Cho*le"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol. Chem.)</fld> <def>Pertaining to, or obtained from, bile; <as>as, <ex>choleic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choler</ent><br/
<hw>Chol"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>coler</ets>, F. <ets>col<egrave/re</ets> anger, L. <ets>cholera</ets> a bilious complaint, fr. Gr. <?/ cholera, fr. <?/, <grk>cholh`</grk>, bile. See <er>Gall</er>, and cf. <er>Cholera</er>.]</ety> <sn>1.</sn> <def>The bile; -- formerly supposed to be the seat and cause of irascibility.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His [Richard Hooker's] complexion . . . was sanguine, with a mixture of <qex>choler</qex>; and yet his motion was slow.</q> <rj><qau>I. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Irritation of the passions; anger; wrath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is rash and very sudden in <qex>choler</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholera</ent><br/
<hw>Chol"er*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a bilious disease. See <er>Choler</er>.]</ety> <fld>(Med.)</fld> <def>One of several diseases affecting the digestive and intestinal tract and more or less dangerous to life, esp. the one commonly called <xex>Asiatic cholera</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Asiatic cholera</b></col>, <cd>a malignant and rapidly fatal disease, originating in Asia and frequently epidemic in the more filthy sections of other lands, to which the germ or specific poison may have been carried. It is characterized by diarrhea, rice-water evacuations, vomiting, cramps, pinched expression, and lividity, rapidly passing into a state of collapse, followed by death, or by a stage of reaction of fever.</cd> -- <col><b>Cholera bacillus</b></col>. <cd>See <cref>Comma bacillus</cref>.</cd> -- <col><b>Cholera infantum</b></col>, <cd>a dangerous summer disease, of infants, caused by hot weather, bad air, or poor milk, and especially fatal in large cities.</cd> -- <col><b>Cholera morbus</b></col>, <cd>a disease characterized by vomiting and purging, with gripings and cramps, usually caused by imprudence in diet or by gastrointestinal disturbance.</cd> -- <col><b>Chicken cholera</b></col>. <cd>See under <er>Chicken</er>.</cd> -- <col><b>Hog cholera</b></col>. <cd>See under <er>Hog</er>.</cd> -- <col><b>Sporadic cholera</b></col>, <cd>a disease somewhat resembling the Asiatic cholera, but originating where it occurs, and rarely becoming epidemic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choleraic</ent><br/
<hw>Chol`er*a"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to, or resulting from, or resembling, cholera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choleric</ent><br/
<hw>Chol"er*ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cholericus</ets>, Gr. <?/: cf. F. <ets>chol<eacute/rique</ets>.]</ety> <sn>1.</sn> <def>Abounding with, or producing choler, or bile.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Easily irritated; irascible; inclined to anger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Angry; indicating anger; excited by anger.</def> <ldquo/<xex>Choleric</xex> speech.<rdquo/  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Choleric temperament</b></col>, <cd>the bilious temperament.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholericly</ent><br/
<hw>Chol"er*ic*ly</hw>, <pos>adv.</pos> <def>In a choleric manner; angrily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choleriform</ent><br/
<hw>Chol"er*i*form`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cholera</ets> + <ets>-form</ets>.]</ety> <def>Resembling cholera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholerine</ent><br/
<hw>Chol"er*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <sd>(a)</sd> <def>The precursory symptoms of cholera.</def> <sd>(b)</sd> <def>The first stage of epidemic cholera.</def> <sd>(c)</sd> <def>A mild form of cholera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choleroid</ent><br/
<hw>Chol"er*oid</hw>, <pos>a.</pos> <ety>[<ets>Cholera</ets> + <ets>-oid</ets>.]</ety> <def>Choleriform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholesteric</ent><br/
<hw>Cho`les*ter"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cholest<eacute/rique</ets>.]</ety> <def>Pertaining to cholesterin, or obtained from it; <as>as, <ex>cholesteric</ex> acid</as>.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholesterin</ent><br/
<hw>Cho*les"ter*in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ bile + <?/ stiff fat: F. <ets>cholest<eacute/rine</ets>. See <er>Stearin</er>.]</ety> <fld>(Chem.)</fld> <def>A white, fatty, crystalline substance, tasteless and odorless, found in animal and plant products and tissue, and especially in nerve tissue, in the bile, and in gallstones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choliambic</ent><br/
<ent>Choliamb</ent><br/
<mhw>{ <hw>Cho"li*amb</hw> <pr>(?)</pr>, <hw>Cho`li*am"bic</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>choliambus</ets>, Gr. <?/; <?/ lame + <?/ an iambus.]</ety> <fld>(Pros.)</fld> <def>A verse having an iambus in the fifth place, and a spondee in the sixth or last.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholinic</ent><br/
<ent>Cholic</ent><br/
<mhw>{ <hw>Chol"ic</hw> <pr>(?)</pr>, <hw>Cho*lin"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/, from <?/ bile.]</ety> <fld>(Physiol. Chem.)</fld> <def>Pertaining to, or obtained from, the bile.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cholic acid</b></col> <fld>(Chem.)</fld>, <cd>a complex organic acid found as a natural constituent of taurocholic and glycocholic acids in the bile, and extracted as a resinous substance, convertible under the influence of ether into white crystals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choline</ent><br/
<hw>Cho"line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ bile.]</ety> <fld>(Physiol. Chem.)</fld> <def>See <er>Neurine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholochrome</ent><br/
<hw>Chol"o*chrome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, bile + <?/ color.]</ety> <fld>(Physiol.)</fld> <def>See <er>Bilirubin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cholophaein</ent><br/
<hw>Chol`o*ph<ae/"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, bile + <?/ dusky.]</ety> <fld>(Physiol.)</fld> <def>See <er>Bilirubin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choltry</ent><br/
||<hw>Chol"try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Hindoo caravansary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chomage</ent><br/
||<hw>Cho`mage"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chomage</ets>.]</ety> <sn>1.</sn>  <def>Stoppage; cessation (of labor).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A standing still or idle (of mills, factories, etc.).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chomp</ent><br/
<hw>Chomp</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To chew loudly and greedily; to champ.</def> <mark>[Prov. Eng. & Colloq. U. S.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chondrichthian</ent><br/
<hw>chondrichthian</hw> <pos>n.</pos> <def>a fish in which the skeleton may be calcified but not ossified; a cartilaginous fish.</def><br/
<syn><b>Syn. --</b> cartilaginous fish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chondrichthyes</ent><br/
<hw>Chondrichthyes</hw> <pos>n.</pos> <def>the class of fishes comprising the cartilaginous fishes, which includes the sharks.</def><br/
<syn><b>Syn. --</b> class <class>Chondrichthyes</class>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chondrification</ent><br/
<hw>Chon`dri*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>Formation of, or conversion into, cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrify</ent><br/
<hw>Chon"dri*fy</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage + <ets>-fy</ets>.]</ety> <def>To convert, or be converted, into cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrigen</ent><br/
<hw>Chon"dri*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage + <ets>-gen</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>The chemical basis of cartilage, converted by long boiling in water into a gelatinous body called chondrin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrigenous</ent><br/
<hw>Chon*drig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage + <ets>-genous</ets>.]</ety> <fld>(Physiol.)</fld> <def>Affording chondrin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrin</ent><br/
<hw>Chon"drin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage.]</ety> <fld>(Physiol. Chem.)</fld> <def>A colorless, amorphous, nitrogenous substance, tasteless and odorless, formed from cartilaginous tissue by long-continued action of boiling water. It is similar to gelatin, and is a large ingredient of commercial gelatin.  See also <er>chondroitin sulfate</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Chondrite</ent><br/
<hw>Chon"drite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> a grain (of wheat or spelt), cartilage.]</ety> <fld>(Min.)</fld> <def>A meteoric stone characterized by the presence of chondrules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondritic</ent><br/
<hw>Chon*drit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Granular; pertaining to, or having the granular structure characteristic of, the class of meteorites called chondrites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondritis</ent><br/
||<hw>Chon*dri"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>cho`ndros</grk> cartilage + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>An inflammation of cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondro-</ent><br/
<hw>Chon"dro-</hw> <pr>(?)</pr>. <ety>[Gr. <grk>cho`ndros</grk> a grain (of wheat or spelt), cartilage.]</ety> <def>A combining form meaning <xex>a grain</xex>, <xex>granular</xex>, <xex>granular cartilage</xex>, <xex>cartilaginous</xex>; <as>as, the <ex>chondro</ex>cranium, the cartilaginous skull of the lower vertebrates and of embryos</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrodite</ent><br/
<hw>Chon"dro*dite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> a grain (of wheat or spelt), cartilage.]</ety> <fld>(Min.)</fld> <def>A fluosilicate of magnesia and iron, yellow to red in color, often occurring in granular form in a crystalline limestone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondroganoidea</ent><br/
||<hw>Chon`dro*ga*noi"de*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>cho`ndros</grk> cartilage + NL. <ets>ganoidei</ets>. See <er>Ganoid</er>.]</ety> <fld>(Zool.)</fld> <def>An order of ganoid fishes, including the sturgeons; -- so called on account of their cartilaginous skeleton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrogen</ent><br/
<hw>Chon"dro*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage + <ets>-gen</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>Same as <er>Chondrigen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrogenesis</ent><br/
<hw>Chon`dro*gen"e*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage + <ets>genesis</ets>.]</ety> <fld>(Physiol.)</fld> <def>The development of cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondroid</ent><br/
<hw>Chon"droid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage + <ets>-oid</ets>.]</ety> <def>Resembling cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chondroitin sulfate</ent><br/
<hw>chon*dro"it*in sul"fate</hw> <pr>(k<ocr/n*dr<omac/"<icr/*t<icr/n s<ucr/l"f<amac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>cho`ndros</grk> cartilage.]</ety> <fld>(Physiol. Chem.)</fld> <def>A colorless, amorphous, mucopolysaccharide having N-acetyl chondrosine as the repeating unit with one sulfate group per disaccharide.  Typical preparations have a molecular weight of about 50,000.  Preparations are sold over-the-counter, often referred to as <altname>chondroitin</altname>, with the putative ability to relieve pain in joints and assist joint cartilage growth or regeneration; such claims are, as of 2001, yet unproven.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chondrology</ent><br/
<hw>Chon*drol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ cartilage + <ets>-logy</ets>: cf. F. <ets>chondrologie</ets>.]</ety> <fld>(Anat.)</fld> <def>The science which treats of cartilages.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondroma</ent><br/
||<hw>Chon*dro"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chondromata</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ cartilage + <ets>-oma</ets>.]</ety> <def>A cartilaginous tumor or growth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrometer</ent><br/
<hw>Chon*drom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a grain (of wheat or spelt), cartilage + <ets>-meter</ets>.]</ety> <def>A steelyard for weighting grain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondropterygian</ent><br/
<hw>Chon*drop`ter*yg"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>chondropterygien</ets>.]</ety> <def>Having a cartilaginous skeleton.</def> -- <def2><pos>n.</pos> <def>One of the Chondropterygii.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondropterygii</ent><br/
||<hw>Chon*drop`te*ryg"i*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ cartilage + <?/, <?/, wing, fin.]</ety> <fld>(Zool.)</fld> <def>A group of fishes, characterized by cartilaginous fins and skeleton. It includes both ganoids (sturgeons, etc.) and selachians (sharks), but is now often restricted to the latter.</def> <altsp>[Written also <asp>Chondropterygia</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrostei</ent><br/
||<hw>Chon*dros"te*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ cartilage + <?/ bone.]</ety> <fld>(Zool.)</fld> <def>An order of fishes, including the sturgeons; -- so named because the skeleton is cartilaginous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrotomy</ent><br/
<hw>Chon*drot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ + <?/ a cutting.]</ety> <fld>(Anat.)</fld> <def>The dissection of cartilages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chondrule</ent><br/
<hw>Chon"drule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. from Gr. <?/ a grain (of wheat or spelt), cartilage.]</ety> <fld>(Min.)</fld> <def>A peculiar rounded granule of some mineral, usually enstatite or chrysolite, found imbedded more or less abundantly in the mass of many meteoric stones, which are hence called <xex>chondrites</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>choo-choo</ent><br/
<hw>choo-choo</hw> <pos>n.</pos> <def>a train or a locomotive; -- a child's word.</def> <mark>[Baby talk]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Choose</ent><br/
<hw>Choose</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Chose</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Chosen</conjf> <pr>(?)</pr>, <conjf>Chose</conjf> (Obs.); <pos>p. pr. & vb. n.</pos> <conjf>Choosing</conjf>.]</vmorph> <ety>[OE. <ets>chesen</ets>, <ets>cheosen</ets>, AS. <ets>ce<oacute/san</ets>; akin to OS. <ets>kiosan</ets>, D. <ets>kiezen</ets>, G. <ets>kiesen</ets>, Icel. <ets>kj<omac/sa</ets>, Goth. <ets>kiusan</ets>, L. <ets>gustare</ets> to taste, Gr. <?/, Skr. <ets>jush</ets> to enjoy. <root/46. Cf. <er>Choice</er>, 2d <er>Gust</er>.]</ety> <sn>1.</sn> <def>To make choice of; to select; to take by way of preference from two or more objects offered; to elect; <as>as, to <ex>choose</ex> the least of two evils</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Choose</qex> me for a humble friend.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wish; to desire; to prefer.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The landlady now returned to know if we did not <qex>choose</qex> a more genteel apartment.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To choose sides</b></col>. <cd>See under <er>Side</er>.</cd></cs></p>

<p><syn><b>Syn.</b> - To select; prefer; elect; adopt; follow.</syn> <usage> -- To <er>Choose</er>, <er>Prefer</er>, <er>Elect</er>. To <xex>choose</xex> is the generic term, and denotes to take or fix upon by an act of the will, especially in accordance with a decision of the judgment. To <xex>prefer</xex> is to choose or favor one thing as compared with, and more desirable than, another, or more in accordance with one's tastes and feelings. To <xex>elect</xex> is to choose or select for some office, employment, use, privilege, etc., especially by the concurrent vote or voice of a sufficient number of electors. To <xex>choose</xex> a profession; to <xex>prefer</xex> private life to a public one; to <xex>elect</xex> members of Congress.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choose</ent><br/
<hw>Choose</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To make a selection; to decide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They had only to <qex>choose</qex> between implicit obedience and open rebellion.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To do otherwise.</def> <ldquo/Can I <xex>choose</xex> but smile?<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Can not choose but</b></col>, <cd>must necessarily.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>canst not choose but</qex> know who I am.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chooser</ent><br/
<hw>Choos"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who chooses; one who has the power or right of choosing; an elector.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>choosy</ent><br/
<ent>choosey</ent><br/
<mhw><hw>choosey</hw> <hw>choosy</hw></mhw> <pos>adj.</pos>  <def>difficult to please, especially in details.</def><br/
<syn><b>Syn. --</b> particular, fussy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chopped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chopping</conjf>.]</vmorph> <ety>[Cf. LG. & D. <ets>kappen</ets>, Dan. <ets>kappe</ets>, Sw. <ets>kappa</ets>. Cf. <er>Chap</er> to crack.]</ety> <sn>1.</sn> <def>To cut by striking repeatedly with a sharp instrument; to cut into pieces; to mince; -- often with <ptcl>up</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 252 --></p>

<p><sn>2.</sn> <def>To sever or separate by one more blows of a sharp instrument; to divide; -- usually with <xex>off</xex> or <xex>down</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chop</qex> off your hand, and it to the king.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To seize or devour greedily; -- with <xex>up</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon the opening of his mouth he drops his breakfast, which the fox presently <qex>chopped</qex> up.</q> <rj><qau>L'estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To make a quick strike, or repeated strokes, with an ax or other sharp instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To do something suddenly with an unexpected motion; to catch or attempt to seize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Out of greediness to get both, he <qex>chops</qex> at the shadow, and loses the substance.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To interrupt; -- with <xex>in</xex> or <xex>out</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This fellow interrupted the sermon, even suddenly <qex>chopping</qex> in.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>v. t.</pos> <ety>[Cf. D. <ets>koopen</ets> to buy. See <er>Cheapen</er>, <pos>v. t.</pos>, and cf. <er>Chap</er>, <pos>v. i.</pos>, to buy.]</ety> <sn>1.</sn> <def>To barter or truck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exchange; substitute one thing for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We go on <qex>chopping</qex> and changing our friends.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To chop logic</b></col>, <cd>to dispute with an affected use of logical terms; to argue sophistically.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To purchase by way of truck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To vary or shift suddenly; <as>as, the wind <ex>chops</ex> about</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wrangle; to altercate; to bandy words.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not the counsel at the bar <qex>chop</qex> with the judge.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>n.</pos> <def>A change; a vicissitude.</def>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>v. t. & i.</pos> <def>To crack. See <er>Chap</er>, <pos>v. t. & i.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of chopping; a stroke.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece chopped off; a slice or small piece, especially of meat; <as>as, a mutton <ex>chop</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A crack or cleft. See <er>Chap</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>n.</pos> <ety>[See <er>Chap</er>.]</ety> <sn>1.</sn> <def>A jaw of an animal; -- commonly in the <pluf>pl.</pluf> See <er>Chops</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A movable jaw or cheek, as of a wooden vise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The land at each side of the mouth of a river, harbor, or channel; <as>as, East <ex>Chop</ex> or West <ex>Chop</ex></as>. See <er>Chops</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop</ent><br/
<hw>Chop</hw>, <pos>n.</pos> <ety>[Chin. & Hind. <ets>ch<amac/p</ets> stamp, brand.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Quality; brand; <as>as, silk of the first <ex>chop</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A permit or clearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chop dollar</b></col>, <cd>a silver dollar stamped to attest its purity.</cd> -- <col><b>chop of tea</b></col>, <cd>a number of boxes of the same make and quality of leaf.</cd> -- <col><b>Chowchow chop</b></col>. <cd>See under <er>Chowchow</er>.</cd> -- <col><b>Grand chop</b></col>, <cd>a ship's port clearance.</cd>  <rj><au>S. W. Williams.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopboat</ent><br/
<hw>Chop"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chin. <ets>chop</ets> sort, quality.]</ety> <def>A licensed lighter employed in the transportation of goods to and from vessels.</def> <mark>[China]</mark>  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopchurch</ent><br/
<hw>Chop"church`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chop</er> to barter.]</ety> <fld>(Old Eng. Law)</fld> <def>An exchanger or an exchange of benefices.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopfallen</ent><br/
<hw>Chop`fall`en</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the lower chop or jaw depressed; hence, crestfallen; dejected; dispirited; downcast. See <er>Chapfallen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chophouse</ent><br/
<hw>Chop"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house where chops, etc., are sold; an eating house.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The freedom of a <qex>chophouse</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chophouse</ent><br/
<hw>Chop"house`</hw>, <pos>n.</pos> <ety>[See <er>Chop</er> quality.]</ety> <def>A customhouse where transit duties are levied.</def> <mark>[China]</mark>  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopin</ent><br/
<hw>Chop"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chopine</ets>, fr. G. <ets>schoppen</ets>.]</ety> <def>A liquid measure formerly used in France and Great Britain, varying from half a pint to a wine quart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopin</ent><br/
<hw>Chop"in</hw>, <pos>n.</pos> <def>See <er>Chopine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopine</ent><br/
<hw>Cho*pine"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>chapin</ets>, <ets>escapin</ets>, Sp. <ets>chapin</ets>, Pg. <ets>chapim</ets>.]</ety> <def>A clog, or patten, having a very thick sole, or in some cases raised upon a stilt to a height of a foot or more.</def> <altsp>[Variously spelt <asp>chioppine</asp>, <asp>chopin</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Your ladyship is nearer to heaven than when I saw you last, by the altitude of a <qex>chopine</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chop-logic</ent><br/
<hw>Chop"-log`ic</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who bandies words or is very argumentative.</def> <mark>[Jocular]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopness</ent><br/
<hw>Chop"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of spade.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopper</ent><br/
<hw>Chop"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, chops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopping</ent><br/
<hw>Chop"ping</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Chubby</er>.]</ety> <def>Stout or plump; large.</def> <mark>[Obs.]</mark>  <rj><au>Fenton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopping</ent><br/
<hw>Chop"ping</hw>, <pos>a.</pos> <ety>[See <er>Chop</er> to barter.]</ety> <def>Shifting or changing suddenly, as the wind; also, having tumbling waves dashing against each other; <as>as, a <ex>chopping</ex> sea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chopping</ent><br/
<hw>Chop"ping</hw>, <pos>n.</pos> <def>Act of cutting by strokes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chopping block</b></col>, <cd>a solid block of wood on which butchers and others chop meat, etc.</cd> -- <col><b>Chopping knife</b></col>, <cd>a knife for chopping or mincing meat, vegetables, etc.; -- usually with a handle at the back of the blade instead of at the end.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choppy</ent><br/
<hw>Chop"py</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Chappy</er>.]</ety> <sn>1.</sn> <def>Full of cracks.</def> <ldquo/<xex>Choppy</xex> finger.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. <er>Chop</er> a change.]</ety> <def>Rough, with short, tumultuous waves; <as>as, a <ex>choppy</ex> sea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chops</ent><br/
<hw>Chops</hw> <pr>(ch<ocr/ps)</pr>, <pos>n. pl.</pos> <ety>[See <er>Chop</er> a jaw.]</ety> <sn>1.</sn> <def>The jaws; also, the fleshy parts about the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sides or capes at the mouth of a river, channel, harbor, or bay; <as>as, the <ex>chops</ex> of the English Channel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chopsticks</ent><br/
<hw>chop"sticks`</hw> <pr>(ch<ocr/p"st<icr/ks`)</pr>, <pos>n.</pos> <sing><singw>chopstick</singw>  <pr>(ch<ocr/p"st<icr/k`)</pr>,</sing> <sn>1.</sn> <def>a pair of slender sticks made of wood, ivory, plastic, etc., used chiefly by the Chinese and Japanese to lift food into the mouth while dining; -- also commonly used around the world by persons of Oriental heritage or in restaurants serving oriental food.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Chop sooy</ent><br/
<ent>Chop suey</ent><br/
<mhw>{ <hw>Chop su"ey</hw> <it>or</it> <hw>Chop soo"y</hw>  }</mhw> <pr>(?)</pr>. <ety>[Chin. (Cantonese) <ets>shap sui</ets> odds and ends, fr. <ets>shap</ets> for <ets>sap</ets> to enter the mouth + <ets>sui</ets> small bits pounded fine.]</ety> <def>A m<eacute/lange served in Chinese restaurants to be eaten with rice, noodles, etc. It consists typically of bean sprouts, onions, mushrooms, etc., and sliced meats, fried and flavored with sesame oil.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Choragic</ent><br/
<hw>Cho*rag"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/.]</ety> <def>Of or pertaining to a choragus.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Choragic monument</b></col>, <cd>a building or column built by a victorious choragus for the reception and exhibition of the tripod which he received as a prize. Those of Lysicrates and Thrasyllus are still to be seen at Athens.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choragus</ent><br/
||<hw>Cho*ra"gus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Choragi</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/, <?/; <?/ chorus + <?/ to lead.]</ety> <fld>(Gr. Antiq.)</fld> <def>A chorus leader; esp. one who provided at his own expense and under his own supervision one of the choruses for the musical contents at Athens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choral</ent><br/
<hw>Cho"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>choralis</ets>, fr. L. <ets>chorus</ets>. See <er>Chorus</er>.]</ety> <def>Of or pertaining to a choir or chorus; singing, sung, or adapted to be sung, in chorus or harmony.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Choral service</b></col>, <cd>a service of song.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>chorale</ent><br/
<ent>Choral</ent><br/
<mhw><hw>Cho"ral</hw>, <hw>chorale</hw></mhw>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A stately hymn tune; a simple sacred tune, sung in unison by the congregation, used mostly in Protestant (especially Lutheran) churches; <as>as, the Lutheran <ex>chorals</ex></as>.</def> <altsp>[Sometimes written <asp>chorale</asp>.]</altsp><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Choralist</ent><br/
<hw>Cho"ral*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A singer or composer of chorals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorally</ent><br/
<hw>Cho"ral*ly</hw>, <pos>adv.</pos> <def>In the manner of a chorus; adapted to be sung by a choir; in harmony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chord</ent><br/
<hw>Chord</hw> <pr>(k<ocir/rd)</pr>, <pos>n.</pos> <ety>[L <ets>chorda</ets> a gut, a string made of a gut, Gr. <grk>chordh`</grk>. In the sense of a string or small rope, in general, it is written <ets>cord</ets>. See <er>Cord</er>.]</ety> <sn>1.</sn> <def>The string of a musical instrument.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A combination of tones simultaneously performed, producing more or less perfect harmony, <as>as, the common <ex>chord</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>A right line uniting the extremities of the arc of a circle or curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anat.)</fld> <def>A cord. See <er>Cord</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Engin.)</fld> <def>The upper or lower part of a truss, usually horizontal, resisting compression or tension.</def>  <rj><au>Waddell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Accidental, Common, <and/ Vocal</b></col> <col><b>chords</b></col></mcol>. <cd>See under <er>Accidental</er>, <er>Common</er>, and <er>Vocal</er>.</cd> -- <col><b>Chord of an arch</b></col>. <cd>See <xex>Illust.</xex> of <er>Arch</er>.</cd> -- <col><b>Chord of curvature</b></col>, <cd>a chord drawn from any point of a curve, in the circle of curvature for that point.</cd> -- <col><b>Scale of chords</b></col>. <cd>See <er>Scale</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chord</ent><br/
<hw>Chord</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chorded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Chording</conjf>.]</vmorph> <def>To provide with musical chords or strings; to string; to tune.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When Jubal struck the <qex>chorded</qex> shell.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Even the solitary old pine tree <qex>chords</qex> his harp.</q> <rj><qau>Beecher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chord</ent><br/
<hw>Chord</hw>, <pos>v. i.</pos> <fld>(Mus.)</fld> <def>To accord; to harmonize together; <as>as, this note <ex>chords</ex> with that</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorda</ent><br/
||<hw>Chor"da</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>chorda</ets>. See <er>Chord</er>.]</ety> <fld>(Anat.)</fld> <def>A cord.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Chorda dorsalis</b></col> <pr>(<?/)</pr>. <ety>[NL., lit., cord of the back.]</ety> <fld>(Anat.)</fld> <cd>See <er>Notochord</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chordal</ent><br/
<hw>Chor"dal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a chord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chordomesoderm</ent><br/
<ent>chordamesoderm</ent><br/
<mhw><hw>chordamesoderm</hw>, <hw>chordomesoderm</hw></mhw> <pos>n.</pos> <fld>(Embryology)</fld> <def>the area of the mesoderm that forms the notochord.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chordata</ent><br/
||<hw>Chor*da"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>chorda</ets> cord.]</ety> <fld>(Zool.)</fld> <def>A comprehensive division of animals including all Vertebrata together with the Tunicata, or all those having a dorsal nervous cord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chordee</ent><br/
<hw>Chor*dee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cord<eacute/</ets>, <ets>cord<eacute/e</ets>, p. p. of <ets>corder</ets> to cord.]</ety> <fld>(Med.)</fld> <def>A painful erection of the penis, usually with downward curvature, occurring in gonorrhea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chordophone</ent><br/
<hw>chordophone</hw> <pos>n.</pos> <fld>(Music)</fld> <def>a stringed instrument of the group including harps, lutes, lyres, and zithers.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chordospartium</ent><br/
<hw>Chordospartium</hw> <pos>n.</pos> <def>a genus containing two species of small New Zealand trees: weeping tree broom; endangered.</def><br/
<syn><b>Syn. --</b> genus <gen>Chordospartium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chore</ent><br/
<hw>Chore</hw> <pr>(ch<omac/r)</pr>, <pos>n.</pos> <ety>[The same word as <ets>char</ets> work done by the day.]</ety> <def>A small job; in the <pluf>pl.</pluf>, the regular or daily light work of a household or farm, either within or without doors.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chore</ent><br/
<hw>Chore</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Choring</conjf>.]</vmorph> <def>To do chores.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chore</ent><br/
<hw>Chore</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A choir or chorus.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorea</ent><br/
||<hw>Cho*re"a</hw> <pr>(?)</pr>. <pos>n.</pos> <ety>[NL., fr. Gr. <?/ dance.]</ety> <fld>(Med.)</fld> <def>St. Vitus's dance; a disease attended with convulsive twitchings and other involuntary movements of the muscles or limbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choree</ent><br/
<hw>Cho*ree"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chor<eacute/e</ets>.]</ety> <def>See <er>Choreus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choregraphical</ent><br/
<ent>Choregraphic</ent><br/
<mhw>{ <hw>Cho`re*graph"ic</hw> <pr>(?)</pr>, <hw>Cho`re*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Pertaining to choregraphy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choregraphy</ent><br/
<hw>Cho*reg"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ dance + <ets>-graphy</ets>.]</ety> <sn>1.</sn> <def>The art of representing dancing by signs, as music is represented by notes; -- also called <altname>choreography</altname>.</def>  <rj><au>Craig.</au></rj> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Choreic</ent><br/
<hw>Cho*re"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of, or pertaining to, chorea; convulsive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choreography</ent><br/
<hw>Chor`e*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ dance + <ets>-graphy</ets>.]</ety> <sn>1.</sn> <def>The art of representing dancing by signs, as music is represented by notes; -- also called <altname>choregraphy</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>the art of composing dances for individuals or groups, including the planning of the movements and steps; also, the planning of movements and steps for figure skaters, performed on ice.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>the planning and coordination of activities for an event, especially one to be held in public.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Chorepiscopal</ent><br/
<hw>Cho`re*pis"co*pal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a chorepiscopus or his charge or authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorepiscopus</ent><br/
||<hw>Cho`re*pis"co*pus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chorepiscopi</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/; <grk>chw^ros</grk>, <grk>chw`ra</grk>, place, country + <?/ bishop. Cf. <er>Bishop</er>.]</ety> <fld>(Eccl.)</fld> <def>A <ldquo/country<rdquo/ or suffragan bishop, appointed in the ancient church by a diocesan bishop to exercise episcopal jurisdiction in a rural district.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choree</ent><br/
<ent>Choreus</ent><br/
<mhw>||<hw>Cho*re"us</hw> <pr>(?)</pr>, <hw>Cho*ree"</hw> <pr>(<?/)</pr></mhw>, <pos>n.</pos> <ety>[L. <ets>choreus</ets>, Gr. <?/, prop. an adj. meaning belonging to a chorus; cf. F. <ets>chor<eacute/e</ets>.]</ety> <fld>(Anc. Pros.)</fld> <sd>(a)</sd> <def>a trochee.</def> <sd>(b)</sd> <def>A tribrach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choriamb</ent><br/
<hw>Cho"ri*amb</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Choriambs</plw> <pr>(<?/)</pr>.</plu> <def>Same as <er>Choriambus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choriambic</ent><br/
<hw>Cho`ri*am"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>choriambicus</ets>, gr. <?/.]</ety> <def>Pertaining to a choriamb.</def> -- <def2><pos>n.</pos> <def>A choriamb.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choriambus</ent><br/
<hw>Cho`ri*am"bus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Choriambi</plw> <pr>(#)</pr>, E. <plw>Choriambuses</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>choriambus</ets>, Gr. <?/; <?/ a choreus + <?/ iambus.]</ety> <fld>(Anc. Pros.)</fld> <def>A foot consisting of four syllables, of which the first and last are long, and the other short (- <cre/ <cre/ -); that is, a choreus, or trochee, and an iambus united.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choric</ent><br/
<hw>Cho"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>choricus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to a chorus.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I remember a <qex>choric</qex> ode in the Hecuba.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chorine</ent><br/
<hw>chorine</hw> <pos>n.</pos> <def>a woman who dances in a chorus line.</def><br/
<syn><b>Syn. --</b> chorus girl, showgirl.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chorioallantois</ent><br/
<hw>chorioallantois</hw> <pos>n.</pos> <def>a very vascular fetal membrane composed of the fused chorion and adjacent wall of the allantois.</def><br/
<syn><b>Syn. --</b> chorioallantoic membrane.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chorion</ent><br/
||<hw>Cho"ri*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The outer membrane which invests the fetus in the womb; also, the similar membrane investing many ova at certain stages of development.</def> <sd>(b)</sd> <def>The true skin, or cutis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The outer membrane of seeds of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorisis</ent><br/
||<hw>Cho"ri*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ separation.]</ety> <fld>(Bot.)</fld> <def>The separation of a leaf or floral organ into two more parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In <xex>collateral chorisis</xex> the parts are side by side. -- In <xex>parallel</xex> or <xex>median chorisis</xex> they are one in front of another.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorist</ent><br/
<hw>Cho"rist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>choriste</ets>.]</ety> <def>A singer in a choir; a chorister.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorister</ent><br/
<hw>Chor"is*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chorus</er>.]</ety> <sn>1.</sn> <def>One of a choir; a singer in a chorus.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who leads a choir in church music.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choristic</ent><br/
<hw>Cho*ris"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Choric; choral.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorograph</ent><br/
<hw>Cho"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ place + <ets>-graph</ets>.]</ety> <def>An instrument for constructing triangles in marine surveying, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorographer</ent><br/
<hw>Cho*rog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who describes or makes a map of a district or region.</def> <ldquo/The <xex>chorographers</xex> of Italy.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A geographical antiquary; one who investigates the locality of ancient places.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorographical</ent><br/
<hw>Cho`ro*graph"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to chorography.</def> -- <wordforms><wf>Cho`ro*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorography</ent><br/
<hw>Cho*rog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>chorographia</ets>, Gr. <?/; <?/ place + <?/ to describe.]</ety> <def>the mapping or description of a region or district.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>chorography</qex> of their provinces.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choroid</ent><br/
<hw>Cho"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[gr. <?/; <?/ chorion + <grk>e'i^dos</grk> form.]</ety> <fld>(Anat.)</fld> <def>resembling the chorion; <as>as, the <ex>choroid</ex> plexuses of the ventricles of the brain, and the <ex>choroid</ex> coat of the eyeball</as>.</def> -- <def2><pos>n.</pos> <def>The choroid coat of the eye. See <er>Eye</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Choroid plexus</b></col> <fld>(Anat.)</fld>, <cd>one of the delicate fringelike processes, consisting almost entirely of blood vessels, which project into the ventricles of the brain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choroidal</ent><br/
<hw>Cho*roid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to the choroid coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorology</ent><br/
<hw>Cho*rol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ place + <ets>-logy</ets>.]</ety> <fld>(Biol.)</fld> <def>The science which treats of the laws of distribution of living organisms over the earth's surface as to latitude, altitude, locality, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Its distribution or <qex>chorology</qex>.</q> <rj><qau>Huxley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorometry</ent><br/
<hw>Cho*rom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ place + <ets>-metry</ets>.]</ety> <def>The art of surveying a region or district.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chortle</ent><br/
<hw>Chor"tle</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chortled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chor"tling</conjf> <pr>(<?/)</pr>.]</vmorph> <def>A word coined by <person>Lewis Carroll</person> (<person>Charles L. Dodgson</person>), and usually explained as a combination of <xex>chuckle</xex> and <xex>snort</xex>.</def> <mark>[Humorous]</mark></p>

<p><q>O frabjous day ! Callooh ! Callay !<br/
He <qex>chortled</qex> in his joy.</q>  <rj><qau>Lewis Carroll.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chorus</ent><br/
<hw>Cho"rus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Choruses</plw> <pr>(#)</pr>.</plu> <ety>[L., a dance in a ring, a dance accompanied with song; a chorus, a band of dancers and singers. Gr. <?/. See <er>Choir</er>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Antiq.)</fld> <def>A band of singers and dancers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Grecian tragedy was at first nothing but a <qex>chorus</qex> of singers.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gr. Drama)</fld> <def>A company of persons supposed to behold what passed in the acts of a tragedy, and to sing the sentiments which the events suggested in couplets or verses between the acts; also, that which was thus sung by the chorus.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What the lofty, grave tragedians taught<br/
In <qex>chorus</qex> or iambic.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An interpreter in a dumb show or play.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>A company of singers singing in concert.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>A composition of two or more parts, each of which is intended to be sung by a number of voices.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>Parts of a song or hymn recurring at intervals, as at the end of stanzas; also, a company of singers who join with the singer or choir in singer or choir in singing such parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The simultaneous of a company in any noisy demonstration; <as>as, a <ex>Chorus</ex> of shouts and catcalls</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chorus</ent><br/
<hw>Cho"rus</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chorused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chorusing</conjf>.]</vmorph> <def>To sing in chorus; to exclaim simultaneously.</def>  <rj><au>W. D. Howells.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chose</ent><br/
||<hw>Chose</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Choses</plw> <pr>(#)</pr>.</plu> <ety>[F., fr. L. <ets>causa</ets> cause, reason. See <er>Cause</er>.]</ety> <fld>(Law)</fld> <def>A thing; personal property.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chose in action</b></col>, <cd>a thing of which one has not possession or actual enjoyment, but only a right to it, or a right to demand it by action at law, and which does not exist at the time in specie; a personal right to a thing not reduced to possession, but recoverable by suit at law; as a right to recover money due on a contract, or damages for a tort, which can not be enforced against a reluctant party without suit.</cd> -- <col><b>Chose in possession</b></col>, <cd>a thing in possession, as distinguished from a thing <xex>in action</xex>.</cd> -- <col><b>Chose local</b></col>, <cd>a thing annexed to a place, as a mill.</cd> -- <col><b>Chose transitory</b></col>, <cd>a thing which is movable.</cd>  <rj><au>Cowell. Blount.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chose</ent><br/
<hw>Chose</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Choose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chosen</ent><br/
<hw>Cho"sen</hw> <pr>(?)</pr>, <pos>p. p.</pos> <mord>of <er>Choose</er></mord>. <def>Selected from a number; picked out; choice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seven hundred <qex>chosen</qex> men left-handed.</q> <rj><qau>Judg. xx. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chosen</ent><br/
<hw>Cho"sen</hw>, <pos>n.</pos> <def>One who, or that which is the object of choice or special favor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chou</ent><br/
||<hw>Chou</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Choux</plw> <pr>(#)</pr></plu>. <ety>[F., fr. L. <ets>caulis</ets> stalk.]</ety>  <sn>1.</sn> <def>A cabbage.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of light pastry, usually in the form of a small round cake, and with a filling, as of jelly or cream.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A bunch, knot, or rosette of ribbon or other material, used as an ornament in women's dress.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chouan</ent><br/
<hw>Chou"an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>One of the royalist insurgents in western France (Brittany, etc.), during and after the French revolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chough</ent><br/
<hw>Chough</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>choughe</ets>, <ets>kowe</ets> (and cf. OE. <ets>ca</ets>), fr. AS. <ets>ce<oacute/</ets>; cf. also D. <ets>kauw</ets>, OHG. <ets>ch<amac/ha</ets>; perh. akin to E. <ets>caw</ets>. <root/22. Cf. <er>Caddow</er>.]</ety> <fld>(Zool.)</fld> <def>A bird of the Crow family (<spn>Fregilus graculus</spn>) of Europe. It is of a black color, with a long, slender, curved bill and red legs; -- also called <altname>chauk</altname>, <altname>chauk-daw</altname>, <altname>chocard</altname>, <altname>Cornish chough</altname>, <altname>red-legged crow</altname>. The name is also applied to several allied birds, as the <altname>Alpine chough</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cornish chough</b></col> <fld>(Her.)</fld>, <cd>a bird represented black, with red feet, and beak; -- called also <altname>aylet</altname> and <altname>sea swallow</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chouicha</ent><br/
||<hw>Chou"i*cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name]</ety> <fld>(Zool.)</fld> <def>The salmon of the Columbia River or California. See <er>Quinnat</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chouka</ent><br/
||<hw>Chou"ka</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name]</ety> <fld>(Zool.)</fld> <def>The Indian four-horned antelope; the chikara.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choule</ent><br/
<hw>Choule</hw> <pr>(?)</pr>, <pos>n.</pos> <mark>[Obs.]</mark> <def>See <er>Jowl</er>.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choultry</ent><br/
||<hw>Choul"try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Choltry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chouse</ent><br/
<hw>Chouse</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Choused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chousing</conjf>.]</vmorph> <ety>[From Turk. <ets>ch<amac/<umac/sh</ets> a messenger or interpreter, one of whom, attached to the Turkish embassy, in 1609 cheated the Turkish merchants resident in England out of <pound/4,000.]</ety> <def>To cheat, trick, defraud; -- followed by <xex>of</xex>, or <xex>out of</xex>; <as>as, to <ex>chouse</ex> one out of his money</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The undertaker of the afore-cited poesy hath <qex>choused</qex> your highness.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chouse</ent><br/
<hw>Chouse</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who is easily cheated; a tool; a simpleton; a gull.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A trick; sham; imposition.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A swindler.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chout</ent><br/
||<hw>Chout</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Mahratta <ets>chauth</ets> one fourth part.]</ety> <def>An assessment equal to a fourth part of the revenue.</def> <mark>[India]</mark>  <rj><au>J. Mill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chow</ent><br/
<hw>Chow</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chin. <ets>chou</ets>.]</ety> <sn>1.</sn> <def>A prefecture or district of the second rank in China, or the chief city of such a district; -- often part of the name of a city, as in <xex>Foochow</xex>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a breed of thick-coated medium-sized dogs with fluffy curled tails and distinctive blue-black tongues; -- same as <er>chowchow{3}</er>, <pos>n.</pos>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chowchow</ent><br/
<hw>Chow"chow`</hw> <pr>(chou"chou`)</pr>, <pos>a.</pos> <ety>[Chin.]</ety> <def>Consisting of several kinds mingled together; mixed; <as>as, <ex>chowchow</ex> sweetmeats (preserved fruits put together)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chow-chow</ent><br/
<hw>chow-chow</hw> <pr>(chou"chou`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>chopped pickles in mustard sauce.</def><br/
<syn><b>Syn. --</b> chowchow.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a Chinese preserve of mixed fruits and ginger.</def><br/
<syn><b>Syn. --</b> chowchow.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>a breed of thick-coated medium-sized dogs with fluffy curled tails and distinctive blue-black tongues; believed to have originated in north China.</def><br/
<syn><b>Syn. --</b> chow, chow chow.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 253 --></p>

<p><cs><col><b>Chowchow chop</b></col>, <cd>the last lighter containing the small sundry packages sent off to fill up a ship.</cd>  <rj><au>S. W. Williams.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chowchow</ent><br/
<hw>Chow"chow`</hw> <pr>(chou"chou`)</pr>, <pos>n.</pos> <fld>(Com.)</fld> <def>A kind of mixed pickles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chowder</ent><br/
<hw>Chow"der</hw> <pr>(chou"d<etil/r)</pr>, <pos>n.</pos> <ety>[F. <ets>chaudi<egrave/re</ets> a kettle, a pot. Cf. <er>Caldron</er>.]</ety> <sn>1.</sn> <fld>(Cookery)</fld> <def>A dish made of fresh fish or clams, biscuit, onions, etc., stewed together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A seller of fish.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chowder beer</b></col>, <cd>a liquor made by boiling black spruce in water and mixing molasses with the decoction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chowder</ent><br/
<hw>Chow"der</hw>, <pos>v. t.</pos> <def>To make a chowder of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chowry</ent><br/
||<hw>Chow"ry</hw> <pr>(chou"r<ycr/)</pr>, <pos>n.</pos> <ety>[Hind. <ets>chaunri</ets>.]</ety> <def>A whisk to keep off files, used in the East Indies.</def>  <rj><au>Malcom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chowter</ent><br/
<hw>Chow"ter</hw> <pr>(chou"t<etil/r)</pr>, <pos>v. t.</pos> <ety>[Cf. OE. <ets>chowre</ets>, and Prov. E. <ets>chow</ets>, to grumble.]</ety> <def>To grumble or mutter like a froward child.</def> <mark>[Obs.]</mark>  <rj><au>E. Phillips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Choy root</ent><br/
<hw>Choy" root`</hw> <pr>(choi" r<oomac/t`)</pr>. <def>See <er>Chay root</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrematistics</ent><br/
<hw>Chre`ma*tis"tics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ 9sc. <?/) the art of traffic, fr. <?/ goods, money, fr. <?/ to use.]</ety> <def>The science of wealth; the science, or a branch of the science, of political economy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chreotechnics</ent><br/
<hw>Chre`o*tech"nics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ useful + <?/ art.]</ety> <def>The science of the useful arts, esp. agriculture, manufactures, and commerce.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrestomathic</ent><br/
<hw>Chres`to*math"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Teaching what is useful.</def> <ldquo/A <xex>chrestomathic</xex> school.<rdquo/  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrestomathy</ent><br/
<hw>Chres*tom"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ useful + <?/, <?/, to learn.]</ety> <def>A selection of passages, with notes, etc., to be used in acquiring a language; <as>as, a Hebrew <ex>chrestomathy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrism</ent><br/
<hw>Chrism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>crisme</ets>, from AS. <ets>crisma</ets>; also OE. <ets>creme</ets>, fr. OF. <ets>cresme</ets>, like the AS. word fr. LL. <ets>chrisma</ets>, fr. Gr. <?/, fr. <?/ to anoint; perh. akin to L. <ets>friare</ets>, <ets>fricare</ets>, to rub, Skr. <ets>gharsh</ets>, E. <ets>friable</ets>, <ets>friction</ets>. Cf. <er>Chrisom</er>.]</ety> <fld>(Gr. & R. C. Church<?/s)</fld><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Olive oil mixed with balm and spices, consecrated by the bishop on Maundy Thursday, and used in the administration of baptism, confirmation, ordination, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The same as <er>Chrisom</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrismal</ent><br/
<hw>Chris"mal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>chrismalis</ets>.]</ety> <def>Of or pertaining to or used in chrism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrismation</ent><br/
<hw>Chris*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>chrismatio</ets>.]</ety> <def>The act of applying the chrism, or consecrated oil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chrismation</qex> or cross-signing with ointment, was used in baptism.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrismatory</ent><br/
<hw>Chris"ma*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>chrismatorium</ets>.]</ety> <def>A cruet or vessel in which chrism is kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrisom</ent><br/
<hw>Chris"om</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chrism</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A white cloth, anointed with chrism, or a white mantle thrown over a child when baptized or christened.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A child which died within a month after its baptism; -- so called from the chrisom cloth which was used as a shroud for it.</def> <mark>[Obs.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christ</ent><br/
<hw>Christ</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Christus</ets>, Gr. <?/, fr. <?/ anointed, fr. <grk>chri`ein</grk> to anoint. See <er>Chrism</er>.]</ety> <def><sc>The Anointed</sc>; an appellation given to Jesus, the Savior.  It is synonymous with the Hebrew <er>Messiah</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christcross</ent><br/
<hw>Christ"cross`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The mark of the cross, as cut, painted, written, or stamped on certain objects, -- sometimes as the sign of 12 o'clock on a dial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fescue of the dial is upon the <qex>christcross</qex> of noon.</q> <rj><qau>Old Play.   Nares.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The beginning and the ending.</def> <mark>[Obs.]</mark>  <rj><au>Quarles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christcross-row</ent><br/
<hw>Christ"cross-row`</hw> <pr>(?)</pr>, <def>The alphabet; -- formerly so called, either from the cross usually set before it, or from a superstitious custom, sometimes practiced, of writing it in the form of a cross, by way of a charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From infant conning of the <qex>Christcross-row</qex>.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>christella</ent><br/
<hw>christella</hw> <pos>n.</pos> <def>any of several tropical ferns of the genus <gen>Christella</gen> having thin brittle fronds.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Christen</ent><br/
<hw>Chris"ten</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Christened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Christening</conjf>.]</vmorph> <ety>[AS. <ets>cristnian</ets> to make a Christian, fr. <ets>cristen</ets> a Christian.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To baptize and give a Christian name to.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give a name; to denominate.</def> <ldquo/<xex>Christen</xex> the thing what you will.<rdquo/  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To Christianize.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To use for the first time.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christendom</ent><br/
<hw>Chris"ten*dom</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cristend<omac/m</ets>; <ets>cristen</ets> a Christian + <ets>-dom</ets>.]</ety> <sn>1.</sn> <def>The profession of faith in Christ by baptism; hence, the Christian religion, or the adoption of it.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The name received at baptism; or, more generally, any name or appelation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pretty, fond, adoptious <qex>christendoms</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That portion of the world in which Christianity prevails, or which is governed under Christian institutions, in distinction from heathen or Mohammedan lands.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Arian doctrine which then divided <qex>Christendom</qex>.</q> <rj><qau>Milton</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A wide and still widening <qex>Christendom</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The whole body of Christians.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christian</ent><br/
<hw>Chris"tian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>christianus</ets>, Gr. <?/; cf. AS. <ets>cristen</ets>. See <er>Christ</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>One who believes, or professes or is assumed to believe, in Jesus Christ, and the truth as taught by Him; especially, one whose inward and outward life is conformed to the doctrines of Christ.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The disciples were called <qex>Christians</qex> first in Antioch.</q> <rj><qau>Acts xi. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One born in a Christian country or of Christian parents, and who has not definitely becomes an adherent of an opposing system.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <sd>(a)</sd> <def>One of a Christian denomination which rejects human creeds as bases of fellowship, and sectarian names. They are congregational in church government, and baptize by immersion. They are also called <altname>Disciples of Christ</altname>, and <altname>Campbellites</altname>.</def> <sd>(b)</sd> <def>One of a sect (called <altname>Christian Connection</altname>) of open-communion immersionists. The Bible is their only authoritative rule of faith and practice.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, often pronounced, but not by the members of the sects, <pr>kr<imac/s"ch<it>a</it>n</pr>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christian</ent><br/
<hw>Chris"tian</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to Christ or his religion; <as>as, <ex>Christian</ex> people</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pertaining to the church; ecclesiastical; <as>as, a <ex>Christian</ex> court</as>.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Characteristic of Christian people; civilized; kind; kindly; gentle; beneficent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The graceful tact; the <qex>Christian</qex> art.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Christian Commission</b></col>. <cd>See under <er>Commission</er>.</cd> -- <col><b>Christian court</b></col>. <cd>Same as <er>Ecclesiastical court</er>.</cd>  -- <col><b>Christian Endeavor, Young People's Society of</b></col>. <cd>In various Protestant churches, a society of young people organized in each individual church to do Christian work; also, the whole body of such organizations, which are united in a corporation called the United Society of Christian Endeavor, organized in 1885. The parent society was founded in 1881 at Portland, Maine, by Rev. Francis E. Clark, a Congregational minister.</cd> [<source>Webster 1913 Suppl.</source>] -- <col><b>Christian era</b></col>, <cd>the present era, commencing with the birth of Christ. It is supposed that owing to an error of a monk (Dionysius Exiguus, d. about 556) employed to calculate the era, its commencement was fixed three or four years too late, so that 1890 should be 1893 or 1894.</cd> -- <col><b>Christian name</b></col>, <cd>the name given in baptism, as distinct from the family name, or surname.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christian Era</ent><br/
<hw>Christian Era</hw>. <def>The era in use in all Christian countries, which was intended to commence with the birth of Christ. The era as now established was first used by Dionysius Exiguus (died about 540), who placed the birth of Christ on the 25th of December in the year of Rome 754, which year he counted as 1 <sc>a. d.</sc> This  date for Christ's birth is now generally thought to be about four years too late.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Christianism</ent><br/
<hw>Chris`tian*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>christianismus</ets>, Gr. <?/: cf. F. <ets>christianisme</ets>.]</ety> <sn>1.</sn> <def>The Christian religion.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The Christian world; Christendom.</def> <mark>[Obs.]</mark>  <rj><au>Johnson</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianite</ent><br/
<hw>Chris"tian*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[In sense (<sd>a</sd>) named after <ets>Christian</ets> Frederic, of Denmark; in sense (<sd>b</sd>) after <ets>Christian</ets> VII., of Denmark.]</ety> <fld>(Min.)</fld> <sd>(a)</sd> <def>Same as <er>Anorthite</er>.</def> <mark>[R.]</mark> <sd>(b)</sd> <def>See <er>Phillipsite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianity</ent><br/
<hw>Chris*tian"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>cristiente</ets>, OF. <ets>cristient<eacute/</ets>, F. <ets>chr<eacute/tient<eacute/</ets>, fr. L. <ets>christianitas</ets>. ]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The religion of Christians; the system of doctrines and precepts taught by Christ.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Practical conformity of one's inward and outward life to the spirit of the Christian religion</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The body of Christian believers.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To Walys fled the <qex>christianitee</qex><br/
Of olde Britons.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianization</ent><br/
<hw>Chris`tian*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of converting or being converted to a true Christianity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianize</ent><br/
<hw>Chris"tian*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Christianized</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Christianizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>christianiser</ets>, L. <ets>christianizare</ets>, fr. Gr. <?/.]</ety> <sn>1.</sn> <def>To make Christian; to convert to Christianity; <as>as, to <ex>Christianize</ex> pagans</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imbue with or adapt to Christian principles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Christianized</qex> philosophers.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianize</ent><br/
<hw>Chris"tian*ize</hw>, <pos>v. i.</pos> <def>To adopt the character or belief of a Christian; to become Christian.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pagans began to <qex>Christianize</qex>.</q> <rj><qau>Latham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianlike</ent><br/
<hw>Chris"tian*like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Becoming to a Christian.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A virtuous and a <qex>Christianlike</qex> conclusion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianly</ent><br/
<hw>Chris"tian*ly</hw>, <pos>adv.</pos> <def>In a manner becoming the principles of the Christian religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sufferings . . . patiently and <qex>Christianly</qex> borne.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianly</ent><br/
<hw>Chris"tian*ly</hw>, <pos>a.</pos> <def>Christianlike.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christianness</ent><br/
<hw>Chris"tian*ness</hw>, <pos>n.</pos> <def>Consonance with the doctrines of Christianity.</def> <mark>[Obs.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christian Science</ent><br/
<hw>Christian Science</hw>. <def>A system of healing disease of mind and body which teaches that all cause and effect is mental, and that sin, sickness, and death will be destroyed by a full understanding of the Divine Principle of Jesus' teaching and healing. The system was founded by Rev. Mary Baker Glover Eddy, of Concord, N. H., in 1866, and bases its teaching on the Scriptures as understood by its adherents.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Christian Scientist</ent><br/
<hw>Christian Scientist</hw>. <def>A believer in Christian Science; one who practices its teachings.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Christian Seneca</ent><br/
<hw>Christian Seneca</hw>. <def>Joseph Hall (1574 -- 1656), Bishop of Norwich, a divine eminent as a moralist.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Christian Socialism</ent><br/
<hw>Christian Socialism</hw>. <def>Any theory or system that aims to combine the teachings of Christ with the teachings of socialism in their applications to life; Christianized socialism; esp., the principles of this nature advocated by F. D. Maurice, Charles Kingsley, and others in England about 1850.</def> -- <wordforms><wf>Christian socialist</wf>.</wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Christless</ent><br/
<hw>Christ"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without faith in Christ; unchristian.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christlike</ent><br/
<hw>Christ"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling Christ in character, actions, etc.</def> -- <wordforms><wf>Christ"like`ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christly</ent><br/
<hw>Christ"ly</hw>, <pos>a.</pos> <def>Christlike.</def>  <rj><au>H. Bushnell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christmas</ent><br/
<hw>Christ"mas</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Christ</ets> + <ets>mass</ets>.]</ety> <def>An annual church festival (December 25) and in some States a legal holiday, in memory of the birth of Christ, often celebrated by a particular church service, and also by special gifts, greetings, and hospitality.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Christmas box</b></col>. <sd>(a)</sd> <cd>A box in which presents are deposited at Christmas.</cd> <sd>(b)</sd> <cd>A present or small gratuity given to young people and servants at Christmas; a Christmas gift.</cd> -- <col><b>Christmas carol</b></col>, <cd>a carol sung at, or suitable for, Christmas.</cd> -- <col><b>Christmas day</b></col>. <cd>Same as <er>Christmas</er>.</cd> -- <col><b>Christmas eve</b></col>, <cd>the evening before Christmas.</cd> -- <col><b>Christmas fern</b></col> <fld>(Bot.)</fld>, <cd>an evergreen North American fern (<spn>Aspidium acrostichoides</spn>), which is much used for decoration in winter.</cd> -- <mcol><col><b>Christmas flower</b></col>, <col><b>Christmas rose</b></col></mcol>, <cd>the black hellebore, a poisonous plant of the buttercup family, which in Southern Europe often produces beautiful roselike flowers midwinter.</cd> -- <col><b>Christmas tree</b></col>, <cd>a small evergreen tree, set up indoors, to be decorated with bonbons, presents, etc., and illuminated on Christmas eve.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christmasberry</ent><br/
<hw>Christmasberry</hw> <pos>n.</pos> <sn>1.</sn> <def>a spiny evergreen shrub of southeastern U. S. (<spn>Lycium carolinianum</spn>) having spreading branches with usually blue or mauve flowers and red berries.</def><br/
<syn><b>Syn. --</b> Christmas berry, <spn>Lycium carolinianum</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an ornamental evergreen treelike shrub of US Pacific coast (<spn>Photinia arbutifolia</spn>) having large white flowers and red berrylike fruits; often placed in genus Photinia.</def><br/
<syn><b>Syn. --</b> toyon, tollon, Christmas berry, Heteromeles arbutifolia, <spn>Photinia arbutifolia</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Christmastide</ent><br/
<hw>Christ"mas*tide`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Christmas</ets> + <ets>tide</ets> time.]</ety> <def>The season of Christmas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christocentric</ent><br/
<hw>Chris"to*cen"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Christ</ets> + <ets>centric</ets>.]</ety> <def>Making Christ the center, about whom all things are grouped, as in religion or history; tending toward Christ, as the central object of thought or emotion.</def>  <rj><au>J. W. Chadwick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christology</ent><br/
<hw>Chris*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Crist</ets> + <ets>-logy</ets>.]</ety> <def>A treatise on Christ; that department of theology which treats of the personality, attributes, or life of Christ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christom</ent><br/
<hw>Chris"tom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chrisom</er>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christophany</ent><br/
<hw>Chris*toph"a*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Christ</ets> + Gr. <?/ to show.]</ety> <def>An appearance of Christ, as to his disciples after the crucifixion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Christ's-thorn</ent><br/
<hw>Christ's-thorn`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>One of several prickly or thorny shrubs found in <geog>Palestine</geog>, especially the <spn>Paliurus aculeatus</spn>, <spn>Zizyphus Spina-Christi</spn>, and <spn>Zizyphus vulgaris</spn>.  The last bears the fruit called <prod>jujube</prod>, and may be considered to have been the most readily obtainable for the Crown of Thorns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromascope</ent><br/
<hw>Chro"ma*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <ets>-scope</ets>.]</ety> <def>An instrument for showing the optical effects of color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromate</ent><br/
<hw>Chro"mate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chromate</ets>. See <er>Chrome</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of chromic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatic</ent><br/
<hw>Chro*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>chromaticus</ets>, Gr. <?/, suited for color, fr. <?/, <?/, color; akin to <?/ color, <?/ skin, color of the skin.]</ety> <sn>1.</sn> <def>Relating to color, or to colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Proceeding by the smaller intervals (half steps or semitones) of the scale, instead of the regular intervals of the diatonic scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The intermediate tones were formerly written and printed in colors.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chromatic aberration</b></col>. <fld>(Opt.)</fld> <cd>See <er>Aberration</er>, <er>4</er>.</cd> -- <col><b>Chromatic printing</b></col>, <cd>printing from type or blocks covered with inks of various colors.</cd> -- <col><b>Chromatic scale</b></col> <fld>(Mus.)</fld>, <cd>the scale consisting of thirteen tones, including the eight scale tones and the five intermediate tones.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatical</ent><br/
<hw>Chro*mat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Chromatic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatically</ent><br/
<hw>Chro*mat"ic*al*ly</hw>, <pos>adv.</pos> <def>In a chromatic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chromaticity</ent><br/
<hw>chromaticity</hw> <pos>n.</pos> <def>the quality of a color as determined by its dominant wavelength.</def><br/
<syn><b>Syn. --</b> hue.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chromatics</ent><br/
<hw>Chro*mat"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science of colors; that part of optics which treats of the properties of colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chromatid</ent><br/
<hw>chromatid</hw> <pos>n.</pos> <fld>(Cell Biol.)</fld> <def>one of two identical strands into which a chromosome splits during mitosis.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chromatin</ent><br/
<hw>Chro"ma*tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, color.]</ety> <sn>1.</sn> <fld>(Biol.)</fld> <def>Tissue which is capable of being stained by dyes.</def> <mark>[archaic]</mark> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatin</ent><br/
<hw>Chro"ma*tin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>The deeply staining substance of the nucleus and chromosomes of eukaryotic cells, composed of DNA and basic proteins (such as histones), the DNA of which comprises the predominant physical basis of inheritance.  It was, at the beginning of the 20th century, supposed to be the same substance as was then termed <altname>idioplasm</altname> or <altname>germ plasm</altname>.  In most eukaryotic cells, there is also DNA in certain plasmids, such as mitochondria, or (in plant cells) chloroplasts; but with the exception of these cytoplasmic genetic factors, the nuclear DNA of the <ex>chromatin</ex> is believed to contain all the genetic information required to code for the development of an adult organism.  In the interphase nucleus the chromosomes are dispersed, but during cell division or meiosis they are condensed into the individually recognizable chromosomes.  The set of chromosomes, or a photographic representation of the full set of chromosomes of a cell (often ordered for presentation) is called a <partof>karyotype</partof>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chromatism</ent><br/
<hw>Chro"ma*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a coloring.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Optics)</fld> <def>The state of being colored, as in the case of images formed by a lens.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>An abnormal coloring of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatogenous</ent><br/
<hw>Chro`ma*tog"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, color + <ets>-genous</ets>.]</ety> <def>Producing color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chromatogram</ent><br/
<hw>chro*mat"o*gram</hw> <pos>n.</pos> <sn>1.</sn> <def>the paper strip, column, gel, or TLC plate on which subsances have been separated by a process of chromatography{2}.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the record of the results of a process of chromatography{2}, produced by an instrument which measures some property related to the concentration of substances over the course of a chromatographic process.  The record may be printed, for example, on a piece of paper, showing the concentration of some substance as a function of time or volume; or it may be retained in a computer and displayed on a video display terminal.</def> <note>The analytical methods for determining which substances have moved to which point on the chromatogram are very varied, and the records of the results of a visualization techique, such as autoradiography, may itself be referred to as a <ex>chromatogram</ex></note>.<br/
[<source>PJC</source>]</p>

<p><ent>chromatograph</ent><br/
<hw>chro*mat"o*graph`</hw> <pos>n.</pos> <def>a piece of equipment used to perform chromatography{2}.</def><br/
[<source>PJC</source>]</p>

<p><ent>chromatographical</ent><br/
<ent>chromatographic</ent><br/
<mhw><hw>chro*mat`o*graph"ic</hw> <hw>chro*mat`o*graph"ic*al</hw></mhw> <pos>adj.</pos> <def>of or pertaining to chromatography.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chromatography</ent><br/
<hw>Chro`ma*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, color + <ets>-graphy</ets>.]</ety> <sn>1.</sn> <def>A treatise on colors</def> <mark>[archaic]</mark> <br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>an analytical and preparative technique for separating substances by differences in their selective adsorption to solids, by passing a liquid over the solid, to which the substances to be separated have usually been adsorbed in a preliminary step.  The major variations are <styp><ecol><b>column chromatography</b></ecol></styp>, in which the substances to be separated are adsorbed to a column with any of a wide variety of adsorbing solids in powdered or granulated form; <styp><ecol><b>paper chromatography</b></ecol></styp>, in which the solids are applied as a spot at one end of a strip of absorbent paper (such as filter paper), and the liquid is percolated through the paper by capillary action; and <styp><ecol><b>thin-layer chromatography</b></ecol></styp> (TLC), which is similar to paper chromatography, but the adsorbent material is, instead of paper, a thin layer of finely powdered material, such as cellulose or silica, on a backing of glass or plastic, called a <uses>TLC plate</uses>.  A modern version of <stype>column chromatography</stype> is <styp><ecol><b>high-performance liquid chromatography</b></ecol></styp>, usually referred to as HPLC.</def><br/
[<source>PJC</source>]</p>

<p><ent>Chromatology</ent><br/
<hw>Chro`ma*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, color + <ets>-logy</ets>.]</ety> <def>A treatise on colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatophore</ent><br/
<hw>Chro"ma*to*phore`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, color + <grk>fe`rein</grk> to bear.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A contractile cell or vesicle containing liquid pigment and capable of changing its form or size, thus causing changes of color in the translucent skin of such animals as possess them. They are highly developed and numerous in the cephalopods.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>One of the granules of protoplasm, which in mass give color to the part of the plant containing them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatoscope</ent><br/
<hw>Chro"ma*to*scope`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, color + <ets>-scope</ets>.]</ety> <fld>(Astron.)</fld> <def>A reflecting telescope, part of which is made to rotate eccentrically, so as to produce a ringlike image of a star, instead of a point; -- used in studying the scintillation of the stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatosphere</ent><br/
<hw>Chro"ma*to*sphere`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chromosphere.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatrope</ent><br/
<hw>Chro"ma*trope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <?/ turn, rotation, <?/  to turn.]</ety> <sn>1.</sn> <fld>(Physics)</fld> <def>An instrument for exhibiting certain chromatic effects of light (depending upon the persistence of vision and mixture of colors) by means of rapidly rotating disks variously colored.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A device in a magic lantern or stereopticon to produce kaleidoscopic effects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromatype</ent><br/
<hw>Chro"ma*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <?/ type.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Photog.)</fld> <def>A colored photographic picture taken upon paper made sensitive with potassium bichromate or some other salt of chromium.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The process by which such picture is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrome</ent><br/
<hw>Chrome</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Chromium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chrome alum</b></col> <fld>(Chem.)</fld>, <cd>a dark violet substance, <chform>(SO4)3Cr2.K2SO4.24H2O</chform>, analogous to, and crystallizing like, common alum. It is regarded as a double sulphate of chromium and potassium.</cd> -- <col><b>Chrome green</b></col> <sd>(a)</sd> <cd>The green oxide of chromium, <chform>Cr2O3</chform>, used in enamel painting, and glass staining.</cd> <sd>(b)</sd> <cd>A pigment made by mixing chrome yellow with Prussian blue.</cd> -- <col><b>Chrome red</b></col>, <cd>a beautiful red pigment originally prepared from the basic chromate of lead, but now made from red oxide of lead.</cd> -- <col><b>Chrome yellow</b></col>, <cd>a brilliant yellow pigment, <chform>PbCrO4</chform>, used by painters.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrome</ent><br/
<hw>Chrome</hw> <pr>(kr<omac/m)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chromed</conjf> <pr>(kr<omac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chroming</conjf>.]</vmorph> <ety>[From <er>Chrome</er>, <pos>n.</pos>]</ety> <def>To treat with a solution of potassium bichromate, as in dyeing.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>chromesthesia</ent><br/
<hw>chromesthesia</hw> <pos>n.</pos> <def>a form of synesthesia in which nonvisual stimulation results in the experience of color sensations.</def> <altsp>[Also spelled <asp>chromaesthesia</asp>.]</altsp><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrome steel</ent><br/
<hw>Chrome steel</hw>. <def>Same as <cref>Chromium steel</cref>, under <er>Steel</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chromic</ent><br/
<hw>Chro"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or obtained from, chromium; -- said of the compounds of chromium in which it has its higher valence.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chromic acid</b></col>, <cd>an acid, <chform>H2CrO4</chform>, analogous to sulphuric acid, not readily obtained in the free state, but forming well known salts, many of which are colored pigments, as chrome yellow, chrome red, etc.</cd> -- <col><b>Chromic anhydride</b></col>, <cd>a brilliant red crystalline substance, <chform>CrO3</chform>, regarded as the anhydride of chromic acid. It is one of the most powerful oxidizers known.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromid</ent><br/
<hw>Chro"mid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a kind of fish.]</ety> <fld>(Zool.)</fld> <def>One of the <fam>Chromid<ae/</fam>, a family of fresh-water fishes abundant in the tropical parts of America and Africa. Some are valuable food fishes, as the <stype>bulti</stype> of the Nile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromidrosis</ent><br/
||<hw>Chro`mi*dro"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ color + <?/ sweat.]</ety> <fld>(Med.)</fld> <def>Secretion of abnormally colored perspiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromism</ent><br/
<hw>Chro"mism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Chromatism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromite</ent><br/
<hw>Chro"mite</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Min.)</fld> <def>A black submetallic mineral consisting of oxide of chromium and iron; -- called also <altname>chromic iron</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A compound or salt of chromous hydroxide regarded as an acid.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromium</ent><br/
<hw>Chro"mi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ color.]</ety> <fld>(Chem.)</fld> <def>A comparatively rare element occurring most abundantly in the mineral chromite. Atomic weight 52.5.  Symbol Cr.  When isolated it is a hard, brittle, grayish white metal, fusible with difficulty. Its chief commercial importance is for its compounds, as potassium chromate, lead chromate, etc., which are brilliantly colored and are used dyeing and calico printing. Called also <altname>chrome</altname>.</def>
<-- used as an ingredient in stainless steel, or chrome steel.   The yellow pigments are also used in paints, as in painting yellow stripes on macadam highways. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromo</ent><br/
<hw>Chro"mo</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chromos</plw> <pr>(#)</pr>.</plu> <ety>[Abbrev. from <ets>chromo</ets>lithograph.]</ety> <def>A chromolithograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromoblast</ent><br/
<hw>Chro"mo*blast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <ets>-blast</ets>.]</ety> <def>An embryonic cell which develops into a pigment cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromogen</ent><br/
<hw>Chro"mo*gen</hw> <pr>(?)</pr>, <ety>[Gr. <?/ color + <ets>-gen</ets>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Biol.)</fld> <def>Vegetable coloring matter other than green; chromule.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Any colored compound, supposed to contain one or more chromophores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromogenic</ent><br/
<hw>Chro"mo*gen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Containing, or capable of forming, chromogen; <as>as, <ex>chromogenic</ex> bacteria</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromograph</ent><br/
<hw>Chro"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <ets>-graph</ets>.]</ety> <def>An apparatus by which a number of copies of {ritten katter, kaps, plons, etc., can be made; -- called also <altname>hectograph</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 254 --></p>

<p><ent>Chromoleucite</ent><br/
<hw>Chro`mo*leu"cite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + E. <ets>leucite</ets>.]</ety> <fld>(Bot.)</fld> <def>A chromoplastid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromolithograph</ent><br/
<hw>Chro`mo*lith"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + E. <ets>lithograph</ets>.]</ety> <def>A picture printed in tints and colors by repeated impressions from a series of stones prepared by the lithographic process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromolithographer</ent><br/
<hw>Chro`mo*li*thog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is engaged in chromolithography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromolithographic</ent><br/
<hw>Chro"mo*lith`o*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertjining tj, or maoe by, coromolithography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromolithography</ent><br/
<hw>Chro"mo*li*thog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Lithography adapted to printing in inks of various colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromophane</ent><br/
<hw>Chro"mo*phane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <?/ to show.]</ety> <fld>(Physiol.)</fld> <def>A general name for the several coloring matters, red, green, yellow, etc., present in the inner segments in the cones of the retina, held in solution by fats, and slowly decolorized by light; distinct from the photochemical pigments of the rods of the retina.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromophore</ent><br/
<hw>Chro"mo*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <grk>fe`rein</grk> to bear.]</ety> <fld>(Chem.)</fld> <def>Any chemical group or residue (as <chform>NO2</chform>; <chform>N2</chform>; or <chform>O2</chform>) which imparts some decided color to the compound of which it is an ingredient.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromophotograph</ent><br/
<hw>Chro`mo*pho"to*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <ets>photograph</ets>.]</ety> <def>A picture made by any of the processes for reproducing photographs in colors; a color photograph.</def> -- <wordforms><wf>Chro`mo*pho`to*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Chromophotography</ent><br/
<hw>Chro`mo*pho*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + E. <ets>photography</ets>.]</ety> <def>The art of producing photographs in colors-</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromophotolithograph</ent><br/
<hw>Chro"mo*pho`to*lith"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A photolithograph printed in colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromoplastid</ent><br/
<hw>Chro`mo*plas"tid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ + E. <ets>plastid</ets>.]</ety> <fld>(Bot.)</fld> <def>A protoplasmic granule of some other color than green; -- also called <altname>chromoleucite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromosome</ent><br/
<hw>Chro"mo*some`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <?/ the body.]</ety> <fld>(Biol.)</fld> <def>One of the minute bodies into which the chromatin of the nucleus is resolved during mitotic cell division; the <altname>idant</altname> of Weismann.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromosphere</ent><br/
<hw>Chro"mo*sphere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + E. <ets>sphere</ets>.]</ety> <fld>(Astron.)</fld> <def>An atmosphere of rare matter, composed principally of incandescent hydrogen gas, surrounding the sun and enveloping the photosphere. Portions of the chromosphere are here and there thrown up into enormous tongues of flame.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromospheric</ent><br/
<hw>Chro`mo*spher"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the chromosphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromotype</ent><br/
<hw>Chro"mo*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <ets>-type</ets>.]</ety> <sn>1.</sn> <def>A sheet printed in colors by any process, as a chromolithograph. See <er>Chromolithograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A photographic picture in the natural colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromous</ent><br/
<hw>Chro"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or derived from, chromium, when this element has a valence lower than that in chromic compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chromous acid</b></col>, <cd>a bluish gray powder, CrO.OH, of weak acid properties and regard as an acid.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chromule</ent><br/
<hw>Chro"mule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ color + <?/ matter.]</ety> <fld>(Bot.)</fld> <def>A general name for coloring matter of plants other than chlorophyll, especially that of petals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronic</ent><br/
<hw>Chron"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>chronicus</ets>, Gr. <?/ concerning time, from <?/ time: cf. F. <ets>chronique</ets>.]</ety> <sn>1.</sn> <def>Relating to time; according to time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Continuing for a long time; lingering; habitual.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chronic disease</b></col>, <cd>one which is inveterate, of long continuance, or progresses slowly, in distinction from an <xex>acute</xex> disease, which speedly terminates.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronical</ent><br/
<hw>Chron"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Chronic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Partly on a <qex>chronical</qex>, and partly on a topical method.</q> <rj><qau>J. A. Alexander.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronicle</ent><br/
<hw>Chron"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>cronicle</ets>, fr. <ets>cronique</ets>, OF. <ets>cronique</ets>, F. <ets>chronique</ets>, L. <ets>chronica</ets>, fr. Gr. <?/, neut. pl. of <?/. See <er>Chronic</er>.]</ety> <sn>1.</sn> <def>An historical register or account of facts or events disposed in the order of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A narrative of events; a history; a record.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>The two canonical books of the Old Testament in which immediately follow 2 Kings.</def></p>

<p><syn><b>Syn.</b> - Register; record; annals. See <er>History</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronicle</ent><br/
<hw>Chron"i*cle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chronicled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chronicling</conjf> <pr>(?)</pr>.]</vmorph> <def>To record in a history or chronicle; to record; to register.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronicler</ent><br/
<hw>Chron"i*cler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A writer of a chronicle; a recorder of events in the order of time; an historian.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such an honest <qex>chronicler</qex> as Griffith.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronique</ent><br/
||<hw>Chro`nique"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Chronicle</er>.]</ety> <def>A chronicle.</def>  <rj><au>L. Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronogram</ent><br/
<hw>Chron"o*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ time + <?/ writing, character: cf. F. <ets>chronogramme</ets>.]</ety> <sn>1.</sn> <def>An inscription in which certain numeral letters, made to appear specially conspicuous, on being added together, express a particular date or epoch, <as>as in the motto of a medal struck by Gustavus Adolphus in 1632:
            ChrIstVs DVX; ergo trIVMphVs.
- the capitals of which give, when added as numerals, the sum 1632</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The record or inscription made by a chronograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronogrammatical</ent><br/
<ent>Chronogrammatic</ent><br/
<mhw>{ <hw>Chron`o*gram*mat"ic</hw> <pr>(?)</pr>, <hw>Chron`o*gram*mat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>chronogrammatique</ets>.]</ety> <def>Belonging to a chronogram, or containing one.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronogrammatist</ent><br/
<hw>Chron`o*gram"ma*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A writer of chronograms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronograph</ent><br/
<hw>Chron"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ time + <ets>-graph</ets>: cf. F. <ets>chronographe</ets>.]</ety> <sn>1.</sn> <def>An instrument for measuring or recording intervals of time, upon a revolving drum or strip of paper moved by clockwork. The action of the stylus or pen is controlled by electricity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Same as <er>Chronogram</er>, 1.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A chronoscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronographer</ent><br/
<hw>Chro*nog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who writes a chronography; a chronologer.</def>  <rj><au>Tooke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronographic</ent><br/
<hw>Chron`o*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a chronograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronography</ent><br/
<hw>Chro*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/. See <er>Chronograph</er>.]</ety> <def>A description or record of past time; history.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronologer</ent><br/
<hw>Chro*nol"o*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Chronologist</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronological</ent><br/
<ent>Chronologic</ent><br/
<mhw>{ <hw>Chron`o*log"ic</hw> <pr>(?)</pr>, <hw>Chron`o*log"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/.]</ety> <def>Relating to chronology; containing an account of events in the order of time; according to the order of time; <as>as, <ex>chronological</ex> tables</as>.</def> <au>Raleigh.</au> -- <wordforms><wf>Chron`o*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronologer</ent><br/
<ent>Chronologist</ent><br/
<mhw>{ <hw>Chro*nol"o*gist</hw> <pr>(?)</pr>, <hw>Chro*nol"o*ger</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Gr. <?/.]</ety> <def>A person who investigates dates of events and transactions; one skilled in chronology.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That learned noise and dust of the <qex>chronologist</qex> is wholly to be avoided.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>THe most exact <qex>chronologers</qex> tell us that Christ was born in October, and not in December.</q> <rj><qau>John Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronology</ent><br/
<hw>Chro*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chronologies</plw> <pr>(#)</pr>.</plu> <ety>[Gr. <?/; <?/ time + <?/ discourse: cf. F. <ets>chronologie</ets>.]</ety> <def>The science which treats of measuring time by regular divisions or periods, and which assigns to events or transactions their proper dates.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If history without <qex>chronology</qex> is dark and confused, <qex>chronology</qex> without history is dry and insipid.</q> <rj><qau>A. Holmes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronometer</ent><br/
<hw>Chro*nom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ time + <ets>-meter</ets>: cf. F. <ets>chronom<egrave/tre</ets>.]</ety> <sn>1.</sn> <def>An instrument for measuring time; a timekeeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A portable timekeeper, with a heavy compensation balance, and usually beating half seconds; -- intended to keep time with great accuracy for use an astronomical observations, in determining longitude, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A metronome.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Box chronometer</b></col>. <cd>See under <er>Box</er>.</cd> -- <col><b>Pocket chronometer</b></col>, <cd>a chronometer in the form of a large watch.</cd> -- <col><b>To rate a chronometer</b></col>. <cd>See <er>Rate</er>, <pos>v. t.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronometrical</ent><br/
<ent>Chronometric</ent><br/
<mhw>{ <hw>Chron`o*met"ric</hw> <pr>(?)</pr>, <hw>Chron`o*met"ric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>chronom<eacute/trique</ets>.]</ety> <def>Pertaining to a chronometer; measured by a chronometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronometry</ent><br/
<hw>Chro*nom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>chronom<eacute/trie</ets>.]</ety> <def>The art of measuring time; the measuring of time by periods or divisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronopher</ent><br/
<hw>Chron"o*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ time + <?/ to carry.]</ety> <def>An instrument signaling the correct time to distant points by electricity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chronophotograph</ent><br/
<hw>Chron`o*pho"to*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ time + <ets>photograph</ets>.]</ety> <def>One of a set of photographs of a moving object, taken for the purpose of recording and exhibiting successive phases of the motion.</def> -- <wordforms><wf>Chron`o*pho*tog"ra*phy</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chronoscope</ent><br/
<hw>Chron"o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ time + <ets>-scope</ets>.]</ety> <def>An instrument for measuring minute intervals of time; used in determining the velocity of projectiles, the duration of short-lived luminous phenomena, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysalid</ent><br/
<hw>Chrys"a*lid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a chrysalis; resembling a chrysalis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysalid</ent><br/
<hw>Chrys"a*lid</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chrysalids</plw>.</plu> <def>See <er>Chrysalis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysalis</ent><br/
<hw>Chrys"a*lis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Chrysalides</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>chrysallis</ets> the gold-colored pupa of butterflies, Gr. <?/, fr. <?/  gold. Cf. <er>Aurelia</er>.]</ety> <fld>(Zool.)</fld> <def>The pupa state of certain insects, esp. of butterflies, from which the perfect insect emerges. See <er>Pupa</er>, and <er>Aurelia</er> <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysaniline</ent><br/
<hw>Chrys*an"i*line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + E. <ets>anilene</ets>.]</ety> <fld>(Chem.)</fld> <def>A yellow substance obtained as a by-product in the manufacture of rosaniline. It dyes silk a fine golden-yellow color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysanthemum</ent><br/
<hw>Chrys*an"the*mum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/; <grk>chryso`s</grk> gold + <?/ flower.]</ety> <fld>(Bot.)</fld> <def>A genus of composite plants, mostly perennial, and of many species including the many varieties of garden chrysanthemums (annual and perennial), and also the feverfew and the oxeye daisy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysarobin</ent><br/
<hw>Chrys`a*ro"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <ets>araroba</ets> a foreign name of Goa powder + <ets>-in</ets>.]</ety> <fld>(Chem.)</fld> <def>A bitter, yellow substance forming the essential constituent of Goa powder, and yielding chrysophanic acid proper; hence formerly called also <altname>chrysphanic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysaurin</ent><br/
<hw>Chrys*au"rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + L. <ets>aurum</ets> gold. So called from its color.]</ety> <def>An orange-colored dyestuff, of artificial production.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chryselephantine</ent><br/
<hw>Chrys`el*e*phan"tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <?/ made of ivory, fr. <?/ ivory, elephant.]</ety> <def>Composed of, or adorned with, gold and ivory.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>chryselephantine</xex> statues of the Greeks were built up with inferior materials, veneered, as it were, with ivory for the flesh, and gold decorated with color for the hair and garments.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysene</ent><br/
<hw>Chry"sene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold.]</ety> <fld>(Chem.)</fld> <def>One of the higher aromatic hydrocarbons of coal tar, allied to naphthalene and anthracene. It is a white crystalline substance, <chform>C18H12</chform>, of strong blue fluorescence, but generally colored yellow by impurities.</def><-- ##err in orig, "naphthalene" was misspelled "napthalene". --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysoberyl</ent><br/
<hw>Chrys"o*ber`yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>chrysoberyllus</ets>, Gr. <?/; <grk>chryso`s</grk> gold + <?/ beryl.]</ety> <fld>(Min.)</fld> <def>A mineral, found in crystals, of a yellow to green or brown color, and consisting of aluminia and glucina. It is very hard, and is often used as a gem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysochlore</ent><br/
<hw>Chrys"o*chlore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <grk>chlwro`s</grk> light green: cf. F. <ets>chrysochlore</ets>.]</ety> <fld>(Zool.)</fld> <def>A South African mole of the genus <gen>Chrysochloris</gen>; the golden mole, the fur of which reflects brilliant metallic hues of green and gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysocolla</ent><br/
<hw>Chrys"o*col`la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>chryso`kolla</grk> gold solder; <grk>chryso`s</grk> gold + <?/ glue.]</ety> <fld>(Min.)</fld> <def>A hydrous silicate of copper, occurring massive, of a blue or greenish blue color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysogen</ent><br/
<hw>Chrys"o*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <ets>-gen</ets>.]</ety> <fld>(Chem.)</fld> <def>A yellow crystalline substance extracted from crude anthracene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysography</ent><br/
<hw>Chry*sog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>chryso`s</grk> gold + <?/ to write.]</ety> <sn>1.</sn> <def>The art of writing in letters of gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A writing executed in letters of gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysoidine</ent><br/
<hw>Chrys*o"<ium/*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <ets>-oid</ets> + <ets>-ine</ets>.]</ety> <fld>(Chem.)</fld> <def>An artificial, yellow, crystalline dye, <chform>C6H5N2.C6H3(NH2)2</chform>. Also, one of a group of dyestuffs resembling chryso<ium/dine proper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysolite</ent><br/
<hw>Chrys"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>chrysolithos</ets>, Gr. <?/; <grk>chryso`s</grk> gold + <?/ stone: cf. F. <ets>chrysolithe</ets>.]</ety> <fld>(Min.)</fld> <def>A mineral, composed of silica, magnesia, and iron, of a yellow to green color. It is common in certain volcanic rocks; -- called also <altname>olivine</altname> and <altname>peridot</altname>. Sometimes used as a gem. The name was also early used for yellow varieties of tourmaline and topaz.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysology</ent><br/
<hw>Chry*sol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <ets>-logy</ets>.]</ety> <def>That branch of political economy which relates to the production of wealth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysopa</ent><br/
||<hw>Chrys*o"pa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>chryso`s</grk> gold + <?/, <?/, eye, face.]</ety> <fld>(Zool.)</fld> <def>A genus of neuropterous insects. See <er>Lacewing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysophane</ent><br/
<hw>Chrys"o*phane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <?/ to show.]</ety> <fld>(Chem.)</fld> <def>A glucoside extracted from rhubarb as a bitter, yellow, crystalline powder, and yielding chrysophanic acid on decomposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysophanic</ent><br/
<hw>Chrys`o*phan"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or derived from, or resembling, chrysophane.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chrysophanic acid</b></col> <fld>(Chem.)</fld>, <cd>a yellow crystalline substance extracted from rhubarb, yellow dock, sienna, chrysarobin, etc., and shown to be a derivative of an anthracene. It is used in the treatment of skin diseases; -- called also <altname>rhein</altname>, <altname>rheic acid</altname>, <altname>rhubarbarin</altname>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysophrys</ent><br/
<hw>Chrysophrys</hw> <pos>n.</pos> <def>a genus of fishes consisting of the australian snapper.</def><br/
<syn><b>Syn. --</b> genus <gen>Chrysophrys</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysophyceae</ent><br/
<hw>Chrysophyceae</hw> <pos>n.</pos> <def>a class of yellow-green algae, all of which have flagella of unequal length.</def><br/
<syn><b>Syn. --</b> class <class>Chrysophyceae</class>, Heterokontae, class <class>Heterokontae</class>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysophyllum</ent><br/
<hw>Chrysophyllum</hw> <pos>n.</pos> <def>a genus of tropical American evergreen trees or shrubs.</def><br/
<syn><b>Syn. --</b> genus <gen>Chrysophyllum</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysophyta</ent><br/
<hw>Chrysophyta</hw> <pos>n.</pos> <def>a division of mostly freshwater eukaryotic algae having the chlorophyll masked by brown or yellow pigment; yellow-green and golden-brown algae and diatoms, including the Xanthophyceae, Chrysophyceae, Bacillariophyceae; some some classification systems superseded or subsumed by Heterokontophyta.</def><br/
<syn><b>Syn. --</b> division Chrysophyta.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chrysopid</ent><br/
<hw>chrysopid</hw> <pos>n.</pos> <def>a pale green unpleasant-smelling lacewing fly having carnivorous larvae.</def><br/
<syn><b>Syn. --</b> green lacewing, stink fly.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysopidae</ent><br/
<hw>Chrysopidae</hw> <pos>n.</pos> <def>a natural family of flies comprising the green lacewings.</def><br/
<syn><b>Syn. --</b> family <fam>Chrysopidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysoprase</ent><br/
<hw>Chrys"o*prase</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>crisopace</ets>, OF. <ets>crisoprace</ets>, F. <ets>chrysoprase</ets>, L. <ets>chrysoprasus</ets>, fr. Gr. <?/; <grk>chryso`s</grk> gold + <?/ leek.]</ety> <fld>(Min.)</fld> <def>An apple-green variety of chalcedony, colored by nickel. It has a dull flinty luster, and is sometimes used in jewelry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysoprasus</ent><br/
||<hw>Chry*sop"ra*sus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>See <er>Chrysoprase</er>.</def>  <rj><au>Rev. xxi. 20.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysosperm</ent><br/
<hw>Chrys"o*sperm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <?/ seed.]</ety> <def>The seed of gold; a means of creating gold.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chrysosplenium</ent><br/
<hw>Chrysosplenium</hw> <pos>n.</pos> <def>a genus of widely distributed semiaquatic herbs with minute greenish-yellow apetalous flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Chrysosplenium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysothamnus</ent><br/
<hw>Chrysothamnus</hw> <pos>n.</pos> <def>a genus of low branching shrubs of West North America.</def><br/
<syn><b>Syn. --</b> genus <gen>Chrysothamnus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chrysotype</ent><br/
<hw>Chrys"o*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>chryso`s</grk> gold + <ets>-type</ets>.]</ety> <sn>1.</sn> <def>A photographic picture taken upon paper prepared by the use of a sensitive salt of iron and developed by the application of chloride of gold.</def>  <rj><au>Abney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>2process, invented by Sir J.Herschel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chthonian</ent><br/
<hw>Chtho"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ in or under the earth, fr. <?/, <?/, earth.]</ety> <def>Designating, or pertaining to, gods or spirits of the underworld; esp., relating to the underworld gods of the Greeks, whose worship is widely considered as more primitive in form than that of the Olympian gods. The characteristics of chthonian worship are propitiatory and magical rites and generalized or euphemistic names of the deities, which are supposed to have been primarily ghosts.</def><br/
<syn><b>Syn. --</b> chthonic, lower, nether.</syn><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chthonic</ent><br/
<hw>Chthon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, the earth.]</ety> <def>Pertaining to the earth; earthy; <as>as, <ex>chthonic</ex> religions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The] <qex>chthonic</qex> character of the wife of Zeus.</q> <rj><qau>Max M<uum/ller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chthonophagy</ent><br/
<ent>Chthonophagia</ent><br/
<mhw>{ ||<hw>Chthon`o*pha"gi*a</hw> <pr>(?)</pr>, <hw>Chtho*noph"a*gy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>chthonophagia</ets>; Gr. <?/, <?/, earth + <?/ to eat.]</ety> <def>A disease characterized by an irresistible desire to eat earth, observed in some parts of the southern United States, the West Indies, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chub</ent><br/
<hw>Chub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[This word seems to signify a large or thick fish. Cf. Sw. <ets>kubb</ets> a short and thick piece of wood, and perh. F. <ets>chabot</ets> chub.]</ety> <fld>(Zool.)</fld> <def>A species to fresh-water fish of the <fam>Cyprinid<ae/</fam> or Carp family. The common European species is <spn>Leuciscus cephalus</spn>; the cheven. In America the name is applied to various fishes of the same family, of the genera <gen>Semotilus</gen>, <gen>Squalius</gen>, <gen>Ceratichthys</gen>, etc., and locally to several very different fishes, as the <altname>tautog</altname>, <altname>black bass</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chub mackerel</b></col> <fld>(Zool.)</fld>, <cd>a species of mackerel (<spn>Scomber colias</spn>) in some years found in abundance on the Atlantic coast, but absent in others; -- called also <altname>bull mackerel</altname>, <altname>thimble-eye</altname>, and <altname>big-eye mackerel</altname>.</cd> -- <col><b>Chub sucker</b></col> <fld>(Zool.)</fld>, <cd>a fresh-water fish of the United States (<spn>Erimyzon sucetta</spn>); -- called also <altname>creekfish</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chubbed</ent><br/
<hw>Chub"bed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Chubby.</def> <mark>[R.]</mark>  <rj><au>H. Brooke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chubbedness</ent><br/
<hw>Chub"bed*ness</hw>, <pos>n.</pos> <def>The state of being chubby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chubby</ent><br/
<hw>Chub"by</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a chub; plump, short, and thick.</def> <ldquo/<xex>Chubby</xex> faces.<rdquo/  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chub-faced</ent><br/
<hw>Chub"-faced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a plump, short face.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw> <pr>(ch<ucr/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chucked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chucking</conjf>.]</vmorph> <ety>[Imitative of the sound.]</ety> <sn>1.</sn> <def>To make a noise resembling that of a hen when she calls her chickens; to cluck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To chuckle; to laugh.</def> <mark>[R.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw>, <pos>v. t.</pos> <def>To call, as a hen her chickens.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw>, <pos>n.</pos> <sn>1.</sn> <def>The chuck or call of a hen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden, small noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A word of endearment; -- corrupted from <xex>chick</xex>.</def> <ldquo/Pray, <xex>chuck</xex>, come hither.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chucked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chucking</conjf>.]</vmorph> <ety>[F. <ets>choquer</ets> to strike. Cf. <er>Shock</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To strike gently; to give a gentle blow to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chucked</qex> the barmaid under the chin.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To toss or throw smartly out of the hand; to pitch.</def> <mark>[Colloq.]</mark> <ldquo/Mahomet Ali will just be <xex>chucked</xex> into the Nile.<rdquo/  <rj><au>Lord Palmerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>To place in a chuck, or hold by means of a chuck, as in turning; to bore or turn (a hole) in a revolving piece held in a chuck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw>, <pos>n.</pos> <sn>1.</sn> <def>A slight blow or pat under the chin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short throw; a toss.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>A contrivance or machine fixed to the mandrel of a lathe, for holding a tool or the material to be operated upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 255 --></p>

<p><cs><col><b>Chuck farthing</b></col>, <cd>a play in which a farthing is pitched into a hole; pitch farthing.</cd> -- <col><b>Chuck hole</b></col>, <cd>a deep hole in a wagon rut.</cd> -- <col><b>Elliptic chuck</b></col>, <cd>a chuck having a slider and an eccentric circle, which, as the work turns round, give it a sliding motion across the center which generates an ellipse.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw> <pr>(ch<ucr/k)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A small pebble; -- called also <altname>chuckstone</altname> and <altname>chuckiestone</altname>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A game played with chucks, in which one or more are tossed up and caught; jackstones.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck</ent><br/
<hw>Chuck</hw>, <pos>n.</pos> <def>A piece of the backbone of an animal, from between the neck and the collar bone, with the adjoining parts, cut for cooking; <as>as, a <ex>chuck</ex> steak; a <ex>chuck</ex> roast</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chucker-out</ent><br/
<hw>chucker-out</hw> <pos>n.</pos> <def>a person whose duty is to throw troublemakers out of a bar or public meeting; a bouncer{5}.</def><br/
<syn><b>Syn. --</b> bouncer.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chuck-full</ent><br/
<hw>chuck-full</hw> <pos>adj.</pos>  <def>same as <er>chock-full</er>.</def> <mark>[predicate]</mark><br/
<syn><b>Syn. --</b> chockablock(predicate), chock-full(predicate), chockfull(predicate), chockful(predicate), choke-full(predicate), cram full.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>chuckhole</ent><br/
<hw>chuck"hole`</hw> <pos>n.</pos> <def>a pit or hole produced by wear or weathering especially in a road surface.</def><br/
<syn><b>Syn. --</b> pothole.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chuckle</ent><br/
<hw>Chuc"kle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Chuckled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chuckling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From 1st <er>Chuck</er>.]</ety> <sn>1.</sn> <def>To call, as a hen her chickens; to cluck.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fondle; to cocker.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuckle</ent><br/
<hw>Chuc"kle</hw>, <pos>n.</pos> <def>A short, suppressed laugh; the expression of satisfaction, exultation, or derision.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuckle</ent><br/
<hw>Chuc"kle</hw>, <pos>v. i.</pos> <ety>[From 1st <er>Chuck</er>.]</ety> <def>To laugh in a suppressed or broken manner, as expressing inward satisfaction, exultation, or derision.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chucklehead</ent><br/
<hw>Chuc"kle*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person with a large head; a numskull; a dunce.</def> <mark>[Low]</mark>  <rj><au>Knowles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuckleheaded</ent><br/
<hw>Chuc"kle*head`ed</hw>, <pos>a.</pos> <def>Having a large head; thickheaded; dull; stupid.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuck-Will's-widow</ent><br/
<hw>Chuck`-Will's-wid"ow</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large whippoorwill-like bird (a species of <isa>goatsucker</isa>) (<spn>Caprimulgus carolinensis</spn>, formerly <spn>Antrostomus Carolinensis</spn>), of the southern United States; -- so called from its note.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chud</ent><br/
<hw>Chud</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Chew</er>, <er>Cud</er>.]</ety> <def>To champ; to bite.</def> <mark>[Obs.]</mark>  <rj><au>A. Stafford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuet</ent><br/
<hw>Chu"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Chew</er>, <pos>v. t.</pos>]</ety> <def>Minced meat.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chufa</ent><br/
||<hw>Chu"fa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <fld>(Bot.)</fld> <def>A sedgelike plant (<spn>Cyperus esculentus</spn>) producing edible tubers, native about the Mediterranean, now cultivated in many regions; the earth almond.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuff</ent><br/
<hw>Chuff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. a modification of <ets>chub</ets>: cf. W. <ets>cyff</ets> stock, stump.]</ety> <def>A coarse or stupid fellow.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuff</ent><br/
<hw>Chuff</hw>, <pos>a.</pos> <def>Stupid; churlish.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuffily</ent><br/
<hw>Chuff"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Clownishly; surlily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuffiness</ent><br/
<hw>Chuff"i*ness</hw>, <pos>n.</pos> <def>The quality of being chuffy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuffy</ent><br/
<hw>Chuff"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Fat or puffed out in the cheeks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rough; clownish; surly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chulan</ent><br/
<hw>Chu"lan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The fragrant flowers of the <spn>Chloranthus inconspicuus</spn>, used in China for perfuming tea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chum</ent><br/
<hw>Chum</hw> <pr>(ch<ucr/m)</pr>, <pos>n.</pos> <ety>[Perh. a contraction fr. <ets>comrade</ets> or <ets>chamber fellow</ets>: cf. also AS. <ets>cuma</ets> a comer, guest.]</ety> <def>A roommate, especially in a college or university; an old and intimate friend.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>New chum</b></col>, <cd>a recent immigrant.</cd> <mark>[Australia]</mark></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chum</ent><br/
<hw>Chum</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. p. p.</pos> <conjf>Chummed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Chumming</conjf>.]</vmorph> <def>To occupy a chamber with another; <as>as, to <ex>chum</ex> together at college</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chum</ent><br/
<hw>Chum</hw>, <pos>n.</pos> <def>Chopped pieces of fish used as bait.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>chummy</ent><br/
<hw>chummy</hw> <pos>adj.</pos> <sn>1.</sn> <def>having the relationship of friends or pals.</def> <mark>[colloquial]</mark><br/
<syn><b>Syn. --</b> matey, pally, palsy-walsy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>associated on close terms.</def> <mark>[informal]</mark><br/
<syn><b>Syn. --</b> thick(predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chump</ent><br/
<hw>Chump</hw> <pr>(ch<ucr/mp)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>kumbr</ets> a chopping, E. <ets>chop</ets>.]</ety> <sn>1.</sn> <def>A short, thick, heavy piece of wood.</def>  <rj><au>Morton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a stupid person; a fool; a dolt; also, a dupe.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Chump end</b></col>, <cd>the thick end; <as>as, the <ex>chump end</ex> of a joint of meat</as>.</cd>  <rj><au>Dickens.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chunam</ent><br/
||<hw>Chu*nam"</hw> <pr>(ch<oomac/*n<acr/m")</pr>, <pos>n.</pos> <ety>[Hind. <ets>ch<umac/n<amac/</ets>, from Skr. <ets>c<umac/r<nsdot/a</ets> powder, dust; or a Dravidian word.]</ety> <def>Quicklime; also, plaster or mortar.</def> <mark>[India]</mark>  <rj><au>Whitworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>chunga</ent><br/
<hw>chunga</hw> <pos>n.</pos> <def>The Argentinian cariama (<spn>Chunga burmeisteri</spn>).</def><br/
<syn><b>Syn. --</b> seriema, <spn>Chunga burmeisteri</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chunk</ent><br/
<hw>Chunk</hw> <pr>(ch<ucr/<nsm/k)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chump</er>.]</ety> <def>A short, thick piece of anything.</def> <mark>[Colloq. U. S. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chunky</ent><br/
<hw>Chunk"y</hw> <pr>(ch<ucr/<nsm/k*<ycr/)</pr>, <pos>a.</pos> <def>Short and thick.</def> <mark>[U. S.]</mark>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chupatty</ent><br/
<hw>Chu*pat"ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-ties</plw> <pr>(#)</pr></plu>. <ety>[Hind. <ets>chap<amac/t<imac/</ets>.]</ety> <def>A kind of griddlecake of unleavened bread, used among the natives of India.</def> <mark>[Anglo-Indian]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Chuprassie</ent><br/
<ent>Chuprassy</ent><br/
<mhw>{ <hw>Chu*pras"sy</hw> <hw>Chu*pras"sie</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>chapr<amac/ss<imac/</ets>, fr. <ets>chapr<amac/s</ets> badge.]</ety> <def>A messenger or servant wearing an official badge.</def> <mark>[Anglo-Indian]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Church</ent><br/
<hw>Church</hw> <pr>(ch<ucir/rch)</pr>, <pos>n.</pos> <ety>[OE. <ets>chirche</ets>, <ets>chireche</ets>, <ets>cherche</ets>, Scot. <ets>kirk</ets>, from AS. <ets>circe</ets>, <ets>cyrice</ets>; akin to D. <ets>kerk</ets>, Icel. <ets>kirkja</ets>, Sw. <ets>kyrka</ets>, Dan. <ets>kirke</ets>, G. <ets>kirche</ets>, OHG. <ets>chirihha</ets>; all fr. Gr. <grk>kyriako`n</grk> the Lord's house, fr. <grk>kyriako`s</grk> concerning a master or lord, fr. <grk>ky`rios</grk> master, lord, fr. <grk>ky^ros</grk> power, might; akin to Skr. <ets><cced/<umac/ra</ets> hero, Zend. <ets><cced/ura</ets> strong, OIr. <ets>caur</ets>, <ets>cur</ets>, hero. Cf. <er>Kirk</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A building set apart for Christian worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A Jewish or heathen temple.</def> <mark>[Obs.]</mark>  <rj><au>Acts xix. 37.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A formally organized body of Christian believers worshiping together.</def> <ldquo/When they had ordained them elders in every <xex>church</xex>.<rdquo/  <rj><au>Acts xiv. 23.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A body of Christian believers, holding the same creed, observing the same rites, and acknowledging the same ecclesiastical authority; a denomination; <as>as, the Roman Catholic <ex>church</ex>; the Presbyterian <ex>church</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The collective body of Christians.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Any body of worshipers; <as>as, the Jewish <ex>church</ex>; the <ex>church</ex> of Brahm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The aggregate of religious influences in a community; ecclesiastical influence, authority, etc.; <as>as, to array the power of the <ex>church</ex> against some moral evil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Remember that both <qex>church</qex> and state are properly the rulers of the people, only because they are their benefactors.</q> <rj><qau>Bulwer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Church</ex> is often used in composition to denote something belonging or relating to the church; <as>as, <ex>church</ex> authority; <ex>church</ex> history; <ex>church</ex> member; <ex>church</ex> music, etc.</as></note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Apostolic church</b></col>. <cd>See under <er>Apostolic</er>.</cd> -- <col><b>Broad church</b></col>. <cd>See <er>Broad Church</er>.</cd> -- <mcol><col><b>Catholic church</b></col> <it>or</it> <col><b>Universal church</b></col></mcol>, <cd>the whole body of believers in Christ throughout the world.</cd> -- <mcol><col><b>Church of England</b></col>, <it>or</it> <col><b>English church</b></col></mcol>, <cd>the Episcopal church established and endowed in England by law.</cd> -- <col><b>Church living</b></col>, <cd>a benefice in an established church.</cd> -- <col><b>Church militant</b></col>. <cd>See under <er>Militant</er>.</cd> -- <col><b>Church owl</b></col> <fld>(Zool.)</fld>, <cd>the white owl. See <er>Barn owl</er>.</cd> -- <col><b>Church rate</b></col>, <cd>a tax levied on parishioners for the maintenance of the church and its services.</cd> -- <col><b>Church session</b></col>. <cd>See under <er>Session</er>.</cd> -- <col><b>Church triumphant</b></col>. <cd>See under <er>Triumphant</er>.</cd> -- <col><b>Church work</b></col>, <cd>work on, or in behalf of, a church; the work of a particular church for the spread of religion.</cd> -- <col><b>Established church</b></col>, <cd>the church maintained by the civil authority; a state church.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Church</ent><br/
<hw>Church</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Churched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Churching</conjf>.]</vmorph> <def>To bless according to a prescribed form, or to unite with in publicly returning thanks in church, as after deliverance from the dangers of childbirth; <as>as, the <ex>churching</ex> of women</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Church-ale</ent><br/
<hw>Church"-ale`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A church or parish festival (as in commemoration of the dedication of a church), at which much ale was used.</def>  <rj><au>Wright. Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Church-bench</ent><br/
<hw>Church"-bench`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seat in the porch of a church.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchdom</ent><br/
<hw>Church"dom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The institution, government, or authority of a church.</def> <mark>[R.]</mark>  <rj><au>Bp. Pearson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchgoer</ent><br/
<hw>Church"go`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who attends church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchgoing</ent><br/
<hw>Church"go`ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Habitually attending church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Summoning to church.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sound of the <qex>churchgoing</qex> bell.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Church-haw</ent><br/
<hw>Church"-haw`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Church</ets> + <ets>haw</ets> a yard.]</ety> <def>Churchyard.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchism</ent><br/
<hw>Church"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Strict adherence to the forms or principles of some church organization; sectarianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchless</ent><br/
<hw>Church"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without a church.</def>  <rj><au>T. Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchlike</ent><br/
<hw>Church"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Befitting a church or a churchman; becoming to a clergyman.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchliness</ent><br/
<hw>Church"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Regard for the church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchly</ent><br/
<hw>Church"ly</hw>, <pos>a.</pos> <def>Pertaining to, or suitable for, the church; ecclesiastical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchman</ent><br/
<hw>Church"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Churchmen</plw> <pr>(#)</pr>.</plu> <sn>1.</sn> <def>An ecclesiastic or clergyman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>An Episcopalian, or a member of the Established Church of England.</def> <ldquo/A zealous <xex>churchman</xex>.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>One was is attached to, or attends, church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchmanly</ent><br/
<hw>Church"man*ly</hw>, <pos>a.</pos> <def>Pertaining to, or becoming, a churchman.</def>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchmanship</ent><br/
<hw>Church"man*ship</hw>, <pos>n.</pos> <def>The state or quality of being a churchman; attachment to the church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Church modes</ent><br/
<hw>Church" modes`</hw> <pr>(?)</pr>. <fld>(Mus.)</fld> <def>The modes or scales used in ancient church music. See <er>Gregorian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchship</ent><br/
<hw>Church"ship</hw>, <pos>n.</pos> <def>State of being a church.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchwarden</ent><br/
<hw>Church"ward`en</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One of the officers (usually two) in an Episcopal church, whose duties vary in different dioceses, but always include the provision of what is necessary for the communion service.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>A clay tobacco pipe, with a long tube.</def> <mark>[Slang, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There was a small wooden table placed in front of the smoldering fire, with decanters, a jar of tobacco, and two long <qex>churchwardens</qex>.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchwardenship</ent><br/
<hw>Church"ward`en*ship</hw>, <pos>n.</pos> <def>The office of a churchwarden.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchy</ent><br/
<hw>Church"y</hw>, <pos>a.</pos> <def>Relating to a church; unduly fond of church forms.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churchyard</ent><br/
<hw>Church"yard`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The ground adjoining a church, in which the dead are buried; a cemetery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like graves in the holy <qex>churchyard</qex>.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Burial place; burying ground; graveyard; necropolis; cemetery; God's acre.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churl</ent><br/
<hw>Churl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>ceorl</ets> a freeman of the lowest rank, man, husband; akin to D. <ets>karel</ets>, <ets>kerel</ets>, G. <ets>kerl</ets>, Dan. & Sw. <ets>karl</ets>, Icel. <ets>karl</ets>, and to the E. proper name <ets>Charles</ets> (orig., <ets>man</ets>, <ets>male</ets>), and perh. to Skr. <ets>j<amac/ra</ets> lover. Cf. <er>Carl</er>, <er>Charles's Wain</er>.]</ety> <sn>1.</sn> <def>A rustic; a countryman or laborer.</def> <ldquo/A peasant or <xex>churl</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your rank is all reversed; let men of cloth<br/
Bow to the stalwart <qex>churls</qex> in overalls.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rough, surly, ill-bred man; a boor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>churl's</qex> courtesy rarely comes, but either for gain or falsehood.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A selfish miser; an illiberal person; a niggard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like to some rich <qex>churl</qex> hoarding up his pelf.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churl</ent><br/
<hw>Churl</hw>, <pos>a.</pos> <def>Churlish; rough; selfish.</def> <mark>[Obs.]</mark>  <rj><au>Ford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churlish</ent><br/
<hw>Churl"ish</hw>, <pos>a.</pos> <sn>1.</sn> <def>Like a churl; rude; cross-grained; ungracious; surly; illiberal; niggardly.</def> <ldquo/<xex>Churlish</xex> benefits.<rdquo/  <rj><au>Ld. Burleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Half mankind maintain a <qex>churlish</qex> strife.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wanting pliancy; unmanageable; unyielding; not easily wrought; <as>as, a <ex>churlish</ex> soil; the <ex>churlish</ex> and intractable nature of some minerals</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churlishly</ent><br/
<hw>Churl"ish*ly</hw>, <pos>adv.</pos> <def>In a churlish manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churlishness</ent><br/
<hw>Churl"ish*ness</hw>, <pos>n.</pos> <def>Rudeness of manners or temper; lack of kindness or courtesy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churly</ent><br/
<hw>Churl"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Rude; churlish; violent.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chirm</ent><br/
<ent>Churme</ent><br/
<mhw>{ <hw>Churme</hw> <pr>(?)</pr>, <hw>Chirm</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[See <er>Chirm</er>.]</ety> <def>Clamor, or confused noise; buzzing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>churme</qex> of a thousand taunts and reproaches.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churn</ent><br/
<hw>Churn</hw> <pr>(ch<ucir/rn)</pr>, <pos>n.</pos> <ety>[OE. <ets>chirne</ets>, <ets>cherne</ets>, AS. <ets>ceren</ets>, <ets>cyrin</ets>; akin to D. <ets>karn</ets>, Dan. <ets>kierne</ets>. See <er>Churn</er>, <pos>v. t.</pos>]</ety> <def>A vessel in which milk or cream is stirred, beaten, or otherwise agitated (as by a plunging or revolving dasher) in order to separate the oily globules from the other parts, and obtain butter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churn</ent><br/
<hw>Churn</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Churned</conjf> <pr>(ch<ucir/rnd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Churning</conjf>.]</vmorph> <ety>[OE. <ets>chernen</ets>, AS. <ets>cernan</ets>; akin to LG. <ets>karnen</ets>, G. <ets>kernen</ets>, D. <ets>karnen</ets>, Dan. <ets>kierne</ets>, Sw. <ets>k<aum/rna</ets>, and also to E. <ets>corn</ets>, <ets>kernel</ets>, the meaning coming from the idea of extracting the kernel or marrow. See <er>Kernel</er>.]</ety> <sn>1.</sn> <def>To stir, beat, or agitate, as milk or cream in a churn, in order to make butter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shake or agitate with violence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Churned</qex> in his teeth, the foamy venom rose.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churn</ent><br/
<hw>Churn</hw>, <pos>v. i.</pos> <def>To perform the operation of churning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>churned-up</ent><br/
<hw>churned-up</hw> <pos>adj.</pos>  <def>agitated vigorously; -- of a liquid.</def><br/
<syn><b>Syn. --</b> churning, roiling, roiled, roily, turbulent.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Churning</ent><br/
<hw>Churn"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who churns.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quantity of butter made at one operation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churr</ent><br/
<hw>Churr</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Chirr</er>.]</ety> <def>A vibrant or whirring noise such as that made by some insects, as the cockchafer, or by some birds, as the nightjar, the partridge, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Churr</ent><br/
<hw>Churr</hw>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Churred</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Churr"ing</conjf>.]</vmorph> <def>To make a churr, as a cockchafer.</def></p>

<p><q>That's the <qex>churring</qex> of the nightjar.</q>  <rj><qau>Hall Caine.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Churr</ent><br/
<hw>Churr</hw>, <pos>v. t.</pos> <def>To utter by churring.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Churrus</ent><br/
||<hw>Chur"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>charas</ets>.]</ety> <def>A powerfully narcotic and intoxicating gum resin which exudes from the flower heads, seeds, etc., of Indian hemp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Churrworm</ent><br/
<hw>Churr"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cyrran</ets>, <ets>cerran</ets>, to turn.]</ety> <fld>(Zool.)</fld> <def>An insect that turns about nimbly; the mole cricket; -- called also <altname>fan cricket</altname>.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chuse</ent><br/
<hw>Chuse</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See Choose.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chute</ent><br/
<hw>Chute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chute</ets>, prop. a fall.]</ety> <sn>1.</sn> <def>A framework, trough, or tube, upon or through which objects are made to slide from a higher to a lower level, or through which water passes to a wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>See <er>Shoot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chutnee</ent><br/
<ent>Chutney</ent><br/
<mhw>{ <hw>Chut"ney</hw> <pr>(?)</pr>, <hw>Chut"nee</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Hind. <ets>chatn<imac/</ets>.]</ety> <sn>1.</sn> <def>A warm or spicy condiment or pickle made in India, compounded of various vegetable substances, such as chopped fruits or green tomatoes, etc., often cooked with sweets and acids such as sugar and vinegar, with ginger and spices.</def><br/
<syn><b>Syn. --</b> Indian relish.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>chutzpah</ent><br/
<ent>chutzpa</ent><br/
<mhw><hw>chutzpa</hw>, <hw>chutzpah</hw></mhw> <pos>n.</pos> <ety>[Yiddish <ets>khutzpa</ets>.]</ety> <def>aggressive boldness or unmitigated effrontery; gall; <as>as, he had the <ex>chutzpah</ex> to question my decision</as>.</def> <altsp>[Also spelled <asp>hutzpa</asp>, and <asp>hutzpah</asp>.]</altsp><br/
<syn><b>Syn. --</b> audacity, audaciousness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chylaceous</ent><br/
<hw>Chy*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Possessed of the properties of chyle; consisting of chyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylaqueous</ent><br/
<hw>Chy*la"que*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chyle</ets> + <ets>aqueous</ets>.]</ety> <fld>(Zool.)</fld> <def>Consisting of chyle much diluted with water; -- said of a liquid which forms the circulating fluid of some inferior animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chyle</ent><br/
<hw>Chyle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>chylus</ets>, Gr. <?/ juice, chyle, fr. <?/ to pour: cf. F. <ets>chyle</ets>; prob. akin to E. <ets>fuse</ets> to melt.]</ety> <fld>(Physiol.)</fld> <def>A milky fluid containing the fatty matter of the food in a state of emulsion, or fine mechanical division; formed from chyme by the action of the intestinal juices. It is absorbed by the lacteals, and conveyed into the blood by the thoracic duct.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylifaction</ent><br/
<hw>Chyl`i*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chyle</ets> + L. <ets>facere</ets> to make.]</ety> <fld>(Physiol.)</fld> <def>The act or process by which chyle is formed from food in animal bodies; chylification, -- a digestive process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylifactive</ent><br/
<hw>Chyl`i*fac"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Producing, or converting into, chyle; having the power to form chyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>chylifactory</ent><br/
<hw>chylifactory</hw> <pos>adj.</pos>  <def>producing or converting into chyle.</def><br/
<syn><b>Syn. --</b> chylific, chylifactive.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Chyliferous</ent><br/
<hw>Chy*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chyle</ets> + <ets>-ferous</ets>: cf. F. <ets>chylif<egrave/re</ets>.]</ety> <fld>(Physiol.)</fld> <def>Transmitting or conveying chyle; <as>as, <ex>chyliferous</ex> vessels</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylific</ent><br/
<hw>Chy*lif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Chylifactive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylification</ent><br/
<hw>Chyl`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The formation of chyle. See <er>Chylifaction</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylificatory</ent><br/
<hw>Chy*lif"i*ca*to*ry</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Chylifactive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylify</ent><br/
<hw>Chy"li*fy</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[<ets>Chyle</ets> + <ets>-ly</ets>.]</ety> <fld>(Physiol.)</fld> <def>To make chyle of; to be converted into chyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylopoetic</ent><br/
<hw>Chy`lo*po*et"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>chylopoiei^n</grk> to make into juice, <grk>chylo`s</grk> juice, chyle + <grk>poiei^n</grk> to make.]</ety> <fld>(Physiol.)</fld> <def>Concerned in the formation of chyle; <as>as, the <ex>chylopoetic</ex> organs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chylous</ent><br/
<hw>Chy"lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>chyleux</ets>.]</ety> <fld>(Physiol.)</fld> <def>Consisting of, or similar to, chyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chyluria</ent><br/
||<hw>Chy*lu"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. from Gr. <?/ chyle + <?/ urine.]</ety> <fld>(Med.)</fld> <def>A morbid condition in which the urine contains chyle or fatty matter, giving it a milky appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chyme</ent><br/
<hw>Chyme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>chymus</ets> chyle, Gr. <?/ juice, like <?/, fr. <?/ to pour: cf. F. <ets>chyme</ets>. See <er>Chyle</er>.]</ety> <fld>(Physiol.)</fld> <def>The pulpy mass of semi-digested food in the small intestines just after its passage from the stomach. It is separated in the intestines into chyle and excrement. See <er>Chyle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chymistry</ent><br/
<ent>Chymist</ent><br/
<ent>Chymic</ent><br/
<mhw>{ <hw>Chym"ic</hw> <pr>(?)</pr>, <hw>Chym"ist</hw>, <hw>Chym"is*try</hw> <pr>(?)</pr>. }</mhw> <mark>[Obs.]</mark> <def>See <er>Chemic</er>, <er>Chemist</er>, <er>Chemistry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chymiferous</ent><br/
<hw>Chy*mif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Chyme</ets> + <ets>-ferous</ets>.]</ety> <fld>(Physiol.)</fld> <def>Bearing or containing chyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chymification</ent><br/
<hw>Chym`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Chyme</ets> + L. <ets>facere</ets> to make: cf. F. <ets>Chymification</ets>.]</ety> <fld>(Physiol.)</fld> <def>The conversion of food into chyme by the digestive action of gastric juice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chymify</ent><br/
<hw>Chym"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<ets>Chyme</ets> + <ets>-fy</ets>: cf. F. <ets>chymifier</ets>.]</ety> <fld>(Physiol.)</fld> <def>To form into chyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chymous</ent><br/
<hw>Chy"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to chyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Chyometer</ent><br/
<hw>Chy*om"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ to pour + <ets>-meter</ets>.]</ety> <fld>(Chem.)</fld> <def>An instrument for measuring liquids. It consists of a piston moving in a tube in which is contained the liquid, the quantity expelled being indicated by the graduation upon the piston rod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cibarious</ent><br/
<hw>Ci*ba"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cibaruus</ets>, fr. <ets>cibus</ets> food.]</ety> <def>Pertaining to food; edible.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cibation</ent><br/
<hw>Ci*ba"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cibatio</ets>, fr. <ets>cibare</ets> to feed.]</ety> <sn>1.</sn> <def>The act of taking food.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Alchemy)</fld> <def>The process or operation of feeding the contents of the crucible with fresh material.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cibol</ent><br/
<hw>Cib"ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>ciboule</ets>, LL. <ets>cepula</ets>, <ets>cepola</ets>, dim. of L. <ets>cepa</ets>, <ets>caepa</ets>, <ets>caepe</ets>, an onion. Cf. <er>Chibbal</er>, <er>Cives</er>.]</ety> <def>A perennial alliaceous plant (<spn>Allium fistulosum</spn>), sometimes called <xex>Welsh onion</xex>. Its fistular leaves areused in cookery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciborium</ent><br/
||<hw>Ci*bo"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>: <plu><it>pl.</it> <plw>Ciboria</plw> <pr>(#)</pr>.</plu> <ety>[LL., fr. L. <ets>ciborium</ets> a cup, fr. Gr. <?/ a seed vessel of the Egyptian bean; also, a cup made from its largeleaves, or resembling its seed vessel in shape.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A canopy usually standing free and supported on four columns, covering the high altar, or, very rarely, a secondary altar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>The coffer or case in which the host is kept; the pyx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicada</ent><br/
<hw>Ci*ca"da</hw> <pr>(s<icr/*k<amac/"d<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Cicadas</plw> <pr>(-d<adot/z)</pr>, L. <plw>Cicad<ae/</plw> <pr>(-d<emac/)</pr>.</plu> <ety>[L.]</ety> <fld>(Zool.)</fld> <def>Any species of the genus <gen>Cicada</gen> or of the family <fam>Cicadidae</fam>. They are large hemipterous insects, with nearly transparent wings. The male makes a shrill sound by peculiar organs in the under side of the abdomen, consisting of a pair of stretched membranes, acted upon by powerful muscles. A noted American species (<spn>Cicada septendecim</spn>) is called the <stype>seventeen year locust</stype>. Another common species is the <stype>dogday cicada</stype>.</def><br/
<syn><b>Syn. --</b> cicala.</syn>
[<source>1913 Webster</source>]</p>

<p><ent>Cicadellidae</ent><br/
<hw>Cicadellidae</hw> <pos>n.</pos> <def>a natural family comprising the leafhoppers.</def><br/
<syn><b>Syn. --</b> family <fam>Cicadellidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cicadidae</ent><br/
<hw>Cicadidae</hw> <pos>n.</pos> <def>a family of insects comprising the cicadas.</def><br/
<syn><b>Syn. --</b> family <fam>Cicadidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cicala</ent><br/
||<hw>Ci*ca"la</hw> <pr>(ch<esl/*k<aum/"l<adot/)</pr>, <pos>n.</pos> <ety>[It., fr. L. <ets>cicada</ets>.]</ety> <def>A cicada. See <er>Cicada</er>.</def> <ldquo/At eve a dry <xex>cicala</xex> sung.<rdquo/  <rj><au>Tennison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrice</ent><br/
<hw>Cic"a*trice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>cicatrix</ets>.]</ety> <def>A cicatrix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatricial</ent><br/
<hw>Cic`a*tri"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Relating to, or having the character of, a cicatrix.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatricle</ent><br/
<hw>Cic"a*tri`cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cicatricule</ets>, fr. L. <ets>cicatricula</ets> a small scar, fr. <ets>cicatrix</ets> a scar.]</ety> <fld>(Biol.)</fld> <def>The germinating point in the embryo of a seed; the point in the yolk of an egg at which development begins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrisive</ent><br/
<hw>Cic"a*tri`sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to promote the formation of a cicatrix; good for healing of a wound.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 256 --></p>

<p><ent>Cicatrix</ent><br/
||<hw>Ci*ca"trix</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cicatrices</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Med.)</fld> <def>The pellicle which forms over a wound or breach of continuity and completes the process of healing in the latter, and which subsequently contracts and becomes white, forming the scar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrizant</ent><br/
<hw>Cic"a*tri`zant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cicatrisant</ets>, properly p. pr. of <ets>cicatriser</ets>.]</ety> <fld>(Med.)</fld> <def>A medicine or application that promotes the healing of a sore or wound, or the formation of a cicatrix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrization</ent><br/
<hw>Cic`a*tri*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cicatrisation</ets>.]</ety> <fld>(Med.)</fld> <def>The process of forming a cicatrix, or the state of being cicatrized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrize</ent><br/
<hw>Cic"a*trize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cicatrized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cicatrizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>cicatriser</ets>, fr. <ets>cicatrice</ets>, L. <ets>cicatrix</ets>, scar.]</ety> <fld>(Med.)</fld> <def>To heal or induce the formation of a cicatrix in, as in wounded or ulcerated flesh.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrize</ent><br/
<hw>Cic"a*trize</hw>, <pos>v. i.</pos> <fld>(Med.)</fld> <def>To heal; to have a new skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicatrose</ent><br/
<hw>Cic"a*trose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of scars.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicely</ent><br/
<hw>Cic"e*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>seselis</ets>, Gr. <?/, <?/; perh. ultimately of Egyptian origin.]</ety> <fld>(Bot.)</fld> <def>Any one of several umbelliferous plants, of the genera <gen>Myrrhis</gen>, <gen>Osmorrhiza</gen>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicero</ent><br/
<hw>Cic"e*ro</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Print.)</fld> <def>Pica type; -- so called by French printers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicerone</ent><br/
||<hw>Ci`ce*ro"ne</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> It. <plw>Ciceroni</plw> <pr>(#)</pr>, E. <plw>Cicerones</plw> <pr>(#)</pr>.</plu> <ety>[It., fr. L. <etsep>Cicero</etsep>, the Roman orator. So called from the ordinary talkativeness of such a guide.]</ety> <def>One who shows strangers the curiosities of a place; a guide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every glib and loquacious hireling who shows strangers about their picture galleries, palaces, and ruins, is termed by them [the Italians] a <qex>cicerone</qex>, or a Cicero.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciceronian</ent><br/
<hw>Cic`e*ro"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Ciceronianus</ets>, fr. <ets>Cicero</ets>, the orator.]</ety> <def>Resembling Cicero in style or action; eloquent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciceronianism</ent><br/
<hw>Cic`e*ro"ni*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Imitation of, or resemblance to, the style or action Cicero; a Ciceronian phrase or expression.</def> <ldquo/Great study in <xex>Ciceronianism</xex>, the chief abuse of Oxford.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cichoraceous</ent><br/
<hw>Cich`o*ra"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Chicory</er>.]</ety> <def>Belonging to, or resembling, a suborder of composite plants of which the chicory (<gen>Cichorium</gen>) is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cich-pea</ent><br/
<hw>Cich"-pea`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The chick-pea.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicisbeism</ent><br/
<hw>Ci*cis"be*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or conduct of a cicisbeo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicisbeo</ent><br/
||<hw>Ci`cis*be"o</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> It. <plw>Cicisbei</plw> <pr>(#)</pr>.</plu> <ety>[It.]</ety>  <sn>1.</sn> <def>A professed admirer of a married woman; a dangler about women.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A knot of silk or ribbon attached to a fan, walking stick, etc.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciclatoun</ent><br/
<hw>Cic"la*toun`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of. <ets>ciclaton</ets>.]</ety> <def>A costly cloth, of uncertain material, used in the Middle Ages.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>checklaton</asp>, <asp>chekelatoun</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>His robe was of <qex>ciclatoun</qex>,<br/
That coste many a Jane.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciconiidae</ent><br/
<hw>Ciconiidae</hw> <pos>n.</pos> <def>a natural family of birds comprising the storks.</def><br/
<syn><b>Syn. --</b> family <fam>Ciconiidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ciconiiformes</ent><br/
<hw>Ciconiiformes</hw> <pos>n.</pos> <def>an order of chiefly tropical marsh-dwelling fish-eating wading birds with long legs and bills and (except for flamingos) unwebbed feet, including the herons; storks; spoonbills; flamingos; and ibises.</def><br/
<syn><b>Syn. --</b> order <ord>Ciconiiformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cicurate</ent><br/
<hw>Cic"u*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>cicurare</ets> to tame, fr. <ets>cicur</ets> tame.]</ety> <def>To tame.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicuration</ent><br/
<hw>Cic`u*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cicuration</ets>.]</ety> <def>The act of taming.</def> <mark>[Obs.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicuta</ent><br/
||<hw>Ci*cu"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., the poison hemlock.]</ety> <fld>(Bot.)</fld> <def>a genus of poisonous umbelliferous plants, of which the water hemlock or cowbane is best known.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name <xex>cicuta</xex> is sometimes erroneously applied to <xex>Conium maculatum</xex>, or <xex>officinal hemlock</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cicutoxin</ent><br/
<hw>Cic`u*tox"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The active principle of the water hemlock (<prodby><gen>Cicuta</gen></prodby>) extracted as a poisonous gummy substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cid</ent><br/
<hw>Cid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. Ar. <ets>seid</ets> lord.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Chief or commander; in Spanish literature, a title of Ruy Diaz, Count of Bivar, a champion of Christianity and of the old Spanish royalty, in the 11th century.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An epic poem, which celebrates the exploits of the Spanish national hero, Ruy Diaz.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cider</ent><br/
<hw>Ci"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cidre</ets>, OF. sidre, fr. L. <ets>sicera</ets> a kind of strong drink, Gr. <?/; of Oriental origin; cf. Heb. <ets>sh<amac/kar</ets> to be intoxicated, <ets>sh<emac/k<amac/r</ets> strong drink.]</ety> <def>The expressed juice of apples. It is used as a beverage, for making vinegar, and for other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cider</xex> was formerly used to signify the juice of other fruits, and other kinds of strong liquor, but was not applied to wine.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cider brandy</b></col>, <cd>a kind of brandy distilled from cider.</cd> -- <col><b>Cider mill</b></col>, <cd>a mill in which cider is made.</cd> -- <col><b>Cider press</b></col>, <cd>the press of a cider mill.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciderist</ent><br/
<hw>Ci`der*ist</hw>, <pos>n.</pos> <def>A maker of cider.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciderkin</ent><br/
<hw>Ci"der*kin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cider</ets> + <ets>-kin</ets>.]</ety> <def>A kind of weak cider made by steeping the refuse pomace in water.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Ciderkin</qex> is made for common drinking, and supplies the place of small beer.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ci-devant</ent><br/
||<hw>Ci`-de*vant"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., hitherto, formerly.]</ety> <def>Former; previous; of times gone by; <as>as, a <ex>ci-devant</ex> governor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cierge</ent><br/
||<hw>Cierge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>cera</ets> wax.]</ety> <def>A wax candle used in religous rites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cigar</ent><br/
<hw>Ci*gar"</hw> <pr>(s<icr/*g<aum/r")</pr>, <pos>n.</pos> <ety>[Sp. <ets>cigarro</ets>, orig., a kind of tobacco in the island of Cuba: cf. F. <ets>cigare</ets>.]</ety> <def>A small roll of tobacco, used for smoking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cigaret</ent><br/
<hw>cigaret</hw> <pos>n.</pos> <def>same as <er>cigarette</er>.</def><br/
<syn><b>Syn. --</b> cigarette, fag.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cigarette</ent><br/
<hw>Cig`a*rette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cigarette</ets>.]</ety> <def>A little cigar; a little fine tobacco rolled in paper for smoking.</def> <altsp>[Also spelled <asp>cigaret</asp>.]</altsp><br/
<syn><b>Syn. --</b> cigarette, fag, weed.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>cigarfish</ent><br/
<ent>Cigar fish</ent><br/
<mhw><hw>Cigar fish</hw>, <hw>cigarfish</hw></mhw> <pos>n.</pos> <fld>(Zool.)</fld>  <def>a small fusiform fish (<spn>Decapterus punctatus</spn>), allied to the mackerel, found in the West Atlantic and on the coast of the Gulf of Mexico.</def><br/
<syn><b>Syn. --</b> round scad, quiaquia, Decapterus punctatus.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>cigarillo</ent><br/
<hw>cigarillo</hw> <pos>n.</pos> <def>a small cigar or cigarette wrapped in tobacco instead of paper.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cigar-shaped</ent><br/
<hw>ci`gar"-shaped`</hw> <pr>(s<icr/*g<aum/r"-sh<amac/pt`)</pr>, <pos>adj.</pos>  <def>cylindrical, tapering at each end; having the shape of a cigar.</def><br/
<syn><b>Syn. --</b> fusiform, spindle-shaped.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cilantro</ent><br/
<hw>cilantro</hw> <pos>n.</pos> <sn>1.</sn> <def>an Old World herb (<spn>Coriandrum sativum</spn>) with aromatic parsleylike leaves and seed.</def><br/
<syn><b>Syn. --</b> coriander, coriander plant, Chinese parsley, <spn>Coriandrum sativum</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a parsleylike herb used as seasoning or garnish.</def><br/
<syn><b>Syn. --</b> coriander, Chinese parsley.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cilia</ent><br/
||<hw>Cil"i*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <wordforms><wf>Cilium</wf>, the sing., is rarely used.</wordforms> <ety>[L. <ets>cilium</ets> eyelid.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The eyelashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Small, generally microscopic, vibrating appendages lining certain organs, as the air passages of the higher animals, and in the lower animals often covering also the whole or a part of the exterior. They are also found on some vegetable organisms. In the Infusoria, and many larval forms, they are locomotive organs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>Hairlike processes, commonly marginal and forming a fringe like the eyelash.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>Small, vibratory, swimming organs, somewhat resembling true cilia, as those of <phylum>Ctenophora</phylum>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cilial</ent><br/
<hw>cilial</hw> <pos>adj.</pos> <def>of or pertaining to a cilium or cilia.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ciliary</ent><br/
<hw>Cil"ia*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>ciliaire</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Anat.)</fld> <def>Pertaining to the cilia, or eyelashes. Also applied to special parts of the eye itself; <as>as, the <ex>ciliary</ex> processes of the choroid coat; the <ex>ciliary</ex> muscle, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Pertaining to or connected with the cilia in animal or vegetable organisms; <as>as, <ex>ciliary</ex> motion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciliata</ent><br/
||<hw>Cil`i*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Cilia</er>.]</ety> <fld>(Zool.)</fld> <def>One of the orders of Infusoria, characterized by having cilia. In some species the cilia cover the body generally, in others they form a band around the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciliated</ent><br/
<ent>Ciliate</ent><br/
<mhw>{ <hw>Cil"i*ate</hw> <pr>(?)</pr>, <hw>Cil"i*a`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Provided with, or surrounded by, cilia; <as>as, a <ex>ciliate</ex> leaf</as>; endowed with vibratory motion; <as>as, the <ex>ciliated</ex> epithelium of the windpipe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cilice</ent><br/
<hw>Cil"ice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Cilicious</er>.]</ety> <def>A kind of haircloth undergarment.</def>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cilician</ent><br/
<hw>Ci*li"cian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Cilicia in Asia Minor.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Cilicia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cilicious</ent><br/
<hw>Ci*li"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cilicium</ets> a covering, orig. made of Cilician goat's hair, fr. <ets>Cilicious</ets> Cilician, fr. <ets>Cilicia</ets>, a province of Asia Minor.]</ety> <def>Made, or consisting, of hair.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>Cilicious</qex> or sackcloth habit.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciliiform</ent><br/
<ent>Ciliform</ent><br/
<mhw>{ <hw>Cil"i*form</hw> <pr>(?)</pr>, <hw>Cil"i*i*form`</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Cilium</ets> + <ets>-form</ets>]</ety> <def>Having the form of cilia; very fine or slender.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciliograde</ent><br/
<hw>Cil"i*o*grade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cilium</ets> + L. <ets>gradi</ets> to step: cf. F. <ets>ciliograde</ets>.]</ety> <fld>(Zool.)</fld> <def>Moving by means of <xex>cilia</xex>, or cilialike organs; <as>as, the <ex>ciliograde</ex> Medus<ae/</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cilium</ent><br/
||<hw>Cil"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., eyelid.]</ety> <def>See <er>Cilia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cill</ent><br/
<hw>Cill</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sill</er>., <pos>n.</pos> a foundation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cillosis</ent><br/
||<hw>Cil*lo"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>cilium</ets> eyelid.]</ety> <fld>(Med.)</fld> <def>A spasmodic trembling of the upper eyelid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cima</ent><br/
<hw>Ci"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A kind of molding. See <er>Cyma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimar</ent><br/
<hw>Ci*mar"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Simar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimbal</ent><br/
<hw>Cim"bal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>ciambella</ets>.]</ety> <def>A kind of confectionery or cake.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimbia</ent><br/
<hw>Cim"bi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A fillet or band placed around the shaft of a column as if to strengthen it.</def> <altsp>[Written also <asp>cimia</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimbrian</ent><br/
<hw>Cim"bri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Cimbri.</def> -- <def2><pos>n.</pos> <def>One of the Cimbri. See <er>Cimbric</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimbric</ent><br/
<hw>Cim"bric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the <xex>Cimbri</xex>, an ancient tribe inhabiting Northern Germany.</def> -- <def2><pos>n.</pos> <def>The language of the Cimbri.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimeliarch</ent><br/
<hw>Ci*me"li*arch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cimeliarcha</ets>, Gr. <?/, treasurer.]</ety> <def>A superintendent or keeper of a church's valuables; a churchwarden.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimeter</ent><br/
<hw>Cim"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scimiter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimex</ent><br/
||<hw>Ci"mex</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cimices</plw> <pr>(#)</pr>.</plu> <ety>[L., a bug.]</ety> <fld>(Zool.)</fld> <def>A genus of hemipterous insects of which the bedbug is the best known example. See <er>Bedbug</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimia</ent><br/
<hw>Cim"i*a</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>See <er>Cimbia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimicifuga</ent><br/
<hw>Cimicifuga</hw> <pos>n.</pos> <def>a small genus of perennial herbs of N temperate regions: bugbane.</def><br/
<syn><b>Syn. --</b> genus <gen>Cimicifuga</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cimiss</ent><br/
<hw>Ci"miss</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cimex</ets>, <ets>-icis</ets>, a bug.]</ety> <fld>(Zool.)</fld> <def>The bedbug.</def> <mark>[Obs.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimmerian</ent><br/
<hw>Cim*me"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Cimmerius</ets>.]</ety> <altsp>[Written also <asp>Kimmerian</asp>.]</altsp> <sn>1.</sn> <def>Pertaining to the Cimmerii, a fabulous people, said to have lived, in very ancient times, in profound and perpetual darkness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without any light; intensely dark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In dark <qex>Cimmerian</qex> desert ever dwell.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cimolite</ent><br/
<hw>Cim"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ (sc. <?/) Cimolian earth, fr. <?/, L. <ets>Cimolus</ets>, an island of the Cyclades.]</ety> <fld>(Min.)</fld> <def>A soft, earthy, clayey mineral, of whitish or grayish color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinch</ent><br/
<hw>Cinch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cincha</ets>, fr. L. <ets>cingere</ets> to gird.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A strong saddle girth, as of canvas.</def> <mark>[West. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tight grip.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinch</ent><br/
<hw>Cinch</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cinched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cinch"ing</conjf>.]</vmorph> <sn>1.</sn> <def>To put a cinch upon; to girth tightly.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To get a sure hold upon; to get into a tight place, as for forcing submission.</def> <mark>[Slang, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinch</ent><br/
<hw>Cinch</hw>, <pos>v. i.</pos> <def>To perform the action of cinching; to tighten the cinch; -- often with <xex>up</xex>.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinch</ent><br/
<hw>Cinch</hw>, <pos>n.</pos> <ety>[Cf. <ets>cinch</ets> a girth, a tight grip, as v., to get a sure hold upon; perh. so named from the tactics used in the game; also cf. Sp. <ets>cinco</ets> five (the five spots of the color of the trump being important cards).]</ety> <def>A variety of auction pitch in which a draw to improve the hand is added, and the five of trumps (called <xex>right pedro</xex>) and the five of the same color (called <xex>left pedro</xex>, and ranking between the five and the four of trumps) each count five on the score. Fifty-one points make a game. Called also <altname>double pedro</altname> and <altname>high five</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinch</ent><br/
<hw>Cinch</hw>, <pos>v. t.</pos> <def>In the game of cinch, to protect (a trick) by playing a higher trump than the five.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinchona</ent><br/
<hw>Cin*cho"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named from the wife of Count <etsep>Chinchon</etsep>, viceroy of Peru in the seventeenth century, who by its use was freed from an intermittent fever, and after her return to Spain, contributed to the general propagation of this remedy.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of trees growing naturally on the Andes in Peru and adjacent countries, but now cultivated in the East Indies, producing a medicinal bark of great value.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The bark of any species of <gen>Cinchona</gen> containing three per cent. or more of bitter febrifuge alkaloids; Peruvian bark; Jesuits' bark.</def><-- C. should be capitalized. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinchonaceous</ent><br/
<hw>Cin`cho*na"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Allied or pertaining to cinchona, or to the plants that produce it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinchonic</ent><br/
<hw>Cin*chon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to, or obtained from, cinchona.</def>  <rj><au>Mayne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinchonidine</ent><br/
<hw>Cin*chon"i*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cinchona</er>.]</ety> <fld>(Chem.)</fld> <def>One of the quinine group of alkaloids, found especially in red cinchona bark. It is a white crystalline substance, <chform>C19H22N2O</chform>, with a bitter taste and qualities similar to, but weaker than, quinine; -- sometimes called also <altname>cinchonidia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinchonine</ent><br/
<hw>Cin"cho*nine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cinchona</er>: cf. F. <ets>cinchonine</ets>.]</ety> <fld>(Chem.)</fld> <def>One of the quinine group of alkaloids isomeric with and resembling cinchonidine; -- called also <altname>cinchonia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinchonism</ent><br/
<hw>Cin"cho*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cinchona</er>.]</ety> <fld>(Med.)</fld> <def>A condition produced by the excessive or long-continued use of quinine, and marked by deafness, roaring in the ears, vertigo, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinchonize</ent><br/
<hw>Cin"cho*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To produce cinchonism in; to poison with quinine or with cinchona.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cincinnati epoch</ent><br/
<hw>Cin`cin*na"ti ep"och</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>An epoch at the close of the American lower Silurian system. The rocks are well developed near <etsep>Cincinnati</etsep>, Ohio. The group includes the Hudson River and Lorraine shales of New York.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cincinnus</ent><br/
<hw>Cin*cin"nus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-ni</plw> <pr>(#)</pr>.</plu>  <altsp>[Also <asp>cicinus</asp>, <asp>cicinnus</asp>.]</altsp> <ety>[L., a curl of hair.]</ety> <fld>(Bot.)</fld> <def>A form of monochasium in which the lateral branches arise alternately on opposite sides of the false axis; -- called also <altname>scorpioid cyme</altname>.</def> -- <wordforms><wf>Cin*cin"nal</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinclidae</ent><br/
<hw>Cinclidae</hw> <pos>n.</pos> <def>a natural family of birds comprising the water ouzels, also called dippers, which dive under water in flowing streams.</def><br/
<syn><b>Syn. --</b> family <fam>Cinclidae</fam>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cinclus</ent><br/
<hw>Cinclus</hw> <pos>n.</pos> <def>the type genus of the bird family <fam>Cinclidae</fam>.  It includes the water ouzels <spn>Cinclus aquaticus</spn> of Europe and <spn>Cinclus mexicanus</spn> of western North America.</def><br/
<syn><b>Syn. --</b> genus <gen>Cinclus</gen>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cincture</ent><br/
<hw>Cinc"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cinctura</ets>, fr. <ets>cingere</ets>, <ets>cinctum</ets>, to gird.]</ety> <sn>1.</sn> <def>A belt, a girdle, or something worn round the body, -- as by an ecclesiastic for confining the alb.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which encompasses or incloses; an inclosure.</def> <ldquo/Within the <xex>cincture</xex> of one wall.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>The fillet, listel, or band next to the apophyge at the extremity of the shaft of a column.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinctured</ent><br/
<hw>Cinc"tured</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Having or wearing a cincture or girdle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinder</ent><br/
<hw>Cin"der</hw> <pr>(s<icr/n"d<etil/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>sinder</ets> slag, dross; akin to Icel. <ets>sindr</ets> dross, Sw. <ets>sinder</ets>, G. <ets>sinter</ets>, D. <ets>sintel</ets>; perh. influenced by F. <ets>cendre</ets> ashes, fr. L. <ets>cinis</ets>. Cf. <er>Sinter</er>.]</ety> <sn>1.</sn> <def>Partly burned or vitrified coal, or other combustible, in which fire is extinct.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hot coal without flame; an ember.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A scale thrown off in forging metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The slag of a furnace, or scoriaceous lava from a volcano.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cinder frame</b></col>, <cd>a framework of wire in front of the tubes of a locomotive, to arrest the escape of cinders.</cd> -- <col><b>Cinder notch</b></col> <fld>(Metal.)</fld>, <cd>the opening in a blast furnace, through which melted cinder flows out.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cindery</ent><br/
<hw>Cin"der*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling, or composed of, cinders; full of cinders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinefaction</ent><br/
<hw>Cin`e*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>cinefactio</ets>: L. <ets>cinis</ets> ashes + <ets>facere</ets> to make: cf. F. <ets>cin<eacute/faction</ets>.]</ety> <def>incineration; reduction to ashes.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cine-film</ent><br/
<hw>cine-film</hw> <pos>n.</pos> <def>photographic film several hundred feet long and wound on a spool, used in a movie camera.</def><br/
<syn><b>Syn. --</b> motion-picture film, movie film.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cinema</ent><br/
<hw>cinema</hw> <pos>n.</pos> <sn>1.</sn> <def>the art of creating motion pictures; <as>as, this story would be good <ex>cinema</ex></as>; -- often used in the phrase <altname>the cinema</altname>.</def><br/
<syn><b>Syn. --</b> the film.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a theater where motion pictures are shown.</def><br/
<syn><b>Syn. --</b> movie theater, movie theatre, movie house, picture palace.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>same as <er>motion picture</er>.</def> <mark>[Chiefly Brit.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Cinematical</ent><br/
<ent>Cinematic</ent><br/
<mhw>{ <hw>Cin`e*mat"ic</hw> <pr>(?)</pr>, <hw>Cin`e*mat"ic*al</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <sn>1.</sn> <def>See <er>Kinematic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the cinema; <as>as, <ex>cinematic</ex> entertainment</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cinematics</ent><br/
<hw>Cin`e*mat"ics</hw> <pr>(?)</pr>, <pos>n. sing.</pos> <def>See <er>Kinematics</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinematograph</ent><br/
<hw>Cin`e*mat"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, motion + <ets>-graph</ets>.]</ety> <sn>1.</sn>  <def>an older name for a <altname>movie projector</altname>, a machine, combining magic lantern and kinetoscope features, for projecting on a screen a series of pictures, moved rapidly (25 to 50 frames per second) and intermittently before an objective lens, and producing by persistence of vision the illusion of continuous motion; a moving-picture projector; also, any of several other machines or devices producing moving pictorial effects. Other older names for the <altname>movie projector</altname> are <altname>animatograph</altname>, <altname>biograph</altname>, <altname>bioscope</altname>, <altname>electrograph</altname>, <altname>electroscope</altname>, <altname>kinematograph</altname>, <altname>kinetoscope</altname>, <altname>veriscope</altname>, <altname>vitagraph</altname>, <altname>vitascope</altname>, <altname>zoogyroscope</altname>, <altname>zoopraxiscope</altname>, etc.</def></p>

<p><q>The <qex>cinematograph</qex>, invented by Edison in 1894, is the result of the introduction of the flexible film into photography in place of glass.</q>  <rj><qau>Encyc. Brit.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A camera for taking chronophotographs for exhibition by the instrument described above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinematographer</ent><br/
<hw>Cin`e*ma*tog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who takes motion pictures with a movie camera, especially one expert in the art.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>one who exhibits motion pictures; an operator of a motion picture projector; a projectionist.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p>-- <wordforms><wf>Cin`e*mat`o*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos> -- <wf>Cin`e*mat`o*graph"ic*al*ly</wf> <pr>(#)</pr>, <pos>adv.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinemograph</ent><br/
<hw>Ci*ne"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ motion + <ets>-graph</ets>.]</ety> <def>An integrating anemometer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cineraceous</ent><br/
<hw>Cin`er*a"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cineraceus</ets>, fr. <ets>cinis</ets> ashes.]</ety> <def>Like ashes; ash-colored; cinereous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cineraria</ent><br/
||<hw>Cin`e*ra"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. LL. <ets>cinerarius</ets> pert. to ashes, fr. <ets>cinis</ets> ashes. So called from the ash-colored down on the leaves.]</ety> <fld>(Bot.)</fld> <def>A Linn<ae/an genus of free-flowering composite plants, mostly from South Africa. Several species are cultivated for ornament.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinerary</ent><br/
<hw>Cin"er*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cinerarius</ets>, fr. <ets>cinis</ets> ashes.]</ety> <def>Pertaining to ashes; containing ashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cinerary urns</b></col>, <cd>vessels used by the ancients to preserve the ashes of the dead when burned.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cineration</ent><br/
<hw>Cin`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cinis</ets> ashes: cf. F. <ets>cin<eacute/ration</ets>.]</ety> <def>The reducing of anything to ashes by combustion; cinefaction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinereous</ent><br/
<hw>Ci*ne"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cinereus</ets>, fr. <ets>cinis</ets> ashes.]</ety> <def>Like ashes; ash-colored; grayish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinerescent</ent><br/
<hw>Cin`er*es"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat cinereous; of a color somewhat resembling that of wood ashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cineritious</ent><br/
<hw>Cin`er*i"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cineritius</ets>, <ets>cinericius</ets>, fr. <ets>cinis</ets> ashes.]</ety> <def>Like ashes; having the color of ashes, -- as the cortical substance of the brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinerulent</ent><br/
<hw>Ci*ner"u*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of ashes.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cingalese</ent><br/
<hw>Cin`ga*lese"</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[Cf. F. <ets>Cingalais</ets>.]</ety> <def>A native or natives of Ceylon descended from its primitive inhabitants</def>; also <def2>(<singf>sing.</singf>), <def>the language of the Cingalese.</def></def2> -- <def2><pos>a.</pos> <def>Of or pertaining to the Cingalese.</def></def2> <altsp>[Written also <asp>Singhalese</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Ceylonese</xex> is applied to the inhabitants of the island in general.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cingle</ent><br/
<hw>Cin"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cingula</ets>, <ets>cingulum</ets>, fr. <ets>cingere</ets> to gird.]</ety> <def>A girth.</def> <mark>[R.]</mark> <see>See <er>Surcingle</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cingulum</ent><br/
||<hw>Cin"gu*lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a girdle.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A distinct girdle or band of color; a raised spiral line as seen on certain univalve shells.</def> <sd>(b)</sd> <def>The clitellus of earthworms.</def> <sd>(c)</sd> <def>The base of the crown of a tooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnabar</ent><br/
<hw>Cin"na*bar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cinnabaris</ets>, Gr. <?/; prob. of Oriental origin; cf. Per. <ets>qinb<amac/r</ets>, Hind. <ets>shangarf</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Min.)</fld> <def>Red sulphide of mercury, occurring in brilliant red crystals, and also in red or brown amorphous masses. It is used in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The artificial red sulphide of mercury used as a pigment; vermilion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cinnabar Gr<ae/corum</b></col> <pr>(<?/)</pr>. <ety>[L. <ets>Graecorum</ets>, gen. pl., of the Greeks.]</ety> <fld>(Med.)</fld> <cd>Same as <er>Dragon's blood</er>.</cd> -- <col><b>Green cinnabar</b></col>, <cd>a green pigment consisting of the oxides of cobalt and zinc subjected to the action of fire.</cd> -- <col><b>Hepatic cinnabar</b></col> <fld>(Min.)</fld>, <cd>an impure cinnabar of a liver-brown color and submetallic luster.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnabarine</ent><br/
<hw>Cin"na*ba*rine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cinabarin</ets>.]</ety> <def>Pertaining to, or resembling, cinnabar; consisting of cinnabar, or containing it; <as>as, <ex>cinnabarine</ex> sand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnamene</ent><br/
<hw>Cin"na*mene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cinnamic</er>.]</ety> <fld>(Chem.)</fld> <def>Styrene (which was formerly called <xex>cinnamene</xex> because obtained from cinnamic acid). See <er>Styrene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnamic</ent><br/
<hw>Cin*nam"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cinnamon</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or obtained from, cinnamon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cinnamic acid</b></col> <fld>(Chem.)</fld>, <cd>a white, crystalline, odorless substance. <chform>C6H5.C2H2C2H2.CO2H</chform>, formerly obtained from storax and oil of cinnamon, now made from certain benzene derivatives in large quantities, and used for the artificial production of indigo.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnamomic</ent><br/
<hw>Cin`na*mom"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cinnamomum</ets> cinnamon.]</ety> <fld>(Chem.)</fld> <def>See <er>Cinnamic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnamon</ent><br/
<hw>Cin"na*mon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>qinn<amac/m<omac/n</ets>; cf. Gr. <?/, <?/, <ets>cinnamomum</ets>, cinnamon. The Heb. word itself seems to have been borrowed from some other language; cf. Malay <ets>k<amac/j<umac/ m<amac/nis</ets> sweet wood.]</ety> <sd>(a)</sd> <def>The inner bark of the shoots of <spn>Cinnamomum Zeylanicum</spn>, a tree growing in Ceylon. It is aromatic, of a moderately pungent taste, and is one of the best cordial, carminative, and restorative spices.</def> <sd>(b)</sd> <def>Cassia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cinnamon stone</b></col> <fld>(Min.)</fld>, <cd>a variety of garnet, of a cinnamon or hyacinth red color, sometimes used in jewelry.</cd> -- <col><b>Oil of cinnamon</b></col>, <cd>a colorless aromatic oil obtained from cinnamon and cassia, and consisting essentially of cinnamic aldehyde, <chform>C6H5.C2H2.CHO</chform>.</cd> -- <col><b>Wild cinnamon</b></col>. <cd>See <er>Canella</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnamone</ent><br/
<hw>Cin"na*mone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cinnamic</ets> + <ets>-one</ets>.]</ety> <def>A yellow crystalline substance, <chform>(C6H5.C2H2)2CO</chform>, the ketone of cinnamic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 257 --></p>

<p><ent>Cinnamyl</ent><br/
<hw>Cin"na*myl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cinnamic</ets> + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hypothetical radical, <chform>(C6H5.C2H2)2C</chform>, of cinnamic compounds.</def> <altsp>[Formerly written also <asp>cinnamule</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinnoline</ent><br/
<hw>Cin"no*line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cinn</ets>amic + quin<ets>oline</ets>.]</ety> <def>A nitrogenous organic base, <chform>C8H6N2</chform>, analogous to quinoline, obtained from certain complex diazo compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinque</ent><br/
<hw>Cinque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cinq</ets>, fr. L. <ets>quinque</ets> five. See <er>Five</er>.]</ety> <def>Five; the number five in dice or cards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinquecentist</ent><br/
<hw>Cin`que*cen"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An Italian of the sixteenth century, esp. a poet or artist.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A student or imitator of the art or literature of the Cinquecento.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cinquecento</ent><br/
||<hw>Cin`que*cen"to</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <ety>[It., five hundred, abbrev. for fifteen hundred. The <ets>Cinquecento</ets> style was so called because it arose after the year 1500.]</ety> <def>The sixteenth century, when applied to Italian art or literature; <as>as, the sculpture of the <ex>Cinquecento</ex>; <ex>Cinquecento</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinquefoil</ent><br/
<hw>Cinque"foil`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cinque</ets> five + <ets>foil</ets>, F. <ets>feuille</ets> leaf. See <er>Foil</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The name of several different species of the genus <gen>Potentilla</gen>; -- also called <altname>five-finger</altname>, because of the resemblance of its leaves to the fingers of the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An ornamental foliation having five points or cups, used in windows, panels, etc.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Marsh cinquefoil</b></col>, <cd>the <spn>Potentilla palustris</spn>, a plant with purple flowers which grows in fresh-water marshes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinque-pace</ent><br/
<hw>Cinque"-pace`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cinque</ets> + <ets>pace</ets>.]</ety> <def>A lively dance (called also <altname>galliard</altname>), the steps of which were regulated by the number five.</def> <mark>[Obs.]</mark>  <rj><au>Nares. Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinque Ports</ent><br/
<hw>Cinque" Ports`</hw> <pr>(?)</pr>. <ety>[<ets>Cinque</ets> + <ets>port</ets>.]</ety> <fld>(Eng. Hist.)</fld> <def>Five English ports, to which peculiar privileges were anciently accorded; -- viz., Hastings, Romney, Hythe, Dover, and Sandwich; afterwards increased by the addition of Winchelsea, Rye, and some minor places.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Baron of the Cinque Ports</b></col>. <cd>See under <er>Baron</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinque-spotted</ent><br/
<hw>Cinque"-spot`ted</hw>, <pos>a.</pos> <def>Five-spotted.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinter</ent><br/
<hw>Cin"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cintre</ets>.]</ety> <fld>(Arch.)</fld> <def>See <er>Center</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cinura</ent><br/
||<hw>Ci*nu"ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ To move + <grk>o'yra`</grk> tail.]</ety> <fld>(Zool.)</fld> <def>The group of Thysanura which includes <xex>Lepisma</xex> and allied forms; the bristletails. See <er>Bristletail</er>, and <er>Lepisma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cion</ent><br/
<hw>Ci"on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cion</ets>. See <er>Scion</er>.]</ety> <def>See <er>Scion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cion</qex> overruleth the stock; and the stock is but passive, and giveth aliment, but no motion, to the graft.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipher</ent><br/
<hw>Ci"pher</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cifre</ets> zero, F. <ets>Chiffre</ets> figure (cf. Sp.<ets>cifra</ets>, LL. <ets>cifra</ets>), fr. Ar. <ets><cced/ifrun</ets>, <ets><cced/afrun</ets>, empty, cipher, zero, fr. <ets><cced/afira</ets> to be empty. Cf. <er>Zero</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Arith.)</fld> <def>A character [0] which, standing by itself, expresses nothing, but when placed at the right hand of a whole number, increases its value tenfold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, or that which, has no weight or influence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here he was a mere <xex>cipher</xex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A character in general, as a figure or letter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This wisdom began to be written in <qex>ciphers</qex> and characters and letters bearing the forms of creatures.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A combination or interweaving of letters, as the initials of a name; a device; a monogram; <as>as, a painter's <ex>cipher</ex>, an engraver's <ex>cipher</ex>, etc.</as>  The cut represents the initials <xex>N. W</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A private alphabet, system of characters, or other mode of writing, contrived for the safe transmission of secrets; also, a writing in such characters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His father . . . engaged him when he was very young to write all his letters to England in <qex>cipher</qex>.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cipher key</b></col>, <cd>a key to assist in reading writings in cipher.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipher</ent><br/
<hw>Ci"pher</hw>, <pos>a.</pos> <def>Of the nature of a cipher; of no weight or influence.</def> <ldquo/Twelve <xex>cipher</xex> bishops.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipher</ent><br/
<hw>Ci"pher</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Ciphered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Ciphering</conjf>.]</vmorph> <def>To use figures in a mathematical process; to do sums in arithmetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T was certain he could write and <qex>cipher</qex> too.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipher</ent><br/
<hw>Ci"pher</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To write in occult characters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His notes he <qex>ciphered</qex> with Greek characters.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To get by ciphering; <as>as, to cipher out the answer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To decipher.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To designate by characters.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipherer</ent><br/
<hw>Ci"pher*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who ciphers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipherhood</ent><br/
<hw>Ci"pher*hood</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Nothingness.</def> <mark>[R.]</mark>  <rj><au>Goodwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cipolin</ent><br/
<hw>Cip"o*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>cippollino</ets>, prop., a little onion, fr. <ets>cipolla</ets> onion (cf. E. <ets>cibol</ets>). So called because its veins consist, like onions, of different strata, one lying upon another.]</ety> <fld>(Min.)</fld> <def>A whitish marble, from Rome, containiing pale greenish zones. It consists of calcium carbonate, with zones and cloudings of talc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cippus</ent><br/
||<hw>Cip"pus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cippi</plw> <pr>(#)</pr>.</plu> <ety>[L., stake, post.]</ety> <def>A small, low pillar, square or round, commonly having an inscription, used by the ancients for various purposes, as for indicating the distances of places, for a landmark, for sepulchral inscriptions, etc.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circ</ent><br/
<hw>Circ</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Circus</er>.]</ety> <def>An amphitheatrical circle for sports; a circus.</def> <mark>[R.]</mark>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>circa</ent><br/
<hw>cir"ca</hw> <pr>(s<etil/r"k<adot/)</pr>, <pos>adv.</pos>, <pos>prep.</pos> <ety>[L.]</ety> <def>Approximately; about; commonly abbreviated <altname>ca.</altname>; -- used especially before dates and numerical measures; <as>as, he was born circa 1650; ca. 50 feet high</as>.</def><br/
<syn><b>Syn. --</b> ca.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Circar</ent><br/
||<hw>Cir*car"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sircar</er>.]</ety> <def>A district, or part of a province. See <er>Sircar</er>.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circassian</ent><br/
<hw>Cir*cas"sian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Circassia, in Asia.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Circassia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circean</ent><br/
<hw>Cir*ce"an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Circaeus</ets>.]</ety> <def>Having the characteristics of Circe, daughter of Sol and Perseis, a mythological enchantress, who first charmed her victims and then changed them to the forms of beasts; pleasing, but noxious; <as>as, a <ex>Circean</ex> draught</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circensian</ent><br/
<ent>Circensial</ent><br/
<mhw>{ <hw>Cir*cen"sial</hw> <pr>(?)</pr>, <hw>Cir*cen"sian</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>Circensis</ets>, ludi <ets>Circenses</ets>, the games in the Circus Maximus.]</ety> <def>Of or pertaining to, or held in, the Circus, In Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pleasure of the <qex>Circensian</qex> shows.</q> <rj><qau>Holyday.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circinal</ent><br/
<hw>Cir"ci*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a circle.]</ety> <fld>(Bot.)</fld> <def>Circinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circinate</ent><br/
<hw>Cir"ci*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circinatus</ets>, p. p. of <ets>circinare</ets> to make round, fr. <ets>circinus</ets> a pair of compasses, from Gr. <?/ a circle.]</ety> <fld>(Bot.)</fld> <def>Rolled together downward, the tip occupying the center; -- a term used in reference to foliation or leafing, as in ferns.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circinate</ent><br/
<hw>Cir"ci*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make a circle around; to encompass.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circination</ent><br/
<hw>Cir`ci*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circinatio</ets> circle.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An orbicular motion.</def> <mark>[Obs.]</mark>  <rj><au>bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A circle; a concentric layer.</def> <mark>[Obs.]</mark> <ldquo/The <xex>circinations</xex> and spherical rounds of onions.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circle</ent><br/
<hw>Cir"cle</hw> <pr>(s<etil/r"k'l)</pr>, <pos>n.</pos> <ety>[OE. <ets>cercle</ets>, F. <ets>cercle</ets>, fr. L. <ets>circulus</ets> (Whence also AS. <ets>circul</ets>), dim. of <ets>circus</ets> circle, akin to Gr. <grk>kri`kos</grk>, <grk>ki`rkos</grk>, circle, ring. Cf. <er>Circus</er>, <er>Circum-</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A plane figure, bounded by a single curve line called its <part>circumference</part>, every part of which is equally distant from a point within it, called the <part>center</part>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The line that bounds such a figure; a circumference; a ring.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>An instrument of observation, the graduated limb of which consists of an entire circle.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When it is fixed to a wall in an observatory, it is called a <stype>mural circle</stype>; when mounted with a telescope on an axis and in Y's, in the plane of the meridian, a <stype>meridian  circle</stype> or <stype>transit circle</stype>; when involving the principle of reflection, like the sextant, a <stype>reflecting circle</stype>; and when that of repeating an angle several times continuously along the graduated limb, a <stype>repeating circle</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A round body; a sphere; an orb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is he that sitteth upon the <qex>circle</qex> of the earth.</q> <rj><qau>Is. xi. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Compass; circuit; inclosure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>circle</qex> of this forest.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A company assembled, or conceived to assemble, about a central point of interest, or bound by a common tie; a class or division of society; a coterie; a set.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As his name gradually became known, the <qex>circle</qex> of his acquaintance widened.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A circular group of persons; a ring.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A series ending where it begins, and repeating itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus in a <qex>circle</qex> runs the peasant's pain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Logic)</fld> <def>A form of argument in which two or more unproved statements are used to prove each other; inconclusive reasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That heavy bodies descend by gravity; and, again, that gravity is a quality whereby a heavy body descends, is an impertinent <qex>circle</qex> and teaches nothing.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Indirect form of words; circumlocution.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Has he given the lie,<br/
In <qex>circle</qex>, or oblique, or semicircle.</q> <rj><qau>J. Fletcher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A territorial division or district.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <col><b>The Circles of the Holy Roman Empire</b></col>, ten in number, were those principalities or provinces which had seats in the German Diet.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Azimuth circle</b></col>. <cd>See under <er>Azimuth</er>.</cd> -- <col><b>Circle of altitude</b></col> <fld>(Astron.)</fld>, <cd>a circle parallel to the horizon, having its pole in the zenith; an almucantar.</cd> -- <col><b>Circle of curvature</b></col>. <cd>See <cref>Osculating circle of a curve</cref> (Below).</cd> -- <col><b>Circle of declination</b></col>. <cd>See under <er>Declination</er>.</cd> -- <col><b>Circle of latitude</b></col>. <sd>(a)</sd> <fld>(Astron.)</fld> <cd>A great circle perpendicular to the plane of the ecliptic, passing through its poles.</cd> <sd>(b)</sd> <fld>(Spherical Projection)</fld> <cd>A small circle of the sphere whose plane is perpendicular to the axis.</cd> -- <col><b>Circles of longitude</b></col>, <cd>lesser circles parallel to the ecliptic, diminishing as they recede from it.</cd> -- <col><b>Circle of perpetual apparition</b></col>, <cd>at any given place, the boundary of that space around the elevated pole, within which the stars never set. Its distance from the pole is equal to the latitude of the place.</cd> -- <col><b>Circle of perpetual occultation</b></col>, <cd>at any given place, the boundary of the space around the depressed pole, within which the stars never rise.</cd> -- <col><b>Circle of the sphere</b></col>, <cd>a circle upon the surface of the sphere, called a <xex>great circle</xex> when its plane passes through the center of the sphere; in all other cases, a <xex>small circle</xex>.</cd> -- <col><b>Diurnal circle</b></col>. <cd>See under <er>Diurnal</er>.</cd> -- <col><b>Dress circle</b></col>, <cd>a gallery in a theater, generally the one containing the prominent and more expensive seats.</cd> -- <col><b>Druidical circles</b></col> <fld>(Eng. Antiq.)</fld>, <cd>a popular name for certain ancient inclosures formed by rude stones circularly arranged, as at Stonehenge, near Salisbury.</cd> -- <col><b>Family circle</b></col>, <cd>a gallery in a theater, usually one containing inexpensive seats.</cd> -- <col><b>Horary circles</b></col> <fld>(Dialing)</fld>, <cd>the lines on dials which show the hours.</cd> -- <col><b>Osculating circle of a curve</b></col> <fld>(Geom.)</fld>, <cd>the circle which touches the curve at some point in the curve, and close to the point more nearly coincides with the curve than any other circle. This circle is used as a measure of the curvature of the curve at the point, and hence is called <xex>circle of curvature</xex>.</cd> -- <col><b>Pitch circle</b></col>. <cd>See under <er>Pitch</er>.</cd> -- <col><b>Vertical circle</b></col>, <cd>an azimuth circle.</cd> -- <mcol><col><b>Voltaic circuit</b></col> <it>or</it> <col><b>Voltaic circle</b></col></mcol>. <cd>See under <er>Circuit</er>.</cd> -- <col><b>To square the circle</b></col>. <cd>See under <er>Square</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Ring; circlet; compass; circuit; inclosure.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circle</ent><br/
<hw>Cir"cle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>cerclen</ets>, F. <ets>cercler</ets>, fr. L. <ets>circulare</ets> to make round. See <er>Circle</er>, <pos>n.</pos>, and cf. <er>Circulate</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To move around; to revolve around.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Other planets <qex>circle</qex> other suns.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To encompass, as by a circle; to surround; to inclose; to encircle.</def>  <rj><au>Prior. Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their heads are <qex>circled</qex> with a short turban.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So he lies, <qex>circled</qex> with evil.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To circle in</b></col>, <cd>to confine; to hem in; to keep together; as, to <xex>circle</xex> bodies <xex>in</xex>.</cd>  <rj><au>Sir K. Digby.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circle</ent><br/
<hw>Cir"cle</hw>, <pos>v. i.</pos> <def>To move circularly; to form a circle; to circulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy name shall <qex>circle</qex> round the gaping through.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circled</ent><br/
<hw>Cir"cled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of a circle; round.</def> <ldquo/Monthly changes in her <xex>circled</xex> orb.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circler</ent><br/
<hw>Cir"cler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mean or inferior poet, perhaps from his habit of wandering around as a stroller; an itinerant poet. Also, a name given to the cyclic poets. See under <er>Cyclic</er>, <pos>a.</pos></def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circlet</ent><br/
<hw>Cir"clet</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A little circle; esp., an ornament for the person, having the form of a circle; that which encircles, as a ring, a bracelet, or a headband.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her fair locks in <qex>circlet</qex> be enrolled.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A round body; an orb.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fairest of stars . . . that crown'st the smiling morn<br/
With thy bright <qex>circlet</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A circular piece of wood put under a dish at table.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circocele</ent><br/
<hw>Cir"co*cele</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cirsocele</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuit</ent><br/
<hw>Cir"cuit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>circuit</ets>, fr. L. <ets>circuitus</ets>, fr. <ets>circuire</ets> or <ets>circumire</ets> to go around; <ets>circum</ets> around + <ets>ire</ets> to go.]</ety> <sn>1.</sn> <def>The act of moving or revolving around, or as in a circle or orbit; a revolution; <as>as, the periodical <ex>circuit</ex> of the earth round the sun</as>.</def>  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The circumference of, or distance round, any space; the measure of a line round an area.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>circuit</qex> or compass of Ireland is 1,800 miles.</q> <rj><qau>J. Stow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which encircles anything, as a ring or crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The golden <qex>circuit</qex> on my head.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The space inclosed within a circle, or within limits.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>circuit</qex> wide inclosed with goodliest trees.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A regular or appointed journeying from place to place in the exercise of one's calling, as of a judge, or a preacher.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <sd>(a)</sd> <fld>(Law)</fld> <def>A certain division of a state or country, established by law for a judge or judges to visit, for the administration of justice.</def> <au>Bouvier</au>. <sd>(b)</sd> <fld>(Methodist Church)</fld> <def>A district in which an itinerant preacher labors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Circumlocution.</def> <mark>[Obs.]</mark> <ldquo/Thou hast used no <xex>circuit</xex> of words.<rdquo/  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Circuit court</b></col> <fld>(Law)</fld>, <cd>a court which sits successively in different places in its circuit (see <er>Circuit</er>, 6). In the United States, the federal circuit courts are commonly presided over by a judge of the supreme court, or a special <ecol><b>circuit judge</b></ecol>, together with the judge of the district court. They have jurisdiction within statutory limits, both in law and equity, in matters of federal cognizance. Some of the individual States also have circuit courts, which have general statutory jurisdiction of the same class, in matters of State cognizance.</cd> -- <mcol><col><b>Circuit of action</b></col> <it>or</it> <col><b>Circuity of action</b></col></mcol> <fld>(Law)</fld>, <cd>a longer course of proceedings than is necessary to attain the object in view.</cd> -- <col><b>To make a circuit</b></col>, <cd>to go around; to go a roundabout way.</cd> -- <mcol><col><b>Voltaic circle</b></col> <it>or</it> <col><b>Galvanic circle</b></col> <it>or</it> <col><b>Voltaic circuit</b></col> <it>or</it> <col><b>Galvanic circuit</b></col></mcol>, <cd>a continous electrical communication between the two poles of a battery; an arrangement of voltaic elements or couples with proper conductors, by which a continuous current of electricity is established.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuit</ent><br/
<hw>Cir"cuit</hw>, <pos>v. i.</pos> <def>To move in a circle; to go round; to circulate.</def> <mark>[Obs.]</mark>  <rj><au>J. Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuit</ent><br/
<hw>Cir"cuit</hw>, <pos>v. t.</pos> <def>To travel around.</def> <mark>[Obs.]</mark> <ldquo/Having <xex>circuited</xex> the air.<rdquo/  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>circuit breaker</ent><br/
<hw>cir"cuit break"er</hw>, <pos>n.</pos> <fld>(Elec.)</fld> <def>A device contained within an electrical circuit designed to interrupt the circuit when the current exceeds a preset value; it is sometimes called a <altname>contact breaker</altname>.  Its function is to prevent fire or damage to the circuit or the devices on the circuit which could be caused by excess current, such as that caused by a short-circuit.  It differs from a <contr>fuse</contr> in not having a conducting element which melts (see 2nd <er>fuse</er>, <pos>n.</pos>).  Circuit breakers are designed to be easily reset, i. e. to reclose the circuit after it has been opened by the circuit breaker; this is usually accomplished by simply moving a switch back and forth. They are commonly used in buildings to protect individual electrical lines, and are often contained in groups within a box called a <partof>circuit breaker panel</partof>, which divides the current from a main electrical line into multiple circuits each protected by a separate circuit breaker. Less commonly they may be found on individual devices.</def><br/
[<source>PJC</source>]</p>

<p><ent>Circuiteer</ent><br/
<hw>Cir`cuit*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A circuiter.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuiter</ent><br/
<hw>Cir"cuit*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who travels a circuit, as a circuit judge.</def> <mark>[R.]</mark>  <rj><au>R. Whitlock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuition</ent><br/
<hw>Cir`cu*i"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circuitio</ets>. See <er>Circuit</er>.]</ety> <def>The act of going round; circumlocution.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuitous</ent><br/
<hw>Cir*cu"i*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>circuitosus</ets>.]</ety> <def>Going round in a circuit; roundabout; indirect; <as>as, a <ex>circuitous</ex> road; a <ex>circuitous</ex> manner of accomplishing an end</as>.</def> -- <wordforms><wf>Cir*cu"i*tous*ly</wf>, <pos>adv.</pos> -- <wf>Cir*cu"i*tous*ness</wf>, <pos>n.</pos></wordforms></p>

<p><syn><b>Syn.</b> -- Tortuous; winding; sinuous; serpentine.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuity</ent><br/
<hw>Cir*cu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A going round in a circle; a course not direct; a roundabout way of proceeding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulable</ent><br/
<hw>Cir"cu*la*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be circulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circular</ent><br/
<hw>Cir"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circularis</ets>, fr. <ets>circulus</ets> circle: cf. F. <ets>circulaire</ets>. See <er>Circle</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>In the form of, or bounded by, a circle; round.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>repeating itself; ending in itself; reverting to the point of beginning; hence, illogical; inconclusive; <as>as, <ex>circular</ex> reasoning</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Adhering to a fixed circle of legends; cyclic; hence, mean; inferior. See <cref>Cyclic poets</cref>, under <er>Cyclic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Had Virgil been a <qex>circular</qex> poet, and closely adhered to history, how could the Romans have had Dido?</q> <rj><qau>Dennis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Addressed to a circle, or to a number of persons having a common interest; circulated, or intended for circulation; <as>as, a <ex>circular</ex> letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A proclamation of Henry III., . . . doubtless <qex>circular</qex> throughout England.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Perfect; complete.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A man so absolute and <qex>circular</qex><br/
In all those wished-for rarities that may take<br/
A virgin captive.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Circular are</b></col>, <cd>any portion of the circumference of a circle.</cd> -- <col><b>Circular cubics</b></col> <fld>(Math.)</fld>, <cd>curves of the third order which are imagined to pass through the two circular points at infinity.</cd> -- <col><b>Circular functions</b></col>. <fld>(Math.)</fld> <cd>See under <er>Function</er>.</cd> -- <col><b>Circular instruments</b></col>, <cd>mathematical instruments employed for measuring angles, in which the graduation extends round the whole circumference of a circle, or 360<deg/.</cd> -- <col><b>Circular lines</b></col>, <cd>straight lines pertaining to the circle, as sines, tangents, secants, etc.</cd> -- <mcol><col><b>Circular note</b></col><col> <it>or</it> <b>Circular letter</b></col></mcol>. <sd>(a)</sd> <fld>(Com.)</fld> <cd>See under <er>Credit</er>.</cd> <sd>(b)</sd> <fld>(Diplomacy)</fld> <cd>A letter addressed in identical terms to a number of persons.</cd> -- <col><b>Circular numbers</b></col> <fld>(Arith.)</fld>, <cd>those whose powers terminate in the same digits as the roots themselves; as 5 and 6, whose squares are 25 and 36.</cd> <au>Bailey.</au> <au>Barlow.</au> -- <col><b>Circular points at infinity</b></col> <fld>(Geom.)</fld>, <cd>two imaginary points at infinite distance through which every circle in the plane is, in the theory of curves, imagined to pass.</cd> -- <col><b>Circular polarization</b></col>. <fld>(Min.)</fld> <cd>See under <er>Polarization</er>.</cd> -- <mcol><col><b>Circular sailing</b></col> <it>or</it> <col><b>Globular sailing</b></col></mcol> <fld>(Naut.)</fld>, <cd>the method of sailing by the arc of a great circle.</cd> -- <col><b>Circular saw</b></col>. <cd>See under <er>Saw</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circular</ent><br/
<hw>Cir"cu*lar</hw>, <pos>n.</pos> <ety>[Cf. (for sense 1) F. <ets>circulaire</ets>, lettre <ets>circulaire</ets>. See <er>Circular</er>, <pos>a.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A circular letter, or paper, usually printed, copies of which are addressed or given to various persons; <as>as, a business <ex>circular</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sleeveless cloak, cut in circular form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circularity</ent><br/
<hw>Cir`cu*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>circularitas</ets>.]</ety> <def>The quality or state of being circular; a circular form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>circular-knit</ent><br/
<hw>circular-knit</hw> <pos>adj.</pos> <def>knit on a circular form so as to have no seams; -- said of stockings.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Circularly</ent><br/
<hw>Cir"cu*lar*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a circular manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulary</ent><br/
<hw>Cir"cu*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Circular; illogical.</def> <mark>[Obs. & .]</mark> <ldquo/Cross and <xex>circulary</xex> speeches.<rdquo/  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulate</ent><br/
<hw>Cir"cu*late</hw> <pr>(#)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circulated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Circulating</conjf>.]</vmorph> <ety>[L. <ets>circulatus</ets>, p. p. of <ets>circulare</ets>, v. t., to surround, make round, <ets>circulari</ets>, v. i., to gather into a circle. See <er>Circle</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To move in a circle or circuitously; to move round and return to the same point; <as>as, the blood <ex>circulates</ex> in the body</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass from place to place, from person to person, or from hand to hand; to be diffused; <as>as, money <ex>circulates</ex>; a story <ex>circulates.</ex></as></def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 258 --></p>

<p><cs><col><b>Circulating decimal</b></col>. <cd>See <er>Decimal</er>.</cd> -- <col><b>Circulating library</b></col>, <cd>a library whose books are loaned to the public, usually at certain fixed rates.</cd> -- <col><b>Circulating medium</b></col>. <cd>See <er>Medium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulate</ent><br/
<hw>Cir"cu*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cause to pass from place to place, or from person to person; to spread; <as>as, to <ex>circulate</ex> a report; to <ex>circulate</ex> bills of credit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Circulating pump</b></col>. <cd>See under <er>Pump</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- To spread; diffuse; propagate; disseminate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>circulating</ent><br/
<hw>circulating</hw> <pos>adj.</pos> <sn>1.</sn> <def>moving or flowing in a circuit and returning to the same point; <as>as, steam <ex>circulating</ex> through the pipes; the <ex>circulating</ex> thyroid hormones</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>passing from one to another.</def> <mark>[prenominal]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Circulation</ent><br/
<hw>Cir`cu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circulatio</ets>: cf. F. <ets>circulation</ets>.]</ety> <sn>1.</sn> <def>The act of moving in a circle, or in a course which brings the moving body to the place where its motion began.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This continual <qex>circulation</qex> of human things.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of passing from place to place or person to person; free diffusion; transmission.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The true doctrines of astronomy appear to have had some popular <qex>circulation</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Currency; circulating coin; notes, bills, etc., current for coin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The extent to which anything circulates or is circulated; the measure of diffusion; <as>as, the <ex>circulation</ex> of a newspaper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physiol.)</fld> <def>The movement of the blood in the blood-vascular system, by which it is brought into close relations with almost every living elementary constituent. Also, the movement of the sap in the vessels and tissues of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulative</ent><br/
<hw>Cir"cu*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Promoting circulation; circulating.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulator</ent><br/
<hw>Cir"cu*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf.  L. <ets>circulator</ets> a peddler.]</ety> <def>One who, or that which, circulates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulatorious</ent><br/
<hw>Cir`cu*la*to"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Travelling from house to house or from town to town; itinerant.</def> <mark>[Obs.]</mark> <ldquo/<xex>Circulatorious</xex> jugglers.<rdquo/  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulatory</ent><br/
<hw>Cir"cu*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circulatorius</ets> pert. to a mountebank: cf. F. <ets>circulatoire</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Circular; <as>as, a <ex>circulatory</ex> letter</as>.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Circulating, or going round.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Subserving the purposes of circulation; <as>as, <ex>circulatory</ex> organs</as>; of or pertaining to the organs of circulation; <as>as, <ex>circulatory</ex> diseases</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulatory</ent><br/
<hw>Cir"cu*la*to*ry</hw>, <pos>n.</pos> <def>A chemical vessel consisting of two portions unequally exposed to the heat of the fire, and with connecting pipes or passages, through which the fluid rises from the overheated portion, and descends from the relatively colder, maintaining a circulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circulet</ent><br/
<hw>Cir"cu*let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A circlet.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circuline</ent><br/
<hw>Cir"cu*line</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Proceeding in a circle; circular.</def>  <mark>[Obs.]</mark> <ldquo/With motion <xex>circuline</xex><rdquo/.  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circum-</ent><br/
<hw>Cir"cum-</hw> <pr>(?)</pr>. <ety>[Akin to <ets>circle</ets>, <ets>circus</ets>.]</ety> <def>A Latin preposition, used as a prefix in many English words, and signifying <xex>around</xex> or <xex>about</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumagitate</ent><br/
<hw>Cir`cum*ag"i*tate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>circum + agitate</ets>.]</ety> <def>To agitate on all sides.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumambage</ent><br/
<hw>Cir`cum*am"bage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. 
<ets>circum-</ets> + <ets>ambage</ets>, obs. sing. of <ets>ambages</ets>.]</ety> <def>A roundabout or indirect course; indirectness.</def> <mark>[Obs.]</mark>  <rj><au>S. Richardson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumambiency</ent><br/
<hw>Cir`cum*am"bi*en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of surrounding or encompassing.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumambient</ent><br/
<hw>Cir`cum*am"bi*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>ambient</ets>.]</ety> <def>Surrounding; inclosing or being on all sides; encompassing.</def> <ldquo/The <xex>circumambient heaven</xex>.<rdquo/  <rj><au>J. Armstrong.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumambulate</ent><br/
<hw>Cir`cum*am"bu*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>circumambulatus</ets>, p. p. of <ets>circumambulare</ets> to walk around; <ets>circum + ambulare</ets>. See <er>Ambulate</er>.]</ety> <def>To walk round about.</def> -- <wordforms><wf>Cir`cum*am`bu*la"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumbendibus</ent><br/
<hw>Cir`cum*bend"i*bus</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A roundabout or indirect way.</def> <mark>[Jocular]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumcenter</ent><br/
<hw>Cir`cum*cen"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>The center of a circle that circumscribes a triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumcise</ent><br/
<hw>Cir"cum*cise</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumcised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circumcising</conjf>.]</vmorph> <ety>[L. <ets>circumcisus</ets>, p. p. of <ets>circumcidere</ets> to cut around, to circumcise; <ets>circum + caedere</ets> to cut; akin to E. <ets>c<ae/sura</ets>, homi<ets>cide</ets>, con<ets>cise</ets>, and prob. to <ets>shed</ets>, v. t.]</ety> <sn>1.</sn> <def>To cut off the prepuce of foreskin of, in the case of males, and the internal labia of, in the case of females.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Script.)</fld> <def>To purify spiritually.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumciser</ent><br/
<hw>Cir"cum*ci`ser</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who performs circumcision.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumcision</ent><br/
<hw>Cir`cum*cision</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumcisio</ets>.]</ety> <sn>1.</sn> <def>The act of cutting off the prepuce or foreskin of males, or the internal labia of females.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>circumcision</xex> of males is practiced as a religious rite by the Jews, Mohammedans, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Script.)</fld> <sd>(a)</sd> <def>The Jews, as a circumcised people.</def> <sd>(b)</sd> <def>Rejection of the sins of the flesh; spiritual purification, and acceptance of the Christian faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumclusion</ent><br/
<hw>Cir`cum*clu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumcludere</ets>, <ets>-clusum</ets>, to inclose.]</ety> <def>Act of inclosing on all sides.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumcursation</ent><br/
<hw>Cir`cum*cur*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumcursare</ets>, <ets>-satum</ets>, to run round about.]</ety> <def>The act of running about; also, rambling language.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumdenudation</ent><br/
<hw>Cir`cum*den`u*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>denudation</ets>.]</ety> <fld>(Geol.)</fld> <def>Denudation around or in the neighborhood of an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hills of circumdenudation</b></col>, <cd>hills which have been produced by surface erosion; the elevations which have been left, after denudation of a mass of high ground.</cd>  <rj><au>Jukes.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumduce</ent><br/
<hw>Cir`cum*duce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Circumduct</er>.]</ety> <fld>(Scots Law)</fld> <def>To declare elapsed, as the time allowed for introducing evidence.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumduct</ent><br/
<hw>Cir`cum*duct"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>circumductus</ets>, p. p. of <ets>circumducere</ets> to lead around; <ets>circum + ducere</ets> to lead.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To lead about; to lead astray.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To contravene; to nullify; <as>as, to <ex>circumduct</ex> acts of judicature</as>.</def> <mark>[Obs.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumduction</ent><br/
<hw>Cir`cum*duc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumductio</ets>.]</ety> <sn>1.</sn> <def>A leading about; circumlocution.</def> <mark>[R.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An annulling; cancellation.</def> <mark>[R.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physiol.)</fld> <def>The rotation of a limb round an imaginary axis, so as to describe a conical surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumesophagal</ent><br/
<hw>Cir`cum*e*soph"a*gal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>esophagal</ets>.]</ety> <fld>(Anat.)</fld> <def>Surrounding the esophagus; -- in <fld>(Zool.)</fld> said of the nerve commissures and ganglia of arthropods and mollusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumesophageal</ent><br/
<hw>Cir`cum*e`so*phag"e*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Circumesophagal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfer</ent><br/
<hw>Cir"cum*fer</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>circumferre</ets>; <ets>circum-</ets> + <ets>ferre</ets> to bear. See 1st <er>Bear</er>.]</ety> <def>To bear or carry round.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumference</ent><br/
<hw>Cir*cum"fer*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumferentia</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The line that goes round or encompasses a circular figure; a periphery.</def>  <rj><au>Millon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A circle; anything circular.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His ponderous shield . . .<br/
Behind him cast. The broad <qex>circumference</qex><br/
Hung on his shoulders like the moon.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The external surface of a sphere, or of any orbicular body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumference</ent><br/
<hw>Cir*cum"fer*ence</hw>, <pos>v. t.</pos> <def>To include in a circular space; to bound.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumferential</ent><br/
<hw>Cir*cum`fer*en"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>circumferentialis</ets>.]</ety> <def>Pertaining to the circumference; encompassing; encircling; circuitous.</def>  <rj><au>Parkhurst.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumferentially</ent><br/
<hw>Cir*cum`fer*en"tial*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>So as to surround or encircle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumferentor</ent><br/
<hw>Cir*cum`fer*en"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Circumfer</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A surveying instrument, for taking horizontal angles and bearings; a surveyor's compass. It consists of a compass whose needle plays over a circle graduated to 360<deg/, and of a horizontal brass bar at the ends of which are standards with narrow slits for sighting, supported on a tripod by a ball and socket joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A graduated wheel for measuring tires; a tire circle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflant</ent><br/
<hw>Cir"cum*flant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circumflans</ets>, p. pr. of <ets>circumflare</ets>.]</ety> <def>Blowing around.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflect</ent><br/
<hw>Cir"cum*flect</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumflected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Circumflecting</conjf>.]</vmorph> <ety>[L. <ets>circumflectere</ets>. See <er>Circumflex</er>.]</ety> <sn>1.</sn> <def>To bend around.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark with the circumflex accent, as a vowel.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflection</ent><br/
<hw>Cir`cum*flec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Circumflexion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflex</ent><br/
<hw>Cir"cum*flex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumflexus</ets> a bending round, fr. <ets>circumflectere</ets>, <ets>circumflexum</ets>, to bend or turn about; <ets>circum + flectere</ets> to bend. See <er>Flexible</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A wave of the voice embracing both a rise and fall or a fall and a rise on the same a syllable.</def>  <rj><au>Walker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A character, or accent, denoting in Greek a rise and of the voice on the same long syllable, marked thus [~ or <?/]; and in Latin and some other languages, denoting a long and contracted syllable, marked [<?/ or ^]. See <er>Accent</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflex</ent><br/
<hw>Cir"cum*flex</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumflexed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circumflexing</conjf> <pr>(?)</pr>.]</vmorph> <def>To mark or pronounce with a circumflex.</def>  <rj><au>Walker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflex</ent><br/
<hw>Cir"cum*flex</hw>, <pos>a.</pos> <ety>[Cf. L. <ets>circumflexus</ets>, p. p.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Moving or turning round; circuitous.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Curved circularly; -- applied to several arteries of the hip and thigh, to arteries, veins, and a nerve of the shoulder, and to other parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumflexion</ent><br/
<hw>Cir`cum*flex"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of bending, or causing to assume a curved form.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A winding about; a turning; a circuity; a fold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfluence</ent><br/
<hw>Cir*cum"flu*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A flowing round on all sides; an inclosing with a fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfluous</ent><br/
<ent>Circumfluent</ent><br/
<mhw>{ <hw>Cir*cum"flu*ent</hw> <pr>(?)</pr>, <hw>Cir*cum"flu*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>circumfluere</ets>, p. pr. of <ets>circumfluere</ets>; <ets>circum + fluere</ets> to flow; also L. <ets>circumfluus</ets>.]</ety> <def>Flowing round; surrounding in the manner of a fluid.</def> <ldquo/The deep, <xex>circumfluent</xex> waves.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumforaneous</ent><br/
<ent>Circumforanean</ent><br/
<mhw>{ <hw>Cir`cum*fo*ra"ne*an</hw> <pr>(?)</pr>, <hw>Cir`cum*fo*ra"ne*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>circumforaneus</ets> found in markets; <ets>circum + forum</ets> a market place.]</ety> <def>Going about or abroad; walking or wandering from house to house.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfulgent</ent><br/
<hw>Cir`cum*ful"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>fulgent</ets>.]</ety> <def>Shining around or about.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfuse</ent><br/
<hw>Cir`cum*fuse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>circumfusus</ets>, p. p. of <ets>circumfundere</ets> to pour around; <ets>circum + fundere</ets> to pour.]</ety> <def>To pour round; to spread round.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His army <qex>circumfused</qex> on either wing.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfusile</ent><br/
<hw>Cir`cum*fu"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + L. <ets>fusilis</ets> fusil, <pos>a.</pos>]</ety> <def>Capable of being poured or spread round.</def> <ldquo/<xex>Circumfusile</xex> gold.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumfusion</ent><br/
<hw>Cir`cum*fu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumfusio</ets>.]</ety> <def>The act of pouring or spreading round; the state of being spread round.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumgestation</ent><br/
<hw>Cir`cum*ges*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumgestare</ets> to carry around; <ets>circum + gestare</ets> to carry.]</ety> <def>The act or process of carrying about.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Circumgestation</qex> of the eucharist to be adored.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumgyrate</ent><br/
<hw>Cir`cum*gy"rate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>gyrate</ets>.]</ety> <def>To roll or turn round; to cause to perform a rotary or circular motion.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumgyration</ent><br/
<hw>Cir`cum*gy*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of turning, rolling, or whirling round.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A certain turbulent and irregular <qex>circumgyration</qex>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumgyratory</ent><br/
<hw>Cir`cum*gy"ra*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moving in a circle; turning round.</def>  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumgyre</ent><br/
<hw>Cir`cum*gyre"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To circumgyrate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumincession</ent><br/
<hw>Cir`cum*in*ces"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>circum-</ets> + L. <ets>incedere</ets>, <ets>incessum</ets>, to walk.]</ety> <fld>(Theol.)</fld> <def>The reciprocal existence in each other of the three persons of the Trinity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumjacence</ent><br/
<hw>Cir`cum*ja"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Condition of being circumjacent, or of bordering on every side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumjacent</ent><br/
<hw>Cir`cum*ja"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circumjacens</ets>, p. pr. of <ets>circumjacere</ets>; <ets>circum</ets> + <ets>jac<emac/re</ets> to lie.]</ety> <def>Lying round; bordering on every side.</def>  <rj><au>T. Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumjovial</ent><br/
<hw>Cir`cum*jo"vi*al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>circum-</ets> + L. <ets>Jupiter</ets>, gen. <ets>Jovis</ets>, Jove.]</ety> <def>One of the moons or satellites of the planet Jupiter.</def> <mark>[Obs.]</mark>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumlittoral</ent><br/
<hw>Cir`cum*lit"to*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + L. <ets>littus</ets>, <ets>littoris</ets>, shore; preferable form, <ets>litus</ets>, <ets>litoris</ets>.]</ety> <def>Adjointing the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumlocution</ent><br/
<hw>Cir`cum*lo*cu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumlocutio</ets>, fr. <ets>circumloqui</ets>, <ets>-locutus</ets>, to make use of circumlocution; <ets>circum</ets> + <ets>loqui</ets> to speak. See <er>Loquacious</er>.]</ety> <def>The use of many words to express an idea that might be expressed by few; indirect or roundabout language; a periphrase.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>the plain Billingsgate way of calling names . . . would save abundance of time lost by <qex>circumlocution</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Circumlocution office</b></col>, <cd>a term of ridicule for a governmental office where business is delayed by passing through the hands of different officials.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumlocutional</ent><br/
<hw>Cir`cum*lo*cu"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to, or consisting of, circumlocutions; periphrastic; circuitous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumlocutory</ent><br/
<hw>Cir`cum*loc"u*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterised by circumlocution; periphrastic.</def>  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The officials set to work in regular <qex>circumlocutory</qex> order.</q> <rj><qau>Chambers's Journal.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circummeridian</ent><br/
<hw>Cir`cum*me*rid"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>meridian</ets>.]</ety> <def>About, or near, the meridian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circummure</ent><br/
<hw>Cir`cum*mure"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>mure</ets>, v. t.]</ety> <def>To encompass with a wall.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumnavigable</ent><br/
<hw>Cir`cum*nav"i*ga*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being sailed round.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumnavigate</ent><br/
<hw>Cir`cum*nav"i*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumnavigated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circumnavigating</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>circumnavigatus</ets>, p. p. of <ets>circumnavigare</ets> to sail round; <ets>circum + navigare</ets> to navigate.]</ety> <def>To sail completely round.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Having <qex>circumnavigated</qex> the whole earth.</q> <rj><qau>T. Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumnavigation</ent><br/
<hw>Cir`cum*nav`i*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of circumnavigating, or sailing round.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumnavigator</ent><br/
<hw>Cir`cum*nav"iga`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sails round.</def>  <rj><au>W. Guthrie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumnutate</ent><br/
<hw>Cir`cum*nu"tate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>nutate</ets>.]</ety> <def>To pass through the stages of circumnutation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumnutation</ent><br/
<hw>Cir`cum*nu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The successive bowing or bending in different directions of the growing tip of the stems of many plants, especially seen in climbing plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumpolar</ent><br/
<hw>Cir`cum*po"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>polar</ets>.]</ety> <def>About the pole; -- applied to stars that revolve around the pole without setting; <as>as, <ex>circumpolar</ex> stars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumposition</ent><br/
<hw>Cir`cum*po*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumpositio</ets>, fr. <ets>circumponere</ets>, - <ets>positium</ets>, to place around.]</ety> <def>The act of placing in a circle, or round about, or the state of being so placed.</def>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumrotatory</ent><br/
<ent>Circumrotary</ent><br/
<mhw>{ <hw>Cir`cum*ro"tary</hw> <pr>(?)</pr>, <hw>Cir`cum*ro"ta*to*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>rotary</ets>, <ets>rotatory</ets>.]</ety> <def>turning, rolling, or whirling round.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumrotate</ent><br/
<hw>Cir`cum*ro"tate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[L. <ets>circumrotare</ets>; <ets>circum + rotare</ets> to turn round.]</ety> <def>To rotate about.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumrotation</ent><br/
<hw>Cir`cum*ro*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of rolling or revolving round, as a wheel; circumvolution; the state of being whirled round.</def>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscissile</ent><br/
<hw>Cir`cum*scis"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>scissle</ets>.]</ety> <fld>(Bot.)</fld> <def>Dehiscing or opening by a transverse fissure extending around (a capsule or pod). See <xex>Illust.</xex> of <er>Pyxidium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscribable</ent><br/
<hw>Cir`cum*scrib"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being circumscribed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscribe</ent><br/
<hw>Cir`cum*scribe"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumscribed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circumscribing</conjf>.]</vmorph> <ety>[L. <ets>circumscribere</ets>, <ets>-scriptum</ets>; <ets>circum + scribere</ets> to write, draw. See <er>Soribe</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>to write or engrave around.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thereon is <qex>circumscribed</qex> this epitaph.</q> <rj><qau>Ashmole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inclose within a certain limit; to hem in; to surround; to bound; to confine; to restrain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>circumscribe</qex> royal power.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>To draw a line around so as to touch at certain points without cutting. See <er>Inscribe</er>, 5.</def></p>

<p><syn><b>Syn.</b> -- To bound; limit; restrict; confine; abridge; restrain; environ; encircle; inclose; encompass.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscriber</ent><br/
<hw>Cir`cum*scrib"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, circumscribes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscriptible</ent><br/
<hw>Cir`cum*scrip"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being circumscribed or limited by bounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscription</ent><br/
<hw>Cir`cum*scrip"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumscriptio</ets>. See <er>Circumscribe</er>.]</ety> <sn>1.</sn> <def>An inscription written around anything.</def> <mark>[R.]</mark>  <rj><au>Ashmole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The exterior line which determines the form or magnitude of a body; outline; periphery.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of limiting, or the state of being limited, by conditions or restraints; bound; confinement; limit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>circumscriptions</qex> of terrestrial nature.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I would not my unhoused, free condition<br/
Put into <qex>circumscription</qex> and confine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscriptive</ent><br/
<hw>Cir`cum*scrip"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Circumscribing or tending to circumscribe; marcing the limits or form of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscriptively</ent><br/
<hw>Cir`cum*scrip"tive*ly</hw>, <pos>adv.</pos> <def>In a limited manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumscriptly</ent><br/
<hw>Cir"cum*script`ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a literal, limited, or narrow manner.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumspect</ent><br/
<hw>Cir"cum*spect</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circumspectus</ets>, p. p. of <ets>circumspicere</ets> to look about one's self, to observe; <ets>circum + spicere</ets>, <ets>specere</ets>, to look. See <er>Spy</er>.]</ety> <def>Attentive to all the circumstances of a case or the probable consequences of an action; cautious; prudent; wary.</def></p>

<p><syn><b>Syn.</b> -- See <er>Cautious</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumspection</ent><br/
<hw>Cir`cum*spec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumspectio</ets>.]</ety> <def>Attention to all the facts and circumstances of a case; caution; watchfulness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With silent <qex>circumspection</qex>, unespied.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> -- Caution; prudence; watchfulness; deliberation; thoughtfulness; wariness; forecast.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 259 --></p>

<p><ent>Circumspective</ent><br/
<hw>Cir`cum*spec"tive</hw> <pr>(s<etil/r`k<ucr/m*sp<ecr/k"t<icr/v)</pr>, <pos>a.</pos> <def>Looking around every way; cautious; careful of consequences; watchful of danger.</def> <ldquo/<xex>Circumspective</xex> eyes.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumspectively</ent><br/
<hw>Cir`cum*spec"tive*ly</hw>, <pos>adv.</pos> <def>Circumspectly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>circumspectly</ent><br/
<hw>cir"cum*spect`ly</hw> <pr>(s<etil/r"k<ucr/m*sp<ecr/kt`l<ycr/)</pr>, <pos>adv.</pos> <def>In a circumspect manner; cautiously; warily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumspectness</ent><br/
<hw>Cir"cum*spect"ness</hw>, <pos>n.</pos> <def>Vigilance in guarding against evil from every quarter; caution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Travel] forces <qex>circumspectness</qex> on those abroad, who at home are nursed in security.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>circumstance</ent><br/
<hw>cir"cum*stance</hw> <pr>(s<etil/r"k<ucr/m*st<acr/ns)</pr>, <pos>n.</pos> <ety>[L. <ets>circumstantia</ets>, fr. <ets>circumstans</ets>, <ets>-antis</ets>, p. pr. of <ets>circumstare</ets> to stand around; <ets>circum + stare</ets> to stand. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>That which attends, or relates to, or in some way affects, a fact or event; an attendant thing or state of things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>circumstances</qex> are well known in the country where they happened.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An event; a fact; a particular incident.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sculptor had in his thoughts the conqueror weeping for new worlds, or the like <qex>circumstances</qex> in history.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Circumlocution; detail.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So without more <qex>circumstance</qex> at all<br/
I hold it fit that we shake hands and part.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Condition in regard to worldly estate; state of property; situation; surroundings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When men are easy in their <qex>circumstances</qex>, they are naturally enemies to innovations.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Not a circumstance</b></col>, <cd>of no account.</cd> <mark>[Colloq.]</mark> -- <col><b>Under the circumstances</b></col>, <cd>taking all things into consideration.</cd></cs></p>

<p><syn><b>Syn.</b> -- Event; occurrence; incident; situation; condition; position; fact; detail; item. See <er>Event</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumstance</ent><br/
<hw>Cir"cum*stance</hw>, <pos>v. t.</pos> <def>To place in a particular situation; to supply relative incidents.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The poet took the matters of fact as they came down to him and <qex>circumstanced</qex> them, after his own manner.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumstanced</ent><br/
<hw>Cir"cum*stanced</hw> <pr>(s<etil/r"k<ucr/m*st<acr/nst)</pr>, <pos>p. a.</pos> <sn>1.</sn> <def>Placed in a particular position or condition; situated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The proposition is, that two bodies so <qex>circumstanced</qex> will balance each other.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Governed by events or circumstances.</def> <mark>[Poetic  & R.]</mark> <ldquo/I must be <xex>circumstanced</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>circumstant</ent><br/
<hw>cir"cum*stant</hw> <pr>(s<etil/r"k<ucr/m*st<acr/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>circumstans</ets>. See <er>Circumstance</er>.]</ety> <def>Standing or placed around; surrounding.</def> <mark>[R.]</mark> <ldquo/<xex>Circumstant</xex> bodies.<rdquo/  <rj><au>Sir K. Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>circumstantiable</ent><br/
<hw>cir`cum*stan"tia*ble</hw> <pr>(s<etil/r`k<ucr/m*st<acr/n"sh<adot/*b'l)</pr> <pos>a.</pos> <def>Capable of being circumstantiated.</def> <mark>[Obs.]</mark>  <rj><au>Jer Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>circumstantial</ent><br/
<hw>cir`cum*stan"tial</hw> <pr>(s<etil/r`k<ucr/m*st<acr/n"sh<ait/l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>circonstanciel</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Consisting in, or pertaining to, circumstances or particular incidents.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The usual character of human testimony is substantial truth under <qex>circumstantial</qex> variety.</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Incidental; relating to, but not essential.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We must therefore distinguish between the essentials in religious worship . . . and what is merely <qex>circumstantial</qex>.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Abounding with circumstances; detailing or exhibiting all the circumstances; minute; particular.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tedious and <qex>circumstantial</qex> recitals.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Circumstantial evidence</b></col> <fld>(Law)</fld>, <cd>evidence obtained from circumstances, which necessarily or usually attend facts of a particular nature, from which arises presumption. According to some authorities <xex>circumstantial</xex> is distinguished from <xex>positive</xex> evidence in that the latter is the testimony of eyewitnesses to a fact or the admission of a party; but the prevalent opinion now is that all such testimony is dependent on circumstances for its support. All testimony is more or less <xex>circumstantial</xex>.</cd> <au>Wharton</au>.</cs></p>

<p><syn><b>Syn.</b> -- See <er>Minute</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumstantial</ent><br/
<hw>Cir`cum*stan"tial</hw>, <pos>n.</pos> <def>Something incidental to the main subject, but of less importance; opposed to an <xex>essential</xex>; -- generally in the plural; <as>as, the <ex>circumstantials</ex> of religion</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumstantiality</ent><br/
<hw>Cir`cum*stan`ti*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state, characteristic, or quality of being circumstantial; particularity or minuteness of detail.</def> <ldquo/I will endeavor to describe with sufficient <xex>circumstantiality</xex>.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumstantially</ent><br/
<hw>Cir`cum*stan"tial*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In respect to circumstances; not essentially; accidentally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of the fancy and intellect, the powers are only <qex>circumstantially</qex> different.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In every circumstance or particular; minutely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To set down somewhat <qex>circumstantially</qex>, not only the events, but the manner of my trials.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumstantiate</ent><br/
<hw>Cir`cum*stan"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumstantiated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circumstantiating</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Circumstantiating</er> (<?/).]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To place in particular circumstances; to invest with particular accidents or adjuncts.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If the act were otherwise <qex>circumstantiated</qex>, it might will that freely which now it wills reluctantly.</q> <rj><qau>Bramhall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prove or confirm by circumstances; to enter into details concerning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither will time permint to <qex>circumstantiate</qex> these particulars, which I have only touched in the general.</q> <rj><qau>State Trials (1661).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumterraneous</ent><br/
<hw>Cir`cum*ter*ra"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>circum-</ets> + L. <ets>terra</ets> earth.]</ety> <def>Being or dwelling around the earth.</def> <ldquo/<xex>Circumterraneous</xex> demouns.<rdquo/  <rj><au>H. Hallywell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumundulate</ent><br/
<hw>Cir`cum*un"du*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>circum-</ets> + <ets>undulate</ets>.]</ety> <def>To flow round, as waves.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvallate</ent><br/
<hw>Cir`cum*val"late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>circumvallatus</ets>, p. p. of <ets>circumvallare</ets> to surround with a wall; <ets>circum</ets> + <ets>vallare</ets> to wall, fr. <ets>vallum</ets> rampart.]</ety> <def>To surround with a rampart or wall.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvallate</ent><br/
<hw>Cir`cum*val"late</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Surrounded with a wall; inclosed with a rampart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Surrounded by a ridge or elevation; <as>as, the <ex>circumvallate</ex> papill<ae/, near the base of the tongue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvallation</ent><br/
<hw>Cir`cum*val*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <sd>(a)</sd> <def>The act of surrounding with a wall or rampart.</def> <sd>(b)</sd> <def>A line of field works made around a besieged place and the besieging army, to protect the camp of the besiegers against the attack of an enemy from without.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvection</ent><br/
<hw>Cir`cum*vec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumvectio</ets>; <ets>circum</ets> + <ets>vehere</ets> to carry.]</ety> <def>The act of carrying anything around, or the state of being so carried.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvent</ent><br/
<hw>Cir`cum*vent"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumvented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Circumventing</conjf>.]</vmorph> <ety>[L. <ets>circumventis</ets>, p. p. of <ets>circumvenire</ets>, to come around, encompass, deceive; <ets>circum</ets> + <ets>venire</ets> to come, akin to E. <ets>come.</ets>]</ety> <def>To gain advantage over by arts, stratagem, or deception; to decieve; to delude; to get around.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>circumvented</qex> whom I could not gain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvention</ent><br/
<hw>Cir`cum*ven"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumventio</ets>.]</ety> <def>The act of prevailing over another by arts, address, or fraud; deception; fraud; imposture; delusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A school in which he learns sly <qex>circumvention</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumventive</ent><br/
<hw>Cir`cum*vent"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to circumvent; deceiving by artifices; deluding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumventor</ent><br/
<hw>Cir`cum*vent"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who circumvents; one who gains his purpose by cunning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvest</ent><br/
<hw>Cir`cum*vest"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>circumvestire</ets>; <ets>circum + vestire</ets> to clothe.]</ety> <def>To cover round, as with a garment; to invest.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Circumvested</qex> with much prejudice.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvolant</ent><br/
<hw>Cir*cum"vo*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>circumvolans</ets>, p. pr. See <er>Circumvolation</er>.]</ety> <def>Flying around.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>circumvolant</qex> troubles of humanity.</q> <rj><qau>G. Macdonald.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvolation</ent><br/
<hw>Cir`cum*vo*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>circumvolate</ets>. <ets>-volatum</ets>, to fly around; <ets>circum + volare</ets> to fly.]</ety> <def>The act of flying round.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvolution</ent><br/
<hw>Cir`cum*vo*lu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Circumvolve</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The act of rolling round; the state of being rolled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thing rolled round another.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A roundabout procedure; a circumlocution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had neither time nor temper for sentimental <qex>circumvolutions</qex>.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvolve</ent><br/
<hw>Cir`cum*volve"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Circumvolved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Circumvolving</conjf>.]</vmorph> <ety>[L. <ets>circumvolvere</ets>, <ets>-volutum</ets>; <ets>circum + volvere</ets> to roll.]</ety> <def>To roll round; to cause to revolve; to put into a circular motion.</def>  <rj><au>Herrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circumvolve</ent><br/
<hw>Cir`cum*volve"</hw>, <pos>v. i.</pos> <def>To roll round; to revolve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Circus</ent><br/
<hw>Cir"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Circuses</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>circus</ets> circle, ring, circus (in sense 1). See <er>Circle</er>, and cf. <er>Cirque</er>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Roman Antiq.)</fld> <def>A level oblong space surrounded on three sides by seats of wood, earth, or stone, rising in tiers one above another, and divided lengthwise through the middle by a barrier around which the track or course was laid out. It was used for chariot races, games, and public shows.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>Circus Maximus</xex> at Rome could contain more than 100,000 spectators.</note>  <rj><au>Harpers' Latin Dict.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A circular inclosure for the exhibition of feats of horsemanship, acrobatic displays, etc. Also, the company of performers, with their equipage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Circuit; space; inclosure.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The narrow <qex>circus</qex> of my dungeon wall.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirl bunting</ent><br/
<hw>Cirl" bun`ting</hw> <pr>(?)</pr>. <ety>[Cf. It. <ets>cirlo</ets>.]</ety> <fld>(Zool.)</fld> <def>A European bunting (<spn>Emberiza cirlus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirque</ent><br/
<hw>Cirque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>circus</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A circle; a circus; a circular erection or arrangement of objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A dismal <qex>cirque</qex><br/
Of Druid stones upon a forlorn moor.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of circular valley in the side of a mountain, walled around by precipices of great height.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrate</ent><br/
<hw>Cir"rate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cirratus</ets> having ringlets, fr. <ets>cirrus</ets> a curl.]</ety> <fld>(Zool.)</fld> <def>Having cirri along the margin of a part or organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrhiferous</ent><br/
<hw>Cir*rhif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Cirriferous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrhose</ent><br/
<hw>Cir"rhose</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Cirrose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrhosis</ent><br/
||<hw>Cir*rho"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ orange-colored: cf. F. <ets>cirrhose</ets>. So called from the yellowish appearance which the diseased liver often presents when cut.]</ety> <fld>(Med.)</fld> <def>A disease of the liver in which it usually becomes smaller in size and more dense and fibrous in consistence; hence sometimes applied to similar changes in other organs, caused by increase in the fibrous framework and decrease in the proper substance of the organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrhotic</ent><br/
<hw>Cir*rhot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, caused by, or affected with, cirrhosis; <as>as, <ex>cirrhotic</ex> degeneration; a <ex>cirrhotic</ex> liver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrhous</ent><br/
<hw>Cir"rhous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Cirrose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrhus</ent><br/
<hw>Cir"rhus</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Cirrus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirri</ent><br/
||<hw>Cir"ri</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Cirrus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirriferous</ent><br/
<hw>Cir*rif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cirrus</ets> + <ets>-ferous</ets>.]</ety> <def>Bearing cirri, as many plants and animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirriform</ent><br/
<hw>Cir"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cirrus</ets> + <ets>-form</ets>.]</ety> <fld>(Biol.)</fld> <def>Formed like a cirrus or tendril; -- said of appendages of both animals and plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrigerous</ent><br/
<hw>Cir*rig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cirrus</ets> + <ets>-gerous</ets>.]</ety> <fld>(Biol.)</fld> <def>Having curled locks of hair; supporting cirri, or hairlike appendages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrigrade</ent><br/
<hw>Cir"ri*grade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cirrus</ets> + L. <ets>gradi</ets> to walk.]</ety> <fld>(Biol.)</fld> <def>Moving or moved by cirri, or hairlike appendages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cirio</ent><br/
<hw>cirio</hw> <pos>n.</pos> <def>the candlewood of Mexico and couthwestern California (<spn>Idria columnaris</spn> or <spn>Fouquieria columnaris</spn>), having tall columnar stems and bearing honey-scented creamy yellow flowers; -- called also the <altname>boojum tree</altname>.</def><br/
<syn><b>Syn. --</b> boojum tree, <spn>Fouquieria columnaris</spn>, <spn>Idria columnaris</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cirriped</ent><br/
<hw>Cir"ri*ped</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cirripedia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirripedia</ent><br/
||<hw>Cir`ri*pe"di*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>cirrus</ets> curl + <ets>pes</ets>, <ets>pedis</ets>, foot.]</ety> <fld>(Zool.)</fld> <def>An order of Crustacea including the barnacles. When adult, they have a calcareous shell composed of several pieces. From the opening of the shell the animal throws out a group of curved legs, looking like a delicate curl, whence the name of the group. See <er>Anatifa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrobranchiata</ent><br/
||<hw>Cir`ro*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. E. <ets>cirrus</ets> + L. <ets>branchiae</ets> gills.]</ety> <fld>(Zool.)</fld> <def>A division of Mollusca having slender, cirriform appendages near the mouth; the Scaphopoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirro-cumulus</ent><br/
<hw>Cir`ro-cu"mu*lus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cirrus</ets> + <ets>cumulus</ets>.]</ety> <fld>(Meteor.)</fld> <def>See under <er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrose</ent><br/
<hw>Cir"rose</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Cirrus</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Bearing a tendril or tendrils; <as>as, a <ex>cirrose</ex> leaf</as>.</def> <sd>(b)</sd> <def>Resembling a tendril or cirrus.</def> <altsp>[Spelt also <asp>cirrhose</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrostomi</ent><br/
||<hw>Cir`ros"to*mi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. E. <ets>cirrus</ets> + Gr. <?/ mouth.]</ety> <fld>(Zool.)</fld> <def>The lowest group of vertebrates; -- so called from the cirri around the mouth; the Leptocardia. See <er>Amphioxus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirro-stratus</ent><br/
<hw>Cir`ro-stra"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cirrus</ets> + <ets>stratus</ets>.]</ety> <fld>(Meteor.)</fld> <def>See under <er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrous</ent><br/
<hw>Cir"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>Cirrose.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Tufted; -- said of certain feathers of birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirrus</ent><br/
||<hw>Cir"rus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cirri</plw> <pr>(#)</pr>.</plu> <ety>[L., lock, curl, ringlet.]</ety> <altsp>[Also written <asp>cirrhus</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Bot.)</fld> <def>A tendril or clasper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A soft tactile appendage of the mantle of many Mollusca, and of the parapodia of Annelida. Those near the head of annelids are <xex>Tentacular cirri</xex>; those of the last segment are <xex>caudal cirri</xex>.</def> <sd>(b)</sd> <def>The jointed, leglike organs of Cirripedia. See <er>Annelida</er>, and <er>Polych<ae/ta</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some of the inferior animals the <xex>cirri</xex> aid in locomotion; in others they are used in feeding; in the Annelida they are mostly organs of touch. Some <xex>cirri</xex> are branchial in function.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The external male organ of trematodes and some other worms, and of certain Mollusca.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Meteor.)</fld> <def>See under <er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirsocele</ent><br/
<hw>Cir"so*cele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a dilated vein + <?/ tumor.]</ety> <fld>(Med.)</fld> <def>The varicose dilatation of the spermatic vein.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirsoid</ent><br/
<hw>Cir"soid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a dilated vein + <ets>-oid</ets>.]</ety> <fld>(Med.)</fld> <def>Varicose.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cirsoid aneurism</b></col>, <cd>a disease of an artery in which it becomes dilated and elongated, like a varicose vein.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cirsotomy</ent><br/
<hw>Cir*sot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a dilated vein + <grk>te`mnein</grk> to cut.]</ety> <fld>(Surg.)</fld> <def>Any operation for the removal of varices by incision.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cis-</ent><br/
<hw>Cis-</hw> <pr>(?)</pr>. <def>A Latin preposition, sometimes used as a prefix in English words, and signifying <xex>on this side</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cisalpine</ent><br/
<hw>Cis*al"pine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Cisalpinus</ets>; <ets>cis</ets> on this side + <ets>Alpinus</ets> Alpine.]</ety> <def>On the hither side of the Alps with reference to Rome, that is, on the south side of the Alps; -- opposed to <xex>transalpine</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cisatlantic</ent><br/
<hw>Cis`at*lan"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>cis-</ets> + <ets>Atlantic</ets>.]</ety> <def>On this side of the Atlantic Ocean; -- used of the eastern or the western side, according to the standpoint of the writer.</def>  <rj><au>Story.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cisco</ent><br/
<hw>Cis"co</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Lake herring (<spn>Coregonus Artedi</spn>), valuable food fish of the Great Lakes of North America. The name is also applied to <spn>Coregonus Hoyi</spn>, a related species of Lake Michigan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ciselure</ent><br/
||<hw>Ci`se*lure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>The process of chasing on metals; also, the work thus chased.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cisleithan</ent><br/
<hw>Cis*lei"than</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>cis-</ets> + <ets>Leitha</ets>.]</ety> <def>On the Austrian side of the river Leitha; Austrian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cismontane</ent><br/
<hw>Cis*mon"tane</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>cis-</ets> + L. <ets>mons</ets> mountain.]</ety> <def>On this side of the mountains. See under <er>Ultramontane</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cispadane</ent><br/
<hw>Cis"pa*dane`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>cis-</ets> + L. <ets>Padanus</ets>, pert. to the <ets>Padus</ets> or Po.]</ety> <def>On the hither side of the river Po with reference to Rome; that is, on the south side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cissoid</ent><br/
<hw>Cis"soid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ like ivy; <?/ ivy + <grk>e'i^dos</grk> form.]</ety> <fld>(Geom.)</fld> <def>A curve invented by Diocles, for the purpose of solving two celebrated problems of the higher geometry; viz., to trisect a plane angle, and to construct two geometrical means between two given straight lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cist</ent><br/
<hw>Cist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cista</ets> box, chest, Gr. <?/ Cf. <er>Chest</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Antiq.)</fld> <def>A box or chest. Specifically: <sd>(a)</sd> A bronze receptacle, round or oval, frequently decorated with engravings on the sides and cover, and with feet, handles, etc., of decorative castings. <sd>(b)</sd> A cinerary urn. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>See <er>Cyst</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cisted</ent><br/
<hw>Cist"ed</hw>, <pos>a.</pos> <def>Inclosed in a cyst. See <er>Cysted</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cistercian</ent><br/
<hw>Cis*ter"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>Cistercium</ets>. F. <ets>C<icir/teaux</ets>, a convent not far from Dijon, in France: cf. F. <ets>cistercien</ets>.]</ety> <fld>(Eccl.)</fld> <def>A monk of the prolific branch of the Benedictine Order, established in 1098 at C<icir/teaux, in France, by Robert, abbot of Molesme. For two hundred years the Cistercians followed the rule of St. Benedict in all its rigor.</def> -- <def2><pos>a.</pos>  <def>Of or pertaining to the Cistercians.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 260 --></p>

<p><ent>Cistern</ent><br/
<hw>Cis"tern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>cisterne</ets>, OF. <ets>cisterne</ets>, F. <ets>cisterne</ets>, fr. L. <ets>cisterna</ets>, fr. <ets>cista</ets> box, chest. See <er>Cist</er>, and cf. <er>chest</er>.]</ety> <sn>1.</sn> <def>An artificial reservoir or tank for holding water, beer, or other liquids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A natural reservoir; a hollow place containing water.</def> <ldquo/The wide <xex>cisterns</xex> of the lakes.<rdquo/ <au>Blackmore.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cistic</ent><br/
<hw>Cist"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Cystic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cistus</ent><br/
<hw>Cistus</hw> <pos>n.</pos> <def>a genus of small to medium-sized evergreen shrubs of southern Europe and northern Africa.</def><br/
<syn><b>Syn. --</b> genus <gen>Cistus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cit</ent><br/
<hw>Cit</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[Contr. fr. <ets>citizen</ets>.]</ety> <def>A citizen; an inhabitant of a city; a pert townsman; -- used contemptuously.</def> <ldquo/Insulted as a <xex>cit</xex><rdquo/.  <rj><au>Johnson</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which past endurance sting the tender <qex>cit</qex>.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citable</ent><br/
<hw>Cit"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being cited.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citadel</ent><br/
<hw>Cit"a*del</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>citadelle</ets>, It. <ets>citadella</ets>, di<?/. of <ets>citt<?/</ets> city, fr. L. <ets>civitas</ets>. See <er>City</er>.]</ety> <def>A fortress in or near a fortified city, commanding the city and fortifications, and intended as a final point of defense.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Syn</b></col>. -- <cd>Stronghold. See <er>Fortress</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cital</ent><br/
<hw>Cit"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cite</er>]</ety> <sn>1.</sn> <def>Summons to appear, as before a judge.</def> <mark>[R.]</mark>  <rj><au>Johnson</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Citation; quotation</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citation</ent><br/
<hw>Ci*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>citation</ets>, LL. <ets>citatio</ets>, fr.L. <ets>citare</ets> to cite. See <er>Cite</er>]</ety> <sn>1.</sn> <def>An official summons or notice given to a person to appear; the paper containing such summons or notice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of citing a passage from a book, or from another person, in his own words; also, the passage or words quoted; quotation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This horse load of <qex>citations</qex> and fathers.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Enumeration; mention; <as>as, a <ex>citation</ex> of facts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>A reference to decided cases, or books of authority, to prove a point in law.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citator</ent><br/
<hw>Ci*ta"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who cites.</def> <mark>[R]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citatory</ent><br/
<hw>Ci"ta*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>citatirius</ets>.]</ety> <def>Having the power or form of a citation; <as>as, letters <ex>citatory</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cite</ent><br/
<hw>Cite</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Citing</conjf>]</vmorph> <ety>[F. <ets>citer</ets>, fr. L. <ets>citare</ets>, intens. of <ets>cire</ets>, <ets>ci<emac/re</ets>, to put in motion, to excite; akin to Gr.<?/ to go, Skr. <?/ to sharpen.]</ety> <sn>1.</sn> <def>To call upon officially or authoritatively to appear, as before a court; to summon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cited</qex> dead,<br/
Of all past ages, to the general doom<br/
Shall hasten.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cited</qex> by finger of God.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To urge; to enjoin.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To quote; to repeat, as a passage from a book, or the words of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The devil can <qex>cite</qex> Scripture for his purpose.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To refer to or specify, as for support, proof, illustration, or confirmation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The imperfections which you have <qex>cited</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To bespeak; to indicate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Aged honor <qex>cites</qex> a virtuous youth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Law)</fld> <def>To notify of a proceeding in court.</def>  <rj><au>Abbot</au></rj></p>

<p><syn><b>Syn.</b> -- To quote; mention, name; refer to; adduce; select; call; summon. See <er>Quote</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citer</ent><br/
<hw>Cit"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who cites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citess</ent><br/
<hw>Cit"ess</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cit</er>.]</ety> <def>A city woman.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cithara</ent><br/
<hw>Cith"a*ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. Cf. <er>Cittern</er>, <er>Guitar</er>.]</ety> <fld>(Mus.)</fld> <def>An ancient stringed musical instrument resembling the harp.</def> <altsp>[Also spelled <asp>kithara</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citharistic</ent><br/
<hw>Cith`a*ris"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ cithara.]</ety> <def>Pertaining, or adapted, to the cithara.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cither</ent><br/
<hw>cither</hw> <pos>n.</pos> <def>a musical instrument resembling a harp with strings stretch over a flat sounding box; the <er>zither</er>.  It is played with a plectrum and with fingers.</def><br/
<syn><b>Syn. --</b> zither, zithern.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cithern</ent><br/
<hw>Cith"ern</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cittern</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citicism</ent><br/
<hw>Cit"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>cit</ets>.]</ety> <def>The manners of a cit or citizen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citied</ent><br/
<hw>Cit"ied</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Belonging to, or resembling, a city.</def> <ldquo/Smoky, <xex>citied</xex> towns<rdquo/ <mark>[R.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing, or covered with, cities.</def> <mark>[R.]</mark> <ldquo/The <xex>citied</xex> earth.<rdquo/  <rj><au>Keats.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citified</ent><br/
<hw>Cit"i*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>City</ets> +<ets>-fy</ets>.]</ety> <def>Aping, or having, the manners of a city.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citigradae</ent><br/
||<hw>Cit`i*gra"d<ae/</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>citus</ets> swift (p. p. of <ets>cire</ets>, <ets>ciere</ets>, to move) + <ets>gradi</ets> to walk. See <er>Cite</er>.]</ety> <fld>(Zool.)</fld> <def>A suborder of Arachnoidea, including the European tarantula and the wolf spiders (<fam>Lycosidae</fam>) and their allies, which capture their prey by rapidly running and jumping. See <er>Wolf spider</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citigrade</ent><br/
<hw>Cit"i*grade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>citigrade</ets>.]</ety> <fld>(Zool.)</fld> <def>Pertaining to the Citigrad<ae/.</def> -- <def2><pos>n.</pos> <def>One of the Citigrad<ae/.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citiner</ent><br/
<hw>Cit"i*ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is born or bred in a city; a citizen.</def> <mark>[Obs.]</mark>  <rj><au>Champan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citizen</ent><br/
<hw>Cit"i*zen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>citisein</ets>, OF. <ets>citeain</ets>, F. <ets>citoyen</ets>, fr. <ets>cit<eacute/</ets> city.  See <er>City</er>, and cf. <er>Cit</er>.]</ety> <sn>1.</sn> <def>One who enjoys the freedom and privileges of a city; a freeman of a city, as distinguished from a foreigner, or one not entitled to its franchises.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That large body of the working men who were not counted as <qex>citizens</qex> and had not so much as a vote to serve as an anodyne to their stomachs.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inhabitant of a city; a townsman.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person, native or naturalized, of either sex, who owes allegiance to a government, and is entitled to reciprocal protection from it.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This protection is . . . national protection, recognition of the individual, in the face of foreign nations, as a member of the state, and assertion of his security and rights abroad as well as at home.</note>  <rj><au>Abbot</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One who is domiciled in a country, and who is a citizen, though neither native nor naturalized, in such a sense that he takes his legal <xex>status</xex> from such country.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citizen</ent><br/
<hw>Cit"i*zen</hw>, <pos>a.</pos> <sn>1.</sn> <def>Having the condition or qualities of a citizen, or of citizens; <as>as, a <ex>citizen</ex> soldiery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the inhabitants of a city; characteristic of citizens; effeminate; luxurious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I am not well,<br/
But not so <qex>citizen</qex> a wanton as<br/
To seem to die ere sick.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citizeness</ent><br/
<hw>Cit"i*zen*ess</hw>, <pos>n.</pos> <def>A female citizen.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>citizens' band</ent><br/
<hw>citizens' band</hw> <pos>n.</pos> <def>that portion of the radio frequency spectrum allocated by the FCC for the use of individual citizens for short-distance personal or business use, from either fixed or mobile stations.  Abreviated <abbr>CB</abbr>.  Called also <altname>Citizens Radio Service</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Citizenship</ent><br/
<hw>Cit"i*zen*ship</hw>, <pos>n.</pos> <def>The state of being a citizen; the status of a citizen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citole</ent><br/
<hw>Cit"ole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>citole</ets>, fr. L. <ets>cithara</ets>. <er>See Cittern</er>.]</ety> <fld>(Mus.)</fld> <def>A musical instrument; a kind of dulcimer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citraconic</ent><br/
<hw>Cit`ra*con"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Citric</ets> + <ets>acon</ets>itic.]</ety> <def>Pertaining to, derived from, or having certain characteristics of, citric and aconitic acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Citraconic acid</b></col> <fld>(Chem.)</fld>, <cd>a white, crystalline, deliquescent substance, <chform>C3H4(CO2H)2</chform>, obtained by distillation of citric acid. It is a compound of the ethylene series.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citrange</ent><br/
<hw>Cit"range</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Citr</ets>us + or<ets>ange</ets>.]</ety> <def>A citrous fruit produced by a cross between the sweet orange and the trifoliate orange (<spn>Citrus trifoliata</spn>). It is more acid and has a more pronounced aroma than the orange; the tree is hardier. There are several varieties.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Citrate</ent><br/
<hw>Cit"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Citric</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of citric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citric</ent><br/
<hw>Cit"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>citrique</ets>. See <er>Citron</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, the citron or lemon; <as>as, <ex>citric</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Citric acid</b></col> <fld>(Chem.)</fld>, <cd>an organic acid, <chform>C3H4OH.(CO2H)3</chform>, extracted from lemons, currants, gooseberries, etc., as a white crystalline substance, having a pleasant sour taste.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>citrin</ent><br/
<hw>citrin</hw> <pos>n.</pos> <def>a vitamin that maintains the resistance of cell and capillary walls to permeation; -- called also <altname>vitamin P</altname>.</def><br/
<syn><b>Syn. --</b> vitamin P, bioflavinoid.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Citrination</ent><br/
<hw>Cit`ri*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Citrine</er>.]</ety> <def>The process by which anything becomes of the color of a lemon; esp., in alchemy, the state of perfection in the philosopher's stone indicated by its assuming a deep yellow color.</def>  <rj><au>Thynne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citrine</ent><br/
<hw>Cit"rine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>citrin</ets>. See <er>Citron</er>.]</ety> <def>Like a citron or lemon; of a lemon color; greenish yellow.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Citrine ointment</b></col> <fld>(Med.)</fld>, <cd>a yellowish mercurial ointment, the <altname>unguentum hydrargyri nitratis</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citrine</ent><br/
<hw>Cit"rine</hw>, <pos>n.</pos> <def>A yellow, pellucid variety of quartz.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citron</ent><br/
<hw>Cit"ron</hw> <pr>(s<icr/t"r<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>citron</ets>, LL. <ets>citro</ets>, fr. L. <ets>citrus</ets> citron tree (cf. <ets>citreum</ets>, sc. malum, a citron), from Gr. <grk>ki`tron</grk> citron]</ety> <sn>1.</sn> <fld>(Bot)</fld> <def>A fruit resembling a lemon, but larger, and pleasantly aromatic; it is produced by the <prodby>citron tree</prodby> (<spn>Citrus medica</spn>).  The thick rind, when candied, is the citron of commerce.  The fruit was once called the <altname>lime</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A citron tree, <spn>Citrus medica</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A citron melon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Citron melon</b></col>. <sd>(a)</sd> <cd>A small variety of muskmelon with sugary greenish flesh.</cd> <sd>(b)</sd> <cd>A small variety of watermelon, whose solid white flesh is used in making sweetmeats and preserves.</cd> -- <col><b>Citron tree</b></col> <fld>(Bot.)</fld>, <cd>the tree which bears citrons.  It was probably a native of northern India, and is now understood to be the typical form of <spn>Citrus Medica</spn>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Citroncirus</ent><br/
<hw>Citroncirus</hw> <pos>n.</pos> <def>a cross between <spn>Citrus sinensis</spn> and <spn>Poncirus trifoliata</spn>.</def><br/
<syn><b>Syn. --</b> genus <gen>Citroncirus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>citronwood</ent><br/
<hw>citronwood</hw> <pos>n.</pos> <sn>1.</sn> <def>wood of a citron tree.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a durable fragrant wood used in building (as in the roof of Cordova Cathedral); -- called also <altname>sandarac</altname>.</def><br/
<syn><b>Syn. --</b> sandarac.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>citrous</ent><br/
<hw>citrous</hw> <pr>(s<icr/t"r<ucr/s)</pr>, <pos>adj.</pos> <def>of or pertaining to a citrus tree; <as>as, a <ex>citrous</ex> disease</as>.</def><br/
<syn><b>Syn. --</b> citrus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>of or pertaining to citrus.</def> <illu>the <ex>citrus</ex> production of Florida</illu><br/
<syn><b>Syn. --</b> citrus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>citrulline</ent><br/
<hw>citrulline</hw> <pos>n.</pos> <def>an amino acid that is an intermediate in the conversion of ornithine to arginine.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Citrullus</ent><br/
<hw>Citrullus</hw> <pos>n.</pos> <def>a genus of vines including the watermelons (<spn>Citrullus lanata</spn>).</def><br/
<syn><b>Syn. --</b> genus <gen>Citrullus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Citrus</ent><br/
||<hw>Cit"rus</hw> <pr>(s<icr/t"r<ucr/s)</pr>, <pos>n.</pos> <ety>[L., a citron tree.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of trees including the orange, lemon, lime, tangerine, citron, grapefruit, etc., originally natives of southern Asia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>any tree belonging to the genus <gen>Citrus</gen>, having leathery evergreen leaves and bearing a soft pulpy fruit covered by a thick skin; -- called also <altname>citrus tree</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>the fruit of a tree belonging to the genus <gen>Citrus</gen>, having a thick shiny skin and a soft, sweet to tart pulp.</def><br/
[<source>PJC</source>]</p>

<p><ent>citrus tree</ent><br/
<hw>cit"rus tree`</hw> <pr>(s<icr/t"r<ucr/s tr<emac/`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>same as <er>citrus{2}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cittern</ent><br/
<hw>Cit"tern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.<ets>cithara</ets>, Gr. <grk>kiqa`ra</grk>. Cf. <er>Cithara</er>, <er>Gittern</er>.]</ety> <fld>(Mus.)</fld> <def>An instrument shaped like a lute, but strung with wire and played with a quill or plectrum.</def> <altsp>[Written also <asp>cithern</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Not to be confounded with <xex>zither</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cittern-head</ent><br/
<hw>Cit"tern-head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Blockhead; dunce; -- so called because the handle of a cittern usually ended with a carved head.</def>  <rj><au>Marsion</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>City</ent><br/
<hw>Cit"y</hw> <pr>(s<icr/t"<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cities</plw> <pr>(s<icr/t"<icr/z)</pr>.</plu> <ety>[OE. <ets>cite</ets>, F. <ets>cit<eacute/</ets>, fr. L. <ets>civitas</ets> citizenship, state, city, fr. <ets>civis</ets> citizen; akin to Goth. <ets>heiwa</ets> (in <ets>heiwa</ets>frauja man of the house), AS. <ets>h<imac/wan</ets>, pl., members of a family, servants, <ets>h<imac/red</ets> family, G.  <ets>heirath</ets> marriage, prop., providing a house, E. <ets>hind</ets> a peasant.]</ety> <sn>1.</sn> <def>A large town.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A corporate town; in the United States, a town or collective body of inhabitants, incorporated and governed by a mayor and aldermen or a city council consisting of a board of aldermen and a common council; in Great Britain, a town corporate, which is or has been the seat of a bishop, or the capital of his see.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>city</qex> is a town incorporated; which is, or has been, the see of a bishop; and though the bishopric has been dissolved, as at Westminster, it yet remaineth a <qex>city</qex>.</q> <rj><qau>Blackstone</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When Gorges constituted York a <qex>city</qex>, he of course meant it to be the seat of a bishop, for the word <qex>city</qex> has no other meaning in English law.</q> <rj><qau>Palfrey</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The collective body of citizens, or inhabitants of a city.</def>  <ldquo/What is the <xex>city</xex> but the people?<rdquo/  <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- See <er>Village</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>City</ent><br/
<hw>Cit"y</hw>, <pos>a.</pos> <def>Of or pertaining to a city.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>City council</b></col>. <cd>See under <er>Council</er>.</cd> -- <col><b>City court</b></col>, <cd>The municipal court of a city.</cd> <mark>[U. S.]</mark> -- <col><b>City ward</b></col>, <cd>a watchman, or the collective watchmen, of a city.</cd> <mark>[Obs.]</mark>  <rj><au>Fairfax.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>city-bred</ent><br/
<hw>city-bred</hw> <pos>adj.</pos>  <def>raised in a city.</def><br/
<syn><b>Syn. --</b> citified, city born.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cityfied</ent><br/
<ent>city-like</ent><br/
<mhw><hw>city-like</hw> <hw>cityfied</hw></mhw> <pos>adj.</pos> <def>having the customs or manners or dress of city people.</def><br/
<syn><b>Syn. --</b> citified.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>citywide</ent><br/
<hw>citywide</hw> <pos>adj.</pos> <def>including or occurring in all parts of a city; <as>as, <ex>citywide</ex> bussing; a <ex>citywide</ex> strike</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cive</ent><br/
<hw>Cive</hw> <pr>(s<imac/v)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Chive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civet</ent><br/
<hw>Civ"et</hw> <pr>(s<icr/v"<ecr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>civette</ets> (cf. It. <ets>zibetto</ets>) civet, civet cat, fr. LGr. <grk>zape`tion</grk>, fr. Ar. <ets>zub<amac/d</ets>, <ets>zab<amac/d</ets>, civet.]</ety> <sn>1.</sn> <def>A substance, of the consistence of butter or honey, taken from glands in the anal pouch of the civet (<spn>Viverra civetta</spn>). It is of clear yellowish or brownish color, of a strong, musky odor, offensive when undiluted, but agreeable when a small portion is mixed with another substance. It is used as a perfume.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool)</fld> <def>The animal that produces civet (<spn>Viverra civetta</spn>); -- called also <altname>civet cat</altname>.  It is carnivorous, from two to three feet long, and of a brownish gray color, with transverse black bands and spots on the body and tail. It is a native of northern Africa and of Asia. The name is also applied to other species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civet</ent><br/
<hw>Civ"et</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To scent or perfume with civet.</def>  <rj><au>Cowper</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civic</ent><br/
<hw>Civ"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L.civicus, fr. <ets>civis</ets> citizen. See <er>City</er>.]</ety> <def>Relating to, or derived from, a city or citizen; relating to man as a member of society, or to civil affairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Civic crown</b></col> <fld>(Rom. Antiq.)</fld>, <cd>a crown or garland of oak leaves and acorns, bestowed on a soldier who had saved the life of a citizen in battle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civicism</ent><br/
<hw>Civ"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The principle of civil government.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civics</ent><br/
<hw>Civ"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science of civil government.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>civies</ent><br/
<hw>civies</hw> <pos>n.</pos> <def>civilian garb as opposed to a military uniform.</def><br/
<syn><b>Syn. --</b> civvies.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Civil</ent><br/
<hw>Civ"il</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>civilis</ets>, fr. <ets>civis</ets> citizen: cf. F. <ets>civil</ets>. See <er>City</er>.]</ety> <sn>1.</sn> <def>Pertaining to a city or state, or to a citizen in his relations to his fellow citizens or to the state; within the city or state.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Subject to government; reduced to order; civilized; not barbarous; -- said of the community.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>England was very rude and barbarous; for it is but even the other day since England grew <qex>civil</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Performing the duties of a citizen; obedient to government; -- said of an individual.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Civil</qex> men come nearer the saints of God than others; they come within a step or two of heaven.</q> <rj><qau>Preston</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having the manners of one dwelling in a city, as opposed to those of savages or rustics; polite; courteous; complaisant; affable.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/A <xex>civil</xex> man now is one observant of slight external courtesies in the mutual intercourse between man and man; a <xex>civil</xex> man once was one who fulfilled all the duties and obligations flowing from his position as a 'civis' and his relations to the other members of that 'civitas.'<rdquo/</note>  <rj><au>Trench</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Pertaining to civic life and affairs, in distinction from military, ecclesiastical, or official state.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Relating to rights and remedies sought by action or suit distinct from criminal proceedings.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Civil action</b></col>, <cd>an action to enforce the rights or redress the wrongs of an individual, not involving a criminal proceeding.</cd> -- <col><b>Civil architecture</b></col>, <cd>the architecture which is employed in constructing buildings for the purposes of civil life, in distinction from military and naval architecture, as private houses, palaces, churches, etc.</cd> -- <col><b>Civil death</b></col>. <fld>(Law.)</fld> <cd>See under <er>Death</er>.</cd> -- <col><b>Civil engineering</b></col>. <cd>See under <er>Engineering</er>.</cd> -- <col><b>Civil law</b></col>. <cd>See under <er>Law</er>.</cd> -- <col><b>Civil list</b></col>. <cd>See under <er>List</er>.</cd> -- <col><b>Civil remedy</b></col> <fld>(Law)</fld>, <cd>that given to a person injured, by action, as opposed to a criminal prosecution.</cd> -- <col><b>Civil service</b></col>, <cd>all service rendered to and paid for by the state or nation other than that pertaining to naval or military affairs.</cd> -- <col><b>Civil service reform</b></col>, <cd>the substitution of business principles and methods for the spoils system in the conduct of the civil service, esp. in the matter of appointments to office.</cd> -- <col><b>Civil state</b></col>, <cd>the whole body of the laity or citizens not included under the military, maritime, and ecclesiastical states.</cd> -- <col><b>Civil suit</b></col>. <cd>Same as <cref>Civil action</cref>.</cd> -- <col><b>Civil war</b></col>. <cd>See under <er>War</er>.</cd> -- <col><b>Civil year</b></col>. <cd>See under <er>Year</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilian</ent><br/
<hw>Ci*vil"ian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Civil</er>]</ety> <sn>1.</sn> <def>One skilled in the civil law.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ancient <qex>civilians</qex> and writers upon government.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A student of the civil law at a university or college.</def>  <rj><au>R. Graves.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One whose pursuits are those of civil life, not military or clerical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilist</ent><br/
<hw>Civ"il*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A civilian.</def> <mark>[R.]</mark>  <rj><au>Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civility</ent><br/
<hw>Ci*vil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Civilities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>civilitas</ets>: cf. F. <ets>civilit<eacute/</ets>. See <er>Civil</er>.]</ety> <sn>1.</sn> <def>The state of society in which the relations and duties of a citizen are recognized and obeyed; a state of civilization.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Monarchies have risen from barbarrism to <qex>civility</qex>, and fallen again to ruin.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The gradual depature of all deeper signification from the word <qex>civility</qex> has obliged the creation of another word -- civilization.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A civil office, or a civil process</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To serve in a <qex>civility</qex>.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Courtesy; politeness; kind attention; good breeding; a polite act or expression.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The insolent <qex>civility</qex> of a proud man is, if possible, more shocking than his rudeness could be.</q> <rj><qau>Chesterfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sweet <qex>civilities</qex> of life.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Urbanity; affability; complaisance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilizable</ent><br/
<hw>Civ"i*li`za*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being civilized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilization</ent><br/
<hw>Civ`i*li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>civilisation</ets>.]</ety> <sn>1.</sn> <def>The act of civilizing, or the state of being civilized; national culture; refinement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our manners, our civilization, and all the good things connected with manners, and with <qex>civilization</qex>, have, in this European world of ours, depended for ages upon two principles -- . . . the spirit of a gentleman, and spirit of religion.</q> <rj><qau>Burke</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Rendering a criminal process civil.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilize</ent><br/
<hw>Civ"i*lize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Civilized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Civilizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>civilizer</ets>, fr.L. <ets>civilis</ets> civil. See <er>Civil</er>.]</ety> <sn>1.</sn> <def>To reclaim from a savage state; to instruct in the rules and customs of civilization; to educate; to refine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet blest that fate which did his arms dispose<br/
Her land to <qex>civilize</qex>, as to subdue.</q> <rj><qau>Dryden</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To admit as suitable to a civilized state.</def> <mark>[Obs. or R.]</mark> <ldquo/<xex>Civilizing</xex> adultery.<rdquo/  <rj><au>Milton.</au></rj></p>

<p><syn><b>Syn.</b> -- To polish; refine; humanize.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilized</ent><br/
<hw>Civ"i*lized</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Reclaimed from savage life and manners; instructed in arts, learning, and civil manners; refined; cultivated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sale of conscience and duty in open market is not reconcilable with the present state of <qex>civilized</qex> society.</q> <rj><qau>J. Quincy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civilizer</ent><br/
<hw>Civ"i*li*zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, civilizes or tends to civilize.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>civil-libertarian</ent><br/
<hw>civil-libertarian</hw> <pos>adj.</pos> <def>having or showing active concern for protection of civil liberties protected by law.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>civil-libertarian</ent><br/
<hw>civil-libertarian</hw> <pos>n.</pos> <def>a person having or showing active concern for protection of civil liberties protected by law.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>civil rights</ent><br/
<ent>civil right</ent><br/
<mhw><hw>civil right</hw>, <hw>civil rights</hw></mhw> <def>a legal right or rights belonging to a person by reason of citizenship, including especially the fundamental freedoms and privileges guaranteed by the 13th and 14th amendments and subsequent acts of congress, including the right to legal and social and economic equality.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Civil Service Commission</ent><br/
<hw>Civil Service Commission</hw>. <def>In the United States, a commission appointed by the President, consisting of three members, not more than two of whom may be adherents of the same party, which has the control, through examinations, of appointments and promotions in the classified civil service. It was created by act of Jan, 16, 1883 (22 Stat. 403).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Civil Service Reform</ent><br/
<hw>Civil Service Reform</hw>. <def>The substitution of business principles and methods for political methods in the conduct of the civil service. esp. the merit system instead of the spoils system in making appointments to office.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Civily</ent><br/
<hw>Civ"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a civil manner; as regards civil rights and privileges; politely; courteously; in a well bred manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Civism</ent><br/
<hw>Civ"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>civisme</ets>, fr.L. <ets>civis</ets> citizen.]</ety> <def>State of citizenship.</def> <mark>[R.]</mark>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cizar</ent><br/
<hw>Ciz"ar</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[From <er>Cizars</er>.]</ety> <def>To clip with scissors.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj></p>

<p><ent>Cizars</ent><br/
<hw>Ciz"ars</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Scissors.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cize</ent><br/
<hw>Cize</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Bulk; largeness. <mark>[Obs.]</mark> See <er>Size</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clabber</ent><br/
<hw>Clab"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Bonnyclabber</er>]</ety> <def>Milk curdled so as to become thick.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clabber</ent><br/
<hw>Clab"ber</hw>, <pos>v. i.</pos> <def>To become clabber; to lopper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clachan</ent><br/
<hw>Clach"an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Scot., fr. Gael.]</ety> <def>A small village containing a church.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sitting at the <qex>clachon alehouse</qex>.</q> <rj><qau>R. L. Stevenson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clack</ent><br/
<hw>Clack</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clacked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clacking</conjf>.]</vmorph> <ety>[Prob. of imitative origin; cf. F.<ets>claquer</ets> to clap, crack, D. <ets>klakken</ets>, MHG. <ets>klac</ets> crack, Ir. <ets>clagaim</ets> I make a noise, ring. Cf. <er>Clack</er>, <pos>n.</pos>, <er>Clatter</er>, <er>Click</er>.]</ety> <sn>1.</sn> <def>To make a sudden, sharp noise, or a succesion of such noises, as by striking an object, or by collision of parts; to rattle; to click.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We heard Mr.Hodson's whip <qex>clacking</qex> on the ahoulders of the poor little wretches.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 261  --></p>

<p><sn>2.</sn> <def>To utter words rapidly and continually, or with abruptness; to let the tongue run.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clack</ent><br/
<hw>Clack</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to make a sudden, sharp noise, or succession of noises; to click.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter rapidly and inconsiderately.</def>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To clack wool</b></col>, <cd>to cut off the sheep's mark, in order to make the wool weigh less and thus yield less duty</cd>. <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clack</ent><br/
<hw>Clack</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>claque</ets> a slap or smack, MHG. <ets>klac</ets> crack, W. <ets>clec</ets> crack, gossip. See <er>Clack</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A sharp, abrupt noise, or succession of noises, made by striking an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything that causes a clacking noise, as the clapper of a mill, or a clack valve.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Continual or importunate talk; prattle; prating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose chief intent is to vaunt his spiritual <qex>clack</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clack box</b></col> <fld>(Mach.)</fld>, <cd>the box or chamber in which a clack valve works.</cd> -- <col><b>Clack dish</b></col>, <cd>a dish with a movable lid, formerly carried by beggars, who clacked the lid to attract notice.</cd>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clack door</b></col> <fld>(Mining)</fld>, <cd>removable cover of the opening through which access is had to a pump valve.</cd> -- <col><b>Clack valve</b></col> <fld>(Mach.)</fld>, <cd>a valve; esp. one hinged at one edge, which, when raised from its seat, falls with a clacking sound.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clacker</ent><br/
<hw>Clack"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who clacks; that which clacks; especially, the clapper of a mill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A claqueur. See <er>Claqueur</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clad</ent><br/
<hw>Clad</hw> <pr>(kl<acr/d)</pr>, <pos>v. t.</pos> <def>To clothe.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clad</ent><br/
<hw>Clad</hw>, <def><pos>imp. & p. p.</pos> of <er>Clothe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cladocera</ent><br/
||<hw>Cla*doc"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a sprout + <?/ a horn.]</ety> <fld>(Zool.)</fld> <def>An order of the Entomostraca.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They have a bivalve shell, covering the body but not the head, and from four to six pairs of legs and two pairs of anten<ae/, for use in swimming. They mostly inhabit fresh water.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>cladode</ent><br/
<hw>cladode</hw> <pos>n.</pos> <def>a flattened stem resembling and functioning as a leaf.</def><br/
<syn><b>Syn. --</b> cladophyll, phylloclad, phylloclade.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cladonia</ent><br/
<hw>Cladonia</hw> <pos>n.</pos> <def>the type genus of <fam>Cladoniaceae</fam>; lichens characterized by a crustose thallus and capitate fruiting bodies borne on simple or branched podetia.</def><br/
<syn><b>Syn. --</b> genus <gen>Cladonia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cladophyll</ent><br/
<hw>Clad"o*phyll</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a sprout + <?/ a leaf.]</ety> <fld>(Bot.)</fld> <def>A special branch, resembling a leaf, as in the apparent foliage of the broom (<gen>Ruscus</gen>) and of the common cultivated smilax (<gen>Myrsiphillum</gen>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claggy</ent><br/
<hw>Clag"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Clog</er>.]</ety> <def>Adhesive; -- said of a roof in a mine to which coal clings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claik</ent><br/
<hw>Claik</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Clake</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claim</ent><br/
<hw>Claim</hw> <pr>(kl<amac/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Claimed</conjf> <pr>(kl<amac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Claiming</conjf>.]</vmorph> <ety>[OE. <ets>clamen</ets>, <ets>claimen</ets>, OF. <ets>clamer</ets>, fr. L. <ets>clamare</ets> to cry out, call; akin to <ets>calare</ets> to proclaim, Gr. <grk>kalei^n</grk> to call, Skr. <ets>kal</ets> to sound, G. <ets>holen</ets> to fetch, E. <ets>hale</ets> haul.]</ety> <sn>1.</sn> <def>To ask for, or seek to obtain, by virtue of authority, right, or supposed right; to challenge as a right; to demand as due.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To proclaim.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To call or name.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To assert; to maintain.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claim</ent><br/
<hw>Claim</hw>, <pos>v. i.</pos> <def>To be entitled to anything; to deduce a right or title; to have a claim.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We must know how the first ruler, from whom any one <qex>claims</qex>, came by his authority.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claim</ent><br/
<hw>Claim</hw>, <pos>n.</pos> <ety>[Of. <ets>claim</ets> cry, complaint, from <ets>clamer</ets>. See <er>Claim</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A demand of a right or supposed right; a calling on another for something due or supposed to be due; an assertion of a right or fact.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A right to claim or demand something; a title to any debt, privilege, or other thing in possession of another; also, a title to anything which another should give or concede to, or confer on, the claimant.</def> <ldquo/A bar to all <xex>claims</xex> upon land.<rdquo/  <rj><au>Hallam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The thing claimed or demanded; that (as land) to which any one intends to establish a right; ; <as>as, a settler's <ex>claim</ex>; a miner's <ex>claim</ex></as>.</def> <mark>[U.S. & Australia]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A loud call.</def> <mark>[Obs.]</mark>  <rj><au>Spenser</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To lay claim to</b></col>, <cd>to demand as a right.</cd> <ldquo/Doth he <qex>lay claim to</qex> thine inheritance?<rdquo/ <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claimable</ent><br/
<hw>Claim"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being claimed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claimant</ent><br/
<hw>Claim"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>clamant</ets>, p. pr. of <ets>clamer</ets>. Cf. <er>Clamant</er>.]</ety> <def>One who claims; one who asserts a right or title; a claimer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claimer</ent><br/
<hw>Claim"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who claims; a claimant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claimless</ent><br/
<hw>Claim"less</hw>, <pos>a.</pos> <def>Having no claim.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clairaudience</ent><br/
<hw>Clair*au"di*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clair</ets> clear + F. & E. <ets>audience</ets> a hearing. See <er>Clear</er>.]</ety> <def>Act of hearing, or the ability to hear, sounds not normally audible; -- usually claimed as a special faculty of spiritualistic mediums, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clairaudient</ent><br/
<hw>Clair*au"di*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or characterized by, clairaudience.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clairaudient</ent><br/
<hw>Clair*au"di*ent</hw>, <pos>n.</pos> <def>One alleged to have the power of clairaudience.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Claire</ent><br/
||<hw>Claire</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A small inclosed pond used for gathering and greening oysters.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clair-obscur</ent><br/
||<hw>Clair"-ob*scur"</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[F. See <er>Clare-obscure</er>.]</ety> <def>See <er>Chiaroscuro</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clairvoyance</ent><br/
<hw>Clair*voy"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A power, attributed to some persons while in a mesmeric state, of discerning objects not perceptible by the senses in their normal condition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clairvoyant</ent><br/
<hw>Clair*voy"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. <ets>clair</ets> clear + <ets>voyant</ets>, p. pr. of <ets>voir</ets> to see. See <er>Clear</er>, and <er>Vision</er>.]</ety> <def>Pertaining to clairvoyance; discerning objects while in a mesmeric state which are not present to the senses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clairvoyant</ent><br/
<hw>Clair*voy"ant</hw> <pos>n.</pos> <def>One who is able, when in a mesmeric state, to discern objects not present to the senses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claik</ent><br/
<ent>Clake</ent><br/
<mhw>{ <hw>Clake</hw>, <hw>Claik</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>The bernicle goose; -- now called <altname>barnacle goose</altname> and also called <altname>clack goose</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clam</ent><br/
<hw>Clam</hw> <pr>(kl<acr/m)</pr>, <pos>n.</pos> <ety>[Cf. <er>Clamp</er>, <er>Clam</er>, <pos>v. t.</pos>, <er>Clammy</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A bivalve mollusk of many kinds, especially those that are edible; <as>as, the long <ex>clam</ex> (<spn>Mya arenaria</spn>), the quahog or round <ex>clam</ex> (<spn>Venus mercenaria</spn>), the sea <ex>clam</ex> or hen <ex>clam</ex> (<spn>Spisula solidissima</spn>), and other species of the United States</as>.  The name is said to have been given originally to the <spn>Tridacna gigas</spn>, a huge East Indian bivalve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You shall scarce find any bay or shallow shore, or cove of sand, where you may not take many <qex>clampes</qex>, or lobsters, or both, at your pleasure.</q> <rj><qau>Capt. John Smith (1616).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clams</qex>, or <qex>clamps</qex>, is a shellfish not much unlike a cockle; it lieth under the sand.</q> <rj><qau>Wood (1634).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>   <fld>(Ship Carp.)</fld> <def>Strong pinchers or forceps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Mech.)</fld> <def>A kind of vise, usually of wood.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blood clam</b></col>. <cd>See under <er>Blood</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clam</ent><br/
<hw>Clam</hw> <pr>(cl<acr/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clammed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clamming</conjf>.]</vmorph> <ety>[Cf. AS. <ets>cl<ae/man</ets> to clam, smear; akin to Icel. <ets>kleima</ets> to smear, OHG. <ets>kleimjan</ets>, <ets>chleimen</ets>, to defile, or E. <ets>clammy</ets>.]</ety> <def>To clog, as with glutinous or viscous matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A swarm of wasps got into a honey pot, and there they cloyed and <qex>clammed</qex> Themselves till there was no getting out again.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clam</ent><br/
<hw>Clam</hw>, <pos>v. i.</pos> <def>To be moist or glutinous; to stick; to adhere.</def> <mark>[R.]</mark>  <rj><au>Dryden</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clam</ent><br/
<hw>Clam</hw>, <pos>n.</pos> <def>Claminess; moisture.</def> <mark>[R.]</mark> <ldquo/The <xex>clam</xex> of death.<rdquo/  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clam</ent><br/
<hw>Clam</hw>, <pos>n.</pos> <ety>[Abbrev. fr. <ets>clamor</ets>.]</ety> <def>A crash or clangor made by ringing all the bells of a chime at once.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clam</ent><br/
<hw>Clam</hw>, <pos>v. t. & i.</pos> <def>To produce, in bell ringing, a clam or clangor; to cause to clang.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamant</ent><br/
<hw>Cla"mant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clamans</ets>, p. pr. of clamare to call. Cf. <er>Claimant</er>.]</ety> <def>Crying earnestly, beseeching clamorously.</def> <ldquo/<xex>Clamant</xex> children.<rdquo/  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamation</ent><br/
<hw>Cla*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>clamatio</ets>, fr. L. <ets>clamare</ets> to call.]</ety> <def>The act of crying out.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamatores</ent><br/
||<hw>Clam`a*to"res</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L. <ets>clamator</ets>, pl. <ets>clamatores</ets>, a bawler.]</ety> <fld>(Zool.)</fld> <def>A division of passerine birds in which the vocal muscles are but little developed, so that they lack the power of singing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamatorial</ent><br/
<hw>Clam`a*to"rial</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Like or pertaining to the Clamatores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clambake</ent><br/
<hw>Clam"bake</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The backing or steaming of clams on heated stones, between layers of seaweed; hence, a picnic party, gathered on such an occasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamber</ent><br/
<hw>Clam"ber</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clambered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clambering</conjf>.]</vmorph> <ety>[OE <ets>clambren</ets>, <ets>clameren</ets>, to heap together, climb; akin to Icel. <ets>klambra</ets> to clamp, G. <ets>klammern</ets>. Cf. <er>Clamp</er>, <er>Climb</er>.]</ety> <def>To climb with difficulty, or with hands and feet; -- also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The narrow street that <qex>clambered</qex> toward the mill.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamber</ent><br/
<hw>Clam"ber</hw>, <pos>n.</pos> <def>The act of clambering.</def>  <rj><au>T. Moore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamber</ent><br/
<hw>Clam"ber</hw>, <pos>v. t.</pos> <def>To ascend by climbing with difficulty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clambering</qex> the walls to eye him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamjamphrie</ent><br/
<hw>Clam*jam"phrie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Low, worthless people; the rabble.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clammily</ent><br/
<hw>Clam"mi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a clammy manner.</def> <ldquo/Oozing so <xex>clammily</xex>.<rdquo/  <rj><au>Hood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamminess</ent><br/
<hw>Clam"mi*ness</hw>, <pos>n.</pos> <def>State of being clammy or viscous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clammy</ent><br/
<hw>Clam"my</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Clammier</adjf> <pr>(?)</pr>; superl. <adjf>Clammiest</adjf>.]</amorph> <ety>[Cf. AS. <ets>cl<amac/m</ets> clay. See <er>Clam</er> to clog, and cf. <er>Clay</er>.]</ety> <def>Having the quality of being viscous or adhesive; soft and sticky; glutinous; damp and adhesive, as if covered with a cold perspiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clammyweed</ent><br/
<hw>clammyweed</hw> <pos>n.</pos> <def>strong-scented herb (<spn>Polanisia graveolens</spn>) common in the southern U. S. covered with intermixed gland and hairs.</def><br/
<syn><b>Syn. --</b> <spn>Polanisia graveolens</spn>, Polanisia dodecandra.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clamor</ent><br/
<hw>Clam"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>clamour</ets>, <ets>clamur</ets>, F. <ets>clameur</ets>, fr. L. <ets>clamor</ets>, fr. <ets>clamare</ets> to cry out. See <er>Claim</er>.]</ety> <sn>1.</sn> <def>A great outcry or vociferation; loud and continued shouting or exclamation from many people.</def>  <rj><au>Shak.</au></rj> <altsp>[Also spelled <asp>clamour</asp>.]</altsp><br/
<syn><b>Syn. --</b> clamor, hue and cry.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>Any loud and continued noise.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A continued expression of dissatisfaction or discontent; a popular outcry.</def>  <rj><au>Macaulay.</au></rj></p>

<p><syn><b>Syn.</b> -- Outcry; exclamation; noise; uproar.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamor</ent><br/
<hw>Clam"or</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clamored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clamoring</conjf>.]</vmorph> <sn>1.</sn> <def>To salute loudly.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The people with a shout<br/
Rifted the air, <qex>clamoring</qex> their god with praise.</q> <rj><qau>Milton</qau></rj>.<br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stun with noise.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter loudly or repeatedly; to shout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clamored</qex> their piteous prayer incessantly.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To clamor bells, to repeat the strokes quickly so as to produce a loud clang.</q> <rj><qau>Bp. Warbur<?/ion.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamor</ent><br/
<hw>Clam"or</hw>, <pos>v. i.</pos> <def>To utter loud sounds or outcries; to vociferate; to talk in a loud voice; to complain; to make importunate demands.</def><br/
<syn><b>Syn. --</b>  clamor, roar, vociferate, holler, hollo.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><q>The obscure bird<br/
<qex>Clamored</qex> the livelong night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to dispute in a loud voice.</def><br/
<syn><b>Syn. --</b> brawl, wrangle, clamor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clamorer</ent><br/
<hw>Clam"or*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who clamors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamorous</ent><br/
<hw>Clam"or*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>clamorosus</ets>, for L. <ets>Clamosus</ets>: cf. OF. <ets>clamoreux</ets>.]</ety> <def>Speaking and repeating loud words; full of clamor; calling or demanding loudly or urgently; vociferous; noisy; bawling; loud; turbulent.</def> <ldquo/My young ones were <xex>clamorous</xex> for a morning's excursion.<rdquo/  <rj><au>Southey.</au></rj></p>

<p>-- <wordforms><wf>Clam"or*ous*ly</wf>, <pos>adv.</pos> -- <wf>Clam"or*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>clamour</ent><br/
<hw>clamour</hw> <pos>n. and v.</pos> <def>same as <er>clamor</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>clamourous</ent><br/
<hw>clamourous</hw> <pos>adj.</pos> <def>same as <er>clamorous</er>.</def><br/
<syn><b>Syn. --</b> blatant, clamant, clamorous, vociferous.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Clamp</ent><br/
<hw>Clamp</hw> <pr>(kl<acr/mp)</pr>, <pos>n.</pos> <ety>[Cf. LG. & D. <ets>klamp</ets>, Dan. <ets>klampe</ets>, also D. <ets>klampen</ets> to fasten, clasp. Cf. <er>Clamber</er>, <er>Cramp</er>.]</ety> <sn>1.</sn> <def>Something rigid that holds fast or binds things together; a piece of wood or metal, used to hold two or more pieces together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>An instrument with a screw or screws by which work is held in its place or two parts are temporarily held together.</def> <sd>(b)</sd> <fld>(Joinery)</fld> <def>A piece of wood placed across another, or inserted into another, to bind or strengthen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of a pair of movable pieces of lead, or other soft material, to cover the jaws of a vise and enable it to grasp without bruising.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Shipbuilding)</fld> <def>A thick plank on the inner part of a ship's side, used to sustain the ends of beams.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A mass of bricks heaped up to be burned; or of ore for roasting, or of coal for coking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A mollusk. See <er>Clam</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clamp nails</b></col>, <cd>nails used to fasten on clamps in ships.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamp</ent><br/
<hw>Clamp</hw> <pr>(kl<acr/mp)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clamped</conjf> <pr>(kl<acr/mt; 215)</pr> <pos>p. pr. & vb. n.</pos> <conjf>Clamping</conjf>.]</vmorph> <sn>1.</sn> <def>To fasten with a clamp or clamps; to apply a clamp to; to place in a clamp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover, as vegetables, with earth.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamp</ent><br/
<hw>Clamp</hw>, <pos>n.</pos> <ety>[Prob. an imitative word. Cf. <er>Clank</er>.]</ety> <def>A heavy footstep; a tramp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clamp</ent><br/
<hw>Clamp</hw>, <pos>v. i.</pos> <def>To tread heavily or clumsily; to clump.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The policeman with <qex>clamping</qex> feet.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clampdown</ent><br/
<hw>clampdown</hw> <pos>n.</pos> <def>sudden restriction on an activity.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clamper</ent><br/
<hw>Clamp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument of iron, with sharp prongs, attached to a boot or shoe to enable the wearer to walk securely upon ice; a creeper.</def>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clamshell</ent><br/
<hw>clamshell</hw> <pos>n.</pos> <sn>1.</sn> <def>the shell of a clam.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a dredging bucket with hinges like the shell of a clam.</def><br/
<syn><b>Syn. --</b> grapple.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clamydospore</ent><br/
<hw>clamydospore</hw> <pos>n.</pos> <def>a thick-walled asexual resting spore of certain fungi and algae.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clan</ent><br/
<hw>Clan</hw> <pr>(kl<acr/n)</pr>, <pos>n.</pos> <ety>[Gael. <ets>clann</ets> offspring, descendants; akin to Ir. <ets>clann</ets>, <ets>cland</ets>, offspring, tribe, family; perh. from L. <ets>plania</ets> scion, slip, cutting. Cf. <er>Plant</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A tribe or collection of families, united under a chieftain, regarded as having the same common ancestor, and bearing the same surname; <as>as, the <ex>clan</ex> of Macdonald</as>.</def> <ldquo/I have marshaled my <xex>clan</xex>.<rdquo/  <rj><au>Campbell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A clique; a sect, society, or body of persons; esp., a body of persons united by some common interest or pursuit; -- sometimes used contemptuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Partidge and the rest of his <qex>clan</qex> may hoot me.</q> <rj><qau>Smolett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole <qex>clan</qex> of the enlightened among us.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clancular</ent><br/
<hw>Clan"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clancularius</ets> , from <ets>clanculum</ets> secretly, adv. dim. of <ets>clam</ets> secretly.]</ety> <def>Conducted with secrecy; clandestine; concealed.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not close and <qex>clancular</qex>, but frank and open.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clancularly</ent><br/
<hw>Clan"cu*lar*ly</hw>, <pos>adv.</pos> <def>privately; secretly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clandestine</ent><br/
<hw>Clan*des"tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clandestinus</ets>, fr. <ets>clam</ets> secretly; akin to <ets>celare</ets>, E. <ets>conceal</ets>: cf. F. <ets>clandestin</ets>.]</ety> <def>Conducted with secrecy; withdrawn from public notice, usually for an evil purpose; kept secret; hidden; private; underhand; <as>as, a <ex>clandestine</ex> marriage</as>.</def>  <rj><au>Locke.</au></rj></p>

<p><syn><b>Syn.</b> -- Hidden; secret; private; concealed; underhand; sly; stealthy; surreptitious; furtive; fraudulent.</syn></p>

<p> -- <wordforms><wf>Clan*des"tine*ly</wf>, <pos>adv.</pos> -- <wf>Clan*des"tine*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clandestinity</ent><br/
<hw>Clan`des*tin"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Privacy or secrecy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clang</ent><br/
<hw>Clang</hw> <pr>(kl<acr/ng)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clanged</conjf> <pr>(kl<acr/ngd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clanging</conjf>.]</vmorph> <ety>[L. <ets>clangere</ets>; akin to Gr. <grk>kla`zein</grk> to clash, scream; or perh.  to E. <ets>clank</ets>.]</ety> <def>To strike together so as to produce a ringing metallic sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fierce Caretes . . . <qex>clanged</qex> their sounding arms.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clang</ent><br/
<hw>Clang</hw>, <pos>v. i.</pos> <def>To give out a clang; to resound.</def> <ldquo/<xex>Clanging</xex> hoofs.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clang</ent><br/
<hw>Clang</hw>, <pos>n.</pos> <sn>1.</sn> <def>A loud, ringing sound, like that made by metallic substances when clanged or struck together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The broadsword's deadly <qex>clang</qex>,<br/
As if a thousand anvils rang.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Quality of tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clanger</ent><br/
<hw>clanger</hw> <pos>n.</pos> <def>A conspicuous mistake whose effects seem to reverberate; <as>as, he dropped a <ex>clanger</ex></as>.</def> <mark>[British]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clanging</ent><br/
<hw>clanging</hw> <pos>adj.</pos>  <def>emitting a series of clangs, as of metal objects colliding.</def><br/
<syn><b>Syn. --</b> clangorous.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clangor</ent><br/
<hw>Clan"gor</hw> <pr>(kl<acr/<nsm/"g<etil/r)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>clangere</ets>. See <er>Clang</er>, <pos>v. t.</pos>]</ety> <def>A sharp, harsh, ringing sound.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clangoring</ent><br/
<hw>clangoring</hw> <pos>n.</pos> <def>a loud resonant repeating noise.</def><br/
<syn><b>Syn. --</b> clang, clangor, clangour, clank, clash, crash, jangle.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clangorous</ent><br/
<hw>Clan"gor*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>clangorosus</ets>.]</ety> <def>Making a clangor; having a ringing, metallic sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clangous</ent><br/
<hw>Clan"gous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Making a clang, or a ringing metallic sound.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clanjamfrie</ent><br/
<hw>Clan*jam"frie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Clamjamphrie</er>.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clank</ent><br/
<hw>Clank</hw> <pr>(kl<acr/<nsm/k)</pr>, <pos>n.</pos> <ety>[Akin to <ets>clink</ets>, and of imitative origin; cf. G. <ets>klang</ets> sound, D. <ets>klank</ets>. Cf. <er>Clang</er>.]</ety> <def>A sharp, brief, ringing sound, made by a collision of metallic or other sonorous bodies; -- usually expressing a duller or less resounding sound than <xex>clang</xex>, and a deeper and stronger sound than <xex>clink</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But not in chains to pine,<br/
His spirit withered with tyeur <qex>clank</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clank</ent><br/
<hw>Clank</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clanked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clanking</conjf>.]</vmorph> <def>To cause to sound with a clank; <as>as, the prisoners <ex>clank</ex> their chains</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clank</ent><br/
<hw>Clank</hw>, <pos>v. i.</pos> <def>To sound with a clank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clankless</ent><br/
<hw>Clank"less</hw>, <pos>a.</pos> <def>Without a clank.</def>  <rj><au>Byreon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clan-na-Gael</ent><br/
<hw>Clan"-na-Gael"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir., clan of the Gaels.]</ety> <def>A secret society of Irish Fenians founded in Philadelphia in 1881.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clannish</ent><br/
<hw>Clan"nish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a clan; closely united, like a clan; disposed to associate only with one's clan or clique; actuated by the traditions, prejudices, habits, etc., of a clan.</def></p>

<p>-- <wordforms><wf>Clan"nish*ly</wf>, <pos>adv.</pos> -- <wf>clan"nish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>clannishness</ent><br/
<hw>clan"nish*ness</hw> <pos>n.</pos> <sn>1.</sn> <def>a tendency to associate with only a select group.</def><br/
<syn><b>Syn. --</b> cliquishness, exclusiveness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a tendency to associate with only one's own clan; the propensity to display the sentiments, habits, and prejudices of one's own clan.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clanship</ent><br/
<hw>Clan"ship</hw>, <pos>n.</pos> <def>A state of being united together as in a clan; an association under a chieftain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clansman</ent><br/
<hw>Clans"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Clansmen</plw> <pr>(#)</pr>.</plu> <def>One belonging to the same clan with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clanswoman</ent><br/
<hw>clanswoman</hw> <pos>n.</pos> <def>a female member of a clan.</def><br/
<syn><b>Syn. --</b> clansman, clan member.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clap</ent><br/
<hw>Clap</hw> <pr>(kl<acr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clapped</conjf> <pr>(kl<acr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clapping</conjf>.]</vmorph> <ety>[AS. <ets>clappan</ets>; akin to Icel. & Sw. <ets>klappa</ets>, D, <ets>klappen</ets>, to clap, prate, G. <ets>klaffen</ets>, v. i., to split open, yelp, <ets>klopfen</ets>, v. t. & i., to knock.]</ety> <sn>1.</sn> <def>To strike; to slap; to strike, or strike together, with a quick motion, so, as to make a sharp noise; <as>as, to <ex>clap</ex> one's hands; a <ex>clapping</ex> of wings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then like a bird it sits and sings,<br/
And whets and <qex>claps</qex> its silver wings.</q> <rj><qau>Marvell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To thrust, drive, put, or close, in a hasty or abrupt manner; -- often followed by <xex>to</xex>, <xex>into</xex>, <xex>on</xex>, or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had just time to get in and <qex>clap</qex> to the door.</q> <rj><qau>Locke</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clap</qex> an extinguisher upon your irony.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To manifest approbation of, by striking the hands together; to applaud; <as>as, to <ex>clap</ex> a performance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To clap hands</b></col>. <sd>(a)</sd> <cd>To pledge faith by joining hands.</cd> <mark>[Obs.]</mark> <au>Shak.</au> <sd>(b)</sd> <def>To express contempt or derision.</def> <mark>[Obs.]</mark> <au>Lam. ii. 15.</au> -- <col><b>To clap hold of</b></col>, <cd>to seize roughly or quickly.</cd> -- <col><b>To clap up</b></col>. <sd>(a)</sd> <cd>To imprison hastily or without due formality.</cd> <sd>(b)</sd> <cd>To make or contrive hastily.</cd> <mark>[Obs.]</mark> <ldquo/Was ever match <xex>clapped up</xex> so suddenly?<rdquo/  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clap</ent><br/
<hw>Clap</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To knock, as at a door.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike the hands together in applause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their ladies bid them <qex>clap</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To come together suddenly with noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The doors around me <qex>clapped</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To enter with alacrity and briskness; -- with <xex>to</xex> or <xex>into</xex>.</def> <mark>[Obs.]</mark> <ldquo/Shall we <xex>clap</xex> into it roundly, without . . . saying we are hoarse?<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To talk noisily; to chatter loudly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 262 --></p>

<p><ent>Clap</ent><br/
<hw>Clap</hw> <pr>(kl<acr/p)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A loud noise made by sudden collision; a bang.</def> <ldquo/Give the door such a <xex>clap</xex>, as you go out, as will shake the whole room.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A burst of sound; a sudden explosion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Horrible <qex>claps</qex> of thunder.</q> <rj><qau>Hakewill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A single, sudden act or motion; a stroke; a blow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What, fifty of my followers at a <qex>clap</qex>!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A striking of hands to express approbation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Unextrected <qex>claps</qex> or hisses.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Noisy talk; chatter.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Falconry)</fld> <def>The nether part of the beak of a hawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clap dish</b></col>. <cd>See <cref>Clack dish</cref>, under <er>Clack</er>, <pos>n.</pos></cd> -- <col><b>Clap net</b></col>, <cd>a net for taking birds, made to close or clap together.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clap</ent><br/
<hw>Clap</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>clapoir</ets>.]</ety> <def>Gonorrhea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clapboard</ent><br/
<hw>Clap"board</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A narrow board, thicker at one edge than at the other; -- used for weatherboarding the outside of houses.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stave for a cask.</def> <mark>[Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clapboard</ent><br/
<hw>Clap"board</hw>, <pos>v. t.</pos> <def>To cover with clapboards; <as>as, to <ex>clapboard</ex> the sides of a house</as>.</def> <mark>[U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clapcake</ent><br/
<ent>Clapbread</ent><br/
<mhw>{ <hw>Clap"bread`</hw> <pr>(?)</pr>, <hw>Clap"cake`</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>Oatmeal cake or bread clapped or beaten till it is thin.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clape</ent><br/
<hw>Clape</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A bird; the flicker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clapper</ent><br/
<hw>Clap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person who claps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which strikes or claps, as the tongue of a bell, or the piece of wood that strikes a mill hopper, etc. See <xex>Illust.</xex> of <er>Bell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clapper rail</b></col> <fld>(Zool.)</fld>, <cd>an Americam species of rail (<spn>Rallus scepitans</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clapper</ent><br/
<hw>Clap"per</hw>, <pos>n.</pos> <ety>[F. <ets>clapier</ets>.]</ety> <def>A rabbit burrow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>clapperboard</ent><br/
<hw>clap"per*board`</hw> <pr>(kl<acr/p"p<etil/r*b<omac/rd`)</pr>, <pos>n.</pos> <def>a device which synchronizes sound and picture while making a motion picture, consisting of boards held in front of a movie camera, which are are banged together.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clapperclaw</ent><br/
<hw>Clap"per*claw</hw> <pr>(kl<acr/p"p<etil/r*kl<add/)</pr>, <pos>v. t.</pos> <ety>[<ets>Clap</ets> + <ets>claw</ets>.]</ety> <sn>1.</sn> <def>To fight and scratch.</def>  <rj><au>C. Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To abuse with the tongue; to revile; to scold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claps</ent><br/
<hw>Claps</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>Variant of <er>Clasp</er></def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claptrap</ent><br/
<hw>Clap"trap`</hw> <pr>(kl<acr/p"tr<acr/p`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A contrivance for clapping in theaters.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A trick or device to gain applause, especially pretentious but empty rhetoric; humbug.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Claptrap</ent><br/
<hw>Clap"trap`</hw>, <pos>a.</pos> <def>Contrived for the purpose of making a show, or gaining applause; deceptive; unreal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claque</ent><br/
||<hw>Claque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A collection of persons employed to applaud at a theatrical exhibition.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a group of sycophantic followers.</def><br/
[<source>PJC</source>]</p>

<p><ent>Claqueur</ent><br/
||<hw>Cla`queur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>One of the claque employed to applaud at a theater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clare</ent><br/
<hw>Clare</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A nun of the order of St. Clare.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarence</ent><br/
<hw>Clar"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A close four-wheeled carriage, with one seat inside, and a seat for the driver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarencieux</ent><br/
<ent>Clarenceux</ent><br/
<mhw>{ <hw>Clar"en*ceux</hw>, <hw>Clar"en*cieux</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>See <er>King-at-arms</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarendon</ent><br/
<hw>Clar"en*don</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A style of type having a narrow and heave face. It is made in all sizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This line is in nonpareil Clarendon.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clare-obscure</ent><br/
<hw>Clare"-ob*scure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>clarus</ets> clear + <ets>obscurus</ets> obscure; cf. F. <ets>clair-obscur</ets>. Cf. <er>Chiaroscuro</er>.]</ety> <fld>(Painting)</fld> <def>See <er>Chiaroscuro</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claret</ent><br/
<hw>Clar"et</hw> <pr>(kl<acr/r"<ecr/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>claret</ets>, <ets>clare</ets>, <ets>clarry</ets>, OF. <ets>claret</ets>, <ets>clar<eacute/</ets>, fr. <ets>cler</ets>, F. <ets>clair</ets>, clear, fr. L. <ets>clarus</ets> clear. See <er>Clear</er>.]</ety> <def>The name first given in England to the red wines of M<eacute/doc, in France, and afterwards extended to all the red Bordeaux wines. The name is also given to similar wines made in the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claribella</ent><br/
<hw>Clar`i*bel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>clarus</ets> clear + <ets>bellus</ets> fine.]</ety> <fld>(Mus.)</fld> <def>A soft, sweet stop, or set of open wood pipes in an organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarichord</ent><br/
<hw>Clar"i*chord</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clatocorde</ets>, fr.L. <ets>clarus</ets> clear + <ets>chorda</ets> string. See <er>Chord</er>.]</ety> <def>A musical instrument, formerly in use, in form of a spinet; -- called also <altname>manichord</altname> and <altname>clavichord</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarification</ent><br/
<hw>Clar`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>clarification</ets>, L. <ets>clarificatio</ets> glorification.]</ety> <sn>1.</sn> <def>The act or process of making clear or transparent, by freeing visible impurities; <as>as, the <ex>clarification</ex> of wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of freeing from obscurities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>clarification</qex> of men's ideas.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarifier</ent><br/
<hw>Clar"i*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which clarifies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel in which the process of clarification is conducted; <as>as, the <ex>clarifier</ex> in sugar works</as>.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarify</ent><br/
<hw>Clar"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clarified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clarifying</conjf>.]</vmorph> <ety>[F. <ets>clarifier</ets>, from L. <ets>clarificare</ets>; <ets>clarus</ets> clear + <ets>facere</ets> to make. See <er>Clear</er>, and <er>Fact</er>.]</ety> <sn>1.</sn> <def>To make clear or bright by freeing from feculent matter; to defecate; to fine; -- said of liquids, as wine or sirup.</def> <ldquo/Boiled and <xex>clarified</xex>.<rdquo/  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make clear; to free from obscurities; to brighten or illuminate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>clarify</qex> his reason, and to rectify his will.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To glorify.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Fadir, <qex>clarifie</qex> thi name.</q> <rj><qau>Wyclif (John ii. 28).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarify</ent><br/
<hw>Clar"i*fy</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To grow or become clear or transparent; to become free from feculent impurities, as wine or other liquid under clarification.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To grow clear or bright; to clear up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever hath his mind fraught with many thoughts, his wits and understanding do <qex>clarify</qex> and break up in the discoursing with another.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarigate</ent><br/
<hw>Clar"i*gate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>clarigare</ets>]</ety> <def>To declare war with certain ceremonies.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarinet</ent><br/
<hw>Clar"i*net`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clarinette</ets>, dim. of <ets>clarine</ets>, from L. <ets>clarus</ets>. See <er>Clear</er>, and cf. <er>Clarion</er>.]</ety> <fld>(Mus.)</fld> <def>A wind instrument, blown by a single reed, of richer and fuller tone than the oboe, which has a double reed. It is the leading instrument in a military band.</def> <note>[Often improperly called <altname>clarionet</altname>.]</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>clarinettist</ent><br/
<ent>clarinetist</ent><br/
<mhw><hw>clarinetist</hw>, <hw>clarinettist</hw></mhw> <pos>n.</pos> <def>a musician who plays the clarinet.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clarino</ent><br/
||<hw>Cla*ri"no</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. a trumpet.]</ety> <fld>(Mus.)</fld> <def>A reed stop in an organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarion</ent><br/
<hw>Clar"i*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>clarioun</ets>, OF. <ets>clarion</ets>, F. <ets>clairon</ets>, LL. <ets>clario</ets>, <ets>claro</ets>; so called from its clear tone, fr. L. <ets>clarus</ets> clear. See <er>Clear</er>.]</ety> <def>A kind of trumpet, whose note is clear and shrill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He sounds his imperial <qex>clarion</qex> along the whole line of battle.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarionet</ent><br/
<hw>Clar`i*o*net"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Clarion</er>, <er>Clarinet</er>.]</ety> <fld>(Mus.)</fld> <def>See <er>Clarinet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarisonus</ent><br/
<hw>Cla*ris"o*nus</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clarisonus</ets>; <ets>clarus + sonus</ets>.]</ety> <def>Having a clear sound.</def> <mark>[Obs.]</mark>  <rj><au>Ash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claritude</ent><br/
<hw>Clar"i*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>claritudo</ets>, fr. <ets>clarus</ets> clear.]</ety> <def>Clearness; splendor.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarity</ent><br/
<hw>Clar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>claritas</ets>, fr. <ets>clarus</ets> clear: cf. F. <ets>clart<eacute/</ets>.]</ety> <def>Clearness; brightness; splendor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Floods, in whose more than crystal <qex>clarity</qex>,<br/
Innumerable virgin graces row.</q> <rj><qau>Beaumont.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claro-obscuro</ent><br/
<hw>Cla"ro-ob*scu"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Chiaroscuro</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarre</ent><br/
<hw>Clar`r<eacute/"</hw>, <pos>n.</pos> <ety>[See <er>Claret</er>.]</ety> <def>Wine with a mixture of honey and species.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clart</ent><br/
<hw>Clart</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. Armor. <ets>kalar</ets> mud, mire, <ets>kalara</ets> to dirt, Sw. <ets>lort</ets> mud.]</ety> <def>To daub, smear, or spread, as with mud, etc.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clarty</ent><br/
<hw>Clart"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sticky and foul; muddy; filthy; dirty.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clary</ent><br/
<hw>Clar"y</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Clarion</er>.]</ety> <def>To make a loud or shrill noise.</def> <mark>[Obs.]</mark>  <rj><au>Golding.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clary</ent><br/
<hw>Cla"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>sclarea</ets>, <ets>scarlea</ets>, D. & G. <ets>scharlei</ets>, F. <ets>sclar<eacute/e</ets>.]</ety> <fld>(Bot.)</fld> <def>A plant (<spn>Salvia sclarea</spn>) of the Sage family, used in flavoring soups.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clary water</b></col>, <cd>a composition of clary flowers with brandy, etc., formerly used as a cardiac.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clash</ent><br/
<hw>Clash</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clashing</conjf>.]</vmorph> <ety>[Of imitative origin; cf. G. <ets>klatschen</ets>, Prov. G. <ets>kleschen</ets>, D. <ets>kletsen</ets>, Dan. <ets>klaske</ets>, E. <ets>clack</ets>.]</ety> <sn>1.</sn> <def>To make a noise by striking against something; to dash noisily together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To meet in opposition; to act in a contrary direction; to come onto collision; to interfere.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>However some of his interests might <qex>clash</qex> with those of the chief adjacent colony.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clash</ent><br/
<hw>Clash</hw>, <pos>v. t.</pos> <def>To strike noisily against or together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clash</ent><br/
<hw>Clash</hw> <pos>n.</pos> <sn>1.</sn> <def>A loud noise resulting from collision; a noisy collision of bodies; a collision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The roll of cannon and <qex>clash</qex> of arms.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Opposition; contradiction; as between differing or contending interests, views, purposes, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clashes</qex> between popes and kings.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clash gear</ent><br/
<hw>Clash gear</hw>. <fld>(Mach.)</fld> <def>A change-speed gear in which the gears are changed by sliding endwise.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clashingly</ent><br/
<hw>Clash"ing*ly</hw>, <pos>adv.</pos> <def>With clashing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clasp</ent><br/
<hw>Clasp</hw> <pr>(kl<adot/sp)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clasped</conjf> <pr>(kl<adot/spt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clasping</conjf>]</vmorph> <ety>[OE. <ets>claspen</ets>, <ets>clapsen</ets>, prob. akin to E. <ets>clap</ets>.]</ety> <sn>1.</sn> <def>To shut or fasten together with, or as with, a clasp; to shut or fasten (a clasp, or that which fastens with a clasp).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inclose and hold in the hand or with the arms; to grasp; to embrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To surround and cling to; to entwine about.</def> <ldquo/<xex>Clasping</xex> ivy.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clasp</ent><br/
<hw>Clasp</hw>, <pos>n.</pos> <sn>1.</sn> <def>An adjustable catch, bent plate, or hook, for holding together two objects or the parts of anything, as the ends of a belt, the covers of a book, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A close embrace; a throwing of the arms around; a grasping, as with the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clasp knife</b></col>, <cd>a large knife, the blade of which folds or shuts into the handle.</cd> -- <col><b>Clasp lock</b></col>, <cd>a lock which closes or secures itself by means of a spring.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clasper</ent><br/
<hw>Clasp"er</hw> <pr>(kl<adot/sp"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, clasps, as a tendril.</def> <ldquo/The <xex>claspers</xex> of vines.<rdquo/  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of a pair of organs used by the male for grasping the female among many of the Crustacea.</def> <sd>(b)</sd> <def>One of a pair of male copulatory organs, developed on the anterior side of the ventral fins of sharks and other elasmobranchs. See <xex>Illust.</xex> of <er>Chim<ae/ra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claspered</ent><br/
<hw>Clasp"ered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with tendrils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Class</ent><br/
<hw>Class</hw> <pr>(kl<adot/s)</pr>, <pos>n.</pos> <ety>[F. <ets>classe</ets>, fr. L. <ets>classis</ets> class, collection, fleet; akin to Gr. <grk>klh^sis</grk> a calling, <grk>kalei^n</grk> to call, E. <ets>claim</ets>, <ets>haul</ets>.]</ety> <sn>1.</sn> <def>A group of individuals ranked together as possessing common characteristics; <as>as, the different <ex>classes</ex> of society; the educated <ex>class</ex>; the lower <ex>classes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A number of students in a school or college, of the same standing, or pursuing the same studies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A comprehensive division of animate or inanimate objects, grouped together on account of their common characteristics, in any classification in natural science, and subdivided into orders, families, tribes, genera, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A set; a kind or description, species or variety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She had lost one <qex>class</qex> energies.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Methodist Church)</fld> <def>One of the sections into which a church or congregation is divided, and which is under the supervision of a <xex>class leader</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>One session of formal instruction in which one or more teachers instruct a group on some subject.  The class may be one of a course of classes, or a single special session.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <def>A high degree of elegance, in dress or behavior; the quality of bearing oneself with dignity, grace, and social adeptness.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Class of a curve</b></col> <fld>(Math.)</fld>, <cd>the kind of a curve as expressed by the number of tangents that can be drawn from any point to the curve. A circle is of the second class.</cd> -- <col><b>Class meeting</b></col> <fld>(Methodist Church)</fld>, <cd>a meeting of a class under the charge of a class leader, for counsel and relegious instruction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Class</ent><br/
<hw>Class</hw> <pr>(kl<adot/s)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Classed</conjf> <pr>(kl<adot/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Classing</conjf>.]</vmorph> <ety>[Cf. F. <ets>classer</ets>. See <er>Class</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To arrange in classes; to classify or refer to some class; <as>as, to <ex>class</ex> words or passages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In scientific arrangement, to <xex>classify</xex> is used instead of to <xex>class</xex>.</note>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To divide into classes, as students; to form into, or place in, a class or classes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Class</ent><br/
<hw>Class</hw>, <pos>v. i.</pos> <def>To be grouped or classed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The genus or family under which it <qex>classes</qex>.</q> <rj><qau>Tatham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Class</ent><br/
<hw>Class</hw> <pr>(kl<adot/s)</pr>, <pos>a.</pos> <def>exhibiting refinement and high character; <as>as, a <ex>class</ex> act</as>.  Opposite of <ant>low-class</ant></def> <mark>[informal]</mark><br/
<syn><b>Syn. --</b> high-class.</syn>
[<source>PJC</source>]</p>

<p><ent>Class day</ent><br/
<hw>Class day</hw>. <def>In American colleges and universities, a day of the commencement season on which the senior class celebrates the completion of its course by exercises conducted by the members, such as the reading of the class histories and poem, the delivery of the class oration, the planting of the class ivy, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Classible</ent><br/
<hw>Class"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being classed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classical</ent><br/
<ent>Classic</ent><br/
<mhw>{ <hw>Clas"sic</hw> <pr>(kl<acr/s"s<icr/k)</pr>, <hw>Clas"sic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>classicus</ets> relating to the classes of the Roman people, and especially to the frist class; hence, of the first rank, superior, from <ets>classis</ets> class: cf. F. <ets>classique</ets>. See <er>Class</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or relating to the first class or rank, especially in literature or art.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give, as thy last memorial to the age,<br/
One <qex>classic</qex> drama, and reform the stage.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Mr. Greaves may justly be reckoned a <qex>classical</qex> author on this subject [Roman weights and coins].</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the ancient Greeks and Romans, esp. to Greek or Roman authors of the highest rank, or of the period when their best literature was produced; of or pertaining to places inhabited by the ancient Greeks and Romans, or rendered famous by their deeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though throned midst Latium's <qex>classic</qex> plains.</q> <rj><qau>Mrs. Hemans.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The epithet <qex>classical</qex>, as applied to ancient authors, is determined less by the purity of their style than by the period at which they wrote.</q> <rj><qau>Brande & C.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Atterbury] directed the <qex>classical</qex> studies of the undergraduates of his college.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Conforming to the best authority in literature and art; chaste; pure; refined; <as>as, a <ex>classical</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Classical</qex>, provincial, and national synods.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Classicals orders</b></col>. <fld>(Arch.)</fld> <cd>See under <er>Order</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classic</ent><br/
<hw>Clas"sic</hw>, <pos>n.</pos> <sn>1.</sn> <def>A work of acknowledged excellence and authority, or its author; -- originally used of Greek and Latin works or authors, but now applied to authors and works of a like character in any language.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In is once raised him to the rank of a legitimate English <qex>classic</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One learned in the literature of Greece and Rome, or a student of classical literature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classicalism</ent><br/
<hw>Clas"sic*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A classical idiom, style, or expression; a classicism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Adherence to what are supposed or assumed to be the classical canons of art.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classicalist</ent><br/
<hw>Clas"sic*al*ist</hw>, <pos>n.</pos> <def>One who adheres to what he thinks the classical canons of art.</def>  <rj><au>Ruskin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classicalness</ent><br/
<ent>Classicality</ent><br/
<mhw>{ <hw>Clas`si*cal"i*ty</hw> <pr>(?)</pr>, <hw>Clas"sic*al*ness</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality of being classical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classically</ent><br/
<hw>Clas"sic*al*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a classical manner; according to the manner of classical authors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the manner of classes; according to a regular order of classes or sets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classicism</ent><br/
<hw>Clas"si*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A classic idiom or expression; a classicalism.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classicist</ent><br/
<hw>Clas"si*cist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One learned in the classics; an advocate for the classics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>classicistic</ent><br/
<hw>classicistic</hw> <pos>adj.</pos> <def>of or pertaining to classicism; <as>as, <ex>classicistic</ex> tradition</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>classicize</ent><br/
<hw>classicize</hw> <pos>v. t.</pos> <def>to make classic or classical.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>classics</ent><br/
<hw>classics</hw> <pos>n. pl.</pos> <def>the branch of learning concerned with study of the literary works of ancient Greece and Rome.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Classifiable</ent><br/
<hw>Clas"si*fi`a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being classified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classific</ent><br/
<hw>Clas*sif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterizing a class or classes; relating to classification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classification</ent><br/
<hw>Clas`si*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>classification</ets>.]</ety> <def>The act of forming into a class or classes; a distribution into groups, as classes, orders, families, etc., according to some common relations or affinities.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Artificial classification</b></col>. <fld>(Science)</fld> <cd>See under <er>Artifitial</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classificatory</ent><br/
<hw>Clas"si*fi*ca`to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to classification; admitting of classification.</def> <ldquo/A <xex>classificatory</xex> system.<rdquo/  <rj><au>Earle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>classified</ent><br/
<hw>classified</hw> <pos>adj.</pos> <sn>1.</sn> <def>arranged into classes or categories; <as>as, <ant>unclassified</ant></as>.</def><br/
<syn><b>Syn. --</b> categorized.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn>  <def>assigned to a class of documents withheld from general circulation; -- of information or documents.  Opposite of <ant>unclassified</ant>.</def> <note>[Narrower terms: <stype>eyes-only</stype>; <stype>confidential</stype>; <stype>restricted</stype>; <stype>secret</stype>; <stype>sensitive</stype>; <stype>top-secret</stype>]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Classifier</ent><br/
<hw>Clas"si*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who classifies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classify</ent><br/
<hw>Clas"si*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Classified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Classifying</conjf>.]</vmorph> <ety>[L. <ets>classis</ets> class + <ets>-fy</ets>.]</ety> <def>To distribute into classes; to arrange according to a system; to arrange in sets according to some method founded on common properties or characters.</def></p>

<p><syn><b>Syn.</b> -- To arrange; distribute; rank.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classis</ent><br/
||<hw>Clas"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Classes</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Class</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A class or order; sort; kind.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His opinion of that <qex>classis</qex> of men.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>An ecclesiastical body or judicatory in certain churches, as the Reformed Dutch. It is intermediate between the consistory and the synod, and corresponds to the presbytery in the Presbyterian church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classman</ent><br/
<hw>Class"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Classmen</plw><pr>(#)</pr>.</plu> <sn>1.</sn> <def>A member of a class; a classmate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A candidate for graduation in arts who is placed in an honor class, as opposed to a passman, who is not classified.</def> <mark>[Oxford, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Classmate</ent><br/
<hw>Class"mate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is in the same class with another, as at school or college.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>classy</ent><br/
<hw>classy</hw> <pos>adj.</pos> <sn>1.</sn> <def>having elegance or taste or refinement in manners or dress.  Opposite of <ant>styleless</ant>.</def> <mark>[informal]</mark> <br/
<syn><b>Syn. --</b> posh, swish, stylish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>exhibiting refinement and high character.   Opposite of <ant>low-class</ant></def> <mark>[informal]</mark> <br/
<syn><b>Syn. --</b> high-class.</syn>
[<source>PJC</source>]</p>

<p><ent>Clastic</ent><br/
<hw>Clas"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ br<?/, fr. <?/ to break.]</ety> <sn>1.</sn> <def>Pertaining to what may be taken apart; <as>as, <ex>clastic</ex> anatomy (of models)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Fragmental; made up of brok<?/ fragments; <as>as, sandstone is a <ex>clastic</ex> rock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clatch</ent><br/
<hw>Clatch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Scot. <ets>clatch</ets> a slap, the noise caused by the collision of soft bodies; prob. of imitative origin.]</ety> <fld>(Scot. & Dial. Eng.)</fld> <sn>1.</sn>  <def>A soft or sloppy lump or mass; <as>as, to throw a <ex>clatch</ex> of mud</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Anything put together or made in a careless or slipshod way; hence, a sluttish or slipshod woman.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clatch</ent><br/
<hw>Clatch</hw>, <pos>v. t. & i.</pos> <def>To daub or smear, as with lime; to make or finish in a slipshod way.</def> <mark>[Scot.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clathraceae</ent><br/
<hw>Clathraceae</hw> <pos>n.</pos> <def>a antural family of fleshy stinkhornlike fungi.</def><br/
<syn><b>Syn. --</b> family <fam>Clathraceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clathrate</ent><br/
<hw>Clath"rate</hw> <pr>(kl<acr/th"r<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>clathri</ets> lattice, Gr. <grk>klh,qra</grk>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Shaped like a lattice; cancellate.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having the surface marked with raised lines resembling a lattice, as many shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clathrus</ent><br/
<hw>Clathrus</hw> <pos>n.</pos> <def>the type genus of the Clathraceae.</def><br/
<syn><b>Syn. --</b> genus <gen>Clathrus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clatter</ent><br/
<hw>Clat"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clattered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clattering</conjf>.]</vmorph> <ety>[AS. <ets>cla<?/rung</ets> a rattle, akin to D. <ets>klateren</ets> to rattle. Cf. <er>Clack</er>.]</ety> <sn>1.</sn> <def>To make a rattling sound by striking hard bodies together; to make a succession of abrupt, rattling sounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clattering</qex> loud with iron clank.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To talk fast and noisily; to rattle with the tongue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I see thou dost but <qex>clatter</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clatter</ent><br/
<hw>Clat"ter</hw>, <pos>v. t.</pos> <def>To make a rattling noise with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>clatter</qex> still your brazen kettle.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clatter</ent><br/
<hw>Clat"ter</hw>, <pos>n.</pos> <sn>1.</sn> <def>A rattling noise, esp. that made by the collision of hard bodies; also, any loud, abrupt sound; a repetition of abrupt sounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The goose let fall a golden egg<br/
With cackle and with <qex>clatter</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Commotion; disturbance.</def> <ldquo/Those mighty feats which made such a <xex>clatter</xex> in story.<rdquo/<-- sic. = history?-->  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 263 --></p>

<p><sn>3.</sn> <def>Rapid, noisy talk; babble; chatter.</def> <ldquo/Hold still thy <xex>clatter</xex>.<rdquo/  <rj><au>Towneley Myst. (15 th Cent. ).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Throw by your <qex>clatter</qex><br/
And handle the matter.</q> <rj><qau>B. Jonson</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clatterer</ent><br/
<hw>Clat"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who clatters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clatteringly</ent><br/
<hw>Clat"ter*ing*ly</hw>, <pos>adv.</pos> <def>With clattering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claude Lorraine glass</ent><br/
<hw>Claude" Lor*raine" glass`</hw> <pr>(?)</pr>. <ety>[Its name is supposed to be derived from the similarity of the effects it gives to those of a picture by <ets>Claude Lorrain</ets> (often written <ets>Lorraine</ets>).]</ety> <def>A slightly convex mirror, commonly of black glass, used as a toy for viewing the reflected landscape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claudent</ent><br/
<hw>Clau"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>claudens</ets>, p. pr. of <ets>claudere</ets> to shut.]</ety> <def>Shutting; confining; drawing together; <as>as, a <ex>claudent</ex> muscle</as>.</def> <mark>[R.]</mark>  <rj><au>Jonson</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claudicant</ent><br/
<hw>Clau"di*cant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>claudicans</ets>, p. pr. of <ets>claudicare</ets> to limp, fr. <ets>claudus</ets> lame.]</ety> <def>Limping.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claudication</ent><br/
<hw>Clau`di*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>claudicatio</ets>.]</ety> <def>A halting or limping.</def> <mark>[R.]</mark>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clause</ent><br/
<hw>Clause</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clause</ets>, LL. <ets>clausa</ets>, equiv. to L. <ets>clausula</ets> clause, prop., close of <?/ rhetorical period, close, fr. <ets>claudere</ets> to shut, to end. See <er>Close</er>.]</ety> <sn>1.</sn> <def>A separate portion of a written paper, paragraph, or sentence; an article, stipulation, or proviso, in a legal document.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The usual attestation <qex>clause</qex> to a will.</q> <rj><qau>Bouvier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>A subordinate portion or a subdivision of a sentence containing a subject and its predicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clause</ent><br/
<hw>Clause</hw>, <pos>n.</pos> <mark>[Obs.]</mark> <def>See <cref>Letters clause</cref> or <cref>Letters close</cref>, under <er>Letter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claustral</ent><br/
<hw>Claus"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. LL. <ets>claustralis</ets>, fr. L. <ets>claustrum</ets>. See <er>Cloister</er>.]</ety> <def>Cloistral.</def>  <rj><au>Ayliffe</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claustrum</ent><br/
||<hw>Claus"trum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Claustra</plw>.</plu> <ety>[L., a bolt or bar.]</ety> <fld>(Anat.)</fld> <def>A thin lamina of gray matter in each cerebral hemisphere of the brain of man.</def> -- <wordforms><wf>Claus"tral</wf>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clausular</ent><br/
<hw>Clau"su*lar</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[From L. <ets>clausula</ets>. See <er>Clause</er>, <pos>n.</pos>]</ety> <def>Consisting of, or having, clauses.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clausure</ent><br/
<hw>Clau"sure</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>clausura</ets>. See <er>Closure</er>.]</ety> <def>The act of shutting up or confining; confinement.</def> <mark>[R.]</mark>  <rj><au>Geddes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavated</ent><br/
<ent>Clavate</ent><br/
<mhw>{ <hw>Cla"vate</hw> <pr>(?)</pr>, <hw>Cla"va*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>clava</ets> club.]</ety> <fld>(Bot. & Zool.)</fld> <def>Club-shaped; having the form of a club; growing gradually thicker toward the top. <note>[See <xex>Illust.</xex> of <er>Antennae</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clave</ent><br/
<hw>Clave</hw> <pr>(kl<amac/v)</pr>, <def><pos>imp.</pos> of <er>Cleave</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavecin</ent><br/
<hw>Clav"e*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>The harpsichord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavel</ent><br/
<hw>Cla"vel</hw> <pr>(kl<amac/"v<ecr/l)</pr>, <pos>n.</pos> <def>See <er>Clevis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavellate</ent><br/
<hw>Clav"el*late</hw> <pr>(kl<acr/v"<ecr/l*l<asl/t)</pr>, <pos>a.</pos> <def>See <er>Clavate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavellated</ent><br/
<hw>Clav"el*la`ted</hw> <pr>(kl<acr/v"<ecr/l*l<amac/"t<ecr/d)</pr>, <pos>a.</pos> <ety>[Cf. LL. cineres <ets>clavelatti</ets> ashes of burnt lees or dregs of wine, F. <ets>clavel</ets> an inferior sort of soda, E. <ets>clavate</ets>.]</ety> <fld>(Old Chem.)</fld> <def>Said of potash, probably in reference to its having been obtained from billets of wood by burning.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claver</ent><br/
<hw>Clav"er</hw> <pr>(?)</pr>, <pos>n.</pos> <mark>[Obs.]</mark> <def>See <er>Clover</er>.</def> <au>Holland.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claver</ent><br/
<hw>Clav"er</hw>, <pos>n.</pos> <def>Frivolous or nonsensical talk; prattle; chattering.</def> <mark>[Scot. & North of Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Emmy found herself entirely at a loss in the midst of their <qex>clavers</qex>.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claviceps</ent><br/
<hw>Claviceps</hw> <pos>n.</pos> <def>a genus of fungi parasitic upon the ovaries of various grasses.</def><br/
<syn><b>Syn. --</b> genus <gen>Claviceps</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clavichord</ent><br/
<hw>Clav"i*chord</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clavicorde</ets>, fr. L. <ets>clavis</ets> key + <ets>chorda</ets> string.]</ety> <fld>(Mus.)</fld> <def>A keyed stringed instrument, now superseded by the pianoforte. See <er>Clarichord</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavicle</ent><br/
<hw>Clav"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clavicule</ets>, fr. L. <ets>clavicula</ets> a little key, tendril, dim. of <ets>clavis</ets> key, akin to <ets>claudere</ets> to shut. See <er>Close</er>, and cf. <er>Clef</er>.]</ety> <fld>(Anat.)</fld> <def>The collar bone, which is joined at one end to the scapula, or shoulder blade, and at the other to the sternum, or breastbone. In man each clavicle is shaped like the letter <?/, and is situated just above the first rib on either side of the neck. In birds the two clavicles are united ventrally, forming the merrythought, or wishbone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavicorn</ent><br/
<hw>Clav"i*corn</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>clavicorne</ets>.]</ety> <fld>(Zool.)</fld> <def>Having club-shaped antenn<ae/. See <er>Antenn<ae/</er></def> -- <def2><pos>n.</pos> <def>One of the Clavicornes.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavicornes</ent><br/
||<hw>Clav`i*cor"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.; Fr. L. <ets>clava</ets> club + <ets>cornu</ets> horn.]</ety> <fld>(Zool.)</fld> <def>A group of beetles having club-shaped antenn<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavicular</ent><br/
<hw>Cla*vic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>claviculaire</ets>. See <er>Clavicle</er>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the clavicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavier</ent><br/
<hw>Cla"vi*er</hw> <pr>(? F. ?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>clavis</ets> key.]</ety> <fld>(Mus.)</fld> <def>The keyboard of an organ, pianoforte, or harmonium.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Clavier</xex> (<?/) is the German name for a pianoforte.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claviform</ent><br/
<hw>Clav"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clava</ets> club + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Club-shaped; clavate.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claviger</ent><br/
||<hw>Clav"i*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>clavis</ets> key + <ets>gerere</ets> to carry.]</ety> <def>One who carries the keys of any place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claviger</ent><br/
||<hw>Clav"i*ger</hw>, <pos>n.</pos> <ety>[L., fr. <ets>clava</ets> club + <ets>gerere</ets> to carry.]</ety> <def>One who carries a club; a club bearer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavigerous</ent><br/
<hw>Cla*vig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bearing a club or a key.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavis</ent><br/
||<hw>Cla"vis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Claves</plw> <pr>(#)</pr>, E. <plw>Clavises</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <def>A key; a glossary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavus</ent><br/
||<hw>Cla"vus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a nail.]</ety> <def>A callous growth, esp. one the foot; a corn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clavy</ent><br/
<hw>Cla"vy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Clavies</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>claveau</ets> centerpiece of an arch.]</ety> <fld>(Arch.)</fld> <def>A mantelpiece.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claw</ent><br/
<hw>Claw</hw> <pr>(kl<add/)</pr>, <pos>n.</pos> <ety>[AS. <ets>clawu</ets>, <ets>cl<amac/</ets>, <ets>cle<oacute/</ets>; akin to D. <ets>klaauw</ets>, G. <ets>klaue</ets>, Icel. <ets>kl<omac/</ets>, Sw. & Dan. <ets>klo</ets>, and perh. to E. <ets>clew</ets>.]</ety> <sn>1.</sn> <def>A sharp, hooked nail, as of a beast or bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The whole foot of an animal armed with hooked nails; the pinchers of a lobster, crab, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything resembling the claw of an animal, as the curved and forked end of a hammer for drawing nails.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>A slender appendage or process, formed like a claw, as the base of petals of the pink.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Claw hammer</b></col>, <cd>a hammer with one end of the metallic head cleft for use in extracting nails, etc.</cd> -- <col><b>Claw hammer coat</b></col>, <cd>a dress coat of the swallowtail pattern.</cd> <mark>[Slang]</mark> -- <col><b>Claw sickness</b></col>, <cd>foot rot, a disease affecting sheep.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claw</ent><br/
<hw>Claw</hw> <pr>(kl<add/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clawed</conjf> <pr>(kl<add/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clawing</conjf>.]</vmorph> <ety>[AS. <ets>clawan</ets>. See <er>Claw</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To pull, tear, or scratch with, or as with, claws or nails.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To relieve from some uneasy sensation, as by scratching; to tickle; hence, to flatter; to court.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Rich men they <qex>claw</qex>, soothe up, and flatter; the poor they contemn and despise.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To rail at; to scold.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In the aforesaid preamble, the king fairly claweth the great monasteries, wherein, saith he, religion, thanks be to God, is right well kept and observed; though he <qex>claweth</qex> them soon after in another acceptation.</q> <rj><qau>T. Fuller</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Claw me, claw thee</b></col>, <cd>stand by me and I will stand by you; -- an old proverb.</cd>  <rj><au>Tyndale.</au></rj> -- <col><b>To claw away</b></col>, <cd>to scold or revile.</cd> <ldquo/The jade Fortune is to be <xex>clawed away</xex> for it, if you should lose it.<rdquo/  <rj><au>L'Estrange.</au></rj> -- <col><b>To claw <plain>(one)</plain> on the back</b></col>, <cd>to tickle; to express approbation.</cd> <mark>(Obs.)</mark>  <rj><au>Chaucer.</au></rj> -- <col><b>To claw <plain>(one)</plain> on the gall</b></col>, <cd>to find fault with; to vex.</cd> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claw</ent><br/
<hw>Claw</hw>, <pos>v. i.</pos> <def>To scrape, scratch, or dig with a claw, or with the hand as a claw.</def> <ldquo/<xex>Clawing</xex> [in ash barrels] for bits of coal.<rdquo/  <rj><au>W. D. Howells.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To claw off</b></col> <fld>(Naut.)</fld>, <cd>to turn to windward and beat, to prevent falling on a lee shore.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clawback</ent><br/
<hw>Claw"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A flatterer or sycophant.</def> <mark>[Obs.]</mark> <ldquo/Take heed of these <xex>clawbacks</xex>.<rdquo/  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clawback</ent><br/
<hw>Claw"back`</hw>, <pos>a.</pos> <def>Flattering; sycophantic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a <qex>clawback</qex> parasite.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clawback</ent><br/
<hw>Claw"back`</hw>, <pos>v. t.</pos> <def>To flatter.</def> <mark>[Obs.]</mark>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clawed</ent><br/
<hw>Clawed</hw> <pr>(kl<add/d)</pr>, <pos>a.</pos> <def>Furnished with claws.</def>  <rj><au>N. Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clawless</ent><br/
<hw>Claw"less</hw>, <pos>a.</pos> <def>Destitute of claws.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clawlike</ent><br/
<hw>clawlike</hw> <pos>adj.</pos> <def>having a base shaped like a claw; -- of flower petals.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>claxon</ent><br/
<hw>claxon</hw> <pos>n.</pos> <def>a kind of loud horn formerly used on motor vehicles.</def><br/
<syn><b>Syn. --</b> klaxon.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>claxon</ent><br/
<hw>claxon</hw> <pos>v.</pos> <def>to sound loudly; -- of car horns.</def><br/
<syn><b>Syn. --</b> honk, blare, beep, toot, klaxon.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to use the horn of a car.</def><br/
<syn><b>Syn. --</b> honk.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clay</ent><br/
<hw>Clay</hw> <pr>(kl<amac/)</pr>, <pos>n.</pos> <ety>[AS. <ets>cl<aemac/g</ets>; akin to LG. <ets>klei</ets>, D. <ets>klei</ets>, and perh. to AS. <ets>cl<amac/m</ets> clay, L. <ets>glus</ets>, <ets>gluten</ets> glue, Gr. <grk>gloio`s</grk> glutinous substance, E. <ets>glue</ets>. Cf. <er>Clog</er>.]</ety> <sn>1.</sn> <def>A soft earth, which is plastic, or may be molded with the hands, consisting of hydrous silicate of aluminium. It is the result of the wearing down and decomposition, in part, of rocks containing aluminous minerals, as granite. Lime, magnesia, oxide of iron, and other ingredients, are often present as impurities.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Poetry & Script.)</fld> <def>Earth in general, as representing the elementary particles of the human body; hence, the human body as formed from such particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I also am formed out of the <qex>clay</qex>.</q> <rj><qau>Job xxxiii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The earth is covered thick with other <qex>clay</qex>,<br/
Which her own <qex>clay</qex> shall cover.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bowlder clay</b></col>. <cd>See under <er>Bowlder</er>.</cd> -- <col><b>Brick clay</b></col>, <cd>the common clay, containing some iron, and therefore turning red when burned.</cd> -- <col><b>Clay cold</b></col>, <cd>cold as clay or earth; lifeless; inanimate.</cd> -- <col><b>Clay ironstone</b></col>, <cd>an ore of iron consisting of the oxide or carbonate of iron mixed with clay or sand.</cd> -- <col><b>Clay marl</b></col>, <cd>a whitish, smooth, chalky clay.</cd> -- <col><b>Clay mill</b></col>, <cd>a mill for mixing and tempering clay; a pug mill.</cd> -- <col><b>Clay pit</b></col>, <cd>a pit where clay is dug.</cd> -- <col><b>Clay slate</b></col> <fld>(Min.)</fld>, <cd>argillaceous schist; argillite.</cd> -- <col><b>Fatty clays</b></col>, <cd>clays having a greasy feel; they are chemical compounds of water, silica, and aluminia, as <stype>halloysite</stype>, <stype>bole</stype>, etc.</cd> -- <col><b>Fire clay</b></col> , <cd>a variety of clay, entirely free from lime, iron, or an alkali, and therefore infusible, and used for fire brick.</cd> -- <col><b>Porcelain clay</b></col>, <cd>a very pure variety, formed directly from the decomposition of feldspar, and often called <altname>kaolin</altname>.</cd> -- <col><b>Potter's clay</b></col>, <cd>a tolerably pure kind, free from iron.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clay</ent><br/
<hw>Clay</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clayed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Claying</conjf>.]</vmorph> <sn>1.</sn> <def>To cover or manure with clay.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To clarify by filtering through clay, as sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clay-brained</ent><br/
<hw>Clay"-brained`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stupid.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clayes</ent><br/
<hw>Clayes</hw> <pr>(kl<amac/z)</pr>, <pos>n. pl.</pos> <ety>[F. <ets>claie</ets> hurdle.]</ety> <fld>(Fort.)</fld> <def>Wattles, or hurdles, made with stakes interwoven with osiers, to cover lodgments.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clayey</ent><br/
<hw>Clay"ey</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of clay; abounding with clay; partaking of clay; like clay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clayish</ent><br/
<hw>Clay"ish</hw>, <pos>a.</pos> <def>Partaking of the nature of clay, or containing particles of it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claymore</ent><br/
<hw>Clay"more`</hw> <pr>(kl<amac/"m<omac/r`)</pr>, <pos>n.</pos> <ety>[Gael. <ets>claidheamhmor</ets> a broadsword; Gael. <ets>claidheamh</ets> sword + <ets>mor</ets> great, large. Cf. <er>Glaymore</er>.]</ety> <def>A large two-handed sword used formerly by the Scottish Highlanders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Claymore mine</ent><br/
<hw>Clay"more mine`</hw> <pr>(kl<amac/"m<omac/r m<imac/n`)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>an antipersonnel land mine designed to produce a high-velocity spray of steel fragments in a relatively narrow fan-shaped cone, directed toward the enemy.  They are usually detonated by an electrical remote control.</def><br/
[<source>PJC</source>]</p>

<p><ent>Claytonia</ent><br/
||<hw>Clay*to"ni*a</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[Named after <person>Dr. John <ets>Clayton</ets></person>, an American botanist.]</ety> <fld>(Bot.)</fld> <def>An American genus of perennial herbs with delicate blossoms; -- sometimes called <altname>spring beauty</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clayware</ent><br/
<hw>clay"ware`</hw> <pos>n.</pos> <def>Objects made from clay and baked in a kiln.</def><br/
<syn><b>Syn. --</b> pottery.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cleading</ent><br/
<hw>Clead"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Scot., clothing. See <er>Cloth</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A jacket or outer covering of wood, etc., to prevent radiation of heat, as from the boiler, cylinder. etc., of a steam engine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The planking or boarding of a shaft, cofferdam, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clean</ent><br/
<hw>Clean</hw> <pr>(kl<emac/n)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Cleaner</adjf> <pr>(kl<emac/n"<etil/r)</pr>; <pos>superl.</pos> <adjf>Cleanest</adjf>.]</amorph> <ety>[OE. <ets>clene</ets>, AS. <ets>cl<aemac/ne</ets>; akin to OHG. <ets>chleini</ets> pure, neat, graceful, small, G. <ets>klein</ets> small, and perh. to W. <ets>glan</ets> clean, pure, bright; all perh. from a primitive, meaning <itrans>bright</itrans>, <itrans>shining</itrans>. Cf. <er>Glair</er>.]</ety> <sn>1.</sn> <def>Free from dirt or filth; <as>as, <ex>clean</ex> clothes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Free from that which is useless or injurious; without defects; <as>as, <ex>clean</ex> land; <ex>clean</ex> timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Free from awkwardness; not bungling; adroit; dexterous; <as>as, a <ex>clean</ex> trick; a <ex>clean</ex> leap over a fence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Free from errors and vulgarisms; <as>as, a <ex>clean</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Free from restraint or neglect; complete; entire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When ye reap the harvest of your land, thou shalt not make <qex>clean</qex> riddance of corners of thy field.</q> <rj><qau>Lev. xxiii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Free from moral defilement; sinless; pure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Create in me a <qex>clean</qex> heart, O God.</q> <rj><qau>Ps. li. 10</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That I am whole, and <qex>clean</qex>, and meet for Heaven</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Script.)</fld> <def>Free from ceremonial defilement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Free from that which is corrupting to the morals; pure in tone; healthy.</def> <ldquo/Lothair is <xex>clean</xex>.<rdquo/  <rj><au>F. Harrison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Well-proportioned; shapely; <as>as, <ex>clean limbs</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A clean bill of health</b></col>, <cd>a certificate from the proper authority that a ship is free from infection.</cd> -- <col><b>Clean breach</b></col>. <cd>See under <er>Breach</er>, <pos>n.</pos>, 4.</cd> -- <col><b>To make a clean breast</b></col>. <cd>See under <er>Breast</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clean</ent><br/
<hw>Clean</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Without limitation or remainder; quite; perfectly; wholly; entirely.</def> <ldquo/Domestic broils <xex>clean</xex> overblown.<rdquo/  <rj><au>Shak.</au></rj></p>

<p><ldquo/<xex>Clean</xex> contrary.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the people were passed <qex>clean</qex> over Jordan.</q> <rj><qau>Josh. iii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without miscarriage; not bunglingly; dexterously.</def> <mark>[Obs.]</mark> <ldquo/Pope came off <xex>clean</xex> with Homer.<rdquo/  <rj><au>Henley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clean</ent><br/
<hw>Clean</hw> <pr>(kl<emac/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cleaned</conjf> <pr>(kl<emac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cleaning</conjf>.]</vmorph> <ety>[See <er>Clean</er>, <pos>a.</pos>, and cf. <er>Cleanse</er>.]</ety> <def>To render clean; to free from whatever is foul, offensive, or extraneous; to purify; to cleanse.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To clean out</b></col>, <cd>to exhaust; to empty; to get away from (one) all his money.</cd> <mark>[Colloq.]</mark>  <rj><au>De Quincey.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clean-cut</ent><br/
<hw>Clean"-cut`</hw> <pr>(kl<emac/n"k<ucr/t)</pr>, <pos>a.</pos> <def>See <er>Clear-cut</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleaner</ent><br/
<hw>Clean"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, cleans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cleaners</ent><br/
<hw>cleaners</hw> <pos>n.</pos> <def>a shop where dry cleaning is done.</def><br/
<syn><b>Syn. --</b> dry cleaners, laundry, laundry shop.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cleaning</ent><br/
<hw>Clean"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of making clean.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The afterbirth of cows, ewes, etc.</def>  <rj><au>Gardner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleanlily</ent><br/
<hw>Clean"li*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a cleanly manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clean-limbed</ent><br/
<hw>Clean"-limbed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>With well-proportioned, unblemished limbs; <as>as, a <ex>clean-limbed</ex> young fellow</as>.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleanliness</ent><br/
<hw>Clean"li*ness</hw> <pr>(kl<ecr/n"l<icr/*n<ecr/s)</pr>, <pos>n.</pos> <ety>[From <er>Cleanly</er>.]</ety> <def>State of being cleanly; neatness of person or dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cleanliness</qex> from head to heel.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clean-living</ent><br/
<hw>clean-living</hw> <pos>adj.</pos>  <def>living in a manner above moral reproach; especially, abstaining from unlawful sexual intercourse and excessive drunkenness.</def><br/
<syn><b>Syn. --</b> chaste.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cleanly</ent><br/
<hw>Clean"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Cleanlier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Cleanliest</adjf>.]</amorph> <ety>[From <er>Clean</er>.]</ety> <sn>1.</sn> <def>Habitually clean; pure; innocent.</def> <ldquo/<xex>Cleanly</xex> joys.<rdquo/  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some plain but <qex>cleanly</qex> country maid.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Displays her <qex>cleanly</qex> platter on the board.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cleansing; fitted to remove moisture; dirt, etc.</def> <mark>[Obs.]</mark> <ldquo/With <xex>cleanly</xex> powder dry their hair.<rdquo/  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Adroit; skillful; dexterous; artful.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Through his fine handling and his <qex>cleanly</qex> play.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleanly</ent><br/
<hw>Clean"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a clean manner; neatly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was very <qex>cleanly</qex> dressed.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Innocently; without stain.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Adroitly; dexterously.</def>  <rj><au>Middleton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleanness</ent><br/
<hw>Clean"ness</hw>, <pos>n.</pos> <ety>[AS. <ets>cl<?/nnes</ets>. See <er>Clean</er>.]</ety> <sn>1.</sn> <def>The state or quality of being clean.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Purity of life or language; freedom from licentious courses.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleansable</ent><br/
<hw>Cleans"a*ble</hw> <pr>(kl<ecr/nz"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being cleansed.</def>  <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleanse</ent><br/
<hw>Cleanse</hw> <pr>(kl<ecr/nz)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cleansed</conjf> <pr>(kl<ecr/nzd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cleansing</conjf>.]</vmorph> <ety>[AS. <ets>cl<aemac/nsian</ets>, fr. <ets>cl<aemac/ne</ets> clean. See <er>Clean</er>.]</ety> <def>To render clean; to free from fith, pollution, infection, guilt, etc.; to clean.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If we walk in the light . . . the blood of Jesus Christ his son <qex>cleanseth</qex> us from all sin.</q> <rj><qau>1 John i. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Can'st thou not minister to a mind diseased,<br/
And with some sweet oblivious antidote<br/
<qex>Cleanse</qex> the suffed bosom of that perilous stuff<br/
Which weighs upon the heart?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cleansed</ent><br/
<hw>cleansed</hw> <pr>(kl<ecr/nzd)</pr>, <pos>adj.</pos>  <def>having been made clean.  Opposite of <ant>dirty</ant>.</def> <hascons>clean</hascons><br/
<syn><b>Syn. --</b> cleaned, scrubbed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cleanser</ent><br/
<hw>Cleans"er</hw> <pr>(kl<ecr/nz"<etil/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>cl<aemac/nsere</ets>.]</ety> <def>One who, or that which, cleanses; especially, a detergent or other preparation used for cleaning.</def>  <rj><au>Arbuthnot.</au></rj><br/
<syn><b>Syn. --</b> cleansing agent, cleaner.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>clean-shaven</ent><br/
<hw>clean-shaven</hw> <pos>adj.</pos> <def>having the surface shaved to smoothness.</def><br/
<syn><b>Syn. --</b> smooth-shaven, well-shaven.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clean-timbered</ent><br/
<hw>Clean"-tim`bered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Well-proportioned; symmetrical.</def> <mark>[Poetic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear</ent><br/
<hw>Clear</hw> <pr>(kl<emac/r)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Clearer</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Clearest</adjf>.]</amorph> <ety>[OE. <ets>cler</ets>, <ets>cleer</ets>, OF. <ets>cler</ets>, F. <ets>clair</ets>, fr.L. <ets>clarus</ets>, clear, bright, loud, distinct, renowned; perh. akin to L. <ets>clamare</ets> to call, E. <ets>claim</ets>. Cf. <er>Chanticleer</er>, <er>Clairvoyant</er>, <er>Claret</er>, <er>Clarify</er>.]</ety> <sn>1.</sn> <def>Free from opaqueness; transparent; bright; light; luminous; unclouded.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stream is so transparent, pure, and <qex>clear</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fair as the moon, <qex>clear</qex> as the sun.</q> <rj><qau>Canticles vi. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Free from ambiguity or indistinctness; lucid; perspicuous; plain; evident; manifest; indubitable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One truth is <qex>clear</qex>; whatever is, is right.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Able to perceive clearly; keen; acute; penetrating; discriminating; <as>as, a <ex>clear</ex> intellect; a <ex>clear</ex> head</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mother of science! now I feel thy power<br/
Within me <qex>clear</qex>, not only to discern<br/
Things in their causes, but to trace the ways<br/
Of highest agents.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not clouded with passion; serene; cheerful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With a countenance as <qex>clear</qex><br/
As friendship wears at feasts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Easily or distinctly heard; audible; canorous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hark! the numbers soft and <qex>clear</qex><br/
Gently steal upon the ear.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Without mixture; entirely pure; <as>as, <ex>clear</ex> sand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Without defect or blemish, such as freckles or knots; <as>as, a <ex>clear</ex> complexion; <ex>clear</ex> lumber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Free from guilt or stain; unblemished.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Statesman, yet friend to truth! in soul sincere,<br/
In action faithful, and in honor <qex>clear</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Without diminution; in full; net; <as>as, <ex>clear</ex> profit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I often wished that I had <qex>clear</qex>,<br/
For life, six hundred pounds a-year.</q> <rj><qau>Swift</qau></rj>.<br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Free from impediment or obstruction; unobstructed; <as>as, a <ex>clear</ex> view; to keep <ex>clear</ex> of debt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My companion . . . left the way <qex>clear</qex> for him.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Free from embarrassment; detention, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cruel corporal whispered in my ear,<br/
Five pounds, if rightly tipped, would set me <qex>clear</qex>.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clear breach</b></col>. <cd>See under <er>Breach</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Clear days</b></col> <fld>(Law.)</fld>, <cd>days reckoned from one day to another, excluding both the first and last day; <as>as, from Sunday to Sunday there are six <ex>clear days</ex></as>.</cd> -- <col><b>Clear stuff</b></col>, <cd>boards, planks, etc., free from knots.</cd></cs></p>

<p><syn><b>Syn.</b> -- Manifest; pure; unmixed; pellucid; transparent; luminous; obvious; visible; plain; evident; apparent; distinct; perspicuous. See <er>Manifest</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear</ent><br/
<hw>Clear</hw> <pr>(kl<emac/r)</pr>, <pos>n.</pos> <fld>(Carp.)</fld> <def>Full extent; distance between extreme limits; especially; the distance between the nearest surfaces of two bodies, or the space between walls; <as>as, a room ten feet square in the <ex>clear</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear</ent><br/
<hw>Clear</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a clear manner; plainly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>clear</qex> I understand<br/
What oft . . . thoughts have searched in vain.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without limitation; wholly; quite; entirely; <as>as, to cut a piece <ex>clear</ex> off</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear</ent><br/
<hw>Clear</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cleared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clearing</conjf>.]</vmorph> <sn>1.</sn> <def>To render bright, transparent, or undimmed; to free from clouds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He sweeps the skies and <qex>clears</qex> the cloudy north.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To free from impurities; to clarify; to cleanse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To free from obscurity or ambiguity; to relive of perplexity; to make perspicuous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many knotty points there are<br/
Which all discuss, but few can <qex>clear</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To render more quick or acute, as the understanding; to make perspicacious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our common prints would <qex>clear</qex> up their understandings.</q> <rj><qau>Addison</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To free from impediment or incumbrance, from defilement, or from anything injurious, useless, or offensive; <as>as, to <ex>clear</ex> land of trees or brushwood, or from stones; to <ex>clear</ex> the sight or the voice; to <ex>clear</ex> one's self from debt</as>; -- often used with <xex>of</xex>, <xex>off</xex>, <xex>away</xex>, or <xex>out</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clear</qex> your mind of cant.</q> <rj><qau>Dr. Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A statue lies hid in a block of marble; and the art of the statuary only <qex>clears</qex> away the superfluous matter.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To free from the imputation of guilt; to justify, vindicate, or acquit; -- often used with <xex>from</xex> before the thing imputed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I . . . am sure he will <qex>clear</qex> me from partiality.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How! wouldst thou <qex>clear</qex> rebellion?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To leap or pass by, or over, without touching or failure; <as>as, to <ex>clear</ex> a hedge; to <ex>clear</ex> a reef</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To gain without deduction; to net.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The profit which she <qex>cleared</qex> on the cargo.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To clear a ship at the customhouse</b></col>, <cd>to exhibit the documents required by law, give bonds, or perform other acts requisite, and procure a permission to sail, and such papers as the law requires.</cd> -- <mcol><col><b>To clear a ship for action</b></col>, <it>or</it> <col><b>To clear for action</b></col></mcol> <fld>(Naut.)</fld>, <cd>to remove incumbrances from the decks, and prepare for an engagement.</cd> -- <col><b>To clear the land</b></col> <fld>(Naut.)</fld>, <cd>to gain such a distance from shore as to have sea room, and be out of danger from the land.</cd> -- <col><b>To clear hawse</b></col> <fld>(Naut.)</fld>, <cd>to disentangle the cables when twisted.</cd> -- <col><b>To clear up</b></col>, <cd>to explain; to dispel, as doubts, cares or fears.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 264 --></p>

<p><ent>Clear</ent><br/
<hw>Clear</hw> <pr>(kl<emac/r)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To become free from clouds or fog; to become fair; -- of the weather; -- often followed by <ptcl>up</ptcl>, <ptcl>off</ptcl>, or <ptcl>away</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So foul a sky <qex>clears</qex> not without a storm.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Advise him to stay till the weather <qex>clears</qex> up.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To become free from turbidity; -- of solutions or suspensions of liquids; <as>as, the salt has not completely dissolved until the suspension <ex>clears up</ex>; when refrigerated, the juice may become cloudy, but when warmed to room temperature, it <ex>clears up</ex> again</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>To disengage one's self from incumbrances, distress, or entanglements; to become free.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He that <qex>clears</qex> at once will relapse; for finding himself out of straits, he will revert to his customs; but he that <qex>cleareth</qex> by degrees induceth a habit of frugality.</q> <rj><qau>Bacon.</qau></rj>
<-- ?? sic. why 2 forms of 3rd p. sing. in the same sentence? --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Banking)</fld> <def>To make exchanges of checks and bills, and settle balances, as is done in a clearing house.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To obtain a clearance; <as>as, the steamer <ex>cleared</ex> for Liverpool to-day</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To clear out</b></col>, <cd>to go or run away; to depart.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearage</ent><br/
<hw>Clear"age</hw> <pr>(kl<emac/r"<asl/j)</pr>, <pos>n.</pos> <def>The act of removing anything; clearance.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearance</ent><br/
<hw>Clear"ance</hw> <pr>(kl<emac/r"<ait/ns)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of clearing; <as>as, to make a thorough <ex>clearance</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A certificate that a ship or vessel has been cleared at the customhouse; permission to sail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every ship was subject to seizure for want of stamped <qex>clearances</qex>.</q> <rj><qau>Durke</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Clear or net profit.</def>  <rj><au>Trollope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <def>The distance by which one object clears another, as the distance between the piston and cylinder head at the end of a stroke in a steam engine, or the least distance between the point of a cogwheel tooth and the bottom of a space between teeth of a wheel with which it engages.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clearance space</b></col> <fld>(Steam engine)</fld>, <cd>the space inclosed in one end of the cylinder, between the valve or valves and the piston, at the beginning of a stroke; waste room. It includes the space caused by the piston's clearance and the space in ports, passageways, etc. Its volume is often expressed as a certain proportion of the volume swept by the piston in a single stroke.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearcole</ent><br/
<hw>Clear"cole`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>claire colle</ets> clear glue; <ets>clair</ets> clear (f. <ets>claire</ets>) + <ets>colle</ets> glue, Gr. <?/.]</ety> <def>A priming of size mixed with whiting or white lead, used in house painting, etc.; also, a size upon which gold leaf is applied in gilding.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clearcole</ent><br/
<hw>Clear"cole`</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clearcoled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clearcoling</conjf> <pr>(?)</pr>.]</vmorph> <def>To coat or paint with clearcole.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clear-cut</ent><br/
<hw>Clear"-cut`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a sharp, distinct outline, like that of a cameo.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She has . . . a cold and <qex>clear-cut</qex> face.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Concisely and distinctly expressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Distinct and unambiguous; unquestionable; <as>as, a <ex>clear-cut</ex> advantage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear-cut</ent><br/
<hw>Clear"-cut`</hw>, <pos>v. t.</pos> <def>To cut (a region of forest) clear of all trees.  It is a method used for efficiently logging a portion of forest, but often has undesirable effects on the environment.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clearedness</ent><br/
<hw>Clear"ed*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being cleared.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Imputed by his friends to the <qex>clearedness</qex>, by his foes to the searedness, of his conscience.</q> <rj><qau>T. Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearer</ent><br/
<hw>Clear"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, clears.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gold is a wonderful <qex>clearer</qex> of the understanding.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A tool of which the hemp for lines and twines, used by sailmakers, is finished.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clear-eyed</ent><br/
<hw>clear-eyed</hw> <pos>adj.</pos> <def>Acutely perceptive or discerning.</def><br/
<syn><b>Syn. --</b> clear-sighted; perceptive; percipient; perspicacious; clear-headed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clear-headed</ent><br/
<hw>Clear"-head`ed</hw> <pr>(kl<emac/r"h<ecr/d`<ecr/d)</pr>, <pos>a.</pos> <def>Having a clear understanding; quick of perception; intelligent.</def> <ldquo/He was laborious and <xex>clear-headed</xex>.<rdquo/  <rj><au>Macaulay.</au></rj></p>

<p>-- <wordforms><wf>Clear"-head`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearing</ent><br/
<hw>Clear"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of making clear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The better <qex>clearing</qex> of this point.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tract of land cleared of wood for cultivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A lonely <qex>clearing</qex> on the shores of Moxie Lake.</q> <rj><qau>J. Burroughs.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A method adopted by banks and bankers for making an exchange of checks held by each against the others, and settling differences of accounts.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, a similar method has been adopted by railroads for adjusting their accounts with each other.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The gross amount of the balances adjusted in the clearing house.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clearing house</b></col>, <cd>the establishment where the business of clearing is carried on. See <er>above</er>, <er>3</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearly</ent><br/
<hw>Clear"ly</hw>, <pos>adv.</pos> <def>In a clear manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearness</ent><br/
<hw>Clear"ness</hw>, <pos>n.</pos> <def>The quality or state of being clear.</def></p>

<p><syn><b>Syn.</b> -- <er>Clearness</er>, <er>Perspicuity</er>.</syn> <usage> <xex>Clearness</xex> has reference to our ideas, and springs from a distinct conception of the subject under consideration. <xex>Perspicuity</xex> has reference to the mode of expressing our ideas and belongs essentially to style. Hence we speak of a writer as having <xex>clear</xex> ideas, a <xex>clear</xex> arrangement, and <xex>perspicuous</xex> phraseology. We do at times speak of a person's having great <xex>clearness</xex> of style; but in such cases we are usually thinking of the clearness of his ideas as manifested in language. <ldquo/Whenever men think <xex>clearly</xex>, and are thoroughly interested, they express themselves with <xex>perspicuity</xex> and force.<rdquo/</usage>  <rj><au>Robertson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear-seeing</ent><br/
<hw>Clear"-see`ing</hw> <pr>(kl<emac/r"s<emac/`<icr/ng)</pr>, <pos>a.</pos> <def>Having a clear physical or mental vision; having a clear understanding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear-shining</ent><br/
<hw>Clear"-shin`ing</hw> <pr>(kl<emac/r"sh<imac/n`<icr/ng)</pr>, <pos>a.</pos> <def>Shining brightly.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clear-sighted</ent><br/
<hw>Clear"-sight`ed</hw> <pr>(kl<emac/r"s<imac/t`<ecr/d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Seeing with clearness; acutely perceptive, discerning; <as>as, <ex>clear-sighted</ex> reason</as>.</def><br/
<syn><b>Syn. --</b> clear-eyed, perceptive, percipient, perspicacious, clear-headed.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>having sharp clear vision.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clear-sightedness</ent><br/
<hw>Clear"-sight`ed*ness</hw>, <pos>n.</pos> <def>Acute discernment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearstarch</ent><br/
<hw>Clear"starch`</hw> <pr>(kl<emac/r"st<aum/rcht`)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clearstarched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clearstraching</conjf>.]</vmorph> <def>To stiffen with starch, and then make clear by clapping with the hands; <as>as, to <ex>clearstarch</ex> muslin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clearstarcher</ent><br/
<hw>Clear"starch`er</hw> <pr>(kl<emac/r"st<aum/rch`<etil/r)</pr>, <pos>n.</pos> <def>One who clearstarches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerestory</ent><br/
<ent>Clearstory</ent><br/
<mhw>{ <hw>Clear"sto`ry</hw> <pr>(kl<emac/r"st<omac/`r<ycr/)</pr>, <hw>Clere"sto`ry</hw>, }</mhw> <pos>n.</pos> <fld>(Arch.)</fld> <def>The upper story of the nave of a church, containing windows, and rising above the aisle roofs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clear-thinking</ent><br/
<hw>clear-thinking</hw> <pos>adj.</pos> <def>able to think clearly and accurately.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clearwing</ent><br/
<hw>Clear"wing`</hw> <pr>(kl<emac/r"w<icr/ng`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A lepidopterous insect with partially transparent wings, of the family <fam><AE/geriad<ae/</fam>, of which the currant and peach-tree borers are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleat</ent><br/
<hw>Cleat</hw> <pr>(kl<emac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>clete</ets> wedge; cf. D. <ets>kloot</ets> ball, Ger. <ets>kloss</ets>, <ets>klotz</ets>, lump. clod, MHG. <ets>kl<omac/z</ets> lump, ball, wedge, OHG. <ets>chl<omac/z</ets> ball, round mass.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Carp.)</fld> <def>A strip of wood or iron fastened on transversely to something in order to give strength, prevent warping, hold position, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A device made of wood or metal, having two arms, around which turns may be taken with a line or rope so as to hold securely and yet be readily released. It is bolted by the middle to a deck or mast, etc., or it may be lashed to a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleat</ent><br/
<hw>Cleat</hw>, <pos>v. t.</pos> <def>To strengthen with a cleat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleavable</ent><br/
<hw>Cleav"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of cleaving or being divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleavage</ent><br/
<hw>Cleav"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of cleaving or splitting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Crystallog.)</fld> <def>The quality possessed by many crystallized substances of splitting readily in one or more definite directions, in which the cohesive attraction is a minimum, affording more or less smooth surfaces; the direction of the dividing plane; a fragment obtained by cleaving, as of a diamond. See <er>Parting</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>Division into lamin<ae/, like slate, with the lamination not necessarily parallel to the plane of deposition; -- usually produced by pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Basal cleavage</b></col>, <cd>cleavage parallel to the base of a crystal, or to the plane of the lateral axes.</cd> -- <col><b>Cell cleavage</b></col> <fld>(Biol.)</fld>, <cd>multiplication of cells by fission. See <er>Segmentation</er>.</cd> -- <col><b>Cubic cleavage</b></col>, <cd>cleavage parallel to the faces of a cube.</cd> -- <col><b>Diagonal cleavage</b></col>, <cd>cleavage parallel to ta diagonal plane.</cd> -- <col><b>Egg clavage</b></col>. <fld>(Biol.)</fld> <cd>See <er>Segmentation</er>.</cd> -- <col><b>Lateral cleavage</b></col>, <cd>cleavage parallel to the lateral planes.</cd> -- <mcol><col><b>Octahedral cleavage</b></col>, <col><b>Dodecahedral cleavage</b></col>, <it>or</it> <col><b>Rhombohedral cleavage</b></col></mcol>, <cd>cleavage parallel to the faces of an octahedron, dodecahedron, or rhombohedron.</cd> -- <col><b>Prismatic cleavage</b></col>, <cd>cleavage parallel to a vertical prism.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleave</ent><br/
<hw>Cleave</hw> <pr>(kl<emac/v)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Cleaved</conjf> <pr>(kl<emac/vd)</pr>, <conjf>Clave</conjf> (<pr>kl<amac/v</pr>, <mark>Obs.</mark>); <pos>p. p.</pos> <conjf>Cleaved</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cleaving</conjf>.]</vmorph> <ety>[OE. <ets>cleovien</ets>, <ets>clivien</ets>, <ets>cliven</ets>, AS. <ets>cleofian</ets>, <ets>clifian</ets>; akin to OS. <ets>klib<omac/n</ets>, G. <ets>kleben</ets>, LG. <ets>kliven</ets>, D. <ets>kleven</ets>, Dan. <ets>kl<ae/be</ets>, Sw. <ets>klibba</ets>, and also to G. <ets>kleiben</ets> to cleve, paste, Icel. <ets>kl<imac/fa</ets> to climb. Cf. <er>Climb</er>.]</ety> <sn>1.</sn> <def>To adhere closely; to stick; to hold fast; to cling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My bones <qex>cleave</qex> to my skin.</q> <rj><qau>Ps. cii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The diseases of Egypt . . . shall <qex>cleave</qex> unto thee.</q> <rj><qau>Deut. xxviii. 60.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sophistry <qex>cleaves</qex> close to and protects<br/
Sin's rotten trunk, concealing its defects.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite or be united closely in interest or affection; to adhere with strong attachment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore shall a man leave his father and his mother, and shall <qex>cleave</qex> unto his wife.</q> <rj><qau>Gen. ii. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cleave</qex> unto the Lord your God.</q> <rj><qau>Josh. xxiii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fit; to be adapted; to assimilate.</def> <mark>[Poetic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>New honors come upon him,<br/
Like our strange garments, <qex>cleave</qex> not to their mold<br/
But with the aid of use.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleave</ent><br/
<hw>Cleave</hw> <pr>(kl<emac/v)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Cleft</conjf> <pr>(kl<ecr/ft)</pr>, <conjf>Clave</conjf> (<pr>kl<amac/v</pr>, <mark>Obs.</mark>), <conjf>Clove</conjf> (<pr>kl<omac/v</pr>, <mark>Obsolescent</mark>); <pos>p. p.</pos> <conjf>Cleft</conjf>, <conjf>Cleaved</conjf> <pr>(kl<emac/vd)</pr> or <conjf>Cloven</conjf> <pr>(kl<omac/"v'n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cleaving</conjf>.]</vmorph> <ety>[OE. <ets>cleoven</ets>, <ets>cleven</ets>, AS. <ets>cle<oacute/fan</ets>; akin to OS. <ets>klioban</ets>, D. <ets>klooven</ets>, G. <ets>klieben</ets>, Icel. <ets>klj<umac/fa</ets>, Sw. <ets>klyfva</ets>, Dan. <ets>kl<oum/ve</ets> and prob. to Gr. <grk>gly`fein</grk> to carve, L. <ets>glubere</ets> to peel. Cf. <er>Cleft</er>.]</ety> <sn>1.</sn> <def>To part or divide by force; to split or rive; to cut.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O Hamlet, thou hast <qex>cleft</qex> my heart in twain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To part or open naturally; to divide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every beast that parteth the hoof, and <qex>cleaveth</qex> the cleft into two claws.</q> <rj><qau>Deut. xiv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleave</ent><br/
<hw>Cleave</hw>, <pos>v. i.</pos> <def>To part; to open; to crack; to separate; as parts of bodies; <as>as, the ground <ex>cleaves</ex> by frost</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Mount of Olives shall <qex>cleave</qex> in the midst.</q> <rj><qau>Zech. xiv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleavelandite</ent><br/
<hw>Cleave"land*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From Professor Parker <etsep>Cleaveland</etsep>.]</ety> <fld>(Min.)</fld> <def>A variety of albite, white and lamellar in structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleaver</ent><br/
<hw>Cleav"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who cleaves, or that which cleaves; especially, a butcher's instrument for cutting animal bodies into joints or pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleavers</ent><br/
<hw>Cleav"ers</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cleave</er> to stick.]</ety> <fld>(Bot.)</fld> <def>A species of <gen>Galium</gen> (<spn>Galium Aparine</spn>), having a fruit set with hooked bristles, which adhere to whatever they come in contact with; -- called also, <altname>goose grass</altname>, <altname>catchweed</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleche</ent><br/
||<hw>Cl<eacute/`ch<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>cl<eacute/ch<eacute/</ets>.]</ety> <fld>(Her.)</fld> <def>Charged with another bearing of the same figure, and of the color of the field, so large that only a narrow border of the first bearing remains visible; -- said of any heraldic bearing. Compare <er>Voided</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clechy</ent><br/
<hw>Cle"chy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Cl<eacute/ch<eacute/</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cledge</ent><br/
<hw>Cledge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Clay</er>.]</ety> <fld>(Mining.)</fld> <def>The upper stratum of fuller's earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cledgy</ent><br/
<hw>Cledg"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stiff, stubborn, clayey, or tenacious; <as>as, a <ex>cledgy</ex> soil</as>.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clee</ent><br/
<hw>Clee</hw> <pr>(kl<emac/)</pr>, <pos>n.</pos> <def>A claw.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clee</ent><br/
<hw>Clee</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The redshank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleek</ent><br/
<hw>Cleek</hw> <pr>(kl<emac/k)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A large hook or crook, as for a pot over a fire; specif., an iron-headed golf club with a straight, narrow face and a long shaft.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Act of cleeking; a clutch.</def> <mark>[Scot.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cleek</ent><br/
<hw>Cleek</hw>, <pos>v. t.</pos>  <vmorph>[<pos>pret.</pos> <conjf>Claught</conjf> <pr>(?)</pr>; <pos>pret. & p. p.</pos> <conjf>Cleeked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cleeking</conjf>.]</vmorph> <ety>[ME. <ets>cleken</ets>, <ets>clechen</ets>, to seize, clutch; perh. akin to E. <ets>clutch</ets>.]</ety> <mark>[Scot & Dial. Eng.]</mark> <sn>1.</sn>  <def>To seize; clutch; snatch; catch; pluck.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To catch or draw out with a cleek, as a fish; to hook.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>To hook or link (together); hence, to marry.</def>  <rj><au>Scott.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clef</ent><br/
<hw>Clef</hw> <pr>(kl<ecr/f; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>clef</ets> key, a key in music, fr. L. <ets>clavis</ets> key. See <er>Clavicle</er>.]</ety> <fld>(Mus.)</fld> <def>A character used in musical notation to determine the position and pitch of the scale as represented on the staff.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>clefs</xex> are three in number, called the C, F, and G <xex>clefs</xex>, and are probably corruptions or modifications of these letters. They indicate that the letters of absolute pitch belonging to the lines upon which they are placed, are respectively C, F, and G. The F or <xex>bass clef</xex>, and the G or <xex>treble clef</xex>, are fixed in their positions upon the staff. The C <xex>clef</xex> may have three positions. It may be placed upon the first or lower line of the staff, in which case it is called <xex>soprano clef</xex>, upon the third line, in which case it called <xex>alto clef</xex>, or upon the fourth line, in which case <xex>tenor clef</xex>. It rarely or never is placed upon the second line, except in ancient music. See other forms of C <xex>clef</xex> under C, 2.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Alto clef</b></col>, <col><b>Bass clef</b></col></mcol>. <cd>See under <er>Alto</er>, <er>Bass</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleft</ent><br/
<hw>Cleft</hw> <pr>(kl<ecr/ft)</pr>, <def><pos>imp. & p. p.</pos> from <er>Cleave</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleft</ent><br/
<hw>Cleft</hw>, <pos>a.</pos> <sn>1.</sn> <def>Divided; split; partly divided or split.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Incised nearly to the midrib; <as>as, a <ex>cleft</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleft</ent><br/
<hw>Cleft</hw>, <pos>n.</pos> <ety>[OE. <ets>clift</ets>; cf. Sw. <ets>klyft</ets> cave, den, Icel. <ets>kluft</ets> cleft, Dan. <ets>kl<oum/ft</ets>, G. <ets>kluft</ets>. See <er>Cleave</er> to split and cf. 2d <er>Clift</er>, 1st <er>Clough</er>.]</ety> <sn>1.</sn> <def>A space or opening made by splitting; a crack; a crevice; <as>as, the <ex>cleft</ex> of a rock</as>.</def>  <rj><au>Is. ii. 21.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece made by splitting; <as>as, a <ex>cleft</ex> of wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Far.)</fld> <def>A disease in horses; a crack on the band of the pastern.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Branchial clefts</b></col>. <cd>See under <er>Branchial</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Crack; crevice; fissure; chink; cranny.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleft-footed</ent><br/
<hw>Cleft"-foot`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a cloven foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleftgraft</ent><br/
<hw>Cleft"graft`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To ingraft by cleaving the stock and inserting a scion.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleg</ent><br/
<hw>Cleg</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Northern Eng. & Scot. <ets>gleg</ets>: cf. Gael. <ets>crethleag</ets>.]</ety> <fld>(Zool.)</fld> <def>A small breeze or horsefly.</def> <mark>[North of Eng. & Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cleistocarp</ent><br/
<hw>cleistocarp</hw> <pos>n.</pos> <def>the closed spore-bearing structure of some fungi (especially  <fam>Aspergillaceae</fam> and <fam>Eurysiphaceae</fam>) from which spores are released only by decay or disintegration; -- called also <altname>cleistothecium</altname>.</def><br/
<syn><b>Syn. --</b> cleistothecium.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cleistogamous</ent><br/
<ent>Cleistogamic</ent><br/
<mhw>{ <hw>Cleis`to*gam"ic</hw> <pr>(?)</pr>, <hw>Cleis*tog"a*mous</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[Gr. <?/ closed (fr. <?/ to shut) + <grk>ga`mos</grk> marriage.]</ety> <fld>(Bot.)</fld> <def>Having, beside the usual flowers, other minute, closed flowers, without petals or with minute petals; -- said of certain species of plants which possess flowers of two or more kinds, the closed ones being so constituted as to insure self-fertilization.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cleistothecium</ent><br/
<hw>cleistothecium</hw> <pos>n.</pos> <def>the closed spore-bearing structure of some fungi (especially <fam>Aspergillaceae</fam> and <fam>Eurysiphaceae</fam>) from which spores are released only by decay or disintegration; -- called also <altname>cleistocarp</altname>.</def><br/
<syn><b>Syn. --</b> cleistocarp.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clem</ent><br/
<hw>Clem</hw> <pr>(kl<ecr/m)</pr>, <pos>v. t. & i.</pos> <ety>[Cf. <ets>clam</ets> to clog, or G. <ets>klemmen</ets> to pinch, Icel. <ets>kl<oum/mbra</ets>, E. <ets>clamp</ets>.]</ety> <def>To starve; to famish.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clematis</ent><br/
<hw>Clem"a*tis</hw> <pr>(kl<ecr/m"<adot/*t<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>klhmati`s</grk> brushwood, also (from its long, lithe branches) clematis. fr. <grk>klh^ma</grk> twig, shoot, fr. <grk>kla^n</grk> to break off.]</ety> <fld>(Bot.)</fld> <def>A genus of flowering plants, of many species, mostly climbers, having feathery styles, which greatly enlarge in the fruit; -- called also <altname>virgin's bower</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clemence</ent><br/
<hw>Clem"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Clemency.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clemency</ent><br/
<hw>Clem"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Clemencies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>clementia</ets>, fr. <ets>clemens</ets> mild, calm.]</ety> <sn>1.</sn> <def>Disposition to forgive and spare, as offenders; mildness of temper; gentleness; tenderness; mercy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great <qex>clemency</qex> and tender zeal toward their subjects.</q> <rj><qau>Stowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They had applied for the royal <qex>clemency</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Mildness or softness of the elements; <as>as, the <ex>clemency</ex> of the season</as>.</def></p>

<p><syn><b>Syn.</b> -- Mildness; tenderness; indulgence; lenity; mercy; gentleness; compassion; kindness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clement</ent><br/
<hw>Clem"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clemens</ets>; <ets>-entis</ets>; cf. F. <ets>cl<?/ment</ets>.]</ety> <def>Mild in temper and disposition; merciful; compassionate.</def>  <rj><au>Shak.</au></rj></p>

<p>-- <wordforms><wf>Clem"ent*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clementine</ent><br/
<hw>Clem"ent*ine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Clement, esp. to St. Clement of Rome and the spurious homilies attributed to him, or to Pope Clement V. and his compilations of canon law.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleopatra</ent><br/
<hw>Cleopatra</hw> <pos>n.</pos> <def>a queen of Egypt; b. 69 <it>b. c.</it>, d. 30 <it>b. c.</it></def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cleopatra's needle</ent><br/
<hw>Cle`o*pa"tra's nee"dle</hw> <pr>(?)</pr>. <ety>[So named after <etsep>Cleopatra</etsep>, queen of Egypt.]</ety> <def>Either of two obelisks which were moved in ancient times from Heliopolis to Alexandria, one of which is now on the Thames Embankment in London, and the other in Central Park, in the City of New York.</def></p>

<p><note><hand/ Some writers consider that only the obelisk now in Central Park is properly called <ex>Cleopatra's needle</ex>.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clench</ent><br/
<hw>Clench</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Clinch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clepe</ent><br/
<hw>Clepe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cleped</conjf> <pr>(?)</pr> <or/ <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cleping</conjf>. Cf. <conjf>Ycleped</conjf>.]</vmorph> <ety>[AS. <ets>clepan</ets>, <ets>cleopian</ets>, <ets>clipian</ets>, <ets>clypian</ets>, to cry, call.]</ety> <def>To call, or name.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That other son was <qex>cleped</qex> Cambalo.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clepe</ent><br/
<hw>Clepe</hw>, <pos>v. i.</pos> <def>To make appeal; to cry out.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wandering in woe, and to the heavens on high<br/
<qex>Cleping</qex> for vengeance of this treachery.</q> <rj><qau>Mir. for Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clepsine</ent><br/
||<hw>Clep"si*ne</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A genus of fresh-water leeches, furnished with a proboscis. They feed upon mollusks and worms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clepsydra</ent><br/
<hw>Clep"sy*dra</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[L. from Gr. <?/; <?/ to steal, conceal + <?/ water.]</ety> <def>A water clock; a contrivance for measuring time by the graduated flow of a liquid, as of water, through a small aperture. See <xex>Illust.</xex> in <er>Appendix</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 265 --></p>

<p><ent>Cleptomania</ent><br/
||<hw>Clep`to*ma"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>See <er>Kleptomania</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerestory</ent><br/
<hw>Clere"sto`ry</hw> <pr>(kl<emac/r"st<omac/`r<ycr/)</pr>, <pos>n.</pos> <def>Same as <er>Clearstory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clergeon</ent><br/
<hw>Cler"geon</hw> <pr>(kl<etil/r"j<ucr/n)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>clerc</ets>. See <er>Clerk</er>.]</ety> <def>A chorister boy.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clergial</ent><br/
<hw>Cler"gi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Learned; erudite; clerical.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clergical</ent><br/
<hw>Cler"gi*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the clergy; clerical; clerkily; learned.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clergy</ent><br/
<hw>Cler"gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>clergie</ets>, <ets>clergi</ets>, <ets>clerge</ets>, OF. <ets>clergie</ets>, F. <ets>clergie</ets> (fr. <ets>clerc</ets> clerc, fr. L. <ets>clericus</ets> priest) confused with OF. <ets>clergi<eacute/</ets>, F. <ets>clerg<eacute/</ets>, fr. LL. <ets>clericatus</ets> office of priest, monastic life, fr. L. <ets>clericus</ets> priest, LL. scholar, clerc. Both the Old French words meant clergy, in sense 1, the former having also sense 2. See <er>Clerk</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The body of men set apart, by due ordination, to the service of God, in the Christian church, in distinction from the laity; in England, usually restricted to the ministers of the Established Church.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Learning; also, a learned profession.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Sophictry . . . rhetoric, and other <qex>cleargy</qex>.</q> <rj><qau>Guy of Warwick.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Put their second sons to learn some <qex>clergy</qex>.</q> <rj><qau>State Papers (1515).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The privilege or benefit of clergy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If convicted of a clergyable felony, he is entitled equally to his <qex>clergy</qex> after as before conviction.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Benefit of clergy</b></col> <fld>(Eng., Law)</fld>, <cd>the exemption of the persons of clergymen from criminal process before a secular judge -- a privilege which was extended to all who could read, such persons being, in the eye of the law, <xex>clerici</xex>, or clerks. This privilege was abridged and modified by various statutes, and finally abolished in the reign of George IV. (1827).</cd> -- <mcol><col><b>Regular clergy</b></col>, <col><b>Secular clergy</b></col></mcol> <cd>See <er>Regular</er>, <pos>n.</pos>, and <er>Secular</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clergyable</ent><br/
<hw>Cler"gy*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Entitled to, or admitting, the benefit of clergy; <as>as, a <ex>clergyable</ex> felony</as>.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clergyman</ent><br/
<hw>Cler"gy*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Clergymen</plw> <pr>(#)</pr>.</plu> <def>An ordained minister; a man regularly authorized to preach the gospel, and administer its ordinances; in England usually restricted to a minister of the Established Church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleric</ent><br/
<hw>Cler"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS., fr. L. <ets>clericus</ets>. See <er>Clerk</er>.]</ety> <def>A clerk, a clergyman.</def> <mark>[R.]</mark>  <rj><au>Bp. Horsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleric</ent><br/
<hw>Cler"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Clerical</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerical</ent><br/
<hw>Cler"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>clericalis</ets>. <ets>See</ets> <er>Clerk</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the clergy; suitable for the clergy.</def> <ldquo/A <xex>clerical</xex> education.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or relating to a clerk or copyist, or to writing.</def> <ldquo/<xex>Clerical</xex> work.<rdquo/  <rj><au>E. Everett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>characteristic of the work performed by a clerk, secretary, or copyist, or suitable to be performed by a clerk.</def> <ldquo/<xex>Clerical</xex> staff.<rdquo/<br/
[<source>PJC</source>]</p>

<p><cs><col><b>A clerical error</b></col>, <cd>an error made in copying or writing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clericalism</ent><br/
<hw>Cler"ic*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An excessive devotion to the interests of the sacerdotal order; undue influence of the clergy; sacerdotalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clericals</ent><br/
<hw>cler"ic*als</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>garments worn by the clergy.</def> <mark>[informal]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Clericity</ent><br/
<hw>Cler*ic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being a clergyman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleridae</ent><br/
<hw>Cleridae</hw> <pos>n.</pos> <def>a natural family of beetles that prey on other insects.</def><br/
<syn><b>Syn. --</b> family <fam>Cleridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clerihew</ent><br/
<hw>clerihew</hw> <pos>n.</pos> <def>a witty satiric verse containing two rhymed couplets and mentioning a famous person.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clerisy</ent><br/
<hw>Cler"i*sy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>clericia</ets>. See <er>Clergy</er>.]</ety> <sn>1.</sn> <def>The literati, or well educated class.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The clergy, or their opinions, as opposed to the laity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerk</ent><br/
<hw>Clerk</hw> <pr>(kl<etil/rk; in Eng. kl<aum/rk; 277)</pr>, <pos>n.</pos> <ety>[Either OF. <ets>clerc</ets>, fr. L. <ets>clericus</ets> a priest, or AS. <ets>clerc</ets>, <ets>cleric</ets>, clerk, priest, fr. L. <ets>clericus</ets>, fr. Gr. <grk>klhriko`s</grk> belonging to the clergy, fr. <grk>klh^ros</grk> lot, allotment, clergy; cf. Deut. xviii. 2. Cf. <er>Clergy</er>.]</ety> <sn>1.</sn> <def>A clergyman or ecclesiastic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All persons were styled <qex>clerks</qex> that served in the church of Christ.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A man who could read; a scholar; a learned person; a man of letters.</def> <mark>[Obs.]</mark> <ldquo/Every one that could read . . . being accounted a <xex>clerk</xex>.<rdquo/  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was no great <qex>clerk</qex>, but he was perfectly well versed in the interests of Europe.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A parish officer, being a layman who leads in reading the responses of the Episcopal church service, and otherwise assists in it.</def> <mark>[Eng.]</mark>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And like unlettered <qex>clerk</qex> still cry <ldquo/Amen<rdquo/.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One employed to keep records or accounts; a scribe; an accountant; <as>as, the <ex>clerk</ex> of a court; a town <ex>clerk</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>clerk</qex> of the crown . . . withdrew the bill.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some cases, <xex>clerk</xex> is synonymous with <xex>secretary</xex>. A clerk is always an officer subordinate to a higher officer, board, corporation, or person; whereas a secretary may be either a subordinate or the head of an office or department.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An assistant in a shop or store.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerk-ale</ent><br/
<hw>Clerk"-ale`</hw> <pr>(? in Eng. <?/)</pr>, <pos>n.</pos> <def>A feast for the benefit of the parish clerk.</def> <mark>[Eng.]</mark>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerkless</ent><br/
<hw>Clerk"less</hw>, <pos>a.</pos> <def>Unlearned.</def> <mark>[Obs.]</mark>  <rj><au>E. Waterhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerklike</ent><br/
<hw>Clerk"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scholarlike.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerkliness</ent><br/
<hw>Clerk"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Scholarship.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerkly</ent><br/
<hw>Clerk"ly</hw>, <pos>a.</pos> <def>Of or pertaining to a clerk.</def>  <rj><au>Cranmer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerkly</ent><br/
<hw>Clerk"ly</hw>, <pos>adv.</pos> <def>In a scholarly manner.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerkship</ent><br/
<hw>Clerk"ship</hw>, <pos>n.</pos> <def>State, quality, or business of a clerk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleromancy</ent><br/
<hw>Cler"o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ lot + <ets>-mancy</ets>; cf. F. <ets>cl<eacute/romancie</ets>.]</ety> <def>A divination by throwing dice or casting lots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleronomy</ent><br/
<hw>Cle*ron"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ inheritance + <?/ to possess.]</ety> <def>Inheritance; heritage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clerstory</ent><br/
<hw>Cler"sto`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Clearstory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clever</ent><br/
<hw>Clev"er</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Origin uncertain. Cf. OE. <ets>cliver</ets> eager, AS. <ets>clyfer</ets> (in comp.) cloven; or <ets>clifer</ets> a claw, perh. connected with E. <ets>cleave</ets> to divide, split, the meaning of E. <ets>clever</ets> perh. coming from the idea of grasping, seizing (with the mind).]</ety> <sn>1.</sn> <def>Possessing quickness of intellect, skill, dexterity, talent, or adroitness; expert.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though there were many <qex>clever</qex> men in England during the latter half of the seventeenth century, there were only two great creative minds.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be good, sweet maid, and let who will be <qex>clever</qex>.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Showing skill or adroitness in the doer or former; <as>as, a <ex>clever</ex> speech; a <ex>clever</ex> trick</as>.</def>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having fitness, propriety, or suitableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T would sound more <qex>clever</qex><br/
To me and to my heirs forever.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Well-shaped; handsome.</def> <ldquo/The girl was a tight, <xex>clever</xex> wench as any was.<rdquo/  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Good-natured; obliging.</def> <mark>[U. S.]</mark></p>

<p><syn><b>Syn.</b> -- See <er>Smart</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleverish</ent><br/
<hw>Clev"er*ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat clever.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleverly</ent><br/
<hw>Clev"er*ly</hw>, <pos>adv.</pos> <def>In a clever manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never was man so <qex>clever</qex> absurd.</q> <rj><qau>C. Smart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cleverness</ent><br/
<hw>Clev"er*ness</hw>, <pos>n.</pos> <def>The quality of being clever; skill; dexterity; adroitness.</def></p>

<p><syn><b>Syn.</b> -- See <er>Ingenuity</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clevis</ent><br/
<hw>Clev"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cleave</er> to adhere, <er>Clavel</er>.]</ety> <def>A piece of metal bent in the form of an oxbow, with the two ends perforated to receive a pin, used on the end of the tongue of a plow, wagen, etc., to attach it to a draft chain, whiffletree, etc.; -- called also <altname>clavel</altname>, <altname>clevy</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clue</ent><br/
<ent>Clew</ent><br/
<mhw>{ <hw>Clew</hw> <pr>(kl<umac/)</pr>, <hw>Clue</hw>, }</mhw> <pos>n.</pos> <ety>[OE. <ets>clewe</ets>, <ets>clowe</ets>, <ets>clue</ets>, AS. <ets>cleowen</ets>, <ets>cliwen</ets>, <ets>clywe</ets> ball of thread; akin to D. <ets>kluwen</ets>, OHG. <ets>chliwa</ets>, <ets>chliuwa</ets>, G. dim. <ets>kleuel</ets>, <ets>kn<aum/uel</ets>, and perch. to L. <ets>gluma</ets> hull, husk, Skr. <ets>glaus</ets> sort of ball or tumor. Perch. akin to E. <ets>claw</ets>. <root/26. Cf. <er>Knawel</er>.]</ety> <sn>1.</sn> <def>A ball of thread, yarn, or cord; also, The thread itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Untwisting his deceitful <qex>clew</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which guides or directs one in anything of a doubtful or intricate nature; that which gives a hint in the solution of a mystery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>clew</qex>, without which it was perilous to enter the vast and intricate maze of countinental politics, was in his hands.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <sd>(a.)</sd> <def>A lower corner of a square sail, or the after corner of a fore-and-aft sail.</def> <sd>(b.)</sd> <def>A loop and thimbles at the corner of a sail.</def> <sd>(c.)</sd> <def>A combination of lines or nettles by which a hammock is suspended.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clew garnet</b></col> <fld>(Naut.)</fld>, <cd>one of the ropes by which the clews of the courses of square-rigged vessels are drawn up to the lower yards.</cd> -- <col><b>Clew line</b></col> <fld>(Naut.)</fld>, <cd>a rope by which a clew of one of the smaller square sails, as topsail, topgallant sail, or royal, is run up to its yard.</cd> -- <col><b>Clew-line block</b></col> <fld>(Naut.)</fld>, <cd>The block through which a clew line reeves. See <xex>Illust.</xex> of <er>Block</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clew</ent><br/
<hw>Clew</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clewed</conjf> <pos>p. pr. & vb. n.</pos> <conjf>Clewing</conjf>.]</vmorph> <ety>[Cf. D. <ets>kluwenen</ets>. See <er>Clew</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To direct; to guide, as by a thread.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Direct and <qex>clew</qex> me out the way to happiness.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To move of draw (a sail or yard) by means of the clew garnets, clew lines, etc.; esp. to draw up the clews of a square sail to the yard.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To clew down</b></col> <fld>(Naut.)</fld>, <cd>to force (a yard) down by hauling on the clew lines.</cd> -- <col><b>To clew up</b></col> <fld>(Naut.)</fld>, <cd>to draw (a sail) up to the yard, as for furling.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliche</ent><br/
||<hw>Cli`ch<eacute/"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>clich<eacute/</ets>, from <ets>clicher</ets> to stereotype.]</ety> <sn>1.</sn> <def>A stereotype plate or any similar reproduction of ornament, or lettering, in relief.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a trite or obvious remark.</def><br/
<syn><b>Syn. --</b> platitude, banality, commonplace, bromide.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><cs><col><b>Clich<eacute/ casting</b></col>, <cd>a mode of obtaining an impression from a die or woodcut, or the like, by striking it suddenly upon metal which has been fused and is just becoming solid; also, the casting so obtained.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cliched</ent><br/
<hw>cliched</hw> <pos>adj.</pos> <def>repeated regularly without thought or originality.</def><br/
<syn><b>Syn. --</b> ready-made, routine, stock(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Click</ent><br/
<hw>Click</hw> <pr>(kl<icr/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clicked</conjf> <pr>(kl<icr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clicking</conjf>.]</vmorph> <ety>[Prob. an onomatopoetic word: cf. OF. <ets>cliquier</ets>. <er>See Clack</er>, and cf. <er>Clink</er>, <er>Clique</er>.]</ety> <def>To make a slight, sharp noise (or a succession of such noises), as by gentle striking; to tick.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The varnished clock that <qex>clicked</qex> behind the door.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Click</ent><br/
<hw>Click</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To move with the sound of a click.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>clicked</qex> back the bolt which held the window sash.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to make a clicking noise, as by striking together, or against something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Jove] <qex>clicked</qex> all his marble thumbs.</q> <rj><qau>Ben Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When merry milkmaids <qex>click</qex> the latch.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Click</ent><br/
<hw>Click</hw>, <pos>n.</pos> <sn>1.</sn> <def>A slight sharp noise, such as is made by the cocking of a pistol.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of articulation used by the natives of Southern Africa, consisting in a sudden withdrawal of the end or some other portion of the tongue from a part of the mouth with which it is in contact, whereby a sharp, clicking sound is produced. The sounds are four in number, and are called cerebral, palatal, dental, and lateral <xex>clicks</xex> or clucks, the latter being the noise ordinarily used in urging a horse forward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Click</ent><br/
<hw>Click</hw>, <pos>v. t.</pos> <ety>[OE. <ets>kleken</ets>, <ets>clichen</ets>. Cf. <er>Clutch</er>.]</ety> <def>To snatch.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Click</ent><br/
<hw>Click</hw>, <pos>n.</pos> <ety>[Cf. 4th <er>Click</er>, and OF. <ets>clique</ets> latch.]</ety> <sn>1.</sn> <def>A detent, pawl, or ratchet, as that which catches the cogs of a ratchet wheel to prevent backward motion. See <xex>Illust.</xex> of <er>Ratched wheel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The latch of a door.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Click beetle</ent><br/
<hw>Click" bee"tle</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See <er>Elater</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clicker</ent><br/
<hw>Click"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stands before a shop door to invite people to buy.</def> <mark>[Low, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>One who as has charge of the work of a companionship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clicket</ent><br/
<hw>Click"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cliquet</ets> the latch of a door. See 5th <er>Click</er>.]</ety> <sn>1.</sn> <def>The knocker of a door.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A latch key.</def> <mark>[Eng.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clickety-click</ent><br/
<ent>clickety-clack</ent><br/
<mhw><hw>clickety-clack</hw>, <hw>clickety-click</hw></mhw> <pos>n.</pos> <def>a fast and rhythmic clicking sound, as of <ldquo/click-clack<rdquo/; <as>as, the <ex>clickety-clack</ex> of the typewriters</as>.</def><br/
<syn><b>Syn. --</b> clickety-clack.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clicky</ent><br/
<hw>Click"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a click; abounding in clicks.</def> <ldquo/Their strange <xex>clicky</xex> language.<rdquo/  <rj><au>The Century.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clidastes</ent><br/
||<hw>Cli*das"tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., prob. from Gr. <grk>klei`s</grk> key.]</ety> <fld>(Paleon.)</fld> <def>A genus of extinct marine reptiles, allied to the Mosasaurus. See <xex>Illust.</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliency</ent><br/
<hw>Cli"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>State of being a client.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Client</ent><br/
<hw>Cli"ent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cliens</ets>, <ets>-emtis</ets>, for <ets>cluens</ets>, one who hears (in relation to his protector), a client, fr. L. <ets>cluere</ets> to be named or called; akin to Gr. <?/ to hear, Skr. <ets><cced/ry</ets>, and E. <ets>loud</ets>: cf. F. <ets>client</ets>. See <er>Loud</er>.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>A citizen who put himself under the protection of a man of distinction and influence, who was called his patron.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dependent; one under the protection of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do think they are your friends and <qex>clients</qex>,<br/
And fearful to disturb you.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>One who consults a legal adviser, or submits his cause to his management.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clientage</ent><br/
<hw>Cli"ent*age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>State of being client.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A body of clients.</def>  <rj><au>E. Everett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliental</ent><br/
<hw>Cli*en"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a client.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A dependent and <qex>cliental</qex> relation.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I sat down in the <qex>cliental</qex> chair.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliented</ent><br/
<hw>Cli"ent*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supplied with clients.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The least <qex>cliented</qex> pettifiggers.</q> <rj><qau>R. Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clientelage</ent><br/
<hw>Cli*en"te*lage</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Clientele</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clientele</ent><br/
<hw>Cli`en*tele"</hw> <pr>(? or ?)</pr>, <pos>n.</pos> <ety>[L. <ets>clientela</ets>: cf. F. <ets>client<egrave/le</ets>.]</ety> <sn>1.</sn> <def>The condition or position of a client; clientship</def>. <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The clients or dependents of a nobleman of patron.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The persons who make habitual use of the services of another person; one's clients, collectively; <as>as, the <ex>clientele</ex> of a lawyer, doctor, notary, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clientship</ent><br/
<hw>Cli"ent*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Condition of a client; state of being under the protection of a patron.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliff</ent><br/
<hw>Cliff</hw> <pr>(kl<icr/f)</pr>, <pos>n.</pos> <ety>[AS. <ets>clif</ets>, <ets>cloef</ets>; akin to OS. <ets>klif</ets>, D. <ets>klif</ets>, <ets>klip</ets>, Icel. <ets>klif</ets>, Dan. & G. <ets>klippe</ets>, Sw. <ets>klippa</ets>; perh. orig. <ets>a climbing place</ets>. See <er>Climb</er>.]</ety> <def>A high, steep rock; a precipice.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cliff swallow</b></col> <fld>(Zool.)</fld>, <cd>a North American swallow (<spn>Petrochelidon lunifrons</spn>), which builds its nest against cliffs; the <altname>eaves swallow</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliff</ent><br/
<hw>Cliff</hw>, <pos>n.</pos> <fld>(Mus.)</fld> <def>See <er>Clef</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cliffbrake</ent><br/
<hw>cliffbrake</hw> <pos>n.</pos> <def>any of several small lithophytic ferns of tropical and warm temperate regions.</def><br/
<syn><b>Syn. --</b> cliff brake, rock brake.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cliffhanger</ent><br/
<hw>cliffhanger</hw> <pos>n.</pos> <def>a contest whose outcome is uncertain up to the very end.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an episode of a drama that ends in suspense.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clifflike</ent><br/
<hw>clifflike</hw> <pos>adj.</pos>  <def>characterized by cliffs; -- of a shore or shoreline; -- contrasted with <ant>beachy</ant>.</def> [Narrower terms: <stype>precipitous, steep</stype>]<br/
<syn><b>Syn. --</b> cliffy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cliff limestone</ent><br/
<hw>Cliff" lime"stone`</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>A series of limestone strata found in Ohio and farther west, presenting bluffs along the rivers and valleys, formerly supposed to be of one formation, but now known to be partly Silurian and partly Devonian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cliffy</ent><br/
<hw>Cliff"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having cliffs; broken; craggy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clift</ent><br/
<hw>Clift</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See 1st <er>Cliff</er>, <pos>n.</pos>]</ety> <def>A cliff.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That gainst the craggy <qex>clifts</qex> did loudly roar.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clift</ent><br/
<hw>Clift</hw>, <pos>n.</pos> <ety>[See <er>Cleft</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A cleft of crack; a narrow opening.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fork of the legs; the crotch.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clifted</ent><br/
<hw>Clift"ed</hw>, <pos>a.</pos> <ety>[From <er>Clift</er> a cleft.]</ety> <def>Broken; fissured.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Climb the Ande<?/ <qex>clifted</qex> side.</q> <rj><qau>Grainger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climacter</ent><br/
<hw>Cli*mac"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/, prop., round of a ladder, fr. <?/ ladder: cf. F. <ets>climact<egrave/re</ets>. See <er>Climax</er>.]</ety> <def>See <er>Climacteric</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climacteric</ent><br/
<hw>Cli*mac"ter*ic</hw> <pr>(? <or/ ?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>climactericus</ets>, Gr. <?/. See <er>Climacter</er>.]</ety> <def>Relating to a climacteric; critical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climacteric</ent><br/
<hw>Cli*mac"ter*ic</hw>, <pos>n.</pos> <sn>1.</sn> <def>A period in human life in which some great change is supposed to take place in the constitution. The critical periods are thought by some to be the years produced by multiplying 7 into the odd numbers 3, 5, 7, and 9; to which others add the 81st year.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any critical period.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is your lot, as it was mine, to live during one of the grand <qex>climacterics</qex> of the world.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>climb-down</ent><br/
<hw>climb-down</hw> <pos>n.</pos> <sn>1.</sn> <def>a retraction of a previously held position.</def><br/
<syn><b>Syn. --</b> withdrawal, backdown.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><cs><mcol><col><b>Grand  climacteric</b></col> <it>or</it>  <col><b>Great climacteric</b></col></mcol>, <cd>the sixty-third year of human life.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>I should hardly yield my rigid fibers to be regenerated by them; nor begin, in my <qex>grand climacteric</qex>, to squall in their new accents, or to stammer, in my second cradle, the elemental sounds of their barbarous metaphysics.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climacterical</ent><br/
<hw>Clim`ac*ter"ic*al</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>See <er>Climacteric</er>.</def>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climactic</ent><br/
<hw>Cli*mac"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a climax; forming, or of the nature of, a climax, or ascending series.</def></p>

<p><q>A fourth kind of parallelism . . . is still sufficiently marked to be noticed by the side of those described by Lowth, viz., <qex>climactic</qex> parallelism (sometimes called <ldquo/ascending rhythm<rdquo/).</q>  <rj><qau>S. R. Driver.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Climatal</ent><br/
<hw>Cli"ma*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Climatic.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatarchic</ent><br/
<hw>Cli`ma*tar"chic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Climate</ets> + Gr. <?/ to rule.]</ety> <def>Presiding over, or regulating, climates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climate</ent><br/
<hw>Cli"mate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>climat</ets>, L. <ets>clima</ets>, <ets>-atis</ets>, fr. Gr. <?/, <?/, slope, the supposed slope of the earth (from the equator toward the pole), hence a region or zone of the earth, fr. <?/ to slope, incline, akin to E. <ets>lean</ets>, v. i. See <er>Lean</er>, <pos>v. i.</pos>, and cf. <er>Clime</er>.]</ety> <sn>1.</sn> <fld>(Anc. Geog.)</fld> <def>One of thirty regions or zones, parallel to the equator, into which the surface of the earth from the equator to the pole was divided, according to the successive increase of the length of the midsummer day.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The condition of a place in relation to various phenomena of the atmosphere, as temperature, moisture, etc., especially as they affect animal or vegetable life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climate</ent><br/
<hw>Cli"mate</hw>, <pos>v. i.</pos> <def>To dwell.</def> <mark>[Poetic]</mark> <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatic</ent><br/
<hw>Cli*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a climate; depending on, or limited by, a climate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatical</ent><br/
<hw>Cli*mat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Climatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatize</ent><br/
<hw>Cli"ma*tize</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Climatized</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Climatizing</conjf>.]</vmorph> <def>To acclimate or become acclimated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatography</ent><br/
<hw>Cli`ma*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Climate</ets> + <ets>-graphy</ets>.]</ety> <def>A description of climates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatological</ent><br/
<hw>Cli`ma*to*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to climatology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatologist</ent><br/
<hw>Cli`ma*tol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One versed in, or who studies, climatology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climatology</ent><br/
<hw>Cli`ma*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Climate</ets> + <ets>-logy</ets>: cf. F. <ets>climatologie</ets>.]</ety> <def>The science which treats of climates and investigates their phenomena and causes.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climature</ent><br/
<hw>Cli"ma*ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>climature</ets>.]</ety> <def>A climate.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climax</ent><br/
<hw>Cli"max</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from Gr. <?/ ladder, staircase, fr. <?/ to make to bend, to lean. See <er>Ladder</er>, <er>Lean</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Upward movement; steady increase; gradation; ascent.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rhet.)</fld> <def>A figure in which the parts of a sentence or paragraph are so arranged that each succeeding one rises above its predecessor in impressiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Tribulation worketh patience, patience experience, and experience hope<rdquo/ -- a happy <qex>climax</qex>.</q> <rj><qau>J. D. Forbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The highest point; the greatest degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We must look higher for the <qex>climax</qex> of earthly good.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 266 --></p>

<p><cs><col><b>To cap the climax</b></col>, <cd>to surpass everything, as in excellence or in absurdity.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climb</ent><br/
<hw>Climb</hw> <pr>(kl<imac/m)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Climbed</conjf> <pr>(kl<imac/md)</pr>, <mark>Obs. or Vulgar</mark> <conjf>Clomb</conjf> <pr>(kl<ocr/m)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Climbing</conjf>.]</vmorph> <ety>[AS. <ets>climban</ets>; akin to OHG. <ets>chlimban</ets>, G. & D. <ets>klimmen</ets>, Icel. <ets>kl<imac/fa</ets>, and E. <ets>cleave</ets> to adhere.]</ety> <sn>1.</sn> <def>To ascend or mount laboriously, esp. by use of the hands and feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ascend as if with effort; to rise to a higher point.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Black vapors <qex>climb</qex> aloft, and cloud the day.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>To ascend or creep upward by twining about a support, or by attaching itself by tendrils, rootlets, etc., to a support or upright surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climb</ent><br/
<hw>Climb</hw>, <pos>v. t.</pos> <def>To ascend, as by means of the hands and feet, or laboriously or slowly; to mount.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climb</ent><br/
<hw>Climb</hw>, <pos>n.</pos> <def>The act of one who climbs; ascent by climbing.</def>  <rj><au>Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climbable</ent><br/
<hw>Climb"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being climbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climber</ent><br/
<hw>Climb"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, climbs</def>: <sd>(a)</sd> <fld>(Bot.)</fld> <def>A plant that climbs.</def> <sd>(b)</sd> <fld>(Zool.)</fld> <def>A bird that climbs, as a woodpecker or a parrot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climber</ent><br/
<hw>Climb"er</hw>, <pos>v. i.</pos> <ety>[From <er>Climb</er>; cf. <er>Clamber</er>.]</ety> <def>To climb; to mount with effort; to clamber.</def> <mark>[Obs.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Climbing</ent><br/
<hw>Climb"ing</hw>, <def><pos>p. pr. & vb. n.</pos> of <er>Climb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Climbing fern</b></col>. <cd>See under <er>Fern</er>.</cd> -- <col><b>Climbing perch</b></col>. <fld>(Zool.)</fld> <cd>See <er>Anabas</er>, and <er>Labyrinthici</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clime</ent><br/
<hw>Clime</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>clima</ets>. See <er>Climate</er>.]</ety> <def>A climate; a tract or region of the earth. See <er>Climate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Turn we to sutvey,<br/
Where rougher <qex>climes</qex> a nobler race display.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinanthium</ent><br/
||<hw>Cli*nan"thi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ bed + <?/ flower.]</ety> <fld>(Bot.)</fld> <def>The receptacle of the flowers in a composite plant; -- also called <altname>clinium</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinch</ent><br/
<hw>Clinch</hw> <pr>(kl<icr/nch; 224)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clinched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clinching</conjf>.]</vmorph> <ety>[OE. <ets>clenchen</ets>, prop. causative of <ets>clink</ets> to cause to clink, to strike; cf. D. <ets>klinken</ets> to tinkle, rivet. See <er>Clink</er>.]</ety> <sn>1.</sn> <def>To hold firmly; to hold fast by grasping or embracing tightly.</def> <ldquo/<xex>Clinch</xex> the pointed spear.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To set closely together; to close tightly; <as>as, to <ex>clinch</ex> the teeth or the first</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bend or turn over the point of (something that has been driven through an object), so that it will hold fast; <as>as, to <ex>clinch</ex> a nail</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make conclusive; to confirm; to establish; <as>as, to <ex>clinch</ex> an argument</as>.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinch</ent><br/
<hw>Clinch</hw>, <pos>v. i.</pos> <def>To hold fast; to grasp something firmly; to seize or grasp one another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinch</ent><br/
<hw>Clinch</hw> <pr>(kl<icr/nch)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of holding fast; that which serves to hold fast; a grip; a grasp; a clamp; a holdfast; <as>as, to get a good <ex>clinch</ex> of an antagonist, or of a weapon; to secure anything by a <ex>clinch</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pun.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A hitch or bend by which a rope is made fast to the ring of an anchor, or the breeching of a ship's gun to the ringbolts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clincher</ent><br/
<hw>Clinch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, clinches; that which holds fast.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which ends a dispute or controversy; a decisive argument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clincher-built</ent><br/
<hw>Clinch"er-built</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Clinker-built</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cling</ent><br/
<hw>Cling</hw> <pr>(kl<icr/ng)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clung</conjf> <pr>(kl<ucr/ng)</pr>, <conjf>Clong</conjf> (kl<ocr/ng), <mark>Obs.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Clinging</conjf>.]</vmorph> <ety>[AS. <ets>clingan</ets> to adhere, to wither; akin to Dan. <ets>klynge</ets> to cluster, crowd. Cf. <er>Clump</er>.]</ety> <def>To adhere closely; to stick; to hold fast, especially by twining round or embracing; <as>as, the tendril of a vine <ex>clings</ex> to its support</as>; -- usually followed by <xex>to</xex> or <xex>together</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And what hath life for thee<br/
That thou shouldst <qex>cling</qex> to it thus?</q> <rj><qau>Mrs. Hemans.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cling</ent><br/
<hw>Cling</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to adhere to, especially by twining round or embracing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>clung</qex> legs as close to his side as I could.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make to dry up or wither.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If thou speak'st false,<br/
Upon the next tree shalt thou hang alive,<br/
Till famine <qex>cling</qex> thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cling</ent><br/
<hw>Cling</hw>, <pos>n.</pos> <def>Adherence; attachment; devotion.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A more tenacious <qex>cling</qex> to worldly respects.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clingfish</ent><br/
<hw>clingfish</hw> <pos>n.</pos> <def>very small (to 3 inches) flattened marine fish with a sucking disc on the abdomen for clinging to rocks etc.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clingstone</ent><br/
<hw>Cling"stone`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the flesh attached closely to the stone, as in some kinds of peaches.</def> -- <def2><pos>n.</pos> <def>A fruit, as a peach, whose flesh adheres to the stone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clingy</ent><br/
<hw>Cling"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Apt to cling; adhesive.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinic</ent><br/
<hw>Clin"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Clinical</er>.]</ety> <sn>1.</sn> <def>One confined to the bed by sickness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>One who receives baptism on a sick bed.</def> <mark>[Obs.]</mark>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>a medical facility, often connected with a school or hospital, which treats primarily outpatients.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A school, or a session of a school or class, in which medicine or surgery is taught by the examination and treatment of patients in the presence of the pupils.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>a lesson or series of lessons taught to persons not expert in some activity, in which the errors of the students are pointed out, and remedial actions are suggested.</def><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <fld>(sports)</fld> <def>a performance so excellent as to be considered a model for emulation.</def> <mark>[fig.]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Clinic</ent><br/
<ent>Clinical</ent><br/
<mhw>{ <hw>Clin"ic*al</hw> <pr>(kl<icr/n"<icr/k*<ait/l)</pr>, <hw>Clin"ic</hw> <pr>(kl<icr/n"<icr/k)</pr> }</mhw>, <pos>a.</pos> <ety>[Gr. <grk>kliniko`s</grk>, fr. <grk>kli`nh</grk> bed, fr. <grk>kli`nein</grk> to lean, recline: cf. F. <ets>clinique</ets>. See <er>Lean</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to a bed, especially, a sick bed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a clinic, or to the study of disease in the living subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clinical baptism</b></col>, <cd>baptism administered to a person on a sick bed.</cd> -- <col><b>Clinical instruction</b></col>, <cd>instruction by means of clinics.</cd> -- <col><b>Clinical lecture</b></col> <fld>(Med.)</fld>, <cd>a discourse upon medical topics illustrated by the exhibition and examination of living patients.</cd> -- <mcol><col><b>Clinical medicine</b></col>, <col><b>Clinical surgery</b></col></mcol>, <cd>that part of medicine or surgery which is occupied with the investigation of disease in the living subject.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinically</ent><br/
<hw>Clin"ic*al*ly</hw>, <pos>adv.</pos> <def>In a clinical manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clinid</ent><br/
<hw>clin"id</hw> <pr>(kl<icr/n"<icr/d)</pr>, <pos>a.</pos> <def>of or pertaining to the <fam>Clinidae</fam>, a family of fish of temperate and tropical seas.</def><br/
[<source>PJC</source>]</p>

<p><ent>clinid</ent><br/
<hw>clin"id</hw> <pr>(kl<icr/n"<icr/d)</pr>, <pos>n.</pos> <def>any fish of the family <fam>Clinidae</fam>, of temperate and tropical seas.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clinidae</ent><br/
<hw>Clinidae</hw> <pos>n.</pos> <def>a natural family of viviparous blennies of temperate and tropical seas.</def><br/
<syn><b>Syn. --</b> family <fam>Clinidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clinique</ent><br/
||<hw>Cli*nique"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Med.)</fld> <def>A clinic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinium</ent><br/
||<hw>Clin"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kli`nh</grk> bed.]</ety> <fld>(Bot.)</fld> <def>See <er>Clinanthium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clink</ent><br/
<hw>Clink</hw> <pr>(kl<icr/<nsm/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clinked</conjf> <pr>(kl<icr/<nsm/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clinking</conjf>.]</vmorph> <ety>[OE. <ets>clinken</ets>; akin to G. <ets>klingen</ets>, D. <ets>klinken</ets>, SW. <ets>klinga</ets>, Dan. <ets>klinge</ets>; prob. of imitative origin. Cf. <er>Clank</er>, <er>Clench</er>, <er>Click</er>, <pos>v. i.</pos>]</ety> <def>To cause to give out a slight, sharp, tinkling, sound, as by striking metallic or other sonorous bodies together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And let me the canakin <qex>clink</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clink</ent><br/
<hw>Clink</hw> <pr>(kl<icr/<nsm/k)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To give out a slight, sharp, tinkling sound.</def> <ldquo/The <xex>clinking</xex> latch.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rhyme. <mark>[Humorous]</mark>.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clink</ent><br/
<hw>Clink</hw>, <pos>n.</pos> <def>A slight, sharp, tinkling sound, made by the collision of sonorous bodies.</def> <ldquo/<qex>Clink</qex> and fall of swords.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clink</ent><br/
<hw>Clink</hw> <pr>(kl<icr/<nsm/k)</pr>, <pos>n.</pos> <def>A prison cell; a lockup; -- probably orig. the name of the noted prison in Southwark, England.</def> <mark>[Colloq.]</mark> <ldquo/I'm here in the <xex>clink</xex>.<rdquo/  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clinkant</ent><br/
<hw>Clin"kant</hw> <pr>(kl<icr/<nsm/"k<ait/nt)</pr>, <pos>a.</pos> <def>See <er>Clinquant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinker</ent><br/
<hw>Clink"er</hw> <pr>(kl<icr/<nsm/k"<etil/r)</pr>, <pos>n.</pos> <ety>[From <ets>clink</ets>; cf. D. <ets>clinker</ets> a brick which is so hard that it makes a sonorous sound, from <ets>clinken</ets> to clink. Cf. <er>Clinkstone</er>.]</ety> <sn>1.</sn> <def>A mass composed of several bricks run together by the action of the fire in the kiln.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Scoria or vitrified incombustible matter, formed in a grate or furnace where anthracite coal in used; vitrified or burnt matter ejected from a volcano; slag.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A scale of oxide of iron, formed in forging.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A kind of brick. See <cref>Dutch clinker</cref>, under <er>Dutch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinker-built</ent><br/
<hw>Clink"er-built</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Having the side planks (af a boat) so arranged that the lower edge of each overlaps the upper edge of the plank next below it like clapboards on a house. See <er>Lapstreak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinkstone</ent><br/
<hw>Clink"stone`</hw> <pr>(?; 110)</pr>, <pos>n.</pos> <ety>[<ets>Clink</ets> + <ets>stone</ets>; -- from its sonorousness.]</ety> <fld>(Min.)</fld> <def>An igneous rock of feldspathic composition, lamellar in structure, and clinking under the hammer. See <er>Phonolite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinodiagonal</ent><br/
<hw>Cli`no*di*ag"o*nal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kli`nein</grk> to incline + E. <ets>diagonal</ets>.]</ety> <fld>(Crystallog.)</fld> <def>That diagonal or lateral axis in a monoclinic crystal which makes an oblique angle with the vertical axis. See <er>Crystallization</er>.</def> -- <def2><pos>a.</pos>  <def>Pertaining to, or the direction of, the clinodiagonal.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinodome</ent><br/
<hw>Cli"no*dome`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kli`nein</grk> to incline + E. <ets>dome</ets>.]</ety> <fld>(Crystallog.)</fld> <def>See under <er>Dome</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinographic</ent><br/
<hw>Cli"no*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kli`nein</grk> to incline + <ets>-graph</ets>.]</ety> <def>Pertaining to that mode of projection in drawing in which the rays of light are supposed to fall obliquely on the plane of projection.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinoid</ent><br/
<hw>Cli"noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kli`nh</grk> bed + <ets>-oid</ets>.]</ety> <fld>(Anat.)</fld> <def>Like a bed; -- applied to several processes on the inner side of the sphenoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinometer</ent><br/
<hw>Cli*nom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kli`nein</grk> to incline + <ets>-meter</ets>.]</ety> <fld>(Geol.)</fld> <def>An instrument for determining the dip of beds or strata, pr the slope of an embankment or cutting; a kind of plumb level.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinometric</ent><br/
<hw>Clin`o*met"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, or ascertained by, the clinometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to the oblique crystalline forms, or to solids which have oblique angles between the axes; <as>as, the <ex>clinometric</ex> systems</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinometry</ent><br/
<hw>Cli*nom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(geol.)</fld> <def>That art or operation of measuring the inclination of strata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinopinacoid</ent><br/
<hw>Cli`no*pin"a*coid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kli`nein</grk> to incline + E. <ets>pinacoid</ets>.]</ety> <fld>(Crystallog.)</fld> <def>The plane in crystals of the monoclinic system which is parallel to the vertical and the inclined lateral (clinidiagonal) axes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinorhombic</ent><br/
<hw>Cli`no*rhom"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kli`nein</grk> to incline + E. <ets>rhombic</ets>: cf. F. <ets>clinorhombique</ets>.]</ety> <fld>(Crystallog.)</fld> <def>Possessing the qualities of a prism, obliquely inclined to a rhombic base; monoclinic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinostat</ent><br/
<hw>Cli"no*stat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ to incline + <?/ to make to stand.]</ety> <fld>(Bot.)</fld> <def>An apparatus consisting of a slowly revolving disk, usually regulated by  clockwork, by means of wich the action of external agents, as light and gravity, on growing plants may be regulated or eliminated.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clinquant</ent><br/
<hw>Clin"quant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>Glittering; dressed in, or overlaid with, tinsel finery.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinquant</ent><br/
<hw>Clin"quant</hw>, <pos>n.</pos> <def>Tinsel; Dutch gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clinton</ent><br/
<hw>Clin"ton</hw> <sn>1.</sn> <def><person>William Jefferson Clinton</person>, b. 1946.  The 42d president of the United States, from 1993-.  Also known as <person><altname>Bill Clinton</altname></person>.</def> <mark>[Prop.]</mark><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def><person>Hillary Rodham Clinton</person>, b. 1947.  Attorney and wife of Bill Clinton, the 42d president of the United States.</def> <mark>[Prop.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>clintonia</ent><br/
<hw>clintonia</hw> <pos>n.</pos> <def>any temperate liliaceous plant of the genus <gen>Clintonia</gen> having broad basal leaves and white or yellowish or purplish flowers followed by blue or black berries.</def><br/
<syn><b>Syn. --</b> Clinton's lily.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clintonomics</ent><br/
<hw>clintonomics</hw> <pos>n.</pos> <def>the economic policies of president <person>Bill Clinton</person>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clio</ent><br/
<hw>Cli"o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/ the proclaimer, fr. <?/ to call, tell of, make famous.]</ety> <fld>(Class. Myth.)</fld> <def>The Muse who presided over history.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clione</ent><br/
<hw>Cli*o"ne</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A genus of naked pteropods. One species (<spn>Clione papilonacea</spn>), abundant in the Arctic Ocean, constitutes a part of the food of the Greenland whale. It is sometimes incorrectly called <altname>Clio</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clip</ent><br/
<hw>Clip</hw> <pr>(kl<icr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clipped</conjf> <pr>(kl<icr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clipping</conjf>.]</vmorph> <ety>[OE. <ets>cluppen</ets>, <ets>clippen</ets>, to embrace, AS. <ets>clyran</ets> to embrace, clasp; cf. OHG. <ets>kluft</ets> tongs, shears, Icel, <ets>kl<ymac/pa</ets> to pinch, squeeze, also OE. <ets>clippen</ets> to cut, shear, Dan. <ets>klippe</ets> to clip, cut, SW. & Icel. <ets>klippa</ets>.]</ety> <sn>1.</sn> <def>To embrace, hence; to encompass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O . . . that Neptune's arms, who <qex>clippeth</qex> thee about,<br/
Would bear thee from the knowledge of thyself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut off; as with shears or scissors; <as>as, to <ex>clip</ex> the hair; to <ex>clip</ex> coin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sentenced to have his ears <qex>clipped</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To curtail; to cut short.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All my reports go with the modest truth;<br/
No more nor <qex>clipped</qex>, but so.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In London they <qex>clip</qex> their words after one manner about the court, another in the city, and a third in the suburbs.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clip</ent><br/
<hw>Clip</hw> <pr>(kl<icr/p)</pr>, <pos>v. i.</pos> <def>To move swiftly; -- usually with indefinite <xex>it</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Straight flies as chek, and <qex>clips</qex> it down the wind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clip</ent><br/
<hw>Clip</hw>, <pos>n.</pos> <sn>1.</sn> <def>An embrace.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cutting; a shearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The product of a single shearing of sheep; a season's crop of wool.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A clasp or holder for letters, papers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An embracing strap for holding parts together; the iron strap, with loop, at the ends of a whiffletree.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Far.)</fld> <def>A projecting flange on the upper edge of a horseshoe, turned up so as to embrace the lower part of the hoof; -- called also <altname>toe clip</altname> and <altname>beak</altname>.</def>  <rj><au>Youatt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A blow or stroke with the hand; <as>as, he hit him a <ex>clip</ex></as>.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mach.)</fld> <def>A part, attachment, or appendage, for seizing, clasping, or holding, an object, as a cable, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn>  <fld>(Angling)</fld> <def>A gaff or hook for landing the fish, as in salmon fishing.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>10.</sn>  <def>A rapid gait.</def> <ldquo/A three-minute <xex>clip</xex>.<rdquo/  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>clipboard</ent><br/
<hw>clipboard</hw> <pos>n.</pos> <def>a small writing board with a clip attached at the top for holding papers.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clip-on</ent><br/
<hw>clip-on</hw> <pos>adj.</pos> <def>having a clip as the means of attachment; <as>as, <ex>clip-on</ex> earrings; a <ex>clip-on</ex> bow tie</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clipped</ent><br/
<hw>clipped</hw> <pos>adj.</pos> <sn>1.</sn> <def>trimmed with clippers; <as>as, a <ex>clipped</ex> hedge</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(music)</fld> <def>staccato; -- contrasted with <contr>legato</contr>.</def><br/
<syn><b>Syn. --</b> short.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>effectively concise.</def><br/
<syn><b>Syn. --</b> brief, crisp, curt, laconic, short, terse, to the point(predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clipper</ent><br/
<hw>Clip"per</hw> <pr>(kl<icr/p"p<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who clips; specifically, one who clips off the edges of coins.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The value is pared off from it into the <qex>clipper's</qex> pocket.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A machine for clipping hair, esp. the hair of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A vessel with a sharp bow, built with a fast hull and tall sails, rigged for fast sailing, and used in trade where the cargo capacity was less important than the speed; -- called also <altname>clipper ship</altname>.</def>  -- <wordforms><wf>Clip"per-built`</wf> <pr>(<?/)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name was first borne by <ldquo/Baltimore clippers<rdquo/ famous as privateers in the early wars of the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Electronics)</fld> <def>a circuit that limits the amplitude of a waveform.</def><br/
<syn><b>Syn. --</b> limiter.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><cs><col><b>Yankee Clipper</b></col>, <sd>(a)</sd> <cd>a clipper ship built in the United States.  See clipper{3}.</cd>  <sd>(b)</sd> <cd><person>Joe DiMaggio</person>; -- a nickname for the player who was a prominent member of the New York Yankees baseball team in the 1940's.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>clippers</ent><br/
<hw>clippers</hw> <pos>n.</pos> <sn>1.</sn> <def>a type of shears for cutting grass or shrubbery; <as>as, hedge <ex>clippers</ex></as>.</def><br/
<syn><b>Syn. --</b> clipper, hedge clippers.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a cutting device for cutting hair or finger nails; <as>as, nail <ex>clippers</ex></as>.</def><br/
<syn><b>Syn. --</b> clipper.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clipping</ent><br/
<hw>Clip"ping</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of embracing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of cutting off, curtailing, or diminishing; the practice of clipping the edges of coins.</def>    <note>This practise was common when precious metals such as silver or gold were used in commonly circulated major coins, such as the dime, quarter, and higher denominations; scoundrels would remove small slivers of precious metal from the edges of many coins, eventually accumulating enough precious metal to be worth a significant sum, while passing on the clipped coins at their nominal values.  After most governments discontinued coinage in silver and gold in the late 1900's, the practice became obsolete.  The serrations, or milling, at the edges of coins was introduced to defeat the practice by making the result of clipping evident.  Many coins continued to be made with milled edges even after the practice of clipping was rendered pointless by use of non-precious metals in coinage.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q><qex>clipping</qex> by Englishmen is robbing the honest man who receives clipped money.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is clipped off or out of something; a piece separated by clipping; <as>as, newspaper <ex>clippings</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Football)</fld>  <def>The act of hitting a player from behind, for the purpose of blocking.  It is illegal in football because it can lead to injury to the blocked player, who cannot anticipate the action.  A penalty of 10 yards or more may be assessed against the team of the offending player.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clique</ent><br/
||<hw>Clique</hw> <pr>(kl<emac/k)</pr>, <pos>n.</pos> <ety>[F., fr. OF. <ets>cliquer</ets> to click.  See <er>Click</er>, <pos>v. i.</pos>]</ety> <def>A narrow circle of persons associated by common interests or for the accomplishment of a common purpose; -- generally used in a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clique</ent><br/
<hw>Clique</hw>, <pos>v. i.</pos> <def>To To associate together in a clannish way; to act with others secretly to gain a desired end; to plot; -- used with <ptcl>together</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cliquish</ent><br/
<hw>cli"quish</hw> <pr>(kl<emac/"k<icr/sh)</pr>, <pos>a.</pos> <def>Of or pertaining to a clique; disposed to from cliques; exclusive in spirit.</def> -- <wordforms><wf>Cli"*quish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>cliquism</ent><br/
<hw>cli"quism</hw> <pr>(kl<emac/"k<icr/z'm)</pr>, <pos>n.</pos> <def>The tendency to associate in cliques; the spirit of cliques.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clit</ent><br/
<hw>clit</hw> <pos>n.</pos> <def>Same as <ex>clitoris</ex>.</def> <mark>[Vulgar slang]</mark><br/
<syn><b>Syn. --</b> clitoris, button.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clitellus</ent><br/
||<hw>cli*tel"lus</hw> <pr>(kl<icr/*t<ecr/l"l<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., prob. fr. L. <ets>clitellae</ets> a packsadle.]</ety> <fld>(Zool.)</fld> <def>A thickened glandular portion of the body of the adult earthworm, consisting of several united segments modified for reproductive purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clitocybe</ent><br/
<hw>Clitocybe</hw> <pos>prop. n.</pos> <def>a genus of white-spored agarics with flat or funnel-shaped cap and elastic stem.</def><br/
<syn><b>Syn. --</b> genus <gen>Clitocybe</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clitoric</ent><br/
<ent>clitoral</ent><br/
<mhw><hw>clitoral</hw> <hw>clitoric</hw></mhw> <pos>adj.</pos> <def>Of or pertaining to the <er>clitoris</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clitoria</ent><br/
<hw>Clitoria</hw> <pos>n.</pos> <def>a genus of tropical shrubs or vines having pinnate leaves and large axillary flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Clitoria</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clitoris</ent><br/
||<hw>Cli"to*ris</hw> <pr>(kl<imac/"t<osl/*r<icr/s <it>or</it>  kl<icr/t"<osl/*r<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kleitori`s</grk>, fr. <grk>klei`ein</grk> to shut up.  It is concealed by the <ets>labia pudendi</ets>.]</ety> <fld>(Anat.)</fld> <def>A small organ at the upper part of the vulva in females, homologous to the penis in the male.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clitter</ent><br/
<hw>clitter</hw> <pos>v. i.</pos> <def>to make a shrill creaking noise by rubbing together special bodily structures, as of male insects such as crickets or grasshoppers.</def><br/
<syn><b>Syn. --</b> stridulate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clivers</ent><br/
<hw>Cliv"ers</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>See <er>Cleavers</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clivity</ent><br/
<hw>Cliv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Clivities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>clivus</ets> hill.]</ety> <def>Inclination; ascent or descent; a gradient.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloaca</ent><br/
||<hw>Clo"a"ca</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cloac<ae/</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <def>A sewer; <as>as, the <ex>Cloaca Maxima</ex> of Rome</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A privy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>The common chamber into which the intestinal, urinary, and generative canals discharge in birds, reptiles, amphibians, and many fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloacal</ent><br/
<hw>Clo*a"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a cloaca.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloak</ent><br/
<hw>Cloak</hw> <pr>(kl<omac/k; 110)</pr>, <pos>n.</pos> <ety>[Of. <ets>cloque</ets> cloak (from the bell-like shape), bell, F. <ets>cloche</ets> bell; perh. of Celtic origin and the same word as E. <ets>clock</ets>. See 1st <er>Clock</er>.]</ety> <sn>1.</sn> <def>A loose outer garment, extending from the neck downwards, and commonly without sleeves. It is longer than a cape, and is worn both by men and by women.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which conceals; a disguise or pretext; an excuse; a fair pretense; a mask; a cover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No man is esteemed any ways considerable for policy who wears religion otherwise than as a <qex>cloak</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cloak bag</b></col>, <cd>a bag in which a cloak or other clothes are carried; a portmanteau.</cd>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloak</ent><br/
<hw>Cloak</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cloaked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cloaking</conjf>.]</vmorph> <def>To cover with, or as with, a cloak; hence, to hide or conceal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now glooming sadly, so to <qex>cloak</qex> her matter.</q> <rj><qau>Spenser.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Palliate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloakedly</ent><br/
<hw>Cloak"ed*ly</hw>, <pos>adv.</pos> <def>In a concealed manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloaking</ent><br/
<hw>Cloak"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of covering with a cloak; the act of concealing anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To take heed of their dissemblings and <qex>cloakings</qex>.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The material of which of which cloaks are made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloakroom</ent><br/
<hw>Cloak"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A room, attached to any place of public resort, where cloaks, overcoats, etc., may be deposited for a time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clobber</ent><br/
<hw>clobber</hw> <pos>n.</pos> <def>personal possessions; -- an informal term; <as>as, did you take all your <ex>clobber</ex>?</as>.</def><br/
<syn><b>Syn. --</b> stuff.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cloche</ent><br/
<hw>Cloche</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prop., bell.]</ety> <fld>(A<eum/ronautics)</fld> <def>An apparatus used in controlling certain kinds of a<eum/roplanes, and consisting principally of a steering column mounted with a universal joint at the base, which is bellshaped and has attached to it the cables for controlling the wing-warping devices, elevator planes, and the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cloche</ent><br/
<hw>cloche</hw> <pos>n.</pos> <sn>1.</sn> <def>a woman's close-fitting helmetlike hat.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a low transparent cover put over young plants to protect them from cold.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clock</ent><br/
<hw>Clock</hw> <pr>(kl<ocr/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>clucge</ets> bell; akin to D. <ets>klok</ets> clock, bell, G. <ets>glocke</ets>, Dan. <ets>klokke</ets>, Sw. <ets>klocka</ets>, Icel. <ets>klukka</ets> bell, LL. <ets>clocca</ets>, <ets>cloca</ets> (whence F. <ets>cloche</ets>); al perh. of Celtic origin; cf. Ir. & Gael. <ets>clog</ets> bell, clock, W. <ets>cloch</ets> bell. Cf. <er>Cloak</er>.]</ety> <sn>1.</sn> <def>A machine for measuring time, indicating the hour and other divisions; in ordinary mechanical clocks for domestic or office use the time is indicated on a typically circular <part>face</part> or <part>dial plate</part> containing two <part>hands</part>, pointing to numbers engraved on the periphery of the face, thus showing the hours and minutes. The works of a mechanical clock are moved by a weight or a spring, and it is often so constructed as to tell the hour by the stroke of a hammer on a bell. In electrical or electronic clocks, the time may be indicated, as on a mechanical clock, by hands, but may also be indicated by direct <it>digital</it> readout, with the hours and minutes in normal Arabic numerals.  The readout using hands is often called <it>analog</it> to distinguish it from the <it>digital</it> readout.  Some clocks also indicate the seconds. Clocks are not adapted, like the watch, to be carried on the person.  Specialized clocks, such as <stype>atomic clocks</stype>, may be constructed on different principles, and may have a very high precision for use in scientific observations.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A watch, esp. one that strikes.</def> <mark>[Obs.]</mark>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The striking of a clock.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A figure or figured work on the ankle or side of a stocking.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The phrases <xex>what o'clock</xex>? <xex>it is nine o'clock</xex>, etc., are contracted from <xex>what of the clock</xex>? <xex>it is nine of the clock</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Alarm clock</b></col>. <cd>See under <er>Alarm</er>.</cd> -- <col><b>Astronomical clock</b></col>. <sd>(a)</sd> <cd>A clock of superior construction, with a compensating pendulum, etc., to measure time with great accuracy, for use in astronomical observatories; -- called a <xex>regulator</xex> when used by watchmakers as a standard for regulating timepieces.</cd> <sd>(b)</sd> <cd>A clock with mechanism for indicating certain astronomical phenomena, as the phases of the moon, position of the sun in the ecliptic, equation of time, etc.</cd> -- <col><b>Electric clock</b></col>. <sd>(a)</sd> <cd>A clock moved or regulated by electricity or electro-magnetism.</cd> <sd>(b)</sd> <cd>A clock connected with an electro-magnetic recording apparatus.</cd> -- <col><b>Ship's clock</b></col> <fld>(Naut.)</fld>, <cd>a clock arranged to strike from one to eight strokes, at half hourly intervals, marking the divisions of the ship's watches.</cd> -- <col><b>Sidereal clock</b></col>, <cd>an astronomical clock regulated to keep sidereal time.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clock</ent><br/
<hw>Clock</hw> <pr>(kl<ocr/k)</pr>, <pos>v. t.</pos> <def>To ornament with figured work, as the side of a stocking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clock</ent><br/
<hw>Clock</hw>, <pos>v. t. & i.</pos> <def>To call, as a hen. See <er>Cluck</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clock</ent><br/
<hw>Clock</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large beetle, esp. the European dung beetle (<spn>Scarab<ae/us stercorarius</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clocking</ent><br/
<hw>clocking</hw> <pos>n.</pos> <def>the time taken to traverse a measured course; <as>as, it was a world record <ex>clocking</ex></as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clocklike</ent><br/
<hw>Clock"like`</hw> <pr>(kl<ocr/k"l<imac/k`)</pr>, <pos>a.</pos> <def>Like a clock or like clockwork; mechanical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their services are <qex>clocklike</qex>, to be set<br/
Backward and forward at their lord's command.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clocks</ent><br/
<hw>clocks</hw> <pos>n.</pos> <sn>1.</sn> <def>European weed naturalized in the southwestern U. S. and Mexico (<spn>Erodium cicutarium</spn>), having reddish decumbent stems with small fernlike leaves and small deep reddish-lavender flowers followed by slender pinlike fruits that stick straight up; it is often grown for forage.</def><br/
<syn><b>Syn. --</b> redstem storksbill, alfilaria, alfileria, filaree, filaria, pin grass, pin clover, <spn>Erodium cicutarium</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clockwise</ent><br/
<hw>clockwise</hw> <pos>adj.</pos> <sn>1.</sn> <def> -- of rotatory motion.</def>  <stype>dextral</stype><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clockwise</ent><br/
<hw>clock"wise`</hw> <pr>(?)</pr>, <pos>a. & adv.</pos> <def>in the same direction as the hands of a clock rotate, as viewed from in front of the clock face; -- said of that direction of a rotation about an axis, or about a point in a plane, which is ordinarily reckoned negative.  Also said of the direction of a spiral, in which case the term <altname>right-handed</altname> is more common.  Opposite of <ant>counterclockwise</ant>, and <ant>left-handed</ant>.</def><br/
<syn><b>Syn. --</b> right-handed, dextrorotary, dextrorotatory.</syn><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clockwork</ent><br/
<hw>Clock"work`</hw> <pr>(-w<ucir/rk`)</pr>, <pos>n.</pos> <def>The machinery of a clock, or machinery resembling that of a clock; machinery which produces regularity of movement.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 267 --></p>

<p><ent>Clod</ent><br/
<hw>Clod</hw> <pr>(kl<ocr/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>clodde</ets>, latter form of <ets>clot</ets>. See <er>Clot</er>.]</ety> <sn>1.</sn> <def>A lump or mass, especially of earth, turf, or clay.</def> <ldquo/<xex>Clods</xex> of a slimy substance.<rdquo/ <au>Carew.</au> <ldquo/<xex>Clods</xex> of iron and brass.<rdquo/ <au>Milton.</au> <ldquo/<xex>Clods</xex> of blood.<rdquo/ <au>E. Fairfax.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>The earth that casteth up from the plow a great <qex>clod</qex>, is not so good as that which casteth up a smaller <qex>clod</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The ground; the earth; a spot of earth or turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>clod</qex><br/
<qex>Where once their sultan's horse has trod</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is earthy and of little relative value, as the body of man in comparison with the soul.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This cold <qex>clod</qex> of clay which we carry about with us.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A dull, gross, stupid fellow; a dolt</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A part of the shoulder of a beef creature, or of the neck piece near the shoulder. See <xex>Illust.</xex> of <er>Beef</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clod</ent><br/
<hw>Clod</hw> <pr>(kl<ocr/d)</pr>, <pos>v. i.</pos> <def>To collect into clods, or into a thick mass; to coagulate; to clot; <as>as, <ex>clodded</ex> gore</as>. See <er>Clot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Clodded</qex> in lumps of clay.</q> <rj><qau>G. Fletcher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clod</ent><br/
<hw>Clod</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To pelt with clods.</def>  <rj><au>Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To throw violently; to hurl.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloddish</ent><br/
<hw>Clod"dish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling clods; gross; low; stupid; boorish.</def>  <rj><au>Hawthorne.</au></rj></p>

<p>-- <wordforms><wf>Clod"dish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloddy</ent><br/
<hw>Clod"dy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of clods; full of clods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clodhopper</ent><br/
<hw>Clod"hop`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rude, rustic fellow.</def>
<-- 2. a shoe or boot of large and heavy contruction; -- often used as a work shoe. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clodhopping</ent><br/
<hw>Clod"hop`ping</hw>, <pos>a.</pos> <def>Boorish; rude.</def>  <rj><au>C. Bront<eacute/.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clodpate</ent><br/
<hw>Clod"pate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A blockhead; a dolt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clodpated</ent><br/
<hw>Clod"pat`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stupid; dull; doltish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clodpoll</ent><br/
<hw>Clod"poll`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Clod</ets> + <ets>poll</ets> head.]</ety> <def>A stupid fellow; a dolt.</def> <altsp>[Written also <asp>clodpole</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloff</ent><br/
<hw>Cloff</hw> <pr>(kl<ocr/f; 115)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>Formerly an allowance of two pounds in every three hundred weight after the tare and tret are subtracted; now used only in a general sense, of small deductions from the original weight.</def> <altsp>[Written also <asp>clough</asp>.]</altsp>  <rj><au>McCulloch.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clog</ent><br/
<hw>Clog</hw> <pr>(kl<ocr/g)</pr>, <pos>n.</pos> <ety>[OE. <ets>clogge</ets> clog, Scot. <ets>clag</ets>, <pos>n.</pos>, a clot, <pos>v.</pos>, to to obstruct, cover with mud or anything adhesive; prob. of the same origin as E. <ets>clay</ets>.]</ety> <sn>1.</sn> <def>That which hinders or impedes motion; hence, an encumbrance, restraint, or impediment, of any kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the ancient, honest, juridical principles and institutions of England are so many <qex>clogs</qex> to check and retard the headlong course of violence and opression.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A weight, as a log or block of wood, attached to a man or an animal to hinder motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a dog . . . but chance breaks loose,<br/
And quits his <qex>clog</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>clog</qex> of lead was round my feet.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A shoe, or sandal, intended to protect the feet from wet, or to increase the apparent stature, and having, therefore, a very thick sole. Cf. <er>Chopine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In France the peasantry goes barefoot; and the middle sort . . . makes use of wooden <qex>clogs</qex>.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clog almanac</b></col>, <cd>a primitive kind of almanac or calendar, formerly used in England, made by cutting notches and figures on the four edges of a clog, or square piece of wood, brass, or bone; -- called also a <altname>Runic staff</altname>, from the Runic characters used in the numerical notation.</cd> -- <col><b>Clog dance</b></col>, <cd>a dance performed by a person wearing clogs, or thick-soled shoes.</cd> -- <wordforms><col><b>Clog dancer</b></col>.</wordforms></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clog</ent><br/
<hw>Clog</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clogged</conjf> <pr>(kl<ocr/gd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clogging</conjf>.]</vmorph> <sn>1.</sn> <def>To encumber or load, especially with something that impedes motion; to hamper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The winds of birds were <qex>clogged</qex> with ace and snow.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To obstruct so as to hinder motion in or through; to choke up; <as>as, to <ex>clog</ex> a tube or a channel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To burden; to trammel; to embarrass; to perplex.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The commodities are <qex>clogged</qex> with impositions.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You 'll rue the time<br/
That <qex>clogs</qex> me with this answer.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Impede; hinder; obstruct; embarrass; burden; restrain; restrict.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clog</ent><br/
<hw>Clog</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To become clogged; to become loaded or encumbered, as with extraneous matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In working through the bone, the teeth of the saw will begin to <qex>clog</qex>.</q> <rj><qau>S. Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To coalesce or adhere; to unite in a mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Move it sometimes with a broom, that the seeds <qex>clog</qex> not together.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clogged</ent><br/
<hw>clogged</hw> <pos>adj.</pos> <sn>1.</sn>  <def>obstructed so as to prevent or hinder flow of a fluid; -- of conduits; <as>as, <ex>clogged</ex> pipes</as>; <ex>clogged</ex> arteries.</def><br/
<syn><b>Syn. --</b> choked.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>filled beyond capacity (with people or vehicles), so as to retard movement; <as>as, The store aisles were <ex>clogged</ex> with shoppers on the day before Christmas.; The <ex>clogged</ex> highways made me miss my appointment.</as>.</def><br/
<syn><b>Syn. --</b> choked.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Clogginess</ent><br/
<hw>Clog"gi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being clogged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clogging</ent><br/
<hw>Clog"ging</hw>, <pos>n.</pos> <def>Anything which clogs.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloggy</ent><br/
<hw>Clog"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Clogging, or having power to clog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloisonne</ent><br/
||<hw>Cloi`son*n<eacute/</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., partitioned, fr. <ets>cloison</ets> a partition.]</ety> <def>Inlaid between partitions: -- said of enamel when the lines which divide the different patches of fields are composed of a kind of metal wire secured to the ground; as distinguished from <contr>champlev<eacute/</contr> enamel, in which the ground is engraved or scooped out to receive the enamel.</def>  <rj><au>S. Wells Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloister</ent><br/
<hw>Clois"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cloistre</ets>, F. <ets>clo<icir/tre</ets>, L. <ets>claustrum</ets>, pl. <ets>claustra</ets>, bar, bolt, bounds, fr. <ets>claudere</ets>, <ets>clausum</ets>, to close. See <er>Close</er>, <pos>v. t.</pos>, and cf. <er>Claustral</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An inclosed place.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A covered passage or ambulatory on one side of a court;</def> (<pluf>pl.</pluf>) <def>the series of such passages on the different sides of any court, esp. that of a monastery or a college.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But let my due feet never fail<br/
To walk the studious <qex>cloister's</qex> pale.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A monastic establishment; a place for retirement from the world for religious duties.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fitter for a <qex>cloister</qex> than a crown.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cloister garth</b></col> <fld>(Arch.)</fld>, <cd>the garden or open part of a court inclosed by the cloisters.</cd></cs></p>

<p><syn><b>Syn.</b> -- <er>Cloister</er>, <er>Monastery</er>, <er>Nunnery</er>, <er>Convent</er>, <er>Abbey</er>, <er>Priory</er>.</syn> <usage> <xex>Cloister</xex> and <xex>convent</xex> are generic terms, and denote a place of seclusion from the world for persons who devote their lives to religious purposes. They differ is that the distinctive idea of <xex>cloister</xex> is that of seclusion from the world, that of <xex>convent</xex>, community of living. Both terms denote houses for recluses of either sex. A <xex>cloister</xex> or <xex>convent</xex> for <xex>monks</xex> is called a <xex>monastery</xex>; for <xex>nuns</xex>, a <xex>nunnery</xex>. An <xex>abbey</xex> is a convent or monastic institution governed by an abbot or an abbess; a <xex>priory</xex> is one governed by a prior or a prioress, and is usually affiliated to an abbey.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloister</ent><br/
<hw>Clois"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cloistered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cloistering</conjf>.]</vmorph> <def>To confine in, or as in, a cloister; to seclude from the world; to immure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>None among them are thought worthy to be styled religious persons but those that <qex>cloister</qex> themselves up in a monastery.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloisteral</ent><br/
<hw>Clois"ter*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cloistral.</def> <mark>[Obs.]</mark>  <rj><au>I. Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloistered</ent><br/
<hw>Clois"tered</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Dwelling in cloisters; solitary.</def> <ldquo/<xex>Cloistered</xex> friars and vestal nuns.<rdquo/  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>cloistered</qex> state let selfish sages dwell,<br/
Proud that their heart is narrow as their cell.</q> <rj><qau>Shenstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Furnished with cloisters.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloisterer</ent><br/
<hw>Clois"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>cloistier</ets>.]</ety> <def>One belonging to, or living in, a cloister; a recluse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloistral</ent><br/
<hw>Clois"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or confined in, a cloister; recluse.</def> <altsp>[Written also <asp>cloisteral</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Best become a <qex>cloistral</qex> exercise.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloistress</ent><br/
<hw>Clois"tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A nun.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloke</ent><br/
<hw>Cloke</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See Cloak.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clomben</ent><br/
<ent>Clomb</ent><br/
<mhw>{ <hw>Clomb</hw> <pr>(?)</pr>, <hw>Clomb"en</hw> <pr>(?)</pr>, }</mhw> <def><pos>imp. & p. p.</pos> of <er>Climb</er> (for <xex>climbed</xex>).</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sonne, he sayde, is <qex>clomben</qex> up on hevene.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clomp</ent><br/
<hw>Clomp</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Clamp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clone</ent><br/
<hw>clone</hw> <pr>(kl<omac/n)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Biol.)</fld> <def>a group of organisms derived from a single individual by some kind of asexual reproduction; -- used mostly of microorganisms such as bacteria and yeast.</def><br/
<syn><b>Syn. --</b> clon.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>an individual organism containing a genetic complement identical to that of another organism, produced by using the genetic material from the second animal in a non-sexual reproduction process.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>something virtually identical to another object.</def><br/
[<source>PJC</source>]</p>

<p><ent>clone</ent><br/
<hw>clone</hw> <pos>v. t.</pos> <sn>1.</sn> <fld>(Biol.)</fld> <def>to make a clone from; to make identical copies of an organism by a non-sexual process of reproduction.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Microbiol.)</fld> <def>to grow colonies of a microorganism by spreading a suspension of the microorganism onto a solid growth medium (such as in a Petri dish), at a concentration such that individual colonies will grow from single cells sufficiently well separated from other colonies so that pure cultures derived from a single organism can be isolated.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <fld>(Biochem.)</fld> <def>to make large quantities of a segment of DNA by inserting it, using biochemical techniques, into the DNA of a microorganism, and growing that microorganism in large numbers; <as>as, to <ex>clone</ex> the gene for growth hormone</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clong</ent><br/
<hw>Clong</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Cling</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clonic</ent><br/
<hw>Clon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>klo`nos</grk> a violent, confused motion; cf. F. <ets>clonique</ets>.]</ety> <fld>(Med.)</fld> <def>Having an irregular, convulsive motion.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clonic spasm</b></col>. <fld>(Med.)</fld> <cd>See under <er>Spasm</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clonus</ent><br/
||<hw>Clo"nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>klo`nos</grk> violent, confused motion.]</ety> <fld>(Med.)</fld> <def>A series of muscular contractions due to sudden stretching of the muscle, -- a sign of certain neuropathies.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cloom</ent><br/
<hw>Cloom</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[A variant of <ets>clam</ets> to clog.]</ety> <def>To close with glutinous matter.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloop</ent><br/
<hw>Cloop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[An onomatop<oe/ia.]</ety> <def>The sound made when a cork is forcibly drawn from a bottle.</def> <ldquo/The <xex>cloop</xex> of a cork wrenched from a bottle.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloot</ent><br/
<hw>Cloot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. dial. <ets>kleuzen</ets> to split.]</ety> <fld>(Scot. & Dial. Eng.)</fld> <sn>1.</sn> <def>One of the divisions of a cleft hoof, as in the ox; also, the whole hoof.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The Devil; Clootie; -- usually in the <pluf>pl.</pluf></def>  <rj><au>Burns.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clootie</ent><br/
<hw>Cloot"ie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Scot. & Dial. Eng.)</fld> <sn>1.</sn>  <def>A little hoof.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The Devil.</def> <ldquo/Satan, Nick, or <xex>Clootie</xex>.<rdquo/  <rj><au>Burns.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>clorella</ent><br/
<hw>clorella</hw> <pos>n.</pos> <def>any alga of the genus <gen>Chlorella</gen>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clorox</ent><br/
<hw>Clorox</hw> <pos>n.</pos> <ety>[a trademark of the Clorox Co.]</ety> <def>a commercial bleaching agent, consisting of a 5.25% aqueous solution of sodium hypochlorite (<chform>NaOCl</chform>).  Other similar solutions of sodium hypochlorite are sometimes referred to as <ex>clorox</ex>.</def> <mark>[trademark]</mark><br/
<syn><b>Syn. --</b> bleach.</syn>
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Close</ent><br/
<hw>Close</hw> <pr>(kl<omac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Closed</conjf> <pr>(kl<omac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Closing</conjf>.]</vmorph> <ety>[From OF. & F. <ets>clos</ets>, p. p. of <ets>clore</ets> to close, fr. L. <ets>claudere</ets>; akin to G. <ets>schliessen</ets> to shut, and to E. <ets>clot</ets>, <ets>cloister</ets>, <ets>clavicle</ets>, <ets>conclude</ets>, <ets>sluice</ets>. Cf. <er>Clause</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To stop, or fill up, as an opening; to shut; <as>as, to <ex>close</ex> the eyes; to <ex>close</ex> a door</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring together the parts of; to consolidate; <as>as, to <ex>close</ex> the ranks of an army; -- often used with <ex>up</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bring to an end or period; to conclude; to complete; to finish; to end; to consummate; <as>as, to <ex>close</ex> a bargain; to <ex>close</ex> a course of instruction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One frugal supper did our studies <qex>close</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To come or gather around; to inclose; to encompass; to confine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The depth <qex>closed</qex> me round about.</q> <rj><qau>Jonah ii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But now thou dost thyself immure and <qex>close</qex><br/
In some one corner of a feeble heart.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A closed sea</b></col>, <cd>a sea within the jurisdiction of some particular nation, which controls its navigation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close</ent><br/
<hw>Close</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To come together; to unite or coalesce, as the parts of a wound, or parts separated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What deep wounds ever <qex>closed</qex> without a scar?</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To end, terminate, or come to a period; <as>as, the debate <ex>closed</ex> at six o'clock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To grapple; to engage in hand-to-hand fight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They boldly <qex>closed</qex> in a hand-to-hand contest.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To close on</b></col> <it>or</it> <col><b>To close upon</b></col></mcol>, <cd>to come to a mutual agreement; to agree on or join in.</cd> <ldquo/Would induce France and Holland to <xex>close upon</xex> some measures between them to our disadvantage.<rdquo/ <au>Sir W. Temple.</au> -- <col><b>To close with</b></col>. <sd>(a)</sd> <cd>To accede to; to consent or agree to; <as>as, to <ex>close with</ex> the terms proposed</as>.</cd> <sd>(b)</sd> <cd>To make an agreement with.</cd> -- <col><b>To close with the land</b></col> <fld>(Naut.)</fld>, <cd>to approach the land.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close</ent><br/
<hw>Close</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The manner of shutting; the union of parts; junction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The doors of plank were; their <qex>close</qex> exquisite.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conclusion; cessation; ending; end.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> His long and troubled life was drawing to a <qex>close</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A grapple in wrestling.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The conclusion of a strain of music; cadence.</def> <sd>(b)</sd> <def>A double bar marking the end.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At every <qex>close</qex> she made, the attending throng<br/
Replied, and bore the burden of the song.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Conclusion; termination; cessation; end; ending; extremity; extreme.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close</ent><br/
<hw>Close</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[OF. & F. <ets>clos</ets> an inclosure, fr. <ets>clos</ets>, p. p. of <ets>clore</ets>. See <er>Close</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>An inclosed place; especially, a small field or piece of land surrounded by a wall, hedge, or fence of any kind; -- specifically, the precinct of a cathedral or abbey.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Closes</qex> surrounded by the venerable abodes of deans and canons.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A narrow passage leading from a street to a court, and the houses within.</def> <mark>[Eng.]</mark>  <rj><au>Halliwell</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>The interest which one may have in a piece of ground, even though it is not inclosed.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close</ent><br/
<hw>Close</hw> <pr>(kl<omac/s)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Closer</adjf> <pr>(kl<omac/"s<etil/r)</pr>; <pos>superl.</pos> <adjf>Closest</adjf>.]</amorph> <ety>[Of. & F. <ets>clos</ets>, p. p. of <ets>clore</ets>. See <er>Close</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Shut fast; closed; tight; <as>as, a <ex>close</ex> box</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From a <qex>close</qex> bower this dainty music flowed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Narrow; confined; <as>as, a <ex>close</ex> alley; <ex>close</ex> quarters</as>.</def> <ldquo/A <xex>close</xex> prison.<rdquo/  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Oppressive; without motion or ventilation; causing a feeling of lassitude; -- said of the air, weather, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the rooms be low-roofed, or full of windows and doors, the one maketh the air <qex>close</qex>, . . . and the other maketh it exceeding unequal.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Strictly confined; carefully quarded; <as>as, a <ex>close</ex> prisoner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Out of the way observation; secluded; secret; hidden.</def> <ldquo/He yet kept himself <xex>close</xex> because of Saul.<rdquo/  <rj><au>1 Chron. xii. 1</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Her <qex>close</qex> intent.<rdquo/</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Disposed to keep secrets; secretive; reticent.</def> <ldquo/For secrecy, no lady <xex>closer</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Having the parts near each other; dense; solid; compact; as applied to bodies; viscous; tenacious; not volatile, as applied to liquids.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The golden globe being put into a press, . . . the water made itself way through the pores of that very <qex>close</qex> metal.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Concise; to the point; <as>as, <ex>close</ex> reasoning</as>.</def> <ldquo/Where the original is <xex>close</xex> no version can reach it in the same compass.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Adjoining; near; either in space; time, or thought; -- often followed by <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Plant the spring crocuses <qex>close</qex> to a wall.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The thought of the Man of sorrows seemed a very <qex>close</qex> thing -- not a faint hearsay.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Short; <as>as, to cut grass or hair <ex>close</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Intimate; familiar; confidential.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>League with you I seek<br/
And mutual amity, so strait, so <qex>close</qex>,<br/
That I with you must dwell, or you with me.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Nearly equal; almost evenly balanced; <as>as, a <ex>close</ex> vote</as>.</def> <ldquo/A <xex>close</xex> contest.<rdquo/  <rj><au>Prescott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>Difficult to obtain; <as>as, money is <ex>close</ex></as>.</def>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>Parsimonious; stingy.</def> <ldquo/A crusty old fellow, as <xex>close</xex> as a vise.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>Adhering strictly to a standard or original; exact; strict; <as>as, a <ex>close</ex> translation</as>.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <def>Accurate; careful; precise; also, attentive; undeviating; strict; not wandering; <as>as, a <ex>close</ex> observer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <fld>(Phon.)</fld> <def>Uttered with a relatively contracted opening of the mouth, as certain sounds of <xex>e</xex> and <xex>o</xex> in French, Italian, and German; -- opposed to <xex>open</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Close borough</b></col>. <cd>See under <er>Borough</er>.</cd> -- <col><b>Close breeding</b></col>. <cd>See under <er>Breeding</er>.</cd> -- <col><b>Close communion</b></col>, <cd>communion in the Lord's supper, restricted to those who have received baptism by immersion.</cd> -- <col><b>Close corporation</b></col>, <cd>a body or corporation which fills its own vacancies.</cd> -- <col><b>Close fertilization</b></col>. <fld>(Bot.)</fld> <cd>See <er>Fertilization</er>.</cd> -- <col><b>Close harmony</b></col> <fld>(Mus.)</fld>, <cd>compact harmony, in which the tones composing each chord are not widely distributed over several octaves.</cd> -- <col><b>Close time</b></col>, <cd>a fixed period during which killing game or catching certain fish is prohibited by law.</cd> -- <col><b>Close vowel</b></col> <fld>(Pron.)</fld>, <cd>a vowel which is pronounced with a diminished aperture of the lips, or with contraction of the cavity of the mouth.</cd> -- <col><b>Close to the wind</b></col> <fld>(Naut.)</fld>, <cd>directed as nearly to the point from which the wind blows as it is possible to sail; closehauled; -- said of a vessel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close</ent><br/
<hw>Close</hw> <pr>(kl<omac/s)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a close manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Secretly; darkly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A wondrous vision which did <qex>close</qex> imply<br/
The course of all her fortune and posterity.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close-banded</ent><br/
<hw>Close"-band`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Closely united.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close-barred</ent><br/
<hw>Close"-barred`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Firmly barred or closed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close-bodied</ent><br/
<hw>Close"-bod`ied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fitting the body exactly; setting close, as a garment.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>closed</ent><br/
<hw>closed</hw> <pos>adj.</pos> <sn>1.</sn> <def>having an opening obstructed.</def> [Narrower terms: <stype>blind</stype>] <see>Also See: <er>obstructed</er>, <er>sealed</er>, <er>shut</er>, <er>unopen</er>, <er>closed</er>.</see>  Antonym: <ant>open</ant>.<br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>of a curve or surface: having no end points or boundary curves; of a set: having members that can be produced by a specific operation on other members of the same set; of an interval: containing both its endpoints.</def> <ant>open</ant><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>Being in a position to obstruct an opening; -- especially of doors.</def> [Narrower terms: <stype>fastened, latched</stype>] <see>Also See: <er>closed</er>.</see>  Antonym: <ant>open</ant>.<br/
<syn><b>Syn. --</b> shut, unopen.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>having skin drawn so as to obstruct the opening; -- used of mouth or eyes.  Opposite of <ant>open</ant>.</def> <illu>he sat quietly with <ex>closed</ex> eyes</illu> [Narrower terms: <stype>blinking, winking</stype>; <stype>compressed, tight</stype>; <stype>squinched, squinting</stype>]<br/
<syn><b>Syn. --</b> shut.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn> <def>requiring union membership; -- of a workplace; <as>as, a <ex>closed</ex> shop</as>.</def> <mark>[prenominal]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn> <def>closed with shutters.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>7.</sn> <def>hidden from the public; <as>as, a <ex>closed</ex> ballot</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>8.</sn> <def>not open to the general public; <as>as, a <ex>closed</ex> meeting</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>9.</sn>  <def>unsympathetic; -- of a person's attitude.</def> <illu>a <ex>closed</ex> mind unreceptive to new ideas</illu><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>10.</sn>  <def>surrounded by walls.</def> <illu>a <ex>closed</ex> porch</illu><br/
<syn><b>Syn. --</b> closed in(predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>11.</sn>  <def>made compact by bending or doubling over; <as>as, a <ex>closed</ex> map</as>.</def><br/
<syn><b>Syn. --</b> folded.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>12.</sn> <def>closed or fastened with or as if with buttons.</def> [Narrower terms: <stype>buttoned (vs. unbuttoned)</stype>]<br/
[<source>WordNet 1.5</source>]</p>

<p><sn>13.</sn>  <def>not engaged in activity; -- of an organization or business establishment.</def> <illu>the airport is <ex>closed</ex> because of the weather</illu>; <illu>the many <ex>closed</ex> shops and factories made the town look deserted</illu><br/
<syn><b>Syn. --</b> shut down.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>closed-chain</ent><br/
<hw>closed-chain</hw> <pos>adj.</pos> <fld>(Chem.)</fld>  <def>having atoms linked by bonds which form a ring.</def><br/
<syn><b>Syn. --</b> closed-ring.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>closed-minded</ent><br/
<hw>closed-minded</hw> <pos>adj.</pos>  <def>not ready to receive new ideas.</def><br/
<syn><b>Syn. --</b> close-minded.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>closedown</ent><br/
<hw>close"down`</hw> <pos>n.</pos> <def>termination of operations; a shutdown.</def><br/
<syn><b>Syn. --</b> closure, closing, shutdown.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>closed-ring</ent><br/
<hw>closed-ring</hw> <pos>adj.</pos> <fld>(Chem.)</fld>  <def>same as <er>closed-chain</er>.</def><br/
<syn><b>Syn. --</b> closed-chain.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Close-fights</ent><br/
<hw>Close"-fights`</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Naut.)</fld> <def>Barriers with loopholes, formerly erected on the deck of a vessel to shelter the men in a close engagement with an enemy's boarders; -- called also <altname>close quarters</altname>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closefisted</ent><br/
<hw>Close"fist`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covetous; niggardly.</def>  <au>Bp. Berkeley.</au>  <ldquo/<qex>Closefisted</qex> contractors.<rdquo/  <au>Hawthorne.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>close-grained</ent><br/
<hw>close-grained</hw> <pos>adj.</pos> <def>dense or compact in structure or texture, as a wood composed of small-diameter cells.</def><br/
<syn><b>Syn. --</b> fine-grained.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Closehanded</ent><br/
<hw>Close"hand`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covetous; penurious; stingy; closefisted.</def>  -- <wordforms><wf>Close"hand`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>close-hauled</ent><br/
<ent>Closehauled</ent><br/
<mhw><hw>Close"hauled`</hw>, <hw>close-hauled</hw></mhw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Under way and moving as nearly as possible toward the direction from which the wind blows; having the sails trimmed for sailing as close to the wind as possible; -- said of a sailing vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>closely-knit</ent><br/
<ent>close-knit</ent><br/
<mhw><hw>close-knit</hw> <hw>closely-knit</hw></mhw> <pos>adj.</pos> <def>bound together by intimate ties, social, personal, cultural or political; <as>as, <ex>closely-knit</ex> little villages</as>; a <ex>close-knit</ex> family.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Closely</ent><br/
<hw>Close"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a close manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Secretly; privately.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That nought she did but wayle, and often steepe<br/
Her dainty couch with tears which<br/
<qex>closely</qex> she did weepe.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>closely-held</ent><br/
<hw>closely-held</hw> <pos>adj.</pos> <def>owned by a relatively few shareholders; -- of business organizations; <as>as, a <ex>closely-held</ex> corporation</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>close-minded</ent><br/
<hw>close-minded</hw> <pos>adj.</pos> <def>not ready to receive new ideas.</def><br/
<syn><b>Syn. --</b> closed-minded.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Closemouthed</ent><br/
<hw>Close"mouthed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cautious in speaking; secret; wary; uncommunicative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closen</ent><br/
<hw>Clos"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make close.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closeness</ent><br/
<hw>Close"ness</hw>, <pos>n.</pos> <def>The state of being close.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Half stifled by the <qex>closeness</qex> of the room.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We rise not against the piercing judgment of Augustus, nor the extreme caution or <qex>closeness</qex> of Tiberius.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An affectation of <qex>closeness</qex> and covetousness.</q> <rj><qau>Addison.</qau></rj></p>

<p><syn><b>Syn.</b> -- Narrowness; oppressiveness; strictness; secrecy; compactness; conciseness; nearness; intimacy; tightness; stinginess; literalness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>close-packed</ent><br/
<hw>close-packed</hw> <pos>adj.</pos> <def>grouped closely together.</def><br/
<syn><b>Syn. --</b> dense, tight.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Closer</ent><br/
<hw>Clos"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, closes; specifically, a boot closer. See under <er>Boot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A finisher; that which finishes or terminates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Masonry)</fld> <def>The last stone in a horizontal course, if of a less size than the others, or a piece of brick finishing a course.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closereefed</ent><br/
<hw>Close"reefed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Having all the reefs taken in; -- said of a sail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>close-set</ent><br/
<hw>close-set</hw> <pos>adj.</pos> <def>arranged close together; <as>as, <ex>close-set</ex> eyes</as>; <ex>close-set</ex> teeth.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Close-stool</ent><br/
<hw>Close"-stool`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A utensil to hold a chamber vessel, for the use of the sick and infirm. It is usually in the form of a box, with a seat and tight cover.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closet</ent><br/
<hw>Clos"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>closet</ets> little inclosure, dim. of <ets>clos</ets>. See <er>Close</er> an inclosure.]</ety> <sn>1.</sn> <def>A small room or apartment for retirement; a room for privacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A chair-lumbered <qex>closet</qex>, just twelve feet by nine.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When thou prayest, enter into thy <qex>closet</qex>.</q> <rj><qau>Matt. vi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small apartment, or recess in the side of a room, for household utensils, clothing, etc.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Closet sin</b></col>, <cd>sin commited in privacy.</cd> <au>Bp. Hall.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closet</ent><br/
<hw>Clos"et</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos><conjf>Closeted</conjf> <pos>p. pr. & vb. n.</pos> <conjf>Closeting</conjf>.]</vmorph> <sn>1.</sn> <def>To shut up in, or as in, a closet; to conceal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Bedlam's <qex>closeted</qex> and handcuffed charge.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make into a closet for a secret interview.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was to call a new legislature, to <qex>closet</qex> its members.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had been <qex>closeted</qex> with De Quadra.</q> <rj><qau>Froude.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Close-tongued</ent><br/
<hw>Close"-tongued`</hw> <pr>(<?/)</pr>, <pos>a.</pos> <def>Closemouthed; silent.</def> <ldquo/<qex>Close-tongued</qex> treason.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>closeup</ent><br/
<hw>close"up</hw> <pr>(kl<omac/s"<ucr/p)</pr>, <pos>n.</pos> <def>a photograph or other recorded image taken at close range, or using a telephoto lens so as to appear to have been taken at close range; -- used especially of images of faces in which the face fills all or most of the photograph.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><-- p. 268 --></p>

<p><ent>Closh</ent><br/
<hw>Closh</hw> <pr>(kl<ocr/sh)</pr>, <pos>n.</pos> <ety>[CF. F. <ets>clocher</ets> to limp, halt.]</ety> <def>A disease in the feet of cattle; laminitis.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Closh</ent><br/
<hw>Closh</hw>, <pos>n.</pos> <ety>[CF. D. <ets>klossen</ets> to play at bowls.]</ety> <def>The game of ninepins.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>closing</ent><br/
<hw>closing</hw> <pos>adj.</pos> <def>final or ending; terminal; <as>as, the <ex>closing</ex> stages of the election</as>; the <ex>closing</ex> weeks of the year; the <ex>closing</ex> scene of the film; <ex>closing</ex> remarks.  Opposite of <ant>opening</ant>.</def> [Narrower terms: <stype>year-end</stype>]<br/
[<source>WordNet 1.5</source>]</p>

<p><ent>closing</ent><br/
<hw>closing</hw> <pos>n.</pos> <sn>1.</sn> <def>the act of closing something.</def><br/
<syn><b>Syn. --</b> shutting.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the last section of a communication.</def><br/
<syn><b>Syn. --</b> conclusion, close, ending.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>termination of operations.</def><br/
<syn><b>Syn. --</b> closure, closedown, shutdown.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn> <def>a concluding action.</def><br/
<syn><b>Syn. --</b> completion, culmination, windup, mop up.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn> <fld>(Commerce)</fld> <def>the final action in a commercial transaction, especially the meeting between buyer and seller (and in some cases mortgagee), or their representatives, in a transaction for sale of real estate in which all documents are signed and all procedures carried out to complete the sale; -- called also <altname>real estate closing</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>clostridium</ent><br/
<hw>clostridium</hw> <pos>n.</pos> <plu><plw>clostridia</plw></plu> <sn>1.</sn> <def>spindle-shaped bacterial cell esp. one swollen at the center by an endospore.</def><br/
<syn><b>Syn. --</b> clostridia.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <mark>[capitalized]</mark>  <def>a genus of anaerobic, spore-forming motile bacteria of the family <fam>Bacillaceae</fam>, including several pathogenic species, such as the causative agents of <causes>gas gangrene</causes> (<styp><spn>Clostridium perfringens</spn></styp>), <causes>botulism</causes> (<stype><spn>Clostridium botulinum</spn></stype>), and <causes>tetanus</causes> (<styp><spn>Clostridium tetani</spn></styp>).</def><br/
[<source>PJC</source>]</p>

<p><ent>Closure</ent><br/
<hw>Clo"sure</hw> <pr>(kl<omac/"zh<usl/r; 135)</pr>, <pos>n.</pos> <ety>[Of. closure, L. <ets>clausura</ets>, fr. <ets>clauedere</ets> to shut. See <er>Close</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of shutting; a closing; <as>as, the <ex>closure</ex> of a chink</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which closes or shuts; that by which separate parts are fastened or closed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without a seal, wafer, or any <qex>closure</qex> whatever.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which incloses or confines; an inclosure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O thou bloody prison . . . <br/
Within the guilty <qex>closure</qex> of thy walls<br/
Richard the Second here was hacked to death.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A conclusion; an end.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Parliamentary Practice)</fld> <def>A method of putting an end to debate and securing an immediate vote upon a measure before a legislative body. It is similar in effect to <xex>the previous question</xex>. It was first introduced into the British House of Commons in 1882. The French word <xex>cl<ocir/ture</xex> was originally applied to this proceeding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Math.)</fld> <def>the property of being mathematically closed under some operation; -- said of sets.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <fld>(Math.)</fld> <def>the intersection of all closed sets containing the given set.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn> <fld>(Psychol.)</fld> <def>achievement of a sense of completeness and release from tension due to uncertainty; <as>as, the <ex>closure</ex> afforded by the funeral of a loved one</as>; also, the sense of completion thus achieved.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clot</ent><br/
<hw>Clot</hw> <pr>(kl<ocr/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>clot</ets>, <ets>clodde</ets>, clod; akin to D. <ets>kloot</ets> ball, G. <ets>kloss</ets> clod, dumpling, <ets>klotz</ets> block, Dan. <ets>klods</ets>, Sw. <ets>klot</ets> bowl, globe, <ets>klots</ets> block; cf. AS. <ets>cl<amac/te</ets> bur. Cf. <er>Clod</er>, <pos>n.</pos>, <er>Clutter</er> to clot.]</ety> <def>A concretion or coagulation; esp. a soft, slimy, coagulated mass, as of blood; a coagulum.</def> <ldquo/<xex>Clots</xex> of pory gore.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Doth bake the egg into <qex>clots</qex> as if it began to poach.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Clod</xex> and <xex>clot</xex> appear to be radically the same word, and are so used by early writers; but in present use <xex>clod</xex> is applied to a mass of earth or the like, and <xex>clot</xex> to a concretion or coagulation of soft matter.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clot</ent><br/
<hw>Clot</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clotted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Clotting</conjf>.]</vmorph> <def>To concrete, coagulate, or thicken, as soft or fluid matter by evaporation; to become a cot or clod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clot</ent><br/
<hw>Clot</hw>, <pos>v. t.</pos> <def>To form into a slimy mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clotbur</ent><br/
<hw>Clot"bur`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Clote</er>.]</ety> <sn>1.</sn> <def>The burdock.</def> <mark>[Prov. Engl.]</mark>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Same as <er>Cocklebur</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clote</ent><br/
<hw>Clote</hw> <pr>(kl<omac/t)</pr>, <pos>n.</pos> <ety>[AS. <ets>cl<amac/te</ets>: cf. G. <ets>klette</ets>.]</ety> <def>The common burdock; the clotbur.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloth</ent><br/
<hw>Cloth</hw> <pr>(kl<ocr/th; 115)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cloths</plw> <pr>(kl<ocr/<th/z; 115)</pr>, except in the sense of garments, when it is <plw>Clothes</plw> (kl<omac/thz <or/ kl<omac/z).</plu> <ety>[OE. <ets>clath</ets> cloth, AS. <ets>cl<amac/<thorn/</ets> cloth, garment; akin to D. <ets>kleed</ets>, Icel. <ets>kl<ae/<edh/i</ets>, Dan. <ets>kl<ae/de</ets>, cloth, Sw. <ets>kl<aum/de</ets>, G. <ets>kleid</ets> garment, dress.]</ety> <sn>1.</sn> <def>A fabric made of fibrous material (or sometimes of wire, as in wire cloth); commonly, a woven fabric of cotton, woolen, or linen, adapted to be made into garments; specifically, woolen fabrics, as distinguished from all others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The dress; raiment. <mark>[Obs.]</mark> See <er>Clothes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll ne'er distust my God for <qex>cloth</qex> and bread.</q> <rj><qau>Quarles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The distinctive dress of any profession, especially of the clergy; hence, the clerical profession.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Appeals were made to the priesthood. Would they tamely permit so gross an insult to be offered to their <qex>cloth</qex>?</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cloth</qex>, the clergy, are constituted for administering and for giving the best possible effect to . . . every axiom.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Body cloth</b></col>. <cd>See under <er>Body</er>.</cd> -- <col><b>Cloth of gold</b></col>, <cd>a fabric woven wholly or partially of threads of gold.</cd> -- <col><b>Cloth measure</b></col>, <cd>the measure of length and surface by which cloth is measured and sold. For this object the standard yard is usually divided into quarters and nails.</cd> -- <col><b>Cloth paper</b></col>, <cd>a coarse kind of paper used in pressing and finishing woolen cloth.</cd> -- Cloth <col><b>shearer</b></col>, <cd>one who shears cloth and frees it from superfluous nap.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cloth-bound</ent><br/
<hw>cloth-bound</hw> <pos>adj.</pos> <def>having rigid front and back covers, covered with cloth; -- of books.  Contrasted to <contr><er>softcover</er></contr>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clothe</ent><br/
<hw>Clothe</hw> <pr>(kl<omac/<th/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clothed</conjf> <pr>(kl<omac/<th/d)</pr> <it>or</it>  <conjf>Clad</conjf> <pr>(kl<acr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clothing</conjf>.]</vmorph> <ety>[OE. <ets>clathen</ets>, <ets>clothen</ets>, <ets>clethen</ets>, AS. <ets>cl<amac/<edh/ian</ets>, <ets>cl<aemac/<edh/an</ets>. See <er>Cloth</er>.]</ety> <sn>1.</sn> <def>To put garments on; to cover with clothing; to dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go with me, to <qex>clothe</qex> you as becomes you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To provide with clothes; <as>as, to feed and <ex>clothe</ex> a family; to <ex>clothe</ex> one's self extravagantly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Drowsiness shall <qex>clothe</qex> a man with rags.</q> <rj><qau>Prov. xxiii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The naked every day he <qex>clad</qex>,<br/
When he put on his clothes.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: To cover or invest, as with a garment; <as>as, to <ex>clothe</ex> one with authority or power</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Language in which they can <qex>clothe</qex> their thoughts.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His sides are <qex>clothed</qex> with waving wood.</q> <rj><qau>J. Dyer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus Belial, with with words <qex>clothed</qex> in reason's garb.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clothe</ent><br/
<hw>Clothe</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To wear clothes.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Care no more to <qex>clothe</qex> eat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clothed</ent><br/
<hw>clothed</hw> <pos>adj.</pos> <sn>1.</sn> <def>wearing clothing.</def> [Narrower terms: <stype>adorned(predicate), bedecked(predicate), decked(predicate), decked out(predicate)</stype>; <stype>appareled, attired, clad, dressed, garbed, garmented, habilimented, robed</stype>; <stype>arrayed, panoplied</stype>;  <stype>breeched, pantalooned, trousered</stype>; <stype>bundled-up</stype>; <stype>caparisoned</stype>; <stype>cassocked</stype>: <stype>costumed</stype>: <stype>decent</stype>] [Narrower terms: <stype>dight</stype>] [Narrower terms: <stype>dressed-up, dressed to the nines(predicate), dressed to kill(predicate), dolled up, spruced up, spiffed up</stype>] [Narrower terms: <stype>gowned</stype>] [Narrower terms: <stype>habited</stype>] [Narrower terms: <stype>heavy-coated</stype>] [Narrower terms: <stype>overdressed</stype>] [Narrower terms: <stype>petticoated</stype>] [Narrower terms: <stype>red-coated, lobster-backed</stype>] [Narrower terms: <stype>surpliced</stype>] [Narrower terms: <stype>togged  dressed esp in smart clothes)</stype>] [Narrower terms: <stype>turned out</stype>] [Narrower terms: <stype>underdressed</stype>] [Narrower terms: <stype>uniformed</stype>] [Narrower terms: <stype>vestmented</stype>] <see>Also See: <er>adorned</er>, <er>decorated</er>.</see>  Antonym: <ant>unclothed</ant>.<br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>covered with or as if with clothes or a wrap or cloak.</def> <illu>fog-<ex>cloaked</ex> meadows</illu><br/
<syn><b>Syn. --</b> cloaked, draped, mantled, wrapped.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clothes</ent><br/
<hw>Clothes</hw> <pr>(kl<omac/<th/z <it>or</it>  kl<omac/z; 277)</pr>, <pos>n. pl.</pos> <ety>[From <er>Cloth</er>.]</ety> <sn>1.</sn> <def>Covering for the human body; dress; vestments; vesture; -- a general term for whatever covering is worn, or is made to be worn, for decency or comfort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She . . . speaks well, and has excellent good <qex>clothes</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If I may touch but his <qex>clothes</qex>, I shall be whole.</q> <rj><qau>Mark. v. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The covering of a bed; bedclothes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She turned each way her frighted head,<br/
Then sunk it deep beneath the <qex>clothes</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Body clothes</b></col>. <cd>See under <er>Body</er>.</cd> -- <col><b>Clothes moth</b></col> <fld>(Zool.)</fld>, <cd>a small moth of the genus <gen>Tinea</gen>. The most common species (<spn>Tinea flavifrontella</spn>) is yellowish white. The larv<ae/ eat woolen goods, furs, feathers, etc. They live in tubular cases made of the material upon which they feed, fastened together with silk.</cd></cs></p>

<p><syn><b>Syn.</b> -- Garments; dress; clothing; apparel; attire; vesture; raiment; garb; costume; habit; habiliments.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>clothesbrush</ent><br/
<hw>clothes"brush`</hw> <pr>(kl<omac/z"br<ucr/sh`)</pr>, <pos>n.</pos> <def>a brush used for cleaning clothing.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clotheshorse</ent><br/
<hw>Clothes"horse`</hw> <pr>(kl<omac/z"h<ocir/rs`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A frame to hang clothes on.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a person who is conspicuous for dressing fashionably, especially one who spends much time or money to acquire clothing.</def><br/
[<source>PJC</source>]</p>

<p><ent>clothesless</ent><br/
<hw>clothesless</hw> <pos>adj.</pos> <def>unclothed.  Opposite of <ant>clothed</ant>.</def><br/
<syn><b>Syn. --</b> garmentless, raimentless.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clothesline</ent><br/
<hw>Clothes"line`</hw> <pr>(kl<omac/z"l<imac/n`)</pr>, <pos>n.</pos> <def>A rope or wire on which clothes are hung to dry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clothespin</ent><br/
<hw>Clothes"pin`</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>A forked piece of wood or plastic, or a small device with a spring clamp, used for fastening clothes on a line.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Clothespress</ent><br/
<hw>Clothes"press`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A receptacle for clothes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clothier</ent><br/
<hw>Cloth"ier</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who makes cloths; one who dresses or fulls cloth.</def>  <rj><au>Hayward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who sells cloth or clothes, or who makes and sells clothes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clothing</ent><br/
<hw>Cloth"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Garments in general; clothes; dress; raiment; covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From others he shall stand in need of nothing,<br/
Yet on his brothers shall depend for <qex>clothing</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As for me, . . . my <qex>clothing</qex> was sackloth.</q> <rj><qau>Ps. xxxv. 13</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The art of process of making cloth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Instructing [refugees] in the art of <qex>clothing</qex>.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A covering of non-conducting material on the outside of a boiler, or steam chamber, to prevent radiation of heat.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <def>See <cref>Card clothing</cref>, under 3d <er>Card</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clothred</ent><br/
<hw>Clot"hred</hw> <pr>(?)</pr>, <pos>p. p.</pos> <def>Clottered.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clotpoll</ent><br/
<hw>Clot"poll`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Clodpoll</er>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clotted</ent><br/
<hw>Clot"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Composed of clots or clods; having the quality or form of a clot; sticky; slimy; foul.</def> <ldquo/The <xex>clotted</xex> glebe.<rdquo/  <rj><au>J. Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When lust . . . <br/
Lets in defilement to the inward parts,<br/
The soul grows <qex>clotted</qex> by contagion.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clotter</ent><br/
<hw>Clot"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[From <er>Clot</er>.]</ety> <def>To concrete into lumps; to clot.</def> <mark>[Obs.]</mark> <ldquo/<xex>Clottered</xex> blood.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clotty</ent><br/
<hw>Clot"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Clot</er>, <pos>n.</pos>]</ety> <def>Full of clots, or clods.</def> <ldquo/<xex>Clotty</xex> matter.<rdquo/  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloture</ent><br/
||<hw>Cl<ocir/`ture"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Parliamentary Practice)</fld> <def>See <er>Closure</er>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clotweed</ent><br/
<hw>Clot"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Clote</er>.]</ety> <def>Cocklebur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud</ent><br/
<hw>Cloud</hw> <pr>(kloud)</pr>, <pos>n.</pos> <ety>[Prob. fr. AS. <ets>cl<umac/d</ets> a rock or hillock, the application arising from the frequent resemblance of clouds to rocks or hillocks in the sky or air.]</ety> <sn>1.</sn> <def>A collection of visible vapor, or watery particles, suspended in the upper atmosphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do set my bow in the <qex>cloud</qex>.</q> <rj><qau>Gen. ix. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A classification of clouds according to their chief forms was first proposed by the meteorologist Howard, and this is still substantially employed. The following varieties and subvarieties are recognized: <sd>(a)</sd> <stype><colf>Cirrus</colf></stype>. This is the most elevated of all the forms of clouds; is thin, long-drawn, sometimes looking like carded wool or hair, sometimes like a brush or room, sometimes in curl-like or fleecelike patches. It is the <xex>cat's-tail</xex> of the sailor, and the <xex>mare's-tail</xex> of the landsman. <sd>(b)</sd> <stype><colf>Cumulus</colf></stype>. This form appears in large masses of a hemispherical form, or nearly so, above, but flat below, one often piled above another, forming great clouds, common in the summer, and presenting the appearance of gigantic mountains crowned with snow. It often affords rain and thunder gusts. <sd>(c)</sd> <stype><colf>Stratus</colf></stype>. This form appears in layers or bands extending horizontally. <sd>(d)</sd> <stype><colf>Nimbus</colf></stype>. This form is characterized by its uniform gray tint and ragged edges; it covers the sky in seasons of continued rain, as in easterly storms, and is the proper <xex>rain cloud</xex>. The name is sometimes used to denote a raining cumulus, or cumulostratus. <sd>(e)</sd> <stype><colf>Cirro-cumulus</colf></stype>. This form consists, like the <xex>cirrus</xex>, of thin, broken, fleecelice clouds, but the parts are more or less rounded and regulary grouped. It is popularly called <xex>mackerel sky</xex>. <sd>(f)</sd> <stype><colf>Cirro-stratus</colf></stype>. In this form the patches of cirrus coalesce in long strata, between cirrus and stratus. <sd>(g)</sd> <stype><colf>Cumulo-stratus</colf></stype>. A form between cumulus and stratus, often assuming at the horizon a black or bluish tint. -- <stype><colf>Fog</colf></stype>, cloud, motionless, or nearly so, lying near or in contact with the earth's surface. -- <stype><colf>Storm scud</colf></stype>, cloud lying quite low, without form, and driven rapidly with the wind.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mass or volume of smoke, or flying dust, resembling vapor.</def> <ldquo/A thick <xex>cloud</xex> of incense.<rdquo/  <rj><au>Ezek. viii. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A dark vein or spot on a lighter material, as in marble; hence, a blemish or defect; <as>as, a <ex>cloud</ex> upon one's reputation; a <ex>cloud</ex> on a title</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which has a dark, lowering, or threatening aspect; that which temporarily overshadows, obscures, or depresses; <as>as, a <ex>cloud</ex> of sorrow; a <ex>cloud</ex> of war; a <ex>cloud</ex> upon the intellect</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A great crowd or multitude; a vast collection.</def> <ldquo/So great a <xex>cloud</xex> of witnesses.<rdquo/  <rj><au>Heb. xii. 1.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A large, loosely-knitted scarf, worn by women about the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cloud on a</b></col> (<it>or</it> the) <col><b>title</b></col></mcol> <fld>(Law)</fld>, <cd>a defect of title, usually superficial and capable of removal by release, decision in equity, or legislation.</cd> -- <col><b>To be under a cloud</b></col>, <cd>to be under suspicion or in disgrace; to be in disfavor.</cd> -- <col><b>In the clouds</b></col>, <cd>in the realm of facy and imagination; beyond reason; visionary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud</ent><br/
<hw>Cloud</hw> <pr>(kloud)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clouded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Clouding</conjf>.]</vmorph> <sn>1.</sn> <def>To overspread or hide with a cloud or clouds; <as>as, the sky is <ex>clouded</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To darken or obscure, as if by hiding or enveloping with a cloud; hence, to render gloomy or sullen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One day too late, I fear me, noble lord,<br/
Hath <qex>clouded</qex> all thy happy days on earth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be not disheartened, then, nor <qex>cloud</qex> those looks.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing <qex>clouds</qex> men's minds and impairs their honesty like prejudice.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To blacken; to sully; to stain; to tarnish; to damage; -- esp. used of reputation or character.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would not be a stander-by to hear<br/
My sovereign mistress <qex>clouded</qex> so, without<br/
My present vengeance taken.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To mark with, or darken in, veins or sports; to variegate with colors; as, to <qex>cloud</qex> yarn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the nice conduct of a <qex>clouded</qex> cane.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud</ent><br/
<hw>Cloud</hw>, <pos>v. i.</pos> <def>To grow cloudy; to become obscure with clouds; -- often used with <ptcl>up</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Worthies, away! The scene begins to <qex>cloud</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloudage</ent><br/
<hw>Cloud"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Mass of clouds; cloudiness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A scudding <qex>cloudage</qex> of shapes.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloudberry</ent><br/
<hw>Cloud"ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A species of raspberry (<spn>Rubus Cham<ae/merous</spn>) growing in the northern regions, and bearing edible, amber-colored fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud-built</ent><br/
<hw>Cloud"-built</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Built of, or in, the clouds; airy; unsubstantial; imaginary.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So vanished my <qex>cloud-built</qex> palace.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud-burst</ent><br/
<hw>Cloud"-burst`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sudden copious rainfall, as if the whole cloud had been precipitated at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud-capped</ent><br/
<hw>Cloud"-capped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having clouds resting on the top or head; reaching to the clouds; <as>as, <ex>cloud-capped</ex> mountains</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloud-compeller</ent><br/
<hw>Cloud"-com*pel`ler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cloud-gatherer; -- an epithet applied to Zeus.</def> <mark>[Poetic.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cloud-cuckoo-land</ent><br/
<hw>cloud-cuckoo-land</hw> <pos>n.</pos> <def>an imaginary place where fantasy and illusion predominate; it is a place where you say people are when they seem optimistically out of touch with reality.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cloudily</ent><br/
<hw>Cloud"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a cloudy manner; darkly; obscurely.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloudiness</ent><br/
<hw>Cloud"i*ness</hw>, <pos>n.</pos> <def>The state of being cloudy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clouding</ent><br/
<hw>Cloud"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>A mottled appearance given to ribbons and silks in the process of dyeing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A diversity of colors in yarn, recurring at regular intervals.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloudland</ent><br/
<hw>Cloud"land`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Dreamland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloudless</ent><br/
<hw>Cloud"less</hw>, <pos>a.</pos> <def>Without a cloud; clear; bright.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cloudless</qex> winter sky.</q> <rj><qau>Bankroft.</qau></rj></p>

<p>-- <wordforms><wf>Cloud"less*ly</wf>, <pos>adv.</pos> -- <wf>Cloud"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloudlet</ent><br/
<hw>Cloud"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little cloud.</def>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Eve's first star through fleecy <qex>cloudlet</qex> peeping.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cloudlike</ent><br/
<hw>cloudlike</hw> <pos>adj.</pos> <def>resembling a cloud.</def><br/
<syn><b>Syn. --</b> nebular.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cloud-nine</ent><br/
<ent>cloud nine</ent><br/
<mhw><hw>cloud" nine"</hw>, <hw>cloud"-nine"</hw></mhw> <pos>n.</pos> <def>a state of extreme or perfect happiness; used especially in the phrase <ecol><b>on cloud nine</b></ecol>.</def><br/
<syn><b>Syn. --</b> bliss, blissfulness, seventh-heaven.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cloudy</ent><br/
<hw>Cloud"y</hw> <pr>(kloud"<ucr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Cloudier</adjf> <pr>(<?/)</pr>; <pos>superl.</pos> <adjf>Cloudiest</adjf>.]</amorph> <ety>[From <ets>Cloud</ets>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Overcast or obscured with clouds; clouded; as, a <qex>cloudy</qex> sky.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting of a cloud or clouds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As Moses entered into the tabernacle, the <qex>cloudy</qex> pillar descended.</q> <rj><qau>Ex. xxxiii. 9</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Indicating gloom, anxiety, sullenness, or ill-nature; not open or cheerful.</def> <ldquo/A <xex>cloudy</xex> countenance.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Confused; indistinct; obscure; dark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cloudy</qex> and confused notions of things.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Lacking clearness, brightness, or luster.</def> <ldquo/A <xex>cloudy</xex> diamond.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Marked with veins or sports of dark or various hues, as marble.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clough</ent><br/
<hw>Clough</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>clough</ets>, <ets>cloghe</ets>, <ets>clou</ets>, <ets>clewch</ets>, AS. (assumed) <ets>cl<omac/h</ets>, akin to G. <ets>klinge</ets> ravine.]</ety> <sn>1.</sn> <def>A cleft in a hill; a ravine; a narrow valley.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sluice used in returning water to a channel after depositing its sediment on the flooded land.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clough</ent><br/
<hw>Clough</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <fld>(Com.)</fld> <def>An allowance in weighing. See <er>Cloff</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clout</ent><br/
<hw>Clout</hw> <pr>(klout)</pr>, <pos>n.</pos> <ety>[AS. <ets>cl<umac/t</ets> a little cloth, piece of metal; cf. Sw. <ets>klut</ets>, Icel. <ets>kl<umac/tr</ets> a kerchief, or W. <ets>clwt</ets> a clout, Gael. <ets>clud</ets>.]</ety> <sn>1.</sn> <def>A cloth; a piece of cloth or leather; a patch; a rag.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His garments, nought but many ragged <qex>clouts</qex>,<br/
With thorns together pinned and patched was.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>clout</qex> upon that head where late the diadem stood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A swadding cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece; a fragment.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The center of the butt at which archers shoot; -- probably once a piece of white cloth or a nail head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A'must shoot nearer or he'll ne'er hit the <qex>clout</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An iron plate on an axletree or other wood to keep it from wearing; a washer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A blow with the hand.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clout nail</b></col>, <cd>a kind of wrought-iron nail heaving a large flat head; -- used for fastening clouts to axletrees, plowshares, etc., also for studding timber, and for various purposes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clout</ent><br/
<hw>Clout</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clouted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Clouting</conjf>.]</vmorph> <ety>[OE. <ets>clutien</ets>. <ets>clouten</ets>, to patch. See <er>Clout</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cover with cloth, leather, or other material; to bandage; patch, or mend, with a clout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And old shoes and <qex>clouted</qex> upon their feet.</q> <rj><qau>Josh. ix. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Paul, yea, and Peter, too, had more skill in . . . <qex>clouting</qex> an old tent than to teach lawyers.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To join or patch clumsily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If fond Bavius vent his <qex>clouted</qex> song.</q> <rj><qau>P. Fletcher</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To quard with an iron plate, as an axletree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To give a blow to; to strike.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The . . . queen of Spain took off one of her chopines and <qex>clouted</qex> Olivarez about the noddle with it.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To stud with nails, as a timber, or a boot sole.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clouted cream</b></col>, <cd>clotted cream, <it>i. e.</it>, cream obtained by warming new milk.</cd>  <rj><au>A. Philips.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/<xex>Clouted</xex> brogues<rdquo/ in Shakespeare and <ldquo/<xex>clouted</xex> shoon<rdquo/ in Milton have been understood by some to mean shoes armed with nails; by others, patched shoes.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clouterly</ent><br/
<hw>Clout"er*ly</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Clout</er>, <pos>n.</pos>]</ety> <def>Clumsy; awkward.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Rough-hewn, <qex>cloutery</qex> verses.</q> <rj><qau>E. Phillips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clove</ent><br/
<hw>Clove</hw> <pr>(?)</pr>, <pos>imp.</pos> <mord>of <er>Cleave</er>.</mord> <def>Cleft.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clove hitch</b></col> <fld>(Naut.)</fld> <cd>See under <er>Hitch</er>.</cd> -- <col><b>Clove hook</b></col> <fld>(Naut.)</fld>, <cd>an iron two-part hook, with jaws overlapping, used in bending chain sheets to the clews of sails; -- called also <altname>clip hook</altname>.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clove</ent><br/
<hw>Clove</hw>, <pos>n.</pos> <ety>[D. <ets>kloof</ets>. See <er>Cleave</er>, <pos>v. t.</pos>]</ety> <def>A cleft; a gap; a ravine; -- rarely used except as part of a proper name; <as>as, Kaaterskill <ex>Clove</ex>; Stone <ex>Clove</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clove</ent><br/
<hw>Clove</hw>, <pos>n.</pos> <ety>[OE. <ets>clow</ets>, fr. F. <ets>clou</ets> nail, <ets>clou de girofle</ets> a clove, lit. nail of clove, fr. L. <ets>clavus</ets> nail, perh. akin to <ets>clavis</ets> key, E. <ets>clavicle</ets>. The clove was so called from its resemblance to a nail. So in D. <ets>kruidnagel</ets> clove, lit. <ets>herb-nail</ets> or <ets>spice-nail</ets>. Cf. <er>Cloy</er>.]</ety> <def>A very pungent aromatic spice, the unexpanded flower bud of the clove tree (<spn>Eugenia aromatica</spn> syn. <spn>Caryophullus aromatica</spn>), a native of the Molucca Isles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clove camphor</b></col>. <fld>(Chem.)</fld> <cd>See <er>Eugenin</er>.</cd> -- <mcol><col><b>Clove gillyflower</b></col>, <col><b>Clove pink</b></col></mcol> <fld>(Bot.)</fld>, <cd>any fragrant self-colored carnation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clove</ent><br/
<hw>Clove</hw>, <pos>n.</pos> <ety>[AS. <ets>clufe</ets> an ear of corn, a clove of garlic; cf. <ets>cle<oacute/fan</ets> to split, E. <ets>cleave</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>One of the small bulbs developed in the axils of the scales of a large bulb, as in the case of garlic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Developing, in the axils of its skales, new bulbs, of what gardeners call <qex>cloves</qex>.</q> <rj><qau>Lindley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 269 --></p>

<p><sn>2.</sn> <def>A weight. A <xex>clove</xex> of cheese is about eight pounds, of wool, about seven pounds.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloven</ent><br/
<hw>Clo"ven</hw> <pr>(kl<omac/"v'n)</pr>, <pos>p. p. & a.</pos> <def>from <er>Cleave</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To show the cloven foot</b></col> <it>or</it> <col><b>To show the cloven hoof</b></col></mcol>, <cd>to reveal a devilish character, or betray an evil purpose, notwithstanding disguises, -- Satan being represented dramatically and symbolically as having cloven hoofs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloven-hoofed</ent><br/
<ent>Cloven-footed</ent><br/
<mhw>{ <hw>Clo"ven-foot`ed</hw> <pr>(?)</pr>, <hw>Clo"ven-hoofed`</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <def>Having the foot or hoof divided into two parts, as the ox.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clover</ent><br/
<hw>Clo"ver</hw> <pr>(kl<omac/"v<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>claver</ets>, <ets>clover</ets>, AS. <ets>cl<aemac/fre</ets>; akin to LG. & Dan. <ets>klever</ets>, D. <ets>klaver</ets>, G. <ets>klee</ets>, Sw. <ets>kl<oum/fver</ets>.]</ety> <fld>(Bot.)</fld> <def>A plant of different species of the genus <gen>Trifolium</gen>; as the common red clover, <spn>Trifolium pratense</spn>, the white, <spn>Trifolium repens</spn>, and the hare's foot, <spn>Trifolium arvense</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clover weevil</b></col> <fld>(Zool.)</fld> <cd>a small weevil (<spn>Apion apricans</spn>), that destroys the seeds of clover.</cd> -- <col><b>Clover worm</b></col> <fld>(Zool.)</fld>, <cd>the larva of a small moth (<spn>Asopia costalis</spn>), often very destructive to clover hay.</cd> -- <col><b>In clover</b></col>, in very pleasant circumstances; fortunate. <mark>[Colloq.]</mark> -- <col><b>Sweet clover</b></col>. <cd>See <er>Meliot</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clovered</ent><br/
<hw>Clo"vered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covered with growing clover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Flocks thick nibbling through the <qex>clovered</qex> vale.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cloverleaf</ent><br/
<hw>cloverleaf</hw> <pos>n.</pos> <def>a highway interchange between two roads in which the connecting road pattern resembles a four-leaf clover, and which allows moving from one road to another without left-hand turns.  One road passes over the other, and the exit from one highway and entrance into the second highway proceeds from the right-hand lane in all directions.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>cloveroot</ent><br/
<hw>cloveroot</hw> <pos>n.</pos> <def>a hairy Eurasian plant (<spn>Geum urbanum</spn>) with small yellow flowers and an astringent root formerly used medicinally.</def><br/
<syn><b>Syn. --</b> herb bennet, wood avens, <spn>Geum urbanum</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>clowder</ent><br/
<hw>clowder</hw> <pos>n.</pos> <ety>[from <ets>clodder</ets>, dial. variant of <er>clotter</er>, to clot, huddle together.  <au>RHUD</au>]</ety> <def>a group of cats.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clowe-gilofre</ent><br/
<hw>Clowe"-gi*lof`re</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See 3d <er>Clove</er>, and <er>Gillyflower</er>.]</ety> <def>Spice clove.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clown</ent><br/
<hw>Clown</hw> <pr>(kloun)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>klunni</ets> a clumsy, boorish fellow, North Fries. <ets>kl<oum/nne</ets> clown, dial. Sw. <ets>klunn</ets> log, Dan. <ets>klunt</ets> log, block, and E. <ets>clump</ets>, n.]</ety> <sn>1.</sn> <def>A man of coarse nature and manners; an awkward fellow; an ill-bred person; a boor.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who works upon the soil; a rustic; a churl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>clown</qex>, the child of nature, without guile.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The fool or buffoon in a play, circus, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>clown</qex> shall make those laugh whose lungs are tickle o'the sere.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clown</ent><br/
<hw>Clown</hw>, <pos>v. i.</pos> <def>To act as a clown; -- with <xex>it</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Beshrew me, he <qex>clowns</qex> it properly indeed.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clownage</ent><br/
<hw>Clown"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Behavior or manners of a clown; clownery.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clownery</ent><br/
<hw>Clown"er*y</hw> <pr>(-<etil/r*<ycr/)</pr>, <pos>n.</pos> <def>Clownishness.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clowning</ent><br/
<hw>clowning</hw> <pos>n.</pos> <sn>1.</sn> <def>acting like a clown or buffoon.</def><br/
<syn><b>Syn. --</b> buffoonery, frivolity, harlequinade, prank.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a comic incident or series of incidents.</def><br/
<syn><b>Syn. --</b> drollery, comedy, funniness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clownish</ent><br/
<hw>Clown"ish</hw>, <pos>a.</pos> <def>Of or resembling a clown, or characteristic of a clown; ungainly; awkward.</def> <ldquo/<xex>Clownish</xex> hands.<rdquo/ <au>Spenser.</au> <ldquo/<qex>Clownish</qex> mimic.<rdquo/ <au>Prior.</au></p>

<p>-- <wordforms><wf>Clown"ish*ly</wf>, <pos>adv.</pos></wordforms></p>

<p><syn><b>Syn.</b> -- Coarse; rough; clumsy; awkward; ungainly; rude; uncivil; ill-bred; boorish; rustic; untutored.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clownishness</ent><br/
<hw>Clown"ish*ness</hw>, <pos>n.</pos> <def>The manners of a clown; coarseness or rudeness of behavior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That plainness which the alamode people call <qex>clownishness</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloy</ent><br/
<hw>Cloy</hw> <pr>(kloi)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cloyed</conjf> <pr>(kloid)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cloying</conjf>.]</vmorph> <ety>[OE. <ets>cloer</ets> to nail up, F. <ets>clouer</ets>, fr. OF. <ets>clo</ets> nail, F. <ets>clou</ets>, fr. L. <ets>clavus</ets> nail. Cf. 3d <er>Clove</er>.]</ety> <sn>1.</sn> <def>To fill or choke up; to stop up; to clog.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The duke's purpose was to have <qex>cloyed</qex> the harbor by sinking ships, laden with stones.</q> <rj><qau>Speed.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To glut, or satisfy, as the appetite; to satiate; to fill to loathing; to surfeit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Who can] <qex>cloy</qex> the hungry edge of appetite<br/
By bare imagination of a feast?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He sometimes <qex>cloys</qex> his readers instead of satisfying.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To penetrate or pierce; to wound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which, with his cruel tusk, him deadly <qex>cloyed</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He never shod horse but he <qex>cloyed</qex> him.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To spike, as a cannon.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To stroke with a claw.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloyless</ent><br/
<hw>Cloy"less</hw>, <pos>a.</pos> <def>That does not cloy.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cloyment</ent><br/
<hw>Cloy"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Satiety.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Club</ent><br/
<hw>Club</hw> <pr>(kl<ucr/b)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>klubba</ets>, <ets>klumba</ets>, club, <ets>klumbuf<omac/ir</ets> a clubfoot, SW. <ets>klubba</ets> club, Dan. <ets>klump</ets> lump, <ets>klub</ets> a club, G. <ets>klumpen</ets> clump, <ets>kolben</ets> club, and E. <ets>clump</ets>.]</ety> <sn>1.</sn> <def>A heavy staff of wood, usually tapering, and wielded with the hand; a weapon; a cudgel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But make you ready your stiff bats and <qex>clubs</qex>;<br/
Rome and her rats are at the point of battle.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. the Spanish name <ets>bastos,</ets> and Sp. <ets>baston</ets> staff, club.]</ety> <def>Any card of the suit of cards having a figure like the trefoil or clover leaf. (<pluf>pl.</pluf>) The suit of cards having such figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An association of persons for the promotion of some common object, as literature, science, politics, good fellowship, etc.; esp. an association supported by equal assessments or contributions of the members.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They talked<br/
At wine, in <qex>clubs</qex>, of art, of politics.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Goldsmith] was one of the nine original members of that celebrated fraternity which has sometimes been called the Literary <qex>Club</qex>, but which has always disclaimed that epithet, and still glories in the simple name of the <qex>Club</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A joint charge of expense, or any person's share of it; a contribution to a common fund.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They laid down the <qex>club</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We dined at a French house, but paid ten shillings for our part of the <qex>club</qex>.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Club law</b></col>, <cd>government by violence; lynch law; anarchy.</cd>  <rj><au>Addison. -</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Club root</b></col> <fld>(Bot.)</fld>, <cd>a disease of cabbages, by which the roots become distorted and the heads spoiled.</cd> -- <col><b>Club topsail</b></col> <fld>(Naut.)</fld>, <cd>a kind of gaff topsail, used mostly by yachts having a fore-and-aft rig. It has a short <ldquo/club<rdquo/ or <ldquo/jack yard<rdquo/ to increase its spread.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Club</ent><br/
<hw>Club</hw> <pr>(kl<ucr/b)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clubbed</conjf> <pr>(kl<ucr/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clubbing</conjf>.]</vmorph> <sn>1.</sn> <def>To beat with a club.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>To throw, or allow to fall, into confusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>club</qex> a battalion implies a temporary inability in the commanding officer to restore any given body of men to their natural front in line or column.</q> <rj><qau>Farrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite, or contribute, for the accomplishment of a common end; <as>as, to <ex>club</ex> exertions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To raise, or defray, by a proportional assesment; <as>as, to <ex>club</ex> the expense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To club a musket</b></col> <fld>(Mil.)</fld>, <cd>to turn the breach uppermost, so as to use it as a club.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Club</ent><br/
<hw>Club</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To form a club; to combine for the promotion of some common object; to unite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till grosser atoms, tumbling in the stream<br/
Of fancy, madly met, and <qex>clubbed</qex> into a dream.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pay on equal or proportionate share of a common charge or expense; to pay for something by contribution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The owl, the raven, and the bat,<br/
<qex>Clubbed</qex> for a feather to his hat.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>To drift in a current with an anchor out.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubbable</ent><br/
<hw>Club"ba*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suitable for membership in a club; sociable.</def> <mark>[Humorous.]</mark> <altsp>[Also spelled <asp>clubable</asp>.]</altsp> <rj><au>G. W. Curtis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clubmoss</ent><br/
<ent>club moss</ent><br/
<mhw><hw>club" moss`</hw>, <hw>clubmoss</hw></mhw> <pos>n.</pos> <fld>(Bot.)</fld> <def>a primitive evergreen mosslike plant with spores in club-shaped strobiles, much used in winter decoration. The best known species is <spn>Lycopodium clavatum</spn>, but other <gen>Lycopodia</gen> are often called by this name. The spores form a highly inflammable powder.</def><br/
<syn><b>Syn. --</b> club moss, lycopod.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Clubbed</ent><br/
<hw>Clubbed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a club; grasped like, or used as, a club.</def>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubber</ent><br/
<hw>Club"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who clubs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of a club.</def> <mark>[R.]</mark>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubbish</ent><br/
<hw>Club"bish</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Rude; clownish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disposed to club together; <as>as, a <ex>clubbish</ex> set</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubbist</ent><br/
<hw>Club"bist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member of a club; a frequenter of clubs.</def> <mark>[R.]</mark>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubfist</ent><br/
<hw>Club"fist`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A large, heavy fist.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coarse, brutal fellow.</def> <mark>[Obs.]</mark>  <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubfisted</ent><br/
<hw>Club"fist`ed</hw>, <pos>a.</pos> <def>Having a large fist.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubfoot</ent><br/
<hw>Club"foot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Club</ets> + <ets>foot</ets>.]</ety> <fld>(Med.)</fld> <def>A short, variously distorted foot; also, the deformity, usually congenital, which such a foot exhibits; talipes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubfooted</ent><br/
<hw>Club"foot`ed</hw>, <pos>a.</pos> <def>Having a clubfoot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubhand</ent><br/
<hw>Club"hand`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A short, distorted hand; also, the deformity of having such a hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubhaul</ent><br/
<hw>Club"haul`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Naut.)</fld> <def>To put on the other tack by dropping the lee anchor as soon as the wind is out of the sails (which brings the vessel's head to the wind), and by cutting the cable as soon as she pays off on the other tack. Clubhauling is attempted only in an exigency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubhouse</ent><br/
<hw>Club"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house occupied by a club.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clubroom</ent><br/
<hw>Club"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The apartment in which a club meets.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Club-rush</ent><br/
<hw>Club"-rush`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A rushlike plant, the reed mace or cat-tail, or some species of the genus <gen>Scirpus</gen>. See <er>Bulrush</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Club-shaped</ent><br/
<hw>Club"-shaped</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Enlarged gradually at the end, as the antenn<ae/ of certain insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluck</ent><br/
<hw>Cluck</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clucked</conjf> <pr>(?)</pr>; <pos>p pr. & vb. n.</pos> <conjf>Clucking</conjf>.]</vmorph> <ety>[AS. <ets>cloccian</ets>; cf. D. <ets>klokken</ets>, G. <ets>glucken</ets>, <ets>glucksen</ets>, LG. <ets>klukken</ets>, Dan. <ets>klukke</ets>; all prob. of imitative origin.]</ety> <def>To make the noise, or utter the call, of a brooding hen.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluck</ent><br/
<hw>Cluck</hw>, <pos>v. t.</pos> <def>To call together, or call to follow, as a hen does her chickens.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She, poor hen, fond of no second brood,<br/
Has <qex>clucked</qex> three to the wars.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluck</ent><br/
<hw>Cluck</hw>, <pos>n.</pos> <sn>1.</sn> <def>The call of a hen to her chickens.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A click. See 3d <er>Click</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clucking</ent><br/
<hw>Cluck"ing</hw>, <pos>n.</pos> <def>The noise or call of a brooding hen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clue</ent><br/
<hw>Clue</hw> <pr>(kl<umac/)</pr>, <pos>n.</pos> <ety>[See <er>Clew</er>, <pos>n.</pos>]</ety> <def>A ball of thread; a thread or other means of guidance. Same as <er>Clew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have wound a goodly <qex>clue</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>clue</qex> once found unravels all the rest.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Serve as <qex>clues</qex> to guide us into further knowledge.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>clueless</ent><br/
<hw>clueless</hw> <pos>adj.</pos> <sn>1.</sn> <def>not providing any clue.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>completely uninformed or unaware; hopelessly ignorant; utterly bewildered.</def><br/
[<source>PJC</source>]</p>

<p><ent>Clum</ent><br/
<hw>Clum</hw> <pr>(kl<ucr/m)</pr>, <pos>interj.</pos> <def>Silence; hush.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumber</ent><br/
<hw>Clum"ber</hw> <pr>(kl<ucr/m"b<etil/r)</pr>, <pos>n.</pos> <ety>[Named from the estate of the Duke of Newcastle.]</ety> <fld>(Zool.)</fld> <def>A kind of field spaniel, with short legs and stout body, which, unlike other spaniels, hunts silently.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clump</ent><br/
<hw>Clump</hw> <pr>(kl<ucr/mp)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>klomp</ets> lump, G. <ets>klump</ets>, <ets>klumpen</ets>, Dan. <ets>klump</ets>, Sw. <ets>klump</ets>; perh. akin to L. <ets>globus</ets>, E. <ets>globe</ets>. Cf. <er>Club</er>.]</ety> <sn>1.</sn> <def>An unshaped piece or mass of wood or other substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cluster; a group; a thicket.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>clump</qex> of shrubby trees.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The compressed clay of coal strata.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clump</ent><br/
<hw>Clump</hw>, <pos>v. t.</pos> <def>To arrange in a clump or clumps; to cluster; to group.</def>  <rj><au>Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clump</ent><br/
<hw>Clump</hw>, <pos>v. i.</pos> <def>To tread clumsily; to clamp.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumper</ent><br/
<hw>Clump"er</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. G. <ets>kl<uum/mpern</ets> to clod. See <er>Clump</er>, <pos>n.</pos>]</ety> <def>To form into clumps or masses.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Vapors . . . <qex>clumpered</qex> in balls of clouds.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumps</ent><br/
<hw>Clumps</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A game in which questions are asked for the purpose of enabling the questioners to discover a word or thing previously selected by two persons who answer the questions; -- so called because the players take sides in two <ldquo/clumps<rdquo/ or groups, the <ldquo/clump<rdquo/ which guesses the word winning the game.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumpy</ent><br/
<hw>Clump"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Clump</er>, <pos>n.</pos>]</ety> <def>Composed of clumps; massive; shapeless.</def>  <rj><au>Leigh Hunt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumsily</ent><br/
<hw>Clum"si*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a clumsy manner; awkwardly; <as>as, to walk <ex>clumsily</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumsiness</ent><br/
<hw>Clum"si*ness</hw>, <pos>n.</pos> <def>The quality of being clumsy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The drudging part of life is chiefly owing to <qex>clumsiness</qex> and ignorance.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clumsy</ent><br/
<hw>Clum"sy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Clumsier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Clumsiest</adjf>.]</amorph> <ety>[OE. <ets>clumsed</ets> benumbed, fr. <ets>clumsen</ets> to be benumbed; cf. Icel. <ets>klumsa</ets> lockjaw, dial. Sw. <ets>klummsen</ets> benumbed with cold. Cf. 1st <er>Clam</er>, and 1st <er>Clamp</er>.]</ety> <sn>1.</sn> <def>Stiff or benumbed, as with cold.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without skill or grace; wanting dexterity, nimbleness, or readiness; stiff; awkward, as if benumbed; unwieldy; unhandy; hence; ill-made, misshapen, or inappropriate; <as>as, a <ex>clumsy</ex> person; a <ex>clumsy</ex> workman; <ex>clumsy</ex> fingers; a <ex>clumsy</ex> gesture; a <ex>clumsy</ex> excuse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But thou in <qex>clumsy</qex> verse, unlicked, unpointed,<br/
Hast shamefully defied the Lord's anointed.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Awkward</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clunch</ent><br/
<hw>Clunch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. fr. <ets>clinch</ets> to make fast]</ety> <sn>1.</sn> <fld>(Mining)</fld> <def>Indurated clay. See <er>Bind</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the hard beds of the lower chalk.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clung</ent><br/
<hw>Clung</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Cling</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clung</ent><br/
<hw>Clung</hw>, <pos>a.</pos> <ety>[Prop. p. p. fr. OE. <ets>clingen</ets> to wither. See <er>Cling</er>, <pos>v. i.</pos>]</ety> <def>Wasted away; shrunken.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluniac</ent><br/
<hw>Clu"ni*ac</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A monk of the reformed branch of the Benedictine Order, founded in 912 at Cluny (or Clugny) in France. -- Also used as <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluniacensian</ent><br/
<hw>Clu`ni*a*cen"sian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cluniac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clupeid</ent><br/
<hw>clupeid</hw> <pos>n.</pos> <def>any of numerous soft-finned schooling food fishes of the family <fam>Clupeidae</fam>, inhabiting the shallow waters of northern seas.  The group includes the herrings, menhaden, sardines, and shad.</def><br/
<syn><b>Syn. --</b> herring, clupeid fish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clupeidae</ent><br/
<hw>Clupeidae</hw> <pos>n.</pos> <def>a natural family of teleostean marine fishes which includes the herrings, shad, sardines, menhaden, and others.</def><br/
<syn><b>Syn. --</b> family <fam>Clupeidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clupeoid</ent><br/
<hw>Clu"pe*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L.<ets>clupea</ets> a kind of fish, NL., generic name of the herring + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Herring family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>clusia</ent><br/
<hw>clusia</hw> <pos>n.</pos> <def>an aromatic tree of the genus <gen>Clusia</gen> having large white or yellow or pink flowers.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clusiaceae</ent><br/
<hw>Clusiaceae</hw> <pos>n.</pos> <def>a widely distributed family of chiefly tropical trees and shrubs and vines that produce oils and resins and some usable timber.</def><br/
<syn><b>Syn. --</b> Guttiferae, family <fam>Guttiferae</fam>, family <fam>Clusiaceae</fam>, St John's wort family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cluster</ent><br/
<hw>Clus"ter</hw> <pr>(kl<ucr/s"t<etil/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>cluster</ets>, <ets>clyster</ets>; cf. LG. <ets>kluster</ets> (also Sw. & Dan. <ets>klase</ets> a cluster of grapes, D. <ets>klissen</ets> to be entangled?.)]</ety> <sn>1.</sn> <def>A number of things of the same kind growing together; a bunch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her deeds were like great <qex>clusters</qex> of ripe grapes,<br/
Which load the bunches of the fruitful vine.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A number of similar things collected together or lying contiguous; a group; <as>as, a <ex>cluster</ex> of islands</as>.</def> <ldquo/<xex>Cluster</xex> of provinces.<rdquo/  <rj><au>Motley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A number of individuals grouped together or collected in one place; a crowd; a mob.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As bees . . . <br/
Pour forth their populous youth about the hive<br/
In <qex>clusters</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We loved him; but, like beasts<br/
And cowardly nobles, gave way unto your <qex>clusters</qex>,<br/
Who did hoot him out o' the city.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluster</ent><br/
<hw>Clus"ter</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clustered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clustering</conjf>.]</vmorph> <def>To grow in clusters or assemble in groups; to gather or unite in a cluster or clusters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His sunny hair<br/
<qex>Cluster'd</qex> about his temples, like a god's.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The princes of the country <qex>clustering</qex> together.</q> <rj><qau>Foxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cluster</ent><br/
<hw>Clus"ter</hw>, <pos>v. t.</pos> <def>To collect into a cluster or clusters; to gather into a bunch or close body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not less the bee would range her cells, . . . <br/
The foxglove <qex>cluster</qex> dappled bells.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Or from the forest falls the <qex>clustered</qex> snow.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clustered column</b></col> <fld>(Arch.)</fld>, <cd>a column which is composed, or appears to be composed, of several columns collected together.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>clustered</ent><br/
<hw>clustered</hw> <pos>adj.</pos> <sn>1.</sn> <def>growing close together but not in dense mats; -- of plants.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>occurring close together in bunches or clusters.</def><br/
<syn><b>Syn. --</b> bunched, bunchy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>clustered together but not coherent.</def><br/
<syn><b>Syn. --</b> agglomerate, agglomerated, agglomerative, aggregate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Clusteringly</ent><br/
<hw>Clus"ter*ing*ly</hw>, <pos>adv.</pos> <def>In clusters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clustery</ent><br/
<hw>Clus"ter*y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cluster</er>, <pos>n.</pos>]</ety> <def>Growing in, or full of, clusters; like clusters.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutch</ent><br/
<hw>Clutch</hw> <pr>(kl<ucr/ch; 224)</pr>, <pos>n.</pos> <ety>[OE. <ets>cloche</ets>, <ets>cloke</ets>, claw, Scot. <ets>clook</ets>, <ets>cleuck</ets>, also OE. <ets>cleche</ets> claw, <ets>clechen</ets>, <ets>cleken</ets>, to seize; cf. AS. <ets>gel<ae/ccan</ets> (where <ets>ge-</ets> is a prefix) to seize. Cf. <er>Latch</er> a catch.]</ety> <sn>1.</sn> <def>A gripe or clinching with, or as with, the fingers or claws; seizure; grasp.</def> <ldquo/The <xex>clutch</xex> of poverty.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An expiring <qex>clutch</qex> at popularity.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But Age, with his stealing steps,<br/
Hath clawed me in his <qex>clutch</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>The hands, claws, or talons, in the act of grasping firmly; -- often figuratively, for power, rapacity, or cruelty; <as>as, to fall into the <ex>clutches</ex> of an adversary</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I must have . . . little care of myself, if I ever more come near the <qex>clutches</qex> of such a giant.</q> <rj><qau>Bp. Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>A device which is used for coupling shafting, etc., so as to transmit motion, and which may be disengaged at pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any device for gripping an object, as at the end of a chain or tackle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The nest complement of eggs of a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bayonet clutch</b></col> <fld>(Mach.)</fld>, <cd>a clutch in which connection is made by means of bayonets attached to arms sliding on a feathered shaft. The bayonets slide through holes in a crosshead fastened on the shaft.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutch</ent><br/
<hw>Clutch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Clutched</conjf> <pr>(kl<ucr/cht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Clutching</conjf>.]</vmorph> <ety>[OE. <ets>clucchen</ets>. See <er>Clutch</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To seize, clasp, or grip with the hand, hands, or claws; -- often figuratively; <as>as, to <ex>clutch</ex> power</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man may set the poles together in his head, and <qex>clutch</qex> the whole globe at one intellectual grasp.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Is this a dagger which I see before me . . . ?<br/
Come, let me <qex>clutch</qex> thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To close tightly; to clinch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not that I have the power to <qex>clutch</qex> my hand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutch</ent><br/
<hw>Clutch</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To reach (at something) as if to grasp; to catch or snatch; -- often followed by <ptcl>at</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to become too tense or frightened to perform properly; used sometimes with <ptcl>up</ptcl>; <as>as, he <ex>clutched</ex> up on the exam</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>clutches</ent><br/
<hw>clutches</hw> <pos>n.</pos> <sn>1.</sn> <def>the act of grasping.</def><br/
<syn><b>Syn. --</b> clasp, clench, clutch, grasp, grip, hold.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>control; power; -- used in the phrase <ecol><b>in one's clutches</b></ecol>.  Same as <er>clutch{2}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>clutch hitter</ent><br/
<hw>clutch hitter</hw> <pos>n.</pos> <fld>(Baseball)</fld> <def>a batter who hits safely more frequently when men are on base or the team is behind in the score.</def><br/
[<source>PJC</source>]</p>

<p><q><qex>Clutching</qex> at the phantoms of the stock market.</q> <rj><qau>Bankroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutter</ent><br/
<hw>Clut"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. W. <ets>cludair</ets> heap, pile, <ets>cludeirio</ets> to heap.]</ety> <sn>1.</sn> <def>A confused collection; hence, confusion; disorder; <as>as, the room is in a <ex>clutter</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He saw what a <qex>clutter</qex> there was with huge, overgrown pots, pans, and spits.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Clatter; confused noise.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutter</ent><br/
<hw>Clut"ter</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cluttered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cluttering</conjf>.]</vmorph> <def>To crowd together in disorder; to fill or cover with things in disorder; to throw into disorder; to disarrange; <as>as, to <ex>clutter</ex> a room</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutter</ent><br/
<hw>Clut"ter</hw>, <pos>v. i.</pos> <def>To make a confused noise; to bustle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [the goose] <qex>cluttered</qex> here, it chuckled there.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clutter</ent><br/
<hw>Clut"ter</hw>, <pos>v. t.</pos> <ety>[From <er>Clod</er>, <pos>n.</pos>]</ety> <def>To clot or coagulate, as blood.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clydesdale</ent><br/
<hw>Clydes"dale</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of a breed of heavy draft horses originally from Clydesdale, Scotland. They are about sixteen hands high and usually brown or bay.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clydesdale terrier</ent><br/
<hw>Clydesdale terrier</hw>. <def>One of a breed of small silky-haired terriers related to, but smaller than, the Skye terrier, having smaller and perfectly erect ears.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Clypeastroid</ent><br/
<hw>Clyp`e*as"troid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Clypeaster</ets> (L. <ets>clupeus</ets> shield + <ets>aster</ets> star) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or related to the genus <gen>Clupeaster</gen>; -- applied to a group of flattened sea urchins, with a rosette of pores on the upper side.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 270 --></p>

<p><ent>Clypeate</ent><br/
<hw>Clyp"e*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clupeatus</ets>, p. p. of <ets>clupeare</ets> to arm with a shield, fr. <ets>clupeus</ets>, <ets>clipeus</ets> shield.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Shaped like a round buckler or shield; scutate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Furnished with a shield, or a protective plate or shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clypeiform</ent><br/
<hw>Clyp"e*i*form`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>clupeus</ets> shield + <ets>-form</ets>.]</ety> <def>Shield-shaped; clypeate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clypeus</ent><br/
||<hw>Clyp"e*us</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Clypei</plw> <pr>(#)</pr>.</plu> <ety>[L., a shield.]</ety> <fld>(Zool.)</fld> <def>The frontal plate of the head of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clysmian</ent><br/
<hw>Clys"mi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a place washed by the waves, fr. <?/. See <er>Clyster</er>.]</ety> <def>Connected with, or related to, the deluge, or to a cataclysm; <as>as, <ex>clysmian</ex> changes</as>.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clysmic</ent><br/
<hw>Clys"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Washing; cleansing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clyster</ent><br/
<hw>Clys"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. G. <?/. fr. <?/ to wash off or out; akin to Goth. <ets>hl<umac/trs</ets> pure, G. <ets>lauter</ets>: cf. F. <ets>clyst<egrave/re</ets>]</ety> <fld>(Med.)</fld> <def>A liquid injected into the lower intestines by means of a syringe; an injection; an enema.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clyster pipe</b></col>, <cd>a tube or pipe used for injections.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Clytie knot</ent><br/
<hw>Cly"tie knot</hw> <pr>(?)</pr>. <def>In hair dressing, a loose, low coil at the back of the head, like the knot on the head of the bust of Clytie by <person>G. F. Watts</person>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cm</ent><br/
<hw>cm</hw> <def>the abbreviation for <er>centimeter</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cm</ent><br/
<hw>Cm</hw>. <def>the chemical symbol for the element <er>curium</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cnemial</ent><br/
<hw>Cne"mi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ the tibia.]</ety> <fld>(Anat.)</fld> <def>Pertaining to the shin bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cnemial crest</b></col>, <cd>a crestlike prominence on the proximal end of the tibia of birds and some reptiles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cnicus</ent><br/
<hw>Cnicus</hw> <pos>n.</pos> <def>a genus of plants having only one species, the blessed thistle.</def><br/
<syn><b>Syn. --</b> genus <gen>Cnicus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cnida</ent><br/
||<hw>Cni"da</hw> <pr>(n<imac/"d<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cnid<ae/</plw> <pr>(n<imac/"d<emac/)</pr>.</plu> <ety>[NL., fr. Gr. <grk>kni`dh</grk> nettle, sea nettle.]</ety> <fld>(Zool.)</fld> <def>One of the peculiar stinging cells found in C<oe/lenterata; a nematocyst; a lasso cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cnidaria</ent><br/
||<hw>Cni*da"ri*a</hw> <pr>(n<icr/*d<amac/"r<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Cnida</er>.]</ety> <fld>(Zool.)</fld> <def>A comprehensive group equivalent to the true C<oe/lenterata, <it>i. e.</it>, exclusive of the sponges. They are so named from presence of stinging cells (<part>cnidae</part>) in the tissues. See <er>Coelenterata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cnidoblast</ent><br/
<hw>Cni"do*blast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cnida</ets> + <ets>-blast</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the cells which, in the C<oe/lenterata, develop into cnid<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cnidocil</ent><br/
<hw>Cni"do*cil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cnida</ets> + <ets>cilium</ets> eyelash.]</ety> <fld>(Zool.)</fld> <def>The fine filiform process of a cnidoblast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>CNS</ent><br/
<hw>CNS</hw> <pos>n.</pos> <def>acronym for <er>central nervous system</er> the portion of the vertebrate nervous system consisting of the brain and spinal cord.</def><br/
<syn><b>Syn. --</b> central nervous system.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Co</ent><br/
<hw>Co</hw> <pos>n.</pos> <def>the chemical formula for <er>cobalt</er>, a ferromagnetic metal of atomic number 27.</def><br/
<syn><b>Syn. --</b> cobalt, atomic number 27.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Co-</ent><br/
<hw>Co-</hw> <pr>(k<osl/-)</pr>. <def>A form of the prefix <xex>com-</xex>, signifying <xex>with</xex>, <xex>together</xex>, <xex>in conjunction</xex>, <xex>joint</xex>. It is used before vowels and some consonants. See <er>Com-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>CO2</ent><br/
<hw>CO2</hw> <pr>(s<emac/"<osl/*t<oomac/`)</pr>, <pos>n.</pos> <def>the chemical formula for <er>carbon dioxide</er>, a heavy odorless gas (<chform>CO2</chform>) formed during respiration and by the combustion or decomposition of organic substances; it is absorbed from the air by plants in photosynthesis.  The formula itself, pronounced out, is often used instead of the full name, especially with reference to fire extinguishers using this gas for fire suppression; <as>as, a <ex>CO2</ex> extinguisher</as>.</def><br/
<syn><b>Syn. --</b> carbon dioxide, carbonic acid gas.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>CO2 extinguisher</ent><br/
<hw>CO2 extinguisher</hw> <pr>(s<emac/"<osl/*t<oomac/` <ecr/ks*t<icr/<nsm/"gw<icr/sh*<etil/r)</pr>, <def>a device for suppressing fire, consisting of a steel cylinder containing compressed carbon dioxide, which may be released by pressing on a handle.  The release of the gas is usually accomplished through a hose attached to a funnel-shaped vent which is directed at the base of the fire, and when released, part of the carbon dioxide is chilled sufficiently to solidify into a powder.  The large quantity of inert carbon dioxide released on top of a small fire is usually sufficient to exinguish the flame by excluding oxygen, and the cooling effect also helps drive the temperature of the combustible material below that required to support compustion.  It is used in situations where putting water on the fire might cause undesirable damage.</def><br/
[<source>PJC</source>]</p>

<p><ent>CoA</ent><br/
<hw>CoA</hw> <pr>(k<osl/*<amac/")</pr>, <fld>(Biochem)</fld> <def>an abbreviation for <er>coenzyme A</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coacervate</ent><br/
<hw>Co`a*cer"vate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coacervatus</ets>, p. p. of <ets>coacervare</ets> to heap up; <ets>co-</ets> + <ets>acervare</ets>. See <er>Acervate</er>.]</ety> <def>Raised into a pile; collected into a crowd; heaped.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coacervate</ent><br/
<hw>Co`a*cer"vate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To heap up; to pile.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coacervation</ent><br/
<hw>Co*ac`er*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coacervatio</ets>.]</ety> <def>A heaping together.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coach</ent><br/
<hw>Coach</hw> <pr>(k<omac/ch; 224)</pr>, <pos>n.</pos> <ety>[F. <ets>coche</ets>, fr. It. <ets>cocchio</ets>, dim. of <ets>cocca</ets> little boat, fr. L. <ets>concha</ets> mussel, mussel shell, Gr. <?/, akin to Skr. <ets><cced/ankha</ets>. Cf. <er>Conch</er>, <er>Cockboat</er>, <er>Cockle</er>.]</ety> <sn>1.</sn> <def>A large, closed, four-wheeled carriage, having doors in the sides, and generally a front and back seat inside, each for two persons, and an elevated outside seat in front for the driver.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Coaches</xex> have a variety of forms, and differ in respect to the number of persons they can carry. <xex>Mail coaches</xex> and <xex>tallyho coaches</xex> often have three or more seats inside, each for two or three persons, and seats outside, sometimes for twelve or more.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A special tutor who assists in preparing a student for examination.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wareham was studying for India with a Wancester <qex>coach</qex>.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A cabin on the after part of the quarter-deck, usually occupied by the captain.</def> <altsp>[Written also <asp>couch</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The commanders came on board and the council sat in the <qex>coach</qex>.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Railroad)</fld> <def>A first-class passenger car, as distinguished from a drawing-room car, sleeping car, etc. It is sometimes loosely applied to any passenger car.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn>  <def>One who coaches;</def> <specif>specif.</specif> <fld>(sports)</fld>, <def>a trainer; one who assists in training individual athletes or the members of a sports team, or who performs other ancillary functions in sports; <as>as, a third base <ex>coach</ex></as>.</def><br/
[<source>+PJC</source>]</p>

<p><ent>Coach</ent><br/
<hw>Coach</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coached</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coaching</conjf>.]</vmorph> <sn>1.</sn> <def>To convey in a coach.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prepare for public examination by private instruction; to train by special instruction.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>coached</qex> him before he got his scholarship.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coach</ent><br/
<hw>Coach</hw>, <pos>v. i.</pos> <def>To drive or to ride in a coach; -- sometimes used with</def> <xex>it</xex>. <mark>[Colloq.]</mark> <ldquo/<xex>Coaching</xex> it to all quarters.<rdquo/  <rj><au>E. Waterhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coachbox</ent><br/
<hw>Coach"box`</hw> <pr>(?)</pr>. <def>The seat of a coachman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coachbuilder</ent><br/
<hw>coachbuilder</hw> <pos>n.</pos> <def>a craftsman who makes the bodies of motor vehicles.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coachdog</ent><br/
<hw>Coach"dog`</hw> <pr>(?; 115)</pr>. <fld>(Zool.)</fld> <def>One of a breed of dogs trained to accompany carriages; the Dalmatian dog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coachee</ent><br/
<hw>Coach"ee</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coachman</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coacher</ent><br/
<hw>Coach"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A coachman.</def> <mark>[Obs.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A coach horse.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>One who coaches;</def> <specif>specif.</specif> <fld>(Baseball)</fld>, <def>one of the side at the bat posted near first or third base to direct a base runner; also called a <altname>coach</altname>; as, third base <ex>coach</ex>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Coachfellow</ent><br/
<hw>Coach"fel`low</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of a pair of horses employed to draw a coach; hence <mark>(Fig.)</mark>, a comrade.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coaching</ent><br/
<hw>coaching</hw> <pos>n.</pos> <def>the job of a professional coach.</def><br/
<syn><b>Syn. --</b> coaching job.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coachman</ent><br/
<hw>Coach"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Coachmen</plw> <pr>(#)</pr>.</plu> <sn>1.</sn> <def>A man whose business is to drive a coach or carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A tropical fish of the Atlantic ocean (<spn>Dutes auriga</spn>); -- called also <altname>charioteer</altname>. The name refers to a long, lashlike spine of the dorsal fin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coachmanship</ent><br/
<hw>Coach"man*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Skill in driving a coach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coachwhip snake</ent><br/
<hw>Coach"whip` snake"</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A large, slender, harmless snake of the southern United States (<spn>Masticophis flagelliformis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Its long and tapering tail has the scales so arranged and colored as to give it a braided appearance, whence the name.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coact</ent><br/
<hw>Co*act"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>coactare</ets>, intens. fr. <ets>cogere</ets>, <ets>coactum</ets>, to force. See <er>Cogent</er>.]</ety> <def>To force; to compel; to drive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The faith and service of Christ ought to be voluntary and not <qex>coacted</qex>.</q> <rj><qau>Foxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coact</ent><br/
<hw>Co*act"</hw>, <pos>v. i.</pos> <ety>[Pref. <ets>co-</ets> + <ets>act</ets>, v. i.]</ety> <def>To act together; to work in concert; to unite.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But if I tell you how these two did <qex>coact</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaction</ent><br/
<hw>Co*ac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coactio</ets>.]</ety> <def>Force; compulsion, either in restraining or impelling.</def>  <rj><au>Sojth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coactive</ent><br/
<hw>Co*ac"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[In sense 1, fr. 1st <er>Coact</er>; in sense 2, fr. 2d <er>Coact</er>.]</ety> <sn>1.</sn> <def>Serving to compel or constrain; compulsory; restrictive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Any <qex>coactive</qex> power or the civil kind.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Acting in concurrence; united in action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With what's unreal thou <qex>coactive</qex> art.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coactively</ent><br/
<hw>Co*ac"tive*ly</hw>, <pos>adv.</pos> <def>In a coactive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coactivity</ent><br/
<hw>Co`ac*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Unity of action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadaptation</ent><br/
<hw>Co*ad`ap*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Mutual adaption.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadapted</ent><br/
<hw>Co`a*dapt"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adapted one to another; <as>as, <ex>coadapted</ex> pulp and tooth</as>.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjument</ent><br/
<hw>Co*ad"ju*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Mutual help; cooperation.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjust</ent><br/
<hw>Co`ad*just"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To adjust by mutual adaptations.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjustment</ent><br/
<hw>Co`ad*just"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Mutual adjustment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjutant</ent><br/
<hw>Co*ad"ju*tant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Mutually assisting or operating; helping.</def>  <rj><au>J. Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjutant</ent><br/
<hw>Co*ad"ju*tant</hw>, <pos>n.</pos> <def>An assistant.</def>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjuting</ent><br/
<hw>Co*ad"ju*ting</hw>, <pos>a.</pos> <def>Mutually assisting.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjutive</ent><br/
<hw>Co*ad"ju*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Rendering mutual aid; coadjutant.</def>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjutor</ent><br/
<hw>Co`ad*ju"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Co</er>-, and <er>Aid</er>.]</ety> <sn>1.</sn> <def>One who aids another; an assistant; a coworker.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Craftily outwitting her perjured <qex>coadjutor</qex>.</q> <rj><qau>Sheridan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>The assistant of a bishop or of a priest holding a benefice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjutorship</ent><br/
<hw>Co`ad*ju"tor*ship</hw>, <pos>n.</pos> <def>The state or office of a coadjutor; joint assistance.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjutrix</ent><br/
<ent>Coadjutress</ent><br/
<mhw>{ <hw>Co`ad*ju"tress</hw> <pr>(?)</pr>, <hw>Co`ad*ju"trix</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A female coadjutor or assistant.</def>  <rj><au>Holland. Smollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjuvancy</ent><br/
<hw>Co*ad"ju*van*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint help; cooperation.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjuvant</ent><br/
<hw>Co*ad"ju*vant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cooperating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadjuvant</ent><br/
<hw>Co*ad"ju*vant</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>An adjuvant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadunate</ent><br/
<hw>Co*ad"u*nate</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[L. <ets>coadunatus</ets>, p. p. of <ets>coadunare</ets> to unite. See <er>Adunation</er>.]</ety> <fld>(Bot.)</fld> <def>United at the base, as contiguous lobes of a leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadunation</ent><br/
<hw>Co*ad`u*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coadunatio</ets>.]</ety> <def>Union, as in one body or mass; unity.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>coadunation</qex> of all the civilized provinces.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadunition</ent><br/
<hw>Co*ad`u*ni"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>co-</ets> + pref. <ets>ad-</ets> + <ets>unition</ets>.]</ety> <def>Coadunation.</def> <mark>[R.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadventure</ent><br/
<hw>Co`ad*ven"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>An adventure in which two or more persons are partakers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadventure</ent><br/
<hw>Co`ad*ven"ture</hw>, <pos>v. i.</pos> <def>To share in a venture.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coadventurer</ent><br/
<hw>Co`ad*ven"tur*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fellow adventurer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coafforest</ent><br/
<hw>Co`af*for"est</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To convert into, or add to, a forest.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coag</ent><br/
<hw>Coag</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Coak</er>, a kind of tenon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagency</ent><br/
<hw>Co*a"gen*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Agency in common; joint agency or agent.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagent</ent><br/
<hw>Co*a"gent</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An associate in an act; a coworker.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagment</ent><br/
<hw>Co`ag*ment"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>coagmentare</ets>, fr. <ets>coagmentum</ets> a joining together, fr. <ets>cogere</ets>. See <er>Cogent</er>.]</ety> <def>To join together.</def> <mark>[Obs.]</mark>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagmentation</ent><br/
<hw>Co*ag`men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coagmentatio</ets>.]</ety> <def>The act of joining, or the state of being joined, together; union.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulability</ent><br/
<hw>Co*ag`u*la*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being coagulable; capacity of being coagulated.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulable</ent><br/
<hw>Co*ag"u*la*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being coagulated.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulant</ent><br/
<hw>Co*ag"u*lant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coagulans</ets>, p. pr.]</ety> <def>That which produces coagulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulate</ent><br/
<hw>Co*ag"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coagulatus</ets>, p. p. of <ets>coagulare</ets> to coagulate, fr. <ets>coagulum</ets> means of coagulation, fr. <ets>cogere</ets>, <ets>coactum</ets>, to drive together, coagulate. See <er>Cogent</er>.]</ety> <def>Coagulated.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulate</ent><br/
<hw>Co*ag"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coagulated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coagulating</conjf> <pr>(?)</pr>.]</vmorph> <def>To cause (a liquid) to change into a curdlike or semisolid state, not by evaporation but by some kind of chemical reaction; to curdle; <as>as, rennet <ex>coagulates</ex> milk; heat <ex>coagulates</ex> the white of an egg</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulate</ent><br/
<hw>Co*ag"u*late</hw>, <pos>v. i.</pos> <def>To undergo coagulation.</def>  <rj><au>Boyle.</au></rj></p>

<p><syn><b>Syn.</b> -- To thicken; concrete; curdle; clot; congeal.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulated</ent><br/
<hw>Co*ag"u*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Changed into, or contained in, a coagulum or a curdlike mass; curdled.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coagulated proteid</b></col> <fld>(Physiol. Chem.)</fld>, <cd>one of a class of bodies formed in the coagulation of a albuminous substance by heat, acids, or other agents.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulation</ent><br/
<hw>Co*ag`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coagulatio</ets>.]</ety> <sn>1.</sn> <def>The change from a liquid to a thickened, curdlike, insoluble state, not by evaporation, but by some kind of chemical reaction; <as>as, the spontaneous <ex>coagulation</ex> of freshly drawn blood; the <ex>coagulation</ex> of milk by rennet, or acid, and the <ex>coagulation</ex> of egg albumin by heat</as>.  Coagulation is generally the change of an albuminous body into an insoluble modification.</def>
<-- by heat is due to denaturation of protein. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The substance or body formed by coagulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulative</ent><br/
<hw>Co*ag"u*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the power to cause coagulation; <as>as, a <ex>coagulative</ex> agent</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulator</ent><br/
<hw>Co*ag"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which causes coagulation.</def>  <rj><au>Hixley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulatory</ent><br/
<hw>Co*ag"u*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to coagulate; produced by coagulation; <as>as, <ex>coagulatory</ex> effects</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coagulum</ent><br/
<hw>Co*ag"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Coagula</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Coagulate</er>, <pos>a.</pos>]</ety> <def>The thick, curdy precipitate formed by the coagulation of albuminous matter; any mass of coagulated matter, as a clot of blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaita</ent><br/
||<hw>Co*ai"ta</hw> <pr>(k<osl/*<aum/<icr/"t<adot/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The native name of certain South American monkeys of the genus <gen>Ateles</gen>, esp. <spn>Ateles paniscus</spn>. The black-faced coaita is <spn>Ateles ater</spn>. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coak</ent><br/
<hw>Coak</hw> <pr>(k<omac/k)</pr>, <pos>n.</pos> <def>See <er>Coke</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coak</ent><br/
<hw>Coak</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Carp.)</fld> <def>A kind of tenon connecting the face of a scarfed timber with the face of another timber, or a dowel or pin of hard wood or iron uniting timbers.</def> <altsp>[Also spelt <asp>coag</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A metallic bushing or strengthening piece in the center of a wooden block sheave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coak</ent><br/
<hw>Coak</hw>, <pos>v. t.</pos> <fld>(Carp.)</fld> <def>To unite, as timbers, by means of tenons or dowels in the edges or faces.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coal</ent><br/
<hw>Coal</hw> <pr>(k<omac/l)</pr>, <pos>n.</pos> <ety>[AS. <ets>col</ets>; akin to D. <ets>kool</ets>, OHG. <ets>chol</ets>, <ets>cholo</ets>, G. <ets>kohle</ets>, Icel. <ets>kol</ets>, pl., Sw. <ets>kol</ets>, Dan. <ets>kul</ets>; cf. Skr. <ets>jval</ets> to burn. Cf. <er>Kiln</er>, <er>Collier</er>.]</ety> <sn>1.</sn> <def>A thoroughly charred, and extinguished or still ignited, fragment from wood or other combustible substance; charcoal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>A black, or brownish black, solid, combustible substance, dug from beds or veins in the earth to be used for fuel, and consisting, like charcoal, mainly of carbon, but more compact, and often affording, when heated, a large amount of volatile matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is often used adjectively, or as the first part of self-explaining compounds; as, <xex>coal</xex>-black; <xex>coal</xex> formation; <xex>coal</xex> scuttle; <xex>coal</xex> ship. etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England the plural <xex>coals</xex> is used, for the broken mineral coal burned in grates, etc.; as, to put <xex>coals</xex> on the fire. In the United States the singular in a collective sense is the customary usage; as, a hod of <xex>coal</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Age of coal plants</b></col>. <cd>See <cref>Age of Acrogens</cref>, under <er>Acrogen</er>.</cd> -- <mcol><col><b>Anthracite</b></col> <it>or</it> <col><b>Glance coal</b></col></mcol>. <cd>See <er>Anthracite</er>.</cd> -- <col><b>Bituminous coal</b></col>. <cd>See under <er>Bituminous</er>.</cd> -- <col><b>Blind coal</b></col>.  <cd>See under <er>Blind</er>.</cd> -- <mcol><col><b>Brown coal</b></col> <it>or</it> <col><b>Brown Lignite</b></col></mcol>. <cd>See <er>Lignite</er>.</cd> -- <col><b>Caking coal</b></col>, <cd>a bituminous coal, which softens and becomes pasty or semi-viscid when heated. On increasing the heat, the volatile products are driven off, and a coherent, grayish black, cellular mass of coke is left.</cd> -- <col><b>Cannel coal</b></col>, <cd>a very compact bituminous coal, of fine texture and dull luster. See <er>Cannel coal</er>.</cd> -- <col><b>Coal bed</b></col> <fld>(Geol.)</fld>, <cd>a layer or stratum of mineral coal.</cd> -- <col><b>Coal breaker</b></col>, <cd>a structure including machines and machinery adapted for crushing, cleansing, and assorting coal.</cd> -- <col><b>Coal field</b></col> <fld>(Geol.)</fld>, <cd>a region in which deposits of coal occur. Such regions have often a basinlike structure, and are hence called <altname>coal basins</altname>. See <er>Basin</er>.</cd> -- <col><b>Coal gas</b></col>, <cd>a variety of carbureted hydrogen, procured from bituminous coal, used in lighting streets, houses, etc., and for cooking and heating.</cd> -- <col><b>Coal heaver</b></col>, <cd>a man employed in carrying coal, and esp. in putting it in, and discharging it from, ships.</cd> -- <col><b>Coal measures</b></col>. <fld>(Geol.)</fld> <sd>(a)</sd> <cd>Strata of coal with the attendant rocks.</cd> <sd>(b)</sd> <cd>A subdivision of the carboniferous formation, between the millstone grit below and the Permian formation above, and including nearly all the workable coal beds of the world.</cd> -- <col><b>Coal oil</b></col>, <cd>a general name for mineral oils; petroleum.</cd> -- <col><b>Coal plant</b></col> <fld>(Geol.)</fld>, <cd>one of the remains or impressions of plants found in the strata of the coal formation.</cd> -- <col><b>Coal tar</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>To haul over the coals</b></col>, <cd>to call to account; to scold or censure.</cd> <mark>[Colloq.]</mark> -- <col><b>Wood coal</b></col>. <cd>See <er>Lignite</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coal</ent><br/
<hw>Coal</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coaled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coaling</conjf>.]</vmorph> <sn>1.</sn> <def>To burn to charcoal; to char.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Charcoal of roots, <qex>coaled</qex> into great pieces.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark or delineate with charcoal.</def>  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To supply with coal; <as>as, to <ex>coal</ex> a steamer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coal</ent><br/
<hw>Coal</hw>, <pos>v. i.</pos> <def>To take in coal; <as>as, the steamer <ex>coaled</ex> at Southampton</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coalbin</ent><br/
<hw>coalbin</hw> <pos>n.</pos> <def>a bin for holding coal.</def><br/
<syn><b>Syn. --</b> coalhole.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coal-black</ent><br/
<hw>Coal"-black`</hw> <pr>(-bl<acr/k`)</pr>, <pos>a.</pos> <def>As black as coal; jet black; very black.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalery</ent><br/
<hw>Coal"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <mark>[Obs.]</mark> <def>See <er>Colliery</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalesce</ent><br/
<hw>Co`a*lesce"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coalesced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coalescing</conjf>.]</vmorph> <ety>[L. <ets>coalescere</ets>, <ets>coalitium</ets>; <ets>co-</ets> + <ets>alescere</ets> to grow up, incho. fr. <ets>alere</ets> to nourish. See <er>Aliment</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To grow together; to unite by growth into one body; <as>as, the parts separated by a wound <ex>coalesce</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite in one body or product; to   combine into one body or community; <as>as, vapors <ex>coalesce</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Jews were incapable of <qex>coalescing</qex> with other nations.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Certain combinations of ideas that, once <qex>coalescing</qex>, could not be shaken loose.</q> <rj><qau>De Quincey.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Add</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>coalesced</ent><br/
<hw>coalesced</hw> <pos>adj.</pos> <sn>1.</sn>  <def>joined together into a whole.</def><br/
<syn><b>Syn. --</b> amalgamate, amalgamated, consolidated, fused.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coalescence</ent><br/
<hw>Co`a*les"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or state of growing together, as similar parts; the act of uniting by natural affinity or attraction; the state of being united; union; concretion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalescent</ent><br/
<hw>Co`a*les"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coalescens</ets>, p. pr.]</ety> <def>Growing together; cohering, as in the organic cohesion of similar parts; uniting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalfish</ent><br/
<hw>Coal"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named from the dark color of the back.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The pollock; -- called also, <altname>coalsey</altname>, <altname>colemie</altname>, <altname>colmey</altname>, <altname>coal whiting</altname>, etc. See <er>Pollock</er>.</def> <sd>(b)</sd> <def>The beshow or candlefish of Alaska.</def> <sd>(c)</sd> <def>The cobia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalgoose</ent><br/
<hw>Coal"goose`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The cormorant; -- so called from its black color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalite</ent><br/
<hw>Co"a*lite</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>coalitus</ets>, p. p. of <ets>coalescere</ets>. See <er>Coalesce</er>.]</ety> <def>To unite or coalesce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let them continue to <qex>coalite</qex>.</q> <rj><qau>Bolingbroke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalite</ent><br/
<hw>Co"a*lite</hw>, <pos>v. t.</pos> <def>To cause to unite or coalesce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Time has by degrees blended . . . and <qex>coalited</qex> the conquered with the conquerors.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalition</ent><br/
<hw>Co`a*li"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>coalitio</ets>: cf. F. <ets>coalition</ets>. See <er>Coalesce</er>.]</ety> <sn>1.</sn> <def>The act of coalescing; union into a body or mass, as of separate bodies or parts; <as>as, a <ex>coalition</ex> of atoms</as>.</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 271 --></p>

<p><sn>2.</sn> <def>A combination, for temporary purposes, of persons, parties, or states, having different interests.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>coalition</qex> of the puritan and the blackleg.</q> <rj><qau>J. Randolph.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>coalition</qex> between the religious and worldly enemies of popery.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- Alliance; confederation; confederacy; league; combination; conjunction; conspiracy; union.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalitioner</ent><br/
<hw>Co`a*li"tion*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coalitionist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalitionist</ent><br/
<hw>Co`a*li"tion*ist</hw>, <pos>n.</pos> <def>One who joins or promotes a coalition; one who advocates coalition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-ally</ent><br/
<hw>Co`-al*ly"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Co-allies</plw> <pr>(#)</pr>.</plu> <def>A joint ally.</def>  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coal-meter</ent><br/
<hw>Coal"-me`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A licensed or official coal measurer in London. See <er>Meter</er>.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalmouse</ent><br/
<hw>Coal"mouse`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small species of titmouse, with a black head; the coletit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalpit</ent><br/
<hw>Coal"pit`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A pit where coal is dug.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place where charcoal is made.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coalsack</ent><br/
<hw>Coal"sack`</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Coal</ets> + 2d <ets>sack</ets>.]</ety> <fld>(Astron.)</fld> <def>Any one of the spaces in the Milky Way which are very black, owing to the nearly complete absence of stars; esp., the large space near the Southern Cross sometimes called the <altname>Black Magellanic Cloud</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coal tar</ent><br/
<hw>Coal" tar`</hw> <pr>(?)</pr>. <def>A thick, black, tarry liquid, obtained by the distillation of bituminous coal in the manufacture of illuminating gas; used for making printer's ink, black varnish, etc. It is a complex mixture from which many substances have been obtained, especially hydrocarbons of the benzene or aromatic series.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Among its important ingredients are benzene, aniline, phenol, naphtalene, anthracene, etc., which are respectively typical of many dye stuffs, as the aniline dyes, the phthale<ium/ns, indigo, alizarin, and many flavoring extracts whose artificial production is a matter of great commercial importance.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coal-whipper</ent><br/
<hw>Coal"-whip`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who raises coal out of the hold of a ship.</def> <mark>[Eng.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coal works</ent><br/
<hw>Coal" works</hw> <pr>(?)</pr>. <def>A place where coal is dug, including the machinery for raising the coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaly</ent><br/
<hw>Coal"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Coal</er>, <pos>n.</pos>]</ety> <def>Pertaining to, or resembling, coal; containing coal; of the nature of coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coamings</ent><br/
<hw>Coam"ings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Cf. <er>Comb</er> a crest.]</ety> <fld>(Naut.)</fld> <def>Raised pieces of wood of iron around a hatchway, skylight, or other opening in the deck, to prevent water from running bellow; esp. the fore-and-aft pieces of a hatchway frame as distinguished from the transverse head ledges.</def> <altsp>[Written also <asp>combings</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coannex</ent><br/
<hw>Co`an*nex"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To annex with something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaptation</ent><br/
<hw>Co`ap*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coaptatio</ets>, fr. <ets>coaptare</ets> to fit together; <ets>co-</ets> + <ets>aptare</ets>. See <er>Aptate</er>.]</ety> <def>The adaptation or adjustment of parts to each other, as of a broken bone or dislocated joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarctate</ent><br/
<ent>Coarct</ent><br/
<mhw>{ <hw>Co*arct"</hw> <pr>(?)</pr>, <hw>Co*arc"tate</hw> <pr>(?)</pr> }</mhw>, <pos>v. t.</pos> <ety>[See <er>Coarctate</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To press together; to crowd; to straiten; to confine closely.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To restrain; to confine.</def> <mark>[Obs.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarctate</ent><br/
<hw>Co*arc"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coarctatus</ets>, p. p. of <ets>coarctare</ets> to press together; <ets>co-</ets> + <ets>arctare</ets> to press together, from <ets>arctus</ets>, p. p. See <er>Arctation</er>.]</ety> <fld>(Zool.)</fld> <def>Pressed together; closely connected; -- applied to insects having the abdomen separated from the thorax only by a constriction.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coarctate pupa</b></col> <fld>(Zool.)</fld>, <cd>a pupa closely covered by the old larval skin, as in most Diptera.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarctation</ent><br/
<hw>Co`arc*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coarctatio</ets>.]</ety> <sn>1.</sn> <def>Confinement to a narrow space.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pressure; that which presses.</def> <mark>[Obs.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A stricture or narrowing, as of a canal, cavity, or orifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarse</ent><br/
<hw>Coarse</hw> <pr>(k<omac/rs)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Coarser</adjf> <pr>(k<omac/rs"<etil/r)</pr>; <pos>superl.</pos> <adjf>Coarsest</adjf>.]</amorph> <ety>[As this word was anciently written <ets>course</ets>, or <ets>cours</ets>, it may be an abbreviation of <ets>of course</ets>, in the common manner of proceeding, common, and hence, homely, made for common domestic use, plain, rude, rough, gross, <it>e. g.</it>, <ldquo/Though the threads be <ets>course</ets>.<rdquo/ <au>Gascoigne.</au> See <er>Course</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Large in bulk, or composed of large parts or particles; of inferior quality or appearance; not fine in material or close in texture; gross; thick; rough; -- opposed to <ant>fine</ant>; <as>as, <ex>coarse</ex> sand; <ex>coarse</ex> thread; <ex>coarse</ex> cloth; <ex>coarse</ex> bread</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not refined; rough; rude; unpolished; gross; indelicate; <as>as, <ex>coarse</ex> manners; <ex>coarse</ex> language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I feel<br/
Of what <qex>coarse</qex> metal ye are molded.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To copy, in my <qex>coarse</qex> English, his beautiful expressions.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Large; thick; rough; gross; blunt; uncouth; unpolished; inelegant; indelicate; vulgar.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarse-grained</ent><br/
<hw>Coarse"-grained`</hw> <pr>(k<omac/rs"gr<amac/nd`)</pr>, <pos>a.</pos> <def>Having a coarse grain or texture, as wood; hence, wanting in refinement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarsely</ent><br/
<hw>Coarse"ly</hw> <pr>(k<omac/rs"l<ycr/)</pr>, <pos>adv.</pos> <def>In a coarse manner; roughly; rudely; inelegantly; uncivilly; meanly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarsen</ent><br/
<hw>Coars"en</hw> <pr>(k<omac/rs"'n)</pr>, <pos>v. t.</pos> <def>To make coarse or vulgar; <as>as, to <ex>coarsen</ex> one's character</as>.</def> <mark>[R.]</mark>  <rj><au>Graham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coarsened</ent><br/
<hw>coarsened</hw> <pos>adj.</pos> <def>made coarse or crude by lack of skill; -- sometimes used to mean <altname>inferior</altname>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coarseness</ent><br/
<hw>Coarse"ness</hw> <pr>(k<omac/rs"n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being coarse; roughness; inelegance; vulgarity; grossness; <as>as, <ex>coarseness</ex> of food, texture, manners, or language</as>.</def> <ldquo/The <xex>coarseness</xex> of the sackcloth.<rdquo/  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pardon the <qex>coarseness</qex> of the illustration.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>coarseness</qex> and vulgarity in all the proceedings.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coarticulation</ent><br/
<hw>Co`ar*tic`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The union or articulation of bones to form a joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-assessor</ent><br/
<hw>Co`-as*sess"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint assessor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coast</ent><br/
<hw>Coast</hw> <pr>(k<omac/st)</pr>, <pos>n.</pos> <ety>[OF. <ets>coste</ets>, F. <ets>c<ocir/te</ets>, rib, hill, shore, coast, L. <ets>costa</ets> rib, side. Cf. <er>Accost</er>, <pos>v. t.</pos>, <er>Cutlet</er>.]</ety> <sn>1.</sn> <def>The side of a thing.</def> <mark>[Obs.]</mark>  <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The exterior line, limit, or border of a country; frontier border.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>From the river, the river Euphrates, even to the uttermost sea, shall your <qex>coast</qex> be.</q> <rj><qau>Deut. xi. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The seashore, or land near it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He sees in English ships the Holland <qex>coast</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We the Arabian <qex>coast</qex> do know<br/
At distance, when the species blow.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The coast is clear</b></col>, <cd>the danger is over; no enemy in sight.</cd> <au>Dryden.</au> <cd2>Fig.: <cd>There are no obstacles.</cd> <ldquo/Seeing that <ex>the coast was clear</ex>, Zelmane dismissed Musidorus.<rdquo/ <au>Sir P. Sidney.</au></cd2> -- <col><b>Coast guard</b></col>. <sd>(a)</sd> <cd>A body of men originally employed along the coast to prevent smuggling; now, under the control of the admiralty, drilled as a naval reserve.</cd> <mark>[Eng.]</mark> <sd>(b)</sd> <cd>The force employed in life-saving stations along the seacoast.</cd> <mark>[U. S.]</mark> -- <col><b>Coast rat</b></col> <fld>(Zool.)</fld>, <cd>a South African mammal (<spn>Bathyergus suillus</spn>), about the size of a rabbit, remarkable for its extensive burrows; -- called also <altname>sand mole</altname>.</cd> -- <col><b>Coast waiter</b></col>, <cd>a customhouse officer who superintends the landing or shipping of goods for the coast trade.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coast</ent><br/
<hw>Coast</hw> <pr>(k<omac/st)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coasted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coasting</conjf>.]</vmorph> <ety>[OE. <ets>costien</ets>, <ets>costeien</ets>, <ets>costen</ets>, OF. <ets>costier</ets>, <ets>costoier</ets>, F. <ets>c<ocir/toyer</ets>, <ets>fr</ets>. <ets>Of</ets>. coste coast, F. <ets>c<ocir/te</ets>. See <er>Coast</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To draw or keep near; to approach.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Anon she hears them chant it lustily,<br/
And all in haste she <qex>coasteth</qex> to the cry.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sail by or near the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ancients <qex>coasted</qex> only in their navigation.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sail from port to port in the same country.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Cf. OF. <ets>coste</ets>, F. <ets>c<ocir/te</ets>, hill, hillside.]</ety> <def>To slide down hill; to slide on a sled, upon snow or ice.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coast</ent><br/
<hw>Coast</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To draw near to; to approach; to keep near, or by the side of.</def> <mark>[Obs.]</mark>  <rj><au>Hakluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sail by or near; to follow the coast line of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nearchus, . . . not knowing the compass, was fain to <qex>coast</qex> that shore.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To conduct along a coast or river bank.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Indians . . . <qex>coasted</qex> me along the river.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coastal</ent><br/
<hw>Coast"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a coast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coast and Geodetic Survey</ent><br/
<hw>Coast and Geodetic Survey</hw>. <def>A bureau of the United States government charged with the topographic and hydrographic survey of the coast and the execution of belts of primary triangulation and lines of precise leveling in the interior. It now belongs to the Department of Commerce and Labor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coaster</ent><br/
<hw>Coast"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A vessel employed in sailing along a coast, or engaged in the coasting trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who sails near the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coasting</ent><br/
<hw>Coast"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sailing along or near a coast, or running between ports along a coast.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coasting trade</b></col>, <cd>trade carried on by water between neighboring ports of the same country, as distinguished from foreign trade or trade involving long voyages.</cd> -- <col><b>Coasting vessel</b></col>, <cd>a vessel employed in coasting; a coaster.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coasting</ent><br/
<hw>Coast"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sailing along a coast, or from port to port; a carrying on a coasting trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sliding down hill; sliding on a sled upon snow or ice.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coastways</ent><br/
<ent>Coastwise</ent><br/
<mhw>{ <hw>Coast"wise`</hw> <pr>(-w<imac/z`)</pr>, <hw>Coast"ways`</hw> <pr>(?)</pr>, }</mhw> <pos>adv.</pos> <def>By way of, or along, the coast; following a coastline; <as>as, <ex>coastwise</ex> winds contributed to the storm</as>.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Coat</ent><br/
<hw>Coat</hw> <pr>(k<omac/t; 110)</pr>, <pos>n.</pos> <ety>[OF. <ets>cote</ets>, F. <ets>cotte</ets>, petticoat, <ets>cotte d'armes</ets> coat of arms, <ets>cotte de mailles</ets> coat of mail, LL. <ets>cota</ets>, <ets>cotta</ets>, tunic, prob. of German origin; cf. OHG. <ets>chozzo</ets> coarse mantle, G. <ets>klotze</ets>, D. <ets>kot</ets>, hut, E. <ets>cot</ets>. Cf. <er>Cot</er> a hut.]</ety> <sn>1.</sn> <def>An outer garment fitting the upper part of the body; especially, such a garment worn by men.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let each<br/
His adamantine <qex>coat</qex> gird well.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A petticoat.</def> <mark>[Obs.]</mark> <ldquo/A child in <xex>coats</xex>.<rdquo/  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The habit or vesture of an order of men, indicating the order or office; cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men of his <qex>coat</qex> should be minding their prayers.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She was sought by spirits of richest <qex>coat</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An external covering like a garment, as fur, skin, wool, husk, or bark; <as>as, the horses <ex>coats</ex> were sleek</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fruit of all kinds, in <qex>coat</qex><br/
Rough or smooth rined, or bearded husk, or shell.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A layer of any substance covering another; a cover; a tegument; <as>as, the <ex>coats</ex> of the eye; the <ex>coats</ex> of an onion; a <ex>coat</ex> of tar or varnish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Same as <xex>Coat of arms</xex>. See below.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hark, countrymen! either renew the fight,<br/
Or tear the lions out of England's <qex>coat</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A coat card. See below.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here's a trick of discarded cards of us! We were ranked with <qex>coats</qex> as long as old master lived.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coat armor</b></col>. <cd>See under <er>Armor</er>.</cd> -- <col><b>Coat of arms</b></col> <fld>(Her.)</fld>, <cd>a translation of the French <xex>cotte d'armes</xex>, a garment of light material worn over the armor in the 15th and 16th centuries. This was often charged with the heraldic bearings of the wearer. Hence, an heraldic achievement; the bearings of any person, taken together.</cd> -- <col><b>Coat card</b></col>, <cd>a card bearing a coated figure; the king, queen, or knave of playing cards.</cd> <ldquo/<lsquo/I am a <xex>coat card</xex> indeed.' <lsquo/Then thou must needs be a knave, for thou art neither king nor queen.'<rdquo/ <au>Rowley.</au> -- <col><b>Coat link</b></col>, <cd>a pair of buttons or studs joined by a link, to hold together the lappels of a double-breasted coat; or a button with a loop for a single-breasted coat.</cd> -- <col><b>Coat of mail</b></col>, <cd>a defensive garment of chain mail.</cd>  See <cref>Chain mail</cref>, under <er>Chain</er>. -- <col><b>Mast coat</b></col> <fld>(Naut.)</fld>, <cd>a piece of canvas nailed around a mast, where it passes through the deck, to prevent water from getting below.</cd> -- <col><b>Sail coat</b></col> <fld>(Naut.)</fld>, <cd>a canvas cover laced over furled sails, and the like, to keep them dry and clean.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coat</ent><br/
<hw>Coat</hw> <pr>(k<omac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coating</conjf>.]</vmorph> <sn>1.</sn> <def>To cover with a coat or outer garment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover with a layer of any substance; <as>as, to <ex>coat</ex> a jar with tin foil; to <ex>coat</ex> a ceiling</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coatdress</ent><br/
<hw>coatdress</hw> <pos>n.</pos> <def>a dress that is tailored like a coat and buttons up the front.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coated</ent><br/
<hw>coated</hw> <pos>adj.</pos> <sn>1.</sn> <def>covered with an outer layer or film; <as>as, sugar-<ex>coated</ex> pills</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>covered with a shiny coating by applying e.g. beaten egg or a sugar or gelatin mixture; -- of foods.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <fld>(Photog.)</fld> <def>coated on the side opposite the emulsion with a substance to absorb light; -- of film.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coatee</ent><br/
<hw>Coat*ee"</hw> <pr>(k<osl/t*<emac/")</pr>, <pos>n.</pos> <def>A coat with short flaps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coati</ent><br/
<hw>Co*a"ti</hw> <pr>(k<osl/*<aum/"t<esl/ <it>or</it> k<osl/*<amac/"t<icr/)</pr>, <pos>n.</pos> <ety>[From the native name: cf. F. <ets>coati</ets>.]</ety> <fld>(Zool.)</fld> <def>A mammal of tropical America of the genus <gen>Nasua</gen>, allied to the raccoon, having a ringed tail but with a longer body, tail, and nose; -- called also <altname>coati mondi</altname> and <altname>coati mundi</altname>.</def><br/
<syn><b>Syn. --</b> coati, coati-mondi, coati-mundi, coon cat, <spn>Nasua narica</spn>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The red coati (<spn>Nasua socialis</spn>), called also <altname>coati mondi</altname>, inhabits Mexico and Central America. The brown coati (<spn>Nasua narica</spn>) is found in Surinam and Brazil.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>coati-mundi</ent><br/
<ent>coati-mondi</ent><br/
<ent>coati mundi</ent><br/
<ent>coati mondi</ent><br/
<mhw><hw>coati mondi</hw>, <hw>coati mundi</hw>, <hw>coati-mondi</hw>, <hw>coati-mundi</hw></mhw> <pos>n.</pos> <def>same as <er>coati</er>.</def><br/
<syn><b>Syn. --</b> coati, coon cat, <spn>Nasua narica</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coating</ent><br/
<hw>Coat"ing</hw> <pr>(k<omac/"<icr/ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A coat or covering; a layer of any substance, as a cover or protection; <as>as, the <ex>coating</ex> of a retort or vial</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cloth for coats; <as>as, an assortment of <ex>coatings</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coatless</ent><br/
<hw>Coat"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not wearing a coat; also, not possessing a coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coatroom</ent><br/
<hw>coatroom</hw> <pos>n.</pos> <def>a room where coats and other articles can be left temporarily; -- called also <altname>cloakroom</altname>.</def><br/
<syn><b>Syn. --</b> cloakroom.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coattail</ent><br/
<hw>coat"tail</hw> <pos>n.</pos> <def>the loose back flap of a coat that hangs below the waist.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coattails</ent><br/
<hw>coattails</hw> <pos>n. pl.</pos> <mark>[fig.]</mark> <def>the influence of another person; the effect of association with another person; <as>as, he was elected to office on the president's <ex>coattails</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>on the coattails of</b></col> <sd>(a)</sd> <cd>helped by association with another person. See <er>coattails</er>.</cd>  <sd>(b)</sd> <cd>caused by, or immediately following (an event).</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>coauthor</ent><br/
<hw>coauthor</hw> <pos>n.</pos> <def>a writer who collaborates with others in writing something.</def><br/
<syn><b>Syn. --</b> joint author.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>co-author</ent><br/
<hw>co-author</hw> <pos>v. t.</pos> <sn>1.</sn> <def>to to be an author of, together with another author or authors; to author jointly with others.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to exist together at the same time.</def><br/
<syn><b>Syn. --</b> coexist.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coax</ent><br/
<hw>Coax</hw> <pr>(k<omac/ks; 110)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coaxed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coaxing</conjf>.]</vmorph> <ety>[Cf. OE. <ets>cokes</ets> fool, a person easily imposed upon, W. <ets>coeg</ets> empty, foolish; F. <ets>coquin</ets> knave, rogue.]</ety> <def>To persuade by gentle, insinuating courtesy, flattering, or fondling; to wheedle; to soothe.</def></p>

<p><syn><b>Syn.</b> -- To wheedle; cajole; flatter; persuade; entice.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coax</ent><br/
<hw>Coax</hw>, <pos>n.</pos> <def>A simpleton; a dupe.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaxation</ent><br/
<hw>Co`ax*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the noise of frogs.]</ety> <def>The act of croaking.</def> <mark>[R]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaxer</ent><br/
<hw>Coax"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who coaxes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coaxial</ent><br/
<hw>coaxial</hw> <pos>adj.</pos> <def>having a common axis.</def><br/
<syn><b>Syn. --</b> coaxal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coaxing</ent><br/
<hw>coax"ing</hw> <pos>adj.</pos> <sn>1.</sn> <def><pos>p. pr.</pos> of <er>coax</er>; <as>as, the boys' <ex>coaxing</ex> voices</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>Pleasingly persuasive or intended to persuade; <as>as, bending in <ex>coaxing</ex> postures over the guns</as>.</def><br/
<syn><b>Syn. --</b> ingratiatory.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coaxingly</ent><br/
<hw>Coax"ing*ly</hw>, <pos>adv.</pos> <def>In a coaxing manner; by coaxing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cob</ent><br/
<hw>Cob</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>cop</ets>, <ets>copp</ets>, head, top, D. <ets>kop</ets>, G. <ets>kopf</ets>, <ets>kuppe</ets>, LL. <ets>cuppa</ets> cup (cf. E. <ets>brainpan</ets>), and also W. <ets>cob</ets> tuft, spider, <ets>cop</ets>, <ets>copa</ets>, top, summit, <ets>cobio</ets> to thump. Cf. <er>Cop</er> top, <er>Cup</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The top or head of anything.</def> <mark>[Obs.]</mark>  <rj><au>W. Gifford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A leader or chief; a conspicuous person, esp. a rich covetous person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All cobbing country chuffs, which make their bellies and their bags their god, are called rich <qex>cobs</qex>.</q> <rj><qau>Nash.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The axis on which the kernels of maize or indian corn grow.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A spider; perhaps from its shape; it being round like a head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A young herring.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>A fish; -- also called <altname>miller's thumb</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A short-legged and stout horse, esp. one used for the saddle.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Zool.)</fld> <def>A sea mew or gull; esp., the black-backed gull (<spn>Larus marinus</spn>).</def> <altsp>[Written also <asp>cobb</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A lump or piece of anything, usually of a somewhat large size, as of coal, or stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A cobnut; <as>as, Kentish <ex>cobs</ex></as>. See <er>Cobnut</er>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Clay mixed with straw.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The poor cottager contenteth himself with <qex>cob</qex> for his walls, and thatch for his covering.</q> <rj><qau>R. Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A punishment consisting of blows inflictod on tho buttocas with a strap or a flat piece of wood.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>A Spanish coin formerly current in Ireland, worth abiut 4s. 6d.</def> <mark>[Obs.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cob coal</b></col>, <cd>coal in rounded lumps from the size of an egg to that of a football; -- called also <altname>cobbles</altname>.</cd> <au>Grose.</au> -- <col><b>Cob loaf</b></col>, <cd>a crusty, uneven loaf, rounded at top.</cd> <au>Wright.</au> -- <col><b>Cob money</b></col>, <cd>a kind of rudely coined gold and silver money of Spanish South America in the eighteenth century. The coins were of the weight of the piece of eight, or one of its aliquot parts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cob</ent><br/
<hw>Cob</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cobbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cobbing</conjf>.]</vmorph> <sn>1.</sn> <def>To strike</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>To break into small pieces, as ore, so as to sort out its better portions.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>To punish by striking on the buttocks with a strap, a flat piece of wood, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobaea</ent><br/
<ent>Cobaea</ent><br/
<mhw><hw>Co*bae"a</hw>, <hw>Co*b<ae/"a</hw></mhw> <pr>(k<osl/*b<emac/"<adot/)</pr>, <pos>prop. n.</pos> <ety>[Named after <person>D. <etsep>Cobo</etsep></person>, a Spanish botanist.]</ety> <def>A genus of climbing plants, native of Mexico and South America. <spn>Cobaea scandens</spn> (called <stype>cup-and-saucer vine</stype>, <stype>monastery bells</stype>, or <stype>Mexican ivy</stype>) is a conservatory climber with large bell-shaped flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobalt</ent><br/
<hw>Co"balt</hw> <pr>(k<omac/"b<ocr/lt; 277, 74)</pr>, <pos>n.</pos> <ety>[G. <ets>kobalt</ets>, prob. fr. <ets>kobold</ets>, <ets>kobel</ets>, goblin, MHG. <ets>kobolt</ets>; perh. akin to G. <ets>koben</ets> pigsty, hut, AS. <ets>cofa</ets> room, <ets>cofgodas</ets> household gods, Icel. <ets>kofi</ets> hut. If so, the ending <ets>-old</ets> stands for older <ets>-walt</ets>, <ets>-wald</ets>, being the same as <ets>-ald</ets> in E. <ets>herald</ets> and the word would mean <ets>ruler</ets> or <ets>governor in a house</ets>, <ets>house spirit</ets>, the metal being so called by miners, because it was poisonous and troublesome. Cf. <er>Kobold</er>, <er>Cove</er>, <er>Goblin</er>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A tough, lustrous, reddish white metal of the iron group, not easily fusible, and somewhat magnetic. Atomic weight 59.1. Symbol Co.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It occurs in nature in combination with arsenic, sulphur, and oxygen, and is obtained from its ores, smaltite, cobaltite, asbolite, etc. Its oxide colors glass or any flux, as borax, a fine blue, and is used in the manufacture of smalt. It is frequently associated with nickel, and both are characteristic ingredients of meteoric iron.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A commercial name of a crude arsenic used as fly poison.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cobalt bloom</b></col>. <cd>Same as <er>Erythrite</er>.</cd> -- <col><b>Cobalt blue</b></col>, <cd>a dark blue pigment consisting of some salt of cobalt, as the phosphate, ignited with alumina; -- called also <altname>cobalt ultramarine</altname>, and <altname>Thenard's blue</altname>.</cd> -- <col><b>Cobalt crust</b></col>, <cd>earthy arseniate of cobalt.</cd> -- <col><b>Cobalt glance</b></col>. <fld>(Min.)</fld> <cd>See <er>Cobaltite</er>.</cd> -- <col><b>Cobalt green</b></col>, <cd>a pigment consisting essentially of the oxides of cobalt and zinc; -- called also <altname>Rinman's green</altname>.</cd> -- <col><b>Cobalt yellow</b></col> <fld>(Chem.)</fld>, <cd>a yellow crystalline powder, regarded as a double nitrite of cobalt and potassium.</cd></cs><-- Cobalt glass, a deep purple glass colored by cobalt, used in tableware and decorative items. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobaltic</ent><br/
<hw>Co*balt"ic</hw> <pr>(?; 74)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cobaltique</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, derived from, or containing, cobalt; -- said especially of those compounds in which cobalt has higher valence; <as>as, <ex>cobaltic</ex> oxide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Luteo-cobaltic compounds</b></col> <fld>(Chem.)</fld>, <cd>an extensive series of complex yellow compounds of ammonia and cobaltic salts.</cd> -- <col><b>Roseo-cobaltic compounds</b></col> <fld>(Chem.)</fld>, <cd>an extensive series of complex red compounds of cobalt and ammonia. Modifications of these are the <stype>purpureo-cobaltic compounds</stype>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobaltiferous</ent><br/
<hw>Co`balt*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cobalt</ets> + <ets>-ferous</ets>.]</ety> <fld>(Min.)</fld> <def>Containing cobalt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobaltite</ent><br/
<ent>Cobaltine</ent><br/
<mhw>{ <hw>Co"balt*ine</hw> <pr>(?)</pr>, <hw>Co"balt*ite</hw> <pr>(?)</pr> }</mhw> <pos>n.</pos> <fld>(Min.)</fld> <def>A mineral of a nearly silver-white color, composed of arsenic, sulphur, and cobalt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobaltous</ent><br/
<hw>Co*balt"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, derived from, or containing, cobalt; -- said esp. of cobalt compounds in which the metal has its lower valence.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cobaltous chloride</b></col>, <cd>a crystalline compound, <chform>CoCl2</chform>, of a pale rose color when hydrous, blue when dehydrated. Its solution is used for a sympathetic ink, the writing being nearly colorless when dried in the air, owing to absorbed moisture, and becoming bright blue when warmed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobbing</ent><br/
<hw>Cob"bing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Haughty; purse-proud. See <er>Cob</er>, <pos>n.</pos>, 2.</def> <mark>[Obs.]</mark>  <rj><au>Withals (1608).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobble</ent><br/
<hw>Cob"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fishing boat. See <er>Coble</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobble</ent><br/
<hw>Cob"ble</hw>, <pos>n.</pos> <ety>[From <er>Cob</er> a lump. See <er>Cob</er>, <pos>n.</pos>, 9, and cf. <er>Copple</er>, <er>Copplestone</er>.]</ety> <sn>1.</sn> <def>A cobblestone.</def> <ldquo/Their slings held <xex>cobbles</xex> round.<rdquo/  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Cob coal. See under <er>Cob</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 272 --></p>

<p><ent>Cobble</ent><br/
<hw>Cob"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cobbled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cobbling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OF. <ets>cobler</ets>, <ets>copler</ets>, to join or knit together, couple, F. <ets>coupler</ets>, L. <ets>copulare</ets> to couple, join. Cf. <er>Couple</er>, <pos>n.</pos> & <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To make or mend coarsely; to patch; to botch; <as>as, to <ex>cobble</ex> shoes</as>.</def> <au>Shak.</au>  <ldquo/A <qex>cobbled</qex> saddle.<rdquo/ <au>Thackeray.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make clumsily.</def> <ldquo/<xex>Cobbled</xex> rhymes.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pave with cobblestones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobbler</ent><br/
<hw>Cob"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A mender of shoes.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A clumsy workman.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A beverage. See <cref>Sherry cobbler</cref>, under <er>Sherry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cobbler fish</b></col> <fld>(Zool.)</fld>, <cd>a marine fish (<spn>Blepharis crinitus</spn>) of the Atlantic. The name alludes to its threadlike fin rays.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobblestone</ent><br/
<hw>Cob"ble*stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large pebble; a rounded stone not too large to be handled; a small boulder; -- used for paving streets and for other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobby</ent><br/
<hw>Cob"by</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cob</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Headstrong; obstinate.</def> <mark>[Obs.]</mark>  <rj><au>Brockett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Stout; hearty; lively.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobelligerent</ent><br/
<hw>Co`bel*lig"er*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Carrying on war in conjunction with another power.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobelligerent</ent><br/
<hw>Co`bel*lig"er*ent</hw>, <pos>n.</pos> <def>A nation or state that carries on war in connection with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobia</ent><br/
<hw>Co"bi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An oceanic fish of large size (<spn>Elacate canada</spn>); the crabeater; -- called also <altname>bonito</altname>, <altname>cubbyyew</altname>, <altname>coalfish</altname>, and <altname>sergeant fish</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobiron</ent><br/
<hw>Cob"i`ron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cob</er> the top.]</ety> <def>An andiron with a knob at the top.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobishop</ent><br/
<hw>Co`bish"op</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint or coadjutant bishop.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coble</ent><br/
<hw>Co"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cuopel</ets>; cf. W. <ets>ceubal</ets> skiff, ferryboat.]</ety> <def>A flat-floored fishing boat with a lug sail, and a drop rudder extending from two to four feet below the keel. It was originally used on the stormy coast of Yorkshire, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobnut</ent><br/
<hw>Cob"nut`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Com.)</fld> <def>A large roundish variety of the cultivated hazelnut.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A game played by children with nuts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coboose</ent><br/
<hw>Co*boose"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Caboose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobourg</ent><br/
<hw>Co"bourg</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named from the town of <etsep>Coburg</etsep> in Germany.]</ety> <def>A thin worsted fabric for women's dresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobra</ent><br/
<hw>Co"bra</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Copra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobra</ent><br/
<hw>Co"bra</hw>, <pos>n.</pos> <def>The cobra de capello.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobra de capello</ent><br/
||<hw>Co"bra de ca*pel"lo</hw> <pr>(?)</pr>. <ety>[Pg., serpent of the hood.]</ety> <fld>(Zool.)</fld> <def>The hooded snake (<spn>Naia tripudians</spn>), a highly venomous serpent inhabiting India.<-- now <gen>Naja</gen> --></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobstone</ent><br/
<hw>Cob"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cobblestone.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobswan</ent><br/
<hw>Cob"swan`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large swan.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobwall</ent><br/
<hw>Cob"wall`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cob</ets> clay mixed with straw + <ets>wall</ets>.]</ety> <def>A wall made of clay mixed with straw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobweb</ent><br/
<hw>Cob"web`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cob</ets> a spider + <ets>web</ets>.]</ety> <sn>1.</sn> <def>The network spread by a spider to catch its prey.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A snare of insidious meshes designed to catch the ignorant and unwary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not but lament thy splendid wit<br/
Entangled in the <qex>cobwebs</qex> of the schools.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is thin and unsubstantial, or flimsy and worthless; rubbish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dust and <qex>cobwebs</qex> of that uncivil age.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The European spotted flycatcher.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cobweb lawn</b></col>, <cd>a fine linen, mentioned in 1640 as being in pieces of fifteen yards.</cd>  <rj><au>Beck. Draper's Dict.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a proud piece of <qex>cobweb lawn</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj></p>

<p>-- <col><b>Cobweb micrometer</b></col>, <cd>a micrometer in which threads of cobweb are substituted for wires.</cd></cs><-- note that here "cobweb" is used as a mass noun (non-count), considered as a material --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobwebbed</ent><br/
<hw>Cob"webbed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in cobwebs.</def> <ldquo/The <xex>cobwebbed</xex> cottage.<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobwebby</ent><br/
<hw>Cob"web`by</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in cobwebs, or any fine web; resembling a cobweb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cobwork</ent><br/
<hw>Cob"work`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Built of logs, etc., laid horizontally, with the ends dovetailed together at the corners, as in a log house; in marine work, often surrounding a central space filled with stones; <as>as, a <ex>cobwork</ex> dock or breakwater</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coca</ent><br/
<hw>Co"ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. native name.]</ety> <def>The dried leaf of a South American shrub (<spn>Erythroxylon Coca</spn>). In med., called <xex>Erythroxylon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Coca leaves resemble tea leaves in size, shape, and odor, and are chewed (with an alkali) by natives of Peru and Bolivia to impart vigor in prolonged exertion, or to sustain strength in absence of food.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Mexican coca</b></col>, <cd>an American herb (<spn>Richardsonia scabra</spn>), yielding a nutritious fodder. Its roots are used as a substitute for ipecacuanha.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocagne</ent><br/
<hw>Coc*agne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cocagne</ets>, <ets>pays de cocagne</ets>; of uncertian origin, cf. Prov. F. <ets>couque</ets> cake, Catal. <ets>coca</ets>, L. <ets>coquere</ets> to cook; as if the houses in this country were covered with cakes. Cf. <er>Cook</er>, <er>Cockney</er>.]</ety> <sn>1.</sn> <def>An imaginary country of idleness and luxury.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The land of cockneys; cockneydom; -- a term applied to London and its suburbs.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocaine</ent><br/
<hw>Co"ca*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A powerful narcotic alkaloid, <chform>C17H21NO4</chform>, obtained from the leaves of coca. It is a bitter, white, crystalline substance, and is remarkable for producing local insensibility to pain.  It is classified as addictive and is not available in the U. S. without a prescription, but is nevertheless one of the most widespread illegal drugs of abuse.  It is used in several forms, including small pellets of free base, called <stype>crack</stype>.  Most of the cacaine illegally used in the U.S. is imported.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cocainism</ent><br/
<hw>Co*ca"in*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A morbid condition produced by the habitual and excessive use of cocaine.</def> -- <wordforms><wf>Co*ca"in*ist</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cocainize</ent><br/
<hw>Co*ca"in*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To treat or an<ae/sthetize with cocaine.</def> -- <wordforms><wf>Co*ca`in*i*za"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>coccidioidomycosis</ent><br/
<hw>coccidioidomycosis</hw> <pos>n.</pos> <def>a disease of the lungs and skin characterized by excessive sputum and nodules.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coccidiosis</ent><br/
<hw>coccidiosis</hw> <pos>n.</pos> <fld>(Med.)</fld> <def>any of a group of infectious diseases caused by protozoan parasites (coccidia) of the order <ord>Coccidia</ord>, which infect the intestines of birds, dogs, and other vertebrates.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>coccidium</ent><br/
<hw>coccidium</hw> <pos>n.</pos>; <plu><it>pl.</it>  <plw>coccidia</plw> <pr>(#)</pr>.</plu> <fld>(Microbiol.)</fld> <def>any sporozoan of the order <ord>Coccidia</ord>, parasitic on the digestive epithelium of vertebrates and higher invertebrates.  It is the causative agent of <causes>coccidiosis</causes>, which can be a problem in poultry rearing.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cocciferous</ent><br/
<hw>Coc*cif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coccum</ets> a berry + <ets>-ferous</ets>. See <er>Coccus</er>.]</ety> <def>Bearing or producing berries; bacciferous; <as>as, <ex>cocciferous</ex> trees or plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccinella</ent><br/
||<hw>Coc`ci*nel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>coccineus</ets> scarlet-colored. See <er>Cochoneal</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of small beetles of many species. They and their larv<ae/ feed on aphids or plant lice, and hence are of great benefit to man. Also called <altname>ladybirds</altname> and <altname>ladybugs</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccinellidae</ent><br/
<hw>Coccinellidae</hw> <pos>n.</pos> <def>a natural family of insects including the ladybugs.</def><br/
<syn><b>Syn. --</b> family <fam>Coccinellidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coccobacterium</ent><br/
||<hw>Coc`co*bac*te"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  <plw>Coccobacteria</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a grain + NL. <ets>bacterium</ets>. So called from its round shape.]</ety> <fld>(Biol.)</fld> <def>One of the round variety of bacteria, a vegetable organism, generally less than a thousandth of a millimeter in diameter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coccoid</ent><br/
<hw>coccoid</hw> <pos>adj.</pos> <fld>(Microbiol.)</fld> <def>spherical; like a coccus; <as>as, a <ex>coccoid</ex> microorganism</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coccoidea</ent><br/
<hw>Coccoidea</hw> <pos>n.</pos> <def>a superfamily of scale insects and mealybugs.</def><br/
<syn><b>Syn. --</b> superfamily <fam>Coccoidea</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coccolite</ent><br/
<hw>Coc"co*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a grain, seed + <ets>-lite</ets>: cf. F. <ets>coccalite</ets>.]</ety> <fld>(Min.)</fld> <def>A granular variety of pyroxene, green or white in color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccolith</ent><br/
<hw>Coc"co*lith</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a grain, seed + <ets>-lith</ets>.]</ety> <fld>(Biol.)</fld> <def>One of a kind of minute, calcareous bodies, probably vegetable, often abundant in deep-sea mud.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccosphere</ent><br/
<hw>Coc"co*sphere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a grain, seed + E. <ets>sphere</ets>.]</ety> <fld>(Biol.)</fld> <def>A small, rounded, marine organism, capable of braking up into coccoliths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccosteus</ent><br/
||<hw>Coc*cos"te*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/ grain, seed + <?/ bone.]</ety> <fld>(Paleon.)</fld> <def>An extinct genus of Devonian ganoid fishes, having the broad plates about the head studded with berrylike tubercles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccothraustes</ent><br/
<hw>Coccothraustes</hw> <pos>n.</pos> <def>a genus of large finches.</def><br/
<syn><b>Syn. --</b> genus <gen>Coccothraustes</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cocculus</ent><br/
<hw>Cocculus</hw> <pos>n.</pos> <def>a genus of climbing plants or shrubs.</def><br/
<syn><b>Syn. --</b> genus <gen>Cocculus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cocculus Indicus</ent><br/
||<hw>Coc"cu*lus In"di*cus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>cocculus</ets> (dim. of L. <ets>coccum</ets> kermes berry) + L. <ets>Indicus</ets> of India.]</ety> <fld>(Bot.)</fld> <def>The fruit or berry of the <spn>Anamirta Cocculus</spn>, a climbing plant of the East Indies. It is a poisonous narcotic and stimulant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccus</ent><br/
||<hw>Coc"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cocci</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ grain, seed. See <er>Cochineal</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>One of the separable carpels of a dry fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of hemipterous insects, including scale insects, and the cochineal insect (<spn>Coccus cacti</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>A form of bacteria, shaped like a globule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccygeal</ent><br/
<hw>Coc*cyg"e*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the coccyx; <as>as, the <ex>coccygeal</ex> vertebr<ae/</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coccygeal glands</b></col> <fld>(Zool.)</fld> <cd>, glands situated at the base of the tail of birds. They secrete the oil with which the plumage is dressed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccygeous</ent><br/
<hw>Coc*cyg"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Coccygeal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coccyx</ent><br/
||<hw>Coc"cyx</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Coccyges</plw> <pr>(#)</pr>.</plu> <ety>[L., cuckoo, Gr. <?/, cuckoo, coccyx. So called from its resemblance to the beak of a cuckoo.]</ety> <fld>(Anat.)</fld> <def>The end of the vertebral column beyond the sacrum in man and tailless monkeys. It is composed of several vertebr<ae/ more or less consolidated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cochin</ent><br/
<hw>cochin</hw> <pos>n.</pos> <ety>[a shortened form of <ets>cochin-china fowl</ets>.]</ety> <def>an Asian breed of large fowl with dense plumage and feathered legs.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cochineal</ent><br/
<hw>Coch"i*neal</hw> <pr>(k<ocr/ch"<icr/*n<emac/l; 277)</pr>, <ety>[Sp. <ets>cochinilla</ets>, dim. from L. <ets>coccineus</ets>, <ets>coccinus</ets>, scarlet, fr. <ets>coccum</ets> the kermes berry, G. <grk>ko`kkos</grk> berry, especially the kermes insect, used to dye scarlet, as the cochineal was formerly supposed to be the grain or seed of a plant, and this word was formerly defined to be the grain of the <spn>Quercus coccifera</spn>; but cf. also Sp. <ets>cochinilla</ets> wood louse, dim. of <ets>cochina</ets> sow, akin to F. <ets>cochon</ets> pig.]</ety> <def>A dyestuff consisting of the dried bodies of females of the <spn>Coccus cacti</spn>, an insect native in Mexico, Central America, etc., and found on several species of cactus, esp. <spn>Opuntia cochinellifera</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ These insects are gathered from the plant, killed by the application of heat, and exposed to the sun to dry. When dried they resemble small, rough berries or seeds, of a brown or purple color, and form the cochineal of the shops, which is used for making carmine, and also as a red dye.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Cochineal contains as its essential coloring matter <xex>carminic acid</xex>, a purple red amorphous substance which yields <xex>carmine red</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochineal fig</ent><br/
<hw>Coch"i*neal fig</hw> <pr>(?)</pr>, <fld>(Bot.)</fld> <def>A plant of Central and Southern America, of the Cactus family, extensively cultivated for the sake of the cochineal insect, which lives on it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochin fowl</ent><br/
<hw>Co"chin fowl`</hw> <pr>(?)</pr>, <fld>(Zool.)</fld> <def>A large variety of the domestic fowl, originally from Cochin China (Anam).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochlea</ent><br/
||<hw>Coch"le*a</hw> <pr>(k<ocr/k"l<esl/*<adot/)</pr>, <pos>n.</pos> <ety>[L., a snail, or snail shell, Gr. <grk>kochli`as</grk> a snail, fr. <grk>ko`chlos</grk> a shellfish with a spiral shell.]</ety> <fld>(Anat.)</fld> <def>An appendage of the labyrinth of the internal ear, which is elongated and coiled into a spiral in mammals. See <er>Ear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochlear</ent><br/
<hw>Coch"le*ar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the cochlea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochleare</ent><br/
||<hw>Coch`le*a"re</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <sn>1.</sn> <def>A spoon.</def>  <rj><au>Andrews.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med)</fld> <def>A spoonful.</def>  <rj><au>Dungleson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochleariform</ent><br/
<hw>Coch`le*ar"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cochleare</ets> + <ets>-form</ets>.]</ety> <def>Spoon-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochleary</ent><br/
<hw>Coch"le*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cochlearum</ets> penfor snails (meaning formerly given, <ets>snail shell</ets>). See <er>Cjchlea</er>.]</ety> <def>Same as <er>Cochleate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cochleated</ent><br/
<ent>Cochleate</ent><br/
<mhw>{ <hw>Coch"le*ate</hw> <pr>(?)</pr>, <hw>Coch"le*a`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cochleatus</ets> spiral or screw-formed. See <er>Cochlea</er>.]</ety> <def>Having the form of a snail shell; spiral; turbinated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw> <pr>(k<ocr/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>coc</ets>; of unknown origin, perh. in imitation of the cry of the cock. Cf. <er>Chicken</er>.]</ety> <sn>1.</sn> <def>The male of birds, particularly of gallinaceous or domestic fowls.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vane in the shape of a cock; a weathercock.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Drenched our steeples, drowned the <qex>cocks</qex>!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A chief man; a leader or master.</def> <mark>[Humorous]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Sir Andrew is the <qex>cock</qex> of the club, since he left us.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The crow of a cock, esp. the first crow in the morning; cockcrow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He begins at curfew, and walks till the first <qex>cock</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A faucet or valve.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Jonsons says, <ldquo/The handle probably had a <xex>cock</xex> on the top; things that were contrived to turn seem anciently to have had that form, whatever was the reason.<rdquo/ Skinner says, because it used to be constructed <xex>in forma crit<ae/ galli</xex>, i.e., in the form of a cock's comb.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The style of gnomon of a dial.</def>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The indicator of a balance.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The bridge piece which affords a bearing for the pivot of a balance in a clock or watch.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>a penis.</def> <mark>[vulgar]</mark><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Ball cock</b></col>. <cd>See under <er>Ball</er>.</cd> -- <col><b>Chaparral cock</b></col>. <cd>See under <er>Chaparral</er>.</cd> -- <mcol><col><b>Cock and bull story</b></col>, <col><b>an extravagant</b></col></mcol>, <cd>boastful story; a canard.</cd> -- <col><b>Cock of the plains</b></col> <fld>(Zool.)</fld> <cd>See <er>Sage cock</er>.</cd> -- <col><b>Cock of the rock</b></col> <fld>(Zool.)</fld>, <cd>a South American bird (<spn>Rupicola aurantia</spn>) having a beautiful crest.</cd> -- <col><b>Cock of the walk</b></col>, <cd>a chief or master; the hero of the hour; one who has overcrowed, or got the better of, rivals or competitors.</cd> -- <col><b>Cock of the woods</b></col>. <cd>See <er>Capercailzie</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw> <pr>(k<ocr/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cocked</conjf> <pr>(k<ocr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cocking</conjf>.]</vmorph> <ety>[Cf. Gael. <ets>coc</ets> to cock.]</ety> <sn>1.</sn> <def>To set erect; to turn up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our Lightfoot barks, and <qex>cocks</qex> his ears.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Dick would <qex>cock</qex> his nose in scorn.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shape, as a hat, by turning up the brim.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To set on one side in a pert or jaunty manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>cocked</qex> their hats in each other's faces.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To turn (the eye) obliquely and partially close its lid, as an expression of derision or insinuation.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cocked hat</b></col>. <sd>(a)</sd> <cd>A hat with large, stiff flaps turned up to a peaked crown, thus making its form triangular; -- called also <altname>three-cornered hat</altname><-- or tricorn -->.</cd> <sd>(b)</sd> <cd>A game similar to ninepins, except that only three pins are used, which are set up at the angles of a triangle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>v. i.</pos> <def>To strut; to swagger; to look big, pert, or menacing.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>n.</pos> <def>The act of cocking; also, the turn so given; <as>as, a <ex>cock</ex> of the eyes; to give a hat a saucy <ex>cock</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>n.</pos> <ety>[It. <ets>cocca</ets> notch of an arrow.]</ety> <sn>1.</sn> <def>The notch of an arrow or crossbow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The hammer in the lock of a firearm.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>At cock</b></col>, <col><b>At full cock</b></col></mcol>, <cd>with the hammer raised and ready to fire; -- said of firearms, also, jocularly, of one prepared for instant action.</cd> -- <col><b>At half cock</b></col>. <cd>See under <er>Half</er>.</cd> -- <col><b>Cock feather</b></col> <fld>(Archery)</fld>, <cd>the feather of an arrow at right angles to the direction of the cock or notch.</cd>  <rj><au>Nares.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>v. t.</pos> <def>To draw the hammer of (a firearm) fully back and set it for firing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>v. i.</pos> <def>To draw back the hammer of a firearm, and set it for firing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cocked</qex>, fired, and missed his man.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>n.</pos> <ety>[Cf. Icel. <ets>k<oum/kkr</ets> lump, Dan. <ets>kok</ets> heap, or E. <ets>cock</ets> to set erect.]</ety> <def>A small concial pile of hay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>v. t.</pos> <def>To put into cocks or heaps, as hay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under the <qex>cocked</qex> hay.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>n.</pos> <ety>[Of. <ets>coque</ets>, F. <ets>coche</ets>, a small vessel, L. <ets>concha</ets> muscle shell, a vessel. See <er>Coach</er>, and cf. <er>Cog</er> a small boat.]</ety> <def>A small boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yond tall anchoring bark [appears]<br/
Diminished to her <qex>cock</qex>; her <qex>cock</qex>, a buoy<br/
Almost too small for sight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock</ent><br/
<hw>Cock</hw>, <pos>n.</pos> <def>A corruption or disguise of the word <xex>God</xex>, used in oaths.</def> <mark>[Obs.]</mark> <ldquo/By <xex>cock</xex> and pie.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockade</ent><br/
<hw>Cock*ade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cocarble</ets>, fr. <ets>coquard</ets> vain, OF. <ets>coquart</ets>, fr. <ets>coq</ets> cock, prob. of imitative origin. The ornament is so named from its resemblance to the crest of a cock. Cf. <er>Coquette</er>.]</ety> <def>A badge, usually in the form of a rosette, or knot, and generally worn upon the hat; -- used as an indication of military or naval service, or party allegiance, and in England as a part of the livery to indicate that the wearer is the servant of a military or naval officer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seduced by military liveries and <qex>cockades</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockaded</ent><br/
<hw>Cock*ad"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wearing a cockade.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cock-a-doodle-doo</ent><br/
<hw>cock-a-doodle-doo</hw> <pos>n.</pos> <def>the crowing sound made by a rooster.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cock-a-hoop</ent><br/
<hw>cock`-a-hoop"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Boastful; defiant; exulting. Also used adverbially.</def><br/
<syn><b>Syn. --</b> boastful, braggart(prenominal), bragging(prenominal), braggy, big, crowing, self-aggrandizing, vainglorious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockal</ent><br/
<hw>Cock"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <sn>1.</sn> <def>A game played with sheep's bones instead of dice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The bone used in playing the game; -- called also <altname>huckle bone</altname>.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A little transverse bone<br/
Which boys and bruckeled children call<br/
(Playing for points and pins) <qex>cockal</qex>.</q> <rj><qau>Herrick.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cock-a-leekie</ent><br/
<ent>Cockaleekie</ent><br/
<mhw><hw>Cock`a*leek"ie</hw>, <hw>cock-a-leekie</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>cock</ets> + <ets>leek</ets>.]</ety> <def>A favorite soup in Scotland, made from a capon highly seasoned, and boiled with leeks and prunes; -- called also <altname>cocky-leeky</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockamaroo</ent><br/
<hw>Cock`a*ma*roo"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The Russian variety of bagatelle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockateel</ent><br/
<ent>cockatiel</ent><br/
<mhw><hw>cock"a*tiel</hw>, <hw>Cock"a*teel</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small gray and white Australian parrot (<spn>Leptolophus hollandicus</spn>, formerly <spn>Calopsitta Nov<ae/-Hollandi<ae/</spn>) with a prominent crest; the male has bright yellow cheeks and crest, but the female has only a pale yellow in the face; -- it is so called from its note.</def><br/
<syn><b>Syn. --</b> cockateel, cockatoo parrot, <spn>Nymphicus hollandicus</spn>.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Cockatoo</ent><br/
<hw>Cock`a*too"</hw> <pr>(k<ocr/k`<adot/*t<oomac/")</pr>, <pos>n.</pos> <ety>[Malayan <ets>kakat<umac/a</ets>.]</ety> <fld>(Zool.)</fld> <def>A bird of the Parrot family, of the subfamily <fam>Cacatuin<ae/</fam>, having a short, strong, and much curved beak, and the head ornamented with a crest, which can be raised or depressed at will. There are several genera and many species; as the <stype>broad-crested cockatoo</stype> (<spn>Plictolophus cristatus</spn> or  <spn>Cacatua cristatus</spn>), the sulphur-crested (<spn>Cacatua galerita</spn> or <spn>Plictolophus galeritus</spn>), etc. The <stype>palm cockatoo</stype> or <stype>great black cockatoo</stype> of Australia is <spn>Probosciger aterrimus</spn> (formerly <spn>Microglossus aterrimus</spn>).</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><-- p. 273 --></p>

<p><ent>Cockatrice</ent><br/
<hw>Cock"a*trice</hw> <pr>(-tr<imac/s; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>cocatrice</ets> crocodile, F. <ets>cocatrix</ets>, <ets>cocatrice</ets>. The word is a corruption from the same source as E. <ets>crocodile</ets>, but was confused with <ets>cock</ets> the bird, F. <ets>coq</ets>, whence arose the fable that the animal was produced from a <ets>cock's</ets> egg. See <er>Crocodile</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A fabulous serpent whose breath and look were said to be fatal. See <er>Basilisk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That bare vowel, I, shall poison more<br/
Than the death-darting eye of <qex>cockatrice</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>A representation of this serpent. It has the head, wings, and legs of a bird, and tail of a serpent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Script.)</fld> <def>A venomous serpent which which cannot now be identified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The weaned child shall put his hand on the <qex>cockatrice's</qex> <note>[Rev. Ver. <qex>basilisk's</qex>]</note> den.</q> <rj><qau>Is. xi. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any venomous or deadly thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This little <qex>cockatrice</qex> of a king.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockbill</ent><br/
<hw>Cock"bill</hw> <pr>(k<ocr/k"b<icr/l`)</pr>, <pos>v. t.</pos> <ety>[See <er>Cock</er> to set erect.]</ety> <fld>(Naut.)</fld> <def>To tilt up one end of so as to make almost vertical; <as>as, to <ex>cockbill</ex> the yards as a sign of mourning</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cockbill the anchor</b></col>, <cd>to suspend it from the cathead preparatory to letting it go. See <er>Acockbill</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockboat</ent><br/
<hw>Cock"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cock</er> a boat.]</ety> <def>A small boat, esp. one used on rivers or near the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cock-brained</ent><br/
<hw>Cock"-brained`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Giddy; rash.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockchafer</ent><br/
<hw>Cock"chaf`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Chafer</er> the beetle.]</ety> <fld>(Zool.)</fld> <def>A beetle of the genus <gen>Melolontha</gen> (esp. <spn>Melolontha vulgaris</spn>) and allied genera; -- called also <altname>May bug</altname>, <altname>chafer</altname>, or <altname>dorbeetle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockcrowing</ent><br/
<ent>Cockcrow</ent><br/
<mhw>{ <hw>Cock"crow</hw> <pr>(?)</pr>, <hw>Cock"crow`ing</hw>, }</mhw> <pos>n.</pos> <def>The time at which cocks first crow; the early morning; the first light of day.</def><br/
<syn><b>Syn. --</b> dawn, dawning, morning, aurora, first light, daybreak, break of day, break of the day, dayspring, sunrise, sunup.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Cocker</ent><br/
<hw>Cock"er</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cockered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cockering</conjf>.]</vmorph> <ety>[OE. <ets>cokeren</ets>; cf. W. <ets>cocru</ets> to indulge, fondle, E. <ets>cock</ets> the bird, F. <ets>coqueliner</ets> to dandle (Cotgrave), to imitate the crow of a cock, to run after the girls, and E. <ets>cockle</ets>, v.]</ety> <def>To treat with too great tenderness; to fondle; to indulge; to pamper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cocker</qex> thy child and he shall make thee afraid.</q> <rj><qau>Ecclesiasticus xxx. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Poor folks cannot afford to <qex>cocker</qex> themselves up.</q> <rj><qau>J. Ingelow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocker</ent><br/
<hw>Cock"er</hw>, <pos>n.</pos> <ety>[From <er>Cock</er> the bird.]</ety> <sn>1.</sn> <def>One given to cockfighting.</def> <mark>[Obs.]</mark>  <rj><au>Steele.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A small dog of the spaniel kind, used for starting up woodcocks, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocker</ent><br/
<hw>Cock"er</hw>, <pos>n.</pos> <ety>[OE. <ets>coker</ets> qyiver, boot, AS. <ets>cocer</ets> quiver; akin to G. <ets>k<oum/cher</ets> quiver, and perh. originally meaning <ets>receptacle</ets>, <ets>holder</ets>. Cf. <er>Quiver</er> (for arrows).]</ety> <def>A rustic high shoe or half-boots.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockerel</ent><br/
<hw>Cock"er*el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. a double dim. of <ets>cock</ets>.]</ety> <def>A young cock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocker spaniel</ent><br/
<hw>Cock"er span"iel</hw>. <def>One of a breed of small or medium-sized spaniels kept for hunting or retrieving game or for household pets. They usually weigh from eighteen to twenty-eight pounds. They have the head of fair length, with square muzzle, the ears long and set low, the legs short or of medium length, and the coat fine and silky, wavy but not curly. Various colors are bred, as black, liver, red, black and white, black and tan, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cocket</ent><br/
<hw>Cock"et</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>coquet</ets> coquettish. See <er>Coquette</er>, <pos>n.</pos>]</ety> <def>Pert; saucy.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocket</ent><br/
<hw>Cock"et</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Eng. Law)</fld> <def>A customhouse seal; a certified document given to a shipper as a warrant that his goods have been duly entered and have paid duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An office in a customhouse where goods intended for export are entered.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A measure for bread.</def> <mark>[Obs.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockeye</ent><br/
<hw>Cock"eye`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>cock</ets> to turn up.]</ety> <def>A squinting eye.</def>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockeye</ent><br/
<hw>Cock"eye`</hw>, <pos>n.</pos> <fld>(Mach.)</fld> <def>The socket in the ball of a millstone, which sits on the cockhead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockfight</ent><br/
<hw>Cock"fight`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A match or contest of gamecocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockfighting</ent><br/
<hw>Cock"fight`ing</hw>, <pos>n.</pos> <def>The act or practice of pitting gamecocks to fight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockfighting</ent><br/
<hw>Cock"fight`ing</hw>, <pos>a.</pos> <def>Addicted to cockfighting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockhead</ent><br/
<hw>Cock"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>The rounded or pointed top of a grinding mill spindle, forming a pivot on which the stone is balanced.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockhorse</ent><br/
<hw>Cock"horse`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A child's rocking-horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ride a <qex>cockhorse</qex> to Banbury cross.</q> <rj><qau>Mother Goose.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A high or tall horse.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockhorse</ent><br/
<hw>Cock"horse`</hw>, <pos>a.</pos> <sn>1.</sn> <def>Lifted up, as one is on a tall horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lofty in feeling; exultant; proud; upstart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our painted fools and <qex>cockhorse</qex> peasantry.</q> <rj><qau>Marlowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockieleekie</ent><br/
<hw>Cock`ie*leek"ie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Cockaleekie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocking</ent><br/
<hw>Cock"ing</hw>, <pos>n.</pos> <def>Cockfighting.</def>  <rj><au>Ben Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockle</ent><br/
<hw>Coc"kle</hw> <pr>(k<ocr/k"k'l)</pr>, <pos>n.</pos> <ety>[OE. <ets>cockes</ets> cockles, AS. <ets>s<aemac/coccas</ets> sea cockles, prob, from Celtic; cf. W. <ets>cocs</ets> cockles, Gael. <ets>cochull</ets> husk. Perh. influenced by F. <ets>coquille</ets> shell, a dim. from the root of E. <ets>conch</ets>. Cf. <er>Coach</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A bivalve mollusk, with radiating ribs, of the genus <gen>Cardium</gen>, especially <spn>Cardium edule</spn>, used in Europe for food; -- sometimes applied to similar shells of other genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cockleshell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The mineral black tourmaline or schorl; -- so called by the Cornish miners.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The fire chamber of a furnace.</def> <mark>[Eng.]</mark>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A hop-drying kiln; an oast.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The dome of a heating furnace.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cockle hat</b></col>, <cd>a hat ornamented with a cockleshell, the badge of a pilgrim.</cd> <au>Shak.</au> -- <col><b>Cockle stairs</b></col>, <cd>winding or spiral stairs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockle</ent><br/
<hw>Coc"kle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cockled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cockling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Of uncertian origin.]</ety> <def>To cause to contract into wrinkles or ridges, as some kinds of cloth after a wetting.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cockling sea</b></col>, <cd>waves dashing against each other with a short and quick motion.</cd>  <rj><au>Ham. Nav. Encyc.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockle</ent><br/
<hw>Coc"kle</hw>, <pos>n.</pos> <ety>[AS. <ets>coccel</ets>, <ets>cocel</ets>; cf. Gael. <ets>cogall</ets> tares, husks, cockle.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A plant or weed that grows among grain; the corn rose (<spn>Luchnis Githage</spn>).</def> <sd>(b)</sd> <def>The <gen>Lotium</gen>, or darnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocklebur</ent><br/
<hw>Coc"kle*bur`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A coarse, composite weed, having a rough or prickly fruit; one of several species of the genus <gen>Xanthium</gen>; -- called also <altname>clotbur</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockled</ent><br/
<hw>Coc"kled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Inclosed in a shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tender horns of <qex>cockled</qex> snails.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockled</ent><br/
<hw>Coc"kled</hw>, <pos>a.</pos> <def>Wrinkled; puckered.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Showers soon drench the camlet's <qex>cockled</qex> grain.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockler</ent><br/
<hw>Coc"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who takes and sells cockles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockleshell</ent><br/
<hw>Coc"kle*shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One of the shells or valves of a cockle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A light boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To board the <qex>cockleshell</qex> in those plunding waters.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockloft</ent><br/
<hw>Cock"loft`</hw> <pr>(?; 115)</pr> <pos>n.</pos> <ety>[Prop., a loft where <ets>cocks</ets> roost.]</ety> <def>An upper loft; a garret; the highest room in a building.</def>  <rj><au>Dryden. Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockmaster</ent><br/
<hw>Cock"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who breeds gamecocks.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockmatch</ent><br/
<hw>Cock"match`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cockfight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockney</ent><br/
<hw>Cock"ney</hw> <pr>(k<ocr/k"n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cockneys</plw> <pr>(-n<icr/z)</pr>.</plu> <ety>[OE. <ets>cocknay</ets>, <ets>cokenay</ets>, a spoiled child, effeminate person, an egg; prob. orig. a cock's egg, a small imperfect egg; OE. <ets>cok</ets> cock + <ets>nay</ets>, <ets>neye</ets>, for <ets>ey</ets> egg (cf. <er>Newt</er>), AS. <ets><ae/g</ets>. See 1st <er>Cock</er>, <er>Egg</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>An effeminate person; a spoilt child.</def> <ldquo/A young heir or <xex>cockney</xex>, that is his mother's darling.<rdquo/  <rj><au>Nash (1592).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This great lubber, the world, will prove a <qex>cockney</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A native or resident of the city of London, especially one living in the East End district; -- sometimes used contemptuously.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>A <qex>cockney</qex> in a rural village was stared at as much as if he had entered a kraal of Hottentots.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>the distinctive dialect of a cockney{2}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cockney</ent><br/
<hw>Cock"ney</hw>, <pos>a.</pos> <def>Of or relating to, or like, cockneys.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockneydom</ent><br/
<hw>Cock"ney*dom</hw> <pr>(k<ocr/k"n<icr/*d<ucr/m)</pr>, <pos>n.</pos> <def>The region or home of cockneys; cockneys, collectively.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockneyfy</ent><br/
<hw>Cock"ney*fy</hw> <pr>(-f<imac/)</pr>, <pos>v. t.</pos> <ety>[<ets>Cockney</ets> + <ets>-fy</ets>.]</ety> <def>To form with the manners or character of a cockney.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockneyish</ent><br/
<hw>Cock"ney*ish</hw>, <pos>a.</pos> <def>Characteristic of, or resembling, cockneys.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockneyism</ent><br/
<hw>Cock"ney*ism</hw> <pr>(k<ocr/k"n<icr/*<icr/z'm)</pr>, <pos>n.</pos> <def>The characteristics, manners, or dialect, of a cockney.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cock-of-the-rock</ent><br/
<ent>cock of the rock</ent><br/
<mhw><hw>cock of the rock</hw>, <hw>cock-of-the-rock</hw></mhw> <pos>n.</pos> <def>a bird of the Andes (<spn>Rupicola peruviana</spn>) similar to <spn>Rupicola rupicola</spn>.</def><br/
<syn><b>Syn. --</b> <spn>Rupicola peruviana</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>tropical bird of Northern South America (<spn>Rupicola rupicola</spn>), the male having brilliant red or orange plumage and an erectile disklike crest.</def><br/
<syn><b>Syn. --</b> cock of the rock, <spn>Rupicola rupicola</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cock-paddle</ent><br/
<hw>Cock"-pad`dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Zool.)</fld> <def>See <er>Lumpfish</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockpit</ent><br/
<hw>Cock"pit`</hw> <pr>(k<ocr/k"p<icr/t)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A pit, or inclosed area, for cockfights.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Henry the Eighth had built . . . a <qex>cockpit</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The Privy Council room at Westminster; -- so called because built on the site of the cockpit of Whitehall palace.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>That part of a war vessel appropriated to the wounded during an engagement.</def> <sd>(b)</sd> <def>In yachts and other small vessels, a space lower than the rest of the deck, which affords easy access to the cabin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In airplanes or boats, the space where the pilot or operator sits to control the vehicle.  In airplanes it is usually in the front of the fuselage.  In larger airplanes it may be closed off from the <contr>cabin</contr>, where the passengers travel.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cockroach</ent><br/
<hw>Cock"roach</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cucaracha</ets>.]</ety> <fld>(Zool.)</fld> <def>An orthopterous insect of the genus <gen>Blatta</gen>, and allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The species are numerous, especially in hot countries. Those most commonly infesting houses in Europe and North America are <spn>Blatta orientalis</spn>, a large species often called <stype>black beetle</stype>, and the <stype>Croton bug</stype> (<spn>Blatta Germanica</spn>, formerly <spn>Ectobia Germanica</spn>), also called the <stype>German cockroach</stype>.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cockscomb</ent><br/
<hw>Cocks"comb`</hw> <pr>(k<ocr/ks"k<omac/m`)</pr>, <pos>n.</pos> <ety>[1st <ets>cock</ets>, n. + <ets>comb</ets> crest.]</ety> <sn>1.</sn> <def>See <er>Coxcomb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A plant (<spn>Celosia cristata</spn>), of many varieties, cultivated for its broad, fantastic spikes of brilliant flowers; -- sometimes called <altname>garden cockscomb</altname>. Also the <gen>Pedicularis</gen>, or lousewort, the <spn>Rhinanthus Crista-galli</spn>, and the <spn>Onobrychis Crista-galli</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cock's-eggs</ent><br/
<hw>cock's-eggs</hw> <pos>n.</pos> <def>a weedy vine of Argentina (<spn>Salpichroa organifolia</spn>) having solitary white flowers followed by egg-shaped white or yellow fruit.</def><br/
<syn><b>Syn. --</b> cock's eggs, <spn>Salpichroa organifolia</spn>, <spn>Salpichroa rhomboidea</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cockshead</ent><br/
<hw>Cocks"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A leguminous herb (<spn>Onobrychis Caput-galli</spn>), having small spiny-crested pods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockshut</ent><br/
<hw>Cock"shut`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of net to catch woodcock.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cockshut time</b></col> <it>or</it> <col><b>Cockshut light</b></col></mcol>, <cd>evening twilight; nightfall; -- so called in allusion to the tome at which the cockshut used to be spread.</cd> <mark>[Obs.]</mark>  <rj><au>Shak. B. Jonson.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockshy</ent><br/
<hw>Cock"shy`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A game in which trinkets are set upon sticks, to be thrown at by the players; -- so called from an ancient popular sport which consisted in <ldquo/shying<rdquo/ or throwing cudgels at live cocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An object at which stones are flung.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Making a <qex>cockshy</qex> of him,<rdquo/ replied the hideous small boy.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockspur</ent><br/
<hw>Cock"spur</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A variety of <gen>Crat<ae/gus</gen>, or hawthorn (<spn>Crat<ae/gus Crus-galli</spn>), having long, straight thorns; -- called also <altname>Cockspur thorn</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cocksucker</ent><br/
<hw>cock"suck*er</hw> <pos>n.</pos> <def>a worthless or despicable person; -- intended as disparaging and invariably offensive.</def> <mark>[vulgar slang]</mark><br/
<syn><b>Syn. --</b> rotter, rat, skunk, stinker, bum, crumb, lowlife, scum bag, motherfucker, mother.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a person who performs fellatio.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cocksure</ent><br/
<hw>Cock"sure`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Perfectly safe.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We steal as in a castle, cocksure: . . . we walk invisible.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Quite certain.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I thought myself <qex>cocksure</qex> of the horse which he readily promised me.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>overconfident; -- of people; <as>as, the team was so <ex>cocksure</ex> of winning that they didn't practice in the week before the big game.</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockswain</ent><br/
<hw>Cock"swain</hw> <pr>(?, <mark>colloq</mark>. ?)</pr>, <pos>n.</pos> <ety>[<ets>Cock</ets> a boat + <ets>swain</ets>; hence, the master of a boat.]</ety> <def>The steersman of a boat; a petty officer who has charge of a boat and its crew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocktail</ent><br/
<hw>Cock"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A beverage made of brandy, whisky, or gin, iced, flavored, and sweetened.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Stock Breeding)</fld> <def>A horse, not of pure breed, but having only one eighth or one sixteenth impure blood in his veins.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mean, half-hearted fellow; a coward.</def> <mark>[Slang, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It was in the second affair that poor little Barney showed he was a <qex>cocktail</qex>.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A species of rove beetle; -- so called from its habit of elevating the tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockup</ent><br/
<hw>Cock"up</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large, highly esteemed, edible fish of India (<spn>Lates calcarifer</spn>); -- also called <altname>begti</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockweed</ent><br/
<hw>Cock"weed</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Peppergrass.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocky</ent><br/
<hw>Cock"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Cocket</er>.]</ety> <def>Pert.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cockyoly, bird</ent><br/
<ent>Cockyolly</ent><br/
<mhw>{ <hw>Cock`y*ol"ly</hw>, <or/ <hw>Cock`y*ol"y, bird</hw>  }</mhw> <pr>(?)</pr>. <ety>[Cf. <er>Cock</er>, fowl; <er>Yellow</er>.]</ety> <def>A pet name for any small bird.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coco palm</ent><br/
<ent>Coco</ent><br/
<mhw>{ <hw>Co"co</hw> <pr>(k<omac/"k<osl/)</pr>, <pos>n.</pos> <it>or</it> <hw>Co"co palm</hw> <pr>(k<omac/"k<osl/ p<aum/m`)</pr>.}</mhw> <def>See <er>Cocoa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocoa palm</ent><br/
<ent>Cocoa</ent><br/
<mhw>{ <hw>Co"coa</hw> <pr>(k<omac/"k<osl/)</pr>, <pos>n.</pos>, <hw>Co"coa palm`</hw> <pr>(k<omac/"k<osl/ p<aum/m`)</pr> }</mhw> <ety>[Sp. & Pg. <ets>coco</ets> cocoanut, in Sp. also, cocoa palm.  The Portuguese name is said to have been given from the monkeylike face at the base of the nut, fr. Pg. <ets>coco</ets> a bugbear, an ugly mask to frighten children. Cf., however, Gr. <grk>koy^ki</grk> the cocoa palm and its fruit, <grk>ko`i:x</grk>, <grk>ko`i:kos</grk>, a kind of Egyptian palm.]</ety> <fld>(Bot.)</fld> <def>A tall palm tree producing the cocoanut (<spn>Cocos nucifera</spn>) as its fruit. It grows in nearly all tropical countries, attaining a height of sixty or eighty feet. The trunk is without branches, and has a tuft of leaves at the top, each being fifteen or twenty feet in length, and at the base of these the nuts hang in clusters; the cocoanut tree. It is widely planted throughout the tropics, and in some locations as an ornamental tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocoa</ent><br/
<hw>Co"coa</hw>, <pos>n.</pos> <ety>[Corrupted fr. <ets>cacao</ets>.]</ety> <def>A preparation made from the seeds of the chocolate tree, and used in making, a beverage; also the beverage made from cocoa or cocoa shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cocoa shells</b></col>, <cd>the husks which separate from the cacao seeds in preparing them for use.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocoanut</ent><br/
<ent>coconut</ent><br/
<mhw><hw>co"co*nut</hw>, <hw>Co"coa*nut`</hw></mhw> <pr>(k<omac/"k<osl/*n<ucr/t`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>the edible white meat of a coconut{3}; often shredded for use in e.g. cakes and curries.</def><br/
<syn><b>Syn. --</b> coconut meat.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the <er>cocoa palm</er>.</def><br/
<syn><b>Syn. --</b> coconut palm, coco palm, coco, cocoa palm, coconut tree, Cocos nucifera.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>The large, hard-shelled oval nut of the <prodby>cocoa palm</prodby>. It has a fibrous husk containing a thick white fibrous meat much used as food, in confections, and in making oil.  It has a central cavity filled (when fresh) with an agreeable milky liquid.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Cocobolas</ent><br/
<ent>Cocobolo</ent><br/
<mhw>{ ||<hw>Co`co*bo"lo</hw> <pr>(?)</pr>, <hw>Co`co*bo"las</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Sp. <ets>cocobolo</ets>.]</ety> <fld>(Bot.)</fld> <def>A very beautiful and hard wood, obtained in the West India Islands. It is used in cabinetmaking, for the handles of tools, and for various fancy articles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocoon</ent><br/
<hw>Co*coon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cocon</ets>, dim. of <ets>coque</ets> shell of egge and insects, fr. L. <ets>concha</ets> mussel shell. See <er>Conch</er>.]</ety> <sn>1.</sn> <def>An oblong case in which the silkworm lies in its chrysalis state. It is formed of threads of silk spun by the worm just before leaving the larval state. From these the silk of commerce is prepared.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The case constructed by any insect to contain its larva or pupa.</def> <sd>(b)</sd> <def>The case of silk made by spiders to protect their eggs.</def> <sd>(c)</sd> <def>The egg cases of mucus, etc., made by leeches and other worms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocoonery</ent><br/
<hw>Co*coon"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A building or apartment for silkworms, when feeding and forming cocoons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cocoswood</ent><br/
<hw>co"cos*wood`</hw> <pr>(?)</pr>. <def>same as <er>cocus wood</er>.</def> <syn><b>Syn. --</b> granadilla wood.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cocozelle</ent><br/
<hw>cocozelle</hw> <pos>n.</pos> <def>a squash plant having a dark green fruit with skin mottled with light green or yellow.</def><br/
<syn><b>Syn. --</b> Italian vegetable marrow.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a squash resembling zucchini.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coctible</ent><br/
<hw>Coc"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Coctile</er>.]</ety> <def>Capable of being cooked.</def>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coctile</ent><br/
<hw>Coc"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coctilis</ets>, fr. <ets>coguere</ets>. <ets>See</ets> <er>Cook</er>.]</ety> <def>Made by baking, or exposing to heat, as a brick.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coction</ent><br/
<hw>Coc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coctio</ets>.]</ety> <sn>1.</sn> <def>Act of boiling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>Digestion.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>The change which the humorists{1} believed morbific matter undergoes before elimination.</def> <mark>[Obs.]</mark>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cocus</ent><br/
<hw>Cocus</hw> <pos>n.</pos> <def>one of the Titans.</def><-- ??## Coeus? Can't find elsewhere. --><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cocus wood</ent><br/
<ent>cocuswood</ent><br/
<mhw><hw>co"cus*wood`</hw>, <hw>co"cus wood`</hw></mhw> <pr>(?)</pr>. <def>A West Indian wood obtained from the <prodby>granadilla tree</prodby>, used for making flutes, clarinets, and other musical instruments.</def> <altsp>[Also spelled <asp>cocoswood</asp>.]</altsp><br/
<syn><b>Syn. --</b> granadilla wood.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>c.o.d.</ent><br/
<hw>c.o.d.</hw> <pos>adj.</pos>  <fld>(Commerce)</fld> <def>an abbreviation of <altname>collect on delivery</altname>; payment due by the recipient on delivery; <as>as, a <ex>COD</ex> parcel</as>.</def> <altsp>[Also spelled <asp>COD</asp>.]</altsp><br/
<syn><b>Syn. --</b> collect, collect on delivery.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>c.o.d.</ent><br/
<hw>c.o.d.</hw> <pos>n.</pos> <mark>abbr.</mark>  <fld>(Commerce)</fld> <def>an abbreviation of <altname>collect on delivery</altname>; a method of payment by which goods are paid for when they are delivered to the customer's home or place of business.  Contrasted to <er>payment in advance</er> or <er>terms</er> or <er>credit</er>.</def><br/
<syn><b>Syn. --</b> collect on delivery.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Cod</ent><br/
<hw>Cod</hw> <pr>(k<ocr/d)</pr>, <pos>n.</pos> <ety>[AS. <ets>codd</ets> small bag; akin to Icel. <ets>koddi</ets> pillow, Sw. <ets>kudde</ets> cushion; cf. W. <ets>cod</ets>, <ets>cwd</ets>, bag, shell.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A husk; a pod; <as>as, a peas<ex>cod</ex></as>.</def> <mark>[Eng.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small bag or pouch.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The scrotum.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A pillow or cushion.</def> <mark>[Prov. Eng.]</mark> <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cod</ent><br/
<hw>Cod</hw>, <pos>n.</pos> <ety>[Cf. G. <ets>gadde</ets>, and (in Heligoland) <ets>gadden</ets>, L. <ets>gadus</ets> merlangus.]</ety> <fld>(Zool.)</fld> <def>An important edible fish (<spn>Gadus morrhua</spn>), taken in immense numbers on the northern coasts of Europe and America. It is especially abundant and large on the Grand Bank of Newfoundland. It is salted and dried in large quantities.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are several varieties; as <stype>shore cod</stype>, from shallow water; <stype>bank cod</stype>, from the distant banks; and <stype>rock cod</stype>, which is found among ledges, and is often dark brown or mottled with red. The <stype>tomcod</stype> is a distinct species of small size. The <stype>bastard</stype>, <stype>blue</stype>, <stype>buffalo</stype>, or <stype>cultus cod</stype> of the Pacific coast belongs to a distinct family. See <stype>Buffalo cod</stype>, under <er>Buffalo</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cod fishery</b></col>, <cd>the business of fishing for cod.</cd> -- <col><b>Cod line</b></col>, <cd>an eighteen-thread line used in catching codfish.</cd>  <rj><au>McElrath.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coda</ent><br/
||<hw>Co"da</hw> <pr>(k<omac/"d<adot/)</pr>, <pos>n.</pos> <ety>[It., tail, fr. L. <ets>cauda</ets>.]</ety> <fld>(Mus.)</fld> <def>A few measures added beyond the natural termination of a composition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codder</ent><br/
<hw>Cod"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A gatherer of cods or peas.</def> <mark>[Obs. or Prov.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codding</ent><br/
<hw>Cod"ding</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lustful.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coddle</ent><br/
<hw>Cod"dle</hw> <pr>(k<ocr/d"d'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coddled</conjf> <pr>(-d'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coddling</conjf> <pr>(-dl<icr/ng)</pr>.]</vmorph> <ety>[Cf. <er>Prov</er>. E. <ets>caddle</ets> to coax, spoil, fondle, and <er>Cade</er>, <pos>a.</pos> & <pos>v. t.</pos>]</ety> <altsp>[Written also <asp>codle</asp>.]</altsp> <sn>1.</sn> <def>To parboil, or soften by boiling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [the guava fruit] may be <qex>coddled</qex>.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat with excessive tenderness; to pamper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How many of our English princes have been <qex>coddled</qex> at home by their fond papas and mammas!</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Lord Byron] never <qex>coddled</qex> his reputation.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 274 --></p>

<p><ent>Coddymoddy</ent><br/
<hw>Cod"dy*mod"dy</hw> <pr>(k<ocr/d"d<ycr/*m<ocr/d"d<ycr/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A gull in the plumage of its first year.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Code</ent><br/
<hw>Code</hw> <pr>(k<omac/d)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>codex</ets>, <ets>caudex</ets>, the stock or stem of a tree, a board or tablet of wood smeared over with wax, on which the ancients originally wrote; hence, a book, a writing.]</ety> <sn>1.</sn> <def>A body of law, sanctioned by legislation, in which the rules of law to be specifically applied by the courts are set forth in systematic form; a compilation of laws by public authority; a digest.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The collection of laws made by the order of Justinian is sometimes called, by way of eminence, <ldquo/<xex>The Code</xex><rdquo/ . <rj><au>Wharton.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any system of rules or regulations relating to one subject; <as>as, the medical <ex>code</ex>, a system of rules for the regulation of the professional conduct of physicians</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any set of symbols or combinations of symbols used for communication in any medium, such as by telegraph or semaphore.  See <er>Morse code</er>, and <cref>error-correcting code</cref>.</def><br/
[<source>PJC</source>]</p>

<p><note> A system of rules for making communications at sea by means of signals has been referred to as the <col>naval code</col>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any set of standards established by the governing authority of a geopolitical entity restricting the ways that certain activities may be performed, especially the manner in which buildings or specific systems within buildings may be constructed; <as>as, a building <ex>code</ex>; a plumbing <ex>code</ex>; a health <ex>code</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>Any system used for secrecy in communication, in which the content of a communication is converted, prior to transmission, into symbols whose meaning is known only to authorized recipients of the message; such codes are used to prevent unauthorized persons from learning the content of the communication.  The process of converting a communication into secret symbols by means of a code is called <er>encoding</er> or <er>encryption</er>.  However, unauthorized persons may learn the code by various means, as in <er>code-breaking</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <def>An <cref>error-correcting code</cref>.  See below.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <fld>(Computers)</fld>  <def>The set of instructions for a computer program written by a programmer, usually in a programming language such as Fortran, C, Cobol, Java, C++, etc.; also, the executable binary <cref>object code</cref>.  All such programs except for the binary <cref>object code</cref> must be converted by a compiler program into <cref>object code</cref>, which is the arrangement of data bits which can be directly interpreted by a computer.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Code civil</b></col> <it>or</it> <col><b>Code Napoleon</b></col></mcol>, <cd>a code enacted in France in 1803 and 1804, embodying the law of rights of persons and of property generally.</cd>  <rj><au>Abbot.</au></rj>  <col><b>error-correcting code</b></col> <fld>(Computers)</fld> <cd>A set of symbols used to represent blocks of binary data, in which the original block of data is represented by a larger block of data which includes additional bits arranged in such a way that the original data may be read even if one or more of the bits of the encoded data is changed, as in a noisy communicaiton channel.  Various codes are available which can correct different numbers or patterns of errors in the transmitted data.  Such codes are used to achieve higher accuracy in data transmission, and in data storage devices such as disk drives and tape drives.</cd>  <col><b>object code</b></col> <fld>(Computers)</fld> <cd>the arrangement of bits stored in computer memory or a data storage device which, when fed to the instruction processor of a computer's central processing unit, can be interpreted directly as instructions for execution.</cd> <col><b>genetic code</b></col> <fld>(Biochemistry, genetics)</fld> <cd>The set of correspondences between sequences of three bases (codons) in a RNA chain to the amino acid which those three bases represent in the process of protein synthesis.  Thus, the sequence <it>UUU</it> codes for phenylalanine, and <it>AUG</it> codes for methionine.  There are twenty-one naturally-occurring amino acids, and sixty-four possible arrangements of three bases in RNA; thus some of the amino acids are represented by more than one codon.   Several codons do not represent amino acids, but cause termination of the synthesis of a growing amnio acid chain.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note>The genetic code is represented by the following table:
<tt>The Genetic Code<br/
=====================================================<br/
UUU Phenylalanine (Phe)		AUU Isoleucine (Ile)<br/
UCU Serine (Ser)			ACU Threonine (Thr)<br/
UAU Tyrosine (Tyr)			AAU Asparagine (Asn)<br/
UGU Cysteine (Cys)			AGU Serine (Ser)<br/
UUC Phe					AUC Ile<br/
UCC Ser					ACC Thr<br/
UAC Tyr					AAC Asn<br/
UGC Cys					AGC Ser<br/
UUA Leucine (Leu)			AUA Ile<br/
UCA Ser					ACA Thr<br/
UAA STOP					AAA Lysine (Lys)<br/
UGA STOP					AGA Arginine (Arg)<br/
UUG Leu					AUG Methionine (Met) or START<br/
UCG Ser					ACG Thr<br/
UAG STOP					AAG Lys<br/
UGG Tryptophan (Trp)		AGG Arg<br/
CUU Leucine (Leu)			GUU Valine Val<br/
CCU Proline (Pro)			GCU Alanine (Ala)<br/
CAU Histidine (His)			GAU Aspartic acid (Asp)<br/
CGU Arginine (Arg)			GGU Glycine (Gly)<br/
CUC Leu					GUC (Val)<br/
CCU Pro					GCC Ala<br/
CAC His					GAC Asp<br/
CGC Arg					GGC Gly<br/
CUA Leu					GUA Val<br/
CCA Pro					GCA Ala<br/
CAA Glutamine (Gln)			GAA Glutamic acid (Glu)<br/
CGA Arg					GGA Gly<br/
CUG Leu					GUG Val<br/
CCG Pro					GCG Ala<br/
CAG Gln					GAG Glu <br/
CGG Arg					GGG Gly<br/
</tt></note><br/
[<source>PJC</source>]</p>

<p><ent>Code</ent><br/
<hw>Code</hw> <pr>(k<omac/d)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To convert (a text or other information) into a encoded form by means of a code{5}.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To write a computer program in a programming language; <as>as, to <ex>code</ex> a sorting routine</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Code</ent><br/
<hw>Code</hw> <pr>(k<omac/d)</pr>, <pos>v. i.</pos> <fld>(Biochemistry, genetics)</fld>  <def>To serve as the nucleotide sequence directing the synthesis of a particular amino acid or sequence of amino acids in protein biosynthesis; <as>as, this sequence of nucleotides <ex>encodes</ex> the hemoglobin alpha chain.</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>code-breaking</ent><br/
<hw>code"-break`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A process of discovering the content of an encoded message or the nature of a <er>code</er>{5}.  The method used may be surreptitious, or may use sophisticated mathematical and computational techniques to discover the code.</def><br/
[<source>PJC</source>]</p>

<p><ent>Codefendant</ent><br/
<hw>Co`de*fend"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint defendant.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codeine</ent><br/
<hw>Co"deine</hw> <pr>(k<omac/"d<emac/n or k<osl/*d<emac/"<icr/n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kw`deia</grk> poppy head: cf. F. <ets>cod<eacute/ine</ets>.]</ety> <fld>(Chem.)</fld> <def>One of the opium alkaloids; a white crystalline substance, <chform>C18H21NO3</chform>, similar to and regarded as a derivative of morphine, but much feebler in its action; -- called also <altname>codeia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codetta</ent><br/
||<hw>Co*det"ta</hw> <pr>(k<osl/*d<ecr/t"t<adot/)</pr>, <pos>n.</pos> <ety>[It., dim. of <ets>coda</ets> tail.]</ety> <fld>(Mus.)</fld> <def>A short passage connecting two sections, but not forming part of either; a short coda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codex</ent><br/
||<hw>Co"dex</hw> <pr>(k<omac/"d<ecr/ks)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Codices</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Code</er>.]</ety> <sn>1.</sn> <def>A book, especially an early form of book with pages stitched together, contrasting with the earlier scrolls; a manuscript; <as>as, a DaVinci <ex>codex</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A collection or digest of laws; a code(senses 2 or 4).</def>  <mark>[archaic]</mark> <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An ancient manuscript of the Sacred Scriptures, or any part of them, particularly the New Testament.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A collection of canons.</def>  <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codfish</ent><br/
<hw>Cod"fish</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A kind of fish. Same as <er>Cod</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codger</ent><br/
<hw>Codg"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cadger</er>.]</ety> <sn>1.</sn> <def>A miser or mean person.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A singular or odd person, especially an old man; -- a familiar, humorous, or depreciatory appellation.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A few of us old <qex>codgers</qex> met at the fireside.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codical</ent><br/
<hw>Cod"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to a codex, or a code.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codicil</ent><br/
<hw>Cod"i*cil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>codicillus</ets>, dim. of <ets>codex</ets>: cf. F. <ets>codicille</ets>. See <er>Code</er>.]</ety> <fld>(Law)</fld> <def>A clause added to a will.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codicillary</ent><br/
<hw>Cod`i*cil"la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>codicillaris</ets>, <ets>codicillarius</ets>.]</ety> <def>Of the nature of a codicil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codification</ent><br/
<hw>Co`di*fi*ca"tion</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>codification</ets>.]</ety> <def>The act or process of codifying or reducing laws to a code.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codifier</ent><br/
<hw>Co"di*fi`er</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>One who codifies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codify</ent><br/
<hw>Co"di*fy</hw> <pr>(? <or/ ?; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Codified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Codifying</conjf>.]</vmorph> <ety>[<ets>Code</ets> + <ets>-fy</ets>: cf. F. <ets>codifier</ets>.]</ety> <def>To reduce to a code, as laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codilla</ent><br/
<hw>Co*dil"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>codicula</ets> a little tail, dim. of <ets>cauda</ets> tail.]</ety> <fld>(Com.)</fld> <def>The coarse tow of flax and hemp.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codille</ent><br/
<hw>Co*dille"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>codile</ets>.]</ety> <def>A term at omber, signifying that the game is won.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codist</ent><br/
<hw>Co"dist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A codifier; a maker of codes.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codle</ent><br/
<hw>Co"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Coddle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codling</ent><br/
<ent>Codlin</ent><br/
<mhw>{ <hw>Cod"lin</hw> <pr>(?)</pr>, <hw>Cod"ling</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. AS. <ets>cod<ae/ppel</ets> a quince.]</ety> <sd>(a)</sd> <def>An apple fit to stew or coddle.</def> <sd>(b)</sd> <def>An immature apple.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>codling</qex> when 't is almost an apple.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Codling moth</b></col> <fld>(Zool.)</fld>, <cd>a small moth (<spn>Carpocapsa Pomonella</spn>), which in the larval state (known as the <xex>apple worm</xex>) lives in apples, often doing great damage to the crop.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codling</ent><br/
<hw>Cod"ling</hw>, <pos>n.</pos> <ety>[Dim. of <ets>cod</ets> the fish.]</ety> <fld>(Zool.)</fld> <def>A young cod; also, a hake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>codlins-and-cream</ent><br/
<hw>codlins-and-cream</hw> <pos>n.</pos> <def>a plant of Europe and Asia (<spn>Epilobium hirsutum</spn>) having purplish-red flowers and hairy stems and leaves; it was introduced into North America.</def><br/
<syn><b>Syn. --</b> hairy willowherb, <spn>Epilobium hirsutum</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cod liver</ent><br/
<hw>Cod" liv`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The liver of the common cod and allied species.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cod-liver oil</b></col>, <cd>an oil obtained from the liver of the codfish, and used extensively in medicine as a means of supplying the body with fat in cases of malnutrition.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Codon</ent><br/
<hw>Co"don</hw> <pr>(k<omac/"d<ocr/n)</pr>, <pos>n.</pos> <fld>(Molecular biology)</fld>  <def>a sequence of three nucleotides in a genome or a DNA or messenger RNA molecule, which specifies the incorporation of one amino acid or is a stop signal, during the biosynthesis of proteins.  Codons occur within the protein-coding segments of the DNA or RNA genome of living organisms.  The amino acid sequence of proteins synthesized on ribosomes is thus determined by the sequence of the nucleotides in the genome.</def><br/
[<source>PJC</source>]</p>

<p><note><hand/  Outside of coding regions, there is no direct correspondence of the nucleotide sequence with protein sequence, and certain signalling functions of nucleic acids are not specified by three-nucleotide codons.  Certain types of polypeptide synthesized in living organisms are not synthesized on ribosomes, and the sequences of these polypeptides do not have a corresponding nucleotide sequence in the genome.</note><br/
[<source>PJC</source>]</p>

<p><ent>Codpiece</ent><br/
<hw>Cod"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cod</ets>, n., <?/ + <ets>piece</ets>.]</ety> <def>A part of male dress in front of the breeches, formerly made very conspicuous.</def>  <rj><au>Shak.</au>  <au>Fosbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coecilian</ent><br/
<hw>C<oe/*cil"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>C<ae/cilian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>co-ed</ent><br/
<hw>co-ed</hw> <pos>n.</pos> <def>a female student at a coeducational college or university.</def><br/
<syn><b>Syn. --</b> college girl.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>co-ed</ent><br/
<hw>co-ed</hw> <pos>adj.</pos> <def>having both female and male students; -- of a school.</def><br/
<syn><b>Syn. --</b> coeducational.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coeducation</ent><br/
<hw>Co*ed`u*ca"tion</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>An educating together, of different sexes or races; -- now used almost exclusively in reference to the education males and females together.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p>-- <wordforms><wf>Co*ed`u*ca"tion*al</wf> <pr>(<?/)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coefficacy</ent><br/
<hw>Co*ef"fi*ca*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint efficacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coefficiency</ent><br/
<hw>Co`ef*fi"cien*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint efficiency; cooperation.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coefficient</ent><br/
<hw>Co`ef*fi"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cooperating; acting together to produce an effect.</def></p>

<p><wordforms><wf>Co`ef*fi"cient*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coefficient</ent><br/
<hw>Co`ef*fi"cient</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which unites in action with something else to produce the same effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. F. <ets>coefficient</ets>.]</ety> <fld>(Math.)</fld> <def>A number or letter put before a letter or quantity, known or unknown, to show how many times the latter is to be taken; <as>as, 6<it>x</it>; <it>bx</it>; here 6 and <it>b</it> are <ex>coefficients</ex> of <it>x</it></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physics)</fld> <def>A number, commonly used in computation as a factor, expressing the amount of some change or effect under certain fixed conditions as to temperature, length, volume, etc.; <as>as, the <ex>coefficient</ex> of expansion; the <ex>coefficient</ex> of friction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Arbitrary coefficient</b></col> <fld>(Math.)</fld>, <cd>a literal coefficient placed arbitrarily in an algebraic expression, the value of the coefficient being afterwards determined by the conditions of the problem.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coehorn</ent><br/
<hw>Coe"horn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From its inventor, Baron <etsep>Coehorn</etsep>.]</ety> <fld>(Mil.)</fld> <def>A small bronze mortar mounted on a wooden block with handles, and light enough to be carried short distances by two men.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coelacanth</ent><br/
<hw>C<oe/l"a*canth</hw> <pr>(? or <?/)</pr>, <pos>a.</pos> <ety>[Gr. <grk>koi^los</grk> hollow + <?/ spine.]</ety> <fld>(Zool.)</fld> <def>Having hollow spines, as some ganoid fishes.</def></p>

<p><ent>Coelectron</ent><br/
<hw>Co`e*lec"tron</hw>, <pos>n.</pos> <def>See <er>Electron</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coelenterata</ent><br/
<ent>Coelentera</ent><br/
<mhw>{ ||<hw>C<oe/*len"te*ra</hw> <pr>(?)</pr> <or/  ||<hw>C<oe/*len`te*ra"ta</hw>, }</mhw> <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>koi^los</grk> hollow + <?/ intestines.]</ety> <fld>(Zool.)</fld> <def>A comprehensive group of Invertebrata, mostly marine, comprising the <class>Anthozoa</class>, <class>Hydrozoa</class>, and <class>Ctenophora</class>. The name implies that the stomach and body cavities are one. The group is sometimes enlarged so as to include the sponges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coelenterate</ent><br/
<hw>C<oe/*len"ter*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Belonging to the C<oe/lentera.</def> -- <def2><pos>n.</pos> <def>One of the C<oe/lentera.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coelia</ent><br/
||<hw>C<oe/"li*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>koili`a</grk> a cavity of the body, a ventricle.]</ety> <fld>(Anat.)</fld> <def>A cavity.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The word is applied to the ventricles of the brain, the different ventricles being indicated by prefixes like those characterizing the parts of the brain in which the cavities are found; as, <xex>epic<oe/lia</xex>, <xex>mesoc<oe/lia</xex>, <xex>metac<oe/lia</xex>, <xex>proc<oe/lia</xex>, etc.</note>  <rj><au>B. G. Wilder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Celiac</ent><br/
<ent>Coeliac</ent><br/
<mhw>{ <hw>C<oe/"li*ac</hw>, <hw>Ce"li*ac</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>coeliacus</ets>, Gr. <?/, fr. <?/ belly, fr. <grk>koi^los</grk> hollow.]</ety> <def>Relating to the abdomen, or to the cavity of the abdomen.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>C<oe/liac artery</b></col> <fld>(Anat.)</fld>, <cd>the artery which issues from the aorta just below the diaphragm; -- called also <altname>c<oe/liac axis</altname>.</cd> -- <mcol><col><b>C<oe/liac flux</b></col>, <col><b>C<oe/liac passion</b></col></mcol> <fld>(Med.)</fld>, <cd>a chronic flux or diarrhea of undigested food.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coelodont</ent><br/
<hw>C<oe/"lo*dont</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>koi^los</grk> hollow + <?/, <?/, tooth.]</ety> <fld>(Zool.)</fld> <def>Having hollow teeth; -- said of a group lizards.</def> -- <def2><pos>n.</pos> <def>One of a group of lizards having hollow teeth.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>coelogyne</ent><br/
<hw>coelogyne</hw> <pos>n.</pos> <def>any of various orchids of the genus <gen>Coelogyne</gen> which may have clusters of fragrant lacy snow-white flowers; sinister salmon-pink solitary flowers; chainlike racemes of topaz and chocolate brown flowers; spikes of delicate white spice-scented flowers; or emerald green flowers marked with blue-black.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coelospermous</ent><br/
<hw>C<oe/l`o*sper"mous</hw> <pr>(? <or/ <?/)</pr>, <pos>a.</pos> <ety>[Gr. <grk>koi^los</grk> hollow + <?/ seed.]</ety> <fld>(Bot.)</fld> <def>Hollow-seeded; having the ventral face of the seedlike carpels incurved at the ends, as in coriander seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coelostat</ent><br/
<hw>coe"lo*stat</hw> <pos>n.</pos> <def>an optical instrument used to follow the path of a celestial body and reflect its light into a telescope; it has a movable and a fixed mirror.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coelum</ent><br/
||<hw>C<oe/"lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a hollow, neut. of <grk>koi^los</grk> hollow.]</ety> <fld>(Anat.)</fld> <def>See <cref>Body cavity</cref>, under <er>Body</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coemption</ent><br/
<hw>Co*emp"tion</hw> <pr>(?; 215)</pr>, <pos>n.</pos> <ety>[L. <ets>co<eum/mptio</ets>, fr. <ets>co<eum/mere</ets> to buy up. See <er>Emption</er>.]</ety> <def>The act of buying the whole quantity of any commodity.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coendoo</ent><br/
||<hw>Co*en"doo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>The Brazilian porcupine (<spn>Cercolades prehensiles</spn> syn. <spn>Sphingurus prehensiles</spn>), remarkable for its prehensile tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coenenchyma</ent><br/
<ent>Coenenchym</ent><br/
<mhw>{ <hw>C<oe/*nen"chym</hw> <pr>(?)</pr>, ||<hw>C<oe/*nen"chy*ma</hw> <pr>(?)</pr> }</mhw> <pos>n.</pos> <ety>[NL. <ets>coenenchyma</ets>, fr. Gr. <grk>koino`s</grk> common + <?/ something poured in. Formed like <ets>parenchyma</ets>.]</ety> <fld>(Zool.)</fld> <def>The common tissue which unites the polyps or zooids of a compound anthozoan or coral. It may be soft or more or less ossified. See <er>Coral</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coenesthesis</ent><br/
||<hw>C<oe/n`es*the"sis</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>koino`s</grk> common + <?/ sensation.]</ety> <fld>(Physiol.)</fld> <def>Common sensation or general sensibility, as distinguished from the special sensations which are located in, or ascribed to, separate organs, as the eye and ear. It is supposed to depend on the ganglionic system.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coenobite</ent><br/
<ent>coenobite</ent><br/
<mhw><hw>coenobite</hw>, <hw>C<oe/n"o*bite</hw></mhw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>a member of a religious order living in common.  Same as <er>Cenobite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coenobitical</ent><br/
<ent>coenobitic</ent><br/
<mhw><hw>coenobitic</hw> <hw>coenobitical</hw></mhw> <pos>adj.</pos>  <def>of or relating to or befitting cenobites or their practices of communal living.  Opposite of <ant>eremitic</ant>.</def><br/
<syn><b>Syn. --</b> cenobitic, cenobitical.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coenoecium</ent><br/
||<hw>C<oe/*n<oe/"ci*um</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>koino`s</grk> common + <grk>o'i^kos</grk> house.]</ety> <fld>(Zool.)</fld> <def>The common tissue which unites the various zooids of a bryozoan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coenogamy</ent><br/
<hw>C<oe/*nog"a*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>koino`s</grk> common + <?/ marraige.]</ety> <def>The state of a community which permits promiscuous sexual intercourse among its members; -- as in certain primitive tribes or communistic societies.</def> <altsp>[Written also <asp>cenogamy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coenosarc</ent><br/
<hw>C<oe/n"o*sarc</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>koino`s</grk> common + <?/, <?/, flesh.]</ety> <fld>(Zool.)</fld> <def>The common soft tissue which unites the polyps of a compound hydroid. See <er>Hydroidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coenurus</ent><br/
||<hw>C<oe/*nu"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. fr. Gr. <grk>koino`s</grk> common + <grk>o'yra`</grk> tail.]</ety> <fld>(Zool.)</fld> <def>The larval stage of a tapeworm (<spn>T<ae/nia c<oe/nurus</spn>) which forms bladderlike sacs in the brain of sheep, causing the fatal disease known as <causes>water brain</causes>, <causes>vertigo</causes>, <causes>staggers</causes> or <causes>gid</causes>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This bladder worm has on its surface numerous small heads, each of which, when swallowed by a dog, becomes a mature tapeworm in the dog's intestine.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>coenzyme</ent><br/
<hw>co*en"zyme</hw> <pr>(k<osl/*<ecr/n"z<imac/m)</pr>, <fld>(Biochem.)</fld> <def>a molecule that is essential for the activity of some enzymes; it may exist free in solution within a living organism, but functions by binding to an enzyme to assist in catalyzing a reaction.  The molecule itself may be temporarily changed during the reaction, but is ultimately restored to its original form.  Many vitamins function as <ex>coenzymes</ex>.</def><br/
[<source>PJC</source>]</p>

<p><ent>coenzyme A</ent><br/
<hw>co*en"zyme A`</hw> <pr>(k<osl/*<ecr/n"z<imac/m <amac/`)</pr>, <fld>(Biochem.)</fld> <def>a <isa>coenzyme</isa> (<chform>C21H36N7O16P3S</chform>) that participates in the transfer of acetyl groups in biochemical reactions; -- abbreviated <altname>CoA</altname>.  It contains adenosine, phosphate, pantothenic acid and cysteamine groups.  The acetyl group to be transferred during biosynthesis is temporarily attached to the free sulhydryl of the cysteamine group to form a thioester, in which state it is called <ecol><b>acetyl coenzyme A</b></ecol>.  The strength of a preparation of coenzyme A may be expressed by the <ecol><b>Lippman unit</b></ecol>; one milligram of CoA contains 413 Lippman units.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coequal</ent><br/
<hw>Co*e"qual</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coaequalis</ets>; <ets>co-</ets> + <ets>aequalis</ets> equal.]</ety> <def>Being on an equality in rank or power.</def> -- <def2><pos>n.</pos> <def>One who is on an equality with another.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><q>In once he come to be a cardinal,<br/
He'll make his cap <qex>coequal</qex> with the crown.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coequality</ent><br/
<hw>Co`e*qual"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being on an equality, as in rank or power.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coequally</ent><br/
<hw>Co*e"qual*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With coequality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coerce</ent><br/
<hw>Co*erce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coerced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coercing</conjf>.]</vmorph> <ety>[L. <ets>co<eum/rcere</ets>; <ets>co-</ets> + <ets>arcere</ets> to shut up, to press together. See <er>Ark</er>.]</ety> <sn>1.</sn> <def>To restrain by force, especially by law or authority; to repress; to curb.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Punishments are manifold, that they may <qex>coerce</qex> this profligate sort.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To compel or constrain to any action; <as>as, to <ex>coerce</ex> a man to vote for a certain candidate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To compel or enforce; <as>as, to <ex>coerce</ex> obedience</as>.</def></p>

<p><syn><b>Syn.</b> -- To <er>Coerce</er>, <er>Compel</er>.</syn> <usage> To <xex>compel</xex> denotes to urge on by force which cannot be resisted. The term aplies equally to physical and moral force; as, <xex>compelled</xex> by hunger; <xex>compelled</xex> adverse circumstances; <xex>compelled</xex> by parental affection. <xex>Coerce</xex> had at first only the negative sense of checking or restraining by force; as, to <xex>coerce</xex> a bad man by punishments or a prisoner with fetters. It has now gained a positive sense., viz., that of driving a person into the performance of some act which is required of him by another; as, to <xex>coerce</xex> a man to sign a contract; to <xex>coerce</xex> obedience. In this sense (which is now the prevailing one), <xex>coerce</xex> differs but little from <xex>compel</xex>, and yet there is a distinction between them. <xex>Coercion</xex> is usually acomplished by indirect means, as threats and intimidation, physical force being more rarely employed in <xex>coercing</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coercible</ent><br/
<hw>Co"er"ci*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being coerced.</def></p>

<p>-- <wordforms><wf>Co*er"ci*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coercion</ent><br/
<hw>Co*er"cion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coercio</ets>, fr. <ets>coercere</ets>. See <er>Coerce</er>.]</ety> <sn>1.</sn> <def>The act or process of coercing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The application to another of either physical or moral force. When the force is physical, and cannot be resisted, then the act produced by it is a nullity, so far as concerns the party coerced. When the force is moral, then the act, though voidable, is imputable to the party doing it, unless he be so paralyzed by terror as to act convulsively. At the same time coercion is not negatived by the fact of submission under force. <ldquo/Coactus volui<rdquo/ (I consented under compulsion) is the condition of mind which, when there is volition forced by coercion, annuls the result of such coercion.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coercitive</ent><br/
<hw>Co*er"ci*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Coercive.</def> <ldquo/<xex>Coercitive</xex> power in laws.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coercive</ent><br/
<hw>Co*er"cive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving or intended to coerce; having power to constrain.</def></p>

<p>-- <wordforms><wf>Co*er"cive*ly</wf>, <pos>adv.</pos> -- Co*er"cive*ness, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Coercive</qex> power can only influence us to outward practice.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Coercive force</b></col> <it>or</it> <col><b>Coercitive force</b></col></mcol> <fld>(Magnetism)</fld>, <cd>the power or force which in iron or steel produces a slowness or difficulty in imparting magnetism to it, and also interposes an obstacle to the return of a bar to its natural state when active magnetism has ceased. It plainly depends on the molecular constitution of the metal.</cd>  <rj><au>Nichol.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The power of resisting magnetization or demagnization is sometimes called <qex>coercive force</qex>.</q> <rj><qau>S. Thompson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coerulignone</ent><br/
<hw>C<oe/`ru*lig"none</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coeruleus cerulean + lignum</ets> wood + E. quin<ets>one</ets>.]</ety> <fld>(Chem.)</fld> <def>A bluish violet, crystalline substance obtained in the purification of crude wood vinegar. It is regarded as a complex quinone derivative of diphenyl; -- called also <altname>cedriret</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coessential</ent><br/
<hw>Co`es*sen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partaking of the same essence.</def> -- <wordforms><wf>Co`es*sen"tial*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>We bless and magnify that <qex>coessential</qex> Spirit, eternally proceeding from both [The Father and the Son].</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coessentiality</ent><br/
<hw>Co`es*sen`ti*al"i*ty</hw> <pr>(? <or/ ?; 106)</pr>, <pos>n.</pos> <def>Participation of the same essence.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coestablishment</ent><br/
<hw>Co`es*tab"lish*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint establishment.</def>  <rj><au>Bp. Watson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coestate</ent><br/
<hw>Co`es*tate"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint estate.</def>  <rj><au>Smolett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coetanean</ent><br/
<hw>Co`e*ta"ne*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person coetaneous with another; a contemporary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A . . . <qex>coetanean</qex> of the late earl of Southampton.</q> <rj><qau>Aubrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coetaneous</ent><br/
<hw>Co`e*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coaetaneus</ets>; <ets>co-</ets> + <ets>aetas</ets> age.]</ety> <def>Of the same age; beginning to exist at the same time; contemporaneous.</def></p>

<p>-- <wordforms><wf>Co`e*ta"ne*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>And all [members of the body] are <qex>coetaneous</qex>.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coeternal</ent><br/
<hw>Co`e*ter"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Equally eternal.</def> -- <wordforms><wf>Co`e*ter"nal*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Hail, holy Light, offspring of Heaven first born!<br/
Or of the Eternal <qex>coeternal</qex> beam.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coeternity</ent><br/
<hw>Co`e*ter"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Existence from eternity equally with another eternal being; equal eternity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coeval</ent><br/
<hw>Co*e"val</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coaevus</ets>; <ets>co-</ets> + <ets>aevum</ets> lifetime, age. See <er>Age</er>, <pos>n.</pos>]</ety> <def>Of the same age; existing during the same period of time, especially time long and remote; -- usually followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Silence! <qex>coeval</qex> with eternity!</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Oaks <qex>coeval</qex> spread a mournful shade.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coeval</ent><br/
<hw>Co*e"val</hw>, <pos>n.</pos> <def>One of the same age; a contemporary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As if it were not enough to have outdone all your <qex>coevals</qex> in wit.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coevous</ent><br/
<hw>Co*e"vous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Coeval</def> <mark>[Obs.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coexecutor</ent><br/
<hw>Co`ex*ec"u*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint executor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coexecutrix</ent><br/
<hw>Co`ex*ec"u*trix</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint executrix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coexist</ent><br/
<hw>Co`ex*ist</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coexisted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coexisting</conjf>.]</vmorph> <def>To exist at the same time; -- sometimes followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of substances no one has any clear idea, farther than of certain simple ideas <qex>coexisting</qex> together.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So much purity and integrity . . . <qex>coexisting</qex> with so much decay and so many infirmities.</q> <rj><qau>Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coexistence</ent><br/
<hw>Co`ex*ist"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Existence at the same time with another; -- contemporary existence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without the help, or so much as the <qex>coexistence</qex>, of any condition.</q> <rj><qau>Jer.  Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coexistent</ent><br/
<hw>Co`ex*ist"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Existing at the same time with another.</def> -- <def2><pos>n.</pos> <def>That which coexists with another.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><q>The law of <qex>coexistent</qex> vibrations.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coexisting</ent><br/
<hw>Co`ex*ist"ing</hw>, <pos>a.</pos> <def>Coexistent.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coextend</ent><br/
<hw>Co`ex*tend</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coextended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coextending</conjf>.]</vmorph> <def>To extend through the same space or time with another; to extend to the same degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>According to which the least body may be <qex>coextended</qex> with the greatest.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Has your English language one single word that is <qex>coextended</qex> through all these significations?</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 275 --></p>

<p><ent>Coextension</ent><br/
<hw>Co`ex*ten"sion</hw> <pr>(k<omac/`<ecr/ks*t<ecr/n"sh<ucr/n)</pr>, <pos>n.</pos> <def>The act of extending equally, or the state of being equally extended.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coextensive</ent><br/
<hw>Co`ex*ten"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Equally extensive; having equal extent; <as>as, consciousness and knowledge are <ex>coextensive</ex></as>.</def> <au>Sir W. Hamilton.</au> -- <wordforms><wf>Co`ex*ten"sive*ly</wf>, <pos>adv.</pos> -- Co`ex*ten"sive*ness, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffea</ent><br/
<hw>Coffea</hw> <pos>n.</pos> <def>the genus comprising the coffee trees.</def><br/
<syn><b>Syn. --</b> genus <gen>Coffea</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coffee</ent><br/
<hw>Cof"fee</hw> <pr>(k<add/"f<esl/; k<ocr/f"f<esl/; 115)</pr>, <pos>n.</pos> <ety>[Turk. <ets>qahveh</ets>, Ar. <ets>qahuah</ets> wine, coffee, a decoction of berries. Cf. <er>Caf<eacute/</er>.]</ety> <sn>1.</sn> <def>The <ldquo/beans<rdquo/ or <ldquo/berries<rdquo/ (pyrenes) obtained from the drupes of a small evergreen tree of the genus <gen>Coffea</gen>, growing in Abyssinia, Arabia, Persia, and other warm regions of Asia and Africa, and also in tropical America.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>The coffee tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are several species of the coffee tree, as, <spn>Coffea Arabica</spn>, <spn>Coffea canephora</spn>, <spn>Coffea occidentalis</spn>, and <spn>Coffea Liberica</spn>.  The white, fragrant flowers grow in clusters at the root of the leaves, and the fruit is a red or purple cherrylike drupe, with sweet pulp, usually containing two pyrenes, commercially called <ldquo/beans<rdquo/ or <ldquo/berries<rdquo/.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The beverage made by decoction of the roasted and ground berry of the coffee tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They have in Turkey a drink called <qex>coffee</qex>.  . . . This drink comforteth the brain and heart, and helpeth digestion.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>a cup of coffee{3}, especially one served in a restaurant; <as>as, we each had two donuts and a <ex>coffee</ex>; three <ex>coffees</ex> to go</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a social gathering at which coffee is served, with optional other foods or refreshments.</def><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <def>a color ranging from medium brown to dark brown.</def><br/
[<source>PJC</source>]</p>

<p><note><hand/ The use of <xex>coffee</xex> is said to have been introduced into England about 1650, when coffeehouses were opened in Oxford and London.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coffee bug</b></col> <fld>(Zool.)</fld>, <cd>a species of scale insect (<spn>Lecanium coff<ae/a</spn>), often very injurious to the coffee tree.</cd> -- <col><b>Coffee rat</b></col> <fld>(Zool.)</fld> <cd>See <er>Musang</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>coffeeberry</ent><br/
<hw>coffeeberry</hw> <pos>n.</pos> <def>an evergreen shrub of Western U.S. (<spn>Rhamnus californicus</spn>), bearing small red or black fruits; -- called also the <altname>California buckthorn</altname>.</def><br/
<syn><b>Syn. --</b> California buckthorn, California coffee, <spn>Rhamnus californicus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coffeecake</ent><br/
<hw>cof"fee*cake`</hw> <pr>(k<add/"f<esl/*k<amac/`)</pr>, <pos>n.</pos> <def>a cake or sweet bread usually glazed after baking, and having added nuts and fruits; it is often served with coffee.</def><br/
<syn><b>Syn. --</b> coffee cake.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Coffeehouse</ent><br/
<hw>Cof"fee*house`</hw> <pr>(k<add/"f<esl/*hous`)</pr>, <pos>n.</pos> <def>A house of entertainment, where guests are supplied with coffee and other refreshments, and where men meet for conversation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>coffeehouse</qex> must not be dismissed with a cursory mention. It might indeed, at that time, have been not improperly called a most important political institution.  . . . The <qex>coffeehouses</qex> were the chief organs through which the public opinion of the metropolis vented itself.  . . . Every man of the upper or middle class went daily to his <qex>coffeehouse</qex> to learn the news and discuss it. Every <qex>coffeehouse</qex> had one or more orators, to whose eloquence the crowd listened with admiration, and who soon became what the journalists of our own time have been called -- a fourth estate of the realm.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffeeman</ent><br/
<hw>Cof"fee*man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who keeps a coffeehouse.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coffeepot</ent><br/
<ent>coffee-pot</ent><br/
<mhw><hw>cof"fee-pot</hw>, <hw>cof"fee*pot</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A covered pot in which coffee is prepared, or is brought upon the table for drinking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a tall pot in which coffee is brewed, especially one in which the heating of the water is accomplished by electricity.</def><br/
<syn><b>Syn. --</b> coffeepot, coffee pot.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coffeeroom</ent><br/
<hw>Cof"fee*room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A public room where coffee and other refreshments may be obtained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffer</ent><br/
<hw>Cof"fer</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <ety>[OF. <ets>cofre</ets>, F. <ets>coffre</ets>, L. <ets>cophinus</ets> basket, fr. Gr. <?/. Cf. <er>Coffin</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A casket, chest, or trunk; especially, one used for keeping money or other valuables.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In ivory <qex>coffers</qex> I have stuffed my crowns.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Treasure or funds; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He would discharge it without any burden to the queen's <qex>coffers</qex>, for honor sake.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hold, here is half my <qex>coffer</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A panel deeply recessed in the ceiling of a vault, dome, or portico; a caisson.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Fort.)</fld> <def>A trench dug in the bottom of a dry moat, and extending across it, to enable the besieged to defend it by a raking fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The chamber of a canal lock; also, a caisson or a cofferdam.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coffer dam</b></col>. <fld>(Engin.)</fld> <cd>See <er>Cofferdam</er>, in the Vocabulary.</cd> -- <col><b>Coffer fish</b></col>. <fld>(Zool.)</fld> <cd>See <er>Cowfish</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffer</ent><br/
<hw>Cof"fer</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To put into a coffer.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining.)</fld> <def>To secure from leaking, as a shaft, by ramming clay behind the masonry or timbering.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To form with or in a coffer or coffers; to furnish with a coffer or coffers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cofferdam</ent><br/
<hw>Cof"fer*dam</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A water-tight inclosure, as of piles packed with clay, from which the water is pumped to expose the bottom (of a river, etc.) and permit the laying of foundations, building of piers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cofferer</ent><br/
<hw>Cof"fer*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who keeps treasures in a coffer.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cofferwork</ent><br/
<hw>Cof"fer*work`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Masonry)</fld> <def>Rubblework faced with stone.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffin</ent><br/
<hw>Cof"fin</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <ety>[OE., a basket, receptacle, OF. <ets>cofin</ets>, fr. L. <ets>cophinus</ets>. See <er>Coffer</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The case in which a dead human body is inclosed for burial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They embalmed him [Joseph], and he was put in a <qex>coffin</qex>.</q> <rj><qau>Gen. 1. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A basket.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (matt. xiv. 20).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A casing or crust, or a mold, of pastry, as for a pie.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of the paste a <qex>coffin</qex> I will rear.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A conical paper bag, used by grocers.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Far.)</fld> <def>The hollow crust or hoof of a horse's foot, below the coronet, in which is the coffin bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coffin bone</b></col>, <cd>the foot bone of the horse and allied animals, inclosed within the hoof, and corresponding to the third phalanx of the middle finger, or toe, of most mammals.</cd> -- <col><b>Coffin joint</b></col>, <cd>the joint next above the coffin bone.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffin</ent><br/
<hw>Cof"fin</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coffined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coffining</conjf>.]</vmorph> <def>To inclose in, or as in, a coffin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Would'st thou have laughed, had I come <qex>coffined</qex> home?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Devotion is not <qex>coffined</qex> in a cell.</q> <rj><qau>John Hall (1646).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffinless</ent><br/
<hw>Cof"fin*less</hw>, <pos>a.</pos> <def>Having no coffin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coffle</ent><br/
<hw>Cof"fle</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <ety>[Ar. <ets>kafala</ets> caravan.]</ety> <def>A gang of negro slaves being driven to market.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cog</ent><br/
<hw>Cog</hw> <pr>(k<ocr/g)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cogged</conjf> <pr>(k<ocr/gd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cogging</conjf>.]</vmorph> <ety>[Cf. W. <ets>coegio</ets> to make void, to beceive, from <ets>coeg</ets> empty, vain, foolish. Cf. <er>Coax</er>, <pos>v. t.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To seduce, or draw away, by adulation, artifice, or falsehood; to wheedle; to cozen; to cheat.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll . . . <qex>cog</qex> their hearts from them.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To obtrude or thrust in, by falsehood or deception; <as>as, to <ex>cog</ex> in a word</as>; to palm off.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Fustian tragedies . . . have, by concerted applauses, been <qex>cogged</qex> upon the town for masterpieces.</q> <rj><qau>J. Dennis</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To cog a die, to load so as to direct its fall; to cheat in playing dice.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cog</ent><br/
<hw>Cog</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To deceive; to cheat; to play false; to lie; to wheedle; to cajole.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For guineas in other men's breeches,<br/
Your gamesters will palm and will <qex>cog</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cog</ent><br/
<hw>Cog</hw>, <pos>n.</pos> <def>A trick or deception; a falsehood.</def>  <rj><au>Wm. Watson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cog</ent><br/
<hw>Cog</hw>, <pos>n.</pos> <ety>[Cf. Sw. <ets>kugge</ets> a cog, or W. <ets>cocos</ets> the cogs of a wheel.]</ety> <sn>1.</sn> <fld>(Mech.)</fld> <def>A tooth, cam, or catch for imparting or receiving motion, as on a gear wheel, or a lifter or wiper on a shaft; originally, a separate piece of wood set in a mortise in the face of a wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Carp.)</fld> <sd>(a)</sd> <def>A kind of tenon on the end of a joist, received into a notch in a bearing timber, and resting flush with its upper surface.</def> <sd>(b)</sd> <def>A tenon in a scarf joint; a coak.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining.)</fld> <def>One of the rough pillars of stone or coal left to support the roof of a mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cog</ent><br/
<hw>Cog</hw>, <pos>v. t.</pos> <def>To furnish with a cog or cogs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cogged breath sound</b></col> <fld>(Auscultation)</fld>, <cd>a form of interrupted respiration, in which the interruptions are very even, three or four to each inspiration.</cd>  <rj><au>Quain.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cog</ent><br/
<hw>Cog</hw>, <pos>n.</pos> <ety>[OE. <ets>cogge</ets>; cf. D. <ets>kog</ets>, Icel. <ets>kuggr</ets> Cf. <er>Cock</er> a boat.]</ety> <def>A small fishing boat.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogency</ent><br/
<hw>Co"gen*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cogent</er>.]</ety> <def>The quality of being cogent; power of compelling conviction; conclusiveness; force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An antecedent argument of extreme <qex>cogency</qex>.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogenial</ent><br/
<hw>Co*ge"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Congenial.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogent</ent><br/
<hw>Co"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cogens</ets>, p. pr. of <ets>cogere</ets> to drive together, to force; <ets>co-</ets> + agere to drive. See <er>Agent</er>, <pos>a.</pos>, and cf. <er>Coact</er> to force, <er>Coagulate</er>, <pos>p. a.</pos>]</ety> <sn>1.</sn> <def>Compelling, in a physical sense; powerful.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cogent</qex> force of nature.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the power to compel conviction or move the will; constraining; conclusive; forcible; powerful; not easily reasisted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No better nor more <qex>cogent</qex> reason.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Proofs of the most <qex>cogent</qex> description.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The tongue whose strains were <qex>cogent</qex> as commands,<br/
Revered at home, and felt in foreign lands.</q> <rj><qau>Cowper.</qau></rj></p>

<p><syn><b>Syn.</b> -- Forcible; powerful; potent; urgent; strong; persuasive; convincing; conclusive; influential.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogently</ent><br/
<hw>Co"gent*ly</hw>, <pos>adv.</pos> <def>In a cogent manner; forcibly; convincingly; conclusively.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogger</ent><br/
<hw>Cog"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cog</er> to wheedle.]</ety> <def>A flatterer or deceiver; a sharper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coggery</ent><br/
<hw>Cog"ger*y</hw>, <pos>n.</pos> <def>Trick; deception.</def>  <rj><au>Bp. Watson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coggle</ent><br/
<hw>Cog"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cog</er> small boat.]</ety> <def>A small fishing boat.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coggle</ent><br/
<hw>Cog"gle</hw>, <pos>n.</pos> <ety>[Cf. <er>Cobble</er> a cobblestone.]</ety> <def>A cobblestone.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitability</ent><br/
<hw>Cog`i*ta*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being cogitable; conceivableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitable</ent><br/
<hw>Cog"i*ta*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cogitabilis</ets>, fr. <ets>cogitare</ets> to think.]</ety> <def>Capable of being brought before the mind as a thought or idea; conceivable; thinkable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Creation is <qex>cogitable</qex> by us only as a putting forth of divine power.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitabund</ent><br/
<hw>Cog"i*ta*bund`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cogitabundus</ets>.]</ety> <def>Full of thought; thoughtful.</def> <mark>[R.]</mark>  <rj><au>Leigh Hunt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitate</ent><br/
<hw>Cog"i*tate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cogitated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cogitating</conjf>.]</vmorph> <ety>[L. <ets>cogitatus</ets>, p. p. of <ets>cogitare</ets> to reflect upon, prob. fr. <ets>co-</ets> + the root of <ets>aio</ets> I say; hence, prop., to discuss with one's self. Cf. <er>Adage</er>.]</ety> <def>To engage in continuous thought; to think.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that calleth a thing into his mind, whether by impression or recordation, <qex>cogitateth</qex> and considereth, and he that employeth the faculty of his fancy also <qex>cogitateth</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitate</ent><br/
<hw>Cog"i*tate</hw>, <pos>v. t.</pos> <def>To think over; to plan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . is our witness, how we both day and night, revolving in our minds, did <qex>cogitate</qex> nothing more than how to satisfy the parts of a good pastor.</q> <rj><qau>Foxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitation</ent><br/
<hw>Cog`i*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cogitatio</ets>: cf. F. <ets>cogitation</ets>.]</ety> <def>The act of thinking; thought; meditation; contemplation.</def> <ldquo/Fixed in <xex>cogitation</xex> deep.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogitative</ent><br/
<hw>Cog"i*ta*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. LL. <ets>cogitativus</ets>.]</ety> <sn>1.</sn> <def>Possessing, or pertaining to, the power of thinking or meditating.</def> <ldquo/<xex>Cogitative</xex> faculties.<rdquo/  <rj><au>Wollaston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Given to thought or contemplation.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogman</ent><br/
<hw>Cog"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dealer in cogware or coarse cloth.</def> <mark>[Obs.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognac</ent><br/
<hw>Co"gnac`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A kind of French brandy, so called from the town of <xex>Cognac</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognate</ent><br/
<hw>Cog"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cognatus</ets>; <ets>co-</ets> + <ets>gnatus</ets>, <ets>natus</ets>, p. p. of <ets>nasci</ets>, anciently <ets>gnasci</ets>, to be born. See <er>Nation</er>, and cf. <er>Connate</er>.]</ety> <sn>1.</sn> <def>Allied by blood; kindred by birth; specifically <fld>(Law)</fld>, related on the mother's side.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of the same or a similar nature; of the same family; proceeding from the same stock or root; allied; kindred; <as>as, a <ex>cognate</ex> language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognate</ent><br/
<hw>Cog"nate</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Law)</fld> <def>One who is related to another on the female side.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of a number of things allied in origin or nature; <as>as, certain letters are <ex>cognates</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognateness</ent><br/
<hw>Cog"nate*ness</hw>, <pos>n.</pos> <def>The state of being cognate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognati</ent><br/
||<hw>Cog*na"ti</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L.]</ety> <fld>(Law)</fld> <def>Relatives by the mother's side.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognation</ent><br/
<hw>Cog*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cognatio</ets>.]</ety> <sn>1.</sn> <def>Relationship by blood; descent from the same original; kindred.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As by our <qex>cognation</qex> to the body of the first Adam.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Participation of the same nature.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A like temper and <qex>cognation</qex>.</q> <rj><qau>Sir K. Digby.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>That tie of consanguinity which exists between persons descended from the same mother; -- used in distinction from <contr>agnation</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognatus</ent><br/
||<hw>Cog*na"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a kinsman.]</ety> <fld>(Law)</fld> <def>A person connected through cognation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognisee</ent><br/
<ent>Cognisor</ent><br/
<mhw>{ <hw>Cog`ni*sor"</hw> <pr>(? <or/ ?)</pr>, <hw>Cog`ni*see</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Cognizor</er>, <er>Cognizee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognition</ent><br/
<hw>Cog*ni"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cognitio</ets>, fr. <ets>cognoscere</ets>, <ets>cognitum</ets>, to become acquainted with, to know; <ets>co-</ets> + <ets>noscere</ets>, <ets>gnoscere</ets>, to get a knowledge of. See <er>Know</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of knowing; knowledge; perception.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will not be myself nor have <qex>cognation</qex><br/
Of what I feel: I am all patience.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is known.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognitive</ent><br/
<hw>Cog"ni*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Knowing, or apprehending by the understanding; <as>as, <ex>cognitive</ex> power</as>.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognizable</ent><br/
<hw>Cog"ni*za*ble</hw> <pr>(? <or/ <?/)</pr>, <pos>a.</pos> <ety>[F. <ets>connaissable</ets>, fr. <ets>conna<icir/tre</ets> to know, L. <ets>cognoscere</ets>. See Cognition.]</ety> <sn>1.</sn> <def>Capable of being known or apprehended; <as>as, <ex>cognizable</ex> causes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fitted to be a subject of judicial investigation; capable of being judicially heard and determined.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cognizable</qex> both in the ecclesiastical and secular courts.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognizably</ent><br/
<hw>Cog"ni*za*bly</hw>, <pos>adv.</pos> <def>In a cognizable manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognizance</ent><br/
<hw>Cog"ni*zance</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>conissance</ets>, <ets>conoissance</ets>, F. <ets>connaissance</ets>, LL. <ets>cognoscentia</ets>, fr. L. <ets>cognoscere</ets> to know. See <er>Cognition</er>, and cf. <er>Cognoscence</er>, <er>Connoisseur</er>.]</ety> <sn>1.</sn> <def>Apprehension by the understanding; perception; observation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Within the <qex>cognizance</qex> and lying under the control of their divine Governor.</q> <rj><qau>Bp. Hurd</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Recollection; recognition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who, soon as on that knight his eye did glance,<br/
Eftsoones of him had perfect <qex>cognizance</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>Jurisdiction, or the power given by law to hear and decide controversies.</def> <sd>(b)</sd> <def>The hearing a matter judicially.</def> <sd>(c)</sd> <def>An acknowledgment of a fine of lands and tenements or confession of a thing done.</def> <mark>[Eng.]</mark> <sd>(d)</sd> <def>A form of defense in the action of replevin, by which the defendant insists that the goods were lawfully taken, as a distress, by defendant, acting as servant for another.</def> <mark>[Eng.]</mark>  <rj><au>Cowell. Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The distinguishing mark worn by an armed knight, usually upon the helmet, and by his retainers and followers: Hence, in general, a badge worn by a retainer or dependent, to indicate the person or party to which he belonged; a token by which a thing may be known.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wearing the liveries and <qex>cognizance</qex> of their master.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This pale and angry rose,<br/
As <qex>cognizance</qex> of my blood-drinking hate.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognizant</ent><br/
<hw>Cog"ni*zant</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[See <er>Cognizance</er>, and cf. <er>Connusant</er>.]</ety> <def>Having cognizance or knowledge. (<it>of</it>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognize</ent><br/
<hw>Cog"nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Cognizant</er>, <er>Recognize</er>.]</ety> <def>To know or perceive; to recognize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The reasoning faculty can deal with no facts until they are <qex>cognized</qex> by it.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognizee</ent><br/
<hw>Cog`ni*zee"</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One to whom a fine of land was acknowledged.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognizor</ent><br/
<hw>Cog`ni*zor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cognizance</er>.]</ety> <fld>(Law)</fld> <def>One who acknowledged the right of the plaintiff or cognizee in a fine; the defendant.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognomen</ent><br/
<hw>Cog*no"men</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: <ets>co-</ets> + (<ets>g</ets>)<ets>nomen</ets> name.]</ety> <sn>1.</sn> <def>The last of the three names of a person among the ancient Romans, denoting his house or family.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eng. Law)</fld> <def>A surname.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognominal</ent><br/
<hw>Cog*nom"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a cognomen; of the nature of a surname.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognominal</ent><br/
<hw>Cog*nom"i*nal</hw>, <pos>n.</pos> <def>One bearing the same name; a namesake.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognomination</ent><br/
<hw>Cog*nom`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cognominatio</ets>.]</ety> <def>A cognomen or surname.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognoscence</ent><br/
<hw>Cog*nos"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>cognoscentia</ets>. See <er>Cognizance</er>.]</ety> <def>Cognizance.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognoscente</ent><br/
||<hw>Cog`nos*cen"te</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cognoscenti</plw> <pr>(#)</pr>.</plu> <ety>[OIt. <ets>cognoscente</ets>, p. pr. of <ets>cognoscere</ets>, It. <ets>conoscere</ets> to know.]</ety> <def>A connoisseur.</def>  <rj><au>Mason.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognoscibility</ent><br/
<hw>Cog*nos`ci*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being cognoscible.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognoscible</ent><br/
<hw>Cog*nos"ci*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being known.</def> <ldquo/Matters intelligible and <xex>cognoscible</xex>.<rdquo/  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liable to judicial investigation.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognoscitive</ent><br/
<hw>Cog*nos"ci*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the power of knowing.</def> <mark>[Obs.]</mark> <ldquo/An innate <xex>cognoscitive</xex> power.<rdquo/  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cognovit</ent><br/
||<hw>Cog*no"vit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., he has acknowledged.]</ety> <fld>(Law)</fld> <def>An instrument in writing whereby a defendant in an action acknowledges a plaintiff's demand to be just.</def>  <rj><au>Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogon</ent><br/
<hw>Co*gon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., prob. fr. a native name.]</ety> <def>A tall, coarse grass (<spn>Imperata arundinacea</spn>) of the Philippine Islands and adjacent countries, used for thatching.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coguardian</ent><br/
<hw>Co*guard"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint guardian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogue</ent><br/
<hw>Cogue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cog</er> a small boat.]</ety> <def>A small wooden vessel; a pail.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogware</ent><br/
<hw>Cog"ware`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coarse, narrow cloth, like frieze, used by the lower classes in the sixteenth century.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cogwheel</ent><br/
<hw>Cog"wheel`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A wheel with cogs or teeth; a gear wheel. See <xex>Illust.</xex> of <er>Gearing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohabit</ent><br/
<hw>Co*hab"it</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cohabited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cohabiting</conjf>.]</vmorph> <ety>[L. <ets>cohabitare</ets>; <ets>co-</ets> + <ets>habitare</ets> to dwell, to have possession of (a place), freg. of <ets>habere</ets> to have. See <er>Habit</er>, <pos>n.</pos> & <pos>v.</pos>]</ety> <sn>1.</sn> <def>To inhabit or reside in company, or in the same place or country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Philistines were worsted by the captived ark . . . : they were not able to <qex>cohabit</qex> with that holy thing.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dwell or live together as husband and wife.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law presumes that husband and wife <qex>cohabit</qex> together, even after a voluntary separation has taken place between them.</q> <rj><qau>Bouvier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ By the common law as existing in the United States, marriage is presumed when a man and woman <xex>cohabit</xex> permanently together, being reputed by those who know them to be husband and wife, and admitting the relationship.</note>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohabitant</ent><br/
<hw>Co*hab"it*ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cohabitans</ets>, p. pr.]</ety> <def>One who dwells with another, or in the same place or country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No small number of the Danes became peaceable <qex>cohabitants</qex> with the Saxons in England.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 276 --></p>

<p><ent>Cohabitation</ent><br/
<hw>Co*hab"i*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cohabitatio</ets>.]</ety> <sn>1.</sn> <def>The act or state of dwelling together, or in the same place with another.</def>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The living together of a man and woman in supposed sexual relationship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That the duty of <qex>cohabitation</qex> is released by the cruelty of one of the parties is admitted.</q> <rj><qau>Lord Stowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohabiter</ent><br/
<hw>Co*hab"it*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cohabitant.</def>  <rj><au>Hobbes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coheir</ent><br/
<hw>Co*heir</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint heir; one of two or more heirs; one of several entitled to an inheritance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coheiress</ent><br/
<hw>Co*heir"ess</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female heir who inherits with other heiresses; a joint heiress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coheirship</ent><br/
<hw>Co*heir"ship</hw>, <pos>n.</pos> <def>The state of being a coheir.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coherald</ent><br/
<hw>Co*her"ald</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint herald.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohere</ent><br/
<hw>Co*here"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cohered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cohering</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>cohaerere</ets>, <ets>cohaesum</ets>; <ets>co-</ets> + <ets>haerere</ets> to stick, adhere. See <er>Aghast</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To stick together; to cleave; to be united; to hold fast, as parts of the same mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither knows he . . . how the solid parts of the body are united or <qex>cohere</qex> together.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be united or connected together in subordination to one purpose; to follow naturally and logically, as the parts of a discourse, or as arguments in a train of reasoning; to be logically consistent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They have been inserted where they best seemed to <qex>cohere</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To suit; to agree; to fit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Had time <qex>cohered</qex> with place, or place with wishing.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To cleave; unite; adhere; stick; suit; agree; fit; be consistent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coherency</ent><br/
<ent>Coherence</ent><br/
<mhw>{ <hw>Co*her"ence</hw> <pr>(?)</pr>, <hw>Co*her"en*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>cohaerentia</ets>: cf. F. <ets>coh<eacute/rence</ets>.]</ety> <sn>1.</sn> <def>A sticking or cleaving together; union of parts of the same body; cohesion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Connection or dependence, proceeding from the subordination of the parts of a thing to one principle or purpose, as in the parts of a discourse, or of a system of philosophy; a logical and orderly and consistent relation of parts; consecutiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Coherence</qex> of discourse, and a direct tendency of all the parts of it to the argument in hand, are most eminently to be found in him.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>the state of cohering.</def><br/
<syn><b>Syn. --</b> cohesion, cohesiveness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coherent</ent><br/
<hw>Co*her"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>cohaerens</ets>, p. pr. See <er>Cohere</er>.]</ety> <sn>1.</sn> <def>Sticking together; cleaving; as the parts of bodies; solid or fluid.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Composed of mutually dependent parts; making a logical whole; consistent; <as>as, a <ex>coherent</ex> plan, argument, or discourse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Logically consistent; -- applied to persons; <as>as, a <ex>coherent</ex> thinker</as>.</def>  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Suitable or suited; adapted; accordant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Instruct my daughter how she shall persever,<br/
That time and place, with this deceit so lawful,<br/
May prove <qex>coherent</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physics, Optics)</fld> <def>Of or pertaining to electromagnetic waves that have a constant phase relationship with each other; having identical phase at all points; <as>as, the <ex>coherent</ex> light produced by a laser</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>coherent light</ent><br/
<hw>co*her"ent light</hw>, <pos>n.</pos> <fld>(Physics, Optics)</fld> <def>Light in which the phases of all electromagnetic waves at each point on a line normal to the direction of the the beam are identical.  Coherent light is usually monochromatic, and the most common source of such light for practical uses is from a laser.</def>  <note>The constant phase relations of coherent light allows the achievement of certain effects, such as holography, not possible with noncoherent light.  The low degree of angular dispersion of coherent light beams also allows the use of such light in laser ranging, over distances as far as from the earth to the moon.</note><br/
[<source>PJC</source>]</p>

<p><ent>Coherently</ent><br/
<hw>Co*her"ent*ly</hw>, <pos>adv.</pos> <def>In a coherent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coherer</ent><br/
<hw>Co*her"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Elec.)</fld> <def>Any device in which an imperfectly conducting contact between pieces of metal or other conductors loosely resting against each other is materially improved in conductivity by the influence of Hertzian waves; -- so called by Sir O. J. Lodge in 1894 on the assumption that the impact of the electic waves caused the loosely connected parts to cohere, or weld together, a condition easily destroyed by tapping. A common form of coherer as used in wireless telegraphy consists of a tube containing filings (usually a pinch of nickel and silver filings in equal parts) between terminal wires or plugs (called <col><b>conductor plugs</b></col>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cohesibility</ent><br/
<hw>Co*he`si*bil"i*ty</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>The state of being cohesible.</def>  <rj><au>Good.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohesible</ent><br/
<hw>Co*he"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of cohesion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohesion</ent><br/
<hw>Co*he"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>coh<eacute/sion</ets>. See <er>Cohere</er>.]</ety> <sn>1.</sn> <def>The act or state of sticking together; close union.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physics)</fld> <def>That from of attraction by which the particles of a body are united throughout the mass, whether like or unlike; -- distinguished from <xex>adhesion</xex>, which unites bodies by their adjacent surfaces.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Solids and fluids differ in the degree of <qex>cohesion</qex>, which, being increased, turns a fluid into a solid.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Logical agreement and dependence; <as>as, the <ex>cohesion</ex> of ideas</as>.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohesive</ent><br/
<hw>Co*he"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Holding the particles of a homogeneous body together; <as>as, <ex>cohesive</ex> attraction</as>; producing cohesion; <as>as, a <ex>cohesive</ex> force</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cohering, or sticking together, as in a mass; capable of cohering; tending to cohere; <as>as, <ex>cohesive</ex> clay</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cohesive attraction</b></col>. <cd>See under <er>Attraction</er>.</cd></cs></p>

<p>-- <wordforms><wf>Co*he"sive*ly</wf>, <pos>adv.</pos> -- <wf>Co*he"sive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohibit</ent><br/
<hw>Co*hib"it</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cohibited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cohibiting</conjf>.]</vmorph> <ety>[L. <ets>cohibitus</ets>, p. p. of <ets>cohibere</ets> to confine; <ets>co-</ets> + <ets>habere</ets> to hold.]</ety> <def>To restrain.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohibition</ent><br/
<hw>Co`hi*bi"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cohibitio</ets>.]</ety> <def>Hindrance; restraint.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohobate</ent><br/
<hw>Co`ho*bate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cohobated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cohobating</conjf>.]</vmorph> <ety>[LL. <ets>cohobare</ets>; prob. of Arabic origin: cf. F. <ets>cohober</ets>.]</ety> <fld>(Anc. Chem.)</fld> <def>To repeat the distillation of, pouring the liquor back upon the matter remaining in the vessel.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohobation</ent><br/
<hw>Co`ho*ba"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cohobation</ets>.]</ety> <fld>(Anc. Chem.)</fld> <def>The process of cohobating.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohorn</ent><br/
<hw>Co"horn</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>See <er>Coehorn</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohort</ent><br/
<hw>Co"hort</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cohors</ets>, prop. an inclosure: cf. F. <ets>cohorte</ets>. See <er>Court</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>A body of about five or six hundred soldiers; the tenth part of a legion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any band or body of warriors.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With him the <qex>cohort</qex> bright<br/
Of watchful cherubim.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A natural group of orders of plants, less comprehensive than a class.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohosh</ent><br/
<hw>Co"hosh</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A perennial American herb (<spn>Caulophyllum thalictroides</spn>), whose rootstock is used in medicine; -- also called <altname>pappoose root</altname>. The name is sometimes also given to the <spn>Cimicifuga racemosa</spn>, and to two species of <gen>Act<ae/a</gen>, plants of the Crowfoot family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cohune palm</ent><br/
<ent>Cohune</ent><br/
<mhw>{ <hw>Co*hune"</hw> <pr>(?)</pr>, <pos>n.</pos>, or <hw>Cohune palm</hw>  }</mhw>. <ety>[Prob. fr. a native name in Honduras.]</ety> <def>A Central and South American pinnate-leaved palm (<spn>Attalea cohune</spn>), the very large and hard nuts of which are turned to make fancy articles, and also yield an oil used as a substitute for coconut oil.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coif</ent><br/
<hw>Coif</hw> <pr>(koif <it>or</it> kw<aum/f)</pr>, <pos>n.</pos> <ety>[OF. <ets>coife</ets>, F. <ets>coiffe</ets>, LL. <ets>cofea</ets>, <ets>cuphia</ets>, fr. OHG. <ets>kuppa</ets>, <ets>kuppha</ets>, miter, perh. fr. L. <ets>cupa</ets> tub. See <er>Cup</er>, <pos>n.</pos>; but cf. also <er>Cop</er>, <er>Cuff</er> the article of dress, <er>Quoif</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A cap.</def> Specifically: <sd>(a)</sd> <def>A close-fitting cap covering the sides of the head, like a small hood without a cape.</def> <sd>(b)</sd> <def>An official headdress, such as that worn by certain judges in England.</def> <altsp>[Written also <asp>quoif</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>From point and saucy ermine down<br/
To the plain <qex>coif</qex> and russet gown.</q> <rj><qau>H. Brocke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The judges, . . . althout they are not of the first magnitude, nor need be of the degree of the <qex>coif</qex>, yet are they considerable.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a <er>coiffure</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coif</ent><br/
<hw>Coif</hw> <pr>(koif <it>or</it> kw<aum/f)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>coiffer</ets>.]</ety> <def>To cover or dress with, or as with, a coif.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>coif</qex> me, where I'm bald, with flowers.</q> <rj><qau>J. G. Cooper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coifed</ent><br/
<hw>Coifed</hw> <pr>(koift)</pr>, <pos>a.</pos> <def>Wearing a coif.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coiffeur</ent><br/
||<hw>Coif`feur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A hairdresser.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>coiffeuse</ent><br/
<hw>coiffeuse</hw> <pos>n.</pos> <def>a woman hairdresser.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coiffure</ent><br/
<hw>Coif"fure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>coiffer</ets>. See <er>Coif</er>.]</ety> <def>A headdress, or manner of dressing the hair.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coign</ent><br/
<hw>Coign</hw> <pr>(koin)</pr>, <pos>n.</pos> <def>A var. spelling of <er>Coin</er>, <er>Quoin</er>, a corner, wedge; -- chiefly used in the phrase <xex>coign of vantage</xex>, a position advantageous for action or observation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>From some shielded nook or <qex>coign</qex> of vantage.</q>  <rj><qau>The Century.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>The lithosphere would be depressed on four faces; . . . the four projecting <qex>coigns</qex> would stand up as continents.</q>  <rj><qau>Nature.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>an expandable metal or wooden wedge used by printers to lock up a form within a chase.</def><br/
<syn><b>Syn. --</b> quoin, coigne.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>the keystone of an arch.</def><br/
<syn><b>Syn. --</b> quoin, coigne.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coigne</ent><br/
<hw>Coigne</hw> <pr>(koin)</pr>, <pos>n.</pos> <ety>[See <er>Coin</er>, <pos>n.</pos>]</ety> <def>A quoin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>See you yound <qex>coigne</qex> of the Capitol? yon corner stone?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coigny</ent><br/
<ent>Coigne</ent><br/
<mhw>{ <hw>Coigne</hw>, <hw>Coign"y</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The practice of quartering one's self as landlord on a tenant; a quartering of one's self on anybody.</def> <mark>[Ireland]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coil</ent><br/
<hw>Coil</hw> <pr>(koil)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coiled</conjf> (koild); <pos>p. pr. & vb. n.</pos> <conjf>Coiling</conjf>.]</vmorph> <ety>[OF. <ets>coillir</ets>, F. <ets>cueillir</ets>, to collect, gather together, L. <ets>coligere</ets>; <ets>col-</ets> + <ets>legere</ets> to gather. See <er>Legend</er>, and cf. <er>Cull</er>, <pos>v. t.</pos>, <er>Collect</er>.]</ety> <sn>1.</sn> <def>To wind cylindrically or spirally; <as>as, to <ex>coil</ex> a rope when not in use; the snake <xex>coiled</xex> itself before springing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To encircle and hold with, or as with, coils.</def> <mark>[Obs. or R.]</mark>  <rj><au>T. Edwards.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coil</ent><br/
<hw>Coil</hw>, <pos>v. i.</pos> <def>To wind itself cylindrically or spirally; to form a coil; to wind; -- often with <xex>about</xex> or <xex>around</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You can see his flery serpents . . . <br/
<qex>Coiting</qex>, playing in the water.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coil</ent><br/
<hw>Coil</hw>, <pos>n.</pos> <sn>1.</sn> <def>A ring, series of rings, or spiral, into which a rope, or other like thing, is wound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wild grapevines that twisted their <qex>coils</qex> from trec to tree.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Entanglement; toil; mesh; perplexity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A series of connected pipes in rows or layers, as in a steam heating apparatus.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Induction coil</b></col>. <fld>(Elec.)</fld> <cd>See under <er>Induction</er>.</cd> -- <col><b>Ruhmkorff's coil</b></col> <fld>(Elec.)</fld>, <cd>an induction coil, sometimes so called from <etsep>Ruhmkorff</etsep> <pr>(<?/)</pr>, a prominent manufacturer of the apparatus.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coil</ent><br/
<hw>Coil</hw>, <pos>n.</pos> <ety>[Of Celtic origin; cf. Gael. <ets>goil</ets> fume, rage.]</ety> <def>A noise, tumult, bustle, or confusion.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coiled</ent><br/
<hw>coiled</hw> <pr>(koild)</pr>, <pos>adj.</pos> <def>curled or wound especially in concentric rings or spirals; <as>as, a <ex>coiled</ex> snake ready to strike; the rope lay <ex>coiled</ex> on the deck</as>.  Opposite of <ant>uncoiled</ant>.</def> <note>[Narrower terms: <stype>coiling, helical, spiral, spiraling, volute, voluted, whorled</stype>; <stype>convolute  rolled longitudinally upon itself</stype>;<stype>curled, curled up</stype>; <stype>involute  closely coiled so that the axis is obscured)</stype>; <stype>looped, whorled</stype>; <stype>twined, twisted</stype>; <stype>convoluted</stype>; <stype>involute, rolled  esp of petals or leaves in bud: having margins rolled inward)</stype>; <stype>wound</stype>]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coiling</ent><br/
<hw>coiling</hw> <pos>adj.</pos> <sn>1.</sn> <fr>1</fr> <br/
<syn><b>Syn. --</b> helical, spiral, spiraling, volute, voluted, whorled.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coilon</ent><br/
<hw>Coi"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Cullion</er>.]</ety> <def>A testicle.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coin</ent><br/
<hw>Coin</hw> <pr>(koin)</pr>, <pos>n.</pos> <ety>[F. <ets>coin</ets>, formerly also <ets>coing</ets>, wedge, stamp, corner, fr. L. <ets>cuneus</ets> wedge; prob. akin to E. <ets>cone</ets>, <ets>hone</ets>. See <er>Hone</er>, <pos>n.</pos>, and cf. <er>Coigne</er>, <er>Quoin</er>, <er>Cuneiform</er>.]</ety> <sn>1.</sn> <def>A quoin; a corner or external angle; a wedge. See <er>Coigne</er>, and <er>Quoin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of metal on which certain characters are stamped by government authority, making it legally current as money; -- much used in a collective sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is alleged that it [a subsidy] exceeded all the current <qex>coin</qex> of the realm.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which serves for payment or recompense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The loss of present advantage to flesh and blood is repaid in a nobler <qex>coin</qex>.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coin balance</b></col>. <cd>See <xex>Illust.</xex> of <er>Balance</er>.</cd> -- <col><b>To pay one in his own coin</b></col>, <cd>to return to one the same kind of injury or ill treatment as has been received from him.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coin</ent><br/
<hw>Coin</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coined</conjf> <pr>(koind)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coining</conjf>.]</vmorph> <sn>1.</sn> <def>To make of a definite fineness, and convert into coins, as a mass of metal; to mint; to manufacture; <as>as, to <ex>coin</ex> silver dollars; to <ex>coin</ex> a medal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make or fabricate; to invent; to originate; <as>as, to <ex>coin</ex> a word</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some tale, some new pretense, he daily <qex>coined</qex>,<br/
To soothe his sister and delude her mind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To acquire rapidly, as money; to make.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tenants cannot <qex>coin</qex> rent just at quarter day.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coin</ent><br/
<hw>Coin</hw>, <pos>v. i.</pos> <def>To manufacture counterfeit money.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They cannot touch me for <qex>coining</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinage</ent><br/
<hw>Coin"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Coin</er>, <pos>v. t.</pos>, cf. <er>Cuinage</er>.]</ety> <sn>1.</sn> <def>The act or process of converting metal into money.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The care of the <qex>coinage</qex> was committed to the inferior magistrates.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Coins; the aggregate coin of a time or place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The cost or expense of coining money.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act or process of fabricating or inventing; formation; fabrication; that which is fabricated or forged.</def> <ldquo/Unnecessary <xex>coinage</xex> . . . of words.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the very <qex>coinage</qex> of your brain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincide</ent><br/
<hw>Co`in*cide"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coincided</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coinciding</conjf>.]</vmorph> <ety>[L. <ets>co-</ets> + <ets>incidere</ets> to fall on; <ets>in + cadere</ets> to fall: cf. F. <ets>co<ium/ncider</ets>. See <er>Chance</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To occupy the same place in space, as two equal triangles, when placed one on the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the equator and the ecliptic had <qex>coincided</qex>, it would have rendered the annual revoluton of the earth useless.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To occur at the same time; to be contemporaneous; <as>as, the fall of Granada <ex>coincided</ex> with the discovery of America</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To correspond exactly; to agree; to concur; <as>as, our aims <ex>coincide</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rules of right jugdment and of good ratiocination often <qex>coincide</qex> with each other.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincidence</ent><br/
<hw>Co*in"ci*dence</hw> <pr>(k<osl/*<icr/n"s<icr/*d<eit/ns)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>co<ium/ncidence</ets>.]</ety> <sn>1.</sn> <def>The condition of occupying the same place in space; <as>as, the <ex>coincidence</ex> of circles, surfaces, etc.</as></def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The condition or fact of happening at the same time; <as>as, the <ex>coincidence</ex> of the deaths of John Adams and Thomas Jefferson</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Exact correspondence in nature, character, result, circumstances, etc.; concurrence; agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The very concurrence and <qex>coincidence</qex> of so many evidences . . . carries a great weight.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who discourse . . . of the nature of truth . . . affirm a perfect <qex>coincidence</qex> between truth and goodness.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincidency</ent><br/
<hw>Co*in"ci*den*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Coincidence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincident</ent><br/
<hw>Co*in"ci*dent</hw> <pr>(k<osl/*<icr/n"s<icr/*d<eit/nt)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>co<ium/ncident</ets>.]</ety> <def>Having coincidence; occupying the same place; contemporaneous; concurrent; -- followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Christianity teaches nothing but what is perfectly suitable to, and <qex>coincident</qex> with, the ruling principles of a virtuous and well-inclined man.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincident</ent><br/
<hw>Co*in"ci*dent</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of two or more coincident events; a coincidence.</def> <mark>[R.]</mark> <ldquo/<xex>Coincidents</xex> and accidents.<rdquo/  <rj><au>Froude.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincidental</ent><br/
<hw>Co*in`ci*den"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Coincident.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincidently</ent><br/
<hw>Co*in"ci*dent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With coincidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coincider</ent><br/
<hw>Co`in*cid"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who coincides with another in an opinion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coindication</ent><br/
<hw>Co*in`di*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>co<ium/dication</ets>.]</ety> <def>One of several signs or symptoms indicating the same fact; <as>as, a <ex>coindication</ex> of disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coiner</ent><br/
<hw>Coin"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who makes or stamps coin; a maker of money; -- usually, a maker of counterfeit money.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Precautions such as are employed by <qex>coiners</qex> and receivers of stolen goods.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inventor or maker, as of words.</def>  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinhabitant</ent><br/
<hw>Co`in*hab"it*ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who dwells with another, or with others.</def> <ldquo/<xex>Coinhabitants</xex> of the same element.<rdquo/  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinhere</ent><br/
<hw>Co`in*here"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To inhere or exist together, as in one substance.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinheritance</ent><br/
<hw>Co`in*her"it*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint inheritance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinheritor</ent><br/
<hw>Co`in*her"it*or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coheir.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinitial</ent><br/
<hw>Co`in*i"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Having a common beginning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinquinate</ent><br/
<hw>Co*in"qui*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>coinquinatus</ets>, p. p. of <ets>coinquinare</ets> to defile. See <er>Inquinate</er>.]</ety> <def>To pollute.</def> <mark>[Obs.]</mark>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinquination</ent><br/
<hw>Co*in`qui*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Defilement.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinstantaneous</ent><br/
<hw>Co*in"stan*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Happening at the same instant.</def>  <rj><au>C. Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coinsurance</ent><br/
<hw>Co`in*sur"ance</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Co-</ets> + <ets>insurance</ets>.]</ety> <def>Insurance jointly with another or others; specif., that system of fire insurance in which the insurer is treated as insuring himself to the extent of that part of the risk not covered by his policy, so that any loss is apportioned between him and the insurance company on the principle of average, as in marine insurance or between other insurers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cointense</ent><br/
<hw>Co`intense"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Equal in intensity or degree; <as>as, the relations between 6 and 12, and 8 and 16, are <ex>cointense</ex></as>.</def>  <rj><au>H. Spencer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cointension</ent><br/
<hw>Co`in*ten"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The condition of being of equal in intensity; -- applied to relations; <as>as, 3:6 and 6:12 are relations of <ex>cointension</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cointension</qex> . . . is chosen indicate the equality of relations in respect of the contrast between their terms.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coir</ent><br/
<hw>Coir</hw> <pr>(koir)</pr>, <pos>n.</pos> <ety>[Tamil <ets>kayiru</ets>.]</ety> <sn>1.</sn> <def>A material for cordage, matting, etc., consisting of the prepared fiber of the outer husk of the cocoanut.</def>  <rj><au>Homans.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cordage or cables, made of this material.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coistril</ent><br/
<hw>Cois"tril</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. from OF. <ets>coustillier</ets> groom or lad. Cf. <er>Custrel</er>.]</ety> <sn>1.</sn> <def>An inferior groom or lad employed by an esquire to carry the knight's arms and other necessaries.</def> <altsp>[Written also <asp>coistrel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mean, paltry fellow; a coward.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coit</ent><br/
<hw>Coit</hw> <pr>(koit)</pr>, <pos>n.</pos> <ety>[See <er>Quoit</er>.]</ety> <def>A quoit.</def> <mark>[Obs.]</mark>  <rj><au>Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coit</ent><br/
<hw>Coit</hw>, <pos>v. t.</pos> <def>To throw, as a stone. <mark>[Obs.]</mark> See <er>Quoit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coition</ent><br/
<hw>Co*i"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>coitio</ets>, fr. <ets>coire</ets> to come together; <ets>co-</ets> + <ets>ire</ets> to go.]</ety> <def>A coming together; sexual intercourse; copulation.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cojoin</ent><br/
<hw>Co*join"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To join; to conjoin.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cojuror</ent><br/
<hw>Co*ju"ror</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who swears to another's credibility.</def>  <rj><au>W. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coke</ent><br/
<hw>Coke</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. akin to <ets>cake</ets>, n.]</ety> <def>Mineral coal charred, or depriver of its bitumen, sulphur, or other volatile matter by roasting in a kiln or oven, or by distillation, as in gas works. It is lagerly used where <?/ smokeless fire is required.</def> <altsp>[Written also <asp>coak</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Gas coke</b></col>, <cd>the coke formed in gas retorts, as distinguished from that made in ovens.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coke</ent><br/
<hw>Coke</hw>, <pos>v. t.</pos> <def>To convert into coke.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cokenay</ent><br/
<hw>Coke"nay</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cockney.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cokernut</ent><br/
<hw>Co"ker*nut`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Com.)</fld> <def>The cocoanut.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A mode of spelling introduced by the London customhouse to distinguish more widely between this and other articles spelt much in the same manner.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cokes</ent><br/
<hw>Cokes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. Cf. <er>Coax</er>.]</ety> <def>A simpleton; a gull; a dupe.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cokewold</ent><br/
<hw>Coke"wold</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cuckold.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Col-</ent><br/
<hw>Col-</hw> <pr>(<?/)</pr>. <def>A prefix signifying <xex>with</xex>, <xex>together</xex>. See <er>Com-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Col</ent><br/
||<hw>Col</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., neck, fr. L. <ets>collum</ets> neck.]</ety> <def>A short ridge connecting two higher elevations or mountains; the pass over such a ridge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cola</ent><br/
||<hw>Co"la</hw>, <pos>n.</pos>, <def>L. <pluf>pl.</pluf> of <er>Colon</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cola</ent><br/
||<hw>Co"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. a native name.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <sn>1.</sn> <def>A genus of sterculiaceous trees, natives of tropical Africa, esp. Guinea, but now naturalized in tropical America, esp. in the West Indies and Brazil.</def> <sd>(b)</sd> <def>Same as <er>Cola nut</er>, below.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a dark-colored carbonated soft drink flavored with extract of the cola nut, and often colored with caramel; also, any soft drink similar in color and flavor to a cola-flavored drink.  The name was predominantly derived from the popular soft drink <etsep>Coca-Cola</etsep>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Colaborer</ent><br/
<hw>Co*la"bor*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who labors with another; an associate in labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colander</ent><br/
<hw>Col"an*der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colans</ets>, <ets>-antis</ets>, p. pr. of <ets>colare</ets> to filter, to strain, fr. <ets>colum</ets> a strainer. Cf. <er>Cullis</er>, <er>Culvert</er>.]</ety> <def>A utensil with a bottom perforated with little holes for straining liquids, mashed vegetable pulp, etc.; a strainer of wickerwork, perforated metal, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cola seed</ent><br/
<ent>Cola nut</ent><br/
<mhw>{ <hw>Cola nut</hw>, <hw>Cola seed</hw>  }</mhw>. <fld>(Bot.)</fld> <def>The bitter fruit of <spn>Cola acuminata</spn>, which is nearly as large as a chestnut, and furnishes a stimulant, which is used in medicine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Colation</ent><br/
<hw>Co*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Colander</er>.]</ety> <def>The act or process of straining or filtering.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colatitude</ent><br/
<hw>Co*lat"i*tude</hw> <pr>(?; 134)</pr>, <pos>n.</pos> <ety>[Formed like <ets>cosine</ets>. See <er>Cosine</er>.]</ety> <def>The complement of the latitude, or the difference between any latitude and ninety degrees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colature</ent><br/
<hw>Col"a*ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>colatura</ets>, from <ets>colare</ets>: cf. F. <ets>colature</ets>. See <er>Colander</er>.]</ety> <def>The process of straining; the matter strained; a strainer.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colbertine</ent><br/
<hw>Col"ber*tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <person>Jean Baptiste <etsep>Colbert</etsep></person>, a minister of <person>Louis XIV.</person>, who encouraged the lace manufacture in France.]</ety> <def>A kind of lace.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pinners edged with <qex>colbertine</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Difference rose between<br/
Mechlin, the queen of lace, and <qex>colbertine</qex>.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colchicaceae</ent><br/
<hw>Colchicaceae</hw> <pos>n.</pos> <def>one of many subfamilies into which some classification systems subdivide the <fam>Liliaceae</fam> but not widely accepted: Colchicum; Gloriosa.</def><br/
<syn><b>Syn. --</b> family <fam>Colchicaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colchicine</ent><br/
<hw>Col"chi*cine</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>colchicine</ets>.]</ety> <fld>(Chem.)</fld> <def>A powerful vegetable alkaloid, <chform>C17H19NO5</chform>, extracted from the <spn>Colchicum autumnale</spn>, or meadow saffron, as a white or yellowish amorphous powder, with a harsh, bitter taste; -- called also <altname>colchicia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colchicum</ent><br/
<hw>Col"chi*cum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a plant with a poisonous root, fr. <ets>Colchicus</ets> Colchian, fr. <ets>Colchis</ets>, Gr. <?/, an ancient province in Asia, east of the Black Sea, where was the home of Media the sorceress.]</ety> <fld>(Bot.)</fld> <def>A genus of bulbous-rooted plants found in many parts of Europe, including the meadow saffron.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Preparations made from the poisonous bulbs and seeds, and perhaps from the flowers, of the <spn>Colchicum autumnale</spn> (meadow saffron) are used as remedies for gout and rheumatism.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 277 --></p>

<p><ent>Colcothar</ent><br/
<hw>Col"co*thar</hw> <pr>(k<ocr/l"k<osl/*th<etil/r)</pr>, <pos>n.</pos> <ety>[NL. <ets>colcothar vitrioli</ets>, fr. Ar. <ets>qolqo<tsdot/ar</ets>.]</ety> <fld>(Chem.)</fld> <def>Polishing rouge; a reddish brown oxide of iron, used in polishing glass, and also as a pigment; -- called also <altname>crocus Martis</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cold</ent><br/
<hw>Cold</hw> <pr>(k<omac/ld)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Colder</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Coldest</adjf>.]</amorph> <ety>[OE. <ets>cold</ets>, <ets>cald</ets>, AS. <ets>cald</ets>, <ets>ceald</ets>; akin to OS. <ets>kald</ets>, D. <ets>koud</ets>, G. <ets>kalt</ets>, Icel. <ets>kaldr</ets>, Dan. <ets>kold</ets>, Sw. <ets>kall</ets>, Goth. <ets>kalds</ets>, L. <ets>gelu</ets> frost, <ets>gelare</ets> to freeze. Orig. p. p. of AS. <ets>calan</ets> to be cold, Icel. <ets>kala</ets> to freeze. Cf. <er>Cool</er>, <pos>a.</pos>, <er>Chill</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Deprived of heat, or having a low temperature; not warm or hot; gelid; frigid.</def> <ldquo/The snowy top of <xex>cold</xex> Olympis.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lacking the sensation of warmth; suffering from the absence of heat; chilly; shivering; <as>as, to be <ex>cold</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not pungent or acrid.</def> <ldquo/<xex>Cold</xex> plants.<rdquo/  <rj><au>Bacon</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Wanting in ardor, intensity, warmth, zeal, or passion; spiritless; unconcerned; reserved.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cold</qex> and unconcerned spectator.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>cold</qex> relation is a zealous citizen.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Unwelcome; disagreeable; unsatisfactory.</def> <ldquo/<xex>Cold</xex> news for me.<rdquo/ <ldquo/<xex>Cold</xex> comfort.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Wanting in power to excite; dull; uninteresting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What a deal of <qex>cold</qex> business doth a man misspend the better part of life in!</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The jest grows <qex>cold</qex> . . . when in comes on in a second scene.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Affecting the sense of smell (as of hunting dogs) but feebly; having lost its odor; <as>as, a <ex>cold</ex> scent</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Not sensitive; not acute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Smell this business with a sense as <qex>cold</qex><br/
As is a dead man's nose.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Distant; -- said, in the game of hunting for some object, of a seeker remote from the thing concealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Paint.)</fld> <def>Having a bluish effect. Cf. <er>Warm</er>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cold abscess</b></col>. <cd>See under <er>Abscess</er>.</cd> -- <col><b>Cold blast</b></col> <cd>See under <er>Blast</er>, <pos>n.</pos>, 2.</cd> -- <col><b>Cold blood</b></col>. <cd>See under <er>Blood</er>, <pos>n.</pos>, 8.</cd> -- <col><b>Cold chill</b></col>, <cd>an ague fit.</cd> <au>Wright.</au> -- <col><b>Cold chisel</b></col>, <cd>a chisel of peculiar strength and hardness, for cutting cold metal.</cd> <au>Weale.</au> -- <col><b>Cold cream</b></col>. <cd>See under <er>Cream</er>.</cd> -- <col><b>Cold slaw</b></col>. <cd>See <er>Cole slaw</er>.</cd> -- <col><b>In cold blood</b></col>, <cd>without excitement or passion; deliberately.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>He was slain in <qex>cold blood</qex> after the fight was over.</q> <rj><qau>Sir W. Scott.</qau></rj></p>

<p><col><b>To give one the cold shoulder</b></col>, <cd>to treat one with neglect.</cd></cs></p>

<p><syn><b>Syn.</b> -- Gelid; bleak; frigid; chill; indifferent; unconcerned; passionless; reserved; unfeeling; stoical.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cold</ent><br/
<hw>Cold</hw>, <pos>n.</pos> <sn>1.</sn> <def>The relative absence of heat or warmth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sensation produced by the escape of heat; chilliness or chillness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When she saw her lord prepared to part,<br/
A deadly <qex>cold</qex> ran shivering to her heart.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A morbid state of the animal system produced by exposure to cold or dampness; a catarrh.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cold sore</b></col> <fld>(Med.)</fld>, <cd>a vesicular eruption appearing about the mouth as the result of a cold, or in the course of any disease attended with fever.<-- causative virus Herpes simplex --></cd> -- <col><b>To leave one out in the cold</b></col>, <cd>to overlook or neglect him.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cold</ent><br/
<hw>Cold</hw>, <pos>v. i.</pos> <def>To become cold.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cold-blooded</ent><br/
<hw>Cold"-blood`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having cold blood; -- said of fish or animals whose blood is but little warmer than the water or air about them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deficient in sensibility or feeling; hard-hearted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not thoroughbred; -- said of animals, as horses, which are derived from the common stock of a country.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coldcock</ent><br/
<hw>coldcock</hw> <pos>v. t.</pos> <def>to to knock to the ground with force.</def><br/
<syn><b>Syn. --</b> deck, dump, knock down, floor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coldcream</ent><br/
<hw>coldcream</hw> <pos>v. t.</pos> <def>to put coldcream on (one's face).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coldfinch</ent><br/
<hw>Cold"finch`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A British wagtail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cold-hearted</ent><br/
<ent>coldhearted</ent><br/
<mhw><hw>cold"heart`ed</hw>, <hw>cold"-heart`ed</hw></mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Wanting passion or feeling or emotional warmth; indifferent.</def></p>

<p>-- <wordforms><wf>cold"-heart`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>cold-heartedness</ent><br/
<hw>cold"-heart`ed*ness</hw> <pos>n.</pos> <def>an absence of concern for the welfare of others; indifference to the suffering of others.</def><br/
<syn><b>Syn. --</b> heartlessness, hardheartedness.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Coldish</ent><br/
<hw>Cold"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat cold; cool; chilly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coldly</ent><br/
<hw>Cold"ly</hw>, <pos>adv.</pos> <def>In a cold manner; without warmth, animation, or feeling; with indifference; calmly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Withdraw unto some private place,<br/
And reason <qex>coldly</qex> of your grievances.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coldness</ent><br/
<hw>Cold"ness</hw>, <pos>n.</pos> <def>The state or quality of being cold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cold-short</ent><br/
<hw>Cold"-short`</hw>, <pos>a.</pos> <ety>[Prob.  fr. Sw. <ets>kallsk<oum/r</ets>; <ets>kall</ets> cold + <ets>sk<oum/r</ets> brittle. <au>Oxf. E. D.</au>]</ety> <fld>(Metal.)</fld> <def>Brittle when cold (that is, below a red heat); <as>as, <ex>cold-short</ex> iron</as>.</def> -- <wordforms><wf>Cold"-short`ness</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cold-shut</ent><br/
<hw>Cold"-shut`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Metal.)</fld> <def>Closed while too cold to become thoroughly welded; -- said of a forging or casting.</def> -- <def2><pos>n.</pos> <def>An imperfection caused by such insufficient welding.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cold wave</ent><br/
<hw>Cold" wave"</hw>. <fld>(Meteor.)</fld> <def>In the terminology of the United States Weather Bureau, an unusual fall in temperature, to or below the freezing point, exceeding 16<deg/ in twenty-four hours or 20<deg/ in thirty-six hours, independent of the diurnal range.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cole</ent><br/
<hw>Cole</hw> <pr>(k<omac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>col</ets>, <ets>caul</ets>, AS. <ets>cawl</ets>, <ets>cawel</ets>, fr. L. <ets>caulis</ets>, the stalk or stem of a plant, esp. a cabbage stalk, cabbage, akin to Gr. <grk>kaylo`s</grk>.  Cf. <er>Cauliflower</er>, <er>Kale</er>.]</ety> <fld>(Bot.)</fld> <def>A plant of the <gen>Brassica</gen> or Cabbage genus; esp. that form of <spn>Brassica oleracea</spn> called <altname>rape</altname> and <altname>coleseed</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-legatee</ent><br/
<hw>Co-leg`a*tee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint legatee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colegoose</ent><br/
<hw>Cole"goose`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Coalgoose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colemanite</ent><br/
<hw>Cole"man*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From W.T. <ets>Coleman</ets> of San Francisco.]</ety> <fld>(Min.)</fld> <def>A hydrous borate of lime occurring in transparent colorless or white crystals, also massive, in Southern California.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colemouse</ent><br/
<hw>Cole"mouse`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Coletit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleopter</ent><br/
<hw>Co`le*op"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Coleoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleoptera</ent><br/
||<hw>Co`le*op"te*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ sheath-winged; <?/ sheath + <?/ wing.]</ety> <fld>(Zool.)</fld> <def>An order of insects having the anterior pair of wings (elytra) hard and horny, and serving as coverings for the posterior pair, which are membranous, and folded transversely under the others when not in use. The mouth parts form two pairs of jaws (mandibles and maxill<ae/) adapted for chewing. Most of the Coleoptera are known as beetles and weevils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleopterous</ent><br/
<ent>Coleopteral</ent><br/
<mhw>{ <hw>Co`le*op"ter*al</hw> <pr>(?)</pr>, <hw>Co`le*op"ter*ous</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>Having wings covered with a case or sheath; belonging to the Coleoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleopteran</ent><br/
<hw>Co`le*op"ter*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the order of Coleoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleopterist</ent><br/
<hw>Co`le*op"ter*ist</hw>, <pos>n.</pos> <def>One versed in the study of the <ord>Coleoptera</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleorhiza</ent><br/
||<hw>Co`le*o*rhi"za</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ sheath + <?/ root.]</ety> <def>A sheath in the embryo of grasses, inclosing the caulicle.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleperch</ent><br/
<hw>Cole"perch`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A kind of small black perch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colera</ent><br/
<hw>Col"e*ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cholera</ets>. See <er>Choler</er>.]</ety> <def>Bile; choler.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleridgian</ent><br/
<hw>Cole*ridg"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to Samuel Taylor <etsep>Coleridge</etsep>, or to his poetry or metaphysics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleseed</ent><br/
<hw>Cole"seed`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The common rape or cole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleslaw</ent><br/
<hw>Cole"slaw`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>kool slaa</ets> cabbage salad.]</ety> <def>A salad made of sliced cabbage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-lessee</ent><br/
<hw>Co`-les*see"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A partner in a lease taken.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-lessor</ent><br/
<hw>Co`-les*sor"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A partner in giving a lease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colestaff</ent><br/
<hw>Cole"staff`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Colstaff</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collet</ent><br/
<ent>Colet</ent><br/
<mhw>{ <hw>Col"et</hw> <pr>(?)</pr>, <hw>Col"let</hw>  }</mhw> <ety>[Corrupted fr. <ets>acolyte</ets>.]</ety> <def>An inferior church servant. <mark>[Obs.]</mark> See <er>Acolyte</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coaltit</ent><br/
<ent>Coletit</ent><br/
<mhw>{ <hw>Cole"tit`</hw> or <hw>Coal"tit</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>A small European titmouse (<spn>Parus ater</spn>), so named from its black color; -- called also <altname>coalmouse</altname> and <altname>colemouse</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coleus</ent><br/
||<hw>Co"le*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a sheath; -- referring to the manner in which the stamens are united.]</ety> <fld>(Bot.)</fld> <def>A plant of several species of the Mint family, cultivated for its bright-colored or variegated leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colewort</ent><br/
<hw>Cole"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cawlwyrt</ets>; <ets>cawl</ets> cole + <ets>wyrt</ets> wort. Cf. <er>Collards</er>.]</ety> <sn>1.</sn> <def>A variety of cabbage in which the leaves never form a compact head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any white cabbage before the head has become firm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colfox</ent><br/
<hw>Col"fox`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A crafty fox.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colic</ent><br/
<hw>Col"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. colique, fr. L. colicus sick with the colic, Gr. <?/, fr. <?/, <?/, the colon. The disease is so named from its being seated in or near the colon. See <er>Colon</er>.]</ety> <fld>(Med.)</fld> <def>A severe paroxysmal pain in the abdomen, due to spasm, obstruction, or distention of some one of the hollow viscera.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hepatic colic</b></col>, <cd>the severe pain produced by the passage of a gallstone from the liver or gall bladder through the bile duct.</cd> -- <mcol><col><b>Intestinal colic</b></col>, <it>or</it> <col><b>Ordinary colic</b></col></mcol>, <cd>pain due to distention of the intestines by gas.</cd> -- <mcol><col><b>Lead colic</b></col>, <col><b>Painter's colic</b></col></mcol>, <cd>a violent form of intestinal colic, associated with obstinate constipation, produced by chronic lead poisoning.</cd> -- <col><b>Renal colic</b></col>, <cd>the severe pain produced by the passage of a calculus from the kidney through the ureter.</cd> -- <col><b>Wind colic</b></col>. <cd>See <cref>Intestinal colic</cref>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colic</ent><br/
<hw>Col"ic</hw>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to colic; affecting the bowels.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the colon; <as>as, the <ex>colic</ex> arteries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colical</ent><br/
<hw>Col"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or of the nature of, colic.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colicky</ent><br/
<hw>Col"ick*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or troubled with, colic; <as>as, a <ex>colicky</ex> disorder</as>;  a <ex>colicky</ex> baby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colicroot</ent><br/
<hw>Col"ic*root`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bitter American herb of the Bloodwort family, with the leaves all radical, and the small yellow or white flowers in a long spike (<spn>Aletris farinosa</spn> and <spn>Aletris aurea</spn>).  Called sometimes <altname>star grass</altname>, <altname>blackroot</altname>, <altname>blazing star</altname>, and <altname>unicorn root</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colin</ent><br/
<hw>Col"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>colin</ets>; prop. a dim. of <ets>Colas</ets>, contr. fr. <ets>Nicolas</ets> Nicholas.]</ety> <fld>(Zool.)</fld> <def>The American quail or bobwhite. The name is also applied to other related species. See <er>Bobwhite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colinus</ent><br/
<hw>Colinus</hw> <pos>n.</pos> <def>a genus of birds including the New World quail and the bobwhites.</def><br/
<syn><b>Syn. --</b> genus <gen>Colinus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coliseum</ent><br/
<hw>Col`i*se"um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. (cf. It. <ets>coliseo</ets>, <ets>colosseo</ets>), fr. L. <ets>colosseus</ets> colossal, fr. <ets>colossus</ets> a colossus. See <er>Colossus</er>, and cf. <er>Colosseum</er>.]</ety> <def>The amphitheater of Vespasian at Rome, the largest in the world.</def> <altsp>[Written also <asp>Colosseum</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colitis</ent><br/
||<hw>Co*li"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>An inflammation of the large intestine, esp. of its mucous membrane; colonitis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coll</ent><br/
<hw>Coll</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OF. <ets>coler</ets>, fr. L. <ets>collum</ets> neck.]</ety> <def>To embrace.</def> <mark>[Obs.]</mark> <ldquo/They <xex>coll</xex> and kiss him.<rdquo/  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>collaborate</ent><br/
<hw>col*lab"o*rate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>to work together with another toward a common goal, especially in an intellectual endeavor; <as>as, four chemists <ex>collaborated</ex> on the synthesis of the compound; three authors <ex>collaborated</ex> in writing the book</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>to willingly cooperate with an enemy, especially an enemy nation occupying one's own country.</def><br/
[<source>PJC</source>]</p>

<p><ent>Collaborateur</ent><br/
||<hw>Col*la`bo*ra*teur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>See <er>Collaborator</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collaboration</ent><br/
<hw>Col*lab`o*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of working together; united labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the act of willingly cooperating with an enemy, especially an enemy nation occupying one's own country.</def><br/
[<source>PJC</source>]</p>

<p><ent>collaborationist</ent><br/
<hw>collaborationist</hw> <pos>n.</pos> <def>someone who collaborates with an enemy occupying force.</def><br/
<syn><b>Syn. --</b> collaborator, quisling.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>collaborative</ent><br/
<hw>col*lab"o*rat*ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>accomplished by collaboration; cooperative; <as>as, <ex>collaborative</ex> effort of industry and the universities</as>.  Opposed to <ant>competitive</ant>.</def><br/
[<source>PJC</source>]</p>

<p><ent>collaborator</ent><br/
<hw>col*lab"o*ra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collaborare</ets> to labor together; <ets>col-</ets> + <ets>laborare</ets> to labor: cf. F. <ets>collaborateur</ets>.]</ety> <sn>1.</sn> <def>An associate in labor, especially in literary or scientific labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>one who willingly cooperates with an enemy, especially an enemy nation occupying one's own country.</def><br/
[<source>PJC</source>]</p>

<p><ent>collage</ent><br/
<hw>collage</hw> <pos>n.</pos> <sn>1.</sn> <def>any picture made by sticking together pieces of paper or photographs, especially in unusual or surprising ways.</def><br/
<syn><b>Syn. --</b> montage.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the technique of producing a work of art that is a collage{1}.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>any composite object, whether abstract or concrete, created by assembling components of varied types.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Cinema)</fld> <def>a film having scenes of different types or from different locations displayed in rapid succession without transitions.</def><br/
[<source>PJC</source>]</p>

<p><ent>Collagen</ent><br/
<hw>Col"la*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`lla</grk> glue + <ets>-gen</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>The chemical basis of ordinary connective tissue, as of tendons or sinews and of bone. On being boiled in water it becomes gelatin or glue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collagenous</ent><br/
<hw>Col*lag"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Containing or resembling collagen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collapse</ent><br/
<hw>Col*lapse"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collapsed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Collapsing</conjf>]</vmorph> <ety>[L. <ets>collapsus</ets>, p. p. of <ets>collabi</ets> to collapse; <ets>col-</ets> + <ets>labi</ets> to fall, slide. See <er>Lapse</er>.]</ety> <sn>1.</sn> <def>To fall together suddenly, as the sides of a hollow vessel; to close by falling or shrinking together; to have the sides or parts of (a thing) fall in together, or be crushed in together; <as>as, a flue in the boiler of a steam engine sometimes <ex>collapses</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A balloon <qex>collapses</qex> when the gas escapes from it.</q> <rj><qau>Maunder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fail suddenly and completely, like something hollow when subject to too much pressure; to undergo a collapse; <as>as, Maximilian's government <ex>collapsed</ex> soon after the French army left Mexico; many financial projects <ex>collapse</ex> after attaining some success and importance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collapse</ent><br/
<hw>Col*lapse"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A falling together suddenly, as of the sides of a hollow vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden and complete failure; an utter failure of any kind; a breakdown.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Extreme depression or sudden failing of all the vital powers, as the result of disease, injury, or nervous disturbance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>collapsible</ent><br/
<hw>collapsible</hw> <pos>adj.</pos> <def>Capable of collapsing or being collapsed; <as>as, a <ex>collapsible</ex> boat</as>.</def> [Narrower terms: <stype>telescopic</stype>; <stype>tip-up</stype>]  <see>Also See: <er>folded</er>.</see>  Antonym: <ant>noncollapsible</ant>.<br/
<syn><b>Syn. --</b> collapsable.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Collapsion</ent><br/
<hw>Col*lap"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collapsio</ets>.]</ety> <def>Collapse.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collar</ent><br/
<hw>Col"lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>coler</ets>, <ets>coller</ets>, OF. <ets>colier</ets>, F. <ets>collier</ets>, necklace, collar, fr. OF. <ets>col</ets> neck, F. <ets>cou</ets>, fr. L. <ets>collum</ets>; akin to AS. <ets>heals</ets>, G. & Goth. <ets>hals</ets>. Cf. <er>Hals</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Something worn round the neck, whether for use, ornament, restraint, or identification; <as>as, the <ex>collar</ex> of a coat; a lady's <ex>collar</ex>; the <ex>collar</ex> of a dog</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A ring or cincture.</def> <sd>(b)</sd> <def>A collar beam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>The neck or line of junction between the root of a plant and its stem.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An ornament worn round the neck by knights, having on it devices to designate their rank or order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A ringlike part of a mollusk in connection with esophagus.</def> <sd>(b)</sd> <def>A colored ring round the neck of a bird or mammal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mech.)</fld> <def>A ring or round flange upon, surrounding, or against an object, and used for restraining motion within given limits, or for holding something to its place, or for hiding an opening around an object; <as>as, a <ex>collar</ex> on a shaft, used to prevent endwise motion of the shaft; a <ex>collar</ex> surrounding a stovepipe at the place where it enters a wall. The flanges of a piston and the gland of a stuffing box are sometimes called <ex>collars</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>An eye formed in the bight or bend of a shroud or stay to go over the masthead; also, a rope to which certain parts of rigging, as dead-eyes, are secured.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mining)</fld> <def>A curb, or a horizontal timbering, around the mouth of a shaft.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Collar beam</b></col> <fld>(Arch.)</fld>, <cd>a horizontal piece of timber connecting and tying together two opposite rafters; -- also, called simply <altname>collar</altname>.</cd> -- <col><b>Collar of brawn</b></col>, <cd>the quantity of brawn bound up in one parcel.</cd> <mark>[Eng.]</mark> <au>Johnson.</au> -- <col><b>Collar day</b></col>, <cd>a day of great ceremony at the English court, when persons, who are dignitaries of honorary orders, wear the collars of those orders.</cd> -- <col><b>To slip the collar</b></col>, <cd>to get free; to disentangle one's self from difficulty, labor, or engagement.</cd>  <rj><au>Spenser.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collar</ent><br/
<hw>Col"lar</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Collaring</conjf>.]</vmorph> <sn>1.</sn> <def>To seize by the collar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put a collar on.</def></p>

<p><sn>3.</sn> <def>to arrest, as a wanted criminal.  Same as <cref>put the collar on</cref>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>To collar beef</b></col> (or other meat), <cd>to roll it up, and bind it close with a string preparatory to cooking it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collar bone</ent><br/
<ent>collarbone</ent><br/
<mhw><hw>collarbone</hw>, <hw>Col"lar bone`</hw></mhw> <pr>(?)</pr>. <fld>(Anat.)</fld> <def>The clavicle, a bone linking the scapula and sternum.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>collard</ent><br/
<hw>collard</hw> <pos>n.</pos> <sn>1.</sn> <def>a variety of kale (<spn>Brassica oleracea</spn>) having smooth leaves; a type of <isa>colewort</isa>.  It is grown in the southern U. S.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>collard greens</ent><br/
<hw>col"lard greens</hw> <pr>(k<ocr/l"l<etil/rd z)</pr>, <pos>n. pl.</pos> <ety>[Corrupted fr. <ets>colewort</ets>.]</ety> <sn>1.</sn> <def>Same as <er>collards</er>{1}.</def> <mark>[Colloq. Southern U. S.]</mark><br/
<syn><b>Syn. -- </b>collards.</syn><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>collards</ent><br/
<hw>col"lards</hw> <pr>(k<ocr/l"l<etil/rdz)</pr>, <pos>n. pl.</pos> <ety>[Corrupted fr. <ets>colewort</ets>.]</ety> <sn>1.</sn> <def>Young cabbage, used as <ldquo/greens<rdquo/; esp. the leaves of a kind of kale (<spn>Brassica oleracea</spn> <varn>acephala</varn>) cultivated for that purpose, the collard or colewort.</def> <mark>[Colloq. Southern U. S.]</mark><br/
<syn><b>Syn. -- </b>collard greens.</syn><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A kind of kale (<spn>Brassica oleracea</spn> <varn>acephala</varn>) cultivated in the southern United States as a vegetable.</def><br/
[<source>PJC</source>]</p>

<p><ent>Collared</ent><br/
<hw>Col"lared</hw> <pr>(k<ocr/l*l<etil/rd)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Wearing a collar.</def> <ldquo/<xex>Collared</xex> with gold.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Wearing a collar; -- said of a man or beast used as a bearing when a collar is represented as worn around the neck or loins.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Rolled up and bound close with a string; <as>as, <ex>collared</ex> beef</as>. See <cref>To collar beef</cref>, under <er>Collar</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>collared lizard</ent><br/
<hw>col"lared liz"ard</hw> <pr>(?)</pr>, <pos>n.</pos> <def>any of several species of lizards of the genus <gen>Crotaphytus</gen> and the family <fam>Iguanidae</fam>, of the central and western U.S. and northern Mexico, having long tails and usually bearing markings on the neck appearing like a collar of two black bands.</def> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>Collarette</ent><br/
<ent>Collaret</ent><br/
<mhw>{ <hw>Col`lar*et"</hw> <pr>(?)</pr>,  ||<hw>Col`la*rette"</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[F. <ets>collerette</ets>, dim. of <ets>collier</ets>. See <er>Collar</er>.]</ety> <def>A small collar; specif., a woman's collar of lace, fur, or other fancy material.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Collatable</ent><br/
<hw>Col*lat"a*ble</hw> <pr>(k<ocr/l*l<amac/t"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being collated.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collate</ent><br/
<hw>Col*late"</hw> <pr>(k<ocr/l*l<amac/t")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Collating</conjf>.]</vmorph> <ety>[From <er>Collation</er>.]</ety> <sn>1.</sn> <def>To compare critically, as books or manuscripts, in order to note the points of agreement or disagreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I must <qex>collate</qex> it, word by word, with the original Hebrew.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To gather and place in order, as the sheets of a book for binding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>To present and institute in a benefice, when the person presenting is both the patron and the ordinary; -- followed by <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To bestow or confer.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collate</ent><br/
<hw>Col*late"</hw>, <pos>v. i.</pos> <fld>(Ecl.)</fld> <def>To place in a benefice, when the person placing is both the patron and the ordinary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the bishop neglects to <qex>collate</qex> within six months, the right to do it devolves on the archbishop.</q> <rj><qau>Encyc. Brit.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collateral</ent><br/
<hw>Col*lat"er*al</hw> <pr>(k<ocr/l*l<acr/t"<etil/r*<ait/l)</pr>, <pos>a.</pos> <ety>[LL. <ets>collateralis</ets>; <ets>col-</ets> + <ets>lateralis</ets> lateral. See <er>Lateral</er>.]</ety> <sn>1.</sn> <def>Coming from, being on, or directed toward, the side; <as>as, <ex>collateral</ex> pressure</as>.</def> <ldquo/<xex>Collateral</xex> light.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Acting in an indirect way.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If by direct or by <qex>collateral</qex> hand<br/
They find us touched, we will our kingdom give . . . <br/
To you in satisfaction.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Related to, but not strictly a part of, the main thing or matter under consideration; hence, subordinate; not chief or principal; <as>as, <ex>collateral</ex> interest; <ex>collateral</ex> issues</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That he [Attebury] was altogether in the wrong on the main question, and on all the <qex>collateral</qex> questions springing out of it, . . . is true.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Tending toward the same conclusion or result as something else; additional; <as>as, <ex>collateral</ex> evidence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet the attempt may give<br/
<qex>Collateral</qex> interest to this homely tale.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Genealogy)</fld> <def>Descending from the same stock or ancestor, but not in the same line or branch or one from the other; -- opposed to <contr>lineal</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Lineal</xex> descendants proceed one from another in a direct line; <ex>collateral</ex> relations spring from a common ancestor, but from different branches of that common <xex>stirps</xex> or stock. Thus the children of brothers are <ex>collateral</ex> relations, having different fathers, but a common grandfather.</note>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 278 --></p>

<p><cs><col><b>Collateral assurance</b></col>, <cd>that which is made, over and above the deed itself.</cd> -- <col><b>Collateral circulation</b></col> <fld>(Med. & Physiol.)</fld>, <cd>circulation established through indirect or subordinate branches when the supply through the main vessel is obstructed.</cd> -- <col><b>Collateral issue</b></col>. <fld>(Law)</fld> <sd>(a)</sd> <cd>An issue taken upon a matter aside from the merits of the case.</cd> <sd>(b)</sd> <cd>An issue raised by a criminal convict who pleads any matter allowed by law in bar of execution, as pardon, diversity of person, etc.</cd> <sd>(c)</sd> <cd>A point raised, on cross-examination, aside from the issue fixed by the pleadings, as to which the answer of the witness, when given, cannot subsequently be contradicted by the party asking the question.</cd> -- <col><b>Collateral security</b></col>, <cd>security for the performance of covenants, or the payment of money, besides the principal security.</cd> -- <col><b>collateral damage</b></col>, <fld>(Mil.)</fld> <cd>damage caused by a military operation, such as a bombing, to objects or persons not themselves the intended target of the attack.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Collateral</ent><br/
<hw>Col*lat"er*al</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A collateral relative.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Collateral security; that which is pledged or deposited as collateral security.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collaterally</ent><br/
<hw>Col*lat"er*al*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Side by side; by the side.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These pulleys . . . placed <qex>collaterally</qex>.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In an indirect or subordinate manner; indirectly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The will hath force upon the conscience <qex>collaterally</qex> and indirectly.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In collateral relation; not lineally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collateralness</ent><br/
<hw>Col*lat"er*al*ness</hw>, <pos>n.</pos> <def>The state of being collateral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collation</ent><br/
<hw>Col*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>collacioun</ets> speech, conference, reflection, OF. <ets>collacion</ets>, F. <ets>collation</ets>, fr. L. <ets>collatio</ets> a bringing together, comparing, fr. <ets>collatum</ets> (used as the supine of <ets>conferre</ets>); <ets>col-</ets> + <ets>latium</ets> (used as the supine of <ets>ferre</ets> to bear), for <ets>tlatum</ets>. See <er>Tolerate</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of collating or comparing; a comparison of one copy er thing (as of a book, or manuscript) with another of a like kind; comparison, in general.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>The gathering and examination of sheets preparatory to binding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of conferring or bestowing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not by the <qex>collation</qex> of the king . . . but by the people.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A conference.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Eccl. Law)</fld> <def>The presentation of a clergyman to a benefice by a bishop, who has it in his own gift.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>The act of comparing the copy of any paper with its original to ascertain its conformity.</def> <sd>(b)</sd> <def>The report of the act made by the proper officers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Scots Law)</fld> <def>The right which an heir has of throwing the whole heritable and movable estates of the deceased into one mass, and sharing it equally with others who are of the same degree of kindred.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This also obtains in the civil law, and is found in the code of Louisiana.</note>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Eccles.)</fld> <def>A collection of the Lives of the Fathers or other devout work read daily in monasteries.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A light repast or luncheon; <as>as, a cold <ex>collation</ex></as>; -- first applied to the refreshment on fast days that accompanied the reading of the collation in monasteries.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>collation</qex> of wine and sweetmeats.</q> <rj><qau>Whiston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Collation of seals</b></col> <fld>(Old Law)</fld>, <cd>a method of ascertaining the genuineness of a seal by comparing it with another known to be genuine.</cd>  <rj><au>Bouvier.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collation</ent><br/
<hw>Col*la"tion</hw>, <pos>v. i.</pos> <def>To partake of a collation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>May 20, 1658, I . . . <qex>collationed</qex> in Spring Garden.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collationer</ent><br/
<hw>Col*la"tion*er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Print.)</fld> <def>One who examines the sheets of a book that has just been printed, to ascertain whether they are correctly printed, paged, etc.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collatitious</ent><br/
<hw>Col`la*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>collatitius</ets>. See <er>Collation</er>.]</ety> <def>Brought together; contributed; done by contributions.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collative</ent><br/
<hw>Col*la"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>collativus</ets> brought together. ]</ety> <def>Passing or held by collation; -- said of livings of which the bishop and the patron are the same person.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collator</ent><br/
<hw>Col*la"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <sn>1.</sn> <def>One who collates manuscripts, books, etc.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl. Law)</fld> <def>One who collates to a benefice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who confers any benefit.</def> <mark>[Obs.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collaud</ent><br/
<hw>Col*laud"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>collaudare</ets>; <ets>col-</ets> + <ets>laudare</ets> to praise.]</ety> <def>To join in praising.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colleague</ent><br/
<hw>Col"league</hw> <pr>(k<ocr/l"l<emac/g)</pr>, <pos>n.</pos> <ety>[F. <ets>coll<egrave/gue</ets>, L. <ets>collega</ets> one chosen at the same time with another, a partner in office; <ets>col-</ets> + <ets>legare</ets> to send or choose as deputy. See <er>Legate</er>.]</ety> <def>A partner or associate in some civil or ecclesiastical office or employment. It is never used of partners in trade or manufactures.</def></p>

<p><syn><b>Syn.</b> -- Helper; assistant; coadjutor; ally; associate; companion; confederate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colleague</ent><br/
<hw>Col*league"</hw> <pr>(k<ocr/l*l<emac/g")</pr>, <pos>v. t. & i.</pos> <def>To unite or associate with another or with others.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colleagueship</ent><br/
<hw>Col"league*ship</hw>, <pos>n.</pos> <def>Partnership in office.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collect</ent><br/
<hw>Col*lect"</hw> <pr>(k<ocr/l*l<ecr/kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Collecting</conjf>.]</vmorph> <ety>[L. <ets>collecrus</ets>, <pos>p. p.</pos> of <ets>collerige</ets> to bind together; <ets>col-</ets> + <ets>legere</ets> to gather: cf. OF. <ets>collecter</ets>. See <er>Legend</er>, and cf. <er>Coil</er>, <pos>v. t.</pos>, <er>Cull</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To gather into one body or place; to assemble or bring together; to obtain by gathering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A band of men<br/
<qex>Collected</qex> choicely from each country.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'Tis memory alone that enriches the mind, by preserving what our labor and industry daily <qex>collect</qex>.</q> <rj><qau>Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To demand and obtain payment of, as an account, or other indebtedness; <as>as, to <ex>collect</ex> taxes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To infer from observed facts; to conclude from premises.</def> <mark>[Archaic.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which sequence, I conceive, is very ill <qex>collected</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To collect one's self</b></col>, <cd>to recover from surprise, embarrassment, or fear; to regain self-control.</cd></cs></p>

<p><syn><b>Syn.</b> -- To gather; assemble; congregate; muster; accumulate; garner; aggregate; amass; infer; deduce.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collect</ent><br/
<hw>Col*lect"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To assemble together; <as>as, the people <ex>collected</ex> in a crowd</as>; to accumulate; <as>as, snow <ex>collects</ex> in banks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To infer; to conclude.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Whence some <qex>collect</qex> that the former word imports a plurality of persons.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collect</ent><br/
<hw>Col"lect</hw>, <pos>n.</pos> <ety>[LL. <ets>collecta</ets>, fr. L. <ets>collecta</ets> a collection in money; an assemblage, fr. <ets>collerige</ets>: cf. F. <ets>collecte</ets>. See <er>Collect</er>, <pos>v. t.</pos>]</ety> <def>A short, comprehensive prayer, adapted to a particular day, occasion, or condition, and forming part of a liturgy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The noble poem on the massacres of Piedmont is strictly a <qex>collect</qex> in verse.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectanea</ent><br/
||<hw>Col`lec*ta"ne*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Neut. pl. from L. <ets>collectaneus</ets> collected, fr. <ets>colligere</ets>. See <er>Collect</er>, <pos>v. t.</pos>]</ety> <def>Passages selected from various authors, usually for purposes of instruction; miscellany; anthology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collected</ent><br/
<hw>Col*lect"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Gathered together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Self-possessed; calm; composed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectedly</ent><br/
<hw>Col*lect"ed*ly</hw>, <pos>adv.</pos> <def>Composedly; coolly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectedness</ent><br/
<hw>Col*lect"ed*ness</hw>, <pos>n.</pos> <def>A collected state of the mind; self-possession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectible</ent><br/
<hw>Col*lect"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being collected.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collection</ent><br/
<hw>Col*lec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collectio</ets>: cf. F. <ets>collection</ets>.]</ety> <sn>1.</sn> <def>The act or process of collecting or of gathering; <as>as, the <ex>collection</ex> of specimens</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is collected</def>; as: <sd>(a)</sd> <def>A gathering or assemblage of objects or of persons.</def> <ldquo/A <xex>collection</xex> of letters.<rdquo/ <au>Macaulay.</au> <sd>(b)</sd> <def>A gathering of money for charitable or other purposes, as by passing a contribution box for freewill offerings.</def> <ldquo/The <xex>collection</xex> for the saints.<rdquo/ <au>1 Cor. xvi. 1</au> <sd>(c)</sd> (<xex>Usually in pl</xex>.) <def>That which is obtained in payment of demands.</def> <sd>(d)</sd> <def>An accumulation of any substance.</def> <ldquo/<xex>Collections</xex> of moisture.<rdquo/ <au>Whewell.</au> <ldquo/A purulent <xex>collection</xex>.<rdquo/ <au>Dunglison.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of inferring or concluding from premises or observed facts; also, that which is inferred.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We may safely say thus, that wrong <qex>collections</qex> have been hitherto made out of those words by modern divines.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The jurisdiction of a collector of excise.</def> <mark>[Eng.]</mark></p>

<p><syn><b>Syn.</b> -- Gathering; assembly; assemblage; group; crowd; congregation; mass; heap; compilation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectional</ent><br/
<hw>Col*lec"tion*al</hw> <pr>(-<it>a</it>l)</pr>, <pos>a.</pos> <def>Of or pertaining to collecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first twenty-five [years] must have been wasted for <qex>collectional</qex> purposes.</q> <rj><qau>H. A. Merewether.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collective</ent><br/
<hw>Col*lect"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>collectivus</ets>: cf. F. <ets>collectif</ets>.]</ety> <sn>1.</sn> <def>Formed by gathering or collecting; gathered into a mass, sum, or body; congregated or aggregated; <as>as, the <er>collective</er> body of a nation</as>.</def>  <rj><au>Bp. Hoadley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deducing consequences; reasoning; inferring.</def> <mark>[Obs.]</mark> <ldquo/Critical and <xex>collective</xex> reason.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>Expressing a collection or aggregate of individuals, by a singular form; <as>as, a <ex>collective</ex> name or noun, like <ex>assembly</ex>, <ex>army</ex>, <ex>jury</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Tending to collect; forming a collection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Local is his throne . . . to fix a point,<br/
A central point, <qex>collective</qex> of his sons.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Having plurality of origin or authority; <as>as, in diplomacy, a note signed by the representatives of several governments is called a <ex>collective</ex> note</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Collective fruit</b></col> <fld>(Bot.)</fld>, <cd>that which is formed from a mass of flowers, as the mulberry, pineapple, and the like; -- called also <altname>multiple fruit</altname>.</cd>  <rj><au>Gray.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collective</ent><br/
<hw>Col*lect"ive</hw>, <pos>n.</pos> <fld>(Gram.)</fld> <def>A collective noun or name.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectively</ent><br/
<hw>Col*lect"ive*ly</hw>, <pos>adv.</pos> <def>In a mass, or body; in a collected state; in the aggregate; unitedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectiveness</ent><br/
<hw>Col*lect"ive*ness</hw>, <pos>n.</pos> <def>A state of union; mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectivism</ent><br/
<hw>Col*lect"iv*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>collectivisme</ets>.]</ety> <fld>(Polit. Econ.)</fld> <def>The doctrine that land and capital should be owned by society collectively or as a whole; communism.</def>  <rj><au>W. G. Summer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectivist</ent><br/
<hw>Col*lect"iv*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>collectiviste</ets>.]</ety> <def>An advocate of collectivism.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>collectivistic</ent><br/
<ent>Collectivist</ent><br/
<mhw><hw>Col*lect"iv*ist</hw>, <hw>col*lec`tiv*is"tic</hw></mhw> <pos>a.</pos> <sn>1.</sn> <def>Relating to, characteristic of, or advocating, collectivism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>operating under collectivist principles.</def>  <note>[Narrower terms: <stype>socialistic (vs. capitalistic), socialist</stype>; <stype>communistic</stype>]</note><br/
<syn><b>Syn. --</b> collectivized, state-controlled.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>collectivise</ent><br/
<hw>col*lec"tiv*ise</hw> <pos>v. t.</pos> <def>to bring under collective control; to organize for operation by the principles of collectivism; -- especially of farms and industrial enterprises.  Same as <er>collectivize</er>.</def> <mark>[Brit.]</mark><br/
<syn><b>Syn. --</b> collectivize.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Collectivity</ent><br/
<hw>Col`lec*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Quality or state of being collective.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The collective sum, aggregate, or mass of anything; specif., the people as a body; the state.</def></p>

<p><q>The proposition to give work by the <qex>collectivity</qex> is supposed to be in contravention of the sacred principle of monopolistic competition.</q>  <rj><qau>W. D. Howells.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <fld>(Polit. Econ.)</fld> <def>Collectivism.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>collectivize</ent><br/
<hw>col*lec"tiv*ize</hw> <pos>v. t.</pos> <def>to bring under collective control; to organize for operation by the principles of collectivism; -- especially of farms and industrial enterprises.</def><br/
<syn><b>Syn. --</b> collectivise.</syn><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><ent>collectivized</ent><br/
<hw>col*lec"tiv*ized</hw> <pos>a.</pos> <def>brought under collective ownership and operating under collectivist principles; -- used especially of organizations operating under the ownership by the state.</def> <note>[Narrower terms: <stype>socialistic (vs. capitalistic), socialist</stype>; <stype>communistic</stype>]</note><br/
<syn><b>Syn. --</b> collective, collectivised, state-controlled.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Collector</ent><br/
<hw>Col*lect"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>collector</ets> one who collects: cf. F. <ets>collecteur</ets>.]</ety> <sn>1.</sn> <def>One who collects things which are separate; esp., one who makes a business or practice of collecting works of art, objects in natural history, etc.; <as>as, a <ex>collector</ex> of coins</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I digress into Soho to explore a bookstall. Methinks I have been thirty years a <qex>collector</qex>.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A compiler of books; one who collects scattered passages and puts them together in one book.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Volumes without the <qex>collector's</qex> own reflections.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>An officer appointed and commissioned to collect and receive customs, duties, taxes, or toll.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A great part of this is now embezzled . . . by <qex>collectors</qex>, and other officers.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One authorized to collect debts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A bachelor of arts in Oxford, formerly appointed to superintend some scholastic proceedings in Lent.</def>  <rj><au>Todd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectorate</ent><br/
<hw>Col*lect"or*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The district of a collector of customs; a collectorship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collectorship</ent><br/
<hw>Col*lect"or*ship</hw>, <pos>n.</pos> <def>The office of a collector of customs or of taxes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colleen</ent><br/
<hw>Col*leen"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir. <ets>cailin</ets>.]</ety> <def>A girl; a maiden.</def> <mark>[Anglo-Irish]</mark></p>

<p><q>Of all the <qex>colleens</qex> in the land<br/
Sweet Mollie is the daisy.</q>  <rj><qau>The Century.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Collegatary</ent><br/
<hw>Col*leg"a*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collegetarius</ets>. See <er>Legatary</er>.]</ety> <fld>(Law)</fld> <def>A joint legatee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>College</ent><br/
<hw>Col"lege</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>coll<egrave/ge</ets>, L. <ets>collegium</ets>, fr. <ets>collega</ets> colleague. See <er>Colleague</er>.]</ety> <sn>1.</sn> <def>A collection, body, or society of persons engaged in common pursuits, or having common duties and interests, and sometimes, by charter, peculiar rights and privileges; <as>as, a <ex>college</ex> of heralds; a <ex>college</ex> of electors; a <ex>college</ex> of bishops</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>college</qex> of the cardinals.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then they made <qex>colleges</qex> of sufferers; persons who, to secure their inheritance in the world to come, did cut off all their portion in this.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A society of scholars or friends of learning, incorporated for study or instruction, esp. in the higher branches of knowledge; <as>as, the <ex>colleges</ex> of Oxford and Cambridge Universities, and many American <ex>colleges</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In France and some other parts of continental Europe, <xex>college</xex> is used to include schools occupied with rudimentary studies, and receiving children as pupils.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A building, or number of buildings, used by a college.</def> <ldquo/The gate of Trinity <xex>College</xex>.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: A community.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thick as the <qex>college</qex> of the bees in May.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>College of justice</b></col>, <cd>a term applied in Scotland to the supreme civil courts and their principal officers.</cd> -- <col><b>The sacred college</b></col>, <cd>the college or cardinals at Rome.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collegial</ent><br/
<hw>Col*le"gi*al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>collegialis</ets>.]</ety> <def>Collegiate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collegian</ent><br/
<hw>Col*le"gi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member of a college, particularly of a literary institution so called; a student in a college.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collegiate</ent><br/
<hw>Col*le"gi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>collegiatus</ets>.]</ety> <def>Of or pertaining to a college; <as>as, <ex>collegiate</ex> studies; a <ex>collegiate</ex> society</as>.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Collegiate church</b></col>. <sd>(a)</sd> <cd>A church which, although not a bishop's seat, resembles a cathedral in having a <xex>college</xex>, or chapter of canons (and, in the Church of England, a dean), as Westminster Abbey.</cd> <sd>(b)</sd> <cd>An association of churches, possessing common revenues and administered under the joint pastorate of several ministers; <as>as, the Reformed (Dutch) <ex>Collegiate Church</ex> of New York.</as></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collegiate</ent><br/
<hw>Col*le"gi*ate</hw>, <pos>n.</pos> <def>A member of a college.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collembola</ent><br/
||<hw>Col*lem"bo*la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ko`lla</grk> glue + <grk>'e`mbolon</grk> wedge, peg; -- so called from their having collophores.]</ety> <fld>(Zool.)</fld> <def>The division of Thysanura which includes <gen>Podura</gen>, and allied forms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collenchyma</ent><br/
||<hw>Col*len"chy*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ko`lla</grk> glue + <?/ an infusion. Formed like <ets>parenchyma</ets>.]</ety> <fld>(Bot.)</fld> <def>A tissue of vegetable cells which are thickend at the angles and (usually) elongated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collet</ent><br/
<hw>Col"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>collet</ets>, dim. fr. L. <ets>collum</ets> neck. See <er>Collar</er>.]</ety> <sn>1.</sn> <def>A small collar or neckband.</def>  <rj><au>Foxe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A small metal ring; a small collar fastened on an arbor; <as>as, the <ex>collet</ex> on the balance arbor of a watch</as>; a small socket on a stem, for holding a drill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Jewelry)</fld> <sd>(a)</sd> <def>The part of a ring containing the bezel in which the stone is set.</def> <sd>(b)</sd> <def>The flat table at the base of a brilliant. See <xex>Illust.</xex> of <er>Brilliant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How full the <qex>collet</qex> with his jewel is!</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colleterial</ent><br/
<hw>Col`le*te"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the colleterium of insects.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colleterium</ent><br/
||<hw>Col`le*te"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Colletic</er>.]</ety> <fld>(Zool.)</fld> <def>An organ of female insects, containing a cement to unite the ejected ova.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colletic</ent><br/
<hw>Col*let"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>colleticus</ets> suitable for gluing, Gr. <?/, fr. <?/ to glue, <grk>ko`lla</grk> glue.]</ety> <def>Agglutinant.</def> -- <def2><pos>n.</pos> <def>An agglutinant.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colley</ent><br/
<hw>Col"ley</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Collie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collide</ent><br/
<hw>Col*lide"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>collidere</ets>, <ets>collisum</ets>; <ets>col-</ets> + <ets>laedere</ets> to strike. See <er>Lesion</er>.]</ety> <def>To strike or dash against each other; to come into collision; to clash; <as>as, the vessels <ex>collided</ex>; their interests <ex>collided</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Across this space the attraction urges them. They <qex>collide</qex>, they recoil, they oscillate.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No longer rocking and swaying, but clashing and <qex>colliding</qex>.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collide</ent><br/
<hw>Col*lide"</hw>, <pos>v. t.</pos> <def>To strike or dash against.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Scintillations are . . . inflammable effluencies from the bodies <qex>collided</qex>.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>collider</ent><br/
<hw>col*lid"er</hw> <pos>n.</pos> <fld>(Physics)</fld> <def>a <isa>particle accelerator</isa> in which two separate beams of particles (usually of opposite charge) are circulated in opposite directions and directed so as to collide head on.  This technique allows the production of collisions of higher energy than would be possible with a single beam produced by the same device.</def><br/
<syn><b>Syn. --</b> particle collider; colliding-beam accelerator; colliding-beam machine.</syn>
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Collidine</ent><br/
<hw>Col"li*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`lla</grk> glue.]</ety> <fld>(Chem.)</fld> <def>One of a class of organic bases, <chform>C8H11N</chform>, usually pungent oily liquids, belonging to the pyridine series, and obtained from bone oil, coal tar, naphtha, and certain alkaloids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>colliding-beam machine</ent><br/
<hw>colliding-beam machine</hw> <pos>n.</pos> <fld>(Physics)</fld> <def>a <isa>particle accelerator</isa> in which two separate beams of particles (usually of opposite charge) are circulated in opposite directions and directed so as to collide head on; -- called also <altname>colliding-beam accelerator</altname> and <altname>collider</altname>.</def>  <note>This technique allows the production of collisions of higher energy than would be possible with a single beam produced by the same device.</note><br/
<syn><b>Syn. --</b> particle collider; colliding-beam accelerator; collider.</syn>
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Collie</ent><br/
<hw>Col"lie</hw> <pr>(k<ocr/l"l<ycr/)</pr>, <pos>n.</pos> <ety>[Gael. <ets>cuilean</ets> whelp, puppy, dog.]</ety> <fld>(Zool.)</fld> <def>The Scotch shepherd dog. There are two breeds, the rough-haired and smooth-haired. It is remarkable for its intelligence, displayed especially in caring for flocks.</def> <altsp>[Written also <asp>colly</asp>, <asp>colley</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collied</ent><br/
<hw>Col"lied</hw> <pr>(?)</pr>, <pos>p. & a.</pos> <def>Darkened. See <er>Colly</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collier</ent><br/
<hw>Col"lier</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>colier</ets>. See <er>Coal</er>.]</ety> <sn>1.</sn> <def>One engaged in the business of digging mineral coal or making charcoal, or in transporting or dealing in coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel employed in the coal trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliery</ent><br/
<hw>Col"lier*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  <plw>Collieries</plw> <pr>(#)</pr>.</plu> <ety>[Cf. <er>Coalery</er>, <er>Collier</er>.]</ety> <sn>1.</sn> <def>The place where coal is dug; a coal mine, and the buildings, etc., belonging to it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The coal trade.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliflower</ent><br/
<hw>Col"li*flow`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cauliflower</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colligate</ent><br/
<hw>Col"li*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Colligated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Colligating</conjf>.]</vmorph> <ety>[L. <ets>colligatus</ets>, p. p. of <ets>colligare</ets> to collect; <ets>co-</ets> + <ets>ligare</ets> to bind.]</ety> <sn>1.</sn> <def>To tie or bind together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pieces of isinglass are <qex>colligated</qex> in rows.</q> <rj><qau>Nicholson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>To bring together by colligation; to sum up in a single proposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had discovered and <qex>colligated</qex> a multitude of the most wonderful . . . phenomena.</q> <rj><qau>Tundall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colligate</ent><br/
<hw>Col"li*gate</hw>, <pos>a.</pos> <def>Bound together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colligation</ent><br/
<hw>Col`li*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colligatio</ets>.]</ety> <sn>1.</sn> <def>A binding together.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>That process by which a number of isolated facts are brought under one conception, or summed up in a general proposition, as when Kepler discovered that the various observed positions of the planet Mars were points in an ellipse.</def> <ldquo/The <xex>colligation</xex> of facts.<rdquo/  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Colligation</qex> is not always induction, but induction is always <qex>colligation</qex>.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collimate</ent><br/
<hw>Col"li*mate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collimated</conjf>; <pos>p. p. & vb. n.</pos> <conjf>Collimating</conjf>.]</vmorph> <ety>[See <er>Collimation</er>.]</ety> <fld>(Physics & Astron.)</fld> <def>To render parallel to a certain line or direction; to bring into the same line, as the axes of telescopes, etc.; to render parallel, as rays of light.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 279 --></p>

<p><cs><col><b>Collimating eyepiece</b></col>, <cd>an eyepiece with a diagonal reflector for illumination, used to determine the error of collimation in a transit instrument by observing the image of a cross wire reflected from mercury, and comparing its position in the field with that of the same wire seen directly.</cd> -- <col><b>Collimating lens</b></col> <fld>(Optics)</fld>, <cd>a lens used for producing parallel rays of light.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collimation</ent><br/
<hw>Col`li*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>collimation</ets>, fr. a false reading (<ets>collimare</ets>) for L. <ets>collineare</ets> to direct in a straight line; <ets>col-</ets> + linea line. Cf. <er>Collineation</er>.]</ety> <def>The act of collimating; the adjustment of the line of the sights, as the axial line of the telescope of an instrument, into its proper position relative to the other parts of the instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Error of collimation</b></col>, <cd>the deviation of the line collimation of an astronomical instrument from the position it ought to have with respect to the axis of motion of the instrument.</cd> -- <col><b>Line of collimation</b></col>, <cd>the axial line of the telescope of an astronomical or geodetic instrument, or the line which passes through the optical center of the object glass and the intersection of the cross wires at its focus.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collimator</ent><br/
<hw>Col"li*ma`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Astron.)</fld> <def>A telescope arranged and used to determine errors of collimation, both vertical and horizontal.</def>  <rj><au>Nichol.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Optics)</fld> <def>A tube having a convex lens at one end and at the other a small opening or slit which is at the principal focus of the lens, used for producing a beam of parallel rays; also, a lens so used.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collin</ent><br/
<hw>Col"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`lla</grk> glue.]</ety> <def>A very pure form of gelatin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colline</ent><br/
<hw>Col"line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>colline</ets>, fr. L. <ets>collis</ets> a hill.]</ety> <def>A small hill or mount.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And watered park, full of fine <qex>collines</qex> and ponds.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collineation</ent><br/
<hw>Col*lin`e*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collineare</ets> to direct in a straight line. See <er>Collimation</er>.]</ety> <def>The act of aiming at, or directing in a line with, a fixed object.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colling</ent><br/
<hw>Coll"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Coll</er>, <pos>v. t.</pos>]</ety> <def>An embrace; dalliance.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collingly</ent><br/
<hw>Coll"ing*ly</hw>, <pos>adv.</pos> <def>With embraces.</def> <mark>[Obs.]</mark>  <rj><au>Gascoigne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collingual</ent><br/
<hw>Col*lin"gual</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having, or pertaining to, the same language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliquable</ent><br/
<hw>Col*liq"ua*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Liable to melt, grow soft, or become fluid.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliquament</ent><br/
<hw>Col*liq"ua*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The first rudiments of an embryo in generation.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliquate</ent><br/
<hw>Col"li*quate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Colliquated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Colliquating</conjf>.]</vmorph> <ety>[Pref. <ets>col-</ets> + L. <ets>liquare</ets>, <ets>liquatum</ets>, to melt.]</ety> <def>To change from solid to fluid; to make or become liquid; to melt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The ore of it is <qex>colliquated</qex> by the violence of the fire.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Ice] will <qex>colliquate</qex> in water or warm oil.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliquation</ent><br/
<hw>Col`li*qua"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A melting together; the act of melting; fusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When sand and ashes are well melted together and suffered to cool, there is generated, by the <qex>colliquation</qex>, that sort of concretion we call <ldquo/glass<rdquo/.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A processive wasting or melting away of the solid parts of the animal system with copious excretions of liquids by one or more passages.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliquative</ent><br/
<hw>Col*liq"ua*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Causing rapid waste or exhaustion; melting; <as>as, <ex>colliquative</ex> sweats</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colliquefaction</ent><br/
<hw>Col*liq`ue*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colliquefactus</ets> melted; <ets>col-</ets> + <ets>liquefacere</ets>; <ets>liqu<emac/re</ets> to be liquid + <ets>facere</ets> to make.]</ety> <def>A melting together; the reduction of different bodies into one mass by fusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The incorporation of metals by simple <qex>colliquefaction</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collish</ent><br/
<hw>Col"lish</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Shoemaking)</fld> <def>A tool to polish the edge of a sole.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collision</ent><br/
<hw>Col*li"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collisio</ets>, fr. <ets>collidere</ets>. See <er>Collide</er>.]</ety> <sn>1.</sn> <def>The act of striking together; a striking together, as of two hard bodies; a violent meeting, as of railroad trains; a clashing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state of opposition; antagonism; interference.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>collision</qex> of contrary false principles.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sensitive to the most trifling <qex>collisions</qex>.</q> <rj><qau>W. Irving.</qau></rj></p>

<p><syn><b>Syn.</b> -- Conflict; clashing; encounter; opposition.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collisive</ent><br/
<hw>Col*li"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Colliding; clashing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collitigant</ent><br/
<hw>Col*lit"i*gant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Disputing or wrangling.</def> <mark>[Obs.]</mark> -- <def2><pos>n.</pos> <def>One who litigates or wrangles.</def> <mark>[Obs.]</mark></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collocate</ent><br/
<hw>Col"lo*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>collocatus</ets>, p. p. of <ets>collocare</ets>. See <er>Couch</er>.]</ety> <def>Set; placed.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collocate</ent><br/
<hw>Col"lo*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collocated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Collocating</conjf> <pr>(?)</pr>.]</vmorph> <def>To set or place; to set; to station.</def>
<-- sic.  why is set repeated? --><br/
[<source>1913 Webster</source>]</p>

<p><q>To marshal and <qex>collocate</qex> in order his battalions.</q> <rj><qau>E. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collocation</ent><br/
<hw>Col`lo*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collocatio</ets>.]</ety> <sn>1.</sn> <def>The act of placing; the state of being placed with something else; disposition in place; arrangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The choice and <qex>collocation</qex> of words.</q> <rj><qau>Sir W. Jones.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Linguistics)</fld> <def>a combination of related words within a sentence that occurs more frequently than would be predicted in a random arrangement of words; a combination of words that occurs with sufficient frequency to be recongizable as a common combination, especially a pair of words that occur adjacent to each other.  Also called <altname>stable collocation</altname>.  Combinations of words having intervening words between them, such as verb and object pairs, may also be <ex>collocations</ex>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Collocution</ent><br/
<hw>Col`lo*cu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collocutio</ets>, fr. <ets>colloqui</ets>, <ets>-locutum</ets>, to converse; <ets>col-</ets> + <ets>loqui</ets> to speak. See <er>Loquacious</er>.]</ety> <def>A speaking or conversing together; conference; mutual discourse.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collocutor</ent><br/
<hw>Col"lo*cu`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collocutor</ets>]</ety> <def>One of the speakers in a dialogue.</def>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collodion</ent><br/
<hw>Col*lo"di*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ like glue; <grk>ko`lla</grk> glue + <?/ form. Cf. <er>Colloid</er>.]</ety> <fld>(Chem.)</fld> <def>A solution of pyroxylin (soluble gun cotton) in ether containing a varying proportion of alcohol. It is strongly adhesive, and is used by surgeons as a coating for wounds; but its chief application is as a vehicle for the sensitive film in photography.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Collodion process</b></col> <fld>(Photog.)</fld>, <cd>a process in which a film of sensitized collodion is used in preparing the plate for taking a picture.</cd> -- <col><b>Styptic collodion</b></col>, <cd>collodion containing an astringent, as tannin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collodionize</ent><br/
<hw>Col*lo"di*on*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To prepare or treat with collodion.</def>  <rj><au>R. Hunt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collodiotype</ent><br/
<hw>Col*lo"di*o*type</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A picture obtained by the collodion process; a melanotype or ambrotype.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collodium</ent><br/
<hw>Col*lo"di*um</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Collodion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collogue</ent><br/
<hw>Col*logue"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. L. <ets>colloqui</ets> and E. <ets>dialogue</ets>. Cf. <er>Collocution</er>.]</ety> <def>To talk or confer secretly and confidentially; to converse, especially with evil intentions; to plot mischief.</def> <mark>[Archaic or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pray go in; and, sister, salve the matter,<br/
<qex>Collogue</qex> with her again, and all shall be well.</q> <rj><qau>Greene.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had been <qex>colloguing</qex> with my wife.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloid</ent><br/
<hw>Col"loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ko`lla</grk> glue + <ets>-oid</ets>. Cf. <er>Collodion</er>.]</ety> <def>Resembling glue or jelly; characterized by a jellylike appearance; gelatinous; <as>as, <ex>colloid</ex> tumors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloid</ent><br/
<hw>Col"loid</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Physiol. Chem.)</fld> <def>A substance (as albumin, gum, gelatin, etc.) which is of a gelatinous rather than a crystalline nature, and which diffuses itself through animal membranes or vegetable parchment more slowly than crystalloids do; -- opposed to <xex>crystalloid</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A gelatinous substance found in colloid degeneration and colloid cancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Styptic colloid</b></col> <fld>(Med.)</fld>, <cd>a preparation of astringent and antiseptic substances with some colloid material, as collodion, for ready use.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloidal</ent><br/
<hw>Col*loid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or of the nature of, colloids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloidality</ent><br/
<hw>Col`loi*dal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being colloidal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collop</ent><br/
<hw>Col"lop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of uncertain origin; cf. OF. <ets>colp</ets> blow, stroke, piece, F. <ets>coup</ets>, fr. L. <ets>colophus</ets> buffet, cuff, Gr. <?/]</ety> <altsp>[Written also <asp>colp</asp>.]</altsp> <sn>1.</sn> <def>A small slice of meat; a piece of flesh.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God knows thou art a <qex>collop</qex> of my flesh.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sweetbread and <qex>collops</qex> were with skewers pricked.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A part or piece of anything; a portion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cut two good <qex>collops</qex> out of the crown land.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloped</ent><br/
<hw>Col"loped</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having ridges or bunches of flesh, like collops.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With that red, gaunt, and <qex>colloped</qex> neck astrain.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collophore</ent><br/
<hw>Col"lo*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`lla</grk> glue + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A suckerlike organ at the base of the abdomen of insects belonging to the Collembola.</def> <sd>(b)</sd> <def>An adhesive marginal organ of the Lucernariae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloquial</ent><br/
<hw>Col*lo"qui*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Colloqui</er>.]</ety> <def>Pertaining to, or used in, conversation, esp. common and familiar conversation; conversational; hence, unstudied; informal; <as>as, <ex>colloquial</ex> intercourse; <ex>colloquial</ex> phrases; a <ex>colloquial</ex> style</as>.</def> -- <wordforms><wf>Col*lo"qui*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>His [Johnson's] <qex>colloquial</qex> talents were, indeed, of the highest order.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloquialism</ent><br/
<hw>Col*lo"qui*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A colloquial expression, not employed in formal discourse or writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloquialize</ent><br/
<hw>Col*lo"qui*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make colloquial and familiar; <as>as, to <ex>colloquialize</ex> one's style of writing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloquist</ent><br/
<hw>Col"lo*quist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A speaker in a colloquy or dialogue.</def>  <rj><au>Malone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colloquy</ent><br/
<hw>Col"lo*quy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Colloquies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>colloquium</ets>. See <er>Collocution</er>.]</ety> <sn>1.</sn> <def>Mutual discourse of two or more persons; conference; conversation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They went to Worms, to the <qex>colloquy</qex> there about religion.</q> <rj><qau>A. Wood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In some American colleges, a part in exhibitions, assigned for a certain scholarship rank; a designation of rank in collegiate scholarship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collotype</ent><br/
<hw>Col"lo*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ glue + <ets>-type</ets>.]</ety> <def>A photomechanical print made directly from a hardened film of gelatin or other colloid; also, the process of making such prints. According to one method, the film is sensitized with potassium dichromate and exposed to light under a reversed negative. After the dichromate has been washed out, the film is soaked in glycerin and water. As this treatment causes swelling in those parts of the film which have been acted on by light, a plate results from which impressions can be taken with prepared ink. The albertype, phototype, and heliotype are collotypes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Collow</ent><br/
<hw>Col"low</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Soot; smut. See 1st <er>Colly</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colluctancy</ent><br/
<hw>Col*luc"tan*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colluctari</ets> to struggle with.]</ety> <def>A struggling to resist; a striving against; resistance; opposition of nature.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colluctation</ent><br/
<hw>Col`luc*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colluctatio</ets>, fr. <ets>colluctari</ets> to struggle with; <ets>col-</ets> + <ets>luctari</ets> to struggle.]</ety> <def>A struggling; a contention.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Colluctation</qex> with old hags and hobgoblins.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collude</ent><br/
<hw>Col*lude"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Colluded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Colluding</conjf>.]</vmorph> <ety>[L. <ets>colludere</ets>, <ets>-lusum</ets>; <ets>col-</ets> + <ets>ludere</ets> to play. See <er>Ludicrous</er>.]</ety> <def>To have secretly a joint part or share in an action; to play into each other's hands; to conspire; to act in concert.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If they let things take their course, they will be represented as <qex>colluding</qex> with sedition.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colluder</ent><br/
<hw>Col*lud"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conspires in a fraud.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collum</ent><br/
||<hw>Col"lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Colla</plw> <pr>(#)</pr>.</plu> <ety>[L., neck.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Anat.)</fld> <def>A neck or cervix.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Same as <er>Collar</er>.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collusion</ent><br/
<hw>Col*lu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>collusio</ets>: cf. F. <ets>collusion</ets>. See <er>Collude</er>.]</ety> <sn>1.</sn> <def>A secret agreement and cooperation for a fraudulent or deceitful purpose; a playing into each other's hands; deceit; fraud; cunning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The foxe, maister of <qex>collusion</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That they [miracles] be done publicly, in the face of the world, that there may be no room to suspect artifice and <qex>collusion</qex>.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By the ignorance of the merchants or dishonesty of the weavers, or the <qex>collusion</qex> of both, the ware was bad and the price excessive.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>An agreement between two or more persons to defraud a person of his rights, by the forms of law, or to obtain an object forbidden by law.</def>  <rj><au>Bouvier. Abbott.</au></rj></p>

<p><syn><b>Syn.</b> -- <er>Collusion</er>, <er>Connivance</er>.</syn> <usage> A person who is guilty of <xex>connivance</xex> intentionally overlooks, and thus sanctions what he was bound to prevent. A person who is guilty of <xex>collusion</xex> unites with others (playing into their hands) for fraudulent purposes.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collusive</ent><br/
<hw>Col*lu"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Characterized by collusion; done or planned in collusion.</def> <ldquo/<xex>Collusive</xex> and sophistical arguings.<rdquo/ <au>J. Trapp.</au>  <ldquo/<xex>Collusive</xex> divorces.<rdquo/ <au>Strype.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Acting in collusion.</def> <ldquo/<xex>Collusive parties</xex>.<rdquo/ <au>Burke.</au></p>

<p>-- <wordforms><wf>Col*lu"sive*ly</wf>, <pos>adv.</pos> -- <wf>Col*lu"sive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collusory</ent><br/
<hw>Col*lu"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>collusorius</ets>.]</ety> <def>Collusive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collutory</ent><br/
<hw>Col"lu*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colluere</ets>, <ets>collutum</ets>, to wash.]</ety> <fld>(Med.)</fld> <def>A medicated wash for the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colluvies</ent><br/
||<hw>Col*lu"vi*es</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a collection of washings, dregs, offscourings, fr. <ets>colluere</ets> to wash; <ets>col-</ets> + <ets>luere</ets> to wash.]</ety> <sn>1.</sn> <def>A collection or gathering, as of pus, or rubbish, or odds and ends.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A medley; offscourings or rabble.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Colly</ent><br/
<hw>Col"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Coal</er>.]</ety> <def>The black grime or soot of coal.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj></p>

<p><ent>Colly</ent><br/

<hw>Col"ly</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Collied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Collying</conjf>.]</vmorph> <def>To render black or dark, as of with coal smut; to begrime.</def> <mark>[Archaic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast not <qex>collied</qex> thy face enough.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Brief as the lighting in the <qex>collied</qex> night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colly</ent><br/
<hw>Col"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of dog. See <er>Collie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collybist</ent><br/
<hw>Col"ly*bist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ a small coin.]</ety> <def>A money changer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In the face of these guilty <qex>collybists</qex>.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Collyrium</ent><br/
<hw>Col*lyr"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Collyriums</plw> <pr>(#)</pr>, L. <plw>Collyria</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/.]</ety> <fld>(Med.)</fld> <def>An application to the eye, usually an eyewater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coloboma</ent><br/
||<hw>Col`o*bo"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. fr. Gr. <?/, the part taken away in mutilation, fr. <?/ to mutilate.]</ety> <fld>(Anat. & Med.)</fld> <def>A defect or malformation; esp., a fissure of the iris supposed to be a persistent embryonic cleft.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Colocasia</ent><br/
<hw>Colocasia</hw> <pos>n.</pos> <def>a small genus of perennial tuberous herbs of the arum family, of tropical Asia and the Pacific islands, including the <stype>taro</stype> (<spn>Colocasia esculente</spn>).</def><br/
<syn><b>Syn. --</b> genus <gen>Colocasia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colocolo</ent><br/
||<hw>Col`o*co"lo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A South American wild cat (<spn>Felis colocolo</spn>), of the size of the ocelot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colocynth</ent><br/
<hw>Col"ocynth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>colocynthis</ets>, Gr. <?/. Cf. <er>Coloquintida</er>.]</ety> <fld>(Med.)</fld> <def>The light spongy pulp of the fruit of the bitter cucumber (<spn>Citrullus colocynthis</spn>, <it>or</it>  <spn>Cucumis colocynthis</spn>), an Asiatic plant allied to the watermelon; coloquintida. It comes in white balls, is intensely bitter, and a powerful cathartic. Called also <altname>bitter apple</altname>, <altname>bitter cucumber</altname>, <altname>bitter gourd</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colocynthin</ent><br/
<hw>Col`o*cyn"thin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>colocynthine</ets>.]</ety> <fld>(Chem.)</fld> <def>The active medicinal principle of colocynth; a bitter, yellow, crystalline substance, regarded as a glucoside.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cologne</ent><br/
<hw>Co*logne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Originally made in <etsep>Cologne</etsep>, the French name of K<oum/ln, a city in Germany.]</ety> <def>A perfumed liquid, composed of alcohol and certain aromatic oils, used in the toilet; -- called also <altname>cologne water</altname> and <altname>eau de cologne</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cologne earth</ent><br/
<hw>Co*logne" earth`</hw> <pr>(?)</pr>. <ety>[From <ets>Cologne</ets> the city.]</ety> <fld>(Min.)</fld> <def>An earth of a deep brown color, containing more vegetable than mineral matter; an earthy variety of lignite, or brown coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colombier</ent><br/
<hw>Col"om*bier</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A large size of paper for drawings. See under <er>Paper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colombin</ent><br/
<hw>Co*lom"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See <er>Calumbin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colombo</ent><br/
<hw>Colombo</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The (official) capital <isa>city</isa> of <country>Sri Lanka</country>.  Population (2000) = 1,994,000.</def><br/
[<source>PJC</source>]</p>

<p><ent>Colombo</ent><br/
<hw>Co*lom"bo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Calumba</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colon</ent><br/
<hw>Co"lon</hw> <pr>(k<omac/"l<ocr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>colon</ets>, <ets>colum</ets>, limb, member, the largest of the intestines, fr. Gr. <grk>kw^lon</grk>, and in sense of the intestine, <grk>ko`lon</grk>: cf. F. <ets>colon</ets>. Cf. <er>Colic</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>That part of the large intestines which extends from the c<ae/cum to the rectum. <note>[See <xex>Illust.</xex> of <er>Digestion</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>A point or character, formed thus [:], used to separate parts of a sentence that are complete in themselves and nearly independent, often taking the place of a conjunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonel</ent><br/
<hw>Colo"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>colonel</ets>, It. <ets>colonello</ets>, prop., the chief or commander of a column, fr. <ets>colonna</ets> column, L. <ets>columna</ets>. See <er>Column</er>.]</ety> <fld>(Mil.)</fld> <def>The chief officer of a regiment; an officer ranking next above a lieutenant colonel and next below a brigadier general.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonelcy</ent><br/
<hw>Colo"nel*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>The office, rank, or commission of a colonel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonelship</ent><br/
<hw>Colo"nel*ship</hw>, <pos>n.</pos> <def>Colonelcy.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coloner</ent><br/
<hw>Col"o*ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A colonist.</def> <mark>[Obs.]</mark>  <rj><au>Holland</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonial</ent><br/
<hw>Co*lo"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>colonial</ets>.]</ety> <def>Of or pertaining to a colony; <as>as, <ex>colonial</ex> rights, traffic, wars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonialism</ent><br/
<hw>Co*lo"ni*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state or quality of, or the relationship involved in, being colonial.</def></p>

<p><q>The last tie of <qex>colonialism</qex> which bound us to the mother country is broken.</q>  <rj><qau>Brander Matthews.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A custom, idea, feature of government, or the like, characteristic of a colony.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>The colonial system of political government or extension of territory, by which one nation exerts political control over another nation, territory, or people, maintaining the colony in a state of dependence, its inhabitants not having the same full rights as those of the colonial power.  The controlling power is typically extended thus by military force or the threat of force.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>4.</sn>  <def>the political or ideological system of beliefs advocating or justifying colonial control of one nation over another nation, territory, or people.</def><br/
[<source>PJC</source>]</p>

<p><ent>colonialist</ent><br/
<hw>colonialist</hw> <pos>n.</pos> <def>a believer in or advocate of colonialism{3}.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>colonic</ent><br/
<hw>colonic</hw> <pos>adj.</pos> <def>of or pertaining to the colon.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colonical</ent><br/
<hw>Co*lon"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>colonus</ets> husbandman.]</ety> <def>Of or pertaining to husbandmen.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonist</ent><br/
<hw>Col"o*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member or inhabitant of a colony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonitis</ent><br/
||<hw>Col`o*ni"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Colitis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonization</ent><br/
<hw>Col`o*ni*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>colonisation</ets>.]</ety> <def>The act of colonizing, or the state of being colonized; the formation of a colony or colonies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wide continent of America invited <qex>colonization</qex>.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonizationist</ent><br/
<hw>Col`o*ni*za"tion*ist</hw>, <pos>n.</pos> <def>A friend to colonization, esp. <mark>(U. S. Hist)</mark> to the colonization of Africa by emigrants from the colored population of the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonize</ent><br/
<hw>Col"o*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Colonized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Colonizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>coloniser</ets>.]</ety> <def>To plant or establish a colony or colonies in; to people with colonists; to migrate to and settle in.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They that would thus <qex>colonize</qex> the stars with inhabitants.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonize</ent><br/
<hw>Col"o*nize</hw>, <pos>v. i.</pos> <def>To remove to, and settle in, a distant country; to make a colony.</def>  <rj><au>C. Buchanan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>colonized</ent><br/
<hw>colonized</hw> <pos>adj.</pos> <sn>1.</sn> <def>inhabited by people who were born in or retain strong ties to another country.</def><br/
<syn><b>Syn. --</b> settled.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>politically ruled by citizens of another country.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn>  <def>having a population (of animals or plants) which moved in from another territory.</def><br/
<syn><b>Syn. --</b> colonized.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Colonizer</ent><br/
<hw>Col"o*ni`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who promotes or establishes a colony; a colonist.</def>  <rj><au>Bancroft.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colonnade</ent><br/
<hw>Col`on*nade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>colonnade</ets>, It. <ets>colonnata</ets>, fr. <ets>colonna</ets> column. See <er>Colonel</er>.]</ety> <fld>(Arch.)</fld> <def>A series or range of columns placed at regular intervals with all the adjuncts, as entablature, stylobate, roof, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When in front of a building, it is called a <stype>portico</stype>; when surrounding a building or an open court or square, a <stype>peristyle</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>colonnaded</ent><br/
<hw>colonnaded</hw> <pos>adj.</pos> <def>having a series of columns arranged at regular intervals; furnished with a colonnade.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colony</ent><br/
<hw>Col"o*ny</hw> <pr>(k<ocr/l"<osl/*n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Colonies</plw> <pr>(k<ocr/l"<osl/*n<icr/z)</pr>.</plu> <ety>[L. <ets>colonia</ets>, fr. <ets>colonus</ets> farmer, fr. <ets>colere</ets> to cultivate, dwell: cf. F. <ets>colonie</ets>. Cf. <er>Culture</er>.]</ety> <sn>1.</sn> <def>A company of people transplanted from their mother country to a remote province or country, and remaining subject to the jurisdiction of the parent state; <as>as, the British <ex>colonies</ex> in America</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first settlers of New England were the best of Englishmen, well educated, devout Christians, and zealous lovers of liberty. There was never a <qex>colony</qex> formed of better materials.</q> <rj><qau>Ames.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The district or country colonized; a settlement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>a territory subject to the ruling governmental authority of another country and not a part of the ruling country.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>A company of persons from the same country sojourning in a foreign city or land; <as>as, the American <ex>colony</ex> in Paris</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Nat. Hist.)</fld> <def>A number of animals or plants living or growing together, beyond their usual range.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>A cell family or group of common origin, mostly of unicellular organisms, esp. among the lower alg<ae/. They may adhere in chains or groups, or be held together by a gelatinous envelope.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn>  <fld>(Zool.)</fld> <def>A cluster or aggregation of zooids of any compound animal, as in the corals, hydroids, certain tunicates, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>8.</sn>  <fld>(Zool.)</fld> <def>A community of social insects, as ants, bees, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn> <fld>(Microbiology)</fld> <def>a group of microorganisms originating as the descendents of one individual cell, growing on a gelled growth medium, as of gelatin or agar; especially, such a group that has grown to a sufficient number to be visible to the naked eye.</def><br/
[<source>PJC</source>]</p>

<p><ent>Colony counter</ent><br/
<hw>Col"o*ny count`er</hw> <pr>(k<ocr/l"<osl/*n<ycr/ kount"<etil/r)</pr>, <pos>n.</pos> <fld>(Microbiology)</fld> <def>an instrument designed to conveniently count or assist counting colonies{9} of microorganisms on a plate containing a gelled growth medium.  One variety uses a pencil-like rod with a metal tip, which is connected by an electrical connection to the gelled growth medium; when touched to a colony{9} on the plate, the completion of the  electrical circuit causes an increment of 1 unit on the readout of the colony counter.</def><br/
[<source>PJC</source>]</p>

<p><ent>Colophany</ent><br/
<hw>Col"o*pha`ny</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>See <er>Colophony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colophene</ent><br/
<hw>Co"lo*phene</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A colorless, oily liquid, formerly obtained by distillation of colophony. It is regarded as a polymeric form of terebenthene. Called also <altname>diterebene</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 280 --></p>

<p><ent>Colophon</ent><br/
<hw>Col"o*phon</hw> <pr>(k<ocr/l"<osl/*f<ocr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>colophon</ets> finishing stroke, Gr. <grk>kolofw`n</grk>; cf. L. <ets>culmen</ets> top, <ets>collis</ets> hill. Cf. <er>Holm</er>.]</ety> <def>An inscription, monogram, or cipher, containing the place and date of publication, printer's name, etc., formerly placed on the last page of a book.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>colophon</qex>, or final description, fell into disuse, and . . . the title page had become the principal direct means of identifying the book.</q> <rj><qau>De Morgan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The book was uninjured from title page to <qex>colophon</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colophonite</ent><br/
<hw>Col"o*pho*nite</hw> <pr>(k<ocr/l"<osl/*f<osl/*n<imac/t <it>or</it> k<osl/*l<ocr/f"<osl/*n<imac/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>colophonite</ets>. So named from its resemblance to the color of <ets>colophony</ets>.]</ety> <fld>(Min.)</fld> <def>A coarsely granular variety of garnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colophony</ent><br/
<hw>Col"o*pho`ny</hw> <pr>(k<ocr/l"<osl/*f<omac/*n<ycr/ <it>or</it> k<osl/*l<ocr/f"<osl/*n<ycr/; 277)</pr>, <pos>n.</pos> <ety>[Gr. <grk>'h kolofwni`a</grk> (sc. <grk>"rhti`nh</grk> resin, gum) resin, fr. <grk>Kolofw`nios</grk> of or from Colophon in Ionia.]</ety> <def>Rosin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coloquintida</ent><br/
<hw>Col`o*quin"ti*da</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Colocynth</er>.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Color</ent><br/
<hw>Col"or</hw> <pr>(k<ucr/l"<etil/r)</pr>, <pos>n.</pos> <altsp>[Written also <asp>colour</asp>.]</altsp> <ety>[OF. <ets>color</ets>, <ets>colur</ets>, <ets>colour</ets>, F. <ets>couleur</ets>, L. <ets>color</ets>; prob. akin to <ets>celare</ets> to conceal (the color taken as that which covers). See <er>Helmet</er>.]</ety> <sn>1.</sn> <def>A property depending on the relations of light to the eye, by which individual and specific differences in the hues and tints of objects are apprehended in vision; <as>as, gay <ex>colors</ex>; sad <ex>colors</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The sensation of <xex>color</xex> depends upon a peculiar function of the retina or optic nerve, in consequence of which rays of light produce different effects according to the length of their waves or undulations, waves of a certain length producing the sensation of red, shorter waves green, and those still shorter blue, etc. White, or ordinary, light consists of waves of various lengths so blended as to produce no effect of color, and the <xex>color</xex> of objects depends upon their power to absorb or reflect a greater or less proportion of the rays which fall upon them.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any hue distinguished from white or black.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The hue or color characteristic of good health and spirits; ruddy complexion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give <qex>color</qex> to my pale cheek.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is used to give color; a paint; a pigment; <as>as, oil <ex>colors</ex> or water <ex>colors</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which covers or hides the real character of anything; semblance; excuse; disguise; appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They had let down the boat into the sea, under <qex>color</qex> as though they would have cast anchors out of the foreship.</q> <rj><qau>Acts xxvii. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That he should die is worthy policy;<br/
But yet we want a <qex>color</qex> for his death.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Shade or variety of character; kind; species.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Boys and women are for the most part cattle of this <qex>color</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A distinguishing badge, as a flag or similar symbol (usually in the plural); <as>as, the <ex>colors</ex> or <ex>color</ex> of a ship or regiment; the <ex>colors</ex> of a race horse (that is, of the cap and jacket worn by the jockey)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the United States each regiment of infantry and artillery has two <qex>colors</qex>, one national and one regimental.</q> <rj><qau>Farrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Law)</fld> <def>An apparent right; as where the defendant in trespass gave to the plaintiff an appearance of title, by stating his title specially, thus removing the cause from the jury to the court.</def> <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Color</xex> is <xex>express</xex> when it is averred in the pleading, and <xex>implied</xex> when it is implied in the pleading.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Body color</b></col>. <cd>See under <er>Body</er>.</cd> -- <col><b>Color blindness</b></col>, <cd>total or partial inability to distinguish or recognize colors. See <er>Daltonism</er>.</cd> -- <col><b>Complementary color</b></col>, <cd>one of two colors so related to each other that when blended together they produce white light; -- so called because each color makes up to the other what it lacks to make it white.  Artificial or pigment colors, when mixed, produce effects differing from those of the primary colors, in consequence of partial absorption.</cd> -- <col><b>Of color</b></col> (as persons, races, etc.), <cd>not of the white race; -- commonly meaning, esp. in the United States, of negro blood, pure or mixed.</cd> -- <col><b>Primary colors</b></col>, <cd>those developed from the solar beam by the prism, viz., red, orange, yellow, green, blue, indigo, and violet, which are reduced by some authors to three, -- red, green, and violet-blue.  These three are sometimes called <altname>fundamental colors</altname>.</cd> -- <mcol><col><b>Subjective color</b></col> <it>or</it> <col><b>Accidental color</b></col></mcol>, <cd>a false or spurious color seen in some instances, owing to the persistence of the luminous impression upon the retina, and a gradual change of its character, as where a wheel perfectly white, and with a circumference regularly subdivided, is made to revolve rapidly over a dark object, the teeth of the wheel appear to the eye of different shades of color varying with the rapidity of rotation. See <cref>Accidental colors</cref>, under <er>Accidental</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Color</ent><br/
<hw>Col"or</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Colored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coloring</conjf>.]</vmorph> <ety>[F. <ets>colorer</ets>.]</ety> <sn>1.</sn> <def>To change or alter the hue or tint of, by dyeing, staining, painting, etc.; to dye; to tinge; to paint; to stain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rays, to speak properly, are not <qex>colored</qex>; in them there is nothing else than a certain power and disposition to stir up a sensation of this or that color.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change or alter, as if by dyeing or painting; to give a false appearance to; usually, to give a specious appearance to; to cause to appear attractive; to make plausible; to palliate or excuse; <as>as, the facts were <ex>colored</ex> by his prejudices</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>colors</qex> the falsehood of <AE/neas by an express command from Jupiter to forsake the queen.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That by his fellowship he <qex>color</qex> might<br/
Both his estate and love from skill of any wight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Color</ent><br/
<hw>Col"or</hw>, <pos>v. i.</pos> <def>To acquire color; to turn red, especially in the face; to blush.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorable</ent><br/
<hw>Col"or*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Specious; plausible; having an appearance of right or justice.</def> <ldquo/<xex>Colorable</xex> pretense for infidelity.<rdquo/ <rj><au>Bp. Stillingfleet.</au></rj></p>

<p>-- <wordforms><wf>Col"or*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Col"or*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Colorable</qex> and subtle crimes, that seldom are taken within the walk of human justice.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coloradan</ent><br/
<hw>Coloradan</hw> <pos>n.</pos> <def>a resident of the state of Colorado.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coloradillo</ent><br/
<hw>coloradillo</hw> <pos>n.</pos> <def>a handsome shrub (<spn>Hamelia patens</spn>) with showy orange to scarlet or crimson flowers; it grows from Florida and West Indies to Mexico and Brazil.</def><br/
<syn><b>Syn. --</b> scarlet bush, scarlet hamelia, <spn>Hamelia patens</spn>, Hamelia erecta.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colorado</ent><br/
<hw>Col`o*ra"do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Sp., red.]</ety> <sn>1.</sn>  <def>Reddish; -- often used in proper names of rivers or creeks.</def> <mark>[Southwestern U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Medium in color and strength; -- said of cigars.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Colorado beetle</ent><br/
<hw>Col`o*ra"do bee"tle</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A yellowish beetle (<spn>Doryphora decemlineata</spn>), with ten longitudinal, black, dorsal stripes. It has migrated eastwards from its original habitat in Colorado, and is very destructive to the potato plant; -- called also <altname>potato beetle</altname> and <altname>potato bug</altname>. See <er>Potato beetle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorado group</ent><br/
<hw>Col`o*ra"do group</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>A subdivision of the cretaceous formation of western North America, especially developed in Colorado and the upper Missouri region.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coloradoite</ent><br/
<hw>Col`o*ra"do*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Mercury telluride, an iron-black metallic mineral, found in Colorado.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorate</ent><br/
<hw>Col"or*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>coloratus</ets>, p. p. of <ets>colorare</ets> to color.]</ety> <def>Colored.</def> <mark>[Obs.]</mark> <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coloration</ent><br/
<hw>Col`or*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or art of coloring; the state of being colored.</def> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The females . . . resemble each other in their general type of <qex>coloration</qex>.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorature</ent><br/
<hw>Col"or*a*ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>coloratur</ets>, fr. LL. <ets>coloratura</ets>.]</ety> <fld>(Mus.)</fld> <def>Vocal music <xex>colored</xex>, as it were, by florid ornaments, runs, or rapid passages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Color-blind</ent><br/
<hw>Col"or-blind</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected with color blindness. See <cref>Color blindness</cref>, under <er>Color</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colored</ent><br/
<hw>Col"ored</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having color; tinged; dyed; painted; stained.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lime rod, <qex>colored</qex> as the glede.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>colored</qex> rainbow arched wide.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specious; plausible; adorned so as to appear well; <as>as, a highly <ex>colored</ex> description</as>.</def> <rj><au>Sir G. C. Lewis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>colored</qex> crime with craft to cloke.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of some other color than black or white.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Ethnol.)</fld> <def>Of some other color than white; having a skin color darker than that of caucasian people; mostly applied to negroes or persons having negro blood; <as>as, a <ex>colored</ex> man; the <ex>colored</ex> people</as>.  Opposite of <ant>white</ant> and <ant>caucasian</ant>.</def><br/
<syn><b>Syn. --</b> coloured, dark-skinned.</syn><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <def>Of some other color than green.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Colored</qex>, meaning, as applied to foliage, of some other color than green.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In botany, green is not regarded as a color, but white is.</note> <rj><au>Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>colorful</ent><br/
<hw>colorful</hw> <pos>adj.</pos> <sn>1.</sn> <def>having striking color.  Opposite of <ant>colorless</ant>.</def> <note>[Narrower terms: <stype>changeable, chatoyant, iridescent, shot</stype>; <stype>deep, rich</stype>; <stype>flaming</stype>;  <stype>fluorescent, glowing</stype>; <stype>prismatic</stype>; <stype>psychedelic</stype>; <stype>red, ruddy, flushed, empurpled</stype>]</note><br/
<syn><b>Syn. --</b> colourful.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>striking in variety and interest.  Opposite of <ant>colorless</ant> or <ant>dull</ant>.</def> [Narrower terms: <stype>brave, fine, gay, glorious</stype>; <stype>flamboyant, resplendent, unrestrained</stype>; <stype>flashy, gaudy, jazzy, showy, snazzy, sporty</stype>; <stype>picturesque</stype>]<br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>having color or a certain color; not black, white or grey; <as>as, <ex>colored</ex> crepe paper</as>.  Opposite of <ant>colorless</ant> and <ant>monochrome</ant>.</def> <note>[Narrower terms: <stype>tinted</stype>;  <stype>touched, tinged</stype>;  <stype>amber, brownish-yellow, yellow-brown</stype>;  <stype>amethyst</stype>;  <stype>auburn, reddish-brown</stype>;  <stype>aureate, gilded, gilt, gold, golden</stype>;  <stype>azure, cerulean, sky-blue, bright blue</stype>;  <stype>bicolor, bicolour, bicolored, bicoloured, bichrome</stype>;  <stype>blue, bluish, light-blue, dark-blue</stype>;  <stype>blushful, blush-colored, rosy</stype>;  <stype>bottle-green</stype>;  <stype>bronze, bronzy</stype>;  <stype>brown, brownish, dark-brown</stype>;  <stype>buff</stype>;  <stype>canary, canary-yellow</stype>;  <stype>caramel, caramel brown</stype>;  <stype>carnation</stype>;  <stype>chartreuse</stype>;  <stype>chestnut</stype>;  <stype>dun</stype>;  <stype>earth-colored, earthlike</stype>;  <stype>fuscous</stype>;  <stype>green, greenish, light-green, dark-green</stype>;  <stype>jade, jade-green</stype>;  <stype>khaki</stype>;  <stype>lavender, lilac</stype>;  <stype>mauve</stype>;  <stype>moss green, mosstone</stype>;  <stype>motley, multicolor, culticolour, multicolored, multicoloured, painted, particolored, particoloured, piebald, pied, varicolored, varicoloured</stype>;  <stype>mousy, mouse-colored</stype>;  <stype>ocher, ochre</stype>;  <stype>olive-brown</stype>;  <stype>olive-drab</stype>;  <stype>olive</stype>;  <stype>orange, orangish</stype>;  <stype>peacock-blue</stype>;  <stype>pink, pinkish</stype>;  <stype>purple, violet, purplish</stype>;  <stype>red, blood-red, carmine, cerise, cherry, cherry-red, crimson, ruby, ruby-red, scarlet</stype>;  <stype>red, reddish</stype>;  <stype>rose, roseate</stype>;  <stype>rose-red</stype>;  <stype>rust, rusty, rust-colored</stype>;  <stype>snuff, snuff-brown, snuff-color, snuff-colour, snuff-colored, snuff-coloured, mummy-brown, chukker-brown</stype>;  <stype>sorrel, brownish-orange</stype>;  <stype>stone, stone-gray</stype>;  <stype>straw-color, straw-colored, straw-coloured</stype>;  <stype>tan</stype>;  <stype>tangerine</stype>;  <stype>tawny</stype>;  <stype>ultramarine</stype>;  <stype>umber</stype>;  <stype>vermilion, vermillion, cinibar, Chinese-red</stype>;  <stype>yellow, yellowish</stype>;  <stype>yellow-green</stype>;  <stype>avocado</stype>;  <stype>bay</stype>;  <stype>beige</stype>;  <stype>blae  bluish-black or gray-blue)</stype>;  <stype>coral</stype>;  <stype>creamy</stype>;  <stype>cress green, cresson, watercress</stype>;  <stype>hazel</stype>;  <stype>honey, honey-colored</stype>;  <stype>hued(postnominal)</stype>;  <stype>magenta</stype>;  <stype>maroon</stype>;  <stype>pea-green</stype>;  <stype>russet</stype>;  <stype>sage, sage-green</stype>;  <stype>sea-green</stype>]</note> <see>[Also See: <er>chromatic</er>, <er>colored</er>, <er>dark</er>, <er>light</er>.]</see><br/
<syn><b>Syn. --</b> colored, coloured, in color(predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colorific</ent><br/
<hw>Col`or*if"ic</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>color</ets> color + <ets>facere</ets> to make: cf. F. <ets>colorifique</ets>.]</ety> <def>Capable of communicating color or tint to other bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorimeter</ent><br/
<hw>Col`or*im"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Color</ets> + <ets>-meter</ets>: cf. F. <ets>colorim<egrave/tre</ets>.]</ety> <def>An instrument for measuring the depth of the color of anything, especially of a liquid, by comparison with a standard liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorimetry</ent><br/
<hw>Col`or*im"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Colorimeter</er>.]</ety> <sn>1.</sn>  <def>The quantitative determination of the depth of color of a substance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A method of quantitative chemical analysis based upon the comparison of the depth of color of a solution with that of a standard liquid.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coloring</ent><br/
<hw>Col"or*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of applying color to; also, that which produces color.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Change of appearance as by addition of color; appearance; show; disguise; misrepresentation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tell the whole story without <qex>coloring</qex> or gloss.</q> <rj><qau>Compton Reade.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dead coloring</b></col>. <cd>See under <er>Dead</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorist</ent><br/
<hw>Col"or*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>coloriste</ets>.]</ety> <def>One who colors; an artist who excels in the use of colors; one to whom coloring is of prime importance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Titian, Paul Veronese, Van Dyck, and the rest of the good <qex>colorists</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colorless</ent><br/
<hw>Col"or*less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Without color; not distinguished by any hue; transparent; <as>as, <ex>colorless</ex> water</as>; a <ex>colorless</ex> gas.</def>  <note>[Narrower terms: <stype>ashen, bloodless, livid, lurid, pale, pallid, pasty, wan, waxen</stype>; <stype>neutral</stype>; <stype>white</stype>]</note> <see>[Also See: <er>achromatic</er>, <er>colorless</er>.]</see><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>Free from any manifestation of partial or peculiar sentiment or feeling; not disclosing likes, dislikes, prejudice, etc.; <as>as, <ex>colorless</ex> music; a <ex>colorless</ex> style; definitions should be <ex>colorless</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>having lost its normal color.</def> <note>[Narrower terms: <stype>blanched, etiolate, etiolated, whitened</stype>; <stype>bleached, faded, washed-out, washy</stype>; <stype>dimmed, dulled, grayed</stype>; <stype>dirty</stype>; <stype>dull, sober, somber, subfusc</stype>]</note> <ant>colored</ant><br/
<syn><b>Syn. --</b> colorless, uncolored, uncoloured.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Colorman</ent><br/
<hw>Col"or*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Colormen</plw> <pr>(#)</pr>.</plu> <def>A vender of paints, etc.</def> <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>colors</ent><br/
<hw>colors</hw> <pos>n.</pos> <sn>1.</sn> <def>a flag flown by a ship to show its nationality.</def><br/
<syn><b>Syn. --</b> ensign, colours.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a distinguishing emblem; <as>as, his tie proclaimed his school colors</as>.</def><br/
<syn><b>Syn. --</b> colours.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Color sergeant</ent><br/
<hw>Col"or ser"geant</hw>. <def>See under <er>Sergeant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colossal</ent><br/
<hw>Co*los"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>colossal</ets>, L. <ets>colosseus</ets>. See <er>Colossus</er>.]</ety> <sn>1.</sn> <def>Of enormous size; gigantic; huge; <as>as, a <ex>colossal</ex> statue</as>.</def> <ldquo/A <xex>colossal</xex> stride.<rdquo/ <rj><au>Motley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Sculpture & Painting)</fld> <def>Of a size larger than heroic. See <er>Heroic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colossean</ent><br/
<hw>Col`os*se"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Colossal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colosseum</ent><br/
<hw>Col`os*se"um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Neut., fr. L. <ets>colosseus</ets> gigantic. See <er>Coliseum</er>.]</ety> <def>The amphitheater of Vespasian in Rome.</def> <altsp>[Also written <asp>Coliseum</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colossus</ent><br/
<hw>Co*los"sus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Colossi</plw> <pr>(#)</pr>, E. <plw>Colossuses</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/.]</ety> <sn>1.</sn> <def>A statue of gigantic size. The name was especially applied to certain famous statues in antiquity, as the <xex>Colossus</xex> of Nero in Rome, the <xex>Colossus</xex> of Apollo at Rhodes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He doth bestride the narrow world<br/
Like a <qex>colossus</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There is no authority for the statement that the legs of the Colossus at Rhodes extended over the mouth of the harbor.</note> <rj><au>Dr. Wm. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any man or beast of gigantic size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colostrum</ent><br/
||<hw>Co*los"trum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., biestings.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>The first milk secreted after delivery; biestings.</def> <sd>(b)</sd> <def>A mixture of turpentine and the yolk of an egg, formerly used as an emulsion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colotomy</ent><br/
<hw>Co*lot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`lon</grk> colon + <grk>tomh`</grk> cutting.]</ety> <fld>(Surg.)</fld> <def>An operation for opening the colon</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colour</ent><br/
<hw>Col"our</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Color</er>.</def> <mark>[Brit.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>coloured</ent><br/
<hw>coloured</hw> <pos>adj.</pos> <def>same as <er>colored</er>.</def> <mark>[Brit.]</mark><br/
<syn><b>Syn. --</b> colored, in color(predicate), colorful.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>colourful</ent><br/
<hw>colourful</hw> <pos>adj.</pos>  <def>same as <er>colorful</er>.</def> <mark>[Brit.]</mark><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>colouring</ent><br/
<hw>colouring</hw> <pos>n.</pos> <def>same as <er>coloring</er>.</def> <mark>[Brit.]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>colours</ent><br/
<hw>colours</hw> <pos>n.</pos> <def>same as <er>colors</er>.</def> <mark>[Brit.]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colp</ent><br/
<hw>Colp</hw> <pr>(k<ocr/kp)</pr>, <pos>n.</pos> <def>See <er>Collop</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colportage</ent><br/
<hw>Col"por`tage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>The distribution of religious books, tracts, etc., by colporteurs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colporter</ent><br/
<hw>Col"por`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Colporteur</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colporteur</ent><br/
<hw>Col"por`teur</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>colporteur</ets> one who carries on his neck, fr. <ets>colporter</ets> to carry on one's neck; <ets>col</ets> (L. <ets>collum</ets>) neck + <ets>porter</ets> (L. <ets>portare</ets>) to carry.]</ety> <def>A hawker; specifically, one who travels about selling and distributing religious tracts and books.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colstaff</ent><br/
<hw>Col"staff`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>col</ets> neck + E. <ets>staff</ets>. Cf. <er>Coll</er>.]</ety> <def>A staff by means of which a burden is borne by two persons on their shoulders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colt</ent><br/
<hw>Colt</hw> <pr>(k<omac/lt; 110)</pr>, <pos>n.</pos> <ety>[OE. <ets>colt</ets> a young horse, ass, or camel, AS. <ets>colt</ets>; cf. dial. Sw. <ets>kullt</ets> a boy, lad.]</ety> <sn>1.</sn> <def>The young of the equine genus or horse kind of animals; -- sometimes distinctively applied to the male, <contr>filly</contr> being the female. Cf. <er>Foal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In sporting circles it is usual to reckon the age of colts from some arbitrary date, as from January 1, or May 1, next preceding the birth of the animal.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young, foolish fellow.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A short knotted rope formerly used as an instrument of punishment in the navy.</def> <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Colt's tooth</b></col>, <cd>an imperfect or superfluous tooth in young horses.</cd> -- <col><b>To cast one's colt's tooth</b></col>, <cd>to cease from youthful wantonness.</cd> <ldquo/Your <xex>colt's tooth</xex> is not cast yet.<rdquo/ <au>Shak.</au> -- <col><b>To have a colt's tooth</b></col>, <cd>to be wanton.</cd> <rj><au>Chaucer.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colt</ent><br/
<hw>Colt</hw> <pr>(k<omac/lt; 110)</pr>, <pos>v. i.</pos> <def>To frisk or frolic like a colt; to act licentiously or wantonly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They shook off their bridles and began to <qex>colt</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colt</ent><br/
<hw>Colt</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To horse; to get with young.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To befool.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colter</ent><br/
<hw>Col"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>culter</ets>, fr. L. <ets>culter</ets> plowshare, knife. Cf. <er>Cutlass</er>.]</ety> <def>A knife or cutter, attached to the beam of a plow to cut the sward, in advance of the plowshare and moldboard.</def> <altsp>[Written also <asp>coulter</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coltish</ent><br/
<hw>Colt"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a colt; wanton; frisky.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was all <qex>coltish</qex>, full of ragery.</q> <rj><qau>Chaucer.</qau></rj></p>

<p>-- <wordforms><wf>Colt"ish*ly</wf>, <pos>adv.</pos> -- <wf>Colt"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colt pistol</ent><br/
<hw>Colt pistol</hw>. <fld>(Firearms)</fld> <def>A self-loading or semi-automatic pistol with removable magazine in the handle holding seven cartridges. The recoil extracts and ejects the empty cartridge case, and reloads ready for another shot. Called also <altname>Browning pistol</altname>, <altname>Colt-Browning pistol</altname>, <altname>automatic pistol</altname> and <altname>automatic</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Colt revolver</ent><br/
<hw>Colt revolver</hw>. <fld>(Firearms)</fld> <def>A revolver made according to a system using a patented revolving cylinder, holding six cartridges, patented by <person>Samuel <etsep>Colt</etsep></person>, an American inventor, in 1835. With various modifications, it was for many years been the standard for the United States army.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coltsfoot</ent><br/
<hw>Colts"foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A perennial herb (<spn>Tussilago Farfara</spn>), whose leaves and rootstock are sometimes employed in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Butterbur coltsfoot</b></col> <fld>(Bot.)</fld>, <cd>a European plant (<spn>Petasites vulgaris</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colt's tooth</ent><br/
<hw>Colt's" tooth`</hw> <pr>(?)</pr>. <def>See under <er>Colt</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coluber</ent><br/
||<hw>Col"u*ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a serpent.]</ety> <fld>(Zool.)</fld> <def>A genus of harmless serpents.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Linn<ae/us placed in this genus all serpents, whether venomous or not, whose scales beneath the tail are arranged in pairs; but by modern writers it is greatly restricted.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>colubrid</ent><br/
<hw>colubrid</hw> <pos>n.</pos> <def>any member of a large family (<fam>Colubridae</fam>) of mostly harmless temperate-to-tropical terrestrial or arboreal or aquatic snakes.</def><br/
<syn><b>Syn. --</b> colubrid snake.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colubridae</ent><br/
<hw>Colubridae</hw> <pos>n.</pos> <def>a broad family including only nonvenomous snakes, containing about two-thirds of all living species.  It includes the bullsnakes, garter snakes, and water snakes as well as many other species.</def><br/
<syn><b>Syn. --</b> family <fam>Colubridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colubrina</ent><br/
<hw>Colubrina</hw> <pos>n.</pos> <def>a genus of mostly tropical American shrubs or small trees with small yellowish flowers and yellow or red fruits.</def><br/
<syn><b>Syn. --</b> genus <gen>Colubrina</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colubrine</ent><br/
<hw>Col"u*brine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>colubrinus</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>like or related to snakes of the genus <gen>Coluber</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a snake; cunning; crafty.</def> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colugo</ent><br/
||<hw>Co*lu"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. an aboriginal name.]</ety> <fld>(Zool.)</fld> <def>A peculiar East Indian mammal (<spn>Galleopithecus volans</spn>), having along the sides, connecting the fore and hind limbs, a parachutelike membrane, by means of which it is able to make long leaps, like the flying squirrel; -- called also <altname>flying lemur</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columba</ent><br/
<hw>Co*lum"ba</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Calumba</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbae</ent><br/
||<hw>Co*lum"b<ae/</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <ety>[L. <ets>columba</ets> pigeon.]</ety> <fld>(Zool.)</fld> <def>An order of birds, including the pigeons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbarium</ent><br/
||<hw>Col`um*ba"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Columbaria</plw> <pr>(#)</pr></plu> <ety>[L. See <er>Columbary</er>.]</ety> <fld>(Rom. Antiq.)</fld> <sd>(a)</sd> <def>A dovecote or pigeon house.</def> <sd>(b)</sd> <def>A sepulchral chamber with niches for holding cinerary urns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbary</ent><br/
<hw>Col"um*ba*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Columbaries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>columbarium</ets>, fr. <ets>columba</ets> a dove.]</ety> <def>A dovecote; a pigeon house.</def> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbate</ent><br/
<hw>Co*lum"bate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>colombate</ets>. See <er>Columbium</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of columbic acid; a niobate. See <er>Columbium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbatz fly</ent><br/
<hw>Co*lum"batz fly`</hw> <pr>(?)</pr>. <ety>[From <ets>Kolumbatz</ets>, a mountain in Germany.]</ety> <fld>(Zool.)</fld> <def>See <cref>Buffalo fly</cref>, under <er>Buffalo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbella</ent><br/
||<hw>Col`um*bel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., dim. of L. <ets>columba</ets> a dove. So called from a fancied resemblance in color and form, of some species.]</ety> <fld>(Zool.)</fld> <def>A genus of univalve shells, abundant in tropical seas. Some species, as <spn>Columbella mercatoria</spn>, were formerly used as shell money.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbia</ent><br/
<hw>Co*lum"bi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <def>America; the United States; -- a poetical appellation given in honor of <etsep>Columbus</etsep>, the discoverer.</def> <rj><au>Dr. T. Dwight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbiad</ent><br/
<hw>Co*lum"bi*ad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Columbia</ets> the United States.]</ety> <fld>(Mil.)</fld> <def>A form of seacoast cannon; a long, chambered gun designed for throwing shot or shells with heavy charges of powder, at high angles of elevation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Since the War of 1812 the <ex>Columbiad</ex> has been much modified, especially by <person>General Rodman</person>, and the improved form now used in seacoast defense is often called the <stype>Rodman gun</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbian</ent><br/
<hw>Co*lum"bi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Columbia</er>.]</ety> <def>Of or pertaining to the United States, or to America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbic</ent><br/
<hw>Co*lum"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Columbium</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or containing, columbium or niobium; niobic.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Columbic acid</b></col> <fld>(Chem.)</fld>, <cd>a weak acid derived from columbic or niobic oxide, <chform>Nb2O5</chform>; -- called also <altname>niobic acid</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbic</ent><br/
<hw>Co*lum"bic</hw>, <pos>a.</pos> <ety>[From <er>Columbo</er>.]</ety> <def>Pertaining to, or derived from, the columbo root.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Columbic acid</b></col> <fld>(Chem.)</fld>, <cd>an organic acid extracted from the columbo root as a bitter, yellow, amorphous substance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbier</ent><br/
<hw>Co*lum"bi*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Colombier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbiferous</ent><br/
<hw>Col"um*bif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Columbium</ets> + <ets>-ferous</ets>.]</ety> <def>Producing or containing columbium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbin</ent><br/
<hw>Co*lum"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white, crystalline, bitter substance. See <er>Calumbin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbine</ent><br/
<hw>Col"um*bine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>columbinus</ets>, fr. <ets>columba</ets> dove.]</ety> <def>Of or pertaining to a dove; dovelike; dove-colored.</def> <ldquo/<xex>Columbine</xex> innocency.<rdquo/ <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbine</ent><br/
<hw>Col"um*bine</hw>, <pos>n.</pos> <ety>[<ets>LL</ets>. <ets>columbina</ets>, L. <ets>columbinus</ets> dovelike, fr. <ets>columba</ets> dove: cf. F. <ets>colombine</ets>. Perh. so called from the beaklike spurs of its flowers.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A plant of several species of the genus <gen>Aquilegia</gen>; <as>as, <spn>Aquilegia vulgaris</spn>, or the common garden columbine; <spn>Aquilegia Canadensis</spn>, the wild red columbine of North America</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The mistress or sweetheart of Harlequin in pantomimes.</def> <rj><au>Brewer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbite</ent><br/
<hw>Co*lum"bite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>colombite</ets>. See <er>Columbium</er>.]</ety> <fld>(Min.)</fld> <def>A mineral of a black color, submetallic luster, and high specific specific gravity. It is a niobate (or columbate) of iron and manganese, containing tantalate of iron; -- first found in New England.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 281 --></p>

<p><ent>Columbium</ent><br/
<hw>Co*lum"bi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. <ets>Columbia</ets> America.]</ety> <fld>(Chem.)</fld> <def>A rare element of the vanadium group, first found in a variety of the mineral columbite occurring in Connecticut, probably at Haddam. Atomic weight 94.2. Symbol Cb or Nb. Now more commonly called <altname>niobium</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbo</ent><br/
<hw>Co*lum"bo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Calumba</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columbus Day</ent><br/
<hw>Co*lum"bus Day</hw> <pr>(?)</pr>. <def>The 12th day of October, on which day in 1492 Christopher Columbus discovered America, landing on one of the Bahama Islands (probably the one now commonly called Watling Island), and naming it <ldquo/San Salvador<rdquo/; -- called also <altname>Discovery Day</altname>. This day is made a legal holiday in many States of The United States.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Columella</ent><br/
<hw>Col`u*mel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., dim. of <ets>columen</ets> column. See <er>Column</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>An axis to which a carpel of a compound pistil may be attached, as in the case of the geranium; or which is left when a pod opens.</def> <sd>(b)</sd><def>A columnlike axis in the capsules of mosses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A term applied to various columnlike parts; <as>as, the <ex>columella</ex>, or epipterygoid bone, in the skull of many lizards; the <ex>columella</ex> of the ear, the bony or cartilaginous rod connecting the tympanic membrane with the internal ear</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The upright pillar in the axis of most univalve shells.</def> <sd>(b)</sd> <def>The central pillar or axis of the calicles of certain corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columelliform</ent><br/
<hw>Col`u*mel"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Columella</ets> + <ets>-form</ets>.]</ety> <def>Shaped like a little column, or columella.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Column</ent><br/
<hw>Col"umn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>columna</ets>, fr. <ets>columen</ets>, <ets>culmen</ets>, fr. <ets>cellere</ets> (used only in comp.), akin to E. <ets>excel</ets>, and prob. to <ets>holm</ets>. <ets>See</ets> <er>Holm</er>, and cf. <er>Colonel</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A kind of pillar; a cylindrical or polygonal support for a roof, ceiling, statue, etc., somewhat ornamented, and usually composed of base, shaft, and capital. See <er>Order</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Illustration of a Column. --></p>

<p><sn>2.</sn> <def>Anything resembling, in form or position, a column in architecture; an upright body or mass; a shaft or obelisk; <as>as, a <ex>column</ex> of air, of water, of mercury, etc.; the <ex>Column</ex> Vend<ocir/me; the spinal <ex>column</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A body of troops formed in ranks, one behind the other; -- contradistinguished from <contr>line</contr>. Compare <er>Ploy</er>, and <er>Deploy</er>.</def> <sd>(b)</sd> <def>A small army.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A number of ships so arranged as to follow one another in single or double file or in squadrons; -- in distinction from <ldquo/line<rdquo/, where they are side by side.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <def>A perpendicular set of lines, not extending across the page, and separated from other matter by a rule or blank space; <as>as, a <ex>column</ex> in a newspaper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Arith.)</fld> <def>A perpendicular line of figures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Bot.)</fld> <def>The body formed by the union of the stamens in the Mallow family, or of the stamens and pistil in the orchids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Print.)</fld> <def>one of a series of articles written in a periodical, usually under the same title and at regular intervals; it may be written and signed by one or more authors, or may appear pseudonymously or anonymously, as an editorial column.</def> <ldquo/<person>Safire's</person> weekly <ex>column</ex> <title>On Language</title> in the <title>New York Times</title> is usually more interesting (and probably more accurate) than his political <ex>column</ex>.<rdquo/ <au>P. Cassidy</au><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Attached column</b></col>. <cd>See under <er>Attach</er>, <pos>v. t.</pos></cd> -- <col><b>Clustered column</b></col>. <cd>See under <er>Cluster</er>, <pos>v. t.</pos></cd> -- <col><b>Column rule</b></col>, <cd>a thin strip of brass separating columns of type in the form, and making a line between them in printing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columnar</ent><br/
<hw>Co*lum"nar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>columnaris</ets>, fr. <ets>columna</ets>.]</ety> <def>Formed in columns; having the form of a column or columns; like the shaft of a column.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Columnar epithelium</b></col> <fld>(Anat.)</fld>, <cd>epithelium in which the cells are prismatic in form, and set upright on the surface they cover.</cd> -- <col><b>Columnar structure</b></col> <fld>(Geol.)</fld>, <cd>a structure consisting of more or less regular columns, usually six-sided, but sometimes with eight or more sides. The columns are often fractured transversely, with a cup joint, showing a concave surface above. This structure is characteristic of certain igneous rocks, as basalt, and is due to contraction in cooling.<-- as at Giant's Causeway, Ireland --></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columnarity</ent><br/
<hw>Col`um*nar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being columnar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columnated</ent><br/
<hw>Co*lum"na*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having columns; <as>as, <ex>columnated</ex> temples</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columned</ent><br/
<hw>Col"umned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having columns.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Troas and Ilion's <qex>columned</qex> citadel.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Columniation</ent><br/
<hw>Co*lum`ni*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The employment or arrangement of columns in a structure.</def> <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>columnist</ent><br/
<hw>col"um*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a journalist who writes or edits a regularly scheduled column{8} in a periodical, usually in editorial style; a type of <isa>editorialist</isa>.</def><br/
[<source>RP</source>]</p>

<p><cs><col><b>fifth columnist</b></col> <cd>see <er>fifth columnist</er> in the vocabulary.</cd></cs></p>

<p><ent>Colure</ent><br/
<hw>Co*lure"</hw> <pr>(k<osl/*l<umac/r")</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Colures</plw> <pr>(k<osl/*l<umac/rz")</pr>.</plu> <ety>[F. <ets>colure</ets>, L. <ets>coluri</ets>, pl., fr. Gr. <grk>ko`loyros</grk> dock-tailed, <grk>a"i ko`loyroi</grk> (sc. <grk>grammai`</grk> lines) the colures; fr. <grk>ko`los</grk> docked, stunted + <grk>o'yra`</grk> tail. So named because a part is always beneath the horizon.]</ety> <fld>(Astron. & Geog.)</fld> <def>One of two great circles intersecting at right angles in the poles of the equator. One of them passes through the equinoctial points, and hence is denominated the <xex>equinoctial</xex> colure; the other intersects the equator at the distance of 90<deg/ from the former, and is called the <xex>solstitial</xex> colure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thrice the equinoctial line<br/
He circled; four times crossed the car of night<br/
From pole to pole, traversing each <qex>colure</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colutea</ent><br/
<hw>Colutea</hw> <pos>n.</pos> <def>small genus of Eurasian shrubs with yellow flowers and bladdery pods.</def><br/
<syn><b>Syn. --</b> genus <gen>Colutea</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coly</ent><br/
<hw>Co"ly</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Colies</plw> <pr>(#)</pr>.</plu> <ety>[NL. <ets>colius</ets>, prob. fr. Gr. <?/ a kind of woodpecker.]</ety> <def>Any bird of the genus <gen>Colius</gen> and allied genera. They inhabit Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Colymbiformes</ent><br/
<hw>Colymbiformes</hw> <pos>n.</pos> <sn>1.</sn> <def>an order of birds, including the family <fam>Podicipedidae</fam>, which consitutes the grebes.</def><br/
<syn><b>Syn. --</b> Podicipitiformes, order <ord>Podicipitiformes</ord>, Podicipediformes, order <ord>Podicipediformes</ord>, order <ord>Colymbiformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Colza</ent><br/
<hw>Col"za</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. D. <ets>koolzaad</ets>, prop., cabbage seed; <ets>kool</ets> (akin to E. <ets>cole</ets>) + <ets>zaad</ets>, akin to E. <ets>seed</ets>.]</ety> <fld>(Bot.)</fld> <def>A variety of cabbage (<spn>Brassica oleracea</spn>), cultivated for its seeds, which yield an oil valued for illuminating and lubricating purposes; summer rape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Com-</ent><br/
<hw>Com-</hw>. <def>A prefix from the Latin preposition <xex>cum</xex>, signifying <xex>with</xex>, <xex>together</xex>, <xex>in conjunction</xex>, <xex>very</xex>, etc. It is used in the form <xex>com-</xex> before <xex>b</xex>, <xex>m</xex>, <xex>p</xex>, and sometimes <xex>f</xex>, and by assimilation becomes <xex>col-</xex> before <xex>l</xex>, <xex>cor-</xex> before <xex>r</xex>, and <xex>con-</xex> before any consonant except <xex>b</xex>, <xex>h</xex>, <xex>l</xex>, <xex>m</xex>, <xex>p</xex>, <xex>r</xex>, and <xex>w</xex>. Before a vowel <xex>com-</xex> becomes <xex>co-</xex>; also before <xex>h</xex>, <xex>w</xex>, and sometimes before other consonants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coma</ent><br/
||<hw>Co"ma</hw> <pr>(k<omac/"m<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kw^ma</grk> lethargy, fr. <grk>koima^n</grk> to put to sleep. See <er>Cemetery</er>.]</ety> <def>A state of profound insensibility from which it is difficult or impossible to rouse a person. See <er>Carus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coma</ent><br/
||<hw>Co"ma</hw>, <pos>n.</pos> <ety>[L., hair, fr. Gr. <grk>ko`mh</grk>.]</ety> <sn>1.</sn> <fld>(Astron.)</fld> <def>The envelope of a comet; a nebulous covering, which surrounds the nucleus or body of a comet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A tuft or bunch, -- as the assemblage of branches forming the head of a tree; or a cluster of bracts when empty and terminating the inflorescence of a plant; or a tuft of long hairs on certain seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coma Berenices</b></col> <pr>(<?/)</pr> <ety>[L.]</ety> <fld>(Astron.)</fld>, <cd>a small constellation north of Virgo; -- called also <altname>Berenice's Hair</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comanches</ent><br/
<hw>Co*man"ches</hw> <pr>(? <or/ ?)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Comanche</singw> <pr>(? <or/ ?)</pr></sing>. <fld>(Ethnol.)</fld> <def>A warlike, savage, and nomadic tribe of the Shoshone family of Indians, inhabiting Mexico and the adjacent parts of the United States; -- called also <altname>Paducahs</altname>. They are noted for plundering and cruelty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comandra</ent><br/
<hw>Comandra</hw> <pos>n.</pos> <def>small genus of chiefly North American parasitic plants.</def><br/
<syn><b>Syn. --</b> genus <gen>Comandra</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comart</ent><br/
<hw>Co"mart`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A covenant.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comate</ent><br/
<hw>Co"mate</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>comatus</ets>, fr. <ets>comare</ets> to clothe with hair, fr. <ets>coma</ets> hair.]</ety> <def>Encompassed with a coma, or bushy appearance, like hair; hairy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-mate</ent><br/
<hw>Co"-mate`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>co-</ets> + <ets>mate</ets>.]</ety> <def>A companion.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comatose</ent><br/
<hw>Co"ma*tose`</hw> <pr>(? <or/ ?; 277)</pr>, <pos>a.</pos> <ety>[From <er>Coma</er> lethargy.]</ety> <def>Relating to, or resembling, coma; drowsy; lethargic; <as>as, <ex>comatose</ex> sleep; <ex>comatose</ex> fever</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comatous</ent><br/
<hw>Co"ma*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Comatose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comatula</ent><br/
||<hw>Co*mat"u*la</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>comatulus</ets> having hair neatly curled, dim. fr. <ets>coma</ets> hair.]</ety> <fld>(Zool.)</fld> <def>A crinoid of the genus <gen>Antedon</gen> and related genera. When young they are fixed by a stem. When adult they become detached and cling to seaweeds, etc., by their dorsal cirri; -- called also <altname>feather stars</altname>.</def><-- original had <altname>feather stars</altname> But the headword is singular and its singular altname is feather star, which has its own entry. So we should remove the "s". PCP --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comatulid</ent><br/
<hw>Co*mat"u*lid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any crinoid of the genus <gen>Antedon</gen> or allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comb</ent><br/
<hw>Comb</hw> <pr>(k<omac/m; 110)</pr>, <pos>n.</pos> <ety>[AS. <ets>camb</ets>; akin to Sw., Dan., & D. <ets>kam</ets>, Icel. <ets>kambr</ets>, G. <ets>kamm</ets>, Gr. <?/ a grinder tooth, Skr. <ets>jambha</ets> tooth.]</ety> <sn>1.</sn> <def>An instrument with teeth, for straightening, cleansing, and adjusting the hair, or for keeping it in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for currying hairy animals, or cleansing and smoothing their coats; a currycomb.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Manuf. & Mech.)</fld> <sd>(a)</sd> <def>A toothed instrument used for separating and cleansing wool, flax, hair, etc.</def> <sd>(b)</sd> <def>The serrated vibratory doffing knife of a carding machine.</def> <sd>(c)</sd> <def>A former, commonly cone-shaped, used in hat manufacturing for hardening the soft fiber into a bat.</def> <sd>(d)</sd> <def>A tool with teeth, used for chasing screws on work in a lathe; a chaser.</def> <sd>(e)</sd> <def>The notched scale of a wire micrometer.</def> <sd>(f)</sd> <def>The collector of an electrical machine, usually resembling a comb.</def> <-- "former" in (c) is a noun. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The naked fleshy crest or caruncle on the upper part of the bill or hood of a cock or other bird. It is usually red.</def> <sd>(b)</sd> <def>One of a pair of peculiar organs on the base of the abdomen of scorpions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The curling crest of a wave.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The waxen framework forming the walls of the cells in which bees store their honey, eggs, etc.; honeycomb.</def> <ldquo/A <xex>comb</xex> of honey.<rdquo/ <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When the bee doth leave her <qex>comb</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The thumbpiece of the hammer of a gunlock, by which it may be cocked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comb</ent><br/
<hw>Comb</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Combed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Combing</conjf>.]</vmorph> <def>To disentangle, cleanse, or adjust, with a comb; to lay smooth and straight with, or as with, a comb; <as>as, to <ex>comb</ex> hair or wool</as>. See under <er>Combing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Comb</qex> down his hair; look, look! it stands upright.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comb</ent><br/
<hw>Comb</hw>, <pos>v. i.</pos> <ety>[See <er>Comb</er>, <pos>n.</pos>, 5.]</ety> <fld>(Naut.)</fld> <def>To roll over, as the top or crest of a wave; to break with a white foam, as waves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combe</ent><br/
<ent>Comb</ent><br/
<mhw>{ <hw>Comb</hw>, <hw>Combe</hw> <pr>(? <or/ ?)</pr>, }</mhw> <pos>n.</pos> <ety>[AS. <ets>comb</ets>, prob. of Celtic origin; cf. W. <ets>cwm</ets> a dale, valley.]</ety> <def>That unwatered portion of a valley which forms its continuation beyond and above the most elevated spring that issues into it.</def> <altsp>[Written also <asp>coombe</asp>.]</altsp> <rj><au>Buckland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A gradual rise the shelving <qex>combe</qex><br/
Displayed.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comb</ent><br/
<hw>Comb</hw>, <pos>n.</pos> <def>A dry measure. See <er>Coomb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combat</ent><br/
<hw>Com"bat</hw> <pr>(? <or/ ?; 277)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Combated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Combating</conjf>.]</vmorph> <ety>[F. <ets>combattre</ets>; pref. <ets>com-</ets> + <ets>battre</ets> to beat, fr. L. <ets>battuere</ets> to strike. See <er>Batter</er>.]</ety> <def>To struggle or contend, as with an opposing force; to fight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>combat</qex> with a blind man I disdain.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>After the fall of the republic, the Romans <qex>combated</qex> only for the choice of masters.</q> <rj><qau>Gibbon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combat</ent><br/
<hw>Com"bat</hw>, <pos>v. t.</pos> <def>To fight with; to oppose by force, argument, etc.; to contend against; to resist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When he the ambitious Norway <qex>combated</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>combated</qex> in silence all these reasons.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Minds <qex>combat</qex> minds, repelling and repelled.</q> <rj><qau>Goldsmith.</qau></rj></p>

<p><syn><b>Syn.</b> -- To fight against; resist; oppose; withstand; oppugn; antagonize; repel; resent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combat</ent><br/
<hw>Com"bat</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>combat</ets>.]</ety> <sn>1.</sn> <def>A fight; a contest of violence; a struggle for supremacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My courage try by <qex>combat</qex>, if thou dar'st.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The noble <qex>combat</qex> that 'twixt joy and sorrow was fought in Paulina.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>An engagement of no great magnitude; or one in which the parties engaged are not armies.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Single combat</b></col>, <cd>one in which a single combatant meets a single opponent, as in the case of David and Goliath; also, a duel.</cd></cs></p>

<p><syn><b>Syn.</b> -- A battle; engagement; conflict; contest; contention; struggle; fight, strife. See <er>Battle</er>, <er>Contest</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combatable</ent><br/
<hw>Com"bat*a*ble</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>combattable</ets>.]</ety> <def>Such as can be, or is liable to be, combated; <as>as, <ex>combatable</ex> foes, evils, or arguments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combatant</ent><br/
<hw>Com"bat*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>combattant</ets>, p. pr.]</ety> <def>Contending; disposed to contend.</def> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combatant</ent><br/
<hw>Com"bat*ant</hw>, <pos>n.</pos> <ety>[F. <ets>combattant</ets>.]</ety> <def>One who engages in combat.  IN military use, opposed to <contr>noncombatant</contr>.</def> <ldquo/The mighty <xex>combatants</xex>.<rdquo/ <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A controversy which long survived the original <qex>combatants</qex>.</q> <rj><qau>Macaulay</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combater</ent><br/
<hw>Com"bat*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who combats.</def> <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combative</ent><br/
<hw>Com"bat*ive</hw> <pr>(? or <?/)</pr>, <pos>a.</pos> <def>Disposed to engage in combat; pugnacious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combativeness</ent><br/
<hw>Com"bat*ive*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality of being combative; propensity to contend or to quarrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phren.)</fld> <def>A cranial development supposed to indicate a combative disposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combattant</ent><br/
||<hw>Com`bat`tant"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <fld>(Her.)</fld> <def>In the position of fighting; -- said of two lions set face to face, each rampant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combbroach</ent><br/
<hw>Comb"broach`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tooth of a wool comb.</def> <altsp>[Written also <asp>combrouch</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combe</ent><br/
<hw>Combe</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>See <er>Comb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comber</ent><br/
<hw>Comb"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who combs; one whose occupation it is to comb wool, flax, etc. Also, a machine for combing wool, flax, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A long, curling wave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comber</ent><br/
<hw>Com"ber</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cumber.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comber</ent><br/
<hw>Com"ber</hw>, <pos>n.</pos> <def>Encumbrance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comber</ent><br/
<hw>Com"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The cabrilla. Also, a name applied to a species of wrasse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combinable</ent><br/
<hw>Com*bin"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>combinable</ets>.]</ety> <def>Capable of combining; consistent with.</def> <mark>[R.]</mark> <rj><au>M. Arnold.</au></rj></p>

<p>-- <wordforms><wf>Com*bin"a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combinate</ent><br/
<hw>Com"bi*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>combinatus</ets>, p. p.]</ety> <def>United; joined; betrothed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 282 --></p>

<p><ent>Combination</ent><br/
<hw>Com`bi*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>combinatio</ets>. See <er>Combine</er>.]</ety> <sn>1.</sn> <def>The act or process of combining or uniting persons and things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Making new compounds by new <qex>combinations</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A solemn <qex>combination</qex> shall be made<br/
Of our dear souls.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The result of combining or uniting; union of persons or things; esp. a union or alliance of persons or states to effect some purpose; -- usually in a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>combination</qex> of the most powerful men in Rome who had conspired my ruin.</q> <rj><qau>Melmoth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>The act or process of uniting by chemical affinity, by which substances unite with each other in definite proportions by weight to form distinct compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <fld>(Math.)</fld> <def>The different arrangements of a number of objects, as letters, into groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In <xex>combinations</xex> no regard is paid to the order in which the objects are arranged in each group, while in <xex>variations</xex> and <xex>permutations</xex> this order is respected.</note> <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Combination car</b></col>, <cd>a railroad car containing two or more compartments used for different purposes.</cd> <mark>[U. S.]</mark> -- <col><b>Combination lock</b></col>, <cd>a lock in which the mechanism is controlled by means of a movable dial (sometimes by several dials or rings) inscribed with letters or other characters. The bolt of the lock can not be operated until after the dial has been so turned as to combine the characters in a certain order or succession.</cd> -- <col><b>Combination room</b></col>, <cd>in the University of Cambridge, Eng., a room into which the fellows withdraw after dinner, for wine, dessert, and conversation.</cd> -- <col><b>Combination by volume</b></col> <fld>(Chem.)</fld>, <cd>the act, process, or ratio by which gaseous elements and compounds unite in definite proportions by volume to form distinct compounds.</cd> -- <col><b>Combination by weight</b></col> <fld>(Chem.)</fld>, <cd>the act, process, or ratio, in which substances unite in proportions by weight, relatively fixed and exact, to form distinct compounds. See <cref>Law of definite proportions</cref>, under <er>Definite</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Cabal; alliance; association; league; union; confederacy; coalition; conspiracy. See <er>Cabal</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>combinational</ent><br/
<hw>combinational</hw> <pos>adj.</pos>  <def>of or relating to combinations.</def><br/
<syn><b>Syn. --</b> combinable, combinatory, combining(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>combinative</ent><br/
<hw>combinative</hw> <pos>adj.</pos> <sn>1.</sn> <def>produced by a combinatorial process.</def> [Narrower terms: <stype>combinable, combinational, combinatory, combining(prenominal)</stype>] <see>Also See: <er>integrative</er>.</see>  Antonym: <ant>noncombinative</ant>.<br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>able to combine; tending to combine.</def> [Narrower terms: <stype>integrative (vs. disintegrative)</stype>]<br/
<syn><b>Syn. --</b> combinatory, combinatorial.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>combinatory</ent><br/
<ent>combinatorial</ent><br/
<mhw><hw>combinatorial</hw> <hw>combinatory</hw></mhw> <pos>adj.</pos> <sn>1.</sn>  <def>able to combine; tending to combine.</def> <note>same as <er>combinative</er>, 2.</note> [Narrower terms: <stype>integrative (vs. disintegrative)</stype>]<br/
<syn><b>Syn. --</b> combinative.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>of or relating to combinations.</def> [Narrower terms: <stype>combinative (vs. noncombinative)</stype>] <source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>produced by a process of combining; <as>as, a <ex>combinatorial</ex> explosion of possibilities</as>; -- used especially in reference to mathematical or statistical processes of computing possible combinations.</def><br/
<source>PJC</source>]</p>

<p><ent>combinatory</ent><br/
<hw>combinatory</hw> <pos>adj.</pos> <sn>1.</sn>  <def>same as <er>combinatorial</er>, 1.</def><br/
<syn><b>Syn. --</b> combinative, combinatorial.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>able to be combined.</def><br/
<syn><b>Syn. --</b> combinable, combinational, combining(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>same as <er>combinatorial</er>, 2.</def> [Narrower terms: <stype>combinative (vs. noncombinative)</stype>]<br/
<syn><b>Syn. --</b> combinatorial.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Combine</ent><br/
<hw>Com*bine"</hw> <pr>(k<ocr/m*b<imac/n")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Combined</conjf> <pr>(k<ocr/m*b<imac/nd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Combining</conjf>.]</vmorph> <ety>[LL. <ets>combinare</ets>, <ets>combinatum</ets>; L. <ets>com-</ets> + <ets>binus</ets>, pl. <ets>bini</ets>, two and two, double: cf. F. <ets>combiner</ets>. See <er>Binary</er>.]</ety> <sn>1.</sn> <def>To unite or join; to link closely together; to bring into harmonious union; to cause or unite so as to form a homogeneous substance, as by chemical union.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So fitly them in pairs thou hast <qex>combined</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Friendship is the cement which really <qex>combines</qex> mankind.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And all <qex>combined</qex>, save what thou must <qex>combine</qex><br/
By holy marriage.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Earthly sounds, though sweet and well <qex>combined</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bind; to hold by a moral tie.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>combined</qex> by a sacred vow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combine</ent><br/
<hw>Com*bine"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To form a union; to agree; to coalesce; to confederate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You with your foes <qex>combine</qex>,<br/
And seem your own destruction to design</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So sweet did harp and voice <qex>combine</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite by affinity or natural attraction; <as>as, two substances, which will not <ex>combine</ex> of themselves, may be made to <ex>combine</ex> by the intervention of a third</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Card Playing)</fld> <def>In the game of casino, to play a card which will take two or more cards whose aggregate number of pips equals those of the card played.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Combining weight</b></col> <fld>(Chem.)</fld>, <cd>that proportional weight, usually referred to hydrogen as a standard, and for each element fixed and exact, by which an element unites with another to form a distinct compound. The combining weights either are identical with, or are multiples or submultiples of, the atomic weight. See <cref>Atomic weight</cref>, under <er>Atomic</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combined</ent><br/
<hw>Com*bined"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>United closely; confederated; chemically united.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combinedly</ent><br/
<hw>Com*bin"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In combination or cooperation; <xex>jointly</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combiner</ent><br/
<hw>Com*bin"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, combines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combing</ent><br/
<hw>Comb"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of using a comb or a number of combs; <as>as, the <ex>combing</ex> of one's hair; the <ex>combing</ex> of wool</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The process of <xex>combing</xex> is used in straightening wool of long staple; short wool is <xex>carded</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <sd>(a)</sd> <def>That which is caught or collected with a comb, as loose, tangled hair.</def> <sd>(b)</sd> <def>Hair arranged to be worn on the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The baldness, thinness, and . . . deformity of their hair is supplied by borders and <qex>combings</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj></p>

<p><sd>(c)</sd> <fld>(Naut.)</fld> <def>See <er>Coamings</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Combing machine</b></col> <fld>(Textile Manuf.)</fld>, <cd>a machine for combing wool, flax, cotton, etc., and separating the longer and more valuable fiber from the shorter. See also <cref>Carding machine</cref>, under <er>Carding</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combless</ent><br/
<hw>Comb"less</hw>, <pos>a.</pos> <def>Without a comb or crest; <as>as, a <ex>combless</ex> cock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>combo</ent><br/
<hw>combo</hw> <pos>n.</pos> <def>a small band of jazz musicians.</def><br/
<syn><b>Syn. --</b> jazz band, jazz group.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comboloio</ent><br/
||<hw>Com`bo*lo"io</hw> <pr>(k<ocr/m`b<osl/*l<omac/"y<osl/)</pr>, <pos>n.</pos> <def>A Mohammedan rosary, consisting of ninety-nine beads.</def> <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>combretum</ent><br/
<hw>combretum</hw> <pos>n.</pos> <def>any of numerous shrubs or small trees of the genus <gen>Combretum</gen> having spikes of small flowers.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comb-shaped</ent><br/
<hw>Comb"-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Pectinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>comburent</ent><br/
<ent>comburant</ent><br/
<mhw><hw>comburant</hw> <hw>comburent</hw></mhw> <pos>adj.</pos>  <def>supporting combustion.</def><br/
<syn><b>Syn. --</b> combustive.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Combust</ent><br/
<hw>Com*bust"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>combustus</ets>, p. p. of <ets>comburere</ets> to burn up; <ets>com-</ets> + <ets>burere</ets> (only in comp.), of uncertain origin; cf. <ets>bustum</ets> funeral pyre, <ets>prurire</ets> to itch, <ets>pruna</ets> a live coal, Gr. <grk>pyrso`s</grk> firebrand, Skr. <ets>plush</ets> to burn.]</ety> <sn>1.</sn> <def>Burnt; consumed.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>So near the sun as to be obscured or eclipsed by his light, as the moon or planets when not more than eight degrees and a half from the sun.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Planets that are oft <qex>combust</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combustibility</ent><br/
<hw>Com*bus`ti*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being combustible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combustible</ent><br/
<hw>Com*bus"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>combustible</ets>.]</ety> <sn>1.</sn> <def>Capable of taking fire and burning; apt to catch fire; inflammable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sin is to the soul like fire to <qex>combustible</qex> matter.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Easily kindled or excited; quick; fiery; irascible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Arnold was a <qex>combustible</qex> character.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combustible</ent><br/
<hw>Com*bus"ti*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A substance that may be set on fire, or which is liable to take fire and burn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All such <qex>combustibles</qex> as are cheap enough for common use go under the name of fuel.</q> <rj><qau>Ure.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combustibleness</ent><br/
<hw>Com*bus"ti*ble*ness</hw>, <pos>n.</pos> <def>Combustibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combustion</ent><br/
<hw>Com*bus"tion</hw> <pr>(?; 106)</pr>, <pos>n.</pos> <ety>[L. <ets>combustio</ets>: cf. F. <ets>combustion</ets>.]</ety> <sn>1.</sn> <def>The state of burning.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>The combination of a combustible with a supporter of combustion, producing heat, and sometimes both light and heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Combustion</qex> results in common cases from the mutual chemical action and reaction of the combustible and the oxygen of the atmosphere, whereby a new compound is formed.</q> <rj><qau>Ure.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Supporter of combustion</b></col> <fld>(Chem.)</fld>, <cd>a gas, as oxygen, the combination of which with a combustible, as coal, constitutes combustion.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Violent agitation; confusion; tumult.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There [were] great <qex>combustions</qex> and divisions among the heads of the university.</q> <rj><qau>Mede.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But say from whence this new <qex>combustion</qex> springs.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Combustion chamber</ent><br/
<hw>Com*bus"tion cham`ber</hw>. <fld>(Mech.)</fld> <sd>(a)</sd> <def>A space over, or in front of, a boiler furnace where the gases from the fire become more thoroughly mixed and burnt.</def> <sd>(b)</sd> <def>The clearance space in the cylinder of an internal combustion engine where the charge is compressed and ignited.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Combustious</ent><br/
<hw>Com*bus"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Inflammable.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Come</ent><br/
<hw>Come</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Came</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Come</conjf> <pr>(?)</pr>; <pos>p. pr & vb. n.</pos> <conjf>Coming</conjf>.]</vmorph> <ety>[OE. <ets>cumen</ets>, <ets>comen</ets>, AS. <ets>cuman</ets>; akin to OS.<ets>kuman</ets>, D. <ets>komen</ets>, OHG. <ets>queman</ets>, G. <ets>kommen</ets>, Icel. <ets>koma</ets>, Sw. <ets>komma</ets>, Dan. <ets>komme</ets>, Goth. <ets>giman</ets>, L. <ets>venire</ets> (<ets>gvenire</ets>), Gr. <?/ to go, Skr. <ets>gam</ets>. <root/23. Cf. <er>Base</er>, <pos>n.</pos>, <er>Convene</er>, <er>Adventure</er>.]</ety> <sn>1.</sn> <def>To move hitherward; to draw near; to approach the speaker, or some place or person indicated; -- opposed to <xex>go</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Look, who <qex>comes</qex> yonder?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I did not <qex>come</qex> to curse thee.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To complete a movement toward a place; to arrive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When we <qex>came</qex> to Rome.</q> <rj><qau>Acts xxviii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lately <qex>come</qex> from Italy.</q> <rj><qau>Acts xviii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To approach or arrive, as if by a journey or from a distance.</def> <ldquo/Thy kingdom <xex>come</xex>.<rdquo/ <rj><au>Matt. vi. 10.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The hour is <qex>coming</qex>, and now is.</q> <rj><qau>John. v. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So quick bright things <qex>come</qex> to confusion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To approach or arrive, as the result of a cause, or of the act of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From whence <qex>come</qex> wars?</q> <rj><qau>James iv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Both riches and honor <qex>come</qex> of thee !</q> <rj><qau>1 Chron. xxix. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To arrive in sight; to be manifest; to appear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then butter does refuse to <qex>come</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To get to be, as the result of change or progress; -- with a predicate; <as>as, to <ex>come</ex> untied</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>come</qex> you thus estranged?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>come</qex> her eyes so bright?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Am come</xex>, <xex>is come</xex>, etc., are frequently used instead of <xex>have come</xex>, <xex>has come</xex>, etc., esp. in poetry. The verb <xex>to be</xex> gives a clearer adjectival significance to the participle as expressing a state or condition of the subject, while the auxiliary <xex>have</xex> expresses simply the completion of the action signified by the verb.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Think not that I <qex>am come</qex> to destroy.</q> <rj><qau>Matt. v. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>are come</qex> off like Romans.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The melancholy days <qex>are come</qex>, the saddest of the year.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><xex>Come</xex> may properly be used (instead of <xex>go</xex>) in speaking of a movement hence, or away, when there is reference to an approach to the person addressed; as, I shall <xex>come</xex> home next week; he will <xex>come</xex> to your house to-day. It is used with other verbs almost as an auxiliary, indicative of approach to the action or state expressed by the verb; as, how <xex>came</xex> you to do it? <xex>Come</xex> is used colloquially, with reference to a definite future time approaching, without an auxiliary; as, it will be two years, <xex>come</xex> next Christmas; <it>i. e.</it>, when Christmas shall come.<br/
[<source>1913 Webster</source>]</p>

<p><q>They were cried<br/
In meeting, <qex>come</qex> next Sunday.</q> <rj><qau>Lowell.</qau></rj></p>

<p><xex>Come</xex>, in the imperative, is used to excite attention, or to invite to motion or joint action; <xex>come</xex>, let us go. <ldquo/This is the heir; <xex>come</xex>, let us kill him.<rdquo/ <au>Matt. xxi. 38.</au> When repeated, it sometimes expresses haste, or impatience, and sometimes rebuke. <ldquo/<xex>Come</xex>, <xex>come</xex>, no time for lamentation now.<rdquo/ <rj><au>Milton.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To come</b></col>, <cd>yet to arrive, future.</cd> <ldquo/In times <xex>to come</xex>.<rdquo/ <au>Dryden.</au> <ldquo/There's pippins and cheese <xex>to come</xex>.<rdquo/ <au>Shak.</au> -- <col><b>To come about</b></col>. <sd>(a)</sd> <cd>To come to pass; to arrive; to happen; to result; as, how did these things <xex>come about</xex>?</cd> <sd>(b)</sd> <cd>To change; to come round; as, the ship <xex>comes about</xex>.</cd> <ldquo/The wind is <xex>come about</xex>.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>On better thoughts, and my urged reasons,<br/
They are <qex>come about</qex>, and won to the true side.</q> <rj><qau>B. Jonson.</qau></rj></p>

<p>-- <col><b>To come abroad</b></col>. <sd>(a)</sd> <cd>To move or be away from one's home or country.</cd> <ldquo/Am <xex>come abroad</xex> to see the world.<rdquo/ <au>Shak.</au> <sd>(b)</sd> <cd>To become public or known.</cd> <mark>[Obs.]</mark> <ldquo/Neither was anything kept secret, but that it should <xex>come abroad</xex>.<rdquo/ <au>Mark. iv. 22.</au> -- <col><b>To come across</b></col>, <cd>to meet; to find, esp. by chance or suddenly.</cd> <ldquo/We <xex>come across</xex> more than one incidental mention of those wars.<rdquo/ <au>E. A. Freeman.</au> <ldquo/Wagner's was certainly one of the strongest and most independent natures I ever <xex>came across</xex>.<rdquo/ <au>H. R. Haweis.</au> -- <col><b>To come after</b></col>. <sd>(a)</sd> <cd>To follow.</cd> <sd>(b)</sd> <cd>To come to take or to obtain; <as>as, to <ex>come after</ex> a book.</as></cd> -- <col><b>To come again</b></col>, <cd>to return</cd>. <ldquo/His spirit <xex>came again</xex> and he revived.<rdquo/ <au>Judges. xv. 19.</au> - - <col><b>To come and go</b></col>. <sd>(a)</sd> <cd>To appear and disappear; to change; to alternate.</cd> <ldquo/The color of the king doth <xex>come and go</xex>.<rdquo/ <au>Shak.</au> <sd>(b)</sd> <fld>(Mech.)</fld> <cd>To play backward and forward.</cd> -- <col><b>To come at</b></col>. <sd>(a)</sd> <cd>To reach; to arrive within reach of; to gain; <as>as, to <ex>come at</ex> a true knowledge of ourselves.</as></cd> <sd>(b)</sd> <cd>To come toward; to attack; <as>as, he <ex>came at</ex> me with fury.</as></cd> -- <col><b>To come away</b></col>, <cd>to part or depart</cd>. -- <col><b>To come between</b></col>, <cd>to intervene; to separate; hence, to cause estrangement</cd>. -- <col><b>To come by</b></col>. <sd>(a)</sd> <cd>To obtain, gain, acquire.</cd> <ldquo/Examine how you <xex>came by</xex> all your state.<rdquo/ <au>Dryden.</au> <sd>(b)</sd> <cd>To pass near or by way of.</cd> -- <col><b>To come down</b></col>. <sd>(a)</sd> <cd>To descend.</cd> <sd>(b)</sd> <cd>To be humbled.</cd> -- <col><b>To come down upon</b></col>, <cd>to call to account, to reprimand.</cd> <mark>[Colloq.]</mark> <au>Dickens.</au> -- <col><b>To come home</b></col>. <sd>(a)</sd> <cd>To return to one's house or family.</cd> <sd>(b)</sd> <cd>To come close; to press closely; to touch the feelings, interest, or reason.</cd> <sd>(c)</sd> <fld>(Naut.)</fld> <cd>To be loosened from the ground; -- said of an anchor.</cd> -- <col><b>To come in</b></col>. <sd>(a)</sd> <cd>To enter, as a town, house, etc.</cd> <ldquo/The thief <xex>cometh in</xex>.<rdquo/ <au>Hos. vii. 1.</au> <sd>(b)</sd> <cd>To arrive; <as>as, when my ship <ex>comes in</ex>.</as></cd> <sd>(c)</sd> <cd>To assume official station or duties; <as>as, when Lincoln <ex>came in</ex></as>.</cd> <sd>(d)</sd> <cd>To comply; to yield; to surrender.</cd> <ldquo/We need not fear his <xex>coming in</xex><rdquo/ <au>Massinger.</au> <sd>(e)</sd> <cd>To be brought into use.</cd> <ldquo/Silken garments did not <xex>come in</xex> till late.<rdquo/ <au>Arbuthnot.</au> <sd>(f)</sd> <cd>To be added or inserted; to be or become a part of.</cd> <sd>(g)</sd> <cd>To accrue as gain from any business or investment.</cd> <sd>(h)</sd> <cd>To mature and yield a harvest; <as>as, the crops <ex>come in</ex> well</as>.</cd> <sd>(i)</sd> <cd>To have sexual intercourse; -- with <xex>to</xex> or <xex>unto</xex>.</cd> <au>Gen. xxxviii. 16.</au> <sd>(j)</sd> <cd>To have young; to bring forth; <as>as, the cow will <ex>come in</ex> next May</as>.</cd> <mark>[U. S.]</mark> -- <col><b>To come in for</b></col>, <cd>to claim or receive.</cd> <ldquo/The rest <xex>came in for</xex> subsidies.<rdquo/ <au>Swift.</au> -- <col><b>To come into</b></col>, <cd>to join with; to take part in; to agree to; to comply with; <as>as, to <ex>come into</ex> a party or scheme</as></cd>. -- <col><b>To come it over</b></col>, <cd>to hoodwink; to get the advantage of.</cd> <mark>[Colloq.]</mark> -- <mcol><col><b>To come near</b></col> <it>or</it> <col><b>To come nigh</b></col></mcol>, <cd>to approach in place or quality; to be equal to.</cd> <ldquo/Nothing ancient or modern seems to <xex>come near</xex> it.<rdquo/ <au>Sir W. Temple.</au> -- <col><b>To come of</b></col>. <sd>(a)</sd> <cd>To descend or spring from.</cd> <ldquo/<xex>Of</xex> Priam's royal race my mother <xex>came</xex>.<rdquo/ <au>Dryden.</au> <sd>(b)</sd> <cd>To result or follow from.</cd> <ldquo/This <xex>comes of</xex> judging by the eye.<rdquo/ <au>L'Estrange.</au> -- <col><b>To come off</b></col>. <sd>(a)</sd> <cd>To depart or pass off from.</cd> <sd>(b)</sd> <cd>To get free; to get away; to escape.</cd> <sd>(c)</sd> <cd>To be carried through; to pass off; <as>as, it <ex>came off</ex> well.</as></cd> <sd>(d)</sd> <cd>To acquit one's self; to issue from (a contest, etc.); <as>as, he <ex>came off</ex> with honor;</as> hence, substantively, a <xex>come-off</xex>, an escape; an excuse; an evasion.</cd> <mark>[Colloq.]</mark> <sd>(e)</sd> <cd>To pay over; to give.</cd> <mark>[Obs.]</mark> <sd>(f)</sd> <cd>To take place; to happen; <as>as, when does the race <ex>come off</ex>?</as></cd> <sd>(g)</sd> <cd>To be or become after some delay; <as>as, the weather <ex>came off</ex> very fine</as>.</cd> <sd>(h)</sd> <cd>To slip off or be taken off, as a garment; to separate.</cd> <sd>(i)</sd> <cd>To hurry away; to get through.</cd> <au>Chaucer.</au> -- <col><b>To come off by</b></col>, <cd>to suffer</cd>. <mark>[Obs.]</mark> <ldquo/<xex>To come off by</xex> the worst.<rdquo/ <au>Calamy.</au> -- <col><b>To come off from</b></col>, <cd>to leave.</cd> <ldquo/<xex>To come off from</xex> these grave disquisitions.<rdquo/ <au>Felton.</au> -- <col><b>To come on</b></col>. <sd>(a)</sd> <cd>To advance; to make progress; to thrive.</cd> <sd>(b)</sd> <cd>To move forward; to approach; to supervene.</cd> -- <col><b>To come out</b></col>. <sd>(a)</sd> <cd>To pass out or depart, as from a country, room, company, etc.</cd> <ldquo/They shall <xex>come out</xex> with great substance.<rdquo/ <au>Gen. xv. 14.</au> <sd>(b)</sd> <cd>To become public; to appear; to be published.</cd> <ldquo/It is indeed <xex>come out</xex> at last.<rdquo/ <au>Bp. Stillingfleet.</au> <sd>(c)</sd> <cd>To end; to result; to turn out; <as>as, how will this affair <ex>come out</ex>? he has <ex>come out</ex> well at last</as>.</cd> <sd>(d)</sd> <cd>To be introduced into society; <as>as, she <ex>came out</ex> two seasons ago</as>.</cd> <sd>(e)</sd> <cd>To appear; to show itself; <as>as, the sun <ex>came out</ex></as>.</cd> <sd>(f)</sd> <cd>To take sides; to announce a position publicly; <as>as, he <ex>came out</ex> against the tariff.</as></cd>  <sd>(g)</sd> <cd>To publicly admit oneself to be homosexual.</cd> -- <col><b>To come out with</b></col>, <cd>to give publicity to; to disclose.</cd> -- <col><b>To come over</b></col>. <sd>(a)</sd> <cd>To pass from one side or place to another.</cd> <ldquo/Perpetually teasing their friends to <xex>come over</xex> to them.<rdquo/ <au>Addison.</au> <sd>(b)</sd> <cd>To rise and pass over, in distillation.</cd> -- <col><b>To come over to</b></col>, <cd>to join.</cd> -- <col><b>To come round</b></col>. <sd>(a)</sd> <cd>To recur in regular course.</cd> <sd>(b)</sd> <cd>To recover.</cd> <mark>[Colloq.]</mark> <sd>(c)</sd> <cd>To change, as the wind.</cd> <sd>(d)</sd> <cd>To relent.</cd> <au>J. H. Newman.</au> <sd>(e)</sd> <cd>To circumvent; to wheedle.</cd> <mark>[Colloq.]</mark> -- <col><b>To come short</b></col>, <cd>to be deficient; to fail of attaining.</cd> <ldquo/All have sinned and <xex>come short</xex> of the glory of God.<rdquo/ <au>Rom. iii. 23.</au> -- <col><b>To come to</b></col>. <sd>(a)</sd> <cd>To consent or yield.</cd> <au>Swift.</au> <sd>(b)</sd> <fld>(Naut.)</fld> (with the accent on <xex>to</xex>) <cd>To luff; to bring the ship's head nearer the wind; to anchor.</cd> <sd>(c)</sd> (with the accent on <xex>to</xex>) <cd>To recover, as from a swoon.</cd> <sd>(d)</sd> <cd>To arrive at; to reach.</cd> <sd>(e)</sd> <cd>To amount to; <as>as, the taxes <ex>come to</ex> a large sum</as>.</cd> <sd>(f)</sd> <cd>To fall to; to be received by, as an inheritance.</cd> <au>Shak.</au> -- <col><b>To come to blows</b></col>. <cd>See under <er>Blow</er>.</cd> -- <col><b>To come to grief</b></col>. <cd>See under <er>Grief</er>.</cd> -- <col><b>To come to a head</b></col>. <sd>(a)</sd> <cd>To suppurate, as a boil.</cd> <sd>(b)</sd> <cd>To mature; to culminate; as a plot.</cd> -- <col><b>To come to one's self</b></col>, <cd>to recover one's senses.</cd> -- <col><b>To come to pass</b></col>, <cd>to happen; to fall out.</cd> -- <col><b>To come to the scratch</b></col>. <sd>(a)</sd> <fld>(Prize Fighting)</fld> <cd>To step up to the scratch or mark made in the ring to be toed by the combatants in beginning a contest;</cd> hence: <sd>(b)</sd> <cd>To meet an antagonist or a difficulty bravely.</cd> <mark>[Colloq.]</mark> <-- I added an extra pair of cd tags to separate senses (a) and (b). PCP --> -- <col><b>To come to time</b></col>. <sd>(a)</sd> <fld>(Prize Fighting)</fld> <cd>To come forward in order to resume the contest when the interval allowed for rest is over and <ldquo/time<rdquo/ is called</cd>; hence: <sd>(b)</sd> <cd>To keep an appointment; to meet expectations.</cd> <mark>[Colloq.]</mark> -- <col><b>To come together</b></col>. <sd>(a)</sd> <cd>To meet for business, worship, etc.; to assemble.</cd> <au>Acts i. 6.</au> <sd>(b)</sd> <cd>To live together as man and wife.</cd> <au>Matt. i. 18.</au> -- <col><b>To come true</b></col>, <cd>to happen as predicted or expected.</cd> -- <col><b>To come under</b></col>, <cd>to belong to, as an individual to a class.</cd> -- <col><b>To come up</b></col> <sd>(a)</sd> <cd>to ascend; to rise.</cd> <sd>(b)</sd> <cd>To be brought up; to arise, as a question.</cd> <sd>(c)</sd> <cd>To spring; to shoot or rise above the earth, as a plant.</cd> <sd>(d)</sd> <cd>To come into use, as a fashion.</cd> -- <col><b>To come up the capstan</b></col> <fld>(Naut.)</fld>, <cd>to turn it the contrary way, so as to slacken the rope about it.</cd> -- <col><b>To come up the tackle fall</b></col> <fld>(Naut.)</fld>, <cd>to slacken the tackle gently.</cd> <au>Totten.</au> -- <col><b>To come up to</b></col>, <cd>to rise to; to equal.</cd> -- <col><b>To come up with</b></col>, <cd>to overtake or reach by pursuit.</cd> -- <col><b>To come upon</b></col>. <sd>(a)</sd> <cd>To befall.</cd> <sd>(b)</sd> <cd>To attack or invade.</cd> <sd>(c)</sd> <cd>To have a claim upon; to become dependent upon for support; <as>as, <ex>to come upon</ex> the town</as>.</cd> <sd>(d)</sd> <cd>To light or chance upon; to find; <as>as, to <ex>come upon</ex> hid treasure</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Come</ent><br/
<hw>Come</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To carry through; to succeed in; <as>as, you can't <ex>come</ex> any tricks here</as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To come it</b></col>, <cd>to succeed in a trick of any sort.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Come</ent><br/
<hw>Come</hw>, <pos>n.</pos> <def>Coming.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Come-along</ent><br/
<hw>Come"-a*long`</hw>, <pos>n.</pos> <def>A gripping device, as for stretching wire, etc., consisting of two jaws so attached to a ring that they are closed by pulling on the ring.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Co-meddle</ent><br/
<hw>Co-med"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To mix; to mingle, to temper.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comedian</ent><br/
<hw>Co*me"di*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>com<eacute/dien</ets>.]</ety> <sn>1.</sn> <def>An actor or player in comedy.</def> <ldquo/The famous <xex>comedian</xex>, Roscius.<rdquo/ <rj><au>Middleton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A writer of comedy.</def> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comedienne</ent><br/
||<hw>Co*m<eacute/`di*enne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fem. of <ets>com<eacute/dien</ets>.]</ety> <def>A women who plays in comedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comedietta</ent><br/
||<hw>Co*me`di*et"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A dramatic sketch; a brief comedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comedo</ent><br/
||<hw>Com"e*do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Comedones</plw> <pr>(#)</pr>.</plu> <ety>[L., a glutton. See <er>Comestible</er>.]</ety> <fld>(Med.)</fld> <def>A small nodule or cystic tumor, common on the nose, etc., which on pressure allows the escape of a yellow wormlike mass of retained oily secretion, with a black head (dirt).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comedown</ent><br/
<hw>Come"down`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A downfall; an humiliation.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comedy</ent><br/
<hw>Com"e*dy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Comedies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>com<eacute/die</ets>, L. <ets>comoedia</ets>, fr. Gr. <?/; <?/ a jovial festivity with music and dancing, a festal procession, an ode sung at this procession (perh. akin to <?/ village, E. <ets>home</ets>) + <?/ to sing; for comedy was originally of a lyric character. See <er>Home</er>, and <er>Ode</er>.]</ety> <def>A dramatic composition, or representation of a bright and amusing character, based upon the foibles of individuals, the manners of society, or the ludicrous events or accidents of life; a play in which mirth predominates and the termination of the plot is happy; -- opposed to <contr>tragedy</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With all the vivacity of <qex>comedy</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Are come to play a pleasant <qex>comedy</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comelily</ent><br/
<hw>Come"li*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a suitable or becoming manner.</def> <mark>[R.]</mark> <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 283 --></p>

<p><ent>Comeliness</ent><br/
<hw>Come"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Comely</er>.]</ety> <def>The quality or state of being comely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Comeliness</qex> is a disposing fair<br/
Of things and actions in fit time and place.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Strength, <qex>comeliness</qex> of shape, or amplest merit.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Comeliness</qex> signifies something less forcible than beauty, less elegant than grace, and less light than prettiness.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comely</ent><br/
<hw>Come"ly</hw> <pr>(k<ucr/m"l<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Comelier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Comeliest</adjf>.]</amorph> <ety>[OE. <ets>comeliche</ets>, AS. <ets>cyml<imac/c</ets>; <ets>cyme</ets> suitable (fr. <ets>cuman</ets> to come, become) + <ets>l<imac/c</ets> like.]</ety> <sn>1.</sn> <def>Pleasing or agreeable to the sight; well-proportioned; good-looking; handsome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that is <qex>comely</qex> when old and decrepit, surely was very beautiful when he was young.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Not once perceive their foul disfigurement<br/
But boast themselves more <qex>comely</qex> than before.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Suitable or becoming; proper; agreeable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is a happier and more <qex>comely</qex> time<br/
Than when these fellows ran about the streets,<br/
Crying confusion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is good to sing praises unto our God; for it is pleasant; and praise is <qex>comely</qex>.</q> <rj><qau>Ps. cxlvii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comely</ent><br/
<hw>Come"ly</hw>, <pos>adv.</pos> <def>In a becoming manner.</def> <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Come-outer</ent><br/
<hw>Come-out"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who comes out or withdraws from a religious or other organization; a radical reformer.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comer</ent><br/
<hw>Com"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who comes, or who has come; one who has arrived, and is present.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>All comers</b></col>, <cd>all who come, or offer, to take part in a matter, especially in a contest or controversy.</cd> <ldquo/To prove it against <xex>all comers</xex>.<rdquo/ <rj><au>Bp. Stillingfleet.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comes</ent><br/
||<hw>Co"mes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a companion.]</ety> <fld>(Mus.)</fld> <def>The answer to the theme (<it>dux</it>) in a fugue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comessation</ent><br/
<hw>Com`es*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comissatio</ets>, <ets>comessatio</ets>.]</ety> <def>A reveling; a rioting.</def> <mark>[Obs.]</mark> <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comestible</ent><br/
<hw>Co*mes"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>comestible</ets>, fr. L. <ets>comesus</ets>, <ets>comestus</ets>, p. p. of <ets>comedere</ets> to eat; <ets>com-</ets> + <ets>edere</ets> to eat.]</ety> <def>Suitable to be eaten; eatable; esculent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some herbs are most <qex>comestible</qex>.</q> <rj><qau>Sir T. Elyot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comestible</ent><br/
<hw>Co*mes"ti*ble</hw>, <pos>n.</pos> <def>Something suitable to be eaten; -- commonly in the plural.</def> <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comet</ent><br/
<hw>Com"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cometes</ets>, <ets>cometa</ets>, from Gr. <?/ comet, prop. long-haired, fr. <?/ to wear long hair, fr. <?/ hair, akin to L. <ets>coma</ets>: cf. F. <ets>com<egrave/te</ets>.]</ety> <fld>(Astron.)</fld> <def>A member of the solar system which usually moves in an elongated orbit, approaching very near to the sun in its perihelion, and receding to a very great distance from it at its aphelion. A comet commonly consists of three parts: the nucleus, the envelope, or coma, and the tail; but one or more of these parts is frequently wanting. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cometarium</ent><br/
||<hw>Com`e*ta"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Astron.)</fld> <def>An instrument, intended to represent the revolution of a comet round the sun.</def> <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cometary</ent><br/
<hw>Com"et*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>com<eacute/taire</ets>.]</ety> <def>Pertaining to, or resembling, a comet.</def> <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comet- seeker</ent><br/
<ent>Comet-finder</ent><br/
<mhw>{ <hw>Com"et-find`er</hw> <pr>(?)</pr>, <or/ <hw>Com"et- seek`er</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <fld>(Astron.)</fld> <def>A telescope of low power, having a large field of view, used for finding comets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comether</ent><br/
<hw>Co*meth"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. dial. pron. of <ets>come hither</ets>, used in calling cows, etc.]</ety> <mark>[Dial. or Colloq., Brit.]</mark> <sn>1.</sn>  <def>Matter; affair.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Friendly communication or association.</def></p>

<p><cs><mcol><col><b>To put the comether on</b></col> <it>or</it>  <col><b>To put one's comether on</b></col></mcol>, <cd>to exercise persuasion upon; to get under one's influence; to beguile; to wheedle.</cd></cs></p>

<p><q>How does ut come about, sorr, that whin a man has <qex>put the comether on</qex> wan woman he's sure bound to put ut on another?</q>  <rj><qau>Kipling.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cometic</ent><br/
<hw>Co*met"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to a comet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cometographer</ent><br/
<hw>Com`et*og"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who describes or writes about comets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cometography</ent><br/
<hw>Com`et*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Comet</ets> + <ets>-graphy</ets>: cf. F. <ets>com<eacute/tographie</ets>.]</ety> <def>A description of, or a treatise concerning, comets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cometology</ent><br/
<hw>Com`et*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Comet</ets> + <ets>-logy</ets>.]</ety> <def>The branch of astronomy relating to comets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>comeuppance</ent><br/
<hw>come`up"pance</hw> <pr>(k<ucr/m`<ucr/p"p<ait/ns)</pr>, <pos>n.</pos> <def>An unpleasant experience a person endures, which is viewed by others as a just retribution for bad behavior; just deserts; <as>as, the Senator took bribes for years, and finally got his <ex>comeuppance</ex> when he was caught in a sting operation</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Comfit</ent><br/
<hw>Com"fit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confit</ets>, prop. a p. p., fr. <ets>confire</ets> to preserve, pickle, fr. L. <ets>conficere</ets> to prepare; <ets>con-</ets> + <ets>facere</ets> to make. See <er>Fact</er>, and cf. <er>Confect</er>.]</ety> <def>A dry sweetmeat; any kind of fruit, root, or seed preserved with sugar and dried; a confection.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfit</ent><br/
<hw>Com"fit</hw>, <pos>v. t.</pos> <def>To preserve dry with sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fruit which does so quickly waste, . . . <br/
Thou <qex>comfitest</qex> in sweets to make it last.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfiture</ent><br/
<hw>Com"fi*ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[F. <ets>confiture</ets>; cf. LL. <ets>confecturae</ets> sweetmeats, <ets>confectura</ets> a preparing. See <er>Comfit</er>, and cf. <er>Confiture</er>.]</ety> <def>See <er>Comfit</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfort</ent><br/
<hw>Com"fort</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Comforted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Comforting.</conjf>]</vmorph> <ety>[F. <ets>conforter</ets>, fr. L. <ets>confortare</ets> to strengthen much; <ets>con-</ets> + <ets>fortis</ets> strong. See <er>Fort</er>.]</ety> <sn>1.</sn> <def>To make strong; to invigorate; to fortify; to corroborate.</def> <mark>[Obs.]</mark> <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God's own testimony . . . doth not a little <qex>comfort</qex> and confirm the same.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To assist or help; to aid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I . . . can not help the noble chevalier:<br/
God <qex>comfort</qex> him in this necessity!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To impart strength and hope to; to encourage; to relieve; to console; to cheer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Light excelleth in <qex>comforting</qex> the spirits of men.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That we may be able to <qex>comfort</qex> them that are in any affliction.</q> <rj><qau>2 Cor. i. 4 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A perfect woman, nobly planned,<br/
To warn, to <qex>comfort</qex>, and command.</q> <rj><qau>Wordsworth.</qau></rj></p>

<p><syn><b>Syn.</b> -- To cheer; solace; console; revive; encourage; enliven; invigorate; inspirit; gladden; recreate; exhilarate; refresh; animate; confirm; strengthen.</syn> <usage> -- <er>To Comfort</er>, <er>Console</er>, <er>Solace</er>. These verbs all suppose some antecedent state of suffering or sorrow. <xex>Console</xex> is confined to the act giving sympathetic relief to the mind under affliction or sorrow, and points to some definite source of that relief; as, the presence of his friend <xex>consoled</xex> him; he was much <xex>consoled</xex> by this intelligence. The act of consoling commonly implies the inculcation of resignation. <xex>Comfort</xex> points to relief afforded by the communication of positive pleasure, hope, and strength, as well as by the diminution of pain; as, <ldquo/They brought the young man alive, and were not a little <xex>comforted</xex>.<rdquo/</usage> <au>Acts xx. 12.</au> <xex>Solace</xex> is from L. <xex>solacium</xex>, which means according to Dumesnil, consolation inwardly felt or applied to the case of the sufferer. Hence, the verb <xex>to solace</xex> denotes the using of things for the purpose of affording relief under sorrow or suffering; as, <xex>to solace</xex> one's self with reflections, with books, or with active employments.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfort</ent><br/
<hw>Com"fort</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>confort</ets>, fr. <ets>conforter</ets>.]</ety> <sn>1.</sn> <def>Assistance; relief; support.</def> <mark>[Obs. except in the phrase <ldquo/aid and <xex>comfort</xex>.<rdquo/ See 5 below.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Encouragement; solace; consolation in trouble; also, that which affords consolation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>comfort</qex> of her mother's fears.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cheer thy spirit with this <qex>comfort</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Speaking words of endearment where words of <qex>comfort</qex> availed not.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A state of quiet enjoyment; freedom from pain, want, or anxiety; also, whatever contributes to such a condition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had much joy and <qex>comfort</qex> in thy love.</q> <rj><qau>Phil. 7 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had the means of living in <qex>comfort</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A wadded bedquilt; a comfortable.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>Unlawful support, countenance, or encouragement; <as>as, to give aid and <ex>comfort</ex> to the enemy</as>.</def></p>

<p><syn><b>Syn.</b> -- <er>Comfort</er>, <er>Consolation</er>.</syn> <usage> <xex>Comfort</xex> has two meanings: <sn>1.</sn> Strength and relief received under affliction; <sn>2.</sn> Positive enjoyment, of a quiet, permanent nature, together with the sources thereof; as, the <xex>comfort</xex> of love; surrounded with <xex>comforts</xex>; but it is with the former only that the word <xex>consolation</xex> is brought into comparison. As thus compared, <xex>consolation</xex> points to some specific source of relief for the afflicted mind; as, the <xex>consolations</xex> of religion. <xex>Comfort</xex> supposes the relief to be afforded by imparting positive enjoyment, as well as a diminution of pain. <ldquo/<xex>Consolation</xex>, or <xex>comfort</xex>, signifies some alleviation to that pain to which it is not in our power to afford the proper and adequate remedy; they imply rather an augmentation of the power of bearing, than a diminution of the burden.<rdquo/ <rj><au>Johnson.</au></rj></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfortable</ent><br/
<hw>Com"fort*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>confortable</ets>.]</ety> <sn>1.</sn> <def>Strong; vigorous; valiant.</def> <mark>[Obs.]</mark> <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy conceit is nearer death than thy powers. For my sake be <qex>comfortable</qex>; hold death a while at the arm's end.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Serviceable; helpful.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Be <qex>comfortable</qex> to my mother, your mistress, and make much of her.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Affording or imparting comfort or consolation; able to comfort; cheering; <as>as, a <ex>comfortable</ex> hope</as>.</def> <ldquo/Kind words and <xex>comfortable</xex>.<rdquo/ <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>comfortable</qex> provision made for their subsistence.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In a condition of comfort; having comforts; not suffering or anxious; hence, contented; cheerful; <as>as, to lead a <ex>comfortable</ex> life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord leans wondrously to discontent;<br/
His <qex>comfortable</qex> temper has forsook him:<br/
He is much out of health.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Free, or comparatively free, from pain or distress; -- used of a sick person.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfortable</ent><br/
<hw>Com"fort*a*ble</hw>, <pos>n.</pos> <def>A stuffed or quilted coverlet for a bed; a comforter; a comfort.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfortableness</ent><br/
<hw>Com"fort*a*ble*ness</hw>, <pos>n.</pos> <def>State of being comfortable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfortably</ent><br/
<hw>Com"fort*a*bly</hw>, <pos>adv.</pos> <def>In a comfortable or comforting manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Speak ye <qex>comfortably</qex> to Jerusalem.</q> <rj><qau>Is. xl. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comforter</ent><br/
<hw>Com"fort*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who administers comfort or consolation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let no <qex>comforter</qex> delight mine ear<br/
But such a one whose wrongs do suit with mine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Script.)</fld> <def>The Holy Spirit, -- referring to his office of comforting believers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But the <qex>Comforter</qex>, which is the Holy Ghost, whom the Father will send in my name, he shall teach you all things.</q> <rj><qau>John xiv. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A knit woolen tippet, long and narrow.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The American schoolboy takes off his <qex>comforter</qex> and unbuttons his jacket before going in for a snowball fight.</q> <rj><qau>Pop. Sci. Monthly.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A wadded bedquilt; a comfortable.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Job's comforter</b></col>, <cd>a boil.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>comforting</ent><br/
<hw>comforting</hw> <pos>adj.</pos> <sn>1.</sn> <def>providing freedom from worry.</def><br/
<syn><b>Syn. --</b> cheering, satisfying.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>affording comfort or solace.</def><br/
<syn><b>Syn. --</b> consolatory, consoling.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comfortless</ent><br/
<hw>Com"fort*less</hw>, <pos>a.</pos> <def>Without comfort or comforts; in want or distress; cheerless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Comfortless</qex> through tyranny or might.</q> <rj><qau>Spenser.</qau></rj></p>

<p><syn><b>Syn.</b> -- Forlorn; desolate; cheerless; inconsolable; disconsolate; wretched; miserable.</syn></p>

<p>-- <wordforms><wf>Com"fort*less*ly</wf>, <pos>adv.</pos> -- <wf>Com"fort*less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>When all is coldly, <qex>comfortlessly</qex> costly.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfortment</ent><br/
<hw>Com"fort*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act or process of administering comfort.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The gentle <qex>comfortment</qex> and entertainment of the said embassador.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfortress</ent><br/
<hw>Com"fort*ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who comforts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To be your <qex>comfortress</qex>, and to preserve you.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comfrey</ent><br/
<hw>Com"frey</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. from F. <ets>conferve</ets>, L. <ets>conferva</ets>, fr. <ets>confervere</ets> to boil together, in medical language, to heal, grow together. So called on account of its healing power, for which reason it was also called <ets>consolida</ets>.]</ety> <fld>(Bot.)</fld> <def>A rough, hairy, perennial plant of several species, of the genus <gen>Symphytum</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A decoction of the mucilaginous root of the <ldquo/common comfrey<rdquo/ (<spn>Symphytum officinale</spn>) is used in cough mixtures, etc.; and the gigantic <ldquo/prickly comfrey<rdquo/ (<spn>Symphytum asperrimum</spn>) is somewhat cultivated as a forage plant.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>comfy</ent><br/
<hw>comfy</hw> <pos>adj.</pos> <sn>1.</sn> <def>comfortable.</def> <mark>[informal].</mark> <ant>uncomfortable</ant><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comic</ent><br/
<hw>Com"ic</hw> <pr>(k<ocr/m"<icr/k)</pr>, <pos>a.</pos> <ety>[L. <ets>comicus</ets> pertaining to comedy, Gr. <grk>kwmiko`s</grk>: cf. F. <ets>comique</ets>. See <er>Comedy</er>.]</ety> <sn>1.</sn> <def>Relating to comedy, as distinct from tragedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not for the stage a drama lay,<br/
Tragic or <qex>comic</qex>, but thou writ'st the play.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Causing mirth; ludicrous.</def> <ldquo/<xex>Comic</xex> shows.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comic</ent><br/
<hw>Com"ic</hw>, <pos>n.</pos> <def>A comedian.</def> <mark>[Obs.]</mark> <rj><au>Steele.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comical</ent><br/
<hw>Com"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Relating to comedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They deny it to be tragical because its catastrophe is a wedding, which hath ever been accounted <qex>comical</qex>.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exciting mirth; droll; laughable; <as>as, a <ex>comical</ex> story</as>.</def> <ldquo/<xex>Comical</xex> adventures.<rdquo/ <rj><au>Dryden.</au></rj></p>

<p><syn><b>Syn.</b> -- Humorous; laughable; funny. See <er>Droll</er>.</syn></p>

<p>-- <wordforms><wf>Com"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Com"ic*al"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comicality</ent><br/
<hw>Com`i*cal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Comicalities</plw> <pr>(#)</pr>.</plu> <def>The quality of being comical; something comical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comicry</ent><br/
<hw>Com"ic*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The power of exciting mirth; comicalness.</def> <mark>[R.]</mark> <rj><au>H. Giles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>comics</ent><br/
<hw>com"ics</hw> <pr>(k<ocr/m"<icr/ks)</pr>, <pos>n. pl.</pos> <def>The section of a newspaper containing mostly <er>comic strips</er>; -- called also <altname>funnies</altname> and <altname>funny papers</altname>.  Many but not all newspapers have a <ex>comics</ex> section.</def><br/
[<source>PJC</source>]</p>

<p><ent>comic strip</ent><br/
<hw>com"ic strip`</hw> <pr>(k<ocr/m"<icr/k str<icr/p)</pr>, <pos>n.</pos> <def>a brief sequence of drawings, usually with characters drawn only sketchily, as in a cartoon, with dialog written in <ldquo/balloons<rdquo/ over a character's head, and depicting a fictional and usually comical incident; -- also called a <altname>cartoon</altname>.  Each <ex>comic strip</ex> contains typically from four to six panels arranged horizontally, but widely varying arrangements are published.  In modern newspapers, weekly <ex>comic strips</ex> are in color, and daily strips are usually in black and white. In some, the story depicted may be serialized and continuous, carried over from day to day or week to week.  Stories of adventure, drama, mystery or an otherwise non-comical nature depicted in the same style are also called <ex>comic strips</ex>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coming</ent><br/
<hw>Com"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Approaching; of the future, especially the near future; the next; <as>as, the <ex>coming</ex> week or year; the <ex>coming</ex> exhibition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Welcome the <qex>coming</qex>, speed the parting, guest.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>coming</qex> days and years.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ready to come; complaisant; fond.</def> <mark>[Obs.]</mark> <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coming</ent><br/
<hw>Com"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Approach; advent; manifestation; <as>as, the <ex>coming</ex> of the train</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically: The Second Advent of Christ, called usually the <altname>second coming</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coming in</b></col>. <sd>(a)</sd> <cd>Entrance; entrance way; manner of entering; beginning.</cd> <ldquo/The goings out thereof, and the <xex>comings in</xex> thereof.<rdquo/ <au>Ezek. xliii. 11</au> <sd>(b)</sd> <cd>Income or revenue.</cd> <ldquo/What are thy <xex>comings in</xex>?<rdquo/ <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>COMINT</ent><br/
<hw>COMINT</hw> <pos>n.</pos> <def>an abbreviation for <altname>communications intelligence</altname>; technical and intelligence information derived from foreign communications by other than the intended recipients.</def><br/
<syn><b>Syn. --</b> communications intelligence.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comitia</ent><br/
||<hw>Co*mi"ti*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L.]</ety> <fld>(Rom. Antiq.)</fld> <def>A public assembly of the Roman people for electing officers or passing laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There were three kinds of <xex>comitia</xex>: <xex>comitia curiata</xex>, or assembly of the patricians, who voted in curi<ae/; <xex>comitia centuriata</xex>, or assembly of the whole Roman people, who voted by centuries; and <xex>comitia tributa</xex>, or assembly of the plebeians according to their division into tribes.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comitial</ent><br/
<hw>Co*mi"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>comitialis</ets>.]</ety> <def>Relating to the comitia, or popular assemblies of the Romans for electing officers and passing laws.</def> <rj><au>Middleton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comitiva</ent><br/
||<hw>Co`mi*ti"va</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A body of followers; -- applied to the lawless or brigand bands in Italy and Sicily.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Comity</ent><br/
<hw>Com"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Comities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>comitas</ets>, fr. <ets>comis</ets> courteous, kind.]</ety> <def>Mildness and suavity of manners; courtesy between equals; friendly civility; <as>as, <ex>comity</ex> of manners; the <ex>comity</ex> of States</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Comity of nations</b></col> <fld>(International Law)</fld>, <cd>the courtesy by which nations recognize within their own territory, or in their courts, the peculiar institutions of another nation or the rights and privileges acquired by its citizens in their own land. By some authorities private international law rests on this comity, but the better opinion is that it is part of the common law of the land, and hence is obligatory as law.</cd></cs></p>

<p><syn><b>Syn.</b> -- Civility; good breeding; courtesy; good will.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comma</ent><br/
<hw>Com"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comma</ets> part of a sentence, comma, Gr. <?/ clause, fr. <?/ to cut off. Cf. <er>Capon</er>.]</ety> <sn>1.</sn> <def>A character or point [,] marking the smallest divisions of a sentence, written or printed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A small interval (the difference between a major and minor half step), seldom used except by tuners.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Comma bacillus</b></col> <fld>(Physiol.)</fld>, <cd>a variety of bacillus shaped like a comma, found in the intestines of patients suffering from cholera. It is considered by some as having a special relation to the disease; -- called also <altname>cholera bacillus</altname>.</cd> -- <col><b>Comma butterfly</b></col> <fld>(Zool.)</fld>, <cd>an American butterfly (<spn>Grapta comma</spn>), having a white comma-shaped marking on the under side of the wings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Command</ent><br/
<hw>Com*mand"</hw> <pr>(?; 61)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commanded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commanding</conjf>.]</vmorph> <ety>[OE. <ets>comaunden</ets>, <ets>commanden</ets>, OF. <ets>comander</ets>, F. <ets>commander</ets>, fr. L. <ets>com-</ets> + <ets>mandare</ets> to commit to, to command. Cf. <er>Commend</er>, <er>Mandate</er>.]</ety> <sn>1.</sn> <def>To order with authority; to lay injunction upon; to direct; to bid; to charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are <qex>commanded</qex> to forgive our enemies, but you never read that we are <qex>commanded</qex> to forgive our friends.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Go to your mistress:<br/
Say, I <qex>command</qex> her come to me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exercise direct authority over; to have control of; to have at one's disposal; to lead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Monmouth <qex>commanded</qex> the English auxiliaries.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such aid as I can spare you shall <qex>command</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have within a sphere of control, influence, access, or vision; to dominate by position; to guard; to overlook.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bridges <qex>commanded</qex> by a fortified house.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Up to the eastern tower,<br/
Whose height <qex>commands</qex> as subject all the vale.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One side <qex>commands</qex> a view of the finest garden.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To have power or influence of the nature of authority over; to obtain as if by ordering; to receive as a due; to challenge; to claim; <as>as, justice <ex>commands</ex> the respect and affections of the people; the best goods <ex>command</ex> the best price</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'Tis not in mortals to <qex>command</qex> success.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To direct to come; to bestow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>command</qex> my blessing upon you.</q> <rj><qau>Lev. xxv. 21.</qau></rj></p>

<p><syn><b>Syn.</b> -- To bid; order; direct; dictate; charge; govern; rule; overlook.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Command</ent><br/
<hw>Com*mand"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have or to exercise direct authority; to govern; to sway; to influence; to give an order or orders.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And reigned, <qex>commanding</qex> in his monarchy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For the king had so <qex>commanded</qex> concerning [Haman].</q> <rj><qau>Esth. iii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have a view, as from a superior position.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Far and wide his eye <qex>commands</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Command</ent><br/
<hw>Com*mand"</hw>, <pos>n.</pos> <sn>1.</sn> <def>An authoritative order requiring obedience; a mandate; an injunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Awaiting what <qex>command</qex> their mighty chief<br/
Had to impose.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The possession or exercise of authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Command</qex> and force may often create, but can never cure, an aversion.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Authority; power or right of control; leadership; <as>as, the forces under his <ex>command</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Power to dominate, command, or overlook by means of position; scope of vision; survey.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The steepy stand<br/
Which overlooks the vale with wide <qex>command</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Control; power over something; sway; influence; <as>as, to have <ex>command</ex> over one's temper or voice; the fort has <ex>command</ex> of the bridge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He assumed an absolute <qex>command</qex> over his readers.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A body of troops, or any naval or military force or post, or the whole territory under the authority or control of a particular officer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Word of command</b></col> <fld>(Mil.)</fld>, <cd>a word or phrase of definite and established meaning, used in directing the movements of soldiers; as, <stype>aim</stype>; <stype>fire</stype>; <stype>shoulder arms</stype>, etc.</cd></cs></p>

<p><syn><b>Syn.</b> -- Control; sway; power; authority; rule; dominion; sovereignty; mandate; order; injunction; charge; behest. See <er>Direction</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 284 --></p>

<p><ent>Commandable</ent><br/
<hw>Com*mand"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being commanded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandant</ent><br/
<hw>Com`man*dant"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., orig. p. pr. of <ets>commander</ets>.]</ety> <def>A commander; the commanding officer of a place, or of a body of men; <as>as, the <ex>commandant</ex> of a navy-yard</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandatory</ent><br/
<hw>Com*mand"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Mandatory; <as>as, <ex>commandatory</ex> authority</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandeer</ent><br/
<hw>Com`man*deer"</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commandeered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Commandeering</conjf>.]</vmorph> <ety>[D. <ets>kommandeeren</ets> to command, in South Africa to commandeer, fr. F. <ets>commander</ets> to command. See <er>Command</er>.]</ety> <sn>1.</sn>  <fld>(Mil.)</fld> <def>To compel to perform military service; to seize for military purposes; -- orig. used of the Boers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To take arbitrary or forcible possession of.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Commander</ent><br/
<hw>Com*mand"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>commandeur</ets>. Cf. <er>Commodore</er>, <er>Commender</er>.]</ety> <sn>1.</sn> <def>A chief; one who has supreme authority; a leader; the chief officer of an army, or of any division of it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A leader and <qex>commander</qex> to the people.</q> <rj><qau>Is. lv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Navy)</fld> <def>An officer who ranks next below a captain, -- ranking with a lieutenant colonel in the army.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The chief officer of a commandery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A heavy beetle or wooden mallet, used in paving, in sail lofts, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commander in chief</b></col>, <cd>the military title of the officer who has supreme command of the land or naval forces or the united forces of a nation or state; a generalissimo. <as>The President is <ex>commander in chief</ex> of the army and navy of the United States</as>.</cd></cs></p>

<p><syn><b>Syn.</b> -- See <er>Chief</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandership</ent><br/
<hw>Com*mand"er*ship</hw>, <pos>n.</pos> <def>The office of a commander.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandery</ent><br/
<hw>Com*mand"er*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Commanderies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>commanderie</ets>.]</ety> <sn>1.</sn> <def>The office or rank of a commander.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A district or a manor with lands and tenements appertaining thereto, under the control of a member of an order of knights who was called a <xex>commander</xex>; -- called also a <altname>preceptory</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An assembly or lodge of Knights Templars (so called) among the Freemasons.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A district under the administration of a military commander or governor.</def> <mark>[R.]</mark> <rj><au>Brougham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commanding</ent><br/
<hw>Com*mand"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Exercising authority; actually in command; <as>as, a <ex>commanding</ex> officer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fitted to impress or control; <as>as, a <ex>commanding</ex> look or presence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Exalted; overlooking; having superior strategic advantages; <as>as, a <ex>commanding</ex> position</as>.</def></p>

<p><syn><b>Syn.</b> -- Authoritative; imperative; imperious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandingly</ent><br/
<hw>Com*mand"ing*ly</hw>, <pos>adv.</pos> <def>In a commanding manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandment</ent><br/
<hw>Com*mand"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>commandement</ets>, F. <ets>commandement</ets>.]</ety> <sn>1.</sn> <def>An order or injunction given by authority; a command; a charge; a precept; a mandate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A new <qex>commandment</qex> I give unto you, that ye love one another.</q> <rj><qau>John xiii. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Script.)</fld> <def>One of the ten laws or precepts given by God to the Israelites at Mount Sinai.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of commanding; exercise of authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And therefore put I on the countenance<br/
Of stern <qex>commandment</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>The offense of commanding or inducing another to violate the law.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>The Commandments</b></col>, <col><b>The Ten Commandments</b></col></mcol>, <cd>the Decalogue, or summary of God's commands, given to Moses at Mount Sinai. (<au>Ex. xx.</au>)</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commando</ent><br/
<hw>Com*man"do</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. See <er>Command</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>In South Africa, a military body or command; also, sometimes, an expedition or raid; <as>as, a <ex>commando</ex> of a hundred Boers</as>.</def></p>

<p><q>The war bands, called <qex>commandos</qex>, have played a great part in the . . . military history of the country.</q>  <rj><qau>James Bryce.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a usually small military unit trained to perform special missions, often behind enemy lines, and usually employing hit-and-run tactics. Such units often have the destruction of fixed targets, such as enemy installations, as their mission, in contrast to normal combat units which engage the enemy army directly.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>a member of a commando{2} unit.</def><br/
[<source>PJC</source>]</p>

<p><ent>Commandress</ent><br/
<hw>Com*mand"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman invested with authority to command.</def> <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commandry</ent><br/
<hw>Com*mand"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Commandery</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commark</ent><br/
<hw>Com"mark`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>comarque</ets>, or LL. <ets>commarca</ets>, <ets>commarcha</ets>; <ets>com-</ets> + <ets>marcha</ets>, boundary. See <er>March</er> a confine.]</ety> <def>The frontier of a country; confines.</def> <mark>[Obs.]</mark> <rj><au>Shelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commaterial</ent><br/
<hw>Com`ma*te"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of the same material.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commatic</ent><br/
<hw>Com*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commaticus</ets>, Gr. <?/. See <er>Comma</er>.]</ety> <def>Having short clauses or sentences; brief; concise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commatism</ent><br/
<hw>Com"ma*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Commatic</er>.]</ety> <def>Conciseness in writing.</def> <rj><au>Bp. Horsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commeasurable</ent><br/
<hw>Com*meas"ur*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Commensurable</er>.]</ety> <def>Having the same measure; commensurate; proportional.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She being now removed by death, a <qex>commeasurable</qex> grief took as full possession of him as joy had done.</q> <rj><qau>I. Walton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commeasure</ent><br/
<hw>Com*meas"ure</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To be commensurate with; to equal.</def> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commemorable</ent><br/
<hw>Com*mem"o*ra"ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commemorabilis</ets>.]</ety> <def>Worthy to be commemorated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commemorate</ent><br/
<hw>Com*mem"o*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commemorated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commemorating</conjf>.]</vmorph> <ety>[L. <ets>commemoratus</ets>, p. p. of <ets>commemorare</ets> to remember; <ets>com-</ets> + <ets>memorare</ets> to mention, fr. <ets>memor</ets> mindful. See <er>Memory</er>.]</ety> <def>To call to remembrance by a special act or observance; to celebrate with honor and solemnity; to honor, as a person or event, by some act of respect or affection, intended to preserve the remembrance of the person or event; <as>as, to <ex>commemorate</ex> the sufferings and dying love of our Savior by the sacrament of the Lord's Supper; to <ex>commemorate</ex> the Declaration of Independence by the observance of the Fourth of July</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are called upon to <qex>commemorate</qex> a revolution.</q> <rj><qau>Atterbury.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Celebrate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>commemorating</ent><br/
<hw>commemorating</hw> <pos>adj.</pos>  <def>of or pertaining to a commemoration; serving to commemorate.</def><br/
<syn><b>Syn. --</b> commemorative.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Commemoration</ent><br/
<hw>Com*mem`o*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commemoratio</ets>.]</ety> <sn>1.</sn> <def>The act of commemorating; an observance or celebration designed to honor the memory of some person or event.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This sacrament was designed to be a standing <qex>commemoration</qex> of the death and passion of our Lord.</q> <rj><qau>Abp. Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The commonwealth which . . . chooses the most flagrant act of murderous regicide treason for a feast of eternal <qex>commemoration</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Whatever serves the purpose of commemorating; a memorial.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commemoration day</b></col>, <cd>at the University of Oxford, Eng., an annual observance or ceremony in honor of the benefactors of the University, at which time honorary degrees are conferred.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commemorative</ent><br/
<hw>Com*mem"o*ra*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending or intended to commemorate; <as>as, a <ex>commemorative</ex> plaque</as>.</def> <ldquo/A sacrifice <xex>commemorative</xex> of Christ's offering up his body for us.<rdquo/ <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An inscription <qex>commemorative</qex> of his victory.</q> <rj><qau>Sir G. C. Lewis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commemorative</ent><br/
<hw>Com*mem"o*ra*tive</hw> <pr>(?)</pr>, <pos>n.</pos> <def>something that commemorates, especially a postage stamp or coin having a design commemorating some event, person, institution, etc.</def><br/
[<source>PJC</source>]</p>

<p><ent>Commemorator</ent><br/
<hw>Com*mem"o*ra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who commemorates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commemoratory</ent><br/
<hw>Com*mem"o*ra*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to commemorate; commemorative.</def> <rj><au>Bp. Hooper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commence</ent><br/
<hw>Com*mence"</hw> <pr>(k<ocr/m*m<ecr/ns")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commenced</conjf> <pr>(k<ocr/m*m<ecr/nst")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Commencing</conjf>.]</vmorph> <ety>[F. <ets>commencer</ets>, OF. <ets>comencier</ets>, fr. L. <ets>com-</ets> + <ets>initiare</ets> to begin. See <er>Initiate</er>.]</ety> <sn>1.</sn> <def>To have a beginning or origin; to originate; to start; to begin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here the anthem doth <qex>commence</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His heaven <qex>commences</qex> ere the world be past.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To begin to be, or to act as.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>commence</qex> judges ourselves.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take a degree at a university.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I question whether the formality of <qex>commencing</qex> was used in that age.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commence</ent><br/
<hw>Com*mence"</hw>, <pos>v. t.</pos> <def>To enter upon; to begin; to perform the first act of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a wooer doth <qex>commence</qex> his suit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is the practice of good writers to use the verbal noun (instead of the infinitive with <xex>to</xex>) after <xex>commence</xex>; <as>as, he <ex>commenced studying</ex>, not he <ex>commenced to study</ex></as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commencement</ent><br/
<hw>Com*mence"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>commencement</ets>.]</ety> <sn>1.</sn> <def>The first existence of anything; act or fact of commencing; rise; origin; beginning; start.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The time of Henry VII. . . . nearly coincides with the <qex>commencement</qex> of what is termed <ldquo/modern history.<rdquo/</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The day when degrees are conferred by colleges and universities upon students and others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commend</ent><br/
<hw>Com*mend"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commending</conjf>.]</vmorph> <ety>[L. <ets>commendare</ets>; <ets>com-</ets> + <ets>mandare</ets> to intrust to one's charge, enjoin, command. Cf. <er>Command</er>, <er>Mandate</er>.]</ety> <sn>1.</sn> <def>To commit, intrust, or give in charge for care or preservation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His eye <qex>commends</qex> the leading to his hand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Father, into thy hands I <qex>commend</qex> my spirit.</q> <rj><qau>Luke xxiii. 46.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To recommend as worthy of confidence or regard; to present as worthy of notice or favorable attention.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Among the objects of knowledge, two especially <qex>commend</qex> themselves to our contemplation.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>commend</qex> unto you Phebe our sister.</q> <rj><qau>Rom. xvi. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To mention with approbation; to praise; <as>as, to <ex>commend</ex> a person or an act</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Historians <qex>commend</qex> Alexander for weeping when he read the actions of Achilles.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To mention by way of courtesy, implying remembrance and good will.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commend</qex> me to my brother.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commend</ent><br/
<hw>Com*mend"</hw>, <pos>n.</pos> <sn>1.</sn> <def>Commendation; praise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Speak in his just <qex>commend</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Compliments; greetings.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hearty <qex>commends</qex> and much endeared love to you.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendable</ent><br/
<hw>Com*mend"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <note>(Formerly accented on the first syllable.)</note> <ety>[L. <ets>commendabilis</ets>.]</ety> <def>Worthy of being commended or praised; laudable; praiseworthy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Order and decent ceremonies in the church are not only comely but <qex>commendable</qex>.</q> <rj><qau>Bacon.</qau></rj></p>

<p>-- <wordforms><wf>Com*mend"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Com*mend"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendam</ent><br/
<hw>Com*men"dam</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. dare in <ets>commendam</ets> to give into trust.]</ety> <fld>(Eng. Eccl. Law)</fld> <def>A vacant living or benefice commended to a cleric (usually a bishop) who enjoyed the revenue until a pastor was provided. A living so held was said to be held <xex>in commendam</xex>. The practice was abolished by law in 1836.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There was [formerly] some sense for <qex>commendams</qex>.</q> <rj><qau>Selden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Partnership in commendam</b></col>. <cd>See under <er>Partnership</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendatary</ent><br/
<hw>Com*mend"a*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>commendataire</ets>, LL. <ets>commendatarius</ets>.]</ety> <def>One who holds a living <xex>in commendam</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendation</ent><br/
<hw>Com`men*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commendatio</ets>.]</ety> <sn>1.</sn> <def>The act of commending; praise; favorable representation in words; recommendation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Need we . . . epistles of <qex>commendation</qex>?</q> <rj><qau>2 Cor. iii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By the <qex>commendation</qex> of the great officers.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is the ground of approbation or praise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Good nature is the most godlike <qex>commendation</qex> of a man.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>A message of affection or respect; compliments; greeting.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hark you, Margaret;<br/
No princely <qex>commendations</qex> to my king?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendator</ent><br/
<hw>Com*mend"a*tor</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[LL.]</ety> <def>One who holds a benefice in commendam; a commendatary.</def> <rj><au>Chalmers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendatory</ent><br/
<hw>Com*mend"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commendatorius</ets>.]</ety> <sn>1.</sn> <def>Serving to commend; containing praise or commendation; commending; praising.</def> <ldquo/<xex>Commendatory</xex> verses.<rdquo/ <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Holding a benefice <xex>in commendam</xex>; <as>as, a <ex>commendatory</ex> bishop</as>.</def> <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commendatory prayer</b></col> <fld>(Book of Common Prayer)</fld>, <cd>a prayer read over the dying.</cd> <ldquo/The <xex>commendatory prayer</xex> was said for him, and, as it ended, he [William III.] died.<rdquo/ <rj><au>Bp. Burnet.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commendatory</ent><br/
<hw>Com*mend"a*to*ry</hw>, <pos>n.</pos> <def>A commendation; eulogy.</def> <mark>[R.]</mark> <ldquo/<xex>Commendatories</xex> to our affection.<rdquo/ <rj><au>Sharp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commender</ent><br/
<hw>Com*mend"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who commends or praises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensal</ent><br/
<hw>Com*men"sal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>commensalis</ets>; L. <ets>com-</ets> + <ets>mensa</ets> table: cf. F. <ets>commensal</ets>. Cf. Mensal.]</ety> <sn>1.</sn> <def>One who eats at the same table.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>An animal, not truly parasitic, which lives in, with, or on, another, partaking usually of the same food. Both species may be benefited by the association.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensal</ent><br/
<hw>Com*men"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the character of a commensal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensalism</ent><br/
<hw>Com*men"sal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of eating together; table fellowship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensality</ent><br/
<hw>Com`men*sal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Fellowship at table; the act or practice of eating at the same table.</def> <mark>[Obs.]</mark> <ldquo/Promiscuous <xex>commensality</xex>.<rdquo/ <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensation</ent><br/
<hw>Com`men*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Commensality.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Daniel . . . declined pagan <qex>commensation</qex>.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurability</ent><br/
<hw>Com*men`su*ra*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>commensurabilit<eacute/</ets>.]</ety> <def>The quality of being commensurable.</def> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurable</ent><br/
<hw>Com*men"su*ra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commensurabilis</ets>; pref. <ets>com-</ets> + <ets>mensurable</ets>. See <er>Commensurate</er>, and cf. <er>Commeasurable</er>.]</ety> <def>Having a common measure; capable of being exactly measured by the same number, quantity, or measure.</def> -- <wordforms><wf>Com*men"su*ra*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Commensurable numbers</b></col> <it>or</it> <col><b>Commensurable quantities</b></col></mcol> <fld>(Math.)</fld>, <cd>those that can be exactly expressed by some common unit; thus a foot and yard are <xex>commensurable</xex>, since both can be expressed in terms of an inch, one being 12 inches, the other 36 inches.</cd> -- <mcol><col><b>Numbers commensurable in power</b></col>, <it>or</it> <col><b>Quantities commensurable in power</b></col></mcol>, <cd>those whose squares are commensurable.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurably</ent><br/
<hw>Com*men"su*ra*bly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a commensurable manner; so as to be commensurable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurate</ent><br/
<hw>Com*men"su*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commensurated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commensurating</conjf>.]</vmorph> <ety>[Pref. <ets>com-</ets> + <ets>mensurate</ets>.]</ety> <sn>1.</sn> <def>To reduce to a common measure.</def> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To proportionate; to adjust.</def> <rj><au>T. Puller</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurate</ent><br/
<hw>Com*men"su*rate</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a common measure; commensurable; reducible to a common measure; <as>as, <ex>commensurate</ex> quantities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Equal in measure or extent; proportionate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who are persuaded that they shall continue forever, can not choose but aspire after a happiness <qex>commensurate</qex> to their duration.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurately</ent><br/
<hw>Com*men"su*rate*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a commensurate manner; so as to be equal or proportionate; adequately.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With equal measure or extent.</def> <rj><au>Goodwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensurateness</ent><br/
<hw>Com*men"su*rate*ness</hw>, <pos>n.</pos> <def>The state or quality of being commensurate.</def> <rj><au>Foster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commensuration</ent><br/
<hw>Com*men`su*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>commensuration</ets>.]</ety> <def>The act of commensurating; the state of being commensurate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All fitness lies in a particular <qex>commensuration</qex>, or proportion of one thing to another.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comment</ent><br/
<hw>Com"ment</hw> <pr>(?; 277)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commenting</conjf>.]</vmorph> <ety>[F. <ets>commenter</ets>, L. <ets>commentari</ets> to meditate upon, explain, v. intens. of <ets>comminisci</ets>, <ets>commentus</ets>, to reflect upon, invent; <ets>com-</ets> + the root of <ets>meminisse</ets> to remember, <ets>mens</ets> mind. See <er>Mind</er>.]</ety> <def>To make remarks, observations, or criticism; especially, to write notes on the works of an author, with a view to illustrate his meaning, or to explain particular passages; to write annotations; -- often followed by <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A physician to <qex>comment</qex> on your malady.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Critics . . . proceed to <qex>comment</qex> on him.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I must translate and <qex>comment</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comment</ent><br/
<hw>Com"ment</hw>, <pos>v. t.</pos> <def>To comment on.</def> <mark>[Archaic.]</mark> <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comment</ent><br/
<hw>Com"ment</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>comment</ets>.]</ety> <sn>1.</sn> <def>A remark, observation, or criticism; gossip; discourse; talk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their lavish <qex>comment</qex> when her name was named.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A note or observation intended to explain, illustrate, or criticise the meaning of a writing, book, etc.; explanation; annotation; exposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the volumes of philosophy,<br/
With all their <qex>comments</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentary</ent><br/
<hw>Com"men*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Commentaries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>commentarius</ets>, <ets>commentarium</ets>, note book, commentary: cf. F. <ets>commentaire</ets>. See <er>Comment</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A series of comments or annotations; esp., a book of explanations or expositions on the whole or a part of the Scriptures or of some other work.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This letter . . . was published by him with a severe <qex>commentary</qex>.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A brief account of transactions or events written hastily, as if for a memorandum; -- usually in the plural; <as>as, Caesar's <ex>Commentaries</ex> on the Gallic War</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentate</ent><br/
<hw>Com"men*tate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[L. <ets>commentatus</ets>, p. p. of <ets>commentari</ets> to meditate.]</ety> <def>To write comments or notes upon; to make comments.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commentate</qex> upon it, and return it enriched.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentation</ent><br/
<hw>Com`men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of commenting or criticising; exposition.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The spirit of <qex>commentation</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The result of the labors of a commentator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentator</ent><br/
<hw>Com"men*ta`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commentator</ets>: cf. F. <ets>commentateur</ets>.]</ety> <def>One who writes a commentary or comments; an expositor; an annotator.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>commentator's</qex> professed object is to explain, to enforce, to illustrate doctrines claimed as true.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentatorial</ent><br/
<hw>Com`men*ta*to"ri*al</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Pertaining to the making of commentaries.</def> <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentatorship</ent><br/
<hw>Com"men*ta`tor*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office or occupation of a commentator.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 285 --></p>

<p><ent>Commenter</ent><br/
<hw>Com"ment`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes or writes comments; a commentator; an annotator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commentitious</ent><br/
<hw>Com`men*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commentitius</ets>.]</ety> <def>Fictitious or imaginary; unreal; <as>as, a <ex>commentitious</ex> system of religion</as>.</def> <mark>[Obs.]</mark> <rj><au>Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commerce</ent><br/
<hw>Com"merce</hw> <pr>(?)</pr>, <pos>n.</pos> <note>(Formerly accented on the second syllable.)</note> <ety>[F. <ets>commerce</ets>, L. <ets>commercium</ets>; <ets>com-</ets> + <ets>merx</ets>, <ets>mercis</ets>, merchandise. See <er>Merchant</er>.]</ety> <sn>1.</sn> <def>The exchange or buying and selling of commodities; esp. the exchange of merchandise, on a large scale, between different places or communities; extended trade or traffic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The public becomes powerful in proportion to the opulence and extensive <qex>commerce</qex> of private men.</q> <rj><qau>Hume.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Social intercourse; the dealings of one person or class in society with another; familiarity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fifteen years of thought, observation, and <qex>commerce</qex> with the world had made him [Bunyan] wiser.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sexual intercourse.</def> <rj><au>W. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A round game at cards, in which the cards are subject to exchange, barter, or trade.</def> <rj><au>Hoyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chamber of commerce</b></col>. <cd>See <er>Chamber</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Trade; traffic; dealings; intercourse; interchange; communion; communication.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commerce</ent><br/
<hw>Com*merce"</hw> <pr>(? <or/ ?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commerced</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Commercing</conjf>.]</vmorph> <ety>[Cf. F. <ets>commercer</ets>, fr. LL. <ets>commerciare</ets>.]</ety> <sn>1.</sn> <def>To carry on trade; to traffic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Beware you <qex>commerce</qex> not with bankrupts.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hold intercourse; to commune.</def> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commercing</qex> with himself.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Musicians . . . taught the people in angelic harmonies to <qex>commerce</qex> with heaven.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commerce destroyer</ent><br/
<hw>Com"merce de*stroy"er</hw>. <fld>(Nav.)</fld> <def>A very fast, unarmored, lightly armed vessel designed to capture or destroy merchant vessels of an enemy. Not being intended to fight, they may be improvised from fast passenger steamers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Commercial</ent><br/
<hw>Com*mer"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>commercial</ets>.]</ety> <def>Of or pertaining to commerce; carrying on or occupied with commerce or trade; mercantile; <as>as, <ex>commercial</ex> advantages; <ex>commercial</ex> relations</as>.</def> <ldquo/Princely <xex>commercial houses</xex>.<rdquo/ <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commercial college</b></col>, <cd>a school for giving instruction in commercial knowledge and business.</cd> -- <col><b>Commercial law</b></col>. <cd>See under <er>Law</er>.</cd> -- <col><b>Commercial note paper</b></col>, <cd>a small size of writing paper, usually about 5 by 7<frac12/ or 8 inches.</cd> -- <col><b>Commercial paper</b></col>, <cd>negotiable paper given in due course of business. It includes bills of exchange, promissory notes, bank checks, etc.</cd> -- <col><b>Commercial traveler</b></col>, <cd>an agent of a wholesale house who travels from town to town to solicit orders.</cd></cs></p>

<p><syn><b>Syn.</b> -- See <er>Mercantile</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>commercialisation</ent><br/
<hw>commercialisation</hw> <pos>n.</pos> <def>same as <er>commercialization</er>.</def> <mark>[Brit.]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>commercialise</ent><br/
<hw>commercialise</hw> <pos>n.</pos> <def>same as <er>commercialize</er>.</def> <mark>[Brit.]</mark><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Commercialism</ent><br/
<hw>Com*mer"cial*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The commercial spirit or method.</def> <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>commercialization</ent><br/
<hw>commercialization</hw> <pos>n.</pos> <def>the act or process of making something commercial in character.  See <er>commercialize</er>.</def> <altsp>[Also spelled <asp>commercialisation</asp>.]</altsp><br/
<syn><b>Syn. --</b> .</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>commercialize</ent><br/
<hw>commercialize</hw> <pos>n.</pos> <def>to make something commercial in character, either by placing it for sale on the open market, or by emphasizing its profit-making aspects; <as>as, the Olympics have been excessively <ex>commercialized</ex></as>.</def> <altsp>[Also spelled <asp>commercialise</asp>.]</altsp><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>commercialized</ent><br/
<hw>commercialized</hw> <pos>adj.</pos> <sn>1.</sn> <def>organized principally for financial gain.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>having its focus changed from non-commercial to commercial; <as>as, Christmas has become a <ex>commercialized</ex> spectacle</as>.</def><br/
<syn><b>Syn. --</b> commercialised.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Commercially</ent><br/
<hw>Com*mer"cial*ly</hw>, <pos>adv.</pos> <def>In a commercial manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commigrate</ent><br/
<hw>Com"mi*grate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>commigrare</ets>, <ets>commigratum</ets>.]</ety> <def>To migrate together.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commigration</ent><br/
<hw>Com`mi*gra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commigratio</ets>.]</ety> <def>Migration together.</def> <mark>[R.]</mark> <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commination</ent><br/
<hw>Com`mi*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comminatio</ets>, from <ets>comminari</ets> to threaten; <ets>com-</ets> + <ets>minari</ets> to threaten: cf. F. <ets>commination</ets>.]</ety> <sn>1.</sn> <def>A threat or threatening; a denunciation of punishment or vengeance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With terrible <qex>comminations</qex> to all them that did resist.</q> <rj><qau>Foxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those thunders of <qex>commination</qex>.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An office in the liturgy of the Church of England, used on Ash Wednesday, containing a recital of God's anger and judgments against sinners.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comminatory</ent><br/
<hw>Com*min"a*to"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>comminatoire</ets>.]</ety> <def>Threatening or denouncing punishment; <as>as, <ex>comminatory</ex> terms</as>.</def> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commingle</ent><br/
<hw>Com*min"gle</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commingled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Commingling</conjf> <pr>(?)</pr>.]</vmorph> <def>To mingle together; to mix in one mass, or intimately; to blend.</def> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commingler</ent><br/
<hw>Com*min"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One that commingles; specif., a device for noiseless heating of water by steam, in a vessel filled with a porous mass, as of pebbles.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Comminute</ent><br/
<hw>Com"mi*nute</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Comminuted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Comminuting</conjf>.]</vmorph> <ety>[L. <ets>comminutus</ets>, p. p. of <ets>comminuere</ets> to comminute; <ets>com-</ets> + <ets>minuere</ets> to lessen. See <er>Minute</er>.]</ety> <def>To reduce to minute particles, or to a fine powder; to pulverize; to triturate; to grind; <as>as, to <ex>comminute</ex> chalk or bones; to <ex>comminute</ex> food with the teeth</as>.</def> <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Comminuted fracture</b></col>. <cd>See under <er>Fracture</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comminution</ent><br/
<hw>Com`mi*nu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of reducing to a fine powder or to small particles; pulverization; the state of being comminuted.</def> <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>Fracture (of a bone) into a number of pieces.</def> <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Gradual diminution by the removal of small particles at a time; a lessening; a wearing away.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Natural and necessary <qex>comminution</qex> of our lives.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commiserable</ent><br/
<hw>Com*mis"er*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pitiable.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commiserate</ent><br/
<hw>Com*mis"er*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commiserated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commiserating</conjf>.]</vmorph> <ety>[L. <ets>commiseratus</ets>, p. p. of <ets>commiserari</ets> to commiserate; <ets>com-</ets> + <ets>miserari</ets> to pity. See <er>Miserable</er>.]</ety> <def>To feel sorrow, pain, or regret for; to pity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then must we those, who groan, beneath the weight<br/
Of age, disease, or want, <qex>commiserate</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We should <qex>commiserate</qex> our mutual ignorance.</q> <rj><qau>Locke.</qau></rj></p>

<p><syn><b>Syn.</b> -- To pity; compassionate; lament; condole.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commiseration</ent><br/
<hw>Com*mis`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>commis<eacute/ration</ets>, fr. L. <ets>commiseratio</ets> a part of an oration intended to excite compassion.]</ety> <def>The act of commiserating; sorrow for the wants, afflictions, or distresses of another; pity; compassion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And pluck <qex>commiseration</qex> of his state<br/
From brassy bosoms and rough hearts of flint.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Sympathy</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commiserative</ent><br/
<hw>Com*mis"er*a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Feeling or expressing commiseration.</def> <rj><au>Todd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commiserator</ent><br/
<hw>Com*mis"er*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who pities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissarial</ent><br/
<hw>Com`mis*sa"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a commissary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissariat</ent><br/
<hw>Com`mis*sa"ri*at</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>commissariat</ets>.]</ety> <fld>(Mil.)</fld> <sd>(a)</sd> <def>The organized system by which armies and military posts are supplied with food and daily necessaries.</def> <sd>(b)</sd> <def>The body of officers charged with such service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissary</ent><br/
<hw>Com"mis*sa*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Commissaries</plw> <pr>(#)</pr>.</plu> <ety>[LL. <ets>commissarius</ets>, fr. L. <ets>commissus</ets>, p. p. of <ets>committere</ets> to commit, intrust to. See <er>Commit</er>.]</ety> <sn>1.</sn> <def>One to whom is committed some charge, duty, or office, by a superior power; a commissioner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great Destiny, the <qex>Commissary</qex> of God.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>An officer of the bishop, who exercises ecclesiastical jurisdiction in parts of the diocese at a distance from the residence of the bishop.</def> <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>An officer having charge of a special service; <as>as, the <ex>commissary</ex> of musters</as>.</def> <sd>(b)</sd> <def>An officer whose business is to provide food for a body of troops or a military post; -- officially called <altname>commissary of subsistence</altname>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Washington wrote to the President of Congress . . . urging the appointment of a commissary general, a quartermaster general, a <qex>commissary</qex> of musters, and a <qex>commissary</qex> of artillery.</q> <rj><qau>W. Irving</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commissary general</b></col>, <cd>an officer in charge of some special department of army service</cd>; as: <sd>(a)</sd> <cd>The officer in charge of the commissariat and transport department, or of the ordnance store department.</cd> <mark>[Eng.]</mark> <sd>(b)</sd> <cd>The commissary general of subsistence.</cd> <mark>[U. S.]</mark> -- <col><b>Commissary general of subsistence</b></col> <fld>(Mil. U. S.)</fld>, <cd>the head of the subsistence department, who has charge of the purchase and issue of provisions for the army.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissaryship</ent><br/
<hw>Com"mis*sa*ry*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office or employment of a commissary.</def> <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commission</ent><br/
<hw>Com*mis"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>commissio</ets>. See <er>Commit</er>.]</ety> <sn>1.</sn> <def>The act of committing, doing, or performing; the act of perpetrating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every <qex>commission</qex> of sin introduces into the soul a certain degree of hardness.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of intrusting; a charge; instructions as to how a trust shall be executed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The duty or employment intrusted to any person or persons; a trust; a charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A formal written warrant or authority, granting certain powers or privileges and authorizing or commanding the performance of certain duties.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let him see our <qex>commission</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A certificate conferring military or naval rank and authority; <as>as, a colonel's <ex>commission</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A company of persons joined in the performance of some duty or the execution of some trust; <as>as, the interstate commerce <ex>commission</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>commission</qex> was at once appointed to examine into the matter.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Com.)</fld> <sd>(a)</sd> <def>The acting under authority of, or on account of, another.</def> <sd>(b)</sd> <def>The thing to be done as agent for another; <as>as, I have three <ex>commissions</ex> for the city</as>.</def> <sd>(c)</sd> <def>The brokerage or allowance made to a factor or agent for transacting business for another; <as>as, a <ex>commission</ex> of ten per cent on sales</as>. See <er>Del credere</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commission of array</b></col>. <fld>(Eng. Hist.)</fld> <cd>See under <er>Array</er>.</cd> -- <col><b>Commission of bankruptcy</b></col>, <cd>a commission appointing and empowering certain persons to examine into the facts relative to an alleged bankruptcy, and to secure the bankrupt's lands and effects for the creditors.</cd> -- <col><b>Commission of lunacy</b></col>, <cd>a commission authorizing an inquiry whether a person is a lunatic or not.</cd> -- <col><b>Commission merchant</b></col>, <cd>one who buys or sells goods on commission, as the agent of others, receiving a rate per cent as his compensation.</cd> -- <mcol><col><b>Commission officer</b></col> <it>or</it>  <col><b>Commissioned officer</b></col></mcol>,  <fld>(Mil.)</fld>, <cd>one who has a commission, in distinction from a noncommissioned or warrant officer.</cd> -- <col><b>Commission of the peace</b></col>, <cd>a commission under the great seal, constituting one or more persons justices of the peace.</cd> <mark>[Eng.]</mark> -- <col><b>on commission</b></col>, <cd>paid partly or completely by collecting as a commision a portion of the sales that one makes.</cd> -- <col><b>out of commission</b></col>, <cd>not operating properly; out of order.</cd> -- <col><b>To put a vessel into commission</b></col> <fld>(Naut.)</fld>, <cd>to equip and man a government vessel, and send it out on service after it has been laid up; esp., <-- to perform ? PCP --> the formal act of taking command of a vessel for service, hoisting the flag, reading the orders, etc.</cd> -- <col><b>To put a vessel out of commission</b></col> <fld>(Naut.)</fld>, <cd>to detach the officers and crew and retire it from active service, temporarily or permanently.</cd> -- <mcol><col><b>To put the great seal into commission</b></col> <it>or</it> <col><b>To put the Treasury into commission</b></col></mcol>, <cd>to place it in the hands of a commissioner or commissioners during the abeyance of the ordinary administration, as between the going out of one lord keeper and the accession of another.</cd> <mark>[Eng.]</mark> -- <col><b>The United States Christian Commission</b></col>, <cd>an organization among the people of the North, during the Civil War, which afforded material comforts to the Union soldiers, and performed services of a religious character in the field and in hospitals.</cd> -- <col><b>The United States Sanitary Commission</b></col>, <cd>an organization formed by the people of the North to cooperate with and supplement the medical department of the Union armies during the Civil War.</cd></cs></p>

<p><syn><b>Syn.</b> -- Charge; warrant; authority; mandate; office; trust; employment.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commission</ent><br/
<hw>Com*mis"sion</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commissioned</conjf> <pr>(?)</pr>; <pos>p. pr & vb. n.</pos> <conjf>Commissioning</conjf>.]</vmorph> <sn>1.</sn> <def>To give a commission to; to furnish with a commission; to empower or authorize; <as>as, to <ex>commission</ex> persons to perform certain acts; to <ex>commission</ex> an officer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To send out with a charge or commission.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A chosen band<br/
He first <qex>commissions</qex> to the Latian land.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- To appoint; depute; authorize; empower; delegate; constitute; ordain.</syn></p>

<p><ent>Commissionaire</ent><br/
<hw>Com*mis`sion*aire"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>commissionnaire</ets>. Cf. <er>Commissioner</er>.]</ety> <sn>1.</sn> <def>One intrusted with a commission, now only a small commission, as an errand; esp., an attendant or subordinate employee in a public office, hotel, or the like.</def> <note>The <ex>commissionaire</ex> familiar to European travelers performs miscellaneous services as a light porter, messenger, solicitor for hotels, etc.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>One of a corps of pensioned soldiers, as in London, employed as doorkeepers, messengers, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>a uniformed doorman.</def> <mark>[British]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Commissionary</ent><br/
<ent>Commissional</ent><br/
<mhw>{ <hw>Com*mis"sion*al</hw> <pr>(?)</pr>, <hw>Com*mis"sion*a*ry</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <def>Of, pertaining to, or conferring, a commission; conferred by a commission or warrant.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Delegate or <qex>commissionary</qex> authority.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissionate</ent><br/
<hw>Com*mis"sion*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To commission</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>commissioned</ent><br/
<hw>commissioned</hw> <pos>adj.</pos> <sn>1.</sn> <fld>(Mil.)</fld> <def>holding by virtue of a commission a rank of second lieutenant or ensign or above; -- of military officers.  Contrasts with <ant>noncommissioned</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>given official approval to act; <as>as, <ex>commissioned</ex> broker</as>.</def><br/
<syn><b>Syn. --</b> accredited, licensed, licenced.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>created according to an order; <as>as, a <ex>commissioned</ex> painting</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Commissioner</ent><br/
<hw>Com*mis"sion*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person who has a commission or warrant to perform some office, or execute some business, for the government, corporation, or person employing him; <as>as, a <ex>commissioner</ex> to take affidavits or to adjust claims</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To another address which requested that a commission might be sent to examine into the state of things in Ireland, William returned a gracious answer, and desired the Commons to name the <qex>commissioners</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An officer having charge of some department or bureau of the public service.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Herbert was first <qex>commissioner</qex> of the Admiralty.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>commissioner</qex> of patents, the <qex>commissioner</qex> of the land office, the <qex>commissioner</qex> of Indian affairs, are subordinates of the secretary of the interior.</q> <rj><qau>Bartlett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commissioner of deeds</b></col>, <cd>an officer having authority to take affidavits, depositions, acknowledgment of deeds, etc., for use in the State by which he is appointed.</cd> <mark>[U. S.]</mark> -- <col><b>County commissioners</b></col>, <cd>certain administrative officers in some of the States, invested by local laws with various powers in reference to the roads, courthouses, financial matters, etc., of the county.</cd> <mark>[U. S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissionnaire</ent><br/
||<hw>Com*mis`sion*naire"</hw> <pr>(?; F. ?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>commissio</ets>.]</ety> <sn>1.</sn> <def>An agent or factor; a commission merchant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of a class of attendants, in some European cities, who perform miscellaneous services for travelers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissionship</ent><br/
<hw>Com*mis"sion*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office of commissioner.</def> <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissive</ent><br/
<hw>Com*mis"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to commission; of the nature of, or involving, commission.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissural</ent><br/
<hw>Com*mis"su*ral</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Of or pertaining to a commissure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commissure</ent><br/
<hw>Com*mis"sure</hw> <pr>(? <or/ ?; 134-6)</pr>, <pos>n.</pos> <ety>[L. <ets>commissura</ets> a joining together: cf. F. <ets>commissure</ets>. See <er>Commit</er>.]</ety> <sn>1.</sn> <def>A joint, seam, or closure; the place where two bodies, or parts of a body, meet and unite; an interstice, cleft, or juncture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat. & Zool.)</fld> <sd>(a)</sd> <def>The point of union between two parts, as the angles of the lips or eyelids, the mandibles of a bird, etc.</def> <sd>(b)</sd> <def>A collection of fibers connecting parts of the brain or spinal marrow; a chiasma.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>The line of junction or cohering face of two carpels, as in the parsnip, caraway, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commit</ent><br/
<hw>Com*mit"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Committed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Committing</conjf>.]</vmorph> <ety>[L. <ets>committere</ets>, <ets>commissum</ets>, to connect, commit; <ets>com-</ets> + <ets>mittere</ets> to send. See <er>Mission</er>.]</ety> <sn>1.</sn> <def>To give in trust; to put into charge or keeping; to intrust; to consign; -- used with <xex>to</xex>, <xex>unto</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commit</qex> thy way unto the Lord.</q> <rj><qau>Ps. xxxvii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid him farewell, <qex>commit</qex> him to the grave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put in charge of a jailor; to imprison.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These two were <qex>committed</qex>.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To do; to perpetrate, as a crime, sin, or fault.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt not <qex>commit</qex> adultery.</q> <rj><qau>Ex. xx. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To join for a contest; to match; -- followed by <xex>with</xex>.</def> <mark>[R.]</mark> <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To pledge or bind; to compromise, expose, or endanger by some decisive act or preliminary step; -- often used reflexively; <as>as, to <ex>commit</ex> one's self to a certain course</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You might have satisfied every duty of political friendship, without <qex>commiting</qex> the honor of your sovereign.</q> <rj><qau>Junius.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Any sudden assent to the proposal . . . might possibly be considered as <qex>committing</qex> the faith of the United States.</q> <rj><qau>Marshall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To confound.</def> <mark>[An obsolete Latinism.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Committing</qex> short and long [quantities].</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To commit a bill</b></col> <fld>(Legislation)</fld>, <cd>to refer or intrust it to a committee or others, to be considered and reported.</cd> -- <mcol><col><b>To commit to memory</b></col>, <it>or</it> <col><b>To commit</b></col></mcol>, <cd>to learn by heart; to memorize.</cd></cs></p>

<p><syn><b>Syn.</b> -- <er>To Commit</er>, <er>Intrust</er>, <er>Consign</er>.</syn> <usage> These words have in common the idea of transferring from one's self to the care and custody of another. <xex>Commit</xex> is the widest term, and may express only the general idea of delivering into the charge of another; as, to <xex>commit</xex> a lawsuit to the care of an attorney; or it may have the special sense of intrusting with or without limitations, as to a superior power, or to a careful servant, or of consigning, as to writing or paper, to the flames, or to prison. To <xex>intrust</xex> denotes the act of committing to the exercise of confidence or trust; as, to <xex>intrust</xex> a friend with the care of a child, or with a secret. To <xex>consign</xex> is a more formal act, and regards the thing transferred as placed chiefly or wholly out of one's immediate control; as, to <xex>consign</xex> a pupil to the charge of his instructor; to <xex>consign</xex> goods to an agent for sale; to <xex>consign</xex> a work to the press.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commit</ent><br/
<hw>Com"mit</hw>, <pos>v. i.</pos> <def>To sin; esp., to be incontinent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commit</qex> not with man's sworn spouse.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commitment</ent><br/
<hw>Com*mit"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of committing, or putting in charge, keeping, or trust; consignment; esp., the act of committing to prison.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They were glad to compound for his bare <qex>commitment</qex> to the Tower, whence he was within few days enlarged.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A warrant or order for the imprisonment of a person; -- more frequently termed a <xex>mittimus</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of referring or intrusting to a committee for consideration and report; <as>as, the <ex>commitment</ex> of a petition or a bill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A doing, or perpetration, in a bad sense, as of a crime or blunder; commission.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The act of pledging or engaging; the act of exposing, endangering, or compromising; also, the state of being pledged or engaged.</def> <rj><au>Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Committable</ent><br/
<hw>Com*mit"ta*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being committed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Committal</ent><br/
<hw>Com*mit"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of committing, or the state of being committed; commitment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>committed</ent><br/
<hw>committed</hw> <pos>adj.</pos> <sn>1.</sn> <def>Bound or obligated, as under a pledge to a particular cause, action, or attitude.  Opposite of <ant>uncommitted</ant>.</def> <note>[Narrower terms: <stype>bound up, involved, wrapped up</stype>; <stype>dedicated, devoted</stype>; <stype>pledged, sworn</stype>]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>Associated in an exclusive sexual relationship; also called <altname>attached</altname>.  Opposite of <ant>unattached</ant>.</def> <note>[Narrower terms: <stype>affianced</stype>, <stype>bespoken</stype>, <stype>betrothed</stype>, <stype>engaged</stype>, <stype>pledged</stype>, <stype>promised</stype>(predicate); <stype>married</stype>]</note> <see>[Also See: <er>loving</er>.]</see><br/
<syn><b>Syn. --</b> attached.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>Consigned involuntarily to custody, as in a prison or mental institution.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>commitedness</ent><br/
<hw>commitedness</hw> <pos>n.</pos> <def>the trait of sincere and steadfast fixity of purpose.</def><br/
<syn><b>Syn. --</b> commitment.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Committee</ent><br/
<hw>Com*mit"tee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>comit<eacute/</ets> company, and LL. <ets>comitatus</ets> jurisdiction or territory of a count, county, assize, army. The word was apparently influenced by the verb <ets>commit</ets>, but not directly formed from it. Cf. <er>County</er>.]</ety> <def>One or more persons elected or appointed, to whom any matter or business is referred, either by a legislative body, or by a court, or by any collective body of men acting together.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Committee of the whole [house]</b></col>, <cd>a committee, embracing all the members present, into which a legislative or deliberative body sometimes resolves itself, for the purpose of considering a particular measure under the operation of different rules from those governing the general legislative proceedings. The <ex>committee of the whole</ex> has its own chairman, and reports its action in the form of recommendations.</cd> -- <col><b>Standing committee</b></col>. <cd>See under <er>Standing</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 286 --></p>

<p><ent>Committee</ent><br/
<hw>Com`mit*tee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Commit</er>, <pos>v. t.</pos>]</ety> <fld>(Law)</fld> <def>One to whom the charge of the person or estate of another, as of a lunatic, is committed by suitable authority; a guardian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Committeeman</ent><br/
<hw>Com*mit"tee*man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member of a committee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Committer</ent><br/
<hw>Com*mit"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who commits; one who does or perpetrates.</def> <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fornicator.</def> <mark>[Obs.]</mark> <rj><au>T. Decker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Committible</ent><br/
<hw>Com*mit"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being committed; liable to be committed.</def> <mark>[R.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commix</ent><br/
<hw>Com*mix"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commixed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Commixing</conjf>.]</vmorph> <ety>[Pref. <ets>com-</ets> + <ets>mix</ets>: cf. L. <ets>commixtus</ets>, p. p. of <ets>commiscere</ets>. See <er>Mix</er>.]</ety> <def>To mix or mingle together; to blend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>commixed</qex> impressions of all the colors do stir up and beget a sensation of white.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>commix</qex><br/
With winds that sailors rail at.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commixion</ent><br/
<hw>Com*mix"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Commix</er>.]</ety> <def>Commixture.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commixtion</ent><br/
<hw>Com*mix"tion</hw> <pr>(?; 106)</pr>, <pos>n.</pos> <ety>[L. <ets>commixtio</ets>.]</ety> <def>Commixture; mingling.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>An exact <qex>commixtion</qex> of the ingredients.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commixture</ent><br/
<hw>Com*mix"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>commixtura</ets>.]</ety> <sn>1.</sn> <def>The act or process of mixing; the state of being mingled; the blending of ingredients in one mass or compound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>commixture</qex> of anything that is more oily or sweet, such bodies are least apt to putrefy.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The mass formed by mingling different things; a compound; a mixture.</def> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commodate</ent><br/
<hw>Com"mo*date</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commodatum</ets> thing lent, loan.]</ety> <fld>(Scots Law)</fld> <def>A gratuitous loan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commode</ent><br/
<hw>Com*mode"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>commode</ets>, fr. <ets>commode</ets> convenient, L. <ets>commodus</ets>; <ets>com-</ets> + <ets>modus</ets> measure, mode. See <er>Mode</er>.]</ety> <sn>1.</sn> <def>A kind of headdress formerly worn by ladies, raising the hair and fore part of the cap to a great height.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or under high <qex>commodes</qex>, with looks erect.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of furniture, so named according to temporary fashion</def>; as: <sd>(a)</sd> <def>A chest of drawers or a bureau.</def> <sd>(b)</sd> <def>A night stand with a compartment for holding a chamber vessel.</def> <sd>(c)</sd> <def>A kind of close stool.</def> <sd>(d)</sd> <def>A movable sink or stand for a wash bowl, with closet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commodious</ent><br/
<hw>Com*mo"di*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>commodiosus</ets>, fr. L. <ets>commodum</ets> convenience, fr. <ets>commodus</ets>. See <er>Commode</er>.]</ety> <def>Adapted to its use or purpose, or to wants and necessities; serviceable; spacious and convenient; roomy and comfortable; <as>as, a <ex>commodious</ex> house</as>.</def> <ldquo/A <xex>commodious</xex> drab.<rdquo/ <au>Shak.</au> <ldquo/<xex>Commodious</xex> gold.<rdquo/ <au>Pope.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>The haven was not <qex>commodious</qex> to winter in.</q> <rj><qau>Acts xxvii. 12.</qau></rj></p>

<p><syn><b>Syn.</b> -- Convenient; suitable; fit; proper; advantageous; serviceable; useful; spacious; comfortable.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commodiously</ent><br/
<hw>Com*mo"di*ous*ly</hw>, <pos>adv.</pos> <def>In a commodious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To pass <qex>commodiously</qex> this life.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commodiousness</ent><br/
<hw>Com*mo"di*ous*ness</hw>, <pos>n.</pos> <def>State of being commodious; suitableness for its purpose; convenience; roominess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of cities, the greatness and riches increase according to the <qex>commodiousness</qex> of their situation.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>commodiousness</qex> of the harbor.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commodity</ent><br/
<hw>Com*mod"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Commodities</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>commodit<eacute/</ets>, fr. L. <ets>commoditas</ets>. See <er>Commode</er>.]</ety> <sn>1.</sn> <def>Convenience; accommodation; profit; benefit; advantage; interest; commodiousness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Drawn by the <qex>commodity</qex> of a footpath.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Men may seek their own <qex>commodity</qex>, yet if this were done with injury to others, it was not to be suffered.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which affords convenience, advantage, or profit, especially in commerce, including everything movable that is bought and sold (except animals), -- goods, wares, merchandise, produce of land and manufactures, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A parcel or quantity of goods.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>commodity</qex> of brown paper and old ginger.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commodore</ent><br/
<hw>Com"mo*dore`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. a corruption of <ets>commander</ets>, or Sp. <ets>comendador</ets> a knight of a military order who holds a commandery; also a superior of a monastery, fr. LL. <ets>commendare</ets> to command. Cf. <er>Commend</er>, <er>Command</er>, <er>Commander</er>.]</ety> <sn>1.</sn> <fld>(U. S. Navy)</fld> <def>An officer who ranks next above a captain; sometimes, by courtesy, the senior captain of a squadron. The rank of commodore corresponds with that of brigadier general in the army.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(British Navy)</fld> <def>A captain commanding a squadron, or a division of a fleet, or having the temporary rank of rear admiral.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A title given by courtesy to the senior captain of a line of merchant vessels, and also to the chief officer of a yachting or rowing club.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A familiar for the flagship, or for the principal vessel of a squadron or fleet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Common</ent><br/
<hw>Com"mon</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Commoner</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Commonest</adjf>.]</amorph> <ety>[OE. <ets>commun</ets>, <ets>comon</ets>, OF. <ets>comun</ets>, F. <ets>commun</ets>, fr. L. <ets>communis</ets>; <ets>com-</ets> + <ets>munis</ets> ready to be of service; cf. Skr. <ets>mi</ets> to make fast, set up, build, Goth. <ets>gamains</ets> common, G. <ets>gemein</ets>, and E. <ets>mean</ets> low, common. Cf. <er>Immunity</er>, <er>Commune</er>, <pos>n.</pos> & <pos>v.</pos>]</ety> <sn>1.</sn> <def>Belonging or relating equally, or similarly, to more than one; <as>as, you and I have a <ex>common</ex> interest in the property</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though life and sense be <qex>common</qex> to men and brutes.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Belonging to or shared by, affecting or serving, all the members of a class, considered together; general; public; <as>as, properties <ex>common</ex> to all plants; the <ex>common</ex> schools; the Book of <ex>Common</ex> Prayer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such actions as the <qex>common</qex> good requireth.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>common</qex> enemy of man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Often met with; usual; frequent; customary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Grief more than <qex>common</qex> grief.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not distinguished or exceptional; inconspicuous; ordinary; plebeian; -- often in a depreciatory sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The honest, heart-felt enjoyment of <qex>common</qex> life.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This fact was infamous<br/
And ill beseeming any <qex>common</qex> man,<br/
Much more a knight, a captain and a leader.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Above the vulgar flight of <qex>common</qex> souls.</q> <rj><qau>A. Murphy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Profane; polluted.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What God hath cleansed, that call not thou <qex>common</qex>.</q> <rj><qau>Acts x. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Given to habits of lewdness; prostitute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A dame who herself was <qex>common</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Common bar</b></col> <fld>(Law)</fld> <cd>Same as <cref>Blank bar</cref>, under <er>Blank</er>.</cd> -- <col><b>Common barrator</b></col> <fld>(Law)</fld>, <cd>one who makes a business of instigating litigation.</cd> -- <col><b>Common Bench</b></col>, <cd>a name sometimes given to the English Court of Common Pleas.</cd> -- <col><b>Common brawler</b></col> <fld>(Law)</fld>, <cd>one addicted to public brawling and quarreling. See <er>Brawler</er>.</cd> -- <col><b>Common carrier</b></col> <fld>(Law)</fld>, <cd>one who undertakes the office of carrying (goods or persons) for hire. Such a carrier is bound to carry in all cases when he has accommodation, and when his fixed price is tendered, and he is liable for all losses and injuries to the goods, except those which happen in consequence of the act of God, or of the enemies of the country, or of the owner of the property himself.</cd> -- <col><b>Common chord</b></col> <fld>(Mus.)</fld>, <cd>a chord consisting of the fundamental tone, with its third and fifth.</cd> -- <col><b>Common council</b></col>, <cd>the representative (legislative) body, or the lower branch of the representative body, of a city or other municipal corporation.</cd> -- <col><b>Common crier</b></col>, <cd>the crier of a town or city.</cd> -- <col><b>Common divisor</b></col> <fld>(Math.)</fld>, <cd>a number or quantity that divides two or more numbers or quantities without a remainder; a common measure.</cd> -- <col><b>Common gender</b></col> <fld>(Gram.)</fld>, <cd>the gender comprising words that may be of either the masculine or the feminine gender.</cd> -- <col><b>Common law</b></col>, <cd>a system of jurisprudence developing under the guidance of the courts so as to apply a consistent and reasonable rule to each litigated case. It may be superseded by statute, but unless superseded it controls.</cd> <au>Wharton.</au> <note>It is by others defined as the unwritten law (especially of England), the law that receives its binding force from immemorial usage and universal reception, as ascertained and expressed in the judgments of the courts. This term is often used in contradistinction from <contr>statute law</contr>. Many use it to designate a law <xex>common</xex> to the whole country. It is also used to designate the whole body of English (or other) law, as distinguished from its subdivisions, local, civil, admiralty, equity, etc. See <er>Law</er>.</note> -- <col><b>Common lawyer</b></col>, <cd>one versed in common law.</cd> -- <col><b>Common lewdness</b></col> <fld>(Law)</fld>, <cd>the habitual performance of lewd acts in public.</cd> -- <col><b>Common multiple</b></col> <fld>(Arith.)</fld> <cd>See under <er>Multiple</er>.</cd> -- <col><b>Common noun</b></col> <fld>(Gram.)</fld>, <cd>the name of any one of a class of objects, as distinguished from a <xex>proper noun</xex> (the name of a particular person or thing).</cd> -- <col><b>Common nuisance</b></col> <fld>(Law)</fld>, <cd>that which is deleterious to the health or comfort or sense of decency of the community at large.</cd> -- <col><b>Common pleas</b></col>, <cd>one of the three superior courts of common law at Westminster, presided over by a chief justice and four puisne judges. Its jurisdiction is confined to civil matters. Courts bearing this title exist in several of the United States, having, however, in some cases, both civil and criminal jurisdiction extending over the whole State. In other States the jurisdiction of the common pleas is limited to a county, and it is sometimes called a <altname>county court</altname>. Its powers are generally defined by statute.</cd> -- <col><b>Common prayer</b></col>, <cd>the liturgy of the Church of England, or of the Protestant Episcopal church of the United States, which all its clergy are enjoined to use. It is contained in the Book of Common Prayer.</cd> -- <col><b>Common school</b></col>, <cd>a school maintained at the public expense, and open to all.</cd> -- <col><b>Common scold</b></col> <fld>(Law)</fld>, <cd>a woman addicted to scolding indiscriminately, in public.</cd> -- <col><b>Common seal</b></col>, <cd>a seal adopted and used by a corporation.</cd> -- <col><b>Common sense</b></col>. <sd>(a)</sd> <cd>A supposed sense which was held to be the common bond of all the others.</cd> <mark>[Obs.]</mark> <au>Trench.</au> <sd>(b)</sd> <cd>Sound judgment. See under <er>Sense</er>.</cd> -- <col><b>Common time</b></col> <fld>(Mus.)</fld>, <cd>that variety of time in which the measure consists of two or of four equal portions.</cd> -- <col><b>In common</b></col>, <cd>equally with another, or with others; owned, shared, or used, in community with others; affecting or affected equally.</cd> -- <col><b>Out of the common</b></col>, <cd>uncommon; extraordinary.</cd> -- <col><b>Tenant in common</b></col>, <cd>one holding real or personal property in common with others, having distinct but undivided interests. See <cref>Joint tenant</cref>, under <er>Joint</er>.</cd> -- <col><b>To make common cause with</b></col>, <cd>to join or ally one's self with.</cd></cs></p>

<p><syn><b>Syn.</b> -- General; public; popular; national; universal; frequent; ordinary; customary; usual; familiar; habitual; vulgar; mean; trite; stale; threadbare; commonplace. See <er>Mutual</er>, <er>Ordinary</er>, <er>General</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Common</ent><br/
<hw>Com"mon</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The people; the community.</def> <mark>[Obs.]</mark> <ldquo/The weal o' the <xex>common</xex>.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inclosed or uninclosed tract of ground for pleasure, for pasturage, etc., the use of which belongs to the public; or to a number of persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>The right of taking a profit in the land of another, in common either with the owner or with other persons; -- so called from the community of interest which arises between the claimant of the right and the owner of the soil, or between the claimants and other commoners entitled to the same right.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Common appendant</b></col>, <cd>a right belonging to the owners or occupiers of arable land to put commonable beasts upon the waste land in the manor where they dwell.</cd> -- <col><b>Common appurtenant</b></col>, <cd>a similar right applying to lands in other manors, or extending to other beasts, besides those which are generally commonable, as hogs.</cd> -- <mcol><col><b>Common because of vicinage</b></col> <it>or</it> <col><b>Common because of neighborhood</b></col></mcol>, <cd>the right of the inhabitants of each of two townships, lying contiguous to each other, which have usually intercommoned with one another, to let their beasts stray into the other's fields.</cd> - - <mcol><col><b>Common in gross</b></col> <it>or</it> <col><b>Common at large</b></col></mcol>, <cd>a common annexed to a man's person, being granted to him and his heirs by deed; or it may be claimed by prescriptive right, as by a parson of a church or other corporation sole.</cd> <au>Blackstone.</au> -- <col><b>Common of estovers</b></col>, <cd>the right of taking wood from another's estate.</cd> -- <col><b>Common of pasture</b></col>, <cd>the right of feeding beasts on the land of another.</cd> <au>Burill.</au> -- <col><b>Common of piscary</b></col>, <cd>the right of fishing in waters belonging to another.</cd> -- <col><b>Common of turbary</b></col>, <cd>the right of digging turf upon the ground of another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Common</ent><br/
<hw>Com"mon</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To converse together; to discourse; to confer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Embassadors were sent upon both parts, and divers means of entreaty were <qex>commoned</qex> of.</q> <rj><qau>Grafton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To participate.</def> <mark>[Obs.]</mark> <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have a joint right with others in common ground.</def> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To board together; to eat at a table in common.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonable</ent><br/
<hw>Com"mon*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Held in common.</def> <ldquo/Forests . . . and other <xex>commonable</xex> places.<rdquo/ <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Allowed to pasture on public commons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commonable</qex> beasts are either beasts of the plow, or such as manure the ground.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonage</ent><br/
<hw>Com"mon*age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>communage</ets>.]</ety> <def>The right of pasturing on a common; the right of using anything in common with others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The claim of <qex>commonage</qex> . . . in most of the forests.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>commonality</ent><br/
<hw>com`mon*al"i*ty</hw> <pos>n.</pos> <sn>1.</sn> <def>the sharing of common attributes.</def><br/
<syn><b>Syn. --</b> commonness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a characteristic held in common; a common feature.</def><br/
[<source>PJC</source>]</p>

<p><ent>commonality</ent><br/
<ent>commonalty</ent><br/
<mhw><hw>com"mon*al*ty</hw>, <hw>com`mon*al"i*ty</hw></mhw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Commonalties</plw> <pr>(#)</pr>.</plu> <ety>[OF. <ets>communalt<eacute/</ets>; F. <ets>communaut<eacute/</ets>, fr. <ets>communal</ets>. See <er>Communal</er>.]</ety> <sn>1.</sn> <def>The common people; those classes and conditions of people who are below the rank of nobility; the commons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>commonalty</qex>, like the nobility, are divided into several degrees.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ancient fare of our kings differed from that of the <qex>commonalty</qex> in plenteousness only.</q> <rj><qau>Landon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The majority or bulk of mankind.</def> <mark>[Obs.]</mark> <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commoner</ent><br/
<hw>Com"mon*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One of the common people; one having no rank of nobility.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All below them [the peers] even their children, were <qex>commoners</qex>, and in the eye of the law equal to each other.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of the House of Commons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who has a joint right in common ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Much good land might be gained from forests . . . and from other commonable places, so as always there be a due care taken that the poor <qex>commoners</qex> have no injury.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One sharing with another in anything.</def> <mark>[Obs.]</mark> <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A student in the university of Oxford, Eng., who is not dependent on any foundation for support, but pays all university charges; - - at Cambridge called a <altname>pensioner</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A prostitute.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonish</ent><br/
<hw>Com"mon*ish</hw>, <pos>a.</pos> <def>Somewhat common; commonplace; vulgar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonition</ent><br/
<hw>Com`mo*ni"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commonitio</ets>. See <er>Monition</er>.]</ety> <def>Advice; warning; instruction.</def> <mark>[Obs.]</mark> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonitive</ent><br/
<hw>Com*mon"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Monitory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Only commemorative and <qex>commonitive</qex>.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonitory</ent><br/
<hw>Com*mon"i*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commonitorius</ets>.]</ety> <def>Calling to mind; giving admonition.</def> <mark>[Obs.]</mark> <rj><au>Foxe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonly</ent><br/
<hw>Com"mon*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>Usually; generally; ordinarily; frequently; for the most part; <as>as, confirmed habits <ex>commonly</ex> continue through life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In common; familiarly.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonness</ent><br/
<hw>Com"mon*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>State or quality of being common or usual; <as>as, the <ex>commonness</ex> of sunlight</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Triteness; meanness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonplace</ent><br/
<hw>Com"mon*place`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Common; ordinary; trite; <as>as, a <ex>commonplace</ex> person, or observation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonplace</ent><br/
<hw>Com"mon*place`</hw>, <pos>n.</pos> <sn>1.</sn> <def>An idea or expression wanting originality or interest; a trite or customary remark; a platitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A memorandum; something to be frequently consulted or referred to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whatever, in my reading, occurs concerning this our fellow creature, I do never fail to set it down by way of <qex>commonplace</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Commonplace book</b></col>, <cd>a book in which records are made of things to be remembered.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonplace</ent><br/
<hw>Com"mon*place`</hw>, <pos>v. t.</pos> <def>To enter in a commonplace book, or to reduce to general heads.</def> <rj><au>Felton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonplace</ent><br/
<hw>Com"mon*place`</hw>, <pos>v. i.</pos> <def>To utter commonplaces; to indulge in platitudes.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonplaceness</ent><br/
<hw>Com"mon*place`ness</hw>, <pos>n.</pos> <def>The quality of being commonplace; commonness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>common-room</ent><br/
<ent>common room</ent><br/
<mhw><hw>common room</hw>, <hw>common-room</hw></mhw> <pos>n.</pos> <def>a room, usually at a school or university, set aside for the common use of everyone.  It usually has comfortable chairs for reading or conversation, and some provision for obtaining refreshments.</def><br/
<syn><b>Syn. --</b> .</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Commons</ent><br/
<hw>Com"mons</hw> <pr>(?)</pr>, <pos>n. pl.</pos>, <sn>1.</sn> <def>The mass of the people, as distinguished from the titled classes or nobility; the commonalty; the common people.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is like the <qex>commons</qex>, rude unpolished hinds,<br/
Could send such message to their sovereign.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The word <qex>commons</qex> in its present ordinary signification comprises all the people who are under the rank of peers.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The House of Commons, or lower house of the British Parliament, consisting of representatives elected by the qualified voters of counties, boroughs, and universities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is agreed that the <qex>Commons</qex> were no part of the great council till some ages after the Conquest.</q> <rj><qau>Hume.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Provisions; food; fare, -- as that provided at a common table in colleges and universities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>commons</qex>, though but coarse, were nothing scant.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A club or association for boarding at a common table, as in a college, the members sharing the expenses equally; <as>as, to board in <ex>commons</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A common; public pasture ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To shake his ears, and graze in <qex>commons</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Doctors' Commons</b></col>, <cd>a place near St. Paul's Churchyard in London where the doctors of civil law used to common together, and where were the ecclesiastical and admiralty courts and offices having jurisdiction of marriage licenses, divorces, registration of wills, etc.</cd> -- <col><b>To be on short commons</b></col>, <cd>to have a small allowance of food.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonty</ent><br/
<ent>Common sense</ent><br/
<hw>Com"mon sense"</hw> <pr>(?)</pr>. <def>See <cref>Common sense</cref>, under <er>Sense</er>.</def></p>

<-- ... and see under Common (adjective), 6. PCP --><p><hw>Com"mon*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Scots Law)</fld> <def>A common; a piece of land in which two or more persons have a common right.</def> <rj><au>Bell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonweal</ent><br/
<hw>Com"mon*weal"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Common</ets> + <ets>weal</ets>.]</ety> <def>Commonwealth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a prince,<br/
So kind a father of the <qex>commonweal</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commonwealth</ent><br/
<hw>Com"mon*wealth`</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[<ets>Common</ets> + <ets>wealth</ets> well-being.]</ety> <sn>1.</sn> <def>A state; a body politic consisting of a certain number of men, united, by compact or tacit agreement, under one form of government and system of laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The trappings of a monarchy would set up an ordinary <qex>commonwealth</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This term is applied to governments which are considered as free or popular, but rarely, or improperly, to an absolute government. The word signifies, strictly, the <xex>common well-being</xex> or <xex>happiness</xex>; and hence, a form of government in which the general welfare is regarded rather than the welfare of any class.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The whole body of people in a state; the public.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eng. Hist.)</fld> <def>Specifically, the form of government established on the death of Charles I., in 1649, which existed under Oliver Cromwell and his son Richard, ending with the abdication of the latter in 1659.</def></p>

<p><syn><b>Syn.</b> -- State; realm; republic.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 287 --></p>

<p><ent>Commorance</ent><br/
<hw>Com"mo*rance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Commorancy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commorancy</ent><br/
<hw>Com"mo*ran*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Law)</fld> <def>A dwelling or ordinary residence in a place; habitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Commorancy</qex> consists in usually lying there.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Am. Law)</fld> <def>Residence temporarily, or for a short time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commorant</ent><br/
<hw>Com"mo*rant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commorans</ets>, p. pr. of <ets>commorari</ets> to abide; <ets>com-</ets> + <ets>morari</ets> to delay.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>Ordinarily residing; inhabiting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All freeholders within the precinct . . . and all persons <qex>commorant</qex> therein.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Am. Law)</fld> <def>Inhabiting or occupying temporarily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commorant</ent><br/
<hw>Com"mo*rant</hw>, <pos>n.</pos> <def>A resident.</def> <rj><au>Bp. Hacket.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commoration</ent><br/
<hw>Com`mo*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commoratio</ets>.]</ety> <def>The act of staying or residing in a place.</def> <mark>[Obs.]</mark> <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commorient</ent><br/
<hw>Com*mo"ri*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commoriens</ets>, p. pr. of <ets>commoriri</ets>.]</ety> <def>Dying together or at the same time.</def> <mark>[R.]</mark> <rj><au>Sir G. Buck.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commorse</ent><br/
<hw>Com*morse"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commorsus</ets>, p. p. of <ets>commordere</ets> to bite sharply.]</ety> <def>Remorse.</def> <mark>[Obs.]</mark> <ldquo/With sad <xex>commorse</xex>.<rdquo/ <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commote</ent><br/
<hw>Com*mote"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Commove</er>.]</ety> <def>To commove; to disturb; to stir up.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Society being more or less <qex>commoted</qex> and made uncomfortable.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commotion</ent><br/
<hw>Com*mo"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commotio</ets>: cf. F. <ets>commotion</ets>. See <er>Motion</er>.]</ety> <sn>1.</sn> <def>Disturbed or violent motion; agitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[What] <qex>commotion</qex> in the winds !</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A popular tumult; public disturbance; riot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When ye shall hear of wars and <qex>commotions</qex>.</q> <rj><qau>Luke xxi. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Agitation, perturbation, or disorder, of mind; heat; excitement.</def> <ldquo/He could not debate anything without some <xex>commotion</xex>.<rdquo/ <rj><au>Clarendon.</au></rj></p>

<p><syn><b>Syn.</b> -- Excitement; agitation; perturbation; disturbance; tumult; disorder; violence.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commove</ent><br/
<hw>Com*move"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commoved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Commoving</conjf>.]</vmorph> <ety>[L. <ets>commovere</ets>, <ets>commotum</ets>; <ets>com-</ets> + <ets>movere</ets> to move.]</ety> <sn>1.</sn> <def>To urge; to persuade; to incite.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put in motion; to disturb; to unsettle.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Straight the sands,<br/
<qex>Commoved</qex> around, in gathering eddies play.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communal</ent><br/
<hw>Com"mu*nal</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>communal</ets>.]</ety> <sn>1.</sn> <def>Pertaining to a commune.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>resembling a commune{4} or the practises of a commune{4}; <as>as, <ex>communal</ex> living</as>.</def><br/
<syn><b>Syn. --</b> collectivist.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Communalism</ent><br/
<hw>Com"mu*nal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A French theory of government which holds that commune should be a kind of independent state, and the national government a confederation of such states, having only limited powers. It is advocated by advanced French republicans; but it should not be confounded with communism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communalist</ent><br/
<hw>Com"mu*nal*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>communaliste</ets>.]</ety> <def>An advocate of communalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communalistic</ent><br/
<hw>Com`mu*nal*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to communalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commune</ent><br/
<hw>Com*mune"</hw> <pr>(k<ocr/m*m<umac/n")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Communed</conjf> <pr>(k<ocr/m*m<umac/nd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Communing</conjf>.]</vmorph> <ety>[OF. <ets>communier</ets>, fr. L. <ets>communicare</ets> to communicate, fr. <ets>communis</ets> common. See <er>Common</er>, and cf. <er>Communicate</er>.]</ety> <sn>1.</sn> <def>To converse together with sympathy and confidence; to interchange sentiments or feelings; to take counsel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would <qex>commune</qex> with you of such things<br/
That want no ear but yours.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To receive the communion; to partake of the eucharist or Lord's supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>commune</qex> under both kinds.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To commune with one's self</b></col> <it>or</it> <col><b>To commune with one's heart</b></col></mcol>, <cd>to think; to reflect; to meditate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commune</ent><br/
<hw>Com"mune</hw> <pr>(k<ocr/m"m<umac/n)</pr>, <pos>n.</pos> <def>Communion; sympathetic intercourse or conversation between friends.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For days of happy <qex>commune</qex> dead.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commune</ent><br/
<hw>Com"mune</hw> <pr>(k<ocr/m"m<umac/n)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>commun</ets>. See <er>Common</er>.]</ety> <sn>1.</sn> <def>The commonalty; the common people.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In this struggle -- to use the technical words of the time -- of the <ldquo/<qex>commune</qex><rdquo/, the general mass of the inhabitants, against the <ldquo/prudhommes<rdquo/ or <ldquo/wiser<rdquo/ few.</q> <rj><qau>J. R. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small territorial district in France under the government of a mayor and municipal council; also, the inhabitants, or the government, of such a district. See <er>Arrondissement</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Absolute municipal self-government.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>a group of people living together as an organized community and owning in common most or all of their property and possessions, and sharing work, income, and many other aspects of daily life.  Such sommunities are oftten organized based on religious or idealistic principles, and they sometimes have unconventional lifestyles, practises, or moral codes.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>The Commune of Paris</b></col>, <it>or</it> <col><b>The Commune</b></col></mcol> <sd>(a)</sd> <cd>The government established in Paris (1792-94) by a usurpation of supreme power on the part of representatives chosen by the communes; the period of its continuance is known as the <ldquo/Reign of Terror.<rdquo/</cd> <sd>(b)</sd> <cd>The revolutionary government, modeled on the commune of 1792, which the communists, so called, attempted to establish in 1871.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicability</ent><br/
<hw>Com*mu`ni*ca*bil"i*ty</hw> <pr>(k<ocr/m*m<umac/`n<icr/*k<adot/*b<icr/l"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>communicabilit<eacute/</ets>.]</ety> <def>The quality of being communicable; capability of being imparted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicable</ent><br/
<hw>Com*mu"ni*ca*ble</hw> <pr>(k<ocr/m*m<umac/"n<icr/*k<adot/*b'l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>communicable</ets>, LL. <ets>communicabilis</ets>.]</ety> <sn>1.</sn> <def>Capable of being communicated, or imparted; <as>as, a <ex>communicable</ex> disease; <ex>communicable</ex> knowledge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Communicative; free-speaking.</def> <mark>[Obs.]</mark> <rj><au>B. Jonson.</au></rj></p>

<p>-- <wordforms><wf>Com*mu"ni*ca*ble*ness</wf>, <pos>n.</pos> -- <wf>Com*mu"ni*ca"bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicant</ent><br/
<hw>Com*mu"ni*cant</hw> <pr>(k<ocr/m*m<umac/"n<icr/*k<ait/nt)</pr>, <pos>n.</pos> <ety>[L. <ets>communicans</ets>, p. pr.]</ety> <sn>1.</sn> <def>One who partakes of, or is entitled to partake of, the sacrament of the Lord's supper; a church member.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A never-failing monthly <qex>communicant</qex>.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who communicates.</def> <rj><au>Foxe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicant</ent><br/
<hw>Com*mu"ni*cant</hw> <pr>(k<ocr/m*m<umac/"n<icr/*k<ait/nt)</pr>, <pos>a.</pos> <def>Communicating.</def> <mark>[R.]</mark> <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicate</ent><br/
<hw>Com*mu"ni*cate</hw> <pr>(k<ocr/m*m<umac/"n<icr/*k<amac/t )</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Communicated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Communicating</conjf>.]</vmorph> <ety>[L. <ets>communicatus</ets>, p. p. of <ets>communicare</ets> to communicate, fr. <ets>communis</ets> common. See <er>Commune</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To share in common; to participate in.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To thousands that <qex>communicate</qex> our loss.</q> <rj><qau>B. Jonson</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To impart; to bestow; to convey; <as>as, to <ex>communicate</ex> a disease or a sensation; to <ex>communicate</ex> motion by means of a crank</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where God is worshiped, there he <qex>communicates</qex> his blessings and holy influences.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make known; to recount; to give; to impart; <as>as, to <ex>communicate</ex> information to any one</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To administer the communion to.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She [the church] . . . may <qex>communicate</qex> him.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This verb was formerly followed by <xex>with</xex> before the person receiving, but now usually takes <xex>to</xex> after it.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>communicated</qex> those thoughts only with the Lord Digby.</q> <rj><qau>Clarendon.</qau></rj></p>

<p><syn><b>Syn.</b> -- To impart; bestow; confer; reveal; disclose; tell; announce; recount; make known.</syn> <usage> -- To <er>Communicate</er>, <er>Impart</er>, <er>Reveal</er>. <xex>Communicate</xex> is the more general term, and denotes the allowing of others to partake or enjoy in common with ourselves. <xex>Impart</xex> is more specific. It is giving to others a part of what we had held as our own, or making them our partners; as, to <xex>impart</xex> our feelings; to <xex>impart</xex> of our property, etc. Hence there is something more intimate in <xex>imparting</xex> intelligence than in <xex>communicating</xex> it. To <xex>reveal</xex> is to disclose something hidden or concealed; as, to <xex>reveal</xex> a secret.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicate</ent><br/
<hw>Com*mu"ni*cate</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To share or participate; to possess or enjoy in common; to have sympathy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye did <qex>communicate</qex> with my affliction.</q> <rj><qau>Philip. iv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give alms, sympathy, or aid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To do good and to <qex>communicate</qex> forget not.</q> <rj><qau>Heb. xiii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have intercourse or to be the means of intercourse; <as>as, to <ex>communicate</ex> with another on business</as>; to be connected; <as>as, a <ex>communicating</ex> artery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Subjects suffered to <qex>communicate</qex> and to have intercourse of traffic.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole body is nothing but a system of such canals, which all <qex>communicate</qex> with one another.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To partake of the Lord's supper; to commune.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The primitive Christians <qex>communicated</qex> every day.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communication</ent><br/
<hw>Com*mu`ni*ca"tion</hw> <pr>(k<ocr/m*m<umac/`n<icr/*k<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>communicatio</ets>.]</ety> <sn>1.</sn> <def>The act or fact of communicating; <as>as, <ex>communication</ex> of smallpox; <ex>communication</ex> of a secret</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Intercourse by words, letters, or messages; interchange of thoughts or opinions, by conference or other means; conference; correspondence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Argument . . . and friendly <qex>communication</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Association; company.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Evil <qex>communications</qex> corrupt good manners.</q> <rj><qau>1 Cor. xv. 33.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Means of communicating; means of passing from place to place; a connecting passage; connection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Euxine Sea is conveniently situated for trade, by the <qex>communication</qex> it has both with Asia and Europe.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which is communicated or imparted; intelligence; news; a verbal or written message.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Participation in the Lord's supper.</def> <rj><au>Bp. Pearson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Rhet.)</fld> <def>A trope, by which a speaker assumes that his hearer is a partner in his sentiments, and says <xex>we</xex>, instead of <xex>I</xex> or <xex>you</xex>.</def> <rj><au>Beattie.</au></rj></p>

<p><syn><b>Syn.</b> -- Correspondence; conference; intercourse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>communicating</ent><br/
<hw>communicating</hw> <pos>n.</pos> <def>the activity of communicating.</def><br/
<syn><b>Syn. --</b> communication.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>communicational</ent><br/
<hw>communicational</hw> <pos>adj.</pos> <def>used in communication; <as>as, he had few <ex>communicational</ex> grooves available for use</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Communicative</ent><br/
<hw>Com*mu"ni*ca*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>Communicatif</ets>, LL. <ets>communicativus</ets>.]</ety> <def>Inclined to communicate; ready to impart to others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Determine, for the future, to be less <qex>communicative</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicativeness</ent><br/
<hw>Com*mu"ni*ca*tive*ness</hw>, <pos>n.</pos> <def>The quality of being communicative.</def> <rj><au>Norris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicator</ent><br/
<hw>Com*mu"ni*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who communicates.</def> <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communicatory</ent><br/
<hw>Com*mu"ni*ca"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>communicatorius</ets>.]</ety> <def>Imparting knowledge or information.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Canonical and <qex>communicatory</qex> letters.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communion</ent><br/
<hw>Com*mun"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>communio</ets>: cf. F. <ets>communion</ets>. See <er>Common</er>.]</ety> <sn>1.</sn> <def>The act of sharing; community; participation.</def> <ldquo/This <xex>communion</xex> of goods.<rdquo/ <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Intercourse between two or more persons; esp., intimate association and intercourse implying sympathy and confidence; interchange of thoughts, purposes, etc.; agreement; fellowship; <as>as, the <ex>communion</ex> of saints</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are naturally induced to seek <qex>communion</qex> and fellowship with others.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>communion</qex> hath light with darkness?</q> <rj><qau>2 Cor. vi. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bare <qex>communion</qex> with a good church can never alone make a good man.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A body of Christians having one common faith and discipline; <as>as, the Presbyterian <ex>communion</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The sacrament of the eucharist; the celebration of the Lord's supper; the act of partaking of the sacrament; <as>as, to go to <ex>communion</ex>; to partake of the <ex>communion</ex></as>; called also <altname>Holy Communion</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Close communion</b></col>. <cd>See under <er>Close</er>, <pos>a.</pos></cd> -- <col><b>Communion elements</b></col>, <cd>the bread and wine used in the celebration of the Lord's supper.</cd> -- <col><b>Communion service</b></col>, <cd>the celebration of the Lord's supper, or the office or service therefor.</cd> -- <col><b>Communion table</b></col>, <cd>the table upon which the elements are placed at the celebration of the Lord's supper.</cd> -- <col><b>Communion in both kinds</b></col>, <cd>participation in both the bread and wine by all communicants.</cd> -- <col><b>Communion in one kind</b></col>, <cd>participation in but one element, as in the Roman Catholic Church, where the laity partake of the bread only.</cd></cs></p>

<p><syn><b>Syn.</b> -- Share; participation; fellowship; converse; intercourse; unity; concord; agreement.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communism</ent><br/
<hw>Com"mu*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>communisme</ets>, fr. <ets>commun</ets> common.]</ety> <def>A scheme of equalizing the social conditions of life; specifically, a scheme which contemplates the abolition of inequalities in the possession of property, as by distributing all wealth equally to all, or by holding all wealth in common for the equal use and advantage of all.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ At different times, and in different countries, various schemes pertaining to socialism in government and the conditions of domestic life, as well as in the distribution of wealth, have been called <xex>communism</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communist</ent><br/
<hw>Com"mu*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>communiste</ets>.]</ety> <sn>1.</sn> <def>An advocate for the theory or practice of communism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A supporter of the commune of Paris.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Communistic</ent><br/
<hw>Com`mu*nis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to communism or communists; <as>as, <ex>communistic</ex> theories</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Living or having their nests in common, as certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Community</ent><br/
<hw>Com*mu"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Communities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>communitas</ets>: cf. OF. <ets>communit<eacute/</ets>. Cf. <er>Commonalty</er>, and see <er>Common</er>.]</ety> <sn>1.</sn> <def>Common possession or enjoyment; participation; <as>as, a <ex>community</ex> of goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The original <qex>community</qex> of all things.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An unreserved <qex>community</qex> of thought and feeling.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A body of people having common rights, privileges, or interests, or living in the same place under the same laws and regulations; <as>as, a <ex>community</ex> of monks</as>. Hence a number of animals living in a common home or with some apparent association of interests.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Creatures that in <qex>communities</qex> exist.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Society at large; a commonwealth or state; a body politic; the public, or people in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Burdens upon the poorer classes of the <qex>community</qex>.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, the term should be used with the definite article; as, the interests of <xex>the</xex> community.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Common character; likeness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The essential <qex>community</qex> of nature between organic growth and inorganic growth.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Commonness; frequency.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Eyes . . . sick and blunted with <qex>community</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commutability</ent><br/
<hw>Com*mu`ta*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being commutable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commutable</ent><br/
<hw>Com*mut"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>commutabilis</ets>.]</ety> <def>Capable of being commuted or interchanged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The predicate and subject are not <qex>commutable</qex>.</q> <rj><qau>Whately.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commutableness</ent><br/
<hw>Com*mut"a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being commutable; interchangeableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commutation</ent><br/
<hw>Com`mu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>commutatio</ets>: cf. F. <ets>commutation</ets>.]</ety> <sn>1.</sn> <def>A passing from one state to another; change; alteration; mutation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So great is the <qex>commutation</qex> that the soul then hated only that which now only it loves.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of giving one thing for another; barter; exchange.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The use of money is . . . that of saving the <qex>commutation</qex> of more bulky commodities.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>The change of a penalty or punishment by the pardoning power of the State; <as>as, the <ex>commutation</ex> of a sentence of death to banishment or imprisonment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Suits are allowable in the spiritual courts for money agreed to be given as a <qex>commutation</qex> for penance.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A substitution, as of a less thing for a greater, esp. a substitution of one form of payment for another, or one payment for many, or a specific sum of money for conditional payments or allowances; <as>as, <ex>commutation</ex> of tithes; <ex>commutation</ex> of fares; <ex>commutation</ex> of copyright; <ex>commutation</ex> of rations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>regular travel from a place of residence to a place where one's daily work is performed; commuting.  Most often, such travel is performed between a suburb and a nearby city.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Angle of commutation</b></col> <fld>(Astron.)</fld>, <cd>the difference of the geocentric longitudes of the sun and a planet.</cd> -- <col><b>Commutation of tithes</b></col>, <cd>the substitution of a regular payment, chargeable to the land, for the annual tithes in kind.</cd> -- <col><b>Commutation ticket</b></col>, <cd>a ticket, as for transportation, which is the evidence of a contract for service at a reduced rate. See 2d <er>Commute</er>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commutation ticket</ent><br/
<hw>Com`mu*ta"tion tick"et</hw>. <def>A ticket for transportation at a reduced rate in consideration of some special circumstance, as increase of travel; specif., a ticket for a certain number of, or for daily, trips between neighboring places at a reduced rate, such as are commonly used by those doing business in a city and living in a suburb. Commutation tickets are excepted from the prohibition against special rates contained in the Interstate Commerce Act of Feb. 4, 1887 (<au>24 Stat. 379</au>), and in 145 U. S. 263 it was held that <xex>party tickets</xex> were also excepted as being <ldquo/obviously within the commuting principle.<rdquo/</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Commutative</ent><br/
<hw>Com*mut"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[CF. F. <ets>commutatif</ets>.]</ety> <sn>1.</sn> <def>Relative to exchange; interchangeable; reciprocal.</def> -- <wordforms><wf>Com*mut"a*tive"ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Rich traders, from their success, are presumed . . . to have cultivated an habitual regard to <qex>commutative</qex> justice.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>having the property of <er>commutativity</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>commutativity</ent><br/
<hw>com`mu*ta*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Math.)</fld> <def>the property of a binary operation or operator such that the result of one term operating on a second is equal to the result of the second term operating on the first.</def><br/
[<source>GG</source>]</p>

<p><note><hand/  If the symbol <ldquo/*<rdquo/ is taken to be a commutative binary operator, then the following equation holds:
             <mathex>a*b = b*a</mathex>

 A common example is the addition operator for real numbers, <ldquo/+<rdquo/:
             <mathex>a+b = b+a</mathex>
</note><br/
[<source>GG</source>]</p>

<p><ent>Commutator</ent><br/
<hw>Com"mu*ta`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Elec.)</fld> <def>A piece of apparatus used for reversing the direction of an electrical current; an attachment to certain electrical machines, by means of which alternating currents are made to be continuous or to have the same direction.  It may be attached to the end of the spindle of an electric motor, where a <part>brush</part> is in contact sequentially with the parts of the spindle that conduct current to the different windings of the motor.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Commute</ent><br/
<hw>Com*mute"</hw> <pr>(k<ocr/m*m<umac/t")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Commuted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Commuting</conjf>.]</vmorph> <ety>[L. <ets>commutare</ets>, <ets>-mutatum</ets>; <ets>com-</ets> + <ets>mutare</ets> to change. See <er>Mutation</er>.]</ety> <sn>1.</sn> <def>To exchange; to put or substitute something else in place of, as a smaller penalty, obligation, or payment, for a greater, or a single thing for an aggregate; hence, to lessen; to diminish; <as>as, to <ex>commute</ex> a sentence of death to one of imprisonment for life; to <ex>commute</ex> tithes; to <ex>commute</ex> charges for fares</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sounds water and fire, being once annexed to those two elements, it was certainly more natural to call beings participating of the first <ldquo/watery<rdquo/, and the last <ldquo/fiery<rdquo/, than to <qex>commute</qex> the terms, and call them by the reverse.</q> <rj><qau>J. Harris</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The utmost that could be obtained was that her sentence should be <qex>commuted</qex> from burning to beheading.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commute</ent><br/
<hw>Com*mute"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To obtain or bargain for exemption or substitution; to effect a commutation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . thinks it unlawful to <qex>commute</qex>, and that he is bound to pay his vow in kind.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pay, or arrange to pay, in gross instead of part by part; <as>as, to <ex>commute</ex> for a year's travel over a route</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>to travel regularly from a place of residence to another place, such as where one's daily work is performed.  Often, such travel is performed between a suburb and a nearby city; <as>as, to <ex>commute</ex> to work</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Commuter</ent><br/
<hw>Com*mut"er</hw> <pr>(k<ocr/m*m<umac/"t<etil/r)</pr>, <pos>n.</pos> <def>One who commutes; especially, one who commutes in traveling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Commuter</ent><br/
<hw>Com*mut"er</hw> <pr>(k<ocr/m*m<umac/"t<etil/r)</pr>, <pos>a.</pos> <def>of or pertaining to commuting, in the sense of traveling; used for commuting; <as>as, a <ex>commuter</ex> airline</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>commuting</ent><br/
<hw>commuting</hw> <pos>n.</pos> <def>the process of travel by a commuter.</def><br/
<syn><b>Syn. --</b> commutation.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Commutual</ent><br/
<hw>Com*mu"tu*al</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[Pref. <ets>com-</ets> + <ets>mutual</ets>.]</ety> <def>Mutual; reciprocal; united.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There, with <qex>commutual</qex> zeal, we both had strove.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comose</ent><br/
<hw>Co"mose</hw> <pr>(k<omac/"m<omac/s <it>or</it> k<osl/*m<omac/s")</pr>, <pos>a.</pos> <ety>[L. <ets>comosus</ets> hairy, from <ets>coma</ets> hair.]</ety> <fld>(Bot.)</fld> <def>Bearing a tuft of soft hairs or down, as the seeds of milkweed.</def> <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 288 --></p>

<p><ent>Compact</ent><br/
<hw>Com*pact"</hw> <pr>(k<ocr/m*p<acr/kt")</pr>, <pos>p. p. & a.</pos> <ety>[L. <ets>compactus</ets>, p. p. of <ets>compingere</ets> to join or unite; <ets>com-</ets> + <ets>pangere</ets> to fasten, fix: cf. F. <ets>compacte</ets>. See <er>Pact</er>.]</ety> <sn>1.</sn> <def>Joined or held together; leagued; confederated.</def> <mark>[Obs.]</mark> <ldquo/<xex>Compact</xex> with her that's gone.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A pipe of seven reeds, <qex>compact</qex> with wax together.</q> <rj><qau>Peacham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Composed or made; -- with <xex>of</xex>.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A wandering fire,<br/
<qex>Compact</qex> of unctuous vapor.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Closely or firmly united, as the particles of solid bodies; firm; close; solid; dense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Glass, crystal, gems, and other <qex>compact</qex> bodies.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Brief; close; pithy; not diffuse; not verbose; <as>as, a <ex>compact</ex> discourse</as>.</def></p>

<p><syn><b>Syn.</b> -- Firm; close; solid; dense; pithy; sententious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compact</ent><br/
<hw>Com*pact"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compacted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Compacting</conjf>.]</vmorph> <sn>1.</sn> <def>To thrust, drive, or press closely together; to join firmly; to consolidate; to make close; -- as the parts which compose a body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now the bright sun <qex>compacts</qex> the precious stone.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite or connect firmly, as in a system.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole body fitly joined together and <qex>compacted</qex> by that which every joint supplieth.</q> <rj><qau>Eph. iv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compact</ent><br/
<hw>Com"pact</hw> <pr>(k<ocr/m"p<acr/kt)</pr>, <pos>n.</pos> <ety>[L. <ets>compactum</ets>, fr. <ets>compacisci</ets>, p. p. <ets>compactus</ets>, to make an agreement with; <ets>com-</ets> + <ets>pacisci</ets> to make an agreement. See <er>Pact</er>.]</ety> <def>An agreement between parties; a covenant or contract.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law of nations depends on mutual <qex>compacts</qex>, treaties, leagues, etc.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wedlock is described as the indissoluble <qex>compact</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The federal constitution has been styled a <qex>compact</qex> between the States by which it was ratified.</q> <rj><qau>Wharton.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Covenant</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>compact disk</ent><br/
<ent>compact disc</ent><br/
<mhw><hw>com"pact disc`</hw>, <hw>com"pact disk`</hw></mhw> <pr>(k<ocr/m"p<acr/kt d<icr/sk`)</pr>, <pos>n.</pos> <def>a disk-shaped optical data storage medium approximately 4-3/4 in. in diameter, which stores binary data as microscopic nonreflective holes or pits in an otherwise reflective surface, and is recorded and played back by rotation in an electronic device containing a laser.  It is often referred to by its abbreviation <abbr>CD</abbr>.  It is a type of <isa>optical data storage</isa> medium.  <ex>Compact disks</ex> are used for recordings of music as well as of data for computer applications.</def><br/
<syn><b>Syn. --</b> CD.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>compact disk player</ent><br/
<mhw><hw>com"pact disk` play"er</hw></mhw> <pr>(k<ocr/m"p<acr/kt d<icr/sk` pl<amac/"<etil/r)</pr>, <pos>n.</pos> <def>an electronic device containing a laser, used to read or play back the data on a compact disk.  The term is usually used for the type of device used to play music recordings.  That used for data storgage n coputer applications is usually called a <contr>CD drive</contr>.  Called also <altname>CD player</altname> and informally, <altname>CD</altname>.</def><br/
<syn><b>Syn. --</b> CD player.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Compacted</ent><br/
<hw>Com*pact"ed</hw> <pr>(k<ocr/m*p<acr/kt"<ecr/d)</pr>, <pos>a.</pos> <def>Compact; pressed close; concentrated; firmly united.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compactedly</ent><br/
<hw>Com*pact"ed*ly</hw>, <pos>adv.</pos> <def>In a compact manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compactedness</ent><br/
<hw>Com*pact"ed*ness</hw>, <pos>n.</pos> <def>A state of being compact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compacter</ent><br/
<hw>Com*pact"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes a compact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compactible</ent><br/
<hw>Com*pact"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be compacted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compaction</ent><br/
<hw>Com*pac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compactio</ets>.]</ety> <def>The act of making compact, or the state of being compact.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compactly</ent><br/
<hw>Com*pact"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a compact manner; with close union of parts; densely; tersely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compactness</ent><br/
<hw>Com*pact"ness</hw>, <pos>n.</pos> <def>The state or quality of being compact; close union of parts; density.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compacture</ent><br/
<hw>Com*pac"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>compactura</ets>.]</ety> <def>Close union or connection of parts; manner of joining; construction.</def> <mark>[Obs.]</mark> <ldquo/With comely compass and <xex>compacture</xex> strong.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compages</ent><br/
||<hw>Com*pa"ges</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[L., fr. <ets>compingere</ets>. See <er>Compact</er>, <pos>v. t.</pos>]</ety> <def>A system or structure of many parts united.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A regular <qex>compages</qex> of pipes and vessels.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compaginate</ent><br/
<hw>Com*pag"i*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>compaginare</ets>, <ets>compaginatum</ets>.]</ety> <def>To unite or hold together; <as>as, the side pieces <ex>compaginate</ex> the frame</as>.</def> <mark>[Obs.]</mark> <rj><au>W. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compagination</ent><br/
<hw>Com*pag`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compaginatio</ets>.]</ety> <def>Union of parts; structure.</def> <mark>[Obs.]</mark> <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companable</ent><br/
<hw>Com"pa*na*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>compaignable</ets>.]</ety> <def>Companionable; sociable.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companator</ent><br/
<hw>Com"pa*na`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>companatores</ets>, pl.]</ety> <fld>(Eccl.)</fld> <def>Same as <er>Impanator</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companiable</ent><br/
<hw>Com*pan"i*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Companionable; sociable.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companion</ent><br/
<hw>Com*pan"ion</hw> <pr>(k<ocr/m*p<acr/n"y<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>compagnon</ets>, OF. <ets>compaing</ets>, fr. an assumed LL. <ets>companio</ets> (cf. <ets>companium</ets> fellowship, a mess), fr. L. <ets>com-</ets> + <ets>panis</ets> bread. See <er>Pantry</er>.]</ety> <sn>1.</sn> <def>One who accompanies or is in company with another for a longer or shorter period, either from choice or casually; one who is much in the company of, or is associated with, another or others; an associate; a comrade; a consort; a partner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>companions</qex> of his fall.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>companion</qex> of fools shall smart for it.</q> <rj><qau>Prov. xiii. 20 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here are your sons again; and I must lose<br/
Two of the sweetest <qex>companions</qex> in the world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>companion</qex> is one with whom we share our bread; a messmate.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A knight of the lowest rank in certain orders; <as>as, a <ex>companion</ex> of the Bath</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fellow; -- in contempt.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Cf. OSp. <ets>compa<ntil/a</ets> an outhouse, office.]</ety> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A skylight on an upper deck with frames and sashes of various shapes, to admit light to a cabin or lower deck.</def> <sd>(b)</sd> <def>A wooden hood or penthouse covering the companion way; a companion hatch.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Companion hatch</b></col> <fld>(Naut.)</fld>, <cd>a wooden porch over the entrance or staircase of the cabin.</cd> -- <col><b>Companion ladder</b></col> <fld>(Naut.)</fld>, <cd>the ladder by which officers ascend to, or descend from, the quarter-deck.</cd> <au>Totten.</au> -- <col><b>Companion way</b></col> <fld>(Naut.)</fld>, <cd>a staircase leading to the cabin.</cd> -- <col><b>Knights companions</b></col>, <cd>in certain honorary orders, the members of the lowest grades as distinguished from knights commanders, knights grand cross, and the like.</cd></cs></p>

<p><syn><b>Syn.</b> -- Associate; comrade; mate; compeer; partner; ally; confederate; coadjutor; accomplice.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companion</ent><br/
<hw>Com*pan"ion</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To be a companion to; to attend on; to accompany.</def> <mark>[R.]</mark> <rj><au>Ruskin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To qualify as a companion; to make equal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Companion</qex> me with my mistress.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companionable</ent><br/
<hw>Com*pan"ion*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fitted to be a companion; fit for good fellowship; agreeable; sociable.</def> <ldquo/Each <xex>companionable</xex> guest.<rdquo/ <au>Mallett.</au> <ldquo/<xex>Companionable</xex> wit.<rdquo/ <au>Clarendon.</au></p>

<p>-- <wordforms><wf>Com*pan"ion*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Com*pan"ion*a*bly</wf>, adv.</wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>companionate</ent><br/
<hw>companionate</hw> <pos>adj.</pos> <def>companionlike; <as>as, <ex>companionate</ex> marriage; a <ex>companionate</ex> dog</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Companionless</ent><br/
<hw>Com*pan"ion*less</hw>, <pos>a.</pos> <def>Without a companion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Companionship</ent><br/
<hw>Com*pan"ion*ship</hw>, <pos>n.</pos> <def>Fellowship; association; the act or fact of keeping company with any one.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He never seemed to avail himself of my sympathy other than by mere <qex>companionship</qex>.</q> <rj><qau>W. Irving</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>companionway</ent><br/
<hw>companionway</hw> <pos>n.</pos> <sn>1.</sn> <def>a stairway or ladder that leads from one deck to another on a ship.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Company</ent><br/
<hw>Com"pa*ny</hw> <pr>(k<ucr/m"p<adot/*n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Companies</plw> <pr>(k<ucr/m"p<adot/*n<icr/z)</pr>.</plu> <ety>[F. <ets>compagnie</ets>, fr. OF. <ets>compaing</ets>. See <er>Companion</er>.]</ety> <sn>1.</sn> <def>The state of being a companion or companions; the act of accompanying; fellowship; companionship; society; friendly intercourse.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Evil <qex>company</qex> doth corrupt good manners.</q> <rj><qau>1 Cor. xv. 33. (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Brethren, farewell: your <qex>company</qex> along<br/
I will not wish.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A companion or companions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To thee and thy <qex>company</qex> I bid<br/
A hearty welcome.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An assemblage or association of persons, either permanent or transient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt meet a <qex>company</qex> of prophets.</q> <rj><qau>1 Sam. x. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Guests or visitors, in distinction from the members of a family; <as>as, to invite <ex>company</ex> to dine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Society, in general; people assembled for social intercourse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nature has left every man a capacity of being agreeable, though not of shining in <qex>company</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An association of persons for the purpose of carrying on some enterprise or business; a corporation; a firm; <as>as, the East India <ex>Company</ex>; an insurance <ex>company</ex>; a joint-stock <ex>company</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Partners in a firm whose names are not mentioned in its style or title; -- often abbreviated in writing; <as>as, Hottinguer & <ex>Co</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mil.)</fld> <def>A subdivision of a regiment of troops under the command of a captain, numbering in the United States (full strength) 100 men.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Naut.)</fld> <def>The crew of a ship, including the officers; <as>as, a whole ship's <ex>company</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The body of actors employed in a theater or in the production of a play.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To keep company with</b></col>. <cd>See under <er>Keep</er>, <pos>v. t.</pos></cd></cs></p>

<p><syn><b>Syn.</b> -- Assemblage; assembly; society; group; circle; crowd; troop; crew; gang; corporation; association; fraternity; guild; partnership; copartnery; union; club; party; gathering.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Company</ent><br/
<hw>Com"pa*ny</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Companied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Companying</conjf>.]</vmorph> <def>To accompany or go with; to be companion to.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Company</ent><br/
<hw>Com"pa*ny</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To associate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men which have <qex>companied</qex> with us all the time.</q> <rj><qau>Acts i. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be a gay companion.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have sexual commerce.</def> <mark>[Obs.]</mark> <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>comparability</ent><br/
<hw>comparability</hw> <pos>n.</pos> <def>a similarity allowing comparison; an approximate equivalence.</def><br/
<syn><b>Syn. --</b> comparison, compare, equivalence.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comparable</ent><br/
<hw>Com"pa*ra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>comparabilis</ets>: cf. F. <ets>comparable</ets>.]</ety> <def>Capable of being compared; worthy of comparison.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no blessing of life <qex>comparable</qex> to the enjoyment of a discreet and virtuous friend.</q> <rj><qau>Addison.</qau></rj></p>

<p>-- <wordforms><wf>Com"pa*ra*ble*ness</wf>, <pos>n.</pos> -- <wf>Com"pa*ra*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparate</ent><br/
<hw>Com"pa*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comparatum</ets>, fr. <ets>comparatus</ets>, p. p. of <ets>comparare</ets>. See 1st <er>Compare</er>.]</ety> <fld>(Logic)</fld> <def>One of two things compared together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparation</ent><br/
<hw>Com`pa*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comparatio</ets>. See <er>Compare</er> to get.]</ety> <def>A making ready; provision.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparative</ent><br/
<hw>Com*par"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>comparativus</ets>: cf. F. <ets>comparatif</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to comparison.</def> <ldquo/The <xex>comparative</xex> faculty.<rdquo/ <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proceeding from, or by the method of, comparison; <as>as, the <ex>comparative</ex> sciences; the <ex>comparative</ex> anatomy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Estimated by comparison; relative; not positive or absolute, as compared with another thing or state.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The recurrence of <qex>comparative</qex> warmth and cold.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The bubble, by reason of its <qex>comparative</qex> levity to the fluid that incloses it, would necessarily ascend to the top.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Gram.)</fld> <def>Expressing a degree greater or less than the positive degree of the quality denoted by an adjective or adverb. The comparative degree is formed from the positive by the use of <xex>-er</xex>, <xex>more</xex>, or <xex>less</xex>; <as>as, brighter, <ex>more</ex> bright, or <ex>less</ex> bright</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Comparative sciences</b></col>, <cd>those which are based on a comprehensive comparison of the range of objects or facts in any branch or department, and which aim to study out and treat of the fundamental laws or systems of relation pervading them; as, <stype>comparative anatomy</stype>, <stype>comparative physiology</stype>, <stype>comparative philology</stype>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparative</ent><br/
<hw>Com*par"a*tive</hw>, <pos>n.</pos> <fld>(Gram.)</fld> <def>The comparative degree of adjectives and adverbs; also, the form by which the comparative degree is expressed; <as>as, <ex>stronger</ex>, <ex>wiser</ex>, <ex>weaker</ex>, <ex>more stormy</ex>, <ex>less windy</ex>, are all <ex>comparatives</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>comparatives</qex> is expressed a relation of two; as in superlatives there is a relation of many.</q> <rj><qau>Angus.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An equal; a rival; a compeer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Gerard ever was<br/
His full <qex>comparative</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who makes comparisons; one who affects wit.</def> <mark>[Obs.]</mark> <ldquo/Every beardless vain <xex>comparative</xex>.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparatively</ent><br/
<hw>Com*par"a*tive*ly</hw>, <pos>adv.</pos> <def>According to estimate made by comparison; relatively; not positively or absolutely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With but <qex>comparatively</qex> few exceptions.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparator</ent><br/
<hw>Com"pa*ra`tor</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[L., a comparer.]</ety> <fld>(Physics)</fld> <def>An instrument or machine for comparing anything to be measured with a standard measure; -- applied especially to a machine for comparing standards of length.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compare</ent><br/
<hw>Com*pare"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Comparing</conjf>.]</vmorph> <ety>[L.<ets>comparare</ets>, fr. <ets>compar</ets> like or equal to another; <ets>com-</ets> + <ets>par</ets> equal: cf. F. <ets>comparer</ets>. See <er>Pair</er>, <er>Peer</er> an equal, and cf. <er>Compeer</er>.]</ety> <sn>1.</sn> <def>To examine the character or qualities of, as of two or more persons or things, for the purpose of discovering their resemblances or differences; to bring into comparison; to regard with discriminating attention.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compare</qex> dead happiness with living woe.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The place he found beyond expression bright,<br/
<qex>Compared</qex> with aught on earth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compare</qex> our faces and be judge yourself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>compare</qex> great things with small.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To represent as similar, for the purpose of illustration; to liken.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Solon <qex>compared</qex> the people unto the sea, and orators and counselors to the winds; for that the sea would be calm and quiet if the winds did not trouble it.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>To inflect according to the degrees of comparison; to state positive, comparative, and superlative forms of; as, most adjectives of one syllable are <ex>compared</ex> by affixing <ldquo/- er<rdquo/ and <ldquo/-est<rdquo/ to the positive form; <as>as, <ex>black</ex>, <ex>blacker</ex>, <ex>blackest</ex></as>; those of more than one syllable are usually <xex>compared</xex> by prefixing <ldquo/more<rdquo/ and <ldquo/most<rdquo/, or <ldquo/less<rdquo/ and <ldquo/least<rdquo/, to the positive; <as>as, <ex>beautiful</ex>, <ex>more beautiful</ex>, <ex>most beautiful</ex></as>.</def></p>

<p><syn><b>Syn.</b> -- To <er>Compare</er>, <er>Compare with</er>, <er>Compare to</er>.</syn> <usage> Things are compared <xex>with</xex> each other in order to learn their relative value or excellence. Thus we compare Cicero <xex>with</xex> Demosthenes, for the sake of deciding which was the greater orator. One thing is compared <xex>to</xex> another because of a real or fanciful likeness or similarity which exists between them. Thus it has been common to compare the eloquence of Demosthenes <xex>to</xex> a thunderbolt, on account of its force, and the eloquence of Cicero <xex>to</xex> a conflagration, on account of its splendor. Burke compares the parks of London <xex>to</xex> the lungs of the human body.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compare</ent><br/
<hw>Com*pare"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be like or equal; to admit, or be worthy of, comparison; <as>as, his later work does not <ex>compare</ex> with his earlier</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I should <qex>compare</qex> with him in excellence.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To vie; to assume a likeness or equality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall pack horses . . . <qex>compare</qex> with C<ae/sars?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compare</ent><br/
<hw>Com*pare"</hw>, <pos>n.</pos> <sn>1.</sn> <def>Comparison.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His mighty champion, strong beyond <qex>compare</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their small galleys may not hold <qex>compare</qex><br/
With our tall ships.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Illustration by comparison; simile.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Rhymes full of protest, of oath, and big <qex>compare</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Beyond compare</b></col>. <cd>See <cref>Beyond comparison</cref>, under <er>Comparison</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compare</ent><br/
<hw>Com*pare"</hw>, <pos>v. t.</pos> <ety>[L. <ets>comparare</ets> to prepare, procure; <ets>com-</ets> + <ets>parare</ets>. See <er>Prepare</er>, <er>Parade</er>.]</ety> <def>To get; to procure; to obtain; to acquire</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To fill his bags, and richesse to <qex>compare</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparer</ent><br/
<hw>Com*par"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who compares.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparison</ent><br/
<hw>Com*par"i*son</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[F. <ets>comparaison</ets>, L. <ets>comparatio</ets>. See 1st <er>Compare</er>.]</ety> <sn>1.</sn> <def>The act of comparing; an examination of two or more objects with the view of discovering the resemblances or differences; relative estimate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As sharp legal practitioners, no class of human beings can bear <qex>comparison</qex> with them.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The miracles of our Lord and those of the Old Testament afford many interesting points of <qex>comparison</qex>.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being compared; a relative estimate; also, a state, quality, or relation, admitting of being compared; <as>as, to bring a thing into <ex>comparison</ex> with another; there is no <ex>comparison</ex> between them</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That to which, or with which, a thing is compared, as being equal or like; illustration; similitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whereto shall we liken the kingdom of God? Or with what <qex>comparison</qex> shall we compare it?</q> <rj><qau>Mark iv. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Gram.)</fld> <def>The modification, by inflection or otherwise, which the adjective and adverb undergo to denote degrees of quality or quantity; <as>as, <ex>little</ex>, <ex>less</ex>, <ex>least</ex>, are examples of <ex>comparison</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Rhet.)</fld> <def>A figure by which one person or thing is compared to another, or the two are considered with regard to some property or quality, which is common to them both; <it>e.g.</it>, the lake sparkled like a jewel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Phren.)</fld> <def>The faculty of the reflective group which is supposed to perceive resemblances and contrasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Beyond comparison</b></col>, <cd>so far superior as to have no likeness, or so as to make comparison needless.</cd> -- <mcol><col><b>In comparison of</b></col>, <col><b>In comparison with</b></col></mcol>, <cd>as compared with; in proportion to.</cd> <mark>[Archaic]</mark> <ldquo/So miserably unpeopled <xex>in comparison of</xex> what it once was.<rdquo/</cs> <au>Addison.</au> -- <col><b>Comparison of hands</b></col> <fld>(Law)</fld>, <cd>a mode of proving or disproving the genuineness of a signature or writing by comparing it with another proved or admitted to be genuine, in order to ascertain whether both were written by the same person.</cd> <au>Bouvier.</au> <au>Burrill.</au></p>

<p><-- <col><b>beyond compare</b></col> same as beyond comparison. Add! --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comparison</ent><br/
<hw>Com*par"i*son</hw>, <pos>v. t.</pos> <def>To compare.</def> <mark>[Obs.]</mark> <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>comparison-shop</ent><br/
<hw>comparison-shop</hw> <pos>v. i.</pos> <def>to compare prices for a given item from different vendors; -- usually for the purpose of finding the lowest price.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Compart</ent><br/
<hw>Com*part"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Comparted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Comparting</conjf>.]</vmorph> <ety>[L. <ets>compartiri</ets>; <ets>com-</ets> + <ets>partiri</ets>, <ets>partire</ets> to share, <ets>pars</ets>, <ets>partis</ets>, part, share: cf. OF. <ets>compartir</ets>. See <er>Part</er>, <pos>v. t.</pos>]</ety> <def>To divide; to mark out into parts or subdivisions.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The crystal surface is <qex>comparted</qex> all<br/
In niches verged with rubies.</q> <rj><qau>Glover.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compartition</ent><br/
<hw>Com`par*ti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>compartitio</ets>.]</ety> <def>The act of dividing into parts or compartments; division; also, a division or compartment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Their temples . . . needed no <qex>compartitions</qex>.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compartment</ent><br/
<hw>Com*part"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>compartiment</ets>, OF. <ets>compartir</ets> to divide. See <er>Compart</er>.]</ety> <sn>1.</sn> <def>One of the parts into which an inclosed portion of space is divided, as by partitions, or lines; <as>as, the <ex>compartments</ex> of a cabinet, a house, or a garden</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the midst was placed a large <qex>compartment</qex> composed of grotesque work.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Shipbuilding)</fld> <def>One of the sections into which the hold of a ship is divided by water-tight bulkheads.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>compartmentalized</ent><br/
<ent>compartmental</ent><br/
<mhw><hw>compartmental</hw> <hw>compartmentalized</hw></mhw> <pos>adj.</pos> <sn>1.</sn> <def>divided up into compartments or categories.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>compartmented</ent><br/
<hw>compartmented</hw> <pos>adj.</pos> <sn>1.</sn> <def>divided up into compartments; <as>as, a <ex>compartmented</ex> box</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Compartner</ent><br/
<hw>Com*part"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Copartner</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compass</ent><br/
<hw>Com"pass</hw> <pr>(k<ucr/m"p<ait/s)</pr>, <pos>n.</pos> <ety>[F. <ets>compas</ets>, fr. LL. <ets>compassus</ets> circle, prop., a stepping together; <ets>com-</ets> + <ets>passus</ets> pace, step. See <er>Pace</er>, <er>Pass</er>.]</ety> <sn>1.</sn> <def>A passing round; circuit; circuitous course.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They fetched a <qex>compass</qex> of seven day's journey.</q> <rj><qau>2 Kings iii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This day I breathed first; time is come round,<br/
And where I did begin, there shall I end;<br/
My life is run his <qex>compass</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inclosing limit; boundary; circumference; <as>as, within the <ex>compass</ex> of an encircling wall</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An inclosed space; an area; extent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their wisdom . . . lies in a very narrow <qex>compass</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 289 --></p>

<p><sn>4.</sn> <def>Extent; reach; sweep; capacity; sphere; <as>as, the <ex>compass</ex> of his eye; the <ex>compass</ex> of imagination</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>compass</qex> of his argument.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Moderate bounds, limits of truth; moderation; due limits; -- used with <xex>within</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In two hundred years before (I speak within <qex>compass</qex>), no such commission had been executed.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>The range of notes, or tones, within the capacity of a voice or instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You would sound me from my lowest note to the top of my <qex>compass</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An instrument for determining directions upon the earth's surface by means of a magnetized bar or needle turning freely upon a pivot and pointing in a northerly and southerly direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that first discovered the use of the <qex>compass</qex> did more for the supplying and increase of useful commodities than those who built workhouses.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A pair of compasses.</def> <mark>[R.]</mark> <see>See <er>Compasses.</er></see></p>

<p><q>To fix one foot of their <qex>compass</qex> wherever they please.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A circle; a continent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The tryne <qex>compas</qex> [the threefold world containing earth, sea, and heaven. <au>Skeat.</au>]</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Azimuth compass</b></col>. <cd>See under <er>Azimuth</er>.</cd> -- <col><b>Beam compass</b></col>. <cd>See under <er>Beam</er>.</cd> -- <col><b>Compass card</b></col>, <cd>the circular card attached to the needles of a mariner's compass, on which are marked the thirty-two points or rhumbs.</cd> -- <col><b>Compass dial</b></col>, <cd>a small pocket compass fitted with a sundial to tell the hour of the day.</cd> -- <col><b>Compass plane</b></col> <fld>(Carp.)</fld>, <cd>a plane, convex in the direction of its length on the under side, for smoothing the concave faces of curved woodwork.</cd> -- <mcol><col><b>Compass plant</b></col>, <col><b>Compass flower</b></col></mcol> <fld>(Bot.)</fld>, <cd>a plant of the American prairies (<spn>Silphium laciniatum</spn>), not unlike a small sunflower; rosinweed. Its lower and root leaves are vertical, and on the prairies are disposed to present their edges north and south.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Its leaves are turned to the north as true as the magnet:<br/
This is the <qex>compass flower</qex>.</q> <rj><qau>Longefellow.</qau></rj></p>

<p>-- <col><b>Compass saw</b></col>, <cd>a saw with a narrow blade, which will cut in a curve; -- called also <altname>fret saw</altname> and <altname>keyhole saw</altname>.</cd> -- <col><b>Compass timber</b></col> <fld>(Shipbuilding)</fld>, <cd>curved or crooked timber.</cd> -- <col><b>Compass window</b></col> <fld>(Arch.)</fld>, <cd>a circular bay window or oriel window.</cd> --  <col><b>Mariner's compass</b></col>, <cd>a kind of compass used in navigation. It has two or more magnetic needles permanently attached to a card, which moves freely upon a pivot, and is read with reference to a mark on the box representing the ship's head. The card is divided into thirty-two points, called also <xex>rhumbs</xex>, and the glass-covered box or bowl containing it is suspended in gimbals within the binnacle, in order to preserve its horizontal position.</cd> -- <col><b>Surveyor's compass</b></col>, <cd>an instrument used in surveying for measuring horizontal angles. See <er>Circumferentor</er>.</cd> -- <col><b>Variation compass</b></col>, <cd>a compass of delicate construction, used in observations on the variations of the needle.</cd> -- <col><b>To fetch a compass</b></col>, <cd>to make a circuit.</cd><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compass</ent><br/
<hw>Com"pass</hw> <pr>(k<ucr/m"p<ait/s)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compassed</conjf> <pr>(k<ucr/m"p<ait/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Compassing</conjf>.]</vmorph> <ety>[F. <ets>compasser</ets>, LL. <ets>compassare</ets>.]</ety> <sn>1.</sn> <def>To go about or entirely round; to make the circuit of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall <qex>compass</qex> the city seven times.</q> <rj><qau>Josh. vi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We the globe can <qex>compass</qex> soon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inclose on all sides; to surround; to encircle; to environ; to invest; to besiege; -- used with <ptcl>about</ptcl>, <ptcl>round</ptcl>, <ptcl>around</ptcl>, and <ptcl>round about</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With terrors and with clamors <qex>compassed</qex> round.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now all the blessings<br/
Of a glad father <qex>compass</qex> thee about.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thine enemies shall cast a trench about thee, and <qex>compass</qex> thee round.</q> <rj><qau>Luke xix. 43.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To reach round; to circumvent; to get within one's power; to obtain; to accomplish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If I can check my erring love, I will:<br/
If not, to <qex>compass</qex> her I'll use my skill.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How can you hope to <qex>compass</qex> your designs?</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To curve; to bend into a circular form.</def> <mark>[Obs. except in carpentry and shipbuilding.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>To purpose; to intend; to imagine; to plot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compassing</qex> and <qex>imagining</qex> the death of the king are synonymous terms; <qex>compassing</qex> signifying the purpose or design of the mind or will, and not, as in common speech, the carrying such design to effect.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassable</ent><br/
<hw>Com"pass*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being compassed or accomplished.</def> <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassed</ent><br/
<hw>Com"passed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Rounded; arched.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She came . . . into the <qex>compassed</qex> window.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compasses</ent><br/
<hw>Com"pass*es</hw> <pr>(?)</pr>, <pos>n.</pos>, <pluf>pl.</pluf> <def>An instrument for describing circles, measuring figures, etc., consisting of two, or (rarely) more, pointed branches, or legs, usually joined at the top by a rivet on which they move.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The compasses for drawing circles have adjustable pen points, pencil points, etc.; those used for measuring without adjustable points are generally called <xex>dividers</xex>. See <er>Dividers</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bow compasses</b></col>. <cd>See <er>Bow-compass</er>.</cd> -- <mcol><col><b>Caliber compasses</b></col>, <col><b>Caliper compasses</b></col></mcol>. <cd>See <er>Calipers</er>.</cd> -- <mcol><col><b>Proportional</b></col>, <col><b>Triangular</b></col>, etc., <col><b>compasses</b></col></mcol>. <cd>See under <er>Proportional</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassing</ent><br/
<hw>Com"pass*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Shipbuilding)</fld> <def>Curved; bent; <as>as, <ex>compassing</ex> timbers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassion</ent><br/
<hw>Com*pas"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>compassio</ets>, fr. <ets>compati</ets> to have compassion; <ets>com-</ets> + <ets>pati</ets> to bear, suffer. See <er>Patient</er>.]</ety> <def>Literally, suffering with another; a sensation of sorrow excited by the distress or misfortunes of another; pity; commiseration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Womanly ingenuity set to work by womanly <qex>compassion</qex>.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- Pity; sympathy; commiseration; fellow-feeling; mercy; condolence. See <er>Pity</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassion</ent><br/
<hw>Com*pas"sion</hw>, <pos>v. t.</pos> <def>To pity.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassionable</ent><br/
<hw>Com*pas"sion*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deserving compassion or pity; pitiable.</def> <mark>[R.]</mark> <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassionate</ent><br/
<hw>Com*pas"sion*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a temper or disposition to pity; sympathetic; merciful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There never was any heart truly great and generous, that was not also tender and <qex>compassionate</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Complaining; inviting pity; pitiable.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- Sympathizing; tender; merciful; pitiful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassionate</ent><br/
<hw>Com*pas"sion*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compassionated</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Compassionating</conjf> <pr>(#)</pr>.]</vmorph> <def>To have compassion for; to pity; to commiserate; to sympathize with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compassionates</qex> my pains, and pities me.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassionately</ent><br/
<hw>Com*pas"sion*ate*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a compassionate manner; mercifully.</def> <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassionateness</ent><br/
<hw>Com*pas"sion*ate*ness</hw>, <pos>n.</pos> <def>The quality or state of being compassionate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compassless</ent><br/
<hw>Com"pass*less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no compass.</def> <rj><au>Knowles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compaternity</ent><br/
<hw>Com`pa*ter"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>compaternitas</ets>, fr. <ets>compater</ets> godfather; <ets>com-</ets> + <ets>pater</ets> father.]</ety> <def>The relation of a godfather to a person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The relation of gossipred or <qex>compaternity</qex> by the canon law is a spiritual affinity.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatibility</ent><br/
<hw>Com*pat`i*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>compatibilit<eacute/</ets>.]</ety> <def>The quality or power of being compatible or congruous; congruity; <as>as, a <ex>compatibility</ex> of tempers; a <ex>compatibility</ex> of properties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatible</ent><br/
<hw>Com*pat"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. LL.<ets>compatibilis</ets>, fr. L. <ets>compati</ets>. See <er>Compassion</er>.]</ety> <def>Capable of existing in harmony; congruous; suitable; not repugnant; -- usually followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our poets have joined together such qualities as are by nature the most <qex>compatible</qex>.</q> <rj><qau>Broome.</qau></rj></p>

<p><syn><b>Syn.</b> -- Consistent; suitable; agreeable; accordant.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatibleness</ent><br/
<hw>Com*pat"i*ble*ness</hw>, <pos>n.</pos> <def>Compatibility; consistency; fitness; agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatibly</ent><br/
<hw>Com*pat"i*bly</hw>, <pos>adv.</pos> <def>In a compatible manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatient</ent><br/
<hw>Com*pa"tient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>compatients</ets>, p. pr. of <ets>compati</ets>. See <er>Compassion</er>.]</ety> <def>Suffering or enduring together.</def> <mark>[Obs.]</mark> <rj><au>Sir G. Buck.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatriot</ent><br/
<hw>Com*pa"tri*ot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>compatriote</ets>, LL. <ets>compatriotus</ets>; <ets>com-</ets> + <ets>patriota</ets> a native. See <er>Patriot</er>, and cf. <er>Copatriot</er>.]</ety> <def>One of the same country, and having like interests and feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The distrust with which they felt themselves to be regarded by their <qex>compatriots</qex> in America.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatriot</ent><br/
<hw>Com*pa"tri*ot</hw>, <pos>a.</pos> <def>Of the same country; having a common sentiment of patriotism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She [Britain] rears to freedom an undaunted race,<br/
<qex>Compatriot</qex>, zealous, hospitable, kind.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compatriotism</ent><br/
<hw>Com*pa"tri*ot*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The condition of being compatriots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compear</ent><br/
<hw>Com*pear"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[F. <ets>comparoir</ets>, L. <ets>compar<emac/re</ets>; <ets>com-</ets> + <ets>par<emac/re</ets> to appear.]</ety> <sn>1.</sn> <def>To appear.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To appear in court personally or by attorney.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compeer</ent><br/
<hw>Com*peer"</hw> <pr>(?)</pr>, <ety>[OE. <ets>comper</ets>, through French fr. L. <ets>compar</ets>; <ets>com-</ets> + <ets>par</ets> equal. See <er>Peer</er> an equal, and cf. 1st <er>Compare</er>.]</ety> <def>An equal, as in rank, age, prowess, etc.; a companion; a comrade; a mate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And him thus answer'd soon his bold <qex>compeer</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>compeer</qex> in arms.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compeer</ent><br/
<hw>Com*peer"</hw>, <pos>v. t.</pos> <def>To be equal with; to match.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In my rights,<br/
By me invested, he <qex>compeers</qex> the best.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compeir</ent><br/
<ent>Compeer</ent><br/
<mhw>{ <hw>Com*peer"</hw>, <hw>Com*peir"</hw> <pr>(?)</pr>, }</mhw> <pos>v. i.</pos> <def>See <er>Compear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compel</ent><br/
<hw>Com*pel"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compelled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Compelling</conjf>.]</vmorph> <ety>[L. <ets>compellere</ets>, <ets>compulsum</ets>, to drive together, to compel, urge; <ets>com-</ets> + <ets>pellere</ets> to drive: cf. OF. <ets>compellir</ets>. See <er>Pulse</er>.]</ety> <sn>1.</sn> <def>To drive or urge with force, or irresistibly; to force; to constrain; to oblige; to necessitate, either by physical or moral force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wolsey . . . <qex>compelled</qex> the people to pay up the whole subsidy at once.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And they <qex>compel</qex> one Simon . . . to bear his cross.</q> <rj><qau>Mark xv. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take by force or violence; to seize; to exact; to extort.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Commissions, which <qex>compel</qex> from each<br/
The sixth part of his substance.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To force to yield; to overpower; to subjugate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Easy sleep their weary limbs <qex>compelled</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>compel</qex> all creatures to my will.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To gather or unite in a crowd or company.</def> <mark>[A Latinism]</mark> <ldquo/In one troop <qex>compelled</qex>.<rdquo/ <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To call forth; to summon.</def> <mark>[Obs.]</mark> <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She had this knight from far <qex>compelled</qex>.</q> <rj><qau>Spenser.</qau></rj></p>

<p><syn><b>Syn.</b> -- To force; constrain; oblige; necessitate; coerce. See <er>Coerce</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compel</ent><br/
<hw>Com*pel"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make one yield or submit.</def> <ldquo/If she can not entreat, I can <xex>compel</xex>.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compellable</ent><br/
<hw>Com*pel"la*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being compelled or constrained.</def> <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compellably</ent><br/
<hw>Com*pel"la*bly</hw>, <pos>adv.</pos> <def>By compulsion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compellation</ent><br/
<hw>Com`pel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compellatio</ets>, fr. <ets>compellare</ets> to accost, fr. <ets>compellere</ets>. See <er>Compel</er>.]</ety> <def>Style of address or salutation; an appellation.</def> <ldquo/Metaphorical <xex>compellations</xex>.<rdquo/ <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He useth this endearing <qex>compellation</qex>, <ldquo/My little children.<rdquo/</q> <rj><qau>Bp. Beveridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The peculiar <qex>compellation</qex> of the kings in France is by <ldquo/Sire,<rdquo/ which is nothing else but father.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compellative</ent><br/
<hw>Com*pel"la*tive</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Gram.)</fld> <def>The name by which a person is addressed; an appellative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compellatory</ent><br/
<hw>Com*pel"la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to compel; compulsory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compeller</ent><br/
<hw>Com*pel"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who compels or constrains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compend</ent><br/
<hw>Com"pend</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A compendium; an epitome; a summary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>compend</qex> and recapitulation of the Mosaical law.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compendiarious</ent><br/
<hw>Com*pen`di*a"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>compendiarius</ets>.]</ety> <def>Short; compendious.</def> <mark>[Obs.]</mark> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compendiate</ent><br/
<hw>Com*pen"di*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>compendiatus</ets>, p. p. of <ets>compendiare</ets> to shorten, fr. <ets>compendium</ets>.]</ety> <def>To sum or collect together.</def> <mark>[Obs.]</mark> <rj><au>Bp. King.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compendious</ent><br/
<hw>Com*pen"di*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>compendiosus</ets>.]</ety> <def>Containing the substance or general principles of a subject or work in a narrow compass; abridged; summarized.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>More <qex>compendious</qex> and expeditious ways.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Three things be required in the oration of a man having authority -- that it be <qex>compendious</qex>, sententious, and delectable.</q> <rj><qau>Sir T. Elyot.</qau></rj></p>

<p><syn><b>Syn.</b> -- Short; summary; abridged; condensed; comprehensive; succinct; brief; concise.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compendiously</ent><br/
<hw>Com*pen"di*ous*ly</hw>, <pos>adv.</pos> <def>In a compendious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compendiously</qex> expressed by the word chaos.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compendiousness</ent><br/
<hw>Com*pen"di*ous*ness</hw>, <pos>n.</pos> <def>The state or quality of being compendious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compendium</ent><br/
<hw>Com*pen"di*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Compendiums</plw> <pr>(#)</pr>, L. <plw>Compendia</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>compendium</ets> that which is weighed, saved, or shortened, a short way, fr. <ets>compendere</ets> to weigh; <ets>com-</ets> + <ets>pendere</ets> to weigh. See <er>Pension</er>, and cf. <er>Compend</er>.]</ety> <def>A brief compilation or composition, containing the principal heads, or general principles, of a larger work or system; an abridgment; an epitome; a compend; a condensed summary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A short system or <qex>compendium</qex> of a science.</q> <rj><qau>I. Watts.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Abridgment</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>compensable</ent><br/
<hw>compensable</hw> <pos>adj.</pos> <sn>1.</sn> <def>receiving money for services rendered.</def><br/
<syn><b>Syn. --</b> paying(prenominal), remunerative, salaried, stipendiary.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>qualifying one for receipt of compensation, such as for a bodily injury; <as>as, a <ex>compensable</ex> injury</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Compensate</ent><br/
<hw>Com"pen*sate</hw> <pr>(? <or/ ?; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compensated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Compensating</conjf>.]</vmorph> <ety>[L. <ets>compensatus</ets>, p. p. of <ets>compensare</ets>, prop., to weigh several things with one another, to balance with one another, verb intens. fr. <ets>compendere</ets>. See <er>Compendium</er>.]</ety> <sn>1.</sn> <def>To make equal return to; to remunerate; to recompense; to give an equivalent to; to requite suitably; <as>as, to <ex>compensate</ex> a laborer for his work, or a merchant for his losses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be equivalent in value or effect to; to counterbalance; to make up for; to make amends for.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The length of the night and the dews thereof do <qex>compensate</qex> the heat of the day.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The pleasures of life do not <qex>compensate</qex> the miseries.</q> <rj><qau>Prior.</qau></rj></p>

<p><syn><b>Syn.</b> -- To recompense; remunerate; indemnify; reward; requite; counterbalance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compensate</ent><br/
<hw>Com"pen*sate</hw>, <pos>v. i.</pos> <def>To make amends; to supply an equivalent; -- followed by <ptcl>for</ptcl>; <as>as, nothing can <ex>compensate</ex> for the loss of reputation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>compensated</ent><br/
<hw>compensated</hw> <pos>adj.</pos> <def>receiving or eligible for compensation.</def><br/
<syn><b>Syn. --</b> remunerated, salaried, stipendiary.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Compensation</ent><br/
<hw>Com`pen*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compensatio</ets> a weighing, a balancing of accounts.]</ety> <sn>1.</sn> <def>The act or principle of compensating.</def> <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which constitutes, or is regarded as, an equivalent; that which makes good the lack or variation of something else; that which compensates for loss or privation; amends; remuneration; recompense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The parliament which dissolved the monastic foundations . . . vouchsafed not a word toward securing the slightest <qex>compensation</qex> to the dispossessed owners.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No pecuniary <qex>compensation</qex> can possibly reward them.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <sd>(a)</sd><def>The extinction of debts of which two persons are reciprocally debtors by the credits of which they are reciprocally creditors; the payment of a debt by a credit of equal amount; a set-off.</def> <au>Bouvier.</au> <au>Wharton.</au> <sd>(b)</sd> <def>A recompense or reward for some loss or service.</def> <sd>(c)</sd> <def>An equivalent stipulated for in contracts for the sale of real estate, in which it is customary to provide that errors in description, etc., shall not avoid, but shall be the subject of <xex>compensation</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Compensation balance</b></col>, <it>or</it> <col><b>Compensated balance</b></col></mcol>, <cd>a kind of balance wheel for a timepiece. The rim is usually made of two different metals having different expansibility under changes of temperature, so arranged as to counteract each other and preserve uniformity of movement.</cd> -- <col><b>Compensation pendulum</b></col>. <cd>See <er>Pendulum</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Recompense; reward; indemnification; consideration; requital; satisfaction; set-off.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compensative</ent><br/
<hw>Com*pen"sa*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>compensativus</ets>.]</ety> <def>Affording compensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compensative</ent><br/
<hw>Com*pen"sa*tive</hw>, <pos>n.</pos> <def>Compensation.</def> <mark>[R.]</mark> <rj><au>Lamb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compensator</ent><br/
<hw>Com"pen*sa`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, compensates; -- a name applied to various mechanical devices.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>An iron plate or magnet placed near the compass on iron vessels to neutralize the effect of the ship's attraction on the needle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compensatory</ent><br/
<hw>Com*pen"sa*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving for compensation; making amends.</def> <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 290 --></p>

<p><ent>Compense</ent><br/
<hw>Com*pense"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>compenser</ets>. See <er>Compensate</er>.]</ety> <def>To compensate.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comperendinate</ent><br/
<hw>Com`pe*ren"di*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>comperendinatus</ets>, p. p. of <ets>comperendinare</ets> to defer (the time of trial.)]</ety> <def>To delay.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compesce</ent><br/
<hw>Com*pesce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>compescere</ets>.]</ety> <def>To hold in check; to restrain.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compete</ent><br/
<hw>Com*pete"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Competed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Competing</conjf>.]</vmorph> <ety>[L. <ets>completere</ets>, <ets>competitum</ets>; <ets>com-</ets> + <ets>petere</ets> to seek. See <er>Petition</er>.]</ety> <def>To contend emulously; to seek or strive for the same thing, position, or reward for which another is striving; to contend in rivalry, as for a prize or in business; <as>as, tradesmen <ex>compete</ex> with one another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rival statesmen, with eyes fixed on America, were all the while <qex>competing</qex> for European alliances.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competency</ent><br/
<ent>Competence</ent><br/
<mhw>{ <hw>Com"pe*tence</hw> <pr>(?)</pr>, <hw>Com"pe*ten*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. F. <ets>comp<eacute/tence</ets>, from L. <ets>competentia</ets> agreement.]</ety> <sn>1.</sn> <def>The state of being competent; fitness; ability; adequacy; power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The loan demonstrates, in regard to instrumental resources, the <qex>competency</qex> of this kingdom to the assertion of the common cause.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To make them act zealously is not in the <qex>competence</qex> of law.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Property or means sufficient for the necessaries and conveniences of life; sufficiency without excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Reason's whole pleasure, all the joys of sense,<br/
Lie in three words -- health, peace, and <qex>competence</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Superfluity comes sooner by white hairs, but <qex>competency</qex> lives longer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>Legal capacity or qualifications; fitness; <as>as, the <ex>competency</ex> of a witness or of a evidence</as>.</def> <sd>(b)</sd> <def>Right or authority; legal power or capacity to take cognizance of a cause; <as>as, the <ex>competence</ex> of a judge or court</as>.</def>  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>the quality of being adequately or well qualified physically and intellectually, especially possession of the skill and knowledge required (for a task).</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Competent</ent><br/
<hw>Com"pe*tent</hw> <pr>(k<ocr/m"p<esl/*t<eit/nt; 94)</pr>, <pos>a.</pos> <ety>[F. <ets>comp<eacute/tent</ets>, p. pr. of <ets>comp<eacute/ter</ets> to be in the competency of, LL. <ets>competere</ets> to strive after together, to agree with; hence, to be fit. See <er>Compete</er>.]</ety> <sn>1.</sn> <def>Answering to all requirements; adequate; sufficient; suitable; capable; legally qualified; fit.</def> <ldquo/A <xex>competent</xex> knowledge of the world.<rdquo/ <au>Atterbury.</au> <ldquo/<xex>Competent</xex> age.<rdquo/ <au>Grafton.</au> <ldquo/<xex>Competent</xex> statesmen.<rdquo/ <au>Palfrey.</au> /<ldquo/A <xex>competent</xex> witness.<rdquo/ <au>Bouvier.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rightfully or properly belonging; incident; -- followed by <xex>to</xex>.</def> <mark>[Rare, except in legal usage.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That is the privilege of the infinite Author of things, . . . but is not <qex>competent</qex> to any finite being.</q> <rj><qau>Locke.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Qualified</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competently</ent><br/
<hw>Com"pe*tent*ly</hw>, <pos>adv.</pos> <def>In a competent manner; adequately; suitably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competible</ent><br/
<hw>Com*pet"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Compatible; suitable; consistent.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competition</ent><br/
<hw>Com`pe*ti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>competition</ets>. See <er>Compete</er>.]</ety> <def>The act of seeking, or endeavoring to gain, what another is endeavoring to gain at the same time; common strife for the same objects; strife for superiority; emulous contest; rivalry, as for approbation, for a prize, or as where two or more persons are engaged in the same business and each seeking patronage; -- followed by <xex>for</xex> before the object sought, and <xex>with</xex> before the person or thing competed with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Competition</qex> to the crown there is none, nor can be.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A portrait, with which one of Titian's could not come in <qex>competition</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no <qex>competition</qex> but for the second place.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where <qex>competition</qex> does not act at all there is complete monopoly.</q> <rj><qau>A. T. Hadley.</qau></rj></p>

<p><syn><b>Syn.</b> -- Emulation; rivalry; rivalship; contest; struggle; contention; opposition; jealousy. See <er>Emulation</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competitive</ent><br/
<hw>Com*pet"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to competition; producing competition; competitory; <as>as, a <ex>competitive</ex> examination</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>competitiveness</ent><br/
<hw>competitiveness</hw> <pos>n.</pos> <def>an agressive willingness to compete.</def><br/
<syn><b>Syn. --</b> fight.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Competitor</ent><br/
<hw>Com*pet"i*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>comp<eacute/titeur</ets>.]</ety> <sn>1.</sn> <def>One who seeks what another seeks, or claims what another claims; one who competes; a rival.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And can not brook <qex>competitors</qex> in love.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An associate; a confederate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every hour more <qex>competitors</qex><br/
Flock to their aid, and still their power increaseth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competitory</ent><br/
<hw>Com*pet"i*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acting in competition; competing; rival.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competitress</ent><br/
<hw>Com*pet"i*tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who competes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Competitrix</ent><br/
<hw>Com*pet"i*trix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A competitress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compilation</ent><br/
<hw>Com"pi*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compilatio</ets>: cf. F. <ets>compilation</ets>.]</ety> <sn>1.</sn> <def>The act or process of compiling or gathering together from various sources.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is compiled; especially, a book or document composed of materials gathering from other books or documents.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His [Goldsmith's] <qex>compilations</qex> are widely distinguished from the <qex>compilations</qex> of ordinary bookmakers.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compilator</ent><br/
<hw>Com"pi*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Compiler.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compile</ent><br/
<hw>Com*pile"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compiled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Compiling</conjf>.]</vmorph> <ety>[F. <ets>compiler</ets>, fr.L. <ets>compilare</ets> to plunder, pillage; <ets>com-</ets> + <ets>pilare</ets> to plunder. See <er>Pill</er>, <pos>v. t.</pos>, <ets>Pillage</ets>.]</ety> <sn>1.</sn> <def>To put together; to construct; to build.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Before that Merlin died, he did intend<br/
A brazen wall in compass to <qex>compile</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To contain or comprise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which these six books <qex>compile</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put together in a new form out of materials already existing; esp., to put together or compose out of materials from other books or documents.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Goldsmith] <qex>compiled</qex> for the use of schools a History of Rome.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To write; to compose.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Computers)</fld> <def>to process (computer program source code) with a compiler{2} to produce an assembly-language program or an executable program in machine language.</def><br/
[<source>PJC</source>]</p>

<p><ent>Compilement</ent><br/
<hw>Com*pile"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Compilation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compiler</ent><br/
<hw>Com*pil"er</hw> <pr>(k<ocr/m*p<imac/l"<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>compiluor</ets>; cf. OF. <ets>compileor</ets>, fr. L. <ets>compilator</ets>.]</ety> <sn>1.</sn> <def>One who compiles; esp., one who makes books by compilation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Computers)</fld> <def>a computer program that decodes instructions written in a higher-level computer language to produce an assembly-language program or an executable program in machine language.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Compinge</ent><br/
<hw>Com*pinge"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>compingere</ets>.]</ety> <def>To compress; to shut up.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complacency</ent><br/
<ent>Complacence</ent><br/
<mhw>{ <hw>Com*pla"cence</hw> <pr>(?)</pr>, <hw>Com*pla"cen*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[LL. <ets>complacentia</ets>: cf. F. <ets>complaisance</ets>. See <er>Complacent</er>, and cf. <er>Complaisance</er>.]</ety> <sn>1.</sn> <def>Calm contentment; satisfaction; gratification.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The inward <qex>complacence</qex> we find in acting reasonably and virtuously.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Others proclaim the infirmities of a great man with satisfaction and <qex>complacency</qex>, if they discover none of the like in themselves.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cause of pleasure or joy.</def> <ldquo/O thou, my sole <xex>complacence</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The manifestation of contentment or satisfaction; good nature; kindness; civility; affability.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Complacency</qex>, and truth, and manly sweetness,<br/
Dwell ever on his tongue, and smooth his thoughts.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With mean <qex>complacence</qex> ne'er betray your trust.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complacent</ent><br/
<hw>Com*pla"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>complacens</ets> very pleasing, p. pr. of <ets>complacere</ets>; <ets>com-</ets> + <ets>placere</ets> to please: cf. F. <ets>complaisant</ets>. See <er>Please</er> and cf. <er>Complaisant</er>.]</ety> <def>Self-satisfied; contented; kindly; <as>as, a <ex>complacent</ex> temper; a <ex>complacent</ex> smile</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They look up with a sort of <qex>complacent</qex> awe . . . to kings.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complacential</ent><br/
<hw>Com`pla*cen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Marked by, or causing, complacence.</def> <mark>[Obs.]</mark> <ldquo/<xex>Complacential</xex> love.<rdquo/  <rj><au>Baxter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complacently</ent><br/
<hw>Com*pla"cent*ly</hw> <pr>(k<ocr/m*pl<amac/"s<eit/nt*l<ycr/)</pr>, <pos>adv.</pos> <def>In a complacent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complain</ent><br/
<hw>Com*plain"</hw> <pr>(k<ocr/m*pl<amac/n")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Complained</conjf> <pr>(k<ocr/m*pl<amac/nd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Complaining</conjf>.]</vmorph> <ety>[F. <ets>complaindre</ets>, LL. <ets>complangere</ets>; <ets>com-</ets> + L. <ets>plangere</ets> to strike, beat, to beat the breast or head as a sign of grief, to lament. See <er>Plaint</er>.]</ety> <sn>1.</sn> <def>To give utterance to expression of grief, pain, censure, regret. etc.; to lament; to murmur; to find fault; -- commonly used with <xex>of</xex>. Also, to creak or squeak, as a timber or wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O loss of sight, of thee I most <qex>complain</qex>!</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a formal accusation; to make a charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, Master Shallow, you'll <qex>complain</qex> of me to the king?</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To repine; grumble; deplore; bewail; grieve; mourn; regret; murmur.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complain</ent><br/
<hw>Com*plain"</hw>, <pos>v. t.</pos> <def>To lament; to bewail.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They might the grievance inwardly <qex>complain</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By chaste Lucrece's soul that late <qex>complain'd</qex><br/
Her wrongs to us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complainable</ent><br/
<hw>Com*plain"a*ble</hw> <pr>(k<ocr/m*pl<amac/n"<adot/*b'l)</pr>, <pos>a.</pos> <def>That may be complained of.</def> <mark>[R.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complainant</ent><br/
<hw>Com*plain"ant</hw> <pr>(k<ocr/m*pl<amac/n"<ait/nt)</pr>, <pos>n.</pos> <ety>[F. <ets>complaignant</ets>, p. pr. of <ets>complaindre</ets>.]</ety> <sn>1.</sn> <def>One who makes complaint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Eager <qex>complainants</qex> of the dispute.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>One who commences a legal process by a complaint.</def> <sd>(b)</sd> <def>The party suing in equity, answering to the plaintiff at common law.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He shall forfeit one moiety to the use of the town, and the other moiety to the use of the <qex>complainant</qex>.</q> <rj><qau>Statutes of Mass.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complainer</ent><br/
<hw>Com*plain"er</hw> <pr>(k<ocr/m*pl<amac/n"<etil/r)</pr>, <pos>n.</pos> <def>One who complains or laments; one who finds fault; a murmurer.</def>  <rj><au>Beattie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Speechless <qex>complainer</qex>, I will learn thy thought.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>complaining</ent><br/
<hw>complaining</hw> <pr>(k<ocr/m*pl<amac/n"<icr/mg)</pr> <pos>adj.</pos>  <def>uttering complaints.  Opposite of <ant>uncomplaining</ant>.</def> <mark>[prenominal]</mark> <note>[Narrower terms: <stype>faultfinding, grumbling(prenominal)</stype>: <stype>fretful, querulous, whiney, whining(prenominal), whiny</stype>; <stype>protesting(prenominal), protestant</stype>]</note><br/
<syn><b>Syn. --</b> complaintive.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Complaint</ent><br/
<hw>Com*plaint"</hw> <pr>(k<ocr/m*pl<amac/nt")</pr>, <pos>n.</pos> <ety>[F. <ets>complainte</ets>. See <er>Complain</er>.]</ety> <sn>1.</sn> <def>Expression of grief, regret, pain, censure, or resentment; lamentation; murmuring; accusation; fault-finding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I poured out my <qex>complaint</qex> before him.</q> <rj><qau>Ps. cxlii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Grievous <qex>complaints</qex> of you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cause or subject of complaint or murmuring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The poverty of the clergy in England hath been the <qex>complaint</qex> of all who wish well to the church.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An ailment or disease of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One in a <qex>complaint</qex> of his bowels.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>A formal allegation or charge against a party made or presented to the appropriate court or officer, as for a wrong done or a crime committed (in the latter case, generally under oath); an information; accusation; the initial bill in proceedings in equity.</def></p>

<p><syn><b>Syn.</b> -- Lamentation; murmuring; sorrow; grief; disease; illness; disorder; malady; ailment.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complaintful</ent><br/
<hw>Com*plaint"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of complaint.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complaisance</ent><br/
<hw>Com"plai*sance`</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>complaisance</ets>. See <er>Complaisant</er>, and cf. <er>Complacence</er>.]</ety> <def>Disposition to please or oblige; obliging compliance with the wishes of others; a deportment indicative of a desire to please; courtesy; civility.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These [ladies] . . . are by the just <qex>complaisance</qex> and gallantry of our nation the most powerful part of our people.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They strive with their own hearts and keep them down,<br/
In <qex>complaisance</qex> to all the fools in town.</q> <rj><qau>Young.</qau></rj></p>

<p><syn><b>Syn.</b> -- Civility; courtesy; urbanity; suavity; affability; good breeding.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complaisant</ent><br/
<hw>Com"plai*sant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>complaisant</ets>, p. pr. of <ets>complaire</ets> to acquiesce as a favor, fr. L. <ets>complacere</ets>. See <er>Complacent</er>.]</ety> <def>Desirous to please; courteous; obliging; compliant; <as>as, a <ex>complaisant</ex> gentleman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There are to whom my satire seems too bold:<br/
Scarce to wise Peter <qex>complaisant</qex> enough.</q> <rj><qau>Pope.</qau></rj></p>

<p><syn><b>Syn.</b> -- Obliging; courteous; affable; gracious; civil; polite; well-bred. See <er>Obliging</er>.</syn></p>

<p>-- <wordforms><wf>Com"plai*sant`ly</wf>, <pos>adv.</pos> -- <wf>Com"plai*sant`ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complanar</ent><br/
<hw>Com*pla"nar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Coplanar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complanate</ent><br/
<hw>Com"pla*nate</hw> <pr>(? <or/ <?/)</pr>, <pos>a.</pos> <ety>[L. <ets>complanatus</ets>, p. p. of <ets>complanare</ets> to make plane. See <er>Plane</er>, <pos>v. t.</pos>]</ety> <def>Flattened to a level surface.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complanate</ent><br/
<hw>Com"pla*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make level.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complected</ent><br/
<hw>Com*plect"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Complexioned.</def> <mark>[Low, New Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complement</ent><br/
<hw>Com"ple*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>complementun</ets>: cf. F. <ets>compl<eacute/ment</ets>. See <er>Complete</er>, <pos>v. t.</pos>, and cf. <er>Compliment</er>.]</ety> <sn>1.</sn> <def>That which fills up or completes; the quantity or number required to fill a thing or make it complete.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is required to supply a deficiency, or to complete a symmetrical whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>History is the <qex>complement</qex> of poetry.</q> <rj><qau>Sir J. Stephen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Full quantity, number, or amount; a complete set; completeness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To exceed his <qex>complement</qex> and number appointed him which was one hundred and twenty persons.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Math.)</fld> <def>A second quantity added to a given quantity to make it equal to a third given quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Something added for ornamentation; an accessory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Without vain art or curious <qex>complements</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>The whole working force of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mus.)</fld> <def>The interval wanting to complete the octave; -- the fourth is the <xex>complement</xex> of the fifth, the sixth of the third.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A compliment.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Arithmetical compliment of a logarithm</b></col>. <cd>See under <er>Logarithm</er>.</cd> -- <col><b>Arithmetical complement of a number</b></col> <fld>(Math.)</fld>, <cd>the difference between that number and the next higher power of 10; as, 4 is the <xex>complement</xex> of 6, and 16 of 84.</cd> -- <mcol><col><b>Complement of an arc</b></col> <it>or</it> <col><b>Complement of an angle</b></col></mcol> <fld>(Geom.)</fld>, <cd>the difference between that arc or angle and 90<deg/.</cd> -- <col><b>Complement of a parallelogram</b></col>. <fld>(Math.)</fld> <cd>See <er>Gnomon</er>.</cd> -- <col><b>In her complement</b></col> <fld>(Her.)</fld>, <cd>said of the moon when represented as full.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complement</ent><br/
<hw>Com"ple*ment</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To supply a lack; to supplement.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To compliment.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complemental</ent><br/
<hw>Com`ple*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Supplying, or tending to supply, a deficiency; fully completing.</def> <ldquo/<xex>Complemental</xex> ceremony.<rdquo/  <rj><au>Prynne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Complimentary; courteous.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Complemental air</b></col> <fld>(Physiol.)</fld>, <cd>the air (averaging 100 cubic inches) which can be drawn into the lungs in addition to the tidal air, by the deepest possible inspiration.</cd> -- <col><b>Complemental males</b></col> <fld>(Zool.)</fld>, <cd>peculiar small males living parasitically on the ordinary hermaphrodite individuals of certain barnacles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complementary</ent><br/
<hw>Com`ple*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to fill out or to complete; <as>as, <ex>complementary</ex> numbers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Complementary colors</b></col>. <cd>See under <er>Color</er>.</cd> -- <col><b>Complementary angles</b></col> <fld>(Math.)</fld>, <cd>two angles whose sum is 90<deg/.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complementary</ent><br/
<hw>Com`ple*men"ta*ry</hw>, <pos>n.</pos> <ety>[See <er>Complimentary</er>.]</ety> <def>One skilled in compliments.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>complementation</ent><br/
<hw>complementation</hw> <pos>n.</pos> <fld>(Linguistics)</fld> <def>the grammatical relation of a word or phrase to a predicate.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Complete</ent><br/
<hw>Com*plete"</hw> <pr>(k<ocr/m*pl<emac/t")</pr>, <pos>a.</pos> <ety>[L. <ets>completus</ets>, p. p. of <ets>complere</ets> to fill up; <ets>com-</ets> + <ets>plere</ets> to fill. See <er>Full</er>, <pos>a.</pos>, and cf. <er>Comply</er>, <er>Compline</er>.]</ety> <sn>1.</sn> <def>Filled up; with no part or element lacking; free from deficiency; entire; perfect; consummate.</def> <ldquo/<xex>Complete</xex> perfections.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye are <qex>complete</qex> in him.</q> <rj><qau>Col. ii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That thou, dead corse, again in <qex>complete</qex> steel<br/
Revisit'st thus the glimpses of the moon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Finished; ended; concluded; completed; <as>as, the edifice is <ex>complete</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This course of vanity almost <qex>complete</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>Having all the parts or organs which belong to it or to the typical form; having calyx, corolla, stamens, and pistil.</def></p>

<p><syn><b>Syn.</b> -- See <er>Whole</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complete</ent><br/
<hw>Com*plete"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Completed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Completing</conjf>.]</vmorph> <def>To bring to a state in which there is no deficiency; to perfect; to consummate; to accomplish; to fulfill; to finish; <as>as, to <ex>complete</ex> a task, or a poem; to <ex>complete</ex> a course of education</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bred only and <qex>completed</qex> to the taste<br/
Of lustful appetence.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And, to <qex>complete</qex> her bliss, a fool for mate.</q> <rj><qau>Pope.</qau></rj></p>

<p><syn><b>Syn.</b> -- To perform; execute; terminate; conclude; finish; end; fill up; achieve; realize; effect; consummate; accomplish; effectuate; fulfill; bring to pass.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>completed</ent><br/
<hw>completed</hw> <pos>adj.</pos> <sn>1.</sn> <def>brought to a conclusion.</def><br/
<syn><b>Syn. --</b> concluded, ended, over(predicate), all over, terminated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>successfully accomplished or achieved.</def><br/
<syn><b>Syn. --</b> accomplished, effected, realized.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>completed by the first act of sexual intercourse after the ceremony; consummated; -- of a marriage.  Opposite of <ant>unconsummated</ant>.</def><br/
<syn><b>Syn. --</b> consummated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn> <fld>(Football)</fld> <def>caught in bounds by a player on the same side as the passer; -- said of a forward pass; <as>as, Simms had six <ex>completed</ex> passes out of seven tries in the game.</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Completely</ent><br/
<hw>Com*plete"ly</hw>, <pos>adv.</pos> <def>In a complete manner; fully.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Completement</ent><br/
<hw>Com*plete"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of completing or perfecting; completion.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Completeness</ent><br/
<hw>Com*plete"ness</hw>, <pos>n.</pos> <def>The state of being complete.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>completing</ent><br/
<hw>completing</hw> <pos>adj.</pos>  <def>serving to complete.</def><br/
<syn><b>Syn. --</b> complemental, complementary, supplemental, supplementary.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Completion</ent><br/
<hw>Com*ple"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>completio</ets> a filling, a fulfillment.]</ety> <sn>1.</sn> <def>The act or process of making complete; the getting through to the end; <as>as, the <ex>completion</ex> of an undertaking, an education, a service</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>completion</qex> of some repairs.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>State of being complete; fulfillment; accomplishment; realization.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Predictions receiving their <qex>completion</qex> in Christ.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Completive</ent><br/
<hw>Com*ple"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>completivus</ets>: cf. F. <ets>compl<eacute/tif</ets>.]</ety> <def>Making complete.</def> <mark>[R.]</mark>  <rj><au>J. Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Completory</ent><br/
<hw>Com*ple"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to fulfill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Completory</qex> of ancient presignifications.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Completory</ent><br/
<hw>Com"ple*to"ry</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[L. <ets>completorium</ets>.]</ety> <fld>(Eccl.)</fld> <def>Same as <er>Compline</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complex</ent><br/
<hw>Com"plex</hw> <pr>(k<ocr/m"pl<ecr/ks)</pr>, <pos>a.</pos> <ety>[L. <ets>complexus</ets>, p. p. of <ets>complecti</ets> to entwine around, comprise; <ets>com-</ets> + <ets>plectere</ets> to twist, akin to <ets>plicare</ets> to fold. See <er>Plait</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Composed of two or more parts; composite; not simple; <as>as, a <ex>complex</ex> being; a <ex>complex</ex> idea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ideas thus made up of several simple ones put together, I call <qex>complex</qex>; such as beauty, gratitude, a man, an army, the universe.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Involving many parts; complicated; intricate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the actual motions of the heavens are calculated in the best possible way, the process is difficult and <qex>complex</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Complex fraction</b></col>. <cd>See <er>Fraction</er>.</cd> -- <col><b>Complex number</b></col> <fld>(Math.)</fld>, <cd>in the theory of numbers, an expression of the form <mathex>a + b<root/-1</mathex>, when <it>a</it> and <it>b</it> are ordinary integers.</cd></cs></p>

<p><syn><b>Syn.</b> -- See <er>Intricate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complex</ent><br/
<hw>Com"plex</hw>, <pos>n.</pos> <ety>[L. <ets>complexus</ets>]</ety> <def>Assemblage of related things; collection; complication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This parable of the wedding supper comprehends in it the whole <qex>complex</qex> of all the blessings and privileges exhibited by the gospel.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Complex of lines</b></col> <fld>(Geom.)</fld>,  <cd>all the possible straight lines in space being considered, the entire system of lines which satisfy a single relation constitute a <xex>complex</xex>; <as>as, all the lines which meet a given curve make up a <ex>complex</ex></as>. The lines which satisfy two relations constitute a <xex>congruency</xex> of lines; as, the entire system of lines, each one of which meets two given surfaces, is a <xex>congruency</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 291 --></p>

<p><ent>Complexed</ent><br/
<hw>Com*plexed"</hw> <pr>(k<ocr/m*pl<ecr/kst")</pr>, <pos>a.</pos> <def>Complex, complicated.</def> <mark>[Obs.]</mark> <ldquo/<xex>Complexed</xex> significations.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexedness</ent><br/
<hw>Com*plex"ed*ness</hw> <pr>(k<ocr/m*pl<ecr/ks"<ecr/d*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being complex or involved; complication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>complexedness</qex> of these moral ideas.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexion</ent><br/
<hw>Com*plex"ion</hw> <pr>(k<ocr/m*pl<ecr/k"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>complexion</ets>, fr. L. <ets>complexio</ets>. See <er>Complex</er>, a.]</ety> <sn>1.</sn> <def>The state of being complex; complexity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though the terms of propositions may be complex, yet . . . it is properly called a simple syllogism, since the <qex>complexion</qex> does not belong to the syllogistic form of it.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A combination; a complex.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This paragraph is . . . a <qex>complexion</qex> of sophisms.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The bodily constitution; the temperament; habitude, or natural disposition; character; nature.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If his <qex>complexion</qex> incline him to melancholy.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is the <qex>complexion</qex> of them all to leave the dam.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The color or hue of the skin, esp. of the face.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tall was her stature, her <qex>complexion</qex> dark.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Between the pale <qex>complexion</qex> of true love,<br/
And the red glow of scorn and proud disdain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The general appearance or aspect; <as>as, the <ex>complexion</ex> of the sky; the <ex>complexion</ex> of the news</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexional</ent><br/
<hw>Com*plex"ion*al</hw> <pr>(-<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to constitutional complexion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A moral rather than a <qex>complexional</qex> timidity.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexionally</ent><br/
<hw>Com*plex"ion*al*ly</hw>, <pos>adv.</pos> <def>Constitutionally.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though corruptible, not <qex>complexionally</qex> vicious.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexionary</ent><br/
<hw>Com*plex"ion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the complexion, or to the care of it.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexioned</ent><br/
<hw>Com*plex"ioned</hw> <pr>(k<ocr/m*pl<ecr/k"sh<ucr/nd)</pr>, <pos>a.</pos> <def>Having (such) a complexion; -- used in composition; <as>as, a dark-<ex>complexioned</ex> or a ruddy-<ex>complexioned</ex> person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A flower is the best-<qex>complexioned</qex> grass, as a pearl is the best-colored clay.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexity</ent><br/
<hw>Com*plex"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Complexities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>complexit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state of being complex; intricacy; entanglement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The objects of society are of the greatest possible <qex>complexity</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is complex; intricacy; complication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many-corridored <qex>complexities</qex><br/
Of Arthur's palace.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexly</ent><br/
<hw>Com"plex`ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a complex manner; not simply.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexness</ent><br/
<hw>Com"plex`ness</hw>, <pos>n.</pos> <def>The state of being complex; complexity.</def>  <rj><au>A. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complexus</ent><br/
||<hw>Com*plex"us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., an embracing.]</ety> <def>A complex; an aggregate of parts; a complication.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliable</ent><br/
<hw>Com*pli"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of bending or yielding; apt to yield; compliant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Another <qex>compliable</qex> mind.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Jews . . . had made their religion <qex>compliable</qex>, and accommodated to their passions.</q> <rj><qau>Jortin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliance</ent><br/
<hw>Com*pli"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Comply</er>.]</ety> <sn>1.</sn> <def>The act of complying; a yielding; as to a desire, demand, or proposal; concession; submission.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>compliances</qex> will remove dissension?</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ready <qex>compliance</qex> with the wishes of his people.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A disposition to yield to others; complaisance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man of few words and of great <qex>compliance</qex>.</q> <rj><qau>Clarendon.</qau></rj></p>

<p><syn><b>Syn.</b> -- Concession; submission; consent; obedience; performance; execution; acquiescence; assent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliancy</ent><br/
<hw>Com*pli"an*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Compliance; disposition to yield to others.</def>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliant</ent><br/
<hw>Com*pli"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Yielding; bending; pliant; submissive.</def> <ldquo/The <xex>compliant</xex> boughs.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliantly</ent><br/
<hw>Com*pli"ant*ly</hw>, <pos>adv.</pos> <def>In a compliant manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicacy</ent><br/
<hw>Com"pli*ca*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A state of being complicate or intricate.</def>  <rj><au>Mitford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicant</ent><br/
<hw>Com"pli*cant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>complicans</ets>, p. pr.]</ety> <fld>(Zool.)</fld> <def>Overlapping, as the elytra of certain beetles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicate</ent><br/
<hw>Com"pli*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>complicatus</ets>, p. p. of <ets>complicare</ets> to fold together. See <er>Complex</er>.]</ety> <sn>1.</sn> <def>Composed of two or more parts united; complex; complicated; involved.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How poor, how rich, how abject, how august,<br/
How <qex>complicate</qex>, how wonderful is man!</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Folded together, or upon itself, with the fold running lengthwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicate</ent><br/
<hw>Com"pli*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Complicated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Complicating</conjf>.]</vmorph> <def>To fold or twist together; to combine intricately; to make complex; to combine or associate so as to make intricate or difficult.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor can his <qex>complicated</qex> sinews fail.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Avarice and luxury very often become one <qex>complicated</qex> principle of action.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When the disease is <qex>complicated</qex> with other diseases.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicately</ent><br/
<hw>Com"pli*cate*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a complex manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicateness</ent><br/
<hw>Com"pli*cate*ness</hw>, <pos>n.</pos> <def>Complexity.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>complicating</ent><br/
<hw>complicating</hw> <pos>n.</pos> <def>the act or process of making something more complex.</def><br/
<syn><b>Syn. --</b> complication.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Complication</ent><br/
<hw>Com`pli*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compliasion</ets>: cf. F. <ets>complication</ets>.]</ety> <sn>1.</sn> <def>The act or process of complicating; the state of being complicated; intricate or confused relation of parts; entanglement; complexity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>complication</qex> of diseases.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Through and beyond these dark <qex>complications</qex> of the present, the New England founders looked to the great necessities of future times.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A disease or diseases, or adventitious circumstances or conditions, coexistent with and modifying a primary disease, but not necessarily connected with it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complice</ent><br/
<hw>Com"plice</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Complices</plw> <pr>(#)</pr>.</plu> <ety>[F., fr. L. <ets>complex</ets>, <ets>-plicis</ets>, closely connected with one, confederate. See <er>Complicate</er>, and cf. <er>Accomplice</er>.]</ety> <def>An accomplice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To quell the rebels and their <qex>complices</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complicity</ent><br/
<hw>Com*plic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Complicities</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>complicit<eacute/</ets>.]</ety> <def>The state of being an accomplice; participation in guilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complier</ent><br/
<hw>Com*pli"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who complies, yields, or obeys; one of an easy, yielding temper.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliment</ent><br/
<hw>Com"pli*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>compliment</ets>. It <ets>complimento</ets>, fr. <ets>comlire</ets> to compliment, finish, suit, fr. L. <ets>complere</ets> to fill up. See <er>Complete</er>, and cf. <er>Complement</er>.]</ety> <def>An expression, by word or act, of approbation, regard, confidence, civility, or admiration; a flattering speech or attention; a ceremonious greeting; <as>as, to send one's <ex>compliments</ex> to a friend</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tedious waste of time, to sit and hear<br/
So many hollow <qex>compliments</qex> and lies.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a <qex>compliment</qex> politely penned.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To make one a compliment</b></col>, <cd>to show one respect; to praise one in a flattering way.</cd> <au>Locke.</au> -- <col><b>To make one's compliments to</b></col>, <cd>to offer formal courtesies to.</cd> -- <col><b>To stand on compliment</b></col>, <cd>to treat with ceremony.</cd></cs></p>

<p><syn><b>Syn.</b> -- See <er>Adulation</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliment</ent><br/
<hw>Com"pli*ment</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To praise, flatter, or gratify, by expressions of approbation, respect, or congratulation; to make or pay a compliment to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Monarchs should their inward soul disguise; . . . <br/
Should <qex>compliment</qex> their foes and shun their friends.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To praise; flatter; adulate; commend.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compliment</ent><br/
<hw>Com"pli*ment</hw>, <pos>v. i.</pos> <def>To pass compliments; to use conventional expressions of respect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I make the interlocutors, upon occasion, <qex>compliment</qex> with one another.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complimental</ent><br/
<hw>Com`pli*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Complimentary.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Languages . . . grow rich and abundant in <qex>complimental</qex> phrases, and such froth.</q> <rj><qau>Sir H. Wotton.</qau></rj></p>

<p>-- <wordforms><wf>Com`pli*men"tal*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark> <au>Boyle.</au>  -- <wf>Com`pli*men"tal*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark>  <au>Hammond.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complimentary</ent><br/
<hw>Com`pli*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Expressive of regard or praise; of the nature of, or containing, a compliment; <as>as, a <ex>complimentary</ex> remark; a <ex>complimentary</ex> ticket</as>.</def> <ldquo/<xex>Complimentary</xex> addresses.<rdquo/  <rj><au>Prescott.</au></rj><-- 2. free; gratis; -- usu designating free items or services given to a customer by a business as a courtesy; as, a complimentary beverage on an airline flight  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complimentative</ent><br/
<hw>Com`pli*men"ta*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Complimentary.</def> <mark>[R.]</mark>  <rj><au>Boswell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complimenter</ent><br/
<hw>Com"pli*ment`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who compliments; one given to complimenting; a flatterer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complin</ent><br/
<ent>Compline</ent><br/
<mhw>{ <hw>Com"pline</hw>, <hw>Com"plin</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From OE. <ets>complie</ets>, OF. <ets>complie</ets>, F. <ets>complies</ets>, pl., fr. LL. <ets>completa</ets> (prop. fem. of L. <ets>completus</ets>) the religious exercise which completes and closes the service of the day. See <er>Complete</er>.]</ety> <fld>(Eccl.)</fld> <def>The last division of the Roman Catholic breviary; the seventh and last of the canonical hours of the Western church; the last prayer of the day, to be said after sunset.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The custom of godly man been to shut up the evening with a <qex>compline</qex> of prayer at nine of the night.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complot</ent><br/
<hw>Com"plot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>complot</ets>, prob. for <ets>comploit</ets>, fr.L. <ets>complicitum</ets>, prop. p. p. of <ets>complicare</ets>, but equiv. to <ets>complicatio</ets> complication, entangling. See <er>Complicate</er>, and cf. <er>Plot</er>.]</ety> <def>A plotting together; a confederacy in some evil design; a conspiracy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I know their <qex>complot</qex> is to have my life.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complot</ent><br/
<hw>Com*plot"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Complotted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Complotting</conjf>.]</vmorph> <ety>[Cf. F. <ets>comploter</ets>, fr. <ets>complot</ets>.]</ety> <def>To plot or plan together; to conspire; to join in a secret design.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We find them <qex>complotting</qex> together, and contriving a new scene of miseries to the Trojans.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complotment</ent><br/
<hw>Com*plot"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A plotting together.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complotter</ent><br/
<hw>Com*plot"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One joined in a plot.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Complutensian</ent><br/
<hw>Com`plu*ten"sian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <etsep>Complutum</etsep> (now Alcala de Henares) a city near Madrid; <as>as, the <ex>Complutensian</ex> Bible</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compluvium</ent><br/
||<hw>Com*plu"vi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Arch.)</fld> <def>A space left unroofed over the court of a Roman dwelling, through which the rain fell into the <xex>impluvium</xex> or cistern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comply</ent><br/
<hw>Com*ply"</hw> <pr>(k<ocr/m*pl<imac/")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Complied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Complying</conjf>.]</vmorph> <ety>[Perh. formed fr. <ets>compliment</ets>, influenced by <ets>ply</ets>, <ets>pliant</ets>, which are of different origin: cf. It. <ets>complire</ets> to compliment, finish, suit. See <er>Compliment</er>, <er>Complete</er>.]</ety> <sn>1.</sn> <def>To yield assent; to accord; agree, or acquiesce; to adapt one's self; to consent or conform; -- usually followed by <ptcl>with</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet this be sure, in nothing to <qex>comply</qex>,<br/
Scandalous or forbidden in our law.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They did servilely <qex>comply</qex> with the people in worshiping God by sensible images.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that <qex>complies</qex> against his will<br/
Is of his own opinion still.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be ceremoniously courteous; to make one's compliments.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comply</ent><br/
<hw>Com*ply"</hw>, <pos>v. t.</pos> <ety>[See <er>comply</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To fulfill; to accomplish.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. L. <ets>complicare</ets> to fold up. See <er>Ply</er>.]</ety> <def>To infold; to embrace.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Seemed to <qex>comply</qex>,<br/
Cloudlike, the daintie deitie.</q> <rj><qau>Herrick.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>complying</ent><br/
<hw>complying</hw> <pos>adj.</pos> <def><pos>pr. p.</pos> of <er>comply</er>.</def> [Narrower terms: <stype>compliant (vs. defiant)</stype>]<br/
<syn><b>Syn. --</b> obliging, yielding.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Compo</ent><br/
<hw>Com"po</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-pos</plw> <pr>(#)</pr>.</plu> <def>Short for <er>Composition</er>; -- used, esp. in England, colloq. in various trade applications;</def> as : <sd>(a)</sd> <def>A mortar made of sand and cement.</def>  <sd>(b)</sd> <def>A carver's mixture of resin, whiting, and glue, used  instead of plaster of Paris for ornamenting walls and cornices.</def> <sd>(c)</sd> <def>A composition for billiard balls.</def> <sd>(d)</sd> <def>A preparation of which printer's rollers are made.</def> <sd>(e)</sd> <def>A preparation used in currying leather.</def> <sd>(f)</sd> <def>Composition paid by a debtor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Compone</ent><br/
<hw>Com*pone"</hw> <pr>(-p<omac/n")</pr>, <pos>v. t.</pos> <ety>[L. <ets>componere</ets>. See <er>Compound</er>.]</ety> <def>To compose; to settle; to arrange.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A good pretense for <qex>componing</qex> peace.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compone</ent><br/
||<hw>Com*po"n<eacute/</hw> <pr>(k<ocr/m*p<omac/"n<asl/)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>See <er>Compony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Component</ent><br/
<hw>Com*po"nent</hw> <pr>(k<ocr/m*p<omac/"n<eit/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>componens</ets>, p. pr. of <ets>componere</ets>. See <er>Compound</er>, <pos>v. t.</pos>]</ety> <def>Serving, or helping, to form; composing; constituting; constituent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>component</qex> parts of natural bodies.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Component</ent><br/
<hw>Com*po"nent</hw>, <pos>n.</pos> <def>A constituent part; an ingredient.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Component of force</b></col> <fld>(Mech.)</fld>, <cd>a force which, acting conjointly with one or more forces, produces the effect of a single force or resultant; one of a number of forces into which a single force may be resolved.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compone</ent><br/
<ent>Compony</ent><br/
<mhw>{ <hw>Com*po"ny</hw> <pr>(?)</pr>, ||<hw>Com*po"n<eacute/</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <ety>[F. compon<eacute/.]</ety> <fld>(Her.)</fld> <def>Divided into squares of alternate tinctures in a single row; -- said of any bearing; or, in the case of a bearing having curved lines, divided into patches of alternate colors following the curve. If there are two rows it is called <xex>counter-compony</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comport</ent><br/
<hw>Com*port"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Comported</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Comporting</conjf>.]</vmorph> <ety>[F. <ets>comporter</ets>, <ets>LL</ets>. <ets>comportare</ets>, fr.L. <ets>comportare</ets> to bring together; <ets>com-</ets> + <ets>portare</ets> to carry. See <er>Port demeanor</er>.]</ety> <sn>1.</sn> <def>To bear or endure; to put up (with); <as>as, to <ex>comport</ex> with an injury</as>.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To agree; to accord; to suit; -- sometimes followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How ill this dullness doth <qex>comport</qex> with greatness.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How their behavior herein <qex>comported</qex> with the institution.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comport</ent><br/
<hw>Com*port"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To bear; to endure; to brook; to put with.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The malcontented sort<br/
That never can the present state <qex>comport</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To carry; to conduct; -- with a reflexive pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Observe how Lord Somers . . . <qex>comported</qex> himself.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comport</ent><br/
<hw>Com"port</hw> <pr>(?, formerly <?/)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>comport</ets>.]</ety> <def>Manner of acting; behavior; conduct; deportment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I knew them well, and marked their rude <qex>comport</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comportable</ent><br/
<hw>Com*port"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suitable; consistent.</def> <mark>[Obs.]</mark> <ldquo/Some <xex>comportable</xex> method.<rdquo/  <rj><au>Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comportance</ent><br/
<hw>Com*port"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Behavior; comport.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Goodly <qex>comportance</qex> each to other bear.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comportation</ent><br/
<hw>Com`por*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comportatio</ets>.]</ety> <def>A bringing together.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Richardson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comportment</ent><br/
<hw>Com*port"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>comportement</ets>.]</ety> <def>Manner of acting; behavior; bearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A graceful <qex>comportment</qex> of their bodies.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Her serious and devout <qex>comportment</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compose</ent><br/
<hw>Com*pose"</hw> <pr>(k<ocr/m*p<omac/z")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Composed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Composing</conjf>.]</vmorph> <ety>[F. <ets>composer</ets>; <ets>com-</ets> + <ets>poser</ets> to place. The sense is that of L. <ets>componere</ets>, but the origin is different. See <er>Pose</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To form by putting together two or more things or parts; to put together; to make up; to fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Zeal ought to be <qex>composed</qex> of the highest degrees of all pious affection.</q> <rj><qau>Bp. Sprat.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form the substance of, or part of the substance of; to constitute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their borrowed gold <qex>composed</qex><br/
The calf in Oreb.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A few useful things . . . <qex>compose</qex> their intellectual possessions.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To construct by mental labor; to design and execute, or put together, in a manner involving the adaptation of forms of expression to ideas, or to the laws of harmony or proportion; <as>as, to <ex>compose</ex> a sentence, a sermon, a symphony, or a picture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me <qex>compose</qex><br/
Something in verse as well as prose.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The genius that <qex>composed</qex> such works as the <ldquo/Standard<rdquo/ and <ldquo/Last Supper<rdquo/.</q> <rj><qau>B. R. Haydon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To dispose in proper form; to reduce to order; to put in proper state or condition; to adjust; to regulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In a peaceful grave my corpse <qex>compose</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How in safety best we may<br/
<qex>Compose</qex> our present evils.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To free from agitation or disturbance; to tranquilize; to soothe; to calm; to quiet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compose</qex> thy mind;<br/
Nor frauds are here contrived, nor force designed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Print.)</fld> <def>To arrange (types) in a composing stick in order for printing; to set (type).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compose</ent><br/
<hw>Com*pose"</hw>, <pos>v. i.</pos> <def>To come to terms.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composed</ent><br/
<hw>Com*posed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Free from agitation; calm; sedate; quiet; tranquil; self-possessed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Mantuan there in sober triumph sate,<br/
<qex>Composed</qex> his posture, and his look sedate.</q> <rj><qau>Pope.</qau></rj></p>

<p>-- <wordforms><wf>Com*pos"ed*ly</wf> <pr>(<?/)</pr>, <pos>adv.</pos> -- <wf>Com*pos"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composer</ent><br/
<hw>Com*pos"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who composes; an author. Specifically, an author of a piece of music.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the thoughts of such authors have nothing in them, they at least . . . show an honest industry and a good intention in the <qex>composer</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His [Mozart's] most brilliant and solid glory is founded upon his talents as a <qex>composer</qex>.</q> <rj><qau>Moore (Encyc. of Mus.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, or that which, quiets or calms; one who adjusts a difference.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sweet <qex>composers</qex> of the pensive soul.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composing</ent><br/
<hw>Com*pos"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Tending to compose or soothe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to, or used in, composition.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Composing frame</b></col> <fld>(Print.)</fld>, <cd>a stand for holding cases of type when in use.</cd> -- <col><b>Composing rule</b></col> <fld>(Print.)</fld>, <cd>a thin slip of brass or steel, against which the type is arranged in a composing stick, or by the aid of which stickfuls or handfuls or type are lifted; -- called also <altname>setting rule</altname>.</cd> -- <col><b>Composing stick</b></col> <fld>(Print.)</fld>, <cd>an instrument usually of metal, which the compositor holds in his left hand, and in which he arranges the type in words and lines. It has one open side, and one adjustable end by means of which the length of the lines, and consequently the width of the page or column, may be determined.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compositae</ent><br/
||<hw>Com*pos"i*t<ae/</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>compositus</ets> made up of parts. See <er>Composite</er>.]</ety> <fld>(Bot.)</fld> <def>A large family of dicotyledonous plants, having their flowers arranged in dense heads of many small florets and their anthers united in a tube. The daisy, dandelion, and asters, are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composite</ent><br/
<hw>Com*pos"ite</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>compositus</ets> made up of parts, p. p. of <ets>componere</ets>. See <er>Compound</er>, <pos>v. t.</pos>, and cf. <er>Compost</er>.]</ety> <sn>1.</sn> <def>Made up of distinct parts or elements; compounded; <as>as, a <ex>composite</ex> language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Happiness, like air and water . . . is <qex>composite</qex>.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>Belonging to a certain order which is composed of the Ionic order grafted upon the Corinthian. It is called also the <altname>Roman</altname> or the <altname>Italic</altname> order, and is one of the five orders recognized by the Italian writers of the sixteenth century. See <er>Capital</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 292 --></p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>Belonging to the order <ord>Composit<ae/</ord>; bearing involucrate heads of many small florets, as the daisy, thistle, and dandelion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Composite carriage</b></col>, <cd>a railroad car having compartments of different classes.</cd> <mark>[Eng.]</mark> -- <col><b>Composite number</b></col> <fld>(Math.)</fld>, <cd>one which can be divided exactly by a number exceeding unity, as 6 by 2 or 3.</cd><-- the opposite of <col><b>prime number</b></col> -->. -- <mcol><col><b>Composite photograph</b></col> <it>or</it> <col><b>Composite portrait</b></col></mcol>, <cd>one made by a combination, or blending, of several distinct photographs.</cd> <au>F. Galton.</au> -- <col><b>Composite sailing</b></col> <fld>(Naut.)</fld>, <cd>a combination of parallel and great circle sailing.</cd> -- <col><b>Composite ship</b></col>, <cd>one with a wooden casing and iron frame.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composite</ent><br/
<hw>Com*pos"ite</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <def>That which is made up of parts or compounded of several elements; composition; combination; compound.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composition</ent><br/
<hw>Com`po*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>composition</ets>, fr. L. <ets>compositio</ets>. See <er>Composite</er>.]</ety> <sn>1.</sn> <def>The act or art of composing, or forming a whole or integral, by placing together and uniting different things, parts, or ingredients.</def> In specific uses: <sd>(a)</sd> <def>The invention or combination of the parts of any literary work or discourse, or of a work of art; <as>as, the <ex>composition</ex> of a poem or a piece of music</as>.</def> <ldquo/The constant habit of elaborate <xex>composition</xex>.<rdquo/ <au>Macaulay.</au> <sd>(b)</sd> <fld>(Fine Arts)</fld> <def>The art or practice of so combining the different parts of a work of art as to produce a harmonious whole; also, a work of art considered as such. See 4, below.</def> <sd>(c)</sd> <def>The act of writing for practice in a language, as English, Latin, German, etc.</def> <sd>(d)</sd> <fld>(Print.)</fld> <def>The setting up of type and arranging it for printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being put together or composed; conjunction; combination; adjustment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>View them in <qex>composition</qex> with other things.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The elementary <qex>composition</qex> of bodies.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mass or body formed by combining two or more substances; <as>as, a chemical <ex>composition</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>composition</qex> that looks . . . like marble.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A literary, musical, or artistic production, especially one showing study and care in arrangement; -- often used of an elementary essay or translation done as an educational exercise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Consistency; accord; congruity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no <qex>composition</qex> in these news<br/
That gives them credit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Mutual agreement to terms or conditions for the settlement of a difference or controversy; also, the terms or conditions of settlement; agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus we are agreed:<br/
I crave our <qex>composition</qex> may be written.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>The adjustment of a debt, or avoidance of an obligation, by some form of compensation agreed on between the parties; also, the sum or amount of compensation agreed upon in the adjustment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compositions</qex> for not taking the order of knighthood.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cleared by <qex>composition</qex> with their creditors.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Synthesis as opposed to analysis.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The investigation of difficult things by the method of analysis ought ever to precede the method of <qex>composition</qex>.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Composition cloth</b></col>, <cd>a kind of cloth covered with a preparation making it waterproof.</cd> -- <col><b>Composition deed</b></col>, <cd>an agreement for composition between a debtor and several creditors.</cd> -- <col><b>Composition plane</b></col> <fld>(Crystallog.)</fld>, <cd>the plane by which the two individuals of a twin crystal are united in their reserved positions.</cd> -- <col><b>Composition of forces</b></col> <fld>(Mech.)</fld>, <cd>the finding of a single force (called the <xex>resultant</xex>) which shall be equal in effect to two or more given forces (called the <xex>components</xex>) when acting in given directions.</cd> <au>Herbert.</au> -- <col><b>Composition metal</b></col>, <cd>an alloy resembling brass, which is sometimes used instead of copper for sheathing vessels; -- also called <altname>Muntz metal</altname> and <altname>yellow metal</altname>.</cd> -- <col><b>Composition of proportion</b></col> <fld>(Math.)</fld>, <cd>an arrangement of four proportionals so that the sum of the first and second is to the second as the sum of the third and fourth to the fourth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compositive</ent><br/
<hw>Com*pos"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>compositivus</ets>.]</ety> <def>Having the quality of entering into composition; compounded.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compositor</ent><br/
<hw>Com*pos"i*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., an arranger.]</ety> <sn>1.</sn> <def>One who composes or sets in order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>One who sets type and arranges it for use.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compositous</ent><br/
<hw>Com*pos"i*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Belonging to the <ord>Composit<ae/</ord>; composite.</def> <mark>[R.]</mark> <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compos mentis</ent><br/
||<hw>Com"pos men"tis</hw> <pr>(?)</pr>  <pos>a.</pos> <ety>[L.]</ety> <fld>(Law)</fld> <def>Sane in mind; being of sound mind, memory, and understanding.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Compos-mentis</ent><br/
<hw>Com"pos-men"tis</hw>, <pos>n.</pos> <def>One who is <er>compos mentis</er>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Compossible</ent><br/
<hw>Com*pos"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>com-</ets> + <ets>possible</ets>.]</ety> <def>Able to exist with another thing; consistent.</def> <mark>[R.]</mark> <rj><au>Chillingworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compost</ent><br/
<hw>Com"post</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>compost</ets>, fr. L. <ets>compositus</ets>, p. p. See <er>Composite</er>.]</ety> <sn>1.</sn> <def>A mixture; a compound.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A sad <qex>compost</qex> of more bitter than sweet.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Agric.)</fld> <def>A mixture for fertilizing land; esp., a composition of various substances (as muck, mold, lime, and stable manure) thoroughly mingled and decomposed, as in a compost heap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And do not spread the <qex>compost</qex> on the weeds<br/
To make them ranker.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compost</ent><br/
<hw>Com"post</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To manure with compost.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mingle, as different fertilizing substances, in a mass where they will decompose and form into a compost.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composture</ent><br/
<hw>Com*pos"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>compositura</ets>, <ets>-postura</ets>, a joining.]</ety> <def>Manure; compost.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Composure</ent><br/
<hw>Com*po"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Compose</er>.]</ety> <sn>1.</sn> <def>The act of composing, or that which is composed; a composition.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Signor Pietro, who had an admirable way both of <qex>composure</qex> [in music] and teaching.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Orderly adjustment; disposition.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Various <qex>composures</qex> and combinations of these corpuscles.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Frame; make; temperament.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>composure</qex> must be rare indeed<br/
Whom these things can not blemish.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A settled state; calmness; sedateness; tranquillity; repose.</def> <ldquo/We seek peace and <xex>composure</xex>.<rdquo/ <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When the passions . . . are all silent, the mind enjoys its most perfect <qex>composure</qex>.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A combination; a union; a bond.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compotation</ent><br/
<hw>Com`po*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compotatio</ets>; <ets>com-</ets> + <ets>potare</ets> to drink.]</ety> <def>The act of drinking or tippling together.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The fashion of <qex>compotation</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compotator</ent><br/
<hw>Com"po*ta`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who drinks with another.</def> <mark>[R.]</mark> <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compote</ent><br/
||<hw>Com"pote</hw> <pr>(k<ocr/m"p<omac/t)</pr>, <pos>n.</pos> <ety>[F. See <er>Compost</er>.]</ety> <sn>1.</sn> <def>A preparation of fruit in sirup in such a manner as to preserve its form, either whole, halved, or quartered; <as>as, a <ex>compote</ex> of pears</as>.</def> <rj><au>Littr<eacute/.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a bowl-shaped dish having a stem, and a base smaller than the bowl, used to serve compotes or other items, such as candies, nuts, or fruit; a compote dish.  Compotes may be made of different materials, such as glass, porcelain, or silver.</def><br/
<syn><b>Syn. --</b> compote dish, compotier.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Compote dish</ent><br/
<hw>Com"pote dish`</hw> <def>a compote{2}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Compotier</ent><br/
||<hw>Com`po`tier"</hw> <pr>(k<ocir/N`p<osl/`ty<amac/")</pr>, <pos>n.</pos>; <plu>pl. <plw>Compotiers</plw> <pr>(<it>F.</it> ty<amac/")</pr></plu>. <ety>[F.]</ety> <def>A dish for holding compotes, fruit, etc.; a compote dish, or compote{2}.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Compound</ent><br/
||<hw>Com"pound</hw> <pr>(k<ocr/m"pound)</pr>, <pos>n.</pos> <ety>[Malay <ets>kompung</ets> a village.]</ety> <def>In the East Indies, an inclosure containing a house, outbuildings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compound</ent><br/
<hw>Com*pound"</hw> <pr>(k<ocr/m*pound")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compounded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Compounding</conjf>.]</vmorph> <ety>[OE. <ets>componen</ets>, <ets>compounen</ets>, L. <ets>componere</ets>, <ets>compositum</ets>; <ets>com-</ets>+ <ets>ponere</ets> to put set. The <ets>d</ets> is excrescent. See <er>Position</er>, and cf. <er>Compon<eacute/</er>.]</ety> <sn>1.</sn> <def>To form or make by combining different elements, ingredients, or parts; <as>as, to <ex>compound</ex> a medicine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Incapacitating him from successfully <qex>compounding</qex> a tale of this sort.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put together, as elements, ingredients, or parts, in order to form a whole; to combine, mix, or unite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have the power of altering and <qex>compounding</qex> those images into all the varieties of picture.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To modify or change by combination with some other thing or part; to mingle with something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Only <qex>compound</qex> me with forgotten dust.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To compose; to constitute.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His pomp and all what state <qex>compounds</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To settle amicably; to adjust by agreement; to compromise; to discharge from obligation upon terms different from those which were stipulated; <as>as, to <ex>compound</ex> a debt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I pray, my lords, let me <qex>compound</qex> this strife.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To compound a felony</b></col>, <cd>to accept of a consideration for forbearing to prosecute, such compounding being an indictable offense. See <er>Theftbote</er>.</cd></cs><-- sic: "to accept of" (?) --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compound</ent><br/
<hw>Com*pound"</hw>, <pos>v. i.</pos> <def>To effect a composition; to come to terms of agreement; to agree; to settle by a compromise; -- usually followed by <xex>with</xex> before the person participating, and <xex>for</xex> before the thing compounded or the <xex>consideration</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here's a fellow will help you to-morrow; . . . <qex>compound</qex> with him by the year.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They were at last glad to <qex>compound</qex> for his bare commitment to the Tower.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cornwall <qex>compounded</qex> to furnish ten oxen after Michaelmas for thirty pounds.</q> <rj><qau>R. Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compound</qex> for sins they are inclined to<br/
By damning those they have no mind to.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compound</ent><br/
<hw>Com"pound</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>compouned</ets>, p. p. of <ets>compounen</ets>. See <er>Compound</er>, <pos>v. t.</pos>]</ety> <def>Composed of two or more elements, ingredients, parts; produced by the union of several ingredients, parts, or things; composite; <as>as, a <ex>compound</ex> word</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Compound</qex> substances are made up of two or more simple substances.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Compound addition</b></col>, <col><b>subtraction</b></col>, <col><b>multiplication</b></col>, <col><b>division</b></col></mcol> <fld>(Arith.)</fld>, <cd>the addition, subtraction, etc., of compound numbers.</cd> -- <col><b>Compound crystal</b></col> <fld>(Crystallog.)</fld>, <cd>a twin crystal, or one seeming to be made up of two or more crystals combined according to regular laws of composition.</cd> -- <col><b>Compound engine</b></col> <fld>(Mech.)</fld>, <cd>a form of steam engine in which the steam that has been used in a high-pressure cylinder is made to do further service in a larger low-pressure cylinder, sometimes in several larger cylinders, successively.</cd> -- <col><b>Compound ether</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Ether</er>.</cd> -- <col><b>Compound flower</b></col> <fld>(Bot.)</fld>, <cd>a flower head resembling a single flower, but really composed of several florets inclosed in a common calyxlike involucre, as the sunflower or dandelion.</cd> -- <col><b>Compound fraction</b></col>. <fld>(Math.)</fld> <cd>See <er>Fraction</er>.</cd> -- <col><b>Compound fracture</b></col>. <cd>See <er>Fracture</er>.</cd> -- <col><b>Compound householder</b></col>, <cd>a householder who compounds or arranges with his landlord that his rates shall be included in his rents.</cd> <mark>[Eng.]</mark> -- <col><b>Compound interest</b></col>. <cd>See <er>Interest</er>.</cd> -- <col><b>Compound larceny</b></col>. <fld>(Law)</fld> <cd>See <er>Larceny</er>.</cd> -- <col><b>Compound leaf</b></col> <fld>(Bot.)</fld>, <cd>a leaf having two or more separate blades or leaflets on a common leafstalk.</cd> -- <col><b>Compound microscope</b></col>. <cd>See <er>Microscope</er>.</cd> -- <col><b>Compound motion</b></col>. <cd>See <er>Motion</er>.</cd> -- <col><b>Compound number</b></col> <fld>(Math.)</fld>, <cd>one constructed according to a varying scale of denomination; as, 3 <xex>cwt.</xex>, 1 <xex>qr.</xex>, 5 <xex>lb.</xex>; -- called also <altname>denominate number</altname>.</cd> -- <col><b>Compound pier</b></col> <fld>(Arch.)</fld>, <cd>a clustered column.</cd> -- <col><b>Compound quantity</b></col> <fld>(Alg.)</fld>, <cd>a quantity composed of two or more simple quantities or terms, connected by the sign + (plus) or - (minus). Thus, <mathex>a + b - c</mathex>, and <mathex>bb - b</mathex>, are compound quantities.</cd> -- <col><b>Compound radical</b></col>. <fld>(Chem.)</fld> <cd>See <er>Radical</er>.</cd> -- <col><b>Compound ratio</b></col> <fld>(Math.)</fld>, <cd>the product of two or more ratios; thus <mathex>ab:cd</mathex> is a ratio compounded of the simple ratios <mathex>a:c</mathex> and <mathex>b:d</mathex>.</cd> -- <col><b>Compound rest</b></col> <fld>(Mech.)</fld>, <cd>the tool carriage of an engine lathe.</cd> -- <col><b>Compound screw</b></col> <fld>(Mech.)</fld>, <cd>a screw having on the same axis two or more screws with different pitch (a differential screw), or running in different directions (a right and left screw).</cd> -- <col><b>Compound time</b></col> <fld>(Mus.)</fld>, <cd>that in which two or more simple measures are combined in one; as, 6-8 time is the joining of two measures of 3-8 time.</cd> -- <col><b>Compound word</b></col>, <cd>a word composed of two or more words; specifically, two or more words joined together by a hyphen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compound</ent><br/
<hw>Com"pound</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which is compounded or formed by the union or mixture of elements ingredients, or parts; a combination of simples; a compound word; the result of composition.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rare <qex>compound</qex> of oddity, frolic, and fun.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When the word <ldquo/bishopric<rdquo/ was first made, it was made as a <qex>compound</qex>.</q> <rj><qau>Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A union of two or more ingredients in definite proportions by weight, so combined as to form a distinct substance; <as>as, water is a <ex>compound</ex> of oxygen and hydrogen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Every definite chemical <xex>compound</xex> always contains the same elements, united in the same proportions by weight, and with the same internal arrangement.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Binary compound</b></col> <fld>(Chem.)</fld>. <cd>See under <er>Binary</er>.</cd> -- <col><b>Carbon compounds</b></col> <fld>(Chem.)</fld>. <cd>See under <er>Carbon</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compoundable</ent><br/
<hw>Com*pound"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be compounded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compound control</ent><br/
<hw>Com"pound con*trol"</hw>. <fld>(A<eum/ronautics)</fld> <def>A system of control in which a separate manipulation, as of a rudder, may be effected by either of two movements, in different directions, of a single lever, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>compounded</ent><br/
<hw>compounded</hw> <pos>adj.</pos> <def>combined into or constituting a chemical compound.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Compounder</ent><br/
<hw>Com*pound"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, compounds or mixes; <as>as, a <ex>compounder</ex> of medicines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who attempts to bring persons or parties to terms of agreement, or to accomplish ends by compromises.</def> <ldquo/<xex>Compounders</xex> in politics.<rdquo/ <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who compounds a debt, obligation, or crime.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Religious houses made <qex>compounders</qex><br/
For the horrid actions of their founders.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One at a university who pays extraordinary fees for the degree he is to take.</def> <mark>[Eng.]</mark> <rj><au>A. Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Eng. Hist.)</fld> <def>A Jacobite who favored the restoration of James II, on condition of a general amnesty and of guarantees for the security of the civil and ecclesiastical constitution of the realm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>compounding</ent><br/
<hw>compounding</hw> <pos>n.</pos> <def>the act of combining things.</def><br/
<syn><b>Syn. --</b> combination, combining.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comprador</ent><br/
||<hw>Com`pra*dor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg., a buyer.]</ety> <def>A kind of steward or agent.</def> <mark>[China]</mark> <rj><au>S. W. Williams</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprecation</ent><br/
<hw>Com`pre*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comprecatio</ets>, fr. <ets>comprecari</ets> to pray to. See <er>Precarious</er>.]</ety> <def>A praying together.</def> <mark>[Obs.]</mark> <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehend</ent><br/
<hw>Com`pre*hend"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Comprehended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Comprehending</conjf>.]</vmorph> <ety>[L. <ets>comprehendere</ets>, <ets>comprehensum</ets>; <ets>com-</ets> + <ets>prehendere</ets> to grasp, seize; <ets>prae</ets> before + <ets>hendere</ets> (used only in comp.). See <er>Get</er>, and cf. <er>Comprise</er>.]</ety> <sn>1.</sn> <def>To contain; to embrace; to include; <as>as, the states <ex>comprehended</ex> in the Austrian Empire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who hath . . . <qex>comprehended</qex> the dust of the earth in a measure.</q> <rj><qau>Is. xl. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take in or include by construction or implication; to comprise; to imply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Comprehended</qex> all in this one word, Discretion.</q> <rj><qau>Hobbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And if there be any other commandment, it is briefly <qex>comprehended</qex> in this saying.</q> <rj><qau>Rom. xiii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take into the mind; to grasp with the understanding; to apprehend the meaning of; to understand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At a loss to <qex>comprehend</qex> the question.</q> <rj><qau>W. Irwing.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Great things doeth he, which we can not <qex>comprehend</qex>.</q> <rj><qau>Job. xxxvii. 5.</qau></rj></p>

<p><syn><b>Syn.</b> -- To contain; include; embrace; comprise; inclose; grasp; embody; involve; imply; apprehend; imagine; conceive; understand. See <er>Apprehend</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>comprehendible</ent><br/
<hw>comprehendible</hw> <pos>adj.</pos>  <def>able to be understood.  Opposite of <ant>incomprehensible</ant>.</def> <note>[Narrower terms: <stype>clear, distinct</stype>]</note> <see>Also See: <er>clear</er>, <er>explicable</er>, <er>fathomable</er>, <er>intelligible</er>.</see><br/
<syn><b>Syn. --</b> comprehensible.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comprehensibility</ent><br/
<hw>Com`pre*hen`si*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being comprehensible; capability of being understood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensible</ent><br/
<hw>Com"pre*hen"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>comprehensibilis</ets>: cf. F. <ets>compre<eacute/hensible</ets>.]</ety> <sn>1.</sn> <def>Capable of being comprehended, included, or comprised.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest this part of knowledge should seem to any not <qex>comprehensible</qex> by axiom, we will set down some heads of it.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being understood; intelligible; conceivable by the mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The horizon sets the bounds . . . between what is and what is not <qex>comprehensible</qex> by us.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensibleness</ent><br/
<hw>Com`pre*hen"si*ble*ness</hw>, <pos>n.</pos> <def>The quality of being comprehensible; comprehensibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensibly</ent><br/
<hw>Com`pre*hen"si*bly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>With great extent of signification; comprehensively.</def> <rj><au>Tillotson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Intelligibly; in a manner to be comprehended or understood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehension</ent><br/
<hw>Com`pre*hen"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comprehensio</ets>: cf. F. <ets>compr<eacute/hension</ets>.]</ety> <sn>1.</sn> <def>The act of comprehending, containing, or comprising; inclusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the Old Testament there is a close <qex>comprehension</qex> of the New; in the New, an open discovery of the Old.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is comprehended or inclosed within narrow limits; a summary; an epitome.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though not a catalogue of fundamentals, yet . . . a <qex>comprehension</qex> of them.</q> <rj><qau>Chillingworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The capacity of the mind to perceive and understand; the power, act, or process of grasping with the intellect; perception; understanding; <as>as, a <ex>comprehension</ex> of abstract principles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Logic)</fld> <def>The complement of attributes which make up the notion signified by a general term.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Rhet.)</fld> <def>A figure by which the name of a whole is put for a part, or that of a part for a whole, or a definite number for an indefinite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensive</ent><br/
<hw>Com`pre*hen"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>compr<eacute/hensif</ets>.]</ety> <sn>1.</sn> <def>Including much; comprising many things; having a wide scope or a full view.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A very <qex>comprehensive</qex> definition.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Large and <qex>comprehensive</qex> idea.</q> <rj><qau>Channing.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the power to comprehend or understand many things.</def> <ldquo/His <xex>comprehensive</xex> head.<rdquo/ <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Possessing peculiarities that are characteristic of several diverse groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term is applied chiefly to early fossil groups which have a combination of structures that appear in more fully developed or specialized forms in later groups. <xex>Synthetic</xex>, as used by Agassiz, is nearly synonymous.</note></p>

<p><syn><b>Syn.</b> -- Extensive; wide; large; full; compendious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensively</ent><br/
<hw>Com`pre*hen"sive*ly</hw>, <pos>adv.</pos> <def>In a comprehensive manner; with great extent of scope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensiveness</ent><br/
<hw>Com`pre*hen"sive*ness</hw>, <pos>n.</pos> <def>The quality of being comprehensive; extensiveness of scope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Compare the beauty and <qex>comprehensiveness</qex> of legends on ancient coins.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprehensor</ent><br/
<hw>Com`pre*hen"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who comprehends; one who has attained to a full knowledge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When I shall have dispatched this weary pilgrimage, and from a traveler shall come to be a <qex>comprehensor</qex>, farewell faith and welcome vision.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compress</ent><br/
<hw>Com*press"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compressed</conjf> <pr>(?)</pr>; <pos>p. pr & vb. n.</pos> <conjf>Compressing</conjf>.]</vmorph> <ety>[L. <ets>compressus</ets>, p. p. of <ets>comprimere</ets> to compress: <ets>com-</ets> + <ets>premere</ets> to press. See Press.]</ety> <sn>1.</sn> <def>To press or squeeze together; to force into a narrower compass; to reduce the volume of by pressure; to compact; to condense; <as>as, to <ex>compress</ex> air or water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Events of centuries . . . <qex>compressed</qex> within the compass of a single life.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The same strength of expression, though more <qex>compressed</qex>, runs through his historical harangues.</q> <rj><qau>Melmoth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To embrace sexually.</def> <mark>[Obs.]</mark> <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Computers)</fld> <def>to reduce the space required for storage (of binary data) by an algorithm which converts the data to a smaller number of bits while preserving the information content.  The compressed data is usually decompressed to recover the initial data format before subsequent use.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- To crowd; squeeze; condense; reduce; abridge.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 293 --></p>

<p><ent>Compress</ent><br/
<hw>Com"press</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>compresse</ets>.]</ety> <fld>(Surg.)</fld> <def>A folded piece of cloth, pledget of lint, etc., used to cover the dressing of wounds, and so placed as, by the aid of a bandage, to make due pressure on any part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compressed</ent><br/
<hw>Com*pressed"</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pressed together; compacted; reduced in volume by pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Flattened lengthwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Compressed-air engine</b></col>, <cd>an engine operated by the elastic force of compressed air.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compressed yeast</ent><br/
<hw>Com*pressed" yeast</hw>. <def>A cake yeast made by filtering the cells from the liquid in which they are grown, subjecting to heavy pressure, and mixing with starch or flour.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Compressibility</ent><br/
<hw>Com*press`i*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>compressibilit<eacute/</ets>.]</ety> <def>The quality of being compressible of being compressible; <as>as, the <ex>compressibility</ex> of elastic fluids</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compressible</ent><br/
<hw>Com*press"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>compressible</ets>.]</ety> <def>Capable of being pressed together or forced into a narrower compass, as an elastic or spongy substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compressibleness</ent><br/
<hw>Com*press"ible*ness</hw>, <pos>n.</pos> <def>The quality of being compressible; compressibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compression</ent><br/
<hw>Com*pres"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compressio</ets>: cf. F. <ets>compression</ets>.]</ety> <sn>1.</sn> <def>The act of compressing, or state of being compressed.</def> <ldquo/<xex>Compression</xex> of thought.<rdquo/ <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Computers)</fld> <def>reduction of the space required for storage (of binary data) by an algorithm which converts the data to a smaller number of bits while preserving the information content.  The act of compressing {3}.</def> <note>Compression may be <stype>lossless compression</stype>, in which all of the information in the original data is preserved, and the original data may be recovered in form identical to its original form; or <stype>lossy compression</stype>, in which some of the information in the original data is lost, and decompression results in a data form slightly different from the original.  <stype>Lossy compression</stype> is used, for example, to compress audio or video recordings, and sometimes images, where the slight differences in the original data and the data recovered after <stype>lossy compression</stype> may be imperceptable to the human eye or ear.  The <stype>JPEG</stype> format is produced by a <stype>lossy compression</stype> algorithm.</note><br/
[<source>PJC</source>]</p>

<p><ent>Compression projectile</ent><br/
<hw>Com*pres"sion pro*jec"tile</hw>. <def>A projectile constructed so as to take the grooves of a rifle by means of a soft copper band firmly attached near its base or, formerly, by means of an envelope of soft metal. In small arms the modern projectile, having a soft core and harder jacket, is subjected to compression throughout the entire cylindrical part.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Compressive</ent><br/
<hw>Com*press"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>compressif</ets>.]</ety> <def>Compressing, or having power or tendency to compress; <as>as, a <ex>compressive</ex> force</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compressor</ent><br/
<hw>Com*press"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Anything which serves to compress</def>; as: <sd>(a)</sd> <fld>(Anat.)</fld> <def>A muscle that compresses certain parts.</def> <sd>(b)</sd> <fld>(Surg.)</fld> <def>An instrument for compressing an artery (esp., the femoral artery) or other part.</def> <sd>(c)</sd> <def>An apparatus for confining or flattening between glass plates an object to be examined with the microscope; -- called also <altname>compressorium</altname>.</def> <sd>(d)</sd> <fld>(Mach.)</fld> <def>A machine for compressing gases; especially, an air compressor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compressure</ent><br/
<hw>Com*pres"sure</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>Compression.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprint</ent><br/
<hw>Com*print"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <sn>1.</sn> <def>To print together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O. Eng. Law)</fld> <def>To print surreptitiously a work belonging to another.</def> <rj><au>E. Phillips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprint</ent><br/
<hw>Com"print</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>The surreptitious printing of another's copy or book; a work thus printed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprisal</ent><br/
<hw>Com*pris"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of comprising or comprehending; a compendium or epitome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>comprisal</qex> . . . and sum of all wickedness.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprise</ent><br/
<hw>Com*prise"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Comprised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Comprising</conjf>.]</vmorph> <ety>[From F. <ets>compris</ets>, <ets>comprise</ets>, p. p. of <ets>comprendre</ets>, L. <ets>comprehendere</ets>. See <er>Comprehend</er>.]</ety> <def>To comprehend; to include.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Comprise</qex> much matter in few words.</q> <rj><qau>Hocker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Friendship does two souls in one <qex>comprise</qex>.</q> <rj><qau>Roscommon.</qau></rj></p>

<p><syn>Syn. -- To embrace; include; comprehend; contain; encircle; inclose; involve; imply.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprobate</ent><br/
<hw>Com"pro*bate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>comprobatus</ets>, p. p. of <ets>comprobare</ets>, to approve wholly.]</ety> <def>To agree; to concur.</def> <mark>[Obs.]</mark> <rj><au>Sir T. Elyot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprobation</ent><br/
<hw>Com`pro*ba"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>comprobatio</ets>.]</ety> <sn>1.</sn> <def>Joint attestation; proof.</def> <mark>[Obs.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Approbation.</def> <mark>[Obs.]</mark> <rj><au>Foxe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compromise</ent><br/
<hw>Com"pro*mise</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>compromis</ets>, fr. L. <ets>compromissum</ets> a mutual promise to abide by the decision of an arbiter, fr. <ets>compromittere</ets> to make such a promise; <ets>com-</ets> + <ets>promittere</ets> to promise. See <er>Promise</er>.]</ety> <sn>1.</sn> <def>A mutual agreement to refer matters in dispute to the decision of arbitrators.</def> <mark>[Obs.]</mark> <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A settlement by arbitration or by mutual consent reached by concession on both sides; a reciprocal abatement of extreme demands or rights, resulting in an agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But basely yielded upon <qex>compromise</qex><br/
That which his noble ancestors achieved with blows.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All government, indeed every human benefit and enjoyment, every virtue and every prudent act, is founded on <qex>compromise</qex> and barter.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An abhorrence of concession and <qex>compromise</qex> is a never failing characteristic of religious factions.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A committal to something derogatory or objectionable; a prejudicial concession; a surrender; <as>as, a <ex>compromise</ex> of character or right</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I was determined not to accept any fine speeches, to the <qex>compromise</qex> of that sex the belonging to which was, after all, my strongest claim and title to them.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compromise</ent><br/
<hw>Com"pro*mise</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compromised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Compromising</conjf>.]</vmorph> <ety>[From <er>Compromise</er>, <pos>n.</pos>; cf. <er>Compromit</er>.]</ety> <sn>1.</sn> <def>To bind by mutual agreement; to agree.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Laban and himself were <qex>compromised</qex><br/
That all the eanlings which were streaked and pied<br/
Should fall as Jacob's hire.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To adjust and settle by mutual concessions; to compound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The controversy may easily be <qex>compromised</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pledge by some act or declaration; to endanger the life, reputation, etc., of, by some act which can not be recalled; to expose to suspicion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To pardon all who had been <qex>compromised</qex> in the late disturbances.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compromise</ent><br/
<hw>Com"pro*mise</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To agree; to accord.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make concession for conciliation and peace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compromiser</ent><br/
<hw>Com"pro*mi`ser</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who compromises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compromissorial</ent><br/
<hw>Com`pro*mis*so"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to compromise.</def> <mark>[R.]</mark> <rj><au>Chalmers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compromit</ent><br/
<hw>Com"pro*mit`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Compromitted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Compromitting</conjf>.]</vmorph> <ety>[L. <ets>compromittere</ets>. See <er>Compromise</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To pledge by some act or declaration; to promise.</def> <rj><au>State Trials (1529).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put to hazard, by some indiscretion; to endanger; to compromise; <as>as, to <ex>compromit</ex> the honor or the safety of a nation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comprovincial</ent><br/
<hw>Com`pro*vin"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to, or associated in, the same province.</def> <mark>[Obs.]</mark> -- <def2><pos>n.</pos> <def>One who belongs to the same province.</def></def2> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The six islands, <qex>comprovincial</qex><br/
In ancient times unto Great Britain.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compsognathus</ent><br/
||<hw>Comp*sog"na*thus</hw> <pr>(k<ocr/mp*s<ocr/g"n<adot/*th<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kompo`s</grk> elegant, pretty + <grk>gna`qos</grk> jaw.]</ety> <fld>(Zool.)</fld> <def>A genus of carnivorous Dinosauria found in the Jurassic formation with a slender body, and remarkable for having several birdlike features.  It reached a length of 30 inches when adult.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Compt</ent><br/
<hw>Compt</hw> <pr>(kount, <it>formerly</it> k<ocr/mt; 215)</pr>, <pos>n.</pos> <ety>[F. <ets>compte</ets>. See <er>Count</er> an account.]</ety> <def>Account; reckoning; computation.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compt</ent><br/
<hw>Compt</hw>, <pos>v. t.</pos> <ety>[F. <ets>compter</ets>. See <er>Count</er>, <pos>v. t.</pos>]</ety> <def>To compute; to count.</def> <mark>[Obs.]</mark> <see>See <er>Count</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compt</ent><br/
<hw>Compt</hw>, <pos>a.</pos> <ety>[L. <ets>comptus</ets>, p. p. of <ets>comere</ets> to care for, comb, arrange, adorn.]</ety> <def>Neat; spruce.</def> <mark>[Obs.]</mark> <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compter</ent><br/
<hw>Compt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A counter.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compte rendu</ent><br/
||<hw>Compte" ren`du</hw> <pr>(?)</pr>. <ety>[F.]</ety> <def>A report of an officer or agent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comptible</ent><br/
<hw>Compt"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Compt</er>, <pos>v. t.</pos>]</ety> <def>Accountable; responsible; sensitive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I am very <qex>comptible</qex> even to the least sinister usage.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comptly</ent><br/
<hw>Compt"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Neatly.</def> <mark>[Obs.]</mark> <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comptograph</ent><br/
<hw>Comp"to*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>compter</ets> to count + <ets>-graph</ets>.]</ety> <def>A machine for adding numbers and making a printed record of the sum.  An older term for an <er>adding machine</er>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Comptometer</ent><br/
<hw>Comp*tom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Count</er>; <er>-meter</er>.]</ety> <def>A calculating machine; an arithmometer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Comptrol</ent><br/
<hw>Comp*trol"</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Control</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comptroler</ent><br/
<hw>Comp*trol"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A controller; a public officer whose duty it is to examine certify accounts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsative</ent><br/
<hw>Com*pul"sa*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From L. <ets>compulsare</ets>, v. intens. of <ets>compellere</ets>. See <er>Compel</er>.]</ety> <def>Compulsatory.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsatively</ent><br/
<hw>Com*pul"sa*tive*ly</hw>, <pos>adv.</pos> <def>By compulsion.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsatory</ent><br/
<hw>Com*pul"sa*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Operating with force; compelling; forcing; constraining; resulting from, or enforced by, compulsion.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To recover of us, by strong hand<br/
And terms <qex>compulsatory</qex>, those foresaid lands.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsion</ent><br/
<hw>Com*pul"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compulsio</ets>. See <er>Compel</er>.]</ety> <def>The act of compelling, or the state of being compelled; the act of driving or urging by force or by physical or moral constraint; subjection to force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If reasons were as plentiful as blackberries, I would give no man a reason upon <qex>compulsion</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With what <qex>compulsion</qex> and laborious flight<br/
We sunk thus low.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> -- See <er>Constraint</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsive</ent><br/
<hw>Com*pul"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having power to compel; exercising or applying compulsion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Religion is . . . inconsistent with all <qex>compulsive</qex> motives.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsively</ent><br/
<hw>Com*pul"sive*ly</hw>, <pos>adv.</pos> <def>By compulsion; by force.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsorily</ent><br/
<hw>Com*pul"so*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a compulsory manner; by force or constraint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compulsory</ent><br/
<hw>Com*pul"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>compulsorius</ets>.]</ety> <sn>1.</sn> <def>Having the power of compulsion; constraining.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Obligatory; enjoined by authority; necessary; due to compulsion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This contribution threatening to fall infinitely short of their hopes, they soon made it <qex>compulsory</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compunct</ent><br/
<hw>Com*punct"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>compunctus</ets>, p. p.]</ety> <def>Affected with compunction; conscience-stricken.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compunction</ent><br/
<hw>Com*punc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>compunction</ets>, F. <ets>componction</ets>, L. <ets>compunctio</ets>, fr. <ets>compungere</ets>, <ets>compunctum</ets>, to prick; <ets>com-</ets> + <ets>pungere</ets> to prick, sting. See <er>Pungent</er>.]</ety> <sn>1.</sn> <def>A pricking; stimulation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That acid and piercing spirit which, with such activity and <qex>compunction</qex>, invadeth the brains and nostrils.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A picking of heart; poignant grief proceeding from a sense of guilt or consciousness of causing pain; the sting of conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He acknowledged his disloyalty to the king, with expressions of great <qex>compunction</qex>.</q> <rj><qau>Clarendon.</qau></rj></p>

<p><syn><b>Syn.</b> -- <er>Compunction</er>, <er>Remorse</er>, <er>Contrition</er>.</syn> <usage> <xex>Remorse</xex> is anguish of soul under a sense of guilt or consciousness of having offended God or brought evil upon one's self or others. <xex>Compunction</xex> is the pain occasioned by a wounded and awakened conscience. Neither of them implies true <xex>contrition</xex>, which denotes self-condemnation, humiliation, and repentance. We speak of the gnawings of <xex>remorse</xex>; of <xex>compunction</xex> for a specific act of transgression; of deep <xex>contrition</xex> in view of our past lives. See <er>Regret</er>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compunctionless</ent><br/
<hw>Com*punc"tion*less</hw>, <pos>a.</pos> <def>Without compunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compunctious</ent><br/
<hw>Com*punc"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of compunction; caused by conscience; attended with, or causing, compunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That no <qex>compunctious</qex> visitings of nature<br/
Shake my fell purpose.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compunctiously</ent><br/
<hw>Com*punc"tious*ly</hw>, <pos>adv.</pos> <def>With compunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compunctive</ent><br/
<hw>Com*punc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sensitive in respect of wrongdoing; conscientious.</def> <mark>[Obs.]</mark> <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compurgation</ent><br/
<hw>Com`pur*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>compurgatio</ets>, fr. <ets>compurgare</ets> to purify wholly; <ets>com-</ets> + <ets>purgare</ets> to make pure. See Purge, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>The act or practice of justifying or confirming a man's veracity by the oath of others; -- called also <altname>wager of law</altname>. See <er>Purgation</er>; also <cref>Wager of law</cref>, under <er>Wager</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exculpation by testimony to one's veracity or innocence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was privileged from his childhood from suspicion of incontinency and needed no <qex>compurgation</qex>.</q> <rj><qau>Bp. Hacket.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compurgator</ent><br/
<hw>Com"pur*ga`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL.]</ety> <def>One who bears testimony or swears to the veracity or innocence of another. See <er>Purgation</er>; also <cref>Wager of law</cref>, under <er>Wager</er>.</def> <-- = character witness. --><br/
[<source>1913 Webster</source>]</p>

<p><q>All they who know me . . . will say they have reason in this matter to be my <qex>compurgators</qex>.</q> <rj><qau>Chillingworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compurgatorial</ent><br/
<hw>Com*pur`ga*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to a compurgator or to compurgation.</def> <ldquo/Their <xex>compurgatorial</xex> oath.<rdquo/ <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Computable</ent><br/
<hw>Com*put"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>computabilis</ets>.]</ety> <def>Capable of being computed, numbered, or reckoned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not easily <qex>computable</qex> by arithmetic.</q> <rj><qau>Sir M. Hale.</qau></rj></p>

<p><-- computable number. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Computation</ent><br/
<hw>Com`pu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>computatio</ets>: cf. F. <ets>computation</ets>.]</ety> <sn>1.</sn> <def>The act or process of computing; calculation; reckoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By just <qex>computation</qex> of the time.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By a <qex>computation</qex> backward from ourselves.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The result of computation; the amount computed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Computers)</fld> <def>The process of performing arithmetical or logical operations, on data by an electronic device.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- Reckoning; calculation; estimate; account.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>computational</ent><br/
<hw>computational</hw> <pos>adj.</pos> <sn>1.</sn> <def>of or pertaining to computation (definition 3).</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>accomplished by a computer{2} or computation{3}; <as>as, <ex>computational</ex> linguistics; <ex>computational</ex> morphology</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Compute</ent><br/
<hw>Com*pute"</hw> <pr>(k<ocr/m*p<umac/t")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Computed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Computing</conjf>.]</vmorph> <ety>[L. <ets>computare</ets>. See <er>Count</er>, <pos>v. t.</pos>]</ety> <def>To determine by calculation; to reckon; to count.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two days, as we <qex>compute</qex> the days of heaven.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What's done we partly may <qex>compute</qex>,<br/
But know not what's resisted.</q> <rj><qau>Burns.</qau></rj></p>

<p><syn><b>Syn.</b> -- To calculate; number; count; reckon; estimate; enumerate; rate. See <er>Calculate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Compute</ent><br/
<hw>Com*pute"</hw>, <pos>n.</pos> <ety>[L. <ets>computus</ets>: cf. F. <ets>comput</ets>.]</ety> <def>Computation.</def> <mark>[R.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Computer</ent><br/
<hw>Com*put"er</hw> <pr>(k<ocr/m*p<umac/t"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who computes.</def></p>

<p><sn>2.</sn> <fld>(Computers)</fld> <def>an electronic device for performing calculations automatically.  It consists of a clock to provide voltage pulses to synchronize the operations of the devices within the computer, a <part>central processing unit</part>, where the arithmetical and logical operations are performed on data, a <part>random-access memory</part>, where the programs and data are stored for rapid access, devices to input data and output results, and various other <part>peripheral devices</part> of widely varied function, as well as circuitry to support the main operations.</def> <note>This modern sense of computer comprises the stored-program computers, in which multiple steps in a calculation may be stored within the computer itself as <er>instructions</er> in a <er>program</er>, and are then executed by the computer without further intervention of the operator.  Different types of computer are variously called <stype>analog computer</stype>, <stype>number cruncher, number-cruncher</stype>, <stype>digital computer</stype>, and <stype>pari-mutuel machine, totalizer, totaliser, totalizator, totalisator</stype>.</note><br/
<syn><b>Syn. --</b> data processor, electronic computer, information processing system.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <fld>(Computers)</fld> <def>same as <er>digital computer</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>computerize</ent><br/
<hw>computerize</hw> <pos>v. t.</pos> <sn>1.</sn> <def>to provide with computers; <as>as, Our office is fully <ex>computerized</ex> now</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to to store in a computer; <as>as, a <ex>computerized</ex> dictionary</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>to cause (a function) to be carried out by a computer or computers; <as>as, They <ex>computerized</ex> their order processing</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>computerized</ent><br/
<hw>computerized</hw> <pos>adj.</pos> <def>stored, processed, or analyzed by computer.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>computer program</ent><br/
<hw>computer program</hw> <pos>n.</pos> <def>a sequence of instructions, stored in any medium, that can be interpreted and executed by a computer; -- called most frequently a <altname>program</altname>.  This term is used both for the written program (a document) and for its corresponding electronic version stored or executed on the computer.  See <er>instruction</er>; <as>as, Version 1.0 of the <ex>program</ex> had a serious bug that caused the computer to crash frequently.</as>.</def><br/
<syn><b>Syn. --</b> program, program, computer programme.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Computist</ent><br/
<hw>Com"pu*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A computer{1}.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comrade</ent><br/
<hw>Com"rade</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[Sp. <ets>camarada</ets>, fr. L. <ets>camara</ets>, a chamber; hence, a chamber-fellowship, and then a chamber-fellow: cf. F. <ets>camarade</ets>. Cf. <er>Chamber</er>.]</ety> <def>A mate, companion, or associate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And turned my flying <qex>comrades</qex> to the charge.</q> <rj><qau>J. Baillie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I abjure all roofs, and choose . . . <br/
To be a <qex>comrade</qex> with the wolf and owl.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>comradely</ent><br/
<hw>comradely</hw> <pos>adj.</pos> <def>like a comrade; heartily friendly and congenial.</def><br/
<syn><b>Syn. --</b> hail-fellow, hail-fellow-well-met.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Comradery</ent><br/
<hw>Com"rade*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>camarederie</ets>.]</ety> <def>The spirit of comradeship; comradeship.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Certainly<rdquo/, said Dunham, with the <qex>comradery</qex> of the smoker.</q> <rj><qau>W. D. Howells.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comradeship</ent><br/
<hw>Com"rade*ship</hw>, <pos>n.</pos> <def>The state of being a comrade; intimate fellowship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comrogue</ent><br/
<hw>Com"rogue`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fellow rogue.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comtism</ent><br/
<hw>Com"tism</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Named after the French philosopher, Auguste <etsep>Comte</etsep>.]</ety> <def>Positivism; the positive philosophy. See <er>Positivism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Comtist</ent><br/
<hw>Com"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A disciple of Comte; a positivist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Con-</ent><br/
<hw>Con-</hw> <pr>(<?/)</pr>. <def>A prefix, fr. L. <xex>cum</xex>, signifying <xex>with</xex>, <xex>together</xex>, etc. See <er>Com-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Con</ent><br/
<hw>Con</hw>, <pos>adv.</pos> <ety>[Abbrev. from L. <ets>contra</ets> against.]</ety> <def>Against the affirmative side; in opposition; on the negative side; -- The antithesis of <xex>pro</xex>, and usually in connection with it. See <er>Pro</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Con</ent><br/
<hw>Con</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conning</conjf>.]</vmorph> <ety>[AS. <ets>cunnan</ets> to know, be able, and (derived from this) <ets>cunnian</ets> to try, test. See <er>Can</er>, <ets>v. t. & i.</ets>]</ety> <sn>1.</sn> <def>To know; to understand; to acknowledge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of muses, Hobbinol, I <qex>con</qex> no skill.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They say they <qex>con</qex> to heaven the highway.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To study in order to know; to peruse; to learn; to commit to memory; to regard studiously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fixedly did look<br/
Upon the muddy waters which he <qex>conned</qex><br/
As if he had been reading in a book.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I did not come into Parliament to <qex>con</qex> my lesson.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To con answer</b></col>, <cd>to be able to answer.</cd> <mark>[Obs.]</mark> -- <col><b>To con thanks</b></col>, <cd>to thank; to acknowledge obligation.</cd> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Con</ent><br/
<hw>Con</hw>, <pos>v. t.</pos> <ety>[See <er>Cond</er>.]</ety> <fld>(Naut.)</fld> <def>To conduct, or superintend the steering of (a vessel); to watch the course of (a vessel) and direct the helmsman how to steer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conacre</ent><br/
<hw>Con*a"cre</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To underlet a portion of, for a single crop; -- said of a farm.</def> <mark>[Ireland]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conacre</ent><br/
<hw>Con*a"cre</hw>, <pos>n.</pos> <def>A system of letting a portion of a farm for a single crop.</def> <mark>[Ireland]</mark> <def2><def>Also used adjectively; <as>as, the <ex>conacre</ex> system or principle</as>.</def></def2> <rj><au>Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conakry</ent><br/
<hw>Conakry</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>Guinea</country>.  Population (2000) = 1,508,000.</def><br/
[<source>PJC</source>]</p>

<p><ent>Conarium</ent><br/
||<hw>Co*na"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kwna`rion</grk>.]</ety> <fld>(Anat.)</fld> <def>The pineal gland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conation</ent><br/
<hw>Co*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conatio</ets>.]</ety> <fld>(Philos.)</fld> <def>The power or act which directs or impels to effort of any kind, whether muscular or psychical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of <qex>conation</qex>, in other words, of desire and will.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conative</ent><br/
<hw>Co"na*tive</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[See <er>Conatus</er>.]</ety> <def>Of or pertaining to conation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This division of mind into the three great classes of the cognitive faculties, the feelings, . . . and the exertive or <qex>conative</qex> powers, . . . was first promulgated by Kant.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conatus</ent><br/
||<hw>Co*na"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>conatus</ets>, p. p. of <ets>conari</ets> to attempt.]</ety> <def>A natural tendency inherent in a body to develop itself; an attempt; an effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>conatus</qex> could give prickles to the porcupine or hedgehog, or to the sheep its fleece?</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concamerate</ent><br/
<hw>Con*cam"er*ate</hw> <pr>(k<ocr/n*k<acr/m"<etil/r*<amac/t)</pr>, <pos>v. t.</pos> <ety>[L. <ets>concameratus</ets>, p. p. of <ets>concamerare</ets> to arch over. See <er>Camber</er>.]</ety> <sn>1.</sn> <def>To arch over; to vault.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of the upper beak an inch and a half consisteth of one <qex>concamerated</qex> bone.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To divide into chambers or cells.</def> <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concameration</ent><br/
<hw>Con*cam`er*a"tion</hw> <pr>(-<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>concameratio</ets>.]</ety> <sn>1.</sn> <def>An arch or vault.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A chamber of a multilocular shell.</def> <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concatenate</ent><br/
<hw>Con*cat"e*nate</hw> <pr>(k<ocr/n*k<acr/t"<esl/*n<amac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concatenated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Concatenating</conjf>.]</vmorph> <ety>[L. <ets>concatenatus</ets>, p. p. of <ets>concatenare</ets> to concatenate. See <er>Catenate</er>.]</ety> <def>To link together; to unite in a series or chain, as things depending on one another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This all things friendly will <qex>concatenate</qex>.</q> <rj><qau>Dr. H. More</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concatenation</ent><br/
<hw>Con*cat`e*na"tion</hw> <pr>(-n<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>concatenatio</ets>.]</ety> <def>A series of links united; a series or order of things depending on each other, as if linked together; a chain, a succession.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stoics affirmed a fatal, unchangeable <qex>concatenation</qex> of causes, reaching even to the illicit acts of man's will.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>concatenation</qex> of explosions.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concause</ent><br/
<hw>Con*cause"</hw> <pr>(-k<add/z")</pr>, <pos>n.</pos> <def>A joint cause.</def> <rj><au>Fotherby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 294 --></p>

<p><ent>Concavation</ent><br/
<hw>Con`ca*va"tion</hw> <pr>(k<ocr/<nsm/`k<adot/*v<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <def>The act of making concave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concave</ent><br/
<hw>Con"cave</hw> <pr>(k<ocr/<nsm/*k<amac/*v" <or/ k<ocr/n"-; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>concavus</ets>; <ets>con-</ets> + <ets>cavus</ets> hollow: cf. F. <ets>concave</ets>. See <er>Cave</er> a hollow.]</ety> <sn>1.</sn> <def>Hollow and curved or rounded; vaulted; -- said of the interior of a curved surface or line, as of the curve of the of the inner surface of an eggshell, in opposition to <contr>convex</contr>; <as>as, a <ex>concave</ex> mirror; the <ex>concave</ex> arch of the sky</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hollow; void of contents.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>concave</qex> . . . as a worm-eaten nut.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concave</ent><br/
<hw>Con"cave</hw>, <pos>n.</pos> <ety>[L. <ets>concavum</ets>.]</ety> <sn>1.</sn> <def>A hollow; an arched vault; a cavity; a recess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Up to the fiery <qex>concave</qex> towering hight.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A curved sheath or breasting for a revolving cylinder or roll.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concave</ent><br/
<hw>Con"cave</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>concaved</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Concaving</conjf>.]</vmorph> <def>To make hollow or concave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concaved</ent><br/
<hw>Con"caved</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Bowed in the form of an arch; -- called also <altname>arched</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concaveness</ent><br/
<hw>Con"cave*ness</hw>, <pos>n.</pos> <def>Hollowness; concavity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concavity</ent><br/
<hw>Con*cav"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Concavities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>concavitas</ets>: cf. F. <ets>concavit<eacute/</ets>. See <er>Concave</er>.]</ety> <def>A concave surface, or the space bounded by it; the state of being concave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concavo-concave</ent><br/
<hw>Con*ca`vo-con"cave</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Concave or hollow on both sides; double concave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concavo-convex</ent><br/
<hw>Con*ca`vo-con"vex</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Concave on one side and convex on the other, as an eggshell or a crescent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Optics)</fld> <def>Specifically, having such a combination of concave and convex sides as makes the focal axis the shortest line between them. See <xex>Illust.</xex> under <er>Lens</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concavous</ent><br/
<hw>Con*ca*"vous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concavus</ets>.]</ety> <def>Concave.</def>  <rj><au>Abp. potter.</au></rj></p>

<p>-- <wordforms><wf>Con*ca"vous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceal</ent><br/
<hw>Con*ceal"</hw> <pr>(k<ocr/n*s<emac/l")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concealed</conjf> <pr>(k<ocr/n*s<emac/ld")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Concealing</conjf>.]</vmorph> <ety>[OF. <ets>conceler</ets>, L. <ets>concelare</ets>; <ets>con-</ets> + <ets>celare</ets>to hide; akin to AS. <ets>helan</ets>, G. hehlen, E. <ets>hele</ets> (<ets>to cover</ets>), <ets>helmet</ets>. See <er>Hell</er>, <er>Helmet</er>.]</ety> <def>To hide or withdraw from observation; to cover; to cover or keep from sight; to prevent the discovery of; to withhold knowledge of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is the glory of God to <qex>conceal</qex> a thing.</q> <rj><qau>Prov. xxv. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Declare ye among the nations, . . . publish and <qex>conceal</qex> not.</q> <rj><qau>Jer. l. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He which finds him shall deserve our thanks, . . . <br/
He that <qex>conceals</qex> him, death.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To hide; secrete; screen; cover; disguise; dissemble; mask; veil; cloak; screen.</syn> <usage> -- <er>To Conceal</er>, <er>Hide</er>, <er>Disguise</er>, <er>Dissemble</er>, <er>Secrete</er>. To <xex>hide</xex> is the generic term, which embraces all the rest. To <xex>conceal</xex> is simply not make known what we wish to keep secret. In the Bible <xex>hide</xex> often has the specific meaning of <xex>conceal</xex>. See <au>1 Sam. iii. 17, 18.</au> To <xex>disguise</xex> or <xex>dissemble</xex> is to conceal by assuming some false appearance. To <xex>secrete</xex> is to hide in some place of secrecy. A man may <xex>conceal</xex> facts, <xex>disguise</xex> his sentiments, <xex>dissemble</xex> his feelings, <xex>secrete</xex> stolen goods.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Bur double griefs afflict <qex>concealing</qex> hearts.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Both <qex>dissemble</qex> deeply their affections.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We have in these words a primary sense, which reveals a future state, and a secondary sense, which <qex>hides</qex> and <qex>secretes</qex> it.</q> <rj><qau>Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concealable</ent><br/
<hw>Con*ceal"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being concealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concealed</ent><br/
<hw>Con*cealed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Hidden; kept from sight; secreted.</def></p>

<p>-- <wordforms><wf>Con*ceal"ed*ly</wf> <pr>(<?/)</pr>, <pos>adv.</pos> -- <wf>Con*ceal"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Concealed weapons</b></col> <fld>(Law)</fld>, <cd>dangerous weapons so carried on the person as to be knowingly or willfully concealed from sight, -- a practice forbidden by statute.<-- in some states! --></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concealer</ent><br/
<hw>Con*ceal"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conceals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concealment</ent><br/
<hw>Con*ceal"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>concelement</ets>.]</ety> <sn>1.</sn> <def>The act of concealing; the state of being concealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But let <qex>concealment</qex>, like a worm i' the bud,<br/
Feed on her damask cheek.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some dear cause<br/
Will in <qex>concealment</qex> wrap me up awhile.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place of hiding; a secret place; a retreat frem observation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cleft tree<br/
Offers its kind <qex>concealment</qex> to a few.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A secret; out of the way knowledge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Well read in strange <qex>concealments</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Suppression of such facts and circumstances as in justice ought to be made known.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concede</ent><br/
<hw>Con*cede"</hw> <pr>(k<ocr/n*s<emac/d")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conceded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conceding</conjf>.]</vmorph> <ety>[L. <ets>concedere</ets>, <ets>concessum</ets>; <ets>con-</ets> + <ets>cedere</ets> to go along, give way, yield: cf. F. <ets>conc<eacute/der</ets>. See <er>Cede</er>.]</ety> <sn>1.</sn> <def>To yield or suffer; to surrender; to grant; <as>as, to <ex>concede</ex> the point in question</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To grant, as a right or privilege; to make concession of.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To admit to be true; to acknowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>concede</qex> that their citizens were those who lived under different forms.</q> <rj><qau>Burke.</qau></rj></p>

<p><syn><b>Syn.</b> -- To grant; allow; admit; yield; surrender.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concede</ent><br/
<hw>Con*cede"</hw>, <pos>v. i.</pos> <def>To yield or make concession.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I wished you to <qex>concede</qex> to America, at a time when she prayed concession at our feet.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conceded</ent><br/
<hw>conceded</hw> <pos>adj. prenom.</pos>  <def>acknowledged.  Opposite of <ant>unacknowledged</ant>.</def><br/
<syn><b>Syn. --</b> admitted(prenominal), avowed(prenominal), confessed(prenominal), self-confessed(prenominal).</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>conceding</ent><br/
<hw>conceding</hw> <pos>adj.</pos>  <def>signifying a concession.</def> <mark>[prenominal]</mark><br/
<syn><b>Syn. --</b> concessive.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conceding</ent><br/
<hw>conceding</hw> <pos>n.</pos> <def>the act of conceding or yielding.</def><br/
<syn><b>Syn. --</b> concession, yielding.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conceit</ent><br/
<hw>Con*ceit"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Through French, fr. L. <ets>conceptus</ets> a conceiving, conception, fr. <ets>concipere</ets> to conceive: cf. OF. p. p. nom. <ets>conciez</ets> conceived. See <er>Conceive</er>, and cf. <er>Concept</er>, <er>Deceit</er>.]</ety> <sn>1.</sn> <def>That which is conceived, imagined, or formed in the mind; idea; thought; image; conception.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In laughing, there ever procedeth a <qex>conceit</qex> of somewhat ridiculous.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man wise in his own <qex>conceit</qex>.</q> <rj><qau>Prov. xxvi. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Faculty of conceiving ideas; mental faculty; apprehension; <as>as, a man of quick <ex>conceit</ex></as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>How often, alas! did her eyes say unto me that they loved! and yet I, not looking for such a matter, had not my <qex>conceit</qex> open to understand them.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Quickness of apprehension; active imagination; lively fancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His wit's as thick as Tewksbury mustard; there's more <qex>conceit</qex> in him than is in a mallet.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A fanciful, odd, or extravagant notion; a quant fancy; an unnatural or affected conception; a witty thought or turn of expression; a fanciful device; a whim; a quip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On his way to the gibbet, a freak took him in the head to go off with a <qex>conceit</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some to <qex>conceit</qex> alone their works confine,<br/
And glittering thoughts struck out at every line.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Tasso is full of <qex>conceits</qex> . . . which are not only below the dignity of heroic verse but contrary to its nature.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An overweening idea of one's self; vanity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Plumed with <qex>conceit</qex> he calls aloud.</q> <rj><qau>Cotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Design; pattern.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In conceit with</b></col>, <cd>in accord with; agreeing or conforming.</cd> -- <col><b>Out of conceit with</b></col>, <cd>not having a favorable opinion of; not pleased with; as, a man is <xex>out of conceit with</xex> his dress.</cd> -- <col><b>To put <plain>[one]</plain> out of conceit with</b></col>, <cd>to make one indifferent to a thing, or in a degree displeased with it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceit</ent><br/
<hw>Con*ceit"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To conceive; to imagine.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The strong, by <qex>conceiting</qex> themselves weak, are therebly rendered as inactive . . . as if they really were so.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One of two bad ways you must <qex>conceit</qex> me,<br/
Either a coward or a flatterer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceit</ent><br/
<hw>Con*ceit"</hw>, <pos>v. i.</pos> <def>To form an idea; to think.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Those whose . . . vulgar apprehensions <qex>conceit</qex> but low of matrimonial purposes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceited</ent><br/
<hw>Con*ceit"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Endowed with fancy or imagination.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was . . . pleasantly <qex>conceited</qex>, and sharp of wit.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Entertaining a flattering opinion of one's self; vain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If you think me too <qex>conceited</qex><br/
Or to passion quickly heated.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conceited</qex> of their own wit, science, and politeness.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Curiously contrived or designed; fanciful.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>conceited</qex> chair to sleep in.</q> <rj><qau>Evelyn.</qau></rj></p>

<p><syn><b>Syn.</b> -- Vain; proud; opinionated; egotistical.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceitedly</ent><br/
<hw>Con*ceit"ed*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In an egotistical manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fancifully; whimsically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceitedness</ent><br/
<hw>Con*ceit"ed*ness</hw>, <pos>n.</pos> <def>The state of being conceited; conceit; vanity.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceitless</ent><br/
<hw>Con*ceit"less</hw>, <pos>a.</pos> <def>Without wit; stupid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Think'st thou I am so shallow, so <qex>conceitless</qex>.<br/
To be seduced by thy flattery?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceivable</ent><br/
<hw>Con*ceiv"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>concevable</ets>.]</ety> <def>Capable of being conceived, imagined, or understood.</def> <ldquo/Any <xex>conceivable</xex> weight.<rdquo/  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is not <qex>conceivable</qex> that it should be indeed that very person whose shape and voice it assumed.</q> <rj><qau>Atterbury.</qau></rj></p>

<p>-- <wordforms><wf>Con*ceiv"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Con*ceiv"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceive</ent><br/
<hw>Con*ceive"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conceived</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conceiving</conjf>.]</vmorph> <ety>[OF. <ets>conzoivre</ets>, <ets>concever</ets>, <ets>conceveir</ets>, F. <ets>concevoir</ets>, fr. L. <ets>oncipere</ets> to take, to conceive; <ets>con-</ets> + <ets>capere</ets> to seize or take. See <er>Capable</er>, and cf. <er>Conception</er>.]</ety> <sn>1.</sn> <def>To receive into the womb and begin to breed; to begin the formation of the embryo of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She hath also <qex>conceived</qex> a son in her old age.</q> <rj><qau>Luke i. 36.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form in the mind; to plan; to devise; to generate; to originate; <as>as, to <ex>conceive</ex> a purpose, plan, hope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was among the ruins of the Capitol that I first <qex>conceived</qex> the idea of a work which has amused and exercised near twenty years of my life.</q> <rj><qau>Gibbon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conceiving</qex> and uttering from the heart words of falsehood.</q> <rj><qau>Is. lix. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To apprehend by reason or imagination; to take into the mind; to know; to imagine; to comprehend; to understand.</def> <ldquo/I <xex>conceive</xex> you.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O horror, horror, horror! Tongue nor heart<br/
Cannot <qex>conceive</qex> nor name thee!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You will hardly <qex>conceive</qex> him to have been bred in the same climate.</q> <rj><qau>Swift.</qau></rj></p>

<p><syn><b>Syn.</b> -- To apprehend; imagine; suppose; understand; comprehend; believe; think.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceive</ent><br/
<hw>Con*ceive"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have an embryo or fetus formed in the womb; to breed; to become pregnant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A virgin shall <qex>conceive</qex>, and bear a son.</q> <rj><qau>Isa. vii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have a conception, idea, or opinion; think; -- with <xex>of</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conceive</qex> of things clearly and distinctly in their own natures.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceiver</ent><br/
<hw>Con*ceiv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conceives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concelebrate</ent><br/
<hw>Con*cel"e*brate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>concelebratus</ets>, p. p. of <ets>concelebrare</ets> to concelebrate.]</ety> <def>To celebrate together.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concent</ent><br/
<hw>Con*cent"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concentus</ets>, fr. <ets>concinere</ets> to sing together; <ets>con-</ets> + <ets>canere</ets> to sing.]</ety> <sn>1.</sn> <def>Concert of voices; concord of sounds; harmony; <as>as, a <ex>concent</ex> of notes</as>.</def> <mark>[Archaic.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That undisturbed song of pure <qex>concent</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consistency; accordance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>concent</qex> to his own principles.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentre</ent><br/
<ent>Concenter</ent><br/
<mhw>{ <hw>Con*cen"ter</hw>, <hw>Con*cen"tre</hw>  }</mhw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concentered</conjf> or <conjf>Concentred</conjf> <pr>(<?/)</pr>; <pos>p. pr & vb. n.</pos> <conjf>Concentering</conjf> <pr>(?)</pr> or <conjf>Concentring</conjf> <pr>(<?/)</pr>.]</vmorph> <ety>[F. <ets>concentrer</ets>, fr. L. <ets>con-</ets> + <ets>centrum</ets> center. See <er>Center</er>, and cf. <er>Concentrate</er>]</ety> <def>To come to one point; to meet in, or converge toward, a common center; to have a common center.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God, in whom all perfections <qex>concenter</qex>.</q> <rj><qau>Bp. Beveridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentre</ent><br/
<ent>Concenter</ent><br/
<mhw>{ <hw>Con*cen"ter</hw>, <hw>Con*cen"tre</hw>  }</mhw>, <pos>v. t.</pos> <def>To draw or direct to a common center; to bring together at a focus or point, as two or more lines; to concentrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In thee <qex>concentering</qex> all their precious beams.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All is <qex>concentered</qex> in a life intense.</q> <rj><qau>Byren.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentrate</ent><br/
<hw>Con*cen"trate</hw> <pr>(? <or/ ?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concentrated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Concentrating</conjf>.]</vmorph> <ety>[Pref. <ets>con-</ets> + L. <ets>centrum</ets> center. Cf. <er>Concenter</er>.]</ety> <sn>1.</sn> <def>To bring to, or direct toward, a common center; to unite more closely; to gather into one body, mass, or force; to fix; <as>as, to <ex>concentrate</ex> rays of light into a focus; to <ex>concentrate</ex> the attention</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>(He) <qex>concentrated</qex> whole force at his own camp.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To increase the strength and diminish the bulk of, as of a liquid or an ore; to intensify, by getting rid of useless material; to condense; <as>as, to <ex>concentrate</ex> acid by evaporation; to <ex>concentrate</ex> by washing</as>; -- opposed to <ant>dilute</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Spirit of vinegar <qex>concentrated</qex> and reduced to its greatest strength.</q> <rj><qau>Arbuthnot.</qau></rj></p>

<p><syn><b>Syn.</b> -- To combine; to condense; to consolidate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentrate</ent><br/
<hw>Con*cen"trate</hw> <pr>(? <or/ ?)</pr>, <pos>v. i.</pos> <def>To approach or meet in a common center; to consolidate; <as>as, population tends to <ex>concentrate</ex> in cities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>concentrated</ent><br/
<hw>concentrated</hw> <pos>adj.</pos> <sn>1.</sn> <def>Having a high density of (the indicated substance); <as>as, a narrow thread of <ex>concentrated</ex> ore</as>.</def> <note>[Narrower terms: <stype>undiluted</stype> (vs. diluted)]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>Gathered together or made less diffuse; <as>as, their <ex>concentrated</ex> efforts; his <ex>concentrated</ex> attention</as>. Opposite of <ant>distributed</ant> or <ant>diffused</ant>.</def> <note>[Narrower terms: <stype>bunched</stype>, <stype>bunchy</stype>, <stype>clustered</stype>; <stype>centered</stype>, <stype>centred</stype>, <stype>centralized</stype>, <stype>focused</stype>;  <stype>undivided</stype>]</note> <see>[Also See: <er>compact</er>.]</see><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>Intense; in an extreme degree; -- of mental phenomena; <as>as, her <ex>concentrated</ex> passion held them at bay</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>being the most concentrated solution possible at a given temperature; unable to dissolve still more of a substance.  Opposite of <ant>dilute</ant> or <ant>unsaturated</ant>.</def> <note>[Narrower terms: <stype>supersaturated</stype>]</note><br/
<syn><b>Syn. --</b> saturated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn>  <def>reduced to a stronger or more concentrated form; <as>as, <ex>concentrated</ex> sulfuric acid</as>.  Opposite of <ant>diluted</ant>.</def><br/
<syn><b>Syn. --</b> condensed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn>  <def>characterized by intensity; especially when imposed from without; -- of actions; <as>as, <ex>concentrated</ex> study</as>.</def><br/
<syn><b>Syn. --</b> intensive.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>7.</sn>  <def>characterized by mental concentration.</def><br/
<syn><b>Syn. --</b> intent.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Concentration</ent><br/
<hw>Con`cen*tra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>concentration</ets>.]</ety> <sn>1.</sn> <def>The act or process of concentrating; the process of becoming concentrated, or the state of being concentrated; concentration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Concentration</qex> of the lunar beams.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Intense <qex>concetration</qex> of thought.</q> <rj><qau>Sir J. Herschel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of reducing the volume of a liquid, as by evaporation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The acid acquires a higher degree of <qex>concentration</qex>.</q> <rj><qau>Knight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <def>The act or process of removing the dress of ore and of reducing the valuable part to smaller compass, as by currents of air or water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentrative</ent><br/
<hw>Con*cen"tra*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving or tending to concentrate; characterized by concentration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A discrimination is only possible by a <qex>concentrative</qex> act, or act of attention.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentrativeness</ent><br/
<hw>Con*cen"tra*tive*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality of concentrating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phren.)</fld> <def>The faculty or propensity which has to do with concentrating the intellectual the intellectual powers.</def>  <rj><au>Combe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentrator</ent><br/
<hw>Con"cen*tra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <sn>1.</sn> <def>An apparatus for the separation of dry comminuted ore, by exposing it to intermittent puffs of air.</def>  <rj><au>Knight.</au></rj></p>

<p><sn>2.</sn> <fld>(Firearms)</fld> <def>A frame or ring of wire or hard paper fitting into the cartridge case used in some shotguns, and holding the shot together when discharged, to secure close shooting; also, a device for slightly narrowing the bore at the muzzle for the same purpose.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Concentrical</ent><br/
<ent>Concentric</ent><br/
<mhw>{ <hw>Con*cen"tric</hw> <pr>(?)</pr>, <hw>Con*cen"tric*al</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <ety>[F. <ets>concentrique</ets>. See <er>Concenter</er>.]</ety> <def>Having a common center, as circles of different size, one within another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Concentric</qex> circles upon the surface of the water.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Concentrical</qex> rings like those of an onion.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentric</ent><br/
<hw>Con*cen"tric</hw>, <pos>n.</pos> <def>That which has a common center with something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Its pecular relations to its <qex>concentrics</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentrically</ent><br/
<hw>Con*cen"tric*al*ly</hw>, <pos>adv.</pos> <def>In a concentric manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentricity</ent><br/
<hw>Con`cen*tric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being concentric.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concentual</ent><br/
<hw>Con*cen"tu*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<it>From</it> <er>Concent</er>.]</ety> <def>Possessing harmony; accordant.</def> <mark>[R.]</mark>  <rj><au>Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concept</ent><br/
<hw>Con"cept</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conceptus</ets> (cf. neut. <ets>conceptum</ets> fetus), p. p. of <ets>concipere</ets> to conceive: cf. F. <ets>concept</ets>. See <er>Conceit</er>.]</ety> <def>An abstract general conception; a notion; a universal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The words <qex>conception</qex>, <qex>concept</qex>, <qex>notion</qex>, should be limited to the thought of what can not be represented in the imagination; as, the thought suggested by a general term.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptacle</ent><br/
<hw>Con*cep"ta*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conceptaculum</ets>, fr. <ets>concipere</ets> to receive. See <er>Conceive</er>.]</ety> <sn>1.</sn> <def>That in which anything is contained; a vessel; a receiver or receptacle.</def> <mark>[Obs.]</mark>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A pericarp, opening longitudinally on one side and having the seeds loose in it; a follicle; a double follicle or pair of follicles.</def> <sd>(b)</sd> <def>One of the cases containing the spores, etc., of flowerless plants, especially of algae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptibility</ent><br/
<hw>Con*cep`ti*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being conceivable; conceivableness.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptible</ent><br/
<hw>Con*cep"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Conceive</er>.]</ety> <def>Capable of being conceived; conceivable.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conception</ent><br/
<hw>Con*cep"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>conception</ets>, L. <ets>conceptio</ets>, fr. <ets>concipere</ets> to conceive. See <er>Conceive</er>.]</ety> <sn>1.</sn> <def>The act of conceiving in the womb; the initiation of an embryonic animal life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will greaty multiply thy sorrow and thy <qex>conception</qex>.</q> <rj><qau>Gen. iii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being conceived; beginning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Joy had the like <qex>conception</qex> in our eyes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The power or faculty of apprehending of forming an idea in the mind; the power of recalling a past sensation or perception.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under the article of <qex>conception</qex>, I shall confine myself to that faculty whose province it is to enable us to form a notion of our past sensations, or of the objects of sense that we have formerly perceived.</q> <rj><qau>Stewart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The formation in the mind of an image, idea, or notion, apprehension.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conception</qex> consists in a conscious act of the understanding, bringing any given object or impression into the same class with any number of other objects or impression, by means of some character or characters common to them all.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The image, idea, or notion of any action or thing which is formed in the mind; a concept; a notion; a universal; the product of a rational belief or judgment. See <er>Concept</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Herodotus] says that the sun draws or attracts the water; a metaphorical term obviously intended to denote some more general and abstract <qex>conception</qex> than that of the visible operation which the word primarily signifies.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Idea; purpose; design.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Note this dangerous <qex>conception</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Conceit; affected sentiment or thought.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . is full of <qex>conceptions</qex>, points of epigram, and witticism.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Idea; notion; perception; apprehemsion; comprehension.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 295 --></p>

<p><ent>Conceptional</ent><br/
<hw>Con*cep"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to conception.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptionalist</ent><br/
<hw>Con*cep"tion*al*ist</hw>, <pos>n.</pos> <def>A conceptualist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptious</ent><br/
<hw>Con*cep"tious</hw>, <pos>a.</pos> <def>Apt to conceive; fruitful.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptive</ent><br/
<hw>Con*cep"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>conceptif</ets>, L. <ets>conceptivus</ets>.]</ety> <def>Capable of conceiving.</def>  <rj><au>Sir T. Browne</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptual</ent><br/
<hw>Con*cep"tu*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to conception.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conceptualisation</ent><br/
<hw>conceptualisation</hw> <pos>n.</pos> <def>the act of formulating or making a concept of something.</def><br/
<syn><b>Syn. --</b> conceptualization, formulation, formularizing, formularising.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conceptualise</ent><br/
<hw>conceptualise</hw> <pos>v. t.</pos> <def>same as <er>conceptualize</er>.</def> <mark>[British]</mark> <br/
<syn><b>Syn. --</b> conceive, conceive of, conceptualize, ideate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conceptualism</ent><br/
<hw>Con*cep"tu*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metaph.)</fld> <def>A theory, intermediate between realism and nominalism, that the mind has the power of forming for itself general conceptions of individual or single objects.</def>  <rj><au>Stewart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conceptualist</ent><br/
<hw>Con*cep"tu*al*ist</hw>, <pos>n.</pos> <fld>(Metaph.)</fld> <def>One who maintains the theory of conceptualism.</def>  <rj><au>Stewart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conceptuality</ent><br/
<hw>conceptuality</hw> <pos>n.</pos> <def>an elaborated concept.</def><br/
<syn><b>Syn. --</b> conceptualization.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conceptualization</ent><br/
<hw>conceptualization</hw> <pos>n.</pos> <def>the act of formulating or making a concept of something.</def><br/
<syn><b>Syn. --</b> conceptualisation, formulation, formularizing, formularising.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an elaborated concept.</def><br/
<syn><b>Syn. --</b> conceptuality.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conceptualize</ent><br/
<hw>conceptualize</hw> <pos>v. t.</pos> <sn>1.</sn> <def>to to form a concept of; <as>as, He could not <ex>conceptualize</ex> a robot that would help paralyzed patients</as>.</def><br/
<syn><b>Syn. --</b> conceive, conceive of, conceptualise, ideate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Concern</ent><br/
<hw>Con*cern"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concerned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Concerning</conjf>.]</vmorph> <ety>[F. <ets>concerner</ets>, LL. <ets>concernere</ets> to regard, concern, fr. L. <ets>concernere</ets> to mix or mingle together, as in a sieve for separating; <ets>con-</ets> + <ets>cernere</ets> to separate, sift, distinguish by the senses, and especially by the eyes, to perceive, see. See <er>Certain</er>.]</ety> <sn>1.</sn> <def>To relate or belong to; to have reference to or connection with; to affect the interest of; to be of importance to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Preaching the kingdom of God, and teaching those things which <qex>concern</qex> the Lord Jesus Christ.</q> <rj><qau>Acts xxviii. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our wars with France have affected us in our most tender interests, and <qex>concerned</qex> us more than those with any other nation.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It much <qex>concerns</qex> a preacher first to learn<br/
The genius of his audience and their turn.</q> <rj><qau>Dodsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ignorant, so far as the usual instruction is <qex>concerned</qex>.</q> <rj><qau>J. F. Cooper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To engage by feeling or sentiment; to interest; <as>as, a good prince <ex>concerns</ex> himself in the happiness of his subjects</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They think themselves out the reach of Providence, and no longer <qex>concerned</qex> to solicit his favor.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concern</ent><br/
<hw>Con*cern"</hw>, <pos>v. i.</pos> <def>To be of importance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which to deny <qex>concerns</qex> more than avails.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concern</ent><br/
<hw>Con*cern"</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which relates or belongs to one; business; affair.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The private <qex>concerns</qex> of fanilies.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which affects the welfare or happiness; interest; moment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mysterious secrets of a high <qex>concern</qex>.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Interest in, or care for, any person or thing; regard; solicitude; anxiety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O Marcia, let me hope thy kind <qex>concerns</qex><br/
And gentle wishes follow me to battle.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Com.)</fld> <def>Persons connected in business; a firm and its business; <as>as, a banking <ex>concern</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The whole concern</b></col>, <cd>all connected with a particular affair or business.</cd></cs></p>

<p><syn><b>Syn.</b> -- Care; anxiety; solicitude; interest; regard; business; affair; matter; moment. See <er>Care</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concerned</ent><br/
<hw>Con*cerned"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Concern</er>, <pos>v. t.</pos>, 2.]</ety> <def>Disturbed; troubled; solicitous; <as>as, to be much <ex>concerned</ex> for the safety of a friend</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concernedly</ent><br/
<hw>Con*cern"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a concerned manner; solicitously; sympathetically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concerning</ent><br/
<hw>Con*cern"ing</hw>, <pos>prep.</pos> <def>Pertaining to; regarding; having relation to; respecting; as regards.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have accepted thee <qex>concerning</qex> this thing.</q> <rj><qau>Gen. xix. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord hath spoken good <qex>concerning</qex> Israel.</q> <rj><qau>Num. x. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concerning</ent><br/
<hw>Con*cern"ing</hw>, <pos>a.</pos> <def>Important.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So great and so <qex>concerning</qex> truth.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concerning</ent><br/
<hw>Con*cern"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That in which one is concerned or interested; concern; affair; interest.</def> <ldquo/Our everlasting <xex>concernments</xex>.<rdquo/  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To mix with thy <qex>concernments</qex> I desist.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Importance; moment; consequence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let every action of <qex>concernment</qex> to begun with prayer.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Concern; participation; interposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He married a daughter to the earl without any other approbation of her father or <qex>concernment</qex> in it, than suffering him and her come into his presence.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Emotion of mind; solicitude; anxiety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While they are so eager to destroy the fame of others, their ambition is manifest in their <qex>concernment</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concert</ent><br/
<hw>Con*cert"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concerted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Concerting</conjf>.]</vmorph> <ety>[F. <ets>concerter</ets>, It. <ets>concertare</ets>, <ets>conertare</ets>, prob. from L. <ets>consertus</ets>, p. p. of <ets>conserere</ets> to join together; <ets>con-</ets> + <ets>serere</ets> to join together, influenced by <ets>concertare</ets> to contend; <ets>con-</ets> + <ets>centare</ets> to strive; properly, to try to decide; fr. <ets>cernere</ets> to distinguish. See <er>Series</er>, and cf. <er>Concern</er>.]</ety> <sn>1.</sn> <def>To plan together; to settle or adjust by conference, agreement, or consultation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was <qex>concerted</qex> to begin the siege in March.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To plan; to devise; to arrange.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A commander had more trouble to <qex>concert</qex> his defense before the people than to plan . . . the campaign.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concert</ent><br/
<hw>Con*cert"</hw>, <pos>v. i.</pos> <def>To act in harmony or conjunction; to form combined plans.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ministers of Denmark were appointed to <qex>concert</qex> with Talbot.</q> <rj><qau>Bp. Burnet</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concert</ent><br/
<hw>Con"cert</hw> <pr>(k<ocr/n"s<etil/rt)</pr>, <pos>n.</pos> <ety>[F. <ets>concert</ets>, It. <ets>concerto</ets>, <ets>conserto</ets>, fr. <ets>concertare</ets>. See <er>Concert</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Agreement in a design or plan; union formed by mutual communication of opinions and views; accordance in a scheme; harmony; simultaneous action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All these discontents, how ruinous soever, have arisen from the want of a due communication and <qex>concert</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Musical accordance or harmony; concord.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us in <qex>concert</qex> to the season sing.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A musical entertainment in which several voices or instruments take part.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Visit by night your lady's chamber window<br/
With some sweet <qex>concert</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And boding screech owls make the <qex>concert</qex> full.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Concert pitch</b></col>. <cd>See under <er>Pitch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertante</ent><br/
<hw>Con`cer*tan"te</hw> <pr>(?; It. ?)</pr>, <pos>n.</pos> <ety>[It., orig p. pr. of <ets>concertare</ets> to form or perform a concert. See <er>Concert</er>.]</ety> <fld>(Mus.)</fld> <def>A concert for two or more principal instruments, with orchestral accompaniment. Also adjectively; <as>as, <ex>concertante</ex> parts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertation</ent><br/
<hw>Con`cer*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concertatio</ets>.]</ety> <def>Strife; contention.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertative</ent><br/
<hw>Con*cer"ta*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concertativus</ets>.]</ety> <def>Contentious; quarrelsome.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concerted</ent><br/
<hw>Con*cert"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Mutually contrived or planned; agreed on; <as>as, <ex>concerted</ex> schemes, signals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Concerted piece</b></col> <fld>(Mus.)</fld>, <cd>a composition in parts for several voices or instrument, as a trio, a quartet, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertina</ent><br/
<hw>Con`cer*ti"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From It. <ets>concerto</ets> a concert.]</ety> <def>A small musical instrument on the principle of the accordion. It is a small elastic box, or bellows, having free reeds on the inside, and keys and handles on the outside of each of the two hexagonal heads.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertino</ent><br/
<hw>Con`cer*ti"no</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Concertina</er>.]</ety> <fld>(Mus.)</fld> <def>A piece for one or more solo instruments with orchestra; -- more concise than the <contr>concerto</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertion</ent><br/
<hw>Con*cer"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of concerting; adjustment.</def> <mark>[R.]</mark>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concertmeister</ent><br/
||<hw>Con*cert`meis"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G.]</ety> <fld>(Mus.)</fld> <def>The head violinist or leader of the strings in an orchestra; the sub-leader of the orchestra; concert master.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concerto</ent><br/
<hw>Con*cer"to</hw> <pr>(?; It. ?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Concertos</plw> <pr>(#)</pr>.</plu> <ety>[It. See <er>Concert</er>, <pos>n.</pos>]</ety> <fld>(Mus.)</fld> <def>A composition (usually in symphonic form with three movements) in which one instrument (or two or three) stands out in bold relief against the orchestra, or accompaniment, so as to display its qualities or the performer's skill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>European concert</ent><br/
<ent>Concert of Europe</ent><br/
<mhw>{ <hw>Concert of Europe</hw>, <it>or</it> <hw>European concert</hw>}</mhw>. <def>An agreement or understanding between the chief European powers to take only joint action in the (European) Eastern Question.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Concert of the powers</ent><br/
<hw>Concert of the powers</hw>. <def>An agreement or understanding between the chief European powers, the United States, and Japan in 1900 to take only joint action in the Chinese aspect of the Eastern Question.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Concession</ent><br/
<hw>Con*ces"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concessio</ets>, fr. <ets>concedere</ets>: cf. F. <ets>concession</ets>. See <er>Concede</er>.]</ety> <sn>1.</sn> <def>The act of conceding or yielding; usually implying a demand, claim, or request, and thus distinguished from <xex>giving</xex>, which is voluntary or spontaneous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By mutual <qex>concession</qex> the business was adjusted.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thing yielded; an acknowledgment or admission; a boon; a grant; esp. a grant by government of a privilege or right to do something; <as>as, a <ex>concession</ex> to build a canal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is therefore a <qex>concession</qex>, that he doth . . . believe the Scriptures to be sufficiently plain.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When a lover becomes satisfied by small compliances without further pursuits, then expect to find popular assemblies content with small <qex>concessions</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concessionnaire</ent><br/
<ent>Concessionaire</ent><br/
<mhw>{ <hw>Con*ces`sion*aire"</hw> <pr>(?)</pr>,  ||<hw>Con`ces`sion`naire"</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[F. <ets>concessionnaire</ets>.]</ety> <def>The beneficiary of a concession or grant.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Concessionary</ent><br/
<hw>Con*ces"sion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a concession.</def> -- <def2><pos>n.</pos>; <plu><it>pl.</it> <plw>-ries</plw> <pr>(<?/)</pr></plu>. <def>A concessionaire.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>concessioner</ent><br/
<hw>concessioner</hw> <pos>n.</pos> <def>someone who holds or operates a concession.</def><br/
<syn><b>Syn. --</b> concessionaire.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Concessionist</ent><br/
<hw>Con*ces"sion*ist</hw>, <pos>n.</pos> <def>One who favors concession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concessive</ent><br/
<hw>Con*ces"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concessivus</ets>.]</ety> <def>Implying concession; <as>as, a <ex>concessive</ex> conjunction</as>.</def>  <rj><au>Lowth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concessively</ent><br/
<hw>Con*ces"sive*ly</hw>, <pos>adv.</pos> <def>By way of concession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concessory</ent><br/
<hw>Con*ces"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Conceding; permissive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concettism</ent><br/
<hw>Con*cet"tism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The use of <ets>concetti</ets> or affected conceits.</def> <mark>[R.]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concetto</ent><br/
||<hw>Con*cet"to</hw> <pr>(?; It. ?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Concetti</plw> <pr>(#)</pr>.</plu> <ety>[It., fr. L. <ets>conceptus</ets>. See <er>Conceit</er>.]</ety> <def>Affected wit; a conceit.</def>  <rj><au>Chesterfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conch</ent><br/
<hw>Conch</hw> <pr>(k<ocr/<nsm/k)</pr>, <pos>n.</pos> <ety>[L. <ets>concha</ets>, Gr. <grk>ko`gchh</grk>. See <er>Coach</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A name applied to various marine univalve shells; esp. to those of the genus <gen>Strombus</gen>, which are of large size. <spn>Strombus gigas</spn> is the large pink West Indian conch.  The large king, queen, and cameo conchs are of the genus <gen>Cassis</gen>. See <er>Cameo</er> and <cref>cameo conch</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <ex>conch</ex> is sometimes used as a horn or trumpet, as in fogs at sea, or to call laborers from work.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In works of art, the shell used by Tritons as a trumpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <mark>[often capitalized]</mark> <def>One of the white natives of the Bahama Islands or one of their descendants in the Florida Keys; -- so called from the commonness of the conch there, or because they use it for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch.)</fld> <def>See <er>Concha</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The external ear. See <er>Concha</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concha</ent><br/
||<hw>Con"cha</hw> <pr>(k<ocr/<nsm/"k<adot/)</pr>, <pos>n.</pos> <ety>[LL. (in sense 1), fr. L. <ets>concha</ets>. See <er>Conch</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>The plain semidome of an apse; sometimes used for the entire apse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>The external ear; esp. the largest and deepest concavity of the external ear, surrounding the entrance to the auditory canal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchal</ent><br/
<hw>Con"chal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to the concha, or external ear; <as>as, the <ex>conchal</ex> cartilage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchifer</ent><br/
<hw>Con"chi*fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>conchof<egrave/re</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the Conchifera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchifera</ent><br/
||<hw>Con*chif"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>concha</ets> + <ets>ferre</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>That class of Mollusca which includes the bivalve shells; the Lamellibranchiata. See <er>Mollusca</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchiferous</ent><br/
<hw>Con*chif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Producing or having shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchiform</ent><br/
<hw>Con"chi*form</hw>, <pos>a.</pos> <ety>[<ets>Conch</ets> + <ets>-form</ets>.]</ety> <def>Shaped like one half of a bivalve shell; shell-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchinine</ent><br/
<hw>Con"chi*nine</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Formed by transposition fr. <ets>cinchonine</ets>.]</ety> <def>See <er>Quinidine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchite</ent><br/
<hw>Con"chite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>conchite</ets>. See <er>Conch</er>.]</ety> <fld>(Paleon.)</fld> <def>A fossil or petrified conch or shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchitic</ent><br/
<hw>Con*chit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Composed of shells; containing many shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchoid</ent><br/
<hw>Con"choid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ shell + <grk>e'i^dos</grk> form: cf. F. <ets>concho<ium/de</ets>.]</ety> <fld>(Geom.)</fld> <def>A curve, of the fourth degree, first made use of by the Greek geometer, Nicomedes, who invented it for the purpose of trisecting an angle and duplicating the cube.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchoidal</ent><br/
<hw>Con*choid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>concho<ium/dal</ets>.]</ety> <fld>(Min.)</fld> <def>Having elevations or depressions in form like one half of a bivalve shell; -- applied principally to a surface produced by fracture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchological</ent><br/
<hw>Con`cho*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to, or connected with, conchology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchologist</ent><br/
<hw>Con*chol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One who studies, or is versed in, conchology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchology</ent><br/
<hw>Con*chol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Conch</ets> + <ets>-logy</ets>.]</ety> <fld>(Zool.)</fld> <def>The science of Mollusca, and of the shells which they form; malacology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchometer</ent><br/
<hw>Con*chom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Conch</ets> + <ets>-meter</ets>.]</ety> <fld>(Zool.)</fld> <def>An instrument for measuring shells, or the angle of their spire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchometry</ent><br/
<hw>Con*chom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The art of measuring shells or their curves; conchyliometry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concho-spiral</ent><br/
<hw>Con`cho-spi"ral</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of spiral curve found in certain univalve shells.</def>  <rj><au>Agassiz.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchyliaceous</ent><br/
<ent>Conchylaceous</ent><br/
<mhw>{ <hw>Con`chy*la"ceous</hw> <pr>(?)</pr>, <hw>Con*chyl`i*a"ceous</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <ety>[L. <ets>conchylium</ets> shell, Gr. <?/, dim. of <?/, equiv. to <?/. See <er>Conch</er>.]</ety> <def>Of or pertaining to shells; resembling a shell; <as>as, <ex>conchyliaceous</ex> impressions</as>.</def>  <rj><au>Kirwan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchyliology</ent><br/
<ent>Conchyliologist</ent><br/
<mhw>{ <hw>Con*chyl`i*ol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Con*chyl`i*ol"o*gy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Conchologist</er>, and <er>Conchology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchyliometry</ent><br/
<hw>Con*chyl`i*om"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ + <ets>-metry</ets>.]</ety> <def>Same as <er>Conchometry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conchylious</ent><br/
<hw>Con*chyl"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Conchylaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciator</ent><br/
<hw>Con"ci*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>conciatore</ets>, fr. <ets>conciare</ets> to adjust, dress, fr. L. <ets>comtus</ets>, p. p. See <er>Compt</er>, <pos>a.</pos>]</ety> <fld>(Glass Works)</fld> <def>The person who weighs and proportions the materials to be made into glass, and who works and tempers them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concierge</ent><br/
||<hw>Con`cierge"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>One who keeps the entrance to an edifice, public or private; a doorkeeper; a janitor, male or female.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciergerie</ent><br/
||<hw>Con`cier`ge*rie"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>The office or lodge of a concierge or janitor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A celebrated prison, attached to the Palais de Justice in Paris.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Conciliable</ent><br/
<hw>Con*cil"i*a*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conciliabulum</ets>, fr. <ets>concitium</ets> assembly: cf. F. <ets>conciliabule</ets>. See <er>Council</er>.]</ety> <def>A small or private assembly, especially of an ecclesiastical nature.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliable</ent><br/
<hw>Con*cil"i*a*ble</hw>, <pos>a.</pos> <ety>[Cf. F. <ets>conciliable</ets>.]</ety> <def>Capable of being conciliated or reconciled.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliabule</ent><br/
<hw>Con*cil"i*a*bule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Conciliable</er>, <pos>n.</pos>]</ety> <def>An obscure ecclesiastical council; a conciliable.</def>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliary</ent><br/
<ent>Conciliar</ent><br/
<mhw>{ <hw>Con*cil"i*ar</hw> <pr>(?)</pr>, <hw>Con*cil"i*a*ry</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>conciliare</ets>.]</ety> <def>Of or pertaining to, or issued by, a council.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliate</ent><br/
<hw>Con*cil"i*ate</hw> <pr>(?; 106)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conciliated</conjf>; <pos>p. pr & vb. n.</pos> <conjf>Conciliating</conjf>.]</vmorph> <ety>[L. <ets>conciliatus</ets>, p. p. of <ets>conciliare</ets> to draw or bring together, unite, from <ets>concilium</ets> council. See <er>Council</er>.]</ety> <def>To win ower; to gain from a state of hostility; to gain the good will or favor of; to make friendly; to mollify; to propitiate; to appease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rapacity of his father's administration had excited such universal discontent, that it was found expedient to <qex>conciliate</qex> the nation.</q> <rj><qau>Hallam.</qau></rj></p>

<p><syn><b>Syn.</b> -- To reconcile; propitiate; appease; pacify.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliation</ent><br/
<hw>Con*cil`i*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conciliatio</ets>.]</ety> <def>The act or process of conciliating; the state of being conciliated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The house has gone further; it has declared <qex>conciliation</qex> admissible previous to any submission on the part of America.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliative</ent><br/
<hw>Con*cil"i*a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Conciliatory.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliator</ent><br/
<hw>Con*cil"i*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who conciliates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciliatory</ent><br/
<hw>Con*cil"i*a*to*ry</hw> <pr>(?; 106)</pr>, <pos>a.</pos> <def>Tending to conciliate; pacific; mollifying; propitiating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The only alternative, therefore, was to have recourse to the <qex>conciliatory</qex> policy.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concinnate</ent><br/
<hw>Con*cin"nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>concinnatus</ets>, p. p. of <ets>concinnare</ets> to concinnate. See <er>Concinnity</er>.]</ety> <def>To place fitly together; to adapt; to clear.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concinnity</ent><br/
<hw>Con*cin"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concinnitas</ets>, fr. <ets>concinnus</ets> skillfully put together, beautiful. Of uncertain origin.]</ety> <def>Internal harmony or fitness; mutual adaptation of parts; elegance; -- used chiefly of style of discourse.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>An exact <qex>concinnity</qex> and eveness of fancy.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concinnous</ent><br/
<hw>Con*cin"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concinnus</ets>.]</ety> <def>Characterized by concinnity; neat; elegant.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The most <qex>concinnous</qex> and most rotund of proffessors, M. Heyne.</q> <rj><qau>De Quiency.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concionate</ent><br/
<hw>Con"cio*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>concionatus</ets>, p. p. of <ets>concionari</ets> to adress.]</ety> <def>To preach.</def> <mark>[Obs.]</mark>  <rj><au>Lithgow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concionator</ent><br/
<hw>Con"cio*na`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <sn>1.</sn> <def>An haranguer of the people; a preacher.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Old Law)</fld> <def>A common councilman.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concionatory</ent><br/
<hw>Con"cio*na`to*ry</hw> <pr>(?; 106)</pr>, <pos>a.</pos> <def>Of or pertaining to preaching or public addresses.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concise</ent><br/
<hw>Con*cise"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concisus</ets> cut off, short, p. p. of <ets>concidere</ets> to cut to pieces; <ets>con-</ets> + <ets>caedere</ets> to cut; perh. akin to <ets>scindere</ets> to cleave, and to E. <ets>shed</ets>, v. t.; cf. F. <ets>concis</ets>.]</ety> <def>Expressing much in a few words; condensed; brief and compacted; -- used of style in writing or speaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>concise</qex> style, which expresseth not enough, but leaves somewhat to be understood.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where the author is . . . too brief and <qex>concise</qex>, amplify a little.</q> <rj><qau>I. Watts.</qau></rj></p>

<p><syn><b>Syn.</b> -- Laconic; terse; brief; short; compendious; summary; succinct. See <er>Laconic</er>, and <er>Terse</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concisely</ent><br/
<hw>Con*cise"ly</hw>, <pos>adv.</pos> <def>In a concise manner; briefly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conciseness</ent><br/
<hw>Con*cise"ness</hw>, <pos>n.</pos> <def>The quality of being concise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concision</ent><br/
<hw>Con*ci"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concisio</ets>: cf. F. <ets>concision</ets>. See <er>Concise</er>.]</ety> <def>A cutting off; a division; a schism; a faction.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concitation</ent><br/
<hw>Con`ci*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concitatio</ets>. See <er>Concite</er>.]</ety> <def>The act of stirring up, exciting, or agitating.</def> <mark>[Obs.]</mark> <ldquo/The <xex>concitation</xex> of humors.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concite</ent><br/
<hw>Con*cite"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>concitare</ets>; <ets>con-</ets> + <ets>citare</ets>. See Cite.]</ety> <def>To excite or stir up.</def> <mark>[Obs.]</mark>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 296 --></p>

<p><ent>Conclamation</ent><br/
<hw>Con`cla*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conclamatio</ets>.]</ety> <def>An outcry or shout of many together.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Before his funeral <qex>conclamation</qex>.</q> <rj><qau>May (Lucan).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclave</ent><br/
<hw>Con"clave</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>conclave</ets> a room that may locked up; <ets>con-</ets> + <ets>clavis</ets> key. See <er>Clavicle</er>.]</ety> <sn>1.</sn> <def>The set of apartments within which the cardinals of the Roman Catholic Church are continuously secluded while engaged in choosing a pope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The body of cardinals shut up in the conclave for the election of a pope; hence, the body of cardinals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was said a cardinal, by reason of his apparent likelihood to step into St. Peter's chair, that in two <qex>conclaves</qex> he went in pope and came out again cardinal.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A private meeting; a close or secret assembly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The verdicts pronounced by this <qex>conclave</qex> (Johnson's Club) on new books, were speedily known over all London.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be in conclave</b></col>, <cd>to be engaged in a secret meeting; -- said of several, or a considerable number of, persons.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclavist</ent><br/
<hw>Con"cla`vist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>conclaviste</ets>, It. <ets>conclavista</ets>.]</ety> <def>One of the two ecclesiastics allowed to attend a cardinal in the conclave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclude</ent><br/
<hw>Con*clude"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concluded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Concluding</conjf>.]</vmorph> <ety>[L. <ets>concludere</ets>, <ets>conclusum</ets>; <ets>con-</ets> + <ets>claudere</ets> to shut. See <er>Close</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To shut up; to inclose.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The very person of Christ [was] <qex>concluded</qex> within the grave.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To include; to comprehend; to shut up together; to embrace.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For God hath <qex>concluded</qex> all in unbelief.</q> <rj><qau>Rom. xi. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Scripture hath <qex>concluded</qex> all under sin.</q> <rj><qau>Gal. iii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To reach as an end of reasoning; to infer, as from premises; to close, as an argument, by inferring; -- sometimes followed by a dependent clause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No man can <qex>conclude</qex> God's love or hatred to any person by anything that befalls him.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore we <qex>conclude</qex> that a man is justified by faith.</q> <rj><qau>Rom. iii. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make a final determination or judgment concerning; to judge; to decide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But no frail man, however great or high,<br/
Can be <qex>concluded</qex> blest before he die.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Is it <qex>concluded</qex> he shall be protector?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To bring to an end; to close; to finish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>conclude</qex> this part with the speech of a counselor of state.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To bring about as a result; to effect; to make; <as>as, to <ex>conclude</ex> a bargain</as>.</def> <ldquo/If we <xex>conclude</xex> a peace.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To shut off; to restrain; to limit; to estop; to bar; -- generally in the passive; <as>as, the defendant is <ex>concluded</ex> by his own plea; a judgment <ex>concludes</ex> the introduction of further evidence argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If therefore they will appeal to revelation for their creation they must be <qex>concluded</qex> by it.</q> <rj><qau>Sir M. Hale.</qau></rj></p>

<p><syn><b>Syn.</b> -- To infer; decide; determine; settle; close; finish; terminate; end.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclude</ent><br/
<hw>Con*clude"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To come to a termination; to make an end; to close; to end; to terminate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A train of lies,<br/
That, made in lust, <qex>conclude</qex> in perjuries.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And, to <qex>conclude</qex>,<br/
The victory fell on us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form a final judgment; to reach a decision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Can we <qex>conclude</qex> upon Luther's instability?</q> <rj><qau>Bp. Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conclude</qex> and be agreed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concludency</ent><br/
<hw>Con*clud"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Deduction from premises; inference; conclusion.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concludent</ent><br/
<hw>Con*clud"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concludens</ets>, p. pr.]</ety> <def>Bringing to a close; decisive; conclusive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Arguments highly consequential and <qex>concludent</qex> to my purpose.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concluder</ent><br/
<hw>Con*clud"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who concludes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concludingly</ent><br/
<hw>Con*clud"ing*ly</hw>, <pos>adv.</pos> <def>Conclusively.</def> <mark>[R.]</mark>  <rj><au>Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclusible</ent><br/
<hw>Con*clu"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Demonstrable; determinable.</def> <mark>[Obs.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclusion</ent><br/
<hw>Con*clu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>conclusio</ets>. See <er>Conclude</er>.]</ety> <sn>1.</sn> <def>The last part of anything; close; termination; end.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fluorish of trumpets announced the <qex>conclusion</qex> of the contest.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Final decision; determination; result.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the <qex>conclusion</qex> is, she shall be thine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any inference or result of reasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Logic)</fld> <def>The inferred proposition of a syllogism; the necessary consequence of the conditions asserted in two related propositions called premises. See <er>Syllogism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He granted him both the major and minor, but denied him the <qex>conclusion</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Drawing of inferences.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Your wife Octavia, with her modest eyes<br/
And still <qex>conclusion</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An experiment, or something from which a conclusion may be drawn.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We practice likewise all <qex>conclusions</qex> of grafting and inoculating.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>The end or close of a pleading, <it>e.g.</it>, the formal ending of an indictment, <ldquo/against the peace,<rdquo/ etc.</def> <sd>(b)</sd> <def>An estoppel or bar by which a person is held to a particular position.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conclusion to the country</b></col> <fld>(Law)</fld>, <cd>the conclusion of a pleading by which a party <ldquo/puts himself upon the country,<rdquo/ <it>i.e.</it>, appeals to the verdict of a jury.</cd> <au>Mozley & W.</au> -- <col><b>In conclusion</b></col>. <sd>(a)</sd> <cd>Finally.</cd> <sd>(b)</sd> <cd>In short.</cd> -- <col><b>To try conclusions</b></col>, <cd>to make a trial or an experiment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Like the famous ape,<br/
<qex>To try conclusions</qex>, in the basket creep.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Inference; deduction; result; consequence; end; decision. See <er>Inference</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclusive</ent><br/
<hw>Con*clu"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>conclusif</ets>.]</ety> <def>Belonging to a close or termination; decisive; convincing; putting an end to debate or question; leading to, or involving, a conclusion or decision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Secret reasons . . . equally <qex>conclusive</qex> for us as they were for them.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conclusive evidence</b></col> <fld>(Law)</fld>, <cd>that of which, from its nature, the law allows no contradiction or explanation.</cd> -- <col><b>Conclusive presumption</b></col> <fld>(Law)</fld>, <cd>an inference which the law makes so peremptorily that it will not allow it to be overthrown by any contrary proof, however strong.</cd></cs></p>

<p><syn><b>Syn.</b> -- Final; ultimate; unanswerable. See <er>Final</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclusively</ent><br/
<hw>Con*clu"sive*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In the way of conclusion; decisively; positively.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclusiveness</ent><br/
<hw>Con*clu"sive*ness</hw>, <pos>n.</pos> <def>The quality of being conclusive; decisiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conclusory</ent><br/
<hw>Con*clu"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Conclusive.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concoct</ent><br/
<hw>Con*coct"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concocted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Concocting</conjf>.]</vmorph> <ety>[L. <ets>concoctus</ets>, p. p. of <ets>concoquere</ets> to cook together, to digest, mature; <ets>con-</ets> + <ets>coquere</ets> to cook. See <er>Cook</er>.]</ety> <sn>1.</sn> <def>To digest; to convert into nourishment by the organs of nutrition.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Food is <qex>concocted</qex>, the heart beats, the blood circulates.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To purify or refine chemically.</def> <mark>[Obs.]</mark>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To prepare from crude materials, as food; to invent or prepare by combining different ingredients; <as>as, to <ex>concoct</ex> a new dish or beverage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To digest in the mind; to devise; to make up; to contrive; to plan; to plot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was a man of a feeble stomach, unable to <qex>concoct</qex> any great fortune.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To mature or perfect; to ripen.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concocter</ent><br/
<hw>Con*coct"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who concocts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concoction</ent><br/
<hw>Con*coc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concoctio</ets>.]</ety> <sn>1.</sn> <def>A change in food produced by the organs of nutrition; digestion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of concocting or preparing by combining different ingredients; also, the food or compound thus prepared.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of digesting in the mind; planning or devising; rumination.</def>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>Abatement of a morbid process, as a fever and return to a normal condition.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The act of perfecting or maturing.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concoctive</ent><br/
<hw>Con*coct"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the power of digesting or ripening; digestive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hence the <qex>concoctive</qex> powers, with various art,<br/
Subdue the cruder aliments to chyle.</q> <rj><qau>J. Armstrong.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concolor</ent><br/
<hw>Con"col`or</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concolor</ets>; <ets>con-</ets> + <ets>color</ets> color.]</ety> <def>Of the same color; of uniform color.</def> <mark>[R.]</mark> <ldquo/<xex>Concolor</xex> animals.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concolorous</ent><br/
<hw>Con"col`or*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of the same color throughout.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concomitancy</ent><br/
<ent>Concomitance</ent><br/
<mhw>{ <hw>Con*com"i*tance</hw> <pr>(?)</pr>, <hw>Con*com"i*tan*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. F. <ets>concomitance</ets>, fr. LL. <ets>concomitantia</ets>.]</ety> <sn>1.</sn> <def>The state of accompanying; accompaniment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The secondary action subsisteth not alone, but in <qex>concomitancy</qex> with the other.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R.C.Ch.)</fld> <def>The doctrine of the existence of the entire body of Christ in the eucharist, under each element, so that the body and blood are both received by communicating in one kind only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concomitant</ent><br/
<hw>Con*com"i*tant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. L.  <ets>con-</ets> + <ets>comitari</ets> to accompany, <ets>comes</ets> companion. See <er>Count</er> a nobleman.]</ety> <def>Accompanying; conjoined; attending.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It has pleased our wise Creator to annex to several objects, as also to several of our thoughts, a <qex>concomitant</qex> pleasure.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concomitant</ent><br/
<hw>Con*com"i*tant</hw>, <pos>n.</pos> <def>One who, or that which, accompanies, or is collaterally connected with another; a companion; an associate; an accompaniment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Reproach is a <qex>concomitant</qex> to greatness.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The other <qex>concomitant</qex> of ingratitude is hardheartedness.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concomitantly</ent><br/
<hw>Con*com"i*tant*ly</hw>, <pos>adv.</pos> <def>In company with others; unitedly; concurrently.</def>  <rj><au>Bp. pearson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concord</ent><br/
<hw>Con"cord</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>concorde</ets>, L. <ets>concordia</ets>, fr. <ets>concors</ets> of the same mind, agreeing; <ets>con-</ets> + <ets>cor</ets>, <ets>cordis</ets>, heart. See <er>Heart</er>, and cf. <er>Accord</er>.]</ety> <sn>1.</sn> <def>A state of agreement; harmony; union.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love quarrels oft in pleasing <qex>concord</qex> end.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Agreement by stipulation; compact; covenant; treaty or league.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>concord</qex> made between Henry and Roderick.</q> <rj><qau>Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>Agreement of words with one another, in gender, number, person, or case.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Old Law)</fld> <def>An agreement between the parties to a fine of land in reference to the manner in which it should pass, being an acknowledgment that the land in question belonged to the complainant. See <er>Fine</er>.</def>  <rj><au>Burril.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Prob. influenced by <ets>chord</ets>.]</ety> <fld>(Mus.)</fld> <def>An agreeable combination of tones simultaneously heard; a consonant chord; consonance; harmony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concord</ent><br/
<hw>Con"cord</hw>, <pos>n.</pos> <def>A variety of American grape, with large dark blue (almost black) grapes in compact clusters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concord</ent><br/
<hw>Con*cord"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[F. <ets>concorder</ets>, L. <ets>concordare</ets>.]</ety> <def>To agree; to act together.</def> <mark>[Obs.]</mark>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concordable</ent><br/
<hw>Con*cord"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concordabilis</ets>.]</ety> <def>Capable of according; agreeing; harmonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concordance</ent><br/
<hw>Con*cord"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>concordantia</ets>.]</ety> <sn>1.</sn> <def>Agreement; accordance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Contrasts, and yet <qex>concordances</qex>.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>Concord; agreement.</def> <mark>[Obs.]</mark>  <rj><au>Aschlam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An alphabetical verbal index showing the places in the text of a book where each principal word may be found, with its immediate context in each place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His knowledge of the Bible was such, that he might have been called a living <qex>concordance</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A topical index or orderly analysis of the contents of a book.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concordancy</ent><br/
<hw>Con*cord"an*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Agreement.</def>  <rj><au>W. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concordant</ent><br/
<hw>Con*cord"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concordans</ets>, p. pr. of <ets>concordare</ets>: cf. F. <ets>concordant</ets>. See <er>Concord</er>.]</ety> <def>Agreeing; correspondent; harmonious; consonant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Were every one employed in points <qex>concordant</qex> to their natures, professions, and arts, commonwealths would rise up of themselves.</q> <rj><qau>Sir T. Browne</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concordantly</ent><br/
<hw>Con*cord"ant*ly</hw>, <pos>adv.</pos> <def>In a concordant manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concordat</ent><br/
<hw>Con*cor"dat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>concordat</ets>, L. <ets>concordato</ets>, prop. p. p. of <ets>concordare</ets>. See <er>Concord</er>.]</ety> <sn>1.</sn> <def>A compact, covenant, or agreement concerning anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An agreement made between the pope and a sovereign or government for the regulation of ecclesiastical matters with which both are concerned; <as>as, the <ex>concordat</ex> between Pope Pius VII and Bonaparte in 1801</as>.</def>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concord buggy</ent><br/
<hw>Con"cord bug"gy</hw> <pr>(k<ocr/<nsm/"k<etil/rd)</pr>. <ety>[From <etsep>Concord</etsep>, New Hampshire, where first made.]</ety> <def>A kind of buggy having a body with low sides, and side springs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Concordist</ent><br/
<hw>Con*cord"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The compiler of a concordance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concorporate</ent><br/
<hw>Con*cor"po*rate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[L. <ets>concorporatus</ets>, p. p. of <ets>concorporare</ets>.]</ety> <def>To unite in one mass or body; to incorporate.</def> <mark>[Archaic.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concorporate</ent><br/
<hw>Con*cor"po*rate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>United in one body; incorporated.</def> <mark>[Archaic]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concorporation</ent><br/
<hw>Con*cor`po*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concorporatio</ets>.]</ety> <def>Union of things in one mass or body.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concourse</ent><br/
<hw>Con"course</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>concours</ets>, L. <ets>concursus</ets>, fr. <ets>concurrere</ets> to run together. See <er>Concur</er>.]</ety> <sn>1.</sn> <def>A moving, flowing, or running together; confluence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The good frame of the universe was not the product of chance or fortuitous <qex>concourse</qex> of particles of matter.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assembly; a gathering formed by a voluntary or spontaneous moving and meeting in one place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Amidst the <qex>concourse</qex> were to be seen the noble ladies of Milan, in gay, fantastic cars, shining in silk brocade.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The place or point of meeting or junction of two bodies.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The drop will begin to move toward the <qex>concourse</qex> of the glasses.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An open space where several roads or paths meet; esp. an open space in a park where several roads meet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Concurrence; cooperation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The divine providence is wont to afford its <qex>concourse</qex> to such proceeding.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concreate</ent><br/
<hw>Con`cre*ate"</hw> <pr>(? <or/ ?)</pr>, <pos>v. t.</pos> <def>To create at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If God did <qex>concreate</qex> grace with Adam.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concremation</ent><br/
<hw>Con`cre*ma"tion</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[L. <ets>concrematio</ets>, fr. <ets>concremare</ets>. See <er>Cremate</er>.]</ety> <def>The act of burning different things together.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrement</ent><br/
<hw>Con"cre*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concrementum</ets>, fr. <ets>concrescere</ets>. See <er>Concrete</er>.]</ety> <def>A growing together; the collection or mass formed by concretion, or natural union.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>concrement</qex> of a pebble or flint.</q> <rj><qau>Sir M. Hale</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrescence</ent><br/
<hw>Con*cres"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concrescentia</ets>.]</ety> <def>Coalescence of particles; growth; increase by the addition of particles.</def> <mark>[R.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrescible</ent><br/
<hw>Con*cres"ci*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>Capable of being changed from a liquid to a solid state.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They formed a . . . fixed <qex>concrescible</qex> oil.</q> <rj><qau>Fourcroy (Trans. ).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrescive</ent><br/
<hw>Con*cres"cive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Growing together, or into union; uniting.</def> <mark>[R.]</mark>  <rj><au>Eclec. Rev.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrete</ent><br/
<hw>Con"crete</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[L. <ets>concretus</ets>, p. p. of <ets>concrescere</ets> to grow together; <ets>con-</ets> + <ets>crescere</ets> to grow; cf. F. <ets>concret</ets>. See <er>Crescent</er>.]</ety> <sn>1.</sn> <def>United in growth; hence, formed by coalition of separate particles into one mass; united in a solid form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first <qex>concrete</qex> state, or consistent surface, of the chaos must be of the same figure as the last liquid state.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <sd>(a)</sd> <def>Standing for an object as it exists in nature, invested with all its qualities, as distinguished from standing for an attribute of an object; -- opposed to <ant>abstract</ant>.</def> <specif>Hence:</specif> <sd>(b)</sd> <def>Applied to a specific object; special; particular; -- opposed to <ant>general</ant>. See <er>Abstract</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Concrete</qex> is opposed to abstract. The names of individuals are <qex>concrete</qex>, those of classes abstract.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Concrete</qex> terms, while they express the quality, do also express, or imply, or refer to, some subject to which it belongs.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Concrete number</b></col>, <cd>a number associated with, or applied to, a particular object, as <xex>three</xex> men, <xex>five</xex> days, etc., as distinguished from an <xex>abstract</xex> number, or one used without reference to a particular object.</cd> -- <col><b>Concrete quantity</b></col>, <cd>a physical object or a collection of such objects.</cd> <au>Davies & Peck.</au> -- <col><b>Concrete science</b></col>, <cd>a physical science, one having as its subject of knowledge concrete things instead of abstract laws.</cd> -- <col><b>Concrete sound <it>or</it> movement of the voice</b></col>, <cd>one which slides continuously up or down, as distinguished from a <contr>discrete</contr> movement, in which the voice leaps at once from one line of pitch to another.</cd>  <rj><au>Rush.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrete</ent><br/
<hw>Con"crete</hw>, <pos>n.</pos> <sn>1.</sn> <def>A compound or mass formed by concretion, spontaneous union, or coalescence of separate particles of matter in one body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To divide all <qex>concretes</qex>, minerals and others, into the same number of distinct substances.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mixture of gravel, pebbles, or broken stone with cement or with tar, etc., used for sidewalks, roadways, foundations, etc., and esp.  for submarine structures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Logic)</fld> <def>A term designating both a quality and the subject in which it exists; a concrete term.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>concretes</qex> <ldquo/father<rdquo/ and <ldquo/son<rdquo/ have, or might have, the abstracts <ldquo/paternity<rdquo/ and <ldquo/filiety<rdquo/.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Sugar Making)</fld> <def>Sugar boiled down from cane juice to a solid mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrete</ent><br/
<hw>Con*crete"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concreted</conjf>; <pos>p. pr & vb. n.</pos> <conjf>Concreting</conjf>.]</vmorph> <def>To unite or coalesce, as separate particles, into a mass or solid body.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Applied to some substances, it is equivalent to <xex>indurate</xex>; as, metallic matter <xex>concretes</xex> into a hard body; applied to others, it is equivalent to <xex>congeal</xex>, <xex>thicken</xex>, <xex>inspissate</xex>, <xex>coagulate</xex>, as in the concretion of blood. <ldquo/The blood of some who died of the plague could not be made to <xex>concrete</xex>.<rdquo/  <rj><au>Arbuthnot.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrete</ent><br/
<hw>Con*crete"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To form into a mass, as by the cohesion or coalescence of separate particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There are in our inferior world divers bodies that are <qex>concreted</qex> out of others.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover with, or form of, concrete, as a pavement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concretely</ent><br/
<hw>Con*crete"ly</hw>, <pos>adv.</pos> <def>In a concrete manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concreteness</ent><br/
<hw>Con*crete"ness</hw>, <pos>n.</pos> <def>The quality of being concrete.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concretion</ent><br/
<hw>Con*cre"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concretio</ets>.]</ety> <sn>1.</sn> <def>The process of concreting; the process of uniting or of becoming united, as particles of matter into a mass; solidification.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 297 --></p>

<p><sn>2.</sn> <def>A mass or nodule of solid matter formed by growing together, by congelation, condensation, coagulation, induration, etc.; a clot; a lump; a calculus.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Accidental ossifications or deposits of phosphates of lime in certain organs . . . are called osseous <qex>concretions</qex>.</q> <rj><qau>Dunglison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>A rounded mass or nodule produced by an aggregation of the material around a center; <as>as, the calcareous <ex>concretions</ex> common in beds of clay</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concretional</ent><br/
<hw>Con*cre"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Concretionary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concretionary</ent><br/
<hw>Con*cre"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or formed by, concretion or aggregation; producing or containing concretions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concretive</ent><br/
<hw>Con*cre"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Promoting concretion.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concretively</ent><br/
<hw>Con*cre"tive*ly</hw>, <pos>adv.</pos> <def>In a concrete manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concreture</ent><br/
<hw>Con*cre"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>A mass formed by concretion.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrew</ent><br/
<hw>Con*crew"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Concrete</er>, <pos>a.</pos>, and <er>Accrue</er>.]</ety> <def>To grow together.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concrimination</ent><br/
<hw>Con*crim`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint accusation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinacy</ent><br/
<hw>Con*cu"bi*na*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practice of concubinage.</def> <mark>[Obs.]</mark>  <rj><au>Strype.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinage</ent><br/
<hw>Con*cu"bi*nage</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The cohabiting of a man and a woman who are not legally married; the state of being a concubine.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some countries, <xex>concubinage</xex> is marriage of an inferior kind, or performed with less solemnity than a true or formal marriage; or marriage with a woman of inferior condition, to whom the husband does not convey his rank or quality. Under Roman law, it was the living of a man and woman in sexual relations without marriage, but in conformity with local law.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>A plea, in which it is alleged that the woman suing for dower was not lawfully married to the man in whose lands she seeks to be endowed, but that she was his concubine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinal</ent><br/
<hw>Con*cu"bi*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concubinalis</ets>.]</ety> <def>Of or pertaining to concubinage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinarian</ent><br/
<hw>Con*cu`bi*na"ri*an</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Concubinary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The married and <qex>concubinarian</qex>, as well as looser clergy.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinary</ent><br/
<hw>Con*cu"bi*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>concubinarius</ets>.]</ety> <def>Relating to concubinage; living in concubinage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinary</ent><br/
<hw>Con*cu"bi*na*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Concubinaries</plw> <pr>(#)</pr>.</plu> <def>One who lives in concubinage.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubinate</ent><br/
<hw>Con*cu"bi*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concubinatus</ets>.]</ety> <def>Concubinage.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concubine</ent><br/
<hw>Con"cu*bine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>concubina</ets>; <ets>con-</ets> + <ets>cubare</ets> to lie down, <ets>concumbere</ets> to lie together, akin to E. <ets>cubit</ets>.]</ety> <sn>1.</sn> <def>A woman who cohabits with a man without being his wife; a paramour.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Concubine</xex> has been sometimes, but rarely, used of a male paramour as well as of a female.</note>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wife of inferior condition; a lawful wife, but not united to the man by the usual ceremonies, and of inferior condition. Such were Hagar and Keturah, the concubines of Abraham; and such concubines were allowed by the Roman laws. Their children were not heirs of their father.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conculcate</ent><br/
<hw>Con*cul"cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concultated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conculcating</conjf>.]</vmorph> <ety>[L. <ets>conculcatus</ets>, p. p. of <ets>conculcare</ets> to conculcate fr. <ets>calx</ets> heel.]</ety> <def>To tread or trample under foot.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Montagu</au></rj></p>

<p>-- <wordforms><wf>Con`cul*ca"tion</wf> <pr>(<?/)</pr>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupiscence</ent><br/
<hw>Con*cu"pis*cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>concupiscentia</ets>.]</ety> <def>Sexual lust; morbid carnal passion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Concupiscence</qex> like a pestilence walketh in darkness.</q> <rj><qau>Horne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupiscent</ent><br/
<hw>Con*cu"pis*cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>concupiscens</ets>, p. pr. of <ets>concupiscere</ets>, v. incho. of <ets>concupere</ets> to long for; <ets>con-</ets> + <ets>cupere</ets>. See <er>Covet</er>.]</ety> <def>Having sexual lust; libidinous; lustful; lecherous; salacious.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupiscential</ent><br/
<hw>Con*cu`pis*cen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to concupiscence.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupiscentious</ent><br/
<hw>Con*cu`pis*cen"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Concupiscent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupiscible</ent><br/
<hw>Con*cu`pis*ci*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>concupiscible</ets>.]</ety> <sn>1.</sn> <def>Exciting to, or liable to be affected by, concupiscence; provoking lustful desires.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exciting desire, good or evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The schools reduce all the passions to these two heads, the <qex>concupiscible</qex> and irascible appetite.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupiscibleness</ent><br/
<hw>Con*cu"pis*ci*ble*ness</hw>, <pos>n.</pos> <def>The state of being concupiscible.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concupy</ent><br/
<hw>Con"cu*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Concupiscence. <note>[Used only in <ldquo/Troilus and Cressida<rdquo/]</note></def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concur</ent><br/
<hw>Con*cur"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Concurred</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Concurring</conjf>.]</vmorph> <ety>[L. <ets>concurrere</ets> to run together, agree; <ets>con-</ets> + <ets>currere</ets> to run. See <er>Current</er>.]</ety> <sn>1.</sn> <def>To run together; to meet.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Anon they fierce encountering both <qex>concurred</qex><br/
With grisly looks and faces like their fates.</q> <rj><qau>J. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To meet in the same point; to combine or conjoin; to contribute or help toward a common object or effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When outward causes <qex>concur</qex>.</q> <rj><qau>Jer. Colier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite or agree (in action or opinion); to join; to act jointly; to agree; to coincide; to correspond.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mr. Burke <qex>concurred</qex> with Lord Chatham in opinion.</q> <rj><qau>Fox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Tories and Whigs had <qex>concurred</qex> in paying honor to Walker.</q> <rj><qau>Makaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>concurs</qex> directly with the letter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To assent; to consent.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj></p>

<p><syn><b>Syn.</b> -- To agree; unite; combine; conspire; coincide; approve; acquiesce; assent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurrence</ent><br/
<hw>Con*cur"rence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., competition, equality of rights, fr. LL. <ets>concurrentia</ets> competition.]</ety> <sn>1.</sn> <def>The act of concurring; a meeting or coming together; union; conjunction; combination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have no other measure but our own ideas, with the <qex>concurence</qex> of other probable reasons, to persuade us.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A meeting of minds; agreement in opinion; union in design or act; -- implying joint approbation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tarquin the Proud was expelled by the universal <qex>concurrence</qex> of nobles and people.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Agreement or consent, implying aid or contribution of power or influence; cooperation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We collect the greatness of the work, and the necessity of the divine <qex>concurrence</qex> to it.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An instinct that works us to its own purposes without our <qex>concurrence</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A common right; coincidence of equal powers; as, a <qex>concurrence</qex> of jurisdiction in two different courts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurrency</ent><br/
<hw>Con*cur"ren*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Concurrence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurrent</ent><br/
<hw>Con*cur"rent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>concurrent</ets>, L. <ets>concurrens</ets>, p. pr. of <ets>concurrere</ets>.]</ety> <sn>1.</sn> <def>Acting in conjunction; agreeing in the same act or opinion; contributing to the same event or effect; cooperating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I join with these laws the personal presence of the kings' son, as a <qex>concurrent</qex> cause of this reformation.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>concurrent</qex> testimony of antiquity.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conjoined; associate; concomitant; existing or happening at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no difference the <qex>concurrent</qex> echo and the iterant but the quickness or slowness of the return.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Changes . . . <qex>concurrent</qex> with the visual changes in the eye.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Joint and equal in authority; taking cognizance of similar questions; operating on the same objects; <as>as, the <ex>concurrent</ex> jurisdiction of courts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Geom.)</fld> <def>Meeting in one point.</def></p>

<p><syn><b>Syn.</b> -- Meeting; uniting; accompanying; conjoined; associated; coincident; united.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurrent</ent><br/
<hw>Con*cur"rent</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, concurs; a joint or contributory cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To all affairs of importance there are three necessary <qex>concurrents</qex> . . . time, industry, and faculties.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One pursuing the same course, or seeking the same objects; hence, a rival; an opponent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Menander . . . had no <qex>concurrent</qex> in his time that came near unto him.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chron.)</fld> <def>One of the supernumerary days of the year over fifty-two complete weeks; -- so called because they <xex>concur</xex> with the solar cycle, the course of which they follow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurrently</ent><br/
<hw>Con*cur"rent*ly</hw>, <pos>adv.</pos> <def>With concurrence; unitedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurrentness</ent><br/
<hw>Con*cur"rent*ness</hw>, <pos>n.</pos> <def>The state or quality of being concurrent; concurrence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concurring</ent><br/
<hw>Con*cur"ring</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Agreeing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Concurring figure</b></col> <fld>(Geom.)</fld>, <cd>one which, being laid on another, exactly meets every part of it, or one which corresponds with another in all its parts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concuss</ent><br/
<hw>Con*cuss"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>concussus</ets>, p. p. of <ets>concutere</ets>. See <er>Concussion</er>.]</ety> <sn>1.</sn> <def>To shake or agitate.</def> <ldquo/<xex>Concussed</xex> with uncertainty.<rdquo/  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To force (a person) to do something, or give up something, by intimidation; to coerce.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concussation</ent><br/
<hw>Con`cus*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A violent shock or agitation.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concussion</ent><br/
<hw>Con*cus"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>concussio</ets>, fr. <ets>concutere</ets>, <ets>concussum</ets>, to shake violenty; <ets>con-</ets> + <ets>quatere</ets> to shake. See <er>Cashier</er>, <er>Quash</er>.]</ety> <sn>1.</sn> <def>A shaking or agitation; a shock; caused by the collision of two bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is believed that great ringing of bells, in populous cities, hath dissipated pestilent air; which may be from the <qex>concussion</qex> of the air.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A condition of lowered functional activity, without visible structural change, produced in an organ by a shock, as by fall or blow; <as>as, a <ex>concussion</ex> of the brain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Civil Law)</fld> <def>The unlawful forcing of another by threats of violence to yield up something of value.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then <qex>concussion</qex>, rapine, pilleries,<br/
Their catalogue of accusations fill.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Concussion fuse</b></col> <fld>(Mil.)</fld>, <cd>one that is ignited by the concussion of the shell when it strikes.</cd></cs></p>

<p><syn><b>Syn.</b> -- See <er>Shock</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Concussive</ent><br/
<hw>Con*cus"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the power or quality of shaking or agitating.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cond</ent><br/
<hw>Cond</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>conduen</ets>, <ets>condien</ets>, F. <ets>conduire</ets> to conduct, fr. L. <ets>conducere</ets>. See <er>Conduct</er>, and cf. <er>Con</er> (<ets>Naut</ets>.), <er>Conn</er>. <er>Cun</er>.]</ety> <fld>(Naut.)</fld> <def>To con, as a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condemn</ent><br/
<hw>Con*demn"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Condemned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Condemning</conjf> <pr>(? <or/ <?/)</pr>.]</vmorph> <ety>[L. <ets>condemnare</ets>; <ets>con-</ets> + <ets>damnare</ets> to condemn: cf. F. <ets>condamner</ets>. See <er>Damn</er>.]</ety> <sn>1.</sn> <def>To pronounce to be wrong; to disapprove of; to censure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Condemn</qex> the fault, and not the actor of it!<br/
Why, every fault's <qex>condemned</qex> ere it be done.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wilt thou <qex>condemn</qex> him that is most just?</q> <rj><qau>Job xxxiv. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To declare the guilt of; to make manifest the faults or unworthiness of; to convict of guilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The queen of the south shall rise up in the judgment with this generation, and shall <qex>condemn</qex> it.</q> <rj><qau>Matt. xii. 42.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pronounce a judicial sentence against; to sentence to punishment, suffering, or loss; to doom; -- with <xex>to</xex> before the penalty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Driven out from bliss, <qex>condemned</qex><br/
In this abhorred deep to utter woe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To each his sufferings; all are men,<br/
<qex>Condemned</qex> alike to groan.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And they shall <qex>condemn</qex> him to death.</q> <rj><qau>Matt. xx. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The thief <qex>condemned</qex>, in law already dead.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No flocks that range the valley free,<br/
To slaughter I <qex>condemn</qex>.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To amerce or fine; -- with <xex>in</xex> before the penalty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king of Egypt . . . <qex>condemned</qex> the land in a hundred talents of silver.</q> <rj><qau>2 Cron. xxxvi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To adjudge or pronounce to be unfit for use or service; to adjudge or pronounce to be forfeited; <as>as, the ship and her cargo were <ex>condemned</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Law)</fld> <def>To doom to be taken for public use, under the right of eminent domain.</def></p>

<p><syn><b>Syn.</b> -- To blame; censure; reprove; reproach; upbraid; reprobate; convict; doom; sentence; adjudge.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condemnable</ent><br/
<hw>Con"dem*na"ble</hw> <pr>(?)</pr>, a. <ety>[L. <ets>condemnabilis</ets>.]</ety> <def>Worthy of condemnation; blamable; culpable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condemnation</ent><br/
<hw>Con"dem*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>condemnatio</ets>.]</ety> <sn>1.</sn> <def>The act of condemning or pronouncing to be wrong; censure; blame; disapprobation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In every other sense of <qex>condemnation</qex>, as blame, censure, reproof, private judgment, and the like.</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of judicially condemning, or adjudging guilty, unfit for use, or forfeited; the act of dooming to punishment or forfeiture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A legal and judicial <qex>condemnation</qex>.</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose <qex>condemnation</qex> is pronounced.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of being condemned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His pathetic appeal to posterity in the hopeless hour of <qex>condemnation</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The ground or reason of condemning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the <qex>condemnation</qex>, that light is come into the world, and men loved darkness rather light, because their deeds were evil.</q> <rj><qau>John iii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condemnatory</ent><br/
<hw>Con*dem"na*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Condemning; containing or imposing condemnation or censure; <as>as, a <ex>condemnatory</ex> sentence or decree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condemned</ent><br/
<hw>Con*demned"</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pronounced to be wrong, guilty, worthless, or forfeited; adjudged or sentenced to punishment, destruction, or confiscation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Used for condemned persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Richard Savage . . . had lain with fifty pounds weight of irons on his legs in the <qex>condemned</qex> ward of Newgate.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condemner</ent><br/
<hw>Con*dem"ner</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>One who condemns or censures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensability</ent><br/
<hw>Con*den`sa*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Capability of being condensed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensable</ent><br/
<hw>Con*den"sa*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>condensable</ets>.]</ety> <def>Capable of being condensed; <as>as, vapor is <ex>condensable</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensate</ent><br/
<hw>Con*den"sate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>condensatus</ets>, p. p. of <ets>condensare</ets>. See <er>Condense</er>, <pos>v. t.</pos>]</ety> <def>Made dense; condensed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Water . . . thickened or <qex>condensate</qex>.</q> <rj><qau>Peacham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensate</ent><br/
<hw>Con*den"sate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Condensated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Condensating</conjf>.]</vmorph> <def>To condense.</def> <mark>[R.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensation</ent><br/
<hw>Con`den*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>condensatio</ets>: cf. F. <ets>condensation</ets>.]</ety> <sn>1.</sn> <def>The act or process of condensing or of being condensed; the state of being condensed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Goldsmith] was a great and perhaps an unequaled master of the arts of selection and <qex>condensation</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physics)</fld> <def>The act or process of reducing, by depression of temperature or increase of pressure, etc., to another and denser form, as gas to the condition of a liquid or steam to water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>A rearrangement or concentration of the different constituents of one or more substances into a distinct and definite compound of greater complexity and molecular weight, often resulting in an increase of density, as the <xex>condensation</xex> of oxygen into ozone, or of acetone into mesitylene.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Condensation product</b></col> <fld>(Chem.)</fld>, <cd>a substance obtained by the polymerization of one substance, or by the union of two or more, with or without separation of some unimportant side products.</cd> -- <col><b>Surface condensation</b></col>, <cd>the system of condensing steam by contact with cold metallic surfaces, in distinction from condensation by the injection of cold water.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensative</ent><br/
<hw>Con*den"sa*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>condensatif</ets>.]</ety> <def>Having the property of condensing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condense</ent><br/
<hw>Con*dense"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Condensed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Condensing</conjf>.]</vmorph> <ety>[L. <ets>condensare</ets>; <ets>con-</ets> + <ets>densare</ets> to make thick or dense, <ets>densus</ets> thick, dense: cf. F. <ets>condenser</ets>. See <er>Dense</er>, and cf. <er>Condensate</er>.]</ety> <sn>1.</sn> <def>To make more close, compact, or dense; to compress or concentrate into a smaller compass; to consolidate; to abridge; to epitomize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In what shape they choose,<br/
Dilated or <qex>condensed</qex>, bright or obscure.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The secret course pursued at Brussels and at Madrid may be <qex>condensed</qex> into the usual formula, dissimulation, procrastination, and again dissimulation.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem. & Physics)</fld> <def>To reduce into another and denser form, as by cold or pressure; <as>as, to <ex>condense</ex> gas into a liquid form, or steam into water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Condensed milk</b></col>, <cd>milk reduced to the consistence of very thick cream by evaporation (usually with addition of sugar) for preservation and transportation.</cd> -- <col><b>Condensing engine</b></col>, <cd>a steam engine in which the steam is condensed after having exerted its force on the piston.</cd></cs></p>

<p><syn><b>Syn.</b> -- To compress; contract; crowd; thicken; concentrate; abridge; epitomize; reduce.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condense</ent><br/
<hw>Con*dense"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To become more compact; to be reduced into a denser form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nitrous acid is gaseous at ordinary temperatures, but <qex>condenses</qex> into a very volatile liquid at the zero of Fahrenheit.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>To combine or unite (as two chemical substances) with or without separation of some unimportant side products.</def> <sd>(b)</sd> <def>To undergo polymerization.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condense</ent><br/
<hw>Con*dense"</hw>, <pos>a.</pos> <ety>[L. <ets>condensus</ets>.]</ety> <def>Condensed; compact; dense.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The huge <qex>condense</qex> bodies of planets.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>condensed</ent><br/
<hw>condensed</hw> <pos>adj.</pos> <sn>1.</sn> <fld>(Psychol)</fld> <def>representing two or more ideas or emotions by a single symbol; <as>as, a <ex>condensed</ex> expression of various feelings and ideas</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>shortened by rewriting with fewer words; -- used of texts; <as>as, a <ex>condensed</ex> book</as>.  Opposite of <ant>unabridged</ant>.</def><br/
<syn><b>Syn. --</b> abridged.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>reduced to a stronger or more concentrated form; <as>as, <ex>condensed</ex> milk</as>.  Opposite of <ant>uncondensed</ant>.</def><br/
<syn><b>Syn. --</b> concentrated[5].</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>narrower than usual for a particular height; -- of printers' type.  Opposite of <ant>expanded</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Condenser</ent><br/
<hw>Con*dens"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, condenses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physic)</fld> <sd>(a)</sd> <def>An instrument for condensing air or other elastic fluids, consisting of a cylinder having a movable piston to force the air into a receiver, and a valve to prevent its escape.</def> <sd>(b)</sd> <def>An instrument for concentrating electricity by the effect of induction between conducting plates separated by a nonconducting plate.</def> <sd>(c)</sd> <def>A lens or mirror, usually of short focal distance, used to concentrate light upon an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 298 --></p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>An apparatus for receiving and condensing the volatile products of distillation to a liquid or solid form, by cooling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Steam Engine)</fld> <def>An apparatus, separate from the cylinder, in which the exhaust steam is condensed by the action of cold water or air. See <xex>Illust.</xex> of <er>Steam engine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Achromatic condenser</b></col> <fld>(Optics)</fld>, <cd>an achromatic lens used as a condenser.</cd> -- <mcol><col><b>Bull's-eye condenser</b></col>, <it>or</it> <col><b>Bull's-eye</b></col></mcol> <fld>(Optics)</fld>, <cd>a lens of short focal distance used for concentrating rays of light.</cd> -- <col><b>Injection condenser</b></col>, <cd>a vessel in which steam is condensed by the direct contact of water.</cd> -- <col><b>Surface condenser</b></col>, <cd>an apparatus for condensing steam, especially the exhaust of a steam engine, by bringing it into contact with metallic surface cooled by water or air.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condensible</ent><br/
<hw>Con*den"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being condensed; <as>as, a gas <ex>condensible</ex> to a liquid by cold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conder</ent><br/
<hw>Cond"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Cond</er>.]</ety> <def>One who watches shoals of fish; a balker. See <er>Balker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condescend</ent><br/
<hw>Con`de*scend"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Condescended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Condescending</conjf>.]</vmorph> <ety>[F. <ets>condescendre</ets>, LL. <ets>condescendere</ets>, fr. L. <ets>con-</ets> + <ets>descendere</ets>. See <er>Descend</er>.]</ety> <sn>1.</sn> <def>To stoop or descend; to let one's self down; to submit; to waive the privilege of rank or dignity; to accommodate one's self to an inferior.</def> <ldquo/<xex>Condescend</xex> to men of low estate.<rdquo/  <rj><au>Rom. xii. 16.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Can they think me so broken, so debased<br/
With corporal servitude, that my mind ever<br/
Will <qex>condescend</qex> to such absurd commands?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Spain's mighty monarch,<br/
In gracious clemency, does <qex>condescend</qex>,<br/
On these conditions, to become your friend.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>Often used ironically, implying an assumption of superiority.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who thought they were honoring me by <qex>condescending</qex> to address a few words to me.</q> <rj><qau>F. W. Robinson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To consent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All parties willingly <qex>condescended</qex> heruento.</q> <rj><qau>R. Carew.</qau></rj></p>

<p><syn><b>Syn.</b> -- To yield; stoop; descend; deign; vouchsafe.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condescendency</ent><br/
<ent>Condescendence</ent><br/
<mhw>{ <hw>Con`de*scend"ence</hw> <pr>(?)</pr>, <hw>Con`de*scend"en*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. F. <ets>condescendance</ets>.]</ety> <def>Condescension.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>condescending</ent><br/
<hw>condescending</hw> <pos>adj.</pos>  <def>exhibiting an attitude of superiority; patronizing; -- used of behavior or attitude.</def><br/
<syn><b>Syn. --</b> arch, patronizing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Condescendingly</ent><br/
<hw>Con`de*scend"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a condescending manner.</def>  <rj><au>Atterbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condescension</ent><br/
<hw>Con`de*scen"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>condescensio</ets>.]</ety> <def>The act of condescending; voluntary descent from one's rank or dignity in intercourse with an inferior; courtesy toward inferiors.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It forbids pride . . . and commands humility, modesty, and <qex>condescension</qex> to others.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a dignity and <qex>condescension</qex> . . . as are suitable to a superior nature.</q> <rj><qau>Addison.</qau></rj></p>

<p><syn><b>Syn.</b> -- Complaisance; courtesy; affability.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condescent</ent><br/
<hw>Con`de*scent"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Condescend</er>, <er>Descent</er>.]</ety> <def>An act of condescension.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condign</ent><br/
<hw>Con*dign"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>condigne</ets>, L. <ets>condignus</ets> very worthy; <ets>con-</ets> + <ets>dignus</ets> worthy. See <er>Deign</er>, and cf. <er>Digne</er>.]</ety> <sn>1.</sn> <def>Worthy; suitable; deserving; fit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Condign</qex> and worthy praise.</q> <rj><qau>Udall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Herself of all that rule she deemend most <qex>condign</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deserved; adequate; suitable to the fault or crime.</def> <ldquo/<xex>Condign</xex> censure.<rdquo/  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Unless it were a bloody murderer . . . <br/
I never gave them <qex>condign</qex> punishment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condignity</ent><br/
<hw>Con*dig"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>condignit<eacute/</ets>.]</ety> <fld>(Scholastic Theol.)</fld> <def>Merit, acquired by works, which can claim reward on the score of general benevolence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a worthiness of <qex>condignity</qex>, and proper merit of the heavenly glory, cannot be found in any the best, most perfect, and excellent of created beings.</q> <rj><qau>Bp. Bull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condignly</ent><br/
<hw>Con*dign"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>According to merit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condignness</ent><br/
<hw>Con*dign"ness</hw>, <pos>n.</pos> <def>Agreeableness to deserts; suitableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condiment</ent><br/
<hw>Con"di*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>condimentum</ets>, fr. <ets>condire</ets>. See <er>Condite</er>.]</ety> <def>Something used to give relish to food, and to gratify the taste; a pungment and appetizing substance, as pepper or mustard; seasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As for radish and the like, they are for <qex>condiments</qex>, and not for nourishment.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condisciple</ent><br/
<hw>Con`dis*ci"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>condiscipulus</ets>. See <er>Disciple</er>.]</ety> <def>A schoolfellow; a fellow-student.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condite</ent><br/
<hw>Con"dite</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conditus</ets>, p. p. of <ets>condire</ets> to preserve, pickle, season. See <er>Recondite</er>.]</ety> <def>Preserved; pickled.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condite</ent><br/
<hw>Con*dite"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To pickle; to preserve; <as>as, to <ex>condite</ex> pears, quinces, etc.</as></def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condition</ent><br/
<hw>Con*di"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>conditio</ets> (better <ets>condicio</ets>) agreement, compact, condition; <ets>con-</ets> + a root signifying to <ets>show</ets>, <ets>point out</ets>, akin to <ets>dicere</ets> to say, <ets>dicare</ets> to proclaim, dedicate. See <er>Teach</er>, <er>Token</er>.]</ety> <sn>1.</sn> <def>Mode or state of being; state or situation with regard to external circumstances or influences, or to physical or mental integrity, health, strength, etc.; predicament; rank; position, estate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am in my <qex>condition</qex><br/
A prince, Miranda; I do think, a king.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And O, what man's <qex>condition</qex> can be worse<br/
Than his whom plenty starves and blessings curse?</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The new <qex>conditions</qex> of life.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Essential quality; property; attribute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It seemed to us a <qex>condition</qex> and property of divine powers and beings to be hidden and unseen to others.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Temperament; disposition; character.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>condition</qex> of a saint and the complexion of a devil.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which must exist as the occasion or concomitant of something else; that which is requisite in order that something else should take effect; an essential qualification; stipulation; terms specified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had as lief take her dowry with this <qex>condition</qex>, to be whipped at the high cross every morning.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many are apt to believe remission of sins, but they believe it without the <qex>condition</qex> of repentance.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>A clause in a contract, or agreement, which has for its object to suspend, to defeat, or in some way to modify, the principal obligation; or, in case of a will, to suspend, revoke, or modify a devise or bequest. It is also the case of a future uncertain event, which may or may not happen, and on the occurrence or non-occurrence of which, the accomplishment, recission, or modification of an obligation or testamentary disposition is made to depend.</def>  <rj><au>Blount. Tomlins. Bouvier. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Equation of condition</b></col>. <fld>(Math.)</fld> <cd>See under <er>Equation</er>.</cd> -- <mcol><col><b>On condition</b></col> <it>or</it> <col><b>Upon condition</b></col> (that)</mcol>, <cd>used for <altname>if</altname> in introducing conditional sentences.</cd> <ldquo/<xex>Upon condition</xex> thou wilt swear to pay him tribute . . . thou shalt be placed as viceroy under him.<rdquo/ <au>Shak.</au> -- <col><b>Conditions of sale</b></col>, <cd>the terms on which it is proposed to sell property by auction; also, the instrument containing or expressing these terms.</cd></cs></p>

<p><syn><b>Syn.</b> -- State; situation; circumstances; station; case; mode; plight; predicament; stipulation; qualification; requisite; article; provision; arrangement. See <er>State</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condition</ent><br/
<hw>Con*di"tion</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conditioned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conditioning</conjf>.]</vmorph> <sn>1.</sn> <def>To make terms; to stipulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pay me back my credit,<br/
And I'll <qex>condition</qex> with ye.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metaph.)</fld> <def>To impose upon an object those relations or conditions without which knowledge and thought are alleged to be impossible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To think of a thing is to <qex>condition</qex>.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condition</ent><br/
<hw>Con*di"tion</hw>, <pos>v. t.</pos> <ety>[Cf. LL. <ets>conditionare</ets>. See <er>Condition</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To invest with, or limit by, conditions; to burden or qualify by a condition; to impose or be imposed as the condition of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seas, that daily gain upon the shore,<br/
Have ebb and flow <qex>conditioning</qex> their march.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To contract; to stipulate; to agree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was <qex>conditioned</qex> between Saturn and Titan, that Saturn should put to death all his male children.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(U. S. Colleges)</fld> <def>To put under conditions; to require to pass a new examination or to make up a specified study, as a condition of remaining in one's class or in college; <as>as, to <ex>condition</ex> a student who has failed in some branch of study</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To test or assay, as silk (to ascertain the proportion of moisture it contains).</def>  <rj><au>McElrath.</au></rj></p>

<p><-- 5. <def>train; acclimate.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditional</ent><br/
<hw>Con*di"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conditionalis</ets>.]</ety> <sn>1.</sn> <def>Containing, implying, or depending on, a condition or conditions; not absolute; made or granted on certain terms; <as>as, a <ex>conditional</ex> promise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every covenant of God with man . . . may justly be made (as in fact it is made) with this <qex>conditional</qex> punishment annexed and declared.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram. & Logic)</fld> <def>Expressing a condition or supposition; <as>as, a <ex>conditional</ex> word, mode, or tense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>conditional</qex> proposition is one which asserts the dependence of one categorical proposition on another.</q> <rj><qau>Whately.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The words hypothetical and <qex>conditional</qex> may be . . . used synonymously.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditional</ent><br/
<hw>Con*di"tion*al</hw>, <pos>n.</pos> <sn>1.</sn> <def>A limitation.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A conditional word, mode, or proposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Disjunctives may be turned into <qex>conditionals</qex>.</q> <rj><qau>L. H. Atwater.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditionality</ent><br/
<hw>Con*di`tion*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being conditional, or limited; limitation by certain terms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditionally</ent><br/
<hw>Con*di"tion*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a conditional manner; subject to a condition or conditions; not absolutely or positively.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditionate</ent><br/
<hw>Con*di"tion*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>conditionatus</ets>, p. p. See <er>Condition</er>, <pos>v. t.</pos>]</ety> <def>Conditional.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Barak's answer is faithful, though <qex>conditionate</qex>.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditionate</ent><br/
<hw>Con*di"tion*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To qualify by conditions; to regulate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put under conditions; to render conditional.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditioned</ent><br/
<hw>Con*di"tioned</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Surrounded; circumstanced; in a certain state or condition, as of property or health; <as>as, a well <ex>conditioned</ex> man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The best <qex>conditioned</qex> and unwearied spirit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having, or known under or by, conditions or relations; not independent; not absolute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under these, thought is possible only in the <qex>conditioned</qex> interval.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>made softer by washing with a chemical agent called a <er>conditioner{3}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>conditioned response</ent><br/
<hw>conditioned response</hw> <pos>n.</pos> <def>a behavioral response to a stimulus that has been acquired by experience or conditioning.</def><br/
<syn><b>Syn. --</b> CER, conditioned emotion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conditioned emotional response</ent><br/
<hw>conditioned emotional response</hw> <pos>n.</pos> <def>an emotional response that has been acquired by conditioning.</def><br/
<syn><b>Syn. --</b> CER, conditioned emotion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conditioner</ent><br/
<hw>conditioner</hw> <pos>n.</pos> <sn>1.</sn> <def>exercise that conditions the body; <as>as, farmwork can be a good <ex>conditioner</ex></as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a trainer of athletes.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>a substance used in washing (clothing or hair) to make things softer.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conditioning</ent><br/
<hw>conditioning</hw> <pos>n.</pos> <def>a learning process in which an organism's behavior becomes dependent on the occurrence of a stimulus in its environment.  See <er>conditioned response</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conditionly</ent><br/
<hw>Con*di"tion*ly</hw>, <pos>adv.</pos> <def>Conditionally.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conditory</ent><br/
<hw>Con"di*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conditories</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>conditorium</ets>, fr. <ets>condere</ets> to hide. See <er>Recondite</er>.]</ety> <def>A repository for holding things; a hinding place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>condo</ent><br/
<hw>con"do</hw> <pos>n.</pos> <def>one of the units in a condominium.</def><br/
<syn><b>Syn. --</b> condominium.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>condog</ent><br/
<hw>con*dog"</hw> <pr>(?; 115)</pr>, <pos>v. i.</pos> <ety>[A punning corruption of con<ets>cur</ets>.]</ety> <def>To concur; to agree.</def> <mark>[Burlesque]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word appears in early dictionaries as a synonym for the word <xex>agree</xex>; thus. <ldquo/Agree; concurre, cohere, <xex>condog</xex>, condescend.<rdquo/  <rj><au>Cockeram.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condolatory</ent><br/
<hw>Con*do"la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Expressing condolence.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condole</ent><br/
<hw>Con*dole"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Condoled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Condoling</conjf>.]</vmorph> <ety>[L. <ets>condolere</ets>; <ets>con-</ets> + <ets>dolere</ets> to feel pain, grieve. See <er>Doleful</er>.]</ety> <def>To express sympathetic sorrow; to grieve in sympathy; -- followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your friends would have cause to rejoice, rather than <qex>condole</qex> with you.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condole</ent><br/
<hw>Con*dole"</hw>, <pos>v. t.</pos> <def>To lament or grieve over.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I come not, Samson, to <qex>condole</qex> thy chance.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condolement</ent><br/
<hw>Con*dole"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Condolence.</def> <ldquo/A pitiful <xex>condolement</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sorrow; mourning; lamentation.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condolence</ent><br/
<hw>Con*do"lence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>condol<eacute/ance</ets>.]</ety> <def>Expression of sympathy with another in sorrow or grief.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their congratulations and their <qex>condolences</qex>.</q> <rj><qau>Steele.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A special mission of <qex>condolence</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condoler</ent><br/
<hw>Con*dol"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who condoles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>condominium</ent><br/
<hw>condominium</hw> <pos>n.</pos> <sn>1.</sn> <def>one of the units in a condominium{2}.</def><br/
<syn><b>Syn. --</b> condo.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a complex of dwelling units (as an apartment house) in which each unit is individually owned (as contrasted with rented).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Condonation</ent><br/
<hw>Con`do*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>condonatio</ets> a giving away.]</ety> <sn>1.</sn> <def>The act of condoning or pardoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Forgiveness, either express or implied, by a husband of his wife or by a wife of her husband, for a breach of marital duty, as adultery, with an implied condition that the offense shall not be repeated.</def>  <rj><au>Bouvier. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condone</ent><br/
<hw>Con*done"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Condoned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Condoning</conjf>.]</vmorph> <ety>[L. <ets>condonare</ets>, <ets>-donatum</ets>, to give up, remit, forgive; <ets>con-</ets> + <ets>donare</ets> to give. See <er>Donate</er>.]</ety> <sn>1.</sn> <def>To pardon; to forgive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fraud which he had either concocted or <qex>condoned</qex>.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It would have been magnanimous in the men then in power to have overlooked all these things, and, <qex>condoning</qex> the politics, to have rewarded the poetry of Burns.</q> <rj><qau>J. C. Shairp.</qau></rj></p>

<p><-- 1b.  to consider or treat as if harmless or trivial --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To pardon; to overlook the offense of; esp., to forgive for a violation of the marriage law; -- said of either the husband or the wife.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condor</ent><br/
<hw>Con"dor</hw> <pr>(k<ocr/n"d<ocr/r; <it>in defs.</it> 2 & 3, k<osl/n"d<osl/r)</pr>, <pos>n.</pos> <ety>[Sp. <ets>condor</ets>, fr. Peruvian <ets>cuntur</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A very large bird of the Vulture family (<spn>Sarcorhamphus gryphus</spn>), found in the most elevated parts of the Andes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The California vulture (<spn>Gymnogyps californianus</spn>), also called <altname>California condor</altname>.</def> <mark>[Local, U. S.]</mark>  <note>In the late 20th century it is classed as an endangered species.  The <ex>California condor</ex> used to number in the thousands and ranged along the entire west coast of the United States.  By 1982 only 21 to 24 individuals could be identified in the wild.  A breeding program was instituted, and by 1996 over 50 birds were alive in captivity.  As of 1997, fewer than ten of the bred birds had been reintroduced into the wild.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>3.</sn>  <def>A gold coin of Chile, bearing the figure of a condor, and equal to twenty pesos. It contains 10.98356 grams of gold, and is equivalent to about $7.29. Called also <altname>colon</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>A gold coin of Colombia equivalent to about $9.65. It is no longer coined.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<hw>Condorcet</hw> <pos>prop. n.</pos> <def><person>Marie Jean Antoine Nicolas de Caritat, Marquis de Condorcet</person>, a French philosopher and mathematician, born 17 Sept 1743, died 29 March 1794.  His most important work was on probability and the philosophy of mathematics. His most important treatise was "Essay on the Application of Analysis to the Probability of Majority Decisions" (1785), an extremely important work in the development of the theory of probability.  His work in probability led him to a study of voting methods, and laid the groundwork for the various ranked-pairs voting methods, which are often referred to as "Condorcet's Method".</def><br/

<p><ent>Condottiere</ent><br/
||<hw>Con`dot*tie"re</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Condottieri</plw> <pr>(#)</pr>.</plu> <ety>[It., captain.]</ety> <def>A military adventurer of the fourteenth and fifteenth centuries, who sold his services, and those of his followers, to any party in any contest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduce</ent><br/
<hw>Con*duce"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conduced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conducing</conjf>.]</vmorph> <ety>[L. <ets>conducere</ets> to bring together, conduce, hire; <ets>con-</ets> + <ets>ducere</ets> to lead. See <er>Duke</er> and cf. Conduct, <pos>n.</pos>, <er>Cond</er>.]</ety> <def>To lead or tend, esp. with reference to a favorable or desirable result; to contribute; -- usually followed by <xex>to</xex> or toward.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was sensible how much such a union would <qex>conduce</qex> to the happiness of both.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The reasons you allege do more <qex>conduce</qex><br/
To the hot passion of distemper'd blood.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To contribute; aid; assist; tend; subserve.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduce</ent><br/
<hw>Con*duce"</hw>, <pos>v. t.</pos> <def>To conduct; to lead; to guide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was sent to <qex>conduce</qex> hither the princess.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conducent</ent><br/
<hw>Con*du"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conducens</ets>, p. pr.]</ety> <def>Conducive; tending.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conducent</qex> to the good success of this business.</q> <rj><qau>Abp. Laud.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conducibility</ent><br/
<hw>Con*du"ci*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being conducible; conducibleness.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conducible</ent><br/
<hw>Con*du"ci*ble</hw> <pr>(k<ocr/n*d<umac/"s<icr/*b'l)</pr>, <pos>a.</pos> <ety>[L. <ets>conducibilis</ets>.]</ety> <def>Conducive; tending; contributing.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All his laws are in themselves <qex>conducible</qex> to the temporal interest of them that observe them.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conducibleness</ent><br/
<hw>Con*du"ci*ble*ness</hw>, <pos>n.</pos> <def>Quality of being conducible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conducibly</ent><br/
<hw>Con*du"ci*bly</hw>, <pos>adv.</pos> <def>In a manner to promote.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conducive</ent><br/
<hw>Con*du"cive</hw> <pr>(k<ocr/n*d<umac/"s<icr/v)</pr>, <pos>a.</pos> <def>Loading or tending; helpful; contributive; tending to promote.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>However <qex>conducive</qex> to the good or our country.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduciveness</ent><br/
<hw>Con*du"cive*ness</hw>, <pos>n.</pos> <def>The quality of conducing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduct</ent><br/
<hw>Con"duct</hw> <pr>(k<ocr/n"d<ucr/kt)</pr>, <pos>n.</pos> <ety>[LL. <ets>conductus</ets> defense, escort, fr. L. <ets>conductus</ets>, p. p. of <ets>conducere</ets>. See <er>Conduce</er>, and cf. <er>Conduit</er>.]</ety> <sn>1.</sn> <def>The act or method of conducting; guidance; management.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Christianity has humanized the <qex>conduct</qex> of war.</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conduct</qex> of the state, the administration of its affairs.</q> <rj><qau>Ld. Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Skillful guidance or management; generalship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conduct</qex> of armies is a prince's art.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Attacked the Spaniards . . . with great impetuosity, but with so little <qex>conduct</qex>, that his forces were totally routed.</q> <rj><qau>Robertson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Convoy; escort; guard; guide.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will be your <qex>conduct</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In my <qex>conduct</qex> shall your ladies come.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which carries or conveys anything; a channel; a conduit; an instrument.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Although thou hast been <qex>conduct</qex> of my shame.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The manner of guiding or carrying one's self; personal deportment; mode of action; behavior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All these difficulties were increased by the <qex>conduct</qex> of Shrewsbury.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What in the <qex>conduct</qex> of our life appears<br/
So well designed, so luckily begun,<br/
But when we have our wish, we wish undone?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Plot; action; construction; manner of development.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The book of Job, in <qex>conduct</qex> and diction.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conduct money</b></col> <fld>(Naut.)</fld>, <cd>a portion of a seaman's wages retained till the end of his engagement, and paid over only if his conduct has been satisfactory.</cd></cs></p>

<p><syn><b>Syn.</b> -- Behavior; carriage; deportment; demeanor; bearing; management; guidance. See <er>Behavior</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduct</ent><br/
<hw>Con*duct"</hw> <pr>(k<ocr/n*d<ucr/kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conducted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conducting</conjf>.]</vmorph> <ety>[See <er>Conduct</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To lead, or guide; to escort; to attend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can <qex>conduct</qex> you, lady, to a low<br/
But loyal cottage, where you may be safe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lead, as a commander; to direct; to manage; to carry on; <as>as, to <ex>conduct</ex> the affairs of a kingdom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Little skilled in the art of <qex>conducting</qex> a siege.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To behave; -- with the reflexive; <as>as, he <ex>conducted</ex> himself well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Physics)</fld> <def>To serve as a medium for conveying; to transmit, as heat, light, electricity, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>To direct, as the leader in the performance of a musical composition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduct</ent><br/
<hw>Con*duct"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To act as a conductor (as of heat, electricity, etc.); to carry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To conduct one's self; to behave.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductance</ent><br/
<hw>Con*duct"ance</hw> <pr>(k<ocr/n*d<ucr/k"t<ait/ns)</pr>, <pos>n.</pos>  <ety>[<ets>Conduct</ets>, v. + <ets>-ance</ets>.]</ety> <fld>(Elec.)</fld> <def>Conducting power; -- the reciprocal of <contr>resistance</contr>. A suggested unit is the <xex>mho</xex>, the reciprocal of the ohm.</def></p>

<p><q><qex>Conductance</qex> is an attribute of any specified conductor, and refers to its shape, length, and other factors. Conductivity is an attribute of any specified material without direct reference to its shape or other factors.</q>  <rj><qau>Sloane's Elec. Dict.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Conductibility</ent><br/
<hw>Con*duct`i*bil"i*ty</hw> <pr>(k<ocr/n*d<ucr/k`t<icr/*b<icr/l"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>conductibilit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Capability of being conducted; <as>as, the <ex>conductibility</ex> of heat or electricity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conductivity; capacity for receiving and transmitting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductible</ent><br/
<hw>Con*duct"i*ble</hw> <pr>(-b'l)</pr>, <pos>a.</pos> <def>Capable of being conducted.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 299 --></p>

<p><ent>Conduction</ent><br/
<hw>Con*duc"tion</hw> <pr>(k<ocr/n*d<ucr/k"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>conductio</ets> a bringing together: cf. F. <ets>conduction</ets>.]</ety> <sn>1.</sn> <def>The act of leading or guiding.</def>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of training up.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physics)</fld> <def>Transmission through, or by means of, a conductor; also, conductivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The] communication [of heat] from one body to another when they are in contact, or through a homogenous body from particle to particle, constitutes <qex>conduction</qex>.</q> <rj><qau>Amer. Cyc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductive</ent><br/
<hw>Con*duct"ive</hw> <pr>(-d<ucr/k"t<icr/v)</pr>, <pos>a.</pos> <def>Having the quality or power of conducting; <as>as, the <ex>conductive</ex> tissue of a pistil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ovarian walls . . . are seen to be distinctly <qex>conductive</qex>.</q> <rj><qau>Goodale (Gray's Bot. ).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductivity</ent><br/
<hw>Con`duc*tiv"i*ty</hw> <pr>(k<ocr/n`d<ucr/k*t<icr/v"<icr/*t<ycr/)</pr>, <pos>n.</pos> <def>The quality or power of conducting, or of receiving and transmitting, as heat, electricity, etc.; <as>as, the <ex>conductivity</ex> of a nerve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thermal conductivity</b></col> <fld>(Physics)</fld>, <cd>the quantity of heat that passes in unit time through unit area of a plate whose thickness is unity, when its opposite faces differ in temperature by one degree.</cd> <-- a [plate] missing in orig. --><au>J. D. Everett.</au> -- <col><b>Thermometic conductivity</b></col> <fld>(Physics)</fld>, <cd>the thermal conductivity when the unit of heat employed is the heat required to raise a unit volume of the substance one degree.</cd></cs><-- a [unit] mir --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductor</ent><br/
<hw>Con*duct"or</hw> <pr>(k<ocr/n*d<ucr/k"t<etil/r)</pr>, <pos>n.</pos> <ety>[LL., a carrier, transporter, L., a lessee.]</ety> <sn>1.</sn> <def>One who, or that which, conducts; a leader; a commander; a guide; a manager; a director.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Zeal, the blind <qex>conductor</qex> of the will.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One in charge of a public conveyance, as of a railroad train or a street car.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>The leader or director of an orchestra or chorus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Physics)</fld> <def>A substance or body capable of being a medium for the transmission of certain forces, esp. heat or electricity; specifically, a lightning rod.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Surg.)</fld> <def>A grooved sound or staff used for directing instruments, as lithontriptic forceps, etc.; a director.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Arch.)</fld> <def>Same as <er>Leader</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Prime conductor</b></col> <fld>(Elec.)</fld>, <cd>the largest conductor of an electrical machine, serving to collect, accumulate, or retain the electricity.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductory</ent><br/
<hw>Con*duct"o*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>conductorius</ets>.]</ety> <def>Having the property of conducting.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conductress</ent><br/
<hw>Con*duct"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who leads or directs; a directress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduit</ent><br/
<hw>Con"duit</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>conductus</ets> escort, conduit. See <er>Conduct</er>.]</ety> <sn>1.</sn> <def>A pipe, canal, channel, or passage for conveying water or fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the <qex>conduits</qex> of my blood froze up.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the fountain of all those bitter waters, of which, through a hundred different <qex>conduits</qex>, we have drunk.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A structure forming a reservoir for water.</def>  <rj><au>Oxf. Gloss.</au></rj></p>

<p><sd>(b)</sd> <def>A narrow passage for private communication.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduit railway</ent><br/
<ent>Conduit system</ent><br/
<hw>Con"duit sys"tem</hw>. <fld>(Elec.)</fld> <def>A system of electric traction, esp. for light railways, in which the actuating current passes along a wire or rail laid in an underground conduit, from which the current is <ldquo/picked up<rdquo/ by a plow or other device fixed to the car or electric locomotive.  Hence <hw>Conduit railway</hw>.</def>
<-- note irregular use of the headword format here --><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Conduplicate</ent><br/
<hw>Con*du"pli*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conduplicatus</ets>, p. p. of <ets>conduplicare</ets>. See <er>Duplicate</er>.]</ety> <fld>(Bot.)</fld> <def>Folded lengthwise along the midrib, the upper face being within; -- said of leaves or petals in vernation or <ae/stivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conduplication</ent><br/
<hw>Con*du`pli*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conduplicatio</ets>.]</ety> <def>A doubling together or folding; a duplication.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condurango</ent><br/
<hw>Con`du*ran"go</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Cundurango</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condurrite</ent><br/
<hw>Con*dur"rite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A variety of the mineral domeykite, or copper arsenide, from the Condurra mine in Cornwall, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condylar</ent><br/
<hw>Con"dy*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to a condyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Condylar foramen</b></col> <fld>(Anat.)</fld>, <cd>a formen in front of each condyle of the occipital bone; -- sometimes called the <altname>anterior condylar foramen</altname> when a second, or posterior, foramen is present behind the condyle, as often happens in man.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condyle</ent><br/
<hw>Con"dyle</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[L. <ets>condylus</ets> knuckle, joint, Gr. <grk>ko`ndylos</grk>: cf. F. <ets>condyle</ets>.]</ety> <fld>(Anat.)</fld> <def>A bony prominence; particularly, an eminence at the end of a bone bearing a rounded articular surface; -- sometimes applied also to a concave articular surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condyloid</ent><br/
<hw>Con"dy*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Condyle</ets> + <ets>-oid</ets>: cf. F. <ets>condylo<ium/de</ets>.]</ety> <fld>(Anat.)</fld> <def>Shaped like or pertaining to a condyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condylome</ent><br/
<ent>Condyloma</ent><br/
<mhw>{ ||<hw>Con`dy*lo"ma</hw> <pr>(-l<omac/"m<adot/)</pr>, ||<hw>Con"dy*lome</hw> <pr>(-l<omac/m)</pr> }</mhw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Condylomata</plw> <pr>(#)</pr> <it>or</it> <pr>(#)</pr>, E. <plw>Condylomes</plw> <pr>(-l<omac/mz)</pr>.</plu> <ety>[NL. <ets>condyloma</ets>, fr. Gr. <?/, from <grk>ko`ndylos</grk> knuckle. See <er>-oma</er>.]</ety> <fld>(Med.)</fld> <def>A wartlike new growth on the outer skin or adjoining mucous membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are two kinds of condylomata, the pointed and the broad, the latter being of syphilitic origin.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Condylopod</ent><br/
||<hw>Con*dyl"o*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`ndylos</grk> knuckle (or joint) + <ets>-pod</ets>.]</ety> <fld>(Zool.)</fld> <def>An arthropod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cone</ent><br/
||<hw>Cone</hw> <pr>(k<omac/n?)</pr>, <pos>n.</pos> <ety>[L. <ets>conus</ets> cone (in sense 1), Gr. <grk>kw^nos</grk>; akin to Skr. <ets><cced/ana</ets> whetstone, L. <ets>cuneus</ets> wedge, and prob. to E. <ets>hone</ets>. See <er>Hone</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A solid of the form described by the revolution of a right-angled triangle about one of the sides adjacent to the right angle; -- called also a <altname>right cone</altname>. More generally, any solid having a vertical point and bounded by a surface which is described by a straight line always passing through that vertical point; a solid having a circle for its base and tapering to a point or vertex.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything shaped more or less like a mathematical cone; <as>as, a volcanic <ex>cone</ex>, a collection of scori<ae/ around the crater of a volcano, usually heaped up in a conical form</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now had Night measured with her shadowy <qex>cone</qex><br/
Half way up hill this vast sublunar vault.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>The fruit or strobile of the <ord>Conifer<ae/</ord>, as of the pine, fir, cedar, and cypress. It is composed of woody scales, each one of which has one or two seeds at its base.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A shell of the genus <gen>Conus</gen>, having a conical form.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cone of rays</b></col> <fld>(Opt.)</fld>, <cd>the pencil of rays of light which proceed from a radiant point to a given surface, as that of a lens, or conversely.</cd> -- <col><b>Cone pulley</b></col>. <cd>See in the Vocabulary.</cd> -- <mcol><col><b>Oblique cone</b></col> <it>or</it> <col><b>Scalene cone</b></col></mcol>, <cd>a cone of which the axis is inclined to the plane of its base.</cd> -- <col><b>Eight cone</b></col>. <cd>See <er>Cone</er>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cone</ent><br/
<hw>Cone</hw> <pr>(k<omac/n)</pr>, <pos>v. t.</pos> <def>To render cone-shaped; to bevfl like whe circwlar segoent of a cone; <as>as, to <ex>cone</ex> the tires of car wheels</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cone clutch</ent><br/
<hw>Cone clutch</hw>. <fld>(Mach.)</fld> <def>A friction clutch with conical bearing surfaces.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coneflower</ent><br/
<hw>Cone"flow`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Any plant of the genus <gen>Rudbeckia</gen>; -- so called from the cone-shaped disk of the flower head. They are cultivated for their large usually yellow daisies with prominent central cones.  Also, any plant of the related genera <gen>Ratibida</gen> and <gen>Brauneria</gen>, the latter usually known as <stype>purple coneflower</stype>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>any of various perennials of eastern US having thick rough leaves and long-stalked showy flowers with drooping rays and a conelike center.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cone-in-cone</ent><br/
<hw>Cone"-in-cone"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Consisting of a series of parallel cones, each made up of many concentric cones closely packed together; -- said of a kind of structure sometimes observed in sedimentary rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coneine</ent><br/
<hw>Co*ne"ine</hw> <pr>(? <or/ ?; 104)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See <er>Conine</er>.</def></p>

<p><ent>cone-nose</ent><br/
<ent>conenose</ent><br/
<mhw><hw>cone"nose</hw>, <hw>cone"-nose`</hw></mhw>, <pos>n.</pos> <def>A large bloodsucking hemipterous insect of the family <fam>Reduviid<ae/</fam>, often found in houses, esp. in the southern and western United States. It bites severely, and is one of the species called <altname>kissing bugs</altname>. It is also called <altname>big bedbug</altname>.</def><br/
<syn><b>Syn. --</b> cone-nosed bug, conenose bug, big bedbug, kissing bug.</syn><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Conepatl</ent><br/
<ent>Conepate</ent><br/
<mhw>{ <hw>Co"ne*pate</hw> <pr>(k<omac/"n<esl/*p<amac/t)</pr>, <hw>Co"ne*patl</hw> <pr>(-p<acr/t'l)</pr> }</mhw>, <pos>n.</pos> <ety>[Mexican <ets>conepatl</ets> and <ets>epatl</ets>.]</ety> <fld>(Zool.)</fld> <def>The skunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cone pulley</ent><br/
<hw>Cone" pul"ley</hw> <pr>(?)</pr>. <def>A pulley for driving machines, etc., having two or more parts or steps of different diameters; a pulley having a conical shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conessi</ent><br/
<hw>conessi</hw> <pos>n.</pos> <def>a tropical Asian tree (<spn>Holarrhena pubescens</spn> or <spn>Holarrhena antidysenterica</spn>) with hard white wood and bark formerly used as a remedy for dysentery and diarrhea.</def><br/
<syn><b>Syn. --</b> ivory tree, kurchi, kurchee, <spn>Holarrhena pubescens</spn>, <spn>Holarrhena antidysenterica</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conestoga wain</ent><br/
<ent>Conestoga wagon</ent><br/
<mhw>{ <hw>Con`es*to"ga wag`on</hw> <it>or</it>  <hw>Con`es*to"ga wain</hw> <pr>(?)</pr> }</mhw>. <ety>[From <etsep>Conestoga</etsep>, Pennsylvania.]</ety> <def>A kind of large broad-wheeled wagon, usually covered, for traveling in soft soil and on prairies.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coney</ent><br/
<hw>Co"ney</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A rabbit. See <er>Cony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A fish. See <er>Cony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confab</ent><br/
<hw>Con"fab</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. from <ets>confabulation</ets>.]</ety> <def>Familiar talk or conversation.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confabulate</ent><br/
<hw>Con*fab"u*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confabulated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confabulating</conjf>.]</vmorph> <ety>[L. <ets>confabulatus</ets>, p. p. of <ets>confabulary</ets>, to converse together; <ets>con-</ets> + <ets>fabulary</ets> to speak, fr. <ets>fabula</ets>. See <er>Fable</er>.]</ety> <def>To talk familiarly together; to chat; to prattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall not ask Jean Jaques Rousseau<br/
If birds <qex>confabulate</qex> or no.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confabulation</ent><br/
<hw>Con*fab`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confabulatio</ets>.]</ety> <def>Familiar talk; easy, unrestrained, unceremonious conversation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Friends' <qex>confabulations</qex> are comfortable at all times, as fire in winter.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confabulatory</ent><br/
<hw>Con*fab"u*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of familiar talk; in the form of a dialogue.</def>  <rj><au>Weever.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confalon</ent><br/
<hw>Con"fa*lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Confalon</er>.]</ety> <fld>(R. C. Ch.)</fld> <def>One of a fraternity of seculars, also called <altname>Penitents</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confarreation</ent><br/
<hw>Con*far`re*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confarreatio</ets>, fr. <ets>confarreare</ets> to marry; <ets>con-</ets> + <ets>farreum</ets> (sc. <ets>libum</ets> cake) a spelt cake, fr. <ets>farreus</ets> made of spelt, fr. <ets>far</ets> a sort of grain.]</ety> <fld>(Antiq.)</fld> <def>A form of marriage among the Romans, in which an offering of bread was made, in presence of the high priest and at least ten witnesses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confated</ent><br/
<hw>Con*fat"ed</hw> <pr>(?)</pr>, <pos>p. a.</pos> <def>Fated or decreed with something else.</def> <mark>[R.]</mark>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confect</ent><br/
<hw>Con*fect"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confecting</conjf>.]</vmorph> <ety>[L. <ets>confectus</ets>, p. p. of <ets>conficere</ets> to prepare. See <er>Comfit</er>.]</ety> <sn>1.</sn> <def>To prepare, as sweetmeats; to make a confection of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Saffron <qex>confected</qex> in Cilicia.</q> <rj><qau>W. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To construct; to form; to mingle or mix.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of this were <qex>confected</qex> the famous everlasting lamps and tapers.</q> <rj><qau>Sir T. Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[My joys] are still <qex>confected</qex> with some fears.</q> <rj><qau>Stirling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confect</ent><br/
<hw>Con"fect</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A comfit; a confection.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>At supper eat a pippin roasted and sweetened with sugar of roses and caraway <qex>confects</qex>.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confection</ent><br/
<hw>Con*fec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>confectio</ets>.]</ety> <sn>1.</sn> <def>A composition of different materials.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A new <qex>confection</qex> of mold.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A preparation of fruits or roots, etc., with sugar; a sweetmeat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Certain <qex>confections</qex> . . . are like to candied conserves, and are made of sugar and lemons.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A composition of drugs.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A soft solid made by incorporating a medicinal substance or substances with sugar, sirup, or honey.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The pharmacop<oe/ias formerly made a distinction between <xex>conserves</xex> (made of fresh vegetable substances and sugar) and <xex>electuaries</xex> (medicinal substances combined with sirup or honey), but the distinction is now abandoned and all are called <xex>confections</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confectionary</ent><br/
<hw>Con*fec"tion*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>confectionaris</ets> a pharmacist.]</ety> <def>A confectioner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He will take your daughters to be <qex>confectionaries</qex>, and to be cooks.</q> <rj><qau>1 Sam. viii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confectionary</ent><br/
<hw>Con*fec"tion*a*ry</hw>, <pos>a.</pos> <def>Prepared as a confection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The biscuit or <qex>confectionary</qex> plum.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confectioner</ent><br/
<hw>Con*fec"tion*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A compounder.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Canidia Neapolitana was <qex>confectioner</qex> of unguents.</q> <rj><qau>Haywood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One whose occupation it is to make or sell confections, candies, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confectioners' sugar</ent><br/
<hw>Con*fec"tion*ers' sug`ar</hw>. <def>A highly refined sugar in impalpable powder, esp. suited to confectioners' uses.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Confectionery</ent><br/
<hw>Con*fec"tion*er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Sweetmeats, in general; things prepared and sold by a confectioner; confections; candies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place where candies, sweetmeats, and similar things are made or sold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confectory</ent><br/
<hw>Con*fec"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the art of making sweetmeats.</def> <mark>[Obs.]</mark>  <rj><au>Beaumont.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confecture</ent><br/
<hw>Con*fec"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Confiture</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confeder</ent><br/
<hw>Con*fed"er</hw> <pr>(k<ocr/n*f<ecr/d"<etil/r)</pr>, <pos>v. i.</pos> <ety>[Cf. F. <ets>conf<eacute/d<eacute/rer</ets>. See <er>Confederate</er>.]</ety> <def>To confederate.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederacy</ent><br/
<hw>Con*fed"er*a*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Confederacies</plw> <pr>(#)</pr>.</plu> <ety>[From <er>Confederate</er>, a.]</ety> <sn>1.</sn> <def>A league or compact between two or more persons, bodies of men, or states, for mutual support or common action; alliance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The friendships of the world are oft<br/
<qex>Confederacies</qex> in vice or leagues of pleasure.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He hath heard of our <qex>confederacy</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Virginia promoted a <qex>confederacy</qex>.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The persons, bodies, states, or nations united by a league; a confederation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Grecian common wealth, . . . the most heroic <qex>confederacy</qex> that ever existed.</q> <rj><qau>Harris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Virgil has a whole <qex>confederacy</qex> against him.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A combination of two or more persons to commit an unlawful act, or to do a lawful act by unlawful means. See <er>Conspiracy</er>.</def></p>

<p><syn><b>Syn.</b> -- League; compact; alliance; association; union; combination; confederation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederacy</ent><br/
<hw>Con*fed"er*a*cy</hw>, <pos>n.</pos> <fld>(Amer. Hist.)</fld> <def>With <xex>the</xex>, the Confederate States of America.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Confederate</ent><br/
<hw>Con*fed"er*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>confoederatus</ets>, p. p. of <ets>confoederare</ets> to join by a league; <ets>con-</ets> + <ets>foederare</ets> to establish by treaty or league, fr. <ets>foedus</ets> league, compact. See <er>Federal</er>.]</ety> <sn>1.</sn> <def>United in a league; allied by treaty; engaged in a confederacy; banded together; allied.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the swords<br/
In Italy, and her <qex>confederate</qex> arms,<br/
Could not have made this peace.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Amer. Hist.)</fld> <def>Of or pertaining to the government of the eleven Southern States of the United States which (1860-1865) attempted to establish an independent nation styled the Confederate States of America; <as>as, the <ex>Confederate</ex> congress; <ex>Confederate</ex> money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederate</ent><br/
<hw>Con*fed"er*ate</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who is united with others in a league; a person or a nation engaged in a confederacy; an ally; also, an accomplice in a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He found some of his <qex>confederates</qex> in gaol.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Amer. Hist.)</fld> <def>A name designating an adherent to the cause of the States which attempted to withdraw from the Union (1860-1865).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederate</ent><br/
<hw>Con*fed"er*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confederated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Confederating</conjf> <pr>(?)</pr>.]</vmorph> <def>To unite in a league or confederacy; to ally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With these the Piercies them <qex>confederate</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederate</ent><br/
<hw>Con*fed"er*ate</hw>, <pos>v. i.</pos> <def>To unite in a league; to join in a mutual contract or covenant; to band together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By words men . . . covenant and <qex>confederate</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederater</ent><br/
<hw>Con*fed"er*a`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A confederate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederation</ent><br/
<hw>Con*fed`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confoederatio</ets>: cf. F. <ets>conf<eacute/d<eacute/ration</ets>.]</ety> <sn>1.</sn> <def>The act of confederating; a league; a compact for mutual support; alliance, particularly of princes, nations, or states.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The three princes enter into some strict league and <qex>confederation</qex> among themselves.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This was no less than a political <qex>confederation</qex> of the colonies of New England.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The parties that are confederated, considered as a unit; a confederacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Articles of confederation</b></col>. <cd>See under <er>Article</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederative</ent><br/
<hw>Con*fed"er*a*tive</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Of or pertaining to a confederation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confederator</ent><br/
<hw>Con*fed"er*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A confederate.</def>  <rj><au>Grafton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confer</ent><br/
<hw>Con*fer"</hw> <pr>(k<ocr/n*f<etil/r")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conferred</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conferring</conjf>.]</vmorph> <ety>[L. <ets>conferre</ets> to bring together, contribute, consult; <ets>con-</ets> + <ets>ferre</ets> to bear: cf. F. <ets>conf<eacute/rer</ets>. See 1st <er>Bear</er>.]</ety> <sn>1.</sn> <def>To bring together for comparison; to compare.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If we <qex>confer</qex> these observations with others of the like nature, we may find cause to rectify the general opinion.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To grant as a possession; to bestow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The public marks of honor and reward<br/
<qex>Conferred</qex> upon me.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To contribute; to conduce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The closeness and compactness of the parts resting together doth much <qex>confer</qex> to the strength of the union.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confer</ent><br/
<hw>Con*fer"</hw>, <pos>v. i.</pos> <def>To have discourse; to consult; to compare views; to deliberate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Festus, when he had <qex>conferred</qex> with the council, answered.</q> <rj><qau>Acts xxv. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You shall hear us <qex>confer</qex> of this.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To counsel; advise; discourse; converse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conferee</ent><br/
<hw>Con`fer*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Referee</er>.]</ety> <sn>1.</sn> <def>One who is conferred with, or who takes part in a conference; <as>as, the <ex>conferees</ex> on the part of the Senate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One upon whom something is conferred.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conference</ent><br/
<hw>Con"fer*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>conf<eacute/rence</ets>. See <er>Confer</er>.]</ety> <sn>1.</sn> <def>The act of comparing two or more things together; comparison.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Helps and furtherances which . . . the mutual <qex>conference</qex> of all men's collections and observations may afford.</q> <rj><qau>Hocker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of consulting together formally; serious conversation or discussion; interchange of views.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor with such free and friendly <qex>conference</qex><br/
As he hath used of old.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A meeting for consultation, discussion, or an interchange of opinions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A meeting of the two branches of a legislature, by their committees, to adjust between them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Methodist Church)</fld> <def>A stated meeting of preachers and others, invested with authority to take cognizance of ecclesiastical matters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A voluntary association of Congregational churches of a district; the district in which such churches are.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conference meeting</b></col>, <cd>a meeting for conference. Specifically, a meeting conducted (usually) by laymen, for conference and prayer.</cd> <mark>[U. S.]</mark> -- <col><b>Conference room</b></col>, <cd>a room for conference and prayer, and for the pastor's less formal addresses.</cd> <mark>[U. S.]</mark></cs><-- (b) any room primarily used for meetings, as in a business establishment.  The size of the room varies greatly, but will generally be smaller than a lecture hall or an auditorium. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conferential</ent><br/
<hw>Con`fer*en"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to conference.</def> <mark>[R.]</mark>  <rj><au>Clarke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conferment</ent><br/
<hw>conferment</hw> <pos>n.</pos> <def>the act of conferring an honor or presenting a gift.  Same as <er>conferral</er>.</def><br/
<syn><b>Syn. --</b> bestowal, conferral.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conferrable</ent><br/
<hw>Con*fer"ra*ble</hw> <pr>(#)</pr> <pos>a.</pos> <def>Capable of being conferred.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conferral</ent><br/
<hw>conferral</hw> <pos>n.</pos> <def>the act of conferring an honor or presenting a gift.</def><br/
<syn><b>Syn. --</b> bestowal, conferment.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conferred</ent><br/
<hw>conferred</hw> <pos>adj.</pos>  <def>given formally or officially.</def><br/
<syn><b>Syn. --</b> bestowed, presented.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conferree</ent><br/
<hw>Con`fer*ree"</hw> <pr>(k<ocr/n`f<etil/r*r<emac/")</pr>, <pos>n.</pos> <def>Same as <er>Conferee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 300  --></p>

<p><ent>Conferrer</ent><br/
<hw>Con*fer"rer</hw> <pr>(k<ocr/n*f<etil/r"r<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who confers; one who converses.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who bestows; a giver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conferruminated</ent><br/
<ent>Conferruminate</ent><br/
<mhw>{ <hw>Con`fer*ru"mi*nate</hw> <pr>(?)</pr>, <hw>Con`fer*ru"mi*na`ted</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <ety>[L. <ets>conferruminare</ets> to cement. See <er>Ferruminate</er>.]</ety> <fld>(Bot.)</fld> <def>Closely united by the coalescence, or sticking together, of contiguous faces, as in the case of the cotyledons of the live-oak acorn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conferva</ent><br/
||<hw>Con*fer"va</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conferv<ae/</plw> <pr>(#)</pr>.</plu> <ety>[L., a kind of water plant. See <er>Comfrey</er>.]</ety> <fld>(Bot.)</fld> <def>Any unbranched, slender, green plant of the fresh-water algae. The word is frequently used in a wider sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confervaceous</ent><br/
<hw>Con`fer*va"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to the confervae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confervoid</ent><br/
<hw>Con*fer"void</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Conferva</ets> + <ets>-oid</ets>.]</ety> <def>Like, or related to, the confervae.</def>  <rj><au>Loudon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confervous</ent><br/
<hw>Con*fer"vous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to confervae; consisting of, or resembling, the confervae.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yon exiguous pool's <qex>confervous</qex> scum.</q> <rj><qau>O. W. Holmes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confess</ent><br/
<hw>Con*fess"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confessed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Confessing</conjf>.]</vmorph> <ety>[F. <ets>confesser</ets>, fr. L. <ets>confessus</ets>, p. p. of <ets>confiteri</ets> to confess; <ets>con-</ets> + <ets>fateri</ets> to confess; akin to <ets>fari</ets> to speak. See 2d <er>Ban</er>, <er>Fame</er>.]</ety> <sn>1.</sn> <def>To make acknowledgment or avowal in a matter pertaining to one's self; to acknowledge, own, or admit, as a crime, a fault, a debt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And there <qex>confess</qex><br/
Humbly our faults, and pardon beg.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I must <qex>confess</qex> I was most pleased with a beautiful prospect that none of them have mentioned.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To acknowledge faith in; to profess belief in.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever, therefore, shall <qex>confess</qex> me before men, him will I <qex>confess</qex>, also, before my Father which is in heaven.</q> <rj><qau>Matt. x. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For the Sadducees say that there is no resurrection, neither angel, nor spirit; but the Pharisees <qex>confess</qex> both.</q> <rj><qau>Acts xxiii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To admit as true; to assent to; to acknowledge, as after a previous doubt, denial, or concealment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I never gave it him. Send for him hither,<br/
And let him <qex>confess</qex> a truth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As I <qex>confess</qex> it needs must be.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As an actor <qex>confessed</qex> without rival to shine.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eccl.)</fld> <sd>(a)</sd> <def>To make known or acknowledge, as one's sins to a priest, in order to receive absolution; -- sometimes followed by the reflexive pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our beautiful votary took an opportunity of <qex>confessing</qex> herself to this celebrated father.</q> <rj><qau>Addison.</qau></rj></p>

<p><sd>(b)</sd> <def>To hear or receive such confession; -- said of a priest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . heard mass, and the prince, his son, with him, and the most part of his company were <qex>confessed</qex>.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To disclose or reveal, as an effect discloses its cause; to prove; to attest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tall thriving trees <qex>confessed</qex> the fruitful mold.</q> <rj><qau>Pope.</qau></rj></p>

<p><syn><b>Syn.</b> -- Admit; grant; concede; avow; own; assent; recognize; prove; exhibit; attest.</syn> <usage> -- <er>To Confess</er>, <er>Acknowledge</er>, <er>Avow</er>. <xex>Acknowledge</xex> is opposed to <xex>conceal</xex>. We <xex>acknowledge</xex> what we feel must or ought to be made known. (See <er>Acknowledge</er>.) <xex>Avow</xex> is opposed to <xex>withhold</xex>. We <xex>avow</xex> when we make an open and public declaration, as against obloquy or opposition; as, to <xex>avow</xex> one's principles; to <xex>avow</xex> one's participation in some act. <xex>Confess</xex> is opposed to <xex>deny</xex>. We <xex>confess</xex> (in the ordinary sense of the word) what we feel to have been wrong; as, to <xex>confess</xex> one's errors or faults. We sometimes use <xex>confess</xex> and <xex>acknowledge</xex> when there is no admission of our being in the wrong; as, this, I <xex>confess</xex>, is my opinion; I <xex>acknowledge</xex> I have always thought so; but in these cases we mean simply to imply that others may perhaps <xex>think</xex> us in the wrong, and hence we use the words by way of deference to their opinions. It was in this way that the early Christians were led to use the Latin <xex>confiteor</xex> and <xex>confessio fidei</xex> to denote the public declaration of their faith in Christianity; and hence the corresponding use in English of the verb <xex>confess</xex> and the noun <xex>confession</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confess</ent><br/
<hw>Con*fess"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To make confession; to disclose sins or faults, or the state of the conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every tongue shall <qex>confess</qex> to God.</q> <rj><qau>Rom. xiv. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To acknowledge; to admit; to concede.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But since<br/
(And I <qex>confess</qex> with right) you think me bound.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessant</ent><br/
<hw>Con*fess"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confessant</ets>.]</ety> <def>One who confesses to a priest.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessary</ent><br/
<hw>Con*fess"a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>confessarius</ets>.]</ety> <def>One who makes a confession.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessedly</ent><br/
<hw>Con*fess"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By confession; without denial.</def> <altsp>[Written also <asp>confessly</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confesser</ent><br/
<hw>Con*fess"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes a confession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confession</ent><br/
<hw>Con*fes"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confession</ets>, L. <ets>confessio</ets>.]</ety> <sn>1.</sn> <def>Acknowledgment; avowal, especially in a matter pertaining to one's self; the admission of a debt, obligation, or crime.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With a crafty madness keeps aloof,<br/
When we would bring him on to some <qex>confession</qex><br/
Of his true state.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Acknowledgment of belief; profession of one's faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With the mouth <qex>confession</qex> is made unto salvation.</q> <rj><qau>Rom. x. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>The act of disclosing sins or faults to a priest in order to obtain sacramental absolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Auricular <qex>confession</qex> . . . or the private and special <qex>confession</qex> of sins to a priest for the purpose of obtaining his absolution.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A formulary in which the articles of faith are comprised; a creed to be assented to or signed, as a preliminary to admission to membership of a church; a confession of faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>An admission by a party to whom an act is imputed, in relation to such act. A judicial confession settles the issue to which it applies; an extrajudical confession may be explained or rebutted.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Confession and avoidance</b></col> <fld>(Law)</fld>, <cd>a mode of pleading in which the party confesses the facts as stated by his adversary, but alleges some new matter by way of avoiding the legal effect claimed for them.</cd>  <rj><au>Mozley & W.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Confession of faith</b></col>, <cd>a formulary containing the articles of faith; a creed.</cd> -- <col><b>General confession</b></col>, <cd>the confession of sins made by a number of persons in common, as in public prayer.</cd> -- <col><b>Westminster Confession</b></col>. <cd>See <cref>Westminster Assembly</cref>, under <er>Assembly</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessional</ent><br/
<hw>Con*fes"sion*al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confessional</ets>.]</ety> <def>The recess, seat, or inclosed place, where a priest sits to hear confessions; often a small structure furnished with a seat for the priest and with a window or aperture so that the penitent who is outside may whisper into the priest's ear without being seen by him or heard by others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessional</ent><br/
<hw>Con*fes"sion*al</hw>, <pos>a.</pos> <def>Pertaining to a confession of faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Confessional equality</b></col>, <cd>equality before the law of persons confessing different creeds.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessionalism</ent><br/
<hw>Con*fes"sion*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>An exaggerated estimate of the importance of giving full assent to any particular formula of the Christian faith.</def>  <rj><au>Shaff.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessionalist</ent><br/
<hw>Con*fes"sion*al*ist</hw>, <pos>n.</pos> <def>A priest hearing, or sitting to hear, confession.</def> <mark>[R.]</mark>  <rj><au>Boucher</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessionary</ent><br/
<hw>Con*fes"sion*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>confessionarium</ets>.]</ety> <def>A confessional.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessionary</ent><br/
<hw>Con*fes"sion*a*ry</hw>, <pos>a.</pos> <def>Pertaining to auricular confession; <as>as, a <ex>confessionary</ex> litany</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessionist</ent><br/
<hw>Con*fes"sion*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>confessioniste</ets>.]</ety> <def>One professing a certain faith.</def>  <rj><au>Bp. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessor</ent><br/
<hw>Con*fess"or</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>confessor</ets>, F. <ets>confesseur</ets>, fr. L. & LL. <ets>confessor</ets>.]</ety> <sn>1.</sn> <def>One who confesses; one who acknowledges a fault, or the truth of a charge, at the risk of suffering; specifically, one who confesses himself a follower of Christ and endures persecution for his faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He who dies for religion is a martyr; he who suffers for it is a <qex>confessor</qex>.</q> <rj><qau>Latham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our religion which hath been sealed with the blood of so many martyrs and <qex>confessors</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A priest who hears the confessions of others and is authorized to grant them absolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confessorship</ent><br/
<hw>Con*fess"or*ship</hw>, <pos>n.</pos> <def>The act or state of suffering persecution for religious faith.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our duty to contend even to <qex>confessorship</qex>.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confestly</ent><br/
<hw>Con*fest"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>See <er>Cofessedly</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confetti</ent><br/
||<hw>Con*fet"ti</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>-fetto</singw> <pr>(<?/)</pr></sing>. <ety>[It. Cf. <er>Comfit</er>.]</ety> <sn>1.</sn> <def>Bonbons; sweetmeats; confections.</def> <mark>[archaic]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <ety>[originally, plaster or paper imitations of, or substitutes for, bonbons (<ets>confetti{1}</ets>).]</ety> <def>small bits or streamers of brightly colored paper, thrown in celebration by carnival revelers, at weddings, etc.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Confidante</ent><br/
<ent>Confidant</ent><br/
<mhw>{ <hw>Con`fi*dant"</hw> <pr>(?)</pr>; 277), <pos>n. masc.</pos>, <hw>Con`fi*dante"</hw> <pr>(?; 277)</pr>, <pos>n. fem.</pos> }</mhw> <ety>[F. <ets>confident</ets>, <ets>confidente</ets>, formerly also spelt <ets>confidant</ets>, <ets>confidante</ets>. See <er>Confide</er>, and cf. <er>Confident</er>.]</ety> <def>One to whom secrets, especially those relating to affairs of love, are confided or intrusted; a confidential or bosom friend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You love me for no other end<br/
Than to become my <qex>confidant</qex> and friend;<br/
As such I keep no secret from your sight.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confide</ent><br/
<hw>Con*fide"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confided</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confiding</conjf>.]</vmorph> <ety>[L. <ets>confidere</ets>; <ets>con-</ets> + <ets>fidere</ets> to trust. See <er>Faith</er>, and cf. <er>Affiance</er>.]</ety> <def>To put faith (<xex>in</xex>); to repose confidence; to trust; -- usually followed by <xex>in</xex>; <as>as, the prince <ex>confides</ex> in his ministers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By thy command I rise or fall,<br/
In thy protection I <qex>confide</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Judge before friendships, then <qex>confide</qex> till death.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confide</ent><br/
<hw>Con*fide"</hw>, <pos>v. t.</pos> <def>To intrust; to give in charge; to commit to one's keeping; -- followed by <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Congress may . . . <qex>confide</qex> to the Circuit jurisdiction of all offenses against the United States.</q> <rj><qau>Story.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confidence</ent><br/
<hw>Con"fi*dence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confidentia</ets> firm trust in, self-confidence: cf. F. <ets>confidence</ets>.]</ety> <sn>1.</sn> <def>The act of confiding, trusting, or putting faith in; trust; reliance; belief; -- formerly followed by <xex>of</xex>, now commonly by <xex>in</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Society is built upon trust, and trust upon <qex>confidence</qex> of one another's integrity.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A cheerful <qex>confidence in the mercy of God</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That in which faith is put or reliance had.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord shall be thy <qex>confidence</qex>.</q> <rj><qau>Prov. iii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of mind characterized by one's reliance on himself, or his circumstances; a feeling of self-sufficiency; such assurance as leads to a feeling of security; self-reliance; -- often with <xex>self</xex> prefixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your wisdom is consumed in <qex>confidence</qex>;<br/
Do not go forth to-day.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But <qex>confidence</qex> then bore thee on secure<br/
Either to meet no danger, or to find<br/
Matter of glorious trial.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Private conversation; (<pluf>pl.</pluf>) secrets shared; <as>as, there were <ex>confidences</ex> between them</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sir, I desire some <qex>confidence</qex> with you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Confidence game</b></col>, <cd>any swindling operation in which advantage is taken of the confidence reposed by the victim in the swindler; several swindlers often work together to create the illusion of truth; -- also called <altname>con game</altname>.</cd> -- <col><b>Confidence man</b></col>, <cd>a swindler.</cd> -- <col><b>To take into one's confidence</b></col>, <cd>to admit to a knowledge of one's feelings, purposes, or affairs.</cd></cs></p>

<p><syn><b>Syn.</b> -- Trust; assurance; expectation; hope.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>confident</qex> that very much be done.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Trustful; without fear or suspicion; frank; unreserved.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be <qex>confident</qex> to speak, Northumberland;<br/
We three are but thyself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having self-reliance; bold; undaunted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>confident</qex> as is the falcon's flight<br/
Against a bird, do I with Mowbray fight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having an excess of assurance; bold to a fault; dogmatical; impudent; presumptuous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fool rageth and is <qex>confident</qex>.</q> <rj><qau>Prov. xiv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Giving occasion for confidence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The cause was more <qex>confident</qex> than the event was prosperous.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confident</ent><br/
<hw>Con"fi*dent</hw>, <pos>n.</pos> <def>See <er>Confidant</er>.</def>  <rj><au>South.</au>  <au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confidential</ent><br/
<hw>Con`fi*den"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>confidentiel</ets>.]</ety> <sn>1.</sn> <def>Enjoying, or treated with, confidence; trusted in; trustworthy; <as>as, a <ex>confidential</ex> servant or clerk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Communicated in confidence; secret.</def> <ldquo/<xex>Confidential</xex> messages.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Confidential communication</b></col> <fld>(Law)</fld> <cd>See <cref>Privileged communication</cref>, under <er>Privileged</er>.</cd> -- <col><b>Confidential creditors</b></col>, <cd>those whose claims are of such a character that they are entitled to be paid before other creditors.</cd> -- <col><b>Confidential debts</b></col>, <cd>debts incurred for borrowed money, and regarded as having a claim to be paid before other debts.</cd>  <rj><au>McElrath.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>confidentiality</ent><br/
<hw>confidentiality</hw> <pos>n.</pos> <def>the state or attribute of being secret; privacy; <as>as, you must respect the <ex>confidentiality</ex> of your client's communications</as>.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>discretion in keeping secret information.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Confidentially</ent><br/
<hw>Con`fi*den"tial*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In confidence; in reliance on secrecy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confidently</ent><br/
<hw>Con"fi*dent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With confidence; with strong assurance; positively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confidentness</ent><br/
<hw>Con"fi*dent*ness</hw>, <pos>n.</pos> <def>The quality of being confident.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confider</ent><br/
<hw>Con*fid"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who confides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiding</ent><br/
<hw>Con*fid"ing</hw>, <pos>a.</pos> <def>That confides; trustful; unsuspicious.</def> -- <wordforms><wf>Con*fid"ing*ly</wf>, <pos>adv.</pos> -- <wf>Con*fid"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Configurate</ent><br/
<hw>Con*fig"ur*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>configuratus</ets>, p. p. of <ets>configurare</ets> to form or after; <ets>con-</ets> + <ets>figurare</ets> to form, <ets>figura</ets> form. See <er>Figure</er>.]</ety> <def>To take form or position, as the parts of a complex structure; to agree with a pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Known by the name of uniformity;<br/
Where pyramids to pyramids relate<br/
And the whole fabric doth <qex>configurate</qex>.</q> <rj><qau>Jordan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Configuration</ent><br/
<hw>Con*fig`u*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>configuratio</ets>.]</ety> <sn>1.</sn> <def>Form, as depending on the relative disposition of the parts of a thing; shape; figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is the variety of <qex>configurations</qex> [of the mouth] . . . which gives birth and origin to the several vowels.</q> <rj><qau>Harris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astrol.)</fld> <def>Relative position or aspect of the planets; the face of the horoscope, according to the relative positions of the planets at any time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They [astrologers] undertook . . . to determine the course of a man's character and life from the <qex>configuration</qex> of the stars at the moment of his birth.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>configurationism</ent><br/
<hw>configurationism</hw> <pos>n.</pos> <fld>(Psychol.)</fld> <def>an approach to psychology that emphasizes the importance of configurational properties.</def><br/
<syn><b>Syn. --</b> Gestalt psychology.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Configure</ent><br/
<hw>Con*fig"ure</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Configured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Configuring</conjf>.]</vmorph> <ety>[L. <ets>configurare</ets>: cf. F. <ets>configurer</ets>. See <er>Configurate</er>.]</ety> <def>To arrange or dispose in a certain form, figure, or shape.</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confinable</ent><br/
<hw>Con*fin"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being confined, restricted, or limited.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not <qex>confinable</qex> to any limits.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confine</ent><br/
<hw>Con*fine"</hw> <pr>(k<ocr/n*f<imac/n")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Confining</conjf>.]</vmorph> <ety>[F. <ets>confiner</ets> to border upon, LL. <ets>confinare</ets> to set bounds to; <ets>con-</ets> + <ets>finis</ets> boundary, end. See <er>Final</er>, <er>Finish</er>.]</ety> <def>To restrain within limits; to restrict; to limit; to bound; to shut up; to inclose; to keep close.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now let not nature's hand<br/
Keep the wild flood <qex>confined</qex>! let order die!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is to <qex>confine</qex> himself to the compass of numbers and the slavery of rhyme.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be confined</b></col>, <cd>to be in childbed.</cd></cs></p>

<p><syn><b>Syn.</b> -- To bound; limit; restrain; imprison; immure; inclose; circumscribe; restrict.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confine</ent><br/
<hw>Con"fine</hw> <pr>(? <or/ <?/); 277)</pr>, <pos>v. i.</pos> <def>To have a common boundary; to border; to lie contiguous; to touch; -- followed by <xex>on</xex> or <xex>with</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Where your gloomy bounds<br/
<qex>Confine</qex> with heaven.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bewixt heaven and earth and skies there stands a place.<br/
<qex>Confining</qex> on all three.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confine</ent><br/
<hw>Con"fine</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Common boundary; border; limit; -- used chiefly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Events that came to pass within the <qex>confines</qex> of Judea.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And now in little space<br/
The <qex>confines</qex> met of empyrean heaven,<br/
And of this world.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On the <qex>confines</qex> of the city and the Temple.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Apartment; place of restraint; prison.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Confines</qex>, wards, and dungeons.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The extravagant and erring spirit hies<br/
To his <qex>confine</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>confined</ent><br/
<hw>confined</hw> <pos>adj.</pos> <sn>1.</sn> <def>having movement restricted to within a certain area; -- usually a building.  Opposite of <ant>unconfined</ant>.</def> <note>[Narrower terms: <stype>claustrophobic</stype>; <stype>close, confining</stype>; <stype>homebound, housebound, shut-in</stype>; <stype>in childbed(prenominal)</stype>; <stype>pent, shut up(predicate)</stype>; <stype>snowbound</stype>; <stype>weather-bound</stype>; <stype>stormbound, storm-bound</stype>]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>deprived of liberty; especially placed under arrest or restraint.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>having movement restricted to within an enclosed outdoor area; -- of animals.</def><br/
<syn><b>Syn. --</b> fenced in, penned.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld>  <def>not invading healthy tissue.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn>  <def>held prisoner.</def><br/
<syn><b>Syn. --</b> captive, imprisoned, jailed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn> <def>having movement or progress restricted to a certain area; <as>as, an outbreak of the plague <ex>confined</ex> to one quarter of the city; wildfires <ex>confined</ex> to within the canyon</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Confineless</ent><br/
<hw>Con"fine`less</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos>  <def>Without limitation or end; boundless.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confinement</ent><br/
<hw>Con*fine"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Restraint within limits; imprisonment; any restraint of liberty; seclusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind hates restraint, and is apt to fancy itself under <qex>confinement</qex> when the sight is pent up.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Restraint within doors by sickness, esp. that caused by childbirth; lying-in.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiner</ent><br/
<hw>Con*fin"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, limits or restrains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiner</ent><br/
<hw>Con"fi`ner</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <def>One who lives on confines, or near the border of a country; a borderer; a near neighbor.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Happy <qex>confiners</qex> you of other lands,<br/
That shift your soil, and oft 'scape tyrants' hands.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confinity</ent><br/
<hw>Con*fin"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>confinit<eacute/</ets>.]</ety> <def>Community of limits; contiguity.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirm</ent><br/
<hw>Con*firm"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confrmed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Confirming</conjf>.]</vmorph> <ety>[OE. <ets>confermen</ets>, <ets>confirmen</ets>, OF. <ets>confermer</ets>, F. <ets>confirmer</ets>, fr. L. <ets>confirmare</ets>; <ets>con-</ets> + <ets>firmare</ets> to make firm, fr. <ets>firmus</ets> firm. See <er>Firm</er>.]</ety> <sn>1.</sn> <def>To make firm or firmer; to add strength to; to establish; <as>as, health is <ex>confirmed</ex> by exercise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Confirm</qex> the crown to me and to mine heirs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>confirmed</qex> the same unto Jacob for a law.</q> <rj><qau>Ps. cv. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strengthen in judgment or purpose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Confirmed</qex>, then, I resolve<br/
Adam shall share with me in bliss or woe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give new assurance of the truth of; to render certain; to verify; to corroborate; <as>as, to <ex>confirm</ex> a rumor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your eyes shall witness and <qex>confirm</qex> my tale.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These likelihoods <qex>confirm</qex> her flight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To render valid by formal assent; to complete by a necessary sanction; to ratify; <as>as, to <ex>confirm</ex> the appoinment of an official; the Senate <ex>confirms</ex> a treaty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That treaty so prejudicial ought to have been remitted rather than <qex>confimed</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Eccl.)</fld> <def>To administer the rite of confirmation to. See <er>Confirmation</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those which are thus <qex>confirmed</qex> are thereby supposed to be fit for admission to the sacrament.</q> <rj><qau>Hammond.</qau></rj></p>

<p><syn><b>Syn.</b> -- To strengthen; corroborate; substantiate; establish; fix; ratify; settle; verify; assure.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 301 --></p>

<p><ent>Confirmable</ent><br/
<hw>Con*firm"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be confirmed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmance</ent><br/
<hw>Con*firm"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Confirmation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmation</ent><br/
<hw>Con`fir*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confirmation</ets>, L. <ets>confirmatio</ets>.]</ety> <sn>1.</sn> <def>The act of confirming or strengthening; the act of establishing, ratifying, or sanctioning; <as>as, the <ex>confirmation</ex> of an appointment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their blood is shed<br/
In <qex>confirmation</qex> of the noblest claim.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which confirms; that which gives new strength or assurance; as to a statement or belief; additional evidence; proof; convincing testimony.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trifles light as air<br/
Are to the jealous <qex>confirmations</qex> strong<br/
As proofs of holy writ.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>A rite supplemental to baptism, by which a person is admitted, through the laying on of the hands of a bishop, to the full privileges of the church, as in the Roman Catholic, the Episcopal Church, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This ordinance is called <qex>confirmation</qex>, because they who duly receive it are confirmed or strengthened for the fulfillment of their Christian duties, by the grace therein bestowed upon them.</q> <rj><qau>Hook.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>A conveyance by which a voidable estate is made sure and not voidable, or by which a particular estate is increased; a contract, express or implied, by which a person makes that firm and binding which was before voidable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmative</ent><br/
<hw>Con*firm"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>confirmativus</ets>: cf. F. <ets>confirmatif</ets>.]</ety> <def>Tending to confirm or establish.</def>  <rj><au>Sherwood.</au></rj></p>

<p>-- <wordforms><wf>Con*firm"a*tive*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmator</ent><br/
<hw>Con"fir*ma`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who, or that which, confirms; a confirmer.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmatory</ent><br/
<hw>Con*firm"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to confirm; corroborative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fact <qex>confirmatory</qex> of the conclusion.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to the rite of confirmation.</def>  <rj><au>Compton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmedly</ent><br/
<hw>Con*firm"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With confirmation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmedness</ent><br/
<hw>Con*firm"ed*ness</hw>, <pos>n.</pos> <def>A fixed state.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmee</ent><br/
<hw>Con`fir*mee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confirm<eacute/</ets>, p. p. of <ets>confirmer</ets>.]</ety> <fld>(Law)</fld> <def>One to whom anything is confirmed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmer</ent><br/
<hw>Con*firm"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, confirms, establishes, or ratifies; one who corroborates.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confirmingly</ent><br/
<hw>Con*firm"ing*ly</hw>, <pos>adv.</pos> <def>In a confirming manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiscable</ent><br/
<hw>Con*fis"ca*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>confiscable</ets>.]</ety> <def>Capable of being confiscated; liable to forfeiture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiscate</ent><br/
<hw>Con"fis*cate</hw> <pr>(? <or/ <?/)</pr>, <pos>a.</pos> <ety>[L. <ets>confiscatus</ets>, p. p. of <ets>confiscare</ets> to confiscate, prop., to lay up in a chest; <ets>con-</ets> + <ets>fiscus</ets> basket, purse, treasury. See <er>Fiscal</er>.]</ety> <def>Seized and appropriated by the government to the public use; forfeited.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest that your goods too soon be <qex>confiscate</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiscate</ent><br/
<hw>Con"fis*cate</hw> <pr>(? <or/ <?/; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confiscated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confiscating</conjf>.]</vmorph> <def>To seize as forfeited to the public treasury; to appropriate to the public use.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was judged that he should be banished and his whole estate <qex>confiscated</qex> and seized.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>confiscated</ent><br/
<hw>confiscated</hw> <pos>adj.</pos>  <def>taken without permission or consent, especially by or as if by a public authority; <as>as, the <ex>confiscated</ex> liquor was poured down the drain; teh customs agents <ex>confiscated</ex> the banned fruits</as>.</def><br/
<syn><b>Syn. --</b> appropriated, confiscate, seized, taken over.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Confiscation</ent><br/
<hw>Con`fis*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confiscatio</ets>.]</ety> <def>The act or process of taking property or condemning it to be taken, as forfeited to the public use.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>confiscations</qex> following a subdued rebellion.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiscator</ent><br/
<hw>Con"fis*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a treasurer.]</ety> <def>One who confiscates.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiscatory</ent><br/
<hw>Con*fis"ca*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Effecting confiscation; characterized by confiscations.</def> <ldquo/<xex>Confiscatory</xex> and exterminatory periods.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confit</ent><br/
<hw>Con"fit</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Comfit</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confitent</ent><br/
<hw>Con"fi*tent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confitens</ets>, p. pr.]</ety> <def>One who confesses his sins and faults.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiteor</ent><br/
||<hw>Con*fit"e*or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., I confess. See <er>Confess</er>.]</ety> <fld>(R.C.Ch.)</fld> <def>A form of prayer in which public confession of sins is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confiture</ent><br/
<hw>Con"fi*ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[F. See <er>Confiture</er>.]</ety> <def>Composition; preparation, as of a drug, or confection; a sweetmeat.</def> <mark>[Obs.]</mark> <ldquo/<xex>Confitures</xex> and pies.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confix</ent><br/
<hw>Con*fix"</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confixed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> Confixing.]</vmorph> <ety>[L. <ets>confixus</ets>, p. p. of <ets>configere</ets> to fasten together.]</ety> <def>To fix; to fasten.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confixure</ent><br/
<hw>Con*fix"ure</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>Act of fastening.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conflagrant</ent><br/
<hw>Con*fla"grant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conflagrans</ets>, p. pr. of <ets>conflagrare</ets>; <ets>con-</ets> + <ets>flagrare</ets> to blaze. See <er>Flagrant</er>.]</ety> <def>Burning together in a common flame.</def> <mark>[R.]</mark> <ldquo/The <er>conflagrant</er> mass.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conflagration</ent><br/
<hw>Con`fla*gra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conflagratio</ets>: cf. F. <ets>conflagration</ets>.]</ety> <def>A fire extending to many objects, or over a large space; a general burning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till one wide <qex>conflagration</qex> swallows all.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conflate</ent><br/
<hw>Con*flate"</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conflated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conflating</conjf>.]</vmorph> <ety>[L. <ets>conflatus</ets>, p. p. of <ets>conflare</ets> to blow together; <ets>con-</ets> + <ets>flare</ets> to blow.]</ety> <sn>1.</sn> <def>To blow together; to bring together; to collect;  to fuse together; to join or weld; to consolidate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The State-General, created and <qex>conflated</qex> by the passionate effort of the whole nation.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to ignore distinctions between, by treating two or more distinguishable objects or ideas as one; to confuse.</def><br/
[<source>PJC</source>]</p>

<p><ent>Conflation</ent><br/
<hw>Con*fla"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conflatio</ets>.]</ety> <sn>1.</sn> <def>A blowing together, as of many instruments in a concert, or of many fires in a foundry.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a fusing together; merger of two or more things or ideas into one.</def><br/
[<source>PJC</source>]</p>

<p><ent>Conflict</ent><br/
<hw>Con"flict</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conflictus</ets> a striking together, fr. <ets>confligere</ets>, <ets>-flictum</ets>, to strike together, to fight: cf. F. <ets>conflit</ets>, formerly also <ets>conflict</ets>. See <er>Conflict</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>A striking or dashing together; violent collision; <as>as, a <ex>conflict</ex> of elements or waves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>A strife for the mastery; hostile contest; battle; struggle; fighting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As soon as he [Atterbury] was himself again, he became eager for action and <qex>conflict</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An irrepressible <qex>conflict</qex> between opposing and enduring forces.</q> <rj><qau>W. H. Seward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conflict of laws</b></col>, <cd>that branch of jurisprudence which deals with individual litigation claimed to be subject to the conflicting laws of two or more states or nations; -- often used as synonymous with <altname>Private international law</altname>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Contest; collision; struggle; combat; strife; contention; battle; fight; encounter. See <er>Contest</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conflict</ent><br/
<hw>Con*flict"</hw> <pr>(?)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conflicted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conflicting</conjf>.]</vmorph> <ety>[L. <ets>conflictus</ets>, p. p. of <ets>confligere</ets> to conflict (cf. <ets>conflictare</ets>); <ets>con-</ets> + <ets>fligere</ets> to strike; cf. Gr. <grk>fli`bein</grk>, <grk>qli`bein</grk>, to press, L. <ets>flagrum</ets> whip.]</ety> <sn>1.</sn> <def>To strike or dash together; to meet in violent collision; to collide.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fire and water <qex>conflicting</qex> together.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>To maintain a conflict; to contend; to engage in strife or opposition; to struggle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man would be content to . . . <qex>conflict</qex> with great difficulties, in hopes of a mighty reward.</q> <rj><qau>Abp. Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>To be in opposition; to be contradictory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The laws of the United States and of the individual States may, in some cases, <qex>conflict</qex> with each other.</q> <rj><qau>Wheaton.</qau></rj></p>

<p><syn><b>Syn.</b> -- To fight; contend; contest; resist; struggle; combat; strive; battle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conflicting</ent><br/
<hw>Con*flict"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Being in conflict or collision, or in opposition; contending; contradictory; incompatible; contrary; opposing; marked by discord.</def><br/
<syn><b>Syn. --</b> antagonistic, at odds(predicate), clashing.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><q>Torn with sundry <qex>conflicting</qex> passions.</q> <rj><qau>Bp. Hurd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>in disagreement; -- of facts or theories.</def><br/
<syn><b>Syn. --</b> at odds(predicate), contradictory, self-contradictory.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conflictive</ent><br/
<hw>Con*flict"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to conflict; conflicting.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confluence</ent><br/
<hw>Con"flu*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confluentia</ets>.]</ety> <sn>1.</sn> <def>The act of flowing together; the meeting or junction of two or more streams; the place of meeting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>New York stood at the <qex>confluence</qex> of two rivers.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>Any running together of separate streams or currents; the act of meeting and crowding in a place; hence, a crowd; a concourse; an assemblage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You see this <qex>confluence</qex>, this great flood of vistors.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>The confluence</qex> . . . of all true joys.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confluent</ent><br/
<hw>Con"flu*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>confluens</ets>, <ets>-entis</ets>, p. pr. of <ets>confluere</ets>, <ets>-fluxum</ets>; <ets>con-</ets> + <ets>fluere</ets> to flow.  See <er>Fluent</er>.]</ety> <sn>1.</sn> <def>Flowing together; meeting in their course; running one into another; flowing together to form a single stream.</def><br/
<syn><b>Syn. --</b> merging(prenominal).</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>confluent</qex> steams make some great river's head.</q> <rj><qau>Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Bot.)</fld> <def>Blended into one; growing together, so as to obliterate all distinction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <fld>(Med.)</fld> <sd>(a)</sd> <def>Running together or uniting, as pimples or pustules.</def> <sd>(b)</sd> <def>Characterized by having the pustules, etc., run together or unite, so as to cover the surface; <as>as, <ex>confluent</ex> smallpox</as>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confluent</ent><br/
<hw>Con"flu*ent</hw>, <pos>n.</pos> <sn>1.</sn> <def>A small steam which flows into a large one.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>The place of meeting of steams, currents, etc.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conflux</ent><br/
<hw>Con"flux</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From L. <ets>confluxus</ets>, p. p. See <er>Confluent</er>.]</ety> <sn>1.</sn> <def>A flowing together; a meeting of currents.</def> <ldquo/The <xex>conflux</xex> of meeting sap.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The general <qex>conflux</qex> and concourse of the whole people.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>A large assemblage; a passing multitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To the gates cast round thine eye, and see<br/
What <qex>conflux</qex> issuing forth, or entering in.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confluxibility</ent><br/
<hw>Con*flux`i*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The  tendency of fluids to run together.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confluxible</ent><br/
<hw>Con*flux"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Inclined to flow or run together.</def> -- <wordforms><wf>Con*flux"i*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confocal</ent><br/
<hw>Con*fo"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Having the same foci; <as>as, <ex>confocal</ex> quadrics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conform</ent><br/
<hw>Con*form"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conformis</ets>; <ets>con-</ets> + <ets>forma</ets> form: cf. F. <ets>conforme</ets>.]</ety> <def>Of the same form; similar in import; conformable.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Care must be taken that the interpretation be every way <qex>conform</qex> to the analogy of faith.</q> <rj><qau>Bp.Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conform</ent><br/
<hw>Con*form"</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conformed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conforming</conjf>.]</vmorph> <ety>[F. <ets>conformer</ets>, L. <ets>conformare</ets>, <ets>-formatum</ets>; <ets>con-</ets> + <ets>formare</ets> to form, <ets>forma</ets> form. See <er>Form</er>.]</ety> <def>To shape in accordance with; to make like; to bring into harmony or agreement with; -- usually with <xex>to</xex> or <xex>unto</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Demand of them wherefore they <qex>conform</qex> not themselves unto the order of the church.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conform</ent><br/
<hw>Con*form"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be in accord or harmony; to comply; to be obedient; to submit; -- with <xex>to</xex> or <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A rule to which experience must <qex>conform</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Eng. Eccl. Hist.)</fld> <def>To comply with the usages of the Established Church; to be a conformist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>About two thousand ministers whose consciences did not suffer them to <qex>conform</qex> were driven from their benefices in a day.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformability</ent><br/
<hw>Con*form`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state of being conformable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Geol.)</fld> <def>The parallelism of two sets of strata which are in contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformable</ent><br/
<hw>Con*form"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Corresponding in form, character, opinions, etc.; similar; like; consistent; proper or suitable; -- usually followed by <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fragments of Sappho give us a taste of her way of writing perfectly <qex>conformable</qex> with that character.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have been to you a true and humble wife, at all times to your will <qex>conformable</qex></q>  <qau>Shakespeare</qau></p>

<p><q><qex>Conformable</qex> to Scripture as well as to philosophy.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To make matters somewhat <qex>conformable</qex> for the old knight.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disposed to compliance or obedience; ready to follow directions; quick to comply; submissive; compliant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have been to you a true and humble wife,<br/
At all times to your will <qex>conformable</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <fld>(Geol.)</fld> <def>Parallel, or nearly so; -- said of strata in contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformableness</ent><br/
<hw>Con*form"a*ble*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being conformable; conformability.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformably</ent><br/
<hw>Con*form"a*bly</hw>, <pos>adv.</pos> <def>With conformity or in conformity; suitably; agreeably.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conformably</qex> to the law and nature of God.</q> <rj><qau>Bp. Beveridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformance</ent><br/
<hw>Con*form"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Conformity.</def> <mark>[R.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformate</ent><br/
<hw>Con*form"ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conformatus</ets>, p. p.  See <er>Conform</er>.]</ety> <def>Having the same form.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformation</ent><br/
<hw>Con`for*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. conformatio: cf. F. <ets>conformation</ets>.]</ety> <sn>1.</sn> <def>The act of conforming; the act of producing conformity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conformation</qex> of our hearts and lives to the duties of true religion and morality.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>The state of being conformed; agreement; hence; structure, as depending on the arrangement of parts; form; arrangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In Hebrew poetry, there may be observed a certain <qex>conformation</qex> of the sentences.</q> <rj><qau>Lowth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A structure and <qex>conformation</qex> of the earth.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformator</ent><br/
<hw>Con"for*ma`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a framer.]</ety> <def>An apparatus for taking the conformation of anything, as of the head for fitting a hat, or, in craniometry, finding the largest horizontal area of the head.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Conformer</ent><br/
<hw>Con*form"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conforms; one who complies with established forms or doctrines.</def><br/
[<source>1913 Webster</source>]</p>




<p><ent>conforming</ent><br/
<hw>conforming</hw> <pos>adj.</pos> <sn>1.</sn> <def>adhering to established customs or doctrines, especially in religion, as e.g. Orthodox Judaism.</def> [Narrower terms: <stype>orthodox (vs. unorthodox)</stype>]<br/
<syn><b>Syn. --</b> conformist.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conformism</ent><br/
<hw>conformism</hw> <pos>n.</pos> <sn>1.</sn> <def>orthodoxy in thoughts and belief.</def><br/
<syn><b>Syn. --</b> conformity.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conformist</ent><br/
<hw>conformist</hw> <pos>adj.</pos> <sn>1.</sn> <def>marked by conformity or convention; not corresponding to current customs or rules or styles; <as>as, underneath the radical image teenagers are surprisingly <ex>conformist</ex></as>.  Opposite of <ant>nonconformist</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>same as <er>conforming</er>.</def> <note>[Narrower terms: <stype>orthodox (vs. unorthodox)</stype>]</note><br/
<syn><b>Syn. --</b> conforming.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conformist</ent><br/
<hw>Con*form"ist</hw>, <pos>n.</pos> <def>One who conforms or complies; esp., one who conforms to the Church of England, or to the Established Church, as distinguished from a <contr>dissenter</contr> or <ant>nonconformist</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A cheeful <qex>conformist</qex> to your judgment.</q> <rj><qau>Jer.Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conformity</ent><br/
<hw>Con*form"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conformities</plw> <pr>(#)</pr></plu>. <ety>[Cf. F. <ets>conformit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Correspondence in form, manner, or character; resemblance; agreement; congruity; -- followed by <xex>to</xex>, <xex>with</xex>, or <xex>between</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By our <qex>conformity</qex> to God.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The end of all religion is but to draw us to a <qex>conformity</qex> with God.</q> <rj><qau>Dr. H.More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>conformity</qex> between the mental taste and the sensitive taste.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Eng. Eccl. Hist.)</fld> <def>Compliance with the usages of the Established Church.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king [James I.] soon afterward put forth a proclamation requiring all ecclesiastical and civil officers to do their duty by enforcing <qex>conformity</qex>.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confortation</ent><br/
<hw>Con`for*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>confortation</ets>, LL. <ets>confortatio</ets>. Cf. <er>Comfort</er>.]</ety> <def>The act of strengthening.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confound</ent><br/
<hw>Con*found"</hw> <pr>(k<ocr/n*found")</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confounded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confounding</conjf>.]</vmorph> <ety>[F. <ets>confondre</ets>, fr. L. <ets>confundere</ets>, <ets>-fusum</ets>, to pour together; <ets>con-</ets> + <ets>fundere</ets> to pour. See <er>Fuse</er> to melt, and cf. <er>Confuse</er>.]</ety> <sn>1.</sn> <def>To mingle and blend, so that different elements can not be distinguished; to confuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They who strip not ideas from the marks men use for them, but <qex>confound</qex> them with words, must have endless dispute.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us go down, and there <qex>confound</qex> their language.</q> <rj><qau>Gen. xi. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>To mistake for another; to identify falsely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They [the tinkers] were generally vagrants and pilferers, and were often <qex>confounded</qex> with the gypsies.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>To throw into confusion or disorder; to perplex; to strike with amazement; to dismay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gods <qex>confound</qex>...<br/
The Athenians both within and out that wall.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They trusted in thee and were not <qex>confounded</qex>.</q> <rj><qau>Ps. xxii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So spake the Son of God, and Satan stood<br/
A while as mute, <qex>confounded</qex> what to say.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn>  <def>To destroy; to ruin; to waste.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>One man's lust these many lives <qex>confounds</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How couldst thou in a mile <qex>confound</qex> an hour?</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To abash; confuse; baffle; dismay; astonish; defeat; terrify; mix; blend; intermingle. See <er>Abash</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confounded</ent><br/
<hw>Con*found"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Confused; perplexed; unclear in mind or intent; bewildered.</def><br/
<syn><b>Syn. --</b> at sea, befuddled, bemused, bewildered, confused, mazed, mixed-up.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><q>A cloudy and <qex>confounded</qex> philosopher.</q> <rj><qau>Cudworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>Excessive; extreme; abominable.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was a most <qex>confounded</qex> tory.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The tongue of that <qex>confounded</qex> woman.</q> <rj><qau>Sir. W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confoundedly</ent><br/
<hw>Con*found"ed*ly</hw>, <pos>adv.</pos> <def>Extremely; odiously; detestably.</def> <mark>[Colloq.]</mark> <ldquo/<xex>Confoundedly</xex> sick.<rdquo/  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confoundedness</ent><br/
<hw>Con*found"ed*ness</hw>, <pos>n.</pos> <def>The state of being confounded.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their witty descant of my <qex>confoundedness</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confounder</ent><br/
<hw>Con*found"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who confounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>confounding</ent><br/
<hw>confounding</hw> <pos>adj.</pos> <def>tending to contradict (a hypothesis).</def><br/
<syn><b>Syn. --</b> contradictory.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>confounding</ent><br/
<hw>confounding</hw> <pos>n.</pos> <def>a mistake that results from taking one thing to be another.</def><br/
<syn><b>Syn. --</b> confusion, mix-up.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Confract</ent><br/
<hw>Con"fract`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>confractus</ets>, p. p. of <ets>confringere</ets>.]</ety> <def>Broken in pieces; severed.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confragose</ent><br/
<hw>Con`fra*gose"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>confragosus</ets>; <ets>con-</ets> + <ets>fragosus</ets>, fr. <ets>frangere</ets>. See <er>Fragile</er>.]</ety> <def>Broken; uneven.</def> <mark>[Obs.]</mark> <ldquo/<xex>Confragose</xex> cataracts.<rdquo/  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confraternity</ent><br/
<hw>Con`fra*ter"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Confraternities</plw> <pr>(#)</pr></plu>. <ety>[LL. <ets>confraternitas</ets>: cf. F. <ets>confraternit<eacute/</ets>. See <er>Fraternity</er>.]</ety> <def>A society or body of men united for some purpose, or in some profession; a brotherhood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These live in one society and <qex>confraternity</qex>.</q> <rj><qau>Stow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confrere</ent><br/
||<hw>Con`frere"</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Fellow member of a fraternity; intimate associate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confrication</ent><br/
<hw>Con`fri*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confricatio</ets>, fr. <ets>confricare</ets> to rub vigorously.]</ety> <def>A rubbing together; friction.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confrier</ent><br/
<hw>Con*fri"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>confr<egrave/re</ets>. See <er>Friar</er>.]</ety> <def>A confr<egrave/re.</def> <mark>[Obs.]</mark>  <rj><au>Weever.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confront</ent><br/
<hw>Con*front"</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confronted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confronting</conjf>.]</vmorph> <ety>[F. <ets>confronter</ets>; L. <ets>con-</ets> + <ets>frons</ets> the forehead or front. See <er>Front</er>.]</ety> <sn>1.</sn> <def>To stand facing or in front of; to face; esp. to face hostilely; to oppose with firmness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We four, indeed, <qex>confronted</qex> were with four<br/
In Russian habit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He spoke and then <qex>confronts</qex> the bull.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hester caught hold of Pearl, and drew her forcibly into her arms, <qex>confronting</qex> the old Puritan magistrate with almost a fierce expression.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was impossible at once to <qex>confront</qex> the might of France and to trample on the liberties of England.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>To put face to face; to cause to face or to meet; <as>as, to <ex>confront</ex> one with the proofs of his wrong doing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>To set in opposition for examination; to put in contrast; to compare.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When I <qex>confront</qex> a medal with a verse, I only show you the same design executed by different hands.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 302 --></p>

<p><ent>Confrontation</ent><br/
<hw>Con`fron*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>confrontatio</ets>.]</ety> <def>Act of confronting.</def>  <rj><au>H. Swinburne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confronte</ent><br/
||<hw>Con`fron`t<eacute/"</hw> <pr>(<?/)</pr>, <pos>a.</pos> <ety>[F., p. p. <er>confronter</er>.]</ety> <fld>(Her.)</fld> <def>Same as <er>Affront<eacute/</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confronter</ent><br/
<hw>Con*front"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who confronts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>confronter</qex> in authority.</q> <rj><qau>Speed.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confrontment</ent><br/
<hw>Con*front"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of confronting; the state of being face to face.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confrontment</ent><br/
<hw>Con*front"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of confronting; the state of being face to face.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confucian</ent><br/
<hw>Con*fu"cian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, or relating to, Confucius, the great Chinese philosopher and teacher, or to Confucianism; <as>as, <ex>Confucian</ex> ethics</as>.</def> -- <def2><pos>n.</pos>  <def>A Confucianist.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confucianism</ent><br/
<hw>Con*fu"cian*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The political morality taught by Confucius and his disciples, which forms the basis of the Chinese jurisprudence and education. It can hardly be called a religion, as it does not inculcate the worship of any god.</def>  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confucianist</ent><br/
<hw>Con*fu"cian*ist</hw>, <pos>n.</pos> <def>A follower of Confucius; a Confucian.</def>  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confucius</ent><br/
<hw>Confucius</hw> <pos>n.</pos> <def>a Chinese philosopher (circa 551-478 BC), the founder of Confucianism.</def><br/
<syn><b>Syn. --</b> Kung futzu, Kung futze, Kung tzu, K'ung fu-tze.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Confus</ent><br/
<hw>Con*fus</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. See <er>Confuse</er>, <pos>a.</pos>]</ety> <def>Confused, disturbed.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confusability</ent><br/
<hw>Con*fus`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Capability of being confused.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confusable</ent><br/
<hw>Con*fus"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being confused.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confuse</ent><br/
<hw>Con*fuse"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>confus</ets>, L. <ets>confusus</ets>, p. p. of <ets>confundere</ets>. See <er>Confound</er>.]</ety> <def>Mixed; confounded.</def> <mark>[Obs.]</mark>  <rj><au>Baret.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confuse</ent><br/
<hw>Con*fuse"</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Confusing</conjf>.]</vmorph> <sn>1.</sn> <def>To mix or blend so that things can not be distinguished; to jumble together; to confound; to render indistinct or obscure; <as>as, to <ex>confuse</ex> accounts</as>; to <xex>confuse</xex> one's vision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A universal hubbub wild<br/
Of stunning sounds and voices all <qex>confused</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>To perplex; to disconcert; to abash; to cause to lose self-possession.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor thou with shadowed hint <qex>confuse</qex><br/
A life that leads melodious days.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Confused</qex> and sadly she at length replied.</q> <rj><qau>Pope.</qau></rj></p>

<p><syn><b>Syn.</b> -- To abash; disorder; disarrange; disconcert; confound; obscure; distract. See <er>Abash</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>confused</ent><br/
<hw>confused</hw> <pos>adj.</pos> <sn>1.</sn>  <def>same as <er>confounded</er>; <as>as, bewildered and <ex>confused</ex></as>.</def><br/
<syn><b>Syn. --</b> at sea, befuddled, bemused, bewildered, confounded, mazed, mixed-up.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>lacking orderly continuity.</def><br/
<syn><b>Syn. --</b> disconnected, disjointed, disordered, disorganized, desultory, garbled, illogical, rambling, scattered, unconnected.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>thrown into disorder; <as>as, His workbench held a <ex>confused</ex> assortment of spare engine parts.</as>.</def><br/
<syn><b>Syn. --</b> disordered, in disarray, upset.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>having lost one's bearings physically or mentally.</def><br/
<syn><b>Syn. --</b> disoriented, lost.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn>  <def>not marked by fine distinctions.</def> <ant>discriminate</ant><br/
<syn><b>Syn. --</b> indiscriminate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn>  <def>causing bafflement and confusion.</def><br/
<syn><b>Syn. --</b> bewildering, confusing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Confusedly</ent><br/
<hw>Con*fus"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a confused manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confusedness</ent><br/
<hw>Con*fus"ed*ness</hw>, <pos>n.</pos> <def>A state of confusion.</def>  <rj><au>Norris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confusely</ent><br/
<hw>Con*fuse"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Confusedly; obscurely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>confusing</ent><br/
<hw>confusing</hw> <pos>adj.</pos> <sn>1.</sn> <def>causing mental confusion and perplexity.</def><br/
<syn><b>Syn. --</b> perplexing, stupefying.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>causing bafflement and confusion; <as>as, he sent <ex>confusing</ex> signals to Iraq</as>.</def><br/
<syn><b>Syn. --</b> bewildering, confused.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Confusion</ent><br/
<hw>Con*fu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>confusion</ets>, L. <ets>confusio</ets>.]</ety> <sn>1.</sn> <def>The state of being mixed or blended so as to produce indistinctness or error; indistinct combination; disorder; tumult.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>confusion</qex> of thought to which the Aristotelians were liable.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Moody beggars starving for a time<br/
Of pellmell havoc and <qex>confusion</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>The state of being abashed or disconcerted; loss self-possession; perturbation; shame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Confusion</qex> dwelt in every face<br/
And fear in every heart.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>Overthrow; defeat; ruin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ruin seize thee, ruthless king,<br/
<qex>Confusion</qex> on thy banners wait.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn>  <def>One who confuses; a confounder.</def> <mark>[Obs.]</mark>  <rj><au>Chapmen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Confusion of goods</b></col> <fld>(Law)</fld>, <cd>the intermixture of the goods of two or more persons, so that their respective portions can no longer be distinguished.</cd>  <rj><au>Blackstone.</au>  <au>Bouvier.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confusive</ent><br/
<hw>Con*fu"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Confusing; having a tendency to confusion.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confutable</ent><br/
<hw>Con*fut"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be confuted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A conceit . . . <qex>confutable</qex> by daily experience.</q> <rj><qau>Sir T.Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confutant</ent><br/
<hw>Con*fut"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confutans</ets>, p. pr. of <ets>confutare</ets>.]</ety> <def>One who undertakes to confute.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confutation</ent><br/
<hw>Con`fu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>confutatio</ets>: cf. F. <ets>confutation</ets>.]</ety> <def>The act or process of confuting; refutation.</def>  <ldquo/For the edification of some and the <xex>confutation</xex> of others.<rdquo/  <rj><au>Bp. Horne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confutative</ent><br/
<hw>Con*fut"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adapted or designed to confute.</def>  <rj><au>Bp. Warburton</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confute</ent><br/
<hw>Con*fute</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Confuted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Confuting</conjf>.]</vmorph> <ety>[L. <ets>confutare</ets> to chek (a boiling liquid), to repress, confute; <ets>con-</ets> + a root seen in <ets>futis</ets> a water vessel), prob. akin to <ets>fundere</ets> to pour: cf. F. <ets>confuter</ets>. See <er>Fuse</er> to melt.]</ety> <def>To overwhelm by argument; to refute conclusively; to prove or show to be false or defective; to overcome; to silence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Satan stood . . . <qex>confuted</qex> and convinced<br/
Of his weak arguing fallacious drift.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No man's error can be <qex>confuted</qex> who doth not . . . grant some true principle that contradicts his error.</q> <rj><qau>Chillingworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>confute</qex> a good profession with a bad conversation.</q> <rj><qau>Fuller.</qau></rj></p>

<p><syn><b>Syn.</b> -- To disprove; overthrow; sed aside; refute; oppugn.</syn> -- <usage>To <er>Confute</er>, <er>Refute.</er> <xex>Refute</xex> is literally to and decisive evidence; as, to <xex>refute</xex> a calumny, charge, etc. <xex>Confute</xex> is literally to check boiling, as when cold water is poured into hot, thus serving to allay, bring down, or neutralize completely. Hence, as applied to arguments (and the word is never applied, like <xex>refute</xex>, to charges), it denotes, to overwhelm by evidence which puts an end to the case and leaves an opponent nothing to say; to silence; as, <ldquo/the atheist is <xex>confuted</xex> by the whole structure of things around him.<rdquo/</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confutement</ent><br/
<hw>Con*fute"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Confutation.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Confuter</ent><br/
<hw>Con*fut"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who confutes or disproves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>confuting</ent><br/
<hw>confuting</hw> <pos>n.</pos> <def>the act of demonstrating that something is false; confutation.</def><br/
<syn><b>Syn. --</b> falsification, falsifying, disproving, refuting.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cong</ent><br/
<hw>Cong</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>An abbreviation of <er>Congius</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conga</ent><br/
<hw>conga</hw> <pos>n.</pos> <sn>1.</sn> <def>music composed for dancing the conga.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a Latin American dance of 3 steps and a kick by people in single file.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conga line</ent><br/
<hw>conga line</hw> <pos>n.</pos> <sn>1.</sn> <def>a line of persons dancing the conga{2}.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a line of persons similar to a <er>conga line{1}</er>.  Sometimes used figuratively or humorously.</def><br/
[<source>PJC</source>]</p>

<p><ent>con game</ent><br/
<hw>con game</hw> <pos>n.</pos> <def>Same as <cref>confidence game</cref>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Conge</ent><br/
||<hw>Con`g<eacute/"</hw> <pr>(k<ocir/N`zh<asl/"; E. k<ocr/n"j<emac/; 277)</pr>, <pos>n.</pos> <ety>[F., leave, permission, fr. L. <ets>commeatus</ets> a going back and forth, a leave of absence, furlough, fr. <ets>commeare</ets>, <ets>-meatum</ets>, to go and come; <ets>com-</ets> + <ets>meare</ets> to go. Cf. <er>Permeate</er>.]</ety> <altsp>[Formerly written <asp>congie</asp>.]</altsp> <sn>1.</sn> <def>The act of taking leave; parting ceremony; farewell; also, dismissal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Should she pay off old Briggs and give her her <qex>cong<eacute/</qex>?</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>The customary act of civility on any occasion; a bow or a courtesy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The captain salutes you with <qex>cong<eacute/</qex> profound.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <fld>(Arch.)</fld> <def>An apophyge.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Cong<eacute/ d'<eacute/lire</b></col> <pr>(<?/)</pr> <ety>[F., leave to choose]</ety> <fld>(Eccl.)</fld>, <cd>the sovereign's license or permission to a dean and chapter to choose as bishop the person nominated in the missive.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conge</ent><br/
<hw>Con"ge</hw> <pr>(k<ocr/n"j<emac/)</pr>, <pos>v. i.</pos> <ety>[Imp. & p. p. <er>Congeed</er> <pr>(k<ocr/n"j<emac/d)</pr>; <pos>p. pr. & vb. n.</pos> <er>Congeing</er>.]</ety> <ety>[OF. <ets>congier</ets>, <ets>congeer</ets>, F. <ets>cong<eacute/dier</ets>, fr. <ets>cong<eacute/</ets>. See <er>Cong<eacute/</er>, <pos>n.</pos>]</ety> <def>To take leave with the customary civilities; to bow ceremoniously, or courtesy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have <qex>congeed</qex> with the duke, done my adieu with his  nearest.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congeable</ent><br/
<hw>Con"ge*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(O. Eng. Law)</fld> <def>Permissible; done lawfully; <as>as, entry <ex>congeable</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congeal</ent><br/
<hw>Con*geal"</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Congealed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Congealing</conjf>.]</vmorph> <ety>[F. <ets>congeler</ets>, L. <ets>congelare</ets>, <ets>-gelatumn</ets>; <ets>con-</ets> + <ets>gelare</ets> to freeze, <ets>gelu</ets> frost. See <er>Gelid</er>.]</ety> <sn>1.</sn> <def>To change from a fluid to a solid state by cold; to freeze.</def><br/
<syn><b>Syn. --</b> jell, set.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>A vapory deluge lies to snow <qex>congealed</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>To affect as if by freezing; to check the flow of, or cause to run cold; to chill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As if with horror to <qex>congeal</qex> his blood.</q> <rj><qau>Stirling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congeal</ent><br/
<hw>Con*geal"</hw>, <pos>v. i.</pos> <def>To grow hard, stiff, or thick, from cold or other causes; to become solid; to freeze; to cease to flow; to run cold; to be chilled.</def><br/
<syn><b>Syn. --</b> jell, set.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest zeal, now melted . . .<br/
Cool and <qex>congeal again to what it was</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>congealed</ent><br/
<hw>congealed</hw> <pos>adj.</pos> <def>solidified; <as>as, <ex>congealed</ex> into jelly</as>.</def><br/
<syn><b>Syn. --</b> jelled, jellied.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Congealable</ent><br/
<hw>Con*geal"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>congelable</ets>.]</ety> <def>Capable of being congealed.</def> -- <wordforms><wf>Con*geal"a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congealedness</ent><br/
<hw>Con*geal"ed*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being congealed.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congealment</ent><br/
<hw>Con*geal"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or the process of congealing; congeliation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>That which is formed by congelation; a clot.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wash the <qex>congealment</qex> from your wounds.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congee</ent><br/
<hw>Con"gee</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Cong<eacute/</er>, <er>Conge</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And unto her his <qex>congee</qex> came to take.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congee</ent><br/
<hw>Con*gee"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <ety>[Tamil <ets>ka<ncir/shi</ets> boilings.]</ety> <def>Boiled rice; rice gruel.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>A jail; a lockup.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Congee discharges</b></col>, <cd>rice water discharges.</cd> <au>Dunglison.</au> -- <col><b>Congee water</b></col>, <cd>water in which rice has been boiled.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congelation</ent><br/
<hw>Con`ge*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cong<eacute/lation</ets>, L. <ets>congelatio</ets>.]</ety> <sn>1.</sn> <def>The act or process of passing, or causing to pass, from a fluid to a solid state, as by the abstraction of heat; the act or process of freezing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The capillary tubes are obstructed either by outward compression or <qex>congelation</qex> of the fluid.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>The state of being congealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>That which is congealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sugar plums . . . with a multitude of <qex>congelations</qex> in jellies of various colors.</q> <rj><qau>Taller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congener</ent><br/
<hw>Con"ge*ner</hw> <pr>(?; 277)</pr>, <pos>n.</pos>  <ety>[From L. <ets>congener</ets>. See <er>Congenerous</er>.]</ety> <def>A thing of the same genus, species, or kind; a thing allied in nature, character, or action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cherry tree has been often grafted on the laurel, to which it is a <qex>congener</qex>.</q> <rj><qau>P. Miller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our elk is more polygamous in his habits than any other deer except his <qex>congener</qex>, the red deer of Europe.</q> <rj><qau>Caton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congeneracy</ent><br/
<hw>Con*gen"er*a*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Similarity of origin; affinity.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenerical</ent><br/
<ent>Congeneric</ent><br/
<mhw>{ <hw>Con`ge*ner"ic</hw> <pr>(?)</pr>, <hw>Con`ge*ner"ic*al</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <def>Belonging to the same genus; allied in origin, nature, or action; of or pertaining to a congener.</def>  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenerous</ent><br/
<hw>Con*gen"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>congener</ets>; <ets>con-</ets> + <ets>genus</ets>, <ets>generis</ets>, birth, kind, race. See <er>Genus</er>, and cf. <er>Congener</er>.]</ety> <def>Allied in origin or cause; congeneric; <as>as, <ex>congenerous</ex> diseases</as>.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj></p>

<p>-- <wordforms><wf>Con*gen"er*ous*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark> <au>Hallywell.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenial</ent><br/
<hw>Con*gen"ial</hw> <pr>(<?/; 106)</pr>, <pos>a.</pos> <ety>[Pref. <ets>con-</ets> + <ets>genial</ets>.]</ety> <sn>1.</sn> <def>Partaking of the same nature; allied by natural characteristics; kindred; sympathetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Congenial</qex> souls! whose life one avarice joins.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>two <qex>congenial</qex> spirits united . . . by mutual confidence and reciprocal virtues</q>  <qau>T. L. Peacock</qau></p>

<p><sn>2.</sn>  <def>Naturally adapted; suited to the disposition; <as>as, a <ex>congenial</ex> atmosphere to work in</as>.</def> <ldquo/<xex>Congenial</xex> clime.<rdquo/  <rj><au>C. J. Fox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To defame the excellence with which it has no sympathy . . . is its <qex>congenial</qex> work.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld>  <def>capable of cross-fertilization or of being grafted; -- used of plants.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Congeniality</ent><br/
<hw>Con*ge`ni*al"i*ty</hw> <pr>(? <or/ ?; 106)</pr>, <pos>n.</pos>  <def>The state or quality of being congenial; natural affinity; adaptation; suitableness.</def>  <rj><au>Sir J. Reynolds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If <qex>congeniality</qex> of tastes could have made a marriage happy, that union should have been thrice blessed.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Compatibility between persons.</def><br/
<syn><b>Syn. --</b> congenialness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>The personality trait of being friendly to a conspicuous degree.</def><br/
<syn><b>Syn. --</b> geniality.</syn><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>The personality trait of being likable or pleasant.</def><br/
<syn><b>Syn. --</b> pleasantness, likability.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Congenialize</ent><br/
<hw>Con*gen"ial*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make congenial.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenially</ent><br/
<hw>Con*gen"ial*ly</hw>, <pos>adv.</pos> <def>In a congenial manner; <as>as, <ex>congenially</ex> married or employed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenialness</ent><br/
<hw>Con*gen"ial*ness</hw>, <pos>n.</pos> <def>Congeniality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenious</ent><br/
<hw>Con*gen"ious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Congeneric.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenital</ent><br/
<hw>Con*gen"i*tal</hw> <pr>(k<ocr/n*j<ecr/n"<icr/*t<ait/l)</pr>, <pos>a.</pos> <ety>[From <er>Congenite</er>.]</ety> <def>Existing at, or dating from, birth; pertaining to one from birth; born with one; connate; constitutional; natural; <as>as, a <ex>congenital</ex> deformity; a <ex>congenital</ex> liar</as>. See <er>Connate</er> and <er>native</er>.</def><br/
<syn><b>Syn. --</b> connate; native; inborn; inherited; hereditary.</syn>
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Congenitally</ent><br/
<hw>Con*gen"i*tal*ly</hw>, <pos>adv.</pos> <def>In a congenital manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congenite</ent><br/
<hw>Con*gen"ite</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>congenitus</ets>; <ets>con-</ets> + <ets>genitus</ets>, p. p. of <ets>gignere</ets> to beget. See <er>Generate</er>.]</ety> <def>Congenital; connate; inborn. See <er>Congenital</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many conclusions, of moral and intellectual truths, seem . . . to be <qex>congenite</qex> with us.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conger</ent><br/
<hw>Con"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conger</ets>, <ets>congrus</ets>, akin to Gr. <?/: cf. F. <ets>congre</ets>.]</ety> <fld>(Zool.)</fld> <def>The conger eel; -- called also <altname>congeree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conger sea</b></col> <fld>(Zool.)</fld>, <cd>the sea eel; a large species of eel (<spn>Conger vulgaris</spn>), which sometimes grows to the length of ten feet.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congeries</ent><br/
<hw>Con*ge"ri*es</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[L., fr. <ets>congerere</ets>. See <er>Congest</er>.]</ety> <def>A collection of particles or bodies into one mass; a heap; an aggregation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congest</ent><br/
<hw>Con*gest"</hw> <pr>(#)</pr>, <pos>v. t.</pos> <ety>[L. <ets>congestus</ets>, p. p. of <ets>congere</ets> to bring together; <ets>con-</ets> + <ets>gerere</ets>. See <er>Gerund</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To collect or gather into a mass or aggregate; to bring together; to accumulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To what will thy <qex>congested</qex> guilt amount?</q> <rj><qau>Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>To cause an overfullness of the blood vessels (esp. the capillaries) of an organ or part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congested</ent><br/
<hw>Con*gest"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>Crowded together.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Containing an unnatural accumulation of blood; hyper<ae/mic; -- said of any part of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congestion</ent><br/
<hw>Con*ges"tion</hw> <pr>(?; 106)</pr>, <pos>n.</pos> <ety>[L. <ets>congestio</ets>: cf. F. <ets>congestion</ets>.]</ety> <sn>1.</sn> <def>The act of gathering into a heap or mass; accumulation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>congestion</qex> of dead bodies one upon another.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Overfullness of the capillary and other blood vessels, etc., in any locality or organ (often producing other morbid symptoms); local hyper<ae/mia, active or passive; <as>as, arterial <ex>congestion</ex>; venous <ex>congestion</ex>; <ex>congestion</ex> of the lungs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congestive</ent><br/
<hw>Con*gest"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Pertaining to, indicating, or attended with, congestion in some part of the body; <as>as, a <ex>congestive</ex> fever</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congiary</ent><br/
<hw>Con"gi*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Congiaries</plw> <pr>(#)</pr></plu>. <ety>[L. <ets>congiarium</ets>, fr. <ets>congius</ets> a liquid measure.]</ety> <def>A present, as of corn, wine, or oil, made by a Roman emperor to the soldiers or the people; -- so called because measured to each in a <xex>congius</xex>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In later years, when gifts of money were distributed, the name <xex>congius</xex> was retained.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congius</ent><br/
||<hw>Con"gi*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <sn>1.</sn> <fld>(Roman Antiq.)</fld> <def>A liquid measure containing about three quarts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A gallon, or four quarts.</def> <altsp>[Often abbreviated to <asp>cong.</asp>]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglaciate</ent><br/
<hw>Con*gla"ci*ate</hw> <pr>(?; 221)</pr>, <pos>v. t. & i.</pos> <ety>[L. <ets>conglaciatus</ets>, p. p. of <ets>conglaciare</ets>. See <er>Glaciate</er>.]</ety> <def>To turn to ice; to freeze.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglaciation</ent><br/
<hw>Con*gla`ci*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>conglaciation</ets>.]</ety> <def>The act or process of changing into ice, or the state of being converted to ice; a freezing; congelation; also, a frost.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglobate</ent><br/
<hw>Con*glo"bate</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>conglobatus</ets>, p. p. of <ets>conglobare</ets> to conglobate. See <er>Globate</er>.]</ety> <def>Collected into, or forming, a rounded mass or ball; <as>as, the <ex>conglobate</ex> [lymphatic] glands; <ex>conglobate</ex> flowers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglobate</ent><br/
<hw>Con*glo"bate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conglobated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conglobating</conjf>.]</vmorph> <ety>[Cf. <er>Conglore</er>.]</ety> <def>To collect or form into a ball or rounded mass; to gather or mass together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conglobated</qex> bubbles undissolved.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglobation</ent><br/
<hw>Con`glo*ba"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conglobatio</ets>: cf. F. <ets>conglobation</ets>.]</ety> <sn>1.</sn> <def>The act or process of forming into a ball.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A round body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglobe</ent><br/
<hw>Con*globe"</hw> <pr>(#)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conglobed</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conglobing</conjf>.]</vmorph> <ety>[L. <ets>conglobare</ets>: cf. F. <ets>conglober</ets>. Cf. <er>Conglobate</er>.]</ety> <def>To gather into a ball; to collect into a round mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then founded, then <qex>conglobed</qex><br/
Like things to like.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglobe</ent><br/
<hw>Con*globe"</hw>, <pos>v. i.</pos> <def>To collect, unite, or coalesce in a round mass.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglobulate</ent><br/
<hw>Con*glob"u*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>con-</ets> + <ets>globule</ets>.]</ety> <def>To gather into a small round mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglomerate</ent><br/
<hw>Con*glom"er*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conglomeratus</ets>, p. p. of <ets>conglomerare</ets> to roll together; <ets>con-</ets> + <ets>glomerare</ets> to wind into a ball. See <er>Glomerate</er>.]</ety> <sn>1.</sn> <def>Gathered into a ball or a mass; collected together; concentrated; <as>as, <ex>conglomerate</ex> rays of light</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beams of light when they are multiplied and <qex>conglomerate</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fluids are separated in the liver and the other conglobate and <qex>conglomerate</qex> glands.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Closely crowded together; densly clustered; <as>as, <ex>conglomerate</ex> flowers</as>.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>Composed of stones, pebbles, or fragments of rocks, cemented together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglomerate</ent><br/
<hw>Con*glom"er*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which is heaped together in a mass or conpacted from various sources; a mass formed of fragments; collection; accumulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>conglomerate</qex> of marvelous anecdotes, marvelously heaped together.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>A rock, composed or rounded fragments of stone cemented together by another mineral substance, either calcareous, siliceous, or argillaceous; pudding stone; -- opposed to <xex>agglomerate</xex>. See <er>Breccia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>conglomerate</qex>, therefore, is simply gravel bound together by a cement.</q> <rj><qau>Lyell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglomerate</ent><br/
<hw>Con*glom"er*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conglomerated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conglomerating</conjf>.]</vmorph> <def>To gather into a ball or round body; to collect into a mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglomeration</ent><br/
<hw>Con*glom`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conglomeratio</ets>: cf. F. <ets>conglomeration</ets>.]</ety> <sn>1.</sn> <def>The act or process of gathering into a mass; the state of being thus collected; collection; accumulation.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is conglomerated; a mixed mass; a hodgepodge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglutin</ent><br/
<hw>Con*glu"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Conglutinate</er>.]</ety> <fld>(Chem.)</fld> <def>A variety of vegetable casein, resembling legumin, and found in almonds, rye, wheat, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglutinant</ent><br/
<hw>Con*glu"ti*nant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L., <ets>conglutinans</ets>, p. pr.]</ety> <def>Cementing together; uniting closely; causing to adhere; promoting healing, as of a wound or a broken bone, by adhesion of the parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglutinate</ent><br/
<hw>Con*glu"ti*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conglutinatus</ets>, p. p. of <ets>conglutinare</ets> to glue; <ets>con-</ets> + <ets>glutinare</ets> to glue, <ets>gluten</ets> glue.]</ety> <def>Glued together; united, as by some adhesive substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglutinate</ent><br/
<hw>Con*glu"ti*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conglutinated</conjf>; <pos>p. pr. & vb. n.</pos> Conglutinating.]</vmorph> <def>To glue together; to unite by some glutinous or tenacious substance; to cause to adhere or to grow together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bones . . . have had their broken parts <qex>conglutinated</qex> within three or four days.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglutinate</ent><br/
<hw>Con*glu"ti*nate</hw>, <pos>v. i.</pos> <def>To unite by the intervention of some glutinous substance; to coalesce.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 303 --></p>

<p><ent>Conglutination</ent><br/
<hw>Con*glu`ti*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conglutinatio</ets>: cf. F. <ets>conglutination</ets>.]</ety> <def>A gluing together; a joining by means of some tenacious substance; junction; union.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conglutination</qex> of parts separated by a wound.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conglutinative</ent><br/
<hw>Con*glu"ti*na"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>conglutinatif</ets>.]</ety> <def>Conglutinant.</def></p>

<p><ent>Congo group</ent><br/
<hw>Con"go group</hw>. <ety>[From <er>Congo red</er>.]</ety> <def>A group of artificial dyes with an affinity for vegetable fibers, so that no mordant is required. Most of them are azo compounds derived from benzidine or tolidine. Called also <altname><col><b>benzidine dyes</b></col></altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Congolese</ent><br/
<hw>Congolese</hw> <pos>adj.</pos> <def>of, relating to, or characteristic of the Congo region or its people; <as>as, <ex>Congolese</ex> rulers; the <ex>Congolese</ex> republic</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Congolese</ent><br/
<hw>Congolese</hw> <pos>n.</pos> <def>a native or inhabitant of the Republic of Congo.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Congo red</ent><br/
<hw>Congo red</hw>. <fld>(Chem.)</fld> <def>An artificial red dye from which the Congo group received its name. It is also widely used either in aqueous solution or as test paper (<col><b>Congo paper</b></col>) for the detection of free acid, which turns it blue.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Congo snake</ent><br/
<hw>Con"go snake"</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>An amphibian (<spn>Amphiuma means</spn>) of the order <ord>Urodela</ord>, found in the southern United States. See <er>Amphiuma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congo</ent><br/
<ent>Congou</ent><br/
<mhw>{ <hw>Con"gou</hw> <pr>(?)</pr>, <hw>Con"go</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Chin. <ets>kung-foo</ets> labor.]</ety> <def>Black tea, of higher grade (finer leaf and less dusty) than the present bohea. Also called <altname>English breakfast tea</altname>. See <er>Tea</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>Of black teas, the great mass is called <qex>Congou</qex>, or the <ldquo/well worked<rdquo/, a name which took the place of the <qex>Bohea</qex> of 150 years ago, and is now itself giving way to the term <ldquo/English breakfast tea.<rdquo/</q> <rj><qau>S. W. Williams.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congratulant</ent><br/
<hw>Con*grat"u*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>congratulans</ets>, p. pr.]</ety> <def>Rejoicing together; congratulatory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With like joy<br/
<qex>Congratulant</qex> approached him.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congratulate</ent><br/
<hw>Con*grat"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Congratulated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Congratulating</conjf>.]</vmorph> <ety>[L. <ets>congratulatus</ets>, p. p. of <ets>congratulari</ets> to wish joy abundantly; <ets>con-</ets> + <ets>gratulari</ets> to wish joy, from <ets>gratus</ets> pleasing. See <er>Grateful</er>.]</ety> <def>To address with expressions of sympathetic pleasure on account of some happy event affecting the person addressed; to wish joy to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is the king's most sweet pleasure and affection to <qex>congratulate</qex> the princess at her pavilion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To congratulate one's self</b></col>, <cd>to rejoice; to feel satisfaction; to consider one's self happy or fortunate.</cd></cs></p>

<p><syn><b>Syn.</b> -- <er>To Congratulate</er>, <er>Felicitate</er>.</syn> <usage> To <xex>felicitate</xex> is simply to wish a person joy. To <xex>congratulate</xex> has the additional signification of uniting in the joy of him whom we congratulate.  Hence they are by no means synonymous. One who has lost the object of his affections by her marriage to a rival, might perhaps <xex>felicitate</xex> that rival on his success, but could never be expected to <xex>congratulate</xex> him on such an event.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Felicitations</qex> are little better than compliments; <qex>congratulations</qex> are the expression of a genuine sympathy and joy.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congratulate</ent><br/
<hw>Con*grat"u*late</hw>, <pos>v. i.</pos> <def>To express of feel sympathetic joy; <as>as, to <ex>congratulate</ex> with one's country</as>.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The subjects of England may <qex>congratulate</qex> to themselves.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congratulation</ent><br/
<hw>Con*grat`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>congratulatio</ets>: cf. F. <ets>congratulation</ets>.]</ety> <def>The act of congratulating; an expression of sympathetic pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With infinite <qex>congratulations</qex> for our safe arrival.</q> <rj><qau>Dr. J. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congratulator</ent><br/
<hw>Con*grat"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who offers congratulation.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congratulatory</ent><br/
<hw>Con*grat"u*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Expressive of sympathetic joy; <as>as, a <ex>congratulatory</ex> letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congree</ent><br/
<hw>Con*gree"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref.<ets>on-</ets>+ L. <ets>gratus</ets> pleasing. Cf. <er>Agree</er>.]</ety> <def>To agree.</def> <mark>[bs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congreet</ent><br/
<hw>Con*greet"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To salute mutually.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregate</ent><br/
<hw>Con"gre*gate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>congregatus</ets>, p. p. of <ets>congregare</ets> to congregate; <ets>on-</ets> + <ets>gregare</ets> to collect into a flock, fr. <ets>grex</ets> flock, herd. See <er>Gregarious</er>.]</ety> <def>Collected; compact; close.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregate</ent><br/
<hw>Con"gre*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Congregated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Congregating</conjf>]</vmorph> <def>To collect into an assembly or assemblage; to assemble; to bring into one place, or into a united body; to gather together; to mass; to compact.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Any multitude of Christian men <qex>congregated</qex> may be termed by the name of a church.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cold <qex>congregates</qex> all bodies.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The great receptacle<br/
Of <qex>congregated</qex> waters he called Seas.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregate</ent><br/
<hw>Con"gre*gate</hw>, <pos>v. i.</pos> <def>To come together; to assemble; to meet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even there where merchants most do <qex>congregate</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregation</ent><br/
<hw>Con`gre*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>congregatio</ets>: cf. F. <ets>congr<eacute/gation</ets>.]</ety> <sn>1.</sn> <def>The act of congregating, or bringing together, or of collecting into one aggregate or mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The means of reduction in the fire is but by the <qex>congregation</qex> of homogeneal parts.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A collection or mass of separate things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A foul and pestilent <qex>congregation</qex> of vapors.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An assembly of persons; a gathering; esp. an assembly of persons met for the worship of God, and for religious instruction; a body of people who habitually so meet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Bunyan] rode every year to London, and preached there to large and attentive <qex>congregations</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anc. Jewish Hist.)</fld> <def>The whole body of the Jewish people; -- called also <altname>Congregation of the Lord</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a sin offering for the <qex>congregation</qex>.</q> <rj><qau>Lev. iv. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(R. C. Ch.)</fld> <sd>(a)</sd> <def>A body of cardinals or other ecclesiastics to whom as intrusted some department of the church business; <as>as, the <ex>Congregation of the Propaganda</ex>, which has charge of the missions of the Roman Catholic Church</as>.</def> <sd>(b)</sd> <def>A company of religious persons forming a subdivision of a monastic order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The assemblage of Masters and Doctors at Oxford or Cambrige University, mainly for the granting of degrees.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Scotch Church Hist.)</fld> <def>the name assumed by the Protestant party under John Knox. The leaders called themselves (1557) <xex>Lords of the Congregation</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregational</ent><br/
<hw>Con`gre*ga"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a congregation; conducted, or participated in, by a congregation; <as>as, <ex>congregational</ex> singing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Belonging to the system of Congregationalism, or to Congregationalist; holding to the faith and polity of Congregationalism; <as>as, a <ex>Congregational</ex> church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregationalism</ent><br/
<hw>Con`gre*ga"tion*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That system of church organization which vests all ecclesiastical power in the assembled brotherhood of each local church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The faith and polity of the Congregational churches, taken collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense (which is its usual signification) <xex>Congregationalism</xex> is the system of faith and practice common to a large body of evangelical Trinitarian churches, which recognize the local brotherhood of each church as independent of all dictation in ecclesiastical matters, but are united in fellowship and joint action, as in councils for mutual advice, and in consociations, conferences, missionary organizations, etc., and to whose membership the designation <ldquo/Congregationalists<rdquo/ is generally restricted; but Unitarian and other churches are Congregational in their polity.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congregationalist</ent><br/
<hw>Con`gre*ga"tion*al*ist</hw>, <pos>n.</pos> <def>One who belongs to a Congregational church or society; one who holds to Congregationalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congress</ent><br/
<hw>Con"gress</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Congresses</plw> <pr>(#)</pr></plu>. <ety>[L. <ets>congressus</ets>, fr. <ets>congredi</ets>, p. p. <ets>-gressus</ets>, to go or come together; <ets>con-</ets> + <ets>grati</ets> to go or step, <ets>gradus</ets> step: cf. F. <ets>congr<?/s</ets>. See <er>Grade</er>.]</ety> <sn>1.</sn> <def>A meeting of individuals, whether friendly or hostile; an encounter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here Pallas urges on, and Lausus there;<br/
Their <qex>congress</qex> in the field great Jove withstands.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden encounter; a collision; a shock; -- said of things.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>From these laws may be deduced the rules of the <qex>congresses</qex> and reflections of two bodies.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The coming together of a male and female in sexual commerce; the act of coition.</def>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A gathering or assembly; a conference.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A formal assembly, as of princes, deputies, representatives, envoys, or commissioners; esp., a meeting of the representatives of several governments or societies to consider and determine matters of common interest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The European powers strove to . . . accommodate their differences at the <qex>congress</qex> of Vienna.</q> <rj><qau>Alison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The collective body of senators and representatives of the people of a nation, esp. of a republic, constituting the chief legislative body of the nation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the Congress of the United States (which took the place of the Federal Congress, March 4, 1789), the Senate consists of two Senators from each State, chosen by the State legislature for a term of six years, in such a way that the terms of one third of the whole number expire every year; the House of Representatives consists of members elected by the people of the several Congressional districts, for a term of two years, the term of all ending at the same time. The united body of Senators and Representatives for any term of two years for which the whole body of Representatives is chosen is called <xex>one Congress</xex>. Thus the session which began in December, 1887, was the first (or long) session, and that which began in December, 1888, was the second (or short) session, of the <xex>Fiftieth Congress</xex>. When an extra session is had before the date of the first regular meeting of a Congress, that is called the first session, and the following regular session is called the second session.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The lower house of the Spanish Cortes, the members of which are elected for three years.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Continental Congress</b></col>, <cd>an assembly of deputies from the thirteen British colonies in America, appointed to deliberate in respect to their common interests. They first met in 1774, and from time thereafter until near the close of the Revolution.</cd> -- <col><b>The Federal Congress</b></col>, <cd>the assembly of representatives of the original States of the American Union, who met under the Articles of Confederation from 1781 till 1789.</cd> -- <mcol><col><b>Congress boot</b></col> or <col><b>Congress gaiter</b></col></mcol>, <cd>a high shoe or half-boot, coming above the ankle, and having the sides made in part of some elastic material which stretches to allow the boot to be drawn on and off.</cd> <mark>[U.S.]</mark> -- <col><b>Congress water</b></col>, <cd>a saline mineral water from the <etsep>Congress</etsep> spring at Saratoga, in the State of New York.</cd></cs></p>

<p><syn><b>Syn.</b> -- Assembly; meeting; convention; convocation; council; diet; conclave; parliament; legislature.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congression</ent><br/
<hw>Con*gres"sion</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[L. <ets>congressio</ets>.]</ety> <def>A coming or bringing together, as in a public meeting, in a dispute, in the act of comparing, or in sexual intercourse.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congressional</ent><br/
<hw>Con*gres"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a congress, especially, to the Congress of the United States; <as>as, <ex>congressional</ex> debates</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Congressional</qex> and official labor.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Congressional District</b></col>, <cd>one of the divisions into which a State is periodically divided (according to population), each of which is entitled to elect a Representative to the Congress of the United States.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congressive</ent><br/
<hw>Con*gres"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Encountering, or coming together.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congressman</ent><br/
<hw>Con"gress*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Congressmen</plw> <pr>(<?/)</pr>.</plu> <def>A member of the Congress of the United States, esp. of the House of Representatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congreve</ent><br/
<hw>Con"greve</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[After Sir William <etsep>Congreve</etsep>, the inventor.]</ety> <sn>1.</sn> <def>Short for <er>Cogreve rocket</er>, a powerful form of rocket formerly used in war, either in the field or for bombardment. In the former case it was armed with shell, shrapnel, or other missiles; in the latter, with an inextinguishable explosive material, inclosed in a metallic case. It was guided by a long wooden stick.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Short for <col><b>Congreve match</b></col>, an early friction match, containing sulphur, potassium chlorate, and antimony sulphide.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Congreve rocket</ent><br/
<hw>Con"greve rock"et</hw> <pr>(?)</pr>. <def>See <er>congreve</er> and cf. <er>Rocket</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Congrue</ent><br/
<hw>Con"grue"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>congruere</ets>. See <er>Congruous</er>.]</ety> <def>To agree; to be suitable.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruence</ent><br/
<hw>Con"gru*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>congruentia</ets>: cf. OF. <ets>cornguence</ets>.]</ety> <def>Suitableness of one thing to another; agreement; consistency.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruency</ent><br/
<hw>Con"gru*en*cy</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos>  <def>Congruence.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Congruency of lines</b></col>. <fld>(Geom.)</fld> <cd>See <cref>Complex of lines</cref>, under <er>Complex</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruent</ent><br/
<hw>Con"gru*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>congruens</ets>, p. pr. of <ets>congruere</ets>: cf. F. <ets>congruent</ets>.]</ety> <def>Possessing congruity; suitable; agreeing; corresponding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>congruent</qex> and harmonious fitting of parts in a sentence.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Congruent figures</b></col> <fld>(Geom.)</fld>, <cd>concurring figures.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruism</ent><br/
<hw>Con"gru*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>congruisme</ets>.]</ety> <fld>(Scholastic Theol.)</fld> <def>See <er>Congruity</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruity</ent><br/
<hw>Con*gru"i*ty</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Congruities</plw> <pr>(#)</pr></plu>. <ety>[Cf. F. <ets>congruit<?/</ets>.]</ety> <sn>1.</sn> <def>The state or quality of being congruous; the relation or agreement between things; fitness; harmony; correspondence; consistency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With what <qex>congruity</qex> doth the church of Rome deny that her enemies do at all appertain to the church of Christ?</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A whole sentence may fail of its <qex>congruity</qex> by wanting one particle.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>Coincidence, as that of lines or figures laid over one another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Scholastic Theol.)</fld> <def>That, in an imperfectly good persons, which renders it suitable for God to bestow on him gifts of grace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruous</ent><br/
<hw>Con"gru*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>congruus</ets>, fr. <ets>congruere</ets> to come together, to coincide, to agree. Of uncertain origin.]</ety> <def>Suitable or concordant; accordant; fit; harmonious; correspondent; consistent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not <qex>congruous</qex> to the nature of epic poetry.</q> <rj><qau>Blair.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is no ways <qex>congruous</qex> that God should be always frightening men into an acknowledgment of the truth.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Congruously</ent><br/
<hw>Con"gru*ous*ly</hw>, <pos>adv.</pos> <def>In a congruous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conhydrine</ent><br/
<hw>Con*hy"drine</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[<ets>Con</ets>ium + <ets>hydr</ets>ate.]</ety> <fld>(Chem.)</fld> <def>A vegetable alkaloid found with conine in the poison hemlock (<spn>Conium maculatum</spn>). It is a white crystalline substance, <chform>C8H17NO</chform>, easily convertible into conine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conia</ent><br/
||<hw>Co*ni"a</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[NL. See <er>Conium</er>.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Conine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conical</ent><br/
<ent>Conic</ent><br/
<mhw>{ <hw>Con"ic</hw> <pr>(?)</pr>, <hw>Con"ic*al</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>conique</ets>. See <er>Cone</er>.]</ety> <sn>1.</sn> <def>Having the form of, or resembling, a geometrical cone; round and tapering to a point, or gradually lessening in circumference; <as>as, a <ex>conic</ex> or <ex>conical</ex> figure; a <ex>conical</ex> vessel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a cone; <as>as, <ex>conic</ex> sections</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conic section</b></col> <fld>(Geom.)</fld>, <cd>a curved line formed by the intersection of the surface of a right cone and a plane. The conic sections are the parabola, ellipse, and hyperbola. The right lines and the circle which result from certain positions of the plane are sometimes, though not generally included.</cd> -- <col><b>Conic sections</b></col>, <cd>that branch of geometry which treats of the parabola, ellipse, and hyperbola.</cd> -- <col><b>Conical pendulum</b></col>. <cd>See <er>Pendulum</er>.</cd> -- <col><b>Conical projection</b></col>, <cd>a method of delineating the surface of a sphere upon a plane surface as if projected upon the surface of a cone; -- much used by makers of maps in Europe.</cd> -- <col><b>Conical surface</b></col> <fld>(Geom.)</fld>, <cd>a surface described by a right line moving along any curve and always passing through a fixed point that is not in the plane of that curve.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conic</ent><br/
<hw>Con"ic</hw>, <pos>n.</pos> <fld>(Math.)</fld> <def>A conic section.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conicality</ent><br/
<hw>Con`i*cal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Conicalness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conically</ent><br/
<hw>Con"ic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In the form of a cone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conicalness</ent><br/
<hw>Con"ic*al*ness</hw>, <pos>n.</pos> <def>State or quality of being conical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conico-</ent><br/
<hw>Con"i*co-</hw> <pr>(<?/)</pr>, <pos>a.</pos> <ety>[See <er>Conic</er>.]</ety> <def>A combining form, meaning <xex>somewhat resembling a cone</xex>; <as>as, <ex>conico</ex>-cylindrical, resembling a cone and a cylinder; <ex>conico</ex>-hemispherical; <ex>conico</ex>-subulate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conicoid</ent><br/
<hw>Con"i*coid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Conic</ets> + <ets>-oid</ets>.]</ety> <fld>(Math.)</fld> <def>Same as <er>Conoidal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conics</ent><br/
<hw>Con"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That branch of geometry which treats of the cone and the curves which arise from its sections.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conic sections.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conidiophore</ent><br/
<hw>conidiophore</hw> <pos>n.</pos> <def>a specialized fungal hypha (a branch of the fungal mycelium) that produces conidia.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conidium</ent><br/
||<hw>Co*nid"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conida</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>A peculiar kind of reproductive cell found in certain fungi, and often containing zoospores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conifer</ent><br/
<hw>Co"ni*fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conifer</ets>; <ets>conus</ets> cone + <ets>ferre</ets> to bear: cf. F. <ets>conif<egrave/re</ets>.]</ety> <fld>(Bot.)</fld> <def>A tree or shrub bearing cones; one of the order <ord>Coniferae</ord>, which includes the pine, cypress, and (according to some) the yew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coniferin</ent><br/
<hw>Co*nif"er*in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A glucoside extracted from the cambium layer of coniferous trees as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coniferophytina</ent><br/
<ent>Coniferophyta</ent><br/
<mhw><hw>Coniferophyta</hw>, <hw>Coniferophytina</hw></mhw> <pos>n.</pos> <def>a class of cone-bearing gymnosperms dating from the Carboniferous; most are substantial trees; it includes the classes Pinopsida (subdivision Pinophytina) and Ginkgopsida (subdivision Ginkgophytina) and Taxopsida (subdivision Taxophytina) which in turn include the surviving orders <ord>Coniferales</ord> and <ord>Taxales</ord> (yews) and sometimes <ord>Ginkgoales</ord> as well as extinct orders such as <ord>Cordaitales</ord> (of the Carboniferous and Permian) and <ord>Volztiales</ord> (of the Permian-Jurassic).</def><br/
<syn><b>Syn. --</b> Coniferopsida, class <class>Coniferopsida</class>, Coniferophytina, subdivision Coniferophytina.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coniferopsida</ent><br/
<hw>Coniferopsida</hw> <pos>n.</pos> <def>same as <er>coniferophyta</er>.</def><br/
<syn><b>Syn. --</b> class Coniferopsida, Coniferophytina, subdivision Coniferophytina, Coniferophyta.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coniferous</ent><br/
<hw>Co*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <sd>(a)</sd> <def>Bearing cones, as the pine and cypress.</def> <sd>(b)</sd> <def>Pertaining to the order <ord>Coniferae</ord>, of which the pine tree is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coniform</ent><br/
<hw>Co"ni*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Cone</ets> + <ets>-form</ets>: cf. F. <ets>coniforme</ets>.]</ety> <def>Cone-shaped; conical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coniine</ent><br/
<hw>Co*ni"ine</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <def>See <er>Conine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conimene</ent><br/
<hw>Co"ni*mene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Olibene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conine</ent><br/
<hw>Co"nine</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[From <er>Conium</er>.]</ety> <fld>(Chem.)</fld> <def>A powerful and very poisonous vegetable alkaloid found in the hemlock (<spn>Conium maculatum</spn>) and extracted as a colorless oil, <chform>C8H17N</chform>, of strong repulsive odor and acrid taste. It is regarded as a derivative of piperidine and likewise of one of the collidines. It occasions a gradual paralysis of the motor nerves. Called also <altname>coniine</altname>, <altname>coneine</altname>, <altname>conia</altname>, etc.  See <er>Conium</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coniroster</ent><br/
||<hw>Co`ni*ros"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>One of the Conirostres.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conirostral</ent><br/
<hw>Co`ni*ros"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Belonging to the Conirostres.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conirostres</ent><br/
||<hw>Co`ni*ros"tres</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>conus</ets> cone + <ets>rostrum</ets> beak: cf. F. <ets>conirostre</ets>.]</ety> <fld>(Zool.)</fld> <def>A tribe of perching birds, including those which have a strong conical bill, as the finches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conisor</ent><br/
<hw>Con`i*sor"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Obs</ets>.]</ety> <def>See <er>Cognizor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conistra</ent><br/
||<hw>Co*nis"tra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ dust.]</ety> <fld>(Greek Antiq.)</fld> <def>Originally, a part of the palestra, or gymnasium among the Greeks; either the place where sand was stored for use in sprinkling the wrestlers, or the wrestling ground itself. Hence, a part of the orchestra of the Greek theater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conite</ent><br/
<hw>Co"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ dust: cf. F. <ets>conite</ets>. So called on account of its gray color.]</ety> <fld>(Min.)</fld> <def>A magnesian variety of dolomite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conium</ent><br/
||<hw>Co*ni"um</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ hemlock.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of biennial, poisonous, white-flowered, umbelliferous plants, bearing ribbed fruit (<ldquo/seeds<rdquo/) and decompound leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 304 --></p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The common hemlock (<spn>Conium maculatum</spn>, poison hemlock, spotted hemlock, poison parsley), a roadside weed of Europe, Asia, and America, cultivated in the United States for medicinal purpose. It is an active poison. The leaves and fruit are used in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conject</ent><br/
<hw>Con*ject"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>conjectus</ets>, p. p. of <ets>conjicere</ets>. See <er>Conjecture</er>, <pos>n.</pos>]</ety> <def>To throw together, or to throw.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conject</ent><br/
<hw>Con*ject"</hw>, <pos>v. t.</pos> <def>To conjecture; also, to plan.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjector</ent><br/
<hw>Con*ject"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who guesses or conjectures.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A great <qex>conjector</qex> at other men by their writings.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecturable</ent><br/
<hw>Con*jec"tur*a*ble</hw> <pr>(?; 135)</pr>, <pos>a.</pos>  <def>Capable of being conjectured or guessed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjectural</ent><br/
<hw>Con*jec"tur*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conjecturalis</ets>: cf. F. <ets>conjectural</ets>.]</ety> <def>Dependent on conjecture; fancied; imagined; guessed at; undetermined; doubtful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And mak'st <qex>conjectural</qex> fears to come into me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A slight expense of <qex>conjectural</qex> analogy.</q> <rj><qau>Hugh Miller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who or what such editor may be, must remain <qex>conjectural</qex>.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecturalist</ent><br/
<hw>Con*jec"tur*al*ist</hw>, <pos>n.</pos> <def>A conjecturer.</def> <mark>[R.]</mark>  <rj><au>Month. rev.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecturally</ent><br/
<hw>Con*jec`tur*al"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which depends upon guess; guesswork.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecturally</ent><br/
<hw>Con*jec`tur*al*ly</hw>, <pos>adv.</pos> <def>In a conjectural manner; by way of conjecture.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecture</ent><br/
<hw>Con*jec"ture</hw> <pr>(; 135?)</pr>, <pos>n.</pos> <ety>[L. <ets>conjectura</ets>, fr. <ets>conjicere</ets>, <ets>conjectum</ets>, to throw together, infer, conjecture; <ets>con-</ets> + <ets>jacere</ets> to throw: cf. F. <ets>conjecturer</ets>. See <er>Jet</er> a shooting forth.]</ety> <def>An opinion, or judgment, formed on defective or presumptive evidence; probable inference; surmise; guess; suspicion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Herodotus] would thus have corrected his first loose <qex>conjecture</qex> by a real study of nature.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conjectures</qex>, fancies, built on nothing firm.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecture</ent><br/
<hw>Con*jec"ture</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conjectured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conjecturing</conjf>.]</vmorph> <ety>[Cf. F. <ets>conjecturer</ets>. Cf. <er>Conject</er>.]</ety> <def>To arrive at by conjecture; to infer on slight evidence; to surmise; to guess; to form, at random, opinions concerning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Human reason can then, at the best, but <qex>conjecture</qex> what will be.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecture</ent><br/
<hw>Con*jec"ture</hw>, <pos>v. i.</pos> <def>To make conjectures; to surmise; to guess; to infer; to form an opinion; to imagine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjecturer</ent><br/
<hw>Con*jec"tur*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conjectures.</def>  <rj><au>Hobbes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjoin</ent><br/
<hw>Con*join</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conjoined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conjoining</conjf>.]</vmorph> <ety>[F. <ets>conjoindre</ets>, fr. L. <ets>conjungere</ets>, <ets>-junctum</ets>; <ets>con-</ets> + <ets>jungere</ets> to join. See <er>Join</er>, and cf. <er>Conjugate</er>, <er>Conjunction</er>.]</ety> <def>To join together; to unite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The English army, that divided was<br/
Into two parties, is now <qex>conjoined</qex> in one.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If either of you know any inward impediment why you should not be <qex>conjoined</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let that which he learns next be nearly <qex>conjoined</qex> with what he knows already.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjoin</ent><br/
<hw>Con*join"</hw>, <pos>v. i.</pos> <def>To unite; to join; to league.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjoined</ent><br/
<hw>Con*joined"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Joined together or touching.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjoint</ent><br/
<hw>Con*joint"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>conjoint</ets>, p. p. of <ets>conjoindre</ets>. See <er>Conjoin</er>, and cf. <er>Conjunct</er>.]</ety> <def>United; connected; associated.</def> <ldquo/Influence <xex>conjoint</xex>.<rdquo/  <rj><au>Glover.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conjoint degrees</b></col> <fld>(Mus.)</fld>, <cd>two notes which follow each other immediately in the order of the scale, as <xex>ut</xex> and <xex>re</xex>.</cd> <au>Johnson.</au> -- <col><b>Conjoint tetrachords</b></col> <fld>(Mus.)</fld>, <cd>two tetrachords or fourths, where the same note is the highest of one and the lowest of the other; -- also written <asp>conjunct</asp>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjointly</ent><br/
<hw>Con*joint"ly</hw>, <pos>adv.</pos> <def>In a conjoint manner; untitedly; jointly; together.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjointness</ent><br/
<hw>Con*joint"ness</hw>, <pos>n.</pos> <def>The quality of being conjoint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjubilant</ent><br/
<hw>Con*ju"bi*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shouting together for joy; rejoicing together.</def> <mark>[R.]</mark>  <rj><au>Neale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugal</ent><br/
<hw>Con"ju*gal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conjugalis</ets>, fr. <ets>conjux</ets> husband, wife, consort, fr. <ets>conjungere</ets> to unite, join in marriage. See <er>Conjoin</er>.]</ety> <def>Belonging to marriage; suitable or appropriate to the marriage state or to married persons; matrimonial; connubial.</def> <ldquo/<xex>Conjugal</xex> affection.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugality</ent><br/
<hw>Con`ju*gal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The conjugal state; sexual intercourse.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugally</ent><br/
<hw>Con"ju*gal*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a conjugal manner; matrimonially; connubially.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugate</ent><br/
<hw>Con"ju*gate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conjugatus</ets>, p. p. or <ets>conjugare</ets> to unite; <ets>con-</ets> + <ets>jugare</ets> to join, yoke, marry, jugum yoke; akin to <ets>jungere</ets> to join. See <er>Join</er>.]</ety> <sn>1.</sn> <def>United in pairs; yoked together; coupled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>In single pairs; coupled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>Containing two or more compounds or radicals supposed to act the part of a single one.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Gram.)</fld> <def>Agreeing in derivation and radical signification; -- said of words.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Math.)</fld> <def>Presenting themselves simultaneously and having reciprocal properties; -- frequently used in pure and applied mathematics with reference to two quantities, points, lines, axes, curves, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conjugate axis of a hyperbola</b></col> <fld>(Math.)</fld>, <cd>the line through the center of the curve, perpendicular to the line through the two foci.</cd> -- <col><b>Conjugate diameters</b></col> <fld>(Conic Sections)</fld>, <cd>two diameters of an ellipse or hyperbola such that each bisects all chords drawn parallel to the other.</cd> -- <col><b>Conjugate focus</b></col> <fld>(Opt.)</fld> <cd>See under <er>Focus</er>.</cd> -- <col><b>Conjugate mirrors</b></col> <fld>(Optics)</fld>, <cd>two mirrors so placed that rays from the focus of one are received at the focus of the other, especially two concave mirrors so placed that rays proceeding from the principal focus of one and reflected in a parallel beam are received upon the other and brought to the principal focus.</cd> -- <col><b>Conjugate point</b></col> <fld>(Geom.)</fld>, <cd>an acnode. See <er>Acnode</er>, and <er>Double point</er>.</cd> -- <col><b>Self-conjugate triangle</b></col> <fld>(Conic Sections)</fld>, <cd>a triangle each of whose vertices is the pole of the opposite side with reference to a conic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugate</ent><br/
<hw>Con`ju*gate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conjugatum</ets> a combining, etymological relationship.]</ety> <sn>1.</sn> <def>A word agreeing in derivation with another word, and therefore generally resembling it in signification.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have learned, in logic, that <qex>conjugates</qex> are sometimes in name only, and not in deed.</q> <rj><qau>Abp. Bramhall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A complex compound formed from the non-covalent union of two other comounds, behaving as a single compound.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugate</ent><br/
<hw>Con"ju*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conjugated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Conjugating</conjf>.]</vmorph> <sn>1.</sn> <def>To unite in marriage; to join.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>To inflect (a verb), or give in order the forms which it assumes in its several voices, moods, tenses, numbers, and persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugate</ent><br/
<hw>Con"ju*gate</hw>, <pos>v. i.</pos> <fld>(Biol.)</fld> <def>To unite in a kind of sexual union, as two or more cells or individuals among the more simple plants and animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conjugated</ent><br/
<hw>conjugated</hw> <pos>adj.</pos> <sn>1.</sn> <fld>(Chem.)</fld>  <fld>(Chem.)</fld> <def>formed by the union of two compounds; <as>as, a <ex>conjugated</ex> protein</as>.</def><br/
<syn><b>Syn. --</b> conjugate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>joined together especially in a pair or pairs.</def><br/
<syn><b>Syn. --</b> conjugate, coupled.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <fld>(Chem.)</fld> <def>containing two or more double bonds each separated from the other by a single bond; -- of an organic compound or of the double bonds thus arranged.</def> <note>Conjugated double bonds have chemical and electronic properties different from those of isolated double bonds.</note><br/
<syn><b>Syn. --</b> conjugate.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Conjugation</ent><br/
<hw>Con`ju*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conjugatio</ets> conjugation (in senses 1 & 3).]</ety> <sn>1.</sn> <def>the act of uniting or combining; union; assemblage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Mixtures and <qex>conjugations</qex> of atoms.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Two things conjoined; a pair; a couple.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sixth <qex>conjugations</qex> or pair of nerves.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <sd>(a)</sd> <def>The act of conjugating a verb or giving in order its various parts and inflections.</def> <sd>(b)</sd> <def>A scheme in which are arranged all the parts of a verb.</def> <sd>(c)</sd> <def>A class of verbs conjugated in the same manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Biol.)</fld> <def>A kind of sexual union; -- applied to a blending of the contents of two or more cells or individuals in some plants and lower animals, by which new spores or germs are developed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugational</ent><br/
<hw>Con`ju*ga"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>relating to conjugation.</def>  <rj><au>Ellis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugial</ent><br/
<hw>Con*ju"gi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conjugialis</ets>, fr. <ets>conjugium</ets>. Cf. <er>Conjugal</er>.]</ety> <def>Conjugal.</def> <mark>[R.]</mark>  <rj><au>Swedenborg.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjugium</ent><br/
||<hw>Con*ju"gi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Rom. Law)</fld> <def>The marriage tie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunct</ent><br/
<hw>Con*junct"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conjunctus</ets>, p. p. See <er>Conjoin</er>.]</ety> <sn>1.</sn> <def>United; conjoined; concurrent.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Same as <er>Conjoined</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunction</ent><br/
<hw>Con*junc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conjunctio</ets>: cf. F. <ets>conjunction</ets>. See <er>Conjoin</er>.]</ety> <sn>1.</sn> <def>The act of conjoining, or the state of being conjoined, united, or associated; union; association; league.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He will unite the white rose and the red:<br/
Smille heaven upon his fair <qex>conjunction</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Man can effect no great matter by his personal strength but as he acts in society and <qex>conjunction</qex> with others.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>The meeting of two or more stars or planets in the same degree of the zodiac; <as>as, the <ex>conjunction</ex> of the moon with the sun, or of Jupiter and Saturn</as>. See the Note under <er>Aspect</er>, <pos>n.</pos>, 6.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Heavenly bodies are said to be in <xex>conjunction</xex> when they are seen in the same part of the heavens, or have the same longitude or right ascension. The <xex>inferior</xex> conjunction of an inferior planet is its position when in conjunction on the same side of the sun with the earth; the <xex>superior</xex> conjunction of a planet is its position when on the side of the sun most distant from the earth.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>A connective or connecting word; an indeclinable word which serves to join together sentences, clauses of a sentence, or words; <as>as, <ex>and</ex>, <ex>but</ex>, <ex>if</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though all <qex>conjunctions</qex> conjoin sentences, yet, with respect to the sense, some are conjunctive and some disjunctive.</q> <rj><qau>Harris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctional</ent><br/
<hw>Con*junc"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to a conjunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctiva</ent><br/
||<hw>Con`junc*ti"va</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>conjunctivus</ets> connective.]</ety> <fld>(Anat.)</fld> <def>The mucous membrane which covers the external surface of the ball of the eye and the inner surface of the lids; the conjunctival membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctival</ent><br/
<hw>Con`junc*ti"val</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Joining; connecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the conjunctiva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctive</ent><br/
<hw>Con*junc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conjunctivus</ets>.]</ety> <sn>1.</sn> <def>Serving to unite; connecting together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Closely united.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conjunctive mood</b></col> <fld>(Gram.)</fld>, <cd>the mood which follows a conjunction or expresses contingency; the subjunctive mood.</cd> -- <col><b>Conjunctive tissue</b></col> <fld>(Anat.)</fld>, <cd>the tissue found in nearly all parts of most animals. It yields gelatin on boiling, and consists of vriously arranged fibers which are imbedded protoplasmic cells, or corpuscles; -- called also <altname>cellular tissue</altname> and <altname>connective tissue</altname>. Adipose or fatty tissue is one of its many forms, and cartilage and bone are sometimes included by the phrase.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctively</ent><br/
<hw>Con*junc"tive*ly</hw>, <pos>adv.</pos> <def>In conjunction or union; together.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctiveness</ent><br/
<hw>Con*junc"tive*ness</hw>, <pos>n.</pos> <def>The state or quality of being conjunctive.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctivitis</ent><br/
<hw>Con*junc`ti*vi"tis</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Inflammation of the conjunctiva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjunctly</ent><br/
<hw>Con*junct"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In union; conjointly; unitedly; together.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjuncture</ent><br/
<hw>Con*junc"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>conjoncture</ets>, LL. <ets>conjunctura</ets>.]</ety> <sn>1.</sn> <def>The act of joining, or state of being joined; union; connection; combination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conjuncture</qex> of philosophy and divinity.</q> <rj><qau>Hobbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A fit <qex>conjuncture</qex> or circumstances.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A crisis produced by a combination of circumstances; complication or combination of events or circumstances; plight resulting from various conditions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Chesterfield] had recently governed Ireland, at a momentous <qex>conjuncture</qex>, with eminent firmness, wisdom, and humanity.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjuration</ent><br/
<hw>Con`ju*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conjuratio</ets>, cf. F. <ets>conjuration</ets>.]</ety> <sn>1.</sn> <def>The act of calling or summoning by a sacred name, or in solemn manner; the act of binding by an oath; an earnest entreaty; adjuration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We charge you, in the name of God, take heed; . . . <br/
Under this <qex>conjuration</qex> speak, my lord.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of invoking supernatural aid by the use of a magical form of words; the practice of magic arts; incantation; enchantment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pretended <qex>conjurations</qex> and prophecies of that event.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A league for a criminal purpose; conspiracy.</def> <mark>[Obs.]</mark> <ldquo/The <xex>conjuration</xex> of Catiline.<rdquo/  <rj><au>Sir T. Elyot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjurator</ent><br/
<hw>Con"ju*ra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL.]</ety> <fld>(O. Eng. Law)</fld> <def>One who swears or is sworn with others; one bound by oath with others; a compurgator.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjure</ent><br/
<hw>Con*jure"</hw> <pr>(k<ocr/n*j<umac/r")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conjured</conjf> <pr>(-j<umac/rd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conjuring</conjf>.]</vmorph> <ety>[F. <ets>conjurer</ets>, fr. L. <ets>conjurare</ets> to swear together, to conspire; <ets>con-</ets> + <ets>jurare</ets> to swear. See <er>Jury</er>.]</ety> <def>To call on or summon by a sacred name or in solemn manner; to implore earnestly; to adjure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>conjure</qex> you, let him know,<br/
Whate'er was done against him, Cato did it.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjure</ent><br/
<hw>Con*jure"</hw>, <pos>v. i.</pos> <def>To combine together by an oath; to conspire; to confederate.</def> <mark>[A Latinism]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Drew after him the third part of Heaven's sons<br/
<qex>Conjured</qex> against the Highest.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjure</ent><br/
<hw>Con"jure</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To affect or effect by conjuration; to call forth or send away by magic arts; to excite or alter, as if by magic or by the aid of supernatural powers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The habitation which your prophet . . . <qex>conjured</qex> the devil into.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To conjure up</b></col>, <cd>or make visible, as a spirit, by magic arts; hence, to invent; <as>as, <ex>to conjure up</ex> a story; <ex>to conjure up</ex> alarms</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjure</ent><br/
<hw>Con"jure</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To practice magical arts; to use the tricks of a conjurer; to juggle; to charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>conjures</qex>; away with her.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjurement</ent><br/
<hw>Con*jure"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Serious injunction; solemn demand or entreaty.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjurer</ent><br/
<hw>Con*jur"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conjures; one who calls, entreats, or charges in a solemn manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjurer</ent><br/
<hw>Con"jur*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who practices magic arts; one who pretends to act by the aid super natural power; also, one who performs feats of legerdemain or sleight of hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dealing with witches and with <qex>conjurers</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>From the account the loser brings,<br/
The <qex>conjurer</qex> knows who stole the things.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who conjectures shrewdly or judges wisely; a man of sagacity.</def> <mark>[Obs.]</mark>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conjuring</ent><br/
<hw>conjuring</hw> <pos>n.</pos> <def>invoking a spirit or devil.  See <er>conjure</er>, <pos>v.</pos>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conjuror</ent><br/
<hw>Con*ju"ror</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One bound by a common oath with others.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conjury</ent><br/
<hw>Con"ju*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practice of magic; enchantment.</def>  <rj><au>Motley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conk</ent><br/
<hw>conk</hw> <pr>(k<ocr/<nsm/k)</pr>, <pos>n.</pos> <sn>1.</sn> <def>the head.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a blow to the head.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>conk</ent><br/
<hw>conk</hw> <pr>(k<ocr/<nsm/k)</pr>, <pos>v. t.</pos> <def>to hit on the head; <as>as, to <ex>conk</ex> someone on the head with a pipe</as>.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>conker</ent><br/
<hw>conk"er</hw> <pr>(k<ocr/<nsm/k"<etil/r)</pr>, <pos>n.</pos> <def>the inedible nutlike seed of the horsechestnut.</def><br/
<syn><b>Syn. --</b> buckeye, horsechestnut, horse-chestnut.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conn</ent><br/
<hw>Conn</hw> <pr>(k<ocr/n)</pr>, <pos>v. t.</pos> <def>See <er>Con</er>, to direct a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connascency</ent><br/
<ent>Connascence</ent><br/
<mhw>{ <hw>Con*nas"cence</hw> <pr>(?)</pr>, <hw>Con*nas"cen*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>con-</ets> + <ets>nascentia</ets> birth, fr. <ets>nascens</ets>, p. pr. of <ets>nasci</ets> to be born.]</ety> <sn>1.</sn> <def>The common birth of two or more at the same tome; production of two or more together.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is born or produced with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of growing together.</def> <mark>[Obs.]</mark>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connascent</ent><br/
<hw>Con*nas"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Born together; produced at the same time.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connate</ent><br/
<hw>Con"nate</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>connatus</ets>; <ets>con-</ets> + <ets>natus</ets> born, p. p. of <ets>nasci</ets>. <ets>See</ets> <er>Cognate</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Born with another; being of the same birth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Congenital; existing from birth.</def> <ldquo/<xex>Connate</xex> notions.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A difference has been made by some; those diseases or conditions which are dependent on original conformation being called <qex>congenital</qex>; while the diseases of affections that may have supervened during gestation or delivery are called <qex>connate</qex>.</q> <rj><qau>Dunglison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>Congenitally united; growing from one base, or united at their bases; united into one body; <as>as, <ex>connate</ex> leaves or athers</as>. See <xex>Illust.</xex> of <er>Connate-perfoliate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connate-perfoliate</ent><br/
<hw>Con"nate-per*fo"li*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Connate or coalescent at the base so as to produce a broad foliaceous body through the center of which the stem passes; -- applied to leaves, as the leaves of the boneset.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connation</ent><br/
<hw>Con*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Connection by birth; natural union.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connatural</ent><br/
<hw>Con*nat"u*ral</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[Pref. <ets>con-</ets> + <ets>natural</ets>.]</ety> <sn>1.</sn> <def>Connected by nature; united in nature; inborn; inherent; natural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These affections are <qex>connatural</qex> to us.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Partaking of the same nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And mix with our <qex>connatural</qex> dust.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connaturality</ent><br/
<hw>Con*nat`u*ral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Participation of the same nature; natural union or connection.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A congruity and <qex>connaturality</qex> between them.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connaturalize</ent><br/
<hw>Con*nat"u*ral*ize</hw> <pr>(?; 135)</pr>, <pos>v. t.</pos> <def>To bring to the same nature as something else; to adapt.</def> <mark>[Obs.]</mark>  <rj><au>Dr. J. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connaturally</ent><br/
<hw>Con*nat"u*ral*ly</hw>, <pos>adv.</pos> <def>By the act of nature; originally; from birth.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connaturalness</ent><br/
<hw>Con*nat"u*ral*ness</hw>, <pos>n.</pos> <def>Participation of the same nature; natural union.</def>  <rj><au>I. Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connature</ent><br/
<hw>Con*na"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>Participation in a common nature or character.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Connature</qex> was defined as likeness in kind between either two changes in consciousness, or two states of consciousness.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 305 --></p>

<p><ent>Connect</ent><br/
<hw>Con*nect"</hw> <pr>(k<ocr/n*n<ecr/kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Connected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Connecting</conjf>.]</vmorph> <ety>[L. <ets>connectere</ets>, <ets>-nexum</ets>; <ets>con-</ets> + <ets>nectere</ets> to bind. See <er>Annex</er>.]</ety> <sn>1.</sn> <def>To join, or fasten together, as by something intervening; to associate; to combine; to unite or link together; to establish a bond or relation between.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He fills, he bounds, <qex>connects</qex> and equals all.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man must see the connection of each intermediate idea with those that it <qex>connects</qex> before he can use it in a syllogism.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To associate (a person or thing, or one's self) with another person, thing, business, or affair.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To establish a communication link; -- used with <ptcl>with</ptcl>; <as>as, his telephone didn't answer, so I <ex>connected</ex> with him by email</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>To electronically or mechanically link (a device) to another device, or to link a device to a common communication line; -- used with <ptcl>with</ptcl>; <as>as, the installer <ex>connected</ex> our telephones on Monday; I <ex>connected</ex> my VCR to the TV set by myself; the plumber <ex>connected</ex> a shut-off valve to my gas line</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Connecting rod</b></col> <fld>(Mach.)</fld>, <cd>a rod or bar joined to, and connecting, two or more moving parts; esp. a rod connecting a crank wrist with a beam, crosshead, piston rod, or piston, as in a steam engine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connect</ent><br/
<hw>Con*nect"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To join, unite, or cohere; to have a close relation; <as>as, one line of railroad <ex>connects</ex> with another; one argument <ex>connects</ex> with another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>connected</ent><br/
<hw>connected</hw> <pos>adj.</pos> <sn>1.</sn> <def><pos>p. p.</pos> of <er>connect</er>.</def> [Narrower terms: <stype>abutting, adjacent, adjoining, bordering(prenominal), conterminous, coterminous, contiguous</stype>] [Narrower terms: <stype>adjunctive</stype>] [Narrower terms: <stype>affined</stype>] [Narrower terms: <stype>attached</stype>] [Narrower terms: <stype>contiguous, in contact</stype>] [Narrower terms: <stype>coupled, joined, linked</stype>] [Narrower terms: <stype>cursive, flowing</stype>] [Narrower terms: <stype>siamese</stype>] [Narrower terms: <stype>socially connected, well-connected</stype>] <ant>unconnected</ant><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>being joined in close association.</def><br/
<syn><b>Syn. --</b> affiliated, attached.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>connected by a conductor so as to allow the flow of electric signals.</def> [Narrower terms: <stype>wired (vs. wireless)</stype>] <source>WordNet 1.5</source>]</p>

<p><sn>4.</sn> <fld>(Music)</fld>  <def>legato.</def> <ant>staccato</ant><br/
<syn><b>Syn. --</b> flowing, smooth.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn>  <def>associated with or accompanying.</def><br/
<syn><b>Syn. --</b> associated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn> <fld>(Computers)</fld>  <def>stored in, controlled by, or in direct communication with a central computer.</def> [Narrower terms: <stype>on-line (vs. off-line), online, on line(predicate)</stype>]<br/
<syn><b>Syn. --</b> machine-accessible.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>7.</sn>  <def>switched on.</def> [Narrower terms: <stype>on-line (vs. off-line), online, on line(predicate)</stype>]<br/
<syn><b>Syn. --</b> ready, on.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>8.</sn>  <def>having some relation.</def><br/
<syn><b>Syn. --</b> related.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>connectedness</ent><br/
<hw>connectedness</hw> <pos>n.</pos> <sn>1.</sn> <def>the state of being connected.</def><br/
<syn><b>Syn. --</b> connection.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a relation between things or events (as in the case of one causing the other or sharing features with it).</def><br/
<syn><b>Syn. --</b> connection, connexion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>connecting</ent><br/
<hw>connecting</hw> <pos>adj.</pos> <sn>1.</sn> <def>having a connection; <as>as, <ex>connecting</ex> rooms</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>forming a connection; <as>as, a <ex>connecting</ex> corridor</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Grammar)</fld> <def>syntactically connecting sentences or elements of a sentence.</def><br/
<syn><b>Syn. --</b> copulative.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Connectedly</ent><br/
<hw>Con*nect"ed*ly</hw>, <pos>adv.</pos> <def>In a connected manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>connecter</ent><br/
<hw>connecter</hw> <pos>n.</pos> <def>same as <er>connector</er>.</def><br/
<syn><b>Syn. --</b> connection, connexion, connector, connective.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Connecticuter</ent><br/
<hw>Connecticuter</hw> <pos>n.</pos> <def>a resident of Connecticut.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>connecting</ent><br/
<hw>connecting</hw> <pos>n.</pos> <def>the act of bringing two things into contact.</def><br/
<syn><b>Syn. --</b> joining, connection, connexion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>connecting</ent><br/
<hw>connecting</hw> <pos>a.</pos> <def>forming a connection; <as>as, a <ex>connecting</ex> flight</as>.</def><br/
<syn><b>Syn. --</b> joining.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Connection</ent><br/
<hw>Con*nec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Connexion</er>.]</ety> <sn>1.</sn> <def>The act of connecting, or the state of being connected; the act or process of bringing two things into contact; junction; union; <as>as, the <ex>connection</ex> between church and state is inescapable; the <ex>connection</ex> of pipes of different diameters requires an adapter</as>.</def><br/
<syn><b>Syn. --</b> link, connectedness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which connects or joins together; bond; tie.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>any relationship between things or events; association; alliance; <as>as, a causal <ex>connection</ex> between interest rates and stock prices</as>.</def><br/
<syn><b>Syn. --</b> relation.</syn><br/
[<source>PJC</source>]</p>

<p><q>He [Algazel] denied the possibility of a known <qex>connection</qex> between cause and effect.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The eternal and inseparable <qex>connection</qex> between virtue and happiness.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Any sort of <qex>connection</qex> which is perceived or imagined between two or more things.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A relation; esp. a person connected with another by marriage rather than by blood; -- used in a loose and indefinite, and sometimes a comprehensive, sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The persons or things that are connected; <as>as, a business <ex>connection</ex>; the Methodist <ex>connection</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men elevated by powerful <qex>connection</qex>.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At the head of a strong parliamentary <qex>connection</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose names, forces, <qex>connections</qex>, and characters were perfectly known to him.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>something that connects other objects.</def><br/
<syn><b>Syn. --</b> connexion, connector, connecter, connective.</syn><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <def>(usually plural) an acquaintance or acquaintances who are influential or in a position of power and to whom you are connected in some way (as by family or friendship); <as>as, he has powerful <ex>connections</ex></as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>7.</sn> <def>a communications channel; <as>as, my cell phone had a bad <ex>connection</ex></as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>8.</sn> <fld>(Transportation)</fld> <def>a vehicle in which one may continue a journey after debarking from another vehicle; the departing vehicle of a <er>connection{9}</er>; <as>as, my <ex>connection</ex> leaves four hours after my arrival; I missed my <ex>connection</ex></as>.</def> <note>A <ex>connection</ex> may be more specifically referred to as a <stype>connecting flight</stype>, a <stype>connecting train</stype>, etc.</note><br/
[<source>PJC</source>]</p>

<p><sn>9.</sn> <fld>(Transportation)</fld> <def>the scheduled arrival of one vehicle and departure of a second, sufficiently close in time and place to allow the departing vehicle serve as a means of continuing a journey begun or continued in the first vehicle; <as>as, we can get a <ex>connection</ex> at Newark to continue on to Paris</as>; -- most commonly used of airplanes, trains, and buses arriving and departing at the same terminal.</def><br/
[<source>PJC</source>]</p>

<p><sn>10.</sn> <fld>(Transportation)</fld> <def>the transfer of a passenger from one vehicle to another to continue a journey; <as>as, the <ex>connection</ex> was made in Copenhagen</as>; -- most commonly of scheduled transportation on common carriers.</def><br/
[<source>PJC</source>]</p>

<p><sn>11.</sn> <fld>(Commerce)</fld> <def>a vendor who can supply desired materials at a favorable price, or under conditions when other sources are unavailable; <as>as, to get a bargain from one's <ex>connection</ex> in the jewelry trade; to have <ex>connections</ex> for the purchase of marijuana</as>; -- often used in the <pluf>pl.</pluf>.</def><br/
[<source>PJC</source>]</p>

<p><sn>12.</sn> <fld>(Psychol.)</fld> <def>the process of bringing ideas or events together in memory or imagination.</def><br/
<syn><b>Syn. --</b> association, connection, connexion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><cs><col><b>In this connection</b></col>, <cd>in connection with this subject. <note>[A phrase objected to by some writers.]</note></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word was formerly written, as by Milton, with <xex>x</xex> instead of <xex>t</xex> in the termination, <xex>connexion</xex>, and the same thing is true of the kindred words <xex>inflexion</xex>, <xex>reflexion</xex>, and the like. But the general usage at present is to spell them <xex>connection</xex>, <xex>inflection</xex>, <xex>reflection</xex>, etc.</note></p>

<p><syn><b>Syn.</b> -- Union; coherence; continuity; junction; association; dependence; intercourse; commerce; communication; affinity; relationship.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connective</ent><br/
<hw>Con*nect"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Connecting, or adapted to connect; involving connection.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Connection tissue</b></col> <fld>(Anat.)</fld> <cd>See <cref>Conjunctive tissue</cref>, under <er>Conjunctive</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connective</ent><br/
<hw>Con*nect"ive</hw>, <pos>n.</pos> <def>That which connects</def>. Specifically: <sd>(a)</sd> <fld>(Gram.)</fld> <def>A word that connect words or sentences; a conjunction or preposition.</def> <sd>(b)</sd> <fld>(Bot.)</fld> <def>That part of an anther which connects its thec<ae/, lobes, or cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connectively</ent><br/
<hw>Con*nect"ive*ly</hw>, <pos>adv.</pos> <def>In connjunction; jointly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connector</ent><br/
<hw>Con*nect"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, connects</def>; as: <sd>(a)</sd> <def>A flexible tube for connecting the ends of glass tubes in pneumatic experiments.</def> <sd>(b)</sd> <def>A device for holding two parts of an electrical conductor in contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conner</ent><br/
<hw>Con"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cunner</er>.]</ety> <fld>(Zool.)</fld> <def>A marine European fish (<spn>Crenilabrus melops</spn>); also, the related American cunner. See <er>Cunner</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connex</ent><br/
<hw>Con*nex"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>connexus</ets>, p. p. See <er>Connect</er>.]</ety> <def>To connect.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connexion</ent><br/
<hw>Con*nex"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>connexio</ets>: cf. F. <ets>connexion</ets>.]</ety> <def>Connection. See <er>Connection</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connexive</ent><br/
<hw>Con*nex"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Connective</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conning tower</ent><br/
<hw>Con"ning tow"er</hw> <pr>(?)</pr>, <sn>1.</sn> <pos>n.</pos> <def>The shot-proof pilot house of a war vessel.</def> <mark>[archaic]</mark> <br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the raised structure rising above the deck of a submarine.</def><br/
[<source>PJC</source>]</p>

<p><ent>Connivance</ent><br/
<hw>Con*niv"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>connivence</ets>, L. <ets>conniventia</ets>.]</ety> <sn>1.</sn> <def>Intentional failure or forbearance to discover a fault or wrongdoing; voluntary oversight; passive consent or cooperation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Corrupt or guilty assent to wrongdoing, not involving actual participation in, but knowledge of, and failure to prevent or oppose it.</def></p>

<p><syn><b>Syn.</b> -- See <er>Collusion</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connive</ent><br/
<hw>Con*nive"</hw> <pr>(k<ocr/n*n<imac/v")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Connived</conjf> <pr>(-n<imac/vd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conniving</conjf>.]</vmorph> <ety>[L. <ets>connivere</ets> to shut the eyes, connive, fr. <ets>con-</ets> + (perh.) a word akin to <ets>nicere</ets> to beckon, <ets>nictare</ets> to wink.]</ety> <sn>1.</sn> <def>To open and close the eyes rapidly; to wink.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The artist is to teach them how to nod judiciously, and to <qex>connive</qex> with either eye.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To close the eyes upon a fault; to wink (at); to fail or forbear by intention to discover an act; to permit a proceeding, as if not aware of it; -- usually followed by <xex>at</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>connive</qex> at what it does not approve.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In many of these, the directors were heartily concurring; in most of them, they were encouraging, and sometimes commanding; in all they were <qex>conniving</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The government thought it expedient, occasionally, to <qex>connive</qex> at the violation of this rule.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connive</ent><br/
<hw>Con*nive"</hw>, <pos>v. t.</pos> <def>To shut the eyes to; to overlook; to pretend not to see.</def> <mark>[R. & Obs.]</mark> <ldquo/Divorces were not <xex>connived</xex> only, but with eye open allowed.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connivency</ent><br/
<hw>Con*niv"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Connivance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connivent</ent><br/
<hw>Con*niv"ent</hw> <pr>(#)</pr>, <pos>a.</pos> <ety>[L. <ets>connivens</ets>, p. pr.]</ety> <sn>1.</sn> <def>Forbearing to see; designedly inattentive; <as>as, <ex>connivent</ex> justice</as>.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Brought close together; arched inward so that the points meet; converging; in close contact; <as>as, the <ex>connivent</ex> petals of a flower, wings of an insect, or folds of membrane in the human system, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conniver</ent><br/
<hw>Con*niv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who connives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connoisseur</ent><br/
<hw>Con`nois*seur"</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>connaisseur</ets>, formerly <ets>connoisseur</ets>, fr. <ets>conna<icir/tre</ets> to know, fr. L. <ets>cognoscere</ets> to become acquainted with; <ets>co-</ets> + <ets>noscere</ets>, <ets>gnoscere</ets>, to learn to know. See <er>Know</er>, and cf. <er>Cognizor</er>.]</ety> <def>One well versed in any subject; a skillful or knowing person; a critical judge of any art, particulary of one of the fine arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>connoisseur</qex> is <ldquo/one who knows,<rdquo/ as opposed to the dilettant, who only <ldquo/thinks he knows.<rdquo/</q> <rj><qau>Fairholt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connoisseurship</ent><br/
<hw>Con`nois*seur"ship</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <def>State of being a connoisseur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connotate</ent><br/
<hw>Con"no*tate</hw> <pr>(k<ocr/n"n<osl/*t<amac/t)</pr>, <pos>v. t.</pos> <ety>[L. <ets>con-</ets> + <ets>notatus</ets>, p. p.of <ets>notare</ets> to mark. Cf. <er>Connote</er>.]</ety> <def>To connote; to suggest or designate (something) as additional; to include; to imply.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connotation</ent><br/
<hw>Con`no*ta"tion</hw> <pr>(k<ocr/n`n<osl/*t<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>connotation</ets>.]</ety> <def>The act of connoting; a making known or designating something additional; implication of something more than is asserted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a meaning implied but not explicitly denoted by some word or expression, which may be understood in addition to the explicit primary meaning.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Logic)</fld> <def>the full set of necessary properties possessed by all the objects within the extension of a term; the intensional meaning of a term, which determines the objects to which the term applies; the intension of a term.</def><br/
<syn><b>Syn. --</b> intension.</syn>
[<source>PJC</source>]</p>

<p><ent>connotational</ent><br/
<hw>connotational</hw> <pos>adj.</pos> <def>Having the power of implying or suggesting something in addition to what is explicit.  Contrasted with <contr>denotative</contr>.</def><br/
<syn><b>Syn. --</b> connotative of (predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Connotative</ent><br/
<hw>Con*no"ta*tive</hw> <pr>(k<ocr/n*n<omac/"t<adot/*t<icr/v <or/ k<ocr/n"n<osl/*t<asl/*t<icr/v)</pr>, <pos>a.</pos>  <sn>1.</sn> <def>Implying something additional; illative.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Log.)</fld> <def>Implying an attribute. See <er>Connote</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Connotative term</b></col>, <cd>one which denotes a subject and implies an attribute.</cd>  <rj><au>J. S. Mill.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connotatively</ent><br/
<hw>Con*no"ta*tive*ly</hw>, <pos>adv.</pos> <def>In a connotative manner; expressing connotation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>connote</ent><br/
<hw>con*note"</hw> <pr>(k<ocr/n*n<omac/t")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>connoted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>connoting</conjf>.]</vmorph> <ety>[See <er>connotate</er>, and <er>cote</er>.]</ety> <sn>1.</sn> <def>To mark along with; to suggest or indicate as additional; to designate by implication; to include in the meaning; to imply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Good, in the general notion of it, <qex>connotes</qex> also a certain suitableness of it to some other thing.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>To imply as an attribute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The word <ldquo/white<rdquo/ denotes all white things, as snow, paper, the foam of the sea, etc., and ipmlies, or as it was termed by the schoolmen, <qex>connotes</qex>, the attribute <ldquo/whiteness.<rdquo/</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connubial</ent><br/
<hw>Con*nu"bi*al</hw> <pr>(#)</pr>, <pos>a.</pos> <ety>[L. <ets>connubialis</ets>, fr. <ets>connubium</ets> marriage; <ets>con-</ets> + <ets>nubere</ets> to veil, to marry. See <er>Nupital</er>.]</ety> <def>Of or pertaining to marriage, or the marriage state; conjugal; nuptial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor Eve the rites<br/
Mysterious of <qex>connubial</qex> love refused.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Kind, <qex>connubial</qex> tenderness.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connubiality</ent><br/
<hw>Con*nu`bi*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being connubial; something characteristics of the conjugal state; an expression of connubial tenderness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>connubialities</qex> which had begun to pass between Mr. and Mrs. B.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connumeration</ent><br/
<hw>Con*nu`mer*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>connumeratio</ets>, fr. L. <ets>connumerare</ets>, <ets>-numeratum</ets>, to number with.]</ety> <def>A reckoning together.</def> <mark>[R.]</mark>  <rj><au>Porson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connusance</ent><br/
<hw>Con"nu*sance</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>See <er>Cognizance</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connusant</ent><br/
<hw>Con"nu*sant</hw> <pr>(#)</pr>, <pos>a.</pos> <fld>(Law)</fld> <def>See <er>Cognizant</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connusor</ent><br/
<hw>Con`nu*sor"</hw> <pr>(#)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>See <er>Cognizor</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Connutritious</ent><br/
<hw>Con`nu*tri"tious</hw> <pr>(#)</pr>, <pos>a.</pos> <def>Nutritious by force of habit; -- said of certain kinds of food.</def> <mark>[Obs.]</mark>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conny</ent><br/
<hw>Con"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<root/45. Cf. <er>Canny</er>, <er>Gunning</er>.]</ety> <def>Brave; fine; canny.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conodont</ent><br/
<hw>Co"no*dont</hw> <pr>(k<omac/"n<osl/*d<ocr/nt)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kw^nos</grk> cone + <grk>'odoy`s</grk>, <grk>'odo`ntos</grk>, tooth.]</ety> <fld>(Zool.)</fld> <def>A peculiar toothlike fossil of many forms, found especially in carboniferous rocks. Such fossils are supposed by some to be the teeth of marsipobranch fishes, but they are probably the jaws of annelids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conodontophorida</ent><br/
<ent>Conodonta</ent><br/
<mhw><hw>Conodonta</hw>, <hw>Conodontophorida</hw></mhw> <pos>n.</pos> <def>an extinct order of primitive vertebrates; the precise taxonomy is not clear; in some classifications it is considered a separate phylum.</def><br/
<syn><b>Syn. --</b> order <ord>Conodonta</ord>, Conodontophorida, order <ord>Conodontophorida</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conoid</ent><br/
<hw>Co"noid</hw> <pr>(k<omac/"noid)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kwnoeidh`s</grk> conical; <grk>kw^nos</grk> cone + <grk>e'i^dos</grk> form: cf. F. <ets>cono<ium/de</ets>.]</ety> <sn>1.</sn> <def>Anything that has a form resembling that of a cone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <sd>(a)</sd> <def>A solid formed by the revolution of a conic section about its axis; <as>as, a parabolic <ex>conoid</ex>, elliptic conoid, etc.</as>; -- more commonly called <altname>paraboloid</altname>, <altname>ellipsoid</altname>, etc.</def> <sd>(b)</sd> <def>A surface which may be generated by a straight line moving in such a manner as always to meet a given straight line and a given curve, and continue parallel to a given plane.</def>  <rj><au>Math. Dict.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conoid</ent><br/
<hw>Co"noid</hw> <pos>a.</pos> <def>Resembling a cone; conoidal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conoidal</ent><br/
<hw>Co*noid"al</hw> <pr>(#)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cono<ium/dal</ets>.]</ety> <def>Nearly, but not exactly, conical.</def>  <rj><au>Lindley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conoidical</ent><br/
<ent>Conoidic</ent><br/
<mhw>{ <hw>Co*noid"ic</hw> <pr>(?)</pr>, <hw>Co*noid"ic*al</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <def>Pertaining to a conoid; having the form of a conoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conominee</ent><br/
<hw>Co*nom`i*nee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One nominated in conjunction with another; a joint nominee.</def>  <rj><au>Kirby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquadrate</ent><br/
<hw>Con*quad"rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>conquadratus</ets>, p. p. of <ets>conquadrare</ets>.]</ety> <def>To bring into a square.</def> <mark>[R.]</mark>  <rj><au>Ash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquassate</ent><br/
<hw>Con*quas"sate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>conquassatus</ets>, p. p. of <ets>conquassare</ets>.]</ety> <def>To shake; to agitate.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj></p>

<p>-- <wordforms><wf>Con`quas*sa"tion</wf> <pr>(#)</pr>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquer</ent><br/
<hw>Con"quer</hw> <pr>(k<ocr/<nsm/"k<etil/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conquered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conquering</conjf>.]</vmorph> <ety>[OF. <ets>conquerre</ets>, F. <ets>conqu<eacute/rir</ets>, fr. L. <ets>conquirere</ets>, <ets>-quisitum</ets>, to seek or search for, to bring together, LL., to conquer; <ets>con-</ets> + <ets>quaerere</ets> to seek. See <er>Quest</er>.]</ety> <sn>1.</sn> <def>To gain or acquire by force; to take possession of by violent means; to gain dominion over; to subdue by physical means; to reduce; to overcome by force of arms; to cause to yield; to vanquish.</def> <ldquo/If thou <xex>conquer</xex> Rome.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If we be <qex>conquer'd</qex>, let men <qex>conquer</qex> us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>conquered</qex> France, but felt our captive's charms.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subdue or overcome by mental or moral power; to surmount; <as>as, to <ex>conquer</ex> difficulties, temptation, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>By winning words to <qex>conquer</qex> hearts,<br/
And make persuasion do the work of fear.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To gain or obtain, overcoming obstacles in the way; to win; <as>as, to <ex>conquer</ex> freedom; to <ex>conquer</ex> a peace</as>.</def></p>

<p><syn><b>Syn.</b> -- To subdue; vanquish; overcome; overpower; overthrow; defeat; rout; discomfit; subjugate; reduce; humble; crush; surmount; subject; master.</syn> <usage> -- <er>To Conquer</er>, <er>Vanquish</er>, <er>Subdue</er>, <er>Subjugate</er>, <er>Overcome</er>. These words agree in the general idea expressed by <xex>overcome</xex>, -- that of bringing under one's power by the exertion of force. <xex>Conquer</xex> is wider and more general than <xex>vanquish</xex>, denoting usually a succession of conflicts. <xex>Vanquish</xex> is more individual, and refers usually to a single conflict. Thus, Alexander <xex>conquered</xex> Asia in a succession of battles, and <xex>vanquished</xex> Darius in one decisive engagement. <xex>Subdue</xex> implies a more gradual and continual pressure, but a surer and more final subjection. We speak of a nation as <xex>subdued</xex> when its spirit is at last broken, so that no further resistance is offered. <xex>Subjugate</xex> is to bring completely under the yoke of bondage. The ancient Gauls were never finally <xex>subdued</xex> by the Romans until they were completely <xex>subjugated</xex>. These words, when used figuratively, have correspondent meanings. We <xex>conquer</xex> our prejudices or aversions by a succesion of conflicts; but we sometimes <xex>vanquish</xex> our reluctance to duty by one decided effort: we endeavor to <xex>subdue</xex> our evil propensities by watchful and persevering exertions. <xex>Subjugate</xex> is more commonly taken in its primary meaning, and when used figuratively has generally a bad sense; as, his reason was completely <xex>subjugated</xex> to the sway of his passions.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquer</ent><br/
<hw>Con"quer</hw> <pr>(k<ocr/<nsm/"k<etil/r)</pr>, <pos>v. i.</pos> <def>To gain the victory; to overcome; to prevail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He went forth <qex>conquering</qex> and to <qex>conquer</qex>.</q> <rj><qau>Rev. vi. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The champions resolved to <qex>conquer</qex> or to die.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquerable</ent><br/
<hw>Con*quer*a*ble</hw> <pr>(k<ocr/<nsm/"k<etil/r*<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being conquered or subdued.</def>  <rj><au>South.</au></rj></p>

<p>-- <wordforms><wf>Con"quer*a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conqueress</ent><br/
<hw>Con"quer*ess</hw>, <pos>n.</pos> <def>A woman who conquers.</def>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conquering</ent><br/
<hw>conquering</hw> <pos>adj. prenom.</pos>  <def><pos>pr. p.</pos> of <er>conquer</er>.</def> [Narrower terms: <stype>undefeated (vs. defeated)</stype>] <source>WordNet 1.5</source>]</p>

<p><ent>conquering</ent><br/
<hw>conquering</hw> <pos>n.</pos> <def>the act of conquering.</def><br/
<syn><b>Syn. --</b> conquest, enslavement, subjection, subjugation.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conqueror</ent><br/
<hw>Con"quer*or</hw> <pr>(k<ocr/<nsm/"k<etil/r*<etil/r)</pr>, <pos>n.</pos> <ety>[OF. <ets>conquereor</ets>, fr. <ets>conquerre</ets>,]</ety> <def>One who conquers.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Conqueror</b></col> <fld>(Eng. Hist.)</fld>. <cd>William the Norman (1027-1067) who invaded England, defeated Harold in the battle of Hastings, and was crowned king, in 1066.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquest</ent><br/
<hw>Con"quest</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>conquest</ets>, <ets>conqueste</ets>, F. <ets>conqu<ecir/te</ets>, LL. <ets>conquistum</ets>, <ets>conquista</ets>, prop. p. p. from L. <ets>conquirere</ets>. See <er>Conquer</er>.]</ety> <sn>1.</sn> <def>The act or process of conquering, or acquiring by force; the act of overcoming or subduing opposition by force, whether physical or moral; subjection; subjugation; victory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>In joys of conquest</qex> he resigns his breath.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Three years sufficed for the <qex>conquest</qex> of the country.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is conquered; possession gained by force, physical or moral.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherefore rejoice? What <qex>conquest</qex> brings he home?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Feudal Law)</fld> <def>The acquiring of property by other means than by inheritance; acquisition.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of gaining or regaining by successful struggle; <as>as, the <ex>conquest</ex> of liberty or peace</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Conquest</b></col> <fld>(Eng. Hist.)</fld>, <cd>the subjugation of England by William of Normandy in 1066.  The <altname><col><b>Norman Conquest</b></col></altname>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Victory; triumph; mastery; reduction; subjugation; subjection.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conquian</ent><br/
<hw>Con"qui*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Card Playing)</fld> <def>A game for two, played with 40 cards, in which each player tries to form three or four of a kind or sequences.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Consanguine</ent><br/
<hw>Con*san"guine</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>related by blood; descended from a common ancestor; -- used as a term of relation between two people.</def><br/
<syn><b>Syn. --</b> akin(predicate), blood-related, cognate, consanguineous, kin(predicate).</syn><br/
[<source>PJC</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>having the same father but different mothers; -- contrasted with <contr><er>uterine{2}</er></contr> and <contr><er>german</er></contr>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Consanguineal</ent><br/
<hw>Con`san*guin"e*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the same blood; related by birth.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consanguined</ent><br/
<hw>Con*san"guined</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of kin blood; related.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consanguineous</ent><br/
<hw>Con`san*guin"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conguineus</ets>; <ets>con-</ets> + <ets>sanguis</ets> blood: cf. F. <ets>consanguin</ets>. See <er>Sanquine</er>.]</ety> <def>Of the same blood; related by birth; descended from the same parent or ancestor.</def>  <rj><au>Shak.</au></rj><br/
<syn><b>Syn. --</b> consanguine.</syn>
[<source>1913 Webster</source>]</p>

<p><ent>Consanguinity</ent><br/
<hw>Con`san*guin"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consanguinitas</ets>: cf. F. <ets>consanguintit<eacute/</ets>.]</ety> <def>The relation of persons by blood, in distinction from <xex>affinity</xex> or relation by marriage; blood relationship; <as>as, lineal <ex>consanguinity</ex>; collateral <ex>consanguinity</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Invoking aid by the ties of <qex>consanguinity</qex>.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consarcination</ent><br/
<hw>Con*sar`ci*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consarcinare</ets>, <ets>-natum</ets>, to patch together.]</ety> <def>A patching together; patchwork.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscience</ent><br/
<hw>Con"science</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>conscience</ets>, fr. L. <ets>conscientia</ets>, fr. consciens, p. pr. of conscire to know, to be conscious; <ets>con-</ets> + <ets>scire</ets> to know. See <er>Science</er>.]</ety> <sn>1.</sn> <def>Knowledge of one's own thoughts or actions; consciousness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sweetest cordial we receive, at last,<br/
Is <qex>conscience</qex> of our virtuous actions past.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The faculty, power, or inward principle which decides as to the character of one's own actions, purposes, and affections, warning against and condemning that which is wrong, and approving and prompting to that which is right; the moral faculty passing judgment on one's self; the moral sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>conscience</qex> hath a thousand several tongues,<br/
And every tongue brings in a several tale,<br/
And every tale condemns me for a villain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>science</qex> means <qex>knowledge</qex>, <qex>conscience</qex> etymologically means <qex>self-knowledge</qex> . . . But the English word implies a moral standard of action in the mind as well as a consciousness of our own actions. . . . <qex>Conscience</qex> is the reason, employed about questions of right and wrong, and accompanied with the sentiments of approbation and condemnation.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The estimate or determination of conscience; conviction or right or duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conscience</qex> supposes the existence of some such [<it>i.e.</it>, moral] faculty, and properly signifies our consciousness of having acted agreeably or contrary to its directions.</q> <rj><qau>Adam Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Tenderness of feeling; pity.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conscience clause</b></col>, <cd>a clause in a general law exempting persons whose religious scruples forbid compliance therewith, -- as from taking judicial oaths, rendering military service, etc.</cd> -- <col><b>Conscience money</b></col>, <cd>stolen or wrongfully acquired money that is voluntarily restored to the rightful possessor. Such money paid into the United States treasury by unknown debtors is called the <xex>Conscience fund</xex>.</cd> -- <col><b>Court of Conscience</b></col>, <cd>a court established for the recovery of small debts, in London and other trading cities and districts.</cd> <mark>[Eng.]</mark> <au>Blackstone.</au> -- <mcol><col><b>In conscience</b></col>, <col><b>In all conscience</b></col></mcol>, <cd>in deference or obedience to conscience or reason; in reason; reasonably.</cd> <ldquo/This is enough <xex>in conscience</xex>.<rdquo/ <au>Howell.</au> <ldquo/Half a dozen fools are, <xex>in all conscience</xex>, as many as you should require.<rdquo/ <au>Swift.</au> -- <mcol><col><b>To make conscience of</b></col>, <col><b>To make a matter of conscience</b></col></mcol>, <cd>to act according to the dictates of conscience concerning (any matter), or to scruple to act contrary to its dictates.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscienced</ent><br/
<hw>Con"scienced</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a conscience.</def> <mark>[R.]</mark> <ldquo/Soft-<qex>conscienced</qex> men.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscienceless</ent><br/
<hw>Con"science*less</hw>, <pos>a.</pos> <def>Without conscience; indifferent to conscience; unscrupulous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conscienceless</qex> and wicked patrons.</q> <rj><qau>Hookre.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscient</ent><br/
<hw>Con"scient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consciens</ets>, <ets>-entis</ets>, p. pr.]</ety> <def>Conscious.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscientious</ent><br/
<hw>Con`sci*en"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>consciencieux</ets>, LL. <ets>conscientiosus</ets>.]</ety> <sn>1.</sn> <def>Influenced by conscience; governed by a strict regard to the dictates of conscience, or by the known or supposed rules of right and wrong; -- said of a person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The advice of wise and <qex>conscientious</qex> men.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 306 --></p>

<p><sn>2.</sn> <def>Characterized by a regard to conscience; conformed to the dictates of conscience; -- said of actions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A holy and <qex>conscientious</qex> course.</q> <rj><qau>Abp. Tillotson.</qau></rj></p>

<p><syn><b>Syn.</b> -- Scrupulous; exact; faithful; just; upright.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscientiously</ent><br/
<hw>Con`sci*en"tious*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a conscientious manner; as a matter of conscience; hence; faithfully; accurately; completely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscientiousness</ent><br/
<hw>Con`sci*en"tious*ness</hw>, <pos>n.</pos> <def>The quality of being conscientious; a scrupulous regard to the dictates of conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscionable</ent><br/
<hw>Con"scion*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Irregularly formed fr. <ets>conscience</ets>.]</ety> <def>Governed by, or according to, conscience; reasonable; just.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let my debtors have <qex>conscionable</qex> satisfaction.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscionableness</ent><br/
<hw>Con"scion*a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being conscionable; reasonableness.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscionably</ent><br/
<hw>Con"scion*a*bly</hw>, <pos>adv.</pos> <def>Reasonably; justly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscious</ent><br/
<hw>Con"scious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conscius</ets>; <ets>con-</ets> + <ets>scire</ets> to know. See <er>Conscience</er>.]</ety> <sn>1.</sn> <def>Possessing the faculty of knowing one's own thoughts or mental operations.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some are thinking or <qex>conscious</qex> beings, or have a power of thought.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Possessing knowledge, whether by internal, conscious experience or by external observation; cognizant; aware; sensible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her <qex>conscious</qex> heart imputed suspicion where none could have been felt.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The man who breathes most healthilly is least <qex>conscious</qex> of his own breathing.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Made the object of consciousness; known to one's self; <as>as, <ex>conscious</ex> guilt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>conscious</qex> terrors vex me round.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> -- Aware; apprised; sensible; felt; known.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consciously</ent><br/
<hw>Con"scious*ly</hw>, <pos>adv.</pos> <def>In a conscious manner; with knowledge of one's own mental operations or actions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consciousness</ent><br/
<hw>Con"scious*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state of being conscious; knowledge of one's own existence, condition, sensations, mental operations, acts, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consciousness</qex> is thus, on the one hand, the recognition by the mind or <ldquo/ego<rdquo/ of its acts and affections; -- in other words, the self-affirmation that certain modifications are known by me, and that these modifications are mine.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Immediate knowledge or perception of the presence of any object, state, or sensation. See the Note under <er>Attention</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Annihilate the <qex>consciousness</qex> of the object, you annihilate the <qex>consciousness</qex> of the operation.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And, when the steam<br/
Which overflowed the soul had passed away,<br/
A <qex>consciousness</qex> remained that it had left.<br/
 . . . images and precious thoughts<br/
That shall not die, and can not be destroyed.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consciousness</qex> of wrong brought with it the <qex>consciousness</qex> of weakness.</q> <rj><qau>Froude.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Feeling, persuasion, or expectation; esp., inward sense of guilt or innocence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>An honest mind is not in the power of a dishonest: to break its peace there must be some guilt or <qex>consciousness</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscribe</ent><br/
<hw>Con*scribe"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>conscribere</ets>. See <er>Conscript</er>.]</ety> <def>To enroll; to enlist.</def> <mark>[Obs.]</mark>  <rj><au>E. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscript</ent><br/
<hw>Con"script</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conscriptus</ets>, p. p. of <ets>conscribere</ets> to write together, to enroll; <ets>con-</ets> + <ets>scribere</ets> to write. See <er>Scribe</er>.]</ety> <def>Enrolled; written; registered.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conscript fathers</b></col> <fld>(Rom. Antiq.)</fld>, <cd>the senators of ancient Rome. When certain new senators were first enrolled with the <ldquo/fathers<rdquo/ the body was called <xex>Patres et Conscripti</xex>; afterward all were called <xex>Patres conscripti</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscript</ent><br/
<hw>Con"script</hw>, <pos>n.</pos> <def>One taken by lot, or compulsorily enrolled, to serve as a soldier or sailor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscript</ent><br/
<hw>Con*script"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To enroll, by compulsion, for military service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscription</ent><br/
<hw>Con*scrip"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conscriptio</ets>: cf. F. <ets>conscription</ets>.]</ety> <sn>1.</sn> <def>An enrolling or registering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conscription</qex> of men of war.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A compulsory enrollment of men for military or naval service; a draft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conscription</ent><br/
<hw>Con*scrip"tion</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to, or of the nature of, a conspiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecrate</ent><br/
<hw>Con"se*crate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consceratus</ets>, p. p. of <ets>conscerare</ets> to conscerate; <ets>con-</ets> + <ets>sacrare</ets> to consecrate, <ets>sacer</ets> sacred. See <er>Sacred</er>.]</ety> <def>Consecrated; devoted; dedicated; sacred.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They were assembled in that <qex>consecrate</qex> place.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecrate</ent><br/
<hw>Con"se*crate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consecrated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Consecrating</conjf>.]</vmorph> <sn>1.</sn> <def>To make, or declare to be, sacred; to appropriate to sacred uses; to set apart, dedicate, or devote, to the service or worship of God; <as>as, to <ex>consecrate</ex> a church; to give (one's self) unreservedly, as to the service of God</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One day in the week is . . . <qex>consecrated</qex> to a holy rest.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To set apart to a sacred office; <as>as, to <ex>consecrate</ex> a bishop</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt <qex>consecrate</qex> Aaron and his sons.</q> <rj><qau>Ex. xxix. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To canonize; to exalt to the rank of a saint; to enroll among the gods, as a Roman emperor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To render venerable or revered; to hallow; to dignify; <as>as, rules or principles <ex>consecrated</ex> by time</as>.</def>  <rj><au>Burke.</au></rj></p>

<p><syn><b>Syn.</b> -- See <er>Addict</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecrater</ent><br/
<hw>Con"se*cra`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Consecrator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecration</ent><br/
<hw>Con`se*cra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consecratio</ets>: cf. F. <ets>cons<eacute/cration</ets>.]</ety> <def>The act or ceremony of consecrating; the state of being consecrated; dedication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Until the days of your <qex>consecration</qex> be at an end.</q> <rj><qau>Lev. viii. 33.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consecration</qex> makes not a place sacred, but only solemnly declares it so.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecrator</ent><br/
<hw>Con"se*cra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who consecrates; one who performs the rites by which a person or thing is devoted or dedicated to sacred purposes.</def> <altsp>[Written also <asp>consecrater</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecratory</ent><br/
<hw>Con"se*cra*to*ry</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos>  <def>Of or pertaining to the act of consecration; dedicatory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consecratory</qex> prayer.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consectaneous</ent><br/
<hw>Con`sec*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consectaneus</ets>.]</ety> <def>Following as a matter of course.</def> <rj><qau>Blount.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consectary</ent><br/
<hw>Con"sec*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consectarius</ets>, fr. <ets>consectari</ets> to follow after eagerly; <ets>con-</ets> + <ets>sectari</ets> to follow eagerly, fr. <ets>sequi</ets> to follow.]</ety> <def>Following by consequence; consequent; deducible.</def> <mark>[R.]</mark> <ldquo/<xex>Consectary</xex> impieties.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consectary</ent><br/
<hw>Con"sec*ta*ry</hw>, <pos>n.</pos> <def>That which follows by consequence or is logically deducible; deduction from premises; corollary.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecute</ent><br/
<hw>Con"se*cute</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To follow closely; to endeavor to overtake; to pursue.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecution</ent><br/
<hw>Con`se*cu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consecutio</ets>. See <er>Consequent</er>.]</ety> <sn>1.</sn> <def>A following, or sequel; actual or logical dependence.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A succession or series of any kind.</def> <mark>[Obs.]</mark>  <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Month of consecution</b></col> <fld>(Astron.)</fld>, <cd>a month as reckoned from one conjunction of the moon with the sun to another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecutive</ent><br/
<hw>Con*sec"u*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cons<eacute/cutif</ets>. See <er>Consequent</er>.]</ety> <sn>1.</sn> <def>Following in a train; succeeding one another in a regular order; successive; uninterrupted in course or succession; with no interval or break; <as>as, fifty <ex>consecutive</ex> years</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Following as a consequence or result; actually or logically dependent; consequential; succeeding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The actions of a man <qex>consecutive</qex> to volition.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>Having similarity of sequence; -- said of certain parallel progressions of two parts in a piece of harmony; <as>as, <ex>consecutive</ex> fifths, or <ex>consecutive</ex> octaves, which are forbidden</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consecutive chords</b></col> <fld>(Mus.)</fld>, <cd>chords of the same kind succeeding one another without interruption.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecutively</ent><br/
<hw>Con*sec"u*tive*ly</hw>, <pos>adv.</pos> <def>In a consecutive manner; by way of sequence; successively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consecutiveness</ent><br/
<hw>Con*sec"u*tive*ness</hw>, <pos>n.</pos> <def>The state or quality of being consecutive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consension</ent><br/
<hw>Con*sen"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consensio</ets>.]</ety> <def>Agreement; accord.</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consensual</ent><br/
<hw>Con*sen"su*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Consent</er>, <pos>v. i.</pos>, and cf. <er>Sensual</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>Existing, or made, by the mutual consent of two or more parties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>Excited or caused by sensation, sympathy, or reflex action, and not by conscious volition; <as>as, <ex>consensual</ex> motions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consensual contract</b></col> <fld>(Law)</fld>, <cd>a contract formed merely by consent, as a marriage contract.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consensus</ent><br/
<hw>Con*sen"sus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Consent</er>.]</ety> <def>Agreement; accord; consent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That traditional <qex>consensus</qex> of society which we call public opinion.</q> <rj><qau>Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consent</ent><br/
<hw>Con*sent"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Consenting</conjf>.]</vmorph> <ety>[F. <ets>consentir</ets>, fr. L. <ets>consentire</ets>, <ets>-sensum</ets>, to feel together, agree; <ets>con-</ets> + <ets>sentire</ets> to feel. See <er>Sense</er>.]</ety> <sn>1.</sn> <def>To agree in opinion or sentiment; to be of the same mind; to accord; to concur.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And Saul was <qex>consenting</qex> unto his death.</q> <rj><qau>Acts. viii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Flourishing many years before Wyclif, and much <qex>consenting</qex> with him in jugdment.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To indicate or express a willingness; to yield to guidance, persuasion, or necessity; to give assent or approval; to comply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My poverty, but not my will, <qex>consents</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And whispering <ldquo/I will ne'er consent,<rdquo/ -- <qex>consented</qex>.</q> <rj><qau>Byron.</qau></rj></p>

<p><syn><b>Syn.</b> -- To accede; yield; assent; comply; agree; allow; concede; permit; admit; concur; acquiesce.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consent</ent><br/
<hw>Con*sent"</hw>, <pos>v. t.</pos> <def>To grant; to allow; to assent to; to admit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Interpreters . . . will not <qex>consent</qex> it to be a true story.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consent</ent><br/
<hw>Con*sent"</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>consent</ets>.]</ety> <sn>1.</sn> <def>Agreement in opinion or sentiment; the being of one mind; accord.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All with one <qex>consent</qex> began to make excuse.</q> <rj><qau>Luke xiv. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They fell together all, as by <qex>consent</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Correspondence in parts, qualities, or operations; agreement; harmony; coherence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The melodious <qex>consent</qex> of the birds.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such is the world's great harmony that springs<br/
From union, order, full <qex>consent</qex> of things.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Voluntary accordance with, or concurrence in, what is done or proposed by another; acquiescence; compliance; approval; permission.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou wert possessed of David's throne<br/
By free <qex>consent</qex> of all.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Capable, deliberate, and voluntary assent or agreement to, or concurrence in, some act or purpose, implying physical and mental power and free action.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physiol.)</fld> <def>Sympathy. See <er>Sympathy</er>, 4.</def></p>

<p><syn><b>Syn.</b> -- Assent; acquiescence; concurrence; agreement; approval; permission. See <er>Assent</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Age of consent</b></col> <fld>(Law)</fld>, <cd>an age, fixed by statute and varying in different jurisdictions, at which one is competent to give consent. Sexual intercourse with a female child under the age of consent is punishable as rape.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consentaneity</ent><br/
<hw>Con*sen`ta*ne"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Mutual agreement.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consentaneous</ent><br/
<hw>Con`sen*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consentaneus</ets>.]</ety> <def>Consistent; agreeable; suitable; accordant to; harmonious; concurrent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A good law and <qex>consentaneous</qex> to reason.</q> <rj><qau>Howell.</qau></rj></p>

<p>-- <wordforms><wf>Con`sen*ta"ne*ous*ly</wf>, <pos>adv.</pos> -- <wf>Con`sen*ta"ne*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consentant</ent><br/
<hw>Con*sent"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., p. pr. of <ets>consentir</ets>.]</ety> <def>Consenting.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consenter</ent><br/
<hw>Con*sent"er</hw> <pr>(?)</pr>, <pos>a.</pos> <def>One who consents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consentient</ent><br/
<hw>Con*sen"tient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consentients</ets>, p. pr. See <er>Consent</er>.]</ety> <def>Agreeing in mind; accordant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consentient</qex> judgment of the church.</q> <rj><qau>Bp. Pearson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consentingly</ent><br/
<hw>Con*sent"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With consent; in a compliant manner.</def>
 <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequence</ent><br/
<hw>Con"se*quence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., <ets>consequentia</ets>: cf. F. <ets>cons<eacute/quence</ets>. See <er>Consequent</er>.]</ety> <sn>1.</sn> <def>That which follows something on which it depends; that which is produced by a cause; a result.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shun to taste,<br/
And shun the bitter <qex>consequence</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>A proposition collected from the agreement of other previous propositions; any conclusion which results from reason or argument; inference.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Chain of causes and effects; consecution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such fatal <qex>consequence</qex> unites us three.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Link follows link by necessary <qex>consequence</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Importance with respect to what comes after; power to influence or produce an effect; value; moment; rank; distinction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a matter of small <qex>consequence</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A sense of your own worth and <qex>consequence</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In consequence</b></col>, <cd>hence; for this cause.</cd> -- <col><b>In consequence of</b></col>, <cd>by reason of; as the effect of.</cd></cs></p>

<p><syn><b>Syn.</b> -- Effect; result; end. See <er>Effect</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequencing</ent><br/
<hw>Con"se*quen`cing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Drawing inference.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequent</ent><br/
<hw>Con"se*quent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consequens</ets>, <ets>-entis</ets>, p. pr. of <ets>consequi</ets> to follow; <ets>con-</ets> + <ets>sequi to follow</ets>: cf. F. <ets>cons<eacute/quent</ets>. See <er>Second</er>, and cf. <er>Consecution</er>.]</ety> <sn>1.</sn> <def>Following as a result, inference, or natural effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The right was <qex>consequent</qex> to, and built on, an act perfectly personal.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>Following by necessary inference or rational deduction; <as>as, a proposition <ex>consequent</ex> to other propositions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Consequent points</b></col>, <col><b>Consequent poles</b></col></mcol> <fld>(Magnetism)</fld>, <cd>a number of poles distributed under certain conditions, along the axis of a magnetized steel bar, which regularly has but the two poles at the extremities.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequent</ent><br/
<hw>Con"se*quent</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which follows, or results from, a cause; a result or natural effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They were ill-governed, which is always a <qex>consequent</qex> of ill payment.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>That which follows from propositions by rational deduction; that which is deduced from reasoning or argumentation; a conclusion, or inference.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Math.)</fld> <def>The second term of a ratio, as the term <xex>b</xex> in the ratio <mathex>a:b</mathex>, the first <xex>a</xex>, being the <xex>antecedent</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequential</ent><br/
<hw>Con`se*quen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Following as a consequence, result, or logical inference; consequent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All that is revealed in Scripture has a <qex>consequential</qex> necessity of being believed . . . because it is of divine authority.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These kind of arguments . . . are highly <qex>consequential</qex> and concludent to my purpose.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Assuming or exhibiting an air of consequence; pretending to importance; pompous; self-important; <as>as, a <ex>consequential</ex> man</as>. See <er>Consequence</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His stately and <qex>consequential</qex> pace.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consequential damage</b></col> <fld>(Law)</fld> <sd>(a)</sd> <cd>Damage so remote as not to be actionable</cd> <sd>(b)</sd> <cd>Damage which although remote is actionable.</cd> <sd>(c)</sd> <cd>Actionable damage, but not following as an immediate result of an act.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequentially</ent><br/
<hw>Con`se*quen"tial*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>With just deduction of consequence; with right connection of ideas; logically.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The faculty of writing <qex>consequentially</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>By remote consequence; not immediately; eventually; <as>as, to do a thing <ex>consequentially</ex></as>.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In a regular series; in the order of cause and effect; with logical concatenation; consecutively; continuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>With assumed importance; pompously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequentialness</ent><br/
<hw>Con`se*quen"tial*ness</hw>, <pos>n.</pos> <def>The quality of being consequential.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consequently</ent><br/
<hw>Con"se*quent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By consequence; by natural or logical sequence or connection.</def></p>

<p><syn><b>Syn.</b> -- See <er>Accordingly</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consertion</ent><br/
<hw>Con*ser"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consertio</ets>, fr. <ets>conserere</ets>, <ets>-sertum</ets> to connect; <ets>con-</ets> + <ets>serere</ets> to join.]</ety> <def>Junction; adaptation</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consertion</qex> of design, how exquisite.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservable</ent><br/
<hw>Con*serv"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conservabilitis</ets>.]</ety> <def>Capable of being preserved from decay or injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservancy</ent><br/
<hw>Con*serv"an*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Conservation, as from injury, defilement, or irregular use.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[An act was] passed in 1866, for vesting in the Conservators of the River Thames the <qex>conservancy</qex> of the Thames and Isis.</q> <rj><qau>Mozley & W.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservant</ent><br/
<hw>Con*serv"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conservans</ets>, p. pr.]</ety> <def>Having the power or quality of conservation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservation</ent><br/
<hw>Con`ser*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conservatio</ets>: cf. F. <ets>conservation</ets>.]</ety> <def>The act of preserving, guarding, or protecting; the keeping (of a thing) in a safe or entire state; preservation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A step necessary for the <qex>conservation</qex> of Protestantism.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A state without the means of some change is without the means of its <qex>conservation</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conservation of areas</b></col> <fld>(Astron.)</fld>, <cd>the principle that the radius vector drawn from a planet to the sun sweeps over equal areas in equal times.</cd> -- <mcol><col><b>Conservation of energy</b></col>, <it>or</it> <col><b>Conservation of force</b></col></mcol> <fld>(Mech.)</fld>, <cd>the principle that the total energy of any material system is a quantity which can neither be increased nor diminished by any action between the parts of the system, though it may be transformed into any of the forms of which energy is susceptible.</cd>  <rj><au>Clerk Maxwell.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservational</ent><br/
<hw>Con`ser*va"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to conserve; preservative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservatism</ent><br/
<hw>Con*serv"a*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>conservatism</ets>.]</ety> <def>The disposition and tendency to preserve what is established; opposition to change; the habit of mind; or conduct, of a conservative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservative</ent><br/
<hw>Con*serv"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>conservatif</ets>.]</ety> <sn>1.</sn> <def>Having power to preserve in a safe of entire state, or from loss, waste, or injury; preservative.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 307 --></p>

<p><sn>2.</sn> <def>Tending or disposed to maintain existing institutions; opposed to change or innovation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to a political party which favors the conservation of existing institutions and forms of government, as the <ex>Conservative</ex> party in England; -- contradistinguished from <contr>Liberal</contr> and <contr>Radical</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have always been conscientiously attached to what is called the Tory, and which might with more propriety be called the <qex>Conservative</qex>, party.</q> <rj><qau>Quart. Rev. (1830).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conservative system</b></col> <fld>(Mech.)</fld>, <cd>a material system of such a nature that after the system has undergone any series of changes, and been brought back in any manner to its original state, the whole work done by external agents on the system is equal to the whole work done by the system overcoming external forces.</cd></cs>
 <rj><qau>Clerk Maxwell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservative</ent><br/
<hw>Con*serv"a*tive</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, preserves from ruin, injury, innovation, or radical change; a preserver; a conserver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Holy Spirit is the great <qex>conservative</qex> of the new life.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who desires to maintain existing institutions and customs; also, one who holds moderate opinions in politics; -- opposed to <contr>revolutionary</contr> or <contr>radical</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eng. Hist.)</fld> <def>A member of the Conservative party.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservativeness</ent><br/
<hw>Con*serv"a*tive*ness</hw>, <pos>a.</pos> <def>The quality of being conservative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservatoire</ent><br/
||<hw>Con`ser"va*toire`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A public place of instruction in any special branch, esp. music and the arts. [See <er>Conservatory</er>, 3].</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservator</ent><br/
<hw>Con"ser*va`tor</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>conservateur</ets>.]</ety> <sn>1.</sn> <def>One who preserves from injury or violation; a protector; a preserver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great Creator and <qex>Conservator</qex> of the world.</q> <rj><qau>Derham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>An officer who has charge of preserving the public peace, as a justice or sheriff.</def> <sd>(b)</sd> <def>One who has an official charge of preserving the rights and privileges of a city, corporation, community, or estate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lords of the secret council were likewise made <qex>conservators</qex> of the peace of the two kingdoms.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conservator</qex> of the estate of an idiot.</q> <rj><qau>Bouvier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conservators of the River Thames</b></col>, <cd>a board of commissioners instituted by Parliament to have the conservancy of the Thames.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservatory</ent><br/
<hw>Con*serv"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>conservatoire</ets>, LL. <ets>conservatorius</ets>.]</ety> <def>Having the quality of preserving from loss, decay, or injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservatory</ent><br/
<hw>Con*serv"a*to*ry</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>conservatoire</ets>, LL. <ets>conservatorium</ets>.]</ety> <sn>1.</sn> <def>That which preserves from injury.</def> <mark>[Obs.]</mark> <ldquo/A <xex>conservatory</xex> of life.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place for preserving anything from loss, decay, waste, or injury; particulary, a greenhouse for preserving exotic or tender plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A public place of instruction, designed to preserve and perfect the knowledge of some branch of science or art, esp. music.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conservatrix</ent><br/
<hw>Con`ser*va"trix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A woman who preserves from loss, injury, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conserve</ent><br/
<hw>Con*serve"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conserved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conserving</conjf>.]</vmorph> <ety>[F. <ets>conserver</ets>, L. <ets>conservare</ets>; <ets>con-</ets> + <ets>servare</ets> to keep, guard. See <er>Serve</er>.]</ety> <sn>1.</sn> <def>To keep in a safe or sound state; to save; to preserve; to protect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The amity which . . . they meant to <qex>conserve</qex> and maintain with the emperor.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prepare with sugar, etc., for the purpose of preservation, as fruits, etc.; to make a conserve of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conserve</ent><br/
<hw>Con"serve</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>conserve</ets>, fr. <ets>conserver</ets>.]</ety> <sn>1.</sn> <def>Anything which is conserved; especially, a sweetmeat prepared with sugar; a confection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall . . . study broths, plasters, and <qex>conserves</qex>, till from a fine lady I become a notable woman.</q> <rj><qau>Tatler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A medicinal confection made of freshly gathered vegetable substances mixed with finely powdered refined sugar. See <er>Confection</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A conservatory.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conserver</ent><br/
<hw>Con*serv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conserves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conserves</ent><br/
<hw>conserves</hw> <pos>n.</pos> <def>fruit preserved by cooking with sugar.</def><br/
<syn><b>Syn. --</b> conserve, preserve, preserves.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conserving</ent><br/
<hw>conserving</hw> <pos>adj.</pos> <def>protecting or saving from harm or loss; <as>as, serves a <ex>conserving</ex> function</as>.</def><br/
<syn><b>Syn. --</b> preserving.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>conserving</ent><br/
<hw>conserving</hw> <pos>n.</pos> <def>the preservation and careful management of the environment and of natural resources.</def><br/
<syn><b>Syn. --</b> conservation, conservancy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Consider</ent><br/
<hw>Con*sid"er</hw> <pr>(k<ocr/n*s<icr/d"<etil/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Considered</conjf> <pr>(k<ocr/n*s<icr/d"<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Considering</conjf>.]</vmorph> <ety>[F. <ets>consid<eacute/rer</ets>, L. <ets>considerare</ets>, <ets>-sideratum</ets>, to consider, view attentively, prob. fr. <ets>con-</ets> + <ets>sidus</ets>, <ets>sideris</ets>, star, constellation; orig., therefore, to look at the stars. See <er>Sidereal</er>, and cf. <er>Desire</er>.]</ety> <sn>1.</sn> <def>To fix the mind on, with a view to a careful examination; to think on with care; to ponder; to study; to meditate on.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>consider</qex> thy testimonies.</q> <rj><qau>Ps. cxix. 95.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thenceforth to speculations high or deep<br/
I turned my thoughts, and with capacious mind<br/
<qex>Considered</qex> all things visible.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To look at attentively; to observe; to examine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>considereth</qex> a field, and buyeth it.</q> <rj><qau>Prov. xxxi. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have regard to; to take into view or account; to pay due attention to; to respect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consider</qex>, sir, the chance of war: the day<br/
Was yours by accident.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>England could grow into a posture of being more united at home, and more <qex>considered</qex> abroad.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To estimate; to think; to regard; to view.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Considered</qex> as plays, his works are absurd.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The proper sense of <xex>consider</xex> is often blended with an idea of the result of <xex>considering</xex>; as, <ldquo/Blessed is he that <xex>considereth</xex> the poor.<rdquo/ <au>Ps. xli. 1.</au>; <it>i.e.</it>, <xex>considers</xex> with sympathy and pity. <ldquo/Which [services] if I have not enough <xex>considered</xex>.<rdquo/ <au>Shak.</au>; <it>i.e.</it>, requited as the sufficient <xex>considering</xex> of them would suggest. <ldquo/<xex>Consider</xex> him liberally.<rdquo/  <rj><au>J. Hooker.</au></rj></note></p>

<p><syn><b>Syn.</b> -- To ponder; weigh; revolve; study; reflect or meditate on; contemplate; examine. See <er>Ponder</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consider</ent><br/
<hw>Con*sid"er</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To think seriously; to make examination; to reflect; to deliberate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We will <qex>consider</qex> of your suit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'T were to <qex>consider</qex> too curiously, to <qex>consider</qex> so.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She wished she had taken a moment to <qex>consider</qex>, before rushing down stairs.</q> <rj><qau>W. Black</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hesitate.</def> <mark>[Poetic & R.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerable</ent><br/
<hw>Con*sid"er*a*ble</hw> <pr>(k<ocr/n*s<icr/d"<etil/r*<adot/*b'l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>consid<eacute/rable</ets>.]</ety> <sn>1.</sn> <def>Worthy of consideration; requiring to be observed, borne in mind, or attended to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is <qex>considerable</qex>, that some urns have had inscriptions on them expressing that the lamps were burning.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Eternity is infinitely the most <qex>considerable</qex> duration.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of some distinction; noteworthy; influential; respectable; -- said of persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You are, indeed, a very <qex>considerable</qex> man.</q> <rj><qau>Junius.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of importance or value.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In painting, not every action, nor every person, is <qex>considerable</qex> enough to enter into the cloth.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>considerable</qex> sum of money.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerableness</ent><br/
<hw>Con*sid"er*a*ble*ness</hw>, <pos>n.</pos> <def>Worthiness of consideration; dignity; value; size; amount.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerably</ent><br/
<hw>Con*sid"er*a*bly</hw>, <pos>adv.</pos> <def>In a manner or to a degree not trifling or unimportant; greatly; much.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The breeds . . . differ <qex>considerably</qex> from each other.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerance</ent><br/
<hw>Con*sid"er*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>considerantia</ets>.]</ety> <def>Act of considering; consideration.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj></p>

<p><ent>Considerate</ent><br/
<hw>Con*sid"er*ate</hw> <pr>(k<ocr/n*s<icr/d"<etil/r*<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>consideratus</ets>, p. p.]</ety> <sn>1.</sn> <def>Given to consideration or to sober reflection; regardful of consequences or circumstances; circumspect; careful; esp. careful of the rights, claims, and feelings of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of dauntless courage and <qex>considerate</qex> pride.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><AE/neas is patient, <qex>considerate</qex>, and careful of his people.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The wisest and most <qex>considerate</qex> men in the world.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having respect to; regardful.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They may be . . . more <qex>considerate</qex> of praise.</q> <rj><qau>Dr. H. More.</qau></rj></p>

<p><syn><b>Syn.</b> -- Thoughtful; reflective; careful; discreet; prudent; deliberate; serious. See <er>Thoughtful</er>.</syn></p>

<p>-- <wordforms><wf>Con*sid"er*ate*ly</wf>, <pos>adv.</pos> -- <wf>Con*sid"er*ate*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>considerateness</ent><br/
<hw>considerateness</hw> <pos>n.</pos> <def>kind and considerate regard for others; consideration.</def><br/
<syn><b>Syn. --</b> consideration, thoughtfulness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Consideration</ent><br/
<hw>Con*sid`er*a"tion</hw> <pr>(k<ocr/n*s<icr/d`<etil/r*<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>consideratio</ets>: cf. F. <ets>consid<eacute/ration</ets>.]</ety> <sn>1.</sn> <def>The act or process of considering; continuous careful thought; examination; contemplation; deliberation; attention.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us think with <qex>consideration</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consideration</qex>, like an angel, came.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Attentive respect; appreciative regard; -- used especially in diplomatic or stately correspondence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The undersigned has the honor to repeat to Mr. Hulseman the assurance of his high <qex>consideration</qex>.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consideration</qex> with which he was treated.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Thoughtful or sympathetic regard or notice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consideration</qex> for the poor is a doctrine of the church.</q> <rj><qau>Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Claim to notice or regard; some degree of importance or consequence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lucan is the only author of <qex>consideration</qex> among the Latin poets who was not explained for . . . the Dauphin.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The result of delibration, or of attention and examonation; matured opinion; a reflection; <as>as, <ex>considerations</ex> on the choice of a profession</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>That which is, or should be, taken into account as a ground of opinion or action; motive; reason.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was obliged, antecedent to all other <qex>considerations</qex>, to search an asylum.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>considerations</qex> which are necessary to the forming of a correct judgment.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>The cause which moves a contracting party to enter into an agreement; the material cause of a contract; the price of a stripulation; compensation; equivalent.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Consideration</xex> is what is done, or promised to be done, in exchange for a promise, and <ldquo/as a mere advantage to the promisor without detriment to the promisee would not avail, the proper test is detriment to the promisee.<rdquo/</note>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerative</ent><br/
<hw>Con*sid"er*a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Considerate; careful; thoughtful.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I love to be <qex>considerative</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerator</ent><br/
<hw>Con*sid"er*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who considers.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Considerer</ent><br/
<hw>Con*sid"er*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who considers; a man of reflection; a thinker.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consideringly</ent><br/
<hw>Con*sid"er*ing*ly</hw>, <pos>adv.</pos> <def>With consideration or deliberation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consign</ent><br/
<hw>Con*sign"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consigned</conjf> 3; <pos>p. pr. & vb. n.</pos> <conjf>Consigning</conjf>.]</vmorph> <ety>[F. <ets>consigner</ets>, L. <ets>consignare</ets>, <ets>-signatu</ets>,, to seal or sign; <ets>con-</ets> + <ets>signare</ets>, fr. <ets>signum</ets> mark. See <er>Sign</er>.]</ety> <sn>1.</sn> <def>To give, transfer, or deliver, in a formal manner, as if by signing over into the possession of another, or into a different state, with the sense of fixedness in that state, or permanence of possession; <as>as, to <ex>consign</ex> the body to the grave</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At the day of general account, good men are to be <qex>consigned</qex> over to another state.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give in charge; to commit; to intrust.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Atrides, parting for the Trojan war,<br/
<qex>Consigned</qex> the youthful consort to his care.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The four evangelists <qex>consigned</qex> to writing that history.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>To send or address (by bill of lading or otherwise) to an agent or correspondent in another place, to be cared for or sold, or for the use of such correspondent; <as>as, to <ex>consign</ex> a cargo or a ship; to <ex>consign</ex> goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To assign; to devote; to set apart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The French commander <qex>consigned</qex> it to the use for which it was intended by the donor.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To stamp or impress; to affect.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consign</qex> my spirit with great fear.</q> <rj><qau>Jer. Taylor.</qau></rj></p>

<p><syn><b>Syn.</b> -- To commit; deliver; intrust; resign. See <er>Commit</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consign</ent><br/
<hw>Con*sign"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To submit; to surrender or yield one's self.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All lovers young, all lovers must<br/
<qex>Consign</qex> to thee, and come to dust.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To yield consent; to agree; to acquiesce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Augment or alter . . . <br/
And we'll <qex>consign</qex> thereto.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignatary</ent><br/
<hw>Con*sig"na*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Consignitary</er>.]</ety> <def>A consignee.</def> <mark>[Obs.]</mark>  <rj><au>Jenkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignation</ent><br/
<hw>Con`sig*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consignatio</ets> written proof, document: cf. F. <ets>consignation</ets> comsignation.]</ety> <sn>1.</sn> <def>The act of consigning; the act of delivering or committing to another person, place, or state.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So is despair a certain <qex>consignation</qex> to eternal ruin.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of ratifying or establishing, as if by signing; confirmation; ratification.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A direct <qex>consignation</qex> of pardon.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stamp; an indication; a sign.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The most certain <qex>consignations</qex> of an excellent virtue.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignatory</ent><br/
<hw>Con*sig"na*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Consignitary</er>.]</ety> <def>One of several that jointly sign a written instrument, as a treaty.</def>  <rj><au>Fallows.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignature</ent><br/
<hw>Con*sig"na*ture</hw> <pr>(?)</pr>; 135), <pos>n.</pos> <def>Joint signature.</def> <mark>[R.]</mark>  <rj><au>Colgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consigne</ent><br/
||<hw>Con"signe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A countersign; a watchword.</def> <sd>(b)</sd> <def>One who is orders to keep within certain limits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignee</ent><br/
<hw>Con`sign*ee"</hw> <pr>(?;  277)</pr>, <pos>n.</pos> <ety>[F. <ets>consign<?/</ets>, p. p. of <ets>consigner</ets>.]</ety> <def>The person to whom goods or other things are consigned; a factor; -- correlative to <xex>consignor</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Consigner and <qex>consignee</qex> are used by merchants to express generally the shipper of merchandise, and the person to whom it is addressed, by bill of lading or otherwise.</q> <rj><qau>De Colange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consigner</ent><br/
<hw>Con*sign"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who consigns. See <er>Consignor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignificant</ent><br/
<hw>Con`sig*nif"i*cant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having joint or equal signification; synonymous.</def> <mark>[R.]</mark>  <rj><au>Spelman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignification</ent><br/
<hw>Con*sig`ni*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint signification.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignificative</ent><br/
<hw>Con`sig*nif"i*ca*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consignificant; jointly significate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignify</ent><br/
<hw>Con*sig"ni*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>con-</ets> + <ets>sognify</ets>.]</ety> <def>To signify or denote in combination with something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cipher . . . only serves to connote and <qex>consignify</qex>, and to change the value or the figures.</q> <rj><qau>Horne Tooke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignment</ent><br/
<hw>Con*sign"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of consigning; consignation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Com.)</fld> <def>The act of consigning or sending property to an agent or correspondent in another place, as for care, sale, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>That which is consigned; the goods or commodities sent or addressed to a consignee at one time or by one conveyance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To increase your <qex>consignments</qex> of this valuable branch of national commerce.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The writing by which anything is consigned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consignor</ent><br/
<hw>Con*sign"or</hw> <pr>(? <or/ <?/; 277)</pr>, <pos>n.</pos> <def>One who consigns something to another; -- opposed to <inv>consignee</inv>.</def> <altsp>[Written also <asp>consigner</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consilience</ent><br/
<hw>Con*sil"i*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>con-</ets> + <ets>salire</ets> to leap.]</ety> <def>Act of concurring; coincidence; concurrence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consilience</qex> of inductions takes place when one class of facts coincides with an induction obtained from another different class.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consimility</ent><br/
<ent>Consimilitude</ent><br/
<mhw>{ <hw>Con`si*mil"i*tude</hw> <pr>(?)</pr>, <hw>Con`si*mil"i*ty</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. F. <ets>consimilitude</ets>. See <er>Similitude</er>.]</ety> <def>Common resemblance.</def> <mark>[Obs.]</mark>  <rj><au>Aubrey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consist</ent><br/
<hw>Con*sist"</hw> <pr>(k<ocr/n*s<icr/st")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consisted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Consisting</conjf>.]</vmorph> <ety>[L. <ets>consistere</ets> to stand still or firm; <ets>con-</ets> + <ets>sistere</ets> to stand, cause to stand, <ets>stare</ets> to stand: cf. F. <ets>consister</ets>. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>To stand firm; to be in a fixed or permanent state, as a body composed of parts in union or connection; to hold together; to be; to exist; to subsist; to be supported and maintained.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is before all things, and by him all things <qex>consist</qex>.</q> <rj><qau>Col. i. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be composed or made up; -- followed by <ptcl>of</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The land would <qex>consist</qex> of plains and valleys.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have as its substance or character, or as its foundation; to be; -- followed by <xex>in</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If their purgation did <qex>consist</qex> in words.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man's life <qex>consisteth not in the abundance of the things which he possesseth</qex>.</q> <rj><qau>Luke xii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be consistent or harmonious; to be in accordance; -- formerly used absolutely, now followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This was a <qex>consisting</qex> story.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Health <qex>consists</qex> with temperance alone.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For orders and degrees<br/
Jar not with liberty, but well <qex>consist</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To insist; -- followed by <xex>on</xex>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- <er>To Consist</er>, <er>Consist of</er>, <er>Consist in</er>.</syn> <usage> The verb <xex>consist</xex> is employed chiefly for two purposes, which are marked and distinguished by the prepositions used. When we wish to indicate the parts which unite to compose a thing, we use <xex>of</xex>; as when we say, <ldquo/Macaulay's Miscellanies consist chiefly <xex>of</xex> articles which were first published in the Edinburgh Review.<rdquo/ When we wish to indicate the true nature of a thing, or that on which it depends, we use <xex>in</xex>; as, <ldquo/There are some artists whose skill consists <xex>in</xex> a certain manner which they have affected.<rdquo/ <ldquo/Our safety consists <xex>in</xex> a strict adherence to duty.<rdquo/</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistency</ent><br/
<ent>Consistence</ent><br/
<mhw>{ <hw>Con*sist"ence</hw> <pr>(?)</pr>, <hw>Con*sist"en*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. F. <ets>consistance</ets>.]</ety> <sn>1.</sn> <def>The condition of standing or adhering together, or being fixed in union, as the parts of a body; existence; firmness; coherence; solidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Water, being divided, maketh many circles, till it restore itself to the natural <qex>consistence</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We are as water, weak, and of no <qex>consistence</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The same form, substance, and <qex>consistency</qex>.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A degree of firmness, density, viscosity, or spissitude; a measure of the ability to hold together when manipulated.</def><br/
<syn><b>Syn. --</b> body.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the expressed juices be boiled into the <qex>consistence</qex> of a sirup.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 308 --></p>

<p><sn>3.</sn> <def>That which stands together as a united whole; a combination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The church of God, as meaning the whole <qex>consistence</qex> of orders and members.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Firmness of constitution or character; substantiality; durability; persistency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His friendship is of a noble make and a lasting <qex>consistency</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Agreement or harmony of all parts of a complex thing among themselves, or of the same thing with itself at different times; the harmony of conduct with profession; congruity; correspondence; <as>as, the <ex>consistency</ex> of laws, regulations, or judicial decisions; <ex>consistency</ex> of opinions; <ex>consistency</ex> of conduct or of character</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>consistency</qex> of behavior whereby he inflexibly pursues those measures which appear the most just.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consistency</qex>, thou art a jewel.</q> <rj><qau>Popular Saying.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistent</ent><br/
<hw>Con*sist"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consistens</ets>, p. pr.: cf. F. <ets>consistant</ets>.]</ety> <sn>1.</sn> <def>Possessing firmness or fixedness; firm; hard; solid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The humoral and <qex>consistent</qex> parts of the body.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having agreement with itself or with something else; having harmony among its parts; possesing unity; accordant; harmonious; congruous; compatible; uniform; not contradictory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Show me one that has it in his power<br/
To act <qex>consistent</qex> with himself an hour.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With reference to such a lord, to serve and to be free are terms not <qex>consistent</qex> only, but equivalent.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Living or acting in conformity with one's belief or professions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was utterly to be at once a <qex>consistent</qex> Quaker and a conspirator.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistently</ent><br/
<hw>Con*sist"ent*ly</hw>, <pos>adv.</pos> <def>In a consistent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistorial</ent><br/
<hw>Con`sis*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>consistorial</ets>.]</ety> <def>Of or pertaining to a consistory.</def> <ldquo/<xex>Consistorial</xex> laws.<rdquo/ <au>Hooker.</au> <ldquo/<xex>Consistorial</xex> courts.<rdquo/ <au>Bp. Hoadley.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistorian</ent><br/
<hw>Con`sis*to"rian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a Presbyterian consistory; -- a contemptuous term of 17th century controversy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You fall next on the <qex>consistorian</qex> schismatics; for so you call Presbyterians.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistory</ent><br/
<hw>Con*sis"to*ry</hw> <pr>(? or ?; 277)</pr> <pos>n.</pos>; <plu><it>pl.</it> <plw>Consistories</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>consistorium</ets> a place of assembly, the place where the emperor's council met, fr. <ets>consistere</ets>: cf. F. <ets>consistoire</ets>, It. <ets>consistorio</ets>. See <er>Consist</er>.]</ety> <sn>1.</sn> <def>Primarily, a place of standing or staying together; hence, any solemn assembly or council.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To council summons all his mighty peers,<br/
Within thick clouds and dark tenfold involved,<br/
A gloomy <qex>consistory</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eng. Ch.)</fld> <def>The spiritual court of a diocesan bishop held before his chancellor or commissioner in his cathedral church or elsewhere.</def>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(R. C. Ch.)</fld> <def>An assembly of prelates; a session of the college of cardinals at Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pius was then hearing of causes in <qex>consistory</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A church tribunal or governing body.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some churches, as the Dutch Reformed in America, a <xex>consistory</xex> is composed of the minister and elders of an individual church, corresponding to a Presbyterian church session, and in others, as the Reformed church in France, it is composed of ministers and elders, corresponding to a presbytery. In some Lutheran countries it is a body of clerical and lay officers appointed by the sovereign to superintend ecclesiastical affairs.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A civil court of justice.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consistory</ent><br/
<hw>Con*sis"to*ry</hw>, <pos>a.</pos> <def>Of the nature of, or pertaining to, a consistory.</def> <ldquo/To hold <xex>consistory</xex> session.<rdquo/  <rj><au>Strype.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consociate</ent><br/
<hw>Con*so"ci*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consociatus</ets>, p. p. of <ets>consociare</ets> to associate, unite; <ets>con-</ets> + <ets>sociare</ets> to join, unite. See <er>Social</er>.]</ety> <def>An associate; an accomplice.</def> <mark>[Archaic]</mark> <ldquo/Wicked <xex>consociates</xex>.<rdquo/  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consociate</ent><br/
<hw>Con*so"ci*ate</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consociated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Consociating</conjf>.]</vmorph> <sn>1.</sn> <def>To bring into alliance, confederacy, or relationship; to bring together; to join; to unite.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Join pole to pole, <qex>consociate</qex> severed worlds.</q> <rj><qau>Mallet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite in an ecclesiastical consociation.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consociate</ent><br/
<hw>Con*so"ci*ate</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be allied, confederated, or associated; to coalescence.</def> <mark>[R.]</mark>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form an ecclesiastical consociation.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consociation</ent><br/
<hw>Con*so`ci*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consociatio</ets>.]</ety> <sn>1.</sn> <def>Intimate union; fellowship; alliance; companionship; confederation; association; intimacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A friendly <qex>consociation</qex> with your kindred elements.</q> <rj><qau>Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A voluntary and permanent council or union of neighboring Congregational churches, for mutual advice and cooperation in ecclesiastical matters; a meeting of pastors and delegates from churches thus united.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In Connecticut some of the Congregational churhes are associated in <xex>consociations</xex> and the others in conferences.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consociational</ent><br/
<hw>Con*so`ci*a"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a consociation.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consol</ent><br/
<hw>Con*sol"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A consolidated annuity (see <er>Consols</er>); -- chiefly in combination or attributively.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Consolable</ent><br/
<hw>Con*sol"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consolabilis</ets>: cf. F. <ets>consolable</ets>.]</ety> <def>Capable of receiving consolation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolate</ent><br/
<hw>Con"so*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>consolatus</ets>, p. p. See <er>Console</er>, <pos>v. t.</pos>]</ety> <def>To console; to comfort.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolation</ent><br/
<hw>Con`so*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consolatio</ets>: cf. F. <ets>consolation</ets>.]</ety> <def>The act of consoling; the state of being consoled; allevation of misery or distress of mind; refreshment of spirit; comfort; that which consoles or comforts the spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Against such cruelties<br/
With inward <qex>consolations</qex> recompensed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Are the <qex>consolations</qex> of God small with thee?</q> <rj><qau>Job xv. 11.</qau></rj></p>

<p><syn><b>Syn.</b> -- Comfort; solace; allevation. See <er>Comfort</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>race</ent><br/
<ent>pot</ent><br/
<ent>match</ent><br/
<ent>Consolation game</ent><br/
<mhw>{ <hw>Con`so*la"tion game</hw>, <hw>match</hw>, <hw>pot</hw>, <hw>race</hw>, etc. }</mhw> <def>A game, match, etc., open only to losers in early stages of contests.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Consolato del mare</ent><br/
||<hw>Con`so*la"to del ma"re</hw> <pr>(?)</pr>. <ety>[It., the consulate of the sea.]</ety> <def>A collection of maritime laws of disputed origin, supposed to have been first published at Barcelona early in the 14th century. It has formed the basis of most of the subsequent collections of maritime laws.</def>  <rj><au>Kent.</au>  <au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolator</ent><br/
<hw>Con"so*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who consoles or comforts.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolatory</ent><br/
<hw>Con*sol"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consolatorius</ets>.]</ety> <def>Of a consoling or comforting nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The punishment of tyrants is a noble and awful act of justice; and it has with truth been said to be <qex>consolatory</qex> to the human mind.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolatory</ent><br/
<hw>Con*sol"a*to*ry</hw>, <pos>n.</pos> <def>That which consoles; a speech or writing intended for consolation.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Console</ent><br/
<hw>Con*sole"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consoled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Consoling</conjf>.]</vmorph> <ety>[L. <ets>consolari</ets>,. p. p. <ets>consolatus</ets>; <ets>con-</ets> + <ets>solari</ets> to console, comfort: cf. F. <ets>consoler</ets>. See <er>Solace</er>.]</ety> <def>To cheer in distress or depression; to alleviate the grief and raise the spirits of; to relieve; to comfort; to soothe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And empty heads <qex>console</qex> with empty sound.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am much <qex>consoled</qex> by the reflection that the religion of Christ has been attacked in vain by all the wits and philosophers, and its triumph has been complete.</q> <rj><qau>P. Henry.</qau></rj></p>

<p><syn><b>Syn.</b> -- To comfort; solace; soothe; cheer; sustain; encourage; support. See <er>Comfort</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Console</ent><br/
<hw>Con"sole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A bracket whose projection is not more than half its height.</def> <sd>(b)</sd> <def>Any small bracket; also, a console table.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Computers)</fld>  <def>The keyboard and monitor of a computer considered together.</def><br/
[<source>GG</source>]</p>

<p><sn>3.</sn> <fld>(Engineering)</fld> <def>The controlling portion of an electrical, electronic, or mechanical device or system, from which the operator may observe the state of the system as indicated by gauges or on some form of <er>display{n3}</er>, and may direct or control the action of the system.</def><br/
[<source>GG</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>the desklike controlling unit of an organ containing the keyboard, pedals, stops, etc. by means of which the organ is played.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a home entertainment device such as a television, radio, phonograph, CD player, or combination of these, designed as a piece of furniture, to stand on the floor rather than on a table or in a separate cabinet; -- also used attributively in the phrase <altname>console model</altname>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Console table</b></col>, <cd>a table whose top is supported by two or more consoles instead of legs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consoler</ent><br/
<hw>Con*sol"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who gives consolation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidant</ent><br/
<hw>Con*sol"i*dant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consolidans</ets>, p. pr. of <ets>consolidare</ets>: cf. F. <ets>consolidant</ets>.]</ety> <def>Serving to unite or consolidate; having the quality of consolidating or making firm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidate</ent><br/
<hw>Con*sol"i*date</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consolidatus</ets>, p. pr. of <ets>consolidare</ets> to make firm; <ets>con-</ets> + <ets>solidare</ets> to make firm; <ets>solidus</ets> solid. See <er>Solid</er>, and cf. <er>Consound</er>.]</ety> <def>Formed into a solid mass; made firm; consolidated.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A gentleman [should learn to ride] while he is tender and the brawns and sinews of his thighs not fully <qex>consolidate</qex>.</q> <rj><qau>Elyot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidate</ent><br/
<hw>Con*sol"i*date</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consolidated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Consolidating</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To make solid; to unite or press together into a compact mass; to harden or make dense and firm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He fixed and <qex>consolidated</qex> the earth.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite, as various particulars, into one mass or body; to bring together in close union; to combine; <as>as, to <ex>consolidate</ex> the armies of the republic</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consolidating</qex> numbers into unity.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Surg.)</fld> <def>To unite by means of applications, as the parts of a broken bone, or the lips of a wound.</def> <mark>[R.]</mark></p>

<p><syn><b>Syn.</b> -- To unite; combine; harden; compact; condense; compress.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidate</ent><br/
<hw>Con*sol"i*date</hw>, <pos>v. i.</pos> <def>To grow firm and hard; to unite and become solid; <as>as, moist clay <ex>consolidates</ex> by drying</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In hurts and ulcers of the head, dryness maketh them more apt to <qex>consolidate</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidated</ent><br/
<hw>Con*sol"i*da`ted</hw> <pr>(?)</pr>, <pos>p. p. & a.</pos> <sn>1.</sn> <def>Made solid, hard, or compact; united; joined; solidified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Aggregate Fund . . . consisted of a great variety of taxes and surpluses of taxes and duties which were [in 1715] <qex>consolidated</qex>.</q> <rj><qau>Rees.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A mass of partially <qex>consolidated</qex> mud.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having a small surface in proportion to bulk, as in the cactus.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consolidated</qex> plants are evidently adapted and designed for very dry regions; in such only they are found.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Consolidated Fund</b></col>, <cd>a British fund formed by consolidating (in 1787) three public funds (the Aggregate Fund, the General Fund, and the South Sea Fund). In 1816, the larger part of the revenues of Great Britian and Ireland was assigned to what has been known as <xex>the Consolidated Fund of the United Kingdom</xex>, out of which are paid the interest of the national debt, the salaries of the civil list, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidation</ent><br/
<hw>Con*sol`i*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consolidatio</ets> a confirming: cf. F. <ets>consolidation</ets>.]</ety> <sn>1.</sn> <def>The act or process of consolidating, making firm, or uniting; the state of being consolidated; solidification; combination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consolidation</qex> of the marble and of the stone did not fall out at random.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consolidation</qex> of the great European monarchies.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>To organic cohesion of different circled in a flower; adnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>The combination of several actions into one.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consolidative</ent><br/
<hw>Con*sol"i*da*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>consolidatif</ets>.]</ety> <def>Tending or having power to consolidate; healing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consoling</ent><br/
<hw>Con*sol"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adapted to console or comfort; cheering; <as>as, this is <ex>consoling</ex> news</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consols</ent><br/
<hw>Con"sols</hw> <pr>(? <or/ <?/; 277)</pr>, <pos>n. pl.</pos> <ety>[A contraction of <ets>consol</ets>idated (annuities).]</ety> <def>The leading British funded government security.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A considerable part of the public debt of Great Britian, which had been contracted in the form of annuities yielding various rates of interest, was, in 1757, consolidated into one fund at 3 per cent interest, the account of which is kept at the Bank of England. This debt has been diminished and increased at different times, and now constitutes somewhat more than half of the entire national debt. The stocks are transferable, and their value in the market constantly fluctuates; the price at any time being regarded as a gauge of the national prosperity and public confidence.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consomme</ent><br/
||<hw>Con`som`m"<eacute/</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., lit. p. p. of <ets>consommer</ets> to finish.]</ety> <fld>(Cookery)</fld> <def>A clear soup or bouillion boiled down so as to be very rich.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonancy</ent><br/
<ent>Consonance</ent><br/
<mhw>{ <hw>Con"so*nance</hw> <pr>(?)</pr>, <hw>Con"so*nan*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>consonantia</ets>: cf. F. <ets>consonnance</ets>.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>Accord or agreement of sounds produced simultaneously, as a note with its third, fifth, and eighth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Agreement or congruity; harmony; accord; consistency; suitableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The perfect <qex>consonancy</qex> of our persecuted church to the doctrines of Scripture and antiquity.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The optic nerve responds to the waves with which it is in <qex>consonance</qex>.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Friendship; concord.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By the <qex>consonancy</qex> of our youth.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Agreement; accord; consistency; unison; harmony; congruity; suitableness; agreeableness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonant</ent><br/
<hw>Con"so*nant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consonans</ets>, <ets>-antis</ets>; p. pr. of <ets>consonare</ets> to sound at the same time, agree; <ets>con-</ets> + <ets>sonare</ets> to sound: cf. F. <ets>consonnant</ets>. See <er>Sound</er> to make a noise.]</ety> <sn>1.</sn> <def>Having agreement; congruous; consistent; according; -- usually followed by <xex>with</xex> or <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each one pretends that his opinion . . . is <qex>consonant</qex> to the words there used.</q> <rj><qau>Bp. Beveridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That where much is given there shall be much required is a thing <qex>consonant</qex> with natural equity.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having like sounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Consonant</qex> words and syllables.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>harmonizing together; accordant; <as>as, <ex>consonant</ex> tones, <ex>consonant</ex> chords</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to consonants; made up of, or containing many, consonants.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No Russian whose dissonant <qex>consonant</qex> name<br/
Almost shatters to fragments the trumpet of fame.</q> <rj><qau>T. Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonant</ent><br/
<hw>Con"so*nant</hw>, <pos>n.</pos> <ety>[L. <ets>consonans</ets>, <ets>-antis</ets>.]</ety> <def>An articulate sound which in utterance is usually combined and sounded with an open sound called a vowel; a member of the spoken alphabet other than a vowel; also, a letter or character representing such a sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>Consonants are divided into various classes, as mutes, spirants, sibilants, nasals, semivowels, etc. All of them are sounds uttered through a closer position of the organs than that of a vowel proper, although the most open of them, as the semivowels and nasals, are capable of being used as if vowels, and forming syllables with other closer consonants, as in the English <xex>feeble</xex> <pr>(-b'l)</pr>, <xex>taken</xex> <pr>(-k'n)</pr>. All the consonants excepting the mutes may be indefinitely, prolonged in utterance without the help of a vowel, and even the mutes may be produced with an aspirate instead of a vocal explosion. Vowels and consonants may be regarded as the two poles in the scale of sounds produced by gradual approximation of the organ, of speech from the most open to the closest positions, the vowel being more open, the consonant closer; but there is a territory between them where the sounds produced partake of the qualities of both.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/A <xex>consonant</xex> is the result of audible friction, squeezing, or stopping of the breath in some part of the mouth (or occasionally of the throath.) The main distinction between vowels and consonants is, that while in the former the mouth configuration merely modifies the vocalized breath, which is therefore an essential element of the vowels, in consonants the narrowing or stopping of the oral passage is the foundation of the sound, and the state of the glottis is something secondary.<rdquo/</note>  <rj><au>H. Sweet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonantal</ent><br/
<hw>Con`so*nan"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of a consonant; pertaining to consonants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonantize</ent><br/
<hw>Con"so*nant*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To change into, or use as, a consonant.</def> <ldquo/The vowel is <xex>consonantized</xex>, that is, made closer in position.<rdquo/  <rj><au>Peile.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonantly</ent><br/
<hw>Con"so*nant*ly</hw>, <pos>adv.</pos> <def>In a consonant, consistent, or congruous manner; agreeably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonantness</ent><br/
<hw>Con"so*nant*ness</hw>, <pos>n.</pos> <def>The quality or condition of being consonant, agreeable, or consistent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consonous</ent><br/
<hw>Con"so*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consonus</ets>. See <er>Consonant</er>.]</ety> <def>Agreeing in sound; symphonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consopiation</ent><br/
<hw>Con*so`pi*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of sleeping, or of lulling, to sleep.</def> <mark>[Obs.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consopite</ent><br/
<hw>Con"so*pite</hw> <pr>(k<ocr/n"s<osl/*p<imac/t)</pr>, <pos>a.</pos> <ety>[L. <ets>consopitus</ets>, p. p. of <ets>consopire</ets>.]</ety> <def>Lulled to sleep.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consopite</ent><br/
<hw>Con"so*pite</hw>, <pos>v. t.</pos> <def>To lull to sleep; to quiet; to compose.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The operation of the masculine faculties of the soul were, for a while, well slacked and <qex>consopited</qex>.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consort</ent><br/
<hw>Con"sort</hw> <pr>(k<ocr/n"s<ocir/rt)</pr>, <pos>n.</pos> <ety>[L. <ets>consore</ets>, <ets>-sortis</ets>; <ets>con-</ets> + <ets>sors</ets> lot, fate, share. See <er>Sort</er>.]</ety> <sn>1.</sn> <def>One who shares the lot of another; a companion; a partner; especially, a wife or husband.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He single chose to live, and shunned to wed,<br/
Well pleased to want a <qex>consort</qex> of his bed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consort</qex> of the queen has passed from this troubled sphere.</q> <rj><qau>Thakeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The snow-white gander, invariably accompanied by his darker <qex>consort</qex>.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A ship keeping company with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Concurrence; conjunction; combination; association; union.</def> <ldquo/By Heaven's <xex>consort</xex>.<rdquo/ <au>Fuller.</au> <ldquo/Working in <xex>consort</xex>.<rdquo/ <au>Hare.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Take it singly, and it carries an air of levity; but, in <qex>consort</qex> with the rest, has a meaning quite different.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[LL. <ets>consortium</ets>.]</ety> <def>An assembly or association of persons; a company; a group; a combination.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In one <qex>consort</qex>' there sat<br/
Cruel revenge and rancorous despite,<br/
Disloyal treason, and heart-burning hate.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lord, place me in thy <qex>consort</qex>.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Perh. confused with <ets>concert</ets>.]</ety> <def>Harmony of sounds; concert, as of musical instruments.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To make a sad <qex>consort</qex>';<br/
Come, let us join our mournful song with theirs.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 309 --></p>

<p><cs><col><b>Prince consort</b></col>, <cd>the husband of a queen regnant.</cd> -- <col><b>Queen consort</b></col>, <cd>the wife of a king, as distinguished from a <contr>queen regnant</contr>, who rules alone, and a <contr>queen dowager</contr>, the window of a king.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consort</ent><br/
<hw>Con*sort"</hw> <pr>(k<ocr/n*s<ocir/rt")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consorted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Consorting</conjf>.]</vmorph> <def>To unite or to keep company; to associate; -- used with <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which of the Grecian chiefs <qex>consorts</qex> with thee?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consort</ent><br/
<hw>Con*sort"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To unite or join, as in affection, harmony, company, marriage, etc.; to associate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He with his <qex>consorted</qex> Eve.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For all that pleasing is to living ears<br/
Was there <qex>consorted</qex> in one harmony.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He begins to <qex>consort</qex> himself with men.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To attend; to accompany.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou, wretched boy, that didst <qex>consort</qex> him here,<br/
Shalt with him hence.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consortable</ent><br/
<hw>Con*sort"a*ble</hw> <pr>(k<ocr/n*s<ocir/rt"<adot/*b'l)</pr>, <pos>a.</pos> <def>Suitable for association or companionship.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consortion</ent><br/
<hw>Con*sor"tion</hw> <pr>(k<ocr/n*s<ocir/r"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>consortio</ets>.]</ety> <def>Fellowship; association; companionship.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consortship</ent><br/
<hw>Con"sort*ship</hw> <pr>(k<ocr/n"s<ocir/rt*sh<icr/p)</pr>, <pos>n.</pos> <def>The condition of a consort; fellowship; partnership.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consound</ent><br/
<hw>Con"sound</hw> <pr>(-sound)</pr>, <pos>n.</pos> <ety>[Corrupted fr. F. <ets>consoude</ets>, fr L. <ets>consolida</ets> comfrey (so called because supposed to have healing power); <ets>con-</ets> + <ets>solidus</ets> solid, <ets>consolidare</ets> to make solid. Cf. <er>Comfrey</er>, <er>Consolidate</er>.]</ety> <fld>(Bot.)</fld> <def>A name applied loosely to several plants of different genera, esp. the comfrey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspecific</ent><br/
<hw>Con`spe*cif"ic</hw> <pr>(k<ocr/n`sp<esl/*s<icr/f"<icr/k)</pr>, <pos>a.</pos> <def>Of the same species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspectuity</ent><br/
<hw>Con`spec*tu"i*ty</hw> <pr>(-sp<ecr/k*t<umac/"<icr/*t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conspectuities</plw> <pr>(-t<icr/z)</pr>.</plu> <def>The faculty of seeing; sight; eye.</def> <note>[A word of Menenius's making. <au>Coriolanus ii. 1</au>.]</note>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspectus</ent><br/
<hw>Con*spec"tus</hw> <pr>(k<ocr/n*sp<ecr/k"t<ucr/s)</pr>, <pos>n.</pos> <def>A general sketch or outline of a subject; a synopsis; an epitome.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspersion</ent><br/
<hw>Con*sper"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conspersio</ets>, fr. <ets>conspergere</ets> to sprinkle.]</ety> <def>The act of sprinkling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conspersion</qex> washing the doorposts.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspicuity</ent><br/
<hw>Con`spi*cu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being clear or bright; brightness; conspicuousness.</def> <mark>[R.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspicuous</ent><br/
<hw>Con*spic"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conspicuus</ets>, fr. <ets>conspicere</ets> to get sight of, to perceive; <ets>con-</ets> + <ets>spicere</ets>, <ets>specere</ets>, to look. See <er>Spy</er>]</ety> <sn>1.</sn> <def>Open to the view; obvious to the eye; easy to be seen; plainly visible; manifest; attracting the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was a rock<br/
Of alabaster, piled up to the clouds,<br/
<qex>Conspicious</qex> far.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conspicious</qex> by her veil and hood,<br/
Signing the cross, the abbess stood.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Obvious to the mental eye; easily recognized; clearly defined; notable; prominent; eminent; distinguished; <as>as, a <ex>conspicuous</ex> excellence, or fault</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man who holds a <qex>conspicuous</qex> place in the political, ecclesiastical, and literary history of England.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- Distinguished; eminent; famous; illustrious; prominent; celebrated. See <er>Distinguished</er>.</syn></p>

<p>-- <wordforms><wf>Con*spic"u*ous*ly</wf>, <pos>adv.</pos> -- <wf>Con*spic"u*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspiracy</ent><br/
<hw>Con*spir"a*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conspiracies</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Conspiration</er>.]</ety> <sn>1.</sn> <def>A combination of people for an evil purpose; an agreement, between two or more persons, to commit a crime in concert, as treason; a plot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When shapen was all his <qex>conspiracy</qex><br/
From point to point.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They made a <qex>conspiracy</qex> against [Amaziah].</q> <rj><qau>2 Kings xiv. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I had forgot that foul <qex>conspiracy</qex></q>
<q>Of the beast Caliban and his confederates.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A concurence or general tendency, as of circumstances, to one event, as if by agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>conspiracy</qex> in all heavenly and earthly things.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>An agreement, manifesting itself in words or deeds, by which two or more persons confederate to do an unlawful act, or to use unlawful to do an act which is lawful; confederacy.</def></p>

<p><syn><b>Syn.</b> -- Combination; plot; cabal.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspirant</ent><br/
<hw>Con*spir"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conspirans</ets>, p. pr. of <ets>conspirare</ets>: cf. F. <ets>conspirant</ets>.]</ety> <def>Engaging in a plot to commit a crime; conspiring.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspiration</ent><br/
<hw>Con`spi*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>conspiration</ets>, L. <ets>conspiratio</ets>.]</ety> <def>Agreement or concurrence for some end or purpose; conspiracy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As soon as it was day, certain Jews made a <qex>conspiration</qex>.</q> <rj><qau>Udall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In our natural body every part has a nacassary sympathy with every other, and all together form, by their harmonious <qex>onspiration</qex>, a healthy whole.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspirator</ent><br/
<hw>Con*spir"a*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who engages in a conspiracy; a plotter.</def>  <rj><au>2 Sam. xv. 31.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conspiratorial</ent><br/
<hw>conspiratorial</hw> <pos>adj.</pos> <def>relating to or characteristic of conspiracy or conspirators; <as>as, a <ex>conspiratorial</ex> whisper</as>.</def><br/
<syn><b>Syn. --</b> conspirative.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conspire</ent><br/
<hw>Con*spire"</hw> <pr>(k<ocr/n*sp<imac/r")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conspired</conjf> <pr>(k<ocr/n*sp<imac/rd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conspiring</conjf>.]</vmorph> <ety>[F. <ets>conspirer</ets>, L. <ets>conspirare</ets> to blow together, harmonize, agree, plot; <ets>con-</ets> + <ets>spirare</ets> to breathe, blow. See <er>Spirit</er>.]</ety> <sn>1.</sn> <def>To make an agreement, esp. a secret agreement, to do some act, as to commit treason or a crime, or to do some unlawful deed; to plot together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>conspired</qex> against [Joseph] to slay him.</q> <rj><qau>Gen. xxxvii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You have <qex>conspired</qex> against our royal person,<br/
Joined with an enemy proclaimed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To concur to one end; to agree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The press, the pulpit, and the stage<br/
<qex>Conspire</qex> to censure and expose our age.</q> <rj><qau>Roscommon.</qau></rj></p>

<p><syn><b>Syn.</b> -- To unite; concur; complot; confederate; league.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspire</ent><br/
<hw>Con*spire"</hw>, <pos>v. t.</pos> <def>To plot; to plan; to combine for.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Angry clouds <qex>conspire</qex> your overthrow.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspirer</ent><br/
<hw>Con*spir"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conspires; a conspirator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspiringly</ent><br/
<hw>Con*spir"ing*ly</hw>, <pos>adv.</pos> <def>In the manner of a conspirator; by conspiracy.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspissation</ent><br/
<hw>Con`spis*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conspissatio</ets>, fr. <ets>conspissare</ets> to make thick.]</ety> <def>A making thick or viscous; thickness; inspissation.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspurcate</ent><br/
<hw>Con*spur"cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>conspurcatus</ets>, p. p. of <ets>conspurcare</ets>.]</ety> <def>To pollute; to defile.</def> <mark>[Obs.]</mark>  <rj><au>Cockeram.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conspurcation</ent><br/
<hw>Con`spur*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conspurcare</ets>, <ets>-spuratum</ets>, to defile.]</ety> <def>The act of defiling; defilement; pollution.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constable</ent><br/
<hw>Con"sta*ble</hw> <pr>(k<ocr/n"st<adot/*b'l <it>or</it> k<ucr/n"st<adot/*b'l)</pr>, <pos>n.</pos> <ety>[OE. <ets>conestable</ets>, <ets>constable</ets>, a constable (in sense 1), OF. <ets>conestable</ets>, F. <ets>conn<eacute/table</ets>, LL. <ets>conestabulus</ets>, <ets>constabularius</ets>, <ets>comes stabuli</ets>, orig., count of the stable, master of the horse, equerry; <ets>comes</ets> count (L. companion) + L. <ets>stabulum</ets> stable. See <er>Count</er> a nobleman, and <er>Stable</er>.]</ety> <sn>1.</sn> <def>A high officer in the monarchical establishments of the Middle Ages.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>constable of France</xex> was the first officer of the crown, and had the chief command of the army. It was also his duty to regulate all matters of chivalry. The office was suppressed in 1627. The <xex>constable</xex>, or <xex>lord high constable</xex>, <xex>of England</xex>, was one of the highest officers of the crown, commander in chief of the forces, and keeper of the peace of the nation. He also had judicial cognizance of many important matters. The office was as early as the Conquest, but has been disused (except on great and solemn occasions), since the attainder of Stafford, duke of Buckingham, in the reign of Henry VIII.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>An officer of the peace having power as a conservator of the public peace, and bound to execute the warrants of judicial officers.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, at the present time, the <xex>constable</xex> is a conservator of the peace within his district, and is also charged by various statutes with other duties, such as serving summons, precepts, warrants, etc. In the United States, <xex>constables</xex> are town or city officers of the peace, with powers similar to those of the constables of England. In addition to their duties as conservators of the peace, they are invested with others by statute, such as to execute civil as well as criminal process in certain cases, to attend courts, keep juries, etc. In some cities, there are officers called <cref>high constables</cref>, who act as chiefs of the constabulary or police force. In other cities the title of constable, as well as the office, is merged in that of the police officer.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>High constable</b></col>, <cd>a constable having certain duties and powers within a hundred.</cd> <mark>[Eng.]</mark> -- <col><b>Petty constable</b></col>, <cd>a conservator of the peace within a parish or tithing; a tithingman.</cd> <mark>[Eng.]</mark> -- <col><b>Special constable</b></col>, <cd>a person appointed to act as constable of special occasions.</cd> -- <mcol><col><b>To</b></col> <col><b>overrun the constable</b></col>, <it>or</it> <col><b>outrun the constable</b></col></mcol>, <cd>to spend more than one's income; to get into debt.</cd> <mark>[Colloq.]</mark>  <rj><au>Smollett.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constablery</ent><br/
<hw>Con"sta*bler*y</hw> <pr>(? <or/ <?/ )</pr>, <pos>n.</pos> <ety>[OF. <ets>conestablerie</ets>. Cf. <er>Constabulary</er>.]</ety> <sn>1.</sn> <def>The constabulary.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The district or jurisdiction of a constable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constableship</ent><br/
<hw>Con"sta*bleship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office or functions of a constable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constabless</ent><br/
<hw>Con"sta*bless</hw>, <pos>n.</pos> <def>The wife of a constable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constablewick</ent><br/
<hw>Con"sta*ble*wick`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Constable</ets> + <ets>wick</ets> a village]</ety> <def>The district to which a constable's power is limited.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constabulary</ent><br/
<hw>Con*stab"u*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>constabularius</ets> an equerry. See <er>Constable</er>.]</ety> <def>Of or pertaining to constables; consisting of constables.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constabulary</ent><br/
<hw>Con*stab"u*la*ry</hw>, <pos>n.</pos> <def>The collective body of constables in any town, district, or country.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constabulatory</ent><br/
<hw>Con*stab"u*la*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A constabulary.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constancy</ent><br/
<hw>Con"stan*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>constantia</ets>: cf. F. <ets>constance</ets>. See <er>Constant</er>.]</ety> <sn>1.</sn> <def>The state or quality of being constant or steadfast; freedom from change; stability; fixedness; immutability; <as>as, the <ex>constancy</ex> of God in his nature and attributes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fixedness or firmness of mind; persevering resolution; especially, firmness of mind under sufferings, steadiness in attachments, or perseverance in enterprise; stability; fidelity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fellow of plain uncoined <qex>constancy</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Constancy</qex> and contempt of danger.</q> <rj><qau>Prescott.</qau></rj></p>

<p><syn><b>Syn.</b> -- Fixedness; stability; firmness; steadiness; permanence; steadfastness; resolution. See <er>Firmness</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constant</ent><br/
<hw>Con"stant</hw> <pr>(k<ocr/n"st<ait/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>onstans</ets>, <ets>-antis</ets>, p. pr. of <ets>constare</ets> to stand firm, to be consistent; <ets>con-</ets> + <ets>stare</ets> to stand: cf. F. <ets>constant</ets>. See <er>Stand</er> and cf. <er>Cost</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Firm; solid; fixed; immovable; -- opposed to <ant>fluid</ant>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If . . . you mix them, you may turn these two fluid liquors into a <qex>constant</qex> body.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not liable, or given, to change; permanent; regular; continuous; continually recurring; steadfast; faithful; not fickle.  Opposite of <ant>changeable</ant> and <ant>variable</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Both loving one fair maid, they yet remained <qex>constant</qex> friends.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>constant</qex> to my purposes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His gifts, his <qex>constant</qex> courtship, nothing gained.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Onward the <qex>constant</qex> current sweeps.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Math. & Physics)</fld> <def>Remaining unchanged or invariable, as a quantity, force, law, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Consistent; logical.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- Fixed; steadfast; unchanging; permanent; unalterable; immutable; invariable; perpetual; continual; resolute; firm; unshaken; determined.</syn> <usage> -- <er>Constant</er>, <er>Continual</er>, <er>Perpetual</er>. These words are sometimes used in an absolute and sometimes in a qualified sense. <xex>Constant</xex> denotes, in its absolute sense, unchangeably fixed; as, a <xex>constant</xex> mind or purpose. In its qualified sense, it marks something as a <ldquo/standing<rdquo/ fact or occurence; as, liable to <xex>constant</xex> interruptions; <xex>constantly</xex> called for. <xex>Continual</xex>, in its absolute sense, coincides with <xex>continuous</xex>. See <er>Continuous</er>. In its qualified sense, it describes a thing as occuring in steady and rapid succession; as, a round of <xex>continual</xex> calls; <xex>continually</xex> changing. <xex>Perpetual</xex> denotes, in its absolute sense, what literally never ceases or comes to an end; as, <xex>perpetual</xex> motion. In its qualified sense, it is used hyperbolically, and denotes that which rarely ceases; as, <xex>perpetual</xex> disturbance; <xex>perpetual</xex> noise; <xex>perpetual</xex> intermeddling.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constant</ent><br/
<hw>Con"stant</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which is not subject to change; that which is invariable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>A quantity that does not change its value; -- used in countradistinction to <ant>variable</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>A number whose value, when ascertained (as by observation) and substituted in a general mathematical formula expressing an astronomical law, completely determines that law and enables predictions to be made of its effect in particular cases.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <fld>(Physics)</fld> <def>A number expressing some property or condition of a substance or of an instrument of precision; <as>as, the dielectric <ex>constant</ex> of quartz; the collimation <ex>constant</ex> of a transit instrument</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn>  <fld>(Computers)</fld> <def>a data structure that does not change during the course of execution of a program.  It may be a number, a string, or a more complex data structure; --  contrasted with <contr>variable</contr>.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Aberration constant</b></col>, <it>or</it>  <col><b>Constant of aberration</b></col></mcol> <fld>(Astron.)</fld>, <cd>a number which by substitution in the general formula for aberration enables a prediction to be made of the effect of aberration on a star anywhere situated. Its value is 20<sec/.47.</cd> -- <col><b>Absolute constant</b></col> <fld>(Math.)</fld>, <cd>one whose value is absolutely the same under all circumstances, as the number 10, or any numeral.</cd> -- <col><b>Arbitrary constant</b></col>, <cd>an undetermined constant in a differential equation having the same value during all changes in the values of the variables.</cd> -- <col><b>Gravitation constant</b></col> <fld>(Physics)</fld>, <cd>the acceleration per unit of time produced by the attraction of a unit of mass at unit distance. When this is known the acceleration produced at any distance can be calculated.</cd> -- <col><b>Solar constant</b></col> <fld>(Astron.)</fld>, <cd>the quantity of heat received by the earth from the sun in a unit of time. It is, on the C. G. S. system, 0.0417 small calories per square centimeter per second.</cd> <au>Young.</au> [<source>Webster 1913 Suppl.</source>] -- <col><b>Constant of integration</b></col> <fld>(Math.)</fld>, <cd>an undetermined constant added to every result of integration.</cd></cs><br/
[<source>1913 Webster</source> + <source>Webster 1913 Suppl.</source>]</p>

<p><ent>constantan</ent><br/
<hw>constantan</hw> <pos>n.</pos> <def>a copper-nickel alloy composed of approximately 55 per cent copper and 45 per cent nickel, with high electrical resistance and a low temperature coefficient; it is used as resistance wire and in thermocouples.</def> <note>It has a resistance factor <rho/ of 44.1 x 10<exp>-6</exp> ohm-cm at 0<deg/ C, in comparison to the value 1.71 x 10<exp>-6</exp> ohm-cm for pure copper and 7.8 x 10<exp>-6</exp> ohm-cm for pure nickel. It is similar to the alloy Eureka, which has a <rho/ = 47 x 10<exp>-6</exp> ohm-cm.  <au>Handbook of Ch. 61</au>.</note><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Constantia</ent><br/
<hw>Con*stan"ti*a</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <def>A superior wine, white and red, from <etsep>Constantia</etsep>, in South Africa.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Constantinople</ent><br/
<hw>Constantinople</hw> <pos>prop. n.</pos> <def>the former capital of the Eastern Roman Empire; it was built on the site of ancient <city>Byzantium</city>, and the name was changed to <city>Istanbul</city> by the Turks.</def><br/
<syn><b>Syn. --</b> Istanbul, Stambul, Stamboul.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><note>The name change was the subject of a humorous song in the 1950's "Istanbul (not Constantinople)":<br/
<pre><tt>
Artists: The Four Lads 
 -- peak Billboard position # 10 in 1953
 -- Words by Jimmy Kennedy and Music by Nat Simon
 -- (C) Chappell & Co. 

 Istanbul was Constantinople
 Now it's Istanbul, not Constantinople
 Been a long time gone, Constantinople
 Now it's Turkish delight on a moonlit night

 Every gal in Constantinople
 Lives in Istanbul, not Constantinople
 So if you've a date in Constantinople
 She'll be waiting in Istanbul

 Even old New York
 Was once New Amsterdam
 Why they changed it I can't say
 People just liked it better that way

 Take me back to Constantinople
 No, you can't go back to Constantinople
 Now it's Istanbul, not Constantinople
 Why did Constantinople get the works?
 That's nobody's business but the Turks'
</tt></pre></note><br/
[<source>PJC</source>]</p>

<p><ent>Constantly</ent><br/
<hw>Con"stant*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With constancy; steadily; continually; perseveringly; without cessation; uniformly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But she <qex>constantly</qex> affirmed that it was even so.</q> <rj><qau>Acts. xii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constat</ent><br/
||<hw>Con"stat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., it is evident.]</ety> <fld>(Law)</fld> <def>A certificate showing what appears upon record touching a matter in question.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constate</ent><br/
<hw>Con*state"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>constater</ets>; L. <ets>con-</ets> + <ets>stare</ets> to stand.]</ety> <def>To ascertain; to verify; to establish; to prove.</def>  <rj><au>F. P. Cobbe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constellate</ent><br/
<hw>Con"stel*late</hw> <pr>(? <or/ <?/)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>con-</ets> + L. <ets>stellatus</ets>, p. p. of <ets>stellare</ets> to cover with stars, <ets>stella</ets> star. See <er>Stellate</er>.]</ety> <def>To join luster; to shine with united radiance, or one general light.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The several things which engage our affections . . . shine forth and <qex>constellate</qex> in God.</q> <rj><qau>Boule.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constellate</ent><br/
<hw>Con"stel*late</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To unite in one luster or radiance, as stars.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Whe know how to <qex>constellate</qex> these lights.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To set or adorn with stars or constellations; <as>as, <ex>constellated</ex> heavens</as>.</def>  <rj><au>J. Barlow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constellation</ent><br/
<hw>Con`stel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>constellation</ets>, L. <ets>constellatio</ets>.]</ety> <sn>1.</sn> <def>A cluster or group of fixed stars, or division of the heavens, designated in most cases by the name of some animal, or of some mythologial personage, within whose imaginary outline, as traced upon the heavens, the group is included.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>constellations</qex> seem to have been almost purposely named and delineated to cause as much confusion and inconvenience as possible.</q> <rj><qau>Sir J. Herschel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In each of the constellations now recognized by astronomers (about 90 in number) the brightest stars, both named and unnamed, are designated nearly in the order of brilliancy by the letters of the Greek alphabet; as, <alpha/ Tauri (Aldebaran) is the first star of Taurus, <gamma/ Orionis (Bellatrix) is the third star of Orion.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assemblage of splendors or excellences.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>constellations</qex> of genius had already begun to show itself . . . which was to shed a glory over the meridian and close of Philip's reign.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fortune; fate; destiny.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is <qex>constellation</qex>, which causeth all that a man doeth.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>consternate</ent><br/
<hw>consternate</hw> <pos>v. t.</pos> <def>to cause to be confused; confuse emotionally; to dismay.</def><br/
<syn><b>Syn. --</b> confuse, flurry, disconcert, put off, bewilder, bemuse, discombobulate, throw.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Consternation</ent><br/
<hw>Con`ster*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consternatio</ets>, fr. <ets>consternare</ets> to overome, perplex, an accessory form of <ets>consternere</ets> to throw down, prostrate; <ets>con</ets> + <ets>sternere</ets> to spread out, throw down: cf. F. <ets>consternation</ets>. See <er>Stratum</er>.]</ety> <def>Amazement or horror that confounds the faculties, and incapacitates for reflection; terror, combined with amazement; dismay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The chiefs around,<br/
In silence wrapped, in <qex>consternation</qex> drowned.<br/
Attend the stern reply.</q> <rj><qau>Pope.</qau></rj></p>

<p><syn><b>Syn.</b> -- Alarm; fright; amazement; astonishment; surprise; panic; perturbation. See <er>Alarm</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constipate</ent><br/
<hw>Con"sti*pate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Constipated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Constipating</conjf>.]</vmorph> <ety>[L. <ets>constipatus</ets>, p. p. of <ets>constipare</ets>; <ets>con-</ets> + <ets>stipare</ets> to crowd together. See <er>Costive</er>.]</ety> <sn>1.</sn> <def>To crowd or cram into a narrow compass; to press together or condense.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of cold the property is to condense and <qex>constipate</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stop (a channel) by filling it, and preventing passage through it; <as>as, to <ex>constipate</ex> the capillary vessels</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>To render costive; to cause constipation in.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>constipated</ent><br/
<hw>constipated</hw> <pos>adj.</pos> <def>having difficult or incomplete or infrequent evacuation of the bowels; costive.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>constipating</ent><br/
<hw>constipating</hw> <pos>adj.</pos>  <def>causing constipation.</def><br/
<syn><b>Syn. --</b> binding.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Constipation</ent><br/
<hw>Con`sti*pa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>constipatio</ets> a crowding together: cf. F. <ets>constipation</ets>.]</ety> <sn>1.</sn> <def>Act of crowding anything into a less compass, or the state of being crowded or pressed together; condensation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Fullness of matter, or a pretty close <qex>constipation</qex> . . . of its particles.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state of the bowels in which the evacuations are infrequent and difficult, or the intestines become filled with hardened f<ae/ces; costiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constituency</ent><br/
<hw>Con*stit"u*en*cy</hw> <pr>(k<ocr/n*st<icr/t"<usl/*<esl/n*s<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  <plw>Constituencies</plw> <pr>(k<ocr/n*st<icr/t"<usl/*<esl/n*s<icr/z)</pr>.</plu> <def>A body of constituents, as the body of citizens or voters in a representative district.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constituent</ent><br/
<hw>Con*stit"u*ent</hw> <pr>(k<ocr/n*st<icr/t"<usl/*<esl/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>constituens</ets>, <ets>-entis</ets>, p. pr. See <er>Constitute</er>.]</ety> <sn>1.</sn> <def>Serving to form, compose, or make up; elemental; component.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Body, soul, and reason are the three parts necessarily <qex>constituent</qex> of a man.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the power of electing or appointing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A question of right arises between the <qex>constituent</qex> and representative body.</q> <rj><qau>Junius.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constituent</ent><br/
<hw>Con*stit"u*ent</hw>, <pos>n.</pos> <sn>1.</sn> <def>The person or thing which constitutes, determines, or constructs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their first composure and origination require a higher and nobler <qex>constituent</qex> than chance.</q> <rj><qau>Sir M. Hale</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which constitutes or composes, as a part, or an essential part; a component; an element.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We know how to bring these <qex>constituents</qex> together, and to cause them to form water.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One for whom another acts; especially, one who is represented by another in a legislative assembly; -- correlative to <inv>representative</inv>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The electors in the district of a representative in Congress, or in the legislature of a State, are termed his <qex>constituents</qex>.</q> <rj><qau>Abbot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To appeal from the representatives to the <qex>constituents</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>A person who appoints another to act for him as attorney in fact.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitute</ent><br/
<hw>Con"sti*tute</hw> <pr>(k<ocr/n"st<icr/*t<umac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Constituted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Constituting</conjf>.]</vmorph> <ety>[L. <ets>constitutus</ets>, p. p. of <ets>constiture</ets> to constitute; <ets>con-</ets> + <ets>statuere</ets> to place, set, fr. <ets>status</ets> station, fr. <ets>stare</ets> to stand. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>To cause to stand; to establish; to enact.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Laws appointed and <qex>constituted</qex> by lawful authority.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 310 --></p>

<p><sn>2.</sn> <def>To make up; to compose; to form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Truth and reason <qex>constitute</qex> that intellectual gold that defies destruction.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To appoint, depute, or elect to an office; to make and empower.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Me didst Thou <qex>constitute</qex> a priest of thine.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Constituted authorities</b></col>, <cd>the officers of government, collectively, as of a nation, city, town, etc.</cd>  <rj><au>Bartlett.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitute</ent><br/
<hw>Con"sti*tute</hw> <pr>(k<ocr/n"st<icr/*t<umac/t)</pr>, <pos>n.</pos> <def>An established law.</def> <mark>[Obs.]</mark>  <rj><au>T. Preston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constituter</ent><br/
<hw>Con"sti*tu`ter</hw> <pr>(k<ocr/n"st<icr/*t<umac/`t<etil/r)</pr>, <pos>n.</pos> <def>One who constitutes or appoints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitution</ent><br/
<hw>Con`sti*tu"tion</hw> <pr>(k<ocr/n`st<icr/*t<umac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>constitution</ets>, L. <ets>constitutio</ets>.]</ety> <sn>1.</sn> <def>The act or process of constituting; the action of enacting, establishing, or appointing; enactment; establishment; formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being; that form of being, or structure and connection of parts, which constitutes and characterizes a system or body; natural condition; structure; texture; conformation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The physical <qex>constitution</qex> of the sun.</q> <rj><qau>Sir J. Herschel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The aggregate of all one's inherited physical qualities; the aggregate of the vital powers of an individual, with reference to ability to endure hardship, resist disease, etc.; <as>as, a robust <ex>constitution</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>constitutions</qex> have never been enfeebled by the vices or luxuries of the old world.</q> <rj><qau>Story.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The aggregate of mental qualities; temperament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He defended himself with . . . less passion than was expected from his <qex>constitution</qex>.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The fundamental, organic law or principles of government of men, embodied in written documents, or implied in the institutions and usages of the country or society; also, a written instrument embodying such organic law, and laying down fundamental rules and principles for the conduct of affairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>constitution</qex> had begun to exist in times when statesmen were not much accustomed to frame exact definitions.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England the constitution is unwritten, and may be modified from time to time by act of Parliament.  In the United States a constitution cannot ordinarily be modified, exept through such processes as the constitution itself ordains.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An authoritative ordinance, regulation or enactment; especially, one made by a Roman emperor, or one affecting ecclesiastical doctrine or discipline; <as>as, the <ex>constitutions</ex> of Justinian</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The positive <qex>constitutions</qex> of our own churches.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>constitution</qex> of Valentinian addressed to Olybrius, then prefect of Rome, for the regulation of the conduct of advocates.</q> <rj><qau>George Long.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Apostolic constitutions</b></col>. <cd>See under <er>Apostolic</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutional</ent><br/
<hw>Con`sti*tu"tion*al</hw> <pr>(k<ocr/n`st<icr/*t<umac/"sh<ucr/n*<ait/l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>constitutionnel</ets>.]</ety> <sn>1.</sn> <def>Belonging to, or inherent in, the constitution, or in the structure of body or mind; <as>as, a <ex>constitutional</ex> infirmity; <ex>constitutional</ex> ardor or dullness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In accordance with, or authorized by, the constitution of a state or a society; <as>as, <ex>constitutional</ex> reforms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Regulated by, dependent on, or secured by, a constitution; <as>as, <ex>constitutional</ex> government; <ex>constitutional</ex> rights</as>.</def>  <rj><au>Hallam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Relating to a constitution, or establishment form of government; <as>as, a <ex>constitutional</ex> risis</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The anient <qex>constitutional</qex> traditions of the state.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>For the benefit or one's constitution or health; <as>as, a <ex>constitutional</ex> walk</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Constitutional law</b></col>, <cd>law that relates to the constitution, as a permanent system of political and juridical government, as distinguished from statutory and common law, which relate to matters subordinate to such constitution.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutional</ent><br/
<hw>Con`sti*tu"tion*al</hw>, <pos>n.</pos> <def>A walk or other exercise taken for one's health or constitution.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The men trudged diurnal <qex>constitutionals</qex> along the different roads.</q> <rj><qau>Compton Reade.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutionalism</ent><br/
<hw>Con`sti*tu"tion*al*ism</hw> <pr>(k<ocr/n`st<icr/*t<umac/"sh<ucr/n*<ait/l*<icr/z'm)</pr>, <pos>n.</pos> <def>The theory, principles, or authority of constitutional government; attachment or adherence to a constitution or constitutional government.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutionalist</ent><br/
<hw>Con`sti*tu"tion*al*ist</hw>, <pos>n.</pos> <def>One who advocates a constitutional form of government; a constitutionalist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutionality</ent><br/
<hw>Con`sti*tu`tion*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> -<plw>ties</plw> <pr>(#)</pr></plu>. <ety>[Cf. F. <ets>constitutionalit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being constitutional, or inherent in the natural frame.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being consistent with the constitution or frame of government, or of being authorized by its provisions.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Constitutionalities</qex>, bottomless cavilings and questionings about written laws.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutionally</ent><br/
<hw>Con`sti*tu"tion*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In accordance with the constitution or natural disposition of the mind or body; naturally; <as>as, he was <ex>constitutionally</ex> timid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The English were <qex>constitutionally</qex> humane.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In accordance with the constitution or fundamental law; legally; <as>as, he was not <ex>constitutionally</ex> appointed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing would indue them to acknowledge that [such] an assembly . . . was <qex>constitutionally</qex> a Parliament.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutionist</ent><br/
<hw>Con`sti*tu"tion*ist</hw>, <pos>n.</pos> <def>One who adheres to the constitution of the country.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutive</ent><br/
<hw>Con"sti*tu`tive</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Tending or assisting to constitute or compose; elemental; essential.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An ingredient and <qex>constitutive</qex> part of every virtue.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having power to enact, establish, or create; instituting; determining.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constitutively</ent><br/
<hw>Con"sti*tu`tive*ly</hw>, <pos>adv.</pos> <def>In a constitutive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrain</ent><br/
<hw>Con*strain"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Constrained</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Constraining</conjf>.]</vmorph> <ety>[OF. <ets>constraindre</ets>, F. <ets>contrainde</ets>, L. <ets>constringere</ets>; <ets>con-</ets> + <ets>stringere</ets> to draw tight. See <er>Strain</er>, and. cf. <er>Constrict</er>, <er>Constringe</er>.]</ety> <sn>1.</sn> <def>To secure by bonds; to chain; to bond or confine; to hold tightly; to constringe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He binds in chains<br/
The drowsy prophet, and his limbs <qex>constrains</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When winter frosts <qex>constrain</qex> the fields with cold.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring into a narrow compass; to compress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How the strait stays the slender waist <qex>constrain</qex>.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hold back by force; to restrain; to repress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My sire in caves <qex>constrains</qex> the winds.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To compel; to force; to necessitate; to oblige.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The love of Christ <qex>constraineth</qex> us.</q> <rj><qau>2. Cor. v. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was <qex>constrained</qex> to appeal unto C<ae/sar.</q> <rj><qau>Acts xxviii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To violate; to ravish.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To produce in such a manner as to give an unnatural effect; <as>as, a <ex>constrained</ex> voice</as>.</def></p>

<p><syn><b>Syn.</b> -- To compel; force; drive; impel; urge; press.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrainable</ent><br/
<hw>Con*strain"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. OF. <ets>constraignable</ets>, F. <ets>contraignable</ets>.]</ety> <def>Capable of being constrained; liable to constraint, or to restraint.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrained</ent><br/
<hw>Con*strained"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Marked by constraint; not free; not voluntary; embarrassed; <as>as, a <ex>constrained</ex> manner; a <ex>constrained</ex> tone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrainedly</ent><br/
<hw>Con*strain"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By constraint or compulsion; in a constrained manner.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrainer</ent><br/
<hw>Con*strain"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who constrains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constraint</ent><br/
<hw>Con*straint"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>constrainte</ets>, F. <ets>constrainte</ets>.]</ety> <def>The act of constraining, or the state of being constrained; that which compels to, or restrains from, action; compulsion; restraint; necessity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Long imprisonment and hard <qex>constraint</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Not by <qex>constraint</qex>, but by my choice, I came.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Compulsion; violence; necessity; urgency.</syn> <usage> -- <er>Constraint</er>, <er>Compulsion</er>. <xex>Constraint</xex> implies strong binding force; as, the <xex>constraint</xex> of necessity; the <xex>constraint</xex> of fear. <xex>Compulsion</xex> implies the exertion of some urgent impelling force; as, driven by <xex>compulsion</xex>. The former prevents us from acting agreeably to our wishes; the latter forces us to act contrary to our will. <xex>Compulsion</xex> is always produced by some active agent; a <xex>constraint</xex> may be laid upon us by the forms of civil society, or by other outward circumstances.  <rj><au>Crabb.</au></rj></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constraintive</ent><br/
<hw>Con*straint"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Constraining; compulsory.</def> <mark>[R.]</mark> <ldquo/Any <xex>constraintive</xex> vow.<rdquo/  <rj><au>R. Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrict</ent><br/
<hw>Con*strict"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Constricted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Constricting</conjf>.]</vmorph> <ety>[L. <ets>constrictus</ets>, p. p. of <ets>constringere</ets>. See <er>Constrain</er>.]</ety> <def>To draw together; to render narrower or smaller; to bind; to cramp; to contract or cause to shrink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such things as <qex>constrict</qex> the fibers.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Membranous organs inclosing a cavity which their contraction serves to <qex>constrict</qex>.</q> <rj><qau>Todd & Bowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constricted</ent><br/
<hw>Con*strict"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Drawn together; bound; contracted; cramped.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Contracted or compressed so as to be smaller in certain places or parts than in others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>constricting</ent><br/
<hw>constricting</hw> <pos>adj.</pos> <sn>1.</sn>  <def>hindering freedom of movement.</def><br/
<syn><b>Syn. --</b> binding.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>being reduced in width.</def><br/
<syn><b>Syn. --</b> constrictive, narrowing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Constriction</ent><br/
<hw>Con*stric"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>constrictio</ets>: cf. F. <ets>constriction</ets>.]</ety> <sn>1.</sn> <def>The act of constricting by means of some inherent power or by movement or change in the thing itself, as distinguished from <xex>compression</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being constricted; the point where a thing is constricted; a narrowing or binding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>constriction</qex> of the parts inservient to speech.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrictive</ent><br/
<hw>Con*strict"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving or tending to bind or constrict.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constrictor</ent><br/
<hw>Con*strict"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which constricts, draws together, or contracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A muscle which contracts or closes an orifice, or which compresses an organ; a sphincter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A serpent that kills its prey by inclosing and crushing it with its folds; <as>as, the boa <ex>constrictor</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constringe</ent><br/
<hw>Con*stringe"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Constringed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Constringing</conjf>.]</vmorph> <ety>[L. <ets>constringere</ets>. See <er>onstrain</er>.]</ety> <def>To dawn together; to contract; to force to contract itself; to constrict; to cause to shrink.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Strong liquors . . . intoxicate, <qex>constringe</qex>, harden the fibers, and coagulate the fluids.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constringent</ent><br/
<hw>Con*strin"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>constringens</ets>, p. pr.]</ety> <def>Having the quality of contracting, binding, or compressing.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Construct</ent><br/
<hw>Con*struct"</hw> <pr>(k<ocr/n*str<ucr/kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Constructed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Constructing</conjf>.]</vmorph> <ety>[L. <ets>constructus</ets>, p. p. of <ets>construere</ets> to bring together, to construct; <ets>con-</ets> + <ets>struere</ets> to pile up, set in order. See <er>Structure</er>, and cf. <er>Construe</er>.]</ety> <sn>1.</sn> <def>To put together the constituent parts of (something) in their proper place and order; to build; to form; to make; <as>as, to <ex>construct</ex> an edifice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To devise; to invent; to set in order; to arrange; <as>as, to <ex>construct</ex> a theory of ethics</as>.</def></p>

<p><syn><b>Syn.</b> -- To build; erect; form; compile; make; fabricate; originate; invent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Construct</ent><br/
<hw>Con"struct</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Formed by, or relating to, construction, interpretation, or inference.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Construct form</b></col> <it>or</it> <col><b>Construct state</b></col></mcol> <fld>(Heb. Gram.)</fld>, <cd>that of a noun used before another which has the genitive relation to it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructer</ent><br/
<hw>Con*struct"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, constructs or frames.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Construction</ent><br/
<hw>Con*struc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>constructio</ets>: cf. F. <ets>construction</ets>.]</ety> <sn>1.</sn> <def>The process or art of constructing; the act of building; erection; the act of devising and forming; fabrication; composition.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The form or manner of building or putting together the parts of anything; structure; arrangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An astrolabe of peculiar <qex>construction</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>The arrangement and connection of words in a sentence; syntactical arrangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some particles . . . in certain <qex>constructions</qex> have the sense of a whole sentence contained in them.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The method of construing, interpreting, or explaining a declaration or fact; an attributed sense or meaning; understanding; explanation; interpretation; sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Any person . . . might, by the sort of <qex>construction</qex> that would be put on this act, become liable to the penalties of treason.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Strictly</qex>, <qex>the term</qex> [<qex>construction</qex>] signifies determining the meaning and proper effect of language by a consideration of the subject matter and attendant circumstances in connection with the words employed.</q> <rj><qau>Abbott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Interpretation properly precedes <qex>construction</qex>, but it does not go beyond the written text.</q> <rj><qau>Parsons.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Construction of an equation</b></col> <fld>(Math.)</fld>, <cd>the drawing of such lines and figures as will represent geometrically the quantities in the equation, and their relations to each other.</cd> -- <col><b>Construction train</b></col> <fld>(Railroad)</fld>, <cd>a train for transporting men and materials for construction or repairs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructional</ent><br/
<hw>Con*struc"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or deduced from, construction or interpretation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructionist</ent><br/
<hw>Con*struc"tion*ist</hw>, <pos>n.</pos> <def>One who puts a certain construction upon some writing or instrument, as the Constitutions of the United States; <as>as, a strict <ex>constructionist</ex>; a broad <ex>constructionist</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructive</ent><br/
<hw>Con*struct"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>constructif</ets>.]</ety> <sn>1.</sn> <def>Having ability to construct or form; employed in construction; <as>as, to exhibit <ex>constructive</ex> power</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>constructive</qex> fingers of Watts.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Derived from, or depending on, construction, inference, or interpretation; not directly expressed, but inferred.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>helpful; promoting improvement; intended to help; <as>as, <ex>constructive</ex> criticism; <ex>constructive</ex> suggestions</as>.  Contrasted with <contr>destructive</contr>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Constructive crimes</b></col> <fld>(Law)</fld>, <cd>acts having effects analogous to those of some statutory or common law crimes; as, <xex>constructive</xex> treason. Constructive crimes are no longer recognized by the courts.</cd> -- <col><b>Constructive notice</b></col>, <cd>notice imputed by construction of law.</cd> -- <col><b>Constructive trust</b></col>, <cd>a trust which may be assumed to exist, though no actual mention of it be made.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructively</ent><br/
<hw>Con*struct"ive*ly</hw>, <pos>adv.</pos> <def>In a constructive manner; by construction or inference.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A neutral must have notice of a blockade, either actually by a formal information, or <qex>constructively</qex> by notice to his government.</q> <rj><qau>Kent.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>constructive-metabolic</ent><br/
<hw>constructive-metabolic</hw> <pos>adj. prenom.</pos>  <def>causing buildup of body tissues or tissue components.</def> <note>[Narrower terms: <stype>anabolic (vs. catabolic)</stype>]</note><br/
<syn><b>Syn. --</b> energy-storing(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Constructiveness</ent><br/
<hw>Con*struct"ive*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Tendency or ability to form or construct.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phren.)</fld> <def>The faculty which enables one to construct, as in mechanical, artistic, or literary matters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructor</ent><br/
<hw>Con*struct"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>constructor</ets>.]</ety> <def>A constructer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constructure</ent><br/
<hw>Con*struc"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos>  <def>That which is constructed or formed; an edifice; a fabric.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Construe</ent><br/
<hw>Con*strue</hw> <pr>(?; <it>Archaic</it> ?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Construed</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Construing</conjf> <pr>(#)</pr>.]</vmorph> <ety>[L. <ets>construere</ets>: cf. F. <ets>construire</ets>. See <er>Construct</er>.]</ety> <sn>1.</sn> <def>To apply the rules of syntax to (a sentence or clause) so as to exhibit the structure, arrangement, or connection of, or to discover the sense; to explain the construction of; to interpret; to translate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put a construction upon; to explain the sense or intention of; to interpret; to understand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus we are put to <qex>construe</qex> and paraphrase our own words to free ourselves either from the ignorance or malice of our enemies.</q> <rj><qau>Bp. Stilingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And to be dull was <qex>construed</qex> to be good.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constuprate</ent><br/
<hw>Con"stu*prate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Construprated</conjf>; <pos>p. p. & vb. n.</pos> <conjf>Constuprating</conjf>.]</vmorph> <ety>[L. <ets>constupratus</ets>, p. p. of <ets>constuprare</ets> to ravish; <ets>con-</ets> + <ets>stuprare</ets> to ravish, <ets>stuprum</ets> rape.]</ety> <def>To ravish; to debauch.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Constupration</ent><br/
<hw>Con`stu*pra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of ravishing; violation; defilement.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantial</ent><br/
<hw>Con`sub*stan"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consubstantialis</ets>; <ets>con-</ets> + <ets>substantialis</ets>: cf. F. <ets>consubstantiel</ets>. See <er>Substantial</er>.]</ety> <def>Of the same kind or nature; having the same substance or essence; coessential.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Christ Jesus . . . coeternal and <qex>consubstantial</qex> with the Father and with the Holy Ghost.</q> <rj><qau>Foxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantialism</ent><br/
<hw>Con`sub*stan"tial*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine of consubstantiation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantialist</ent><br/
<hw>Con`sub*stan"tial*ist</hw>, <pos>n.</pos> <def>One who believes in consubstantiation.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantiality</ent><br/
<hw>Con`sub*stan"ti*al"i*ty</hw> <pr>(?;  106)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>consubstantialit<eacute/</ets>.]</ety> <def>Participation of the same nature; coexistence in the same substance.</def> <ldquo/His [the Son's] . . . <xex>consubstantiality</xex> with the Father.<rdquo/  <rj><au>Hammend.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantially</ent><br/
<hw>Con`sub*stan"tial*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a consubstantial manner; with identity of substance or nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantiate</ent><br/
<hw>Con`sub*stan"ti*ate</hw> <pr>(?; 106)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consubstantiated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Consubstantiating</conjf>.]</vmorph> <def>To cause to unite, or to regard as united, in one common substance or nature.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His soul must be <qex>consubstantiated</qex> with reason.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantiate</ent><br/
<hw>Con`sub*stan"ti*ate</hw>, <pos>v. i.</pos> <def>To profess or belive the doctrine of consubstantion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>consubstantiating</qex> church and priest.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantiate</ent><br/
<hw>Con`sub*stan"ti*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partaking of the same substance; united; consubstantial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We must love her [the wife] that is thus <qex>consubstantiate</qex> with us.</q> <rj><qau>Feltham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consubstantiation</ent><br/
<hw>Con`sub*stan`ti*a"tion</hw> <pr>(?; 106)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An identity or union of substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>The actual, substantial presence of the body of Christ with the bread and wine of the sacrament of the Lord's Supper; impanation; -- opposed to <xex>transubstantiation</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This view, held by Luther himself, was called <xex>consubstantiation</xex> by non Lutheran writers in contradistinction to transsubstantiation, the Catholic view.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consuetude</ent><br/
<hw>Con"sue*tude</hw> <pr>(?; 144)</pr>, <pos>n.</pos> <ety>[L. <ets>consuetudo</ets>. See <er>Custom</er>.]</ety> <def>Custom, habit; usage.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To observe this <qex>consuetude</qex> or law.</q> <rj><qau>Barnes</qau></rj>.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Consuetudinal</ent><br/
<hw>Con`sue*tu"di*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>consuetudinalis</ets>.]</ety> <def>According to custom; customary; usual.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consuetudinary</ent><br/
<hw>Con`sue*tu"di*na"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>consuetudinarius</ets>.]</ety> <def>Customary.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 311 --></p>

<p><ent>Consuetudinary</ent><br/
<hw>Con`sue*tu"di*na*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Consuetudinaries</plw> <pr>(<?/)</pr>.</plu> <def>A manual or ritual of customary devotional exercises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consul</ent><br/
<hw>Con"sul</hw> <pr>(k<ocr/n"s<ucr/l)</pr>, <pos>n.</pos> <ety>[L., prob. fr. <ets>consulere</ets> to deliberate. See <er>Consult</er>.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>One of the two chief magistrates of the republic.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They were chosen annually, originally from the patricians only, but later from the plebeians also.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A senator; a counselor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many of the <qex>consuls</qex>, raised and met,<br/
Are at the duke's already.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With kings and <qex>consuls</qex> of the earth.</q> <rj><qau>Job. iii. 14 (Douay Ver. )</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fr. Hist.)</fld> <def>One of the three chief magistrates of France from 1799 to 1804, who were called, respectively, first, second, and third consul.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An official commissioned to reside in some foreign country, to care for the commercial interests of the citizens of the appointing government, and to protect its seamen.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consul general</b></col>, <cd>a consul of the first rank, stationed in an important place, or having jurisdiction in several places or over several consuls.</cd> -- <col><b>Vice consul</b></col>, <cd>a consular officer holding the place of a consul during the consul's absence or after he has been relieved.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consulage</ent><br/
<hw>Con"sul*age</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Com.)</fld> <def>A duty or tax paid by merchants for the protection of their commerce by means of a consul in a foreign place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consular</ent><br/
<hw>Con"su*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>consularis</ets>; cf. F. <ets>consulaire</ets>.]</ety> <def>Of or pertaining to a consul; performing the duties of a consul; <as>as, <ex>consular</ex> power; <ex>consular</ex> dignity; <ex>consular</ex> officers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consulary</ent><br/
<hw>Con"su*la"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consular.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consulate</ent><br/
<hw>Con"su*late</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consulatus</ets>: cf. F. <ets>consulat</ets>.]</ety> <sn>1.</sn> <def>The office of a consul.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The jurisdiction or residence of a consul.</def>  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Consular government; term of office of a consul.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consulship</ent><br/
<hw>Con"sul*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The office of a consul; consulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The term of office of a consul.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consult</ent><br/
<hw>Con*sult"</hw> <pr>(k<ocr/n*s<ucr/lt")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consulted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Consulting</conjf>.]</vmorph> <ety>[L. <ets>consultare</ets>, fr. <ets>consulere</ets> to consult: cf. f. <ets>consulter</ets>. Cf. <er>Counsel</er>.]</ety> <def>To seek the opinion or advice of another; to take counsel; to deliberate together; to confer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us <qex>consult</qex> upon to-morrow's business.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the laws of England have been made by the kings England, <qex>consulting</qex> with the nobility and commons.</q> <rj><qau>Hobbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consult</ent><br/
<hw>Con*sult"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To ask advice of; to seek the opinion of; to apply to for information or instruction; to refer to; <as>as, to <ex>consult</ex> a physician; to <ex>consult</ex> a dictionary</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men forgot, or feared, to <qex>consult</qex> nature . . . ; they were content to <qex>consult</qex> libraries.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have reference to, in judging or acting; to have regard to; to consider; <as>as, to <ex>consult</ex> one's wishes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are . . .  to <qex>consult</qex> the necessities of life, rather than matters of ornament and delight.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To deliberate upon; to take for.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Manythings were there <qex>consulted</qex> for the future, yet nothing was positively resolved.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To bring about by counsel or contrivance; to devise; to contrive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast <qex>consulted</qex> shame to thy house by cutting off many people.</q> <rj><qau>Hab. ii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consult</ent><br/
<hw>Con*sult"</hw> <pr>(k<ocr/n*s<ucr/lt" <it>or</it>  k<ocr/n"s<ucr/lt)</pr>, <pos>n.</pos>  <sn>1.</sn> <def>The act of consulting or deliberating; consultation; also, the result of consulation; determination; decision.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The council broke;<br/
And all grave <qex>consults</qex> dissolved in smoke.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A council; a meeting for consultation.</def> <mark>[Obs.]</mark> <ldquo/A <xex>consult</xex> of coquettes.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Agreement; concert</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>consultant</ent><br/
<hw>consultant</hw> <pos>n.</pos> <def>an expert who gives advice.</def><br/
<syn><b>Syn. --</b> adviser, advisor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Consultary</ent><br/
<hw>Con*sult"a*ry</hw> <pr>(k<ocr/n*s<ucr/lt"<adot/*r<ycr/)</pr>, <pos>a.</pos> <def>Formed by consultation; resulting from conference.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consultary response</b></col> <fld>(Law)</fld>, <cd>the opinion of a court on a special case.</cd></cs>
 <rj><qau>Wharton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consultation</ent><br/
<hw>Con`sul*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>consultatio</ets>: cf. F. <ets>consultation</ets>.]</ety> <sn>1.</sn> <def>The act of consulting or conferring; deliberation of two or more persons on some matter, with a view to a decision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus they doubtful <qex>consultations</qex> dark<br/
Ended.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A council or conference, as of physicians, held to consider a special case, or of lawyers restained in a cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Writ of consultation</b></col> <fld>(Law)</fld>, <cd>a writ by which a cause, improperly removed by prohibition from one court to another, is returned to the court from which it came; -- so called because the judges, on <xex>consultation</xex>, find the prohibition ill-founded.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consultative</ent><br/
<hw>Con*sult"a*tive</hw> <pr>(k<ocr/n*s<ucr/lt"<adot/*t<icr/v)</pr>, <pos>a.</pos> <def>Pertaining to consultation; having the privilege or right of conference.</def> <ldquo/A <xex>consultative</xex> . . . power.<rdquo/  <rj><au>Abp. Bramhall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consultatory</ent><br/
<hw>Con*sult"a*to*ry</hw> <pr>(k<ocr/n*s<ucr/lt"<adot/*t<osl/*r<ycr/)</pr>, <pos>a.</pos> <def>Formed by, or resulting from, consultation; advisory.</def>  <rj><au>Bancroft.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consulter</ent><br/
<hw>Con*sult"er</hw> <pr>(k<ocr/n*s<ucr/lt"<etil/r)</pr>, <pos>n.</pos> <def>One who consults, or asks counsel or information.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consulting</ent><br/
<hw>Con*sult"ing</hw>, <pos>a.</pos> <def>That consults.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consulting physician</b></col> <fld>(Med.)</fld>, <cd>a physician who consults with the attending practitioner regarding any case of disease.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consultive</ent><br/
<hw>Con*sult"ive</hw> <pr>(k<ocr/n*s<ucr/lt"<icr/v)</pr>, <pos>a.</pos> <def>Determined by, or pertaining to, consultation; deliberate; consultative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that remains in the grace of God sins not by any deliberative, <qex>consultive</qex>, knowing act.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumable</ent><br/
<hw>Con*sum"a*ble</hw> <pr>(k<ocr/n*s<umac/m"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being consumed; that may be destroyed, dissipated, wasted, or spent.</def> <ldquo/<xex>Consumable</xex> commodities.<rdquo/  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consume</ent><br/
<hw>Con*sume"</hw> <pr>(k<ocr/n*s<umac/m")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consumed</conjf> <pr>(k<ocr/n*s<umac/md")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Consuming</conjf>.]</vmorph> <ety>[L. <ets>consumere</ets> to take wholly or completely, to consume; <ets>con-</ets> + <ets>sumere</ets> to take; <ets>sub</ets> + <ets>emere</ets> to buy. See <er>Redeem</er>.]</ety> <def>To destroy, as by decomposition, dissipation, waste, or fire; to use up; to expend; to waste; to burn up; to eat up; to devour.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If he were putting to my house the brand<br/
That shall <qex>consume</qex> it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lay up for yourselves treasures in heaven, where neither moth nor rust doth <qex>consume</qex>.</q> <rj><qau>Matt. vi. 20 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me alone . . . that I may <qex>consume</qex> them.</q> <rj><qau>Ex. xxxii. 10.</qau></rj></p>

<p><syn><b>Syn.</b> -- To destroy; swallow up; ingulf; absorb; waste; exhaust; spend; expend; squander; lavish; dissipate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consume</ent><br/
<hw>Con*sume"</hw> <pr>(k<ocr/n*s<umac/m")</pr>, <pos>v. i.</pos> <def>To waste away slowly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore, let Benedick, like covered fire,<br/
<qex>Consume</qex> away in sighs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>consumed</ent><br/
<hw>consumed</hw> <pos>adj.</pos> <sn>1.</sn> <def>completely used up.</def><br/
<syn><b>Syn. --</b> used-up(prenominal), used up(predicate).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>eaten or drunk up.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>consuming</ent><br/
<hw>consuming</hw> <pos>adj.</pos> <def>taking up most of one's attention; ardent; <as>as, politics is his <ex>consuming</ex> passion</as>.</def><br/
<syn><b>Syn. --</b> overwhelming.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Consumedly</ent><br/
<hw>Con*sum"ed*ly</hw> <pr>(k<ocr/n*s<umac/m"<ecr/d*l<ycr/)</pr>, <pos>adv.</pos> <def>Excessively.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He's so <qex>consumedly</qex> proud of it.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumer</ent><br/
<hw>Con*sum"er</hw> <pr>(k<ocr/n*s<umac/m"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, consumes; <as>as, the <ex>consumer</ex> of food</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Econ.)</fld> <def>the person or organization that uses some item of commerce or service in its own acitities, as opposed to reselling the item or including it as part of another item for resale; -- called also the <altname>end user</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>consumer price index</ent><br/
<hw>consumer price index</hw> <pos>n.</pos> <def>An index of the cost of all goods and services to a typical consumer, calculated and published by the United States Bureau of Labor Statistics; abbreviated <altname>CPI</altname>, and usually referred to by that acronym.  The Bureau maintains several indices for different groups of consumers, but the most commonly referred to is the index for <ldquo/All urban consumers<rdquo/, called the <stype>CPI-U</stype>. The increase of this value each year is one measure of monetary inflation.</def><br/
<syn><b>Syn. --</b> CPI, cost-of-living index.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><note><hand/  For a table of values from the Bureau of Labor Statistics of the CPI over time, see <a href="more\cpiai.htm">CPI-U from 1913 to 1998</a>.</note><br/
[<source>PJC</source>]</p>

<p><ent>Consumer's goods</ent><br/
<hw>Con*sum"er's goods</hw> <pr>(?)</pr>. <fld>(Polit. Econ.)</fld> <def>Economic goods that directly satisfy human wants or desires, such as food, clothes, pictures, etc.; -- called also <altname>consumption goods</altname>, or <altname>goods of the first order</altname>, and opposed to <contr>producer's goods</contr>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Consumer's surplus</ent><br/
<hw>Consumer's surplus</hw>. <fld>(Polit. econ.)</fld> <def>The excess that a purchaser would be willing to pay for a commodity over that he does pay, rather than go without the commodity; -- called also <col><b>consumer's rent</b></col>.</def></p>

<p><q>The price which a person pays for a thing can never exceed, and seldom comes up to, that which he would be willing to pay rather than go without it. . . . The excess of the price which he would be willing to pay rather than go without it, over that which he actually does pay, is the economic measure of this surplus satisfaction.  It has some analogies to a rent; but is perhaps best called simply <qex>consumer's surplus</qex>.</q>  <rj><qau>Alfred Marshall.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Consumingly</ent><br/
<hw>Con*sum"ing*ly</hw>, <pos>adv.</pos> <def>In a consuming manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consummate</ent><br/
<hw>Con*sum"mate</hw> <pr>(k<ocr/n*s<ucr/m"m<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>consummatus</ets>, p. p. or <ets>consummare</ets> to accomplish, sum up; <ets>con-</ets> + <ets>summa</ets> sum. See <er>Sum</er>.]</ety> <def>Carried to the utmost extent or degree; of the highest quality; complete; perfect.</def> <ldquo/A man of perfect and <xex>consummate</xex> virtue.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The little band held the post with <qex>consummate</qex> tenacity.</q> <rj><qau>Motley</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consummate</ent><br/
<hw>Con"sum*mate</hw> <pr>(k<ocr/n"s<ucr/m*m<amac/t <it>or</it> k<ocr/n*s<ucr/m"m<amac/t; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Consummated</conjf> <pr>(k<ocr/n"s<ucr/m*m<amac/`t<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Consummating</conjf> <pr>(k<ocr/n"s<ucr/m*m<amac/`t<icr/ng)</pr>.]</vmorph> <def>To bring to completion; to raise to the highest point or degree; to complete; to finish; to perfect; to achieve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>consummate</qex> this business happily.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consummately</ent><br/
<hw>Con*sum"mate*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a consummate manner; completely.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consummation</ent><br/
<hw>Con`sum*ma"tion</hw> <pr>(k<ocr/n`s<ucr/m*m<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>consummatio</ets>.]</ety> <def>The act of consummating, or the state of being consummated; completion; perfection; termination; end (as of the world or of life).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is a <qex>consummation</qex><br/
Devoutly to be wished.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>From its original to its <qex>consummation</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Quiet <qex>consummation</qex> have,<br/
And renown<eacute/d be thy grave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consummation of marriage</b></col>, <cd>completion of the connubial relation by actual cohabition.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consummative</ent><br/
<hw>Con*sum"ma*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to consummate; completing.</def> <ldquo/The final, the <xex>consummative</xex> procedure of philosophy.<rdquo/  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumption</ent><br/
<hw>Con*sump"tion</hw> <pr>(?; 215)</pr>, <pos>n.</pos> <ety>[L. <ets>consumptio</ets>: cf. F. <ets>consomption</ets>.]</ety> <sn>1.</sn> <def>The act or process of consuming by use, waste, etc.; decay; destruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every new advance of the price to the consumer is a new incentive to him to retrench the quality of his <qex>consumption</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state or process of being consumed, wasted, or diminished; waste; diminution; loss; decay.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A progressive wasting away of the body; esp., that form of wasting, attendant upon pulmonary phthisis and associated with cough, spitting of blood, hectic fever, etc.; pulmonary phthisis; -- called also <altname>pulmonary consumption</altname>.</def><-- tuberculosis --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Consumption of the bowels</b></col> <fld>(Med.)</fld>, <cd>inflammation and ulceration of the intestines from tubercular disease.</cd></cs></p>

<p><syn><b>Syn.</b> -- Decline; waste; decay. See <er>Decline</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumptive</ent><br/
<hw>Con*sump"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>consomptif</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to consumption; having the quality of consuming, or dissipating; destructive; wasting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [prayer] is not <qex>consumptive</qex> or our time.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A long <qex>consumptive</qex> war.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Affected with, or inclined to, consumption.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lean, <qex>consumptive</qex> wench, with coughs decayed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumptive</ent><br/
<hw>Con*sump"tive</hw>, <pos>n.</pos> <def>One affected with consumption; <as>as, a resort for <ex>consumptives</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumptively</ent><br/
<hw>Con*sump"tive*ly</hw>, <pos>adv.</pos> <def>In a way tending to or indication consumption.</def>  <rj><au>Beddoes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Consumptiveness</ent><br/
<hw>Con*sump"tive*ness</hw>, <pos>n.</pos> <def>A state of being consumptive, or a tendency to a consumption.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contabescent</ent><br/
<hw>Con`ta*bes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contabescenc</ets>, p. pr. of <ets>contabescere</ets>.]</ety> <def>Wasting away gradually.</def>  <rj><au>Darwin.</au></rj></p>

<p>-- <wordforms><wf>Con*ta*bes"cence</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contact</ent><br/
<hw>Con"tact</hw> <pr>(k<ocr/n"t<acr/kt)</pr>, <pos>n.</pos> <ety>[L. <ets>contactus</ets>, fr. <ets>contingere</ets>, <ets>-tactum</ets>, to touch on all sides. See <er>Contingent</er>.]</ety> <sn>1.</sn> <def>A close union or junction of bodies; a touching or meeting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>The property of two curves, or surfaces, which meet, and at the point of meeting have a common direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>The plane between two adjacent bodies of dissimilar rock.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Electricity)</fld> <def>A metallic conducting component of an electrical device connected to a circuit within and so situated that it may form a conducting pathway to an external power source or device when contacted by another conductor; <as>as, the <ex>contact</ex> on a standard light bulb has the shape of a screw for easy insertion into the socket</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>A person who serves to commmunicate information to or from one group to another, whether formally or informally; <as>as, a good Washington reporter has <ex>contacts</ex> in the White House</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Contact level</b></col>, <cd>a delicate level so pivoted as to tilt when two parts of a measuring apparatus come into contact with each other; -- used in precise determinations of lengths and in the accurate graduation of instruments.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>contacting</ent><br/
<hw>contacting</hw> <pos>n.</pos> <def>the act of touching physically.</def><br/
<syn><b>Syn. --</b> contact.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contaction</ent><br/
<hw>Con*tac"tion</hw> <pr>(k<ocr/n*t<acr/k"sh<ucr/n)</pr>, <pos>n.</pos> <def>Act of touching.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>contadino</ent><br/
<hw>contadino</hw> <pos>n.</pos> <def>an Italian farmer.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contagion</ent><br/
<hw>Con*ta"gion</hw> <pr>(k<ocr/n*t<amac/"j<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>contagio</ets>: cf. F. <ets>contagion</ets>. See <er>Contact</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>The transmission of a disease from one person to another, by direct or indirect contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term has been applied by some to the action of miasmata arising from dead animal or vegetable matter, bogs, fens, etc., but in this sense it is now abandoned.</note>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And will he steal out of his wholesome bed<br/
To dare the vile <qex>contagion</qex> of the night?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which serves as a medium or agency to transmit disease; a virus produced by, or exhalation proceeding from, a diseased person, and capable of reproducing the disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act or means of communicating any influence to the mind or heart; <as>as, the <ex>contagion</ex> of enthusiasm</as>.</def> <ldquo/The <xex>contagion</xex> of example.<rdquo/  <rj><au>Eikon Basilike.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When lust . . . <br/
Lets in defilement to the inward parts,<br/
The soul grows clotted by <qex>contagion</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Venom; poison.</def> <mark>[Obs.]</mark> <ldquo/I'll touch my point with this <xex>contagion</xex>.<rdquo/  <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- See <er>Infection</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contagioned</ent><br/
<hw>Con*ta"gioned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected by contagion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contagionist</ent><br/
<hw>Con*ta"gion*ist</hw>, <pos>n.</pos> <def>One who believes in the contagious character of certain diseases, as of yellow fever.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contagious</ent><br/
<hw>Con*ta"gious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contagiosus</ets>: cf. F. <ets>contagieux</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Communicable by contact, by a virus, or by a bodily exhalation; catching; <as>as, a <ex>contagious</ex> disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conveying or generating disease; pestilential; poisonous; <as>as, <ex>contagious</ex> air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Spreading or communicable from one to another; exciting similar emotions or conduct in others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His genius rendered his courage more <qex>contagious</qex>.</q> <rj><qau>Wirt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The spirit of imitation is <qex>contagious</qex>.</q> <rj><qau>Ames.</qau></rj></p>

<p><syn><b>Syn.</b> -- <er>Contagious</er>, <er>Infectious</er>.</syn> <usage> Although often used as synonyms, originally these words were used in very diverse senses; but, in general, a <xex>contagious</xex> disease has been considered as one which is caught from another by some near contact, by the breath, by bodily effluvia, etc.; while an <xex>infectious</xex> disease supposed some entirely different cause acting by a hidden influence, like the miasma of prison ships, of marshes, etc., <xex>infecting</xex> the system with disease.  In either case, a pathogenic microorganism is the direct cause of the disease.  This distinction, though not universally admitted by medical men, as to the literal meaning of the words, certainly applies to them in their figurative use. Thus we speak of the <xex>contagious</xex> influence of evil associates; their <xex>contagion</xex> of bad example, the <xex>contagion</xex> of fear, etc., when we refer to transmission by proximity or contact. On the other hand, we speak of <xex>infection</xex> by bad principles, etc., when we consider anything as diffused by some hidden influence.</usage><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Contagious disease</ent><br/
<hw>Con*ta"gious dis*ease"</hw>. <fld>(Med.)</fld> <def>A disease communicable by contact with a patient suffering from it, or with some secretion of, or object touched by, such a patient. Most such diseases have already been proved to be germ diseases, and their communicability depends on the transmission of the living germs.  Many germ diseases are not contagious, some special method of transmission or inoculation of the germs being required.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contagiously</ent><br/
<hw>Con*ta"gious*ly</hw>, <pos>adv.</pos> <def>In a contagious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contagiousness</ent><br/
<hw>Con*ta"gious*ness</hw>, <pos>n.</pos> <def>Quality of being contagious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contagium</ent><br/
||<hw>Con*ta"gi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Contagion; contagious matter.</def> <ldquo/<xex>Contagium</xex> of measles.<rdquo/  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contain</ent><br/
<hw>Con*tain"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contained</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Containing</conjf>.]</vmorph> <ety>[OE. <ets>contenen</ets>, <ets>conteinen</ets>, F. <ets>contenir</ets>, fr. L. <ets>continere</ets>, <ets>-tentum</ets>; <ets>con-</ets> + <ets>tenere</ets> to hold. See <er>Tenable</er>, and cf. <er>Countenance</er>.]</ety> <sn>1.</sn> <def>To hold within fixed limits; to comprise; to include; to inclose; to hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold, heaven and the heaven of heavens can not <qex>contain</qex> thee; how much less this house!</q> <rj><qau>2 Chron. vi. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When that this body did <qex>contain</qex> a spirit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What thy stores <qex>contain</qex> bring forth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have capacity for; to be able to hold; to hold; to be equivalent to; <as>as, a bushel <ex>contains</ex> four pecks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put constraint upon; to restrain; to confine; to keep within bounds.</def> <mark>[Obs., exept as used reflexively.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The king's person <qex>contains</qex> the unruly people from evil occasions.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fear not, my lord: we can <qex>contain</qex> ourselves.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contain</ent><br/
<hw>Con*tain"</hw>, <pos>v. i.</pos> <def>To restrain desire; to live in continence or chastity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But if they can not <qex>contain</qex>, let them marry.</q> <rj><qau>1 Cor. vii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Containable</ent><br/
<hw>Con*tain"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being contained or comprised.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Containant</ent><br/
<hw>Con*tain"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A container.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Container</ent><br/
<hw>Con*tain"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, contains; particularly, an artifactual object that is designed to contain some fluid or solid material, object or objects, especially for convenience in transporting the contained objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a large metallic box designed to hold many smaller boxes or packages, and used for convenience in loading and unloading large quantities of freight, such as on ships, trains, or airplanes.</def><br/
[<source>PJC</source>]</p>

<p><ent>containerful</ent><br/
<hw>containerful</hw> <pos>n.</pos> <def>the quantity that a container will hold.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>containerized</ent><br/
<hw>containerized</hw> <pos>n.</pos> <def>placed and transported in a container{2}.</def><br/
[<source>PJC</source>]</p>

<p><ent>containership</ent><br/
<ent>container ship</ent><br/
<mhw><hw>container ship</hw>, <hw>containership</hw></mhw> <pos>n.</pos> <def>a ship designed to hold containerized cargoes.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Containment</ent><br/
<hw>Con*tain"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which is contained; the extent; the substance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>containment</qex> of a rich man's estate.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the act of containing.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Diplomacy)</fld> <def>the act or policy of restricting the influence or territorial growth of a hostile nation.</def> <note>The policy of containment is employed when the defeat of a hostile nation or overthrow of its government is considered impractical or too costly.</note><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>the act of restricting some deleterious substance within a confined space, especially when such material is released unintentionally or by accident; <as>as, <ex>containment</ex> of nuclear waste; <ex>containment</ex> of an oil spill</as>.  Also used attributively, as a <ex>containment</ex> boom.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a structure surrounding a nuclear power plant designed to prevent release of radioactive materials into the environment in the event of an accident.</def><br/
[<source>PJC</source>]</p>

<p><ent>Contaminable</ent><br/
<hw>Con*tam"i*na*ble</hw> <pr>(k<ocr/n*t<acr/m"<icr/*n<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being contaminated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contaminate</ent><br/
<hw>Con*tam"i*nate</hw> <pr>(k<ocr/n*t<acr/m"<icr/*n<amac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contaminated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contaminating</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>contaminatus</ets>, p. p. of <ets>contaminare</ets> to bring into contact, to contaminate, fr. <ets>contamen</ets> contagion, for <ets>contagmen</ets>; <ets>con-</ets> + root of <ets>tangere</ets> to touch. See <er>Contact</er>.]</ety> <def>To soil, stain, or corrupt by contact; to tarnish; to sully; to taint; to pollute; to defile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall we now<br/
<qex>Contaminate</qex> our figures with base bribes?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I would neither have simplicity imposed upon, nor virtue <qex>contaminated</qex>.</q> <rj><qau>Goldsmith.</qau></rj></p>

<p><syn><b>Syn.</b> -- To pollute; defile; sully; taint; tarnish; soil; stain; corrupt.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contaminate</ent><br/
<hw>Con*tam"i*nate</hw> <pr>(-n<asl/t)</pr>, <pos>a.</pos> <def>Contaminated; defiled; polluted; tainted.</def> <ldquo/<xex>Contaminate</xex> drink.<rdquo/  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>contaminated</ent><br/
<hw>contaminated</hw> <pos>adj.</pos> <sn>1.</sn> <def>containing undesired or infective microorganisms.</def><br/
<syn><b>Syn. --</b> dirty, infected.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>rendered impure by addition of deleterious substances; <as>as, had to boil the <ex>contaminated</ex> water</as>.</def><br/
<syn><b>Syn. --</b> polluted.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <fr>1</fr> <fld>(Computers)</fld>  <def>damaged by inclusion of incorrect data or information not belonging; <as>as, The file was <ex>contaminated</ex> by a buggy editor</as>.</def><br/
<syn><b>Syn. --</b> corrupted.</syn><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn>  <def>appearing unethical due to a questionable source or motives; -- of money or activities; <as>as, The fund was <ex>contaminated</ex> by a large donation from a mobster</as>.</def><br/
<syn><b>Syn. --</b> tainted.</syn><br/
[<source>PJC</source>]</p>

<p><ent>contaminating</ent><br/
<hw>contaminating</hw> <pos>adj.</pos>  <def>spreading contamination; especially radioactive contamination.</def> <ant>clean</ant><br/
<syn><b>Syn. --</b> dirty.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>causing moral or behavioral degredation.</def> <illu><ex>the contaminating</ex> influence of violent movies</illu><br/
<syn><b>Syn. --</b> corrupting.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Contamination</ent><br/
<hw>Con*tam`i*na"tion</hw> <pr>(k<ocr/n*t<acr/m`<icr/*n<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>contaminatio</ets>.]</ety> <def>The act or process of contaminating; pollution; defilement; taint; also, that which contaminates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contamitive</ent><br/
<hw>Con*tam"i*tive</hw> <pr>(k<ocr/n*t<acr/m"<icr/*n<adot/*t<icr/v)</pr>, <pos>a.</pos> <def>Tending or liable to contaminate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contango</ent><br/
<hw>Con*tan"go</hw> <pr>(k<ocr/n*t<acr/<nsm/"g<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contangoes</plw> <pr>(#)</pr>.</plu> <ety>[Prob. a corruption of <ets>contingent</ets>.]</ety> <sn>1.</sn> <fld>(Stock Exchange)</fld> <def>The premium or interest paid by the buyer to the seller, to be allowed to defer paying for the stock purchased until the next fortnightly settlement day.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The postponement of payment by the buyer of stock on the payment of a premium to the seller. See <er>Backwardation</er>.</def>  <rj><au>N. Biddle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conte</ent><br/
||<hw>Conte</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Contes</plw> <pr>(#)</pr></plu>. <ety>[F.]</ety> <def>A short narrative or tale, esp. one dealing with surprising or marvelous events.</def></p>

<p><q>The <qex>conte</qex> (sic) is a tale something more than a sketch, it may be, and something less than a short story. . . . The <ldquo/Canterbury Tales<rdquo/ are <qex>contes</qex>, most of them, if not all, and so are some of the <ldquo/Tales of a Wayside Inn.<rdquo/</q>  <rj><qau>Brander Matthews.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contection</ent><br/
<hw>Con*tec"tion</hw> <pr>(-t<ecr/k"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>contegere</ets>, <ets>-tectum</ets>, to cover up.]</ety> <def>A covering.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contek</ent><br/
<hw>Con"tek</hw> <pr>(k<ocr/n"t<ecr/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>conteck</ets>, <ets>conteke</ets>, <ets>contake</ets>, perh. a corruption either of <ets>contact</ets> or <ets>contest</ets>.]</ety> <sn>1.</sn> <def>Quarrel; contention; contest.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Contek</qex> with bloody knife.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Contumely; reproach.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemn</ent><br/
<hw>Con*temn"</hw> <pr>(k<ocr/n*t<ecr/m")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contemned</conjf> <pr>(-t<ecr/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contemning</conjf> <pr>(-t<ecr/m"n<icr/ng <it>or</it> -t<ecr/m"<icr/ng)</pr>.]</vmorph> <ety>[L. <ets>contemnere</ets>, <ets>-temptum</ets>; <ets>con-</ets> + <ets>temnere</ets> to slight, despise: cf. OF. <ets>contemner</ets>.]</ety> <def>To view or treat with contempt, as mean and despicable; to reject with disdain; to despise; to scorn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy pompous delicacies I <qex>contemn</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One who <qex>contemned</qex> divine and human laws.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- To despise; scorn; disdain; spurn; slight; neglect; underrate; overlook.</syn> <usage> -- To <er>Contemn</er>, <er>Despise</er>, <er>Scorn</er>, <er>Disdain</er>. <xex>Contemn</xex> is the generic term, and is applied especially to objects, qualities, etc., which are deemed contemptible, and but rarely to individuals; to <xex>despise</xex> is to regard or treat as mean, unbecoming, or worthless; to <xex>scorn</xex> is stronger, expressing a quick, indignant contempt; <xex>disdain</xex> is still stronger, denoting either unwarrantable pride and haughtiness or an abhorrence of what is base.</usage><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 312 --></p>

<p><ent>Contemner</ent><br/
<hw>Con*tem"ner</hw> <pr>(k<ocr/n*t<ecr/m"n<etil/r <it>or</it> -t<ecr/m"<etil/r)</pr>, <pos>n.</pos>  <def>One who contemns; a despiser; a scorner.</def> <ldquo/<xex>Contemners</xex> of the gods.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemningly</ent><br/
<hw>Con*tem"ning*ly</hw>, <pos>adv.</pos> <def>Contemptuously.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemper</ent><br/
<hw>Con*tem"per</hw> <pr>(k<ocr/n*t<ecr/m"p<etil/r)</pr>, <pos>v. t.</pos> <ety>[L. <ets>contemperare</ets>, <ets>-temperatum</ets>; <ets>con-</ets> + <ets>temperare</ets> to temper. Cf. <er>Contemperate</er>.]</ety> <def>To modify or temper; to allay; to qualify; to moderate; to soften.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The antidotes . . . have allayed its bitterness and <qex>contempered</qex> its malignancy.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemperate</ent><br/
<hw>Con*tem"per*ate</hw> <pr>(k<ocr/n*t<ecr/m"p<etil/r*<amac/t)</pr>, <pos>v. t.</pos> <ety>[See <er>Contemper</er>.]</ety> <def>To temper; to moderate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Moisten and <qex>contemperate</qex> the air.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemperation</ent><br/
<hw>Con*tem`per*a"tion</hw> <pr>(k<ocr/n*t<ecr/m`p<etil/r*<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of tempering or moderating.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proportionate mixture or combination.</def> <ldquo/<xex>Contemperation</xex> of light and shade.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemperature</ent><br/
<hw>Con*tem"per*a*ture</hw> <pr>(k<ocr/n*t<ecr/m"p<etil/r*<adot/*t<usl/r; 135)</pr>, <pos>n.</pos>  <def>The condition of being tempered; proportionate mixture; temperature.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The different <qex>contemperature</qex> of the elements.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplance</ent><br/
<hw>Con*tem"plance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Contemplation.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplant</ent><br/
<hw>Con*tem"plant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contemplans</ets>, p. pr.]</ety> <def>Given to contemplation; meditative.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplate</ent><br/
<hw>Con"tem*plate</hw> <pr>(?; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contemplated</conjf> <pr>(# <or/ #)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contemplating</conjf>.]</vmorph> <ety>[L. <ets>contemplatus</ets>, p. p. of <ets>contemplari</ets> to contemplate; <ets>con-</ets> + <ets>templum</ets> a space for observation marked out by the augur. See <er>Temple</er>.]</ety> <sn>1.</sn> <def>To look at on all sides or in all its bearings; to view or consider with continued attention; to regard with deliberate care; to meditate on; to study.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To love, at least <qex>contemplate</qex> and admire,<br/
What I see excellent.<br/
<rj><qau>Milton.</qau></rj></q><br/
[<source>1913 Webster</source>]</p>

<p><q>We thus dilate<br/
Our spirits to the size of that they <qex>contemplate</qex>.</q><br/ <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To consider or have in view, as contingent or probable; to look forward to; to purpose; to intend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There remain some particulars to complete the information <qex>contemplated</qex> by those resolutions.</q> <rj><qau>A. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If a treaty contains any stipulations which <qex>contemplate</qex> a state of future war.</q> <rj><qau>Kent.</qau></rj></p>

<p><syn><b>Syn.</b> -- To view; behold; study; ponder; muse; meditate on; reflect on; consider; intend; design; plan; propose; purpose. See <er>Meditate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplate</ent><br/
<hw>Con"tem*plate</hw>, <pos>v. i.</pos> <def>To consider or think studiously; to ponder; to reflect; to muse; to meditate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So many hours must I <qex>contemplate</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplation</ent><br/
<hw>Con`tem*pla"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>contemplation</ets>, L. <ets>contemplatio</ets>.]</ety> <sn>1.</sn> <def>The act of the mind in considering with attention; continued attention of the mind to a particular subject; meditation; musing; study.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>contemplation</qex> of created things,<br/
By steps we may ascend to God.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Contemplation</qex> is keeping the idea which is brought into the mind for some time actually in view.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Holy meditation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To live in prayer and <qex>contemplation</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of looking forward to an event as about to happen; expectation; the act of intending or purposing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>contemplation</qex> of returning at an early date, he left.</q> <rj><qau>Reid.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To have in contemplation</b></col>, <cd>to inted or purpose, or to have under consideration.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplatist</ent><br/
<hw>Con*tem"pla*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A contemplator.</def> <mark>[R.]</mark>  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplative</ent><br/
<hw>Con*tem"pla*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>contemplatif</ets>, L. <ets>contemplativus</ets>.]</ety> <sn>1.</sn> <def>Pertaining to contemplation; addicted to, or employed in, contemplation; meditative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fixed and <qex>contemplative</qex> their looks.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the power of contemplation; <as>as, <ex>contemplative</ex> faculties</as>.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplative</ent><br/
<hw>Con*tem"pla*tive</hw>, <pos>n.</pos> <fld>(R. C. Ch.)</fld> <def>A religious or either sex devoted to prayer and meditation, rather than to active works of charity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplatively</ent><br/
<hw>Con*tem"pla*tive*ly</hw>, <pos>adv.</pos> <def>With contemplation; in a contemplative manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplativeness</ent><br/
<hw>Con*tem"pla*tive*ness</hw>, <pos>n.</pos> <def>The state of being contemplative; thoughtfulness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemplator</ent><br/
<hw>Con"tem*pla`tor</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who contemplates.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemporaneity</ent><br/
<hw>Con*tem`po*ra*ne"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being contemporaneous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lines of <qex>contemporaneity</qex> in the oolitic system.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemporaneous</ent><br/
<hw>Con*tem`po*ra"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contemporaneus</ets>; <ets>con-</ets> + <ets>tempus</ets> time. See <er>Temporal</er>, and cf. <er>Contemporaneous</er>.]</ety> <def>Living, existing, or occurring at the same time; contemporary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great age of Jewish philosophy, that of Aben Esra, Maimonides, and Kimchi, had been <qex>contemporaneous</qex> with the later Spanish school of Arabic philosophy.</q> <rj><qau>Milman</qau></rj></p>

<p>-- <wordforms><wf>Con*tem`po*ra"ne*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemporaneously</ent><br/
<hw>Con*tem`po*ra"ne*ous*ly</hw>, <pos>adv.</pos> <def>At the same time with some other event.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>contemporaries</ent><br/
<hw>contemporaries</hw> <pos>n.</pos> <def>all the people living at the same time or of approximately the same age.</def><br/
<syn><b>Syn. --</b> coevals, generation.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contemporariness</ent><br/
<hw>Con*tem"po*ra*ri*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Existence at the same time; contemporaneousness.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemporary</ent><br/
<hw>Con*tem"po*ra*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>con-</ets> + L. <ets>temporarius</ets> of belonging to time, <ets>tempus</ets> time. See <er>Temporal</er>, and cf. <er>Contemporaneous</er>.]</ety> <sn>1.</sn> <def>Living, occuring, or existing, at the same time; done in, or belonging to, the same times; contemporaneous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This king [Henry VIII.] was <qex>contemporary</qex> with the greatest monarchs of Europe.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of the same age; coeval.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A grove born with himself he sees,<br/
And loves his old <qex>contemporary</qex> trees.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemporary</ent><br/
<hw>Con*tem"po*ra*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contemporaries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>One who lives at the same time with another; <as>as, Petrarch and Chaucer were <ex>contemporaries</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a person of nearly the same age as another.</def><br/
<syn><b>Syn. --</b> coeval.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>contemporize</ent><br/
<hw>contemporize</hw> <pos>v. i.</pos> <def>to happen at the same time.</def><br/
<syn><b>Syn. --</b> co-occur.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>contemporize</ent><br/
<hw>contemporize</hw> <pos>v. t.</pos> <def>to arrange or represent events so that they co-occur.</def><br/
<syn><b>Syn. --</b> synchronize.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contempt</ent><br/
<hw>Con*tempt"</hw> <pr>(k<ocr/n*t<ecr/mt"; 215)</pr>, <pos>n.</pos> <ety>[L. <ets>contemptus</ets>, fr. <ets>contemnere</ets>: cf. OF. <ets>contempt</ets>. See <er>Contemn</er>.]</ety> <sn>1.</sn> <def>The act of contemning or despising; the feeling with which one regards that which is esteemed mean, vile, or worthless; disdain; scorn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Criminal <qex>contempt</qex> of public feeling.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing, says Longinus, can be great, the <qex>contempt</qex> of which is great.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being despised; disgrace; shame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Contempt</qex> and begarry hangs upon thy back.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An act or expression denoting contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Little insults and <qex>contempts</qex>.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>contempt</qex> and anger of his lip.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Disobedience of the rules, orders, or process of a court of justice, or of rules or orders of a legislative body; disorderly, contemptuous, or insolent language or behavior in presence of a court, tending to disturb its proceedings, or impair the respect due to its authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Contempt</xex> is in some jurisdictions extended so as to include publications reflecting injuriously on a court of justice, or commenting unfairly on pending proceedings; in other jurisdictions the courts are prohibited by statute or by the constitution from thus exercising this process.</note></p>

<p><syn><b>Syn.</b> -- Disdain; scorn; derision; mockery; contumely; neglect; disregard; slight.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptibility</ent><br/
<hw>Con*tempt`i*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being contemptible; contemptibleness.</def>  <rj><au>Speed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptible</ent><br/
<hw>Con*tempt"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Worthy of contempt; deserving of scorn or disdain; mean; vile; despicable.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The arguments of tyranny are as<qex>contemptible</qex> as its force is dreadful.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Despised; scorned; neglected; abject.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Insolent; scornful; contemptuous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If she should make tender of her love, 't is very possible he 'll scorn it; for the man . . . hath a <qex>contemptible</qex> spirit.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Despicable; abject; vile; mean; base; paltry; worthless; sorry; pitiful; scurrile.</syn> See <er>Contemptuous</er>. -- <usage><er>Contemptible</er>, <er>Despicable</er>, <er>Pitiful</er>, <er>Paltry</er>. <xex>Despicable</xex> is stronger than <xex>contemptible</xex>, as <xex>despise</xex> is stronger than <xex>contemn</xex>. It implies keen disapprobation, with a mixture of anger. A man is <xex>despicable</xex> chiefly for low actions which mark his life, such as servility, baseness, or mean adulation. A man is <xex>contemptible</xex> for mean qualities which distinguish his character, especially those which show him to be weak, foolish, or worthless. Treachery is <xex>despicable</xex>, egotism is <xex>contemptible</xex>. <xex>Pitiful</xex> and <xex>paltry</xex> are applied to cases which are beneath anger, and are simply <xex>contemptible</xex> in a high degree.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptibleness</ent><br/
<hw>Con*tempt"i*ble*ness</hw>, <pos>n.</pos> <def>The state or quality of being contemptible, or of being despised.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptibly</ent><br/
<hw>Con*tempt"i*bly</hw>, <pos>adv.</pos> <def>In a contemptible manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptuous</ent><br/
<hw>Con*temp"tu*ous</hw> <pr>(?; 135)</pr>, <pos>a.</pos>  <def>Manifesting or expressing contempt or disdain; scornful; haughty; insolent; disdainful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A proud, <qex>contemptuous</qex> behavior.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Savage invective and <qex>contemptuous</qex> sarcasm.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rome . . . entertained the most <qex>contemptuous</qex> opinion of the Jews.</q> <rj><qau>Atterbury.</qau></rj></p>

<p><syn><b>Syn.</b> -- Scornful; insolent; haughty; disdainful; supercilious; insulting; contumelious.</syn> <usage> -- <er>Contemptuous</er>, <er>Contemptible</er>. These words, from their similarity of sound, are sometimes erroneously interchanged, as when a person speaks of having <ldquo/a very <xex>contemptible</xex> opinion of another.<rdquo/ <xex>Contemptible</xex> is applied to that which is the object of contempt; as, <xex>contemptible</xex> conduct; a<xex>contemptible</xex> fellow. <xex>Contemptuous</xex> is applied to that which indicates contempt; as, a <xex>contemptuous</xex> look; a <xex>contemptuous</xex> remark; <xex>contemptuous</xex> treatment. A person, or whatever is personal, as an action, an expression, a feeling, an opinion, may be either <xex>contemptuous</xex> or <xex>contemptible</xex>; a thing may be <xex>contemptible</xex>, but can not be <xex>contemptuous</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptuously</ent><br/
<hw>Con*temp"tu*ous*ly</hw>, <pos>adv.</pos> <def>In a contemptuous manner; with scorn or disdain; despitefully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The apostles and most eminent Christians were poor, and used <qex>contemptuously</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contemptuousness</ent><br/
<hw>Con*temp"tu*ous*ness</hw>, <pos>n.</pos> <def>Disposition to or manifestion of contempt; insolence; haughtiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contend</ent><br/
<hw>Con*tend"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Contending</conjf>.]</vmorph> <ety>[OF. <ets>contendre</ets>, L. <ets>contendere</ets>, <ets>-tentum</ets>; <ets>con-</ets> + <ets>tendere</ets> to strech. See <er>Tend</er>.]</ety> <sn>1.</sn> <def>To strive in opposition; to contest; to dispute; to vie; to quarrel; to fight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For never two such kingdoms did <qex>contend</qex><br/
Without much fall of blood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord said unto me, Distress not the Moabites, neither <qex>contend</qex> with them in battle.</q> <rj><qau>Deut. ii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In ambitious strength I did<br/
<qex>Contend</qex> against thy valor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To struggle or exert one's self to obtain or retain possession of, or to defend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You sit above, and see vain men below<br/
<qex>Contend</qex> for what you only can bestow.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strive in debate; to engage in discussion; to dispute; to argue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The question which our author would <qex>contend</qex> for.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many things he fiercely <qex>contended</qex> about were trivial.</q> <rj><qau>Dr. H. More.</qau></rj></p>

<p><syn><b>Syn.</b> -- To struggle; fight; combat; vie; strive; oppose; emulate; contest; litigate; dispute; debate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contend</ent><br/
<hw>Con*tend"</hw>, <pos>v. t.</pos> <def>To struggle for; to contest.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Carthage shall <qex>contend</qex> the world with Rome.Dryden.</q><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contendent</ent><br/
<hw>Con*tend"ent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contendens</ets>, p. pr.]</ety> <def>An antagonist; a contestant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In all notable changes and revolutions the <qex>contendents</qex> have been still made a prey to the third party.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contender</ent><br/
<hw>Con*tend"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who contends; a contestant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contendress</ent><br/
<hw>Con*tend"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female contestant.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contenement</ent><br/
<hw>Con*ten"e*ment</hw> <pr>(k<ocr/n*t<ecr/n"<esl/*m<eit/nt)</pr>, <pos>n.</pos> <ety>[Pref. <ets>con-</ets> + <ets>tenement</ets>.]</ety> <fld>(Law)</fld> <def>That which is held together with another thing; that which is connected with a tenement, or thing holden, as a certain quantity of land adjacent to a dwelling, and necessary to the reputable enjoyment of the dwelling; appurtenance.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Content</ent><br/
<hw>Con*tent"</hw> <pr>(k<ocr/n*t<ecr/nt")</pr>, <pos>a.</pos> <ety>[F. <ets>content</ets>, fr. L. <ets>contentus</ets>, p. p. of <ets>contenire</ets> to hold together, restrain. See <er>Contain</er>.]</ety> <def>Contained within limits; hence, having the desires limited by that which one has; not disposed to repine or grumble; satisfied; contented; at rest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Having food and rai
ment, let us be therewith <qex>content</qex>.</q> <rj><qau>1 Tim. vi. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Content</ent><br/
<hw>Con"tent</hw> <pr>(k<ocr/n"t<ecr/nt <it>or</it> k<ocr/n*t<ecr/nt"; 277)</pr>, <pos>n.</pos>; <xex>usually in pl</xex>., <plw>Contents</plw>. <sn>1.</sn> <def>That which is contained; the thing or things held by a receptacle or included within specified limits; <as>as, the <ex>contents</ex> of a cask or bale or of a room; the <ex>contents</ex> of a book</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall prove these writings . . . authentic, and the <qex>contents</qex> true, and worthy of a divine original.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Power of containing; capacity; extent; size.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Strong ship's, of great <qex>content</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>Area or quantity of space or matter contained within certain limits; <as>as, solid <ex>contents</ex>; superficial <ex>contents</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The geometrical <qex>content</qex>, figure, and situation of all the lands of a kingdom.</q> <rj><qau>Graunt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Table of contents</b></col>, <it>or</it> <col><b>Contents</b></col></mcol>, <cd>a table or list of topics in a book, showing their order and the place where they may be found: a summary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Content</ent><br/
<hw>Con*tent"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>contenter</ets>, LL. <ets>contentare</ets>, fr. L. <ets>contentus</ets>, p. p. See <er>Content</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To satisfy the desires of; to make easy in any situation; to appease or quiet; to gratify; to please.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do not <qex>content</qex> yourselves with obscure and confused ideas, where clearer are to be attained.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pilate, willing to <qex>content</qex> the people, released Barabbas unto them.</q> <rj><qau>Mark xv. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To satisfy the expectations of; to pay; to requite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come the next Sabbath, and I will <qex>content</qex> you.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To satisfy; appease; please. See <er>Satiate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Content</ent><br/
<hw>Con*tent"</hw>, <pos>n.</pos> <sn>1.</sn> <def>Rest or quietness of the mind in one's present condition; freedom from discontent; satisfaction; contentment; moderate happiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such is the fullness of my heart's <qex>content</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Acquiescence without examination.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sense they humbly take upon <qex>content</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which contents or satisfies; that which if attained would make one happy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So will I in England work your grace's full <qex>content</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eng. House of Lords)</fld> <def>An expression of assent to a bill or motion; an affirmative vote; also, a member who votes <ldquo/Content.<rdquo/.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Supposing the number of <ldquo/<qex>Contents</qex><rdquo/ and <ldquo/Not contents<rdquo/ strictly equal in number and consequence.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contentation</ent><br/
<hw>Con`ten*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>contentatio</ets>.]</ety> <def>Content; satisfaction.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contented</ent><br/
<hw>Con*tent"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Content; easy in mind; satisfied; quiet; willing.</def> -- <wordforms><wf>Con*tent"ed*ly</wf>, <pos>adv.</pos> -- <wf>Con*tent"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>contentedness</ent><br/
<hw>con*tent"ed*ness</hw> <pos>n.</pos> <def>acceptance of one's situation in life.</def><br/
<syn><b>Syn. --</b> content.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contentful</ent><br/
<hw>Con*tent"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of content.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contention</ent><br/
<hw>Con*ten"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>contention</ets>, L. <ets>contentio</ets>. See <er>Contend</er>.]</ety> <sn>1.</sn> <def>A violent effort or struggle to obtain, or to resist, something; contest; strife.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would my arms could match thee in <qex>contention</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Strife in words; controversy; altercation; quarrel; dispute; <as>as, a bone of <ex>contention</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Contentions</qex> and strivings about the law.</q> <rj><qau>Titus iii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vehemence of endeavor; eagerness; ardor; zeal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An end . . . worthy our utmost <qex>contention</qex> to obtain.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A point maintained in an argument, or a line of argument taken in its support; the subject matter of discussion or strife; a position taken or contended for.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All men seem agreed what is to be done; the <qex>contention</qex> is how the subject is to be divided and defined.</q> <rj><qau>Bagehot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This was my original <qex>contention</qex>, and I still maintain that you should abide by your former decision.</q> <rj><qau>Jowett.</qau></rj></p>

<p><syn><b>Syn.</b> -- Struggle; strife; contest; quarrel; combat; conflict; feud; litigation; controversy; dissension; variance; disagreement; debate; competition; emulation.</syn> <usage> -- <er>Contention</er>, <er>Strife</er>. A struggle between two parties is the idea common to these two words. <xex>Strife</xex> is a struggle for mastery; <xex>contention</xex> is a struggle for the possession of some desired object, or the accomplishment of some favorite end. Neither of the words is necessarily used in a bad sense, since there may be a generous <xex>strife</xex> or <xex>contention</xex> between two friends as to which shall incur danger or submit to sacrifices. Ordinarily, however, these words denote a struggle arising from bad passions. In that case, <xex>strife</xex> usually springs from a quarrelsome temper, and <xex>contention</xex> from, a selfish spirit which seeks its own aggrandizement, or is fearful lest others should obtain too much. <xex>Strife</xex> has more reference to the manner than to the object of a struggle, while <xex>contention</xex> takes more account of the end to be gained.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contentious</ent><br/
<hw>Con*ten"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contentiosus</ets>: cf. F. <ets>contentieux</ets>.]</ety> <sn>1.</sn> <def>Fond of contention; given to angry debate; provoking dispute or contention; quarrelsome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Despotic and <qex>contentious</qex> temper.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to contention or strife; involving or characterized by contention.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>More cheerful, though not less <qex>contentious</qex>, regions.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Contested; litigated; litigious; having power to decide controversy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contentious jurisdiction</b></col> <fld>(Eng. Eccl. Law)</fld>, <cd>jurisdiction over matters in controversy between parties, in contradistinction to <contr>voluntary jurisdiction</contr>, or that exercised upon matters not opposed or controverted.</cd></cs></p>

<p><syn><b>Syn.</b> -- Quarrelsome; pugnacious; dissentious; wrangling; litigious; perverse; peevish.</syn></p>

<p>-- <wordforms><wf>Con*ten"tious*ly</wf>, <pos>adv.</pos> -- <wf>Con*ten"tious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contentless</ent><br/
<hw>Con*tent"less</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Content</ets> + <ets>-less</ets>.]</ety> <def>Discontented; dissatisfied.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contently</ent><br/
<hw>Con*tent"ly</hw>, <pos>adv.</pos> <def>In a contented manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 313 --></p>

<p><ent>Contentment</ent><br/
<hw>Con*tent"ment</hw> <pr>(k<ocr/n*t<ecr/nt"m<eit/nt)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>contentement</ets>. See <er>Content</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The state of being contented or satisfied; content.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Contentment</qex> without external honor is humility.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Godliness with <qex>contentment</qex> is great gain.</q> <rj><qau>1 Tim. vi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of contenting or satisfying; <as>as, the <ex>contentment</ex> of avarice is impossible</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Gratification; pleasure; satisfaction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>At Paris the prince spent one whole day to give his mind some <qex>contentment</qex> in viewing of a famous city.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contents</ent><br/
<hw>Con*tents</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n. pl.</pos>  <def>See <er>Content</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conterminable</ent><br/
<hw>Con*ter"mi*na*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the same bounds; terminating at the same time or place; conterminous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love and life not <qex>conterminable</qex>.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conterminal</ent><br/
<hw>Con*ter"mi*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>conterminalis</ets>.]</ety> <def>Conterminous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conterminant</ent><br/
<hw>Con*ter"mi*nant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the same limits; ending at the same time; conterminous.</def>  <rj><au>Lamb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conterminate</ent><br/
<hw>Con*ter"mi*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conterminare</ets> to border upon, fr. <ets>conterminus</ets> conterminous; <ets>con-</ets> + <ets>terminus</ets> border.]</ety> <def>Having the same bounds; conterminous.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conterminous</ent><br/
<hw>Con*ter"mi*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conterminus</ets>.  Cf. <er>Conterminous</er>.]</ety> <def>Having the same bounds, or limits; bordering upon; contiguous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This conformed so many of them as were <qex>conterminous</qex> to the colonies and garrisons, to the Roman laws.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conterraneous</ent><br/
<ent>Conterranean</ent><br/
<mhw>{ <hw>Con`ter*ra"ne*an</hw> <pr>(?)</pr>, <hw>Con`ter*ra"ne*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>conterraneus</ets>; <ets>con-</ets> + <ets>terra</ets> country.]</ety> <def>Of or belonging to the same country.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contesseration</ent><br/
<hw>Con*tes`ser*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contesseratio</ets>, from <ets>contesserare</ets> to contract friendship by means of the <ets>tesserae</ets> (friendship tokens).]</ety> <def>An assemblage; a collection; harmonious union.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That person of his [George Herbert], which afforded so unusual a <qex>contesseration</qex> of elegancies.</q> <rj><qau>Oley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contest</ent><br/
<hw>Con*test"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contested</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Contesting</conjf>.]</vmorph> <ety>[F. <ets>contester</ets>, fr. L. <ets>contestari</ets> to call to witness, <ets>contestari litem</ets> to introduce a lawsuit by calling witnesses, to bring an action; <ets>con-</ets> + <ets>testari</ets> to be a witness, <ets>testic</ets> witness. See <er>Testify</er>.]</ety> <sn>1.</sn> <def>To make a subject of dispute, contention, litigation, or emulation; to contend for; to call in question; to controvert; to oppose; to dispute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The people . . . <qex>contested</qex> not what was done.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Few philosophical aphorisms have been more frequenty repeated, few more <qex>contested</qex> than this.</q> <rj><qau>J. D. Morell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strive earnestly to hold or maintain; to struggle to defend; <as>as, the troops <ex>contested</ex> every inch of ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>To make a subject of litigation; to defend, as a suit; to dispute or resist; as a claim, by course of law; to controvert.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To contest an election</b></col>. <fld>(Polit.)</fld> <sd>(a)</sd> <cd>To strive to be elected.</cd> <sd>(b)</sd> <cd>To dispute the declared result of an election.</cd></cs></p>

<p><syn><b>Syn.</b> -- To dispute; controvert; debate; litigate; oppose; argue; contend.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contest</ent><br/
<hw>Con*test"</hw>, <pos>v. i.</pos> <def>To engage in contention, or emulation; to contend; to strive; to vie; to emulate; -- followed usually by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The difficulty of an argument adds to the pleasure of <qex>contesting</qex> with it, when there are hopes of victory.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of man, who dares in pomp with Jove <qex>contest</qex>?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contest</ent><br/
<hw>Con"test</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Earnest dispute; strife in argument; controversy; debate; altercation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Leave all noisy <qex>contests</qex>, all immodest clamors and brawling language.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Earnest struggle for superiority, victory, defense, etc.; competition; emulation; strife in arms; conflict; combat; encounter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The late battle had, in effect, been a <qex>contest</qex> between one usurper and another.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was fully expected that the <qex>contest</qex> there would be long and fierce.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- Conflict; combat; battle; encounter; shock; struggle; dispute; altercation; debate; controvesy; difference; disagreement; strife.</syn> <usage> -- <er>Contest</er>, <er>Conflict</er>, <er>Combat</er>, <er>Encounter</er>. <xex>Contest</xex> is the broadest term, and had originally no reference to actual fighting. It was, on the contrary, a legal term signifying to <xex>call witnesses</xex>, and hence came to denote first a struggle in argument, and then a struggle for some common object between opposing parties, usually one of considerable duration, and implying successive stages or acts. <xex>Conflict</xex> denotes literally a close personal engagement, in which sense it is applied to actual fighting. It is, however, more commonly used in a figurative sense to denote strenuous or direct opposition; as, a mental <xex>conflict</xex>; <xex>conflicting</xex> interests or passions; a <xex>conflict</xex> of laws. An <xex>encounter</xex> is a direct meeting face to face. Usually it is a hostile meeting, and is then very nearly coincident with <xex>conflict</xex>; as, an <xex>encounter</xex> of opposing hosts. Sometimes it is used in a looser sense; as, <ldquo/this keen <xex>encounter</xex> of our wits.<rdquo/ <au>Shak.</au> <xex>Combat</xex> is commonly applied to actual fighting, but may be used figuratively in reference to a strife or words or a struggle of feeling.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contestable</ent><br/
<hw>Con*test"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>contestable</ets>.]</ety> <def>Capable of being contested; debatable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contestant</ent><br/
<hw>Con*test"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>contestant</ets>.]</ety> <def>One who contests; an opponent; a litigant; a disputant; one who claims that which has been awarded to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contestation</ent><br/
<hw>Con`tes*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contestatio</ets> testimony: cf. F. <ets>contestation</ets> a contesting.]</ety> <sn>1.</sn> <def>The act of contesting; emulation; rivalry; strife; dispute.</def> <ldquo/Loverlike <xex>contestation</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>After years spent in domestic, unsociable <qex>contestations</qex>, she found means to withdraw.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proof by witness; attestation; testimony.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A solemn <qex>contestation</qex> ratified on the part of God.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contestingly</ent><br/
<hw>Con*test"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a contending manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contex</ent><br/
<hw>Con*tex</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To context.</def> <mark>[Obs.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Context</ent><br/
<hw>Con*text"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contextus</ets>, p. p. of <ets>contexere</ets> to weave, to unite; <ets>con-</ets> + <ets>texere</ets> to weave. See <er>Text</er>.]</ety> <def>Knit or woven together; close; firm.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The coats, without, are <qex>context</qex> and callous.</q> <rj><qau>Derham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Context</ent><br/
<hw>Con"text</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contextus</ets>; cf. F. <ets>contexte</ets> .]</ety> <def>The part or parts of something written or printed, as of Scripture, which precede or follow a text or quoted sentence, or are so intimately associated with it as to throw light upon its meaning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>According to all the light that the <qex>contexts</qex> afford.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Context</ent><br/
<hw>Con*text"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To knit or bind together; to unite closely.</def> <mark>[Obs.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole world's frame, which is <qex>contexted</qex> only by commerce and contracts.</q> <rj><qau>R. Junius.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contextural</ent><br/
<hw>Con*tex"tur*al</hw> <pr>(?; 135)</pr>, <pos>a.</pos>  <def>Pertaining to contexture or arrangement of parts; producing contexture; interwoven.</def>  <rj><au>Dr. John Smith (1666).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contexture</ent><br/
<hw>Con*tex"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>contexture</ets>.]</ety> <def>The arrangement and union of the constituent parts of a thing; a weaving together of parts; structural character of a thing; system; constitution; texture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That wonderful <qex>contexture</qex> of all created beings.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was not of any delicate <qex>contexture</qex>; his limbs rather sturdy than dainty.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contextured</ent><br/
<hw>Con*tex"tured</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <def>Formed into texture; woven together; arranged; composed.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conticent</ent><br/
<hw>Con"ti*cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conticens</ets>, p. pr. of <ets>conticere</ets>; <ets>con-</ets> + <ets>tacere</ets> to be silent.]</ety> <def>Silent.</def> <mark>[R.]</mark> <ldquo/The guests sit <xex>conticent</xex>.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contignation</ent><br/
<hw>Con`tig*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contignatio</ets>, fr. <ets>contignare</ets> to join with beams; <ets>con-</ets> + <ets>tignum</ets> beam.]</ety> <sn>1.</sn> <def>The act or process of framing together, or uniting, as beams in a fabric.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A framework or fabric, as of beams.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contiguate</ent><br/
<hw>Con*tig"u*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>contiguatus</ets>.]</ety> <def>Contiguous; touching.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contiguity</ent><br/
<hw>Con`ti*gu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>contiguit<eacute/</ets>, LL. <ets>contiguitas</ets>.]</ety> <def>The state of being contiguous; intimate association; nearness; proximity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The convicinity and <qex>contiguity</qex> of the two parishes.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contiguous</ent><br/
<hw>Con*tig"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contiguus</ets>; akin to <ets>contigere</ets> to touch on all sides. See <er>Contingent</er>.]</ety> <def>In actual contact; touching; also, adjacent; near; neighboring; adjoining.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The two halves of the paper did not appear fully divided . . . but seemed <qex>contiguous</qex> at one of their angles.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sees no <qex>contiguous</qex> palace rear its head.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contiguous angles</b></col>. <cd>See <cref>Adjacent angles</cref>, under <er>Angle</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Adjoining; adjacent. See <er>Adjacent</er>.</syn></p>

<p>-- <wordforms><wf>Con*tig"u*ous*ly</wf>, <pos>adv.</pos> -- <wf>Con*tig"u*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continency</ent><br/
<ent>Continence</ent><br/
<mhw>{ <hw>Con"ti*nence</hw> <pr>(?)</pr>, <hw>Con"ti*nen*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[F. <ets>continence</ets>, L. <ets>continentia</ets>. See <er>Continent</er>, and cf. <er>Countenance</er>.]</ety> <sn>1.</sn> <def>Self-restraint; self-command.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He knew what to say; he knew also, when to leave off, -- a <qex>continence</qex> which is practiced by few writers.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The restraint which a person imposes upon his desires and passions; the act or power of refraining from indulgence of the sexual appetite, esp. from unlawful indulgence; sometimes, moderation in sexual indulgence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If they [the unmarried and widows] have not <qex>continency</qex>, let them marry.</q> <rj><qau>1 Cor. vii. 9 (Rev. Ver. ).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Chastity is either abstinence or <qex>continence</qex>: abstinence is that of virgins or widows; <qex>continence</qex>, that of married persons.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Uninterrupted course; continuity.</def> <mark>[Obs.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continent</ent><br/
<hw>Con"ti*nent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>continens</ets>, <ets>-entis</ets>, prop., p. pr. of <ets>continere</ets> to hold together, to repress: cf. F. <ets>continent</ets>. See <er>Contain</er>.]</ety> <sn>1.</sn> <def>Serving to restrain or limit; restraining; opposing.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exercising restraint as to the indulgence of desires or passions; temperate; moderate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Have a <qex>continent</qex> forbearance till the speed of his rage goes slower.</q>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Abstaining from sexual intercourse; exercising restraint upon the sexual appetite; esp., abstaining from illicit sexual intercourse; chaste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My past life</q><br/
[<source>1913 Webster</source>]</p>

<p><q>Hath been as <qex>continent</qex>, as chaste, as true,</q><br/
[<source>1913 Webster</source>]</p>

<p><q>As I am now unhappy.</q>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not interrupted; connected; continuous; <as>as, a <ex>continent</ex> fever</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The northeast part of Asia is, if not <qex>continent</qex> with the west side of America, yet certainly it is the least disoined by sea of all that coast.</q> <rj><qau>Berrewood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continent</ent><br/
<hw>Con"ti*nent</hw>, <pos>n.</pos> <ety>[L. <ets>continens</ets>, prop., a holding together: cf. F. <ets>continent</ets>. See <er>Continent</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>That which contains anything; a receptacle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The smaller <qex>continent</qex> which we call a pipkin.</q> <rj><qau>Bp. Kennet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the grand divisions of land on the globe; the main land; specifically <fld>(Phys. Geog.)</fld>, a large body of land differing from an island, not merely in its size, but in its structure, which is that of a large basin bordered by mountain chains; <as>as, the <ex>continent</ex> of North America</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>continents</xex> are now usually regarded as six in number: North America, South America, Europe, Asia, Africa, and Australia. But other large bodies of land are also reffered to as continents; as, the Antarctic <xex>continent</xex>; the <xex>continent</xex> of Greenland. Europe, Asia, and Africa are often grouped together as the <xex>Eastern Continent</xex>, and North and South America as the <xex>Western Continent</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Continent</b></col>, <cd>the main land of Europe, as distinguished from the islands, especially from England.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continental</ent><br/
<hw>Con`ti*nen"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a continent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the main land of Europe, in distinction from the adjacent islands, especially England; <as>as, a <ex>continental</ex> tour; a <ex>continental</ex> coalition</as>.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No former king had involved himself so frequently in the labyrinth of <qex>continental</qex> alliances.</q>  <rj><au>Hallam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Amer. Hist.)</fld> <def>Of or pertaining to the confederated colonies collectively, in the time of the Revolutionary War; <as>as, <ex>Continental</ex> money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The army before Boston was designated as the <qex>Continental</qex> army, in contradistinction to that under General Gage, which was called the <ldquo/Ministerial army.<rdquo/</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Continental Congress</b></col>. <cd>See under <er>Congress</er>.</cd> -- <col><b>Continental system</b></col> <fld>(Hist.)</fld>, <cd>the blockade of Great Britain ordered by Napoleon by the decree of Berlin, Nov. 21, 1806; the object being to strike a blow at the maritime and commercial supremacy of Great Britain, by cutting her off from all intercourse with the continent of Europe.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continental</ent><br/
<hw>Con`ti*nen"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Amer. Hist.)</fld> <def>A soldier in the Continental army. See <er>Continental</er>, <pos>a.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Amer. Hist.)</fld> <def>a piece of the Continental currency, paper money issued under authority of the Continental Congress. See <er>Continental</er>, <pos>a.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>"Not worth a <ex>continental</ex>." was said of Continental currency after the American revolution, when it was considered almost worthless.  Eventually, under <person>Alexander Hamilton's</person> direction at the Treasury department, the currency was all redeemed at full value.</note><br/
[<source>PJC</source>]</p>

<p><ent>Continental drift</ent><br/
<hw>Continental drift</hw>. <def>the movements of continents relative to each other across the Earth's surface; see <er>plate tectonics</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Continental drive</ent><br/
<hw>Con`ti*nen"tal drive</hw>. <fld>(Automobiles)</fld> <def>A transmission arrangement in which the longitudinal crank shaft drives the rear wheels through a clutch, change-speed gear, countershaft, and two parallel side chains, in order.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Continental glacier</ent><br/
<hw>Continental glacier</hw>. <def>A broad ice sheet resting on a plain or plateau and spreading outward from a central n<eacute/v<eacute/, or region of accumulation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Continental pronunciation</ent><br/
<hw>Continental pronunciation</hw> <pr>(of Latin and Greek.)</pr> <def>A method of pronouncing Latin and Greek in which the vowels have their more familiar Continental values, as in German and Italian, the consonants being pronounced mostly as in English. The stricter form of this method of pronouncing Latin approaches the Roman, the modified form the English, pronunciation. The Continental method of Greek pronunciation is often called <stype>Erasmian</stype>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Continental system</ent><br/
<hw>Continental system</hw>. <fld>(Hist.)</fld> <def>The system of commercial blockade aiming to exclude England from commerce with the Continent instituted by the <col><b>Berlin decree</b></col>, which Napoleon I. issued from Berlin Nov. 21, 1806, declaring the British Isles to be in a state of blockade, and British subjects, property, and merchandise subject to capture, and excluding British ships from all parts of Europe under French dominion. The retaliatory measures of England were followed by the <col><b>Milan decree</b></col>, issued by Napoleon from Milan Dec. 17, 1807, imposing further restrictions, and declaring every ship going to or from a port of England or her colonies to be lawful prize.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Continently</ent><br/
<hw>Con"ti*nent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a continent manner; chastely; moderately; temperately.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>continent-wide</ent><br/
<hw>continent-wide</hw> <pos>adj.</pos> <def>involving the entire continent.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contingence</ent><br/
<hw>Con*tin"gence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Contingency</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contingency</ent><br/
<hw>Con*tin"gen*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contingencies</plw> <pr>(#)</pr></plu>. <ety>[Cf. F. <ets>contingence</ets>.]</ety> <sn>1.</sn> <def>Union or connection; the state of touching or contact.</def> <ldquo/Point of <xex>contingency</xex>.<rdquo/  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality or state of being contingent or casual; the possibility of coming to pass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Aristotle says we are not to build certain rules on the <qex>contingency</qex> of human actions.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An event which may or may not occur; that which is possible or probable; a fortuitous event; a chance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The remarkable position of the queen rendering her death a most important <qex>contingency</qex>.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An adjunct or accessory.</def>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>A certain possible event that may or may not happen, by which, when happening, some particular title may be affected.</def></p>

<p><syn><b>Syn.</b> -- Casualty; accident; chance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contingent</ent><br/
<hw>Con*tin"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contingens</ets>, <ets>-entis</ets>, p. pr. of <ets>contingere</ets> to touch on all sides, to happen; <ets>con-</ets> + <ets>tangere</ets> to touch: cf. F. <ets>contingent</ets>. See <er>Tangent</er>, <er>Tact</er>.]</ety> <sn>1.</sn> <def>Possible, or liable, but not certain, to occur; incidental; casual.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Weighing so much actual crime against so much <qex>contingent</qex> advantage.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Dependent on that which is undetermined or unknown; <as>as, the success of his undertaking is <ex>contingent</ex> upon events which he can not control</as>.</def> <ldquo/Uncertain and <xex>contingent</xex> causes.<rdquo/  <rj><au>Tillotson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Dependent for effect on something that may or may not occur; <as>as, a <ex>contingent</ex> estate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If a <qex>contingent</qex> legacy be left to any one when he attains, or if he attains, the age of twenty-one.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contingent</ent><br/
<hw>Con*tin"gent</hw>, <pos>n.</pos> <sn>1.</sn> <def>An event which may or may not happen; that which is unforeseen, undetermined, or dependent on something future; a contingency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His understanding could almost pierce into future <qex>contingents</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which falls to one in a division or apportionment among a number; a suitable share; proportion; esp., a quota of troops.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From the Alps to the border of Flanders, <qex>contingents</qex> were required . . . 200,000 men were in arms.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contingently</ent><br/
<hw>Con*tin"gent*ly</hw>, <pos>adv.</pos> <def>In a contingent manner; without design or foresight; accidentally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contingentness</ent><br/
<hw>Con*tin"gent*ness</hw>, <pos>n.</pos> <def>The state of being contingent; fortuitousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuable</ent><br/
<hw>Con*tin"u*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being continued</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continual</ent><br/
<hw>Con*tin"u*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>continuel</ets>, F. <ets>continuel</ets>. See <er>Continue</er>.]</ety> <sn>1.</sn> <def>Proceeding without interruption or cesstaion; continuous; unceasing; lasting; abiding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that is of a merry heart hath a <qex>continual</qex> feast.</q> <rj><qau>Prov. xv. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Occuring in steady and rapid succession; very frequent; often repeated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The eye is deligh by a <qex>continental</qex> succession of small landscapes.</q> <rj><qau>W. Irwing.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Continual proportionals</b></col> <fld>(Math.)</fld>, <cd>quantities in continued proportion.</cd>  <rj><au>Brande & C.</au></rj></cs></p>

<p><syn><b>Syn.</b> -- Constant; prepetual; incessant; unceasing; uninterrupted; unintermitted; continuous. See <er>Constant</er>, and <er>Continuous</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continually</ent><br/
<hw>Con*tin"u*al*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Without cessation; unceasingly; continuously; <as>as, the current flows <ex>continually</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why do not all animals <qex>continually</qex> increase in bigness?</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In regular or repeated succession; very often.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt eat bread at my table <qex>continually</qex>.</q> <rj><qau>2 Sam. ix. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuance</ent><br/
<hw>Con*tin"u*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>continuance</ets>.]</ety> <sn>1.</sn> <def>A holding on, or remaining in a particular state; permanence, as of condition, habits, abode, etc.; perseverance; constancy; duration; stay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great plagues, and of long <qex>continuance</qex>.</q> <rj><qau>Deut. xxviii. 59.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Patient <qex>continuance</qex> in well-doing.</q> <rj><qau>Rom. ii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Uninterrupted succession; continuation; constant renewal; perpetuation; propagation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The brute immediately regards his own preservation or the <qex>continuance</qex> of his species.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A holding together; continuity.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>The adjournment of the proceedings in a cause from one day, or from one stated term of a court, to another.</def> <sd>(b)</sd> <def>The entry of such adjournment and the grounds thereof on the record.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuant</ent><br/
<hw>Con*tin"u*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Continuing; prolonged; sustained; <as>as, a <ex>continuant</ex> sound</as>.</def> -- <def2><pos>n.</pos>  <def>A continuant sound; a letter whose sound may be prolonged.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuate</ent><br/
<hw>Con*tin"u*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>continuatus</ets>, p. p. See <er>Continue</er>.]</ety> <sn>1.</sn> <def>Immediately united together; intimately connected.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We are of Him and in Him, even as though our very flesh and bones should be made <qex>continuate</qex> with his.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Uninterrupted; unbroken; continual; continued.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An untirable and <qex>continuate</qex> goodness.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 314 --></p>

<p><ent>Continuation</ent><br/
<hw>Con*tin`u*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>continuatio</ets>: cf. F. <ets>connuation</ets>.]</ety> <sn>1.</sn> <def>That act or state of continuing; the state of being continued; uninterrupted extension or succession; prolongation; propagation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Preventing the <qex>continuation</qex> of the royal line.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which extends, increases, supplements, or carries on; <as>as, the <ex>continuation</ex> of a story</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>continuation</qex> of the version of Statius.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuative</ent><br/
<hw>Con*tin"u*a*tive</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>continuatif</ets>.]</ety> <sn>1.</sn> <fld>(Logic)</fld> <def>A term or expression denoting continuance.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To these may be added <qex>continuatives</qex>; as, Rome remains to this day; which includes, at least, two propositions, viz., Rome was, and Rome is.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>A word that continues the connection of sentences or subjects; a connective; a conjunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Continuatives</qex> . . . consolidate sentences into one continuous whole.</q> <rj><qau>Harris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuator</ent><br/
<hw>Con*tin"u*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>continuateur</ets>.]</ety> <def>One who, or that which, continues; esp., one who continues a series or a work; a continuer.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continue</ent><br/
<hw>Con*tin"ue</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Continued</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Continuing</conjf>.]</vmorph> <ety>[F. <ets>continuer</ets>, L. <ets>continuare</ets>, <ets>-tinuatum</ets>, to connect, continue, fr. <ets>continuus</ets>. See <er>Continuous</er>, and cf. <er>Continuate</er>.]</ety> <sn>1.</sn> <def>To remain in a given place or condition; to remain in connection with; to abide; to stay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here to <qex>continue</qex>, and build up here<br/
A growing empire.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>continue</qex> with me now three days, and have nothing to eat.</q> <rj><qau>Matt. xv. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be permanent or durable; to endure; to last.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But now thy kingdom shall not <qex>continue</qex>.</q> <rj><qau>1 Sam. xiii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be steadfast or constant in any course; to persevere; to abide; to endure; to persist; to keep up or maintain a particular condition, course, or series of actions; <as>as, the army <ex>continued</ex> to advance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If ye <qex>continue</qex> in my word, then are ye my disciples indeed.</q> <rj><qau>John viii. 31.</qau></rj></p>

<p><syn><b>Syn.</b> -- To persevere; persist. See <er>Persevere</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continue</ent><br/
<hw>Con*tin"ue</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To unite; to connect.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>the use of the navel is to <qex>continue</qex> the infant unto the mother.</q> <rj><qau>Sir T. browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To protract or extend in duration; to preserve or persist in; to cease not.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>continue</qex> thy loving kindness unto them that know thee.</q> <rj><qau>Ps. xxxvi. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You know how to make yourself happy by only <qex>continuing</qex> such a life as you have been long accustomed to lead.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To carry onward or extend; to prolong or produce; to add to or draw out in length.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bridge of wond'rous length,<br/
From hell <qex>continued</qex>, reaching th' utmost orb<br/
of this frail world.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To retain; to suffer or cause to remain; <as>as, the trustees were <ex>continued</ex></as>; also, to suffer to live.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And how shall we <qex>continue</qex> Claudio.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continued</ent><br/
<hw>Con*tin"ued</hw> <pr>(?)</pr>, <pos>p. p. & a.</pos> <def>Having extension of time, space, order of events, exertion of energy, etc.; extended; protracted; uninterrupted; also, resumed after interruption; extending through a succession of issues, session, etc.; <as>as, a <ex>continued</ex> story</as>.</def> <ldquo/<xex>Continued</xex> woe.<rdquo/ <au>Jenyns.</au> <ldquo/<xex>Continued</xex> succession.<rdquo/ <au>Locke.</au><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Continued bass</b></col> <fld>(Mus.)</fld>, <cd>a bass continued through an entire piece of music, while the other parts of the harmony are indicated by figures beneath the bass; the same as <xex>thorough bass</xex> or <xex>figured bass</xex>; basso continuo.</cd> [It.] -- <col><b>Continued fever</b></col> <fld>(Med.)</fld>, <cd>a fever which presents no interruption in its course.</cd> -- <col><b>Continued fraction</b></col> <fld>(Math.)</fld>, <cd>a fraction whose numerator is 1, and whose denominator is a whole number plus a fraction whose numerator is 1 and whose denominator is a whole number, plus a fraction, and so on.</cd> -- <col><b>Continued proportion</b></col> <fld>(Math.)</fld>, <cd>a proportion composed of two or more equal ratios, in which the consequent of each preceding ratio is the same with the antecedent of the following one; as, 4 : 8 : 8 : 16 :: 16 : 32.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuedly</ent><br/
<hw>Con*tin"u*ed*ly</hw> <pr>(? <or/ <?/)</pr>, <pos>adv.</pos> <def>Continuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuer</ent><br/
<hw>Con*tin"u*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who continues; one who has the power of perseverance or persistence.</def> <ldquo/Indulgent <xex>continuers</xex> in sin.<rdquo/  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I would my horse had the speed of your tongue, and so good a <qex>continuer</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuity</ent><br/
<hw>Con`ti*nu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Continuities</plw> <pr>(#)</pr></plu>. <ety>[L. <ets>continuitas</ets>: cf. F. <ets>continuit<eacute/</ets>. See <er>Continuous</er>.]</ety> <def>the state of being continuous; uninterrupted connection or succession; close union of parts; cohesion; <as>as, the <ex>continuity</ex> of fibers</as>.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sight would be tired, if it were attracted by a <qex>continuity</qex> of glittering objects.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Law of continuity</b></col> <fld>(Math. & Physics)</fld>, <cd>the principle that nothing passes from one state to another without passing through all the intermediate states.</cd> -- <col><b>Solution of continuity</b></col>. <fld>(Math.)</fld> <cd>See under <er>Solution</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuo</ent><br/
||<hw>Con*ti"nu*o</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[<ets>It</ets>.]</ety> <fld>(Mus.)</fld> <def>Basso continuo, or continued bass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuous</ent><br/
<hw>Con*tin"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>continuus</ets>, fr. <ets>continere</ets> to hold together. See <er>Continent</er>.]</ety> <sn>1.</sn> <def>Without break, cessation, or interruption; without intervening space or time; uninterrupted; unbroken; continual; unceasing; constant; continued; protracted; extended; <as>as, a <ex>continuous</ex> line of railroad; a <ex>continuous</ex> current of electricity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>he can hear its <qex>continuous</qex> murmur.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Not deviating or varying from uninformity; not interrupted; not joined or articulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Continuous brake</b></col> <fld>(Railroad)</fld>, <cd>a brake which is attached to each car a train, and can be caused to operate in all the cars simultaneously from a point on any car or on the engine.</cd> -- <col><b>Continuous impost</b></col>. <cd>See <er>Impost</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- <er>Continuous</er>, <er>Continual</er>.</syn> <usage> <xex>Continuous</xex> is the stronger word, and denotes that the continuity or union of parts is absolute and uninterrupted; as, a <xex>continuous</xex> sheet of ice; a <xex>continuous</xex> flow of water or of argument. So <person>Daniel Webster</person> speaks of <ldquo/a <xex>continuous</xex> and unbroken strain of the martial airs of England.<rdquo/ <xex>Continual</xex>, in most cases, marks a close and unbroken succession of things, rather than absolute continuity. Thus we speak of <xex>continual</xex> showers, implying a repetition with occasional interruptions; we speak of a person as liable to <xex>continual calls</xex>, <xex>or as subject to continual</xex> applications for aid, etc. See <er>Constant</er>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Continuously</ent><br/
<hw>Con*tin"u*ous*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a continuous maner; without interruption.</def> -- <wordforms><wf>Con*tin"u*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contline</ent><br/
<hw>Cont"line`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Ropemaking)</fld> <def>The space between the strands on the outside of a rope.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>The space between the bilges of two casks stowed side by side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contopus</ent><br/
<hw>Contopus</hw> <pos>n.</pos> <def>a genus of birds comprised of pewees, and including the <stype>wood pewees</stype> <spn>Contopus sordidulus</spn> and <spn>Contopus virens</spn>.</def><br/
<syn><b>Syn. --</b> genus <gen>Contopus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contorniate</ent><br/
<ent>Contorniate</ent><br/
<mhw>{ <hw>Con*tor"ni*ate</hw> <pr>(?)</pr>, ||<hw>Con*tor"ni*a`te</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos>, <ety>[It. <ets>contorniato</ets>, p. pr. of <ets>contorniare</ets> to make a circuit or outline, fr. <ets>contorno</ets> circuit, outline. See <er>Contour</er>.]</ety> <fld>(Numis.)</fld> <def>A species of medal or medallion of bronze, having a deep furrow on the contour or edge; -- supposed to have been struck in the days of <person>Constantine</person> and his successors.</def>  <rj><au>R. S. Poole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contorsion</ent><br/
<hw>Con*tor"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Contortion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contort</ent><br/
<hw>Con*tort"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>contortus</ets>, p. p. of <ets>contorquere</ets> to twist; <ets>con-</ets> + <ets>torquere</ets> to twist. See <er>Torture</er>.]</ety> <def>To twist, or twist together; to turn awry; to bend; to distort; to wrest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The vertebral arteries are variously <qex>contorted</qex>.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Kant <qex>contorted</qex> the term category from the proper meaning of attributed.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contorted</ent><br/
<hw>Con*tort"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Twisted, or twisted together.</def> <ldquo/A <xex>contorted</xex> chain of icicles.<rdquo/  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Twisted back upon itself, as some parts of plants.</def> <sd>(b)</sd> <def>Arranged so as to overlap each other; <as>as, petals in <ex>contorted</ex> or convolute <ae/stivation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contortion</ent><br/
<hw>Con*tor"tion</hw> <pr>(k<ocr/n*t<ocir/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>contortio</ets>: cf. F. <ets>contorsion</ets>. See <er>Contort</er>, and cf. <er>Torsion</er>.]</ety> <def>A twisting; a writhing; wry motion; a twist; <as>as, the <ex>contortion</ex> of the muscles of the face</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the <qex>contortions</qex> of the sibyl, without the inspiration.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contortionist</ent><br/
<hw>Con*tor"tion*ist</hw> <pr>(k<ocr/n*t<ocir/"sh<ucr/n*<icr/st)</pr>, <pos>n.</pos> <def>One who makes or practices contortions;</def> <specif>especially,</specif> <def>a gymnast who is able to twist the limbs into positions impossible for normal people to achieve.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Contortive</ent><br/
<hw>Con*tor"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Expressing contortion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contortuplicate</ent><br/
<hw>Con`tor*tu"pli*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contortuplicatus</ets>; <ets>contortus</ets> contorted + <ets>plicare</ets> to fold.]</ety> <fld>(Bot.)</fld> <def>Plaited lengthwise and twisted in addition, as the bud of the morning-glory.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contour</ent><br/
<hw>Con*tour"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>contour</ets>, fr. <ets>contourner</ets> to mark the outlines; <ets>con-</ets> + <ets>tourner</ets> to turn. See <er>Turn</er>.]</ety> <sn>1.</sn> <def>The outline of a figure or body, or the line or lines representing such an outline; the line that bounds; periphery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Titian's coloring and <qex>contours</qex>.</q> <rj><qau>A. Drummond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>The outline of a horizontal section of the ground, or of works of fortification.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contour feathers</b></col> <fld>(Zool.)</fld>, <cd>those feathers that form the general covering of a bird.</cd> -- <col><b>Contour of ground</b></col> <fld>(Surv.)</fld>, <cd>the outline of the surface of ground with respect to its undulation, etc.</cd> -- <col><b>Contour line</b></col> <fld>(Topographical Suv.)</fld>, <cd>the line in which a horizontal plane intersects a portion of ground, or the corresponding line in a map or chart.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contourne</ent><br/
||<hw>Con`tour`n<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., p. p. of <ets>contourner</ets> to twist.]</ety> <fld>(Her.)</fld> <def>Turned in a direction which is not the usual one; -- said of an animal turned to the sinister which is usually turned to the dexter, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contourniated</ent><br/
<hw>Con*tour"ni*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Contorniate</er>.]</ety> <fld>(Numis.)</fld> <def>Having furrowed edges, as if turned in a lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contra</ent><br/
<hw>Con"tra</hw> <pr>(?)</pr>. <def>A Latin adverb and preposition, signifying <xex>against</xex>, <xex>contrary</xex>, <xex>in opposition</xex>, etc., entering as a prefix into the composition of many English words. Cf. <er>Counter</er>, <pos>adv. & pref.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraband</ent><br/
<hw>Con"tra*band</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>contrabando</ets>; <ets>contra</ets> + <ets>bando</ets> ban, proclamation: cf. F. <ets>contrebande</ets>. See <er>Ban</er> an edict.]</ety> <sn>1.</sn> <def>Illegal or prohibited traffic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Persons the most bound in duty to prevent <qex>contraband</qex>, and the most interested in the seizures.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Goods or merchandise the importation or exportation of which is forbidden.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A negro slave, during the Civil War, escaped to, or was brought within, the Union lines. Such slave was considered contraband of war.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contraband of war</b></col>, <cd>that which, according to international law, cannot be supplied to a hostile belligerent except at the risk of seizure and condemnation by the aggrieved belligerent.</cd>  <rj><au>Wharton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraband</ent><br/
<hw>Con"tra*band</hw>, <pos>a.</pos> <def>Prohibited or excluded by law or treaty; forbidden; <as>as, <ex>contraband</ex> goods, or trade</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>contraband</qex> will always keep pace, in some measure, with the fair trade.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraband</ent><br/
<hw>Con"tra*band</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To import illegally, as prohibited goods; to smuggle.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To declare prohibited; to forbid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The law severly <qex>contrabands</qex><br/
Our taking business of men's hands.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrabandism</ent><br/
<hw>Con"tra*band*ism</hw> <pr>(-<icr/z'm)</pr>, <pos>n.</pos> <def>Traffic in contraband goods; smuggling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrabandist</ent><br/
<hw>Con"tra*band`ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who traffics illegally; a smuggler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrabass</ent><br/
<hw>Con`tra*bass"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The lowest stringed instrument of the violin family; the <er>double bass</er>; -- also applied to any instrument of the same deep range as the stringed double bass; <as>as, the <ex>contrabass</ex> ophicleide; the <ex>contrabass</ex> tuba or bombardon</as>.</def><br/
[<source>1913 Webster</source> + <source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contrabasso</ent><br/
<hw>Con`tra*bas"so</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>contrabasso</ets>.]</ety> <fld>(Mus.)</fld> <def>The largest kind of bass viol. See <er>Violone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>contraception</ent><br/
<hw>contraception</hw> <pos>n.</pos> <def>the intentional prevention of conception or impregnation, especially when accomplished by the use of contraceptive medications or devices; a form of <methodfor>birth control</methodfor>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>contraceptive</ent><br/
<hw>contraceptive</hw> <pos>adj.</pos> <def>capable of preventing conception or impregnation; <as>as, <ex>contraceptive</ex> devices and medications</as>.</def><br/
<syn><b>Syn. --</b> prophylactic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>contraceptive</ent><br/
<hw>contraceptive</hw> <pos>n.</pos> <def>a substance or device used to prevent conception.</def><br/
<syn><b>Syn. --</b> preventive, preventative, contraceptive device, prophylactic device, prophylactic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>contraclockwise</ent><br/
<hw>contraclockwise</hw> <pos>adj.</pos>  <def>in the direction opposite to the rotation of the hands of a clock; counterclockwise.  Opposite of <ant>clockwise</ant>.</def> <see>[Also See: <er>sinistral</er>.]</see><br/
<syn><b>Syn. --</b> counterclockwise, anticlockwise.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contract</ent><br/
<hw>Con*tract"</hw> <pr>(k<ocr/n*tr<acr/kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contracted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Contracting</conjf>.]</vmorph> <ety>[L. <ets>contractus</ets>, p. p. of <ets>contrahere</ets> to contract; <ets>con-</ets> + <ets>trahere</ets> to draw: cf. F. <ets>contracter</ets>. See <er>Trace</er>, and cf. <er>Contract</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To draw together or nearer; to reduce to a less compass; to shorten, narrow, or lessen; <as>as, to <ex>contract</ex> one's sphere of action</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In all things desuetude doth <qex>contract</qex> and narrow our faculties.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw together so as to wrinkle; to knit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou didst <qex>contract</qex> and purse thy brow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bring on; to incur; to acquire; <as>as, to <ex>contract</ex> a habit; to <ex>contract</ex> a debt; to <ex>contract</ex> a disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each from each <qex>contract</qex> new strength and light.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such behavior we <qex>contract</qex> by having much conversed with persons of high station.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To enter into, with mutual obligations; to make a bargain or covenant for.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have <qex>contracted</qex> an inviolable amity, peace, and lague with the aforesaid queen.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many persons . . . had <qex>contracted</qex> marriage within the degrees of consanguinity . . . prohibited by law.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To betroth; to affiance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The truth is, she and I, long since <qex>contracted</qex>,<br/
Are now so sure, that nothing can dissolve us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Gram.)</fld> <def>To shorten by omitting a letter or letters or by reducing two or more vowels or syllables to one.</def></p>

<p><syn><b>Syn.</b> -- To shorten; abridge; epitomize; narrow; lessen; condense; reduce; confine; incur; assume.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contract</ent><br/
<hw>Con*tract"</hw> <pr>(k<ocr/n*tr<acr/kt")</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be drawn together so as to be diminished in size or extent; to shrink; to be reduced in compass or in duration; <as>as, iron <ex>contracts</ex> in cooling; a rope <ex>contracts</ex> when wet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Years <qex>contracting</qex> to a moment.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make an agreement; to covenant; to agree; to bargain; <as>as, to <ex>contract</ex> for carrying the mail</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contract</ent><br/
<hw>Con"tract</hw> <pr>(k<ocr/n"tr<acr/kt)</pr>, <pos>a.</pos> <def>Contracted; <as>as, a <ex>contract</ex> verb</as>.</def>  <rj><au>Goodwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contract</ent><br/
<hw>Con*tract"</hw> <pr>(k<ocr/n*tr<acr/kt")</pr>, <pos>a.</pos> <ety>[L. <ets>contractus</ets>, p. p.]</ety> <def>Contracted; affianced; betrothed.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contract</ent><br/
<hw>Con"tract</hw> <pr>(k<ocr/n"tr<acr/kt)</pr>, <pos>n.</pos> <ety>[L. <ets>contractus</ets>, fr. <ets>contrahere</ets>: cf. F. <ets>contrat</ets>, formerly also <ets>contract</ets>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>The agreement of two or more persons, upon a sufficient consideration or cause, to do, or to abstain from doing, some act; an agreement in which a party undertakes to do, or not to do, a particular thing; a formal bargain; a compact; an interchange of legal rights.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A formal writing which contains the agreement of parties, with the terms and conditions, and which serves as a proof of the obligation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of formally betrothing a man and woman.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the the night of the <qex>contract</qex>.</q> <rj><qau>Longwellow.</qau></rj></p>

<p><syn><b>Syn.</b> -- Covenant; agreement; compact; stipulation; bargain; arrangement; obligation. See <er>Covenant</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>contractable</ent><br/
<hw>con*tract"a*ble</hw> <pr>(k<ocr/n*tr<acr/kt"<adot/*b'l)</pr>, <pos>adj.</pos>  <def>capable of being transmitted by infection; same as <er>communicable</er>, 1; -- of disease.</def><br/
<syn><b>Syn. --</b> catching, communicable, contagious, transmissible, transmittable.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contracted</ent><br/
<hw>Con*tract"ed</hw> <pr>(k<ocr/n*tr<acr/kt"<ecr/d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Drawn together; shrunken; wrinkled; narrow; <as>as, a <ex>contracted</ex> brow; a <ex>contracted</ex> noun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Narrow; illiberal; selfish; <as>as, a <ex>contracted</ex> mind; <ex>contracted</ex> views</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Bargained for; betrothed; <as>as, a <ex>contracted</ex> peace</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Inquire me out <qex>contracted</qex> bachelors.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractedness</ent><br/
<hw>Con*tract"ed*ness</hw>, <pos>n.</pos> <def>The state of being contracted; narrowness; meanness; selfishness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractibility</ent><br/
<hw>Con*tract`i*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Capability of being contracted; quality of being contractible; <as>as, the <ex>contractibility</ex> and dilatability of air</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractible</ent><br/
<hw>Con*tract"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of contraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Small air bladders distable and <qex>contractible</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractibleness</ent><br/
<hw>Con*tract"i*ble*ness</hw>, <pos>n.</pos> <def>Contractibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractile</ent><br/
<hw>Con*tract"ile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>contractile</ets>.]</ety> <def>tending to contract; having the power or property of contracting, or of shrinking into shorter or smaller dimensions; <as>as, the <ex>contractile</ex> tissues</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The heart's <qex>contractile</qex> force.</q> <rj><qau>H. Brooke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Each cilium seems to be composed of <qex>contractile</qex> substance.</q> <rj><qau>Hixley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contractile vacuole</b></col> <fld>(Zool.)</fld>, <cd>a pulsating cavity in the interior of a protozoan, supposed to be excretory in function. There may be one, two, or more.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractility</ent><br/
<hw>Con`trac*til"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or property by which bodies shrink or contract.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The power possessed by the fibers of living muscle of contracting or shortening.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When subject to the will, as in the muscles of locomotion, such power is called <xex>voluntary contractility</xex>; when not controlled by the will, as in the muscles of the heart, it is <xex>involuntary contractility</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>contracting</ent><br/
<hw>contracting</hw> <pos>n.</pos> <def>the act or process of acquiring an infectious disease; contraction; <as>as, the <ex>contracting</ex> of a serious illness can be financially catastrophic</as>.</def><br/
<syn><b>Syn. --</b> catching.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contraction</ent><br/
<hw>Con*trac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contractio</ets>: cf. F. <ets>contraction</ets>.]</ety> <sn>1.</sn> <def>The act or process of contracting, shortening, or shrinking; the state of being contracted; <as>as, <ex>contraction</ex> of the heart, of the pupil of the eye, or of a tendon; the <ex>contraction</ex> produced by cold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>The process of shortening an operation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of incurring or becoming subject to, as liabilities, obligation, debts, etc.; the process of becoming subject to; <as>as, the <ex>contraction</ex> of a disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Something contracted or abbreviated, as a word or phrase; -- as, <xex>plenipo</xex> for <xex>plenipotentiary</xex>; <xex>crim. con.</xex> for <xex>criminal conversation</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gram.)</fld> <def>The shortening of a word, or of two words, by the omission of a letter or letters, or by reducing two or more vowels or syllables to one; as, <xex>ne'er</xex> for <xex>never</xex>; <xex>can't</xex> for <xex>can not</xex>; <xex>don't</xex> for <xex>do not</xex>; <xex>it's</xex> for <xex>it is</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A marriage contract.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractive</ent><br/
<hw>Con*tract"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to contract; having the property or power or power of contracting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contractor</ent><br/
<hw>Con*tract"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who contracts; one of the parties to a bargain; one who covenants to do anything for another; specifically, one who contracts to perform work on a rather large scale, at a certain price or rate, as in building houses or making a railroad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contract system</ent><br/
<hw>Con"tract sys"tem</hw>. <sn>1.</sn> <def>The sweating system.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The system of employing convicts by selling their labor (to be performed inside the prison) at a fixed price per day to contractors who are allowed to have agents in the prison to superintend the work.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contract tablet</ent><br/
<hw>Con"tract tablet</hw>. <fld>(Babylonian & Assyrian Antiq.)</fld> <def>A clay tablet on which was inscribed a contract, for safe keeping.  Such tablets were inclosed in an outer case (often called the <it>envelope</it>), on which was inscribed a duplicate of the inscription on the inclosed tablet.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contracture</ent><br/
<hw>Con*trac"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>contractura</ets> a drawing together.]</ety> <fld>(Med.)</fld> <def>A state of permanent rigidity or contraction of the muscles, generally of the flexor muscles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradance</ent><br/
<hw>Con"tra*dance`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>contra-</ets> + <ets>dance</ets>: cf. F. <ets>contrdance</ets>. Cf. <er>Country-dance</er>.]</ety> <def>A dance in which the partners are arranged face to face, or in opposite lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 315 --></p>

<p><ent>Contradict</ent><br/
<hw>Con`tra*dict"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contradicted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Contradicting</conjf>.]</vmorph> <ety>[L. <ets>contradictus</ets>, p. p. of <ets>contradicere</ets> to speak against; <ets>contra</ets> + <ets>dicere</ets> to speak. See <er>Diction</er>.]</ety> <sn>1.</sn> <def>To assert the contrary of; to oppose in words; to take issue with; to gainsay; to deny the truth of, as of a statement or a speaker; to impugn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dear Duff, I prithee, <qex>contradict</qex> thyself,<br/
And say it is not so.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The future can not <qex>contradict</qex> the past.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be contrary to; to oppose; to resist.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>No truth can <qex>contradict</qex> another truth.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A greater power than we can <qex>contradict</qex><br/
Hath thwarted our intents.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradict</ent><br/
<hw>Con`tra*dict</hw>, <pos>v. i.</pos> <def>To oppose in words; to gainsay; to deny, or assert the contrary of, something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . spake against those things which were spoken by Paul, <qex>contradicting</qex> and blaspheming.</q> <rj><qau>Acts xiii. 45.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictable</ent><br/
<hw>Con`tra*dict"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being contradicted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradicter</ent><br/
<hw>Con`tra*dict"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>one who contradicts.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradiction</ent><br/
<hw>Con`tra*dic"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contradictio</ets> answer, objection: cf. F. <ets>contradiction</ets>.]</ety> <sn>1.</sn> <def>An assertion of the contrary to what has been said or affirmed; denial of the truth of a statement or assertion; contrary declaration; gainsaying.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His fair demands<br/
Shall be accomplished without <qex>contradiction</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Direct opposition or repugnancy; inconsistency; incongruity or contrariety; one who, or that which, is inconsistent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>can he make deathless death? That were to make<br/
Strange <qex>contradiction</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We state our experience and then we come to a manly resolution of acting in <qex>contradiction</qex> to it.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Both parts of a <qex>contradiction</qex> can not possibly be true.</q> <rj><qau>Hobbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of <qex>contradictions</qex> infinite the slave.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Principle of contradiction</b></col> <fld>(Logic)</fld>, <cd>the axiom or law of thought that a thing cannot <xex>be</xex> and <xex>not be</xex> at the same time, or a thing must either be or not be, or the same attribute can not at the same time be affirmed and and denied of the same subject; also called the <altname>law of the excluded middle</altname>.</cd> <note>It develops itself in three specific forms which have been called the <ldquo/Three Logical Axioms.<rdquo/ <xex>First</xex>, <ldquo/A is A.<rdquo/ <xex>Second</xex>, <ldquo/A is not Not-A<rdquo/ <xex>Third</xex>, <ldquo/Everything is either A or Not-A.<rdquo/</note></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictional</ent><br/
<hw>Con`tra*dic"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Contradictory; inconsistent; opposing.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictious</ent><br/
<hw>Con`tra*dic"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Filled with contradictions; inconsistent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inclined to contradict or cavil</def> <mark>[Obs.]</mark>  <rj><au>Sharp.</au></rj></p>

<p>-- <wordforms><wf>Con`tra*dic"tious*ness</wf>, <pos>n.</pos></wordforms>  <rj><au>Norris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictive</ent><br/
<hw>Con`tra*dict"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Contradictory; inconsistent.</def> -- <wordforms><wf>Con`tra*dict"ive*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictor</ent><br/
<hw>Con`tra*dict"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A contradicter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictorily</ent><br/
<hw>Con`tra*dict"o*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a contradictory manner.</def>  <rj><au>Sharp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictoriness</ent><br/
<hw>Con"tra*dict`o*ri*ness</hw>, <pos>n.</pos> <def>The quality of being contradictory; opposition; inconsistency.</def>  <rj><au>J. Whitaker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictory</ent><br/
<hw>Con`tra*dict"o*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>contradictorius</ets>: cf. F. <ets>contradictoire</ets>.]</ety> <sn>1.</sn> <def>Affirming the contrary; implying a denial of what has been asserted; also, mutually contradicting; inconsistent.</def> <ldquo/<xex>Contradictory</xex> assertions.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Opposing or opposed; repugnant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Schemes . . . <qex>contradictory</qex> to common sense.</q> <rj><qau>Addisn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradictory</ent><br/
<hw>Con`tra*dict"o*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contradictories</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A proposition or thing which denies or opposes another; contrariety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is common with princes to will <qex>contradictories</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Logic)</fld> <def>propositions with the same terms, but opposed to each other both in quality and quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradistinct</ent><br/
<hw>Con`tra*dis*tinct"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Distinguished by opposite qualities.</def>  <rj><au>J. Goodwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradistinction</ent><br/
<hw>Con`tra*dis*tinc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Distinction by contrast.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That there are such things as sins of infirmity in <qex>contradistinction</qex> to those of presumption is not to be questioned.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradistinctive</ent><br/
<hw>Con`tra*dis*tinc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>having the quality of contradistinction; distinguishing by contrast.</def> -- <wordforms><wf>Con`tra*dis*tinc"tive</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contradistinguish</ent><br/
<hw>Con`tra*dis*tin"guish</hw> <pr>(?; 144)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contradistinguished</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contradistinguishing</conjf>.]</vmorph> <def>To distinguish by a contrast of opposite qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These are our complex ideas of soul and body, as <qex>contradistinguished</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrafagetto</ent><br/
<hw>Con`tra*fa*get"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>The double bassoon, an octave deeper than the bassoon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrafissure</ent><br/
<hw>Con`tra*fis"sure</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A fissure or fracture on the side opposite to that which received the blow, or at some distance from it.</def>
 <rj><qau>Coxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrahent</ent><br/
<hw>Con"tra*hent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>contrahens</ets>, p. pr. See <er>Contract</er>.]</ety> <def>Entering into covenant; contracting; <as>as, <ex>contrahent</ex> parties</as>.</def> <mark>[Obs.]</mark>  <rj><au>Mede.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraindicant</ent><br/
<hw>Con"tra*in"di*cant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Something, as a symptom, indicating that the usual mode of treatment is not to be followed.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraindicate</ent><br/
<hw>Con"tra*in*"di*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contraindicated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contraindicating</conjf> <pr>(?)</pr>.]</vmorph> <fld>(Med.)</fld> <def>To indicate, as by a symptom, some method of treatment contrary to that which the general tenor of the case would seem to require; especially, to indicate that a particular method of treatment should not be used; <as>as, penicilin is <ex>contraindicated</ex> in patients who have shown hypersensitivity</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q><qex>Contraindicating</qex> symptoms must be observed.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraindication</ent><br/
<hw>Con"tra*in`di*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(med.)</fld> <def>An indication or symptom which forbids the method of treatment usual in such cases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contralto</ent><br/
<hw>Con*tral"to</hw> <pr>(? <or/ <?/)</pr>, <pos>n.</pos> <ety>[It., fr. <ets>contra</ets> + <ets>alto</ets>. See <er>Alto</er>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The part sung by the highest male or lowest female voices; the alto or counter tenor.</def> <sd>(b)</sd> <def>the voice or singer performing this part; <as>as, her voice is a <ex>contralto</ex>; she is a <ex>contralto</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The usual range of the contralto voice is from G, below middle C, to the C above that; though exceptionally it embraces two octaves.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contralto</ent><br/
<hw>Con*tral"to</hw> <pr>(? <or/ <?/)</pr>, <pos>a.</pos> <fld>(Mus.)</fld> <def>Of or pertaining to a contralto, or to the part in music called contralto; <as>as, a <ex>contralto</ex> voice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contramure</ent><br/
<hw>Con"tra*mure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Countermure</er>.]</ety> <fld>(fort.)</fld> <def>An outer wall.</def> <mark>[Obs.]</mark>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contranatural</ent><br/
<hw>Con"tra*nat"u*ral</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[Cf. <er>Counternatural</er>.]</ety> <def>Opposed to or against nature; unnatural.</def> <mark>[R.]</mark>  <rj><au>Bp. Rust.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraplex</ent><br/
<hw>Con"tra*plex</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Contra-</ets> + <ets>-plex</ets> as in du<ets>plex</ets>.]</ety> <fld>(Teleg.)</fld> <def>Pertaining to the sending of two messages in opposite directions at the same time.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contraposition</ent><br/
<hw>Con"tra*po*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>contra-</ets> + <ets>position</ets>: cf. f. <ets>conterposition</ets>.]</ety> <sn>1.</sn> <def>A placing over against; opposite position.</def> <mark>[Obs.]</mark>  <rj><au>F. Potter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>A so-called immediate inference which consists in denying the original subject of the contradictory predicate; <it>e. g.</it>: Every S is P; therefore, no Not-P is S.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraption</ent><br/
<hw>Con*trap"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A contrivance; a new-fangled device; -- used scornfully.</def> <mark>[Colloq. or Dial.]</mark> -- <wordforms><wf>Con*trap"tious</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms></p>

<p><q>We all remember some of the extraordinary <qex>contraptions</qex> which have been thus evolved and put upon the market.</q>  <rj><qau>F. M. Ware.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contrapuntal</ent><br/
<hw>Con`tra*pun"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It. <ets>contrappunto</ets> counterpoint. See <er>Counterpoint</er>.]</ety> <fld>(Mus.)</fld> <def>Pertaining to, or according to the rules of, counterpoint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrapuntist</ent><br/
<hw>Con`tra*pun"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>contrappuntista</ets>.]</ety> <fld>(Mus.)</fld> <def>One skilled in counterpoint.</def>  <rj><au>L. Mason.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraremonstrant</ent><br/
<hw>Con"tra*re*mon"strant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who remonstrates in opposition or answer to a remonstrant.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They did the synod wrong to make this distinction of <qex>contraremonstrants</qex> and remonstrants.</q> <rj><qau>Hales.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>contrarian</ent><br/
<hw>con*tra"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>contrarians</ets>, p. pr. of <ets>contrariare</ets> to oppose, fr. L. <ets>contrarius</ets>: cf. F. <ets>contrariant</ets>, p. pr. of <ets>contrarier</ets> to contradict. See <er>Contrary</er>.]</ety> <def>taking an opposing view, especially a view opposite to that taken by the majority.</def><br/
[<source>PJC</source>]</p>

<p><ent>contrarian</ent><br/
<hw>con*tra"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[see <er>contrarian</er>, <pos>a.</pos>.]</ety> <def>a person who habitually takes a view opposite to that held by the majority; <as>as, the <ex>contrarians</ex> in the stock market prefer to sell when most analysts advise us to buy</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Contrariant</ent><br/
<hw>Con*tra"ri*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>contrarians</ets>, p. pr. of <ets>contrariare</ets> to oppose, fr. L. <ets>contrarius</ets>: cf. F. <ets>contrariant</ets>, p. pr. of <ets>contrarier</ets> to contradict. See <er>Contrary</er>.]</ety> <def>Contrary; opposed; antagonistic; inconsistent; contradictory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The struggles of <qex>contrariant</qex> factions.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrariantly</ent><br/
<hw>Con*tra"ri*ant*ly</hw>, <pos>adv.</pos> <def>Contrarily.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraries</ent><br/
<hw>Con"tra*ries</hw> <pr>(? <or/ <?/; 48)</pr>, <pos>n. pl.</pos> <ety>[Pl. of <er>Contrary</er>, <pos>n.</pos>]</ety> <fld>(Logic)</fld> <def>Propositions which directly and destructively contradict each other, but of which the falsehood of one does not establish the truth of the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If two universals differ in quality, they are <qex>contraries</qex>; as, every vine is a tree; no vine is a tree. These can never be both true together; but they may be both false.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrariety</ent><br/
<hw>Con`tra*ri"e*ty</hw> <pr>(?)</pr> <pos>n.</pos>; <plu><it>pl.</it> <plw>Contrarieties</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>contrarietas</ets>: cf. F. <ets>contrari<eacute/t<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state or quality of being contrary; opposition; repugnance; disagreement; antagonism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a <qex>contrariety</qex> between those things that conscience inclines to, and those that entertain the senses.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something which is contrary to, or inconsistent with, something else; an inconsistency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How can these <qex>contrarieties</qex> agree?</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Inconsistency; discrepancy; repugnance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrarily</ent><br/
<hw>Con"tra*ri*ly</hw> <pr>(? or <?/)</pr>, <pos>adv.</pos> <def>In a contrary manner; in opposition; on the other side; in opposite ways.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrariness</ent><br/
<hw>Con"tra*ri*ness</hw>, <pos>n.</pos> <def>state or quality of being contrary; opposition; inconsistency; contrariety; perverseness; obstinacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrarious</ent><br/
<hw>Con*tra"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>contrariosus</ets>: cf. OF. <ets>contrarios</ets> <ets>contralius</ets>.]</ety> <def>Showing contrariety; repugnant; perverse.</def> <mark>[Archaic]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She flew <qex>contrarious</qex> in the face of God.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrariously</ent><br/
<hw>Con*tra"ri*ous*ly</hw>, <pos>adv.</pos> <def>Contrarily; oppositely.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrariwise</ent><br/
<hw>Con"tra*ri*wise</hw> <pr>(? <or/ <?/)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>On the contrary; oppositely; on the other hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not rendering evil for evil, or railing for railing; but <qex>contrariwise</qex>, blessing.</q> <rj><qau>1 Pet. iii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a contrary order; conversely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Everything that acts upon the fluids must, at the same time, act upon the solids, and <qex>contrariwise</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrarotation</ent><br/
<hw>Con`tra*ro*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Circular motion in a direction contrary to some other circular motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrary</ent><br/
<hw>Con"tra*ry</hw> <pr>(? <or/ ?; 48)</pr>, <pos>a.</pos> <ety>[OE. <ets>contrarie</ets>, <ets>contraire</ets>, F. <ets>contraire</ets>, fr. L. <ets>contrarius</ets>, fr. <ets>contra</ets>. See <er>Contra-</er>.]</ety> <sn>1.</sn> <def>Opposite; in an opposite direction; in opposition; adverse; <as>as, <ex>contrary</ex> winds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And if ye walk <qex>contrary</qex> unto me, and will not hearken unto me.</q> <rj><qau>Lev. xxvi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We have lost our labor; they are gone a <qex>contrary</qex> way.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Opposed; contradictory; repugnant; inconsistent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fame, if not double-faced, is double mouthed,<br/
And with <qex>contrary</qex> blast proclaims most deeds.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The doctrine of the earth's motion appeared to be <qex>contrary</qex> to the sacred Scripture.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Given to opposition; perverse; forward; wayward; <as>as, a <ex>contrary</ex> disposition; a <ex>contrary</ex> child</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Logic)</fld> <def>Affirming the opposite; so opposed as to destroy each other; <as>as, <ex>contrary</ex> propositions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contrary motion</b></col> <fld>(Mus.)</fld>, <cd>the progression of parts in opposite directions, one ascending, the other descending.</cd></cs></p>

<p><syn><b>Syn.</b> -- Adverse; repugnant; hostile; inimical; discordant; inconsistent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrary</ent><br/
<hw>Con"tra*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contraries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A thing that is of contrary or opposite qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>contraries</qex> hold more antipathy<br/
Than I and such a knave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An opponent; an enemy.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>the opposite; a proposition, fact, or condition incompatible with another; <as>as, slender proofs which rather show the <ex>contrary</ex></as>. See <er>Converse</er>, <pos>n.</pos>, 1.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Logic)</fld> <def>See <er>Contraries</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>On the contrary</b></col>, <cd>in opposition; on the other hand.</cd> <au>Swift.</au> -- <col><b>To the contrary</b></col>, <cd>to an opposite purpose or intent; on the other side.</cd> <ldquo/They did it, not for want of instruction <xex>to the contrary</xex>.<rdquo/ <au>Bp. Stillingfleet.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrary</ent><br/
<hw>Con"tra*ry</hw>, <pos>v. t.</pos> <ety>[F. <ets>contrarier</ets>. See <er>Contrary</er>, <pos>a.</pos>]</ety> <def>To contradict or oppose; to thwart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I was advised not to <qex>contrary</qex> the king.</q> <rj><qau>Bp. Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrast</ent><br/
<hw>Con*trast"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contrasted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Contrasting</conjf>.]</vmorph> <ety>[F. <ets>contraster</ets>, LL. <ets>contrastare</ets> to resist, withstand, fr. L. <ets>contra + stare</ets> to stand. See <er>Stand</er>.]</ety> <def>To stand in opposition; to exhibit difference, unlikeness, or opposition of qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The joints which divide the sandstone <qex>contrast</qex> finely with the divisional planes which separate the basalt into pillars.</q> <rj><qau>Lyell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrast</ent><br/
<hw>Con*trast"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To set in opposition, or over against, in order to show the differences between, or the comparative excellences and defects of; to compare by difference or contrariety of qualities; <as>as, to <ex>contrast</ex> the present with the past</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fine Arts)</fld> <def>To give greater effect to, as to a figure or other object, by putting it in some relation of opposition to another figure or object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>the figures of the groups must not be all on side . . . but must <qex>contrast</qex> each other by their several position.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrast</ent><br/
<hw>Con"trast</hw> <pr>(k<ocr/n"tr<adot/st)</pr>, <pos>n.</pos> <ety>[F. <ets>contraste</ets>: cf. It. <ets>contrasto</ets>.]</ety> <sn>1.</sn> <def>The act of contrasting, or the state of being contrasted; comparison by contrariety of qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>place the prospect of the soul<br/
In sober <qex>contrast</qex> with reality.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Opposition or dissimilitude of things or qualities; unlikeness, esp. as shown by juxtaposition or comparison.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>contrasts</qex> and resemblances of the seasons.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fine Arts)</fld> <def>The opposition of varied forms, colors, etc., which by such juxtaposition more vividly express each other's peculiarities.</def> <rj><qau>Fairholt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrastimulant</ent><br/
<hw>Con`tra*stim"u*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Counteracting the effects of stimulants; relating to a course of medical treatment based on a theory of contrastimulants.</def> -- <def2><pos>n.</pos> <fld>(Med.)</fld> <def>An agent which counteracts the effect of a stimulant.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrate</ent><br/
<hw>Con"trate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Contra-</er>.]</ety> <def>Having cogs or teeth projecting parallel to the axis, instead of radiating from it.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contrate wheel</b></col>. <cd>See <er>Crown wheel</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contratenor</ent><br/
<hw>Con"tra*ten`or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Counter tenor</er>.]</ety> <fld>(Mus.)</fld> <def>Counter tenor; contralto.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contravallation</ent><br/
<hw>Con`tra*val*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>contra-</ets> + <ets>vallation</ets>: cf. F. <ets>contrevallation</ets>. Cf. <er>Countervallation</er>.]</ety> <fld>(Fort.)</fld> <def>A trench guarded with a parapet, constructed by besiegers, to secure themselves and check sallies of the besieged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contravene</ent><br/
<hw>Con`tra*vene"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contravened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contravening</conjf>.]</vmorph> <ety>[LL. <ets>contravenire</ets>; L. <ets>contra + venire</ets> to come: cf. F. <ets>contrevenir</ets>. See <er>Come</er>.]</ety> <sn>1.</sn> <def>To meet in the way of opposition; to come into conflict with; to oppose; to contradict; to obstruct the operation of; to defeat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So plain a proposition . . . was not likely to be <qex>contravened</qex>.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To violate; to nullify; to be inconsistent with; <as>as, to <ex>contravene</ex> a law</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Laws that place the subjects in such a state <qex>contravene</qex> the first principles of the compact of authority.</q> <rj><qau>Johnson.</qau></rj></p>

<p><syn><b>Syn.</b> -- To contradict; set aside; nullify; defeat; cross; obstruct; baffle; thwart.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contravener</ent><br/
<hw>Con`tra*ven"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who contravenes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contravention</ent><br/
<hw>Con`tra*ven"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>contravention</ets>.]</ety> <def>The act of contravening; opposition; obstruction; transgression; violation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Warrants in <qex>contravention</qex> of the acts of Parliament.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>contravention</qex> of all his marriage stipulations.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contraversion</ent><br/
<hw>Con`tra*ver"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A turning to the opposite side; antistrophe.</def>
 <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrayerva</ent><br/
<hw>Con`tra*yer"va</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>contrayerba</ets>, literally, a counter herb, hence, an antidote for poison, fr. l. <ets>contra + herba</ets> herb.]</ety> <fld>(Bot.)</fld> <def>A species of <gen>Dorstenia</gen> (<spn>Dorstenia Contrayerva</spn>), a South American plant, the aromatic root of which is sometimes used in medicine as a gentle stimulant and tonic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrecoup</ent><br/
||<hw>Con`tre*coup"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>contre</ets> (L. <ets>contra</ets>) + <ets>coup</ets> a blow.]</ety> <fld>(med.)</fld> <def>A concussion or shock produced by a blow or other injury, in a part or region opposite to that at which the blow is received, often causing rupture or disorganisation of the parts affected.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contredanse</ent><br/
<hw>Con"tre*danse`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>contredanse</ets> (fr. E. <er>Country-dance</er>). ]</ety> <sn>1.</sn> <sd>(a)</sd> <def>A dance in which the partners are arranged face to face, or in opposite lines.</def> <sd>(b)</sd> <def>The quadrille.</def> <mark>[Obs.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Music)</fld> <def>A piece of music in the rhythm of such a dance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contretemps</ent><br/
||<hw>Con`tre*temps"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>contre</ets> (L. <ets>conta</ets>) + <ets>temps</ets> time, fr. L. <ets>tempus</ets>.]</ety> <def>An unexpected and untoward accident; something inopportune or embarrassing; a hitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In this unhappy <qex>contretemps</qex>.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contributable</ent><br/
<hw>Con*trib"u*ta*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being contributed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contributary</ent><br/
<hw>Con*trib"u*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Contributory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tributary; contributing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It was situated on the Ganges, at the place where this river received a <qex>contributary</qex> stream.</q> <rj><qau>D'Anville (Trans. ).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contribute</ent><br/
<hw>Con*trib"ute</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contributed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Contributing</conjf>.]</vmorph> <ety>[L. <ets>contributus</ets>, p. p. of <ets>contribuere</ets> to bring together, to add; <ets>con-</ets> + <ets>tribuere</ets> to grant, impart. See <er>Tribute</er>.]</ety> <def>To give or grant i common with others; to give to a common stock or for a common purpose; to furnish or suply in part; to give (money or other aid) for a specified object; <as>as, to <ex>contribute</ex> food or fuel for the poor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>England <qex>contributes</qex> much more than any other of the allies.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contribute</ent><br/
<hw>Con*trib"ute</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To give a part to a common stock; to lend assistance or aid, or give something, to a common purpose; to have a share in any act or effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are engaged in war; the secretary of state calls upon the colonies to <qex>contribute</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give or use one's power or influence for any object; to assist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These men also <qex>contributed</qex> to obstruct the progress of wisdom.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contribution</ent><br/
<hw>Con`tri*bu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contributio</ets>: cf. F. <ets>contribution</ets>.]</ety> <sn>1.</sn> <def>The act of contributing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is contributed; -- either the portion which an individual furnishes to the common stock, or the whole which is formed by the gifts of individuals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A certain <qex>contribution</qex> for the poor saints which are at jerusalem.</q> <rj><qau>Rom. xv. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Aristotle's actual <qex>contributions</qex> to the physical sciences.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>An irregular and arbitrary imposition or tax leved on the people of a town or country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These sums, . . . and the forced <qex>contributions</qex> paid by luckless peasants, enabled him to keep his straggling troops together.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Payment, by each of several jointly liable, of a share in a loss suffered or an amount paid by one of their number for the common benefit.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 316 --></p>

<p><ent>Contributional</ent><br/
<hw>Con`tri*bu"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or furnishing, a contribution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contribution plan</ent><br/
<hw>Con`tri*bu"tion plan</hw>. <fld>(Life Insurance)</fld> <def>A plan of distributing surplus by giving to each policy the excess of premiums and interest earned thereon over the expenses of management, cost of insurance, and the policy value at the date of computation. This excess is called the <xex>contribution</xex> of the policy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Contributive</ent><br/
<hw>Con*trib"u*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Contributing, or tending to contribute.</def>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contributor</ent><br/
<hw>Con*trib"u*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, contributes;</def> <specif>specifically,</specif> <def>one who writes articles for a newspaper, magazine, or book.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contributory</ent><br/
<hw>Con*trib"u*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Contributing to the same stock or purpose; promoting the same end; bringing assistance to some joint design, or increase to some common stock; contributive.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bonfires of <qex>contributory</qex> wood.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contributory negligence</b></col> <fld>(Law)</fld>, <cd>negligence by an injured party, which combines with the negligence of the injurer in producing the injury, and which bars recovery when it is the proximate cause of the injury.</cd>  <rj><au>Wharton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contributory</ent><br/
<hw>Con*trib"u*to*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contributories</plw> <pr>(<?/)</pr>.</plu> <def>One who contributes, or is liable to be called upon to contribute, as toward the discharge of a common indebtedness.</def>  <rj><au>Abbott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrist</ent><br/
<hw>Con*trist"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>contrister</ets>. See <er>Contristate</er>.]</ety> <def>To make sad.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To deject and <qex>contrist</qex> myself.</q> <rj><qau>Sterne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contristate</ent><br/
<hw>Con*tris"tate</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[L. <ets>contristatus</ets>, p. p. of <ets>contristare</ets> to sadden; <ets>con-</ets> + <ets>tristis</ets> sad.]</ety> <def>To make sorrowful.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrite</ent><br/
<hw>Con"trite</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>contritus</ets> bruised, p. p. of <ets>contrere</ets> to grind, bruise; <ets>con-</ets> + <ets>terere</ets> to rub, grind: cf. F. <ets>contrit</ets> See <er>Trite</er>.]</ety> <sn>1.</sn> <def>Thoroughly bruised or broken.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Broken down with grief and penitence; deeply sorrowful for sin because it is displeasing to God; humbly and thoroughly penitent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>contrite</qex> heart, O God, thou wilt not despise.</q> <rj><qau>Ps. li. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be penitent, and for thy fault <qex>contrite</qex>.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> -- Penitent; repentant; humble; sorrowful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrite</ent><br/
<hw>Con"trite</hw>, <pos>n.</pos> <def>A contrite person.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrite</ent><br/
<hw>Con"trite</hw>, <pos>v.</pos> <def>In a contrite manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contriteness</ent><br/
<hw>Con"trite`ness</hw>, <pos>n.</pos> <def>Deep sorrow and penitence for sin; contrition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrition</ent><br/
<hw>Con*tri"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>contrition</ets>, L. <ets>contritio</ets>.]</ety> <sn>1.</sn> <def>The act of grinding or ribbing to powder; attrition; friction; rubbing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The breaking of their parts into less parts by <qex>contrition</qex>.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being contrite; deep sorrow and repentance for sin, because sin is displeasing to God; humble penitence; through repentance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My future days shall be one whole <qex>contrition</qex>.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- repentance; penitence; humiliation; compunction; self-reproach; remorse.</syn> <usage> -- <er>Contrition</er>, <er>Attrition</er>, <er>repentance</er>. -- <xex>Contrition</xex> is deep sorrow and self-condemnation, with through repetance for sin because it is displeasing to God, and implies a feeling of love toward God. <xex>Attrition</xex> is sorrow for sin, or imperfect repentance produced by fear of punishment or a sense of the baseness of sin. <xex>Repentance</xex> is a penitent renunciation of, and turning from, sin; thorough repentance produces a new life. <xex>Repentance</xex> is often used as synonymous with <xex>contrition</xex>. See <er>Compunction</er>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contriturate</ent><br/
<hw>Con*trit"u*rate</hw> <pr>(?; 135)</pr>, <pos>v. t.</pos>  <def>To triturate; to pulverize.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrivable</ent><br/
<hw>Con*triv"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being contrived, planned, invented, or devised.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A perpetual motion may seem easily <qex>contrivable</qex>.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrivance</ent><br/
<hw>Con*triv"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or faculty of contriving, inventing, devising, or planning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The machine which we are inspecting demonstrates, by its construction, <qex>contrivance</qex> and design. <qex>Contrivance</qex> must have had a contriver.</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The thing contrived, invented, or planned; disposition of parts or causes by design; a scheme; plan; artifice; arrangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Government is a <qex>contrivance</qex> of human wisdom to provide for human wants.</q> <rj><qau>Burke.</qau></rj></p>

<p><syn><b>Syn.</b> -- Device; plan; scheme; invention; machine; project; design; artifice; shift. See <er>Device</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrive</ent><br/
<hw>Con*trive"</hw> <pr>(k<ocr/n*tr<imac/v")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contrived</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contriving</conjf>.]</vmorph> <ety>[OE. <ets>contriven</ets>, <ets>contreven</ets>, <ets>controven</ets>, to invent, OF. <ets>controver</ets>, <ets>contruver</ets>; <ets>con-</ets> + <ets>trouver</ets> to find. See <er>Troubadour</er>, <er>trover</er>.]</ety> <def>To form by an exercise of ingenuity; to devise; to invent; to design; to plan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What more likely to <qex>contrive</qex> this admirable frame of the universe than infinite wisdom.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>neither do thou imagine that I shall <qex>contrive</qex> aught against his life.</q> <rj><qau>Hawthorne.</qau></rj></p>

<p><syn><b>Syn.</b> -- To invent; discover; plan; design; project; plot; concert; hatch.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrive</ent><br/
<hw>Con*trive"</hw>, <pos>v. i.</pos> <def>To make devices; to form designs; to plan; to scheme; to plot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Fates with traitors do <qex>contrive</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast <qex>contrived</qex> against th very life<br/
Of the defendant.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contrivement</ent><br/
<hw>Con*trive"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Contrivance; invention; arrangement; design; plan.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Consider the admirable <qex>contrivement</qex> and artifice of this great fabric.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Active to meet their <qex>contrivements</qex>.</q> <rj><qau>Sir G. Buck.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contriver</ent><br/
<hw>Con*triv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who contrives, devises, plans, or schemas.</def>
 <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Control</ent><br/
<hw>Con*trol"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>contr<ocir/le</ets> a counter register, contr. fr. <ets>contr-r<ocir/le</ets>; <ets>contre</ets> (L. <ets>contra</ets>) + <ets>r<ocir/le</ets> roll, catalogue. See <er>Counter</er> and <er>Roll</er>, and cf. <er>Counterroll</er>.]</ety> <sn>1.</sn> <def>A duplicate book, register, or account, kept to correct or check another account or register; a counter register.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which serves to check, restrain, or hinder; restraint.</def> <ldquo/Speak without <xex>control</xex>.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Power or authority to check or restrain; restraining or regulating influence; superintendence; government; <as>as, children should be under parental <ex>control</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The House of Commons should exercise a <qex>control</qex> over all the departments of the executive administration.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <def>The complete apparatus used to control a mechanism or machine in operation, as a flying machine in flight;</def> <specif>specifically</specif> <fld>(A<eum/ronautics)</fld>, <def>the mechanism controlling the rudders and ailerons.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn>  <fld>(Climatology)</fld> <def>Any of the physical factors determining the climate of any particular place, as latitude,distribution of land and water, altitude, exposure, prevailing winds, permanent high- or low-barometric-pressure areas, ocean currents, mountain barriers, soil, and vegetation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn>  <fld>(Technology)</fld> <def>in research, an object or subject used in an experimental procedure, which is treated identically to the primary subject of the experiment, except for the omission of the specific treatment or conditions whose effect is being investigated.  If the <ex>control</ex> is a group of living organisms, as is common in medical research, it is called the <col><b>control group</b></col>.</def> <note> For most experimental procedures, the results are not considered valid and reliable unless a proper <ex>control</ex> experiment is performed.  There are various types of <ex>control</ex> used in experimental science, and often several groups of subjects serve as <ex>controls</ex>, being subjected to different variations of the experimental procedure, or controlling for several variables being tested.  When the effects caused by an experimental treatment are not consistent and obvious, statistical analysis of the results is typically used to determine if there are any significant differences between the effects of different experimental conditions.</note><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn>  <fld>(Technology)</fld> <def>the part of an experimental procedure in which the <ex>controls{6}</ex> are subjected to the experimental conditions.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn>  <def>the group of technical specialists exercising control by remote communications over a distant operation, such as a space flight; <as>as, the American Mission <ex>Control</ex> for manned flights is located in Houston</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Board of control</b></col>. <cd>See under <er>Board</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Control</ent><br/
<hw>Con*trol"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Controlled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Controlling</conjf>.]</vmorph> <ety>[F. <ets>contr<ocir/ler</ets>, fr. <ets>contr<ocir/le</ets>.]</ety> <altsp>[Formerly written <asp>comptrol</asp> and <asp>controul</asp>.]</altsp> <sn>1.</sn> <def>To check by a counter register or duplicate account; to prove by counter statements; to confute.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This report was <qex>controlled</qex> to be false.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exercise restraining or governing influence over; to check; to counteract; to restrain; to regulate; to govern; to overpower.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give me a staff of honor for mine age,<br/
But not a scepter to <qex>control</qex> the world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I feel my virtue struggling in my soul:<br/
But stronger passion does its power <qex>control</qex>.</q> <rj><qau>Dryden.</qau></rj></p>

<p><sn>3.</sn> <def>to assure the validity of an experimental procedure by using a <er>control{7}</er>.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- To restrain; rule; govern; manage; guide; regulate; hinder; direct; check; curb; counteract; subdue.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controllability</ent><br/
<hw>Con*trol`la*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Capability of being controlled; controllableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controllable</ent><br/
<hw>Con*trol"la*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being controlled, checked, or restrained; amenable to command.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Passion is the drunkeness of the mind, and, therefore, . . . not always <qex>controllable</qex> by reason.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controllableness</ent><br/
<hw>Con*trol"la*ble*ness</hw>, <pos>n.</pos> <def>Capability of being controlled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controller</ent><br/
<hw>Con*trol"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>control</ets>, v. t.: cf. F. <ets>contr<ocir/leur</ets>.]</ety> <sn>1.</sn> <def>One who, or that which, controls or restraines; one who has power or authority to regulate or control; one who governs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great <qex>controller</qex> of our fate<br/
Deigned to be man, and lived in low estate.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An officer appointed to keep a counter register of accounts, or to examine, rectify, or verify accounts.</def> <altsp>[More commonly written <asp>controller</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>An iron block, usually bolted to a ship's deck, for controlling the running out of a chain cable. The links of the cable tend to drop into hollows in the block, and thus hold fast until disengaged.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Elec.)</fld> <def>Any electric device for controlling a circuit or system;</def> <specif>specif.</specif>: <sd>(a)</sd> <def>An electromagnet, excited by the main current, for throwing a regulator magnet into or out of circuit in an automatic device for constant current regulation.</def> <sd>(b)</sd> <def>A kind of multiple switch for gradually admitting the current to, or shutting it off from, an electric motor; <as>as, a car <ex>controller</ex> for an electric railway car</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn>  <fld>(Mach.)</fld> <def>A lever controlling the speed of an engine; -- applied esp. to the lever governing a throttle valve, as of a steam or gasoline engine, esp. on an automobile.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Controllership</ent><br/
<hw>Con*trol"ler*ship</hw>, <pos>n.</pos> <def>The office of a controller.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controlment</ent><br/
<hw>Con*trol"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The power or act of controlling; the state of being restrained; control; restraint; regulation; superintendence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You may do it without <qex>controlment</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Opposition; resistance; hostility.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here have we war for war, and blood for blood,<br/
<qex>Controlment</qex> for controlment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Control theory</ent><br/
<hw>Con*trol" the`o*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Math., Engineering)</fld> <def>That branch of Mathematics and Engineering which deals with the design, identification and analysis of systems with a view towards controlling them, <it>i. e.</it>, to make them perform specific tasks or make them behave in a desired way.</def><br/
[<source>GG</source>]</p>

<p><ent>Controversal</ent><br/
<hw>Con`tro*ver"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Turning or looking opposite ways.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The temple of Janus, with his two <qex>controversal</qex> faces.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Controversial.</def> <mark>[Obs.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversary</ent><br/
<hw>Con`tro*ver"sa*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Controversial.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controverse</ent><br/
<hw>Con"tro*verse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>controverse</ets>.]</ety> <def>Controversy.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controverse</ent><br/
<hw>Con"tro*verse</hw>, <pos>v. t.</pos> <ety>[L. <ets>controversari</ets>, fr. <ets>controversus</ets> turned against, disputed.]</ety> <def>To dispute; to controvert.</def> <mark>[Obs.]</mark> <ldquo/<xex>Controversed</xex> causes.<rdquo/  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controverser</ent><br/
<hw>Con"tro*ver`ser</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A disputant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversial</ent><br/
<hw>Con`tro*ver"sial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. LL. <ets>controversialis</ets>.]</ety> <def>Relating to, or consisting of, controversy; disputatious; polemical; <as>as, <ex>controversial</ex> divinity</as>.</def></p>

<p><-- 2. arousing controversy; not viewed similarly by everyone. --><br/
[<source>1913 Webster</source>]</p>

<p><q>Whole libraries of <qex>controversial</qex> books.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversialist</ent><br/
<hw>Con`tro*ver"sial*ist</hw>, <pos>n.</pos> <def>One who carries on a controversy; a disputant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Johnson] was both intellectually and morally of the stuff of which <qex>controversialists</qex> are made.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversially</ent><br/
<hw>Con`tro*ver"sial*ly</hw>, <pos>adv.</pos> <def>In a controversial manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversion</ent><br/
<hw>Con`tro*ver"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of controverting; controversy.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversor</ent><br/
<hw>Con"tro*ver`sor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A controverser.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controversy</ent><br/
<hw>Con"tro*ver`sy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Controversies</plw> <pr>(#)</pr></plu>. <ety>[L. <ets>controversia</ets>, fr. <ets>controversus</ets> turned against, disputed; <ets>contro-</ets> = <ets>contra</ets> + <ets>versus</ets>, p. p. of <ets>vertere</ets> to turn. See <er>Verse</er>.]</ety> <sn>1.</sn> <def>Contention; dispute; debate; discussion; agitation of contrary opinions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This left no room for <qex>controversy</qex> about the title.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A dispute is commonly oral, and a <qex>controversy</qex> in writing.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Quarrel; strife; cause of variance; difference.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord hath a <qex>controversy</qex> with the nations.</q> <rj><qau>Jer. xxv. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A suit in law or equity; a question of right.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When any man that had a <qex>controversy</qex> came to the king for judgment.</q> <rj><qau>2 Sam. xv. 2.</qau></rj></p>

<p><syn><b>Syn.</b> -- Dispute; debate; disputation; disagreement; altercation; contention; wrangle; strife; quarrel.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controvert</ent><br/
<hw>Con"tro*vert</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Controverted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Controverting</conjf>.]</vmorph> <ety>[See <er>Controversy</er>.]</ety> <def>To make matter of controversy; to dispute or oppose by reasoning; to contend against in words or writings; to contest; to debate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>controverted</qex> points had decided according to the sense of the best jurists.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controverter</ent><br/
<hw>Con"tro*ver`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who controverts; a controversial writer; a controversialist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>controverters</qex> in divinity are like swaggerers in a tavern.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controvertible</ent><br/
<hw>Con`tro*ver"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being controverted; disputable; admitting of question.</def> -- <wordforms><wf>Con`tro*ver"ti*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Controvertist</ent><br/
<hw>Con"tro*ver`tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in or given to controversy; a controversialist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How unfriendly is the <qex>controvertist</qex> to the discernment of the critic!</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contubernial</ent><br/
<ent>Contubernal</ent><br/
<mhw>{ <hw>Con*tu"ber*nal</hw> <pr>(k<ocr/n*t<umac/"b<etil/r*n<ait/l)</pr>, <hw>Con`tu*ber"ni*al</hw> <pr>(k<ocr/n`t<usl/*b<etil/r"n<icr/*<ait/l)</pr> }</mhw>, <pos>a.</pos> <ety>[L. <ets>contubernalis</ets> a tent companion, fr. <ets>contubernium</ets> tent companionship.]</ety> <def>Living or messing together; familiar; in companionship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Humble folk ben Christes friends: they ben <qex>contubernial</qex> with the Lord, thy King.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contumacious</ent><br/
<hw>Con`tu*ma"cious</hw> <pr>(k<ocr/n`t<usl/*m<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>contumax</ets>, <ets>-acis</ets>. See <er>Contumacy</er>.]</ety> <sn>1.</sn> <def>Exhibiting contumacy; contemning authority; obstinate; perverse; stubborn; disobedient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is another very, efficacious method for subduing the most obstinate, <qex>contumacious</qex> sinner.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Willfully disobedient to the summons or orders of a court.</def>  <rj><au>Blackstone.</au></rj></p>

<p><syn><b>Syn.</b> -- Stubborn; obstinate; obdurate; disobedient; perverse; unyielding; headstrong.</syn></p>

<p> -- <wordforms><wf>Con`tu*ma"cious*ly</wf>, <pos>adv.</pos> -- <wf>Con`tu*ma"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contumacy</ent><br/
<hw>Con"tu*ma*cy</hw> <pr>(k<ocr/n"t<usl/*m<adot/*s<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Contumacies</plw> <pr>(k<ocr/n"t<usl/*m<adot/*s<icr/z)</pr></plu>. <ety>[L. <ets>contumacia</ets>, fr. <ets>contumax</ets>, <ets>-acis</ets>, insolent; prob. akin to <ets>contemnere</ets> to despise: cf. F. <ets>contumace</ets>. Cf. <er>Contemn</er>.]</ety> <sn>1.</sn> <def>Stubborn perverseness; pertinacious resistance to authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bishop commanded him . . . to be thrust into the stocks for his manifest and manifold <qex>contumacy</qex>.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>A willful contempt of, and disobedience to, any lawful summons, or to the rules and orders of court, as a refusal to appear in court when legally summoned.</def></p>

<p><syn><b>Syn.</b> -- Stubbornness; perverseness; obstinacy.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contumelious</ent><br/
<hw>Con`tu*me"li*ous</hw> <pr>(?<or/ ?; 106)</pr>, <pos>a.</pos> <ety>[L. <ets>contumeliosus</ets>.]</ety> <sn>1.</sn> <def>Exhibiting contumely; rudely contemptuous; insolent; disdainful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Scoffs, and scorns, and <qex>contumelious</qex> taunts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Curving a <qex>contumelious</qex> lip.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shameful; disgraceful.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj></p>

<p>-- <wordforms><wf>Con`tu*me"li*ous*ly</wf>, <pos>adv.</pos> -- <wf>Con`tu*me"li*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contumely</ent><br/
<hw>Con"tu*me*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>contumelia</ets>, prob. akin to <ets>contemnere</ets> to despise: cf. OF. <ets>contumelie</ets>. Cf. <er>Contumacy</er>.]</ety> <def>Rudeness compounded of haughtiness and contempt; scornful insolence; despiteful treatment; disdain; contemptuousness in act or speech; disgrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The oppressor's wrong, the proud man's <qex>contumely</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing aggravates tyranny so much as <qex>contumely</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Contuse</ent><br/
<hw>Con*tuse"</hw> <pr>(k<ocr/n*t<umac/z")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Contused</conjf> <pr>(k<ocr/n*t<umac/zd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Contusing</conjf>.]</vmorph> <ety>[L. <ets>contusus</ets>, p. p. of <ets>contundere</ets> to beat, crush; <ets>con-</ets> + <ets>tundere</ets> to beat, akin to Skr. <ets>tud</ets> (for <ets>stud</ets>) to strike, Goth. <ets>stautan</ets>. See <er>Stutter</er>.]</ety> <sn>1.</sn> <def>To beat, pound, or bray together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Roots, barks, and seeds <qex>contused</qex> together.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bruise; to injure or disorganize a part without breaking the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Contused wound</b></col>, <cd>a wound attended with bruising.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>contusioned</ent><br/
<ent>contused</ent><br/
<mhw><hw>contused</hw> <hw>contusioned</hw></mhw> <pos>adj.</pos>  <def>injured without breaking the skin; <as>as, a cut arm and <ex>contused</ex> cheek</as>.</def><br/
<syn><b>Syn. --</b> bruised.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Contusion</ent><br/
<hw>Con*tu"sion</hw> <pr>(k<ocr/n*t<umac/"zh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>contusio</ets>: cf. F. <ets>contusion</ets>.]</ety> <sn>1.</sn> <def>The act or process of beating, bruising, or pounding; the state of being beaten or bruised.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A bruise; an injury attended with more or less disorganization of the subcutaneous tissue and effusion of blood beneath the skin, but without apparent wound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conundrum</ent><br/
<hw>Co*nun"drum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Origin unknown.]</ety> <sn>1.</sn> <def>A kind of riddle based upon some fanciful or fantastic resemblance between things quite unlike; a puzzling question, of which the answer is or involves a pun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or pun ambiguous, or <qex>conundrum</qex> quaint.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A question to which only a conjectural answer can be made.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do you think life is long enough to let me speculate on <qex>conundrums</qex> like that?</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conure</ent><br/
<hw>Co*nure"</hw> <pr>(k<osl/*n<umac/r")</pr>, <pos>n.</pos> <ety>[NL. <ets>conurus</ets>, fr. Gr. <grk>kw^nos</grk> a cone + <grk>o'yra`</grk> tail.  The name alludes to the tapering tail.]</ety> <fld>(Zool.)</fld> <def>An American parrakeet of the genus <gen>Conurus</gen>.  Many species are known.  See <er>Parrakeet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conus</ent><br/
||<hw>Co"nus</hw> <pr>(k<omac/"n<ucr/s)</pr>, <pos>n.</pos> <ety>[L., a cone.]</ety> <sn>1.</sn> <def>A cone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A Linnean genus of mollusks having a conical shell. See <er>Cone</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conusable</ent><br/
<hw>Con"u*sa*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cognizable; liable to be tried or judged.</def> <mark>[Obs.]</mark> <rj><au>Bp. Barlow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conusant</ent><br/
<hw>Con"u*sant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Law)</fld> <def>See <er>Cognizant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conusor</ent><br/
<hw>Con`u*sor"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>See <er>Cognizor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convalesce</ent><br/
<hw>Con`va*lesce"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convalesced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convalescing</conjf>.]</vmorph> <ety>[L. <ets>convalscere</ets>; <ets>con-</ets> + <ets>valescere</ets> to grow strong, v. incho. of <ets>valere</ets> to be strong. See <er>Valiant</er>.]</ety> <def>To recover health and strength gradually, after sickness or weakness; <as>as, a patient begins to <ex>convalesce</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convalesced</ent><br/
<hw>Con`va*lesced"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Convalescent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He found the queen somewhat <qex>convalesced</qex>.</q> <rj><qau>J. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convalescency</ent><br/
<ent>Convalescence</ent><br/
<mhw>{ <hw>Con`va*les"cence</hw> <pr>(?)</pr>, <hw>Con`va*les"cen*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>convalescentia</ets>: cf. F. <ets>convalescence</ets>.]</ety> <def>The recovery of heath and strength after disease; the state of a body renewing its vigor after sickness or weakness; the time between the subsidence of a disease and complete restoration to health.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convalescent</ent><br/
<hw>Con`va*les"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>convalescens</ets>, <ets>-entis</ets>, p. pr.: cf. F. <ets>convalescent</ets>.]</ety> <sn>1.</sn> <def>Recovering from sickness or debility; partially restored to health or strength.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to convalescence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convalescent</ent><br/
<hw>Con`va*les"cent</hw>, <pos>n.</pos> <def>One recovering from sickness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convalescently</ent><br/
<hw>Con`va*les"cent*ly</hw>, <pos>adv.</pos> <def>In the manner of a convalescent; with increasing strength or vigor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convallariaceae</ent><br/
<hw>Convallariaceae</hw> <pos>prop. n.</pos> <def>One of many subfamilies into which some classification systems subdivide the <fam>Liliaceae</fam> but not widely accepted.</def><br/
<syn><b>Syn. --</b> family <fam>Convallariaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><-- p. 317 --></p>

<p><ent>Convallamarin</ent><br/
<hw>Con*val`la*ma"rin</hw> <pr>(k<ocr/n*v<acr/l`l<adot/*m<amac/"r<icr/n)</pr>, <pos>n.</pos> <ety>[<ets>Convallaria</ets> + L. <ets>amarus</ets> bitter.]</ety> <fld>(Chem.)</fld> <def>A white, crystalline, poisonous substance, regarded as a glucoside, extracted from the lily of the valley (<spn>Convallaria Majalis</spn>). Its taste is first bitter, then sweet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convallaria</ent><br/
||<hw>Con`val*la"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>convallis</ets> a valley; <ets>con-</ets> + <ets>vallis</ets> valley.]</ety> <fld>(Bot. & Med.)</fld> <def>The lily of the valley.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convallarin</ent><br/
<hw>Con`val*la"rin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white, crystalline glucoside, of an irritating taste, extracted from the convallaria or lily of the valley.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>convect</ent><br/
<hw>convect</hw> <pos>v. i. & t.</pos> <def>to circulate hot air by convection, as of certain ovens.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Convection</ent><br/
<hw>Con*vec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>convectio</ets>, fr. <ets>convehere</ets> to bring together; <ets>con-</ets> + <ets>vehere</ets> to carry.]</ety> <sn>1.</sn> <def>The act or process of conveying or transmitting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physics)</fld> <def>A process of transfer or transmission, as of heat or electricity, by means of currents in liquids or gases, resulting from changes of temperature and other causes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Liquids are generally heated by <qex>convection</qex> -- when heat is applied from below.</q> <rj><qau>Nichol.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convective</ent><br/
<hw>Con*vec"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Caused or accomplished by convection; <as>as, a <ex>convective</ex> discharge of electricity</as>.</def>  <rj><au>Faraday.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convectively</ent><br/
<hw>Con*vec"tive*ly</hw>, <pos>adv.</pos> <def>In a convective manner.</def>  <rj><au>Hare.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convellent</ent><br/
<hw>Con*vel"lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>convellens</ets>, p. pr. of <ets>convellere</ets>. See <er>Convulse</er>.]</ety> <def>Tending to tear or pull up.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The ends of the fragment . . . will not yield to the <qex>convellent</qex> force.</q> <rj><qau>Todd & Bowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convenable</ent><br/
<hw>Con*ven"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being convened or assembled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convenable</ent><br/
<hw>Con"ve*na*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>convenable</ets>, fr. <ets>convenir</ets>. See <er>Convene</er>.]</ety> <def>Consistent; accordant; suitable; proper; <as>as, <ex>convenable</ex> remedies</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With his wod his work is <qex>convenable</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convenance</ent><br/
<hw>Con"ve*nance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fitness, suitableness.]</ety> <def>That which is suitable, agreeable, or convenient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And they missed<br/
Their wonted <qex>convenance</qex>, cheerly hid the loss.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convene</ent><br/
<hw>Con*vene"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convenong</conjf>.]</vmorph> <ety>[L. <ets>convenire</ets>; <ets>con-</ets> + <ets>venire</ets> to come: cf. F. <ets>convenir</ets> to agree, to be fitting, OF. also, to assemble. See <er>Come</er>, and cf. <er>Covenant</er>.]</ety> <sn>1.</sn> <def>To come together; to meet; to unite.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In shortsighted men . . . the rays converge and <qex>convene</qex> in the eyes before they come at the bottom.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To come together, as in one body or for a public purpose; to meet; to assemble.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Parliament of Scotland now <qex>convened</qex>.</q> <rj><qau>Sir R. Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Faint, underneath, the household fowls <qex>convene</qex>.</q> <rj><qau>Thomson.</qau></rj></p>

<p><syn><b>Syn.</b> -- To meet; to assemble; to congregate; to collect; to unite.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convene</ent><br/
<hw>Con*vene"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to assemble; to call together; to convoke.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And now the almighty father of the gods<br/
<qex>Convenes</qex> a council in the blest abodes.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To summon judicially to meet or appear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By the papal canon law, clerks . . . can not be <qex>convened</qex> before any but an ecclesiastical judge.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convener</ent><br/
<hw>Con*ven"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who convenes or meets with others.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who calls an assembly together or convenes a meeting; hence, the chairman of a committee or other organized body.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conveniency</ent><br/
<ent>Convenience</ent><br/
<mhw>{ <hw>Con*ven"ience</hw> <pr>(?; 106)</pr>, <hw>Con*ven"ien*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>convenientia</ets> agreement, fitness. See <er>Convenient</er>.]</ety> <sn>1.</sn> <def>The state or quality of being convenient; fitness or suitableness, as of place, time, etc.; propriety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let's further think of this;<br/
Weigh what <qex>convenience</qex> both of time and means<br/
May fit us to our shape.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With all brief and plain <qex>conveniency</qex>,<br/
Let me have judgment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Freedom from discomfort, difficulty, or trouble; commodiousness; ease; accommodation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus necessity invented stools,<br/
<qex>Convenience</qex> next suggested elbow chairs.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We are rather intent upon the end of God's glory than our own <qex>conveniency</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is convenient; that which promotes comfort or advantage; that which is suited to one's wants; an accommodation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A pair of spectacles and several other little <qex>conveniences</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A convenient or fit time; opportunity; <as>as, to do something at one's <ex>convenience</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conveniences</ent><br/
<hw>conveniences</hw> <pos>n.</pos> <def>things that make one comfortable and at ease.</def><br/
<syn><b>Syn. --</b> comforts, creature comforts, amenities.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Convenient</ent><br/
<hw>Con*ven"ient</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>conveniens</ets>, <ets>-entis</ets>, suitable, p. pr. of <ets>convenire</ets> to be suitable, to come. See <er>Convene</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Fit or adapted; suitable; proper; becoming; appropriate.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Feed me with food <qex>convenient</qex> for me.</q> <rj><qau>Prov. xxx. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither filthiness, nor foolish talking, nor jesting, which are not <qex>convenient</qex>.</q> <rj><qau>Eph. v. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affording accommodation or advantage; well adapted to use; handly; <as>as, a <ex>convenient</ex> house; <ex>convenient</ex> implements or tools</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Seasonable; timely; opportune; <as>as, a <ex>convenient</ex> occasion; a <ex>convenient</ex> season</as>.</def>  <rj><au>Acts xxiv. 25.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Near at hand; easy of access.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hereties used to be brought thither, <qex>convenient</qex> for burning.</q> <rj><qau>Thackeray.</qau></rj></p>

<p><syn><b>Syn.</b> -- Fit; suitable; proper; adapted; fitted; suited; handly; commodious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conveniently</ent><br/
<hw>Con*ven"ient*ly</hw>, <pos>adv.</pos> <def>In a convenient manner, form, or situation; without difficulty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convent</ent><br/
<hw>Con"vent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conventus</ets> a meeting, LL. also, a convent. See <er>Convene</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A coming together; a meeting.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A usual ceremony at their [the witches] <qex>convents</qex> or meetings.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An association or community of recluses devoted to a religious life; a body of monks or nuns.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One of our <qex>convent</qex>, and his [the duke's] confessor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A house occupied by a community of religious recluses; a monastery or nunnery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One seldom finds in Italy a spot of ground more agreeable than ordinary that is not covered with a <qex>convent</qex>.</q> <rj><qau>Addison.</qau></rj></p>

<p><syn><b>Syn.</b> -- Nunnery; monastery; abbey. See <er>Cloister</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convent</ent><br/
<hw>Con*vent"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>conventus</ets>, p. p. of <ets>convenire</ets>. See <er>Convene</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To meet together; to concur.</def> <mark>[obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be convenient; to serve.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When that is known and golden time <qex>convents</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convent</ent><br/
<hw>Con*vent"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To call before a judge or judicature; to summon; to convene.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventical</ent><br/
<hw>Con*vent"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or from, or pertaining to, a convent.</def> <ldquo/<xex>Conventical</xex> wages.<rdquo/  <rj><au>Sterne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conventical prior</b></col>. <cd>See <er>Prior</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventicle</ent><br/
<hw>Con*ven"ti*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conventiculum</ets>, dim. of <ets>conventus</ets>: cf. F. <ets>conventicule</ets>. See <er>Convent</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A small assembly or gathering; esp., a secret assembly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are commanded to abstain from all <qex>conventicles</qex> of men whatsoever.</q>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assembly for religious worship; esp., such an assembly held privately, as in times of persecution, by Nonconformists or Dissenters in England, or by Covenanters in Scotland; -- often used opprobriously, as if those assembled were heretics or schismatics.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first Christians could never have had recourse to nocturnal or clandestine <qex>conventicles</qex> till driven to them by the violence of persecution.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A sort of men who . . . attend its [the curch of England's] service in the morning, and go with their wives to a <qex>conventicle</qex> in the afternoon.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventicler</ent><br/
<hw>Con*ven"ti*cler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who supports or frequents conventicles.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventicling</ent><br/
<hw>Con*ven"ti*cling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging or going to, or resembling, a conventicle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conventicling</qex> schools . . . set up and taught secretly by fanatics.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convention</ent><br/
<hw>Con*ven"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conventio</ets>: cf. F. <ets>convention</ets>. See <er>Convene</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>The act of coming together; the state of being together; union; coalition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conventions</qex> or associations of several particles of matter into bodies of any certain denomination.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>General agreement or concurrence; arbitrary custom; usage; conventionality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There are thousands now<br/
Such women, but <qex>convention</qex> beats them down.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A meeting or an assembly of persons, esp. of delegates or representatives, to accomplish some specific object, -- civil, social, political, or ecclesiastical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He set himself to the making of good laws in a grand <qex>convention</qex> of his nobles.</q> <rj><qau>Sir R. Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>convention</qex> of delegates from all the States, to meet in Philadelphia, for the sole and express purpose of reserving the federal system, and correcting its defects.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eng. Hist)</fld> <def>An extraordinary assembly of the parkiament or estates of the realm, held without the king's writ, -- as the assembly which restored Charles II. to the throne, and that which declared the throne to be abdicated by James II.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our gratitude is due . . . to the Long Parliament, to the <qex>Convention</qex>, and to William of Orange.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An agreement or contract less formal than, or preliminary to, a treaty; an informal compact, as between commanders of armies in respect to suspension of hostilities, or between states; also, a formal agreement between governments or sovereign powers; <as>as, a postal <ex>convention</ex> between two governments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>convention</qex>, I think from my soul, is nothing but a stipulation for national ignominy; a truce without a suspension of hostilities.</q> <rj><qau>Ld. Chatham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>convention</qex> with the State of Georgia has been ratified by their Legislature.</q> <rj><qau>T. Jefferson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventional</ent><br/
<hw>Con*ven"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conventionalis</ets>: cf. F. <ets>conventionnel</ets>.]</ety> <sn>1.</sn> <def>Formed by agreement or compact; stipulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conventional</qex> services reserved by tenures upon grants, made out of the crown or knights' service.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Growing out of, or depending on, custom or tacit agreement; sanctioned by general concurrence or usage; formal.</def> <ldquo/<xex>Conventional</xex> decorum.<rdquo/  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conventional</qex> language appropriated to monarchs.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ordinary salutations, and other points of social behavior, are <qex>conventional</qex>.</q> <rj><qau>Latham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fine Arts)</fld> <sd>(a)</sd> <def>Based upon tradition, whether religious and historical or of artistic rules.</def> <sd>(b)</sd> <def>Abstracted; removed from close representation of nature by the deliberate selection of what is to be represented and what is to be rejected; <as>as, a <ex>conventional</ex> flower; a <ex>conventional</ex> shell.</as>  Cf. <er>Conventionalize</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>conventionalise</ent><br/
<hw>conventionalise</hw> <pos>v.</pos> <def>to make conventional.</def><br/
<syn><b>Syn. --</b> conventionalize.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conventionalism</ent><br/
<hw>Con*ven"tion*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which is received or established by convention or arbitrary agreement; that which is in accordance with the fashion, tradition, or usage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the artifice and <qex>conventionalism</qex> of life.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They gaze on all with dead, dim eyes, -- wrapped in <qex>conventionalisms</qex>, . . . simulating feelings according to a received standard.</q> <rj><qau>F. W. Robertson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fine Arts)</fld> <def>The principles or practice of conventionalizing. See <er>Conventionalize</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionalist</ent><br/
<hw>Con*ven"tion*al*ist</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who adheres to a convention or treaty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is governed by conventionalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionality</ent><br/
<hw>Con*ven`tion*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conventionalities</plw> <pr>(<?/)</pr>.</plu> <def>The state of being conventional; adherence to social formalities or usages; that which is established by conventional use; one of the customary usages of social life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionalization</ent><br/
<hw>Con*ven`tion*al*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Fine Arts)</fld> <sd>(a)</sd> <def>The act of making conventional.</def> <sd>(b)</sd> <def>The state of being conventional.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionalize</ent><br/
<hw>Con*ven"tion*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conventionalized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conventionalizing</conjf>.]</vmorph> <sn>1.</sn> <def>To make conventional; to bring under the influence of, or cause to conform to, conventional rules; to establish by usage.</def> <altsp>[Also spelled <asp>conventionalise</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fine Arts)</fld> <sd>(a)</sd> <def>To represent by selecting the important features and those which are expressible in the medium employed, and omitting the others.</def> <sd>(b)</sd> <def>To represent according to an established principle, whether religious or traditional, or based upon certain artistic rules of supposed importance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionalize</ent><br/
<hw>Con*ven"tion*al*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <fld>(Fine Arts)</fld> <def>To make designs in art, according to conventional principles. Cf. <er>Conventionalize</er>, <pos>v. t.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionally</ent><br/
<hw>Con*ven"tion*al*ly</hw>, <pos>adv.</pos> <def>In a conventional manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionary</ent><br/
<hw>Con*ven"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acting under contract; settled by express agreement; <as>as, <ex>conventionary</ex> tenants</as>.</def> <mark>[Obs.]</mark>  <rj><au>R. Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventioner</ent><br/
<hw>Con*ven"tion*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who belongs to a convention or assembly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventionist</ent><br/
<hw>Con*ven"tion*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who enters into a convention, covenant, or contract.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventual</ent><br/
<hw>Con*ven"tu*al</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[LL. <ets>conventualis</ets>: cf. F. <ets>conventuel</ets>.]</ety> <def>Of or pertaining to a convent; monastic.</def> <ldquo/A <xex>conventual</xex> garb.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conventual church</b></col>, <cd>a church attached or belonging to a convent or monastery.</cd>  <rj><au>Wordsworth.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conventual</ent><br/
<hw>Con*ven"tu*al</hw>, <pos>n.</pos> <def>One who lives in a convent; a monk or nun; a recluse.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converge</ent><br/
<hw>Con*verge"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Converged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Converging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Pref. <ets>con-</ets> + L. <ets>vergere</ets> to turn, incline; cf. F. <ets>converger</ets>. See <er>Verge</er>, <pos>v. i.</pos>]</ety> <def>To tend to one point; to incline and approach nearer together; <as>as, lines <ex>converge</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mountains <qex>converge</qex> into a single ridge.</q> <rj><qau>Jefferson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converge</ent><br/
<hw>Con*verge"</hw>, <pos>v. t.</pos> <def>To cause to tend to one point; to cause to incline and approach nearer together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>converge</qex> its rays to a focus of dazzling brilliancy.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>


<p><ent>Convergency</ent><br/
<ent>Convergence</ent><br/
<mhw>{ <hw>Con*ver"gence</hw> <pr>(?)</pr>, <hw>Con*ver"gen*cy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Cf. F. <ets>convergence</ets>.]</ety> <sn>1.</sn> <def>The condition or quality of converging; tendency to one point; the occurrence of two or more things coming together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>the approach of an infinite series to a finite limit.</def><br/
<syn><b>Syn. --</b> convergency.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>a representation of common ground between theories or phenomena.</def><br/
<syn><b>Syn. --</b> overlap, intersection.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn> <def>the act of converging (coming closer).</def><br/
<syn><b>Syn. --</b> converging, convergency.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>5.</sn> <fld>(Biol.)</fld> <def>a similarity of form or function in two or more organisms caused by evolutionary adaptations to a similarity in the environment, rather than to a common heredity.</def><br/
[<source>PJC</source>]</p>

<p><q>The <qex>convergence</qex> or divergence of the rays falling on the pupil.</q> <rj><qau>Berkeley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convergent</ent><br/
<hw>Con*ver"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>convergent</ets>.]</ety> <def>tending to one point of focus; tending to approach each other; converging.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As many rays of light, as conveniently can be let in, and made <qex>convergent</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The vast dome of its cathedral . . . directing its <qex>convergent</qex> curves to heaven.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converging</ent><br/
<hw>Con*ver"ging</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to one point; approaching each other; convergent; <as>as, <ex>converging</ex> lines</as>.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Converging rays</b></col><fld>(Opt.)</fld>, <cd>rays of light, which, proceeding from different points of an object, tend toward a single point.</cd> -- <col><b>Converging series</b></col> <fld>(Math.)</fld>, <cd>a series in which if an indefinitely great number of terms be taken, their sum will become indefinitely near in value to a fixed quantity, which is called the <xex>sum of the series</xex>; -- opposed to a <contr>diverging</contr> series.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversable</ent><br/
<hw>Con*vers"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>conversable</ets>.]</ety> <def>Qualified for conversation; disposed to converse; sociable; free in discourse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While young, humane, <qex>conversable</qex>, and kind.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversableness</ent><br/
<hw>Con*vers"a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being conversable; disposition to converse; sociability.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversably</ent><br/
<hw>Con*vers"a*bly</hw>, <pos>adv.</pos> <def>In a conversable manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversance</ent><br/
<hw>Con"ver*sance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being conversant; habit of familiarity; familiar acquaintance; intimacy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversancy</ent><br/
<hw>Con"ver*san*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Conversance</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversant</ent><br/
<hw>Con"ver*sant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>conversans</ets>, p. pr. of <ets>conversari</ets>: cf. F. <ets>conversant</ets>.]</ety> <sn>1.</sn> <def>Having frequent or customary intercourse; familiary associated; intimately acquainted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have been <qex>conversant</qex> with the first persons of the age.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Familiar or acquainted by use or study; well-informed; versed; -- generally used with <xex>with</xex>, sometimes with <xex>in</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Deeply <qex>conversant</qex> in the Platonic philosophy.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>he uses the different dialects as one who had been <qex>conversant</qex> with them all.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conversant</qex> only with the ways of men.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Concerned; occupied.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Education . . . is <qex>conversant</qex> about children.</q> <rj><qau>W. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversant</ent><br/
<hw>Con*vers"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who converses with another; a convenser.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversantly</ent><br/
<hw>Con"ver*sant*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a familiar manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversation</ent><br/
<hw>Con`ver*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>conversacio</ets> (in senses 1 & 2), OF. <ets>conversacion</ets>, F. <ets>conversation</ets>, fr. L. <ets>conversatio</ets> frequent abode in a place, intercourse, LL. also, manner of life.]</ety> <sn>1.</sn> <def>General course of conduct; behavior.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let your <qex>conversation</qex> be as it becometh the gospel.</q> <rj><qau>Philip. i. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Familiar intercourse; intimate fellowship or association; close acquaintance.</def> <ldquo/<xex>Conversation</xex> with the best company.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I set down, out of long experience in business and much <qex>conversation</qex> in books, what I thought pertinent to this business.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Commerce; intercourse; traffic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All traffic and mutual <qex>conversation</qex>.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Colloquial discourse; oral interchange of sentiments and observations; informal dialogue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The influence exercised by his [Johnson's] <qex>conversation</qex> was altogether without a parallel.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Sexual intercourse; <as>as, criminal <ex>conversation</ex></as>.</def></p>

<p><syn><b>Syn.</b> -- Intercourse; communion; commerce; familiarity; discourse; dialogue; colloquy; talk; chat.</syn> <usage> -- <er>Conversation</er>, <er>Talk</er>. There is a looser sense of these words, in which they are synonymous; there is a stricter sense, in which they differ. <xex>Talk</xex> is usually broken, familiar, and versatile. <xex>Conversation</xex> is more continuous and sustained, and turns ordinarily upon topics or higher interest. Children <xex>talk</xex> to their parents or to their companions; men <xex>converse</xex> together in mixed assemblies. Dr. Johnson once remarked, of an evening spent in society, that there had been a great deal of <xex>talk</xex>, but no <xex>conversation</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 318 --></p>

<p><ent>Conversational</ent><br/
<hw>Con`ver*sa"tion*al</hw> <pr>(k<ocr/n`v<etil/r*s<amac/"sh<ucr/n*<ait/l)</pr>, <pos>a.</pos> <def>Pertaining to conversation; in the manner of one conversing; <as>as, a <ex>conversational</ex> style</as>.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversationalist</ent><br/
<hw>Con`ver*sa"tion*al*ist</hw>, <pos>n.</pos> <def>A conversationist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversationed</ent><br/
<hw>Conver*sa"tioned</hw> <pr>(-sh<ucr/nd)</pr>, <pos>a.</pos> <def>Aaquaintea with m`nners aod deportment; behaved.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Till she be better <qex>conversationed</qex>, . . . I'll keep<br/
As far from her as the gallows.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversationism</ent><br/
<hw>Con`ver*sa"tion*ism</hw> <pr>(-<icr/z'm)</pr>, <pos>n.</pos> <def>A word or phrase used in conversation; a colloquialism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversationist</ent><br/
<hw>Con`ver*sa"tion*ist</hw>, <pos>n.</pos> <def>One who converses much, or who excels in conversation.</def>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversative</ent><br/
<hw>Con*ver"sa*tive</hw> <pr>(k<ocr/n*v<etil/r"s<adot/*t<icr/v)</pr>, <pos>a.</pos> <def>Relating to intercourse with men; social; -- opposed to <xex>contemplative</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She chose . . . to endue him with the <qex>conversative</qex> qualities of youth.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversazione</ent><br/
||<hw>Con`ver*sa`zi*o"ne</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Conversazioni</plw> <pr>(#)</pr></plu>. <ety>[It. See <er>Conversation</er>.]</ety> <def>A meeting or assembly for conversation, particularly on literary or scientific subjects.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>conversazioni</qex> [at Florence] resemble our card assemblies.</q> <rj><qau>A. Drummond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converse</ent><br/
<hw>Con*verse"</hw> <pr>(k<ocr/n*v<etil/rs")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conversed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conversing</conjf>.]</vmorph> <ety>[F. <ets>converser</ets>, L. <ets>conversari</ets> to associate with; <ets>con-</ets> + <ets>versari</ets> to be turned, to live, remain, fr. <ets>versare</ets> to turn often, v. intens. of <ets>vertere</ets> to turn See <er>Convert</er>.]</ety> <sn>1.</sn> <def>To keep company; to hold intimate intercourse; to commune; -- followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To seek the distant hills, and there <qex>converse</qex><br/
With nature.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conversing</qex> with the world, we use the world's fashions.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But to <qex>converse</qex> with heaven -<br/
This is not easy.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To engage in familiar colloquy; to interchange thoughts and opinions in a free, informal manner; to chat; -- followed by <xex>with</xex> before a person; by <xex>on</xex>, <xex>about</xex>, <xex>concerning</xex>, etc., before a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Companions<br/
That do <qex>converse</qex> and waste the time together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We had <qex>conversed</qex> so often on that subject.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have knowledge of, from long intercourse or study; -- said of things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>According as the objects they <qex>converse</qex> with afford greater or less variety.</q> <rj><qau>Locke.</qau></rj></p>

<p><syn><b>Syn.</b> -- To associate; commune; discourse; talk; chat.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converse</ent><br/
<hw>Con"verse</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Frequent intercourse; familiar communion; intimate association.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is but to hold<br/
<qex>Converse</qex> with Nature's charms, and view her stores unrolled.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Familiar discourse; free interchange of thoughts or views; conversation; chat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Formed by thy <qex>converse</qex> happily to steer<br/
From grave to gay, from lively to severe.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converse</ent><br/
<hw>Con"verse</hw>, <pos>a.</pos> <ety>[L. <ets>conversus</ets>, p. p. of <ets>convertere</ets>. See <er>Convert</er>.]</ety> <def>Turned about; reversed in order or relation; reciprocal; <as>as, a <ex>converse</ex> proposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converse</ent><br/
<hw>Con"verse</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Logic)</fld> <def>A proposition which arises from interchanging the terms of another, as by putting the predicate for the subject, and the subject for the predicate; <as>as, no virtue is vice, no vice is virtue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It should not (as is often done) be confounded with the <xex>contrary</xex> or <xex>opposite</xex> of a proposition, which is formed by introducing the negative <xex>not</xex> or <xex>no</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>A proposition in which, after a conclusion from something supposed has been drawn, the order is inverted, making the conclusion the supposition or premises, what was first supposed becoming now the conclusion or inference. Thus, if two sides of a sides of a triangle are equal, the angles opposite the sides are equal; and the <xex>converse</xex> is true, <it>i.e.</it>, if these angles are equal, the two sides are equal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversely</ent><br/
<hw>Con"verse*ly</hw> <pr>(? <or/ <?/; 277)</pr>, <pos>adv.</pos> <def>In a converse manner; with change of order or relation; reciprocally.</def>  <rj><au>J. S. Mill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converser</ent><br/
<hw>Con*vers"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who engages in conversation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversible</ent><br/
<hw>Con*ver"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being converted or reversed.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversion</ent><br/
<hw>Con*ver"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conversio</ets>: cf. F. <ets>conversion</ets>. See <er>Convert</er>.]</ety> <sn>1.</sn> <def>The act of turning or changing from one state or condition to another, or the state of being changed; transmutation; change.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Artificial <qex>conversion</qex> of water into ice.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>conversion</qex> of the aliment into fat.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of changing one's views or course, as in passing from one side, party, or from of religion to another; also, the state of being so changed.</def> <ldquo/<xex>Conversion</xex> to Christianity.<rdquo/  <rj><au>Prescott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>An appropriation of, and dealing with the property of another as if it were one's own, without right; <as>as, the <ex>conversion</ex> of a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or bring my action of <qex>conversion</qex><br/
And trover for my goods.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Logic)</fld> <def>The act of interchanging the terms of a proposition, as by putting the subject in the place of the predicate, or the contrary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Math.)</fld> <def>A change or reduction of the form or value of a proposition; <as>as, the <ex>conversion</ex> of equations; the <ex>conversion</ex> of proportions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A change of front, as a body of troops attacked in the flank.</def> <sd>(b)</sd> <def>A change of character or use, as of smoothbore guns into rifles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Theol.)</fld> <def>A spiritual and moral change attending a change of belief with conviction; a change of heart; a change from the service of the world to the service of God; a change of the ruling disposition of the soul, involving a transformation of the outward life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He oft<br/
Frequented their assemblies, . . . and to them preached<br/
<qex>Conversion</qex> and repentance, as to souls<br/
In prison under judgments imminent.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conversive</ent><br/
<hw>Con*ver"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being converted or changed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ready to converse; social.</def> <mark>[Archaic]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convert</ent><br/
<hw>Con*vert"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Converted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Converting</conjf>.]</vmorph> <ety>[L. <ets>convertere</ets>, <ets>-versum</ets>; <ets>con-</ets> + <ets>vertere</ets> to turn: cf. F. <ets>convertir</ets>. See <er>Verse</er>.]</ety> <sn>1.</sn> <def>To cause to turn; to turn.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O, which way shall I first <qex>convert</qex> myself?</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change or turn from one state or condition to another; to alter in form, substance, or quality; to transform; to transmute; <as>as, to <ex>convert</ex> water into ice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the whole atmosphere were <qex>converted</qex> into water.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That still lessens<br/
The sorrow, and <qex>converts</qex> it nigh to joy.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To change or turn from one belief or course to another, as from one religion to another or from one party or sect to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No attempt was made to <qex>convert</qex> the Moslems.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To produce the spiritual change called conversion in (any one); to turn from a bad life to a good one; to change the heart and moral character of (any one) from the controlling power of sin to that of holiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He which <qex>converteth</qex> the sinner from the error of his way shall save a soul from death.</q> <rj><qau>Lames v. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To apply to any use by a diversion from the proper or intended use; to appropriate dishonestly or illegally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When a bystander took a coin to get it changed, and <qex>converted</qex> it, [it was] held no larceny.</q> <rj><qau>Cooley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To exchange for some specified equivalent; <as>as, to <ex>convert</ex> goods into money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Logic)</fld> <def>To change (one proposition) into another, so that what was the subject of the first becomes the predicate of the second.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To turn into another language; to translate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which story . . . Catullus more elegantly <qex>converted</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Converted guns</b></col>, <cd>cast-iron guns lined with wrought-iron or steel tubes.</cd> <au>Farrow.</au> -- <col><b>Converting furnace</b></col> <fld>(Steel Manuf.)</fld>, <cd>a furnace in which wrought iron is converted into steel by cementation.</cd></cs></p>

<p><syn><b>Syn.</b> -- To change; turn; transmute; appropriate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convert</ent><br/
<hw>Con*vert"</hw>, <pos>v. i.</pos> <def>To be turned or changed in character or direction; to undergo a change, physically or morally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If Nebo had had the preaching that thou hast, they [the Neboites] would have <qex>converted</qex>.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A red dust which <qex>converth</qex> into worms.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The public hope<br/
And eye to thee <qex>converting</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convert</ent><br/
<hw>Con"vert</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person who is converted from one opinion or practice to another; a person who is won over to, or heartily embraces, a creed, religious system, or party, in which he has not previously believed; especially, one who turns from the controlling power of sin to that of holiness, or from unbelief to Christianity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Jesuits did not persuade the <qex>converts</qex> to lay aside the use of images.</q> <rj><qau>Bp. Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lay friar or brother, permitted to enter a monastery for the service of the house, but without orders, and not allowed to sing in the choir.</def></p>

<p><syn><b>Syn.</b> -- Proselyte; neophyte.</syn> <usage> -- <er>Convert</er>, <er>Proselyte</er>, <er>Pervert</er>. A <xex>convert</xex> is one who turns from what he believes to have been a decided error of faith or practice. Such a change may relate to religion, politics, or other subjects. properly considered, it is not confined to speculation alone, but affects the whole current of one's feelings and the tenor of his actions. As such a change carries with it the appearance of sincerity, the term <xex>convert</xex> is usually taken in a good sense. <xex>Proselyte</xex> is a term of more ambiguous use and application. It was first applied to an adherent of one religious system who had transferred himself externally to some other religious system; and is also applied to one who makes a similar transfer in respect to systems of philosophy or speculation. The term has little or no reference to the state of the heart. <xex>Pervert</xex> is a term of recent origin, designed to express the contrary of <xex>convert</xex>, and to stigmatize a person as drawn off perverted from the true faith. It has been more particulary applied by members of the Church of England to those who have joined the Roman Catholic Church.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>converted</ent><br/
<hw>converted</hw> <pos>adj.</pos> <def>spiritually reborn or converted; <as>as, a <ex>converted</ex> sinner</as>.</def><br/
<syn><b>Syn. --</b> born-again, reborn.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Convertend</ent><br/
<hw>Con`ver*tend"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>convertenus</ets> to be converted.]</ety> <fld>(Logic)</fld> <def>Any proposition which is subject to the process of conversion; -- so called in its relation to itself as converted, after which process it is termed the <xex>converse</xex>. See <er>Converse</er>, <pos>n.</pos> <fld>(Logic)</fld>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Converter</ent><br/
<hw>Con*vert"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who converts; one who makes converts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Steel Manuf.)</fld> <def>A retort, used in the Bessemer process, in which molten cast iron is decarburized and converted into steel by a blast of air forced through the liquid metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convertibility</ent><br/
<hw>Con*vert`i*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The condition or quality of being convertible; capability of being exchanged; convertibleness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mutual <qex>convertibility</qex> of land into money, and of money into land.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convertible</ent><br/
<hw>Con*vert"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>convertibilis</ets>: cf. F. <ets>convertible</ets>.]</ety> <sn>1.</sn> <def>Capable of being converted; susceptible of change; transmutable; transformable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Minerals are not <qex>convertible</qex> into another species, though of the same genus.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being exchanged or interchanged; reciprocal; interchangeable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So long as we are in the regions of nature, miraculous and improbable, miraculous and incredible, may be allowed to remain <qex>convertible</qex> terms.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convertibleness</ent><br/
<hw>Con*vert"i*ble*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being convertible; convertibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convertibly</ent><br/
<hw>Con*vert"i*bly</hw>, <pos>adv.</pos> <def>In a convertible manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convertite</ent><br/
<hw>Con"vert*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. It. <ets>convertito</ets>, p. p. of <ets>convertire</ets> to convert.]</ety> <def>A convert.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>convertor</ent><br/
<hw>convertor</hw> <pos>n.</pos> <def>a device for changing one substance or form or state into another.</def><br/
<syn><b>Syn. --</b> converter.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Convex</ent><br/
<hw>Con"vex</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>convexus</ets> vaulted, arched, convex, concave, fr. <ets>convehere</ets> to bring together: cf. F. <ets>convexe</ets>. See <er>Vehicle</er>.]</ety> <def>Rising or swelling into a spherical or rounded form; regularly protuberant or bulging; -- said of a spherical surface or curved line when viewed from without, in opposition to <xex>concave</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Drops of water naturally form themselves into figures with a <qex>convex</qex> surface.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Double convex</b></col>, <cd>convex on both sides; convexo-convex.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convex</ent><br/
<hw>Con"vex</hw>, <pos>n.</pos> <def>A convex body or surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Half heaven's <qex>convex</qex> glitters with the flame.</q> <rj><qau>Tickell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word was often pronounced <it>con-vex'</it> by early writers, as by Milton, and occasionally by later poets.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexed</ent><br/
<hw>Con"vexed</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos>  <def>Made convex; protuberant in a spherical form.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexedly</ent><br/
<hw>Con*vex"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a convex form; convexly.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexedness</ent><br/
<hw>Con*vex"ed*ness</hw>, <pos>n.</pos> <def>Convexity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexity</ent><br/
<hw>Con*vex"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Convexities</plw> <pr>(#)</pr></plu>. <ety>[L. <ets>convexitas</ets>: cf. F. <ets>convexit<eacute/</ets>.]</ety> <def>The state of being convex; the exterior surface of a convex body; roundness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A smooth, uniform <qex>convexity</qex> and rotundity of a globe.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexly</ent><br/
<hw>Con"vex*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a convex form; <as>as, a body <ex>convexly</ex> shaped</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexness</ent><br/
<hw>Con"vex*ness</hw>, <pos>n.</pos> <def>The state of being convex; convexity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexo-concave</ent><br/
<hw>Con*vex"o-con"cave</hw> <pr>(?<or/ ?)</pr>, <pos>a.</pos> <def>Convex on one side, and concave on the other. The curves of the convex and concave sides may be alike or may be different. See <er>Meniscus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexo-convex</ent><br/
<hw>Con*vex"o-con"vex</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Convex on both sides; double convex. See under <er>Convex</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convexo-plane</ent><br/
<hw>Con*vex"o-plane`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Convex on one side, and flat on the other; plano-convex.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convey</ent><br/
<hw>Con*vey"</hw> <pr>(k<ocr/n*v<amac/")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Conveyed</conjf> <pr>(k<ocr/n*v<amac/d")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Conveying</conjf>.]</vmorph> <ety>[OF. <ets>conveir</ets>, <ets>convoier</ets>, to escort, convoy, F. <ets>convoyer</ets>, LL. <ets>conviare</ets>, fr. L. <ets>con-</ets> + <ets>via</ets> way. See <er>Viaduct</er>, <er>Voyage</er>, and cf. <er>Convoy</er>.]</ety> <sn>1.</sn> <def>To carry from one place to another; to bear or transport.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>convey</qex> them by sea in floats.</q> <rj><qau>1 Kings v. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Convey</qex> me to my bed, then to my grave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to pass from one place or person to another; to serve as a medium in carrying (anything) from one place or person to another; to transmit; <as>as, air <ex>conveys</ex> sound; words <ex>convey</ex> ideas</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To transfer or deliver to another; to make over, as property; more strictly <fld>(Law)</fld>, to transfer (real estate) or pass (a title to real estate) by a sealed writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Earl of Desmond . . . secretly <qex>conveyed</qex> all his lands to feoffees in trust.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To impart or communicate; <as>as, to <ex>convey</ex> an impression; to <ex>convey</ex> information</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men fill one another's heads with noise and sound, but <qex>convey</qex> not thereby their thoughts.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To manage with privacy; to carry out.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I . . . will <qex>convey</qex> the business as I shall find means.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To carry or take away secretly; to steal; to thieve.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To accompany; to convoy.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj></p>

<p><syn><b>Syn.</b> -- To carry; transport; bear; transmit; transfer.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convey</ent><br/
<hw>Con*vey"</hw>, <pos>v. i.</pos> <def>To play the thief; to steal.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But as I am Crack, I will <qex>convey</qex>, crossbite, and cheat upon Simplicius.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conveyable</ent><br/
<hw>Con*vey"a*ble</hw> <pr>(k<ocr/n*v<amac/"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being conveyed or transferred.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conveyance</ent><br/
<hw>Con*vey"ance</hw> <pr>(k<ocr/n*v<amac/"<ait/ns)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of conveying, carrying, or transporting; carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The long journey was to be performed on horseback, -- the only sure mode of <qex>conveyance</qex>.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Following the river downward, there is <qex>conveyance</qex> into the countries named in the text.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The instrument or means of carrying or transporting anything from place to place; the vehicle in which, or means by which, anything is carried from one place to another; <as>as, stagecoaches, omnibuses, etc., are <ex>conveyances</ex>; a canal or aqueduct is a <ex>conveyance</ex> for water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These pipes and these <qex>conveyances</qex> of our blood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act or process of transferring, transmitting, handing down, or communicating; transmission.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tradition is no infallible way of <qex>conveyance</qex>.</q> <rj><qau>Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>The act by which the title to property, esp. real estate, is transferred; transfer of ownership; an instrument in writing (as a deed or mortgage), by which the title to property is conveyed from one person to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] found the <qex>conveyances</qex> in law to be so firm, that in justice he must decree the land to the earl.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Dishonest management, or artifice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>the very Jesuits themselves . . . can not possibly devise any juggling <qex>conveyance</qex> how to shift it off.</q> <rj><qau>Hakewill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conveyancer</ent><br/
<hw>Con*vey"an*cer</hw> <pr>(k<ocr/n*v<amac/"<ait/n*s<etil/r)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One whose business is to draw up conveyances of property, as deeds, mortgages, leases, etc.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conveyancing</ent><br/
<hw>Con*vey"an*cing</hw> <pr>(k<ocr/n*v<amac/"<ait/n*s<icr/ng)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The business of a conveyancer; the act or business of drawing deeds, leases, or other writings, for transferring the title to property from one person to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 319 --></p>

<p><ent>Conveyer</ent><br/
<hw>Con*vey"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, conveys or carries, transmits or transfers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One given to artifices or secret practices; a juggler; a cheat; a thief.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>conveying</ent><br/
<hw>conveying</hw> <pos>n.</pos> <def>the act of transferring a property title from one person to another.</def><br/
<syn><b>Syn. --</b> conveyance, conveyance of title, conveyancing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Conveyor</ent><br/
<hw>Con*vey"or</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>A contrivance for carrying objects from place to place; esp., one for conveying grain, coal, etc., -- as a spiral or screw turning in a pipe or trough, an endless belt with buckets, or a truck running along a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conviciate</ent><br/
<hw>Con*vi"ci*ate</hw>, <pos>v. i.</pos> <ety>[L. <ets>conviciatus</ets>, p. p. of <ets>conviciari</ets> to revile, fr. <ets>convicium</ets> loud reproach.]</ety> <def>To utter reproaches; to raise a clamor; to rail.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>conviciate</qex> instead of accusing.</q> <rj><qau>Laud.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convicinity</ent><br/
<hw>Con`vi*cin"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Convicinities</plw> <pr>(<?/)</pr>.</plu> <def>Immediate vicinity; neighborhood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>convicinity</qex> and contiguity of the two parishes.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convicious</ent><br/
<hw>Con*vi"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Expressing reproach; abusive; railing; taunting.</def> <mark>[Obs.]</mark> <ldquo/<xex>Convicious</xex> words.<rdquo/  <rj><au>Queen Elizabeth (1559).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convict</ent><br/
<hw>Con*vict"</hw> <pr>(k<ocr/n*v<icr/kt")</pr>, <pos>p. a.</pos> <ety>[L. <ets>convictus</ets>, p. p. of <ets>convincere</ets> to convict, prove. See <er>Convice</er>.]</ety> <def>Proved or found guilty; convicted.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Convict</qex> by flight, and rebel to all law.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convict</ent><br/
<hw>Con"vict</hw> <pr>(k<ocr/n"v<icr/kt)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person proved guilty of a crime alleged against him; one legally convicted or sentenced to punishment for some crime.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A criminal sentenced to penal servitude.</def></p>

<p><syn><b>Syn.</b> -- Malefactor; culprit; felon; criminal.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convict</ent><br/
<hw>Con*vict"</hw> <pr>(k<ocr/n*v<icr/kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convicted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Convicting</conjf>.]</vmorph> <sn>1.</sn> <def>To prove or find guilty of an offense or crime charged; to pronounce guilty, as by legal decision, or by one's conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Baxter] . . . had been <qex>convicted</qex> by a jury.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They which heard it, being <qex>convicted</qex> by their own conscience, went out one by one.</q> <rj><qau>John viii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prove or show to be false; to confute; to refute.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To demonstrate by proof or evidence; to prove.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Imagining that these proofs will <qex>convict</qex> a testament, to have that in it which other men can nowhere by reading find.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To defeat; to doom to destruction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A whole armado of <qex>convicted</qex> sail.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- To confute; defect; convince; confound.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>convictfish</ent><br/
<hw>convictfish</hw> <pos>n.</pos> <def>a type of greenling (<spn>Oxylebius pictus</spn>) with a whitish body marked with black bands.</def><br/
<syn><b>Syn. --</b> painted greenling, <spn>Oxylebius pictus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Convictible</ent><br/
<hw>Con*vict*i*ble</hw> <pr>(k<ocr/n*v<icr/kt"<icr/*b'l)</pr>, <pos>a.</pos> <def>Capable of being convicted.</def> <mark>[R.]</mark>  <rj><au>Ash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conviction</ent><br/
<hw>Con*vic"tion</hw> <pr>(k<ocr/n*v<icr/k"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>convictio</ets> proof: cf. F. <ets>conviction</ets> conviction (in sense 3 & 4). See <er>Convict</er>, <er>Convince</er>.]</ety> <sn>1.</sn> <def>The act of convicting; the act of proving, finding, or adjudging, guilty of an offense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The greater certainty of <qex>conviction</qex> and the greater certainty of punishment.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>A judgment of condemnation entered by a court having jurisdiction; the act or process of finding guilty, or the state of being found guilty of any crime by a legal tribunal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Conviction</qex> may accrue two ways.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of convincing of error, or of compelling the admission of a truth; confutation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For all his tedious talk is but vain boast,<br/
Or subtle shifts <qex>conviction</qex> to evade.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The state of being convinced or convicted; strong persuasion or belief; especially, the state of being convicted of sin, or by one's conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To call good evil, and evil good, against the <qex>conviction</qex> of their own consciences.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And did you presently fall under the power of this <qex>conviction</qex>?</q> <rj><qau>Bunyan.</qau></rj></p>

<p><syn><b>Syn.</b> -- <er>Conviction</er>; <er>persuasion</er>.</syn> <usage> -- <xex>Conviction</xex> respects soley matters of belief or faith; <xex>persuasion</xex> respects matters of belief or practice. <xex>Conviction</xex> respects our most important duties; <xex>persuasion</xex> is frequently applied to matters of indifference. <au>Crabb.</au> -- <xex>Conviction</xex> is the result of the [operation of the] understanding; <xex>persuasion</xex>, of the will. <xex>Conviction</xex> is a necessity of the mind, <xex>persuasion</xex> an acquiescence of the inclination.</usage> <au>C. J. Smith.</au> -- <xex>Persuasion</xex> often induces men to act in opposition to their <xex>conviction</xex> of duty.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Convictism</ent><br/
<hw>Con"vict*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The policy or practice of transporting convicts to penal settlements.</def> <ldquo/The evils of <xex>convictism</xex>.<rdquo/  <rj><au>W. Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convictive</ent><br/
<hw>Con*vict"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Convincing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The best and most <qex>convictive</qex> argument.</q> <rj><qau>Glanwill.</qau></rj></p>

<p>-- <wordforms><wf>Con*vict"ive*ly</wf>, <pos>adv.</pos> -- <wf>Con*vict"ive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convince</ent><br/
<hw>Con*vince"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convinced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convincing</conjf>.]</vmorph> <ety>[L. <ets>convincere</ets>, <ets>-victum</ets>, to refute, prove; <ets>con-</ets> + <ets>vincere</ets> to conquer. See <er>Victor</er>, and cf. <er>Convict</er>.]</ety> <sn>1.</sn> <def>To overpower; to overcome; to subdue or master.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His two chamberlains<br/
Will I with wine and wassail so <qex>convince</qex><br/
That memory, the warder of the brain,<br/
Shall be a fume.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To overcome by argument; to force to yield assent to truth; to satisfy by proof.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such convincing proofs and assurances of it as might enable them to <qex>convince</qex> others.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To confute; to prove the fallacy of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>God never wrought miracle to <qex>convince</qex> atheism, because his ordinary works <qex>convince</qex> it.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To prove guilty; to convict.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which of you <qex>convinceth</qex> me of sin?</q> <rj><qau>John viii. 46.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Seek not to <qex>convince</qex> me of a crime<br/
Which I can ne'er repent, nor you can pardon.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- To persuade; satisfy; convict.</syn> <usage> -- To <er>Convince</er>, <er>persuade</er>. To <xex>convince</xex> is an act of the understanding; to <xex>persuade</xex>, of the will or feelings. The one is effected by argument, the other by motives. There are cases, however, in which <xex>persuade</xex> may seem to be used in reference only to the assent of the understanding; as when we say, I am <xex>persuaded</xex> it is so; I can not <xex>persuade</xex> myself of the fact. But in such instances there is usually or always a degree of awakened feeling which has had its share in producing the assent of the understanding.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convincement</ent><br/
<hw>Con*vince"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of convincing, or state of being convinced; conviction.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The fear of a <qex>convincement</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convincer</ent><br/
<hw>Con*vin"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, convinces; one who wins over by proof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convincible</ent><br/
<hw>Con*vin"ci*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being convinced or won over.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being confuted and disproved by argument; refutable.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>convincing</ent><br/
<hw>convincing</hw> <pos>adj.</pos> <def>causing one to believe the truth of something; having the power to influence or convince; cogent; -- of evidence or testimony; <as>as, a <ex>convincing</ex> manner</as>.  Opposite of <ant>unconvincing</ant>.</def> <note>[Narrower terms: <stype>disenchanting, disillusioning</stype>]</note> <see>[Also See: <er>persuasive</er>.]</see><br/
<syn><b>Syn. --</b> cogent, potent.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>convincing</ent><br/
<hw>convincing</hw> <pos>n.</pos> <def>a successful persuasion.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Convincingly</ent><br/
<hw>Con*vin"cing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>in a convincing manner; in a manner to compel assent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convincingness</ent><br/
<hw>Con*vin"cing*ness</hw>, <pos>n.</pos> <def>The power of convincing, or the quality of being convincing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convival</ent><br/
<hw>Con*viv"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>convivalis</ets>. See <er>Convive</er>.]</ety> <def>pertaining to a feast or to festivity; convivial.</def> <mark>[Obs.]</mark> <ldquo/A <xex>convival</xex> dish.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convive</ent><br/
<hw>Con*vive"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>convivari</ets>; akin to <ets>convivium</ets> a feast, <ets>convivere</ets> to live or feast together; <ets>con-</ets> + <ets>vivere</ets> to live.]</ety> <def>To feast together; to be convivial.</def> <mark>[Obs.]</mark> <ldquo/There, in the full, <xex>convive</xex> we.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convive</ent><br/
<hw>Con"vive</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>conviva</ets>: cf. F. <ets>convive</ets>.]</ety> <def>A quest at a banquet.</def> <mark>[R.]</mark>  <rj><au>Beaumont.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convivial</ent><br/
<hw>Con*viv"i*al</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[From L. <ets>convivium</ets> a feast; <ets>con-</ets> + <ets>vivere</ets> to live. See <er>Victuals</er>, and cf. <er>Convive</er>.]</ety> <def>Of or relating to a feast or entertainment, or to eating and drinking, with accompanying festivity; festive; social; gay; jovial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which feasts <qex>convivial</qex> meetings we did name.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convivialist</ent><br/
<hw>Con*viv"i*al*ist</hw>, <pos>n.</pos> <def>A person of convivial habits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conviviality</ent><br/
<hw>Con*viv`i*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Convivialities</plw> <pr>(<?/)</pr>.</plu> <def>The good humor or mirth indulged in upon festive occasions; a convivial spirit or humor; festivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convivially</ent><br/
<hw>Con*viv"i*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a convivial manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convocate</ent><br/
<hw>Con"vo*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convocated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Convocating</conjf>.]</vmorph> <ety>[L. <ets>convocatus</ets>, p. p. of <ets>convocare</ets> to convocate; <ets>con-</ets> + <ets>vocare</ets> to call. See <er>Vocal</er>, and cf. <er>Convoce</er>.]</ety> <def>To convoke; to call together.</def> <mark>[Obs.]</mark>  <rj><au>May (Lucan).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convocation</ent><br/
<hw>Con`vo*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>convocatio</ets>: cf. F. <ets>convocation</ets>. <ets>See</ets> <er>Convoke</er>.]</ety> <sn>1.</sn> <def>The act of calling or assembling by summons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assembly or meeting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the first day there shall be a holy <qex>convocation</qex>.</q> <rj><qau>Ex. xii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Ch. of Eng.)</fld> <def>An assembly of the clergy, by their representatives, to consult on ecclesiastical affairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, the provinces of Canterbury and York have each their convocation, but no session for business were allowed from 1717 to 1861. The <xex>Convocation of Canterbury</xex> consists of two houses. In the <xex>Convocation of York</xex> the business has been generally conducted in one assembly.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Oxf. University)</fld> <def>An academical assembly, in which the business of the university is transacted.</def></p>

<p><syn><b>Syn.</b> -- meeting; assembly; congregation; congress; diet; convention; synod; council.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convocational</ent><br/
<hw>Con`vo*ca"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a convocation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convocationist</ent><br/
<hw>Con`vo*ca"tion*ist</hw>, <pos>n.</pos> <def>An advocate or defender of convocation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convoke</ent><br/
<hw>Con*voke"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convoked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convoking</conjf>.]</vmorph> <ety>[L. <ets>convocare</ets>: cf. F. <ets>convoquer</ets>. See <er>Convocate</er>.]</ety> <def>To call together; to summon to meet; to assemble by summons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There remained no resource but the dreadful one of <qex>convoking</qex> a parliament.</q> <rj><qau>palfrey.</qau></rj></p>

<p><syn><b>Syn.</b> -- To summon; assemble; convene. See <er>Call</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convolute</ent><br/
<hw>Con"vo*lute</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>convolutus</ets>, p. p. of <ets>convolvere</ets>. See <er>Convolve</er>.]</ety> <fld>(Bot.)</fld> <def>Rolled or wound together, one part upon another; -- said of the leaves of plants in <ae/stivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convoluted</ent><br/
<hw>Con"vo*lu`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having convolutions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>beaks recurved and <qex>convoluted</qex> like a ram's horn.</q> <rj><qau>Pennant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Folded in tortuous windings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A highly <qex>convoluted</qex> brain.</q> <rj><qau>North Amer. Rev.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convolution</ent><br/
<hw>Con`vo*lu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of rolling anything upon itself, or one thing upon another; a winding motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O'er the calm sea, in <qex>convolution</qex> swift,<br/
The feathered eddy floats.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being rolled upon itself, or rolled or doubled together; a tortuous or sinuous winding or fold, as of something rolled or folded upon itself.</def>  <rj><au>Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>An irregular, tortuous folding of an organ or part; <as>as, the <ex>convolutions</ex> of the intestines; the cerebral <ex>convolutions</ex>. See <er>Brain</er></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convolve</ent><br/
<hw>Con*volve"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convolved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convolving</conjf>.]</vmorph> <ety>[L. <ets>convolvere</ets>, <ets>-volutum</ets>; <ets>con-</ets> + <ets>volvere</ets> to roll. See <er>Voluble</er>.]</ety> <def>To roll or wind together; to roll or twist one part on another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then Satan first knew pain,<br/
And writhed him to and fro <qex>convolved</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convolvulaceous</ent><br/
<hw>Con*vol`vu*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Convolvus</er>.]</ety> <fld>(Bot.)</fld> <def>Of, pertaining to, or resembling, the family of plants of which the bindweed and the morning-glory are common examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convolvulin</ent><br/
<hw>Con*vol"vu*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A glucoside occurring in jalap (the root of a convolvulaceous plant), and extracted as a colorless, tasteless, gummy mass of powerful purgative properties.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convolvulus</ent><br/
<hw>Con*vol"vu*lus</hw> <pr>(k<ocr/n*v<ocr/l"v<usl/*l<ucr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L.<plw>Convolvuli</plw> <pr>(#)</pr>, E. <plw>Convoluluses</plw> <pr>(#)</pr>.</plu> <ety>[L., bindweed, fr. <ets>convolvere</ets> to roll around. So named from its twining stems.]</ety> <fld>(Bot.)</fld> <def>A large genus of plants having monopetalous flowers, including the common bindweed (<spn>Convolvulus arvensis</spn>), and formerly the morning-glory, but this is now transferred to the genus <gen>Ipom<ae/a</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The luster of the long <qex>convolvuluses</qex><br/
That coiled around the stately stems.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convoy</ent><br/
<hw>Con*voy"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convoyed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convoying</conjf>.]</vmorph> <ety>[F. <ets>convoyer</ets>, OF. <ets>conveier</ets>, <ets>convoier</ets>. See <er>Convey</er>.]</ety> <def>To accompany for protection, either by sea or land; to attend for protection; to escort; <as>as, a frigate <ex>convoys</ex> a merchantman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I know ye skillful to <qex>convoy</qex><br/
The total freight of hope and joy.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convoy</ent><br/
<hw>Con"voy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>convoi</ets>.]</ety> <sn>1.</sn> <def>The act of attending for defense; the state of being so attended; protection; escort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To obtain the <qex>convoy</qex> of a man-of-war.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel or fleet, or a train or trains of wagons, employed in the transportation of munitions of war, money, subsistence, clothing, etc., and having an armed escort.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A protection force accompanying ships, etc., on their way from place to place, by sea or land; an escort, for protection or guidance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When every morn my bosom glowed<br/
To watch the <qex>convoy</qex> on the road.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Conveyance; means of transportation.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A drag or brake applied to the wheels of a carriage, to check their velocity in going down a hill.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convoy pennant</ent><br/
<hw>Con"voy pen"nant</hw>. <def>A white pennant with red border, carried: <it>(a)</it> Forward on all vessels on convoy duty. <it>(b)</it> Alone by a senior officer present during evolutions or drills, when it commands <ldquo/Silence.<rdquo/  <it>(c)</it> Over a signal number, when it refers to the signal number of an officer in the Annual Navy Register.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Convulse</ent><br/
<hw>Con*vulse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Convulsed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Convulsing</conjf>.]</vmorph> <ety>[L. <ets>convulsus</ets>, p. p. of <ets>convellere</ets> to tear up, to shake; <ets>con-</ets> + <ets>vellere</ets> to pluck, pull.]</ety> <sn>1.</sn> <def>To contract violently and irregulary, as the muscular parts of an animal body; to shake with irregular spasms, as in excessive laughter, or in agony from grief or pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With emotions which checked his voice and <qex>convulsed</qex> his powerful frame.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To agitate greatly; to shake violently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world is <qex>convulsed by the agonies of great nations</qex>.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- To agitate; disturb; shake; tear; rend.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsion</ent><br/
<hw>Con*vul"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>convulsio</ets>: cf. F. <ets>convulsion</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>An unnatural, violent, and unvoluntary contraction of the muscular parts of an animal body.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any violent and irregular motion or agitation; a violent shaking; a tumult; a commotion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those two massy pillars,<br/
With horrible <qex>convulsion</qex>, to and fro<br/
He tugged, he shook, till down they came.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Times of violence and <qex>convulsion</qex>.</q> <rj><qau>Ames.</qau></rj></p>

<p><syn><b>Syn.</b> -- Agitation; commotion; tumult; disturbance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsional</ent><br/
<hw>Con*vul"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or having, convulsions; convulsionary.</def> <mark>[R.]</mark>  <rj><au>Lamb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsionary</ent><br/
<hw>Con*vul"sion*a*ry</hw> <pr>(<?/)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>convulsionnaire</ets>.]</ety> <def>Pertaining to convulsion; convulsive.</def> <ldquo/<xex>Convulsionary</xex> struggles.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsionary</ent><br/
<hw>Con*vul"sion*a*ry</hw>, <pos>n.</pos> <def>A convulsionist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsionist</ent><br/
<hw>Con*vul"sion*ist</hw>, <pos>n.</pos> <def>One who has convulsions; esp., one of a body of fanatics in France, early in the eighteenth century, who went into convulsions under the influence of religious emotion; <as>as, the <ex>Convulsionists</ex> of St. M<eacute/dard</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsive</ent><br/
<hw>Con*vul"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>convulsif</ets>.]</ety> <def>Producing, or attended with, convulsions or spasms; characterized by convulsions; convulsionary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An irregular, <qex>convulsive</qex> movement may be necessary to throw off an irregular, <qex>convulsive</qex> disease.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Convulsively</ent><br/
<hw>Con*vul"sive*ly</hw>, <pos>adv.</pos> <def>in a convulsive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cony</ent><br/
<hw>Co"ny</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>coning</ets>, <ets>conig</ets>, <ets>coni</ets>, OF. <ets>connin</ets>, <ets>conin</ets>, <ets>connil</ets>, fr. L. <ets>cuniculus</ets> a rabbit, cony, prob. an Hispanic word.]</ety> <altsp>[Written also <asp>coney</asp>.]</altsp> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A rabbit, esp., the European rabbit (<spn>Lepus cuniculus</spn>)</def>. <sd>(b)</sd> <def>The chief hare.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>cony</xex> of Scripture is thought to be <spn>Hyrax Syriacus</spn>, called also <altname>daman</altname>, and <altname>cherogril</altname>. See <er>Daman</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A simpleton.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a most simple animal; whence are derived our usual phrases of <qex>cony</qex> and <qex>cony catcher</qex>.</q> <rj><qau>Diet's Dry Dinner (1599).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>An important edible West Indian fish (<spn>Epinephelus apua</spn>); the hind of Bermuda.</def> <sd>(b)</sd> <def>A local name of the burbot.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cony-catch</ent><br/
<hw>Co"ny-catch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To deceive; to cheat; to trick.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Take heed, Signor Baptista, lest you be <qex>cony-catched</qex> in the this business.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cony-catcher</ent><br/
<hw>Co"ny-catch`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cheat; a sharper; a deceiver.</def> <mark>[Obs.]</mark>  <rj><au>Minsheu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conylene</ent><br/
<hw>Con"y*lene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Con</ets>ine + acetylene.]</ety> <def>An oily substance, <chform>C8H14</chform>, obtained from several derivatives of conine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Conyrine</ent><br/
<hw>Con"y*rine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Conine</er>.]</ety> <fld>(Chem.)</fld> <def>A blue, fluorescent, oily base (regarded as a derivative of pyridine), obtained from conine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coo</ent><br/
<hw>Coo</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cooed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cooing</conjf>.]</vmorph> <sn>1.</sn> <def>To make a low repeated cry or sound, like the characteristic note of pigeons or doves.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stockdove only through the forest <qex>cooes</qex>,<br/
Mournfully hoarse.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To show affection; to act in a loving way. See under <er>Bill</er>, <pos>v. i.</pos></def> <ldquo/Billing or <xex>cooing</xex>.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooee</ent><br/
<ent>Cooey</ent><br/
<mhw>{ <hw>Coo"ey</hw>, <hw>Coo"ee</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Of imitative origin.]</ety> <def>A peculiar cry uttered by the Australian aborigines as a call to attract attention, and also in common use among the Australian colonists. In the actual call the first syllable is much prolonged (k<oomac/"-) and the second ends in a shrill, staccato <emac/. To represent the sound itself the spelling <asp>cooee</asp> is generally used.</def> <altsp>[Written also <asp>cooie</asp>.]</altsp></p>

<p><cs><col><b>Within cooey</b></col>, <cd>within earshot.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cooee</ent><br/
<ent>Cooey</ent><br/
<mhw>{ <hw>Coo"ey</hw>, <hw>Coo"ee</hw>  }</mhw>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cooeyed</conjf> or <conjf>Cooeed</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cooeying</conjf> or <conjf>Cooeeing</conjf>.]</vmorph> <def>To call out <xex>cooee</xex>.</def> <mark>[Australia]</mark></p>

<p><q>I <qex>cooeyed</qex> and beckoned them to approach.</q>  <rj><qau>E. Giles.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cooing</ent><br/
<hw>cooing</hw> <pos>adj.</pos> <ety>[pr. Participle of verb <er>coo</er> (definition 2)]</ety> <def>emitting a cry like that of a dove; <as>as, The <ex>cooing</ex> pigeons</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cook</ent><br/
<hw>Cook</hw> <pr>(k<oomac/k)</pr>, <pos>v. i.</pos> <ety>[Of imitative origin.]</ety> <def>To make the noise of the cuckoo.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Constant cuckoos <qex>cook</qex> on every side.</q> <rj><qau>The Silkworms (1599).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cook</ent><br/
<hw>Cook</hw> <pr>(k<oocr/k)</pr>, <pos>v. t.</pos> <ety>[Etymol. unknown.]</ety> <def>To throw.</def> <mark>[Prov.Eng.]</mark> <ldquo/<xex>Cook</xex> me that ball.<rdquo/  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cook</ent><br/
<hw>Cook</hw> <pr>(k<oocr/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>c<omac/c</ets>, fr. L. <ets>cocus</ets>, <ets>coquus</ets>, <ets>coquus</ets>, fr. <ets>coquere</ets> to cook; akin to Gr. <grk>pe`ptein</grk>, Skr. <ets>pac</ets>, and to E. <ets>apricot</ets>, <ets>biscuit</ets>, <ets>concoct</ets>, <ets>dyspepsia</ets>, <ets>precocious</ets>. Cf. <er>Pumpkin</er>.]</ety> <sn>1.</sn> <def>One whose occupation is to prepare food for the table; one who dresses or cooks meat or vegetables for eating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A fish, the European striped wrasse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cook</ent><br/
<hw>Cook</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cooked</conjf> <pr>(k<oocr/kt)</pr>; <pos>p. pr & vb. n.</pos> <conjf>Cooking</conjf>.]</vmorph> <sn>1.</sn> <def>To prepare, as food, by boiling, roasting, baking, broiling, etc.; to make suitable for eating, by the agency of fire or heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To concoct or prepare; hence, to tamper with or alter; to garble; -- often with <xex>up</xex>; <as>as, to <ex>cook</ex> up a story; to <ex>cook</ex> an account</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They all of them receive the same advices from abroad, and very often in the same words; but their way of <qex>cooking</qex> it is so different.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 320 --></p>

<p><ent>Cook</ent><br/
<hw>Cook</hw> <pr>(k<oocr/k)</pr>, <pos>v. i.</pos> <def>To prepare food for the table.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cookbook</ent><br/
<hw>Cook"book`</hw> <pr>(-b<oocr/k`)</pr>, <pos>n.</pos> <def>A book of directions and receipts for cooking; a cookery book.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Just How<rdquo/: a key to the <qex>cookbooks</qex>.</q> <rj><qau>Mrs. A. D. T. Whitney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cooked-over</ent><br/
<hw>cooked-over</hw> <pos>adj.</pos> <def>heated again after having cooled down; -- often used of food remaining from a previous day.</def><br/
<syn><b>Syn. --</b> warmed-over, reheated.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cookee</ent><br/
<hw>Cook*ee"</hw> <pr>(k<oocr/k*<emac/")</pr>, <pos>n.</pos><def>A female cook.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cooker</ent><br/
<hw>cooker</hw> <pos>n.</pos> <def>an implement for cooking.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cookery</ent><br/
<hw>Cook"er*y</hw> <pr>(k<oocr/k"<etil/r*<ycr/)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The art or process of preparing food for the table, by dressing, compounding, and the application of heat; cooking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A delicacy; a dainty.</def> <mark>[Obs.]</mark>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cookie</ent><br/
<ent>Cookey</ent><br/
<mhw>{ <hw>Cook"ey</hw>, <hw>Cook"ie</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cooky</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cooking</ent><br/
<hw>cooking</hw> <pos>n.</pos> <sn>1.</sn> <def>the practice or manner of preparing food or the food so prepared; cookery.</def><br/
<syn><b>Syn. --</b> cookery, cuisine, culinary art.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the act of preparing something (as food) by the application of heat.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cookmaid</ent><br/
<hw>Cook"maid`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female servant or maid who dresses provisions and assists the cook.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cookroom</ent><br/
<hw>Cook"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A room for cookery; a kitchen; the galley or caboose of a ship.</def>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cookshop</ent><br/
<hw>Cook`shop</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An eating house.</def> <ldquo/A subterranean <xex>cookshop</xex>.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cookstove</ent><br/
<hw>cookstove</hw> <pos>n.</pos> <def>a stove for cooking, especially a wood- or coal-burning kitchen stove.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cookware</ent><br/
<hw>cookware</hw> <pos>n.</pos> <def>cooking utensils, such as pots, pans, or baking dishes made of heat-resistant material.</def><br/
<syn><b>Syn. --</b> cooking utensil.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cooky</ent><br/
<hw>Cook"y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cookies</plw> <pr>(#)</pr></plu>. <ety>[Cf. D. <ets>koek</ets> cake, dim. <ets>koekje</ets>; akin to G. <ets>kuchen</ets>, E. <ets>cake</ets>; or cf. OE. <ets>coket</ets>, prob., a sort of cake, and prob. of French origin.]</ety> <def>A small, flat, sweetened cake of various kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cool</ent><br/
<hw>Cool</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Cooler</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Coolest</adjf>.]</amorph> <ety>[AS. <ets>c<omac/l</ets>; akin to D. <ets>koel</ets>, G. <ets>k<uum/hl</ets>, OHG. chouli, Dan. <ets>k<oum/lig</ets>, Sw. <ets>kylig</ets>, also to AS. <ets>calan</ets> to be cold, Icel. <ets>kala</ets>. See <er>Cold</er>, and cf. <er>Chill</er>.]</ety> <sn>1.</sn> <def>Moderately cold; between warm and cold; lacking in warmth; producing or promoting coolness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fanned with <qex>cool</qex> winds.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not ardent, warm, fond, or passionate; not hasty; deliberate; exercising self-control; self-possessed; dispassionate; indifferent; <as>as, a <ex>cool</ex> lover; a <ex>cool</ex> debater</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For a patriot, too <qex>cool</qex>.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not retaining heat; light; <as>as, a <ex>cool</ex> dress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Manifesting coldness or dislike; chilling; apathetic; <as>as, a <ex>cool</ex> manner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Quietly impudent; negligent of propriety in matters of minor importance, either ignorantly or willfully; presuming and selfish; audacious; <as>as, <ex>cool</ex> behavior</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Its <qex>cool</qex> stare of familiarity was intolerable.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Applied facetiously, in a vague sense, to a sum of money, commonly as if to give emphasis to the largeness of the amount.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had lost a <qex>cool</qex> hundred.</q> <rj><qau>Fielding.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Leaving a <qex>cool</qex> thousand to Mr. Matthew Pocket.</q> <rj><qau>Dickens.</qau></rj></p>

<p><syn><b>Syn.</b> -- Calm; dispassionate; self-possessed; composed; repulsive; frigid; alienated; impudent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cool</ent><br/
<hw>Cool</hw>, <pos>n.</pos> <def>A moderate state of cold; coolness; -- said of the temperature of the air between hot and cold; <as>as, the <ex>cool</ex> of the day; the <ex>cool</ex> of the morning or evening</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cool</ent><br/
<hw>Cool</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cooled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cooling</conjf>.]</vmorph> <sn>1.</sn> <def>To make cool or cold; to reduce the temperature of; <as>as, ice <ex>cools</ex> water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Send Lazarus, that he may dip the tip of his finger in water, and <qex>cool</qex> my tongue.</q> <rj><qau>Luke xvi. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To moderate the heat or excitement of; to allay, as passion of any kind; to calm; to moderate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have reason to <qex>cool</qex> our raging motions, our carnal stings, our unbitted lusts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cool the heels</b></col>, <cd>to dance attendance; to wait, as for admission to a patron's house.</cd> <mark>[Colloq.]</mark>  <rj><au>Dryden.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cool</ent><br/
<hw>Cool</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To become less hot; to lose heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw a smith stand with his hammer, thus,<br/
the whilst his iron did on the anvil <qex>cool</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lose the heat of excitement or passion; to become more moderate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will not give myself liberty to think, lest I should <qex>cool</qex>.</q> <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coolant</ent><br/
<hw>coolant</hw> <pos>n.</pos> <def>a fluid (gas or liquid) used to cool a device by transferring heat away from one part to another.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cooled</ent><br/
<hw>cooled</hw> <pos>adj.</pos> <def>made or become cool or made cool as specified; often used as a combining form; <as>as, air-<ex>cooled</ex> auto engine; the <ex>cooled</ex> milk was put in the refrigerator</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cooler</ent><br/
<hw>Cool"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which cools, or abates heat or excitement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If acid things were used only as <qex>coolers</qex>, they would not be so proper in this case.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything in or by which liquids or other things are cooled, as an ice chest, a vessel for ice water, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An alcoholic beverage containing liquor or wine plus a carbonated beverage, usually served with ice.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>jail; -- usually used in the form <ecol><b>the cooler</b></ecol>.</def> <mark>[slang]</mark> <br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>an air conditioner.</def><br/
[<source>PJC</source>]</p>

<p><ent>coolheaded</ent><br/
<ent>cool-headed</ent><br/
<mhw><hw>cool"-head`ed</hw>, <hw>cool"head*ed</hw></mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a temper not easily excited; free from passion; unshakably calm and collected.</def> -- <wordforms><wf>Cool"-head`ed*ness</wf>, <pos>n.</pos></wordforms><br/
<syn><b>Syn. --</b> cool, imperturbable, unflappable.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Coolie</ent><br/
<hw>Coo"lie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Cooly</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooling</ent><br/
<hw>Cool"ing</hw> <pr>(?)</pr>, <pos>p. a.</pos> <def>Adapted to cool and refresh; allaying heat.</def> <ldquo/The <xex>cooling</xex> brook.<rdquo/  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cooling card</b></col>, <cd>something that dashes hopes.</cd> <mark>[Obs.]</mark> -- <col><b>Cooling time</b></col> <fld>(Law)</fld>, <cd>such a lapse of time as ought, taking all the circumstances of the case in view, to produce a subsiding of passion previously provoked.</cd>  <rj><au>Wharton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coolish</ent><br/
<hw>Cool"ish</hw>, <pos>a.</pos> <def>Somewhat cool.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The nights began to grow a little <qex>coolish</qex>.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coolly</ent><br/
<hw>Cool"ly</hw>, <pos>a.</pos> <def>Coolish; cool.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coolly</ent><br/
<hw>Cool"ly</hw>, <pos>adv.</pos> <def>In a cool manner; without heat or excessive cold; without passion or ardor; calmly; deliberately; with indifference; impudently.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coolness</ent><br/
<hw>Cool"ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of being cool; a moderate degree of cold; a moderate degree, or a want, of passion; want of ardor, zeal, or affection; calmness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Calm impudence; self-possession.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coolung</ent><br/
<hw>Coo"lung</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <fld>(Zool.)</fld> <def>The great gray crane of India (<spn>Grus cinerea</spn>).</def> <altsp>[Also written <asp>coolen</asp> and <asp>cullum</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coolie</ent><br/
<ent>Cooly</ent><br/
<mhw>{ <hw>Coo"ly</hw>, <hw>Coo"lie</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Coolies</plw> <pr>(#)</pr>.</plu> <ety>[Hind. <ets>k<?/l<imac/</ets> a laborer, porter: cf. Turk. <ets>k<?/l</ets>, <ets>ky<?/leh</ets>, slave.]</ety> <def>An East Indian porter or carrier; a laborer transported from the East Indies, China, or Japan, for service in some other country.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coom</ent><br/
<hw>Coom</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>kahm</ets> mold gathered on liquids, D. <ets>kam</ets>, Sw. <ets>kimr<oum/k</ets> pine soot, smoke black, Icel. <ets>k<amac/m</ets> grime, film of dirt.]</ety> <def>Soot; coal dust; refuse matter, as the dirty grease which comes from axle boxes, or the refuse at the mouth of an oven.</def>  <rj><au>Phillips. Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coomb</ent><br/
<hw>Coomb</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cumb</ets> a liquid measure, perh. from LL. <ets>cumba</ets> boat, tomb of stone, fr. Gr. <?/ hollow of a vessel, cup, boat, but cf. G. <ets>kumpf</ets> bowl.]</ety> <def>A dry measure of four bushels, or half a quarter.</def> <altsp>[Written also <asp>comb</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coombe</ent><br/
<ent>Coomb</ent><br/
<mhw>{ <hw>Coomb</hw>, <hw>Coombe</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Comb</er>, <er>Combe</er>, in this sense.]</ety> <def>A hollow in a hillside. <mark>[Prov. Eng.]</mark> See <er>Comb</er>, <er>Combe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coon</ent><br/
<hw>Coon</hw> <pr>(k<oomac/n)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A raccoon. See <er>Raccoon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooncan</ent><br/
<hw>Coon"can</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupt of <ets>conquian</ets>.]</ety> <def>A game of cards derived from conquian, played by two or more players with one or two full packs of cards.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>coondog</ent><br/
<hw>coon"dog`</hw> <pos>n.</pos> <def>any dog trained to hunt raccoons.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coonhound</ent><br/
<hw>coon"hound`</hw> <pos>n.</pos> <def>any of several breeds of hound developed for hunting raccoons.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coonskin</ent><br/
<hw>coonskin</hw> <pos>n.</pos> <def>a cap made from the skin of a raccoon, with the tail hanging down the back; -- called also <ecol><b>coonskin cap</b></ecol>.</def><br/
<syn><b>Syn. --</b> coonskin cap.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coontie</ent><br/
<hw>Coon"tie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A cycadaceous plant of Florida and the West Indies, the <spn>Zamia integrifolia</spn>, from the stems of which a kind of sago is prepared.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>co-op</ent><br/
<hw>co"-op</hw> <pr>(k<omac/"<ocr/p)</pr>, <pos>n.</pos> <ety>[ contraction of <ets>co-operative enterprise</ets>.]</ety> <def>a commercial enterprise run for the benefit of its owners.</def><br/
<syn><b>Syn. --</b> cooperative, coop.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coop</ent><br/
<hw>Coop</hw> <pr>(k<oomac/p)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>cypa</ets> a measure, D. <ets>kuip</ets> tub, Icel. <ets>kupa</ets> bowl, G. <ets>kufe</ets> coop tub; all fr. L. <ets>cupa</ets> vat, tub, LL. <ets>cupa</ets>, <ets>copa</ets>, cup. See <er>Cup</er>, and cf. <er>Keeve</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A barrel or cask for liquor.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inclosure for keeping small animals; a pen; especially, a grated box for confining poultry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A cart made close with boards; a tumbrel.</def> <mark>[Scotch]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coop</ent><br/
<hw>Coop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cooped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cooping</conjf>.]</vmorph> <def>To confine in a coop; hence, to shut up or confine in a narrow compass; to cramp; -- usually followed by <xex>up</xex>, sometimes by <xex>in</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Trojans <qex>cooped</qex> within their walls so long.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The contempt of all other knowledge . . . <qex>coops</qex> the understanding up within narrow bounds.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To work upon in the manner of a cooper.</def> <mark>[Obs.]</mark> <ldquo/Shaken tubs . . . be new <xex>cooped</xex>.<rdquo/  <rj><au>Holland.</au></rj></p>

<p><syn><b>Syn.</b> -- To crowd; confine; imprison.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coopee</ent><br/
<hw>Coo*pee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Coupe</er>.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooper</ent><br/
<hw>Coop"er</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[From <er>Coop</er>.]</ety> <def>One who makes barrels, hogsheads, casks, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooper</ent><br/
<hw>Coop"er</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coopered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coopering</conjf>.]</vmorph> <def>To do the work of a cooper upon; <as>as, to <ex>cooper</ex> a cask or barrel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooperage</ent><br/
<hw>Coop"er*age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Work done by a cooper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The price paid for coopers' work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A place where coopers' work is done.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooperant</ent><br/
<hw>Co*<oum/p"er*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>coop<eacute/rant</ets>.]</ety> <def>Operating together; <as>as, <ex>cooperant</ex> forces</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooperate</ent><br/
<hw>Co*<oum/p"er*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cooperated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cooperating</conjf>.]</vmorph> <ety>[L. <ets>cooperatus</ets>, p. p. of <ets>cooperari</ets> to cooperate; <ets>co + operari</ets> to work, <ets>opus</ets> work. See <er>Operate</er>.]</ety> <def>To act or operate jointly with another or others; to concur in action, effort, or effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whate'er <qex>cooperates</qex> to the common mirth.</q> <rj><qau>Crashaw.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooperation</ent><br/
<hw>Co*<oum/p`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cooperatio</ets>: cf. F. <ets>coop<eacute/ration</ets>.]</ety> <sn>1.</sn> <def>The act of cooperating, or of operating together to one end; joint operation; concurrent effort or labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not holpen by the <qex>cooperation</qex> of angels.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Polit. Econ.)</fld> <def>The association of a number of persons for their benefit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooperative</ent><br/
<hw>Co*<oum/p"er*a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Operating jointly to the same end.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cooperative society</b></col>, <cd>a society established on the principle of a joint-stock association, for the production of commodities, or their purchase and distribution for consumption, or for the borrowing and lending of capital among its members.</cd> -- <col><b>Cooperative store</b></col>, <cd>a store established by a cooperative society, where the members make their purchases and share in the profits or losses.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooperator</ent><br/
<hw>Co*<oum/p"er*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>coop<eacute/rateur</ets>.]</ety> <def>One who labors jointly with others to promote the same end.</def> <ldquo/<xex>Cooperators</xex> with the truth.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coopering</ent><br/
<hw>Coop"er*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work done by a cooper in making or repairing barrels, casks, etc.; the business of a cooper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coopery</ent><br/
<hw>Coop"er*y</hw>, <pos>a.</pos> <def>Relating to a cooper; coopered.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Coopery</qex> vessels made of wood.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coopery</ent><br/
<hw>Coop"er*y</hw>, <pos>n.</pos> <def>The occupation of a cooper.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coopt</ent><br/
<ent>co-opt</ent><br/
<mhw><hw>co-opt"</hw>, <hw>co*<oum/pt"</hw></mhw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Cooptate</er>. Cf. F. <ets>coopter</ets>.]</ety> <def>To choose or elect in concert with another.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Each of the hundred was to <qex>coopt</qex> three others.</q> <rj><qau>Jowett (Thucyd.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To choose or elect as a colleague or fellow member of a group; <as>as, The church members <ex>co-opted</ex> individuals from similar backgrounds to replenish the congregation</as>.</def><br/
[<source>PJC</source> + <source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>To assimilate (a smaller group) into a larger group.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>To persuade an opponent to join one's own side.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>To appoint summarily (with or without the appointee's consent).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>6.</sn> <def>To appropriate (something rightly belonging to another) as one's own; to preempt; <as>as, to <ex>co-opt</ex> someone's name</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cooptate</ent><br/
<hw>Co*<oum/p"tate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>cooptatus</ets>, p. p. of <ets>cootare</ets> to elect to something; <ets>co-</ets> + <ets>optare</ets> to choose.]</ety> <def>To choose; to elect; to coopt.</def> <mark>[Obs.]</mark>  <rj><au>Cockeram.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cooptation</ent><br/
<ent>co-optation</ent><br/
<mhw><hw>co-optation</hw>, <hw>co`<oum/p*ta"tion</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>cooptatio</ets>.]</ety> <def>The act of choosing; selection; choice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The first election and <qex>cooptation</qex> of a friend.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the selection of a new member (usually by a vote of the existing membership).  See co-opt{2}.</def><br/
<syn><b>Syn. --</b> co-option.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>the act of appointing summarily (with or without the appointee's consent).</def><br/
<syn><b>Syn. --</b> co-option.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>co-option</ent><br/
<hw>co-option</hw> <pos>n.</pos> <def>same as <er>co-optation</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coordain</ent><br/
<hw>Co`<oum/r*dain</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To ordain or appoint for some purpose along with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coordinance</ent><br/
<hw>Co*<oum/r"di*nance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Joint ordinance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coordinate</ent><br/
<hw>Co*<oum/r"di*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>co-</ets> + L. <ets>ordinatus</ets>, p. p. of <ets>ordinare</ets> to regulate. See <er>Ordain</er>.]</ety> <def>Equal in rank or order; not subordinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether there was one Supreme Governor of the world, or many <qex>coordinate</qex> powers presiding over each country.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Conjunctions joint sentences and <qex>coordinate</qex> terms.</q> <rj><qau>Rev. R. Morris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coordinate adjectives</b></col>, <cd>adjectives disconnected as regards one another, but referring equally to the same subject.</cd> -- <col><b>Coordinate conjunctions</b></col>, <cd>conjunctions joining independent propositions.</cd>  <rj><au>Rev. R. Morris.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>coordinate</ent><br/
<ent>co-ordinate</ent><br/
<mhw><hw>co-ordinate</hw>, <hw>co*<oum/r"di*nate</hw></mhw> <pr>(-n<amac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coordinated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coordinating</conjf>.]</vmorph> <sn>1.</sn> <def>To make coordinate; to put in the same order or rank; <as>as, to <ex>coordinate</ex> ideas in classification</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give a common action, movement, or condition to; to regulate and combine so as to produce harmonious action; to adjust; to harmonize; <as>as, to <ex>coordinate</ex> muscular movements</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>to be co-ordinated; <as>as, These activities <ex>co-ordinate</ex> well</as>.</def><br/
<syn><b>Syn. --</b> coordinate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coordinate</ent><br/
<hw>Co*<oum/r"di*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A thing of the same rank with another thing; one two or more persons or things of equal rank, authority, or importance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It has neither <qex>coordinate</qex> nor analogon; it is absolutely one.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Math.)</fld> <def>Lines, or other elements of reference, by means of which the position of any point, as of a curve, is defined with respect to certain fixed lines, or planes, called <xex>coordinate axes</xex> and <xex>coordinate planes</xex>. See <er>Abscissa</er>.</def>
<-- this note refers to an accompanying diagram -->
<note><hand/ <xex>Coordinates</xex> are of several kinds, consisting in some of the different cases, of the following elements, namely: <sd>(a)</sd> <fld>(Geom. of Two Dimensions)</fld> The abscissa and ordinate of any point, taken together; as the abscissa PY and ordinate PX of the point P (Fig. 2, referred to the coordinate axes AY and AX. <sd>(b)</sd> Any radius vector PA (Fig. 1), together with its angle of inclination to a fixed line, APX, by which any point A in the same plane is referred to that fixed line, and a fixed point in it, called the <xex>pole</xex>, P. <sd>(c)</sd> <fld>(Geom. of Three Dimensions)</fld> Any three lines, or distances, PB, PC, PD (Fig. 3), taken parallel to three coordinate axes, AX, AY, AZ, and measured from the corresponding coordinate fixed planes, YAZ, XAZ, XAY, to any point in space, P, whose position is thereby determined with respect to these planes and axes. <sd>(d)</sd> A radius vector, the angle which it makes with a fixed plane, and the angle which its projection on the plane makes with a fixed line line in the plane, by which means any point in space at the free extremity of the radius vector is referred to that fixed plane and fixed line, and a fixed point in that line, the pole of the radius vector.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cartesian coordinates</b></col>. <cd>See under <er>Cartesian</er>.</cd> -- <col><b>Geographical coordinates</b></col>, <cd>the latitude and longitude of a place, by which its relative situation on the globe is known. The height of the above the sea level constitutes a third coordinate.</cd> -- <col><b>Polar coordinates</b></col>, <cd>coordinates made up of a radius vector and its angle of inclination to another line, or a line and plane; as those defined in <sd>(b)</sd> and <sd>(d)</sd> above.</cd> -- <col><b>Rectangular coordinates</b></col>, <cd>coordinates the axes of which intersect at right angles.</cd> -- <col><b>Rectilinear coordinates</b></col>, <cd>coordinates made up of right lines. Those defined in <sd>(a)</sd> and <sd>(c)</sd> above are called also <stype>Cartesian coordinates</stype>.</cd> -- <mcol><col><b>Trigonometrical coordinates</b></col> <it>or</it> <col><b>Spherical coordinates</b></col></mcol>, <cd>elements of reference, by means of which the position of a point on the surface of a sphere may be determined with respect to two great circles of the sphere.</cd> -- <col><b>Trilinear coordinates</b></col>, <cd>coordinates of a point in a plane, consisting of the three ratios which the three distances of the point from three fixed lines have one to another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>coordinated</ent><br/
<hw>coordinated</hw> <pos>adj.</pos> <sn>1.</sn> <def>dexterous in the use of more than one set of muscle movements.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><q>She was usually good with her hands and well <qex>coordinated</qex>.</q> <qau>Mary McCarthy</qau></p>

<p><sn>2.</sn>  <def>matched in color and pattern so as to be pleasing to the esthetic sense; <as>as, The curtains and walls were color <ex>coordinated</ex></as>.</def><br/
<syn><b>Syn. --</b> matching.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>operating as a unit; <as>as, a <ex>coordinated</ex> development plan</as>.</def><br/
<syn><b>Syn. --</b> coordinated, concerted, interconnected, unified.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>co-ordinating</ent><br/
<ent>coordinating</ent><br/
<mhw><hw>coordinating</hw> <hw>co-ordinating</hw></mhw> <pos>adj.</pos>  <fld>(Grammar)</fld> <def>joining together words or phrases of equal grammatical rank.</def> <ant>subordinative</ant><br/
<syn><b>Syn. --</b> coordinative.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coordinative</ent><br/
<hw>coordinative</hw> <pos>adj.</pos> <fld>(Grammar)</fld> <def>same as <er>coordinating</er>.</def> <ant>subordinative</ant><br/
<syn><b>Syn. --</b> co-ordinating, coordinating.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coordinately</ent><br/
<hw>Co*<oum/r"di*nate*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a coordinate manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coordinateness</ent><br/
<hw>Co*<oum/r"di*nate*ness</hw>, <pos>n.</pos> <def>The state of being coordinate; equality of rank or authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coordination</ent><br/
<hw>Co*<oum/r`di*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of coordinating; the act of putting in the same order, class, rank, dignity, etc.; <as>as, the <ex>coordination</ex> of the executive, the legislative, and the judicial authority in forming a government</as>; the act of regulating and combining so as to produce harmonious results; harmonious adjustment; <as>as, a <ex>coordination</ex> of functions</as>.</def> <ldquo/<xex>Coordination</xex> of muscular movement by the cerebellum.<rdquo/  <rj><au>Carpenter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being coordinate, or of equal rank, dignity, power, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In this high court of parliament, there is a rare <qex>coordination</qex> of power.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coordinative</ent><br/
<hw>Co*<oum/r"di*na*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Gram.)</fld> <def>Expressing coordination.</def>  <rj><au>J. W. Gibbs.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coot</ent><br/
<hw>Coot</hw> <pr>(k<oomac/t)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>koet</ets>, W. <ets>cwtair</ets>; <ets>cwta</ets> short, bodtailed + <ets>iar</ets> hen; cf. <ets>cwtau</ets> to dock. Cf. <er>Cut</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A wading bird with lobate toes, of the genus <gen>Fulica</gen>.  The common European or bald coot is <spn>Fulica atra</spn> (see under <er>bald</er>); the American is <spn>Fulica Americana</spn></def>.  <sd>(b)</sd> <def>The surf duck or scoter. In the United States all the species of (<gen><OE/demia</gen> are called coots. See <er>Scoter</er>.</def> <ldquo/As simple as a <xex>coot</xex>.<rdquo/  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stupid fellow; a simpleton; <as>as, a silly <ex>coot</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cooter</ent><br/
<hw>Coot"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A fresh-water tortoise (<spn>Pseudemus concinna</spn>) of Florida.</def> <sd>(b)</sd> <def>The box tortoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cootfoot</ent><br/
<hw>Coot"foot`</hw> <pr>(-f<oocr/t`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The phalarope; -- so called because its toes are like the coot's.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coothay</ent><br/
<hw>Coo*thay"</hw> <pr>(k<oomac/*th<amac/")</pr>, <pos>n.</pos><def>A striped satin made in India.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cop</ent><br/
<hw>Cop</hw> <pr>(k<ocr/p)</pr>, <pos>n.</pos> <ety>[AS. <ets>cop</ets>; cf. G. <ets>kopf</ets> head. Cf. <er>Cup</er>, <er>Cob</er>.]</ety> <sn>1.</sn> <def>The top of a thing; the head; a crest.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cop</qex> they used to call<br/
The tops of many hills.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A conical or conical-ended mass of coiled thread, yarn, or roving, wound upon a spindle, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A tube or quill upon which silk is wound.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil. Arch.)</fld> <def>Same as <er>Merlon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A policeman.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cop waste</b></col>, <cd>a kind of cotton waste, composed chiefly of remnants of cops from which the greater part of the yarn has been unwound.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>copasetic</ent><br/
<ent>copacetic</ent><br/
<mhw><hw>copacetic</hw> <hw>copasetic</hw></mhw> <pos>adj.</pos> <sn>1.</sn> <def>completely satisfactory.</def><br/
<syn><b>Syn. --</b> copesetic, copesettic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><q>You had to be a good judge of what a man was like, and the English was <qex>copacetic</qex>.</q> <qau>John O'Hara</qau></p>

<p><ent>Copaiva</ent><br/
<ent>Copaiba</ent><br/
<mhw>{ <hw>Co*pai"ba</hw> <pr>(?; 277)</pr>, <hw>Co*pai"va</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Sp. & Pg., fr. Brazil. <ets>cupa<uacute/ba</ets>.]</ety> <fld>(Med.)</fld> <def>A more or less viscid, yellowish liquid, the bitter oleoresin of several species of <gen>Copaifera</gen>, a genus of trees growing in South America and the West Indies. It is stimulant and diuretic, and was formerly much used in affections of the mucous membranes.  It is also used in varnishes and lacquers, and in cleaning oil paintings. -- called also <altname>balsam of copaiba</altname>, <altname>copaiba balsam</altname>, <altname>balsam capivi</altname>, and <altname>Jesuits' resin</altname>.</def> <altsp>[Written also <asp>capivi</asp>.]</altsp><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Copal</ent><br/
<hw>Co"pal</hw> <pr>(k<omac/"p<ait/l; 277)</pr>, <ety>[Sp., fr. Mexican <ets>copalli</ets>, a generic name of resins. <au>Clavigero</au>.]</ety> <def>A resinous substance flowing spontaneously from trees of Zanzibar, Madagascar, and South America (<spn>Trachylobium Hornemannianum</spn>, <spn>Trachylobium verrucosum</spn>, and <spn>Hymen<ae/a Courbaril</spn>), and dug from earth where forests have stood in Africa; -- used chiefly in making varnishes.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 321 --></p>

<p><ent>Copalm</ent><br/
<hw>Co"palm`</hw> <pr>(k<omac/"p<aum/m`)</pr>, <pos>n.</pos> <def>The yellowish, fragrant balsam yielded by the sweet gum; also, the tree itself.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coparcenary</ent><br/
<hw>Co*par"ce*na*ry</hw> <pr>(k<osl/*p<aum/r"s<esl/*n<asl/*r<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Coparcenaries</plw> <pr>(-r<icr/z)</pr>.</plu> <ety>[Pref. <ets>co-</ets> + <ets>parcenary</ets>]</ety> <fld>(Law)</fld> <def>Partnership in inheritance; joint heirship; joint right of succession to an inheritance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coparcener</ent><br/
<hw>Co*par"ce*ner</hw> <pr>(-n<etil/r)</pr>, <pos>n.</pos> <ety>[Pref. <ets>co-</ets> + <ets>parcener</ets>.]</ety> <fld>(Law)</fld> <def>One who has an equal portion with others of an inheritance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the <qex>coparceners</qex> together make but one heir, and have but one estate among them.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coparceny</ent><br/
<hw>Co*par"ce*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. of <er>Coparcenary</er>.]</ety> <fld>(Law)</fld> <def>An equal share of an inheritance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copart</ent><br/
<hw>Co*part</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Compart</er>]</ety> <def>To share.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For, of all miserias, I hold that chief<br/
Wretched to be, when none <qex>coparts</qex> our grief.</q> <rj><qau>Webster (1661).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copartment</ent><br/
<hw>Co*part"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A compartment.</def> <mark>[Obs.]</mark>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copartner</ent><br/
<hw>Co*part"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is jointly concerned with one or more persons in business, etc.; a partner; an associate; a partaker; a sharer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>the associates and <qex>copartners</qex> of our loss.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copartnership</ent><br/
<hw>Co*part"ner*ship</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of being a copartner or of having a joint interest in any matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A partnership or firm; <as>as, A</as>. and B. have this day formed a <xex>copartnership</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copartnery</ent><br/
<hw>Co*part"ner*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Copartneries</plw> <pr>(<?/)</pr>.</plu> <def>the state of being copartners in any undertaking.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copatain</ent><br/
<hw>Cop"a*tain</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Formed fr. <ets>cop</ets>, in imitation of <ets>captain</ets>. See <er>Cop</er>, <er>Captain</er>.]</ety> <def>Having a high crown, or a point or peak at top.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>copatain</qex> hat made on a Flemish block.</q> <rj><qau>Gascoigne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copatriot</ent><br/
<hw>Co*pa"tri*ot</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A joint patriot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cope</ent><br/
<hw>Cope</hw> <pr>(k<omac/p)</pr>, <pos>n.</pos> <ety>[A doublet of <ets>cape</ets>. See <er>Cape</er>, <er>Cap</er>.]</ety> <sn>1.</sn> <def>A covering for the head.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything regarded as extended over the head, as the arch or concave of the sky, the roof of a house, the arch over a door.</def> <ldquo/The starry <xex>cope</xex> of heaven.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An ecclesiastical vestment or cloak, semicircular in form, reaching from the shoulders nearly to the feet, and open in front except at the top, where it is united by a band or clasp. It is worn in processions and on some other occasions.</def>  <rj><au>Piers plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A hundred and sixty priests all in their <qex>copes</qex>.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An ancient tribute due to the lord of the soil, out of the lead mines in Derbyshire, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Founding)</fld> <def>The top part of a flask or mold; the outer part of a loam mold.</def>  <rj><au>Knight. De Colange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cope</ent><br/
<hw>Cope</hw>, <pos>v. i.</pos> <def>To form a cope or arch; to bend or arch; to bow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some bending down and <qex>coping</qex> toward the earth.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cope</ent><br/
<hw>Cope</hw>, <pos>v. t.</pos> <fld>(Falconry)</fld> <def>To pare the beak or talons of (a hawk).</def>  <rj><au>J. H. Walsh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cope</ent><br/
<hw>Cope</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coped</conjf> <pr>(k<omac/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coping</conjf>.]</vmorph> <ety>[OE. <ets>copen</ets>, <ets>coupen</ets>, to buy, bargain, prob. from D. <ets>koopen</ets> to buy, orig., to bargain. See <er>Cheap</er>.]</ety> <sn>1.</sn> <def>To exchange or barter.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To encounter; to meet; to have to do with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Horatio, thou art e'en as just a man<br/
As e'er my conversation <qex>coped</qex> withal.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To enter into or maintain a hostile contest; to struggle; to combat; especially, to strive or contend on equal terms or with success; to match; to equal; -- usually followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Host <qex>coped</qex> with host, dire was the din of war.</q> <rj><qau>Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their generals have not been able to <qex>cope</qex> with the troops of Athens.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cope</ent><br/
<hw>Cope</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To bargain for; to buy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make return for; to requite; to repay.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>three thousand ducats due unto the Jew,<br/
We freely <qex>cope</qex> your courteous pains withal.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To match one's self against; to meet; to encounter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I love to <qex>cope</qex> him in these sullen fits.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They say he yesterday <qex>coped</qex> Hector in the battle, and struck him down.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cope-chisel</ent><br/
<hw>Cope"-chis`el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A narrow chisel adapted for cutting a groove.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copeck</ent><br/
<hw>Co"peck</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>kopeika</ets>]</ety> <def>A Russian copper coin. See <er>Kopeck</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coped</ent><br/
<hw>Coped</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Clad in a cope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copelata</ent><br/
||<hw>Cop`e*la"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a rower.]</ety> <fld>(Zool.)</fld> <def>See <er>Larvalla</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copeman</ent><br/
<hw>Cope"man</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>koopman</ets>, fr. <ets>koopen</ets> to buy. See <er>Cope</er>, <pos>v. i.</pos> <er>Chapman</er>.]</ety> <def>A chapman; a dealer; a merchant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He would have sold his part of paradise<br/
For ready money, had he met a <qex>copeman</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copenhagen</ent><br/
<hw>Copenhagen</hw> <pos>prop. n.</pos> <fld>(Geography)</fld> <def>The capital <isa>city</isa> of <country>Denmark</country>.  Population (2000) = 1,339,395.</def><br/
[<source>PJC</source>]</p>

<p><ent>Copenhagen</ent><br/
<hw>Co`pen*ha"gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <etsep>Copenhagen</etsep>, Denmark.]</ety> <sn>1.</sn>  <def>A sweetened hot drink of spirit and beaten eggs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A children's game in which one player is inclosed by a circle of others holding a rope.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Copepod</ent><br/
<hw>Cop"e*pod</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Copepoda.</def> -- <def2><pos>n.</pos>  <def>One of the Copepoda.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copepoda</ent><br/
||<hw>Co*pep"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ an oar + <ets>-poda</ets>.]</ety> <fld>(Zool.)</fld> <def>An order of Entomostraca, including many minute Crustacea, both fresh-water and marine.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They have a distinct carapace. The eggs are carried in a pair of external pouches. Some are parasites of fishes.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copernican</ent><br/
<hw>Co*per"ni*can</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to Copernicus, a Prussian by birth (<it>b.</it> 1473, <it>d.</it> 1543), who taught the world the solar system now received, called the <xex>Copernican</xex> system.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>copesettic</ent><br/
<ent>copesetic</ent><br/
<mhw><hw>copesetic</hw> <hw>copesettic</hw></mhw> <pos>adj.</pos> <sn>1.</sn>  <def>same as <er>copacetic</er>.</def><br/
<syn><b>Syn. --</b> copacetic, copasetic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Copesmate</ent><br/
<hw>Copes"mate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An associate or companion; a friend; a partner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Misshapen time, <qex>copesmate</qex> of ugly Night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copestone</ent><br/
<hw>Cope"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A stone for coping. See <er>Coping</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>copied</ent><br/
<hw>copied</hw> <pos>adj.</pos> <def>reproducing the features of an original especially in an inferior manner.</def> <ant>original</ant><br/
<syn><b>Syn. --</b> imitative.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Copier</ent><br/
<hw>Cop"i*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From. <er>Copy</er>.]</ety> <sn>1.</sn> <def>One who copies; one who writes or transcribes from an original; a transcriber.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An imitator; one who imitates an example; hence, a plagiarist.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>a machine that makes copies of documents, especially by xerography; a photocopier; <as>as, to run twenty copies off on the <ex>copier</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coping</ent><br/
<hw>Cop"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cope</er>, <pos>n.</pos>]</ety> <fld>(Arch.)</fld> <def>The highest or covering course of masonry in a wall, often with sloping edges to carry off water; -- sometimes called <altname>capping</altname>.</def>  <rj><au>Gwill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copious</ent><br/
<hw>Co"pi*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>copiosus</ets>, fr. <ets>copia</ets> abundance: cf. F. <ets>copieux</ets>. See <er>Copy</er>, <er>Opulent</er>.]</ety> <def>Large in quantity or amount; plentiful; abundant; fruitful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Kindly pours its <qex>copious</qex> treasures forth.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hail, Son of God, Savior of men! thy name<br/
Shall be the <qex>copious</qex> matter of my song.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> -- Ample; abundant; plentiful; plenteous; rich; full; exuberant; overflowing; full. See <er>Ample</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copiously</ent><br/
<hw>Co"pi*ous*ly</hw>, <pos>adv.</pos> <def>In a copious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copiousness</ent><br/
<hw>Co"pi*ous*ness</hw>, <pos>n.</pos> <def>The state or quality of being copious; abudance; plenty; also, diffuseness in style.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To imitatethe <qex>copiousness</qex> of Homer.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Abudance; plenty; richness; exuberance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copist</ent><br/
<hw>Cop"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>copiste</ets>. See <er>Copy</er>.]</ety> <def>A copier.</def> <mark>[Obs.]</mark> <ldquo/A <xex>copist</xex> after nature.<rdquo/  <rj><au>Shaftesbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coplanar</ent><br/
<hw>Co*plan"ar</hw> <pr>(k<osl/*pl<amac/n"<etil/r)</pr>, <pos>a.</pos> <ety>[Pref. <ets>co-</ets> + <ets>plane</ets>.]</ety> <fld>(Math.)</fld> <def>Situated in one plane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copland</ent><br/
<hw>Cop"land`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Cop</ets> + <ets>land</ets>.]</ety> <def>A piece of ground terminating in a point or acute angle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coportion</ent><br/
<hw>Co*por"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Equal share.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Myself will bear . . . <qex>coportion</qex> of your pack.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copped</ent><br/
<hw>Copped</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Cop</er>.]</ety> <def>Rising to a point or head; conical; pointed; crested.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppel</ent><br/
<hw>Cop"pel</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Cupel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper</ent><br/
<hw>Cop"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>coper</ets> (cf. D. <ets>koper</ets>, Sw. <ets>koppar</ets>, Dan. <ets>kobber</ets>, G. <ets>kupfer</ets>), LL. <ets>cuper</ets>, fr. L. <ets>cuprum</ets> for earlier <ets>Cyprium</ets>, <ets>Cyprium aes</ets>, i.e., Cyprian brass, fr. Gr. <?/ of Cyprus (Gr. <?/), anciently renowned for its copper mines. Cf. <er>Cypreous</er>.]</ety> <sn>1.</sn> <def>A common metal of a reddish color, both ductile and malleable, and very tenacious. It is one of the best conductors of heat and electricity. Symbol Cu. Atomic weight 63.3. It is one of the most useful metals in itself, and also in its alloys, brass and bronze.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Copper is the only metal which occurs native abundantly in large masses; it is found also in various ores, of which the most important are chalcopyrite, chalcocite, cuprite, and malachite. Copper mixed with tin forms bell metal; with a smaller proportion, bronze; and with zinc, it forms brass, pinchbeck, and other alloys.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coin made of copper; a penny, cent, or other minor coin of copper.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>My friends filled my pockets with <qex>coppers</qex>.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A vessel, especially a large boiler, made of copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> Specifically <fld>(Naut.)</fld>, <def>the boilers in the galley for cooking; <as>as, a ship's <ex>coppers</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Copper</xex> is often used adjectively, commonly in the sense of <xex>made</xex> or <xex>consisting of copper</xex>, or <xex>resembling copper</xex>; as, a <xex>copper</xex> boiler, tube, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>All in a hot and <qex>copper</qex> sky.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>It is sometimes written in combination; as, <xex>copper</xex>plate, <xex>copper</xex>smith, <xex>copper</xex>-colored.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Copper finch</b></col>. <fld>(Zool.)</fld> <cd>See <er>Chaffinch</er>.</cd> -- <mcol><col><b>Copper glance</b></col>, <it>or</it> <col><b>Vitreous copper</b></col></mcol>. <fld>(Min.)</fld> <cd>See <er>Chalcocite</er>.</cd> -- <col><b>Indigo copper</b></col>. <fld>(Min.)</fld> <cd>See <er>Covelline</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper</ent><br/
<hw>Cop"per</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coppered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coppering</conjf>.]</vmorph> <def>To cover or coat with copper; to sheathe with sheets of copper; <as>as, to <ex>copper</ex> a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copperas</ent><br/
<hw>Cop"per*as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>coperose</ets>, F. <ets>couperose</ets>, fr. (assumed?) L. <ets>cuprirosa</ets>, equiv. to G. <grk>cha`lkanqos</grk>, i. e. copper flower, vitriol.  See <er>Copper</er> and <er>Rose.</er>]</ety> <def>Green vitriol, or sulphate of iron; a green crystalline substance, of an astringent taste, used in making ink, in dyeing black, as a tonic in medicine, etc. It is made on a large scale by the oxidation of iron pyrites. Called also <altname>ferrous sulphate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>copperas</xex> was formerly synonymous with <xex>vitriol</xex>, and included the green, blue, and white vitriols, or the sulphates of iron, copper, and zinc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>copperbottom</ent><br/
<ent>copper-bottom</ent><br/
<mhw><hw>copper-bottom</hw>, <hw>copperbottom</hw></mhw> <pos>v. t.</pos> <def>to provide with a copper bottom.</def><br/
<syn><b>Syn. --</b> copperbottom.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to bind with a cord, tie with a cord.</def><br/
<syn><b>Syn. --</b> cord up.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Copper-bottomed</ent><br/
<hw>Cop"per-bot`tomed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a bottom made of copper, as a tin boiler or other vessel, or sheathed with copper, as a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper-faced</ent><br/
<hw>Cop"per-faced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Faced or covered with copper; <as>as, <ex>copper-faced</ex> type</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper-fastened</ent><br/
<hw>Cop"per-fas`tened</hw> <pr>(?)</pr>, <pos>a.</pos><def>Fastened with copper bolts, as the planks of ships, etc.; <as>as, a <ex>copper-fastened</ex> ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copperhead</ent><br/
<hw>Cop"per*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From its color.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A poisonous American serpent (<spn>Ancistrodon conotortrix</spn>), closely allied to the rattlesnake, but without rattles; -- called also <altname>copper-belly</altname>, and <altname>red viper</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A nickname applied to a person in the Northern States who sympathized with the South during the Civil War.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppering</ent><br/
<hw>Cop"per*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of covering with copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An envelope or covering of copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copperish</ent><br/
<hw>Cop"per*ish</hw>, <pos>a.</pos> <def>Containing, or partaking of the nature of, copper; like copper; <as>as, a <ex>copperish</ex> taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper-nickel</ent><br/
<hw>Cop"per-nick`el</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Niccolite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper-nose</ent><br/
<hw>Cop"per-nose</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A red nose.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copperplate</ent><br/
<hw>Cop"per*plate`</hw> <pr>(k<ocr/p"p<etil/r*pl<amac/t`)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>A plate of polished copper on which a design or writing is engraved.</def> <sd>(b)</sd> <def>An impression on paper taken from such a plate.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In printing from a copper- or steel plate the lines are filled with ink, the surface of the plate is wiped clean, the paper laid upon it, and the impression taken by pressing it under the roller of a plate press.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Copperplate press</b></col>. <cd>See <cref>Plate press</cref>, under <er>Plate</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppersmith</ent><br/
<hw>Cop"per*smith`</hw> <pr>(-sm<icr/th`)</pr>, <pos>n.</pos> <def>One whose occupation is to manufacture copper utensils; a worker in copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copper works</ent><br/
<hw>Cop"per works`</hw> <pr>(?)</pr>. <def>A place where copper is wrought or manufactured.</def>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copperworm</ent><br/
<hw>Cop"per*worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The teredo; -- so called because it injures the bottoms of vessels, where not protected by copper.</def> <sd>(b)</sd> <def>The ringworm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppery</ent><br/
<hw>Cop"per*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Mixed with copper; containing copper, or made of copper; like copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppice</ent><br/
<hw>Cop"pice</hw> <pr>(k<ocr/p"p<icr/s)</pr>, <pos>n.</pos> <ety>[OF. <ets>copeiz</ets>, fr. <ets>coper</ets>, <ets>couper</ets>, to cut, F. <ets>couper</ets>, fr. <ets>cop</ets>, <ets>coup</ets>, <ets>colp</ets>, a blow, F. <ets>coup</ets>, L. <ets>colaphus</ets>, fr. Gr. <grk>ko`lafos</grk>. Cf. <er>Copse</er>, and cf. <er>Coup<eacute/</er>, <er>Coupee</er>.]</ety> <def>A grove of small growth; a thicket of brushwood; a wood cut at certain times for fuel or other purposes. See <er>Copse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rate of <qex>coppice</qex> lands will fall, upon the discovery of coal mines.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppice</ent><br/
<hw>Cop"pice</hw> <pr>(k<ocr/p"p<icr/s)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coppiced</conjf> <pr>(k<ocr/p"p<icr/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coppicing</conjf> <pr>(k<ocr/p"p<icr/*s<icr/ng)</pr>.]</vmorph> <fld>(Forestry)</fld> <def>To cause to grow in the form of a coppice; to cut back (as young timber) so as to produce shoots from stools or roots.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coppin</ent><br/
<hw>Cop"pin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cop</er>.]</ety> <def>A cop of thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copple</ent><br/
<hw>Cop"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A dim. of <er>Cop</er>.]</ety> <def>Something rising in a conical shape; specifically, a hill rising to a point.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A low cape, and upon it a <qex>copple</qex> not very high.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copple-crown</ent><br/
<hw>Cop"ple-crown</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A created or high-topped crown or head.</def> <ldquo/Like the <xex>copple-crown</xex> the lapwing has.<rdquo/  <rj><au>T. Randolph.</au></rj></p>

<p>-- <wordforms><wf>Cop"ple-crowned`</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coppled</ent><br/
<hw>Cop"pled</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Copple</er>.]</ety> <def>Rising to a point; conical; copped.</def> <mark>[Obs.]</mark>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copple dust</ent><br/
<hw>Cop"ple dust`</hw> <pr>(?)</pr>. <def>Cupel dust.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Powder of steel, or <qex>copple dust</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copplestone</ent><br/
<hw>Cop"ple*stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cobblestone.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copps</ent><br/
<hw>Copps</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Copse</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copra</ent><br/
<hw>Co"pra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay<aacute/lam <ets>koppara</ets> or Hind. <ets>khopr<amac/</ets>.]</ety> <fld>(Com.)</fld> <def>The dried meat of the cocoanut, from which cocoanut oil is expressed.</def> <altsp>[Written also <asp>cobra</asp>, <asp>copperah</asp>, <asp>coppra</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coprolite</ent><br/
<hw>Cop"ro*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`pros</grk> dung + <ets>-lite</ets>.]</ety> <fld>(Paleon.)</fld> <def>A piece of petrified dung; a fossil excrement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coprolitic</ent><br/
<hw>Cop`ro*lit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing, pertaining to, or of the nature of, coprolites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coprophagan</ent><br/
<hw>Co*proph"a*gan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Coprophagous</er>.]</ety> <fld>(Zool.)</fld> <def>A kind of beetle which feeds upon dung.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coprophagous</ent><br/
<hw>Co*proph"a*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ exrement + <?/ to eat.]</ety> <fld>(Zool.)</fld> <def>Feeding upon dung, as certain insects.</def></p>

<p><ent>Cop-rose</ent><br/
<hw>Cop-rose`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>coprose</ets>, of uncertain origin; cf. D. <ets>klaproos</ets>, <ets>klapperroos</ets>.]</ety> <def>The red, or corn, poppy.</def> <altsp>[Written also <asp>cup-rose</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cops</ent><br/
<hw>Cops</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>cops</ets>, <ets>cosp</ets>, fetter.]</ety> <def>The connecting crook of a harrow.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copse</ent><br/
<hw>Copse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. from <ets>coppice</ets>.]</ety> <def>A wood of small growth; a thicket of brushwood. See <er>Coppice</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Near yonder <qex>copse</qex> where once the garden smiled.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copse</ent><br/
<hw>Copse</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To trim or cut; -- said of small trees, brushwood, tufts of grass, etc.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To plant and preserve, as a copse.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copsewood</ent><br/
<hw>Copse"wood</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Brushwood; coppice.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copsy</ent><br/
<hw>Cops"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterized by copses.</def>  <ldquo/<xex>Copsy</xex> villages.<rdquo/  <ldquo/<xex>Copsy</xex> banks.<rdquo/  <rj><au>J. Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coptic</ent><br/
<hw>Cop"tic</hw> <pr>(k<ocr/p"t<icr/k)</pr>, <pos>a.</pos> <ety>[Abbrev. from L. <ets>Aegyptius</ets> an Egyptian, Gr. <?/, Ar. <ets>kibt<imac/</ets>, pl. <ets>kibt</ets>.]</ety> <def>Of or pertaining to the Copts.</def> -- <def2><pos>n.</pos>  <def>The language of the Copts.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coptic Church</ent><br/
<hw>Coptic Church</hw>. <def>The native church of Egypt or church of Alexandria, which in general organization and doctrines resembles the Roman Catholic Church, except that it holds to the Monophysitic doctrine which was condemned (<sc>a. d.</sc> 451) by the council of Chalcedon, and allows its priests to marry. The <ldquo/pope and patriarch<rdquo/ has jurisdiction over the Abyssinian Church. Since the 7th century the Coptic Church has been so isolated from modifying influences that in many respects it is the most ancient monument of primitive Christian rites and ceremonies. But centuries of subjection to Moslem rule have weakened and degraded it.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coptis</ent><br/
<hw>Coptis</hw> <pos>n.</pos> <def>small genus of low perennial herbs having yellow rhizomes and white or yellow flowers.</def><br/
<syn><b>Syn. --</b> genus <gen>Coptis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Copts</ent><br/
<hw>Copts</hw> <pr>(k<ocr/pts")</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Copt</singw> <pr>(k<ocr/pt)</pr>.</sing> <ety>[See <er>Coptic</er>.]</ety> <fld>(Etnol.)</fld> <sn>1.</sn> <def>An Egyptian race thought to be descendants of the ancient Egyptians.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The principal sect of Christians in Egypt and the valley of the Nile.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ they belong to the Jacobite sect of Monophysite Christians, and for eleven centuries have had possession of the patriarchal chair of Alexandria.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copula</ent><br/
<hw>Cop"u*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., bond, band. See <er>Couple</er>.]</ety> <sn>1.</sn> <fld>(Logic & Gram.)</fld> <def>The word which unites the subject and predicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The stop which connects the manuals, or the manuals with the pedals; -- called also <altname>coupler</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulate</ent><br/
<hw>Cop"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>copulatus</ets>, p. p. of <ets>copulare</ets> to couple, fr. <ets>copula</ets>. See <er>Copula</er>.]</ety> <sn>1.</sn> <def>Joined; associated; coupled.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>Joining subject and predicate; copulative.</def>  <rj><au>F. A. March.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulate</ent><br/
<hw>Cop"u*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Copulated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Copulating</conjf>.]</vmorph> <def>To unite in sexual intercourse; to come together in the act of generation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulation</ent><br/
<hw>Cop`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>copulatio</ets>: cf. F. <ets>copulation</ets>.]</ety> <sn>1.</sn> <def>The act of coupling or joining; union; conjunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wit, you know, is the unexpected <qex>copulation</qex> of ideas.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The coming together of male and female in the act of generation; sexual union; coition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulative</ent><br/
<hw>Cop"u*la"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>copulativus</ets>: cf. F. <ets>copulatif</ets>.]</ety> <def>Serving to couple, unite, or connect; <as>as, a <ex>copulative</ex> conjunction like <ldquo/and<rdquo/</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulative</ent><br/
<hw>Cop"u*la*tive</hw>, <pos>n.</pos> <sn>1.</sn> <def>Connection.</def> <mark>[Obs.]</mark>  <rj><au>Rycaut.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>A copulative conjunction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulatively</ent><br/
<hw>Cop"u*la"tive*ly</hw>, <pos>adv.</pos> <def>In a copulative manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copulatory</ent><br/
<hw>Cop"u*la*to*ry</hw> <pr>(k<ocr/p"<usl/*l<adot/*t<osl/*r<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to copulation; tending or serving to unite; copulative.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Used in sexual union; <as>as, the <ex>copulatory</ex> organs of insects</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copy</ent><br/
<hw>Cop"y</hw> <pr>(k<ocr/p"<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Copies</plw> <pr>(-<icr/z)</pr></plu>. <ety>[F. <ets>copie</ets>, fr. L. <ets>copia</ets> abundance, number, LL. also, a transcript; <ets>co-</ets> + the root of <ets>opes</ets> riches. See <er>Opulent</er>, and cf. <er>Copious</er>.]</ety> <sn>1.</sn> <def>An abundance or plenty of anything.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She was blessed with no more <qex>copy</qex> of wit, but to serve his humor thus.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An imitation, transcript, or reproduction of an original work; <as>as, a <ex>copy</ex> of a letter, an engraving, a painting, or a statue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not the vanity to think my <qex>copy</qex> equal to the original.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An individual book, or a single set of books containing the works of an author; <as>as, a <ex>copy</ex> of the Bible; a <ex>copy</ex> of the works of Addison</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is to be imitated, transcribed, or reproduced; a pattern, model, or example; <as>as, his virtues are an excellent <ex>copy</ex> for imitation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let him first learn to write, after a <qex>copy</qex>, all the letters.</q> <rj><qau>Holder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(print.)</fld> <def>Manuscript or printed matter to be set up in type; <as>as, the printers are calling for more <ex>copy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A writing paper of a particular size. Same as <er>Bastard</er>. See under <er>Paper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Copyhold; tenure; lease.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Copy book</b></col>, <cd>a book in which copies are written or printed for learners to imitate.</cd> -- <col><b>Examined copies</b></col> <fld>(Law)</fld>, <cd>those which have been compared with the originals.</cd> -- <col><b>Exemplified copies</b></col>, <cd>those which are attested under seal of a court.</cd> -- <mcol><col><b>Certified copies</b></col> <it>or</it> <col><b>Office copies</b></col></mcol>, <cd>those which are made or attested by officers having charge of the originals, and authorized to give copies officially.</cd>  <rj><au>Abbot.</au></rj></cs></p>

<p><syn><b>Syn.</b> -- Imitation; transcript; duplicate; counterfeit.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 322 --></p>

<p><ent>Copy</ent><br/
<hw>Cop"y</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Copied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Copying</conjf>.]</vmorph> <ety>[Cf. F. <ets>copir</ets>, fr. LL. <ets>copiare</ets>. See <er>Copy</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To make a copy or copies of; to write; print, engrave, or paint after an original; to duplicate; to reproduce; to transcribe; <as>as, to <ex>copy</ex> a manuscript, inscription, design, painting, etc.</as>; -- often with <xex>out</xex>, sometimes with <xex>off</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I like the work well; ere it be demanded<br/
(As like enough it will), I'd have it <qex>copied</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let this be <qex>copied</qex> out,<br/
And keep it safe for our remembrance.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imitate; to attempt to resemble, as in manners or course of life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>copy</qex> instinctively the voices of our companions, their accents, and their modes of pronunciation.</q> <rj><qau>Stewart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copy</ent><br/
<hw>Cop"y</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To make a copy or copies; to imitate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To yield a duplicate or transcript; <as>as, the letter did not <ex>copy</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some . . . never fail, when they <qex>copy</qex>, to follow the bad as well as the good things.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>copybook</ent><br/
<hw>copybook</hw> <pos>n.</pos> <def>a book containing models of good penmanship; used in teaching penmanship.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>copy cat</ent><br/
<ent>copycat</ent><br/
<mhw><hw>cop"y*cat`</hw>, <hw>cop"y cat`</hw></mhw> <pos>n.</pos> <def>someone who copies the words or behavior of another.</def> <br/
<syn><b>Syn. --</b> imitator, ape, aper.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Copyer</ent><br/
<hw>Cop"y*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Copier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copygraph</ent><br/
<hw>Cop"y*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A contrivance for producing manifold copies of a writing or drawing; -- made obsolete by later developments in copying technology; see <er>xerography</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ The writing or drawing is made with aniline ink on paper, and a reverse copy transfered by pressure to a slab of gelatin softened with glycerin. A large number of transcripts can be taken while the ink is fresh.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Various names have been given to the process [the gelatin copying process], some of them acceptable and others absurd; hectograph, polygraph, <qex>copygraph</qex>, lithogram, etc.</q> <rj><qau>Knight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copyhold</ent><br/
<hw>Cop"y*hold`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eng. Law)</fld> <sd>(a)</sd> <def>A tenure of estate by copy of court roll; or a tenure for which the tenant has nothing to show, except the rolls made by the steward of the lord's court.</def> <au>Blackstone.</au> <sd>(b)</sd> <def>Land held in copyhold.</def> <au>Milton.</au><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Copyholds</ex> do not exist in the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copyholder</ent><br/
<hw>Cop"y*hold`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Eng. Law)</fld> <def>One possessed of land in copyhold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(print.)</fld> <sd>(a)</sd> <def>A device for holding copy for a compositor.</def> <sd>(b)</sd> <def>One who reads copy to a proof reader.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copying</ent><br/
<hw>Cop"y*ing</hw>, <pos>a. & n.</pos> <def>From <er>Copy</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Copying ink</b></col>. <cd>See under <er>Ink</er>.</cd> -- <col><b>Copying paper</b></col>, <cd>thin unsized paper used for taking copies of letters, etc., in a copying press.</cd> -- <col><b>Copying press</b></col>, <cd>a machine for taking by pressure, an exact copy of letters, etc., written in copying ink.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copyist</ent><br/
<hw>Cop"y*ist</hw>, <pos>n.</pos> <def>A copier; a transcriber; an imitator; a plagiarist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>copyreader</ent><br/
<hw>copyreader</hw> <pos>n.</pos> <def>an editor who prepares text for publication.</def><br/
<syn><b>Syn. --</b> copy editor, text editor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Copyright</ent><br/
<hw>Cop"y*right</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The right of an author or his assignee, under statute, to print and publish his literary or artistic work, exclusively of all other persons. This right may be had in maps, charts, engravings, plays, and musical compositions, as well as in books.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States in 1913 a copyright was valid for the term of twenty-eight years, with right of renewal for fourteen years on certain conditions.   The term was extended in stages, and in 1997 the term of a copyright was life plus 50 years for individuals retaining their copyright, or 75 years for works created for hire.  Further extension is still (1998) being discussed.</note>
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>International copyright</b></col>, <cd>an author's right in his productions as secured by treaty between nations.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Copyright</ent><br/
<hw>Cop"y*right`</hw>, <pos>v. t.</pos> <def>To secure a copyright on.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coque</ent><br/
<hw>Coque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prop., a shell.]</ety> <def>A small loop or bow of ribbon used in making hats, boas, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coquelicot</ent><br/
||<hw>Coque"li*cot`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The wild poppy, or red corn rose.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The color of the wild poppy; a color nearly red, like orange mixed with scarlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquet</ent><br/
<hw>Co*quet"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coquetted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coquetting</conjf>.]</vmorph> <def>To attempt to attract the notice, admiration, or love of; to treat with a show of tenderness or regard, with a view to deceive and disappoint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You are <qex>coquetting</qex> a maid of honor.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquet</ent><br/
<hw>Co*quet"</hw>, <pos>v. i.</pos> <def>To trifle in love; to stimulate affection or interest; to play the coquette; to deal playfully instead of seriously; to play (with); <as>as, we have <ex>coquetted</ex> with political crime</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquetry</ent><br/
<hw>Co*quet"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Coquetries</plw> <pr>(#)</pr></plu>. <ety>[F. <ets>coquetterie</ets>.]</ety> <def>Attempts to attract admiration, notice, or love, for the mere gratification of vanity; trifling in love.</def> <ldquo/Little affectations of <xex>coquetry</xex>.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquette</ent><br/
<hw>Co*quette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>coquet</ets>, <ets>coquette</ets>, coquettish, orig., cocklike, strutting like a cock, fr. <ets>coq</ets> a cock. Cf. <er>Cock</er>, <er>Cocket</er>, <er>Cocky</er>, <er>Cockade</er>.]</ety> <sn>1.</sn> <def>A vain, trifling woman, who endeavors to attract admiration from a desire to gratify vanity; a flirt; -- formerly sometimes applied also to men.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A tropical humming bird of the genus <gen>Lophornis</gen>, with very elegant neck plumes. Several species are known. See <xex>Illustration</xex> under <er>Spangle</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquettish</ent><br/
<hw>Co*quet"tish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Practicing or exhibiting coquetry; alluring; enticing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A pretty, <qex>coquettish</qex> housemaid.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquettishly</ent><br/
<hw>Co*quet"tish*ly</hw>, <pos>adv.</pos> <def>In a coquettish manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquilla nut</ent><br/
<hw>Co*quil"la nut</hw> <pr>(?)</pr>. <ety>[Pg. <ets>coquilho</ets>, Sp. <ets>coquillo</ets>, dim. of <ets>coco</ets> a cocoanut.]</ety> <fld>(Bot.)</fld> <def>The fruit of a Brazilian tree (<spn>Attalea funifera</spn> of Martius.).</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Its shell is hazel-brown in color, very hard and close in texture, and much used by turners in forming ornamental articles, such as knobs for umbrella handles.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquille</ent><br/
<hw>Co*quille"</hw> <pr>(k<osl/*k<emac/l"; <it>F.</it> k<osl/`k<emac/"y')</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Lit., a shell;</def> <specif>hence:</specif> <sd>(a)</sd> <def>A shell or shell-like dish or mold in which viands are served.</def> <sd>(b)</sd> <def>The expansion of the guard of a sword, dagger, etc.</def> <sd>(c)</sd> <def>A form of ruching used as a dress trimming or for neckwear, and named from the manner in which it is gathered or fulled.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coquimbite</ent><br/
<hw>Co*quim"bite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mineral consisting principally of sulphate of iron; white copperas; -- so called because found in the province of <ets>Coquimbo</ets>, Chili.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coquina</ent><br/
||<hw>Co*qui"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., shellfish, cockle.]</ety> <def>A soft, whitish, coral-like stone, formed of broken shells and corals, found in the southern United States, and used for roadbeds and for building material, as in the fort at St. Augustine, Florida.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cor-</ent><br/
<hw>Cor-</hw> <pr>(k<ocr/r-)</pr>. <def>A prefix signifying <xex>with</xex>, <xex>together</xex>, etc. See <er>Com-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cor</ent><br/
<hw>Cor</hw> <pr>(k<ocir/r)</pr>, <pos>n.</pos> <ety>[Heb. <ets>k<omac/r</ets>.]</ety> <def>A Hebrew measure of capacity; a homer.</def> <altsp>[Written also <asp>core</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cora</ent><br/
||<hw>Co"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Arabian gazelle (<spn>Gazella Arabica</spn>), found from persia to North Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coracan</ent><br/
<hw>coracan</hw> <pos>n.</pos> <def>an East Indian cereal grass (<spn>Eleusine coracana</spn>) whose seeds yield a somewhat bitter flour, a staple in the Orient.</def><br/
<syn><b>Syn. --</b> finger millet, ragi, ragee, African millet, corakan, kurakkan, <spn>Eleusine coracana</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coracias</ent><br/
<hw>Coracias</hw> <pos>n.</pos> <def>the type genus of the <fam>Coraciidae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Coracias</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coraciidae</ent><br/
<hw>Coraciidae</hw> <pos>n.</pos> <fld>(Zool.)</fld> <def>a family of birds comprising the rollers.</def><br/
<syn><b>Syn. --</b> family <fam>Coraciidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coraciiformes</ent><br/
<hw>Coraciiformes</hw> <pos>n.</pos> <fld>(Zool.)</fld> <def>an order of birds including the rollers; kingfishers; hornbills; hoopoes; motmots; bee-eaters; todies.</def><br/
<syn><b>Syn. --</b> order <ord>Coraciiformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coracle</ent><br/
<hw>Cor"a*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[W. <ets>corwgl</ets>, <ets>cwrwgl</ets>, fr. <ets>corwg</ets>, <ets>cwrwg</ets>, any round body or vessel, the trunk of the body, carcass.]</ety> <def>A boat made by covering a wicker frame with leather or oilcloth. It was used by the ancient Britons, and is still used by fisherman in Wales and some parts of Ireland. Also, a similar boat used in Tibet and in Egypt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coracoid</ent><br/
<hw>Cor"a*coid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>ko`rax</grk> crow + <grk>e'i^dos</grk> form.]</ety> <sn>1.</sn> <def>Shaped like a crow's beak.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Pertaining to a bone of the shoulder girdle in most birds, reptiles, and amphibians, which is reduced to a process of the scapula in most mammals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coracoid</ent><br/
<hw>Cor"a*coid</hw>, <pos>n.</pos> <def>The coracoid bone or process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corage</ent><br/
<hw>Cor"age</hw> <pr>(?; OF. <?/)</pr>, <pos>n.</pos> <def>See <er>Courage</er></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To Canterbury with full devout <qex>corage</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corah</ent><br/
<hw>Co"rah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>k<omac/r<amac/</ets> virgin, plain.]</ety> <def>Plain; undyed; -- applied to Indian silk.</def> -- <def2><pos>n.</pos>  <def>Corah silk.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>corakan</ent><br/
<hw>corakan</hw> <pos>n.</pos> <def>same as <er>coracan</er>.</def><br/
<syn><b>Syn. --</b> finger millet, ragi, ragee, African millet, coracan, kurakkan, <spn>Eleusine coracana</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coral</ent><br/
<hw>Cor"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of. <ets>coral</ets>, F, <ets>corail</ets>, L. <ets>corallum</ets>, <ets>coralium</ets>, fr. Gr. <grk>kora`llion</grk>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The hard parts or skeleton of various Anthozoa, and of a few Hydrozoa. Similar structures are also formed by some Bryozoa.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The large stony corals forming coral reefs belong to various genera of <fam>Madreporaria</fam>, and to the hydroid genus, <gen>Millepora</gen>. The red coral, used in jewelry, is the stony axis of the stem of a gorgonian (<spn>Corallium rubrum</spn>) found chiefly in the Mediterranean. The <stype>fan corals</stype>, <stype>plume corals</stype>, and <stype>sea feathers</stype> are species of <fam>Gorgoniacea</fam>, in which the axis is horny. <xex>Organ-pipe</xex> coral is formed by the genus <gen>Tubipora</gen>, an Alcyonarian, and <stype>black coral</stype> is in part the axis of species of the genus <gen>Antipathes</gen>. See <er>Anthozoa</er>, <er>Madrepora</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The ovaries of a cooked lobster; -- so called from their color.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece of coral, usually fitted with small bells and other appurtenances, used by children as a plaything.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Brain coral</b></col>, <it>or</it> <col><b>Brain stone coral</b></col></mcol>. <cd>See under <er>Brain</er>.</cd> -- <col><b>Chain coral</b></col>. <cd>See under <er>Chain</er>.</cd> -- <col><b>Coral animal</b></col> <fld>(Zool.)</fld>, <cd>one of the polyps by which corals are formed. They are often very erroneously called <altname>coral insects</altname>.</cd> -- <col><b>Coral fish</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Coral reefs</b></col> <fld>(Phys. Geog.)</fld>, <cd>reefs, often of great extent, made up chiefly of fragments of corals, coral sands, and the solid limestone resulting from their consolidation. They are classed as <stype>fringing reefs</stype>, when they border the land; <stype>barrier reefs</stype>, when separated from the shore by a broad belt of water; <stype>atolls</stype>, when they constitute separate islands, usually inclosing a lagoon. See <er>Atoll</er>.</cd> -- <col><b>Coral root</b></col> <fld>(Bot.)</fld>, <cd>a genus (<gen>Corallorhiza</gen>) of orchideous plants, of a yellowish or brownish red color, parasitic on roots of other plants, and having curious jointed or knotted roots not unlike some kinds of coral. See <xex>Illust.</xex> under <er>Coralloid</er>.</cd> -- <col><b>Coral snake</b></col>. <fld>(Zo)</fld> <sd>(a)</sd> <cd>A small, venomous, Brazilian snake <spn>(Elaps corallinus)</spn>, coral-red, with black bands.</cd> <sd>(b)</sd> <cd>A small, harmless, South American snake (<spn>Tortrix scytale</spn>).</cd> -- <col><b>Coral tree</b></col> <fld>(Bot.)</fld>, <cd>a tropical, leguminous plant, of several species, with showy, scarlet blossoms and coral-red seeds. The best known is <spn>Erythrina Corallodendron</spn>.</cd> -- <col><b>Coral wood</b></col>, <cd>a hard, red cabinet wood.</cd> <au>McElrath.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>coralberry</ent><br/
<hw>coralberry</hw> <pos>n.</pos> <sn>1.</sn> <def>a North American deciduous shrub (<spn>Symphoricarpos orbiculatus</spn>) cultivated for its abundant clusters of coral-red berrylike fruits.</def><br/
<syn><b>Syn. --</b> Indian currant, <spn>Symphoricarpos orbiculatus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>shrub (<spn>Ardisia crenata</spn>) with coral-red berries; it is found from Japan to northern India.</def><br/
<syn><b>Syn. --</b> spiceberry, <spn>Ardisia crenata</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coraled</ent><br/
<hw>Cor"aled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having coral; covered with coral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coral fish</ent><br/
<hw>Cor"al fish`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any bright-colored fish of the genera <gen>Ch<ae/todon</gen>, <gen>Pomacentrus</gen>, <gen>Apogon</gen>, and related genera, which live among reef corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corallaceous</ent><br/
<hw>Cor`al*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like coral, or partaking of its qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corallian</ent><br/
<hw>Co*ral"li*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>A deposit of coralliferous limestone forming a portion of the middle division of the oolite; -- called also coral-rag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralliferous</ent><br/
<hw>Cor`al*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>corallum</ets> coral + <ets>-ferous</ets>.]</ety> <def>Containing or producing coral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralliform</ent><br/
<hw>Cor"al*li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>corallum</ets> coral + <ets>-form</ets>.]</ety> <def>resembling coral in form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralligena</ent><br/
||<hw>Cor`al*lig"e*na</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>corallum</ets> coral + root of <ets>gignere</ets> to produce.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Anthozoa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralligenous</ent><br/
<hw>Cor`al*lig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>producing coral; coralligerous; coralliferous.</def>  <rj><au>Humble.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralligerous</ent><br/
<hw>Cor`al*lig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>corallum</ets> coral + <ets>-gerous</ets>.]</ety> <def>Producing coral; coralliferous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corallin</ent><br/
<hw>Cor"al*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named in allusion to the color of red corallin, fr. L. <ets>corallum</ets> coral.]</ety> <fld>(Chem.)</fld> <def>A yellow coal-tar dyestuff which probably consists chiefly of rosolic acid. See <er>Aurin</er>, and <cref>Rosolic acid</cref> under <er>Rosolic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Red corallin</b></col>, <cd>a red dyestuff which is obtained by treating aurin or rosolic acid with ammonia; -- called also <altname>p<ae/onin</altname>.</cd> -- <col><b>Yellow corallin</b></col>. <cd>See <er>Aurin</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralline</ent><br/
<hw>Cor"al*line</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>corallinus</ets> coralred.]</ety> <def>Composed of corallines; <as>as, <ex>coralline</ex> limestone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralline</ent><br/
<hw>Cor"al*line</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>coralline</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A submarine, semicalcareous or calcareous plant, consisting of many jointed branches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Formerly any slender coral-like animal; -- sometimes applied more particulary to bryozoan corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corallinite</ent><br/
<hw>Cor"al*lin*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>A fossil coralline.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corallite</ent><br/
<hw>Cor"al*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>corallum</ets> coral.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>A mineral substance or petrifaction, in the form of coral.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the individual members of a compound coral; or that part formed by a single coral animal.</def> <altsp>[Written also <asp>corallet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralloid</ent><br/
<hw>Cor"al*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>corallum</ets> coral + <ets>-oid</ets>: cf. F. <ets>corallo<ium/de</ets>.]</ety> <def>Having the form of coral; branching like coral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coralloidal</ent><br/
<hw>Cor`al*loid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>resembling coral; coralloid.</def>  <rj><au>Sir T. browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corallorhiza</ent><br/
<hw>Corallorhiza</hw> <pos>n.</pos> <def>a genus of leafless root-parasitic orchids having small purplish or yellowish racemose flowers with lobed lips; it is widely distributed in temperate regions.</def><br/
<syn><b>Syn. --</b> genus <gen>Corallorhiza</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corallum</ent><br/
||<hw>Co*ral"lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Zool.)</fld> <def>The coral or skeleton of a zoophyte, whether calcareous of horny, simple or compound. See <er>Coral</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coral-rag</ent><br/
<hw>Cor"al-rag`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(geol.)</fld> <def>Same as <er>Corallian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>coralroot</ent><br/
<hw>coralroot</hw> <pos>n.</pos> <def>a European bittercress (<spn>Dentaria bulbifera</spn>) having a knotted white rootstock.</def><br/
<syn><b>Syn. --</b> coral-root bittercress, coralwort, Cardamine bulbifera, <spn>Dentaria bulbifera</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coralwood</ent><br/
<hw>coralwood</hw> <pos>n.</pos> <def>an East Indian tree (<spn>Adenanthera pavonina</spn>) with racemes of yellow-white flowers; cultivated as an ornamental.</def><br/
<syn><b>Syn. --</b> red sandalwood, Barbados pride, peacock flower fence, <spn>Adenanthera pavonina</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coralwort</ent><br/
<hw>Cor"al*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A cruciferous herb of certain species of <gen>Dentaria</gen>; -- called also <altname>toothwort</altname>, <altname>tooth violet</altname>, or <altname>pepper root</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coranach</ent><br/
<hw>Cor"a*nach</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>coranach</ets>, or <ets>corranach</ets>, a crying, the Irish funeral cry (the <ets>keen</ets>), a dirge; <ets>comh</ets> with + <ets>ranaich</ets> a roaring, <ets>ran</ets> to roar, shriek.]</ety> <def>A lamentation for the dead; a dirge.</def> <altsp>[Written also <asp>coranich</asp>, <asp>corrinoch</asp>, <asp>coronach</asp>, <asp>cronach</asp>, etc.]</altsp> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coranto</ent><br/
<ent>Corant</ent><br/
<mhw>{ <hw>Co*rant</hw> <pr>(?)</pr>, <hw>Co*ran"to</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[See <er>Courant</er>.]</ety> <def>A sprightly but somewhat stately dance, now out of fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is harder to dance a <qex>corant</qex> well, than a jig.</q> <rj><qau>Sir W. temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Dancing a <qex>coranto</qex> with him upon the heath.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corb</ent><br/
<hw>Corb</hw> <pr>(k<ocir/rb)</pr>, <pos>n.</pos> <ety>[L. <ets>corbis</ets> basket. Cf. <er>Corbeil</er>, <er>Corp</er>.]</ety> <sn>1.</sn> <def>A basket used in coal mines, etc. see <er>Corf</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An ornament in a building; a corbel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corban</ent><br/
<hw>Cor"ban</hw> <pr>(k<ocir/r"b<acr/n)</pr>, <pos>n.</pos> <ety>[Heb. <ets>qorb<amac/n</ets>, akin to Ar. qurb<amac/n.]</ety> <sn>1.</sn> <fld>(Jewish Antiq.)</fld> <def>An offering of any kind, devoted to God and therefore not to be appropriated to any other use; esp., an offering in fulfillment of a vow.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the old Testament the hebrew word is usually translated <ldquo/oblation<rdquo/ as in <au>Numb. xviii. 9, xxxi. 50.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The traditionists laid down that a man might interdict himself by vow, not only from using for himself, but from giving to another, or receiving from him, some particular object, whether of food or any other kind. A person might thus exempt himself from assisting parents in distress, under plea of <xex>corban</xex>. <rj><au>Dr. W. Smith.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An alms basket; a vessel to receive gifts of charity; a treasury of the church, where offerings are deposited.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corbe</ent><br/
<hw>Corbe</hw> <pr>(k<ocir/rb)</pr>, <pos>a.</pos> <ety>[OF. <ets>corbe</ets>, fr. L. <ets>curvus</ets>. See <er>Cuve</er>.]</ety> <def>Crooked.</def> <mark>[Obs.]</mark> <ldquo/<xex>Corbe</xex> shoulder.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corbelling</ent><br/
<ent>Corbeling</ent><br/
<mhw>{ <hw>Cor"bel*ing</hw>, <hw>Cor"bel*ling</hw>  }</mhw>, <pos>n.</pos> <def>Corbel work or the construction of corbels; a series of corbels or piece of continuous corbeled masonry, sometimes of decorative purpose, as in the stalactite ornament of the Moslems.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corbeil</ent><br/
<hw>Cor"beil</hw> <pr>(k<ocir/r"b<ecr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>corbeille</ets>, fr. L. <ets>corbicula</ets> a little basket, dim. of <ets>corbis</ets> basket. Cf. <er>Corbel</er>, <er>Corb</er>, <er>Corvette</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A sculptured basket of flowers; a corbel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Fort.)</fld> <def>Small gabions.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corbel</ent><br/
<hw>Cor"bel</hw> <pr>(k<ocir/r"b<ecr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>corbeau</ets>, for older <ets>corbel</ets>, dim. of L. <ets>corbis</ets> basket. (Corbels were often in the form of a basket.) See <er>Corbeil</er>.]</ety> <fld>(Arch.)</fld> <def>A bracket supporting a superincumbent object, or receiving the spring of an arch. Corbels were employed largely in Gothic architecture.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A common form of corbel consists of courses of stones or bricks, each projecting slightly beyond the next below it.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corbel</ent><br/
<hw>Cor"bel</hw>, <pos>v. t.</pos> <def>To furnish with a corbel or corbels; to support by a corbel; to make in the form of a corbel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To corbel out</b></col>, <cd>to furnish with a corbel of courses, each projecting beyond the one next below it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corbel-table</ent><br/
<hw>Cor"bel-ta`ble</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A horizontal row of corbels, with the panels or filling between them; also, less properly used to include the stringcourse on them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corby</ent><br/
<ent>Corbie</ent><br/
<mhw>{ <hw>Cor"bie</hw> <it>or</it> <hw>Cor"by</hw>  }</mhw> <pr>(k<ocir/r"b<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corbies</plw> <pr>(-b<icr/z)</pr>.</plu> <ety>[F. <ets>corbeau</ets>, OF. <ets>corbel</ets>, dim. fr. L. <ets>corvus</ets> raven.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The raven.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(her.)</fld> <def>A raven, crow, or chough, used as a charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corbie crow</b></col>, <cd>the carrion crow.</cd> <mark>[Scot.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corbiestep</ent><br/
<hw>Cor"bie*step`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>One of the steps in which a gable wall is often finished in place of a continuous slope; -- also called <altname>crowstep</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corchorus</ent><br/
<hw>Cor"cho*rus</hw> <pr>(k<ocir/r"k<osl/*r<ucr/s)</pr>, <pos>n.</pos> <ety>[Nl., fr. L. <ets>corchorus</ets> a poor kind of pulse, Gr. <grk>ko`rchoros</grk> a wild plant of bitter taste.]</ety> <fld>(Bot.)</fld> <def>The common name of the <spn>Kerria Japonica</spn> or Japan globeflower, a yellow-flowered, perennial, rosaceous plant, seen in old-fashioned gardens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corcule</ent><br/
<ent>Corcle</ent><br/
<mhw>{ <hw>Cor"cle</hw> <pr>(k<ocir/r"k'l)</pr>, <hw>Cor"cule</hw> <pr>(-k<usl/l)</pr> }</mhw>, <pos>n.</pos> <ety>[L. <ets>corculum</ets> a little heart, dim. of <ets>cor</ets> heart.]</ety> <fld>(Bot.)</fld> <def>The heart of the seed; the embryo or germ.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cord</ent><br/
<hw>Cord</hw> <pr>(k<ocir/rd)</pr>, <pos>n.</pos> <ety>[F. <ets>corde</ets>, L. <ets>chorda</ets> catgut, chord, cord, fr. Gr. <grk>chordh`</grk>; cf. <grk>chola`des</grk> intestines, L. <ets>harus</ets>pex soothsayer (inspector of entrails), Icel. <ets>g<oum/rn</ets>, pl. <ets>garnir</ets> gut, and E. <ets>yarn</ets>. Cf. <er>Chord</er>, <er>Yarn</er>.]</ety> <sn>1.</sn> <def>A string, or small rope, composed of several strands twisted together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A solid measure, equivalent to 128 cubic feet; a pile of wood, or other coarse material, eight feet long, four feet high, and four feet broad; -- originally measured with a cord or line.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 323 --></p>

<p><sn>3.</sn> <def>Fig.: Any moral influence by which persons are caught, held, or drawn, as if by a cord; an enticement; <as>as, the <ex>cords</ex> of the wicked; the <ex>cords</ex> of sin; the <ex>cords</ex> of vanity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The knots that tangle human creeds,<br/
The wounding <qex>cords</qex> that bind and strain<br/
The heart until it bleeds.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anat.)</fld> <def>Any structure having the appearance of a cord, esp. a tendon or a nerve. See under <er>Spermatic</er>, <er>Spinal</er>, <er>Umbilical</er>, <er>Vocal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>See <er>Chord</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cord wood</b></col>, <cd>wood for fuel cut to the length of four feet (when of full measure).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cord</ent><br/
<hw>Cord</hw> <pr>(k<ocir/rd)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cording</conjf>.]</vmorph> <sn>1.</sn> <def>To bind with a cord; to fasten with cords; to connect with cords; to ornament or finish with a cord or cords, as a garment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To arrange (wood, etc.) in a pile for measurement by the cord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordage</ent><br/
<hw>Cord"age</hw> <pr>(k<ocir/rd"<asl/j)</pr>, <pos>n.</pos> <ety>[F. <ets>cordage</ets>. See <er>Cord</er>.]</ety> <def>Ropes or cords, collectively; hence, anything made of rope or cord, as those parts of the rigging of a ship which consist of ropes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordaitales</ent><br/
<hw>Cordaitales</hw> <pos>n.</pos> <def>an order of extinct plants having tall arborescent trunks comparable to or more advanced than cycads; known from the Pennsylvanian; probably extinct since the Mesozoic.</def><br/
<syn><b>Syn. --</b> order <ord>Cordaitales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cordaites</ent><br/
<hw>Cordaites</hw> <pos>n.</pos> <def>a genus of tall Paleozoic trees superficially resembling modern screw pines; they were structurally intermediate in some ways between cycads and conifers.</def><br/
<syn><b>Syn. --</b> genus <gen>Cordaites</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cordal</ent><br/
<hw>Cord"al</hw> <pr>(k<ocir/rd"<ait/l)</pr>, <pos>n.</pos> <def>Same as <er>Cordelle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordate</ent><br/
<hw>Cordate</hw> <pr>(k<ocir/r"d<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>cor</ets>, <ets>cordis</ets>, heart.]</ety> <fld>(Bot.)</fld> <def>Heart-shaped; <as>as, a <ex>cordate</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordately</ent><br/
<hw>Cor"date*ly</hw>, <pos>adv.</pos> <def>In a cordate form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corded</ent><br/
<hw>Cord"ed</hw> <pr>(k<ocir/rd"<ecr/d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Bound or fastened with cords.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Piled in a form for measurement by the cord.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Made of cords.</def> <mark>[Obs.]</mark> <ldquo/A <xex>corded</xex> ladder.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Striped or ribbed with cords; <as>as, cloth with a <ex>corded</ex> surface</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Her.)</fld> <def>Bound about, or wound, with cords.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordelier</ent><br/
<hw>Cor`de*lier"</hw> <pr>(k<ocir/r`d<esl/*l<emac/r")</pr>, <pos>n.</pos> <ety>[F., fr. OF. <ets>cordel</ets>, F. <ets>cordeau</ets>, dim. fr. <ets>corde</ets> string, rope. See <er>Cord</er>.]</ety> <sn>1.</sn> <fld>(Eccl. Hist.)</fld> <def>A Franciscan; -- so called in France from the girdle of knotted cord worn by all Franciscans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fr. Hist.)</fld> <def>A member of a French political club of the time of the first Revolution, of which Danton and Marat were members, and which met in an old Cordelier convent in Paris.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordeling</ent><br/
<hw>Cor"del*ing</hw> <pr>(k<ocir/r"d<ecr/*<icr/ng)</pr>, <pos>a.</pos> <ety>[F. <ets>cordeler</ets> to twist, fr. OF. <ets>cordel</ets>. See <er>Cordelier</er>.]</ety> <def>Twisting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordelle</ent><br/
||<hw>Cor*delle"</hw> <pr>(k<ocir/r*d<ecr/l")</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>corde</ets> cord.]</ety> <def>A twisted cord; a tassel.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordial</ent><br/
<hw>Cor"dial</hw> <pr>(k<ocir/r"j<ait/l, <it>formally</it> k<ocir/rd"y<ait/l; 106, 277)</pr>, <pos>a.</pos> <ety>[LL.  <ets>cordialis</ets>, fr. L. <ets>cor</ets> heart: cf. F.  <ets>cordial</ets>. See <er>Heart</er>.]</ety> <sn>1.</sn> <def>Proceeding from the heart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A rib with <qex>cordial</qex> spirits warm.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hearty; sincere; warm; affectionate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . with looks of <qex>cordial</qex> love<br/
Hung over her enamored.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Tending to revive, cheer, or invigorate; giving strength or spirits.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold this <qex>cordial</qex> julep here<br/
That flames and dances in his crystal bounds.</q> <rj><qau>Milton.</qau></rj></p>

<p><syn><b>Syn.</b> -- Hearty; sincere; heartfelt; warm; affectionate; cheering; invigorating. See <er>Hearty</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordial</ent><br/
<hw>Cor"dial</hw>, <pos>n.</pos> <sn>1.</sn> <def>Anything that comforts, gladdens, and exhilarates.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Charms to my sight, and <qex>cordials</qex> to my mind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med)</fld> <def>Any invigorating and stimulating preparation; <as>as, a peppermint <ex>cordial</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>Aromatized and sweetened spirit, used as a beverage; a liqueur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordiality</ent><br/
<hw>Cor*dial"i*ty</hw> <pr>(k<ocir/r*j<acr/l"<icr/*t<ycr/ <or/ k<ocir/r`d<icr/*<acr/l"-; 106)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cordialities</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[LL. <ets>cordialitas</ets>, fr. <ets>cordialis</ets> sincere: cf. F.  <ets>cordialit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Relation to the heart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That the ancients had any respect of <qex>cordiality</qex> or reference unto the heart, will much be doubted.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sincere affection and kindness; warmth of regard; heartiness.</def>  <rj><au>Motley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordialize</ent><br/
<hw>Cor"dial*ize</hw> <pr>(k<ocir/r"j<ait/l*<imac/z <or/  k<ocir/rd"y<ait/l*<imac/z; 106)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To make into a cordial.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To render cordial; to reconcile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordialize</ent><br/
<hw>Cor"dial*ize</hw>, <pos>v. i.</pos> <def>To grow cordial; to feel or express cordiality.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordially</ent><br/
<hw>Cor"dial*ly</hw>, <pos>adv.</pos> <def>In a cordial manner.</def>  <rj><au> Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordialness</ent><br/
<hw>Cor"dial*ness</hw>, <pos>n.</pos> <def>Cordiality.</def>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordierite</ent><br/
<hw>Cor"di*er*ite</hw> <pr>(k<ocir/r"d<icr/*<etil/r*<imac/t)</pr>, <pos>n.</pos> <ety>[Named after the geologist <etsep>Cordier</etsep>.]</ety> <fld>(Min.)</fld> <def>See <er>Iolite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordiform</ent><br/
<hw>Cor"di*form</hw> <pr>(k<ocir/r"d<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cor</ets>, <ets>cordis</ets>, heart + <ets>-form</ets>, cf. F. <ets>cordiforme</ets>.]</ety> <def>Heart-shaped.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordillera</ent><br/
<hw>Cor*dil"ler*a</hw> <pr>(k<ocir/r*d<icr/l"l<etil/r*<adot/; Sp. k<ocir/r`d<esl/*ly<asl/"r<adot/)</pr>, <pos>n.</pos> <ety>[Sp., fr. OSp. <ets>cordilla</ets>, <ets>cordiella</ets>, dim. of <ets>cuerda</ets> a rope, string. See <er>Cord</er>.]</ety> <fld>(Geol.)</fld> <def>A mountain ridge or chain.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cordillera</xex> is sometimes applied, in geology, to the system of mountain chains near the border of a continent; thus, the western <ex>cordillera</ex> of North America in the United States includes the Rocky Mountains, Sierra Nevada, Coast and Cascade ranges.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordiner</ent><br/
<hw>Cor"di*ner</hw> <pr>(k<ocir/r"d<icr/*n<etil/r)</pr>, <pos>n.</pos> <def>A cordwainer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordite</ent><br/
<hw>Cord"ite</hw> <pr>(k<ocir/rd"<imac/t)</pr>, <pos>n.</pos> <ety>[From <er>Cord</er>, <pos>n.</pos>]</ety> <fld>(Mil.)</fld> <def>A smokeless powder composed of nitroglycerin, guncotton, and mineral jelly, and used by the British army and in other services. In making it the ingredients are mixed into a paste with the addition of acetone and pressed out into cords  (of various diameters) resembling brown twine, which are dried and cut to length. A variety containing less nitroglycerin than the original is known  as <stype><b>cordite M. D.</b></stype></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cordless</ent><br/
<hw>cordless</hw> <pos>adj.</pos> <sn>1.</sn> <def>operating without a wire connection to the companion communicating unit; -- of telephones and other devices using e.g. radio or infrared signals to allow communication between devices without a direct wire link; <as>as, <ex>cordless</ex> telephones have a very restricted range compared with cellular phones</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>using a battery as a source of power; -- of tools which usually use line current for power; <as>as, <ex>cordless</ex> electric drills are very convenient for use outdoors; <ex>cordless</ex> razors make shaving while travelling easy</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cordoba</ent><br/
<hw>C<oacute/r"do*ba</hw> <pr>(k<ocir/r"d<osl/*v<aum/)</pr>, <pos>n.</pos> <ety>[Prob. fr. the Spanish explorer <person>Francisco Hern<aacute/ndez de <etsep>C<oacute/rdoba</etsep></person>.]</ety> <def>The monetary unit of <country>Nicaragua</country>, equivalent to the <country>United States</country> gold dollar.  It is divided into 100 <part>centavos</part>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cordon</ent><br/
<hw>Cor"don</hw> <pr>(k<ocir/r"d<ocr/n; F. k<ocir/r`d<ocir/N")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>corde</ets>. See <er>Cord</er>.]</ety> <sn>1.</sn> <def>A cord or ribbon bestowed or borne as a badge of honor; a broad ribbon, usually worn after the manner of a baldric, constituting a mark of a very high grade in an honorary order. Cf. <er>Grand cordon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cord worn by a Franciscan friar.</def>  <rj><au>Sir E. Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fort.)</fld> <def>The coping of the scarp wall, which projects beyong the face of the wall a few inches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>A line or series of sentinels, or of military posts, inclosing or guarding any place or thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A rich and ornamental lace or string, used to secure a mantle in some costumes of state.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>||Cordon sanitaire</b></col> <pr>(k<?/r`<er>d<?/n</er>" s<?/`n<?/`t<?/r")</pr> <ety>[F., sanitary cordon]</ety>, <cd>a line of troops or military posts around a district infected with disease, to cut off communication, and thus prevent the disease from spreading.  Also used figuratively, of a group of neutral states that forms a barrier between two hostile states.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordon bleu</ent><br/
||<hw>Cordon bleu</hw> <pr>(k<?/r`<er>d<?/n</er>" bl<?/")</pr> <ety>[F., blue cordon]</ety>, <sn>1.</sn> <def>a first-rate cook, or one worthy to be the cook of the <ecol><b>cordons bleus</b></ecol>, or <ecol><b>Knights of the Holy Ghost</b></ecol>, a distinguished order of French knights, famous for their good dinners.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the blue ribbon worn by the Knights of the Holy Ghost.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <ety>[one entitled to wear the <ets>cordon bleu{2}</ets>.]</ety> <def>a person of high distinction.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Cookery)</fld> <def>a dish prepared with thin slices of meat separated by layers of ham and cheese, and then sauteed; <as>as, chicken <ex>cordon bleu</ex> is a popular dish</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cordonnet</ent><br/
||<hw>Cor`don`net"</hw> <pr>(k?r`d?n`n?")</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>cordon</ets>. See <er>Cardon</er>.]</ety> <def>Doubled and twisted thread, made of coarse silk, and used for tassels, fringes, etc.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordovan</ent><br/
<hw>Cor"do*van</hw> <pr>(k<ocir/r"d<osl/*v<acr/n)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cordoban</ets>, fr.  <ets>Cordova</ets>, or <ets>Cordoba</ets>, in Spain. Cf. <er>Cordwain</er>.]</ety> <def>Same as <er>Cordwain</er>.  In England the name is applied to leather made from horsehide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cords</ent><br/
<hw>cords</hw> <pos>n.</pos> <ety>[contraction of <ets>corduroys</ets>.]</ety> <def>trousers made of corduroy cloth; corduroys.</def><br/
<syn><b>Syn. --</b> corduroys.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corduroy</ent><br/
<hw>Cor"du*roy`</hw> <pr>(k<ocir/r"d<usl/*roi` <or/ k<ocir/r`d<usl/*roi")</pr>, <pos>n.</pos> <ety>[Prob. for F. <ets>corde du roi</ets> king's cord.]</ety> <sn>1.</sn> <def>A sort of cotton velveteen, having the surface raised in ridges.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Trousers or breeches of corduroy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corduroy road</b></col>, <cd>a roadway formed of logs laid side by side across it, as in marshy places; -- so called from its rough or ribbed surface, resembling corduroy.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corduroy</ent><br/
<hw>Cor"du*roy`</hw>, <pos>v. t.</pos> <def>To form of logs laid side by side.</def> <ldquo/Roads were <xex>corduroyed</xex>.<rdquo/ <au>Gen. W. T. Sherman.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordwain</ent><br/
<hw>Cord"wain</hw> <pr>(k?rd"w?n)</pr>, <pos>n.</pos> <ety>[OE. <ets>cordewan</ets>, <ets>cordian</ets>, OF.  <ets>cordoan</ets>, <ets>cordouan</ets>, fr. Sp.  <ets>cordoban</ets>. See <er>Cordovan</er>.]</ety> <def>A term used in the Middle Ages for Spanish leather (goatskin tanned and dressed), and hence, any leather handsomely finished, colored, gilded, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Buskins he wore of costliest <qex>cordwain</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordwainer</ent><br/
<hw>Cord"wain*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <ety>[OE.  <ets>cordwaner</ets>, <ets>cordiner</ets>, fr. OF. <ets>cordoanier</ets>, <ets>cordouanier</ets>, F. <ets>cordonnier</ets>.]</ety> <def>A worker in cordwain, or cordovan leather; a shoemaker.</def> <mark>[Archaic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cordy</ent><br/
<hw>Cord"y</hw> <pr>(k<ocir/r"d<ycr/)</pr>, <pos>a.</pos>  <amorph>[<pos>Compar.</pos> <adjf>Cordier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Cordiest</adjf>.]</amorph> <def>Of, or like, cord; having cords or cordlike parts.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cordylidae</ent><br/
<hw>Cordylidae</hw> <pos>n.</pos> <def>a small family of spiny ovoviviparous African lizards.</def><br/
<syn><b>Syn. --</b> family <fam>Cordylidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cordyline</ent><br/
<hw>Cordyline</hw> <pos>n.</pos> <def>a genus of Asiatic and Pacific trees or shrubs remarkable for the fact that fragments of the trunk will regrow to form whole plants.</def><br/
<syn><b>Syn. --</b> genus <gen>Cordyline</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cordylus</ent><br/
<hw>Cordylus</hw> <pos>n.</pos> <def>the type genus of the <fam>Cordylidae</fam>; they are spiny lizards somewhat resembling tiny crocodiles.</def><br/
<syn><b>Syn. --</b> genus <gen>Cordylus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Core</ent><br/
<hw>Core</hw> <pr>(k<omac/r)</pr>, <pos>n.</pos> <ety>[F.  <ets>corps</ets>. See <er>Corps</er>.]</ety> <def>A body of individuals; an assemblage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was in a <qex>core</qex> of people.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Core</ent><br/
<hw>Core</hw>, <pos>n.</pos> <ety>[Cf. <er>Chore</er>.]</ety> <fld>(Mining.)</fld> <def>A miner's underground working time or shift.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The twenty-four hours are divided into three or four <xex>cores</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Core</ent><br/
<hw>Core</hw>, <pos>n.</pos> <ety>[Heb. <ets>k<omac/r</ets>: cf. Gr. <grk>ko`ros</grk>.]</ety> <def>A Hebrew dry measure; a cor or homer.</def>  <rj><au>Num. xi. 32 (Douay version).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Core</ent><br/
<hw>Core</hw>, <pos>n.</pos> <ety>[OF.  <ets>cor</ets>, <ets>coer</ets>, <ets>cuer</ets>, F.  <ets>c<oe/ur</ets>, fr. L.  <ets>cor</ets> heart. See <er>Heart</er>.]</ety> <sn>1.</sn> <def>The heart or inner part of a thing, as of a column, wall, rope, of a boil, etc.; especially, the central part of fruit, containing the kernels or seeds; <as>as, the <ex>core</ex> of an apple or quince</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fever at the <qex>core</qex>,<br/
Fatal to him who bears, to all who ever bore.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The center or inner part, as of an open space; <as>as, the <ex>core</ex> of a square</as>.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The most important part of a thing; the essence; <as>as, the <ex>core</ex> of a subject</as>; -- also used attributively, <as>as the <ex>core</ex> curriculum at a college</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <fld>(Founding)</fld> <def>The portion of a mold which shapes the interior of a cylinder, tube, or other hollow casting, or which makes a hole in or through a casting; a part of the mold, made separate from and inserted in it, for shaping some part of the casting, the form of which is not determined by that of the pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A disorder of sheep occasioned by worms in the liver.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Anat.)</fld> <def>The bony process which forms the central axis of the horns in many animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Elec.)</fld> <def>A mass of iron or other ferrous metal, forming the central part of an electromagnet, such as those upon which the conductor of an armature, a transformer, or an induction coil is wound.</def> <note>The presence of the iron intensifies the magnetic field created by a a current passing through the windings.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>8.</sn> <fld>(mining)</fld> <def>a sample of earth or rock extracted from underground by a drilling device in such a manner that the layers of rock are preserved in the same order as they exist underground; <as>as, to drill a <ex>core</ex>; to extract a <ex>core</ex></as>. The sample is typically removed with a rotating drill bit having a hollow center, and is thus shaped like a cylinder.</def><br/
[<source>PJC</source>]</p>

<p><sn>9.</sn> <fld>(Computers)</fld> <def>The main working memory of a digital computer system, which typically retains the program code being executed as well as the data structures that are manipulated by the program.  Contrasted to <contr>ROM</contr> and <contr>data storage device</contr>.</def> <note>The term was applied originally to the main memory, consisting of small ferromagnetic rings, that were used to store data in older computers, where each ring representing one bit of information by virtue of its state of magnetization.  They were superseded by electronic data storage devices.</note><br/
<syn><b>Syn. --</b> core memory, random access memory, RAM</syn><br/
[<source>PJC</source>]</p>

<p><sn>10.</sn> <fld>(Geol.)</fld> <def>the central part of the  earth, believed to be a sphere with a radius of about 2100 miles, and composed primarily of molten iron with some nickel.  It is distinguished from the <comp>crust</comp> and <comp>mantle</comp>.</def><br/
[<source>PJC</source>]</p>

<p><sn>11.</sn> <fld>(Engineering)</fld> <def>the central part of a nuclear reactor, containing the fissionable fuel.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Core box</b></col> <fld>(Founding)</fld>, <cd>a box or mold, usually divisible, in which cores are molded.</cd> -- <col><b>Core print</b></col> <fld>(Founding)</fld>, <cd>a projecting piece on a pattern which forms, in the mold, an impression for holding in place or steadying a core.</cd> -- <col><b>Core dump</b></col> <cd>See <er>core dump</er> in the vocabulary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Core</ent><br/
<hw>Core</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cord</conjf> <pr>(k?rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coring</conjf>.]</vmorph> <sn>1.</sn> <def>To take out the core or inward parts of; <as>as, to <ex>core</ex> an apple</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He's like a corn upon my great toe . . . he must be <qex>cored</qex> out.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form by means of a core, as a hole in a casting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To extract a cylindrical sample from, with a boring device.  See <er>core{8}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>core dump</ent><br/
<hw>core" dump`</hw> <pr>(k<ocir/r"d<ucr/mp`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Computers)</fld> <def>An complete and exact copy of the contents of a computer <er>core</er>{9}, usually produced as a file when some serious error occurs in the execution of a computer program, and used for debugging the program which produced the error.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>A full account of a person's knowledge on some specific topic, usually produced in response to a question of some kind.  It is a mildly deprecatory term, suggesting that the person producing the account was unable to prepare a more concise and understandable summary of the information; <as>as, I just need the essentials, not a <ex>core dump</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>co-regent</ent><br/
<hw>co-re"gent</hw> <pr>(k?-r?"j<it>e</it>nt)</pr>, <pos>n.</pos> <def>A joint regent or ruler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coregonidae</ent><br/
<hw>Coregonidae</hw> <pos>n.</pos> <def>a natural family of soft-finned fishes comprising the freshwater whitefishes; formerly included in the family <fam>Salmonidae</fam>.</def><br/
<syn><b>Syn. --</b> family <fam>Coregonidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coregonus</ent><br/
<hw>Coregonus</hw> <pos>n.</pos> <def>the type genus of the <fam>Coregonidae</fam>; whitefishes.</def><br/
<syn><b>Syn. --</b> genus <gen>Coregonus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coreid</ent><br/
<hw>coreid</hw> <pos>n.</pos> <def>a true bug.</def><br/
<syn><b>Syn. --</b> coreid bug.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coreidae</ent><br/
<hw>Coreidae</hw> <pos>n.</pos> <def>a natural family containing the squash bugs and leaf-footed bugs.</def><br/
<syn><b>Syn. --</b> family <fam>Coreidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corelate</ent><br/
<hw>corelate</hw> <pos>v.</pos> <def>same as <er>correlate</er>.</def> <mark>[Rare]</mark><br/
<syn><b>Syn. --</b> correlate, correlate with.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>co-relation</ent><br/
<hw>co`-re*la"tion</hw> <pr>(k?`r?-l?"sh?n)</pr>, <pos>n.</pos> <def>Corresponding relation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>co-religionist</ent><br/
<ent>coreligionist</ent><br/
<mhw><hw>coreligionist</hw>, <hw>co`-re*li"gion*ist</hw></mhw> <pr>(-l?j"?n-?st)</pr>, <pos>n.</pos> <def>One of the same religion with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Core loss</ent><br/
<hw>Core loss</hw>. <fld>(Elec.)</fld> <def>Energy wasted by hysteresis or eddy currents in the core of an armature, transformer, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coreopsis</ent><br/
||<hw>Co`re*op"sis</hw> <pr>(k<omac/`r<esl/*<ocr/p"s<icr/s)</pr>, <pos>prop. n.</pos> <ety>[NL., fr. Gr. <grk>ko`ris</grk> bug + <grk>'o`psis</grk> appearance.]</ety> <fld>(Bot.)</fld> <def>A genus of herbaceous composite plants, having the achenes two-horned and remotely resembling some insect; tickseed.  <spn>Coreopsis tinctoria</spn>, of the Western plains, the commonest plant of the genus, has been used in dyeing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coreplasty</ent><br/
<hw>Cor"e*plas`ty</hw> <pr>(k<ocr/r"<esl/*pl<acr/s`t<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`rh</grk> pupil + <ets>-plasty</ets>.]</ety> <fld>(Med.)</fld> <def>A plastic operation on the pupil, as for forming an artificial pupil.</def> -- <wordforms><wf>Cor`e*plas"tic</wf> <pr>(-pl<acr/s"t<icr/k)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corer</ent><br/
<hw>Cor"er</hw> <pr>(k?rr"?rr)</pr>, <pos>n.</pos> <def>That which cores; an instrument for coring fruit; <as>as, an apple <ex>corer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-respondent</ent><br/
<hw>Co`-re*spond"ent</hw> <pr>(k?`rr?-sp?nd"<it>e</it>nt)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One who is called upon to answer a summons or other proceeding jointly with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corf</ent><br/
<hw>Corf</hw> <pr>(k<ocir/rf)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corves</plw> <pr>(k<ocir/rvz)</pr>.</plu> <ety>[Cf. LG. & D. <ets>korf</ets> basket, G. <ets>korb</ets>, fr. L.  <ets>corbis</ets>.]</ety> <sn>1.</sn> <def>A basket.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <sd>(a)</sd> <def>A large basket used in carrying or hoisting coal or ore.</def> <sd>(b)</sd> <def>A wooden frame, sled, or low-wheeled wagon, to convey coal or ore in the mines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corfute</ent><br/
<ent>Corfiote</ent><br/
<mhw>{ <hw>Cor"fi*ote</hw> <pr>(k<ocir/r"f?-?t)</pr>, <hw>Cor"fute</hw> <pr>(k?r"f?t)</pr>, }</mhw> <pos>n.</pos> <def>A native or inhabitant of Corfu, an island in the Mediterranean Sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>corgi</ent><br/
<hw>corgi</hw> <pos>n.</pos> <def>either of two Welsh breeds of long-bodied short-legged dogs with erect ears and a foxlike head.</def><br/
<syn><b>Syn. --</b> Welsh corgi.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coriaceous</ent><br/
<hw>Co`ri*a"ceous</hw> <pr>(k?`r?-?"sh?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>coriaceous</ets>, fr.  <ets>corium</ets> leather. See <er>Cuirass</er>.]</ety> <sn>1.</sn> <def>Consisting of or resembling, leather; leatherlike; tough.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Stiff, like leather or parchment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coriander</ent><br/
<hw>Co`ri*an"der</hw> <pr>(k?`r?-?n"d?r)</pr>, <pos>n.</pos> <ety>[L. <ets>coriandrum</ets>, fr. Gr. <?/<?/<?/<?/, <?/<?/<?/<?/, perh. fr. <?/<?/<?/ bug, on account of the buglike or fetid smell of its leaves: cf. F. <ets>coriandre</ets>.]</ety> <fld>(Bot.)</fld> <def>An umbelliferous plant, the <spn>Coriandrum sativum</spn>, the fruit or seeds of which have a strong smell and a spicy taste, and in medicine are considered as stomachic and carminative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coridine</ent><br/
<hw>Co"ri*dine</hw> <pr>(k?"r?-d?n; 104)</pr>, <pos>n.</pos> <ety>[From L.  <ets>cortium</ets> leather.]</ety> <def>A colorless or yellowish oil, <chform>C10H15N</chform>, of a leathery odor, occuring in coal tar, Dippel's oil, tobacco smoke, etc., regarded as an organic base, homologous with pyridine. Also, one of a series of metameric compounds of which coridine is a type.</def> <altsp>[Written also <asp>corindine</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corindon</ent><br/
<hw>Co*rin"don</hw> <pr>(k?-r?n"d?n)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Corrundum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corinne</ent><br/
||<hw>Co`rinne"</hw> <pr>(k?`r?n")</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The common gazelle (<spn>Gazella dorcas</spn>). See <er>Gazelle</er>.</def> <altsp>[Written also <asp>korin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corinth</ent><br/
<hw>Cor"inth</hw> <pr>(k?r"?nth)</pr>, <pos>n.</pos> <ety>[L. <ets>Corinthus</ets>, Gr. <?/<?/<?/<?/. Cf.  <er>Currant</er>.]</ety> <sn>1.</sn> <def>A city of Greece, famed for its luxury and extravagance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small fruit; a currant.</def> <mark>[Obs.]</mark>  <rj><au>Broome.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corinthiac</ent><br/
<hw>Co*rin"thi*ac</hw> <pr>(k?-r?n"th?-?k)</pr>, <pos>a.</pos> <ety>[L.  <ets>Corinthiacus</ets>.]</ety> <def>Pertaining to Corinth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corinthian</ent><br/
<hw>Co*rin"thi*an</hw> <pr>(-<it>a</it>n)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or relating to Corinth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>Of or pertaining to the Corinthian order of architecture, invented by the Greeks, but more commonly used by the Romans.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the lightest and most ornamental of the three orders used by the Greeks.</q> <rj><qau>Parker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Debauched in character or practice; impure.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to an amateur sailor or yachtsman; <as>as, a <ex>corinthian</ex> race (one in which the contesting yachts must be manned by amateurs</as>.)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corinthian</ent><br/
<hw>Co*rin"thi*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>A native or inhabitant of Corinth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gay, licentious person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A man of fashion given to pleasuring or sport; a fashionable man about town; esp., a man of means who drives his own horse, sails his own yacht, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corium</ent><br/
||<hw>Co"ri*um</hw> <pr>(k?"r?-?m)</pr>, <pos>n.</pos> <ety>[L. <ets>corium</ets> leather.]</ety> <sn>1.</sn> <def>Armor made of leather, particularly that used by the Romans; used also by Enlish soldiers till the reign of Edward I.</def>  <rj><au>Fosbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Same as <er>Dermis</er>.</def> <sd>(b)</sd> <def>The deep layer of mucous membranes beneath the epithelium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corival</ent><br/
<hw>Co*ri"val</hw> <pr>(k<osl/*r<imac/"v<ait/l)</pr>, <pos>n.</pos> <def>A rival; a corrival.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corival</ent><br/
<hw>Co*ri"val</hw>, <pos>v. t.</pos> <def>To rival; to pretend to equal.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corivalship</ent><br/
<ent>Corivalry</ent><br/
<mhw>{ <hw>Co*ri"val*ry</hw>, <hw>Co*ri"val*ship</hw>  }</mhw>, <pos>n.</pos> <def>Joint rivalry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cork</ent><br/
<hw>Cork</hw> <pr>(k<ocir/rk)</pr>, <pos>n.</pos> <ety>[Cf. G., Dan., & Sw. <ets>kork</ets>, D.  <ets>kurk</ets>; all fr. Sp. <ets>corcho</ets>, fr. L. <ets>cortex</ets>, <ets>corticis</ets>, bark, rind. Cf. <er>Cortex</er>.]</ety> <sn>1.</sn> <def>The outer layer of the bark of the cork tree (<spn>Quercus Suber</spn>), of which stoppers for bottles and casks are made. See <er>Cutose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stopper for a bottle or cask, cut out of cork.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mass of tabular cells formed in any kind of bark, in greater or less abundance.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Cork</ex> is sometimes used wrongly for <xex>calk</xex>, <xex>calker</xex>; <xex>calkin</xex>, a sharp piece of iron on the shoe of a horse or ox.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cork jackets</b></col>, <cd>a jacket having thin pieces of cork inclosed within canvas, and used to aid in swimming.</cd> -- <col><b>Cork tree</b></col> <fld>(Bot.)</fld>, <cd>the species of oak (<spn>Quercus Suber</spn> of Southern Europe) whose bark furnishes the cork of commerce.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cork</ent><br/
<hw>Cork</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corked</conjf> <pr>(k<ocir/rkt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Corking</conjf>.]</vmorph> <sn>1.</sn> <def>To stop with a cork, as a bottle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish or fit with cork; to raise on cork.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tread on <qex>corked</qex> stilts a prisoner's pace.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ To <ex>cork</ex> is sometimes used erroneously for to <xex>calk</xex>, to furnish the shoe of a horse or ox with sharp points, and also in the meaning of cutting with a calk.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corkage</ent><br/
<hw>Cork"age</hw> <pr>(-<asl/j)</pr>, <pos>n.</pos> <def>The charge made by innkeepers for drawing the cork and taking care of bottles of wine bought elsewhere by a guest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corked</ent><br/
<hw>Corked</hw> <pr>(k?rkt)</pr>, <pos>a.</pos> <def>having acquired an unpleasant taste from the cork; <as>as, a bottle of wine is <ex>corked</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cork fossil</ent><br/
<hw>Cork" fos`sil</hw> <pr>(k<ocir/rk" f<ocr/s`s<icr/l)</pr>. <fld>(Min.)</fld> <def>A variety of amianthus which is very light, like cork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corkiness</ent><br/
<hw>Cork"i*ness</hw> <pr>(-<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality of being corky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>corking pin</ent><br/
<hw>cork"ing pin`</hw> <pr>(k<ocir/rk"<icr/ng p<icr/n`)</pr>. <def>A pin of a large size, formerly used attaching a woman's headdress to a cork mold.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>corkscrew</ent><br/
<hw>cork"screw`</hw> <pr>(k<ocir/rk"skr<udd/`)</pr>, <pos>n.</pos> <def>An instrument with a screw or a steel spiral for drawing corks from bottles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>corkscrew</ent><br/
<hw>cork"screw`</hw> <pr>(k<ocir/rk"skr<udd/`)</pr>, <pos>a.</pos> <def>shaped like a corkscrew; spiral; helical.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>corkscrew stairs</b></col>, <cd>a spiral staircase around a solid newel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corkscrew</ent><br/
<hw>Cork"screw`</hw>, <pos>v. t.</pos> <def>To press forward in a winding way; <as>as, to <ex>corkscrew</ex> one's way through a crowd</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corkwing</ent><br/
<hw>Cork"wing`</hw> <pr>(-w<icr/ng`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A fish; the goldsinny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corkwood</ent><br/
<hw>Cork"wood`</hw> <pr>(k<ocir/rk"w<oocr/d`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The wood of the cork oak.</def> <mark>[Obs.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Any one of several trees or shrubs having light or corky wood;</def> <specif>esp.</specif>: <sd>(a)</sd> <def>In the United States, the tree <spn>Leitneria floridana</spn>, a very small deciduous dioecious tree or shrub of damp habitats in the southeastern US having extremely light wood; -- called also the <altname>corkwood tree</altname>.</def> <sd>(b)</sd> <def>In the West Indies: (1) Either of the cotton trees <spn>Ochroma lagopus</spn> and <spn>Pariti tiliaceum</spn>.</def>  (2) <def>The tree producing the aligator apple.</def> (3) <def>The blolly.</def><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Corky</ent><br/
<hw>Cork"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Consisting of, or like, cork; dry shriveled up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bind fast hiss <qex>corky</qex> arms.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tasting of cork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corm</ent><br/
<hw>Corm</hw> <pr>(k<ocir/rm)</pr>, <pos>n.</pos> <ety>[See <er>Cormus</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A solid bulb-shaped root, as of the crocus. See <er>Bulb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Same as <er>Cormus</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cormogeny</ent><br/
<hw>Cor*mog"e*ny</hw> <pr>(k<ocir/r*m<ocr/j"<esl/*n<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kormo`s</grk> trunk of a tree + root of <grk>gi`gnesqai</grk> to be born.]</ety> <fld>(Biol.)</fld> <def>The embryological history of groups or families of individuals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cormophylogeny</ent><br/
<hw>Cor`mo*phy*log"e*ny</hw> <pr>(k?r`m?-f?-l?j"?-n?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kormo`s</grk> trunk of a tree + E. <ets>phylogeny</ets>.]</ety> <fld>(Biol.)</fld> <def>The phylogeny of groups or families of individuals.</def>  <rj><au>Haeckel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 324 --></p>

<p><ent>Cormophyta</ent><br/
<ent>Cormophytes</ent><br/
<mhw>{ <hw>Cor"mo*phytes</hw> <pr>(k<ocir/r"m<osl/*f<imac/ts)</pr>, ||<hw>Cor*moph"y*ta</hw> <pr>(k<ocr/r*m<ocr/f"<icr/*t<adot/)</pr>, }</mhw> <pos>n. pl.</pos> <ety>[NL. <ets>cormophyta</ets>, fr. Gr. <grk>kormo`s</grk> trunk of a tree + <grk>fyto`n</grk> plant.]</ety> <fld>(Bot.)</fld> <def>A term proposed by Endlicher to include all plants with an axis containing vascular tissue and with foliage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cormorant</ent><br/
<hw>Cor"mo*rant</hw> <pr>(k<ocir/r"m<osl/*r<ait/nt)</pr>, <pos>n.</pos> <ety>[F.  <ets>cormoran</ets>, fr. Armor. <ets>m<omac/r-vran</ets> a sea raven; <ets>m<omac/r</ets> sea +  <ets>bran</ets> raven, with <ets>cor</ets>, equiv. to L.  <ets>corvus</ets> raven, pleonastically prefixed; or perh. fr. L.  <ets>corvus marinus</ets> sea raven.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any species of <gen>Phalacrocorax</gen>, a genus of sea birds having a sac under the beak; the shag. Cormorants devour fish voraciously, and have become the emblem of gluttony. They are generally black, and hence are called <altname>sea ravens</altname>, and <altname>coalgeese</altname>.</def> <altsp>[Written also <asp>corvorant</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A voracious eater; a glutton, or gluttonous servant.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cormoraut</ent><br/
<hw>Cor"mo*raut</hw>, <pos>a.</pos> <def>Ravenous; voracious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cormorant</qex>, devouring time.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cormus</ent><br/
||<hw>Cor"mus</hw> <pr>(k<ocir/r"m<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kormo`s</grk> the trunk of a tree (with the boughs cut off), fr. <grk>kei`rein</grk> to shear.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Bot.)</fld> <def>See <er>Corm</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>A vegetable or animal made up of a number of individuals, such as, for example, would be formed by a process of budding from a parent stalk wherre the buds remain attached.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corn</ent><br/
<hw>Corn</hw> <pr>(k<ocir/rn)</pr>, <pos>n.</pos> <ety>[L.  <ets>cornu</ets> horn: cf. F. <ets>corne</ets> horn, hornlike excrescence. See <er>Horn</er>.]</ety> <def>A thickening of the epidermis at some point, esp. on the toes, by friction or pressure. It is usually painful and troublesome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Welcome, gentlemen! Ladies that have their toes<br/
Unplagued with <qex>corns</qex>, will have a bout with you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The substance of a corn usually resembles horn, but where moisture is present, as between the toes, it is white and sodden, and is called a <stype>soft corn</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corn</ent><br/
<hw>Corn</hw>, <pos>n.</pos> <ety>[AS.  <ets>corn</ets>; akin to OS.  <ets>korn</ets>, D. <ets>koren</ets>, G., Dan., Sw., & Icel. <ets>korn</ets>, Goth. <ets>ka<uacute/rn</ets>, L.  <ets>granum</ets>, Russ. <ets>zerno</ets>. Cf.  <er>Grain</er>, <er>Kernel</er>.]</ety> <sn>1.</sn> <def>A single seed of certain plants, as wheat, rye, barley, and maize; a grain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The various farinaceous grains of the cereal grasses used for food, as wheat, rye, barley, maize, oats.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In Scotland, <ex>corn</ex> is generally restricted to <xex>oats</xex>, in the United States, to maize, or <stype>Indian corn</stype> (see sense 3), and in England to wheat.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>a tall cereal plant (<spn>Zea mays</spn>) bearing its seeds as large kernels in multiple rows on the surface of a hard cylindrical <part>ear</part>, the core of which (the <part>cob</part>) is not edible; -- also called <altname>Indian corn</altname> and, in technical literature, <altname>maize</altname>. There are several kinds; as, <stype>yellow corn</stype>, which grows chiefly in the Northern States, and is yellow when ripe; <stype>white corn</stype> <it>or</it>  <stype>southern corn</stype>, which grows to a great height, and has long white kernels; <stype>sweet corn</stype>, comprising a number of sweet and tender varieties, grown chiefly at the North, some of which have kernels that wrinkle when ripe and dry; <stype>pop corn</stype>, any small variety, used for popping.  Corn seeds may be cooked while on the ear and eaten directly, or may be stripped from the ear and cooked subsequently.  The term <stype>Indian corn</stype> is often used to refer to a primitive type of corn having kernels of varied color borne on the same cob; it is used for decoration, especially in the fall.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>The plants which produce corn, when growing in the field; the stalks and ears, or the stalks, ears, and seeds, after reaping and before thrashing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In one night, ere glimpse of morn,<br/
His shadowy flail had thrashed the <qex>corn</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A small, hard particle; a grain.</def> <ldquo/<xex>Corn</xex> of sand.<rdquo/ <au>Bp. Hall.</au> <ldquo/A <xex>corn</xex> of powder.<rdquo/ <au>Beau. & Fl.</au><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corn ball</b></col>, <cd>a ball of popped corn stuck together with soft candy from molasses or sugar.</cd> -- <col><b>Corn bread</b></col>, <cd>bread made of Indian meal.</cd> -- <col><b>Corn cake</b></col>, <cd>a kind of corn bread; johnny cake; hoecake.</cd> -- <col><b>Corn cockle</b></col> <fld>(Bot.)</fld>, <cd>a weed (<spn>Agrostemma Githago</spn> syn. <spn>Lychnis Githago</spn>), having bright flowers, common in grain fields.</cd> -- <col><b>Corn flag</b></col> <fld>(Bot.)</fld>, <cd>a plant of the genus <gen>Gladiolus</gen>; -- called also <altname>sword lily</altname>.</cd> -- <col><b>Corn fly</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small fly which, in the larval state, is injurious to grain, living in the stalk, and causing the disease called <ldquo/gout,<rdquo/ on account of the swelled joints. The common European species is <spn>Chlorops t<ae/niopus</spn>.</cd> <sd>(b)</sd> <cd>A small fly (<spn>Anthomyia ze</spn>) whose larva or maggot destroys seed corn after it has been planted.</cd> -- <col><b>Corn fritter</b></col>, <cd>a fritter having green Indian corn mixed through its batter.</cd> <mark>[U. S.]</mark> -- <col><b>Corn laws</b></col>, <cd>laws regulating trade in corn, especially those in force in Great Britain till 1846, prohibiting the importation of foreign grain for home consumption, except when the price rose above a certain rate.</cd> -- <col><b>Corn marigold</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Marigold</er>.</cd> -- <col><b>Corn oyster</b></col>, <cd>a fritter containing grated green Indian corn and butter, the combined taste resembling that of oysters.</cd> <mark>[U.S.]</mark> -- <col><b>Corn parsley</b></col> <fld>(Bot.)</fld>, <cd>a plant of the parsley genus (<spn>Petroselinum segetum</spn>), a weed in parts of Europe and Asia.</cd> -- <col><b>Corn popper</b></col>, <cd>a utensil used in popping corn.</cd> -- <col><b>Corn poppy</b></col> <fld>(Bot.)</fld>, <cd>the red poppy (<spn>Papaver Rh<oe/as</spn>), common in European cornfields; -- also called <altname>corn rose</altname>.</cd> -- <col><b>Corn rent</b></col>, <cd>rent paid in corn.</cd> -- <col><b>Corn rose</b></col>. <cd>See <cref>Corn poppy</cref>.</cd> -- <col><b>Corn salad</b></col> <fld>(Bot.)</fld>, <cd>a name given to several species of <gen>Valerianella</gen>, annual herbs sometimes used for salad. <spn>Valerianella olitoria</spn> is also called <stype>lamb's lettuce</stype>.</cd> -- <col><b>Corn stone</b></col>, <cd>red limestone.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Corn violet</b></col> <fld>(Bot.)</fld>, <cd>a species of <gen>Campanula</gen>.</cd> -- <col><b>Corn weevil</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small weevil which causes great injury to grain.</cd> <sd>(b)</sd> <cd>In America, a weevil (<spn>Sphenophorus ze<ae/</spn>) which attacks the stalk of maize near the root, often doing great damage. See <cref>Grain weevil</cref>, under <er>Weevil</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corn</ent><br/
<hw>Corn</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corned</conjf> <pr>(k?rnd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Corning</conjf>.]</vmorph> <sn>1.</sn> <def>To preserve and season with salt in grains; to sprinkle with salt; to cure by salting; now, specifically, to salt slightly in brine or otherwise; <as>as, to <ex>corn</ex> beef; to <ex>corn</ex> a tongue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form into small grains; to granulate; <as>as, to <ex>corn</ex> gunpowder</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To feed with corn or (in Sctland) oats; <as>as, to <ex>corn</ex> horses</as>.</def>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To render intoxicated; <as>as, ale strong enough to <ex>corn</ex> one</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corning house</b></col>, <cd>a house or place where powder is corned or granulated.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornaceae</ent><br/
<hw>Cornaceae</hw> <pos>n.</pos> <def>a natural family of trees including the genera <gen>Aucuba</gen>; <gen>Cornus</gen>; <gen>Corokia</gen>; <gen>Curtisia</gen>; <gen>Griselinia</gen>; and <gen>Helwingia</gen>.</def><br/
<syn><b>Syn. --</b> family <fam>Cornaceae</fam>, dogwood family.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cornage</ent><br/
<hw>Cor"nage</hw> <pr>(k?r"n?j)</pr>, <pos>n.</pos> <ety>[OF.,, horn-blowing, tax on horned cattle, fr. F.  <ets>corne</ets> a horn, L.  <ets>cornu</ets>.]</ety> <fld>(Law)</fld> <def>An ancient tenure of land, which obliged the tenant to give notice of an invasion by blowing a horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornamute</ent><br/
<hw>Cor"na*mute</hw> <pr>(k?r"n?-m?t)</pr>, <pos>n.</pos> <def>A cornemuse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornball</ent><br/
<hw>Corn"ball</hw> <pr>(k<ocir/rn"b<add/l)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a <cref>corn ball</cref>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a person who is corny; one who is overly sentimental.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>an unsophisticated person; a country bumpkin.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>cornball</ent><br/
<hw>corn"ball</hw> <pr>(k<ocir/rn"b<add/l)</pr>, <pos>a.</pos> <def>same as <er>corny{5}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>cornbind</ent><br/
<hw>corn"bind`</hw> <pr>(k<ocir/rn"b<imac/nd`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A weed that binds stalks of corn, as <spn>Convolvulus arvensis</spn>, <spn>Polygonum Convolvulus</spn>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>corn bread</ent><br/
<hw>corn" bread`</hw> <pr>(k<ocir/rn"br<ecr/d`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a bread made from corn meal.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>in the northeastern U. S., a moist and heavy sourdough rye bread.</def> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>corncob</ent><br/
<hw>corn"cob`</hw> <pr>(k<ocir/rn"k<ocr/b`)</pr>, <pos>n.</pos> <def>The cob or axis on which the kernels of <partof>Indian corn</partof> grow.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>corncob pipe</ent><br/
<hw>corn"cob pipe`</hw> <pr>(k<ocir/rn"k<ocr/b p<imac/p`)</pr>, <pos>n.</pos> <def>a pipe{3} for smoking tobacco with a bowl made from a corncob.</def><br/
[<source>PJC</source>]</p>

<p><ent>corncockle</ent><br/
<hw>corncockle</hw> <pos>n.</pos> <def>a European annual (<spn>Agrostemma githago</spn>) having large trumpet-shaped reddish-purple flowers and poisonous seed; a common weed in grainfields and beside roadways; naturalized in America.</def><br/
<syn><b>Syn. --</b> corn cockle, corn-cockle, corn campion, crown-of-the-field, <spn>Agrostemma githago</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corncrake</ent><br/
<hw>Corn"crake`</hw> <pr>(k<ocir/rn"kr<amac/k`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A bird (<spn>Crex crex</spn> or <spn>Crex pratensis</spn>) which frequents grain fields; the European crake or land rail; -- called also <altname>corn bird</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corncrib</ent><br/
<hw>Corn"crib`</hw> <pr>(k<ocir/rn"kr<icr/b`)</pr>, <pos>n.</pos> <def>A crib for storing corn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corncutter</ent><br/
<hw>Corn"cut`ter</hw> <pr>(k<ocir/rn"k?t`t?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A machine for cutting up stalks of corn for food of cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An implement consisting of a long blade, attached to a handle at nearly a right angle, used for cutting down the stalks of Indian corn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corndodger</ent><br/
<hw>Corn"dodg`er</hw> <pr>(k<ocir/rn"d?j`?r)</pr>, <pos>n.</pos> <def>A cake made of the meal of Indian corn, wrapped in a covering of husks or paper, and baked under the embers.</def> <mark>[U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornea</ent><br/
<hw>Cor"ne*a</hw> <pr>(k?r"n?-?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corneas</plw> <pr>(-<?/z)</pr>.</plu> <ety>[Fem. sing., fr. L. <ets>corneus</ets> horny, fr. <ets>cornu</ets> a horn. See <er>Horn</er>.]</ety> <fld>(Anat.)</fld> <def>The transparent part of the coat of the eyeball which covers the iris and pupil and admits light to the interior. See <er>Eye</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corneal</ent><br/
<hw>Cor"ne*al</hw> <pr>(-<it>al</it>)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to the cornea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornel</ent><br/
<hw>Cor"nel</hw> <pr>(-n?l)</pr>, <pos>n.</pos> <ety>[OF.  <ets>cornille</ets>, <ets>cornoille</ets>, F.  <ets>cornouille</ets>, cornel berry, LL.  <ets>cornolium</ets> cornel tree, fr. L.  <ets>cornus</ets>, fr.  <ets>cornu</ets> horn, in allusion to the hardness of the wood. See <er>Horn</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The cornelian cherry (<spn>Cornus Mas</spn>), a European shrub with clusters of small, greenish flowers, followed by very acid but edible drupes resembling cherries.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any species of the genus <gen>Cornus</gen>, as <spn>Cornus florida</spn>, the flowering cornel; <spn>Cornus stolonifera</spn>, the osier cornel; <spn>Cornus Canadensis</spn>, the dwarf cornel, or bunchberry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornelian</ent><br/
<hw>Cor*nel"ian</hw> <pr>(k?r-n?ly<it>a</it>n)</pr>, <pos>n.</pos> <ety>[F.  <ets>cornaline</ets>, OF.  <ets>corneline</ets>, fr. L. <ets>cornu</ets> horn. So called from its horny appearance when broken. See <er>Horn</er>, and cf. <er>Carnelian</er>.]</ety> <fld>(Min.)</fld> <def>Same as <er>Carnelian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornemuse</ent><br/
<hw>Corne"muse</hw> <pr>(k?rn"m?z)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A wind instrument nearly identical with the bagpipe.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corneocalcareous</ent><br/
<hw>Cor"ne*o*cal*ca"re*ous</hw> <pr>(k?rn?-?-k?l-k?"r?-?s)</pr>, <pos>a.</pos><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Zool.)</fld> <def>Formed of a mixture of horny and calcareous materials, as some shells and corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Horny on one side and calcareous on the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corneous</ent><br/
<hw>Cor"ne*ous</hw> <pr>(-?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>corneus</ets>, fr.  <ets>cornu</ets> horn.]</ety> <def>Of a texture resembling horn; horny; hard.</def>  <rj><au> Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corner</ent><br/
<hw>Cor"ner</hw> <pr>(k?r"n?r)</pr>, <pos>n.</pos> <ety>[OF. <ets>corniere</ets>, <ets>cornier</ets>, LL.  <ets>cornerium</ets>, <ets>corneria</ets>, fr. L.  <ets>cornu</ets> horn, end, point. See <er>Horn</er>.]</ety> <sn>1.</sn> <def>The point where two converging lines meet; an angle, either external or internal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The space in the angle between converging lines or walls which meet in a point; <as>as, the chimney <ex>corner</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An edge or extremity; the part farthest from the center; hence, any quarter or part.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From the four <qex>corners</qex> of the earth they come.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A secret or secluded place; a remote or out of the way place; a nook.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This thing was not done in a <qex>corner</qex>.</q> <rj><qau>Acts xxvi. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Direction; quarter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sits the wind in that <qex>corner</qex>!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The state of things produced by a combination of persons, who buy up the whole or the available part of any stock or species of property, which compels those who need such stock or property to buy of them at their own price; <as>as, a <ex>corner</ex> in a railway stock</as>.</def> <mark>[Broker's Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corner stone</b></col>, <cd>the stone which lies at the corner of two walls, and unites them; the principal stone; especially, the stone which forms the corner of the foundation of an edifice; hence, that which is fundamental importance or indispensable.</cd> <ldquo/A prince who regarded uniformity of faith as the <xex>corner stone</xex> of his government.<rdquo/ <au>Prescott.</au> -- <col><b>Corner tooth</b></col>, <cd>one of the four teeth which come in a horse's mouth at the age of four years and a half, one on each side of the upper and of the lower jaw, between the middle teeth and the tushes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corner</ent><br/
<hw>Cor"ner</hw>, <pos>n.</pos> <fld>(Association Football)</fld> <altname>[More fully <col><b>corner kick</b></col>.]</altname> <def>A free kick from close to the nearest corner flag post, allowed to the opposite side when a player has sent the ball behind his own goal line.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corner</ent><br/
<hw>Cor"ner</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cornered</conjf> <pr>(-n?rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cornering</conjf>.]</vmorph> <sn>1.</sn> <def>To drive into a corner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To drive into a position of great difficulty or hopeless embarrassment; <as>as, to <ex>corner</ex> a person in argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To get command of (a stock, commodity, etc.), so as to be able to put one's own price on it; <as>as, to <ex>corner</ex> the shares of a railroad stock; to <ex>corner</ex> petroleum</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornercap</ent><br/
<hw>Cor"ner*cap`</hw> <pr>(-k?p`)</pr>, <pos>n.</pos> <def>The chief ornament.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou makest the triumviry the <qex>cornercap</qex> of society.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornered</ent><br/
<hw>Cor"nered</hw> <pr>(-n?rd)</pr>, <pos>p. a.</pos> <def>1 Having corners or angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a possition of great difficulty; brought to bay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornerwise</ent><br/
<hw>Cor"ner*wise`</hw> <pr>(-w?z`)</pr>, <pos>adv.</pos> <def>With the corner in front; diagonally; not square.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornet</ent><br/
<hw>Cor"net</hw> <pr>(k?r"n?t)</pr>, <pos>n.</pos> <ety>[F.  <ets>cornet</ets>, m. (for senses 1 & 2), <ets>cornette</ets>, f. & m. (for senses 3 & 4), dim. of <ets>corne horn</ets>, L.  <ets>cornu</ets>. See <er>Horn</er>.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>An obsolete rude reed instrument (Ger. <xex>Zinken</xex>), of the oboe family.</def> <sd>(b)</sd> <def>A brass instrument, with cupped mouthpiece, and furnished with valves or pistons, now used in bands, and, in place of the trumpet, in orchestras. See <er>Cornet-<agrave/-piston</er>.</def> <sd>(c)</sd> <def>A certain organ stop or register.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cap of paper twisted at the end, used by retailers to inclose small wares.</def>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <sd>(a)</sd> <def>A troop of cavalry; -- so called from its being accompanied by a cornet player.</def> <mark>[Obs.]</mark> <ldquo/A body of five <xex>cornets</xex> of horse.<rdquo/ <au>Clarendon.</au> <sd>(b)</sd> <def>The standard of such a troop.</def> <mark>[Obs.]</mark> <sd>(c)</sd> <def>The lowest grade of commissioned officer in a British cavalry troop, who carried the standard. The office was abolished in 1871.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A headdress</def>: <sd>(a)</sd> <def>A square cap anciently worn as a mark of certain professions.</def> <sd>(b)</sd> <def>A part of a woman's headdress, in the 16th century.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Cf.  <er>Coronet</er>.]</ety> <fld>(Far.)</fld> <def>See <er>Coronet</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornet-a-piston</ent><br/
||<hw>Cor"net-<agrave/-pis`ton</hw> <pr>(k?r"n?t-?-p?s"t?n; F. k?r`n?`?p?s`<er>t?n</er>")</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cornets-<agrave/-piston</plw>.</plu> <ety>[F.]</ety> <fld>(Mus.)</fld> <def>A brass wind instrument, like the trumpet, furnished with valves moved by small pistons or sliding rods; a cornopean; a cornet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornetcy</ent><br/
<hw>Cor"net*cy</hw> <pr>(k?r"n?t-s?)</pr>, <pos>n.</pos> <def>The commission or rank of a cornet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corneter</ent><br/
<hw>Cor"net*er</hw> <pr>(k?r"n?t-?r)</pr>, <pos>n.</pos> <def>One who blows a cornet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corneule</ent><br/
<hw>Cor"neule</hw> <pr>(k<ocir/r"n<umac/l)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>corn<eacute/e</ets> the cornea.]</ety> <fld>(Zool.)</fld> <def>One of the corneas of a compound eye in the invertebrates.</def>  <rj><au>Carpenter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornfield</ent><br/
<hw>Corn"field`</hw> <pr>(k?rn"f?ld`)</pr>, <pos>n.</pos> <def>A field where corn is or has been growing; -- in England, a field of wheat, rye, barley, or oats; in America, a field of Indian corn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornfloor</ent><br/
<hw>Corn"floor`</hw> <pr>(k<ocir/rn"fl<omac/r`)</pr>, <pos>n.</pos> <def>A thrashing floor.</def>  <rj><au>Hos. ix. 1.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>corn flour</ent><br/
<ent>cornflour</ent><br/
<mhw><hw>corn"flour`</hw>, <hw>corn" flour`</hw></mhw> <pos>n.</pos> <def>a flour or starch prepared from the grains of corn; it is used in cooking as a thickener.</def><br/
<syn><b>Syn. --</b> cornstarch.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cornflower</ent><br/
<hw>Corn"flow`er</hw> <pr>(k<ocir/rn"flou`<etil/r)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A conspicuous wild flower (<spn>Centaurea Cyanus</spn>), growing in grainfields.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornhusker</ent><br/
<hw>Cornhusker</hw> <pos>n.</pos> <def>a resident of Nebraska; -- a nickname.</def><br/
<syn><b>Syn. --</b> Nebraskan.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cornic</ent><br/
<hw>Cor"nic</hw> <pr>(k<ocir/r"n<icr/k)</pr>, <pos>a.</pos> <def>Pertaining to, derived from, or resembling, the dogwood (<spn>Cornus florida</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornice</ent><br/
<hw>Cor"nice</hw> <pr>(k?r"n?s)</pr>, <pos>n.</pos> <ety>[F. <ets>corniche</ets>, It. <ets>cornice</ets>, LL. <ets>coronix</ets>, <ets>cornix</ets>, fr. L.  <ets>coronis</ets> a curved line, a flourish with the pen at the end of a book or chapter, Gr. <?/<?/<?/; akin to L.  <ets>corona</ets> crown. sEE <er>Crown</er>, and cf.  <er>Coronis</er>.]</ety> <fld>(Arch.)</fld> <def>Any horizontal, molded or otherwise decorated projection which crowns or finishes the part to which it is affixed; <as>as, the <ex>cornice</ex> of an order, pedestal, door, window, or house</as>.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cornice ring</b></col>, <cd>the ring on a cannon next behind the muzzle ring.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corniced</ent><br/
<hw>Cor"niced</hw> <pr>(k?r"n?st)</pr>, <pos>a.</pos> <def>Having a cornice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornicle</ent><br/
<hw>Cor"ni*cle</hw> <pr>(k?r"n?-k'l)</pr>, <pos>n.</pos> <ety>[L. <ets>corniculum</ets>, dim. of <ets>cornu</ets> horn.]</ety> <def>A little horn.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornicular</ent><br/
<hw>Cor*nic"u*lar</hw> <pr>(-l?r)</pr>, <pos>n.</pos> <ety>[L.  <ets>cornicularius</ets>.]</ety> <def>A secretary or clerk.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corniculate</ent><br/
<hw>Cor*nic"u*late</hw> <pr>(k?r-n?k"?-l?t)</pr>, <pos>a.</pos> <ety>[L.  <ets>corniculatus</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Horned; having horns.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having processes resembling small horns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corniculum</ent><br/
||<hw>Cor*nic"u*lum</hw> <pr>(k?r-n?k"?-l?m)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  <plw>Cornicula</plw> <pr>(-l<?/)</pr>.</plu> <ety>[L.  <ets>corniculum</ets> little horn.]</ety> <fld>(Anat.)</fld> <def>A small hornlike part or process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corniferous</ent><br/
<hw>Cor*nif"er*ous</hw> <pr>(k?r-n?f"?r-?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>cornu</ets> horn + <ets>-ferous</ets>.]</ety> <fld>(Geol.)</fld> <def>Of or pertaining to the lowest period of the Devonian age. (See the Diagram, under <er>Geology</er>.) The <xex>Corniferous period</xex> has been so called from the numerous seams of <xex>hornstone</xex> which characterize the later part of the period, as developed in the State of New York.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornific</ent><br/
<hw>Cor*nif"ic</hw> <pr>(k?r-n?f"?k)</pr>, <pos>a.</pos> <ety>[L.  <ets>cornu</ets> horn +  <ets>facere</ets> to make.]</ety> <def>Producing horns; forming horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornification</ent><br/
<hw>Cor`ni*fi*ca"tion</hw> <pr>(k?r`n?-f?-k?"sh?n)</pr>, <pos>n.</pos> <def>Conversion into, or formation of, horn; a becoming like horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornified</ent><br/
<hw>Cor"ni*fied</hw> <pr>(k?r"n?-f?d)</pr>, <pos>a.</pos> <ety>[L.  <ets>cornu</ets> horn +  <ets>-fy</ets>.]</ety> <fld>(Anat.)</fld> <def>Converted into horn; horny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corniform</ent><br/
<hw>Cor"ni*form</hw> <pr>(-f?rm)</pr>, <pos>a.</pos> <ety>[L.  <ets>cornu</ets> horn +  <ets>-form</ets>.]</ety> <def>Having the shape of a horn; horn-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornigerous</ent><br/
<hw>Cor*nig"er*ous</hw> <pr>(k?r-n?j"?r-?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>corniger</ets>; <ets>cornu</ets> horn +  <ets>gerere</ets> to bear.]</ety> <def>Horned; having horns; <as>as, <ex>cornigerous</ex> animals</as>.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornin</ent><br/
<hw>Cor"nin</hw> <pr>(k?r"n?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A bitter principle obtained from dogwood (<spn>Cornus florida</spn>), as a white crystalline substance; -- called also <altname>cornic acid</altname>.</def> <sd>(b)</sd> <def>An extract from dogwood used as a febrifuge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corniplume</ent><br/
<hw>Cor"ni*plume</hw> <pr>(k?r"n?-pl?m)</pr>, <pos>n.</pos> <ety>[L.  <ets>cornu</ets> horn +  <ets>pluma</ets> feather.]</ety> <fld>(Zool.)</fld> <def>A hornlike tuft of feathers on the head of some birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornish</ent><br/
<hw>Cor"nish</hw> <pr>(k?r"n?sh)</pr>, <pos>a.</pos> <def>Of or pertaining to Cornwall, in England.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cornish chough</b></col>. <cd>See <er>Chough</er>.</cd> -- <col><b>Cornish engine</b></col>, <cd>a single-acting pumping engine, used in mines, in Cornwall and elsewhere, and for water works. A heavy pump rod or plunger, raised by the steam, forces up the water by its weight, in descending.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornish</ent><br/
<hw>Cor"nish</hw>, <pos>n.</pos> <def>The dialect, or the people, of Cornwall.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornist</ent><br/
<hw>Cor"nist</hw>, <pos>n.</pos> <def>A performer on the cornet or horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornloft</ent><br/
<hw>Corn"loft`</hw> <pr>(k?rn"l?ft`)</pr>, <pos>n.</pos> <def>A loft for corn; a granary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornmuse</ent><br/
<hw>Corn"muse</hw> <pr>(-m?z)</pr>, <pos>n.</pos> <def>A cornemuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corno di bassetto</ent><br/
||<hw>Cor"no di bas*set"to</hw> <pr>(k?r"n? d? b?s-s?t"t? <or/ b?s-s?t"t?)</pr>; <plu><it>pl.</it> <plw>Corni <pr>(-n<?/)</pr> di basseto</plw>.</plu> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A tenor clarinet; -- called also <altname>basset horn</altname>, and sometimes confounded with the <xex>English horn</xex>, which is a tenor oboe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corno Inglese</ent><br/
||<hw>Cor"no In*gle"se</hw> <pr>(?n-gl?"z?)</pr>; <plu><it>pl.</it> <plw>Corni Inglesi</plw> <pr>(-z<?/)</pr>.</plu> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A reed instrument, related to the oboe, but deeper in pitch; the English horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornopean</ent><br/
<hw>Cor*no"pe*an</hw> <pr>(k?r-n?"p?-<it>a</it>n)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>An obsolete name for the cornet-<agrave/-piston.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornsheller</ent><br/
<hw>Corn"shell`er</hw> <pr>(k?rn"sh?l`?r)</pr>, <pos>n.</pos> <def>A machine that separates the kernels of corn from the cob.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornshuck</ent><br/
<hw>Corn"shuck`</hw> <pr>(-sh<ucr/k`)</pr>, <pos>n.</pos> <def>The husk covering an ear of Indian corn.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornstalk</ent><br/
<hw>Corn"stalk`</hw> <pr>(-st<add/k`)</pr>, <pos>n.</pos> <def>A stalk of Indian corn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornstarch</ent><br/
<hw>Corn"starch`</hw> <pr>(-st<aum/rch`)</pr>, <pos>n.</pos> <def>Starch made from Indian corn, esp. a fine white flour used for puddings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornu</ent><br/
||<hw>Cor"nu</hw> <pr>(k<ocir/r"n<usl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cornua</plw> <pr>(-n<usl/*<adot/)</pr>.</plu> <ety>[L.]</ety> <def>A horn, or anything shaped like or resembling a horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornu Ammonis</ent><br/
<hw>Cor"nu Am*mo"nis</hw> <pr>(<acr/m*m<omac/"n<icr/s)</pr>; <plu><it>pl.</it>  <plw>||Cornua Ammonis</plw></plu>. <ety>[L., horn of Ammon. See <er>Ammonite</er>.]</ety> <fld>(Paleon.)</fld> <def>A fossil shell, curved like a ram's horn; an obsolete name for an ammonite.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 325 --></p>

<p><ent>Cornucopia</ent><br/
<hw>Cor`nu*co"pi*a</hw> <pr>(k<ocir/r`n<usl/*k<omac/"p<icr/*<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cornucopias</plw> <pr>(-<adot/z)</pr>.</plu> <ety>[L. <ets>cornu copiae</ets> horn of plenty. See <er>Horn</er>, and <er>Copious</er>.]</ety> <sn>1.</sn> <def>The horn of plenty, from which fruits and flowers are represented as issuing. It is an emblem of abundance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Bot.)</fld> <def>A genus of grasses bearing spikes of flowers resembling the cornucopia in form.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Some writers maintain that this word should be written, in the singular, <asp>cornu copi<ae/</asp>, and in the plural, <asp>cornua copi<ae/</asp>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornuted</ent><br/
<ent>Cornute</ent><br/
<mhw>{ <hw>Cor"nute</hw> <pr>(k?r"n?t <or/ k?r-n?t")</pr>, <hw>Cor*nut"ed</hw> <pr>(k?r-n?"t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L.  <ets>cornutus</ets> horned, from <ets>cornu</ets> horn.]</ety> <sn>1.</sn> <def>Bearing horns; horned; horn-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cuckolded.</def> <mark>[R.]</mark> <ldquo/My being <xex>cornuted</xex>.<rdquo/  <rj><au>LEstrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornute</ent><br/
<hw>Cor*nute"</hw> <pr>(k?r-n?t")</pr>, <pos>v. t.</pos> <def>To bestow horns upon; to make a cuckold of; to cuckold.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornuto</ent><br/
||<hw>Cor*nu"to</hw> <pr>(k?r-n?"t?)</pr>, <pos>n.</pos> <ety>[It., fr. L.  <ets>cornutus</ets> horned.]</ety> <def>A man that wears the horns; a cuckold.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cornutor</ent><br/
<hw>Cor*nu"tor</hw> <pr>(-t?r)</pr>, <pos>n.</pos> <def>A cuckold maker.</def> <mark>[R.]</mark>  <rj><au>Jordan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corny</ent><br/
<hw>Cor"ny</hw> <pr>(k?r"n?)</pr>, <pos>a.</pos> <ety>[L.  <ets>cornu</ets> horn.]</ety> <def>Strong, stiff, or hard, like a horn; resembling horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Up stood the <qex>cornu</qex> reed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corny</ent><br/
<hw>Corn"y</hw>, <pos>a.</pos> <sn>1.</sn> <def>Producing corn or grain; furnished with grains of corn.</def> <mark>[R.]</mark> <ldquo/The <xex>corny</xex> ear.<rdquo/  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing corn; tasting well of malt.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A draught of moist and <qex>corny</qex> ale.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Tipsy.</def> <mark>[Vulgar, Eng.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>overly or simplistically sentimental.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>trite or tiresome; too weak to be effective; -- said of unsubtle attempts at humor; <as>as, a <ex>corny</ex> joke; a <ex>corny</ex> skit</as>.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Corocore</ent><br/
<hw>Cor"o*core</hw> <pr>(k?r"?-k?r)</pr>, <pos>n.</pos> <def>A kind of boat of various forms, used in the Indian Archipelago.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corody</ent><br/
<hw>Cor"o*dy</hw> <pr>(k?r"?-d?)</pr>, <pos>n.</pos> <ety>[LL. <ets>corrodium</ets>, <ets>corredium</ets>, <ets>conredium</ets>, furniture, provision: cf. OF.  <ets>conroi</ets>. <ets>See</ets> <er>Curry</er>.]</ety> <fld>(Old Law)</fld> <def>An allowance of meat, drink, or clothing due from an abbey or other religious house for the sustenance of such of the king's servants as he may designate to receive it.</def> <altsp>[Written also <asp>corrody</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corol</ent><br/
<hw>Cor"ol</hw> <pr>(k?r"?l)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corolla</ent><br/
<hw>Co*rol"la</hw> <pr>(k?-r?l"l?)</pr>, <pos>n.</pos> <ety>[L.  <ets>corolla</ets> a little crown or garland, dim. of <ets>corona</ets>. See <er>Crown</er>.]</ety> <fld>(Bot.)</fld> <def>The inner envelope of a flower; the part which surrounds the organs of fructification, consisting of one or more leaves, called <xex>petals</xex>. It is usually distinguished from the calyx by the fineness of its texture and the gayness of its colors. See the Note under <er>Blossom</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corollaceous</ent><br/
<hw>Cor`ol*la"ceous</hw> <pr>(k?r`?l-l?"sh?s)</pr>, <pos>a.</pos> <def>Pertaining to, or resembling, a corolla; having the form or texture of a corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corollary</ent><br/
<hw>Cor"ol*la*ry</hw> <pr>(k?r"?l-l?-r?; 277)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corollaries</plw> <pr>(-r<?/z)</pr>.</plu> <ety>[L.  <ets>corollarium</ets> gift, corollary, fr.  <ets>corolla</ets>. See <er>Corolla</er>.]</ety> <sn>1.</sn> <def>That which is given beyond what is actually due, as a garland of flowers in addition to wages; surplus; something added or superfluous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Now come, my Ariel; bring a <qex>corollary</qex>,<br/
Rather than want a spirit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>Something which follows from the demonstration of a proposition; an additional inference or deduction from a demonstrated proposition; a consequence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corollated</ent><br/
<ent>Corollate</ent><br/
<mhw>{ <hw>Cor"ol*late</hw> <pr>(k?r"?l-l?t)</pr>, <hw>Cor"ol*la`ted</hw> <pr>(-l?`t?d)</pr>, }</mhw> <pos>a.</pos> <def>Having a corolla or corollas; like a corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corollet</ent><br/
<hw>Cor"ol*let</hw> <pr>(k?r"?l-l?t)</pr>, <pos>n.</pos>  <ety>[Dim. fr.  <ets>corolla</ets>.]</ety> <fld>(Bot.)</fld> <def>A floret in an aggregate flower.</def> <mark>[Obs.]</mark>  <rj><au>Martyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corolliflorous</ent><br/
<ent>Corollifloral</ent><br/
<mhw>{ <hw>Co*rol`li*flo"ral</hw> <pr>(k?-r?l`l?-fl?"r<it>a</it>l)</pr>, <hw>Co*rol`li*flo"rous</hw> <pr>(-fl?"r?s)</pr>, }</mhw> <pos>a.</pos>  <ety>[<ets>Corolla</ets> + L.  <ets>flos</ets>, <ets>floris</ets>, flower.]</ety> <fld>(Bot.)</fld> <def>Having the stamens borne on the petals, and the latter free from the calyx. Compare <er>Calycifloral</er> and <er>Thalamifloral</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corolline</ent><br/
<hw>Cor"ol*line</hw> <pr>(-l?n)</pr>, <pos>a.</pos> <def>Of or pertaining to a corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coromandel</ent><br/
<hw>Cor`o*man"del</hw> <pr>(k?r`?-m?n"d<it>e</it>l)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>The west coast, or a portion of the west coast, of the Bay of Bengal.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coromandel gooseberry</b></col>. <cd>See <er>Carambola</er>.</cd> -- <col><b>Coromandel wood</b></col>, <cd>Calamander wood.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corona</ent><br/
<hw>Co*ro"na</hw> <pr>(k?-r?"n?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L.  <plw>Coron<ae/</plw> (-n<?/), E.  <plw>Coronas</plw> <pr>(-n<?/z)</pr>.</plu> <ety>[L. <ets>corona</ets> crown. See <er>Crown</er>.]</ety> <sn>1.</sn> <def>A crown or garland bestowed among the Romans as a reward for distinguished services.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>The projecting part of a Classic cornice, the under side of which is cut with a recess or channel so as to form a drip. See <xex>Illust.</xex> of <er>Column</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>The upper surface of some part, as of a tooth or the skull; a crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The shelly skeleton of a sea urchin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Astronomy)</fld> <def>A peculiar luminous appearance, or aureola, which surrounds the sun, and which is seen only when the sun is totally eclipsed by the moon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>An inner appendage to a petal or a corolla, often forming a special cup, as in the daffodil and jonquil.</def> <sd>(b)</sd> <def>Any crownlike appendage at the top of an organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Meteorol.)</fld> <sd>(a)</sd> <def>A circle, usually colored, seen in peculiar states of the atmosphere around and close to a luminous body, as the sun or moon.</def> <sd>(b)</sd> <def>A peculiar phase of the <cref>aurora borealis</cref>, formed by the concentration or convergence of luminous beams around the point in the heavens indicated by the direction of the dipping needle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A crown or circlet suspended from the roof or vaulting of churches, to hold tapers lighted on solemn occasions. It is sometimes formed of double or triple circlets, arranged pyramidically. Called also <altname>corona lucis</altname>.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mus.)</fld> <def>A character [<pause/]  called the <altname>pause</altname> or <altname>hold</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronach</ent><br/
<hw>Cor"o*nach</hw> <pr>(k?r"?-n?k)</pr>, <pos>n.</pos> <def>See <er>Coranach</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronal</ent><br/
<hw>Cor"o*nal</hw> <pr>(k?r"?-n<it>a</it>l <or/, <it>esp. in science</it>, k?-r?"n<it>a</it>l; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>coronalis</ets>: cf. F.  <ets>coronal</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a corona (in any of the senses).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>coronal</qex> light during the eclipse is faint.</q> <rj><qau>Abney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a king's crown, or coronation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law and his <qex>coronal</qex> oath require his undeniable assent to what laws the Parliament agree upon.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to the top of the head or skull.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>Of or pertaining to the shell of a sea urchin.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coronal suture</b></col> <fld>(Anat.)</fld>, <cd>a suture extending across the skull between the parietal and frontal bones; the <xex>fronto-parietal</xex> suture.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronal</ent><br/
<hw>Cor"o*nal</hw>, <pos>n.</pos> <sn>1.</sn> <def>A crown; wreath; garland.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The frontal bone, over which the ancients wore their coron<ae/ or garlands.</def>  <rj><au>Hooper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronamen</ent><br/
<hw>Cor`o*na"men</hw> <pr>(k?r`-n?"m?n)</pr>, <pos>n.</pos> <ety>[L., a crowning.]</ety> <fld>(Zool.)</fld> <def>The upper margin of a hoof; a coronet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronary</ent><br/
<hw>Cor"o*na*ry</hw> <pr>(k?r"?-n?-r?)</pr>, <pos>a.</pos> <ety>[L. <ets>coronarius</ets>: cf. F. <ets>coronaire</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a crown; forming, or adapted to form, a crown or garland.</def> <ldquo/<xex>Coronary</xex> thorns.<rdquo/  <rj><au>Bp. Pearson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The catalogue of <qex>coronary</qex> plants is not large in Theophrastus.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Resembling, or situated like, a crown or circlet; <as>as, the <ex>coronary</ex> arteries and veins of the heart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronary</ent><br/
<hw>Cor"o*na*ry</hw>, <pos>n.</pos> <sn>1.</sn> <def>A small bone in the foot of a horse.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>Informal shortening of <er>coronary thrombosis</er>, also used informally to mean <er>heart attack</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coronary bone</ent><br/
<hw>Cor"o*na*ry bone</hw>. <def>The small pastern bone of the horse and allied animals.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coronary cushion</ent><br/
<hw>Coronary cushion</hw>. <def>A cushionlike band of vascular tissue at the upper border of the wall of the hoof of the horse and allied animals. It takes an important part in the secretion of the horny walls.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coronary thrombosis</ent><br/
<hw>Cor"o*na*ry throm*bo"sis</hw>, <pos>n.</pos> <def>the blockage of the coronary artery of the heart by a thrombus.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coronated</ent><br/
<ent>Coronate</ent><br/
<mhw>{ <hw>Cor"o*nate</hw> <pr>(k?r"?-n?t)</pr>, <hw>Cor"o*na`ted</hw> <pr>(-n?`t?ed)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>coronatus</ets>, p. p. of <ets>coronare</ets> to crown, fr. <ets>corona</ets>. See <er>Crown</er>.]</ety> <sn>1.</sn> <def>Having or wearing a crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having the coronal feathers lengthened or otherwise distinguished; -- said of birds.</def> <sd>(b)</sd> <def>Girt about the spire with a row of tubercles or spines; -- said of spiral shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>Having a crest or a crownlike appendage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronation</ent><br/
<hw>Cor`o*na"tion</hw> <pr>(k?r`?-n?"sh?n)</pr>, <pos>n.</pos> <ety>[See <er>Coronate</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The act or solemnity of crowning a sovereign; the act of investing a prince with the insignia of royalty, on his succeeding to the sovereignty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The pomp or assembly at a coronation.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronel</ent><br/
<hw>Coro"nel</hw> <pr>(k?r"n<it>e</it>l)</pr>, <pos>n.</pos> <ety>[See <er>Colonel</er>.]</ety> <def>A colonel.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronel</ent><br/
<hw>Cor"o*nel</hw> <pr>(k?r"?-n?l <or/ k?r"n?l)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cronel</er>, <er>Crown</er>.]</ety> <fld>(Anc. Armor)</fld> <def>The iron head of a tilting spear, divided into two, three, or four blunt points.</def> <altsp>[Written also <asp>cronel</asp>.]</altsp>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coroner</ent><br/
<hw>Cor"o*ner</hw> <pr>(k?r"?-n?r)</pr>, <pos>n.</pos> <ety>[From OE. <ets>coronen</ets> to crown, OF. <ets>coroner</ets>, fr. L.  <ets>coronare</ets>, fr. <ets>corona</ets> crown. Formed as a translation of LL. <ets>coronator</ets> coroner, fr. L. <ets>corona</ets> crown, the coroner having been originally a prosecuting officer of the crown. See <er>Crown</er>.]</ety> <def>An officer of the peace whose principal duty is to inquire, with the help of a jury, into the cause of any violent, sudden or mysterious death, or death in prison, usually on sight of the body and at the place where the death occurred.</def> <altsp>[In England formerly also written and pronounced <asp>crowner</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some of the United States the office of <xex>coroner</xex> is abolished, that of <xex>medical examiner</xex> taking its place.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coroner's inquest</b></col>. <cd>See under <er>Inquest</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronet</ent><br/
<hw>Cor"o*net</hw> <pr>(k?r"?-n?t)</pr>, <pos>n.</pos> <ety>[Dim. of OE. <ets>corone</ets> crown; cf. OF. <ets>coronete</ets>. See <er>Crown</er>, and cf. <er>Crownet</er>, <er>Cronet</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An ornamental or honorary headdress, having the shape and character of a crown; particularly, a crown worn as the mark of high rank lower than sovereignty. The word is used by Shakespeare to denote also a kingly crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without a star, a <qex>coronet</qex>, or garter.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>coronet</xex> of the Prince of Wales consist of a circlet of gold with four crosses <xex>patt<eacute/e</xex> around the edge between as many <xex>fleurs-de-lis</xex>. The center crosses are connected by an arch which is surmounted by a globe or cross. The <xex>coronet</xex> of a British duke is adorned with strawberry leaves; that of a marquis has leaves with pearls interposed; that of an earl raises the pearls above the leaves; that of a viscount is surrounded with pearls only; that of a baron has only four pearls.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Far.)</fld> <def>The upper part of a horse's hoof, where the horn terminates in skin.</def>  <rj><au>James White.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anc. Armor)</fld> <def>The iron head of a tilting spear; a coronel.</def>  <rj><au>Crose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coroneted</ent><br/
<hw>Cor"o*net*ed</hw> <pr>(-n?t-?d)</pr>, <pos>a.</pos><def>Wearing, or entitled to wear, a coronet; of noble birth or rank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coroniform</ent><br/
<hw>Co*ron"i*form</hw> <pr>(k?-r?n"?-f?rm <or/ k?-r?"n?-)</pr>, <pos>a.</pos> <ety>[L. <ets>corona</ets> crown + <ets>-form</ets>.]</ety> <def>Having the form of a crown or coronet; resembling a crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronilla</ent><br/
<hw>Cor`o*nil"la</hw> <pr>(k?r`?-n?l"l?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>corona</ets> crown: cf. F. <ets>coronille</ets>.]</ety> <fld>(Bot.)</fld> <def>A genus of plants related to the clover, having their flowers arranged in little heads or tufts resembling coronets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronis</ent><br/
<hw>Co*ro"nis</hw> <pr>(k?-r?"n?s)</pr>, <pos>n.</pos> <ety>[Gr. <grk>korwni`s</grk> anything curved. See <er>Cornice</er>.]</ety> <sn>1.</sn> <def>In Greek grammar, a sign ['] sometimes placed over a contracted syllable.</def>  <rj><au>W. W. Goodwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The curved line or flourish at the end of a book or chapter; hence, the end.</def> <mark>[R.]</mark>  <rj><au>Bp. Hacket.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronium</ent><br/
<hw>Co*ro"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Corona</er>.]</ety> <fld>(Chem. & Astron.)</fld> <def>The principal gaseous substance forming the solar corona, characterized by a green line in the coronal spectrum.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coronoid</ent><br/
<hw>Cor"o*noid</hw> <pr>(k<ocr/r"<osl/*noid)</pr>, <pos>a.</pos> <ety>[Gr. <grk>korw`nh</grk> crow + <ets>-oid</ets>: cf. F. <ets>corono<ium/de</ets>.]</ety> <fld>(Anat.)</fld> <def>Resembling the beak of a crow; <as>as, the <ex>coronoid</ex> process of the jaw, or of the ulna</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coronule</ent><br/
<hw>Cor"o*nule</hw> <pr>(k?r"?-n?l)</pr>, <pos>n.</pos> <ety>[L. <ets>coronula</ets>, dim. of <ets>corona</ets> crown.]</ety> <fld>(Bot.)</fld> <def>A coronet or little crown of a seed; the downy tuft on seeds. See <er>Pappus</er>.</def>  <rj><au>Martyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coroun</ent><br/
<hw>Co*roun"</hw> <pr>(k?-roun")</pr>, <pos>v. & n.</pos> <def>Crown.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corosso</ent><br/
<ent>Corozo</ent><br/
<mhw>{ ||<hw>Co*ro"zo</hw> ||<hw>Co*ros"so</hw>  }</mhw> <pr>(k?-r?"th? <or/ -s?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>cerozo</ets> a kind of palm tree.]</ety> <def>The name in Central America for the seed of a true palm; also, a commercial name for the true ivory nut. See <er>Ivory nut</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporace</ent><br/
<hw>Cor"po*race</hw> <pr>(k?r"p?-r?s)</pr>, <pos>n.</pos> <def>See <er>Corporas</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporal</ent><br/
<hw>Cor"po*ral</hw> <pr>(k<ocir/r"p<osl/*r<ait/l)</pr>, <pos>n.</pos> <ety>[Corrupted fr. F. <ets>caporal</ets>, It. <ets>caporale</ets>, fr. <ets>capo</ets> head, chief, L. <ets>caput</ets>. See <er>Chief</er>, and cf. <er>Caporal</er>.]</ety> <fld>(Mil.)</fld> <def>A noncommissioned officer, next below a sergeant. In the United States army he is the lowest noncommissioned officer in a company of infantry. He places and relieves sentinels.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corporal's guard</b></col>, <cd>a detachment such as would be in charge of a corporal for guard duty, etc.; hence, derisively, a very small number of persons.</cd> -- <col><b>Lance corporal</b></col>, <cd>an assistant corporal on private's pay.</cd> <au>Farrow.</au> -- <col><b>Ship's corporal</b></col> <fld>(Naut.)</fld>, <cd>a petty officer who assists the master at arms in his various duties.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporal</ent><br/
<hw>Cor"po*ral</hw>, <pos>a.</pos> <ety>[L. <ets>corporalis</ets>, fr. <ets>corpus</ets> body. See <er>Corpse</er>.]</ety> <sn>1.</sn> <def>Belonging or relating to the body; bodily.</def> <ldquo/Past <xex>corporal</xex> toil.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pillories and other <qex>corporal</qex> infections.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corporal punishment</b></col> <fld>(law)</fld>, <cd>punishment applied to the body of the offender, including the death penalty, whipping, and imprisonment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a body or substance; not spiritual; material. In this sense now usually written <xex>corporeal</xex>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>corporal</qex> heaven . . . .where the stare are.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What seemed <qex>corporal</qex> melted<br/
As breath into the wind.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- <er>Corporal</er>, <er>Bodily</er>, <er>Corporeal</er>.</syn> <usage> <xex>Bodily</xex> is opposed to <xex>mental</xex>; as, <xex>bodily</xex> affections. <xex>Corporeal</xex> refers to the whole physical structure or nature, of the body; as, <xex>corporeal</xex> substance or frame. <xex>Corporal</xex>, as now used, refers more to punishment or some infliction; as, <xex>corporal</xex> punishment. To speak of <xex>corporeal</xex> punishment is an error. <xex>Bodily</xex> austerities; the <xex>corporeal</xex> mold.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporale</ent><br/
<ent>Corporal</ent><br/
<mhw>{ <hw>Cor"po*ral</hw> <pr>(k<ocir/r"p<osl/*r<ait/l)</pr>, ||<hw>Cor`po*ra"le</hw> <pr>(-r?"l?)</pr>, }</mhw> <pos>n.</pos> <ety>[LL. <ets>corporale</ets>: cf. F. <ets>corporal</ets>. See <er>Corporal</er>,<pos>a.</pos>]</ety> <def>A fine linen cloth, on which the sacred elements are consecrated in the eucharist, or with which they are covered; a communion cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corporal oath</b></col>, <cd>a solemn oath; -- so called from the fact that it was the ancient usage for the party taking it to touch the corporal, or cloth that covered the consecrated elements.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporality</ent><br/
<hw>Cor`po*ral"i*ty</hw> <pr>(k?r`p?-r?l"l?-t?)</pr>, <pos>n.</pos>: <plu><it>pl.</it> <plw>Corporalities</plw> <pr>(-t<?/z)</pr>.</plu> <ety>[L.  <ets>corporalitas</ets>: cf. F.<ets>corporalit<?/</ets>.]</ety> <sn>1.</sn> <def>The state of being or having a body; bodily existence; corporeality; -- opposed to <xex>spirituality</xex>.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A confraternity; a guild.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporally</ent><br/
<hw>Cor"po*ral*ly</hw> <pr>(k?r"p?-r<it>a</it>l-ly)</pr>, <pos>adv.</pos> <def>In or with the body; bodily; <as>as, to be <ex>corporally</ex> present</as>.</def>  <rj><au>Sharp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporalship</ent><br/
<hw>Cor"po*ral*ship</hw>, <pos>n.</pos> <fld>(Mil.)</fld> <def>A corporal's office.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporas</ent><br/
<hw>Cor"po*ras</hw> <pr>(k?r"p?-r?s)</pr>, <pos>n.</pos> <ety>[Prop. pl. of <ets>corporal</ets>.]</ety> <def>The corporal, or communion cloth.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporate</ent><br/
<hw>Cor"po*rate</hw> <pr>(k?r"p?-r?t)</pr>, <pos>a.</pos> <ety>[L. <ets>corporatus</ets>, p. p. of <ets>corporare</ets> to shape into a body, fr.  <ets>corpus</ets> body. See <er>Corpse</er>.]</ety> <sn>1.</sn> <def>Formed into a body by legal enactment; united in an association, and endowed by law with the rights and liabilities of an individual; incorporated; <as>as, a <ex>corporate</ex> town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Belonging to a corporation or incorporated body.</def> <ldquo/<xex>Corporate</xex> property.<rdquo/  <rj><au>Hallam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>United; general; collectively one.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They answer in a joint and <qex>corporate</qex> voice.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corporate member</b></col>, <cd>an actual or voting member of a corporation, as distinguished from an associate or an honorary member; as, a <xex>corporate member</xex> of the American Board.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporate</ent><br/
<hw>Cor"po*rate</hw> <pr>(-r?t)</pr>, <pos>v. t.</pos> <def>To incorporate.</def> <mark>[Obs.]</mark>  <rj><au> Stow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporate</ent><br/
<hw>Cor"po*rate</hw>, <pos>v. i.</pos> <def>To become incorporated.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporately</ent><br/
<hw>Cor"po*rate*ly</hw> <pr>(-r?t-l?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a corporate capacity; acting as a corporate body.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In, or as regarda, the body.</def>  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>corporate raider</ent><br/
<hw>corporate raider</hw> <pos>n.</pos> <fld>(Finance)</fld> <def>a person who purchases or attempts to purchase a controlling interest in a publicly-traded company against the wishes of the current management.</def> <note>such a buyout is caled a <er>hostile takeover</er>.</note><br/
<syn><b>Syn. --</b> buyout specialist, takeover artist.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Corporation</ent><br/
<hw>Cor`po*ra"tion</hw> <pr>(k<ocir/r`p<osl/*r<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L.  <ets>corporatio</ets> incarnation: cf. F. <ets>corporation</ets> corporation.]</ety> <def>A body politic or corporate, formed and authorized by law to act as a single person, and endowed by law with the capacity of succession; a society having the capacity of transacting business as an individual.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Corporations</xex> are <xex>aggregate</xex> or <xex>sole</xex>. <stype>Corporations aggregate</stype> consist of two or more persons united in a society, which is preserved by a succession of members, either forever or till the corporation is dissolved by the power that formed it, by the death of all its members, by surrender of its charter or franchises, or by forfeiture. Such corporations are the mayor and aldermen of cities, the head and fellows of a college, the dean and chapter of a cathedral church, the stockholders of a bank or insurance company, etc. A <stype>corporation sole</stype> consists of a single person, who is made a body corporate and politic, in order to give him some legal capacities, and especially that of succession, which as a natural person he can not have. Kings, bishops, deans, parsons, and vicars, are in England sole corporations. A fee will not pass to a corporation sole without the word <ldquo/successors<rdquo/ in the grant. There are instances in the United States of a minister of a parish seized of parsonage lands in the right of his parish, being a corporation sole, as in Massachusetts. <xex>Corporations</xex> are sometimes classified as <xex>public</xex> and <xex>private</xex>; <xex>public</xex> being convertible with <xex>municipal</xex>, and <stype>private corporations</stype> being all corporations not municipal.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Close corporation</b></col>. <cd>See under <er>Close</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporator</ent><br/
<hw>Cor"po*ra`tor</hw> <pr>(k<ocir/"p?-r?`t?r)</pr>, <pos>n.</pos> <def>A member of a corporation, esp. one of the original members.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporature</ent><br/
<hw>Cor"po*ra*ture</hw> <pr>(k<ocir/r"p?-r?-t?r)</pr>, <pos>n.</pos> <def>The state of being embodied; bodily existence.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporeal</ent><br/
<hw>Cor*po"re*al</hw> <pr>(k<ocir/r*p<omac/"r<esl/*<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>corporeus</ets>, fr.  <ets>corpus</ets> body.]</ety> <def>Having a body; consisting of, or pertaining to, a material body or substance; material; -- opposed to <contr>spiritual</contr> or <contr>immaterial</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His omnipotence<br/
That to <qex>corporeal</qex> substance could add<br/
Speed almost spiritual.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corporeal property</b></col>, <cd>such as may be seen and handled (as opposed to <xex>incorporeal</xex>, which can not be seen or handled, and exists only in contemplation).</cd>  <rj><au>Mozley & W.</au></rj></cs></p>

<p><syn><b>Syn.</b> -- Corporal; bodily. See <er>Corporal</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporealism</ent><br/
<hw>Cor*po"re*al*ism</hw> <pr>(-?z'm)</pr>, <pos>n.</pos> <def>Materialism.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 326 --></p>

<p><ent>Corporealist</ent><br/
<hw>Cor*po"re*al*ist</hw> <pr>(k<ocr/r*p<omac/"r<esl/*<ait/l*<icr/st)</pr>, <pos>n.</pos> <def>One who denies the reality of spiritual existences; a materialist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>corporealists</qex> pretended . . . to make a world without a God.</q> <rj><qau>Bp. Berkeley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporeality</ent><br/
<hw>Cor*po`re*al"i*ty</hw> <pr>(-?l"?-t?)</pr>, <pos>n.</pos>: <plu><it>pl.</it> <plw>Corporealities</plw> <pr>(-t<icr/z)</pr>.</plu> <def>The state of being corporeal; corporeal existence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporeally</ent><br/
<hw>Cor*po"re*al*ly</hw> <pr>(k<ocr/r*p<omac/"r<esl/*<ait/l*l<ycr/)</pr>, <pos>adv.</pos> <def>In the body; in a bodily form or manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporealness</ent><br/
<hw>Cor*po"re*al*ness</hw> <pr>(-n?s)</pr>, <pos>n.</pos> <def>Corporeality; corporeity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporeity</ent><br/
<hw>Cor`po*re"i*ty</hw> <pr>(k?r`p?-r?"?-t?)</pr>, <pos>n.</pos> <ety>[LL.  <ets>corporeitas</ets>:  cf. F. <ets>corpor<?/it<?/</ets>.]</ety> <def>The state of having a body; the state of being corporeal; materiality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The one attributed <qex>corporeity</qex> to God.</q> <rj><qau>Bp. Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who deny light to be matter, do not therefore deny its <qex>corporeity</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corporify</ent><br/
<hw>Cor*por"i*fy</hw> <pr>(k?r-p?r"?-f?)</pr>, <pos>v. t.</pos> <ety>[L.  <ets>corpus</ets> body + <ets>-fy</ets>: cf. F. <ets>corporifier</ets>.]</ety> <def>To embody; to form into a body.</def> <mark>[Obs.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corposant</ent><br/
<hw>Cor"po*sant</hw> <pr>(k?r"p?-z?nt)</pr>, <pos>n.</pos> <ety>[It. <ets>corpo santo</ets> holy body.]</ety> <def>St. Elmo's fire. See under <er>Saint</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corps</ent><br/
<hw>Corps</hw> <pr>(k<omac/r, <it>pl.</it> k<omac/rz)</pr>, <pos>n. sing. & pl.</pos> <ety>[F., fr. L.  <ets>corpus</ets> body. See <er>Corpse</er>.]</ety> <sn>1.</sn> <def>The human body, whether living or dead.</def> <mark>[Obs.]</mark> <see>See <er>Corpse</er>, 1.</see><br/
[<source>1913 Webster</source>]</p>

<p><q>By what craft in my <qex>corps</qex>, it cometh [commences] and where.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A body of men; esp., an organized division of the military establishment; <as>as, the marine <ex>corps</ex>; the <ex>corps</ex> of topographical engineers</as>; specifically, an army corps.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>corps</qex> operating with an army should consist of three divisions of the line, a brigade of artillery, and a regiment of cavalry.</q> <rj><qau>Gen. Upton (U. S. Tactics. )</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A body or code of laws.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole <qex>corps</qex> of the law.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eccl.)</fld> <def>The land with which a prebend or other ecclesiastical office is endowed.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The prebendaries over and above their reserved rents have a <qex>corps</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Ger.]</ety> <def>In some countries of Europe, a form of students' social society binding the members to strict adherence to certain student customs and its code of honor; -- Ger. spelling usually <asp>korps</asp>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Army corps</b></col>, <it>or</it> (French) <col><b>Corps d'arm<eacute/e</b></col> <pr>(k<osl/r` d<aum/r`m<asl/")</pr></mcol>, <cd>a body containing two or more divisions of a large army, organized as a complete army in itself.</cd> -- <col><b>||Corps de logis</b></col> <pr>(k<osl/r` d<eit/ l<osl/`zh<esl/")</pr> <ety>[F., body of the house]</ety>, <cd>the principal mass of a building, considered apart from its wings.</cd> -- <col><b>Corps diplomatique</b></col> <pr>(k<?/r d<?/`pl<?/`m<?/-t<?/k")</pr> <ety>[F., diplomatic body]</ety>, <cd>the body of ministers or envoys accredited to a government.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpse</ent><br/
<hw>Corpse</hw> <pr>(k<ocir/rps)</pr>, <pos>n.</pos> <ety>[OF.  <ets>cors</ets> (sometimes written <ets>corps</ets>), F.  <ets>corps</ets>, L.  <ets>corpus</ets>; akin to AS.  <ets>hrif</ets> womb. See <er>Midriff</er>, and cf. <er>Corse</er>, <er>Corselet</er>, <er>Corps</er>, <er>Cuerpo</er>.]</ety> <sn>1.</sn> <def>A human body in general, whether living or dead; -- sometimes contemptuously.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Formerly written (after the French form) <xex>corps</xex>. See <er>Corps</er>, <pos>n.</pos>, 1.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The dead body of a human being; -- used also Fig.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He touched the dead <qex>corpse</qex> of Public Credit, and it sprung upon its feet.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corpse candle</b></col>. <sd>(a)</sd> <cd>A thick candle formerly used at a lich wake, or the customary watching with a corpse on the night before its interment. <sd>(b)</sd> A luminous appearance, resembling the flame of a candle, sometimes seen in churchyards and other damp places, superstitiously regarded as portending death.</cd> -- <col><b>Corpse gate</b></col>, <cd>the gate of a burial place through which the dead are carried, often having a covered porch; -- called also <altname>lich gate</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpulency</ent><br/
<ent>Corpulence</ent><br/
<mhw>{ <hw>Cor"pu*lence</hw> <pr>(k?r"p?-l<it>e</it>ns)</pr>, <hw>Cor"pu*len*cy</hw> <pr>(k?r"p?-l<it>e</it>n-s?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>corpulentia</ets>: cf. F.  <ets>corpulence</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Excessive fatness; fleshiness; obesity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Thickness; density; compactness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The heaviness and <qex>corpulency</qex> of water requiring a great force to divide it.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpulent</ent><br/
<hw>Cor"pu*lent</hw> <pr>(-p?-l<it>e</it>nt)</pr>, <pos>a.</pos> <ety>[L.  <ets>corpulentus</ets>, fr. <ets>corpus</ets>: cf. F.  <ets>corpulent</ets>. See <er>Corpse</er>.]</ety> <sn>1.</sn> <def>Very fat; obese.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Solid; gross; opaque.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj></p>

<p><syn><b>Syn.</b> -- Stout; fleshy; bulky; obese. See <er>Stout</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpulently</ent><br/
<hw>Cor"pu*lent*ly</hw>, <pos>adv.</pos> <def>In a corpulent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpus</ent><br/
||<hw>Cor"pus</hw> <pr>(-p<ucr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corpora</plw> <pr>(-p<osl/*r<adot/)</pr>.</plu> <ety>[L.]</ety> <def>A body, living or dead; the corporeal substance of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corpus callosum</b></col> <pr>(k<acr/l*l<omac/"s<ucr/m)</pr>; <plu><it>pl.</it> <plw>Corpora callosa</plw> (-s<?/)</plu> <ety>[NL., callous body]</ety> <fld>(Anat.)</fld>, <cd>the great band of commissural fibers uniting the cerebral hemispheres. See <er>Brain</er>.</cd> -- <col><b>Corpus Christi</b></col> <pr>(kr<icr/s"t<imac/)</pr> <ety>[L., body of Christ]</ety> <fld>(R. C. Ch.)</fld>, <cd>a festival in honor of the eucharist, observed on the Thursday after Trinity Sunday.</cd> -- <col><b>Corpus Christi cloth</b></col>. <cd>Same as <cref>Pyx cloth</cref>, under <er>Pyx</er>.</cd> -- <col><b>Corpus delicti</b></col> <pr>(d<esl/*l<icr/k"t<imac/)</pr> <ety>[L., the body of the crime]</ety> <fld>(Law)</fld>, <cd>the substantial and fundamental fact of the comission of a crime; the proofs essential to establish a crime.</cd> -- <col><b>Corpus luteum</b></col> <pr>(l<umac/"t<esl/*<ucr/m)</pr>; <plu><it>pl.</it>  <plw>Corpora lutea</plw> <pr>(-<adot/)</pr>.</plu> <ety>[NL., luteous body]</ety> <fld>(Anat.)</fld>, <cd>the reddish yellow mass which fills a ruptured Graafian follicle in the mammalian ovary.</cd> -- <col><b>Corpus striatum</b></col> <pr>(str<isl/*<amac/"t<ucr/m)</pr>; <plu><it>pl.</it> <plw>Corpora striata</plw> <pr>(-t<adot/)</pr>.</plu> <ety>[NL., striate body]</ety> <fld>(Anat.)</fld>, <cd>a ridge in the wall of each lateral ventricle of the brain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpuscle</ent><br/
<hw>Cor"pus*cle</hw> <pr>(-p<ucr/s*s'l)</pr>, <pos>n.</pos> <ety>[L.  <ets>corpusculum</ets>, dim. of <ets>corpus</ets>.]</ety> <sn>1.</sn> <def>A minute particle; an atom; a molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A protoplasmic animal cell; esp., such as float free, like blood, lymph, and pus <xex>corpuscles</xex>; or such as are imbedded in an intercellular matrix, like connective tissue and cartilage <xex>corpuscles</xex>.  See <er>Blood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Virchow showed that the <qex>corpuscles</qex> of bone are homologous with those of connective tissue.</q> <rj><qau>Quain's Anat.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physics)</fld> <def>An electron.</def> <mark>[archaic]</mark> <br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Red blood corpuscles</b></col> <fld>(Physiol.)</fld>, <cd>in man, yellowish, biconcave, circular discs varying from <fract>1/3500</fract> to <fract>1/3200</fract> of an inch in diameter and about <fract>1/12400</fract> of an inch thick. They are composed of a colorless stroma filled in with semifluid h<ae/moglobin and other matters. In most mammals the red corpuscles are circular, but in the camels, birds, reptiles, and the lower vertebrates generally, they are oval, and sometimes more or less spherical in form. In Amphioxus, and most invertebrates, the blood corpuscles are all white or colorless.</cd> -- <col><b>White blood corpuscles</b></col> <fld>(Physiol.)</fld>, <cd>rounded, slightly flattened, nucleated cells, mainly protoplasmic in composition, and possessed of contractile power. In man, the average size is about <fract>1/2500</fract> of an inch, and they are present in blood in much smaller numbers than the red corpuscles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpuscular</ent><br/
<hw>Cor*pus"cu*lar</hw> <pr>(k?r-p?s"k?-l?r)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>corpusculaire</ets>.]</ety> <def>Pertaining to, or composed of, corpuscles, or small particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corpuscular philosophy</b></col>, <cd>that which attempts to account for the phenomena of nature, by the motion, figure, rest, position, etc., of the minute particles of matter.</cd> -- <col><b>Corpuscular theory</b></col> <fld>(Opt.)</fld>, <cd>the theory enunciated by Sir Isaac Newton, that light consists in the emission and rapid progression of minute particles or corpuscles. The theory is now generally rejected, and supplanted by the <xex>undulatory theory</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpuscularian</ent><br/
<hw>Cor*pus`cu*la"ri*an</hw> <pr>(-l?"r?-<ait/n)</pr>, <pos>a.</pos> <def>Corpuscular.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpuscularian</ent><br/
<hw>Cor*pus`cu*la"ri*an</hw>, <pos>n.</pos> <def>An adherent of the corpuscular philosophy.</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpuscule</ent><br/
<hw>Cor*pus"cule</hw> <pr>(k?r-p?s"k?l)</pr>, <pos>n.</pos> <def>A corpuscle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corpusculous</ent><br/
<hw>Cor*pus"cu*lous</hw> <pr>(-k?-l?s)</pr>, <pos>a.</pos> <def>Corpuscular.</def>  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrade</ent><br/
<hw>Cor*rade"</hw> <pr>(k?r-r?d")</pr>, <pos>v. t.</pos> <ety>[L.  <ets>corradere</ets>, <ets>-rasum</ets>; <ets>cor-</ets> + <ets> radere</ets> to rub.]</ety> <sn>1.</sn> <def>To gnaw into; to wear away; to fret; to consume.</def> <mark>[Obs.]</mark>  <rj><au>Dr. R. Clerke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>To erode, as the bed of a stream. See <er>Corrosion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corradial</ent><br/
<hw>Cor*ra"di*al</hw> <pr>(k?r-r?"d?-<ait/l)</pr>, <pos>a.</pos> <def>Radiating to or from the same point.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corradiate</ent><br/
<hw>Cor*ra"di*ate</hw> <pr>(k?r-r?"d?-?t)</pr>, <pos>v. t.</pos> <def>To converge to one point or focus, as light or rays.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corradiation</ent><br/
<hw>Cor*ra`di*a"tion</hw> <pr>(k?r-r?`d?-?"sh?n)</pr>, <pos>n.</pos> <def>A conjunction or concentration of rays in one point.</def>  <rj><au>Bacom</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corral</ent><br/
<hw>Cor*ral"</hw> <pr>(k?r-r?l"; Sp. k?r-r?l")</pr>, <pos>n.</pos> <ety>[Sp., a yard, a yard for cattle, fr. <ets>corro</ets> a circle or ring, fr. L.  <ets>currere</ets> to run. Cf. <er>Kraal</er>.]</ety> <def>A pen for animals; esp., an inclosure made with wagons, by emigrants in the vicinity of hostile Indians, as a place of security for horses, cattle, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corral</ent><br/
<hw>Cor*ral"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corraled</conjf> <pr>(-r?ld" <or/ -r?ld")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Corralling</conjf>.]</vmorph> <def>To surround and inclose; to coop up; to put into an inclosed space; -- primarily used with reference to securing horses and cattle in an inclosure of wagons while traversing the plains, but in the Southwestern United States now colloquially applied to the capturing, securing, or penning of anything.</def>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrasion</ent><br/
<hw>Cor*ra"sion</hw> <pr>(k?r-r?"zh?n)</pr>, <pos>n.</pos> <ety>[See <er>Corrade</er>.]</ety> <fld>(Geol.)</fld> <def>The erosion of the bed of a stream by running water, principally by attrition of the detritus carried along by the stream, but also by the solvent action of the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrasive</ent><br/
<hw>Cor*ra"sive</hw> <pr>(-s?v)</pr>, <pos>a.</pos> <def>Corrosive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Corrasive</qex> sores which eat into the flesh.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correct</ent><br/
<hw>Cor*rect"</hw> <pr>(k<ocir/r*r<ecr/kt")</pr>, <pos>a.</pos> <ety>[L. <ets>correctus</ets>, p. p. of <ets>corrigere</ets> to make straight, to correct; <ets>cor-</ets> + <ets>regere</ets> to lead straight: cf. F. <ets>correct</ets>. See <er>Regular</er>, <er>Right</er>, and cf. <er>Escort</er>.]</ety> <def>Set right, or made straight; hence, conformable to truth, rectitude, or propriety, or to a just standard; not faulty or imperfect; free from error; <as>as, <ex>correct</ex> behavior; <ex>correct</ex> views</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Always use the most <qex>correct</qex> editions.</q> <rj><qau>Felton.</qau></rj></p>

<p><syn><b>Syn.</b> -- Accurate; right, exact; precise; regular; faultless. See <er>Accurate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correct</ent><br/
<hw>Cor*rect"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corrected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Correcting</conjf>.]</vmorph> <sn>1.</sn> <def>To make right; to bring to the standard of truth, justice, or propriety; to rectify; <as>as, to <ex>correct</ex> manners or principles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is a defect in the first make of some men's minds which can scarce ever be <qex>corrected</qex> afterwards.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To remove or retrench the faults or errors of; to amend; to set right; <as>as, to <ex>correct</ex> the proof (that is, to mark upon the margin the changes to be made, or to make in the type the changes so marked)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bring back, or attempt to bring back, to propriety in morals; to reprove or punish for faults or deviations from moral rectitude; to chastise; to discipline; <as>as, a child should be <ex>corrected</ex> for lying</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My accuser is my 'prentice; and when I did <qex>correct</qex> him for his fault the other day, he did vow upon his knees he would be even with me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To counteract the qualities of one thing by those of another; -- said of whatever is wrong or injurious; <as>as, to <ex>correct</ex> the acidity of the stomach by alkaline preparations</as>.</def></p>

<p><syn><b>Syn.</b> -- To amend; rectify; emend; reform; improve; chastise; punish; discipline; chasten. See <er>Amend</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctable</ent><br/
<ent>Correctible</ent><br/
<mhw>{ <hw>Cor*rect"i*ble</hw> <pr>(-r?k"t?-b'l)</pr>, <hw>Cor*rect"a*ble</hw> <pr>(-r?k"t?-b'l)</pr>, }</mhw> <pos>a.</pos> <def>Capable of being corrected.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctify</ent><br/
<hw>Cor*rect"i*fy</hw> <pr>(k?r-r?k"t?-f?)</pr>, <pos>v. t.</pos> <def>To correct.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When your worship's plassed to <qex>correctify</qex> a lady.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>correcting</ent><br/
<hw>correcting</hw> <pos>n.</pos> <def>the act of offering an improvement to replace a mistake.</def><br/
<syn><b>Syn. --</b> correction, rectification.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Correction</ent><br/
<hw>Cor*rec"tion</hw> <pr>(k?r-r?k"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>correctio</ets>: cf. F. <ets>correction</ets>.]</ety> <sn>1.</sn> <def>The act of correcting, or making that right which was wrong; change for the better; amendment; rectification, as of an erroneous statement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The due <qex>correction</qex> of swearing, rioting, neglect of God's word, and other scandalouss vices.</q> <rj><qau>Strype.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of reproving or punishing, or that which is intended to rectify or to cure faults; punishment; discipline; chastisement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Correction</qex> and instruction must both work<br/
Ere this rude beast will profit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is substituted in the place of what is wrong; an emendation; <as>as, the <ex>corrections</ex> on a proof sheet should be set in the margin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Abatement of noxious qualities; the counteraction of what is inconvenient or hurtful in its effects; <as>as, the <ex>correction</ex> of acidity in the stomach</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An allowance made for inaccuracy in an instrument; <as>as, chronometer <ex>correction</ex>; compass <ex>correction</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Correction line</b></col> <fld>(Surv.)</fld>, <cd>a parallel used as a new base line in laying out township in the government lands of the United States. The adoption at certain intervals of a <xex>correction line</xex> is necessitated by the convergence of of meridians, and the statute requirement that the townships must be squares.</cd> -- <col><b>House of correction</b></col>, <cd>a house where disorderly persons are confined; a bridewell.</cd> -- <col><b>Under correction</b></col>, <cd>subject to correction; admitting the possibility of error.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctional</ent><br/
<hw>Cor*rec"tion*al</hw> <pr>(k?r-r?k"sh?n-<ait/l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>correctionnel</ets>.]</ety> <def>Tending to, or intended for, correction; used for correction; <as>as, a <ex>correctional</ex> institution</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctioner</ent><br/
<hw>Cor*rec"tion*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who is, or who has been, in the house of correction.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>correctitude</ent><br/
<hw>correctitude</hw> <pos>n.</pos> <def>correct or appropriate behavior.</def><br/
<syn><b>Syn. --</b> propriety, properness.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corrective</ent><br/
<hw>Cor*rect"ive</hw> <pr>(k?rr-r?k"t?v)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>correctif</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Having the power to correct; tending to rectify; <as>as, <ex>corrective</ex> penalties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mulberries are pectoral, <qex>corrective</qex> of billious alkali.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Qualifying; limiting.</def> <ldquo/The Psalmist interposeth . . . this <xex>corrective</xex> particle.<rdquo/  <rj><au>Holdsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrective</ent><br/
<hw>Cor*rect"ive</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which has the power of correcting, altering, or counteracting what is wrong or injurious; <as>as, alkalies are <ex>correctives</ex> of acids; penalties are <ex>correctives</ex> of immoral conduct</as>.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Limitation; restriction.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctly</ent><br/
<hw>Cor*rect"ly</hw> <pr>(k?r-r?kt"l?)</pr>, <pos>adv.</pos> <def>In a correct manner; exactly; acurately; without fault or error.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctness</ent><br/
<hw>Cor*rect"ness</hw>, <pos>n.</pos> <def>The state or quality of being correct; <as>as, the <ex>correctness</ex> of opinions or of manners; <ex>correctness</ex> of taste; <ex>correctness</ex> in writing or speaking; the <ex>correctness</ex> of a text or copy</as>.</def></p>

<p><syn><b>Syn.</b> -- Accuracy; exactness; precision; propriety.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrector</ent><br/
<hw>Cor*rect"or</hw> <pr>(k?r-r?kt"?r)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who, or that which, corrects; <as>as, a <ex>corrector</ex> of abuses; a <ex>corrector</ex> of the press; an alkali is a <ex>corrector</ex> of acids</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctory</ent><br/
<hw>Cor*rect"o*ry</hw> <pr>(-?-r?)</pr>, <pos>a.</pos> <def>Containing or making correction; corrective.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correctress</ent><br/
<hw>Cor*rect"ress</hw> <pr>(-r?s)</pr>, <pos>n.</pos> <def>A woman who corrects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corregidor</ent><br/
||<hw>Cor*reg"i*dor</hw> <pr>(k?r-r?j"?-d?r; Sp. k?r-r?`h?-d?r")</pr>, <pos>n.</pos> <ety>[Sp., orig., a corrector.]</ety> <def>The chief magistrate of a Spanish town.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correi</ent><br/
<hw>Cor"rei</hw> <pr>(k?r"r?)</pr>, <pos>n.</pos> <ety>[Scot., perh. fr. Celt.  <ets>cor</ets> a corner.]</ety> <def>A hollow in the side of a hill, where game usually lies.</def> <ldquo/Fleet foot on the <xex>correi</xex>.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlatable</ent><br/
<hw>Cor`re*lat"a*ble</hw> <pr>(k3r`r?-l?t"?-b'l)</pr>, <pos>a.</pos> <def>Such as can be correlated; <as>as, <ex>correlatable</ex> phenomena</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlate</ent><br/
<hw>Cor`re*late"</hw> <pr>(k<ocr/r`r<esl/*l<amac/t" <or/  k<ocr/r"r<esl/*l<amac/t`)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Correlated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Correlating</conjf>.]</vmorph> <ety>[Pref. <ets>cor-</ets> + <ets>relate</ets>.]</ety> <def>To have reciprocal or mutual relations; to be mutually related.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Doctrine and worship <qex>correlate</qex> as theory and practice.</q> <rj><qau>Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlate</ent><br/
<hw>Cor`re*late"</hw>, <pos>v. t.</pos> <def>To put in relation with each other; to connect together by the disclosure of a mutual relation; <as>as, to <ex>correlate</ex> natural phenomena</as>.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlate</ent><br/
<hw>Cor"re*late</hw> <pr>(k?r"r?-l?t)</pr>, <pos>n.</pos> <def>One who, or that which, stands in a reciprocal relation to something else, as father to son; a correlative.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>correlated</ent><br/
<hw>correlated</hw> <pos>adj.</pos> <sn>1.</sn>  <def>mutually related.</def><br/
<syn><b>Syn. --</b> correlative, correlate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Mathematics, statistics)</fld> <def>showing a statistically significant relationship between the values of two or more variables; <as>as, The statures of fathers and sons are <ex>correlated</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Correlation</ent><br/
<hw>Cor`re*la"tion</hw> <pr>(-l?"sh?n)</pr>, <pos>n.</pos> <ety>[LL. <ets>correlatio</ets>; L. <ets>cor-</ets> + <ets>relatio</ets>: cf. F.  <ets>corr<eacute/lation</ets>. Cf. <er>Correlation</er>.]</ety> <def>Reciprocal relation; corresponding similarity or parallelism of relation or law; capacity of being converted into, or of giving place to, one another, under certain conditions; <as>as, the <ex>correlation</ex> of forces, or of zymotic diseases</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Correlation of energy</b></col>, <cd>the relation to one another of different forms of energy; -- usually having some reference to the principle of conservation of energy.  See <cref>Conservation of energy</cref>, under <er>Conservation</er>.</cd> -- <col><b>Correlation of forces</b></col>, <cd>the relation between the forces which matter, endowed with various forms of energy, may exert.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlative</ent><br/
<hw>Cor*rel"a*tive</hw> <pr>(k?r-r?l"?-t?v)</pr>, <pos>a.</pos> <ety>[Cf. F.  <ets>corr<eacute/latif</ets>.]</ety> <def>Having or indicating a reciprocal relation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Father and son, prince and subject, stranger and citizen, are <qex>correlative</qex> terms.</q> <rj><qau>Hume.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlative</ent><br/
<hw>Cor*rel"a*tive</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, stands in a reciprocal relation, or is correlated, to some other person or thing.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Spiritual things and spiritual men are <qex>correlatives</qex>.</q> <rj><qau>Spelman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>The antecedent of a pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlatively</ent><br/
<hw>Cor*rel"a*tive*ly</hw>, <pos>adv.</pos> <def>In a correlative relation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correlativeness</ent><br/
<hw>Cor*rel"a*tive*ness</hw>, <pos>n.</pos> <def>Quality of being correlative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correligionist</ent><br/
<hw>Cor`re*li"gion*ist</hw> <pr>(k?r`r?-l?j"?n-?st)</pr>, <pos>n.</pos> <def>A co-religion<?/ist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correption</ent><br/
<hw>Cor*rep"tion</hw> <pr>(k?r-r?p"sh?n)</pr>, <pos>n.</pos> <ety>[L.  <ets>correptio</ets>, <ets>fr</ets>.  <ets>corripere</ets> to seize.]</ety> <def>Chiding; reproof; reproach.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Angry, passionate <qex>correption</qex> being rather apt to provoke, than to amend.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspond</ent><br/
<hw>Cor`re*spond"</hw> <pr>(k?r`r?-sp?nd")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corresponded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Corresponding</conjf>.]</vmorph> <ety>[Pref.  <ets>cor-</ets> + <ets>respond</ets>: cf. f. <ets>correspondre</ets>.]</ety> <sn>1.</sn> <def>To be like something else in the dimensions and arrangement of its parts; -- followed by <xex>with</xex> or <xex>to</xex>; <as>as, concurring figures <ex>correspond</ex> with each other throughout</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>None of them [the forms of Sidney's sonnets]  <qex>correspond</qex> to the Shakespearean type.</q> <rj><qau>J. A. Symonds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be adapted; to be congruous; to suit; to agree; to fit; to answer; -- followed by <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Words being but empty sounds, any farther than they are signs of our ideas, we can not but assent to them as they <qex>correspond</qex> to those ideas we have, but no farther.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have intercourse or communion; especially, to hold intercourse or to communicate by sending and receiving letters; -- followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After having been long in indirect communication with the exiled family, he [Atterbury] began to <qex>correspond</qex> directly with the Pretender.</q> <rj><qau>Macaulay.</qau></rj></p>

<p><syn><b>Syn.</b> -- To agree; fit; answer; suit; write; address.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspondence</ent><br/
<hw>Cor`re*spond"ence</hw> <pr>(-sp?nd"<it>e</it>ns)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>correspondance</ets>.]</ety> <sn>1.</sn> <def>Friendly intercourse; reciprocal exchange of civilities; especially, intercourse between persons by means of letters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Holding also good <qex>correspondence</qex> with the other great men in the state.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To facilitate <qex>correspondence</qex> between one part of London and another, was not originally one of the objects of the post office.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 327 --></p>

<p><sn>2.</sn> <def>The letters which pass between correspondents.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mutual adaptation, relation, or agreement, of one thing to another; agreement; congruity; fitness; relation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspondence school</ent><br/
<hw>Cor`res*pond"ence school</hw>. <def>A school that teaches by correspondence, the instruction being based on printed instruction sheets and the recitation papers written by the student in answer to the questions or requirements of these sheets. In the broadest sense of the term <xex>correspondence school</xex> may be used to include any educational institution or department for instruction by correspondence, as in a university or other educational bodies, but the term is commonly applied to various educational institutions organized on a commercial basis, some of which offer a large variety of courses in general and technical subjects, conducted by specialists.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Correspondency</ent><br/
<hw>Cor`re*spond"en*cy</hw> <pr>(k?r`r?-sp?nd"<it>e</it>n-s?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Correspondencies</plw> <pr>(-s<?/z)</pr>.</plu> <def>Same as <er>Correspondence</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>correspondencies</qex> of types and antitypes . . . may be very reasonable confirmations.</q> <rj><qau>S. Clarke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspondent</ent><br/
<hw>Cor`re*spond"ent</hw> <pr>(-<it>e</it>nt)</pr>, <pos>a.</pos> <ety>[Cf. F.  <ets>correspondant</ets>.]</ety> <def>Suitable; adapted; fit; corresponding; congruous; conformable; in accord or agreement; obedient; willing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Action <qex>correspondent</qex> or repugnant unto the law.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As fast the <qex>correspondent</qex> passions rise.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will be <qex>correspondent</qex> to command.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspondent</ent><br/
<hw>Cor`re*spond"ent</hw>, <pos>n.</pos> <sn>1.</sn> <def>One with whom intercourse is carried on by letter.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who communicates information, etc., by letter or telegram to a newspaper or periodical.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>One who carries on commercial intercourse by letter or telegram with a person or firm at a distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspondently</ent><br/
<hw>Cor`re*spond"ent*ly</hw>, <pos>adv.</pos> <def>In a a corresponding manner; conformably; suitably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corresponding</ent><br/
<hw>Cor`re*spond"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Answering; conformable; agreeing; suiting; <as>as, <ex>corresponding</ex> numbers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Carrying on intercourse by letters.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corresponding member of a society</b></col>, <cd>one residing at a distance, who has been invited to correspond with the society, and aid in carrying out its designs without taking part in its management.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Correspondingly</ent><br/
<hw>Cor`re*spond"ing*ly</hw>, <pos>adv.</pos> <def>In a corresponding manner; conformably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corresponsive</ent><br/
<hw>Cor`re*spon"sive</hw> <pr>(-r?-sp?n"s?v)</pr>, <pos>a.</pos> <def>Corresponding; conformable; adapted.</def> <au>Shak.</au> -- <wordforms><wf>Cor`re*spon"sive*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corridor</ent><br/
<hw>Cor"ri*dor</hw> <pr>(k?r"r?-d?r <or/ -d?r)</pr>, <pos>n.</pos> <ety>[F., fr. Itt. <ets>corridpore</ets>, or Sp.  <ets>corredor</ets>; prop., a runner, hence, a running or long line, a gallery, fr. L.  <ets>currere</ets> to run. See <er>Course</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Arch.)</fld> <def>A gallery or passageway leading to several apartments of a house.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fort.)</fld> <def>The covered way lying round the whole compass of the fortifications of a place.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>any relatively narrow passageway or route, such as a strip of land through a foreign territory.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a densely populated stretch of land; <as>as, the Northeast <ex>corridor</ex>, extending from Richmond, Virginia into Maine</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Corridor train</ent><br/
<hw>Cor"ri*dor train</hw>. <def>A train whose coaches are connected so as to have through its entire length a continuous corridor, into which the compartments open.</def> <mark>[Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corrie</ent><br/
<hw>Cor"rie</hw> <pr>(k?r"r?)</pr>, <pos>n.</pos> <def>Same as <er>Correi</er>.</def> <mark>[Scot.]</mark>  <rj><au>Geikie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>corrigenda</ent><br/
<hw>corrigenda</hw> <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>corrigendum</singw></sing>. <def>a list of printing errors in a book along with their corrections; <as>as, an insert with addenda and <ex>corrigenda</ex></as>.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Corrigendum</ent><br/
||<hw>Cor`ri*gen"dum</hw> <pr>(k?r`r?-j?n"d?m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corrigenda</plw> <pr>(-d<?/)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <def>A fault <it>or</it> error to be corrected.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A correction of an error, especially in a printed document.</def><br/
[<source>PJC</source>]</p>

<p><ent>Corrigent</ent><br/
<hw>Cor"ri*gent</hw> <pr>(k?r"r?--j<it>e</it>nt)</pr>, <pos>n.</pos> <ety>[L. <ets>corrigens</ets>, p. pr. of <ets>corrigere</ets> to correct.]</ety> <fld>(Med.)</fld> <def>A substance added to a medicine to mollify or modify its action.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrigibility</ent><br/
<hw>Cor`ri*gi*bil"i*ty</hw> <pr>(-j?-b?l"?-t?)</pr>, <pos>n.</pos> <def>Quality of being corrigible; capability of being corrected; corrigibleness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrigible</ent><br/
<hw>Cor"ri*gi*ble</hw> <pr>(k?r"r?-j?-b'l)</pr>, <pos>a.</pos> <ety>[LL. <ets>corribilis</ets>, fr. L.  <ets>corrigere</ets> to correct: cf. F. <ets>corrigible</ets>. See <er>Correrct</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Capable of being set right, amended, or reformed; <as>as, a <ex>corrigible</ex> fault</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Submissive to correction; docile.</def> <ldquo/Bending down his <xex>corrigible</xex> neck.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Deserving chastisement; punishable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was taken up very short, and adjudged <qex>corrigible</qex> for such presumptuous language.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having power to correct; corrective.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The . . . .<qex>corrigible</qex> authority of this lies in our wills.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrigibleness</ent><br/
<hw>Cor"ri*gi*ble*ness</hw>, <pos>n.</pos> <def>The state or quality of being corrigible; corrigibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrival</ent><br/
<hw>Cor*ri"val</hw> <pr>(k?r-r?"v<it>a</it>l)</pr>, <pos>n.</pos> <def>A fellow rival; a competitor; a rival; also, a companion.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrival</ent><br/
<hw>Cor*ri"val</hw>, <pos>a.</pos> <def>Having rivaling claims; emulous; in rivalry.</def> <mark>[R.]</mark>  <rj><au>Bp. Fleetwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrival</ent><br/
<hw>Cor*ri"val</hw>, <pos>v. i. & t.</pos> <def>To compete with; to rival.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrivalry</ent><br/
<hw>Cor*ri"val*ry</hw> <pr>(k?r-r?"v<it>a</it>l-r?)</pr>, <pos>n.</pos> <def>Corivalry.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrivalship</ent><br/
<hw>Cor*ri"val*ship</hw>, <pos>n.</pos> <def>Corivalry.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By the <qex>corrivalship</qex> of Shager his false friend.</q> <rj><qau>Sir T. Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrivate</ent><br/
<hw>Cor"ri*vate</hw> <pr>(k?r"r?-v?t)</pr>, <pos>v. t.</pos> <ety>[L. <ets>corrivatus</ets>, p. p. of <ets>corrivare</ets> to corrivate.]</ety> <def>To cause to flow together, as water drawn from several streams.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrivation</ent><br/
<hw>Cor`ri*va"tion</hw> <pr>(-v?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>corrivatio</ets>.]</ety> <def>The flowing of different streams into one.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroborant</ent><br/
<hw>Cor*rob"o*rant</hw> <pr>(k?r-r?b"?-r<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[L. <ets>corroborans</ets>, p. pr. See <er>Corroborate</er>.]</ety> <def>Strengthening; supporting; corroborating.</def> <au>Bacon.</au> -- <def2><pos>n.</pos> <def>Anything which gives strength or support; a tonic.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><q>The brain, with its proper <qex>corroborants</qex>, especially with sweet odors and with music.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroborate</ent><br/
<hw>Cor*rob"o*rate</hw> <pr>(k?r-r?b"?-r?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corroborated</conjf> <pr>(-r?`t?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Corroborating</conjf> <pr>(-r?`t?ng)</pr>. ]</vmorph> <ety>[L. <ets>corroboratus</ets>, p. p. of <ets>corroborare</ets> to corroborate; <ets>cor-</ets> + <ets>roborare</ets> to strengthen, <ets>robur</ets> strength. See <er>Robust</er>.]</ety> <sn>1.</sn> <def>To make strong, or to give additional strength to; to strengthen.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As any limb well and duly exercised, grows stronger, the nerves of the body are <qex>corroborated</qex> thereby.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make more certain; to confirm; to establish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The concurrence of all <qex>corroborates</qex> the same truth.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroborate</ent><br/
<hw>Cor*rob"o*rate</hw> <pr>(-r?t)</pr>, <pos>a.</pos> <def>Corroborated.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>corroborated</ent><br/
<hw>corroborated</hw> <pos>adj.</pos> <def>supported or established by evidence or proof; <as>as, <ex>corroborated</ex> testimony is especially convincing</as>.</def><br/
<syn><b>Syn. --</b> substantiated, verified.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corroboration</ent><br/
<hw>Cor*rob`o*ra"tion</hw> <pr>(k?r-r?b`?-r?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>corroboration</ets>.]</ety> <sn>1.</sn> <def>The act of corroborating, strengthening, or confirming; addition of strength; confirmation; <as>as, the <ex>corroboration</ex> of an argument, or of information</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which corroborates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroborative</ent><br/
<hw>Cor*rob"o*ra*tive</hw> <pr>(k?r-r?b"?-r?-t?v)</pr>, <pos>a.</pos> <ety>[Cf. F.  <ets>corroboratif</ets>.]</ety> <def>Tending to strengthen of confirm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroborative</ent><br/
<hw>Cor*rob"o*ra*tive</hw>, <pos>n.</pos> <def>A medicine that strengthens; a corroborant.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroboratory</ent><br/
<hw>Cor*rob"o*ra*to*ry</hw> <pr>(-t?-r?)</pr>, <pos>a.</pos> <def>Tending to strengthen; corroborative; <as>as, <ex>corroboratory</ex> facts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroboree</ent><br/
<hw>Cor*rob"o*ree`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Also <ets>corrobboree</ets>, <ets>corrobori</ets>, etc.]</ety> <ety>[Native name.]</ety>   <sn>1.</sn> <def>A nocturnal festivity with which the Australian aborigines celebrate tribal events of importance. Symbolic dances are given by the young men of the tribe, while the women act as musicians.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A song or chant made for such a festivity.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A festivity or social gathering, esp. one of a noisy or uproarious character; hence, tumult; uproar.</def> <mark>[Australia]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corrobory</ent><br/
<hw>Cor*rob"o*ry</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Corroboree</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corrode</ent><br/
<hw>Cor*rode"</hw> <pr>(k?r-r?d")</pr> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corroded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Corroding</conjf>.]</vmorph> <ety>[L. <ets>corrodere</ets>, <ets>-rosum</ets>; <ets>cor</ets> + <ets>rodere</ets> to gnaw: cf. F. <ets>corroder</ets>. See <er>Rodent</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To eat away by degrees; to wear away or diminish by gradually separating or destroying small particles of, as by action of a strong acid or a caustic alkali.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Aqua fortis <qex>corroding</qex> copper . . . is wont to reduce it to a green-blue solution.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To consume; to wear away; to prey upon; to impair.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrode</ent><br/
<hw>Cor*rode"</hw>, <pos>v. i.</pos> <def>To have corrosive action; to be subject to corrosion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corroding lead</b></col>, <cd>lead sufficiently pure to be used in making white lead by a process of corroding.</cd></cs></p>

<p><syn><b>Syn.</b> -- To canker; gnaw; rust; waste; wear away.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrodent</ent><br/
<hw>Cor*rod"ent</hw> <pr>(k?r-r?"d<it>e</it>nt)</pr>, <pos>a.</pos> <ety>[L. <ets>corrodens</ets>, p. pr. of <ets>corrodere</ets>.]</ety> <def>Corrosive.</def> <mark>[R.]</mark>  <rj><au>Bp. King.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrodent</ent><br/
<hw>Cor*rod"ent</hw>, <pos>n.</pos> <def>Anything that corrodes.</def>  <rj><au>Bp. King.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrodiate</ent><br/
<hw>Cor*ro"di*ate</hw> <pr>(k?r-r?"d?-?t)</pr>, <pos>v. t.</pos> <ety>[See <er>Corrode</er>.]</ety> <def>To eat away by degrees; to corrode.</def> <mark>[Obs.]</mark>  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrodibility</ent><br/
<hw>Cor*ro`di*bil"i*ty</hw> <pr>(k?r-r?`d?-b?l"?-t?)</pr>, <pos>n.</pos> <def>The quality of being corrodible.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrodible</ent><br/
<hw>Cor*rod"i*ble</hw> <pr>(k?r-r?"d?-b'l)</pr>, <pos>a.</pos> <def>Capable of being corroded; corrosible.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrosibility</ent><br/
<hw>Cor*ro`si*bil"i*ty</hw> <pr>(k?r-r?`s?-b?l"?-t?)</pr>, <pos>n.</pos> <def>Corrodibility.</def> <ldquo/<xex>Corrosibility</xex> . . . answers corrosiveness.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrosible</ent><br/
<hw>Cor*ro"si*ble</hw> <pr>(k?r-r?"s?-b'l)</pr>, <pos>a.</pos> <def>Corrodible.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrosibleness</ent><br/
<hw>Cor*ro"si*ble*ness</hw>, <pos>n.</pos> <def>The quality or state of being corrosible.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrosion</ent><br/
<hw>Cor*ro"sion</hw> <pr>(k?r-r?"zh?n)</pr>, <pos>n.</pos> <ety>[LL. <ets>corrosio</ets>: cf. F. <ets>corrosion</ets>. See <er>Corrode</er>.]</ety> <def>The action or effect of corrosive agents, or the process of corrosive change; <as>as, the rusting of iron is a variety of <ex>corrosion</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Corrosion</qex> is a particular species of dissolution of bodies, either by an acid or a saline menstruum.</q> <rj><qau>John Quincy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrosive</ent><br/
<hw>Cor*ro"sive</hw> <pr>(k?r-r?"s?v)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>corrosif</ets>.]</ety> <sn>1.</sn> <def>Eating away; having the power of gradually wearing, changing, or destroying the texture or substance of a body; <as>as, the <ex>corrosive</ex> action of an acid</as>.</def> <ldquo/<xex>Corrosive</xex> liquors.<rdquo/ <au>Grew.</au> <ldquo/<xex>Corrosive</xex> famine.<rdquo/ <au>Thomson.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the quality of fretting or vexing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Care is no cure, but <qex>corrosive</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corrosive sublimate</b></col> <fld>(Chem.)</fld>, <cd>mercuric chloride, <chform>HgCl2</chform>; so called because obtained by sublimation, and because of its harsh irritating action on the body tissue. Usually it is in the form of a heavy, transparent, crystalline substance, easily soluble, and of an acrid, burning taste. It is a virulent poison, a powerful antiseptic, and an excellent antisyphilitic; called also <altname>mercuric bichloride</altname>. It is to be carefully distinguished from calomel, the mild chloride of mercury.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrosive</ent><br/
<hw>Cor*ro"sive</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which has the quality of eating or wearing away gradually.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[<qex>Corrosives</qex>] act either directly, by chemically destroying the part, or indirectly by causing inflammation and gangrene.</q> <rj><qau>Dunglison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which has the power of fretting or irritating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such speeches . . . are grievous <qex>corrosives</qex>.</q> <rj><qau>Hooker.</qau></rj></p>

<p>-- <wordforms><wf>Cor*ro"sive*ly</wf>, <pos>adv.</pos> -- <wf>Cor*ro"sive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corroval</ent><br/
<hw>Cor*ro"val</hw> <pr>(kr-r?"v<it>a</it>l)</pr>, <pos>n.</pos> <def>A dark brown substance of vegetable origin, allied to curare, and used by the natives of New Granada as an arrow poison.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrovaline</ent><br/
<hw>Cor*ro"va*line</hw> <pr>(-v?-l?n <or/ -l?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A poisonous alkaloid extracted from corroval, and characterized by its immediate action in paralyzing the heart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrugant</ent><br/
<hw>Cor"ru*gant</hw> <pr>(k?r"r?-g<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[L. <ets>corrugans</ets>, p. pr. See <er>Corrugate</er>.]</ety> <def>Having the power of contracting into wrinkles.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrugate</ent><br/
<hw>Cor"ru*gate</hw> <pr>(k?r"r?-g?t)</pr>, <pos>a.</pos> <ety>[L. <ets>corrugatus</ets>, p. p. of <ets>corrugare</ets>; <ets>cor-+ rugare</ets> to wrinkle, <ets>ruga</ets> wrinkle; of uncertain origin.]</ety> <def>Wrinkled; crumpled; furrowed; contracted into ridges and furrows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrugate</ent><br/
<hw>Cor"ru*gate</hw> <pr>(-g?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corrugated</conjf> <pr>(-g?`t?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Corrugating</conjf> <pr>(-g?`t?ng)</pr>.]</vmorph> <def>To form or shape into wrinkles or folds, or alternate ridges and grooves, as by drawing, contraction, pressure, bending, or otherwise; to wrinkle; to purse up; <as>as, to <ex>corrugate</ex> plates of iron; to <ex>corrugate</ex> the forehead</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corrugated iron</b></col>, <cd>sheet iron bent into a series of alternate ridges and grooves in parallel lines, giving it greater stiffness.</cd> -- <col><b>Corrugated paper</b></col>, <cd>a thick, coarse paper corrugated in order to give it elasticity. It is used as a wrapping material for fragile articles, as bottles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>corrugated</ent><br/
<hw>corrugated</hw> <pos>adj.</pos> <def>shaped into parallel folds alternately grooved and ridged; <as>as, the surface of the ocean was rippled and <ex>corrugated</ex></as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corrugation</ent><br/
<hw>Cor`ru*ga"tion</hw> <pr>(k?r`r?-g?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>corrugation</ets>.]</ety> <def>The act corrugating; contraction into wrinkles or alternate ridges and grooves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrugator</ent><br/
<hw>Cor"ru*ga`tor</hw> <pr>(k?r"r?-g?`t?r)</pr>, <pos>n.</pos> <ety>[NL.; cf. F. <ets>corrugateur</ets>.]</ety> <fld>(Anat.)</fld> <def>A muscle which contracts the skin of the forehead into wrinkles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrugent</ent><br/
<hw>Cor*ru"gent</hw> <pr>(k?r-r?"j<it>e</it>nt)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Drawing together; contracting; -- said of the corrugator.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrump</ent><br/
<hw>Cor*rump"</hw> <pr>(k?r-r?mp")</pr>, <pos>v. t.</pos> <ety>[L. <ets>corrumpere</ets>.]</ety> <def>To corrupt. See <er>Corrupt</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chauser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrumpable</ent><br/
<hw>Cor*rump"a*ble</hw> <pr>(-?-b'l)</pr>, <pos>a.</pos> <def>Corruptible.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrupt</ent><br/
<hw>Cor*rupt`</hw> <pr>(k?r-r?pt")</pr>, <pos>a.</pos> <ety>[L. <ets>corruptus</ets>, p. p. of <ets>corrumpere</ets> to corrupt; <ets>cor-</ets> + <ets>rumpere</ets> to break. See <er>Rupture</er>.]</ety> <sn>1.</sn> <def>Changed from a sound to a putrid state; spoiled; tainted; vitiated; unsound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who with such <qex>corrupt</qex> and pestilent bread would feed them.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Changed from a state of uprightness, correctness, truth, etc., to a worse state; vitiated; depraved; debased; perverted; <as>as, <ex>corrupt</ex> language; <ex>corrupt</ex> judges</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At what ease<br/
Might <qex>corrupt</qex> minds procure knaves as <qex>corrupt</qex><br/
To swear against you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Abounding in errors; not genuine or correct; <as>as, the text of the manuscript is <ex>corrupt</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrupt</ent><br/
<hw>Cor*rupt"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corrupted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Corrupting</conjf>.]</vmorph> <sn>1.</sn> <def>To change from a sound to a putrid or putrescent state; to make putrid; to putrefy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change from good to bad; to vitiate; to deprave; to pervert; to debase; to defile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Evil communications <qex>corrupt</qex> good manners.</q> <rj><qau>1. Cor. xv. 33.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To draw aside from the path of rectitude and duty; <as>as, to <ex>corrupt</ex> a judge by a bribe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heaven is above all yet; there sits a Judge<br/
That no king can <qex>corrupt</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To debase or render impure by alterations or innovations; to falsify; <as>as, to <ex>corrupt</ex> language; to <ex>corrupt</ex> the sacred text</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that makes an ill use of it [language], though he does not <qex>corrupt</qex> the fountains of knowledge, . . . yet he stops the pines.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To waste, spoil, or consume; to make worthless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lay not up for yourselves treasures upon earth, where moth and rust doth <qex>corrupt</qex>.</q> <rj><qau>Matt. vi. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrupt</ent><br/
<hw>Cor*rupt"</hw> <pr>(k?r-r?pt")</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To become putrid or tainted; to putrefy; to rot.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To become vitiated; to lose purity or goodness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corrupter</ent><br/
<hw>Cor*rupt"er</hw> <pr>(k?r-r?p"t?r)</pr>, <pos>n.</pos> <def>One who corrupts; one who vitiates or taints; <as>as, a <ex>corrupter</ex> of morals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptful</ent><br/
<hw>Cor*rupt"ful</hw> <pr>(-f?l)</pr>, <pos>a.</pos> <def>Tending to corrupt; full of corruption.</def> <mark>[Obs.]</mark> <ldquo/<xex>Corruptful</xex> bribes.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptibility</ent><br/
<hw>Cor*rupt`i*bil"i*ty</hw> <pr>(k?r-r?p`t?-b?l"?-t?)</pr>, <pos>n.</pos> <ety>[L. <ets>corruptibilitas</ets>: cf. F. <ets>corruptibilit<eacute/</ets>.]</ety> <def>The quality of being corruptible; the possibility or liability of being corrupted; corruptibleness.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptible</ent><br/
<hw>Cor*rupt"i*ble</hw> <pr>(k?r-r?p"t?-b'l)</pr>, <pos>a.</pos> <ety>[L.  <ets>corruptibilis</ets>: cf. F. <ets>corruptible</ets>.]</ety> <sn>1.</sn> <def>Capable of being made corrupt; subject to decay.</def> <ldquo/Our <xex>corruptible</xex> bodies.<rdquo/  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye were not redeemed with <qex>corruptible</qex> things, as silver and gold.</q> <rj><qau>1 Pet. i. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being corrupted, or morally vitiated; susceptible of depravation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They systematically corrupt very <qex>corruptible</qex> race.</q> <rj><qau>Burke.</qau></rj></p>

<p>-- <wordforms><wf>Cor*rupt"i*ble*ness</wf>, <pos>n.</pos> -- <wf>Cor*rupt"i*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptible</ent><br/
<hw>Cor*rupt"i*ble</hw>, <pos>n.</pos> <def>That which may decay and perish; the human body.</def> <mark>[Archaic]</mark>  <rj><au>1 Cor. xv. 53.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptingly</ent><br/
<hw>Cor*rupt"ing*ly</hw>, <pos>adv.</pos> <def>In a manner that corrupts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruption</ent><br/
<hw>Cor*rup"tion</hw> <pr>(k?r-r?p"sh?n)</pr>, <pos>n.</pos> <ety>[F. <ets>corruption</ets>, L.  <ets>corruptio</ets>.]</ety> <sn>1.</sn> <def>The act of corrupting or making putrid, or state of being corrupt or putrid; decomposition or disorganization, in the process of putrefaction; putrefaction; deterioration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The inducing and accelerating of putrefaction is a subject of very universal inquiry; for <qex>corruption</qex> is a reciprocal to <ldquo/generation<rdquo/.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The product of corruption; putrid matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of corrupting or of impairing integrity, virtue, or moral principle; the state of being corrupted or debased; loss of purity or integrity; depravity; wickedness; impurity; bribery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was necessary, by exposing the gross <qex>corruptions</qex> of monasteries, . . . to exite popular indignation against them.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They abstained from some of the worst methods of <qex>corruption</qex> usual to their party in its earlier days.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Corruption</xex>, when applied to officers, trustees, etc., signifies the inducing a violation of duty by means of pecuniary considerations.</note>  <rj><au>Abbott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of changing, or of being changed, for the worse; departure from what is pure, simple, or correct; <as>as, a <ex>corruption</ex> of style; <ex>corruption</ex> in language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corruption of blood</b></col> <fld>(Law)</fld>, <cd>taint or impurity of blood, in consequence of an act of attainder of treason or felony, by which a person is disabled from inheriting any estate or from transmitting it to others.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Corruption of blood</qex> can be removed only by act of Parliament.</q> <rj><qau>Blackstone.</qau></rj></p>

<p><syn><b>Syn.</b> -- Putrescence; putrefaction; defilement; contamination; deprivation; debasement; adulteration; depravity; taint. See <er>Depravity</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptionist</ent><br/
<hw>Cor*rup"tion*ist</hw>, <pos>n.</pos> <def>One who corrupts, or who upholds corruption.</def>  <rj><au>Sydney Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptive</ent><br/
<hw>Cor*rupt"ive</hw> <pr>(k?r-r?p"t?v)</pr>, <pos>a.</pos> <ety>[L. <ets>corruptivus</ets>: cf. F. <ets>corruptif</ets>.]</ety> <def>Having the quality of tainting or vitiating; tending to produce corruption.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It should be endued with some <qex>corruptive</qex> quality for so speedy a dissolution of the meat.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptless</ent><br/
<hw>Cor*rupt"less</hw> <pr>(k?r-r?pt"l?s)</pr>, <pos>a.</pos> <def>Not susceptible of corruption or decay; incorruptible.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptly</ent><br/
<hw>Cor*rupt"ly</hw>, <pos>adv.</pos> <def>In a corrupt manner; by means of corruption or corrupting influences; wrongfully.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptness</ent><br/
<hw>Cor*rupt"ness</hw>, <pos>n.</pos> <def>The quality of being corrupt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corruptress</ent><br/
<hw>Cor*rupt"ress</hw> <pr>(-r?s)</pr>, <pos>n.</pos> <def>A woman who corrupts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou studied old <qex>corruptress</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corsac</ent><br/
<hw>Cor"sac</hw> <pr>(k?r"s?k)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The corsak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corsage</ent><br/
<hw>Cor"sage</hw> <pr>(k<ocir/r"s<asl/j)</pr>, <pos>n.</pos> <ety>[F. See <er>Corset</er>.]</ety> <sn>1.</sn> <def>The waist or bodice of a lady's dress; <as>as, a low <ex>corsage</ex></as>.</def></p>

<p><sn>2.</sn> <pr>(k<ocir/r*s<aum/zh")</pr> <def>a flower or small arrangement of flowers worn by a person as a personal ornament.  Typically worn by women on special occasions (as, at a ball or an anniversary celebration), a <ex>corsage</ex> may be worn pinned to the chest, or tied to the wrist. It is usually larger or more elaborate than a <er>boutonniere</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Corsair</ent><br/
<hw>Cor"sair</hw> <pr>(k?r"s?r)</pr>, <pos>n.</pos> <ety>[F. <ets>corsaire</ets> (cf. It. <ets>corsare</ets>, <ets>corsale</ets>, Pr.  <ets>corsari</ets>), LL. <ets>corsarius</ets>, fr. L.  <ets>cursus</ets> a running, course, whence Sp. <ets>corso</ets> cruise, <ets>corsa</ets> cruise, coasting voyage, <ets>corsear</ets> to cruise against the enemy, to pirate, <ets>corsario</ets> cruising, a privateer authorized to cruise against the enemy. See <er>Course</er>.]</ety> <sn>1.</sn> <def>A pirate; one who cruises about without authorization from any government, to seize booty on sea or land.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piratical vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Barbary <qex>corsairs</qex> . . . infested the coast of the Mediterranean.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A Californian market fish (<spn>Sebastichthys rosaceus</spn>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Corsak</ent><br/
<hw>Cor"sak</hw> <pr>(k?r"s?k)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small foxlike mammal (<spn>Cynalopex corsac</spn>), found in Central Asia.</def> <altsp>[Written also <asp>corsac</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corse</ent><br/
<hw>Corse</hw> <pr>(k?rs <or/ k?rs; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>cors</ets>, F. <ets>corps</ets>. See <er>Corpse</er>.]</ety> <sn>1.</sn> <def>A living body or its bulk.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For he was strong, and of so mighty <qex>corse</qex><br/
As ever wielded spear in warlike hand.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A corpse; the dead body of a human being.</def> <mark>[Archaic or Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Set down the <qex>corse</qex>; or, by <person>Saint Paul</person>,<br/
I'll make a <qex>corse</qex> of him that disobeys.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corselet</ent><br/
<hw>Corse"let</hw> <pr>(k?rs"l?t)</pr>, <pos>n.</pos> <ety>[F., dim. of OF. <ets>cors</ets>. F. <ets>corps</ets>, body. See <er>Corse</er>.]</ety> <sn>1.</sn> <def>Armor for the body, as, the body breastplate and backpiece taken together; -- also, used for the entire suit of the day, including breastplate and backpiece, tasset and headpiece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The thorax of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corsepresent</ent><br/
<hw>Corse"pres`ent</hw> <pr>(k?rs"pr?z`<it>e</it>nt <or/ k?rs"-)</pr>, <pos>n.</pos> <fld>(Engl.Law)</fld> <def>An offering made to the church at the interment of a dead body.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corset</ent><br/
<hw>Cor"set</hw> <pr>(k?r"s?t)</pr>, <pos>n.</pos> <ety>[F., dim. of OF. <ets>cors</ets>, F. <ets>corps</ets>, body. See <er>Corse</er>.]</ety> <sn>1.</sn> <def>In the Middle Ages, a gown or basque of which the body was close fitting, worn by both men and women.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 328 --></p>

<p><sn>2.</sn> <def>An article of dress inclosing the chest and waist worn (chiefly by women) to support the body or to modify its shape; stays.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corset</ent><br/
<hw>Cor"set</hw> <pr>(k?r"s?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Corseted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Corseting</conjf>.]</vmorph> <def>To inclose in corsets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corsica</ent><br/
<hw>Corsica</hw> <pos>n.</pos> <sn>1.</sn> <def>an island in the Mediterranean; with adjacent islets it constitutes a region of France.</def><br/
<syn><b>Syn. --</b> Corse.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a region of France.</def><br/
<syn><b>Syn. --</b> Corse.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corsican</ent><br/
<hw>Corsican</hw> <pos>adj.</pos> <def>of or pertaining to Corsica (definition 2).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corslet</ent><br/
<hw>Cors"let</hw> <pr>(k<ocir/rs"l?t)</pr>, <pos>n.</pos> <def>A corselet.</def> <mark>[Obs.]</mark>  <rj><au>Hakluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corsned</ent><br/
<hw>Cors"ned</hw> <pr>(k<ocir/rs"n<ecr/d)</pr>, <pos>n.</pos> <ety>[AS. <ets>corsn<aemac/d</ets>.]</ety> <fld>(AS. Laws)</fld> <def>The morsel of execration; a species of ordeal consisting in the eating of a piece of bread consecrated by imprecation. If the suspected person ate it freely, he was pronounced innocent; but if it stuck in his throat, it was considered as a proof of his guilt.</def>  <rj><au>Burril.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cortege</ent><br/
<ent>cortege</ent><br/
<mhw><hw>cortege</hw>, ||<hw>Cor`t<eacute/ge"</hw></mhw> <pr>(k<ocir/r`t<asl/zh")</pr>, <pos>n.</pos> <ety>[F., fr. It. <ets>corteggio</ets> train, fr. <ets>corte</ets> court. See <er>Court</er>.]</ety> <def>A train of attendants; a group following and attending to some important person.</def><br/
<syn><b>Syn. --</b> retinue, suite, entourage.</syn><br/
[<source>1913 Webster</source> <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a procession, especially a funeral procession following the casket carrying a dead person.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cortes</ent><br/
<hw>Cor"tes</hw> <pr>(k<ocir/r"t<ecr/s)</pr>, <pos>n. pl.</pos> <ety>[Sp. & Pg., fr. <ets>corte</ets> court.]</ety> <def>The legislative assembly, composed of nobility, clergy, and representatives of cities, which in Spain and in Portugal answers, in some measure, to the Parliament of Great Britain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cortes Geraes</ent><br/
||<hw>Cor"tes Ge*ra"es</hw> <pr>(?)</pr>. <ety>[Pg.]</ety> <def>See <er>Legislature</er>, <xex>Portugal</xex>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cortex</ent><br/
||<hw>Cor"tex</hw> <pr>(k<ocir/r"t<ecr/ks)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cortices</plw> <pr>(-t<?/-s<?/z)</pr>.</plu> <ety>[L., bark. Cf. <er>Cork</er>.]</ety> <sn>1.</sn> <def>Bark, as of a tree; hence, an outer covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Bark; rind; specifically, cinchona bark.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>The outer or superficial part of an organ; <as>as, the <ex>cortex</ex> or gray exterior substance of the brain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cortical</ent><br/
<hw>Cor"ti*cal</hw> <pr>(k?r"t?-k<it>a</it>l)</pr>, <pos>a.</pos> <ety>[L. <ets>cortex</ets> bark: cf. F. <ets>cortical</ets>.]</ety> <def>Belonging to, or consisting of, bark or rind; resembling bark or rind; external; outer; superficial; <as>as, the <ex>cortical</ex> substance of the kidney</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corticated</ent><br/
<ent>Corticate</ent><br/
<mhw>{ <hw>Cor"ti*cate</hw> <pr>(k?r"t?-k?t)</pr>, <hw>Cor"ti*ca`ted</hw> <pr>(-k?`t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>corticatus</ets>.]</ety> <def>Having a special outer covering of a nature unlike the interior part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corticifer</ent><br/
<hw>Cor*tic"i*fer</hw> <pr>(k?r-t?s"?-f?r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the <suborder>Gorgoniacea</suborder>; -- so called because the fleshy part surrounds a solid axis, like a bark.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corticiferous</ent><br/
<hw>Cor`ti*cif"er*ous</hw> <pr>(k?r`t?-s?f"?r-?s)</pr>, <pos>a.</pos> <ety>[L. <ets>cortex</ets>, <ets>corticis</ets>, bark -- <ets>-ferous</ets>: cf. F. <ets>corticif<?/re</ets>.]</ety> <sn>1.</sn> <def>Producing bark or something that resembling that resembles bark.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having a barklike c<?/nenchyms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corticiform</ent><br/
<hw>Cor*tic"i*form</hw> <pr>(k?r-t?s"?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cortex</ets>, <ets>corticis</ets>, bark + <ets>-form</ets>: cf. F.  <ets>corticiforme</ets>.]</ety> <def>Resembling, or having the form of, bark or rind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corticine</ent><br/
<hw>Cor"ti*cine</hw> <pr>(k?r"t?-s?n)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>cortex</ets>, <ets>corticis</ets>, bark.]</ety> <def>A material for carpeting or floor covering, made of ground cork and caoutchouc or India rubber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cortico-</ent><br/
<hw>cor"ti*co-</hw> <pr>(?)</pr>, <pos>a.</pos> <def>a combining form signifying <xex>cortex</xex>.</def><br/
[<source>PJC</source>]</p>

<p><ent>corticoafferent</ent><br/
<hw>corticoafferent</hw> <pos>adj.</pos> <fld>(Med.)</fld>  <def>passing toward the cerebral cortex; -- of nerves and nerve impulses.</def><br/
<syn><b>Syn. --</b> corticipetal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corticoefferent</ent><br/
<hw>corticoefferent</hw> <pos>adj.</pos> <fld>(Med.)</fld>  <def>directed from the cerebral cortex; -- of nerves and nerve impulses.</def><br/
<syn><b>Syn. --</b> corticofugal, corticifugal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corticofugal</ent><br/
<hw>corticofugal</hw> <pos>adj.</pos> <def>same as <er>corticoefferent</er>; <as>as, <ex>corticofugal</ex> discharges</as>.</def><br/
<syn><b>Syn. --</b> corticoefferent, corticifugal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cortico-hypothalamic</ent><br/
<hw>cortico-hypothalamic</hw> <pos>adj.</pos> <def>of or relating to the cortex and the hypothalamus.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corticoid</ent><br/
<hw>corticoid</hw> <pos>n.</pos> <def>a steroid compound produced by the adrenal cortex, or a synthetic analog of such a compound.</def><br/
<syn><b>Syn. --</b> corticosteroid.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Corticose</ent><br/
<hw>Cor"ti*cose`</hw> <pr>(-k?s`)</pr>, <pos>a.</pos> <ety>[L.  <ets>corticosus</ets>.]</ety> <def>Abounding in bark; resembling bark; barky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>corticosteroid</ent><br/
<hw>corticosteroid</hw> <pos>n.</pos> <def>a steroid compound produced by the adrenal cortex, or a synthetic analog of such a compound.</def><br/
<syn><b>Syn. --</b> corticoid.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corticosterone</ent><br/
<hw>corticosterone</hw> <pos>n.</pos> <def>any of several steroid compounds secreted by the adrenal cortex; they are involved in regulating water and electrolyte balance in the body.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corticotropin</ent><br/
<ent>corticotrophin</ent><br/
<mhw><hw>corticotrophin</hw>, <hw>corticotropin</hw></mhw> <pos>n.</pos> <def>a growth hormone produced by the anterior pituitary gland; it stimulates the adrenal cortex; -- called also <altname>adrenocorticotropin</altname> and abbreviated <altname>ACTH</altname>.</def><br/
<syn><b>Syn. --</b> adrenocorticotropic hormone, adrenocorticotrophic hormone, adrenocorticotropin, adrenocorticotropin, corticotrophin, ACTH.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corticous</ent><br/
<hw>Cor"ti*cous</hw> <pr>(-k?s)</pr>, <pos>a.</pos> <def>Relating to, or resembling, bark; corticose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cortile</ent><br/
<hw>Cor"tile</hw> <pr>(k?r"t?l; It. k?r-t?"l?)</pr>, <pos>n.</pos> <ety>[It., fr. <ets>corte</ets> court.]</ety> <def>An open internal courtyard inclosed by the walls of a large dwelling house or other large and stately building.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cortina</ent><br/
<hw>cor*ti"na</hw> <pr>(k<ocir/r*t<imac/"n<adot/ <it>or</it> k<ocir/r*t<emac/"n<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>cortinae</plw> <pr>(k<ocir/r*t<emac/"n<emac/)</pr>.</plu> <fld>(Biology)</fld> <def>a cobwebby remnant of the partial veil which in some mature mushrooms hang from the edges of the cap.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cortinaria</ent><br/
<hw>Cortinaria</hw> <pos>n.</pos> <def>a large genus of rusty-spored agarics having prominent cortinae.</def><br/
<syn><b>Syn. --</b> genus <gen>Cortinaria</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cortisol</ent><br/
<hw>cortisol</hw> <pos>n.</pos> <def>a steroid hormone (<chform>C21H30O5</chform>) active in carbohydrate and protein metabolism; -- called also <altname>hydrocortisone</altname>.  It is used in medicine as an antiinflammatory agent</def><br/
<syn><b>Syn. --</b> hydrocortisone.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cortisone</ent><br/
<hw>cortisone</hw> <pos>n.</pos> <fld>(Chem.)</fld> <def>a corticosteroid hormone (<chform>C21H28O5</chform>) produced by the adrenal cortex.  It is used in medicine as an antiinflammatory agent.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corundom</ent><br/
<hw>corundom</hw> <pos>n.</pos> <def>a very hard mineral used as an abrasive; same as <er>corundun</er>.</def><br/
<syn><b>Syn. --</b> corundum.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corundum</ent><br/
<hw>Co*run"dum</hw> <pr>(k<osl/*r<ucr/n"d<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Corundums</plw> <pr>(k<osl/*r<ucr/n"d<ucr/mz)</pr>.</plu> [Also <ets>corindon</ets>.] <ety>[From Hind. <ets>kurand</ets> corundum stone.]</ety> <fld>(Min.)</fld> <def>The mineral alumina (<chform>Al2O3</chform>), as found native in a crystalline state.  Transparent varieties are used as gemstones, including <stype>sapphire</stype>, which is the fine blue variety; the <stype>oriental ruby</stype>, or red sapphire; the <stype>oriental amethyst</stype>, or purple sapphire; and <stype>adamantine spar</stype>, the hair-brown variety. It is the hardest substance found native, next to the diamond.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ The name <ex>corundum</ex> is sometimes restricted to the non-transparent or coarser kinds. <stype>Emery</stype> is a dark-colored granular variety, usually admixed with magnetic iron ore.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coruscant</ent><br/
<hw>Co*rus"cant</hw> <pr>(k?-r?s"k<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[L. <ets>coruscans</ets>, p. pr. See <er>Coruscate</er>.]</ety> <def>Glittering in flashes; flashing.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coruscate</ent><br/
<hw>Cor"us*cate</hw> <pr>(k?r"?s-k?t <or/ k?-r?s"k?r)</pr>, <pos>v. i.</pos> <ety>[L. <ets>coruscare</ets> to flash, vibrate.]</ety> <def>To glitter in flashes; to flash.</def></p>

<p><syn><b>Syn.</b> -- To glisten; gleam; sparkle; radiate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coruscation</ent><br/
<hw>Cor`us*ca"tion</hw> <pr>(k?r`?s-k?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>coruscatio</ets>: cf. F. <ets>coruscattion</ets>.]</ety> <sn>1.</sn> <def>A sudden flash or play of light.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A very vivid but exceeding short-lived splender, not to call <?/t a little <qex>coruscation</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flash of intellectual brilliancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He might have illuminated his times with the incessant <qex>cor<?/<?/cations</qex> of his genius.</q> <rj><qau>I. Taylor.</qau></rj></p>

<p><syn><b>Syn.</b> -- Flash; glitter; blaze; gleam; sparkle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corve</ent><br/
<hw>Corve</hw> <pr>(k?rv)</pr>, <pos>n.</pos> <def>See <er>Corf</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corvee</ent><br/
||<hw>Cor`vee"</hw> <pr>(k?r`v" <or/ -v?")</pr>, <pos>n.</pos> <ety>[F. <ets>corv<eacute/e</ets>, fr. LL. <ets>corvada</ets>, <ets>corrogata</ets>, fr. L.  <ets>corrogare</ets> to entreat together; <ets>cor-</ets> + <ets>rogare</ets> to ask.]</ety> <fld>(Feudal Law)</fld> <def>An obligation to perform certain services, as the repair of roads, for the lord or sovereign.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corven</ent><br/
<hw>Cor"ven</hw> <pr>(k?r"v<it>e</it>n)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Carve</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corvette</ent><br/
<ent>Corvet</ent><br/
<mhw>{ <hw>Cor"vet</hw> <pr>(k?r"v?t)</pr>, <hw>Cor*vette"</hw> <pr>(k?r-v?r")</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>corvette</ets>, fr. Pg. <ets>corveta</ets> or Sp. <ets>corbeta</ets>, fr. L.  <ets>corbita</ets> a slow-sailing ship of burden, fr, <ets>corbis</ets> basket. Cf. <er>Corbeil</er>.]</ety> <fld>(Naut.)</fld> <def>A war vessel, ranking next below a frigate, and having usually only one tier of guns; -- called in the United States navy a <altname>sloop of war</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corvetto</ent><br/
||<hw>Cor*vet"to</hw> <pr>(-v?t"t?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A curvet.</def>  <rj><au>Peacham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corvine</ent><br/
<hw>Cor"vine</hw> <pr>(k?r"v?n)</pr>, <pos>a.</pos> <ety>[L. <ets>corvinus</ets>, fr. <ets>corvus</ets> crow.]</ety> <def>Of or pertaining to the crow; crowlike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corvorant</ent><br/
<hw>Cor"vo*rant</hw> <pr>(k?r"v?-r<it>a</it>nt)</pr>, <pos>n.</pos> <def>See <er>Cormorant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coryanthes</ent><br/
<hw>Coryanthes</hw> <pos>n.</pos> <def>a small genus of tropical American epiphytic or lithophytic orchids.</def><br/
<syn><b>Syn. --</b> genus <gen>Coryanthes</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corybant</ent><br/
<hw>Cor"y*bant</hw> <pr>(k?r"?-b?nt)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Corybants</plw> (-b<?/nts), oftener L. <plw>Corybantes</plw> <pr>(-b<?/n"t<?/z)</pr>.</plu> <ety>[L. <ets>Corybas</ets>, Gr. <grk>Kory`bas</grk>.]</ety> <def>One of the priests of Cybele in Phrygia. The rites of the Corybants were accompanied by wild music, dancing, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corybantiasm</ent><br/
<hw>Cor`y*ban"ti*asm</hw> <pr>(-b?n"t?-?z'm)</pr>, <pos>n.</pos> <ety>[Gr. <grk>Korybantiasmo`s</grk> a corybantic frenzy.]</ety> <fld>(Med.)</fld> <def>A kind of frenzy in which the patient is tormented by fantastic visions and want of sleep.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>corybantic</ent><br/
<hw>cor`y*ban"tic</hw> <pr>(k?r`?-b?n"t?k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>Korybantiko`s</grk>, fr. <grk>Kory`bas</grk> a Corybant.]</ety> <def>Of, pertaining to, or resembling, the Corybantes or their rites; frantic; frenzied; <as>as, a <ex>corybantic</ex> dance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corydalidae</ent><br/
<hw>Corydalidae</hw> <pos>prop. n.</pos> <def>A natural family of insects including the dobsonflies.</def><br/
<syn><b>Syn. --</b> family <fam>Corydalidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>corydalis</ent><br/
<hw>corydalis</hw> <pos>n.</pos> <def>a plant of the genus <gen>Corydalis</gen>, with beautiful compound foliage and spurred tubular flowers.</def><-- of the poppy family or fumitory family?? dictionaries disagree! --><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Corydalus</ent><br/
<hw>Corydalus</hw> <pos>prop. n.</pos> <def>The type genus of the <fam>Corydalidae</fam>.  It includes the dobsonfly (<spn>Corydalus cornutus</spn>), whose aquatic larva, the <stype>hellgrammite</stype>, is used as bait in fishing.</def><br/
<syn><b>Syn. --</b> genus <gen>Corydalus</gen>, genus Corydalis.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Corylaceae</ent><br/
<hw>Corylaceae</hw> <pos>prop. n.</pos> <def>A natural family of shrubs or small trees, used in some classification systems for the genus <gen>Corylus</gen>.</def><br/
<syn><b>Syn. --</b> subfamily Corylaceae, family <fam>Corylaceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corylopsis</ent><br/
<hw>Corylopsis</hw> <pos>n.</pos> <def>small genus of deciduous shrubs of temperate regions of Asia.</def><br/
<syn><b>Syn. --</b> genus <gen>Corylopsis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corylus</ent><br/
<hw>Corylus</hw> <pos>n.</pos> <def>a genus of deciduous monoecious nut-bearing shrubs or small trees: hazel; sometimes placed in the subfamily or family <fam>Corylaceae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Corylus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Corymb</ent><br/
<hw>Cor"ymb</hw> <pr>(k?r"?mb <or/ -?m; 220)</pr>, <pos>n.</pos> <ety>[L. <ets>corymbus</ets> cluster of flowers, Gr. <grk>ko`rymbos</grk>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A flat-topped or convex cluster of flowers, each on its own footstalk, and arising from different points of a common axis, the outermost blossoms expanding first, as in the hawthorn.</def> <sd>(b)</sd> <def>Any flattish flower cluster, whatever be the order of blooming, or a similar shaped cluster of fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corymbed</ent><br/
<hw>Cor"ymbed</hw> <pr>(k?r"?mbd)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Corymbose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corymbiferous</ent><br/
<hw>Cor`ym*bif"er*ous</hw> <pr>(k?r`?m-b?f"?r-?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>corymbifer</ets>; <ets>corymbus</ets> a cluster of flowers + <ets>ferre</ets> to bear; cf. F. <ets>corimbif<egrave/re</ets>.]</ety> <fld>(Bot.)</fld> <def>Bearing corymbs of flowers or fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corymbose</ent><br/
<hw>Co*rym"bose</hw> <pr>(k?-r?m"b?s <or/ k?r"?m-b?s`)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Consisting of corymbs, or resembling them in form.</def> <altsp>[Written also <asp>corymbous</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corymbosely</ent><br/
<hw>Co*rym"bose*ly</hw>, <pos>adv.</pos> <def>In corymbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>corynebacterium</ent><br/
<hw>co`ry*ne`bac*ter"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(MIcrobiol.)</fld> <def>Any member of the genus <gen>Corynebacterium</gen>, consisting of gram-positive usually nonmotile irregularly rod-shaped bacteria, some of which are pathogenic or parasitic in humans and domestic animals, such as <spn>Corynebacterium diphtheriae</spn>, which causes diphtheria.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coryphaenoid</ent><br/
<hw>Cor`y*ph<ae/"noid</hw> <pr>(k<ocr/r`<icr/*f<emac/"noid)</pr>, <pos>a.</pos> <ety>[NL.  <ets>coryphaena</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Belonging to, or like, the genus <gen>Coryph<ae/na</gen>. See <er>Dolphin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coryphee</ent><br/
||<hw>Co`ry`ph<eacute/e"</hw> <pr>(k<osl/`r<esl/`f<asl/")</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Drama)</fld> <def>A ballet dancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coryphene</ent><br/
<hw>Cor"y*phene`</hw> <pr>(k?r"?-f?n`)</pr>, <pos>n.</pos> <ety>[NL. <ets>coryphena</ets>, fr. Gr. <grk>koryfh`</grk> head, summit, peak: cf. F. <ets>coryph<egrave/ne</ets>.]</ety> <fld>(Zool.)</fld> <def>A fish of the genus <gen>Coryph<ae/na</gen>. See <er>Dolphin</er>. (2)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Corypheus</ent><br/
<hw>Cor`y*phe"us</hw> <pr>(k?r`?-f?"?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Corypheuses</plw> (-<ecr/z), L. <plw>Coryphei</plw> <pr>(-f<?/"<?/)</pr>.</plu> <ety>[L. <ets>coryphaeus</ets>, fr. Gr. <?/<?/<?/, fr. <grk>koryfh`</grk> head.]</ety> <fld>(Gr. Antiq.)</fld> <def>The conductor, chief, or leader of the dramatic chorus; hence, the chief or leader of a party or interest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That noted <qex>corypheus</qex> [Dr. John Owen] of the Independent faction.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coryphodon</ent><br/
||<hw>Co*ryph"o*don</hw> <pr>(k?-r?f"?-d?n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>koryfh`</grk> head, peak + <grk>'odoy`s</grk>, <grk>'odo`ntos</grk>, tooth.]</ety> <fld>(Paleon.)</fld> <def>A genus of extinct mammals from the eocene tertiary of Europe and America. Its species varied in size between the tapir and rhinoceros, and were allied to those animals, but had short, plantigrade, five-toed feet, like the elephant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coryphodont</ent><br/
<hw>Co*ryph"o*dont</hw> <pr>(-d?nt)</pr>, <pos>a.</pos> <fld>(Paleon.)</fld> <def>Pertaining to, or resembling, the genus <gen>Coryphodon</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coryza</ent><br/
||<hw>Co*ry"za</hw> <pr>(k?-r?"z?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ko`ryza</grk> catarh.]</ety> <fld>(Med.)</fld> <def>Nasal catarrh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coscinomancy</ent><br/
<hw>Cos*cin"o*man`cy</hw> <pr>(k?s-s?n"?-m?n`s? <or/ k?s"s?-n?-)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`skinon</grk> sieve + <ets>-mancy</ets>.]</ety> <def>Divination by means of a suspended sieve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coscoroba</ent><br/
||<hw>Cos`co*ro"ba</hw> <pr>(k?s`k?-r?"b?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>A large, white, South American duck, of the genus <gen>Cascoroba</gen>, resembling a swan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosecant</ent><br/
<hw>Co*se"cant</hw> <pr>(k?-s?"k?nt)</pr>, <pos>n.</pos> <ety>[For <ets>co. secans</ets>, an abbrev. of L. <ets>complementi secans</ets>.]</ety> <fld>(Trig.)</fld> <def>The secant of the complement of an arc or angle. See <xex>Illust.</xex> of <er>Functions</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosen</ent><br/
<hw>Cos"en</hw> <pr>(k?z"'n)</pr>, <pos>v. t.</pos> <def>See <er>Cozen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosenage</ent><br/
<hw>Cos"en*age</hw> <pr>(k?z"'n-?j)</pr>, <pos>n.</pos> <def>See <er>Cozenage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosening</ent><br/
<hw>Cos"en*ing</hw>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>Anything done deceitfully, and which could not be properly designated by any special name, whether belonging to contracts or not.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosentient</ent><br/
<hw>Co*sen"tient</hw> <pr>(k?-s?n"sh<it>e</it>nt)</pr>, <pos>a.</pos> <def>Perceiving together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosey</ent><br/
<hw>Co"sey</hw> <pr>(k?"z?)</pr>, <pos>a.</pos> <def>See <er>Cozy</er>.</def>  <rj><au> Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cosh</ent><br/
<hw>cosh</hw> <pos>n.</pos> <def>a piece of leather-covered metal with a flexible handle; used for hitting people.</def> <mark>[slang]</mark> <br/
<syn><b>Syn. --</b> blackjack, sap.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cosh</ent><br/
<hw>cosh</hw> <pos>v.</pos> <sn>1.</sn> <def>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cosh</ent><br/
<hw>cosh</hw> <pos>v. t.</pos> <def>to hit (someone) with a cosh or similar bludgeon, usually on the head.</def> <mark>[slang]</mark> <br/
[<source>WordNet 1.5</source> + <source>PJC</source>]</p>

<p><ent>Cosher</ent><br/
<hw>Cosh"er</hw> <pr>(k?sh"?r)</pr>, <pos>v. t.</pos> <ety>[Ir. <ets>cosair</ets> a feast, a banquet? or cf. F. <ets>coucher</ets> to lie. Cf. <er>Couch</er>, <er>Coshering</er>.]</ety> <sn>1.</sn> <fld>(Old Law)</fld> <def>To levy certain exactions or tribute upon; to lodge and eat at the expense of. See <er>Coshering</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat with hospitality; to pet.</def> <mark>[Ireland]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosherer</ent><br/
<hw>Cosh"er*er</hw> <pr>(k?sh"?r-?r)</pr>, <pos>n.</pos> <def>One who coshers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coshering</ent><br/
<hw>Cosh"er*ing</hw>, <pos>n.</pos> <fld>(Old Law)</fld> <def>A feudal prerogative of the lord of the soil entitling him to lodging and food at his tenant's house.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sometimes he contrived, in deflance of the law, to live by <qex>coshering</qex>, that is to say, by quartering himself on the old tentants of his family, who, wretched as was their own condition, could not refuse a portion of their pittance to one whom they still regarded as their rightful lord.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosier</ent><br/
<hw>Co"sier</hw> <pr>(k?"zh?r)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>coussier</ets> maker of mattresses; or <ets>couseor</ets> tailor, fr. OF. & F. <ets>coudre</ets>, p. p. <ets>cousu</ets> to sew, fr. L. <ets>consuere</ets> to sew together; <ets>con-</ets> + <ets>seure</ets> to sew. See <er>Sew</er> to stitch.]</ety> <def>A tailor who botches his work.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosignificative</ent><br/
<hw>Co`sig*nif"i*ca*tive</hw> <pr>(<it>k<?/</it>`<it>s<?/g-n<?/f</it>"<it><?/-k<?/-t<?/v</it>)</pr>, <pos>a.</pos> <def>Having the same signification.</def>  <rj><au>Cockerham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosignitary</ent><br/
<hw>Co*sig"ni*ta*ry</hw> <pr>(k?-s?g"n?-t?-r?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>co-</ets> + <ets>sign</ets>. Cf. <er>Signatory</er>.]</ety> <def>Signing some important public document with another or with others; <as>as, a treaty violated by one of the <ex>cosignitary</ex> powers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosignitary</ent><br/
<hw>Co*sig"ni*ta*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cosignitaries</plw> <pr>(-r<?/z)</pr>.</plu> <def>One who signs a treaty or public document along with others or another; <as>as, the <ex>cosignitaries</ex> of the treaty of Berlin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosily</ent><br/
<hw>Co"si*ly</hw> <pr>(k?"z?-l?)</pr>, <pos>adv.</pos> <def>See <er>Cozily</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosinage</ent><br/
<hw>Cos"in*age</hw> <pr>(k?s"'n-?j)</pr>, <pos>n.</pos> <ety>[See <er>Cousinage</er>.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>Collateral relationship or kindred by blood; consanguinity.</def> <au>Burrill.</au> <sd>(b)</sd> <def>A writ to recover possession of an estate in lands, when a stranger has entered, after the death of the grandfather's grandfather, or other distant collateral relation.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosine</ent><br/
<hw>Co"sine</hw> <pr>(k?"s?n)</pr>, <pos>n.</pos> <ety>[For <ets>co</ets>. <ets>sinus</ets>, an abbrev. of L. <ets>complementi sinus</ets>.]</ety> <fld>(Trig.)</fld> <def>The sine of the complement of an arc or angle. See <xex>Illust.</xex> of <er>Functions</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmetical</ent><br/
<ent>Cosmetic</ent><br/
<mhw>{ <hw>Cos*met"ic</hw> <pr>(k?z-m?t"?k)</pr>, <hw>Cos*met"ic*al</hw> <pr>(-?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>kosmitiko`s</grk> skilled in decorating, fr. <grk>ko`smos</grk> order, ornament: cf. F. <ets>cosm<eacute/tique</ets>. See <er>Cosmos</er>.]</ety> <def>Imparting or improving beauty, particularly the beauty of the complexion; <as>as, a <ex>cosmetical</ex> preparation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>First, robed in white, the nymph intent adores,<br/
With head uncovered, the <qex>cosmetic</qex> powers.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmetic</ent><br/
<hw>Cos*met"ic</hw>, <pos>n.</pos> <def>Any external application intended to beautify and improve the complexion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cosmetologist</ent><br/
<hw>cosmetologist</hw> <pos>n.</pos> <def>an expert in the use of cosmetics.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cosmical</ent><br/
<ent>Cosmic</ent><br/
<mhw>{ <hw>Cos"mic</hw> <pr>(k?z"m?k)</pr>, <hw>Cos"mic*al</hw> <pr>(-m?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>kosmiko`s</grk> of the world, fr. <grk>ko`smos</grk>: cf. F.  <ets>cosmique</ets>. See <er>Cosmos</er>.]</ety> <sn>1.</sn> <def>Pertaining to the universe, and having special reference to universal law or order, or to the one grand harmonious system of things; hence; harmonious; orderly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to the solar system as a whole, and not to the earth alone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Characteristic of the cosmos or universe; inconceivably great; vast; <as>as, <ex>cosmic</ex> speed</as>.</def> <ldquo/<xex>Cosmic</xex> ranges of time.<rdquo/  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Astron.)</fld> <def>Rising or setting with the sun; -- the opposite of <ant>acronycal</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>of unsurpassed size, extent, or significance; vast; <as>as, of <ex>cosmic</ex> proportions; of <ex>cosmic</ex> importance</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cosmically</ent><br/
<hw>Cos"mic*al*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>With the sun at rising or setting; <as>as, a star is said to rise or set <ex>cosmically</ex> when it rises or sets with the sun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Universally.</def> <mark>[R.]</mark>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmogonical</ent><br/
<ent>Cosmogonic</ent><br/
<ent>Cosmogonal</ent><br/
<mhw>{ <hw>Cos*mog"o*nal</hw> <pr>(k?z-m?g"?-n<it>a</it>l)</pr>, <hw>Cos`mo*gon"ic</hw> <pr>(k?z`m?-g?n"?k)</pr>, <hw>Cos`mo*gon"ic*al</hw> <pr>(-g?n"?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <def>Belonging to cosmogony.</def>  <rj><au>B. Powell. Gladstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmogonist</ent><br/
<hw>Cos*mog"o*nist</hw> <pr>(k?z-m?g"?-n?st)</pr>, <pos>n.</pos> <def>One who treats of the origin of the universe; one versed in cosmogony.</def><-- cosmologist --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmogony</ent><br/
<hw>Cos*mog"o*ny</hw> <pr>(-n?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cosmogonies</plw> <pr>(-n<?/z)</pr>.</plu> <ety>[Gr. <grk>kosmogoni`a</grk>; <grk>ko`smos</grk> the world + root of <grk>gi`gnesthai</grk> to be born: cf. F. <ets>cosmogonie</ets>.]</ety> <def>The creation of the world or universe; a theory or account of such creation; <as>as, the poetical <ex>cosmogony</ex> of Hesoid; the <xex>cosmogonies</xex> of Thales, Anaxagoras, and Plato</as>.</def><-- =cosmology --><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cosmogony</qex> or creation of the world has puzzled philosophers of all ages.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmographer</ent><br/
<hw>Cos*mog"ra*pher</hw> <pr>(-r?-f?r)</pr>, <pos>n.</pos> <def>One who describes the world or universe, including the heavens and the earth.</def><-- =cosmologist --><br/
[<source>1913 Webster</source>]</p>

<p><q>The name of this island is nowhere found among the old and ancient <qex>cosmographers</qex>.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmographical</ent><br/
<ent>Cosmographic</ent><br/
<mhw>{ <hw>Cos`mo*graph"ic</hw> <pr>(k?z`m?-gr?f"?k)</pr>, <hw>Cos`mo*graph"ic*al</hw> <pr>(-?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>cosmographique</ets>.]</ety> <def>Of or pertaining to cosmography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmographically</ent><br/
<hw>Cos`mo*graph"ic*al*ly</hw>, <pos>adv.</pos> <def>In a cosmographic manner; in accordance with cosmography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmography</ent><br/
<hw>Cos*mog"ra*phy</hw> <pr>(k?z-m?g"r?-f?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cosmographies</plw> <pr>(-f<?/z)</pr>.</plu> <ety>[Gr. <grk>kosmografi`a</grk>; <grk>ko`smos</grk> the world + <grk>gra`fein</grk> to write: cf. F. <ets>cosmographie</ets>.]</ety> <def>A description of the world or of the universe; or the science which teaches the constitution of the whole system of worlds, or the figure, disposition, and relation of all its parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmolabe</ent><br/
<hw>Cos"mo*labe</hw> <pr>(k?z"m?-l?b)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + <grk>lamba`nein</grk> to take: cf. F. <ets>cosmolade</ets>.]</ety> <def>An instrument resembling the astrolabe, formerly used for measuring the angles between heavenly bodies; -- called also <altname>pantacosm</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmolatry</ent><br/
<hw>Cos*mol"a*try</hw> <pr>(k?z-m?l"?-tr?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + <grk>latre`yein</grk> to worship.]</ety> <def>Worship paid to the world.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmoline</ent><br/
<hw>Cos"mo*line</hw> <pr>(k?z"m?-l?n)</pr>, <pos>n.</pos> <ety>[Prob. fr. <ets>cosmetic</ets> + L. <ets>ole</ets>um oil.]</ety> <fld>(Chem.)</fld> <def>A substance obtained from the residues of the distillation of petroleum, essentially the same as <xex>vaseline</xex>, but of somewhat stiffer consistency, and consisting of a mixture of the higher paraffines; a kind of petroleum jelly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmological</ent><br/
<hw>Cos`mo*log"ic*al</hw> <pr>(k?z`m?-l?j"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Of or pertaining to cosmology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmologist</ent><br/
<hw>Cos*mol"o*gist</hw> <pr>(k?z-m?l"?-j?st)</pr>, <pos>n.</pos> <def>One who describes the universe; one skilled in cosmology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cosmology</ent><br/
<hw>cos*mol"o*gy</hw> <pr>(k<ocr/z*m<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + <ets>-logy</ets>: cf. F. <ets>cosmologie</ets>.]</ety> <sn>1.</sn> <def>The branch of science or philosophy dealing with the origin and nature of the universe as a whole.</def> <specif>specifically</specif> <sd>(a)</sd> <fld>(Philosophy)</fld> <def>the branch of metaphysics speculating on the structure and nature of the most fundamental parts of the system of creation, such as space and time, the elements of bodies, the structure of the universe, the modifications of material things, causality, the laws of motion, and the order and course of nature.</def>  <sd>(b)</sd> <fld>(Astronomy)</fld> <def>the branch of astronomy dealing with the origin and structure of the universe, including the evolution of its present observable structure, using the methods of observational astronomy as well as mathematical physics.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a treatise dealing with the original and structure of the universe.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cosmometry</ent><br/
<hw>Cos*mom"e*try</hw> <pr>(k?z-m?m"?-tr?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + <ets>-metry</ets>.]</ety> <def>The art of measuring the world or the universe.</def>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cosmonaut</ent><br/
<hw>cosmonaut</hw> <pos>n.</pos> <ety>[Russ. <ets>kosmonavt</ets> from <ets>kosmos</ets> universe.]</ety> <def>an <er>astronaut</er>; -- a term used by the Soviets and Russians.</def><br/
<syn><b>Syn. --</b> astronaut, spaceman.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Cosmoplastic</ent><br/
<hw>Cos`mo*plas"tic</hw> <pr>(k?z`m?-pl?s"t?k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + <grk>pla`ssein</grk> to form.]</ety> <def>Pertaining to a plastic force as operative in the formation of the world independently of God; world-forming.</def> <ldquo/<xex>Cosmoplastic</xex> and hylozoic atheisms.<rdquo/  <rj><au>Gudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmopolite</ent><br/
<ent>Cosmopolitan</ent><br/
<mhw>{ <hw>Cos`mo*pol"i*tan</hw> <pr>(-p?l"?-t<it>a</it>n)</pr>, <hw>Cos*mop"o*lite</hw> <pr>(k?z-m?p"?-l?t)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <grk>kosmopoli`ths</grk>; <grk>ko`smos</grk> the world + <grk>poli`ths</grk> citizen, <grk>po`lis</grk> city: cf. F. <ets>cosmopolitain</ets>, <ets>cosmopolite</ets>.]</ety> <def>One who has no fixed residence, or who is at home in every place; a citizen of the world.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmopolite</ent><br/
<ent>Cosmopolitan</ent><br/
<mhw>{ <hw>Cos`mo*pol"i*tan</hw>, <hw>Cos*mop"o*lite</hw>, }</mhw> <pos>a.</pos> <sn>1.</sn> <def>Having no fixed residence; at home in any place; free from local attachments or prejudices; not provincial; liberal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In other countries taste is perphaps too exclusively national, in Germany it is certainly too <qex>cosmopolite</qex>.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Common everywhere; widely spread; found in all parts of the world.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Cheiroptera are <qex>cosmopolitan</qex>.</q> <rj><qau>R. Owen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmopolitanism</ent><br/
<hw>Cos`mo*pol"i*tan*ism</hw> <pr>(k?z`m?-p?l"?-t<it>a</it>n-?z'm)</pr>, <pos>n.</pos> <def>The quality of being cosmopolitan; cosmopolitism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmopolite</ent><br/
<hw>Cos*mop"o*lite</hw> <pr>(-m?p"?-l?t)</pr>, <pos>a. & n.</pos> <def>See <er>Cosmopolitan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmopolitical</ent><br/
<hw>Cos`mo*po*lit"ic*al</hw> <pr>(k?z`m?-p?-l?t"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Having the character of a cosmopolite.</def> <mark>[R.]</mark>  <rj><au>Hackluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmopolitism</ent><br/
<hw>Cos*mop"o*li*tism</hw> <pr>(k?z-m?p"?-l?-t?z'm)</pr>, <pos>n.</pos> <def>The condition or character of a cosmopolite; disregard of national or local peculiarities and prejudices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmorama</ent><br/
<hw>Cos`mo*ra"ma</hw> <pr>(k?z`m?-r?"m? <or/ -r?"m?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ko`smos</grk> the world + <grk>"o`rama</grk> a sight, spectacle, fr. <grk>"ora^n</grk> to see.]</ety> <def>An exhibition in which a series of views in various parts of the world is seen reflected by mirrors through a series of lenses, with such illumination, etc., as will make the views most closely represent reality.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 329 --></p>

<p><ent>Cosmoramic</ent><br/
<hw>Cos`mo*ram"ic</hw> <pr>(k<ocr/z`m<osl/*r<acr/m"<icr/k)</pr>, <pos>a.</pos> <def>Of or pertaining to a cosmorama.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmos</ent><br/
||<hw>Cos"mos</hw> <pr>(k<ocr/z"m<ocr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ko`smos</grk> order, harmony, the world (from its perfect order and arrangement); akin to Skr. <ets><cced/ad</ets> to distinguish one's self.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The universe or universality of created things; -- so called from the order and harmony displayed in it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The theory or description of the universe, as a system displaying order and harmony.</def>  <rj><au>Humboldt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmos</ent><br/
||<hw>Cos"mos</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A genus of composite plants closely related to <gen>Bidens</gen>, usually with very showy flowers, some with yellow, others with red, scarlet, purple, white, or lilac rays.  They are natives of the warmer parts of America, and many species are cultivated.  <spn>Cosmos bipinnatus</spn> and <spn>Cosmos diversifolius</spn> are among the best-known species; <spn>Cosmos caudatus</spn>, of the West Indies, is widely naturalized.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cosmosphere</ent><br/
<hw>Cos"mo*sphere</hw> <pr>(k<ocr/z"m?-sf?r)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + E. <ets>sphere</ets>.]</ety> <def>An apparatus for showing the position of the earth, at any given time, with respect to the fixed stars. It consist of a hollow glass globe, on which are depicted the stars and constellations, and within which is a terrestrial globe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmotheism</ent><br/
<hw>Cos"mo*the`ism</hw> <pr>(k?z"m?-th?`?z'm)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ko`smos</grk> the world + <?/<?/<?/ god.]</ety> <def>Same as <er>Pantheism</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosmothetic</ent><br/
<hw>Cos`mo*thet"ic</hw> <pr>(k?z`m?-th?t"?k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ko`smos</grk> universe + <?/<?/<?/ to place or arrange.]</ety> <fld>(Metaph.)</fld> <def>Assuming or positing the actual existence or reality of the physical or external world.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cosmothetic idealists</b></col> <fld>(Metaph.)</fld>, <cd>those who assume, without attempting to prove, the reality of external objects as corresponding to, and being the ground of, the ideas of which only the mind has direct cognizance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cosmothetic idealists</qex> . . . deny that mind is immediately conscious of matter.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosovereign</ent><br/
<hw>Co*sov"er*eign</hw> <pr>(k?-s?v"?r-?n <or/ k?-s?v"-)</pr>, <pos>n.</pos> <def>A joint sovereign.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coss</ent><br/
<hw>Coss</hw> <pr>(k<ocr/s)</pr>, <pos>n.</pos> <ety>[Cf. Pers. <ets>k<omac/s</ets> a road measure of about two miles; or Skr. <ets>kr<omac/<cced/a</ets>.]</ety> <def>A Hindoo measure of distance, varying from one and a half to two English miles.</def>  <rj><au>Whitworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coss</ent><br/
<hw>Coss</hw>, <pos>n.</pos> <ety>[It. <ets>cosa</ets>.]</ety> <def>A thing (only in phrase below).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Rule of Coss</b></col>, <cd>an old name for <xex>Algebra</xex>. <ety>[It. <xex>regola di cosa</xex> rule of thing, the unknown quantity being called the <xex>cosa</xex>, or the thing.]</ety></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cossack</ent><br/
<hw>Cos"sack</hw> <pr>(k?s"s?k)</pr>, <pos>n.</pos> <ety>[Russ. <ets>kozak'</ets>, <ets>kazak'</ets>: cf. Turk. <ets>kaz<amac/k</ets>.]</ety> <def>One of a warlike, pastoral people, skillful as horsemen, inhabiting different parts of the Russian empire and furnishing valuable contingents of irregular cavalry to its armies, those of Little Russia and those of the Don forming the principal divisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cossack post</ent><br/
<hw>Cos"sack post</hw>. <fld>(Mil.)</fld> <def>An outpost consisting of four men, forming one of a single line of posts substituted for the more formal line of sentinels and line of pickets.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cossas</ent><br/
<hw>Cos"sas</hw> <pr>(k?s"s?s)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Plain India muslin, of various qualities and widths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosset</ent><br/
<hw>Cos"set</hw> <pr>(k?s"s?t)</pr>, <pos>n.</pos> <ety>[Cf. AS.  <ets>cotsetla</ets> cottager, G. <ets>kossat</ets>, <ets>kothsasse</ets>, fr. <ets>kot</ets>, <ets>koth</ets> E. (<ets>cot</ets>) hut, and cf. also E. <ets>cade</ets>, <ets>a.</ets>, <ets>cot</ets> a cade lamb.]</ety> <def>A lamb reared without the aid of the dam. Hence: A pet, in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosset</ent><br/
<hw>Cos"set</hw>, <pos>v. t.</pos> <def>To treat as a pet; to fondle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She was <qex>cosseted</qex> and posseted and prayed over and made much of.</q> <rj><qau>O. W. Holmes.</qau></rj></p>

<p><ent>Cossette</ent><br/
||<hw>Cos*sette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>One of the small chips or slices into which beets are cut in sugar making.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cossical</ent><br/
<ent>Cossic</ent><br/
<mhw>{ <hw>Cos"sic</hw> <pr>(k?s"s?k)</pr>, <hw>Cos"sic*al</hw> <pr>(-s?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[It. <ets>cossico</ets>. See 2d <er>Coss</er>.]</ety> <def>Of or relating to algebra; <as>as, <ex>cossic</ex> numbers, or the <ex>cossic</ex> art</as>.</def> <mark>[Obs.]</mark> <ldquo/Art of numbers <xex>cossical</xex>.<rdquo/  <rj><au>Digges (1579).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cost</ent><br/
<hw>Cost</hw> <pr>(k<ocr/st; 115)</pr>, <pos>n.</pos> <ety>[L. <ets>costa</ets> rib. See <er>Coast</er>.]</ety> <sn>1.</sn> <def>A rib; a side; a region or coast.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Betwixt the <qex>costs</qex> of a ship.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>See <er>Cottise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cost</ent><br/
<hw>Cost</hw> <pr>(k<ocr/st; 115)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cost</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Costing</conjf>.]</vmorph> <ety>[OF. <ets>coster</ets>, <ets>couster</ets>, F.  <ets>co<ucir/ter</ets>, fr. L. <ets>constare</ets> to stand at, to cost; <ets>con-</ets> + <ets>stare</ets> to stand. See <er>Stand</er>, and cf. <er>Constant</er>.]</ety> <sn>1.</sn> <def>To require to be given, expended, or laid out therefor, as in barter, purchase, acquisition, etc.; to cause the cost, expenditure, relinquishment, or loss of; <as>as, the ticket <ex>cost</ex> a dollar; the effort <ex>cost</ex> his life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A diamond gone, <qex>cost</qex> me two thousand ducats.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Though it <qex>cost</qex> me ten nights' watchings.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To require to be borne or suffered; to cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To do him wanton rites, which <qex>cost</qex> them woe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cost dear</b></col>, <cd>to require or occasion a large outlay of money, or much labor, self-denial, suffering, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cost</ent><br/
<hw>Cost</hw>, <pos>n.</pos> <ety>[OF. <ets>cost</ets>, F. <ets>co<ucir/t</ets>. See <er>Cost</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>The amount paid, charged, or engaged to be paid, for anything bought or taken in barter; charge; expense; hence, whatever, as labor, self-denial, suffering, etc., is requisite to secure benefit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One day shall crown the alliance on 't so please you,<br/
Here at my house, and at my proper <qex>cost</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At less <qex>cost</qex> of life than is often expended in a skirmish, [Charles V.] saved Europe from invasion.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Loss of any kind; detriment; pain; suffering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I know thy trains,<br/
Though dearly to my <qex>cost</qex>, thy gins and toils.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Law)</fld> <def>Expenses incurred in litigation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Costs</xex> in actions or suits are either between attorney and client, being what are payable in every case to the attorney or counsel by his client whether he ultimately succeed or not, or between party and party, being those which the law gives, or the court in its discretion decrees, to the prevailing, against the losing, party.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bill of costs</b></col>. <cd>See under <er>Bill</er>.</cd> -- <col><b>Cost free</b></col>, <cd>without outlay or expense.</cd> <ldquo/Her duties being to talk French, and her privileges to live <xex>cost free</xex> and to gather scraps of knowledge.<rdquo/  <rj><au>Thackeray.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costa</ent><br/
||<hw>Cos"ta</hw> <pr>(k<ocr/s"t<adot/)</pr>, <pos>n.</pos> <ety>[L., rib. See <er>Coast</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A rib of an animal or a human being.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A rib or vein of a leaf, especially the midrib.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The anterior rib in the wing of an insect.</def> <sd>(b)</sd> <def>One of the riblike longitudinal ridges on the exterior of many corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costage</ent><br/
<hw>Cost"age</hw> <pr>(k?st"?j; 115)</pr>, <pos>n.</pos> <ety>[OF. <ets>coustage</ets>.]</ety> <def>Expense; cost.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costal</ent><br/
<hw>Cos"tal</hw> <pr>(k?s"t<it>a</it>l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>costal</ets>. See <er>Costa</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>Pertaining to the ribs or the sides of the body; <as>as, <ex>costal</ex> nerves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot. & Zool.)</fld> <def>Relating to a costa, or rib.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Costal cartilage</b></col>. <cd>See <er>Cartilage</er>, and <xex>Illust.</xex> of <er>Thorax</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costal-nerved</ent><br/
<hw>Cos"tal-nerved`</hw> <pr>(k?s"t<it>a</it>l-n?rvd`)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the nerves spring from the midrib.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>co-star</ent><br/
<hw>co-star</hw> <pos>v. t.</pos> <def>to feature as the co-star in a performance; <as>as, the film <ex>co-starred</ex> Robin Williams as the psychiatrist</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>co-star</ent><br/
<hw>co-star</hw> <pos>v. i.</pos> <def>to be the co-star in a performance; <as>as, Robin Williams co-starred in the film</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Costard</ent><br/
<hw>Cos"tard</hw> <pr>(k?s"t?rd)</pr>, <pos>n.</pos> <ety>[Prob. fr. OF. <ets>coste</ets> rib, side, F. <ets>c<ocir/te</ets>, and meaning orig., a ribbed apple, from the ribs or angles on its sides. See <er>Coast</er>.]</ety> <sn>1.</sn> <def>An apple, large and round like the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some [apples] consist more of air than water . . . ; others more of water than wind, as your <qex>costards</qex> and pomewaters.</q> <rj><qau>Muffett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The head; -- used contemptuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Try whether your <qex>costard</qex> or my bat be the harder.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costardmonger</ent><br/
<hw>Cos"tard*mon`ger</hw> <pr>(-m?n`g?r)</pr>, <pos>n.</pos> <def>A costermonger.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costated</ent><br/
<ent>Costate</ent><br/
<mhw>{ <hw>Cos"tate</hw> <pr>(k?s"t?t)</pr>, <hw>Cos"ta*ted</hw> <pr>(-t?-t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>costatus</ets>, fr. <ets>costa</ets> rib.]</ety> <def>Having ribs, or the appearance of ribs; <fld>(Bot.)</fld> having one or more longitudinal ribs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costean</ent><br/
<hw>Cos"tean`</hw> <pr>(k?s"t?n`)</pr>, <pos>v. i.</pos> <ety>[Cornish <ets>cothas</ets> dropped + <ets>stean</ets> tin.]</ety> <def>To search after lodes. See <er>Costeaning</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costeaning</ent><br/
<hw>Cos"tean`ing</hw>, <pos>n.</pos> <def>The process by which miners seek to discover metallic lodes. It consist in sinking small pits through the superficial deposits to the solid rock, and then driving from one pit to another across the direction of the vein, in such manner as to cross all the veins between the two pits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costellate</ent><br/
<hw>Cos*tel"late</hw> <pr>(k?s-t?l"l?t)</pr>, <pos>a.</pos> <ety>[L.  <ets>costa</ets> rib.]</ety> <def>Finely ribbed or costated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coster</ent><br/
<hw>Cos"ter</hw> <pr>(k?s"t?r)</pr>, <pos>n.</pos> <ety>[Abbrev. of <ets>costermonger</ets>.]</ety> <def>One who hawks about fruit, green vegetables, fish, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costermonger</ent><br/
<hw>Cos"ter*mon`ger</hw> <pr>(k?s"t?r-m?n`g?r)</pr>, <pos>n.</pos> <ety>[See <er>Costard</er>.]</ety> <def>An apple seller; a hawker of, or dealer in, any kind of fruit or vegetables; a fruiterer.</def> <altsp>[Written also <asp>costardmonger</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costiferous</ent><br/
<hw>Cos*tif"er*ous</hw> <pr>(k?s-t?f"?r-?s)</pr>, <pos>a.</pos> <ety>[<ets>Costa</ets> + <ets>-ferous</ets>.]</ety> <fld>(Anat.)</fld> <def>Rib-bearing, as the dorsal vertebr<ae/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>costing</ent><br/
<hw>costing</hw> <pos>n.</pos> <def>cost accounting.</def> <mark>[British]</mark> <br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Costive</ent><br/
<hw>Cos"tive</hw> <pr>(k?s"t?v)</pr>, <pos>a.</pos> <ety>[OF. <ets>costev<eacute/</ets>, p. p. of <ets>costever</ets>, F.  <ets>constiper</ets>, L. <ets>constipare</ets> to press closely together, to cram; <ets>con-</ets> + <ets>stipare</ets> to press together, cram. See <er>Stipulate</er>, <er>Stiff</er>, and cf. <er>Constipate</er>.]</ety> <sn>1.</sn> <def>Retaining fecal matter in the bowels; having too slow a motion of the bowels; constipated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reserved; formal; close; cold.</def> <mark>[Obs.]</mark> <ldquo/A <xex>costive</xex> brain.<rdquo/ <au>Prior.</au> <ldquo/<xex>Costive</xex> of laughter.<rdquo/ <au>B. Jonson.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>You must be frank, but without indiscretion; and close, but without being <qex>costive</qex>.</q> <rj><qau>Lord Chesterfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dry and hard; impermeable; unyielding.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Clay in dry seasons is <qex>costive</qex>, hardening with the sun and wind.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costively</ent><br/
<hw>Cos"tive*ly</hw>, <pos>adv.</pos> <def>In a costive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costiveness</ent><br/
<hw>Cos"tive*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>An unnatural retention of the fecal matter of the bowels; constipation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inability to express one's self; stiffness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A reverend disputant of the same <qex>costiveness</qex> in public elocution with myself.</q> <rj><qau>Wakefield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costless</ent><br/
<hw>Cost"less</hw> <pr>(k?st"l?s; 115)</pr>, <pos>a.</pos> <def>Costing nothing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costlewe</ent><br/
<hw>Cost"lewe</hw> <pr>(-l?)</pr>, <pos>a.</pos> <def>Costly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costliness</ent><br/
<hw>Cost"li*ness</hw> <pr>(-l?-n?s)</pr>, <pos>n.</pos> <def>The quality of being costy; expensiveness; sumptuousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costly</ent><br/
<hw>Cost"ly</hw> <pr>(k?st"l?; 115)</pr>, <pos>a.</pos> <ety>[From Cost expense.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Of great cost; expensive; dear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had fitted up his palace in the most <qex>costly</qex> and sumptuous style, for the accomodation of the princess.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gorgeous; sumptuous.</def> <mark>[Poetic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To show how <qex>costly</qex> summer was at hand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costmary</ent><br/
<hw>Cost"ma*ry</hw> <pr>(k?st"m?-r?)</pr>, <pos>n.</pos> <ety>[L. <ets>costum</ets> an Oriental aromatic plant (Gr. <?/<?/<?/, cf. Ar. <ets>kost</ets>, <ets>kust</ets>) + <ets>Maria</ets> Mary. Cf. <er>Alecost</er>.]</ety> <fld>(Bot.)</fld> <def>A garden plant (<spn>Chrysanthemum Balsamita</spn>) having a strong balsamic smell, and nearly allied to tansy. It is used as a pot herb and salad plant and in flavoring ale and beer. Called also <altname>alecost</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coston lights</ent><br/
<hw>Cos"ton lights</hw> <pr>(?)</pr>. <def>Signals made by burning lights of different colors and used by vessels at sea, and in the life-saving service; -- named after their inventor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Costotome</ent><br/
<hw>Cos"to*tome</hw> <pr>(k?s"t?-t?m)</pr>, <pos>n.</pos> <ety>[<ets>Costa</ets> + Gr. <?/<?/<?/<?/ to cut.]</ety> <def>An instrument (chisel or shears) to cut the ribs and open the thoracic cavity, in post-mortem examinations and dissections.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cost-plus</ent><br/
<hw>cost-plus</hw> <pos>adj.</pos> <def>determining payment based on the actual cost of production plus an agreed-upon fee or rate of profit; <as>as, a <ex>cost-plus</ex> government contract</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Costrel</ent><br/
<hw>Cos"trel</hw> <pr>(k?s"tr?l)</pr>, <pos>n.</pos> <ety>[CF. W. <ets>costrel</ets>, OF.  <ets>costrel</ets>, LL.  <ets>costrellum</ets>, a liquid measure, <ets>costrellus</ets> a wine cup.]</ety> <def>A bottle of leather, earthenware, or wood, having ears by which it was suspended at the side.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A youth, that, following with a <qex>costrel</qex>, bore<br/
The means of goodly welcome, flesh and wine.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costume</ent><br/
<hw>Cos"tume`</hw> <pr>(k?s"t?m` <or/ k?s-t?m")</pr>, <pos>n.</pos> <ety>[F. <ets>costume</ets>, It. <ets>costume</ets> custom, dress, fr. L. <ets>consuetumen</ets> (not found), for <ets>consuetudo</ets> custom. See <er>Custom</er>, and cf. <er>Consuetude</er>.]</ety> <sn>1.</sn> <def>Dress in general; esp., the distinctive style of dress of a people, class, or period.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Such an arrangement of accessories, as in a picture, statue, poem, or play, as is appropriate to the time, place, or other circumstances represented or described.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I began last night to read Walter Scott's Lay of the Last Minstrel . . . .I was extremely delighted with the poetical beauty of some parts . . . .The <qex>costume</qex>, too, is admirable.</q> <rj><qau>Sir J. Mackintosh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A character dress, used at fancy balls or for dramatic purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Costumer</ent><br/
<hw>Cos"tum`er</hw> <pr>(-t?m`?r)</pr>, <pos>n.</pos> <def>One who makes or deals in costumes, as for theaters, fancy balls, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>costusroot</ent><br/
<hw>costusroot</hw> <pos>n.</pos> <def>an annual herb (<spn>Saussurea costus</spn>) of the eastern Himalayas (Kashmir) having purple florets and a fragrant root that yields a volatile oil used in perfumery and for preserving furs.</def><br/
<syn><b>Syn. --</b> <spn>Saussurea costus</spn>, Saussurea lappa.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Co-sufferer</ent><br/
<hw>Co-suf"fer*er</hw> <pr>(k?-s?f"f?r-?r)</pr>, <pos>n.</pos> <def>One who suffers with another.</def>  <rj><au>Wycherley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosupreme</ent><br/
<hw>Co`su*preme"</hw> <pr>(k?`s?-pr?m")</pr>, <pos>n.</pos> <def>A partaker of supremacy; one jointly supreme.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosurety</ent><br/
<hw>Co*sure"ty</hw> <pr>(k?-sh?r"t?; 136)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cosureties</plw> <pr>(-t<?/z)</pr>.</plu> <def>One who is surety with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cosy</ent><br/
<hw>Co"sy</hw> <pr>(k?"z?)</pr>, <pos>a.</pos> <def>See <er>Cozy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cot</ent><br/
<hw>Cot</hw> <pr>(k<ocr/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>cot</ets>, <ets>cote</ets>, AS. <ets>cot</ets>, <ets>cote</ets>, cottage; akin to D. & Icel. <ets>kot</ets>, G.  <ets>koth</ets>, <ets>kot</ets>, <ets>kothe</ets>. Cf. <er>Coat</er>.]</ety> <sn>1.</sn> <def>A small house; a cottage or hut.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sheltered <qex>cot</qex>, the cultivated farm.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pen, coop, or like shelter for small domestic animals, as for sheep or pigeons; a cote.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A cover or sheath; <as>as, a roller <ex>cot</ex> (the clothing of a drawing roller in a spinning frame); a <ex>cot</ex> for a sore finger</as>.  See also <er>finger cot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Cf. Ir. <ets>cot</ets>.]</ety> <def>A small, rudely-formed boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bell cot</b></col>. <fld>(Arch.)</fld> <cd>See under <er>Bell</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cot</ent><br/
<hw>Cot</hw> <pr>(k<ocr/t)</pr>, <pos>n.</pos> <ety>[AS. <ets>cot</ets> cottage, bedchamber; or cf. OF. <ets>coite</ets>, F. <ets>couette</ets> (E. <ets>quilt</ets>), LL. <ets>cottum</ets>, <ets>cottus</ets>, mattress. See <er>Cot</er> a cottage.]</ety> <def>A sleeping place of limited size; a little bed; a cradle; a piece of canvas extended by a frame, used as a bed.</def> <altsp>[Written also <asp>cott</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotangent</ent><br/
<hw>Co*tan"gent</hw> <pr>(k?-t?n"j<it>e</it>nt)</pr>, <pos>n.</pos> <ety>[For <ets>co</ets>. <ets>tangens</ets>, an abbrev. of L. <ets>complementi tangens</ets>. See <er>Tangent</er>.]</ety> <fld>(Trig.)</fld> <def>The tangent of the complement of an arc or angle. See <xex>Illust.</xex> of <er>Functions</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotarnine</ent><br/
<hw>Co*tar"nine</hw> <pr>(k?-t?r"n?n <or/ -n?n)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>narcotine</ets>, by transposition of letters.]</ety> <fld>(Chem.)</fld> <def>A white, crystalline substance, <chform>C12H13NO3</chform>, obtained as a product of the decomposition of narcotine. It has weak basic properties, and is usually regarded as an alkaloid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cote</ent><br/
<hw>Cote</hw> <pr>(k<omac/t)</pr>, <pos>n.</pos> <ety>[See 1st <er>Cot</er>.]</ety> <sn>1.</sn> <def>A cottage or hut.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A shed, shelter, or inclosure for small domestic animals, as for sheep or doves.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Watching where shepherds pen their flocks, at eve,<br/
In hurdled <qex>cotes</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cote</ent><br/
<hw>Cote</hw>, <pos>v. t.</pos> <ety>[Prob. from F. <ets>c<ocir/t<eacute/</ets> side, OF. <ets>costet</ets>, LL. <ets>costatus</ets>, <ets>costatum</ets>, fr. L. <ets>costu</ets> rib, side: cf. F. <ets>c<ocir/toyer</ets> to go or keep at the side of. See <er>Coast</er>.]</ety> <def>To go side by side with; hence, to pass by; to outrun and get before; <as>as, a dog <ex>cotes</ex> a hare</as>.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>coted</qex> them on the way, and hither are they coming.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cote</ent><br/
<hw>Cote</hw>, <pos>v. t.</pos> <ety>[See <er>Quote</er>.]</ety> <def>To quote.</def> <mark>[Obs.]</mark>  <rj><au>Udall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coteau</ent><br/
||<hw>Co`teau"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Coteaux</plw> <pr>(#)</pr></plu>. <ety>[F., a hill.]</ety> <mark>[Canada & U. S.]</mark>  <sn>1.</sn> <def>A hilly upland including the divide between two valleys; a divide.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The side of a valley.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cotemporaneous</ent><br/
<hw>Co*tem`po*ra"ne*ous</hw> <pr>(k?-t?m`p?-r?"n?-?s)</pr>, <pos>a.</pos> <ety>[See <er>Contemporaneous</er>.]</ety> <def>Living or being at the same time; contemporaneous.</def> -- <wordforms><wf>Co*tem`po*ra"ne*ous*ly</wf>, <pos>adv.</pos> -- <wf>Co*tem`po*ra"ne*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotemporary</ent><br/
<hw>Co*tem"po*ra*ry</hw> <pr>(k?-t?m"p?-r?-r?)</pr>, <pos>a.</pos> <def>Living or being at the same time; contemporary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotemporary</ent><br/
<hw>Co*tem"po*ra*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cotemporaries</plw> <pr>(-r<icr/z)</pr>.</plu> <def>One who lives at the same time with another; a contemporary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotenant</ent><br/
<hw>Co*ten"ant</hw> <pr>(k?-t?n"<it>a</it>nt)</pr>, <pos>n.</pos> <def>A tenant in common, or a joint tenant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coterie</ent><br/
||<hw>Co`te*rie"</hw> <pr>(k?`t<it>e</it>-r?"; 277)</pr>, <pos>n.</pos> <ety>[F., prob. from OF. <ets>coterie</ets> servile tenure, fr. <ets>colier</ets> cotter; of German origin. See 1st <er>Cot</er>.]</ety> <def>A set or circle of persons who meet familiarly, as for social, literary, or other purposes; a clique.</def> <ldquo/The queen of your <xex>coterie</xex>.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coterminous</ent><br/
<hw>Co*ter"mi*nous</hw> <pr>(k?-t?r"m?-n?s)</pr>, <pos>a.</pos> <ety>[Cf.  <er>Conterminous</er>.]</ety> <def>Bordering; conterminous; -- followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotgare</ent><br/
<hw>Cot"gare`</hw> <pr>(k?t"g?r`)</pr>, <pos>n.</pos> <def>Refuse wool.</def> <mark>[Obs. or Prov.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cothurn</ent><br/
<hw>Co"thurn</hw> <pr>(k?"th?rn)</pr>, <pos>n.</pos> <ety>[L. <ets>cothurnus</ets>, Gr. <?/<?/<?/<?/. Cf.  <er>Cothurnus</er>.]</ety> <def>A buskin anciently used by tragic actors on the stage; hence, tragedy in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The moment had arrived when it was thought that the mask and the <qex>cothurn</qex> might be assumed with effect.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cothurnated</ent><br/
<ent>Cothurnate</ent><br/
<mhw>{ <hw>Co*thur"nate</hw> <pr>(k?-th?r"n?t)</pr>, <hw>Co*thur"na*ted</hw> <pr>(-n?-t?d)</pr>, }</mhw> <pos>a.</pos> <sn>1.</sn> <def>Wearing a cothurn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to tragedy; solemn; grave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cothurnus</ent><br/
||<hw>Co*thur"nus</hw> <pr>(-n?s)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Same as <er>Cothurn</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coticular</ent><br/
<hw>Co*tic"u*lar</hw> <pr>(k?-t?k"?-l?r)</pr>, <pos>a.</pos> <ety>[L. <ets>coticula</ets> a small touchstone, dim. <ets>cos</ets>, <ets>cotis</ets>, whetstone.]</ety> <def>Pertaining to whetstones; like or suitable for whetstones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotidal</ent><br/
<hw>Co*tid"al</hw> <pr>(k?-t?d"<it>a</it>l)</pr>, <pos>a.</pos> <def>Marking an equality in the tides; having high tide at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cotidal lines</b></col> <fld>(Phys. Geog.)</fld>, <cd>lines on a map passing through places that have high tide at the same time.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotillion</ent><br/
<ent>Cotillon</ent><br/
<mhw>{ ||<hw>Co`til`lon"</hw> <pr>(k<osl/`t<esl/`y<ocir/N" <it>or</it> k<osl/`t<esl/l`-; 277)</pr>, <hw>Co*til"lion</hw> <pr>(k<osl/*t<icr/l"y<ucr/n)</pr>, }</mhw> <pos>n.</pos> <ety>[F.  <ets>cotillon</ets>, fr. OF. <ets>cote</ets> coat, LL. <ets>cotta</ets> tunic. See <er>Coat</er>.]</ety> <sn>1.</sn> <def>A brisk dance, performed by eight persons; a quadrille.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tune which regulates the dance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of woolen material for women's skirts.</def></p>

<p><sn>4.</sn> <def>A formal ball, especially one at which debutantes are first presented to society.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cotinga</ent><br/
||<hw>Co*tin"ga</hw> <pr>(k<osl/*t<esl/<nsm/"g<adot/)</pr>, <pos>n.</pos> <ety>[Native South American name.]</ety> <fld>(Zool.)</fld> <def>A bird of the family <fam>Cotingid<ae/</fam>, including numerous bright-colored South American species; -- called also <altname>chatterers</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotise</ent><br/
<hw>Cot"ise</hw> <pr>(k<ocr/t"<icr/s)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>See <er>Cottise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotised</ent><br/
<hw>Cot"ised</hw> <pr>(-?st)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>See <er>Cottised</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotland</ent><br/
<hw>Cot"land</hw> <pr>(k?t"l?nd)</pr>, <pos>n.</pos> <def>Land appendant to a cot or cottage, or held by a cottager or cotter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotquean</ent><br/
<hw>Cot"quean`</hw> <pr>(k?t"kw?n`)</pr>, <pos>n.</pos> <ety>[<ets>Cot</ets> a cottage + <ets>quean</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A man who busies himself with affairs which properly belong to women.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A she-cuckold; a cucquean; a henhussy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What, shall a husband be afraid of his wife's face?<br/
We are a king, <qex>cotquean</qex>, and we will reign in our pleasures.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotqueanity</ent><br/
<hw>Cot*quean"i*ty</hw> <pr>(k?t-kw?n"?-t?)</pr>, <pos>n.</pos> <def>The condition, character, or conduct of a cotquean.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotrustee</ent><br/
<hw>Co`trus*tee"</hw> <pr>(k?`tr?s-t?")</pr>, <pos>n.</pos> <def>A joint trustee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotswold</ent><br/
<hw>Cots"wold`</hw> <pr>(k?ts"w?ld`)</pr>, <pos>n.</pos> <ety>[<ets>Cot</ets> a cottage or hut + <ets>wold</ets> an open country.]</ety> <def>An open country abounding in sheepcotes, as in the Cotswold hills, in Gloucestershire, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cotswold sheep</b></col>, <cd>a long-wooled breed of sheep, formerly common in the counties of Gloucester, Hereford, and Worcester, Eng.; -- so called from the <ets>Cotswold</ets> Hills. The breed is now chiefly amalgamated with others.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotta</ent><br/
<hw>Cot"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. See <er>Coat</er>.]</ety> <sn>1.</sn>  <fld>(Eccl.)</fld> <def>A surplice, in England and America usually one shorter and less full than the ordinary surplice and with short sleeves, or sometimes none.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of very coarse woolen blanket.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cottage</ent><br/
<hw>Cot"tage</hw> <pr>(k?t"t?j; 48)</pr>, <pos>n.</pos> <ety>[From <er>Cot</er> a cottage.]</ety> <def>A small house; a cot; a hut.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term was formerly limited to a habitation for the poor, but is now applied to any small tasteful dwelling; and at places of summer resort, to any residence or lodging house of rustic architecture, irrespective of size.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cottage allotment</b></col>. <cd>See under <er>Alloment</er>.</cd> <mark>[Eng.]</mark> -- <col><b>Cottage cheese</b></col>, <cd>the thick part of clabbered milk strained, salted, and pressed into a ball.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottaged</ent><br/
<hw>Cot"taged</hw> <pr>(-t?jd)</pr>, <pos>a.</pos> <def>Set or covered with cottages.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even humble Harting's <qex>cottaged</qex> vale.</q> <rj><qau>Collins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottagely</ent><br/
<hw>Cot"tage*ly</hw> <pr>(-t?j-l?)</pr>, <pos>a.</pos> <def>Cottagelike; suitable for a cottage; rustic.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottager</ent><br/
<hw>Cot"ta*ger</hw> <pr>(k?t"t?-j?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who lives in a cottage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>One who lives on the common, without paying any rent, or having land of his own.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottar</ent><br/
<ent>Cotter</ent><br/
<mhw>{ <hw>Cot"ter</hw>, <hw>Cot"tar</hw>  }</mhw> <pr>(k?t"t?r)</pr>, <pos>n.</pos> <ety>[LL. <ets>cotarius</ets>, <ets>cottarius</ets>, <ets>coterius</ets>. See <er>Cot</er>.]</ety> <def>A cottager; a cottier.</def>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Through Sandwich Notch the West Wind sang<br/
Good morrow to the <qex>cotter</qex>.</q> <rj><qau>Whittier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 330 --></p>

<p><ent>Cotter</ent><br/
<hw>Cot"ter</hw> <pr>(k<ocr/t"t<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A piece of wood or metal, commonly wedge-shaped, used for fastening together parts of a machine or structure. It is driven into an opening through one or all of the parts. <note>[See <xex>Illust.</xex>]</note> In the United States a cotter is commonly called a <altname>key</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A toggle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotter</ent><br/
<hw>Cot"ter</hw>, <pos>v. t.</pos> <def>To fasten with a cotter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottidae</ent><br/
<hw>Cottidae</hw> <pos>n.</pos> <def>a natural family of fish including the sculpins.</def><br/
<syn><b>Syn. --</b> family <fam>Cottidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cottier</ent><br/
<hw>Cot"ti*er</hw> <pr>(-t<icr/*<etil/r)</pr>, <pos>n.</pos> <ety>[OF. <ets>cotier</ets>. See <er>Coterie</er>, and cf. <er>Cotter</er>.]</ety> <def>In Great Britain and Ireland, a person who hires a small cottage, with or without a plot of land. Cottiers commonly aid in the work of the landlord's farm.</def> <altsp>[Written also <asp>cottar</asp> and <asp>cotter</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottise</ent><br/
<hw>Cot"tise</hw> <pr>(k<ocr/t"t<icr/s)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>c<ocir/t<eacute/</ets> side, L. <ets>costa</ets> rib.]</ety> <fld>(Her.)</fld> <def>A diminutive of the bendlet, containing one half its area or one quarter the area of the bend. When a single cottise is used alone it is often called a <stype>cost</stype>. See also <er>Couple-close</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottised</ent><br/
<hw>Cot"tised</hw> <pr>(-t?st)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Set between two cottises, -- said of a bend; or between two barrulets, -- said of a bar or fess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottoid</ent><br/
<hw>Cot"toid</hw> <pr>(k?t"toid)</pr>, <pos>a.</pos> <ety>[NL. <ets>cottus</ets> sculpin + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like a fish of the genus <gen>Cottus</gen>.</def> -- <def2><pos>n.</pos>  <def>A fish belonging to, or resembling, the genus <gen>Cottus</gen>. See <er>Sculpin</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottolene</ent><br/
<hw>Cot"to*lene`</hw> <pr>(k<ocr/t"t<osl/*l<emac/n`)</pr>, <pos>n.</pos> <def>A product from cotton-seed, used as lard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotton</ent><br/
<hw>Cot"ton</hw> <pr>(k<ocr/t"t'n)</pr>, <pos>n.</pos> <ety>[F. <ets>coton</ets>, Sp. <ets>algodon</ets> the cotton plant and its wool, <ets>coton</ets> printed cotton, cloth, fr. Ar. <ets>qutun</ets>, <ets>alqutun</ets>, cotton wool. Cf. <er>Acton</er>, <er>Hacqueton</er>.]</ety> <sn>1.</sn> <def>A soft, downy substance, resembling fine wool, consisting of the unicellular twisted hairs which grow on the seeds of the cotton plant. Long-staple cotton has a fiber sometimes almost two inches long; short-staple, from two thirds of an inch to an inch and a half.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cotton plant. See <cref>Cotten plant</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Cloth made of cotton.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Cotton</xex> is used as an adjective before many nouns in a sense which commonly needs no explanation; as, <xex>cotton</xex> bagging; <xex>cotton</xex> cloth; <xex>cotton</xex> goods; <xex>cotton</xex> industry; <xex>cotton</xex> mill; <xex>cotton</xex> spinning; <xex>cotton</xex> tick.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cotton cambric</b></col>. <cd>See <er>Cambric</er>, <pos>n.</pos>, 2.</cd> -- <col><b>Cotton flannel</b></col>, <cd>the manufactures' name for a heavy cotton fabric, twilled, and with a long plush nap. In England it is called <altname>swan's-down cotton</altname>, or <altname>Canton flannel</altname>.</cd> -- <col><b>Cotton gin</b></col>, <cd>a machine to separate the seeds from cotton, invented by <person>Eli Whitney</person>.</cd> -- <col><b>Cotton grass</b></col> <fld>(Bot.)</fld>, <cd>a genus of plants (<gen>Eriphorum</gen>) of the Sedge family, having delicate capillary bristles surrounding the fruit (seedlike achenia), which elongate at maturity and resemble tufts of cotton.</cd> -- <col><b>Cotton mouse</b></col> <fld>(Zool.)</fld>, <cd>a field mouse (<spn>Hesperomys gossypinus</spn>), injurious to cotton crops.</cd> -- <col><b>Cotton plant</b></col> <fld>(Bot.)</fld>, <cd>a plant of the genus <gen>Gossypium</gen>, of several species, all growing in warm climates, and bearing the cotton of commerce.  The common species, originally Asiatic, is <spn>Gossypium herbaceum</spn>.</cd> -- <col><b>Cotton press</b></col>, <cd>a building and machinery in which cotton bales are compressed into smaller bulk for shipment; a press for baling cotton.</cd> -- <col><b>Cotton rose</b></col> <fld>(Bot.)</fld>, <cd>a genus of composite herbs (<gen>Filago</gen>), covered with a white substance resembling cotton.</cd> -- <col><b>Cotton scale</b></col> <fld>(Zool.)</fld>, <cd>a species of bark louse (<spn>Pulvinaria innumerabilis</spn>), which does great damage to the cotton plant.</cd> -- <col><b>Cotton shrub</b></col>. <cd>Same as <xex>Cotton plant</xex>.</cd> -- <col><b>Cotton stainer</b></col> <fld>(Zool.)</fld>, <cd>a species of hemipterous insect (<spn>Dysdercus suturellus</spn>), which seriously damages growing cotton by staining it; -- called also <altname>redbug</altname>.</cd> -- <col><b>Cotton thistle</b></col> <fld>(Bot.)</fld>, <cd>the Scotch thistle. See under <er>Thistle</er>.</cd> -- <col><b>Cotton velvet</b></col>, <cd>velvet in which the warp and woof are both of cotton, and the pile is of silk; also, velvet made wholly of cotton.</cd> -- <col><b>Cotton waste</b></col>, <cd>the refuse of cotton mills.</cd> -- <col><b>Cotton wool</b></col>, <cd>cotton in its raw or woolly state.</cd> -- <col><b>Cotton worm</b></col> <fld>(Zool.)</fld>, <cd>a lepidopterous insect (<spn>Aletia argillacea</spn>), which in the larval state does great damage to the cotton plant by eating the leaves. It also feeds on corn, etc., and hence is often called <altname>corn worm</altname>, and <altname>Southern army worm</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotton</ent><br/
<hw>Cot"ton</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To rise with a regular nap, as cloth does.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>cottons</qex> well; it can not choose but bear<br/
A pretty nap.</q> <rj><qau>Family of Love.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To go on prosperously; to succeed.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>New, Hephestion, does not this matter <qex>cotton</qex> as I would?</q> <rj><qau>Lyly.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite; to agree; to make friends; -- usually followed by <xex>with</xex>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A quarrel will end in one of you being turned off, in which case it will not be easy to <qex>cotton</qex> with another.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Didst see, Frank, how the old goldsmith <qex>cottoned</qex> in with his beggarly companion?</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To take a liking to; to stick to one as cotton; -- used with <xex>to</xex>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottonade</ent><br/
<hw>Cot"ton*ade`</hw> <pr>(k?t"t'n-?d`)</pr>, <pos>n.</pos> <ety>[F. <ets>cottonade</ets>.]</ety> <def>A somewhat stout and thick fabric of cotton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottonary</ent><br/
<hw>Cot"ton*a*ry</hw> <pr>(-?-r?)</pr>, <pos>a.</pos> <def>Relating to, or composed of, cotton; cottony.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cottonary</qex> and woolly pillows.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotton batting</ent><br/
<hw>Cot"ton bat"ting</hw>. <def>Cotton prepared in sheets or rolls for quilting, upholstering, and similar purposes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cottonmouth</ent><br/
<hw>cottonmouth</hw> <pos>n.</pos> <def>venomous semiaquatic snake (<spn>Agkistrodon piscivorus</spn>) of swamps in southern U.S.; -- called also <altname>water moccasin</altname>, <altname>cottonmouth moccasin</altname>, and <altname>cottonmouth water moccasin</altname>.  It grows to a length of about 4 feet.  The name refers to the whiteness of the lips and inside of the mouth.</def><br/
<syn><b>Syn. --</b> water moccasin, cottonmouth moccasin, <spn>Agkistrodon piscivorus</spn>.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cottonous</ent><br/
<hw>Cot"ton*ous</hw> <pr>(-?s)</pr>, <pos>a.</pos> <def>Resembling cotton.</def> <mark>[R.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cottonseed</ent><br/
<ent>cotton seed</ent><br/
<mhw>{ <hw>cotton seed</hw>, <it>or</it> , <it>usually collectively</it>, <hw>cot"ton*seed`</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>The seed of the cotton plant.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cottonseed meal</ent><br/
<hw>Cottonseed meal</hw>. <def>A meal made from hulled cotton seeds after the oil has been expressed.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cottonseed oil</ent><br/
<hw>Cottonseed oil</hw>. <def>A fixed, semidrying oil extracted from cottonseed. It is pale yellow when pure (sp. gr., .92-.93). and is extensively used in soap making, in cookery, and as an adulterant of other oils.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cotton State</ent><br/
<hw>Cotton State</hw>. <def>Alabama; -- a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cottontail</ent><br/
<hw>Cot"ton*tail`</hw> <pr>(k<ocr/t"t'n*t<amac/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The American wood rabbit (<spn>Lepus sylvaticus</spn>); -- also called <altname>Molly cottontail</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottonweed</ent><br/
<hw>Cot"ton*weed`</hw> <pr>(-w<emac/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Cudweed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cottonwick</ent><br/
<hw>cottonwick</hw> <pos>n.</pos> <def>a type of grunt (<spn>Haemulon melanurum</spn>) of warm Atlantic waters.</def><br/
<syn><b>Syn. --</b> <spn>Haemulon melanurum</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cottonwood</ent><br/
<hw>Cot"ton*wood`</hw> <pr>(-w<oocr/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An American tree of the genus <gen>Populus</gen> or poplar, having the seeds covered with abundant cottonlike hairs; esp., the <spn>Populus monilifera</spn> and <spn>Populus angustifolia</spn> of the Western United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottony</ent><br/
<hw>Cot"ton*y</hw> <pr>(-?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Covered with hairs or pubescence, like cotton; downy; nappy; woolly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to cotton; resembling cotton in appearance or character; soft, like cotton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cottrel</ent><br/
<hw>Cot"trel</hw> <pr>(k?t"tr?l)</pr>, <pos>n.</pos> <def>A trammel, or hook to support a pot over a fire.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coturnix</ent><br/
<hw>Coturnix</hw> <pos>n.</pos> <def>a genus of birds comprising the Old World quail.</def><br/
<syn><b>Syn. --</b> genus <gen>Coturnix</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cotyle</ent><br/
<ent>Cotyla</ent><br/
<mhw>{ <hw>Cot"y*la</hw> <pr>(k?t"?-l?)</pr>, <hw>Cot"y*le</hw> <pr>(k?t"?-l?)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <?/<?/<?/ anything hollow, cup of a joint, small meassure: cf. L.  <ets>cotyla</ets> a measure.]</ety> <fld>(Anat.)</fld> <def>A cuplike cavity or organ. Same as <er>Acetabulum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotyledon</ent><br/
<hw>Cot`y*le"don</hw> <pr>(k?t`?-l?"d?n)</pr>, <pos>n.</pos> <ety>[Gr.<?/<?/<?/ a cupshaped hollow, fr. <?/<?/<?/. See <er>Cotyle</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>One of the patches of villi found in some forms of placenta.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A leaf borne by the caulicle or radicle of an embryo; a seed leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Many plants, as the bean and the maple, have two cotyledons, the grasses only one, and pines have several. In one African plant (<gen>Welwitschia</gen>) the cotyledons are permanent and grow to immense proportions.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotyledonal</ent><br/
<hw>Cot`y*led"on*al</hw> <pr>(k?t`?-l?d"?n-<ait/l)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a cotyledon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotyledonary</ent><br/
<hw>Cot`y*led"on*a*ry</hw> <pr>(-?-r?)</pr>, <pos>a.</pos> <def>Having a cotyledon; tufted; <as>as, the <ex>cotyledonary</ex> placenta of the cow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotyledonous</ent><br/
<hw>Cot`y*led"on*ous</hw> <pr>(-?s; 277)</pr>, <pos>a.</pos> <def>Of or pertaining to a cotyledon or cotyledons; having a seed lobe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotyliform</ent><br/
<hw>Co*tyl"i*form</hw> <pr>(k?-t?l"?-f?rm)</pr>, <pos>a.</pos> <ety>[<ets>Cotyle</ets> + <ets>-form</ets>.]</ety> <fld>(Zool.)</fld> <def>Shaped like a cotyle or a cup.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cotyligerous</ent><br/
<hw>Cot`y*lig"er*ous</hw> <pr>(k?t`?-l?j"?r-?s)</pr>, <pos>a.</pos> <ety>[<ets>Cotyle</ets> + <ets>-gerous</ets>.]</ety> <fld>(Zool.)</fld> <def>Having cotyles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cotyloidal</ent><br/
<ent>cotyloid</ent><br/
<mhw><hw>cot"y*loid</hw>, <hw>cotyloidal</hw></mhw> <pr>(k?t"?-loid)</pr>, <pos>a.</pos> <ety>[<ets>Cotyle</ets> + <ets>-oid</ets>]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Shaped like a cup; cup-shaped; <as>as, the <ecol><b>cotyloid cavity</b></ecol>, the cup-shaped socket which receives the head of the thigh bone</as>.</def> <sd>(b)</sd> <def>Pertaining to a cotyloid cavity; <as>as, the <ex>cotyloid</ex> ligament, or notch</as>.</def><br/
<syn><b>Syn. --</b> acetabular, cotyloid.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Coucal</ent><br/
||<hw>Cou"cal</hw> <pr>(k??"k?l)</pr>, <pos>n.</pos> <ety>[Prob. native name.]</ety> <fld>(Zool.)</fld> <def>A large, Old World, ground cuckoo of the genus <gen>Centropus</gen>, of several species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couch</ent><br/
<hw>Couch</hw> <pr>(kouch)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Couched</conjf> <pr>(koucht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Couching</conjf>.]</vmorph> <ety>[F. <ets>coucher</ets> to lay down, lie down, OF. <ets>colchier</ets>, fr. L.  <ets>collocare</ets> to lay, put, place; <ets>col-</ets> + <ets>locare</ets> to place, fr. <ets>locus</ets> place. See <er>Locus</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To lay upon a bed or other resting place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where unbruised youth, with unstuffed brain,<br/
Does <qex>couch</qex> his limbs, there golden sleep doth reign.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To arrange or dispose as in a bed; -- sometimes followed by the reflexive pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The waters <qex>couch</qex> themselves as may be to the center of this globe, in a spherical convexity.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To lay or deposit in a bed or layer; to bed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is at this day in use at Gaza, to <qex>couch</qex> potsherds, or vessels of earth, in their walls.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Paper Making)</fld> <def>To transfer (as sheets of partly dried pulp) from the wire cloth mold to a felt blanket, for further drying.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To conceal; to include or involve darkly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is all this, and more, that lies naturally <qex>couched</qex> under this allegory.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To arrange; to place; to inlay.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To put into some form of language; to express; to phrase; -- used with <ptcl>in</ptcl> and <ptcl>under</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A well-<qex>couched</qex> invective.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I had received a letter from Flora <qex>couched</qex> in rather cool terms.</q> <rj><qau>Blackw. Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Med.)</fld> <def>To treat by pushing down or displacing the opaque lens with a needle; <as>as, to <ex>couch</ex> a cataract</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To couch a spear</b></col> <it>or</it> <col><b>To couch a lance</b></col></mcol>, <cd>to lower to the position of attack; to place in rest.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>He stooped his head, and <qex>couched his spear</qex>,<br/
And spurred his steed to full career.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To couch malt</b></col>, <cd>to spread malt on a floor.</cd> <au>Mortimer.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couch</ent><br/
<hw>Couch</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To lie down or recline, as on a bed or other place of rest; to repose; to lie.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where souls do <qex>couch</qex> on flowers, we 'll hand in hand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If I court moe women, you 'll <qex>couch</qex> with moe men.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lie down for concealment; to hide; to be concealed; to be included or involved darkly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We 'll <qex>couch</qex> in the castle ditch, till we see the light of our fairies.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The half-hidden, hallf-revealed wonders, that yet <qex>couch</qex> beneath the words of the Scripture.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bend the body, as in reverence, pain, labor, etc.; to stoop; to crouch.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>An aged squire<br/
That seemed to <qex>couch</qex> under his shield three-square.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couch</ent><br/
<hw>Couch</hw>, <pos>n.</pos> <ety>[F.  <ets>couche</ets>, OF. <ets>colche</ets>, <ets>culche</ets>, fr. <ets>colchier</ets>. See <er>Couch</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>A bed or place for repose or sleep; particularly, in the United States, a lounge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gentle sleep . . . why liest thou with the vile<br/
In loathsome beds, and leavest the kingly <qex>couch</qex>?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Like one that wraps the drapery of his <qex>couch</qex><br/
About him, and lies down to pleasant dreams.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any place for repose, as the lair of a beast, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mass of steeped barley spread upon a floor to germinate, in malting; or the floor occupied by the barley; <as>as, <ex>couch</ex> of malt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Painting & Gilding)</fld> <def>A preliminary layer, as of color, size, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couchancy</ent><br/
<hw>Couch"an*cy</hw> <pr>(kouch"<it>a</it>n-s?)</pr>, <pos>n.</pos> <def>State of lying down for repose.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couchant</ent><br/
<hw>Couch"ant</hw> <pr>(kouch"<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[F., p. pr. of <ets>coucher</ets>. See <er>Couch</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Lying down with head erect; squatting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Lying down with the head raised, which distinguishes the posture of <xex>couchant</xex> from that of <xex>dormant</xex>, or sleeping; -- said of a lion or other beast.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Couchant and levant</b></col> <fld>(Law)</fld>, <cd>rising up and lying down; -- said of beasts, and indicating that they have been long enough on land, not belonging to their owner, to lie down and rise up to feed, -- such time being held to include a day and night at the least.</cd>  <rj><au>Blackstone.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couche</ent><br/
||<hw>Cou`ch<eacute/"</hw> <pr>(k??`sh?")</pr>, <pos>a.</pos> <ety>[F., p. p.  of <ets>coucher</ets>. See <er>Couch</er>, <pos>v. t.</pos> ]</ety> <fld>(Her.)</fld> <sd>(a)</sd> <def>Not erect; inclined; -- said of anything that is usually erect, as an escutcheon.</def> <sd>(b)</sd> <def>Lying on its side; thus, a chevron <xex>couch<eacute/</xex> is one which emerges from one side of the escutcheon and has its apex on the opposite side, or at the fess point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couched</ent><br/
<hw>Couched</hw> <pr>(koucht)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Same as <er>Couch<?/</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couchee</ent><br/
<hw>Cou"chee</hw> <pr>(k??"sh?; F. k??"sh?")</pr>, <pos>n.</pos> <ety>[F. <ets>couch<?/e</ets> a sleeping place from <ets>coucher</ets>. See <er>Couch</er>, <pos>v. t.</pos> ]</ety> <def>A reception held at the time of going to bed, as by a sovereign or great prince.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj>
<q>The duke's levees and <qex>couchees</qex> were so crowded that the antechambers were full.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coucher</ent><br/
<hw>Couch"er</hw> <pr>(kouch"?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who couches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Paper Manuf.)</fld> <def>One who couches paper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Cf. L. <ets>collectarius</ets>.]</ety> <fld>(O. Eng. Law)</fld> <sd>(a)</sd> <def>A factor or agent resident in a country for traffic.</def> <au>Blount.</au> <sd>(b)</sd> <def>The book in which a corporation or other body registers its particular acts.</def> <mark>[Obs.]</mark> <au>Cowell.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couch grass</ent><br/
<hw>Couch" grass`</hw> <pr>(gr?s`)</pr>. <fld>(Bot.)</fld> <def>See <er>Quitch grass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couching</ent><br/
<hw>Couch"ing</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Med.)</fld> <def>The operation of putting down or displacing the opaque lens in cataract.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Embroidering by laying the materials upon the surface of the foundation, instead of drawing them through.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couchless</ent><br/
<hw>Couch"less</hw> <pr>(kouch"l?s)</pr>, <pos>a.</pos> <def>Having no couch or bed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coudee</ent><br/
<hw>Cou"dee</hw> <pr>(k??"d?; F. k??`d?")</pr>, <pos>n.</pos> <ety>[F. <ets>coud<?/e</ets>, from <ets>coude</ets> elbow.]</ety> <def>A measure of length; the distance from the elbow to the end of the middle finger; a cubit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cougar</ent><br/
<hw>Cou"gar</hw> <pr>(k??"g?r)</pr>, <pos>n.</pos> <ety>[F.  <ets>couguar</ets>, from the native name in the South American dialects, <ets>cuguacuara</ets>, <ets>cuguacuarana</ets>.]</ety> <fld>(Zool.)</fld> <def>An American feline quadruped (<spn>Felis concolor</spn>), resembling the African panther in size and habits. Its color is tawny, without spots; hence writers often called it the <altname>American lion</altname>. Called also <altname>puma</altname>, <altname>panther</altname>, <altname>mountain lion</altname>, and <altname>catamount</altname>. See <er>Puma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cough</ent><br/
<hw>Cough</hw> <pr>(k?f)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coughed</conjf> <pr>(k?ft)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coughing</conjf>.]</vmorph> <ety>[Cf. D. <ets>kuchen</ets>, MHG. <ets>k<?/chen</ets> to breathe, G. <ets>keuchen</ets> to pant, and E.  <ets>chincough</ets>, the first part of which is prob. akin to <ets>cough</ets>; cf. also E. <ets>choke</ets>.]</ety> <def>To expel air, or obstructing or irritating matter, from the lungs or air passages, in a noisy and violent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cough</ent><br/
<hw>Cough</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To expel from the lungs or air passages by coughing; -- followed by <xex>up</xex>; <as>as, to <ex>cough</ex> up phlegm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring to a specified state by coughing; <as>as, he <ex>coughed</ex> himself hoarse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cough down</b></col>, <cd>to silence or put down (an objectionable speaker) by simulated coughing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cough</ent><br/
<hw>Cough</hw>, <pos>n.</pos> <ety>[Cg. D.  <ets>kuch</ets>. See <er>Cough</er>, <pos>v. i.</pos> ]</ety> <sn>1.</sn> <def>A sudden, noisy, and violent expulsion of air from the chest, caused by irritation in the air passages, or by the reflex action of nervous or gastric disorder, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The more or less frequent repetition of coughing, constituting a symptom of disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Stomach cough</b></col>, <col><b>Ear cough</b></col></mcol>, <cd>cough due to irritation in the stomach or ear.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cougher</ent><br/
<hw>Cough"er</hw> <pr>(k?f"?r)</pr>, <pos>n.</pos> <def>One who coughs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couhage</ent><br/
<hw>Cou"hage</hw> <pr>(kou"?j)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Cowhage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Could</ent><br/
<hw>Could</hw> <pr>(k??d)</pr>, <pos>imp.</pos> <mord>of <er>Can</er></mord>. <ety>[OF. <ets>coude</ets>. The <ets>l</ets> was inserted by mistake, under the influence of <ets>should</ets> and <ets>would</ets>.]</ety> <def>Was, should be, or would be, able, capable, or susceptible. Used as an auxiliary, in the past tense or in the conditional present.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coulee</ent><br/
||<hw>Cou`lee"</hw> <pr>(k??`l?")</pr>, <pos>n.</pos> <ety>[F. <ets>coul<eacute/e</ets>, fr. <ets>couler</ets> to run or flow.]</ety> <def>A stream</def>; <fld>(Geol.)</fld> <def>a stream of lava. Also, in the Western United States, the bed of a stream, even if dry, when deep and having inclined sides; distinguished from a <xex>ca<ntil/on</xex>, which has precipitous sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couleur</ent><br/
||<hw>Cou`leur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>Color; -- chiefly used in a few French phrases, as <xex>couler de rose</xex>, color of rose; and hence, adjectively, rose-colored; roseate.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A suit of cards, as hearts or clubs; -- used in some French games.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coulisse</ent><br/
||<hw>Cou*lisse"</hw> <pr>(k<oocr/*l<icr/s"; <it>F.</it> k<oomac/`l<esl/s")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>couler</ets> to flow, glide.]</ety> <sn>1.</sn> <def>A piece of timber having a groove in which something glides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the side scenes of the stage in a theater, or the space included between the side scenes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coulisse</ent><br/
<hw>Cou*lisse"</hw> <pr>(k<oomac/*l<emac/s")</pr>, <pos>n.</pos> <sn>1.</sn> <def>A fluting in a sword blade.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The outside stock exchange, or <ldquo/curb market,<rdquo/ of Paris.</def> <mark>[French Use]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Couloir</ent><br/
||<hw>Cou`loir"</hw> <pr>(k??`lw?r")</pr>, <pos>n.</pos> <ety>[F., a strainer.]</ety> <sn>1.</sn> <def>A deep gorge; a gully.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Hydraul. Engin.)</fld> <def>A dredging machine for excavating canals, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coulomb</ent><br/
||<hw>Cou`lomb"</hw> <pr>(k??`l?n")</pr>, <pos>n.</pos> <ety>[From <ets>Coulomb</ets>, a French physicist and electrican.]</ety> <fld>(Physics)</fld> <def>The standard unit of quantity in electrical measurements. It is the quantity of electricity conveyed in one second by the current produced by an electro-motive force of one volt acting in a circuit having a resistance of one ohm, or the quantity transferred by one amp<egrave/re in one second. Formerly called <altname>weber</altname>.</def><-- ##!! note that "weber" now has a totally different meaning. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coulomb meter</ent><br/
<hw>Cou`lomb" me"ter</hw> <pr>(?)</pr>. <fld>(Elec.)</fld> <def>Any instrument by which electricity can be measured in coulombs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coulomb's law</ent><br/
<hw>Cou`lomb's" law</hw> <pr>(?)</pr>. <fld>(Physics)</fld> <def>The law that the force exerted between two electric or magnetic charges is directly proportional to the product of the charges and inversely to the square of the distance between them.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coulter</ent><br/
<hw>Coul"ter</hw> <pr>(k<omac/l"t<etil/r)</pr>, <pos>n.</pos> <def>Same as <er>Colter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coulterneb</ent><br/
<hw>Coul"ter*neb`</hw> <pr>(-n<ecr/b`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The puffin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coulure</ent><br/
<hw>Cou*lure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prop., a dropping.]</ety> <fld>(Hort.)</fld> <def>A disease affecting grapes, esp. in California, manifested by the premature dropping of the fruit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coumaric</ent><br/
<hw>Cou*mar"ic</hw> <pr>(k??-m?r"?k)</pr>, <pos>a.</pos> <def>Relating to, derived from, or like, the <spn>Dipterix odorata</spn>, a tree of Guiana.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coumaric acid</b></col> <fld>(Chem.)</fld>, <cd>one of a series of aromatic acids, related to cinnamic acid, the most important of which is a white crystalline substance, <chform>HO.C6H4.C2H2.CO2H</chform>, obtained from the tonka bean, sweet clover, etc., and also produced artificially.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coumarin</ent><br/
<hw>Cou"ma*rin</hw> <pr>(k<oomac/"m<adot/*r<icr/n)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>coumarou</ets>, a tree of Guiana.]</ety> <fld>(Chem.)</fld> <def>The concrete essence of the tonka bean, the fruit of <spn>Dipterix (<plain>formerly</plain> Coumarouna) odorata</spn> and consisting essentially of coumarin proper, which is a white crystalline substance, <chform>C9H6O2</chform>, of vanilla-like odor, regarded as an anhydride of coumaric acid, and used in flavoring. Coumarin in also made artificially.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 331 --></p>

<p><ent>Coumarou</ent><br/
||<hw>Cou"ma*rou</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Coumarin</er>.]</ety> <fld>(Bot.)</fld> <def>The tree (<spn>Dipteryx odorata</spn>) which bears the tonka bean; also, the bean itself.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Council</ent><br/
<hw>Coun"cil</hw> <pr>(koun"s<icr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>concile</ets>, fr. L. <ets>concilium</ets>; <ets>con-</ets> + <ets>calare</ets> to call, akin to Gr. <?/<?/<?/ to call, and E. <ets>hale</ets>, v., <ets>haul</ets>. Cf. <er>Conciliate</er>. This word is often confounded with <ets>counsel</ets>, with which it has no connection.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An assembly of men summoned or convened for consultation, deliberation, or advice; <as>as, a <ex>council</ex> of physicians for consultation in a critical case</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A body of man elected or appointed to constitute an advisory or a legislative assembly; <as>as, a governor's <ex>council</ex>; a city <ex>council</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An old lord of the <qex>council</qex> rated me the other day.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Act of deliberating; deliberation; consultation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Satan . . . void of rest,<br/
His potentates to <qex>council</qex> called by night.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O great in action and in <qex>council</qex> wise.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Aulic council</b></col>. <cd>See under <er>Aulic</er>.</cd> -- <col><b>Cabinet council</b></col>. <cd>See under <er>Cabinet</er>.</cd> -- <col><b>City council</b></col>, <cd>the legislative branch of a city government, usually consisting of a board of aldermen and common council, but sometimes otherwise constituted.</cd> -- <col><b>Common council</b></col>. <cd>See under <er>Common</er>.</cd> -- <mcol><col><b>Council board</b></col>, <col><b>Council table</b></col></mcol>, <cd>the table round which a council holds consultation; also, the council itself in deliberation.</cd> -- <col><b>Council chamber</b></col>, <cd>the room or apartment in which a council meets.</cd> -- <col><b>Council fire</b></col>, <cd>the ceremonial fire kept burning while the Indians hold their councils.</cd> <mark>[U.S.]</mark> <au>Bartlett.</au> -- <col><b>Council of war</b></col>, <cd>an assembly of officers of high rank, called to consult with the commander in chief in regard to measures or importance or nesessity.</cd> -- <col><b>Ecumenical council</b></col> <fld>(Eccl.)</fld>, <cd>an assembly of prelates or divines convened from the whole body of the church to regulate matters of doctrine or discipline.</cd> -- <col><b>Executive council</b></col>, <cd>a body of men elected as advisers of the chief magistrate, whether of a State or the nation.</cd> <mark>[U.S.]</mark> -- <col><b>Legislative council</b></col>, <cd>the upper house of a legislature, usually called the <xex>senate</xex>.</cd> -- <col><b>Privy council</b></col>. <cd>See under <er>Privy</er>.</cd> <mark>[Eng.]</mark></cs></p>

<p><syn><b>Syn.</b> -- Assembly; meeting; congress; diet; parliament; convention; convocation; synod.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Councilist</ent><br/
<hw>Coun"cil*ist</hw> <pr>(koun"s?l-?st)</pr>, <pos>n.</pos> <def>One who belongs to a council; one who gives an opinion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will in three months be an expert <qex>counsilist</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Councilman</ent><br/
<hw>Coun"cil*man</hw> <pr>(koun"s<icr/l*m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Councilmen</plw> <pr>(koun"s<icr/l*m<eit/n)</pr></plu>. <def>A member of a council, especially of the common council of a city; a councilor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Councilor</ent><br/
<hw>Coun"cil*or</hw> <pr>(koun"s?l-?r)</pr>, <pos>n.</pos> <def>A member of a council.</def> <altsp>[Written also <asp>councillor</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The distinction between <xex>councilor</xex>, a member of a council, and <xex>counselor</xex>, one who gives counsel, was not formerly made, but is now very generally recognized and observed.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>councillorship</ent><br/
<ent>councilorship</ent><br/
<mhw><hw>councilorship</hw>, <hw>councillorship</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>the position of council member.</def><br/
<syn><b>Syn. --</b> councilorship.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Co-une</ent><br/
<hw>Co`-une"</hw> <pr>(k?`?n")</pr>, <pos>v. t.</pos> <ety>[L. <ets>co-</ets> + <ets>unus</ets> one.]</ety> <def>To combine or unite.</def> <mark>[Obs.]</mark> <ldquo/<xex>Co-uned</xex> together.<rdquo/  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-unite</ent><br/
<hw>Co`-u*nite"</hw> <pr>(k?`?-n?t")</pr>, <pos>v. t.</pos> <def>To unite.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Co-unite</ent><br/
<hw>Co`-u*nite"</hw>, <pos>a.</pos> <def>United closely with another.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counsel</ent><br/
<hw>Coun"sel</hw> <pr>(koun"s<ecr/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>conseil</ets>, F.  <ets>conseil</ets>, fr. L. <ets>consilium</ets>, fr. the root of <ets>consulere</ets> to consult, of uncertain origin. Cf. <er>Consult</er>, <er>Consul</er>.]</ety> <sn>1.</sn> <def>Interchange of opinions; mutual advising; consultation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the chief priest and elders of the people took <qex>counsel</qex> against Jesus, to put him to death.</q> <rj><qau>Matt. xxvii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Examination of consequences; exercise of deliberate judgment; prudence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They all confess, therefore, in the working of that first cause, that <qex>counsel</qex> is used.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Result of consultation; advice; instruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I like thy <qex>counsel</qex>; well hast thou advised.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was ill <qex>counsel</qex> had misled the girl.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Deliberate purpose; design; intent; scheme; plan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>counsel</qex> of the Lord standeth forever.</q> <rj><qau>Ps. xxxiii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>counsels</qex> of the wicked are deceit.</q> <rj><qau>Prov. xii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A secret opinion or purpose; a private matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thilke lord . . . to whom no <qex>counsel</qex> may be hid.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>One who gives advice, especially in legal matters; one professionally engaged in the trial or management of a cause in court; also, collectively, the legal advocates united in the management of a case; <as>as, the defendant has able <ex>counsel</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The King found his <qex>counsel</qex> as refractory as his judges.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some courts a distinction is observed between the attorney and the counsel in a cause, the former being employed in the management of the more mechanical parts of the suit, the latter in attending to the pleadings, managing the cause at the trial, and in applying the law to the exigencies of the case during the whole progress of the suit. In other courts the same person can exercise the powers of each. See <er>Attorney</er>.</note>  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In counsel</b></col>, <cd>in secret.</cd> <mark>[Obs.]</mark> <au>Chaucer.</au> -- <col><b>To keep counsel</b></col>, <it>or</it> <col><b>To keep one's own counsel</b></col>, <cd>to keep one's thoughts, purposes, etc., undisclosed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The players can not <qex>keep counsel</qex>: they 'll tell all.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Advice; consideration; consultation; purpose; scheme; opinion.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counsel</ent><br/
<hw>Coun"sel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counseled</conjf> <pr>(-s<ecr/ld)</pr> <or/ <conjf>Counselled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Counseling</conjf> <or/ <conjf>Counselling</conjf>.]</vmorph> <ety>[OE. <ets>conseilen</ets>, <ets>counseilen</ets>, F.  <ets>conseiller</ets>, fr. L. <ets>consiliari</ets>, fr. <ets>consilium</ets> counsel.]</ety> <sn>1.</sn> <def>To give advice to; to advice, admonish, or instruct, as a person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Good sir, I do in friendship <qex>counsel</qex> you<br/
To leave this place.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To advise or recommend, as an act or course.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They who <qex>counsel</qex> war.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus Belial, with words clothed in reason's garb,<br/
<qex>Counseled</qex> ignoble ease and peaceful sloth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counselable</ent><br/
<hw>Coun"sel*a*ble</hw> <pr>(-<adot/*b'l)</pr>, <pos>a.</pos> <ety>[Written also <ets>counsellable</ets>.]</ety> <sn>1.</sn> <def>Willing to receive counsel or follow advice.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Few men of so great parts were upon all occasions more <qex>counselable</qex> than he.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Suitable to be advised; advisable, wise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He did not believe it <qex>counselable</qex>.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>counseling</ent><br/
<hw>counseling</hw> <pos>n.</pos> <def>direction or advice as to a decision or course of action.</def><br/
<syn><b>Syn. --</b> guidance, counsel, direction.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>counsellor</ent><br/
<hw>counsellor</hw> <pos>n.</pos> <def>same as <er>counselor</er>.</def><br/
<syn><b>Syn. --</b> counselor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>counsellorship</ent><br/
<hw>counsellorship</hw> <pos>n.</pos> <def>the position of counselor; same as <er>counselorship</er>.</def><br/
<syn><b>Syn. --</b> counselorship.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Counselor</ent><br/
<hw>Coun"sel*or</hw> <pr>(koun"s<ecr/l*<etil/r)</pr>, <pos>n.</pos> <altsp>[Written also <asp>counsellor</asp>.]</altsp> <ety>[OE. <ets>conseiler</ets>, F.  <ets>conseiller</ets>, fr. L. <ets>consiliarius</ets>, fr. <ets>consilium</ets> counsel.]</ety> <sn>1.</sn> <def>One who counsels; an adviser.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Can he that speaks with the tongue of an enemy be a good <qex>counselor</qex>, or no?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of council; one appointed to advise a sovereign or chief magistrate. <note>[See under <er>Consilor</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One whose profession is to give advice in law, and manage causes for clients in court; a barrister.</def><br/
<syn><b>Syn. --</b> advocate, counsel, counselor, counselor-at-law; attorney.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>Good <qex>counselors</qex> lack no clients.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>someone who has supervisory duties at a summer camp.</def><br/
[<source>PJC</source>]</p>

<p><ent>counselor-at-law</ent><br/
<hw>counselor-at-law</hw> <pos>n.</pos> <def>a lawyer who pleads cases in court.</def><br/
<syn><b>Syn. --</b> advocate, counsel, counselor, counsellor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Counselorship</ent><br/
<hw>Coun"sel*or*ship</hw> <pr>(koun"s?l-?r-sh?p)</pr>, <pos>n.</pos> <def>The function and rank or office of a counselor.</def>
 <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Count</ent><br/
<hw>Count</hw> <pr>(kount)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Counting</conjf>.]</vmorph> <ety>[OF. <ets>conter</ets>, and later (etymological spelling) <ets>compter</ets>, in modern French thus distinguished; <ets>conter</ets> to relate (cf. <er>Recount</er>, <er>Account</er>), <ets>compter</ets> to count; fr. L. <ets>computuare</ets> to reckon, compute; <ets>com-</ets> + <ets>putare</ets> to reckon, settle, order, prune, orig., to clean. See <er>Pure</er>, and cf. <er>Compute</er>.]</ety> <sn>1.</sn> <def>To tell or name one by one, or by groups, for the purpose of ascertaining the whole number of units in a collection; to number; to enumerate; to compute; to reckon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who can <qex>count</qex> the dust of Jacob?</q> <rj><qau>Num. xxiii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In a journey of forty miles, Avaux <qex>counted</qex> only three miserable cabins.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To place to an account; to ascribe or impute; to consider or esteem as belonging.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Abracham believed God, and it was <qex>counted</qex> unto him for righteousness.</q> <rj><qau>Rom. iv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To esteem; to account; to reckon; to think, judge, or consider.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>count</qex> myself in nothing else so happy<br/
As in a soul remembering my good friends.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To count out</b></col>. <sd>(a)</sd> <cd>To exclude (one) from consideration; to be assured that (one) will not participate or cannot be depended upon.</cd> <sd>(b)</sd> <fld>(House of Commons)</fld> <cd>To declare adjourned, as a sitting of the House, when it is ascertained that a quorum is not present.</cd> <sd>(c)</sd> <cd>To prevent the accession of (a person) to office, by a fraudulent return or count of the votes cast; -- said of a candidate really elected.</cd> <mark>[Colloq.]</mark></cs></p>

<p><syn><b>Syn.</b> -- To calculate; number; reckon; compute; enumerate. See <er>Calculate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Count</ent><br/
<hw>Count</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To number or be counted; to possess value or carry weight; hence, to increase or add to the strength or influence of some party or interest; <as>as, every vote <ex>counts</ex>; accidents <ex>count</ex> for nothing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This excellent man . . . <qex>counted</qex> among the best and wisest of English statesmen.</q> <rj><qau>J. A. Symonds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reckon; to rely; to depend; -- with <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was brewer to the palace; and it was apprehended that the government <qex>counted</qex> on his voice.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I think it a great error to <qex>count</qex> upon the genius of a nation as a standing argument in all ages.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take account or note; -- with</def> <xex>of</xex>. <mark>[Obs.]</mark> <ldquo/No man <xex>counts</xex> of her beauty.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eng. Law)</fld> <def>To plead orally; to argue a matter in court; to recite a count.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Count</ent><br/
<hw>Count</hw>, <pos>n.</pos> <ety>[F. <ets>conte</ets> and <ets>compte</ets>, with different meanings, fr. L. <ets>computus</ets> a computation, fr. <ets>computare</ets>. See <er>Count</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of numbering; reckoning; also, the number ascertained by counting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of blessed saints for to increase the <qex>count</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By this <qex>count</qex>, I shall be much in years.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An object of interest or account; value; estimation.</def> <mark>[Obs.]</mark> <ldquo/All his care and <xex>count</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A formal statement of the plaintiff's case in court; in a more technical and correct sense, a particular allegation or charge in a declaration or indictment, separately setting forth the cause of action or prosecution.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the old law books, <xex>count</xex> was used synonymously with <xex>declaration</xex>. When the plaintiff has but a single cause of action, and makes but one statement of it, that statement is called indifferently <xex>count</xex> or <xex>declaration</xex>, most generally, however, the latter. But where the suit embraces several causes, or the plaintiff makes several different statements of the same cause of action, each statement is called a <xex>count</xex>, and all of them combined, a <xex>declaration</xex>.</note>  <rj><au>Bouvier. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Count</ent><br/
<hw>Count</hw>, <pos>n.</pos> <ety>[F. <ets>conte</ets>, fr. L. <ets>comes</ets>, <ets>comitis</ets>, associate, companion, one of the imperial court or train, properly, one who goes with another; <ets>com-</ets> + <ets>ire</ets> to go, akin to Skr. <ets>i</ets> to go.]</ety> <def>A nobleman on the continent of Europe, equal in rank to an English earl.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Though the tittle <xex>Count</xex> has never been introduced into Britain, the wives of Earls have, from the earliest period of its history, been designated as <xex>Countesses</xex>.</note>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Count palatine</b></col>. <sd>(a)</sd> <cd>Formerly, the proprietor of a county who possessed royal prerogatives within his county, as did the Earl of Chester, the Bishop of Durham, and the Duke of Lancaster.</cd> <mark>[Eng.]</mark> See <cref>County palatine</cref>, under <er>County</er>. <sd>(b)</sd> <cd>Originally, a high judicial officer of the German emperors; afterward, the holder of a fief, to whom was granted the right to exercise certain imperial powers within his own domains.</cd> <mark>[Germany]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countable</ent><br/
<hw>Count"a*ble</hw> <pr>(-?-b'l)</pr>, <pos>a.</pos> <def>Capable of being numbered.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countenance</ent><br/
<hw>Coun"te*nance</hw> <pr>(koun"t<esl/*n<ait/ns)</pr>, <pos>n.</pos> <ety>[OE. <ets>contenance</ets>, <ets>countenaunce</ets>, demeanor, composure, F. <ets>contenance</ets> demeanor, fr. L. <ets>continentia</ets> continence, LL. also, demeanor, fr. L. <ets>continere</ets> to hold together, repress, contain. See <er>Contain</er>, and cf. <er>Continence</er>.]</ety> <sn>1.</sn> <def>Appearance or expression of the face; look; aspect; mien.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So spake the Son, and into terror changed<br/
His <qex>countenance</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The face; the features.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>countenance</qex> somewhat doth resemble you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Approving or encouraging aspect of face; hence, favor, good will, support; aid; encouragement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast made him . . . glad with thy <qex>countenance</qex>.</q> <rj><qau>Ps. xxi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the magistrate's peculiar province, to give <qex>countenance</qex> to piety and virtue, and to rebuke vice.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Superficial appearance; show; pretense.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The election being done, he made <qex>countenance</qex> of great discontent thereat.</q> <rj><qau>Ascham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In countenance</b></col>, <cd>in an assured condition or aspect; free from shame or dismay.</cd> <ldquo/It puts the learned <xex>in countenance</xex>, and gives them a place among the fashionable part of mankind.<rdquo/ <au>Addison.</au> -- <col><b>Out of countenance</b></col>, <cd>not bold or assured; confounded; abashed.</cd> <ldquo/Their best friends were <xex>out of countenance</xex>, because they found that the imputations . . . were well grounded.<rdquo/ <au>Clarendon.</au> -- <col><b>To keep the countenance</b></col>, <cd>to preserve a composed or natural look, undisturbed by passion or emotion.</cd> <au>Swift.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countenance</ent><br/
<hw>Coun"te*nance</hw> <pr>(koun"t?-n<it>a</it>ns)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countenanced</conjf> <pr>(-n<it>a</it>nst)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countenancing</conjf>.]</vmorph> <sn>1.</sn> <def>To encourage; to favor; to approve; to aid; to abet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This conceit, though <qex>countenanced</qex> by learned men, is not made out either by experience or reason.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Error supports custom, custom <qex>countenances</qex> error.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a show of; to pretend.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which to these ladies love did <qex>countenance</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countenancer</ent><br/
<hw>Coun"te*nan*cer</hw> <pr>(-n<it>a</it>n-s?r)</pr>, <pos>n.</pos> <def>One who countenances, favors, or supports.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter-</ent><br/
<hw>Coun"ter-</hw> <pr>(koun"t<etil/r-)</pr>. <note>[See <er>Counter</er>, <pos>adv.</pos> ]</note> <def>A prefix meaning <xex>contrary</xex>, <xex>opposite</xex>, <xex>in opposition</xex>; <as>as, <ex>counter</ex>act, <ex>counter</ex>balance, <ex>counter</ex>check</as>. See <er>Counter</er>, <pos>adv. & a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter</ent><br/
<hw>Count"er</hw> <pr>(koun"t<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>countere</ets>, <ets>countour</ets>, a counter (in sense 1), OF.  <ets>contere</ets>, <ets>conteor</ets>, fr. <ets>conter</ets> to count. See <er>Count</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>One who counts, or reckons up; a calculator; a reckoner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of metal, ivory, wood, or bone, used in reckoning, in keeping account of games, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The old gods of our own race whose names . . . serve as <qex>counters</qex> reckon the days of the week.</q> <rj><qau>E. B. Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What comes the wool to? . . . I can not do it without <qex>counters</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Money; coin; -- used in contempt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To lock such rascal <qex>counters</qex> from his friends.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A prison; either of two prisons formerly in London.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Anne Aysavugh . . . imprisoned in the <qex>Counter</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A telltale; a contrivance attached to an engine, printing press, or other machine, for the purpose of counting the revolutions or the pulsations.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter</ent><br/
<hw>Coun"ter</hw>, <pos>n.</pos> <ety>[OE. <ets>countour</ets>, OF. <ets>contouer</ets>, <ets>comptouer</ets>, F. <ets>comptoir</ets>, LL. <ets>computatorium</ets>, prop., a computing place, place of accounts, fr. L. <ets>computare</ets>. See <er>Count</er>, <pos>v. t.</pos>]</ety> <def>A table or board on which money is counted and over which business is transacted; a long, narrow table or bench, on which goods are laid for examination by purchasers, or on which they are weighed or measured.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Over the counter</b></col> <sd>(a)</sd> <fld>(Stock Exchanges)</fld>, <cd>in an office; -- said of business so done, as distinguished from that done at an exchange.</cd> <mark>[Cant]</mark> -- <sd>(a)</sd> <cd>without a prescription; needing no prescription; -- said of medicines that can be legally bought without a physician's prescription.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Counter</ent><br/
<hw>Coun"ter</hw>, <pos>adv.</pos> <ety>[F. <ets>contre</ets>, fr. L. <ets>contra</ets> against. Cf. <er>Contra-</er>.]</ety> <sn>1.</sn> <def>Contrary; in opposition; in an opposite direction; contrariwise; -- used chiefly with <xex>run</xex> or <xex>go</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Running <qex>counter</qex> to all the rules of virtue.</q> <rj><qau>Locks.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the wrong way; contrary to the right course; <as>as, a hound that runs <ex>counter</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is <qex>counter</qex>, you false Danish dogs!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>At or against the front or face.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which [darts] they never throw <qex>counter</qex>, but at the back of the flier.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter</ent><br/
<hw>Coun"ter</hw>, <pos>a.</pos> <def>Contrary; opposite; contrasted; opposed; adverse; antagonistic; <as>as, a <ex>counter</ex> current; a <ex>counter</ex> revolution; a <ex>counter</ex> poison; a <ex>counter</ex> agent; <ex>counter</ex> fugue</as>.</def> <ldquo/Innumerable facts attesting the <xex>counter</xex> principle.<rdquo/  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Counter approach</b></col> <fld>(Fort.)</fld>, <cd>a trench or work pushed forward from defensive works to meet the approaches of besiegers. See <er>Approach</er>.</cd> -- <col><b>Counter bond</b></col> <fld>(Law)</fld>, <cd>in old practice, a bond to secure one who has given bond for another.</cd> -- <col><b>Counter brace</b></col>. <cd>See <er>Counter brace</er>, in Vocabulary.</cd> -- <col><b>Counter deed</b></col> <fld>(Law)</fld>, <cd>a secret writing which destroys, invalidates, or alters, a public deed.</cd> -- <col><b>Counter distinction</b></col>, <cd>contradistinction.</cd> <mark>[Obs.]</mark> -- <col><b>Counter drain</b></col>, <cd>a drain at the foot of the embankment of a canal or watercourse, for carrying off the water that may soak through.</cd> -- <col><b>Counter extension</b></col> <fld>(Surg.)</fld>, <cd>the fixation of the upper part of a limb, while extension is practiced on the lower part, as in cases of luxation or fracture.</cd> -- <col><b>Counter fissure</b></col> <fld>(Surg.)</fld> <cd>Same as <er>Contrafissure</er>.</cd> -- <col><b>Counter indication</b></col>. <fld>(Med.)</fld> <cd>Same as <er>Contraindication</er>.</cd> -- <col><b>Counter irritant</b></col> <fld>(Med.)</fld>, <cd>an irritant to produce a blister, a pustular eruption, or other irritation in some part of the body, in order to relieve an existing irritation in some other part.</cd> <ldquo/<xex>Counter irritants</xex> are of as great use in moral as in physical diseases.<rdquo/ <au>Macaulay.</au> -- <col><b>Counter irritation</b></col> <fld>(Med.)</fld>, <cd>the act or the result of applying a <xex>counter irritant</xex>.</cd> -- <col><b>Counter opening</b></col>, <cd>an aperture or vent on the opposite side, or in a different place.</cd> -<col><b>Counter parole</b></col> <fld>(Mil.)</fld>, <cd>a word in addition to the password, given in time of alarm as a signal.</cd> -- <col><b>Counter plea</b></col> <fld>(Law)</fld>, <cd>a replication to a plea.</cd> <au>Cowell.</au> -- <col><b>Counter pressure</b></col>, <cd>force or pressure that acts in a contrary direction to some other opposing pressure.</cd> -- <col><b>Counter project</b></col>, <cd>a project, scheme, or proposal brought forward in opposition to another, as in the negotiation of a treaty.</cd> <au>Swift.</au> -- <col><b>Counter proof</b></col>, <cd>in engraving, a print taken off from another just printed, which, by being passed through the press, gives a copy in reverse, and of course in the same position as that of plate from which the first was printed, the object being to enable the engraver to inspect the state of the plate.</cd> -- <col><b>Counter revolution</b></col>, <cd>a revolution opposed to a former one, and restoring a former state of things.</cd> -- <col><b>Counter revolutionist</b></col>, <cd>one engaged in, or befriending, a counter revolution.</cd> -- <col><b>Counter round</b></col> <fld>(Mil.)</fld>, <cd>a body of officers whose duty it is to visit and inspect the rounds and sentinels.</cd> -- <col><b>Counter sea</b></col> <fld>(Naut.)</fld>, <cd>a sea running in an opposite direction from the wind.</cd> -- <col><b>Counter sense</b></col>, <cd>opposite meaning.</cd> -- <col><b>Counter signal</b></col>, <cd>a signal to answer or correspond to another.</cd> -- <col><b>Counter signature</b></col>, <cd>the name of a secretary or other officer countersigned to a writing.</cd> <au>Tooke.</au> -- <col><b>Counter slope</b></col>, <cd>an overhanging slope; as, a wall with a <xex>counter slope</xex>.</cd> <au>Mahan.</au> -- <col><b>Counter statement</b></col>, <cd>a statement made in opposition to, or denial of, another statement.</cd> -- <col><b>Counter surety</b></col>, <cd>a counter bond, or a surety to secure one who has given security.</cd> -- <col><b>Counter tally</b></col>, <cd>a tally corresponding to another.</cd> -- <col><b>Counter tide</b></col>, <cd>contrary tide.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter</ent><br/
<hw>Coun"ter</hw>, <pos>n.</pos> <ety>[See <er>Counter</er>, <pos>adv.</pos>, <er>Contra</er>.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>The after part of a vessel's body, from the water line to the stern, -- below and somewhat forward of the stern proper.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 332 --></p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Same as <er>Contra</er>. Formerly used to designate any under part which served for contrast to a principal part, but now used as equivalent to <altname>counter tenor</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Far.)</fld> <def>The breast, or that part of a horse between the shoulders and under the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The back leather or heel part of a boot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter</ent><br/
<hw>Coun"ter</hw> <pr>(koun"t?r)</pr>, <pos>n.</pos> <def>An encounter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With kindly <qex>counter</qex> under mimic shade.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter</ent><br/
<hw>Coun"ter</hw>, <pos>v. i.</pos> <fld>(Boxing)</fld> <def>To return a blow while receiving one, as in boxing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His left hand <qex>countered</qex> provokingly.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counteract</ent><br/
<hw>Coun`ter*act"</hw> <pr>(koun`t?r-?kt")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counteracted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Counteracting</conjf>.]</vmorph> <def>To act in opposition to; to hinder, defeat, or frustrate, by contrary agency or influence; <as>as, to <ex>counteract</ex> the effect of medicines; to <ex>counteract</ex> good advice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counteraction</ent><br/
<hw>Coun`ter*ac"tion</hw> <pr>(koun`t?r-?k"sh?n)</pr>, <pos>n.</pos> <def>Action in opposition; hindrance resistance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] do not . . . overcome the <qex>counteraction</qex> of a false principle or of stubborn partiality.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counteractive</ent><br/
<hw>Coun`ter*act"ive</hw> <pr>(-?kt"?v)</pr>, <pos>a.</pos> <def>Tending to counteract.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counteractive</ent><br/
<hw>Coun`ter*act"ive</hw>, <pos>n.</pos> <def>One who, or that which, counteracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counteractively</ent><br/
<hw>Coun`ter*act"ive*ly</hw>, <pos>adv.</pos> <def>By counteraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>counterattraction</ent><br/
<hw>counterattraction</hw> <pos>n.</pos> <def>a rival attraction.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Counterbalance</ent><br/
<hw>Coun`ter*bal"ance</hw> <pr>(-b?l"<it>a</it>ns)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterbalanced</conjf> <pr>(-<it>a</it>nst)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Counterbalancing</conjf>.]</vmorph> <def>To oppose with an equal weight or power; to counteract the power or effect of; to countervail; to equiponderate; to balance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The remaining air was not able to <qex>counterbalance</qex> the mercurial cylinder.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The study of mind is necessary to <qex>counterbalance</qex> and correct the influence of the study of nature.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterbalance</ent><br/
<hw>Coun"ter*bal`ance</hw> <pr>(koun"t?r-b?l`<it>a</it>ns)</pr>, <pos>n.</pos> <def>A weight, power, or agency, acting against or balancing another</def>; as: <sd>(a)</sd> <def>A mass of metal in one side of a driving wheel or fly wheel, to balance the weight of a crank pin, etc., on the opposite side of the wheel</def>. <sd>(b)</sd> <def>A counterpoise to balance the weight of anything, as of a drawbridge or a scale beam.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Money is the <qex>counterbalance</qex> to all other things purchasable by it.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>counterbalanced</ent><br/
<ent>counter-balanced</ent><br/
<mhw><hw>counter-balanced</hw> <hw>counterbalanced</hw></mhw> <pos>adj.</pos>  <def>brought into equipoise by means of a weight or force that offsets another.</def><br/
<syn><b>Syn. --</b> counterpoised.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>counterblast</ent><br/
<hw>counterblast</hw> <pos>n.</pos> <def>a vigorous and unrestrained verbal response; <as>as, her tirade provoked a <ex>counterblast</ex> from her husband</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Counterbore</ent><br/
<hw>Coun"ter*bore`</hw> <pr>(-b?r`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A flat-bottomed cylindrical enlargement of the mouth of a hole, usually of slight depth, as for receiving a cylindrical screw head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of pin drill with the cutting edge or edges normal to the axis; -- used for enlarging a hole, or for forming a flat-bottomed recess at its mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterbore</ent><br/
<hw>Coun`ter*bore"</hw> <pr>(koun`t?r-b?r")</pr>, <pos>v. t.</pos> <def>To form a counterbore in, by boring, turning, or drilling; to enlarge, as a hole, by means of a counterbore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter brace</ent><br/
<hw>Coun"ter brace`</hw> <pr>(br?s`)</pr>. <sn>1.</sn> <fld>(Naut.)</fld> <def>The brace of the fore-topsail on the leeward side of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Engin.)</fld> <def>A brace, in a framed structure, which resists a strain of a character opposite to that which a main brace is designed to receive.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In a quadrilateral system of bracing, the <xex>main brace</xex> is usually in the direction of one diagonal, and the <xex>counter brace</xex> in the direction of the other. Strains in counter braces are occasioned by the live load only, as, in a roof, by the wind, or, in a bridge, by a moving train.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterbrace</ent><br/
<hw>Coun"ter*brace`</hw>, <pos>v. t.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <def>To brace in opposite directions; <as>as, to <ex>counterbrace</ex> the yards, <it>i. e.</it>, to brace the head yards one way and the after yards another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Engin.)</fld> <def>To brace in such a way that opposite strains are resisted; to apply counter braces to.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterbuff</ent><br/
<hw>Coun`ter*buff"</hw> <pr>(koun`t?r-b?f")</pr>, <pos>v. t.</pos> <def>To strike or drive back or in an opposite direction; to stop by a blow or impulse in front.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterbuff</ent><br/
<hw>Coun"ter*buff`</hw> <pr>(koun"t?r-b?f`)</pr>, <pos>n.</pos> <def>A blow in an opposite direction; a stroke that stops motion or cause a recoil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercast</ent><br/
<hw>Coun"ter*cast`</hw> <pr>(koun"t?r-k?st`)</pr>, <pos>n.</pos> <def>A trick; a delusive contrivance.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercaster</ent><br/
<hw>Coun"ter*cast`er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>A caster of accounts; a reckoner; a bookkeeper; -- used contemptuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterchange</ent><br/
<hw>Coun`ter*change"</hw> <pr>(koun`t<etil/r*ch<amac/nj)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterchanged</conjf> <pr>(-ch?njd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Counterchanging</conjf>.]</vmorph> <sn>1.</sn> <def>To give and receive; to cause to change places; to exchange.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To checker; to diversify, as in heraldic counterchanging. See <er>Counterchaged</er>, <pos>a.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Witch-elms, that <qex>counterchange</qex> the floor<br/
Of this flat lawn with dusk and bright.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterchange</ent><br/
<hw>Coun"ter*change`</hw> <pr>(koun"t<etil/r*ch<amac/nj`)</pr>, <pos>n.</pos> <def>Exchange; reciprocation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterchanged</ent><br/
<hw>Coun`ter*changed"</hw> <pr>(-ch?njd")</pr>, <pos>a.</pos> <sn>1.</sn> <def>Exchanged.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Having the tinctures exchanged mutually; thus, if the field is divided palewise, <xex>or</xex> and <xex>azure</xex>, and cross is borne <xex>counterchanged</xex>, that part of the cross which comes on the <xex>azure</xex> side will be <xex>or</xex>, and that on the <xex>or</xex> side will be <xex>azure</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercharge</ent><br/
<hw>Coun"ter*charge`</hw> <pr>(koun"t?r-ch?rj`)</pr>, <pos>n.</pos> <def>An opposing charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercharm</ent><br/
<hw>Coun`ter*charm"</hw> <pr>(koun`t?r-ch?rm")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countercharmed</conjf> <pr>(-ch?rmd`)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countercharming</conjf>.]</vmorph> <def>To destroy the effect of a charm upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercharm</ent><br/
<hw>Coun"ter*charm`</hw> <pr>(koun"t?r-ch?rm`)</pr>, <pos>n.</pos> <def>That which has the power of destroying the effect of a charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercheck</ent><br/
<hw>Coun`ter*check"</hw> <pr>(koun`t?r-ch?k")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterchecked</conjf> <pr>(-ch?ckt")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Counterchecking</conjf>.]</vmorph> <def>To oppose or check by some obstacle; to check by a return check.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercheck</ent><br/
<hw>Coun"ter*check`</hw> <pr>(koun"t?r-ch?k`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A check; a stop; a rebuke, or censure to check a reprover.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any force or device designed to restrain another restraining force; a check upon a check.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The system of checks and <qex>counterchecks</qex>.</q> <rj><qau>J. H. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterclaim</ent><br/
<hw>Coun"ter*claim`</hw> <pr>(-kl<amac/m`)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>A claim made by a person as an offset to a claim made on him.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>counterclockwise</ent><br/
<hw>coun"ter*clock"wise</hw> <pos>adj. & adv.</pos> <def>in the opposite direction to that in which the hands of a clock rotate, as viewed from in front of the clock face; -- of rotatory motion or spiral direction.  Opposite of <ant>clockwise</ant>, or <ant>right-handed</ant></def>  <note>[Narrower terms: <stype>sinistral</stype> <stype>levororotary, levorotatory</stype>]</note><br/
<syn><b>Syn. --</b> left-handed.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Counter-compony</ent><br/
<hw>Coun"ter-com*po`ny</hw> <pr>(-k<ocr/m*p<omac/`n<ycr/)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>See <er>Compony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter-couchant</ent><br/
<hw>Coun"ter-couch`ant</hw> <pr>(koun"t<etil/r*kouch"<ait/nt)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Lying down, with their heads in opposite directions; -- said of animals borne in a coat of arms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter-courant</ent><br/
<hw>Coun"ter-cou*rant"</hw> <pr>(-k??-r?nt")</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Running in opposite directions; -- said of animals borne in a coast of arms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercurrent</ent><br/
<hw>Coun"ter*cur`rent</hw> <pr>(koun"t?r-k?r`-r<it>e</it>nt)</pr>, <pos>a.</pos> <def>Running in an opposite direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countercurrent</ent><br/
<hw>Coun"ter*cur`rent</hw>, <pos>n.</pos> <def>A current running in an opposite direction to the main current.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterdraw</ent><br/
<hw>Coun`ter*draw"</hw> <pr>(koun`t<etil/r*dr<add/")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Counterdrew</conjf> <pr>(-dr<udd/")</pr>; <pos>p. p.</pos> <conjf>Counterdrawn</conjf> <pr>(-dr?n")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Counterdrawing</conjf>.]</vmorph> <def>To copy, as a design or painting, by tracing with a pencil on oiled paper, or other transparent substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>counterfactual</ent><br/
<hw>counterfactual</hw> <pos>adj.</pos> <def>contrary to fact; -- of assertions, ideas, assumptions.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>counterfactuality</ent><br/
<hw>counterfactuality</hw> <pos>n.</pos> <def>the quality of being contrary to fact.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Counterfaisance</ent><br/
<hw>Coun"ter*fai"sance</hw> <pr>(koun"t?r-f?"<it>za</it>ns)</pr>, <pos>n.</pos> <def>See <er>Counterfesance</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfeit</ent><br/
<hw>Coun"ter*feit</hw> <pr>(koun"t?r-f?t)</pr>, <pos>a.</pos> <ety>[F. <ets>contrefait</ets>, p. p. of <ets>contrefaire</ets> to counterfeit; <ets>contre</ets> (L. <ets>contra</ets>) + <ets>faire</ets> to make, fr. L. <ets>facere</ets>. See <er>Counter</er>, <pos>adv.</pos>, and <er>Fact</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Representing by imitation or likeness; having a resemblance to something else; portrayed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Look here upon this picture, and on this-<br/
The <qex>counterfeit</qex> presentment of two brothers.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fabricated in imitation of something else, with a view to defraud by passing the false copy for genuine or original; <as>as, <ex>counterfeit</ex> antiques; <ex>counterfeit</ex> coin</as>.</def> <ldquo/No <xex>counterfeit</xex> gem.<rdquo/  <rj><au>Robinson (More's Utopia).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Assuming the appearance of something; false; spurious; deceitful; hypocritical; <as>as, a <ex>counterfeit</ex> philanthropist</as>.</def> <ldquo/An arrant <xex>counterfeit</xex> rascal.<rdquo/  <rj><au>Shak.</au></rj></p>

<p><syn><b>Syn.</b> -- Forged; fictitious; spurious; false.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfeit</ent><br/
<hw>Coun"ter*feit</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which resembles or is like another thing; a likeness; a portrait; a counterpart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou drawest a <qex>counterfeit</qex><br/
Best in all Athens.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Even Nature's self envied the same,<br/
And grudged to see the <qex>counterfeit</qex> should shame<br/
The thing itself.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is made in imitation of something, with a view to deceive by passing the false for the true; <as>as, the bank note was a <ex>counterfeit</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never call a true piece of gold a <qex>counterfeit</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some of these <qex>counterfeits</qex> are fabricated with such exquisite taste and skill, that it is the achievement of criticism to distinguish them from originals.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who pretends to be what he is not; one who personates another; an impostor; a cheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I fear thou art another <qex>counterfeit</qex>;<br/
And yet, in faith, thou bear'st thee like a king.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfeit</ent><br/
<hw>Coun"ter*feit</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterfeited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Counterfeiting</conjf>.]</vmorph> <sn>1.</sn> <def>To imitate, or put on a semblance of; to mimic; <as>as, to <ex>counterfeit</ex> the voice of another person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Full well they laughed with <qex>counterfeited</qex> glee<br/
At all his jokes, for many a joke had he.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imitate with a view to deceiving, by passing the copy for that which is original or genuine; to forge; <as>as, to <ex>counterfeit</ex> the signature of another, coins, notes, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfeit</ent><br/
<hw>Coun"ter*feit</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To carry on a deception; to dissemble; to feign; to pretend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The knave <qex>counterfeits</qex> well; a good knave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make counterfeits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfeiter</ent><br/
<hw>Coun"ter*feit`er</hw> <pr>(-f?t`?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who counterfeits; one who copies or imitates; especially, one who copies or forges bank notes or coin; a forger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The coin which was corrupted by <qex>counterfeiters</qex>.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who assumes a false appearance or semblance; one who makes false pretenses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Counterfeiters</qex> of devotion.</q> <rj><qau>Sherwood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfeitly</ent><br/
<hw>Coun"ter*feit`ly</hw>, <pos>adv.</pos> <def>By forgery; falsely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfesance</ent><br/
<hw>Coun"ter*fe`sance</hw> <pr>(-f?"<it>za</it>ns)</pr>, <pos>n.</pos> <ety>[OF. <ets>contrefaisance</ets>, fr. <ets>contrefaire</ets>. See <er>Counterfeit</er>, <pos>a.</pos>]</ety> <def>The act of forging; forgery.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>counterfaisance</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfleury</ent><br/
<hw>Coun"ter*fleu`ry</hw> <pr>(koun"t?r-fl?`r?)</pr>, <pos>a.</pos> <ety>[F. <ets>contrefleuri</ets>.]</ety> <fld>(Her.)</fld> <def>Counterflory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterflory</ent><br/
<hw>Coun"ter*flo`ry</hw> <pr>(-fl?`r?)</pr>, <pos>a.</pos> <ety>[See <er>Counterfleury</er>.]</ety> <fld>(Her.)</fld> <def>Adorned with flowers (usually fleurs-de-lis) so divided that the tops appear on one side and the bottoms on the others; -- said of any ordinary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfoil</ent><br/
<hw>Coun"ter*foil`</hw> <pr>(-foil)</pr>, <pos>n.</pos> <ety>[<ets>Counter-</ets> + <ets>foil</ets> a leaf.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>That part of a tally, formerly in the exchequer, which was kept by an officer in that court, the other, called the <xex>stock</xex>, being delivered to the person who had lent the king money on the account; -- called also <altname>counterstock</altname>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The part of a writing (as the stub of a bank check) in which are noted the main particulars contained in the corresponding part, which has been issued.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterforce</ent><br/
<hw>Coun"ter*force`</hw> <pr>(-f?rs`)</pr>, <pos>n.</pos> <def>An opposing force.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterfort</ent><br/
<hw>Coun"ter*fort`</hw> <pr>(-f?rt`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Fort.)</fld> <def>A kind of buttress of masonry to strengthen a revetment wall.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spur or projection of a mountain.</def>  <rj><au>Imp. Dict.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countergage</ent><br/
<hw>Coun"ter*gage`</hw> <pr>(-g<amac/j`)</pr>, <pos>n.</pos> <fld>(Carp.)</fld> <def>An adjustable gage, with double points for transferring measurements from one timber to another, as the breadth of a mortise to the place where the tenon is to be made.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterglow</ent><br/
<hw>Coun"ter*glow`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Astron.)</fld> <def>An exceedingly faint roundish or somewhat oblong nebulous light near the ecliptic and opposite the sun, best seen during September and October, when in the constellations Sagittarius and Pisces. Its cause is not yet understood. Called also <altname>Gegenschein</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Counterguard</ent><br/
<hw>Coun"ter*guard`</hw> <pr>(koun"t<etil/r*g<aum/rd`)</pr>, <pos>n.</pos> <fld>(Fort.)</fld> <def>A low outwork before a bastion or ravelin, consisting of two lines of rampart parallel to the faces of the bastion, and protecting them from a breaching fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterirritation</ent><br/
<ent>Counterirritant</ent><br/
<mhw><hw>Coun"ter*ir`ri*tant</hw> <pr>(-?r"r?-t<it>a</it>nt)</pr>, <pos>n.</pos>, <hw>Coun"ter*ir`ri*ta"tion</hw>, <pos>n.</pos></mhw> <def>See <cref>Counter irritant</cref>, etc., under <er>Counter</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterirritate</ent><br/
<hw>Coun"ter*ir"ri*tate</hw> <pr>(koun"t<etil/r-?r"r?-t?t)</pr>, <pos>v. t.</pos> <fld>(Med.)</fld> <def>To produce counter irritation in; to treat with one morbid process for the purpose of curing another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterjumper</ent><br/
<hw>Coun"ter*jump`er</hw> <pr>(koun"t<etil/r-j?mp`?r)</pr>, <pos>n.</pos> <def>A salesman in a shop; a shopman; -- used contemptuously.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterlath</ent><br/
<hw>Coun"ter*lath`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Building)</fld> <sd>(a)</sd> <def>A batten laid lengthwise between two rafters to afford a bearing for laths laid crosswise.</def> <sd>(b)</sd> <def>Any lath laid without actual measurement between two gauged laths.</def> <sd>(c)</sd> <def>Any of a series of laths nailed to the timbers to raise the sheet lathing above their surface to afford a key for plastering.</def> <sd>(d)</sd> <def>One of many laths used in preparing one side of a partition or framed wall, when the other side has been covered in and finished.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Counterman</ent><br/
<hw>Coun"ter*man</hw> <pr>(koun"t<etil/r*m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Countermen</plw> <pr>(-m<eit/n)</pr>.</plu> <def>A man who attends at the counter of a shop to sell goods.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermand</ent><br/
<hw>Coun`ter*mand"</hw> <pr>(koun`t<etil/r*m<adot/nd")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countermanded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Countermanding</conjf>.]</vmorph> <ety>[F. <ets>contremander</ets>; <ets>contre</ets> (L. <ets>contra</ets>) + <ets>mander</ets> to command, fr. L. <ets>mandare</ets>. Cf. <er>Mandate</er>.]</ety> <sn>1.</sn> <def>To revoke (a former command); to cancel or rescind by giving an order contrary to one previously given; <as>as, to <ex>countermand</ex> an order for goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prohibit; to forbid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Avicen <qex>countermands</qex> letting blood in choleric bodies.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To oppose; to revoke the command of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For us to alter anything, is to lift ourselves against God; and, as it were, to <qex>countermand</qex> him.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermand</ent><br/
<hw>Coun"ter*mand</hw> <pr>(koun"t<etil/r*m<adot/nd)</pr>, <pos>n.</pos> <def>A contrary order; revocation of a former order or command.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Have you no <qex>countermand</qex> for Claudio yet,<br/
But he must die to-morrow?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermandable</ent><br/
<hw>Coun`ter*mand"a*ble</hw> <pr>(-m?nd"?-b'l)</pr>, <pos>a.</pos> <def>Capable of being countermanded; revocable.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermarch</ent><br/
<hw>Coun`ter*march"</hw> <pr>(koun`t?r-m?rch")</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countermarched</conjf> <pr>(-m?rcht")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countermarching</conjf>.]</vmorph> <fld>(Mil.)</fld> <def>To march back, or to march in reversed order.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The two armies marched and <qex>countermarched</qex>, drew near and receded.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermarch</ent><br/
<hw>Coun"ter*march`</hw> <pr>(koun"t?r-m?rch`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A marching back; retrocession.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>An evolution by which a body of troops change front or reverse the direction of march while retaining the same men in the front rank; also, a movement by which the rear rank becomes the front one, either with or without changing the right to the left.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A change of measures; alteration of conduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such <qex>countermarches</qex> and retractions as we do not willingly impute to wisdom.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermark</ent><br/
<hw>Coun"ter*mark`</hw> <pr>(-m?rk`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A mark or token added to those already existing, in order to afford security or proof; as, an additional or special mark put upon a package of goods belonging to several persons, that it may not be opened except in the presence of all; a mark added to that of an artificer of gold or silver work by the Goldsmiths' Company of London, to attest the standard quality of the gold or silver; a mark added to an ancient coin or medal, to show either its change of value or that it was taken from an enemy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Far.)</fld> <def>An artificial cavity made in the teeth of horses that have outgrown their natural mark, to disguise their age.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermark</ent><br/
<hw>Coun`ter*mark"</hw> <pr>(koun`t<etil/r*m<aum/rk")</pr>, <pos>v. t.</pos> <def>To apply a countermark to; <as>as, to <ex>countermark</ex> silverware; to <ex>countermark</ex> a horse's teeth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermine</ent><br/
<hw>Coun"ter*mine`</hw> <pr>(koun"t<etil/r*m<imac/n`)</pr>, <pos>n.</pos> <ety>[<ets>Counter-</ets> + <ets>mine</ets> underground gallery: cf. F. <ets>contermine</ets>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>An underground gallery excavated to intercept and destroy the mining of an enemy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stratagem or plot by which another sratagem or project is defeated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thinking himself contemned, knowing no <qex>countermine</qex> against contempt but terror.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermine</ent><br/
<hw>Coun`ter*mine"</hw> <pr>(koun`t<etil/r*m<imac/n")</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>contreminer</ets>.]</ety> <vmorph>[<pos>imp. & p. p.</pos>  <conjf>Countermined</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Countermining</conjf>.]</vmorph> <sn>1.</sn> <fld>(Mil.)</fld> <def>To oppose by means of a countermine; to intercept with a countermine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To frustrate or counteract by secret measures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermine</ent><br/
<hw>Coun`ter*mine"</hw>, <pos>v. i.</pos> <def>To make a countermine or counterplot; to plot secretly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'Tis hard for man to <qex>countermine</qex> with God.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermove</ent><br/
<hw>Coun`ter*move"</hw> <pr>(koun`t?r-m??v")</pr>, <pos>v. t. & i.</pos> <def>To move in a contrary direction to.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermovement</ent><br/
<ent>Countermove</ent><br/
<mhw>{ <hw>Coun"ter*move`</hw> <pr>(-m??v`)</pr>, <pos>n.</pos> <hw>Coun"ter*move`ment</hw> <pr>(-m<it>e</it>nt)</pr>. }</mhw> <def>A movement in opposition to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermure</ent><br/
<hw>Coun"ter*mure`</hw> <pr>(-m?r`)</pr>, <pos>n.</pos> <ety>[<ets>Counter-</ets> + <ets>mure</ets>: cf. F. <ets>contremur</ets>.]</ety> <fld>(Fort.)</fld> <def>A wall raised behind another, to supply its place when breached or destroyed. <mark>[R.]</mark> Cf. <er>Contramure</er>.</def>  <rj><au>Knolles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countermure</ent><br/
<hw>Coun`ter*mure"</hw> <pr>(koun`t?r-m?r")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countermured</conjf> <pr>(-m?rd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countermuring</conjf>.]</vmorph> <ety>[Cf. F. <ets>contremurer</ets>.]</ety> <def>To fortify with a wall behind another wall.</def> <mark>[R.]</mark>  <rj><au>Kyd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counternatural</ent><br/
<hw>Coun"ter*nat`u*ral</hw> <pr>(koun"t?r-n?t`?-r<it>a</it>l; 135)</pr>, <pos>a.</pos> <def>Contrary to nature.</def> <mark>[R.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter-paly</ent><br/
<hw>Coun"ter-pa`ly</hw> <pr>(-p?`l?)</pr>, <pos>a.</pos> <ety>[F. <ets>contre-pal<eacute/</ets>.]</ety> <fld>(Her.)</fld> <def>Paly, and then divided fesswise, so that each vertical piece is cut into two, having the colors used alternately or <xex>counterchanged</xex>. Thus the escutcheon in the illustration may also be blazoned <xex>paly of six per fess</xex> counterchanged <xex>argent and azure</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpane</ent><br/
<hw>Coun"ter*pane`</hw> <pr>(koun"t?r-p?n`)</pr>, <pos>n.</pos> <ety>[See <er>Counterpoint</er>, corrupted into counter<ets>pane</ets>, from the employment of <ets>pane-</ets>shaped figures in these coverlets. ]</ety> <def>A coverlet for a bed, -- originally stitched or woven in squares or figures.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On which a tissue <qex>counterpane</qex> was cast.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpane</ent><br/
<hw>Coun"ter*pane`</hw>, <pos>n.</pos> <ety>[OF. <ets>contrepan</ets> a pledge, security; <ets>contre + pan</ets> a skirt, also, a pawn or gage, F. <ets>pan</ets> a skirt. See <er>Pane</er>, and cf. <er>Pawn</er>.]</ety> <fld>(O. Law)</fld> <def>A duplicate part or copy of an indenture, deed, etc., corresponding with the original; -- now called <xex>counterpart</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Read, scribe; give me the <qex>counterpane</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 333 --></p>

<p><ent>Counterpart</ent><br/
<hw>Coun"ter*part`</hw> <pr>(koun"t<etil/r*p<aum/rt`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A part corresponding to another part; anything which answers, or corresponds, to another; a copy; a duplicate; a facsimile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In same things the laws of Normandy agreed with the laws of England, so that they seem to be, as it were, copies or <qex>counterparts</qex> one of another.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>One of two corresponding copies of an instrument; a duplicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person who closely resembles another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A thing may be applied to another thing so as to fit perfectly, as a seal to its impression; hence, a thing which is adapted to another thing, or which supplements it; that which serves to complete or complement anything; hence, a person or thing having qualities lacking in another; an opposite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>counterpart</qex><br/
Of our soft sex, well are you made our lords.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpassant</ent><br/
<hw>Coun"ter*pas`sant</hw> <pr>(-p?s"s<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[<ets>Counter-</ets> + <ets>passant</ets>: cf. F.  <ets>contrepassant</ets>.]</ety> <fld>(Her.)</fld> <def>Passant in opposite directions; -- said of two animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterplead</ent><br/
<hw>Coun`ter*plead"</hw> <pr>(koun`t?r-pl?d")</pr>, <pos>v. t.</pos> <def>To plead the contrary of; to plead against; to deny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterplot</ent><br/
<hw>Coun`ter*plot"</hw> <pr>(koun`t?r-pl?t")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterplotted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Counterplotting</conjf>.]</vmorph> <def>To oppose, as another plot, by plotting; to attempt to frustrate, as a stratagem, by stratagem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every wile had proved abortive, every plot had been <qex>counterplotted</qex>.</q> <rj><qau>De Quinsey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterplot</ent><br/
<hw>Coun"ter*plot`</hw> <pr>(koun"t<etil/r-pl?t`)</pr>, <pos>n.</pos> <def>A plot or artifice opposed to another.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpoint</ent><br/
<hw>Coun"ter*point`</hw> <pr>(koun"t?r-point`)</pr>, <pos>n.</pos> <ety>[<ets>Counter-</ets> + <ets>point</ets>.]</ety> <def>An opposite point</def> <mark>[Obs.]</mark>  <rj><au>Sir E. Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpoint</ent><br/
<hw>Coun"ter*point`</hw>, <pos>n.</pos> <ety>[F. <ets>contrepoint</ets>; cf. It. <ets>contrappunto</ets>. Cf. <er>Contrapuntal</er>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The setting of note against note in harmony; the adding of one or more parts to a given <cref>canto fermo</cref> or melody</def>. <sd>(b)</sd> <def>The art of polyphony, or composite melody, <it>i. e.</it>, melody not single, but moving attended by one or more related melodies.</def> <sd>(c)</sd> <def>Music in parts; part writing; harmony; polyphonic music. See <er>Polyphony</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Counterpoint</qex>, an invention equivalent to a new creation of music.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpoint</ent><br/
<hw>Coun"ter*point`</hw>, <pos>n.</pos> <ety>[OF.  <ets>contrepoincte</ets>, corruption of earlier <ets>counstepointe</ets>, <ets>countepointe</ets>, F. <ets>courtepointe</ets>, fr. L.  <ets>culcita</ets> cushion, mattress (see <er>Quilt</er>, and cf. <er>Cushion</er>) +  <ets>puncta</ets>, fem. p. p. of <ets>pungere</ets> to prick (see <er>Point</er>). The word properly meant a stitched quilt, with the colors broken one into another.]</ety> <def>A coverlet; a cover for a bed, often stitched or broken into squares; a counterpane.  See 1st <er>Counterpane</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Embroidered coverlets or <qex>counterpoints</qex> of purple silk.</q> <rj><qau>Sir T. North.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpoise</ent><br/
<hw>Coun"ter*poise`</hw> <pr>(koun"t?r-poiz`; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterpoised</conjf> <pr>(-poizd`)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Counterpoising</conjf>.]</vmorph> <ety>[OE.  <ets>countrepesen</ets>, <ets>counterpeisen</ets>, F.  <ets>contrepeser</ets>. See <er>Counter</er>, <pos>adv.</pos>, and <er>Poise</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>To act against with equal weight; to equal in weight; to balance the weight of; to counterbalance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Weights, <qex>counterpoising</qex> one another.</q> <rj><qau>Sir K. Digby.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To act against with equal power; to balance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So many freeholders of English will be able to beard and <qex>counterpoise</qex> the rest.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpoise</ent><br/
<hw>Coun"ter*poise`</hw> <pr>(koun"t?r-poiz`)</pr>, <pos>n.</pos> <ety>[OE. <ets>countrepese</ets>, OF.  <ets>contrepois</ets>, F.  <ets>contrepods</ets>. See <er>Counter</er>, <ets>adv</ets>., and <er>Poise</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A weight sufficient to balance another, as in the opposite scale of a balance; an equal weight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fastening that to our exact balance, we put a metalline <qex>counterpoise</qex> into the opposite scale.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An equal power or force acting in opposition; a force sufficient to balance another force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The second nobles are a <qex>counterpoise</qex> to the higher nobility, that they grow not too potent.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The relation of two weights or forces which balance each other; equilibrium; equiponderance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pendulous round eart, with balanced air,<br/
In <qex>counterpoise</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterpole</ent><br/
<hw>Coun"ter*pole`</hw> <pr>(-p?l`)</pr>, <pos>n.</pos> <def>The exact opposite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The German prose offers the <qex>counterpole</qex> to the French style.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterponderate</ent><br/
<hw>Coun`ter*pon"der*ate</hw> <pr>(-p?n"d?r-?t)</pr>, <pos>v. t.</pos> <def>To equal in weight; to counterpoise; to equiponderate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterprove</ent><br/
<hw>Coun`ter*prove"</hw> <pr>(koun`t?r-pr??v")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Counterproved</conjf> <pr>(-pr??vd")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Counterproving</conjf>.]</vmorph> <def>To take a counter proof of, or a copy in reverse, by taking an impression directly from the face of an original.  See <cref>Counter proof</cref>, under <er>Counter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter-roll</ent><br/
<hw>Coun"ter-roll`</hw> <pr>(-r?l`)</pr>, <pos>n.</pos> <ety>[Cf. <er>Control</er>.]</ety> <fld>(O. Eng. Law)</fld> <def>A duplicate roll (record or account) kept by an officer as a check upon another officer's roll.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ As a verb this word is contracted into <xex>control</xex>. See <er>Control</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterrolment</ent><br/
<hw>Coun`ter*rol"ment</hw> <pr>(koun`t<etil/r-r?l"m<it>e</it>nt)</pr>, <pos>n.</pos> <def>A counter account. See <er>Control</er>.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>counter-sabotage</ent><br/
<hw>counter-sabotage</hw> <pos>n.</pos> <def>counterintelligence designed to detect and counteract sabotage.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Counter-salient</ent><br/
<hw>Coun`ter-sa"li*ent</hw> <pr>(koun`t<etil/r-s<amac/"l<icr/*<eit/>nt <or/ -s<amac/l"y<eit/nt; 106)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Leaping from each other; -- said of two figures on a coat of arms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterscale</ent><br/
<hw>Coun"ter*scale`</hw> <pr>(koun`t<etil/r-sk<amac/l`)</pr>, <pos>n.</pos> <def>Counterbalance; balance, as of one scale against another.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterscarf</ent><br/
<hw>Coun"ter*scarf`</hw> <pr>(koun`t<etil/r*sk<aum/rf`)</pr>, <pos>n.</pos> <def>See <er>counterscarp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterscarp</ent><br/
<hw>Coun"ter*scarp`</hw> <pr>(koun`t<etil/r*sk<aum/rp`)</pr> <pos>n.</pos> <ety>[<ets>Counter-</ets> + <ets>scarp</ets>: cf. F. <ets>contrescarpe</ets>.]</ety> <fld>(Fort.)</fld> <def>The exterior slope or wall of the ditch; -- sometimes, the whole covered way, beyond the ditch, with its parapet and glacis; <as>as, the enemy have lodged themselves on the <ex>counterscarp</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterseal</ent><br/
<hw>Coun`ter*seal"</hw> <pr>(koun`t<etil/r*s<emac/l")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countersealed</conjf> <pr>(koun`t<etil/r*s<emac/ld")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countersealing</conjf>.]</vmorph> <def>To seal or ratify with another or others.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countersecure</ent><br/
<hw>Coun`ter*se*cure"</hw> <pr>(-s?-k?r")</pr>, <pos>v. t.</pos> <def>To give additional security to or for.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countershaft</ent><br/
<hw>Coun"ter*shaft`</hw> <pr>(koun"t?r-sh?ft`)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>An intermediate shaft; esp., one which receives motion from a line shaft in a factory and transmits it to a machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countersign</ent><br/
<hw>Coun`ter*sign"</hw> <pr>(-s?n`; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countersigned</conjf> <pr>(-s?nd`)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countersigning</conjf>.]</vmorph> <ety>[<ets>Counter-</ets> + <ets>sign</ets>: cf. F.  <ets>contresigner</ets>.]</ety> <def>To sign on the opposite side of (an instrument or writing); hence, to sign in addition to the signature of a principal or superior, in order to attest the authenticity of a writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countersign</ent><br/
<hw>Coun"ter*sign`</hw>, <pos>a.</pos> <sn>1.</sn> <def>The signature of a secretary or other officer to a writing signed by a principal or superior, to attest its authenticity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A private signal, word, or phrase, which must be given in order to pass a sentry; a watchword.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>countersignature</ent><br/
<hw>countersignature</hw> <pos>n.</pos> <def>a second confirming signature endorsing a document already signed.</def><br/
<syn><b>Syn. --</b> countersign.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Countersink</ent><br/
<hw>Coun"ter*sink`</hw> <pr>(koun"t<etil/r*s<icr/<nsm/k`; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos>  <conjf>Countersunk</conjf> <pr>(-s<ucr/<nsm/k`)</pr>; <pos>p. pr. & vb. n.</pos>  <conjf>Countersinking</conjf>.]</vmorph> <sn>1.</sn> <def>To chamfer or form a depression around the top of (a hole in wood, metal, etc.) for the reception of the head of a screw or bolt below the surface, either wholly or in part; <as>as, to <ex>countersink</ex> a hole for a screw</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to sink even with or below the surface; <as>as, to <ex>countersink</ex> a screw or bolt into woodwork</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countersink</ent><br/
<hw>Coun"ter*sink`</hw>, <pos>n.</pos> <sn>1.</sn> <def>An enlargement of the upper part of a hole, forming a cavity or depression for receiving the head of a screw or bolt.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States a flaring cavity formed by chamfering the edges of a round hole is called a <xex>countersink</xex>, while a cylindrical flat-bottomed enlargement of the mouth of the hole is usually called <stype>a conterbore</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A drill or cutting tool for countersinking holes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterstand</ent><br/
<hw>Coun"ter*stand`</hw> <pr>(-st<?/nd`)</pr>, <pos>n.</pos> <def>Resistance; opposition; a stand against.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Making <qex>counterstand</qex> to Robert Guiscard.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterstep</ent><br/
<hw>Coun"ter*step`</hw> <pr>(koun"t?r-st?p`)</pr>, <pos>n.</pos> <def>A contrary method of procedure; opposite course of action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterstock</ent><br/
<hw>Coun"ter*stock`</hw> <pr>(-st?k`)</pr>, <pos>n.</pos> <def>See <er>Counterfoil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterstroke</ent><br/
<hw>Coun"ter*stroke`</hw> <pr>(-str?k`)</pr>, <pos>n.</pos> <def>A stroke or blow in return.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>counter-subversion</ent><br/
<hw>counter-subversion</hw> <pos>n.</pos> <def>the aspect of counterintelligence designed to detect and prevent subversive activities.</def><br/
<syn><b>Syn. --</b> countersubversion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Countersunk</ent><br/
<hw>Coun"ter*sunk`</hw> <pr>(-s?nk`)</pr>, <pos>p. p. & a.</pos> <mord>from <er>Countersink</er></mord>. <sn>1.</sn> <def>Chamfered at the top; -- said of a hole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sunk into a chamfer; <as>as, a <ex>countersunk</ex> bolt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Beveled on the lower side, so as to fit a chamfered countersink; <as>as, a <ex>countersunk</ex> nailhead</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countersway</ent><br/
<hw>Coun"ter*sway`</hw> <pr>(-sw<amac/`)</pr>, <pos>n.</pos> <def>A swaying in a contrary direction; an opposing influence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>countersway</qex> of restraint, curbing their wild exorbitance.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter tenor</ent><br/
<hw>Coun"ter ten`or</hw> <pr>(t?n`?r)</pr>. <ety>[OF. <ets>contreteneur</ets>. Cf. <er>Contratenor</er>, and see <er>Tenor</er> a part in music.]</ety> <fld>(Mus.)</fld> <def>One of the middle parts in music, between the tenor and the treble; high tenor.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Counter-tenor clef</b></col> <fld>(Mus.)</fld>, <cd>the C clef when placed on the third line; -- also called <altname>alto clef</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterterm</ent><br/
<hw>Coun"ter*term`</hw> <pr>(-t?rm`)</pr>, <pos>n.</pos> <def>A term or word which is the opposite of, or antithesis to, another; an antonym; -- the opposite of <xex>synonym</xex>; <as>as, <ldquo/foe<rdquo/ is the <ex>counterterm</ex> of <ldquo/friend<rdquo/</as>.</def>  <rj><au>C. J. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countertime</ent><br/
<hw>Coun"ter*time`</hw> <pr>(-t?m`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Man.)</fld> <def>The resistance of a horse, that interrupts his cadence and the measure of his manege, occasioned by a bad horseman, or the bad temper of the horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resistance; opposition.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Give not shus the <qex>countertime</qex> to fate.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countertrippant</ent><br/
<hw>Coun"ter*trip`pant</hw> <pr>(-tr?p`p<it>a</it>nt)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Trippant in opposite directions. See <er>Trippant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countertripping</ent><br/
<hw>Coun"ter*trip`ping</hw> <pr>(-tr?p`p?ng)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Same as <er>Countertrippant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterturn</ent><br/
<hw>Coun"ter*turn`</hw> <pr>(-t<?/rn`)</pr>, <pos>n.</pos> <def>The critical moment in a play, when, contrary to expectation, the action is embroiled in new difficulties.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countervail</ent><br/
<hw>Coun`ter*vail"</hw> <pr>(koun`t?r-v?l")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Countervailed</conjf> <pr>(-v?ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Countervailing</conjf>.]</vmorph> <ety>[OF.  <ets>contrevaloir</ets>; <ets>contre</ets> (L. <ets>contra</ets>) +  <ets>valoir</ets> to avail, fr. L.  <ets>valere</ets> to be strong, avail. See <er>Vallant</er>.]</ety> <def>To act against with equal force, power, or effect; to thwart or overcome by such action; to furnish an equivalent to or for; to counterbalance; to compensate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon balancing the account, the profit at last will hardly <qex>countervail</qex> the inconveniences that go allong with it.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countervail</ent><br/
<hw>Coun"ter*vail`</hw> <pr>(koun"t?r-v?l`)</pr>, <pos>n.</pos> <def>Power or value sufficient to obviate any effect; equal weight, strength, or value; equivalent; compensation; requital.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Surely, the present pleasure of a sinful act is a poor <qex>countervail</qex> for the bitterness of the review.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countervallation</ent><br/
<hw>Coun`ter*val*la"tion</hw> <pr>(-v?l-l?"s??n)</pr>, <pos>n.</pos> <fld>(Fort.)</fld> <def>See <er>Contravallation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterview</ent><br/
<hw>Coun"ter*view`</hw> <pr>(koun"t?r-v?`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An opposite or opposing view; opposition; a posture in which two persons front each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Within the gates of hell sat Death and Sin,<br/
In <qex>counterview</qex>.</q> <rj><qau>Milton</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>M. Peisse has ably advocated the <qex>counterview</qex> in his preface and appendix.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A position in which two dissimilar things illustrate each other by opposition; contrast.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have drawn some lines of Linger's character, on purpose to place it in <qex>counterview</qex>, or contrast with that of the other company.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countervote</ent><br/
<hw>Coun`ter*vote"</hw> <pr>(koun`t<etil/r*v<omac/t")</pr>, <pos>v. t.</pos> <def>To vote in opposition to; to balance or overcome by voting; to outvote.</def>  <rj><au>Dr. J. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterwait</ent><br/
<hw>Coun`ter*wait"</hw> <pr>(koun`t<etil/r*w<amac/t")</pr>, <pos>v. t.</pos> <def>To wait or watch for; to be on guard against.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><-- the original pronunc. had a plain "e" in place of the "<etil/". --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterweigh</ent><br/
<hw>Coun`ter*weigh"</hw> <pr>(-w<amac/")</pr>, <pos>v. t.</pos> <def>To weigh against; to counterbalance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counter weight</ent><br/
<hw>Coun"ter *weight`</hw> <pr>(-w<amac/t`)</pr>, <pos>n.</pos> <def>A counterpoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterwheel</ent><br/
<hw>Coun`ter*wheel"</hw> <pr>(-hw<emac/l")</pr>, <pos>v. t.</pos> <fld>(Mil.)</fld> <def>To cause to wheel or turn in an opposite direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Counterwork</ent><br/
<hw>Coun`ter*work"</hw> <pr>(-w<ucir/rk")</pr>, <pos>v. t.</pos> <def>To work in opposition to; to counteract.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>counterworks</qex> each folly and caprice.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countess</ent><br/
<hw>Count"ess</hw> <pr>(kount"?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Countesses</plw> <pr>(-<?/s)</pr>.</plu> <ety>[F.  <ets>comtesse</ets>.  See <er>Count</er> a nobleman.]</ety> <def>The wife of an earl in the British peerage, or of a count in the Continental nobility; also, a lady possessed of the same dignity in her own right. See the Note under <er>Count</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countingroom</ent><br/
<ent>Countinghouse</ent><br/
<mhw>{ <hw>Count"ing*house`</hw> <pr>(kount"?ng-hous`)</pr>, <hw>Count"ing*room`</hw> <pr>(kount"?ng-r??m`)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Count</er>, <pos>v.</pos>]</ety> <def>The house or room in which a merchant, trader, or manufacturer keeps his books and transacts business; the offices used by the accountants of a business.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Countless</ent><br/
<hw>Count"less</hw> <pr>(-l?s)</pr>, <pos>a.</pos> <def>Incapable of being counted; not ascertainable; innumerable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countor</ent><br/
<hw>Count"or</hw> <pr>(kount"?r)</pr>, <pos>n.</pos> <ety>[From <er>Count</er>, <pos>v. t.</pos> (in sense 4).]</ety> <fld>(O. Eng. Law)</fld> <def>An advocate or professional pleader; one who counted for his client, that is, orally pleaded his cause.</def> <mark>[Obs.]</mark>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countourhouse</ent><br/
<ent>Countour</ent><br/
<mhw>{ <hw>Coun*tour"</hw> <pr>(k??n-t??r")</pr>, <hw>Coun*tour"house`</hw> <pr>(-hous`)</pr>, }</mhw> <pos>n.</pos> <ety>[See 2d <er>Counter</er>.]</ety> <def>A merchant's office; a countinghouse.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countre-</ent><br/
<hw>Coun"tre-</hw> <pr>(koun"ter-)</pr>. <def>Same as prefix <er>Counter-</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countreplete</ent><br/
<hw>Coun`tre*plete"</hw> <pr>(-pl?t")</pr>, <pos>v. t.</pos> <ety>[<ets>Countre-</ets> + <ets>plete</ets> to plead.]</ety> <def>To counterplead.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countretaille</ent><br/
<hw>Coun"tre*taille`</hw> <pr>(koun"t?r-t?l`)</pr>, <pos>n.</pos> <ety>[F. <ets>contretaille</ets>; <ets>contre</ets> (L. <ets>contra</ets>) +  <ets>taille</ets> cut. See <er>Tally</er>.]</ety> <def>A counter tally; correspondence (in sound).</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>At the countretaille</b></col>, <cd>in return.</cd>  <rj><au>Chaucer.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countrified</ent><br/
<hw>Coun"tri*fied</hw> <pr>(k<ucr/n"tr<icr/*f<imac/d)</pr>, <pos>p. a.</pos> <altsp>[Also spelled <asp>countryfied</asp>.]</altsp> <def>Having the appearance and manners of a rustic; rude; <as>as, <ex>countrified</ex> clothes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As being one who took no pride,<br/
And was a deal too <qex>countrified</qex>.</q> <rj><qau>Lloyd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>rendered in a manner resembling rural style; <as>as, <ex>countrified</ex> rock music</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn>  <def>unsophisticated.</def><br/
[<source>PJC</source>]</p>

<p><ent>Countrify</ent><br/
<hw>Coun"tri*fy</hw> <pr>(k?n"tr?-f?)</pr>, <pos>v. t.</pos> <def>To give a rural appearance to; to cause to appear rustic.</def>  <rj><au>Lamb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Country</ent><br/
<hw>Coun"try</hw> <pr>(k?n"tr?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Countries</plw> <pr>(-tr<?/z)</pr>.</plu> <ety>[F. <ets>contr<eacute/e</ets>, LL. <ets>contrata</ets>, fr. L.  <ets>contra</ets> over against, on the opposite side. Cf. <er>Counter</er>, <pos>adv.</pos>, <er>Contra</er>.]</ety> <sn>1.</sn> <def>A tract of land; a region; the territory of an independent nation; (as distinguished from any other region, and with a personal pronoun) the region of one's birth, permanent residence, or citizenship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Return unto thy <qex>country</qex>, and to thy kindred.</q> <rj><qau>Gen. xxxxii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I might have learned this by my last exile,<br/
that change of <qex>countries</qex> cannot change my state.</q> <rj><qau>Stirling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a famous realm<br/
And <qex>country</qex>, whereof here needs no account</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rural regions, as opposed to a city or town.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As they walked, on their way into the <qex>country</qex>.</q> <rj><qau>Mark xvi. 12 (Rev. Ver. ).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God made the <qex>covatry</qex>, and man made the town.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Only very great men were in the habit of dividing the year between town and <qex>country</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The inhabitants or people of a state or a region; the populace; the public. Hence: <sd>(a)</sd> One's constituents. <sd>(b)</sd> The whole body of the electors of state; <as>as, to dissolve Parliament and appeal to the <ex>country</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the <qex>country</qex> in a general voice<br/
Cried hate upon him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>A jury, as representing the citizens of a country.</def> <sd>(b)</sd> <def>The inhabitants of the district from which a jury is drawn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mining.)</fld> <def>The rock through which a vein runs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Conclusion to the country</b></col>. <cd>See under <er>Conclusion</er>.</cd> -- <mcol><col><b>To put one's self upon the country</b></col>, <it>or</it> <col><b>To throw one's self upon the country</b></col></mcol>, <cd>to appeal to one's constituents; to stand trial before a jury.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Country</ent><br/
<hw>Coun"try</hw>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to the regions remote from a city; rural; rustic; <as>as, a <ex>country</ex> life; a <ex>country</ex> town; the <ex>country</ex> party, as opposed to city</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Destitute of refinement; rude; unpolished; rustic; not urbane; <as>as, <ex>country</ex> manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pertaining, or peculiar, to one's own country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She, bowing herself towards him, laughing the cruel tyrant to scorn, spake in her <qex>country</qex> language.</q> <rj><qau>2 Macc. vii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Country bank</ent><br/
<hw>Coun"try bank</hw>. <fld>(Banking)</fld> <def>A national bank not in a reserve city.</def> <mark>[Colloq., U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Country-base</ent><br/
<hw>Coun"try-base`</hw> <pr>(-b?s`)</pr>, <pos>n.</pos> <def>Same as <er>Prison base</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Country club</ent><br/
<hw>Coun"try club</hw>. <def>A club usually located in the suburbs or vicinity of a city or town and devoted mainly to outdoor sports, and usually having an attached golf course.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Country cousin</ent><br/
<hw>Coun"try cousin</hw>. <def>A relative from the country visiting the city and unfamiliar with city manners and sights.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Country-dance</ent><br/
<hw>Coun"try-dance`</hw> <pr>(-d?ns`)</pr>, <pos>n.</pos> <ety>[Prob. an adaptation of <ets>contradance</ets>.]</ety> <def>See <er>Contradance</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had introduced the English <qex>country-dance</qex> to the knowledge of the Dutch ladies.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>countryfied</ent><br/
<hw>countryfied</hw> <pos>adj.</pos> <def>same as <er>countrified</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Countryman</ent><br/
<hw>Coun"try*man</hw> <pr>(k<ucr/n"tr<icr/-m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Countrymen</plw> <pr>(-m<eit/n)</pr>.</plu> <sn>1.</sn> <def>An inhabitant or native of a region.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One born in the same country with another; a compatriot; -- used with a possessive pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In perils of waters, in perils of robbers, in perils by mine own <qex>countrymen</qex>.</q> <rj><qau>2 Cor. xi. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who dwells in the country, as distinguished from a townsman or an inhabitant of a city; a rustic; a husbandman or farmer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A simple <qex>countryman</qex> that brought her figs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Country seat</ent><br/
<hw>Coun"try seat`</hw> <pr>(k?n"tr? s?t`)</pr>. <def>A dwelling in the country, used as a place of retirement from the city.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Countryside</ent><br/
<hw>Coun"try*side`</hw> <pr>(-s?d`)</pr>, <pos>n.</pos> <def>A particular rural district; a country neighborhood.</def> <mark>[Eng.]</mark>  <rj><au>W. Black. Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>country-style</ent><br/
<hw>country-style</hw> <pos>adj.</pos> <def>of a style associated with rural areas; <as>as, <ex>country-style</ex> sausage</as>.</def><br/
<syn><b>Syn. --</b> country(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Countrywoman</ent><br/
<hw>Coun"try*wom`an</hw> <pr>(-w??m`<it>a</it>n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Countrywomen</plw> <pr>(-w<?/m`<?/n)</pr>.</plu> <def>A woman born, or dwelling, in the country, as opposed to the city; a woman born or dwelling in the same country with another native or inhabitant.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Count-wheel</ent><br/
<hw>Count"-wheel`</hw> <pr>(kount"hw?l`)</pr>, <pos>n.</pos> <def>The wheel in a clock which regulates the number of strokes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>countrywide</ent><br/
<hw>countrywide</hw> <pos>adj.</pos>  <def>extending throughout a country or nation as a whole; <as>as, a <ex>countrywide</ex> fund-raising campaign</as>.</def><br/
<syn><b>Syn. --</b> nationwide.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>County</ent><br/
<hw>Coun"ty</hw> <pr>(koun"t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Counties</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[F. <ets>comt<eacute/</ets>, fr. LL.  <ets>comitatus</ets>. See <er>Count</er>.]</ety> <sn>1.</sn> <def>An earldom; the domain of a count or earl.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A circuit or particular portion of a state or kingdom, separated from the rest of the territory, for certain purposes in the administration of justice and public affairs; -- called also a <altname>shire</altname>. See <er>Shire</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every <qex>county</qex>, every town, every family, was in agitation.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 334 --></p>

<p><sn>3.</sn> <def>A count; an earl or lord.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>County commissioners</b></col>. <cd>See <er>Commissioner</er>.</cd> -- <col><b>County corporate</b></col>, <cd>a city or town having the privilege to be a county by itself, and to be governed by its own sheriffs and other magistrates, irrespective of the officers of the county in which it is situated; as London, York, Bristol, etc.</cd> <mark>[Eng.]</mark> <au>Mozley & W.</au> -- <col><b>County court</b></col>, <cd>a court whose jurisdiction is limited to county.</cd> -- <col><b>County palatine</b></col>, <cd>a county distinguished by particular privileges; -- so called <xex>a palatio</xex> (from the palace), because the owner had originally royal powers, or the same powers, in the administration of justice, as the king had in his palace; but these powers are now abridged. The counties palatine, in England, are Lancaster, Chester, and Durham.</cd> -- <col><b>County rates</b></col>, <cd>rates levied upon the county, and collected by the boards of guardians, for the purpose of defraying the expenses to which counties are liable, such as repairing bridges, jails, etc.</cd>  <mark>[Eng.]</mark> -- <col><b>County seat</b></col>, <cd>a county town</cd>. <mark>[U.S.]</mark> -- <col><b>County sessions</b></col>, <cd>the general quarter sessions of the peace for each county, held four times a year.</cd> <mark>[Eng.]</mark> -- <col><b>County town</b></col>, <cd>the town of a county, where the county business is transacted; a shire town.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>countywide</ent><br/
<hw>countywide</hw> <pos>adj.</pos> <def>including or occurring in all parts of a county; <as>as, a <ex>countywide</ex> war on drugs; <ex>countywide</ex> elections</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coup</ent><br/
||<hw>Coup</hw> <pr>(k<oomac/)</pr>, <pos>n.</pos> <ety>[F., fr.L.  <ets>colaphus</ets> a cuff, Gr. <grk>ko`lafos</grk>.]</ety> <sn>1.</sn> <def>A sudden stroke delivered with promptness and force; -- used also in various ways to convey the idea of an unexpected, clever, and successful tactic or stratagem.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A single roll of the wheel at roulette, or a deal at rouge et noir.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>Among some tribes of North American Indians especially of the Great Plains, the act of striking or touching an enemy in warfare with the hand or at close quarters, as with a short stick, in such a manner as by custom to entitle the doer to count the deed an act of bravery; hence, any of various other deeds recognized by custom as acts of bravery or honor.</def></p>

<p><q>While the <qex>coup</qex> was primarily, and usually, a blow with something held in the hand, other acts in warfare which involved great danger to him who performed them were also reckoned <qex>coups</qex> by some tribes.</q>  <rj><qau>G. B. Grinnell.</qau></rj></p>

<p><q>Among the Blackfeet the capture of a shield, bow, gun, war bonnet, war shirt, or medicine pipe was deemed a <qex>coup</qex>.</q>  <rj><qau>G. B. Grinnell.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Coup de grace</b></col> <pr>(k<oomac/` d<eit/ gr<adot/s")</pr> <ety>[F.]</ety>, <cd>the stroke of mercy with which an executioner ends by death the sufferings of the condemned; hence, a decisive, finishing stroke.</cd> -- <col><b>Coup de main</b></col> <pr>(k<oomac/` d<eit/ m<acr/N")</pr> <ety>[F.]</ety> <fld>(Mil.)</fld>, <cd>a sudden and unexpected movement or attack.</cd> -- <col><b>Coup de soleil</b></col> <pr>(k<omac/` d<eit/ s<osl/*l<asl/l <or/ -l<asl/"y')</pr> <ety>[F.]</ety> <fld>(Med.)</fld>, <cd>a sunstroke. See <er>Sunstroke</er>.</cd> -- <col><b>Coup d'<eacute/tat</b></col> <pr>(k<oomac/" d<asl/*t<aum/")</pr> <ety>[F.]</ety> <fld>(Politics)</fld>, <cd>a sudden, decisive exercise of power whereby the existing government is subverted without the consent of the people; an unexpected measure of state, more or less violent; a stroke of policy.</cd> -- <col><b>Coup d'<oe/il</b></col> <pr>(k<oomac/` d<etil/l")</pr>. <ety>[F.]</ety> <sd>(a)</sd> <cd>A single view; a rapid glance of the eye; a comprehensive view of a scene; as much as can be seen at one view.</cd> <sd>(b)</sd> <cd>The general effect of a picture.</cd> <sd>(c)</sd> <fld>(Mil.)</fld> <cd>The faculty or the act of comprehending at a glance the weakness or strength of a military position, of a certain arrangement of troops, the most advantageous position for a battlefield, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coup</ent><br/
<hw>Coup</hw>. <pos>v. i.</pos> <def>To make a coup.</def></p>

<p><q>Woe to the Sioux if the Northern Cheyennes get a chance to <qex>coup</qex> !</q>  <rj><qau>F. Remington.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coupable</ent><br/
<hw>Cou"pa*ble</hw> <pr>(k<oomac/"p<adot/*b'l)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>Culpable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coupe</ent><br/
||<hw>Cou`p<eacute/"</hw> <pr>(k<oomac/`p<asl/")</pr>, <pos>n.</pos> <ety>[F., fr.  <ets>coup<eacute/</ets>, p. p. of <ets>couper</ets> to cut. See <er>Coppice</er>.]</ety> <sn>1.</sn> <def>The front compartment of a French diligence; also, the front compartment (usually for three persons) of a car or carriage on British railways.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A four-wheeled enclosed carriage for two persons inside, with an outside seat for the driver; -- so called because giving the appearance of a larger carriage cut off.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>a two-door automobile with front seats and a luggage compartment.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Couped</ent><br/
<hw>Couped</hw> <pr>(k<oomac/pt)</pr>, <pos>a.</pos> <ety>[F.  <ets>couper</ets> to cut.]</ety> <fld>(Her.)</fld> <def>Cut off smoothly, as distinguished from erased; -- used especially for the head or limb of an animal. See <er>Erased</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coupee</ent><br/
<hw>Cou*pee"</hw> <pr>(k<oomac/*p<emac/"; F. k<oomac/`p<asl/")</pr>, <pos>n.</pos> <ety>[F. coup<eacute/, n., properly p. p. of couper to cut. Cf. <er>Coup<eacute/</er>, <er>Coopee</er>.]</ety> <def>A motion in dancing, when one leg is a little bent, and raised from the floor, and with the other a forward motion is made.</def>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coupe-gorge</ent><br/
||<hw>Coupe`-gorge"</hw> <pr>(k<oomac/p`g<ocir/rzh")</pr>, <pos>n.</pos> <ety>[F., cut throat.]</ety> <fld>(Mil.)</fld> <def>Any position giving the enemy such advantage that the troops occupying it must either surrender or be cut to pieces.</def>  <rj><au>Farrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couple</ent><br/
<hw>Cou"ple</hw> <pr>(k<ucr/p"'l)</pr>, <pos>n.</pos> <ety>[F. <ets>couple</ets>, fr. L. <ets>copula</ets> a bond, band; <ets>co-</ets> + <ets>apere</ets>, <ets>aptum</ets>, to join. See <er>Art</er>, <pos>a.</pos>, and cf. <er>Copula</er>.]</ety> <sn>1.</sn> <def>That which joins or links two things together; a bond or tie; a coupler.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is in some sort with friends as it is with dogs in <qex>couples</qex>; <qex>they should be of the same size and humor</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll go in <qex>couples</qex> with her.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Two of the same kind connected or considered together; a pair; a brace.</def> <ldquo/A <xex>couple</xex> of shepherds.<rdquo/  <au>Sir P. Sidney.</au>  <ldquo/A <xex>couple</xex> of drops<rdquo/ <au>Addison.</au>  <ldquo/A <xex>couple</xex> of miles.<rdquo/ <au>Dickens.</au> <ldquo/A <xex>couple</xex> of weeks.<rdquo/ <au>Carlyle.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Adding one to one we have the complex idea of a <qex>couple</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Ziba] met him with a <qex>couple</qex> of asses saddled.</q> <rj><qau>2 Sam. xvi. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A male and female associated together; esp., a man and woman who are married or betrothed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such were our <qex>couple</qex>, man and wife.</q> <rj><qau>Lloyd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fair <qex>couple</qex> linked in happy, nuptial league.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch.)</fld> <def>See <er>Couple-close</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Elec.)</fld> <def>One of the pairs of plates of two metals which compose a voltaic battery; -- called a <altname>voltaic couple</altname> or <altname>galvanic couple</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mech.)</fld> <def>Two rotations, movements, etc., which are equal in amount but opposite in direction, and acting along parallel lines or around parallel axes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The effect of a <xex>couple</xex> of forces is to produce a rotation. A <xex>couple</xex> of rotations is equivalent to a motion of translation.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couple</ent><br/
<hw>Cou"ple</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coupled</conjf> <pr>(k<ucr/p"'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coupling</conjf> <pr>(k<ucr/p"l<icr/ng)</pr>.]</vmorph> <ety>[F. <ets>coupler</ets>, fr. L.  <ets>copulare</ets>. See <er>Couple</er>, <pos>n.</pos>, and cf. <er>Copulate</er>, <er>Cobble</er>, <pos>v.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To link or tie, as one thing to another; to connect or fasten together; to join.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Huntsman, I charge thee, tender well my hounds, . . . <br/
And <qex>couple</qex> Clowder with the deep-mouthed brach.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To join in wedlock; to marry.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A parson who <qex>couples</qex> all our beggars.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couple</ent><br/
<hw>Cou"ple</hw>, <pos>v. i.</pos> <def>To come together as male and female; to copulate.</def> <mark>[Obs.]</mark>  <rj><au>Milton. Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couple-beggar</ent><br/
<hw>Cou"ple-beg`gar</hw> <pr>(-b?g`g?r)</pr>, <pos>n.</pos> <def>One who makes it his business to marry beggars to each other.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couple-close</ent><br/
<hw>Cou"ple-close`</hw> <pr>(k<ucr/p"'l*kl<omac/s`)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Couple-closes</plw> (k<ucr/p"'l*kl<omac/"s<ecr/z).</plu> <sn>1.</sn> <fld>(Her.)</fld> <def>A diminutive of the chevron, containing one fourth of its surface. Couple-closes are generally borne one on each side of a chevron, and the  blazoning may then be either a chevron between two couple-closes or chevron cottised.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Arch.)</fld> <def>A pair of rafters framed together with a tie fixed at their feet, or with a collar beam.</def> <mark>[Engl.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>coupled</ent><br/
<hw>coupled</hw> <pos>adj.</pos> <sn>1.</sn>  <def>joined together especially in a pair or pairs.</def><br/
<syn><b>Syn. --</b> conjugate, conjugated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>connected by a physical link, as railway cars or trailer trucks.  Opposite of <ant>unconnected</ant> or <ant>unlinked</ant></def><br/
<syn><b>Syn. --</b> joined, linked.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Couplement</ent><br/
<hw>Cou"ple*ment</hw> <pr>(k?p"'l-m<it>e</it>nt)</pr>, <pos>n.</pos>  <ety>[Cf. OF.  <ets>couplement</ets>.]</ety> <def>Union; combination; a coupling; a pair.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And forth together rode, a goodly <qex>couplement</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coupler</ent><br/
<hw>Coup"ler</hw> <pr>(k?p"l?r)</pr>, <pos>n.</pos> <def>One who couples; that which couples, as a link, ring, or shackle, to connect cars.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coupler of an organ</b></col>, <cd>a contrivance by which any two or more of the ranks of keys, or keys and pedals, are connected so as to act together when the organ is played.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couplet</ent><br/
<hw>Coup"let</hw> <pr>(-l?t)</pr>, <pos>n.</pos> <ety>[F.  <ets>couplet</ets>, dim. of <ets>couple</ets>.  See <er>Couple</er>, <pos>n.</pos> ]</ety> <def>Two taken together; a pair or couple; especially two lines of verse that rhyme with each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sudden <qex>couplet</qex> rushes on your mind.</q> <rj><qau>Crabbe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coupling</ent><br/
<hw>Coup"ling</hw> <pr>(-l?ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of bringing or coming together; connection; sexual union.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>A device or contrivance which serves to couple or connect adjacent parts or objects; <as>as, a belt <ex>coupling</ex>, which connects the ends of a belt; a car <ex>coupling</ex>, which connects the cars in a train; a shaft <ex>coupling</ex>, which connects the ends of shafts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Box coupling</b></col>, <col><b>Chain coupling</b></col></mcol>. <cd>See under <er>Box</er>, Chain.</cd> -- <col><b>Coupling box</b></col>, <cd>a coupling shaped like a journal box, for clamping together the ends of two shafts, so that they may revolve together.</cd> -- <col><b>Coupling pin</b></col>, <cd>a pin or bolt used in coupling or joining together railroad cars, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coupon</ent><br/
<hw>Cou"pon</hw> <pr>(k??"p?n; F. k??`p?n")</pr>, <pos>n.</pos> <ety>[F., fr.  <ets>couper</ets> to cut, cut off. See <er>Coppice</er>.]</ety> <sn>1.</sn> <fld>(Com.)</fld> <def>A certificate of interest due, printed at the bottom of transferable bonds (state, railroad, etc.), given for a term of years, designed to be cut off and presented for payment when the interest is due; an interest warrant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A section of a ticket, showing the holder to be entitled to some specified accomodation or service, as to a passage over a designated line of travel, a particular seat in a theater, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>a ticket or form allowing the bearer to purchase a specific quantity of rationed goods; -- such <ex>coupons</ex> are typically issued by a governmental agency.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a printed form, often distributed as part of an advertisement, entitling the bearer to purchase a specific item of merchandise at a discount; or, a similar form which may be submitted to the issuer to obtain goods or information.  Such <ex>coupons</ex> may be issued by the manufacturer (<stype>manufacturer's coupon</stype>) or by a retail establishment; in the latter case they may then be redeemed only at that particular retail store.</def><br/
[<source>PJC</source>]</p>

<p><ent>Coupstick</ent><br/
<hw>Coup"stick`</hw> <pr>(k<oomac/"st<icr/k`)</pr>, <pos>n.</pos>  <ety>[<ets>Coup</ets> + <ets>stick</ets>.]</ety> <def>A stick or switch used among some American Indians in making or counting a coup.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coupure</ent><br/
||<hw>Cou*pure"</hw> <pr>(k??-p?r")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>couper</ets> to cut.]</ety> <fld>(Fort.)</fld> <def>A passage cut through the glacis to facilitate sallies by the besieged.</def>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courage</ent><br/
<hw>Cour"age</hw> <pr>(k<ucr/r"<asl/j; 48)</pr>, <pos>n.</pos> <ety>[OE.  <ets>corage</ets> heart, mind, will, courage, OF. <ets>corage</ets>, F.  <ets>courage</ets>, fr. a LL. derivative of L. <ets>cor</ets> heart. See <er>Heart</er>.]</ety> <sn>1.</sn> <def>The heart; spirit; temper; disposition.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So priketh hem nature in here <qex>corages</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord, cheer up your spirits; our foes are nigh,<br/
and this soft <qex>courage</qex> makes your followers faint.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Heart; inclination; desire; will.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'd such a <qex>courage</qex> to do him good.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That quality of mind which enables one to encounter danger and difficulties with firmness, or without fear, or fainting of heart; valor; boldness; resolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king-becoming graces . . . <br/
Devotion, patience, <qex>courage</qex>, fortitude,<br/
I have no relish of them.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Courage</qex> that grows from constitution often forsakes a man when he has occasion for it.</q> <rj><qau>Addison.</qau></rj></p>

<p><syn><b>Syn.</b> -- Heroism; bravery; intrepidity; valor; gallantry; daring; firmness; hardihood; boldness; dauntlessness; resolution.</syn> <usage> See <er>Heroism</er>. -- <er>Courage</er>, <er>Bravery</er>, <er>Fortitude</er>, <er>Intrepidity</er>, <er>Gallantry</er>, <er>Valor</er>. <xex>Courage</xex> is that firmness of spirit and swell of soul which meets danger without fear. <xex>Bravery</xex> is daring and impetuous courage, like that of one who has the reward continually in view, and displays his courage in daring acts.  <xex>Fortitude</xex> has often been styled <ldquo/passive courage,<rdquo/ and consist in the habit of encountering danger and enduring pain with a steadfast and unbroken spirit. <xex>Valor</xex> is courage exhibited in war, and can not be applied to single combats; it is never used figuratively. <xex>Intrepidity</xex> is firm, unshaken courage. <xex>Gallantry</xex> is adventurous courage, which courts danger with a high and cheerful spirit. A man may show <xex>courage</xex>, <xex>fortitude</xex>, or <xex>intrepidity</xex> in the common pursuits of life, as well as in war. <xex>Valor</xex>, <xex>bravery</xex>, and <xex>gallantry</xex> are displayed in the contest of arms. <xex>Valor</xex> belongs only to battle; <xex>bravery</xex> may be shown in single combat; <xex>gallantry</xex> may be manifested either in attack or defense; but in the latter case, the defense is usually turned into an attack.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courage</ent><br/
<hw>Cour"age</hw>, <pos>v. t.</pos> <def>To inspire with courage; to encourage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Paul writeth unto Timothy . . . to <qex>courage</qex> him.</q> <rj><qau>Tyndale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courageous</ent><br/
<hw>Cour*a"geous</hw> <pr>(k?r-?"j?s)</pr>, <pos>a.</pos> <ety>[F. <ets>courageux</ets>.]</ety> <def>Possessing, or characterized by, courage; brave; bold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With this victory, the women became most <qex>courageous</qex> and proud, and the men waxed . . . fearful and desperate.</q> <rj><qau>Stow.</qau></rj></p>

<p><syn><b>Syn.</b> -- Gallant; brave; bold; daring; valiant; valorous; heroic; intrepid; fearless; hardy; stout; adventurous; enterprising. See <er>Gallant</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courageously</ent><br/
<hw>Cour*a"geous*ly</hw>, <pos>adv.</pos> <def>In a courageous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courageousness</ent><br/
<hw>Cour*a"geous*ness</hw>, <pos>n.</pos> <def>The quality of being courageous; courage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courant</ent><br/
<hw>Cou*rant"</hw> <pr>(k??-r?nt")</pr>, <pos>a.</pos> <ety>[F., p. pr. of <ets>courir</ets> to run, L.  <ets>currere</ets>. Cf. <er>Current</er>.]</ety> <fld>(Her.)</fld> <def>Represented as running; -- said of a beast borne in a coat of arms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courant</ent><br/
<hw>Cou*rant"</hw> <pr>(k??-r?nt")</pr>, <pos>n.</pos> <ety>[F.  <ets>courante</ets>, fr.  <ets>courant</ets>, <pos>p. pr.</pos>]</ety> <sn>1.</sn> <def>A piece of music in triple time; also, a lively dance; a coranto.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A circulating gazette of news; a newspaper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couranto</ent><br/
<hw>Cou*ran"to</hw> <pr>(-r?n"t?)</pr>, <pos>n.</pos> <def>A sprightly dance; a coranto; a courant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courap</ent><br/
<hw>Cou*rap"</hw> <pr>(k??-r?p)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A skin disease, common in India, in which there is perpetual itching and eruption, esp. of the groin, breast, armpits, and face.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courb</ent><br/
<hw>Courb</hw> <pr>(k??rb)</pr>, <pos>a.</pos> <ety>[F.  <ets>courbe</ets>, fr. L.  <ets>curvus</ets>. See <er>Curve</er>, <pos>a.</pos>]</ety> <def>Curved; rounded.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Her neck is short, her shoulders <qex>courb</qex>.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courb</ent><br/
<hw>Courb</hw> <pr>(k??rb)</pr>, <pos>v. i.</pos> <ety>[F.  <ets>courber</ets>. See <er>Curs</er>.]</ety> <def>To bend; to stop; to bow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Then I <qex>courbed</qex> on my knees.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courbaril</ent><br/
<hw>Cour"ba*ril</hw> <pr>(k??r"b?-r?l)</pr>, <pos>n.</pos> <ety>[F. <ets>courbaril</ets>, from a South American word.]</ety> <def>See <er>Anim<eacute/</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courche</ent><br/
<hw>Courche</hw> <pr>(k??rsh)</pr>, <pos>n.</pos> <ety>[Cf. <er>Kerchief</er>.]</ety> <def>A square piece of linen used formerly by women instead of a cap; a kerchief.</def> <mark>[Scot.]</mark> <altsp>[Written also <asp>curch</asp>.]</altsp>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>courgette</ent><br/
<hw>courgette</hw> <pos>n.</pos> <sn>1.</sn> <def>a marrow squash plant whose fruit are eaten when small; -- called also <altname>zucchini</altname>.</def> <mark>[Chiefly Brit.]</mark><br/
<syn><b>Syn. --</b> zucchini.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a small cucumber-shaped vegetable marrow; typically dark green; -- called also <altname>zucchini</altname>.</def> <mark>[Chiefly Brit.]</mark><br/
<syn><b>Syn. --</b> zucchini.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Courier</ent><br/
<hw>Cou"ri*er</hw> <pr>(k??"r?-?r)</pr>, <pos>n.</pos> <ety>[F. <ets>courrier</ets>, fr. <ets>courre</ets>, <ets>courir</ets>, to run, L.  <ets>currere</ets>. See <er>Course</er>, <er>Current</er>.]</ety> <sn>1.</sn> <def>A messenger sent with haste to convey letters or dispatches, usually on public business.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wary Bassa . . . by speedy <qex>couriers</qex>, advertised Solyman of the enemy's purpose.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An attendant on travelers, whose business it is to make arrangements for their convenience at hotels and on the way.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courlan</ent><br/
<hw>Cour"lan</hw> <pr>(k??r"l?n)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A South American bird, of the genus <gen>Aramus</gen>, allied to the rails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Course</ent><br/
<hw>Course</hw> <pr>(k<omac/rs)</pr>, <pos>n.</pos> <ety>[F. <ets>cours</ets>, <ets>course</ets>, L.  cursus, fr. <ets>currere</ets> to run. See <er>Current</er>.]</ety> <sn>1.</sn> <def>The act of moving from one point to another; progress; passage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And when we had finished our <qex>course</qex> from Tyre, we came to Ptolemais.</q> <rj><qau>Acts xxi. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The ground or path traversed; track; way.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The same horse also run the round <qex>course</qex> at Newmarket.</q> <rj><qau>Pennant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Motion, considered as to its general or resultant direction or to its goal; line progress or advance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A light by which the Argive squadron steers<br/
Their silent <qex>course</qex> to Ilium's well known shore.</q> <rj><qau>Dennham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Westward the <qex>course</qex> of empire takes its way.</q> <rj><qau>Berkeley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Progress from point to point without change of direction; any part of a progress from one place to another, which is in a straight line, or on one direction; <as>as, a ship in a long voyage makes many <ex>courses</ex>; a <ex>course</ex> measured by a surveyor between two stations</as>; also, a progress without interruption or rest; a heat; <as>as, one <ex>course</ex> of a race</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Motion considered with reference to manner; or derly progress; procedure in a certain line of thought or action; <as>as, the <ex>course</ex> of an argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>course</qex> of true love never did run smooth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Customary or established sequence of events; recurrence of events according to natural laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>course</qex> of nature and of law.</q> <rj><qau>Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Day and night,<br/
Seedtime and harvest, heat and hoary frost,<br/
Shall hold their <qex>course</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Method of procedure; manner or way of conducting; conduct; behavior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord of York commends the plot and the general <qex>course</qex> of the action.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By perseverance in the <qex>course</qex> prescribed.</q> <rj><qau>Wodsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You hold your <qex>course</qex> without remorse.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A series of motions or acts arranged in order; a succession of acts or practices connectedly followed; <as>as, a <ex>course</ex> of medicine; a <ex>course</ex> of lectures on chemistry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The succession of one to another in office or duty; order; turn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He appointed . . . the <qex>courses</qex> of the priests</q> <rj><qau>2 Chron. viii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>That part of a meal served at one time, with its accompaniments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Goldsmith] wore fine clothes, gave dinners of several <qex>courses</qex>, paid court to venal beauties.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Arch.)</fld> <def>A continuous level range of brick or stones of the same height throughout the face or faces of a building.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Naut.)</fld> <def>The lowest sail on any mast of a square-rigged vessel; <as>as, the fore <ex>course</ex>, main <ex>course</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <pluf>pl.</pluf> <fld>(Physiol.)</fld> <def>The menses.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In course</b></col>, <cd>in regular succession.</cd> -- <col><b>Of course</b></col>, <cd>by consequence; as a matter of course; in regular or natural order.</cd> -- <col><b>In the course of</b></col>, <cd>at same time or times during.</cd> <ldquo/<xex>In the course of</xex> human events.<rdquo/  <rj><au>T. Jefferson.</au></rj></cs></p>

<p><syn><b>Syn.</b> -- Way; road; route; passage; race; series; succession; manner; method; mode; career; progress.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Course</ent><br/
<hw>Course</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coursed</conjf> <pr>(k?rst)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Coursing</conjf>.]</vmorph> <sn>1.</sn> <def>To run, hunt, or chase after; to follow hard upon; to pursue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>coursed</qex> him at the heels.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to chase after or pursue game; <as>as, to <ex>course</ex> greyhounds after deer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To run through or over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bounding steed <qex>courses</qex> the dusty plain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Course</ent><br/
<hw>Course</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To run as in a race, or in hunting; to pursue the sport of coursing; <as>as, the sportsmen <ex>coursed</ex> over the flats of Lancashire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move with speed; to race; <as>as, the blood <ex>courses</ex> through the veins</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coursed</ent><br/
<hw>Coursed</hw> <pr>(k?rst)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Hunted; <as>as, a <ex>coursed</ex> hare</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Arranged in courses; <as>as, <ex>coursed</ex> masonry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courser</ent><br/
<hw>Cours"er</hw> <pr>(k?rs"?r)</pr>, <pos>n.</pos> <ety>[F.  <ets>coursier</ets>.]</ety> <sn>1.</sn> <def>One who courses or hunts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> leash is a leathern thong by which . . . a <qex>courser</qex> leads his greyhound.</q>  <rj><au>Hanmer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A swift or spirited horse; a racer or a war horse; a charger.</def> <mark>[Poetic.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A grallatorial bird of Europe <spn>(Cursorius cursor)</spn>, remarkable for its speed in running. Sometimes, in a wider sense, applied to running birds of the Ostrich family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coursey</ent><br/
<hw>Cour"sey</hw> <pr>(k?r"s?)</pr>, <pos>n.</pos> <ety>[Cf. OF.  <ets>corsie</ets>, <ets>coursie</ets>, passage way to the stern. See <er>Course</er>, <pos>n.</pos> ]</ety> <fld>(Naut.)</fld><def>A space in the galley; a part of the hatches.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 335 --></p>

<p><ent>Coursing</ent><br/
<-- missing "of" (after running) in original? -->
<hw>Cours"ing</hw> <pr>(k?rs"?ng)</pr>, <pos>n.</pos> <def>The pursuit or running game with dogs that follow by sight instead of by scent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>coursing</qex> of a deer, or hart, with greyhounds.</q> <rj><qau>Bacon</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court</ent><br/
<hw>Court</hw> <pr>(k<omac/rt)</pr>, <pos>n.</pos> <ety>[OF.  <ets>court</ets>, <ets>curt</ets>, <ets>cort</ets>, F. <ets>cour</ets>, LL.  <ets>cortis</ets>, fr. L.  <ets>cohors</ets>, <ets>cors</ets>, <ets>chors</ets>, gen.  <ets>cohortis</ets>, <ets>cortis</ets>, <ets>chortis</ets>, an inclosure, court, thing inclosed, crowd, throng; <ets>co-</ets> + a root akin to Gr. <grk>chorto`s</grk> inclosure, feeding place, and to E.  <ets>garden</ets>, <ets>yard</ets>, <ets>orchard</ets>. See <er>Yard</er>, and cf. <er>Cohort</er>, <er>Curtain</er>.]</ety> <sn>1.</sn> <def>An inclosed space; a courtyard; an uncovered area shut in by the walls of a building, or by different building; also, a space opening from a street and nearly surrounded by houses; a blind alley.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>courts</qex> of the house of our God.</q> <rj><qau>Ps. cxxxv. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And round the cool green <qex>courts</qex> there ran a row<br/
Of cloisters.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Goldsmith took a garret in a miserable <qex>court</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The residence of a sovereign, prince, nobleman, or other dignitary; a palace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Attends the emperor in his royal <qex>court</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This our <qex>court</qex>, infected with their manners,<br/
Shows like a riotous inn.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The collective body of persons composing the retinue of a sovereign or person high in authority; all the surroundings of a sovereign in his regal state.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord, there is a nobleman of the <qex>court</qex> at door would speak with you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Love rules the <qex>court</qex>, the camp, the grove.</q> <rj><qau>Sir. W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any formal assembling of the retinue of a sovereign; <as>as, to hold a <ex>court</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The princesses held their <qex>court</qex> within the fortress.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Attention directed to a person in power; conduct or address designed to gain favor; courtliness of manners; civility; compliment; flattery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No solace could her paramour intreat<br/
Her once to show, ne <qex>court</qex>, nor dalliance.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I went to make my <qex>court</qex> to the Duke and Duchess of Newcastle.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>The hall, chamber, or place, where justice is administered.</def> <sd>(b)</sd> <def>The persons officially assembled under authority of law, at the appropriate time and place, for the administration of justice; an official assembly, legally met together for the transaction of judicial business; a judge or judges sitting for the hearing or trial of causes.</def> <sd>(c)</sd> <def>A tribunal established for the administration of justice.</def> <sd>(d)</sd> <def>The judge or judges; as distinguished from the counsel or jury, or both.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Most heartily I do beseech the <qex>court</qex><br/
To give the judgment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The session of a judicial assembly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Any jurisdiction, civil, military, or ecclesiastical.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A place arranged for playing the game of tennis; also, one of the divisions of a tennis court.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Christian court</b></col>, <cd>the English ecclesiastical courts in the aggregate, or any one of them.</cd> -- <col><b>Court breeding</b></col>, <cd>education acquired at court.</cd> -- <col><b>Court card</b></col>. <cd>Same as <cref>Coat card</cref>.</cd> -- <col><b>Court circular</b></col>, <cd>one or more paragraphs of news respecting the sovereign and the royal family, together with the proceedings or movements of the court generally, supplied to the newspapers by an officer specially charged with such duty.</cd> <mark>[Eng.]</mark> <au>Edwards.</au> -- <col><b>Court of claims</b></col> <fld>(Law)</fld>, <cd>a court for settling claims against a state or government; specif., a court of the United States, created by act of Congress, and holding its sessions at Washington. It is given jurisdiction over claims on contracts against the government, and sometimes may advise the government as to its liabilities.</cd> [<source>Webster 1913 Suppl.</source>] -- <col><b>Court day</b></col>, <cd>a day on which a court sits to administer justice.</cd> -- <col><b>Court dress</b></col>, <cd>the dress prescribed for appearance at the court of a sovereign.</cd> -- <col><b>Court fool</b></col>, <cd>a buffoon or jester, formerly kept by princes and nobles for their amusement.</cd> -- <col><b>Court guide</b></col>, <cd>a directory of the names and adresses of the nobility and gentry in a town.</cd> -- <col><b>Court hand</b></col>, <cd>the hand or manner of writing used in records and judicial proceedings.</cd> <au>Shak.</au> -- <col><b>Court lands</b></col> <fld>(Eng. Law)</fld>, <cd>lands kept in demesne, -- that is, for the use of the lord and his family.</cd> -- <col><b>Court marshal</b></col>, <cd>one who acts as marshal for a court.</cd> -- <col><b>Court party</b></col>, <cd>a party attached to the court.</cd> -- <col><b>Court rolls</b></col>, <cd>the records of a court. See<er>Roll</er>.</cd> -- <mcol><col><b>Court in banc</b></col>, <it>or</it>  <col><b>Court in bank</b></col></mcol>, <cd>The full court sitting at its regular terms for the hearing of arguments upon questions of law, as distinguished from a sitting at <xex>nisi prius</xex>.</cd> -- <mcol><col><b>Court of Arches</b></col>, <col><b>audience</b></col></mcol>, etc. <cd>See under <er>Arches</er>, <er>Audience</er>, <it>etc.</it></cd> -- <col><b>Court of Chancery</b></col>. <cd>See <er>Chancery</er>, <pos>n.</pos></cd> -- <col><b>Court of Common pleas</b></col>. <fld>(Law)</fld> <cd>See <cref>Common pleas</cref>, under <er>Common</er>.</cd> -- <col><b>Court of Equity</b></col>. <cd>See under <er>Equity</er>, and <er>Chancery</er>.</cd> -- <col><b>Court of Inquiry</b></col> <fld>(Mil.)</fld> <cd>, a court appointed to inquire into and report on some military matter, as the conduct of an officer.</cd> -- <col><b>Court of St. James</b></col>, <cd>the usual designation of the British Court; -- so called from the old palace of St. James, which is used for the royal receptions, levees, and drawing-rooms.</cd> -- <col><b>The court of the Lord</b></col>, <cd>the temple at Jerusalem; hence, a church, or Christian house of worship.</cd> -- <col><b>General Court</b></col>, <cd>the legislature of a State; -- so called from having had, in the colonial days, judicial power; <as>as, the <ex>General Court</ex> of Massachusetts</as>.</cd> <mark>[U.S.]</mark> -- <col><b>To pay one's court</b></col>, <cd>to seek to gain favor by attentions.</cd> <ldquo/Alcibiades was assiduous in <xex>paying his</xex> <er>court</er> to Tissaphernes.<rdquo/ <au>Jowett.</au> -- <col><b>To put out of court</b></col>, <cd>to refuse further judicial hearing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court</ent><br/
<hw>Court</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Courted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Courting</conjf>.]</vmorph> <sn>1.</sn> <def>To endeavor to gain the favor of by attention or flattery; to try to ingratiate one's self with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By one person, hovever, Portland was still assiduously <qex>courted</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To endeavor to gain the affections of; to seek in marriage; to woo.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If either of you both love Katharina . . . <br/
Leave shall you have to <qex>court</qex> her at your pleasure.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To attempt to gain; to solicit; to seek.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They might almost seem to have <qex>courted</qex> the crown of martyrdom.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Guilt and misery . . . <qex>court</qex> privacy and solitude.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To invite by attractions; to allure; to attract.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A well-worn pathway <qex>courted</qex> us<br/
To one green wicket in a privet hedge.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court</ent><br/
<hw>Court</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To play the lover; to woo; <as>as, to go <ex>courting</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-baron</ent><br/
<hw>Court"-bar`on</hw> <pr>(-b?r`?n)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>An inferior court of civil jurisdiction, attached to a manor, and held by the steward; a baron's court; -- now fallen into disuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtbred</ent><br/
<hw>Court"bred`</hw> <pr>(-br?d`)</pr>, <pos>a.</pos> <def>Bred, or educated, at court; polished; courtly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-craft</ent><br/
<hw>Court"-craft`</hw> <pr>(k?rt"kr?ft`)</pr>, <pos>n.</pos> <def>The artifices, intrigues, and plottings, at courts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-cupboard</ent><br/
<hw>Court"-cup`board</hw> <pr>(-k?b`b?rd)</pr>, <pos>n.</pos> <def>A movable sideboard or buffet, on which plate and other articles of luxury were displayed on special ocasions.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A way with the joint stools, remove the <qex>court-cupboard</qex>, look to the plate.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courteous</ent><br/
<hw>Cour"te*ous</hw> <pr>(k?r"t?-?s; 277)</pr>, <pos>a.</pos> <ety>[OE.  <ets>cortais</ets>, <ets>corteis</ets>, <ets>cortois</ets>, rarely <ets>corteous</ets>, OF. <ets>corties</ets>, <ets>corteis</ets>, F. <ets>courtois</ets>. See <er>Court</er>.]</ety> <def>Of courtlike manners; pertaining to, or expressive of, courtesy; characterized by courtesy; civil; obliging; well bred; polite; affable; complaisant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A patient and <qex>courteous</qex> bearing.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His behavior toward his people is grave and <qex>courteous</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courteously</ent><br/
<hw>Cour"te*ous*ly</hw>, <pos>adv.</pos> <def>In a courteous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courteousness</ent><br/
<hw>Cour"te*ous*ness</hw>, <pos>n.</pos> <def>The quality of being courteous; politeness; courtesy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtepy</ent><br/
<hw>Cour"te*py</hw> <pr>(k??r"t?-p?)</pr>, <pos>n.</pos> <ety>[D.  <ets>kort</ets> short + <ets>pije</ets> a coarse cloth.]</ety> <def>A short coat of coarse cloth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Full threadbare was his overeste <qex>courtepy</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courter</ent><br/
<hw>Court"er</hw> <pr>(k?rt"?r)</pr>, <pos>n.</pos> <def>One who courts; one who plays the lover, or who solicits in marriage; one who flatters and cajoles.</def>  <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtesan</ent><br/
<hw>Cour"te*san</hw> <pr>(k?r"t?-z?n; 277)</pr>, <pos>n.</pos> <ety>[F.  <ets>courtisane</ets>, fr. <ets>courtisan</ets> courtier, It.  <ets>cortigiano</ets>; or directly fr. It.  <ets>cortigiana</ets>, or Sp.  <ets>cortesana</ets>. See <er>Court</er>.]</ety> <def>A woman who prostitutes herself for hire; a prostitute; a harlot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lasciviously decked like a <qex>courtesan</qex>.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtesanship</ent><br/
<hw>Cour"te*san*ship</hw>, <pos>n.</pos> <def>Harlotry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtesy</ent><br/
<hw>Cour"te*sy</hw> <pr>(k?r"t?-s?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Courtesies</plw> <pr>(-s<?/z)</pr>.</plu> <ety>[OE. <ets>cortaisie</ets>, <ets>corteisie</ets>, <ets>courtesie</ets>, OF.  <ets>curteisie</ets>, <ets>cortoisie</ets>, OF. <ets>curteisie</ets>, <ets>cortoisie</ets>, F. <ets>courtoisie</ets>, fr.  <ets>curteis</ets>, <ets>corteis</ets>.  See <er>Courteous</er>.]</ety> <sn>1.</sn> <def>Politeness; civility; urbanity; courtliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And trust thy honest-offered <qex>courtesy</qex>,<br/
With oft is sooner found in lowly sheds,<br/
With smoky rafters, than in tapestry walls<br/
And courts of princes, where it first was named,<br/
And yet is most pretended.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pardon me, Messer Claudio, if once more<br/
I use the ancient <qex>courtesies</qex> of speech.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An act of civility or respect; an act of kindness or favor performed with politeness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord, for your many <qex>courtesies</qex> I thank you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Favor or indulgence, as distinguished from right; <as>as, a title given one by <ex>courtesy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Courtesy title</b></col>, <cd>a title assumed by a person, or popularly conceded to him, to which he has no valid claim; as, the <xex>courtesy title</xex> of <xex>Lord</xex> prefixed to the names of the younger sons of noblemen.</cd></cs></p>

<p><syn><b>Syn.</b> -- Politeness; urbanity; civility; complaisance; affability; courteousness; elegance; refinement; courtliness; good breeding. See <er>Politeness</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtesy</ent><br/
<hw>Courte"sy</hw> <pr>(k<ucir/rt"s<ycr/)</pr>, <pos>n.</pos> <ety>[See the preceding word.]</ety> <def>An act of civility, respect, or reverence, made by women, consisting of a slight depression or dropping of the body, with bending of the knees.</def> <altsp>[Written also <asp>curtsy</asp> and <asp>curtsey</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The lady drops a <qex>courtesy</qex> in token of obedience, and the ceremony proceeds as usual.</q> <rj><qau>Golgsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtesy</ent><br/
<hw>Courte"sy</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Courtesied</conjf> <pr>(-s<icr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Courtesying</conjf>.]</vmorph> <def>To make a respectful salutation or movement of respect; esp. (with reference to women), to bow the body slightly, with bending of the knes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtesy</ent><br/
<hw>Courte"sy</hw>, <pos>v. t.</pos> <def>To treat with civility.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courthouse</ent><br/
<hw>Court"house`</hw> <pr>(k<omac/rt"hous`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A house in which established courts are held, or a house appropriated to courts and public meetings.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A county town; -- so called in Virginia and some others of the Southern States.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Providence, the county town of Fairfax, is unknown by that name, and passes as Fairfax <qex>Court House</qex>.</q> <rj><qau>Barlett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtier</ent><br/
<hw>Court"ier</hw> <pr>(k<omac/rt"y<etil/r)</pr>, <pos>n.</pos> <ety>[From <er>Court</er>.]</ety> <sn>1.</sn> <def>One who is in attendance at the court of a prince; one who has an appointment at court.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You know I am no <qex>courtier</qex>, nor versed in state affairs.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>courtier</qex> got a frigate, and that a company.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who courts or solicits favor; one who flatters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There was not among all our princes a greater <qex>courtier</qex> of the people than Richard III.</q> <rj><qau>Suckling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtiery</ent><br/
<hw>Court"ier*y</hw> <pr>(-?)</pr>, <pos>n.</pos> <def>The manners of a courtier; courtliness.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-leet</ent><br/
<hw>Court"-leet`</hw> <pr>(-l?t`)</pr>, <pos>n.</pos> <fld>(Eng. Law)</fld> <def>A court of record held once a year, in a particular hundred, lordship, or manor, before the steward of the leet.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtlike</ent><br/
<hw>Court"like`</hw> <pr>(-l?k`)</pr>, <pos>a.</pos> <def>After the manner of a court; elegant; polite; courtly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtliness</ent><br/
<hw>Court"li*ness</hw> <pr>(-l?-n?s)</pr>, <pos>n.</pos> <ety>[From <er>Courtly</er>.]</ety> <def>The quality of being courtly; elegance or dignity of manners.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtling</ent><br/
<hw>Court"ling</hw> <pr>(-l?ng)</pr>, <pos>n.</pos> <ety>[<ets>Court</ets> +  <ets>-ling</ets>.]</ety> <def>A sycophantic courtier.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtly</ent><br/
<hw>Court"ly</hw> <pr>(-l?)</pr>, <pos>a.</pos> <ety>[From <er>Court</er>.]</ety> <sn>1.</sn> <def>Relating or belonging to a court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Elegant; polite; courtlike; flattering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>courtly</qex> company or at my beads.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Disposed to favor the great; favoring the policy or party of the court; obsequious.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtly</ent><br/
<hw>Court"ly</hw>, <pos>adv.</pos> <def>In the manner of courts; politely; gracefully; elegantly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They can produce nothing so <qex>courtly</qex> writ.</q> <rj><qau>Dryden</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-martial</ent><br/
<hw>Court`-mar"tial</hw> <pr>(k?rt`m?r"sh<it>a</it>l)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Courts-martial</plw> <pr>(k<?/rts`-)</pr>.</plu> <def>A court consisting of military or naval officers, for the trial of one belonging to the army or navy, or of offenses against military or naval law.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-martial</ent><br/
<hw>Court`-mar"tial</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Court-martialed</conjf> <pr>(-sh<it>a</it>ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Court-martialing</conjf>.]</vmorph> <def>To subject to trial by a court-martial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court-plaster</ent><br/
<hw>Court"-plas`ter</hw> <pr>(k?rt"pl?s`t?r)</pr>, <pos>n.</pos> <def>Sticking plaster made by coating taffeta or silk on one side with some adhesive substance, commonly a mixture of isinglass and glycerin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtship</ent><br/
<hw>Court"ship</hw> <pr>(k?rt"sh?p)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of paying court, with the intent to solicit a favor.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of wooing in love; solicitation of woman to marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This method of <qex>courtship</qex>, [by which] both sides are prepared for all the matrimonial adventures that are to follow.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Courtliness; elegance of manners; courtesy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Trim gallants, full of <qex>courtship</qex> and of state.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Court policy; the character of a courtier; artifice of a court; court-craft; finesse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She [the Queen] being composed of <qex>courtship</qex> and Popery.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Court tennis</ent><br/
<hw>Court" ten"nis</hw> <pr>(k?rt" t?n"n?s)</pr>. <def>See under <er>Tennis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Courtyard</ent><br/
<hw>Court"yard</hw> <pr>(k?rt"y?rd`)</pr>, <pos>n.</pos> <def>A court or inclosure attached to a house.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couscous</ent><br/
<hw>Cous"cous`</hw> <pr>(k??s"k??s`)</pr>, <pos>n.</pos> <def>A kind of food used by the natives of Western Africa, made of millet flour with flesh, and leaves of the baobab; -- called also <altname>lalo</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couscousou</ent><br/
<hw>Cous`cou*sou"</hw> <pr>(k<oomac/s`k<oomac/*s<oomac/")</pr>, <pos>n.</pos> <def>A favorite dish in Barbary. See <er>Couscous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousin</ent><br/
<hw>Cous"in</hw> <pr>(k<ucr/z"'n)</pr>, <pos>n.</pos> <ety>[F. <ets>cousin</ets>, LL. <ets>cosinus</ets>, <ets>cusinus</ets>, contr. from L.  <ets>consobrinus</ets> the child of a mother's sister, cousin; <ets>con-</ets> + <ets>sobrinus</ets> a cousin by the mother's side, a form derived fr. <ets>soror</ets> (for <ets>sosor</ets>) sister. See <er>Sister</er>, and cf. <er>Cozen</er>, <er>Coz</er>.]</ety> <sn>1.</sn> <def>One collaterally related more remotely than a brother or sister; especially, the son or daughter of an uncle or aunt.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The children of brothers and sisters are usually denominated <stype>first cousins</stype>, or <stype>cousins-german</stype>. In the second generation, they are called <stype>second cousins</stype>. See <er>Cater-cousin</er>, and <er>Quater-cousin</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art, great lord, my father's sister's son,<br/
A <qex>cousin-german</qex> to great Priam's seed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A title formerly given by a king to a nobleman, particularly to those of the council. In English writs, etc., issued by the crown, it signifies any earl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My noble lords and <qex>cousins</qex>, all, good morrow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousin</ent><br/
<hw>Cous"in</hw>, <pos>n.</pos> <def>Allied; akin.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousinage</ent><br/
<hw>Cous"in*age</hw> <pr>(-?j)</pr>, <pos>n.</pos> <ety>[F.  <ets>cousinage</ets>, OF., also, <ets>cosinage</ets>. Cf. <er>Cosinage</er>, <er>Cozenage</er>.]</ety> <def>Relationship; kinship.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousin-german</ent><br/
<hw>Cous"in-ger"man</hw> <pr>(-j?r"m<it>a</it>n)</pr>, <pos>n.</pos> <ety>[<ets>Cousin</ets> + <ets>german</ets> closely akin.]</ety> <def>A first cousin. See Note under <er>Cousin</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousinhood</ent><br/
<hw>Cous"in*hood</hw> <pr>(-h??d)</pr>, <pos>n.</pos> <def>The state or condition of a cousin; also, the collective body of cousins; kinsfolk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousinly</ent><br/
<hw>Cous"in*ly</hw>, <pos>a.</pos> <def>Like or becoming a cousin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousinry</ent><br/
<hw>Cous"in*ry</hw> <pr>(k?z"'n-r?)</pr>, <pos>n.</pos> <def>A body or collection of cousins; the whole number of persons who stand in the relation of cousins to a given person or persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cousinship</ent><br/
<hw>Cous"in*ship</hw>, <pos>n.</pos> <def>The relationship of cousins; state of being cousins; cousinhood.</def>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coussinet</ent><br/
<hw>Cous"si*net`</hw> <pr>(k??s"s?-n?t`)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>coussin cushion</ets>. See <er>Cushionet</er>.]</ety> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A stone placed on the impost of a pier for receiving the first stone of an arch.</def> <sd>(b)</sd> <def>That part of the Ionic capital between the abacus and quarter round, which forms the volute.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couteau</ent><br/
<hw>Cou*teau"</hw> <pr>(k??-t?")</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A knife; a dagger.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couth</ent><br/
<hw>Couth</hw> <pr>(k??th)</pr>, <pos>imp. & p. p.</pos> <mord>of <er>Can</er></mord>.  <ety>[See <er>Can</er>, and cf. <er>Uncouth</er>.]</ety> <def>Could; was able; knew or known; understood.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Above all other one Daniel<br/
He loveth, for he <qex>couth</qex> well<br/
Divine, that none other <qex>couth</qex>;<br/
To him were all things <qex>couth</qex>,<br/
As he had it of God's grace.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>couture</ent><br/
<hw>couture</hw> <pos>n.</pos> <def>high fashion designing and dressmaking.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>couturier</ent><br/
<hw>couturier</hw> <pos>n.</pos> <def>someone who designs clothing.</def><br/
<syn><b>Syn. --</b> fashion designer, clothes designer, designer.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Couvade</ent><br/
||<hw>Cou`vade"</hw> <pr>(k<oomac/`v<adot/d")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>couver</ets>. See <er>Covey</er>.]</ety> <def>A custom, among certain barbarous tribes, that when a woman gives birth to a child her husband takes to his bed, as if ill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world-wide custom of the <qex>couvade</qex>, where at childbirth the husband undergoes medical treatment, in many cases being put to bed for days.</q> <rj><qau>Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Couveuse</ent><br/
||<hw>Cou`veuse"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Med.)</fld> <def>An incubator for sickly infants, esp. those prematurely born.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>covariance</ent><br/
<hw>covariance</hw> <pos>n.</pos> <def>a statistical measure of the relationship of two variables, formed by multiplying the difference of each variable from its mean, both variables being measured at the same time, and averaging all such products.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Covariant</ent><br/
<hw>Co*va"ri*ant</hw> <pr>(k?-v?"r?-<ait/nt)</pr>, <pos>n.</pos> <fld>(Higher Alg.)</fld> <def>A function involving the coefficients and the variables of a quantic, and such that when the quantic is lineally transformed the same function of the new variables and coefficients shall be equal to the old function multiplied by a factor. An invariant is a like function involving only the coefficients of the quantic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cove</ent><br/
<hw>Cove</hw> <pr>(k<omac/v)</pr>, <pos>n.</pos> <ety>[AS.  <ets>cofa</ets> room; akin to G.  <ets>koben</ets> pigsty, orig., hut, Icel. <ets>kofi</ets> hut, and perh. to E. <ets>cobalt</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A retired nook; especially, a small, sheltered inlet, creek, or bay; a recess in the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Vessels which were in readiness for him within secret <qex>coves</qex> and nooks.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A strip of prairie extending into woodland; also, a recess in the side of a mountain.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A concave molding.</def> <sd>(b)</sd> <def>A member, whose section is a concave curve, used especially with regard to an inner roof or ceiling, as around a skylight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cove</ent><br/
<hw>Cove</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coved</conjf> <pr>(k?vd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coving</conjf>.]</vmorph> <fld>(Arch.)</fld> <def>To arch over; to build in a hollow concave form; to make in the form of a cove.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mosques and other buildings of the Arabians are rounded into domes and <qex>coved</qex> roofs.</q> <rj><qau>H. Swinburne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coved ceiling</b></col>, <cd>a ceiling, the part of which next the wail is constructed in a cove.</cd> -- <col><b>Coved vault</b></col>, <cd>a vault composed of four coves meeting in a central point, and therefore the reverse of a groined vault.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cove</ent><br/
<hw>Cove</hw>, <pos>v. t.</pos> <ety>[CF. F.  <ets>couver</ets>, It. <ets>covare</ets>. See <er>Covey</er>.]</ety> <def>To brood, cover, over, or sit over, as birds their eggs.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not being able to <qex>cove</qex> or sit upon them [eggs], she [the female tortoise] bestoweth them in the gravel.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cove</ent><br/
<hw>Cove</hw>, <pos>n.</pos> <ety>[A gypsy word, <ets>covo</ets> that man, <ets>covi</ets> that woman.]</ety> <def>A boy or man of any age or station.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There's a gentry <qex>cove</qex> here.</q> <rj><qau>Wit's Recreations (1654).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, look to it, <qex>coves</qex>, that all the beef and drink<br/
Be not filched from us.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covellite</ent><br/
<ent>Covelline</ent><br/
<mhw>{ <hw>Co*vel"line</hw> <pr>(k?-v?l"l?n)</pr>, <hw>Co*vel"lite</hw>  }</mhw> <pr>(-l?t)</pr>, <pos>n.</pos> <ety>[After <ets>Covelli</ets>, the discoverer.]</ety> <fld>(Min.)</fld> <def>A native sulphide of copper, occuring in masses of a dark blue color; -- hence called <altname>indigo copper</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenable</ent><br/
<hw>Cov"e*na*ble</hw> <pr>(k?v"?-n?-b'l)</pr>, <pos>a.</pos> <ety>[OF. <ets>covenable</ets>, F.  <ets>convenable</ets>. See <er>Covenant</er>.]</ety> <def>Fit; proper; suitable.</def> <mark>[Obs.]</mark> <ldquo/A <xex>covenable</xex> day.<rdquo/  <rj><au>Wyclif (Mark vi. 21).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 336 --></p>

<p><ent>Covenably</ent><br/
<hw>Cov"e*na*bly</hw> <pr>(k?v"?-n?-bly)</pr>, <pos>adv.</pos> <def>Fitly; suitably.</def> <mark>[Obs.]</mark> <ldquo/Well and <xex>covenably</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenant</ent><br/
<hw>Cov"e*nant</hw> <pr>(k?v"?-n<it>a</it>nt)</pr>, <pos>n.</pos> <ety>[OF. <ets>covenant</ets>, fr. F. & OF. <ets>convenir</ets> to agree, L. <ets>convenire</ets>. See <er>Convene</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A mutual agreement of two or more persons or parties, or one of the stipulations in such an agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then Jonathan and David made a <qex>covenant</qex>.</q> <rj><qau>1 Sam. xviiii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let there be <qex>covenants</qex> drawn between us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If we conclude a peace,<br/
It shall be with such strict and severe <qex>covenants</qex><br/
As little shall the Frenchmen gain thereby.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl. Hist.)</fld> <def>An agreement made by the Scottish Parliament in 1638, and by the English Parliament in 1643, to preserve the reformed religion in Scotland, and to extirpate popery and prelacy; -- usually called the <ldquo/Solemn League and Covenant.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Wharton] was born in the days of the <qex>Covenant</qex>, and was the heir of a covenanted house.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Theol.)</fld> <def>The promises of God as revealed in the Scriptures, conditioned on certain terms on the part of man, as obedience, repentance, faith, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will establish my <qex>covenant</qex> between me and thee and thy seed after thee in their generations for an everlasting <qex>covenant</qex>, to be a God unto thee, and to thy seed after thee.</q> <rj><qau>Gen. xvii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A solemn compact between members of a church to maintain its faith, discipline, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>An undertaking, on sufficient consideration, in writing and under seal, to do or to refrain from some act or thing; a contract; a stipulation; also, the document or writing containing the terms of agreement.</def> <sd>(b)</sd> <def>A form of action for the violation of a promise or contract under seal.</def></p>

<p><syn><b>Syn.</b> -- Agreement; contract; compact; bargain; arrangement; stipulation.</syn> <usage> -- <er>Covenant</er>, <er>Contract</er>, <er>Compact</er>, <er>Stipulation</er>. These words all denote a mutual agreement between two parties.  <xex>Covenant</xex> is frequently used in a religious sense; as, the <xex>covenant</xex> of works or of grace; a church <xex>covenant</xex>; the Solemn League and <xex>Covenant</xex>. <xex>Contract</xex> is the word most used in the business of life. Crabb and Taylor are wrong in saying that a <xex>contract</xex> must always be in writing. There are oral and implied <xex>contracts</xex> as well as written ones, and these are equally enforced by law. In legal usage, the word <xex>covenant</xex> has an important place as connected with contracts. A <xex>compact</xex> is only a stronger and more solemn contract. The term is chiefly applied to political alliances. Thus, the old Confederation was a <xex>compact</xex> between the States. Under the present Federal Constitution, no individual State can, without consent of Congress, enter into a <xex>compact</xex> with any other State or foreign power. A <xex>stipulation</xex> is one of the articles or provisions of a contract.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenant</ent><br/
<hw>Cov"e*nant</hw> <pr>(k?v"?-n?nt)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Covenanted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Covenanting</conjf>.]</vmorph> <def>To agree (with); to enter into a formal agreement; to bind one's self by contract; to make a stipulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jupiter <qex>covenanted</qex> with him, that it should be hot or cold, wet or dry, . . . as the tenant should direct.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And they <qex>covenanted</qex> with him for thyrty pieces of silver.</q> <rj><qau>Matt. xxvi. 15.</qau></rj></p>

<p><syn><b>Syn.</b> -- To agree; contract; bargain; stipulate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenant</ent><br/
<hw>Cov"e*nant</hw>, <pos>v. t.</pos> <def>To grant or promise by covenant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My covenant of peace that I <qex>covenanted</qex> with you.</q> <rj><qau>Wyclif.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenantee</ent><br/
<hw>Cov`e*nan*tee"</hw> <pr>(k?v`?-n<it>a</it>n-t?")</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The person in whose favor a covenant is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenanter</ent><br/
<hw>Cov"e*nant*er</hw> <pr>(k?v"?-n?nt-?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who makes a covenant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl. Hist.)</fld> <def>One who subscribed and defended the <ldquo/Solemn League and Covenant.<rdquo/ See <er>Covenant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenanting</ent><br/
<hw>Cov"e*nant*ing</hw>, <pos>a.</pos> <def>Belonging to a covenant. Specifically, belonging to the Scotch Covenanters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be they <qex>covenanting</qex> traitors,<br/
Or the brood of false Argyle?</q> <rj><qau>Aytoun.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenantor</ent><br/
<hw>Cov"e*nant*or`</hw> <pr>(-?r`)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The party who makes a covenant.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covenous</ent><br/
<hw>Cov"e*nous</hw> <pr>(k?v"?-n?s)</pr>, <pos>a.</pos> <def>See <er>Covinous</er>, and <er>Covin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covent</ent><br/
<hw>Cov"ent</hw> <pr>(k?v"<it>e</it>nt)</pr>, <pos>n.</pos> <ety>[OF. <ets>covent</ets>, F. <ets>couvent</ets>. See <er>Convent</er>.]</ety> <def>A convent or monastery.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Covent Garden</b></col>, <cd>a large square in London, so called because originally it was the garden of a monastery.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coventry</ent><br/
<hw>Cov"en*try</hw> <pr>(k?v"<it>e</it>n-tr?)</pr>, <pos>n.</pos> <def>A town in the county of Warwick, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To send to Coventry</b></col>, <cd>to exclude from society; to shut out from social intercourse, as for ungentlemanly conduct.</cd> -- <col><b>Coventry blue</b></col>, <cd>blue thread of a superior dye, made at Coventry, England, and used for embroidery.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cover</ent><br/
<hw>Cov"er</hw> <pr>(k?v"?r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Covered</conjf> <pr>(-?rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Covering</conjf>.]</vmorph> <ety>[OF.  <ets>covrir</ets>, F.  <ets>couvrir</ets>, fr. L.  <ets>cooperire</ets>; <ets>co-  +  operire</ets> to cover; probably fr.  <ets>ob</ets> towards, over + the root appearing in <ets>aperire</ets> to open. Cf. <er>Aperient</er>, <er>Overt</er>, <er>Curfew</er>.]</ety> <sn>1.</sn> <def>To overspread the surface of (one thing) with another; <as>as, to <ex>cover</ex> wood with paint or lacquer; to <ex>cover</ex> a table with a cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To envelop; to clothe, as with a mantle or cloak.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And with the majesty of darkness round<br/
<qex>Covers</qex> his throne.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All that beauty than doth <qex>cover</qex> thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To invest (one's self <xex>with</xex> something); to bring upon (one's self); <as>as, he <ex>covered</ex> himself with glory</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The powers that <qex>covered</qex> themselves with everlasting infamy by the partition of Poland.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hide sight; to conceal; to cloak; <as>as, the enemy were <ex>covered</ex> from our sight by the woods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A cloud <qex>covered</qex> the mount.</q> <rj><qau>Exod. xxiv. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In vain shou striv'st to <qex>cover</qex> shame with shame.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To brood or sit on; to incubate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While the hen is <qex>covering</qex> her eggs, the male . . . diverts her with his songs.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> To overwhelm; to spread over.<br/
[<source>1913 Webster</source>]</p>

<p><q>The waters returned and <qex>covered</qex> the chariots and the horsemen.</q> <rj><qau>Ex. xiv. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To shelter, as from evil or danger; to protect; to defend; <as>as, the cavalry <ex>covered</ex> the retreat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His calm and blameless life<br/
Does with substantial blessedness abound,<br/
And the soft wings of peace <qex>cover</qex> him round.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To remove from remembrance; to put away; to remit.</def> <ldquo/Blessed is he whose is <xex>covered</xex>.<rdquo/  <rj><au>Ps. xxxii. 1.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To extend over; to be sufficient for; to comprehend, include, or embrace; to account for or solve; to counterbalance; <as>as, a mortgage which fully <ex>covers</ex> a sum loaned on it; a law which <ex>covers</ex> all possible cases of a crime; receipts than do not <ex>cover</ex> expenses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To put the usual covering or headdress on.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cover</qex> thy head . . . ; nay, prithee, be <qex>covered</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To copulate with (a female); to serve; <as>as, a horse <ex>covers</ex> a mare</as>; -- said of the male.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To cover ground</b></col> <it>or</it> <col><b>To cover distance</b></col></mcol>, <cd>to pass over; <as>as, the rider <ex>covered the ground</ex> in an hour</as>.</cd> -- <col><b>To cover one's short contracts</b></col> <fld>(Stock Exchange)</fld>, <cd>to buy stock when the market rises, as a dealer who has sold short does in order to protect himself.</cd> -- <col><b>Covering party</b></col> <fld>(Mil.)</fld>, <cd>a detachment of troops sent for the protection of another detachment, as of men working in the trenches.</cd> -- <col><b>To cover into</b></col>, <cd>to transfer to; <as>as, <ex>to cover into</ex> the treasury</as>.</cd></cs></p>

<p><syn><b>Syn.</b> -- To shelter; screen; shield; hide; overspread.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cover</ent><br/
<hw>Cov"er</hw> <pr>(k?v"?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Anything which is laid, set, or spread, upon, about, or over, another thing; an envelope; a lid; <as>as, the <ex>cover</ex> of a book</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which veils or conceals; a screen; disguise; a cloak.</def> <ldquo/Under <xex>cover</xex> of the night.<rdquo/  <rj><au> Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A handsome <qex>cover</qex> for imperfections.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Shelter; protection; <as>as, the troops fought under <ex>cover</ex> of the batteries; the woods afforded a good <ex>cover</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Being compelled to lodge in the field . . . whilst his army was under <qex>cover</qex>, they might be forced to retire.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Hunting)</fld> <def>The woods, underbrush, etc., which shelter and conceal game; covert; <as>as, to beat a <ex>cover</ex>; to ride to <ex>cover</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> That portion of a slate, tile, or shingle, which is hidden by the overlap of the course above.  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Steam Engine)</fld> <def>The lap of a slide valve.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <ety>[Cf. F. <ets>couvert</ets>.]</ety> <def>A tablecloth, and the other table furniture; esp., the table furniture for the use of one person at a meal; <as>as, <ex>covers</ex> were laid for fifty guests</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To break cover</b></col>, <cd>to start from a covert or lair; -- said of game.</cd> -- <col><b>Under cover</b></col>, <cd>in an envelope, or within a letter; -- said of a written message.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Letters . . . dispatched <qex>under cover</qex> to her ladyship.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cover</ent><br/
<hw>Cov"er</hw>, <pos>v. i.</pos> <def>To spread a table for a meal; to prepare a banquet.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coverage</ent><br/
<hw>Cov"er*age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>the range of items covered.  <ex>Coverage</ex> may be small (<stype>narrow coverage</stype>}, or large (<stype>broad coverage</stype> or <stype>wide coverage</stype>).</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>The aggregate of risks covered by the terms of a contract of insurance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>the state of being insured; insurance.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>the areas reached by a radio transmission signal, or group of people reached by a communications medium, such as television or a newspaper.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>the range of topics treated in a discourse or document, or the depth to which a subject is discussed; <as>as, a news program may have extensive <ex>coverage</ex> of an event</as>.</def> <ldquo/gavel-to-gavel <xex>coverage</xex> of the Democratic convention.<rdquo/<br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <def>the news as presented by reporters for newspapers or radio of television; <as>as, they accused the paper of biased <ex>coverage</ex> of race relations</as>.</def><br/
<syn><b>Syn. --</b> reporting, reportage.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>coverall</ent><br/
<hw>coverall</hw> <pos>n.</pos> <def>a loose-fitting one-piece garment that is worn over other clothing, especially one with trouser-like pants legs.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Coverchief</ent><br/
<hw>Cov"er*chief</hw> <pr>(ch<emac/f)</pr>, <pos>n.</pos> <ety>[See <er>Kerchief</er>.]</ety> <def>A covering for the head.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covercle</ent><br/
<hw>Cov"er*cle</hw> <pr>(k?v"?r-k'l)</pr>, <pos>n.</pos> <ety>[OF. <ets>covercle</ets>, F. <ets>couvercle</ets>, fr. L. <ets>cooperculum</ets> fr. <ets>cooperire</ets>. See <er>cover</er>]</ety> <def>A small cover; a lid.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cover crop</ent><br/
<hw>Cov"er crop</hw>. <def>A catch crop planted, esp. in orchards. as a protection to the soil in winter, as well as for the benefit of the soil when plowed under in spring.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Covered</ent><br/
<hw>Cov"ered</hw> <pr>(k?v"?rd)</pr>, <pos>a.</pos> <def>Under cover; screened; sheltered; not exposed; hidden.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Covered way</b></col> <fld>(Fort.)</fld>, <cd>a corridor or banquette along the top of the counterscarp and covered by an embankment whose slope forms the glacis. It gives the garrison an open line of communication around the works, and a standing place beyond the ditch. See <xex>Illust.</xex>  of <er>Ravelin</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coverer</ent><br/
<hw>Cov"er*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who, or that which, covers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covering</ent><br/
<hw>Cov"er*ing</hw>, <pos>n.</pos> <def>Anything which covers or conceals, as a roof, a screen, a wrapper, clothing, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Noah removed the <qex>covering</qex> of the ark.</q> <rj><qau>Gen. viii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They cause the naked to lodge without clothing, that they have no <qex>covering</qex> in the cold.</q> <rj><qau>Job. xxiv. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>covering</qex> over the well's mouth.</q> <rj><qau>2 Sam. xvii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coverlet</ent><br/
<hw>Cov"er*let</hw> <pr>(k?v"?r-l?t)</pr>, <pos>n.</pos> <ety>[F.  <ets>couvre-lit</ets>; <ets>couvrir</ets> to cover + <ets>lit</ets> bed, fr. L. <ets>lectus</ets> bed. See <er>Cover</er>.]</ety> <def>The uppermost cover of a bed or of any piece of furniture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lay her in lilies and in violets . . . <br/
And odored sheets and arras <qex>coverlets</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coverlid</ent><br/
<hw>Cov"er*lid</hw> <pr>(-l<icr/d)</pr>, <pos>n.</pos> <def>A coverlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the <qex>coverlid</qex> was cloth of gold.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cover-point</ent><br/
<hw>Cov"er-point`</hw> <pr>(-point!)</pr>, <pos>n.</pos> <def>The fielder in the games of cricket and lacrosse who supports <ldquo/point.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coversed sine</ent><br/
<hw>Co*versed" sine</hw> <pr>(k?-v?rst" s?n`)</pr>. <ety>[<ets>Co-</ets> (=<ets>co-</ets> in <ets>co-</ets> sine) +  <ets>versed sine</ets>.]</ety> <fld>(Geom.)</fld> <def>The versed sine of the complement of an arc or angle. See <xex>Illust.</xex> of <er>Functions</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cover-shame</ent><br/
<hw>Cov"er-shame`</hw> <pr>(-sh?m`)</pr>, <pos>n.</pos> <def>Something used to conceal infamy.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coverside</ent><br/
<hw>Cov"er*side`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A region of country having covers; a hunting country.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Covert</ent><br/
<hw>Cov"ert</hw> <pr>(k?v"?rt)</pr>, <pos>a.</pos> <ety>[OF. <ets>covert</ets>, F.  <ets>couvert</ets>, p. p. of <ets>couvrir</ets>. See <er>Cover</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Covered over; private; hid; secret; disguised.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>covert</qex> matters may be best disclosed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether of open war or <qex>covert</qex> guile.</q> <rj><qau>Milton</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sheltered; not open or exposed; retired; protected; <as>as, a <ex>covert</ex> nook</as>.</def>  <rj><au> Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of either side the green, to plant a <qex>covert</qex> alley.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Under cover, authority or protection; <as>as, a <ex>feme covert</ex>, a married woman who is considered as being under the protection and control of her husband</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Covert way</b></col>, <fld>(Fort.)</fld> <cd>See <cref>Covered way</cref>, under <er>Covered</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Hidden; secret; private; covered; disguised; insidious; concealed. See <er>Hidden</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covert</ent><br/
<hw>Cov"ert</hw>, <pos>n.</pos> <ety>[OF. See <er>Covert</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A place that covers and protects; a shelter; a defense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A tabernacle . . . for a <qex>covert</qex> from storm.</q> <rj><qau>Is. iv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The highwayman has darted from his <qex>covered</qex> by the wayside.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. F. <ets>couverte</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the special feathers covering the bases of the quills of the wings and tail of a bird. See <xex>Illust.</xex> of <er>Bird</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covert baron</ent><br/
<hw>Cov"ert bar`on</hw> <pr>(b?r`?n)</pr>. <fld>(Law)</fld> <def>Under the protection of a husband; married.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covertly</ent><br/
<hw>Cov"ert*ly</hw>, <pos>adv.</pos> <def>Secretly; in private; insidiously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covertness</ent><br/
<hw>Cov"ert*ness</hw> <pr>(k?v"?rt-n?s)</pr>, <pos>n.</pos> <def>Secrecy; privacy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coverture</ent><br/
<hw>Cov"er*ture</hw> <pr>(k?v"?r-t?r; 135)</pr>, <pos>n.</pos> <ety>[OF.  <ets>coverture</ets>,F.<ets>couverture</ets>.]</ety> <sn>1.</sn> <def>Covering; shelter; defense; hiding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Protected by walls or other like <qex>coverture</qex>.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Beatrice, who even now<br/
Is couched in the woodbine <qex>coverture</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The condition of a woman during marriage, because she is considered under the cover, influence, power, and protection of her husband, and therefore called a <cref>feme covert</cref>, or <cref>femme couverte</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covet</ent><br/
<hw>Cov"et</hw> <pr>(k?v"?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Covered</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Coveting</conjf>.]</vmorph> <ety>[OF. <ets>coveitier</ets>, <ets>covoitier</ets>, F. <ets>convoiter</ets>, from a derivative fr. L. <ets>cupere</ets> to desire; cf. Skr.  <ets>kup</ets> to become excited. Cf.  <er>Cupidity</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To wish for with eagerness; to desire possession of; -- used in a good sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Covet</qex> earnestly the best gifts.</q> <rj><qau>1. Cor. xxii. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If it be a sin to <qex>covet</qex> honor,<br/
I am the most offending soul alive.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To long for inordinately or unlawfully; to hanker after (something forbidden).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt not <qex>covet</qex> thy neighbor's house.</q> <rj><qau>Ex. xx. 17.</qau></rj></p>

<p><syn><b>Syn.</b> -- To long for; desire; hanker after; crave.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covet</ent><br/
<hw>Cov"et</hw>, <pos>v. i.</pos> <def>To have or indulge inordinate desire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which [money] while some <qex>coveted</qex> after, they have erred from the faith.</q> <rj><qau>1 Tim. vi. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covetable</ent><br/
<hw>Cov"et*a*ble</hw> <pr>(k?v"?t-?-b'l)</pr>, <pos>a.</pos> <def>That may be coveted; desirable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coveter</ent><br/
<hw>Cov"et*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who covets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covetise</ent><br/
<hw>Cov"et*ise</hw> <pr>(-?s)</pr>, <pos>n.</pos> <ety>[OF.  <ets>coveitise</ets>, F.  <ets>convoitise</ets>. See <er>Covet</er>, <pos>v. t.</pos> ]</ety> <def>Avarice.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covetiveness</ent><br/
<hw>Cov"et*ive*ness</hw> <pr>(-?v-)</pr>, <pos>n.</pos> <fld>(Phren.)</fld> <def>Acquisitiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covetous</ent><br/
<hw>Cov"et*ous</hw> <pr>(k?v"?t-?s)</pr>, <pos>a.</pos> <ety>[OF. <ets>coveitos</ets>, F.  <ets>convoiteux</ets>. See <er>Covet</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Very desirous; eager to obtain; -- used in a good sense.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Covetous</qex> of wisdom and fair virtue.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Covetous</qex> death bereaved us all,<br/
To aggrandize one funeral.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inordinately desirous; excessively eager to obtain and possess (esp. money); avaricious; -- in a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>covetous</qex> person lives as if the world were madealtogether for him, and not he for the world.</q> <rj><qau>South.</qau></rj></p>

<p><syn><b>Syn.</b> -- Avaricious; parsimonious; penurious; misrely; niggardly. See <er>Avaricious</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covetously</ent><br/
<hw>Cov"et*ous*ly</hw>, <pos>adv.</pos> <def>In a covetous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covetousness</ent><br/
<hw>Cov"et*ous*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Strong desire.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When workmen strive to do better than well,<br/
They do confound their skill in <qex>covetousness</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A strong or inordinate desire of obtaining and possessing some supposed good; excessive desire for riches or money; -- in a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Covetousness</qex>, by a greed of getting more, deprivess itself of the true end of getting.</q> <rj><qau>Sprat.</qau></rj></p>

<p><syn><b>Syn.</b> -- Avarice; cupidity; eagerness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covey</ent><br/
<hw>Cov"ey</hw> <pr>(k?v"?)</pr>, <pos>n.</pos> <ety>[OF.  <ets>cov<?/e</ets>, F.  <ets>couv<?/e</ets>, fr.  <ets>cover</ets>, F.  <ets>couver</ets>, to sit or brood on, fr. L.  <ets>cubare</ets> to lie down; cf. E.  <ets>incubate</ets>. See <er>Cubit</er>, and cf.  <er>Cove</er> to brood.]</ety> <sn>1.</sn> <def>A brood or hatch of birds; an old bird with her brood of young; hence, a small flock or number of birds together; -- said of game; <as>as, a <ex>covey</ex> of partridges</as>.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A company; a bevy; <as>as, a <ex>covey</ex> of girls</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covey</ent><br/
<hw>Cov"ey</hw>, <pos>v. i.</pos> <def>To brood; to incubate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[Tortoises]  <qex>covey</qex> a whole year before they hatch.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covey</ent><br/
<hw>Cov"ey</hw>, <pos>n.</pos> <def>A pantry.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Parker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Covin</ent><br/
<hw>Cov"in</hw> <pr>(k?v"?n)</pr>, <pos>n.</pos> <ety>[OF. <ets>covine</ets>, <ets>covaine</ets>, fr.  <ets>covenir</ets> to agree. See <er>Covenant</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>A collusive agreement between two or more persons to prejudice a third.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deceit; fraud; artifice.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coving</ent><br/
<hw>Cov"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A cove or series of coves, as the concaved surface under the overhang of a projecting upper story.</def> <sd>(b)</sd> <def>The splayed jambs of a flaring fireplace.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Covinous</ent><br/
<hw>Cov"in*ous</hw> <pr>(k?v"?n-?s)</pr>, <pos>a.</pos> <fld>(Law)</fld> <def>Deceitful; collusive; fraudulent; dishonest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow</ent><br/
<hw>Cow</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cows</plw> (kouz); old pl. <plw>Kine</plw> <pr>(k<imac/n)</pr>.</plu> <ety>[OE. <ets>cu</ets>, <ets>cou</ets>, AS. <ets>c<umac/</ets>; akin to D.  <ets>koe</ets>, G. <ets>kuh</ets>, OHG. <ets>kuo</ets>, Icel. <ets>k<ymac/r</ets>, Dan. & Sw.  <ets>ko</ets>, L.  <ets>bos</ets> ox, cow, Gr. <grk>boy^s</grk>, Skr. <ets>g<omac/</ets>. <root/223. Cf.  <er>Beef</er>, <er>Bovine</er>, <er>Bucolic</er>, <er>Butter</er>, <er>Nylghau</er>.]</ety> <br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The mature female of bovine animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The female of certain large mammals, as whales, seals, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow</ent><br/
<hw>Cow</hw> <pr>(kou)</pr>, <pos>n.</pos> <ety>[See <er>Cowl</er> a hood.]</ety> <def>A chimney cap; a cowl</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow</ent><br/
<hw>Cow</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cowed</conjf> <pr>(koud)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cowing</conjf>.]</vmorph> <ety>[Cf. Icel.  <ets>kuga</ets>, Sw.  <ets>kufva</ets> to check, subdue, Dan.  <ets>kue</ets>.  Cf. <er>Cuff</er>, <pos>v. t.</pos>]</ety> <def>To depress with fear; to daunt the spirits or courage of; to overawe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To vanquish a people already <qex>cowed</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>THe French king was <qex>cowed</qex>.</q> <rj><qau>J. R. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow</ent><br/
<hw>Cow</hw>, <pos>n.</pos> <ety>[Prob. from same root as <ets>cow</ets>, v. t.]</ety> <fld>(Mining)</fld> <def>A wedge, or brake, to check the motion of a machine or car; a chock.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowage</ent><br/
<hw>Cow"age</hw> <pr>(kou"<asl/j)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Cowhage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowalker</ent><br/
<hw>Co`walk"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A phantasmic or <ldquo/astral<rdquo/ body deemed to be separable from the physical body and capable of acting independently; a doppelg<aum/nger.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cowan</ent><br/
<hw>Cow"an</hw> <pr>(kou"<ait/n)</pr>, <pos>n.</pos> <ety>[Cf. OF.  <ets>couillon</ets> a coward, a cullion.]</ety> <def>One who works as a mason without having served a regular apprenticeship.</def> <mark>[Scot.]</mark> <note>Among Freemasons, it is a cant term for <xex>pretender</xex>, <xex>interloper</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coward</ent><br/
<hw>Cow"ard</hw> <pr>(kou"?rd)</pr>, <pos>a.</pos> <ety>[OF.  <ets>couard</ets>, <ets>coard</ets>, <ets>coart</ets>, <ets>n.</ets> and adj., F.  <ets>couard</ets>, fr. OF.  <ets>coe</ets>, <ets>coue</ets>, tail, F.  <ets>queue</ets> (fr. L.  <ets>coda</ets>, a form of <ets>cauda</ets> tail) + <ets>-ard</ets>; orig., short-tailed, as an epithet of the hare, or perh., turning tail, like a scared dog. Cf. <er>Cue</er>, <er>Queue</er>, <er>Caudal</er>.]</ety> <sn>1.</sn> <fld>(Her.)</fld> <def>Borne in the escutcheon with his tail doubled between his legs; -- said of a lion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Destitute of courage; timid; cowardly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fie, <qex>coward</qex> woman, and soft-hearted wretch.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Belonging to a coward; proceeding from, or expressive of, base fear or timidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He raised the house with loud and <qex>coward</qex> cries.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Invading fears repel my <qex>coward</qex> joy.</q> <rj><qau>Proir.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coward</ent><br/
<hw>Cow"ard</hw>, <pos>n.</pos> <def>A person who lacks courage; a timid or pusillanimous person; a poltroon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fool is nauseous, but a <qex>coward</qex> worse.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Craven; poltroon; dastard.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coward</ent><br/
<hw>Cow"ard</hw>, <pos>v. t.</pos> <def>To make timorous; to frighten.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That which <qex>cowardeth</qex> a man's heart.</q> <rj><qau>Foxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardice</ent><br/
<hw>Cow"ard*ice</hw> <pr>(-<icr/s)</pr>, <pos>n.</pos> <ety>[F. <ets>couardise</ets>, fr. <ets>couard</ets>. See <er>Coward</er>.]</ety> <def>Want of courage to face danger; extreme timidity; pusillanimity; base fear of danger or hurt; lack of spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cowardice</qex> of doing wrong.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Moderation was despised as <qex>cowardice</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 337 --></p>

<p><ent>Cowardie</ent><br/
<hw>Cow"ard*ie</hw> <pr>(kou"<etil/rd*<ycr/)</pr>, <pos>n.</pos> <ety>[OF.  <ets>couardie</ets>.]</ety> <def>Cowardice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardish</ent><br/
<hw>Cow"ard*ish</hw>, <pos>a.</pos> <def>Cowardly.</def> <mark>[Obs.]</mark> <ldquo/A base and a <xex>cowardish</xex> mind.<rdquo/  <rj><au>Robynson (More's Utopia).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardize</ent><br/
<hw>Cow"ard*ize</hw> <pr>(-<?/z)</pr>, <pos>v. t.</pos> <def>To render cowardly</def>. <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>God . . . <qex>cowardizeth</qex> . . . insolent spirits.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardliness</ent><br/
<hw>Cow"ard*li*ness</hw> <pr>(-l?-n?s)</pr>, <pos>n.</pos> <def>Cowardice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardly</ent><br/
<hw>Cow"ard*ly</hw>, <pos>a.</pos> <sn>1.</sn> <def>Wanting courage; basely or weakly timid or fearful; pusillanimous; spiritless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cowardly</qex> rascals that ran from the battle.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proceeding from fear of danger or other consequences; befitting a coward; dastardly; base; <as>as, <ex>cowardly</ex> malignity</as>.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cowardly</qex> rashness of those who dare not look danger in the face.</q> <rj><qau>Burke.</qau></rj></p>

<p><syn><b>Syn.</b> -- Timid; fearful; timorous; dastardly; pusillanimous; recreant; craven; faint-hearted; chicken-hearted; white-livered.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardly</ent><br/
<hw>Cow"ard*ly</hw>, <pos>adv.</pos> <def>In the manner of a coward.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowardship</ent><br/
<hw>Cow"ard*ship</hw>, <pos>n.</pos> <def>Cowardice.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowbane</ent><br/
<hw>Cow"bane`</hw> <pr>(kou"b<amac/n`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A poisonous umbelliferous plant; in England, the <spn>Cicuta virosa</spn>; in the United States, the <spn>Cicuta maculata</spn> and the <spn>Archemora rigida</spn>. See <er>Water hemlock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowberry</ent><br/
<hw>Cow"ber`ry</hw> <pr>(-b<ecr/r`r<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cowberries</plw> <pr>(-r<icr/z)</pr>.</plu> <fld>(Bot.)</fld> <def>A species of <gen>Vaccinium</gen> (<spn>Vaccinium Vitis-id<ae/a</spn>), which bears acid red berries which are sometimes used in cookery; -- locally called <altname>mountain cranberry</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowbird</ent><br/
<hw>Cow"bird`</hw> <pr>(-b<etil/rd`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The cow blackbird (<spn>Molothrus ater</spn>), an American starling. Like the European cuckoo, it builds no nest, but lays its eggs in the nests of other birds; -- so called because frequently associated with cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowblakes</ent><br/
<hw>Cow"blakes`</hw> <pr>(-bl<amac/ks`)</pr>, <pos>n. pl.</pos> <def>Dried cow dung used as fuel.</def><mark>[Prov. Eng.]</mark>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowboy</ent><br/
<hw>Cow"boy`</hw> <pr>(-boi`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A cattle herder; a drover; specifically, one of an adventurous class of herders and drovers on the plains of the Western and Southwestern United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the marauders who, in the Revolutionary War infested the neutral ground between the American and British lines, and committed depredations on the Americans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowcatcher</ent><br/
<hw>Cow"catch`er</hw> <pr>(-k?ch`?r)</pr>, <pos>n.</pos> <def>A strong inclined frame, usually of wrought-iron bars, in front of a locomotive engine, for catching or throwing off obstructions on a railway, as cattle; the <altname>pilot</altname>.</def> <mark>[U.S.]</mark><br/
<syn><b>Syn. --</b> fender, buffer, pilot.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>cow-cockle</ent><br/
<hw>cow-cockle</hw> <pos>n.</pos> <def>a European annual (<spn>Vaccaria hispanica</spn>) with pale rose-colored flowers; cultivated flower or self-sown grainfield weed; introduced in North America; sometimes classified as a soapwort.</def><br/
<syn><b>Syn. --</b> cowherb, cow cockle, <spn>Vaccaria hispanica</spn>, <spn>Vaccaria pyramidata</spn>, <spn>Saponaria vaccaria</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cowdie</ent><br/
<hw>Cow"die</hw> <pr>(kou"d<ycr/)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Kauri</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cowed</ent><br/
<hw>cowed</hw> <pos>adj.</pos>  <def>frightened into submission or compliance.</def><br/
<syn><b>Syn. --</b> browbeaten, bullied, hangdog, intimidated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cower</ent><br/
<hw>Cow"er</hw> <pr>(kou"<etil/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cowered</conjf> <pr>(kou"<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cowering</conjf>.]</vmorph> <ety>[Cf. Icel. <ets>kera</ets> to doze, liequiet, Sw. <ets>kura</ets>, Dan.  <ets>kure</ets>, G.  <ets>kauern</ets> to cower, W. <ets>cwrian</ets>.]</ety> <def>To stoop by bending the knees; to crouch; to squat; hence, to quail; to sink through fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our dame sits <qex>cowering</qex> o'er a kitchen fire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Like falcons, <qex>cowering</qex> on the nest.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cower</ent><br/
<hw>Cow"er</hw> <pr>(kou"<etil/r)</pr>, <pos>v. t.</pos> <def>To cherish with care.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>cowering</ent><br/
<hw>cowering</hw> <pos>adj.</pos>  <def>characterized by or showing abject fear.</def> <mark>[prenominal]</mark><br/
<syn><b>Syn. --</b> craven, cringing(prenominal), fearful, recreant.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cowfish</ent><br/
<hw>Cow"fish`</hw> <pr>(kou"f<icr/sh`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The grampus.</def> <sd>(b)</sd> <def>A California dolphin (<spn>Tursiops Gillii</spn>).</def> <sd>(c)</sd> <def>A marine plectognath fish (<spn>Ostracoin quadricorne</spn>, and allied species), having two projections, like horns, in front; -- called also <altname>cuckold</altname>, <altname>coffer fish</altname>, <altname>trunkfish</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowhage</ent><br/
<hw>Cow"hage</hw> <pr>(kou"h<asl/j)</pr>, <pos>n.</pos> <ety>[Cf. Hind. <ets>kaw<amac/nch</ets>, <ets>ko<amac/nch</ets>.]</ety> <fld>(Bot.)</fld> <def>A leguminous climbing plant of the genus <gen>Mucuna</gen>, having crooked pods covered with sharp hairs, which stick to the fingers, causing intolerable itching. The spicul<ae/ are sometimes used in medicine as a mechanical vermifuge.</def> <altsp>[Written also <asp>couhage</asp>, <asp>cowage</asp>, and <asp>cowitch</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowhearted</ent><br/
<hw>Cow"heart`ed</hw> <pr>(-h?rt`?d)</pr>, <pos>a.</pos> <def>Cowardly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lady Powis . . . patted him with her fan, and called him a <qex>cowhearted</qex> fellow.</q> <rj><qau>R. North.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cowherb</ent><br/
<hw>cowherb</hw> <pos>n.</pos> <def>a European annual with pale rose-colored flowers; cultivated flower or self-sown grainfield weed; introduced in North America; sometimes classified as a soapwort; -- also called the <altname>cow-cockle</altname>.</def><br/
<syn><b>Syn. --</b> cow cockle, cow-cockle, Vaccaria hispanica, Vaccaria pyramidata, Saponaria vaccaria.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cowherd</ent><br/
<hw>Cow"herd`</hw> <pr>(-h?rd`)</pr>, <pos>n.</pos> <ety>[AS. <ets>c<umac/hyrde</ets>; <ets>c<umac/</ets> cow + <ets>hyrde</ets> a herder.]</ety> <def>One whose occupation is to tend cows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowhide</ent><br/
<hw>Cow"hide`</hw> <pr>(-h?d`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The hide of a cow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Leather made of the hide of a cow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A coarse whip made of untanned leather.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowhide</ent><br/
<hw>Cow"hide`</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cowhided</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cowhiding</conjf>.]</vmorph> <def>To flog with a cowhide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowish</ent><br/
<hw>Cow"ish</hw> <pr>(kou"<icr/sh)</pr>, <pos>a.</pos> <ety>[From <er>Cow</er>, <pos>v. t.</pos>]</ety> <def>Timorous; fearful; cowardly.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowish</ent><br/
<hw>Cow"ish</hw>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An umbelliferous plant (<spn>Peucedanum Cous</spn>) with edible tuberous roots, found in Oregon.</def> <altsp>[Written also <asp>cous</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowitch</ent><br/
<hw>Cow"itch</hw> <pr>(kou"?ch; 224)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Cowhage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowl</ent><br/
<hw>Cowl</hw> <pr>(koul)</pr>, <pos>n.</pos> <ety>[AS. <ets>cuhle</ets>, <ets>cugle</ets>, <ets>cugele</ets>; cf. dial. G.  <ets>kogel</ets>, <ets>gugel</ets>, OF.  <ets>coule</ets>, <ets>goule</ets>; all fr. LL.  <ets>cuculla</ets>, <ets>cucullus</ets>, fr. L.  <ets>cucullus</ets> cap, hood; perh. akin to <ets>celare</ets> to conceal, <ets>cella</ets> cell. Cf. <er>Cucullate</er>.]</ety> <sn>1.</sn> <def>A monk's hood; -- usually attached to the gown. The name was also applied to the hood and garment together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What differ more, you cry, than crown and <qex>cowl</qex>?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cowl-shaped cap, commonly turning with the wind, used to improve the draft of a chimney, ventilating shaft, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A wire cap for the smokestack of a locomotive.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(aviation)</fld> <def>a removable metal covering for an aircraft engine, providing streamlining to minimize wind resistance; -- also called <altname>cowling</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a covering for a chimney or other ventilating shaft functioning to increase the draft.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cowl</ent><br/
<hw>Cowl</hw>, <pos>n.</pos> <ety>[Cf. OF.  <ets>cuvele</ets>, <ets>cuvel</ets>, dim. of F.  <ets>cuve</ets> tub, vat, fr. L.  <ets>cupa</ets>. See <er>Cup</er>.]</ety> <def>A vessel carried on a pole between two persons, for conveyance of water.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowled</ent><br/
<hw>Cowled</hw> <pr>(kould)</pr>, <pos>a.</pos> <def>Wearing a cowl; hooded; <as>as, a <ex>cowled</ex> monk</as>.</def> <ldquo/That <xex>cowled</xex> churchman.<rdquo/  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowleech</ent><br/
<hw>Cow"leech`</hw> <pr>(kou"l?ch`)</pr>, <pos>n.</pos> <ety>[2d <ets>cow + leech</ets> a physician.]</ety> <def>One who heals diseases of cows; a cow doctor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowleeching</ent><br/
<hw>Cow"leech`ing</hw>, <pos>n.</pos> <def>Healing the distemper of cows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowlick</ent><br/
<hw>Cow"lick`</hw> <pr>(-l?k`)</pr>, <pos>n.</pos> <def>A tuft of hair turned up or awry (usually over the forehead), as if licked by a cow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowlike</ent><br/
<hw>Cow"like`</hw> <pr>(-l?k`)</pr>, <pos>a.</pos> <def>Resembling a cow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>cowlike</qex> udders and with oxlike eyes.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowlstaff</ent><br/
<hw>Cowl"staff`</hw> <pr>(koul"st?f`)</pr>, <pos>n.</pos> <ety>[<ets>Cowl</ets> a vessel + <ets>staff</ets>.]</ety> <def>A staff or pole on which a vessel is supported between two persons.</def>  <rj><au>Suckling.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coworker</ent><br/
<ent>co-worker</ent><br/
<mhw><hw>co`-work"er</hw>, <hw>co`work"er</hw></mhw> <pr>(k?`w?rk"?r)</pr>, <pos>n.</pos> <def>One who works with another; a cooperator.</def><br/
<syn><b>Syn. --</b> colleague, coworker, fellow worker, fellow-worker, workfellow.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Cow parsley</ent><br/
<hw>Cow" pars`ley</hw> <pr>(kou` p?rs`l?)</pr>. <fld>(Bot.)</fld> <def>An umbelliferous plant of the genus <gen>Ch<ae/rophyllum</gen> (<spn>Ch<ae/rophyllum temulum</spn> and <spn>Ch<ae/rophyllum sylvestre</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow parsnip</ent><br/
<hw>Cow" pars`nip</hw> <pr>(-n?p)</pr>. <fld>(Bot.)</fld> <def>A coarse umbelliferous weed of the genus <gen>Heracleum</gen> (<spn>Heracleum sphondylium</spn> in England, and <spn>Heracleum lanatum</spn> in America).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowpea</ent><br/
<hw>Cow"pea`</hw> <pr>(kou"p<emac/`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The seed of one or more leguminous plants of the genus <gen>Dolichos</gen>; also, the plant itself. Many varieties are cultivated in the southern part of the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A leguminous plant (<spn>Vigna Sinensis</spn>, syn. <spn>Vigna Catjang</spn>) found throughout the tropics of the Old World. It is extensively cultivated in the Southern United States for fodder, and the seed is used as food for man.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cowper's glands</ent><br/
<hw>Cow"per's glands`</hw> <pr>(kou"p?rz gl?ndz`)</pr>. <ety>[After the discoverer, William <etsep>Cowper</etsep>, an English surgeon.]</ety> <fld>(Anat.)</fld> <def>Two small glands discharging into the male urethra.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow-pilot</ent><br/
<hw>Cow"-pi`lot</hw> <pr>(kou"p<imac/`l<ucr/t)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A handsomely banded, coral-reef fish, of Florida and the West Indies (<spn>Pomacentrus saxatilis</spn>); -- called also <altname>mojarra</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowpock</ent><br/
<hw>Cow"pock`</hw> <pr>(-p<ocr/k`)</pr>, <pos>n.</pos> <def>See <er>Cowpox</er>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cowpoke</ent><br/
<hw>cowpoke</hw> <pos>n.</pos> <def>a hired hand who tends cattle and performs other duties on horseback; -- an informal name for <altname>cowboy</altname>.</def><br/
<syn><b>Syn. --</b> cowboy, cowpuncher, puncher, cowman, cattleman, cowhand, cowherd.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cowpox</ent><br/
<hw>Cow"pox`</hw> <pr>(-p<ocr/ks`)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A pustular eruptive disease of the cow, which, when communicated to the human system, as by vaccination, protects from the smallpox; vaccinia; -- called also <altname>kinepox</altname>, <altname>cowpock</altname>, and <altname>kinepock</altname>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowquake</ent><br/
<hw>Cow"quake`</hw> <pr>(-kw<amac/k`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A genus of plants (<gen>Briza</gen>); quaking grass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowrie</ent><br/
<hw>Cow"rie</hw> <pr>(-r<ycr/)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Kauri</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowry</ent><br/
<ent>Cowrie</ent><br/
<mhw>{ <hw>Cow"rie</hw> <hw>Cow"ry</hw> }</mhw> <pr>(kou"r<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cowries</plw> <pr>(-r<icr/z)</pr>.</plu> <ety>[Hind. <ets>kaur<imac/</ets>.]</ety> <fld>(Zool.)</fld> <def>A marine shell of the genus <gen>Cypr<ae/a</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are numerous species, many of them ornamental. Formerly <spn>Cypr<ae/a moneta</spn> and several other species were largely used as money in Africa and some other countries, and they are still so used to some extent.  The value is always trifling, and varies at different places.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>cows</ent><br/
<hw>cows</hw> <pos>n. pl.</pos> <ety>[<pos>pl.</pos> of <ets>cow</ets>.]</ety> <def>domesticated bovine animals as a group regardless of sex or age; <as>as, wait till the <ex>cows</ex> come home</as>.</def><br/
<syn><b>Syn. --</b> cattle, kine, oxen, Bos taurus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cowshed</ent><br/
<hw>cowshed</hw> <pos>n.</pos> <def>a barn for cows.</def><br/
<syn><b>Syn. --</b> cowbarn, cow barn, cowhouse.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cowskin</ent><br/
<hw>cowskin</hw> <pos>n.</pos> <def>leather from the skin of a cow.</def><br/
<syn><b>Syn. --</b> cowhide.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cowslip</ent><br/
<hw>Cow"slip`</hw> <pr>(-sl<icr/p`)</pr>, <pos>n.</pos> <ety>[AS.  <ets>c<umac/slyppe</ets>, <ets>c<umac/sloppe</ets>, prob. orig., cow's droppings. Cf. <er>Slop</er>, <pos>n.</pos>]</ety> <fld>(Bot.)</fld> <sn>1.</sn> <def>A common flower in England (<spn>Primula veris</spn>) having yellow blossoms and appearing in early spring. It is often cultivated in the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the United States, the marsh marigold (<spn>Caltha palustris</spn>), appearing in wet places in early spring and often used as a pot herb. It is nearer to a buttercup than to a true cowslip. See <xex>Illust.</xex> of <er>Marsh marigold</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>American cowslip</b></col> <fld>(Bot.)</fld>, <cd>a pretty flower of the West (<spn>Dodecatheon Meadia</spn>), belonging to the same order (<ord>Primulace<ae/</ord>) with the English cowslip.</cd> -- <col><b>French cowslip</b></col> <fld>(Bot.)</fld>, <cd>bear's-ear (<spn>Primula Auricula</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowslipped</ent><br/
<hw>Cow"slipped`</hw> <pr>(-sl<icr/pt`)</pr>, <pos>a.</pos> <def>Adorned with cowslips.</def> <ldquo/<xex>Cowslipped</xex> lawns.<rdquo/  <rj><au>Keats.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow's lungwort</ent><br/
<hw>Cow's" lung"wort`</hw> <pr>(kouz" l?ng"w?rt`)</pr>. <def>Mullein.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cow tree</ent><br/
<hw>Cow" tree`</hw> <pr>(kou" tr?`)</pr>. <ety>[Cf. SP.  <ets>palo de vaca</ets>.]</ety> <fld>(Bot.)</fld> <def>A tree (<spn>Galactodendron utile</spn> or <spn>Brosimum Galactodendron</spn>) of South America, which yields, on incision, a nourishing fluid, resembling milk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowweed</ent><br/
<hw>Cow"weed"</hw> <pr>(-wEd`)</pr>,, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Cow parsley</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cowwheat</ent><br/
<hw>Cow"wheat`</hw> <pr>(-hw?t`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A weed of the genus <gen>Melampyrum</gen>, with black seeds, found on European wheatfields.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cox</ent><br/
<hw>Cox</hw> <pr>(k?ks)</pr>, <pos>n.</pos> <ety>[OE.  <ets>cokes</ets>. Cf. <er>Coax</er>.]</ety> <def>A coxcomb; a simpleton; a gull.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Go; you're a brainless <qex>cox</qex>, a toy, a fop.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxa</ent><br/
||<hw>Cox"a</hw> <pr>(k?ks"?)</pr>, <pos>n.</pos> <ety>[L., the hip.]</ety> <fld>(Zool.)</fld> <def>The first joint of the leg of an insect or crustacean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxalgy</ent><br/
<ent>Coxalgia</ent><br/
<mhw>{ ||<hw>Cox*al"gi*a</hw> <pr>(-?l"j?-?)</pr>, <hw>Cox"al`gy</hw> <pr>(k?ks"?l`j?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>coxalgia</ets>, fr. L. <ets>coxa</ets> hip. + Gr. <?/<?/<?/ pain: cf. F. <ets>coxalgie</ets>.]</ety> <fld>(Med.)</fld> <def>Pain in the hip.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxcomb</ent><br/
<hw>Cox"comb`</hw> <pr>(k?ks"k?m`)</pr>, <pos>n.</pos> <ety>[A corrupted spelling of <ets>cock's comb</ets>.]</ety> <sn>1.</sn> <sd>(a)</sd> <def>A strip of red cloth notched like the comb of a cock, which licensed jesters formerly wore in their caps.</def> <sd>(b)</sd> <def>The cap itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The top of the head, or the head itself</def>.<br/
[<source>1913 Webster</source>]</p>

<p><q>We will belabor you a little better,<br/
And beat a little more care into your <qex>coxcombs</qex>.</q> <rj><qau>Beau  & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A vain, showy fellow; a conceited, silly man, fond of display; a superficial pretender to knowledge or accomplishments; a fop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fond to be seen, she kept a bevy<br/
Of powdered <qex>coxcombs</qex> at her levee.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some are bewildered in the maze of schools,<br/
And some made <qex>coxcombs</qex>, nature meant but fools.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>A name given to several plants of different genera, but particularly to <spn>Celosia cristata</spn>, or garden cockscomb. Same as <er>Cockscomb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxcombical</ent><br/
<hw>Cox*comb"ic*al</hw> <pr>(k?ks-k?m"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Befitting or indicating a coxcomb; like a coxcomb; foppish; conceited.</def> -- <wordforms><wf>Cox*comb"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Studded all over in <qex>coxcombical</qex> fashion with little brass nails.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxcombly</ent><br/
<hw>Cox"comb"ly</hw> <pr>(k?ks"k?m`l?)</pr>, <pos>a.</pos> <def>like a coxcomb.</def> <mark>[Obs.]</mark> <ldquo/You <xex>coxcombly</xex> ass, you!<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxcombry</ent><br/
<hw>Cox"comb`ry</hw> <pr>(-r?)</pr>, <pos>n.</pos> <def>The manners of a coxcomb; foppishness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxcomical</ent><br/
<hw>Cox*com"ic*al</hw> <pr>(k?ks-k?m"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Coxcombical.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxcomically</ent><br/
<hw>Cox*com"ic*al*ly</hw>, <pos>adv.</pos> <def>Conceitedly.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coxswain</ent><br/
<hw>Cox"swain`</hw> <pr>(k?k"sw?n, <mark>Colloq.</mark> k?k"s'n)</pr>, <pos>n.</pos> <def>See <er>Cockswain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coy</ent><br/
<hw>Coy</hw> <pr>(koi)</pr>, <pos>a.</pos> <ety>[OE.  <ets>coi</ets> quiet, still, OF.  <ets>coi</ets>, <ets>coit</ets>, fr.L. quietus quiet, p. p. of <ets>quiescere</ets> to rest, <ets>quie</ets> rest; prob. akin to E. <ets>while</ets>. See <er>While</er>, and cf. <er>Quiet</er>, <er>Quit</er>, <er>Quite</er>.]</ety> <sn>1.</sn> <def>Quiet; still.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shrinking from approach or familiarity; reserved; bashful; shy; modest; -- usually applied to women, sometimes with an implication of coquetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Coy</qex>, and difficult to win.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Coy</qex> and furtive graces.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor the <qex>coy</qex> maid, half willings to be pressed,<br/
Shall kiss the cup, to pass it to the rest.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Soft; gentle; hesitating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Enforced hate,<br/
Instead of love's <qex>coy</qex> touch, shall rudely tear thee.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Shy; shriking; reserved; modest; bashful; backward; distant.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coy</ent><br/
<hw>Coy</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Coyed</conjf> <pr>(koid)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Coying</conjf>.]</vmorph> <sn>1.</sn> <def>To allure; to entice; to decoy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A wiser generation, who have the art to <qex>coy</qex> the fonder sort into their nets.</q> <rj><qau>Bp. Rainbow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To caress with the hand; to stroke.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come sit thee down upon this flowery bed,<br/
While I thy amiable cheeks do <qex>coy</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coy</ent><br/
<hw>Coy</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To behave with reserve or coyness; to shrink from approach or familiarity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus to <qex>coy</qex> it,<br/
With one who knows you too!</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make difficulty; to be unwilling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If he <qex>coyed</qex><br/
To hear Cominius speak, I 'll keep at home.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>coydog</ent><br/
<hw>coydog</hw> <pos>n.</pos> <def>the offspring of a coyote and a dog.</def><br/
<syn><b>Syn. --</b> coy-dog.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coyish</ent><br/
<hw>Coy"ish</hw>, <pos>a.</pos> <def>Somewhat coy or reserved.</def>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coyly</ent><br/
<hw>Coy"ly</hw>, <pos>adv.</pos> <def>In a coy manner; with reserve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coyness</ent><br/
<hw>Coy"ness</hw>, <pos>n.</pos> <def>The quality of being coy; feigned o<?/ bashful unwillingness to become familiar; reserve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the kind nymph would <qex>coyness</qex> feign,<br/
And hides but to be found again.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Reserve; shrinking; shyness; backwardness; modesty; bashfulness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>coyol</ent><br/
<hw>coyol</hw> <pos>n.</pos> <def>a tropical American palm (<spn>Acrocomia vinifera</spn>) having edible nuts and yielding a useful fiber.</def><br/
<syn><b>Syn. --</b> coyol palm, <spn>Acrocomia vinifera</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Coyote</ent><br/
<hw>Coy"o*te</hw> <pr>(k?"?-t? <or/ k?"?t)</pr>, <pos>n.</pos> <ety>[Spanish Amer., fr. Mexican <ets>coyotl</ets>.]</ety> <fld>(Zool.)</fld> <def>A carnivorous animal (<spn>Canis latrans</spn>), allied to the dog, found in the western part of North America; -- called also <altname>prairie wolf</altname>. Its voice is a snapping bark, followed by a prolonged, shrill howl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coyote State</ent><br/
<hw>Coyote State</hw>. <def>South Dakota; -- a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coyotillo</ent><br/
||<hw>Co`yo*til"lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Mex. Sp. dim. See <er>Coyote</er>.]</ety> <def>A low rhamnaceous shrub (<spn>Karwinskia humboldtiana</spn>) of the southwestern United States and Mexico. Its berries are said to be poisonous to the coyote.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Coypu</ent><br/
||<hw>Coy"pu</hw> <pr>(koi"p??)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>A South American rodent (<spn>Myopotamus coypus</spn>), allied to the beaver. It produces a valuable fur called <prod>nutria</prod>.</def> <altsp>[Written also <asp>coypou</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coystrel</ent><br/
<hw>Coys"trel</hw> <pr>(kois"tr<ecr/l)</pr>, <pos>n.</pos> <def>Same as <er>Coistril</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coz</ent><br/
<hw>Coz</hw> <pr>(k<ucr/z)</pr>, <pos>n.</pos> <def>A contraction of <xex>cousin</xex>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozen</ent><br/
<hw>Coz"en</hw> <pr>(k?z"'n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cozened</conjf> <pr>(-'nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cozening</conjf> <pr>(-'n-?ng)</pr>. ]</vmorph> <ety>[From <ets>cousin</ets>, hence, literally, to deceive through pretext of relationship, F. <ets>cousiner</ets>.]</ety> <def>To cheat; to defraud; to beguile; to deceive, usually by small arts, or in a pitiful way.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had <qex>cozened</qex> the world by fine phrases.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Children may be <qex>cozened</qex> into a knowledge of the letters.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Goring loved no man so well but that he would <qex>cozen</qex> him,<br/
and expose him to public mirth for having been cozened.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozen</ent><br/
<hw>Coz"en</hw>, <pos>v. i.</pos> <def>To deceive; to cheat; to act deceitfully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some cogging, <qex>cozening</qex> slave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozenage</ent><br/
<hw>Coz"en*age</hw> <pr>(-<asl/j)</pr>, <pos>n.</pos> <ety>[See <er>Cozen</er>, and cf. <er>Cousinage</er>.]</ety> <def>The art or practice of cozening; artifice; fraud.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozener</ent><br/
<hw>Coz"en*er</hw> <pr>(k?z"'n-?r)</pr>, <pos>n.</pos> <def>One who cheats or defrauds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozier</ent><br/
<hw>Co*zier</hw> <pr>(k?"zh?r)</pr>, <pos>n.</pos> <def>See <er>Cosier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozily</ent><br/
<hw>Co"zi*ly</hw> <pr>(k?"z?-l?)</pr>, <pos>adv.</pos> <def>Snugly; comfortably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Coziness</ent><br/
<hw>Co"zi*ness</hw>, <pos>n.</pos> <def>The state or quality of being cozy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozy</ent><br/
<hw>Co"zy</hw> <pr>(k?"z?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Cozier</adjf> <pr>(-z?-?r)</pr>; <pos>superl.</pos> <adjf>Coziest</adjf>.]</amorph> <ety>[Cf. Scot.  <ets>cosie</ets>, <ets>cozie</ets>, prob. from Gael. <ets>cosach</ets> abounding in hollows, or <ets>cosagach</ets> full of holes or crevices, snug, sheltered, from <ets>cos</ets> a hollow, a crevice.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Snug; comfortable; easy; contented.</def> <altsp>[Written also <asp>cosey</asp> and <asp>cosy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. F. <ets>causer</ets> to chat, talk.]</ety> <def>Chatty; talkative; sociable; familiar.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cozy</ent><br/
<hw>Co"zy</hw>, <pos>n.</pos> <ety>[See <er>Cozy</er>,<pos>a.</pos>]</ety> <def>A wadded covering, often decorative, for a teakettle or other vessel to keep the contents hot.  Also called <altname>tea cozy</altname></def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>CPA</ent><br/
<hw>CPA</hw> <pos>n.</pos> <def>a certified public accountant; an accountant certified by the state.</def> <mark>[acronym]</mark><br/
<syn><b>Syn. --</b> certified public accountant.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>CPI</ent><br/
<hw>CPI</hw> <pos>n.</pos> <def>the <er>consumer price index</er>, an index of the cost of all goods and services to a typical consumer.  The increase of this value each year is one measure of monetary inflation.</def> <mark>[acronym]</mark><br/
<syn><b>Syn. --</b> consumer price index, cost-of-living index.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><note><hand/  For a table of values from the Bureau of Labor Statistics of the CPI over time, see <a href="more\cpiai.htm">CPI-U from 1913 to 1998</a>.</note><br/
[<source>PJC</source>]</p>

<p><ent>c.p.u.</ent><br/
<ent>CPU</ent><br/
<mhw><hw>CPU</hw>, <hw>c.p.u.</hw></mhw> <pos>n.</pos> <mark>abbr.</mark> <sn>1.</sn> <fld>(Computers)</fld>  <def>The <er>central processing unit</er>, that part of the electronic circuitry of a computer in which the arithmetic and logical operations are performed on input data, which are thereby converted to output data; it is usually located on the mainboard, or motherboard, of a computer.  The CPU and the memory form the central part of a computer to which the peripherals are attached.  Most personal computers as of 1998 had only one CPU, but some computers may have more than one CPU.</def>  <mark>[acronym]</mark><br/
<syn><b>Syn. --</b> central processing unit, CPU, C.P.U., central processor, processor.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><-- p. 338 --></p>

<p><ent>C Q D</ent><br/
<hw>C Q D</hw>. <def>In radiotelegraphy, the letters signified by the code call formerly used (cf. S O S) by ships in distress, formed by combining the code call C Q (formerly used as a general call for all stations) with D for distress.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crab</ent><br/
<hw>Crab</hw> <pr>(kr<acr/b)</pr>, <pos>n.</pos> <ety>[AS. <ets>crabba</ets>; akin to D. <ets>krab</ets>, G.  <ets>krabbe</ets>, <ets>krebs</ets>, Icel. <ets>krabbi</ets>, Sw. <ets>krabba</ets>, Dan. <ets>krabbe</ets>, and perh. to E. <ets>cramp</ets>. Cf. <er>Crawfish</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>One of the brachyuran Crustacea. They are mostly marine, and usually have a broad, short body, covered with a strong shell or carapace. The abdomen is small and curled up beneath the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is applied to all the Brachyura, and to certain Anomura, as the hermit <ex>crabs</ex>.  Formerly, it was sometimes applied to Crustacea in general. Many species are edible, the blue crab of the Atlantic coast being one of the most esteemed. The large European edible crab is <spn>Cancer padurus</spn>. <stype>Soft-shelled crabs</stype> are blue crabs that have recently cast their shells. See <er>Cancer</er>; also, <cref>Box crab</cref>, <cref>Fiddler crab</cref>, <cref>Hermit crab</cref>, <cref>Spider crab</cref>, etc., under <er>Box</er>, <er>Fiddler</er>. etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The zodiacal constellation Cancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[See <er>Crab</er>, <pos>a.</pos>]</ety> <fld>(Bot.)</fld> <def>A crab apple; -- so named from its harsh taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When roasted <qex>crabs</qex> hiss in the bowl,<br/
Then nightly sings the staring owl.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A cudgel made of the wood of the crab tree; a crabstick.</def> <mark>[Obs.]</mark>  <rj><au>Garrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mech.)</fld> <sd>(a)</sd> <def>A movable winch or windlass with powerful gearing, used with derricks, etc.</def> <sd>(b)</sd> <def>A form of windlass, or geared capstan, for hauling ships into dock, etc.</def> <sd>(c)</sd> <def>A machine used in ropewalks to stretch the yarn.</def> <sd>(d)</sd> <def>A claw for anchoring a portable machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calling crab</b></col>. <fld>(Zool.)</fld> <cd>See <er>Fiddler</er>., <pos>n.</pos>, 2.</cd> -- <col><b>Crab apple</b></col>, <cd>a small, sour apple, of several kinds; also, the tree which bears it; as, the European <ex>crab apple</ex> (<spn>Pyrus Malus</spn> var. <varn>sylvestris</varn>); the Siberian <ex>crab apple</ex> (<spn>Pyrus baccata</spn>); and the American (<spn>Pyrus coronaria</spn>).</cd> -- <col><b>Crab grass</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A grass (<spn>Digitaria sanguinalis</spn> syn. <spn>Panicum sanguinalis</spn>); -- called also <altname>finger grass</altname>.</cd> <sd>(b)</sd> <cd>A grass of the genus <gen>Eleusine</gen> (<spn>Eleusine Indica</spn>); -- called also <altname>dog's-tail grass</altname>, <altname>wire grass</altname>, etc.</cd> -- <col><b>Crab louse</b></col> <fld>(Zool.)</fld>, <cd>a species of louse (<spn>Phthirius pubis</spn>), sometimes infesting the human body.</cd> -- <col><b>Crab plover</b></col> <fld>(Zool.)</fld>, <cd>an Asiatic plover (<spn>Dromas ardeola</spn>).</cd> -- <mcol><col><b>Crab's eyes</b></col>, <it>or</it> <col><b>Crab's stones</b></col></mcol>, <cd>masses of calcareous matter found, at certain seasons of the year, on either side of the stomach of the European crawfishes, and formerly used in medicine for absorbent and antacid purposes; the gastroliths.</cd> -- <col><b>Crab spider</b></col> <fld>(Zool.)</fld>, <cd>one of a group of spiders (<fam>Laterigrad<ae/</fam>); -- called because they can run backwards or sideways like a crab.</cd> -- <col><b>Crab tree</b></col>, <cd>the tree that bears crab applies.</cd> -- <col><b>Crab wood</b></col>, <cd>a light cabinet wood obtained in Guiana, which takes a high polish.</cd> <au>McElrath.</au> -- <col><b>To catch a crab</b></col> <fld>(Naut.)</fld>, <cd>a phrase used of a rower</cd>: <sd>(a)</sd> <cd>when he fails to raise his oar clear of the water</cd>; <sd>(b)</sd> <cd>when he misses the water altogether in making a stroke.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crab</ent><br/
<hw>Crab</hw> <pr>(kr<acr/b)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To make sour or morose; to embitter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Sickness sours or <qex>crabs</qex> our nature.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beat with a crabstick.</def> <mark>[Obs.]</mark>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crab</ent><br/
<hw>Crab</hw>, <pos>v. i.</pos> <fld>(Naut.)</fld><def>To drift sidewise or to leeward, as a vessel.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crab</ent><br/
<hw>Crab</hw>, <pos>a.</pos> <ety>[Prob. from the same root as <ets>crab</ets>, <ets>n.</ets>]</ety> <def>Sour; rough; austere.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crab</qex> vintage of the neighb'ring coast.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabbed</ent><br/
<hw>Crab"bed</hw> <pr>(kr?b"b?d)</pr>, <pos>a.</pos> <ety>[See <er>Crab</er>,<pos>n.</pos>]</ety> <sn>1.</sn> <def>Characterized by or manifesting, sourness, peevishness, or moroseness; harsh; cross; cynical; -- applied to feelings, disposition, or manners.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Crabbed</qex> age and youth can not live together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characterized by harshness or roughness; unpleasant; -- applied to things; <as>as, a <ex>crabbed</ex> taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Obscure; difficult; perplexing; trying; <as>as, a <ex>crabbed</ex> author</as>.</def> <ldquo/<xex>Crabbed</xex> eloquence.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How charming is divine philosophy!<br/
Not harsh and <qex>crabbed</qex>, as dull fools suppose.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Cramped; irregular; <as>as, <ex>crabbed</ex> handwriting</as>.</def></p>

<p>-- <wordforms><wf>Crab"bed*ly</wf>, <pos>adv.</pos>  -- <wf>Crab"bed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabber</ent><br/
<hw>Crab"ber</hw> <pr>(kr?b"b?r)</pr>, <pos>n.</pos> <def>One who catches crabs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabbing</ent><br/
<hw>Crab"bing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or art of catching crabs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Falconry)</fld> <def>The fighting of hawks with each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Woolen Manuf.)</fld> <def>A process of scouring cloth between rolls in a machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabbish</ent><br/
<hw>Crab"bish</hw> <pr>(kr<acr/b"b<icr/sh)</pr>, <pos>a.</pos> <def>Somewhat sour or cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whips of the most <qex>crabbish</qex> Satyristes.</q> <rj><qau>Decker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabby</ent><br/
<hw>Crab"by</hw> <pr>(-b<ycr/)</pr>, <pos>a.</pos> <def>Crabbed; difficult, or perplexing.</def> <ldquo/Persius is <xex>crabby</xex>, because ancient.<rdquo/  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabeater</ent><br/
<hw>Crab"eat`er</hw> <pr>(kr<acr/b"<emac/t`<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The cobia.</def> <sd>(b)</sd> <def>An etheostomoid fish of the southern United States (<spn>Hadropterus nigrofasciatus</spn>).</def> <sd>(c)</sd> <def>A small European heron (<spn>Ardea minuta</spn>, and other allied species).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craber</ent><br/
<hw>Cra"ber</hw> <pr>(kr<amac/"b<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The water rat.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabfaced</ent><br/
<hw>Crab"faced`</hw> <pr>(kr<acr/b"f<amac/st`)</pr>, <pos>a.</pos> <def>Having a sour, disagreeable countenance.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabsidle</ent><br/
<hw>Crab"si`dle</hw> <pr>(-s<imac/`d'l)</pr>, <pos>v. i.</pos> <def>To move sidewise, as a crab. <mark>[Jocular]</mark>.</def>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crabstick</ent><br/
<hw>Crab"stick`</hw> <pr>(-st<icr/k`)</pr>, <pos>n.</pos> <def>A stick, cane, or cudgel, made of the wood of the crab tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crab tree</ent><br/
<hw>Crab" tree</hw> <pr>(tr<emac/`)</pr>. <def>See under <er>Crab</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crab-yaws</ent><br/
<hw>Crab"-yaws`</hw> <pr>(kr?b"y?z`)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A disease in the West Indies. It is a kind of ulcer on the soles of the feet, with very hard edges. See <er>Yaws</er>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crache</ent><br/
<hw>Crache</hw> <pr>(kr<acr/ch)</pr>, <pos>v.</pos> <def>To scratch.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crack</ent><br/
<hw>Crack</hw> <pr>(kr<acr/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cracked</conjf> <pr>(kr<acr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cracking</conjf>.]</vmorph> <ety>[OE. <ets>cracken</ets>, <ets>craken</ets>, to crack, break, boast, AS. <ets>cracian</ets>, <ets>cearcian</ets>, to crack; akin to D.  <ets>kraken</ets>, G. <ets>krachen</ets>; cf. Skr. <ets>garj</ets> to rattle, or perh. of imitative origin. Cf. <er>Crake</er>, <er>Cracknel</er>, <er>Creak</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To break or burst, with or without entire separation of the parts; <as>as, to <ex>crack</ex> glass; to <ex>crack</ex> nuts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rend with grief or pain; to affect deeply with sorrow; hence, to disorder; to distract; to craze.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, madam, my old heart is <qex>cracked</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He thought none poets till their brains were <qex>cracked</qex>.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to sound suddenly and sharply; to snap; <as>as, to <ex>crack</ex> a whip</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To utter smartly and sententiously; <as>as, to <ex>crack</ex> a joke</as>.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cry up; to extol; -- followed by <xex>up</xex>.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To crack a bottle</b></col>, <cd>to open the bottle and drink its contents.</cd> -- <col><b>To crack a crib</b></col>, <cd>to commit burglary.</cd> <mark>[Slang]</mark> -- <col><b>To crack on</b></col>, <cd>to put on; as, <xex>to crack on</xex> more sail, or more steam.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crack</ent><br/
<hw>Crack</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To burst or open in chinks; to break, with or without quite separating into parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By misfortune it <qex>cracked</qex> in the coling.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mirror <qex>cracked</qex> from side to side.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be ruined or impaired; to fail.</def> <mark>[Collog.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The credit . . . of exchequers <qex>cracks</qex>, when little comes in and much goes out.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter a loud or sharp, sudden sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As thunder when the clouds in autumn <qex>crack</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To utter vain, pompous words; to brag; to boast; -- with <xex>of</xex>.</def> <mark>[Archaic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ethoipes of their sweet complexion <qex>crack</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crack</ent><br/
<hw>Crack</hw>, <pos>n.</pos> <sn>1.</sn> <def>A partial separation of parts, with or without a perceptible opening; a chink or fissure; a narrow breach; a crevice; <as>as, a <ex>crack</ex> in timber, or in a wall, or in glass</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rupture; flaw; breach, in a moral sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My love to thee is sound, sans <qex>crack</qex> or flaw.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sharp, sudden sound or report; the sound of anything suddenly burst or broken; <as>as, the <ex>crack</ex> of a falling house; the <ex>crack</ex> of thunder; the <ex>crack</ex> of a whip</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Will the stretch out to the <qex>crack</qex> of doom?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The tone of voice when changed at puberty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though now our voices<br/
Have got the mannish <qex>crack</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Mental flaw; a touch of craziness; partial insanity; <as>as, he has a <ex>crack</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A crazy or crack-brained person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I . . . can not get the Parliament to listen to me, who look upon me as a <qex>crack</qex> and a projector.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A boast; boasting.</def> <mark>[Obs.]</mark> <ldquo/<xex>Crack</xex> and brags.<rdquo/ <au>Burton.</au> <ldquo/Vainglorius <xex>cracks</xex>.<rdquo/ <au>Spenser.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Breach of chastity.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A boy, generally a pert, lively boy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qperson>Val.</qperson> 'T is a noble child. <qperson>Vir.</qperson> A <qex>crack</qex>, madam.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A brief time; an instant; <as>as, to be with one in a <ex>crack</ex></as>.</def> <mark>[Eng. & Scot. Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Free conversation; friendly chat.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What is <qex>crack</qex> in English? . . . A <qex>crack</qex> is . . . a chat with a good, kindly human heart in it.</q> <rj><qau>P. P. Alexander.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>a witty remark; a wisecrack.</def><br/
[<source>PJC</source>]</p>

<p><sn>13.</sn> <def>a chance or opportunity to do something; an attempt; <as>as, I'll take a <ex>crack</ex> at it</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>14.</sn> <def>a form of cocaine, highly purified and prepared as small pellets, especially suitable for smoking; -- also called <altname>rock</altname>.  Used in this form it appears to be more addicting than cocaine powder.</def> <mark>[slang]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Crack</ent><br/
<hw>Crack</hw>, <pos>a.</pos> <def>Of superior excellence; having qualities to be boasted of; <as>as, a <ex>crack</ex> shot</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>One of our <qex>crack</qex> speakers in the Commons.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crackajack</ent><br/
<hw>Crack"a*jack`</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Now usually spelled <asp>crackerjack</asp>.]</altsp> <ety>[prob. from <ets>crack</ets> + <ets>jack</ets>, a man.]</ety> <sn>1.</sn> <def>An individual of marked ability or excellence, esp. in some sport; <as>as, he is a <ex>crackajack</ex> at tennis</as>.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A preparation of popped corn, candied and pressed into small cakes.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Crackajack</ent><br/
<hw>Crack"a*jack`</hw>, <pos>a.</pos> <altsp>[Now usually spelled <asp>crackerjack</asp>.]</altsp> <def>Of marked ability or excellence.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Crack-brained</ent><br/
<ent>crackbrained</ent><br/
<mhw><hw>crackbrained</hw>, <hw>Crack"-brained`</hw></mhw> <pr>(-br<amac/nd`)</pr>, <pos>a.</pos> <def>Having an impaired intellect; whimsical; extremely foolish; crazy; <as>as, <ex>crackbrained</ex> notions</as>.</def>  <rj><au>Pope.</au></rj><br/
<syn><b>Syn. --</b> idiotic.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cracked</ent><br/
<hw>Cracked</hw> <pr>(kr<acr/kt)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Coarsely ground or broken; <as>as, <ex>cracked</ex> wheat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Crack-brained.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cracker</ent><br/
<hw>Crack"er</hw> <pr>(kr<acr/k"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, cracks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A noisy boaster; a swaggering fellow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>cracker</qex> is this same that deafs our ears?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small firework, consisting of a little powder inclosed in a thick paper cylinder with a fuse, and exploding with a sharp noise; -- usually called <altname>firecracker</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A thin, dry biscuit, often hard or crisp; <as>as, a Boston <ex>cracker</ex>; a Graham <ex>cracker</ex>; a soda <ex>cracker</ex>; an oyster <ex>cracker</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A nickname to designate a poor white in some parts of the Southern United States.</def>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>The pintail duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <fld>(Mach.)</fld> <def>A pair of fluted rolls for grinding caoutchouc.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cracker-barrel</ent><br/
<hw>cracker-barrel</hw> <pos>adj.</pos> <def>characteristic of country life; <as>as, <ex>cracker-barrel</ex> philosophy</as>.</def><br/
<syn><b>Syn. --</b> folksy, homespun.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crackerberry</ent><br/
<hw>crackerberry</hw> <pos>n.</pos> <def>a creeping red-berried perenial herb (<spn>Cornus canadensis</spn>) distinguished by clustered leaf whorls at tips of shoots; Greenland to Alaska.</def><br/
<syn><b>Syn. --</b> bunchberry, dwarf cornel, pudding berry, <spn>Cornus canadensis</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crackerjack</ent><br/
<hw>Crack"er*jack`</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[also spelled <asp>crackerjack</asp>.]</altsp> <ety>[prob. from <ets>crack</ets> + <ets>jack</ets>, a man.]</ety> <sn>1.</sn> <def>An individual of marked ability or excellence, esp. in some sport; <as>as, he is a <ex>crackajack</ex> at tennis</as>.</def> <mark>[Slang]</mark><br/
<syn><b>Syn. --</b> jimdandy, jimhickey.</syn><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn>  <def>A preparation of popped corn, candied and pressed into small cakes.  The name Cracker Jack(TM) is now a trademark for a similar confection.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>something excellent of its kind.</def><br/
<syn><b>Syn. --</b> jimdandy, jimhickey.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crackerjack</ent><br/
<hw>Crack"er*jack`</hw>, <pos>a.</pos> <altsp>[also spelled <asp>crackajack</asp>.]</altsp> <def>Of marked ability or excellence.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>crackers</ent><br/
<hw>crackers</hw> <pos>adj.</pos>  <def>crazy.</def> <mark>[informal or slang]</mark><br/
<syn><b>Syn. --</b> balmy, barmy, bats, batty, bonkers, buggy, cracked, daft, dotty, fruity, haywire, kooky, kookie, loco, loony, loopy, nuts, nutty, wacky.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cracker State</ent><br/
<hw>Cracker State</hw>. <def>Georgia; -- a nickname. See <er>Cracker</er>, <pos>n.</pos> 5.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cracking</ent><br/
<hw>cracking</hw> <pos>n.</pos> <sn>1.</sn> <def>the act of cracking something.</def><br/
<syn><b>Syn. --</b> fracture, crack.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>the process of making lower molecular weight hydrocarbons from heavier hydrocarbons in petroleum, by exposure to heat and catalysts.  It is used to convert heavier alkanes into gasoline, or to improve the octane number of an alkane mixture.</def><br/
[<source>PJC</source>]</p>

<p><ent>cracking</ent><br/
<hw>cracking</hw> <pos>adj.</pos>  <def>same as <er>groovy</er>, sense 1.</def> <mark>[informal]</mark><br/
<syn><b>Syn. --</b> bang-up, bully, cool, corking, dandy, great, groovy, keen, neat, nifty, not bad(predicate), peachy, slap-up, swell, smashing.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crackle</ent><br/
<hw>Crac"kle</hw> <pr>(kr<acr/k"k'l)</pr>, <pos>v. i.</pos> <ety>[Dim. of <ets>crack</ets>.]</ety> <def>To make slight cracks; to make small, sharp, sudden noises, rapidly or frequently repeated; to crepitate; <as>as, burning thorns <ex>crackle</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The unknown ice that <qex>crackles</qex> underneath them.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crackle</ent><br/
<hw>Crac"kle</hw>, <pos>n.</pos> <sn>1.</sn> <def>The noise of slight and frequent cracks or reports; a crackling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crackle</qex> of fireworks.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A kind of crackling sound or r<acir/le, heard in some abnormal states of the lungs; <as>as, dry <ex>crackle</ex>; moist <ex>crackle</ex></as>.</def>  <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fine Arts)</fld> <def>A condition produced in certain porcelain, fine earthenware, or glass, in which the glaze or enamel appears to be cracked in all directions, making a sort of reticulated surface; <as>as, Chinese <ex>crackle</ex>; Bohemian <ex>crackle</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crackled</ent><br/
<hw>Crac"kled</hw> <pr>(-k'ld)</pr>, <pos>a.</pos> <fld>(Fine Arts)</fld> <def>Covered with minute cracks in the glaze; -- said of some kinds of porcelain and fine earthenware.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crackleware</ent><br/
<hw>Crac"kle*ware`</hw> <pr>(-w?r`)</pr>, <pos>n.</pos> <def>See <er>Crackle</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crackling</ent><br/
<hw>Crac"kling</hw> <pr>(kr?k"kl?ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The making of small, sharp cracks or reports, frequently repeated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As the <qex>crackling</qex> of thorns under a pot, so is the laughter of the fool.</q> <rj><qau>Eccl. vii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The well-browned, crisp rind of roasted pork.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For the first time in his life he tested <qex>crackling</qex>.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Food for dogs, made from the refuse of tallow melting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crackaloo</ent><br/
<ent>Crack-loo</ent><br/
<mhw>{ <hw>Crack"-loo`</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Crack"a*loo`</hw>  }</mhw>. <def>A kind of gambling game consisting in pitching coins to or towards the ceiling of a room so that they shall fall as near as possible to a certain crack in the floor.</def> <mark>[Gamblers' Cant, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cracknel</ent><br/
<hw>Crack"nel</hw> <pr>(kr?k"n?l)</pr>, <pos>n.</pos> <ety>[F. <ets>craquelin</ets>, fr. D.  <ets>krakeling</ets>, fr. <ets>krakken</ets> to crack. See <er>Crack</er>, <pos>v. t.</pos>]</ety> <def>A hard brittle cake or biscuit.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crackpot</ent><br/
<hw>crackpot</hw> <pos>n.</pos> <def>a whimsically eccentric person.</def><br/
<syn><b>Syn. --</b> crank, nut, nutcase, screwball.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cracksman</ent><br/
<hw>Cracks"man</hw> <pr>(kr?ks"m<it>a</it>n)</pr>, <pos>n.</pos>, <plu><it>pl.</it> <plw>Cracksmen</plw> <pr>(-m<xex>e</xex>n)</pr>.</plu> <def>A burglar.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cracovian</ent><br/
<hw>Cra*co"vi*an</hw> <pr>(kr?-k?"v?-<it>a</it>n)</pr>, <pos>a.</pos> <def>Of or pertaining to Cracow in Poland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cracovienne</ent><br/
||<hw>Cra*co`vi*enne"</hw> <pr>(kr?-k?`v?-?n")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>Cracow</ets>, the city.]</ety> <fld>(Mus.)</fld> <def>A lively Polish dance, in 2-4 time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cracowes</ent><br/
<hw>Cra"cowes</hw> <pr>(kr?"k?z)</pr>, <pos>n. pl.</pos> <def>Long-toed boots or shoes formerly worn in many parts of Europe; -- so called from <etsep>Cracow</etsep>, in Poland, where they were first worn in the fourteenth century.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cradle</ent><br/
<hw>Cra"dle</hw> <pr>(kr<amac/d'l)</pr>, <pos>n.</pos> <ety>[AS. <ets>cradel</ets>, <ets>cradol</ets>, prob. from Celtic; cf. Gael. <ets>creathall</ets>, Ir. <ets>craidhal</ets>, W. <ets>cryd</ets> a shaking or rocking, a cradle; perh. akin to E. <ets>crate</ets>.]</ety> <sn>1.</sn> <def>A bed or cot for a baby, oscillating on rockers or swinging on pivots; hence, the place of origin, or in which anything is nurtured or protected in the earlier period of existence; <as>as, a <ex>cradle</ex> of crime; the <ex>cradle</ex> of liberty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cradle</qex> that received thee at thy birth.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No sooner was I crept out of my <qex>cradle</qex><br/
But I was made a king, at nine months old.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Infancy, or very early life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From their <qex>cradles</qex> bred together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A form of worship in which they had been educated from their <qex>cradles</qex>.</q>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Agric.)</fld> <def>An implement consisting of a broad scythe for cutting grain, with a set of long fingers parallel to the scythe, designed to receive the grain, and to lay it evenly in a swath.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Engraving)</fld> <def>A tool used in mezzotint engraving, which, by a rocking motion, raises burrs on the surface of the plate, so preparing the ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A framework of timbers, or iron bars, moving upon ways or rollers, used to support, lift, or carry ships or other vessels, heavy guns, etc., as up an inclined plane, or across a strip of land, or in launching a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>A case for a broken or dislocated limb.</def> <sd>(b)</sd> <def>A frame to keep the bedclothes from contact with the person.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mining)</fld> <sd>(a)</sd> <def>A machine on rockers, used in washing out auriferous earth; -- also called a <altname>rocker</altname>.</def> <mark>[U.S.]</mark> <sd>(b)</sd> <def>A suspended scaffold used in shafts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Carp.)</fld> <def>The ribbing for vaulted ceilings and arches intended to be covered with plaster.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Naut.)</fld> <def>The basket or apparatus in which, when a line has been made fast to a wrecked ship from the shore, the people are brought off from the wreck.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cat's cradle</b></col>. <cd>See under <er>Cat</er>.</cd> -- <col><b>Cradle hole</b></col>, <cd>a sunken place in a road, caused by thawing, or by travel over a soft spot.</cd> -- <col><b>Cradle scythe</b></col>, <cd>a broad scythe used in a cradle for cutting grain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cradle</ent><br/
<hw>Cra"dle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cradled</conjf> <pr>(-d'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cradling</conjf> <pr>(-dl?ng)</pr>.]</vmorph> <sn>1.</sn> <def>To lay to rest, or rock, as in a cradle; to lull or quiet, as by rocking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>cradles</qex> their fears to sleep.</q> <rj><qau>D. A. Clark.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To nurse or train in infancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that hath been <qex>cradled</qex> in majesty will not leave the throne to play with beggars.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut and lay with a cradle, as grain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To transport a vessel by means of a cradle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In Lombardy . . . boats are <qex>cradled</qex> and transported over the grade.</q> <rj><qau>Knight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cradle a picture</b></col>, <cd>to put ribs across the back of a picture, to prevent the panels from warping.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cradle</ent><br/
<hw>Cra"dle</hw>, <pos>v. i.</pos> <def>To lie or lodge, as in a cradle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Withered roots and husks wherein the acorn <qex>cradled</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cradleland</ent><br/
<hw>Cra"dle*land`</hw> <pr>(kr<amac/"d'l*l<acr/nd`)</pr>, <pos>n.</pos> <def>Land or region where one was cradled; hence, land of origin.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cradlesong</ent><br/
<hw>cradlesong</hw> <pos>n.</pos> <def>a quiet song that lulls a child to sleep.</def><br/
<syn><b>Syn. --</b> lullaby.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cradling</ent><br/
<hw>Cra"dling</hw> <pr>(-dl?ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of using a cradle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Coopering)</fld> <def>Cutting a cask into two pieces lengthwise, to enable it to pass a narrow place, the two parts being afterward united and rehooped.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Carp.)</fld> <def>The framework in arched or coved ceilings to which the laths are nailed.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craft</ent><br/
<hw>Craft</hw> <pr>(kr<adot/ft)</pr>, <pos>n.</pos> <ety>[AS. <ets>cr<ae/ft</ets> strength, skill, art, cunning; akin to OS., G., Sw., & Dan. <ets>kraft</ets> strength, D. <ets>kracht</ets>, Icel. <ets>kraptr</ets>; perh. originally, a drawing together, stretching, from the root of E. <ets>cramp</ets>.]</ety> <sn>1.</sn> <def>Strength; might; secret power.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Art or skill; dexterity in particular manual employment; hence, the occupation or employment itself; manual art; a trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye know that by this <qex>craft</qex> we have our wealth.</q> <rj><qau>Acts xix. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A poem is the work of the poet; poesy is his skill or <qex>craft</qex> of making.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Since the birth of time, throughout all ages and nations,<br/
Has the <qex>craft</qex> of the smith been held in repute.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Those engaged in any trade, taken collectively; a guild; <as>as, the <ex>craft</ex> of ironmongers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The control of trade passed from the merchant guilds to the new <qex>craft</qex> guilds.</q> <rj><qau>J. R. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Cunning, art, or skill, in a bad sense, or applied to bad purposes; artifice; guile; skill or dexterity employed to effect purposes by deceit or shrewd devices.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have that crooked wisdom which is called <qex>craft</qex>.</q> <rj><qau>Hobbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The chief priests and the scribes sought how they might take him by <qex>craft</qex>, and put him to death.</q> <rj><qau>Mark xiv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>A vessel; vessels of any kind; -- generally used in a collective sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The evolutions of the numerous tiny <qex>craft</qex> moving over the lake.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Small crafts</b></col>, <cd>small vessels, as sloops, schooners, ets.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craft</ent><br/
<hw>Craft</hw>, <pos>v. t.</pos> <def>To play tricks; to practice artifice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You have <qex>crafted</qex> fair.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craftily</ent><br/
<hw>Craft"i*ly</hw> <pr>(-?-l?)</pr>, <pos>adv.</pos> <ety>[See <er>Crafty</er>.]</ety> <def>With craft; artfully; cunningly.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 339 --></p>

<p><ent>Craftiness</ent><br/
<hw>Craft"i*ness</hw> <pr>(kr?ft"?-n?s)</pr>, <pos>n.</pos> <def>Dexterity in devising and effecting a purpose; cunning; artifice; stratagem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He taketh the wise in their own <qex>craftiness</qex>.</q> <rj><qau>Job. v. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craftless</ent><br/
<hw>Craft"less</hw> <pr>(-l?s)</pr>, <pos>a.</pos> <def>Without craft or cunning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Helpless, <qex>craftless</qex>, and innocent people.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craftsman</ent><br/
<hw>Crafts"man</hw> <pr>(kr?fts"m<it>a</it>n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Craftsmen</plw> <pr>(-m<it>e</it>n)</pr>.</plu> <def>One skilled in some trade or manual occupation; an artificer; a mechanic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craftsmanship</ent><br/
<hw>Crafts"man*ship</hw>, <pos>n.</pos> <def>The work of a craftsman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craftsmaster</ent><br/
<hw>Crafts"mas`ter</hw> <pr>(-m?s`t?r)</pr>, <pos>n.</pos> <def>One skilled in his craft or trade; one of superior cunning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In cunning persuasion his <qex>craftsmaster</qex>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crafty</ent><br/
<hw>Craft"y</hw> <pr>(kr?ft"?)</pr>, <pos>a.</pos> <ety>[AS.  <ets>cr<ae/ftig</ets>.]</ety> <sn>1.</sn> <def>Relating to, or characterized by, craft or skill; dexterous.</def> <mark>[Obs.]</mark> <ldquo/<xex>Crafty</xex> work.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Possessing dexterity; skilled; skillful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A noble <qex>crafty</qex> man of trees.</q> <rj><qau>Wyclif.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Skillful at deceiving others; characterized by craft; cunning; wily.</def> <ldquo/A pair of <xex>crafty</xex> knaves.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With anxious care and <qex>crafty</qex> wiles.</q> <rj><qau>J. Baillie.</qau></rj></p>

<p><syn><b>Syn.</b> -- Skillful; dexterous; cunning; artful; wily; sly; fraudulent; deceitful; subtle; shrewd. See <er>Cunning</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crag</ent><br/
<hw>Crag</hw> <pr>(kr<acr/g)</pr>, <pos>n.</pos> <ety>[W. <ets>craig</ets>; akin to Gael. <ets>creag</ets>, Corn. <ets>karak</ets>, Armor. <ets>karrek</ets>.]</ety> <sn>1.</sn> <def>A steep, rugged rock; a rough, broken cliff, or point of a rock, on a ledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From <qex>crag</qex> to <qex>crag</qex> the signal flew.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>A partially compacted bed of gravel mixed with shells, of the Tertiary age.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crag</ent><br/
<hw>Crag</hw>, <pos>n.</pos> <ety>[A form of <ets>craw</ets>: cf. D.  <ets>kraag</ets> neck, collar, G.  <ets>kragen</ets>. See <er>Craw</er>.]</ety> <sn>1.</sn> <def>The neck or throat</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And bear the <qex>crag</qex> so stiff and so state.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The neck piece or scrag of mutton.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cragged</ent><br/
<hw>Crag"ged</hw> <pr>(-g<ecr/d)</pr>, <pos>a.</pos> <def>Full of crags, or steep, broken rocks; abounding with prominences, points, and inequalities; rough; rugged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Into its <qex>cragged</qex> rents descend.</q> <rj><qau>J. Baillie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craggedness</ent><br/
<hw>Crag"ged*ness</hw>, <pos>n.</pos> <def>The quality or state of being cragged; cragginess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cragginess</ent><br/
<hw>Crag"gi*ness</hw> <pr>(-g?-n?s)</pr>, <pos>n.</pos> <def>The state of being craggy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craggy</ent><br/
<hw>Crag"gy</hw> <pr>(kr<acr/g"g<ycr/)</pr>, <pos>a.</pos> <def>Full of crags; rugged with projecting points of rocks; <as>as, the <ex>craggy</ex> side of a mountain</as>.</def> <ldquo/The <xex>craggy</xex> ledge.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cragsman</ent><br/
<hw>Crags"man</hw> <pr>(kr<acr/gz"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cragsmen</plw> <pr>(-m<eit/n)</pr>.</plu> <def>One accustomed to climb rocks or crags; esp., one who makes a business of climbing the cliffs overhanging the sea to get the eggs of sea birds or the birds themselves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craie</ent><br/
<hw>Craie</hw> <pr>(kr<amac/)</pr>, <pos>n.</pos> <def>See <er>Crare</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craig flounder</ent><br/
<hw>Craig" floun`der</hw> <pr>(kr<amac/g" floun`d<etil/r)</pr>. <ety>[Scot.  <ets>craig</ets> a rock. See 1st <er>Crag</er>.]</ety> <fld>(Zool.)</fld> <def>The pole flounder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crail</ent><br/
<hw>Crail</hw> <pr>(kr<amac/l)</pr>, <pos>n.</pos> <ety>[See <er>Creel</er>.]</ety> <def>A creel or osier basket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crake</ent><br/
<hw>Crake</hw> <pr>(kr<amac/k)</pr>, <pos>v. t. & i.</pos> <ety>[See <er>Crack</er>.]</ety> <sn>1.</sn> <def>To cry out harshly and loudly, like the bird called <xex>crake</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To boast; to speak loudly and boastfully.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Each man may <qex>crake</qex> of that which was his own.</q> <rj><qau>Mir. for Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crake</ent><br/
<hw>Crake</hw>, <pos>n.</pos> <def>A boast. See <er>Crack</er>, <pos>n.</pos></def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crake</ent><br/
<hw>Crake</hw>, <pos>n.</pos> <ety>[Cf. Icel.  <ets>kr<amac/ka</ets> crow, <ets>kr<amac/kr</ets> raven, Sw. <ets>kr<aring/ka</ets>, Dan.  <ets>krage</ets>; perh. of imitative origin.  Cf. <er>Crow</er>.]</ety> <fld>(Zool.)</fld> <def>Any species or rail of the genera <gen>Crex</gen> and <gen>Porzana</gen>; -- so called from its singular cry. See <er>Corncrake</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crakeberry</ent><br/
<hw>Crake"ber`ry</hw> <pr>(-b?r`r?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Crowberry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craker</ent><br/
<hw>Crak"er</hw> <pr>(kr?k"?r)</pr>, <pos>n.</pos> <def>One who boasts; a braggart.</def> <mark>[Obs.]</mark>  <rj><au>Old Play.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cram</ent><br/
<hw>Cram</hw> <pr>(kr<acr/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crammed</conjf> <pr>(kr<acr/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cramming</conjf>.]</vmorph> <ety>[AS.  <ets>crammian</ets> to cram; akin to Icel.  <ets>kremja</ets> to squeeze, bruise, Sw.  <ets>krama</ets> to press. Cf. <er>Cramp</er>.]</ety> <sn>1.</sn> <def>To press, force, or drive, particularly in filling, or in thrusting one thing into another; to stuff; to crowd; to fill to superfluity; <as>as, to <ex>cram</ex> anything into a basket; to <ex>cram</ex> a room with people</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their storehouses <qex>crammed</qex> with grain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He will <qex>cram</qex> his brass down our throats.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill with food to satiety; to stuff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Children would be freer from disease if they were not <qex>crammed</qex> so much as they are by fond mothers.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cram</qex> us with praise, and make us<br/
As fat as tame things.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put hastily through an extensive course of memorizing or study, as in preparation for an examination; <as>as, a pupil is <ex>crammed</ex> by his tutor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cram</ent><br/
<hw>Cram</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To eat greedily, and to satiety; to stuff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gluttony . . . .<br/
<qex>Crams</qex>, and blasphemes his feeder.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make crude preparation for a special occasion, as an examination, by a hasty and extensive course of memorizing or study.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cram</ent><br/
<hw>Cram</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of cramming.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Information hastily memorized; <as>as, a <ex>cram</ex> from an examination</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Weaving)</fld> <def>A warp having more than two threads passing through each dent or split of the reed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crambo</ent><br/
<hw>Cram"bo</hw> <pr>(-b<osl/)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cramp</er>, <pos>a.</pos>, difficult.]</ety> <sn>1.</sn> <def>A game in which one person gives a word, to which another finds a rhyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw in one corner . . . a cluster of men and women, diverting themselves with a game at <qex>crambo</qex>. I heard several double rhymes . . . which raised a great deal of mirth.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A word rhyming with another word.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His similes in order set<br/
And every <qex>crambo</qex> he could get.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dumb crambo</b></col>, <cd>a game in which one party of players give a word which rhymes with another, which last to be guessed by the opposing party, who represent in dumb show what they think it to be.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crammer</ent><br/
<hw>Cram"mer</hw> <pr>(kr<acr/m"m<etil/r)</pr>, <pos>n.</pos> <def>One who crams; esp., one who prepares a pupil hastily for an examination, or a pupil who is thus prepared.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cramoisy</ent><br/
<ent>Cramoisie</ent><br/
<mhw>{ <hw>Cra*moi"sie</hw> <hw>Cra*moi"sy</hw>}</mhw> <pr>(kr?-moi"z?)</pr>, <pos>a.</pos> <ety>[F.  <ets>cramoisi</ets> crimson. See <er>Crimson</er>.]</ety> <def>Crimson.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A splendid seignior, magnificent in <qex>cramoisy</qex> velevet.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cramp</ent><br/
<hw>Cramp</hw> <pr>(kr<acr/mp)</pr>, <pos>n.</pos> <ety>[OE.  <ets>crampe</ets>, <ets>craumpe</ets>; akin to D. & Sw. <ets>kramp</ets>, Dan. <ets>krampe</ets>, G. <ets>krampf</ets> (whence F. <ets>crampe</ets>), Icel. <ets>krappr</ets> strait, narrow, and to E. <ets>crimp</ets>, <ets>crumple</ets>; cf. <ets>cram</ets>. See <er>Grape</er>.]</ety> <sn>1.</sn> <def>That which confines or contracts; a restraint; a shackle; a hindrance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A narrow fortune is a <qex>cramp</qex> to a great mind.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Crippling his pleasures with the <qex>cramp</qex> of fear.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Masonry)</fld> <def>A device, usually of iron bent at the ends, used to hold together blocks of stone, timbers, etc.; a cramp iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Carp.)</fld> <def>A rectangular frame, with a tightening screw, used for compressing the joints of framework, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A piece of wood having a curve corresponding to that of the upper part of the instep, on which the upper leather of a boot is stretched to give it the requisite shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>A spasmodic and painful involuntary contraction of a muscle or muscles, as of the leg.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cramp</qex>, divers nights, gripeth him in his legs.</q> <rj><qau>Sir T. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <def>A paralysis of certain muscles due to excessive use; <as>as, writer's <ex>cramp</ex>; milker's <ex>cramp</ex>, etc.</as></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Cramp bone</b></col>, <cd>the patella of a sheep; -- formerly used as a charm for the cramp.</cd> <au>Halliwell.</au> <ldquo/He could turn <xex>cramp bones</xex> into chess men.<rdquo/ <au>Dickens.</au> -- <col><b>Cramp ring</b></col>, <cd>a ring formerly supposed to have virtue in averting or curing cramp, as having been consecrated by one of the kings of England on Good Friday.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cramp</ent><br/
<hw>Cramp</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cramped</conjf> (kr<acr/mt; 215); <pos>p. pr. & vb. n.</pos> <conjf>Cramping</conjf>.]</vmorph> <sn>1.</sn> <def>To compress; to restrain from free action; to confine and contract; to hinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind my be as much <qex>cramped</qex> by too much knowledge as by ignorance.</q> <rj><qau>Layard.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten or hold with, or as with, a cramp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> Hence, <def>to bind together; to unite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The . . . fabric of universal justic is well <qex>cramped</qex> and bolted together in all its parts.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To form on a cramp; <as>as, to <ex>cramp</ex> boot legs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To afflict with cramp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the gout <qex>cramps</qex> my joints.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cramp the wheels of wagon</b></col>, <cd>to turn the front wheels out of line with the hind wheels, so that one of them shall be against the body of the wagon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cramp</ent><br/
<hw>Cramp</hw>, <pos>a.</pos> <ety>[See <er>Cramp</er>, <pos>n.</pos>]</ety> <def>Knotty; difficult.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Care being taken not to add any of the <qex>cramp</qex> reasons for this opinion.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crampbark</ent><br/
<hw>crampbark</hw> <pos>n.</pos> <def>a deciduous thicket-forming Old World shrub (<spn>Viburnum opulus</spn>) with clusters of white flowers and small bright red berries.</def><br/
<syn><b>Syn. --</b> guelder rose, European cranberrybush, European cranberry bush, cranberry-tree, <spn>Viburnum opulus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cramped</ent><br/
<hw>cramped</hw> <pos>adj.</pos> <def>inconveniently small; restricting movement; -- of living quarters or workspace; <as>as, <ex>cramped</ex> quarters; a <ex>cramped</ex> office</as>.</def><br/
<syn><b>Syn. --</b> constricted, inconvenient, uncomfortably small.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crampet</ent><br/
<hw>Cram"pet</hw> <pr>(kr?m"p?t)</pr>, <pos>n.</pos> <ety>[See <er>Cramp</er>,<pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>A cramp iron or cramp ring; a chape, as of a scabbard.</def> <altsp>[Written also <asp>crampit</asp> and <asp>crampette</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the plates of iron, with attached spikes, forming a pair of crampoons</def>; <specif>hence</specif> <fld>(Curling)</fld>, <def>an iron plate for a player to stand on when delivering the stones.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crampfish</ent><br/
<hw>Cramp"fish`</hw> <pr>(kr?mp"f?sh`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The torpedo, or electric ray, the touch of which gives an electric shock. See <er>Electric fish</er>, and <er>Torpedo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cramp iron</ent><br/
<hw>Cramp" i`ron</hw> <pr>(?`rn)</pr>. <def>See <er>Cramp</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crampit</ent><br/
<hw>Cram"pit</hw> <pr>(kr<acr/m"p<icr/t)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>See <er>Crampet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crampon</ent><br/
||<hw>Cram"pon</hw> <pr>(kr<acr/m"p<ocr/n)</pr>, <pos>n.</pos> <ety>[F. See <er>Crampoons</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An a<eum/rial rootlet for support in climbing, as of ivy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a device with pointed metal projections worn over the shoes to provide traction when walking over ice, or in mountaineering; -- usually used in the plural.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>same as <er>crampoon</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cramponee</ent><br/
<hw>Cram`po*nee"</hw> <pr>(kr<acr/m`p?-n?")</pr>, <pos>a.</pos> <ety>[F.  <ets>cramponn<eacute/</ets>. See <er>Crampoons</er>.]</ety> <fld>(Her.)</fld> <def>Having a cramp or square piece at the end; -- said of a cross so furnished.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crampoons</ent><br/
<hw>Cram*poons"</hw> <pr>(kr<acr/m*p<oomac/nz")</pr>, <pos>n. pl.</pos> <ety>[F.  <ets>crampon</ets>, fr. OHG. <ets>chramph</ets> crooked; akin to G. <ets>krampf</ets> cramp. See <er>Cramp</er>,<pos>n.</pos>, and cf. <er>Crampon</er>.]</ety> <sn>1.</sn> <def>A clutch formed of hooked pieces of iron, like double calipers, for raising stones, lumber, blocks of ice, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Iron instruments with sharp points, worn on the shoes to assist in gaining or keeping a foothold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crampy</ent><br/
<hw>Cramp"y</hw> <pr>(kr<acr/mp"<ycr/)</pr>, <sn>1.</sn> <def>Affected with cramp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Productive of, or abounding in, cramps.</def> <ldquo/This <xex>crampy</xex> country.<rdquo/  <rj><au>Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crane</ent><br/
<ent>Cran</ent><br/
<mhw>{ <hw>Cran</hw> <pr>(kr<acr/n)</pr>, <hw>Crane</hw> <pr>(kr<amac/n)</pr> }</mhw>, <pos>n.</pos> <ety>[Scot., fr. Gael. <ets>crann</ets>.]</ety> <def>A measure for fresh herrings, -- as many as will fill a barrel.</def> <mark>[Scot.]</mark>  <rj><au>H. Miller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranage</ent><br/
<hw>Cran"age</hw> <pr>(kr<amac/n"<asl/j)</pr>, <pos>n.</pos> <ety>[See <er>Crane</er>.]</ety> <sn>1.</sn> <def>The liberty of using a crane, as for loading and unloading vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The money or price paid for the use of a crane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranberry</ent><br/
<hw>Cran"ber*ry</hw> <pr>(kr<acr/n"b<ecr/r*r<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cranberries</plw> (-r<icr/z).</plu> <ety>[So named from its fruit being ripe in the spring when the <ets>cranes</ets> return. <au>Dr. Prior.</au>]</ety> <fld>(Bot.)</fld> <def>A red, acid berry, much used for making sauce, etc.; also, the plant producing it (several species of <gen>Vaccinum</gen> or <gen>Oxycoccus</gen>.) The <stype>high cranberry</stype> or <stype>cranberry tree</stype> is a species of <gen>Viburnum</gen> (<spn>Viburnum Opulus</spn>), and the other is sometimes called <stype>low cranberry</stype> or <stype>marsh cranberry</stype> to distinguish it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cranberry-tree</ent><br/
<ent>cranberry tree</ent><br/
<mhw><hw>cranberry tree</hw>, <hw>cranberry-tree</hw></mhw> <pos>n.</pos> <def>a deciduous thicket-forming Old World shrub (<spn>Viburnum opulus</spn>) with clusters of white flowers and small bright red berries.  It is sometimes called <altname>high cranberry</altname> to distinguish it from the <contr>marsh cranberry</contr> or <contr>low cranberry</contr>.</def><br/
<syn><b>Syn. --</b> guelder rose, European cranberrybush, European cranberry bush, crampbark, <spn>Viburnum opulus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cranch</ent><br/
<hw>Cranch</hw> <pr>(kr?nch)</pr>, <pos>v. t.</pos> <def>See <er>Craunch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crandall</ent><br/
<hw>Cran"dall</hw> <pr>(kr<acr/n"d<ait/l)</pr>, <pos>n.</pos> <ety>[Prob. from <etsep>Crandall</etsep>, a proper name.]</ety> <fld>(Stonecutting)</fld> <def>A kind of hammer having a head formed of a group of pointed steel bars, used for dressing ashlar, etc.</def> -- <def2><pos>v. t.</pos> <def>To dress with a crandall.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crane</ent><br/
<hw>Crane</hw> <pr>(kr<amac/n)</pr>, <pos>n.</pos> <ety>[AS. <ets>cran</ets>; akin to D. & LG. <ets>craan</ets>, G.  <ets>kranich</ets>, <ets>krahn</ets> (this in sense 2), Gr. <grk>ge`ranos</grk>, L. <ets>grus</ets>, W. & Armor. <ets>garan</ets>, OSlav. <ets>zerav<icr/</ets>, Lith. <ets>gerve</ets>, Icel.  <ets>trani</ets>, Sw. <ets>trana</ets>, Dan. <ets>trane</ets>. <root/24. Cf. <er>Geranium</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A wading bird of the genus <gen>Grus</gen>, and allied genera, of various species, having a long, straight bill, and long legs and neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European crane is <spn>Grus cinerea</spn>. The sand-hill crane (<spn>Grus Mexicana</spn>) and the whooping crane (<spn>Grus Americana</spn>) are large American species. The Balearic or crowned crane is <spn>Balearica pavonina</spn>. The name is sometimes erroneously applied to the herons and cormorants.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any arm which swings about a vertical axis at one end, used for supporting a suspended weight.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>A machine for raising and lowering heavy weights, and, while holding them suspended, transporting them through a limited lateral distance. In one form it consists of a projecting arm or jib of timber or iron, a rotating post or base, and the necessary tackle, windlass, etc.; -- so called from a fancied similarity between its arm and the neck of a crane See <xex>Illust.</xex> of <er>Derrick</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An iron arm with horizontal motion, attached to the side or back of a fireplace, for supporting kettles, etc., over a fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A siphon, or bent pipe, for drawing liquors out of a cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>A forked post or projecting bracket to support spars, etc., -- generally used in pairs. See <er>Crotch</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn>  <fld>(Zool.)</fld> <def>The American blue heron (<spn>Ardea herodias</spn>).</def> <mark>[Local, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Crane fly</b></col> <fld>(Zool.)</fld>, <cd>a dipterous insect with long legs, of the genus <gen>Tipula</gen>.</cd> -- <col><b>Derrick crane</b></col>. <cd>See <er>Derrick</er>.</cd> -- <col><b>Gigantic crane</b></col>. <fld>(Zool.)</fld> <cd>See <er>Adjutant</er>, <pos>n.</pos>, 3.</cd> -- <mcol><col><b>Traveling crane</b></col>, <col><b>Traveler crane</b></col>, <col><b>Traversing crane</b></col></mcol> <fld>(Mach.)</fld>, <cd>a crane mounted on wheels; esp., an overhead crane consisting of a crab or other hoisting apparatus traveling on rails or beams fixed overhead, as in a machine shop or foundry.</cd> -- <col><b>Water crane</b></col>, <cd>a kind of hydrant with a long swinging spout, for filling locomotive tenders, water carts, etc., with water.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crane</ent><br/
<hw>Crane</hw> <pr>(kr<amac/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Craned</conjf> <pr>(kr<amac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Craning</conjf>.]</vmorph> <sn>1.</sn> <def>To cause to rise; to raise or lift, as by a crane; -- with <xex>up</xex>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What engines, what instruments are used in <qex>craning</qex> up a soul, sunk below the center, to the highest heavens.</q> <rj><qau>Bates.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An upstart <qex>craned</qex> up to the height he has.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stretch, as a crane stretches its neck; <as>as, to <ex>crane</ex> the neck disdainfully</as>.</def>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crane</ent><br/
<hw>crane</hw>, <pos>v. i.</pos> <def>to reach forward with head and neck, in order to see better; <as>as, a hunter <ex>cranes</ex> forward before taking a leap</as>.</def>  <rj><au>Beaconsfield. Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The passengers eagerly <qex>craning</qex> forward over the bulwarks.</q> <rj><qau>Howells.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crane's-bill</ent><br/
<hw>Crane's"-bill`</hw> <pr>(kr?nz"b?l`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>The geranium; -- so named from the long axis of the fruit, which resembles the beak of a crane.</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>A pair of long-beaked forceps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crang</ent><br/
<hw>Crang</hw> <pr>(kr?ng)</pr>, <pos>n.</pos> <def>See <er>Krang</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crangonidae</ent><br/
<hw>Crangonidae</hw> <pos>n.</pos> <def>shrimps.</def><br/
<syn><b>Syn. --</b> family <fam>Crangonidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crania</ent><br/
||<hw>Cra"ni*a</hw> <pr>(kr?"n?-?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A genus of living Brachiopoda; -- so called from its fancied resemblance to the cranium or skull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranial</ent><br/
<hw>Cra"ni*al</hw> <pr>(kr?"n?-<ait/l)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the cranium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranioclasm</ent><br/
<hw>Cra"ni*o*clasm</hw> <pr>(kr?"n?-?-kl?z'm)</pr>, <pos>n.</pos> <ety>[<ets>Cranium</ets> + Gr. <?/<?/<?/ to break.]</ety> <fld>(Med.)</fld> <def>The crushing of a child's head, as with the cranioclast or craniotomy forceps in cases of very difficult delivery.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranioclast</ent><br/
<hw>Cra"ni*o*clast</hw> <pr>(-kl?st)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>An instrument for crushing the head of a fetus, to facilitate delivery in difficult eases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniofacial</ent><br/
<hw>Cra`ni*o*fa"cial</hw> <pr>(-f?"sh<it>a</it>l)</pr>, <pos>a.</pos> <def>Of or pertaining to the cranium and face; <as>as, the <ex>craniofacial</ex> angle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniognomy</ent><br/
<hw>Cra`ni*og"no*my</hw> <pr>(-?g"n?-m?)</pr>, <pos>n.</pos> <ety>[<ets>Cranium</ets> + Gr. <?/<?/<?/<?/, <?/<?/<?/<?/<?/. to know.]</ety> <def>The science of the form and characteristics of the skull.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniological</ent><br/
<hw>Cra`ni*o*log"ic*al</hw> <pr>(-?-l?j"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Of or pertaining to craniology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniologist</ent><br/
<hw>Cra`ni*ol"o*gist</hw> <pr>(-?l"?-j?st)</pr>, <pos>n.</pos> <def>One proficient in craniology; a phrenologist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniology</ent><br/
<hw>Cra`ni*ol"o*gy</hw> <pr>(-j?)</pr>, <pos>n.</pos> <ety>[<ets>Cranium</ets> + <ets>-logy</ets>.]</ety> <def>The department of science (as of ethnology or arch<ae/ology) which deals with the shape, size, proportions, indications, etc., of skulls; the study of skulls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniometer</ent><br/
<hw>Cra`ni*om"e*ter</hw> <pr>(kr?`n?-?m"?-t?r)</pr>, <pos>n.</pos> <ety>[<ets>Cranium</ets>  +  <ets>-meter</ets>.]</ety> <def>An instrument for measuring the size of skulls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniometrical</ent><br/
<ent>Craniometric</ent><br/
<mhw>{ <hw>Cra`ni*o*met"ric</hw> <pr>(-?-m?t"r?k)</pr>, <hw>Cra`ni*o*met"ric*al</hw> <pr>(-r?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <def>Pertaining to craniometry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniometry</ent><br/
<hw>Cra`ni*om"e*try</hw> <pr>(kr?`n?-?m"?-tr?)</pr>, <pos>n.</pos> <def>The art or act of measuring skulls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranioscopist</ent><br/
<hw>Cra`ni*os"co*pist</hw> <pr>(kr?`n?-?s"k?-p?st)</pr>, <pos>n.</pos> <def>One skilled in, or who practices, cranioscopy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was found of equal dimension in a literary man whose skull puzzied the <qex>cranioscopists</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranioscopy</ent><br/
<hw>Cra`ni*os"co*py</hw> <pr>(-p?)</pr>, <pos>n.</pos> <ety>[<ets>Cranium</ets> +  <ets>-scopy</ets>.]</ety> <def>Scientific examination of the cranium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniota</ent><br/
||<hw>Cra`ni*o"ta</hw> <pr>(kr?`n?-?t?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr.  <ets>cranium</ets>.]</ety> <fld>(Zool.)</fld> <def>A comprehensive division of the Vertebrata, including all those that have a skull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craniotomy</ent><br/
<hw>Cra`ni*ot"o*my</hw> <pr>(kr?`n?-?t"?-m?)</pr>, <pos>n.</pos> <ety>[<ets>Cranium</ets>  + Gr. <?/<?/<?/<?/ to cut off.]</ety> <fld>(Med.)</fld> <def>The operation of opening the fetal head, in order to effect delivery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranium</ent><br/
<hw>Cra"ni*um</hw> <pr>(kr?"n?-?m)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  E.  <plw>Craniums</plw> (-<?/mz), L. <plw>Crania</plw> <pr>(-<?/)</pr>.</plu> <ety>[NL., fr. Gr. <?/<?/<?/<?/; akin to <grk>ka`ra</grk> head.]</ety> <def>The skull of an animal; especially, that part of the skull, either cartilaginous or bony, which immediately incloses the brain; the brain case or brainpan. See <er>Skull</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crank</ent><br/
<hw>Crank</hw> <pr>(kr<acr/<nsm/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>cranke</ets>; akin to E.  <ets>cringe</ets>, <ets>cringle</ets>, <ets>crinkle</ets>, and to <ets>crank</ets>, <ets>a.</ets>, the root meaning, probably, <ldquo/to turn, twist.<rdquo/ See <er>Cringe</er>.]</ety> <sn>1.</sn> <fld>(Mach.)</fld> <def>A bent portion of an axle, or shaft, or an arm keyed at right angles to the end of a shaft, by which motion is imparted to or received from it; also used to change circular into reciprocating motion, or reciprocating into circular motion. See <er>Bell crank</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any bend, turn, or winding, as of a passage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So many turning <qex>cranks</qex> these have, so many crooks.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 340 pr=PK --></p>

<p><sn>3.</sn> <def>A twist or turn in speech; a conceit consisting in a change of the form or meaning of a word.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Quips, and <qex>cranks</qex>, and wanton wiles.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A twist or turn of the mind; caprice; whim; crotchet; also, a fit of temper or passion.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Violent of temper; subject to sudden <qex>cranks</qex>.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A person full of crotchets; one given to fantastic or impracticable projects; one whose judgment is perverted in respect to a particular matter.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A sick person; an invalid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art a counterfeit <qex>crank</qex>, a cheater.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crank axle</b></col> <fld>(Mach.)</fld>, <cd>a driving axle formed with a crank or cranks, as in some kinds of locomotives.</cd> -- <col><b>Crank pin</b></col> <fld>(Mach.)</fld>, <cd>the cylindrical piece which forms the handle, or to which the connecting rod is attached, at the end of a crank, or between the arms of a double crank.</cd> -- <col><b>Crank shaft</b></col>, <cd>a shaft bent into a crank, or having a crank fastened to it, by which it drives or is driven.</cd> -- <col><b>Crank wheel</b></col>, <cd>a wheel acting as a crank, or having a wrist to which a connecting rod is attached.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crank</ent><br/
<hw>Crank</hw> <pr>(kr<acr/<nsm/k)</pr>, <pos>a.</pos> <ety>[AS.  <ets>cranc</ets> weak; akin to Icel. <ets>krangr</ets>, D. & G. <ets>krank</ets> sick, weak (cf. D. <ets>krengen</ets> to careen). Cf.  <er>Crank</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Sick; infirm.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>Liable to careen or be overset, as a ship when she is too narrow, or has not sufficient ballast, or is loaded too high, to carry full sail.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Full of spirit; brisk; lively; sprightly; overconfident; opinionated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He who was, a little before, bedrid, . . . was now <qex>crank</qex> and lusty.</q> <rj><qau>Udall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If you strong electioners did not think you were among the elect, you would not be so <qex>crank</qex> about it.</q> <rj><qau>Mrs. Stowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crank</ent><br/
<hw>Crank</hw>, <pos>v. i.</pos> <ety>[See <er>Crank</er>, <pos>n.</pos>]</ety> <def>To run with a winding course; to double; to crook; to wind and turn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>See how this river comes me <qex>cranking</qex> in.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crankbird</ent><br/
<hw>Crank"bird`</hw> <pr>(-b?rd`)</pr>, <pos>n.</pos>  <fld>(Zool.)</fld> <def>A small European woodpecker (<spn>Picus minor</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crankcase</ent><br/
<hw>crankcase</hw> <pos>n.</pos> <def>the housing for a crankshaft and connecting parts in an internal-combustion engine.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cranked</ent><br/
<hw>Cranked</hw> <pr>(kr<acr/<nsm/kt)</pr>, <pos>a.</pos> <def>Formed with, or having, a bend or crank; <as>as, a <ex>cranked</ex> axle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crankiness</ent><br/
<hw>Crank"i*ness</hw> <pr>(kr?nk"?-n?s)</pr>, <pos>n.</pos> <def>Crankness.</def>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crankle</ent><br/
<hw>Cran"kle</hw> <pr>(kr?n"k'l)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Crinkle</er>.]</ety> <def>To break into bends, turns, or angles; to crinkle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Old Veg's stream . . . drew her humid train aslope,<br/
<qex>Crankling</qex> her banks.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crankle</ent><br/
<hw>Cran"kle</hw>, <pos>v. i.</pos> <def>To bend, turn, or wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Along the <qex>crankling</qex> path.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crankle</ent><br/
<hw>Cran"kle</hw>, <pos>n.</pos> <def>A bend or turn; a twist; a crinkle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crankness</ent><br/
<hw>Crank"ness</hw> <pr>(kr?nk"n?s)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <def>Liability to be overset; -- said of a ship or other vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sprightliness; vigor; health.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranky</ent><br/
<hw>Crank"y</hw> <pr>(-?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of spirit; crank.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Addicted to crotchets and whims; unreasonable in opinions; crotchety.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Unsteady; easy to upset; crank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crannied</ent><br/
<hw>Cran"nied</hw> <pr>(kr?n"n?d)</pr>, <pos>a.</pos> <def>Having crannies, chinks, or fissures; <as>as, a <ex>crannied</ex> wall</as>.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crannoge</ent><br/
<ent>Crannog</ent><br/
<mhw>{ <hw>Cran"nog</hw> <pr>(kr?n"n?g)</pr>, <hw>Cran"noge</hw> <pr>(kr?n"n?j)</pr> }</mhw>, <pos>n.</pos> <ety>[From Celtic; cf. Gael. <ets>crann</ets> a tree.]</ety> <def>One of the stockaded islands in Scotland and Ireland which in ancient times were numerous in the lakes of both countries. They may be regarded as the very latest class of prehistoric strongholds, reaching their greatest development in early historic times, and surviving through the Middle Ages. See also <cref>Lake dwellings</cref>, under <er>Lake</er>.</def>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranny</ent><br/
<hw>Cran"ny</hw> <pr>(kr<acr/n"n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crannies</plw> <pr>(-n<icr/z)</pr>.</plu> <ety>[F. <ets>cran</ets> notch, prob. from L. <ets>crena</ets> (a doubful word).]</ety> <sn>1.</sn> <def>A small, narrow opening, fissure, crevice, or chink, as  in a wall, or other substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In a firm building, the cavities ought not to be filled with rubbish, but with brick or stone fitted to the <qex>crannies</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He peeped into every <qex>cranny</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Glass Making)</fld> <def>A tool for forming the necks of bottles, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranny</ent><br/
<hw>Cran"ny</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crannied</conjf> <pr>(-n?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crannying</conjf>.]</vmorph> <sn>1.</sn> <def>To crack into, or become full of, crannies.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The ground did <qex>cranny</qex> everywhere.</q> <rj><qau>Golding.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To haunt, or enter by, crannies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All tenantless, save to the <qex>crannying</qex> wind.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cranny</ent><br/
<hw>Cran"ny</hw>, <pos>a.</pos> <ety>[Perh. for <ets>cranky</ets>. See <er>Crank</er>, <pos>a.</pos> ]</ety> <def>Quick; giddy; thoughtless.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crantara</ent><br/
<hw>Cran*ta"ra</hw> <pr>(kr?n-t?"r? <or/ -t?"r?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>cranntara</ets>.]</ety> <def>The fiery cross, used as a rallying signal in the Highlands of Scotland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crants</ent><br/
<hw>Crants</hw> <pr>(kr<acr/nts)</pr>, <pos>n.</pos> <ety>[Cf. D.  <ets>krans</ets>, G.  <ets>kranz</ets>.]</ety> <def>A garland carried before the bier of a maiden.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet here she is allowed her virgin <qex>crants</qex>, Her maiden strewments.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crap</ent><br/
<hw>Crap</hw> <pr>(kr<acr/p)</pr>, <pos>n.</pos> <sn>1.</sn> <def>In the game of craps, a first throw of the dice in which the total is two, three, or twelve, in which case the caster loses.  Also called <altname>craps</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>same as <er>excrement</er> and <er>feces</er>.</def> <mark>[vulgar]</mark><br/
<syn><b>Syn. --</b> shit.</syn>
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>nonsense; balderdash; bullshit; -- also used as an expletive.</def> <mark>[vulgar]</mark><br/
<syn><b>Syn. --</b> bullshit.</syn>
[<source>PJC</source>]</p>

<p><ent>Crap</ent><br/
<hw>Crap</hw> <pr>(kr<acr/p)</pr>, <pos>v.</pos> <sn>1.</sn> <def>to defecate.  Same as <er>take a crap</er>.</def> <mark>[vulgar]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Crapaud</ent><br/
<hw>Cra*paud"</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>crapawd</asp>, <asp>crapald</asp>, <asp>crepaud</asp>, etc.]</altsp> <ety>[F. <ets>crapaud</ets>.]</ety> <sn>1.</sn>  <def>A toad.</def> <mark>[Obs.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  (<it>Pronounced</it> <pr>kr<adot/`p<omac/"</pr>) <def>As a proper name, <altname>Johnny Crapaud</altname>, or <ex>Crapaud</ex>, a nickname for a Frenchman.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>large toothed frog of South and Central America (<spn>Leptodactylus pentadactylus</spn>) resembling the bullfrog.</def><br/
<syn><b>Syn. --</b> South American bullfrog, <spn>Leptodactylus pentadactylus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crapaudine</ent><br/
<hw>Crap"au*dine</hw> <pr>(kr?p"?-d?n)</pr>, <pos>a.</pos> <ety>[F., <pos>n.</pos>]</ety> <fld>(Arch.)</fld> <def>Turning on pivots at the top and bottom; -- said of a door.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crapaudine</ent><br/
<hw>Crap"au*dine</hw>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Far.)</fld> <def>An ulcer on the coronet of a horse.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crape</ent><br/
<hw>Crape</hw> <pr>(kr<amac/p)</pr>, <pos>n.</pos> <ety>[F.  <ets>cr<ecir/pe</ets>, fr. L.  <ets>crispus</ets> curled, crisped. See <er>Crisp</er>.]</ety> <def>A thin, crimped stuff, made of raw silk gummed and twisted on the mill.  Black crape is much used for mourning garments, also for the dress of some clergymen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A saint in <qex>crape</qex> is twice a saint in lawn.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crape myrtle</b></col> <fld>(Bot.)</fld>, <cd>a very ornamental shrub (<spn>Lagerstroemia Indica</spn>) from the East Indies, often planted in the Southern United States. Its foliage is like that of the myrtle, and the flower has wavy crisped petals.</cd> -- <col><b>Oriental crape</b></col>. <cd>See <er>Canton crape</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crape</ent><br/
<hw>Crape</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Craped</conjf> <pr>(kr<amac/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Craping</conjf>.]</vmorph> <ety>[F. <ets>cr<ecir/per</ets>, fr. L. <ets>crispare</ets> to curl, crisp, fr. <ets>crispus</ets>. See <er>Crape</er>, <pos>n.</pos>]</ety> <def>To form into ringlets; to curl; to crimp; to friz; <as>as, to <ex>crape</ex> the hair; to <ex>crape</ex> silk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hour for curling and <qex>craping</qex> the hair.</q> <rj><qau>Mad. D'Arblay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crape fern</ent><br/
<hw>crape fern</hw> <pos>n.</pos> <def>a fern of New Zealand (<spn>Leptopteris superba</spn>) with pinnate fronds and a densely woolly stalks; sometimes included in genus Todea.</def><br/
<syn><b>Syn. --</b> crape fern, Prince-of-Wales fern, Prince-of-Wales feather, Prince-of-Wales plume, <spn>Leptopteris superba</spn>, Todea superba.</syn> 
[<source>WordNet 1.5</source>]</p>

<p><ent>Crapefish</ent><br/
<hw>Crape"fish`</hw> <pr>(kr?p"f?sh`)</pr>, <pos>n.</pos> <def>Salted codfish hardened by pressure.</def>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crape jasmine</ent><br/
<hw>crape jasmine</hw> <pos>n.</pos> <def>a tropical shrub (<spn>Tabernaemontana divaricata</spn>), native to India, having glossy foliage and fragrant nocturnal flowers with crimped or wavy corollas; Northern India to Thailand.</def><br/
<syn><b>Syn. --</b> crepe jasmine, crepe gardenia, pinwheel flower, East Indian rosebay, Adam's apple, Nero's crown, coffee rose, Tabernaemontana divaricata.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crape myrtle</ent><br/
<hw>crape myrtle</hw> <pos>n.</pos> <def>an tall East Indian and Chinese shrub (<spn>Lagerstroemia indica</spn> of the loosestrife family, commonly planted in Southern and Western  U. S. as an ornamental shrub.  It has clusters of red, white, purple, or pink flowers.</def><br/
<syn><b>Syn. --</b> crepe myrtle, crepe flower, Lagerstroemia indica.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crapette</ent><br/
<hw>crapette</hw> <pos>n.</pos> <def>a game in which two play solitaire with separate packs.</def><br/
<syn><b>Syn. --</b> Russian bank.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crapnel</ent><br/
<hw>Crap"nel</hw> <pr>(kr?p"n<it>e</it>l)</pr>, <pos>n.</pos> <def>A hook or drag; a grapnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crapola</ent><br/
<hw>crapola</hw> <pos>n.</pos> <def>same as <er>bullshit</er>.</def> <mark>[vulgar]</mark><br/
[<source>PJC</source>]</p>

<p><ent>crapper</ent><br/
<hw>crap"per</hw> <pr>(kr<acr/p"p<etil/r)</pr>, <pos>n.</pos> <def>a toilet.</def> <mark>[vulgar]</mark> <br/
<syn><b>Syn. --</b> toilet, can, commode, pot, potty, stool, throne.</syn><br/
[<source>WordNet 1.5</source> + <source>PJC</source>]</p>

<p><ent>Crappie</ent><br/
<hw>Crap"pie</hw> <pr>(kr?p"p?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A kind of fresh-water bass of the genus <gen>Pomoxys</gen>, found in the rivers of the Southern United States and Mississippi valley. There are several species.</def> <altsp>[Written also <asp>croppie</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crapple</ent><br/
<hw>Crap"ple</hw> <pr>(kr?p"p'l)</pr>, <pos>n.</pos> <ety>[See <er>Graple</er>.]</ety> <def>A claw.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>crappy</ent><br/
<hw>crap"py</hw> <pr>(kr<acr/p"p<ycr/)</pr>, <pos>a.</pos> <def>of very poor quality.</def> <mark>[slang]</mark><br/
<syn><b>Syn. --</b> lousy; crummy.</syn>
[<source>PJC</source>]</p>

<p><ent>crap out</ent><br/
<hw>crap out</hw> <pr>(kr<acr/p` out")</pr>, <pos>v. i.</pos> <fld>(Games)</fld> <def>to throw a 2, 3, or 12 on the first throw in the game of craps, thereby losing that turn.</def><br/
[<source>PJC</source>]</p>

<p><ent>Craps</ent><br/
<hw>Craps</hw> <pr>(kr<acr/ps)</pr>, <pos>n.</pos> <def>A gambling game with dice.  It is one of the more popular games in casinos.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Crap shooting</ent><br/
<hw>Crap shooting</hw>. <def>Same as <er>Craps</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crapule</ent><br/
<ent>Crapula</ent><br/
<mhw>{ ||<hw>Crap"u*la</hw> <pr>(kr?p"?-l?)</pr>, <hw>Crap"ule</hw> <pr>(kr?p"?l)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>crapula</ets> intoxication.]</ety> <def>Same as <er>Crapulence</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crapulence</ent><br/
<hw>Crap"u*lence</hw> <pr>(-?-l<it>e</it>ns)</pr>, <pos>n.</pos> <def>The sickness occasioned by intemperance; surfeit.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crapulous</ent><br/
<ent>Crapulent</ent><br/
<mhw>{ <hw>Crap"u*lent</hw> <pr>(-l<it>e</it>nt)</pr>, <hw>Crap"u*lous</hw> <pr>(-l?s)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>crapulentus</ets>, <ets>crapulosus</ets>: cf. F.  <ets>crapuleux</ets>.]</ety> <def>Surcharged with liquor; sick from excessive indulgence in liquor; drunk; given to excesses.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crapy</ent><br/
<hw>Crap"y</hw> <pr>(kr?p"?)</pr>, <pos>a.</pos> <def>Resembling crape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crare</ent><br/
<hw>Crare</hw> <pr>(kr?r)</pr>, <pos>n.</pos> <ety>[OF. <ets>craier</ets>, <ets>creer</ets>, <ets>croyer</ets>, ship of war, LL. <ets>craiera</ets>, <ets>creyera</ets>, perh. from G. <ets>krieger</ets> warrior, or D. <ets>krijger</ets>.]</ety> <def>A slow unwieldy trading vessel.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>crayer</asp>, <asp>cray</asp>, and <asp>craie</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crase</ent><br/
<hw>Crase</hw> <pr>(kr<amac/z)</pr>, <pos>v. t.</pos> <ety>[See <er>Craze</er>.]</ety> <def>To break in pieces; to crack.</def> <mark>[Obs.]</mark> <ldquo/The pot was <xex>crased</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crash</ent><br/
<hw>Crash</hw> <pr>(kr<acr/sh)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crashed</conjf> <pr>(kr<acr/sht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crashing</conjf>.]</vmorph> <ety>[OE. <ets>crashen</ets>, the same word as <ets>crasen</ets> to break, E.  <ets>craze</ets>. See <er>Craze</er>.]</ety> <def>To break in pieces violently; to dash together with noise and violence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He shakt his head, and <qex>crasht</qex> his teeth for ire.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crash</ent><br/
<hw>Crash</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To make a loud, clattering sound, as of many things falling and breaking at once; to break in pieces with a harsh noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Roofs were blazing and walls <qex>crashing</qex> in every part of the city.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To break with violence and noise; <as>as, the chimney in falling <ex>crashed</ex> through the roof</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crash</ent><br/
<hw>Crash</hw>, <pos>n.</pos> <sn>1.</sn> <def>A loud, sudden, confused sound, as of many things falling and breaking at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wreck of matter and the <qex>crash</qex> of worlds.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ruin; failure; sudden breaking down, as of a business house or a commercial enterprise; <as>as, the stock market c<ex>rash</ex> of 1929</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>    The last week of October 1929 remains forever imprinted in the American memory.  It was, of course, the week of the Great <qex>Crash</qex>, the stock market collapse that signaled the collapse of the world economy and the Great Depression of the 1930s. From an all-time high of 381 in early September 1929, the Dow Jones Industrial Average drifted down to a level of 326 on October 22, then, in a series of traumatic selling waves, to 230 in the course of the following six trading days.<br/
    The stock market's drop was far from over; it continued its sickening slide for nearly three more years, reaching an ultimate low of 41 in July 1932. But it was that last week of October 1929 that burned itself into the American consciousness. After a decade of unprecedented boom and prosperity, there suddenly was panic, fear, a yawning gap in the American fabric.  The party was over.</q>  <qau>Wall street Journal, October 28, 1977.</qau><br/
[<source>PJC</source>]</p>

<p><ent>Crash</ent><br/
<hw>Crash</hw>, <pos>n.</pos> <ety>[L. <ets>crassus</ets> coarse. See <er>Crass</er>.]</ety> <def>Coarse, heavy, narrow linen cloth, used esp. for towels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crash-dive</ent><br/
<hw>crash-dive</hw> <pos>v. i. & t.</pos> <def>to descend steeply and rapidly; -- of aircraft.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crashing</ent><br/
<hw>Crash"ing</hw>, <pos>n.</pos> <def>The noise of many things falling and breaking at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There shall be . . . a great <qex>crashing</qex> from the hills.</q> <rj><qau>Zeph. i. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crasis</ent><br/
<hw>Cra"sis</hw> <pr>(kr?"s?s)</pr>, <pos>n.</pos> <ety>[LL., temperament, fr. Gr. <?/<?/<?/<?/, fr. <?/<?/<?/<?/ to mix.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A mixture of constituents, as of the blood; constitution; temperament.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>A contraction of two vowels (as the final and initial vowels of united words) into one long vowel, or into a diphthong; syn<ae/resis; as, <xex>cogo</xex> for <xex>coago</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craspedota</ent><br/
||<hw>Cras`pe*do"ta</hw> <pr>(kr?s`p?-d?"t?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ to be bordered or edged.]</ety> <fld>(Zool.)</fld> <def>The hydroid or naked-eyed medus<ae/. See <er>Hydroidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craspedote</ent><br/
<hw>Cras"pe*dote</hw> <pr>(kr?s"p?-d?t)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Craspedota.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crass</ent><br/
<hw>Crass</hw> <pr>(kr?s)</pr>, <pos>a.</pos> <ety>[L. <ets>crassus</ets> thick, fat, gross, prob. orig., closely woven.  See <er>Grease</er> animal fat, and cf. <er>Crate</er>, <er>Hurdle</er>.]</ety> <def>Gross; thick; dense; coarse; not elaborated or refined.</def> <ldquo/<xex>Crass</xex> and fumid exhalations.<rdquo/ <au>Sir. T. Browne.</au> <ldquo/<xex>Crass</xex> ignorance<rdquo/ <au>Cudworth.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crassamentum</ent><br/
<ent>Crassament</ent><br/
<mhw>{ <hw>Cras"sa*ment</hw> <pr>(kr?s"s?-m<it>e</it>nt)</pr>, ||<hw>Cras`sa*men"tum</hw> <pr>(-m?n"t?m)</pr>, }</mhw> <pos>n.</pos>  <ety>[L. <ets>crassamentum</ets>, fr.  <ets>crassare</ets> to make thick. See <er>Crass</er>, <pos>a.</pos>]</ety> <def>A semisolid mass or clot, especially that formed in coagulation of the blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crassiment</ent><br/
<hw>Cras"si*ment</hw> <pr>(kr?s"s?-m<it>e</it>nt)</pr>, <pos>n.</pos> <def>See <er>Crassament</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crassitude</ent><br/
<hw>Cras"si*tude</hw> <pr>(-t?d)</pr>, <pos>n.</pos> <ety>[L. <ets>crassitudo</ets>.]</ety> <def>Grossness; coarseness; thickness; density.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crassness</ent><br/
<hw>Crass"ness</hw> <pr>(kr?s"n?s)</pr>, <pos>n.</pos> <def>Grossness.</def> <mark>[Obs.]</mark>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crastination</ent><br/
<hw>Cras`ti*na"tion</hw> <pr>(kr?s`t?-n?"sh?n)</pr>, <pos>n.</pos> <ety>[L.  <ets>crastinus</ets> of to-morrow, from <ets>cras</ets> to-morrow.]</ety> <def>Procrastination; a putting off till to-morrow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crataegus</ent><br/
||<hw>Cra*t<ae/"gus</hw> <pr>(kr?-t?"g?s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>krataigo`s</grk>.]</ety> <fld>(Bot.)</fld> <def>A genus of small, hardy trees, including the hawthorn, much used for ornamental purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cratch</ent><br/
<hw>Cratch</hw> <pr>(kr?ch; 224)</pr>, <pos>n.</pos> <ety>[OE. <ets>cracche</ets>, <ets>crecche</ets>, F. <ets>cr<egrave/che</ets> crib, manger, fr. OHG. <ets>krippa</ets>, <ets>krippea</ets>, G. <ets>krippe</ets> crib. See <er>Crib</er>.]</ety> <def>A manger or open frame for hay; a crib; a rack.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Begin from first where He encradled was,<br/
In simple <qex>cratch</qex>, wrapt in a wad of hay.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cratch cradle</b></col>, <cd>a representation of the figure of the cratch, made upon the fingers with a string; cat's cradle; -- called also <altname>scratch cradle</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crate</ent><br/
<hw>Crate</hw> <pr>(kr<amac/t)</pr>, <pos>n.</pos> <ety>[L. <ets>cratis</ets> hurdle; perh. akin to E.  <ets>cradle</ets>.  See <er>Hurdle</er>, and cf. <er>Crate</er> a framework.]</ety> <sn>1.</sn> <def>A large basket or hamper of wickerwork, used for the transportation of china, crockery, and similar wares.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A box or case whose sides are of wooden slats with interspaces, -- used especially for transporting fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crate</ent><br/
<hw>crate</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Crating</conjf>.]</vmorph> <def>To pack in a crate or case for transportation; <as>as, to <ex>crate</ex> a sewing machine; to <ex>crate</ex> peaches</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crateful</ent><br/
<hw>crate"ful</hw> <pos>n.</pos> <def>the quantity contained in a crate.</def><br/
<syn><b>Syn. --</b> crate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crater</ent><br/
<hw>Cra"ter</hw> <pr>(kr<amac/"t<etil/r)</pr>, <pos>n.</pos> <ety>[L.  <ets>crater</ets>, <ets>cratera</ets>, a mixing vessel, the mouth of a volcano, Gr. <grk>krath`r</grk>, fr. <grk>keranny`nai</grk> to mix; cf. Skr. <ets><cced/r<imac/</ets> to mix, <ets><cced/ir</ets> to cook, <ets><cced/r<amac/</ets> to cook. Cf. <er>Grail</er>, in Holy <ets>Grail</ets>.]</ety> <sn>1.</sn> <def>The basinlike opening or mouth of a volcano, through which the chief eruption comes; similarly, the mouth of a geyser, about which a cone of silica is often built up.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>The pit left by the explosion of a bomb, shell, or mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>A constellation of the southen hemisphere; -- called also the <altname>Cup</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crateriform</ent><br/
<hw>Cra*ter"i*form</hw> <pr>(kr?-t?r"?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cratera</ets> + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the form of a shallow bowl; -- said of a corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craterous</ent><br/
<hw>Cra"ter*ous</hw> <pr>(kr?"t?r-?s)</pr>, <pos>a.</pos> <def>Pertaining to, or resembling, a crater.</def> <mark>[R.]</mark>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craunch</ent><br/
<hw>Craunch</hw> <pr>(kr?nch)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Craunched</conjf> <pr>(kr?ncht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Craunching</conjf>.]</vmorph> <ety>[See <er>Crunch</er>.]</ety> <def>To crush with the teeth; to chew with violence and noise; to crunch.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cravat</ent><br/
<hw>Cra*vat"</hw> <pr>(kr?-v?t")</pr>, <pos>n.</pos> <ety>[F. <ets>cravate</ets>, fr. <ets>Cravate</ets> a Croat, an inhabitant of Croatia, one of a body of Austrian troops, from whom, in 1636, this article of dress was adopted in France.]</ety> <def>A neckcloth; a piece of silk, fine muslin, or other cloth, worn by men about the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While his wig was combed and his <qex>cravat</qex> tied.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cravatted</ent><br/
<hw>Cra*vat`ted</hw> <pr>(kr?-v?t"t?d)</pr>, <pos>a.</pos> <def>Wearing a cravat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The young men faultlessly appointed, handsomely <qex>cravatted</qex>.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crave</ent><br/
<hw>Crave</hw> <pr>(kr<amac/v)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Craved</conjf> <pr>(kr<amac/vd)</pr>; <pos>p pr. & vb. n.</pos> <conjf>Craving</conjf>.]</vmorph> <ety>[AS. <ets>crafian</ets>; akin to Icel. <ets>krefja</ets>, Sw. <ets>kr<aum/fva</ets>, Dan.  <ets>kr<ae/ve</ets>.]</ety> <sn>1.</sn> <def>To ask with earnestness or importunity; to ask with submission or humility; to beg; to entreat; to beseech; to implore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>crave</qex> your honor's pardon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Joseph . . . went in boldly unto Pilate, and <qex>craved</qex> the body of Jesus.</q> <rj><qau>Mark xv. 43.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To call for, as a gratification; to long for; hence, to require or demand; <as>as, the stomach <ex>craves</ex> food</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His path is one that eminently <qex>craves</qex> weary walking.</q> <rj><qau>Edmund Gurney.</qau></rj></p>

<p><syn><b>Syn.</b> -- To ask; seek; beg; beseech; implore; entreat; solicit; request; supplicate; adjure.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crave</ent><br/
<hw>Crave</hw>, <pos>v. i.</pos> <def>To desire strongly; to feel an insatiable longing; <as>as, a <ex>craving</ex> appetite</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Once one may <qex>crave</qex> for love.</q> <rj><qau>Suckling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craven</ent><br/
<hw>Cra"ven</hw> <pr>(kr?"v'n)</pr>, <pos>a.</pos> <ety>[OE.  <ets>cravant</ets>, <ets>cravaunde</ets>, OF. <ets>cravant<?/</ets> struck down, p. p. of <ets>cravanter</ets>, <ets>crevanter</ets>, to break, crush, strike down, fr. an assumed LL.  <ets>crepantare</ets>, fr. L.  <ets>crepans</ets>, p. pr. of <ets>crepare</ets> to break, crack, rattle. Cf. <er>Crevice</er>, <er>Crepitate</er>.]</ety> <def>Cowardly; fainthearted; spiritless.</def> <ldquo/His <xex>craven</xex> heart.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The poor <qex>craven</qex> bridegroom said never a word.</q> <rj><qau>Sir. W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>craven</qex> fear of the sarcasm of Dorset.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craven</ent><br/
<hw>Cra"ven</hw>, <pos>n.</pos> <ety>[Formerly written also <ets>cravant</ets> and <ets>cravent</ets>.]</ety> <def>A recreant; a coward; a weak-hearted, spiritless fellow. See <er>Recreant</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q><qpers>King Henry</qpers>.  Is it fit this soldier keep his oath?<br/
<qpers>Fluellen</qpers>.  He is a <qex>craven</qex> and a villain else.</q> <rj><qau>Shak.</qau></rj></p>

<p><syn><b>Syn.</b> -- Coward; poltroon; dastard.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craven</ent><br/
<hw>Cra"ven</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cravened</conjf> <pr>(-v'nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cravening</conjf>.]</vmorph> <def>To make recreant, weak, spiritless, or cowardly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a prohibition so divine,<br/
That <qex>cravens</qex> my weak hand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craver</ent><br/
<hw>Crav"er</hw> <pr>(kr?v"?r)</pr>, <pos>n.</pos> <def>One who craves or begs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craving</ent><br/
<hw>Crav"ing</hw> <pr>(-?ng)</pr>, <pos>n.</pos> <def>Vehement or urgent desire; longing for; beseeching.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A succession of <qex>cravings</qex> and satiety.</q> <rj><qau>L'Estrange.</qau></rj></p>

<p>-- <wordforms><wf>Crav"ing*ly</wf>, <pos>adv.</pos> -- <wf>Crav"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craw</ent><br/
<hw>Craw</hw> <pr>(kr<add/)</pr>, <pos>n.</pos> <ety>[Akin to D.  <ets>kraag</ets> neck, collar, G.  <ets>kragen</ets>, Sw.  <ets>kr<aum/fva</ets> craw, Dan.  <ets>kro</ets>, and possibly to Gr. <?/<?/<?/<?/ (E. <ets>bronchus</ets>), or <grk>bro`chqos</grk> throat. <root/25. Cf. <er>Crag</er> neck.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The crop of a bird.</def> <sd>(b)</sd> <def>The stomach of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crawdaddy</ent><br/
<ent>crawdad</ent><br/
<mhw><hw>craw"dad</hw>  <pr>(kr<add/"d<acr/d)</pr>, <hw>craw"dad*dy</hw></mhw> <pr>(kr<add/"d<acr/d*d<ycr/)</pr>, <pos>n.</pos>  <fld>(Zool.)</fld> <def>same as <er>crawfish</er></def><br/
[<source>PJC</source>]</p>

<p><ent>Crayfish</ent><br/
<ent>Crawfish</ent><br/
<mhw>{ <hw>Craw"fish`</hw> <pr>(kr<add/"f<icr/sh`)</pr>, <hw>Cray"fish`</hw> <pr>(kr<amac/"f<icr/sh`)</pr> }</mhw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-fishes</plw> or <plw>-fish</plw>.</plu> <ety>[Corrupted fr. OE.  <ets>crevis</ets>, <ets>creves</ets>, OF.  <ets>crevice</ets>, F.  <ets><eacute/crevisse</ets>, fr. OHG.  <ets>krebiz</ets> crab, G.  <ets>krebs</ets>. See <er>Crab</er>. The ending <ets>-fish</ets> arose from confusion with E. <ets>fish</ets>.]</ety> <fld>(Zool.)</fld> <def>Any decapod crustacean of the family <fam>Astacid<ae/</fam> (genera <gen>Cambarus</gen> and <gen>Cambarus</gen>), resembling the lobster, but smaller, and found in fresh waters. Crawfishes are esteemed very delicate food both in Europe and America. The North American species are numerous and mostly belong to the genus <gen>Cambarus</gen>. The blind crawfish of the Mammoth Cave is <spn>Cambarus pellucidus</spn>. The common European species is <spn>Astacus fluviatilis</spn>.</def><br/
<syn><b>Syn. --</b> crawdad, crawdaddy.</syn><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>tiny lobsterlike crustaceans usually boiled briefly.</def><br/
<syn><b>Syn. --</b> crawdad, ecrevisse.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>a large edible marine crustacean having a spiny carapace but lacking the large pincers of true lobsters.</def><br/
<syn><b>Syn. --</b> spiny lobster, langouste, rock lobster, crayfish, sea crawfish.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crawfish</ent><br/
<hw>crawfish</hw> <pos>v. i.</pos> <def>to back out in a humilating manner; <as>as, We'll have to <ex>crawfish</ex> out from meeting with him</as>.</def><br/
<syn><b>Syn. --</b> retreat, back out, back away, crawfish out, withdraw.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crawford</ent><br/
<hw>Craw"ford</hw> <pr>(kr<add/"f<etil/rd)</pr>, <pos>n.</pos> <def>A Crawford peach; a well-known freestone peach, with yellow flesh, first raised by Mr. <person>William <etsep>Crawford</etsep></person>, of New Jersey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crawl</ent><br/
<hw>Crawl</hw> <pr>(kr<add/l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crawled</conjf> <pr>(kr<add/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crawling</conjf>.]</vmorph> <ety>[Dan.  <ets>kravle</ets>, or Icel. <ets>krafla</ets>, to paw, scrabble with the hands; akin to Sw. <ets>kr<aum/la</ets> to crawl; cf. LG.  <ets>krabbeln</ets>, D.  <ets>krabbelen</ets> to scratch.]</ety> <sn>1.</sn> <def>To move slowly by drawing the body along the ground, as a worm; to move slowly on hands and knees; to creep.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A worm finds what it searches after only by feeling, as it <qex>crawls</qex> from one thing to another.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Hence, <def>to move or advance in a feeble, slow, or timorous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was hardly able to <qex>crawl</qex> about the room.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The meanest thing that <qex>crawl'd</qex> beneath my eyes.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 341 pr=PK --></p>

<p><sn>3.</sn> <def>To advance slowly and furtively; to insinuate one's self; to advance or gain influence by servile or obsequious conduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Secretly <qex>crawling</qex> up the battered walls.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hath <qex>crawled</qex> into the favor of the king.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Absurd opinions <qex>crawl</qex> about the world.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To have a sensation as of insect creeping over the body; <as>as, the flesh <ex>crawls</ex></as>.  See <er>Creep</er>, <pos>v. i.</pos>, 7.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crawl</ent><br/
<hw>Crawl</hw> <pr>(kr?l)</pr>, <pos>n.</pos> <def>The act or motion of crawling; slow motion, as of a creeping animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crawl</ent><br/
<hw>Crawl</hw>, <pos>n.</pos> <ety>[Cf. <er>Kraal</er>.]</ety> <def>A pen or inclosure of stakes and hurdles on the seacoast, for holding fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crawler</ent><br/
<hw>Crawl"er</hw> <pr>(kr?l"?r)</pr>, <pos>n.</pos> <def>One who, or that which, crawls; a creeper; a reptile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crawly</ent><br/
<hw>Crawl"y</hw> <pr>(kr?l"?)</pr>, <pos>a.</pos> <def>Creepy.</def> <mark>[Colloq.]</mark></p>

<p><ent>Crawl stroke</ent><br/
<hw>Crawl stroke</hw>. <fld>(Swimming)</fld> <def>A racing stroke, in which the swimmer, lying flat on the water with face submerged, takes alternate overhand arm strokes while moving his legs up and down alternately from the knee.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crayer</ent><br/
<ent>Cray</ent><br/
<mhw>{ <hw>Cray</hw> <pr>(kr<amac/)</pr>, <hw>Cray"er</hw> <pr>(-<etil/r)</pr> }</mhw>, <pos>n.</pos> <def>See <er>Crare</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crayfish</ent><br/
<hw>Cray"fish</hw> <pr>(kr<amac/"f<icr/sh)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Crawfish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crayon</ent><br/
<hw>Cray"on</hw> <pr>(kr?"?n)</pr>, <pos>n.</pos> <ety>[F., a crayon, a lead pencil (<ets>crayon Cont<eacute/</ets> Cont<eacute/'s pencil, <it>i. e.</it>, one made a black compound invented by Cont<eacute/), fr. <ets>craie</ets> chalk, L.  <ets>creta</ets>; said to be, properly, Cretan earth, fr. <ets>Creta</ets> the island Crete.  Cf. <er>Cretaceous</er>.]</ety> <sn>1.</sn> <def>An implement for drawing, made of clay and plumbago, or of some preparation of chalk, usually sold in small prisms or cylinders.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let no day pass over you . . . without giving some strokes of the pencil or the <qex>crayon</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The black crayon gives a deeper black than the lead pencil. This and the colored crayons are often called <xex>chalks</xex>. The red crayon is also called <xex>sanguine</xex>. See <er>Chalk</er>, and <er>Sanguine</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A crayon drawing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Electricity)</fld> <def>A pencil of carbon used in producing electric light.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crayon board</b></col>, <cd>cardboard with a surface prepared for crayon drawing.</cd> -- <col><b>Crayon drawing</b></col>, <cd>the act or art of drawing with crayons; a drawing made with crayons.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crayon</ent><br/
<hw>Cray"on</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crayoned</conjf> <pr>(-?nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crayoning</conjf>.]</vmorph> <ety>[Cf. F.  <ets>crayonner</ets>.]</ety> <def>To sketch, as with a crayon; to sketch or plan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He soon afterwards composed that discourse, conformably to the plan which he had <qex>crayoned</qex> out.</q> <rj><qau>Malone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crax</ent><br/
<hw>Crax</hw> <pr>(kr<acr/ks)</pr>, <pos>n.</pos> <def>the type genus of the <fam>Cracidae</fam>; curassows.</def><br/
<syn><b>Syn. --</b> genus <gen>Crax</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Craze</ent><br/
<hw>Craze</hw> <pr>(kr<amac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crazed</conjf> <pr>(kr<amac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crazing</conjf>.]</vmorph> <ety>[OE.  <ets>crasen</ets> to break, fr. Scand., perh. through OF.; cf. Sw.  <ets>krasa</ets> to crackle, <ets>sl<aring/ i kras</ets>, to break to pieces, F. <ets><eacute/craser</ets> to crush, fr. the Scand. Cf. <er>Crash</er>.]</ety> <sn>1.</sn> <def>To break into pieces; to crush; to grind to powder. See <er>Crase</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God, looking forth, will trouble all his host, And <qex>craze</qex> their chariot wheels.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To weaken; to impair; to render decrepit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Till length of years,<br/
And sedentary numbness, <qex>craze</qex> my limbs.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To derange the intellect of; to render insane.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Any man . . . that is <qex>crazed</qex> and out of his wits.</q> <rj><qau>Tilloston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Grief hath <qex>crazed</qex> my wits.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craze</ent><br/
<hw>Craze</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be crazed, or to act or appear as one that is crazed; to rave; to become insane.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She would weep and he would <qex>craze</qex>.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To crack, as the glazing of porcelain or pottery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craze</ent><br/
<hw>Craze</hw>, <pos>n.</pos> <sn>1.</sn> <def>Craziness; insanity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A strong habitual desire or fancy; a crotchet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was quite a <qex>craze</qex> with him [Burns] to have his Jean dressed genteelly.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A temporary passion or infatuation, as for same new amusement, pursuit, or fashion; a fad; <as>as, the bric-a-brac <ex>craze</ex>; the <ae/sthetic <ex>craze</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Various <qex>crazes</qex> concerning health and disease.</q> <rj><qau>W. Pater.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Ceramics)</fld> <def>A crack in the glaze or enamel such as is caused by exposure of the pottery to great or irregular heat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crazedness</ent><br/
<hw>Craz"ed*ness</hw> <pr>(-?d-n?s)</pr>, <pos>n.</pos> <def>A broken state; decrepitude; an impaired state of the intellect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crazing-mill</ent><br/
<ent>Craze-mill</ent><br/
<mhw>{ <hw>Craze"-mill`</hw> <pr>(kr<amac/z"m<icr/l`)</pr>, <hw>Craz"ing-mill`</hw> <pr>(kr?"z?ng-)</pr> }</mhw>, <pos>n.</pos> <ety>[See 1st <er>Craze</er>.]</ety> <def>A mill for grinding tin ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crazily</ent><br/
<hw>Cra"zi*ly</hw> <pr>(kr<amac/"z<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a crazy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Craziness</ent><br/
<hw>Cra"zi*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of being broken down or weakened; <as>as, the <ex>craziness</ex> of a ship, or of the limbs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being broken in mind; imbecility or weakness of intellect; derangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crazing</ent><br/
<hw>Craz"ing</hw> <pr>(?)</pr>, <pos>p. pr. & vb. n.</pos> <mord>of <er>Craze</er>, <pos>v.</pos></mord> <specif>Hence:</specif> <pos>n.</pos> <def>Fine cracks resulting from shrinkage on the surface of glazed pottery, concrete, or other material. The admired crackle in some Oriental potteries and porcelains is crazing produced in a foreseen and regulated way. In common pottery it is often the result of exposure to undue heat, and the beginning of disintegration.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crazy</ent><br/
<hw>Cra"zy</hw> <pr>(kr<amac/"z<ycr/)</pr>, <pos>a.</pos> <ety>[From <er>Craze</er>.]</ety> <sn>1.</sn> <def>Characterized by weakness or feebleness; decrepit; broken; falling to decay; shaky; unsafe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Piles of mean and<qex>crazy</qex> houses.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One of great riches, but a <qex>crazy</qex> constitution.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . got a <qex>crazy</qex> boat to carry them to the island.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Broken, weakened, or dissordered in intellect; shattered; demented; deranged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Over moist and <qex>crazy</qex> brains.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Inordinately desirous; foolishly eager.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The girls were <qex>crazy</qex> to be introduced to him.</q> <rj><qau>R. B. Kimball.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crazy bone</b></col>, <cd>the bony projection at the end of the elbow (<xex>olecranon</xex>), behind which passes the ulnar nerve; -- so called on account of the curiously painful tingling felt, when, in a particular position, it receives a blow; -- called also <altname>funny bone</altname>.</cd> -- <col><b>Crazy quilt</b></col>, <cd>a bedquilt made of pieces of silk or other material of various sizes, shapes, and colors, fancifully stitched together without definite plan or arrangement.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>crazyweed</ent><br/
<hw>crazyweed</hw> <pos>n.</pos> <def>any of several leguminous plants of Western North America causing locoism in livestock.</def><br/
<syn><b>Syn. --</b> locoweed, crazy weed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Creable</ent><br/
<hw>Cre"a*ble</hw> <pr>(kr?"?-b'l)</pr>, <pos>a.</pos> <ety>[L. <ets>creabilis</ets>, from <ets>creare</ets> to create. See <er>Create</er>.]</ety> <def>Capable of being created.</def> <mark>[Obs.]</mark>  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaght</ent><br/
<hw>Creaght</hw> <pr>(kr<amac/t)</pr>, <pos>n.</pos> <ety>[Ir. & Gael.  <ets>graidh</ets>, <ets>graigh</ets>.]</ety> <def>A drove or herd.</def> <mark>[Obs.]</mark>  <rj><au>Haliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaght</ent><br/
<hw>Creaght</hw>, <pos>v. i.</pos> <def>To graze.</def> <mark>[Obs.]</mark>  <rj><au>Sir. L. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creak</ent><br/
<hw>Creak</hw> <pr>(kr<emac/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Creaked</conjf> <pr>(kr<emac/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Creaking</conjf>.]</vmorph> <ety>[OE.  <ets>creken</ets>, prob. of imitative origin; cf. E.  <ets>crack</ets>, <ets>and</ets>. D. <ets>krieken</ets> to crackle, chirp.]</ety> <def>To make a prolonged sharp grating or squeaking sound, as by the friction of hard substances; <as>as, shoes <ex>creak</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>creaking</qex> locusts with my voice conspire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Doors upon their hinges <qex>creaked</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creak</ent><br/
<hw>Creak</hw>, <pos>v. t.</pos> <def>To produce a creaking sound with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Creaking</qex> my shoes on the plain masonry.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creak</ent><br/
<hw>Creak</hw> <pr>(kr<emac/k)</pr>, <pos>n.</pos> <def>The sound produced by anything that creaks; a creaking.</def>  <rj><au>Roget.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaking</ent><br/
<hw>Creak"ing</hw>, <pos>n.</pos> <def>A harsh grating or squeaking sound, or the act of making such a sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Start not at the <qex>creaking</qex> of the door.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>creaky</ent><br/
<hw>creaky</hw> <pos>adj.</pos> <sn>1.</sn> <def>worn down with age or use; in poor condition.</def><br/
<syn><b>Syn. --</b> decrepit, flea-bitten, run-down, rundown, woebegone.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn>  <def>producing a rasping or grating sound under some circumsatance; <as>as, they were alerted by the <ex>creaky</ex> stairs; my <ex>creaky</ex> kitchen door needs oiling</as>.</def><br/
<syn><b>Syn. --</b> creaking, screaky.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>aching when flexed; -- of body parts; <as>as, my <ex>creaky</ex> old joints</as>.</def><br/
<syn><b>Syn. --</b> arthritic, rheumatic, rheumatoid.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cream</ent><br/
<hw>Cream</hw> <pr>(kr<emac/m)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<ecir/me</ets>, perh. fr. LL. <ets>crema</ets> cream of milk; cf. L. <ets>cremor</ets> thick juice or broth, perh. akin to <ets>cremare</ets> to burn.]</ety> <sn>1.</sn> <def>The rich, oily, and yellowish part of milk, which, when the milk stands unagitated, rises, and collects on the surface. It is the part of milk from which butter is obtained.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The part of any liquor that rises, and collects on the surface.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A delicacy of several kinds prepared for the table from cream, etc., or so as to resemble cream.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A cosmetic; a creamlike medicinal preparation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In vain she tries her paste and <qex>creams</qex>,<br/
To smooth her skin or hide its seams.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The best or choicest part of a thing; the quintessence; <as>as, the <ex>cream</ex> of a jest or story; the <ex>cream</ex> of a collection of books or pictures</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Welcome, O flower and <qex>cream</qex> of knights errant.</q> <rj><qau>Shelton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bavarian cream</b></col>, <cd>a preparation of gelatin, cream, sugar, and eggs, whipped; -- to be eaten cold.</cd> -- <col><b>Cold cream</b></col>, <cd>an ointment made of white wax, almond oil, rose water, and borax, and used as a salve for the hands and lips.</cd> -- <col><b>Cream cheese</b></col>, <cd>a kind of cheese made from curd from which the cream has not been taken off, or to which cream has been added.</cd> -- <col><b>Cream gauge</b></col>, <cd>an instrument to test milk, being usually a graduated glass tube in which the milk is placed for the cream to rise.</cd> -- <col><b>Cream nut</b></col>, <cd>the Brazil nut.</cd> -- <col><b>Cream of lime</b></col>. <sd>(a)</sd> <cd>A scum of calcium carbonate which forms on a solution of milk of lime from the carbon dioxide of the air.</cd> <sd>(b)</sd> <cd>A thick creamy emulsion of lime in water.</cd> -- <col><b>Cream of tartar</b></col> <fld>(Chem.)</fld>, <cd>purified tartar or argol; so called because of the crust of crystals which forms on the surface of the liquor in the process of purification by recrystallization. It is a white crystalline substance, with a gritty acid taste, and is used very largely as an ingredient of baking powders; -- called also <altname>potassium bitartrate</altname>, <altname>acid potassium tartrate</altname>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream</ent><br/
<hw>Cream</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Creamed</conjf> <pr>(kr?md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Creaming</conjf>.]</vmorph> <sn>1.</sn> <def>To skim, or take off by skimming, as cream.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take off the best or choicest part of.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To furnish with, or as with, cream.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Creaming</qex> the fragrant cups.</q> <rj><qau>Mrs. Whitney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cream butter</b></col> <fld>(Cooking)</fld>, <cd>to rub, stir, or beat, butter till it is of a light creamy consistency.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream</ent><br/
<hw>Cream</hw>, <pos>v. i.</pos> <def>To form or become covered with cream; to become thick like cream; to assume the appearance of cream; hence, to grow stiff or formal; to mantle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There are a sort of men whose visages<br/
Do <qex>cream</qex> and mantle like a standing pool.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creamcake</ent><br/
<hw>Cream"cake`</hw> <pr>(-k?k`)</pr>, <pos>n.</pos> <fld>(Cookery)</fld> <def>A kind of cake filled with custard made of cream, eggs, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream-colored</ent><br/
<hw>Cream"-col`ored</hw> <pr>(-k?l`?rd)</pr>, <pos>a.</pos> <def>Of the color of cream; light yellow.</def> <ldquo/<xex>Cream-colored</xex> horses.<rdquo/  <rj><au>Hazlitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>creamcups</ent><br/
<hw>creamcups</hw> <pos>n.</pos> <def>a California plant (<spn>Platystemon californicus</spn>) with small pale yellow flowers.</def><br/
<syn><b>Syn. --</b> <spn>Platystemon californicus</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>creamer</ent><br/
<hw>creamer</hw> <pos>n.</pos> <def>a small pitcher for serving cream; <as>as, the coffee set included a sugar bowl and <ex>creamer</ex></as>.</def><br/
<syn><b>Syn. --</b> cream pitcher.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Creamery</ent><br/
<hw>Cream"er*y</hw> <pr>(-?r-?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Creameries</plw> <pr>(-<?/z)</pr>.</plu> <ety>[CF. F. <ets>cr<?/meric</ets>.]</ety> <sn>1.</sn> <def>A place where butter and cheese are made, or where milk and cream are put up in cans for market.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place or apparatus in which milk is set for raising cream.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An establishment where cream is sold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream-faced</ent><br/
<hw>Cream"-faced`</hw> <pr>(kr?m"f?st`)</pr>, <pos>a.</pos> <def>White or pale, as the effect of fear, or as the natural complexion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>cream-faced</qex> loon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream-fruit</ent><br/
<hw>Cream"-fruit`</hw> <pr>(kr?m"fr?t`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant of Sierra Leone which yields a wholesome, creamy juice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaminess</ent><br/
<hw>Cream"i*ness</hw> <pr>(-?-n?s)</pr>, <pos>n.</pos> <def>The quality of being creamy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream laid</ent><br/
<hw>Cream" laid`</hw> <pr>(kr?m" l?d`)</pr>. <def>See under <er>Laid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream-slice</ent><br/
<hw>Cream"-slice`</hw> <pr>(-sl?s`)</pr>, <pos>n.</pos> <def>A wooden knife with a long thin blade, used in handling cream or ice cream.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cream-white</ent><br/
<hw>Cream"-white`</hw> <pr>(-hw?t`)</pr>, <pos>a.</pos> <def>As white as cream.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creamy</ent><br/
<hw>Cream"y</hw> <pr>(kr?m"?)</pr>, <pos>a.</pos> <def>Full of, or containing, cream; resembling cream, in nature, appearance, or taste; creamlike; unctuous.</def> <ldquo/<xex>Creamy</xex> bowls.<rdquo/ <au>Collins.</au> <ldquo/Lines of <xex>creamy</xex> spray.<rdquo/ <au>Tennyson.</au> <ldquo/Your <xex>creamy</xex> words but cozen.<rdquo/ <au>Beau. & Fl.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creance</ent><br/
<hw>Cre"ance</hw> <pr>(kr<emac/"<it>a</it>ns)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<eacute/ance</ets>, lit., credence, fr. L. <ets>credere</ets> to trust. See <er>Credence</er>.]</ety> <sn>1.</sn> <def>Faith; belief; creed.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Falconry)</fld> <def>A fine, small line, fastened to a hawk's leash, when it is first lured.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creance</ent><br/
<hw>Cre"ance</hw> <pr>(kr<emac/"<it>a</it>ns)</pr>, <pos>v. i. & t.</pos> <def>To get on credit; to borrow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creant</ent><br/
<hw>Cre"ant</hw> <pr>(kr<emac/"<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[L.  <ets>creans</ets>, p. pr. of <ets>creare</ets> to create.]</ety> <def>Creative; formative.</def> <mark>[R.]</mark>  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crease</ent><br/
<hw>Crease</hw> <pr>(kr<emac/s)</pr>, <pos>n.</pos> <def>See <er>Creese</er>.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crease</ent><br/
<hw>Crease</hw>, <pos>n.</pos> <ety>[Cf. LG. <ets>krus</ets>, G.  <ets>krause</ets>, crispness, <ets>krausen</ets>, <ets>kr<aum/usen</ets>, to crisp, curl, lay on folds; or perh. of Celtic origin; cf. Armor. <ets>kriz</ets> a wrinkle, crease, <ets>kriza</ets> to wrinkle, fold, W. <ets>crych</ets> a wrinkle, <ets>crychu</ets> to rumple, ripple, crease.]</ety> <sn>1.</sn> <def>A line or mark made by folding or doubling any pliable substance; hence, a similar mark, however produced.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cricket)</fld> <def>One of the lines serving to define the limits of the bowler and the striker.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Lacrosse)</fld> <def>The combination of four lines forming a rectangle inclosing either goal, or the inclosed space itself, within which no attacking player is allowed unless the ball is there; -- called also <altname>goal crease</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Bowling crease</b></col> <fld>(Cricket)</fld>, <cd>a line extending three feet four inches on each side of the central strings at right angles to the line between the wickets.</cd> -- <col><b>Return crease</b></col> <fld>(Cricket)</fld>, <cd>a short line at each end of the <xex>bowling crease</xex> and at right angles to it, extending toward the bowler.</cd> -- <col><b>Popping crease</b></col> <fld>(Cricket)</fld>,<cd>, a line drawn in front of the wicket, four feet distant from it, parallel to the <xex>bowling crease</xex> and at least as long as the latter.</cd>  <rj><au>J. H. Walsh (Encyc. of Rural Sports).</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crease</ent><br/
<hw>Crease</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Creased</conjf> <pr>(kr?st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Creasing</conjf>.]</vmorph> <def>To make a crease or mark in, as by folding or doubling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Creased</qex>, like dog's ears in a folio.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaser</ent><br/
<hw>Creas"er</hw> <pr>(kr?s"?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A tool, or a sewing-machine attachment, for making lines or creases on leather or cloth, as guides to sew by.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tool for making creases or beads, as in sheet iron, or for rounding small tubes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bookbinding)</fld> <def>A tool for making the band impression distinct on the back.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creasing</ent><br/
<hw>Creas"ing</hw> <pr>(kr<emac/s"<icr/ng)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A layer of tiles forming a corona for a wall.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creasote</ent><br/
<hw>Cre"a*sote</hw> <pr>(kr<emac/"<adot/*s<omac/t)</pr>, <pos>n.</pos> <def>See <er>Creosote</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creasy</ent><br/
<hw>Creas"y</hw> <pr>(kr?s"?)</pr>, <pos>a.</pos> <def>Full of creases.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creat</ent><br/
<hw>Cre"at</hw> <pr>(kr<emac/"<acr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<eacute/at</ets>, ultimately fr. L. <ets>creatus</ets> created, begotten; cf. It.  <ets>creato</ets> pupil, servant, Sp.  <ets>criado</ets> a servant, client.]</ety> <fld>(Man.)</fld> <def>An usher to a riding master.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatable</ent><br/
<hw>Cre*at"a*ble</hw> <pr>(kr?-?"t?-b'l)</pr>, <pos>a.</pos> <def>That may be created.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Create</ent><br/
<hw>Cre*ate"</hw> <pr>(kr<esl/*<amac/t")</pr>, <pos>a.</pos> <ety>[L. <ets>creatus</ets>, p. p. of <ets>creare</ets> to create; akin to Gr. <grk>krai`nein</grk> to accomplish, Skr. <ets>k<rsdot/</ets> to make, and to E. ending <ets>-cracy</ets> in <ets>aristocracy</ets>, also to <ets>crescent</ets>, <ets>cereal</ets>.]</ety> <def>Created; composed; begotten.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hearts <qex>create</qex> of duty and zeal.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Create</ent><br/
<hw>Cre*ate"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Created</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Creating</conjf>.]</vmorph> <sn>1.</sn> <def>To bring into being; to form out of nothing; to cause to exist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the beginning, God <qex>created</qex> the heaven and the earth.</q> <rj><qau>Gen. i. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To effect by the agency, and under the laws, of causation; to be the occasion of; to cause; to produce; to form or fashion; to renew.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your eye in Scotland<br/
Would <qex>create</qex> soldiers.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Create</qex> in me a clean heart.</q> <rj><qau>Ps. li. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To invest with a new form, office, or character; to constitute; to appoint; to make; <as>as, to <ex>create</ex> one a peer</as>.</def> <ldquo/I <xex>create</xex> you companions to our person.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatic</ent><br/
<hw>Cre*at"ic</hw> <pr>(kr?-?t"?k)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/, <?/<?/<?/, flesh.]</ety> <def>Relating to, or produced by, flesh or animal food; <as>as, <ex>creatic</ex> nausea</as>.</def> <altsp>[Written also <asp>kreatic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatin</ent><br/
<hw>Cre"a*tin</hw> <pr>(kr?"?-t?n)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ flesh.]</ety> <fld>(Physiol. Chem.)</fld> <def>A white, crystalline, nitrogenous substance found abundantly in muscle tissue.</def> <altsp>[Written also <asp>kreatine</asp>.]</altsp><-- = creatine, C4H9N3O2 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatinin</ent><br/
<hw>Cre*at"i*nin</hw> <pr>(kr?-?t"?-n?n)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>A white, crystalline, nitrogenous body closely related to creatin but more basic in its properties, formed from the latter by the action of acids, and occurring naturally in muscle tissue and in urine.</def> <altsp>[Written also <asp>kretinine</asp>.]</altsp><-- = creatinine, C4H7N3O --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creation</ent><br/
<hw>Cre*a"tion</hw> <pr>(kr?-A"sh?n)</pr>, <pos>n.</pos> <ety>[L.  <ets>creatio</ets>: cf. F.  <ets>cr<?/ation</ets>. <ets>See</ets> <er>Create</er>.]</ety> <sn>1.</sn> <def>The act of creating or causing to exist. Specifically, the act of bringing the universe or this world into existence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From the <qex>creation</qex> to the general doom.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As when a new particle of matter dotn begin to exist, <qex>in rerum natura</qex>, which had before no being; and this we call <qex>creation</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is created; that which is produced or caused to exist, as the world or some original work of art or of the imagination; nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We know that the whole <qex>creation</qex> groaneth.</q> <rj><qau>Rom. viii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A dagger of the mind, a false <qex>creation</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Choice pictures and <qex>creations</qex> of curious art.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of constituting or investing with a new character; appointment; formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An Irish peer of recent <qex>creation</qex>.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creational</ent><br/
<hw>Cre*a"tion*al</hw> <pr>(-<it>a</it>l)</pr>, <pos>a.</pos> <def>Of or pertaining to creation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creationism</ent><br/
<hw>Cre*a"tion*ism</hw> <pr>(-?z'm)</pr>, <pos>n.</pos> <def>The doctrine that a soul is specially created for each human being as soon as it is formed in the womb; -- opposed to <xex>traducianism</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creative</ent><br/
<hw>Cre*a"tive</hw> <pr>(-t?v)</pr>, <pos>a.</pos> <def>Having the power to create; exerting the act of creation.</def> <ldquo/<xex>Creative</xex> talent.<rdquo/  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>creative</qex> force exists in the germ.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creativeness</ent><br/
<hw>Cre*a"tive*ness</hw>, <pos>n.</pos> <def>The quality of being creative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creator</ent><br/
<hw>Cre*a"tor</hw> <pr>(kr<esl/*<amac/"t<etil/r)</pr>, <pos>n.</pos> <ety>[L.  <ets>creator</ets>: cf. F. <ets>cr<eacute/ateur</ets>.]</ety> <def>One who creates, produces, or constitutes. Specifically, the Supreme Being.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To sin's rebuke and my <qex>Creater's</qex> praise.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The poets and artists of Greece, who are at the same time its prophets, the <qex>creators</qex> of its divinities, and the revealers of its theological beliefs.</q> <rj><qau>Caird.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatorship</ent><br/
<hw>Cre*a"tor*ship</hw>, <pos>n.</pos> <def>State or condition of a creator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatress</ent><br/
<hw>Cre*a"tress</hw>, <pos>n.</pos> <ety>[L. <ets>creatrix</ets>: cf. F. <ets>cr<eacute/atrice</ets>.]</ety> <def>She who creates.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatrix</ent><br/
||<hw>Cre*a"trix</hw> <pr>(-tr?ks)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A creatress.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatural</ent><br/
<hw>Crea"tur*al</hw> <pr>(kr?"t?r-<ait/l; 135)</pr>, <pos>a.</pos> <def>Belonging to a creature; having the qualities of a creature.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creature</ent><br/
<hw>Crea"ture</hw> <pr>(kr<emac/"t<umac/r; 135)</pr>, <pos>n.</pos> <ety>[F.  <ets>cr<eacute/ature</ets>, L.  <ets>creatura</ets>. See <er>Create</er>.]</ety> <sn>1.</sn> <def>Anything created; anything not self-existent; especially, any being created with life; an animal; a man.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He asked water, a <qex>creature</qex> so common and needful that it was against the law of nature to deny him.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God's first <qex>creature</qex> was light.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On earth, join, all ye <qex>creatures</qex>, to extol<br/
Him first, him last, him midst, and without end.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And most attractive is the fair result<br/
Of thought, the <qex>creature</qex> of a polished mind.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A human being, in pity, contempt, or endearment; <as>as, a poor <ex>creature</ex>; a pretty <ex>creature</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world hath not a sweeter <qex>creature</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person who owes his rise and fortune to another; a servile dependent; an instrument; a tool.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>creature</qex> of the queen's, Lady Anne Bullen.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Both Charles himself and his <qex>creature</qex>, Laud.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A general term among farmers for horses, oxen, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Creature comforts</b></col>, <cd>those objects, as food, drink, and shelter, which minister to the comfort of the body.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Creatureless</ent><br/
<hw>Crea"ture*less</hw>, <pos>a.</pos> <def>Without created beings; alone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God was alone And <qex>creatureless</qex> at first.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaturely</ent><br/
<hw>Crea"ture*ly</hw>, <pos>a.</pos> <def>Creatural; characteristic of a creature.</def> <mark>[R.]</mark> <ldquo/<xex>Creaturely</xex> faculties.<rdquo/  <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creatureship</ent><br/
<hw>Crea"ture*ship</hw>, <pos>n.</pos> <def>The condition of being a creature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creaturize</ent><br/
<hw>Crea"tur*ize</hw> <pr>(-<imac/z)</pr>, <pos>v. t.</pos> <def>To make like a creature; to degrade</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Degrade and <qex>creaturize</qex> that mundane soul.</q> <rj><qau>Cudworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 342 pr=PK --></p>

<p><ent>Creaze</ent><br/
<hw>Creaze</hw> <pr>(kr<emac/z)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>The tin ore which collects in the central part of the washing pit or buddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crebricostate</ent><br/
<hw>Cre`bri*cos"tate</hw> <pr>(kr<emac/`br<icr/*k<ocr/s"t<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>creber</ets> close + <ets>costa</ets> rib.]</ety> <fld>(Zool.)</fld> <def>Marked with closely set ribs or ridges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crebrisulcate</ent><br/
<hw>Cre`bri*sul"cate</hw> <pr>(kr<emac/`br<icr/*s<ucr/l"k<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>creber</ets> close + <ets>sulcus</ets> furrow.]</ety> <fld>(Zool.)</fld> <def>Marked with closely set transverse furrows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crebritude</ent><br/
<hw>Creb"ri*tude</hw> <pr>(kr<emac/b"r<icr/*t<umac/d)</pr>, <pos>n.</pos> <ety>[L. <ets>crebritudo</ets>, fr. <ets>creber</ets> close.]</ety> <def>Frequency.</def> <mark>[Obs.]</mark> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crebrous</ent><br/
<hw>Cre"brous</hw> <pr>(kr<emac/"br<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>creber</ets> close set, frequent.]</ety> <def>Frequent; numerous.</def> <mark>[Obs.]</mark> <rj><au>Goodwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creche</ent><br/
||<hw>Cr<egrave/che</hw> <pr>(kr<asl/sh)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A public nursery, where the young children of poor women are cared for during the day, while their mothers are at work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a day-care center for young children.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>a three-dimensional model of the scene described in the Bible at the birth of Jesus Christ in a stable at Bethlehem, with Mary and Joseph near a manger in which a model of the infant Christ child is lain, and usually including figures of animals, shepherds, and the three wise men; -- also called a <altname>Nativity scene</altname>.  The figures in the scene are typically made as individual statues or figurines.  Smaller models are displayed in homes and other indoor locations during the Christmans season, and larger models, often life-size, may be displayed out of doors.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Biol.)</fld> <def>a nest where the young of several animals are cared for in a communal fashion.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crecy</ent><br/
<hw>Crecy</hw> <pos>n.</pos> <sn>1.</sn> <def>the first decisive battle of the Hundred Years' War; in 1346 the English under Edward III defeated the French under King Philip of Valois.</def><br/
<syn><b>Syn. --</b> battle of Crecy.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Credence</ent><br/
<hw>Cre"dence</hw> <pr>(kr<emac/"d<eit/ns)</pr>, <pos>n.</pos> <ety>[LL. <ets>credentia</ets>, fr. L. <ets>credens</ets>, <ets>-entis</ets>, p. pr. of <ets>credere</ets> to trust, believe: cf. OF. <ets>credence</ets>. See <er>Creed</er>, and cf. <er>Credent</er>, <er>Creance</er>.]</ety><-- now usu. <asp>credenza</asp> for the furniture --><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Reliance of the mind on evidence of facts derived from other sources than personal knowledge; belief; credit; confidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To give <qex>credence</qex> to the Scripture miracles.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An assertion which might easily find <qex>credence</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which gives a claim to credit, belief, or confidence; <as>as, a letter of <ex>credence</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>The small table by the side of the altar or communion table, on which the bread and wine are placed before being consecrated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A cupboard, sideboard, or cabinet, particularly one intended for the display of rich vessels or plate, and consisting chiefly of open shelves for that purpose.</def><-- from IT. credenza, same as credenza{2} --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credence</ent><br/
<hw>Cre"dence</hw>, <pos>v. t.</pos> <def>To give credence to; to believe.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credendum</ent><br/
||<hw>Cre*den"dum</hw> <pr>(kr?-d?n"d?m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Credenda</plw> <pr>(-d<?/)</pr>.</plu> <ety>[L., fr. <ets>credere</ets> to believe.]</ety> <fld>(Theol.)</fld> <def>A thing to be believed; an article of faith; -- distinguished from <contr>agendum</contr>, a practical duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great articles and <qex>credenda</qex> of Christianity.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credent</ent><br/
<hw>Cre"dent</hw> <pr>(kr?"d<it>e</it>nt)</pr>, <pos>a.</pos> <ety>[. <ets>credens</ets>, <ets>-entis</ets>, p. pr. of <ets>credere</ets> to trust, believe. See <er>Creed</er>.]</ety> <sn>1.</sn> <def>Believing; giving credence; credulous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If with too <qex>credent</qex> ear you list songs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having credit or authority; credible.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For my authority bears of a <qex>credent</qex> bulk.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credential</ent><br/
<hw>Cre*den"tial</hw> <pr>(kr<esl/*d<ecr/n"sh<ait/l)</pr>, <pos>a.</pos> <ety>[Cf. It. <ets>credenziale</ets>, fr. LL. <ets>credentia</ets>. See <er>Credence</er>.]</ety> <def>Giving a title or claim to credit or confidence; accrediting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>credential</qex> letters on both sides.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credential</ent><br/
<hw>Cre*den"tial</hw>, <pos>n.</pos> <ety>[Cf. It. <ets>credenziale</ets>.]</ety> <sn>1.</sn> <def>That which gives a title to credit or confidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Testimonials showing that a person is entitled to credit, or has right to exercise official power, as the letters given by a government to an ambassador or envoy, or a certificate that one is a duly elected delegate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The committee of estates excepted against the <qex>credentials</qex> of the English commissioners.</q> <rj><qau>Whitelocke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Had they not shown undoubted <qex>credentials</qex> from the Divine Person who sent them on such a message.</q> <rj><qau>Addison.</qau></rj></p>

<p><ent>credentialed</ent><br/
<hw>credentialed</hw> <pos>adj.</pos> <def>having an official document certifying fitness for a particular task.</def><br/
<syn><b>Syn. --</b> certificated, documented.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>credenza</ent><br/
<hw>credenza</hw> <pos>n.</pos> <sn>1.</sn> <def>a <er>credence{3}</er>.</def><br/
<syn><b>Syn. --</b> credence.</syn><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a buffet or sideboard similar to a credence{4}, especially one without legs.</def>  <au>MW10</au><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Credibility</ent><br/
<hw>Cred`i*bil"i*ty</hw> <pr>(kr<ecr/d`<icr/*b<icr/l"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cr<eacute/dibilit<eacute/</ets>.]</ety> <def>The quality of being credible; credibleness; <as>as, the <ex>credibility</ex> of facts; the <ex>credibility</ex> of witnesses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credible</ent><br/
<hw>Cred"i*ble</hw> <pr>(kr<ecr/d"<icr/*b'l)</pr>, <pos>a.</pos> <ety>[L. <ets>credibilis</ets>, fr. <ets>credere</ets>. See <er>Creed</er>.]</ety> <def>Capable of being credited or believed; worthy of belief; entitled to confidence; trustworthy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Things are made <qex>credible</qex> either by the known condition and quality of the utterer or by the manifest likelihood of truth in themselves.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A very diligent and observing person, and likewise very sober and <qex>credible</qex>.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credibleness</ent><br/
<hw>Cred"i*ble*ness</hw>, <pos>n.</pos> <def>The quality or state of being credible; worthiness of belief; credibility.</def> <mark>[R.]</mark> <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credibly</ent><br/
<hw>Cred"i*bly</hw>, <pos>adv.</pos> <def>In a manner inducing belief; <as>as, I have been <ex>credibly</ex> informed of the event</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credit</ent><br/
<hw>Cred"it</hw> <pr>(kr<ecr/d"<icr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<eacute/dit</ets> (cf. It. <ets>credito</ets>), L. <ets>creditum</ets> loan, prop. neut. of <ets>creditus</ets>, p. p. of <ets>credere</ets> to trust, loan, believe. See <er>Creed</er>.]</ety> <sn>1.</sn> <def>Reliance on the truth of something said or done; belief; faith; trust; confidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When Jonathan and the people heard these words they gave no <qex>credit</qex> unto them, nor received them.</q> <rj><qau>1 Macc. x. 46.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reputation derived from the confidence of others; esteem; honor; good name; estimation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>John Gilpin was a citizen<br/
Of <qex>credit</qex> and renown.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A ground of, or title to, belief or confidence; authority derived from character or reputation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The things which we properly believe, be only such as are received on the <qex>credit</qex> of divine testimony.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which tends to procure, or add to, reputation or esteem; an honor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I published, because I was told I might please such as it was a <qex>credit</qex> to please.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Influence derived from the good opinion, confidence, or favor of others; interest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Having <qex>credit</qex> enough with his master to provide for his own interest.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Com.)</fld> <def>Trust given or received; expectation of future playment for property transferred, or of fulfillment or promises given; mercantile reputation entitling one to be trusted; -- applied to individuals, corporations, communities, or nations; <as>as, to buy goods on <ex>credit</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Credit</qex> is nothing but the expectation of money, within some limited time.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The time given for payment for lands or goods sold on trust; <as>as, a long <ex>credit</ex> or a short <ex>credit</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Bookkeeping)</fld> <def>The side of an account on which are entered all items reckoned as values received from the party or the category named at the head of the account; also, any one, or the sum, of these items; -- the opposite of <ant>debit</ant>; <as>as, this sum is carried to one's <ex>credit</ex>, and that to his <xex>debit</xex>; A has several <ex>credits</ex> on the books of B</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Bank credit</b></col>, <it>or</it> <col><b>Cash credit</b></col></mcol>. <cd>See under <er>Cash</er>.</cd> -- <col><b>Bill of credit</b></col>. <cd>See under <er>Bill</er>.</cd> -- <col><b>Letter of credit</b></col>, <cd>a letter or notification addressed by a banker to his correspondent, informing him that the person named therein is entitled to draw a certain sum of money; when addressed to several different correspondents, or when the money can be drawn in fractional sums in several different places, it is called a <stype>circular letter of credit</stype>.</cd> -- <col><b>Public credit</b></col>. <sd>(a)</sd> <cd>The reputation of, or general confidence in, the ability or readiness of a government to fulfill its pecuniary engagements.</cd> <sd>(b)</sd> <cd>The ability and fidelity of merchants or others who owe largely in a community.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>He touched the dead corpse of <qex>Public Credit</qex>, and it sprung upon its feet.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credit</ent><br/
<hw>Cred"it</hw> <pr>(kr<ecr/d"<icr/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Credited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Crediting</conjf>.]</vmorph> <sn>1.</sn> <def>To confide in the truth of; to give credence to; to put trust in; to believe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How shall they <qex>credit</qex><br/
A poor unlearned virgin?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring honor or repute upon; to do credit to; to raise the estimation of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>credit</qex> the church as much by your government as you did the school formerly by your wit.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bookkeeping)</fld> <def>To enter upon the credit side of an account; to give credit for; <as>as, to <ex>credit</ex> the amount paid</as>; to set to the credit of; <as>as, to <ex>credit</ex> a man with the interest paid on a bond</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To credit with</b></col>, <cd>to give credit for; to assign as justly due to any one.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Crove, Helmholtz, and Meyer, are more than any others <qex>to be credited with</qex> the clear enunciation of this doctrine.</q> <rj><qau>Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creditable</ent><br/
<hw>Cred"it*a*ble</hw> <pr>(-?-b'l)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Worthy of belief.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Divers <qex>creditable</qex> witnesses deposed.</q> <rj><qau>Ludlow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deserving or possessing reputation or esteem; reputable; estimable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This gentleman was born of <qex>creditable</qex> parents.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Bringing credit, reputation, or honor; honorable; <as>as, such conduct is highly <ex>creditable</ex> to him</as>.</def> <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He settled him in a good <qex>creditable</qex> way of living.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creditableness</ent><br/
<hw>Cred"it*a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being creditable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creditably</ent><br/
<hw>Cred"it*a*bly</hw> <pr>(-?-bl?)</pr>, <pos>adv.</pos> <def>In a creditable manner; reputably; with credit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credit foncier</ent><br/
||<hw>Cr<eacute/`dit" fon`cier"</hw> <pr>(kr?d?" f?n`s??")</pr>. <ety>[F. <ets>cr<eacute/dit</ets> credit & <ets>foncier</ets> relating to land, landed.]</ety> <def>A company licensed for the purpose of carrying out improvements, by means of loans and advances upon real securities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credit mobilier</ent><br/
||<hw>Cr<eacute/`dit" mo`bi`lier"</hw> <pr>(m?`b?`ly?")</pr>. <ety>[F. <ets>cr<eacute/dit</ets> credit & <ets>mobilier</ets> personal, pertaining to personal property.]</ety> <def>A joint stock company, formed for general banking business, or for the construction of public works, by means of loans on personal estate, after the manner of the <cref>cr<eacute/dit foncier</cref> on real estate. In practice, however, this distinction has not been strictly observed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creditor</ent><br/
<hw>Cred"it*or</hw> <pr>(kr?d"?t-?r)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>cr<?/diteur</ets>. See <er>Credit</er>.]</ety> <sn>1.</sn> <def>One who credits, believes, or trusts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The easy <qex>creditors</qex> of novelties.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who gives credit in business matters; hence, one to whom money is due; -- correlative to <xex>debtor</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Creditors</qex> have better memories than debtors.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creditrix</ent><br/
<ent>Creditress</ent><br/
<mhw>{ <hw>Cred"it*ress</hw> <pr>(kr?d"?t-r?s)</pr>, <hw>Cred"i*trix</hw> <pr>(kr?d"?-tr?ks)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>creditrix</ets>.]</ety> <def>A female creditor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>creditworthy</ent><br/
<ent>credit-worthy</ent><br/
<mhw><hw>credit-worthy</hw> <hw>creditworthy</hw></mhw> <pos>adj.</pos>  <def>having an acceptable credit rating; worthy of having credit extended; <as>as, a <ex>credit-worthy</ex> customer</as>.</def><br/
<syn><b>Syn. --</b> responsible.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Credo</ent><br/
<hw>Cre"do</hw> <pr>(kr?"d?)</pr>, <pos>n.</pos> <ety>[L. See <er>Creed</er>.]</ety> <def>The creed, as sung or read in the Roman Catholic church.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He repeated Aves and <qex>Credos</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credulity</ent><br/
<hw>Cre*du"li*ty</hw> <pr>(kr?-d?"l?-t?)</pr>, <pos>n.</pos> <ety>[L. <ets>credulitas</ets>, fr. <ets>credulus</ets>: cf. F. <ets>cr<eacute/dulit<eacute/</ets>. See <er>Credulous</er>.]</ety> <def>Readiness of belief; a disposition to believe on slight evidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That implict <qex>credulity</qex> is the mark of a feeble mind will not be disputed.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credulous</ent><br/
<hw>Cred"u*lous</hw> <pr>(kr?d"?-l?s; 135)</pr>, <pos>a.</pos> <ety>[L. <ets>credulus</ets>, fr. <ets>credere</ets>. See <er>Creed</er>.]</ety> <sn>1.</sn> <def>Apt to believe on slight evidence; easily imposed upon; unsuspecting.</def> <rj><au>Landor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Eve, our <qex>credulous</qex> mother.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Believed too readily.</def> <mark>[Obs.]</mark> <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credulously</ent><br/
<hw>Cred"u*lous*ly</hw>, <pos>adv.</pos> <def>With credulity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Credulousness</ent><br/
<hw>Cred"u*lous*ness</hw>, <pos>n.</pos> <def>Readiness to believe on slight evidence; credulity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beyond all credulity is the <qex>credulousness</qex> of atheists.</q> <rj><qau>S. Clarke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creed</ent><br/
<hw>Creed</hw> <pr>(kr<emac/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>credo</ets>, <ets>crede</ets>, AS. <ets>creda</ets>, fr. L. <ets>credo</ets> I believe, at the beginning of the Apostles' creed, fr. <ets>credere</ets> to believe; akin to OIr. <ets>cretim</ets> I believe, and Skr. <ets><cced/raddadh<amac/mi</ets>; <ets><cced/rat</ets> trust + <ets>dh<amac/</ets> to put. See <er>Do</er>, <pos>v. t.</pos>, and cf. <er>Credo</er>, <er>Grant</er>.]</ety> <sn>1.</sn> <def>A definite summary of what is believed; esp., a summary of the articles of Christian faith; a confession of faith for public use; esp., one which is brief and comprehensive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the Protestant system the <qex>creed</qex> is not coordinate with, but always subordinate to, the Bible.</q> <rj><qau>Schaff-Herzog Encyc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any summary of principles or opinions professed or adhered to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I love him not, nor fear him; there's my <qex>creed</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Apostles' creed</b></col>, <col><b>Athanasian creed</b></col>, <col><b>Nicene creed</b></col>.</mcol> <cd>See under <er>Apostle</er>, <er>Athanasian</er>, <er>Nicene</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creed</ent><br/
<hw>Creed</hw>, <pos>v. t.</pos> <def>To believe; to credit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That part which is so <qex>creeded</qex> by the people.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creedless</ent><br/
<hw>Creed"less</hw>, <pos>a.</pos> <def>Without a creed.</def> <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creek</ent><br/
<hw>Creek</hw> <pr>(kr<emac/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>crecca</ets>; akin to D. <ets>kreek</ets>, Icel. <ets>kriki</ets> crack, nook; cf. W. <ets>crig</ets> crack, <ets>crigyll</ets> ravine, creek. Cf. <er>Crick</er>, <er>Crook</er>.]</ety> <sn>1.</sn> <def>A small inlet or bay, narrower and extending further into the land than a cove; a recess in the shore of the sea, or of a river.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each <qex>creek</qex> and cavern of the dangerous shore.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They discovered a certain <qex>creek</qex>, with a shore.</q> <rj><qau>Acts xxvii. 39.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stream of water smaller than a river and larger than a brook.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lesser streams and rivulets are denominated <qex>creeks</qex>.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any turn or winding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The passages of alleys, <qex>creeks</qex>, and narrow lands.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creekfish</ent><br/
<hw>Creek"fish</hw> <pr>(kr?k"f?sh)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The chub sucker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creeks</ent><br/
<hw>Creeks</hw> <pr>(kr<emac/ks)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Creek</singw></sing>. <fld>(Ethnol.)</fld> <def>A tribe or confederacy of North American Indians, including the Muskogees, Seminoles, Uchees, and other subordinate tribes. They formerly inhabited Georgia, Florida, and Alabama.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creeky</ent><br/
<hw>Creek"y</hw> <pr>(kr<emac/k"<ycr/)</pr>, <pos>a.</pos> <def>Containing, or abounding in, creeks; characterized by creeks; like a creek; winding.</def> <ldquo/The <xex>creeky</xex> shore.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creel</ent><br/
<hw>Creel</hw> <pr>(kr<emac/l)</pr>, <pos>n.</pos> <ety>[Gael. <ets>craidhleag</ets> basket, creel.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An osier basket, such as anglers use.</def> <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Spinning)</fld> <def>A bar or set of bars with skewers for holding paying-off bobbins, as in the roving machine, throstle, and mule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creep</ent><br/
<hw>Creep</hw> <pr>(kr<emac/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Crept</conjf> <pr>(kr<ecr/pt)</pr> (<conjf>Crope</conjf> <pr>(kr<omac/p)</pr>, <mark>Obs</mark>.); <pos>p. p.</pos> <conjf>Crept</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Creeping</conjf>.]</vmorph> <ety>[OE. <ets>crepen</ets>, <ets>creopen</ets>, AS. <ets>cre<oacute/pan</ets>; akin to D. <ets>kruipen</ets>, G. <ets>kriechen</ets>, Icel. <ets>krjupa</ets>, Sw. <ets>krypa</ets>, Dan. <ets>krybe</ets>. Cf. <er>Cripple</er>, <er>Crouch</er>.]</ety> <sn>1.</sn> <def>To move along the ground, or on any other surface, on the belly, as a worm or reptile; to move as a child on the hands and knees; to crawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye that walk<br/
The earth, and stately tread, or lowly <qex>creep</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move slowly, feebly, or timorously, as from unwillingness, fear, or weakness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whining schoolboy . . . <qex>creeping</qex>, like snail,<br/
Unwillingly to school.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a guilty thing, I <qex>creep</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To move in a stealthy or secret manner; to move imperceptibly or clandestinely; to steal in; to insinuate itself or one's self; <as>as, age <ex>creeps</ex> upon us</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sophistry which <qex>creeps</qex> into most of the books of argument.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of this sort are they which <qex>creep</qex> into houses, and lead captive silly women.</q> <rj><qau>2. Tim. iii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To slip, or to become slightly displaced; <as>as, the collodion on a negative, or a coat of varnish, may <ex>creep</ex> in drying; the quicksilver on a mirror may <ex>creep</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To move or behave with servility or exaggerated humility; to fawn; <as>as, a <ex>creeping</ex> sycophant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To come as humbly as they used to <qex>creep</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To grow, as a vine, clinging to the ground or to some other support by means of roots or rootlets, or by tendrils, along its length.</def> <ldquo/Creeping vines.<rdquo/ <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To have a sensation as of insects creeping on the skin of the body; to crawl; <as>as, the sight made my flesh <ex>creep</ex></as>. See <er>Crawl</er>, <pos>v. i.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To drag in deep water with creepers, as for recovering a submarine cable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creep</ent><br/
<hw>Creep</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of creeping.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A distressing sensation, or sound, like that occasioned by the creeping of insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>creep</qex> of undefinable horror.</q> <rj><qau>Blackwood's Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Out of the stillness, with gathering <qex>creep</qex>,<br/
Like rising wind in leaves.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A slow rising of the floor of a gallery, occasioned by the pressure of incumbent strata upon the pillars or sides; a gradual movement of mining ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creeper</ent><br/
<hw>Creep"er</hw> <pr>(kr<emac/p"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, creeps; any creeping thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Standing waters are most unwholesome, . . . full of mites, <qex>creepers</qex>; slimy, muddy, unclean.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A plant that clings by rootlets, or by tendrils, to the ground, or to trees, etc.; <as>as, the Virginia <ex>creeper</ex> (<ex>Ampelopsis quinquefolia</ex>)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A small bird of the genus <gen>Certhia</gen>, allied to the wrens. The brown or common European creeper is <spn>Certhia familiaris</spn>, a variety of which (var. <varn>Americana</varn>) inhabits America; -- called also <altname>tree creeper</altname> and <altname>creeptree</altname>.  The American black and white creeper is <spn>Mniotilta varia</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A kind of patten mounted on short pieces of iron instead of rings; also, a fixture with iron points worn on a shoe to prevent one from slipping.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <def>A spurlike device strapped to the boot, which enables one to climb a tree or pole; -- called often <altname>telegraph creepers</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A small, low iron, or dog, between the andirons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <def>An instrument with iron hooks or claws for dragging at the bottom of a well, or any other body of water, and bringing up what may lie there.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Any device for causing material to move steadily from one part of a machine to another, as an apron in a carding machine, or an inner spiral in a grain screen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <pluf>pl.</pluf> <fld>(Arch.)</fld> <def>Crockets. See <er>Crocket</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creephole</ent><br/
<hw>Creep"hole`</hw> <pr>(-h?l`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A hole or retreat into which an animal may creep, to escape notice or danger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A subterfuge; an excuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creepie</ent><br/
<hw>Creep"ie</hw> <pr>(-<ycr/)</pr>, <pos>n.</pos> <def>A low stool.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creepiness</ent><br/
<hw>Creep"i*ness</hw> <pr>(-<icr/-n<ecr/s)</pr>, <pos>n.</pos> <def>An uneasy sensation as of insects creeping on the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She felt a curious, uneasy <qex>creepiness</qex>.</q> <rj><qau>Mrs. Alexander.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creeping</ent><br/
<hw>Creep"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Crawling, or moving close to the ground.</def> <ldquo/Every <xex>creeping</xex> thing.<rdquo/ <rj><au>Gen. vi. 20.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Growing along, and clinging to, the ground, or to a wall, etc., by means of rootlets or tendrils.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Casements lined with <qex>creeping</qex> herbs.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ceeping crowfoot</b></col> <fld>(Bot.)</fld>, <cd>a plant, the <spn>Ranunculus repens</spn>.</cd> -- <col><b>Creeping snowberry</b></col>, <cd>an American plant (<spn>Chiogenes hispidula</spn>) with white berries and very small round leaves having the flavor of wintergreen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creeping Charlie</ent><br/
<hw>Creep"ing Char"lie</hw>. <def>The stonecrop (<spn>Sedum acre</spn>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Creepingly</ent><br/
<hw>Creep"ing*ly</hw>, <pos>adv.</pos> <def>by creeping slowly; in the manner of a reptile; insidiously; cunningly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How slily and <qex>creepingly</qex> did he address himself to our first parents.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creeple</ent><br/
<hw>Cree"ple</hw> <pr>(kr<emac/"p'l)</pr>, <pos>n.</pos> <ety>[See <er>Cripple</er>.]</ety> <sn>1.</sn> <def>A creeping creature; a reptile.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is one creeping beast, or long <qex>creeple</qex> (as the name is in Devonshire), that hath a rattle at his tail that doth discover his age.</q> <rj><qau>Morton (1632).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is lame; a cripple.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou knowest how lame a <qex>creeple</qex> this world is.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creepy</ent><br/
<hw>Creep"y</hw> <pr>(kr<emac/p"<ycr/)</pr>, <pos>a.</pos> <def>Crawly; having or producing a sensation like that caused by insects creeping on the skin.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>One's whole blood grew curdling and <qex>creepy</qex>.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 343 --></p>

<p><ent>Crees</ent><br/
<hw>Crees</hw> <pr>(kr<emac/z)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Cree</singw></sing>. <fld>(Ethnol.)</fld> <def>An Algonquin tribe of Indians, inhabiting a large part of British America east of the Rocky Mountains and south of Hudson's Bay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creese</ent><br/
<hw>Creese</hw> <pr>(kr<emac/s)</pr>, <pos>n.</pos> <ety>[Malay. <ets>kris</ets>.]</ety> <def>A dagger or short sword used by the Malays, commonly having a serpentine blade.</def> <altsp>[Written also <asp>crease</asp> and <asp>kris</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>From a Malayan <qex>creese</qex> to a sailor's jackknife.</q> <rj><qau>Julian Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremaillere</ent><br/
||<hw>Cr<eacute/`mail`l<egrave/re"</hw> <pr>(kr?`m?`ly?r" <or/ -m?`y?r")</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Fort.)</fld> <def>An indented or zigzaged line of intrenchment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremaster</ent><br/
||<hw>Cre*mas"ter</hw> <pr>(kr?-m?s"t?r)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/<?/<?/, fr. <?/<?/<?/<?/ to hang.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A thin muscle which serves to draw up the testicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The apex of the last abdominal segment of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremasteric</ent><br/
<hw>Crem`as*ter"ic</hw> <pr>(kr?m`3s-t?r"?k)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the cremaster; <as>as, the <ex>cremasteric</ex> artery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremate</ent><br/
<hw>Cre"mate</hw> <pr>(kr?"m?t <or/ kr?-m?t")</pr>, <pos>v. t.</pos> <ety>[L. <ets>crematus</ets>, p. p. of <ets>cremare</ets> to burn; cf. Skr. <ets>cr<?/</ets> to cook.]</ety> <def>To burn; to reduce to ashes by the action of fire, either directly or in an oven or retort; to incremate or incinerate; <as>as, to <ex>cremate</ex> a corpse, instead of burying it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremation</ent><br/
<hw>Cre*ma"tion</hw> <pr>(kr?-m?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>crematio</ets>.]</ety> <def>A burning; esp., the act or practice of cremating the dead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without <qex>cremation</qex> . . . of their bodies.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremationist</ent><br/
<hw>Cre*ma"tion*ist</hw>, <pos>n.</pos> <def>One who advocates the practice of cremation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremator</ent><br/
<hw>Cre*ma"tor</hw> <pr>(-t?r)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who, or that which, cremates or consumes to ashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crematory</ent><br/
<ent>Crematorium</ent><br/
<mhw>{ <hw>Crem`a*to"ri*um</hw> <pr>(kr?m`?-t?"r?-?m)</pr>, <hw>Crem"a*to*ry</hw> <pr>(kr?m"?-t?-r?)</pr>, }</mhw> <pos>n.</pos>; <plu><it>pl.</it> <plw>Crematoriums</plw> (-<ucr/mz), <plw>Crematories</plw> <pr>(-r<?/z)</pr>.</plu> <ety>[NL.  <ets>crematorium</ets>, fr. L.  <ets>cremator</ets>.]</ety> <def>A furnace for cremating corpses; a building containing such a furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crematory</ent><br/
<hw>Crem"a*to*ry</hw>, <pos>a.</pos> <def>Pertaining to, or employed in, cremation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creme</ent><br/
||<hw>Cr<ecir/me</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Cream; -- a term used esp. in cookery, names of liqueurs, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cremocarp</ent><br/
<hw>Crem"o*carp</hw> <pr>(kr<ecr/m"<osl/*k<aum/rp <or/ kr<emac/"m<osl/-)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kremanny`nai</grk> to hang + <grk>karpo`s</grk> fruit.]</ety> <fld>(Bot.)</fld> <def>The peculiar fruit of fennel, carrot, parsnip, and the like, consisting of a pair of carpels pendent from a supporting axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremona</ent><br/
<hw>Cre*mo"na</hw> <pr>(kr<esl/*m<omac/"n<adot/)</pr>, <pos>n.</pos> <def>A superior kind of violin, formerly made at <etsep>Cremona</etsep>, in Italy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremor</ent><br/
||<hw>Cre"mor</hw> <pr>(kr?"m?r)</pr>, <pos>n.</pos> <ety>[L. CF. <er>Cream</er>.]</ety> <def>Cream; a substance resembling cream; yeast; scum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cremosin</ent><br/
<hw>Crem"o*sin</hw> <pr>(kr?m"?-s?n)</pr>, <pos>n.</pos> <def>See <er>Crimson</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crems</ent><br/
<hw>Crems</hw> <pr>(kr<ecr/mz)</pr>, <pos>n.</pos> <def>See <er>Krems</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenated</ent><br/
<ent>Crenate</ent><br/
<mhw>{ <hw>Cre"nate</hw> <pr>(kr<emac/"n<asl/t)</pr>, <hw>Cre"na*ted</hw> <pr>(kr<emac/"n<asl/*t<ecr/d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>crena</ets> notch. See <er>Cranny</er>.]</ety> <fld>(Bot.)</fld> <def>Having the margin cut into rounded teeth notches, or scallops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenation</ent><br/
<hw>Cre*na"tion</hw> <pr>(kr?-n?"sh?n)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>A rounded tooth on the edge of a leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The condition of being crenate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenature</ent><br/
<hw>Cren"a*ture</hw> <pr>(kr?n"?-t?r <or/ kr?"n?-; 135)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>A rounded tooth or notch of a crenate leaf, or any part that is crenate; -- called also <altname>crenelle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being crenated or notched.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenel</ent><br/
<hw>Cre*nel"</hw> <pr>(kr?-n?l")</pr>, <pos>n.</pos> <def>See <er>Crenelle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenelate</ent><br/
<hw>Cren"el*ate</hw> <pr>(kr?n"?l-?t <or/  kr?"n?l-?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crenelated</conjf> <pr>(-?`t?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crenelating</conjf> <pr>(-?`t?ng)</pr>.]</vmorph> <ety>[LL.  <ets>crenellare</ets>, <ets>kernellare</ets>: cf. F.  <ets>cr<?/neler</ets> to indent. See <er>Crenelle</er>.]</ety> <altsp>[Written also <asp>crenellate</asp>.]</altsp> <sn>1.</sn> <def>To furnish with crenelles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To indent; to notch; <as>as, a <ex>crenelated</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crenelated molding</b></col> <fld>(Arch.)</fld>, <cd>a kind of indented molding used in Norman buildings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>crenelated</ent><br/
<hw>crenelated</hw> <pos>adj.</pos> <sn>1.</sn>  <def>having repeated square indentations like those in a battlement; <as>as, a <ex>crenelated</ex> molding</as>.</def><br/
<syn><b>Syn. --</b> embattled, crenelate, crenellated, crenellate, indented.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>having turrets and battlements in the style of a castle; -- of a building.</def><br/
<syn><b>Syn. --</b> battlemented, castellated, castled, crenellated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crenelation</ent><br/
<hw>Cren`el*a"tion</hw> <pr>(-?"sh?n)</pr>, <pos>n.</pos> <def>The act of crenelating, or the state of being crenelated; an indentation or an embrasure.</def> <altsp>[Written also <asp>crenellation</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenel</ent><br/
<ent>Crenelle</ent><br/
<mhw>{ <hw>Cre*nelle"</hw>, <hw>Cre*nel"</hw>  }</mhw> <pr>(kr?-n?l")</pr>, <pos>n.</pos> <ety>[OF. <ets>crenel</ets>, F.  <ets>cr<?/neau</ets>, LL. <ets>crenellus</ets>, <ets>kernellus</ets>, dim. (prob.) fr. L.  <ets>crena</ets> notch. See <er>Crenny</er>.]</ety> <sn>1.</sn> <def>An embrasure or indentation in a battlement; a loophole in a fortress; an indentation; a notch. See <er>Merlon</er>, and <xex>Illust.</xex> of <er>Battlement</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Same as <er>Crenature</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenelled</ent><br/
<hw>Cre*nelled"</hw> <pr>(kr<?/-n<?/ld")</pr> <pos>a.</pos> <fld>(Bot.)</fld> <def>Same as <er>Crenate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenkle</ent><br/
<ent>Crengle</ent><br/
<mhw>{ <hw>Cren"gle</hw> <pr>(kr?n"g'l)</pr>, <hw>Cren"kle</hw> <pr>(-k'l)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Cringle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenulated</ent><br/
<ent>Crenulate</ent><br/
<mhw>{ <hw>Cren"u*late</hw> <pr>(kr?n"?-l?t)</pr>, <hw>Cren"u*la`ted</hw> <pr>(-l?`t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[Dim. of <ets>crenate</ets>.]</ety> <fld>(Bot.)</fld> <def>Minutely crenate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crenulation</ent><br/
<hw>Cren`u*la"tion</hw> <pr>(-l?"sh?n)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A minute crenation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being minutely scalloped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creole</ent><br/
<hw>Cre"ole</hw> <pr>(kr?"?l)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<?/ole</ets>, Sp. <ets>criollo</ets>, from an American negro word, perh. a corruption of a Sp.  <ets>criadillo</ets>, dim. of <ets>criado</ets> servant, formerly also, child, fr. L. <ets>creatus</ets>, p. p. of <ets>creare</ets> to create. Cf. <er>Create</er>.]</ety> <def>One born of European parents in the American colonies of France or Spain or in the States which were once such colonies, esp. a person of French or Spanish descent, who is a native inhabitant of Louisiana, or one of the States adjoining, bordering on the Gulf of of Mexico.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/The term <xex>creole negro</xex> is employed in the English West Indies to distinguish the negroes born there from the Africans imported during the time of the slave trade. The application of this term to the colored people has led to an idea common in some parts of the United States, though wholly unfounded, that it implies an admixture greater or less of African blood.<rdquo/</note>  <rj><au>R. Hildreth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/The title [Creole] did not first belong to the descendants of Spanish, but of French, settlers, But such a meaning implied a certain excellence of origin, and so came early to include any native of French or Spanish descent by either parent, whose nonalliance with the slave race entitled him to social rank. Later, the term was adopted by, not conceded to, the natives of mixed blood, and is still so used among themselves.  . . .  Besides French and Spanish, there are even, for convenience of speech, 'colored' <xex>Creoles</xex>; but there are no Italian, or Sicilian, nor any English, Scotch, Irish, or 'Yankee' <xex>Creoles</xex>, unless of parentage married into, and themselves thoroughly proselyted in, <xex>Creole</xex> society.<rdquo/</note>  <rj><au>G. W. Cable.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creole</ent><br/
<hw>Cre"ole</hw> <pr>(kr?"?l)</pr>, <pos>a.</pos> <def>Of or pertaining to a Creole or the Creoles.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In New Orleans the word <xex>Creole</xex> is applied to any product, or variety of manufacture, peculiar to Louisiana; as, <xex>Creole</xex> ponies, chickens, cows, shoes, eggs, wagons, baskets, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creolian</ent><br/
<ent>Creolean</ent><br/
<mhw>{ <hw>Cre*o"le*an</hw> <pr>(kr?-?"l?-<ait/n)</pr>, <hw>Cre*o"li*an</hw>  }</mhw>, <pos>a.</pos> <def>Pertaining to, or characteristic of, the Creoles.</def> -- <def2><pos>n.</pos> <def>A Creole.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creole State</ent><br/
<hw>Creole State</hw>. <def>Louisiana; -- a nickname. See <er>Creole</er>, <pos>n. & a.</pos></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Creosol</ent><br/
<hw>Cre"o*sol</hw> <pr>(kr<emac/"<osl/*s<omac/l)</pr>, <pos>n.</pos> <ety>[<ets>Creos</ets>ote + phen<ets>ol</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless liquid resembling phenol or carbolic acid, homologous with pyrocatechin, and obtained from beechwood tar and gum guaiacum.</def> <altsp>[Written also <asp>creasol</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creosote</ent><br/
<hw>Cre"o*sote</hw> <pr>(kr<emac/"<osl/*s<omac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kre`as</grk>, gen. <grk>kre`ws</grk>, flesh + <grk>sw`zein</grk> to preserve.]</ety> <fld>(Chem.)</fld> <def>Wood-tar oil; an oily antiseptic liquid, of a burning smoky taste, colorless when pure, but usually colored yellow or brown by impurity or exposure.  It is a complex mixture of various phenols and their ethers, and is obtained by the distillation of wood tar, especially that of beechwood.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is remarkable as an antiseptic and deodorizer in the preservation of wood, flesh, etc., and in the prevention of putrefaction; but it is a poor germicide, and in this respect has been overrated.  Smoked meat, as ham, owes its preservation and taste to a small quantity of creosote absorbed from the smoke to which it is exposed. <part><er>Carbolic acid</er></part> is <part><er>phenol{1}</er></part> proper, while <ex>creosote</ex> is a mixture of several phenols.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coal-tar creosote</b></col> <fld>(Chem.)</fld>, <cd>a colorless or yellow, oily liquid, obtained in the distillation of coal tar, and resembling wood-tar oil, or creosote proper, in composition and properties.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creosote</ent><br/
<hw>Cre"o*sote</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Creosoted</conjf> <pr>(-s?"t?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Creosoting</conjf>.]</vmorph> <def>To saturate or impregnate with creosote, as timber, for the prevention of decay.</def></p>

<p><ent>Creosote bush</ent><br/
<hw>Cre"o*sote bush</hw>. <def>A shrub (<spn>Covillea mexicana</spn>) found in desert regions from Colorado to California and southward through Mexico. It has yellow flowers and very resinous foliage with a strong odor of creosote.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crepane</ent><br/
<ent>Crepance</ent><br/
<mhw>{ <hw>Cre"pance</hw> <pr>(kr?"p<it>a</it>ns)</pr>, <hw>Cre"pane</hw> <pr>(kr?"p?n)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. L. <ets>crepare</ets> to crack.]</ety> <fld>(Far.)</fld> <def>An injury in a horse's leg, caused by the shoe of one hind foot striking and cutting the other leg. It sometimes forms an ulcer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepe</ent><br/
||<hw>Cr<ecir/pe</hw> <pr>(kr<asl/p)</pr>, <ety>[F.]</ety> <pos>n.</pos> <sn>1.</sn> <def>Same as <er>Crape</er>.</def> <altsp>[Also spelled <asp>crepe</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pr>(kr<acir/p; <it>Eng.</it> kr<amac/p)</pr>, <def>Any of various crapelike fabrics, whether crinkled or not.</def></p>

<p><sn>3.</sn> <def>A small thin pancake.</def> <wns>[wns16=2]</wns><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>Paper with a finely crinkle texture, usually sold in rolls of 2-3 inches width; crepe paper; -- it is usually colored brightly and used for decoration.</def> <wns>[wns16=1]</wns><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Cr<ecir/pe de Chine</b></col> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[F. <ets>de Chine</ets> of China]</ety>, <cd>Canton crape or an inferior gauzy fabric resembling it.</cd> <altsp>[Also spelled <asp>crepe de Chine</asp>.]</altsp> -- <col><b>Cr<ecir/pe lisse</b></col> <pr>(l<emac/s)</pr> <ety>[F. <ets>lisse</ets> smooth]</ety>, <cd>smooth, or unwrinkled, crape{2}.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>crepe paper</ent><br/
||<hw>crepe` pa"per</hw> <pr>(kr<amac/p` p<amac/"p<etil/r)</pr> <def>Paper with a finely crinkle texture, usually sold in rolls of 2-3 inches width; crepe paper; -- it is usually colored brightly and used for decoration.  Same as <er>crepe</er>{4}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crepe Suzettes</ent><br/
<ent>Crepes Suzette</ent><br/
<ent>Crepe Suzette</ent><br/
<hw>Cr<ecir/pe` Su*zette"</hw> <pr>(kr?p` s?*z<ecr/t")</pr>, <plu><it>pl.</it> <hw>Cr<ecir/pes` Su*zette"</hw> or <hw>Cr<ecir/pe` Su*zettes"</hw></plu>. <ety>[F.]</ety> <pos>n.</pos> <def>A rolled or folded <er>crepe{3}</er>, heated in an orange-flavored liqueur (or in a hot orange-butter sauce and moistened with a liqueur) and set on fire when served.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crepis</ent><br/
<hw>Crepis</hw> <pos>prop. n.</pos> <def>A genus of plants including the hawk's beard; cosmopolitan in the northern hemisphere.</def><br/
<syn><b>Syn. --</b> genus <gen>Crepis</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crepitant</ent><br/
<hw>crep"i*tant</hw> <pr>(kr?p"?-t<it>a</it>nt)</pr>, <pos>a.</pos> <ety>[See <er>Crepitate</er>.]</ety> <def>Having a crackling sound; crackling; rattling.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crepitant rale</b></col> <fld>(Med.)</fld>, <cd>a peculiar crackling sound audible with inspiration in pneumonia and other lung disease.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepitate</ent><br/
<hw>Crep"i*tate</hw> <pr>(kr<?/p"<?/-t<?/t)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crepitated</conjf> <pr>(-t<?/`t<?/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crepitating</conjf> <pr>(-t<?/`t<?/ng)</pr>.]</vmorph> <ety>[L. <ets>crepitatus</ets>, <pos>p. p.</pos> of <ets>crepitare</ets> to crackle, <pos>v.</pos> intensive of <ets>crepare</ets> to crack. Cf. <er>Crevice</er>.]</ety> <def>To make a series of small, sharp, rapidly repeated explosions or sounds, as salt in fire; to crackle; to snap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepitation</ent><br/
<hw>Crep`i*ta"tion</hw> <pr>(kr?p`?-t?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F. <xex>cr<eacute/pitation</xex>.]</ety> <sn>1.</sn> <def>The act of crepitating or crackling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>A grating or crackling sensation or sound, as that produced by rubbing two fragments of a broken bone together, or by pressing upon cellular tissue containing air.</def> <sd>(b)</sd> <def>A crepitant r<acir/le.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepitus</ent><br/
||<hw>Crep"i*tus</hw> <pr>(kr?p"?-t?s)</pr>, <pos>n.</pos> <ety>[L., fr.  <ets>crepare</ets> to crack.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>The noise produced by a sudden discharge of wind from the bowels.</def> <sd>(b)</sd> <def>Same as <er>Crepitation</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepon</ent><br/
||<hw>Cre"pon</hw> <pr>(kr?"p?n; F. kr?`<er>p?n</er>")</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A thin stuff made of the finest wool or silk, or of wool and silk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crept</ent><br/
<hw>Crept</hw> <pr>(kr<ecr/pt)</pr>, <def><pos>imp. & p. p.</pos> of <er>Creep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepuscule</ent><br/
<ent>Crepuscle</ent><br/
<mhw>{ <hw>Cre*pus"cle</hw> <pr>(kr<esl/*p<ucr/s"s'l)</pr>, <hw>Cre*pus"cule</hw> <pr>(kr<esl/*p<ucr/s"k<usl/l)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>crepusculum</ets>, fr. <ets>creper</ets> dusky, dark: cf. F.  <ets>cr<eacute/puscule</ets>.]</ety> <def>Twilight.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepusculous</ent><br/
<ent>Crepuscular</ent><br/
<mhw>{ <hw>Cre*pus"cu*lar</hw> <pr>(-k?-l?r)</pr>, <hw>Cre*pus"cu*lous</hw> <pr>(-l?s)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>cr<eacute/pusculaire</ets>.]</ety> <sn>1.</sn> <def>Pertaining to twilight; glimmering; hence, imperfectly clear or luminous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This semihistorical and <qex>crepuscular</qex> period.</q> <rj><qau>Sir G. C. Lewis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Flying in the twilight or evening, or before sunrise; -- said certain birds and insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others feed only in the twilight, as bats and owls, and are called <qex>crepuscular</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crepusculine</ent><br/
<hw>Cre*pus"cu*line</hw> <pr>(-l<icr/n)</pr>, <pos>a.</pos> <def>Crepuscular.</def> <mark>[Obs.]</mark>  <rj><au>Sprat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescence</ent><br/
<hw>Cres"cence</hw> <pr>(kr<ecr/s"s<eit/ns)</pr>, <pos>n.</pos> <ety>[See <er>Crescent</er>.]</ety> <def>Increase; enlargement.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And toward the moon's attractive <qex>crescence</qex> bend.</q> <rj><qau>H. Brooke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescendo</ent><br/
<hw>Cres*cen"do</hw> <pr>(kr?s-s?n"d?; It. kr?-sh?n"d?)</pr>, <pos>a. & adv.</pos> <ety>[It., from <ets>crescere</ets> to increase. See <er>Crescent</er>.]</ety> <fld>(Mus.)</fld> <def>With a constantly increasing volume of voice; with gradually increasing strength and fullness of tone; -- a direction for the performance of music, indicated by the mark, or by writing the word on the score.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescendo</ent><br/
<hw>Cres*cen"do</hw>, <pos>n.</pos> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A gradual increase in the strength and fullness of tone with which a passage is performed.</def> <sd>(b)</sd> <def>A passage to be performed with constantly increasing volume of tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescent</ent><br/
<hw>Cres"cent</hw> <pr>(kr<ecr/s"s<eit/nt)</pr>, <pos>n.</pos> <ety>[OE.  <ets>cressent</ets>, <ets>cressaunt</ets>, crescent (in sense 1), OF.  <ets>creissant</ets> increasing, F.  <ets>croissant</ets>, p. pr. of <ets>cro<icir/tre</ets>, OF.  <ets>creistre</ets>, fr. L. <ets>crescere</ets> to increase, v. incho.; akin to <ets>creare</ets> to create. See <er>Create</er>, and cf. <er>Accrue</er>, <er>Increase</er>, <er>Crescendo</er>.]</ety> <sn>1.</sn> <def>The increasing moon; the moon in her first quarter, or when defined by a concave and a convex edge; also, applied improperly to the old or decreasing moon in a like state.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything having the shape of a crescent or new moon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A representation of the increasing moon, often used as an emblem or badge</def>; as: <sd>(a)</sd> <def>A symbol of Artemis, or Diana.</def> <sd>(b)</sd> <def>The ancient symbol of Byzantium or Constantinople.</def> Hence: <sd>(c)</sd> <def>The emblem of the Turkish Empire, adopted after the taking of Constantinople.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cross of our faith is replanted,<br/
The pale, dying <qex>crescent</qex> is daunted.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any one of three orders of knighthood; the first instituted by Charles I., king of Naples and Sicily, in 1268; the second by Ren<eacute/ of Anjou, in 1448; and the third by the Sultan Selim III., in 1801, to be conferred upon foreigners to whom Turkey might be indebted for valuable services.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Her.)</fld> <def>The emblem of the increasing moon with horns directed upward, when used in a coat of arms; -- often used as a mark of cadency to distinguish a second son and his descendants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescent</ent><br/
<hw>Cres"cent</hw> <pr>(kr<ecr/s"s<eit/nt)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Shaped like a crescent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Astarte, queen of heaven, with <qex>crescent</qex> horns.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Increasing; growing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, I see the <qex>crescent</qex> promise of my spirit hath not set.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescent</ent><br/
<hw>Cres"cent</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To form into a crescent, or something resembling a crescent.</def> <mark>[R.]</mark>  <rj><au>Anna Seward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To adorn with crescents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescentic</ent><br/
<hw>Cres*cen"tic</hw> <pr>(kr<ecr/s*s<ecr/n"t<icr/k)</pr>, <pos>a.</pos> <def>Crescent-shaped.</def> <ldquo/<xex>Crescentic</xex> lobes.<rdquo/  <rj><au>R. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescentwise</ent><br/
<hw>Cres"cent*wise`</hw> <pr>(kr<ecr/s"s<eit/nt*w<imac/z`)</pr>, <pos>adv.</pos> <def>In the form of a crescent; like a crescent.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crescive</ent><br/
<hw>Cres"cive</hw> <pr>(kr?s"s?v)</pr>, <pos>a.</pos> <ety>[L.  <ets>crescere</ets> to increase.]</ety> <def>Increasing; growing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Unseen, yet <qex>crescive</qex> in his faculty.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cresol</ent><br/
<hw>Cre"sol</hw> <pr>(kr<emac/"s<omac/l)</pr>, <pos>n.</pos> <ety>[From <er>Creosote</er>.]</ety> <fld>(Chem.)</fld> <def>Any one of three metameric substances, <chform>CH3.C6H4.OH</chform>, homologous with and resembling phenol. They are obtained from coal tar and wood tar, and are colorless, oily liquids or solids. <note>[Called also <altname>cresylic acid</altname>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cresorcin</ent><br/
<hw>Cre*sor"cin</hw> <pr>(kr?-s?r"s?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Same as <er>Isorcin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cress</ent><br/
<hw>Cress</hw> <pr>(kr<ecr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cresses</plw> <pr>(kr<ecr/s"<ecr/z)</pr>.</plu> <ety>[OE. <ets>ces</ets>, <ets>cresse</ets>, <ets>kers</ets>, <ets>kerse</ets>, AS. <ets>cresse</ets>, <ets>cerse</ets>; akin to D. <ets>kers</ets>, G. <ets>kresse</ets>, Dan. <ets>karse</ets>, Sw. <ets>krasse</ets>, and possibly also to OHG. <ets>chresan</ets> to creep.]</ety> <fld>(Bot.)</fld> <def>A plant of various species, chiefly cruciferous. The leaves have a moderately pungent taste, and are used as a salad and antiscorbutic.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The garden cress, called also <altname>peppergrass</altname>, is the <spn>Lepidium sativum</spn>; the water cress is the <spn>Nasturtium officinale</spn>. Various other plants are sometimes called <xex>cresses</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>To strip the brook with mantling <qex>cresses</qex> spread.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bitter cress</b></col>. <cd>See under <er>Bitter</er>.</cd> -- <mcol><col><b>Not worth a cress</b></col>, <it>or</it> <col><b><ldquo/<xex>not worth a kers</xex>.<rdquo/</b></col></mcol> <cd>a common old proverb, now turned into the meaningless <ldquo/<xex>not worth a curse</xex>.<rdquo/</cd>  <rj><au>Skeat.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cresselle</ent><br/
<hw>Cres*selle"</hw> <pr>(kr?s-s?l")</pr>, <pos>n.</pos> <ety>[F. <ets>cr<eacute/celle</ets> rattle.]</ety> <fld>(Eccl.)</fld> <def>A wooden rattle sometimes used as a substitute for a bell, in the Roman Catholic church, during the latter part of Holy Week, or the last week of Lent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cresset</ent><br/
<hw>Cres"set</hw> <pr>(kr?s"s?t)</pr>, <pos>n.</pos> <ety>[OF. <ets>crasset</ets>, <ets>cresset</ets>, sort of lamp or torch; perh. of Dutch or German origin, and akin to E. <ets>cruse</ets>, F.  <ets>creuset</ets> crucible, E.  <ets>crucible</ets>.]</ety> <sn>1.</sn> <def>An open frame or basket of iron, filled with combustible material, to be burned as a beacon; an open lamp or firrepan carried on a pole in nocturnal processions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Starry lamps and blazing <qex>cressets</qex>, fed<br/
With naphtha and asphaltus.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As a <qex>cresset</qex> true that darts its length<br/
Of beamy luster from a tower of strength.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Coopering)</fld> <def>A small furnace or iron cage to hold fire for charring the inside of a cask, and making the staves flexible.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cressy</ent><br/
<hw>Cress"y</hw> <pr>(kr<ecr/s"<ycr/)</pr>, <pos>a.</pos> <def>Abounding in cresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cressy</qex> islets white in flower.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crest</ent><br/
<hw>Crest</hw> <pr>(kr<ecr/st)</pr>, <pos>n.</pos> <ety>[OF. <ets>creste</ets>, F. <ets>cr<ecir/te</ets>, L.  <ets>crista</ets>.]</ety> <sn>1.</sn> <def>A tuft, or other excrescence or natural ornament, growing on an animal's head; the comb of a cock; the swelling on the head of a serpent; the lengthened feathers of the crown or nape of bird, etc.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Attack] his rising <qex>crest</qex>, and drive the serpent back.</q> <rj><qau>C. Pitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The plume of feathers, or other decoration, worn on a helmet; the distinctive ornament of a helmet, indicating the rank of the wearer; hence, also, the helmet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Stooping low his lofty <qex>crest</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And on his head there stood upright<br/
A <qex>crest</qex>, in token of a knight.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Her.)</fld> <def>A bearing worn, not upon the shield, but usually above it, or separately as an ornament for plate, liveries, and the like. It is a relic of the ancient cognizance. See <er>Cognizance</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The upper curve of a horse's neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Throwing the base thong from his bending <qex>crest</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The ridge or top of a wave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like wave with <qex>crest</qex> of sparkling foam.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The summit of a hill or mountain ridge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The helm or head, as typical of a high spirit; pride; courage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now the time is come<br/
That France must vail her lofty plumed <qex>crest</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Arch.)</fld> <def>The ornamental finishing which surmounts the ridge of a roof, canopy, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The finials of gables and pinnacles are sometimes called <qex>crests</qex>.</q> <rj><qau>Parker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Engin.)</fld> <def>The top line of a slope or embankment.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crest tile</b></col>, <cd>a tile made to cover the ridge of a roof, fitting upon it like a saddle.</cd> -- <col><b>Interior crest</b></col> <fld>(Fort.)</fld>, <cd>the highest line of the parapet.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crest</ent><br/
<hw>Crest</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crested</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cresting</conjf>.]</vmorph> <sn>1.</sn> <def>To furnish with, or surmount as, a crest; to serve as a crest for.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 344 --></p>

<p><q>His legs bestrid the ocean, his reared arm<br/
<qex>Crested</qex> the world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Mid groves of clouds that <qex>crest</qex> the mountain's brow.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark with lines or streaks, like, or regarded as like, waving plumes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like as the shining sky in summer's night, . . . <br/
Is <qex>crested</qex> with lines of fiery light.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crest</ent><br/
<hw>Crest</hw> <pr>(kr<ecr/st)</pr>, <pos>v. i.</pos> <def>To form a crest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crested</ent><br/
<hw>Crest"ed</hw> <pr>(kr<ecr/st"<ecr/d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a crest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But laced <qex>crested</qex> helm.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having a crest of feathers or hair upon the head.</def> <ldquo/The <xex>crested</xex> bird.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bott.)</fld> <def>Bearing any elevated appendage like a crest, as an elevated line or ridge, or a tuft.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crestfallen</ent><br/
<hw>Crest"fall`en</hw> <pr>(-f?l`'n)</pr>, <pos>a.</pos> <sn>1.</sn> <def>With hanging head; hence, dispirited; dejected; cowed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let it make thee <qex>crestfullen</qex>;<br/
Ay, and allay this thy abortive pride.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the crest, or upper part of the neck, hanging to one side; -- said of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cresting</ent><br/
<hw>Crest"ing</hw>, <pos>n.</pos> <fld>(Arch.)</fld> <def>An ornamental finish on the top of a wall or ridge of a roof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crestless</ent><br/
<hw>Crest"less</hw>, <pos>a.</pos> <def>Without a crest or escutcheon; of low birth.</def> <ldquo/<xex>Crestless</xex> yeomen.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cresylic</ent><br/
<hw>Cre*syl"ic</hw> <pr>(kr<esl/*s<icr/l"<icr/k)</pr>, <pos>a.</pos> <ety>[From <er>Creosote</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or derived from, cresol, creosote, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cresylic acid</b></col>. <fld>(Chem.)</fld> <cd>See <er>Cresol</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretaceous</ent><br/
<hw>Cre*ta"ceous</hw> <pr>(kr<esl/*t<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>cretaceus</ets>, fr.  <ets>creta</ets> chalk. See <er>Crayon</er>.]</ety> <sn>1.</sn> <def>Having the qualities of chalk; abounding with chalk; chalky; <as>as, <ex>cretaceous</ex> rocks and formations</as>. See <er>Chalk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretacic</ent><br/
<sn>2.</sn> <fld>(Geol.)</fld> <def>Of, pertaining to, or designating, the period of time following the Jurassic and preceding the Tertiary, generally given as from 144 million years b. p. to 65 million years b. p..  Also called <hw>Cre*tac"ic</hw>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><cs><col><b>Cretaceous acid</b></col>, <cd>an old name for carbonic acid.</cd> -- <col><b>Cretaceous formation</b></col> <fld>(Geol.)</fld>, <cd>the series of strata of various kinds, including beds of chalk, green sand, etc., formed in the Cretaceous period; -- called also the <altname>chalk formation</altname>. See the Diagram under <er>Geology</er>.</cd> -- <col><b>Cretaceous period</b></col> <fld>(Geol.)</fld>, <cd>the time in the latter part of the Mesozoic age during which the Cretaceous formation was deposited, and at the end of which the dinosaurs died out.  See <er>Cretaceous</er>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cretaceously</ent><br/
<hw>Cre*ta"ceous*ly</hw>, <pos>adv.</pos> <def>In a chalky manner; as chalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretaceous-Tertiary boundary</ent><br/
<hw>Cre*ta"ceous-Tert"i*ar*y boun"da*ry</hw> <pr>(kr<esl/*t<amac/"sh<ucr/s)</pr>, <pos>n.</pos> <def>a thin layer of geologic deposits, of varying thickness in different parts of the world, found between the geological strata identified as Cretaceous and the strata above, identified as Tertiary; also, the time point or period marking the boundary between the Cretaceous and Tertiary periods.</def><br/
<syn><b>Syn. --</b> K/T boundary.</syn><br/
[<source>PJC</source>]</p>

<p><note><hand/According to a theory gaining acceptance (as of 1997), these deposits were formed as the debris of a large comet or meteorite impact on the earth, which threw up a large quantity of dust into the atmosphere, causing profound though temporary climatic change, and caused or hastened the extinction of numerous species, including the dinosaurs.  This hypothesis was first postulated by Luis and Walter Alvarez on the basis of an excess of iridium found in the boundary layer, and was later supported by additional evidence of various types.  The impact is believed to have occurred at the edge of the Yucatan peninsula in Mexico, forming what is termed the <a href="http://dsaing.uqac.uquebec.ca/~mhiggins/MIAC/chicxulub.htm">Chicxulub crater</a>, which is partly under the Gulf of Mexico, is not evident from surface topography, and was detected primarily by gravity anomaly readings and subsurface geological characteristics.</note><br/
[<source>PJC</source>]</p>

<p><ent>Cretan</ent><br/
<hw>Cre"tan</hw> <pr>(kr<emac/"t<ait/n)</pr>, <pos>a.</pos> <def>Pertaining to Crete, or Candia.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Crete or Candia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crete</ent><br/
<hw>Crete</hw> <pr>(kr<emac/t)</pr>, <pos>n.</pos> <ety>[L. <ets>Cres</ets>, <ets>Cretis</ets>.]</ety> <def>A Cretan</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretian</ent><br/
<hw>Cre"tian</hw> <pr>(kr<emac/"sh<ait/n)</pr>, <pos>a. & n.</pos> <def>See <er>Cretan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretic</ent><br/
<hw>Cre"tic</hw> <pr>(kr<emac/"t<icr/k)</pr>, <pos>n.</pos> <ety>[L. <ets>Creticus</ets> (sc. <ets>pes</ets> foot), Gr. <grk>Kritiko`s</grk> (sc. <grk>poy`s</grk> foot), prop., a Cretan (metrical) foot.]</ety> <fld>(Gr. & Lat. Pros.)</fld> <def>A poetic foot, composed of one short syllable between two long ones (- <cre/ -).</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creticism</ent><br/
<hw>Cre"ti*cism</hw> <pr>(-t<icr/*s<icr/z'm)</pr>, <pos>n.</pos> <def>Falsehood; lying; cretism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretin</ent><br/
<hw>Cre"tin</hw> <pr>(kr<emac/"t<icr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<eacute/tin</ets>; of uncertain origin.]</ety> <def>One afflicted with cretinism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretinism</ent><br/
<hw>Cre"tin*ism</hw> <pr>(kr<emac/"t<icr/n*<icr/z'm)</pr>, <pos>n.</pos> <ety>[F. <ets>cr<eacute/tinisme</ets>.]</ety> <def>A condition of endemic or inherited idiocy, accompanied by physical degeneracy and deformity (usually with goiter), frequent in certain mountain valleys, esp. of the Alps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretinous</ent><br/
<hw>Cre"tin*ous</hw> <pr>(-<ucr/s)</pr>, <pos>a.</pos> <def>Having the characteristics of a cretin.</def> <ldquo/<xex>Cretinous</xex> stupefaction.<rdquo/  <rj><au>Ruskin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretism</ent><br/
<hw>Cre"tism</hw> <pr>(kr<emac/"t<icr/z'm)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/ lying, fr. <?/<?/<?/<?/<?/ to act like a Cretan, that is, to lie. <ldquo/The Cretians are always liars.<rdquo/ <au>Titus i. 12.</au>]</ety> <def>A Cretan practice; lying; a falsehood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretonne</ent><br/
<hw>Cre*tonne"</hw> <pr>(kr?-t?n")</pr>, <pos>n.</pos> <ety>[F., gr. <ets>Creton</ets>, its first manufacturer.]</ety> <sn>1.</sn> <def>A strong white fabric with warp of hemp and weft of flax.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fabric with cotton warp and woolen weft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of chintz with a glossy surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cretose</ent><br/
<hw>Cre"tose</hw> <pr>(kr?"t?s)</pr>, <pos>a.</pos> <ety>[L. <ets>cretosus</ets>, fr.  <ets>creta</ets> chalk.]</ety> <def>Chalky; cretaceous.</def> <mark>[Obs.]</mark>  <rj><au>Ash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creutzer</ent><br/
<hw>Creut"zer</hw> <pr>(kroit"s<etil/r)</pr> <pos>n.</pos> <def>See <er>Kreutzer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creux</ent><br/
||<hw>Creux</hw> <pr>(kr?)</pr>, <pos>n.</pos> <ety>[F., <pos>adj.</pos>, hollow, <pos>n.</pos>, a hollow.]</ety> <def>Used in English only in the expression <xex>en creux</xex>. Thus, engraving <xex>en creux</xex> is engraving in intaglio, or by sinking or hollowing out the design.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crevalle</ent><br/
||<hw>Cre`val*le"</hw> <pr>(kr?`v?l-l?")</pr>, <pos>n.</pos> <ety>[Prob. of same origin as <ets>cavally</ets>. See <er>Cavally</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The cavally or jurel.</def> See <er>Cavally</er>, and <er>Jurel</er>. <sd>(b)</sd> <def>The pompano (<spn>Trachynotus Carolinus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crevasse</ent><br/
||<hw>Cre`vasse"</hw> <pr>(kr?`v?s")</pr>, <pos>n.</pos> <ety>[F. See <er>Crevice</er>.]</ety> <sn>1.</sn> <def>A deep crevice or fissure, as in embankment; one of the clefts or fissure by which the mass of a glacier is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A breach in the levee or embankment of a river, caused by the pressure of the water, as on the lower Mississippi.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crevet</ent><br/
<hw>Crev"et</hw> <pr>(kr?v"?t)</pr>, <pos>n.</pos> <ety>[Cf. <er>Creut</er>.]</ety> <def>A crucible or melting pot; a cruset.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crevice</ent><br/
<hw>Crev"ice</hw> <pr>(kr?v"?s)</pr>, <pos>n.</pos> <ety>[OE. <ets>crevace</ets>, <ets>crevice</ets>. F.  <ets>crevasse</ets>, fr. <ets>crever</ets> to break, burst, fr. L. <ets>crepare</ets> to crack,break. Cf. <er>Craven</er>, <er>Crepitate</er>, <er>Crevasse</er>.]</ety> <def>A narrow opening resulting from a split or crack or the separation of a junction; a cleft; a fissure; a rent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mouse,<br/
Behind the moldering wainscot, shrieked,<br/
Or from the <qex>crevice</qex> peered about.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crevice</ent><br/
<hw>Crev"ice</hw>, <pos>v. t.</pos> <def>To crack; to flaw.</def> <mark>[R.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Creviced</ent><br/
<hw>Crev"iced</hw> <pr>(-?st)</pr>, <pos>a.</pos> <def>Having a crevice or crevices; <as>as, a <ex>creviced</ex> structure for storing ears of corn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trickling through the <qex>creviced</qex> rock.</q> <rj><qau>J. Cunningham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crevis</ent><br/
<hw>Crev"is</hw> <pr>(-?s)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The crawfish.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crew</ent><br/
<hw>Crew</hw> <pr>(kr<udd/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Manx shearwater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crew</ent><br/
<hw>Crew</hw> <pr>(kr<udd/)</pr>, <pos>n.</pos> <ety>[From older <ets>accrue</ets> accession, reenforcement, hence, company, crew; the first syllable being misunderstood as the indefinite article. See <er>Accrue</er>, <er>Crescent</er>.]</ety> <sn>1.</sn> <def>A company of people associated together; an assemblage; a throng.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There a noble <qex>crew</qex><br/
Of lords and ladies stood on every side.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Faithful to whom? to thy rebellious <qex>crew</qex>?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The company of seamen who man a ship, vessel, or at; the company belonging to a vessel or a boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The word <xex>crew</xex>, in law, is ordinarily used as equivalent to <xex>ship's company</xex>, including master and other officers. When the master and other officers are excluded, the context always shows it.</note>  <rj><au>Story.</au>  <au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In an extended sense, any small body of men associated for a purpose; a gang; <as>as <fld>(Naut.)</fld>, the carpenter's <ex>crew</ex>; the boatswain's <ex>crew</ex></as>.</def></p>

<p><syn><b>Syn.</b> -- Company; band; gang; horde; mob; herd; throng; party.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crew</ent><br/
<hw>Crew</hw> <pr>(kr<udd/)</pr>, <def><pos>imp.</pos> of <er>Crow</er></def>.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Crewel</ent><br/
<hw>Crew"el</hw> <pr>(kr<udd/"<ecr/l)</pr>, <pos>n.</pos> <ety>[Perh. for <ets>clewel</ets>, dim. of <ets>clew</ets> a ball of thread; or cf. D.  <ets>krul</ets> curl, E.  <ets>curl</ets>. <root/26.]</ety> <def>Worsted yarn,, slackly twisted, used for embroidery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crewelwork</ent><br/
<hw>Crew"el*work`</hw> <pr>(-w?rk`)</pr>, <pos>n.</pos> <def>Embroidery in crewels, commonly done upon some plain material, such as linen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crewet</ent><br/
<hw>crew"et</hw> <pr>(kr<udd/"<ecr/t)</pr>, <pos>n.</pos> <def>See <er>Cruet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crewman</ent><br/
<hw>crew"man</hw> <pr>(kr<udd/"m<ait/n)</pr>, <pos>n.</pos> <def>any member of a ship's crew.</def><br/
<syn><b>Syn. --</b> sailor.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a member of a flight crew.</def><br/
<syn><b>Syn. --</b> crew member.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>a member of a work crew.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crex</ent><br/
<hw>Crex</hw> <pos>n.</pos> <def>a genus of birds including the corncrake (<spn>Crex crex</spn>).</def><br/
<syn><b>Syn. --</b> genus <gen>Crex</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crib</ent><br/
<hw>Crib</hw> <pr>(kr<icr/b)</pr>, <pos>n.</pos> <ety>[AS. <ets>crybb</ets>; akin to OS. <ets>kribbja</ets>, D. <ets>krib</ets>, <ets>kribbe</ets>, Dan. <ets>krybbe</ets>, G. <ets>krippe</ets>, and perh. to MHG. <ets>krebe</ets> basket, G, <ets>korb</ets>, and E. <ets>rip</ets> a sort of wicker basket.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A manger or rack; a feeding place for animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The steer lion at one <qex>crib</qex> shall meet.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stall for oxen or other cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where no oxen are, the <qex>crib</qex> is clean.</q> <rj><qau>Prov. xiv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small inclosed bedstead or cot for a child.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A box or bin, or similar wooden structure, for storing grain, salt, etc.; <as>as, a <ex>crib</ex> for corn or oats</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A hovel; a hut; a cottage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why rather, Sleep, liest thou in smoky <qex>cribs</qex>, . . . <br/
Than in the perfumed chambers of the great?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mining)</fld> <def>A structure or frame of timber for a foundation, or for supporting a roof, or for lining a shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A structure of logs to be anchored with stones; -- used for docks, pier, dams, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A small raft of timber.</def> <mark>[Canada]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A small theft; anything purloined; a plagiarism; hence, a translation or key, etc., to aid a student in preparing or reciting his lessons.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Latin version technically called a <qex>crib</qex>.</q> <rj><qau>Ld. Lytton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Occasional perusal of the Pagan writers, assisted by a <qex>crib</qex>.</q> <rj><qau>Wilkie Collins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A miner's luncheon.</def> <mark>[Cant]</mark>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Card Playing)</fld> <def>The discarded cards which the dealer can use in scoring points in cribbage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crib</ent><br/
<hw>Crib</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cribbed</conjf> <pr>(kr<icr/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cribbing</conjf>.]</vmorph> <sn>1.</sn> <def>To shut up or confine in a narrow habitation; to cage; to cramp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If only the vital energy be not <qex>cribbed</qex> or cramped.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now I am cabin'd, <qex>cribbed</qex>, confined.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pilfer or purloin; hence, to steal from an author; to appropriate; to plagiarize; <as>as, to <ex>crib</ex> a line from Milton</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Child, being fond of toys, <qex>cribbed</qex> the necklace.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crib</ent><br/
<hw>Crib</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To crowd together, or to be confined, as in a crib or in narrow accommodations.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Who sought to make . . . bishops to <qex>crib</qex> in a Presbyterian trundle bed.</q> <rj><qau>Gauden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make notes for dishonest use in recitation or examination.</def> <mark>[College Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To seize the manger or other solid object with the teeth and draw in wind; -- said of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribbage</ent><br/
<hw>Crib"bage</hw> <pr>(kr?b"?j)</pr>, <pos>n.</pos> <ety>[From <er>Crib</er>, <pos>v. t.</pos>, 2.]</ety> <def>A game of cards, played by two or four persons, in which there is a crib. (See <er>Crib</er>, 11.) It is characterized by a great variety of chances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man's fancy would be summed up in <qex>cribbage</qex>.</q> <rj><qau>John Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cribbage board</b></col>, <cd>a board with holes and pegs, used by cribbage players to score their game.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crib-biter</ent><br/
<ent>Cribber</ent><br/
<mhw>{ <hw>Crib"ber</hw> <pr>(kr?b"?r)</pr>, <hw>Crib"-bit`er</hw> <pr>(-b?t"?r)</pr> }</mhw>, <pos>n.</pos> <def>A horse that has the habit of cribbing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribbing</ent><br/
<hw>Crib"bing</hw> <pr>(kr?b"b?ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of inclosing or confining in a crib or in close quarters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Purloining; stealing; plagiarizing.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A framework of timbers and plank backing for a shaft lining, to prevent caving, percolation of water, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A vicious habit of a horse; crib-biting. The horse lays hold of the crib or manger with his teeth and draws air into the stomach with a grunting sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crib-biting</ent><br/
<hw>Crib"-bit`ing</hw> <pr>(kr?b"b?t`?ng)</pr>, <pos>n.</pos> <def>Same as <er>Cribbing</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribble</ent><br/
<hw>Crib"ble</hw> <pr>(kr?b"b'l)</pr>, <pos>n.</pos> <ety>[F. <ets>crible</ets>, LL. <ets>criblus</ets> sieve, fr. L. <ets>cribrum</ets>.]</ety> <sn>1.</sn> <def>A coarse sieve or screen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Coarse flour or meal.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribble</ent><br/
<hw>Crib"ble</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cribbled</conjf> <pr>(-b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cribbling</conjf> <pr>(-bl?ng)</pr>.]</vmorph> <ety>[Cf. F.  <ets>cribler</ets>.]</ety> <def>To cause to pass through a sieve or riddle; to sift.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribble</ent><br/
<hw>Crib"ble</hw>, <pos>a.</pos> <def>Coarse; <as>as, <ex>cribble</ex> bread</as>.</def> <mark>[Obs.]</mark>  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribellum</ent><br/
||<hw>Cri*bel"lum</hw> <pr>(kr?b?l"l?m)</pr>, <pos>n.</pos> <ety>[L., a small sieve, dim. of <ets>cribrum</ets> sieve.]</ety> <fld>(Zool.)</fld> <def>A peculiar perforated organ of certain spiders (<fam>Ciniflonid<ae/</fam>), used for spinning a special kind of silk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribrate</ent><br/
<hw>Crib"rate</hw> <pr>(kr?b"r?t)</pr>, <pos>a.</pos> <ety>[L. <ets>cribratus</ets>, p. p. of <ets>cribrare</ets> to sift, fr. <ets>cribrum</ets> a sieve.]</ety> <def>Cribriform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribration</ent><br/
<hw>Cri*bra"tion</hw> <pr>(kr?-br?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>cribration</ets>, fr. L. <ets>cribrare</ets> to sift. See <er>Cribble</er>, <pos>n.</pos>]</ety> <fld>(Pharmacy)</fld> <def>The act or process of separating the finer parts of drugs from the coarser by sifting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribriform</ent><br/
<hw>Crib"ri*form</hw> <pr>(kr?b"r?f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cribrum</ets> sieve + <ets>-form</ets>: cf. F. <ets>cribriforme</ets>.]</ety> <def>Resembling, or having the form of, a sieve; pierced with holes; <as>as, the <ex>cribriform</ex> plate of the ethmoid bone; a <ex>cribriform</ex> compress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cribriform cells</b></col> <fld>(Bot.)</fld>, <cd>those which have here and there oblique or transverse sieve plates, or places perforated with many holes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cribrose</ent><br/
<hw>Crib"rose</hw> <pr>(kr?b"r?s)</pr>, <pos>a.</pos> <ety>[L. <ets>cribrum</ets> sieve.]</ety> <def>Perforated like a sieve; cribriform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cric</ent><br/
<hw>Cric</hw> <pr>(kr<icr/k)</pr>, <pos>n.</pos> <ety>[prob. fr. F. <ets>cric</ets> a jackscrew.]</ety> <def>The ring which turns inward and condenses the flame of a lamp.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crick</ent><br/
<hw>Crick</hw> <pr>(kr<icr/k)</pr>, <pos>n.</pos> <ety>[See <er>Creak</er>.]</ety> <def>The creaking of a door, or a noise resembling it.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crick</ent><br/
<hw>Crick</hw>, <pos>n.</pos> <ety>[The same as <ets>creek</ets> a bending, twisting. See <er>Creek</er>, <er>Crook</er>.]</ety> <sn>1.</sn> <def>A painful, spasmodic affection of the muscles of some part of the body, as of the neck or back, rendering it difficult to move the part.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> To those also that, with a <qex>crick</qex> or cramp, have thei necks drawn backward.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. F. <ets>cric</ets>.]</ety> <def>A small jackscrew.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cricket</ent><br/
<hw>Crick"et</hw> <pr>(kr?k"?t)</pr>, <pos>n.</pos> <ety>[OE. <ets>criket</ets>, OF.  <ets>crequet</ets>, <ets>criquet</ets>; prob. of German origin, and akin to E. <ets>creak</ets>; cf. D. <ets>kriek</ets> a cricket. See <er>Creak</er>.]</ety> <fld>(Zool.)</fld> <def>An orthopterous insect of the genus <gen>Gryllus</gen>, and allied genera. The males make chirping, musical notes by rubbing together the basal parts of the veins of the front wings.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European cricket is <spn>Gryllus domesticus</spn>; the common large black crickets of America are <spn>Gryllus niger</spn>, <spn>Gryllus neglectus</spn>, and others.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Balm cricket</b></col>. <cd>See under <er>Balm</er>.</cd> -- <col><b>Cricket bird</b></col>, <cd>a small European bird (<spn>Silvia locustella</spn>); -- called also <altname>grasshopper warbler</altname>.</cd> -- <col><b>Cricket frog</b></col>, <cd>a small American tree frog (<spn>Acris gryllus</spn>); -- so called from its chirping.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cricket</ent><br/
<hw>Crick"et</hw>, <pos>n.</pos> <ety>[AS.  <ets>cricc</ets>, <ets>crycc</ets>, crooked staff, crutch. Perh. first used in sense 1, a stool probably having been first used as a wicket. See <er>Crutch</er>.]</ety> <sn>1.</sn> <def>A low stool.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A game much played in England, and sometimes in America, with a ball, bats, and wickets, the players being arranged in two contesting parties or sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A small false roof, or the raising of a portion of a roof, so as to throw off water from behind an obstacle, such as a chimney.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cricket</ent><br/
<hw>Crick"et</hw>, <pos>v. i.</pos> <def>To play at cricket.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cricketer</ent><br/
<hw>Crick"et*er</hw> <pr>(kr?k"?t-?r)</pr>, <pos>n.</pos> <def>One who plays at cricket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cricoid</ent><br/
<hw>Cri"coid</hw> <pr>(kr?"koid)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/<?/ ring + <ets>-oid</ets>.]</ety> <fld>(Anat.)</fld> <def>Resembling a ring; -- said esp. of the cartilage at the larynx, and the adjoining parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cricothyroid</ent><br/
<hw>Cri`co*thy"roid</hw> <pr>(-k?-th?"roid)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining both to the cricoid and the thyroid cartilages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cried</ent><br/
<hw>Cried</hw> <pr>(kr<imac/d)</pr>, <def><pos>imp. & p. p.</pos> of <er>Cry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crier</ent><br/
<hw>Cri"er</hw> <pr>(kr?"?r)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>crieur</ets>. See <er>Cry</er>.]</ety> <def>One who cries; one who makes proclamation.</def> <specif>Specifically,</specif> <def>an officer who proclaims the orders or directions of a court, or who gives public notice by loud proclamation; <as>as, a town-<ex>crier</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He openeth his mouth like a <xex>crier</xex>.</q> <rj><qau>Ecclus. xx. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crime</ent><br/
<hw>Crime</hw> <pr>(kr<imac/m)</pr>, <pos>n.</pos> <ety>[F. <ets>crime</ets>, fr. L.  <ets>crimen</ets> judicial decision, that which is subjected to such a decision, charge, fault, crime, fr. the root of <ets>cernere</ets> to decide judicially. See <er>Certain</er>.]</ety> <sn>1.</sn> <def>Any violation of law, either divine or human; an omission of a duty commanded, or the commission of an act forbidden by law.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gross violation of human law, in distinction from a misdemeanor or trespass, or other slight offense. Hence, also, any aggravated offense against morality or the public welfare; any outrage or great wrong.</def> <ldquo/To part error from <xex>crime</xex>.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Crimes</xex>, in the English common law, are grave offenses which were originally capitally punished (murder, rape, robbery, arson, burglary, and larceny), as distinguished from misdemeanors, which are offenses of a lighter grade. See <er>Misdemeanors</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any great wickedness or sin; iniquity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>crime</qex> was thine, if 'tis no <qex>crime</qex> to love.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which occasion crime.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The tree of life, the <qex>crime</qex> of our first father's fall.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Capital crime</b></col>, <cd>a crime punishable with death.</cd></cs></p>

<p><syn><b>Syn.</b> -- Sin; vice; iniquity; wrong.</syn> <usage> -- <er>Crime</er>, <er>Sin</er>,<er>Vice</er>.  <xex>Sin</xex> is the generic term, embracing wickedness of every kind, but specifically denoting an offense as committed against God. <xex>Crime</xex> is strictly a violation of law either human or divine; but in present usage the term is commonly applied to actions contrary to the laws of the State. <xex>Vice</xex> is more distinctively that which springs from the inordinate indulgence of the natural appetites, which are in themselves innocent. Thus intemperance, unchastity, duplicity, etc., are <xex>vices</xex>; while murder, forgery, etc., which spring from the indulgence of selfish passions, are <xex>crimes</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimea</ent><br/
<hw>Crimea</hw> <pos>n.</pos> <def>a Ukrainian peninsula between the Black Sea and the Sea of Azov.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crimeful</ent><br/
<hw>Crime"ful</hw> <pr>(kr?m"f?l)</pr>, <pos>a.</pos> <def>Criminal; wicked; contrary to law, right, or dury.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimeless</ent><br/
<hw>Crime"less</hw>, <pos>a.</pos> <def>Free from crime; innocent.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminal</ent><br/
<hw>Crim"i*nal</hw> <pr>(kr?m"?-n<it>a</it>l)</pr>, <pos>a.</pos> <ety>[L. <ets>criminalis</ets>, fr.  <ets>crimen</ets>: cf. F.  <ets>criminel</ets>. See <er>Crime</er>.]</ety> <sn>1.</sn> <def>Guilty of crime or sin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The neglect of any of the relative duties renders us <qex>criminal</qex> in the sight of God.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Involving a crime; of the nature of a crime; -- said of an act or of conduct; <as>as, <ex>criminal</ex> carelessness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Foppish and fantastic ornaments are only indications of vice, not <qex>criminal</qex> in themselves.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Relating to crime; -- opposed to <xex>civil</xex>; <as>as, the <ex>criminal</ex> code</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The officers and servants of the crown, violating the personal liberty, or other right of the subject . . . were in some cases liable to <qex>criminal</qex> process.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Criminal action</b></col> <fld>(Law)</fld>, <cd>an action or suit instituted to secure conviction and punishment for a crime.</cd> -- <col><b>Criminal conversation</b></col> <fld>(Law)</fld>, <cd>unlawful intercourse with a married woman; adultery; -- usually abbreviated, <xex>crim. con</xex>.</cd> -- <col><b>Criminal law</b></col>, <cd>the law which relates to crimes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminal</ent><br/
<hw>Crim"i*nal</hw>, <pos>n.</pos> <def>One who has commited a crime; especially, one who is found guilty by verdict, confession, or proof; a malefactor; a felon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminalist</ent><br/
<hw>Crim"i*nal*ist</hw>, <pos>n.</pos> <def>One versed in criminal law.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 345 --></p>

<p><ent>Criminality</ent><br/
<hw>Crim`i*nal"i*ty</hw> <pr>(kr?m`?-n?l"?-t?)</pr>, <pos>n.</pos> <ety>[LL. <ets>criminalitas</ets>, fr. L. <ets>criminalis</ets>. See <er>Criminal</er>.]</ety> <def>The quality or state of being criminal; that which constitutes a crime; guiltiness; guilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is by no means the only criterion of <qex>criminality</qex>.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>criminalize</ent><br/
<hw>criminalize</hw> <pos>v. t.</pos> <def>to declare (an act) to be illegal.</def><br/
<syn><b>Syn. --</b> outlaw, illegalize, ban.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Criminally</ent><br/
<hw>Crim"i*nal*ly</hw> <pr>(kr?m"?-n<it>a</it>l-l?)</pr>, <pos>adv.</pos> <def>In violation of law; wickedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminalness</ent><br/
<hw>Crim"i*nal*ness</hw>, <pos>n.</pos> <def>Criminality.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminate</ent><br/
<hw>Crim"i*nate</hw> <pr>(kr<?/m"<?/-n<?/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Criminated</conjf> <pr>(-n<?/`t<?/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Criminating</conjf> <pr>(-n<?/"t<?/ng)</pr>.]</vmorph> <ety>[L. <ets>criminatus</ets>, p. p. of <ets>criminare</ets>, <ets>criminari</ets>, to criminate, fr. <ets>crimen</ets>. See <er>Crime</er>.]</ety> <sn>1.</sn> <def>To accuse of, or charge with, a crime.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>criminate</qex>, with the heavy and ungrounded charge of disloyalty and disaffection, an uncorrupt, independent, and reforming parliament.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To involve in a crime or in its consequences; to render liable to a criminal charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Impelled by the strongest pressure of hope and fear to <qex>criminate</qex> him.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimination</ent><br/
<hw>Crim`i*na"tion</hw> <pr>(kr?m`?-n?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>criminatio</ets>.]</ety> <def>The act of accusing; accusation; charge; complaint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>criminations</qex> and recriminations of the adverse parties.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminative</ent><br/
<hw>Crim"i*na*tive</hw> <pr>(kr?m"?-n?-t?v)</pr>, <pos>a.</pos> <def>Charging with crime; accusing; criminatory.</def>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminatory</ent><br/
<hw>Crim"i*na*to*ry</hw> <pr>(-t?-r?)</pr>, <pos>a.</pos> <def>Relating to, or involving, crimination; accusing; <as>as, a <ex>criminatory</ex> conscience</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>criminological</ent><br/
<hw>criminological</hw> <pos>adj.</pos> <def>of or pertaining to criminology.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Criminology</ent><br/
<hw>Crim`i*nol"o*gy</hw> <pr>(-n?l"?-j?)</pr>, <pos>n.</pos> <ety>[L. <ets>crimen</ets>, <ets>crimenis</ets>, crime +  <ets>-logy</ets>.]</ety> <sn>1.</sn> <def>A treatise on crime or the criminal population.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The branch of sociology which studies crime and criminals.</def><br/
[<source>PJC</source>]</p>

<p> -- <wordforms><wf>Crim`i*nol"o*gist</wf> <pr>(-j<?/st)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criminous</ent><br/
<hw>Crim"i*nous</hw> <pr>(kr?m"?-n?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>criminosus</ets>, fr.  <ets>crimen</ets>. See <er>Crime</er>.]</ety> <def>Criminal; involving great crime or grave charges; very wicked; heinous.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj></p>

<p>-- <wordforms><wf>Crim"i*nous*ly</wf>, <pos>adv.</pos> -- <wf>Crim"i*nous*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimosin</ent><br/
<hw>Crim"o*sin</hw> <pr>(kr?m"?-z?n)</pr>, <pos>n.</pos> <mark>[Obs.]</mark> <def>See <er>Crimson</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimp</ent><br/
<hw>Crimp</hw> <pr>(kr<icr/mp)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crimped</conjf> <pr>(kr<icr/mt; 215)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crimping</conjf>.]</vmorph> <ety>[Akin to D.  <ets>krimpen</ets> to shrink, shrivel, Sw. <ets>krympa</ets>, Dan.  <ets>krympe</ets>, and to E. <ets>cramp</ets>.  See <er>Cramp</er>.]</ety> <sn>1.</sn> <def>To fold or plait in regular undulation in such a way that the material will retain the shape intended; to give a wavy appearance to; <as>as, to <ex>crimp</ex> the border of a cap; to <ex>crimp</ex> a ruffle</as>.  Cf. <er>Crisp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The comely hostess in a <qex>crimped</qex> cap.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pinch and hold; to seize.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> Hence, <def>to entrap into the military or naval service; <as>as, to <ex>crimp</ex> seamen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Coaxing and courting with intent to <qex>crimp</qex> him.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Cookery)</fld> <def>To cause to contract, or to render more crisp, as the flesh of a fish, by gashing it, when living, with a knife; <as>as, to <ex>crimp</ex> skate, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Firearms)</fld> <def>In cartridge making, to fold the edge of (a cartridge case) inward so as to close the mouth partly and confine the charge.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Crimping house</b></col>, <cd>a low lodging house, into which men are decoyed and plied with drink, to induce them to ship or enlist as sailors or soldiers.</cd> -- <col><b>Crimping iron</b></col>. <sd>(a)</sd> <cd>An iron instrument for crimping and curling the hair.</cd> <sd>(b)</sd> <cd>A crimping machine.</cd> -- <col><b>Crimping machine</b></col>, <cd>a machine with fluted rollers or with dies, for crimping ruffles, leather, iron, etc.</cd> -- <col><b>Crimping pin</b></col>, <cd>an instrument for crimping or puckering the border of a lady's cap.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimp</ent><br/
<hw>Crimp</hw>, <pos>a.</pos> <sn>1.</sn> <def>Easily crumbled; friable; brittle.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Now the fowler . . . treads the <qex>crimp</qex> earth.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Weak; inconsistent; contradictory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The evidence is <qex>crimp</qex>; the witnesses swear backward and forward, and contradict themselves.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimp</ent><br/
<hw>Crimp</hw>, <pos>n.</pos> <sn>1.</sn> <def>A coal broker.</def> <mark>[Prov. Eng.]</mark>  <rj><au>De Foe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who decoys or entraps men into the military or naval service.</def>  <rj><au> Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A keeper of a low lodging house where sailors and emigrants are entrapped and fleeced.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hair which has been crimped; -- usually in <it>pl.</it></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A game at cards.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Boot crimp</b></col>. <cd>See under <er>Boot</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimpage</ent><br/
<hw>Crimp"age</hw> <pr>(-?j)</pr>, <pos>n.</pos> <def>The act or practice of crimping; money paid to a crimp for shipping or enlisting men.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimper</ent><br/
<hw>Crimp"er</hw> <pr>(-<atil/r)</pr>, <pos>n.</pos> <def>One who, or that which, crimps</def>; as: <sd>(a)</sd> <def>A curved board or frame over which the upper of a boot or shoe is stretched to the required shape.</def> <sd>(b)</sd> <def>A device for giving hair a wavy appearance.</def> <sd>(c)</sd> <def>A machine for crimping or ruffling textile fabrics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimple</ent><br/
<hw>Crim"ple</hw> <pr>(kr?m"p'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crimpled</conjf> <pr>(-p'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crimpling</conjf> <pr>(-pl?ng)</pr>.]</vmorph> <ety>[Dim. of <ets>crimp</ets>, v. t. ]</ety> <def>To cause to shrink or draw together; to contract; to curl.</def> <mark>[R.]</mark>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimpy</ent><br/
<hw>Crimp"y</hw> <pr>(kr?mp"?)</pr>, <pos>a.</pos> <def>Having a crimped appearance; frizzly; <as>as, the <ex>crimpy</ex> wool of the Saxony sheep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimson</ent><br/
<hw>Crim"son</hw> <pr>(kr<icr/m"z'n)</pr>, <pos>n.</pos> <ety>[OE. <ets>crimson</ets>, OF.  <ets>crimoisin</ets>, F.  <ets>cramoisi</ets> (cf. Sp. <ets>carmesi</ets>.) LL.  <ets>carmesinus</ets>, fr. Ar. <ets>qermazi</ets>, fr. <ets>qermez</ets> crimson, kermes, fr. Skr.  <ets>k<rsdot/mija</ets> produced by a worm; <ets>k<?/mi</ets> worm or insect +  <ets>jan</ets> to generate; akin to E.  <ets>kin</ets>. CF.  <er>Carmine</er>, <er>Kermes</er>.]</ety> <def>A deep red color tinged with blue; also, red color in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though your sins be as scarlet, they shall be as white as snow; though they be red like <qex>crimson</qex>, they shall be as wool.</q> <rj><qau>Is. i. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A maid yet rosed over with the virgin <qex>crimson</qex> of modesty.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimson</ent><br/
<hw>Crim"son</hw>, <pos>a.</pos> <def>Of a deep red color tinged with blue; deep red.</def> <ldquo/A <xex>crimson</xex> tide.<rdquo/  <rj><au>Mrs. Hemans.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The blushing poppy with a <qex>crimson</qex> hue.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimson</ent><br/
<hw>Crim"son</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crimsoned</conjf> <pr>(-z'nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crimsoning</conjf>.]</vmorph> <def>To dye with crimson or deep red; to redden.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Signed in thy spoil and <qex>crimsoned</qex> in thy lethe.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crimson</ent><br/
<hw>Crim"son</hw>, <pos>v. t.</pos> <def>To become crimson; to blush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ancient towers . . . beginning to <qex>crimson</qex> with the radiant luster of a cloudless July morning.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinal</ent><br/
<hw>Cri"nal</hw> <pr>(kr<imac/"n<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>crinalis</ets>, fr. <ets>crinis</ets> the hair.]</ety> <def>Of or pertaining to the hair.</def> <mark>[R.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinated</ent><br/
<hw>Cri"na*ted</hw> <pr>(kr<imac/"n<asl/*t<ecr/d)</pr>, <pos>a.</pos> <def>Having hair; hairy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinatory</ent><br/
<hw>Cri"na*to*ry</hw> <pr>(kr?"n?-t?-r?)</pr>, <pos>a.</pos> <def>Crinitory.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crincum</ent><br/
<hw>Crin"cum</hw> <pr>(kr?n"k?m)</pr>, <pos>n.</pos> <ety>[Cf. <er>Crinkle</er>.]</ety> <def>A twist or bend; a turn; a whimsey.</def> <mark>[Colloq.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crincum-crancum</ent><br/
<hw>Crin"cum-cran"cum</hw> <pr>(kr?n"k?m-kr?n"k?m)</pr>, <pos>n.</pos> <def>A twist; a whimsey or whim.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crined</ent><br/
<hw>Crined</hw> <pr>(kr<imac/nd)</pr>, <pos>a.</pos> <ety>[L. <ets>crinis</ets> hair.]</ety> <fld>(Her.)</fld> <def>Having the hair of a different tincture from the rest of the body; <as>as, a charge <ex>crined</ex> of a red tincture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinet</ent><br/
<ent>Crinel</ent><br/
<mhw>{ <hw>Cri"nel</hw> <pr>(kr?"nEl)</pr>, <hw>Cri"net</hw> <pr>(kr?"n?t)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>crinis</ets> hair.]</ety> <def>A very fine, hairlike feather.</def>  <rj><au>Booth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringe</ent><br/
<hw>Cringe</hw> <pr>(kr<icr/nj)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cringed</conjf> <pr>(kr<icr/njd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cringing</conjf>.]</vmorph> <ety>[As. <ets>crincgan</ets>, <ets>cringan</ets>, <ets>crincan</ets>, to jield, fall; akin to E.  <ets>crank</ets>.]</ety> <def>To draw one's self together as in fear or servility; to bend or crouch with base humility; to wince; hence, to make court in a degrading manner; to fawn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When they were come up to the place where the lions were, the boys that went before were glad to <qex>cringe</qex> behind, for they were afraid of the lions.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sly hypocrite, . . . who more than thou<br/
Once fawned and <qex>cringed</qex>, and servilely adored<br/
Heaven's awful monarch?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Flatterers . . . are always bowing and <qex>cringing</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringe</ent><br/
<hw>Cringe</hw>, <pos>v. t.</pos> <def>To contract; to draw together; to cause to shrink or wrinkle; to distort.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Till like a boy you see him <qex>cringe</qex> his face,<br/
And whine aloud for mercy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringe</ent><br/
<hw>Cringe</hw>, <pos>n.</pos> <def>Servile civility; fawning; a shrinking or bowing, as in fear or servility.</def> <ldquo/With <xex>cringe</xex> and shrug, and bow obsequious.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringeling</ent><br/
<hw>Cringe"ling</hw>, <pos>n.</pos> <def>One who cringes meanly; a fawner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringer</ent><br/
<hw>Crin"ger</hw> <pr>(kr?n"j?r)</pr>, <pos>n.</pos> <def>One who cringes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringingly</ent><br/
<hw>Crin"ging*ly</hw>, <pos>adv.</pos> <def>In a cringing manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cringle</ent><br/
<hw>Crin"gle</hw> <pr>(kr<icr/<nsm/"g'l)</pr>, <pos>n.</pos> <ety>[Icel. <ets>kringla</ets> orb; akin to <ets>kring</ets> around, and to D.  <ets>kring</ets> circle, and to E.  <ets>cringe</ets>, <ets>crank</ets>.]</ety> <sn>1.</sn> <def>A withe for fastening a gate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>An iron or pope thimble or grommet worked into or attached to the edges and corners of a sail; -- usually in the plural.  The cringles are used for making fast the bowline bridles, earings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinicultural</ent><br/
<hw>Crin`i*cul"tur*al</hw> <pr>(kr?n`?-k?l"t?r-<ait/l; 135)</pr>, <pos>a.</pos> <ety>[L.  <ets>crinis</ets> hair + <ets>cultura</ets>.]</ety> <def>Relating to the growth of hair.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinigerous</ent><br/
<hw>Cri*nig"er*ous</hw> <pr>(kr?-n?j"?r-?s)</pr>, <pos>a.</pos> <ety>[L. <ets>criniger</ets>; <ets>crinis</ets> hair + <ets>gerere</ets> to bear.]</ety> <def>Bearing hair; hairy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinital</ent><br/
<hw>Cri"ni*tal</hw> <pr>(kr?"n?-t<it>a</it>l)</pr>, <pos>a.</pos> <def>Same as <er>Crinite</er>, <sn>1.</sn></def><br/
[<source>1913 Webster</source>]</p>

<p><q>He the star <qex>crinital</qex> adoreth.</q> <rj><qau>Stanyhurst.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinite</ent><br/
<hw>Cri"nite</hw> <pr>(kr?"n?t)</pr>, <pos>a.</pos> <ety>[L. <ets>crinitus</ets>, p. p. of <ets>crinire</ets> to provide or cover with hair, fr. <ets>crinis</ets> hair.]</ety> <sn>1.</sn> <def>Having the appearance of a tuft of hair; having a hairlike tail or train.</def> <ldquo/Comate, <xex>crinite</xex>, caudate stars.<rdquo/  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Bearded or tufted with hairs.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinitory</ent><br/
<hw>Cri"ni*to*ry</hw> <pr>(kr?"n?-t?-r?)</pr>, <pos>a.</pos> <def>Of or relating to hair; <as>as, a <ex>crinitory</ex> covering</as>.</def>  <rj><au>T. Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinkle</ent><br/
<hw>Crin"kle</hw> <pr>(kr<icr/<nsm/"k'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crinkled</conjf> <pr>(-k'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crinkling</conjf> <pr>(-kl<icr/ng)</pr>.]</vmorph> <ety>[A dim., fr. the root of <ets>cringe</ets>; akin to D. <ets>krinkelen</ets> to wind or twist. Cf. <er>Cringle</er>, <er>Cringe</er>.]</ety> <def>To form with short turns, bends, or wrinkles; to mold into inequalities or sinuosities; to cause to wrinkle or curl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The house<?/s <qex>crinkled</qex> to and fro.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Her face all bowsy,<br/
Comely <qex>crinkled</qex>,<br/
Wondrously wrinkled.</q> <rj><qau>Skelton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The flames through all the casements pushing forth,<br/
Like red-not devils <qex>crinkled</qex> into snakes.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinkle</ent><br/
<hw>Crin"kle</hw>, <pos>v. i.</pos> <def>To turn or wind; to run in and out in many short bends or turns; to curl; to run in waves; to wrinkle; also, to rustle, as stiff cloth when moved.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The green wheat <qex>crinkles</qex> like a lake.</q> <rj><qau>L. T. Trowbridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And all the rooms<br/
Were full of <qex>crinkling</qex> silks.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinkle</ent><br/
<hw>Crin"kle</hw>, <pos>n.</pos> <def>A winding or turn; wrinkle; sinuosity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crinkles</qex> in this glass, making objects appear double.</q> <rj><qau>A. Tucker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinkled</ent><br/
<hw>Crin"kled</hw> <pr>(kr?n"k'ld)</pr>, <pos>a.</pos> <def>Having short bends, turns, or wrinkles; wrinkled; wavy; zigzag.</def>  <ldquo/The <xex>crinkled</xex> lightning.<rdquo/  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crinkleroot</ent><br/
<ent>crinkle root</ent><br/
<mhw><hw>crinkle root</hw>, <hw>crinkleroot</hw></mhw> <pos>n.</pos> <def>a North American herb (<spn>Dentaria diphylla</spn>) with pungent scaly or toothed roots; -- called also <altname>toothwort</altname>.</def><br/
<syn><b>Syn. --</b> pepper root, toothwort, <spn>Cardamine diphylla</spn>, <spn>Dentaria diphylla</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crinkly</ent><br/
<hw>Crin"kly</hw> <pr>(-kl?)</pr>, <pos>a.</pos> <def>Having crinkles; wavy; wrinkly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinoid</ent><br/
<hw>Cri"noid</hw> <pr>(kr<imac/"noid)</pr>, <pos>a.</pos> <ety>[See <er>Crinoidea</er>.]</ety> <fld>(Zool.)</fld> <def>Crinoidal.</def> -- <def2><pos>n.</pos> <def>One of the Crinoidea.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinoidal</ent><br/
<hw>Cri*noid"al</hw> <pr>(kr<isl/*noid"<ait/l)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of pertaining to crinoids; consisting of, or containing, crinoids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinoidea</ent><br/
||<hw>Cri*noid"e*a</hw> <pr>(kr<isl/*noid"<esl/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>kri`non</grk> lily + <ets>-oid</ets>: cf. F.  <ets>crino<ium/de</ets>.]</ety> <fld>(Zool.)</fld> <def>A large class of Echinodermata, including numerous extinct families and genera, but comparatively few living ones.  Most of the fossil species, like some that are recent, were attached by a jointed stem.  See <er>Blastoidea</er>, <er>Cystoidea</er>, <er>Comatula</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinoidean</ent><br/
<hw>Cri*noid"e*an</hw> <pr>(-<it>a</it>n)</pr>, <pos>n.</pos> <fld>(Zool)</fld> <def>One of the Crinoidea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinoline</ent><br/
<hw>Crin"o*line</hw> <pr>(kr?n"?-l?n)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>crin</ets> hair,L. <ets>crinis</ets>.]</ety> <sn>1.</sn> <def>A kind of stiff cloth, used chiefly by women, for underskirts, to expand the gown worn over it; -- so called because originally made of <xex>hair</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lady's skirt made of any stiff material; latterly, a hoop skirt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinose</ent><br/
<hw>Cri*nose"</hw> <pr>(kr?-n?s")</pr>, <pos>a.</pos> <ety>[L.  <ets>crinis</ets> hair.]</ety> <def>Hairy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinosity</ent><br/
<hw>Cri*nos"i*ty</hw> <pr>(kr?-n?s"?-t?)</pr>, <pos>n.</pos> <def>Hairiness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crinum</ent><br/
||<hw>Cri"num</hw> <pr>(kr<imac/"n<ucr/m)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kri`non</grk> lily.]</ety> <fld>(Bot.)</fld> <def>A genus of bulbous plants, of the order <ord>Amaryllidace<ae/</ord>, cultivated as greenhouse plants on account of their beauty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criosphinx</ent><br/
||<hw>Cri"o*sphinx`</hw> <pr>(kr<imac/"<osl/*sf<icr/<nsm/ks`)</pr>, <pos>n.</pos> <ety>[Gr. <grk>krio`s</grk> ram + <grk>sfi`gx</grk> sphinx.]</ety> <def>A sphinx with the head of a ram.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cripple</ent><br/
<hw>Crip"ple</hw> <pr>(kr<icr/p"p'l)</pr>, <pos>n.</pos> <ety>[OE.  <ets>cripel</ets>, <ets>crepel</ets>, <ets>crupel</ets>, AS. <ets>crypel</ets> (akin to D.  <ets>kreuple</ets>, G.  <ets>kr<uum/ppel</ets>, Dan.  <ets>kr<oum/bling</ets>, Icel.  <ets>kryppill</ets>), prop., one that can not walk, but must creep, fr. AS. <ets>cre<oacute/pan</ets> to creep. See <er>Creep</er>.]</ety> <def>One who creeps, halts, or limps; one who has lost, or never had, the use of a limb or limbs; a lame person; hence, one who is partially disabled.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a <qex>cripple</qex> in my limbs; but what decays are in my mind, the reader must determine.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cripple</ent><br/
<hw>Crip"ple</hw>, <pr>(kr<icr/p"p'l)</pr>, <pos>n.</pos> <mark>[Local. U. S.]</mark> <sd>(a)</sd> <def>Swampy or low wet ground, often covered with brush or with thickets; bog.</def></p>

<p><q>The flats or <qex>cripple</qex> land lying between high- and low-water lines, and over which the waters of the stream ordinarily come and go.</q>  <rj><qau>Pennsylvania Law Reports.</qau></rj></p>

<p><sd>(b)</sd> <def>A rocky shallow in a stream; -- a lumberman's term.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cripple</ent><br/
<hw>Crip"ple</hw> <pr>(kr<icr/p"p'l)</pr>, <pos>a.</pos> <def>Lame; halting.</def> <mark>[R.]</mark> <ldquo/The <xex>cripple</xex>, tardy-gaited night.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cripple</ent><br/
<hw>Crip"ple</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crippled</conjf> <pr>(-p'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crippling</conjf> <pr>(-pl?ng)</pr>.]</vmorph> <sn>1.</sn> <def>To deprive of the use of a limb, particularly of a leg or foot; to lame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had <qex>crippled</qex> the joints of the noble child.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deprive of strength, activity, or capability for service or use; to disable; to deprive of resources; <as>as, to be financially <ex>crippled</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>More serious embarrassments . . . were <qex>crippling</qex> the energy of the settlement in the Bay.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An incumbrance which would permanently <qex>cripple</qex> the body politic.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crippled</ent><br/
<hw>Crip"pled</hw> <pr>(kr?p"p'ld)</pr>, <pos>a.</pos> <def>Lamed; lame; disabled; impeded.</def> <ldquo/The <xex>crippled</xex> crone.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crippleness</ent><br/
<hw>Crip"ple*ness</hw>, <pos>n.</pos> <def>Lameness.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crippler</ent><br/
<hw>Crip"pler</hw> <pr>(-pl?r)</pr>, <pos>n.</pos> <def>A wooden tool used in graining leather.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crippling</ent><br/
<hw>Crip"pling</hw> <pr>(-pl?ng)</pr>, <pos>n.</pos> <def>Spars or timbers set up as a support against the side of a building.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cripply</ent><br/
<hw>Crip"ply</hw> <pr>(-pl?)</pr>, <pos>a.</pos> <def>Lame; disabled; in a crippled condition.</def> <mark>[R.]</mark>  <rj><au>Mrs. Trollope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisis</ent><br/
<hw>Cri"sis</hw> <pr>(kr?"s?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crises</plw> <pr>(-s<?/z)</pr>.</plu> <ety>[L.  <ets>crisis</ets>, Gr. <?/<?/<?/<?/, fr. <?/<?/<?/<?/ to separate. See <er>Certain</er>.]</ety> <sn>1.</sn> <def>The point of time when it is to be decided whether any affair or course of action must go on, or be modified or terminate; the decisive moment; the turning point.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This hour's the very <qex>crisis</qex> of your fate.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The very times of <qex>crisis</qex> for the fate of the country.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>That change in a disease which indicates whether the result is to be recovery or death; sometimes, also, a striking change of symptoms attended by an outward manifestation, as by an eruption or sweat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till some safe <qex>crisis</qex> authorize their skill.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisp</ent><br/
<hw>Crisp</hw> <pr>(kr?sp)</pr>, <pos>a.</pos> <ety>[AS. <ets>crisp</ets>, fr. L.  <ets>crispus</ets>; cf. <ets>carpere</ets> to pluck, card (wool), and E. <ets>harvest</ets>.  Cf. <er>Crape</er>.]</ety> <sn>1.</sn> <def>Curling in stiff curls or ringlets; <as>as, <ex>crisp</ex> hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Curled with the ripple of the water.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You nymphs called Naiads, of the winding brooks . . . <br/
Leave jour <qex>crisp</qex> channels.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Brittle; friable; in a condition to break with a short, sharp fracture; <as>as, <ex>crisp</ex> snow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cakes at tea ate short and <qex>crisp</qex>.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Possessing a certain degree of firmness and freshness; in a fresh, unwilted condition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [laurel] has been plucked nine months, and yet looks as hale and <qex>crisp</qex> as if it would last ninety years.</q> <rj><qau>Leigh Hunt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Lively; sparking; effervescing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your neat <qex>crisp</qex> claret.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Brisk; crackling; cheerful; lively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The snug, small room, and the <qex>crisp</qex> fire.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisp</ent><br/
<hw>Crisp</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crisped</conjf> <pr>(kr?spt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crisping</conjf>.]</vmorph> <ety>[L. <ets>crispare</ets>, fr.  <ets>crispus</ets>. See <er>Crisp</er>. <pos>a.</pos> ]</ety> <sn>1.</sn> <def>To curl; to form into ringlets, as hair, or the nap of cloth; to interweave, as the branches of trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to undulate irregularly, as crape or water; to wrinkle; to cause to ripple. Cf. <er>Crimp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lover with the myrtle sprays<br/
Adorns his <qex>crisped</qex> tresses.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Along the <qex>crisped</qex> shades and bowers.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crisped</qex> brooks,<br/
Rolling on orient pearl and sands of gold.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make crisp or brittle, as in cooking.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crisping iron</b></col>, <cd>an instrument by which hair or any textile fabric is crisped.</cd> -- <col><b>Crisping pin</b></col>, <cd>the simplest form of crisping iron.</cd> <au>Is. iii. 22.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisp</ent><br/
<hw>Crisp</hw>, <pos>v. i.</pos> <def>To undulate or ripple.  Cf.  <er>Crisp</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q>To watch the <qex>crisping</qex> ripples on the beach.</q> <rj><qau>Tennuson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisp</ent><br/
<hw>Crisp</hw>, <pos>n.</pos> <def>That which is crisp or brittle; the state of being crisp or brittle; <as>as, burned to a <ex>crisp</ex></as>; specifically, the rind of roasted pork; crackling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crispated</ent><br/
<ent>Crispate</ent><br/
<mhw>{ <hw>Cris"pate</hw> <pr>(kr?s"p?t)</pr>, <hw>Cris"pa*ted</hw> <pr>(-p?-t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L.  <ets>crispatus</ets>, p. p. of <ets>crispare</ets>.]</ety> <def>Having a crisped appearance; irregularly curled or twisted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crispation</ent><br/
<hw>Cris*pa"tion</hw> <pr>(kr?s-p?"sh?n)</pr>, <pos>n.</pos> <ety>[CF. F.  <ets>crispation</ets>.]</ety> <sn>1.</sn> <def>The act or process of curling, or the state of being curled.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A very slight convulsive or spasmodic contraction of certain muscles, external or internal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Few men can look down from a great height without creepings and <qex>crispations</qex>.</q> <rj><qau>O. W. Holmes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crispature</ent><br/
<hw>Cris"pa*ture</hw> <pr>(kr?s"p?-t?r; 135)</pr>, <pos>n.</pos> <def>The state of being crispate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisper</ent><br/
<hw>Crisp"er</hw> <pr>(kr?s"p?r)</pr>, <pos>n.</pos> <def>One who, or that which, crisps or curls; an instrument for making little curls in the nap of cloth, as in chinchilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crispin</ent><br/
<hw>Cris"pin</hw> <pr>(kr?s"p?n)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A shoemaker; -- jocularly so called from the patron saint of the craft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of a union or association of shoemakers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisply</ent><br/
<hw>Crisp"ly</hw> <pr>(kr?sp"l?)</pr>, <pos>adv.</pos> <def>In a crisp manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crispness</ent><br/
<hw>Crisp"ness</hw>, <pos>n.</pos> <def>The state or quality of being crisp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crispy</ent><br/
<hw>Crisp"y</hw> <pr>(-?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Formed into short, close ringlets; frizzed; crisp; <as>as, <ex>crispy</ex> locks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Crisp; brittle; <as>as, a <ex>crispy</ex> pie crust</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crissal</ent><br/
<hw>Cris"sal</hw> <pr>(kr?s"s<it>a</it>l)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <sn>1.</sn> <def>Pertaining to the crissum; <as>as, <ex>crissal</ex> feathers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having highly colored under tail coverts; <as>as, the <ex>crissal</ex> thrasher</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisscross</ent><br/
<hw>Criss"cross`</hw> <pr>(kr?s"kr?s`; 115)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>Christcross</ets>.]</ety> <sn>1.</sn> <def>A mark or cross, as the signature of a person who is unable to write.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A child's game played on paper or on a slate, consisting of lines arranged in the form of a cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisscross</ent><br/
<hw>Criss"cross`</hw>, <pos>v. t.</pos> <def>To mark or cover with cross lines; <as>as, a paper was <ex>crisscrossed</ex> with red marks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 346 --></p>

<p><ent>Crisscross</ent><br/
<hw>Criss"cross`</hw> <pr>(kr?s"kr?s`;115)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In opposite directions; in a way to cross something else; crossing one another at various angles and in various ways.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Logs and tree luing <qex>crisscross</qex> in utter confusion.</q> <rj><qau>W. E. Boardman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With opposition or hindrance; at cross purposes; contrarily; <as>as, things go <ex>crisscross</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crisscross-row</ent><br/
<hw>Criss"cross-row`</hw> <pr>(-r?`)</pr>, <pos>n.</pos> <def>See <er>Christcross-row</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crissum</ent><br/
||<hw>Cris"sum</hw> <pr>(kr?s"s?m)</pr>,, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crissa</plw> <pr>(-s<?/)</pr>.</plu> <ety>[NL.; cf. L.  <ets>crisso</ets> to move the haunches.]</ety> <fld>(Zool.)</fld> <def>That part of a bird, or the feathers, surrounding the cloacal opening; the under tail coverts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cristate</ent><br/
<hw>Cris"tate</hw> <pr>(kr?s"t?t)</pr>, <pos>a.</pos> <ety>[L.  <ets>ctistatus</ets>, fr.  <ets>crista</ets> crest.]</ety> <fld>(Bot. & Zool.)</fld> <def>Crested.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>criterional</ent><br/
<ent>criterial</ent><br/
<mhw><hw>criterial</hw> <hw>criterional</hw></mhw> <pos>adj.</pos> <def>Serving as a basis for evaluation.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Criterion</ent><br/
<hw>Cri*te"ri*on</hw> <pr>(kr?-t?"r?-?n)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  <plw>Criteria</plw> (-<?/), sometimes <plw>Criterions</plw> <pr>(-<?/nz)</pr>.</plu> <ety>[Gr. <?/<?/<?/<?/<?/ a means for judging, fr. <?/<?/<?/<?/ decider, judge, fr. <?/<?/<?/<?/<?/ to separate. See <er>Certain</er>.]</ety> <def>A standard of judging; any approved or established rule or test, by which facts, principles opinions, and conduct are tried in forming a correct judgment respecting them.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of the diseases of the mind there is no <qex>criterion</qex>.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Inferences founded on such enduring <qex>criteria</qex>.</q> <rj><qau>Sir G. C. Lewis.</qau></rj></p>

<p><syn><b>Syn.</b> -- Standard; measure; rule.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crith</ent><br/
<hw>Crith</hw> <pr>(kr<icr/th)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kriqh`</grk> a barleycorn, a small weight.]</ety> <fld>(Chem.)</fld> <def>The unit for estimating the weight of a<eum/riform substances; -- the weight of a liter of hydrogen at 0<deg/ centigrade, and with a tension of 76 centimeters of mercury. It is 0.0896 of a gram, or 1.38274 grains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crithomancy</ent><br/
<hw>Crith"o*man`cy</hw> <pr>(kr<icr/th"<osl/*m<acr/n`s<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kriqai`</grk>, pl., barley + <ets>-mancy</ets>: cf. F. <ets>crithomancie</ets>.]</ety> <def>A kind of divination by means of the dough of the cakes offered in the ancient sacrifices, and the meal strewed over the victims.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Critic</ent><br/
<hw>Crit"ic</hw> <pr>(kr<icr/t"<icr/k)</pr>, <pos>n.</pos> <ety>[L. <ets>criticus</ets>, Gr. <grk>kritiko`s</grk>, a critic; prop., an adj. meaning <xlati>able to discuss</xlati>, from <grk>kri`nein</grk> to judge, discern. See <er>Certain</er>, and cf. <er>Critique</er>.]</ety> <sn>1.</sn> <def>One skilled in judging of the merits of literary or artistic works; a connoisseur; an adept; hence, one who examines literary or artistic works, etc., and passes judgment upon them; a reviewer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The opininon of the most skillful <qex>critics</qex> was, that nothing finer [than Goldsmith's <ldquo/Traveler<rdquo/] had appeared in verse since the fourth book of the <ldquo/Dunciad.<rdquo/</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who passes a rigorous or captious judgment; one who censures or finds fault; a harsh examiner or judge; a caviler; a carper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When an author has many beauties consistent with virtue, piety, and truth, let not little <qex>critics</qex> exalt themselves, and shower down their ill nature.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You know who the <qex>critics</qex> are? the men who have failed in literature and art.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The art of criticism.</def> <mark>[Obs.]</mark>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An act of criticism; a critique.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And make each day a <qex>critic</qex> on the last.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Critic</ent><br/
<hw>Crit"ic</hw>, <pos>a.</pos> <def>Of or pertaining to critics or criticism; critical.</def> <mark>[Obs.]</mark> <ldquo/<xex>Critic</xex> learning.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Critic</ent><br/
<hw>Crit"ic</hw>, <pos>v. i.</pos> <ety>[Cf. F.  <ets>critiquer</ets>.]</ety> <def>To criticise; to play the critic.</def> <mark>[Obs.]</mark><br/
<syn><b>Syn. --</b> critique.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>Nay, if you begin to <qex>critic</qex> once, we shall never have done.</q> <rj><qau>A. Brewer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Critical</ent><br/
<hw>Crit"ic*al</hw> <pr>(kr<icr/t"<icr/*k<ait/l)</pr>, <pos>a.</pos> <ety>[See <er>Critic</er>, <pos>n.</pos>, <er>Crisis</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Qualified to criticise, or pass judgment upon, literary or artistic productions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is submitted to the judgment of more <qex>critical</qex> ears to direct and determine what is graceful and what is not.</q> <rj><qau>Holder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to criticism or the critic's art; of the nature of a criticism; accurate; <as>as, <ex>critical</ex> knowledge; a <ex>critical</ex> dissertation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Inclined to make nice distinctions, or to exercise careful judgment and selection; exact; nicely judicious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Virgil was so <qex>critical</qex> in the rites of religion, that he would never have brought in such prayers as these, if they had not been agreeable to the Roman customs.</q> <rj><qau>Bp. Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Inclined to criticise or find fault; fastidious; captious; censorious; exacting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O gentle lady, do not put me to 't,<br/
For I am nothing, if not <qex>critical</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Characterized by thoroughness and a reference to principles, as becomes a critic; <as>as, a <ex>critical</ex> analysis of a subject</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[See <er>Crisis</er>.]</ety> <def>Pertaining to, or indicating, a crisis, turning point, or specially important juncture; important as regards consequences; hence, of doubtful issue; attended with risk; dangerous; <as>as, the <ex>critical</ex> stage of a fever; a <ex>critical</ex> situation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our circumstances are indeed <qex>critical</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The small moment, the exact point, the <qex>critical</qex> minute, on which every good work so much depends.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Critical angle</b></col> <fld>(Optics)</fld>, <cd>that angle of incidence of a luminous ray at which it is wholly reflected, and no portion of it transmitted. The sine of this angle is the reciprocal of the refractive index of the medium.</cd> -- <col><b>Critical philosophy</b></col>, <cd>the metaphysical system of Kant; -- so called from his most important work, the <ldquo/Critique of Pure Reason.<rdquo/</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Critically</ent><br/
<hw>Crit"ic*al*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a critical manner; with nice discernment; accurately; exactly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Critically</qex> to discern good writers from bad.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>At a crisis; at a critical time; in a situation, place, or condition of decisive consequence; <as>as, a fortification <ex>critically</ex> situated</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Coming <qex>critically</qex> the night before the session.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticalness</ent><br/
<hw>Crit"ic*al*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state or quality of being critical, or of occurring at a critical time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Accuracy in examination or decision; exactness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>critical point</ent><br/
<hw>crit"ic*al point</hw> <fld>(Physics)</fld>, <def>That combination of volume and pressure, at the critical temperature of the substance, at which the liquid and gaseous phases of a given quantity of a substance have identical values for their densities and other properties.</def><br/
[<source>PJC</source>]</p>

<p><ent>critical temperature</ent><br/
<hw>crit"ic*al tem"per*a*ture</hw> <fld>(Physics)</fld>, <def>A certain temperature, different for different gases, but always the same for each gas, regarded as the temperature above which no amount of pressure can produce condensation to a liquid; the temperature above which a gas cannot be liquified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticaster</ent><br/
<hw>Crit"ic*as`ter</hw> <pr>(kr<icr/t"<icr/k*<acr/s`t<etil/r)</pr>, <pos>n.</pos> <def>A contemptible or vicious critic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rancorous and reptile crew of poeticules, who decompose into <qex>criticasters</qex>.</q> <rj><qau>Swinburne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticisable</ent><br/
<hw>Crit"i*cis`a*ble</hw> <pr>(kr<icr/t"<icr/*s<imac/z`<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being criticised.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticise</ent><br/
<hw>Crit"i*cise</hw> <pr>(kr?t"?-s?z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Criticised</conjf> <pr>(-s?zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Criticising</conjf>.]</vmorph> <ety>[Written also, more analogically, but less commonly, <ets>criticize</ets>.]</ety> <ety>[Cf. G.  <ets>kritisiren</ets>. See <er>Critic</er>.]</ety> <sn>1.</sn> <def>To examine and judge as a critic; to pass literary or artistic judgment upon; <as>as, to <ex>criticise</ex> an author; to <ex>criticise</ex> a picture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To express one's views as to the merit or demerit of; esp., to animadvert upon; to find fault with; <as>as, to <ex>criticise</ex> conduct</as>.</def>  <rj><au>Blackwood's Mag.</au></rj><br/
<syn><b>Syn. --</b> criticize, pick apart.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticise</ent><br/
<hw>Crit"i*cise</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To act as a critic; to pass literary or artistic judgment; to play the critic; -- formerly used with <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Several of these ladies, indeed, <qex>criticised</qex> upon the form of the association.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To discuss the merits or demerits of a thing or person; esp., to find fault.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cavil you may, but never <qex>criticise</qex>.</q> <rj><qau> Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticiser</ent><br/
<hw>Crit"i*ci`ser</hw> <pr>(-s?`z?r)</pr>, <pos>n.</pos> <def>One who criticises; a critic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Criticism</ent><br/
<hw>Crit"i*cism</hw> <pr>(kr?t"?-s?z'm)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The rules and principles which regulate the practice of the critic; the art of judging with knowledge and propriety of the beauties and faults of a literary performance, or of a production in the fine arts; <as>as, dramatic <ex>criticism</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The elements of<qex>criticism</qex> depend on the two principles of Beauty and Truth, one of which is the final end or object of study in every one of its pursuits: Beauty, in letters and the arts; Truth, in history and sciences.</q> <rj><qau>Brande & C.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>criticism</qex>, as it was first instituted by Aristotle, was meant a standard of judging well.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of criticising; a critical judgment passed or expressed; a critical observation or detailed examination and review; a critique; animadversion; censure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>About the plan of <ldquo/Rasselas<rdquo/ little was said by the critics; and yet the faults of the plan might seem to invite severe <qex>criticism</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>criticize</ent><br/
<hw>criticize</hw> <pos>v. t.</pos> <def>same as <er>criticise</er>; <as>as, The paper <ex>criticized</ex> the new movie</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Critique</ent><br/
<hw>Cri*tique"</hw> <pr>(kr<icr/*t<emac/k")</pr>, <pos>n.</pos> <ety>[F. <ets>critique</ets>, f., fr. Gr. <grk>kritikh`</grk> (sc. <grk>te`chnh</grk>) the critical art, from <grk>kritiko`s</grk>. See <er>Critic</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The art of criticism.</def> <altsp>[Written also <asp>critic</asp>.]</altsp> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A critical examination or estimate of a work of literature or art; a critical dissertation or essay; a careful and thorough analysis of any subject; a criticism; <as>as, Kant's <ldquo/<ex>Critique</ex> of Pure Reason</as>.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I should as soon expect to see a <qex>critique</qex> on the poesy of a ring as on the inscription of a medal.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A critic; one who criticises.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A question among <qex>critiques</qex> in the ages to come.</q> <rj><qau>Bp. Lincoln.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Critique</ent><br/
<hw>Cri*tique"</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Critic</er>, <pos>v.</pos>]</ety> <def>To criticise or pass judgment upon.</def> <mark>[Obs.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>critter</ent><br/
<hw>crit"ter</hw> <pr>(kr<icr/t"t<etil/r)</pr>, <pos>n.</pos> <ety>[from <ets>creature</ets>.]</ety> <altsp>[Also spelled <asp>crittur</asp>.]</altsp> <sn>1.</sn> <def>Any animal; <as>as, lots of <ex>critters</ex> come out only at night</as>.</def> <mark>[U. S., western dialect]</mark> <br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <def>A domestic animal or a non-predatory wild animal; -- contrasted with <contr>varmint</contr>, also dialectal.</def> <mark>[U. S., western dialect]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Crizzel</ent><br/
<hw>Criz"zel</hw> <pr>(kr<icr/z"z'l)</pr>, <pos>n.</pos> <ety>[Cf.  <ets>grizzle</ets> darkish gray, or G. <ets>griselig</ets> gravelly, granular, speckled.]</ety> <def>A kind of roughness on the surface of glass, which clouds its transparency.</def> <altsp>[Written also <asp>crizzeling</asp> and <asp>crizzle</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>CRO</ent><br/
<hw>CRO</hw> <pos>n.</pos> <fld>(Electronics)</fld> <def>acronym for <er>cathode-ray oscilloscope</er>, an electronic device which provides visual images of varying electrical quantities.  See <er>oscilloscope</er>.</def><br/
<syn><b>Syn. --</b> oscilloscope, scope, cathode-ray oscilloscope.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Croak</ent><br/
<hw>Croak</hw> <pr>(kr<omac/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Croaked</conjf>. (kr<omac/kt); <pos>p. pr. & vb. n.</pos> <conjf>Croaking</conjf>.]</vmorph> <ety>[From the primitive of AS.  <ets>cracettan</ets> to croak as a raven; akin to G.  <ets>kr<aum/chzen</ets> to croak, and to E. <ets>creak</ets>, <ets>crake</ets>.]</ety> <sn>1.</sn> <def>To make a low, hoarse noise in the throat, as a frog, a raven, or a crow; hence, to make any hoarse, dismal sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Loud thunder to its bottom shook the bog,<br/
And the hoarse nation <qex>croaked</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To complain; especially, to grumble; to forebode evil; to utter complaints or forebodings habitually.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Marat . . . <qex>croaks</qex> with reasonableness.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croak</ent><br/
<hw>Croak</hw>, <pos>v. t.</pos> <def>To utter in a low, hoarse voice; to announce by croaking; to forebode; <as>as, to <ex>croak</ex> disaster</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The raven himself is hoarse,<br/
That <qex>croaks</qex> the fatal entrance of Duncan.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Two ravens now began to <qex>croak</qex><br/
Their nuptial song.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croak</ent><br/
<hw>Croak</hw>, <pos>n.</pos> <def>The coarse, harsh sound uttered by a frog or a raven, or a like sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croaker</ent><br/
<hw>Croak"er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who croaks, murmurs, grumbles, or complains unreasonably; one who habitually forebodes evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small American fish (<spn>Micropogon undulatus</spn>), of the Atlantic coast.</def> <sd>(a)</sd> <def>An American fresh-water fish (<spn>Aplodinotus grunniens</spn>); -- called also <altname>drum</altname>.</def> <sd>(c)</sd> <def>The surf fish of California.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When caught these fishes make a croaking sound; whence the name, which is often corrupted into <xex>crocus</xex>.</note></p>

<p><ent>croaky</ent><br/
<hw>croaky</hw> <pos>adj.</pos> <def>like the sounds of frogs and crows; <as>as, acres of <ex>croaky</ex> frogs</as>.</def><br/
<syn><b>Syn. --</b> croaking, guttural.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Croat</ent><br/
<hw>Cro"at</hw> <pr>(kr?"?t)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cravat</er>.]</ety> <sn>1.</sn> <def>A native or resident of Croatia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An irregular soldier, generally from Croatia.</def> <mark>[archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croatia</ent><br/
<hw>Croatia</hw> <pos>n.</pos> <def>a Slavic-speaking country on the Adriatic, part of the Balkan region.  It was formerly part of Yugoslavia.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Croatian</ent><br/
<hw>Cro*a"tian</hw> <pr>(kr?-?"sh<it>a</it>n)</pr>, <pos>a.</pos> <def>Of or pertaining to Croatia.</def> -- <def2><pos>n.</pos> <def>A Croat.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>croc</ent><br/
<hw>croc</hw>, <pos>n.</pos> <def>a contraction for <er>crocodile</er>.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Crocein</ent><br/
<hw>Cro"ce*in</hw> <pr>(kr?"s?-?n)</pr>, <pos>n.</pos> <ety>[See <er>Croceous</er>.]</ety> <fld>(Chem.)</fld> <def>A name given to any one of several yellow or scarlet dyestuffs of artificial production and complex structure.  In general they are diazo and sulphonic acid derivatives of benzene and naphthol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croceous</ent><br/
<hw>Cro"ceous</hw> <pr>(kr?"sh?s)</pr>, <pos>a.</pos> <ety>[L. <ets>croceus</ets>, fr.  <ets>crocus</ets> saffron.  See <er>Crocus</er>.]</ety> <def>Of, pertaining to, or like, saffron; deep reddish yellow.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocetin</ent><br/
<hw>Cro"ce*tin</hw> <pr>(kr?"s?-t?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A dyestuff, obtained from the Chinese crocin, which produces a brilliant yellow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croche</ent><br/
<hw>Croche</hw> <pr>(kr<omac/ch)</pr>, <pos>n.</pos> <ety>[OF. <ets>croche</ets>, equiv. to F. <ets>crochet</ets>, <ets>croc</ets>, hook. See <er>Crotchet</er>, <er>Crook</er>.]</ety> <def>A little bud or knob at the top of a deer's antler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crochet</ent><br/
<hw>Cro*chet"</hw> <pr>(kr?-sh?")</pr>, <pos>n.</pos> <ety>[F.  <ets>crochet</ets> small hook. See <er>Croche</er>.]</ety> <def>A kind of knitting done by means of a hooked needle, with worsted, silk, or cotton; crochet work. Commonly used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Crochet hook</b></col>, <col><b>Crochet needle</b></col></mcol>, <cd>a small hook, or a hooked needle (often of bone), used in crochet work.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crochet</ent><br/
<hw>Cro*chet"</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crocheted</conjf> <pr>(sh<amac/d")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crocheting</conjf> <pr>(-sh<amac/"<icr/ng)</pr>.]</vmorph> <def>To knit with a crochet needle or hook; <as>as, to <ex>crochet</ex> a shawl</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crocheting</ent><br/
<hw>crocheting</hw> <pos>n.</pos> <sn>1.</sn> <def>something created by interlocking looped stitches with a hooked needle.</def><br/
<syn><b>Syn. --</b> crochet.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>creating a garment of needlework.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crociary</ent><br/
<hw>Cro"ci*a*ry</hw> <pr>(kr?"sh?-?-r?)</pr>, <pos>n.</pos> <ety>[See <er>Crosier</er>.]</ety> <fld>(Eccl.)</fld> <def>One who carries the cross before an archbishop.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocidolite</ent><br/
<hw>Cro*cid"o*lite</hw> <pr>(kr?-s?d"?-l?t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kroky`s</grk> nap on cloth + <ets>-lite</ets>.]</ety> <fld>(Min.)</fld> <def>A mineral occuring in silky fibers of a lavender blue color. It is related to hornblende and is essentially a silicate of iron and soda; -- called also <altname>blue asbestus</altname>. A silicified form, in which the fibers penetrating quartz are changed to oxide of iron, is the yellow brown <stype>tiger-eye</stype> of the jewelers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocin</ent><br/
<hw>Cro"cin</hw> <pr>(kr?"s?n)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ saffron.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>The coloring matter of Chinese yellow pods, the fruit of <spn>Gardenia grandiflora</spn>.</def> <au>Watts.</au> <sd>(b)</sd> <def>A red powder (called also <altname>polychroite</altname>), which is made from the saffron (<spn>Crocus sativus</spn>). See <er>Polychroite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw> <pr>(kr<ocr/k)</pr>, <pos>n.</pos> <ety>[Cf. W. <ets>croeg</ets> cover, Scot.  <ets>crochit</ets> covered.]</ety> <def>The loose black particles collected from combustion, as on pots and kettles, or in a chimney; soot; smut; also, coloring matter which rubs off from cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw>, <pos>n.</pos> <def>A low stool.</def> <ldquo/I . . . seated her upon a little <xex>crock</xex>.<rdquo/  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw> <pr>(kr<ocr/k)</pr>, <pos>n.</pos> <ety>[AS.  <ets>croc</ets>, <ets>croca</ets>, <ets>crog</ets>, <ets>croh</ets>; akin to D. <ets>kruik</ets>, G. <ets>krug</ets>, Icel.  <ets>krukka</ets>, Dan. <ets>krukke</ets>, Sw. <ets>kruka</ets>; but cf. W. <ets>crwc</ets> bucket, pail, <ets>crochan</ets> pot, <ets>cregen</ets> earthen vessel, jar. Cf. <er>Cruet</er>.]</ety> <def>Any piece of crockery, especially of coarse earthenware; an earthen pot or pitcher.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like foolish flies about an honey <qex>crock</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw>, <pos>n.</pos> <sn>1.</sn> <def>a person who is worn out with age or illness.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>an old person who complains frequently about illness, especially imaginary ailments.</def><br/
[<source>PJC</source>]</p>

<p><ent>crock</ent><br/
<hw>crock</hw>, <pos>n.</pos> <def>nonsense; balderdash; humbug; -- usually used in the phrase <xex>a crock</xex>.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crocked</conjf> <pr>(kr<ocr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crocking</conjf>.]</vmorph> <def>To soil by contact, as with soot, or with the coloring matter of badly dyed cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw>, <pos>v. i.</pos> <def>To give off crock or smut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crock</ent><br/
<hw>Crock</hw>, <pos>v. t.</pos> <def>To lay up in a crock; <as>as, to <ex>crock</ex> butter</as>.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crocked</ent><br/
<hw>crocked</hw> <pos>adj.</pos>  <def>drunk, inebriated.  Opposite of <ant>sober</ant>.</def> <mark>[slang]</mark><br/
<syn><b>Syn. --</b> blotto, fuddled, loaded, pissed, pixilated, plastered, potty, slopped, sloshed, smashed, soused, sozzled, squiffy, stiff, tiddly, tiddley, tight, tipsy, wet.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crocker</ent><br/
<hw>Crock"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>A potter.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crockery</ent><br/
<hw>Crock"er*y</hw> <pr>( kr?k"?r-?)</pr>, <pos>n.</pos> <ety>[From <er>Crock</er> an earthen vessel.]</ety> <def>Earthenware; vessels formed of baked clay, especially the coarser kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocket</ent><br/
<hw>Crock"et</hw> <pr>(kr?k"?t)</pr>, <pos>n.</pos> <ety>[OF.  <ets>croquet</ets>, F.  <ets>crochet</ets>, dim. of <ets>croc</ets> hook. See <er>Crook</er>, and cf. <er>Crotchet</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>An ornament often resembling curved and bent foliage, projecting from the sloping edge of a gable, spire, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A croche, or knob, on the top of a stag's antler.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The antlers and the <qex>crockets</qex>.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocketed</ent><br/
<hw>Crock"et*ed</hw>, <pos>a.</pos> <fld>(Arch.)</fld> <def>Ornamented with crockets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocketing</ent><br/
<hw>Crock"et*ing</hw>, <pos>n.</pos> <fld>(Arch.)</fld> <def>Ornamentation with crockets.</def>  <rj><au>Ruskin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocky</ent><br/
<hw>Crock"y</hw> <pr>(-?)</pr>, <pos>a.</pos> <ety>[From <er>Crock</er> soot.]</ety> <def>Smutty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocodile</ent><br/
<hw>Croc"o*dile</hw> <pr>(kr<ocr/k"<osl/*d<imac/l; 277)</pr>, <pos>n.</pos> <ety>[L.  <ets>crocodilus</ets>, Gr. <grk>kroko`deilos</grk>: cf. F. <ets>crocodile</ets>. Cf.  <er>Cookatrice</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A large reptile of the genus <gen>Crocodilus</gen>, of several species.  They grow to the length of sixteen or eighteen feet, and inhabit the large rivers of Africa, Asia, and America.  The eggs, laid in the sand, are hatched by the sun's heat.  The best known species is that of the Nile (<spn>Crocodilus vulgaris</spn>, or <spn>Crocodilus Niloticus</spn>).  The Florida crocodile (<spn>Crocodilus Americanus</spn>) is much less common than the alligator and has longer jaws.  The name is also sometimes applied to the species of other related genera, as the gavial and the alligator.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>A fallacious dilemma, mythically supposed to have been first used by a crocodile.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crocodile bird</b></col> <fld>(Zool.)</fld>, <cd>an African plover (<spn>Pluvianus <ae/gypticus</spn>) which alights upon the crocodile and devours its insect parasites, even entering its open mouth (according to reliable writers) in pursuit of files, etc.; -- called also <altname>Nile bird</altname>. It is the <altname>trochilos</altname> of ancient writers.</cd> -- <col><b>Crocodile tears</b></col>, <cd>false or affected tears; hypocritical sorrow; -- derived from the fiction of old travelers, that crocodiles shed tears over their prey.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocodilia</ent><br/
||<hw>Croc`o*dil"i*a</hw> <pr>(-d?l"?-?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>crocodilus</ets> crocodile.]</ety> <fld>(Zool.)</fld> <def>An order of reptiles including the crocodiles, gavials, alligators, and many extinct kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocodilian</ent><br/
<hw>Croc`o*dil"i*an</hw> <pr>(kr?k`?-d?l"?-<ait/n)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Like, or pertaining to, the crocodile; characteristic of the crocodile.</def> -- <def2><pos>n.</pos> <def>One of the Crocodilia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocodility</ent><br/
<hw>Croc`o*dil"i*ty</hw> <pr>(-?-t?)</pr>, <pos>n.</pos> <fld>(Logic)</fld> <def>A caption or sophistical mode of arguing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocoisite</ent><br/
<hw>Cro"cois*ite</hw> <pr>(kr?"kois-?t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>croco<?/se</ets>.]</ety> <fld>(Min.)</fld> <def>Same as <er>Crocoite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocoite</ent><br/
<hw>Cro"co*ite</hw> <pr>(kr?"k?-?t)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ saffron.]</ety> <fld>(Min.)</fld> <def>Lead chromate occuring in crystals of a bright hyacinth red color; -- called also <altname>red lead ore</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croconate</ent><br/
<hw>Cro"con*ate</hw> <pr>(kr?"k?n-?t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt formed by the union of croconic acid with a base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croconic</ent><br/
<hw>Cro*con"ic</hw> <pr>(kr?-k?n"?k)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/<?/ saffron.]</ety> <sn>1.</sn> <def>Of, pertaining to, or resembling saffron; having the color of saffron; <as>as, <ex>croconic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to, or derived from, croconic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 347 --></p>

<p><cs><col><b>Croconic acid</b></col> <fld>(Chem.)</fld>, <cd>a yellow crystalline substance, <chform>C5O3(OH)2</chform>, obtained from potassium carboxide, rhodizonic acid, and various phenol and quinone derivatives of benzene, and forming yellow or orange colored salts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocose</ent><br/
<hw>Cro"cose</hw> <pr>(kr<omac/"k<omac/s)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kro`kos</grk> saffron.]</ety> <fld>(Chem.)</fld> <def>A white crystalline sugar, metameric with glucose, obtained from the coloring matter of saffron.</def> <altsp>[Written also <asp>crokose</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crocus</ent><br/
<hw>Cro"cus</hw> <pr>(kr<omac/"k<ucr/s)</pr>, <pos>n.</pos> <ety>[L., saffron, fr. Gr. <grk>kro`kos</grk>; cf. Heb.  <ets>kark<omac/m</ets>, Ar.  <ets>kurkum</ets>, Skr.  <ets>ku<ndot/kuma</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of iridaceous plants, with pretty blossoms rising separately from the bulb or corm. <spn>Crocus vernus</spn> is one of the earliest of spring-blooming flowers; <spn>Crocus sativus</spn> produces the saffron, and blossoms in the autumn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A deep yellow powder; the oxide of some metal calcined to a red or deep yellow color; esp., the oxide of iron (<stype>Crocus of Mars</stype> or <stype>colcothar</stype>) thus produced from salts of iron, and used as a polishing powder.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crocus of Venus</b></col> <fld>(Old Chem.)</fld>, <cd>oxide of copper.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croesus</ent><br/
<hw>Cr<oe/"sus</hw> <pr>(kr<emac/"s<ucr/s)</pr>, <pos>prop. n.</pos> <ety>[L., fr. G. <grk>Kroi^sos</grk>.]</ety> <def>A king of Lydia who flourished in the 6th century <sc>b. c.</sc>, and was renowned for his vast wealth; hence, a common appellation for a very rich man; <as>as, he is a veritable <ex>Cr<oe/sus</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croft</ent><br/
<hw>Croft</hw> <pr>(kr<ocr/ft; 115)</pr>, <pos>n.</pos> <ety>[AS.  <ets>croft</ets>; akin to D. <ets>kroft</ets> hillock; cf. Gael. <ets>croit</ets> hump, croft.]</ety> <def>A small, inclosed field, adjoining a house; a small farm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A few small <qex>crofts</qex> of stone-encumbered ground.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crofter</ent><br/
<hw>Croft"er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who rents and tills a small farm or holding; <as>as, the <ex>crofters</ex> of Scotland</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crofting</ent><br/
<hw>Croft"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Croftland.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Textile Manuf.)</fld> <def>Exposing linen to the sun, on the grass, in the process of bleaching.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croftland</ent><br/
<hw>Croft"land</hw> <pr>(-l?nd)</pr>, <pos>n.</pos> <def>Land of superior quality, on which successive crops are raised.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crofton system</ent><br/
<hw>Crof"ton sys"tem</hw> <pr>(?)</pr>. <ety>[After <person>Sir Walter <etsep>Crofton</etsep></person>, Irish penologist.]</ety> <fld>(Penology)</fld> <def>A system of prison discipline employing for consecutive periods cellular confinement, associated imprisonment under the mark system, restraint intermediate between imprisonment and freedom, and liberation on ticket of leave.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crois</ent><br/
<hw>Crois</hw> <pr>(krois)</pr>.  <pos>n.</pos> <ety>[OF.]</ety> <def>See <er>Cross</er>, <pos>n.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croisado</ent><br/
<ent>Croisade</ent><br/
<mhw>{ <hw>Croi*sade"</hw> <pr>(kroi-s?d")</pr>, <hw>Croi*sa"do</hw> <pr>(-s?"d?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>criosade</ets>. See <er>Crusade</er>.]</ety> <def>A holy war; a crusade.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croise</ent><br/
<hw>Croise</hw> <pr>(krois)</pr>, <pos>n.</pos> <ety>[F. <ets>crois<eacute/</ets> crusader, fr. OF.  <ets>crois</ets>, F.  <ets>croix</ets>, cross. See <er>Cross</er>.]</ety> <sn>1.</sn> <def>A pilgrim bearing or wearing a cross.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A crusader.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The conquests of the <qex>croises</qex> extending over Palestine.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croissante</ent><br/
||<hw>Crois`san`t<eacute/"</hw> <pr>(krw?`s?n`t?")</pr>, <pos>a.</pos> <ety>[F.  <ets>croissant</ets>, adj. & <ets>n.</ets>, crescent.]</ety> <fld>(Her.)</fld> <def>Terminated with crescents; -- said of a cross the ends of which are so terminated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croker</ent><br/
<hw>Cro"ker</hw> <pr>(kr<omac/"k<etil/r)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kro`kos</grk> saffron.]</ety> <def>A cultivator of saffron; a dealer in saffron.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croma</ent><br/
||<hw>Cro"ma</hw> <pr>(kr<omac/"m<adot/)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A quaver.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cro-magnon</ent><br/
<hw>Cro-magnon</hw> <pos>prop. n.</pos> <sn>1.</sn> <def>a race of human-like hominids of the Upper Paleolithic in Europe, considered of the same species as modern humans (<spn>Homo sapiens</spn>).  Their skeletal remains were found mostly in southern France.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a member of the Cro-Magnon race.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cromlech</ent><br/
<hw>Crom"lech</hw> <pr>(kr<ocr/m"l<ecr/k)</pr>, <pos>n.</pos> <ety>[W.  <ets>cromlech</ets>; <ets>crom</ets> bending or bent, concave + <ets>llech</ets> a flat stone; akin to Ir. <ets>cromleac</ets>.]</ety> <fld>(Arch<ae/ol.)</fld> <def>A monument of rough stones composed of one or more large ones supported in a horizontal position upon others. They are found chiefly in countries inhabited by the ancient Celts, and are of a period anterior to the introduction of Christianity into these countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cromorna</ent><br/
<hw>cro*mor"na</hw> <pr>(kr<osl/*m<ocir/r"n<adot/)</pr>, <pos>n.</pos> <ety>[F.  <ets>cromorne</ets> (cf. It. <ets>cromorno</ets>), fr. G.  <ets>krummhorn</ets> crooked horn, cornet, an organ pipe turned like a trumpet; <ets>krumm</ets> crooked + <ets>horn</ets> horn.]</ety> <fld>(Mus.)</fld> <def>A certain reed stop in the organ, of a quality of tone resembling that of the oboe.</def> <altsp>[Corruptly written <asp>cromona</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cromwell</ent><br/
<hw>Cromwell</hw> <pos>prop. n.</pos> <def><person>Oliver Cromwell</person>, b. 1599, d. 1658.</def><br/
<syn><b>Syn. --</b> Oliver Cromwell.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cromwellian</ent><br/
<hw>Cromwellian</hw> <pos>prop. adj.</pos> <def>of or pertaining to <person>Oliver Cromwell</person>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cronartium</ent><br/
<hw>Cronartium</hw> <pos>n.</pos> <def>A genus of rust fungi having aecia produced in raised or swollen sori and teliospores borne in waxy columns.</def><br/
<syn><b>Syn. --</b> genus <gen>Cronartium</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crone</ent><br/
<hw>crone</hw> <pr>(kr<omac/n)</pr>, <pos>n.</pos> <ety>[OD.  <ets>kronie</ets>, <ets>karonie</ets>, an old sheep, OF. <ets>carogne</ets>, F.  <ets>charogne</ets>, carrion (also F.  <ets>carogne</ets> ill-natured woman.). See <er>Carrion</er>, and <er>Crony</er>.]</ety> <sn>1.</sn> <def>An old ewe.</def> <mark>[Obs.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An old woman; -- usually in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But still the <qex>crone</qex> was constant to her note.</q>  <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An old man; especially, a man who talks and acts like an old woman.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The old <qex>crone</qex> [a negro man] lived in a hovel, . . . which his master had given him.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A few old battered <qex>crones</qex> of office.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cronel</ent><br/
<hw>cro"nel</hw> <pr>(kr<omac/"n<ecr/l)</pr>, <pos>n.</pos> <ety>[Cf. <er>Coronel</er> spearhead, <er>Crown</er>.]</ety> <def>The iron head of a tilting spear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cronet</ent><br/
<hw>cro"net</hw> <pr>(kr<omac/"n<ecr/t)</pr>, <pos>n.</pos> <ety>[Cf. <er>Coronet</er>, <er>Crownet</er>.]</ety> <def>The coronet of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cronian</ent><br/
<hw>Cro"ni*an</hw> <pr>(kr<omac/"n<icr/*<ait/n)</pr>, <pos>a.</pos> <ety>[Gr. <grk>Kro`nios</grk> Saturnian, fr. <grk>Kro`nos</grk> Saturn.]</ety> <def>Saturnian; -- applied to the North Polar Sea.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cronstedtite</ent><br/
<hw>cron"stedt*ite</hw> <pr>(kr<ocr/n"st<ecr/t*<imac/t)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A mineral consisting principally of silicate of iron, and crystallizing in hexagonal prisms with perfect basal cleavage; -- so named from the Swedish mineralogist <etsep>Cronstedt</etsep>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crony</ent><br/
<hw>cro"ny</hw> <pr>(kr<omac/"n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cronies</plw> <pr>(kr<omac/"n<icr/z)</pr>.</plu> <ety>[Orig., an old woman. See <er>Crone</er>.]</ety> <sn>1.</sn> <def>A crone.</def> <mark>[Obs.]</mark> <ldquo/Marry not an old <xex>crony</xex>.<rdquo/  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An intimate companion; a familiar friend</def>. <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He soon found his former <qex>cronies</qex>, though all rather the worse for the wear and tear of time.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>croodle</ent><br/
<hw>croo"dle</hw> <pr>(kr<oomac/"d'l)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Cruddle</er>, <er>Crudle</er>.]</ety> <sn>1.</sn> <def>To cower or cuddle together, as from fear or cold; to lie close and snug together, as pigs in straw.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Wright.</au>  <au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A dove to fly home to her nest and <qex>croodle</qex> there.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fawn or coax.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To coo.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>crook</ent><br/
<hw>crook</hw> <pr>(kr<oocr/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>crok</ets>; akin to Icel. <ets>kr<omac/kr</ets> hook, bend, SW. <ets>krok</ets>, Dan. <ets>krog</ets>, OD. <ets>krooke</ets>; or cf. Gael. <ets>crocan</ets> crook, hook, W. <ets>crwca</ets> crooked. Cf. <er>Crosier</er>, <er>Crotchet</er>, <er>Crutch</er>, <er>Encroach</er>.]</ety> <sn>1.</sn> <def>A bend, turn, or curve; curvature; flexure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through lanes, and <qex>crooks</qex>, and darkness.</q> <rj><qau>Phaer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any implement having a bent or crooked end.</def> <specif>Especially:</specif> <sd>(a)</sd> <def>The staff used by a shepherd, the hook of which serves to hold a runaway sheep.</def> <sd>(b)</sd> <def>A bishop's staff of office. Cf. <er>Pastoral staff</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He left his <qex>crook</qex>, he left <qex>his flocks</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A pothook.</def> <ldquo/As black as the <xex>crook</xex>.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An artifice; trick; tricky device; subterfuge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For all yuor brags, hooks, and <qex>crooks</qex>.</q> <rj><qau>Cranmer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>A small tube, usually curved, applied to a trumpet, horn, etc., to change its pitch or key.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A person given to fraudulent practices; an accomplice of thieves, forgers, etc.</def> <mark>[Cant, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>By hook or by crook</b></col>, <cd>in some way or other; by fair means or foul.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crook</ent><br/
<hw>Crook</hw> <pr>(kr??k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crooked</conjf> <pr>(kr??kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crooking</conjf>.]</vmorph> <ety>[OE. <ets>croken</ets>; cf. Sw.  <ets>kr<?/ka</ets>, Dan. <ets>kr<?/ge</ets>. See Crook, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To turn from a straight line; to bend; to curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Crook</qex> the pregnant hinges of the knee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To turn from the path of rectitude; to pervert; to misapply; to twist.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no one thing that <qex>crooks</qex> youth more than such unlawfull games.</q> <rj><qau>Ascham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What soever affairs pass such a man's hands, he <qex>crooketh</qex> them to his own ends.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crook</ent><br/
<hw>Crook</hw>, <pos>v. i.</pos> <def>To bend; to curve; to wind; to have a curvature.</def> <ldquo/ The port . . .  <xex>crooketh</xex> like a bow.<rdquo/  <rj><au>Phaer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their shoes and pattens are snouted, and piked more than a finger long, <qex>crooking</qex> upwards.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crookback</ent><br/
<hw>Crook"back`</hw> <pr>(kr<oocr/k"b<acr/k`)</pr>, <pos>n.</pos> <def>A crooked back; one who has a crooked or deformed back; a hunchback.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crookback</ent><br/
<hw>Crook"back`</hw>, <pos>a.</pos> <def>Hunched.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crookbill</ent><br/
<hw>Crook"bill`</hw> <pr>(kr<oocr/k"b<icr/l`)</pr>, <pos>n.</pos> <fld>(Zool)</fld> <def>A New Zealand plover (<spn>Anarhynchus frontalis</spn>), remarkable for having the end of the beak abruptly bent to the right.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crooked</ent><br/
<hw>Crook"ed</hw> <pr>(kr??k"?d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Characterized by a crook or curve; not straight; turning; bent; twisted; deformed.</def> <ldquo/<xex>Crooked</xex> paths.<rdquo/  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>he is deformed, <qex>crooked</qex>, old, and sere.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not straightforward; deviating from rectitude; distorted from the right.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are a perverse and <qex>crooked</qex> generation.</q> <rj><qau>Deut. xxxii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>False; dishonest; fraudulent; <as>as, <ex>crooked</ex> dealings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crooked whisky</b></col>, <cd>whisky on which the payment of duty has been fraudulently evaded.</cd> <mark>[Slang, U.S.]</mark>  <rj><au>Barlett.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crookedly</ent><br/
<hw>Crook"ed*ly</hw>, <pos>adv.</pos> <def>In a curved or crooked manner; in a perverse or untoward manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crookedness</ent><br/
<hw>Crook"ed*ness</hw>, <pos>n.</pos> <def>The condition or quality of being crooked; hence, deformity of body or of mind; deviation from moral rectitude; perverseness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crooken</ent><br/
<hw>Crook"en</hw> <pr>(kr??k"'n)</pr>, <pos>v. t.</pos> <def>To make crooked.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crookes space</ent><br/
<hw>Crookes space</hw> <pr>(kr<oocr/ks)</pr>. <ety>[After Sir William <etsep>Crookes</etsep>, English chemist, who first described it.]</ety> <fld>(Physics)</fld> <def>The dark space within the negative-pole glow at the cathode of a vacuum tube, observed only when the pressure is low enough to give a striated discharge; -- called also <altname>Crookes layer</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crookes tube</ent><br/
<hw>Crookes" tube`</hw> <pr>(kr??ks" t?b`)</pr>. <fld>(Phys.)</fld> <def>A vacuum tube in which the exhaustion is carried to a very high degree, with the production of a distinct class of effects; -- so called from <person>W. <etsep>Crookes</etsep></person> who introduced it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crookneck</ent><br/
<hw>crook"neck`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Either of two varieties of squash, distinguished by their tapering, recurved necks. The <stype>summer crookneck</stype> is botanically a variety of the pumpkin (<spn>Cucurbita pepo</spn>) and matures early in the season. It is pale yellow in color, with warty excrescences. The <stype>winter crookneck</stype> belongs to a distinct species (<spn>Cucurbita moschata</spn>) and is smooth and often striped.</def> <mark>[U. S.]</mark><br/
<syn><b>Syn. --</b> crookneck squash.</syn><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Croquante</ent><br/
||<hw>Cro`quante"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A brittle cake or other crisp pastry.</def></p>

<p><ent>Croon</ent><br/
<hw>Croon</hw> <pr>(kr<oomac/n)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>croinen</ets>, cf. D.  <ets>kreunen</ets> to moan. <root/24.]</ety> <sn>1.</sn> <def>To make a continuous hollow moan, as cattle do when in pain.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hum or sing in a low tone; to murmur softly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here an old grandmother was <qex>crooning</qex> over a sick child, and rocking it to and fro.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sing in a soft, evenly modulated manner adapted to amplifying systems, especially to sing in such a way with exaggerated sentimentality.</def> <au>MW10</au> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>Croon</ent><br/
<hw>Croon</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crooned</conjf> <pr>(kr<oomac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crooning</conjf>.]</vmorph> <sn>1.</sn> <def>To sing in a low tone, as if to one's self; to hum.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hearing such stanzas <qex>crooned</qex> in her praise.</q> <rj><qau>C. Bront<eacute/.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To soothe by singing softly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fragment of the childish hymn with which he sung and <qex>crooned</qex> himself asleep.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croon</ent><br/
<hw>Croon</hw>, <pos>n.</pos> <sn>1.</sn> <def>A low, continued moan; a murmur.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A low singing; a plain, artless melody.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crooner</ent><br/
<hw>croon"er</hw> <pos>n.</pos> <def>a singer of popular ballads.</def><br/
<syn><b>Syn. --</b> balladeer.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crooning</ent><br/
<hw>crooning</hw> <pos>n.</pos> <sn>1.</sn> <def>singing in a soft low tone; <as>as, her <ex>crooning</ex> soon put the child to sleep</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the act of singing popular songs in a sentimental manner.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crop</ent><br/
<hw>Crop</hw> <pr>(kr<ocr/p)</pr>, <pos>n.</pos> <ety>[OE.  <ets>crop</ets>, <ets>croppe</ets>, craw, top of a plant, harvest, AS.  <ets>crop</ets>, <ets>cropp</ets>, craw, top, bunch, ear of corn; akin to D. <ets>krop</ets> craw, G. <ets>kropf</ets>, Icel. <ets>kroppr</ets> hump or bunch on the body, body; but cf. also W. <ets>cropa</ets>, <ets>croppa</ets>, crop or craw of a bird, Ir. & Gael.  <ets>sgroban</ets>. Cf.  <er>Croup</er>, <er>Crupper</er>, <er>Croup</er>.]</ety> <sn>1.</sn> <def>The pouchlike enlargement of the gullet of birds, serving as a receptacle for food; the craw.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The top, end, or highest part of anything, especially of a plant or tree.</def> <mark>[Obs.]</mark> <ldquo/<xex>Crop</xex> and root.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is cropped, cut, or gathered from a single felld, or of a single kind of grain or fruit, or in a single season; especially, the product of what is planted in the earth; fruit; harvest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lab'ring the soil, and reaping plenteous <qex>crop</qex>,<br/
Corn, wine, and oil.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Grain or other product of the field while standing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything cut off or gathered.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Guiltless of steel, and from the razor free,<br/
It falls a plenteous <qex>crop</qex> reserved for thee.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Hair cut close or short, or the act or style of so cutting; <as>as, a convict's <ex>crop</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Arch.)</fld> <def>A projecting ornament in carved stone. Specifically, a finial.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mining.)</fld> <sd>(a)</sd> <def>Tin ore prepared for smelting.</def> <sd>(b)</sd> <def>Outcrop of a vein or seam at the surface.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A riding whip with a loop instead of a lash.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Neck and crop</b></col>, <cd>altogether; roughly and at once.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crop</ent><br/
<hw>Crop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cropped</conjf> <pr>(kr<ocr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cropping</conjf>.]</vmorph> <sn>1.</sn> <def>To cut off the tops or tips of; to bite or pull off; to browse; to pluck; to mow; to reap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>crop</qex> off from the top of his young twigs a tender one.</q> <rj><qau>Ezek. xvii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To cut off, as if in harvest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Death . . . .<qex>crops</qex> the growing boys.</q>  <rj><au>Creech.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to bear a crop; <as>as, to <ex>crop</ex> a field</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>to cut off an unnecessary portion at the edges; -- of photographs and other two-dimensional images; <as>as, to <ex>crop</ex> her photograph up to the shoulders</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crop</ent><br/
<hw>Crop</hw>, <pos>v. i.</pos> <def>To yield harvest.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To crop out</b></col>. <sd>(a)</sd> <fld>(Geol.)</fld> <cd>To appear above the surface, as a seam or vein, or inclined bed, as of coal.</cd> <sd>(b)</sd> <cd>To come to light; to be manifest; to appear; as, the peculiarities of an author <xex>crop out</xex>.</cd> -- <col><b>To crop up</b></col>, <cd>to sprout; to spring up; to appear suddenly.</cd> <ldquo/Cares <xex>crop up</xex> in villas.<rdquo/  <rj><au>Beaconsfield.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crop-ear</ent><br/
<hw>Crop"-ear`</hw> <pr>(kr?p"?r`)</pr>, <pos>n.</pos> <def>A person or animal whose ears are cropped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crop-eared</ent><br/
<hw>Crop"-eared`</hw> <pr>(kr?p"?rd`)</pr>, <pos>a.</pos> <def>Having the ears cropped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cropful</ent><br/
<hw>Crop"ful</hw> <pr>(-f?l)</pr>, <pos>a.</pos> <def>Having a full crop or belly; satiated.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cropped</ent><br/
<hw>cropped</hw> <pos>adj.</pos> <sn>1.</sn> <def>cut very short; <as>as, her <ex>cropped</ex> hair</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>having an unnecessary portion at the edges cut off; -- of photographs and other two-dimensional images.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cropper</ent><br/
<hw>Crop"per</hw> <pr>(kr?p"p?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One that crops.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A variety of pigeon with a large crop; a pouter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>A machine for cropping, as for shearing off bolts or rod iron, or for facing cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A fall on one's head when riding at full speed, as in hunting; hence, a sudden failure or collapse.</def> <mark>[Slang.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cropsick</ent><br/
<hw>Crop"sick`</hw> <pr>(kr?"s?k`)</pr>, <pos>a.</pos> <def>Sick from excess in eating or drinking.</def> <mark>[Obs.]</mark> <ldquo/<xex>Cropsick</xex> drunkards.<rdquo/ <au>Tate.</au> -- <wordforms><wf>Crop"sick`ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark> <au>Whitlock.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crop-tailed</ent><br/
<hw>Crop"-tailed`</hw> <pr>(-t?ld`)</pr>, <pos>a.</pos> <def>Having the tail cropped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croquet</ent><br/
<hw>Cro*quet"</hw> <pr>(kr?-k?")</pr>, <pos>n.</pos> <ety>[From French; cf. Walloon <ets>croque</ets> blow, fillip. F. <ets>croquet</ets> a crisp biscuit, <ets>croquer</ets> to crunch, fr.  <ets>croc</ets> a crackling sound, of imitative origin. <ets>Croquet</ets> then properly meant a smart tap on the ball.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An open-air game in which two or more players endeavor to drive wooden balls, by means of mallets, through a series of hoops or arches set in the ground according to some pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of croqueting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croquet</ent><br/
<hw>Cro*quet"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Croqueted</conjf> <pr>(-k?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Croqueting</conjf> <pr>(-k?"?ng)</pr>.]</vmorph> <def>In the game of croquet, to drive away an opponent's ball, after putting one's own in contact with it, by striking one's own ball with the mallet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croquette</ent><br/
||<hw>Cro*quette"</hw> <pr>(kr?-k?t")</pr>, <pos>n.</pos> <ety>[F., fr.  <ets>croquer</ets> to crunch.]</ety> <fld>(Cookery)</fld> <def>A ball of minced meat, fowl, rice, vegetables, or other ingredients, often in a thick white sauce, highly seasoned, breaded, and fried; <as>as, a dish of crab <ex>croquettes</ex></as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Crore</ent><br/
<hw>Crore</hw> <pr>(kr<omac/r)</pr>, <pos>n.</pos> <ety>[Hind. <ets>karo<rsdot/</ets>, Skr.  <ets>ko<tsdot/i</ets>.]</ety> <def>Ten millions; <as>as, a <ex>crore</ex> of rupees (which is nearly $5,000,000)</as>.</def> <mark>[East Indies]</mark>  <rj><au>Malcolm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosier</ent><br/
<hw>Cro"sier</hw> <pr>(kr?"zh?r)</pr>, <pos>n.</pos> <ety>[OE. <ets>rocer</ets>, <ets>croser</ets>, <ets>croyser</ets>, fr.  <ets>croce</ets> crosier, OF.  <ets>croce</ets>, <ets>croche</ets>, F.  <ets>crosse</ets>, fr. LL.  <ets>crocea</ets>, <ets>crocia</ets>, from the same German or Celtic sourse as F. <ets>croc</ets> hook; akin to E. <ets>crook</ets>.]</ety> <def>The pastoral staff of a bishop (also of an archbishop, being the symbol of his office as a shepherd of the flock of God.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The true shape of the <xex>crosier</xex> was with a hooked or curved top; the archbishop's staff alone bore a cross instead of a crook, and was of exceptional, not of regular form.</note>  <rj><au>Skeat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosiered</ent><br/
<hw>Cro"siered</hw> <pr>(-zh?rd)</pr>, <pos>a.</pos> <def>Bearing a crosier.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croslet</ent><br/
<hw>Cros"let</hw> <pr>(kr?s"l?t; 115)</pr>, <pos>n.</pos> <def>See <er>Crosslet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross</ent><br/
<hw>Cross</hw> <pr>(kr<ocr/s; 115)</pr>, <pos>n.</pos> <ety>[OE. <ets>crois</ets>, <ets>croys</ets>, <ets>cros</ets>; the former fr. OF.  <ets>crois</ets>, <ets>croiz</ets>, F.  <ets>croix</ets>, fr. L.  <ets>crux</ets>; the second is perh. directly fr. Prov.  <ets>cros</ets>, <ets>crotz</ets>. fr. the same L.  <ets>crux</ets>; cf. Icel.  <ets>kross</ets>. Cf. <er>Crucial</er>, <er>Crusade</er>, <er>Cruise</er>, <er>Crux</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A gibbet, consisting of two pieces of timber placed transversely upon one another, in various forms, as a <universbold>T</universbold>, or +, with the horizontal piece below the upper end of the upright, or as an <universbold>X</universbold>. It was anciently used in the execution of criminals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nailed to the <qex>cross</qex><br/
By his own nation.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sign or mark of the cross, made with the finger, or in ink, etc., or actually represented in some material; the symbol of Christ's death; the ensign and chosen symbol of Christianity, of a Christian people, and of Christendom.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The custom of making the sign of the <qex>cross</qex> with the hand or finger, as a means of conferring blessing or preserving from evil, is very old.</q> <rj><qau>Schaff-Herzog Encyc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Before the <qex>cross</qex> has waned the crescent's ray.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Tis where the <qex>cross</qex> is preached.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Affiction regarded as a test of patience or virtue; trial; disappointment; opposition; misfortune.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heaven prepares a good man with <qex>crosses</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A piece of money stamped with the figure of a cross, also, that side of such a piece on which the cross is stamped; hence, money in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I should bear no <qex>cross</qex> if I did bear you; for I think you have no money in your purse.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An appendage or ornament or anything in the form of a cross; a badge or ornamental device of the general shape of a cross; hence, such an ornament, even when varying considerably from that form; thus, the Cross of the British Order of St. George and St. Michael consists of a central medallion with seven arms radiating from it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Arch.)</fld> <def>A monument in the form of a cross, or surmounted by a cross, set up in a public place; <as>as, a market <ex>cross</ex>; a boundary <ex>cross</ex>; Charing <ex>Cross</ex> in London</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dun-Edin's <qex>Cross</qex>, a pillared stone,<br/
Rose on a turret octagon.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Her.)</fld> <def>A common heraldic bearing, of which there are many varieties. See the Illustration, above.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The crosslike mark or symbol used instead of a signature by those unable to write.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Five Kentish abbesses . . . .subscribed their names and <qex>crosses</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Church lands.</def> <mark>[Ireland]</mark> <mark>[Obs.]</mark>  <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A line drawn across or through another line.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> Hence: <def>A mixing of breeds or stock, especially in cattle breeding; or the product of such intermixture; a hybrid of any kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> Toning down the ancient Viking into a sort of a <qex>cross</qex> between Paul Jones and Jeremy Diddler.</q> <rj><qau>Lord Dufferin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 348 --></p>

<p><sn>12.</sn> <fld>(Surveying)</fld> <def>An instrument for laying of offsets perpendicular to the main course.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Mech.)</fld> <def>A pipe-fitting with four branches the axes of which usually form's right angle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cross and pile</b></col>, <cd>a game with money, at which it is put to chance whether a coin shall fall with that side up which bears the cross, or the other, which is called <xex>pile</xex>, or <xex>reverse</xex>; the game called <altname>heads or tails</altname>.</cd> -- <col><b>Cross bottony</b></col> <it>or</it> <col><b>Cross botton<eacute/</b></col>. <cd>See under <er>Bottony</er>.</cd> -- <col><b>Cross estoil<eacute/</b></col> <fld>(Her.)</fld>. <cd>a cross, each of whose arms is pointed like the ray of a star; that is, a star having four long points only.</cd> -- <col><b>Cross of Calvary</b></col>. <cd>See <er>Calvary</er>, 3.</cd> -- <col><b>Southern cross</b></col>. <fld>(Astron.)</fld> <cd>See under <er>Southern</er>.</cd> -- <col><b>To do a thing on the cross</b></col>, <cd>to act dishonestly; -- opposed to acting <xex>on the square</xex>.</cd> <mark>[Slang]</mark> -- <col><b>To take up the cross</b></col>, <cd>to bear troubles and afflictions with patience from love to Christ.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross</ent><br/
<hw>Cross</hw> <pr>(kr<ocr/s)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Not parallel; lying or falling athwart; transverse; oblique; intersecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cross</qex> refraction of the second prism.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not accordant with what is wished or expected; interrupting; adverse; contrary; thwarting; perverse.</def> <ldquo/A <xex>cross</xex> fortune.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cross</qex> and unlucky issue of my design.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The article of the resurrection seems to lie marvelously <qex>cross</qex> to the common experience of mankind.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We are both love's captives, but with fates so <qex>cross</qex>,<br/
One must be happy by the other's loss.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Characterized by, or in a state of, peevishness, fretfulness, or ill humor; <as>as, a <ex>cross</ex> man or woman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had received a <qex>cross</qex> answer from his mistress.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Made in an opposite direction, or an inverse relation; mutually inverse; interchanged; <as>as, <ex>cross</ex> interrogatories; <ex>cross</ex> marriages, as when a brother and sister marry persons standing in the same relation to each other</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cross action</b></col> <fld>(Law)</fld>, <cd>an action brought by a party who is sued against the person who has sued him, upon the same subject matter, as upon the same contract.</cd> <au>Burrill.</au> -- <col><b>Cross aisle</b></col> <fld>(Arch.)</fld>, <cd>a transept; the lateral divisions of a cruciform church.</cd> -- <col><b>Cross axle</b></col>.  <sd>(a)</sd> <fld>(Mach.)</fld> <cd>A shaft, windlass, or roller, worked by levers at opposite ends, as in the copperplate printing press.</cd> <sd>(b)</sd> <cd>A driving axle, with cranks set at an angle of 90<deg/ with each other.</cd> -- <col><b>Cross bedding</b></col> <fld>(Geol.)</fld>, <cd>oblique lamination of horizontal beds.</cd> -- <col><b>Cross bill</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Cross bitt</b></col>. <cd>Same as <er>Crosspiece</er>.</cd> -- <col><b>Cross bond</b></col>, <cd>a form of bricklaying, in which the joints of one stretcher course come midway between those of the stretcher courses above and below, a course of headers and stretchers intervening. See <er>Bond</er>, <pos>n.</pos>, 8.</cd> -- <col><b>Cross breed</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Cross breeding</b></col>. <cd>See under <er>Breeding</er>.</cd> -- <col><b>Cross buttock</b></col>, <cd>a particular throw in wrestling; hence, an unexpected defeat or repulse.</cd> <au>Smollet.</au> -- <col><b>Cross country</b></col>, <cd>across the country; not by the road.</cd> <ldquo/The <xex>cross-country</xex> ride.<rdquo/ <au>Cowper.</au> -- <col><b>Cross fertilization</b></col>, <cd>the fertilization of the female products of one physiological individual by the male products of another, -- as the fertilization of the ovules of one plant by pollen from another. See <er>Fertilization</er>.</cd> -- <col><b>Cross file</b></col>, <cd>a double convex file, used in dressing out the arms or crosses of fine wheels.</cd> --  <col><b>Cross fire</b></col> <fld>(Mil.)</fld>, <cd>lines of fire, from two or more points or places, crossing each other.</cd> -- <col><b>Cross forked</b></col>. <fld>(Her.)</fld> <cd>See under <er>Forked</er>.</cd> -- <col><b>Cross frog</b></col>. <cd>See under <er>Frog</er>.</cd> -- <col><b>Cross furrow</b></col>, <cd>a furrow or trench cut across other furrows to receive the water running in them and conduct it to the side of the field.</cd> -- <col><b>Cross handle</b></col>, <cd>a handle attached transversely to the axis of a tool, as in the augur.</cd> <au>Knight.</au> -- <col><b>Cross lode</b></col> <fld>(Mining)</fld>, <cd>a vein intersecting the true or principal lode.</cd> -- <col><b>Cross purpose</b></col>. <cd>See <er>Cross-purpose</er>, in the Vocabulary.</cd> -- <col><b>Cross reference</b></col>, <cd>a reference made from one part of a book or register to another part, where the same or an allied subject is treated of.</cd> -- <col><b>Cross sea</b></col> <fld>(Naut.)</fld>, <cd>a chopping sea, in which the waves run in contrary directions.</cd> -- <col><b>Cross stroke</b></col>, <cd>a line or stroke across something, as across the letter <it>t</it>.</cd> -- <col><b>Cross wind</b></col>, <cd>a side wind; an unfavorable wind.</cd> -- <col><b>Cross wires</b></col>, <cd>fine wires made to traverse the field of view in a telescope, and moved by a screw with a graduated head, used for delicate astronomical observations; spider lines. <as>Fixed <ex>cross wires</ex> are also used in microscopes, etc.</as></cd><-- sim. to cross hairs --></cs></p>

<p><syn><b>Syn.</b> -- Fretful; peevish. See <er>Fretful</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross</ent><br/
<hw>Cross</hw>, <pos>prep.</pos> <def>Athwart; across.</def> <mark>[Archaic or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A fox was taking a walk one night <xex>cross</xex> a village.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To go cross lots</b></col>, <cd>to go across the fields; to take a short cut.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross</ent><br/
<hw>Cross</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crossed</conjf> <pr>(kr<ocr/st; 115)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crossing</conjf>.]</vmorph> <sn>1.</sn> <def>To put across or athwart; to cause to intersect; <as>as, to <ex>cross</ex> the arms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lay or draw something, as a line, across; <as>as, to <ex>cross</ex> the letter <it>t</it></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pass from one side to the other of; to pass or move over; to traverse; <as>as, to <ex>cross</ex> a stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A hunted hare . . . <qex>crosses</qex> and confounds her former track.</q> <rj><qau> I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pass, as objects going in an opposite direction at the same time.</def> <ldquo/Your kind letter <xex>crossed</xex> mine.<rdquo/  <rj><au>J. D. Forbes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To run counter to; to thwart; to obstruct; to hinder; to clash or interfere with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In each thing give him way; <qex>cross</qex> him in nothing.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An oyster may be <qex>crossed</qex> in love.</q> <rj><qau> Sheridan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To interfere and cut off; to debar.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>cross</qex> me from the golden time I look for.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To make the sign of the cross upon; -- followed by the reflexive pronoun; <as>as, he <ex>crossed</ex> himself</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To cancel by marking crosses on or over, or drawing a line across; to erase; -- usually with <xex>out</xex>, <xex>off</xex>, or <xex>over</xex>; <as>as, to <ex>cross</ex> out a name</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To cause to interbreed; -- said of different stocks or races; to mix the breed of.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cross a check</b></col> <fld>(Eng. Banking)</fld>, <cd>to draw two parallel transverse lines across the face of a check, with or without adding between them the words <ldquo/and company<rdquo/, with or without the words <ldquo/not negotiable<rdquo/, or to draw the transverse lines simply, with or without the words <ldquo/not negotiable<rdquo/ (the check in any of these cases being <xex>crossed generally</xex>). Also, to write or print across the face of a check the name of a banker, with or without the words <ldquo/not negotiable<rdquo/ (the check being then <xex>crossed specially</xex>). A check crossed generally is payable only when presented through a bank; one crossed specially, only when presented through the bank mentioned.</cd> [<source>Webster 1913 Suppl.</source>] -- <col><b>To cross one's path</b></col>, <cd>to oppose one's plans.</cd> <au>Macaulay.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross</ent><br/
<hw>Cross</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To lie or be athwart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move or pass from one side to the other, or from place to place; to make a transit; <as>as, to <ex>cross</ex> from New York to Liverpool</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be inconsistent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Men's actions do not always <qex>cross</qex> with reason.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To interbreed, as races; to mix distinct breeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If two individuals of distinct races <qex>cross</qex>, a third is invariably produced different from either.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-armed</ent><br/
<hw>Cross"-armed`</hw> <pr>(kr?s"?rmd)</pr>, <pos>a.</pos> <def>With arms crossed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-banded</ent><br/
<hw>Cross"-band`ed</hw> <pr>(-b?nd`?d)</pr>, <pos>a.</pos> <def>A term used when a narrow ribbon of veneer is inserted into the surface of any piece of furniture, wainscoting, etc., so that the grain of it is contrary to the general surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbar</ent><br/
<hw>Cross"bar`</hw> <pr>(-b<aum/r`)</pr>, <pos>n.</pos> <def>A transverse bar or piece, as a bar across a door, or as the iron bar or stock which passes through the shank of an anchor to insure its turning fluke down.</def>  <rj><au>Russell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crossbar shot</b></col>, <cd>a projectile which folds into a sphere for loading, but on leaving the gun expands to a cross with a quarter ball at the end of each arm; -- used in naval actions for cutting the enemy's rigging.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbarred</ent><br/
<hw>Cross"barred`</hw> <pr>(-b<aum/rd`)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Secured by, or furnished with, crossbars.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Made or patterned in lines crossing each other; <as>as, <ex>crossbarred</ex> muslin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbeak</ent><br/
<hw>Cross"beak`</hw> <pr>(-b<emac/k`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Crossbill</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbeam</ent><br/
<hw>Cross"beam`</hw> <pr>(-b<emac/m`)</pr>.  <pos>n.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>A girder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A beam laid across the bitts, to which the cable is fastened when riding at anchor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-bearer</ent><br/
<hw>Cross"-bear`er</hw> <pr>(-b?r`?r)</pr>, <pos>n.</pos> <fld>(R.C.Ch.)</fld> <def>A subdeacon who bears a cross before an archbishop or primate on solemn occasions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crossbencher</ent><br/
<hw>crossbencher</hw> <pos>n.</pos> <def>a member of the House of Commons of Great Britain who does not vote regularly with either the Government or the Opposition.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crossbill</ent><br/
<hw>Cross"bill`</hw> <pr>(-b<icr/l`)</pr>. <fld>(Law)</fld> <def>A bill brought by a defendant, in an equity or chancery suit, against the plaintiff, respecting the matter in question in that suit.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In criminal practice, <xex>cross bills</xex> of indictment for assault, in which the prosecutor in once case is the defendant in another, may be tried together.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbill</ent><br/
<hw>Cross"bill`</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A bird of the genus <gen>Loxia</gen>, allied to the finches. Their mandibles are strongly curved and cross each other; the crossbeak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-birth</ent><br/
<hw>Cross"-birth`</hw> <pr>(kr<ocr/s"b<etil/rth`)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Any preternatural labor, in which the body of the child lies across the pelvis of the mother, so that the shoulder, arm, or trunk is the part first presented at the mouth of the uterus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbite</ent><br/
<hw>Cross"bite`</hw> <pr>(kr<ocr/s"b<imac/t`)</pr>, <pos>n.</pos> <def>A deception; a cheat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbite</ent><br/
<hw>Cross"bite"</hw>, <pos>v. t.</pos> <def>To deceive; to trick; to gull.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbones</ent><br/
<hw>Cross"bones`</hw> <pr>(kr<ocr/s"b<omac/nz`)</pr>, <pos>n. pl.</pos> <def>A representation of two of the leg bones or arm bones of a skeleton, laid crosswise, often surmounted with a skull, and serving as a symbol of death.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Crossbones</qex>, scythes, hourglasses, and other lugubrious emblems of mortality.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbow</ent><br/
<hw>Cross"bow`</hw> <pr>(kr<ocr/s"b<omac/`)</pr>, <pos>n.</pos> <fld>(Archery)</fld> <def>A weapon, used in discharging arrows or bolts, formed by placing a bow crosswise on a stock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbower</ent><br/
<hw>Cross"bow`er</hw> <pr>(-b?`?r)</pr>, <pos>n.</pos> <def>A crossbowman.</def><mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbowman</ent><br/
<hw>Cross"bow`man</hw> <pr>(-m<it>a</it>n)</pr>, <pos>n.</pos> <def>One who shoots with a crossbow. See <er>Arbalest</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbred</ent><br/
<hw>Cross"bred`</hw> <pr>(-br?d`)</pr>, <pos>a.</pos> <fld>(Stock Breeding)</fld> <def>Produced by mixing distinct breeds; mongrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossbreed</ent><br/
<hw>Cross"breed`</hw> <pr>(-br<emac/d`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A breed or an animal produced from parents of different breeds; a new variety, as of plants, combining the qualities of two parent varieties or stocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything partaking of the natures of two different things; a hybrid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-bun</ent><br/
<hw>Cross"-bun`</hw> <pr>(-b<ucr/n`)</pr>, <pos>n.</pos> <def>A bun or cake marked with a cross  of icing, and intended to be eaten on Good Friday; also, called <altname><er>hot cross bun</er></altname>, even when not hot.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cross-buttock</ent><br/
<hw>Cross"-but`tock</hw>, <pos>n.</pos> <fld>(Wrestling)</fld> <def>A throw in which the wrestler turns his left side to his opponent, places his left leg across both legs of his opponent, and pulls him forward over his hip; hence, an unexpected defeat or repulse.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cross-country</ent><br/
<hw>cross-country</hw> <pos>adj.</pos> <sn>1.</sn> <def>from one side of a country to the other; <as>as, a <ex>cross-country</ex> railway</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>moving across open country rather than following tracks or roads; <as>as, a <ex>cross-country</ex> race</as>.  Opposite of <ant>road</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-country</ent><br/
<hw>cross-country</hw> <pos>n.</pos> <def>a race over a course including countryside, rather than over roads or prepared paths.</def><br/
<syn><b>Syn. --</b> cross country.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cross-crosslet</ent><br/
<hw>Cross`-cross"let</hw> <pr>(-kr?s"l?t; 115)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>A cross having the three upper ends crossed, so as to from three small crosses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-cultural</ent><br/
<hw>cross-cultural</hw> <pos>adj.</pos> <def>dealing with or comparing two or more cultures; <as>as, a <ex>cross-cultural</ex> survey</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crosscurrent</ent><br/
<hw>crosscurrent</hw> <pos>n.</pos> <sn>1.</sn> <def>a stretch of turbulent water in the sea.</def><br/
<syn><b>Syn. --</b> rip, riptide, tide rip.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a tendency or sentiment within a group which conflicts with that of the majority; -- often used in the plural.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crosscut</ent><br/
<hw>Cross"cut`</hw> <pr>(-k<ucr/t`)</pr>, <pos>v. t.</pos>  <def>To cut across or through; to intersect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosscut</ent><br/
<hw>Cross"cut`</hw>, <pos>n.</pos> <sn>1.</sn> <def>A short cut across; a path shorter than by the high road.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A level driven across the course of a vein, or across the main workings, as from one gangway to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crosscut saw</b></col>. <sd>(a)</sd> <cd>A saw, the teeth of which are so set as to adapt it for sawing wood crosswise of the grain rather than lengthwise.</cd> <sd>(b)</sd> <cd>A saw managed by two men, one at each end, for cutting large logs crosswise.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-days</ent><br/
<hw>Cross"-days`</hw> <pr>(-d?z`)</pr>, <pos>n. pl.</pos> <fld>(Eccl.)</fld> <def>The three days preceding the Feast of the Ascension.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-division</ent><br/
<hw>cross-division</hw> <pos>n.</pos> <def>classification according to more than one attribute at the same time.</def><br/
<syn><b>Syn. --</b> cross-classification.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-dress</ent><br/
<hw>cross-dress</hw> <pos>v.</pos> <def>to dress in the clothes of the other sex.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-dresser</ent><br/
<hw>cross-dresser</hw> <pos>n.</pos> <def>someone who adopts the dress or manner or sexual role of the opposite sex.</def><br/
<syn><b>Syn. --</b> transvestite.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crosse</ent><br/
<hw>Crosse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., <ets>crosier</ets>, hooked stick.]</ety> <def>The implement with which the ball is thrown and caught in the game of lacrosse.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crossette</ent><br/
<hw>Cros*sette"</hw> <pr>(kr?s-s?t`)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>crosse</ets>. See <er>Crosier</er>.]</ety> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A return in one of the corners of the architrave of a door or window; -- called also <altname>ancon</altname>, <altname>ear</altname>, <altname>elbow</altname>.</def> <sd>(b)</sd> <def>The shoulder of a joggled keystone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-eyed</ent><br/
<hw>cross-eyed</hw> <pos>adj.</pos> <def>having convergent strabismus.  Contrasted with <contr>walleyed</contr>.</def><br/
<syn><b>Syn. --</b> crosseyed, boss-eyed[Brit. informal].</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-eye</ent><br/
<hw>cross-eye</hw> <pos>adj.</pos> <def>convergent strabismus; a disorder in which one or both eyes turn inward toward the nose rather than directly at the object of vision; crossed eyes.</def><br/
[<source>PJC</source>]</p>

<p><ent>cross-examination</ent><br/
<hw>cross"-ex*am`i*na"tion</hw> <pr>(kr?s"?gz-?m`?-n?"sh?n; 115)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The interrogating or questioning of a witness by the party against whom he has been called and examined. See <er>Examination</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>[fig.]</mark> <def>close or detailed questioning.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cross-examine</ent><br/
<hw>Cross"-ex*am"ine</hw> <pr>(-?m"?n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cross-examined</conjf> <pr>(-?nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cross-examining</conjf>.]</vmorph> <fld>(Law)</fld> <def>To examine or question, as a witness who has been called and examined by the opposite party.</def> <ldquo/The opportunity to <xex>cross-examine</xex> the witnesses.<rdquo/  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-examiner</ent><br/
<hw>Cross"-ex*am"in*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who cross-examines or conducts a crosse-examination.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-eye</ent><br/
<hw>Cross"-eye`</hw> <pr>(-?`)</pr>, <pos>n.</pos> <def>See <er>Strabismus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-eyed</ent><br/
<hw>Cross"-eyed`</hw> <pr>(-?d`)</pr>, <pos>a.</pos> <def>Affected with strabismus; squint-eyed; squinting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-fertilization</ent><br/
<ent>cross-fertilisation</ent><br/
<mhw><hw>cross-fertilisation</hw>, <hw>cross-fertilization</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>fertilization by the union of male and female gametes from different individuals of the same species.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cross-fertilize</ent><br/
<hw>Cross"-fer"ti*lize</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cross-fertilized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cross-fertilizing</conjf> <pr>(?)</pr>.]</vmorph> <fld>(Bot.)</fld> <def>To fertilize, as the stigmas of a flower or plant, with the pollen from another individual of the same species.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cross-file</ent><br/
<hw>cross-file</hw> <pos>v. i. & t.</pos> <def>to register as a candidate for more than one political party in the same election.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crossfish</ent><br/
<hw>Cross"fish`</hw> <pr>(-f?sh`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A starfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossflow</ent><br/
<hw>Cross"flow`</hw> <pr>(-fl?`)</pr>, <pos>v. i.</pos> <def>To flow across, or in a contrary direction.</def> <ldquo/His <xex>crossflowing</xex> course.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-garnet</ent><br/
<hw>Cross"-gar`net</hw> <pr>(kr?s"g?r`n?t)</pr>, <pos>n.</pos> <def>A hinge having one strap perpendicular and the other strap horizontal giving it the form of an Egyptian or T cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossgrained</ent><br/
<hw>Cross"grained</hw> <pr>(-gr?nd`)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having the grain or fibers run diagonally, or more or less transversely and irregularly, so as to interfere with splitting or planing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the stuff proves <qex>crossgrained</qex>, . . . then you must turn your stuff to plane it the contrary way.</q> <rj><qau>Moxon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Perverse; untractable; contrary; difficult to deal with.</def><br/
<syn><b>Syn. --</b> balky, contrarious.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><q>She was none of your <qex>crossgrained</qex>, termagant, scolding jades.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosshatch</ent><br/
<hw>Cross"hatch`</hw> <pr>(-h?ch`; 224)</pr>, <pos>v. t.</pos> <def>To shade by means of crosshatching.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosshatching</ent><br/
<hw>Cross"hatch`ing</hw>, <pos>n.</pos> <def>In drawing and line engraving, shading with lines that cross one another at an angle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosshead</ent><br/
<hw>Cross"head`</hw> <pr>(-h?d)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>A beam or bar across the head or end of a rod, etc., or a block attached to it and carrying a knuckle pin; as the solid crosspiece running between parallel slides, which receives motion from the piston of a steam engine and imparts it to the connecting rod, which is hinged to the crosshead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crossheading</ent><br/
<hw>crossheading</hw> <pos>n.</pos> <def>a heading of a subsection printed within the body of the text.</def><br/
<syn><b>Syn. --</b> crosshead.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-index</ent><br/
<hw>cross-index</hw> <pos>n.</pos> <def>a reference at one place in a work to information at another place in the same work.</def><br/
<syn><b>Syn. --</b> cross-reference.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-index</ent><br/
<hw>cross-index</hw> <pos>v. t.</pos> <def>to provide cross-references in (a book or other document).</def><br/
<syn><b>Syn. --</b> cross-reference.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Crossing</ent><br/
<hw>Cross"ing</hw>, <pos>n.</pos> <ety>[See <er>Cross</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>The act by which anything is crossed; <as>as, the <ex>crossing</ex> of the ocean</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of making the sign of the cross.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of interbreeding; a mixing of breeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Intersection, as of two paths or roads.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A place where anything (as a stream) is crossed; a paved walk across a street, or a set of marks across the street pavement indicating that this is a designated location for pedestrians to cross.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>6.</sn> <def>Contradiction; thwarting; obstruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do not bear these <qex>crossings</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crossing guard</ent><br/
<hw>cross"ing guard`</hw>, <pos>n.</pos> <def>a person who stands at a street crossing to assist children to cross the street safely; -- usually used near schools during the times that children are entering or leaving the school.</def>  <note><ex>Crossing guards</ex> may be employed by a municipality as auxiliary police officers, or may be unpaid volunteers.  Some wear uniforms, others do not.   They are commonly older citizens who are retired or not employed full-time at other tasks.  At some schools, older students from the school are trained to perform the function of <ex>crossing guards</ex>.</note><br/
[<source>PJC</source>]</p>

<p><ent>Crossjack</ent><br/
<hw>Cross"jack`</hw> <pr>(kr?s"j?k` <or/ kr?"j?k`)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The lowest square sail, or the lower yard of the mizzenmast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosslegged</ent><br/
<hw>Cross"legged`</hw> <pr>(-l?gd`)</pr>, <pos>a.</pos> <def>Having the legs crossed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosslet</ent><br/
<hw>Cross"let</hw> <pr>(-lEt)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>cross</ets>.]</ety> <sn>1.</sn> <def>A small cross.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. OF.  <ets>croisel</ets> crucible, and <er>E</er>. <er>Cresset</er>.]</ety> <def>A crucible.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosslet</ent><br/
<hw>Cross"let</hw>, <pos>a.</pos> <fld>(Her.)</fld> <def>Crossed again; -- said of a cross the arms of which are crossed. See<er>Cross-crosslet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-linguistic</ent><br/
<hw>cross-linguistic</hw> <pos>adj.</pos> <def>of, relating to, or derived from more than one family of languages; <as>as, <ex>cross-linguistic</ex> evidence</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-link</ent><br/
<hw>cross-link</hw> <pos>n.</pos> <fld>(Chem., Biochem.)</fld> <def>a covalent bond that links two chains of atoms, or two sections of one chain, in a polymeric molecule; the <ex>cross-link</ex> is created by a third bond in a monomer unit in addition to the two bonds forming the polymeric chain; -- a <ex>cross-link</ex> may be internal to a single chain, rather than between two otherwise unlinked chains; <as>as, ultraviolet irradiation creates <ex>cross-links</ex> between the two chains of a DNA double helix; many enzymes have <ex>cross-links</ex> formed by disulfide bonds; polystyrene resins have their porosity controlled by the proportion of <ex>cross-links</ex></as>.  Called also <altname>cross-linkage</altname>.</def><br/
<syn><b>Syn. --</b> cross-linkage.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-link</ent><br/
<hw>cross-link</hw> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>cross-linked</conjf> <pr>(?)</pr>, <pos>p. pr. & vb. n.</pos> <conjf>cross-linking</conjf>.]</vmorph> <fld>(Chem., Biochem.)</fld> <def>to create cross links in; -- of polymeric molecules.</def><br/
[<source>PJC</source>]</p>

<p><ent>cross-linkage</ent><br/
<hw>cross-linkage</hw> <pos>n.</pos> <def>same as <er>cross-link</er>, <pos>n.</pos></def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-linked</ent><br/
<hw>cross-linked</hw> <pos>a.</pos> <fld>(Chem., Biochem.)</fld> <def>containing <er>cross-links</er>; -- of polymeric molecules.</def><br/
<syn><b>Syn. --</b> cross-linkage.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Crossly</ent><br/
<hw>Cross"ly</hw>, <pos>adv.</pos> <def>Athwart; adversely; unfortunately; peevishly; fretfully; with ill humor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-match</ent><br/
<hw>cross-match</hw> <pos>v. t.</pos> <def>to match items from two or more lists.</def><br/
[<source>PJC</source>]</p>

<p><ent>cross-mate</ent><br/
<hw>cross-mate</hw> <pos>v. t.</pos> <def>to crossbreed.</def><br/
[<source>PJC</source>]</p>

<p><ent>cross-modal</ent><br/
<hw>cross-modal</hw> <pos>adj.</pos> <fld>(Grammar)</fld> <def>relating to different sense modalities.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-modality</ent><br/
<hw>cross-modality</hw> <pos>n.</pos> <fld>(Grammar)</fld> <def>ability to integrate information from different sense modalities.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crossness</ent><br/
<hw>Cross"ness</hw>, <pos>n.</pos> <def>The quality or state of being cross; peevishness; fretfulness; ill humor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossopterygian</ent><br/
<hw>Cros*sop`ter*yg"i*an</hw> <pr>(kr?s-s?p`t?r-?j?-<ait/n)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Crossopterygii.</def> -- <def2><pos>n.</pos> <def>One of the Crossopterygii.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossopterygii</ent><br/
||<hw>Cros*sop`te*ryg"i*i</hw> <pr>(kr?s-s?p`t?-r?j?-?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ tassels, a fringe + <?/<?/<?/<?/<?/, dim. of <?/<?/<?/<?/ wing, fin.]</ety> <fld>(Zool.)</fld> <def>An order of ganoid fishes including among living species the bichir (<gen>Polypterus</gen>). See <er>Brachioganoidei</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crossover</ent><br/
<hw>crossover</hw> <pos>n.</pos> <sn>1.</sn> <fld>(genetics)</fld> <def>the interchange of sections between pairing homologous chromosomes during the prophase of meiosis.</def><br/
<syn><b>Syn. --</b> crossing over.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a structure, such as a bridge, bearing a path for crossing over a river or highway.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>votes from members of one political party cast for candidates of another party; -- called also <altname>crossover vote</altname>; <as>as, there was a high <ex>crossover</ex> in this election</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a member of one political party who votes for a candidate of another party; -- called also <altname>crossover voter</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Plumbing)</fld> <def>a U-shaped section of pipe which serves to pass one pipeline over another which is directly in its path.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crosspatch</ent><br/
<hw>Cross"patch`</hw> <pr>(-p?ch`; 224)</pr>, <pos>n.</pos> <def>An ill-natured person.</def> <mark>[Colloq.]</mark> <ldquo/<xex>Crosspatch</xex>, draw the latch.<rdquo/  <rj><au>Mother Goose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-pawl</ent><br/
<hw>Cross"-pawl`</hw> <pr>(-p?l`)</pr>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <def>Same as <er>Cross-spale</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosspiece</ent><br/
<hw>Cross"piece`</hw> <pr>(kr?s"p?s`; 115)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A piece of any structure which is fitted or framed crosswise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A bar or timber connecting two knightheads or two bitts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-pollinate</ent><br/
<hw>cross-pollinate</hw> <pos>v.</pos> <def>to fertilize by transfering pollen.</def><br/
<syn><b>Syn. --</b> pollinate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-pollination</ent><br/
<hw>cross-pollination</hw> <pos>n.</pos> <def>fertilization by transfer of pollen from the anthers of one flower to the stigma of another.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-purpose</ent><br/
<hw>cross"-pur`pose</hw> <pr>(-p?r`p?s)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A counter or opposing purpose; a contrary aim; hence, that which is inconsistent or contradictory.</def>  <rj><au>Shaftesbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A conversational game, in which questions and answers are made so as to involve ludicrous combinations of ideas.</def>  <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be at cross-purposes</b></col>, <cd>to misunderstand or to act counter to one another without intending it; -- said of persons.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-question</ent><br/
<hw>Cross"-ques`tion</hw> <pr>(-kw?s`ch?n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cross-questioned</conjf> <pr>(-ch?nd)</pr>, <pos>p. pr. & vb. n.</pos> <conjf>Cross-questioning</conjf>.]</vmorph> <def>To cross-examine; to subject to close questioning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-question</ent><br/
<hw>cross-question</hw> <pos>n.</pos> <def>a question asked in cross-examination.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cross-reading</ent><br/
<hw>Cross"-read`ing</hw> <pr>(r?d`?ng)</pr>, <pos>n.</pos> <def>The reading of the lines of a newspaper directly across the page, instead of down the columns, thus producing a ludicrous combination of ideas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross-refer</ent><br/
<hw>cross-refer</hw> <pos>v.</pos> <def>to refer from one entry to another, as in catalogues, books, and lists.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cross-reference</ent><br/
<hw>cross-reference</hw> <pos>n.</pos> <def>a reference at one place in a work to information at another place in the same work.</def><br/
<syn><b>Syn. --</b> cross-index.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crossroad</ent><br/
<hw>Cross"road`</hw> <pr>(-r?d`)</pr>, <pos>n.</pos> <def>A road that crosses another; an obscure road intersecting or avoiding the main road.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossrow</ent><br/
<hw>Cross"row`</hw> <pr>(-r?`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The alphabet; -- called also <altname>Christcross-row</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And from the <qex>crossrow</qex> plucks the letter G.</q> <rj><qau> Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A row that crosses others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossruff</ent><br/
<hw>Cross"ruff`</hw> <pr>(-r?f`)</pr>, <pos>n.</pos> <fld>(Whist)</fld> <def>The play in bridge or whist where partners trump each a different suit, and lead to each other for that purpose; -- called also <altname>seesaw</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cross section</ent><br/
<hw>cross section</hw> <pos>n.</pos> <sn>1.</sn> <def>a flat plane cutting through a three-dimensional object, usually at right angles to the longest axis of the object.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>any visual representation of a <er>cross section{1}</er>, showing the internal structure of the object in the plane of the cross section; <as>as, the technician prepared a series of MRI <ex>cross sections</ex> of the skull</as>.</def> <note>Different <ex>cross sections</ex> created by different techniques may show different aspects of internal structure.  Thus computerized axial tomography using X-rays shows different structures than are visualized by MRI.</note><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>a thin slice of an object made by cutting it transversely; <as>as, to view a <ex>cross section</ex> of a bacterium with an electron microscope after staining the DNA; <ex>cross sections</ex> were prepared with a microtome</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a representative sample of a complex group; <as>as, the town contained a <ex>cross section</ex> of the American population</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Physics)</fld> <def>a measure of the probability that a nucleus will interact in a specified way with a bombarding particle, expressed as the effective area that the nucleus presents to the particle; -- called also <altname>nuclear cross section</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>cross section</ent><br/
<hw>cross section</hw> <pos>v. t.</pos> <def>to create one or a series of <er>cross sections{3}</er> by cutting (an object) into thin slices.</def><br/
[<source>PJC</source>]</p>

<p><ent>cross-sectional</ent><br/
<hw>cross-sectional</hw> <pos>adj.</pos> <def>of or pertaining to a cross section; <as>as, a <ex>cross-sectional</ex> slice</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cross-spall</ent><br/
<ent>Cross-spale</ent><br/
<mhw>{ <hw>Cross"-spale`</hw> <pr>(-sp?l`)</pr>, <hw>Cross"-spall`</hw> <pr>(-sp?l`)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Spale</er> & <er>Spall</er>.]</ety> <fld>(Shipbuilding)</fld> <def>One of the temporary wooden braces, placed horizontally across a frame to hold it in position until the deck beams are in; a cross-pawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-springer</ent><br/
<hw>Cross"-spring`er</hw> <pr>(-spr?ng`?r)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>One of the ribs in a groined arch, springing from the corners in a diagonal direction. <note>[See <xex>Illustr</xex>. of <er>Groined vault</er>.]</note></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-staff</ent><br/
<hw>Cross"-staff`</hw> <pr>(-st?f`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An instrument formerly used at sea for taking the altitudes of celestial bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A surveyor's instrument for measuring offsets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-stitch</ent><br/
<hw>Cross"-stitch`</hw> <pr>(-st?ch`; 224)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A form of stitch, where the stitches are diagonal and in pairs, the thread of one stitch crossing that of the other.</def> <ldquo/Tent and <xex>cross-stitch</xex>.<rdquo/ <au>Sir W. Scott.</au> -- <wordforms><wf>Cross"-stitch`</wf>, <pos>v. t. & i.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>embroidery done in cross-stitch.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cross-stone</ent><br/
<hw>Cross"-stone`</hw> <pr>(-st?n`)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Harmotome</er>, and <er>Staurotide</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-tail</ent><br/
<hw>Cross"-tail`</hw> <pr>(-t?l`)</pr>, <pos>n.</pos> <fld>(Steam Engine)</fld> <def>A bar connecting the ends of the side rods or levers of a backaction or side-lever engine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-tie</ent><br/
<hw>Cross"-tie`</hw> <pr>(-t?`)</pr>, <pos>n.</pos> <fld>(Railroad)</fld> <def>A sleeper supporting and connecting the rails, and holding them in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 349 --></p>

<p><ent>Cross-tining</ent><br/
<hw>Cross"-tin`ing</hw> <pr>(kr?s"t?n`?ng)</pr>, <pos>n.</pos> <fld>(Agric.)</fld> <def>A mode of harrowing crosswise, or transversely to the ridges.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosstrees</ent><br/
<hw>Cross"trees`</hw> <pr>(-tr?z`)</pr>, <pos>n. pl.</pos> <fld>(Naut.)</fld> <def>Pieces of timber at a masthead, to which are attached the upper shrouds.  At the head of lower masts in large vessels, they support a semicircular platform called the <ldquo/top.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-vaulting</ent><br/
<hw>Cross"-vault`ing</hw> <pr>(-v?lt`?ng)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>Vaulting formed by the intersection of two or more simple vaults.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crossway</ent><br/
<hw>Cross"way`</hw> <pr>(-w?`)</pr>, <pos>n.</pos> <def>See <er>Crossroad</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cross-week</ent><br/
<hw>Cross"-week`</hw> <pr>(-w?k`)</pr>, <pos>n.</pos> <def>Rogation week, when the cross was borne in processions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosswise</ent><br/
<hw>Cross"wise`</hw> <pr>(-w?z`)</pr>, <pos>adv.</pos> <def>In the form of a cross; across; transversely.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crosswort</ent><br/
<hw>Cross"wort`</hw> <pr>(-w?rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to several inconspicuous plants having leaves in whorls of four, as species of <xex>Crucianella</xex>, <xex>Valantia</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crotal</ent><br/
<hw>crotal</hw> <pos>n.</pos> <def>any of several lichens of the genus <gen>Parmelia</gen> from which reddish brown or purple dyes are made.</def><br/
<syn><b>Syn. --</b> crottle, crottal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crotalaria</ent><br/
||<hw>Crot`a*la"ri*a</hw> <pr>(kr?t`?-l?"r?-?  <or/ kr?`t?-l?"r?-A)</pr>, <pos>n.</pos> <ety>[NL. See <er>Crotalum</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of leguminous plants; rattlebox.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <spn>Crotalaria juncea</spn> furnishes the fiber called <xex>sunn</xex> or <xex>Bombay hemp</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotaline</ent><br/
<hw>Crot"a*line</hw> <pr>(kr?t"?-l?n <or/ kr?`t?-)</pr>, <pos>a.</pos> <ety>[See <er>Crotalus</er>.]</ety> <fld>(Zool.)</fld> <def>Resembling, or pertaining to, the <fam>Crotalidae</fam>, or Rattlesnake family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotalo</ent><br/
||<hw>Crot"a*lo</hw> <pr>(-l?)</pr>, <pos>n.</pos> <def>A Turkish musical instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotalum</ent><br/
||<hw>Crot"a*lum</hw> <pr>(-l?m)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/<?/ rattle.]</ety> <fld>(Mus.)</fld> <def>A kind of castanet used by the Corybantes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotalus</ent><br/
||<hw>Crot"a*lus</hw> <pr>(-l?s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ rattle.]</ety> <fld>(Zool.)</fld> <def>A genus of poisonous serpents, including the rattlesnakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotaphite</ent><br/
<hw>Crot"a*phite</hw> <pr>(kr?t"?-f?t)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ the side of the forehead.]</ety> <fld>(Anat.)</fld> <def>The temple or temporal fossa. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotaphitic</ent><br/
<hw>Crot`a*phit"ic</hw> <pr>(kr?t`?-f?t"?k)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>Pertaining to the temple; temporal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotaphytus</ent><br/
<hw>Crotaphytus</hw> <pos>n.</pos> <def>a genus of lizards of the family <fam>Iguanidae</fam>, including the collared lizards.</def><br/
<syn><b>Syn. --</b> genus <gen>Crotaphytus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crotch</ent><br/
<hw>Crotch</hw> <pr>(kr<ocr/ch; 224)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crotches</plw> <pr>(kr<ocr/ch"<ecr/z)</pr>.</plu> <ety>[Cf. <er>Crotchet</er>, <er>Crutch</er>.]</ety> <sn>1.</sn> <def>The angle formed by the parting of two legs or branches; a fork; the point where a trunk divides; <as>as, the <ex>crotch</ex> of a tree</as>.</def> <specif>more specifically,</specif> <def>the space on the human torso between the two legs; also, the corresponding part between the legs of a pair of pants, which is in contact with the crotch of the wearer; <as>as, pants with a tight <ex>crotch</ex> have become very popular</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A stanchion or post of wood or iron, with two arms for supporting a boom, spare yards, etc.; -- called also <altname>crane</altname> and <altname>crutch</altname>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Billiards)</fld> <def>In the three-ball carom game, a small space at each corner of the table. See <er>Crotched</er>, below.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crotch</ent><br/
<hw>Crotch</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crotched</conjf> <pr>(kr<ocr/cht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crotch"ing</conjf>.]</vmorph> <sn>1.</sn> <def>To provide with a crotch; to give the form of a crotch to; <as>as, to <ex>crotch</ex> the ends of ropes in splicing or tying knots</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Logging)</fld> <def>To notch (a log) on opposite sides to provide a grip for the dogs in hauling.</def> <mark>[Western, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crotch chain</ent><br/
<hw>Crotch chain</hw>. <fld>(Logging)</fld> <def>A form of tackle for loading a log sideways on a sled, skidway, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crotched</ent><br/
<hw>Crotched</hw> <pr>(kr<ocr/cht)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a crotch; forked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cross; peevish.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Billiards)</fld> <def>Lying within a crotch; -- said of the object balls in the three-ball carom game whenever the centers of both lie within a 4<frac12/-inch square at a corner of the table, in which case but three counts are allowed unless one or both balls be forced out of the crotch.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crotchet</ent><br/
<hw>Crotch"et</hw> <pr>(kr?ch"?t; 224)</pr>, <pos>n.</pos> <ety>[F. <ets>crochet</ets>, prop., a little hook, a dim. from the same source as <ets>croc</ets> hook. See <er>Crook</er>, and cf. <er>Crochet</er>, <er>Crocket</er>, <er>Crosier</er>.]</ety> <sn>1.</sn> <def>A forked support; a crotch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crotchets</qex> of their cot in columns rise.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A time note, with a stem, having one fourth the value of a semibreve, one half that of a minim, and twice that of a quaver; a quarter note.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fort.)</fld> <def>An indentation in the glacis of the covered way, at a point where a traverse is placed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>The arrangement of a body of troops, either forward or rearward, so as to form a line nearly perpendicular to the general line of battle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <def>A bracket. See <er>Bracket</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <def>An instrument of a hooked form, used in certain cases in the extraction of a fetus.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A perverse fancy; a whim which takes possession of the mind; a conceit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He ruined himself and all that trusted in him by <qex>crotchets</qex> that he could never explain to any rational man.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotchet</ent><br/
<hw>Crotch"et</hw>, <pos>v. i.</pos> <def>To play music in measured time.</def> <mark>[Obs.]</mark>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotcheted</ent><br/
<hw>Crotch"et*ed</hw>, <pos>a.</pos> <def>Marked or measured by crotchets; having musical notation.</def>  <rj><au>Harmar (1587).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotchetiness</ent><br/
<hw>Crotch"et*i*ness</hw> <pr>(kr?ch"?t-?-n?s)</pr>, <pos>n.</pos> <def>The state or character of being crotchety, or whimsical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This belief in rightness is a kind of conscientiousness, and when it degenerates it becomes <qex>crotchetiness</qex>.</q> <rj><qau>J. Grote.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotchety</ent><br/
<hw>Crotch"et*y</hw> <pr>(kr?ch"?t-?)</pr>, <pos>a.</pos> <def>Given to crotchets; subject to whims; <as>as, a <ex>crotchety</ex> man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croton</ent><br/
<hw>Cro"ton</hw> <pr>(kr?"t?n)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/, prop., a tick, which the seed of the croton resembles.]</ety> <fld>(Bot.)</fld> <def>A genus of euphorbiaceous plants belonging to tropical countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Croton oil</b></col> <fld>(Med.)</fld>, <cd>a viscid, acrid, brownish yellow oil obtained from the seeds of <spn>Croton Tiglium</spn>, a small tree of the East Indies. It is a most powerful drastic cathartic, and is used externally as a pustulant.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croton bug</ent><br/
<hw>Cro"ton bug`</hw> <pr>(b<ucr/g`)</pr>. <ety>[From the <ets>Croton</ets> water of New York.]</ety> <fld>(Zool.)</fld> <def>A small, active, winged species of cockroach (<spn>Ectobia Germanica</spn>), the water bug. It is common aboard ships, and in houses in cities, esp. in those with hot-water pipes; -- called also the <altname>German cockroach</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Crotonic</ent><br/
<hw>Cro*ton"ic</hw> <pr>(kr?-t?n"?k)</pr>, <pos>a.</pos> <def>Of or pertaining to, or derived from, a plant of the genus <gen>Croton</gen>, or from croton oil.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crotonic acid</b></col> <fld>(Chem.)</fld>, <cd>a white crystalline organic acid, <chform>C3H5.CO2H</chform>, of the ethylene, or acrylic acid series. It was so named because formerly supposed to exist in croton oil. Also, any acid metameric with crotonic acid proper.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The acid characteristic of croton oil is <xex>tiglic</xex> or <xex>tiglinic acid</xex>, a derivative of <cref>crotonic acid</cref>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotonine</ent><br/
<hw>Cro"ton*ine</hw> <pr>(kr?"t?n-?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A supposed alkaloid obtained from croton oil by boiling it with water and magnesia, since found to be merely a magnesia soap of the oil.</def>  <rj><au>Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotonylene</ent><br/
<hw>Cro*ton"y*lene</hw> <pr>(kr?-t?n"?-l?n)</pr>, <pos>n.</pos> <ety>[<ets>Crotonic</ets>  + acet-<ets>ylene</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless, volatile, pungent liquid, <chform>C4H6</chform>, produced artificially, and regarded as an unsaturated hydrocarbon of the acetylene series, and analogous to crotonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crotophaga</ent><br/
<hw>Crotophaga</hw> <pos>n.</pos> <def>a genus of birds including the anis.  See <er>ani</er>.</def><br/
<syn><b>Syn. --</b> genus <gen>Crotophaga</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crottal</ent><br/
<hw>crottal</hw> <pos>n.</pos> <def>any of several lichens of the genus Parmelia from which reddish brown or purple dyes are made.</def>  <altsp>[Also spelled <asp>crottle</asp> and <asp>crotal</asp>.]</altsp><br/
<syn><b>Syn. --</b> .</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>crottle</ent><br/
<hw>crottle</hw> <pos>n.</pos> <def>see <er>crottal</er>.</def><br/
<syn><b>Syn. --</b> crottal, crotal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crottles</ent><br/
<hw>Crot"tles</hw> <pr>(kr?t"t'lz)</pr>, <pos>n. pl.</pos> <ety>[Gael.  <ets>crotal</ets>.]</ety> <def>A name given to various lichens gathered for dyeing.  See <er>crottal</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouch</ent><br/
<hw>Crouch</hw> <pr>(krouch; 129)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crouched</conjf> <pr>(kroucht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crouching</conjf>.]</vmorph> <ety>[OE.  <ets>cruchen</ets>, <ets>crouchen</ets>, <ets>crouken</ets>; cf. E.  <ets>creep</ets>, G.  <ets>krauchen</ets>, <ets>kriechen</ets>, or E.  <ets>crook</ets> to bend, also <ets>crouch</ets> to cross.]</ety> <sn>1.</sn> <def>To bend down; to stoop low; to lie close to the ground with the logs bent, as an animal when waiting for prey, or in fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>crouch</qex> like a cur.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bend servilely; to stoop meanly; to fawn; to cringe.</def> <ldquo/A <xex>crouching</xex> purpose.<rdquo/  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Must I stand and <qex>crouch</qex><br/
Under your testy humor?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouch</ent><br/
<hw>Crouch</hw>, <pos>v. t.</pos> <ety>[OE.  <ets>cruchen</ets>, <ets>crouchen</ets>, from <ets>cruche</ets>, <ets>crouche</ets>, cross. Cf. <er>Crosier</er>, <er>Crook</er>.]</ety> <sn>1.</sn> <def>To sign with the cross; to bless.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bend, or cause to bend, as in humility or fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She folded her arms across her chest,<br/
And <qex>crouched</qex> her head upon her breast.</q> <rj><qau>Colerige.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouched</ent><br/
<hw>Crouched</hw> <pr>(kroucht)</pr>, <pos>a.</pos> <def>Marked with the sign of the cross.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crouched friar</b></col>. <cd>See <cref>Crutched friar</cref>, under <er>Crutched</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croud</ent><br/
<hw>Croud</hw> <pr>(kroud)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>See <er>Crowd</er>, a violin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouke</ent><br/
<hw>Crouke</hw> <pr>(krouk)</pr>, <pos>n.</pos> <def>A crock; a jar.</def> <mark>[Obs.]</mark>  <rj><au>Chauser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croup</ent><br/
<hw>Croup</hw> <pr>(kr<oomac/p)</pr>, <pos>n.</pos> <ety>[F.  <ets>croupe</ets> hind quarters, croup, rump, of German or Icel. origin; cf. Icel.  <ets>kryppa</ets> hump; akin to Icel.  <ets>kroppr</ets>.  Cf. <er>Crop</er>.]</ety> <def>The hinder part or buttocks of certain quadrupeds, especially of a horse; hence, the place behind the saddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So light to the <qex>croup</qex> the fair lady he swung,<br/
So light to the saddle before her he sprung.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croup</ent><br/
<hw>Croup</hw> <pr>(kr<oomac/p)</pr>, <pos>n.</pos> <ety>[Scot.  <ets>croup</ets>, cf.  <ets>croup</ets>, <ets>crowp</ets>, to croak, to cry or speak with a hoarse voice; cf. also LG. <ets>kropp</ets>, G.  <ets>kropf</ets>, the crop or craw of a bird, and tumor on the anterior part of the neck, a wen, etc. Cf. <er>Crop</er>.]</ety> <fld>(Med.)</fld> <def>An inflammatory affection of the larynx or trachea, accompanied by a hoarse, ringing cough and stridulous, difficult breathing; esp., such an affection when associated with the development of a false membrane in the air passages (also called <altname>membranous croup</altname>). See <cref>False croup</cref>, under <er>False</er>, and <er>Diphtheria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croupade</ent><br/
<hw>Crou*pade"</hw> <pr>(kr??-p?d")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>croupe</ets> hind quarters.]</ety> <fld>(Man.)</fld> <def>A leap in which the horse pulls up his hind legs toward his belly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croupal</ent><br/
<hw>Croup"al</hw> <pr>(kr??p"<it>a</it>l)</pr>, <pos>a.</pos> <def>Croupy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouper</ent><br/
<hw>Croup"er</hw> <pr>(kr<oomac/p"<etil/r)</pr>, <pos>n.</pos> <def>See <er>Crupper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croupier</ent><br/
<hw>Crou"pi*er</hw> <pr>(kr<oomac/"p<icr/*<etil/r)</pr>, <pos>n.</pos> <ety>[F.; prop., one who sits on the croup, and hence, in the second place; an assistant. See 1st <er>Croup</er>.]</ety> <sn>1.</sn> <def>One who presides at a gaming table and collects the stakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, at a public dinner party, sits at the lower end of the table as assistant chairman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croupous</ent><br/
<hw>Croup"ous</hw> <pr>(kr??p"?s)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Relating to or resembling croup; especially, attended with the formation of a deposit or membrane like that found in membranous croup; <as>as, <ex>croupous</ex> laryngitis</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Croupous pneumonia</b></col>, <cd>pneumonia attended with deposition of fibrinous matter in the air vesicles of the lungs; ordinary acute pneumonia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croupy</ent><br/
<hw>Croup"y</hw> <pr>(kr<oomac/p"<ycr/)</pr>, <pos>a.</pos> <def>Of or pertaining to croup; resembling or indicating croup; <as>as, a <ex>croupy</ex> cough</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouse</ent><br/
<hw>Crouse</hw> <pr>(kr<oomac/s)</pr>, <pos>a.</pos> <ety>[Etymol. uncertain.]</ety> <def>Brisk; lively; bold; self-complacent.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croustade</ent><br/
||<hw>Crou`stade"</hw> <pr>(kr??`st?d")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>cro<?/te</ets> a crust, OF.  <ets>crouste</ets>.]</ety> <fld>(Cookery)</fld> <def>Bread baked in a mold, and scooped out, to serve minces upon.</def>  <rj><au>Bishop.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crout</ent><br/
<hw>Crout</hw> <pr>(krout)</pr>, <pos>n.</pos> <ety>[G.  <ets>kraut</ets>.]</ety> <def>See <er>Sourkrout</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crouton</ent><br/
||<hw>Crou`ton"</hw> <pr>(kr??`t?n")</pr>, <pos>n.</pos> <ety>[F.  <ets>cro<?/ton</ets>, fr.  <ets>cro<?/te</ets> a crust.]</ety> <fld>(Cookery)</fld> <def>Bread cut in various forms, and fried lightly in butter or oil, to garnish hashes, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crow</ent><br/
<hw>Crow</hw> <pr>(kr<omac/)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Crew</conjf> <pr>(kr<udd/)</pr> or <conjf>Crowed</conjf> <pr>(kr<omac/d)</pr>; <pos>p. p.</pos> <conjf>Crowed</conjf> (<conjf>Crown</conjf> <pr>(kr<omac/n)</pr>, <mark>Obs.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Crowing</conjf>.]</vmorph> <ety>[AS. <ets>cr<amac/wan</ets>; akin to D. <ets>kraijen</ets>, G.  <ets>kr<aum/hen</ets>, cf. Lith.  <ets>groti</ets> to croak. <root/24. Cf.  <er>Crake</er>.]</ety> <sn>1.</sn> <def>To make the shrill sound characteristic of a cock, either in joy, gayety, or defiance.</def> <ldquo/The cock had <xex>crown</xex>.<rdquo/  <rj><au>Bayron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The morning cock <qex>crew</qex> loud.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shout in exultation or defiance; to brag.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter a sound expressive of joy or pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sweetest little maid,<br/
That ever <qex>crowed</qex> for kisses.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To crow over</b></col>, <cd>to exult over a vanquished antagonist.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Sennacherib <qex>crowing over</qex> poor Jerusalem.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crow</ent><br/
<hw>Crow</hw>, <pos>n.</pos> <ety>[AS. <ets>cr<amac/we</ets> a crow (in sense 1); akin to D.  <ets>kraai</ets>, G.  <ets>kr<aum/he</ets>; cf. Icel.  <ets>kr<amac/ka</ets> crow. So named from its cry, from AS.  <ets>cr<amac/wan</ets> to crow. See <er>Crow</er>, <pos>v. i.</pos> ]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Zool.)</fld> <def>A bird, usually black, of the genus <gen>Corvus</gen>, having a strong conical beak, with projecting bristles.  It has a harsh, croaking note. See <er>Caw</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common crow of Europe, or carrion crow, is <spn>Corvus corone</spn>.  The common American crow is <spn>Corvus Americanus</spn>. See <cref>Carrion crow</cref>, and <xex>Illustr.</xex>, under <er>Carrion</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bar of iron with a beak, crook, or claw; a bar of iron used as a lever; a crowbar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Get me an iron <qex>crow</qex>, and bring it straight<br/
Unto my cell.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The cry of the cock. See <er>Crow</er>, <pos>v. i.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The mesentery of a beast; -- so called by butchers.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Carrion crow</b></col>. <cd>See under <er>Carrion</er>.</cd> -- <col><b>Crow blackbird</b></col> <fld>(Zool.)</fld>, <cd>an American bird (<spn>Quiscalus quiscula</spn>); -- called also <altname>purple grackle</altname>.</cd> -- <col><b>Crow pheasant</b></col> <fld>(Zool.)</fld>, <cd>an Indian cuckoo; the common coucal. It is believed by the natives to give omens. See <er>Coucal</er>.</cd> -- <col><b>Crow shrike</b></col> <fld>(Zool.)</fld>, <cd>any bird of the genera <gen>Gymnorhina</gen>, <gen>Craticus</gen>, or <gen>Strepera</gen>, mostly from Australia.</cd> -- <col><b>Red-legged crow</b></col>. <cd>See <er>Crough</er>.</cd> -- <col><b>As the crow flies</b></col>, <cd>in a direct line.</cd> -- <mcol><col><b>To pick a crow</b></col>, <col><b>To pluck a crow</b></col></mcol>, <cd>to state and adjust a difference or grievance (with any one).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowbar</ent><br/
<hw>Crow"bar`</hw> <pr>(kr?"b?r)</pr>, <pos>n.</pos> <def>A bar of iron sharpened at one end, and used as a lever.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowberry</ent><br/
<hw>Crow`ber`ry</hw> <pr>(kr?"b?r`r?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A heathlike plant of the genus <gen>Empetrum</gen>, and its fruit, a black, scarcely edible berry; -- also called <altname>crakeberry</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crowing</ent><br/
<hw>crowing</hw> <pos>adj.</pos> <sn>1.</sn>  <def>same as <er>bragging</er>.</def><br/
<syn><b>Syn. --</b> boastful, braggart(prenominal), bragging(prenominal), braggy, big, cock-a-hoop, self-aggrandizing, vainglorious.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crowd</ent><br/
<hw>Crowd</hw> <pr>(kroud)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crowded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Crowding</conjf>.]</vmorph> <ety>[OE.  <ets>crouden</ets>, <ets>cruden</ets>, AS. <ets>cr<umac/dan</ets>; cf. D.  <ets>kruijen</ets> to push in a wheelbarrow.]</ety> <sn>1.</sn> <def>To push, to press, to shove.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To press or drive together; to mass together.</def> <ldquo/<xex>Crowd</xex> us and crush us.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fill by pressing or thronging together; hence, to encumber by excess of numbers or quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The balconies and verandas were <qex>crowded</qex> with spectators, anxious to behold their future sovereign.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To press by solicitation; to urge; to dun; hence, to treat discourteously or unreasonably.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To crowd out</b></col>, <cd>to press out; specifically, to prevent the publication of; as, the press of other matter <xex>crowded out</xex> the article.</cd> -- <col><b>To crowd sail</b></col> <fld>(Naut.)</fld>, <cd>to carry an extraordinary amount of sail, with a view to accelerate the speed of a vessel; to carry a press of sail.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowd</ent><br/
<hw>Crowd</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To press together or collect in numbers; to swarm; to throng.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole company <qex>crowded</qex> about the fire.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Images came <qex>crowding</qex> on his mind faster than he could put them into words.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To urge or press forward; to force one's self; <as>as, a man <ex>crowds</ex> into a room</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowd</ent><br/
<hw>Crowd</hw>, <pos>n.</pos> <ety>[AS.  <ets>croda</ets>. See <er>Crowd</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>A number of things collected or closely pressed together; also, a number of things adjacent to each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>crowd</qex> of islands.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A number of persons congregated or collected into a close body without order; a throng.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crowd</qex> of Vanity Fair.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Crowds</qex> that stream from yawning doors.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The lower orders of people; the populace; the vulgar; the rabble; the mob.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To fool the <qex>crowd</qex> with glorious lies.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He went not with the <qex>crowd</qex> to see a shrine.</q> <rj><qau>Dryden.</qau></rj></p>

<p><syn><b>Syn.</b> -- Throng; multitude. See <er>Throng</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowd</ent><br/
<hw>Crowd</hw>, <pos>n.</pos> <ety>[W. <ets>crwth</ets>; akin to Gael. <ets>cruit</ets>. Perh. named from its shape, and akin to Gr. <grk>kyrto`s</grk> curved, and E.  <ets>curve</ets>. Cf. <er>Rote</er>.]</ety> <def>An ancient instrument of music with six strings; a kind of violin, being the oldest known stringed instrument played with a bow.</def> <altsp>[Written also <asp>croud</asp>, <asp>crowth</asp>, <asp>cruth</asp>, and <asp>crwth</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>A lackey that . . . can warble upon a <qex>crowd</qex> a little.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowd</ent><br/
<hw>Crowd</hw>, <pos>v. t.</pos> <def>To play on a crowd; to fiddle.</def> <mark>[Obs.]</mark> <ldquo/Fiddlers, <xex>crowd</xex> on.<rdquo/  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crowded</ent><br/
<hw>crowded</hw> <pos>adj.</pos> <sn>1.</sn> <def>overfilled or compacted or concentrated; filled to excess; <as>as, a <ex>crowded</ex> program</as>.  Opposite of <ant>uncrowded</ant>.</def> <note>[Narrower terms: <stype>full, jammed, jam-packed, packed</stype>]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>filled with a crowd; <as>as, a <ex>crowded</ex> marketplace</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>having an uncomfortable density of people; filled to excess with people; <as>as, <ex>crowded</ex> trains</as>; a <ex>crowded</ex> theater.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crowder</ent><br/
<hw>Crowd"er</hw> <pr>(kroud"<etil/r)</pr>, <pos>n.</pos> <def>One who plays on a crowd; a fiddler.</def> <mark>[Obs.]</mark> <ldquo/Some blind <xex>crowder</xex>.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowder</ent><br/
<hw>Crowd"er</hw>, <pos>n.</pos> <def>One who crowds or pushes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowdy</ent><br/
<hw>Crow"dy</hw> <pr>(krou"d<ycr/)</pr>, <pos>n.</pos> <def>A thick gruel of oatmeal and milk or water; food of the porridge kind.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowflower</ent><br/
<hw>Crow"flow`er</hw> <pr>(kr?"flou`?r)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of campion; according to Gerarde, the <xex>Lychnis Flos-cuculi</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowfoot</ent><br/
<hw>Crow"foot`</hw> <pr>(kr?"f??t`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>The genus <gen>Ranunculus</gen>, of many species; some are common weeds, others are flowering plants of considerable beauty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A number of small cords rove through a long block, or euphroe, to suspend an awning by.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>A caltrop.</def> <altsp>[Written also <asp>crow's-foot</asp>.]</altsp> <br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Well Boring)</fld> <def>A tool with a side claw for recovering broken rods, etc.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowkeeper</ent><br/
<hw>Crow"keep`er</hw> <pr>(-k?p`?r)</pr>, <pos>n.</pos> <def>A person employed to scare off crows; hence, a scarecrow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Scaring the ladies like a <qex>crowkeeper</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown</ent><br/
<hw>Crown</hw> <pr>(kr?n)</pr>, <def><pos>p. p.</pos> of <er>Crow</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown</ent><br/
<hw>Crown</hw> <pr>(kroun)</pr>, <pos>n.</pos> <ety>[OE. <ets>corone</ets>, <ets>coroun</ets>, <ets>crune</ets>, <ets>croun</ets>, OF.  <ets>corone</ets>, <ets>corune</ets>, F.  <ets>couronne</ets>, fr. L.  <ets>corona</ets> crown, wreath; akin to Gr. <grk>korw`nh</grk> anything curved, crown; cf. also L.  <ets>curvus</ets> curved, E.  <ets>curve</ets>, <ets>curb</ets>, Gael.  <ets>cruinn</ets> round, W.  <ets>crwn</ets>. Cf. <er>Cornice</er>, <er>Corona</er>, <er>Coroner</er>, <er>Coronet</er>.]</ety> <sn>1.</sn> <def>A wreath or garland, or any ornamental fillet encircling the head, especially as a reward of victory or mark of honorable distinction; hence, anything given on account of, or obtained by, faithful or successful effort; a reward.</def> <ldquo/An olive branch and laurel <xex>crown</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They do it to obtain a corruptible <qex>crown</qex>; but we an incorruptible.</q> <rj><qau>1 Cor. ix. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be thou faithful unto death, and I will give thee a <qex>crown</qex> of life.</q> <rj><qau>Rev. ii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A royal headdress or cap of sovereignty, worn by emperors, kings, princes, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Nobles wear <xex>coronets</xex>; the triple <xex>crown</xex> of the pope is usually called a <xex>tiara</xex>. The <xex>crown</xex> of England is a circle of gold with crosses, fleurs-de-lis, and imperial arches, inclosing a crimson velvet cap, and ornamented with thousands of diamonds and precious stones.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The person entitled to wear a regal or imperial crown; the sovereign; -- with the definite article.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Parliament may be dissolved by the demise of the <qex>crown</qex>.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Large arrears of pay were due to the civil and military servants of the <qex>crown</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Imperial or regal power or dominion; sovereignty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a power behind the <qex>crown</qex> greater than the <qex>crown</qex> itself.</q> <rj><qau>Junius.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything which imparts beauty, splendor, honor, dignity, or finish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hoary head is a <qex>crown</qex> of glory, if it be found in the way of righteousness.</q> <rj><qau>Prov. xvi. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A virtuous woman is a <qex>crown</qex> to her husband.</q> <rj><qau>Prov. xvi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 350 --></p>

<p><sn>6.</sn> <def>Highest state; acme; consummation; perfection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mutual love, the <qex>crown</qex> of all our bliss.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The topmost part of anything; the summit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The steepy <qex>crown</qex> of the bare mountains.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The topmost part of the head (see <xex>Illust.</xex> of <er>Bird</er>.); that part of the head from which the hair descends toward the sides and back; also, the head or brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From toe to <qex>crown</qex> he'll fill our skin with pinches.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Twenty things which I set down:<br/
This done, I twenty more-had in my <qex>crown</qex>.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The part of a hat above the brim.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Anat.)</fld> <def>The part of a tooth which projects above the gum; also, the top or grinding surface of a tooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Arch.)</fld> <def>The vertex or top of an arch; -- applied generally to about one third of the curve, but in a pointed arch to the apex only.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Bot.)</fld> <def>Same as <er>Corona</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>That part of an anchor where the arms are joined to the shank.</def> <sd>(b)</sd> <def>The rounding, or rounded part, of the deck from a level line.</def> <sd>(c)</sd> <pluf>pl.</pluf> <def>The bights formed by the several turns of a cable.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>The upper range of facets in a rose diamond.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>The dome of a furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <fld>(Geom.)</fld> <def>The area inclosed between two concentric perimeters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <fld>(Eccl.)</fld> <def>A round spot shaved clean on the top of the head, as a mark of the clerical state; the tonsure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>A size of writing paper. See under <er>Paper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <def>A coin stamped with the image of a crown; hence,a denomination of money; <as>as, the English <ex>crown</ex>, a silver coin of the value of five shillings sterling, or a little more than $1</as>.20; the Danish or Norwegian <xex>crown</xex>, a money of account, etc., worth nearly twenty-seven cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>20.</sn> <def>An ornaments or decoration representing a crown; <as>as, the paper is stamped with a <ex>crown</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crown of aberration</b></col> <fld>(Astron.)</fld>, <cd>a spurious circle around the true circle of the sun.</cd> -- <col><b>Crown antler</b></col> <fld>(Zool.)</fld>, <cd>the topmost branch or tine of an antler; also, an antler having a cuplike top, with tines springing from the rim.</cd> -- <col><b>Crown bar</b></col>, <cd>one of the bars which support the crown sheet of steam-boiler furnace.</cd> -- <col><b>Crown glass</b></col>. <cd>See under <er>Glass</er>.</cd> -- <col><b>Crown imperial</b></col>. <fld>(Bot.)</fld> <cd>See in the Vocabulary.</cd> -- <col><b>Crown jewels</b></col>, <cd>the jewels appertaining to the sovereign while wearing the crown.</cd> <mark>[Eng.]</mark> <ldquo/She pawned and set to sale the <xex>crown jewels</xex>.<rdquo/ <au>Milton.</au> -- <col><b>Crown land</b></col>, <cd>land belonging to the crown, that is, to the sovereign.</cd> -- <col><b>Crown law</b></col>, <cd>the law which governs criminal prosecutions.</cd> <mark>[Eng.]</mark> -- <col><b>Crown lawyer</b></col>, <cd>one employed by the crown, as in criminal cases.</cd> <mark>[Eng.]</mark> -- <col><b>Crown octavo</b></col>. <cd>See under <er>Paper</er>.</cd> -- <col><b>Crown office</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Crown paper</b></col>.  <cd>See under <er>Paper</er>.</cd> -- <col><b>Crown piece</b></col>.  <cd>See in the Vocabulary.</cd> -- <col><b>Crown Prince</b></col>, <cd>the heir apparent to a crown or throne.</cd> -- <col><b>Crown saw</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Crown scab</b></col> <fld>(Far.)</fld>, <cd>a cancerous sore formed round the corners of a horse's hoof.</cd> -- <col><b>Crown sheet</b></col>, <cd>the flat plate which forms the top of the furnace or fire box of an internally fired steam boiler.</cd> -- <col><b>Crown shell</b></col>.  <fld>(Zool.)</fld> <cd>See <er>Acorn-shell</er>.</cd> -- <col><b>Crown side</b></col>. See <er>Crown office</er>. -- <col><b>Crown tax</b></col> <fld>(Eccl. Hist.)</fld>, <cd>a golden crown, or its value, which was required annually from the Jews by the king of Syria, in the time of the Maccabees.</cd> <au>1 Macc. x. 20.</au> -- <col><b>Crown wheel</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Crown work</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Pleas of the crown</b></col> <fld>(Engl. law)</fld>, <cd>criminal actions.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown</ent><br/
<hw>Crown</hw> <pr>(kroun)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crowned</conjf> <pr>(kround)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crowning</conjf>.]</vmorph> <ety>[OE. <ets>coronen</ets>, <ets>corunen</ets>, <ets>crunien</ets>, <ets>crounien</ets>, OF.  <ets>coroner</ets>, F.  <ets>couronner</ets>, fr. L.  <ets>coronare</ets>, fr. <ets>corona</ets> a crown. See <er>Crown</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cover, decorate, or invest with a crown; hence, to invest with royal dignity and power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her who fairest does appear,<br/
<qex>Crown</qex> her queen of all the year.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Crown</qex> him, and say, <ldquo/Long live our emperor.<rdquo/</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bestow something upon as a mark of honor, dignity, or recompense; to adorn; to dignify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou . . . hast <qex>crowned</qex> him with glory and honor.</q> <rj><qau>Ps. viii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To form the topmost or finishing part of; to complete; to consummate; to perfect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Amidst the grove that <qex>crowns</qex> yon tufted hill.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One day shall <qex>crown</qex> the alliance.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>crown</qex> the whole, came a proposition.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech.)</fld> <def>To cause to round upward; to make anything higher at the middle than at the edges, as the face of a machine pulley.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mil.)</fld> <def>To effect a lodgment upon, as upon the crest of the glacis, or the summit of the breach.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To crown a knot</b></col> <fld>(Naut.)</fld>, <cd>to lay the ends of the strands over and under each other.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>crownbeard</ent><br/
<hw>crownbeard</hw> <pos>n.</pos> <def>any plant of the genus <gen>Verbesina</gen> having clustered white or yellow flower heads.</def><br/
<syn><b>Syn. --</b> crown beard.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crown colony</ent><br/
<hw>Crown colony</hw>. <def>A colony of the British Empire not having an elective magistracy or a parliament, but governed by a chief magistrate (called Governor) appointed by the Crown, with executive councilors nominated by him and not elected by the people.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crowned</ent><br/
<hw>Crowned</hw> <pr>(kround)</pr>, <pos>p. p. & a.</pos> <sn>1.</sn> <def>Having or wearing a crown; surmounted, invested, or adorned, with a crown, wreath, garland, etc.; honored; rewarded; completed; consummated; perfected.</def> <ldquo/<xex>Crowned</xex> with one crest.<rdquo/ <au>Shak.</au> <ldquo/<xex>Crowned</xex> with conquest.<rdquo/ <au>Milton.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>With surpassing glory <qex>crowned</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Great; excessive; supreme.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowner</ent><br/
<hw>Crown"er</hw> <pr>(kroun"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, crowns.</def>  <rj><au>Beau. & FL.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. <er>Coroner</er>.]</ety> <def>A coroner.</def> <mark>[Prov. Eng. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crownet</ent><br/
<hw>Crown"et</hw> <pr>(kroun"?t)</pr>, <pos>n.</pos> <ety>[See <er>Crown</er>, <er>Coronet</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A coronet.</def> <mark>[R.]</mark>  <rj><au>P. Whitehead.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The ultimate end and result of an undertaking; a chief end.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O this false soul of Egypt! this grave charm . . . .<br/
Whose bosom was my <qex>crownet</qex>, my chief end.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown-imperial</ent><br/
<hw>Crown"-im*pe"ri*al</hw> <pr>(-?m-p?"r?-<it>a</it>l)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A spring-blooming plant (<spn>Fritillaria imperialis</spn>) of the Lily family, having at the top of the stalk a cluster of pendent bell-shaped flowers surmounted with a tuft of green leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crownland</ent><br/
<hw>Crown"land`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>kronland</ets>.]</ety> <def>In Austria-Hungary, one of the provinces, or largest administrative divisions of the monarchy; <as>as, the <ex>crownland</ex> of Lower Austria</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Crownless</ent><br/
<hw>Crown"less</hw>, <pos>a.</pos> <def>Without a crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crownlet</ent><br/
<hw>Crown"let</hw> <pr>(-l?t)</pr>, <pos>n.</pos> <def>A coronet.</def> <mark>[Poetic]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown office</ent><br/
<hw>Crown" of`fice</hw> <pr>(?f`f?s; 115)</pr>. <fld>(Eng. Law)</fld> <def>The criminal branch of the Court of King's or Queen's Bench, commonly called the <xex>crown side</xex> of the court, which takes cognizance of all criminal cases.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crown-of-the-field</ent><br/
<hw>crown-of-the-field</hw> <pos>n.</pos> <def>a European annual (<spn>Agrostemma githago</spn>) having large trumpet-shaped reddish-purple flowers and poisonous seed; a common weed in grainfields and beside roadways; naturalized in America.</def><br/
<syn><b>Syn. --</b> corn cockle, corncockle, corn-cockle, corn campion, <spn>Agrostemma githago</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crownpiece</ent><br/
<hw>Crown"piece`</hw> <pr>(-p?s`)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>A piece or part which passes over the head, as in a bridle.</def> <sd>(b)</sd> <def>A coin [In sense <sd>(b)</sd> properly <xex>crown piece</xex>.] See <er>Crown</er>, 19.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown-post</ent><br/
<hw>Crown"-post`</hw> <pr>(kroun"p?st`)</pr>, <pos>n.</pos> <def>Same as <er>King-post</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown-saw</ent><br/
<hw>Crown"-saw`</hw> <pr>(-s?`)</pr>, <pos>n.</pos> <ety>[From its supposed resemblance to a <ets>crown</ets>.]</ety> <fld>(Mech.)</fld> <def>A saw in the form of a hollow cylinder, with teeth on the end or edge, and operated by a rotative motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The trephine was the first of the class of crownsaws.</note>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown side</ent><br/
<hw>Crown" side`</hw> <pr>(s?d`)</pr>. <def>See <er>Crown office</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crown wheel</ent><br/
<hw>Crown" wheel`</hw> <pr>(hw?l`)</pr>.  <ety>[Named from its resemblance to a <ets>crown</ets>.]</ety> <fld>(Mach.)</fld> <def>A wheel with cogs or teeth set at right angles to its plane; -- called also a <altname>contrate wheel</altname> or <altname>face wheel</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crownwork</ent><br/
<hw>Crown"work`</hw> <pr>(-w?rk`)</pr>, <pos>n.</pos> <fld>(Fort.)</fld> <def>A work consisting of two or more bastioned fronts, with their outworks, covering an enceinte, a bridgehead, etc., and connected by wings with the main work or the river bank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crow-quill</ent><br/
<hw>Crow"-quill`</hw> <pr>(kr?"kw?l`)</pr>, <pos>n.</pos> <def>A quill of the crow, or a very fine pen made from such a quill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crows</ent><br/
<hw>Crows</hw> <pr>(kr<omac/z)</pr>, <pos>n. pl.</pos>; <sing><it>sing.</it> <singw>Crow</singw></sing>.  <fld>(Ethnol.)</fld> <def>A tribe of Indians of the Dakota stock, living in Montana; -- also called <altname>Upsarokas</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crow's-foot</ent><br/
<hw>Crow's"-foot`</hw> <pr>(kr?z"f??t`)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crow's-feet</plw> <pr>(-f<emac/t`)</pr>.</plu><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <pluf>pl.</pluf> <def>The wrinkles that appear, as the effect of age or dissipation, under and around the outer corners of the eyes.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A caltrop.</def> <altsp>[Written also <asp>crowfoot</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>Same as <er>Bird's-mouth</er>.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crow-silk</ent><br/
<hw>Crow"-silk`</hw> <pr>(kr?"s?lk`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A filamentous fresh-water alga (<spn>Conferva rivularis</spn> of Linnaeus, <spn>Rhizoclonium rivulare</spn> of Kutzing).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crow's nest</ent><br/
<ent>crow's-nest</ent><br/
<mhw><hw>crow's"-nest`</hw>, <hw>crow's" nest`</hw></mhw> <pr>(kr<omac/z"n<ecr/st`)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A box or platform near the top of a mast, esp. in whalers, to shelter the man on the lookout.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowstep</ent><br/
<hw>Crow"step`</hw> <pr>(kr?"st?p`)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>See <er>Corriestep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowstone</ent><br/
<hw>Crow"stone`</hw> <pr>(kr?"st?n`)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The top stone of the gable end of a house.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowth</ent><br/
<hw>Crowth</hw> <pr>(krouth)</pr>, <pos>n.</pos> <def>An ancient musical instrument. See 4th <er>Crowd</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crowtoe</ent><br/
<hw>Crow"toe`</hw> <pr>(kr<omac/"t<omac/`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sn>1.</sn> <def>The <spn>Lotus corniculatus</spn>.</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An unidentified plant, probably the crowfoot.</def> <ldquo/The tufted <xex>crowtoe</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crow-trodden</ent><br/
<hw>Crow"-trod`den</hw> <pr>(kr<omac/"tr?d`d'n)</pr>, <pos>a.</pos> <def>Marked with crow's-feet, or wrinkles, about the eyes.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Do I look as if I were <qex>crow-trodden</qex>?</q> <rj><qau>Beau. & FL.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croydon</ent><br/
<hw>Croy"don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Croydon</ets>, England.]</ety> <sn>1.</sn>  <def>A kind of carriage like a gig, orig. of wicker-work.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of cotton sheeting; also, a calico.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Croylstone</ent><br/
<hw>Croyl"stone`</hw> <pr>(kroil"st<omac/n`)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Crystallized cawk, in which the crystals are small.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croys</ent><br/
<hw>Croys</hw> <pr>(krois)</pr>, <pos>n.</pos> <def>See <er>Cross</er>, <pos>n.</pos></def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croze</ent><br/
<hw>Croze</hw> <pr>(kr?z)</pr>, <pos>n.</pos> <ety>[Cf. <er>Cross</er>, and <er>Crosier</er>.]</ety> <def>A cooper's tool for making the grooves for the heads of casks, etc.; also, the groove itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crozier</ent><br/
<hw>Cro"zier</hw> <pr>(kr?"zh?r)</pr>, <pos>n.</pos> <def>See <er>Crosier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Croziered</ent><br/
<hw>Cro"ziered</hw> <pr>(-zh?rd)</pr>, <pos>a.</pos> <def>Crosiered.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucial</ent><br/
<hw>Cru"cial</hw> <pr>(kr?"sh<it>a</it>l)</pr>, <pos>a.</pos> <ety>[F. <ets>crucial</ets>, fr. L. <ets>crux</ets>, <ets>crucis</ets>, cross, torture. See <er>Cross</er>.]</ety> <sn>1.</sn> <def>Having the form of a cross; appertaining to a cross; cruciform; intersecting; <as>as, <ex>crucial</ex> ligaments; a <ex>crucial</ex> incision</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Severe; trying or searching, as if bringing to the cross; decisive; <as>as, a <ex>crucial</ex> test</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucian carp</ent><br/
<hw>Cru"cian carp`</hw> <pr>(-sh<it>a</it>n k?rp`)</pr>. <ety>[Cf. Sw. <ets>karussa</ets>, G. <ets>karausche</ets>, F.  <ets>carousse</ets>, <ets>-assin</ets>, <ets>corassin</ets>, LL.  <ets>coracinus</ets>, Gr. <?/<?/<?/<?/<?/  a sort of fish.]</ety> <fld>(Zool.)</fld> <def>A kind of European carp (<spn>Carasius vulgaris</spn>), inferior to the common carp; -- called also <altname>German carp</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The gibel or Prussian carp is now generally considered a variety of the crucian carp, or perhaps a hybrid between it and the common carp.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruciate</ent><br/
<hw>Cru"ci*ate</hw> <pr>(kr?"sh?-?t <or/ -sh?t; 106)</pr>, <pos>a.</pos> <ety>[L. <ets>cruciatus</ets>, p. p. of <ets>cruciare</ets> to crucify, torture, fr. <ets>crux</ets>, <ets>crucis</ets>, a cross. See <er>Cross</er>.]</ety> <sn>1.</sn> <def>Tormented.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having the leaves or petals arranged in the form of a cross; cruciform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruciate</ent><br/
<hw>Cru"ci*ate</hw> <pr>(kr?"sh?-?t)</pr>, <pos>v. t.</pos> <def>To torture; to torment. <mark>[Obs.]</mark> See <er>Excruciate</er>.</def>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruciation</ent><br/
<hw>Cru`ci*a"tion</hw> <pr>(kr?`sh?-?"sh?n)</pr>, <pos>n.</pos> <ety>[LL. <ets>cruciatio</ets>.]</ety> <def>The act of torturing; torture; torment.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucible</ent><br/
<hw>Cru"ci*ble</hw> <pr>(kr<udd/"s<icr/*b'l)</pr>, <pos>n.</pos> <ety>[LL. <ets>crucibulum</ets> a hanging lamp, an earthen pot for melting metals (cf. OF. <ets>croisel</ets>, <ets>creuseul</ets>, sort of lamp, crucible, F. <ets>creuset</ets> crucible), prob. of German origin; cf. OHG.  <ets>kr<umac/sul</ets>, LG. <ets>kr<uum/sel</ets>, hanging lamp, <ets>kroos</ets>, <ets>kruus</ets>, mug, jug, jar, D. <ets>kroes</ets> cup, crucible, Dan.  <ets>kruus</ets>, Sw.  <ets>krus</ets>, E.  <ets>cruse</ets>.  It was confused with derivatives of L.  <ets>crux</ets> cross (cf. <er>Crosslet</er>), and crucibles were said to have been marked with a cross, to prevent the devil from marring the chemical operation. See <er>Cruse</er>, and cf. <er>Cresset</er>.]</ety> <sn>1.</sn> <def>A vessel or melting pot, composed of some very refractory substance, as clay, graphite, platinum, and used for melting and calcining substances which require a strong degree of heat, as metals, ores, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hollow place at the bottom of a furnace, to receive the melted metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A test of the most decisive kind; a severe trial; <as>as, the <ex>crucible</ex> of affliction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hessian crucible</b></col> <fld>(Chem.)</fld>, <cd>a cheap, brittle, and fragile, but very refractory crucible, composed of the finest fire clay and sand, and commonly used for a single heating; -- named from the place of manufacture.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucible steel</ent><br/
<hw>Cru"ci*ble steel</hw>. <def>Cast steel made by fusing in crucibles crude or scrap steel, wrought iron, and other ingredients and fluxes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>crucifer</ent><br/
<hw>cru"ci*fer</hw> <pr>(-f?r)</pr>, <pos>n.</pos> <ety>[See <er>Cruciferous</er>.]</ety> <fld>(Bot.)</fld> <def>Any plant of the family <fam>Crucifer<ae/</fam>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cruciferae</ent><br/
<hw>Cruciferae</hw> <pos>n.</pos> <def>a natural family of plants with four-petaled flowers; the mustard family.</def><br/
<syn><b>Syn. --</b> family <fam>Cruciferae</fam>, Brassicaceae, family <fam>Brassicaceae</fam>, mustard family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cruciferous</ent><br/
<hw>Cru*cif"er*ous</hw> <pr>(kr<usdot/*s?f"?r-?s)</pr>, <pos>a.</pos> <ety>[L. <ets>crux</ets>, <ets>crucis</ets>, cross + <ets>-ferous</ets>: cf. F. <ets>crucif<egrave/re</ets>.]</ety> <sn>1.</sn> <def>Bearing a cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Of, pertaining to, or resembling, a family of plants which have four petals arranged like the arms of a cross, as the mustard, radish, turnip, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucifier</ent><br/
<hw>Cru"ci*fi`er</hw> <pr>(kr?"s?-f?`?r)</pr>, <pos>n.</pos> <def>One who crucifies; one who subjects himself or another to a painful trial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucifix</ent><br/
<hw>Cru"ci*fix</hw> <pr>(kr?"s?-f?ks)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crucifixes</plw> <pr>(-<?/z)</pr>.</plu> <ety>[F. <ets>crucifix or LL</ets>. <ets>crucifixum</ets>, fr. L.  <ets>crux</ets>, <ets>crucis</ets>, cross +  <ets>figere</ets>, <ets>fixum</ets>, to fix. See <er>Cross</er>, and <er>Fix</er>, and cf. Crucify.]</ety> <sn>1.</sn> <def>A representation in art of the figure of Christ upon the cross; esp., the sculptured figure affixed to a real cross of wood, ivory, metal, or the like, used by the Roman Catholics in their devotions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cross, too, by degrees, become the <qex>crucifix</qex>.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And kissing oft her <qex>crucifix</qex>,<br/
Unto the block she drew.</q> <rj><qau>Warner.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cross or religion of Christ.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucifixion</ent><br/
<hw>Cru`ci*fix"ion</hw> <pr>(kr?`s?-f?k"sh?n)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of nailing or fastening a person to a cross, for the purpose of putting him to death; the use of the cross as a method of capital punishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of one who is nailed or fastened to a cross; death upon a cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Intense suffering or affliction; painful trial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do ye prove<br/
What <qex>crucifixions</qex> are in love?</q> <rj><qau>Herrick.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruciform</ent><br/
<hw>Cru"ci*form</hw> <pr>(kr?"s?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>crux</ets>, <ets>crucis</ets>, cress + <ets>-form</ets>: cf. F.  <ets>cruciforme</ets>.]</ety> <def>Cross-shaped; <fld>(Bot.)</fld> having four parts arranged in the form of a cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucify</ent><br/
<hw>Cru"ci*fy</hw> <pr>(-f?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crucified</conjf> <pr>(-f?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crucifying</conjf>.]</vmorph> <ety>[F. <ets>crucifier</ets>, fr. (assumed) LL.  <ets>crucificare</ets>, for <ets>crucifigere</ets>, fr, L. <ets>crux</ets>, <ets>crucis</ets>, cross + <ets>figere</ets> to fix, the ending <ets>-figere</ets> being changed to <ets>-ficare</ets>, F. <ets>-fier</ets> (in compounds), as if fr. L.  <ets>facere</ets> to do, make. See <er>Cross</er>, and <er>Fix</er>, and cf. <er>Crucifix</er>.]</ety> <sn>1.</sn> <def>To fasten to a cross; to put to death by nailing the hands and feet to a cross or gibbet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They cried, saying, <qex>Crucify</qex> him, <qex>cricify</qex> him.</q> <rj><qau>Luke xxiii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To destroy the power or ruling influence of; to subdue completely; to mortify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They that are Christ's have <qex>crucified</qex> the flesh, with the affections and lusts.</q> <rj><qau>Gal. v. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To vex or torment.</def>  <rj><au>Beau. & FL.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crucigerous</ent><br/
<hw>Cru*cig"er*ous</hw> <pr>(kr?-s?j"?r-?s)</pr>, <pos>a.</pos> <ety>[L. <ets>crux</ets>, <ets>cricis</ets>, cross + <ets>-gerous</ets>.]</ety> <def>Bearing the cross; marked with the figure of a cross.</def>  <rj><au>Sir. T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crud</ent><br/
<hw>Crud</hw> <pr>(kr?d)</pr>, <pos>n.</pos> <def>See <er>Curd</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruddle</ent><br/
<hw>Crud"dle</hw> <pr>(-d'l)</pr>, <pos>v. i.</pos> <def>To curdle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>See how thy blood <qex>cruddles</qex> at this.</q> <rj><qau>Bea<?/ & FL.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crude</ent><br/
<hw>Crude</hw> <pr>(kr<udd/d)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Cruder</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Crudest</adjf>.]</amorph> <ety>[L. <ets>crudus</ets> raw; akin to <ets>cruor</ets> blood (which flows from a wound). See <er>Raw</er>, and cf. <er>Cruel</er>.]</ety> <sn>1.</sn> <def>In its natural state; not cooked or prepared by fire or heat; undressed; not altered, refined, or prepared for use by any artificial process; raw; <as>as, <ex>crude</ex> flesh</as>.</def> <ldquo/Common <xex>crude</xex> salt.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Molding to its will each successive deposit of the <qex>crude</qex> materials.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unripe; not mature or perfect; immature.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I come to pluck your berries harsh and <qex>crude</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not reduced to order or form; unfinished; not arranged or prepared; ill-considered; immature.</def> <ldquo/<xex>Crude</xex> projects.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Crude</qex>, undigested masses of suggestion, furnishing rather raw materials for composition.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The originals of Nature in their <qex>crude</qex><br/
Conception.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Undigested; unconcocted; not brought into a form to give nourishment.</def> <ldquo/<xex>Crude</xex> and inconcoct.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Having, or displaying, superficial and undigested knowledge; without culture or profundity; <as>as, a <ex>crude</ex> reasoner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Paint.)</fld> <def>Harsh and offensive, as a color; tawdry or in bad taste, as a combination of colors, or any design or work of art.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crudely</ent><br/
<hw>Crude"ly</hw>, <pos>adv.</pos> <def>In a crude, immature manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crudeness</ent><br/
<hw>Crude"ness</hw>, <pos>n.</pos> <def>A crude, undigested, or unprepared state; rawness; unripeness; immatureness; unfitness for a destined use or purpose; <as>as, the <ex>crudeness</ex> of iron ore; <ex>crudeness</ex> of theories or plans</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crudites</ent><br/
<hw>crudites</hw> <pos>n.</pos> <fld>(Cookery)</fld> <def>an appetizer consisting of raw vegetables cut into bite-sized strips and served with a dip.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crudity</ent><br/
<hw>Cru"di*ty</hw> <pr>(kr<udd/"d<icr/*t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crudities</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[L. <ets>cruditas</ets>, fr.  <ets>crudus</ets>: cf. F.  <ets>crudit<eacute/</ets>. See <er>Crude</er>.]</ety> <sn>1.</sn> <def>The condition of being crude; rawness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is in a crude or undigested state; hence, superficial, undigested views, not reduced to order or form.</def> <ldquo/<xex>Crudities</xex> in the stomach.<rdquo/  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crudle</ent><br/
<hw>Cru"dle</hw> <pr>(-d'l)</pr>, <pos>v. i.</pos> <def>See <er>Cruddle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crudy</ent><br/
<hw>Crud"y</hw> <pr>(kr?d"?)</pr>, <pos>a.</pos> <ety>[From <er>Crud</er>.]</ety> <def>Coagulated.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His cruel wounds with <qex>crudy</qex> blood congealed.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crudy</ent><br/
<hw>Cru"dy</hw> <pr>(kr?"d?)</pr>, <pos>a.</pos> <ety>[From <er>Crude</er>.]</ety> <def>Characterized by crudeness; raw.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The foolish and dull and <qex>crudy vapors</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruel</ent><br/
<hw>Cru"el</hw> <pr>(kr<udd/"<ecr/l)</pr>, <pos>n.</pos> <def>See <er>Crewel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruel</ent><br/
<hw>Cru"el</hw> <pr>(kr<udd/"<ecr/l)</pr>, <pos>a.</pos> <ety>[F.  <ets>cruel</ets>, fr. L.  <ets>crudelis</ets>, fr. <ets>crudus</ets>. See <er>Crude</er>.]</ety> <sn>1.</sn> <def>Disposed to give pain to others; willing or pleased to hurt, torment, or afflict; destitute of sympathetic kindness and pity; savage; inhuman; hard-hearted; merciless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold a people cometh from the north country; . . . they are <qex>cruel</qex> and have no mercy.</q> <rj><qau>Jer. vi. 22,23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Causing, or fitted to cause, pain, grief, or misery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cruel</qex> wars, wasting the earth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cursed be their anger, for it was fierce; and their wrath for it was <qex>cruel</qex>.</q> <rj><qau>Gen. xlix. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Attended with cruetly; painful; harsh.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have seen <qex>cruel</qex> proof of this man's strength.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruelly</ent><br/
<hw>Cru"el*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a cruel manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Extremely; very.</def> <mark>[Colloq.]</mark>  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruelness</ent><br/
<hw>Cru"el*ness</hw>, <pos>n.</pos><def>Cruelty.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruels</ent><br/
<hw>Cru"els</hw> <pr>(kr<udd/"<ecr/lz)</pr>, <pos>n. pl.</pos> <ety>[Corrupt. fr. F. <ets><eacute/crouelles</ets> scrofula.]</ety> <def>Glandular scrofulous swellings in the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruelty</ent><br/
<hw>Cru"el*ty</hw> <pr>(-t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cruelties</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[OF. <ets>cruelt<eacute/</ets>, F.  <ets>cruaut<eacute/</ets>, fr. L.  <ets>crudelitas</ets>, fr.  <ets>crudelis</ets>. See <er>Cruel</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 351 --></p>

<p><sn>1.</sn> <def>The attribute or quality of being cruel; a disposition to give unnecessary pain or suffering to others; inhumanity; barbarity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pierced through the heart with your stern <qex>cruelty</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cruel and barbarous deed; inhuman treatment; the act of willfully causing unnecessary pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cruelties</qex> worthy of the dungeons of the Inquisition.</q>
<q>Macaulay.</q><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruentate</ent><br/
<hw>Cru"en*tate</hw> <pr>(kr?"?n-t?t)</pr>, <pos>a.</pos> <ety>[L.  <ets>cruentatus</ets>, p. p. of <ets>cruentare</ets> to make bloody, fr.  <ets>cruentus</ets> bloody, fr.  <ets>cruor</ets>. See <er>Crude</er>.]</ety> <def>Smeared with blood.</def> <mark>[Obs.]</mark>  <rj><au>Glanwill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruentous</ent><br/
<hw>Cru*en"tous</hw> <pr>(kr?-?n"t?s)</pr>, <pos>a.</pos> <ety>[L.  <ets>cruentus</ets>.]</ety> <def>Bloody; cruentate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruet</ent><br/
<hw>Cru"et</hw> <pr>(kr<udd/"<ecr/t)</pr>, <pos>n.</pos> <ety>[Anglo-French <ets>cruet</ets>, a dim. from OF. <ets>crue</ets>, <ets>cruie</ets>; of German or Celtic origin, and akin to E.  <ets>crock</ets> an earthen vessel.]</ety> <sn>1.</sn> <def>A bottle or vessel; esp., a vial or small glass bottle for holding vinegar, oil, pepper, or the like, for the table; a caster.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A vessel used to hold wine, oil, or water for the service of the altar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cruet stand</ent><br/
<ent>cruet-stand</ent><br/
<mhw><hw>cruet-stand</hw>, <hw>cruet stand</hw></mhw> <pos>n.</pos> <def>a frame for holding cruets; a caster; a stand for cruets together with the cruets containing various condiments.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cruise</ent><br/
<hw>Cruise</hw> <pr>(kr<udd/s)</pr>, <pos>n.</pos> <def>See <er>Cruse</er>, a small bottle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruise</ent><br/
<hw>Cruise</hw> <pr>(kr<udd/z)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cruised</conjf> <pr>(kr<udd/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cruising</conjf>.]</vmorph> <ety>[D.  <ets>kruisen</ets> to move crosswise or in a zigzag, to cruise, fr. <ets>kruis</ets> cross, fr. OF.  <ets>crois</ets>, <ets>croiz</ets>, F. <ets>croix</ets>, or directly fr. OF.  <ets>croisier</ets>, F. <ets>croiser</ets>, to cross, cruise, fr.  <ets>crois</ets> a cross. See <er>Cross</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To sail back and forth on the ocean; to sail, as for the protection of commerce, in search of an enemy, for plunder, or for pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A ship cruises <xex>in</xex> any particular sea or ocean; as, <xex>in</xex> the Baltic or <xex>in</xex> the Atlantic.  She cruises <xex>off</xex> any cape; as, <xex>off</xex> the Lizard; <xex>off</xex> Ushant.  She cruises <xex>on</xex> a coast; as, <xex>on</xex> the coast of Africa.  A pirate <xex>cruises</xex> to seize vessels; a yacht <xex>cruises</xex> for the pleasure of the owner.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Ships of war were sent to <qex>cruise</qex> near the isle of Bute.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'Mid sands, and rocks, and storms to <qex>cruise</qex> for pleasure.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wander hither and thither on land.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Forestry)</fld> <def>To inspect forest land for the purpose of estimating the quantity of lumber it will yield.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <def>To travel primarily for pleasure, or without any fixed purpose, rather than with the main goal of reaching a particular destination.</def>  <illu>To <ex>cruise</ex> the streets of town, looking for an interesting party to crash.</illu><br/
[<source>PJC</source>]</p>

<p><ent>Cruise</ent><br/
<hw>Cruise</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cruise over or about.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Forestry)</fld> <def>To explore with reference to capacity for the production of lumber; <as>as, to <ex>cruise</ex> a section of land</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cruise</ent><br/
<hw>Cruise</hw>, <pos>n.</pos> <sn>1.</sn> <def>A voyage made in various directions, as of an armed vessel, for the protection of other vessels, or in search of an enemy; a sailing to and fro, as for exploration or for pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He feigned a compliance with some of his men, who were bent upon going a <qex>cruise</qex> to Manilla.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>A voyage aboard a ship, in which the activities on the ship itself form a major objective of the voyage; -- used particularly of vacation voyages, or voyages during which some special activity occurs on board the ship, such as a series of seminars.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cruiser</ent><br/
<hw>Cruis"er</hw> <pr>(kr?"z?r)</pr>, <pos>n.</pos> <def>One who, or a vessel that, cruises;</def> <specif>Specif.</specif>: <fld>(Nav.)</fld> <def>A man-of-war less heavily armed and armored than a battle ship, having great speed, and generally of from two thousand to twelve thousand tons displacement.</def><br/
[<source>1913 Webster</source> + <source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cruive</ent><br/
<hw>Cruive</hw> <pr>(kr<usdot/v)</pr>, <pos>n.</pos>  <def>A kind of weir or dam for trapping salmon; also, a hovel.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crull</ent><br/
<hw>Crull</hw> <pr>(kr<usdot/l)</pr>, <pos>a.</pos> <ety>[See <er>Curl</er>.]</ety> <def>Curly; curled.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruller</ent><br/
<hw>Crul"ler</hw> <pr>(kr<ucr/l"l<etil/r)</pr>, <pos>n.</pos> <ety>[Cf. <er>Curl</er>.]</ety> <def>A kind of sweet cake cut in strips and curled or twisted, and fried crisp in boiling fat.</def> <altsp>[Also written  <asp>kruller</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumb</ent><br/
<hw>Crumb</hw> <pr>(kr<ucr/m)</pr>, <pos>n.</pos>  <ety>[AS.  <ets>cruma</ets>, akin to D. <ets>kruim</ets>, G.  <ets>krume</ets>; cf. G.  <ets>krauen</ets> to scratch, claw.]</ety> <altsp>[Written also <asp>crum</asp>.]</altsp> <sn>1.</sn> <def>A small fragment or piece; especially, a small piece of bread or other food, broken or cut off.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Desiring to be fed with the <qex>crumbs</qex> which fell from the rich man's table.</q> <rj><qau>Luke xvi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>Fig.: A little; a bit; <as>as, a <ex>crumb</ex> of comfort</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn>  <def>The soft part of bread.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dust unto dust, what must be, must;<br/
If you can't get <qex>crumb</qex>, you'd best eat crust.</q> <rj><qau>Old Song.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crumb brush</b></col>, <cd>a brush for sweeping crumbs from a table.</cd> -- <col><b>To a crum</b></col>, <cd>with great exactness; completely.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumb</ent><br/
<hw>Crumb</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crumbed</conjf> <pr>(kr<ucr/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crumbing</conjf> <pr>(kr<ucr/m"<icr/ng)</pr>.]</vmorph> <def>To break into crumbs or small pieces with the fingers; <as>as, to <ex>crumb</ex> bread</as>.</def> <altsp>[Written also <asp>crum</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumbcloth</ent><br/
<hw>Crumb"cloth`</hw> <pr>(-kl<ocr/th`; 115)</pr>, <pos>n.</pos> <def>A cloth to be laid under a dining table to receive falling fragments, and keep the carpet or floor clean.</def> <altsp>[Written also <asp>crumcloth</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumble</ent><br/
<hw>Crum"ble</hw> <pr>(kr<ucr/m"b'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crumbled</conjf> <pr>(kr<ucr/m"b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crumbling</conjf> <pr>(kr<ucr/m"bl<icr/ng)</pr>.]</vmorph> <ety>[Dim. of <ets>crumb</ets>, v. t., akin to D.  <ets>kruimelen</ets> G.  <ets>kr<uum/meln</ets>.]</ety> <def>To break into small pieces; to cause to fall in pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He with his bare wand can unthread thy joints,<br/
And <qex>crumble</qex> all thy sinews.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumble</ent><br/
<hw>Crum"ble</hw>, <pos>v. i.</pos> <def>To fall into small pieces; to break or part into small fragments; hence, to fall to decay or ruin; to become disintegrated; to perish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the stone is brittle, it will <qex>crumble</qex> and pass into the form of gravel.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The league deprived of its principal supports must soon <qex>crumble</qex> to pieces.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crumbled</ent><br/
<hw>crumbled</hw> <pos>adj.</pos> <def>broken into small fragments; <as>as, <ex>crumbled</ex> cookies</as>.</def><br/
<syn><b>Syn. --</b> fragmented.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crumbly</ent><br/
<hw>Crum"bly</hw> <pr>(-bl<ycr/)</pr>, <pos>a.</pos> <def>Easily crumbled; friable; brittle.</def> <ldquo/The <xex>crumbly</xex> soil.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
<syn><b>Syn. --</b> crimp.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumenal</ent><br/
<hw>Cru"me*nal</hw> <pr>(kr?"m?-n<it>a</it>l)</pr>, <pos>n.</pos> <ety>[L.  <ets>crumena</ets> purse.]</ety> <def>A purse.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crummable</ent><br/
<hw>Crum"ma*ble</hw> <pr>(kr<ucr/m"m<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being crumbed or broken into small pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crummy</ent><br/
<hw>Crum"my</hw> <pr>(kr<ucr/m"m<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of crumb or crumbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Soft, as the crumb of bread is; not crusty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>of very poor quality.</def> <mark>[informal]</mark><br/
<syn><b>Syn. --</b> bum, cheap, cheesy, chintzy, crummy, punk, sleazy, tinny.</syn>
[<source>WordNet 1.5</source>]</p>

<p><ent>Crump</ent><br/
<hw>Crump</hw> <pr>(kr<ucr/mp)</pr>, <pos>a.</pos> <ety>[AS.  <ets>crumb</ets> stooping, bent down; akin to OHG.  <ets>chrumb</ets>, G.  <ets>krumm</ets>, Dan.  <ets>krum</ets>, D.  <ets>krom</ets>, and E.  <ets>cramp</ets>.]</ety> <sn>1.</sn> <def>Crooked; bent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Crooked backs and <qex>crump</qex> shoulders.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hard or crusty; dry baked; <as>as, a <ex>crump</ex> loaf</as>.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Hallivell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crumpet</ent><br/
<hw>crump"et</hw> <pr>(kr<ucr/mp"<ecr/t)</pr>, <pos>n.</pos> <ety>[Prob. from W. <ets>crempog</ets>, <ets>crammwgth</ets>, a pancake or fritter.]</ety> <sn>1.</sn> <def>A kind of large, thin, unsweetened muffin or cake, light and spongy, and cooked on a griddle or spider, or sometimes toasted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a sexually attractive woman.</def> <mark>[British slang]</mark> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>Crumple</ent><br/
<hw>Crum"ple</hw> <pr>(kr<ucr/m"p'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crumpled</conjf> <pr>(-p'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crumpling</conjf> <pr>(-pl<icr/ng)</pr>.]</vmorph> <ety>[Dim. fr.  <ets>crump</ets>, <ets>a.</ets>]</ety> <def>To draw or press into wrinkles or folds; to crush together; to rumple; <as>as, to <ex>crumple</ex> paper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>crumpled</qex> it into all shapes, and diligently scanned every wrinkle that could be made.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumple</ent><br/
<hw>Crum"ple</hw>, <pos>v. i.</pos> <def>To contract irregularly; to show wrinkles after being crushed together; <as>as, leaves <ex>crumple</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crumpy</ent><br/
<hw>Crump"y</hw> <pr>(kr<ucr/mp"<ycr/)</pr>, <pos>a.</pos> <def>Brittle; crisp.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crunch</ent><br/
<hw>Crunch</hw> <pr>(kr<ucr/nch)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crunched</conjf> <pr>(kr<ucr/ncht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crunching</conjf>.]</vmorph> <ety>[Prob. of imitative origin; or cf. D.  <ets>schransen</ets> to eat heartily, or E. <ets>scrunch</ets>.]</ety> <sn>1.</sn> <def>To chew with force and noise; to craunch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And their white tusks <qex>crunched</qex> o'er the whiter skull.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To grind or press with violence and noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ship <qex>crunched</qex> through the ice.</q> <rj><qau>Kane.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To emit a grinding or craunching noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crunching</qex> and ratting of the loose stones.</q> <rj><qau>H. James.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crunch</ent><br/
<hw>Crunch</hw>, <pos>v. t.</pos> <def>To crush with the teeth; to chew with a grinding noise; to craunch; <as>as, to <ex>crunch</ex> a biscuit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crunkle</ent><br/
<ent>Crunk</ent><br/
<mhw>{ <hw>Crunk</hw> <pr>(kr<ucr/<nsm/k)</pr>, <hw>Crun"kle</hw> <pr>(kr<ucr/<nsm/"k'l)</pr>, }</mhw> <pos>v. i.</pos> <ety>[Cf. Icel.  <ets>kr<?/nka</ets> to croak.]</ety> <def>To cry like a crane.</def> <mark>[Obs.]</mark> <ldquo/The crane <xex>crunketh</xex>.<rdquo/  <rj><au>Withals (1608).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crunodal</ent><br/
<hw>Cru*no"dal</hw> <pr>(kr?-n?"d<it>a</it>l)</pr>, <pos>a.</pos> <fld>(Geom.)</fld> <def>Possessing, or characterized by, a crunode; -- used of curves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crunode</ent><br/
<hw>Cru"node</hw> <pr>(kr?"n?d)</pr>, <pos>n.</pos> <ety>[Prob. fr. L.  <ets>crux</ets> a cross + E.  <ets>node</ets>.]</ety> <fld>(Geom.)</fld> <def>A point where one branch of a curve crosses another branch. See <cref>Double point</cref>, under <er>Double</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruor</ent><br/
||<hw>Cru"or</hw> <pr>(kr?"?r)</pr>, <pos>n.</pos> <ety>[L., blood. See <er>Crude</er>.]</ety> <def>The coloring matter of the blood; the clotted portion of coagulated blood, containing the coloring matter; gore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruorin</ent><br/
<hw>Cru"o*rin</hw> <pr>(-?-r?n)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The coloring matter of the blood in the living animal; h<ae/moglobin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crup</ent><br/
<hw>Crup</hw> <pr>(kr?p)</pr>, <pos>a.</pos> <ety>[Cf. OHG.  <ets>grop</ets>, G. <ets>grob</ets>, coarse.]</ety> <def>Short; brittle; <as>as, <ex>crup</ex> cake</as>.</def>  <rj><au>Todd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crup</ent><br/
<hw>Crup</hw> <pr>(kr?p)</pr>, <pos>n.</pos> <def>See <er>Croup</er>, the rump of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crupper</ent><br/
<hw>Crup"per</hw> <pr>(kr?p"p?r <it>in U.S.</it>; kr?p"?r <it>in Eng.</it>)</pr>, <pos>n.</pos> <ety>[F. <ets>croupi<?/re</ets>, fr.  <ets>croupe</ets>.  See <er>Croup</er> the rump of a horse.]</ety> <altsp>[Written also <asp>crouper</asp>.]</altsp> <sn>1.</sn> <def>The buttocks or rump of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A leather loop, passing under a horse's tail, and buckled to the saddle to keep it from slipping forwards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crupper</ent><br/
<hw>Crup"per</hw>, <pos>v. t.</pos> <def>To fit with a crupper; to place a crupper upon; <as>as, to <ex>crupper</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crura</ent><br/
||<hw>Cru"ra</hw> <pr>(kr?"r?)</pr>, <pos>n. pl.</pos> <fld>(Anat.)</fld> <def>See <er>Crus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crural</ent><br/
<hw>Cru"ral</hw> <pr>(-r<it>a</it>l)</pr>, <pos>a.</pos> <ety>[L.  <ets>cruralis</ets>, fr.  <ets>crus</ets>, <ets>cruris</ets>, leg: cf. F. <ets>crural</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the thigh or leg, or to any of the parts called <xex>crura</xex>; <as>as, the <ex>crural</ex> arteries; <ex>crural</ex> arch; <ex>crural</ex> canal; <ex>crural</ex> ring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crus</ent><br/
||<hw>Crus</hw> <pr>(kr?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crura</plw> <pr>(kr<?/"r<?/)</pr>.</plu> <ety>[L., the leg.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>That part of the hind limb between the femur, or thigh, and the ankle, or tarsus; the shank.</def> <sd>(b)</sd> <def>Often applied, especially in the plural, to parts which are supposed to resemble a pair of legs; <as>as, the <ex>crura</ex> of the diaphragm, a pair of muscles attached to it; <ex>crura cerebri</ex>, two bundles of nerve fibers in the base of the brain, connecting the medulla and the forebrain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusade</ent><br/
<hw>Cru*sade"</hw> <pr>(kr?-s?d")</pr>, <pos>n.</pos> <ety>[F. <ets>croisade</ets>, fr. Pr.  <ets>crozada</ets>, or Sp <ets>cruzada</ets>, or It.  <ets>crociata</ets>, from a verb signifying to take the cross, mark one's self with a cross, fr. L.  <ets>crux</ets> cross; or possibly taken into English directly fr. Pr. Cf. <er>Croisade</er>, <er>Crosado</er>, and see <er>Cross</er>.]</ety> <sn>1.</sn> <def>Any one of the military expeditions undertaken by Christian powers, in the 11th, 12th, and 13th centuries, for the recovery of the Holy Land from the Mohammedans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any enterprise undertaken with zeal and enthusiasm; <as>as, a <ex>crusade</ex> against intemperance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A Portuguese coin. See <er>Crusado</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusade</ent><br/
<hw>Cru*sade"</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crusaded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Crusading</conjf>.]</vmorph> <def>To engage in a crusade; to attack in a zealous or hot-headed manner.</def> <ldquo/Cease <xex>crusading</xex> against sense.<rdquo/  <rj><au>M. Green.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusader</ent><br/
<hw>Cru*sad"er</hw> <pr>(-s?"d?r)</pr>, <pos>n.</pos> <def>One engaged in a crusade; <as>as, the <ex>crusaders</ex> of the Middle Ages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Azure-eyed and golden-haired,<br/
Forth the young <qex>crusaders</qex> fared.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusading</ent><br/
<hw>Cru*sad"ing</hw>, <pos>a.</pos> <def>Of or pertaining to a crusade; <as>as, a <ex>crusading</ex> spirit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusado</ent><br/
<hw>Cru*sa"do</hw> <pr>(-s?"d?)</pr>, <pos>n.</pos> <ety>[Pg.  <ets>cruzado</ets>, fr. <ets>cruz</ets>, fr. L.  <ets>crux</ets>. See <er>Crusade</er>, 3.]</ety> <def>An old Portuguese coin, worth about seventy cents.</def> <altsp>[Written also <asp>cruade</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruse</ent><br/
<hw>Cruse</hw> <pr>(kr?s)</pr>, <pos>n.</pos> <ety>[Akin to LG.  <ets>kruus</ets>, <ets>kroos</ets>, mug, jug, jar, D.  <ets>kroes</ets>, G.  <ets>krause</ets>, Icel.  <ets>krus</ets>, Sw.  <ets>krus</ets>, Dan.  <ets>kruus</ets>. Cf. <er>Crucible</er>, <er>Cresset</er>.]</ety> <sn>1.</sn> <def>A cup or dish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Take with thee . . . a <qex>cruse</qex> of honey.</q> <rj><qau>1 Kings xiv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bottle for holding water, oil, honey, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So David took . . . the <qex>cruse</qex> of water.</q> <rj><qau>1 Sam. xxvi. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruset</ent><br/
<hw>Cru"set</hw> <pr>(kr?"s?t)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>creuset</ets>. See <er>Cruse</er>, <er>Crucible</er>.]</ety> <def>A goldsmith's crucible or melting pot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crush</ent><br/
<hw>Crush</hw> <pr>(kr<ucr/sh)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crushed</conjf> <pr>(kr<ucr/sht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crushing</conjf>.]</vmorph> <ety>[OE.  <ets>cruschen</ets>, <ets>crousshen</ets>, Of.  <ets>cruisir</ets>, <ets>croissir</ets>, fr. LL.  <ets>cruscire</ets>, prob. of Ger. origin, from a derivative of the word seen in Goth.  <ets>kruistan</ets> to gnash; akin to Sw.  <ets>krysta</ets> to squeeze, Dan.  <ets>kryste</ets>, Icel. <ets>kreysta</ets>.]</ety> <sn>1.</sn> <def>To press or bruise between two hard bodies; to squeeze, so as to destroy the natural shape or integrity of the parts, or to force together into a mass; <as>as, to <ex>crush</ex> grapes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall not offer unto the Lord that which is bruised, or <qex>crushed</qex>, or broken, or cut.</q> <rj><qau>Lev. xxii. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ass . . . thrust herself unto the wall, and <qex>crushed</qex> Balaam's foot against the wall.</q> <rj><qau>Num. xxii. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reduce to fine particles by pounding or grinding; to comminute; <as>as, to <ex>crush</ex> quartz</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To overwhelm by pressure or weight; to beat or force down, as by an incumbent weight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>crush</qex> the pillars which the pile sustain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Truth, <qex>crushed</qex> to earth, shall rise again.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To oppress or burden grievously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt be only oppressed and <qex>crushed</qex> alway.</q> <rj><qau>Deut. xxviii. 33.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To overcome completely; to subdue totally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Speedily overtaking and <qex>crushing</qex> the rebels.</q> <rj><qau>Sir. W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>to subdue or overwhelm (a person) by argument or a cutting remark; to cause (a person) to feel chagrin or humiliation; to squelch.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>To crush a cup</b></col>, <cd>to drink.</cd> <mark>[Obs.]</mark> -- <col><b>To crush out</b></col>. <sd>(a)</sd> <cd>To force out or separate by pressure, as juice from grapes.</cd> <sd>(b)</sd> <cd>To overcome or destroy completely; to suppress.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crush</ent><br/
<hw>Crush</hw> <pr>(kr<ucr/sh)</pr>, <pos>v. i.</pos> <def>To be or become broken down or in, or pressed into a smaller compass, by external weight or force; <as>as, an eggshell <ex>crushes</ex> easily</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crush</ent><br/
<hw>Crush</hw>, <pos>n.</pos> <sn>1.</sn> <def>A violent collision or compression; a crash; destruction; ruin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wreck of matter, and the <qex>crush</qex> of worlds.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Violent pressure, as of a crowd; a crowd which produced uncomfortable pressure; <as>as, a <ex>crush</ex> at a reception</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crush hat</b></col>, <cd>a hat which collapses, and can be carried under the arm, and when expanded is held in shape by springs; hence, any hat not injured by compressing.</cd> -- <col><b>Crush room</b></col>, <cd>a large room in a theater, opera house, etc., where the audience may promenade or converse during the intermissions; a foyer.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Politics leave very little time for the bow window at White's in the day, or for the <qex>crush room</qex> of the opera at night.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>crushed</ent><br/
<hw>crushed</hw> <pos>adj.</pos> <sn>1.</sn> <def>treated so as to have a permanently wrinkled appearance; -- of fabrics; <as>as, <ex>crushed</ex> velvet</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>utterly defeated.</def><br/
<syn><b>Syn. --</b> subdued.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>brought low in condition or status by confusion, humiliation, or severe disappointment.</def><br/
<syn><b>Syn. --</b> broken, humbled, humiliated, low.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>4.</sn>  <def>broken or pounded into small fragments; used of e.g. ore or stone.</def> <illu>paved with <ex>crushed</ex> bluestone</illu><br/
<syn><b>Syn. --</b> ground.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crusher</ent><br/
<hw>Crush"er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who, or that which, crushes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crusher gauge</b></col>, <cd>an instrument for measuring the explosive force of gunpowder, etc., by its effect in compressing a piece of metal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crushing</ent><br/
<hw>Crush"ing</hw>, <pos>a.</pos> <def>That crushes; overwhelming.</def> <ldquo/The blow must be quick and <xex>crushing</xex>.<rdquo/  <rj><au>Macualay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crust</ent><br/
<hw>Crust</hw> <pr>(kr?st)</pr>, <pos>n.</pos> <ety>[L.  <ets>crusta</ets>: cf. OF.  <ets>crouste</ets>, F. <ets>cro<ucir/te</ets>; prob. akin to Gr. <?/<?/<?/<?/<?/ ice, E.  <ets>crystal</ets>, from the same root as E.  <ets>crude</ets>, <ets>raw</ets>. See <er>Raw</er>, and cf. <er>Custard</er>.]</ety> <sn>1.</sn> <def>The hard external coat or covering of anything; the hard exterior surface or outer shell; an incrustation; <as>as, a <ex>crust</ex> of snow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have known the statute of an emperor quite hid under a <qex>crust</qex> of dross.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Below this icy <qex>crust</qex> of conformity, the waters of infidelity lay dark and deep as ever.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cookery)</fld> <sd>(a)</sd> <def>The hard exterior or surface of bread, in distinction from the soft part or crumb; or a piece of bread grown dry or hard.</def> <sd>(b)</sd> <def>The cover or case of a pie, in distinction from the soft contents.</def> <sd>(c)</sd> <def>The dough, or mass of doughy paste, cooked with a potpie; -- also called <altname>dumpling</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Th' impenetrable <qex>crust</qex> thy teeth defies.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that keeps nor <qex>crust</qex> nor crumb.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . made the <qex>crust</qex> for the venison pasty.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>The exterior portion of the earth, formerly universally supposed to inclose a molten interior.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The shell of crabs, lobsters, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>A hard mass, made up of dried secretions blood, or pus, occurring upon the surface of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An incrustation on the interior of wine bottles, the result of the ripening of the wine; a deposit of tartar, etc. See <er>Beeswing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crust</ent><br/
<hw>Crust</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crusted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Crusting</conjf>.]</vmorph> <ety>[Cf. OF.  <ets>crouster</ets>, L.  <ets>crustare</ets>. See <er>Crust</er>, <pos>n.</pos> ]</ety> <def>To cover with a crust; to cover or line with an incrustation; to incrust.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole body is <qex>crusted</qex> over with ice.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And now their legs, and breast, and bodies stood<br/
<qex>Crusted</qex> with bark.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Very foul and <qex>crusted</qex> bottles.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their minds are <qex>crusted</qex> over, like diamonds in the rock.</q> <rj><qau>Felton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crust</ent><br/
<hw>Crust</hw>, <pos>v. i.</pos> <def>To gather or contract into a hard crust; to become incrusted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The place that was burnt . . . <qex>crusted</qex> and healed.</q> <rj><qau>Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusta</ent><br/
||<hw>Crus"ta</hw> <pr>(kr?s"t?)</pr>, <pos>n.</pos> <ety>[L., shell, crust, inlaid work.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A crust or shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gem engraved, or a plate embossed in low relief, for inlaying a vase or other object.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustacea</ent><br/
||<hw>Crus*ta"ce*a</hw> <pr>(kr?s-t?"sh?-?)</pr>, <pos>n. pl.</pos> <ety>[Neut. pl. of NL.  <ets>crustaceus</ets> pert. to the crust or shell, from L. <ets>crusta</ets> the hard surfsce of a body, rind, shell.]</ety> <fld>(Zool.)</fld> <def>One of the classes of the arthropods, including lobsters and crabs; -- so called from the crustlike shell with which they are covered.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The body usually consists of an anterior part, made up of the head and thorax combined, called the <xex>cephalothorax</xex>, and of a posterior jointed part called the <xex>abdomen</xex>, <xex>postabdomen</xex>, and (improperly) <xex>tail</xex>. They breathe by means of gills variously attached to some of the limbs or to the sides the body, according to the group. They are divisible into two subclasses, Entomostraca and Malacostraca, each of which includes several orders.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustacean</ent><br/
<hw>Crus*ta"cean</hw> <pr>(kr?s-t?"sh<it>a</it>n; 97)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Crustacea; crustaceous.</def> -- <def2><pos>n.</pos> <def>An animal belonging to the class <class>Crustacea</class>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustaceological</ent><br/
<hw>Crus*ta`ce*o*log"ic*al</hw> <pr>(-sh?-?-l?j"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Pertaining to crustaceology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustaceologist</ent><br/
<hw>Crus*ta`ce*ol"o*gist</hw> <pr>(-?ll"?-j?st)</pr>, <pos>n.</pos> <def>One versed in crustaceology; a crustalogist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustaceology</ent><br/
<hw>Crus*ta`ce*ol"o*gy</hw> <pr>(-j?)</pr>, <pos>n.</pos> <ety>[<ets>Crustacea</ets> + <ets>-logy</ets>.]</ety> <def>That branch of Zoology which treats of the Crustacea; malacostracology; carcinology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustaceous</ent><br/
<hw>Crus*ta"ceous</hw> <pr>(kr?s-t?"sh?s; 97)</pr>, <pos>a.</pos> <ety>[NL. <ets>crustaceous</ets>. See <er>crustacea</er>.]</ety> <sn>1.</sn> <def>Pertaining to, or of the nature of, crust or shell; having a crustlike shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Belonging to the Crustacea; crustacean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustaceousness</ent><br/
<hw>Crus*ta"ceous*ness</hw>, <pos>n.</pos> <def>The state or quality of being crustaceous or having a crustlike shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustal</ent><br/
<hw>Crust"al</hw> <pr>(kr?st"<it>a</it>l)</pr>, <pos>a.</pos> <def>Relating to a crust.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustalogical</ent><br/
<hw>Crus`ta*log"ic*al</hw> <pr>(kr?s`t?-l?j"?-k<it>a</it>l)</pr>, <pos>a.</pos> <def>Pertaining to crustalogy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustalogist</ent><br/
<hw>Crus*tal"o*gist</hw> <pr>(-t<?/l"<?/-j<?/st)</pr>, <pos>n.</pos> <def>One versed in crustalogy.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 352 --></p>

<p><ent>Crustalogy</ent><br/
<hw>Crus*tal"o*gy</hw> <pr>(kr?s-t?l"?-j?)</pr>, <pos>n.</pos> <ety>[L.  <ets>crusta</ets> shell + <ets>-logy</ets>.]</ety> <def>Crustaceology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustated</ent><br/
<hw>Crus"ta*ted</hw> <pr>(kr?s"t?-t?d)</pr>, <pos>a.</pos> <ety>[L.  <ets>crustatus</ets>, p. p. of <ets>crustare</ets>, fr.  <ets>crusta</ets>. See <er>Crust</er>.]</ety> <def>Covered with a crust; <as>as, <ex>crustated</ex> basalt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustation</ent><br/
<hw>Crus*ta"tion</hw> <pr>(kr?s-t?"sh?n)</pr>, <pos>n.</pos> <def>An adherent crust; an incrustation.</def>  <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusted</ent><br/
<hw>Crust"ed</hw> <pr>(kr?st"?d)</pr>, <pos>a.</pos> <def>Incrusted; covered with, or containing, crust; <as>as, old, <ex>crusted</ex> port wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustific</ent><br/
<hw>Crus*tif`ic</hw> <pr>(kr?s-t?f"?k)</pr>, <pos>a.</pos> <ety>[L.  <ets>crusta</ets> crust + <ets>-facere</ets> to make.]</ety> <def>Producing or forming a crust or skin.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustily</ent><br/
<hw>Crust"i*ly</hw> <pr>(kr<ucr/st"<icr/-l<ycr/)</pr>, <pos>adv.</pos> <def>In a crusty or surly manner; morosely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crustiness</ent><br/
<hw>Crust"i*ness</hw> <pr>(-<icr/-n<ecr/s)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state or quality of having crust or being like crust; hardness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality of being crusty or surly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Old Christy forgot his usual <qex>crustiness</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crusty</ent><br/
<hw>Crust"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having the nature of crust; pertaining to a hard covering; <as>as, a <ex>crusty</ex> coat; a <ex>crusty</ex> surface or substance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Possibly a corruption of <ets>cursty</ets>. Cf. <er>Curst</er>, <er>Curstness</er>.]</ety> <def>Having a hard exterior, or a short, rough manner, though kind at heart; snappish; peevish; surly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>crusty</qex> batch of nature, what's the news?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crut</ent><br/
<hw>Crut</hw> <pr>(kr<ucr/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cro<ucir/te</ets> crust.]</ety> <def>The rough, shaggy part of oak bark.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crutch</ent><br/
<hw>Crutch</hw> <pr>(kr<ucr/ch; 224)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Crutches</plw> <pr>(-<ecr/z)</pr>.</plu> <ety>[OE. <ets>crucche</ets>, AS. <ets>crycc</ets>, <ets>cricc</ets>; akin to D.  <ets>kruk</ets>, G.  <ets>kr<uum/cke</ets>, Dan. <ets>krykke</ets>, Sw.  <ets>krycka</ets>, and to E. <ets>crook</ets>. See <er>Crook</er>, and cf.  <er>Cricket</er> a low stool.]</ety> <sn>1.</sn> <def>A staff with a crosspiece at the head, to be placed under the arm or shoulder, to support the lame or infirm in walking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll lean upon one <qex>crutch</qex>, and fight with the other.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rhyme is a <qex>crutch</qex> that lifts the weak alone.</q> <rj><qau>H. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A form of pommel for a woman's saddle, consisting of a forked rest to hold the leg of the rider.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A knee, or piece of knee timber</def>. <sd>(b)</sd> <def>A forked stanchion or post; a crotch. See <er>Crotch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crutch</ent><br/
<hw>Crutch</hw>, <pos>v. t.</pos> <def>To support on crutches; to prop up.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Two fools that <qex>crutch</qex> their feeble sense on verse.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crutched</ent><br/
<hw>Crutched</hw> <pr>(kr?cht)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Supported upon crutches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[See <er>Crouch</er>, <pos>v. t.</pos>, and <er>Crouched</er>, <pos>a.</pos> ]</ety> <def>Marked with the sign of the cross; crouched.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crutched friar</b></col> <fld>(Eccl.)</fld>, <cd>one of a religious order, so called because its members bore the sign of the cross on their staves and habits; -- called also <altname>crossed friar</altname> and <altname>crouched friar</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cruth</ent><br/
<hw>Cruth</hw> <pr>(kr?th)</pr>, <pos>n.</pos> <ety>[W.  <ets>crwth</ets>.]</ety> <fld>(Mus.)</fld> <def>See 4th <er>Crowd</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crux</ent><br/
||<hw>Crux</hw> <pr>(kr<ucr/ks)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E.  <plw>Cruxes</plw> <pr>(-<ecr/z)</pr>, L.  <plw>Cruces</plw> <pr>(kr<udd/"s<emac/z)</pr>.</plu> <ety>[L., cross, torture, trouble.]</ety> <def>Anything that is very puzzling or difficult to explain.</def>  <rj><au>Dr. Sheridan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The perpetual <qex>crux</qex> of New Testament chronologists.</q> <rj><qau>Strauss.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crux ansata</ent><br/
||<hw>Crux an*sa"ta</hw> <pr>(?)</pr>. <ety>[L., cross with a handle.]</ety> <def>A cross in the shape of the ankh.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cruzado</ent><br/
||<hw>Cru*za"do</hw> <pr>(kr?-z?"d?)</pr>, <pos>n.</pos> <def>A coin. See <er>Crusado</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crwth</ent><br/
||<hw>Crwth</hw> <pr>(kr<oomac/th)</pr>, <pos>n.</pos> <ety>[W.]</ety> <fld>(Mus.)</fld> <def>See 4th <er>Crowd</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cry</ent><br/
<hw>Cry</hw> <pr>(kr<imac/)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cried</conjf> <pr>(kr<imac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crying</conjf>.]</vmorph> <ety>[F. <ets>crier</ets>, cf. L.  <ets>quiritare</ets> to raise a plaintive cry, scream, shriek, perh. fr.  <ets>queri</ets> to complain; cf. Skr. <ets>cvas</ets> to pant, hiss, sigh. Cf. <er>Quarrel</er> a brawl, <er>Querulous</er>.]</ety> <sn>1.</sn> <def>To make a loud call or cry; to call or exclaim vehemently or earnestly; to shout; to vociferate; to proclaim; to pray; to implore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And about the ninth hour, Jesus <qex>cried</qex> with a loud voice.</q> <rj><qau> Matt. xxvii. 46.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Clapping their hands, and <qex>crying</qex> with loud voice.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hear the voice of my supplications when I <qex>cry</qex> unto thee.</q> <rj><qau> Ps. xxviii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The voice of him that <qex>crieth</qex> in the wilderness, Prepare ye the way of the Lord.</q> <rj><qau>Is. xl. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>cried</qex> after him to return.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter lamentations; to lament audibly; to express pain, grief, or distress, by weeping and sobbing; to shed tears; to bawl, as a child.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall <qex>cry</qex> for sorrow of heart.</q> <rj><qau>Is. lxv. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I could find it in my heart to disgrace my man's apparel and to <qex>cry</qex> like a woman.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter inarticulate sounds, as animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The young ravens which <qex>cry</qex>.</q> <rj><qau>Ps. cxlvii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In a cowslip's bell I lie<br/
There I couch when owls do <qex>cry</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To cry on</b></col> <it>or</it> <col><b>To cry upon</b></col></mcol>, <cd>to call upon the name of; to beseech.</cd> <ldquo/No longer <xex>on</xex> Saint Denis will we <xex>cry</xex>.<rdquo/ <au>Shak.</au> -- <col><b>To cry out</b></col>. <sd>(a)</sd> <cd>To exclaim; to vociferate; to scream; to clamor.</cd> <sd>(b)</sd> <cd>To complain loudly; to lament.</cd> -- <col><b>To cry out against</b></col>, <cd>to complain loudly of; to censure; to blame.</cd> -- <mcol><col><b>To cry out on</b></col> <it>or</it> <col><b>To cry out upon</b></col></mcol>, <cd>to denounce; to censure.</cd> <ldquo/<xex>Cries out upon</xex> abuses.<rdquo/ <au>Shak.</au> -- <col><b>To cry to</b></col>, <cd>to call on in prayer; to implore.</cd> -- <col><b>To cry you mercy</b></col>, <cd>to beg your pardon.</cd> <ldquo/I <xex>cry you mercy</xex>, madam; was it you?<rdquo/ <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cry</ent><br/
<hw>Cry</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter loudly; to call out; to shout; to sound abroad; to declare publicly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All, all, <qex>cry</qex> shame against ye, yet I 'll speak.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The man . . . ran on,<qex>crying</qex>, Life! life! Eternal life!</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to do something, or bring to some state, by crying or weeping; <as>as, to <ex>cry</ex> one's self to sleep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make oral and public proclamation of; to declare publicly; to notify or advertise by outcry, especially things lost or found, goods to be sold, ets.; <as>as, to <ex>cry</ex> goods, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love is lost, and thus she <qex>cries</qex> him.</q> <rj><qau>Crashaw.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> Hence, <def>to publish the banns of, as for marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I should not be surprised if they were <qex>cried</qex> in church next Sabbath.</q> <rj><qau>Judd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cry aim</b></col>. <cd>See under <er>Aim</er>.</cd> -- <col><b>To cry down</b></col>, <cd>to decry; to depreciate; to dispraise; to condemn.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Men of dissolute lives <qex>cry down</qex> religion, because they would not be under the restraints of it.</q> <rj><qau>Tillotson.</qau></rj></p>

<p> -- <col><b>To cry out</b></col>, <cd>to proclaim; to shout.</cd> <ldquo/Your gesture <xex>cries</xex> it <xex>out</xex>.<rdquo/ <au>Shak.</au> -- <col><b>To cry quits</b></col>, <cd>to propose, or declare, the abandonment of a contest.</cd> -- <col><b>To cry up</b></col>, <cd>to enhance the value or reputation of by public and noisy praise; to extol; to laud publicly or urgently.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cry</ent><br/
<hw>Cry</hw> <pr>(kr?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cries</plw> <pr>(kr<?/z)</pr>.</plu> <ety>[F.  <ets>cri</ets>, fr. <ets>crier</ets> to cry. See <er>Cry</er>, <pos>v. i.</pos> ]</ety> <sn>1.</sn> <def>A loud utterance; especially, the inarticulate sound produced by one of the lower animals; <as>as, the <ex>cry</ex> of hounds; the <xex>cry</xex> of wolves</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Outcry; clamor; tumult; popular demand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Again that <qex>cry</qex> was found to have been as unreasonable as ever.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any expression of grief, distress, etc., accompanied with tears or sobs; a loud sound, uttered in lamentation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There shall be a great <qex>cry</qex> throughout all the land.</q> <rj><qau>Ex. xi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An infant crying in the night,<br/
An infant crying for the light;<br/
And with no language but a <qex>cry</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Loud expression of triumph or wonder or of popular acclamation or favor.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cry</qex> went once on thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Importunate supplication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, the most piteous <qex>cry</qex> of the poor souls.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Public advertisement by outcry; proclamation, as by hawkers of their wares.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The street <qex>cries</qex> of London.</q> <rj><qau>Mayhew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Common report; fame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>cry</qex> goes that you shall marry her.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A word or phrase caught up by a party or faction and repeated for effect; <as>as, the party <ex>cry</ex> of the Tories</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All now depends upon a good <qex>cry</qex>.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A pack of hounds.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>cry</qex> more tunable<br/
Was never hollaed to, nor cheered with horn.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A pack or company of persons; -- in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Would not this . . . get me a fellowship in a <qex>cry</qex> of players?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>The crackling noise made by block tin when it is bent back and forth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A far cry</b></col>, <cd>a long distance; -- in allusion to the sending of criers or messengers through the territory of a Scottish clan with an announcement or summons.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryal</ent><br/
<hw>Cry"al</hw> <pr>(kr?"<it>a</it>l)</pr>, <pos>n.</pos> <ety>[Cf. W.  <ets>creyr</ets>, <ets>cryr</ets>, <ets>crychydd</ets>. Cf. <er>Cruer</er> a hawk.]</ety> <def>The heron</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryer</ent><br/
<hw>Cry"er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <ety>[F. faucon <ets>gruyer</ets> a falcon trained to fly at the crane, fr. <ets>crye</ets> crane, fr. L. <ets>crus</ets> crane. Cf. <er>Cryal</er>.]</ety> <def>The female of the hawk; a falcon-gentil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crying</ent><br/
<hw>Cry"ing</hw>, <pos>a.</pos> <def>Calling for notice; compelling attention; notorious; heinous; <as>as, a <ex>crying</ex> evil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Too much fondness for meditative retirement is not the <qex>crying</qex> sin of our modern Christianity.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryohydrate</ent><br/
<hw>Cry`o*hy"drate</hw> <pr>(kr?`?-h?"dr?t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`os</grk> cold + E.  <ets>hydrate</ets>.]</ety> <fld>(Chem.)</fld> <def>A substance, as salt, ammonium chloride, etc., which crystallizes with water of crystallization only at low temperatures, or below the freezing point of water.</def>  <rj><au> F. Guthrie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryolite</ent><br/
<hw>Cry"o*lite</hw> <pr>(kr<imac/"?-l?t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`os</grk> icy cold, frost + <ets>-lite</ets>: cf. F. <ets>cryolithe</ets>.]</ety> <fld>(Min.)</fld> <def>A fluoride of sodium and aluminum, found in Greenland, in white cleavable masses; -- used as a source of soda and alumina.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cryometer</ent><br/
<hw>cry*om"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ cold, frost + <ets>-meter</ets>.]</ety> <fld>(Physics)</fld> <def>A thermometer for the measurement of low temperatures, esp. such an instrument containing alcohol or some other liquid of a lower freezing point than mercury.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cryopathy</ent><br/
<hw>cryopathy</hw> <pos>n.</pos> <def>destruction of tissue by freezing and characterized by tingling, blistering and possibly gangrene.</def><br/
<syn><b>Syn. --</b> frostbite.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cryophobia</ent><br/
<hw>cryophobia</hw> <pos>n.</pos> <def>a morbid fear of freezing.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cryophorus</ent><br/
<hw>cry*oph"o*rus</hw> <pr>(kr<isl/*<ocr/f"<osl/*r<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kry`os</grk> icy cold, frost + <grk>fe`rein</grk> to bear.]</ety> <fld>(Chem.)</fld> <def>An instrument used to illustrate the freezing of water by its own evaporation. The ordinary form consists of two glass bulbs, connected by a tube of the same material, and containing only a quantity of water and its vapor, devoid of air. The water is in one of the bulbs, and freezes when the other is cooled below 32<deg/ Fahr.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cryoscope</ent><br/
<hw>cry"o*scope</hw> <pos>n.</pos> <def>an instrument for measuring freezing and melting points.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cryostat</ent><br/
<hw>cry"o*stat</hw> <pos>n.</pos> <def>a thermostat that operates at very low temperatures.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Crypt</ent><br/
<hw>Crypt</hw> <pr>(kr<icr/pt)</pr>, <pos>n.</pos> <ety>[L.  <ets>crypta</ets> vault, crypt, Gr. <grk>kry`pth</grk>, fr. <grk>kry`ptein</grk> to hide. See <er>Grot</er>, <er>Grotto</er>.]</ety> <sn>1.</sn> <def>A vault wholly or partly under ground; especially, a vault under a church, whether used for burial purposes or for a subterranean chapel or oratory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Priesthood works out its task age after age, . . . treasuring in convents and <qex>crypts</qex> the few fossils of antique learning.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My knees are bowed in <qex>crypt</qex> and shrine.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A simple gland, glandular cavity, or tube; a follicle; <as>as, the <ex>crypts</ex> of Lieberk<uum/hn, the simple tubular glands of the small intestines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptal</ent><br/
<hw>Crypt"al</hw> <pr>(-<it>a</it>l)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to crypts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cryptanalysis</ent><br/
<hw>crypt"a*nal"y*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>the science which studies methods to discovering the secret meaning of encrypted messages for which one does not possess the secret decoding information (called the <er>key</er>).</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the methods and procedures used to discover the secret meaning hidden in encrypted messages.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cryptical</ent><br/
<ent>Cryptic</ent><br/
<mhw>{ <hw>Cryp"tic</hw> <pr>(kr<icr/p"t<icr/k)</pr>, <hw>Cryp"tic*al</hw> <pr>(-t?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[L.  <ets>crypticus</ets>, Gr. <grk>kryptiko`s</grk>, fr. <grk>kry`ptein</grk> to hide.]</ety> <def>Hidden; secret; occult.</def> <ldquo/Her [nature's] more <xex>cryptic</xex> ways of working.<rdquo/  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>incomprehensible to those not familiar with the culture or jargon; <as>as, the new insurance policy is written without <ex>cryptic</ex> or mysterious terms</as>.</def><br/
<syn><b>Syn. --</b> inscrutable, mysterious, mystifying.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>having a secret or hidden meaning; <as>as, <ex>cryptic</ex> symbols engraved in stone; <ex>cryptic</ex> writings</as>.</def><br/
<syn><b>Syn. --</b> cabalistic, occult, secret, sibylline.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>having a puzzling terseness; <as>as, a <ex>cryptic</ex> note</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>not evident; unrecognized; <as>as, a <ex>cryptic</ex> infection</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn>  <def>written in a code or cipher; <as>as, a <ex>cryptic</ex> message</as>.</def><br/
<syn><b>Syn. --</b> encoded, enciphered, encrypted.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Cryptically</ent><br/
<hw>Cryp"tic*al*ly</hw>, <pos>adv.</pos> <def>Secretly; occultly; in a manner so as to hide or obscure meaning.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cryptidine</ent><br/
<hw>Cryp"ti*dine</hw> <pr>(kr?p"t?-d?n; 104)</pr>, <pos>n.</pos> <ety>[Gr. <grk>krypto`s</grk> hidden.]</ety> <fld>(Chem.)</fld> <def>One of the quinoline bases, obtained from coal tar as an oily liquid, <chform>C11H11N</chform>; also, any one of several substances metameric with, and resembling, cryptidine proper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptobranchiata</ent><br/
||<hw>Cryp`to*bran`chi*a"ta</hw> <pr>(kr?p`t?-br<it>a</it>?`k?-?"t?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>krypto`s</grk> hidden + L. <ets>branchia</ets> a gill.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A division of the Amphibia; the Derotremata.</def> <sd>(b)</sd> <def>A group of nudibranch mollusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptobranchiate</ent><br/
<hw>Cryp`to*bran"chi*ate</hw> <pr>(-br??"k?-?t)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having concealed or rudimentary gills.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptocercus</ent><br/
<hw>Cryptocercus</hw> <pos>n.</pos> <def>a genus of cockroaches.</def><br/
<syn><b>Syn. --</b> genus <gen>Cryptocercus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cryptocoryne</ent><br/
<hw>cryptocoryne</hw> <pos>n.</pos> <def>any plant of the genus Cryptocoryne; evergreen perennials growing in fresh or brackish water; tropical Asia.</def><br/
<syn><b>Syn. --</b> water trumpet.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cryptocrystalline</ent><br/
<hw>Cryp`to*crys"tal*line</hw> <pr>(-kr?s"t<it>a</it>l-l?n)</pr>, <pos>a.</pos> <ety>[Gr. <grk>krypto`s</grk> hidden + E. <ets>crystalline</ets>.]</ety> <fld>(Geol.)</fld> <def>Indistinctly crystalline; -- applied to rocks and minerals, whose state of aggregation is so fine that no distinct particles are visible, even under the microscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptogam</ent><br/
<hw>Cryp"to*gam</hw> <pr>(kr?p"t?-g?m)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>cryptogame</ets>. See <er>Cryptogamia</er>.]</ety> <fld>(Bot.)</fld> <def>A plant belonging to the Cryptogamia.</def>  <rj><au>Henslow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptogamia</ent><br/
||<hw>Cryp`to*ga"mi*a</hw> <pr>(kr?p`t?-g?"m?-?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cryptogami<ae/</plw> <pr>(-<?/)</pr>.</plu> <ety>[NL., fr. Gr. <grk>krypto`s</grk> hidden, secret + <grk>ga`mos</grk> marriage.]</ety> <fld>(Bot.)</fld> <def>The series or division of flowerless plants, or those never having true stamens and pistils, but propagated by spores of various kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The subdivisions have been variously arranged. The following arrangement recognizes four classes: --</p>

<p>I. <stype><er>Pteridophyta</er>, or <er>Vascular Acrogens</er>.</stype> These include <xex>Ferns</xex>, <ord>Equiseta</ord> or Scouring rushes, <ord>Lycopodiace<ae/</ord> or Club mosses, <ord>Selaginelle<ae/</ord>, and several other smaller orders. Here belonged also the extinct coal plants called <gen>Lepidodendron</gen>, <gen>Sigillaria</gen>, and <gen>Calamites</gen>.</p>

<p>II. <stype><er>Bryophita</er>, or <er>Cellular Acrogens</er></stype>. These include <ord>Musci</ord>, or Mosses, <ord>Hepatic<ae/</ord>, or Scale mosses and Liverworts, and possibly <ord>Charace<ae/</ord>, the Stoneworts.</p>

<p>III. <stype><er>Alg<ae/</er></stype>, which are divided into <er>Floride<ae/</er>, the Red Seaweeds, and the orders <ord>Dictyote<ae/</ord>, <ord>Oospore<ae/</ord>, <ord>Zoospore<ae/</ord>, <ord>Conjugat<ae/</ord>, <ord>Diatomace<ae/</ord>, and <ord>Cryptophyce<ae/</ord>.</p>

<p>IV. <stype><er>Fungi</er></stype>.  The molds, mildews, mushrooms, puffballs, etc., which are variously grouped into several subclasses and many orders. The <ord>Lichenes</ord> or Lichens are now considered to be of a mixed nature, each plant partly a Fungus and partly an Alga.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptogamous</ent><br/
<ent>Cryptogamic</ent><br/
<ent>Cryptogamian</ent><br/
<mhw>{ <hw>Cryp`to*ga"mi*an</hw> <pr>(kr?p`t?-g?"m?-<ait/n)</pr>, <hw>Cryp`to*gam"ic</hw> <pr>(kr?p`t?-g?m"?k)</pr>, <hw>Cryp*to"gam*ous</hw> <pr>(#)</pr> }</mhw> <pos>a.</pos> <def>Of or pertaining to the series Cryptogamia, or to plants of that series.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptogamist</ent><br/
<hw>Cryp*tog"a*mist</hw> <pr>(-m?st)</pr>, <pos>n.</pos> <def>One skilled in cryptogamic botany.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptogram</ent><br/
<hw>Cryp"to*gram</hw> <pr>(kr?p"t?-gr?m)</pr>, <pos>n.</pos> <def>A cipher writing. Same as <er>Cryptograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptogramma</ent><br/
<hw>Cryptogramma</hw> <pos>n.</pos> <def>a genus sometimes placed in the family <fam>Polypodiaceae</fam> or <fam>Cryptogrammataceae</fam>.</def><br/
<syn><b>Syn. --</b> genus Cryptogramma.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cryptogrammataceae</ent><br/
<hw>Cryptogrammataceae</hw> <pos>n.</pos> <def>one of a number of families into which the family <fam>Polypodiaceae</fam> has been subdivided in some classification systems.</def><br/
<syn><b>Syn. --</b> family <fam>Cryptogrammataceae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cryptograph</ent><br/
<hw>Cryp"to*graph</hw> <pr>(-gr?f)</pr>, <pos>n.</pos> <ety>[Gr. <grk>krypto`s</grk> hidden + <ets>-graph</ets>: cf. F.  <ets>cryptographe</ets>.]</ety> <def>Cipher; something written in cipher.</def> <ldquo/Decipherers of <xex>cryptograph</xex>.<rdquo/  <rj><au>J. Earle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptographal</ent><br/
<hw>Cryp*tog"ra*phal</hw> <pr>(kr?p-t?g"r?-f<it>a</it>l)</pr>, <pos>a.</pos> <def>Pertaining to cryptography; cryptographical.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cryptographer</ent><br/
<hw>cryp*tog"ra*pher</hw> <pr>(kr?p-t?g"r?-f?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who writes in cipher, or secret characters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>one who studies methods for encoding and decoding messages; one who studies cryptography.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cryptographical</ent><br/
<ent>Cryptographic</ent><br/
<mhw>{ <hw>Cryp`to*graph"ic</hw> <pr>(kr?p`t?-gr?f"?k)</pr>, <hw>Cryp`to*graph"ic*al</hw> <pr>(kr?p`t?-gr?f"?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <def>Relating to cryptography; written in secret characters or in cipher, or with sympathetic ink.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptographist</ent><br/
<hw>Cryp*tog"ra*phist</hw> <pr>(kr?p-t?g"r?-f?st)</pr>, <pos>n.</pos> <def>Same as <er>Cryptographer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptography</ent><br/
<hw>Cryp*tog"ra*phy</hw> <pr>(-f?)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>cryptographie</ets>.]</ety> <sn>1.</sn> <def>The act or art of writing in code or secret characters; also, secret characters, codes or ciphers, or messages written in a secret code.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>The science which studies methods for encoding messages so that they can be read only by a person who knows the secret information required for decoding, called the <part>key</part>; it includes <er>cryptanalysis</er>, the science of decoding encrypted messages without possessing the proper key, and has several other branches; see for example <er>steganography</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cryptology</ent><br/
<hw>Cryp*tol"o*gy</hw> <pr>(kr?p-t?l"?-j?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>krypto`s</grk> hidden + <ets>-logy</ets>.]</ety> <def>Secret or enigmatical language.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptonym</ent><br/
<hw>Cryp"to*nym</hw> <pr>(kr?p"t?-n?m)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ secret + <?/<?/<?/<?/ name.]</ety> <def>A secret name; a name by which a person is known only to the initiated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cryptopine</ent><br/
<hw>Cryp"to*pine</hw> <pr>(kr?p"t?-p?n; 104)</pr>, <pos>n.</pos> <ety>[Gr. <grk>krypto`s</grk> hidden + E. <ets>op</ets>ium.]</ety> <fld>(Chem.)</fld> <def>A colorless crystalline alkaloid obtained in small quantities from opium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crypturi</ent><br/
||<hw>Cryp*tu"ri</hw> <pr>(kr<icr/p*t<umac/"r<imac/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>krypto`s</grk> hidden + <grk>o'yra`</grk> tail.]</ety> <fld>(Zool.)</fld> <def>An order of flying, drom<ae/ognathous birds, including the tinamous of South America. See <er>Tinamou</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystal</ent><br/
<hw>Crys"tal</hw> <pr>(kr<icr/s"t<ait/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>cristal</ets>, F.  <ets>cristal</ets>, L.  <ets>crystallum</ets> crystal, ice, fr. Gr. <grk>kry`stallos</grk>, fr. <grk>kry`os</grk> icy cold, frost; cf. AS. <ets>crystalla</ets>, fr. L.  <ets>crystallum</ets>; prob. akin to E.  <ets>crust</ets>.  See <er>Crust</er>, <er>Raw</er>.]</ety> <sn>1.</sn> <fld>(Chem. & Min.)</fld> <def>The regular form which a substance tends to assume in solidifying, through the inherent power of cohesive attraction. It is bounded by plane surfaces, symmetrically arranged, and each species of crystal has fixed axial ratios. See <er>Crystallization</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The material of quartz, in crystallization transparent or nearly so, and either colorless or slightly tinged with gray, or the like; -- called also <altname>rock crystal</altname>. Ornamental vessels are made of it. Cf. <er>Smoky quartz</er>, <er>Pebble</er>; also <cref>Brazilian pebble</cref>, under <er>Brazilian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A species of glass, more perfect in its composition and manufacture than common glass, and often cut into ornamental forms. See <er>Flint glass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The glass over the dial of a watch case.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything resembling crystal, as clear water, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The blue <qex>crystal</qex> of the seas.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blood crystal</b></col>. <cd>See under <er>Blood</er>.</cd> -- <col><b>Compound crystal</b></col>. <cd>See under <er>Compound</er>.</cd> -- <col><b>Iceland crystal</b></col>, <cd>a transparent variety of calcite, or crystallized calcium carbonate, brought from Iceland, and used in certain optical instruments, as the polariscope.</cd> -- <mcol><col><b>Rock crystal</b></col>, <it>or</it> <col><b>Mountain crystal</b></col></mcol>, <cd>any transparent crystal of quartz, particularly of limpid or colorless quartz.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystal</ent><br/
<hw>Crys"tal</hw>, <pos>a.</pos> <def>Consisting of, or like, crystal; clear; transparent; lucid; pellucid; crystalline.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through <qex>crystal</qex> walls each little mote will peep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>crystal</qex> streams that murmur through the meads.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>crystal</qex> pellets at the touch congeal,<br/
And from the ground rebounds the ratting hail.</q> <rj><qau>H. Brooks.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallin</ent><br/
<hw>Crys"tal*lin</hw> <pr>(-l?n)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>See <er>Gobulin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystalline</ent><br/
<hw>Crys"tal*line</hw> <pr>(kr?s"t<it>a</it>l-l?n <or/ -l?n; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>crystallinus</ets>, from Gr. <?/<?/<?/<?/: cf. F.  <ets>cristallin</ets>. See <er>Crystal</er>.]</ety> <sn>1.</sn> <def>Consisting, or made, of crystal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mount, eagle, to my palace <qex>crystalline</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Formed by crystallization; like crystal in texture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>crystalline</qex> structure.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Imperfectly crystallized; <as>as, granite is only <ex>crystalline</ex>, while quartz crystal is perfectly crystallized</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: Resembling crystal; pure; transparent; pellucid.</def> <ldquo/The <xex>crystalline</xex> sky.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Crystalline heavens</b></col>, <it>or</it> <col><b>Crystalline spheres</b></col></mcol>, <cd>in the Ptolemaic system of astronomy, two transparent spheres imagined to exist between the region of the fixed stars and the <xex>primum mobile</xex> (or outer circle of the heavens, which by its motion was supposed to carry round all those within it), in order to explain certain movements of the heavenly bodies.</cd> -- <col><b>Crystalline lens</b></col> <fld>(Anat.)</fld>, <cd>the capsular lenslike body in the eye, serving to focus the rays of light. It consists of rodlike cells derived from the external embryonic epithelium.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystalline</ent><br/
<hw>Crys"tal*line</hw>, <pos>n.</pos> <sn>1.</sn> <def>A crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>See <er>Aniline</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallite</ent><br/
<hw>Crys"tal*lite</hw> <pr>(kr?s"t<it>a</it>l-l?t)</pr>, <pos>n.</pos> <ety>[See <er>Crystal</er>.]</ety> <fld>(Min.)</fld> <def>A minute mineral form like those common in glassy volcanic rocks and some slags, not having a definite crystalline outline and not referable to any mineral species, but marking the first step in the crystallization process. According to their form crystallites are called <stype>trichites</stype>, <stype>belonites</stype>, <stype>globulites</stype>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 353 --></p>

<p><ent>Crystallizable</ent><br/
<hw>Crys"tal*li`za*ble</hw> <pr>(kr<icr/s"t<ait/l*l<imac/`z<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being crystallized; that may be formed into crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallization</ent><br/
<hw>Crys`tal*li*za"tion</hw> <pr>(kr<icr/s`t<ait/l*l<icr/*z<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cristallization</ets>.]</ety> <sn>1.</sn> <fld>(Chem. & Min.)</fld> <def>The act or process by which a substance in solidifying assumes the form and structure of a crystal, or becomes crystallized; the formation of crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The body formed by crystallizing; <as>as, silver on precipitation forms arborescent <ex>crystallizations</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The systems of crystallization are the several classes to which the forms are mathematically referable. They are most simply described according to the relative lengths and inclinations of certain assumed lines called axes; but the real distinction is the degree of symmetry characterizing them. 1. <stype><colf>The Isometric system</colf></stype>, <it>or</it> <stype><colf>The Monometric system</colf></stype> has the axes all equal, as in the cube, octahedron, etc. 2. <stype><colf>The Tetragonal system</colf></stype>, <it>or</it> <stype><colf>The Dimetric system</colf></stype> has a varying vertical axis, while the lateral are equal, as in the right square prism. 3. <stype><colf>The Orthorhombic system</colf></stype>, <it>or</it> <stype><colf>The Trimetric system</colf></stype> has the three axes unequal, as in the rectangular and rhombic prism.  In this system, the lateral axes are called, respectively, <xex>macrodiagonal</xex> and <xex>brachydiagonal</xex>. -- The preceding are erect forms, the axes intersecting at right angles.  The following are oblique. 4. <stype><colf>The Monoclinic system</colf></stype>, having one of the intersections oblique, as in the oblique rhombic prism. In this system, the lateral axes are called respectively, <xex>clinodiagonal</xex> and <xex>orthodiagonal</xex>. 5. <stype><colf>The Triclinic system</colf></stype>, having all the three intersections oblique, as in the oblique rhomboidal prism. There is also: 6. <stype><colf>The Hexagonal system</colf></stype> (one division of which is called <xex>Rhombohedral</xex>), in which there are three equal lateral axes, and a vertical axis of variable length, as in the hexagonal prism and the rhombohedron.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>The Diclinic system</xex>, sometimes recognized, with two oblique intersections, is only a variety of the <xex>Triclinic</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallize</ent><br/
<hw>Crys"tal*lize</hw> <pr>(kr<icr/s"t<ait/l*l<imac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Crystallized</conjf> <pr>(-l<imac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Crystallizing</conjf>.]</vmorph> <ety>[Cf. F.  <ets>cristalliser</ets>. See <er>Crystal</er>.]</ety> <def>To cause to form crystals, or to assume the crystalline form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>crystallize</ent><br/
<hw>crys"tal*lize</hw>, <pos>v. i.</pos> <def>To be converted into a crystal; to take on a crystalline form, through the action of crystallogenic or cohesive attraction; to precipitate from a solution in the form of crystals.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>crystallized</ent><br/
<hw>crystallized</hw> <pos>adj.</pos> <sn>1.</sn>  <def>smoothly coated with crystals of sugar; -- used especially of fruits.</def><br/
<syn><b>Syn. --</b> candied, glac<eacute/, glac<eacute/ed.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>caused to form into crystals; hence, in a crystalline form.</def><br/
[<source>PJC</source>]</p>

<p><ent>Crystallogenical</ent><br/
<ent>Crystallogenic</ent><br/
<mhw>{ <hw>Crys`tal*lo*gen"ic</hw> <pr>(-l?-j?n"?k)</pr>, <hw>Crys`tal*lo*gen"ic*al</hw> <pr>(-?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <def>Pertaining to the production of crystals; crystal-producing; <as>as, <ex>crystallogenic</ex> attraction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallogeny</ent><br/
<hw>Crys`tal*log"e*ny</hw> <pr>(kr?s`t<it>a</it>l-l?j"?-n?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + root of <grk>gi`gnesqai</grk> to be born.]</ety> <def>The science which pertains to the production of crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallographer</ent><br/
<hw>Crys`tal*log"ra*pher</hw> <pr>(kr<icr/s"t<ait/l*l<ocr/g"r<adot/*f<etil/r)</pr>, <pos>n.</pos> <def>One who describes crystals, or the manner of their formation; one versed in crystallography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallographical</ent><br/
<ent>Crystallographic</ent><br/
<mhw>{ <hw>Crys`tal*lo*graph"ic</hw> <pr>(-l?-gr?f"?k)</pr>, <hw>Crys`tal*lo*graph"ic*al</hw> <pr>(-?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>crystallographique</ets>.]</ety> <def>Pertaining to crystallography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallographically</ent><br/
<hw>Crys`tal*lo*graph"ic*al*ly</hw>, <pos>adv.</pos> <def>In the manner of crystallography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallography</ent><br/
<hw>Crys`tal*log"ra*phy</hw> <pr>(kr<icr/s"t<ait/l*l<ocr/g"r<adot/*f<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + <ets>-graphy</ets>: cf. F.  <ets>cristallographie</ets>. See <er>Crystal</er>.]</ety> <sn>1.</sn> <def>The doctrine or science of crystallization, teaching the system of forms among crystals, their structure, and their methods of formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A discourse or treatise on crystallization.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystalloid</ent><br/
<hw>Crys"tal*loid</hw> <pr>(kr<icr/s"t<ait/l*loid)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + <ets>-oid</ets>.]</ety> <def>Crystal-like; transparent like crystal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystalloid</ent><br/
<hw>Crys"tal*loid</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>A body which, in solution, diffuses readily through animal membranes, and generally is capable of being crystallized; -- opposed to <xex>colloid</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>One of the microscopic particles resembling crystals, consisting of protein matter, which occur in certain plant cells; -- called also <altname>protein crystal</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallology</ent><br/
<hw>Crys`tal*lol"o*gy</hw> <pr>(kr<icr/s`t<ait/l*l<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + <ets>-logy</ets>.]</ety> <def>The science of the crystalline structure of inorganic bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallomancy</ent><br/
<hw>Crys"tal*lo*man`cy</hw> <pr>(-l?-m?n`s?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + <ets>-mancy</ets>.]</ety> <def>Divination by means of a crystal or other transparent body, especially a beryl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallometry</ent><br/
<hw>Crys`tal*lom"e*try</hw> <pr>(-l?m"?-tr?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + <ets>-metry</ets>.]</ety> <def>The art of measuring crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Crystallurgy</ent><br/
<hw>Crys"tal*lur`gy</hw> <pr>(-l?r`j?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kry`stallos</grk> crystal + <grk>'e`rgon</grk> work.]</ety> <def>Crystallization.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>CSA</ent><br/
<hw>CSA</hw> <pos>prop. n.</pos> <def>acronym for the <er>Confederate States of America</er>.</def><br/
<syn><b>Syn. --</b> Confederate States of America, the confederacy.</syn><br/
[<source>PJC</source>]</p>

<p><ent>C-section</ent><br/
<hw>C-section</hw> <pos>n.</pos> <def>contraction of <er>caesarean section</er>.</def><br/
<syn><b>Syn. --</b> cesarean section, cesarian section, , cesarean, cesarian, caesarean.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Ctenocyst</ent><br/
<hw>Cte"no*cyst</hw> <pr>(t<emac/"n<osl/*s<icr/st)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ktei`s</grk>, <grk>kteno`s</grk>, comb + <grk>ky`stis</grk> bladder.]</ety> <fld>(Zool.)</fld> <def>An organ of the Ctenophora, supposed to be sensory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ctenoid</ent><br/
<hw>Cte"noid</hw> <pr>(t<emac/"noid <it>or</it> t<ecr/n"oid)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having a comblike margin, as a ctenoid scale</def>. <sd>(b)</sd> <def>Pertaining to the Ctenoidei.</def> -- <def2><pos>n.</pos> <def>A ctenoidean.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ctenoidean</ent><br/
<hw>Cte*noid"e*an</hw> <pr>(t<esl/*noid"<esl/*<ait/n)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Relating to the Ctenoidei.</def> -- <def2><pos>n.</pos> <def>One of the Ctenoidei.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ctenoidei</ent><br/
||<hw>Cte*noid"e*i</hw> <pr>(-?-?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/, <?/<?/<?/<?/, comb + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>A group of fishes, established by Agassiz, characterized by having scales with a pectinated margin, as in the perch. The group is now generally regarded as artificial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ctenophora</ent><br/
||<hw>Cte*noph"o*ra</hw> <pr>(t?-n?f"?-r?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/<?/<?/, <?/<?/<?/, comb + <?/<?/<?/<?/ to carry.]</ety> <fld>(Zool.)</fld> <def>A phylum of invertebrates, commonly ellipsoidal in shape, swimming by means of eight longitudinal rows of paddles.  They are commonly called the <altname>comb jellies</altname>, because the separate paddles somewhat resemble combs.  This phylum was formerly classified as a subdivision (class) within the <phylum>C<oe/lenterata</phylum>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Ctenophore</ent><br/
<hw>Cten"o*phore</hw> <pr>(t?n"?-f?r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Ctenophora.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ctenophorous</ent><br/
<ent>Ctenophoric</ent><br/
<mhw>{ <hw>Cten`o*phor"ic</hw> <pr>(t?n`?-f?r"?k)</pr>, <hw>Cte*noph"o*rous</hw> <pr>(t?-n?f"?-r?s)</pr>, }</mhw> <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Ctenophora.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ctenostomata</ent><br/
||<hw>Cten`o*stom"a*ta</hw> <pr>(t?n`?-st?m"?-t?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/, <?/<?/<?/, comb + <?/<?/<?/<?/, -<?/<?/<?/ mouth.]</ety> <fld>(Zool.)</fld> <def>A suborder of Bryozoa, usually having a circle of bristles below the tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cu</ent><br/
<hw>Cu</hw> <pos>n.</pos> <def>the chemical symbol for <er>copper</er>.</def><br/
<syn><b>Syn. --</b> copper.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cub</ent><br/
<hw>Cub</hw> <pr>(k<ucr/b)</pr>, <pos>n.</pos> <ety>[Cf. Ir.  <ets>cuib</ets> cub, whelp, young dog, Ir. & Gael. <ets>cu</ets> dog; akin to E. <ets>hound</ets>.]</ety> <sn>1.</sn> <def>A young animal, esp. the young of the bear.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Jocosely or in contempt, a boy or girl, esp. an awkward, rude, ill-mannered boy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, thou dissembling <qex>cub</qex>! what wilt thou be<br/
When time hath sowed a grizzle on thy case?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cub</ent><br/
<hw>Cub</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cubbed</conjf> (k<ucr/bd); <pos>p. pr. & vb. n.</pos> <conjf>Cubbing</conjf>.]</vmorph> <def>To bring forth; -- said of animals, or in contempt, of persons.</def> <ldquo/<xex>Cubb'd</xex> in a cabin.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cub</ent><br/
<hw>Cub</hw>, <pos>n.</pos> <ety>[Cf. <er>Cub</er> a young animal.]</ety> <sn>1.</sn> <def>A stall for cattle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I would rather have such . . . .in <qex>cub</qex> or kennel than in my closet or at my table.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cupboard.</def> <mark>[Obs.]</mark>  <rj><au>Laud.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cub</ent><br/
<hw>Cub</hw>, <pos>v. t.</pos> <def>To shut up or confine.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuba</ent><br/
<hw>Cuba</hw> <pr>(k<umac/"b<adot/)</pr>, <pos>prop. n.</pos> <sn>1.</sn> <def>a country on the island of Cuba.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the largest island in the West Indies.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cuban</ent><br/
<hw>Cu"ban</hw> <pr>(k<umac/"b<ait/n)</pr>, <pos>prop. a.</pos> <def>Of or pertaining to Cuba or its inhabitants.</def> -- <def2><pos>n.</pos> <def>A native or an inhabitant of Cuba.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubation</ent><br/
<hw>Cu*ba"tion</hw> <pr>(k?-b?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>cubatio</ets>, fr. <ets>cubare</ets> to lie down.]</ety> <def>The act of lying down; a reclining.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubatory</ent><br/
<hw>Cu"ba*to*ry</hw> <pr>(k?"b?-t?-r?)</pr>, <pos>a.</pos> <ety>[L. <ets>cubator</ets> he who lies down, fr.  <ets>cubare</ets>.]</ety> <def>Lying down; recumbent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubature</ent><br/
<hw>Cu"ba*ture</hw> <pr>(k?"b?-t?r; 135)</pr>, <pos>n.</pos> <ety>[L.  <ets>cubus</ets> cube: cf. F.  <ets>cubature</ets>.  See <er>Cube</er>.]</ety> <def>The process of determining the solid or cubic contents of a body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubbridge-head</ent><br/
<hw>Cub"bridge-head`</hw> <pr>(k?b"r?jj-h?d)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A bulkhead on the forecastle and half deck of a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubbyhole</ent><br/
<ent>Cubby</ent><br/
<mhw>{ <hw>Cub"by</hw> <pr>(k?b"b?)</pr>, <hw>Cub"by*hole`</hw> <pr>(-h?l`)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Cub</er> a stall.]</ety> <def>A snug or confined or secluded place; a small room or a snug space within a room.</def><br/
<syn><b>Syn. --</b> snuggery, snug.</syn><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>cubbyhole</ent><br/
<hw>cubbyhole</hw> <pos>n.</pos> <def>a small compartment.</def><br/
<syn><b>Syn. --</b> pigeonhole.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cubdrawn</ent><br/
<hw>Cub"*drawn`</hw> <pr>(k?b"dr?n`)</pr>, <pos>a.</pos> <def>Sucked by cubs.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This night, wherein the <qex>cub-drawn</qex> bear would couch.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cube</ent><br/
<hw>Cube</hw> <pr>(k<umac/b)</pr>, <pos>n.</pos> <ety>[F. <ets>cube</ets>, L.  <ets>cubus</ets>, fr. Gr. <?/<?/<?/<?/ a cube, a cubical die.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A regular solid body, with six equal square sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>The product obtained by taking a number or quantity three times as a factor; <as>as, <mathex>4x4=16</mathex>, and <mathex>16x4=64</mathex>, the cube of 4</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cube ore</b></col> <fld>(Min.)</fld>, <cd>pharmacosiderite. It commonly crystallizes in cubes of a green color.</cd> -- <col><b>Cube root</b></col>. <fld>(Math.)</fld>, <cd>the number or quantity which, multiplied into itself, and then into the product, produces the given cube; thus, 3 is the cube root of 27, for <mathex>3x3x3 = 27</mathex>.</cd> -- <col><b>Cube spar</b></col> <fld>(Min.)</fld>, <cd>anhydrite; anhydrous calcium sulphate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cube</ent><br/
<hw>Cube</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cubed</conjf> <pr>(k<umac/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cubing</conjf>.]</vmorph> <def>To raise to the third power; to obtain the cube of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubeb</ent><br/
<hw>Cu"beb</hw> <pr>(k<umac/"b<ecr/b)</pr>, <pos>n.</pos> <ety>[F. <ets>cub<egrave/be</ets> (cf. It.  <ets>cubebe</ets>, Pr., Sp., Pg., & NL.  <ets>cubeba</ets>), fr. Ar. <ets>kab<amac/bat</ets>.]</ety> <def>The small, spicy berry of a species of pepper (<spn>Piper Cubeba</spn>; in <fld>(Med.)</fld>, <spn>Cubeba officinalis</spn>), native in Java and Borneo, but now cultivated in various tropical countries.  The dried unripe fruit is much used in medicine as a stimulant and purgative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cubebic</ent><br/
<hw>cu*beb"ic</hw> <pr>(k?-b?b"?k)</pr>, <pos>a.</pos> <def>Pertaining to, or derived from, cubebs; <as>as, <ex>cubebic</ex> acid (a soft olive-green resin extracted from cubebs)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cubical</ent><br/
<ent>cube-shaped</ent><br/
<mhw><hw>cube-shaped</hw> <hw>cubical</hw></mhw> <pos>adj.</pos> <def>Shaped like or approximately like a cube.</def><br/
<syn><b>Syn. --</b> cubelike, cubiform, cuboid, cuboidal.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cubhood</ent><br/
<hw>Cub"hood</hw> <pr>(k?b"h??d)</pr>, <pos>n.</pos> <def>The state of being a cub.</def> <mark>[Jocose]</mark> <ldquo/From <xex>cubhood</xex> to old age.<rdquo/  <rj><au>W. B. Dawkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubical</ent><br/
<ent>Cubic</ent><br/
<mhw>{ <hw>Cu"bic</hw> <pr>(k?"b?k)</pr>, <hw>Cu"bic*al</hw> <pr>(-b?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[L.  <ets>cubicus</ets>, Gr. <?/<?/<?/<?/<?/: cf. F. <ets>cubique</ets>. See <er>Cube</er>.]</ety> <sn>1.</sn> <def>Having the form or properties of a cube; contained, or capable of being contained, in a cube.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Crystallog.)</fld> <def>Isometric or monometric; <as>as, <ex>cubic</ex> cleavage</as>. See <er>Crystallization</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cubic equation</b></col>, <cd>an equation in which the highest power of the unknown quantity is a cube.</cd> -- <col><b>Cubic foot</b></col>, <cd>a volume equivalent to a cubical solid which measures a foot in each of its dimensions.</cd> -- <col><b>Cubic number</b></col>, <cd>a number produced by multiplying a number into itself, and that product again by the same number. See <er>Cube</er>.</cd> -- <col><b>Cubical parabola</b></col> <fld>(Geom.)</fld>, <cd>two curves of the third degree, one plane, and one on space of three dimensions.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubic</ent><br/
<hw>Cu"bic</hw>, <pos>n.</pos> <fld>(Geom.)</fld> <def>A curve of the third degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Circular cubic</b></col>. <cd>See under <er>Circular</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubically</ent><br/
<hw>Cu"bic*al*ly</hw>, <pos>adv.</pos> <def>In a cubical method.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubicalness</ent><br/
<hw>Cu"bic*al*ness</hw>, <pos>n.</pos> <def>The quality of being cubical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubicle</ent><br/
<hw>Cu"bi*cle</hw> <pr>(k?"b?-k'l)</pr>, <pos>n.</pos> <ety>[L. <ets>cubiculum</ets>.]</ety> <def>A loding room; esp., a sleeping place partitioned off from a large dormitory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubicular</ent><br/
<hw>Cu*bic"u*lar</hw> <pr>(k?-??k"?-l?r)</pr>, <pos>a.</pos> <ety>[L. <ets>cubicularis</ets>, fr. <ets>cubiculum</ets> a sleeping room, fr.  <ets>cubare</ets> to lie down.]</ety> <def>Belonging to a chamber or bedroom.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubiform</ent><br/
<hw>Cu"bi*form</hw> <pr>(k?"b?-f?rm)</pr>, <pos>a.</pos> <def>Of the form of a cube.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubile</ent><br/
||<hw>Cu*bi"le</hw> <pr>(k?-b?"l?)</pr>, <pos>n.</pos> <ety>[L., bed.]</ety> <def>The lowest course of stones in a building.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubilose</ent><br/
<hw>Cu"bi*lose`</hw> <pr>(k?"b?-l?s`)</pr>, <pos>n.</pos> <ety>[L.  <ets>cubile</ets> bed, nest.]</ety> <def>A mucilagenous secretion of certain birds found as the characteristic ingredient of edible bird's-nests.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubism</ent><br/
<hw>Cu"bism</hw> <pr>(k<umac/"b<icr/z'm)</pr>, <pos>n.</pos> <fld>(Painting)</fld> <def>A movement or phase in post-impressionism (which see, below).</def> -- <wordforms><wf>Cu"bist</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cubit</ent><br/
<hw>Cu"bit</hw> <pr>(k<umac/"b<icr/t)</pr>, <pos>n.</pos> <ety>[L.  <ets>cubitum</ets>, <ets>cubitus</ets>; elbow, ell, cubit, fr. (because the elbow serves for leaning upon) <ets>cubare</ets> to lie down, recline; cf. Gr. <grk>ky`biton</grk> elbow, <grk>ky`ptein</grk> to bend, stoop, <grk>kyfo`s</grk> bent, stooping, humpbacked. Cf. <er>Incumbent</er>, <er>Covey</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The forearm; the ulna, a bone of the arm extending from elbow to wrist.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A measure of length, being the distance from the elbow to the extremity of the middle finger.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The cubit varies in length in different countries, the Roman cubit being 17.47 inches, the Greek 18.20, the Hebrew somewhat longer, and the English 18 inches.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubital</ent><br/
<hw>Cu"bit*al</hw> <pr>(k?"b?t-<ait/l)</pr>, <pos>a.</pos> <ety>[L.  <ets>cubitalis</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to the cubit or ulna; <as>as, the <ex>cubital</ex> nerve; the <ex>cubital</ex> artery; the <ex>cubital</ex> muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of the length of a cubit.</def>  <rj><au>Sir. T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubital</ent><br/
<hw>Cu"bit*al</hw>, <pos>n.</pos> <def>A sleeve covering the arm from the elbow to the hand.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubited</ent><br/
<hw>Cu"bit*ed</hw>, <pos>a.</pos> <def>Having the measure of a cubit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubless</ent><br/
<hw>Cub"less</hw> <pr>(k?b"l?s)</pr>, <pos>a.</pos> <def>Having no cubs.</def>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuboid</ent><br/
<hw>Cu"boid</hw> <pr>(k?"boid)</pr>, <pos>a.</pos> <ety>[<ets>Cube</ets> + <ets>-oid</ets>: cf. Gr. <?/<?/<?/.]</ety> <fld>(Anat.)</fld> <def>Cube-shaped, or nearly so; <as>as, the <ex>cuboid</ex> bone of the foot</as>.</def> -- <def2><pos>n.</pos> <fld>(Anat.)</fld> <def>The bone of the tarsus, which, in man and most mammals, supports the metatarsals of the fourth and fifth toes.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuboidal</ent><br/
<hw>Cu*boid"al</hw> <pr>(k?-boid"<it>a</it>l)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Cuboid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubo-octahedral</ent><br/
<hw>Cu`bo-oc`ta*he"dral</hw> <pr>(k?`b?-?k`t?-h?dr<it>a</it>l)</pr>, <pos>a.</pos> <def>Presenting a combination of a cube and an octahedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cubo-octahedron</ent><br/
<hw>Cu`bo-oc`ta*he"dron</hw> <pr>(-dr?n)</pr>, <pos>n.</pos> <fld>(Crystallog.)</fld> <def>A combination of a cube and octahedron, esp. one in which the octahedral faces meet at the middle of the cubic edges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuca</ent><br/
||<hw>Cu"ca</hw> <pr>(k??"k?)</pr>, <pos>n.</pos> <ety>[Sp., fr. native name.]</ety> <def>See <er>Coca</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucking stool</ent><br/
<hw>Cuck"ing stool`</hw> <pr>(k<?/k"<?/ng st<?/<?/l`)</pr>. <ety>[Cf. AS. <ets>scealfingst<omac/l</ets>, a word of similar meaning, allied to <ets>scealfor</ets> a diver, mergus avis; or possibly from F. <ets>coquine</ets> a hussy, slut, jade, f. of <ets>coquin</ets>, OE.  <ets>cokin</ets>, a rascal; or cf. Icel. <ets>k<?/ka</ets> to dung, <ets>k<?/kr</ets> dung, the name being given as to a disgracing or infamous punishment.]</ety> <def>A kind of chair formerly used for punishing scolds, and also dishonest tradesmen, by fastening them in it, usually in front of their doors, to be pelted and hooted at by the mob, but sometimes to be taken to the water and ducked; -- called also a <altname>castigatory</altname>, a <altname>tumbrel</altname>, and a <altname>trebuchet</altname>; and often, but not so correctly, a <altname>ducking stool</altname>.</def>  <rj><au>Sir. W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckold</ent><br/
<hw>Cuck"old</hw> <pr>(k?k"?ld)</pr>, <pos>n.</pos> <ety>[OE. <ets>kukeweld</ets>, <ets>cokewold</ets>, <ets>cokold</ets>, fr. OF.  <ets>coucoul</ets>, <ets>cucuault</ets>, the last syllable being modified by the OE. suffix <ets>-wold</ets> (see <er>Herald</er>); cf. F. <ets>cocu</ets> a cuckold, formerly also, a cuckoo, and L.  <ets>cuculus</ets> a cuckoo. The word alludes to the habit of the female cuckoo, who lays her eggs in the nests of other birds, to be hatched by them.]</ety> <sn>1.</sn> <def>A man whose wife is unfaithful; the husband of an adulteress.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A West Indian plectognath fish (<spn>Ostracion triqueter</spn>).</def> <sd>(b)</sd> <def>The cowfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckold</ent><br/
<hw>Cuck"old</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cuckolded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cuckolding</conjf>.]</vmorph> <def>To make a cuckold of, as a husband, by seducing his wife, or by her becoming an adulteress.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckoldize</ent><br/
<hw>Cuck"old*ize</hw> <pr>(-?z)</pr>, <pos>v. t.</pos> <def>To cuckold.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckoldly</ent><br/
<hw>Cuck"old*ly</hw>, <pos>a.</pos> <def>Having the qualities of a cuckold; mean-spirited; sneaking.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckoldom</ent><br/
<hw>Cuck"ol*dom</hw> <pr>(-?l-d?m)</pr>, <pos>n.</pos> <def>The state of a cuckold; cuckolds, collectively.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckoldry</ent><br/
<hw>Cuck"old*ry</hw> <pr>(-?ld-r?)</pr>, <pos>n.</pos> <def>The state of being a cuckold; the practice of making cuckolds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckold's knot</ent><br/
<hw>Cuck"old's knot`</hw> <pr>(k?k"?ldz n?t`)</pr>. <fld>(Naut.)</fld> <def>A hitch or knot, by which a rope is secured to a spar, the two parts of the rope being crossed and seized together; -- called also <altname>cuckold's neck</altname>.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckoo</ent><br/
<hw>Cuck"oo</hw> <pr>(k<oocr/k"<oomac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>coccou</ets>, <ets>cukkow</ets>, F.  <ets>coucou</ets>, prob. of imitative origin; cf. L.  <ets>cuculus</ets>, Gr. <?/<?/<?/<?/, Skr. <ets>k<?/ki<?/a</ets>, G. <ets>kuckuk</ets>, D. <ets>koekoek</ets>.]</ety> <fld>(Zool.)</fld> <def>A bird belonging to <gen>Cuculus</gen>, <gen>Coccyzus</gen>, and several allied genera, of many species.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The European cuckoo (<spn>Cuculus canorus</spn>) builds no nest of its own, but lays its eggs in the nests of other birds, to be hatched by them.  The American yellow-billed cuckoo (<spn>Coccyzus Americanus</spn>) and the black-billed cuckoo (<spn>Coccyzus erythrophthalmus</spn>) build their own nests.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cuckoo clock</b></col>, <cd>a clock so constructed that at the time for striking it gives forth sounds resembling the cry of the cuckoo.</cd> -- <col><b>Cuckoo dove</b></col> <fld>(Zool.)</fld>, <cd>a long-tailed pigeon of the genus <gen>Macropygia</gen>.  Many species inhabit the East Indies.</cd> -- <col><b>Cuckoo fish</b></col> <fld>(Zool.)</fld>, <cd>the European red gurnard (<spn>Trigla cuculus</spn>). The name probably alludes to the sound that it utters.</cd> -- <col><b>Cuckoo falcon</b></col> <fld>(Zool.)</fld>, <cd>any falcon of the genus <gen>Baza</gen>.  The genus inhabits Africa and the East Indies.</cd> -- <col><b>Cuckoo maid</b></col> <fld>(Zool.)</fld>, <cd>the wryneck; -- called also <altname>cuckoo mate</altname>.</cd> -- <col><b>Cuckoo ray</b></col> <fld>(Zool.)</fld>, <cd>a British ray (<spn>Raia miraletus</spn>).</cd> -- <mcol><col><b>Cuckoo spit</b></col>, <it>or</it> <col><b>Cuckoo spittle</b></col></mcol>. <sd>(a)</sd> <cd>A frothy secretion found upon plants, exuded by the larvae of certain insects, for concealment; -- called also <altname>toad spittle</altname> and <altname>frog spit</altname>.</cd> <sd>(b)</sd> <fld>(Zool.)</fld> <cd>A small hemipterous insect, the larva of which, living on grass and the leaves of plants, exudes this secretion. The insects belong to <gen>Aphrophora</gen>, <gen>Helochara</gen>, and allied genera.</cd> -- <col><b>Ground cuckoo</b></col>, <cd>the chaparral cock.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 354 --></p>

<p><ent>cuckoo bee</ent><br/
<hw>cuckoo bee</hw> <pos>n.</pos> <fld>(Zool.)</fld>, <def>a bee, parasitic in the larval stage in the nests of other bees, feeding either upon their food or larvae. They belong to the genera <gen>Nomada</gen>, <gen>Melecta</gen>, <gen>Epeolus</gen>, and others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cuckoo-bumblebee</ent><br/
<hw>cuckoo-bumblebee</hw> <pos>n.</pos> <sn>1.</sn> <def>a bee that is parasitic in the nests of bumblebees.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cuckoo-pint</ent><br/
<ent>cuckoopint</ent><br/
<mhw><hw>cuck"oo*pint`</hw>, <hw>cuck"oo-pint`</hw></mhw> <pos>n.</pos> <def>a common European arum (<spn>Arum maculatum</spn>) with lanceolate spathe and short purple spadix; it emerges in early spring, and is the source of a sagolike starch called <prod>arum</prod>.</def><br/
<syn><b>Syn. --</b> cuckoopint, lords and ladies, lords-and-ladies, jack-in-the-pulpit, <spn>Arum maculatum</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cuckoobud</ent><br/
<hw>Cuck"oo*bud"</hw> <pr>(k??k"??-b?d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A species of <gen>Ranunculus</gen> (<spn>Ranunculus bulbosus</spn>); -- called also <altname>butterflower</altname>, <altname>buttercup</altname>, <altname>kingcup</altname>, <altname>goldcup</altname>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckooflower</ent><br/
<hw>Cuck"oo*flow`er</hw> <pr>(-flou`?r)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A species of <gen>Cardamine</gen> (<spn>Cardamine pratensis</spn>), or lady's smock. Its leaves are used in salads. Also, the ragged robin (<spn>Lychnis Flos-cuculi</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuckoopint</ent><br/
<hw>Cuck"oo*pint`</hw> <pr>(-p?nt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Arum</gen> (<spn>Arum maculatum</spn>); the European wake-robin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucquean</ent><br/
<hw>Cuc"quean`</hw> <pr>(k?k"kw?n`)</pr>, <pos>n.</pos> <ety>[<ets>Cucko</ets>ld + <ets>quean</ets>.]</ety> <def>A woman whose husband is unfaithful to her.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucujo</ent><br/
||<hw>Cu*cu"jo</hw> <pr>(k??-k??"h?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>The fire beetle of Mexico and the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucullated</ent><br/
<ent>Cucullate</ent><br/
<mhw>{ <hw>Cu"cul*late</hw> <pr>(k?"k?l-l?t <or/ k?-k?l"l?t)</pr>, <hw>Cu"cul*la`ted</hw> <pr>(-l?`t?d <or/ -l?-t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[LL. <ets>cullatus</ets>, fr. L.  <ets>cucullus</ets> a cap, hood. See <er>Cowl</er> a hood.]</ety> <sn>1.</sn> <def>Hooded; cowled; covered, as with a hood.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having the edges toward the base rolled inward, as the leaf of the commonest American blue violet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having the prothorax elevated so as to form a sort of hood, receiving the head, as in certain insects.</def> <sd>(b)</sd> <def>Having a hoodlike crest on the head, as certain birds, mammals, and reptiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucullus</ent><br/
||<hw>Cu*cul"lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Cuculli</plw> <pr>(#)</pr></plu>. <ety>[L., a hood.]</ety> <sn>1.</sn>  <fld>(Bot.)</fld> <def>A hood-shaped organ, resembling a cowl or monk's hood, as certain concave and arched sepals or petals.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Zool.)</fld> <def>A color marking or structure on the head somewhat resembling a hood.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cuculoid</ent><br/
<hw>Cu"cu*loid</hw> <pr>(k?k?-loid)</pr>, <pos>a.</pos> <ety>[L.<ets>cuculus</ets> a cuckoo + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or belonging to the cuckoos (<fam>Cuculid<ae/</fam>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucumber</ent><br/
<hw>Cu"cum*ber</hw> <pr>(k?`k?m-b?r, <it>formerly</it> kou"k?m-b?r)</pr>, <pos>n.</pos> <ety>[OE. <ets>cucumer</ets>, <ets>cocumber</ets>, <ets>cucumber</ets>, fr. L. <ets>cucmis</ets>, gen.<ets>cucumeris</ets>; cf. OF. <ets>cocombre</ets>,F. <ets>concombre</ets>.]</ety> <fld>(Bot.)</fld> <def>A creeping plant, and its fruit, of several species of the genus <gen>Cucumis</gen>, esp. <spn>Cucumis sativus</spn>, the unripe fruit of which is eaten either fresh or picked. Also, similar plants or fruits of several other genera. See below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bitter cucumber</b></col> <fld>(Bot.)</fld>, <cd>the <spn>Citrullus Colocynthis</spn> syn. <spn>Cucumis Colocynthis</spn>. See <er>Colocynth</er>.</cd> -- <col><b>Cucumber beetle.</b></col> <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small, black flea-beetle (<spn>Crepidodera cucumeris</spn>), which destroys the leaves of cucumber, squash, and melon vines.</cd> <sd>(b)</sd> <cd>The squash beetle.</cd> -- <col><b>Cucumber tree</b></col>. <sd>(a)</sd> <cd>A large ornamental or shade tree of the genus <gen>Magnolia</gen> <spn>(Magnolia acuminata)</spn>, so called from a slight resemblance of its young fruit to a small cucumber.</cd> <sd>(b)</sd> <cd>An East Indian plant (<spn>Averrhoa Bilimbi</spn>) which produces the fruit known as bilimbi.</cd> -- <mcol><col><b>Jamaica cucumber</b></col>, <col><b>Jerusalem cucumber</b></col></mcol>, <cd>the prickly-fruited gherkin (<spn>Cucumis Anguria</spn>).</cd> -- <col><b>Snake cucumber</b></col>, <cd>a species (<spn>Cucumis flexuosus</spn>) remarkable for its long, curiously-shaped fruit.</cd> -- <col><b>Squirting cucumber</b></col>, <cd>a plant (<spn>Ecbalium Elaterium</spn>) whose small oval fruit separates from the footstalk when ripe and expels its seeds and juice with considerable force through the opening thus made. See <er>Elaterium</er>.</cd> -- <col><b>Star cucumber</b></col>, <cd>a climbing weed (<spn>Sicyos angulatus</spn>) with prickly fruit.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucumiform</ent><br/
<hw>Cu*cu"mi*form</hw> <pr>(k?-k?"m?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cucumis</ets> cucumber + <ets>-form</ets>.]</ety> <def>Having the form of a cucumber; having the form of a cylinder tapered and rounded at the ends, and either straight or curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucumis</ent><br/
||<hw>Cu"cu*mis</hw> <pr>(k?k?--m?s)</pr>, <pos>n.</pos> <ety>[L., cucumber.]</ety> <fld>(Bot.)</fld> <def>A genus of plants including the cucumber, melon, and same kinds of gourds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucurbite</ent><br/
<ent>Cucurbit</ent><br/
<mhw>{ <hw>Cu*cur"bit</hw> <hw>Cu*cur"bite</hw> }</mhw> <pr>(k<usl/*k<ucir/r"b<icr/t)</pr>, <pos>n.</pos> <ety>[L.  <ets>cucurbita</ets> a gourd: cf. F. <ets>cucurbite</ets>. See <er>Gourd</er>.]</ety> <fld>(Chem.)</fld> <def>A vessel or flask for distillation, used with, or forming part of, an alembic; a matrass; -- originally in the shape of a gourd, with a wide mouth. See <er>Alembic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucurbita</ent><br/
<hw>Cucurbita</hw> <pos>n.</pos> <def>the type genus of the <fam>Cucurbitaceae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Cucurbita</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cucurbitaceae</ent><br/
<hw>Cucurbitaceae</hw> <pos>n.</pos> <def>a natural family of plants including the cucumber; melon; squash; and pumpkin.</def><br/
<syn><b>Syn. --</b> family <fam>Cucurbitaceae</fam>, gourd family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cucurbitaceous</ent><br/
<hw>Cu*cur`bi*ta"ceous</hw> <pr>(-b<icr/*t<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cucurbitac<eacute/</ets>.]</ety> <fld>(Bot.)</fld> <def>Of, pertaining to, or resembling, a family of plants of which the cucumber, melon, and gourd are common examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cucurbitive</ent><br/
<hw>Cu*cur"bi*tive</hw> <pr>(k?-k?r"b?-t?v)</pr>, <pos>a.</pos> <def>Having the shape of a gourd seed; -- said of certain small worms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cud</ent><br/
<hw>Cud</hw> <pr>(k<ucr/d)</pr>, <pos>n.</pos> <ety>[AS. <ets>cudu</ets>, <ets>cwudu</ets>,<ets>cwidu</ets>,<ets>cweodo</ets>, of uncertain origin; cf, G. <ets>k<oum/der</ets> bait, Icel. <ets>kvi<edh/r</ets> womb, Goth. <ets>qi<thorn/us</ets>. Cf. <er>Quid</er>.]</ety> <sn>1.</sn> <def>That portion of food which is brought up into the mouth by ruminating animals from their first stomach, to be chewed a second time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whatsoever parteth the hoof, and is cloven-footed, and cheweth the <qex>cud</qex>, among the beasts, that shall ye eat.</q> <rj><qau>Levit. xi. 3</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A portion of tobacco held in the mouth and chewed; a quid.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The first stomach of ruminating beasts.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To chew the cud</b></col>, <cd>to ruminate; to meditate; used with <xex>of</xex>; <as>as, to <ex>chew the cud</ex> of bitter memories</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Chewed the</qex> thrice turned <qex>cud</qex> of wrath.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cudbear</ent><br/
<hw>Cud"bear`</hw> <pr>(k?d"b?r`)</pr>, <pos>n.</pos> <ety>[Also <asp>cudbeard</asp>, corrupted fr. the name of Dr. <etsep>Cuthbert</etsep> Gordon, a Scotchman, who first brought it into notice.]</ety> <sn>1.</sn> <def>A powder of a violet red color, difficult to moisten with water, used for making violet or purple dye. It is prepared from certain species of lichen, especially <spn>Lecanora tartarea</spn>.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A lichen (<spn>Lecanora tartarea</spn>), from which the powder is obtained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cudden</ent><br/
<hw>Cud"den</hw> <pr>(k?d"d'n)</pr>, <pos>n.</pos> <ety>[For sense 1, cf. Scot. <ets>cuddy</ets> an ass; for sense 2, see 3d <er>Cuddy</er>.]</ety> <sn>1.</sn> <def>A clown; a low rustic; a dolt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The slavering <qex>cudden</qex>, propped upon his staff.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The coalfish. See 3d <er>Cuddy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuddle</ent><br/
<hw>Cud"dle</hw> <pr>(k<ucr/d"d'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cuddled</conjf> <pr>(-d'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cuddling</conjf> <pr>(-dl<icr/ng)</pr>.]</vmorph> <ety>[Prob. for <ets>couthle</ets>, fr. <ets>couth</ets> known; cf. OE. <ets>ku<thorn/<thorn/en</ets> to cuddle, or <ets>cu<edh/lechen</ets> to make friends with. See<er>Couth</er>, <er>Uncouth</er>, <er>Can</er>.]</ety> <def>To lie close or snug; to crouch; to nestle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>cuddles</qex> low beneath the brake;<br/
Nor would she stay, nor dares she fly.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuddle</ent><br/
<hw>Cud"dle</hw>, <pos>v. t.</pos> <def>To embrace closely; to fondle.</def>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuddle</ent><br/
<hw>Cud"dle</hw>, <pos>n.</pos> <def>A close embrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cuddly</ent><br/
<ent>cuddlesome</ent><br/
<mhw><hw>cuddlesome</hw> <hw>cuddly</hw></mhw> <pos>adj.</pos> <def>inviting cuddling or hugging; <as>as, a <ex>cuddlesome</ex> baby</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cuddling</ent><br/
<hw>cuddling</hw> <pos>n.</pos> <sn>1.</sn> <def>holding close in a tender and afectionate manner.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>lying closely together while caressing; nestling together.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>foreplay without contact with the genital organs.</def> <mark>[slang]</mark> <br/
<syn><b>Syn. --</b> caressing, fondling, hugging, kissing, necking, petting, smooching, snuggling.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cuddy</ent><br/
<hw>Cud"dy</hw> <pr>(-d?)</pr>, <pos>n.</pos> <ety>[See <er>Cudden</er>. ]</ety> <sn>1.</sn> <def>An ass; esp., one driven by a huckster or greengrocer.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Hence: <def>A blockhead; a lout.</def>  <rj><au>Hood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>A lever mounted on a tripod for lifting stones, leveling up railroad ties, etc.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuddy</ent><br/
<hw>Cud"dy</hw> <pr>(k?d"d?)</pr>, <pos>n.</pos> <ety>[Prob. a contraction fr. D. <ets>kajuit</ets> cabin: cf. F.<ets>cahute</ets> hut.]</ety> <fld>(Naut.)</fld> <def>A small cabin: also, the galley or kitchen of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuddy</ent><br/
<hw>Cud"dy</hw>, <pos>n.</pos> <ety>[Scot.; cf. Gael. <ets>cudaig</ets>, <ets>cudainn</ets>, or E.<ets>cuttlefish</ets>, or <ets>cod</ets>, <ets>codfish</ets>.]</ety> <fld>(Zool)</fld> <def>The coalfish (<spn>Pollachius carbonarius</spn>).</def> <altsp>[Written also <asp>cudden</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cudgel</ent><br/
<hw>Cudg"el</hw> <pr>(k?j"?l)</pr>, <pos>n.</pos> <ety>[OE. <ets>kuggel</ets>; cf. G. <ets>keule</ets> club (with a round end), <ets>kugel</ets> ball, or perh. W. <ets>cogyl</ets> cudgel, or D. <ets>cudse</ets>, <ets>kuds</ets>, cudgel.]</ety> <def>A staff used in cudgel play, shorter than the quarterstaff, and wielded with one hand; hence, any heavy stick used as a weapon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He getteth him a grievous crabtree <qex>cudgel</qex> and . . . falls to rating of them as if they were dogs.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cudgel play</b></col>, <cd>a fight or sportive contest with cudgels.</cd> -- <col><b>To cross the cudgels</b></col>, <cd>to forbear or give up the contest; -- a phrase borrowed from the practice of cudgel players, who lay one cudgel over another when the contest is ended.</cd> -- <col><b>To take up cudgels for</b></col>, <cd>to engage in a contest in behalf of (some one or something).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cudgel</ent><br/
<hw>Cudg"el</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cudgeled</conjf> or <conjf>Cudgelled</conjf> <pr>(-<?/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cudgeling</conjf> or <conjf>cudgelling</conjf>.]</vmorph> <def>To beat with a cudgel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An he here, I would <qex>cudgel</qex> him like a dog.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cudgel one's brains</b></col>, <cd>to exercise one's wits.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cudgeler</ent><br/
<hw>Cudg"el*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <def>One who beats with a cudgel.</def> <altsp>[Written also <asp>cudgeller</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cudweed</ent><br/
<hw>Cud"weed`</hw> <pr>(k?d"w?d`)</pr>, <pos>n.</pos> <ety>[Apparently fr.  <ets>cud</ets> + <ets>weed</ets>, but perh. a corruption of <ets>cottonweed</ets>; or of <ets>cut weed</ets>, so called from its use as an application to cuts and chafings.]</ety> <fld>(Bot.)</fld> <def>A small composite plant with cottony or silky stem and leaves, primarily a species of <gen>Gnaphalium</gen>, but the name is now given to many plants of different genera, as <gen>Filago</gen>, <gen>Antennaria</gen>, etc.; cottonweed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cue</ent><br/
<hw>Cue</hw> <pr>(k<umac/)</pr>, <pos>n.</pos> <ety>[ OF. <ets>coue</ets>, <ets>coe</ets>, F. <ets>queue</ets>, fr. L. <ets>coda</ets>, <ets>cauda</ets>, tail. Cf. <er>Caudal</er>, <er>Coward</er>, <er>Queue</er>.]</ety> <sn>1.</sn> <def>The tail; the end of a thing; especially, a tail-like twist of hair worn at the back of the head; a queue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The last words of a play actor's speech, serving as an intimation for the next succeeding player to speak; any word or words which serve to remind a player to speak or to do something; a catchword.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When my <qex>cue</qex> comes, call me, and I will answer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A hint or intimation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give them [the servants] their <qex>cue</qex> to attend in two lines as he leaves the house.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The part one has to perform in, or as in, a play.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Were it my <qex>cue</qex>to fight, I should have known it<br/
Without a prompter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Humor; temper of mind.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A straight tapering rod used to impel the balls in playing billiards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cue</ent><br/
<hw>Cue</hw>, <pos>v. t.</pos> <def>To form into a cue; to braid; to twist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cue</ent><br/
<hw>Cue</hw>, <pos>n.</pos> <ety>[From <ets>q</ets>, an abbreviation for <ets>quadrans</ets> a farthing.]</ety> <def>A small portion of bread or beer; the quantity bought with a farthing or half farthing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term was formerly current in the English universities, the letter <xex>q</xex> being the mark in the buttery books to denote such a portion.  <rj><au>Nares.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><q>Hast thou worn<br/
Gowns in the university, tossed logic,<br/
Sucked philosophy, eat <qex>cues</qex>?</q> <rj><qau>Old Play.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuerpo</ent><br/
||<hw>Cuer"po</hw> <pr>(kw<etil/r"p<osl/)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cuerpo</ets>, fr. L. <ets>corpus</ets> body. See <er>Corpse</er>.]</ety> <def>The body.</def> <illu>dale a tu <ex>cuerpo</ex> alegria, Macarena! (1996)</illu><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In cuerpo</b></col>, <cd>without full dress, so that the shape of the Body is exposed; hence, naked or uncovered.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Exposed <qex>in cuerpo</qex> to their rage.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuesta</ent><br/
||<hw>Cues"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>A sloping plain, esp. one with the upper end at the crest of a cliff; a hill or ridge with one face steep and the opposite face gently sloping.</def> <mark>[Southwestern U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cuff</ent><br/
<hw>Cuff</hw> <pr>(k?f)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cuffed</conjf> <pr>(k<?/ft)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cuffing</conjf>.]</vmorph> <ety>[Cf. Sw. <ets>kuffa</ets> to knock, push,<ets>kufva</ets> to check, subdue, and E. <ets>cow</ets>, v. t. ]</ety> <sn>1.</sn> <def>To strike; esp., to smite with the palm or flat of the hand; to slap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I swear I'll <qex>cuff</qex> you, if you strike again.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They with their quills did all the hurt they could,<br/
And <qex>cuffed</qex> the tender chickens from their food.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To buffet.</def> <ldquo/<xex>Cuffed</xex> by the gale.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuff</ent><br/
<hw>Cuff</hw>, <pos>v. i.</pos> <def>To fight; to scuffle; to box.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While the peers <qex>cuff</qex> to make the rabble sport.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuff</ent><br/
<hw>Cuff</hw>, <pos>n.</pos> <def>A blow; esp.,, a blow with the open hand; a box; a slap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Snatcheth his sword, and fiercely to him flies;<br/
Who well it wards, and quitten <qex>cuff</qex> with <qex>cuff</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a bitter kick and <qex>cuff</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuff</ent><br/
<hw>Cuff</hw>, <pos>n.</pos> <ety>[Perh. from F. <ets>coiffe</ets> headdress, hood, or coif; as if the cuff were a cap for the hand. Cf. <er>Coif</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The fold at the end of a sleeve; the part of a sleeve turned back from the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He would visit his mistress in a morning gown, band, short <qex>cuffs</qex>, and a peaked beard.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any ornamental appendage at the wrist, whether attached to the sleeve of the garment or separate; especially, in modern times, such an appendage of starched linen, or a substitute for it of paper, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuffy</ent><br/
<hw>Cuf"fy</hw> <pr>(k<?/f`f<?/)</pr>, <pos>n.</pos> <def>A name for a negro.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cufic</ent><br/
<hw>Cu"fic</hw> <pr>(k?`f?k)</pr>, <pos>a.</pos> <ety>[So called from the town of <etsep>Cufa</etsep>, in the province of Bagdad.]</ety> <def>Of or pertaining to the older characters of the Arabic language.</def> <altsp>[Written also <asp>Kufic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cui bono</ent><br/
||<hw>Cui` bo"no</hw> <pr>(?)</pr>. <ety>[L.]</ety> <def>Lit., for whose benefit; incorrectly understood, it came to be used in the sense, of what good or use; and hence, (what) purpose; object; specif., the ultimate object of life.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cuinage</ent><br/
<hw>Cuin"age</hw> <pr>(kw?n`?j)</pr>, <pos>n.</pos> <ety>[Corrupted fr. <ets>coinage</ets>.]</ety> <def>The stamping of pigs of tin, by the proper officer, with the arms of the duchy of Cornwall.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuirass</ent><br/
<hw>Cui*rass"</hw> <pr>(kw<esl/*r<adot/s", <or/ kw<emac/"r<acr/s; 277)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cuirasses</plw>(-<ecr/z)</plu>. <ety>[F.<ets>cuirasse</ets>, orig., a breastplate of leather, for OF. <ets>cuir<eacute/e</ets>, <ets>cuirie</ets> influenced by It. <ets>corazza</ets>, or Sp. <ets>coraza</ets>, fr. an assumed LL. <ets>coriacea</ets>, fr. L. <ets>coriaceus</ets>, adj., of leather, fr. <ets>corium</ets> leather, hide; akin to Gr. <grk>cho`rion</grk> intestinal membrane, OSlav. <ets>skora</ets> hide, Lith. <ets>skura</ets> hide, leather. Cf. <er>Coriaceous</er>.]</ety> <sn>1.</sn> <sd>(a)</sd> <def>A piece of defensive armor, covering the body from the neck to the girdle</def>. <sd>(b)</sd> <def>The breastplate taken by itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>cuirass</xex> covered the body before and behind. It consisted of two parts, a breast- and backpiece of iron fastened together by means of straps and buckles or other like contrivances. It was originally, as the name imports, made of leather, but afterward of metal.
<rj><au>Grose.</au></rj></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool)</fld> <def>An armor of bony plates, somewhat resembling a cuirass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuirassed</ent><br/
<hw>Cui*rassed"</hw> <pr>(kw<esl/*r<adot/st" <it>or</it> kw<emac/"r<acr/st)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Wearing a cuirass.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool)</fld> <def>Having a covering of bony plates, resembling a cuirass; -- said of certain fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuirassier</ent><br/
<hw>Cui`ras*sier"</hw> <pr>(kw?`r?s-s?r")</pr>, <pos>n.</pos> <ety>[F. <ets>cuirassier</ets>. See <er>Curass</er>.]</ety> <fld>(Mil.)</fld> <def>A soldier armed with a cuirass; especially, a soldier of the heaviest cavalry, wearing a cuirass only when in full dress.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source> <source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cuir bouilli</ent><br/
||<hw>Cuir" bou`illi"</hw> <pr>(?)</pr>. <ety>[F.]</ety> <def>In decorative art, boiled leather, fitted by the process to receive impressed patterns, like those produced by chasing metal, and to retain the impression permanently.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cuish</ent><br/
<hw>Cuish</hw> <pr>(kw?s)</pr>, <pos>n.</pos> <ety>[F. <ets>cuisse</ets> thigh, fr. L. <ets>coxa</ets> hip: cf. F. <ets>cuissard</ets>, OF, <ets>cuissot</ets>, armor for the thigh, cuish. Cf. <er>Hough</er>.]</ety> <def>Defensive armor for the thighs.</def> <altsp>[ Written also <asp>cuisse</asp>, and <asp>quish</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuisine</ent><br/
||<hw>Cui`sine"</hw> <pr>(kw?`z?n")</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>coquina</ets> kitchen, fr. <ets>coquere</ets> to cook. See <er>Kitchen</er>.]</ety> <sn>1.</sn> <def>The kitchen or cooking department.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Manner or style of cooking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cul</ent><br/
<hw>cul</hw> <pos>n.</pos> <def>a contraction of <er>cul-de-sac</er>.</def><br/
<syn><b>Syn. --</b> cul de sac, deadend.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Culasse</ent><br/
||<hw>Cu`lasse"</hw> <pr>(ku`l?s")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>cul</ets> back.]</ety> <def>The lower faceted portion of a brilliant-cut diamond.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culcita</ent><br/
<hw>Culcita</hw> <pos>n.</pos> <def>a genus that includes some plants usually placed in e.g. genus <gen>Dicksonia</gen>; terrestrial brackenlike ferns of tropical America, Malasia to Australia and Polynesia; also native in southwestern Europe and Atlantic islands.</def><br/
<syn><b>Syn. --</b> genus Culcita.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Culicidae</ent><br/
<hw>Culicidae</hw> <pos>n.</pos> <def>a family of insects comprising the mosquitoes.</def><br/
<syn><b>Syn. --</b> family <fam>Culicidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Culdee</ent><br/
<hw>Cul*dee"</hw> <pr>(k<?/l-d<?/" <or/ k<?/l"d<?/)</pr>, <pos>n.</pos> <ety>[ Prob. fr. Gael.<ets>cuilteach</ets>; cf. Ir. <ets>ceilede</ets>.]</ety> <def>One of a class of anchorites who lived in various parts of Scotland, Ireland, and Wales.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pure <qex>Culdees</qex><br/
Were Albyn's earliest priests of God.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cul-de-sac</ent><br/
||<hw>Cul`-de-sac"</hw> <pr>(ku`de-s?k" <or/ kul`de-s?k")</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Culs-de-sac</plw> <pr>(ku`- <xex>or</xex> kulz`-)</pr>.</plu> <ety>[ F., lit., bottom of a bag.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A passage with only one outlet, as a street closed at one end; a blind alley; hence, a trap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>a position in which an army finds itself with no way of exit but to the front.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Any bag-shaped or tubular cavity, vessel, or organ, open only at one end.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culerage</ent><br/
<hw>Cul"er*age</hw> <pr>(k?l"?r-?j)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Culrage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culex</ent><br/
<hw>Cu"lex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a gnat.]</ety> <fld>(Zool.)</fld> <def>A genus of mosquitoes of the family <fam>Culicidae</fam>, to which most of the North American species belong.  Some members of this genus are exceedingly annoying, as <spn>Culex sollicitans</spn>, which breeds in enormous numbers in the salt marshes of the Atlantic coast, and <spn>Culex pipiens</spn>, the common house mosquito, breeding very widely in the fresh waters of North America. (For characters distinguishing these from the malaria mosquitoes, see <er>Anopheles</er>, above).  The yellow-fever mosquito is now placed in another genus, <gen>Stegomyia</gen>.</def>  <note>This genus formerly (ca. 1900) included the gnat as well as the mosquito.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Culicid</ent><br/
<hw>Cu"li*cid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>culex</ets>, <ets>-icis</ets>, gnat.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to the Mosquito family (<fam>Culicid<ae/</fam>).</def> -- <def2><pos>n.</pos>  <def>A culicid insect.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Culiciform</ent><br/
<hw>Cu*lic"i*form</hw> <pr>(k?-l?s"i-f?rm)</pr>. <pos>a.</pos> <ety>[L. <ets>culex</ets> a gnat + <ets>-form</ets>:cf. F. <ets>culiciforme</ets>.]</ety> <fld>(Zool.)</fld> <def>Gnat-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culinarily</ent><br/
<hw>Cu"li*na*ri*ly</hw> <pr>(k?`l?-n?-r?-l?)</pr>, <pos>adv.</pos> <def>In the manner of a kitchen; in connection with a kitchen or cooking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culinary</ent><br/
<hw>Cu"li*na*ry</hw> <pr>(k?"l?-n?-r?)</pr>, <pos>a.</pos> <ety>[L. <ets>culinarius</ets>, fr. <ets>culina</ets> kitchen, perh. akin to <ets>carbo</ets> coal: cf. F. <ets>culinare</ets>.]</ety> <def>Relating to the kitchen, or to the art of cookery; used in kitchens; <as>as, a <ex>culinary</ex> vessel; the <ex>culinary</ex> art</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cull</ent><br/
<hw>Cull</hw> <pr>(k?l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Culled</conjf> <pr>(k?ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Culling</conjf>.]</vmorph> <ety>[OE.<ets>cullen</ets>, OF. <ets>cuillir</ets>, <ets>coillir</ets>, F.<ets>cueillir</ets>, to gather, pluck, pick, fr. L. <ets>colligere</ets>. See <er>Coil</er>, <pos>v. t.</pos>, and cf. <er>Collect</er>.]</ety> <def>To separate, select, or pick out; to choose and gather or collect; <as>as, to <ex>cull</ex> flowers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From his herd he <qex>culls</qex>,<br/
For slaughter, from the fairest of his bulls.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whitest honey in fairy gardens <qex>culled</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cull</ent><br/
<hw>Cull</hw>, <pos>n.</pos> <def>A cully; a dupe; a gull. See <er>Cully</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullender</ent><br/
<hw>Cul"len*der</hw> <pr>(k?l"l?n-d?r)</pr>, <pos>n.</pos> <def>A strainer. See <er>Colander</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culler</ent><br/
<hw>Cull"er</hw> <pr>(k?l"?r)</pr>, <pos>n.</pos> <def>One who picks or chooses; esp., an inspector who selects wares suitable for market.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullet</ent><br/
<hw>Cul"let</hw> <pr>(k<ucr/l"l<ecr/t)</pr>, <pos>n.</pos> <ety>[From <er>Cull</er>, <pos>v. t.</pos> ]</ety> <def>Broken glass for remelting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullet</ent><br/
<hw>Cul"let</hw>, <pos>n.</pos> <ety>[A dim. from F. <ets>cul</ets> back.]</ety> <def>A small central plane in the back of a cut gem. See <er>Collet</er>, 3 <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullibility</ent><br/
<hw>Cul`li*bil"i*ty</hw> <pr>(-l?-b?l"?-ty)</pr>, <pos>n.</pos> <ety>[From <ets>cully</ets> to trick, cheat.]</ety> <def>Gullibility.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullible</ent><br/
<hw>Cul"li*ble</hw> <pr>(k?l"l?-b'l)</pr>, <pos>a.</pos> <def>Easily deceived; gullible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culling</ent><br/
<hw>Cull"ing</hw> <pr>(k?l"?ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who culls.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Anything separated or selected from a mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullion</ent><br/
<hw>Cul"lion</hw> <pr>(k?l"y?n)</pr>, <pos>n.</pos> <ety>[OF. <ets>couillon</ets>, <ets>coillon</ets>, F. <ets>co<?/on</ets>, a vile fellow, coward, dupe, from OF. <ets>couillon</ets>, <ets>coillon</ets>, testicle, fr. <ets>il</ets> the scrotum, fr. L. <ets>coleus</ets> a leather bag, the scrotum.]</ety> <def>A mean wretch; a base fellow; a poltroon; a scullion.</def> <ldquo/Away, base <xex>cullions</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullionly</ent><br/
<hw>Cul"lion*ly</hw>, <pos>a.</pos> <def>Mean; base.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullis</ent><br/
<hw>Cul"lis</hw> <pr>(k<?/l"l<?/s)</pr>, <pos>n.</pos> <ety>[OF. <ets>cole<ium/s</ets>, F. <ets>coulis</ets>, fr. OF. & F. <ets>couler</ets> to strain, to flow, fr. L. <ets>colare</ets> to filter, strain; cf. LL. <ets>coladicium</ets>. Cf. <er>Colander</er>.]</ety> <def>A strong broth of meat, strained and made clear for invalids; also, a savory jelly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When I am exellent at caudles<br/
And <qex>cullises</qex> . . . you shall be welcome to me.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullis</ent><br/
<hw>Cul"lis</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cullises</plw> <pr>(-<?/z)</pr>.</plu> <ety>[F.<ets>coulisse</ets> groove, fr. the same source as E. <ets>cullis</ets> broth.]</ety> <fld>(Arch.)</fld> <def>A gutter in a roof; a channel or groove.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culls</ent><br/
<hw>Culls</hw> <pr>(k?lz)</pr>, <pos>n. pl.</pos> <ety>[From Cull,, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Refuse timber, from which the best part has been culled out.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any refuse stuff, as rolls not properly baked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cully</ent><br/
<hw>Cul"ly</hw> <pr>(k?l"l?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cullies</plw> <pr>(-l<?/z)</pr>.</plu> <ety>[Abbrev. fr. <ets>cullion</ets>.]</ety> <def>A person easily deceived, tricked, or imposed on; a mean dupe; a gull.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have learned that . . . I am not the first <qex>cully</qex> whom she has passed upon for a countess.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cully</ent><br/
<hw>Cul"ly</hw>, <pos>v. t.</pos> <ety>[See <er>Cully</er>,<pos>n.</pos>, and cf. D. <ets>kullen</ets> to cheat, gull.]</ety> <def>To trick, cheat, or impose on; to deceive.</def> <ldquo/Tricks to <xex>cully</xex> fools.<rdquo/  <rj><au>Pomfret.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cullyism</ent><br/
<hw>Cul"ly*ism</hw> <pr>(-?z'm)</pr>, <pos>n.</pos> <def>The state of being a cully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Less frequent instances of eminent <qex>cullyism</qex>.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culm</ent><br/
<hw>Culm</hw> <pr>(k<ucr/lm)</pr>, <pos>n.</pos> <ety>[L. <ets>culmus</ets> stalk, stem; akin to <ets>calamus</ets>. See<er>Halm</er>.]</ety> <fld>(Bot.)</fld> <def>The stalk or stem of grain and grasses (including the bamboo), jointed and usually hollow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culm</ent><br/
<hw>Culm</hw>, <pos>n.</pos> <ety>[Perh. from W. <ets>cwlm</ets> knot or tie, applied to this species of coal, which is much found in balls or knots in some parts of Wales: cf. OE. <ets>culme</ets> smoke, soot.]</ety> <fld>(Min.)</fld> <sd>(a)</sd> <def>Mineral coal that is not bituminous; anthracite, especially when found in small masses.</def> <sd>(b)</sd> <def>The waste of the Pennsylvania anthracite mines, consisting of fine coal, dust, etc., and used as fuel.</def>  <rj><au> Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culmen</ent><br/
||<hw>Cul"men</hw> <pr>(k?l"m?n)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>cellere</ets> (in comp.) to impel; cf. <ets>celsus</ets> pushed upward, lofty.]</ety> <sn>1.</sn> <def>Top; summit; acme.</def>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The dorsal ridge of a bird's bill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culmiferous</ent><br/
<hw>Cul*mif"er*ous</hw> <pr>(k?l-m?f"?r-?s)</pr>, <pos>a.</pos> <ety>[L. <ets>culmus</ets> stalk or stem + <ets>-ferous</ets>: cf. F. <ets>culmif<egrave/re</ets>.]</ety> <def>Having jointed stems or culms.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 355 --></p>

<p><ent>Culmiferous</ent><br/
<hw>Cul*mif"er*ous</hw> <pr>(k<ucr/l*m<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[2d <ets>culm</ets> + <ets>-ferous</ets>.]</ety> <fld>(Min.)</fld> <def>Containing, or abounding in, culm or glance coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culminal</ent><br/
<hw>Cul"mi*nal</hw> <pr>(k<ucr/l"m<icr/*n<ait/l)</pr>, <pos>a.</pos> <def>Pertaining to a culmen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culminant</ent><br/
<hw>Cul"mi*nant</hw> <pr>(-n<ait/nt)</pr>, <pos>a.</pos> <def>Being vertical, or at the highest point of altitude; hence, predominant.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culminate</ent><br/
<hw>Cul"mi*nate</hw> <pr>(k<ucr/l"m<icr/*n<amac/t)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Culminated</conjf> <pr>(-n<amac/`t<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Culminating</conjf> (-n<amac/`t<icr/ng.]</vmorph> <ety>[L. <ets>cuimen</ets> top or ridge. See <er>Column</er>.]</ety> <sn>1.</sn> <def>To reach its highest point of altitude; to come to the meridian; to be vertical or directly overhead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As when his beams at noon<br/
<qex>Culminate</qex> from the equator.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reach the highest point, as of rank, size, power, numbers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The reptile race <qex>culminated</qex> in the secondary era.</q> <rj><qau>Dana.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The house of Burgundy was rapidly <qex>culminating</qex>.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culminate</ent><br/
<hw>Cul"mi*nate</hw> <pr>(k<ucr/l"m<icr/*n<asl/t)</pr>, <pos>a.</pos> <def>Growing upward, as distinguished from a lateral growth; -- applied to the growth of corals.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culmination</ent><br/
<hw>Cul"mi*na"tion</hw> <pr>(k<ucr/l`m?-n?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F.<ets>culmination</ets>]</ety> <sn>1.</sn> <def>The attainment of the highest point of altitude reached by a heavenly body; passage across the meridian; transit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Attainment or arrival at the highest pitch of glory, power, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>culottes</ent><br/
<hw>culottes</hw> <pos>n. pl.</pos> <def>a type of women's trousers with wide legs cut loose and full so as to resemble a skirt.</def><br/
<syn><b>Syn. --</b> culotte.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Culpa</ent><br/
||<hw>Cul"pa</hw> <pr>(k<ucr/l"p<adot/)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Law)</fld> <def>Negligence or fault, as distinguishable from <xex>dolus</xex> (deceit, fraud), which implies intent, <xex>culpa</xex> being imputable to defect of intellect, <xex>dolus</xex> to defect of heart.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culpability</ent><br/
<hw>Cul`pa*bil"i*ty</hw> <pr>(k?l`p?-b?l`?-t?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Culpabilities</plw> <pr>(-t<?/z)</pr>.</plu> <ety>[Cf. F. <ets>culpabilit<eacute/</ets>.]</ety> <def>The state of being culpable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culpable</ent><br/
<hw>Cul"pa*ble</hw> <pr>(k?l"p?-b'l)</pr>, <pos>a.</pos> <ety>[OE. <ets>culpable</ets>, <ets>coulpable</ets>, <ets>coupable</ets>, F. <ets>coupable</ets>, formerly also <ets>coupable</ets>, formerly also <ets>coulpable</ets>, <ets>culpable</ets>, fr. L. <ets>culpabilis</ets>, fr. <ets>culpare</ets> to blame, fr. <ets>culpa</ets> fault.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Deserving censure; worthy of blame; faulty; immoral; criminal.</def>  <rj><au>State Trials (1413).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If he acts according to the best reason he hath, he is not <qex>culpable</qex>, though he be mistaken in his measures.</q> <rj><qau>Sharp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Guilty; <as>as, <ex>culpable</ex> of a crime</as>.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj></p>

<p>-- <wordforms><wf>Cul"pa*ble*ness</wf>, <pos>n.</pos> -- <wf>Cul"pa*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culpatory</ent><br/
<hw>Cul"pa*to*ry</hw> <pr>(-t?-r?)</pr>, <pos>a.</pos> <def>Expressing blame; censuring; reprehensory; inculpating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Adjectives . . . commonly used by Latian authors in a <qex>culpatory</qex> sense.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culpe</ent><br/
<hw>Culpe</hw> <pr>(k<ucr/lp)</pr>, <pos>n.</pos> <ety>[F. <ets>coulpe</ets>, fr.L.<ets>culpa</ets>.]</ety> <def>Blameworthiness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Banished out of the realme . . . without <qex>culpe</qex>.</q> <rj><qau>E. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culpon</ent><br/
<hw>Cul"pon</hw> <pr>(k<ucr/l"p<ocr/n)</pr>, <pos>n.</pos> <ety>[See <er>Coupon</er>.]</ety> <def>A shred; a fragment; a strip of wood.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culprit</ent><br/
<hw>Cul"prit</hw> <pr>(k<ucr/l"pr?t)</pr>, <pos>n.</pos> <ety>[Prob. corrupted for <ets>culpate</ets>, fr. Law Latin <ets>culpatus</ets> the accused, <pos>p. p.</pos> of L. <ets>culpare</ets> to blame. See <er>Culpable</er>.]</ety> <sn>1.</sn> <def>One accused of, or arraigned for, a crime, as before a judge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An author is in the condition of a <qex>culprit</qex>; the public are his judges.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One quilty of a fault; a criminal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culrage</ent><br/
<hw>Cul"rage</hw> <pr>(k?l"r?j)</pr>, <pos>n.</pos> <ety>[OE. <ets>culrage</ets>, <ets>culrache</ets>; prob. fr. F. <ets>cul</ets> the buttok + F. & E. <ets>rage</ets>; F. <ets>curage</ets>.]</ety> <fld>(Bot.)</fld> <def>Smartweed (<spn>Polygonum Hydropiper</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cult</ent><br/
<hw>Cult</hw> <pr>(k<ucr/lt)</pr> <pos>n.</pos> <ety>[F. <ets>culte</ets>, L. <ets>cultus</ets> care, culture, fr. <ets>colere</ets> to cultivate. Cf. <er>Cultus</er>.]</ety> <sn>1.</sn> <def>Attentive care; homage; worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every one is convinced of the reality of a better self, and of the <qex>cult</qex> or homage which is due to it.</q> <rj><qau>Shaftesbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A system of religious belief and worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That which was the religion of Moses is the ceremonial or <qex>cult</qex> of the religion of Christ.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A system of intense religious veneration of a particular person, idea, or object, especially one considered spurious or irrational by traditional religious bodies; <as>as, the Moonie <ex>cult</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>The group of individuals who adhere to a cult (senses 2 or 3).</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>A strong devotion or interest in a particular person, idea or thing without religious associations, or the people holding such an interest; <as>as, the <ex>cult</ex> of James Dean; the <ex>cult</ex> of personality in totalitarian societies</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cultch</ent><br/
<hw>Cultch</hw> <pr>(k?lch; 224)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>Empty oyster shells and other substances laid down on oyster grounds to furnish points for the attachment of the spawn of the oyster.</def> <altsp>[Also written <asp>cutch</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Young or seed oysters together with the shells and other objects to which they are usually attached.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>Rubbish; d<eacute/bris; refuse.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Culter</ent><br/
<hw>Cul"ter</hw> <pr>(k?l"t?r)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A colter. See <er>Colter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultirostral</ent><br/
<hw>Cul`ti*ros"tral</hw> <pr>(-t?-r?s"tral)</pr>, <pos>a.</pos> <ety>[See <er>Cultirostres</er>.]</ety> <fld>(Zool.)</fld> <def>Having a bill shaped like the colter of a plow, or like a knife, as the heron, stork, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultirostres</ent><br/
||<hw>Cul`ti*ros"tres</hw> <pr>(-tr?z)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>culter</ets> colter of a plow, knife + <ets>rostrum</ets> bill.]</ety> <fld>(Zool.)</fld> <def>A tribe of wading birds including the stork, heron, crane, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultivable</ent><br/
<hw>Cul"ti*va*ble</hw> <pr>(k?l"t?-v?-b'l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cultivable</ets>.]</ety> <def>Capable of being cultivated or tilled.</def>  <rj><au>Todd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cultivar</ent><br/
<hw>cultivar</hw> <pos>n.</pos> <def>a variety of a plant developed from a natural species and maintained under cultivation.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cultivatable</ent><br/
<hw>Cul"ti*va`ta*ble</hw> <pr>(k?l"t?-v?`t?-b'l)</pr>, <pos>a.</pos> <def>Cultivable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultivate</ent><br/
<hw>Cul"ti*vate</hw> <pr>(k?l"t?-v?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cultivated</conjf> <pr>(-v?`t?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cultivating</conjf> <pr>(-v?`-t?ng)</pr>.]</vmorph> <ety>[LL. <ets>cultivatus</ets>, p. p. of <ets>cultivare</ets> to cultivate, fr. <ets>cultivus</ets> cultivated, fr. L. <ets>cultus</ets>, p. p. of <ets>colere</ets> to till, cultivate. Cf. <er>Colony</er>.]</ety> <sn>1.</sn> <def>To bestow attention, care, and labor upon, with a view to valuable returns; to till; to fertilize; <as>as, to <ex>cultivate</ex> soil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To direct special attention to; to devote time and thought to; to foster; to cherish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Leisure . . . to <qex>cultivate</qex> general literature.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To seek the society of; to court intimacy with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I ever looked on Lord Keppel as one of the greatest and best men of his age; and I loved and <qex>cultivated</qex> him accordingly.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To improve by labor, care, or study; to impart culture to; to civilize; to refine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>cultivate</qex> the wild, licentious savage.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind of man hath need to be prepared for piety and virtue; it must be <qex>cultivated</qex> to the end.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To raise or produce by tillage; to care for while growing; <as>as, to <ex>cultivate</ex> corn or grass</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cultivated</ent><br/
<hw>cultivated</hw> <pos>adj.</pos> <sn>1.</sn>  <def>marked by refinement in taste and manners; <as>as, <ex>cultivated</ex> tastes in art</as>.</def><br/
<syn><b>Syn. --</b> civilized, cultured, genteel, polite.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>used for raising crops; -- of land or soil.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cultivation</ent><br/
<hw>Cul`ti*va"tion</hw> <pr>(k?l`t?-v?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cultivation</ets>.]</ety> <sn>1.</sn> <def>The art or act of cultivating; improvement for agricultural purposes or by agricultural processes; tillage; production by tillage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bestowal of time or attention for self-improvement or for the benefit of others; fostering care.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of being cultivated; advancement in physical, intellectual, or moral condition; refinement; culture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Italy . . . was but imperfectly reduced to <qex>cultivation</qex> before the irruption of the barbarians.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultivator</ent><br/
<hw>Cul"ti*va`tor</hw> <pr>(k?l"t?-v?`t?r)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cultivateur</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>One who cultivates; <as>as, a <ex>cultivator</ex> of the soil; a <ex>cultivator</ex> of literature</as>.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An agricultural implement used in the tillage of growing crops, to loosen the surface of the earth and kill the weeds; esp., a triangular frame set with small shares, drawn by a horse and by handles.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In a broader signification it includes any complex implement for pulverizing or stirring the surface of the soil, as harrows, grubbers, horse hoes, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultrated</ent><br/
<ent>Cultrate</ent><br/
<mhw>{ <hw>Cul"trate</hw> <pr>(k?l"tr?t)</pr>, <hw>Cul"tra*ted</hw> <pr>(-tr?-t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cultratus</ets> knife-shaped, from<ets>culter</ets>, <ets>cultri</ets>, knife.]</ety> <fld>(Bot. & Zool.)</fld> <def>Sharp-edged and pointed; shaped like a pruning knife, as the beak of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultriform</ent><br/
<hw>Cul"tri*form</hw> <pr>(-tr?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>culter</ets>, <ets>cultri</ets>, knife + <ets>-form</ets>.]</ety> <fld>(Bot. & Zool.)</fld> <def>Shaped like a pruning knife; cultrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultrivorous</ent><br/
<hw>Cul*triv"o*rous</hw> <pr>(k?l-tr?b"?-r?s)</pr>, <pos>a.</pos> <ety>[L. <ets>culter</ets>, <ets>cultri</ets>, knife + <ets>vorare</ets> to devour.]</ety> <def>Devouring knives; swallowing, or pretending to swallow, knives; -- applied to persons who have swallowed, or have seemed to swallow, knives with impunity.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culturable</ent><br/
<hw>Cul"tur*a*ble</hw> <pr>(k?l"t?r-?-b'l; 135)</pr>, <pos>a.</pos> <def>Capable of, or fit for, being cultivated; capable or becoming cultured.</def>  <rj><au>London Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultural</ent><br/
<hw>Cul"tur*al</hw> <pr>(k?l"t?r-<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to culture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culture</ent><br/
<hw>Cul"ture</hw> <pr>(k?l"t?r; 135)</pr>, <pos>n.</pos> <ety>[F. <ets>culture</ets>, L. <ets>cultura</ets>, fr. <ets>colere</ets> to till, cultivate; of uncertain origin. Cf. <er>Colony</er>.]</ety> <sn>1.</sn> <def>The act or practice of cultivating, or of preparing the earth for seed and raising crops by tillage; <as>as, the <ex>culture</ex> of the soil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of, or any labor or means employed for, training, disciplining, or refining the moral and intellectual nature of man; <as>as, the <ex>culture</ex> of the mind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If vain our toil<br/
We ought to blame the <qex>culture</qex>, not the soil.</q> <rj><qau>Pepe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of being cultivated; result of cultivation; physical improvement; enlightenment and discipline acquired by mental and moral training; civilization; refinement in manners and taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What the Greeks expressed by their <grk>paidei`a</grk>, the Romans by their <qex>humanitas</qex>, we less happily try to express by the more artificial word <qex>culture</qex>.</q> <rj><qau>J. C. Shairp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The list of all the items of the general life of a people represents that whole which we call its <qex>culture</qex>.</q> <rj><qau>Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Biol.)</fld> <sd>(a)</sd> <def>The cultivation of bacteria or other organisms (such as fungi or eukaryotic cells from mulitcellular organisms) in artificial media or under artificial conditions.</def> <sd>(b)</sd> <def>The collection of organisms resulting from such a cultivation.</def> <note>The growth of cells obtained from multicellular animals or plants in artificial media is called <stype>tissue culture</stype>.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><note><hand/ The word is used adjectively with the above senses in many phrases, such as: <xex>culture medium</xex>, any one of the various mixtures of gelatin, meat extracts, etc., in which organisms cultivated; <xex>culture flask</xex>, <xex>culture oven</xex>, <xex>culture tube</xex>, <xex>gelatin culture</xex>, <xex>plate culture</xex>, etc.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn>  <fld>(Cartography)</fld> <def>Those details of a map, collectively, which do not represent natural features of the area delineated, as names and the symbols for towns, roads, houses, bridges, meridians, and parallels.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Culture fluid</b></col>, <col><b>Culture medium</b></col></mcol> <cd>a fluid in which microscopic organisms are made to develop, either for purposes of study or as a means of modifying their virulence.  If the fluid is gelled by, for example, the use of agar, it then is called, depending on the vessel in which the gelled medium is contained, a plate, a slant, or a stab.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Culture</ent><br/
<hw>Cul"ture</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cultured</conjf> <pr>(-t?rd; 135)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Culturing</conjf>.]</vmorph> <def>To cultivate; to educate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They came . . . into places well inhabited and <qex>cultured</qex>.</q> <rj><qau>Usher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultured</ent><br/
<hw>Cul"tured</hw> <pr>(k?l"t?rd)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Under culture; cultivated.</def> <ldquo/<xex>Cultured</xex> vales.<rdquo/  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characterized by mental and moral training; disciplined; refined; well-educated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sense of beauty in nature, even among <qex>cultured</qex> people, is less often met with than other mental endowments.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The cunning hand and <qex>cultured</qex> brain.</q> <rj><qau>Whittier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culture features</ent><br/
<hw>Culture features</hw>. <fld>(Surv.)</fld> <def>The artificial features of a district as distinguished from the natural.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cultureless</ent><br/
<hw>Cul"ture*less</hw>, <pos>a.</pos> <def>Having no culture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culture myth</ent><br/
<hw>Culture myth</hw>. <def>A myth accounting for the discovery of arts and sciences or the advent of a higher civilization, as in the Prometheus myth.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Culturist</ent><br/
<hw>Cul"tur*ist</hw>, <pos>n.</pos> <sn>1.</sn> <def>A cultivator.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is an advocate of culture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>culturists</qex>, by which term I mean not those who esteem culture (as what intelligent man does not<?/) but those its exclusive advocates who recommend it as the panacea for all the ills of humanity, for its effects in cultivating the whole man.</q> <rj><qau>J. C. Shairp</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultus</ent><br/
||<hw>Cul"tus</hw> <pr>(k?l"t?s)</pr>, <pos>n. sing. & pl.</pos>; <plu>E. pl. <plw>Cultuses</plw> <pr>(-<?/z)</pr>.</plu> <ety>[L., cultivation, culture.  See <er>Cult</er>.]</ety> <def>Established or accepted religious rites or usages of worship; state of religious development.  Cf. <er>Cult</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cultus</ent><br/
<hw>Cul"tus</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Cultus cod</er>.]</ety> <def>Bad, worthless; no good.</def> <mark>[Northwestern U. S.]</mark></p>

<p><q><ldquo/A bad horse, <qex>cultus</qex> [no good] !<rdquo/ he said, beating it with his whip.</q>  <rj><qau>F. H. Balch.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cultus cod</ent><br/
<hw>Cul"tus cod`</hw> <pr>(k?d`)</pr>. <ety>[Chinook <ets>cultus</ets> of little worth.]</ety> <fld>(Zool.)</fld> <def>See <er>Cod</er>, and <cref>Buffalo cod</cref>, under <er>Buffalo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culver</ent><br/
<hw>Cul"ver</hw> <pr>(k?"v?r)</pr>, <pos>n.</pos> <ety>[AS. <ets>culfre</ets>, perh. fr. L. <ets>columba</ets>.]</ety> <def>A dove.</def> <ldquo/<xex>Culver</xex> in the falcon's fist.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culver</ent><br/
<hw>Cul"ver</hw>, <pos>n.</pos> <ety>[Abbrev. fr. <er>Culverin</er>.]</ety> <def>A culverin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Falcon and <qex>culver</qex> on each tower<br/
Stood prompt their deadly hail to shower.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culverhouse</ent><br/
<hw>Cul"ver*house`</hw> <pr>(-hous`)</pr>, <pos>n.</pos> <def>A dovecote.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culverin</ent><br/
<hw>Cul"ver*in</hw> <pr>(k?l"v?r-?n)</pr>, <pos>n.</pos> <ety>[F. <ets>coulevrine</ets>, prop. fem. of <ets>couleuvrin</ets> like a serpent, fr. <ets>couleuvre</ets> adder, fr. L. <ets>coluber</ets>, <ets>colubra</ets>.]</ety> <def>A long cannon of the 16th century, usually an 18-pounder with serpent-shaped handles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trump, and drum, and roaring <qex>culverin</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culverkey</ent><br/
<hw>Cul"ver*key`</hw> <pr>(-k?`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A bunch of the keys or samaras of the ash tree.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An English meadow plant, perhaps the columbine or the bluebell squill (<spn>Scilla nutans</spn>).</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A girl cropping <qex>culverkeys</qex> and cowslips to make garlands.</q> <rj><qau>Walton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culver's root</ent><br/
<ent>Culver's physic</ent><br/
<mhw>{ <hw>Cul"ver's phys"ic</hw> <pr>(?)</pr>, <it>or</it> <hw>Cul"ver's root`</hw> <pr>(?)</pr> }</mhw>. <ety>[So called after a Dr. <etsep>Culver</etsep>, who used it.]</ety> <fld>(Bot.)</fld> <def>The root of a handsome erect herb (<spn>Leptandra Virginica</spn> syn. <spn>Veronica Virginica</spn>) common in most moist woods of North America , used as an active cathartic and emetic; also, the plant itself.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Culvert</ent><br/
<hw>Cul"vert</hw> <pr>(k?l"v?rt)</pr>, <pos>n.</pos> <ety>[Prob. from OF. <ets>coulouere</ets>, F. <ets>couloir</ets>, channel, gutter, gallery, fr. <ets>couler</ets> to flow. See <er>Cullis</er>.]</ety> <def>A transverse drain or waterway of masonry under a road, railroad, canal, etc.; a small bridge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culvertail</ent><br/
<hw>Cul"ver*tail`</hw> <pr>(k?l"v?r-t?l`)</pr>, <pos>n.</pos> <fld>(Carp.)</fld> <def>Dovetail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Culvertailed</ent><br/
<hw>Cul"ver*tailed`</hw> <pr>(-t?ld`)</pr>, <pos>a.</pos> <def>United or fastened by a dovetailed joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cum</ent><br/
<hw>cum</hw> <pr>(k<ucr/m; k<oocr/m)</pr>, <pos>prep.</pos> <ety>[L. with <pos>prep.</pos>]</ety> <def>with; together with; along with; <as>as, I work at home in an office-<ex>cum</ex>-bedroom</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>cum</ent><br/
<hw>cum</hw> <pr>(k<ucr/m)</pr>, <pos>n.</pos> <def>same as <er>semen{2}</er>; -- also spelled <asp>come</asp>.</def> <mark>[vulgar slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Cumacea</ent><br/
||<hw>Cu*ma"ce*a</hw> <pr>(k?-m?"sh?-? or -s?-?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An order of marine Crustacea, mostly of small size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumbent</ent><br/
<hw>Cum"bent</hw> <pr>(k?m"bent)</pr>, <pos>a.</pos> <ety>[Cf. <er>Recumbent</er>, <er>Covey</er>.]</ety> <def>Lying down; recumbent.</def>  <rj><au>J. Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumber</ent><br/
<hw>Cum"ber</hw> <pr>(k?m"b?r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cumbered</conjf> <pr>(-b?rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cumbering</conjf>.]</vmorph> <ety>[OE. <ets>combren</ets>, <ets>cumbren</ets>,OF. <ets>combrer</ets> to hinder, from LL. <ets>cumbrus</ets> a heap, fr. L. <ets>cumulus</ets>; cf. Skr. <ets><?/<?/</ets> to increase, grow strong. Cf. <er>Cumulate</er>.]</ety> <def>To rest upon as a troublesome or useless weight or load; to be burdensome or oppressive to; to hinder or embarrass in attaining an object, to obstruct or occupy uselessly; to embarrass; to trouble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why asks he what avails him not in fight,<br/
And would but <qex>cumber</qex> and retard his flight?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Martha was <qex>cumbered</qex> about much serving.</q> <rj><qau>Luke x. 40.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cut it down; why <qex>cumbereth</qex> it the ground?</q> <rj><qau> Luke xiii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The multiplying variety of arguments, especially frivolous ones, . . . but <qex>cumbers</qex> the memory.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumber</ent><br/
<hw>Cum"ber</hw> <pr>(k?m"b?r)</pr>, <pos>n.</pos> <ety>[Cf. <ets>encombre</ets> hindrance, impediment. See Cuber,<pos>v.</pos>]</ety> <def>Trouble; embarrassment; distress.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>comber</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>A place of much distraction and <qex>cumber</qex>.</q> <rj><qau> Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sage counsel in <qex>cumber</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumberland River</ent><br/
<ent>Cumberland</ent><br/
<mhw><hw>Cumberland</hw>, <hw>Cumberland River</hw></mhw> <pos>prop. n.</pos> <def>a tributary of the Ohio River.</def><br/
<syn><b>Syn. --</b> .</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cumbersome</ent><br/
<hw>cum"ber*some</hw> <pr>(k<?/m"b<?/r-s<?/m)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Burdensome or hindering, as a weight or drag; embarrassing; vexatious; cumbrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To perform a <qex>cumbersome</qex> obedience.</q> <rj><qau>Sir. P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not easily managed; <as>as, a <ex>cumbersome</ex> contrivance or machine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He holds them in utter contempt, as lumbering, <qex>cumbersome</qex>, circuitous.</q> <rj><qau>I. Taylor.</qau></rj></p>

<p>-- <wordforms><wf>Cum"ber*some*ly</wf>, <pos>adv.</pos> -- <wf>Cum"ber*some*ness</wf>,<pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumbrance</ent><br/
<hw>Cum"brance</hw> <pr>(k?m"br<it>a</it>ns)</pr>, <pos>n.</pos> <def>Encumbrance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Extol not riches then, the toil of fools,<br/
The wise man's <qex>cumbrance</qex>, if not snare.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumbrian</ent><br/
<hw>Cum"bri*an</hw> <pr>(k?m"br?-<ait/n)</pr>, <pos>a.</pos> <def>Pertaining to Cumberland, England, or to a system of rocks found there.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cumbrian system</b></col> <fld>(Geol.)</fld>, <cd>the slate or graywacke system of rocks, now included in the <xex>Cambrian</xex> or <xex>Silurian</xex> system; -- so called because most prominent at Cumberland.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumbrous</ent><br/
<hw>Cum"brous</hw> <pr>(k?m"br?s)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Rendering action or motion difficult or toilsome; serving to obstruct or hinder; burdensome; clogging.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He sunk beneath the <qex>cumbrous</qex> weight.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>cumbrous</qex>and unwieldy style which disfigures English composition so extensively.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Giving trouble; vexatious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A clud of <qex>cumbrous</qex> gnats.</q> <rj><qau>Spenser.</qau></rj></p>

<p>-- <wordforms><wf>Cum"brous*ly</wf>, <pos>adv.</pos> -- <wf>Cum"brous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumene</ent><br/
<hw>Cu"mene</hw> <pr>(k?"m?n)</pr>, <pos>n.</pos> <ety>[From <er>Cumin</er>.]</ety> <fld>(Chem.)</fld> <def>A colorless oily hydrocarbon, <chform>C6H5.C3H7</chform>, obtained by the distillation of cuminic acid; -- called also <altname>cumol</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumfrey</ent><br/
<hw>Cum"frey</hw> <pr>(k?m"fr?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Comfrey</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumic</ent><br/
<hw>Cu"mic</hw> <pr>(k?"m?k)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>See <er>Cuming</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumidine</ent><br/
<hw>Cu"mi*dine</hw> <pr>(k?"m?-d?n <or/ -d?n)</pr>, <pos>n.</pos> <ety>[From <er>Cumin</er>.]</ety> <fld>(Chem.)</fld> <def>A strong, liquid, organic base, <chform>C3H7.C6H4.NH2</chform>, homologous with aniline.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumin</ent><br/
<hw>Cum"in</hw> <pr>(k?m"?n)</pr>, <pos>n.</pos> <ety>[OE.<ets>comin</ets>, AS. <ets>cymen</ets>, fr. L. <ets>cuminum</ets>, Gr. <?/<?/<?/<?/<?/<?/<?/; of Semitic origin, cf. Ar. <ets>kamm<?/n</ets>, Heb. <ets>kamm<?/n</ets>; cf. OF. <ets>comin</ets>, F. <ets>cumin</ets>. Cf. <er>Kummel</er>.]</ety> <fld>(Bot.)</fld> <def>A dwarf umbelliferous plant, somewhat resembling fennel (<spn>Cuminum Cyminum</spn>), cultivated for its seeds, which have a bitterish, warm taste, with an aromatic flavor, and are used like those of anise and caraway.</def> <altsp>[Written also <asp>cummin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Rank-smelling rue, and <qex>cumin</qex> good for eyes.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black cumin</b></col> <fld>(Bot.)</fld>, <cd>a plant (<spn>Nigella sativa</spn>) with pungent seeds, used by the Afghans, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuminic</ent><br/
<hw>Cu*min"ic</hw> <pr>(k?-m?n"?k)</pr>, <pos>a.</pos> <def>Pertaining to, or derived from, cumin, or from oil of caraway; <as>as, <ex>cuminic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cuminic acid</b></col> <fld>(Chem.)</fld>, <cd>white crystalline substance, <chform>C3H7.C6H4.CO2H</chform>, obtained from oil of caraway.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuminil</ent><br/
<hw>Cu"mi*nil</hw> <pr>(k?"m?-n?l)</pr>, <pos>n.</pos> <def>A substance, analogous to benzil, obtained from oil of caraway.</def><-- benzil = benzile? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuminol</ent><br/
<hw>Cu"mi*nol</hw> <pr>(-n?l)</pr>, <pos>n.</pos> <ety>[<ets>Cuminic</ets> + L. <ets>ole</ets>um.]</ety> <def>A liquid, <chform>C3H7.C6H4.CHO</chform>, obtained from oil of caraway; -- called also <altname>cuminic aldehyde</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cummerbund</ent><br/
<hw>Cum"mer*bund`</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>kummerbund</asp>, <asp>cummerband</asp>, etc.]</altsp> <ety>[Hind. <ets>kamarband</ets>, fr. Per. <ets>Kamar</ets> loins + <ets>band</ets> fastening.]</ety> <def>A sash for the waist; a girdle.</def> <mark>[India]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cummin</ent><br/
<hw>Cum"min</hw> <pr>(k?m"m?n)</pr>, <pos>n.</pos> <def>Same as <er>Cumin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye pay tithe of mint, and <qex>cummin</qex>.</q> <rj><qau> Matt. xxiii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cumquat</ent><br/
<hw>cum"quat</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Kumquat</er>.</def><br/
<syn><b>Syn. --</b> kumquat, kumquat tree.</syn><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cumshaw</ent><br/
<hw>Cum"shaw</hw> <pr>(k?m"sha)</pr>, <pos>n.</pos> <ety>[Chin. <ets>kom-tsie</ets>.]</ety> <def>A present or bonus; -- originally applied to that paid on ships which entered the port of Canton.</def>  <rj><au> S. Wells Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumshaw</ent><br/
<hw>Cum"shaw</hw>, <pos>v. t.</pos> <def>To give or make a present to.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumu-cirro-stratus</ent><br/
<hw>Cu"mu-cir`ro-stra"tus</hw> <pr>(k?`m?-s?r`r?-str?"t?s)</pr>, <pos>n.</pos> <fld>(Meteor.)</fld> <def>Nimbus, or rain cloud. See <er>Nimbus</er>, and <er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulate</ent><br/
<hw>Cu"mu*late</hw> <pr>(k?"m?-l?t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cumulated</conjf> <pr>(-l?`t?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cumulating</conjf> <pr>(-l?`t?ng)</pr>.]</vmorph> <ety>[L. <ets>cumulatus</ets>, p. p. of <ets>cumulare</ets> to heap up, fr. <ets>cumulus</ets> a heap. See <er>Cumber</er>.]</ety> <def>To gather or throw into a heap; to heap together; to accumulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shoals of shells, bedded and <qex>cumulated</qex> heap upon heap.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulation</ent><br/
<hw>Cu`mu*la"tion</hw> <pr>(k?`m?-l?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cumulation</ets>.]</ety> <def>The act of heaping together; a heap. See <er>Accumulation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulatist</ent><br/
<hw>Cu"mu*la*tist</hw> <pr>(k?"m?-l?-t?st)</pr>, <pos>n.</pos> <def>One who accumulates; one who collects.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulative</ent><br/
<hw>Cu"mu*la*tive</hw> <pr>(k?"m?-l?-t?v)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cumulatif</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Composed of parts in a heap; forming a mass; aggregated.</def> <ldquo/As for knowledge which man receiveth by teaching, it is <xex>cumulative</xex>, not original.<rdquo/  <rj><au>Bacon</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Augmenting, gaining, or giving force, by successive additions; <as>as, a <ex>cumulative</ex> argument, <it>i. e.</it>, one whose force increases as the statement proceeds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The argument . . . is in very truth not logical and single, but moral and <qex>cumulative</qex>.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>Tending to prove the same point to which other evidence has been offered; -- said of evidence.</def> <sd>(b)</sd> <def>Given by same testator to the same legatee; -- said of a legacy.</def>  <rj><au>Bouvier.</au>  <au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 356 --></p>

<p><cs><col><b>Cumulative action</b></col> <fld>(Med.)</fld>, <cd>that action of certain drugs, by virtue of which they produce, when administered in small doses repeated at considerable intervals, the same effect as if given in a single large dose.</cd> -- <col><b>Cumulative poison</b></col>, <cd>a poison the action of which is cumulative.</cd> -- <mcol><col><b>Cumulative vote</b></col> <it>or</it> <col><b>Cumulative system of voting</b></col></mcol> <fld>(Politics)</fld>, <cd>that system which allows to each voter as many votes as there are persons to be voted for, and permits him to accumulate these votes upon one person, or to distribute them among the candidates as he pleases.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulose</ent><br/
<hw>Cu"mu*lose`</hw> <pr>(k<umac/"m<usl/*l<omac/s`)</pr>, <pos>a.</pos> <ety>[From <er>Cumulus</er>.]</ety> <def>Full of heaps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulostratus</ent><br/
<hw>Cu"mu*lo*stra"tus</hw> <pr>(k<umac/"m<usl/*l<osl/*str<amac/"t<ucr/s)</pr>, <pos>n.</pos> <fld>(Meteor.)</fld> <def>A form of cloud. See <er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cumulus</ent><br/
||<hw>Cu"mu*lus</hw> <pr>(k<umac/"m<usl/*l<ucr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cumuli</plw> <pr>(-l<imac/)</pr>.</plu> <ety>[L., a heap. See <er>Cumber</er>.]</ety> <fld>(Meteor.)</fld> <def>One of the four principal forms of clouds. See<er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cun</ent><br/
<hw>Cun</hw> <pr>(k<ucr/n)</pr>, <pos>v. t.</pos> <ety>[See <er>Cond</er>.]</ety> <def>To con (a ship).</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cun</ent><br/
<hw>Cun</hw>, <pos>v. t.</pos> <ety>[See 1st <er>Con</er>.]</ety> <def>To know. See <er>Con</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunabula</ent><br/
||<hw>Cu*nab"u*la</hw> <pr>(k<usl/*n<acr/b"<usl/*l<adot/)</pr>, <pos>n. pl.</pos> <ety>[L., a cradle, earliest abode, fr. <ets>cunae</ets> cradle.]</ety> <sn>1.</sn> <def>The earliest abode; original dwelling place; originals; <as>as, the <ex>cunabula</ex> of the human race</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bibliography)</fld> <def>The extant copies of the first or earliest printed books, or of such as were printed in the 15th century.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunctation</ent><br/
<hw>Cunc*ta"tion</hw> <pr>(k?nk-t?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>cunctatio</ets>, fr. <ets>cunctari</ets>, p. p. <ets>cunctatus</ets>, to delay.]</ety> <def>Delay; procrastination.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunctative</ent><br/
<hw>Cunc"ta*tive</hw> <pr>(k?nk"t?-t?v)</pr>, <pos>a.</pos> <def>Slow; tardy; dilatory; causing delay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunctator</ent><br/
||<hw>Cunc*ta"tor</hw> <pr>(k?nk-t?"t?r)</pr>, <pos>n.</pos> <ety>[L., lit., a delayer; -- applied as a surname to <person>Q. Fabius Maximus</person>.]</ety> <def>One who delays or lingers.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunctipotent</ent><br/
<hw>Cunc*tip"o*tent</hw> <pr>(k?nk-t?p"?-t<it>e</it>nt)</pr>, <pos>a.</pos> <ety>[L. <ets>cunctipotens</ets>; <ets>cunctus</ets> all + <ets>potens</ets> powerful.]</ety> <def>All-powerful; omnipotent.</def> <mark>[R]</mark> <ldquo/God <xex>cunctipotent</xex>.<rdquo/  <rj><au>Neale (Trans. Rhythm of St. Bernard).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cund</ent><br/
<hw>Cund</hw> <pr>(k<ucr/nd)</pr>, <pos>v. t.</pos> <ety>[See <er>Cond</er>.]</ety> <def>To con (a ship).</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cundurango</ent><br/
<hw>Cun`du*ran"go</hw> <pr>(k?n`d?-r?n"g?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The bark of a South American vine (<spn>Gonolobus Condurango</spn>) of the Milkweed family. It has been supposed, but erroneously, to be a cure for cancer.</def> <altsp>[Written also <asp>condurango</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuneal</ent><br/
<hw>Cu"ne*al</hw> <pr>(k?"n?-<ait/l)</pr>, <ety>[L. <ets>cuneus</ets> a wege. See <er>Coin</er>.]</ety> <def>Relating to a wedge; wedge-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuneated</ent><br/
<ent>Cuneate</ent><br/
<mhw>{ <hw>Cu"ne*ate</hw> <pr>(k?"n?-?t)</pr>, <hw>Cu"ne*a`ted</hw> <pr>(-?`tEd)</pr>, }</mhw> <pos>a.</pos> <ety>[L.<ets>cuneatus</ets>, fr. <ets>cuneus</ets> a wege See<er>Coin</er>.]</ety> <def>Wedge-shaped</def>; <fld>(Bot.)</fld>, <def>wedge-shaped, with the point at the base; <as>as, a <ex>cuneate</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuneatic</ent><br/
<hw>Cu`ne*at"ic</hw> <pr>(k?`n?-?t"?k)</pr>, <pos>a.</pos> <def>Cuneiform.</def> <ldquo/<xex>Cuneatic</xex> decipherment.<rdquo/  <rj><au>Sayce.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuniform</ent><br/
<ent>Cuneiform</ent><br/
<mhw>{ <hw>Cu*ne"i*form</hw> <pr>(k?-n?"?-f?rm)</pr>, <hw>Cu"ni*form</hw> <pr>(k?"n?-f?rm)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cuneus</ets> a wedge + <ets>-form</ets>: cf. F. <ets>cunei-forme</ets>. See <er>Coin</er>.]</ety> <sn>1.</sn> <def>Wedge-shaped; <as>as, a <ex>cuneiform</ex> bone</as>; -- especially applied to the wedge-shaped or arrowheaded characters of ancient Persian and Assyrian inscriptions. See <er>Arrowheaded</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to, or versed in, the ancient wedge-shaped characters, or the inscriptions in them.</def> <ldquo/A <xex>cuneiform</xex> scholar.<rdquo/  <rj><au>Rawlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuniform</ent><br/
<ent>Cuneiform</ent><br/
<mhw>{ <hw>Cu*ne"i*form</hw>, <hw>Cu"ni*form</hw>  }</mhw>, <pos>n.</pos> <sn>1.</sn> <def>The wedge-shaped characters used in ancient Persian and Assyrian inscriptions.</def>  <rj><au>I. Taylor (The Alphabet).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>One of the three tarsal bones supporting the first, second third metatarsals. They are usually designated as external, middle, and internal, or <stype>ectocuniform</stype>, <stype>mesocuniform</stype>, and <stype>entocuniform</stype>, respectively.</def> <sd>(b)</sd> <def>One of the carpal bones usually articulating with the ulna; -- called also <altname>pyramidal</altname> and <altname>ulnare</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunette</ent><br/
||<hw>Cu*nette"</hw> <pr>(k<usl/*n<ecr/t")</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Fort.)</fld> <def>A drain trench, in a ditch or moat; -- called also <altname>cuvette</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunner</ent><br/
<hw>Cun"ner</hw> <pr>(k<ucr/n"n<etil/r)</pr>, <pos>n.</pos> <ety>[Cf. <er>Conner</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small edible fish of the Atlantic coast (<spn>Ctenolabrus adspersus</spn>); -- called also <altname>chogset</altname>, <altname>burgall</altname>, <altname>blue perch</altname>, and <altname>bait stealer</altname>.</def> <altsp>[Written also <asp>conner</asp>.]</altsp> <sd>(b)</sd> <def>A small shellfish; the limpet or patella.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cunnilinctus</ent><br/
<hw>cun`ni*linc"tus</hw> <pos>n.</pos> <def>Same as <er>cunnilingus</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cunnilingus</ent><br/
<hw>cun`ni*lin"gus</hw> <pos>n.</pos> <ety>[NL., fr. L. <ets>cunnus</ets> vulva + <ets>lingere</ets> to lick.]</ety> <def>Stimulation of the vulva or clitoris of one person by the tongue of another, for the purpose of giving sexual gratification.</def><br/
<syn><b>Syn. --</b> cunnilinctus.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cunning</ent><br/
<hw>Cun"ning</hw> <pr>(k<ucr/n"n<icr/ng)</pr>, <pos>a.</pos> <ety>[AS. <ets>cunnan</ets> to know, to be able. See 1st <er>Con</er>, <er>Can</er>.]</ety> <sn>1.</sn> <def>Knowing; skillful; dexterous.</def> <ldquo/A <xex>cunning</xex> workman.<rdquo/  <rj><au> Ex. xxxviii. 23.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Tis beauty truly blent, whose red and white<br/
Nature's own sweet and <qex>cunning</qex> hand laid on.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Esau was a <qex>cunning</qex> hunter.</q> <rj><qau>Gen xxv. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wrought with, or exhibiting, skill or ingenuity; ingenious; curious; <as>as, <ex>cunning</ex> work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Over them Arachne high did lift</q>
<q>Her <qex>cunning</qex> web.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Crafty; sly; artful; designing; deceitful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are resolved to be <qex>cunning</qex>; let others run the hazard of being sincere.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Pretty or pleasing; <as>as, a <ex>cunning</ex> little boy</as>.</def> <mark>[Colloq. U.S.]</mark>  <rj><au>Barlett.</au></rj></p>

<p><syn><b>Syn.</b> -- <er>Cunning</er>, <er>Artful</er>, <er>Sly</er>, <er>Wily</er>, <er>Crafty</er>.</syn> -- <usage>These epithets agree in expressing an aptitude for attaining some end by peculiar and secret means. <xex>Cunning</xex> is usually low; as, a <xex>cunning</xex> trick. <xex>Artful</xex> is more ingenious and inventive; as, an <xex>artful</xex> device. <xex>Sly</xex> implies a turn for what is double or concealed; as, <xex>sly</xex> humor; a <xex>sly</xex> evasion. <xex>Crafty</xex> denotes a talent for dexterously deceiving; as, a <xex>crafty</xex> manager. <xex>Wily</xex> describes a talent for the use of stratagems; as, a <xex>wily</xex> politician.  A <xex>cunning</xex> man often shows his dexterity in simply concealing. An <xex>artful</xex> man goes further, and exerts his ingenuity in misleading. A <xex>crafty</xex> man mingles cunning with art, and so shapes his actions as to lull suspicions. The young may be <xex>cunning</xex>, but the experienced only can be <xex>crafty</xex>. <xex>Slyness</xex> is a vulgar kind of cunning; the <xex>sly</xex> man goes cautiously and silently to work. <xex>Wiliness</xex> is a species of cunning or craft applicable only to cases of attack and defense.<rdquo/  <rj><au>Crabb.</au></rj></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunning</ent><br/
<hw>Cun"ning</hw>, <pos>n.</pos> <ety>[AS. <ets>cunnung</ets> trial, or Icel. <ets>kunnandi</ets> knowledge. See <er>Cunning</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Knowledge; art; skill; dexterity.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let my right hand forget her <qex>cunning</qex>.</q> <rj><qau>Ps. cxxxvii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A carpenter's desert<br/
Stands more in <qex>cunning</qex> than in power.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The faculty or act of using stratagem to accomplish a purpose; fraudulent skill or dexterity; deceit; craft.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Discourage <qex>cunning</qex> in a child; <qex>cunning</qex> is the ape of wisdom.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We take <qex>cunning</qex> for a sinister or crooked wisdom.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cunningly</ent><br/
<hw>Cun"ning*ly</hw> <pr>(k?n"n?ng-l?)</pr>, <pos>adv.</pos> <def>In a cunning manner; with cunning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cunningman</ent><br/
<hw>cun"ning*man`</hw> <pr>(-m?n`)</pr>, <pos>n.</pos> <def>A fortune teller; one who pretends to reveal mysteries.</def> <mark>[Obs.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cunningness</ent><br/
<hw>cun"ning*ness</hw>, <pos>n.</pos> <def>Quality of being cunning; craft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cunt</ent><br/
<hw>cunt</hw>, <pos>n.</pos> <sn>1.</sn> <def>The female pudenda; <specif>specifically</specif> the vagina.</def> <mark>[vulgar slang]</mark><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>A woman; -- usually used derogatorily and considered obscene.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cuon</ent><br/
<hw>Cuon</hw> <pos>prop. n.</pos> <def>a genus of canids including the Asiatic wild dog.</def><br/
<syn><b>Syn. --</b> Cyon, genus <gen>Cuon</gen>, genus <gen>Cyon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cup</ent><br/
<hw>Cup</hw> <pr>(k<ucr/p)</pr>, <pos>n.</pos> <ety>[AS. <ets>cuppe</ets>, LL. <ets>cuppa</ets> cup; cf. L. <ets>cupa</ets> tub, cask; cf. also Gr. <grk>ky`ph</grk> hut, Skr. <ets>k<umac/pa</ets> pit, hollow, OSlav.  <ets>kupa</ets> cup. Cf. <er>Coop</er>, <er>Cupola</er>, <er>Cowl</er> a water vessel, and <er>Cob</er>, <er>Coif</er>, <er>Cop</er>.]</ety> <sn>1.</sn> <def>A small vessel, used commonly to drink from; <as>as, a tin <ex>cup</ex>, a silver <ex>cup</ex>, a wine <ex>cup</ex></as>; especially, in modern times, the pottery or porcelain vessel, commonly with a handle, used with a saucer in drinking tea, coffee, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The contents of such a vessel; a cupful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give me a <qex>cup</qex> of sack, boy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Repeated potations; social or excessive indulgence in intoxicating drinks; revelry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thence from <qex>cups</qex> to civil broils.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is to be received or indured; that which is allotted to one; a portion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O my Father, if it be possible, let this <qex>cup</qex> pass from me.</q> <rj><qau>Matt. xxvi. 39.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything shaped like a cup; <as>as, the <ex>cup</ex> of an acorn, or of a flower</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cowslip's golden <qex>cup</qex> no more I see.</q> <rj><qau>Shenstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <def>A cupping glass or other vessel or instrument used to produce the vacuum in cupping.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cup and ball</b></col>, <cd>a familiar toy of children, having a cup on the top of a piece of wood to which, a ball is attached by a cord; the ball, being thrown up, is to be caught in the cup; bilboquet.</cd> <au>Milman.</au> -- <col><b>Cup and can</b></col>, <cd>familiar companions.</cd> -- <mcol><col><b>Dry cup</b></col>, <col><b>Wet cup</b></col></mcol> <fld>(Med.)</fld>, <cd>a cup used for <xex>dry</xex> or <xex>wet cupping</xex>. See under <er>Cupping</er>.</cd> -- <col><b>To be in one's cups</b></col>, <cd>to be drunk.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cup</ent><br/
<hw>Cup</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cupped</conjf> <pr>(k<ucr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cupping</conjf>.]</vmorph> <sn>1.</sn> <def>To supply with cups of wine.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Cup</qex> us, till the world go round.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>To apply a cupping apparatus to; to subject to the operation of cupping. See <er>Cupping</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>To make concave or in the form of a cup; <as>as, to <ex>cup</ex> the end of a screw</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupbearer</ent><br/
<hw>Cup"bear`er</hw> <pr>(-b<acir/r`<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One whose office it is to fill and hand the cups at an entertainment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Antiq.)</fld> <def>One of the attendants of a prince or noble, permanently charged with the performance of this office for his master.</def> <ldquo/I was the king's <xex>cupbearer</xex>.<rdquo/  <rj><au>Neh. i. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupboard</ent><br/
<hw>Cup"board</hw> <pr>(k<ucr/b"b<etil/rd)</pr>, <pos>n.</pos> <ety>[Cup + <ets>board</ets>.]</ety> <sn>1.</sn> <def>A board or shelf for cups and dishes.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small closet in a room, with shelves to receive cups, dishes, food, etc.; hence, any small closet.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cupboard love</b></col>, <cd>interested love, or that which has an eye to the cupboard.</cd>  <ldquo/A <xex>cupboard love</xex> is seldom true.<rdquo/ <au>Poor Robin.</au> <mark>[Colloq.]</mark> -- <col><b>To cry cupboard</b></col>, <cd>to call for food; to express hunger.</cd>  <mark>[Colloq.]</mark> <ldquo/My stomach cries cupboard.<rdquo/ <au>W. Irving.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupboard</ent><br/
<hw>Cup"board</hw>, <pos>v. t.</pos> <def>To collect, as into a cupboard; to hoard.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupel</ent><br/
<hw>Cu"pel</hw> <pr>(k<umac/"p<ecr/l)</pr>, <pos>n.</pos> <ety>[LL. <ets>cupella</ets> cup (cf. L. <ets>cupella</ets>, small cask, dim. of <ets>cupa</ets>) : cf. F. <ets>coupelle</ets>. See <er>Cup</er>, and cf. <er>Coblet</er>.]</ety> <def>A shallow porous cup, used in refining precious metals, commonly made of bone ashes (phosphate of lime).</def> <altsp>[Written also <asp>coppel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cupel dust</b></col>, <cd>powder used in purifying metals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupel</ent><br/
<hw>Cu*pel"</hw> <pr>(k<usl/*p<ecr/l")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cupelled</conjf> <pr>(-p?ld")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cupelling</conjf>.]</vmorph> <def>To refine by means of a cupel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupellation</ent><br/
<hw>Cu`pel*la"tion</hw> <pr>(k<umac/`p<ecr/l*l<amac/"sh<ucr/n)</pr> <pos>n.</pos> <ety>[See <er>Cupel</er>.]</ety> <def>The act or process of refining gold or silver, etc., in a cupel.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The process consist in exposing the cupel containing the metal to be assayed or refined to a hot blast, by which the lead, copper, tin, etc., are oxidized, dissolved, and carried down into the porous cupel, leaving the unoxidizable precious metal. If lead is not already present in the alloy it must be added before cupellation.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupful</ent><br/
<hw>Cup"ful</hw> <pr>(k?p"f?l)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cupfuls</plw> <pr>(-f<?/lz)</pr>.</plu> <def>As much as a cup will hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cup-gall</ent><br/
<hw>Cup"-gall`</hw> <pr>(-g?l`)</pr>, <pos>n.</pos> <def>A kind of oak-leaf gall. See <er>Gall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupid</ent><br/
<hw>Cu"pid</hw> <pr>(k?"p?d)</pr>, <pos>n.</pos> <ety>[L.<ets>Cupido</ets>, fr. <ets>cupido</ets> desire, desire of love, fr. <ets>cupidus</ets>. See <er>Cupidity</er>.]</ety> <fld>(Rom. Myth.)</fld> <def>The god of love, son of Venus; usually represented as a naked, winged boy with bow and arrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pretty dimpled boys, like smiling <qex>cupids</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupidity</ent><br/
<hw>Cu*pid"i*ty</hw> <pr>(k?-p?d"?-t?)</pr>, <pos>n.</pos> <ety>[F. <ets>cupidite</ets>, L. <ets>cupiditas</ets>, fr. <ets>cupidus</ets> longing, desiring, fr. <ets>cupere</ets> to long for, desire. See <er>Covet</er>.]</ety> <sn>1.</sn> <def>A passionate desire; love.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Eager or inordinate desire, especially for wealth; greed of gain; avarice; covetousness</def>.<br/
[<source>1913 Webster</source>]</p>

<p><q>With the feelings of political distrust were mingled those of <qex>cupidity</qex> and envy, as the Spaniard saw the fairest provinces of the south still in the hands of the accursed race of Ishmael.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cup-moss</ent><br/
<hw>Cup"-moss`</hw> <pr>(k?p"m?s`; 115)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of lichen, of the genus <gen>Cladonia</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupola</ent><br/
<hw>Cu"po*la</hw> <pr>(k?"p?-l?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cupolas</plw> <pr>(-l<?/z)</pr>.</plu> <ety>[It. <ets>cupola</ets>, LL. <ets>cupula</ets>, <ets>cuppula</ets> (cf. L. <ets>cupula</ets> little tub). fr. <ets>cupa</ets>, <ets>cuppa</ets>, cup; cf. L. <ets>cupa</ets> tub. So called on account of its resemblance to a cup turned over. See <er>Cup</er>, and cf. <er>Cupule</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A roof having a rounded form, hemispherical or nearly so; also, a ceiling having the same form. When on a large scale it is usually called <stype>dome</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small structure standing on the top of a dome; a lantern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A furnace for melting iron or other metals in large quantity, -- used chiefly in foundries and steel works.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A revolving shot-proof turret for heavy ordnance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Anat.)</fld> <def>The top of the spire of the cochlea of the ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cuppa</ent><br/
<hw>cuppa</hw> <pos>n.</pos> <def>a cup of tea.</def> <mark>[British]</mark> <br/
<syn><b>Syn. --</b> cupper.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cupper</ent><br/
<hw>Cup"per</hw> <pr>(k<ucr/p"p<etil/r)</pr>, <pos>n.</pos> <ety>[Fropm <ets>cup</ets>.]</ety> <def>One who performs the operation of cupping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupping</ent><br/
<hw>Cup"ping</hw> <pr>(k?p"p?ng)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The operation of drawing blood to or from the surface of the person by forming a partial vacuum over the spot. Also, sometimes, a similar operation for drawing pus from an abscess.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cupping glass</b></col>, <cd>a glass cup in which a partial vacuum is produced by heat, in the process of cupping.</cd> -- <col><b>Dry cupping</b></col>, <cd>the application of a cupping instrument without scarification, to draw blood to the surface, produce counter irritation, etc.</cd> -- <col><b>Wet cupping</b></col>, <cd>the operation of drawing blood by the application of a cupping instrument after scarification.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuppy</ent><br/
<hw>Cup"py</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Hollow; cuplike; also, full of cups, or small depressions.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Characterized by cup shakes; -- said of timber.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cupreous</ent><br/
<hw>Cu"pre*ous</hw> <pr>(k?"pr?-?s)</pr>, <pos>a.</pos> <ety>[L. <ets>cupreus</ets>, fr.  <ets>cuprum</ets>.]</ety> <def>Consisting of copper or resembling copper; coppery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupressus</ent><br/
<hw>Cupressus</hw> <pos>n.</pos> <def>one of the genera of cypress trees, the type genus of the <fam>Cupressaceae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Cupressus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cupric</ent><br/
<hw>Cu"pric</hw> <pr>(k?"pr?k)</pr>, <pos>a.</pos> <ety>[From <er>Cuprum</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, copper; containing copper; -- said of those compounds of copper in which this element is present in its lowest proportion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupriferous</ent><br/
<hw>Cu*prif"er*ous</hw> <pr>(k?-pr?f"?r-?s)</pr>, <pos>a.</pos> <ety>[<ets>Cuprum</ets> + <ets>-ferous</ets>.]</ety> <def>Containing copper; <as>as, <ex>cupriferous</ex> silver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuprite</ent><br/
<hw>Cu"prite</hw> <pr>(k?"pr?t)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>The red oxide of copper; red copper; an important ore of copper, occurring massive and in isometric crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuproid</ent><br/
<hw>Cu"proid</hw> <pr>(k?"proid)</pr>, <pos>n.</pos> <ety>[<ets>Cuprum</ets> + <ets>-oid</ets>.]</ety> <fld>(Crystalloq.)</fld> <def>A solid related to a tetrahedron, and contained under twelve equal triangles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cup-rose</ent><br/
<hw>Cup"-rose</hw> <pr>(k?p"r?z)</pr>, <pos>n.</pos> <def>Red poppy. See <er>Cop-rose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuprous</ent><br/
<hw>Cu"prous</hw> <pr>(k?"pr?s)</pr>, <pos>a.</pos> <ety>[From <er>Cuprum</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, copper; containing copper; -- said of those compounds of copper in which this element is present in its highest proportion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuprum</ent><br/
||<hw>Cu"prum</hw> <pr>(k?"pr?m)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Chem.)</fld> <def>Copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cup shake</ent><br/
<hw>Cup shake</hw>. <fld>(Forestry)</fld> <def>A shake or fissure between the annual rings of a tree, found oftenest near the roots.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>cup-shaped</ent><br/
<hw>cup-shaped</hw> <pos>adj.</pos>  <def>shaped like a cup.</def><br/
<syn><b>Syn. --</b> cupular, cupulate.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cupulate</ent><br/
<hw>Cu"pu*late</hw> <pr>(k?"p?-l?t)</pr>, <pos>a.</pos> <def>Having or bearing cupules; cupuliferous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupule</ent><br/
<hw>Cu"pule</hw> <pr>(k?"p?l)</pr>, <pos>n.</pos> <ety>[See <er>Cupola</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A cuplet or little cup, as of the acorn; the husk or bur of the filbert, chestnut, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A sucker or acetabulum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cupuliferous</ent><br/
<hw>Cu`pu*lif"er*ous</hw> <pr>(k?`p?-l?f"?r-?s)</pr>, <pos>a.</pos> <ety>[<ets>Cupule</ets> + <ets>-ferous</ets>: cf. F. <ets>cupulif<egrave/re</ets>.]</ety> <def>Of, pertaining to, or resembling, the family of plants of which the oak and the chestnut are examples, -- trees bearing a smooth, solid nut inclosed in some kind of cup or bur; bearing, or furnished with, a cupule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cur</ent><br/
<hw>Cur</hw> <pr>(k<ucir/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>curre</ets>, <ets>kur</ets>; cf. dial. Sw. <ets>kurre</ets> dog, OD.  <ets>korre</ets> watchdog, and Icel. <ets>kurra</ets> to murmur, grumble, Sw. <ets>kurra</ets> to rumble, croak, Dan. <ets>kurre</ets> to coo, whirr; prob. of imitative origin.]</ety> <sn>1.</sn> <def>A mongrel or inferior dog.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . like to village <qex>curs</qex>,<br/
Bark when their fellows do.</q> <rj><qau> Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A worthless, snarling fellow; -- used in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What would you have, you <qex>curs</qex>,<br/
That like nor peace nor war?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curability</ent><br/
<hw>Cur`a*bil"i*ty</hw> <pr>(k?r`?-b?l"?-t?)</pr>, <pos>n.</pos> <def>The state of being curable; curableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curable</ent><br/
<hw>Cur"a*ble</hw> <pr>(k?r"?-b'l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>curable</ets>. See <er>Cure</er>, <pos>v. t.</pos>]</ety> <def>Capable of being cured; admitting remedy.</def> <ldquo/<xex>Curable</xex> diseases.<rdquo/ <au>Harvey.</au> -- <wordforms><wf>Cur"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Cur`a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curacoa</ent><br/
<ent>Cura<cced/ao</ent><br/
<mhw>{ <hw>Cu`ra*<cced/ao"</hw>, <hw>Cu`ra*<cced/oa"</hw>, }</mhw> <pr>(k??`r?-s?")</pr>, <pos>n.</pos> <def>A liqueur, or cordial, flavored with the peel from the sour orange, and sometimes with cinnamon and mace; -- first made at the island of <etsep>Cura<cced/cao</etsep> in the Netherlands Antilles near <country>Venezuela</country>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Curacy</ent><br/
<hw>Cu"ra*cy</hw> <pr>(k?"r?-s?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curacies</plw> <pr>(-s<?/z)</pr>.</plu> <ety>[See <er>Cure</er>, <er>Curate</er>.]</ety> <def>The office or employment of a curate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curari</ent><br/
<ent>Curare</ent><br/
<mhw>{ <hw>Cu*ra"re</hw> <hw>Cu*ra"ri</hw> }</mhw> <pr>(k?-r?"r?)</pr>, <pos>n.</pos> <ety>[Native name. Cf. <er>Wourall</er>.]</ety> <def>A black resinoid extract prepared by the South American Indians from the bark of several species of <gen>Strychnos</gen> (<spn>Strychnos toxifera</spn>, etc.). It sometimes has little effect when taken internally, but is quickly fatal when introduced into the blood, and used by the Indians as an arrow poison.</def> <altsp>[Written also <asp>urari</asp>, <asp>woorali</asp>, <asp>woorari</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curarine</ent><br/
<hw>Cu"ra*rine</hw> <pr>(k?"r?-r?n <or/ k?-r?"r?n; 104)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A deadly alkaloid extracted from the curare poison and from the <spn>Strychnos toxifera</spn>. It is obtained in crystalline colorless salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curarize</ent><br/
<hw>Cu"ra*rize</hw> <pr>(-r?z)</pr>, <pos>v. t.</pos> <def>To poison with curare.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curassow</ent><br/
<hw>Cu*ras"sow</hw> <pr>(k?-r?s"s?)</pr>, <pos>n.</pos> <ety>[Native name in Brazil.]</ety> <fld>(Zool.)</fld> <def>A large gallinaceous bird of the American genera <gen>Crax</gen>, <gen>Ourax</gen>, etc., of the family <fam>Cracid<ae/</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The crested curassow (<spn>Crax alector</spn>) is black, and about the size of a small hen-turkey, with an erectile crest of curled feathers. It ranges from Mexico to Brazil. The galeated curassow or cushew bird (<spn>Ourax Pauxi</spn>) is similar in size, and has a large, hollow, blue, pear-shaped protuberance on the head.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curat</ent><br/
<hw>Cu"rat</hw> <pr>(k?"r?t)</pr>, <pos>n.</pos> <ety>[See<er>Cuirass</er>.]</ety> <def>A cuirass or breastplate.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 357 --></p>

<p><ent>Curate</ent><br/
<hw>Cu"rate</hw> <pr>(k<umac/"r<asl/t)</pr>, <pos>n.</pos> <ety>[LL. <ets>curatus</ets>, prop., one who is charged with the care (L. <ets>cura</ets>) of souls. See <er>Cure</er>, <pos>n.</pos>, and cf. <er>Cur<eacute/</er>]</ety> <def>One who has the cure of souls; originally, any clergyman, but now usually limited to one who assists a rector or vicar.</def>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All this the good old man performed alone,<br/
He spared no pains, for <qex>curate</qex> he had none.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curateship</ent><br/
<hw>Cu"rate*ship</hw>, <pos>n.</pos> <def>A curacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curation</ent><br/
<hw>Cu*ra"tion</hw> <pr>(k?-r?"sh?n)</pr>, <pos>n.</pos> <ety>[Cf. OF.<ets>curacion</ets>.]</ety> <def>Cure; healing.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curative</ent><br/
<hw>Cur"a*tive</hw> <pr>(k?r"?-t?v)</pr>, <pos>a.</pos> <ety>[Cf. F.<ets>curatif</ets>. See <er>Cure</er>, <pos>v. t.</pos>]</ety> <def>Relating to, or employed in, the cure of diseases; tending to cure.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curator</ent><br/
<hw>Cu*ra"tor</hw> <pr>(k?-r?"t?r)</pr>. <pos>n.</pos> <ety>[L., fr. <ets>curare</ets> to take care of, fr. <ets>cura</ets> care.]</ety> <sn>1.</sn> <def>One who has the care and superintendence of anything, as of a museum; a custodian; a keeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One appointed to act as guardian of the estate of a person not legally competent to manage it, or of an absentee; a trustee; a guardian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curatorial</ent><br/
<hw>curatorial</hw> <pos>adj.</pos> <def>of or pertaining to curator; <as>as, <ex>curatorial</ex> duties</as>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curatorship</ent><br/
<hw>Cu*ra"tor*ship</hw>, <pos>n.</pos> <def>The office of a curator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curatrix</ent><br/
<hw>Cu*ra"trix</hw> <pr>(-tr?ks)</pr>, <pos>n.</pos> <ety>[L.]</ety> <sn>1.</sn> <def>A woman who cures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A woman who is a guardian or custodian.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curb</ent><br/
<hw>Curb</hw> <pr>(k<ucir/rb)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curbed</conjf> <pr>(k<ucir/rbd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curbing</conjf>.]</vmorph> <ety>[F. <ets>courber</ets> to bend, curve, L.<ets>curvare</ets>, fr. <ets>curvus</ets> bent, curved; cf. Gr. <grk>kyrto`s</grk> curved. Cf. <er>Curve</er>.]</ety> <sn>1.</sn> <def>To bend or curve.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Crooked and <qex>curbed</qex> lines.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To guide and manage, or restrain, as with a curb; to bend to one's will; to subject; to subdue; to restrain; to confine; to keep in check.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Part wield their arms, part <qex>curb</qex> the foaming steed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where pinching want must <qex>curb</qex> thy warm desires.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To furnish with a curb, as a well; also, to restrain by a curb, as a bank of earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curb</ent><br/
<hw>Curb</hw>, <pos>v. i.</pos> <def>To bend; to crouch; to cringe.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Virtue itself of vice must pardon beg,<br/
Yea, <qex>curb</qex> and woo for leave to do him good.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curb</ent><br/
<hw>Curb</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which curbs, restrains, or subdues; a check or hindrance; esp., a chain or strap attached to the upper part of the branches of a bit, and capable of being drawn tightly against the lower jaw of the horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that before ran in the pastures wild<br/
Felt the stiff <qex>curb</qex> control his angry jaws.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>By these men</qex>, <qex>religion</qex>,<qex>that should be</qex><br/
The <qex>curb</qex>, is made the spur of tyranny.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An assemblage of three or more pieces of timber, or a metal member, forming a frame around an opening, and serving to maintain the integrity of that opening; also, a ring of stone serving a similar purpose, as at the eye of a dome.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A frame or wall round the mouth of a well; also, a frame within a well to prevent the earth caving in.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A curbstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Far.)</fld> <def>A swelling on the back part of the hind leg of a horse, just behind the lowest part of the hock joint, generally causing lameness.</def>  <rj><au>James Law.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curb bit</b></col>, <cd>a stiff bit having branches by which a leverage is obtained upon the jaws of horse.</cd> <au>Knight.</au> -- <col><b>Curb pins</b></col> <fld>(Horology)</fld>, <cd>the pins on the regulator which restrain the hairspring.</cd> -- <col><b>Curb plate</b></col> <fld>(Arch.)</fld>, <cd>a plate serving the purpose of a curb.</cd> -- <col><b>Deck curb</b></col>. <cd>See under <er>Deck</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>curbed</ent><br/
<hw>curbed</hw> <pos>adj.</pos> <def>held back from some action, especially by force.  Opposite of <ant>unrestrained</ant>.</def><br/
<syn><b>Syn. --</b> checked, restrained.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curbless</ent><br/
<hw>Curb"less</hw>, <pos>a.</pos> <def>Having no curb or restraint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curb roof</ent><br/
<hw>Curb" roof`</hw> <pr>(r??f`)</pr>. <def>A roof having a double slope, or composed, on each side, of two parts which have unequal inclination; a gambrel roof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curbside</ent><br/
<hw>curbside</hw> <pos>n.</pos> <def>the edge of a sidewalk that borders a curb; <as>as, policemen stood at intervals along the <ex>curbside</ex></as></def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curbstone</ent><br/
<hw>Curb"stone`</hw> <pr>(k?rb"st?n`)</pr>, <pos>n.</pos> <def>A stone set along a margin as a limit and protection, as along the edge of a sidewalk next the roadway; an edge stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curbstone broker</b></col>. <cd>See under <er>Broker</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curch</ent><br/
<hw>Curch</hw> <pr>(k??rch)</pr>, <pos>n.</pos> <def>See <er>Courche</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curculio</ent><br/
<hw>Cur*cu"li*o</hw> <pr>(k?r-r?"l?-?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curculios</plw> <pr>(-<?/z)</pr>.</plu> <ety>[L., a grain weevil.]</ety> <fld>(Zool.)</fld> <def>One of a large group of beetles (<ord>Rhynchophora</ord>) of many genera; -- called also <altname>weevils</altname>, <altname>snout beetles</altname>, <altname>billbeetles</altname>, and <altname>billbugs</altname>. Many of the species are very destructive, as the plum curculio, the corn, grain, and rice weevils, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curculionidous</ent><br/
<hw>Cur`cu*li*on"i*dous</hw> <pr>(k?r`-k?-l?-?n"?-d?s)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to the <fam>Curculionide<ae/</fam>, or weevil tribe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curcuma</ent><br/
||<hw>Cur"cu*ma</hw> <pr>(k?r"k?-m?)</pr>, <pos>n.</pos> <ety>[Cf. F., It., & Sp. <ets>curcuma</ets>; all fr. Ar. <ets>kurkum</ets>. Cf. <er>Turmeric</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of plants of the order <ord>Scitamine<ae/</ord>, including the turmeric plant (<spn>Curcuma longa</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curcuma paper</b></col>. <fld>(Chem.)</fld> <cd>See <cref>Turmeric paper</cref>, under <er>Turmeric</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curcumin</ent><br/
<hw>Cur"cu*min</hw> <pr>(-m?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The coloring principle of turmeric, or curcuma root, extracted as an orange yellow crystalline substance, <chform>C14H14O4</chform>, with a green fluorescence.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It possesses acid properties and with alkalies forms brownish salts. This change in color from yellow to brown is the characteristic reaction of tumeric paper. See <cref>Turmeric paper</cref>, under <er>Turmeric</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curd</ent><br/
<hw>Curd</hw> <pr>(k<ucir/rd)</pr>, <pos>n.</pos> <ety>[Of Celtic origin; cf. Gael. <ets>gruth</ets>, Ir, <ets>gruth</ets>, <ets>cruth</ets>, curd, <ets>cruthaim</ets> I milk.]</ety> <altsp>[Sometimes written <asp>crud</asp>.]</altsp> <sn>1.</sn> <def>The coagulated or thickened part of milk, as distinguished from the whey, or watery part. It is eaten as food, especially when made into cheese.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Curds</qex> and cream, the flower of country fare.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The coagulated part of any liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The edible flower head of certain brassicaceous plants, as the broccoli and cauliflower.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Broccoli should be cut while the <qex>curd</qex>, as the flowering mass is termed, is entire.</q> <rj><qau>R. Thompson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cauliflowers should be cut for use while the head, or <qex>curd</qex>, is still close and compact.</q> <rj><qau>F. Burr.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curd</ent><br/
<hw>Curd</hw> <pr>(k<ucir/rd)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Curding</conjf>.]</vmorph> <def>To cause to coagulate or thicken; to cause to congeal; to curdle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Does it <qex>curd</qex> thy blood<br/
To say I am thy mother?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curd</ent><br/
<hw>Curd</hw>, <pos>v. i.</pos> <def>To become coagulated or thickened; to separate into curds and whey</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curdiness</ent><br/
<hw>Curd"i*ness</hw> <pr>(-?-n?s)</pr>, <pos>n.</pos> <def>The state of being curdy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curdle</ent><br/
<hw>Cur"dle</hw> <pr>(k<ucir/r"d'l)</pr>, <pos>v. i.</pos> <ety>[From <er>Curd</er>.]</ety> <altsp>[Sometimes written <asp>crudle</asp> and <asp>cruddle</asp>.]</altsp> <sn>1.</sn> <def>To change into curd; to coagulate; <as>as, rennet causes milk to <ex>curdle</ex></as>.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To thicken; to congeal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then Mary could feel her heart's blood <qex>curdle</qex> cold.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curdle</ent><br/
<hw>Cur"dle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curdled</conjf> <pr>(-d'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curdling</conjf> <pr>(-dl<icr/ng)</pr>.]</vmorph> <sn>1.</sn> <def>To change into curd; to cause to coagulate.</def> <ldquo/To <xex>curdle</xex> whites of eggs<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To congeal or thicken.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My chill blood is <qex>curdled</qex> in my veins.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curdless</ent><br/
<hw>Curd"less</hw> <pr>(k<ucir/rd"l<ecr/s)</pr>, <pos>a.</pos> <def>Destitute of curd.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curdy</ent><br/
<hw>Curd"y</hw> <pr>(k<ucir/rd"<ycr/)</pr>, <pos>a.</pos> <def>Like curd; full of curd; coagulated.</def> <ldquo/A <xex>curdy</xex> mass.<rdquo/  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cure</ent><br/
<hw>Cure</hw> <pr>(k<umac/r)</pr>, <pos>n.</pos> <ety>[OF, <ets>cure</ets> care, F., also, cure, healing, cure of souls, L. <ets>cura</ets> care, medical attendance, cure; perh. akin to <ets>cavere</ets> to pay heed, E. <ets>cution</ets>. <ets>Cure</ets> is not related to <ets>care</ets>.]</ety> <sn>1.</sn> <def>Care, heed, or attention.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of study took he most <qex>cure</qex> and most heed.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Vicarages of great<qex>cure</qex>, but small value.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Spiritual charge; care of soul; the office of a parish priest or of a curate; hence, that which is committed to the charge of a parish priest or of a curate; a curacy; <as>as, to resign a <ex>cure</ex>; to obtain a <ex>cure</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The appropriator was the incumbent parson, and had the <qex>cure</qex> of the souls of the parishioners.</q> <rj><qau>Spelman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Medical or hygienic care; remedial treatment of disease; a method of medical treatment; <as>as, to use the water <ex>cure</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Act of healing or state of being healed; restoration to health from disease, or to soundness after injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Past hope! past<qex>cure</qex>! past help.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I do <qex>cures</qex> to-day and to-morrow.</q> <rj><qau>Luke xii. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Means of the removal of disease or evil; that which heals; a remedy; a restorative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cold, hunger, prisons, ills without a <qex>cure</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The proper <qex>cure</qex> of such prejudices.</q> <rj><qau>Bp. Hurd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cure</ent><br/
<hw>Cure</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cured</conjf> <pr>(k<umac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curing</conjf>.]</vmorph> <ety>[OF. <ets>curer</ets> to take care, to heal, F., only, to cleanse, L. <ets>curare</ets> to take care, to heal, fr.  <ets>cura</ets>. See <er>Cure</er>,.]</ety> <sn>1.</sn> <def>To heal; to restore to health, soundness, or sanity; to make well; -- said of a patient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The child was <qex>cured</qex> from that very hour.</q> <rj><qau>Matt. xvii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subdue or remove by remedial means; to remedy; to remove; to heal; -- said of a malady.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>cure</qex> this deadly grief.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then he called his twelve disciples together, and gave them power . . . to <qex>cure</qex> diseases.</q> <rj><qau>Luke ix. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To set free from (something injurious or blameworthy), as from a bad habit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I never knew any man <qex>cured</qex> of inattention.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To prepare for preservation or permanent keeping; to preserve, as by drying, salting, etc.; <as>as, to <ex>cure</ex> beef or fish; to <ex>cure</ex> hay</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cure</ent><br/
<hw>Cure</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To pay heed; to care; to give attention.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To restore health; to effect a cure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose smile and frown, like to Achilles' spear,<br/
Is able with the change to kill and <qex>cure</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To become healed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One desperate grief <qex>cures</qex> with another's languish.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cure</ent><br/
||<hw>Cu`r<eacute/"</hw> <pr>(k<usdot/`r<asl/")</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>curatus</ets>. See <er>Curate</er>.]</ety> <def>A curate; a pardon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cureall</ent><br/
<hw>Cure"*all`</hw> <pr>(k?r"?l`)</pr>, <pos>n.</pos> <def>A remedy for all diseases, or for all ills; a panacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cureless</ent><br/
<hw>Cure"less</hw>, <pos>a.</pos> <def>Incapable of cure; incurable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With patience undergo<br/
A <qex>cureless</qex> ill, since fate will have it so.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curer</ent><br/
<hw>Cur"er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who cures; a healer; a physician.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who prepares beef, fish, etc., for preservation by drying, salting, smoking, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curettage</ent><br/
<hw>cu`ret*tage"</hw> <pr>(k<umac/`r<ecr/*t<aum/zh" <it>or</it> k<umac/*r<ecr/t"<asl/j)</pr>, <pos>n.</pos> <def>surgery to remove tissue or growths from a bodily cavity (as the uterus) by scraping with a curette; the act of scraping with a curette.</def><br/
<syn><b>Syn. --</b> curettement.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curette</ent><br/
||<hw>Cu*rette"</hw> <pr>(k<usdot/*r<ecr/t")</pr>, <pos>n.</pos> <ety>[F., fr. <ets>curer</ets> to cleanse.]</ety> <fld>(Med.)</fld> <def>A scoop or ring with either a blunt or a cutting edge, for removing substances from the walls of a cavity, as from the eye, ear, or womb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curette</ent><br/
<hw>Cu*rette"</hw> <pr>(k<usdot/*r<ecr/t")</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curetted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curetting</conjf>.]</vmorph> <fld>(Med.)</fld> <def>To scrape with a curette.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>curettement</ent><br/
<hw>curettement</hw> <pos>n.</pos> <def>surgery to remove tissue or growths from a bodily cavity (as the uterus) by scraping with a curette.</def><br/
<syn><b>Syn. --</b> curettage.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curfew</ent><br/
<hw>Cur"few</hw> <pr>(k<ucir/r"f<umac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>courfew</ets>, <ets>curfu</ets>, fr. OF. <ets>cuevrefu</ets>, <ets>covrefeu</ets>, F. <ets>couvre-feu</ets>; <ets>covrir</ets> to cover + <ets>feu</ets> fire, fr. L. <ets>focus</ets> fireplace, hearth. See <er>Cover</er>, and <er>Focus</er>.]</ety> <sn>1.</sn> <def>The ringing of an evening bell, originally a signal to the inhabitants to cover fires, extinguish lights, and retire to rest, -- instituted by <person>William the Conqueror</person>; also, the bell itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He begins at <qex>curfew</qex>, and walks till the first cock.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The village <qex>curfew</qex>, as it tolled profound.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A utensil for covering the fire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For pans, pots, <qex>curfews</qex>, counters and the like.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curia</ent><br/
||<hw>Cu"ri*a</hw> <pr>(k?"r?-?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curle</plw> <pr>(-<?/)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <sd>(a)</sd> <def>One of the thirty parts into which the Roman people were divided by Romulus.</def> <sd>(b)</sd> <def>The place of assembly of one of these divisions.</def> <sd>(c)</sd> <def>The place where the meetings of the senate were held; the senate house.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Middle Ages)</fld> <def>The court of a sovereign or of a feudal lord; also; his residence or his household.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Any court of justice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The Roman See in its temporal aspects, including all the machinery of administration; -- called also <altname>curia Romana</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curial</ent><br/
<hw>Cu"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the papal curia; <as>as, the <ex>curial</ex> etiquette of the Vatican</as>.</def> -- <def2><pos>n.</pos>  <def>A member of a curia, esp. of that of Rome or the later Italian sovereignties.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Curialism</ent><br/
<hw>Cu"ri*a*lism</hw> <pr>(k<umac/"r<icr/*<adot/*l<icr/?z'm)</pr>, <pos>n.</pos> <def>The view or doctrine of the ultramontane party in the Latin Church.</def>  <rj><au>Gladstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curialist</ent><br/
<hw>Cu"ri*a*list</hw> <pr>(k?"r?-?-l?st)</pr>, <pos>n.</pos> <def>One who belongs to the ultramontane party in the Latin Church.</def>  <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curialistic</ent><br/
<hw>Cu`ri*a*lis"tic</hw> <pr>(-l?s"t?k)</pr>, <pos>a.</pos> <ety>[L.<ets>curialis</ets> belonging to the imperial court, fr. <ets>curia</ets>, LL., also, counselors and retinue of a king.]</ety> <sn>1.</sn> <def>Pertaining to a court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating or belonging to the ultramontane party in the Latin Church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curiality</ent><br/
<hw>Cu`ri*al"i*ty</hw> <pr>(-?l"?-t?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>curialitas</ets> courtesy, fr. <ets>curialis</ets>.]</ety> <def>The privileges, prerogatives, or retinue of a court.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curiet</ent><br/
<hw>Cu"ri*et</hw> <pr>(k?"r?-?t)</pr>, <pos>n.</pos> <def>A cuirass.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curing</ent><br/
<hw>Cur"ing</hw> <pr>(k?r"?ng)</pr>, <def><pos>p. a. & vb. n.</pos> of <er>Cure</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curing house</b></col>, <cd>a building in which anything is cured; especially, in the West Indies, a building in which sugar is drained and dried.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curio</ent><br/
<hw>Cu"ri*o</hw> <pr>(k?"r?-?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curios</plw> <pr>(-<?/z)</pr>.</plu> <ety>[Abbreviation of <ets>curiosity</ets>.]</ety> <def>Any curiosity{3} or article of virtu; any object esteemed for its unusual nature.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>The busy world, which does not hunt poets as collectors hunt for <xex>curios</xex>.</q> <rj><qau>F. Harrison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curiologic</ent><br/
<hw>Cu`ri*o*log"ic</hw> <pr>(-?-l?j"?k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kyriologiko`s</grk> speaking literally (applied to curiologic hieroglyphics); <grk>ky`rios</grk> authoritative, proper + <grk>lo`gos</grk> word, thought. Cf. <er>Cyriologic</er>.]</ety> <def>Pertaining to a rude kind of hieroglyphics, in which a thing is represented by its picture instead of by a symbol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curiosity</ent><br/
<hw>Cu`ri*os"i*ty</hw> <pr>(k<umac/`r<icr/*<ocr/s"<icr/*t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curiosities</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[OE. <ets>curiouste</ets>, <ets>curiosite</ets>, OF. <ets>curioset<eacute/</ets>, <ets>curiosit<eacute/</ets>, F.  <ets>curiosit<eacute/</ets>, fr. L. <ets>curiositas</ets>, fr. <ets>curiosus</ets>. See <er>Curious</er>, and cf. <er>Curio</er>.]</ety> <sn>1.</sn> <def>The state or quality or being curious; nicety; accuracy; exactness; elaboration.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When thou wast in thy gilt and thy perfume, they mocked thee for too much <qex>curiosity</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A screen accurately cut in tapiary work . . . with great <qex>curiosity</qex>.</q> <rj><qau>Evelin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disposition to inquire, investigate, or seek after knowledge; a desire to gratify the mind with new information or objects of interest; inquisitiveness.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is curious, or fitted to excite or reward attention.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We took a ramble together to see the <qex>curiosities</qex> of this great town.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There hath been practiced also a <qex>curiosity</qex>, to set a tree upon the north side of a wall, and, at a little hieght, to draw it through the wall, etc.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curioso</ent><br/
||<hw>Cu`ri*o"so</hw> <pr>(k??`r?-?"z? <or/ k?`r?-?"s?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curiosos</plw> <pr>(-z<?/z <xex>or</xex> -s<?/z)</pr>.</plu> <ety>[It. See <er>Curious</er>.]</ety> <def>A virtuoso.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curious</ent><br/
<hw>Cu"ri*ous</hw> <pr>(k?"r?-?s)</pr>, <pos>a.</pos> <ety>[OF. <ets>curios</ets>, <ets>curius</ets>, F. <ets>curieux</ets>, L. <ets>curiosus</ets> careful, inquisitive, fr. <ets>cura</ets> care. See <er>Cure</er>.]</ety> <sn>1.</sn> <def>Difficult to please or satisfy; solicitous to be correct; careful; scrupulous; nice; exact.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Little <qex>curious</qex> in her clothes.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How shall we,<br/
If he be <qex>curious</qex>, work upon his faith?</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exhibiting care or nicety; artfully constructed; elaborate; wrought with elegance or skill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To devise <qex>curious</qex> works.</q> <rj><qau>Ex. xxxv. 32</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His body couched in a <qex>curious</qex> bed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Careful or anxious to learn; eager for knowledge; given to research or inquiry; habitually inquisitive; prying; -- sometimes with <xex>after</xex> or <xex>of</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a pity a gentleman so very <qex>curious</qex> after things that were elegant and beautiful should not have been as <qex>curious</qex> as to their origin, their uses, and their natural history.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Exciting attention or inquiry; awakening surprise; inviting and rewarding inquisitiveness; not simple or plain; strange; rare.</def> <ldquo/A<xex>curious</xex> tale<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A multitude of <qex>curious</qex> analogies.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a quaint and <qex>curious</qex> volume of forgotten lore.</q> <rj><qau>E. A. Poe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Abstruse investigations in recondite branches of learning or sciense often bring to light <qex>curious</qex> results.</q> <rj><qau>C. J. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curious arts</b></col>, <cd>magic.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many . . . which used <qex>curious arts</qex> brought their books together, and burned them.</q> <rj><qau>Acts xix. 19.</qau></rj></cs></p>

<p><syn><b>Syn.</b> -- Inquisitive; prying. See <er>Inquisitive</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curiously</ent><br/
<hw>Cu"ri*ous*ly</hw>, <pos>adv.</pos> <def>In a curious manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curiousness</ent><br/
<hw>Cu"ri*ous*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Carefulness; painstaking.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>My father's care<br/
With <qex>curiousness</qex> and cost did train me up.</q> <rj><qau> Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being curious; exactness of workmanship; ingenuity of contrivance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Inquisitiveness; curiosity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curium</ent><br/
<hw>curium</hw> <pos>n.</pos> <ety>[after Pierre and <person>Marie <etsep>Curie</etsep></person>.]</ety> <def>a radioactive transuranic element of atomic number 96, having an atomic weight of 247 for its most stable isotope (half-life 1.6 x 10<exp>7</exp> years).  The chemical symbol is <abbr>Cm</abbr>.</def>  <note>It was first prepared in 1944 by helium-ion bombardment of plutonium in a cyclotron by Glenn Seaborg and colleagues in California.  It has never been detected in nature, though minute quantities may exist in uranium deposits as a consequence of the neutron flux there.</note><br/
<syn><b>Syn. --</b> Cm, atomic number 96.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curl</ent><br/
<hw>Curl</hw> <pr>(k<ucir/rl)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curled</conjf> <pr>(k<ucir/rld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curling</conjf>.]</vmorph> <ety>[Akin to D. <ets>krullen</ets>, Dan. <ets>kr<oum/lle</ets>, dial. Sw.  <ets>krulla</ets> to curl, crisp; possibly akin to E. <ets>crook</ets>. Cf. <er>Curl</er>, <pos>n.</pos>, <er>Cruller</er>.]</ety> <sn>1.</sn> <def>To twist or form into ringlets; to crisp, as the hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But <qex>curl</qex> their locks with bodkins and with braid.</q> <rj><qau>Cascoigne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To twist or make onto coils, as a serpent's body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of his tortuous train,<br/
<qex>Curled</qex> many a wanton wreath in sight of Eve.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To deck with, or as with, curls; to ornament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thicker than the snaky locks<br/
That <qex>curled</qex>Meg<ae/ra.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Curling</qex> with metaphors a plain intention.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To raise in waves or undulations; to ripple.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seas would be pools without the brushing air<br/
To <qex>curl</qex> the waves.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Hat Making)</fld> <def>To shape (the brim) into a curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curl</ent><br/
<hw>Curl</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To contract or bend into curls or ringlets, as hair; to grow in curls or spirals, as a vine; to be crinkled or contorted; to have a curly appearance; <as>as, leaves lie <ex>curled</ex> on the ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou seest it [hair] will not <qex>curl</qex> by nature.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move in curves, spirals, or undulations; to contract in curving outlines; to bend in a curved form; to make a curl or curls.</def> <ldquo/<xex>Cirling</xex> billows.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then round her slender waist he <qex>curled</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Curling</qex> smokes from village tops are seen.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Gayly <qex>curl</qex> the waves before each dashing prow.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He smiled a king of sickly smile, and <qex>curled</qex> up on the floor.</q> <rj><qau>Bret Harte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To play at the game called <xex>curling</xex>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 358 --></p>

<p><ent>Curl</ent><br/
<hw>Curl</hw> <pr>(k<ucir/rl)</pr>, <pos>n.</pos> <ety>[Akin to D. <ets>krul</ets>, Dan. <ets>kr<oum/lle</ets>. See <er>Curl</er>, <pos>v.</pos> ]</ety> <sn>1.</sn> <def>A ringlet, especially of hair; anything of a spiral or winding form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under a coronet, his flowing hair<br/
In <qex>curls</qex> on either cheek played.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An undulating or waving line or streak in any substance, as wood, glass, etc.; flexure; sinuosity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the glass of the prisms . . . be without those numberless waves or <qex>curls</qex> which usually arise from the sand holes.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A disease in potatoes, in which the leaves, at their first appearance, seem curled and shrunken.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blue curls</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Blue</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curled</ent><br/
<hw>Curled</hw> <pr>(k<ucir/rld)</pr>, <pos>a.</pos> <def>Having curls; curly; sinuous; wavy; <as>as, <ex>curled</ex> maple (maple having fibers which take a sinuous course)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curled hair</b></col> <fld>(Com.)</fld>, <cd>the hair of the manes and tails of horses, prepared for upholstery purposes.</cd>  <rj><au>McElrath.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curledness</ent><br/
<hw>Curl"ed*ness</hw>, <pos>n.</pos> <def>State of being curled; curliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curler</ent><br/
<hw>Curl"er</hw> <pr>(k<ucir/rl"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, curls.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A player at the game called <er>curling</er>.</def>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small cylindrical object sometimes having a clamping attachment, around which hair is wound so as to produce curls; <as>as, she slept all night with a head full of <ex>curlers</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>an electrical appliance with a handle and a metal rod-shaped tip which is heated and around which hair is wound, to produce  curls in the hair; -- called also <ecol><b>curling iron</b></ecol>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Curlew</ent><br/
<hw>Cur"lew</hw> <pr>(k<ucir/r"l<umac/)</pr>, <pos>n.</pos> <ety>[F. <ets>courlieu</ets>, <ets>corlieu</ets>, <ets>courlis</ets>; perh. of imitative origin, but cf. OF. <ets>corlieus</ets> courier; L. <ets>currere</ets> to run + <ets>levis</ets> light.]</ety> <fld>(Zool.)</fld> <def>A wading bird of the genus <gen>Numenius</gen>, remarkable for its long, slender, curved bill.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European curlew is <spn>Numenius arquatus</spn>. The long-billed (<spn>Numenius longirostris</spn>), the Hudsonian (<spn>Numenius Hudsonicus</spn>), and the Eskimo curlew (<spn>Numenius borealis</spn>, are American species.  The name is said to imitate the note of the European species.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curlew Jack</b></col> <fld>(Zool.)</fld> <cd>the whimbrel or lesser curlew.</cd> -- <col><b>Curlew sandpiper</b></col> <fld>(Zool.)</fld>, <cd>a sandpiper (<spn>Tringa ferruginea</spn> <it>or</it> <spn>Tringa subarquata</spn>), common in Europe, rare in America, resembling a curlew in having a long, curved bill. See <xex>Illustation</xex> in Appendix.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>curlicue</ent><br/
<hw>curl"i*cue</hw> <pr>(k?rl"?-k?)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>caracole</ets>.]</ety> <def>Some thing curled or spiral, as a flourish made with a pen on paper, or with skates on the ice; a trick; a frolicsome caper.</def> <altsp>[Sometimes  written <asp>curlycue</asp> or <asp>carlicue</asp>.]</altsp> <mark>[ Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curliness</ent><br/
<hw>Curl"i*ness</hw> <pr>(k?rl"?-n?s)</pr>, <pos>n.</pos> <def>State of being curly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curling</ent><br/
<hw>Curl"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or state of that which curls; <as>as, the <ex>curling</ex> of smoke when it rises; the <ex>curling</ex> of a ringlet</as>; also, the act or process of one who curls something, as hair, or the brim of hats.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A scottish game in which heavy weights of stone or iron are propelled by hand over the ice towards a mark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Curling</qex> . . . is an amusement of the winter, and played on the ice, by sliding from one mark to another great stones of 40 to 70 pounds weight, of a hemispherical form, with an iron or wooden handle at top. The object of the player is to lay his stone as near to the mark as possible, to guard that of his partner, which has been well laid before, or to strike off that of his antagonist.</q> <rj><qau>Pennant (Tour in Scotland. 1772).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Curling irons</b></col>, <col><b>Curling tong</b></col></mcol>, <cd>an instrument for curling the hair; -- commonly heated when used.  Called also <er>curler{4}</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curlingly</ent><br/
<hw>Curl"ing*ly</hw>, <pos>adv.</pos> <def>With a curl, or curls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curly</ent><br/
<hw>Curl"y</hw> <pr>(k?rl"?)</pr>, <pos>a.</pos> <def>Curling or tending to curl; having curls; full of ripples; crinkled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curlycue</ent><br/
<hw>curl"y*cue</hw> <pr>(k?rl"?-k?)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>caracole</ets>.]</ety> <def>Some thing curled or spiral, as a flourish made with a pen on paper, or with skates on the ice; a trick; a frolicsome caper.  Same as <er>curlicue</er>.</def> <altsp>[Sometimes written <asp>carlicue</asp>.]</altsp> <mark>[ Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cut a curlycue</b></col>, <cd>to make a flourish; to cut a caper.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>I gave a flourishing about the room and <qex>cut a curlycue</qex> with my right foot.</q> <rj><qau>McClintock.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>curly-heads</ent><br/
<hw>curly-heads</hw> <pos>n.</pos> <def>a shrubby clematis (<spn>Clematis ochreleuca</spn>) of eastern US having curly foliage.</def><br/
<syn><b>Syn. --</b> <spn>Clematis ochreleuca</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curmudgeon</ent><br/
<hw>Cur*mudg"eon</hw> <pr>(k?r-m?j"?n)</pr>, <pos>n.</pos> <ety>[OE. <ets>cornmudgin</ets>, where <ets>-mudgin</ets> is prob. from OF. <ets>muchier</ets>, <ets>mucier</ets>, F. <ets>musser</ets> to hide; of uncertain origin; cf. OE. <ets>muchares</ets> skulking thieves, E. <ets>miche</ets>, <ets>micher</ets>.]</ety> <def>An avaricious, grasping fellow; a miser; a niggard; a churl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A gray-headed <qex>curmudgeon</qex> of a negro.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curmudgeonly</ent><br/
<hw>Cur*mudg"eon*ly</hw>, <pos>a.</pos> <def>Like a curmudgeon; niggardly; churlish; <as>as, a <ex>curmudgeonly</ex> fellow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curmurring</ent><br/
<hw>Cur*mur"ring</hw> <pr>(k?r-m?r"r?ng)</pr>, <pos>n.</pos> <def>Murmuring; grumbling; -- sometimes applied to the rumbling produced by a slight attack of the gripes.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curr</ent><br/
<hw>Curr</hw> <pr>(k<ucir/r)</pr>, <pos>v. i.</pos> <ety>[Prob. imitative.]</ety> <def>To coo.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The owlets hoot, the owlets <qex>curr</qex>.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currant</ent><br/
<hw>Cur"rant</hw> <pr>(k?r"r<it>a</it>nt)</pr>, <pos>n.</pos> <ety>[F. <ets>corinthe</ets> (raisins de <ets>Corinthe</ets> raisins of Corinth) currant (in sense 1), from the city of <ets>Corinth</ets> in Greece, whence, probably, the small dried grape (1) was first imported, the <ets>Ribes</ets> fruit (2) receiving the name from its resemblance to that grape.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A small kind of seedless raisin, imported from the Levant, chiefly from Zante and Cephalonia; -- used in cookery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The acid fruit or berry of the <spn>Ribes rubrum</spn> or common red currant, or of its variety, the white currant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A shrub or bush of several species of the genus <gen>Ribes</gen> (a genus also including the gooseberry); esp., the <spn>Ribes rubrum</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black currant</b></col>,<cd>a shrub or bush (<spn>Ribes nigrum</spn> and <spn>Ribes floridum</spn>) and its black, strong-flavored, tonic fruit.</cd> -- <col><b>Cherry currant</b></col>, <cd>a variety of the red currant, having a strong, symmetrical bush and a very large berry.</cd> -- <col><b>Currant borer</b></col> <fld>(Zool.)</fld>, <cd>the larva of an insect that bores into the pith and kills currant bushes; specif., the larvae of a small clearwing moth (<spn><AE/geria tipuliformis</spn>) and a longicorn beetle (<spn>Psenocerus supernotatus</spn>).</cd> -- <col><b>Currant worm</b></col> <fld>(Zool.)</fld>, <cd>an insect larva which eats the leaves or fruit of the currant.  The most injurious are the currant sawfly (<spn>Nematus ventricosus</spn>), introduced from Europe, and the spanworm (<spn>Eufitchia ribearia</spn>).  The fruit worms are the larva of a fly (<spn>Epochra Canadensis</spn>), and a spanworm (<gen>Eupithecia</gen>).</cd> -- <mcol><col><b>Flowering currant</b></col>, <col><b>Missouri currant</b></col></mcol>, <cd>a species of <gen>Ribes</gen> (<spn>Ribes aureum</spn>), having showy yellow flowers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>currawong</ent><br/
<hw>cur"ra*wong</hw> <pr>(k<ucir/r"<adot/*w<ocr/ng)</pr>, <pos>n.</pos> <def>any of several bluish black fruit-eating birds of Australia of the genus <gen>Strepera</gen> having a bell-like call.</def><br/
<syn><b>Syn. --</b> bell magpie.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Currency</ent><br/
<hw>Cur"ren*cy</hw> <pr>(k?r"r?n-c?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Currencies</plw> <pr>(-s<?/z)</pr>.</plu> <ety>[Cf. LL. <ets>currentia</ets> a current, fr. L. <ets>currens</ets>, p. pr. of <ets>currere</ets> to run. See <er>Current</er>.]</ety> <sn>1.</sn> <def>A continued or uninterrupted course or flow like that of a stream; <as>as, the <ex>currency</ex> of time</as>.</def> <mark>[Obs.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state or quality of being current; general acceptance or reception; a passing from person to person, or from hand to hand; circulation; <as>as, a report has had a long or general <ex>currency</ex>; the <ex>currency</ex> of bank notes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is in circulation, or is given and taken as having or representing value; <as>as, the <ex>currency</ex> of a country; a specie <ex>currency</ex>; esp., government or bank notes circulating as a substitute for metallic money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fluency; readiness of utterance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Current value; general estimation; the rate at which anything is generally valued.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . takes greatness of kingdoms according to their bulk and <qex>currency</qex>, and not after intrinsic value.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The bare name of Englishman . . . too often gave a transient <qex>currency</qex> to the worthless and ungrateful.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Current</ent><br/
<hw>Cur"rent</hw> <pr>(k?r"r<it>e</it>nt)</pr>, <pos>a.</pos> <ety>[OE. <ets>currant</ets>, OF. <ets>curant</ets>, <ets>corant</ets>, p. pr. of <ets>curre</ets>, <ets>corre</ets>, F.  <ets>courre</ets>, <ets>courir</ets>, to run, from L. <ets>currere</ets>; perh. akin to E.  <ets>horse</ets>. Cf.  <er>Course</er>, <er>Concur</er>, <er>Courant</er>, <er>Coranto</er>.]</ety> <sn>1.</sn> <def>Running or moving rapidly.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Like the <qex>current</qex> fire, that renneth<br/
Upon a cord.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To chase a creature that was <qex>current</qex> then<br/
In these wild woods, the hart with golden horns.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Now passing, as time; <as>as, the <ex>current</ex> month</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Passing from person to person, or from hand to hand; circulating through the community; generally received; common; <as>as, a <ex>current</ex> coin; a <ex>current</ex> report; <ex>current</ex> history</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That there was <qex>current</qex> money in Abraham's time is past doubt.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your fire-new stamp of honor is scarce <qex>current</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>current</qex> value, which is less or more as men have occasion for him.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Commonly estimated or acknowledged.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fitted for general acceptance or circulation; authentic; passable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O Buckingham, now do I play the touch<br/
To try if thou be <qex>current</qex> gold indeed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Account current</b></col>. <cd>See under <er>Account</er>.</cd> -- <col><b>Current money</b></col>, <cd>lawful money.</cd>  <rj><au>Abbott.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Current</ent><br/
<hw>Cur"rent</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>courant</ets>. See <er>Current</er>, <pos>a.</pos> ]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A flowing or passing; onward motion. Hence: A body of fluid moving continuously in a certain direction; a stream; esp., the swiftest part of it; <as>as, a <ex>current</ex> of water or of air</as>; that which resembles a stream in motion; <as>as, a <ex>current</ex> of electricity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two such silver <qex>currents</qex>, when they join,<br/
Do glorify the banks that bound them in.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The surface of the ocean is furrowed by <qex>currents</qex>, whose direction . . . the navigator should know.</q> <rj><qau>Nichol.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>General course; ordinary procedure; progressive and connected movement; <as>as, the <ex>current</ex> of time, of events, of opinion, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Current meter</b></col>, <cd>an instrument for measuring the velocity, force, etc., of currents.</cd> -- <col><b>Current mill</b></col>, <cd>a mill driven by a current wheel.</cd> -- <col><b>Current wheel</b></col>, <cd>a wheel dipping into the water and driven by the current of a stream or by the ebb and flow of the tide.</cd></cs></p>

<p><syn><b>Syn.</b> -- Stream; course. See <er>Stream</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currently</ent><br/
<hw>Cur"rent*ly</hw>, <pos>adv.</pos> <def>In a current manner; generally; commonly; <as>as, it is <ex>currently</ex> believed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currentness</ent><br/
<hw>Cur"rent*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality of being current; currency; circulation; general reception.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Easiness of pronunciation; fluency.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When <qex>currentness</qex> [combineth] with staidness, how can the language . . . sound other than most full of sweetness?</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curricle</ent><br/
<hw>Cur"ri*cle</hw> <pr>(k?r"r?-k'l)</pr>, <pos>n.</pos> <ety>[L.<ets>curriculum</ets> a running, a race course, fr. <ets>currere</ets> to run. See <er>Current</er>, and cf. <er>Curriculum</er>.]</ety> <sn>1.</sn> <def>A small or short course.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon a <qex>curricle</qex> in this world depends a long course of the next.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A two-wheeled chaise drawn by two horses abreast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curriculae</ent><br/
<hw>cur*ric"u*lae</hw> <pr>(k<ucr/r*r<icr/k"<umac/*l<emac/)</pr>, <pos>n. pl.</pos> <ety>[Presumably a pseudo-Latinate coinage based on <er>curricula</er>. See <er>Curriculum</er>.]</ety> <def>Same as <er>curricula</er>, plural of <er>curriculum</er>.  This is used commonly in the same sense as <er>curricula</er>, and appears to have arisen due to an incorrect assumption that <er>curricula</er> is a Latin-derived singular word.</def> <mark>[Ungrammatical]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Curriculum</ent><br/
<hw>Cur*ric"u*lum</hw> <pr>(k?r-r?k"?-l?m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Curriculums</plw>  (-l<?/mz), L. <plw>Curricula</plw> <pr>(-l<adot/)</pr>.</plu> <ety>[L. See <er>Curricle</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A race course; a place for running.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A course; particularly, a specified fixed course of study, as in a university.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curriculum vitae</ent><br/
<hw>cur*ric"u*lum vi"tae</hw> <pr>(k<ucr/r*r<icr/k"<umac/*l<ucr/m v<imac/"t<emac/, k<ucr/r*r<icr/k"<umac/*l<ucr/m v<imac/"t<imac/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>curricula vitae</plw>  (-l<adot/).</plu> <ety>[L.  the <it>course</it> of one's life or career.  See <er>Curriculum</er>.]</ety> <def>A brief biographical summary of the main points of a person's life, especially one's education and training, the jobs one has held, and other notable activities one has participated in, as well as other notable points such as honors one has received.  It is prepared and used commonly by a person who is submitting an application for a job or position of responsibility.  It is also called a <altname>vita</altname> or <altname>vitae</altname>, and is abbreviated <abbr>CV</abbr>.</def> <br/
[<source>PJC</source>]</p>

<p><ent>Currie</ent><br/
<hw>Cur"rie</hw> <pr>(k?r"r?)</pr>, <pos>n. & v.</pos> <def>See 2d & 3d <er>Curry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curried</ent><br/
<hw>Cur"ried</hw> <pr>(-r<?/d)</pr>, <pos>p. a.</pos> <ety>[See <er>Curry</er>, <pos>v. t.</pos>, and <er>Curry</er>, <pos>n.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Dressed by currying; cleaned; prepared.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Prepared with curry; <as>as, <ex>curried</ex> rice, fowl, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currier</ent><br/
<hw>Cur"ri*er</hw> <pr>(k?"r?-?r)</pr>, <pos>n.</pos> <ety>[From 1st <er>Curry</er>.]</ety> <def>One who curries and dresses leather, after it is tanned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currish</ent><br/
<hw>Cur"rish</hw> <pr>(k?r"r?sh)</pr>, <pos>a.</pos> <ety>[From <er>Cur</er>.]</ety> <def>Having the qualities, or exhibiting the characteristics, of a cur; snarling; quarrelsome; snappish; churlish; hence, also malicious; malignant; brutal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>currish</qex> spirit<br/
Governed a wolf.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>currish</qex> plot, -- some trick.</q> <rj><qau>Lockhart.</qau></rj></p>

<p>-- <wordforms><wf>Cur"rish*ly</wf>, <pos>adv.</pos> -- <wf>Cur"rish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curry</ent><br/
<hw>Cur"ry</hw> <pr>(k?r"r?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curried</conjf> <pr>(-r?d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Currying</conjf>.]</vmorph> <ety>[OE. <ets>curraien</ets>, <ets>curreien</ets>, OF. <ets>cunreer</ets>, <ets>correier</ets>, to prepare, arrange, furnish, curry (a horse), F. <ets>corroyer</ets> to curry (leather) (cf. OF. <ets>conrei</ets>, <ets>conroi</ets>, order, arrangement, LL. <ets>conredium</ets>); <ets>cor-</ets> (L.<ets>com-</ets>) + <ets>roi</ets>, <ets>rei</ets>, arrangement, order; prob. of German origin, and akin to E. <ets>ready</ets>. See <er>Ready</er>, <er>Greith</er>, and cf. <er>Corody</er>, <er>Array</er>.]</ety> <sn>1.</sn> <def>To dress or prepare for use by a process of scraping, cleansing, beating, smoothing, and coloring; -- said of leather.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dress the hair or coat of (a horse, ox, or the like) with a currycomb and brush; to comb, as a horse, in order to make clean.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your short horse is soon <qex>curried</qex>.</q> <rj><qau>Beau. & FL.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To beat or bruise; to drub; -- said of persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have seen him <qex>curry</qex> a fellow's carcass handsomely.</q> <rj><qau>Beau. & FL.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To curry favor</b></col>, <cd>to seek to gain favor by flattery or attentions. See <er>Favor</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curry</ent><br/
<hw>Cur"ry</hw>, <pos>n.</pos> <ety>[Tamil <ets>kari</ets>.]</ety> <altsp>[Written also <asp>currie</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Cookery)</fld> <def>A kind of sauce much used in India, containing garlic, pepper, ginger, and other strong spices.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stew of fowl, fish, or game, cooked with curry.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curry powder</b></col> <fld>(Cookery)</fld>, <cd>a condiment used for making curry, formed of various materials, including strong spices, as pepper, ginger, garlic, coriander seed, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curry</ent><br/
<hw>Cur"ry</hw> <pr>(k?r"r?)</pr>, <pos>v. t.</pos> <def>To flavor or cook with curry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currycomb</ent><br/
<hw>Cur"ry*comb`</hw> <pr>(k?r"r?-k?m`)</pr>, <pos>n.</pos> <def>A kind of card or comb having rows of metallic teeth or serrated ridges, used in currying a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Currycomb</ent><br/
<hw>Cur"ry*comb`</hw>, <pos>v. t.</pos> <def>To comb with a currycomb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curse</ent><br/
<hw>Curse</hw> <pr>(k?rs)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cursed</conjf> <pr>(k?rst)</pr> or <conjf>Curst</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cursing</conjf>.]</vmorph> <ety>[AS. <ets>cursian</ets>, <ets>corsian</ets>, perh. of Scand. origin; cf. Dan. <ets>korse</ets> to make the sign of the cross, Sw. <ets>korsa</ets>, fr. Dan. & Sw. <ets>kors</ets> cross, Icel <ets>kross</ets>, all these Scand. words coming fr. OF.  <ets>crois</ets>, <ets>croiz</ets>, fr. L. <ets>crux</ets> cross. Cf. <er>Cross</er>.]</ety> <sn>1.</sn> <def>To call upon divine or supernatural power to send injury upon; to imprecate evil upon; to execrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt not . . . <qex>curse</qex> the ruler of thy people.</q> <rj><qau>Ex. xxii. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ere sunset I'll make thee <qex>curse</qex> the deed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring great evil upon; to be the cause of serious harm or unhappiness to; to furnish with that which will be a cause of deep trouble; to afflict or injure grievously; to harass or torment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On impious realms and barbarous kings impose<br/
Thy plagues, and <qex>curse</qex> 'em with such sons as those.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To curse by bell, book, and candle</b></col>. <cd>See under <er>Bell</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curse</ent><br/
<hw>Curse</hw>, <pos>v. i.</pos> <def>To utter imprecations or curses; to affirm or deny with imprecations; to swear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then began he to <qex>curse</qex> and to swear.</q> <rj><qau>Matt. xxi. 74.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His spirits hear me,<br/
And yet I need must <qex>curse</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curse</ent><br/
<hw>Curse</hw>, <pos>n.</pos> <ety>[AS. <ets>curs</ets>. See <er>Curse</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>An invocation of, or prayer for, harm or injury; malediction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lady, you know no rules of charity,<br/
Which renders good for bad, blessings for <qex>curses</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Evil pronounced or invoked upon another, solemnly, or in passion; subjection to, or sentence of, divine condemnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> The priest shall write these <qex>curses</qex> in a book.</q> <rj><qau>Num. v. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Curses</qex>, like chickens, come home to roost.</q> <rj><qau>Old Proverb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The cause of great harm, evil, or misfortune; that which brings evil or severe affliction; torment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The common <qex>curse</qex> of mankind, folly and ignorance.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All that I eat, or drink, or shall beget,<br/
Is propagated <qex>curse</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The curse of Scotland</b></col> <fld>(Card Playing)</fld>, <cd>the nine of diamonds.</cd> -- <col><b>Not worth a curse</b></col>. <cd>See under <er>Cress</er>.</cd></cs></p>

<p><syn><b>Syn.</b> -- Malediction; imprecation; execration. See <er>Malediction</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursed</ent><br/
<hw>Curs"ed</hw> <pr>(k?rs"?d)</pr>, <pos>a.</pos> <def>Deserving a curse; execrable; hateful; detestable; abominable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us fly this <qex>cursed</qex> place.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>cursed</qex> quarrel be no more renewed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursedly</ent><br/
<hw>Curs"ed*ly</hw>, <pos>adv.</pos> <def>In a cursed manner; miserably; in a manner to be detested; enormously.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursedness</ent><br/
<hw>Curs"ed*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of being under a curse or of being doomed to execration or to evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wickedness; sin; cursing.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Shrewishness.</def> <ldquo/My wife's <xex>cursedness</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curser</ent><br/
<hw>Curs"er</hw> <pr>(k?rs"?r)</pr>, <pos>n.</pos> <def>One who curses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curship</ent><br/
<hw>Cur"ship</hw> <pr>(k?r"sh?p)</pr>, <pos>n.</pos> <ety>[<ets>Cur</ets> +<ets>-ship</ets>.]</ety> <def>The state of being a cur; one who is currish.</def> <mark>[Jocose]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>How durst he, I say, oppose thy <qex>curship</qex>!</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursitating</ent><br/
<hw>Cur"si*ta`ting</hw> <pr>(k?r"s?-t?`t?ng)</pr>, <pos>a.</pos> <ety>[See <er>Cursitor</er>.]</ety> <def>Moving about slightly.</def> <mark>[R.]</mark>  <rj><au>H. Bushnell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursitor</ent><br/
<hw>Cur"si*tor</hw> <pr>(k?r"s?-t?r)</pr>, <pos>n.</pos> <ety>[LL. <ets>cursitor</ets>, equiv. to L. <ets>cursor</ets>, fr. <ets>cursare</ets> to run hither and thither, fr. <ets>currere</ets> to run. See <er>Current</er>, and cf. <er>Cursor</er>.]</ety> <sn>1.</sn> <def>A courier or runner.</def> <mark>[Obs.]</mark> <ldquo/<xex>Cursitors</xex> to and fro.<rdquo/  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eng.Law)</fld> <def>An officer in the Court of Chancery, whose business is to make out original writs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursive</ent><br/
<hw>Cur"sive</hw> <pr>(k?r"s?v)</pr>, <pos>a.</pos> <ety>[LL. <ets>cursivus</ets>: cf. F. <ets>cursif</ets> See <er>Cursitor</er>.]</ety> <def>Running; flowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cursive hand</b></col>,<cd>a running handwriting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursive</ent><br/
<hw>Cur"sive</hw>, <pos>n.</pos> <sn>1.</sn> <def>A character used in cursive writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A manuscript, especially of the New Testament, written in small, connected characters or in a running hand; -- opposed to <xex>uncial</xex>.</def>  <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursor</ent><br/
<hw>Cur"sor</hw> <pr>(k?r"s?r)</pr>, <pos>n.</pos> <ety>[L., a runner. See <er>Cursitor</er>.]</ety> <def>Any part of a mathematical instrument that moves or slides backward and forward upon another part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursorary</ent><br/
<hw>Cur"so*ra*ry</hw> <pr>(-s?-r?-r?)</pr>, <pos>a.</pos> <def>Cursory; hasty.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With a <qex>cursorary</qex> eye o'erglanced the articles.</q>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursores</ent><br/
||<hw>Cur*so"res</hw> <pr>(k?r-s?"rEz)</pr>, <pos>n. pl.</pos> <ety>[L. <ets>cursor</ets>, pl. <ets>cursores</ets>, a runner.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>An order of running birds including the ostrich, emu, and allies; the Ratita<ae/.</def> <sd>(b)</sd> <def>A group of running spiders; the wolf spiders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursorial</ent><br/
<hw>Cur*so"ri*al</hw> <pr>(k?r-s?"r?-<it>a</it>l)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Adapted to running or walking, and not to prehension; <as>as, the limbs of the horse are <ex>cursorial</ex></as>. See <xex>Illust.</xex> of <er>Aves</er>.</def> <sd>(b)</sd> <def>Of or pertaining to the <ord>Cursores</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursorily</ent><br/
<hw>Cur"so*ri*ly</hw> <pr>(k?r"s?-r?-l?)</pr>, <pos>adv.</pos> <def>In a running or hasty manner; carelessly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursoriness</ent><br/
<hw>Cur"so*ri*ness</hw>, <pos>n.</pos> <def>The quality of being cursory; superficial performance; <as>as, <ex>cursoriness</ex> of view</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cursory</ent><br/
<hw>Cur"so*ry</hw> <pr>(k?r"s?-r?)</pr>, <pos>a.</pos> <ety>[L. <ets>cursorius</ets>, fr. <ets>cursor</ets>. See <er>Cursor</er>.]</ety> <sn>1.</sn> <def>Running about; not stationary.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characterized by haste; hastily or superficially performed; slight; superficial; careless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Events far too important to be treated in a <qex>cursory</qex> manner.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curst</ent><br/
<hw>Curst</hw> <pr>(k?rst)</pr>, <def><pos>imp. & p. p.</pos> of <er>Curse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curst</ent><br/
<hw>Curst</hw>, <pos>a.</pos> <ety>[See<er>Curse</er>.]</ety> <def>Froward; malignant; mischievous; malicious; snarling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though his mind<br/
 Be ne'er so <qex>curst</qex>, his tonque is kind.</q> <rj><qau>Crashaw.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curstfully</ent><br/
<hw>Curst"ful*ly</hw> <pr>(-f<usdot/l*l<ycr/)</pr>, <pos>adv.</pos> <def>Peevishly; vexatiously; detestably.</def> <mark>[Obs.]</mark> <ldquo/<xex>Curstfully</xex> mad.<rdquo/  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 359 --></p>

<p><ent>Curstness</ent><br/
<hw>Curst"ness</hw> <pr>(k<ucir/rst"n<ecr/s)</pr>, <pos>n.</pos> <def>Peevishness; malignity; frowardness; crabbedness; surliness.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curt</ent><br/
<hw>Curt</hw> <pr>(k<ucr/rt)</pr>, <pos>a.</pos> <ety>[L. <ets>curtus</ets>; cf. Skr. <ets>kart</ets> to cut. Cf. <er>Curtail</er>.]</ety> <def>Characterized by excessive brevity; short; rudely concise; <as>as, <ex>curt</ex> limits; a <ex>curt</ex> answer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>curt</qex>, yet comprehensive reply.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtail</ent><br/
<hw>Cur*tail"</hw> <pr>(k<ucr/r*t<amac/l")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curtailed</conjf> <pr>(-t<amac/ld")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curtailing</conjf>.]</vmorph> <ety>[See <er>Curtal</er>.]</ety> <def>To cut off the end or tail, or any part, of; to shorten; to abridge; to diminish; to reduce.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I, that am <qex>curtailed</qex> of this fair proportion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our incomes have been <qex>curtailed</qex>; his salary has been doubled.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtail</ent><br/
<hw>Cur"tail</hw> <pr>(k?r"t?l)</pr>, <pos>n.</pos> <def>The scroll termination of any architectural member, as of a step, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtail dog</ent><br/
<hw>Cur"tail dog`</hw> <pr>(d<ocr/g`; 115)</pr>. <def>A dog with a docked tail; formerly, the dog of a person not qualified to course, which, by the forest laws, must have its tail cut short, partly as a mark, and partly from a notion that the tail is necessary to a dog in running; hence, a dog not fit for sporting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hope is a <qex>curtail dog</qex> in some affairs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtailer</ent><br/
<hw>Cur*tail"er</hw> <pr>(k<ucr/r*t<amac/l"<etil/r)</pr>, <pos>n.</pos> <def>One who curtails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtailment</ent><br/
<hw>Cur*tail"ment</hw> <pr>(k?r-t?l"m<it>e</it>nt)</pr>, <pos>n.</pos> <def>The act or result of curtailing or cutting off.</def>  <rj><au>Bancroft.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtain</ent><br/
<hw>Cur"tain</hw> <pr>(k<ucir/r"t<icr/n; 48)</pr>, <pos>n.</pos> <ety>[OE.<ets>cortin</ets>, <ets>curtin</ets>,fr. OF. <ets>cortine</ets>, <ets>curtine</ets>, F. <ets>courtine</ets>, LL. <ets>cortina</ets>, curtian (in senses 1 and 2), also, small court, small inclosure surrounded by walls, from <ets>cortis</ets> court. See <er>Court</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A hanging screen intended to darken or conceal, and admitting of being drawn back or up, and reclosed at pleasure; esp., drapery of cloth or lace hanging round a bed or at a window; in theaters, and like places, a movable screen for concealing the stage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fort.)</fld> <def>That part of the rampart and parapet which is between two bastions or two gates. See <xex>Illustrations</xex> of <er>Ravelin</er> and <er>Bastion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>That part of a wall of a building which is between two pavilions, towers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A flag; an ensign; -- in contempt.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Behind the curtain</b></col>, <cd>in concealment; in secret.</cd> -- <col><b>Curtain lecture</b></col>, <cd>a querulous lecture given by a wife to her husband within the bed curtains, or in bed.</cd> <au>Jerrold.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>curtain lecture</qex> is worth all the sermons in the world for teaching the virtues of patience and long-suffering.</q> <rj><qau>W. Irving.</qau></rj></p>

<p>-- <col><b>The curtain falls</b></col>, <cd>the performance closes.</cd> -- <col><b>The curtain rises</b></col>, <cd>the performance begins.</cd> -- <col><b>To draw the curtain</b></col>, <cd>to close it over an object, or to remove it</cd>; hence: <sd>(a)</sd> <cd>To hide or to disclose an object.</cd> <sd>(b)</sd> <cd>To commence or close a performance.</cd> -- <col><b>To drop the curtain</b></col>, <cd>to end the tale, or close the performance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtain</ent><br/
<hw>Cur"tain</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curtained</conjf> <pr>(k<ucir/r"t<icr/nd; 48)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curtaining</conjf>.]</vmorph> <def>To inclose as with curtains; to furnish with curtains.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So when the sun in bed<br/
<qex>Curtained</qex> with cloudy red.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>curtained</ent><br/
<hw>curtained</hw> <pr>(k<ucir/r"t<icr/nd; 48)</pr>, <pos>adj.</pos> <def>furnished or concealed with curtains or draperies; <as>as, a <ex>curtained</ex> alcove</as>.  Opposite of <ant>curtainless</ant>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>curtainless</ent><br/
<hw>curtainless</hw> <pos>adj.</pos> <def>not provided with curtains; <as>as, blank, <ex>curtainless</ex> windows stared back at her</as>.  Opposite of <ant>curtained</ant>.</def><br/
<syn><b>Syn. --</b> uncurtained.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curtal</ent><br/
<hw>Cur"tal</hw> <pr>(k<ucir/r"t<ait/l)</pr>, <pos>a.</pos> <ety>[OF. <ets>courtault</ets>, F. <ets>courtaud</ets>, having a docked tail (cf. It. <ets>cortaldo</ets>), fr. <ets>court</ets> short, L. <ets>curtus</ets>. See <er>Curt</er>, and <er>Curtail</er>.]</ety> <def>Curt; brief; laconic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Essays and <qex>curtal</qex> aphorisms.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curtal dog</b></col>. <cd>See <er>Curtail dog</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtal</ent><br/
<hw>Cur"tal</hw>, <pos>n.</pos> <def>A horse with a docked tail; hence, anything cut short.</def> <mark>[Obs]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtelasse</ent><br/
<ent>Curtle ax</ent><br/
<ent>Curtal ax</ent><br/
<mhw>{ <hw>Cur"tal ax`</hw> <pr>(?ks`)</pr>, <hw>Cur"tle ax`</hw>, <hw>Curte"lasse</hw> <pr>(k?rt"l<it>as</it>)</pr> }</mhw>. <def>A corruption of <er>Cutlass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtal friar</ent><br/
<hw>Cur"tal fri`ar</hw> <pr>(fr?`?r)</pr>. <def>A friar who acted as porter at the gate of a monastery.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtana</ent><br/
<hw>Cur*ta"na</hw> <pr>(k?r-t?"n?)</pr>, <pos>n.</pos> <def>The pointless sword carried before English monarchs at their coronation, and emblematically considered as the sword of mercy; -- also called the <altname>sword of Edward the Confessor</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtate</ent><br/
<hw>Cur"tate</hw> <pr>(k?r"t?t)</pr>, <pos>a.</pos> <ety>[L. <ets>curtatus</ets>, p. p. of <ets>curtare</ets> to shorten, fr. <ets>curtus</ets>. See <er>Curt</er>.]</ety> <fld>(Astron.)</fld> <def>Shortened or reduced; -- said of the distance of a planet from the sun or earth, as measured in the plane of the ecliptic, or the distance from the sun or earth to that point where a perpendicular, let fall from the planet upon the plane of the ecliptic, meets the ecliptic.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curtate cycloid</b></col>. <fld>(Math.)</fld> <cd>See <er>Cycloid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtation</ent><br/
<hw>Cur*ta"tion</hw> <pr>(k?r-t?"sh?n)</pr>, <pos>n.</pos> <fld>(Astron.)</fld> <def>The interval by which the curtate distance of a planet is less than the true distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtein</ent><br/
<hw>Cur*tein"</hw> <pr>(k?r-t?n")</pr>, <pos>n.</pos> <def>Same as <er>Curtana</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtes</ent><br/
<hw>Cur*tes"</hw> <pr>(k?r-t?s")</pr>, <pos>a.</pos> <def>Courteous.</def> <mark>[Obs.]</mark> <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtesy</ent><br/
<hw>Cur"te*sy</hw> <pr>(k?r"t?-s?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Curtesies</plw> <pr>(-s<?/z)</pr>.</plu> <ety>[Either fr. <ets>courlesy</ets>, the lands being held as it were by favor; or fr. <ets>court</ets>  (LL. <ets>curtis</ets>), the husband being regarded as holding the lands as a vassal of the court. See <er>Court</er>, <er>Courtesy</er>.]</ety> <fld>(Law)</fld> <def>the life estate which a husband has in the lands of his deceased wife, which by the common law takes effect where he has had issue by her, born alive, and capable of inheriting the lands.</def>  <rj><au>Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtilage</ent><br/
<hw>Cur"ti*lage</hw> <pr>(k<?/r"t<?/-l<?/j)</pr>, <pos>n.</pos> <ety>[OF. <ets>cortillage</ets>, <ets>curtillage</ets>, fr. <ets>cortil</ets> court, courtyard, LL. <ets>cortis</ets> court. See <er>Court</er>.]</ety> <fld>(Law)</fld> <def>A yard, courtyard, or piece of ground, included within the fence surrounding a dwelling house.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtly</ent><br/
<hw>Curt"ly</hw> <pr>(k<ucir/rt"l<ycr/)</pr>, <pos>adv.</pos> <def>In a curt manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curtness</ent><br/
<hw>Curt"ness</hw>, <pos>n.</pos> <def>The quality of bing curt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curtsy</ent><br/
<ent>curtsey</ent><br/
<mhw><hw>curt"sey</hw>, <hw>curt"sy</hw></mhw> <pr>(k<ucir/rt"s<ycr/)</pr>, <pos>n.</pos> <def>An act of civility, respect, or reverence, made by women, consisting of a slight depression or dropping of the body, with bending of the knees.  Same as 2nd <er>Courtesy</er>, <pos>n.</pos>.</def><br/
<syn><b>Syn. --</b> curtsy; courtesy.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>curtsy</ent><br/
<ent>curtsey</ent><br/
<mhw><hw>curtsey</hw>, <hw>curtsy</hw></mhw> <pos>v. i.</pos> <def>to perform a <er>curtsy</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curule</ent><br/
<hw>Cu"rule</hw> <pr>(k<umac/"r<udd/l)</pr>, <pos>a.</pos> <ety>[L. <ets>curulis</ets>, fr. <ets>currus</ets> a chariot: cf. F. <ets>curule</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a chariot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rom. Antiq.)</fld> <def>Of or pertaining to a kind of chair appropriated to Roman magistrates and dignitaries; pertaining to, having, or conferring, the right to sit in the curule chair; hence, official.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>curule</xex> chair was usually shaped like a camp stool, and provided with curved legs. It was at first ornamented with ivory, and later sometimes made of ivory and inlaid with gold.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curule dignity</b></col> <cd>right of sitting in the curule chair.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cururo</ent><br/
||<hw>Cu*ru"ro</hw> <pr>(k??-r??"r?)</pr>, <pos>n.</pos> <ety>[Chilian name.]</ety> <fld>(Zool.)</fld> <def>A Chilian burrowing rodent of the genus <gen>Spalacopus</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curvaceous</ent><br/
<hw>curvaceous</hw> <pos>adj.</pos> <def>having a pronounced womanly shape; having a slender waist with prominent breasts and hips.</def> <mark>[chiefly dialect]</mark><br/
<syn><b>Syn. --</b> bosomy, buxom, full-bosomed, sonsie, sonsy, voluptuous.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Curvant</ent><br/
<ent>Curval</ent><br/
<mhw>{ <hw>Cur"val</hw> <pr>(k?r"v<it>a</it>l)</pr>, <hw>Cur"vant</hw> <pr>(-v<it>a</it>nt)</pr> }</mhw>, <pos>a.</pos> <ety>[L. <ets>curvans</ets>, <pos>p. pr.</pos> ]</ety> <fld>(Her.)</fld> <def>Bowed; bent; curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvated</ent><br/
<ent>Curvate</ent><br/
<mhw>{ <hw>Cur"vate</hw> <pr>(k?r"v?t)</pr>, <hw>Cur"va*ted</hw> <pr>(-v?-t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>curvatus</ets> p. p. of <ets>curvare</ets> to curve, fr.  <ets>curvus</ets>. See <er>Curve</er>.]</ety> <def>Bent in a regular form; curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvation</ent><br/
<hw>Cur*va"tion</hw> <pr>(k?r-v?"sh?n)</pr>, <pos>n.</pos> <ety>[L. <ets>curvatio</ets>.]</ety> <def>The act of bending or crooking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvative</ent><br/
<hw>Cur"va*tive</hw> <pr>(k?r"v?-t?v)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the margins only a little curved; -- said of leaves.</def>  <rj><au>Henslow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvature</ent><br/
<hw>Cur"va*ture</hw> <pr>(k?r"v?-t?r; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>curvatura</ets>. See <er>Curvate</er>.]</ety> <sn>1.</sn> <def>The act of curving, or the state of being bent or curved; a curving or bending, normal or abnormal, as of a line or surface from a rectilinear direction; a bend; a curve.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The elegant <qex>curvature</qex> of their fronds.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>The amount of degree of bending of a mathematical curve, or the tendency at any point to depart from a tangent drawn to the curve at that point.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Aberrancy of curvature</b></col> <fld>(Geom.)</fld>, <cd>the deviation of a curve from a circular form.</cd> -- <col><b>Absolute curvature</b></col>. <cd>See under <er>Absolute</er>.</cd> -- <col><b>Angle of curvature</b></col> <fld>(Geom.)</fld>, <cd>one that expresses the amount of curvature of a curve.</cd> -- <col><b>Chord of curvature</b></col>. <cd>See under <er>Chord</er>.</cd> -- <col><b>Circle of curvature</b></col>. <cd>See <cref>Osculating circle of a curve</cref>, under <er>Circle</er>.</cd> -- <col><b>Curvature of the spine</b></col> <fld>(Med.)</fld>, <cd>an abnormal curving of the spine, especially in a lateral direction.</cd> -- <col><b>Radius of curvature</b></col>, <cd>the radius of the circle of curvature, or osculatory circle, at any point of a curve.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curve</ent><br/
<hw>Curve</hw> <pr>(k<ucir/rv)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> bent, curved. See <er>Cirb</er>.]</ety> <def>Bent without angles; crooked; curved; <as>as, a <ex>curve</ex> line; a <ex>curve</ex> surface</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curve</ent><br/
<hw>Curve</hw>, <pos>n.</pos> <ety>[See <er>Curve</er>, <pos>a.</pos>, <er>Cirb</er>.]</ety> <sn>1.</sn> <def>A bending without angles; that which is bent; a flexure; <as>as, a <ex>curve</ex> in a railway or canal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>A line described according to some low, and having no finite portion of it a straight line.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Axis of a curve</b></col>. <cd>See under <er>Axis</er>.</cd> -- <col><b>Curve of quickest descent</b></col>. <cd>See <er>Brachystochrone</er>.</cd> -- <col><b>Curve tracing</b></col> <fld>(Math.)</fld>, <cd>the process of determining the shape, location, singular points, and other peculiarities of a curve from its equation.</cd> -- <col><b>Plane curve</b></col> <fld>(Geom.)</fld>, <cd>a curve such that when a plane passes through three points of the curve, it passes through all the other points of the curve. Any other curve is called a <contr>curve of double curvature</contr>, or a <contr>twisted curve</contr>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curve</ent><br/
<hw>Curve</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curved</conjf> <pr>(k<ucir/rvd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Curving</conjf>.]</vmorph> <ety>[L. <ets>curvare</ets>., fr. <ets>curvus</ets>. See <er>Curve</er>, <pos>a.</pos>, <er>Curb</er>.]</ety> <def>To bend; to crook; <as>as, to <ex>curve</ex> a line; to <xex>curve</xex> a pipe</as>; to cause to swerve from a straight course; <as>as, to <ex>curve</ex> a ball in pitching it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curve</ent><br/
<hw>Curve</hw>, <pos>v. i.</pos> <def>To bend or turn gradually from a given direction; <as>as, the road <ex>curves</ex> to the right</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>curved</ent><br/
<hw>curved</hw> <pos>adj.</pos> <sn>1.</sn>  <def>not straight; having or marked by curves.  Opposite of <ant>straight</ant>.</def> <note>[Narrower terms: <stype>arced, arched, arching, arciform, arcuate, bowed</stype>; <stype>falcate, sickle-shaped</stype>; <stype>flexuous</stype>; <stype>incurvate, incurved</stype>: <stype>recurved, recurvate</stype>; <stype>semicircular</stype>: <stype>serpentine, snaky</stype>: <stype>sinuate, sinuous, wavy</stype>: <stype>sinusoidal</stype>]</note><br/
<syn><b>Syn. --</b> curving.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(Botany)</fld> <def>curved with the micropyle near the base almost touching its stalk; -- of a plant ovule.  Opposite of <ant>orthotropous</ant>.</def><br/
<syn><b>Syn. --</b> campylotropous.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Curvedness</ent><br/
<hw>Curv"ed*ness</hw> <pr>(-?d-n?s)</pr>, <pos>n.</pos> <def>The state of being curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvet</ent><br/
<hw>Cur"vet</hw> <pr>(k<ucir/r"v?t <or/  k<ucir/r-v?t"; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>corvet</ets>, It.<ets>corvetta</ets>: cf. F. <ets>courbette</ets>. See <er>Curve</er>, and cf. <er>Corvetto</er>.]</ety> <sn>1.</sn> <fld>(Man.)</fld> <def>A particular leap of a horse, when he raises both his fore legs at once, equally advanced, and, as his fore legs are falling, raises his hind legs, so that all his legs are in the air at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A prank; a frolic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvet</ent><br/
<hw>Cur"vet</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Curveted</conjf> or <conjf>-vetted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Curveting</conjf> or <conjf>-vetting</conjf>.]</vmorph> <ety>[Cf. It. <ets>corvettare</ets>. See <er>Curvet</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To make a curvet; to leap; to bound.</def>  <ldquo/Oft and high he did <xex>curvet</xex>.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To leap and frisk; to frolic.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvet</ent><br/
<hw>Cur"vet</hw>, <pos>v. t.</pos> <def>To cause to curvet.</def>  <rj><au>Landor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvicaudate</ent><br/
<hw>Cur`vi*cau"date</hw> <pr>(k?r`v?-k?"d?t)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> bent + E. <ets>caudate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having a curved or crooked tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvicostate</ent><br/
<hw>Cur`vi*cos"tate</hw> <pr>(k?r`v?-k?s"t?t)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> + E. <ets>costate</ets>.]</ety> <fld>(Bot.)</fld> <def>Having bent ribs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvidentate</ent><br/
<hw>Cur`vi*den"tate</hw> <pr>(k?r`v?-d?n"t?t)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> + E. <ets>dentate</ets>.]</ety> <def>Having curved teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curviform</ent><br/
<hw>Cur"vi*form</hw> <pr>(k?r"v?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus + -form</ets>.]</ety> <def>Having a curved form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvilinead</ent><br/
<hw>Cur`vi*lin"e*ad</hw> <pr>(k?r`v?-l?n"?-?d)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>An instrument for drawing curved lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvilinear</ent><br/
<ent>Curvilineal</ent><br/
<mhw>{ <hw>Cur`vi*lin"e*al</hw> <pr>(-<it>a</it>l)</pr>, <hw>Cur`vi*lin"e*ar</hw> <pr>(-?r)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>curvus</ets> bent + E. <ets>lineal</ets>, <ets>linear</ets>.]</ety> <def>Consisting of, or bounded by, curved lines; <as>as, a <ex>curvilinear</ex> figure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvilinearity</ent><br/
<hw>Cur`vi*lin`e*ar"i*ty</hw> <pr>(-?r"?-t?)</pr>, <pos>n.</pos> <def>The state of being curvilinear or of being bounded by curved lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvilinearly</ent><br/
<hw>Cur`vi*lin"e*ar*ly</hw> <pr>(-?r-l?)</pr>, <pos>adv.</pos> <def>In a curvilinear manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvinerved</ent><br/
<hw>Cur"vi*nerved`</hw> <pr>(-n?rvd`)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> bent + E. <ets>nerve</ets>. ]</ety> <fld>(Bot.)</fld> <def>Having the ribs or the veins of the leaves curved; -- called also <altname>curvinervate</altname> and <altname>curve-veined</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvirostral</ent><br/
<hw>Cur`vi*ros"tral</hw> <pr>(-r?s"tr<it>a</it>l)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> + E. <ets>rostral</ets>.]</ety> <fld>(Zool.)</fld> <def>Having a crooked beak, as the crossbill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvirostres</ent><br/
||<hw>Cur"vi*ros"tres</hw> <pr>(-r?s"tr?z)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>curvus</ets> curved + <ets>rostrum</ets> beak, rostrum.]</ety> <fld>(Zool.)</fld> <def>A group of passerine birds, including the creepers and nuthatches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curviserial</ent><br/
<hw>Cur`vi*se"ri*al</hw> <pr>(-s?"r?-al)</pr>, <pos>a.</pos> <ety>[L. <ets>curvus</ets> bent + E. <ets>serial</ets>.]</ety> <fld>(Bot.)</fld> <def>Distributed in a curved line, as leaves along a stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvity</ent><br/
<hw>Cur"vi*ty</hw> <pr>(k?r"v?-y?)</pr>, <pos>n.</pos> <ety>[L. <ets>curvitas</ets>, from <ets>curvus</ets> bent: cf. F. <ets>curvit<eacute/</ets>.]</ety> <def>The state of being curved; a bending in a regular form; crookedness.</def>  <rj><au>Holder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Curvograph</ent><br/
<hw>Cur"vo*graph</hw> <pr>(k?r"v?-gr?f)</pr>, <pos>n.</pos> <ety>[L. <ets>curvus</ets> bent + <ets>-graph</ets>.]</ety> <fld>(Geom.)</fld> <def>An arcograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuscus</ent><br/
<hw>Cus"cus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[The same word as <er>Couscous</er>, fr. F. <ets>couscous</ets> couscous, Ar. <ets>kuskus</ets>.]</ety> <fld>(Bot.)</fld> <def>A soft grass (<spn>Pennisetum typhoideum</spn>) found in all tropical regions, used as food for men and cattle in Central Africa.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cuscus oil</ent><br/
<hw>Cuscus oil</hw>. <def>Same as <er>Vetiver oil</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cushat</ent><br/
<hw>Cush"at</hw> <pr>(k??sh"?t)</pr>, <pos>n.</pos> <ety>[AS. <ets>cusceote</ets>.]</ety> <fld>(Zool.)</fld> <def>The ringdove or wood pigeon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Scarce with <qex>cushat's</qex> homely song can vie.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cushewbird</ent><br/
<hw>Cush"ew*bird</hw> <pr>(k?sh"?-b?rd`)</pr>, <pos>n.</pos> <fld>(Zool)</fld> <def>The galeated curassow. See <er>Curassow</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cushion</ent><br/
<hw>Cush"ion</hw> <pr>(k??sh"?n)</pr>, <pos>n.</pos> <ety>[OE. <ets>cuischun</ets>, <ets>quisshen</ets>, OF. <ets>coissin</ets>, <ets>cuissin</ets>, F.  <ets>coussin</ets>, fr. (assumed) LL. <ets>culcitinum</ets>, dim. of L. <ets>culcita</ets> cushion, mattress, pillow. See <er>Quilt</er>, and cf. <er>Counterpoint</er> a <er>coverlet</er>.]</ety> <sn>1.</sn> <def>A case or bag stuffed with some soft and elastic material, and used to sit or recline upon; a soft pillow or pad.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two <qex>cushions</qex> stuffed with straw, the seat to raise.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything resembling a cushion in properties or use</def>; as: <sd>(a)</sd> <def>a pad on which gilders cut gold leaf</def>; <sd>(b)</sd> <def>a mass of steam in the end of the cylinder of a steam engine to receive the impact of the piston</def>; <sd>(c)</sd> <def>the elastic edge of a billiard table.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A riotous kind of dance, formerly common at weddings; -- called also <altname>cushion dance</altname>.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cushion capital</b></col>.<fld>(Arch.)</fld> <cd>A capital so sculptured as to appear like a cushion pressed down by the weight of its entablature.</cd> <sd>(b)</sd> <cd>A name given to a form of capital, much used in the Romanesque style, modeled like a bowl, the upper part of which is cut away on four sides, leaving vertical faces.</cd> -- <col><b>Cushion star</b></col> <fld>(Zool.)</fld> <cd>a pentagonal starfish belonging to <gen>Goniaster</gen>, <gen>Astrogonium</gen>, and other allied genera; -- so called from its form.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cushion</ent><br/
<hw>Cush"ion</hw> <pr>(k<oocr/sh"<ucr/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cushioned</conjf> <pr>(-<ucr/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cushioning</conjf>.]</vmorph> <sn>1.</sn> <def>To seat or place on, or as on a cushion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many who are <qex>cushioned</qex> on thrones would have remained in obscurity.</q> <rj><qau>Bolingbroke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish with cushions; <as>as, to <ex>cushion</ex> a chaise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To conceal or cover up, as under a cushion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cushioned hammer</b></col>, <cd>a dead-stroke hammer. See under <er>Dead-stroke</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cushioned</ent><br/
<hw>cushioned</hw> <pr>(k<oocr/sh"<ucr/nd)</pr>, <pos>adj.</pos> <sn>1.</sn> <def>furnished with a cushion or other device to reduce hardness.</def><br/
<syn><b>Syn. --</b> cushiony, padded.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>having the severity reduced; having the unpleasant effects mitigated.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn>  <def>protected against shock by adding soft padding or other device to reduce deceleration in a collision.</def><br/
<syn><b>Syn. --</b> cushioned.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Cushionet</ent><br/
<hw>Cush"ion*et</hw> <pr>(k<oocr/sh"<ucr/n*<ecr/t)</pr>, <pos>n.</pos> <ety>[OF. <ets>coissinet</ets>, F. <ets>coussinet</ets>. See <er>Cushion</er>, and cf. <er>Coussinet</er>.]</ety> <def>A little cushion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cushioning</ent><br/
<hw>cushioning</hw> <pos>n.</pos> <def>soft or resilient material used to fill or give shape or protect or add comfort.</def><br/
<syn><b>Syn. --</b> padding.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cushionless</ent><br/
<hw>Cush"ion*less</hw>, <pos>a.</pos> <def>Not furnished with a cushion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rows of long, <qex>cushionless</qex> benches, supplying the place of pews.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cushion tire</ent><br/
<hw>Cushion tire</hw>. <def>A thick solid-rubber tire, as for a bicycle, with a hollow groove running lengthwise on the inside.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cushiony</ent><br/
<hw>Cush"ion*y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <def>Like a cushion; soft; pliable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A flat and <qex>cushiony</qex> nose.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cushite</ent><br/
<hw>Cush"ite</hw> <pr>(k<ucr/sh"<imac/t)</pr>, <pos>n.</pos> <def>A descendant of <etsep>Cush</etsep>, the son of Ham and grandson of Noah.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cushy</ent><br/
<hw>cushy</hw> <pr>(k<oocr/sh"<ycr/)</pr>, <pos>adj.</pos> <def>not requiring strong efforts; easy; -- said of paid employment; <as>as, He got a <ex>cushy</ex> job in a large company</as>.</def> <mark>[informal]</mark><br/
<syn><b>Syn. --</b> soft.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cusk</ent><br/
<hw>Cusk</hw> <pr>(k<ucr/sk)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large, edible, marine fish (<spn>Brosmius brosme</spn>), allied to the cod, common on the northern coasts of Europe and America; -- called also <altname>tusk</altname> and <altname>torsk</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuskin</ent><br/
<hw>Cus"kin</hw> <pr>(k<ucr/s"k?n)</pr>, <pos>n.</pos> <def>A kind of drinking cup.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cusp</ent><br/
<hw>Cusp</hw> <pr>(k<ucr/sp)</pr>, <pos>n.</pos> <ety>[L. <ets>cuspis</ets>, <ets>-idis</ets>, point, pointed end.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Arch.)</fld> <def>A triangular protection from the intrados of an arch, or from an inner curve of tracery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astrol.)</fld> <def>The beginning or first entrance of any house in the calculations of nativities, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron)</fld> <def>The point or horn of the crescent moon or other crescent-shaped luminary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Math.)</fld> <def>A multiple point of a curve at which two or more branches of the curve have a common tangent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Anat.)</fld> <def>A prominence or point, especially on the crown of a tooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>A sharp and rigid point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cusp</ent><br/
<hw>Cusp</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cusped</conjf> <pr>(k<ucr/spt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cusping</conjf>.]</vmorph> <def>To furnish with a cusp or cusps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cusped</ent><br/
<ent>cuspate</ent><br/
<mhw><hw>cuspate</hw> <hw>cusped</hw></mhw> <pos>adj.</pos> <def>having cusps or points.</def><br/
<syn><b>Syn. --</b> cuspated, cuspidal, cuspidate, cuspidated.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cuspated</ent><br/
<hw>Cus"pa*ted</hw> <pr>(k?s"p?-t?d)</pr>, <pos>a.</pos> <def>Ending in a point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuspid</ent><br/
<hw>Cus"pid</hw> <pr>(k<ucr/s"p<icr/d)</pr>, <pos>n.</pos> <ety>[See <er>Cusp</er>.]</ety> <fld>(Anat.)</fld> <def>One of the canine teeth; -- so called from having but one point or cusp on the crown. See <er>Tooth</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuspidal</ent><br/
<hw>Cus"pi*dal</hw> <pr>(-p?-d<it>a</it>l)</pr>, <pos>a.</pos> <ety>[From L. <ets>cuspis</ets>, <ets>cuspidis</ets>. See <er>Cusp</er>.]</ety> <def>Ending in a point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuspidate</ent><br/
<hw>Cus"pi*date</hw> <pr>(-d?t)</pr>, <pos>v. t.</pos> <def>To make pointed or sharp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuspidated</ent><br/
<ent>Cuspidate</ent><br/
<mhw>{ <hw>Cus"pi*date</hw> <pr>(k?s"p?-d?t)</pr>, <hw>Cus"pi*da`ted</hw> <pr>(-d?`t?d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cuspidatus</ets>, p. p. of <ets>cuspidare</ets> to make pointed, fr. <ets>cuspis</ets>. See <er>Cusp</er>.]</ety> <def>Having a sharp end, like the point of a spear; terminating in a hard point; <as>as, a <ex>cuspidate</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cuspidation</ent><br/
<hw>cuspidation</hw> <pos>n.</pos> <def>a decoration using cusps.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cuspidor</ent><br/
<hw>Cus"pi*dor</hw> <pr>(-d?r)</pr>, <pos>n.</pos> <ety>[Pg. <ets>cuspideria</ets>, fr. <ets>cuspir</ets> to spit.]</ety> <def>Any ornamental vessel used as a spittoon; hence, to avoid the common term, a spittoon of any sort.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuspis</ent><br/
||<hw>Cus"pis</hw> <pr>(k?s"p?s)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A point; a sharp end.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cussed</ent><br/
<hw>cus"sed</hw> <pr>(c<ucr/s"<ecr/d)</pr>, <pos>adj.</pos> <def>stubbornly persistent in wrongdoing; obstinate.</def>  <mark>[Slang or Colloq., U. S.]</mark> <note>[Narrower terms: <stype>unregenerate (vs. regenerate), unregenerated</stype>]</note><br/
<syn><b>Syn. --</b> obdurate, obstinate, perverse, unrepentant.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cussedness</ent><br/
<hw>Cuss"ed*ness</hw> <pr>(c<ucr/s"<ecr/d*n<ecr/s)</pr>, <pos>n.</pos>  <ety>[<ets>Cussed</ets> (for <ets>cursed</ets>) + <ets>-ness</ets>.]</ety> <def>Disposition to willful wrongdoing; malignity; perversity; cantankerousness; obstinacy.</def> <mark>[Slang or Colloq., U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>In her opinion it was all pure <ldquo/<qex>cussedness</qex>.<rdquo/</q>  <rj><qau>Mrs. Humphry Ward.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>Disputatiousness and perversity (what the Americans call <ldquo/<qex>cussedness</qex><rdquo/).</q>  <rj><qau>James Bryce.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Custard</ent><br/
<hw>Cus"tard</hw> <pr>(k<ucr/s"t<etil/rd)</pr>, <pos>n.</pos> <ety>[Prob. the same word as OE. <ets>crustade</ets>, <ets>crustate</ets>, a pie made with a crust, fr. L. <ets>crustatus</ets> covered with a crust, p. p. of <ets>crustare</ets>, fr. <ets>crusta</ets> crust; cf. OF. <ets>croustade</ets> pasty, It. <ets>crostata</ets>, or F. <ets>coutarde</ets>. See <er>Crust</er>, and cf. <er>Crustated</er>.]</ety> <def>A mixture of milk and eggs, sweetened, and baked or boiled.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Custard apple</b></col> <fld>(Bot.)</fld>, <cd>a low tree or shrub of tropical America, including several species of <gen>Anona</gen> (<spn>Anona squamosa</spn>, <spn>Anona reticulata</spn>, etc.), having a roundish or ovate fruit the size of a small orange, containing a soft, yellowish, edible pulp.</cd> -- <col><b>Custard coffin</b></col>, <cd>pastry, or crust, which covers or <xex>coffins</xex> a custard</cd> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custode</ent><br/
<hw>Cus"tode</hw> <pr>(k?s"t?d)</pr>, <pos>n.</pos> <ety>[F. or It. <ets>custode</ets>, fr. L. <ets>custos</ets>, <ets>-odis</ets>.]</ety> <def>See <er>Custodian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custodial</ent><br/
<hw>Cus*to"di*al</hw> <pr>(k?s-t?"d?-<it>a</it>l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>custodial</ets>, fr. L. <ets>custodia</ets>. See <er>Custody</er>.]</ety> <def>Relating to custody or guardianship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custodian</ent><br/
<hw>Cus*to"di*an</hw> <pr>(k?s-t?"d?-<it>a</it>n)</pr>, <pos>n.</pos> <ety>[From <er>Custody</er>.]</ety> <def>One who has care or custody, as of some public building; a keeper or superintendent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custodianship</ent><br/
<hw>Cus*to"di*an*ship</hw>, <pos>n.</pos> <def>Office or duty of a custodian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custodier</ent><br/
<hw>Cus*to"di*er</hw> <pr>(-?r)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>custodiarus</ets>.]</ety> <def>A custodian.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custody</ent><br/
<hw>Cus"to*dy</hw> <pr>(k?s"t?-d?)</pr>, <pos>n.</pos> <ety>[L.  <ets>custodia</ets>, fr. <ets>custos</ets> guard; prob. akin to Gr. <?/<?/<?/<?/<?/<?/ to hide, and E. <ets>hide</ets>. Seee <er>Hide</er> to cover.]</ety> <sn>1.</sn> <def>A keeping or guarding; care, watch, inspection, for keeping, preservation, or security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fleet of thirty ships for the <qex>custody</qex> of the narrow seas.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Judicial or penal safe-keeping.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jailer, take him to thy <qex>custody</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 360 --></p>

<p><sn>3.</sn> <def>State of being guarded and watched to prevent escape; restraint of liberty; confinement; imprisonment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What pease will be given<br/
To us enslaved, but <qex>custody</qex> severe,<br/
And stripes and arbitrary punishment?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custom</ent><br/
<hw>Cus"tom</hw> <pr>(k<ucr/s"t<ucr/m)</pr>, <pos>n.</pos> <ety>[OF. <ets>custume</ets>, <ets>costume</ets>, Anglo-Norman <ets>coustome</ets>, F.  <ets>coutume</ets>, fr. (assumed) LL. <ets>consuetumen</ets> custom, habit, fr. L. <ets>consuetudo</ets>, <ets>-dinis</ets>, fr. <ets>consuescere</ets> to accustom, verb inchoative fr. <ets>consuere</ets> to be accustomed; <ets>con-</ets> + <ets>suere</ets> to be accustomed, prob. originally, to make one's own, fr. the root of <ets>suus</ets> one's own; akin to E.  <ets>so</ets>, adv. Cf. <er>Consuetude</er>, <er>Costume</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Frequent repetition of the same act; way of acting common to many; ordinary manner; habitual practice; usage; method of doing or living.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And teach <qex>customs</qex> which are not lawful.</q> <rj><qau>Acts xvi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Moved beyond his <qex>custom</qex>, Gama said.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>custom</qex><br/
More honored in the breach than the observance.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Habitual buying of goods; practice of frequenting, as a shop, manufactory, etc., for making purchases or giving orders; business support.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let him have your <qex>custom</qex>, but not your votes.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Long-established practice, considered as unwritten law, and resting for authority on long consent; usage. See <er>Usage</er>, and <er>Prescription</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Usage</xex> is a fact. <xex>Custom</xex> is a law. There can be no <xex>custom</xex> without <xex>usage</xex>, though there may be <xex>usage</xex> without <xex>custom</xex>.</note>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Familiar aquaintance; familiarity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Age can not wither her, nor <qex>custom</qex> stale<br/
Her infinite variety.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Custom of merchants</b></col>, <cd>a system or code of customs by which affairs of commerce are regulated.</cd> -- <col><b>General customs</b></col>, <cd>those which extend over a state or kingdom.</cd> -- <col><b>Particular customs</b></col>, <cd>those which are limited to a city or district; as, the <xex>customs</xex> of London.</cd></cs></p>

<p><syn><b>Syn.</b> -- Practice; fashion. See <er>Habit</er>, and <er>Usage</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custom</ent><br/
<hw>Cus"tom</hw>, <pos>v. t.</pos> <ety>[Cf. OF.  <ets>costumer</ets>. Cf. <er>Accustom</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To make familiar; to accustom.</def> <mark>[Obs.]</mark>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To supply with customers.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custom</ent><br/
<hw>Cus"tom</hw>, <pos>v. i.</pos> <def>To have a custom.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>On a bridge he <qex>custometh</qex> to fight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custom</ent><br/
<hw>Cus"tom</hw>, <pos>n.</pos> <ety>[OF.  <ets>coustume</ets>, F. <ets>coutume</ets>, tax, <it>i. e.</it>, the <ets>usual</ets> tax. See 1st <er>Custom</er>.]</ety> <sn>1.</sn> <def>The customary toll, tax, or tribute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Render, therefore, to all their dues: tribute to whom tribute is due; <qex>custom</qex> to whom <qex>custom</qex>.</q> <rj><qau>Rom. xiii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Duties or tolls imposed by law on commodities, imported or exported.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custom</ent><br/
<hw>Cus"tom</hw>, <pos>v. t.</pos> <def>To pay the customs of.</def> <mark>[Obs.]</mark>  <rj><au>Marlowe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customable</ent><br/
<hw>Cus"tom*a*ble</hw> <pr>(-<adot/*b'l)</pr>, <pos>a.</pos> <ety>[Cf. OF. <ets>coustumable</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Customary.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Subject to the payment of customs; dutiable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customableness</ent><br/
<hw>Cus"tom*a*ble*ness</hw>, <pos>n.</pos> <def>Quality of being customable; conformity to custom.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customably</ent><br/
<hw>Cus"tom*a*bly</hw>, <pos>adv.</pos> <def>Usually.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customarily</ent><br/
<hw>Cus"tom*a*ri*ly</hw> <pr>(-<asl/*r<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a customary manner; habitually.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customariness</ent><br/
<hw>Cus"tom*a*ri*ness</hw>, <pos>n.</pos> <def>Quality of being customary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customary</ent><br/
<hw>Cus"tom*a*ry</hw> <pr>(k<ucr/s"t<ucr/m*<asl/*r<ycr/)</pr>, <pos>a.</pos> <ety>[CF. OF. <ets>coustumier</ets>, F. <ets>coutumier</ets>. See <er>Custom</er>, and cf. <er>Customer</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Agreeing with, or established by, custom; established by common usage; conventional; habitual.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even now I met him<br/
With <qex>customary</qex> compliment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A formal <qex>customary</qex> attendance upon the offices.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Holding or held by custom; <as>as, <ex>customary</ex> tenants; <ex>customary</ex> service or estate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customary</ent><br/
<hw>Cus"tom*a*ry</hw>, <pos>n.</pos> <ety>[OF. <ets>coustumier</ets>, F.  <ets>coutumier</ets>.]</ety> <def>A book containing laws and usages, or customs; <as>as, the <ex>Customary</ex> of the Normans</as>.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customer</ent><br/
<hw>Cus"tom*er</hw> <pr>(k<ucr/s"t<ucr/m*<etil/r)</pr>, <pos>n.</pos> <ety>[A doublet of <ets>customary</ets>, <ets>a.</ets>: cf. LL. <ets>custumarius</ets> toll gatherer. See <er>Custom</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>One who collect customs; a toll gatherer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>customers</qex> of the small or petty custom and of the subsidy do demand of them custom for kersey cloths.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who regularly or repeatedly makes purchases of a trader; a purchaser; a buyer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He has got at last the character of a good <qex>customer</qex>; by this means he gets credit for something considerable, and then never pays for it.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person with whom a business house has dealings; <as>as, the <ex>customers</ex> of a bank</as>.</def>  <rj><au>J. A. H. Murray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A peculiar person; -- in an indefinite sense; <as>as, a queer <ex>customer</ex>; an ugly <ex>customer</ex></as>.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A lewd woman.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Customhouse</ent><br/
<hw>Cus"tom*house"</hw> <pr>(-hous`)</pr>, <pos>n.</pos> <def>The building where customs and duties are paid, and where vessels are entered or cleared.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Customhouse broker</b></col>, <cd>an agent who acts for merchants in the business of entering and clearing goods and vessels.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>customize</ent><br/
<hw>customize</hw> <pos>v.</pos> <sn>1.</sn> <def>to make to specifications.</def><br/
<syn><b>Syn. --</b> custom-make, taylor-make.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>to make according to requirements.</def><br/
<syn><b>Syn. --</b> customise.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>custom-made</ent><br/
<hw>custom-made</hw> <pos>adj.</pos> <sn>1.</sn>  <def>made specially for a specific purpose; -- of articles of manufacture.  Contrasted with <er>mass-produced</er>, <er>standardized</er>.</def><br/
<syn><b>Syn. --</b> made-to-order.</syn><br/
[<source>PJC</source> <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>made or tailored to fit a specific person; -- of clothing.  Contrasted with <er>off-the-rack</er>, <er>ready-to-wear</er>.</def> <illu>a custom-made suit</illu><br/
<syn><b>Syn. --</b> made-to-order.</syn><br/
[<source>PJC</source>]</p>

<p><ent>customs</ent><br/
<hw>customs</hw> <pos>n. pl.</pos> <sn>1.</sn> <def>money collected under a tariff; a duty imposed on imported goods.</def><br/
<syn><b>Syn. --</b> customs duty, custom, impost.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>the government department administering the collection of import tariffs.  Used with as singular verb.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>that area within an airport, sea port, or other border station where freight or the baggage of travellers is checked for dutiable materials or contraband; <as>as, it took an hour to get through <ex>customs</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Custos</ent><br/
||<hw>Cus"tos</hw> <pr>(k<ucr/s"t<ocr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Custodes</plw> <pr>(k<ucr/s*t<omac/"d<emac/z)</pr>.</plu> <ety>[L.]</ety> <def>A keeper; a custodian; a superintendent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Custos rotulorum</b></col> <pr>(r<ocr/t`<usl/*l<omac/"r<ucr/m)</pr> <ety>[LL., keeper of the rolls]</ety> <fld>(Eng. Law)</fld>, <cd>the principal justice of the peace in a county, who is also keeper of the rolls and records of the sessions of the peace.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custrel</ent><br/
<hw>Cus"trel</hw> <pr>(k<ucr/s"tr<eit/l)</pr>, <pos>n.</pos> <ety>[OF. <ets>coustillier</ets>. See <er>Coistril</er>.]</ety> <def>An armor-bearer to a knight.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custrel</ent><br/
<hw>Cus"trel</hw>, <pos>n.</pos> <def>See <er>Costrel</er>.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Custumary</ent><br/
<hw>Cus"tu*ma*ry</hw> <pr>(-t<usl/*m<asl/*r<ycr/)</pr>, <pos>a.</pos> <def>See <er>Customary</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cut</ent><br/
<hw>Cut</hw> <pr>(k<ucr/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cut</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Cutting</conjf>.]</vmorph> <ety>[OE. <ets>cutten</ets>, <ets>kitten</ets>, <ets>ketten</ets>; prob. of Celtic origin; cf. W. <ets>cwtau</ets> to shorten, curtail, dock, <ets>cwta</ets> bobtailed, <ets>cwt</ets> tail, skirt, Gael.  <ets>cutaich</ets> to shorten, curtail, dock, <ets>cutach</ets> short, docked, <ets>cut</ets> a bobtail, piece, Ir. <ets>cut</ets> a short tail, <ets>cutach</ets> bobtailed. Cf. <er>Coot</er>.]</ety> <sn>1.</sn> <def>To separate the parts of with, or as with, a sharp instrument; to make an incision in; to gash; to sever; to divide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You must <qex>cut</qex> this flesh from off his breast.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Before the whistling winds the vessels fly,<br/
With rapid swiftness <qex>cut</qex> the liquid way.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sever and cause to fall for the purpose of gathering; to hew; to mow or reap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy servants can skill to <qex>cut</qex> timer.</q> <rj><qau>2. Chron. ii. 8</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sever and remove by cutting; to cut off; to dock; <as>as, to <ex>cut</ex> the hair; to <ex>cut</ex> the nails</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To castrate or geld; <as>as, to <ex>cut</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To form or shape by cutting; to make by incision, hewing, etc.; to carve; to hew out.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why should a man. whose blood is warm within,<br/
Sit like his grandsire <qex>cut</qex> in alabaster?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Loopholes <qex>cut</qex> through thickest shade.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To wound or hurt deeply the sensibilities of; to pierce; to lacerate; <as>as, sarcasm <ex>cuts</ex> to the quick</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The man was <qex>cut</qex> to the heart.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To intersect; to cross; <as>as, one line <ex>cuts</ex> another at right angles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To refuse to recognize; to ignore; <as>as, to <ex>cut</ex> a person in the street; to <ex>cut</ex> one's acquaintance</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To absent one's self from; <as>as, to <ex>cut</ex> an appointment, a recitation</as>. etc.</def><-- to cut class --> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>An English tradesman is always solicitous to <qex>cut</qex> the shop whenever he can do so with impunity.</q> <rj><qau>Thomas Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Cricket)</fld> <def>To deflect (a bowled ball) to the off, with a chopping movement of the bat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>11.</sn> <fld>(Billiards, etc.)</fld> <def>To drive (an object ball) to either side by hitting it fine on the other side with the cue ball or another object ball.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>12.</sn> <fld>(Lawn Tennis, etc.)</fld> <def>To strike (a ball) with the  racket inclined or struck across the ball so as to put a certain spin on the ball.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>13.</sn>  <fld>(Croquet)</fld> <def>To drive (a ball) to one side by hitting with another ball.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>To cut a caper</b></col>. <cd>See under <er>Caper</er>.</cd> -- <col><b>To cut the cards</b></col>, <cd>to divide a pack of cards into portions, in order to determine the deal or the trump, or to change the cards to be dealt.</cd> -- <col><b>To cut both ways</b></col>, <cd>to have effects both advantageous and disadvantageous.</cd> -- <col><b>To cut corners</b></col>, <cd>to deliberately do an incomplete or imperfect job in order to save time or money.</cd> -- <mcol><col><b>To cut a dash</b></col> <it>or</it> <col><b>To cut a figure</b></col></mcol>, <cd>to make a display of oneself; to give a conspicuous impression.</cd> <mark>[Colloq.]</mark> -- <col><b>To cut down</b></col>. <sd>(a)</sd> <cd>To sever and cause to fall; to fell; to prostrate.</cd> <ldquo/Timber . . . <xex>cut down</xex> in the mountains of Cilicia.<rdquo/ <au>Knolles.</au> <sd>(b)</sd> <cd>To put down; to abash; to humble.</cd> <mark>[Obs]</mark> <ldquo/So great is his natural eloquence, that he <xex>cuts down</xex> the finest orator.<rdquo/ <au>Addison</au> <sd>(c)</sd> <cd>To lessen; to retrench; to curtail; <as>as, <ex>to cut down</ex> expenses</as>.</cd> <sd>(d)</sd> <fld>(Naut.)</fld> <cd>To raze; as, <xex>to cut down</xex> a frigate into a sloop.</cd> -- <mcol><col><b>To cut the knot</b></col> <it>or</it> <col><b>To cut the Gordian knot</b></col></mcol>, <cd>to dispose of a difficulty summarily; to solve it by prompt, arbitrary action, rather than by skill or patience.</cd> -- <col><b>To cut lots</b></col>, <cd>to determine lots by cuttings cards; to draw lots.</cd> -- <col><b>To cut off</b></col>. <sd>(a)</sd> <cd>To sever; to separate.</cd><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>I would to God, . . . <br/
The king had <qex>cut off</qex> my brother's.</q> <rj><qau>Shak.</qau></rj></p>

<p><sd>(b)</sd> <cd>To put an untimely death; to put an end to; to destroy.</cd> <ldquo/Iren<ae/us was likewise <xex>cut off</xex> by martyrdom.<rdquo/ <au>Addison.</au> <sd>(c)</sd> <cd>To interrupt; as, <xex>to cut off</xex> communication; <xex>to cut off</xex> (the flow of) steam from (the boiler to) a steam engine.</cd> <sd>(d)</sd> <cd>To intercept; as,, <xex>to cut off</xex> an enemy's retreat.</cd> <sd>(e)</sd> <cd>To end; to finish; as, <xex>to cut off</xex> further debate.</cd> -- <col><b>To cut out</b></col>. <sd>(a)</sd> <cd>To remove by cutting or carving; as, <xex>to cut out</xex> a piece from a board.</cd> <sd>(b)</sd> <cd>To shape or form by cutting; as, <xex>to cut out</xex> a garment.</cd> <ldquo/ A large forest <xex>cut out</xex> into walks.<rdquo/ <au>Addison.</au> <sd>(c)</sd> <cd>To scheme; to contrive; to prepare; as, <xex>to cut out</xex> work for another day.</cd> <ldquo/Every man had <xex>cut out</xex> a place <xex>for himself</xex>.<rdquo/ <au>Addison.</au> <sd>(d)</sd> <cd>To step in and take the place of; to supplant; as, <xex>to cut out</xex> a rival.</cd> <mark>[Colloq.]</mark> <sd>(e)</sd> <cd>To debar.</cd> <ldquo/I am <xex>cut out</xex> from anything but common acknowledgments.<rdquo/ <au>Pope.</au> <sd>(f)</sd> <cd>To seize and carry off (a vessel) from a harbor, or from under the guns of an enemy.</cd> <sd>(g)</sd> <cd>to separate from the midst of a number; <as>as, <ex>to cut out</ex> a steer from a herd; <ex>to cut out</ex> a car from a train</as>.</cd> <sd>(h)</sd> <cd>to discontinue; <as>as, to <ex>cut out</ex> smoking</as>.</cd> -- <col><b>To cut to pieces</b></col>. <sd>(a)</sd> <cd>To cut into pieces; <as>as, <ex>to cut</ex> cloth <ex>to pieces</ex></as>.</cd> <sd>(b)</sd> <cd>To slaughter; as, <xex>to cut</xex> an army <xex>to pieces</xex>.</cd> -- <col><b>To cut a play</b></col> <fld>(Drama)</fld>, <cd>to shorten it by leaving out passages, to adapt it for the stage.</cd> -- <col><b>To cut rates</b></col> <fld>(Railroads, etc.)</fld>, <cd>to reduce the charges for transportation below the rates established between competing lines.</cd> -- <col><b>To cut short</b></col>, <cd>to arrest or check abruptly; to bring to a sudden termination.</cd> <ldquo/Achilles <xex>cut</xex> him <xex>short</xex>, and thus replied.<rdquo/ <au>Dryden.</au> -- <col><b>To cut stick</b></col>, <cd>to make off clandestinely or precipitately.</cd> <mark>[Slang]</mark> -- <col><b>To cut teeth</b></col>, <cd>to put forth teeth; to have the teeth pierce through the gum and appear.</cd> -- <col><b>To have cut one's eyeteeth</b></col>, <cd>to be sharp and knowing.</cd> <mark>[Colloq.]</mark> -- <col><b>To cut one's wisdom teeth</b></col>, <cd>to come to years of discretion.</cd> -- <col><b>To cut under</b></col>, <cd>to undersell; <as>as, <ex>to cut under</ex> a competitor in trade</as>; more commonly referred to as <altname>undercut</altname>.</cd> -- <col><b>To cut up</b></col>. <sd>(a)</sd> <cd>To cut to pieces; <as>as, <ex>to cut up</ex> an animal, or bushes</as>.</cd> <sd>(b)</sd> <cd>To damage or destroy; to injure; to wound; <as>as, <ex>to cut up</ex> a book or its author by severe criticism</as>.</cd>  <ldquo/This doctrine <ex>cuts up</ex> all government by the roots.<rdquo/ <au>Locke.</au> <sd>(c)</sd> <cd>To afflict; to discourage; to demoralize; <as>as, the death of his friend <ex>cut</ex> him <ex>up</ex> terribly</as>.</cd> <mark>[Colloq.]</mark> <au>Thackeray.</au></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Cut</ent><br/
<hw>Cut</hw> <pr>(k<ucr/t)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To do the work of an edged tool; to serve in dividing or gashing; <as>as, a knife <ex>cuts</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To admit of incision or severance; to yield to a cutting instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Panels of white wood that <qex>cuts</qex> like cheese.</q> <rj><qau>Holmes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To perform the operation of dividing, severing, incising, intersecting, etc.; to use a cutting instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He saved the lives of thousands by his manner of <qex>cutting</qex> for the stone.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make a stroke with a whip.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To interfere, as a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To move or make off quickly.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To divide a pack of cards into two portion to decide the deal or trump, or to change the order of the cards to be dealt.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cut across</b></col>, <cd>to pass over or through in the most direct way; <as>as, to <ex>cut across</ex> a field</as>.</cd> -- <col><b>To cut and run</b></col>, <cd>to make off suddenly and quickly; -- from the cutting of a ship's cable, when there is not time to raise the anchor.</cd> <mark>[Colloq.]</mark> -- <mcol><col><b>To cut in</b></col> <it>or</it> <col><b>To cut into</b></col></mcol>, <cd>to interrupt; to join in anything suddenly.</cd> -- <col><b>To cut up</b></col>. <sd>(a)</sd> <cd>To play pranks.</cd> <mark>[Colloq.]</mark> <sd>(b)</sd> <cd>To divide into portions well or ill; to have the property left at one's death turn out well or poorly when divided among heirs, legatees, etc.</cd>  <mark>[Slang.]</mark> <ldquo/When I die, may I <xex>cut up</xex> as well as Morgan Pendennis.<rdquo/</cs> <au>Thackeray.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cut</ent><br/
<hw>Cut</hw>, <pos>n.</pos> <sn>1.</sn> <def>An opening made with an edged instrument; a cleft; a gash; a slash; a wound made by cutting; <as>as, a sword <ex>cut</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stroke or blow or cutting motion with an edged instrument; a stroke or blow with a whip.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which wounds the feelings, as a harsh remark or criticism, or a sarcasm; personal discourtesy, as neglecting to recognize an acquaintance when meeting him; a slight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rip called him by name, but the cur snarled, snapped his teeth, and passed on. This was an unkind <qex>cut</qex> indeed.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A notch, passage, or channel made by cutting or digging; a furrow; a groove; <as>as, a <ex>cut</ex> for a railroad</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This great <qex>cut</qex> or ditch Secostris . . . purposed to have made a great deal wider and deeper.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The surface left by a cut; <as>as, a smooth or clear <ex>cut</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A portion severed or cut off; a division; <as>as, a <ex>cut</ex> of beef; a <ex>cut</ex> of timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It should be understood, moreover, . . . that the group are not arbitrary <qex>cuts</qex>, but natural groups or types.</q> <rj><qau>Dana.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An engraved block or plate; the impression from such an engraving; <as>as, a book illustrated with fine <ex>cuts</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <sd>(a)</sd> <def>The act of dividing a pack cards.</def> <sd>(b)</sd> <def>The right to divide; as, whose <xex>cut</xex> is it?</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Manner in which a thing is cut or formed; shape; style; fashion; <as>as, the <ex>cut</ex> of a garment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With eyes severe and beard of formal <qex>cut</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A common work horse; a gelding.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He'll buy me a <qex>cut</qex>, forth for to ride.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>The failure of a college officer or student to be present at any appointed exercise.</def> <mark>[College Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A skein of yarn.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Lawn Tennis, etc.)</fld> <def>A slanting stroke causing the ball to spin and bound irregularly; also, the spin so given to the ball.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>14.</sn>  <fld>(Cricket)</fld> <def>A stroke on the off side between point and the wicket; also, one who plays this stroke.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>A cut in rates</b></col> <fld>(Railroad)</fld>, <cd>a reduction in fare, freight charges, etc., below the established rates.</cd> -- <col><b>A short cut</b></col>, <cd>a cross route which shortens the way and cuts off a circuitous passage.</cd> -- <col><b>The cut of one's jib</b></col>, <cd>the general appearance of a person.</cd> <mark>[Colloq.]</mark> -- <col><b>To draw cuts</b></col>, <cd>to draw lots, as of paper, etc., cut unequal lengths.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Now draweth <qex>cut</qex> . . . <br/
The which that hath the shortest shall begin.</q> <rj><qau>Chaucer.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cut</ent><br/
<hw>Cut</hw> <pr>(k<ucr/t)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Gashed or divided, as by a cutting instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Formed or shaped as by cutting; carved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Overcome by liquor; tipsy.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cut and dried</b></col>, <cd>prepered beforehand; not spontaneous.</cd> -- <col><b>Cut glass</b></col>, <cd>glass having a surface ground and polished in facets or figures.</cd> -- <col><b>Cut nail</b></col>, <cd>a nail cut by machinery from a rolled plate of iron, in distinction from a <xex>wrought nail</xex>.</cd> -- <col><b>Cut stone</b></col>, <cd>stone hewn or chiseled to shape after having been split from the quarry.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cutaneal</ent><br/
<hw>cu*ta"ne*al</hw> <pos>adj.</pos> <def>same as <er>cutaneous</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cutaneous</ent><br/
<hw>cu*ta"ne*ous</hw> <pr>(k<usl/*t<amac/"n<esl/*<ucr/s)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cutan<eacute/</ets>, fr. L.  <ets>cutis</ets> skin. See <er>Cuticle</er>.]</ety> <def>Of or pertaining to the skin; existing on, or affecting, the skin; <as>as, a <ex>cutaneous</ex> disease; <ex>cutaneous</ex> absorption; <ex>cutaneous</ex> respiration; <ex>cutaneous</ex> nerves</as>; a <ex>cutaneous</ex> infection.</def><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Cutaway</ent><br/
<hw>Cut"a*way`</hw> <pr>(k<ucr/t"<adot/*w<amac/`)</pr>, <pos>a.</pos> <def>Having a part cut off or away; having the corners rounded or cut away.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cutaway coat</b></col>, <cd>a coat whose skirts are cut away in front so as not to meet at the bottom.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cutback</ent><br/
<hw>cutback</hw> <pos>n.</pos> <sn>1.</sn> <def>a reduction in quantity or rate; a reduction in the amount of an activity or the funding for an activity; <as>as, <ex>cutbacks</ex> in government research funding increased unemployment among scientists; the recession caused a <ex>cutback</ex> in auto production</as>.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a return to an earlier scene or story line in a novel, movie, etc.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Sports)</fld> <def>a sudden switching of directions of a running football player, a surfboarder, etc.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cutch</ent><br/
<hw>Cutch</hw> <pr>(k<ucr/ch; 224)</pr>, <pos>n.</pos> <def>See <er>Catechu</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutch</ent><br/
<hw>Cutch</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Cultch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutchery</ent><br/
<hw>Cutch"er*y</hw> <pr>(k<ucr/ch"<etil/r*<ycr/)</pr>, <pos>n.</pos> <ety>[Hind. <ets>kachahri</ets>.]</ety> <def>A hindu hall of justice.</def>  <rj><au>Malcom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cute</ent><br/
<hw>Cute</hw> <pr>(k<umac/t)</pr>, <pos>a.</pos> <ety>[An abbrev. of <ets>acute</ets>.]</ety> <sn>1.</sn> <def>Clever; sharp; shrewd; ingenious; cunning.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>pleasantly attractive, in a delicate way; -- said of children, young ladies, pet animals and some objects, but seldom of adults; <as>as, a <ex>cute</ex> child; a <ex>cute</ex> kitten; a <ex>cute</ex> little house; a <ex>cute</ex> statue</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>smart-alecky or impertinent; <as>as, a <ex>cute</ex> remark</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>affectedly clever.</def> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>Cuteness</ent><br/
<hw>Cute"ness</hw>, <pos>n.</pos> <def>Acuteness; cunning.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuterebridae</ent><br/
<hw>Cuterebridae</hw> <pos>n.</pos> <def>a natural family of New World botflies.</def><br/
<syn><b>Syn. --</b> family <fam>Cuterebridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cutgrass</ent><br/
<hw>Cut"grass`</hw> <pr>(k<ucr/t"gr<adot/s`)</pr>. <def>A grass with leaves having edges furnished with very minute hooked prickles, which form a cutting edge; one or more species of <gen>Leersia</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuticle</ent><br/
<hw>Cu"ti*cle</hw> <pr>(k<umac/"t<icr/*k'l)</pr>, <pos>n.</pos> <ety>[L. <ets>cuticula</ets>, dim. of <ets>cutis</ets> skin; akin to E. <ets>hide</ets> skin of an animal.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The scarfskin or epidermis. See <er>Skin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The outermost skin or pellicle of a plant, found especially in leaves and young stems.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A thin skin formed on the surface of a liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cuticula</ent><br/
<hw>cuticula</hw> <pos>n.</pos> <def>the outer body wall of an insect.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cuticular</ent><br/
<hw>Cu*tic"u*lar</hw> <pr>(k<usl/*t<icr/k"<usl/*l<etil/r)</pr>, <pos>a.</pos> <def>Pertaining to the cuticle, or external coat of the skin; epidermal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cut-in</ent><br/
<hw>cut-in</hw> <pos>n.</pos> <sn>1.</sn> <fld>(film)</fld> <def>a still inserted and interrupting the action.</def><br/
<syn><b>Syn. --</b> insert.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <fld>(broadcasting)</fld> <def>a local announcement inserted into a network broadcast.</def><br/
<syn><b>Syn. --</b> insert.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cutin</ent><br/
<hw>Cu"tin</hw> <pr>(k<umac/"t<icr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>cutis</ets> skin, outside.]</ety> <fld>(Plant Physiol.)</fld> <def>A waxy substance containing fatty acids, soaps, and resinous material which, combined with cellulose, forms a substance nearly impervious to water and constituting the cuticle in plants.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cutinization</ent><br/
<hw>Cu`tin*i*za"tion</hw> <pr>(k?`t?n-?-z?"sh?n)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The conversion of cell walls into a material which repels water, as in cork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutinize</ent><br/
<hw>Cu"tin*ize</hw> <pr>(k?"t?n-?z)</pr>, <pos>v. t. & i.</pos> <def>To change into cutin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutis</ent><br/
||<hw>Cu"tis</hw> <pr>(k?"t?s)</pr>, <pos>n.</pos> <ety>[L. See <er>Cuticle</er>.]</ety> <fld>(Anat.)</fld> <def>See <er>Dermis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cutlas</ent><br/
<hw>cutlas</hw> <pos>n.</pos> <def>variant spelling of <er>cutlass</er>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cutlass</ent><br/
<hw>Cut"lass</hw> <pr>(k<ucr/t"l<ait/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cutlasses</plw> <pr>(-<ecr/z)</pr>.</plu> <ety>[F. <ets>coutelas</ets> (cf. It. <ets>coltellaccio</ets>), augm. fr. L. <ets>cuttellus</ets> a small knife, dim. of <ets>culter</ets> knife. See <er>Colter</er>, and cf. <er>Curtal ax</er>.]</ety> <def>A short, heavy, curving sword, used in the navy. See <er>Curtal ax</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cutlassfish</ent><br/
<ent>cutlass fish</ent><br/
<mhw><hw>cutlass fish</hw>, <hw>cutlassfish</hw></mhw> <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>a peculiar, long, thin, marine fish (<spn>Trichiurus lepturus</spn>) of the southern United States and West Indies, having a long whiplike scaleless body and sharp daggerlike teeth; -- called also <altname>frostfish</altname>, <altname>saber fish</altname>, <altname>silver eel</altname>, and, improperly, <altname>swordfish</altname>; also, several related members of the genus <gen>Trichiurus</gen>.  It is closely related to snake mackerel.</def><br/
<syn><b>Syn. --</b> frost fish, frostfish, hairtail.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Cutler</ent><br/
<hw>Cut"ler</hw> <pr>(k<ucr/t"l<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>coteler</ets>, F. <ets>coutelier</ets>, LL. <ets>cultellarius</ets>, fr. L. <ets>cultellus</ets>. See <er>Cutlass</er>.]</ety> <def>One who makes or deals in cutlery, or knives and other cutting instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutlery</ent><br/
<hw>Cut"ler*y</hw> <pr>(k<ucr/t"l<etil/r*<ycr/)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The business of a cutler.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Edged or cutting instruments, collectively, especially knives for cutting food.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>eating utensils such as knives, forks, and spoons.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cutlet</ent><br/
<hw>Cut"let</hw> <pr>(k<ucr/t"l<ecr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>c<ocir/telette</ets>, prop., little rib, dim. of <ets>c<ocir/te</ets> rib, fr. L.  <ets>costa</ets>. See <er>Coast</er>.]</ety> <def>A piece of meat, especially of veal or mutton, cut for broiling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutling</ent><br/
<hw>Cut"ling</hw> <pr>(k<ucr/t"l<icr/ng)</pr>, <pos>n.</pos>, <ety>[Cf. <er>Cuttle</er> a knife.]</ety> <def>The art of making edged tools or cutlery.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cut-off</ent><br/
<hw>Cut"-off`</hw> <pr>(k<ucr/t"<ocr/f`; 115)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which cuts off or shortens, as a nearer passage or road.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>The valve gearing or mechanism by which steam is cut off from entering the cylinder of a steam engine after a definite point in a stroke, so as to allow the remainder of the stroke to be made by the expansive force of the steam already let in.  See <cref>Expansion gear</cref>, under <er>Expansion</er>.</def> <sd>(b)</sd> <def>Any device for stopping or changing a current, as of grain or water in a spout.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutose</ent><br/
<hw>Cu"tose</hw> <pr>(k<umac/"t<omac/s)</pr>, <pos>n.</pos> <ety>[L.  <ets>cutis</ets> skin.]</ety> <fld>(Chem.)</fld> <def>A variety of cellulose, occuring as a fine transparent membrane covering the aerial organs of plants, and forming an essential ingredient of cork; by oxidation it passes to suberic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 361 --></p>

<p><ent>Cut-out</ent><br/
<hw>Cut"-out`</hw> <pr>(k<ucr/t"out`)</pr>, <pos>n.</pos> <sd>(a)</sd> <sn>1.</sn> <fld>(Telegraphy)</fld> <def>A species of switch for changing the current from one circuit to another, or for shortening a circuit.</def> <sd>(b)</sd> <fld>(Elec.)</fld> <def>A device for breaking or separating a portion of circuit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something cut out from another object, especially a piece of thick paper or cardboard with a picture on it (as of a person or other object), embedded in a larger sheet of paper and having perforations around the edges of the figure to allow easy removal from the larger sheet.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>A valve in the exhaust system of an internal combustion engine which, when activated, permits the exhaust gases to bypass the muffler.</def><br/
[<source>PJC</source>]</p>

<p><ent>cut-price</ent><br/
<hw>cut-price</hw> <pos>adj. prenom.</pos>  <def>same as <er>cut-rate</er>.</def> <mark>[Chiefly British]</mark><br/
<syn><b>Syn. --</b> cut-rate(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cutpurse</ent><br/
<hw>Cut"purse`</hw> <pr>(k<ucr/t"p<ucir/rs`)</pr>, <pos>n.</pos> <def>One who cuts purses for the sake of stealing them or their contents (an act common when men wore purses fastened by a string to their girdles); one who steals from the person; a pickpocket</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To have an open ear, a quick eye, and a nimble hand, is necessary for a <qex>cutpurse</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>cut-rate</ent><br/
<hw>cut-rate</hw> <pos>adj.</pos> <sn>1.</sn>  <def>offering goods or services at less than standard price or rate; <as>as, <ex>cut-rate</ex> store</as>.</def><br/
<syn><b>Syn. --</b> cut-price(prenominal).</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn>  <def>costing less than standard price.</def><br/
<syn><b>Syn. --</b> bargain-priced, cut-price, dirt-cheap.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cut-rate</ent><br/
<hw>cut-rate</hw> <pos>n.</pos> <def>a price below the standard price.</def><br/
<syn><b>Syn. --</b> bargain rate, cheapness, cut rate, cut price.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cuttable</ent><br/
<hw>cuttable</hw> <pos>adj.</pos>  <def>easy to cut or chew.</def><br/
<syn><b>Syn. --</b> chewable.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cutter</ent><br/
<hw>Cut"ter</hw> <pr>(k<ucr/t"t<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who cuts; <as>as, a stone <ex>cutter</ex>; a die <ex>cutter</ex>; esp., one who cuts out garments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which cuts; a machine or part of a machine, or a tool or instrument used for cutting, as that part of a mower which severs the stalk, or as a paper <xex>cutter</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fore tooth; an incisor.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A boat used by ships of war.</def> <sd>(b)</sd> <def>A fast sailing vessel with one mast, rigged in most essentials like a <contr>sloop</contr>.  A cutter is narrower and deeper than a <contr>sloop</contr> of the same length, and depends for stability on a deep keel, often heavily weighted with lead.</def> <sd>(c)</sd> <def>In the United States, a sailing vessel with one mast and a bowsprit, setting one or two headsails.  In Great Britain and Europe, a cutter sets two headsails, with or without a bowsprit.</def> <sd>(d)</sd> <def>A small armed vessel, usually a steamer, in the revenue marine service; -- also called <altname>revenue cutter</altname>.</def><br/
[<source>1913 Webster</source> <source>+RH</source>]</p>

<p><sn>5.</sn> <def>A small, light one-horse sleigh.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An officer in the exchequer who notes by cutting on the tallies the sums paid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A ruffian; a bravo; a destroyer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A kind of soft yellow brick, used for facework; -- so called from the facility with which it can be cut.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cutter bar</b></col>. <fld>(Mach.)</fld> <sd>(a)</sd> <cd>A bar which carries a cutter or cutting tool, as in a boring machine.</cd> <sd>(b)</sd> <cd>The bar to which the triangular knives of a harvester are attached.</cd> -- <col><b>Cutter head</b></col> <fld>(Mach.)</fld>, <cd>a rotating head, which itself forms a cutter, or a rotating stock to which cutters may be attached, as in a planing or matching machine.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutthroat</ent><br/
<hw>Cut"throat`</hw> <pr>(k<ucr/t"thr<omac/t`)</pr>, <pos>n.</pos> <def>One who cuts throats; a murderer; an assassin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutthroat</ent><br/
<hw>Cut"throat`</hw>, <pos>a.</pos> <sn>1.</sn> <def>Murderous; cruel; barbarous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ruthless; conducted without restraint; <as>as, <ex>cutthroat</ex> competition</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cutting</ent><br/
<hw>Cut"ting</hw> <pr>(k<ucr/t"t<icr/ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of making an incision, or of severing, felling, shaping, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something cut, cut off, or cut out, as a twig or scion cut off from a stock for the purpose of grafting or of rooting as an independent plant; something cut out of a newspaper; an excavation cut through a hill or elsewhere to make a way for a railroad, canal, etc.; a cut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutting</ent><br/
<hw>Cut"ting</hw>, <pos>a.</pos> <sn>1.</sn> <def>Adapted to cut; <as>as, a <ex>cutting</ex> tool</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Chilling; penetrating; sharp; <as>as, a <ex>cutting</ex> wind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Severe; sarcastic; biting; <as>as, a <ex>cutting</ex> reply; a <ex>cutting</ex> remark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuttingly</ent><br/
<hw>Cut"ting*ly</hw>, <pos>adv.</pos> <def>In a cutting manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuttle</ent><br/
<hw>Cut"tle</hw> <pr>(k?t"t'l)</pr>, <pos>n.</pos> <ety>[OF. <ets>cultel</ets>, <ets>coltel</ets>, <ets>coutel</ets>, fr. L. <ets>cultellus</ets>. See <er>Cutlass</er>.]</ety> <def>A knife.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuttlefish</ent><br/
<ent>Cuttle</ent><br/
<mhw>{ <hw>Cut"tle</hw> <pr>(k<ucr/t"t'l)</pr>, <hw>Cut"tle*fish`</hw> <pr>(-f<icr/sh`)</pr>, }</mhw> <pos>n.</pos> <ety>[OE. <ets>codule</ets>, AS. <ets>cudele</ets>; akin to G. <ets>kuttelfish</ets>; cf. G. <ets>k<oum/tel</ets>, D. <ets>keutel</ets>, dirt from the guts, G. <ets>kuttel</ets> bowels, entrails. AS. <ets>cwi<thorn/</ets> womb, Goth. <ets>qi<thorn/us</ets> belly, womb.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A cephalopod of the genus <gen>Sepia</gen>, having an internal shell, large eyes, and ten arms furnished with denticulated suckers, by means of which it secures its prey. The name is sometimes applied to dibranchiate cephalopods generally.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It has an <xex>ink bag</xex>, opening into the siphon, from which, when pursued, it throws out a dark liquid that clouds the water, enabling it to escape observation.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A foul-mouthed fellow.</def> <ldquo/An you play the saucy <xex>cuttle</xex> with me.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuttle bone</ent><br/
<hw>Cut"tle bone`</hw> <pr>(b<omac/n`)</pr>. <def>The shell or bone of cuttlefishes, used for various purposes, as for making polishing powder, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuttoo plate</ent><br/
<hw>Cut*too" plate`</hw> <pr>(k?t-t??" pl?t`)</pr>. <def>A hood over the end of a wagon wheel hub to keep dirt away from the axle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutty</ent><br/
<hw>Cut"ty</hw> <pr>(k<ucr/t"t<ycr/)</pr>, <pos>a.</pos> <ety>[Cf. Ir. & Gael. <ets>cut</ets> a short tail, <ets>cutach</ets> bobtailed. See <er>Cut</er>.]</ety> <def>Short; <as>as, a <ex>cutty</ex> knife; a <ex>cutty</ex> sark</as>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutty</ent><br/
<hw>Cut"ty</hw> <pr>(k?t"t?)</pr>, <pos>n.</pos> <ety>[Scotch.]</ety> <sn>1.</sn> <def>A short spoon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short tobacco pipe.</def>  <rj><au>Ramsay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A light or unchaste woman.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuttystool</ent><br/
<hw>Cut"ty*stool`</hw> <pr>(-st<oomac/l`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A low stool.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A seat in old Scottish churches, where offenders were made to sit, for public rebuke by the minister.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutwal</ent><br/
||<hw>Cut"wal</hw> <pr>(k<ucr/t"w<add/l)</pr>, <pos>n.</pos> <ety>[Per. <ets>kotw<amac/l</ets>.]</ety> <def>The chief police officer of a large city.</def> <mark>[East Indies]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutwater</ent><br/
<hw>Cut"wa`ter</hw> <pr>(k<ucr/t"w<add/`t<etil/r)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <sn>1.</sn> <def>The fore part of a ship's prow, which cuts the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A starling or other structure attached to the pier of a bridge, with an angle or edge directed up stream, in order better to resist the action of water, ice, etc.; the sharpened upper end of the pier itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A sea bird of the Atlantic (<spn>Rhynchops nigra</spn>); -- called also <altname>black skimmer</altname>, <altname>scissorsbill</altname>, and <altname>razorbill</altname>. See <er>Skimmer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutwork</ent><br/
<hw>Cut"work`</hw> <pr>(k<ucr/t"w<ucir/rk`)</pr>, <pos>n.</pos> <fld>(Fine Arts)</fld> <def>An ancient term for embroidery, esp. applied to the earliest form of lace, or to that early embroidery on linen and the like, from which the manufacture of lace was developed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cutworm</ent><br/
<hw>Cut"worm`</hw> <pr>(-w?rm`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A caterpillar which at night eats off young plants of cabbage, corn, etc., usually at the ground. Some kinds ascend fruit trees and eat off the flower buds. During the day, they conceal themselves in the earth. The common cutworms are the larv<ae/ of various species of <gen>Agrotis</gen> and related genera of noctuid moths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cuvette</ent><br/
||<hw>Cu*vette"</hw> <pr>(k?-w?t")</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>cuve</ets> a tub.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A pot, bucket, or basin, in which molten plate glass is carried from the melting pot to the casting table.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fort.)</fld> <def>A cunette.</def></p>

<p><-- <sn>3.</sn> <fld>(Spectrometry)</fld> <fld>(Analytical chemistry)</fld> <def>A small vessel with at least two flat and transparent sides, used to hold a liquid sample to be analysed in the light path of a spectrometer.</def> <note>The shape and materials vary; for ultraviolet spectrometry, quartz is typically used.  For visible-light spectrometry, plastic cuvettes may be employed.  Occasionally, small vessels used for other laboratory purposes are called <ex>cuvettes.</ex></note><br/
<br/
<cs><col><b>cuvette holder</b></col>, <fld>(Spectrometry)</fld> <cd>A small device used to hold one or more cuvettes[3], shaped specifically to fit in the sample chamber of a particular type of spectrometer, with openings to permit light to pass through the holder and the cuvettes, and designed so as to hold the cuvette accurately and reproducibly within the light path of the spectrometer.  For cuvettes with a square horizontal cross-section, the compartments will have a corresponding square cross-section, usu. slightly larger than the cuvette.</cd></cs> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>C.V.</ent><br/
<ent>CV</ent><br/
<mhw><hw>CV</hw>, <hw>C.V.</hw></mhw> <pr>(s<emac/`v<emac/")</pr>, <pos>n.</pos> <def>Same as <er>curriculum vitae</er>; -- a commonly used initialism.</def> <mark>[initialism]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Cyamelide</ent><br/
<hw>Cy*am"e*lide</hw> <pr>(s<isl/*<acr/m"<esl/*l<icr/d <it>or</it> -l<imac/d; 104)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white amorphous substance, regarded as a polymeric modification of isocyanic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyamellone</ent><br/
<hw>Cy*am"el*lone</hw> <pr>(s<isl/*<acr/m"<ecr/l*l<omac/n)</pr>, <pos>n.</pos> <fld>(Chem)</fld> <def>A complex derivative of cyanogen, regarded as an acid, and known chiefly in its salts; -- called also <altname>hydromellonic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyamus</ent><br/
<hw>Cyamus</hw> <pos>prop. n.</pos> <def>a genus of whale lice.</def><br/
<syn><b>Syn. --</b> genus <gen>Cyamus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyanate</ent><br/
<hw>Cy"a*nate</hw> <pr>(s?"?-n?t)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>cuanate</ets>. See <er>Cyanic</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of cyanic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ammonium cyanate</b></col> <fld>(Chem.)</fld>, <cd>a remarkable white crystalline substance, <chform>NH4.O.CN</chform>, which passes, on standing, to the organic compound, urea, <chform>CO.(NH2)2</chform>.</cd></cs>
<-- *note* there was an *error* in the urea formula "<chform>CO.(NH)2</chform>" in the original --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanaurate</ent><br/
<hw>Cy`an*au"rate</hw> <pr>(s?`?n-?"r?t)</pr>, <pos>n.</pos> <def>See <er>Aurocyanide</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanean</ent><br/
<hw>Cy*a"ne*an</hw> <pr>(s?-?"n?-<ait/n)</pr>, <pos>a.</pos> <ety>[Gr. <grk>kya`neos</grk> dark blue.]</ety> <def>Having an azure color.</def>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanic</ent><br/
<hw>Cy*an"ic</hw> <pr>(s?-?n"?k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ky`anos</grk> a dark blue substance: cf. F. <ets>cyanique</ets>. Cf. <er>Kyanite</er>.]</ety> <sn>1.</sn> <def>Pertaining to, or containing, cyanogen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a blue color.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cyanic acid</b></col> <fld>(Chem.)</fld>, <cd>an acid, <chform>HOCN</chform>, derived from cyanogen, well known in its salts, but never isolated in the free state.</cd> -- <col><b>Cyanic colors</b></col> <fld>(Bot.)</fld>, <cd>those colors (of flowers) having some tinge of blue; -- opposed to <contr>xanthic colors</contr>. A color of either series may pass into red or white, but not into the opposing color. Red and pure white are more common among flowers of cyanic tendency than in those of the other class.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanide</ent><br/
<hw>Cy"a*nide</hw> <pr>(s?"?-n?d <or/ -n?d; 104)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>cyanide</ets>. See <er>Cyanic</er>.]</ety> <fld>(Chem.)</fld> <def>A compound formed by the union of cyanogen with an element or radical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanin</ent><br/
<hw>Cy"a*nin</hw> <pr>(s?"?-n?n)</pr>, <pos>n.</pos> <ety>[See <er>Cyanic</er>.]</ety> <fld>(Chem.)</fld> <def>The blue coloring matter of flowers; -- called also <altname>anthokyan</altname> and <altname>anthocyanin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanine</ent><br/
<hw>Cy"a*nine</hw> <pr>(s?"?-n?n <or/ -n?n; 104)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>One of a series of artificial blue or red dyes obtained from quinoline and lepidine and used in calico printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanite</ent><br/
<hw>Cy"a*nite</hw> <pr>(-n?t)</pr>, <pos>n.</pos> <ety>[See <er>Cyanic</er>.]</ety> <fld>(Min.)</fld> <def>A mineral occuring in thin-bladed crystals and crystalline aggregates, of a sky-blue color. It is a silicate of aluminium.</def> <altsp>[Written also <asp>kyanite</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>cyanobacteria</ent><br/
<ent>cyanobacterium</ent><br/
<hw>cyanobacterium</hw> <plu><it>pl.</it> <hw>cyanobacteria</hw></plu> <ety>[from the pigment phyco<ets>cyan</ets>in.]</ety> <def>any of a group of photosynthetic autotrophic prokaryotic microorganisms possessing characteristics of both bacteria and plants.  When classed as bacteria, they are assigned to the <class>Cyanobacteria</class>; when classed as plants, they are assigned to the <phylum>Cyanophyta</phylum>.  They were earlier named <altname>blue-green algae</altname>, a term less used now in technical discussions.  Since the chlorophyll within the cyanobacteria is diffused throughout the cell, rather than being contained in chloroplasts, they are no longer thought of as true plants.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cyanogen</ent><br/
<hw>Cy*an"o*gen</hw> <pr>(s?-?n"?-j?n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`anos</grk> a dark blue substance + <ets>-gen</ets>: cf. F.  <ets>cyanog<egrave/ne</ets>. So called because it produced blue dyes.]</ety> <fld>(Chem.)</fld> <def>A colorless, inflammable, poisonous gas, <chform>C2N2</chform>, with a peach-blossom odor, so called from its tendency to form <xex>blue</xex> compounds; obtained by heating ammonium oxalate, mercuric cyanide, etc. It is obtained in combination, forming an alkaline cyanide when nitrogen or a nitrogenous compound is strongly ignited with carbon and soda or potash. It conducts itself like a member of the halogen group of elements, and shows a tendency to form complex compounds. The name is also applied to the univalent radical, <chform>CN</chform> (the half molecule of cyanogen proper), which was one of the first compound radicals recognized.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Cyanogen</ex> is found in the commercial substances, potassium cyanide, or prussiate of potash, yellow prussiate of potash, Prussian blue, Turnbull's blue, prussic acid, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanometer</ent><br/
<hw>Cy`a*nom"e*ter</hw> <pr>(s?`?-n?m"?-t?r)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`anos</grk> a dark blue substance + <ets>-meter</ets>: cf. F. <ets>cyanom<egrave/tre</ets>.]</ety> <def>An instrument for measuring degress of blueness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanopathy</ent><br/
<hw>Cy`a*nop"a*thy</hw> <pr>(-n?p"?-th?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`anos</grk> a dark blue substance + <grk>pa`qos</grk> affection.]</ety> <fld>(Med.)</fld> <def>A disease in which the body is colored blue in its surface, arising usually from a malformation of the heart, which causes an imperfect arterialization of the blood; blue jaundice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanophyll</ent><br/
<hw>Cy*an"o*phyll</hw> <pr>(s?-?n"?-f?l)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`anos</grk> a dark blue substance + <grk>fy`llon</grk> leaf.]</ety> <fld>(Bot.)</fld> <def>A blue coloring matter supposed by some to be one of the component parts of chlorophyll.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanosed</ent><br/
<hw>Cy"a*nosed</hw> <pr>(s?"?-n?st)</pr>, <pos>a.</pos> <ety>[See <er>Cyanic</er>.]</ety> <def>Rendered blue, as the surface of the body, from cyanosis or deficient a<eum/ration of the blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanosis</ent><br/
||<hw>Cy`a*no"sis</hw> <pr>(s?`?-n?"s?s)</pr>, <pos>n.</pos> <ety>[NL. See <er>Cyanic</er>.]</ety> <fld>(Med.)</fld> <def>A condition in which, from insufficient a<eum/ration of the blood, the surface of the body becomes blue. See <er>Cyanopathy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanosite</ent><br/
<hw>Cy*an"o*site</hw> <pr>(s?-?n"?-s?t)</pr>, <pos>n.</pos> <ety>[See <er>Cyanic</er>.]</ety> <fld>(Min.)</fld> <def>Native sulphate of copper. Cf. <cref>Blue vitriol</cref>, under <er>Blue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanotic</ent><br/
<hw>Cy`a*not"ic</hw> <pr>(s?`?-n?t"?k)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Relating to cyanosis; affected with cyanosis; <as>as, a <ex>cyanotic</ex> patient</as>; having the hue caused by cyanosis; <as>as, a <ex>cyanotic</ex> skin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanotype</ent><br/
<hw>Cy*an"o*type</hw> <pr>(s?-?n"?-t?p)</pr>, <pos>n.</pos> <ety>[<ets>Cyanide</ets> + <ets>-type</ets>.]</ety> <def>A photographic picture obtained by the use of a cyanide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanurate</ent><br/
<hw>Cy"an"u*rate</hw> <pr>(s?-?n"?-r?t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of cyanuric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanuret</ent><br/
<hw>Cy*an"u*ret</hw> <pr>(-r?t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A cyanide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanuric</ent><br/
<hw>Cy`a*nu"ric</hw> <pr>(s?`?-n?"r?k)</pr>, <pos>a.</pos>   <ety>[<ets>Cyanic</ets> + <ets>uric</ets>: Cf. F. <ets>cyanurique</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or derived from, cyanic and uric acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyanuric acid</ent><br/
<hw>Cyanuric acid</hw> <fld>(Chem.)</fld>, <def>an organic acid, <chform>C3O3N3H3</chform>, first obtained by heating <xex>uric</xex> acid or <xex>urea</xex>, and called <altname>pyrouric acid</altname>; afterwards obtained from <cref>isocyanic acid</cref>. It is a white crystalline substance, odorless and almost tasteless; -- called also <altname>tricarbimide</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyathiform</ent><br/
<hw>Cy*ath"i*form</hw> <pr>(s?-?th"?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cyathus</ets> a cup (Gr, <grk>ky`aqos</grk>) <ets>-form</ets>:cf. F.  <ets>cyathiforme</ets>.]</ety> <def>In the form of a cup, a little widened at the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyatholith</ent><br/
<hw>Cy*ath"olith</hw> <pr>(s?-?th"?-l?th)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`aqos</grk> a cup + <ets>-lith</ets>.]</ety> <fld>(Biol.)</fld> <def>A kind of coccolith, which in shape resembles a minute cup widened at the top, and varies in size from <frac1x6000/ to <frac1x8000/ of an inch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyathophylloid</ent><br/
<hw>Cy`a*tho*phyl"loid</hw> <pr>(s?`?-th?-f?l"loid)</pr>, <pos>a.</pos> <ety>[NL. <ets>cyathophyllum</ets>, fr. Gr. <grk>ky`aqos</grk> a cup + <grk>fy`llon</grk> a leaf.]</ety> <fld>(Paleon.)</fld> <def>Like, or pertaining to, the family <fam>Cyathophyllid<ae/</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyathophylloid</ent><br/
<hw>Cy`a*tho*phyl"loid</hw>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>A fossil coral of the family <fam>Cyathophyllid<ae/</fam>; sometimes extended to fossil corals of other related families belonging to the group Rugosa; -- also called <altname>cup corals</altname>. Thay are found in paleozoic rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycad</ent><br/
<hw>Cy"cad</hw> <pr>(s<imac/"k<acr/d)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the natural order <ord>Cycadace<ae/</ord>, as the sago palm, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycadaceae</ent><br/
<hw>Cycadaceae</hw> <pos>n.</pos> <def>a natural family of ancient palmlike plants closely related to ferns in that fertilization is by means of spermatozoids.</def><br/
<syn><b>Syn. --</b> family <fam>Cycadaceae</fam>, cycad family.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cycadaceous</ent><br/
<hw>Cyc`a*da"ceous</hw> <pr>(s?k`?-d?"sh?s <or/  s?`k?-)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Pertaining to, or resembling, an order of plants like the palms, but having exogenous wood. The sago palm is an example.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycadales</ent><br/
<hw>Cycadales</hw> <pos>n.</pos> <def>an order of primitive tropical gymnosperms abundant in the Mesozoic, now reduced to a few scattered tropical forms.</def><br/
<syn><b>Syn. --</b> order <ord>Cycadales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cycadofilicales</ent><br/
<hw>Cycadofilicales</hw> <pos>n.</pos> <def>an order of fossil gymnospermous trees or climbing plants from the Devonian: seed ferns.</def><br/
<syn><b>Syn. --</b> order <ord>Cycadofilicales</ord>, Lyginopteridales, order <ord>Lyginopteridales</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cycadopsida</ent><br/
<ent>Cycadophytina</ent><br/
<ent>Cycadophyta</ent><br/
<mhw><hw>Cycadophyta</hw>, <hw>Cycadophytina</hw>, <hw>Cycadopsida</hw></mhw> <pos>n.</pos> <def>a class or subdivision of plants comprising palmlike gymnosperms: it includes the surviving order <ord>Cycadales</ord> and several extinct orders; possibly not a natural group; in some systems considered a class (<class>Cycadopsida</class>) and in others a subdivision (Cycadophytina or Cycadophyta).</def><br/
<syn><b>Syn. --</b> class Cycadopsida, subdivision Cycadophytina, subdivision Cycadophyta.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cycas</ent><br/
<hw>Cy"cas</hw> <pr>(s?"k?s)</pr>, <pos>n.</pos> <ety>[Of uncertain origin. Linn<ae/us derives it from one of the <ldquo/obscure Greek words.<rdquo/]</ety> <fld>(Bot.)</fld> <def>A genus of trees, intermediate in character between the palms and the pines. The pith of the trunk of some species furnishes a valuable kind of sago.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclades</ent><br/
<hw>Cyclades</hw> <pos>n.</pos> <sn>1.</sn> <def>the pre-Mycenaean civilization on the Cyclades islands in the S Aegean sea.</def><br/
<syn><b>Syn. --</b> Cycladic civilization, Cycladic culture.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>an archipelago consisting of over 200 islands in the southern Aegean sea.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyclamen</ent><br/
<hw>Cyc"la*men</hw> <pr>(s?k"l?-m?n)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kykla`minos</grk>, <grk>kyklami`s</grk>.]</ety> <fld>(Bot.)</fld> <def>A genus of plants of the Primrose family, having depressed rounded corms, and pretty nodding flowers with the petals so reflexed as to point upwards, whence it is called <altname>rabbits' ears</altname>. It is also called <altname>sow bread</altname>, because hogs are said to eat the corms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclamin</ent><br/
<hw>Cyc"la*min</hw> <pr>(-m?n)</pr>, <pos>n.</pos> <def>A white amorphous substance, regarded as a glucoside, extracted from the corm of <spn>Cyclamen Europ<ae/um</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclas</ent><br/
<hw>Cy"clas</hw> <pr>(s?"kl?s)</pr>, <pos>n.</pos> <ety>[Cf. <er>Ciclatoun</er>.]</ety> <def>A long gown or surcoat (cut off in front), worn in the Middle Ages. It was sometimes embroidered or interwoven with gold. Also, a rich stuff from which the gown was made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycle</ent><br/
<hw>Cy"cle</hw> <pr>(s?"k'l)</pr>, <pos>n.</pos> <ety>[F. <ets>ycle</ets>, LL. <ets>cyclus</ets>, fr. Gr. <grk>ky`klos</grk> ring or circle, cycle; akin to Skr. <ets>cakra</ets> wheel, circle. See <er>Wheel</er>.]</ety> <sn>1.</sn> <def>An imaginary circle or orbit in the heavens; one of the celestial spheres.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An interval of time in which a certain succession of events or phenomena is completed, and then returns again and again, uniformly and continually in the same order; a periodical space of time marked by the recurrence of something peculiar; <as>as, the <ex>cycle</ex> of the seasons, or of the year</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wages . . . bear a full proportion . . . to the medium of provision during the last bad <qex>cycle</qex> of twenty years.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An age; a long period of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Better fifty years of Europe than a <qex>cycle</qex> of Cathay.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An orderly list for a given time; a calendar.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We . . . present our gardeners with a complete <qex>cycle</qex> of what is requisite to be done throughout every month of the year.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The circle of subjects connected with the exploits of the hero or heroes of some particular period which have served as a popular theme for poetry, as the legend of Arthur and the knights of the Round Table, and that of Charlemagne and his paladins.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>One entire round in a circle or a spire; <as>as, a <ex>cycle</ex> or set of leaves</as>.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A bicycle or tricycle, or other light velocipede.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A motorcycle.</def><br/
[<source>PJC</source>]</p>

<p><sn>9.</sn> <fld>(Thermodynamics)</fld> <def>A series of operations in which heat is imparted to (or taken away from) a working substance which by its expansion gives up a part of its internal energy in the form of mechanical work (or being compressed increases its internal energy) and is again brought back to its original state.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>10.</sn> <fld>(Technology)</fld> <def>A complete positive and negative, or forward and reverse, action of any periodic process, such as a vibration, an electric field oscillation, or a current alternation; one period.</def> <specif>Hence:</specif> <fld>(Elec.)</fld> <def>A complete positive and negative wave of an alternating current. The number of cycles (per second) is a measure of the frequency of an alternating current.</def><br/
[<source>Webster 1913 Suppl.</source> + <source>PJC</source>]</p>

<p><cs><col><b>Calippic cycle</b></col>, <cd>a period of 76 years, or four Metonic cycles; -- so called from Calippus, who proposed it as an improvement on the Metonic cycle.</cd> -- <col><b>Cycle of eclipses</b></col>, <cd>a period of about 6,586 days, the time of revolution of the moon's node; -- called <altname>Saros</altname> by the Chaldeans.</cd><-- Saros cycle. --> -- <col><b>Cycle of indiction</b></col>, <cd>a period of 15 years, employed in Roman and ecclesiastical chronology, not founded on any astronomical period, but having reference to certain judicial acts which took place at stated epochs under the Greek emperors.</cd> -- <mcol><col><b>Cycle of the moon</b></col>, <it>or</it> <col><b>Metonic cycle</b></col></mcol>, <cd>a period of 19 years, after the lapse of which the new and full moon returns to the same day of the year; -- so called from Meton, who first proposed it.</cd> -- <mcol><col><b>Cycle of the sun</b></col>, <col><b>Solar cycle</b></col></mcol>, <cd>a period of 28 years, at the end of which time the days of the month return to the same days of the week. The <xex>dominical</xex> or <xex>Sunday letter</xex> follows the same order; hence the <ex>solar cycle</ex> is also called the <altname>cycle of the Sunday letter</altname>. In the Gregorian calendar the <xex>solar cycle</xex> is in general interrupted at the end of the century.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycle</ent><br/
<hw>Cy"cle</hw> <pr>(s?"k'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Cycled</conjf>. <pr>(-k'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Cycling</conjf> <pr>(-kl<?/ng)</pr>.]</vmorph> <sn>1.</sn> <def>To pass through a cycle{2} of changes; to recur in cycles.</def>  <rj><au>Tennyson.</au>  <au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ride a bicycle, tricycle, or other form of cycle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cycle</ent><br/
<hw>cy"cle</hw> <pr>(s?"k'l)</pr>, <pos>v. t.</pos> <def>To cause to pass through a cycle{2}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cyclical</ent><br/
<ent>Cyclic</ent><br/
<mhw>{ <hw>Cyc"lic</hw> <pr>(s?k"l?k <or/ s?"kl?k)</pr>, <hw>Cyc"lic*al</hw> <pr>(s?k"l?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F.  <ets>cycluque</ets>, Gr. <grk>kykliko`s</grk>, fr. <grk>ky`klos</grk> See <er>Cycle</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to a cycle or circle; moving in cycles; <as>as, <ex>cyclical</ex> time</as>.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Chemistry)</fld> <def>Having atoms bonded to form a ring structure.  Opposite of <ant>acyclic</ant>.</def>  <note>Used most commonly in respect to organic compounds.</note> <note>[Narrower terms: <stype>bicyclic</stype>; <stype>heterocyclic</stype>; <stype>homocyclic, isocyclic</stype>]</note><br/
<syn><b>Syn. --</b>  closed-chain, closed-ring.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn>  <def>Recurring in cycles{2}; having a pattern that repeats at approximately equal intervals; periodic.  Opposite of <ant>noncyclic</ant>.</def>  <note>[Narrower terms: <stype>alternate(prenominal), alternating(prenominal)</stype>; <stype>alternate(prenominal), every other(prenominal), every second(prenominal)</stype>; <stype>alternating(prenominal), oscillating(prenominal)</stype>; <stype>biyearly</stype>; <stype>circadian  exhibiting 24-hour periodicity)</stype>; <stype>circular</stype>; <stype>daily, diurnal</stype>; <stype>fortnightly, biweekly</stype>; <stype>hourly</stype>; <stype>midweek, midweekly</stype>; <stype>seasonal</stype>; <stype>semestral, semestrial</stype>; <stype>semiannual, biannual, biyearly</stype>; <stype>semiweekly, biweekly</stype>; <stype>weekly</stype>; <stype>annual, yearly</stype>; <stype>biennial</stype>; <stype>bimonthly, bimestrial</stype>; <stype>half-hourly</stype>; <stype>half-yearly</stype>; <stype>monthly</stype>; <stype>tertian, alternate(prenominal)</stype>; <stype>triennial</stype>]</note><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>4.</sn>  <def>Marked by repeated cycles{2}.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><cs><col><b>Cyclic chorus</b></col>, <cd>the chorus which performed the songs and dances of the dithyrambic odes at Athens, dancing round the altar of Bacchus in a circle.</cd> -- <col><b>Cyclic poets</b></col>, <cd>certain epic poets who followed Homer, and wrote merely on the Trojan war and its heroes; -- so called because keeping within the circle of a single subject. Also, any series or coterie of poets writing on one subject.</cd>  <rj><au>Milman.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 362 --></p>

<p><ent>cyclide</ent><br/
<hw>cy"clide</hw> <pr>(s?"kl?d)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`klos</grk> circle.]</ety> <fld>(Geom.)</fld> <def>A surface of the fourth degree, having certain special relations to spherical surfaces. The tore or anchor ring is one of the cyclides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycling</ent><br/
<hw>Cy"cling</hw> <pr>(s?"kl?ng)</pr>, <pos>n.</pos>  <def>The act, art, or practice, of riding a cycle, esp. a bicycle or tricycle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclist</ent><br/
<hw>Cy"clist</hw> <pr>(s?"kl?st)</pr>, <pos>n.</pos> <def>A cycler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclo-</ent><br/
<hw>Cy"clo-</hw> <pr>(s?"kl?-)</pr>. <ety>[Gr. <grk>ky`klos</grk> circle, wheel.]</ety> <def>A combining form meaning <xex>circular</xex>, <xex>of a circle</xex> or <xex>wheel</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclobranchiate</ent><br/
<hw>Cy`clo*bran"chi*ate</hw> <pr>(s?`kl?-br?n"k?-?t)</pr>, <pos>a.</pos> <ety>[<ets>Cyclo-</ets> + <ets>branchiate</ets>.]</ety> <fld>(Zool)</fld> <def>Having the gills around the margin of the body, as certain limpets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloganoid</ent><br/
<hw>Cy`clo*ga"noid</hw> <pr>(s?`kl?-g?"noid <or/ -g?n"oid)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Cycloganoidei.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloganoid</ent><br/
<hw>Cy`clo*ga"noid</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cycloganoidei.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloganoidei</ent><br/
||<hw>Cy`clo*ga*noi"de*i</hw> <pr>(s?"kl?-g?-noi"d?-?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ky`klos</grk> circle + NL. <ets>ganoidei</ets>. See <er>Ganoid</er>.]</ety> <fld>(Zool.)</fld> <def>An order of ganoid fishes, having cycloid scales. The bowfin (<spn>Amia calva</spn>) is a living example.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclograph</ent><br/
<hw>Cy"clo*graph</hw> <pr>(s?"kl?-gr?f)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo-</ets> + <ets>-graph</ets>.]</ety> <def>See <er>Arcograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloid</ent><br/
<hw>Cy"cloid</hw> <pr>(s?"kloid)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo-</ets> + <ets>-oid</ets>: cf. F. <ets>cyclo<ium/de</ets>.]</ety> <fld>(Geom.)</fld> <def>A curve generated by a point in the plane of a circle when the circle is rolled along a straight line, keeping always in the same plane.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>common cycloid</xex> is the curve described when the generating point (<xex>p</xex>) is on the circumference of the generating circle; the <xex>curtate cycloid</xex>, when that point lies without the circumference; the <xex>prolate</xex> or <xex>inflected cycloid</xex>, when the generating point (<xex>p</xex>) lies within that circumference.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloid</ent><br/
<hw>Cy"cloid</hw>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Cycloidei.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cycloid scale</b></col> <fld>(Zool.)</fld>, <cd>a fish scale which is thin and shows concentric lines of growth, without serrations on the margin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloid</ent><br/
<hw>Cy"cloid</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cycloidei.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloidal</ent><br/
<hw>Cy*cloid"al</hw> <pr>(-<it>a</it>l)</pr>, <pos>a.</pos> <def>Pertaining to, or resembling, a cycloid; <as>as, the <ex>cycloidal</ex> space is the space contained between a cycloid and its base</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cycloidal engine</b></col>. <cd>See <er>Geometric lathe</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloidei</ent><br/
||<hw>Cy*cloi"de*i</hw> <pr>(s?-kloi"d?-?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ky`klos</grk> circle + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>An order of fishes, formerly proposed by Agassiz, for those with thin, smooth scales, destitute of marginal spines, as the herring and salmon. The group is now regarded as artificial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloidian</ent><br/
<hw>Cy*cloid"i*an</hw> <pr>(s?-kloid"?-<it>a</it>n)</pr>, <pos>a. & n.</pos> <fld>(Zool.)</fld> <def>Same as 2d and 3d <er>Cycloid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclometer</ent><br/
<hw>Cy*clom"e*ter</hw> <pr>(s?-kl?m"?-t?r)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo-</ets>  + <ets>-meter</ets>.]</ety> <def>A contrivance for recording the revolutions of a wheel, as of a bicycle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclometry</ent><br/
<hw>Cy*clom"e*try</hw> <pr>(-tr?)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo-</ets> + <ets>-metry</ets>: cf. F. <ets>cyclom<eacute/trie</ets>.]</ety> <fld>(Geom.)</fld> <def>The art of measuring circles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclone</ent><br/
<hw>Cy"clone</hw> <pr>(s?"kl?n)</pr>, <pos>n.</pos> <ety>[Gr.<?/<?/<?/<?/<?/ moving in a circle, p. pr. of <?/<?/<?/<?/<?/, fr. <grk>ky`klos</grk> circle.]</ety> <sn>1.</sn> <fld>(Meteor.)</fld> <def>A violent storm, often of vast extent, characterized by high winds rotating about a calm center of low atmospheric pressure. This center moves onward, often with a velocity of twenty or thirty miles an hour.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The atmospheric disturbance usually accompanying a cyclone, marked by an onward moving area of high pressure, is called an <xex>anticyclone</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Meteor.)</fld> <def>In general, a condition of the atmosphere characterized by a central area of pressure much lower than that of surrounding areas, and a system of winds blowing inward and around (clockwise in the southern hemisphere and counter-clockwise in the northern); -- called also a <altname>low-area storm</altname>. It is attended by high temperature, moist air, abundant precipitation, and clouded sky. The term includes the hurricane, typhoon, and tropical storms; it should not be applied to the moderate disturbances attending ordinary areas of low pressure nor to tornadoes, waterspouts, or <ldquo/twisters,<rdquo/ in which the vertical motion is more important than the horizontal.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A tornado. See above, and <er>Tornado</er>.</def> <mark>[Middle U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cyclone pit</ent><br/
<ent>Cyclone cellar</ent><br/
<mhw>{ <hw>Cyclone cellar</hw> <it>or</it>  <hw>Cyclone pit</hw>  }</mhw>. <def>A cellar or excavation used for refuge from a cyclone, or tornado.</def> <mark>[Middle U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cyclonic</ent><br/
<hw>Cy*clon"ic</hw> <pr>(s?-kl?n"?k)</pr>, <pos>a.</pos> <def>Pertaining to a cyclone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclonoscope</ent><br/
<hw>Cy*clo"no*scope</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Cyclone</ets> + <ets>-scope</ets>.]</ety> <def>An apparatus to assist in locating the center of a cyclone.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cyclop</ent><br/
<hw>Cy"clop</hw> <pr>(s?"kl?p)</pr>, <pos>n.</pos> <def>See Note under <er>Cyclops</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclopean</ent><br/
<hw>Cy`clo*pe"an</hw> <pr>(s?`kl?-p?"<it>a</it>n)</pr>, <pos>a.</pos> <ety>[L. <ets>Cyclopeus</ets>, Gr. <?/<?/<?/<?/<?/, fr. <?/<?/<?/<?/<?/ Cyclops: cf. F. <ets>cyclopeen</ets>.]</ety> <def>Pertaining to the Cyclops; characteristic of the Cyclops; huge; gigantic; vast and rough; massive; <as>as, <ex>Cyclopean</ex> labors; <ex>Cyclopean</ex> architecture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclopaedia</ent><br/
<ent>Cyclopedia</ent><br/
<mhw>{ <hw>Cy`clo*pe"di*a</hw> <hw>Cy`clo*p<ae/"di*a</hw> }</mhw> <pr>(s?`kl?-p?"d?-?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>ky`klos</grk> circle + <grk>paidei`a</grk> the bringing up of a child, education, erudition, fr. <grk>paidey`ein</grk> to bring up a child. See <er>Cycle</er>, and cf. <er>Encyclopedia</er>, <er>Pedagogue</er>.]</ety> <sn>1.</sn> <def>The circle or compass of the arts and sciences (originally, of the seven so-called liberal arts and sciences); circle of human knowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>A work containing, in alphabetical order, information in all departments of knowledge, or on a particular department or branch; <as>as, a <ex>cyclopedia</ex> of the physical sciences, or of mechanics</as>; an encyclopedia.  See <er>Encyclopedia</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cyclopedic</ent><br/
<hw>Cy`clo*ped"ic</hw> <pr>(s?`kl?-p?d"?k <or/  -p?"d?k)</pr>, <pos>a.</pos> <def>Belonging to the circle of the sciences, or to a cyclopedia; of the nature of a cyclopedia; hence, of great range, extent, or amount; <as>as, a man of <ex>cyclopedic</ex> knowledge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclopedist</ent><br/
<hw>Cy`clo*pe"dist</hw> <pr>(-p?"d?st)</pr>, <pos>n.</pos> <def>A maker of, or writer for, a cyclopedia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclophorus</ent><br/
<hw>Cyclophorus</hw> <pos>n.</pos> <def>a genus of tropical Old World ferns having closely crowded circular sori and no indusia.</def><br/
<syn><b>Syn. --</b> genus <gen>Cyclophorus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyclopic</ent><br/
<hw>Cy*clop"ic</hw> <pr>(s?-kl?p"?k)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/<?/<?/, fr. <?/<?/<?/<?/.]</ety> <def>Pertaining to the Cyclops; Cyclopean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cyclopropane</ent><br/
<hw>cyclopropane</hw> <pos>n.</pos> <def>a colorless flammable gas (<chform>C3H6</chform>) with a three-carbon ring, sometimes used as an anesthetic.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyclops</ent><br/
<hw>Cy"clops</hw> <pr>(s?"kl?ps)</pr>, <pos>n. sing. & pl.</pos> <ety>[L. <ets>Cyclops</ets>, Gr. <grk>Ky`klwps</grk> (strictly round-eyed), pl. <grk>Ky`klwpes</grk>; <grk>ky`klos</grk> circle + <grk>'w`ps</grk> eye.]</ety> <sn>1.</sn> <fld>(Gr. Myth.)</fld> <def>One of a race of giants, sons of Neptune and Amphitrite, having but one eye, and that in the middle of the forehead. They were fabled to inhabit Sicily, and to assist in the workshops of Vulcan, under Mt. Etna.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Pope, in his translation of the <ldquo/Odyssey,<rdquo/ uniformly spells this word <xex>Cyclop</xex>, when used in the singular.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of minute Entomostraca, found both in fresh and salt water. See <er>Copepoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A portable forge, used by tinkers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclopteridae</ent><br/
<hw>Cyclopteridae</hw> <pos>n.</pos> <def>a natural family comprising the lumpfishes.</def><br/
<syn><b>Syn. --</b> family <fam>Cyclopteridae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyclopterus</ent><br/
<hw>Cyclopterus</hw> <pos>n.</pos> <def>the type genus of the <fam>Cyclopteridae</fam>, consisting of lumpfishes.  It includes <spn>Cyclopterus lumpus</spn> of the North Atlantic.</def><br/
<syn><b>Syn. --</b> genus <gen>Cyclopterus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyclorama</ent><br/
<hw>Cy`clo*ra"ma</hw> <pr>(s?`kl?-r?"m?  <or/ -r?"m?)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo-</ets> + Gr. <grk>"o`rama</grk> sight, spectacle.]</ety> <def>A pictorial view which is extended circularly, so that the spectator is surrounded by the objects represented as by things in nature. The realistic effect is increased by putting, in the space between the spectator and the picture, things adapted to the scene represented, and in some places only parts of these objects, the completion of them being carried out pictorially.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cycloscope</ent><br/
<hw>Cy"clo*scope</hw> <pr>(s?"kl?-sk?p)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo-</ets> + <ets>-scope</ets>.]</ety> <def>A machine for measuring at any moment velocity of rotation, as of a wheel of a steam engine.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclosis</ent><br/
||<hw>Cy*clo"sis</hw> <pr>(s?-kl?"s?s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ky`klwsis</grk> circulation, from <grk>kykloy^n</grk>. See <er>Cyclone</er>.]</ety> <fld>(Bot.)</fld> <def>The circulation or movement of protoplasmic granules within a living vegetable cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclosorus</ent><br/
<hw>Cyclosorus</hw> <pos>n.</pos> <def>a small genus of terrestrial ferns of tropical and subtropical southern hemisphere.</def><br/
<syn><b>Syn. --</b> genus <gen>Cyclosorus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyclosporeae</ent><br/
<hw>Cyclosporeae</hw> <pos>n.</pos> <def>in more recent classifications superseded by the order Fucales.</def><br/
<syn><b>Syn. --</b> class Cyclosporeae.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cyclosporine</ent><br/
<ent>cyclosporin</ent><br/
<mhw><hw>cyclosporin</hw>, <hw>cyclosporine</hw></mhw> <pos>n.</pos> <def>a chemical substance produced by some soil fungi, which suppresses the cellular immune response by inhibiting T cell activation, and has been used in medicine to reduce foreign tissue rejection, especially subsequent to organ transplant surgery.</def><br/
[<source>PJC</source>]</p>

<p><ent>Cyclostoma</ent><br/
<ent>Cyclostomata</ent><br/
<mhw>||<hw>Cy`clo*stom"a*ta</hw> <pr>(s?`kl?-st?m"?-t?)</pr>, ||<hw>Cy*clos"to*ma</hw> <pr>(s?-kl?s"t?-m?)</pr></mhw>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ky`klos</grk> circle + <grk>sto`ma</grk>, <grk>-atos</grk> mouth.]</ety> <fld>(Zool.)</fld> <def>A division of Bryozoa, in which the cells have circular apertures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclostomous</ent><br/
<ent>Cyclostome</ent><br/
<mhw>{ <hw>Cy"clo*stome</hw> <pr>(s?"kl?-st?m)</pr>, <hw>Cy*clos"to*mous</hw> <pr>(s?-kl?s"t?-m?s)</pr> }</mhw>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to the Cyclostomi.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclostomi</ent><br/
||<hw>Cy*clos"to*mi</hw> <pr>(s?-kl?s"t?-m?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Cyclostomata</er>.]</ety> <fld>(Zool.)</fld> <def>A glass of fishes having a suckerlike mouth, without jaws, as the lamprey; the Marsipobranchii.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclostylar</ent><br/
<hw>Cy`clo*sty"lar</hw> <pr>(s?`kl?-st?"?r)</pr>, <pos>a.</pos> <ety>[<ets>Cyclo-</ets> + Gr. <grk>sty^los</grk> column.]</ety> <def>Relating to a structure composed of a circular range of columns, without a core or building within.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyclostyle</ent><br/
<hw>Cy"clo*style</hw> <pr>(s?"kl?-st?l)</pr>, <pos>n.</pos> <ety>[<ets>Cyclo</ets> + <ets>style</ets>.]</ety> <def>A contrivance for producing manifold copies of writing or drawing. The writing or drawing is done with a style carrying a small wheel at the end which makes minute punctures in the paper, thus converting it into a stencil. Copies are transferred with an inked roller.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cyclothymia</ent><br/
<hw>cyclothymia</hw> <pos>n.</pos> <def>a mild bipolar disorder.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cyclothymic</ent><br/
<hw>cyclothymic</hw> <pos>adj.</pos> <def>of or pertaining to cyclothymia.</def><br/
<syn><b>Syn. --</b> cyclothymic.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cyclotron</ent><br/
<hw>cyclotron</hw> <pos>n.</pos> <def>a particle accelerator that 
imparts energies of several million electron-volts to rapidly moving particles; it is used in investigations in nuclear physics and particle physics.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Cycnoches</ent><br/
<hw>Cycnoches</hw> <pos>n.</pos> <def>a genus of epiphytic or terrestrial tropical American orchids.</def><br/
<syn><b>Syn. --</b> genus <gen>Cycnoches</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyder</ent><br/
<hw>Cy"der</hw> <pr>(s?"d?r)</pr>, <pos>n.</pos> <def>See <er>Cider</er>.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cydippidea</ent><br/
<ent>Cydippida</ent><br/
<mhw><hw>Cydippida</hw>, <hw>Cydippidea</hw></mhw> <pos>n.</pos> <def>an order of ctenophores having two long pinnate tentacles.</def><br/
<syn><b>Syn. --</b> order <ord>Cydippida</ord>, Cydippidea, order Cydippidea, Cydippea, order Cydippea.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cydonia</ent><br/
<hw>Cydonia</hw> <pos>n.</pos> <def>a genus of plants including the quince.</def><br/
<syn><b>Syn. --</b> genus <gen>Cydonia</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cydonin</ent><br/
<hw>Cy*do"nin</hw> <pr>(s?-d?"n?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A peculiar mucilaginous substance extracted from the seeds of the quince (<spn>Cydonia vulgaris</spn>), and regarded as a variety of amylose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cygnet</ent><br/
<hw>Cyg"net</hw> <pr>(s<icr/g"n<ecr/t)</pr>, <pos>n.</pos> <ety>[Dim. of F. <ets>cygne</ets> swan, L. <ets>cycnus</ets>. <ets>cygnus</ets>, fr. Gr. <grk>ky`klos</grk>: but F. <ets>cygne</ets> seems to be an etymological spelling of OF. <ets>cisne</ets>, fr. LL. <ets>cecinus</ets>, <ets>cicinus</ets>, perh. ultimately also fr. Gr. <grk>ky`klos</grk>.]</ety> <fld>(Zool.)</fld> <def>A young swan.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cygnus</ent><br/
<hw>Cyg"nus</hw> <pr>(s?g"n?s)</pr>, <pos>n.</pos> <ety>[L., a swan.]</ety> <fld>(Astron.)</fld> <def>A constellation of the northern hemisphere east of, or following, Lyra; the Swan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cylinder</ent><br/
<hw>Cyl"in*der</hw> <pr>(s?l"?n-d?r)</pr>, <pos>n.</pos> <ety>[F. <ets>cylindre</ets>, OF. <ets>cilindre</ets>, L. <ets>cylindrus</ets>, fr. Gr. <grk>ky`lindros</grk>, fr. <grk>kyli`ndein</grk>, <grk>kyli`ein</grk>, to roll. Cf. <er>Calender</er> the machine.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Geom.)</fld> <sd>(a)</sd> <def>A solid body which may be generated by the rotation of a parallelogram round one its sides; or a body of rollerlike form, of which the longitudinal section is oblong, and the cross section is circular.</def> <sd>(b)</sd> <def>The space inclosed by any cylindrical surface. The space may be limited or unlimited in length.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any hollow body of cylindrical form</def>, as: <sd>(a)</sd> <def>The chamber of a steam engine in which the piston is moved by the force of steam.</def> <sd>(b)</sd> <def>The barrel of an air or other pump.</def> <sd>(c)</sd> <fld>(Print.)</fld> <def>The revolving platen or bed which produces the impression or carries the type in a cylinder press.</def> <sd>(d)</sd> <def>The bore of a gun; the turning chambered breech of a revolver.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The revolving square prism carrying the cards in a Jacquard loom.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cylinder axis</b></col>. <fld>(Anat.)</fld> <cd>See <cref>Axis cylinder</cref>, under <er>Axis</er>.</cd> -- <col><b>Cylinder engine</b></col> <fld>(Paper Making)</fld>, <cd>a machine in which a cylinder takes up the pulp and delivers it in a continuous sheet to the dryers.</cd> -- <col><b>Cylinder escapement</b></col>. <cd>See <er>Escapement</er>.</cd> -- <col><b>Cylinder glass</b></col>. <cd>See <er>Glass</er>.</cd> -- <col><b>Cylinder mill</b></col>. <cd>See <er>Roller mill</er>.</cd> -- <col><b>Cylinder press</b></col>. <cd>See <er>Press</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cylindraceous</ent><br/
<hw>Cyl`in*dra"ceous</hw> <pr>(-dr?"sh?s)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cylyndrac<eacute/</ets>]</ety> <def>Cylindrical, or approaching a cylindrical form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cylindrical</ent><br/
<ent>Cylindric</ent><br/
<mhw>{ <hw>Cy*lin"dric</hw> <pr>(s?-l?n"dr?k)</pr>, <hw>Cy*lin"dric*al</hw> <pr>(-dr?-k<it>a</it>l)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>kylindriko`s</grk>, from <grk>ky`lindros</grk> cylinder: cf. F. <ets>cylindrique</ets>.]</ety> <def>Having the form of a cylinder, or of a section of its convex surface; partaking of the properties of the cylinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cylindrical lens</b></col>, <cd>a lens having one, or more than one, cylindrical surface.</cd> -- <mcol><col><b>Cylindric surface</b></col> <it>or</it> <col><b>Cylindrical surface</b></col></mcol>,  <fld>(Geom.)</fld>, <cd>a surface described by a straight line that moves according to any law, but so as to be constantly parallel to a given line.</cd> -- <col><b>Cylindrical vault</b></col>. <fld>(Arch.)</fld> <cd>See under <er>Vault</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cylindricality</ent><br/
<hw>cylindricality</hw> <pos>n.</pos> <sn>1.</sn> <def>the roundness of a 3-dimensional cylinder.</def><br/
<syn><b>Syn. --</b> cylindricalness; cylindricity.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cylindrically</ent><br/
<hw>cy*lin"dric*al*ly</hw> <pr>(s?-l?n"dr?-k<it>a</it>l-l?)</pr>, <pos>adv.</pos> <def>In the manner or shape of a cylinder; so as to be cylindrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cylindricalness</ent><br/
<hw>cylindricalness</hw> <pos>n.</pos> <sn>1.</sn> <def>the roundness of a 3-dimensional cylinder.</def><br/
<syn><b>Syn. --</b> cylindricality; cylindricity.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cylindricity</ent><br/
<hw>cyl*`in*dric"i*ty</hw> <pr>(s?l`?n-dr?s"?-t?)</pr>, <pos>n.</pos> <def>The quality or condition of being cylindrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cylindriform</ent><br/
<hw>Cy*lin"dri*form</hw> <pr>(s?-l?n"dr?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cylindrus</ets> (Gr. <grk>ky`lindros</grk>) cylinder + <ets>-form</ets>: cf. F. <ets>cylindriforme</ets>.]</ety> <def>Having the form of a cylinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cylindroid</ent><br/
<hw>Cyl"in*droid</hw> <pr>(s?l"?n-droid)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`lindros</grk> cylinder + <ets>-oid</ets>: cf. F. <ets>cylindro<ium/de</ets>.]</ety> <sn>1.</sn> <def>A solid body resembling a right cylinder, but having the bases or ends elliptical.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>A certain surface of the third degree, described by a moving straight line; -- used to illustrate the motions of a rigid body and also the forces acting on the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cylindrometric</ent><br/
<hw>Cy*lin`dro*met"ric</hw> <pr>(s?-l?n`dr?-m?t"r?k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ky`lindros</grk> + <grk>me`tron</grk> measure.]</ety> <def>Belonging to a scale used in measuring cylinders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyma</ent><br/
||<hw>Cy"ma</hw> <pr>(s<imac/"m<adot/)</pr> <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ky^ma</grk> See <er>Cyme</er>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Arch.)</fld> <def>A member or molding of the cornice, the profile of which is wavelike in form.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A cyme. See <er>Cyme</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Cyma recta</b></col>, <it>or</it> <col><b>Cyma</b></col></mcol>, <cd>a cyma, hollow in its upper part and swelling below.</cd> -- <mcol><col><b>Cyma reversa</b></col>, <it>or</it> <col><b>Cyma Ogee</b></col></mcol>, <cd>a cyma swelling out on the upper part and hollow below.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymar</ent><br/
<hw>Cy*mar"</hw> <pr>(s?-m?r")</pr>, <pos>n.</pos> <ety>[F. <ets>simarre</ets>. See <er>Chimere</er>.]</ety> <def>A slight covering; a scarf. See <er>Simar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her body shaded with a light <qex>cymar</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymatium</ent><br/
||<hw>Cy*ma"ti*um</hw> <pr>(s?-m?"sh?-?m)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/<?/<?/, dim. of <?/<?/<?/<?/ a wave.]</ety> <fld>(Arch.)</fld> <def>A capping or crowning molding in classic architecture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymbal</ent><br/
<hw>Cym"bal</hw> <pr>(s?m"b<it>a</it>l)</pr>. <pos>n.</pos> <ety>[OE. <ets>cimbale</ets>, <ets>simbale</ets>, OF. <ets>cimbale</ets>, F.  <ets>cymbale</ets>, L. <ets>cymbalum</ets>, fr. Gr. <grk>ky`mbalon</grk>, fr. <grk>ky`mbh</grk>, <grk>ky`mbos</grk>, anything hollow, hollow vessel, basin, akin to Skr. <ets>kubha</ets> pot. Cf. <er>Chime</er>.]</ety> <sn>1.</sn> <def>A musical instrument used by the ancients. It is supposed to have been similar to the modern kettle drum, though perhaps smaller.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A musical instrument of brass, shaped like a circular dish or a flat plate, with a handle at the back; -- used in pairs to produce a sharp ringing sound by clashing them together.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In orchestras, one cymbal is commonly attached to the bass drum, and the other heid in the drummer's left hand, while his right hand uses the drumstick.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A musical instrument used by gypsies and others, made of steel wire, in a triangular form, on which are movable rings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymbalist</ent><br/
<hw>Cym"bal*ist</hw>, <pos>n.</pos> <def>A performer upon cymbals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymbiform</ent><br/
<hw>Cym"bi*form</hw> <pr>(s?m"b?-f?rm)</pr>,, <pos>a.</pos> <ety>[L. <ets>cymba</ets> boat (Gr. <?/<?/<?/<?/) + <ets>-form</ets>: cf. F. <ets>cymbiforme</ets>.]</ety> <def>Shaped like a boat; <fld>(Bot.)</fld> elongated and having the upper surface decidedly concave, as the glumes of many grasses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymbium</ent><br/
||<hw>Cym"bi*um</hw> <pr>(s?m"b?-?m)</pr>, <pos>n.</pos> <ety>[L., a small cup, fr. Gr. <?/<?/<?/<?/.]</ety> <fld>(Zool.)</fld> <def>A genus of marine univalve shells; the gondola.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyme</ent><br/
<hw>Cyme</hw> <pr>(s?m)</pr>, <pos>n.</pos> <ety>[L. <ets>cyma</ets> the young sprount of a cabbage, fr. Gr. <?/<?/<?/, prop., anything swollen, hence also cyme, wave, fr. <?/<?/<?/ to be pregnant.]</ety> <fld>(Bot.)</fld> <def>A flattish or convex flower cluster, of the centrifugal or determinate type, differing from a corymb chiefly in the order of the opening of the blossoms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymene</ent><br/
<hw>Cy"mene</hw> <pr>(s?"m?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A colorless, liquid, combustible hydrocarbon, <chform>CH3.C6H4.C3H7</chform>, of pleasant odor, obtained from oil of cumin, oil of caraway, carvacrol, camphor, etc.; -- called also <altname>paracymene</altname>, and formerly <altname>camphogen</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymenol</ent><br/
<hw>Cy"me*nol</hw> <pr>(s?"m?-n?l)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See <er>Carvacrol</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymidine</ent><br/
<hw>Cy"mi*dine</hw> <pr>( s?"m?-d?n <or/ -d?n; 104)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A liquid organic base, <chform>C10H13.NH2</chform>, derived from cymene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymiferous</ent><br/
<hw>Cy*mif"er*ous</hw> <pr>(s?-m?f"?r-?s)</pr>, <pos>a.</pos> <ety>[<ets>Cyme</ets> + <ets>-ferous</ets>.]</ety> <def>Producing cymes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymbling</ent><br/
<ent>Cymling</ent><br/
<mhw>{ <hw>Cym"ling</hw>, <hw>Cymb"ling</hw> <pr>(s?m"l?ng)</pr> }</mhw>, <pos>n.</pos> <def>A scalloped or <ldquo/pattypan<rdquo/ variety of summer squash.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymogene</ent><br/
<hw>Cy"mo*gene</hw> <pr>(s?"m?-j?n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A highly volatile liquid, condensed by cold and pressure from the first products of the distillation of petroleum; -- used for producing low temperatures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymograph</ent><br/
<hw>Cy"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Cyma</ets> + <ets>-graph</ets>.]</ety> <sd>(a)</sd> <def>An instrument for making tracings of the outline or contour of profiles, moldings, etc.</def> <sd>(b)</sd> <def>Var. of <er>Kymograph</er>.</def> -- <wordforms><wf>Cy`mo*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cymograph</ent><br/
<hw>Cy"mo*graph</hw>, <pos>v. t.</pos> <def>To trace or copy with a cymograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cymoid</ent><br/
<hw>Cy"moid</hw> <pr>(s?"moid)</pr>, <pos>a.</pos> <ety>[<ets>Cyme</ets> + <ets>-oid</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the form of a cyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymometer</ent><br/
<hw>Cy*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ wave <ets>-meter</ets>.]</ety> <def>An instrument for exhibiting and measuring wave motion</def>; <specif>specif.</specif> <fld>(Elec.)</fld>, <def>an instrument for determining the frequency of electic wave oscillations, esp. in connection with wireless telegraphy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cymophane</ent><br/
<hw>Cym"o*phane</hw> <pr>(s?m"?-f?n <or/ s?"m?-)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ wave + <?/<?/<?/ To appear: cf. F. <ets>cymophane</ets>. So named in allusion to a peculiar opalescence often seen in it.]</ety> <fld>(Min.)</fld> <def>See <er>Chrysoberyl</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymophanous</ent><br/
<hw>Cy*moph"a*nous</hw> <pr>(s?-m?f"?-n?s <or/ s?-)</pr>, <pos>a.</pos> <def>Having a wavy, floating light; opalescent; chatoyant.</def></p>

<p><ent>Cymoscope</ent><br/
<hw>Cy"mo*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ wave + <ets>-scope</ets>.]</ety> <fld>(Elec.)</fld> <def>Any device for detecting the presence of electric waves. The influence of electric waves on the resistance of a particular kind of electric circuit, on the magnetization of steel, on the polarization of an electrolytic cell, or on the electric condition of a vacuum has been applied in the various cymoscopes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Cymous</ent><br/
<ent>Cymose</ent><br/
<mhw>{ <hw>Cy"mose</hw> <pr>(s?"m?s; 277)</pr>, <hw>Cy"mous</hw> <pr>(s?"m?s)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cymosus</ets> full of shoots: cf. FF. <ets>cymeux</ets>. See Cyme.]</ety> <fld>(Bot.)</fld> <def>Having the nature of a cyme, or derived from a cyme; bearing, or pertaining to, a cyme or cymes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymric</ent><br/
<hw>Cym"ric</hw> <pr>(k?m"r?k)</pr>, <pos>a.</pos> <ety>[W. <ets>Cymru</ets> Wales.]</ety> <def>Welsh.</def> -- <def2><pos>n.</pos> <def>The Welsh language.</def></def2> <altsp>[Written also <asp>Kymric</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymry</ent><br/
<hw>Cym"ry</hw> <pr>(-r?)</pr>, <pos>n.</pos> <ety>[W., <ets>pl</ets>.]</ety> <def>A collective term for the Welsh race; -- so called by themselves .</def> <altsp>[<asp>Written also Cymri</asp>, <asp>Cwmry</asp>, <asp>Kymry</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cymule</ent><br/
<hw>Cy"mule</hw> <pr>(s?"m?l)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>cymula</ets> a tender sprout, dim. of <ets>cyna</ets>. See <er>Cyme</er>.]</ety> <fld>(Bot.)</fld> <def>A small cyme, or one of very few flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynanche</ent><br/
||<hw>Cy*nan"che</hw> <pr>(s?-n?n"k?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/<?/<?/ a dog's collar, a bad kind of sore throat. Cf. <er>Quinsy</er>.]</ety> <fld>(Med.)</fld> <def>Any disease of the tonsils, throat, or windpipe, attended with inflammation, swelling, and difficulty of breathing and swallowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynanthropy</ent><br/
<hw>Cy*nan"thro*py</hw> <pr>(s?-n?n"thr?-p?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/ of a dog-man; <?/<?/<?/<?/, <?/<?/<?/, dog + <?/<?/<?/<?/<?/ man: cf. F. <ets>cynanthropie</ets>.]</ety> <fld>(Med.)</fld> <def>A kind of madness in which men fancy themselves changed into dogs, and imitate the voice and habits of that animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynarctomachy</ent><br/
<hw>Cyn`arc*tom"a*chy</hw> <pr>(s?n`?rk-t?m"?-k?)</pr>. <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/, <?/<?/<?/<?/, dog + <?/<?/<?/ bear + <?/<?/<?/ fight.]</ety> <def>Bear baiting with a dog.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynarrhodium</ent><br/
<hw>Cyn`ar*rho"di*um</hw> <pr>(s?n`?r-r?"d?-?m)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/<?/<?/<?/<?/ dog-rose; <?/<?/<?/<?/, <?/<?/<?/, dog + <?/<?/<?/ rose.]</ety> <fld>(Bot.)</fld> <def>A fruit like that of the rose, consisting of a cup formed of the calyx tube and receptacle, and containing achenes.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 363 --></p>

<p><ent>Cynegetics</ent><br/
<hw>Cyn`e*get"ics</hw> <pr>(s<icr/n`<esl/*j<ecr/t"<icr/ks)</pr>, <pos>n.</pos> <ety>[Gr. <grk>kynhgetikh`</grk> (sc. <grk>te`chnh</grk> art), fr. <grk>kynhge`ths</grk> hunter; <grk>ky`wn</grk>, <grk>kyno`s</grk>, dog + <?/<?/<?/ to lead.]</ety> <def>The art of hunting with dogs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynical</ent><br/
<ent>Cynic</ent><br/
<mhw>{ <hw>Cyn"ic</hw> <pr>(s<icr/n"<icr/k)</pr>, <hw>Cyn"ic*al</hw> <pr>(-<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>cynicus</ets> of the sect of Cynics, fr. Gr. <grk>kyniko`s</grk>, prop., dog-like, fr. <grk>ky`wn</grk>, <grk>kyno`s</grk>, dog. See <er>Hound</er>.]</ety> <sn>1.</sn> <def>Having the qualities of a surly dog; snarling; captious; currish.</def></p>

<p><q>I hope it is no very <qex>cynical</qex> asperity not to confess obligations where no benefit has been received.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to the Dog Star; <as>as, the <ex>cynic</ex>, or Sothic, year; <ex>cynic</ex> cycle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Belonging to the sect of philosophers called cynics; having the qualities of a cynic; pertaining to, or resembling, the doctrines of the cynics.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Given to sneering at rectitude and the conduct of life by moral principles; believing the worst of human nature and motives; disbelieving in the reality of any human purposes which are not suggested or directed by self-interest or self-indulgence; having a sneering disbelief in the selflessness of others; <as>as, a <ex>cynical</ex> man who scoffs at pretensions of integrity</as>; characterized by such opinions; <as>as, <ex>cynical</ex> views of human nature</as>.</def><br/
<syn><b>Syn. --</b> misanthropic, misanthropical.</syn><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In prose, <xex>cynical</xex> is used rather than <xex>cynic</xex>, in the senses 1 and 4.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cynic spasm</b></col> <fld>(Med.)</fld>, <cd>a convulsive contraction of the muscles of one side of the face, producing a sort of grin, suggesting certain movements in the upper lip of a dog.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynic</ent><br/
<hw>Cyn"ic</hw>, <pos>n.</pos> <fld>(Gr. Philos)</fld> <sn>1.</sn> <def>One of a sect or school of philosophers founded by Antisthenes, and of whom Diogenes was a disciple. The first Cynics were noted for austere lives and their scorn for social customs and current philosophical opinions. Hence the term Cynic symbolized, in the popular judgment, moroseness, and contempt for the views of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who holds views resembling those of the Cynics; a snarler; a misanthrope; particularly, a person who believes that human conduct is directed, either consciously or unconsciously, wholly by self-interest or self-indulgence, and that appearances to the contrary are superficial and untrustworthy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He could obtain from one morose <qex>cynic</qex>, whose opinion it was impossible to despise, scarcely any not acidulated with scorn.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynically</ent><br/
<hw>Cyn"ic*al*ly</hw> <pr>(s<icr/n"<icr/*k<ait/l*l<ycr/)</pr>, <pos>adv.</pos> <def>In a cynical manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynicalness</ent><br/
<hw>Cyn"ic*al*ness</hw>, <pos>n.</pos> <def>The quality of being cynical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynicism</ent><br/
<hw>Cyn"i*cism</hw> <pr>(s<icr/n"<icr/*s<icr/z'm)</pr>, <pos>n.</pos> <def>The doctrine of the Cynics; the quality of being cynical; the mental state, opinions, or conduct, of a cynic; morose and contemptuous views and opinions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynocephalus</ent><br/
<hw>Cynocephalus</hw> <pos>n.</pos> <def>the type genus of the family <fam>Cynocephalidae</fam>.</def><br/
<syn><b>Syn. --</b> genus <gen>Cynocephalus</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynodon</ent><br/
<hw>Cynodon</hw> <pos>n.</pos> <def>a genus of creeping perennial grasses of tropical and southern Africa.</def><br/
<syn><b>Syn. --</b> genus <gen>Cynodon</gen>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cynodont</ent><br/
<hw>cynodont</hw> <pos>n.</pos> <def>any member of the Cynodontia, a group of small carnivorous reptiles.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynodontia</ent><br/
<hw>Cynodontia</hw> <pos>n.</pos> <def>a division of Triassic Therapsida comprising small carnivorous reptiles often with mammallike teeth.</def><br/
<syn><b>Syn. --</b> division Cynodontia.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynoglossidae</ent><br/
<hw>Cynoglossidae</hw> <pos>n.</pos> <def>a natural family of fish including the tonguefishes.</def><br/
<syn><b>Syn. --</b> family <fam>Cynoglossidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynoidea</ent><br/
||<hw>Cy*noi"de*a</hw> <pr>(s?-noi"d?-a)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/, <?/<?/<?/, a dog + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>A division of Carnivora, including the dogs, wolves, and foxes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynomys</ent><br/
<hw>Cynomys</hw> <pos>n.</pos> <def>a genus of mammals comprising the prairie dogs.</def><br/
<syn><b>Syn. --</b> genus Cynomys.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynorexia</ent><br/
||<hw>Cyn`o*rex"i*a</hw> <pr>(s?n`?-r?ks"?-?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/, <?/<?/<?/, dog + <?/<?/<?/<?/<?/ appetite. ]</ety> <fld>(Med.)</fld> <def>A voracious appetite, like that of a starved dog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynoscephalae</ent><br/
<hw>Cynoscephalae</hw> <pos>n.</pos> <sn>1.</sn> <def>the battle that ended the second Macedonian War (197 BC); the Romans defeated King Philip of Macedon.</def><br/
<syn><b>Syn. --</b> battle of Cynoscephalae.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynoscion</ent><br/
<hw>Cynoscion</hw> <pos>n.</pos> <def>a genus of sea trout.</def><br/
<syn><b>Syn. --</b> genus Cynoscion.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cynosural</ent><br/
<hw>Cy`no*su"ral</hw> <pr>(s?`n?-sh?"r<it>a</it>l <or/  s?n`?-)</pr>, <pos>a.</pos> <def>Of or pertaining to a cynosure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynosure</ent><br/
<hw>Cy"no*sure</hw> <pr>(s?"n?-sh?r <or/ s?n"?-sh?r; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>Cynosura</ets> theconstellation Cynosure, Gr. <?/<?/<?/<?/<?/ dog's tail, the constellation Cynosure; <?/<?/<?/<?/, <?/<?/<?/<?/, dog + <grk>o'yra`</grk> tail. See<er>Cynic</er>.]</ety> <sn>1.</sn> <def>The constellation of the Lesser Bear (Ursa Minor), to which, as containing the polar star, the eyes of mariners and travelers were often directed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which serves to direct.</def>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything to which attention is strongly turned; a center of attraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where perhaps some beauty lies,<br/
The <qex>cynosure</qex> of neighboring eyes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cynthia</ent><br/
<hw>Cynthia</hw> <pos>n.</pos> <fld>(Classical mythology)</fld> <def>the virgin goddess of the hunt and the moon; daughter of Leto and twin sister of Apollo; identified with Roman Diana.</def><br/
<syn><b>Syn. --</b> Artemis.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyon</ent><br/
<hw>Cy"on</hw> <pr>(s?"?n)</pr>, <pos>n.</pos> <def>See <er>Cion</er>, and <er>Scion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyperaceous</ent><br/
<hw>Cyp`er*a"ceous</hw> <pr>(s?p`?r-?"sh?s <or/ s?`p?r-)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of, pertaining to, or resembling, a large family of plants of which the sedge is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyperus</ent><br/
<hw>Cyp"e*rus</hw> <pr>(s?p"?-r?s)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/<?/<?/<?/ sedge.]</ety> <fld>(Bot.)</fld> <def>A large genus of plants belonging to the Sedge family, and including the species called galingale, several bulrushes, and the Egyptian papyrus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypher</ent><br/
<hw>Cy"pher</hw> <pr>(s?"f?r)</pr>, <pos>n. & v.</pos> <def>See <er>Cipher</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyphonautes</ent><br/
||<hw>Cyph`o*nau"tes</hw> <pr>(s?f`?-n?"t?z)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ bent + <?/<?/<?/<?/ sailor.]</ety> <fld>(Zool.)</fld> <def>The free-swimming, bivalve larva of certain Bryozoa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyphonism</ent><br/
<hw>Cyph"o*nism</hw> <pr>(s?f`?-n?z'm <or/ s<?/"f<?/-)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/, fr. <?/<?/<?/<?/ a crooked piece of wood, a sort of pillory, fr. <?/<?/<?/ bent, stooping.]</ety> <def>A punishment sometimes used by the ancients, consisting in the besmearing of the criminal with honey, and exposing him to insects. It is still in use among some Oriental nations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypraea</ent><br/
||<hw>Cy*pr<ae/"a</hw> <pr>(s?-pr?"?)</pr>, <pos>n.</pos> <ety>[NL.; cf. Gr. <?/<?/<?/<?/ a name of Venus.]</ety> <fld>(Zool.)</fld> <def>A genus of mollusks, including the cowries. See <er>Cowrie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypraeidae</ent><br/
<hw>Cypraeidae</hw> <pos>n.</pos> <def>a natural family of marine gastropods comprising the cowries.</def><br/
<syn><b>Syn. --</b> family <fam>Cypraeidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cypre</ent><br/
<hw>cypre</hw> <pos>n.</pos> <def>a large tropical American tree (<spn>Cordia alliodora</spn>) grown for its abundant creamy white flowers and valuable wood.</def><br/
<syn><b>Syn. --</b> Spanish elm, Equador laurel, salmwood, princewood, <spn>Cordia alliodora</spn>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cypres</ent><br/
<hw>Cy`pres"</hw> <pr>(s?`pr?"  <or/ s?`pr?s")</pr>, <pos>n.</pos> <ety>[OF., nearly.]</ety> <fld>(Law)</fld> <def>A rule for construing written instruments so as to conform as nearly to the intention of the parties as is consistent with law.</def>  <rj><au>Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypress</ent><br/
<hw>Cy"press</hw> <pr>(s?"pr?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cypresses</plw> (-<?/z).</plu> <ety>[OE. <ets>cipres</ets>, <ets>cipresse</ets>, OF. <ets>cipres</ets>, F.  <ets>cypr<?/s</ets>, L.  <ets>cupressus</ets>, <ets>cyparissus</ets> (cf. the usual Lat. form <ets>cupressus</ets>), fr. Gr. <?/<?/<?/<?/, perh. of Semitic origin; cf. Heb. <ets>g<?/pher</ets>, Gen. vi. 14.]</ety> <fld>(Bot)</fld> <def>A coniferous tree of the genus <gen>Cupressus</gen>. The species are mostly evergreen, and have wood remarkable for its durability.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Among the trees called <ex>cypress</ex> are the common Oriental cypress, <spn>Cupressus sempervirens</spn>, the evergreen American cypress, <spn>Cupressus thyoides</spn> (now called <spn>Chamaecyparis sphaeroidea</spn>), and the deciduous American cypress, <spn>Taxodium distichum</spn>. As having anciently been used at funerals, and to adorn tombs, the Oriental species is an emblem of mourning and sadness.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cypress vine</b></col> <fld>(Bot.)</fld>, <cd>a climbing plant with red or white flowers (<spn>Ipot<oe/a Quamoclit</spn>, formerly <spn>Quamoclit vulgaris</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>cypress-pine</ent><br/
<hw>cypress-pine</hw> <pos>n.</pos> <def>any of several evergreen trees or shrubs of Australia and N Caledonia.</def><br/
<syn><b>Syn. --</b> cypress pine.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyprian</ent><br/
<hw>Cyp"ri*an</hw> <pr>(s?p"r?-<ait/n)</pr>, <pos>a.</pos> <ety>[L. <ets>Cyprius</ets>, fr. <ets>Cyprus</ets>, Gr. <?/<?/<?/<?/, an island in the Mediterranean, renowned for the worship of Venus.]</ety> <sn>1.</sn> <def>Belonging to Cyprus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of, pertaining, or conducing to, lewdness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprian</ent><br/
<hw>Cyp"ri*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>A native or inhabitant of Cyprus, especially of ancient Cyprus; a Cypriot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lewd woman; a harlot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprine</ent><br/
<hw>Cyp"rine</hw> <pr>(s?p"r?n <or/ s?"pr?n)</pr>, <pos>a.</pos> <ety>[Cf. <er>Cypress</er>.]</ety> <def>Of or pertaining to the cypress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprine</ent><br/
<hw>Cyp"rine</hw>, <pos>a.</pos> <ety>[See <er>Cyprinoid</er>.]</ety> <fld>(Zool.)</fld> <def>Cyprinoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprinidae</ent><br/
<hw>Cyprinidae</hw> <pos>n.</pos> <def>a natural family of fishes including the carp; tench; roach; rudd; and dace.</def><br/
<syn><b>Syn. --</b> family <fam>Cyprinidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cypriniformes</ent><br/
<hw>Cypriniformes</hw> <pos>n.</pos> <def>an order of fishes including almost entirely freshwater fishes: characins; loaches; carp; suckers; sometimes classified as <altname>Cyprinoidea</altname>, a suborder of <ord>Ostariophysi</ord>.</def><br/
<syn><b>Syn. --</b> order <ord>Cypriniformes</ord>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyprinodont</ent><br/
<hw>Cy*prin"o*dont</hw> <pr>(s?-pr?n"?-d?nt)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ kind of carp + <?/<?/<?/<?/, <?/<?/<?/<?/, a tooth.]</ety> <fld>(Zool.)</fld> <def>One of the <fam>Cyprinodontidae</fam>, a family of fishes including the killifishes or minnows. See <er>Minnow</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprinodontidae</ent><br/
<hw>Cyprinodontidae</hw> <pos>n.</pos> <def>a large family of small soft-finned fishes; killifishes; flagfishes; swordtails; guppies.</def><br/
<syn><b>Syn. --</b> family <fam>Cyprinodontidae</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cyprinoid</ent><br/
<hw>Cyp"ri*noid</hw> <pr>(s?p"r?-noid)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/<?/ a kind of carp + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like the carp (<gen>Cyprinus</gen>).</def> -- <def2><pos>n.</pos> <def>One of the <fam>Cyprinidae</fam>, or Carp family, as the goldfish, barbel, etc.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprinus</ent><br/
<hw>Cyprinus</hw> <pos>n.</pos> <def>the type genus of the family <fam>Cyprinidae</fam>; carp.</def><br/
<syn><b>Syn. --</b> genus <fam>Cyprinus</fam>.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cypriote</ent><br/
<ent>Cypriot</ent><br/
<mhw><hw>Cyp"ri*ot</hw>, <hw>Cyp"ri*ote</hw></mhw> <pr>(s?p"r?-?t)</pr>, <pos>n.</pos> <ety>[F. <ets>Cypriot</ets>, <ets>Chypriot</ets>.]</ety> <def>A native or inhabitant of Cyprus.</def><br/
<syn><b>Syn. --</b> Cypriot, Cyprian.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypriote</ent><br/
<ent>Cypriot</ent><br/
<mhw><hw>Cyp"ri*ot</hw>, <hw>Cyp"ri*ote</hw></mhw> <pos>adj.</pos>  <def>of or relating to Cyprus or its people or culture; <as>as, <ex>Cypriote</ex> monasteries</as>.</def><br/
<syn><b>Syn. --</b> Cyprian, Cypriot.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cypripedia</ent><br/
<hw>cypripedia</hw> <pos>n.</pos> <def>a plant or flower of the genus <gen>Cypripedium</gen>.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Cypripedium</ent><br/
||<hw>Cyp`ri*pe"di*um</hw> <pr>(s?p`r?-p?"d?-?m)</pr>, <pos>n.</pos> <ety>[NL., fr. <ets>Cypris</ets> Venus + <ets>pes</ets>, <ets>pedis</ets>, foot.]</ety> <fld>(Bot.)</fld> <def>A genus of orchidaceous plants including the lady's slipper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypris</ent><br/
||<hw>Cy"pris</hw> <pr>(s?"pr?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cyprides</plw> <pr>(s<?/p"r<?/-d<?/z)</pr>.</plu> <ety>[L. <ets>Cypris</ets>, the Cyprian goddess Venus, Gr. <grk>Ky`pris</grk>. See <er>Cyprian</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of small, bivalve, fresh-water Crustacea, belonging to the Ostracoda; also, a member of this genus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyprus</ent><br/
<hw>Cy"prus</hw> <pr>(s?"pr?s)</pr>, <pos>n.</pos> <ety>[OE. <ets>cipres</ets>, <ets>cypirs</ets>; perh. so named as being first manufactured in Cyprus. Cf. <er>Cipers</er>.]</ety> <def>A thin, transparent stuff, the same as, or corresponding to, crape. It was either white or black, the latter being most common, and used for mourning.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Lawn as white as driven snow,<br/
<qex>Cyprus</qex> black as e'er was crow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypruslawn</ent><br/
<hw>Cy"prus*lawn`</hw> <pr>(-l?n`)</pr>, <pos>n.</pos> <def>Same as <er>Cyprus</er>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypsela</ent><br/
||<hw>Cyp"se*la</hw> <pr>(s?p"s?-l?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>kypse`lh</grk> any hollow vessel.]</ety> <fld>(Bot.)</fld> <def>A one-seeded, one-celled, indehiscent fruit; an achene with the calyx tube adherent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cypseliform</ent><br/
<hw>Cyp*sel"i*form</hw> <pr>(s?p-s?l"?-f?rm)</pr>, <pos>a.</pos> <ety>[L. <ets>cypselus</ets> a kind of swallow, Gr. <?/<?/<?/<?/ + <ets>-form</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or belonging to the swifts (<fam>Cypselid<ae/</fam>.)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyrenaic</ent><br/
<hw>Cyr`e*na"ic</hw> <pr>(s?r`?-n?"?k <or/ s?`r?-)</pr>, <pos>a.</pos> <ety>[L. <ets>Cyrenaicus</ets>, fr. <ets>Cyrene</ets>, in Libya.]</ety> <def>Pertaining to Cyrenaica, an ancient country of northern Africa, and to Cyrene, its principal city; also, to a school of philosophy founded by Aristippus, a native of Cyrene.</def> -- <def2><pos>n.</pos> <def>A native of Cyrenaica; also, a disciple of the school of Aristippus. See <er>Cyrenian</er>, <pos>n.</pos></def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyrenian</ent><br/
<hw>Cy*re"ni*an</hw> <pr>(s?-r?"n?-<ait/n)</pr>, <pos>a.</pos> <def>Pertaining to Cyrene, in Africa; Cyrenaic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyrenian</ent><br/
<hw>Cy*re"ni*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>A native or inhabitant of Cyrene.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of a school of philosophers, established at Cyrene by Aristippus, a disciple of Socrates. Their doctrines were nearly the same as those of the Epicureans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyriologic</ent><br/
<hw>Cyr`i*o*log"ic</hw> <pr>(s?r`?-?-l?j"?k <or/ s?`r?-)</pr>, <pos>a.</pos> <ety>[See <er>Curiologic</er>.]</ety> <def>Relating to capital letters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyrtostyle</ent><br/
<hw>Cyr"to*style</hw> <pr>(s?r"t?-st?l)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ bent + <?/<?/<?/ pillar.]</ety> <fld>(Arch.)</fld> <def>A circular projecting portion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cyst</ent><br/
<hw>Cyst</hw> <pr>(s<icr/st)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`stis</grk> bladder, bag, pouch, fr. <grk>ky`ein</grk> to be pregnant. Cf. <er>Cyme</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>A pouch or sac without opening, usually membranous and containing morbid matter, which is accidentally developed in one of the natural cavities or in the substance of an organ.</def> <sd>(b)</sd> <def>In old authors, the urinary bladder, or the gall bladder.</def> <altsp>[Written also <asp>cystis</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>One of the bladders or air vessels of certain alg<ae/, as of the great kelp of the Pacific, and common rockweeds (<gen>Fuci</gen>) of our shores.</def>  <rj><au>D. C. Eaton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small capsule or sac of the kind in which many immature entozoans exist in the tissues of living animals; also, a similar form in Rotifera, etc.</def> <sd>(b)</sd> <def>A form assumed by Protozoa in which they become saclike and quiescent. It generally precedes the production of germs. See <er>Encystment</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cysted</ent><br/
<hw>Cyst"ed</hw> <pr>(s?s"t?d)</pr>, <pos>a.</pos> <def>Inclosed in a cyst.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystic</ent><br/
<hw>Cyst"ic</hw> <pr>(s?s"t?k)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>cystique</ets>.]</ety> <sn>1.</sn> <def>Having the form of, or living in, a cyst; <as>as, the <ex>cystic</ex> entozoa</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing cysts; cystose; <as>as, <ex>cystic</ex> sarcoma</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Pertaining to, or contained in, a cyst; esp., pertaining to, or contained in, either the urinary bladder or the gall bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cystic duct</b></col>, <cd>the duct from the gall bladder which unites with the hepatic to form the common bile duct.</cd> -- <col><b>Cystic worm</b></col> <fld>(Zool.)</fld>, <cd>a larval tape worm, as the cysticercus and echinococcus.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cysticercus</ent><br/
<ent>Cysticerce</ent><br/
<mhw>{ <hw>Cys"ti*cerce</hw> <pr>(s?s"t?-s?rs)</pr>, <hw>Cys`ti*cer"cus</hw> <pr>(-s?r"k?s)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>cysticercus</ets>, fr. Gr. <?/<?/<?/<?/ bladder + <grk>ke`rkos</grk> tail: cf. F. <ets>cysticerque</ets>.]</ety> <fld>(Zool.)</fld> <def>The larval form of a tapeworm, having the head and neck of a tapeworm attached to a saclike body filled with fluid; -- called also <altname>bladder worm</altname>, <altname>hydatid</altname>, and <altname>measle</altname> (as, pork <xex>measle</xex>).</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ These larvae live in the tissues of various living animals, and, when swallowed by a suitable carnivorous animal, develop into adult tapeworms in the intestine. See <er>Measles</er>, 4, <er>Tapeworm</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cysticule</ent><br/
<hw>Cys"ti*cule</hw> <pr>(s?s"t?-k?l)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>cyst</ets>.]</ety> <fld>(Anat.)</fld> <def>An appendage of the vestibular ear sac of fishes.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystid</ent><br/
<hw>Cys"tid</hw> <pr>(s?s"t?d)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cystidea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystidea</ent><br/
||<hw>Cys*tid"e*a</hw> <pr>(s?s-t?d"?-?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ a bladder, pouch.]</ety> <fld>(Zool.)</fld> <def>An order of Crinoidea, mostly fossils of the Paleozoic rocks. They were usually roundish or egg-shaped, and often unsymmetrical; some were sessile, others had short stems.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystidean</ent><br/
<hw>Cys*tid"e*an</hw> <pr>(-t?d"?-<ait/n)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Cystidea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystine</ent><br/
<hw>Cyst"ine</hw> <pr>(s?s"t?n; 104)</pr>, <pos>n.</pos> <ety>[See <er>Cyst</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A white crystalline substance, <chform>C3H7NSO2</chform>, containing sulphur, occuring as a constituent of certain rare urinary calculi, and occasionally found as a sediment in urine.</def><-- cysteine? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystis</ent><br/
<hw>Cys"tis</hw> <pr>(s?s"t?s)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A cyst. See <er>Cyst</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystitis</ent><br/
<hw>Cys*ti"tis</hw> <pr>(s?s-t?"t?s)</pr>, <pos>n.</pos> <ety>[<ets>Cyst</ets> +  <ets>-itis</ets>: cf. F. <ets>cystite</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystocarp</ent><br/
<hw>Cys"to*carp</hw> <pr>(s?s"t?-k?rp)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ bladder + <grk>karpo`s</grk> fruit.]</ety> <fld>(Bot.)</fld> <def>A minute vesicle in a red seaweed, which contains the reproductive spores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystocele</ent><br/
<hw>Cys"to*cele</hw> <pr>(-s?l)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ bladder + <?/<?/<?/ tumor: cf. F. <ets>cystocele</ets>.]</ety> <fld>(Med.)</fld> <def>Hernia in which the urinary bladder protrudes; vesical hernia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystoidean</ent><br/
<ent>Cystoid</ent><br/
<mhw>{ <hw>Cys"toid</hw>, <hw>Cys*toid"e*an</hw>  }</mhw>, <pos>n.</pos> <def>Same as <er>Cystidean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystoidea</ent><br/
<hw>Cys*toi"de*a</hw> <pr>(s?s-toi"d?-?)</pr>, <pos>n.</pos> <def>Same as <er>Cystidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystolith</ent><br/
<hw>Cys"to*lith</hw> <pr>(s?s"t?-l?th)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ bladder + <ets>-lith</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A concretion of mineral matter within a leaf or other part of a plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A urinary calculus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystolithic</ent><br/
<hw>Cys`to*lith"ic</hw> <pr>(-l?th"?k)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Relating to stone in the bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystoplast</ent><br/
<hw>Cys"to*plast</hw> <pr>(-pl?st)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ bladder + <?/<?/<?/ to form.]</ety> <fld>(Biol.)</fld> <def>A nucleated cell having an envelope or cell wall, as a red blood corpuscle or an epithelial cell; a cell concerned in growth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystose</ent><br/
<hw>Cyst"ose</hw> <pr>(s?s"t?s)</pr>, <pos>a.</pos> <def>Containing, or resembling, a cyst or cysts; cystic; bladdery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>cytosine</ent><br/
<hw>cy"to*sine</hw> <pr>(s<imac/"t<osl/*s<icr/n <it>or</it> s<imac/"t<osl/*s<emac/n)</pr>, <pos>n.</pos> <fld>(Biochemistry)</fld> <def>A pyrimidine (<chform>C4H5N3O</chform>) which is one of the four major basic components of DNA and RNA in most organisms, forming glyosides with ribose and deoxyribose.  It is the basic component of <partof>cytidine</partof>, <partof>deoxycytidine</partof>, <partof>cytosine</partof>, <partof>cytidine monophosphate</partof>, and derivatives of those compounds.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytoskeletal</ent><br/
<hw>cy`to*skel"e*tal</hw> <pr>(s<imac/`t<osl/*sk<ecr/l"<ecr/*t<ait/l)</pr>, <pos>a.</pos> <fld>(Cell Biology)</fld> <def>Of or pertaining to the cytoskeleton; <as>as, <ex>cytoskeletal</ex> microtubules</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytoskeleton</ent><br/
<hw>cy`to*skel"e*ton</hw> <pr>(s<imac/`t<osl/*sk<ecr/l"<icr/*t'n)</pr>, <pos>n.</pos> <fld>(Cell Biology)</fld> <def>An arrangement of microtubules, microfilaments, and larger filaments within a cell serving to provide structural support of components of the cell, and to transport components from one part of the cell to another; the filaments are composed of protein and form a latticelike arrangement which may change rapidly with time.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytosol</ent><br/
<hw>cy"to*sol</hw> <pr>(s<imac/"t<osl/*s<add/l <it>or</it> s<imac/"t<osl/*s<ocr/l)</pr>, <pos>n.</pos> <fld>(Biology)</fld> <def>The soluble components of the fluid matter enclosed within the cellular membrane; the portion of the cytoplasm which remains after removal of particulate components.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytostasis</ent><br/
<hw>cy`to*sta"sis</hw> <pr>(s<imac/`t<osl/*st<amac/"s<icr/s)</pr>, <pos>n.</pos> <fld>(Biology)</fld> <def>The inhibition or cessation of cell growth or division.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytostatic</ent><br/
<hw>cy`to*stat"ic</hw> <pr>(s<imac/`t<osl/*st<acr/t"<icr/k)</pr>, <pos>a.</pos> <fld>(Biology)</fld> <def>Having an inhibitory action on cell growth or cell division.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytostome</ent><br/
<hw>cytostome</hw> <pos>n.</pos> <def>The mouth of a protozoan.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>cytotaxis</ent><br/
<hw>cy`to*tax"is</hw> <pr>(s<imac/`t<osl/*t<acr/k"s<icr/s)</pr>, <pos>n.</pos> <fld>(Biology)</fld> <def>The movement of cells toward or away from other cells, as a result of some stimulation, such as chemical signals transmitted and received by the cells.</def><br/
[<source>PJC</source>]</p>

<p><ent>cytotaxonomy</ent><br/
<hw>cy`to*tax*on"o*my</hw> <pr>(s<imac/`t<osl/*t<acr/ks*<ocr/n"<osl/*m<ycr/)</pr>, <pos>n.</pos> <fld>(Biology)</fld> <def>A branch of taxonomy which uses the characteristics of cellular structures, such as somatic chromosomes, to classify organisms.</def><br/
[<source>PJC</source>]</p>

<p>  -- <wordforms><wf>cy`to*tax*on"o*mist</wf>, <pos>n.</pos> -- <wf>cy`to*tax`o*nom"ic</wf>, <pos>a.</pos> -- <wf>cy`to*tax`o*nom"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>PJC</source>]</p>

<p><ent>Cystotome</ent><br/
<hw>Cys"to*tome</hw> <pr>(s?s"t?-t?m)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ bladder + <?/<?/<?/<?/ to cut: cf. F. <ets>cystotome</ets>.]</ety> <fld>(Surg.)</fld> <def>A knife or instrument used in cystotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cystotomy</ent><br/
<hw>Cys*tot"o*my</hw> <pr>(s??s-t?t"?-m?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ bladder + <?/<?/<?/<?/ to cut: cf. F. <ets>cystotomie</ets>.]</ety> <def>The act or practice of opening cysts; esp., the operation of cutting into the bladder, as for the extraction of a calculus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytherean</ent><br/
<hw>Cyth`er*e"an</hw> <pr>(s?th`?r--?"<it>a</it>n)</pr>, <pos>a.</pos> <ety>[L. <ets>Cythereus</ets>, from <ets>Cythera</ets>, Gr. <?/<?/<?/, now <ets>Cerigo</ets>, an island in the <AE/gean Sea, celebrated for the worship of Venus.]</ety> <def>Pertaining to the goddess Venus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytoblast</ent><br/
<hw>Cy"to*blast</hw> <pr>(s?"t?-bl?st)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`tos</grk> hollow vessel + <ets>-blast</ets>.]</ety> <fld>(Biol.)</fld> <def>The nucleus of a cell; the germinal or active spot of a cellule, through or in which cell development takes place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytoblastema</ent><br/
<hw>Cy`to*blas*te"ma</hw> <pr>(-bl?s-t?"m?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ky`tos</grk> hollow vessel + <?/<?/<?/<?/ growth.]</ety> <fld>(Biol.)</fld> <def>See <er>Protoplasm</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytococcus</ent><br/
<hw>Cy`to*coc"cus</hw> <pr>(-k?k"k?s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Cytococci</plw> <pr>(-s<?/)</pr>.</plu> <ety>[NL., fr. Gr. <grk>ky`tos</grk> hollow vessel + <?/<?/<?/<?/ kernel.]</ety> <fld>(Biol.)</fld> <def>The nucleus of the cytula or parent cell.</def>  <rj><au>H<ae/ckel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytode</ent><br/
<hw>Cy"tode</hw> <pr>(s?"t?d)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`tos</grk> hollow vessel, from <?/<?/<?/<?/. See <er>Cyst</er>.]</ety> <fld>(Biol.)</fld> <def>A nonnucleated mass of protoplasm, the supposed simplest form of independent life differing from the <contr>amoeba</contr>, in which nuclei are present.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytogenesis</ent><br/
<hw>Cy`to*gen"e*sis</hw> <pr>(s?`t?-j?n"?-s?s)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`tos</grk> hollow vessel + E. <ets>genesis</ets>.]</ety> <fld>(Biol.)</fld> <def>Development of cells in animal and vegetable organisms. See <er>Gemmation</er>, <er>Budding</er>, <er>Karyokinesis</er>; also <cref>Cell development</cref>, under <er>Cell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytogenetic</ent><br/
<ent>Cytogenic</ent><br/
<mhw>{ <hw>Cy`to*gen"ic</hw> <pr>(s?`t?-j?n"?k)</pr>, <hw>Cy`to*ge*net"ic</hw> <pr>(-j?-n?t"?k)</pr>, }</mhw> <pos>a.</pos> <fld>(Biol.)</fld> <def>Of or pertaining to <xex>cytogenesis</xex> or cell development.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytogenous</ent><br/
<hw>Cy*tog"e*nous</hw> <pr>(s?-t?j"?-n?s)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Producing cells; -- applied esp. to lymphatic, or adenoid, tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytogeny</ent><br/
<hw>Cy*tog"e*ny</hw> <pr>(-n?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>Cell production or development; cytogenesis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytoid</ent><br/
<hw>Cy"toid</hw> <pr>(s?"toid)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ky`tos</grk> hollow vessel + <ets>-oid</ets>.]</ety> <fld>(Physiol.)</fld> <def>Cell-like; -- applied to the corpuscles of lymph, blood, chyle, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytoplasm</ent><br/
<hw>Cy"to*plasm</hw> <pr>(s?"t?-pl?z'm)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ky`tos</grk> hollow vessel + <?/<?/<?/<?/ a mold.]</ety> <fld>(Biol.)</fld> <def>The substance of the body of a cell, as distinguished from the <contr>karyoplasma</contr>, or substance of the nucleus.</def> -- <wordforms><wf>Cy`to*plas"mic</wf> <pr>(-pl<?/z"m<?/k)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Cytula</ent><br/
||<hw>Cyt"u*la</hw> <pr>(s?t"?-l?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ky`tos</grk> a hollow vessel.]</ety> <fld>(Biol.)</fld> <def>The fertilized egg cell or parent cell, from the development of which the child or other organism is formed.</def>  <rj><au>H<ae/ckel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>czar</ent><br/
<hw>czar</hw> <pr>(z<aum/r)</pr>, <pos>n.</pos> <ety>[Russ. <ets>tsare</ets>, fr. L. <ets>Caesar</ets> C<ae/sar; cf. OPol. <ets>czar</ets>, Pol.  <ets>car</ets>. ]</ety> <def>A king; a chief; the title of the emperor of Russia.</def> <altsp>[Written also <asp>tsar</asp> and <asp>tzar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>czarevna</ent><br/
<hw>cza*rev"na</hw> <pr>(z?-r?v"n?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>tsarevna</ets>.]</ety> <def>The title of the wife of the czarowitz.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>czarina</ent><br/
<hw>cza*ri"na</hw> <pr>(z?-r?"n?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>Zarin</ets>, <ets>Czarin</ets>, fem., Russ. <ets>tsaritsa</ets>.]</ety> <def>The title of the empress of Russia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Czarinian</ent><br/
<hw>Cza*rin"i*an</hw> <pr>(z?-r?n"?-<it>a</it>n)</pr>, <pos>a.</pos> <def>Of or pertaining to the czar or the czarina; czarish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Czarish</ent><br/
<hw>Czar"ish</hw> <pr>(z?r"?sh)</pr>, <pos>a.</pos> <def>Of or pertaining to the czar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Czarowitz</ent><br/
<hw>Czar"o*witz</hw> <pr>(z?r"?-w?ts <or/  t??r"?-v?ch)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Czarowitzes</plw> <pr>(-<?/z)</pr>.</plu> <ety>[Russ. <ets>tsar<eacute/vich'</ets>.]</ety> <def>The title of the eldest son of the czar of Russia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Czech</ent><br/
<hw>Czech</hw> <pr>(ch<ecr/k; 204)</pr>, <pos>prop. n.</pos> <sn>1.</sn> <def>One of the Czechs; an inhabitant or native of the <country>Czech Republic</country>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The language of the Czechs (often called Bohemian), the harshest and richest of the Slavic languages.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to <country>Czechoslovakia</country> or the <country>Czech Republic</country>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Czechic</ent><br/
<hw>Czech"ic</hw> <pr>(ch?k"?k)</pr>, <pos>prop. a.</pos> <def>Of or pertaining to the Czechs.</def> <ldquo/One <xex>Czechic</xex> realm.<rdquo/  <rj><au>The Nation.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Czechoslovakian</ent><br/
<hw>Czechoslovakian</hw> <pos>prop. adj.</pos> <def>Of or pertaining to Czechoslovakia.   Since the dissolution of Czechoslovakia into the Czech republic and Slovakia in January 1993, this term no longer refers to any current country.</def><br/
<syn><b>Syn. --</b> Czechoslovak, Czech.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn>  <def>Of, pertaining to, or characteristic of the inhabitants of Czechoslovakia or the Czech Republic; <as>as, the <ex>Czech</ex> population</as>.</def><br/
<syn><b>Syn. --</b> Czechoslovak, Czech.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Czechoslovakian</ent><br/
<hw>Czechoslovakian</hw> <pos>prop. n.</pos> <def>a native or inhabitant of Czechoslovakia.  Since the dissolution of Czechoslovakia into the Czech republic and Slovakia in January 1993, this term no longer refers to a citizen of any current country.  The natives of the former Czechoslovakia are now Czechs or Slovaks.</def><br/
<syn><b>Syn. --</b> Chechoslovak, Czech.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Czechs</ent><br/
<hw>Czechs</hw> <pr>(ch<ecr/ks)</pr>, <pos>prop. n. pl.</pos>; <sing><it>sing.</it> <singw>Czech</singw></sing>.  <ety>[Named after their chieftain, <etsep>Czech</etsep>.]</ety> <fld>(Ethnol.)</fld> <def>The most westerly branch of the great Slavic family of nations, numbering now more than 6,000,000, and found principally in the <country>Czech Republic</country>, consisting of the old regions of Bohemia and Moravia.</def></p>

<p><-- bottom of page 363.  next page, 364, starts letter "D" --></p>

